#!/usr/bin/perl
package acl::AclLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'acl::AclLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# acl-lib.pl
# Library for editing webmin users, passwords and access rights

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();
$self->{'access_hash'}->{'switch'} = 0 if ($self->is_readonly_mode());

1;

1;
		return $self;
}

# list_users()
# Returns a list of hashes containing user details
sub list_users
{
	my $self = shift;

local(%miniserv, $_, @rv, %acl, %logout);
$self->read_acl(undef, \%acl);
$self->get_miniserv_config(\%miniserv);
foreach my $a (split(/\s+/, $miniserv{'logouttimes'})) {
	if ($a =~ /^([^=]+)=(\S+)$/) {
		$logout{$1} = $2;
		}
	}
open(PWFILE, $miniserv{'userfile'});
while(<PWFILE>) {
	s/\r|\n//g;
	local @user = split(/:/, $_);
	if (@user) {
		local(%user);
		$user{'name'} = $user[0];
		$user{'pass'} = $user[1];
		$user{'sync'} = $user[2];
		$user{'cert'} = $user[3];
		if ($user[4] =~ /^(allow|deny)\s+(.*)/) {
			$user{$1} = $2;
			}
		if ($user[5] =~ /days\s+(\S+)/) {
			$user{'days'} = $1;
			}
		if ($user[5] =~ /hours\s+(\d+\.\d+)-(\d+\.\d+)/) {
			$user{'hoursfrom'} = $1;
			$user{'hoursto'} = $2;
			}
		$user{'modules'} = $acl{$user[0]};
		$user{'lang'} = $self->{'gconfig_hash'}->{"lang_$user[0]"};
		$user{'notabs'} = $self->{'gconfig_hash'}->{"notabs_$user[0]"};
		$user{'skill'} = $self->{'gconfig_hash'}->{"skill_$user[0]"};
		$user{'risk'} = $self->{'gconfig_hash'}->{"risk_$user[0]"};
		$user{'rbacdeny'} = $self->{'gconfig_hash'}->{"rbacdeny_$user[0]"};
		$user{'theme'} = $self->{'gconfig_hash'}->{"theme_$user[0]"};
		$user{'readonly'} = $self->{'gconfig_hash'}->{"readonly_$user[0]"};
		$user{'ownmods'} = [ split(/\s+/,
					   $self->{'gconfig_hash'}->{"ownmods_$user[0]"}) ];
		$user{'logouttime'} = $logout{$user[0]};
		push(@rv, \%user);
		}
	}
close(PWFILE);
return @rv;
}

# list_groups()
# Returns a list of hashes, one per group.
# Group membership is stored in /etc/webmin/webmin.groups, and other attributes
# in the config file.
sub list_groups
{
	my $self = shift;

local @rv;
open(GROUPS, "$self->{'config_directory'}/cpplus.groups");
while(<GROUPS>) {
	s/\r|\n//g;
	local @g = split(/:/, $_);
	local $group = { 'name' => $g[0],
			 'members' => [ split(/\s+/, $g[1]) ],
			 'modules' => [ split(/\s+/, $g[2]) ],
			 'desc' => $g[3],
			 'ownmods' => [ split(/\s+/, $g[4]) ] };
	push(@rv, $group);
	}
close(GROUPS);
return @rv;
}

# list_modules()
# Returns a list of the dirs of all modules available on this system
sub list_modules
{
	my $self = shift;

return map { $_->{'dir'} } $self->list_module_infos();
}

# list_module_infos()
# Returns a list of the details of all  modules available on this system
sub list_module_infos
{
	my $self = shift;

local @mods = grep { $self->check_os_support($_) } $self->get_all_module_infos();
return sort { $a->{'desc'} cmp $b->{'desc'} } @mods;
}

# create_user(&details, clone)
sub create_user
{
	my $self = shift;

local(%user, %miniserv, @mods);
%user = %{$_[0]};

$self->lock_file($ENV{'MINISERV_CONFIG'});
$self->get_miniserv_config(\%miniserv);
if ($user{'theme'}) {
	$miniserv{"preroot_".$user{'name'}} = $user{'theme'};
	}
elsif (defined($user{'theme'})) {
	$miniserv{"preroot_".$user{'name'}} = "";
	}
if (defined($user{'logouttime'})) {
	local @logout = split(/\s+/, $miniserv{'logouttimes'});
	push(@logout, "$user{'name'}=$user{'logouttime'}");
	$miniserv{'logouttimes'} = join(" ", @logout);
	}
$self->put_miniserv_config(\%miniserv);
$self->unlock_file($ENV{'MINISERV_CONFIG'});

local @times;
push(@times, "days", $user{'days'}) if ($user{'days'} ne '');
push(@times, "hours", $user{'hoursfrom'}."-".$user{'hoursto'})
	if ($user{'hoursfrom'});
$self->lock_file($miniserv{'userfile'});
$self->open_tempfile(PWFILE, ">>$miniserv{'userfile'}");
$self->print_tempfile(PWFILE,
	"$user{'name'}:$user{'pass'}:$user{'sync'}:$user{'cert'}:",
	($user{'allow'} ? "allow $user{'allow'}" :
	 $user{'deny'} ? "deny $user{'deny'}" : ""),":",
	join(" ", @times),
	"\n");
$self->close_tempfile(PWFILE);
$self->unlock_file($miniserv{'userfile'});

$self->lock_file($self->acl_filename());
@mods = $self->list_modules();
$self->open_tempfile(ACL, ">>".$self->acl_filename());
$self->print_tempfile(ACL, $self->acl_line(\%user, \@mods));
$self->close_tempfile(ACL);
$self->unlock_file($self->acl_filename());

delete($self->{'gconfig_hash'}->{"lang_".$user{'name'}});
$self->{'gconfig_hash'}->{"lang_".$user{'name'}} = $user{'lang'} if ($user{'lang'});
delete($self->{'gconfig_hash'}->{"notabs_".$user{'name'}});
$self->{'gconfig_hash'}->{"notabs_".$user{'name'}} = $user{'notabs'} if ($user{'notabs'});
delete($self->{'gconfig_hash'}->{"skill_".$user{'name'}});
$self->{'gconfig_hash'}->{"skill_".$user{'name'}} = $user{'skill'} if ($user{'skill'});
delete($self->{'gconfig_hash'}->{"risk_".$user{'name'}});
$self->{'gconfig_hash'}->{"risk_".$user{'name'}} = $user{'risk'} if ($user{'risk'});
delete($self->{'gconfig_hash'}->{"rbacdeny_".$user{'name'}});
$self->{'gconfig_hash'}->{"rbacdeny_".$user{'name'}} = $user{'rbacdeny'} if ($user{'rbacdeny'});
delete($self->{'gconfig_hash'}->{"ownmods_".$user{'name'}});
$self->{'gconfig_hash'}->{"ownmods_".$user{'name'}} = join(" ", @{$user{'ownmods'}})
	if (@{$user{'ownmods'}});
delete($self->{'gconfig_hash'}->{"theme_".$user{'name'}});
$self->{'gconfig_hash'}->{"theme_".$user{'name'}} = $user{'theme'} if (defined($user{'theme'}));
$self->{'gconfig_hash'}->{"readonly_".$user{'name'}} = $user{'readonly'}
	if (defined($user{'readonly'}));
$self->write_file("$self->{'config_directory'}/config", \%{$self->{'gconfig_hash'}});

if ($_[1]) {
	foreach $m ("", @mods) {
		local $file = "$self->{'config_directory'}/$m/$_[1].acl";
		local $dest = "$self->{'config_directory'}/$m/$user{'name'}.acl";
		if (-r $file) {
			local %macl;
			$self->read_file($file, \%macl);
			$self->write_file($dest, \%macl);
			}
		}
	}
}

# modify_user(old-name, &details)
sub modify_user
{
	my $self = shift;

local(%user, %miniserv, @pwfile, @acl, @mods, $_, $m);
%user = %{$_[1]};

$self->lock_file($ENV{'MINISERV_CONFIG'});
$self->get_miniserv_config(\%miniserv);
delete($miniserv{"preroot_".$_[0]});
if ($user{'theme'}) {
	$miniserv{"preroot_".$user{'name'}} = $user{'theme'};
	}
elsif (defined($user{'theme'})) {
	$miniserv{"preroot_".$user{'name'}} = "";
	}
local @logout = split(/\s+/, $miniserv{'logouttimes'});
@logout = grep { $_ !~ /^$_[0]=/ } @logout;
if (defined($user{'logouttime'})) {
	push(@logout, "$user{'name'}=$user{'logouttime'}");
	}
$miniserv{'logouttimes'} = join(" ", @logout);
$self->put_miniserv_config(\%miniserv);
$self->unlock_file($ENV{'MINISERV_CONFIG'});

local @times;
push(@times, "days", $user{'days'}) if ($user{'days'} ne '');
push(@times, "hours", $user{'hoursfrom'}."-".$user{'hoursto'})
	if ($user{'hoursfrom'});
$self->lock_file($miniserv{'userfile'});
open(PWFILE, $miniserv{'userfile'});
@pwfile = <PWFILE>;
close(PWFILE);
$self->open_tempfile(PWFILE, ">$miniserv{'userfile'}");
foreach (@pwfile) {
	if (/^([^:]+):/ && $1 eq $_[0]) {
		$self->print_tempfile(PWFILE,
			"$user{'name'}:$user{'pass'}:",
			"$user{'sync'}:$user{'cert'}:",
			($user{'allow'} ? "allow $user{'allow'}" :
			 $user{'deny'} ? "deny $user{'deny'}" : ""),":",
			join(" ", @times),"\n");
		}
	else {
		$self->print_tempfile(PWFILE, $_);
		}
	}
$self->close_tempfile(PWFILE);
$self->unlock_file($miniserv{'userfile'});

$self->lock_file($self->acl_filename());
@mods = $self->list_modules();
open(ACL, $self->acl_filename());
@acl = <ACL>;
close(ACL);
$self->open_tempfile(ACL, ">".$self->acl_filename());
foreach (@acl) {
	if (/^(\S+):/ && $1 eq $_[0]) {
		$self->print_tempfile(ACL, $self->acl_line($_[1], \@mods));
		}
	else {
		$self->print_tempfile(ACL, $_);
		}
	}
$self->close_tempfile(ACL);
$self->unlock_file($self->acl_filename());

delete($self->{'gconfig_hash'}->{"lang_".$_[0]});
$self->{'gconfig_hash'}->{"lang_".$user{'name'}} = $user{'lang'} if ($user{'lang'});
delete($self->{'gconfig_hash'}->{"notabs_".$_[0]});
$self->{'gconfig_hash'}->{"notabs_".$user{'name'}} = $user{'notabs'} if ($user{'notabs'});
delete($self->{'gconfig_hash'}->{"skill_".$_[0]});
$self->{'gconfig_hash'}->{"skill_".$user{'name'}} = $user{'skill'} if ($user{'skill'});
delete($self->{'gconfig_hash'}->{"risk_".$_[0]});
$self->{'gconfig_hash'}->{"risk_".$user{'name'}} = $user{'risk'} if ($user{'risk'});
delete($self->{'gconfig_hash'}->{"rbacdeny_".$_[0]});
$self->{'gconfig_hash'}->{"rbacdeny_".$user{'name'}} = $user{'rbacdeny'} if ($user{'rbacdeny'});
delete($self->{'gconfig_hash'}->{"ownmods_".$_[0]});
$self->{'gconfig_hash'}->{"ownmods_".$user{'name'}} = join(" ", @{$user{'ownmods'}})
	if (@{$user{'ownmods'}});
delete($self->{'gconfig_hash'}->{"theme_".$_[0]});
$self->{'gconfig_hash'}->{"theme_".$user{'name'}} = $user{'theme'} if (defined($user{'theme'}));
delete($self->{'gconfig_hash'}->{"readonly_".$_[0]});
$self->{'gconfig_hash'}->{"readonly_".$user{'name'}} = $user{'readonly'}
	if (defined($user{'readonly'}));
$self->write_file("$self->{'config_directory'}/config", \%{$self->{'gconfig_hash'}});

if ($_[0] ne $user{'name'}) {
	# Rename all .acl files if user renamed
	foreach $m (@mods, "") {
		local $file = "$self->{'config_directory'}/$m/$_[0].acl";
		if (-r $file) {
			$self->rename_file($file, "$self->{'config_directory'}/$m/$user{'name'}.acl");
			}
		}
	local $file = "$self->{'config_directory'}/$_[0].acl";
	if (-r $file) {
		$self->rename_file($file, "$self->{'config_directory'}/$user{'name'}.acl");
		}
	}

if ($miniserv{'session'} && $_[0] ne $user{'name'}) {
	# Modify all sessions for the renamed user
	$self->rename_session_user('miniserv', $_[0], $user{'name'});
	}
}

# delete_user(name)
# Delete some user from the ACL and password files
sub delete_user
{
	my $self = shift;

local($_, @pwfile, @acl, %miniserv);

$self->lock_file($ENV{'MINISERV_CONFIG'});
$self->get_miniserv_config(\%miniserv);
delete($miniserv{"preroot_".$_[0]});
local @logout = split(/\s+/, $miniserv{'logouttimes'});
@logout = grep { $_ !~ /^$_[0]=/ } @logout;
$miniserv{'logouttimes'} = join(" ", @logout);
$self->put_miniserv_config(\%miniserv);
$self->unlock_file($ENV{'MINISERV_CONFIG'});

$self->lock_file($miniserv{'userfile'});
open(PWFILE, $miniserv{'userfile'});
@pwfile = <PWFILE>;
close(PWFILE);
$self->open_tempfile(PWFILE, ">$miniserv{'userfile'}");
foreach (@pwfile) {
	if (!/^([^:]+):/ || $1 ne $_[0]) {
		$self->print_tempfile(PWFILE, $_);
		}
	}
$self->close_tempfile(PWFILE);
$self->unlock_file($miniserv{'userfile'});

$self->lock_file($self->acl_filename());
open(ACL, $self->acl_filename());
@acl = <ACL>;
close(ACL);
$self->open_tempfile(ACL, ">".$self->acl_filename());
foreach (@acl) {
	if (!/^([^:]+):/ || $1 ne $_[0]) {
		$self->print_tempfile(ACL, $_);
		}
	}
$self->close_tempfile(ACL);
$self->unlock_file($self->acl_filename());

delete($self->{'gconfig_hash'}->{"lang_".$_[0]});
delete($self->{'gconfig_hash'}->{"notabs_".$_[0]});
delete($self->{'gconfig_hash'}->{"skill_".$_[0]});
delete($self->{'gconfig_hash'}->{"risk_".$_[0]});
delete($self->{'gconfig_hash'}->{"ownmods_".$_[0]});
delete($self->{'gconfig_hash'}->{"theme_".$_[0]});
delete($self->{'gconfig_hash'}->{"readonly_".$_[0]});
$self->write_file("$self->{'config_directory'}/config", \%{$self->{'gconfig_hash'}});

# Delete all module .acl files
$self->unlink_file(map { "$self->{'config_directory'}/$_/$_[0].acl" } $self->list_modules());
$self->unlink_file("$self->{'config_directory'}/$_[0].acl");

if ($miniserv{'session'}) {
	# Delete all sessions for the deleted user
	$self->delete_session_user(\%miniserv, $_[0]);
	}
}

# create_group(&group, [clone])
# Add a new webmin group
sub create_group
{
	my $self = shift;

$self->lock_file("$self->{'config_directory'}/cpplus.groups");
open(GROUP, ">>$self->{'config_directory'}/cpplus.groups");
print GROUP $self->group_line($_[0]),"\n";
close(GROUP);
$self->unlock_file("$self->{'config_directory'}/cpplus.groups");

if ($_[1]) {
	foreach $m ("", $self->list_modules()) {
		local $file = "$self->{'config_directory'}/$m/$_[1].gacl";
		local $dest = "$self->{'config_directory'}/$m/$_[0]->{'name'}.gacl";
		if (-r $file) {
			local %macl;
			$self->read_file($file, \%macl);
			$self->write_file($dest, \%macl);
			}
		}
	}
}

# modify_group(name, &group)
# Update a webmin group
sub modify_group
{
	my $self = shift;

$self->lock_file("$self->{'config_directory'}/cpplus.groups");
local $lref = $self->read_file_lines("$self->{'config_directory'}/cpplus.groups");
foreach $l (@$lref) {
	if ($l =~ /^([^:]+):/ && $1 eq $_[0]) {
		$l = $self->group_line($_[1]);
		}
	}
$self->flush_file_lines();
$self->unlock_file("$self->{'config_directory'}/cpplus.groups");

if ($_[0] ne $_[1]->{'name'}) {
	# Rename all .gacl files if group renamed
	foreach $m (@{$_[1]->{'modules'}}, "") {
		local $file = "$self->{'config_directory'}/$m/$_[0].gacl";
		if (-r $file) {
			$self->rename_file($file,
				     "$self->{'config_directory'}/$m/$_[1]->{'name'}.gacl");
			}
		}
	}
}

# delete_group(name)
# Delete a webmin group
sub delete_group
{
	my $self = shift;

$self->lock_file("$self->{'config_directory'}/cpplus.groups");
local $lref = $self->read_file_lines("$self->{'config_directory'}/cpplus.groups");
@$lref = grep { !/^([^:]+):/ || $1 ne $_[0] } @$lref;
$self->flush_file_lines();
$self->unlock_file("$self->{'config_directory'}/cpplus.groups");
$self->unlink_file(map { "$self->{'config_directory'}/$_/$_[0].gacl" } $self->list_modules());
}

# group_line(&group)
sub group_line
{
	my $self = shift;

return join(":", $_[0]->{'name'},
		 join(" ", @{$_[0]->{'members'}}),
		 join(" ", @{$_[0]->{'modules'}}),
		 $_[0]->{'desc'},
		 join(" ", @{$_[0]->{'ownmods'}}) );
}

# acl_line(&user, &allmodules)
# Internal function to generate an ACL file line
sub acl_line
{
	my $self = shift;

local(%user);
%user = %{$_[0]};
return "$user{'name'}: ".join(' ', @{$user{'modules'}})."\n";
}

# can_edit_user(user, [&groups])
sub can_edit_user
{
	my $self = shift;

return 1 if ($self->{'access_hash'}->{'users'} eq '*');
if ($self->{'access_hash'}->{'users'} eq '~') {
	return $self->{'base_remote_user'} eq $_[0];
	}
local $u;
local $glist = $_[1] ? $_[1] : [ $self->list_groups() ];
foreach $u (split(/\s+/, $self->{'access_hash'}->{'users'})) {
	if ($u =~ /^_(\S+)$/) {
		foreach $g (@$glist) {
			return 1 if ($g->{'name'} eq $1 &&
				     $self->indexof($_[0], @{$g->{'members'}}) >= 0);
			}
		}
	else {
		return 1 if ($u eq $_[0]);
		}
	}
return 0;
}

# open_session_db(\%miniserv)
sub open_session_db
{
	my $self = shift;

local $sfile = $_[0]->{'sessiondb'} ? $_[0]->{'sessiondb'} :
	       $_[0]->{'pidfile'} =~ /^(.*)\/[^\/]+$/ ? "$1/sessiondb"
						      : return;
eval "use SDBM_File";
dbmopen(%{$self->{'sessiondb_hash'}}, $sfile, 0700);
eval { $self->{'sessiondb_hash'}->{'1111111111'} = 'foo bar' };
if ($@) {
	dbmclose(%{$self->{'sessiondb_hash'}});
	eval "use NDBM_File";
	dbmopen(%{$self->{'sessiondb_hash'}}, $sfile, 0700);
	}
else {
	delete($self->{'sessiondb_hash'}->{'1111111111'});
	}
}

# delete_session_id(\%miniserv, id)
# Deletes one session from the database
sub delete_session_id
{
	my $self = shift;

return 1 if ($self->is_readonly_mode());
$self->open_session_db($_[0]);
local $ex = exists($self->{'sessiondb_hash'}->{$_[1]});
delete($self->{'sessiondb_hash'}->{$_[1]});
dbmclose(%{$self->{'sessiondb_hash'}});
return $ex;
}

# delete_session_user(\%miniserv, user)
# Deletes all sessions for some user
sub delete_session_user
{
	my $self = shift;

return 1 if ($self->is_readonly_mode());
$self->open_session_db($_[0]);
foreach my $s (keys %sessiondb) {
	local ($u,$t) = split(/\s+/, $sessiondb{$s});
	if ($u eq $_[1]) {
		delete($sessiondb{$s});
		}
	}
dbmclose(%sessiondb);
}

# rename_session_user(\%{$self->{'miniserv_hash'}}, olduser, newuser)
# Changes the username in all sessions for some user
sub rename_session_user
{
	my $self = shift;

return 1 if ($self->is_readonly_mode());
$self->open_session_db(\%{$self->{'miniserv_hash'}});
foreach my $s (keys %sessiondb) {
	local ($u,$t) = split(/\s+/, $sessiondb{$s});
	if ($u eq $_[1]) {
		$sessiondb{$s} = "$_[2] $t";
		}
	}
dbmclose(%sessiondb);
}

# $self->update_members(&allusers, &allgroups, &modules, &members)
# Update the members users and groups of some group
sub update_members
{
	my $self = shift;

local $m;
foreach $m (@{$_[3]}) {
	if ($m !~ /^\@(.*)$/) {
		# Member is a user
		local ($u) = grep { $_->{'name'} eq $m } @{$_[0]};
		if ($u) {
			$u->{'modules'} = [ @{$_[2]}, @{$u->{'ownmods'}} ];
			$self->modify_user($u->{'name'}, $u);
			}
		}
	else {
		# Member is a group
		local $gname = substr($m, 1);
		local ($g) = grep { $_->{'name'} eq $gname } @{$_[1]};
		if ($g) {
			$g->{'modules'} = [ @{$_[2]}, @{$g->{'ownmods'}} ];
			$self->modify_group($g->{'name'}, $g);
			$self->update_members($_[0], $_[1], $g->{'modules'},
					$g->{'members'});
			}
		}
	}
}

# copy_acl_files(from, to, &modules)
# Copy all .acl files from some user to another in a list of modules
sub copy_acl_files
{
	my $self = shift;

local $m;
foreach $m (@{$_[2]}) {
	$self->unlink_file("$self->{'config_directory'}/$m/$_[1].acl");
	local %acl;
	if ($self->read_file("$self->{'config_directory'}/$m/$_[0].acl", \%acl)) {
		$self->write_file("$self->{'config_directory'}/$m/$_[1].acl", \%acl);
		}
	}
}

# copy_group_acl_files(from, to, &modules)
# Copy all .acl files from some group to another in a list of modules
sub copy_group_acl_files
{
	my $self = shift;

local $m;
foreach $m (@{$_[2]}) {
	$self->unlink_file("$self->{'config_directory'}/$m/$_[1].gacl");
	local %acl;
	if ($self->read_file("$self->{'config_directory'}/$m/$_[0].gacl", \%acl)) {
		$self->write_file("$self->{'config_directory'}/$m/$_[1].gacl", \%acl);
		}
	}
}

# copy_group_user_acl_files(from, to, &modules)
# Copy all .acl files from some group to a user in a list of modules
sub copy_group_user_acl_files
{
	my $self = shift;

local $m;
foreach $m (@{$_[2]}) {
	$self->unlink_file("$self->{'config_directory'}/$m/$_[1].acl");
	local %acl;
	if ($self->read_file("$self->{'config_directory'}/$m/$_[0].gacl", \%acl)) {
		$self->write_file("$self->{'config_directory'}/$m/$_[1].acl", \%acl);
		}
	}
}

# $self->set_acl_files(&allusers, &allgroups, module, &members, &access)
# Recursively update the ACL for all sub-users and groups of a group
sub set_acl_files
{
	my $self = shift;

local $m;
foreach $m (@{$_[3]}) {
	if ($m !~ /^\@(.*)$/) {
		# Member is a user
		local ($u) = grep { $_->{'name'} eq $m } @{$_[0]};
		if ($u) {
			local $aclfile =
				"$self->{'config_directory'}/$_[2]/$u->{'name'}.acl";
			$self->lock_file($aclfile);
			$self->write_file($aclfile, $_[4]);
			chmod(0640, $aclfile);
			$self->unlock_file($aclfile);
			}
		}
	else {
		# Member is a group
		local $gname = substr($m, 1);
		local ($g) = grep { $_->{'name'} eq $gname } @{$_[1]};
		if ($g) {
			local $aclfile =
				"$self->{'config_directory'}/$_[2]/$g->{'name'}.gacl";
			$self->lock_file($aclfile);
			$self->write_file($aclfile, $_[4]);
			chmod(0640, $aclfile);
			$self->unlock_file($aclfile);
			$self->set_acl_files($_[0], $_[1], $_[2], $g->{'members'}, $_[4]);
			}
		}
	}
}

# get_ssleay()
# Returns the path to OpenSSL or equivalent
sub get_ssleay
{
	my $self = shift;

if ($self->has_command($self->{'config_hash'}->{'ssleay'})) {
	return $self->has_command($self->{'config_hash'}->{'ssleay'});
	}
elsif ($self->has_command("openssl")) {
	return $self->has_command("openssl");
	}
elsif ($self->has_command("ssleay")) {
	return $self->has_command("ssleay");
	}
else {
	return undef;
	}
}

# encrypt_password(password, [salt])
sub encrypt_password
{
	my $self = shift;

local ($pass, $salt) = @_;
if ($self->{'gconfig_hash'}->{'md5pass'}) {
	$salt ||= substr(time(), -8);
	return crypt($pass, '$1$'.$salt);
	}
else {
	$self->seed_random();
	$salt ||= chr(int(rand(26))+65).chr(int(rand(26))+65);
	return $self->unix_crypt($pass, $salt);
	}
}

# get_unixauth(\%miniserv)
# Returns a list of Unix users/groups/all and the Webmin user that they
# authenticate as
sub get_unixauth
{
	my $self = shift;

local @rv;
local @ua = split(/\s+/, $_[0]->{'unixauth'});
foreach my $ua (@ua) {
	if ($ua =~ /^(\S+)=(\S+)$/) {
		push(@rv, [ $1, $2 ]);
		}
	else {
		push(@rv, [ "*", $ua ]);
		}
	}
return @rv;
}

# save_unixauth(\%miniserv, &authlist)
# Updates %miniserv with the given Unix auth list
sub save_unixauth
{
	my $self = shift;

local @ua;
foreach my $ua (@{$_[1]}) {
	if ($ua->[0] ne "*") {
		push(@ua, "$ua->[0]=$ua->[1]");
		}
	else {
		push(@ua, $ua->[1]);
		}
	}
$_[0]->{'unixauth'} = join(" ", @ua);
}

# delete_from_groups(user|@group)
# Removes the specified user from all groups
sub delete_from_groups
{
	my $self = shift;

local ($user) = @_;
foreach my $g ($self->list_groups()) {
	local @mems = @{$g->{'members'}};
	local $i = $self->indexof($user, @mems);
	if ($i >= 0) {
		splice(@mems, $i, 1);
		$g->{'members'} = \@mems;
		$self->modify_group($g->{'name'}, $g);
		}
	}
}


1;

