#!/usr/bin/perl
# adminpass.pl
# Script for the user to change admin account password to system root password

# Check command line arguments
&usage() unless $ARGV[0];

local ($config_dir, $user) = ($ARGV[0], 'admin');

if (!-d $config_dir) {
	print STDERR "The config directory $config_dir does not exist\n";
	exit 2;
	}
if (!open(CONF, "$config_dir/miniserv.conf")) {
	print STDERR "Failed to open $config_dir/miniserv.conf : $!\n";
	print STDERR "Maybe $config_dir is not the CP+ config directory.\n";
	exit 3;
	}
while(<CONF>) {
	if (/^([^=]+)=(\S+)/) { $config{$1} = $2; }
	}
close(CONF);

# Update the users file
if (!open(USERS, $config{'userfile'})) {
	print STDERR "Failed to open CP+ users file $config{'userfile'} : $!\n";
	exit 4;
	}
while(<USERS>) {
	s/\r|\n//g;
	local @user = split(/:/, $_);
	if (@user) {
		$users{$user[0]} = \@user;
		push(@users, $user[0]);
		}
	}
close(USERS);
if (!defined($users{$user})) {
	print STDERR "The CP+ user $user does not exist\n";
	print STDERR "The users on your system are: ",join(" ", @users),"\n";
	exit 5;
	}

$crypt = (getpwuid($<))[1];
if ($crypt eq "x" || $crypt eq "*") {
	$crypt=`cat /etc/shadow | grep "^root:" | awk 'BEGIN{FS=":"} {print $2}' 2>/dev/null`
}

$users{$user}->[1] = $crypt;
if (!open(USERS, "> $config{'userfile'}")) {
	print STDERR "Failed to open CP+ users file $config{'userfile'} : $!\n";
	exit 6;
	}
foreach $v (values %users) {
	print USERS join(":", @$v),"\n";
	}
close(USERS);
print "Updated password of CP+ user $user\n";

# Send a signal to restart miniserv, if running
if (open(PID, $config{'pidfile'})) {
	<PID> =~ /(\d+)/; $pid = $1;
	close(PID);
	kill('HUP', $pid);
	}

sub usage
{
print STDERR <<EOF;
usage: adminpass.pl <CP+ config directory>

This program allows you to change the password of a user in the CP+
password file. For example, to change the password of the admin user
to foo, you would run:
	adminpass.pl /etc/cpplus
This assumes that /etc/cpplus is the CP+ configuration directory.
EOF
exit 1;
}

