#!/usr/bin/perl
package amavis::AmavisLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'amavis::AmavisLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# amavisd-lib.pl
# common functions for amavis configuration

# main part
&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

1;
1;
		return $self;
}

# get_version_amavisd()
# return amavisd version or 0
sub get_version_amavisd
{
	my $self = shift;

 # from amavis
 my $version;
 open(AMAVIS, $self->{'config_hash'}->{'amavisd_cmd'});
 while (<AMAVIS>)
 {
  if ($_=~/^\$myversion_id\s*?=\s*?'([0-9]+?\.[0-9]+?\.[0-9]+?)'\s*?;.*$/)
  {
   $version = $1;
   last;
  }
 } 
 close AMAVIS;
 return $version if ($version);
 
 # from rpm
 return $self->get_rpm_version();

 return 0;
}

# get_rpm_version()
# returns amavisd-new rpm version or 0
sub get_rpm_version 
{
	my $self = shift;

 my $out;
 my $rpm_opts = "-q amavisd-new --queryformat \"\%\{VERSION\}\"";
 if (-x '/bin/rpm') 
 {
  $out = `/bin/rpm $rpm_opts 2>/dev/null`;
  return $out if !$? && $out =~ /^[\d\.]+$/;
 }
 elsif ($self->has_command('rpm')) 
 {
  $out = `rpm $rpm_opts 2>/dev/null`;
  return $out if !$? && $out =~ /^[\d\.]+$/;
 }
 return 0;
}

# get_pid()
# return process pid or 0
sub get_pid
{
	my $self = shift;

 if ($self->{'config_hash'}->{'pid_file'})
 {
  return $self->check_pid_file($self->{'config_hash'}->{'pid_file'});
 }
 elsif ($self->{'config_hash'}->{'amavisd_conf'})
 {
  my $conf = $self->parse_conf_file();
  my $value = $self->get_config_value('$pid_file',$conf);
  return $self->check_pid_file($value->{'check'});
 }
 else { return 0; }
}

# get_home()
# return path to amavis home dir or 0
sub get_home
{
	my $self = shift;

 if ($self->{'config_hash'}->{'home_dir'})
 {
  return $self->{'config_hash'}->{'home_dir'};
 }
 elsif ($self->{'config_hash'}->{'amavisd_conf'})
 {
  my $conf = $self->parse_conf_file();
  my $value = $self->get_config_value('$MYHOME',$conf);
  return $value->{'check'};
 }
 else { return 0; }
}

# get_quarantine()
# return path to amavis quarantine dir or 0
sub get_quarantine
{
	my $self = shift;

 if ($self->{'config_hash'}->{'quarantine_dir'})
 {
  return $self->{'config_hash'}->{'quarantine_dir'};
 }
 elsif ($self->{'config_hash'}->{'amavisd_conf'})
 {
  my $conf = $self->parse_conf_file();
  my $value = $self->get_config_value('$QUARANTINEDIR',$conf);
  return $value->{'check'};
 }
 else { return 0; }
}

# get_log_file()
# return path to amavis log file or 0
sub get_log_file
{
	my $self = shift;

 if ($self->{'config_hash'}->{'amavisd_conf'})
 {
  my $conf = $self->parse_conf_file();
  my $syslog = $self->get_config_value('$DO_SYSLOG',$conf);
  if ($syslog->{'check'}){ return 0; }
  else
  {
   # get amavis log
   if ($self->{'config_hash'}->{'log_file'})
   {
    # from module config
    return $self->{'config_hash'}->{'log_file'};
   }
   else
   {
    # from amavisd config   
    my $logfile = $self->get_config_value('$LOGFILE',$conf);
    return $logfile->{'check'};
   }
  }
 }
 else { return 0; }
}

# status_amavisd()
# return 1 if daemon start, return 0 if daemon stop or unrecognize
sub status_amavisd
{
	my $self = shift;

 if ($self->{'config_hash'}->{'status_cmd'})
 {
  # check with script
  my $out = $self->backquote_logged("$self->{'config_hash'}->{'status_cmd'} 2>&1 </dev/null");
  return 1 if !$?;
 }
 else
 {
  # check with pid
  my $pid = $self->get_pid();
  return 1 if $pid;
 }
 # service stoped or unrecognize
 return 0;
}

# start_amavisd()
# start daemon amavisd or call &error
sub start_amavisd
{
	my $self = shift;

 unless ($self->status_amavisd())
 {
  # service stoped
  if ($self->{'config_hash'}->{'start_cmd'})
  {
   # start with script
   my $out = $self->backquote_logged("$self->{'config_hash'}->{'start_cmd'} 2>&1 </dev/null");
   if ($?){ $self->error($out); }
   else { $self->webmin_log("start"); }
  }
  elsif ($self->{'config_hash'}->{'saslauthd_cmd'})
  {
   # start with command
   my $out = $self->backquote_logged("$self->{'config_hash'}->{'amavisd_cmd'} -c $self->{'config_hash'}->{'amavisd_conf'} start 2>&1 </dev/null");
   if ($?){ $self->error($out); }
   else { $self->webmin_log("start"); }
  }
  else { $self->error($self->{'text_hash'}->{'error_start'}); }
 }
 else
 {
  # service started
  $self->error($self->{'text_hash'}->{'proces_status'});
 }
}

# stop_amavisd()
# stop daemon amavisd or call &error
sub stop_amavisd
{
	my $self = shift;

 if ($self->status_amavisd())
 {
  # service started
  if ($self->{'config_hash'}->{'stop_cmd'})
  {
   # stop with script
   my $out = $self->backquote_logged("$self->{'config_hash'}->{'stop_cmd'} 2>&1 </dev/null");
   if ($?){ $self->error($out); }
   else { $self->webmin_log("stop"); }
  }
  else
  {
   # stop with pid
   my $pid = $self->get_pid();
   if ($pid)
   {
    $self->kill_logged('TERM', $pid);
    $self->webmin_log("stop");
   }
   else
   {
    # stop with name
    $self->kill_logged('TERM', $self->find_byname("amavisd"));
    $self->webmin_log("stop");
   }
  }
 }
 else
 {
  # service stoped
  $self->error($self->{'text_hash'}->{'proces_status2'});
 }
}

# restart_amavisd()
# restart daemon amavisd or call &error
sub restart_amavisd
{
	my $self = shift;

 if ($self->status_amavisd())
 {
  # service started
  if ($self->{'config_hash'}->{'restart_cmd'})
  {
   # restart with script
   my $out = $self->backquote_logged("$self->{'config_hash'}->{'restart_cmd'} 2>&1 </dev/null");
   if ($?){ $self->error($out); }
   else { $self->webmin_log("restart"); }
  }
  else
  {
   # restart with pid
   my $pid = $self->get_pid();
   if ($pid)
   {
    $self->kill_logged('HUP', $pid);
    $self->webmin_log("restart");
   }
   else
   {
    # restart with name
    $self->kill_logged('HUP', $self->find_byname("amavisd"));
    $self->webmin_log("restart");
   }
  }
 }
 else
 {
  # service stoped
  $self->error($self->{'text_hash'}->{'proces_status2'});
 }
}

# parse_conf_file([file])
# return hash with name=value pairs from amavisd configuration file or call &error
sub parse_conf_file
{
	my $self = shift;

 my $conf_file = $_[0] ? $_[0] : $self->{'config_hash'}->{'amavisd_conf'};
 $self->lock_file($conf_file);
 my $conf;
 open (INFL,$conf_file) || $self->error($self->{'text_hash'}->{'error_open'});
 while (<INFL>)
 {
  chomp;
  $self->trunc_space($_);
  if ($_=~/^\$[A-Za-z0-9_]+?\s*?=\s*?.*?;.*$/ && !($_=~/^#/))
  {
   my @options=split (/=/,$_);
   $options[0]=$self->trunc_space($options[0]);
   $options[1]=$self->trunc_space($options[1]);
   $conf -> {$options[0]} = $options[1];
  }
 }
 close INFL;
 $self->unlock_file($conf_file);
 return $conf;
}

# $self->get_config_value(name, \%hash)
# return value for name from hash
sub get_config_value
{
	my $self = shift;

 my ($full,$value) = $_[1]->{$_[0]};
 $value->{'full_value'} = $full;
 if ($full =~ /^'(.*?)'\s*?;.*$/)
 {
  $value->{'value'} = $value->{'check'} = $1;
 }
 elsif ($full =~ /^"(.*?)"\s*?;.*$/ || $full =~ /^(.*?);.*$/)
 {
  $value->{'value'} = $full = $1;
  while ($full =~ /(\$[A-Za-z0-9_]+)/)
  {
   my $input = $self->get_config_value($1,$_[1]);
   my $ch = $input->{'check'};
   $full =~ s/(\$[A-Za-z0-9_]+)/$ch/;
   $full =~ (/^"(.*)"\s*;.*$/ || /^(.*);.*$/);
  }
  $value->{'check'} = $full;
 }
 return $value;
}

# set_new_value(name,value,\%hash)
# set new value for name from hash
sub set_new_value
{
	my $self = shift;

 if (($_[1] eq 'D_REJECT') || ($_[1] eq 'D_PASS') || ($_[1] eq 'D_DISCARD') || ($_[1] eq 'D_BOUNCE') || ($_[1] eq 'undef'))
 {
  $_[2]->{$_[0]}="$_[1];";
 }
 elsif ($_[1] =~ /(\$[A-Za-z0-9_]+)/)
 {
  $_[2]->{$_[0]}="\"$_[1]\";";
 }
 elsif ($_[1] =~ /^[0-9]+?\.?[0-9]*?$/ || $_[1] =~ /^[0-9]+?\.?[0-9]*?\*/ || $_[1] =~ /^\[/)
 {
  $_[2]->{$_[0]}="$_[1];";
 }
 else
 {
  $_[2]->{$_[0]}="'$_[1]';";
 }
}

# del_config_value(name,\%hash)
# del name from hash
sub del_config_value
{
	my $self = shift;

 delete $_[1]->{$_[0]};
}

# save_new_options(\%hash,[file])
# save all new options from hash to amavis configuration file, change union options value, delete all other options
sub save_new_options
{
	my $self = shift;

 # get config file
 my $conf_file = $_[1] ? $_[1] : $self->{'config_hash'}->{'amavisd_conf'};
  
 # parse config file
 $self->lock_file($conf_file);
 my $old_conf = $self->parse_conf_file($conf_file);
 
 # get new options
 my @new;
 foreach (keys %{$_[0]})
 {
  unless (exists $old_conf->{$_}) { push (@new,$_); }
 }
 # get del & union options
 my (@del,@union);
 foreach (keys %{$old_conf})
 {
  if (exists $_[0]->{$_}) { push (@union,$_); }
  else { push (@del,$_); }
 }
 # read config file
 my $old_file= $self->read_file_lines($conf_file);
 
 # modify options
 foreach (@union)
 {
  foreach my $i(@{$old_file})
  {
   if ($i=~/^\$[A-Za-z0-9_]+?\s*?=\s*?.*?;.*$/ && !($i=~/^#/))
   {
    my @str_old=split (/=/,$i);
    my $str=$self->trunc_space($str_old[0]);
    if ($str eq $_)
    {
     $self->{'value'} = $self->get_config_value($_,$_[0]);
     $self->{'value_old'} = $self->get_config_value($_,$old_conf);
     $i="$_ = $_[0]->{$_}" unless ($self->{'value'}->{'value'} eq $self->{'value_old'}->{'value'});
    }
   }
  }
 }
 # delete options
 foreach (@del)
 {
  my @temp;
  foreach my $i(@{$old_file})
  {
   my @str_old=split (/=/,$i);
   my $str=$self->trunc_space($str_old[0]);
   push (@temp,$i) unless ($str eq $_);
  }
  @{$old_file}=@temp;
 }
 # add new options
 foreach (@new)
 {
  my (@temp,$comment);
  foreach my $i (@{$old_file})
  {
   # add new options under comment
   push (@temp,$i);
   my $name=quotemeta ($_);
   if (!$comment && $i=~/^#\s*$name\s*=/)
   {
    push (@temp,"$_ = $_[0]->{$_}");
    $comment=1;
   }
  }
  # add new options in the end of file
  my $end=pop @temp;
  push (@temp,"$_ = $_[0]->{$_}") if (!$comment);
  push (@temp,$end);
  @{$old_file}=@temp;
 } 
 # save config file
 $self->flush_file_lines();
 $self->unlock_file($conf_file);
}

# get_option_list()
# return array of configuration options (section,name,text,type,version,glob/adv) for installed amavisd version
# type: 1 - (yes,no,default), 2 - (select,default),0 - (text,default)
sub get_option_list
{
	my $self = shift;

 my $version = $self->get_version_amavisd();
 my @option = (
 		#---Daemon and MTA Settings---
		['daemon','MYHOME','myhome',0,0],
		['daemon','mydomain','mydomain',0,0],
		['daemon','myhostname','myhostname',0,0],
		['daemon','daemon_user','daemon_user',0,0],
		['daemon','daemon_group','daemon_group',0,0],
		['daemon','TEMPBASE','tempbase',0,0],
		['daemon','db_home','db_home',0,0],
		['daemon','helpers_home','helpers_home',0,0],
		['daemon','daemon_chroot_dir','daemon_chroot_dir',0,0],
		['daemon','pid_file','pid_file',0,0],
		['daemon','lock_file','lock_file',0,0],
		#['daemon','ENV{TMPDIR}','env',0,0,1],
		['daemon','enable_db','enable_db',1,0],
		['daemon','enable_global_cache','enable_global_cache',1,0],
		['daemon','forward_method','forward_method',0,0],
		['daemon','notify_method','notify_method',0,0],
		['daemon','max_servers','max_servers',0,0],
		['daemon','max_requests','max_requests',0,0],
		['daemon','child_timeout','child_timeout',0,0],
		#---MTA specific Options---
		['mta','insert_received_line','insert_received_line',0,0],
		['mta','unix_socketname','unix_socketname',0,0],
		['mta','inet_socket_port','inet_socket_port',0,0],
		['mta','inet_socket_bind','inet_socket_bind',0,0],
		#---Logging Options---
		['logging','DO_SYSLOG','do_syslog',1,0],
		['logging','SYSLOG_LEVEL','syslog_level',0,0],
		['logging','LOGFILE','log file',0,0],
		['logging','log_level','log_level',0,0],
		['logging','log_templ','log_templ',0,0],
		['logging','log_recip_templ','log_recip_templ',0,0],
		#---Notifications Options---
		['notif','hdr_encoding','hdr_encoding',0,0],
		['notif','hdr_encoding_qb','hdr_encoding_qb',0,0],
		['notif','bdy_encoding','bdy_encoding',0,0],
		['notif','notify_sender_templ','notify_sender_templ',0,0],
		['notif','notify_virus_sender_templ','notify_virus_sender_templ',0,0],
		['notif','notify_virus_admin_templ','notify_virus_admin_templ',0,0],
		['notif','notify_virus_recips_templ','notify_virus_recips_templ',0,0],
		['notif','notify_spam_sender_templ','notify_spam_sender_templ',0,0],
		['notif','notify_spam_admin_templ','notify_spam_admin_templ',0,0],
		['notif','warnvirussender','warnvirussender',1,0],
		['notif','warnspamsender','warnspamsender',1,0],
		['notif','warnbannedsender','warnbannedsender',1,0],
		['notif','warnbadhsender','warnbadhsender',1,0],
		['notif','warnvirusrecip','warnvirusrecip',1,0],
		['notif','warnbadhrecip','warnbadhrecip',1,0],
		['notif','warnbannedrecip','warnbannedrecip',1,0],
		['notif','warn_offsite','warn_offsite',1,0],
		['notif','virus_admin','virus_admin',0,0],
		['notif','spam_admin','spam_admin',0,0],
		['notif','mailfrom_notify_admin','mailfrom_notify_admin',0,0],
		['notif','mailfrom_notify_recip','mailfrom_notify_recip',0,0],
		['notif','mailfrom_notify_spamadmin','mailfrom_notify_spamadmin',0,0],
		['notif','hdrfrom_notify_sender','hdrfrom_notify_sender',0,0],
		['notif','hdrfrom_notify_admin','hdrfrom_notify_admin',0,0],
		['notif','hdrfrom_notify_spamadmin','hdrfrom_notify_spamadmin',0,0],
		['notif','X_HEADER_TAG','X_HEADER_TAG',0,0],
		['notif','X_HEADER_LINE','X_HEADER_LINE',0,0],
		['notif','undecipherable_subject_tag','undecipherable_subject_tag',0,0],
		['notif','defang_virus','defang_virus',1,0],
		['notif','defang_banned','defang_banned',1,0],
		['notif','defang_bad_header','defang_bad_header',1,0],
		['notif','defang_undecipherable','defang_undecipherable',1,0],
		['notif','defang_spam','defang_spam',1,0],
		['notif','remove_existing_x_scanned_headers','remove_existing_x_scanned_headers',1,0],
		['notif','remove_existing_spam_headers','remove_existing_spam_headers',1,0],
		['notif','bypass_decode_parts','bypass_decode_parts',1,0],
		['notif','banned_filename_re','banned_filename_re',0,0],
		['notif','banned_namepath_re','banned_namepath_re',0,0],
		#---Virus scanners---
		['notif','first_infected_stops_scan','first_infected_stops_scan',1,0],
		#---Destiny Options---
		['destiny','final_virus_destiny','final_virus_destiny',2,0],
		['destiny','final_banned_destiny','final_banned_destiny',2,0],
		['destiny','final_spam_destiny','final_spam_destiny',2,0],
		['destiny','final_bad_header_destiny','final_bad_header_destiny',2,0],
		#---Quarantine Options---
		['quarantine','mailfrom_to_quarantine','mailfrom_to_quarantine',0,0],
		['quarantine','QUARANTINEDIR','QUARANTINEDIR',0,0],
		['quarantine','virus_quarantine_method','virus_quarantine_method',0,0],
		['quarantine','spam_quarantine_method','spam_quarantine_method',0,0],
		['quarantine','banned_files_quarantine_method','banned_files_quarantine_method',0,0],
		['quarantine','bad_header_quarantine_method','bad_header_quarantine_method',0,0],
		['quarantine','virus_quarantine_to','virus_quarantine_to',0,0],
		['quarantine','banned_quarantine_to','banned_quarantine_to',0,0],
		['quarantine','bad_header_quarantine_to','bad_header_quarantine_to',0,0],
		['quarantine','spam_quarantine_to','spam_quarantine_to',0,0],
		['quarantine','spam_quarantine_bysender_to','spam_quarantine_bysender_to',0,0],
		#---Per-recipient and Per-sender Options---
		['per','sql_select_policy','sql_select_policy',0,0],
		['per','sql_select_white_black_list','sql_select_white_black_list',0,0],
		['per','recipient_delimiter','recipient_delimiter',0,0],
		['per','replace_existing_extension','replace_existing_extension',1,0],
		['per','localpart_is_case_sensitive','localpart_is_case_sensitive',1,0],
		#---Resource limits---
		['limits','smtpd_recipient_limit','smtpd_recipient_limit',0,0],
		['limits','MAXLEVELS','MAXLEVELS',0,0],
		['limits','MAXFILES','MAXFILES',0,0],
		['limits','MIN_EXPANSION_QUOTA','MIN_EXPANSION_QUOTA',0,0],
		['limits','MAX_EXPANSION_QUOTA','MAX_EXPANSION_QUOTA',0,0],
		['limits','MIN_EXPANSION_FACTOR','MIN_EXPANSION_FACTOR',0,0],
		['limits','MAX_EXPANSION_FACTOR','MAX_EXPANSION_FACTOR',0,0],
		['limits','virus_check_negative_ttl','virus_check_negative_ttl',0,0],
		['limits','virus_check_positive_ttl','virus_check_positive_ttl',0,0],
		['limits','spam_check_negative_ttl','spam_check_negative_ttl',0,0],
		['limits','spam_check_positive_ttl','spam_check_positive_ttl',0,0],
		#---External programs---
		['external','path','path',0,0],
		['external','file','file',0,0],
		['external','gzip','gzip',0,0],
		['external','bzip2','bzip2',0,0],
		['external','lzop','lzop',0,0],
		['external','rpm2cpio','rpm2cpio',0,0],
		['external','cabextract','cabextract',0,0],
		['external','uncompress','uncompress',0,0],	
		['external','unfreeze','unfreeze',0,0],
		['external','arc','arc',0,0],
		['external','unarj','unarj',0,0],
		['external','unrar','unrar',0,0],
		['external','zoo','zoo',0,0],
		['external','lha','lha',0,0],
		['external','cpio','cpio',0,0],
		['external','ar','ar',0,0],
		['external','dspam','dspam',0,0],
		#---SpamAssassin Options---
		['spam','sa_local_tests_only','sa_local_tests_only',1,0],
		['spam','sa_auto_whitelist','sa_auto_whitelist',1,0],
		['spam','sa_mail_body_size_limit','sa_mail_body_size_limit',0,0],
		['spam','sa_tag_level_deflt','sa_tag_level_deflt',0,0],
		['spam','sa_tag2_level_deflt','sa_tag2_level_deflt',0,0],
		['spam','sa_kill_level_deflt','sa_kill_level_deflt',0,0],
		['spam','sa_dsn_cutoff_level','sa_dsn_cutoff_level',0,0],
		['spam','sa_spam_subject_tag','sa_spam_subject_tag',0,0],
		['spam','sa_spam_level_char','sa_spam_level_char',0,0],
		['spam','sa_spam_report_header','sa_spam_report_header',1,0],
		['spam','sa_debug','sa_debug',1,0]);
 my @option_ver;
 my @version = split (/\./,$version);
 my $v1=$version[0]*1000000+$version[1]*1000+$version[2];
 foreach (@option)
 {
  my @vers_opt = split (/\./,$_->[4]);
  push (@option_ver,$_) if ($v1 >= ($vers_opt[0]*1000000+$vers_opt[1]*1000+$vers_opt[2]));
 }
 return \@option_ver;
}

# choice_input(name,value,check)
# print checkbox (yes,no,default) 
sub choice_input
{
	my $self = shift;

 my ($value,$check,$yes,$no,$default) = ($_[1],$_[2]);
 if ($check && $value) {$yes = 'checked';}
 elsif ($check && !$value){$no = 'checked';}
 else {$default = 'checked';}
 my $chekbox = "<input type=radio name=$_[0] value='' $default>Default &nbsp;\n";
 $chekbox .= "<input type=radio name=$_[0] value=1 $yes>Yes &nbsp;\n";
 $chekbox .= "<input type=radio name=$_[0] value=0 $no>No &nbsp;\n"; 
 $chekbox .= "<input type=hidden name=$_[0]_def value=1>\n";
 return $chekbox;
}

# text_input(name,value,check)
# print checkbox & text input (value,default)
sub text_input
{
	my $self = shift;

 my ($value,$check,$yes,$default) = ($_[1],$_[2]);
 if ($check) {$yes = 'checked';}
 else {$default = 'checked';}
 my $input = "<input type=radio name=$_[0]_def value=0 $default>Default &nbsp;\n";
 $input .="<input type=radio name=$_[0]_def value=1 $yes> &nbsp;\n";
 $input .= "<input type=text name=$_[0] value=\"$value\"> &nbsp;\n";
 return $input;
}

# text_select(name,value,check)
# print checkbox & select (value,default)
sub text_select
{
	my $self = shift;

 my ($value,$check,$yes,$default) = ($_[1],$_[2]);
 if ($check) {$yes = 'checked';}
 else {$default = 'checked';}
 my $input = "<input type=radio name=$_[0]_def value=0 $default>Default &nbsp;\n";
 $input .="<input type=radio name=$_[0]_def value=1 $yes> &nbsp;\n";
 $input .= "<select name=$_[0]>\n";
 my @d = ('D_REJECT','D_PASS','D_DISCARD','D_BOUNCE');
 foreach (@d)
 {
  if ($value eq $_)
  {
   $input .= "<option selected>$_";
  }
  else
  {
   $input .= "<option>$_";
  }
 }
 $input .= "</select>\n";
 return $input;
}

# show_log(file,[lines])
# print all log file or some lines
# print clear submit
sub show_log 
{
	my $self = shift;

 my ($file, $lines) = ($_[0], $_[1]);
 print $self->ui_form_start('log_view.cgi', 'POST');
 print $self->ui_hidden('logs', $file);
 print $self->ui_table_start("$self->{'text_hash'}->{'log_info'}", 'width=100%');
 if ($lines) 
 {
  @{$self->{'text_array'}} = `/usr/bin/tail -n $lines $file`;
 } 
 else 
 {
  open (LOG, "$file");
  @{$self->{'text_array'}} = <LOG>;
  close (LOG);
 }
 print $self->ui_table_row(qq(<textarea cols=80 rows=30>@{$self->{'text_array'}}</textarea>\n), "");
 print $self->ui_table_span();
 print $self->ui_table_end();
 print $self->ui_form_end([['clear', $self->{'text_hash'}->{'log_clear'}]]);
}

# trunc_space(string)
# delete spaces from string
sub trunc_space
{
	my $self = shift;

 local $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}

#---OLD FUNCS---
#parses file $_[0] and returns a reference to an array of hashes (spliter - =)
sub parse_config_file
{
	my $self = shift;

 my @options,$conf_file;
 open (INFL,$_[0]) || $self->error($self->{'text_hash'}->{'error_open'});
 while (<INFL>)
 {
  chomp;
  $self->trunc_space($_);
  unless ($_=~/^#/ || $_=~/^$/)
  {
   if ($_=~/^\$/)
   {
    @options=split (/=/,$_);
    $options[0]=$self->trunc_space($options[0]);
    $options[1]=$self->trunc_space($options[1]);
    push (@{$conf_file},{'name'=>$options[0],'value'=>$options[1]});
   }
  }
 }
 close INFL;
 return $conf_file;
}

#set new value $_[1] of parameter with name $_[0] from array $_[2]
#or add new string to array $_[2]
sub set_config_value
{
	my $self = shift;

 my $str_old;
 foreach (@{$_[2]})
 {
  if ($_->{'name'} eq $_[0])
  {
   $_->{'value'}=$_[1];
   $str_old=1;
  }
 }
 push (@{$_[2]},{'name'=>$_[0],'value'=>$_[1]}) if ($str_old!=1);
}

#save hash $_[0] to amavis config file
sub save_config_file
{
	my $self = shift;

 $self->lock_file($self->{'config_hash'}->{'amavisd_conf'});
 my $old_file= $self->read_file_lines($self->{'config_hash'}->{'amavisd_conf'});
 foreach (keys %{$_[0]})
 {
  my $old;
  my $comment;
  foreach my $i (@{$old_file})
  {
   if ($i=~/^\$/)
   {
    @{$self->{'str_old_array'}}=split (/=/,$i);
    $self->{'str_old_array'}->[0]=$self->trunc_space($self->{'str_old_array'}->[0]);
    if (($_ eq $self->{'str_old_array'}->[0]))
    {
     #modify old options
     $i="$_=$_[0]->{$_}";
     $old=1;
     last;
    }
   }
  }
  my @temp;
  foreach my $i (@{$old_file})
  {
   push (@temp,$i);
   $self->{'name'}=quotemeta ($_);
   if (!$old && !$comment && $i=~/^#\s*$self->{'name'}\s*=/)
   {
    push (@temp,"$_=$_[0]->{$_}");
    $comment=1;
   }
  }
  if (!$comment && !$old)
  {
   #add new options in the end of file
   my $end=pop @temp;
   push (@temp,"$_=$_[0]->{$_}");
   push (@temp,$end);
  }
  @{$old_file}=@temp;
 }
 $self->flush_file_lines();
 $self->unlock_file($self->{'config_hash'}->{'amavisd_conf'});
}


1;

