#!/usr/bin/perl
package apache::ApacheLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'apache::ApacheLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# apache-lib.pl
# Common functions for apache configuration

$self->{'directive_type_count'} = 20;

if ($self->{'module_name'} ne 'htaccess') {
	&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
	$self->init_config();
	&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
	%{$self->{'access_hash'}} = $self->get_module_acl();
	@{$self->{'access_types_array'}} = $self->{'access_hash'}->{'types'} eq '*' ? (0 .. $self->{'directive_type_count'})
					: split(/\s+/, $self->{'access_hash'}->{'types'});
	}
else {
	@{$self->{'access_types_array'}} = (0 .. $self->{'directive_type_count'});
	}
map { $self->{'access_types_hash'}->{$_}++ } @{$self->{'access_types_array'}};
$self->{'site_file'} = ($self->{'config_hash'}->{'webmin_apache'} || $self->{'module_config_directory'})."/site";

# Check if a list of supported modules needs to be built
if ($self->{'module_name'} ne 'htaccess') {
	 %{$self->{'oldsite_hash'}};
	 $self->{'httpd'} = $self->find_httpd();
	 @{$self->{'st_array'}} = stat($self->{'httpd'});
	$self->read_file($self->{'site_file'}, \%{$self->{'oldsite_hash'}});
	if ($self->{'oldsite_hash'}->{'path'} ne $self->{'httpd'} ||
	    $self->{'oldsite_hash'}->{'size'} != $self->{'st_array'}->[7] ||
	    $self->{'oldsite_hash'}->{'webmin'} != $self->get_webmin_version()) {
		# Need to build list of supported modules
		 ($self->{'ver'}, $self->{'mods'}) = $self->httpd_info($self->{'httpd'});
		if ($self->{'ver'}) {
			 @{$self->{'mods_array'}} = map { "$_/$self->{'ver'}" } $self->configurable_modules();
			 %{$self->{'site_hash'}} = ( 'size' => $self->{'st_array'}->[7],
					'path' => $self->{'httpd'},
					'modules' => join(' ', @{$self->{'mods_array'}}),
					'webmin' => $self->get_webmin_version() );
			$self->lock_file($self->{'site_file'});
			$self->write_file($self->{'site_file'}, \%{$self->{'site_hash'}});
			chmod(0644, $self->{'site_file'});
			$self->unlock_file($self->{'site_file'});
			}
		}
	}

# Read the site-specific setup file, then require in all the module-specific
# .pl files
if ($self->read_file($self->{'site_file'}, \%{$self->{'site_hash'}})) {
	($m, $f, $d);
	$self->{'httpd_size'} = $self->{'site_hash'}->{'size'};
	foreach $m (split(/\s+/, $self->{'site_hash'}->{'modules'})) {
		if ($m =~ /(\S+)\/(\S+)/) { $self->{'httpd_modules_hash'}->{$1} = $2; }
		}
	foreach $m (keys %{$self->{'httpd_modules_hash'}}) {
		if (!-r "$self->{'module_root_directory'}/$m.pl") {
			delete($self->{'httpd_modules_hash'}->{$m});
			}
		}
	foreach $f (split(/\s+/, $self->{'site_hash'}->{'htaccess'})) {
		if (-r $f) { push(@{$self->{'htaccess_files_array'}}, $f); }
		}
	foreach $m (keys %{$self->{'httpd_modules_hash'}}) {
		{
    my $pkg = $self->lib_to_pkg("$m.pl", "apache");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
		}
	foreach $d (split(/\s+/, $self->{'site_hash'}->{'defines'})) {
		$self->{'httpd_defines_hash'}->{$d}++;
		}
	}

$self->{'apache_docbase'} = $self->{'config_hash'}->{'apache_docbase'} ? $self->{'config_hash'}->{'apache_docbase'} :
		  $self->{'httpd_modules_hash'}->{'core'} >= 2.0 ?
			"http://httpd.apache.org/docs-2.0/mod/" :
			"http://httpd.apache.org/docs/mod/";

1;

1;
		return $self;
}

# $self->parse_config_file(handle, lines, file, [recursive])
# Parses lines of text from some config file into a data structure. The
# return value is an array of references, one for each directive in the file.
# Each reference points to an associative array containing
#  line -	The line number this directive is at
#  eline -	The line number this directive ends at
#  file -	The file this directive is from
#  type -	0 for a normal directive, 1 for a container directive
#  name -	The name of this directive
#  value -	Value (possibly with spaces)
#  members -	For type 1, a reference to the array of members
sub parse_config_file
{
	my $self = shift;

local($fh, @rv, $line, %dummy);
$fh = $_[0];
$dummy{'line'} = $dummy{'eline'} = $_[1]-1;
$dummy{'file'} = $_[2];
$dummy{'type'} = 0;
$dummy{'name'} = "dummy";
@rv = (\%dummy);
local %defs = map { $_, 1 } $self->get_httpd_defines();
while($line = <$fh>) {
	chop;
	$line =~ s/^\s*#.*$//g;
	if ($line =~ /^\s*<\/(\S+)\s*(.*)>/) {
		# end of a container directive. This can only happen in a
		# recursive call to this function
		$_[1]++;
		last if ($_[3]);
		}
	elsif ($line =~ /^\s*<IfModule\s+(\!?)(\S+)\.c>/i ||
	       $line =~ /^\s*<IfModule\s+(\!?)(\S+)>/i) {
		# start of an IfModule block. Read it, and if the module
		# exists put the directives in this section.
		local ($not, $mod) = ($1, $2);
		local $oldline = $_[1];
		$_[1]++;
		local @dirs = $self->parse_config_file($fh, $_[1], $_[2], 1);
		local $altmod = $mod;
		$altmod =~ s/^(\S+)_module$/mod_$1/g;
		if (!$not && $self->{'httpd_modules_hash'}->{$mod} ||
		    $not && !$self->{'httpd_modules_hash'}->{$mod} ||
		    !$not && $self->{'httpd_modules_hash'}->{$altmod} ||
		    $not && !$self->{'httpd_modules_hash'}->{$altmod}) {
			# use the directives..
			push(@rv, { 'line', $oldline,
				    'eline', $oldline,
				    'file', $_[2],
				    'name', "<IfModule $not$mod>" });
			push(@rv, @dirs);
			push(@rv, { 'line', $_[1]-1,
				    'eline', $_[1]-1,
				    'file', $_[2],
				    'name', "</IfModule>" });
			}
		}
	elsif ($line =~ /^\s*<IfDefine\s+(\!?)(\S+)>/i) {
		# start of an IfDefine block. Read it, and if the define
		# exists put the directives in this section
		local ($not, $def) = ($1, $2);
		local $oldline = $_[1];
		$_[1]++;
		local @dirs = $self->parse_config_file($fh, $_[1], $_[2], 1);
		if (!$not && $defs{$def} ||
		    $not && !$defs{$def}) {
			# use the directives..
			push(@rv, { 'line', $oldline,
				    'eline', $oldline,
				    'file', $_[2],
				    'name', "<IfDefine $not$def>" });
			push(@rv, @dirs);
			push(@rv, { 'line', $_[1]-1,
				    'eline', $_[1]-1,
				    'file', $_[2],
				    'name', "</IfDefine>" });
			}
		}
	elsif ($line =~ /^\s*<IfVersion\s+(\!?)(\S*)\s*(\S+)>/i) {
		# Start of an IfVersion block. Read it, and if the version
		# matches put the directives in this section
		local ($not, $op, $ver) = ($1, $2, $3);
		local $oldline = $_[1];
		$_[1]++;
		local @dirs = $self->parse_config_file($fh, $_[1], $_[2], 1);
		$op ||= "=";
		local $match = 0;
		local $myver = $self->{'httpd_modules_hash'}->{'core'};
		$myver =~ s/^(\d+)\.(\d)(\d+)$/$1.$2.$3/;
		if ($op eq "=" || $op eq "==") {
			if ($ver =~ /^\/(.*)\/$/) {
				$match = 1 if ($myver =~ /$1/);
				}
			else {
				$match = 1 if ($myver eq $ver);
				}
			}
		elsif ($op eq ">") {
			$match = 1 if ($myver > $ver);
			}
		elsif ($op eq ">=") {
			$match = 1 if ($myver >= $ver);
			}
		elsif ($op eq "<") {
			$match = 1 if ($myver < $ver);
			}
		elsif ($op eq "<=") {
			$match = 1 if ($myver <= $ver);
			}
		elsif ($op eq "~") {
			$match = 1 if ($myver =~ /$ver/);
			}
		$match = !$match if ($not);
		if ($match) {
			# use the directives..
			push(@rv, { 'line', $oldline,
				    'eline', $oldline,
				    'file', $_[2],
				    'name', "<IfVersion $not$op $ver>" });
			push(@rv, @dirs);
			push(@rv, { 'line', $_[1]-1,
				    'eline', $_[1]-1,
				    'file', $_[2],
				    'name', "</IfVersion>" });
			}
		}
	elsif ($line =~ /^\s*<(\S+)\s*(.*)>/) {
		# start of a container directive. The first member is a dummy
		# directive at the same line as the container
		local(%dir, @members);
		%dir = ('line', $_[1],
			'file', $_[2],
			'type', 1,
			'name', $1,
			'value', $2);
		$dir{'value'} =~ s/\s+$//g;
		$dir{'words'} = $self->wsplit($dir{'value'});
		$_[1]++;
		@members = $self->parse_config_file($fh, $_[1], $_[2], 1);
		$dir{'members'} = \@members;
		$dir{'eline'} = $_[1]-1;
		push(@rv, \%dir);
		}
	elsif ($line =~ /^\s*(\S+)\s*(.*)$/) {
		# normal directive
		local(%dir);
		%dir = ('line', $_[1],
			'eline', $_[1],
			'file', $_[2],
			'type', 0,
			'name', $1,
			'value', $2);
		if ($dir{'value'} =~ s/\\$//g) {
			# multi-line directive!
			while($line = <$fh>) {
				chop($line);
				$self->{'cont'} = ($line =~ s/\\$//g);
				$dir{'value'} .= $line;
				$dir{'eline'} = ++$_[1];
				if (!$self->{'cont'}) { last; }
				}
			}
		$dir{'value'} =~ s/\s+$//g;
		$dir{'words'} = $self->wsplit($dir{'value'});
		push(@rv, \%dir);
		$_[1]++;
		}
	else {
		# blank or comment line
		$_[1]++;
		}
	}
return @rv;
}

# wsplit(string)
# Splits a string like  foo "foo \"bar\"" bazzz  into an array of words
sub wsplit
{
	my $self = shift;

local($s, @rv); $s = $_[0];
$s =~ s/\\\"/\0/g;
while($s =~ /^"([^"]*)"\s*(.*)$/ || $s =~ /^(\S+)\s*(.*)$/) {
	$self->{'w'} = $1; $s = $2;
	$self->{'w'} =~ s/\0/"/g; push(@rv, $self->{'w'});
	}
return \@rv;
}

# wjoin(word, word, ...)
sub wjoin
{
	my $self = shift;

local(@rv, $w);
foreach $w (@_) {
	if ($w =~ /^\S+$/) { push(@rv, $w); }
	else { push(@rv, "\"$w\""); }
	}
return join(' ', @rv);
}

# find_directive(name, &directives, [1stword])
# Returns the values of directives matching some name
sub find_directive
{
	my $self = shift;

local (@vals, $ref);
foreach $ref (@{$_[1]}) {
	if (lc($ref->{'name'}) eq lc($_[0])) {
		push(@vals, $_[2] ? $ref->{'words'}->[0] : $ref->{'value'});
		}
	}
return wantarray ? @vals : !@vals ? undef : $vals[$#vals];
}

# find_directive_struct(name, &directives)
# Returns references to directives maching some name
sub find_directive_struct
{
	my $self = shift;

local (@vals, $ref);
foreach $ref (@{$_[1]}) {
	if (lc($ref->{'name'}) eq lc($_[0])) {
		push(@vals, $ref);
		}
	}
return wantarray ? @vals : !@vals ? undef : $vals[$#vals];
}

# find_vdirective(name, &virtualdirectives, &directives, [1stword])
# Looks for some directive in a <VirtualHost> section, and then in the 
# main section
sub find_vdirective
{
	my $self = shift;

if ($_[1]) {
	$self->{'rv'} = $self->find_directive($_[0], $_[1], $_[3]);
	if ($self->{'rv'}) { return $self->{'rv'}; }
	}
return $self->find_directive($_[0], $_[2], $_[3]);
}

# get_config()
# Returns the entire config structure
sub get_config
{
	my $self = shift;

local($acc, $res, $lnum, $conf, @virt, $v, $mref, $inc);
if (@{$self->{'get_config_cache_array'}}) {
	return \@{$self->{'get_config_cache_array'}};
	}

# read primary config file
($conf) = $self->find_httpd_conf();
return undef if (!$conf);
@{$self->{'get_config_cache_array'}} = $self->get_config_file($conf);

# Read main resource and access config files
$lnum = 0;
$res = $self->find_directive("ResourceConfig", \@{$self->{'get_config_cache_array'}});
if (!$res) { $res = $self->{'config_hash'}->{'srm_conf'}; }
if (!$res) { $res = "$self->{'config_hash'}->{'httpd_dir'}/conf/srm.conf"; }
if (!-r $self->translate_filename($res)) {
	$res = "$self->{'config_hash'}->{'httpd_dir'}/etc/srm.conf";
	}
push(@{$self->{'get_config_cache_array'}}, $self->get_config_file($res));

$lnum = 0;
$acc = $self->find_directive("AccessConfig", \@{$self->{'get_config_cache_array'}});
if (!$acc) { $acc = $self->{'config_hash'}->{'access_conf'}; }
if (!$acc) { $acc = "$self->{'config_hash'}->{'httpd_dir'}/conf/access.conf"; }
if (!-r $self->translate_filename($acc)) {
	$acc = "$self->{'config_hash'}->{'httpd_dir'}/etc/access.conf";
	}
push(@{$self->{'get_config_cache_array'}}, $self->get_config_file($acc));

# Read extra config files in VirtualHost sections
@virt = $self->find_directive_struct("VirtualHost", \@{$self->{'get_config_cache_array'}});
foreach $v (@virt) {
	$mref = $v->{'members'};
	foreach $idn ("ResourceConfig", "AccessConfig", "Include") {
		local $inc = $self->find_directive_struct($idn, $mref);
		next if (!$inc);
		local @incs = $self->expand_apache_include($inc->{'words'}->[0]);
		foreach my $ginc (@incs) {
			push(@$mref, $self->get_config_file($ginc));
			}
		}
	}

return \@{$self->{'get_config_cache_array'}};
}

# $self->get_config_file(filename)
sub get_config_file
{
	my $self = shift;

local ($file) = @_;
$file = $self->resolve_links($file);		# Convert sites-enabled to real path
					# in sites-available
local @rv;
if (opendir(DIR, $file)) {
	# Is a directory .. parse all files!
	local $f;
	local @files = readdir(DIR);
	closedir(DIR);
	foreach $f (@files) {
		next if ($f =~ /^\./);
		push(@rv, $self->get_config_file("$file/$f"));
		}
	}
else {
	# Just a normal config file
	local $lnum = 0;
	$self->open_readfile(*CONF, $file);
	@rv = $self->parse_config_file(CONF, $lnum, $file);
	close(CONF);
	}

# Expand Include directives
foreach $inc ($self->find_directive_struct("Include", \@rv)) {
	local @incs = $self->expand_apache_include($inc->{'words'}->[0]);
	foreach my $ginc (@incs) {
		push(@rv, $self->get_config_file($ginc));
		}
	}

return @rv;
}

# expand_apache_include(dir)
# Given an include directive value, returns a list of matching files
sub expand_apache_include
{
	my $self = shift;

local ($incdir) = @_;
if ($incdir !~ /^\//) { $incdir = "$self->{'config_hash'}->{'httpd_dir'}/$incdir"; }
if ($incdir =~ /^(.*)\[\^([^\]]+)\](.*)$/) {
	# A glob like /etc/[^.#]*.conf , which cannot be handled
	# by Perl's glob function!
	local $before = $1;
	local $after = $3;
	local %reject = map { $_, 1 } split(//, $2);
	$reject{'*'} = $reject{'?'} = $reject{'['} = $reject{']'} =
	  $reject{'/'} = $reject{'$'} = $reject{'('} = $reject{')'} =
	  $reject{'!'} = 1;
	local $accept = join("", grep { !$reject{$_} } map { chr($_) } (32 .. 126));
	$incdir = $before."[".$accept."]".$after;
	}
return glob($incdir);
}

# get_virtual_config(index)
sub get_virtual_config
{
	my $self = shift;

local($conf, $c, $v);
$conf = $self->get_config();
if (!$_[0]) { $c = $conf; $v = undef; }
else {
	$c = $conf->[$_[0]]->{'members'};
	$v = $conf->[$_[0]];
	}
return wantarray ? ($c, $v) : $c;
}

# get_htaccess_config(file)
sub get_htaccess_config
{
	my $self = shift;

local($lnum, @conf);
$self->open_readfile(*HTACCESS, $_[0]);
@conf = $self->parse_config_file(HTACCESS, $lnum, $_[0]);
close(HTACCESS);
return \@conf;
}

# save_directive(name, &values, &directives, &config)
# Updates the config file(s) and the directives structure with new values
# for the given directives.
# If a directive's value is merely being changed, then its value only needs
# to be updated in the directives array and in the file.
sub save_directive
{
	my $self = shift;

local($i, @old, $lref, $change, $len, $v);
@old = $self->find_directive_struct($_[0], $_[2]);
for($i=0; $i<@old || $i<@{$_[1]}; $i++) {
	$v = ${$_[1]}[$i];
	if ($i >= @old) {
		# a new directive is being added. If other directives of this
		# type exist, add it after them. Otherwise, put it at the end of
		# the first file in the section
		if ($change) {
			# Have changed some old directive.. add this new one
			# after it, and update change
			local(%v, $j);
			%v = (	"line", $change->{'line'}+1,
				"eline", $change->{'line'}+1,
				"file", $change->{'file'},
				"type", 0,
				"name", $_[0],
				"value", $v);
			$j = $self->indexof($change, @{$_[2]})+1;
			$self->renumber($_[3], $v{'line'}, $v{'file'}, 1);
			splice(@{$_[2]}, $j, 0, \%v);
			$lref = $self->read_file_lines($v{'file'});
			splice(@$lref, $v{'line'}, 0, "$_[0] $v");
			$change = \%v;
			}
		else {
			# Adding a new directive to the end of the list
			# in this section
			local($f, %v, $j);
			$f = $_[2]->[0]->{'file'};
			for($j=0; $_[2]->[$j]->{'file'} eq $f; $j++) { }
			$self->{'l'} = $_[2]->[$j-1]->{'eline'}+1;
			%v = (	"line", $self->{'l'},
				"eline", $self->{'l'},
				"file", $f,
				"type", 0,
				"name", $_[0],
				"value", $v);
			$self->renumber($_[3], $self->{'l'}, $f, 1);
			splice(@{$_[2]}, $j, 0, \%v);
			$lref = $self->read_file_lines($f);
			splice(@$lref, $self->{'l'}, 0, "$_[0] $v");
			}
		}
	elsif ($i >= @{$_[1]}) {
		# a directive was deleted
		$lref = $self->read_file_lines($old[$i]->{'file'});
		$self->{'idx'} = $self->indexof($old[$i], @{$_[2]});
		splice(@{$_[2]}, $self->{'idx'}, 1);
		$len = $old[$i]->{'eline'} - $old[$i]->{'line'} + 1;
		splice(@$lref, $old[$i]->{'line'}, $len);
		$self->renumber($_[3], $old[$i]->{'line'}, $old[$i]->{'file'}, -$len);
		}
	else {
		# just changing the value
		$lref = $self->read_file_lines($old[$i]->{'file'});
		$len = $old[$i]->{'eline'} - $old[$i]->{'line'} + 1;
		$self->renumber($_[3], $old[$i]->{'eline'}+1,
			  $old[$i]->{'file'},1-$len);
		$old[$i]->{'value'} = $v;
		$old[$i]->{'eline'} = $old[$i]->{'line'};
		splice(@$lref, $old[$i]->{'line'}, $len, "$_[0] $v");
		$change = $old[$i];
		}
	}
}

# $self->renumber(&config, line, file, offset)
# Recursively changes the line number of all directives from some file 
# beyond the given line.
sub renumber
{
	my $self = shift;

local($d);
if (!$_[3]) { return; }
foreach $d (@{$_[0]}) {
	if ($d->{'file'} eq $_[2] && $d->{'line'} >= $_[1]) {
		$d->{'line'} += $_[3];
		}
	if ($d->{'file'} eq $_[2] && $d->{'eline'} >= $_[1]) {
		$d->{'eline'} += $_[3];
		}
	if ($d->{'type'}) {
		$self->renumber($d->{'members'}, $_[1], $_[2], $_[3]);
		}
	}
}

# server_root(path, &directives)
sub server_root
{
	my $self = shift;

if (!$_[0]) { return undef; }
elsif ($_[0] =~ /^\//) { return $_[0]; }
else { return "$self->{'config_hash'}->{'httpd_dir'}/$_[0]"; }
}

sub dump_config
{
	my $self = shift;

local($c, $mref);
print "<table border>\n";
print "<tr> <td>Name</td> <td>Value</td> <td>File</td> <td>Line</td> </tr>\n";
foreach $c (@_) {
	printf "<tr> <td>%s</td> <td>%s</td><td>%s</td><td>%s</td> </tr>\n",
		$c->{'name'}, $c->{'value'}, $c->{'file'}, $c->{'line'};
	if ($c->{'type'}) {
		print "<tr> <td colspan=4>\n";
		$mref = $c->{'members'};
		$self->dump_config(@$mref);
		print "</td> </tr>\n";
		}
	}
print "</table>\n";
}

sub def
{
	my $self = shift;

return $_[0] ? $_[0] : $_[1];
}

# make_directives(ref, version, module)
sub make_directives
{
	my $self = shift;

local(@rv, $aref);
$aref = $_[0];
local $ver = $_[1];
if ($ver =~ /^(1)\.(3)(\d+)$/) {
	$ver = sprintf "%d.%d%2.2d", $1, $2, $3;
	}
foreach $d (@$aref) {
	local(%dir);
	$dir{'name'} = $d->[0];
	$dir{'multiple'} = $d->[1];
	$dir{'type'} = int($d->[2]);
	$dir{'subtype'} = $d->[2] - $dir{'type'};
	$dir{'module'} = $_[2];
	$dir{'version'} = $ver;
	$dir{'priority'} = $d->[5];
	foreach $c (split(/\s+/, $d->[3])) { $dir{$c}++; }
	if (!$d->[4]) { push(@rv, \%dir); }
	elsif ($d->[4] =~ /^-([\d\.]+)$/ && $ver < $1) { push(@rv, \%dir); }
	elsif ($d->[4] =~ /^([\d\.]+)$/ && $ver >= $1) { push(@rv, \%dir); }
	elsif ($d->[4] =~ /^([\d\.]+)-([\d\.]+)$/ && $ver >= $1 && $ver < $2)
		{ push(@rv, \%dir); }
	}
return @rv;
}

# editable_directives(type, context)
# Returns an array of references to associative arrays, one for each 
# directive of the given type that can be used in the given context
sub editable_directives
{
	my $self = shift;

local($m, $func, @rv, %done);
foreach $m (keys %{$self->{'httpd_modules_hash'}}) {
	$func = $m."_directives";
	push(@rv, $self->$func($self->{'httpd_modules_hash'}->{$m}));
	}
@rv = grep { $_->{'type'} == $_[0] && $_->{$_[1]} &&
	     !$done{$_->{'name'}}++ } @rv;
@rv = grep { $self->can_edit_directive($_->{'name'}) } @rv;
@rv = sort { local $td = $a->{'subtype'} <=> $b->{'subtype'};
	     local $pd = $b->{'priority'} <=> $a->{'priority'};
	     local $md = $a->{'module'} cmp $b->{'module'};
	     $td ? $td : $pd ? $pd : $md ? $md : $a->{'name'} cmp $b->{'name'} }
		@rv;
return @rv;
}

# can_edit_directive(name)
# Returns 1 if the Apache directive named can be edited by the current user
sub can_edit_directive
{
	my $self = shift;

local ($name) = @_;
if ($self->{'access_hash'}->{'dirsmode'} == 0) {
	return 1;
	}
else {
	local %dirs = map { lc($_), 1 } split(/\s+/, $self->{'access_hash'}->{'dirs'});
	if ($self->{'access_hash'}->{'dirsmode'} == 1) {
		return $dirs{lc($name)};
		}
	else {
		return !$dirs{lc($name)};
		}
	}
}

# generate_inputs(&editors, &directives)
# Displays a 2-column list of options, for use inside a table
sub generate_inputs
{
	my $self = shift;

local($e, $sw, @args, @rv, $func, $lastsub);
foreach $e (@{$_[0]}) {
	if (defined($lastsub) && $lastsub != $e->{'subtype'}) {
		print $self->ui_table_hr();
		}
	$lastsub = $e->{'subtype'};

	# Build arg list for the editing function. Each arg can be a single
	# directive struct, or a reference to an array of structures.
	$func = "edit";
	undef(@args);
	foreach $ed (split(/\s+/, $e->{'name'})) {
		local(@vals);
		$func .= "_$ed";
		@vals = $self->find_directive_struct($ed, $_[1]);
		if ($e->{'multiple'}) { push(@args, \@vals); }
		elsif (!@vals) { push(@args, undef); }
		else { push(@args, $vals[$#vals]); }
		}
	push(@args, $e);

	# call the function
	@rv = $self->$func(@args);
	local $names;
	if ($self->{'config_hash'}->{'show_names'} || $self->{'userconfig_hash'}->{'show_names'}) {
		$names = " (";
		foreach $ed (split(/\s+/, $e->{'name'})) {
			# nodo50 v0.1 - Change 000004 - Open new window for Help in Apache module and mod_apachessl Help from http://www.apache-ssl.org and
			# nodo50 v0.1 - Change 000004 - Abre nueva ventana para Ayuda del m�ulo Apache y para mod_apachessl busca la Ayuda en http://www.apache-ssl.org and
			$names .= "<tt><a href='".($e->{'module'} eq 'mod_apachessl' ? 'http://www.apache-ssl.org/docs.html#'.$ed : $self->{'apache_docbase'}."/".$e->{'module'}.".html#".lc($ed))."'>".$ed."</a></tt> ";
			#$names .= "<tt><a href='".$self->{'apache_docbase'}."/".$e->{'module'}.".html#".lc($ed)."'>".$ed."</a></tt> ";
			# nodo50 v0.1 - Change 000004 - End
			}
		$names .= ")";
		}
	if ($rv[0] >= 2) {
		# spans 2 columns..
		if ($rv[0] == 3) {
			# Takes up whole row
			print $self->ui_table_row(undef, $rv[2], 4);
			}
		else {
			# Has title on left
			print $self->ui_table_row($rv[1], $rv[2], 3);
			}
		}
	else {
		# only spans one column
		print $self->ui_table_row($rv[1], $rv[2]);
		}
	}
}

# parse_inputs(&editors, &directives, &config)
# Reads user choices from a form and update the directives and config files.
sub parse_inputs
{
	my $self = shift;

# First call editor functions to get new values. Each function returns
# an array of references to arrays containing the new values for the directive.
$self->before_changing();
$self->lock_apache_files();
foreach $e (@{$_[0]}) {
	@{$self->{'dirs_array'}} = split(/\s+/, $e->{'name'});
	$self->{'func'} = "save_".join('_', @{$self->{'dirs_array'}});
	@{$self->{'rv_array'}} = do { my $fn = $self->{'func'}; $self->$fn($e)};
	for($i=0; $i<@{$self->{'dirs_array'}}; $i++) {
		push(@{$self->{'chname_array'}}, $self->{'dirs_array'}->[$i]);
		push(@{$self->{'chval_array'}}, $self->{'rv_array'}->[$i]);
		}
	}

# Assuming everything went OK, update the configuration
for($i=0; $i<@{$self->{'chname_array'}}; $i++) {
	$self->save_directive($self->{'chname_array'}->[$i], $self->{'chval_array'}->[$i], $_[1], $_[2]);
	}
$self->flush_file_lines();
$self->unlock_apache_files();
$self->after_changing();
}

# opt_input(value, name, default, size)
sub opt_input
{
	my $self = shift;

return $self->ui_opt_textbox($_[1], $_[0], $_[3], $_[2]);
}

# parse_opt(name, regexp, error, [noquotes])
sub parse_opt
{
	my $self = shift;

local($i, $re);
local $v = $self->{'in_hash'}->{$_[0]};
if ($self->{'in_hash'}->{"$_[0]_def"}) { return ( [ ] ); }
for($i=1; $i<@_; $i+=2) {
	$re = $_[$i];
	if ($v !~ /$re/) { $self->error($_[$i+1]); }
	}
return ( [ $v =~ /\s/ && !$_[3] ? "\"$v\"" : $v ] );
}

# choice_input(value, name, default, [choice]+)
# Each choice is a display,value pair
sub choice_input
{
	my $self = shift;

local($i, $rv);
for($i=3; $i<@_; $i++) {
	$_[$i] =~ /^([^,]*),(.*)$/;
	$rv .= $self->ui_oneradio($_[0], $2, $1, lc($2) eq lc($_[0]) ||
				!defined($_[0]) && lc($2) eq lc($_[2]))."\n";
	}
return $rv;
}

# choice_input_vert(value, name, default, [choice]+)
# Each choice is a display,value pair
sub choice_input_vert
{
	my $self = shift;

local($i, $rv);
for($i=3; $i<@_; $i++) {
	$_[$i] =~ /^([^,]*),(.*)$/;
	$rv .= sprintf "<input type=radio name=$_[1] value=\"$2\" %s> $1<br>\n",
		lc($2) eq lc($_[0]) || !defined($_[0]) &&
				       lc($2) eq lc($_[2]) ? "checked" : "";
	}
return $rv;
}

# parse_choice(name, default)
sub parse_choice
{
	my $self = shift;

if (lc($self->{'in_hash'}->{$_[0]}) eq lc($_[1])) { return ( [ ] ); }
else { return ( [ $self->{'in_hash'}->{$_[0]} ] ); }
}

# select_input(value, name, default, [choice]+)
sub select_input
{
	my $self = shift;

local($i, $rv);
$rv = "<select name=\"$_[1]\">\n";
for($i=3; $i<@_; $i++) {
	$_[$i] =~ /^([^,]*),(.*)$/;
	$rv .= sprintf "<option value=\"$2\" %s> $1\n",
		lc($2) eq lc($_[0]) || !defined($_[0]) && lc($2) eq lc($_[2]) ? "selected" : "";
	}
$rv .= "</select>\n";
return $rv;
}

# $self->parse_choice(name, default)
sub parse_select
{
	my $self = shift;

return $self->parse_choice(@_);
}

# handler_input(value, name)
sub handler_input
{
	my $self = shift;

local($m, $func, @hl, $rv, $h);
local $conf = $self->get_config();
push(@hl, "");
foreach $m (keys %{$self->{'httpd_modules_hash'}}) {
	$func = $m."_handlers";
	if ($self->can($func)) {
		push(@hl, $self->$func($conf, $self->{'httpd_modules_hash'}->{$m}));
		}
	}
if ($self->indexof($_[0], @hl) < 0) { push(@hl, $_[0]); }
$rv = "<select name=$_[1]>\n";
foreach $h ($self->unique(@hl)) {
	$rv .= sprintf "<option value=\"$h\" %s>$h\n",
		$h eq $_[0] ? "selected" : "";
	}
$rv .= sprintf "<option value=\"None\" %s>&lt;$self->{'text_hash'}->{'core_none'}&gt;\n",
	$_[0] eq "None" ? "selected" : "";
$rv .= "</select>\n";
return $rv;
}

# parse_handler(name)
sub parse_handler
{
	my $self = shift;

if ($self->{'in_hash'}->{$_[0]} eq "") { return ( [ ] ); }
else { return ( [ $self->{'in_hash'}->{$_[0]} ] ); }
}

# filters_input(&values, name)
sub filters_input
{
	my $self = shift;

local($m, $func, @fl, $rv, $f);
local $conf = $self->get_config();
foreach $m (keys %{$self->{'httpd_modules_hash'}}) {
	$func = $m."_filters";
	if ($self->can($func)) {
		push(@fl, $self->$func($conf, $self->{'httpd_modules_hash'}->{$m}));
		}
	}
foreach $f (@{$_[0]}) {
	push(@fl, $f) if ($self->indexof($f, @fl) < 0);
	}
foreach $f ($self->unique(@fl)) {
	$rv .= sprintf "<input type=checkbox name=$_[1] value='%s' %s> %s\n",
			$f, $self->indexof($f, @{$_[0]}) < 0 ? "" : "checked", $f;
	}
return $rv;
}

# parse_filters(name)
sub parse_filters
{
	my $self = shift;

local @f = split(/\0/, $self->{'in_hash'}->{$_[0]});
return @f ? ( [ join(";", @f) ] ) : ( [ ] );
}

# virtual_name(struct, [forlog])
sub virtual_name
{
	my $self = shift;

if ($_[0]) {
	local $n = $self->find_directive("ServerName", $_[0]->{'members'});
	if ($n =~ /(.*):(\d+)$/) {
		$n = $1;
		}
	if ($n) {
		return $self->html_escape($_[0]->{'value'} =~ /:(\d+)$/ ? "$n:$1"
								  : $n);
		}
	else {
		return $self->html_escape($_[0]->{'value'});
		}
	}
else { return $_[1] ? "*" : $self->{'text_hash'}->{'default_serv'}; }
}

# dir_name(struct)
sub dir_name
{
	my $self = shift;

local($dfm, $mat);
$_[0]->{'name'} =~ /^(Directory|Files|Location)(Match)?$/;
$dfm = $1; $mat = $2;
if ($mat) {
	return "$dfm regexp <tt>".$self->html_escape($_[0]->{'words'}->[0])."</tt>";
	}
elsif ($_[0]->{'words'}->[0] eq "~") {
	return "$dfm regexp <tt>".$self->html_escape($_[0]->{'words'}->[1])."</tt>";
	}
else { return "$dfm <tt>".$self->html_escape($_[0]->{'words'}->[0])."</tt>"; }
}

# list_user_file(file, &user,  &pass)
sub list_user_file
{
	my $self = shift;

local($_);
$self->open_readfile(*USERS, $_[0]);
while(<USERS>) {
	/^(\S+):(\S+)/;
	push(@{$_[1]}, $1); $_[2]->{$1} = $2;
	}
close(USERS);
}

# config_icons(context, program)
# Displays up to 18 icons, one for each type of configuration directive, for
# some context (global, virtual, directory or htaccess)
sub config_icons
{
	my $self = shift;

local($m, $func, $e, %etype, $i, $c);
foreach $m (sort { $a cmp $b } (keys %{$self->{'httpd_modules_hash'}})) {
        $func = $m."_directives";
	foreach $e ($self->$func($self->{'httpd_modules_hash'}->{$m})) {
		if ($e->{$_[0]}) { $etype{$e->{'type'}}++; }
		}
        }
local (@titles, @links, @icons);
for($i=0; $self->{'text_hash'}->{"type_$i"}; $i++) {
	if ($etype{$i} && $self->{'access_types_hash'}->{$i}) {
		push(@links, $_[1]."type=$i");
		push(@titles, $self->{'text_hash'}->{"type_$i"});
		push(@icons, "images/type_icon_$i.gif");
		}
	}
for($i=2; $i<@_; $i++) {
	push(@links, $_[$i]->{'link'});
	push(@titles, $_[$i]->{'name'});
	push(@icons, $_[$i]->{'icon'});
	}
$self->icons_table(\@links, \@titles, \@icons, 5);
print "<p>\n";
}

# restart_button()
# Returns HTML for a link to put in the top-right corner of every page
sub restart_button
{
	my $self = shift;

local $rv;
$self->{'args'} = "redir=".$self->urlize($self->this_url());
local @rv;
if ($self->is_apache_running()) {
	if ($self->{'access_hash'}->{'apply'}) {
		push(@rv, "<a href=\"restart.cgi?$self->{'args'}\">$self->{'text_hash'}->{'apache_apply'}</a>\n");
		}
	if ($self->{'access_hash'}->{'stop'}) {
		push(@rv, "<a href=\"stop.cgi?$self->{'args'}\">$self->{'text_hash'}->{'apache_stop'}</a>\n");
		}
	}
elsif ($self->{'access_hash'}->{'stop'}) {
	push(@rv, "<a href=\"start.cgi?$self->{'args'}\">$self->{'text_hash'}->{'apache_start'}</a>\n");
	}
return join("<br>\n", @rv);
}

# this_url()
# Returns the URL in the apache directory of the current script
sub this_url
{
	my $self = shift;

local($url);
$url = $ENV{'SCRIPT_NAME'};
if ($ENV{'QUERY_STRING'} ne "") { $url .= "?$ENV{'QUERY_STRING'}"; }
return $url;
}

# $self->find_all_directives(config, name)
# Recursively finds all directives of some type
sub find_all_directives
{
	my $self = shift;

local(@rv, $d);
foreach $d (@{$_[0]}) {
	if ($d->{'name'} eq $_[1]) { push(@rv, $d); }
	if ($d->{'type'} == 1) {
		push(@rv, $self->find_all_directives($d->{'members'}, $_[1]));
		}
	}
return @rv;
}

# httpd_info(executable)
# Returns the httpd version and modules array
sub httpd_info
{
	my $self = shift;

local(@mods, $verstr, $ver, $minor);
$verstr = $self->backquote_command("\"$_[0]\" -v 2>&1");
if ($self->{'config_hash'}->{'httpd_version'}) {
	$self->{'config_hash'}->{'httpd_version'} =~ /(\d+)\.([\d\.]+)/;
	$ver = $1; $minor = $2; $minor =~ s/\.//g; $ver .= ".$minor";
	}
elsif ($verstr =~ /Apache(\S*)\/(\d+)\.([\d\.]+)/) {
	# standard apache
	$ver = $2; $minor = $3; $minor =~ s/\.//g; $ver .= ".$minor";
	}
elsif ($verstr =~ /HP\s*Apache-based\s*Web\s*Server(\S*)\/(\d+)\.([\d\.]+)/) {
	# HP's apache
	$ver = $2; $minor = $3; $minor =~ s/\.//g; $ver .= ".$minor";
	}
elsif ($verstr =~ /Red\s*Hat\s+Secure\/2\.0/i) {
	# redhat secure server 2.0
	$ver = 1.31;
	}
elsif ($verstr =~ /Red\s*Hat\s+Secure\/3\.0/i) {
	# redhat secure server 3.0
	$ver = 1.39;
	}
elsif ($self->has_command("rpm") &&
       $self->backquote_command("rpm -q apache 2>&1") =~ /^apache-(\d+)\.([\d\.]+)/) {
	# got version from the RPM
	$ver = $1; $minor = $2; $minor =~ s/\.//g; $ver .= ".$minor";
	}
else {
	# couldn't get version
	return (0, undef);
	}
if ($ver < 1.2) {
	# apache 1.1 has no -l option! Use the standard list
	@mods = ("core", "mod_mime", "mod_access", "mod_auth", "mod_include",
		 "mod_negotiation", "mod_dir", "mod_cgi", "mod_userdir",
		 "mod_alias", "mod_env", "mod_log_common", $self->{'gconfig_hash'}->{'ztl_mode'} ? "mod_ztl" : "mod_cpplus");
	}
else {
	# ask apache for the module list
	@mods = ("core", $self->{'gconfig_hash'}->{'ztl_mode'} ? "mod_ztl" : "mod_cpplus");
	$self->open_execute_command(*APACHE, "\"$_[0]\" -l", 1);
	while(<APACHE>) {
		if (/(\S+)\.c/) { push(@mods, $1); }
		}
	close(APACHE);
	@mods = $self->unique(@mods);
	}
return ($ver, \@mods);
}

# print_line(directive, text, indent, link)
sub print_line
{
	my $self = shift;

local $line = $_[0]->{'line'} + 1;
local $lstr = "$_[0]->{'file'} ($line)";
local $txt = join("", @{$_[1]});
local $left = 85 - length($lstr) - $_[2];
if (length($txt) > $left) {
	$txt = substr($txt, 0, $left)." ..";
	}
local $txtlen = length($txt);
$txt = $self->html_escape($txt);
print " " x $_[2];
if ($_[3]) {
	print "<a href=\"$_[3]\">",$txt,"</a>";
	}
else { print $txt; }
print " " x (90 - $txtlen - $_[2] - length($lstr));
print $lstr,"\n";
}

# can_edit_virt(struct)
sub can_edit_virt
{
	my $self = shift;

return 1 if ($self->{'access_hash'}->{'virts'} eq '*');
local %vcan = map { $_, 1 } split(/\s+/, $self->{'access_hash'}->{'virts'});
local ($can) = grep { $vcan{$_} } $self->virt_acl_name($_[0]);
return $can ? 1 : 0;
}

# virt_acl_name(struct)
# Give a virtual host, returns a list of names that could be used in the ACL
# to refer to it
sub virt_acl_name
{
	my $self = shift;

return ( "__default__" ) if (!$_[0]);
local $n = $self->find_directive("ServerName", $_[0]->{'members'});
local @rv;
local $p;
if ($_[0]->{'value'} =~ /(:\d+)/) { $p = $1; }
if ($n) {
	push(@rv, $n.$p);
	}
else {
	push(@rv, $_[0]->{'value'});
	}
foreach $n ($self->find_directive_struct("ServerAlias", $_[0]->{'members'})) {
	local $a;
	foreach $a (@{$n->{'words'}}) {
		push(@rv, $a.$p);
		}
	}
return @rv;
}

# $self->allowed_auth_file(file)
sub allowed_auth_file
{
	my $self = shift;

local $_;
return 1 if ($self->{'access_hash'}->{'dir'} eq '/');
return 0 if ($_[0] =~ /\.\./);
local $f = $self->server_root($_[0], $self->get_config());
return 0 if (-l $f && !$self->allowed_auth_file(readlink($f)));
local $l = length($self->{'access_hash'}->{'dir'});
return length($f) >= $l && substr($f, 0, $l) eq $self->{'access_hash'}->{'dir'};
}

# directory_exists(file)
# Returns 1 if the directory in some path exists
sub directory_exists
{
	my $self = shift;

local $path = $self->server_root($_[0], $self->get_config());
if ($path =~ /^(\S*\/)([^\/]+)$/) {
	return -d $1;
	}
else {
	return 0;
	}
}

# allowed_doc_dir(dir)
# Returns 1 if some directory is under the allowed root for alias targets
sub allowed_doc_dir
{
	my $self = shift;

return $self->{'access_hash'}->{'aliasdir'} eq '/' ||
       $self->is_under_directory($self->{'access_hash'}->{'aliasdir'}, $_[0]);
}

sub lock_apache_files
{
	my $self = shift;

local $conf = $self->get_config();
local $f;
foreach $f ($self->unique(map { $_->{'file'} } @$conf)) {
	$self->lock_file($f);
	}
}

sub unlock_apache_files
{
	my $self = shift;

local $conf = $self->get_config();
local $f;
foreach $f ($self->unique(map { $_->{'file'} } @$conf)) {
	$self->unlock_file($f);
	}
}

# $self->directive_lines(directive, ...)
sub directive_lines
{
	my $self = shift;

local @rv;
foreach $d (@_) {
	next if ($d->{'name'} eq 'dummy');
	if ($d->{'type'}) {
		push(@rv, "<$d->{'name'} $d->{'value'}>");
		push(@rv, $self->directive_lines(@{$d->{'members'}}));
		push(@rv, "</$d->{'name'}>");
		}
	else {
		push(@rv, "$d->{'name'} $d->{'value'}");
		}
	}
return @rv;
}

# test_config()
# If possible, test the current configuration and return an error message,
# or undef.
sub test_config
{
	my $self = shift;

if ($self->{'httpd_modules_hash'}->{'core'} >= 1.301) {
	# Test the configuration with the available command
	local $cmd;
	if ($self->{'config_hash'}->{'test_apachectl'} &&
	    -x $self->translate_filename($self->{'config_hash'}->{'apachectl_path'})) {
		# Test with apachectl
		$cmd = "\"$self->{'config_hash'}->{'apachectl_path'}\" configtest";
		}
	else {
		# Test with httpd
		local $httpd = $self->find_httpd();
		$cmd = "\"$httpd\" -d \"$self->{'config_hash'}->{'httpd_dir'}\" -t";
		if ($self->{'config_hash'}->{'httpd_conf'}) {
			$cmd .= " -f \"$self->{'config_hash'}->{'httpd_conf'}\"";
			}
		foreach $d ($self->get_httpd_defines()) {
			$cmd .= " -D$d";
			}
		}
	local $out = $self->backquote_command("$cmd 2>&1");
	if ($out && $out !~ /syntax\s+ok/i) {
		return $out;
		}
	}
return undef;
}

# before_changing()
# If testing all changes, backup the config files so they can be reverted
# if necessary.
sub before_changing
{
	my $self = shift;

if ($self->{'config_hash'}->{'test_always'} || $self->{'access_hash'}->{'test_always'}) {
	local $conf = $self->get_config();
	local @files = $self->unique(map { $_->{'file'} } @$conf);
	local $/ = undef;
	local $f;
	foreach $f (@files) {
		if ($self->open_readfile(*BEFORE, $f)) {
			$self->{'before_changing_hash'}->{$f} = <BEFORE>;
			close(BEFORE);
			}
		}
	}
}

# after_changing()
# If testing all changes, test now and revert the configs and show an error
# message if a problem was found.
sub after_changing
{
	my $self = shift;

if ($self->{'config_hash'}->{'test_always'} || $self->{'access_hash'}->{'test_always'}) {
	local $err = $self->test_config();
	if ($err) {
		# Something failed .. revert all files
		$self->rollback_apache_config();
		$self->error($self->text('eafter', "<pre>$err</pre>"));
		}
	}
}

# rollback_apache_config()
# Copy back all config files from their originals
sub rollback_apache_config
{
	my $self = shift;

local $f;
foreach $f (keys %{$self->{'before_changing_hash'}}) {
	$self->open_tempfile(AFTER, ">$f");
	$self->print_tempfile(AFTER, $self->{'before_changing_hash'}->{$f});
	$self->close_tempfile(AFTER);
	}
}

# find_httpd_conf()
# Returns the path to the http.conf file, and the last place looked
# (without any translation).
sub find_httpd_conf
{
	my $self = shift;

local $conf = $self->{'config_hash'}->{'httpd_conf'};
return ( -f $self->translate_filename($conf) ? $conf : undef, $conf ) if ($conf);
$conf = "$self->{'config_hash'}->{'httpd_dir'}/conf/httpd.conf";
$conf = "$self->{'config_hash'}->{'httpd_dir'}/conf/httpd2.conf"
	if (!-f $self->translate_filename($conf));
$conf = "$self->{'config_hash'}->{'httpd_dir'}/etc/httpd.conf"
	if (!-f $self->translate_filename($conf));
$conf = "$self->{'config_hash'}->{'httpd_dir'}/etc/httpd2.conf"
	if (!-f $self->translate_filename($conf));
$conf = undef if (!-f $self->translate_filename($conf));
return ( $conf, "$self->{'config_hash'}->{'httpd_dir'}/conf/httpd.conf" );
}

# find_httpd()
# Returns the path to the httpd executable, by appending '2' if necessary
sub find_httpd
{
	my $self = shift;

return $self->{'config_hash'}->{'httpd_path'}
	if (-x $self->translate_filename($self->{'config_hash'}->{'httpd_path'}) &&
	    !-d $self->translate_filename($self->{'config_hash'}->{'httpd_path'}));
return $self->{'config_hash'}->{'httpd_path'}.'2'
	if (-x $self->translate_filename($self->{'config_hash'}->{'httpd_path'}.'2') &&
	    !-d $self->translate_filename($self->{'config_hash'}->{'httpd_path'}.'2'));
return undef;
}

# get_pid_file()
# Returns the path to the PID file (without any translation)
sub get_pid_file
{
	my $self = shift;

return $self->{'config_hash'}->{'pid_file'} if ($self->{'config_hash'}->{'pid_file'});
local $conf = $self->get_config();
local $pidfilestr = $self->find_directive_struct("PidFile", $conf);
local $pidfile = $pidfilestr ? $pidfilestr->{'words'}->[0]
		       	     : "logs/httpd.pid";
return $self->server_root($pidfile, $conf);
}

# restart_apache()
# Re-starts Apache, and returns undef on success or an error message on failure
sub restart_apache
{
	my $self = shift;

local $pidfile = $self->get_pid_file();
if ($self->{'config_hash'}->{'apply_cmd'} eq 'restart') {
	# Call stop and start functions
	local $err = $self->stop_apache();
	return $err if ($err);
	local $err = $self->start_apache();
	return $err if ($err);
	}
elsif ($self->{'config_hash'}->{'apply_cmd'}) {
	# Use the configured start command
	$self->clean_environment();
	local $out = $self->backquote_logged("$self->{'config_hash'}->{'apply_cmd'} 2>&1");
	$self->reset_environment();
	if ($?) {
		return "<pre>$out</pre>";
		}
	}
elsif (-x $self->translate_filename($self->{'config_hash'}->{'apachectl_path'})) {
	# Use apachectl to restart
	if ($self->{'httpd_modules_hash'}->{'core'} >= 2) {
		# Do a graceful restart
		$self->clean_environment();
		local $out = $self->backquote_logged("$self->{'config_hash'}->{'apachectl_path'} graceful 2>&1");
		$self->reset_environment();
		if ($?) {
			return "<pre>$out</pre>";
			}
		}
	else {
		$self->clean_environment();
		local $out = $self->backquote_logged("$self->{'config_hash'}->{'apachectl_path'} restart 2>&1");
		$self->reset_environment();
		if ($out !~ /httpd restarted/) {
			return "<pre>$out</pre>";
			}
		}
	}
else {
	# send SIGHUP directly
	$self->open_readfile(*PID, $pidfile) || return $self->text('restart_epid', $pidfile);
	<PID> =~ /(\d+)/ || return $self->text('restart_epid2', $pidfile);
	close(PID);
	$self->kill_logged('HUP', $1) || return $self->text('restart_esig', $1);
	}
return undef;
}

# stop_apache()
# Attempts to stop the running Apache process, and returns undef on success or
# an error message on failure
sub stop_apache
{
	my $self = shift;

local $out;
if ($self->{'config_hash'}->{'stop_cmd'}) {
	# use the configured stop command
	$out = $self->backquote_logged("($self->{'config_hash'}->{'stop_cmd'}) 2>&1");
	if ($?) {
		return "<pre>$out</pre>";
		}
	}
elsif (-x $self->{'config_hash'}->{'apachectl_path'}) {
	# use the apachectl program
	$out = $self->backquote_logged("($self->{'config_hash'}->{'apachectl_path'} stop) 2>&1");
	if ($self->{'httpd_modules_hash'}->{'core'} >= 2 ? $? : $out !~ /httpd stopped/) {
		return "<pre>$out</pre>";
		}
	}
else {
	# kill the process
	$self->{'pidfile'} = $self->get_pid_file();
	open(PID, $self->{'pidfile'}) || return $self->text('stop_epid', $self->{'pidfile'});
	<PID> =~ /(\d+)/ || return $self->text('stop_epid2', $self->{'pidfile'});
	close(PID);
	$self->kill_logged('TERM', $1) || return $self->text('stop_esig', $1);
	}
return undef;
}

# start_apache()
# Attempts to start Apache, and returns undef on success or an error message
# upon failure.
sub start_apache
{
	my $self = shift;

local $out;
$self->clean_environment();
if ($self->{'config_hash'}->{'start_cmd'}) {
	# use the configured start command
	if ($self->{'config_hash'}->{'stop_cmd'}) {
		# execute the stop command to clear lock files
		$self->system_logged("($self->{'config_hash'}->{'stop_cmd'}) >/dev/null 2>&1");
		}
	$out = $self->backquote_logged("($self->{'config_hash'}->{'start_cmd'}) 2>&1");
	$self->reset_environment();
	if ($?) {
		return "<pre>$out</pre>";
		}
	}
elsif (-x $self->{'config_hash'}->{'apachectl_path'}) {
	# use the apachectl program
	$out = $self->backquote_logged("($self->{'config_hash'}->{'apachectl_path'} start) 2>&1");
	$self->reset_environment();
	if ($out =~ /\S/ && $out !~ /httpd started/) {
		return "<pre>$out</pre>";
		}
	}
else {
	# start manually
	local $httpd = $self->find_httpd();
	local $cmd = "$httpd -d $self->{'config_hash'}->{'httpd_dir'}";
	if ($self->{'config_hash'}->{'httpd_conf'}) {
		$cmd .= " -f $self->{'config_hash'}->{'httpd_conf'}";
		}
	foreach $d ($self->get_httpd_defines()) {
		$cmd .= " -D$d";
		}
	local $temp = $self->transname();
	local $rv = $self->system_logged("( $cmd ) >$temp 2>&1 </dev/null");
	$out = `cat $temp`;
	unlink($temp);
	$self->reset_environment();
	if ($out =~ /\S/ && $out !~ /httpd started/) {
		return "<pre>$cmd :\n$out</pre>";
		}
	}

# check if startup was successful. Later apache version return no
# error code, but instead fail to start and write the reason to
# the error log file!
sleep(3);
local $conf = $self->get_config();
if (!$self->is_apache_running()) {
	# Not running..  find out why
	local $errorlogstr = $self->find_directive_struct("ErrorLog", $conf);
	local $errorlog = $errorlogstr ? $errorlogstr->{'words'}->[0]
				       : "logs/error_log";
	if ($out =~ /\S/) {
		return "$self->{'text_hash'}->{'start_eafter'} : <pre>$out</pre>";
		}
	elsif ($errorlog eq 'syslog' || $errorlog =~ /^\|/) {
		return $self->{'text_hash'}->{'start_eunknown'};
		}
	else {
		$errorlog = $self->server_root($errorlog, $conf);
		$out = `tail -5 $errorlog`;
		return "$self->{'text_hash'}->{'start_eafter'} : <pre>$out</pre>";
		}
	}
return undef;
}

sub is_apache_running
{
	my $self = shift;

if ($self->{'gconfig_hash'}->{'os_type'} eq 'windows') {
	# No such thing as a PID file on Windows
	local ($pid) = $self->find_byname("Apache.exe");
	return $pid;
	}
else {
	# Check PID file
	local $pidfile = $self->get_pid_file();
	return $self->check_pid_file($pidfile);
	}
}

# configurable_modules()
# Returns a list of Apaches that are compiled in or dynamically loaded, and
# supported by Webmin.
sub configurable_modules
{
	my $self = shift;

local ($ver, $mods) = $self->httpd_info($self->find_httpd());
local @rv;
local $m;

# add compiled-in modules
foreach $m (@$mods) {
	if (-r "$self->{'module_root_directory'}/$m.pl") {
		push(@rv, $m);
		}
	}

# add dynamically loaded modules
local $conf = $self->get_config();
foreach $l ($self->find_directive_struct("LoadModule", $conf)) {
	if ($l->{'words'}->[1] =~ /(mod_\S+)\.(so|dll)/ &&
	    -r "$self->{'module_root_directory'}/$1.pl") {
		push(@rv, $1);
		}
	elsif ($l->{'words'}->[1] =~ /libssl\.so/ &&
	       -r "$self->{'module_root_directory'}/mod_apachessl.pl") {
		push(@rv, "mod_apachessl");
		}
	elsif ($l->{'words'}->[1] =~ /lib([^\/\s]+)\.(so|dll)/ &&
	       -r "$self->{'module_root_directory'}/mod_$1.pl") {
		push(@rv, "mod_$1");
		}
	}

return $self->unique(@rv);
}

# get_httpd_defines(automatic-only)
# Returns a list of defines that need to be passed to Apache
sub get_httpd_defines
{
	my $self = shift;

local ($auto) = @_;
local @rv;
if (!$auto) {
	push(@rv, keys %{$self->{'httpd_defines_hash'}});
	}
if ($self->{'config_hash'}->{'defines_file'}) {
	# Add defines from an environment file
	local %def;
	$self->read_env_file($self->{'config_hash'}->{'defines_file'}, \%def);
	local $var = $def{$self->{'config_hash'}->{'defines_name'}};
	foreach my $v (split(/\s+/, $var)) {
		if ($v =~ /^-[Dd](\S+)$/) {
			push(@rv, $1);
			}
		else {
			push(@rv, $v);
			}
		}
	}
foreach my $md (split(/\t+/, $self->{'config_hash'}->{'defines_mods'})) {
	# Add HAVE_ defines from modules
	opendir(DIR, $md);
	while(my $m = readdir(DIR)) {
		if ($m =~ /^(mod_|lib)(.*).so$/i) {
			push(@rv, "HAVE_".uc($2));
			}
		}
	closedir(DIR);
	}
foreach my $d (split(/\s+/, $self->{'config_hash'}->{'defines'})) {
	push(@rv, $d);
	}
return @rv;
}

# create_webfile_link(file)
# Creates a link in the debian-style link directory for a new website's file
sub create_webfile_link
{
	my $self = shift;

local ($file) = @_;
if ($self->{'config_hash'}->{'link_dir'}) {
	local $short = $file;
	$short =~ s/^.*\///;
	local $linksrc = "$self->{'config_hash'}->{'link_dir'}/$short";
	$self->lock_file($linksrc);
	symlink($file, $linksrc);
	$self->unlock_file($linksrc);
	}
}

# delete_webfile_link(file)
# If the given path is in a directory like /etc/apache2/sites-available, delete
# the link to it from /etc/apache2/enabled-available
sub delete_webfile_link
{
	my $self = shift;

local ($file) = @_;
if ($self->{'config_hash'}->{'link_dir'}) {
	local $short = $file;
	$short =~ s/^.*\///;
	local $linksrc = "$self->{'config_hash'}->{'link_dir'}/$short";
	$self->lock_file($linksrc);
	unlink($linksrc);
	$self->unlock_file($linksrc);
	}
}


1;

