#!/usr/bin/perl
package apache::Core;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'apache::Core');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# core.pl
# Defines the core module directives

@{$self->{'AllowOverride_v_array'}} = ("AuthConfig", "FileInfo", "Indexes", "Limit", "Options");
@{$self->{'AllowOverride_d_array'}} = ("$self->{'text_hash'}->{'core_auth'}", "$self->{'text_hash'}->{'core_mime'}",
                    "$self->{'text_hash'}->{'core_indexing'}", "$self->{'text_hash'}->{'core_hostacc'}",
                    "$self->{'text_hash'}->{'core_diropts'}");

1;

1;
		return $self;
}

# core_directives(version)
# Returns ar array of references to associative arrays, each containing
# information about some directive. The keys of each array are:
#  name -	The name of this directive
#  type -	What kind of directive this in. Possible values are
#		0 - Processes and limits
#		1 - Networking and addresses
#		2 - Apache Modules
#		3 - Log files
#		4 - Access control
#		5 - document options
#		6 - MIME types
#		7 - Error handling
#		8 - Users and Groups
#		9 - Miscellaneous
#		10- Aliases and redirects
#		11- CGI programs
#		12- Directory indexing
#		13- Proxying
#		14- SSL
#		15- Perl
#		16- PHP
#		17- Vhost aliases
#		18- Filters
#		19- Character Sets
#		20- Image maps
#  multiple -	Can this directive appear multiple times
#  global -	Can be used in the global server context
#  virtual -	Can be used in a VirtualHost section or in the global section
#  directory -	Can be used in a Directory section context
#  htaccess -	Can be used in a .htaccess file
sub core_directives
{
	my $self = shift;

local($rv);
$rv = [	[ 'AccessFileName', 0, 5, 'virtual', undef, 5 ],
	[ 'AddDefaultCharset', 0, 19, 'virtual directory htaccess', 2.0 ],
	[ 'ClearModuleList AddModule', 1, 2, 'global', -2.0 ],
	[ 'AllowOverride', 0, 5, 'directory' ],
	[ 'AuthName', 0, 4, 'directory htaccess', undef, 10 ],
	[ 'AuthType', 0, 4, 'directory htaccess', undef, 8 ],
	[ 'BindAddress Listen Port', 1, 1, 'global', -2.0, 10 ],
	[ 'ContentDigest', 0, 5, 'virtual directory htaccess' ],
	[ 'CoreDumpDirectory', 0, 9, 'global', '1.3-2.0' ],
	[ 'DefaultType', 0, 6, 'virtual directory htaccess' ],
	[ 'DocumentRoot', 0, 5, 'virtual', undef, 10 ],
	[ 'ErrorDocument', 1, 7, 'virtual directory htaccess' ],
	[ 'ErrorLog', 0, 3, 'virtual' ],
	[ 'FileETag', 0, 5, 'virtual directory htaccess', 2.0 ],
	[ 'ForceType', 0, 6, 'directory htaccess', 2.0 ],
	[ 'Group', 0, 8, 'virtual', -2.0 ],
	[ 'HostNameLookups', 0, 1, 'virtual directory' ],
	[ 'IdentityCheck', 0, 1, 'virtual directory' ],
	[ 'KeepAlive MaxKeepAliveRequests', 0, 1, 'global' ],
	[ 'KeepAliveTimeout', 0, 1, 'global' ],
	[ 'ListenBacklog', 0, 1, 'global', '1.2-2.0' ],
	[ 'LockFile', 0, 9, 'global', -2.0 ],
	[ 'LimitRequestBody', 0, 0, 'virtual directory htaccess', 1.302 ],
	[ 'LimitRequestFields', 0, 0, 'global', 1.302 ],
	[ 'LimitRequestFieldsize', 0, 0, 'global', 1.302 ],
	[ 'LimitRequestLine', 0, 0, 'global', 1.302 ],
	[ 'LimitXMLRequestBody', 0, 0, 'virtual directory htaccess', 2.0 ],
	[ 'LogLevel', 0, 3, 'virtual', 1.3 ],
	[ 'MaxClients', 0, 0, 'global', -2.0 ],
	[ 'MaxRequestsPerChild', 0, 0, 'global', -2.0 ],
	[ 'StartServers', 0, 0, 'global', -2.0 ],
	[ 'MinSpareServers', 0, 0, 'global', -2.0 ],
	[ 'MaxSpareServers', 0, 0, 'global', -2.0 ],
	[ 'NameVirtualHost', 1, 1, 'global', 1.3, 5 ],
	[ 'Options', 0, 5, 'virtual directory htaccess', undef, 3 ],
	[ 'PidFile', 0, 9, 'global', -2.0 ],
	[ 'require', 0, 4, 'directory htaccess', undef, 6 ],
	[ 'RLimitCPU', 0, 0, 'virtual', 1.2 ],
	[ 'RLimitMEM', 0, 0, 'virtual', 1.2 ],
	[ 'RLimitNPROC', 0, 0, 'virtual', 1.2 ],
	[ 'Satisfy', 0, 4, 'directory htaccess', 1.2, 4 ],
	[ 'ScoreBoardFile', 0, 9, 'global', '1.2-2.0' ],
	[ 'SendBufferSize', 0, 1, 'global', -2.0 ],
	[ 'ServerAdmin', 0, 1, 'virtual' ],
	$self->{'access_hash'}->{'names'} ? (
		[ 'ServerAlias', 1, 1, 'virtual virtualonly', 1.2 ],
		[ 'ServerName', 0, 1, 'virtual' ] ) : ( ),
	[ 'ServerPath', 0, 5, 'virtual' ],
	[ 'ServerType', 0, 9, 'global', -2.0 ],
	[ 'ServerTokens', 0, 9, 'global', 1.3 ],
	[ 'ServerSignature', 0, 5, 'virtual directory htaccess', 1.3 ],
	[ 'SetOutputFilter', 0, 18, 'virtual directory htaccess', 2.0 ],
	[ 'SetInputFilter', 0, 18, 'virtual directory htaccess', 2.0 ],
	[ 'TimeOut', 0, 1, 'global' ],
	[ 'UseCanonicalName', 0, 1, 'virtual directory', 1.3 ],
	[ 'User', 0, 8, 'virtual', -2.0, 10 ] ];
return $self->make_directives($rv, $_[0], "core");
}

# core_handlers(config, version)
# Returns an array of all available handlers
sub core_handlers
{
	my $self = shift;

return ();
}

#########################################################################
# Process and limit directives
sub edit_MaxClients
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'core_maxconc'},
	$self->opt_input($_[0]->{'value'}, "MaxClients", $self->{'text_hash'}->{'core_default'}, 4));
}

sub save_MaxClients
{
	my $self = shift;

return $self->parse_opt("MaxClients", '^\d+$',
		  $self->{'text_hash'}->{'core_emaxconc'});
}

sub edit_MaxKeepAliveRequests
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'core_maxkeep'},
	$self->opt_input($_[0]->{'value'}, "MaxKeepAliveRequests", $self->{'text_hash'}->{'core_default'}, 4));
}

sub save_MaxKeepAliveRequests
{
	my $self = shift;

return $self->parse_opt("MaxKeepAliveRequests", '^\d+$',
		  $self->{'text_hash'}->{'core_emaxkeep'});
}

sub edit_MaxRequestsPerChild
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'core_maxreq'},
	$self->opt_input($_[0]->{'value'}, "MaxRequestsPerChild", $self->{'text_hash'}->{'core_default'}, 5));
}

sub save_MaxRequestsPerChild
{
	my $self = shift;

return $self->parse_opt("MaxRequestsPerChild", '^\d+$',
		  $self->{'text_hash'}->{'core_emaxreq'});
}

sub edit_MinSpareServers
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'core_minspare'},
	$self->opt_input($_[0]->{'value'},"MinSpareServers",$self->{'text_hash'}->{'core_default'}, 4));
}

sub save_MinSpareServers
{
	my $self = shift;

return $self->parse_opt("MinSpareServers", '^\d+$',
		  $self->{'text_hash'}->{'core_eminspare'});
}

sub edit_MaxSpareServers
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'core_maxspare'},
	$self->opt_input($_[0]->{'value'},"MaxSpareServers",$self->{'text_hash'}->{'core_default'}, 4));
}

sub save_MaxSpareServers
{
	my $self = shift;

return $self->parse_opt("MaxSpareServers", '^\d+$',
		  $self->{'text_hash'}->{'core_emaxspare'});
}

sub edit_StartServers
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'core_initial'},
	$self->opt_input($_[0]->{'value'}, "StartServers", $self->{'text_hash'}->{'core_default'}, 4));
}

sub save_StartServers
{
	my $self = shift;

return $self->parse_opt("StartServers", '^\d+$',
		  $self->{'text_hash'}->{'core_einitial'});
}

sub edit_RLimitCPU
{
	my $self = shift;

return $self->rlimit_input("RLimitCPU", $self->{'text_hash'}->{'core_cpulimit'}, $_[0]);
}

sub save_RLimitCPU
{
	my $self = shift;

return $self->parse_rlimit("RLimitCPU", $self->{'text_hash'}->{'core_cpulimit2'});
}

sub edit_RLimitMEM
{
	my $self = shift;

return $self->rlimit_input("RLimitMEM", $self->{'text_hash'}->{'core_memlimit'}, $_[0]);
}

sub save_RLimitMEM
{
	my $self = shift;

return $self->parse_rlimit("RLimitMEM", $self->{'text_hash'}->{'core_memlimit2'});
}

sub edit_RLimitNPROC
{
	my $self = shift;

return $self->rlimit_input("RLimitNPROC", $self->{'text_hash'}->{'core_proclimit'}, $_[0]);
}

sub save_RLimitNPROC
{
	my $self = shift;

return $self->parse_rlimit("RLimitNPROC", $self->{'text_hash'}->{'core_proclimit2'});
}

# rlimit_input(name, desc, value)
sub rlimit_input
{
	my $self = shift;

local(@v, $rv);
@v = split(/\s+/, $_[2]->{'value'});
$rv = sprintf "<input type=radio name=$_[0]_mode value=0 %s> $self->{'text_hash'}->{'core_default'}<br>\n",
	@v ? "" : "checked";
$rv .= sprintf "<input type=radio name=$_[0]_mode value=1 %s>\n",
	@v == 1 ? "checked" : "";
$rv .= sprintf "$self->{'text_hash'}->{'core_slimit'}<input name=$_[0]_soft1 size=5 value=\"%s\"><br>\n",
	@v == 1 ? $v[0] : "";
$rv .= sprintf "<input type=radio name=$_[0]_mode value=2 %s>\n",
	@v == 2 ? "checked" : "";
$rv .= sprintf "$self->{'text_hash'}->{'core_slimit'}<input name=$_[0]_soft2 size=5 value=\"%s\">\n",
	@v == 2 ? $v[0] : "";
$rv .= sprintf "$self->{'text_hash'}->{'core_hlimit'}<input name=$_[0]_hard2 size=5 value=\"%s\"><br>\n",
	@v == 2 ? $v[1] : "";
return (1, $_[1], $rv);
}

# parse_rlimit(name, desc)
sub parse_rlimit
{
	my $self = shift;

if ($self->{'in_hash'}->{"$_[0]_mode"} == 0) { return ( [ ] ); }
elsif ($self->{'in_hash'}->{"$_[0]_mode"} == 1) {
	$self->{'in_hash'}->{"$_[0]_soft1"} =~ /^(\d+|max)$/ ||
		$self->error($self->text('core_eslimit', $self->{'in_hash'}->{"$_[0]_soft1"}, $_[1]));
	return ( [ $self->{'in_hash'}->{"$_[0]_soft1"} ] );
	}
elsif ($self->{'in_hash'}->{"$_[0]_mode"} == 2) {
	$self->{'in_hash'}->{"$_[0]_soft2"} =~ /^(\d+|max)$/ ||
		$self->error($self->text('core_eslimit', $self->{'in_hash'}->{"$_[0]_soft2"}, $_[1]));
	$self->{'in_hash'}->{"$_[0]_hard2"} =~ /^(\d+|max)$/ ||
		$self->error($self->text('core_ehlimit', $self->{'in_hash'}->{"$_[0]_hard2"}, $_[1]));
	return ( [ $self->{'in_hash'}->{"$_[0]_soft2"}." ".$self->{'in_hash'}->{"$_[0]_hard2"} ] );
	}
}

#########################################################################
# Networking and address directives
sub edit_BindAddress_Listen_Port
{
	my $self = shift;

local($bref, $lref, $pref, @blist, @plist, $inp);
$bref = $_[0]; $lref = $_[1]; $pref = $_[2];
if (@$lref) {
	# listen directives in use.. so BindAddress and Port are unused
	foreach $l (@$lref) {
		if ($l->{'value'} =~ /^(\S+):(\d+)$/) {
			push(@blist, $1); push(@plist, $2);
			}
		elsif ($l->{'value'} =~ /^(\d+)$/) {
			push(@blist, "*"); push(@plist, $1);
			}
		elsif ($l->{'value'} =~ /^(\S+)$/) {
			push(@blist, $1); push(@plist, undef);
			}
		}
	}
else {
	# no listen directives... check for BindAddress
	if (@$bref) { push(@blist, $bref->[@$bref-1]->{'value'}); }
	else { push(@blist, "*"); }
	push(@plist, undef);
	}
$self->{'port'} = @$pref ? $pref->[@$pref-1]->{'value'} : 80;
if ($_[3]->{'version'} < 2.0) {
	$inp = "<b>$self->{'text_hash'}->{'core_dport'}</b> <input name=Port size=6 value=\"$self->{'port'}\"><br>\n";
	}
$inp .=	"<table border>\n".
	"<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'core_address'}</b></td> <td><b>$self->{'text_hash'}->{'core_port'}</b></td> </tr>\n";
for($i=0; $i<@blist+1; $i++) {
	$inp .= sprintf
		"<tr><td><input type=radio name=BindAddress_def_$i value=2 %s>".
		" $self->{'text_hash'}->{'core_none'} <input type=radio name=BindAddress_def_$i value=1 %s>".
		" $self->{'text_hash'}->{'core_all'} <input type=radio name=BindAddress_def_$i value=0 %s> ".
		"<input name=BindAddress_$i size=20 value=\"%s\"></td>",
		$blist[$i] ? "" : "checked",
		$blist[$i] eq "*" ? "checked" : "",
		$blist[$i] && $blist[$i] ne "*" ? "checked" : "",
		$blist[$i] eq "*" ? "" : $blist[$i];
	if ($_[3]->{'version'} < 2.0) {
		$inp .= "<td>".$self->opt_input($plist[$i], "Port_$i", "$self->{'text_hash'}->{'core_default'}", 5)."</td>";
		}
	else {
		$inp .= "<td><input name=Port_$i size=5 value='$plist[$i]'></td>\n";
		}
	}
$inp .= "</table>\n";
return (2, $self->{'text_hash'}->{'core_listen'}, $inp);
}

sub save_BindAddress_Listen_Port
{
	my $self = shift;

local(@blist, @plist, $bdef, $b, $p);

# build list of addresses and ports
for($i=0; defined($self->{'in_hash'}->{"Port_$i"}); $i++) {
	$bdef = $self->{'in_hash'}->{"BindAddress_def_$i"}; $b = $self->{'in_hash'}->{"BindAddress_$i"};
	$self->{'pdef'} = $self->{'in_hash'}->{"Port_${i}_def"}; $p = $self->{'in_hash'}->{"Port_$i"};
	if ($bdef == 2) { next; }

	if ($bdef) { push(@blist, "*"); }
	elsif ($b =~ /^\S+$/ && gethostbyname($b)) { push(@blist, $b); }
	else { $self->error($self->text('core_eaddress', $b)); }

	if ($self->{'pdef'}) { push(@plist, undef); }
	elsif ($p =~ /^\d+$/) { push(@plist, $p); }
	else { $self->error($self->text('core_eport', $p)); }
	}
if (!@blist) { $self->error($self->{'text_hash'}->{'core_eoneaddr'}); }

# Return directives
if ($_[0]->{'version'} < 2.0) {
	# Older apaches have a port directive as well
	$self->{'in_hash'}->{'Port'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'core_edefport'});
	if (@blist == 1 && !$plist[0]) {
		# Only one address, and the default port
		return ( $blist[0] eq "*" ? [ ] : [ $blist[0] ], [ ],
			 [ $self->{'in_hash'}->{'Port'} ] );
		}
	else {
		# More than one address, or a non-default port. Must use Listens
		for($i=0; $i<@blist; $i++) {
			if ($blist[$i] ne "*" && $plist[$i]) {
				push(@{$self->{'l_array'}}, "$blist[$i]:$plist[$i]");
				}
			elsif ($blist[$i] ne "*") { push(@{$self->{'l_array'}}, $blist[$i]); }
			elsif ($plist[$i]) { push(@{$self->{'l_array'}}, "*:$plist[$i]"); }
			else { push(@{$self->{'l_array'}}, $self->{'in_hash'}->{'Port'}); }
			}
		return ( [], \@{$self->{'l_array'}}, [ $self->{'in_hash'}->{'Port'} ] );
		}
	}
else {
	# Apache 2.0 just uses Listen directives
	for($i=0; $i<@blist; $i++) {
		if ($blist[$i] ne "*" && $plist[$i]) {
			push(@{$self->{'l_array'}}, "$blist[$i]:$plist[$i]");
			}
		elsif ($blist[$i] ne "*") { push(@{$self->{'l_array'}}, $blist[$i]); }
		else { push(@{$self->{'l_array'}}, "*:$plist[$i]"); }
		}
	return ( [], \@{$self->{'l_array'}} );
	}
}

sub edit_KeepAlive_MaxKeepAliveRequests
{
	my $self = shift;

$self->{'kref'} = $_[0]; $self->{'mref'} = $_[1];
if ($_[2]->{'version'} >= 1.2) {
	# two separate directives for keep-alives
	$self->{'inp'} = sprintf
		"<input type=radio name=KeepAlive_def value=0 %s> $self->{'text_hash'}->{'core_none'}\n".
		"<input type=radio name=KeepAlive_def value=1 %s> $self->{'text_hash'}->{'core_default'}\n".
		"<input type=radio name=KeepAlive_def value=2 %s> ".
		"<input name=KeepAlive size=5 value=\"%s\">",
		$self->{'kref'}->{'value'} =~ /off/i ? "checked" : "",
		$self->{'kref'}->{'value'} !~ /off/i && !$self->{'mref'} ? "checked" : "",
		$self->{'mref'} ? "checked" : "",
		$self->{'mref'} ? $self->{'mref'}->{'value'} : "";
	}
else {
	# only one directive
	$self->{'inp'} = sprintf
		"<input type=radio name=KeepAlive_def value=0 %s> $self->{'text_hash'}->{'core_none'}\n".
		"<input type=radio name=KeepAlive_def value=1 %s> $self->{'text_hash'}->{'core_default'}\n".
		"<input type=radio name=KeepAlive_def value=2 %s> ".
		"<input name=KeepAlive size=5 value=\"%s\">",
		$self->{'kref'}->{'value'} eq "0" ? "checked" : "",
		$self->{'kref'} ? "" : "checked" ,
		$self->{'kref'}->{'value'} ? "checked" : "",
		$self->{'kref'}->{'value'} ? $self->{'kref'}->{'value'} : "";
	}
return (1, $self->{'text_hash'}->{'core_multi'}, $self->{'inp'});
}

sub save_KeepAlive_MaxKeepAliveRequests
{
	my $self = shift;

if ($_[0]->{'version'} >= 1.2) {
	# two separate directives
	if ($self->{'in_hash'}->{'KeepAlive_def'} == 0) { return ( [ "off" ], [ ] ); }
	elsif ($self->{'in_hash'}->{'KeepAlive_def'} == 1) { return ( [ "on" ], [ ] ); }
	elsif ($self->{'in_hash'}->{'KeepAlive'} !~ /^\d+$/) {
		$self->error($self->text('core_ekeep', $self->{'in_hash'}->{'KeepAlive'}));
		}
	else { return ( [ "on" ], [ $self->{'in_hash'}->{'KeepAlive'} ] ); }
	}
else {
	# only one directive
	if ($self->{'in_hash'}->{'KeepAlive_def'} == 0) { return ( [ 0 ], [ ] ); }
	elsif ($self->{'in_hash'}->{'KeepAlive_def'} == 1) { return ( [ ], [ ] ); }
	elsif ($self->{'in_hash'}->{'KeepAlive'} !~ /^\d+$/) {
		$self->error($self->text('core_ekeep', $self->{'in_hash'}->{'KeepAlive'}));
		}
	else { return ( [ $self->{'in_hash'}->{'KeepAlive'} ], [ ] ); }
	}
}

sub edit_KeepAliveTimeout
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'core_keeptout'},
	$self->opt_input($_[0]->{'value'}, "KeepAliveTimeout", $self->{'text_hash'}->{'core_default'}, 5));
}

sub save_KeepAliveTimeout
{
	my $self = shift;

return $self->parse_opt("KeepAliveTimeout", '^\d+$',
		  $self->{'text_hash'}->{'core_ekeeptout'});
}

sub edit_ListenBacklog
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'core_lqueue'},
	$self->opt_input($_[0]->{'value'}, "ListenBacklog", $self->{'text_hash'}->{'core_default'}, 4));
}

sub save_ListenBacklog
{
	my $self = shift;

return $self->parse_opt("ListenBacklog", '^\d+$',
		  $self->{'text_hash'}->{'core_elqueue'});
}

sub edit_SendBufferSize
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'core_bufsize'},
	$self->opt_input($_[0]->{'value'}, "SendBufferSize", $self->{'text_hash'}->{'core_osdefault'}, 4));
}

sub save_SendBufferSize
{
	my $self = shift;

return $self->parse_opt("SendBufferSize", '^\d+$',
		  $self->{'text_hash'}->{'core_ebufsize'});
}

sub edit_ServerAdmin
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_admin'},
        $self->opt_input($_[0]->{'value'}, "ServerAdmin", $self->{'text_hash'}->{'core_noadmin'}, 25));
}

sub save_ServerAdmin
{
	my $self = shift;

return $self->parse_opt("ServerAdmin");
}

sub edit_TimeOut
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'core_rtout'},
	$self->opt_input($_[0]->{'value'}, "TimeOut", $self->{'text_hash'}->{'core_default'}, 4));
}

sub save_TimeOut
{
	my $self = shift;

return $self->parse_opt("TimeOut", '^\d+$',
		  $self->{'text_hash'}->{'core_ertout'});
}

sub edit_UseCanonicalName
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_bhostname'},
        $self->choice_input($_[0]->{'value'}, "UseCanonicalName",
	               "", "$self->{'text_hash'}->{'yes'},off", "$self->{'text_hash'}->{'no'},on", "$self->{'text_hash'}->{'core_default'},"));
}

sub save_UseCanonicalName
{
	my $self = shift;

return $self->parse_choice("UseCanonicalName", "");
}

sub edit_HostNameLookups
{
	my $self = shift;

if ($_[1]->{'version'} >= 1.3) {
	return (1, $self->{'text_hash'}->{'core_lookup'},
		$self->choice_input($_[0]->{'value'}, "HostNameLookups", "",
		       "$self->{'text_hash'}->{'no'},off", "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'core_ltwice'},double", "$self->{'text_hash'}->{'core_default'},"));
	}
else {
	return (1, $self->{'text_hash'}->{'core_lookup'},
		$self->choice_input($_[0]->{'value'}, "HostNameLookups", "",
			      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off", "$self->{'text_hash'}->{'core_default'},"));
	}
}

sub save_HostNameLookups
{
	my $self = shift;

return $self->parse_choice("HostNameLookups", "");
}

sub edit_IdentityCheck
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_useauth'},
	$self->choice_input($_[0]->{'value'}, "IdentityCheck", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off", "$self->{'text_hash'}->{'core_default'},"));
}

sub save_IdentityCheck
{
	my $self = shift;

return $self->parse_choice("IdentityCheck", "");
}

sub edit_ServerAlias
{
	my $self = shift;

local($a, @al);
foreach $a (@{$_[0]}) { push(@al, split(/\s+/, $a->{'value'})); }
return (1, $self->{'text_hash'}->{'core_altnames'},
	sprintf "<textarea name=ServerAlias rows=3 cols=25>%s</textarea>\n",
		join("\n", @al) );
}

sub save_ServerAlias
{
	my $self = shift;

local(@al);
@al = split(/\s+/, $self->{'in_hash'}->{'ServerAlias'});
if (@al) {
	local @spal;
	while(@al > 200) {
		push(@spal, join(" ", @al[0 .. 199]));
		@al = @al[200 .. $#al];
		}
	push(@spal, join(" ", @al)) if (@al);
	return ( \@spal );
	}
else {
	return ( [ ] );
	}
}

sub edit_ServerName
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_hostname'},
	$self->opt_input($_[0]->{'value'}, "ServerName", $self->{'text_hash'}->{'core_auto'}, 25));
}

sub save_ServerName
{
	my $self = shift;

return $self->parse_opt("ServerName", '^\S+$', $self->{'text_hash'}->{'core_ehostname'});
}

sub edit_NameVirtualHost
{
	my $self = shift;

local(@nv, $nv, $star);
foreach $nv (@{$_[0]}) {
	if ($nv->{'value'} eq "*" && $_[1]->{'version'} >= 1.312) { $star++; }
	else { push(@nv, $nv->{'value'}); }
	}
if ($_[1]->{'version'} >= 1.312) {
	$self->{'starui'} = sprintf
	  "<input type=checkbox name=NameVirtualHost_star value=1 %s> %s<br>\n",
	  $star ? "checked" : "", $self->{'text_hash'}->{'core_virtaddr_star'};
	}
return (1, $self->{'text_hash'}->{'core_virtaddr'},
	$self->{'starui'}.
	"<textarea name=NameVirtualHost rows=3 cols=30>".
	join("\n", @nv)."</textarea>");
}

sub save_NameVirtualHost
{
	my $self = shift;

local(@nv, $nv, $addr);
@nv = split(/\s+/, $self->{'in_hash'}->{'NameVirtualHost'});
@nv = ( "*", @nv ) if ($self->{'in_hash'}->{'NameVirtualHost_star'});
foreach $nv (@nv) {
	if ($nv =~ /^(\S+):(\d+|\*)$/) { $addr = $1; }
	else { $addr = $nv; }
	if (!gethostbyname($addr) && $addr ne '*') {
		$self->error($self->text('core_evirtaddr', $addr));
		}
	}
if (@nv) { return ( \@nv ); }
else { return ( [ ] ); }
}

#########################################################################
# Document directives
sub edit_AccessFileName
{
	my $self = shift;

return (1,
        $self->{'text_hash'}->{'core_optfile'},
        $self->opt_input($_[0]->{'value'}, "AccessFileName", $self->{'text_hash'}->{'core_default'}, 20));
}

sub save_AccessFileName
{
	my $self = shift;

if ($_[0]->{'version'} < 1.3) {
	return $self->parse_opt("AccessFileName", '^(\S+)$',
	                  $self->{'text_hash'}->{'core_eoptfile'});
	}
else {
	return $self->parse_opt("AccessFileName", '\S',
			     $self->{'text_hash'}->{'core_enoopt'});
	}
}

sub edit_AllowOverride
{
	my $self = shift;

local($rv, @ov, %over, $rv);
$rv = $self->choice_input($_[0] ? 0 : 1, "AllowOverride_def", 1,
		 "$self->{'text_hash'}->{'core_default'},1", "$self->{'text_hash'}->{'core_filesel'},0");
$rv .= "<table border><tr><td>\n";
if ($_[0]) { @ov = split(/\s+/, $_[0]->{'value'}); }
else { @ov = ("All"); }
foreach $ov (@ov) { $over{$ov}++; }
if ($over{'All'}) { foreach $ov (@{$self->{'AllowOverride_v_array'}}) { $over{$ov}++; }	}
elsif ($over{'None'}) { %over = (); }

for($i=0; $i<@{$self->{'AllowOverride_v_array'}}; $i++) {
	$rv .= sprintf "<input type=checkbox name=AllowOverride_%s %s> %s<br>\n",
	        $self->{'AllowOverride_v_array'}->[$i],
	        $over{$self->{'AllowOverride_v_array'}->[$i]} ? "checked" : "",
	        $self->{'AllowOverride_d_array'}->[$i];
	}
$rv .= "</td></tr></table>\n";
return (1, $self->{'text_hash'}->{'core_overr'}, $rv);
}

sub save_AllowOverride
{
	my $self = shift;

local(@ov, $ov);
if ($self->{'in_hash'}->{'AllowOverride_def'}) { return ( [ ] ); }
foreach $ov (@{$self->{'AllowOverride_v_array'}}) {
	if ($self->{'in_hash'}->{"AllowOverride_$ov"}) { push(@ov, $ov); }
	}
if (!@ov) { return ( [ "None" ] ); }
elsif (@ov == @{$self->{'AllowOverride_v_array'}}) { return ( [ "All" ] ); }
else { return ( [ join(' ', @ov) ] ); }
}

sub edit_ContentDigest
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'core_genmd5'},
	$self->choice_input($_[0]->{'value'}, "ContentDigest", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off", "$self->{'text_hash'}->{'core_default'},"));
}

sub save_ContentDigest
{
	my $self = shift;

return $self->parse_choice("ContentDigest", "");
}

sub edit_DocumentRoot
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'core_docroot'},
	$self->opt_input($_[0]->{'words'}->[0], "DocumentRoot", $self->{'text_hash'}->{'core_default'}, 40).
	$self->file_chooser_button("DocumentRoot", 1));
}

sub save_DocumentRoot
{
	my $self = shift;

if (!$self->{'in_hash'}->{'DocumentRoot_def'}) {
	-d $self->{'in_hash'}->{'DocumentRoot'} ||
		$self->error($self->text('core_enodoc', $self->{'in_hash'}->{'DocumentRoot'}));
	$self->allowed_doc_dir($self->{'in_hash'}->{'DocumentRoot'}) ||
		$self->error($self->text('core_ecandoc', $self->{'in_hash'}->{'DocumentRoot'}));
	}
return $self->parse_opt("DocumentRoot");
}

sub edit_Options
{
	my $self = shift;

local(@po, @o, $o, %opts, $opts, $po, @pon, $i);
@po = ("ExecCGI", "FollowSymLinks", "Includes", "IncludesNOEXEC",
       "Indexes", "MultiViews", "SymLinksIfOwnerMatch");
@pon = ("$self->{'text_hash'}->{'core_execcgi'}", "$self->{'text_hash'}->{'core_flink'}",
	"$self->{'text_hash'}->{'core_inclexe'}", "$self->{'text_hash'}->{'core_incl'}",
	"$self->{'text_hash'}->{'core_genind'}", "$self->{'text_hash'}->{'core_genmview'}",
	"$self->{'text_hash'}->{'core_flinkmatch'}");
$opts = $self->choice_input($_[0] ? 0 : 1, "Options_def", 1, "$self->{'text_hash'}->{'core_default'},1",
		      "$self->{'text_hash'}->{'core_optsel'},0")."<br>\n";
@o = split(/\s+/, $_[0]->{'value'});
foreach $o (split(/\s+/, $_[0]->{'value'})) {
	if ($o =~ /^\+(.*)$/) { $opts{$1} = 2; }
	elsif ($o =~ /^\-(.*)$/) { $opts{$1} = 3; }
	else { $opts{$o} = 1; }
	}
if ($opts{'All'}) {
	local($all); $all = $opts{'All'};
	undef(%opts);
	foreach $o (grep {!/MultiViews/} @po) {
		$opts{$o} = $all;
		}
	}
$opts .= "<table border>\n";
$opts .= "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'core_option'}</b></td> <td><b>$self->{'text_hash'}->{'core_setdir'}</b></td>\n";
$opts .= "<td><b>$self->{'text_hash'}->{'core_merge'}</b></td> </tr>\n";
for($i=0; $i<@po; $i++) {
	$po = $po[$i];
	$opts .= "<tr $self->{'cb'}> <td><b>$pon[$i]</b></td> <td>\n";
	$opts .= sprintf "<input type=radio name=$po value=1 %s> $self->{'text_hash'}->{'yes'}\n",
			$opts{$po}==1 ? "checked" : "";
	$opts .= sprintf "<input type=radio name=$po value=0 %s> $self->{'text_hash'}->{'no'}\n",
			$opts{$po}==0 ? "checked" : "";
	$opts .= "</td> <td>\n";
	$opts .= sprintf "<input type=radio name=$po value=2 %s> $self->{'text_hash'}->{'core_enable'}\n",
			$opts{$po}==2 ? "checked" : "";
	$opts .= sprintf "<input type=radio name=$po value=3 %s> $self->{'text_hash'}->{'core_disable'}\n",
			$opts{$po}==3 ? "checked" : "";
	$opts .= "</td> </tr>\n";
	}
$opts .= "</table>\n";
return (2, $self->{'text_hash'}->{'core_diropts'}, $opts);
}

sub save_Options
{
	my $self = shift;

local(@po, $po, @rv);
if ($self->{'in_hash'}->{'Options_def'}) { return ( [ ] ); }
@po = ("ExecCGI", "FollowSymLinks", "Includes", "IncludesNOEXEC",
       "Indexes", "MultiViews", "SymLinksIfOwnerMatch");
foreach $po (@po) {
	if ($self->{'in_hash'}->{$po} == 1) { push(@rv, $po); }
	elsif ($self->{'in_hash'}->{$po} == 2) { push(@rv, "+$po"); }
	elsif ($self->{'in_hash'}->{$po} == 3) { push(@rv, "-$po"); }
	}
return @rv ? ( [ join(' ', @rv) ] ) : ( [ "None" ] );
}

sub edit_ServerPath
{
	my $self = shift;

return (2,
	$self->{'text_hash'}->{'core_virtpath'},
	$self->opt_input($_[0]->{'value'}, "ServerPath", $self->{'text_hash'}->{'core_default'}, 40).
	$self->file_chooser_button("ServerPath", 1));
}

sub save_ServerPath
{
	my $self = shift;

return $self->parse_opt("ServerPath", '^\/\S*$',
		  $self->{'text_hash'}->{'core_evirtpath'});
}

sub edit_ServerSignature
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_footer'},
	$self->select_input($_[0]->{'value'}, "ServerSignature", undef,
		      "$self->{'text_hash'}->{'core_sigemail'},Email", "$self->{'text_hash'}->{'core_signame'},On",
		      "$self->{'text_hash'}->{'core_signone'},Off", "$self->{'text_hash'}->{'core_default'},"));
}

sub save_ServerSignature
{
	my $self = shift;

return $self->parse_select("ServerSignature", undef);
}

sub edit_FileETag
{
	my $self = shift;

local (%et, $rv);
map { $et{lc($_)}++ } @{$_[0]->{'words'}} if ($_[0]);
$rv .= sprintf "<input type=radio name=FileETag_def value=1 %s> %s\n",
			$_[0] ? "" : "checked", $self->{'text_hash'}->{'default'};
$rv .= sprintf "<input type=radio name=FileETag_def value=0 %s> %s\n",
			$_[0] ? "checked" : "", $self->{'text_hash'}->{'core_fileetag_sel'};
foreach $e ('INode', 'MTime', 'Size') {
	$rv .= sprintf "<input type=checkbox name=FileETag value=%s %s> %s\n",
			$e, $et{lc($e)} || $et{'all'} ? "checked" : "",
			$self->{'text_hash'}->{'core_fileetag_'.lc($e)};
	}
return (2, $self->{'text_hash'}->{'core_fileetag'}, $rv);
}

sub save_FileETag
{
	my $self = shift;

if ($self->{'in_hash'}->{'FileETag_def'}) {
	return ( [ ] );
	}
else {
	local @e = split(/\0/, $self->{'in_hash'}->{'FileETag'});
	return @e ? ( [ join(" ", @e) ] ) : ( [ "None" ] );
	}
}

#########################################################################
# MIME directives directives
sub edit_DefaultType
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'core_defmime'},
	$self->opt_input($_[0]->{'value'}, "DefaultType", $self->{'text_hash'}->{'core_default'}, 20));
}

sub save_DefaultType
{
	my $self = shift;

return $self->parse_opt("DefaultType", '^(\S+)\/(\S+)$',
		  $self->{'text_hash'}->{'core_edefmime'});
}

sub edit_ForceType
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_mime_defmime'},
	$self->opt_input($_[0]->{'value'}, "ForceType", $self->{'text_hash'}->{'mod_mime_real'}, 15));
}

sub save_ForceType
{
	my $self = shift;

return $self->parse_opt("ForceType", '^\S+\/\S+$', $self->{'text_hash'}->{'mod_mime_etype'});
}

sub edit_SetOutputFilter
{
	my $self = shift;

local @vals = split(/[\s;]+/, $_[0]->{'value'});
return (2, $self->{'text_hash'}->{'core_outfilter'},
	$self->filters_input(\@vals, "SetOutputFilter"));
}

sub save_SetOutputFilter
{
	my $self = shift;

return $self->parse_filters("SetOutputFilter");
}

sub edit_SetInputFilter
{
	my $self = shift;

local @vals = split(/[\s;]+/, $_[0]->{'value'});
return (2, $self->{'text_hash'}->{'core_infilter'},
	$self->filters_input(\@vals, "SetInputFilter"));
}

sub save_SetInputFilter
{
	my $self = shift;

return $self->parse_filters("SetInputFilter");
}

sub edit_AddDefaultCharset
{
	my $self = shift;

local $rv;
local $m = lc($_[0]->{'value'}) eq 'off' ? 2 :
	   $_[0]->{'value'} ? 0 : 1;
$rv .= sprintf "<input type=radio name=AddDefaultCharset_def value=1 %s> %s\n",
		$m == 1 ? "checked" : "", $self->{'text_hash'}->{'default'};
$rv .= sprintf "<input type=radio name=AddDefaultCharset_def value=2 %s> %s\n",
		$m == 2 ? 'checked' : "", $self->{'text_hash'}->{'core_none'};
$rv .= sprintf "<input type=radio name=AddDefaultCharset_def value=0 %s>\n",
		$m == 0 ? "checked" : "";
$rv .= sprintf "<input name=AddDefaultCharset size=12 value='%s'>\n",
		$m == 0 ? $_[0]->{'value'} : "";
return (1, $self->{'text_hash'}->{'core_defchar'}, $rv);
}

sub save_AddDefaultCharset
{
	my $self = shift;

if ($self->{'in_hash'}->{'AddDefaultCharset_def'} == 1) {
	return ( [ ] );
	}
elsif ($self->{'in_hash'}->{'AddDefaultCharset_def'} == 2) {
	return ( [ "Off" ] );
	}
else {
	$self->{'in_hash'}->{'AddDefaultCharset'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'core_edefchar'});
	return ( [ $self->{'in_hash'}->{'AddDefaultCharset'} ] );
	}
}

#########################################################################
# Access control directives
sub edit_AuthName
{
	my $self = shift;

my $val = $_[1]->{'version'} >= 1.3 ? $_[0]->{'words'}->[0]
				    : $_[0]->{'value'};
return (1, $self->{'text_hash'}->{'core_realm'},
        $self->opt_input($val, "AuthName", $self->{'text_hash'}->{'core_default'}, 25));
}

sub save_AuthName
{
	my $self = shift;

return $self->{'in_hash'}->{'AuthName_def'}	 ? ( [ ] ) :
       $_[0]->{'version'} >= 1.3 ? ( [ "\"$self->{'in_hash'}->{'AuthName'}\"" ] ) :
				   ( [ $self->{'in_hash'}->{'AuthName'} ] );
}

sub edit_AuthType
{
	my $self = shift;

local($rv, $a);
$rv = "<select name=AuthType>\n";
foreach $a ("", "Basic", "Digest") {
	$rv .= sprintf "<option %s>$a\n",
	        lc($_[0]->{'value'}) eq lc($a) ? "selected" : "";
	}
$rv .= "</select>";
return (1, $self->{'text_hash'}->{'core_authtype'}, $rv);
}

sub save_AuthType
{
	my $self = shift;

if ($self->{'in_hash'}->{'AuthType'}) { return ( [ $self->{'in_hash'}->{'AuthType'} ] ); }
else { return ( [ ] ); }
}

sub edit_require
{
	my $self = shift;

local($rv, $mode, $list);
$_[0]->{'value'} =~ /^(\S+)\s*(.*)$/; $mode = $1; $list = $2;

# All users
$rv = sprintf
      "<input type=radio name=require_mode value=0 %s> $self->{'text_hash'}->{'default'}<br>\n",
      $mode ? "" : "checked";

# Only some users
$rv .= sprintf
      "<input type=radio name=require_mode value=1 %s> $self->{'text_hash'}->{'core_users'}:\n",
      $mode eq "user" ? "checked" : "";
$rv .= sprintf
      "<input name=require_user size=20 value=\"%s\"><br>\n",
      $mode eq "user" ? $list : "";

# Only members of groups
$rv .= sprintf
      "<input type=radio name=require_mode value=2 %s> $self->{'text_hash'}->{'core_groups'}:\n",
      $mode eq "group" ? "checked" : "";
$rv .= sprintf
      "<input name=require_group size=20 value=\"%s\"><br>\n",
      $mode eq "group" ? $list : "";

# All users
$rv .= sprintf
      "<input type=radio name=require_mode value=3 %s> $self->{'text_hash'}->{'core_allusers'}<br>\n",
      $mode eq "valid-user" ? "checked" : "";

if ($self->{'httpd_modules_hash'}->{'mod_authz_owner'} >= 2.2) {
	# File owner / group matches
	$rv .= sprintf
	      "<input type=radio name=require_mode value=4 %s> $self->{'text_hash'}->{'core_fileowner'}<br>\n",
	      $mode eq "file-owner" ? "checked" : "";
	$rv .= sprintf
	      "<input type=radio name=require_mode value=5 %s> $self->{'text_hash'}->{'core_filegroup'}<br>\n",
	      $mode eq "file-group" ? "checked" : "";

	}
return (1, $self->{'text_hash'}->{'core_authlog'}, $rv);
}

sub save_require
{
	my $self = shift;

if ($self->{'in_hash'}->{'require_mode'} == 0) { return ( [ ] ); }
elsif ($self->{'in_hash'}->{'require_mode'} == 1) { return ( [ "user $self->{'in_hash'}->{'require_user'}" ] ); }
elsif ($self->{'in_hash'}->{'require_mode'} == 2) { return ( [ "group $self->{'in_hash'}->{'require_group'}" ] ); }
elsif ($self->{'in_hash'}->{'require_mode'} == 3) { return ( [ "valid-user" ] ); }
elsif ($self->{'in_hash'}->{'require_mode'} == 4) { return ( [ "file-owner" ] ); }
elsif ($self->{'in_hash'}->{'require_mode'} == 5) { return ( [ "file-group" ] ); }
else { return ( [ ] ); }	# huh?
}

sub edit_Satisfy
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_satisfy'},
	$self->choice_input_vert($_[0]->{'value'}, "Satisfy", "", "$self->{'text_hash'}->{'core_default'},",
			   "$self->{'text_hash'}->{'core_authall'},all","$self->{'text_hash'}->{'core_authany'},any"));
}

sub save_Satisfy
{
	my $self = shift;

return $self->parse_choice("Satisfy", "");
}

#########################################################################
# Misc. directives
sub edit_CoreDumpDirectory
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_coredir'},
	 $self->opt_input($_[0]->{'words'}->[0], "CoreDumpDirectory", $self->{'text_hash'}->{'core_sroot'}, 20).
	 $self->file_chooser_button("CoreDumpDirectory", 1));
}

sub save_CoreDumpDirectory
{
	my $self = shift;

return $self->parse_opt("CoreDumpDirectory", '^\S+$', $self->{'text_hash'}->{'core_ecore'});
}

sub edit_LockFile
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_lockfile'},
        $self->opt_input($_[0]->{'words'}->[0], "LockFile", $self->{'text_hash'}->{'core_default'}, 20).
        $self->file_chooser_button("LockFile", 0));
}

sub save_LockFile
{
	my $self = shift;

return $self->parse_opt("LockFile", '^\S+', $self->{'text_hash'}->{'core_elock'});
}

sub edit_LimitRequestBody
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_maxbody'},
	$self->opt_input($_[0]->{'value'}, "LimitRequestBody", $self->{'text_hash'}->{'core_default'}, 8)
		.$self->{'text_hash'}->{'bytes'});
}

sub save_LimitRequestBody
{
	my $self = shift;

return $self->parse_opt("LimitRequestBody", '^\d+$', $self->{'text_hash'}->{'core_ebody'});
}

sub edit_LimitXMLRequestBody
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_maxxml'},
	$self->opt_input($_[0]->{'value'}, "LimitXMLRequestBody",
		   $self->{'text_hash'}->{'core_default'}, 8).$self->{'text_hash'}->{'bytes'});
}

sub save_LimitXMLRequestBody
{
	my $self = shift;

return $self->parse_opt("LimitXMLRequestBody", '^\d+$', $self->{'text_hash'}->{'core_exml'});
}

sub edit_LimitRequestFields
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_maxhead'},
	$self->opt_input($_[0]->{'value'}, "LimitRequestFields", $self->{'text_hash'}->{'core_default'}, 6));
}

sub save_LimitRequestFields
{
	my $self = shift;

return $self->parse_opt("LimitRequestFields", '^\d+$', $self->{'text_hash'}->{'core_ehead'});
}

sub edit_LimitRequestFieldsize
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_maxshead'},
	$self->opt_input($_[0]->{'value'}, "LimitRequestFieldsize", $self->{'text_hash'}->{'core_default'}, 6));
}

sub save_LimitRequestFieldsize
{
	my $self = shift;

return $self->parse_opt("LimitRequestFieldsize", '^\d+$', $self->{'text_hash'}->{'core_eshead'});
}

sub edit_LimitRequestLine
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_maxline'},
	$self->opt_input($_[0]->{'value'}, "LimitRequestLine", $self->{'text_hash'}->{'core_default'}, 6));
}

sub save_LimitRequestLine
{
	my $self = shift;

return $self->parse_opt("LimitRequestLine", '^\d+$', $self->{'text_hash'}->{'core_eline'});
}

sub edit_PidFile
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_pid'},
        $self->opt_input($_[0]->{'words'}->[0], "PidFile", $self->{'text_hash'}->{'core_default'}, 20).
        $self->file_chooser_button("PidFile", 0));
}

sub save_PidFile
{
	my $self = shift;

return $self->parse_opt("PidFile", '^\S+$', $self->{'text_hash'}->{'core_epid'});
}

sub edit_ScoreBoardFile
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_memsco'},
        $self->opt_input($_[0]->{'words'}->[0], "ScoreBoardFile", $self->{'text_hash'}->{'core_default'}, 20).
        $self->file_chooser_button("ScoreBoardFile", 0));
}

sub save_ScoreBoardFile
{
	my $self = shift;

return $self->parse_opt("ScoreBoardFile", '^\S+$', $self->{'text_hash'}->{'core_escore'});
}

sub edit_ServerType
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_exec'},
        $self->choice_input($_[0]->{'value'}, "ServerType", "standalone",
                      "$self->{'text_hash'}->{'core_salone'},standalone", "$self->{'text_hash'}->{'core_inetd'},inetd"));
}

sub save_ServerType
{
	my $self = shift;

return $self->parse_choice("ServerType", "standalone");
}

sub edit_ServerTokens
{
	my $self = shift;

local $v = $_[0]->{'value'};
$v = "ProductOnly" if ($v eq "Prod");
return (1, $self->{'text_hash'}->{'core_header'},
	$self->select_input($v, "ServerTokens", "Full",
		      "$self->{'text_hash'}->{'core_verosmod'},Full",
		      "$self->{'text_hash'}->{'core_veros'},OS", "$self->{'text_hash'}->{'core_ver'},Min",
		      $_[1]->{'version'} >= 1.313 ?
			("$self->{'text_hash'}->{'core_product'},ProductOnly") : (),
		      $_[1]->{'version'} >= 2.041 ?
			("$self->{'text_hash'}->{'core_major'},Major") : ()
			));
}

sub save_ServerTokens
{
	my $self = shift;

return $self->parse_select("ServerTokens", "Full");
}

#########################################################################
# User/group directives
sub edit_Group
{
	my $self = shift;

local($rv, @ginfo);
$rv = sprintf "<input type=radio name=Group value=0 %s>$self->{'text_hash'}->{'core_default'}&nbsp;\n",
       $_[0] ? "" : "checked";
$rv .= sprintf "<input type=radio name=Group value=1 %s>$self->{'text_hash'}->{'core_group'}\n",
        $_[0] && $_[0]->{'words'}->[0] !~ /^#/ ? "checked" : "";
$rv .= sprintf "<input name=Group_name size=8 value=\"%s\"> %s&nbsp;\n",
	$_[0]->{'words'}->[0] !~ /^#/ ? $_[0]->{'words'}->[0] : "",
	$self->group_chooser_button("Group_name", 0);
$rv .= sprintf "<input type=radio name=Group value=2 %s>$self->{'text_hash'}->{'core_gid'}\n",
        $_[0]->{'words'}->[0] =~ /^#/ ? "checked" : "";
$rv .= sprintf "<input name=Group_id size=6 value=\"%s\">\n",
	 $_[0]->{'words'}->[0] =~ /^#(.*)$/ ? $1 : "";
return (2, $self->{'text_hash'}->{'core_asgroup'}, $rv);
}

sub save_Group
{
	my $self = shift;

if ($self->{'in_hash'}->{'Group'} == 0) { return ( [ ] ); }
elsif ($self->{'in_hash'}->{'Group'} == 1) { return ( [ $self->{'in_hash'}->{'Group_name'} ] ); }
elsif ($self->{'in_hash'}->{'Group_id'} !~ /^\-?\d+$/) {
	$self->error($self->text('core_euid', $self->{'in_hash'}->{'Group_id'}));
	}
else { return ( [ "\"#$self->{'in_hash'}->{'Group_id'}\"" ] ); }
}

sub edit_User
{
	my $self = shift;

local($rv, @uinfo);
$rv = sprintf "<input type=radio name=User value=0 %s>$self->{'text_hash'}->{'core_default'}&nbsp;\n",
       $_[0] ? "" : "checked";
$rv .= sprintf "<input type=radio name=User value=1 %s>$self->{'text_hash'}->{'core_user'}\n",
        $_[0] && $_[0]->{'words'}->[0] !~ /^#/ ? "checked" : "";
$rv .= sprintf "<input name=User_name size=8 value=\"%s\"> %s&nbsp;\n",
	$_[0]->{'words'}->[0] !~ /^#/ ? $_[0]->{'words'}->[0] : "",
	$self->user_chooser_button("User_name", 0);
$rv .= sprintf "<input type=radio name=User value=2 %s>$self->{'text_hash'}->{'core_uid'}\n",
        $_[0]->{'words'}->[0] =~ /^#/ ? "checked" : "";
$rv .= sprintf "<input name=User_id size=6 value=\"%s\">\n",
	 $_[0]->{'words'}->[0] =~ /^#(.*)$/ ? $1 : "";
return (2, $self->{'text_hash'}->{'core_asuser'}, $rv);
}

sub save_User
{
	my $self = shift;

if ($self->{'in_hash'}->{'User'} == 0) { return ( [ ] ); }
elsif ($self->{'in_hash'}->{'User'} == 1) { return ( [ $self->{'in_hash'}->{'User_name'} ] ); }
elsif ($self->{'in_hash'}->{'User_id'} !~ /^\-?\d+$/) {
	$self->error($self->text('core_egid', $self->{'in_hash'}->{'User_id'}));
	}
else { return ( [ "\"#$self->{'in_hash'}->{'User_id'}\"" ] ); }
}

#########################################################################
# Error handling directives
sub edit_ErrorDocument
{
	my $self = shift;

local($rv, $len, $i);
$rv = "<table border width=100%>\n";
$rv .= "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'core_error'}</b></td> <td><b>$self->{'text_hash'}->{'core_resp'}</b></td> ".
       "<td><b>$self->{'text_hash'}->{'core_urlmsg'}</b></td> </tr>\n";
$len = @{$_[0]} + 1;
for($i=0; $i<$len; $i++) {
	$self->{'v'} = $_[0]->[$i]->{'value'};
	if ($self->{'v'} =~ /^(\d+)\s+((http|ftp|gopher):\S+)$/)
		{ $self->{'code'} = $1; $self->{'type'} = 0; $self->{'url'} = $2; }
	elsif ($self->{'v'} =~ /^(\d+)\s+(\/.*)$/) { $self->{'code'} = $1; $self->{'type'} = 0; $self->{'url'} = $2; }
	elsif ($self->{'v'} =~ /^(\d+)\s+"(.*)"$/) { $self->{'code'} = $1; $self->{'type'} = 1; $self->{'url'} = $2; }
	elsif ($self->{'v'} =~ /^(\d+)\s+"?(.*)$/) { $self->{'code'} = $1; $self->{'type'} = 1; $self->{'url'} = $2; }
	else { $self->{'code'} = ""; $self->{'type'} = 0; $self->{'url'} = ""; }
	$rv .= "<tr $self->{'cb'}>\n";
	$rv .= "<td><input name=ErrorDocument_code_$i size=3 value=$self->{'code'}></td>\n";
	$rv .= "<td><input type=radio name=ErrorDocument_type_$i value=0 ".
	       ($self->{'type'}==0 ? "checked" : "").">$self->{'text_hash'}->{'core_tourl'}\n";
	$rv .= "<input type=radio name=ErrorDocument_type_$i value=1 ".
	       ($self->{'type'}==1 ? "checked" : "").">$self->{'text_hash'}->{'core_mesg'}</td>\n";
	$rv .= "<td><input name=ErrorDocument_url_$i size=40 value=\"$self->{'url'}\"></td>\n";
	$rv .= "</tr>\n";
	}
$rv .= "</table>\n";
return (2, $self->{'text_hash'}->{'core_custom'}, $rv);
}

sub save_ErrorDocument
{
	my $self = shift;

local($i, $code, $url, @rv);
for($i=0; defined($self->{'in_hash'}->{"ErrorDocument_code_$i"}); $i++) {
	$code = $self->{'in_hash'}->{"ErrorDocument_code_$i"}; $url = $self->{'in_hash'}->{"ErrorDocument_url_$i"};
	if ($code !~ /\S/ || $url !~ /\S/) { next; }
	$code =~ /^\d\d\d$/ || $self->error($self->text('core_eerror', $code));
	if ($self->{'in_hash'}->{"ErrorDocument_type_$i"} == 0) {
		$url =~ /^\S+$/ || $self->error($self->text('core_eurl', $url));
		push(@rv, "$code $url");
		}
	elsif ($_[0]->{'version'} >= 2.0) { push(@rv, "$code \"$url\""); }
	elsif ($_[0]->{'version'} >= 1.2) { push(@rv, "$code \"$url"); }
	else { push(@rv, "$code $url"); }
	}
return ( \@rv );
}

#########################################################################
# Logging directives
sub edit_ErrorLog
{
	my $self = shift;

if ($_[1]->{'version'} < 1.3) {
	return (1, $self->{'text_hash'}->{'core_errfile'},
		$self->opt_input($_[0]->{'value'}, "ErrorLog", $self->{'text_hash'}->{'core_default'}, 20).
		$self->file_chooser_button("ErrorLog", 0));
	}
else {
	local $v = $_[0]->{'words'}->[0];
	local $t = !$v ? 3 :
		   $v eq 'syslog' ? 2 :
		   $v =~ /^\|/ ? 1 : 0;
	$self->{'rv'} = sprintf "<input type=radio name=ErrorLog_type value=3 %s>\n",
		$t == 3 ? "checked" : "";
	$self->{'rv'} .= "$self->{'text_hash'}->{'core_default'}";
	$self->{'rv'} .= sprintf "<input type=radio name=ErrorLog_type value=2 %s>\n",
		$t == 2 ? "checked" : "";
	$self->{'rv'} .= "$self->{'text_hash'}->{'core_syslog'}<br>\n";
	$self->{'rv'} .= sprintf "<input type=radio name=ErrorLog_type value=0 %s>\n",
		$t == 0 ? "checked" : "";
	$self->{'rv'} .= sprintf "$self->{'text_hash'}->{'core_filelog'}<input name=ErrorLog_file size=25 value='%s'>\n",
		$t == 0 ? $v : "";
	$self->{'rv'} .= sprintf "<input type=radio name=ErrorLog_type value=1 %s>\n",
		$t == 1 ? "checked" : "";
	$self->{'rv'} .= sprintf "$self->{'text_hash'}->{'core_proglog'}<input name=ErrorLog_prog size=25 value='%s'><br>\n",
		$t == 1 ? substr($v, 1) : "";
	return (2, $self->{'text_hash'}->{'core_logto'}, $self->{'rv'});
	}
}

sub save_ErrorLog
{
	my $self = shift;

if ($_[0]->{'version'} < 1.3) {
	$self->{'in_hash'}->{'ErrorLog_def'} || $self->allowed_auth_file($self->{'in_hash'}->{'ErrorLog'}) ||
		$self->error($self->{'text_hash'}->{'core_edirlog'});
	$self->{'in_hash'}->{'ErrorLog_def'} || $self->directory_exists($self->{'in_hash'}->{'ErrorLog'}) ||
		    $self->error($self->{'text_hash'}->{'core_eerrordir'});
	return $self->parse_opt("ErrorLog", '^\S+$', $self->{'text_hash'}->{'core_efilelog'});
	}
else {
	if ($self->{'in_hash'}->{'ErrorLog_type'} == 3) {
		return ( [ ] );
		}
	elsif ($self->{'in_hash'}->{'ErrorLog_type'} == 0) {
		$self->{'in_hash'}->{'ErrorLog_file'} =~ /\S/ ||
		    $self->error($self->{'text_hash'}->{'core_efilemiss'});
		$self->allowed_auth_file($self->{'in_hash'}->{'ErrorLog_file'}) ||
		    $self->error($self->{'text_hash'}->{'core_edirlog'});
		$self->directory_exists($self->{'in_hash'}->{'ErrorLog_file'}) ||
		    $self->error($self->{'text_hash'}->{'core_eerrordir'});
		return ( [ $self->{'in_hash'}->{'ErrorLog_file'} ] );
		}
	elsif ($self->{'in_hash'}->{'ErrorLog_type'} == 1) {
		$self->{'in_hash'}->{'ErrorLog_prog'} =~ /\S/ ||
		    $self->error($self->{'text_hash'}->{'core_eprogmiss'});
		$self->{'access_hash'}->{'pipe'} ||
		    $self->error($self->{'text_hash'}->{'core_eperm'});
		return ( [ "\"|$self->{'in_hash'}->{'ErrorLog_prog'}\"" ] );
		}
	else {
		return ( [ "syslog" ] );
		}
	}
}

sub edit_LogLevel
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'core_loglevel'},
	$self->select_input($_[0]->{'value'}, "LogLevel", "",
		      "$self->{'text_hash'}->{'core_log_emerg'} (emerg),emerg",
		      "$self->{'text_hash'}->{'core_log_alert'} (alert),alert",
		      "$self->{'text_hash'}->{'core_log_crit'} (crit),crit",
		      "$self->{'text_hash'}->{'core_log_error'} (error),error",
		      "$self->{'text_hash'}->{'core_log_warn'} (warn),warn",
		      "$self->{'text_hash'}->{'core_log_notice'} (notice),notice",
		      "$self->{'text_hash'}->{'core_log_info'} (info),info",
		      "$self->{'text_hash'}->{'core_log_debug'} (debug),debug"));
}

sub save_LogLevel
{
	my $self = shift;

return $self->parse_select("LogLevel", "");
}

#########################################################################
# Module directives
sub edit_ClearModuleList_AddModule
{
	my $self = shift;

local($mods, @allmods, $d, %mods, $m, $i, $rv);
local $httpd = $self->find_httpd();
($self->{'ver'}, $mods) = $self->httpd_info($httpd);
@allmods = grep { !/^core$/ } @$mods;
local $conf = $self->get_config();
foreach $d ($self->find_directive_struct("LoadModule", $conf)) {
	# Add mod_ like modules
	if ($d->{'words'}->[1] =~ /(mod_\S+)\.(so|dll)/) {
		push(@allmods, $1);
		}
	# nodo50 v0.1 - Change 000002 - Bug fixed: Apache-ssl module detected as mod_ssl. Now Apache-ssl module included as mod_apachessl
	# nodo50 v0.1 - Change 000002 - Bug corregido: El modulo Apache-ssl se detecta como mod_ssl. Ahora Apache-ssl se incluye como mod_apachessl
	# Add apache-ssl libssl.so as mod_apachessl module
	elsif ($d->{'words'}->[1] =~ /libssl\.so/) {
		push(@allmods, "mod_apachessl");
		}
	# Add others lib* like as mod_ modules
	# nodo50 v0.1 - Change 000002 - End
	elsif ($d->{'words'}->[1] =~ /lib([^\/\s]+)\.(so|dll)/) {
		push(@allmods, "mod_$1");
		}
	}

if (@{$_[0]}) {
	# Only selected modules have been enabled
	foreach $d (@{$_[1]}) {
		local $modc = $d->{'value'};
		$modc =~ s/\.c$//;
		$mods{$modc} = "checked";
		}
	}
else { foreach $m (@allmods) { $mods{$m} = "checked"; } }
$rv = $self->choice_input(@{$_[0]} ? 1 : 0, "ClearModuleList", 1,
		    "$self->{'text_hash'}->{'core_allmod'},0", "$self->{'text_hash'}->{'core_selmod'},1")."<br>\n";
$rv .= "<table>\n";
foreach $m (@allmods) {
	if ($i%4 == 0) { $rv .= "<tr>\n"; }
	$rv .= "<td><input name=AddModule type=checkbox value=$m $mods{$m}> $m</td>\n";
	if ($i++%4 == 3) { $rv .= "</tr>\n"; }
	}
$rv .= "</table>\n";
return (2, $self->{'text_hash'}->{'core_actmod'}, $rv);
}

sub save_ClearModuleList_AddModule
{
	my $self = shift;

if ($self->{'in_hash'}->{'ClearModuleList'}) {
	local @mods = split(/\0/, $self->{'in_hash'}->{'AddModule'});
	return ( [ "" ], [ map { $_.".c" } @mods ] );
	}
else { return ( [ ], [ ] ); }
}


1;

