#!/usr/bin/perl
package apache::Mod_ssl;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'apache::Mod_ssl');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# mod_ssl.pl
# Defines editors for mod_ssl directives

@{$self->{'sslprotos_array'}} = ("SSLv2", "SSLv3", "TLSv1");

1;
		return $self;
}

sub mod_ssl_directives
{
	my $self = shift;

local($rv);
$rv = [ [ 'SSLEngine', 0, 14, 'virtual', undef, 11 ],
	[ 'SSLProtocol', 0, 14, 'virtual', undef, 11 ],
	[ 'SSLCertificateChainFile', 0, 14, 'virtual', undef, 10 ],
	[ 'SSLCertificateFile', 0, 14, 'virtual', undef, 9 ],
	[ 'SSLCertificateKeyFile', 0, 14, 'virtual', undef, 8 ],
	[ 'SSLPassPhraseDialog', 0, 14, 'virtual', 2.0, 7.5 ],
	[ 'SSLVerifyClient', 0, 14, 'virtual directory htaccess', undef, 7 ],
	[ 'SSLVerifyDepth', 0, 14, 'virtual directory htaccess', undef, 6 ],
	[ 'SSLLog', 0, 14, 'virtual', undef, 5 ],
	[ 'SSLRequireSSL', 0, 14, 'directory htaccess', undef, 4 ],
      ];
return $self->make_directives($rv, $_[0], "mod_ssl");
}

sub edit_SSLEngine
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_ssl_enable'},
	$self->choice_input($_[0]->{'value'}, "SSLEngine", "",
	      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off", "$self->{'text_hash'}->{'default'},"));
}

sub save_SSLEngine
{
	my $self = shift;

return $self->parse_choice("SSLEngine");
}

sub edit_SSLProtocol
{
	my $self = shift;

local ($rv, $p, %prot);
local @list = $_[0] ? @{$_[0]->{'words'}} : ("all");
foreach $p (@list) {
	if ($p =~ /^\+?all$/i) { map { $prot{lc($_)} = 1 } @{$self->{'sslprotos_array'}}; }
	elsif ($p =~ /^\-all$/i) { undef(%prot); }
	elsif ($p =~ /^\-(\S+)/) { $prot{lc($1)} = 0; }
	elsif ($p =~ /^\+(\S+)/) { $prot{lc($1)} = 1; }
	}
foreach $p (@{$self->{'sslprotos_array'}}) {
	$rv .= sprintf "<input type=checkbox name=SSLProtocol value=$p %s> $p ",
		$prot{lc($p)} ? "checked" : "";
	}
return (1, $self->{'text_hash'}->{'mod_ssl_proto'}, $rv);
}

sub save_SSLProtocol
{
	my $self = shift;

local @sel = split(/\0/, $self->{'in_hash'}->{'SSLProtocol'});
if (scalar(@sel) == scalar(@{$self->{'sslprotos_array'}})) { return ( [ ] ); }
return ( [ join(" ", (map { "+$_" } @sel)) ] );
}

sub edit_SSLCertificateChainFile
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_ssl_chfile'},
	$self->opt_input($_[0]->{'value'}, "SSLCertificateChainFile", $self->{'text_hash'}->{'mod_ssl_default'}, 35).
	$self->file_chooser_button("SSLCertificateChainFile", 0));
}

sub save_SSLCertificateChainFile
{
	my $self = shift;

return $self->parse_opt("SSLCertificateChainFile", '\S', $self->{'text_hash'}->{'mod_ssl_echfile'});
}

sub edit_SSLCertificateFile
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_ssl_cfile'},
	$self->opt_input($_[0]->{'value'}, "SSLCertificateFile", $self->{'text_hash'}->{'mod_ssl_default'}, 35).
	$self->file_chooser_button("SSLCertificateFile", 0));
}

sub save_SSLCertificateFile
{
	my $self = shift;

return $self->parse_opt("SSLCertificateFile", '\S', $self->{'text_hash'}->{'mod_ssl_ecfile'});
}

sub edit_SSLCertificateKeyFile
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_ssl_kfile'},
	$self->opt_input($_[0]->{'value'}, "SSLCertificateKeyFile", $self->{'text_hash'}->{'mod_ssl_default'}, 35).
	$self->file_chooser_button("SSLCertificateKeyFile", 0));
}

sub save_SSLCertificateKeyFile
{
	my $self = shift;

return $self->parse_opt("SSLCertificateKeyFile", '\S', $self->{'text_hash'}->{'mod_ssl_ekfile'});
}

sub edit_SSLVerifyClient
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_ssl_clcert'},
	$self->select_input($_[0]->{'value'}, "SSLVerifyClient", "",
		      "$self->{'text_hash'}->{'default'},", "$self->{'text_hash'}->{'mod_ssl_nreq'},none",
		      "$self->{'text_hash'}->{'mod_ssl_opt'},optional",
		      "$self->{'text_hash'}->{'mod_ssl_req'},require",
		      "$self->{'text_hash'}->{'mod_ssl_optca'},optional_no_ca"));
}

sub save_SSLVerifyClient
{
	my $self = shift;

return $self->parse_select("SSLVerifyClient");
}

sub edit_SSLVerifyDepth
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_ssl_cdepth'},
	$self->opt_input($_[0]->{'value'}, "SSLVerifyDepth", $self->{'text_hash'}->{'mod_ssl_default'}, 6));
}

sub save_SSLVerifyDepth
{
	my $self = shift;

return $self->parse_opt("SSLVerifyDepth", '^\d+$', $self->{'text_hash'}->{'mod_ssl_ecdepth'});
}

sub edit_SSLLog
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_ssl_log'},
	$self->opt_input($_[0]->{'value'}, "SSLLog", $self->{'text_hash'}->{'mod_ssl_default'}, 20));
}

sub save_SSLLog
{
	my $self = shift;

return $self->parse_opt("SSLLog", '\S', $self->{'text_hash'}->{'mod_ssl_elog'});
}

sub edit_SSLRequireSSL
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_ssl_onlyssl'},
	$self->choice_input($_[0] ? 1 : 0, "SSLRequireSSL", 0, "$self->{'text_hash'}->{'yes'},1", "$self->{'text_hash'}->{'no'},0"));
}

sub save_SSLRequireSSL
{
	my $self = shift;

return $self->{'in_hash'}->{'SSLRequireSSL'} ? ( [ "" ] ) : ( [ ] );
}

sub edit_SSLPassPhraseDialog
{
	my $self = shift;

local ($mode, $script, $pass, $file);
if ($_[0]->{'value'} eq 'builtin') {
	$mode = 1;
	}
elsif ($_[0]->{'value'} =~ /^exec:(.*)$/) {
	$file = $1;
	local $data = $self->read_file_contents($1);
	if ($data =~ /^#!\/bin\/sh\necho\s(.*)\n$/) {
		$pass = $1;
		$mode = 2;
		}
	else {
		$script = $file;
		$file = undef;
		$mode = 3;
		}
	}
elsif ($_[0]->{'value'}) {
	$script = $_[0]->{'value'};
	$mode = 1;
	}
else {
	$mode = 0;
	}
return (2, $self->{'text_hash'}->{'mod_ssl_pass'},
	$self->ui_radio("SSLPassPhraseDialog", $mode,
		[ [ 0, $self->{'text_hash'}->{'default'} ],
		  [ 1, $self->{'text_hash'}->{'mod_ssl_builtin'}."<br>" ],
		  [ 2, $self->text('mod_ssl_passph',
		     $self->ui_textbox("SSLPassPhraseDialog_pass", $pass, 20))."<br>" ],
		  [ 3, $self->text('mod_ssl_passsc', 
		     $self->ui_textbox("SSLPassPhraseDialog_script", $script, 40)) ],
		])."\n".
	$self->ui_hidden("SSLPassPhraseDialog_file", $file));
}

sub save_SSLPassPhraseDialog
{
	my $self = shift;

if ($self->{'in_hash'}->{'SSLPassPhraseDialog'} == 0) {
	return ( [ ] );
	}
elsif ($self->{'in_hash'}->{'SSLPassPhraseDialog'} == 1) {
	return ( [ "builtin" ] );
	}
elsif ($self->{'in_hash'}->{'SSLPassPhraseDialog'} == 2) {
	$self->{'in_hash'}->{'SSLPassPhraseDialog_pass'} =~ /\S/ ||
		$self->error($self->{'text_hash'}->{'mod_ssl_epassph'});
	local $file = $self->{'in_hash'}->{'SSLPassPhraseDialog_file'} ||
		"$self->{'config_hash'}->{'httpd_dir'}/passphrase.".time().".sh";
	$self->open_tempfile(PASS, ">$file");
	$self->print_tempfile(PASS, "#!/bin/sh\n");
	$self->print_tempfile(PASS, "echo ",$self->{'in_hash'}->{'SSLPassPhraseDialog_pass'},"\n");
	$self->close_tempfile(PASS);
	$self->set_ownership_permissions(undef, undef, 0755, $file);
	return ( [ "exec:$file" ] );
	}
elsif ($self->{'in_hash'}->{'SSLPassPhraseDialog'} == 3) {
	if ($self->{'in_hash'}->{'SSLPassPhraseDialog_script'} =~ /^[a-z]+:/) {
		return ( [ $self->{'in_hash'}->{'SSLPassPhraseDialog_script'} ] );
		}
	else {
		$self->{'in_hash'}->{'SSLPassPhraseDialog_script'} =~ /^\/\S/ ||
			$self->error($self->{'text_hash'}->{'mod_ssl_epasssc'});
		return ( [ "exec:".$self->{'in_hash'}->{'SSLPassPhraseDialog_script'} ] );
		}
	}
}


1;

