#!/usr/bin/perl
package apache::Perchild;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'apache::Perchild');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# perchild.pl
# Defines editors for the per-child UID module in apache 2.0
# The actual functions for most of these are still in core.pl

1;
		return $self;
}

sub perchild_directives
{
	my $self = shift;

local $rv;
$rv = [ [ 'AssignUserId', 0, 8, 'virtual virtualonly', 2.0 ],
	[ 'ChildPerUserId', 1, 8, 'global', 2.0 ],
	[ 'CoreDumpDirectory', 0, 9, 'global', 2.0 ],
	[ 'BindAddress Listen Port', 1, 1, 'global', 2.0, 10 ],
	[ 'ListenBacklog', 0, 1, 'global', 2.0 ],
	[ 'LockFile', 0, 9, 'global', 2.0 ],
	[ 'MaxRequestsPerChild', 0, 0, 'global', 2.0 ],
	[ 'MinSpareThreads', 0, 0, 'global', 2.0 ],
	[ 'MaxSpareThreads', 0, 0, 'global', 2.0 ],
	[ 'MaxThreadsPerChild', 0, 0, 'global', 2.0 ],
	[ 'NumServers', 0, 0, 'global', 2.0 ],
	[ 'PidFile', 0, 9, 'global', 2.0 ],
	[ 'ScoreBoardFile', 0, 9, 'global', 2.0 ],
	[ 'SendBufferSize', 0, 1, 'global', 2.0 ],
	[ 'StartThreads', 0, 0, 'global', 2.0 ],
	[ 'Group', 0, 8, 'global', 2.0, 9 ],
	[ 'User', 0, 8, 'global', 2.0, 10 ] ];
return $self->make_directives($rv, $_[0], "perchild");
}

sub edit_AssignUserId
{
	my $self = shift;

local $rv;
$rv .= sprintf "<input type=radio name=AssignUserId_def value=1 %s> %s\n",
		$_[0] ? "" : "checked", $self->{'text_hash'}->{'core_none'};
$rv .= sprintf "<input type=radio name=AssignUserId_def value=0 %s>\n",
		$_[0] ? "checked" : "";
$rv .= $self->text('perchild_assignug',
	"<input name=AssignUserId_uid size=8 value='$_[0]->{'words'}->[0]'>",
	"<input name=AssignUserId_gid size=8 value='$_[0]->{'words'}->[1]'>");
return (2, $self->{'text_hash'}->{'perchild_assign'}, $rv);
}

sub save_AssignUserId
{
	my $self = shift;

if ($self->{'in_hash'}->{'AssignUserId_def'}) {
	return ( [ ] );
	}
else {
	$self->{'in_hash'}->{'AssignUserId_uid'} =~ /^-?\d+$/ || $self->error($self->{'text_hash'}->{'perchild_euid'});
	$self->{'in_hash'}->{'AssignUserId_gid'} =~ /^-?\d+$/ || $self->error($self->{'text_hash'}->{'perchild_egid'});
	return ( [ "$self->{'in_hash'}->{'AssignUserId_uid'} $self->{'in_hash'}->{'AssignUserId_gid'}" ] );
	}
}

sub edit_ChildPerUserId
{
	my $self = shift;

local $rv = "<table border>\n".
	    "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'perchild_num'}</b></td>\n".
	    "<td><b>$self->{'text_hash'}->{'perchild_uid'}</b></td>\n".
	    "<td><b>$self->{'text_hash'}->{'perchild_gid'}</b></td> </tr>\n";
local ($c, $i = 0);
foreach $c (@{$_[0]}, undef) {
	local @v = $c ? @{$c->{'words'}} : ();
	$rv .= "<tr $self->{'cb'}>\n";
	$rv .= "<td><input name=ChildPerUserId_n_$i size=5 value='$v[2]'></td>\n";
	$rv .= "<td><input name=ChildPerUserId_u_$i size=8 value='$v[0]'></td>\n";
	$rv .= "<td><input name=ChildPerUserId_g_$i size=8 value='$v[1]'></td>\n";
	$rv .= "</tr>\n";
	$i++;
	}
$rv .= "</table>\n";
return (2, $self->{'text_hash'}->{'perchild_child'}, $rv);
}

sub save_ChildPerUserId
{
	my $self = shift;

local (@rv, $i);
for($i=0; defined($self->{'in_hash'}->{"ChildPerUserId_n_$i"}); $i++) {
	next if (!$self->{'in_hash'}->{"ChildPerUserId_n_$i"});
	$self->{'in_hash'}->{"ChildPerUserId_n_$i"} =~ /^[1-9]\d*$/ ||
		$self->error($self->{'text_hash'}->{'perchild_enum'});
	$self->{'in_hash'}->{"ChildPerUserId_u_$i"} =~ /^-?\d+$/ ||
		$self->error($self->{'text_hash'}->{'perchild_euid'});
	$self->{'in_hash'}->{"ChildPerUserId_g_$i"} =~ /^-?\d+$/ ||
		$self->error($self->{'text_hash'}->{'perchild_egid'});
	push(@rv, $self->{'in_hash'}->{"ChildPerUserId_u_$i"}." ".$self->{'in_hash'}->{"ChildPerUserId_g_$i"}." ".
		  $self->{'in_hash'}->{"ChildPerUserId_n_$i"});
	}
return ( \@rv );
}

sub edit_MinSpareThreads
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'worker_minspare'},
	$self->opt_input($_[0]->{'value'},"MinSpareThreads",$self->{'text_hash'}->{'default'}, 4));
}

sub save_MinSpareThreads
{
	my $self = shift;

return $self->parse_opt("MinSpareThreads", '^\d+$',
		  $self->{'text_hash'}->{'worker_eminspare'});
}

sub edit_MaxSpareThreads
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'worker_maxspare'},
	$self->opt_input($_[0]->{'value'},"MaxSpareThreads",$self->{'text_hash'}->{'default'}, 4));
}

sub save_MaxSpareThreads
{
	my $self = shift;

return $self->parse_opt("MaxSpareThreads", '^\d+$',
		  $self->{'text_hash'}->{'worker_emaxspare'});
}

sub edit_StartThreads
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'perchild_sthreads'},
	$self->opt_input($_[0]->{'value'},"StartThreads",$self->{'text_hash'}->{'default'}, 4));
}

sub save_StartThreads
{
	my $self = shift;

return $self->parse_opt("StartThreads", '^\d+$',
		  $self->{'text_hash'}->{'perchild_esthreads'});
}

sub edit_NumServers
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'perchild_numservers'},
	$self->opt_input($_[0]->{'value'},"NumServers",$self->{'text_hash'}->{'default'}, 4));
}

sub save_NumServers
{
	my $self = shift;

return $self->parse_opt("NumServers", '^\d+$',
		  $self->{'text_hash'}->{'perchild_enumservers'});
}

sub edit_MaxThreadsPerChild
{
	my $self = shift;

return (1,
	$self->{'text_hash'}->{'perchild_maxthreads'},
	$self->opt_input($_[0]->{'value'},"MaxThreadsPerChild",$self->{'text_hash'}->{'default'}, 4));
}

sub save_MaxThreadsPerChild
{
	my $self = shift;

return $self->parse_opt("MaxThreadsPerChild", '^\d+$',
		  $self->{'text_hash'}->{'perchild_emaxthreads'});
}


1;

