#!/usr/bin/perl
package bandwidth::BandwidthLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'bandwidth::BandwidthLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for parsing the system log for iptables entries
# XXX ipf support
#	XXX ipf missing some packets? large FTP transfer?
# XXX option to specify ports considered 'server' (by ranges)
#	XXX option to make ports without names not server ports
#	XXX use is_server_port function

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
if ($self->foreign_installed("syslog_ng")) {
	$self->foreign_require("syslog_ng", "syslog-ng-lib.pl");
	$self->{'syslog_module'} = "syslog_ng";
	}
elsif ($self->foreign_installed("syslog")) {
	$self->foreign_require("syslog", "syslog-lib.pl");
	$self->{'syslog_module'} = "syslog";
	}
else {
	$self->{'syslog_module'} = undef;
	}
$self->foreign_require("cron", "cron-lib.pl");
$self->foreign_require("net", "net-lib.pl");

%{$self->{'access_hash'}} = $self->get_module_acl();

$self->{'bandwidth_log'} = $self->{'config_hash'}->{'bandwidth_log'} || "/var/log/bandwidth";
$self->{'hours_dir'} = $self->{'config_hash'}->{'bandwidth_dir'} || "$self->{'module_config_directory'}/hours";
$self->{'cron_cmd'} = "$self->{'module_config_directory'}/rotate.pl";
$self->{'pid_file'} = "$self->{'module_config_directory'}/rotate.pid";

############### functions for IPtables #################

############### functions for ipfw #################

############### functions for Shorewall #################

############### functions for ipfilter #################

1;

1;
		return $self;
}

# list_hours()
# Returns a list of all hours for which traffic is available
sub list_hours
{
	my $self = shift;

opendir(DIR, $self->{'hours_dir'});
local @rv = grep { $_ =~ /^\d+$/ } readdir(DIR);
closedir(DIR);
return @rv;
}

# get_hour(num)
sub get_hour
{
	my $self = shift;

if (!defined($self->{'hour_cache_hash'}->{$_[0]})) {
	local $file = "$self->{'hours_dir'}/$_[0]";
	local %hour;
	if ($self->read_file($file, \%hour)) {
		$self->{'hour_cache_hash'}->{$_[0]} = \%hour;
		}
	else {
		$self->{'hour_cache_hash'}->{$_[0]} = { 'hour' => $_[0] };
		}
	}
return $self->{'hour_cache_hash'}->{$_[0]};
}

# save_hour(hour)
sub save_hour
{
	my $self = shift;

mkdir($self->{'hours_dir'}, 0755);
local $file = "$self->{'hours_dir'}/$_[0]->{'hour'}";
$self->write_file($file, $_[0]);
}

# find_rule(&table, chain, interface, dir)
sub find_rule
{
	my $self = shift;

local ($table, $chain, $iface, $dir) = @_;
local $r;
local $dd = $dir eq "i" ? "IN" : "OUT";
foreach $r (@{$table->{'rules'}}) {
	next if ($r->{'chain'} ne $chain);
	local $da = $r->{$dir};
	if ($iface) {
		next if ($da->[1] ne $iface);
		}
	else {
		next if (!$da);
		}
	next if ($r->{'j'}->[1] ne 'LOG');
	next if ($r->{'args'} !~ /\-\-log\-prefix\s+BANDWIDTH_\Q$dd\E:/ &&
		 $r->{'args'} !~ /\-\-log\-prefix\s+"BANDWIDTH_\Q$dd\E:"/ &&
		 $r->{'logprefix'}->[1] ne "BANDWIDTH_$dd");
	return $r;
	}
return undef;
}

# find_sysconf(&conf)
# Returns the syslog entry for kernel debug messages to the log file
sub find_sysconf
{
	my $self = shift;

local ($conf) = @_;
local $c;
foreach $c (@$conf) {
	next if (!$c->{'active'});
	next if ($c->{'file'} ne $self->{'bandwidth_log'});
	next if ($c->{'sel'}->[0] ne $self->get_loglevel());
	return $c;
	}
return undef;
}

# find_sysconf_ng(&conf)
# Returns the destination, filter and log objects for kernel debug messages
sub find_sysconf_ng
{
	my $self = shift;

local ($conf) = @_;
local @dests = $main::syslog_ng_object->find("destination", $conf);
local ($dest) = grep { $_->{'value'} eq "d_bandwidth" } @dests;
local @filters = $main::syslog_ng_object->find("filter", $conf);
local ($filter) = grep { $_->{'value'} eq "f_bandwidth" } @filters;
local @logs = $main::syslog_ng_object->find("log", $conf);
local $log;
if ($dest && $filter) {
	foreach my $l (@logs) {
		local ($ldest) = $main::syslog_ng_object->find("destination",
						  $l->{'members'});
		local ($lfilter) = $main::syslog_ng_object->find("filter",
						    $l->{'members'});
		if ($ldest->{'value'} eq $dest->{'value'} &&
		    $lfilter->{'value'} eq $filter->{'value'}) {
			$log = $l;
			last;
			}
		}
	}
return ($dest, $filter, $log);
}

# find_cron_job()
# Returns the cron job used for bandwidth counting, or undef
sub find_cron_job
{
	my $self = shift;

local @jobs = $main::cron_object->list_cron_jobs();
local ($job) = grep { $_->{'user'} eq 'root' &&
		      $_->{'command'} eq $self->{'cron_cmd'} &&
		      $_->{'active'} } @jobs;
return $job;
}

# date_input(day, month, year, prefix)
sub date_input
{
	my $self = shift;

return $self->ui_textbox("$_[3]_day", $_[0], 2)."/".
       $self->ui_select("$_[3]_month", $_[1],
		  [ map { [ $_, $self->{'text_hash'}->{"smonth_".$_} ] } (1 .. 12) ])."/". 
       $self->ui_textbox("$_[3]_year", $_[2], 4).
       $self->date_chooser_button("$_[3]_day", "$_[3]_month", "$_[3]_year");
}

# hourmin_input(hour, min, prefix)
sub hourmin_input
{
	my $self = shift;

return $self->ui_textbox("$_[2]_hour", $_[0], 2).":".
       $self->ui_textbox("$_[2]_min", $_[1], 2);
}

# detect_firewall_system()
# Guesses which firewall is installed
sub detect_firewall_system
{
	my $self = shift;

local $m;
foreach $m ("shorewall", "firewall", "ipfw", "ipfilter") {
	return $m if ($self->check_firewall_system($m));
	}
return undef;
}

# check_firewall_system(type)
# Returns 1 if some firewall is installed
sub check_firewall_system
{
	my $self = shift;

# XXX shorewall check should look for actual rules?
return $self->foreign_installed($_[0], 1);
}

sub check_rules
{
	my $self = shift;

local $cfunc = "check_".$self->{'config_hash'}->{'firewall_system'}."_rules";
return $self->$cfunc(@_);
}

sub setup_rules
{
	my $self = shift;

local $sfunc = "setup_".$self->{'config_hash'}->{'firewall_system'}."_rules";
return $self->$sfunc(@_);
}

sub delete_rules
{
	my $self = shift;

local $dfunc = "delete_".$self->{'config_hash'}->{'firewall_system'}."_rules";
return $self->$dfunc(@_);
}

sub process_line
{
	my $self = shift;

local $pfunc = "process_".$self->{'config_hash'}->{'firewall_system'}."_line";
return $self->$pfunc(@_);
}

sub pre_process
{
	my $self = shift;

local $pfunc = "pre_".$self->{'config_hash'}->{'firewall_system'}."_process";
if ($self->can($pfunc)) {
	$self->$pfunc(@_);
	}
}

sub get_loglevel
{
	my $self = shift;

local $lfunc = "get_".$self->{'config_hash'}->{'firewall_system'}."_loglevel";
return $self->$lfunc(@_);
}

# is_server_port(num)
sub is_server_port
{
	my $self = shift;

}

# check_firewall_rules()
# Returns 1 if the IPtables rules needed are setup, 0 if not
sub check_firewall_rules
{
	my $self = shift;

$self->foreign_require("firewall", "firewall-lib.pl");
local @tables = $main::firewall_object->get_iptables_save();
local ($filter) = grep { $_->{'name'} eq 'filter' } @tables;

local $inrule = $self->find_rule($filter, "INPUT", $self->{'config_hash'}->{'iface'}, "i");
local $outrule = $self->find_rule($filter, "OUTPUT", $self->{'config_hash'}->{'iface'}, "o");
local $fwdinrule = $self->find_rule($filter, "FORWARD", $self->{'config_hash'}->{'iface'}, "i");
local $fwdoutrule = $self->find_rule($filter, "FORWARD", $self->{'config_hash'}->{'iface'}, "o");
return $inrule && $outrule && $fwdinrule && $fwdoutrule;
}

# setup_firewall_rules(iface)
# If any IPtables rules are missing, add them
sub setup_firewall_rules
{
	my $self = shift;

$self->foreign_require("firewall", "firewall-lib.pl");
local @tables = $main::firewall_object->get_iptables_save();
local ($filter) = grep { $_->{'name'} eq 'filter' } @tables;
$filter ||= { 'name' => 'filter',
	      'defaults' => { 'INPUT' => 'ACCEPT',
			      'OUTPUT' => 'ACCEPT',
			      'FORWARD' => 'ACCEPT' },
	      'rules' => [ ],
	    };
local $inrule = $self->find_rule($filter, "INPUT", $self->{'config_hash'}->{'iface'}, "i");
local $outrule = $self->find_rule($filter, "OUTPUT", $self->{'config_hash'}->{'iface'}, "o");
local $fwdinrule = $self->find_rule($filter, "FORWARD", $self->{'config_hash'}->{'iface'}, "i");
local $fwdoutrule = $self->find_rule($filter, "FORWARD", $self->{'config_hash'}->{'iface'}, "o");
local $missingrule = !$inrule || !$outrule || !$fwdinrule || !$fwdoutrule;
if (!$inrule) {
	splice(@{$filter->{'rules'}}, 0, 0,
	       { 'chain' => 'INPUT',
		 'j' => [ undef, 'LOG' ],
		 'i' => [ undef, $_[0] ],
		 'args' => "--log-level 7 --log-prefix BANDWIDTH_IN:" });
	}
if (!$outrule) {
	splice(@{$filter->{'rules'}}, 0, 0,
	       { 'chain' => 'OUTPUT',
		 'j' => [ undef, 'LOG' ],
		 'o' => [ undef, $_[0] ],
		 'args' => "--log-level 7 --log-prefix BANDWIDTH_OUT:" });
	}
if (!$fwdinrule) {
	splice(@{$filter->{'rules'}}, 0, 0,
	       { 'chain' => 'FORWARD',
		 'j' => [ undef, 'LOG' ],
		 'i' => [ undef, $_[0] ],
		 'args' => "--log-level 7 --log-prefix BANDWIDTH_IN:" });
	}
if (!$fwdoutrule) {
	splice(@{$filter->{'rules'}}, 0, 0,
	       { 'chain' => 'FORWARD',
		 'j' => [ undef, 'LOG' ],
		 'o' => [ undef, $_[0] ],
		 'args' => "--log-level 7 --log-prefix BANDWIDTH_OUT:" });
	}

if ($missingrule) {
	# Save and apply
	$self->lock_file($main::firewall_object->{'iptables_save_file'});
	$main::firewall_object->run_before_command();
	$main::firewall_object->save_table($filter);
	$main::firewall_object->run_after_command();
	$self->unlock_file($main::firewall_object->{'iptables_save_file'});
	return $main::firewall_object->apply_configuration();
	}
return undef;
}

# delete_firewall_rules()
# Delete firewall rules for bandwidth logging
sub delete_firewall_rules
{
	my $self = shift;

$self->foreign_require("firewall", "firewall-lib.pl");
local @tables = $main::firewall_object->get_iptables_save();
local ($filter) = grep { $_->{'name'} eq 'filter' } @tables;
local $inrule = $self->find_rule($filter, "INPUT", $self->{'config_hash'}->{'iface'}, "i");
local $outrule = $self->find_rule($filter, "OUTPUT", $self->{'config_hash'}->{'iface'}, "o");
local $fwdinrule = $self->find_rule($filter, "FORWARD", $self->{'config_hash'}->{'iface'}, "i");
local $fwdoutrule = $self->find_rule($filter, "FORWARD", $self->{'config_hash'}->{'iface'}, "o");
local $anyrule = $inrule || $outrule || $fwdinrule || $fwdoutrule;
if ($anyrule) {
	@{$filter->{'rules'}} = grep { $_ ne $inrule &&
				       $_ ne $outrule &&
				       $_ ne $fwdinrule &&
				       $_ ne $fwdoutrule }@{$filter->{'rules'}};

	# Save and apply firewall
	$self->lock_file($main::firewall_object->{'iptables_save_file'});
	$main::firewall_object->run_before_command();
	$main::firewall_object->save_table($filter);
	$main::firewall_object->run_after_command();
	$self->unlock_file($main::firewall_object->{'iptables_save_file'});
	return $main::firewall_object->apply_configuration();
	}
return undef;
}

# process_firewall_line(line, &hours, time-now)
# Process an IPtables firewall line, and returns 1 if successful
sub process_firewall_line
{
	my $self = shift;

local ($line, $hours, $time_now) = @_;
if ($line =~ /^(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+).*BANDWIDTH_(IN|OUT):(IN=.*)/) {
	# Found a valid line
	local ($mon, $day, $hr, $min, $sec) = ($1, $2, $3, $4, $5);
	local $dir = lc($6);
	local %line;
	local $w;
	foreach $w (split(/\s+/, $7)) {
		($self->{'n'}, $self->{'v'}) = split(/=/, $w);
		if ($self->{'n'}) {
			$line{lc($self->{'n'})} = $self->{'v'};
			}
		}

	# Work out the real time
	local $tm = timelocal($sec, $min, $hr, $day,
			      $self->month_to_number($mon), $self->{'time_now_array'}->[5]);
	if ($tm > $time_now + 24*60*60) {
		# Was really last year
		$tm = timelocal($sec, $min, $hr, $day,
				$self->month_to_number($mon), $self->{'time_now_array'}->[5]-1);
		}
	local $htm = int($tm/(60*60));

	# Update the appropriate counters
	local $hour = $self->get_hour($htm);
	if ($self->indexof($hour, @$hours) < 0) {
		push(@$hours, $hour);
		}
	local $port;
	if ($line{'proto'} eq 'TCP' || $line{'proto'} eq 'UDP') {
		if ($dir eq "in") {
			$port = '_'.$line{'dpt'}.'_'.$line{'spt'};
			}
		else {
			$port = '_'.$line{'spt'}.'_'.$line{'dpt'};
			}
		}
	local $host = $dir eq "in" ? $line{'dst'} : $line{'src'};
	local $key = $host.'_'.$line{'proto'}.$port;
	local ($in, $out) = split(/ /, $hour->{$key});
	if ($dir eq "in") {
		$in += $line{'len'};
		}
	else {
		$out += $line{'len'};
		}
	$hour->{$key} = int($in)." ".int($out);
	return 1;
	}
else {
	return 0;
	}
}

# get_firewall_loglevel()
sub get_firewall_loglevel
{
	my $self = shift;

return "kern.=debug";
}

sub check_ipfw_rules
{
	my $self = shift;

$self->foreign_require("ipfw", "ipfw-lib.pl");
local $rules = $main::ipfw_object->get_config();
local $rule = $self->find_ipfw_rule($rules, $self->{'config_hash'}->{'iface'});
return $rule ? 1 : 0;
}

# setup_ipfw_rules(iface)
# Add the logging rule used for ipfw
sub setup_ipfw_rules
{
	my $self = shift;

$self->foreign_require("ipfw", "ipfw-lib.pl");
local $rules = $main::ipfw_object->get_config();
local $rule = $self->find_ipfw_rule($rules, $self->{'config_hash'}->{'iface'});
if (!$rule) {
	# Add the rule
	local $num = 100;
	if (@$rules && $rules->[0]->{'num'} < 100) {
		$num = int($rules->[0]->{'num'} / 2);
		}
	$rule = { 'action' => 'allow',
		  'log' => 1,
		  'proto' => 'all',
		  'from' => 'any',
		  'to' => 'any',
		  'num' => $num,
		  'via' => $_[0] };
	splice(@$rules, 0, 0, $rule);
	$main::ipfw_object->save_config($rules);

	# Apply config
	return $main::ipfw_object->apply_rules($rules);
	}
return undef;
}

# delete_ipfw_rules()
# Delete the logging rule used for ipfw
sub delete_ipfw_rules
{
	my $self = shift;

$self->foreign_require("ipfw", "ipfw-lib.pl");
local $rules = $main::ipfw_object->get_config();
local $rule = $self->find_ipfw_rule($rules, $self->{'config_hash'}->{'iface'});
if ($rule) {
	@$rules = grep { $_ ne $rule } @$rules;
	$main::ipfw_object->save_config($rules);
	return $main::ipfw_object->apply_rules();
	}
return undef;
}

# find_ipfw_rule(&rules, iface)
sub find_ipfw_rule
{
	my $self = shift;

local ($rule) = grep { $_->{'action'} eq 'allow' &&
		       $_->{'log'} &&
		       ($_->{'proto'} eq 'all' || $_->{'proto'} eq 'ip') &&
		       $_->{'from'} eq 'any' &&
		       $_->{'to'} eq 'any' &&
		       $_->{'via'} eq $_[1] } @{$_[0]};
return $rule;
}

# pre_ipfw_process()
# Called before processing of the logs, to get the average packet size
sub pre_ipfw_process
{
	my $self = shift;

$self->foreign_require("ipfw", "ipfw-lib.pl");
local $active = $main::ipfw_object->get_config("$main::ipfw_object->{'config_hash'}->{'ipfw'} show |", \$self->{'out'});
local $rule = $self->find_ipfw_rule($active, $self->{'config_hash'}->{'iface'});
if ($rule) {
	$self->{'average_packet_size'} = $rule->{'count2'} / $rule->{'count1'};
	system("$main::ipfw_object->{'config_hash'}->{'ipfw'} zero $rule->{'num'} >/dev/null 2>&1");
	}
else {
	$self->{'average_packet_size'} = 1;
	}
}

# process_ipfw_line(line, &hours, time-now)
# Process a BSD IPFW firewall line, and returns 1 if successful
sub process_ipfw_line
{
	my $self = shift;

local ($line, $hours, $time_now) = @_;
if ($line =~ /^(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+).*ipfw:\s+\S+\s+Accept\s+(\S+)\s+(\S+)\s+(\S+)\s+(in|out)\s+via\s+(\S+)/) {
	# Found a valid line
	local ($mon, $day, $hr, $min, $sec) = ($1, $2, $3, $4, $5);
	local ($proto, $src, $dest, $dir, $iface) = ($6, $7, $8, $9, $10);
	local ($srchost, $srcport) = split(/:/, $src);
	local ($desthost, $destport) = split(/:/, $dest);
	$proto =~ s/:.*//;
	return undef if ($iface ne $self->{'config_hash'}->{'iface'});

	# Work out the real time
	local $tm = timelocal($sec, $min, $hr, $day,
			      $self->month_to_number($mon), $self->{'time_now_array'}->[5]);
	if ($tm > $time_now + 24*60*60) {
		# Was really last year
		$tm = timelocal($sec, $min, $hr, $day,
				$self->month_to_number($mon), $self->{'time_now_array'}->[5]-1);
		}
	local $htm = int($tm/(60*60));

	# Update the appropriate counters
	local $hour = $self->get_hour($htm);
	if ($self->indexof($hour, @$hours) < 0) {
		push(@$hours, $hour);
		}
	local $port;
	if ($proto eq 'TCP' || $proto eq 'UDP') {
		if ($dir eq "in") {
			$port = '_'.$destport.'_'.$srcport;
			}
		else {
			$port = '_'.$srcport.'_'.$destport;
			}
		}
	local $host = $dir eq "in" ? $dest : $src;
	local $key = $host.'_'.$proto.$port;
	local ($in, $out) = split(/ /, $hour->{$key});
	if ($dir eq "in") {
		$in += $self->{'average_packet_size'};
		}
	else {
		$out += $self->{'average_packet_size'};
		}
	$hour->{$key} = int($in)." ".int($out);
	return 1;
	}
else {
	return 0;
	}
}

# get_ipfw_loglevel()
sub get_ipfw_loglevel
{
	my $self = shift;

return "security.*";
}

sub check_shorewall_rules
{
	my $self = shift;

$self->foreign_require("shorewall", "shorewall-lib.pl");
local $lref = $self->read_file_lines("$main::shorewall_object->{'config_hash'}->{'config_dir'}/start");
local $rule1 = $self->find_shorewall_rule($lref, "INPUT", "-i", $self->{'config_hash'}->{'iface'});
local $rule2 = $self->find_shorewall_rule($lref, "FORWARD", "-i", $self->{'config_hash'}->{'iface'});
local $rule3 = $self->find_shorewall_rule($lref, "FORWARD", "-o", $self->{'config_hash'}->{'iface'});
local $rule4 = $self->find_shorewall_rule($lref, "OUTPUT", "-o", $self->{'config_hash'}->{'iface'});
return $rule1 && $rule2 && $rule3 && $rule4;
}

# setup_shorewall_rules(iface)
# Add lines to the Shorewall start script to add logging IPtables rules
sub setup_shorewall_rules
{
	my $self = shift;

$self->foreign_require("shorewall", "shorewall-lib.pl");
local $lref = $self->read_file_lines("$main::shorewall_object->{'config_hash'}->{'config_dir'}/start");
local $rule1 = $self->find_shorewall_rule($lref, "INPUT", "-i", $_[0]);
local $rule2 = $self->find_shorewall_rule($lref, "FORWARD", "-i", $_[0]);
local $rule3 = $self->find_shorewall_rule($lref, "FORWARD", "-o", $_[0]);
local $rule4 = $self->find_shorewall_rule($lref, "OUTPUT", "-o", $_[0]);
local $gotall = $rule1 && $rule2 && $rule3 && $rule4;
if (!$rule1) {
	push(@$lref, "run_iptables -I INPUT -i $_[0] -j LOG --log-prefix BANDWIDTH_IN: --log-level debug");
	}
if (!$rule2) {
	push(@$lref, "run_iptables -I FORWARD -i $_[0] -j LOG --log-prefix BANDWIDTH_IN: --log-level debug");
	}
if (!$rule3) {
	push(@$lref, "run_iptables -I FORWARD -o $_[0] -j LOG --log-prefix BANDWIDTH_OUT: --log-level debug");
	}
if (!$rule4) {
	push(@$lref, "run_iptables -I OUTPUT -o $_[0] -j LOG --log-prefix BANDWIDTH_OUT: --log-level debug");
	}
$self->flush_file_lines();

if (!$gotall) {
	# Apply config with a shorewall restart
	$main::shorewall_object->run_before_apply_command();
	local $out = $self->backquote_logged("$main::shorewall_object->{'config_hash'}->{'shorewall'} restart 2>&1");
	return "<pre>$out</pre>" if ($?);
	$main::shorewall_object->run_after_apply_command();
	}
return undef;
}

sub delete_shorewall_rules
{
	my $self = shift;

$self->foreign_require("shorewall", "shorewall-lib.pl");
local $lref = $self->read_file_lines("$main::shorewall_object->{'config_hash'}->{'config_dir'}/start");
local $rule1 = $self->find_shorewall_rule($lref, "INPUT", "-i", $self->{'config_hash'}->{'iface'});
splice(@$lref, $rule1, 1) if ($rule1);
local $rule2 = $self->find_shorewall_rule($lref, "FORWARD", "-i", $self->{'config_hash'}->{'iface'});
splice(@$lref, $rule2, 1) if ($rule2);
local $rule3 = $self->find_shorewall_rule($lref, "FORWARD", "-o", $self->{'config_hash'}->{'iface'});
splice(@$lref, $rule3, 1) if ($rule3);
local $rule4 = $self->find_shorewall_rule($lref, "OUTPUT", "-o", $self->{'config_hash'}->{'iface'});
splice(@$lref, $rule4, 1) if ($rule4);
$self->flush_file_lines();

if ($rule1 || $rule2 || $rule3 || $rule4) {
	# Apply config with a shorewall restart
	$main::shorewall_object->run_before_apply_command();
	local $out = $self->backquote_logged("$main::shorewall_object->{'config_hash'}->{'shorewall'} restart 2>&1");
	return "<pre>$out</pre>" if ($?);
	$main::shorewall_object->run_after_apply_command();
	}
return undef;
}

# find_shorewall_rule(&lref, chain, dir, iface)
# Returns the line indexes of the shorewall start script entries that add
# the extra logging IPtables rules
sub find_shorewall_rule
{
	my $self = shift;

local ($lref, $chain, $dir, $iface) = @_;
local $io = $dir eq "-i" ? "BANDWIDTH_IN:" : "BANDWIDTH_OUT:";
local (@rv, $i);
for($i=0; $i<@$lref; $i++) {
	if ($lref->[$i] =~ /^run_iptables\s+-I\s+$chain\s+$dir\s+$iface\s+-j\s+LOG\s+--log-prefix\s+$io\s+--log-level\s+debug/) {
		return $i;
		}
	}
return undef;
}

# get_shorewall_loglevel()
sub get_shorewall_loglevel
{
	my $self = shift;

return "kern.=debug";
}

sub process_shorewall_line
{
	my $self = shift;

return $self->process_firewall_line(@_);
}

sub check_ipfilter_rules
{
	my $self = shift;

$self->foreign_require("ipfilter", "ipfilter-lib.pl");
local $rules = $main::ipfilter_object->get_config();
local $rule1 = $self->find_ipfilter_rule($rules, $self->{'config_hash'}->{'iface'}, "in");
local $rule2 = $self->find_ipfilter_rule($rules, $self->{'config_hash'}->{'iface'}, "out");
return $rule1 && $rule2;
}

# setup_ipfilter_rules(iface)
# Add the logging rule used for ipfilter
sub setup_ipfilter_rules
{
	my $self = shift;

$self->foreign_require("ipfilter", "ipfilter-lib.pl");
local $rules = $main::ipfilter_object->get_config();
local $rule1 = $self->find_ipfilter_rule($rules, $_[0], "in");
local $rule2 = $self->find_ipfilter_rule($rules, $_[0], "out");
local $gotall = $rule1 && $rule2;
if (!$rule1) {
	$rule1 = { 'action' => 'log',
		   'on' => $_[0],
		   'log-level' => 'local7.debug',
		   'dir' => 'in',
		   'all' => 1,
		   'active' => 1 };
	if (@$rules) {
		$main::ipfilter_object->insert_rule($rule1, $rules->[0]);
		}
	else {
		$main::ipfilter_object->create_rule($rule1);
		}
	}
if (!$rule2) {
	$rule2 = { 'action' => 'log',
		   'on' => $_[0],
		   'log-level' => 'local7.debug',
		   'dir' => 'out',
		   'all' => 1,
		   'active' => 1 };
	if (@$rules) {
		$main::ipfilter_object->insert_rule($rule2, $rules->[0]);
		}
	else {
		$main::ipfilter_object->create_rule($rule2);
		}
	}

if (!$gotall) {
	# Apply config
	return $main::ipfilter_object->apply_configuration();
	}
return undef;
}

# delete_ipfilter_rules()
# Delete the logging rules used for ipfilter
sub delete_ipfilter_rules
{
	my $self = shift;

$self->foreign_require("ipfilter", "ipfilter-lib.pl");
local $rules = $main::ipfilter_object->get_config();
local $rule1 = $self->find_ipfilter_rule($rules, $self->{'config_hash'}->{'iface'}, "in");
local $rule2 = $self->find_ipfilter_rule($rules, $self->{'config_hash'}->{'iface'}, "out");
$main::ipfilter_object->delete_rule($rule1) if ($rule1);
$main::ipfilter_object->delete_rule($rule2) if ($rule2);
if ($rule1 || $rule2) {
	return $main::ipfilter_object->apply_configuration();
	}
return undef;
}

# find_ipfilter_rule(&rules, iface, dir)
sub find_ipfilter_rule
{
	my $self = shift;

local ($rule) = grep { $_->{'action'} eq 'log' &&
		       $_->{'on'} eq $_[1] &&
		       $_->{'all'} &&
		       $_->{'active'} &&
		       $_->{'log-level'} eq "local7.debug" &&
		       $_->{'dir'} eq $_[2] } @{$_[0]};
return $rule;
}

# process_ipfilter_line(line, &hours, time-now)
# Process a IPFilter firewall line, and returns 1 if successful
sub process_ipfilter_line
{
	my $self = shift;

local ($line, $hours, $time_now) = @_;
if ($line =~ /^(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+).*ipmon\S*:.*\s$self->{'config_hash'}->{'iface'}\s+\S+\s+\S+\s+(\S+)\s+->\s+(\S+)\s+\S+\s+(\S+)\s+len\s+(\d+)\s+\(?(\d+)\)?.*(IN|OUT)/) {
	# Found a valid line
	local ($mon, $day, $hr, $min, $sec) = ($1, $2, $3, $4, $5);
	local ($src, $dest, $proto, $len, $dir) = ($6, $7, uc($8), $10, lc($11));
	local ($srchost, $srcport) = split(/,/, $src);
	local ($desthost, $destport) = split(/,/, $dest);
	$proto =~ s/:.*//;
	local $len = 1024;

	# Work out the real time
	local $tm = timelocal($sec, $min, $hr, $day,
			      $self->month_to_number($mon), $self->{'time_now_array'}->[5]);
	if ($tm > $time_now + 24*60*60) {
		# Was really last year
		$tm = timelocal($sec, $min, $hr, $day,
				$self->month_to_number($mon), $self->{'time_now_array'}->[5]-1);
		}
	local $htm = int($tm/(60*60));

	# Update the appropriate counters
	local $hour = $self->get_hour($htm);
	if ($self->indexof($hour, @$hours) < 0) {
		push(@$hours, $hour);
		}
	local $port;
	if ($proto eq 'TCP' || $proto eq 'UDP') {
		if ($dir eq "in") {
			$port = '_'.$destport.'_'.$srcport;
			}
		else {
			$port = '_'.$srcport.'_'.$destport;
			}
		}
	local $host = $dir eq "in" ? $dest : $src;
	local $key = $host.'_'.$proto.$port;
	local ($in, $out) = split(/ /, $hour->{$key});
	if ($dir eq "in") {
		$in += $len;
		}
	else {
		$out += $len;
		}
	$hour->{$key} = int($in)." ".int($out);
	return 1;
	}
else {
	return 0;
	}
}

# get_ipfilter_loglevel()
sub get_ipfilter_loglevel
{
	my $self = shift;

return "local7.debug";
}


1;

