#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Show a form for selecting a report

&ModuleLoader::preload_module('bandwidth_class', "bandwidth::BandwidthLib");
$self = new bandwidth::BandwidthLib:: 'bandwidth_class',%$self;

$self->ReadParse();
use Time::Local;
use Socket;

# Validate the bandwidth file
if (-d $self->{'bandwidth_log'}) {
	$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", "intro",
			 1, 1);
	$self->ui_print_endpage($self->text('index_elog', "<tt>$self->{'bandwidth_log'}</tt>",
			  "../config.cgi?$self->{'module_name'}"));
	}

# Validate the bandwidth directory
if ($self->{'config_hash'}->{'bandwidth_dir'} && !-d $self->{'hours_dir'}) {
	$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", "intro",
			 1, 1);
	$self->ui_print_endpage($self->text('index_edir', "<tt>$self->{'hours_dir'}</tt>",
			  "../config.cgi?$self->{'module_name'}"));
	}

# Make sure the net and cron modules work
foreach $m (split(/\s+/, $self->{'module_info_hash'}->{'depends'})) {
	next if ($m =~ /^[0-9\.]+$/);
	if (!$self->foreign_installed($m)) {
		$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", "intro",
				 1, 1);
		%{$self->{'minfo_hash'}} = $self->get_module_info($m);
		$self->ui_print_endpage($self->text('index_emod', $self->{'minfo_hash'}->{'desc'}));
		}
	}

# Make sure one of the syslog modules works
if (!$self->{'syslog_module'}) {
	$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", "intro",
			 1, 1);
	$self->ui_print_endpage($self->text('index_esyslog'));
	}

# Check the firewall system
if (!$self->{'config_hash'}->{'firewall_system'}) {
	# Try automatic detection
	$self->{'sys'} = $self->detect_firewall_system();
	if (!$self->{'sys'}) {
		$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", "intro",
				 1, 1);
		$self->ui_print_endpage($self->{'text_hash'}->{'index_efiresys'});
		}
	$self->{'config_hash'}->{'firewall_system'} = $self->{'sys'};
	$self->save_module_config();
	}
else {
	# Is the selected one installed?
	if (!$self->check_firewall_system($self->{'config_hash'}->{'firewall_system'})) {
		$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", "intro",
				 1, 1);
		$self->ui_print_endpage($self->text('index_efiresys2',
				  $self->{'text_hash'}->{'system_'.$self->{'config_hash'}->{'firewall_system'}},
				  "../config.cgi?$self->{'module_name'}"));
		}
	}
$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", "intro",
		 1, 1, 0, undef, undef, undef,
		 $self->text('index_firesys',
		       $self->{'text_hash'}->{'system_'.$self->{'config_hash'}->{'firewall_system'}},
		       $self->{'text_hash'}->{'syslog_'.$self->{'syslog_module'}}));

# Make sure the needed firewall rules and syslog entry are in place
$self->{'missingrule'} = !$self->check_rules();
if ($self->{'syslog_module'} eq "syslog") {
	# Normal syslog
	$self->{'conf'} = $main::syslog_object->get_config();
	$self->{'sysconf'} = $self->find_sysconf($self->{'conf'});
	}
else {
	# Syslog-ng
	$self->{'conf'} = $main::syslog_ng_object->get_config();
	($self->{'ngdest'}, $self->{'ngfilter'}, $self->{'nglog'}) = $self->find_sysconf_ng($self->{'conf'});
	$self->{'sysconf'} = $self->{'ngdest'} && $self->{'ngfilter'} && $self->{'nglog'};
	}

if (($self->{'missingrule'} || !$self->{'sysconf'}) && $self->{'access_hash'}->{'setup'}) {
	# Something is missing .. offer to set up
	print "$self->{'text_hash'}->{'index_setupdesc'}\n";
	if ($self->{'missingrule'} && !$self->{'sysconf'}) {
		print $self->{'text_hash'}->{'index_missing3'};
		}
	elsif ($self->{'missingrule'}) {
		print $self->{'text_hash'}->{'index_missing2'};
		}
	elsif (!$self->{'sysconf'}) {
		print $self->{'text_hash'}->{'index_missing1'};
		}
	print "<p>\n";
	if ($self->{'iptableserr'}) {
		print $self->{'iptableserr'},"<p>\n";
		}
	print $self->ui_form_start("setup.cgi");
	print "<b>$self->{'text_hash'}->{'index_iface'}</b>\n";
	foreach $i ($main::net_object->active_interfaces(), $main::net_object->boot_interfaces()) {
		push(@{$self->{'ifaces_array'}}, $i->{'fullname'}) if ($i->{'virtual'} eq '');
		}
	print $self->ui_select("iface", $self->{'config_hash'}->{'iface'},
			 [ map { [ $_, $_ ] } $self->unique(@{$self->{'ifaces_array'}}) ]);
	print $self->ui_submit($self->{'text_hash'}->{'index_setup'});
	print $self->ui_form_end();
	print "<p>\n";
	}
elsif ($self->{'missingrule'} || !$self->{'sysconf'}) {
	print "$self->{'text_hash'}->{'index_setupdesc'}\n";
	print "$self->{'text_hash'}->{'index_setupcannot'}<p>\n";
	}

# See if any hours have been summarized yet
@{$self->{'hours_array'}} = $self->list_hours();
if (@{$self->{'hours_array'}}) {
	# Show reporting form
	print $self->ui_form_start("index.cgi");
	print "<table>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'index_by'}</b></td>\n";
	print "<td>",$self->ui_select("by", $self->{'in_hash'}->{'by'},
		[ [ 'hour', $self->{'text_hash'}->{'index_hour'} ],
		  [ 'day', $self->{'text_hash'}->{'index_day'} ],
		  [ 'host', $self->{'text_hash'}->{'index_host'} ],
		  [ 'proto', $self->{'text_hash'}->{'index_proto'} ],
		  [ 'iport', $self->{'text_hash'}->{'index_iport'} ],
		  [ 'oport', $self->{'text_hash'}->{'index_oport'} ],
		  [ 'port', $self->{'text_hash'}->{'index_port'} ] ]),"</td>\n";

	print "<td><b>$self->{'text_hash'}->{'index_for'}</b></td>\n";
	print "<td>",$self->ui_select("for", $self->{'in_hash'}->{'for'},
		[ [ '', $self->{'text_hash'}->{'index_all'} ],
		  [ 'host', $self->{'text_hash'}->{'index_forhost'} ],
		  [ 'proto', $self->{'text_hash'}->{'index_forproto'} ],
		  [ 'iport', $self->{'text_hash'}->{'index_foriport'} ],
		  [ 'oport', $self->{'text_hash'}->{'index_foroport'} ] ]),"\n";
	print $self->ui_textbox("what", $self->{'in_hash'}->{'for'} ? $self->{'in_hash'}->{'what'} : "", 20),
	      "</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'index_from'}</b></td>\n";
	print "<td colspan=4>",
		$self->date_input($self->{'in_hash'}->{'from_day'}, $self->{'in_hash'}->{'from_month'},
			    $self->{'in_hash'}->{'from_year'}, "from"),
		$self->hourmin_input($self->{'in_hash'}->{'from_hour'}, "00", "from"),"</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'index_to'}</b></td>\n";
	print "<td colspan=4>",
		$self->date_input($self->{'in_hash'}->{'to_day'}, $self->{'in_hash'}->{'to_month'},
			    $self->{'in_hash'}->{'to_year'}, "to"),
		$self->hourmin_input($self->{'in_hash'}->{'to_hour'}, "00", "to"),"</td> </tr>\n";

	if (!%{$self->{'in_hash'}}) {
		# Enable by default
		$self->{'in_hash'}->{'low'} = 1;
		}
	print "<tr> <td></td> <td colspan=4>\n";
	print $self->ui_checkbox("low", 1, $self->{'text_hash'}->{'index_low'}, $self->{'in_hash'}->{'low'});
	print $self->ui_checkbox("resolv", 1, $self->{'text_hash'}->{'index_resolv'}, $self->{'in_hash'}->{'resolv'});
	print "</td> </tr>\n";

	print "<tr> <td colspan=4>",
		$self->ui_submit($self->{'text_hash'}->{'index_search'}),"</td> </td>\n";

	print "</table>\n";
	print $self->ui_form_end();
	}
elsif (!$self->{'missingrule'} && $self->{'sysconf'}) {
	print "<b>$self->{'text_hash'}->{'index_none'}</b><p>\n";
	}

# Find and show any results
if ($self->{'in_hash'}->{'by'}) {
	# Work out the time range, if any
	$self->error_setup($self->{'text_hash'}->{'index_err'});
	$self->{'fhour'} = &parse_hour("from");
	$self->{'thour'} = &parse_hour("to");

	# First find traffic that matches the 'for' part
	if ($self->{'in_hash'}->{'for'} eq 'host') {
		if ($self->{'in_hash'}->{'what'} =~ /^(\d+)\.(\d+)\.(\d+)\.0$/) {
			%{$self->{'forhost_hash'}} = map { ("$1.$2.$3.$_", 1) } (0 .. 255);
			}
		else {
			$self->{'forhost'} = $self->to_ipaddress($self->{'in_hash'}->{'what'});
			$self->{'forhost'} || $self->error($self->{'text_hash'}->{'index_ehost'});
			$self->{'forhost_hash'}->{$self->{'forhost'}}++;
			}
		}
	elsif ($self->{'in_hash'}->{'for'} eq 'proto') {
		$self->{'forproto'} = uc($self->{'in_hash'}->{'what'});
		$self->{'forproto'} || $self->error($self->{'text_hash'}->{'index_eproto'});
		}
	elsif ($self->{'in_hash'}->{'for'} eq 'iport' || $self->{'in_hash'}->{'for'} eq 'oport') {
		if ($self->{'in_hash'}->{'what'} =~ /^\d+$/) {
			$self->{'forportmin'} = $self->{'forportmax'} = $self->{'in_hash'}->{'what'};
			}
		elsif ($self->{'in_hash'}->{'what'} =~ /^(\d+)\-(\d+)$/) {
			$self->{'forportmin'} = $1;
			$self->{'forportmax'} = $2;
			}
		else {
			$self->{'forportmin'} = getservbyname($self->{'in_hash'}->{'what'}, 'tcp');
			$self->{'forportmin'} ||= getservbyname($self->{'in_hash'}->{'what'}, 'udp');
			$self->{'forportmin'} || $self->error($self->{'text_hash'}->{'index_eport'});
			$self->{'forportmax'} = $self->{'forportmin'};
			}
		}
	foreach $h (@{$self->{'hours_array'}}) {
		next if ($self->{'fhour'} && $h < $self->{'fhour'});
		next if ($self->{'thour'} && $h > $self->{'thour'});
		$self->{'hour'} = $self->get_hour($h);

		# Work out start time for this day
		@{$self->{'tm_array'}} = localtime($h*60*60);
		$self->{'thisday'} = timelocal(0, 0, 0, $self->{'tm_array'}->[3], $self->{'tm_array'}->[4], $self->{'tm_array'}->[5])/(60*60);

		# Scan all traffic for the hour
		foreach $k (keys %{$self->{'hour'}}) {
			# Skip this count if not relevant
			($self->{'host'}, $self->{'proto'}, $self->{'iport'}, $self->{'oport'}) = split(/_/, $k);
			next if (!$self->{'proto'});
			next if (%{$self->{'forhost_hash'}} && !$self->{'forhost_hash'}->{$self->{'host'}});
			next if ($self->{'forproto'} && $self->{'proto'} ne $self->{'forproto'});
			next if ($self->{'in_hash'}->{'for'} eq 'iport' &&
			         ($self->{'iport'} < $self->{'forportmin'} || $self->{'iport'} >$self->{'forportmax'}));
			next if ($self->{'in_hash'}->{'for'} eq 'oport' &&
			         ($self->{'oport'} < $self->{'forportmin'} || $self->{'oport'} >$self->{'forportmax'}));

			# Skip this count if classifying by port and there
			# isn't one
			next if ($self->{'in_hash'}->{'by'} eq 'iport' && !$self->{'iport'} ||
				 $self->{'in_hash'}->{'by'} eq 'oport' && !$self->{'oport'} ||
				 $self->{'in_hash'}->{'by'} eq 'port' && !$self->{'iport'} && !$self->{'oport'});

			# Work out a nice service name
			local ($nsname, $nsoname, $nsiname);
			local $relport;
			if ($self->{'in_hash'}->{'by'} eq 'iport') {
				$nsname = $nsiname = getservbyport($self->{'iport'}, lc($self->{'proto'}));
				}
			elsif ($self->{'in_hash'}->{'by'} eq 'oport') {
				$nsname = $nsoname = getservbyport($self->{'oport'}, lc($self->{'proto'}));
				}
			elsif ($self->{'in_hash'}->{'by'} eq 'port') {
				$nsoname = getservbyport($self->{'oport'}, lc($self->{'proto'}));
				$nsiname = getservbyport($self->{'iport'}, lc($self->{'proto'}));
				$nsname = $nsoname || $nsiname;
				}

			# Resolv the hostname
			local $resolved;
			if ($self->{'in_hash'}->{'resolv'} && $self->{'in_hash'}->{'by'} eq 'host') {
				$resolved = gethostbyaddr(inet_aton($self->{'host'}),
							  AF_INET);
				}

			# Skip traffic to high ports, if requested
			next if ($self->{'in_hash'}->{'low'} && $self->{'in_hash'}->{'by'} eq 'iport' &&
				 $self->{'iport'} >= 1024 && !$nsname);
			next if ($self->{'in_hash'}->{'low'} && $self->{'in_hash'}->{'by'} eq 'oport' &&
				 $self->{'oport'} >= 1024 && !$nsname);

			# Update the relevant category
			($self->{'in'}, $self->{'out'}) = split(/ /, $self->{'hour'}->{$k});
			if ($self->{'in_hash'}->{'by'} eq 'hour') {
				$self->{'count_hash'}->{$h} += $self->{'in'}+$self->{'out'};
				$self->{'icount_hash'}->{$h} += $self->{'in'};
				$self->{'ocount_hash'}->{$h} += $self->{'out'};
				}
			elsif ($self->{'in_hash'}->{'by'} eq 'day') {
				$self->{'count_hash'}->{$self->{'thisday'}} += $self->{'in'}+$self->{'out'};
				$self->{'icount_hash'}->{$self->{'thisday'}} += $self->{'in'};
				$self->{'ocount_hash'}->{$self->{'thisday'}} += $self->{'out'};
				}
			elsif ($self->{'in_hash'}->{'by'} eq 'host') {
				$self->{'count_hash'}->{$resolved || $self->{'host'}} += $self->{'in'}+$self->{'out'};
				$self->{'icount_hash'}->{$resolved || $self->{'host'}} += $self->{'in'};
				$self->{'ocount_hash'}->{$resolved || $self->{'host'}} += $self->{'out'};
				}
			elsif ($self->{'in_hash'}->{'by'} eq 'proto') {
				$self->{'count_hash'}->{$self->{'proto'}} += $self->{'in'}+$self->{'out'};
				$self->{'icount_hash'}->{$self->{'proto'}} += $self->{'in'};
				$self->{'ocount_hash'}->{$self->{'proto'}} += $self->{'out'};
				}
			elsif ($self->{'in_hash'}->{'by'} eq 'iport') {
				$self->{'count_hash'}->{$nsname || "$self->{'proto'} $self->{'iport'}"} += $self->{'in'}+$self->{'out'};
				$self->{'icount_hash'}->{$nsname || "$self->{'proto'} $self->{'iport'}"} += $self->{'in'};
				$self->{'ocount_hash'}->{$nsname || "$self->{'proto'} $self->{'iport'}"} += $self->{'out'};
				}
			elsif ($self->{'in_hash'}->{'by'} eq 'oport') {
				$self->{'count_hash'}->{$nsname || "$self->{'proto'} $self->{'oport'}"} += $self->{'in'}+$self->{'out'};
				$self->{'icount_hash'}->{$nsname || "$self->{'proto'} $self->{'oport'}"} += $self->{'in'};
				$self->{'ocount_hash'}->{$nsname || "$self->{'proto'} $self->{'oport'}"} += $self->{'out'};
				}
			elsif ($self->{'in_hash'}->{'by'} eq 'port') {
				if (!$self->{'in_hash'}->{'low'} || $self->{'oport'} < 1024 || $nsoname) {
					$self->{'count_hash'}->{$nsoname || "$self->{'proto'} $self->{'oport'}"} += $self->{'in'};
					$self->{'icount_hash'}->{$nsoname || "$self->{'proto'} $self->{'oport'}"} += $self->{'in'};
					}
				if (!$self->{'in_hash'}->{'low'} || $self->{'iport'} < 1024 || $nsiname) {
					$self->{'count_hash'}->{$nsiname || "$self->{'proto'} $self->{'iport'}"} += $self->{'out'};
					$self->{'ocount_hash'}->{$nsiname || "$self->{'proto'} $self->{'iport'}"} += $self->{'out'};
					}
				}
			}
		}

	# Find max and size
	$self->{'max'} = 0;
	foreach $k (keys %{$self->{'count_hash'}}) {
		if ($self->{'count_hash'}->{$k} > $self->{'max'}) {
			$self->{'max'} = $self->{'count_hash'}->{$k};
			}
		}
	$self->{'width'} = 500;

	# Fill in missing hours or days
	if ($self->{'in_hash'}->{'by'} eq 'hour' || $self->{'in_hash'}->{'by'} eq 'day') {
		@{$self->{'order_array'}} = sort { $b <=> $a } keys %{$self->{'count_hash'}};
		$self->{'inc'} = $self->{'in_hash'}->{'by'} eq 'hour' ? 1 : 24;
		$self->{'plus'} = $self->{'in_hash'}->{'by'} eq 'hour' ? 0 : 1;
		for($i=$self->{'order_array'}->[0]; $i>=$self->{'order_array'}->[$#{$self->{'order_array'}}]; $i-=$self->{'inc'}) {
			$self->{'count_hash'}->{$i} = 0 if (!$self->{'count_hash'}->{$i} &&
					   !$self->{'count_hash'}->{$i+$self->{'plus'}} && !$self->{'count_hash'}->{$i-$self->{'plus'}});
			}
		}

	# Show graph
	if ($self->{'in_hash'}->{'by'} eq 'hour' || $self->{'in_hash'}->{'by'} eq 'day') {
		@{$self->{'order_array'}} = sort { $b <=> $a } keys %{$self->{'count_hash'}};
		}
	else {
		@{$self->{'order_array'}} = sort { $self->{'count_hash'}->{$b} <=> $self->{'count_hash'}->{$a} } keys %{$self->{'count_hash'}};
		}
	if ($self->{'in_hash'}->{'by'} ne 'hour' && $self->{'in_hash'}->{'by'} ne 'day') {
		@{$self->{'order_array'}} = grep { $self->{'count_hash'}->{$_} } @{$self->{'order_array'}};
		}
	if (@{$self->{'order_array'}}) {
		print "<table width=100% cellpadding=0 cellspacing=0>\n";
		print "<tr>\n";
		print "<td><b>",$self->{'text_hash'}->{'index_h'.$self->{'in_hash'}->{'by'}},"</b></td>\n";
		print "<td colspan=2><b>$self->{'text_hash'}->{'index_usage'}</b></td>\n";
		print "</tr>\n";
		$self->{'total'} = 0;
		foreach $k (@{$self->{'order_array'}}) {
			print "<tr>\n";
			if ($self->{'in_hash'}->{'by'} eq 'hour') {
				print "<td>",$self->make_date($k*60*60),"</td>\n";
				}
			elsif ($self->{'in_hash'}->{'by'} eq 'day') {
				$self->{'date'} = &make_date_day($k*60*60);
				print "<td>$self->{'date'}</td>\n";
				}
			else {
				print "<td>$k</td>\n";
				}
			print "<td>";
			printf "<img src=images/red.gif width=%d height=10>",
				$self->{'max'} ? int($self->{'width'} * $self->{'icount_hash'}->{$k}/$self->{'max'})+1 : 1;
			printf "<img src=images/blue.gif width=%d height=10>",
				$self->{'max'} ? int($self->{'width'} * $self->{'ocount_hash'}->{$k}/$self->{'max'})+1 : 1;
			print "</td>";
			print "<td>",$self->nice_size($self->{'count_hash'}->{$k}),"</td>\n";
			$self->{'total'} += $self->{'count_hash'}->{$k};
			print "</tr>\n";
			}
		print "<tr> <td colspan=2></td> <td><b>",
		$self->nice_size($self->{'total'}),"</td> </tr>\n";
		print "</table>\n";
		}
	else {
		print "<b>$self->{'text_hash'}->{'index_nomatch'}</b><p>\n";
		}
}

if (!$self->{'missingrule'} && $self->{'sysconf'}) {
	print "<hr>\n";
	print $self->ui_buttons_start();

	# Show button to rotate now
	print $self->ui_buttons_row("rotate.cgi", $self->{'text_hash'}->{'index_rotate'},
			      $self->{'text_hash'}->{'index_rotatedesc'});

	if ($self->{'access_hash'}->{'setup'}) {
		# Show button to turn off reporting
		print $self->ui_buttons_row("turnoff.cgi", $self->{'text_hash'}->{'index_turnoff'},
				      $self->{'text_hash'}->{'index_turnoffdesc'});
		}

	print $self->ui_buttons_end();
	}

$self->ui_print_footer("/", $self->{'text_hash'}->{'index'});

1;
		
}

sub parse_hour
{
	shift if ($self == $_[0]);

local ($pfx) = @_;
local ($day, $month, $year) =
	($self->{'in_hash'}->{$pfx."_day"}, $self->{'in_hash'}->{$pfx."_month"}-1, $self->{'in_hash'}->{$pfx."_year"}-1900);
local $hour = $self->{'in_hash'}->{$pfx."_hour"};
return undef if (!$day);
if ($hour eq "") {
	$hour = $pfx eq "from" ? 0 : 23;
	}

eval { $self->{'tm'} = timelocal(0, 0, $hour, $day, $month, $year) };
if (!$self->{'tm'} || $@) {
	$self->error($self->{'text_hash'}->{'index_e'.$pfx});
	}
return int($self->{'tm'}/(60*60));
}

# make_date_day(seconds)
# Converts a Unix date/time in seconds to a human-readable form
sub make_date_day
{
	shift if ($self == $_[0]);

local(@tm);
@tm = localtime($_[0]);
return sprintf "%d/%s/%d",
		$tm[3], $self->{'text_hash'}->{"smonth_".($tm[4]+1)},
		$tm[5]+1900;
}


1;

