#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Add needed firewall rules and syslog entry, and apply configurations

&ModuleLoader::preload_module('bandwidth_class', "bandwidth::BandwidthLib");
$self = new bandwidth::BandwidthLib:: 'bandwidth_class',%$self;

$self->ReadParse();
$self->{'access_hash'}->{'setup'} || $self->error($self->{'text_hash'}->{'setup_ecannot'});

# Add missing firewall rules
$self->{'err'} = $self->setup_rules($self->{'in_hash'}->{'iface'});
$self->error($self->{'err'}) if ($self->{'err'});

if ($self->{'syslog_module'} eq "syslog") {
	# Add syslog entry
	$conf = $main::syslog_object->get_config();
	$self->{'sysconf'} = $self->find_sysconf($conf);
	if (!$self->{'sysconf'}) {
		$self->lock_file($main::syslog_object->{'config_hash'}->{'syslog_conf'});
		if ($main::syslog_object->{'config_hash'}->{'tags'}) {
			local $conf = $main::syslog_object->get_config();
			($self->{'tag'}) = grep { $_->{'tag'} eq '*' } @$conf;
			}
		$main::syslog_object->create_log({ 'file' => $self->{'bandwidth_log'},
				      'active' => 1,
				      'section' => $self->{'tag'},
				      'sel' => [ $self->get_loglevel() ] });
		$self->unlock_file($main::syslog_object->{'config_hash'}->{'syslog_conf'});
		$self->{'err'} = $main::syslog_object->restart_syslog();
		$self->error($self->{'err'}) if ($self->{'err'});
		}
	}
else {
	# Add syslog_ng entry
	$conf = $main::syslog_ng_object->get_config();
	($self->{'dest'}, $self->{'filter'}, $self->{'log'}) = $self->find_sysconf_ng($conf);
	$self->lock_file($main::syslog_ng_object->{'config_hash'}->{'syslogng_conf'});
	if (!$self->{'dest'}) {
		# Create destination file entry
		$self->{'dest'} = { 'name' => 'destination',
			  'type' => 1,
			  'values' => [ 'd_bandwidth' ],
			  'members' => [
				{ 'name' => 'file',
				  'values' => [ $self->{'bandwidth_log'} ] }
					]
			};
		$main::syslog_ng_object->save_directive($conf, undef, undef, $self->{'dest'}, 0);
		}
	if (!$self->{'filter'}) {
		# Create filter for facility and level
		($self->{'fac'}, $self->{'lvl'}) = split(/\./, $self->get_loglevel());
		$self->{'lvl'} =~ s/^=//;
		$self->{'filter'} = { 'name' => 'filter',
			    'type' => 1,
			    'values' => [ 'f_bandwidth' ],
			    'members' => [ ]
			  };
		if ($self->{'fac'} ne "*") {
			push(@{$self->{'filter'}->{'members'}},
			     { 'name' => 'facility',
			       'values' => [ $self->{'fac'} ] });
			}
		if ($self->{'fac'} ne "*" && $self->{'lvl'} ne "*") {
			push(@{$self->{'filter'}->{'members'}}, "and");
			}
		if ($self->{'lvl'} ne "*") {
			push(@{$self->{'filter'}->{'members'}},
			     { 'name' => 'priority',
			       'values' => [ $self->{'lvl'} ] });
			}
		$main::syslog_ng_object->save_directive($conf, undef, undef, $self->{'filter'}, 0);
		}
	if (!$self->{'log'}) {
		# Create log for the default source, destination and filter
		@{$self->{'sources_array'}} = $main::syslog_ng_object->find("source", $conf);
		$self->{'log'} = { 'name' => 'log',
			 'type' => 1,
			 'values' => [ ],
			 'members' => [
				{ 'name' => 'source',
				  'values' => [ $self->{'sources_array'}->[0]->{'value'} ], },
				{ 'name' => 'filter',
				  'values' => [ "f_bandwidth" ], },
				{ 'name' => 'destination',
				  'values' => [ "d_bandwidth" ], },
					]
			};
		$main::syslog_ng_object->save_directive($conf, undef, undef, $self->{'log'}, 0);
		}
	$self->unlock_file($main::syslog_ng_object->{'config_hash'}->{'syslogng_conf'});
	}

# Save the interface
$self->lock_file($self->{'module_config_file'});
$self->{'config_hash'}->{'iface'} = $self->{'in_hash'}->{'iface'};
$self->save_module_config();
$self->unlock_file($self->{'module_config_file'});

# Setup the rotation cron job
$self->{'job'} = $self->find_cron_job();
if (!$self->{'job'}) {
	$main::cron_object->create_wrapper($self->{'cron_cmd'}, $self->{'module_name'}, "rotate.pl");
	$self->{'job'} = { 'user' => 'root',
		 'active' => 1,
		 'command' => $self->{'cron_cmd'},
		 'mins' => '0',
		 'hours' => '*',
		 'days' => '*',
		 'months' => '*',
		 'weekdays' => '*',
		};
	$self->lock_file($main::cron_object->cron_file($self->{'job'}));
	$main::cron_object->create_cron_job($self->{'job'});
	$self->unlock_file($main::cron_object->cron_file($self->{'job'}));
	}

$self->webmin_log("setup", undef, $self->{'in_hash'}->{'iface'});
$self->redirect("");

1;
		
}


1;

