#!/usr/bin/perl
package bind8::Bind8Lib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'bind8::Bind8Lib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# bind8-lib.pl
# Common functions for bind8 config files

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "bind8::RecordsLib");
$self = new bind8::RecordsLib:: ref($self), %$self; 
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
@{$self->{'extra_forward_array'}} = split(/\s+/, $self->{'config_hash'}->{'extra_forward'});
@{$self->{'extra_reverse_array'}} = split(/\s+/, $self->{'config_hash'}->{'extra_reverse'});
%{$self->{'is_extra_hash'}} = map { $_, 1 } (@{$self->{'extra_forward_array'}}, @{$self->{'extra_reverse_array'}});
%{$self->{'access_hash'}} = $self->get_module_acl();
$self->{'zone_names_cache'} = "$self->{'module_config_directory'}/zone-names";
$self->{'zone_names_version'} = 2;

if (open(VERSION, "$self->{'module_config_directory'}/version")) {
	chop($self->{'bind_version'} = <VERSION>);
	close(VERSION);
	}

# address_port_input(addresstext, portlabeltext, portnametext, defaulttext,
#                    addressname, portname, &config, size, type)
sub address_port_input
  {
    my $rv = &address_input($_[0], $_[4], $_[6], $_[8]);
    my $v = &find($_[4], $_[6]);

    my $port;
    for ($i = 0; $i < @{$v->{'values'}}; $i++) {
      if ($v->{'values'}->[$i] eq $_[5]) {
	$port = $v->{'values'}->[$i+1];
	last;
      }
    }
    my $n;
    ($n = $_[5]) =~ s/[^A-Za-z0-9_]/_/g;
    $rv .= "<td valign=top><b>$_[1]</b></td> <td nowrap valign=top>\n";
    $rv .= sprintf "<input type=radio name=${n}_def value=1 %s> $_[3]\n",
      defined($port) ? "" : "checked";
    $rv .= sprintf "<input type=radio name=${n}_def value = 0 %s> ",
      defined($port) ? "checked" : "";
    $rv .= sprintf "<input name=$n size=$_[7] value=\"%s\"> $_[2]</td>\n",
      defined($port) ? $port : "";

    return $rv;
  }

# directives that need their value to be quoted
@{$self->{'need_quote_array'}} = ( "file", "zone", "pid-file", "statistics-file",
	        "dump-file", "named-xfer", "secret" );
foreach $need (@{$self->{'need_quote_array'}}) {
	$self->{'need_quote_hash'}->{$need}++;
	}

1;

if ($self->{'bind_version'} >= 9) {
	@{$self->{'cat_list_array'}} = ( 'default', 'general', 'database', 'security', 'config',
		      'resolver', 'xfer-in', 'xfer-out', 'notify', 'client',
		      'unmatched', 'network', 'update', 'queries', 'dispatch',
		      'dnssec', 'lame-servers' );
	}
else {
	@{$self->{'cat_list_array'}} = ( 'default', 'config', 'parser', 'queries',
		      'lame-servers', 'statistics', 'panic', 'update',
		      'ncache', 'xfer-in', 'xfer-out', 'db',
		      'eventlib', 'packet', 'notify', 'cname', 'security',
		      'os', 'insist', 'maintenance', 'load', 'response-checks');
	}

@{$self->{'syslog_levels_array'}} = ( 'kern', 'user', 'mail', 'daemon', 'auth', 'syslog',
		   'lpr', 'news', 'uucp', 'cron', 'authpriv', 'ftp',
		   'local0', 'local1', 'local2', 'local3',
		   'local4', 'local5', 'local6', 'local7' );

@{$self->{'severities_array'}} = ( 'critical', 'error', 'warning', 'notice', 'info',
		'debug', 'dynamic' );

$self->{'heiropen_file'} = "$self->{'module_config_directory'}/heiropen";

1;

1;
		return $self;
}


# get_config()
# Returns an array of references to assocs, each containing the details of
# one directive
sub get_config
{
	my $self = shift;

if (!@{$self->{'get_config_cache_array'}}) {
	@{$self->{'get_config_cache_array'}} = $self->read_config_file($self->{'config_hash'}->{'named_conf'});
	}
return \@{$self->{'get_config_cache_array'}};
}

# get_config_parent([file])
# Returns a structure containing the top-level config as members
sub get_config_parent
{
	my $self = shift;

local $file = $_[0] || $self->{'config_hash'}->{'named_conf'};
if (!defined($self->{'get_config_parent_cache_hash'}->{$file})) {
	local $conf = $self->get_config();
	if (!defined($self->{'lines_count_hash'}->{$file})) {
		local $lref = $self->read_file_lines($file);
		$self->{'lines_count_hash'}->{$file} = @$lref;
		}
	$self->{'get_config_parent_cache_hash'}->{$file} =
	       { 'file' => $file,
		 'type' => 1,
		 'line' => -1,
		 'eline' => $self->{'lines_count_hash'}->{$file},
		 'members' => $conf };
	}
return $self->{'get_config_parent_cache_hash'}->{$file};
}

# read_config_file(file, [expand includes], [no chroot])
# Reads a config file and returns an array of values
sub read_config_file
{
	my $self = shift;

local($lnum, $line, $cmode, @ltok, @lnum, @tok,
      @rv, $i, $t, $j, $ifile, @inc, $str);
$lnum = 0;
open(FILE, $_[2] ? $_[0] : $self->make_chroot($_[0]));
while($line = <FILE>) {
	# strip comments
	$line =~ s/\r|\n//g;
	$line =~ s/#.*$//g;
	$line =~ s/\/\/.*$//g if ($line !~ /".*\/\/.*"/);
	$line =~ s/\/\*.*\*\///g;
	while(1) {
		if (!$cmode && $line =~ /\/\*/) {
			# start of a C-style comment
			$cmode = 1;
			$line =~ s/\/\*.*$//g;
			}
		elsif ($cmode) {
			if ($line =~ /\*\//) {
				# end of comment
				$cmode = 0;
				$line =~ s/^.*\*\///g;
				}
			else { $line = ""; last; }
			}
		else { last; }
		}

	# split line into tokens
	undef(@ltok);
	while(1) {
		if ($line =~ /^\s*\"([^"]*)"(.*)$/) {
			push(@ltok, $1); $line = $2;
			}
		elsif ($line =~ /^\s*([{};])(.*)$/) {
			push(@ltok, $1); $line = $2;
			}
		elsif ($line =~ /^\s*([^{}; \t]+)(.*)$/) {
			push(@ltok, $1); $line = $2;
			}
		else { last; }
		}
	foreach $t (@ltok) {
		push(@tok, $t); push(@lnum, $lnum);
		}
	$lnum++;
	}
close(FILE);
$self->{'lines_count_hash'}->{$_[0]} = $lnum;

# parse tokens into data structures
$i = 0; $j = 0;
while($i < @tok) {
	$str = $self->parse_struct(\@tok, \@lnum, \$i, $j++, $_[0]);
	if ($str) { push(@rv, $str); }
	}
if (!@rv) {
	# Add one dummy directive, so that the file is known
	push(@rv, { 'name' => 'dummy',
		    'line' => 0,
		    'eline' => 0,
		    'index' => 0,
		    'file' => $_[0] });
	}

if (!$_[1]) {
	# expand include directives
	while($self->recursive_includes(\@rv, $self->base_directory(\@rv))) {
		# This is done repeatedly to handle includes within includes
		}
	}

return @rv;
}

# $self->recursive_includes(&dirs, base)
sub recursive_includes
{
	my $self = shift;

local ($i, $j);
local $any = 0;
for($i=0; $i<@{$_[0]}; $i++) {
	if (lc($_[0]->[$i]->{'name'}) eq "include") {
		# found one.. replace the include directive with it
		$self->{'ifile'} = $_[0]->[$i]->{'value'};
		if ($self->{'ifile'} !~ /^\//) {
			$self->{'ifile'} = "$_[1]/$self->{'ifile'}";
			}
		local @inc = $self->read_config_file($self->{'ifile'}, 1);

		# update index of included structures
		local $j;
		for($j=0; $j<@inc; $j++) {
			$inc[$j]->{'index'} += $_[0]->[$i]->{'index'};
			}

		# update index of structures after include
		for($j=$i+1; $j<@{$_[0]}; $j++) {
			$_[0]->[$j]->{'index'} += scalar(@inc) - 1;
			}
		splice(@{$_[0]}, $i--, 1, @inc);
		$any++;
		}
	elsif ($_[0]->[$i]->{'type'} == 1) {
		# Check sub-structures too
		$any += $self->recursive_includes($_[0]->[$i]->{'members'}, $_[1]);
		}
	}
return $any;
}

# $self->parse_struct(&tokens, &lines, &line_num, index, file)
# A structure can either have one value, or a list of values.
# Pos will end up at the start of the next structure
sub parse_struct
{
	my $self = shift;

local (%str, $i, $j, $t, @vals, $str);
$i = ${$_[2]};
$str{'name'} = lc($_[0]->[$i]);
$str{'line'} = $_[1]->[$i];
$str{'index'} = $_[3];
$str{'file'} = $_[4];
if ($str{'name'} eq 'inet') {
	# The inet directive doesn't have sub-structures, just multiple
	# values with { } in them
	$str{'type'} = 2;
	$str{'members'} = { };
	while(1) {
		$t = $_[0]->[++$i];
		if ($_[0]->[$i+1] eq "{") {
			# Start of a named sub-structure ..
			$i += 2;	# skip {
			$j = 0;
			while($_[0]->[$i] ne "}") {
				$str = $self->parse_struct($_[0], $_[1], \$i, $j++, $_[4]);
				if ($str) {
					push(@{$str{'members'}->{$t}}, $str);
					}
				}
			next;
			}
		elsif ($t eq ";") { last; }
		push(@vals, $t);
		}
	$i++;	# skip trailing ;
	$str{'values'} = \@vals;
	$str{'value'} = $vals[0];
	}
else {
	# Normal directive, like foo bar; or foo bar { smeg; };
	while(1) {
		$t = $_[0]->[++$i];
		if ($t eq "{" || $t eq ";" || $t eq "}") { last; }
		elsif (!defined($t)) { ${$_[2]} = $i; return undef; }
		else { push(@vals, $t); }
		}
	$str{'values'} = \@vals;
	$str{'value'} = $vals[0];
	if ($t eq "{") {
		# contains sub-structures.. parse them
		local(@mems, $j);
		$i++;		# skip {
		$str{'type'} = 1;
		$j = 0;
		while($_[0]->[$i] ne "}") {
			if (!defined($_[0]->[$i])) { ${$_[2]} = $i; return undef; }
			$str = $self->parse_struct($_[0], $_[1], \$i, $j++, $_[4]);
			if ($str) { push(@mems, $str); }
			}
		$str{'members'} = \@mems;
		$i += 2;	# skip trailing } and ;
		}
	else {
		# only a single value..
		$str{'type'} = 0;
		if ($t eq ";") {
			$i++;	# skip trailing ;
			}
		}
	}
$str{'eline'} = $_[1]->[$i-1];	# ending line is the line number the trailing
				# ; is on
${$_[2]} = $i;
return \%str;
}

# find(name, &array)
sub find
{
	my $self = shift;

local($c, @rv);
foreach $c (@{$_[1]}) {
	if ($c->{'name'} eq $_[0]) {
		push(@rv, $c);
		}
	}
return @rv ? wantarray ? @rv : $rv[0]
           : wantarray ? () : undef;
}

# find_value(name, &array)
sub find_value
{
	my $self = shift;

local(@v);
@v = $self->find($_[0], $_[1]);
if (!@v) { return undef; }
elsif (wantarray) { return map { $_->{'value'} } @v; }
else { return $v[0]->{'value'}; }
}

# $self->base_directory([&config], [no-cache])
# Returns the base directory for named files
sub base_directory
{
	my $self = shift;

if ($_[1] || !-r $self->{'zone_names_cache'}) {
	# Actually work out base
	local ($opts, $dir, $conf);
	$conf = $_[0] ? $_[0] : $self->get_config();
	if (($opts = $self->find("options", $conf)) &&
	    ($dir = $self->find("directory", $opts->{'members'}))) {
		return $dir->{'value'};
		}
	if ($self->{'config_hash'}->{'named_conf'} =~ /^(.*)\/[^\/]+$/ && $1) {
		return $1;
		}
	return "/etc";
	}
else {
	# Use cache
	local %znc;
	$self->read_file_cached($self->{'zone_names_cache'}, \%znc);
	return $znc{'base'} || $self->base_directory($_[0], 1);
	}
}

# save_directive(&parent, name|&old, &values, indent, [structonly])
# Given a structure containing a directive name, type, values and members
# add, update or remove that directive in config structure and data files.
# Updating of files assumes that there is no overlap between directives -
# each line in the config file must contain part or all of only one directive.
sub save_directive
{
	my $self = shift;

local(@oldv, @newv, $pm, $i, $o, $n, $lref, @nl);
$pm = $_[0]->{'members'};
@oldv = ref($_[1]) ? @{$_[1]} : $self->find($_[1], $pm);
@newv = @{$_[2]};
for($i=0; $i<@oldv || $i<@newv; $i++) {
	if ($i >= @oldv && !$_[5]) {
		# a new directive is being added.. put it at the end of
		# the parent
		if (!$_[4]) {
			local $parent = $self->get_config_parent($newv[$i]->{'file'} ||
							   $_[0]->{'file'});
			$lref = $self->read_file_lines(
				$self->make_chroot($newv[$i]->{'file'} ||
					     $_[0]->{'file'}));
			@nl = $self->directive_lines($newv[$i], $_[3]);
			splice(@$lref, $_[0]->{'eline'}, 0, @nl);
			$newv[$i]->{'file'} = $_[0]->{'file'};
			$newv[$i]->{'line'} = $_[0]->{'eline'};
			$newv[$i]->{'eline'} =
				$_[0]->{'eline'} + scalar(@nl) - 1;
			$self->renumber($parent, $_[0]->{'eline'}-1,
				  $_[0]->{'file'}, scalar(@nl));
			}
		push(@$pm, $newv[$i]);
		}
	elsif ($i >= @oldv && $_[5]) {
		# a new directive is being added.. put it at the start of
		# the parent
		if (!$_[4]) {
			local $parent = $self->get_config_parent($newv[$i]->{'file'} ||
							   $_[0]->{'file'});
			$lref = $self->read_file_lines(
				$self->make_chroot($newv[$i]->{'file'} ||
					     $_[0]->{'file'}));
			@nl = $self->directive_lines($newv[$i], $_[3]);
			splice(@$lref, $_[0]->{'line'}+1, 0, @nl);
			$newv[$i]->{'file'} = $_[0]->{'file'};
			$newv[$i]->{'line'} = $_[0]->{'line'}+1;
			$newv[$i]->{'eline'} =
				$_[0]->{'line'} + scalar(@nl);
			$self->renumber($parent, $_[0]->{'line'},
				  $_[0]->{'file'}, scalar(@nl));
			}
		splice(@$pm, 0, 0, $newv[$i]);
		}
	elsif ($i >= @newv) {
		# a directive was deleted
		if (!$_[4]) {
			local $parent = $self->get_config_parent($oldv[$i]->{'file'});
			$lref = $self->read_file_lines(
					$self->make_chroot($oldv[$i]->{'file'}));
			$self->{'ol'} = $oldv[$i]->{'eline'} - $oldv[$i]->{'line'} + 1;
			splice(@$lref, $oldv[$i]->{'line'}, $self->{'ol'});
			$self->renumber($parent, $oldv[$i]->{'eline'},
				  $oldv[$i]->{'file'}, -$self->{'ol'});
			}
		splice(@$pm, $self->indexof($oldv[$i], @$pm), 1);
		}
	else {
		# updating some directive
		if (!$_[4]) {
			local $parent = $self->get_config_parent($oldv[$i]->{'file'});
			$lref = $self->read_file_lines(
					$self->make_chroot($oldv[$i]->{'file'}));
			@nl = $self->directive_lines($newv[$i], $_[3]);
			$self->{'ol'} = $oldv[$i]->{'eline'} - $oldv[$i]->{'line'} + 1;
			splice(@$lref, $oldv[$i]->{'line'}, $self->{'ol'}, @nl);
			$newv[$i]->{'file'} = $_[0]->{'file'};
			$newv[$i]->{'line'} = $oldv[$i]->{'line'};
			$newv[$i]->{'eline'} =
				$oldv[$i]->{'line'} + scalar(@nl) - 1;
			$self->renumber($parent, $oldv[$i]->{'eline'},
				  $oldv[$i]->{'file'}, scalar(@nl) - $self->{'ol'});
			}
		$pm->[$self->indexof($oldv[$i], @$pm)] = $newv[$i];
		}
	}
}

# $self->directive_lines(&directive, tabs)
# Renders some directive into a number of lines of text
sub directive_lines
{
	my $self = shift;

local(@rv, $v, $m, $i);
$rv[0] = "\t" x $_[1];
$rv[0] .= "$_[0]->{'name'}";
foreach $v (@{$_[0]->{'values'}}) {
	if ($self->{'need_quote_hash'}->{$_[0]->{'name'}} && !$i) { $rv[0] .= " \"$v\""; }
	else { $rv[0] .= " $v"; }
	$i++;
	}
if ($_[0]->{'type'} == 1) {
	# multiple values.. include them as well
	$rv[0] .= " {";
	foreach $m (@{$_[0]->{'members'}}) {
		push(@rv, $self->directive_lines($m, $_[1]+1));
		}
	push(@rv, ("\t" x ($_[1]+1))."}");
	}
elsif ($_[0]->{'type'} == 2) {
	# named sub-structures .. include them too
	foreach my $sn (sort { $a cmp $b } (keys %{$_[0]->{'members'}})) {
		$rv[0] .= " ".$sn." {";
		foreach $m (@{$_[0]->{'members'}->{$sn}}) {
			$rv[0] .= " ".join(" ", $self->directive_lines($m, 0));
			}
		$rv[0] .= " }";
		}
	}
$rv[$#rv] .= ";";
return @rv;
}

# $self->renumber(&parent, line, file, count)
# Runs through the given array of directives and increases the line numbers
# of all those greater than some line by the given count
sub renumber
{
	my $self = shift;

if ($_[0]->{'file'} eq $_[2]) {
	if ($_[0]->{'line'} > $_[1]) { $_[0]->{'line'} += $_[3]; }
	if ($_[0]->{'eline'} > $_[1]) { $_[0]->{'eline'} += $_[3]; }
	}
if ($_[0]->{'type'} == 1) {
	# Do sub-members
	local $d;
	foreach $d (@{$_[0]->{'members'}}) {
		$self->renumber($d, $_[1], $_[2], $_[3]);
		}
	}
elsif ($_[0]->{'type'} == 2) {
	# Do sub-members
	local ($sm, $d);
	foreach $sm (keys %{$_[0]->{'members'}}) {
		foreach $d (@{$_[0]->{'members'}->{$sm}}) {
			$self->renumber($d, $_[1], $_[2], $_[3]);
			}
		}
	}
}

# choice_input(text, name, &config, [display, option]+)
sub choice_input
{
	my $self = shift;

local($rv, $v, $i, @ops);
$rv = "<td valign=top><b>$_[0]</b></td> <td valign=top>";
$v = $self->find_value($_[1], $_[2]);
for($i=3; $i<@_; $i+=2) {
	$rv .= sprintf "<input type=radio name=%s value='%s' %s> %s\n",
		$_[1], $_[$i+1], $v eq $_[$i+1] ? "checked" : "", $_[$i];
	}
return $rv."</td>\n";
}

# save_choice(name, &parent, indent)
sub save_choice
{
	my $self = shift;

local($nd);
if ($self->{'in_hash'}->{$_[0]}) { $nd = { 'name' => $_[0], 'values' => [ $self->{'in_hash'}->{$_[0]} ] }; }
$self->save_directive($_[1], $_[0], $nd ? [ $nd ] : [ ], $_[2]);
}

# addr_match_input(text, name, &config)
# A field for editing a list of addresses, ACLs and partial IP addresses
sub addr_match_input
{
	my $self = shift;

local($v, $rv, $av, @av);
$v = $self->find($_[1], $_[2]);
$rv = "<td valign=top><b>$_[0]</b></td> <td valign=top>";
$rv .= "<input type=radio name=$_[1]_def value=1 ".
       ($v ? "" : "checked")."> $self->{'text_hash'}->{'default'}";
$rv .= "<input type=radio name=$_[1]_def value=0 ".
       ($v ? "checked" : "")."> $self->{'text_hash'}->{'listed'}<br>";
foreach $av (@{$v->{'members'}}) { push(@av, $av->{'name'}); }
$rv .= "<textarea name=$_[1] rows=3 cols=15>".
	join("\n", @av)."</textarea></td>\n";
}

# save_addr_match(name, &parent, indent)
sub save_addr_match
{
	my $self = shift;

local($addr, @vals, $dir);
if ($self->{'in_hash'}->{"$_[0]_def"}) { $self->save_directive($_[1], $_[0], [ ], $_[2]); }
else {
	foreach $addr (split(/\s+/, $self->{'in_hash'}->{$_[0]})) {
		push(@vals, { 'name' => $addr });
		}
	$dir = { 'name' => $_[0], 'type' => 1, 'members' => \@vals };
	$self->save_directive($_[1], $_[0], [ $dir ], $_[2]);
	}
}

# address_input(text, name, &config, type)
sub address_input
{
	my $self = shift;

local($v, $rv, $av, @av);
$v = $self->find($_[1], $_[2]);
foreach $av (@{$v->{'members'}}) {
	push(@av, join(" ", $av->{'name'}, @{$av->{'values'}}));
	}
if ($_[3] == 0) {
	# text area
	$rv = "<td valign=top><b>$_[0]</b></td> <td valign=top>";
	$rv .= "<textarea name=$_[1] rows=3 cols=15>".
		join("\n", @av)."</textarea></td>\n";
	}
else {
	$rv = "<td valign=top><b>$_[0]</b></td> <td colspan=3 valign=top>";
	$rv .= "<input name=$_[1] size=50 value=\"".join(' ',@av)."\"></td>\n";
	}
return $rv;
}

# save_port_address(name, portname, &config, indent)
sub save_port_address {
	my $self = shift;

  local($addr, $port, @vals, $dir);
  foreach $addr (split(/\s+/, $self->{'in_hash'}->{$_[0]})) {
    $addr =~ /^\S+$/ || $self->error($self->text('eipacl', $addr));
    push(@vals, { 'name' => $addr });
  }
  $dir = { 'name' => $_[0], 'type' => 1, 'members' => \@vals };
  ($self->{'n'} = $_[1]) =~ s/[^A-Za-z0-9_]/_/g;
  $dir->{'values'} = [ $_[1], $self->{'in_hash'}->{$_[1]} ] if (!$self->{'in_hash'}->{"$self->{'n'}_def"});
  $self->save_directive($_[2], $_[0], @vals ? [ $dir ] : [ ], $_[3]);
}

# save_address(name, &parent, indent, ips-only)
sub save_address
{
	my $self = shift;

local ($addr, @vals, $dir, $i);
local @sp = split(/\s+/, $self->{'in_hash'}->{$_[0]});
for($i=0; $i<@sp; $i++) {
	!$_[3] || $self->check_ipaddress($sp[$i]) || $self->error($self->text('eip', $sp[$i]));
	if (lc($sp[$i]) eq "key") {
		push(@vals, { 'name' => $sp[$i],
			      'values' => [ $sp[++$i] ] });
		}
	else {
		push(@vals, { 'name' => $sp[$i] });
		}
	}
$dir = { 'name' => $_[0], 'type' => 1, 'members' => \@vals };
$self->save_directive($_[1], $_[0], @vals ? [ $dir ] : [ ], $_[2]);
}

# forwarders_input(text, name, &config)
sub forwarders_input
{
	my $self = shift;

local($v, $rv, $av, @ips, @prs);
$v = $self->find($_[1], $_[2]);
foreach $av (@{$v->{'members'}}) {
	push(@ips, $av->{'name'});
	if ($av->{'values'}->[0] eq 'port') {
		push(@prs, $av->{'values'}->[1]);
		}
	else {
		push(@prs, undef);
		}
	}
$rv = "<td valign=top><b>$_[0]</b></td> <td valign=top>\n";
$rv .= "<table border>\n";
$rv .= "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'forwarding_ip'}</b></td> ".
       "<td><b>$self->{'text_hash'}->{'forwarding_port'}</b></td> </tr>\n";
for($i=0; $i<@ips+3; $i++) {
	$rv .= "<tr $self->{'cb'}>\n";
	$rv .= "<td><input name=$_[1]_ip_$i size=20 value='$ips[$i]'></td>\n";
	$rv .= "<td><input name=$_[1]_pr_$i size=15 value='$prs[$i]'></td>\n";
	$rv .= "</tr>\n";
	}
$rv .= "</table>\n";
return $rv;
}

# save_forwarders(name, &parent, indent)
sub save_forwarders
{
	my $self = shift;

local ($i, $ip, $pr, @vals);
for($i=0; defined($ip = $self->{'in_hash'}->{"$_[0]_ip_$i"}); $i++) {
	next if (!$ip);
	$self->check_ipaddress($ip) || $self->error($self->text('eip', $ip));
	$pr = $self->{'in_hash'}->{"$_[0]_pr_$i"};
	!$pr || $pr =~ /^\d+$/ || $self->error($self->text('eport', $pr));
	push(@vals, { 'name' => $ip,
		      'values' => $pr ? [ "port", $pr ] : [ ] });
	}
local $dir = { 'name' => $_[0], 'type' => 1, 'members' => \@vals };
$self->save_directive($_[1], $_[0], @vals ? [ $dir ] : [ ], $_[2]);
}

# opt_input(text, name, &config, default, size, units)
sub opt_input
{
	my $self = shift;

local($v, $rv, $n);
$v = $self->find($_[1], $_[2]);
($n = $_[1]) =~ s/[^A-Za-z0-9_]/_/g;
$rv = "<td valign=top><b>$_[0]</b></td> <td nowrap valign=top";
$rv .= $_[4] > 30 ? " colspan=3>\n" : ">\n";
$rv .= sprintf "<input type=radio name=${n}_def value=1 %s> $_[3]\n",
	$v ? "" : "checked";
$rv .= sprintf "<input type=radio name=${n}_def value=0 %s> ",
	$v ? "checked" : "";
$rv .= sprintf "<input name=$n size=$_[4] value=\"%s\"> $_[5]</td>\n",
	$v ? $v->{'value'} : "";
return $rv;
}

sub save_opt
{
	my $self = shift;

local($dir, $n);
($n = $_[0]) =~ s/[^A-Za-z0-9_]/_/g;
if ($self->{'in_hash'}->{"${n}_def"}) { $self->save_directive($_[2], $_[0], [ ], $_[3]); }
elsif ($self->{'err'} = &{$_[1]}($self->{'in_hash'}->{$n})) {
	$self->error($self->{'err'});
	}
else {
	$dir = { 'name' => $_[0], 'values' => [ $self->{'in_hash'}->{$n} ] };
	$self->save_directive($_[2], $_[0], [ $dir ], $_[3]);
	}
}

# find_reverse(address, [view])
# Returns the zone and record structures for the PTR record for some address
sub find_reverse
{
	my $self = shift;

local($conf, @zl, $rev, $z, $revconf, $revfile, $revrec, @revrecs, $addr, $rr,
      @octs, $i, @hexs, $ipv6, @zero);

# find reverse domain
local @zl = grep { $_->{'type'} ne 'view' } $self->list_zone_names();
if ($_[1] ne '') {
	@zl = grep { $_->{'view'} && $_->{'viewindex'} == $_[1] } @zl;
	}
else {
	@zl = grep { !$_->{'view'} } @zl;
	}
$ipv6 = $self->{'config_hash'}->{'support_aaaa'} && $self->check_ip6address($_[0]);
if ($ipv6) {
	@zero = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
	$addr = $self->expandall_ip6($_[0]);
	$addr =~ s/://g;
	@hexs = split('', $addr);
	DOMAIN: for($i=30; $i>=0; $i--) {
		$addr = join(':',split(/(.{4})/,join('', (@hexs[0..$i],@zero[$i..30]))));
		$addr =~ s/::/:/g;
		$addr =~ s/(^:|:$)//g;
		$rev = $self->net_to_ip6int($addr, 4*($i+1));
		$rev =~ s/\.$//g;
		foreach $z (@zl) {
			if (lc($z->{'name'}) eq $rev && $z->{'type'} eq 'master') {
				# found the reverse master domain
				$revconf = $z;
				last DOMAIN;
				}
			}
		}
	}
else {
	@octs = split(/\./, $_[0]);
	DOMAIN: for($i=2; $i>=-1; $i--) {
		$rev = $i<0 ? "in-addr.arpa"
			    : $self->ip_to_arpa(join('.', @octs[0..$i]));
		$rev =~ s/\.$//g;
		foreach $z (@zl) {
			if ((lc($z->{'name'}) eq $rev ||
			     lc($z->{'name'}) eq "$rev.") &&
			    $z->{'type'} eq "master") {
				# found the reverse master domain
				$revconf = $z;
				last DOMAIN;
				}
			}
		}
	}

# find reverse record
if ($revconf) {
	$revfile = $self->absolute_path($revconf->{'file'});
	@revrecs = $self->read_zone_file($revfile, $revconf->{'name'});
	if ($ipv6) {
		$addr = $self->net_to_ip6int($_[0], 128);
		}
	else {
		$addr = $self->ip_to_arpa($_[0]);
		}
	foreach $rr (@revrecs) {
		if ($rr->{'type'} eq "PTR" &&
		    lc($rr->{'name'}) eq lc($addr)) {
			# found the reverse record
			$revrec = $rr;
			last;
			}
		}
	}
return ($revconf, $revfile, $revrec);
}

# find_forward(address, [view])
# Returns the zone and record structures for the A record for some address
sub find_forward
{
	my $self = shift;

local ($fwdconf, $i, $fwdfile, $fwdrec, $fr, $ipv6);

# find forward domain
local $host = $_[0]; $host =~ s/\.$//;
local @zl = grep { $_->{'type'} ne 'view' } $self->list_zone_names();
if ($_[1] ne '') {
	@zl = grep { $_->{'view'} && $_->{'viewindex'} == $_[1] } @zl;
	}
else {
	@zl = grep { !$_->{'view'} } @zl;
	}
local @parts = split(/\./, $host);
DOMAIN: for($i=1; $i<@parts; $i++) {
	local $fwd = join(".", @parts[$i .. @parts-1]);
	foreach $z (@zl) {
		local $typed;
		if ((lc($z->{'name'}) eq $fwd ||
		     lc($z->{'name'}) eq "$fwd.") &&
		    $z->{'type'} eq "master") {
			# Found the forward master!
			$fwdconf = $z;
			last DOMAIN;
			}
		}
	}

# find forward record
if ($fwdconf) {
	$fwdfile = $self->absolute_path($fwdconf->{'file'});
	local @fwdrecs = $self->read_zone_file($fwdfile, $fwdconf->{'name'});
	foreach $fr (@fwdrecs) {
		if ($ipv6 ? $fr->{'type'} eq "AAAA" : $fr->{'type'} eq "A" &&
		    $fr->{'name'} eq $_[0]) {
			# found the forward record!
			$fwdrec = $fr;
			last;
			}
		}
	}

return ($fwdconf, $fwdfile, $fwdrec);
}

# can_edit_zone(&zone, [&view] | &cachedzone)
# Returns 1 if some zone can be edited
sub can_edit_zone
{
	my $self = shift;

local %zcan;
local ($zn, $vn, $file);
if ($_[0]->{'members'}) {
	# A full zone structure
	$zn = $_[0]->{'value'};
	$vn = $_[1] ? 'view_'.$_[1]->{'value'} : undef;
	$file = $self->find_value("file", $_[0]->{'members'});
	}
else {
	# A cached zone object
	$zn = $_[0]->{'name'};
	$vn = $_[0]->{'view'} eq '*' ? undef : $_[0]->{'view'};
	$file = $_[0]->{'file'};
	}

# Check zone name
if ($self->{'access_hash'}->{'zones'} eq '*') {
	# Always can
	}
elsif ($self->{'access_hash'}->{'zones'} =~ /^\!/) {
	# List of denied zones
	foreach (split(/\s+/, $self->{'access_hash'}->{'zones'})) {
		return 0 if ($_ eq $zn || ($vn && $_ eq $vn));
		}
	}
else {
	# List of allowed zones
	local $ok;
	foreach (split(/\s+/, $self->{'access_hash'}->{'zones'})) {
		$ok++ if ($_ eq $zn || ($vn && $_ eq $vn));
		}
	return 0 if (!$ok);
	}

if ($self->{'access_hash'}->{'dironly'}) {
	# Check directory access control 
	return 1 if (!$file);
	$file = $self->absolute_path($file);
	return 0 if (!$self->allowed_zone_file(\%{$self->{'access_hash'}}, $file));
	}
return 1;
}

# can_edit_reverse(&zone)
sub can_edit_reverse
{
	my $self = shift;

return $self->{'access_hash'}->{'reverse'} || $self->can_edit_zone($_[0]);
}

# record_input(zoneindex, view, type, file, origin, [num], [record], [dom])
# Display a form for editing or creating a DNS record
sub record_input
{
	my $self = shift;

local(%rec, @recs, $ttl, $ttlunit);
local $type = $_[6] ? $_[6]->{'type'} : $_[2];
print "<form action=save_record.cgi>\n";
print "<input type=hidden name=index value='$_[0]'>\n";
print "<input type=hidden name=view value='$_[1]'>\n";
print "<input type=hidden name=file value='$_[3]'>\n";
print "<input type=hidden name=origin value='$_[4]'>\n";
print "<input type=hidden name=sort value='$self->{'in_hash'}->{'sort'}'>\n";
print "<input type=hidden name=dom value=$_[7]>\n" if ($_[7]);
if (@_ >= 6) {
	print "<input type=hidden name=num value=$_[5]>\n";
	%rec = %{$_[6]};
	}
else { print "<input type=hidden name=new value=1>\n"; }
print "<input type=hidden name=type value='$type'>\n";
print "<input type=hidden name=redirtype value='$_[2]'>\n";
print "<table border>\n";
print "<tr $self->{'tb'}><td><b>",$self->text(@_ >= 6 ? 'edit_edit' : 'edit_add',
			      $self->{'text_hash'}->{"edit_".$type}),"</b></td> </tr>\n";
print "<tr $self->{'cb'}><td><table>\n";

if ($type eq "PTR") {
	print "<tr> <td><b>$self->{'text_hash'}->{'edit_addr'}</b></td>\n";
	printf "<td><input name=name value=\"%s\" size=30></td>\n",
		!%rec && $_[4] =~ /^(\d+)\.(\d+)\.(\d+)\.in-addr/ ?
			"$3.$2.$1." : $self->ip6int_to_net($self->arpa_to_ip($rec{'name'}));
	}
elsif ($type eq "NS") {
	print "<tr> <td><b>$self->{'text_hash'}->{'edit_zonename'}</b></td>\n";
	print "<td><input name=name value=\"$rec{'name'}\" size=30></td>\n";
	}
elsif ($type eq "SRV") {
	local ($serv, $proto, $name) =
		$rec{'name'} =~ /^([^\.]+)\.([^\.]+)\.(\S+)/ ? ($1, $2, $3) :
			(undef, undef, undef);
	$serv =~ s/^_//;
	$proto =~ s/^_//;
	print "<tr> <td><b>$self->{'text_hash'}->{'edit_name'}</b></td>\n";
	print "<td><input name=name value=\"$name\" size=30></td>\n";

	print "<td><b>$self->{'text_hash'}->{'edit_proto'}</b></td>\n";
	print "<td>",$self->ui_select("proto", $proto,
				[ [ "tcp", "TCP" ],
				  [ "udp", "UDP" ],
				  [ "tls", "TLS" ] ], undef, undef, 1),
	      "</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'edit_serv'}</b></td>\n";
	print "<td><input name=serv value=\"$serv\" size=20></td>\n";
	}
else {
	print "<tr> <td><b>$self->{'text_hash'}->{'edit_name'}</b></td>\n";
	print "<td><input name=name value=\"$rec{'name'}\" size=30></td>\n";
	}
if ($rec{'ttl'} =~ /^(\d+)([SMHDW]?)$/i) {
	$ttl = $1; $ttlunit = $2;
	}
else {
	$ttl = $rec{'ttl'}; $ttlunit = "";
	}
print "<td><b>$self->{'text_hash'}->{'edit_ttl'}</b></td> <td nowrap>\n";
printf "<input type=radio name=ttl_def value=1 %s> $self->{'text_hash'}->{'default'}\n",
	defined($rec{'ttl'}) ? "" : "checked";
printf "<input type=radio name=ttl_def value=0 %s>\n",
	defined($rec{'ttl'}) ? "checked" : "";
print "<input name=ttl size=8 value=\"$ttl\">\n";
print $self->time_unit_choice("ttlunit", $ttlunit);
print "</td> </tr>\n";

@{$self->{'v_array'}} = @{$rec{'values'}};
if ($type eq "A" || $type eq "AAAA") {
	print "<tr> <td><b>$self->{'text_hash'}->{'value_A1'}</b></td>\n";
	print "<td><input name=value0 size=20 value=\"$self->{'v_array'}->[0]\">\n";
	if (@_ < 6 && $type eq "A") {
		print $self->free_address_button("value0");
		}
	print "</td> </tr>\n";
	if (@_ >= 6) {
		print "<input type=hidden name=oldname ",
		      "value=\"$rec{'name'}\">\n";
		print "<input type=hidden name=oldvalue0 value=\"$self->{'v_array'}->[0]\">\n";
		}
	}
elsif ($type eq "NS") {
	print "<tr> <td><b>$self->{'text_hash'}->{'value_NS1'}</b></td>\n";
	print "<td colspan=3><input name=value0 size=30 value=\"$self->{'v_array'}->[0]\">\n";
	print "($self->{'text_hash'}->{'edit_cnamemsg'})</td> </tr>\n";
	}
elsif ($type eq "CNAME") {
	print "<tr> <td><b>$self->{'text_hash'}->{'value_CNAME1'}</b></td>\n";
	print "<td colspan=3><input name=value0 size=30 value=\"$self->{'v_array'}->[0]\">\n";
	print "($self->{'text_hash'}->{'edit_cnamemsg'})</td> </tr>\n";
	}
elsif ($type eq "MX") {
	print "<tr> <td><b>$self->{'text_hash'}->{'value_MX2'}</b></td>\n";
	print "<td><input name=value1 size=30 value=\"$self->{'v_array'}->[1]\"></td>\n";
	print "<td><b>$self->{'text_hash'}->{'value_MX1'}</b></td>\n";
	print "<td><input name=value0 size=8 value=\"$self->{'v_array'}->[0]\"></td> </tr>\n";
	}
elsif ($type eq "HINFO") {
	print "<tr> <td><b>$self->{'text_hash'}->{'value_HINFO1'}</b></td>\n";
	print "<td><input name=value0 size=20 value=\"$self->{'v_array'}->[0]\"></td>\n";
	print "<td><b>$self->{'text_hash'}->{'value_HINFO2'}</b></td>\n";
	print "<td><input name=value1 size=20 value=\"$self->{'v_array'}->[1]\"></td> </tr>\n";
	}
elsif ($type eq "TXT") {
	print "<tr> <td><b>$self->{'text_hash'}->{'value_TXT1'}</b></td>\n";
	print "<td><input name=value0 size=30 value=\"",
		$self->html_escape($self->{'v_array'}->[0]),"\"></td> </tr>\n";
	}
elsif ($type eq "WKS") {
	print "<tr> <td><b>$self->{'text_hash'}->{'value_WKS1'}</b></td>\n";
	print "<td><input name=value0 size=15 value=\"$self->{'v_array'}->[0]\"></td>\n";
	print "<td><b>$self->{'text_hash'}->{'value_WKS2'}</b></td>\n";
	print "<td><select name=value1>\n";
	printf "<option %s>TCP\n", $self->{'v_array'}->[1] =~ /tcp/ ? "selected" : "";
	printf "<option %s>UDP\n", $self->{'v_array'}->[1] =~ /udp/ ? "selected" : "";
	print "</select></td>\n";
	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'value_WKS3'}</b></td>\n";
	print "<td><textarea name=value2 rows=3 cols=20 wrap>",
		join(' ', @{$self->{'v_array'}}[2..$#{$self->{'v_array'}}]),"</textarea></td> </tr>\n";
	}
elsif ($type eq "RP") {
	print "<tr> <td><b>$self->{'text_hash'}->{'value_RP1'}</b></td>\n";
	$self->{'v_array'}->[0] = $self->dotted_to_email($self->{'v_array'}->[0]);
	print "<td><input name=value0 size=20 value=\"$self->{'v_array'}->[0]\"></td>\n";
	print "<td><b>$self->{'text_hash'}->{'value_RP2'}</b></td>\n";
	print "<td><input name=value1 size=30 value=\"$self->{'v_array'}->[1]\"></td> </tr>\n";
	}
elsif ($type eq "PTR") {
	print "<tr> <td><b>$self->{'text_hash'}->{'value_PTR1'}</b></td>\n";
	print "<td><input name=value0 size=30 value=\"$self->{'v_array'}->[0]\"></td> </tr>\n";
	if (@_ >= 6) {
		print "<input type=hidden name=oldname ",
		      "value=\"$rec{'name'}\">\n";
		print "<input type=hidden name=oldvalue0 value=\"$self->{'v_array'}->[0]\">\n";
		}
	}
elsif ($type eq "SRV") {
	print "<tr> <td><b>$self->{'text_hash'}->{'value_SRV1'}</b></td>\n";
	print "<td><input name=value0 size=8 value='$self->{'v_array'}->[0]'></td>\n";

	print "<td><b>$self->{'text_hash'}->{'value_SRV2'}</b></td>\n";
	print "<td><input name=value1 size=8 value='$self->{'v_array'}->[1]'></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'value_SRV3'}</b></td>\n";
	print "<td><input name=value2 size=8 value='$self->{'v_array'}->[2]'></td>\n";

	print "<td><b>$self->{'text_hash'}->{'value_SRV4'}</b></td>\n";
	print "<td><input name=value3 size=30 value='$self->{'v_array'}->[3]'></td> </tr>\n";
	}
elsif ($type eq "LOC") {
	print "<tr> <td><b>$self->{'text_hash'}->{'value_LOC1'}</b></td>\n";
	printf "<td colspan=3><input name=value0 size=45 value=\"%s\"></td> </tr>\n", join(" ", @{$self->{'v_array'}});
	}
elsif ($type eq "KEY") {
	print "<tr> <td><b>$self->{'text_hash'}->{'value_KEY1'}</b></td>\n";
	print "<td><input name=value0 size=8 value='$self->{'v_array'}->[0]'></td>\n";

	print "<td><b>$self->{'text_hash'}->{'value_KEY2'}</b></td>\n";
	print "<td><input name=value1 size=8 value='$self->{'v_array'}->[1]'></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'value_KEY3'}</b></td>\n";
	print "<td><input name=value2 size=8 value='$self->{'v_array'}->[2]'></td> </tr>\n";

	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'value_KEY4'}</b></td>\n";
	print "<td colspan=3><textarea name=value3 rows=5 cols=80>";
	print join("\n", $self->wrap_lines($self->{'v_array'}->[3], 80));
	print "</textarea></td> </tr>\n";
	}
elsif ($type eq "SPF") {
	# SPF records are complex, as they have several attributes encoded
	# in the TXT value
	local $spf = $self->parse_spf($self->{'v_array'}->[0]);
	print "<tr> <td><b>$self->{'text_hash'}->{'value_spfa'}</b></td>\n";
	print "<td colspan=3>",$self->ui_yesno_radio("spfa", $spf->{'a'} ? 1 : 0),
	      "</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'value_spfmx'}</b></td>\n";
	print "<td colspan=3>",$self->ui_yesno_radio("spfmx", $spf->{'mx'} ? 1 : 0),
	      "</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'value_spfptr'}</b></td>\n";
	print "<td colspan=3>",$self->ui_yesno_radio("spfptr", $spf->{'ptr'} ? 1 : 0),
	      "</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'value_spfas'}</b></td>\n";
	print "<td colspan=3>",
		$self->ui_textarea("spfas", join("\n", @{$spf->{'a:'}}),
		  	     3, 40),"</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'value_spfmxs'}</b></td>\n";
	print "<td colspan=3>",
		$self->ui_textarea("spfmxs", join("\n", @{$spf->{'mx:'}}),
		  	     3, 40),"</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'value_spfip4s'}</b></td>\n";
	print "<td colspan=3>",
		$self->ui_textarea("spfip4s", join("\n", @{$spf->{'ip4:'}}),
		  	     3, 40),"</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'value_spfincludes'}</b></td>\n";
	print "<td colspan=3>",
		$self->ui_textarea("spfincludes", join("\n", @{$spf->{'include:'}}),
		  	     3, 40),"</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'value_spfall'}</b></td>\n";
	print "<td colspan=3>",$self->ui_select("spfall", int($spf->{'all'}),
			[ [ 3, $self->{'text_hash'}->{'value_spfall3'} ],
			  [ 2, $self->{'text_hash'}->{'value_spfall2'} ],
			  [ 1, $self->{'text_hash'}->{'value_spfall1'} ],
			  [ 0, $self->{'text_hash'}->{'value_spfall0'} ],
			  [ undef, $self->{'text_hash'}->{'value_spfalldef'} ] ]),"</td></tr>\n";
	}
else {
	# All other types just have a text box
	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'value_other'}</b></td>\n";
	print "<td colspan=3><textarea name=values rows=3 cols=40>",
		join("\n", @{$self->{'v_array'}}),"</textarea></td> </tr>\n";
	}
if ($type ne "WKS") {
	printf "<tr> <td><b>%s</b></td>\n",
		$self->{'config_hash'}->{'allow_comments'} ? $self->{'text_hash'}->{'edit_comment'} : "";
	printf "<td><input %s name=comment size=30 value=\"%s\"></td> </tr>\n",
		$self->{'config_hash'}->{'allow_comments'} ? "" : "type=hidden", $rec{'comment'};
	}
if ($type eq "A" || $type eq "AAAA") {
	print "<tr> <td><b>$self->{'text_hash'}->{'edit_uprev'}</b></td>\n";
	printf "<td><input type=radio name=rev value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		$self->{'config_hash'}->{'rev_def'} == 0 ? 'checked' : "";
	if (@_ < 6) {
		printf "<input type=radio name=rev value=2 %s> %s\n",
			$self->{'config_hash'}->{'rev_def'} == 2 ? 'checked' : '',
			$self->{'text_hash'}->{'edit_over'};
		}
	printf "<input type=radio name=rev value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		$self->{'config_hash'}->{'rev_def'} == 1 ? 'checked' : '';
	}
elsif ($type eq "PTR") {
	print "<tr> <td><b>$self->{'text_hash'}->{'edit_upfwd'}</b></td>\n";
	printf "<td><input type=radio name=fwd value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		$self->{'config_hash'}->{'rev_def'} ? '' : 'checked';
	printf "<input type=radio name=fwd value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		$self->{'config_hash'}->{'rev_def'} ? 'checked' : '';
	}
else { print "<tr> <td colspan=2></td>\n"; }
print "<td colspan=2 align=right>\n";
if (!$self->{'access_hash'}->{'ro'}) {
	if (@_ >= 6) {
		print "<input type=submit value=\"$self->{'text_hash'}->{'save'}\">\n";
		print "<input type=submit name=delete ",
		      "value=\"$self->{'text_hash'}->{'delete'}\">\n";
		}
	else { print "<input type=submit value=\"$self->{'text_hash'}->{'create'}\">\n"; }
	}
print "</td></tr></table></td></tr></table></form>\n";
}

# zones_table(&links, &titles, &types, &deletes)
# Prints a table of zones, with checkboxes to delete
sub zones_table
{
	my $self = shift;

local($i);
local @tds = ( "width=5" );
print $self->ui_columns_start([ "", $self->{'text_hash'}->{'index_zone'}, $self->{'text_hash'}->{'index_type'} ],
			100, 0, \@tds);
for($i=0; $i<@{$_[0]}; $i++) {
	local @cols = ( "<a href=\"$_[0]->[$i]\">$_[1]->[$i]</a>",
			$_[2]->[$i] );
	if (defined($_[3]->[$i])) {
		print $self->ui_checked_columns_row(\@cols, \@tds, "d", $_[3]->[$i]);
		}
	else {
		print $self->ui_columns_row(\@cols, \@tds);
		}
	}
print $self->ui_columns_end();
}

# convert_illegal(text)
# Convert text containing special HTML characters to properly display it.
sub convert_illegal
{
	my $self = shift;

$_[0] =~ s/&/&amp;/g;
$_[0] =~ s/>/&gt;/g;
$_[0] =~ s/</&lt;/g;
$_[0] =~ s/"/&quot;/g;
$_[0] =~ s/ /&nbsp;/g;
return $_[0];
}

sub check_net_ip
{
	my $self = shift;

local($j, $arg = $_[0]);
if ($arg !~ /^(\d{1,3}\.){0,3}([0-9\-\/]+)$/) {
	return 0;
	}
foreach $j (split(/\./, $arg)) {
	$j =~ /^(\d+)-(\d+)$/ && $1 < 255 && $2 < 255 ||
	$j =~ /^(\d+)\/(\d+)$/ && $1 < 255 && $2 <= 32 ||
		$j <= 255 || return 0;
	}
return 1;
}

# expand_ip6(ip)
# Transform compact (with ::) IPv6 address to the unique expanded form
# (without :: and leading zeroes in all parts) 
sub expand_ip6
{
	my $self = shift;

local($n);
for($n = 6 - ($_[0] =~ s/([^:]):(?=[^:])/$1:/g); $n > 0; $n--) {
	$_[0] =~ s/::/:0::/;
	}
$_[0] =~ s/::/:/;
$_[0] =~ s/^:/0:/;
$_[0] =~ s/:$/:0/;
$_[0] =~ s/(:|^)0(?=\w)/$1/;
$_[0] =~ tr/[A-Z]/[a-z]/;
return $_[0];
}

# expandall_ip6(ip)
# Transform IPv6 address to the expanded form containing all internal 0's 
sub expandall_ip6
{
	my $self = shift;

$self->expand_ip6($_[0]);
$_[0] =~ s/(:|^)(\w{3})(?=:|$)/:0$2/g;
$_[0] =~ s/(:|^)(\w{2})(?=:|$)/:00$2/g;
$_[0] =~ s/(:|^)(\w)(?=:|$)/:000$2/g;
return $_[0];
}

# check_ip6address(ip)
# Check if some IPv6 address is properly formatted
sub check_ip6address
{
	my $self = shift;

local($ip6);
$ip6 = $_[0];
$ip6 = $self->expand_ip6($ip6);
return ($ip6 =~ /^([\da-f]{1,4}:){7}([\da-f]{1,4})$/i);
}

sub time_unit_choice 
{
	my $self = shift;

local ($name, $value) = @_;
return $self->ui_select($name, $value =~ /^(S?)$/i ? "" :
			 $value =~ /M/i ? "M" :
			 $value =~ /H/i ? "H" :
			 $value =~ /D/i ? "D" :
			 $value =~ /W/i ? "W" : $value,
		  [ [ "", $self->{'text_hash'}->{'seconds'} ],
		    [ "M", $self->{'text_hash'}->{'minutes'} ],
		    [ "H", $self->{'text_hash'}->{'hours'} ],
		    [ "D", $self->{'text_hash'}->{'days'} ],
		    [ "W", $self->{'text_hash'}->{'weeks'} ] ], 1, 0, 1);
}

sub extract_time_units
{
	my $self = shift;

local(@ret);
foreach $j (@_) {
	if ($j =~ /^(\d+)([SMHDW]?)$/is) {
		push(@ret, $2); $j = $1;
		}
	}
return @ret;
}

sub email_to_dotted
{
	my $self = shift;

local $v = $_[0];
$v =~ s/\.$//;
if ($v =~ /^([^.]+)\@(.*)$/) {
	return "$1.$2.";
	}
elsif ($v =~ /^(.*)\@(.*)$/) {
	local ($u, $d) = ($1, $2);
	$u =~ s/\./\\\./g;
	return "\"$u.$d.\"";
	}
else {
	return $v;
	}
}

sub dotted_to_email
{
	my $self = shift;

local $v = $_[0];
if ($v ne ".") {
	$v =~ s/([^\\])\./$1\@/;
	$v =~ s/\\\./\./g;
	$v =~ s/\.$//;
	}
return $v;
}

# set_ownership(file)
# Sets the BIND ownership and permissions on some file
sub set_ownership
{
	my $self = shift;

local ($user, $group, $perms);
if ($self->{'config_hash'}->{'file_owner'}) {
	($user, $group) = split(/:/, $self->{'config_hash'}->{'file_owner'});
	}
if ($self->{'config_hash'}->{'file_perms'}) {
	$perms = oct($self->{'config_hash'}->{'file_perms'});
	}
$self->set_ownership_permissions($user, $group, $perms, $_[0]);
}

# can_edit_view(&view | &viewcache)
# Returns 1 if some view can be edited
sub can_edit_view
{
	my $self = shift;

local %vcan;
local $vn = $_[0]->{'members'} ? $_[0]->{'value'} : $_[0]->{'name'};

if ($self->{'access_hash'}->{'vlist'} eq '*') {
	return 1;
	}
elsif ($self->{'access_hash'}->{'vlist'} =~ /^\!/) {
	foreach (split(/\s+/, $self->{'access_hash'}->{'vlist'})) {
		return 0 if ($_ eq $vn);
		}
	return 1;
	}
else {
	foreach (split(/\s+/, $self->{'access_hash'}->{'vlist'})) {
		return 1 if ($_ eq $vn);
		}
	return 0;
	}
}

# wrap_lines(text, width)
# Given a multi-line string, return an array of lines wrapped to
# the given width
sub wrap_lines
{
	my $self = shift;

local $rest = $_[0];
local @rv;
while(length($rest) > $_[1]) {
	push(@rv, substr($rest, 0, $_[1]));
	$rest = substr($rest, $_[1]);
	}
push(@rv, $rest) if ($rest ne '');
return @rv;
}

# add_zone_access(domain)
# Add a new zone to the current user's access list
sub add_zone_access
{
	my $self = shift;

if ($self->{'access_hash'}->{'zones'} ne '*' && $self->{'access_hash'}->{'zones'} !~ /^\!/) {
	$self->{'access_hash'}->{'zones'} = join(" ", $self->unique(
				split(/\s+/, $self->{'access_hash'}->{'zones'}), $_[0]));
	$self->save_module_acl(\%{$self->{'access_hash'}});
	}
}

# is_config_valid()
sub is_config_valid
{
	my $self = shift;

local $conf = $self->get_config();
local ($opts, $dir);
if (($opts = $self->find("options", $conf)) &&
    ($dir = $self->find("directory", $opts->{'members'})) &&
    !(-d $self->make_chroot($dir->{'value'}))) {
	return 0;
	}
return 1;
}

# check_bind_8()
# Returns the --help output if non BIND 8/9, or undef if is
sub check_bind_8
{
	my $self = shift;

local $out = `$self->{'config_hash'}->{'named_path'} -help 2>&1`;
return $out !~ /\[-f\]/ && $out !~ /\[-f\|/ ? $out : undef;
}

# get_chroot()
# Returns the chroot directory BIND is running under
sub get_chroot
{
	my $self = shift;

if ($self->{'config_hash'}->{'auto_chroot'}) {
	local $out = `$self->{'config_hash'}->{'auto_chroot'} 2>/dev/null`;
	if (!$?) {
		$out =~ s/\r|\n//g;
		return $out;
		}
	}
return $self->{'config_hash'}->{'chroot'};
}

# make_chroot(file, [is-pid])
sub make_chroot
{
	my $self = shift;

local $chroot = $self->get_chroot();
return $_[0] if (!$chroot);
return $_[0] if ($_[0] eq $self->{'config_hash'}->{'named_conf'} && $self->{'config_hash'}->{'no_chroot'});
return $_[0] if ($_[0] eq $self->{'config_hash'}->{'rndc_conf'});	# don't chroot rndc.conf
if ($self->{'config_hash'}->{'no_pid_chroot'} && $_[1]) {
	return $_[0];
	}
return $chroot.$_[0];
}

# has_ndc(exclude-mode)
# Returns 2 if rndc is installed, 1 if ndc is instaled, or 0
sub has_ndc
{
	my $self = shift;

if ($self->{'config_hash'}->{'rndc_cmd'} =~ /^(\S+)/ && $self->has_command("$1") && $_[0] != 2) {
	return 2;
	}
if ($self->{'config_hash'}->{'ndc_cmd'} =~ /^(\S+)/ && $self->has_command("$1") && $_[0] != 1) {
	return 1;
	}
return 0;
}

# $self->get_pid_file([no-cache])
# Returns the BIND pid file path, relative to any chroot
sub get_pid_file
{
	my $self = shift;

if ($_[0] || !-r $self->{'zone_names_cache'}) {
	# Read real config
	local $conf = $self->get_config();
	local ($opts, $pidopt);
	if (!$self->{'config_hash'}->{'pid_file'} && ($opts = $self->find("options", $conf)) &&
	    ($pidopt = $self->find("pid-file", $opts->{'members'}))) {
		# read from PID file
		local $pidfile = $pidopt->{'value'};
		if ($pidfile !~ /^\//) {
			local $dir = $self->find("directory", $opts->{'members'});
			$pidfile = $dir->{'value'}."/".$pidfile;
			}
		return $pidfile;
		}

	# use default file
	local $p;
	foreach $p (split(/\s+/, $self->{'config_hash'}->{'pid_file'})) {
		if (-r $self->make_chroot($p, 1)) {
			return $p;
			}
		}
	return "/var/run/named.pid";
	}
else {
	# Use cache if possible
	local %znc;
	$self->read_file_cached($self->{'zone_names_cache'}, \%znc);
	if ($znc{'pidfile'} && -r $znc{'pidfile'}) {
		return $znc{'pidfile'};
		}
	else {
		return $self->get_pid_file(1);
		}
	}
}

# can_edit_type(record-type)
sub can_edit_type
{
	my $self = shift;

return 1 if (!$self->{'access_hash'}->{'types'});
local $t;
foreach $t (split(/\s+/, $self->{'access_hash'}->{'types'})) {
	return 1 if (lc($t) eq lc($_[0]));
	}
return 0;
}

# add_to_file()
# Returns the filename to which new zones should be added (possibly relative to
# a chroot directory)
sub add_to_file
{
	my $self = shift;

if ($self->{'config_hash'}->{'zones_file'}) {
	local $conf = $self->get_config();
	local $f;
	foreach $f ($self->get_all_config_files($conf)) {
		if ($self->same_file($f, $self->{'config_hash'}->{'zones_file'})) {
			return $self->{'config_hash'}->{'zones_file'};
			}
		}
	}
return $self->{'config_hash'}->{'named_conf'};
}

# $self->get_all_config_files(&conf)
# Returns a list of all config files used by named.conf, including includes
sub get_all_config_files
{
	my $self = shift;

local ($conf) = @_;
local @rv;
foreach my $c (@$conf) {
	push(@rv, $c->{'file'});
	if ($c->{'type'} == 1) {
		push(@rv, $self->get_all_config_files($c->{'members'}));
		}
	}
return $self->unique(@rv);
}

# free_address_button(name, [form])
sub free_address_button
{
	my $self = shift;

local $form = defined($_[2]) ? $_[2] : 0;
return "<input type=button onClick='ifield = document.forms[$form].$_[0]; chooser = window.open(\"free_chooser.cgi\", \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=150,height=500\"); chooser.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
}

# create_slave_zone(name, master-ip, [view], [file])
# A convenience function for creating a new slave zone, if it doesn't exist
# yet. Mainly useful for Virtualmin, to avoid excessive transfer of BIND
# configuration data.
# Returns 0 on success, 1 if BIND is not setup, 2 if the zone already exists,
# or 3 if the view doesn't exist, or 4 if the slave file couldn't be created
sub create_slave_zone
{
	my $self = shift;

local $parent = $self->get_config_parent();
local $conf = $parent->{'members'};
local $opts = $self->find("options", $conf);
if (!$opts) {
	return 1;
	}

# Check if exists in the view
if ($_[2]) {
	local ($v) = grep { $_->{'value'} eq $_[2] } $self->find("view", $conf);
	@{$self->{'zones_array'}} = $self->find("zone", $v->{'members'});
	}
else {
	@{$self->{'zones_array'}} = $self->find("zone", $conf);
	}
local ($z) = grep { $_->{'value'} eq $_[0] } @{$self->{'zones_array'}};
return 2 if ($z);

# Create it
local $masters = { 'name' => 'masters',
                   'type' => 1,
                   'members' => [ { 'name' => $_[1] } ] };
local $dir = { 'name' => 'zone',
               'values' => [ $_[0] ],
               'type' => 1,
               'members' => [ { 'name' => 'type',
                                'values' => [ 'slave' ] },
                                $masters
                            ]
	     };
local $base = $self->{'config_hash'}->{'slave_dir'} || $self->base_directory();
local $file;
if (!$_[3]) {
	# File has default name and is under default directory
	$file = $self->automatic_filename($_[0], $_[0] =~ /in-addr/i ? 1 : 0, $base,
				    $_[2]);
	push(@{$dir->{'members'}}, { 'name' => 'file',
				     'values' => [ $file ] } );
	}
elsif ($_[3] ne "none") {
	# File was specified
	$file = $_[3] =~ /^\// ? $_[3] : $base."/".$_[3];
	push(@{$dir->{'members'}}, { 'name' => 'file',
				     'values' => [ $file ] } );
	}

# Create the slave file, so that BIND can write to it
if ($file) {
	$self->open_tempfile(ZONE, ">".$self->make_chroot($file), 1, 1) || return 4;
	$self->close_tempfile(ZONE);
        $self->set_ownership($self->make_chroot($file));
	}

local $view;
if ($_[2]) {
	# Get and validate view
	($view) = grep { $_->{'value'} eq $_[2] } $self->find("view", $conf);
	return 3 if (!$view);
	}

# Create the zone
$self->create_zone($dir, $conf, $view ? $view->{'index'} : undef);

return 0;
}

# delete_zone(name, [view], [file-too])
# Delete one zone from named.conf
# Returns 0 on success, 1 if the zone was not found, or 2 if the view was not found.
sub delete_zone
{
	my $self = shift;

local $parent = $self->get_config_parent();
local $conf = $parent->{'members'};
local @zones;
if ($_[1]) {
	# Look in one view
	local ($v) = grep { $_->{'value'} eq $_[1] } $self->find("view", $conf);
	return 2 if (!$v);
	@zones = $self->find("zone", $v->{'members'});
	$parent = $v;
	}
else {
	# Look in all views
	@zones = $self->find("zone", $conf);
	local $v;
	foreach $v ($self->find("view", $conf)) {
		push(@zones, $self->find("zone", $v->{'members'}));
		}
	}
local ($z) = grep { $_->{'value'} eq $_[0] } @zones;
return 1 if (!$z);

# Remove from config file
$self->lock_file($z->{'file'});
$self->save_directive($parent, [ $z ], [ ]);
$self->unlock_file($z->{'file'});
$self->flush_file_lines();

if ($_[2]) {
	# Remove file
	local $f = $self->find("file", $z->{'members'});
	if ($f) {
		$self->lock_file($self->make_chroot($self->absolute_path($f->{'value'})));
		unlink($self->make_chroot($self->absolute_path($f->{'value'})));
		$self->unlock_file($self->make_chroot($self->absolute_path($f->{'value'})));
		}
	}

$self->flush_zone_names();
return 0;
}

# rename_zone(oldname, newname, [view])
# Changes the name of some zone, and perhaps it's file
# Returns 0 on success, 1 if the zone was not found, or 2 if the view was
# not found.
sub rename_zone
{
	my $self = shift;

local $parent = $self->get_config_parent();
local $conf = $parent->{'members'};
local @zones;
if ($_[2]) {
	# Look in one view
	local ($v) = grep { $_->{'value'} eq $_[2] } $self->find("view", $conf);
	return 2 if (!$v);
	@zones = $self->find("zone", $v->{'members'});
	$parent = $v;
	}
else {
	# Look in all views
	@zones = $self->find("zone", $conf);
	local $v;
	foreach $v ($self->find("view", $conf)) {
		push(@zones, $self->find("zone", $v->{'members'}));
		}
	}
local ($z) = grep { $_->{'value'} eq $_[0] } @zones;
return 1 if (!$z);

$z->{'values'} = [ $_[1] ];
local $file = $self->find("file", $z->{'members'});
if ($file) {
	# Update the file too
	local $newfile = $file->{'values'}->[0];
	$newfile =~ s/$_[0]/$_[1]/g;
	if ($newfile ne $file->{'values'}->[0]) {
		rename($self->make_chroot($file->{'values'}->[0]),
		       $self->make_chroot($newfile));
		$file->{'values'}->[0] = $newfile;
		}
	}

$self->save_directive($parent, [ $z ], [ $z ]);
$self->flush_file_lines();
return 0;
}

# restart_bind()
# A convenience function for re-starting BIND. Returns undef on success, or
# an error message on failure.
sub restart_bind
{
	my $self = shift;

if ($self->{'config_hash'}->{'restart_cmd'}) {
	local $out = $self->backquote_logged("$self->{'config_hash'}->{'restart_cmd'} 2>&1 </dev/null");
	if ($?) {
		return $self->text('restart_ecmd', "<pre>$out</pre>");
		}
	}
else {
	local $pidfile = $self->get_pid_file();
	local $pid = $self->check_pid_file($self->make_chroot($pidfile, 1));
	if (!$pid) {
		return $self->text('restart_epidfile', $pidfile);
		}
	elsif (!$self->kill_logged('HUP', $pid)) {
		return $self->text('restart_esig', $pid, $!);
		}
	}
return undef;
}

# start_bind()
# Attempts to start the BIND DNS server, and returns undef on success or an
# error message on failure
sub start_bind
{
	my $self = shift;

local $chroot = $self->get_chroot();
local $user;
local $cmd;
if ($self->{'config_hash'}->{'named_user'}) {
	$user = "-u $self->{'config_hash'}->{'named_user'}";
	if ($self->{'config_hash'}->{'named_group'}) {
		$user .= " -g $self->{'config_hash'}->{'named_group'}";
		}
	else {
		local @u = getpwnam($self->{'config_hash'}->{'named_user'});
		local @g = getgrgid($u[3]);
		$user .= " -g $g[0]";
		}
	}
if ($self->{'config_hash'}->{'start_cmd'}) {
	$cmd = $self->{'config_hash'}->{'start_cmd'};
	}
elsif (!$chroot) {
	$cmd = "$self->{'config_hash'}->{'named_path'} -c $self->{'config_hash'}->{'named_conf'} $user </dev/null 2>&1";
	}
elsif (`$self->{'config_hash'}->{'named_path'} -help 2>&1` =~ /\[-t/) {
	# use named's chroot option
	$cmd = "$self->{'config_hash'}->{'named_path'} -c $self->{'config_hash'}->{'named_conf'} -t $chroot $user </dev/null 2>&1";
	}
else {
	# use the chroot command
	$cmd = "chroot $chroot $self->{'config_hash'}->{'named_path'} -c $self->{'config_hash'}->{'named_conf'} $user </dev/null 2>&1";
	}

local $temp = $self->transname();
local $rv = $self->system_logged("$cmd </dev/null >$temp 2>&1");
local $out = `cat $temp`;
unlink($temp);
if ($rv || $out =~ /chroot.*not available/i) {
	return $self->text('start_error', $out ? "<tt>$out</tt>" : "Unknown error");
	}
return undef;
}

# stop_bind()
# Kills the running DNS server, and returns undef on success or an error message
# upon failure
sub stop_bind
{
	my $self = shift;

if ($self->{'config_hash'}->{'stop_cmd'}) {
	# Just use a command
	local $out = $self->backquote_logged("($self->{'config_hash'}->{'stop_cmd'}) 2>&1");
	if ($?) {
		return "<pre>$out</pre>";
		}
	}
else {
	# Kill the process
	local $pidfile = $self->get_pid_file();
	local $pid = $self->check_pid_file($self->make_chroot($pidfile, 1));
	if (!$pid || !$self->kill_logged('TERM', $pid)) {
		return $self->{'text_hash'}->{'stop_epid'};
		}
	}
return undef;
}

# is_bind_running()
# Returns the PID if BIND is running
sub is_bind_running
{
	my $self = shift;

local $pidfile = $self->get_pid_file();
return $self->check_pid_file($self->make_chroot($pidfile, 1));
}

# version_atleast(v1, v2, v3)
sub version_atleast
{
	my $self = shift;

local @vsp = split(/\./, $self->{'bind_version'});
local $i;
for($i=0; $i<@vsp || $i<@_; $i++) {
	return 0 if ($vsp[$i] < $_[$i]);
	return 1 if ($vsp[$i] > $_[$i]);
	}
return 1;	# same!
}

# get_zone_index(name, [view])
# Returns the index of some zone in the real on-disk configuration
sub get_zone_index
{
	my $self = shift;

undef(@{$self->{'get_config_cache_array'}});
local $conf = $self->get_config();
local $vconf = $_[1] ne '' ? $conf->[$self->{'in_hash'}->{'view'}]->{'members'} : $conf;
local $c;
foreach $c (@$vconf) {
	if ($c->{'name'} eq 'zone' && $c->{'value'} eq $_[0]) {
		return $c->{'index'};
		}
	}
return undef;
}

# create_zone(&zone, &conf, [view-idx])
# Convenience function for adding a new zone
sub create_zone
{
	my $self = shift;

local ($dir, $conf, $viewidx) = @_;
if ($viewidx ne "") {
	# Adding inside a view
	local $view = $conf->[$viewidx];
        $self->lock_file($self->make_chroot($view->{'file'}));
        $self->save_directive($view, undef, [ $dir ], 1);
        $self->flush_file_lines();
        $self->unlock_file($self->make_chroot($view->{'file'}));
	}
else {
	# Adding at top level
        $dir->{'file'} = $self->add_to_file();
        local $pconf = $self->get_config_parent($dir->{'file'});
        $self->lock_file($self->make_chroot($dir->{'file'}));
        $self->save_directive($pconf, undef, [ $dir ], 0);
        $self->flush_file_lines();
        $self->unlock_file($self->make_chroot($dir->{'file'}));
	}
$self->flush_zone_names();
}

# get_heiropen()
# Returns an array of open categories
sub get_heiropen
{
	my $self = shift;

open(HEIROPEN, $self->{'heiropen_file'});
local @heiropen = <HEIROPEN>;
chop(@heiropen);
close(HEIROPEN);
return @heiropen;
}

# save_heiropen(&heir)
sub save_heiropen
{
	my $self = shift;

$self->open_tempfile(HEIR, ">$self->{'heiropen_file'}");
foreach $h (@{$_[0]}) {
	$self->print_tempfile(HEIR, $h,"\n");
	}
$self->close_tempfile(HEIR);
}

# list_zone_names()
# Returns a list of zone names, types, files and views based on a cache
# built from the primary configuration.
sub list_zone_names
{
	my $self = shift;

local @st = stat($self->{'zone_names_cache'});
local %znc;
$self->read_file_cached($self->{'zone_names_cache'}, \%znc);

# Check if any files have changed, or if the master config has changed, or
# the PID file.
local @files;
local ($k, $changed, $filecount, %donefile);
foreach $k (keys %znc) {
	if ($k =~ /^file_(.*)$/) {
		$filecount++;
		$donefile{$1}++;
		local @fst = stat($1);
		if ($fst[9] > $st[9]) {
			$changed = 1;
			}
		}
	}
if ($changed || !$filecount || $znc{'version'} != $self->{'zone_names_version'} ||
    !$donefile{$self->make_chroot($self->{'config_hash'}->{'named_conf'})} ||
    $self->{'config_hash'}->{'pid_file'} ne $znc{'pidfile_config'}) {
	# Yes .. need to rebuild
	%znc = ( );
	local $conf = $self->get_config();
	local @views = $self->find("view", $conf);
	local ($v, $z);
	local $n = 0;
	foreach $v (@views) {
		local @vz = $self->find("zone", $v->{'members'});
		foreach $z (@vz) {
			local $type = $self->find_value("type", $z->{'members'});
			local $file = $self->find_value("file", $z->{'members'});
			$znc{"zone_".($n++)} = "$z->{'value'} $z->{'index'} $type $v->{'value'} $file";
			$self->{'files_hash'}->{$z->{'file'}}++;
			}
		$znc{"view_".($n++)} = "$v->{'value'} $v->{'index'}";
		$self->{'files_hash'}->{$v->{'file'}}++;
		}
	foreach $z ($self->find("zone", $conf)) {
		local $type = $self->find_value("type", $z->{'members'});
		local $file = $self->find_value("file", $z->{'members'});
		$znc{"zone_".($n++)} = "$z->{'value'} $z->{'index'} $type * $file";
		$self->{'files_hash'}->{$z->{'file'}}++;
		}

	# Store the base directory and PID file
	$znc{'base'} = $self->base_directory($conf, 1);
	$znc{'pidfile'} = $self->get_pid_file(1);
	$znc{'pidfile_config'} = $self->{'config_hash'}->{'pid_file'};

	# Store source files
	foreach $f (keys %{$self->{'files_hash'}}) {
		local $realf = $self->make_chroot($self->absolute_path($f));
		local @st = stat($realf);
		$znc{"file_".$realf} = $st[9];
		}

	$znc{'version'} = $self->{'zone_names_version'};
	$self->write_file($self->{'zone_names_cache'}, \%znc);
	undef(@{$self->{'list_zone_names_cache_array'}});
	}

# Use in-memory cache
if (defined(@{$self->{'list_zone_names_cache_array'}})) {
	return @{$self->{'list_zone_names_cache_array'}};
	}

# Construct the return value from the hash
local (@rv, %viewidx);
foreach $k (keys %znc) {
	if ($k =~ /^zone_(\d+)$/) {
		local ($name, $index, $type, $view, $file) =
			split(/\s+/, $znc{$k}, 5);
		push(@rv, { 'name' => $name,
			    'type' => $type,
			    'index' => $index,
			    'view' => $view eq '*' ? undef : $view,
			    'file' => $file });
		}
	elsif ($k =~ /^view_(\d+)$/) {
		local ($name, $index) = split(/\s+/, $znc{$k}, 2);
		push(@rv, { 'name' => $name,
			    'index' => $index,
			    'type' => 'view' });
		$viewidx{$name} = $index;
		}
	}
local $z;
foreach $z (@rv) {
	if ($z->{'type'} ne 'view' && $z->{'view'} ne '*') {
		$z->{'viewindex'} = $viewidx{$z->{'view'}};
		}
	}
@{$self->{'list_zone_names_cache_array'}} = @rv;
return @rv;
}

# flush_zone_names()
# Clears the in-memory and on-disk zone name caches
sub flush_zone_names
{
	my $self = shift;

undef(@{$self->{'list_zone_names_cache_array'}});
unlink($self->{'zone_names_cache'});
}

# get_zone_name(index|name, [viewindex|"any"])
# Returns a zone cache object, looked up by name or index
sub get_zone_name
{
	my $self = shift;

local @zones = $self->list_zone_names();
local $field = $_[0] =~ /^\d+$/ ? "index" : "name";
local $z;
foreach $z (@zones) {
	if ($z->{$field} eq $_[0] &&
	    ($_[1] eq 'any' ||
	     $_[1] eq '' && !defined($z->{'viewindex'}) ||
	     $_[1] ne '' && $z->{'viewindex'} == $_[1])) {
		return $z;
		}
	}
return undef;
}

# list_slave_servers()
# Returns a list of Webmin servers on which slave zones are created / deleted
sub list_slave_servers
{
	my $self = shift;

$self->foreign_require("servers", "servers-lib.pl");
local %ids = map { $_, 1 } split(/\s+/, $self->{'config_hash'}->{'servers'});
local %secids = map { $_, 1 } split(/\s+/, $self->{'config_hash'}->{'secservers'});
local @servers = $main::servers_object->list_servers();
if (%ids) {
	local @rv = grep { $ids{$_->{'id'}} } @servers;
	foreach my $s (@rv) {
		$s->{'sec'} = $secids{$s->{'id'}};
		}
	return @rv;
	}
elsif ($self->{'config_hash'}->{'default_slave'} && !defined($self->{'config_hash'}->{'servers'})) {
	# Migrate old-style setting of single slave
	local ($serv) = grep { $_->{'host'} eq $self->{'config_hash'}->{'default_slave'} }
			     @servers;
	if ($serv) {
		$self->add_slave_server($serv);
		return ($serv);
		}
	}
return ( );
}

# add_slave_server(&server)
sub add_slave_server
{
	my $self = shift;

$self->lock_file($self->{'module_config'});
$self->foreign_require("servers", "servers-lib.pl");
local @sids = split(/\s+/, $self->{'config_hash'}->{'servers'});
$self->{'config_hash'}->{'servers'} = join(" ", @sids, $_[0]->{'id'});
if ($_[0]->{'sec'}) {
	local @secsids = split(/\s+/, $self->{'config_hash'}->{'secservers'});
	$self->{'config_hash'}->{'secservers'} = join(" ", @secsids, $_[0]->{'id'});
	}
$self->sync_default_slave();
$self->save_module_config();
$self->unlock_file($self->{'module_config'});
$main::servers_object->save_server($_[0]);
}

# delete_slave_server(&server)
sub delete_slave_server
{
	my $self = shift;

$self->lock_file($self->{'module_config'});
local @sids = split(/\s+/, $self->{'config_hash'}->{'servers'});
$self->{'config_hash'}->{'servers'} = join(" ", grep { $_ != $_[0]->{'id'} } @sids);
local @secsids = split(/\s+/, $self->{'config_hash'}->{'secservers'});
$self->{'config_hash'}->{'secservers'} = join(" ", grep { $_ != $_[0]->{'id'} } @secsids);
$self->sync_default_slave();
$self->save_module_config();
$self->unlock_file($self->{'module_config'});
}

sub sync_default_slave
{
	my $self = shift;

local @servers = $self->list_slave_servers();
if (@servers) {
	$self->{'config_hash'}->{'default_slave'} = $servers[0]->{'host'};
	}
else {
	$self->{'config_hash'}->{'default_slave'} = '';
	}
}

# server_name(&server)
sub server_name
{
	my $self = shift;

return $_[0]->{'desc'} ? $_[0]->{'desc'} : $_[0]->{'host'};
}

# create_master_records(file, zone, master, email, refresh, retry, expiry, min,
#			add-master-ns, add-slaves-ns, add-template, tmpl-ip,
#			add-template-reverse)
# Creates the records file for a new master zone. Returns undef on success, or
# an error message on failure.
sub create_master_records
{
	my $self = shift;

local ($file, $zone, $master, $email, $refresh, $retry, $expiry, $min,
       $add_master, $add_slaves, $add_tmpl, $ip, $addrev) = @_;

# Create the zone file
$self->lock_file($self->make_chroot($file));
$self->open_tempfile(ZONE, ">".$self->make_chroot($file), 1) ||
	return $self->text('create_efile3', $file, $!);
$self->print_tempfile(ZONE, "\$ttl $min\n")
	if ($self->{'config_hash'}->{'master_ttl'});
$self->close_tempfile(ZONE);

# create the SOA and NS records
local $serial;
if ($self->{'config_hash'}->{'soa_style'} == 1) {
        $serial = $self->date_serial().sprintf("%2.2d", $self->{'config_hash'}->{'soa_start'});
        }
else {
	# Use Unix time for date and running number serials
        $serial = time();
        }
local $vals = "$master $email (\n".
        "\t\t\t$serial\n".
        "\t\t\t$refresh\n".
        "\t\t\t$retry\n".
        "\t\t\t$expiry\n".
        "\t\t\t$min )";
$self->create_record($file, "$zone.", undef, "IN", "SOA", $vals);
$self->create_record($file, "$zone.", undef, "IN", "NS", $master)
	if ($add_master);
if ($add_slaves) {
	local $slave;
	foreach $slave ($self->list_slave_servers()) {
		local @bn = $slave->{'nsname'} ||
				gethostbyname($slave->{'host'});
		local $full = "$bn[0].";
		$self->create_record($file, "$zone.", undef, "IN", "NS", $full);
		}
	}

if ($add_tmpl) {
	# Create template records
	local %bumped;
	for(my $i=0; $self->{'config_hash'}->{"tmpl_$i"}; $i++) {
		local @c = split(/\s+/, $self->{'config_hash'}->{"tmpl_$i"}, 3);
		local $name = $c[0] eq '.' ? "$zone." : $c[0];
		local $fullname = $name =~ /\.$/ ? $name : "$name.$zone.";
		local $recip = $c[2] || $ip;
		$self->create_record($file, $name, undef, "IN", $c[1], $recip);
		if ($addrev && ($c[1] eq "A" || $c[1] eq "AAAA")) {
			# Consider adding reverse record
			local ($revconf, $revfile, $revrec) = $self->find_reverse(
				$recip, $self->{'view'});
			if ($revconf && $self->can_edit_reverse($revconf) &&
			    !$revrec) {
				# Yes, add one
				local $rname = $c[1] eq "A" ?
					$self->ip_to_arpa($recip) :
					$self->net_to_ip6int($recip);
				$self->lock_file($self->make_chroot($revfile));
				$self->create_record($revfile, $rname,
					undef, "IN", "PTR", $fullname);
				if (!$bumped{$revfile}++) {
					local @rrecs = $self->read_zone_file(
						$revfile, $revconf->{'name'});
					$self->bump_soa_record($revfile, \@rrecs);
					}
				}
			}
		}
	if ($self->{'config_hash'}->{'tmpl_include'}) {
		# Add whatever is in the template file
		local $tmpl = $self->read_file_contents($self->{'config_hash'}->{'tmpl_include'});
		local %hash = ( 'ip' => $ip,
				'dom' => $zone );
		$tmpl = $self->substitute_template($tmpl, \%hash);
		$self->open_tempfile(FILE, ">>".$self->make_chroot($file));
		$self->print_tempfile(FILE, $tmpl);
		$self->close_tempfile(FILE);
		}
	}
$self->unlock_file($self->make_chroot($file));
$self->set_ownership($self->make_chroot($file));
return undef;
}

# automatic_filename(domain, is-reverse, base, [viewname])
# Returns a filename for a new zone
sub automatic_filename
{
	my $self = shift;

local ($zone, $rev, $base, $viewname) = @_;
local ($subs, $format);
if ($rev) {
	# create filename for reverse zone
	$subs = $self->ip6int_to_net($self->arpa_to_ip($zone));
	$subs =~ s/\//_/;
	$format = $self->{'config_hash'}->{'reversezonefilename_format'};
	}
else {
	# create filename for forward zone
	$format = $self->{'config_hash'}->{'forwardzonefilename_format'};
	$subs = $zone;
	}
if ($viewname) {
	$subs .= ".".$viewname;
	}
$format =~ s/ZONE/$subs/g;
return $self->{'file'} = $base."/".$format;
}

# create_on_slaves(zone, master-ip, file, [&hostnames])
# Creates the given zone on all configured slave servers, and returns a list
# of errors
sub create_on_slaves
{
	my $self = shift;

local ($zone, $master, $file, $hosts) = @_;
local %on = map { $_, 1 } @$hosts;
$self->remote_error_setup(\&slave_error_handler);
local $slave;
local @slaveerrs;
foreach $slave ($self->list_slave_servers()) {
	next if (%on && !$on{$slave->{'host'}});

	# Connect to server
	$self->{'slave_error'} = undef;
	$self->remote_foreign_require($slave, "bind8", "bind8-lib.pl");
	if ($self->{'slave_error'}) {
		push(@slaveerrs, [ $slave, $self->{'slave_error'} ]);
		next;
		}

	# Create the zone
	local $err = $self->remote_foreign_call($slave, "bind8",
		"create_slave_zone", $zone, $master,
		$slave->{'bind8_view'}, $file);
	if ($err == 1) {
		push(@slaveerrs, [ $slave, $self->{'text_hash'}->{'master_esetup'} ]);
		}
	elsif ($err == 2) {
		push(@slaveerrs, [ $slave, $self->{'text_hash'}->{'master_etaken'} ]);
		}
	elsif ($err == 3) {
		push(@slaveerrs, [ $slave, $self->text('master_eview',
					 $slave->{'bind8_view'}) ]);
		}
	}
$self->remote_error_setup();
return @slaveerrs;
}

# delete_on_slaves(domain, [&slave-hostnames])
# Delete some domain or all or listed slave servers
sub delete_on_slaves
{
	my $self = shift;

local $dom = $_[0];
local %on = map { $_, 1 } @{$_[1]};
$self->remote_error_setup(\&slave_error_handler);
local $slave;
local @slaveerrs;
foreach $slave ($self->list_slave_servers()) {
	next if (%on && !$on{$slave->{'host'}});

	# Connect to server
	$self->{'slave_error'} = undef;
	$self->remote_foreign_require($slave, "bind8", "bind8-lib.pl");
	if ($self->{'slave_error'}) {
		push(@slaveerrs, [ $slave, $self->{'slave_error'} ]);
		next;
		}

	# Delete the zone
	$self->{'err'} = $self->remote_foreign_call($slave, "bind8", "delete_zone",
			    $dom, $slave->{'bind8_view'}, 1);
	if ($self->{'err'} == 1) {
		push(@slaveerrs, [ $slave, $self->{'text_hash'}->{'delete_ezone'} ]);
		}
	elsif ($self->{'err'} == 2) {
		push(@slaveerrs, [ $slave, $self->text('master_eview',
					 $slave->{'bind8_view'}) ]);
		}
	}
$self->remote_error_setup();
return @slaveerrs;
}

# rename_on_slaves(olddomain, newdomain, [&slave-hostnames])
# Changes the name of some domain on all or listed slave servers
sub rename_on_slaves
{
	my $self = shift;

local ($olddom, $newdom, $on) = @_;
local %on = map { $_, 1 } @$on;
$self->remote_error_setup(\&slave_error_handler);
local $slave;
local @slaveerrs;
foreach $slave ($self->list_slave_servers()) {
	next if (%on && !$on{$slave->{'host'}});

	# Connect to server
	$self->{'slave_error'} = undef;
	$self->remote_foreign_require($slave, "bind8", "bind8-lib.pl");
	if ($self->{'slave_error'}) {
		push(@slaveerrs, [ $slave, $self->{'slave_error'} ]);
		next;
		}

	# Delete the zone
	$self->{'err'} = $self->remote_foreign_call($slave, "bind8", "rename_zone",
			    $olddom, $newdom, $slave->{'bind8_view'});
	if ($self->{'err'} == 1) {
		push(@slaveerrs, [ $slave, $self->{'text_hash'}->{'delete_ezone'} ]);
		}
	elsif ($self->{'err'} == 2) {
		push(@slaveerrs, [ $slave, $self->text('master_eview',
					 $slave->{'bind8_view'}) ]);
		}
	}
$self->remote_error_setup();
return @slaveerrs;
}

# restart_on_slaves([&slave-hostnames])
# Re-starts BIND on all or listed slave servers, and returns a list of errors
sub restart_on_slaves
{
	my $self = shift;

local %on = map { $_, 1 } @{$_[0]};
$self->remote_error_setup(\&slave_error_handler);
local $slave;
local @slaveerrs;
foreach $slave ($self->list_slave_servers()) {
	next if (%on && !$on{$slave->{'host'}});

	# Find the PID file
	$self->{'slave_error'} = undef;
	$self->remote_foreign_require($slave, "bind8", "bind8-lib.pl");
	if ($self->{'slave_error'}) {
		push(@slaveerrs, [ $slave, $self->{'slave_error'} ]);
		next;
		}
	local $sver = $self->remote_foreign_call($slave, "bind8",
				     "get_webmin_version");
	local $pidfile;
	if ($sver >= 1.140) {
		# Call new function to get PID file from slave
		$pidfile = $self->remote_foreign_call(
			$slave, "bind8", "get_pid_file");
		$pidfile = $self->remote_foreign_call(
			$slave, "bind8", "make_chroot", $pidfile, 1);
		}
	else {
		push(@slaveerrs, [ $slave, $self->text('restart_eversion',
						 $slave->{'host'}, 1.140) ]);
		next;
		}

	# Read the PID and restart
	local $pid = $self->remote_foreign_call($slave, "bind8",
				    "check_pid_file", $pidfile);
	if (!$pid) {
		push(@slaveerrs, [ $slave, $self->text('restart_erunning2',
						 $slave->{'host'}) ]);
		next;
		}
	$self->{'err'} = $self->remote_foreign_call($slave, "bind8", "restart_bind");
	if ($self->{'err'}) {
		push(@slaveerrs, [ $slave, $self->text('restart_esig2',
						 $slave->{'host'}, $self->{'err'}) ]);
		}
	}
$self->remote_error_setup();
return @slaveerrs;
}

sub slave_error_handler
{
	my $self = shift;

$self->{'slave_error'} = $_[0];
}

sub get_forward_record_types
{
	my $self = shift;

return ("A", "NS", "CNAME", "MX", "HINFO", "TXT", "SPF", "WKS", "RP", "PTR", "LOC", "SRV", "KEY", $self->{'config_hash'}->{'support_aaaa'} ? ( "AAAA" ) : ( ), @{$self->{'extra_forward_array'}});
}

sub get_reverse_record_types
{
	my $self = shift;

return ("PTR", "NS", "CNAME", @{$self->{'extra_reverse_array'}});
}

# Get index "view" using name of zone
sub get_view_by_name($)
{
	my $self = shift;

my(@views, $conf);
%{$self->{'in_hash'}};

$conf = $self->get_config();

@views = $self->find("view",$conf);
unless(@views) { return undef;}
@views = map( $_->{'index'}, grep { $self->can_edit_view($_) } @views);
unless(@views) { return undef;}
foreach (@views) {
    $self->{'in_hash'}->{'view'} = $_;
    if( defined($self->get_zone_index($_[0], $_))) { return $_;}
    }
$self->{'in_hash'}->{'view'} = '';
return undef;
}


1;

