#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Add or update a server or group from the webmin servers module

&ModuleLoader::preload_module('bind8_class', "bind8::Bind8Lib");
$self = new bind8::Bind8Lib:: 'bind8_class',%$self;

$self->{'access_hash'}->{'slaves'} || $self->error($self->{'text_hash'}->{'slaves_ecannot'});
$self->ReadParse();
$self->foreign_require("servers", "servers-lib.pl");
@{$self->{'allservers_array'}} = grep { $_->{'user'} } $main::servers_object->list_servers();

if ($self->{'in_hash'}->{'add'}) {
	# Add a single host
	@{$self->{'add_array'}} = grep { $_->{'id'} eq $self->{'in_hash'}->{'server'} } @{$self->{'allservers_array'}};
	$self->error_setup($self->{'text_hash'}->{'add_err'});
	$self->{'msg'} = $self->text('add_msg', $self->server_name($self->{'add_array'}->[0]));
	$self->{'in_hash'}->{'name_def'} || $self->valdnsname($self->{'in_hash'}->{'name'}) ||
		$self->error($self->{'text_hash'}->{'add_ename'});
	}
else {
	# Add all from a group
	($self->{'group'}) = grep { $_->{'name'} eq $self->{'in_hash'}->{'group'} }
			$main::servers_object->list_all_groups(\@{$self->{'allservers_array'}});
	foreach $m (@{$self->{'group'}->{'members'}}) {
		push(@{$self->{'add_array'}}, grep { $_->{'host'} eq $m } @{$self->{'allservers_array'}});
		}
	$self->error_setup($self->{'text_hash'}->{'add_gerr'});
	$self->{'msg'} = $self->text('add_gmsg', $self->{'in_hash'}->{'group'});
	$self->{'in_hash'}->{'name_def'} || $self->error($self->{'text_hash'}->{'add_egname'});
	}
$self->{'in_hash'}->{'view_def'} || $self->{'in_hash'}->{'view'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'add_eview'});

$self->ui_print_header(undef, $self->{'text_hash'}->{'add_title'}, "");
print "<b>$self->{'msg'}</b><p>\n";

$self->remote_error_setup(\&add_error);

# Make sure each host is set up for BIND
@{$self->{'zones_array'}} = grep { $_->{'type'} eq 'master' } $self->list_zone_names();
$self->{'myip'} = $self->{'config_hash'}->{'this_ip'} || $self->to_ipaddress($self->get_system_hostname());
foreach $s (@{$self->{'add_array'}}) {
	$s->{'bind8_view'} = $self->{'in_hash'}->{'view_def'} ? undef : $self->{'in_hash'}->{'view'};
	$self->{'add_error_msg'} = undef;
	local $bind8 = $self->remote_foreign_check($s, "bind8");
	if ($self->{'add_error_msg'}) {
		print "$self->{'add_error_msg'}<p>\n";
		next;
		}
	if (!$bind8) {
		print $self->text('add_echeck', $s->{'host'}),"<p>\n";
		next;
		}
	$self->remote_foreign_require($s, "bind8", "bind8-lib.pl");
	local $inst = $self->remote_foreign_call($s, "bind8",
					   "foreign_installed", "bind8", 1);
	if (!$inst) {
		print $self->text('add_emissing', $s->{'host'}),"<p>\n";
		next;
		}
	if ($self->remote_foreign_call($s, "bind8",
				 "get_webmin_version") < 1.202) {
		print $self->text('add_eversion', $s->{'host'}, 1.202),"<p>\n";
		next;
		}

	@{$self->{'rzones_array'}} = grep { $_->{'type'} ne 'view' }
		       $self->remote_foreign_call($s, "bind8", "list_zone_names");
	print $self->text('add_ok', $s->{'host'}, scalar(@{$self->{'rzones_array'}})),"<p>\n";
	$s->{'sec'} = $self->{'in_hash'}->{'sec'};
	$s->{'nsname'} = $self->{'in_hash'}->{'name_def'} ? undef : $self->{'in_hash'}->{'name'};
	$self->add_slave_server($s);
	%{$self->{'rgot_hash'}} = map { $_->{'name'}, 1 } @{$self->{'rzones_array'}};

	if ($self->{'in_hash'}->{'sync'}) {
		# Add all master zones from this server to the slave
		$self->{'zcount'} = 0;
		$self->{'zerr'} = 0;
		foreach $zone (grep { !$self->{'rgot_hash'}->{$_->{'name'}} } @{$self->{'zones_array'}}) {
			($self->{'slaveerr'}) = $self->create_on_slaves($zone->{'name'}, $self->{'myip'},
						       undef, [ $s->{'host'} ]);
			if ($self->{'slaveerr'}) {
				$self->{'zerrs_hash'}->{$self->{'slaveerr'}->[0]->{'host'}} ||= $self->{'slaveerr'}->[1];
				$self->{'zerr'}++;
				}
			else {
				$self->{'zcount'}++;
				}
			}

		# Restart the slave too
		if ($self->{'zcount'}) {
			$self->remote_foreign_call($s, "bind8", "restart_bind");
			}

		# Tell the user
		if ($self->{'zerr'}) {
			print $self->text('add_createerr', $s->{'host'}, $self->{'zcount'}, $self->{'zerr'}),"<br>\n";
			foreach $k (keys %{$self->{'zerrs_hash'}}) {
				print "$k : $self->{'zerrs_hash'}->{$k}<br>\n";
				}
			print "<p>\n";
			}
		else {
			print $self->text('add_createok', $s->{'host'}, $self->{'zcount'}),"<p>\n";
			}
		}
	}
$self->remote_finished();
if ($self->{'in_hash'}->{'add'}) {
	$self->webmin_log("add", "host", $self->{'add_array'}->[0]->{'host'});
	}
else {
	$self->webmin_log("add", "group", $self->{'in_hash'}->{'group'});
	}

$self->ui_print_footer("list_slaves.cgi", $self->{'text_hash'}->{'slaves_return'});

1;
		
}

# Setup error handler for down hosts
sub add_error
{
	shift if ($self == $_[0]);

$self->{'add_error_msg'} = join("", @_);
}


1;

