#!/usr/bin/perl
package bsdexports::BsdexportsLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'bsdexports::BsdexportsLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# bsdexports-lib.pl
# Functions for managing the FreeBSD exports file

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

1;
		return $self;
}

# list_exports()
# Returns the current exports list
sub list_exports
{
	my $self = shift;

local(@rv, $lnum, $_);
open(EXP, $self->{'config_hash'}->{'exports_file'});
$lnum = -1; $self->{'index'} = 0;
while(<EXP>) {
	$lnum++;
	s/\r|\n//g;	# remove newlines
	s/#.*$//g;	# remove comments
	next if (!/\S/);
	local @w = split(/[\s=]+/, $_);
	local %exp;
	for($i=0; $i<@w; $i++) {
		if ($w[$i] =~ /^\//) { push(@{$exp{'dirs'}}, $w[$i]); }
		elsif ($w[$i] eq "-maproot") { $exp{'maproot'} = $w[++$i]; }
		elsif ($w[$i] eq "-r") { $exp{'maproot'} = $w[++$i]; }
		elsif ($w[$i] eq "-mapall") { $exp{'mapall'} = $w[++$i]; }
		elsif ($w[$i] eq "-kerb") { $exp{'kerb'}++; }
		elsif ($w[$i] eq "-ro") { $exp{'ro'}++; }
		elsif ($w[$i] eq "-alldirs") { $exp{'alldirs'}++; }
		elsif ($w[$i] eq "-network") { $exp{'network'} = $w[++$i]; }
		elsif ($w[$i] eq "-mask") { $exp{'mask'} = $w[++$i]; }
		elsif ($w[$i] eq "-public") { $exp{'public'}++; }
		elsif ($w[$i] eq "-webnfs") { $exp{'webnfs'}++; }
		elsif ($w[$i] eq "-index") { $exp{'index'} = $w[++$i]; }
		else { push(@{$exp{'hosts'}}, $w[$i]); }
		}
	$exp{'line'} = $lnum;
	$exp{'index'} = $self->{'index'}++;
	push(@rv, \%exp);
	}
close(EXP);
return @rv;
}

# create_export(&export)
sub create_export
{
	my $self = shift;

$self->open_tempfile(EXP, ">> $self->{'config_hash'}->{'exports_file'}");
$self->print_tempfile(EXP, $self->export_line($_[0]));
$self->close_tempfile(EXP);
}

# modify_export(&old, &new)
sub modify_export
{
	my $self = shift;

$self->replace_file_line($self->{'config_hash'}->{'exports_file'}, $_[0]->{'line'},
		   $self->export_line($_[1]));
}

# delete_export(&export)
sub delete_export
{
	my $self = shift;

$self->replace_file_line($self->{'config_hash'}->{'exports_file'}, $_[0]->{'line'});
}

# export_line(&export)
sub export_line
{
	my $self = shift;

local %exp = %{$_[0]};
local $rv = join(' ', @{$exp{'dirs'}});
if ($exp{'alldirs'}) { $rv .= " -alldirs"; }
if ($exp{'ro'}) { $rv .= " -ro"; }
if ($exp{'kerb'}) { $rv .= " -kerb"; }
if ($exp{'maproot'}) { $rv .= " -maproot $exp{'maproot'}"; }
if ($exp{'mapall'}) { $rv .= " -mapall $exp{'mapall'}"; }
if ($exp{'mask'}) { $rv .= " -network $exp{'network'} -mask $exp{'mask'}"; }
if ($exp{'public'}) { $rv .= " -public"; }
if ($exp{'webnfs'}) { $rv .= " -webnfs"; }
if ($exp{'index'}) { $rv .= " -index $exp{'index'}"; }
else { $rv .= " ".join(" ", @{$exp{'hosts'}}); }
$rv .= "\n";
return $rv;
}

# restart_mountd()
# Attempt to apply the NFS configuration, returning undef on success or an
# error message on failure
sub restart_mountd
{
	my $self = shift;

local $out = $self->backquote_logged("($self->{'config_hash'}->{'restart_command'}) </dev/null 2>&1");
if ($?) {
	return "<pre>$out</pre>";
	}
return undef;
}


1;

