#!/usr/bin/perl
package clamav::ClamavLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'clamav::ClamavLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# clamav-lib.pl
# Common functions for clamav configuration

# main part
&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

#--- Confmaster funcs ---

1;
1;
		return $self;
}

# &get_version_clamav()
# get version string of daemon clamavd
sub get_version_clamav
{
	my $self = shift;

 return `$self->{'config_hash'}->{'clamav_cmd'} --version 2>&1`;
}

# &get_version_number_clamav()
# get version of daemon clamavd
sub get_version_number_clamav
{
	my $self = shift;

 $self->get_version_clamav() =~ /^.*?(\d\.\d{1,3}?\.\d{1,3}?)/;
 return $1;
}

# &status_clamav_daemon()
# return 0 if daemon is stopped
# return 1 if daemon is running
sub status_clamav_daemon
{
	my $self = shift;

 return $self->status_daemon($self->{'config_hash'}->{'clamav_daemon'}, $self->{'config_hash'}->{'clamav_pid_file'});
}

# &status_freshclamav_daemon()
# return 0 if daemon is stopped
# return 1 if daemon is running
sub status_freshclamav_daemon
{
	my $self = shift;

 return $self->status_daemon($self->{'config_hash'}->{'freshclamav_daemon'}, $self->{'config_hash'}->{'freshclamav_pid_file'});
}

# &status_clamsmtpd_daemon()
# return 0 if daemon is stopped
# return 1 if daemon is running
sub status_clamsmtpd_daemon
{
	my $self = shift;

 return $self->status_daemon($self->{'config_hash'}->{'clamsmtpd_daemon'}, $self->{'config_hash'}->{'clamsmtpd_pid_file'});
}

# &status_daemon($daemon, $pid)
# return 0 if daemon is stopped
# return 1 if daemon is running
sub status_daemon
{
	my $self = shift;

 if (-e $_[0])
 {
  # use init script
  my $out = $self->backquote_logged("$_[0] status 2>&1 </dev/null");
  if ($? || $out=~/stopped/ || $out=~/is not running/) { return 0; }
  else { return 1; }
 }
 else
 {
  # use PID file 
  return 1 if (-e $_[1]);
  return 0;
 }
}

# &start_clamav_daemon()
# start daemon clamd
sub start_clamav_daemon
{
	my $self = shift;

 unless ($self->status_clamav_daemon())
 {
  $self->start_daemon($self->{'config_hash'}->{'clamav_daemon'}, $self->{'config_hash'}->{'clamav_cmd'});
 }
 else { $self->error($self->text('process_status', 'clamd')); }
}

# &start_freshclamav_daemon()
# start daemon freshclamav
sub start_freshclamav_daemon
{
	my $self = shift;

 unless ($self->status_freshclamav_daemon())
 {
  $self->start_daemon($self->{'config_hash'}->{'freshclamav_daemon'},$self->{'config_hash'}->{'freshclam_cmd'},'-d');
 }
 else { $self->error($self->text('process_status', 'freshclam')); }
}

# &start_clamsmtpd_daemon()
# start daemon clamsmtpd
sub start_clamsmtpd_daemon
{
	my $self = shift;

 unless ($self->status_clamsmtpd_daemon())
 {
  $self->start_daemon($self->{'config_hash'}->{'clamsmtpd_daemon'}, $self->{'config_hash'}->{'clamsmtpd_cmd'});
 }
 else { $self->error($self->text('process_status', 'clamsmtpd')); }
}

# &start_daemon($daemon, [$cmd], [$options])
# start clamav daemons
sub start_daemon
{
	my $self = shift;

 # use init script
 my $out = $self->backquote_logged("$_[0] start 2>&1 </dev/null");
 if ($?)
 {
  # use command 
  if ($_[1])
  {
   $out = $self->backquote_logged("$_[1] $_[2] 2>&1 </dev/null");
  }
  $self->error($out) if ($?);
 }
 $self->webmin_log("start");
}

# &restart_clamav_daemon()
# restart daemon clamd
sub restart_clamav_daemon
{
	my $self = shift;

 if ($self->status_clamav_daemon())
 {
  $self->restart_daemon($self->{'config_hash'}->{'clamav_daemon'}, $self->{'config_hash'}->{'clamav_pid_file'});
 }
 else { $self->error($self->text('process_status2'), 'clamd'); }
}

# &restart_freshclamav_daemon()
# restart daemon freshclamav
sub restart_freshclamav_daemon
{
	my $self = shift;

 if ($self->status_freshclamav_daemon())
 {
  $self->restart_daemon($self->{'config_hash'}->{'freshclamav_daemon'}, $self->{'config_hash'}->{'freshclamav_pid_file'});
 }
 else { $self->error($self->text('process_status2'), 'freshclamav'); }
}

# &restart_clamsmtpd_daemon()
# restart daemon clamsmtpd
sub restart_clamsmtpd_daemon
{
	my $self = shift;

 if ($self->status_clamsmtpd_daemon())
 {
  $self->restart_daemon($self->{'config_hash'}->{'clamsmtpd_daemon'}, $self->{'config_hash'}->{'clamsmtpd_pid_file'});
 }
 else { $self->error($self->text('process_status2'), 'clamsmtpd'); }
}

# &restart_daemon($daemon, $pid)
# restart clamav daemons
sub restart_daemon
{
	my $self = shift;

 # use init script
 my $out = $self->backquote_logged("$_[0] restart 2>&1 </dev/null");
 if ($?)
 {
  # use PID
  my $pid = $self->check_pid_file($_[1]);
  if ($pid)
  {
   $out = $self->kill_logged('HUP',$pid);
   $self->error($out) if ($?);
   $self->webmin_log("restart");
  }
  else {$self->error($out)};
 }
 else { $self->webmin_log("restart"); }
}

# &stop_clamav_daemon()
# stop daemon clamd
sub stop_clamav_daemon
{
	my $self = shift;

 if ($self->status_clamav_daemon())
 {
  $self->stop_daemon($self->{'config_hash'}->{'clamav_daemon'}, $self->{'config_hash'}->{'clamav_pid_file'});
 }
 else { $self->error($self->text('process_status2'), 'clamd'); }
}

# &stop_freshclamav_daemon()
# stop daemon freshclamav
sub stop_freshclamav_daemon
{
	my $self = shift;

 if ($self->status_freshclamav_daemon())
 {
  $self->stop_daemon($self->{'config_hash'}->{'freshclamav_daemon'}, $self->{'config_hash'}->{'freshclamav_pid_file'});
 }
 else { $self->error($self->text('process_status2'), 'freshclam'); }
}

# &stop_clamsmtpd_daemon()
# stop daemon clamsmtpd
sub stop_clamsmtpd_daemon
{
	my $self = shift;

 if ($self->status_clamsmtpd_daemon())
 {
  $self->stop_daemon($self->{'config_hash'}->{'clamsmtpd_daemon'}, $self->{'config_hash'}->{'clamsmtpd_pid_file'});
 }
 else { $self->error($self->text('process_status2'), 'clamsmtpd'); }
}

# &stop_daemon($daemon, $pid)
# stop clamav daemons
sub stop_daemon
{
	my $self = shift;

 # use init script
 my $out = $self->backquote_logged("$_[0] stop 2>&1 </dev/null");
 if ($?)
 {
  # use PID
  my $pid = $self->check_pid_file($_[1]);
  if ($pid)
  {
   $out = $self->kill_logged('TERM',$pid);
   $self->error($out) if ($?);
   $self->webmin_log("stop");
  }
  else {$self->error($out)};
 }
 else {$self->webmin_log("stop");}
}

sub get_config {
	my $self = shift;

	local %hs;
	$self->{'conf'} = $self->parse_config_file($_[0]);
	foreach (@{$self->{'conf'}}) {
		$self->{'name'} = $_->{'name'}; chomp $self->{'name'};
		$self->{'value'} = $_->{'value'}; chomp $self->{'value'};
		$self->{'value'} =~ s/[\"|\']//g;
		$hs{$self->{'name'}} = $self->{'value'};
	}
	return %hs;
}

# &show_clam_config ("clamav" || "freshclamav");
sub show_clam_config {
	my $self = shift;

	local ($name_conf, $number_group) = ("$_[0]_conf", $_[1]);

	%{$self->{'conf_hash'}} = $self->get_config($self->{'config_hash'}->{$name_conf});
	
	%{$self->{'param_hash'}} = $self->get_hash_param($_[0]);
	@{$self->{'groups_array'}} = $self->get_group_settings($_[0]);

	# check version
	my $ver = $self->get_version_number_clamav();
	my $new_config = $self->check_version_support($ver, '0.90.2');

	print $self->ui_form_start('save.cgi', 'POST');
	print $self->ui_hidden('conf_file', $self->{'config_hash'}->{$name_conf});
	print $self->ui_hidden('config', $_[0]);
	print $self->ui_table_start("$self->{'groups_array'}->[$number_group-1]", 'width=80%');
	foreach $key (sort keys %{$self->{'param_hash'}}) {
			$self->{'st'} = '';
			next if ($self->{'param_hash'}->{$key}->[0] != $number_group);
			if ($self->{'param_hash'}->{$key}->[1] == '0') {
					if ($new_config){
						# from version 0.90.2
						my $yes = 1 if (($self->{'conf_hash'}->{$key} eq '1') || ($self->{'conf_hash'}->{$key} =~ /^yes$/i) || ($self->{'conf_hash'}->{$key} =~ /^true$/i));
						my $no = 1 if (($self->{'conf_hash'}->{$key} eq '0') || ($self->{'conf_hash'}->{$key} =~ /^no$/i) || ($self->{'conf_hash'}->{$key} =~ /^false$/i));
						$self->{'st'} .= sprintf ("<input type=radio name=\"$key\" value=\"1\" %s>Yes\n", $yes ? 'checked' : 'unchecked');
						$self->{'st'} .= sprintf ("<input type=radio name=\"$key\" value=\"\" %s>No\n", $no ? 'checked' : 'unchecked');
						}
					else {
						# old config parser
						$self->{'st'} .= sprintf ("<input type=radio name=\"$key\" value=\"1\" %s>Yes\n", exists $self->{'conf_hash'}->{$key} ? 'checked' : 'unchecked');
						$self->{'st'} .= sprintf ("<input type=radio name=\"$key\" value=\"\" %s>No\n", ! exists $self->{'conf_hash'}->{$key} ? 'checked' : 'unchecked');
						}
					print $self->ui_table_row("$key", $self->{'st'});
				} elsif ($self->{'param_hash'}->{$key}->[1] == '1') {
					$self->{'st'} .= sprintf ("<input type=radio name=\"rb$key\" value=\"\" %s>Default\n", ! exists $self->{'conf_hash'}->{$key} ? 'checked' : 'unchecked');
					$self->{'st'} .= sprintf ("<input type=radio name=\"rb$key\" value=\"1\" %s>", exists $self->{'conf_hash'}->{$key} ? 'checked' : 'unchecked');
					$self->{'st'} .= sprintf ("<input type=text name=\"$key\" value=\"%s\">", $self->{'conf_hash'}->{$key});
				    print $self->ui_table_row("$key", $self->{'st'});
				} elsif ($self->{'param_hash'}->{$key}->[1] == '2') {
					$self->{'st'} .= sprintf ("<input type=text name=\"$key\" value=\"%s\">", $self->{'conf_hash'}->{$key});
				    print $self->ui_table_row("$key", $self->{'st'});
 				}
		}
		print $self->ui_table_end();
	print $self->ui_form_end([['save', $self->{'text_hash'}->{'save'}]]);
}

#&save_config ($self->{'conf_file'});
sub save_config {
	my $self = shift;

	local %param_in = %{$_[0]};
	$self->{'conf_file'} = $param_in{'conf_file'};
	$self->{'config'} = $param_in{'config'};
	%{$self->{'param_hash'}} = $self->get_hash_param($self->{'config'});
	$self->{'n'} = "\n";
	delete($param_in{'conf_file'});
	delete($param_in{'config'});
	delete($param_in{'save'});
	@{$self->{'key_in_array'}} = keys %param_in;
	foreach (@{$self->{'key_in_array'}}) {
		$param_in{"$_"} = '' if (exists $param_in{"rb$_"} && !$param_in{"rb$_"});
		delete($param_in{"rb$_"}) if (exists $param_in{"rb$_"});
	}
	
	# check version
	my $ver = $self->get_version_number_clamav();
	my $new_config = $self->check_version_support($ver, '0.90.2');

	@{$self->{'key_in_array'}} = keys %param_in;
	$self->lock_file($self->{'conf_file'});
	my $old_file= $self->read_file_lines($self->{'conf_file'});
 	foreach $ln (@{$old_file}) {
		foreach (@{$self->{'key_in_array'}}) {
			if ($ln =~ m/^$_\s+/ || $ln =~ m/^$_\=/ || $ln =~ m/^$_$/) { # it was enabled
				if ($param_in{$_} eq '') {# have to disable parametr
					if ($self->{'param_hash'}->{$_}->[1] == 0 && $new_config) {
						$ln = "$_ no";
						}
					elsif ($self->{'param_hash'}->{$_}->[1] == 2) {
						$ln = "$_=\"\"";
						}
					else { $ln = ""; }
			   } else { #have to edit
						if ($self->{'param_hash'}->{$_}->[1] == 0){
							$ln = $new_config ? "$_ yes" : "$_";
							}
						$ln = "$_ $param_in{$_}" if ($self->{'param_hash'}->{$_}->[1] == 1);
						$ln = "$_=\"$param_in{$_}\"" if ($self->{'param_hash'}->{$_}->[1] == 2);
				    	}
				delete ($param_in{$_});
		    }
		}
	}

	@{$self->{'key_in_array'}} = keys %param_in;
 	foreach $ln (@{$old_file}) {
		foreach (@{$self->{'key_in_array'}}) {
			 if ($ln =~ m/^#\s*$_\s+/ || $ln =~ m/^#\s*$_$/) { # it was disable
				if ($new_config){
						# from version 0.90.2
						if ($param_in{$_} ne '') { #have to add to file
							$self->{'have_to_add'} = 1;
							$self->{'add_ln'} = "$_ yes" if ($self->{'param_hash'}->{$_}->[1] == 0);
							$self->{'add_ln'} = "$_ $param_in{$_}" if ($self->{'param_hash'}->{$_}->[1] == 1);
							}
						else {
							if ($self->{'param_hash'}->{$_}->[1] == 0){
								$self->{'have_to_add'} = 1;
								$self->{'add_ln'} = "$_ no";
								}
							}
						}						
				else {
					# old versions
					if ($param_in{$_} ne '') { #have to add to file
						$self->{'have_to_add'} = 1;
						$self->{'add_ln'} = "$_" if ($self->{'param_hash'}->{$_}->[1] == 0);
						$self->{'add_ln'} = "$_ $param_in{$_}" if ($self->{'param_hash'}->{$_}->[1] == 1);
						}
					}
				delete ($param_in{$_});
				}
			 if ($self->{'have_to_add'} && $ln =~ m/^\s*$/) {
				$ln = $self->{'add_ln'};
				$self->{'have_to_add'} = 0;
				}
		}
	}

	@{$self->{'key_in_array'}} = keys %param_in;
	foreach (@{$self->{'key_in_array'}}) {
		if ($new_config){
			# from version 0.90.2
			if ($param_in{$_} eq ''){
				if ($self->{'param_hash'}->{$_}->[1] == 0){
					push (@{$old_file}, "$_ no") if ($self->{'param_hash'}->{$_}->[1] == 0);
					}
				else { next; }
				}
			else {
				push (@{$old_file}, "$_ yes") if ($self->{'param_hash'}->{$_}->[1] == 0);
				push (@{$old_file}, "$_ $param_in{$_}") if ($self->{'param_hash'}->{$_}->[1] == 1);
				}
			}
		else {
			# old versions
			next if ($param_in{$_} eq '');
			push (@{$old_file}, $_) if ($self->{'param_hash'}->{$_}->[1] == 0);
			push (@{$old_file}, "$_ $param_in{$_}") if ($self->{'param_hash'}->{$_}->[1] == 1);
			}
	}
 	$self->flush_file_lines();
 	$self->unlock_file($self->{'conf_file'});
}

#  &get_last_db (file_db)
sub get_last_db {
	my $self = shift;

  my $file_db = $_[0];
  my ($date, $info, $version, $level) = ();
  if (-e $file_db) {
    $date = $self->make_date((stat($file_db))[9]);
 	@{$self->{'mas_array'}} = split(':', `sigtool --info $file_db 2>&1`);
 	chomp(@{$self->{'mas_array'}});
  	foreach $ln (@{$self->{'mas_array'}}) {
 		$version = $ln if ($ln =~ s/#\s*of\s*signatures//);
		$level = $ln if ($ln =~ s/\s*Builder//);
  	}
    $info = "$self->{'text_hash'}->{'update_version'}: $version - $self->{'text_hash'}->{'update_level'}: $level";
  }
  return ($date, $info);
}

sub clamav_daemon_table {
	my $self = shift;

  my $freq = $_[0];
  $self->{'st'} = "<select name=\"freq\">";
  foreach (1..50)
  {
    $self->{'default'} = ($_ == $freq) ? ' SELECTED' : '';
    $self->{'st'} .= qq(<option value="$_"$self->{'default'}>$_</option>\n);
  }
  $self->{'st'} .= qq(</select>);
  return $self->{'st'};
}

sub clamav_update {
	my $self = shift;

  print $self->ui_table_start("$self->{'text_hash'}->{'update_report_table'}", 'width=100%');
  open (H, "freshclam 2>&1 |");
  while (<H>)
  {
    s/^*\n$/<br>/g;
    if (/load/) {print "."} else {print};
  }
  close (H);
  print $self->ui_table_end();
}

sub clamav_update_auto {
	my $self = shift;

	local ($update_type, %tmp_in, %job) = ($_[0], %{$_[1]});
	if (-f $self->{'config_hash'}->{'fresh_daemon_conf'}) {
		%{$self->{'conf_hash'}} = $self->get_config($self->{'config_hash'}->{'fresh_daemon_conf'});
		} else {
			%{$self->{'conf_hash'}} = $self->get_config($self->{'config_hash'}->{'freshclamav_conf'});
			}
   	$self->{'command'} = "\$($self->{'config_hash'}->{'freshclam_cmd'} --quiet --config-file $self->{'config_hash'}->{'freshclamav_conf'} --log $self->{'conf_hash'}->{'UpdateLogFile'} --datadir $self->{'conf_hash'}->{'DatabaseDirectory'})";
	$self->freshclamav_clear_cron($self->{'command'});
	if ($update_type == 2) {
    		if ($self->{'config_hash'}->{'freshclamav_stop'}) {
	            system ("$self->{'config_hash'}->{'freshclamav_stop'}");
        	    } else  {
            		$self->kill_byname_logged("$self->{'config_hash'}->{'freshclam_cmd'}", TERM);
            	    }
		$self->foreign_require('cron', 'cron-lib.pl');
		$main::cron_object->parse_times_input(\%job, \%tmp_in);
 		$job{'user'} = 'root';
 		$job{'active'} = 1;
		$job{'command'} = $self->{'command'};
 		$self->lock_file($main::cron_object->cron_file(\%job));
 		$main::cron_object->create_cron_job(\%job);
 		$self->unlock_file($main::cron_object->cron_file(\%job));
		}
	if ($update_type == 1) {
		$self->freshclamav_clear_cron($self->{'command'});
		$self->freshclamav_set_checks($tmp_in{'freq'});
		}
}

sub freshclamav_set_checks {
	my $self = shift;

	local $checks = $_[0];
	if (-f $self->{'config_hash'}->{'fresh_daemon_conf'}) {
		%{$self->{'conf_hash'}} = $self->get_config($self->{'config_hash'}->{'fresh_daemon_conf'});
		%{$self->{'conf_hash'}} = ('conf_file' => "$self->{'config_hash'}->{'fresh_daemon_conf'}",
				 'config' => 'fresh_daemon',
			 	 'CHECKS' => $checks);
		} else {
			%{$self->{'conf_hash'}} = $self->get_config($self->{'config_hash'}->{'freshclamav_conf'});
			%{$self->{'conf_hash'}} = ('conf_file' => "$self->{'config_hash'}->{'freshclamav_conf'}",
				     'config' => 'freshclamav',
		 	   		 'Checks' => $checks);
			}
 	$self->save_config(\%{$self->{'conf_hash'}});
        if ($self->{'config_hash'}->{'freshclamav_stop'}) {
            system ("$self->{'config_hash'}->{'freshclamav_stop'}");
            } else  {
             $self->kill_byname_logged("$self->{'config_hash'}->{'freshclam_cmd'}", TERM);
             }
        if ($self->{'config_hash'}->{'freshclamav_start'}) {
            system ("$self->{'config_hash'}->{'freshclamav_start'}");
            } else  {
                system ("$self->{'config_hash'}->{'freshclam_cmd'} -d");
             }	
}

sub freshclamav_clear_cron {
	my $self = shift;

		local $command = $_[0];
		$self->foreign_require("cron", "cron-lib.pl");
		local @jobs = $main::cron_object->list_cron_jobs();
		local ($job) = grep { $_->{'user'} eq 'root' &&
						      $_->{'command'} eq $command } @jobs;
		$main::cron_object->delete_cron_job($job);
}

sub get_freshclamav_update_status {
	my $self = shift;

	local ($daemon, $cron);
	if (-f $self->{'config_hash'}->{'fresh_daemon_conf'}) {
		%{$self->{'conf_freshclamav_hash'}} = $self->get_config($self->{'config_hash'}->{'fresh_daemon_conf'});
		$daemon = defined ($self->{'conf_freshclamav_hash'}->{'CHECKS'}) ? $self->{'conf_freshclamav_hash'}->{'CHECKS'} : 0;
	} else {
		%{$self->{'conf_freshclamav_hash'}} = $self->get_config($self->{'config_hash'}->{'freshclamav_conf'});
		$daemon = defined ($self->{'conf_freshclamav_hash'}->{'Checks'}) ? $self->{'conf_freshclamav_hash'}->{'Checks'} : 0;
		}

	$self->{'command'} = "\$($self->{'config_hash'}->{'freshclam_cmd'} --quiet --config-file $self->{'config_hash'}->{'freshclamav_conf'} --log $self->{'conf_freshclamav_hash'}->{'UpdateLogFile'} --datadir $self->{'conf_freshclamav_hash'}->{'DatabaseDirectory'})";
	$self->foreign_require("cron", "cron-lib.pl");
	local @jobs = $main::cron_object->list_cron_jobs();
	local ($job) = grep { $_->{'user'} eq 'root' &&
					      $_->{'command'} eq $self->{'command'} } @jobs;

	$self->{'cron_job'} = $job if ($job);
	return ($daemon, $self->{'cron_job'})
}

sub check_dir {
	my $self = shift;

	local %tmp_in = %{$_[0]};
	local $only_list_files = $_[1];
	$self->error($self->{'text_hash'}->{'local_check_error1'}) unless (-r $tmp_in{'dir'});
	if ($tmp_in{'move'}) {
		$self->error($self->{'text_hash'}->{'local_check_error2'}) unless ( -r $self->{'config_hash'}->{'clamav_quarantine'});
		$self->{'config_hash'}->{'clamav_quarantine'} .= '/' if ($self->{'config_hash'}->{'clamav_quarantine'} !~ m/\/$/);
		}

	local ($dir, $sub_dir, $show, $move_dir) = ($tmp_in{'dir'},
												($tmp_in{'sub'}) ? ' -r ' : '',
												($tmp_in{'show'}) ? '1' : '0',
												($tmp_in{'move'}) ? " --move $self->{'config_hash'}->{'clamav_quarantine'}" : '');
	local $cmd = "$self->{'config_hash'}->{'clamscan_cmd'} $move_dir $sub_dir $dir";
	local $report = 0;


	$self->{'number_form'} = ($only_list_files) ? 0 : 1;
    print $self->select_all_link("infect", $self->{'number_form'})."&nbsp\n";
    print $self->select_invert_link("infect", $self->{'number_form'});

	if ($only_list_files) {
		print $self->ui_table_start("$self->{'text_hash'}->{'quarantine_table'}", 'width=100%');
		print $self->ui_form_start('quarantine.cgi', 'GET');
	} else {
			print $self->ui_table_start("$self->{'text_hash'}->{'local_check_cmd'} $cmd", 'width=100%');
			print $self->ui_form_start('local_check.cgi', 'GET');
			}

  	open (H, "$cmd 2>&1 |");
   	while (<H>)  {
    	/^(.*):(.*)$/;
    	my ($file, $state) = ($1, $2);
       	$file =~ m/([^\/]+)$/;
		my $tmp_file = "$self->{'config_hash'}->{'clamav_quarantine'}$1";
		if ($state =~ s/FOUND//) {
			$self->{'infect'}++;
			printf ($self->ui_table_row("$file", "%s<font color='red'>$state</font>", 2), ($tmp_in{'move'}) ? qq(<input type="checkbox" name="infect" value="$tmp_file">) : qq(<input type="checkbox" name="infect" value="$file">));
		} elsif ($state =~ m/OK/) {
				print $self->ui_table_row("$file", "<font color='green'>$state</font>", 2) if (! $show);
				}
     if ($file =~ /known viruses/i && $report == 0) {
		$report = 1;
		print $self->ui_table_end();
		print $self->ui_form_end([['delete', $self->{'text_hash'}->{'local_check_delete'}]]);
		print "<br>";
		print $self->ui_table_start("$self->{'text_hash'}->{'local_check_report'}", 'width=100%') unless $only_list_files;
	 }

	 if ($report && !$only_list_files) {
	 	$file =~ tr/a-z/A-Z/;
        $file =~ s/ /_/g;
		print $self->ui_table_row("$file", "$state", 2);
	 }
	}
	print $self->ui_table_end() if ($report);
	close (H);
}

sub clamav_viruses_count
{
	my $self = shift;

  my $count = `sigtool --list-sigs | wc -l`;
  $count =~ s/ //g;
  chomp $count;
  return $count;
}

sub clamav_vdb_search {
	my $self = shift;

   my ($virus, $ext, $case, $sort,$page,$bl) = @_;
   my $limit=50;
   $page= $page ? $page : 1;
   $bl=$bl? $bl:0;
   $case = ($case) ? ' ' : ' -i ';
   $sort = ($sort) ? ' sort |' : '';

   $self->{'command'} = "sigtool --list-sigs";
   $self->{'command'} .= " | grep $case " if ($virus ne '' && $case);
   $self->{'command'} .= $ext ? " \"^$virus\$\" " : " $virus " if ($virus ne '');
   $self->{'command'} .= " | $sort "  if ($virus ne '' && $sort);

   @{$self->{'mas_array'}} = `$self->{'command'}`; chomp (@{$self->{'mas_array'}});
   return 0 unless scalar(@{$self->{'mas_array'}});

   my $pages=sprintf("%.f",$#{$self->{'mas_array'}}/$limit);
   my $bloks=sprintf("%.f",$pages/10);
if($pages>1){
    if($bl!=0) { 
    print "<a href=\"?name=$virus&limit=$limit&cace=$case&sort=$sort&ext=$ext&page=0&bl=0&submit=Search\" ><img src=/images/d_left.gif border=0 align=middle></a>" if($bl!=0);
    }

my $st=$bl*10+1;
my $en=$bl*10+10;
$en=$pages if($en>$pages);

if($bl!=0){
	my $ppage=$st-10;
	print "<a href=\"?name=$virus&limit=$limit&cace=$case&sort=$sort&ext=$ext&page=$ppage&bl="; print $bl-1; print "&submit=Search\"><img src=/images/left.gif border=0 align=middle></a>";} 

    for ($self->{'i'}=$st; $self->{'i'}<=$en ;$self->{'i'}++){	
	if($page!=$self->{'i'}){
	print "<a href=\"?name=$virus&limit=$limit&cace=$case&sort=$sort&ext=$ext&page=$self->{'i'}&bl=$bl&submit=Search\" >"."[",$self->{'i'}*$limit-$limit,"..",$self->{'i'}*$limit,"]</a>";
	}else{
	print "<b>[",$self->{'i'}*$limit-$limit,"..",$self->{'i'}*$limit,"]</b>";
	}
    }
my $npage=$self->{'i'}++;
print "<a href=\"?name=$virus&limit=$limit&cace=$case&sort=$sort&ext=$ext&page=$npage&bl=".++$bl."&submit=Search\" ><img src=/images/right.gif border=0 align=middle></a>" if($en<$pages);
print "<a href=\"?name=$virus&limit=$limit&cace=$case&sort=$sort&page=$pages&ext=$ext&bl=$bloks&submit=Search\" ><img src=/images/d_right.gif border=0 align=middle></a>" if( $npage<$pages );
$self->{'command'} .=" | head -n ".$limit*$page." |tail -n $limit";
@{$self->{'mas_array'}} = `$self->{'command'}`; chomp (@{$self->{'mas_array'}});
}

print qq(<table width="100%"><tr $self->{'tb'}>					<td>#</td>
									<td><b>$self->{'text_hash'}->{'search_name'}</b></td>
									<td><b>Virus Alias Database</b></td>
									<td><b>Symantec</b></td>
									<td><b>McAfee</b></td>
									<td><b>Viruspool</b></td>
									<td><b>Viruslist</b></td>
							</tr>);
my $n=1;
foreach (@{$self->{'mas_array'}}) {
	my $name_pure = $self->urlize( $self->clamav_get_pure_name($_));
my $name = $self->urlize($_);

    my $vdb_url = "http://www.rainingfrogs.co.uk/index.php?virus=$name&search=contains";
my $symantec_url ="http://search.symantec.com/custom/us/query.html?" .
	 "lk=1&rf=0&qt=$name_pure&qp=" . $self->urlize("url:http://securityresponse.symantec.com/avcenter/venc/data url:/avcenter/venc/data url:/avcenter/venc/auto/index");
my $mcafee_url ="http://us.mcafee.com/virusInfo/default.asp?id=alphar&SearchType=2&searchString=$name_pure";
my $viruspool_url =	"http://viruspool.vanderkooij.org/virus.cms?name=$name";
my $viruslist_url = "http://www.viruslist.com/en/find?search_mode=virus&words=$name_pure";

my $np=$page*$limit-$limit+$n++;
	print qq(<tr>
				<td $self->{'cb'}>$np </td>
				<td $self->{'cb'}>$_</td>
				<td $self->{'cb'}><a href="$vdb_url" target="_BLANK">$self->{'text_hash'}->{'search_more'}</a></td>
				<td $self->{'cb'}><a href="$symantec_url" target="_BLANK">$self->{'text_hash'}->{'search_more'}</a></td>
				<td $self->{'cb'}><a href="$mcafee_url" target="_BLANK">$self->{'text_hash'}->{'search_more'}</a></td>
				<td $self->{'cb'}><a href="$viruspool_url" target="_BLANK">$self->{'text_hash'}->{'search_more'}</a></td>
				<td $self->{'cb'}><a href="$viruslist_url" target="_BLANK">$self->{'text_hash'}->{'search_more'}</a></td>
			</tr>);
}
print qq(</table>);
}

sub clamav_get_pure_name {
	my $self = shift;

  my $name = shift;
  my ($b, $m, $e);
  if ($name =~ /^(.*)[\.\-](.*)[\.\-](.*)$/)  {
    ($b, $m, $e) = ($1, $2, $3);
     return ($m !~ /[a-z,A-Z]/ || length ($m) == 1) ?
        $self->clamav_get_pure_name("$b") : $m;
  	}

  if ($name =~ /^(.*)[\.\-](.*)$/)  {
    ($b, $e) = ($1, $2);
    if ($e !~ /[a-z,A-Z]/) {return $b;} elsif ($e) {return $e;}
  	}

  return $1 if ($name =~ /^(.*)[\.\-].*$/);
  return $name;
}

sub show_log {
	my $self = shift;

	my ($file, $lines) = ($_[0], $_[1]);
	print $self->ui_form_start('log_view.cgi', 'POST');
	print $self->ui_hidden('logs', $file);
	print $self->ui_table_start("$self->{'text_hash'}->{'log_info'}", 'width=100%');
	if ($lines) {
		@{$self->{'text_array'}} = `/usr/bin/tail -n $lines $file`;
		} else {
  			open (H, "$file");
	    	@{$self->{'text_array'}} = <H>;
		  	close (H);
			}
	print $self->ui_table_row(qq(<textarea cols=80 rows=30>@{$self->{'text_array'}}</textarea>\n), "");
	print $self->ui_table_span();
	print $self->ui_table_end();
	print $self->ui_form_end([['clear', $self->{'text_hash'}->{'log_clear'}]]);
}

sub get_hash_param {
	my $self = shift;


    # name => (group, type value)
    # 0 - check, 1 - string, 2 - file
    my %param = (
      'DatabaseOwner' => [1, 1],
      'AllowSupplementaryGroups' => [1, 0],
      'DatabaseDirectory' => [1, 1],
      'Checks' => [1, 1],
      'UpdateLogFile' => [1, 1],
      'LogSyslog' => [2, 0],
      'LogFacility' => [2, 1],
      'LogVerbose' => [2, 0],
      'DNSDatabaseInfo' => [1, 1],
      'DatabaseMirror' => [1, 1],
      'MaxAttempts' => [1, 1],
      'HTTPProxyServer' => [3, 1],
      'HTTPProxyPort' => [3, 1],
      'HTTPProxyUsername' => [3, 1],
      'HTTPProxyPassword' => [3, 1],
		'HTTPUserAgent' => [3, 1, '0.90.2'],
      'LocalIPAddress' => [1, 1],
      'NotifyClamd' => [1, 1],
      'OnUpdateExecute' => [1, 1],
      'OnErrorExecute' => [1, 1],
    ) if ($_[0] eq 'freshclamav');

    %param = (
      'LogFile' => [1, 1],
      'LogFileUnlock' => [1, 0],
      'LogFileMaxSize' => [1, 1],
      'LogTime' => [1, 0],
      'LogClean' => [1, 0],
      'LogSyslog' => [1, 0],
      'LogFacility' => [1, 1],
      'LogVerbose' => [1, 0],
      'PidFile' => [2, 1],
      'DataDirectory' => [2, 1],
      'LocalSocket' => [2, 1],
      'FixStaleSocket' => [2, 0],
      'TCPSocket' => [2, 1],
      'TCPAddr' => [2, 1],
      'MaxConnectionQueueLength' => [3, 1],
      'MaxThreads' => [3, 1],
      'ReadTimeout' => [3, 1],
      'MaxDirectoryRecursion' => [3, 1],
      'FollowDirectorySymlinks' => [2, 0],
      'FollowFileSymlinks' => [2, 0],
      'SelfCheck' => [2, 1],
      'VirusEvent' => [2, 1],
      'ExitOnOOM' => [2, 0],
      'User' => [2, 1],
      'AllowSupplementaryGroups' => [2, 0],
      'Foreground' => [2, 0],
      'Debug' => [2, 0],
      'StreamMaxLength' => [3, 1],
      'StreamMinPort' => [3, 1],
      'StreamMaxPort' => [3, 1],
      'DisableDefaultScanOptions' => [2, 0, 0,'0.90.2'],
      'ScanPE' => [6, 0],
      'DetectBrokenExecutables' => [2, 0],
      'ScanOLE2' => [6, 0],
      'ScanHTML' => [6, 0],
      'ScanMail' => [6, 0],
		'ScanPDF' => [6, 0, '0.90.2'],
		'ScanELF' => [6, 0, '0.90.2'],
      'MailFollowURLs' => [2, 0],
      'ScanArchive' => [6, 0],
      'ScanRAR' => [6, 0, 0,'0.90.2'],
      'ArchiveMaxFileSize' => [5, 1],
      'ArchiveMaxRecursion' => [5, 1],
      'ArchiveMaxFiles' => [5, 1],
      'ArchiveMaxCompressionRatio' => [5, 1],
      'ArchiveLimitMemoryUsage' => [5, 0],
      'ArchiveBlockEncrypted' => [5, 0],
      'ArchiveBlockMax' => [5, 0],
		'ClamukoScanOnAccess' => [4, 0],
      'ClamukoScanOnLine' => [4, 0],
      'ClamukoScanOnOpen' => [4, 0],
      'ClamukoScanOnClose' => [4, 0],
      'ClamukoScanOnExec' => [4, 0],
      'ClamukoIncludePath' => [4, 1],
      'ClamukoExcludePath' => [4, 0],
      'ClamukoMaxFileSize' => [4, 1],
      'ClamukoScanArchive' => [4, 0]
    ) if ($_[0] eq 'clamav');

    %param = (
        'LOGFILE' => [1, 2],
        'CHECKS' => [1, 2],
        'HTTPPROXY' => [1, 2],
        'PROXYUSER' => [1, 2],
        'ONERROR' => [1, 2],
        'ONSUCCESS' => [1, 2],
        'OPTIONS' => [1, 2]
    ) if ($_[0] eq 'fresh_daemon');

 my $ver = $self->get_version_number_clamav();
 my %ret_param;
 foreach (keys %param) {
	my $old = $self->check_version_support($ver, $param{$_}->[3]) if ($param{$_}->[3]);
	$ret_param{$_} = $param{$_} if ($self->check_version_support($ver, $param{$_}->[2]) && !$old);
	}
 return %ret_param;
}

# &check_version_support($clamav_ver, $ver)
# return 1 if $clamav_ver >= $ver
sub check_version_support
{
	my $self = shift;

 my ($ver1, $ver2) = @_;
 my @ver1 = split(/\./, $ver1);
 my @ver2 = split(/\./, $ver2);
 return 1 if (($ver1[0]*1000000 + $ver1[1]*1000 + $ver1[2]) >= ($ver2[0]*1000000 + $ver2[1]*1000 + $ver2[2]));
}

sub get_group_settings {
	my $self = shift;

    @{$self->{'groups_clamav_array'}}=();
    foreach $i (1..6) {
    	 push (@{$self->{'groups_clamav_array'}}, $self->{'text_hash'}->{"group_clamav_$i"});
	}

    @{$self->{'groups_freshclamav_array'}}=();
    foreach $i (1..3) {
	 push (@{$self->{'groups_freshclamav_array'}}, $self->{'text_hash'}->{"group_freshclamav_$i"});
	}

	@{$self->{'groups_fresh_daemon_array'}}=();
    foreach $i (1..1) {
    	 push (@{$self->{'groups_fresh_daemon_array'}}, $self->{'text_hash'}->{"group_fresh_daemon_$i"});
	}

   return @{$self->{'groups_fresh_daemon_array'}} if ($_[0] eq 'fresh_daemon');
   return @{$self->{'groups_clamav_array'}} if ($_[0] eq 'clamav');
   return @{$self->{'groups_freshclamav_array'}} if ($_[0] eq 'freshclamav');
}

# &parse_config_file($conf_file)
# parses file $_[0] and returns a reference to an array of hashes (spliter - \s)
sub parse_config_file
{
	my $self = shift;

 my (@options, $conf_file);
 open (INFL,$_[0]) || $self->error($self->{'text_hash'}->{'error_open'});
 while (<INFL>)
 {
  chomp;
  $self->trunc_space($_);
  unless ($_=~/^#/ || $_=~/^$/)
  {
  	if ($_ =~ m/^[^\s]+\=/) {
		@options=split ('=',$_);
		} else {
   			@options=split (/\s+/,$_);
   			}
   $options[0]=$self->trunc_space($options[0]);
   $options[1]=$self->trunc_space($options[1]);
   push (@{$conf_file},{'name'=>$options[0],'value'=>$options[1]});
  }
 }
 close INFL;
 return $conf_file;
}

# &get_config_value($name, \@array)
# return 1 and value of parameter with name from array
sub get_config_value
{
	my $self = shift;

 my ($key, $value);
 foreach (@{$_[1]})
 {
  if ($_->{'name'} eq $_[0])
  {
   $key=1;
   $value=$_->{'value'};
  }
 }
 return ($key, $value);
}

# &set_config_value($name, $value, \@array)
# set new value of parameter with name from array
# or add new string to array
sub set_config_value
{
	my $self = shift;

 my $str_old;
 foreach (@{$_[2]})
 {
  if ($_->{'name'} eq $_[0])
  {
   $_->{'value'}=$_[1];
   $str_old=1;
  }
 }
 push (@{$_[2]},{'name'=>$_[0],'value'=>$_[1]}) if ($str_old!=1);
}

# &save_config_file(\@array, [$file])
# save array to file (spliter - \s)
sub save_config_file
{
	my $self = shift;

 my $conf_file = $_[1] ? $_[1] : $self->{'config_hash'}->{'clamav_conf'};
 
 # read file lines into hash
 $self->lock_file($conf_file);
 my $old_file= $self->read_file_lines($conf_file);
 
 # modify conf file options
 my (@new, %un);
 foreach (@{$old_file})
 {
  unless ($_=~/^#/ || $_=~/^$/)
  {
   my @str_old = split (/\s+/,$_);
   $str_old[0] = $self->trunc_space($str_old[0]);
   $str_old[1] = $self->trunc_space($str_old[1]);
	foreach my $i (@{$_[0]})
	{
	 if ($i->{'name'} eq $str_old[0])
    {
	  # change option
	  $un{$i->{'name'}} = 1;
	  if ($i->{'value'} eq '#')
	  {
	   # comment option
	   $_="# $i->{'name'}";
	  }
	  else
	  {
	   # change value
      $_="$i->{'name'} $i->{'value'}";
	  }
	  last;
    }
	}
  }
 }

 # new options
 foreach (@{$_[0]})
 {
  push (@new, $_) if (!$un{$_->{'name'}} && ($_->{'value'} ne '#')); 
 }
  
 # add change options
 foreach (@new)
 {
  my $comm;
  my $name = $_->{'name'};
  foreach my $i (@{$old_file})
  {
   if ($i =~ /^#\s*?$name/)
	{
	 # add after comment
	 $i = "$i\n$_->{'name'} $_->{'value'}";
	 $comm = 1;
	 last;
	}
  }
  # add in the end of file
  push (@{$old_file}, "$_->{'name'} $_->{'value'}") unless ($comm);
 }

 # save chage
 $self->flush_file_lines();
 $self->unlock_file($conf_file);
}

# &trunc_space($string)
# delete spaces from string
sub trunc_space
{
	my $self = shift;

 local $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}


1;

