# clamav-lib.pl
# Common functions for clamav configuration

# main part
do '../web-lib.pl';
&init_config();
require '../ui-lib.pl';

# &get_version_clamav()
# get version string of daemon clamavd
sub get_version_clamav
{
 return `$config{'clamav_cmd'} --version 2>&1`;
}

# &get_version_number_clamav()
# get version of daemon clamavd
sub get_version_number_clamav
{
 &get_version_clamav() =~ /^.*?(\d\.\d{1,3}?\.\d{1,3}?)/;
 return $1;
}

# &status_clamav_daemon()
# return 0 if daemon is stopped
# return 1 if daemon is running
sub status_clamav_daemon
{
 return &status_daemon($config{'clamav_daemon'}, $config{'clamav_pid_file'});
}

# &status_freshclamav_daemon()
# return 0 if daemon is stopped
# return 1 if daemon is running
sub status_freshclamav_daemon
{
 return &status_daemon($config{'freshclamav_daemon'}, $config{'freshclamav_pid_file'});
}

# &status_clamsmtpd_daemon()
# return 0 if daemon is stopped
# return 1 if daemon is running
sub status_clamsmtpd_daemon
{
 return &status_daemon($config{'clamsmtpd_daemon'}, $config{'clamsmtpd_pid_file'});
}

# &status_daemon($daemon, $pid)
# return 0 if daemon is stopped
# return 1 if daemon is running
sub status_daemon
{
 if (-e $_[0])
 {
  # use init script
  my $out = &backquote_logged("$_[0] status 2>&1 </dev/null");
  if ($? || $out=~/stopped/ || $out=~/is not running/) { return 0; }
  else { return 1; }
 }
 else
 {
  # use PID file 
  return 1 if (-e $_[1]);
  return 0;
 }
}

# &start_clamav_daemon()
# start daemon clamd
sub start_clamav_daemon
{
 unless (&status_clamav_daemon())
 {
  &start_daemon($config{'clamav_daemon'}, $config{'clamav_cmd'});
 }
 else { &error(&text('process_status', 'clamd')); }
}

# &start_freshclamav_daemon()
# start daemon freshclamav
sub start_freshclamav_daemon
{
 unless (&status_freshclamav_daemon())
 {
  &start_daemon($config{'freshclamav_daemon'},$config{'freshclam_cmd'},'-d');
 }
 else { &error(&text('process_status', 'freshclam')); }
}

# &start_clamsmtpd_daemon()
# start daemon clamsmtpd
sub start_clamsmtpd_daemon
{
 unless (&status_clamsmtpd_daemon())
 {
  &start_daemon($config{'clamsmtpd_daemon'}, $config{'clamsmtpd_cmd'});
 }
 else { &error(&text('process_status', 'clamsmtpd')); }
}

# &start_daemon($daemon, [$cmd], [$options])
# start clamav daemons
sub start_daemon
{
 # use init script
 my $out = &backquote_logged("$_[0] start 2>&1 </dev/null");
 if ($?)
 {
  # use command 
  if ($_[1])
  {
   $out = &backquote_logged("$_[1] $_[2] 2>&1 </dev/null");
  }
  &error($out) if ($?);
 }
 &webmin_log("start");
}

# &restart_clamav_daemon()
# restart daemon clamd
sub restart_clamav_daemon
{
 if (&status_clamav_daemon())
 {
  &restart_daemon($config{'clamav_daemon'}, $config{'clamav_pid_file'});
 }
 else { &error(&text('process_status2'), 'clamd'); }
}

# &restart_freshclamav_daemon()
# restart daemon freshclamav
sub restart_freshclamav_daemon
{
 if (&status_freshclamav_daemon())
 {
  &restart_daemon($config{'freshclamav_daemon'}, $config{'freshclamav_pid_file'});
 }
 else { &error(&text('process_status2'), 'freshclamav'); }
}

# &restart_clamsmtpd_daemon()
# restart daemon clamsmtpd
sub restart_clamsmtpd_daemon
{
 if (&status_clamsmtpd_daemon())
 {
  &restart_daemon($config{'clamsmtpd_daemon'}, $config{'clamsmtpd_pid_file'});
 }
 else { &error(&text('process_status2'), 'clamsmtpd'); }
}

# &restart_daemon($daemon, $pid)
# restart clamav daemons
sub restart_daemon
{
 # use init script
 my $out = &backquote_logged("$_[0] restart 2>&1 </dev/null");
 if ($?)
 {
  # use PID
  my $pid = &check_pid_file($_[1]);
  if ($pid)
  {
   $out = &kill_logged('HUP',$pid);
   &error($out) if ($?);
   &webmin_log("restart");
  }
  else {&error($out)};
 }
 else { &webmin_log("restart"); }
}

# &stop_clamav_daemon()
# stop daemon clamd
sub stop_clamav_daemon
{
 if (&status_clamav_daemon())
 {
  &stop_daemon($config{'clamav_daemon'}, $config{'clamav_pid_file'});
 }
 else { &error(&text('process_status2'), 'clamd'); }
}

# &stop_freshclamav_daemon()
# stop daemon freshclamav
sub stop_freshclamav_daemon
{
 if (&status_freshclamav_daemon())
 {
  &stop_daemon($config{'freshclamav_daemon'}, $config{'freshclamav_pid_file'});
 }
 else { &error(&text('process_status2'), 'freshclam'); }
}

# &stop_clamsmtpd_daemon()
# stop daemon clamsmtpd
sub stop_clamsmtpd_daemon
{
 if (&status_clamsmtpd_daemon())
 {
  &stop_daemon($config{'clamsmtpd_daemon'}, $config{'clamsmtpd_pid_file'});
 }
 else { &error(&text('process_status2'), 'clamsmtpd'); }
}

# &stop_daemon($daemon, $pid)
# stop clamav daemons
sub stop_daemon
{
 # use init script
 my $out = &backquote_logged("$_[0] stop 2>&1 </dev/null");
 if ($?)
 {
  # use PID
  my $pid = &check_pid_file($_[1]);
  if ($pid)
  {
   $out = &kill_logged('TERM',$pid);
   &error($out) if ($?);
   &webmin_log("stop");
  }
  else {&error($out)};
 }
 else {&webmin_log("stop");}
}

sub get_config {
	local %hs;
	$conf = &parse_config_file($_[0]);
	foreach (@{$conf}) {
		$name = $_->{'name'}; chomp $name;
		$value = $_->{'value'}; chomp $value;
		$value =~ s/[\"|\']//g;
		$hs{$name} = $value;
	}
	return %hs;
}

# &show_clam_config ("clamav" || "freshclamav");
sub show_clam_config {
	local ($name_conf, $number_group) = ("$_[0]_conf", $_[1]);

	%conf = &get_config($config{$name_conf});
	
	%param = &get_hash_param($_[0]);
	@groups = &get_group_settings($_[0]);

	# check version
	my $ver = &get_version_number_clamav();
	my $new_config = &check_version_support($ver, '0.90.2');

	print &ui_form_start('save.cgi', 'POST');
	print &ui_hidden('conf_file', $config{$name_conf});
	print &ui_hidden('config', $_[0]);
	print &ui_table_start("$groups[$number_group-1]", 'width=80%');
	foreach $key (sort keys %param) {
			$st = '';
			next if ($param{$key}->[0] != $number_group);
			if ($param{$key}->[1] == '0') {
					if ($new_config){
						# from version 0.90.2
						my $yes = 1 if (($conf{$key} eq '1') || ($conf{$key} =~ /^yes$/i) || ($conf{$key} =~ /^true$/i));
						my $no = 1 if (($conf{$key} eq '0') || ($conf{$key} =~ /^no$/i) || ($conf{$key} =~ /^false$/i));
						$st .= sprintf ("<input type=radio name=\"$key\" value=\"1\" %s>Yes\n", $yes ? 'checked' : 'unchecked');
						$st .= sprintf ("<input type=radio name=\"$key\" value=\"\" %s>No\n", $no ? 'checked' : 'unchecked');
						}
					else {
						# old config parser
						$st .= sprintf ("<input type=radio name=\"$key\" value=\"1\" %s>Yes\n", exists $conf{$key} ? 'checked' : 'unchecked');
						$st .= sprintf ("<input type=radio name=\"$key\" value=\"\" %s>No\n", ! exists $conf{$key} ? 'checked' : 'unchecked');
						}
					print &ui_table_row("$key", $st);
				} elsif ($param{$key}->[1] == '1') {
					$st .= sprintf ("<input type=radio name=\"rb$key\" value=\"\" %s>Default\n", ! exists $conf{$key} ? 'checked' : 'unchecked');
					$st .= sprintf ("<input type=radio name=\"rb$key\" value=\"1\" %s>", exists $conf{$key} ? 'checked' : 'unchecked');
					$st .= sprintf ("<input type=text name=\"$key\" value=\"%s\">", $conf{$key});
				    print &ui_table_row("$key", $st);
				} elsif ($param{$key}->[1] == '2') {
					$st .= sprintf ("<input type=text name=\"$key\" value=\"%s\">", $conf{$key});
				    print &ui_table_row("$key", $st);
 				}
		}
		print &ui_table_end();
	print &ui_form_end([['save', $text{'save'}]]);
}

#&save_config ($conf_file);
sub save_config {
	local %param_in = %{$_[0]};
	$conf_file = $param_in{'conf_file'};
	$config = $param_in{'config'};
	%param = &get_hash_param($config);
	$n = "\n";
	delete($param_in{'conf_file'});
	delete($param_in{'config'});
	delete($param_in{'save'});
	@key_in = keys %param_in;
	foreach (@key_in) {
		$param_in{"$_"} = '' if (exists $param_in{"rb$_"} && !$param_in{"rb$_"});
		delete($param_in{"rb$_"}) if (exists $param_in{"rb$_"});
	}
	
	# check version
	my $ver = &get_version_number_clamav();
	my $new_config = &check_version_support($ver, '0.90.2');

	@key_in = keys %param_in;
	&lock_file($conf_file);
	my $old_file= &read_file_lines($conf_file);
 	foreach $ln (@{$old_file}) {
		foreach (@key_in) {
			if ($ln =~ m/^$_\s+/ || $ln =~ m/^$_\=/ || $ln =~ m/^$_$/) { # it was enabled
				if ($param_in{$_} eq '') {# have to disable parametr
					if ($param{$_}->[1] == 0 && $new_config) {
						$ln = "$_ no";
						}
					elsif ($param{$_}->[1] == 2) {
						$ln = "$_=\"\"";
						}
					else { $ln = ""; }
			   } else { #have to edit
						if ($param{$_}->[1] == 0){
							$ln = $new_config ? "$_ yes" : "$_";
							}
						$ln = "$_ $param_in{$_}" if ($param{$_}->[1] == 1);
						$ln = "$_=\"$param_in{$_}\"" if ($param{$_}->[1] == 2);
				    	}
				delete ($param_in{$_});
		    }
		}
	}

	@key_in = keys %param_in;
 	foreach $ln (@{$old_file}) {
		foreach (@key_in) {
			 if ($ln =~ m/^#\s*$_\s+/ || $ln =~ m/^#\s*$_$/) { # it was disable
				if ($new_config){
						# from version 0.90.2
						if ($param_in{$_} ne '') { #have to add to file
							$have_to_add = 1;
							$add_ln = "$_ yes" if ($param{$_}->[1] == 0);
							$add_ln = "$_ $param_in{$_}" if ($param{$_}->[1] == 1);
							}
						else {
							if ($param{$_}->[1] == 0){
								$have_to_add = 1;
								$add_ln = "$_ no";
								}
							}
						}						
				else {
					# old versions
					if ($param_in{$_} ne '') { #have to add to file
						$have_to_add = 1;
						$add_ln = "$_" if ($param{$_}->[1] == 0);
						$add_ln = "$_ $param_in{$_}" if ($param{$_}->[1] == 1);
						}
					}
				delete ($param_in{$_});
				}
			 if ($have_to_add && $ln =~ m/^\s*$/) {
				$ln = $add_ln;
				$have_to_add = 0;
				}
		}
	}

	@key_in = keys %param_in;
	foreach (@key_in) {
		if ($new_config){
			# from version 0.90.2
			if ($param_in{$_} eq ''){
				if ($param{$_}->[1] == 0){
					push (@{$old_file}, "$_ no") if ($param{$_}->[1] == 0);
					}
				else { next; }
				}
			else {
				push (@{$old_file}, "$_ yes") if ($param{$_}->[1] == 0);
				push (@{$old_file}, "$_ $param_in{$_}") if ($param{$_}->[1] == 1);
				}
			}
		else {
			# old versions
			next if ($param_in{$_} eq '');
			push (@{$old_file}, $_) if ($param{$_}->[1] == 0);
			push (@{$old_file}, "$_ $param_in{$_}") if ($param{$_}->[1] == 1);
			}
	}
 	&flush_file_lines();
 	&unlock_file($conf_file);
}


#  &get_last_db (file_db)
sub get_last_db {
  my $file_db = $_[0];
  my ($date, $info, $version, $level) = ();
  if (-e $file_db) {
    $date = &make_date ((stat($file_db))[9]);
 	@mas = split(':', `sigtool --info $file_db 2>&1`);
 	chomp(@mas);
  	foreach $ln (@mas) {
 		$version = $ln if ($ln =~ s/#\s*of\s*signatures//);
		$level = $ln if ($ln =~ s/\s*Builder//);
  	}
    $info = "$text{'update_version'}: $version - $text{'update_level'}: $level";
  }
  return ($date, $info);
}

sub clamav_daemon_table {
  my $freq = $_[0];
  $st = "<select name=\"freq\">";
  foreach (1..50)
  {
    $default = ($_ == $freq) ? ' SELECTED' : '';
    $st .= qq(<option value="$_"$default>$_</option>\n);
  }
  $st .= qq(</select>);
  return $st;
}

sub clamav_update {
  print &ui_table_start("$text{'update_report_table'}", 'width=100%');
  open (H, "freshclam 2>&1 |");
  while (<H>)
  {
    s/^*\n$/<br>/g;
    if (/load/) {print "."} else {print};
  }
  close (H);
  print &ui_table_end();
}

sub clamav_update_auto {
	local ($update_type, %tmp_in, %job) = ($_[0], %{$_[1]});
	if (-f $config{'fresh_daemon_conf'}) {
		%conf = &get_config($config{'fresh_daemon_conf'});
		} else {
			%conf = &get_config($config{'freshclamav_conf'});
			}
   	$command = "\$($config{'freshclam_cmd'} --quiet --config-file $config{'freshclamav_conf'} --log $conf{'UpdateLogFile'} --datadir $conf{'DatabaseDirectory'})";
	&freshclamav_clear_cron($command);
	if ($update_type == 2) {
    		if ($config{'freshclamav_stop'}) {
	            system ("$config{'freshclamav_stop'}");
        	    } else  {
            		&kill_byname_logged("$config{'freshclam_cmd'}", TERM);
            	    }
		&foreign_require('cron', 'cron-lib.pl');
		&cron::parse_times_input(\%job, \%tmp_in);
 		$job{'user'} = 'root';
 		$job{'active'} = 1;
		$job{'command'} = $command;
 		&lock_file(&cron::cron_file(\%job));
 		&cron::create_cron_job(\%job);
 		&unlock_file(&cron::cron_file(\%job));
		}
	if ($update_type == 1) {
		&freshclamav_clear_cron($command);
		&freshclamav_set_checks($tmp_in{'freq'});
		}
}

sub freshclamav_set_checks {
	local $checks = $_[0];
	if (-f $config{'fresh_daemon_conf'}) {
		%conf = &get_config($config{'fresh_daemon_conf'});
		%conf = ('conf_file' => "$config{'fresh_daemon_conf'}",
				 'config' => 'fresh_daemon',
			 	 'CHECKS' => $checks);
		} else {
			%conf = &get_config($config{'freshclamav_conf'});
			%conf = ('conf_file' => "$config{'freshclamav_conf'}",
				     'config' => 'freshclamav',
		 	   		 'Checks' => $checks);
			}
 	&save_config(\%conf);
        if ($config{'freshclamav_stop'}) {
            system ("$config{'freshclamav_stop'}");
            } else  {
             &kill_byname_logged("$config{'freshclam_cmd'}", TERM);
             }
        if ($config{'freshclamav_start'}) {
            system ("$config{'freshclamav_start'}");
            } else  {
                system ("$config{'freshclam_cmd'} -d");
             }	
}

sub freshclamav_clear_cron {
		local $command = $_[0];
		&foreign_require("cron", "cron-lib.pl");
		local @jobs = &cron::list_cron_jobs();
		local ($job) = grep { $_->{'user'} eq 'root' &&
						      $_->{'command'} eq $command } @jobs;
		&cron::delete_cron_job($job);
}

sub get_freshclamav_update_status {
	local ($daemon, $cron);
	if (-f $config{'fresh_daemon_conf'}) {
		%conf_freshclamav = &get_config($config{'fresh_daemon_conf'});
		$daemon = defined ($conf_freshclamav{'CHECKS'}) ? $conf_freshclamav{'CHECKS'} : 0;
	} else {
		%conf_freshclamav = &get_config($config{'freshclamav_conf'});
		$daemon = defined ($conf_freshclamav{'Checks'}) ? $conf_freshclamav{'Checks'} : 0;
		}

	$command = "\$($config{'freshclam_cmd'} --quiet --config-file $config{'freshclamav_conf'} --log $conf_freshclamav{'UpdateLogFile'} --datadir $conf_freshclamav{'DatabaseDirectory'})";
	&foreign_require("cron", "cron-lib.pl");
	local @jobs = &cron::list_cron_jobs();
	local ($job) = grep { $_->{'user'} eq 'root' &&
					      $_->{'command'} eq $command } @jobs;

	$cron_job = $job if ($job);
	return ($daemon, $cron_job)
}

sub check_dir {
	local %tmp_in = %{$_[0]};
	local $only_list_files = $_[1];
	&error($text{'local_check_error1'}) unless (-r $tmp_in{'dir'});
	if ($tmp_in{'move'}) {
		&error($text{'local_check_error2'}) unless ( -r $config{'clamav_quarantine'});
		$config{'clamav_quarantine'} .= '/' if ($config{'clamav_quarantine'} !~ m/\/$/);
		}

	local ($dir, $sub_dir, $show, $move_dir) = ($tmp_in{'dir'},
												($tmp_in{'sub'}) ? ' -r ' : '',
												($tmp_in{'show'}) ? '1' : '0',
												($tmp_in{'move'}) ? " --move $config{'clamav_quarantine'}" : '');
	local $cmd = "$config{'clamscan_cmd'} $move_dir $sub_dir $dir";
	local $report = 0;


	$number_form = ($only_list_files) ? 0 : 1;
    print &select_all_link("infect", $number_form)."&nbsp\n";
    print &select_invert_link("infect", $number_form);

	if ($only_list_files) {
		print &ui_table_start("$text{'quarantine_table'}", 'width=100%');
		print &ui_form_start('quarantine.cgi', 'GET');
	} else {
			print &ui_table_start("$text{'local_check_cmd'} $cmd", 'width=100%');
			print &ui_form_start('local_check.cgi', 'GET');
			}

  	open (H, "$cmd 2>&1 |");
   	while (<H>)  {
    	/^(.*):(.*)$/;
    	my ($file, $state) = ($1, $2);
       	$file =~ m/([^\/]+)$/;
		my $tmp_file = "$config{'clamav_quarantine'}$1";
		if ($state =~ s/FOUND//) {
			$infect++;
			printf (&ui_table_row("$file", "%s<font color='red'>$state</font>", 2), ($tmp_in{'move'}) ? qq(<input type="checkbox" name="infect" value="$tmp_file">) : qq(<input type="checkbox" name="infect" value="$file">));
		} elsif ($state =~ m/OK/) {
				print &ui_table_row("$file", "<font color='green'>$state</font>", 2) if (! $show);
				}
     if ($file =~ /known viruses/i && $report == 0) {
		$report = 1;
		print &ui_table_end();
		print &ui_form_end([['delete', $text{'local_check_delete'}]]);
		print "<br>";
		print &ui_table_start("$text{'local_check_report'}", 'width=100%') unless $only_list_files;
	 }

	 if ($report && !$only_list_files) {
	 	$file =~ tr/a-z/A-Z/;
        $file =~ s/ /_/g;
		print &ui_table_row("$file", "$state", 2);
	 }
	}
	print &ui_table_end() if ($report);
	close (H);
}

sub clamav_viruses_count
{
  my $count = `sigtool --list-sigs | wc -l`;
  $count =~ s/ //g;
  chomp $count;
  return $count;
}

sub clamav_vdb_search {
   my ($virus, $ext, $case, $sort,$page,$bl) = @_;
   my $limit=50;
   $page= $page ? $page : 1;
   $bl=$bl? $bl:0;
   $case = ($case) ? ' ' : ' -i ';
   $sort = ($sort) ? ' sort |' : '';

   $command = "sigtool --list-sigs";
   $command .= " | grep $case " if ($virus ne '' && $case);
   $command .= $ext ? " \"^$virus\$\" " : " $virus " if ($virus ne '');
   $command .= " | $sort "  if ($virus ne '' && $sort);

   @mas = `$command`; chomp (@mas);
   return 0 unless scalar(@mas);

   my $pages=sprintf("%.f",$#mas/$limit);
   my $bloks=sprintf("%.f",$pages/10);
if($pages>1){
    if($bl!=0) { 
    print "<a href=\"?name=$virus&limit=$limit&cace=$case&sort=$sort&ext=$ext&page=0&bl=0&submit=Search\" ><img src=/images/d_left.gif border=0 align=middle></a>" if($bl!=0);
    }

my $st=$bl*10+1;
my $en=$bl*10+10;
$en=$pages if($en>$pages);

if($bl!=0){
	my $ppage=$st-10;
	print "<a href=\"?name=$virus&limit=$limit&cace=$case&sort=$sort&ext=$ext&page=$ppage&bl="; print $bl-1; print "&submit=Search\"><img src=/images/left.gif border=0 align=middle></a>";} 

    for ($i=$st; $i<=$en ;$i++){	
	if($page!=$i){
	print "<a href=\"?name=$virus&limit=$limit&cace=$case&sort=$sort&ext=$ext&page=$i&bl=$bl&submit=Search\" >"."[",$i*$limit-$limit,"..",$i*$limit,"]</a>";
	}else{
	print "<b>[",$i*$limit-$limit,"..",$i*$limit,"]</b>";
	}
    }
my $npage=$i++;
print "<a href=\"?name=$virus&limit=$limit&cace=$case&sort=$sort&ext=$ext&page=$npage&bl=".++$bl."&submit=Search\" ><img src=/images/right.gif border=0 align=middle></a>" if($en<$pages);
print "<a href=\"?name=$virus&limit=$limit&cace=$case&sort=$sort&page=$pages&ext=$ext&bl=$bloks&submit=Search\" ><img src=/images/d_right.gif border=0 align=middle></a>" if( $npage<$pages );
$command .=" | head -n ".$limit*$page." |tail -n $limit";
@mas = `$command`; chomp (@mas);
}

print qq(<table width="100%"><tr $tb>					<td>#</td>
									<td><b>$text{'search_name'}</b></td>
									<td><b>Virus Alias Database</b></td>
									<td><b>Symantec</b></td>
									<td><b>McAfee</b></td>
									<td><b>Viruspool</b></td>
									<td><b>Viruslist</b></td>
							</tr>);
my $n=1;
foreach (@mas) {
	my $name_pure = &urlize ( &clamav_get_pure_name($_));
my $name = &urlize ($_);

    my $vdb_url = "http://www.rainingfrogs.co.uk/index.php?virus=$name&search=contains";
my $symantec_url ="http://search.symantec.com/custom/us/query.html?" .
	 "lk=1&rf=0&qt=$name_pure&qp=" . &urlize ("url:http://securityresponse.symantec.com/avcenter/venc/data url:/avcenter/venc/data url:/avcenter/venc/auto/index");
my $mcafee_url ="http://us.mcafee.com/virusInfo/default.asp?id=alphar&SearchType=2&searchString=$name_pure";
my $viruspool_url =	"http://viruspool.vanderkooij.org/virus.cms?name=$name";
my $viruslist_url = "http://www.viruslist.com/en/find?search_mode=virus&words=$name_pure";

my $np=$page*$limit-$limit+$n++;
	print qq(<tr>
				<td $cb>$np </td>
				<td $cb>$_</td>
				<td $cb><a href="$vdb_url" target="_BLANK">$text{'search_more'}</a></td>
				<td $cb><a href="$symantec_url" target="_BLANK">$text{'search_more'}</a></td>
				<td $cb><a href="$mcafee_url" target="_BLANK">$text{'search_more'}</a></td>
				<td $cb><a href="$viruspool_url" target="_BLANK">$text{'search_more'}</a></td>
				<td $cb><a href="$viruslist_url" target="_BLANK">$text{'search_more'}</a></td>
			</tr>);
}
print qq(</table>);
}


sub clamav_get_pure_name {
  my $name = shift;
  my ($b, $m, $e);
  if ($name =~ /^(.*)[\.\-](.*)[\.\-](.*)$/)  {
    ($b, $m, $e) = ($1, $2, $3);
     return ($m !~ /[a-z,A-Z]/ || length ($m) == 1) ?
        &clamav_get_pure_name ("$b") : $m;
  	}

  if ($name =~ /^(.*)[\.\-](.*)$/)  {
    ($b, $e) = ($1, $2);
    if ($e !~ /[a-z,A-Z]/) {return $b;} elsif ($e) {return $e;}
  	}

  return $1 if ($name =~ /^(.*)[\.\-].*$/);
  return $name;
}

sub show_log {
	my ($file, $lines) = ($_[0], $_[1]);
	print &ui_form_start('log_view.cgi', 'POST');
	print &ui_hidden('logs', $file);
	print &ui_table_start("$text{'log_info'}", 'width=100%');
	if ($lines) {
		@text = `/usr/bin/tail -n $lines $file`;
		} else {
  			open (H, "$file");
	    	@text = <H>;
		  	close (H);
			}
	print &ui_table_row(qq(<textarea cols=80 rows=30>@text</textarea>\n), "");
	print &ui_table_span();
	print &ui_table_end();
	print &ui_form_end([['clear', $text{'log_clear'}]]);
}

sub get_hash_param {

    # name => (group, type value)
    # 0 - check, 1 - string, 2 - file
    my %param = (
      'DatabaseOwner' => [1, 1],
      'AllowSupplementaryGroups' => [1, 0],
      'DatabaseDirectory' => [1, 1],
      'Checks' => [1, 1],
      'UpdateLogFile' => [1, 1],
      'LogSyslog' => [2, 0],
      'LogFacility' => [2, 1],
      'LogVerbose' => [2, 0],
      'DNSDatabaseInfo' => [1, 1],
      'DatabaseMirror' => [1, 1],
      'MaxAttempts' => [1, 1],
      'HTTPProxyServer' => [3, 1],
      'HTTPProxyPort' => [3, 1],
      'HTTPProxyUsername' => [3, 1],
      'HTTPProxyPassword' => [3, 1],
		'HTTPUserAgent' => [3, 1, '0.90.2'],
      'LocalIPAddress' => [1, 1],
      'NotifyClamd' => [1, 1],
      'OnUpdateExecute' => [1, 1],
      'OnErrorExecute' => [1, 1],
    ) if ($_[0] eq 'freshclamav');

    %param = (
      'LogFile' => [1, 1],
      'LogFileUnlock' => [1, 0],
      'LogFileMaxSize' => [1, 1],
      'LogTime' => [1, 0],
      'LogClean' => [1, 0],
      'LogSyslog' => [1, 0],
      'LogFacility' => [1, 1],
      'LogVerbose' => [1, 0],
      'PidFile' => [2, 1],
      'DataDirectory' => [2, 1],
      'LocalSocket' => [2, 1],
      'FixStaleSocket' => [2, 0],
      'TCPSocket' => [2, 1],
      'TCPAddr' => [2, 1],
      'MaxConnectionQueueLength' => [3, 1],
      'MaxThreads' => [3, 1],
      'ReadTimeout' => [3, 1],
      'MaxDirectoryRecursion' => [3, 1],
      'FollowDirectorySymlinks' => [2, 0],
      'FollowFileSymlinks' => [2, 0],
      'SelfCheck' => [2, 1],
      'VirusEvent' => [2, 1],
      'ExitOnOOM' => [2, 0],
      'User' => [2, 1],
      'AllowSupplementaryGroups' => [2, 0],
      'Foreground' => [2, 0],
      'Debug' => [2, 0],
      'StreamMaxLength' => [3, 1],
      'StreamMinPort' => [3, 1],
      'StreamMaxPort' => [3, 1],
      'DisableDefaultScanOptions' => [2, 0, 0,'0.90.2'],
      'ScanPE' => [6, 0],
      'DetectBrokenExecutables' => [2, 0],
      'ScanOLE2' => [6, 0],
      'ScanHTML' => [6, 0],
      'ScanMail' => [6, 0],
		'ScanPDF' => [6, 0, '0.90.2'],
		'ScanELF' => [6, 0, '0.90.2'],
      'MailFollowURLs' => [2, 0],
      'ScanArchive' => [6, 0],
      'ScanRAR' => [6, 0, 0,'0.90.2'],
      'ArchiveMaxFileSize' => [5, 1],
      'ArchiveMaxRecursion' => [5, 1],
      'ArchiveMaxFiles' => [5, 1],
      'ArchiveMaxCompressionRatio' => [5, 1],
      'ArchiveLimitMemoryUsage' => [5, 0],
      'ArchiveBlockEncrypted' => [5, 0],
      'ArchiveBlockMax' => [5, 0],
		'ClamukoScanOnAccess' => [4, 0],
      'ClamukoScanOnLine' => [4, 0],
      'ClamukoScanOnOpen' => [4, 0],
      'ClamukoScanOnClose' => [4, 0],
      'ClamukoScanOnExec' => [4, 0],
      'ClamukoIncludePath' => [4, 1],
      'ClamukoExcludePath' => [4, 0],
      'ClamukoMaxFileSize' => [4, 1],
      'ClamukoScanArchive' => [4, 0]
    ) if ($_[0] eq 'clamav');

    %param = (
        'LOGFILE' => [1, 2],
        'CHECKS' => [1, 2],
        'HTTPPROXY' => [1, 2],
        'PROXYUSER' => [1, 2],
        'ONERROR' => [1, 2],
        'ONSUCCESS' => [1, 2],
        'OPTIONS' => [1, 2]
    ) if ($_[0] eq 'fresh_daemon');

 my $ver = &get_version_number_clamav();
 my %ret_param;
 foreach (keys %param) {
	my $old = &check_version_support($ver, $param{$_}->[3]) if ($param{$_}->[3]);
	$ret_param{$_} = $param{$_} if (&check_version_support($ver, $param{$_}->[2]) && !$old);
	}
 return %ret_param;
}

# &check_version_support($clamav_ver, $ver)
# return 1 if $clamav_ver >= $ver
sub check_version_support
{
 my ($ver1, $ver2) = @_;
 my @ver1 = split(/\./, $ver1);
 my @ver2 = split(/\./, $ver2);
 return 1 if (($ver1[0]*1000000 + $ver1[1]*1000 + $ver1[2]) >= ($ver2[0]*1000000 + $ver2[1]*1000 + $ver2[2]));
}

sub get_group_settings {
    @groups_clamav=();
    foreach $i (1..6) {
    	 push (@groups_clamav, $text{"group_clamav_$i"});
	}

    @groups_freshclamav=();
    foreach $i (1..3) {
	 push (@groups_freshclamav, $text{"group_freshclamav_$i"});
	}

	@groups_fresh_daemon=();
    foreach $i (1..1) {
    	 push (@groups_fresh_daemon, $text{"group_fresh_daemon_$i"});
	}

   return @groups_fresh_daemon if ($_[0] eq 'fresh_daemon');
   return @groups_clamav if ($_[0] eq 'clamav');
   return @groups_freshclamav if ($_[0] eq 'freshclamav');
}


#--- Confmaster funcs ---

# &parse_config_file($conf_file)
# parses file $_[0] and returns a reference to an array of hashes (spliter - \s)
sub parse_config_file
{
 my (@options, $conf_file);
 open (INFL,$_[0]) || &error($text{'error_open'});
 while (<INFL>)
 {
  chomp;
  &trunc_space($_);
  unless ($_=~/^#/ || $_=~/^$/)
  {
  	if ($_ =~ m/^[^\s]+\=/) {
		@options=split ('=',$_);
		} else {
   			@options=split (/\s+/,$_);
   			}
   $options[0]=&trunc_space($options[0]);
   $options[1]=&trunc_space($options[1]);
   push (@{$conf_file},{'name'=>$options[0],'value'=>$options[1]});
  }
 }
 close INFL;
 return $conf_file;
}

# &get_config_value($name, \@array)
# return 1 and value of parameter with name from array
sub get_config_value
{
 my ($key, $value);
 foreach (@{$_[1]})
 {
  if ($_->{'name'} eq $_[0])
  {
   $key=1;
   $value=$_->{'value'};
  }
 }
 return ($key, $value);
}

# &set_config_value($name, $value, \@array)
# set new value of parameter with name from array
# or add new string to array
sub set_config_value
{
 my $str_old;
 foreach (@{$_[2]})
 {
  if ($_->{'name'} eq $_[0])
  {
   $_->{'value'}=$_[1];
   $str_old=1;
  }
 }
 push (@{$_[2]},{'name'=>$_[0],'value'=>$_[1]}) if ($str_old!=1);
}

# &save_config_file(\@array, [$file])
# save array to file (spliter - \s)
sub save_config_file
{
 my $conf_file = $_[1] ? $_[1] : $config{'clamav_conf'};
 
 # read file lines into hash
 &lock_file($conf_file);
 my $old_file= &read_file_lines($conf_file);
 
 # modify conf file options
 my (@new, %un);
 foreach (@{$old_file})
 {
  unless ($_=~/^#/ || $_=~/^$/)
  {
   my @str_old = split (/\s+/,$_);
   $str_old[0] = &trunc_space($str_old[0]);
   $str_old[1] = &trunc_space($str_old[1]);
	foreach my $i (@{$_[0]})
	{
	 if ($i->{'name'} eq $str_old[0])
    {
	  # change option
	  $un{$i->{'name'}} = 1;
	  if ($i->{'value'} eq '#')
	  {
	   # comment option
	   $_="# $i->{'name'}";
	  }
	  else
	  {
	   # change value
      $_="$i->{'name'} $i->{'value'}";
	  }
	  last;
    }
	}
  }
 }

 # new options
 foreach (@{$_[0]})
 {
  push (@new, $_) if (!$un{$_->{'name'}} && ($_->{'value'} ne '#')); 
 }
  
 # add change options
 foreach (@new)
 {
  my $comm;
  my $name = $_->{'name'};
  foreach my $i (@{$old_file})
  {
   if ($i =~ /^#\s*?$name/)
	{
	 # add after comment
	 $i = "$i\n$_->{'name'} $_->{'value'}";
	 $comm = 1;
	 last;
	}
  }
  # add in the end of file
  push (@{$old_file}, "$_->{'name'} $_->{'value'}") unless ($comm);
 }

 # save chage
 &flush_file_lines();
 &unlock_file($conf_file);
}

# &trunc_space($string)
# delete spaces from string
sub trunc_space
{
 local $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}

1;
1;
