# confmaster-lib.pl
# Common functions for confmaster module

# main part
do '../web-lib.pl';
&init_config();
require '../ui-lib.pl';

# server or appliance list
if ($config{'ztl_mode'})
{
 @servers_list = ('web','mysql','postgres','dns','mail','cifs','pdc','print','proxy','firewall','dhcp');
 @sub_file_list = ('cifs','pdc','print');
 @sub_db_list = ('mysql','postgres');
 @sub_lan_list = ('proxy','firewall','dhcp');
 @web_list = ('web','ftp','quota');
}
else
{
 @servers_list = ('web', 'mail', 'dns', 'mysql', 'postgres', 'ftp', 'quota');
 @web_list = ('web');
}

# server or appliance funcs require
foreach my $i (@servers_list)
{
 if ($config{'ztl_mode'} && ($i eq 'web'))
 {
  foreach my $j (@web_list)
  {
   require "./server-$j.pl";
  }
 }
 else {require "./server-$i.pl";}
}

# detecting available updaters
if ($config{'type_pkg'} == 0){
	# autodetect
	$use_yum = &has_command("yum");
	$use_rhn = &has_command("up2date");
	$use_swup = &has_command("swup");
	$use_apt = &has_command("apt-get");
	$updater = 'Any';
	}
elsif ($config{'type_pkg'} == 1){
	# YUM
	$use_yum = &has_command("yum");
	$updater = 'YUM';
	}
elsif ($config{'type_pkg'} == 2){
	# SWUP
	$use_swup = &has_command("swup");
	$updater = 'SWUP';
	}
elsif ($config{'type_pkg'} == 3){
	# RHN
	$use_rhn = &has_command("up2date");
	$updater = 'RHN';
	}
elsif ($config{'type_pkg'} == 4){
	# RHN or YUM
	$use_yum = &has_command("yum");
	$use_rhn = &has_command("up2date");
	$updater = 'RHN or YUM';
	}
elsif ($config{'type_pkg'} == 5){
	# PkG_ADD
	$use_pkg = $pkg;
	}
elsif ($config{'type_pkg'} == 6){
	# APT-GET
	$use_apt = &has_command("apt-get");
	$updater = 'APT';
	}

# --- RPMS list functions ---

# &get_rpms_filename($server_name);
# return file name with rpms list for current os
sub get_rpms_filename
{
 my ($srv) = ($_[0]);

 # Find any range-number config files
 my $rangefile = undef;
 my ($os, $ver) = ($gconfig{'os_type'}, $gconfig{'os_version'});
 opendir(DIR, $module_root_directory);
 while($f = readdir(DIR))
 {
  if ($f =~ /^rpms\-$srv\-\Q$os\E\-([0-9\.]+)\-([0-9\.]+)$/ && $ver >= $1 && $ver <= $2)
  {
   $rangefile = "$module_root_directory/$f";
  }
  elsif ($f =~ /^rpms\-$srv\-\Q$os\E\-([0-9\.]+)\-\*$/ && $ver >= $1)
  {
   $rangefile = "$module_root_directory/$f";
  }
  elsif ($f =~ /^rpms\-$srv\-\Q$os\E\-\*\-([0-9\.]+)$/ && $ver <= $1)
  {
   $rangefile = "$module_root_directory/$f";
  }
 }
 closedir(DIR);

 # Find the best-matching config file
 my $conf = undef;
 if (-r "$module_root_directory/rpms-$srv-$os-$ver")
 {
  $conf = "$module_root_directory/rpms-$srv-$os-$ver";
 }
 elsif ($rangefile)
 {
  $conf = $rangefile;
 }
 elsif (-r "$module_root_directory/rpms-$srv-$os")
 {
  $conf = "$module_root_directory/rpms-$srv-$os";
 }
 elsif ($os =~ /^(\S+)-(\S+)$/ && -r "$module_root_directory/rpms-$srv-*-$2")
 {
  $conf = "$module_root_directory/rpms-$srv-*-$2";
 }
 elsif (-r "$module_root_directory/rpms-$srv")
 {
  $conf = "$module_root_directory/rpms-$srv";
 }
 return $conf;
}

# &get_rpms_list($server_name);
# return list of rpm-files for current os
sub get_rpms_list
{
 my ($srv,@rpms) = ($_[0]);
 my $conf = &get_rpms_filename($srv);
 my $str = &read_file_lines($conf);
 foreach (@{$str}){
 	push (@rpms,$_);
	}
 return \@rpms;
}

# &update_rpms_list($server_name, $rpms);
sub update_rpms_list
{
 my ($srv, $rpms) = ($_[0], $_[1]);
 local $func = "update_rpms_$srv";
 return $rpms unless defined(&$func);
 return &$func($rpms);
}

# --- ZTL Appliances functions ---

# &install_appl($server_name);
# install and upgrade appliance (or upgrade system)
sub install_appl 
{
 # applince name
 my $srv = $_[0];
 my $appl;
 if (($srv eq 'mail') && ($config{'mail_system'} == 3)) { $appl = "ztl-$srv-sendmail"; }
 elsif (($srv eq 'mail') && ($config{'mail_system'} == 4)) { $appl = "ztl-$srv-postfix"; }
 else { $appl = "ztl-$srv"; }
 
 # install applince package
 my $temp = $ENV{'REQUEST_METHOD'};
 $ENV{'REQUEST_METHOD'} = undef; 
 if ($use_swup) 
 {
  # use swup system updater (for trustix oses)
  &foreign_require("swup", "swup-lib.pl");
  if ($srv eq "system") 
  {
   # upgrade all system packages
   # get upgraded rpms list
   my @installed = &swup::get_upgrade_pkgs_list();
   if (@installed) 
   {
    &swup::upgrade_all_packages() ;
	 print "<tr><td colspan=2>$text{'setup_ok_sys'}</td></tr>";
   }
   else { print "<tr><td colspan=2>$text{'setup_already_sys'}</td></tr>"; }
  }
  else
  {
   # install packages
	&swup::install_package($appl);
  }
 }
 elsif ($use_yum)
 {
  # use yum system updater (for redhat-like oses)
  if ($srv eq 'system')
  {
   # upgrade all packages
   &upgrade_system_yum();
  }
  else
  {
   # install some packages
   &install_system_yum([$appl]);
  }
 }
 elsif ($use_rhn) 
 {
  # use up2date system updater (for redhat-like oses)
  &install_system_rhn([$appl]);
 }
 elsif ($use_pkg) 
 {
  # use freebsd packages updater
 }
 $ENV{'REQUEST_METHOD'} = $temp;
 return 0;
}

# &upgrade_appl($server_name);
# upgrade appliance
sub upgrade_appl 
{
 # applince name
 my $srv = $_[0];
 my $appl;
 if (($srv eq 'mail') && ($config{'mail_system'} == 3)) { $appl = "ztl-$srv-sendmail"; }
 elsif (($srv eq 'mail') && ($config{'mail_system'} == 4)) { $appl = "ztl-$srv-postfix"; }
 else { $appl = "ztl-$srv"; }
  
 # install applince package
 my $temp = $ENV{'REQUEST_METHOD'};
 $ENV{'REQUEST_METHOD'} = undef;
 if ($use_swup) 
 {
  # use swup system updater (for trustix oses)
  &foreign_require("swup", "swup-lib.pl");
  my $appl = "ztl-$srv";
  &swup::upgrade_package($appl);
 }
 elsif ($use_yum)
 {
  # use yum system updater (for redhat-like oses)
  &upgrade_system_yum([$appl]);
 }
 elsif ($use_rhn) 
 {
  # use up2date system updater (for redhat-like oses)
 }
 elsif ($use_pkg) 
 {
  # use freebsd packages updater
 }
 $ENV{'REQUEST_METHOD'} = $temp;
 return 0;
}

# &uninstall_appl($server_name);
# uninstall appliance
sub uninstall_appl 
{
 # applince name
 my $srv = $_[0];
 my $appl;
 if (($srv eq 'mail') && ($config{'mail_system'} == 3)) { $appl = "ztl-$srv-sendmail"; }
 elsif (($srv eq 'mail') && ($config{'mail_system'} == 4)) { $appl = "ztl-$srv-postfix"; }
 else { $appl = "ztl-$srv"; }
  
 # install applince package
 my $temp=$ENV{'REQUEST_METHOD'};
 $ENV{'REQUEST_METHOD'}=0;
 if ($use_swup) 
 {
  # use swup system updater (for trustix oses)
  &foreign_require("swup", "swup-lib.pl");
  
  # get installed rpms list
  local @installed = &swup::get_installed_rpms_list();
  local ($is) = grep { $_->{'name'} eq $appl } @installed;
  &error(&text('uninstall_eappl', $appl)) unless $is;
  &swup::uninstall_package($appl);
 }
 elsif ($use_yum)
 {
  # use yum system updater (for redhat-like oses)
  &uninstall_system_yum([$appl]);
 }
 elsif ($use_rhn) 
 {
  # use up2date system updater (for redhat-like oses)
 }
 elsif ($use_pkg) 
 {
  # use freebsd packages updater
 }
 $ENV{'REQUEST_METHOD'}=$temp;
}

# &check_ztl_appliance($srv)
# check installed ztl appliance
# return 1 
sub check_ztl_appliance
{
 my $srv = $_[0];
 my @installed;
 if ($use_swup) 
 {
  # use swup system updater (for trustix oses)
  &foreign_require("swup", "swup-lib.pl");
  @installed = &swup::get_installed_rpms_list();  
  return grep {$_->{'name'} eq "ztl-$srv"} @installed;
 }
 elsif ($use_yum)
 {
  # use yum system updater (for redhat-like oses)
  &foreign_require("software", "software-lib.pl");
  my $pkg = "ztl-$srv";
  if ($srv eq 'mail')
  {
   $pkg .= "-postfix";
   @installed = &software::package_info($pkg);
	return 1 if ($installed[0] eq $pkg);
	$pkg = "ztl-".$srv."-sendmail";
  }
  @installed = &software::package_info($pkg);
  return 1 if ($installed[0] eq $pkg);
 }
 elsif ($use_rhn) 
 {
  # use up2date system updater (for redhat-like oses)
 }
 elsif ($use_pkg) 
 {
  # use freebsd packages updater
 }
 return undef;
}

# --- Install packages functions ---

# &install_rpms(\@rpms);
# install packages
sub install_rpms 
{
 my @rpms = @{$_[0]};
 return unless $_[0];
 if ($use_rhn) 
 {
  # use up2date system updater (for redhat-like oses)
  &install_system_rhn($_[0]);
 }
 elsif ($use_yum)
 {
  # use yum system updater (for redhat-like oses)
  &install_system_yum($_[0]);
 }
 elsif ($use_swup) 
 {
  # use swup system updater (for trustix oses)
  &foreign_require("swup", "swup-lib.pl");
  
  # get list of installed rpms
  my @installed = &swup::get_installed_rpms_list();
  my %installed = ();
  map { $installed{$_->{'name'}}++ } @installed;
  my @upgrade = grep { $installed{$_} } @rpms;
  
  # upgrating packages
  &swup::upgrade_package_list(\@upgrade) if @upgrade;
  
  # get list of installed rpms again (may be some new have been installed)
  @installed = &swup::get_installed_rpms_list();
  %installed = ();
  map { $installed{$_->{'name'}}++ } @installed;
  my @install = grep { ! $installed{$_} } @rpms;
  
  # installing packages
  &swup::install_package_list(\@install) if @install;
 }
 elsif ($use_pkg)
 {
  # use pkg_add packages updater (for freebsd)
 }
 elsif ($use_apt)
 {
  # use apt-get packages updater (for debian-like oses)
  &install_system_apt($_[0]);
 }
 else 
 {
  # no any packages updater
  &error(&text('index_err_any', $updater,"$gconfig{'webprefix'}/config.cgi?$module_name"));
 }
}

# &upgrade_rpms(\@rpms)
# upgrade packages
sub upgrade_rpms
{
 my @rpms = @{$_[0]};
 return unless $_[0];
 if ($use_rhn) 
 {
  # use up2date system updater (for redhat-like oses)
  &install_system_rhn($_[0]);
 }
 elsif ($use_yum)
 {
  # use yum system updater (for redhat-like oses)
  &upgrade_system_yum($_[0]);
 }
 elsif ($use_swup) 
 {
  # use swup system updater (for trustix oses)
  &foreign_require("swup", "swup-lib.pl");
  
  # get list of installed rpms
  my @installed = &swup::get_installed_rpms_list();
  my %installed = ();
  map { $installed{$_->{'name'}}++ } @installed;
  my @upgrade = grep { $installed{$_} } @rpms;
  
  # upgrating packages
  &swup::upgrade_package_list(\@upgrade) if @upgrade;
  
  # get list of installed rpms again (may be some new have been installed)
  @installed = &swup::get_installed_rpms_list();
  %installed = ();
  map { $installed{$_->{'name'}}++ } @installed;
  my @install = grep { ! $installed{$_} } @rpms;
  
  # installing packages
  &swup::install_package_list(\@install) if @install;
 }
 elsif ($use_apt)
 {
  # use apt-get packages updater (for debian-like oses)
  &install_system_apt($_[0]);
 }
 else 
 {
  # no any packages updater
  &error(&text('index_err_any', $updater,"$gconfig{'webprefix'}/config.cgi?$module_name"));
 }
}

# --- YUM functions ---

# &install_system_yum(\@rpms)
# Install packages with yum
sub install_system_yum
{
 # return if no found yum
 $use_yum || return;
 
 # print header messages
 print "<tr $tb><td colspan=2><b>$text{'setup_installing'}</b></td><tr>\n";
 print "<tr $cb><td style=\"overflow:hidden\" colspan=2><pre>";
 
 # install packages
 my $err = &run_yum_command('install', $_[0]);
 print "</pre></td></tr>\n";

 if ($config{'ztl_mode'})
 {
  # print postinstall scripts log
  print "<tr $tb><td colspan=2><b>$text{'setup_appliance'}</b></td><tr>\n";
  print "<tr $cb><td style=\"overflow:hidden\" colspan=2>";
  my $log = &read_file_lines("$module_config_directory\/install.log");
  foreach (@{$log}) { print "$_<br>"; }
  &unlink_file("$module_config_directory\/install.log");
  print "</td></tr>\n";
 }
 if ($err)
 {
  # exec with error message if command error
  print "</table>\n";
  &error($text{'setup_eupdating'});
 }
}

# &upgrade_system_yum(\@rpms)
# upgrade packages with yum
sub upgrade_system_yum
{
 # return if no found yum
 $use_yum || return;
 
 # print header messages
 print "<tr $tb><td colspan=2><b>$text{'setup_updating'}</b></td><tr>\n";
 print "<tr $cb><td style=\"overflow:hidden\" colspan=2><pre>";
 
 # upgrade packages
 my $err = &run_yum_command('update', $_[0]);
 print "</pre></td></tr>\n";
 if ($err)
 {
  # exec with error message if command error
  print "</table>\n";
  &error($text{'setup_eupdating'});
 }
}

# &uninstall_system_yum(\@rpms)
# uninstall packages with yum
sub uninstall_system_yum
{
 # return if no found yum
 $use_yum || return;
 
 # print header messages
 print "<tr $tb><td colspan=2><b>$text{'setup_uninstalling'}</b></td><tr>\n";
 print "<tr $cb><td style=\"overflow:hidden\" colspan=2><pre>";
 
 # uninstall packages
 my $err = &run_yum_command('remove', $_[0]);
 
 # print postinstall scripts log
 print "<tr $tb><td colspan=2><b>$text{'uninstall_appliance'}</b></td><tr>\n";
 print "<tr $cb><td style=\"overflow:hidden\" colspan=2>";
 my $log = &read_file_lines("$module_config_directory\/uninstall.log");
 foreach (@{$log}) { print "$_<br>"; }
 &unlink_file("$module_config_directory\/uninstall.log");
 print "</pre></td></tr>\n";
 if ($err)
 {
  # exec with error message if command error
  print "</table>\n";
  &error($text{'setup_eupdating'});
 }
}

# &run_yum_command($command, \@rpms)
# run some yum command for packages
# return error code or undef
sub run_yum_command
{
 # get command & rpms list 
 my $cmd = $_[0];
 my $rpms = join(' ', @{$_[1]});
 
 # run yum command
 &additional_log('exec', undef, "yum -y $cmd $rpms");
 open(CMD, "yum -y $cmd $rpms 2>&1 </dev/null |");
 while(<CMD>)
 {
  if (!/ETA/ && !/\%\s+done\s+\d+\/\d+\s*$/)
  {
   # print command output
   unless ($_=~/^.{0,80}$/)
   {
    my @s;
	 $_=~/(^.{70}\S{0,10})(.*)$/;
	 push (@s, $1);
	 my $st = $2;
    while ($st !~ /^.{0,80}$/) {
		$st =~ /(^.{70}\S{0,10})(.*)$/;
		push (@s, $1);
		$st = $2;
		}
	 push (@s, $st) if $st;
    foreach my $i (@s){
		print &html_escape($i)."<BR>";
		}
   }
   else { print &html_escape($_);}
  }
 }
 close(CMD);
 my $err = 1 if $?;
  
 # return error code or undef
 return $err ? $err : undef;
}

# --- UP2DATE functions ---

# &install_system_rhn(\@rpms)
# Install some package with up2date
sub install_system_rhn 
{
 $use_rhn || return;
 
 my @rv;
 my $rpms = join(' ', @{$_[0]});
 print "<tr $tb><td colspan=2><b>$text{'setup_installing'}</b></td><tr>\n";
 print "<tr $cb><td colspan=2><pre>";
 open(CMD, "echo \"q\" | up2date --nox --nosig $rpms 2>&1 |");
 local $got_error = 0;
 while(<CMD>) {
	while(s/^[^\015]+\015([^\012])/$1/) { }
	$got_error++ if (/error|failed/i);
	print;
	}
 close(CMD);
 print "</pre>\n";
 print "</td></tr>\n";

 if ($got_error || $?)
 {
  print "</table>\n";
  &error($text{'setup_eupdating'});
 }
}

# --- APT-GET functions ---

# &install_system_apt(\@rpms)
# Install some package with apt-get
sub install_system_apt
{
 $use_apt || return;
 
 # changed sources.list
 &apt_source_change();
 
 # tmp file
 my $tmp = &transname();
 system ("echo Y\nY\nY\nY\nY > $tmp");
 
 # run apt-get
 my @rv;
 my $rpms = join(' ', @{$_[0]});
 print "<tr $tb><td colspan=2><b>$text{'setup_installing'}</b></td><tr>\n";
 print "<tr $cb><td colspan=2><pre>";
 $ENV{'DEBIAN_FRONTEND'}='noninteractive';
 open(CMD, "apt-get -yV install $rpms 2>&1 < $tmp|");
 my $dpkg_error;
 while(<CMD>) 
 {
  # dpkg error
  $dpkg_error = 1 if (/dpkg\s+?returned\s+?an\s+?error\s+?code\s+?/i);
  
  # print command output
  unless ($_=~/^.{0,80}$/)
  {
   my @s;
	$_=~/(^.{70}\S{0,10})(.*)$/;
	push (@s, $1);
	my $st = $2;
   while ($st !~ /^.{0,80}$/) {
		$st =~ /(^.{70}\S{0,10})(.*)$/;
		push (@s, $1);
		$st = $2;
		}
	push (@s, $st) if $st;
   foreach my $i (@s){
		print &html_escape($i)."<BR>";
		}
  }
  else { print &html_escape($_);}
 }
 close(CMD);
 print "</pre></td></tr>\n";
 
 # run dpkg --configure -a
 my $got_error = $?;
 $got_error = &dpkg_reconfigure if $dpkg_error;
 
 # restore sources.list
 &restore_config($config{'sources_list'});
 
 # print error message
 if ($got_error)
 {
  print "</table>\n";
  &error($text{'setup_eupdating'});
 }
}

# &dpkg_reconfigure()
# reconfigure an unpacked packages
# return an error code
sub dpkg_reconfigure
{
 print "<tr $tb><td colspan=2><b>$text{'dpkg_reconfigure'}</b></td><tr>\n";
 print "<tr $cb><td colspan=2><pre>";
 open(CMD, "dpkg --configure -a 2>&1 |");
 while(<CMD>) 
 {
  # print command output
  unless ($_=~/^.{0,80}$/)
  {
   my @s;
	$_=~/(^.{70}.*?\s)(.*)$/;
	push (@s, $1);
	my $st = $2;
   while ($st !~ /^.{0,80}$/) {
		$st =~ /(^.{70}.*?\s)(.*)$/;
		push (@s, $1);
		$st = $2;
		}
	push (@s, $st) if $st;
   foreach my $i (@s){
		print &html_escape($i)."<BR>";
		}
  }
  else { print &html_escape($_);}
 }
 close(CMD);
 print "</pre></td></tr>\n";
 return $?;
}

# &apt_source_change()
# changed sources.list
sub apt_source_change
{
 # backup sources.list
 my $out = &backup_config($config{'sources_list'});
 return unless $out;
 
 # changed sources.list
 my $ch;
 my $conf = &read_file_lines($config{'sources_list'});
 foreach (@{$conf})
 {
  unless (/^\s*?deb(?:-src)?\s+?(?:ftp:|http:)/ || /^\s*?#/ || /^\s*?$/) 
  {
   $_ = "# ".$_;
	$ch = 1;
  }
 }
 &flush_file_lines() if $ch;
}

# --- Configuration servers funcs ---

# &configure_server($server)
# configure server or appliance
sub configure_server
{
 my $srv = $_[0];

 # delete log file
 my $log_file="$module_config_directory\/$_[0].log";
 &unlink_file ($log_file) if (-e $log_file);
 
 # install
 if ($config{'ztl_mode'} && !$reconf)
 {
  # for install in the ztl mode
  print "$text{'setup_configuring'}\n";
  print "=================================\n";
  if (($srv eq 'web'))
  {
   foreach my $i (@web_list)
   {
    local $func = "configure_$i";
    &error(&text('setup_econfsub', $text{"server_$i"})) unless defined(&$func);
    &$func();
   }
  }
  else
  {
   local $func = "configure_$srv";
   &error(&text('setup_econfsub', $text{"server_$srv"})) unless defined(&$func);
   &$func();
  }
  print "\n$text{'setup_complete'}\n";
  print "=================================\n\n";
 }
 else
 {
  # for install in the cpplus mode & reconfigure in the ztl mode
  print "<tr $tb><td colspan=2><b>$text{'setup_configuring'}</b></td><tr>\n";
  if (($srv eq 'web'))
  {
   foreach my $i (@web_list)
   {
    local $func = "configure_$i";
    &error(&text('setup_econfsub', $text{"server_$srv"})) unless defined(&$func);
    print "<tr $cb><td colspan=2>";
    &$func();
    print "</td></tr>\n";
   }
  }
  else
  {
   local $func = "configure_$srv";
   &error(&text('setup_econfsub', $text{"server_$srv"})) unless defined(&$func);
   print "<tr $cb><td colspan=2>";
   &$func();
   print "</td></tr>\n";
  }
  print "<tr $cb><td colspan=2><b><br>$text{'setup_complete'}<b></td></tr>\n";
 }
}

# &uninstall_server($server)
# uninstall server or appliance
sub uninstall_server
{
 my $srv = $_[0];

 # remove log file
 my $log_file="$module_config_directory\/$_[0].log";
 &unlink_file ($log_file) if (-e $log_file);

 if ($config{'ztl_mode'})
 {
  # for ztl
  print "$text{'uninstall_configuring'}\n";
  print "=================================\n";
  if ($srv eq 'web')
  {
   foreach my $i (@web_list)
   {
    local $func = "uninstall_$i";
    &error(&text('uninstall_econfsub', $text{"server_$i"})) unless defined(&$func);
    &$func();
   }
  }
  else
  {
   local $func = "uninstall_$srv";
   &error(&text('uninstall_econfsub', $text{"server_$srv"})) unless defined(&$func);
   &$func();
  }
  # add confmaster log
  &write_postinstall_log($srv,'complete',1);

  print "\n$text{'uninstall_complete'}\n";
  print "=================================\n\n";
 }
 else
 {
  # for cpplus
  print "<tr $tb><td colspan=2><b>$text{'uninstall_configuring'}</b></td><tr>\n";
  local $func = "uninstall_$srv";
  &error(&text('uninstall_econfsub', $text{"server_$srv"})) unless defined(&$func);
  print "<tr $cb><td colspan=2>";
  &$func();

  #add confmaster log
  &write_postinstall_log($srv,'complete',1);

  print "</td></tr><tr $cb><td colspan=2><b><br>$text{'uninstall_complete'}<b></td></tr>\n";
 }
}

# --- Postinstall funcs ---

# &run_module_postinstall($server_name);
# run postinstall.pl script for modules of server
sub run_modules_postinstall
{
 my $srv = $_[0];
 
 # product name
 my $product = $config{'ztl_mode'} ? 'ZTL' : 'CP+';
 
 if ($config{'ztl_mode'} && !$reconf)
 {
  # for ztl
  print &text('setup_postinstalling',$product)."\n";
  print "=================================\n";
  if ($srv eq 'web')
  {
   foreach my $i (@web_list)
   {
    local $func_mods = "get_modules_$i";
    return 1 unless defined(&$func_mods);
    my ($func, $lib) = ('module_install', 'postinstall.pl');
    @mods = &$func_mods();
    return 2 unless @mods;
    foreach my $m (@mods)
    {
     next unless -x "$root_directory/$m/$lib";
     &msg_posttitle($m);
     &foreign_require($m, $lib);
     &foreign_call($m,  $func) if &foreign_defined($m,  $func);
     &msg_done();
    }
   }
  }
  else
  {
   local $func_mods = "get_modules_$srv";
   return 1 unless defined(&$func_mods);
   my ($func, $lib) = ('module_install', 'postinstall.pl');
   @mods = &$func_mods();
   return 2 unless @mods;
   foreach my $m (@mods)
   {
    next unless -x "$root_directory/$m/$lib";
    &msg_posttitle($m);
    &foreign_require($m, $lib);
    &foreign_call($m,  $func) if &foreign_defined($m,  $func);
    &msg_done();
   }
  }
  print "\n$text{'setup_complete'}\n";
  print "=================================\n\n";
 }
 else
 {
  # for install in the cpplus mode & reconfigure in the ztl mode
  local $func_mods = "get_modules_$srv";
  return 1 unless defined(&$func_mods);
  my ($func, $lib) = ('module_install', 'postinstall.pl');
  @mods = &$func_mods();
  return 2 unless @mods;
  print "<tr $tb><td colspan=2><b>".&text('setup_postinstalling',$product)."</b></td><tr>\n";
  foreach my $m (@mods)
  {
   next unless -x "$root_directory/$m/$lib";
   print "<tr $cb><td colspan=2>";
   &msg_posttitle($m);
   &foreign_require($m, $lib);
   &foreign_call($m,  $func) if &foreign_defined($m,  $func);
   &msg_done();
	print "</td><tr>\n";
  }
  print "<tr $cb><td colspan=2>";
  print "<b><br>$text{'setup_complete'}<b>";
  print "</td></tr>\n";
 }
 return 0;
}

# --- Website options configuration funcs ---

# &websites_configure($server_name);
# configure website options for server
sub websites_configure
{
 my $srv = $_[0];
 
 if ($config{'ztl_mode'} && !$reconf)
 {
  # for install in the ztl mode
  print "$text{'setup_wconfiguring'}\n";
  print "=================================\n";
  if ($srv eq 'web')
  {
   foreach my $i (@web_list)
   {
    local $func = "get_websites_configure_$i";
    &error(&text('setup_econfsub', $text{"server_$i"})) unless defined(&$func);
    my %wconfig = &$func();
    &foreign_require('virtual-server', 'virtual-server-lib.pl');
    map { $virtual_server::config{$_} = $wconfig{$_} } keys %wconfig;
    &virtual_server::save_module_config();
   }
  }
  else
  {
   local $func = "get_websites_configure_$srv";
   &error(&text('setup_econfsub', $text{"server_$srv"})) unless defined(&$func);
   my %wconfig = &$func();
   &foreign_require('virtual-server', 'virtual-server-lib.pl');
   map { $virtual_server::config{$_} = $wconfig{$_} } keys %wconfig;
   &virtual_server::save_module_config();
  }
  print "\n$text{'setup_complete'}\n";
  print "=================================\n\n";
 }
 else
 {
  # for install in the cpplus mode & reconfigure in the ztl mode
  print "<tr $tb><td colspan=2><b>$text{'setup_wconfiguring'}</b></td><tr>\n";
  local $func = "get_websites_configure_$srv";
  &error(&text('setup_econfsub', $text{"server_$srv"})) unless defined(&$func);
  print "<tr $cb><td colspan=2>";
  my %wconfig = &$func();
  &foreign_require('virtual-server', 'virtual-server-lib.pl');
  map { $virtual_server::config{$_} = $wconfig{$_} } keys %wconfig;
  &virtual_server::save_module_config();
  print "</td></tr>\n";
  print "<tr $cb><td colspan=2><br><b>$text{'setup_complete'}<b></td></tr>\n";
 }
}

# &uninstall_websites($server_name);
# delete website options for server
sub uninstall_websites_configure
{
 my $srv = $_[0];

 if ($config{'ztl_mode'})
 {
  # for ztl
  print "\n$text{'setup_wconfiguring'}\n";
  print "=================================\n";
  if ($srv eq 'web')
  {
   foreach my $i (@web_list)
   {
    local $func = "uninstall_websites_configure_$i";
    &error(&text('setup_econfsub', $text{"server_$i"})) unless defined(&$func);
    my %wconfig = &$func();
    &foreign_require('virtual-server', 'virtual-server-lib.pl');
    map { $virtual_server::config{$_} = $wconfig{$_} } keys %wconfig;
    &virtual_server::save_module_config();
   }
  }
  else
  {
   local $func = "uninstall_websites_configure_$srv";
   &error(&text('setup_econfsub', $text{"server_$srv"})) unless defined(&$func);
   my %wconfig = &$func();
   &foreign_require('virtual-server', 'virtual-server-lib.pl');
   map { $virtual_server::config{$_} = $wconfig{$_} } keys %wconfig;
   &virtual_server::save_module_config();
  }
  print "\n$text{'setup_complete'}\n";
  print "=================================\n\n";
 }
 else
 {
  # for cpplus
  print "<tr $tb><td colspan=2><b>$text{'setup_wconfiguring'}</b></td><tr>\n";
  local $func = "uninstall_websites_configure_$srv";
  &error(&text('setup_econfsub', $text{"server_$srv"})) unless defined(&$func);
  print "<tr $cb><td colspan=2>";
  my %wconfig = &$func();
  &foreign_require('virtual-server', 'virtual-server-lib.pl');
  map { $virtual_server::config{$_} = $wconfig{$_} } keys %wconfig;
  &virtual_server::save_module_config();
  print "</td></tr>\n";
  print "<tr $cb><td colspan=2><br><b>$text{'setup_complete'}<b></td></tr>\n";
 }
}

#------ Confmaster flags funcs ------

# &set_configured_flag($server)
# set config value in 1 for server
sub set_configured_flag
{
 $config{"configured_$_[0]"}=1;
 &save_module_config();
 if ($config{'ztl_mode'})
 {
  $gconfig{"$_[0]_app_on"}=1;
  &save_module_config(\%gconfig,'/');
 }
 # add confmaster log
 &write_postinstall_log($_[0],'complete',1);
}

# &del_configured_flag($server)
# set config value in 0 for server
sub del_configured_flag
{
 $config{"configured_$_[0]"}=0;
 &save_module_config();
 if ($config{'ztl_mode'})
 {
  $gconfig{"$_[0]_app_on"}=0;
  &save_module_config(\%gconfig,'/');
 }
}

# &get_configured_flag($server)
# get config value for server
sub get_configured_flag
{
 if ($_[0] eq 'database')
 {
  return ($config{"configured_mysql"} && $config{"configured_postgres"});
 }
 elsif ($_[0] eq 'file')
 {
  return ($config{"configured_pdc"} && $config{"configured_cifs"} && $config{"configured_print"});
 }
 elsif ($_[0] eq 'lan')
 {
  return ($config{"configured_proxy"} && $config{"configured_dhcp"} && $config{"configured_firewall"});
 }
 else {return $config{"configured_$_[0]"};}
}

#------ Confmaster log funcs ------

# &write_postinstall_log(server)
# write postinstall.log file for server in the confmaster module directory
sub write_postinstall_log
{
 my $log_file="$module_config_directory\/$_[0].log";
 my $log;
 &read_file($log_file,$log);
 $log->{$_[1]}=$_[2];
 &write_file($log_file,$log);
}

# &write_debug_log()
# write /var/log/confmaster.log file
sub write_debug_log
{
 my ($text,$name,@param)=(@_);
 my $log_file="$var_directory\/confmaster.log";
 my $log;

 # open confmaster.log
 open (LOG,">>$log_file");
 my @log_file=<LOG>;

 # print time
 print LOG "[".localtime (time)."]";

 # print caller contecst
 my @contecst=caller;
 print LOG "  $contecst[0]  $contecst[1]  $contecst[2]: $text\n";

 if ($name)
 {
  # print functions call
  print LOG "$name (";
  my $last=pop(@param);
  foreach (@param)
  {
   if (defined $_){print LOG "$_,";}
   else {print LOG 'undef,';}
  }
  print LOG "$last)\n";
 }
 # close confmaster.log
 close (LOG);
}

# &get_confmaster_log($srv)
# return \%hash (name=value) from confmaster.log
sub get_confmaster_log
{
 my $log_file="$module_config_directory\/$_[0].log";
 my %log;
 &read_file($log_file,\%log);
 return \%log;
}

# &temp_chmod()
# change permissions for temp directory
sub temp_chmod
{
 my @temp=split (/:/,$config{'temp_path'});
 foreach (@temp)
 {
  if (-k $_)
  {
   my @mode=stat ($_);
   &backquote_logged("chmod 1777 $_ </dev/null 2>&1") if ($mode[2] ne '17407');
  }
 }
}

#------ Interface funcs ------

# &print_servers_table([submenu]);
# print icons for server or appliance
sub print_servers_table
{
 print "<table cellpadding=10>\n";
 my @srvs;
 if ($config{'ztl_mode'})
 {
  # for ztl
  if ($_[0] eq 'database') { @srvs = @sub_db_list; }
  elsif ($_[0] eq 'file') { @srvs = @sub_file_list; }
  elsif ($_[0] eq 'lan') { @srvs = @sub_lan_list; }
  else {@srvs = ('web','database','dns','mail','file','lan');}
 }
 else 
 {
  # for cpplus
  @srvs = @servers_list; 
 }
 # print icons
 foreach (@srvs)
 {
  my $srv = $_;
  my $flag = &get_configured_flag($srv);
  my $icon = $flag ? "$srv.gif" : "${srv}_off.gif";
  if ($config{'ztl_mode'})
  {
   # for ztl
   if ($srv eq 'database' || $srv eq 'file' || $srv eq 'lan')
   {
    my $descr = $flag ? $text{"descr_appl_$srv"}."<br>".&text('config_flag', $text{"appliances"}) : $text{"descr_appl_$srv"};
    print "<tr><td class=\"infotitle\" width=30%><a href=\"index.cgi?srv=".$srv."\"><img width=48 height=48 src=\"images/$icon\" border=\"0\" />&nbsp; <font size=+1>".$text{"server_appl_$srv"}."</font></a><td>$descr</td><tr>\n";
   }
   else
   {
	 my ($descr, $mta, $imap, $ftp);
	 if ($srv eq 'mail')
	 {  
	  $mta = ($config{'mta'} == 1) ? $text{'sendmail'} : $text{'postfix'};
	  $imap = ($config{'imap'} == 0) ? $text{'courier'} : ($config{'imap'} == 1) ? $text{'dovecot'} : $text{'imap'};
	  $descr = $flag ? &text("descr_appl_$srv", $mta, $imap)."<br>".&text('config_flag', $text{"appliance"})."<a class=menu href=\"appl_info.cgi?srv=".$srv."\">".&text('appl_log',undef)."</a>" : &text("descr_appl_$srv", $mta, $imap);
	 }
	 elsif ($srv eq 'web')
	 {
	  $ftp = ($config{'ftp_system'} == 1) ? $text{'vsftpd'} : $text{'proftpd'};
	  $descr = $flag ? &text("descr_appl_$srv", $ftp)."<br>".&text('config_flag', $text{"appliance"})."<a class=menu href=\"appl_info.cgi?srv=".$srv."\">".&text('appl_log',undef)."</a>" : &text("descr_appl_$srv", $ftp);
	 }
	 else
	 {
     $descr = $flag ? $text{"descr_appl_$srv"}."<br>".&text('config_flag', $text{"appliance"})."<a class=menu href=\"appl_info.cgi?srv=".$srv."\">".&text('appl_log',undef)."</a>" : $text{"descr_appl_$srv"};
	 }
    print "<tr><td class=\"infotitle\" width=30%><a href=\"configure.cgi?srv=".$srv."\"><img width=48 height=48 src=\"images/$icon\" border=\"0\" />&nbsp; <font size=+1>".$text{"server_appl_$srv"}."</font></a><td>$descr</td><tr>\n";
   }
  }
  else
  {
   # for cpplus
   my $descr = $flag ? $text{"descr_$srv"}."<br>".&text('config_flag', $text{"server"}) : $text{"descr_$srv"};
   print "<tr><td class=\"infotitle\"><a href=\"configure.cgi?srv=".$srv."\"><img width=48 height=48 src=\"images/$icon\" border=\"0\" />&nbsp; <font size=+1>".$text{"server_$srv"}."</font></a><td>$descr</td><tr>\n";
  }
 }
 print "</table>\n";
}

# &print_conf_errors($srv, $types)
# print module configuration errors
sub print_conf_errors
{
 my ($srv, $types, $err) = @_;
  if (($gconfig{'os_type'} eq 'trustix-linux')) {
 	# trustux os
	if ($srv eq 'mail'){
   	# mta errors
 		if ($config{'mta'} == 1) {
			print &text('index_conf_err', "<b>$text{'sendmail'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";
			$err++;
			}	
		# imap errors
		if ($config{'imap'} == 1) {
			print &text('index_conf_err', "<b>$text{'dovecot'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";
			$err++;
			}	
		elsif ($config{'imap'} == 2) {
			print &text('index_conf_err', "<b>$text{'imap'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}
# 		if ($gconfig{'os_version'} >= 3.0) {
# 			# saslauthd errors
#  			if ($config{'sasl'}) {
# 				print &text('index_conf_err', "<b>$text{'cyrus-sasl'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";
# 				$err++;
# 				}	
# 				
# 			# amavis errors
#  			if ($config{'amavis'}) {
# 				print &text('index_conf_err', "<b>$text{'amavis'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";		
# 				$err++;
# 				}	
# 		
# 			# clamav errors
#  			if ($config{'clamav'}) {
# 				print &text('index_conf_err', "<b>$text{'clamav'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";		
# 				$err++;
# 				}	
# 		
# 			# razor errors
#  			if ($config{'razor'}) {
# 				print &text('index_conf_err', "<b>$text{'razor'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";		
# 				$err++;
# 				}	
# 			}
		}
	if ($srv eq 'web'){
		# webalizer errors
 		if ($config{'web_system'} == 1) {
			print &text('index_conf_err', "<b>$text{'webalizer'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";
			$err++;
			}
		}
	if ($srv eq 'ftp'){
		# ftp errors
 		if ($config{'ftp_system'} == 1) {
			print &text('index_conf_err', "<b>$text{'vsftpd'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";
			$err++;
			}
		}
 	}
 elsif (($gconfig{'os_type'} eq 'redhat-linux')) {
 	# redhat os
	if ($srv eq 'mail'){
		# mta errors
 		if ($config{'mta'} && $config{'mailman'}) {
			print &text('index_conf_sendmail_err', "<b>$text{'mailman'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}		
	
		# imap errors
		if ($config{'imap'} == 0) {
			print &text('index_conf_err', "<b>$text{'courier'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}
		elsif ($config{'imap'} == 1 && ($gconfig{'os_version'} < 11.0)) {
			# centos 3x, old redhat
			print &text('index_conf_err', "<b>$text{'dovecot'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}	
		elsif ($config{'imap'} == 2 && ($gconfig{'os_version'} >= 11.0)) {
			# centos 4x
			print &text('index_conf_err', "<b>$text{'imap'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}	
			
		# amavis errors
 		if ($config{'amavis'}) {
			print &text('index_conf_err', "<b>$text{'amavis'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}	
		
		# clamav errors
 		if ($config{'clamav'}) {
			print &text('index_conf_err', "<b>$text{'clamav'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}	
		
		# razor errors
 		if ($config{'razor'}) {
			print &text('index_conf_err', "<b>$text{'razor'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}	
		}
	if ($srv eq 'ftp'){
		# ftp errors
 		if ($config{'ftp_system'} == 0) {
			print &text('index_conf_err', "<b>$text{'proftpd'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}
		}
 	}
 elsif (($gconfig{'os_type'} eq 'debian-linux')) {
 	# debian os
	if ($srv eq 'mail'){
		# mta errors
 		if ($config{'mta'} == 1) {
			print &text('index_conf_err', "<b>$text{'sendmail'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";
			$err++;
			}	
			
		# imap errors
		if ($config{'imap'} == 1) {
			# dovecot
			print &text('index_conf_err', "<b>$text{'dovecot'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}	
		elsif ($config{'imap'} == 2) {
			# imap2002
			print &text('index_conf_err', "<b>$text{'imap'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}	
			
		# amavis errors
 		if ($config{'amavis'}) {
			print &text('index_conf_err', "<b>$text{'amavis'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}	
		
		# clamav errors
 		if ($config{'clamav'}) {
			print &text('index_conf_err', "<b>$text{'clamav'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}	
		
		# razor errors
 		if ($config{'razor'}) {
			print &text('index_conf_err', "<b>$text{'razor'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}	
			
		# mailman errors
 		if ($config{'mailman'}) {
			print &text('index_conf_err', "<b>$text{'mailman'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}	
		}
		
	if ($srv eq 'ftp'){
		# ftp errors
 		if ($config{'ftp_system'} == 1) {
			print &text('index_conf_err', "<b>$text{'vsftpd'}</b>","$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";			
			$err++;
			}
		}
 	}
 
 # exit
 if ($err){ 
	&ui_print_footer("", &text('index_return', $types));
	exit;
	}
}

# &conf_title($name)
# print name title message
sub msg_conftitle
{
 if ($config{'ztl_mode'} && !$reconf) {print &text('setup_configure', $_[0]) . "\n";}
 else {print &text('setup_configure', $_[0]) . "<br>\n";}
}

# &conf_title($name)
# print name uninstall message
sub msg_conftitle_uninstall
{
 if ($config{'ztl_mode'} && !$reconf) {print &text('uninstall_configure', $_[0]) . "\n";}
 else {print &text('uninstall_configure', $_[0]) . "<br>\n";}
}

# &conf_title($name)
# print name postinstall message
sub msg_posttitle
{
 if ($config{'ztl_mode'} && !$reconf) {print &text('setup_postinstall', $_[0]) . "\n";}
 else {print &text('setup_postinstall', $_[0]) . "<br>\n";}
}

# &msg_done();
# print done message
sub msg_done
{
 # for install in the ztl mode
 if ($config{'ztl_mode'} && !$reconf)
 {
  if (!$_[0]) {print "$text{'index_done'}\n";}
  else {print "$text{'index_failed'}\n";}
 }
 # for install in the cpplus mode & upgrade in the ztl mode
 else
 {
  if (!$_[0]) {print "$text{'index_done'}<br>\n";}
  else {print "<b>$text{'index_failed'}</b><br>\n";}
 }
}

# &print_postinstall_log(server,[1])
# print summary information for install or uninstall server or appliance
# if $_[1] = 1 - uninstall mode, else - install
sub print_postinstall_log
{
 # get log file
 my $log_file = "$module_config_directory\/$_[0].log";
 
 # get templete file name
 my $tpl_file;
 if ($_[1]) {
 	# uninstall mode
	$tpl_file = "$root_directory\/$module_name\/template.uninstall";
	}
 else {
	# install mode
	$tpl_file = "$root_directory\/$module_name\/template.install";
	}
 
 # get text mode
 my $complete_mode = $config{'ztl_mode'} ? 'Appliance:' : 'Server:' if ($_[0] ne 'quota');
 
 # get section name
 my $bl = $_[0];
 
 if(-e $tpl_file && -e $log_file)
 {
  # print summary logs
  print "<tr $tb><td colspan=2><b>$text{'summary_log'}</b></td><tr>\n";
  
  # read log file
  &read_file($log_file, \%log);
  
  # read templete file
  my (@show_keys, $read);
  open(TPL,"$tpl_file");
  while(<TPL>)
  { 
   chomp;	
	# check read status
   if(/\[\/$bl\]/) { undef($read); }
	elsif(/\[$bl\]/)
	{
	 $read = 1;
	 next;
	}
   if ($read)
   {
    # read templete block
    my @par = split(/:/,$_);	    
    if($par[1]=~/\W(\S+)\W/)
    {  
     # get [name] from line
     $par[1] = $1;
     my @log_keys = keys(%log); 
     foreach my $tkey(@log_keys)
     {
      # create array from log file like [name]
      if($tkey=~/^($par[1])/){$show_keys[++$#show_keys] = $tkey;}
     }
    }
    else { @show_keys = $par[1]; }
	 
    foreach my $tt (@show_keys)
    {
     # show all summary information 
     $par[1] = $tt;
     if($log{$par[1]})
     {
      # done   
      if(($log{$par[1]} ne 1) && !$par[2])
      { 
		 # print advanced string
       print "<tr $cb><td>&nbsp;&nbsp;&nbsp;&nbsp;".$par[0]." </td><td><b><font color=blue>".$log{$par[1]}."</b> </font></td></tr>\n";
       next;
      }
		
		# print main finish string
		$par[0] = "<b>".$par[0]." $complete_mode</b>" if ($par[1] eq "complete");

		# print done string
      print "<tr $cb><td>".$par[0]." </td><td><b><font color=green>DONE</b> </font></td></tr>\n" if (eval($par[2]) || !$par[2]);
     }
     elsif(!$log{$par[1]} && eval($par[2]))
     {
      # failed
      if($par[1] eq "complete")
		{
		 # print main finish string
		 $par[0]="<b>".$par[0]." $complete_mode</b>";
		}
		# print error string
      print "<tr $cb><td>".$par[0]." </td><td><font color=red><b>FAILED</b> </font></td></tr>\n";
     }
     undef($tt);
    }
    undef(@show_keys);
   }
  }
  close(TPL);
  
  # delete log file
  &unlink_file ($log_file) if ($_[1]);
 }    
}


# --- subsidiary ---

# &samba_restart()
# start or restart samba services
sub samba_restart
{
 # service samba status
 my $out;
 unless (&samba::is_samba_running())
 {
  # start samba service
  if ($samba::config{'start_cmd'})
  {
   # with script
   $out = &backquote_logged("$samba::config{'start_cmd'} 2>&1");
   &error($out) if $?;
   &samba::webmin_log("start");
  }
  else { &error(&samba::text('start_fail',$samba::config{'start_cmd'})); }
 }
 else
 {
  # restart samba service
  if ($samba::config{'stop_cmd'})
  {
   # with script
   $out = &backquote_logged("$samba::config{'stop_cmd'} 2>&1");
   &error($out) if $?;
  }
  else { &error(&samba::text('start_fail', $samba::config{'stop_cmd'})); }
  
  if ($samba::config{'start_cmd'})
  {
   # with script
   $out = &backquote_logged("$samba::config{'start_cmd'} 2>&1");
   &error($out) if $?;
  }
  else { &error(&samba::text('start_fail', $samba::config{'start_cmd'})); }
  
  &samba::webmin_log("restart");
 }
}

# &samba_stop()
# stop samba services
sub samba_stop
{
 # service samba status
 if (&samba::is_samba_running())
 {
  # stop samba service
  if ($samba::config{'stop_cmd'})
  {
   # with script
   my $out = &system_logged("$samba::config{'stop_cmd'} 2>&1");
   &error($out) if $?;
   &samba::webmin_log("stop");
  }
  else { &error(&samba::text('start_fail', $samba::config{'stop_cmd'})); }
 }
}

# &save_samba_directives(share,\%hash)
# save new samba share directives from hash
sub save_samba_directives
{
 my $share = &samba::get_share($_[0]);

 # set options
 foreach (keys %{$_[1]})
 {
  &samba::setval($_,$_[1]->{$_});
 }
 # save options
 if ($share) { &samba::modify_share($_[0], $_[0]); }
 else { &samba::create_share($_[0]); }
}

# &delete_samba_directives(share,\%hash)
# delete share samba directives from hash
sub delete_samba_directives
{
 my $share = &samba::get_share($_[0]);

 #set options
 foreach (keys %{$_[1]})
 {
  &samba::delval($_,$_[1]->{$_});
 }
 #save options
 &samba::modify_share($share,$share);
}

# &samba_passdb_check()
# make clean file - passdb.tdb if needed
sub samba_passdb_check
{
 my @path = split (/\//,$samba::config{'smb_conf'});
 pop (@path);
 my $pass_file = join ('/',@path)."/passdb.tdb";
 unless (-e $pass_file) 
 { 
  # make clean file
  my $out = &backquote_command ("touch $pass_file 2>&1 </dev/null"); 
  &error ($out) if $?;
  
  # set permissions
  chmod(0644, $pass_file);
 }
}
 
# &lan_check([$all])
# return array of lan interfaces
# only for LAN if !$all
sub lan_check
{
 # require librarys from module net
 &foreign_require('net','net-lib.pl');

 # list of interfaces brought up at boot time
 my @boot = &net::boot_interfaces();
 my (@lan, @all_int);
 foreach (@boot)
 {
  if (!$_->{'dhcp'} && !$_->{'bootp'} && $_->{'fullname'} ne 'lo')
  {
   push (@lan, $_) if ($_->{'address'}=~/^192\.168\./ || $_->{'address'}=~/^172\.[16..31]\./ || $_->{'address'}=~/^10\./);
	push (@all_int, $_);
  }
 }
 return @all_int if $_[0];
 return @lan;
}

# &backup_config($filename)
# backup config file to filename.confmaster_save
sub backup_config
{
 if (-e $_[0] && !(-e $_[0].'.confmaster_save'))
 {
  return &copy_source_dest ($_[0],$_[0].'.confmaster_save');
 }
}

# &restore_config($filename)
# restore config file from filename.confmaster_save
sub restore_config
{
 if (-e $_[0].'.confmaster_save')
 {
  my $out=&copy_source_dest ($_[0].'.confmaster_save',$_[0]);
  &unlink_file($_[0].'.confmaster_save');
  return $out;
 }
}

# &selinux_disable()
# disable SELinux
sub selinux_disable
{
 # check selinux command
 my $cmd = &has_command('getenforce');
 return 0 unless $cmd;
 
 # get selinux status
 my $out = `$cmd`;
 return 0 if ($out =~ /disabled/i);
 
 # permissive selinux
 $cmd = &has_command('setenforce');
 $out = `$cmd 0`;
 return wantarray ? ($out, $?) : $out;
}

1;

