# configure BIND 8 DNS Server

# &configure_dns();
# call configure functions for all dns services
sub configure_dns
{
 if (&configured('bind8'))
 {
  &msg_conftitle($text{'bind8'});
  &bind8_module_configure();
  &msg_done();
 }
 else {&error(&text('index_bind8','/bind8/index.cgi'));}
}

# &uninstall_dns()
# call uninstall functions for all dns services
sub uninstall_dns
{
 &msg_conftitle_uninstall($text{'bind8'});
 &bind8_module_uninstall();
 &msg_done();
}

# &get_modules_dns()
# return all name modules for search postinstall scripts
sub get_modules_dns
{
 return ('bind8');
}

# &get_websites_configure_dns()
# return %rv with new options for virtual-server config
sub get_websites_configure_dns
{
 local %rv = ();
 $rv{'dns'} = &configured('bind8') ? 1 : 0;
 return %rv;
}

# &uninstall_websites_configure_dns()
# return %rv with zero options for virtual-server config
sub uninstall_websites_configure_dns
{
 local %rv = ();
 $rv{'dns'} = 0;
 return %rv;
}

# &bind8_module_configure()
# configure bind8
sub bind8_module_configure
{
 # require librarys from modules bind & init
 &foreign_require('bind8','bind8-lib.pl');
 &foreign_require('init','init-lib.pl');
 &foreign_require('useradmin','user-lib.pl');
 &foreign_require("net", "net-lib.pl");    

 # check service name
 my $service_name='named';

 # backup named.conf
 my $out;
 unless (-e $bind8::config{'named_conf'}.'.confmaster_save' || -e $bind8::config{'named_conf'}.'.ztl_save')
 {
  &write_debug_log(&text('backup','Named'),'&backup_config',$bind8::config{'named_conf'});
  $out = &backup_config($bind8::config{'named_conf'});
  
  # add confmaster log
  &write_postinstall_log('dns','backup',$bind8::config{'named_conf'}) if $out;
 } 

  # New conf
  # Stop named before configuring
  &write_debug_log(&text('stop',$service_name),'&bind8::stop_bind');
  &bind8::stop_bind();
      
  my( $chr_dir, $wrk_dir, $cnf_dir, $run_dir, $rndc_conf_dir, $cnf_file, 
  $is_rndc, @hosts, @group, $pid_file, $var, $tmp);

  # Default values
  my %def = ( 'chr_dir' => "/var/named/chroot",
    'wrk_dir' => "/",
    'cnf_dir' => "/etc",
    'cnf_file' => "named.conf",
    'run_dir' => "/var/run/cpplus/named",
    'pid_file' => "named.pid",
    'rndc_conf_file' => "rndc.conf"
    );   

# For trustix-linux system
if( $gconfig{'os_type'} eq 'trustix-linux') {
 $cnf_file = "/etc/named.conf";
 if( $gconfig{'os_version'} =~ /^2/) {
    # We need fix named.conf for trustix ver2
    &write_debug_log(&text('copy_file', 
    "$module_root_directory/addons/dns/named.conf_trustix3",
    $cnf_file, '&copy_source_dest'));
    if( -T $cnf_file) { unlink($cnf_file);}
    unless( &copy_source_dest("$module_root_directory/addons/dns/named.conf_trustix3", 
    $cnf_file)) { &error(&text('dns_conf_err', $cnf_file));}
    chmod( 0644, $cnf_file);	
    }          
 # Parse conf file and try to find work_dir
 &lock_file($cnf_file);  $var = &read_file_lines( $cnf_file);
 foreach $tmp ( @{$var}) { 
    if( $tmp =~ /\s?directory\s+\"(\/.*)\/?\"\;/) { 
	$wrk_dir = $1 unless( $wrk_dir);}
    if( $tmp =~ /\s?include\s+\"\/.*\/(named|cpplus)\.zones\"\;/) { 
	$tmp = qq(\tinclude "$wrk_dir/cpplus.zones";);
	}
    if( $tmp =~ /\s?view\s+\"((internal|external)[^\"]+)\"\s+in\s+/) { 
	$is_rndc = $1 if($1);
	}
    }  
 &flush_file_lines($cnf_file); &unlock_file($cnf_file);
 
 # If work_dir is defined, copy cpplus.zones file to 
 # that location
 if( $wrk_dir) {
    &write_debug_log(&text('create_tree', "$wrk_dir", '&create_tree'));
    $var = &create_tree('/', "$wrk_dir");

    unless( -e "$wrk_dir/cpplus.zones") {   
	&write_debug_log(&text('copy_file', "$module_root_directory/addons/" .
	"dns/cpplus.zones","$wrk_dir/cpplus.zones", '&copy_source_dest'));
        if( &copy_source_dest("$module_root_directory/addons/dns" .
	"/cpplus.zones", "$wrk_dir/cpplus.zones")) {
	    # Set permissions on file
	    chmod( 0644, "$wrk_dir/cpplus.zones");	
	    $bind8::config{'zones_file'} = "$wrk_dir/cpplus.zones";
	    $virtual_server::config{'dns_view'} = $is_rndc;
	    }
	else { print &text('dns_zone_err', "$wrk_dir/cpplus.zones") . "<br>";}
	}
    else { 
	$bind8::config{'zones_file'} = "$wrk_dir/cpplus.zones";
	$virtual_server::config{'dns_view'} = $is_rndc;
	}
    }
 # Change bind8 config value:
 $bind8::config{'by_view'} = 1;
 $bind8::config{'soa_style'} = 1;
 $bind8::config{'show_list'} = 1;
 $bind8::config{'max_zones'} = 50;
 $bind8::config{'master_ttl'} = 1;
 $bind8::config{'updserial_on'} = 1;
 $bind8::config{'forwardzonefilename_format'} = "ZONE.hosts";
 $bind8::config{'reversezonefilename_format'} = "ZONE.rev";
 $bind8::config{'pid_file'} =  ""; 
}

# For other system we are trying create config
else {
  # Get chroot directory
  # "/" -bad choice, ignore it
  if( $bind8::config{'auto_chroot'}) {
    $chr_dir = `$bind8::config{'auto_chroot'} 2>/dev/null`;
    if ( ! $?) { $chr_dir =~ s/\r|\n//g;}
    else { $chr_dir = '';}
    }       
  if( ! $chr_dir && $bind8::config{'chroot'}) { 
    $chr_dir = $bind8::config{'chroot'};
    }

  if( ! $chr_dir) {
    if( $bind8::config{'master_dir'} and 
	$bind8::config{'master_dir'} ne '/') { 
	    $chr_dir = $bind8::config{'master_dir'};
	    $chr_dir =~ /(^.*)\/master[^\/]?\/?/;
	    if( $1) { $chr_dir = $1; $chr_dir .= "/chroot";}
	    else { $chr_dir .= "/chroot"; }
	}
    else { $chr_dir = $def{'chr_dir'}; }
    }
  elsif( $chr_dir eq '/') { $chr_dir = "/chroot"; }   	    

  # Get work dir
  if( $wrk_dir = $bind8::config{'master_dir'}) {
    if( $wrk_dir =~ /(^\/.*)\/master[^\/]?\/?/ ) { $wrk_dir = $1; }
    elsif( $wrk_dir !~ /^\// ) { $wrk_dir = $def{'wrk_dir'};}
    }
  elsif( $chr_dir =~ /^(\/.*)\/[^\/]{1,}$/) { $wrk_dir = $1;}
  else { $wrk_dir = $def{'wrk_dir'};}                
  
  # Last choice: $chr_dir, $wrk_dir and $config{'master_dir'}
  # must not be equivalent 
  if( $wrk_dir eq $chr_dir or "$wrk_dir/master_dir" eq $chr_dir) { 
    $chr_dir = "$wrk_dir/chroot";
    }

  # Change bind8 config value:
  $bind8::config{'chroot'} = $chr_dir;
  
  # Get directory with config file (usually /etc)
  # and name for config file (usually named.conf)
  if ( $cnf_dir = "$bind8::config{'named_conf'}") {
    $cnf_dir =~ /^(\/.*)\/([^\/\s]{1,})\s?$/;        
    if( $1) { $cnf_dir = $1;}
    else { $cnf_dir = '/';}
    if( $2) { $cnf_file = $2;}
    else { $cnf_file = $def{'cnf_file'};}
    }                
  else { $cnf_dir = $def{'cnf_dir'}; $cnf_file = $def{'cnf_file'};}

  # Change bind8 config value:
  $bind8::config{'named_conf'} =  "$cnf_dir/$cnf_file"; 
 
  # Get directory with pid file (usually /var/run/named)
  $run_dir = $def{'run_dir'}; $pid_file = $def{'pid_file'};

  # Change bind8 config value:
  $bind8::config{'pid_file'} =  "$run_dir/$pid_file"; 
  $bind8::config{'master_dir'} =  "$wrk_dir/master_dir"; 
  $bind8::config{'slave_dir'} =  "$wrk_dir/slave_dir"; 
                      
  # Check needed directory structure   
  # must exist $cnf_dir, $wrk_dir, $run_dir, $chr_dir 
  # $chr_dir/$cnf_dir, $chr_dir/$wrk_dir/data_dir, $chr_dir/$run_dir
  # $chr_dir/$wrk_dir/master_dir, $chr_dir/$wrk_dir/slave_dir 
  &write_debug_log(&text('create_tree', "$cnf_dir, $wrk_dir, " .
  "$run_dir, $chr_dir", '&create_tree'));

  foreach ( "$cnf_dir", "$wrk_dir", "$chr_dir") {
    if ( "$_" eq "$run_dir") { $var = &create_tree('/', $_, '0755', '0777');	}	  
    else { $var = &create_tree('/', $_);}
    unless( $var == 1) { &error(&text('dns_mkdir_err',$_));}   
    }    

  &write_debug_log(&text('create_tree', "$chr_dir$cnf_dir, " .
  "$chr_dir$wrk_dir/data_dir, $chr_dir$wrk_dir/master_dir, " .
  "$chr_dir$wrk_dir/slave_dir, $chr_dir$run_dir", '&create_tree'));

  foreach ("$cnf_dir", "$wrk_dir/data_dir", "$wrk_dir/master_dir", 
  "$wrk_dir/slave_dir", "$run_dir") {
    if ( "$_" eq "$run_dir") { 
	$var = &create_tree("$chr_dir", $_, '0755', '0777');
	}	  
    else { $var = &create_tree("$chr_dir", $_);}
    unless( $var == 1) { &error(&text('dns_mkdir_err',"$chr_dir$_"));}
    }   

  # Create links from $wrk_dir to $chr_dir$wrk_dir for 
  # dirs: data_dir, master_dir, $slave_dir
  &write_debug_log(&text('create_link', "$wrk_dir/data_dir(master_dir, slave_dir) -> " .
  "$chr_dir$wrk_dir/data_dir(master_dir, slave_dir)" ,'&symlink_logged'));

  foreach ( "data_dir", "master_dir", "slave_dir") {
    if( -l "$wrk_dir/$_") { unlink("$wrk_dir/$_");}
    unless( &symlink_logged("$chr_dir$wrk_dir/$_", "$wrk_dir/$_")) { 
	print &text('dns_zone_err', "$wrk_dir/$_") . "<br>";
	}
    }
  
  # Create dir for pid-file out of chroot dir
  $run_dir =~ /(^.*)\/([^\/]+)\/?$/;
  &write_debug_log(&text('create_tree', "$1", '&create_tree'));

  $var = &create_tree("/", $1);
  unless( $var == 1) { &error(&text('dns_mkdir_err',"$1"));}

  # Create links for $run_dir->$chr_dir$run_dir    
  &write_debug_log(&text('create_link', "$run_dir -> " .
  "$chr_dir$run_dir" ,'&symlink_logged'));

  if( -l "$1/$2") { unlink("$1/$2");}
  unless( &symlink_logged("$chr_dir$run_dir", "$1/$2")) { 
        print &text('dns_zone_err', "$run_dir") . "<br>";
	}

  # Check is rndc in the system
  $is_rndc = &bind8::has_ndc();
  if ( $is_rndc == 2) { 
    $tmp = undef;
    # Find directory for rndc.conf
    if ( $rndc_conf_dir = "$bind8::config{'rndc_conf'}") {
	$rndc_conf_dir =~ /^(\/.*)\/([^\/\s]+)\s?$/;        
	if( $1) { $rndc_conf_dir = $1; $tmp = $2;}
        else { $rndc_conf_dir = $cnf_dir; $tmp = $def{'rndc_conf_file'};}
    	
	# Create needed directory tree:
	foreach ( "$rndc_conf_dir", "$chr_dir$rndc_conf_dir") {
	    $var = create_tree('/', $_);
	    unless( $var == 1) { $is_rndc = 0; last; }
	    }
	}   
    else { $rndc_conf_dir = $cnf_dir;}
    if ( $is_rndc) { 
	if( $tmp) { $is_rndc = $tmp;}
	else { $is_rndc = $def{'rndc_conf_file'};}
	} 
    }                

  # Get group for named service ( named or bind)
  # If group doesn't exists, disable using rndc
  @group = &useradmin::list_groups(); $tmp = undef;
  foreach (@group) {
    if( $_->{'group'} eq 'named') { $tmp = "named"; last;}
    elsif( $_->{'group'} eq 'bind') { $tmp = "bind"; last;}
    }
  unless( $tmp) { $is_rndc = 0;}

  if( $is_rndc) {      
    &write_debug_log(&text('copy_file', "$chr_dir$cnf_dir/rndc.key, " .
    "$chr_dir$rndc_conf_dir/$is_rndc", '&copy_source_dest'));
    &write_debug_log(&text('create_link', "$cnf_dir/rndc.key -> " .
    "$chr_dir$cnf_dir/rndc.key,  $rndc_conf_dir/$is_rndc -> " .
    "$chr_dir$rndc_conf_dir/$is_rndc" ,'&symlink_logged'));
    }
  foreach ( "$cnf_dir/rndc.key", "$rndc_conf_dir/$is_rndc") {
    unless ( $is_rndc) { last; }
    # Check is exists file $_ and replace with our template 
    if( -T "$chr_dir/$_") { unlink("$chr_dir/$_");}
    /^\/.*\/([^\/]{1,})$/; $var = $1;
    if( $var ne "rndc.key") { $var = "rndc.conf"; }       
    unless( &copy_source_dest( "$module_root_directory/addons/dns/$var", 
    "$chr_dir$_")) { $is_rndc = 0; last;}
    
    # Change permissions
    if( &backquote_command("chown .$tmp $chr_dir$_ >/dev/null 2>&1"))
	{ $is_rndc = 0; last;}
    chmod( 0640, "$chr_dir$_");
    	 
    # Create link from $conf_dir to $chr_dir/$_
    if( -l "$_" or -T "$_") { unlink("$_");}
    unless( &symlink_logged("$chr_dir$_", "$_")) { $is_rndc = 0; last;}
    }
 
  # Replace template in rndc.key, rndc.conf
  $value = &sys_true_rand(24);
  foreach ( "$chr_dir$cnf_dir/rndc.key", "$chr_dir$rndc_conf_dir/$is_rndc") {
    unless ( $is_rndc) { last; }
    &lock_file($_);  $var = &read_file_lines($_);
    foreach $tmp ( @{$var}) { 
	$tmp =~ s/#SECRET#/$value/;  
	$tmp =~ s/#PATH_KEY#/$cnf_dir\/rndc.key/;
	}  
    &flush_file_lines($_); &unlock_file($_);
    }

  if( $is_rndc) {
    # Change bind8 config value:
    $bind8::config{'rndc_conf'} = "$rndc_conf_dir/$is_rndc";

    $is_rndc = "controls {\ninet 127.0.0.1 port 953 allow { localhost; }" .
    " keys { \"rndckey\"; };\n};\n\ninclude \"$cnf_dir/rndc.key\";";
    }

  # Try to get the version of BIND
  $tmp = `$bind8::config{'named_path'} -v 2>&1`;
  if( $tmp =~ /(bind|named)\s+([0-9\.]+)/i) { 
    $tmp = $2;
    if( $tmp =~ /^9/) { $tmp = "named.conf_v9";}
    else { $tmp = "named.conf_v8";}
    }    
  else { $tmp = "named.conf_v8";}
      
  &write_debug_log(&text('copy_file', "$chr_dir$cnf_dir/$cnf_file", 
  '&copy_source_dest'));
  &write_debug_log(&text('create_link', "$cnf_dir/$cnf_file -> " .
  "$chr_dir$cnf_dir/$cnf_file" ,'&symlink_logged'));

  # Now we create named.conf in dir $chroot/$cnf_dir
  if( -l "$chr_dir$cnf_dir/$cnf_file" or -T "$chr_dir$cnf_dir/$cnf_file") { 
    unlink("$chr_dir$cnf_dir/$cnf_file");
    } 
  if( -l "$cnf_dir/$cnf_file" or -T "$cnf_dir/$cnf_file") {
    unlink("$cnf_dir/$cnf_file");
    }
  unless(&copy_source_dest( "$module_root_directory/addons/dns/$tmp",
  "$chr_dir$cnf_dir/$cnf_file")) {
    &error(&text('dns_conf_err', "$chr_dir$cnf_dir/$cnf_file")); 
    }
  # Make link in $cnf_dir
  unless( &symlink_logged("$chr_dir$cnf_dir/$cnf_file", "$cnf_dir/$cnf_file")) {
    &error(&text('dns_conf_err', "$cnf_dir/$cnf_file")); 
    } 

  # Set permissions for config file
  chmod( 0644, "$chr_dir$cnf_dir/$cnf_file");
  
  #Prepare needed info
  @hosts = &get_trusted_hosts();
  $tmp = join(";\n", @hosts);
  unless( $is_rndc) { $is_rndc = undef;}

  # Replace template in named.conf
  &lock_file("$chr_dir$cnf_dir/$cnf_file");
  $var = &read_file_lines("$chr_dir$cnf_dir/$cnf_file");
  foreach ( @{$var}) { 
    s/#WORK_DIR#/$wrk_dir/; s/#RUN_DIR#/$run_dir/;
    s/#PID_FILE#/$pid_file/; s/#CONTROLS#/$is_rndc/;
    s/#TRUSTED_IP#/$tmp/;
    }  
  &flush_file_lines("$chr_dir$cnf_dir/$cnf_file"); 
  &unlock_file("$chr_dir$cnf_dir/$cnf_file");

  # Copy needed files 
  &write_debug_log(&text('copy_file', "localdomain.zone, localhost.zone, " .
  "named.broadcast, named.ca, named.ip6.local, named.local, " .
  "named.zero", '&copy_source_dest'));
 
  foreach ( "localdomain.zone", "localhost.zone", "named.broadcast", 
  "named.ca", "named.ip6.local", "named.local", "named.zero") {
    # Check is exists file $_ and replace it
    if( -T "$chr_dir$wrk_dir/master_dir/$_") { 
	unlink("$chr_dir$wrk_dir/master_dir/$_");
	}       
    unless(&copy_source_dest( "$module_root_directory/addons/dns/$_", 
    "$chr_dir$wrk_dir/master_dir/$_")) { 
        print &text('dns_zone_err', "$chr_dir$wrk_dir/master_dir/$_") . "<br>";
        next; 
        }
    # Set permissions on file
    chmod( 0644, "$chr_dir$wrk_dir/master_dir/$_");	
    }

  &write_debug_log(&text('copy_file', "ca.zones, cpplus.zones, " .
  "default.zones", '&copy_source_dest'));
  &write_debug_log(&text('create_link', "ca.zones, cpplus.zones, " .
  "default.zones from $wrk_dir ".
  "to $chr_dir$wrk_dir" ,'&symlink_logged'));

  foreach ( "ca.zones", "cpplus.zones", "default.zones") {
    # Remove old zone description files 
    if( -T "$chr_dir$wrk_dir/$_" && $_ ne "cpplus.zones") {
	unlink( "$chr_dir$wrk_dir/$_");
	}
    # Check is exists file $_ 
    unless( -T "$chr_dir$wrk_dir/$_") {        
	unless(&copy_source_dest( "$module_root_directory/addons/dns/$_", 
	"$chr_dir$wrk_dir/$_")) { 
	    print &text('dns_zone_err', "$chr_dir$wrk_dir/$_") . "<br>";
	    next; 
	    }
        # Set permissions on file
	chmod( 0644, "$chr_dir$wrk_dir/$_");	
	}
    
    # Create link from $wrk_dir to $chr_dir/$wrk_dir/$_
    if( -l "$wrk_dir/$_" or -T "$wrk_dir/$_") { unlink("$wrk_dir/$_");}
    unless( &symlink_logged("$chr_dir$wrk_dir/$_", "$wrk_dir/$_")) 
	{ print &text('dns_zone_err', "$wrk_dir/$_") . "<br>";}
    }
 
 # Change bind8 config value:
 $bind8::config{'zones_file'} = "$wrk_dir/cpplus.zones";
 $bind8::config{'by_view'} = 1;
 $bind8::config{'soa_style'} = 1;
 $bind8::config{'show_list'} = 1;
 $bind8::config{'max_zones'} = 50;
 $bind8::config{'master_ttl'} = 1;
 $bind8::config{'updserial_on'} = 1;
 $bind8::config{'forwardzonefilename_format'} = "ZONE.hosts";
 $bind8::config{'reversezonefilename_format'} = "ZONE.rev";
 
 # For virtual server (correct work with "view" directives)
 $virtual_server::config{'dns_view'} = "external";

 }
 
 # Save config for module BIND
 &write_debug_log(&text('modify','Bind8'), '&save_module_config', 
 \%bind8::config, 'bind8');
 &save_module_config(\%bind8::config,'bind8');
  
 # Save config for module virtual-server
 &write_debug_log(&text('modify',$virtual_server::module_name), '&save_module_config', 
 \%virtual_server::config, $virtual_server::module_name);
 &save_module_config(\%virtual_server::config,$virtual_server::module_name);
 
 # start or restart named
 &write_debug_log(&text('status',$service_name),'&bind8::is_bind_running');
 if (&bind8::is_bind_running())
 {
  # restart named
  &write_debug_log(&text('restart',$service_name),'&bind8::restart_bind');
  $out = &bind8::restart_bind();
  &error($out) if $out;
 }
 else
 {
  # start named
  &write_debug_log(&text('start',$service_name),'&bind8::start_bind');
  $out = &bind8::start_bind();
  &error($out) if $out;
 }
 
 # enable named on boot 
 &write_debug_log(&text('chk_en', $service_name), '&foreign_call', 'init', 
 'enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);

 #add confmaster log
 &write_postinstall_log('dns','dns',1);
}

# &bind8_module_uninstall()
# uninstall bind8
sub bind8_module_uninstall
{
 # require librarys from modules bind8 & init
 &foreign_require('bind8','bind8-lib.pl');
 &foreign_require('init','init-lib.pl');

 # check service name
 my $service_name='named';

 # stop named service
 &write_debug_log(&text('stop',$service_name),'&bind8::stop_bind');
 my $out = &bind8::stop_bind(); #if (&bind8::is_bind_running());
 # &error($out) if $out;

 # disable named service from boot
 &write_debug_log(&text('chk_dis',$service_name),'&foreign_call','init','disable_at_boot',$service_name);
 &foreign_call('init','disable_at_boot',$service_name);

 # restore named.conf
 &write_debug_log(&text('restore','Named'),'&restore_config',$bind8::config{'named_conf'});
 my $out=&restore_config($bind8::config{'named_conf'});
 
 # add confmaster log
 &write_postinstall_log('dns','restore',$bind8::config{'named_conf'}) if ($out);

 # add confmaster log
 &write_postinstall_log('dns','dns',1);
}


### New function for configure bind server

sub create_tree($$;$$)
# Check and create directory structure
# $1 -root directory
# $2 -needed directory tree (/1/2/3...)
# $3 -default right on directory
# $4 -rights on directory(last in tree!)
# Return:
# 1 -directory created( or exists)
# 2 -root directory don't exists (or it's -file)
# 3 -can't create tree, because some part of tree is a file
# 0 -some error
{
my( $root, $str, $def_right, $right, @tree);
( $root, $str, $def_right, $right) = @_;

$def_right ||= '0755';
$right ||= $def_right;

unless( -d $root) { return 2;}
@tree = split('/', $str);  
for ( $str = 1; $str <= $#tree; $str++) {
    if( -e "$root/$tree[$str]" && ! -d "$root/$tree[$str]") { return 3;}
    elsif ( -d "$root/$tree[$str]") { 
	if( $str == $#tree) { chmod( oct($right), "$root/$tree[$str]");}   
	else { chmod( oct($def_right), "$root/$tree[$str]");}
	$root = "$root/$tree[$str]"; next; 
	}
    if( $str == $#tree) { $def_right = "$right"; }
    mkdir ("$root/$tree[$str]") || return 0;     
    chmod (oct($def_right), "$root/$tree[$str]");
    $root = "$root/$tree[$str]";
    }
return 1;
}


sub get_trusted_hosts()
# Get array with trusted hosts(nets)
# require librarys from module net
{
my(@trust, $pref);

foreach ( &net::active_interfaces()) {
    if( $_->{'name'} eq 'lo') { next; }
    if( $_->{'address'} =~ /^192\.168\./ || 
	$_->{'address'} =~ /^172\.[16..31]\./ || $_->{'address'} =~ /^10\./) {
	    $pref = &net::mask_to_prefix($_->{'netmask'});
	    $_->{'address'} = $_->{'address'} . "/$pref";
	}
    push ( @trust, $_->{'address'});
    }
return @trust;
}

sub sys_true_rand($)
# Number of symbol in returned string 
# Used for generating rndc key
{   

my ($dev_line, $rand_line);
open(UR,"</dev/urandom") || return "as3rdHKvlcZxa40Ytd9gh3bP";
do {
    read (UR, $dev_line, 4096);
    $dev_line =~ s/[^\w\d]//g;
    $rand_line .= $dev_line;
    $rand_line =~ tr/[A-Za-z0-9]//cd;} 
    until (length($rand_line) >= $num_char);
close(UR);
return substr($rand_line,0, $_[0]);
}

1;
