# server-ftp.pl
# configure FTP Server
 
#---------------------------- Global ----------------------

$log_dest = $config{'ztl_mode'} ? 'web' : 'ftp';

# &update_rpms_ftp($rpms)
# only needed packages
sub update_rpms_ftp
{
 foreach (@{$_[0]}){
 	push (@rpms, $_) unless 
	
	# remove vsftpd
	((($config{'ftp_system'} == 0) && $_=~/^vsftpd/) ||
	
	# remove proftpd
	(($config{'ftp_system'} == 1) && $_=~/^proftpd/));
	}
 return \@rpms;
}

# &configure_ftp()
# call configure functions for all ftp services
sub configure_ftp
{
 if ($config{'ftp_system'} == 0)
 {
  # configure proftpd
  if (&configured('proftpd'))
  {
   # configure
   &msg_conftitle($text{'proftpd'});
   &proftpd_module_configure();
   &msg_done();
  }
  else {&error(&text('index_proftpd','/proftpd/index.cgi')); }
 }
 elsif ($config{'ftp_system'} == 1)
 {
  # configure vsftpd
  if (&configured('vsftpd'))
  {
   # configure
   &msg_conftitle($text{'vsftpd'});
   &vsftpd_module_configure();
   &msg_done();
  }
  else {&error(&text('index_vsftpd','/vsftpd/index.cgi')); }
 }
}

# &uninstall_ftp()
# call uninstall functions for all ftp services
sub uninstall_ftp
{
 if ($config{'ftp_system'} == 0)
 {
  # uninstall proftpd
  &msg_conftitle_uninstall($text{'proftpd'});
  &proftpd_module_uninstall();
  &msg_done();
 }
 elsif ($config{'ftp_system'} == 1)
 {
  # uninstall vsftpd
  &msg_conftitle_uninstall($text{'vsftpd'});
  &vsftpd_module_uninstall();
  &msg_done();
 }
}

# &get_modules_ftp()
# return all name modules for search postinstall scripts
sub get_modules_ftp
{
 # configure vsftpd
 return ('proftpd') if ($config{'ftp_system'} == 0);;
 
 # configure vsftpd
 return ('vsftpd') if ($config{'ftp_system'} == 1);
}

# &get_websites_configure_ftp()
# return %rv with new options for virtual-server config
sub get_websites_configure_ftp
{
 local %rv = ();
 $rv{'ftp'} = 1;
 $rv{'ftp_system'} = 1 if ($config{'ftp_system'} == 0);
 $rv{'ftp_system'} = 2 if ($config{'ftp_system'} == 1);
 return %rv;
}

# &uninstall_websites_configure_ftp()
# return %rv with zero options for virtual-server config
sub uninstall_websites_configure_ftp
{
 local %rv = ();
 $rv{'ftp'} = 0;
 return %rv;
}

#---------------------------- ProFTPd configure ----------------------

# &proftpd_module_configure()
# configure proftpd
sub proftpd_module_configure
{
 # require librarys from module proftpd
 &foreign_require('proftpd','proftpd-lib.pl');
 &foreign_require('init','init-lib.pl');

 # check service name
 my $service_name = 'proftpd';
 
 if (&configured('vsftpd')) {
 	# stop & disable vsftpd
	&disable_vsftpd();
	}
 
 # Find out how ProFTPd is running
 my $inet = &proftpd::running_under_inetd();
 my $out;
 if (!$inet) {
	# Get the FTP server pid
	&write_debug_log(&text('status', $service_name),'&proftpd::get_proftpd_pid');
	if (&proftpd::get_proftpd_pid()){
		# restart proftpd
		&write_debug_log(&text('start',$service_name),'&proftpd::apply_configuration');
		&proftpd::apply_configuration();
		}
	else {
		# start proftpd
		&write_debug_log(&text('start',$service_name),'&proftpd::start_proftpd');
		$out = &proftpd::start_proftpd()
		}
	&error($out) if $out;
	}

 # enable proftpd on boot
 &write_debug_log(&text('chk_en',$service_name), '&foreign_call', 'init','enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);

 # add confmaster log
 &write_postinstall_log($log_dest,'proftpd',1);
}

# &proftpd_module_uninstall()
# uninstall proftpd
sub proftpd_module_uninstall
{
 # require librarys from modules proftpd & init
 &foreign_require('proftpd','proftpd-lib.pl');
 &foreign_require('init','init-lib.pl');

 # check service name
 my $service_name = 'proftpd';

 # stop & disable proftpd
 &disable_proftpd();

 # add confmaster log
 &write_postinstall_log($log_dest,'proftpd',1);
}

# &disable_proftpd()
# stop & disable proftpd
sub disable_proftpd
{
 # require librarys from module proftpd
 &foreign_require('proftpd','proftpd-lib.pl');

 # check service name
 my $service_name = 'proftpd';

 # Find out how ProFTPd is running
 my $inet = &proftpd::running_under_inetd();
 if (!$inet) {
	# Get the FTP server pid
	&write_debug_log(&text('status', $service_name),'&proftpd::get_proftpd_pid');
	if (&proftpd::get_proftpd_pid()){
		# stop proftpd
		&write_debug_log(&text('stop',$service_name),'&proftpd::stop_proftpd');
		my $out = &proftpd::stop_proftpd();
		&error($out) if $out;
		}
	}

 # disable proftpd service from boot
 &write_debug_log(&text('chk_dis',$service_name),'&foreign_call','init','disable_at_boot', $service_name);
 &foreign_call('init','disable_at_boot',$service_name);
}

#---------------------------- VsFTPd configure ----------------------

# &vsftpd_module_configure()
# configure vsftpd
sub vsftpd_module_configure
{
 # require librarys from module vsftpd
 &foreign_require('vsftpd','vsftpd-lib.pl');
 &foreign_require('init','init-lib.pl');
 
 # check service name
 my $service_name = 'vsftpd';
 
 if (&configured('proftpd')) {
 	# stop & disable proftpd
	&disable_proftpd();
	}
  
 # backup vsftpd.conf
 &write_debug_log(&text('backup','VsFTPd'),'&backup_config',$vsftpd::config{'vsftpd_conf'});
 my $out = &backup_config($vsftpd::config{'vsftpd_conf'});
 
 # add confmaster log
 &write_postinstall_log($log_dest,'backup_vsftpd',$vsftpd::config{'vsftpd_conf'}) if $out;
 
 # configure vsftpd.conf 
 my $conf = &vsftpd::parse_config_file();
 
 # set first ne dhcp IP or lo to 'listen_address' options
 &write_debug_log($text{'ip_lan'},'lan_check', 1);
 my @ip = &lan_check(1);
 my $ip = $ip[0]->{'address'} ? $ip[0]->{'address'} : '127.0.0.1';
 &vsftpd::set_config_value('listen_address',$ip,$conf);
 
 # set standalone mode
 &vsftpd::set_config_value('listen','YES',$conf);
 
 # allow local users 
 &vsftpd::set_config_value('local_enable','YES',$conf);
 
 # save options
 &write_debug_log(&text('modify','VsFTPd'),'&vsftpd::save_new_options',$conf);
 &vsftpd::save_new_options($conf); 
 
 # add confmaster log
 &write_postinstall_log($log_dest,'ip_vsftpd',$ip);
 
 # start or restart vsftpd
 &write_debug_log(&text('status',$service_name),'&vsftpd::status_vsftpd');
 if (&vsftpd::status_vsftpd())
 {
  # restart vsftpd
  &write_debug_log(&text('restart',$service_name),'&vsftpd::restart_vsftpd');
  &vsftpd::restart_vsftpd();
 }
 else
 {
  # start vsftpd
  &write_debug_log(&text('start',$service_name),'&vsftpd::start_vsftpd');
  &vsftpd::start_vsftpd();
 }
 
 # enable vsftpd on boot
 &write_debug_log(&text('chk_en', $service_name), '&foreign_call', 'init', 'enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);

 # add confmaster log
 &write_postinstall_log($log_dest,'vsftpd',1);
}

# &vsftpd_module_uninstall()
# uninstall vsftpd
sub vsftpd_module_uninstall
{
 # require librarys from module vsftpd
 &foreign_require('vsftpd','vsftpd-lib.pl');
 &foreign_require('init','init-lib.pl');
 
 # check service name
 my $service_name='vsftpd';
 
 # stop & disable vsftpd
 &disable_vsftpd();

 # restore vsftpd.conf
 &write_debug_log(&text('restore','VsFTPd'),'&restore_config',$vsftpd::config{'vsftpd_conf'});
 my $out = &restore_config($vsftpd::config{'vsftpd_conf'});

 # add confmaster log
 &write_postinstall_log($log_dest,'restore_vsftpd',$vsftpd::config{'vsftpd_conf'}) if $out;
 &write_postinstall_log($log_dest,'vsftpd',1);
}

# &disable_vsftpd()
# stop & disable vsftpd
sub disable_vsftpd
{
 # require librarys from module vsftpd 
 &foreign_require('vsftpd','vsftpd-lib.pl');
 
 # check service name
 my $service_name = 'vsftpd';
 
 &write_debug_log(&text('status',$service_name),'&vsftpd::status_vsftpd');
 if (&vsftpd::status_vsftpd())
 {
  # stop vsftpd
  &write_debug_log(&text('stop',$service_name),'&vsftpd::stop_vsftpd');
  &vsftpd::stop_vsftpd();
 }
  
 # disable vsftpd service from boot
 &write_debug_log(&text('chk_dis', $service_name),'&foreign_call','init','disable_at_boot', $service_name);
 &foreign_call('init','disable_at_boot', $service_name);
}

1;
