# server-mail.pl
# configure Mail Server

#---------------------------- Global ----------------------

# &update_rpms_mail($rpms)
# only needed packages
sub update_rpms_mail
{
 foreach (@{$_[0]})
 {
  push (@rpms, $_) unless 
  
  # mta packages
  (($config{'mta'} != 1 && $_=~/^sendmail/) ||
  ($config{'mta'} && $_=~/^postfix/) ||
	
	# imap/pop3 packages
	($config{'imap'} != 2 && $_=~/^imap/) ||
	($config{'imap'} != 1 && $_=~/^dovecot/) ||
	($config{'imap'} && $_=~/^courier/) ||
	
	# other packages
	(!$config{'mailman'} && $_=~/^mailman/) ||
	(!$config{'amavis'} && $_=~/^amavis/) ||
	(!$config{'clamav'} && $_=~/clam/) ||
	(!$config{'spam'} && $_=~/^spam/) ||
	(!$config{'razor'} && $_=~/^razor/) ||
	(!$config{'sasl'} && $_=~/^sasl/) ||
	(!$config{'procmail'} && $_=~/^procmail/));
 }
 return \@rpms;
}

# &get_modules_mail()
# return all name modules for search postinstall scripts
sub get_modules_mail
{
 my @mods;
 
 # mta modules
 if ($config{'mta'} == 1){
 	push (@mods, 'sendmail');
	}
 elsif ($config{'mta'} == 0){
 	push (@mods, 'postfix');
	}
	
 # imap/pop3 modules
 if ($config{'mta'} == 2){
 	push (@mods, 'xinetd');
	}
 elsif ($config{'mta'} == 1){
 	push (@mods, 'dovecot');
	}
 elsif ($config{'mta'} == 0){
 	push (@mods, 'courier-imap');
	}
	
 # other modules 
 push (@mods, 'cyrus-sasl') if $config{'sasl'};
 push (@mods, 'amavis') if $config{'amavis'};
 push (@mods, 'clamav') if $config{'clamav'};
 push (@mods, 'spam') if $config{'spam'};
 push (@mods, 'procmail') if $config{'procmail'};
 
 return @mods;
}

# &get_websites_configure_mail()
# return hash with new options for virtual-server config
sub get_websites_configure_mail
{
 my %rv = ();
 
 # mail feature enable
 $rv{'mail'} = 1;
 
 # mail system type
 $rv{'mail_system'} = $config{'mta'};

 # mailman enable 
 $rv{'mailman'} = 1 if $config{'mailman'};
 
 return %rv;
}

# &uninstall_websites_configure_mail()
# return hash with zero options for virtual-server config
sub uninstall_websites_configure_mail
{
 my %rv = ();
 $rv{'mail'} = 0;
 $rv{'mailman'} = 0;
 return %rv;
}

# &configure_mail()
# call configure functions for all mail services
sub configure_mail
{
 # require librarys from module init, postfix
 &foreign_require('init','init-lib.pl');
 &foreign_require('postfix','postfix-lib.pl');

 if ($config{'sasl'})
 {
  # configure cyrus-sasl
  if (&configured('cyrus-sasl'))
  {
   # configure
   &msg_conftitle($text{'cyrus-sasl'});
   &cyrus_module_configure();
   &msg_done();
  }
  else { &error(&text('index_cyrus-sasl','/cyrus-sasl/index.cgi'));}
 }
 
 if ($config{'spam'})
 { 
  # configure spamassassin
  if (&configured('spam'))
  {
   # configure
   &msg_conftitle($text{'spam'});
   &spam_module_configure();
   &msg_done();
  }
  else {&error(&text('index_spam','/spam/index.cgi'));}
 }
 
 if ($config{'mailman'})
 {
  # configure mailman
  if (&configured('mailman'))
  {
   # configure
   &msg_conftitle($text{'mailman'});
   &mailman_module_configure();
   &msg_done();
  }
  else {&error(&text('index_mailman','/mailman/index.cgi'));}
 }
 
 if ($config{'clamav'})
 {
  # configure clamav
  if (&configured('clamav'))
  {
   # configure
   &msg_conftitle($text{'clamav'});
   &clamav_module_configure();
   &msg_done();
  }
  else {&error(&text('index_clamav','/clamav/index.cgi'));}
 }
 
 if ($config{'amavis'})
 {
   # configure amavis
  if (&configured('amavis'))
  {
   # configure
   &msg_conftitle($text{'amavis'});
   &amavis_module_configure();
   &msg_done();
  }
  else {&error(&text('index_amavis','/amavis/index.cgi'));}
 }
 
 if ($config{'mta'} == 0)
 {
  # configure postfix
  if (&configured('postfix'))
  {
   # configure
   &msg_conftitle($text{'postfix'});
   &postfix_module_configure();
   &msg_done();
  }
  else {&error(&text('index_postfix', '/postfix/index.cgi'));}
 }
 elsif ($config{'mta'} == 1)
 {
  # configure sendmail
  if (&configured('sendmail'))
  {
   # configure
   &msg_conftitle($text{'sendmail'});
   &sendmail_module_configure();
   &msg_done();
  }
  else {&error(&text('index_sendmail', '/sendmail/index.cgi'));}
 }
 
 if ($config{'imap'} == 0)
 { 
  # configure courier
  if (&configured('courier-imap'))
  {
   # configure
   &msg_conftitle($text{'courier'});
   &courier_module_configure();
   &msg_done();
  }
  else {&error(&text('index_courier','/courier-imap/index.cgi'));}
 }
 elsif ($config{'imap'} == 1)
 { 
  # configure dovecot
  if (&configured('dovecot'))
  {
   # configure
   &msg_conftitle($text{'dovecot'});
   &dovecot_module_configure();
   &msg_done();
  }
  else {&error(&text('index_dovecot','/dovecot/index.cgi'));}
 }
 elsif ($config{'imap'} == 2)
 { 
  # configure imap2002d
  if (&configured('xinetd'))
  {
   # configure
   &msg_conftitle($text{'imap'});
   &xinetd_module_configure();
   &msg_done();
  }
  else {&error(&text('index_xinetd','/xinetd/index.cgi'));}
 }
  
 if ($config{'procmail'})
 {
  # configure procmail
  if (&configured('procmail'))
  {
   # configure
   &msg_conftitle($text{'procmail'});
   &procmail_module_configure();
   &msg_done();
  }
  else {&error(&text('index_procmail','/procmail/index.cgi'));}
 }
}

# &uninstall_mail()
# call uninstall functions for all mail services
sub uninstall_mail
{
 # require librarys from module init
 &foreign_require('init','init-lib.pl');

 if ($config{'sasl'})
 {
  # uninstall cyrus-sasl
  &msg_conftitle_uninstall($text{'cyrus-sasl'});
  &cyrus_module_uninstall();
  &msg_done();
 }
 
 if ($config{'spam'})
 {
  # uninstall spamassassin
  &msg_conftitle_uninstall($text{'spam'});
  &spam_module_uninstall();
  &msg_done();
 }

 if ($config{'mailman'})
 {
  # uninstall mailman
  &msg_conftitle_uninstall($text{'mailman'});
  &mailman_module_uninstall();
  &msg_done();
 }
 
 if ($config{'clamav'})
 { 
  # uninstall clamav
  &msg_conftitle_uninstall($text{'clamav'});
  &clamav_module_uninstall();
  &msg_done();
 }

 if ($config{'mta'} == 0)
 {
  # uninstall postfix
  &msg_conftitle_uninstall($text{'postfix'});
  &postfix_module_uninstall();
  &msg_done();
 }
 elsif ($config{'mta'} == 1)
 {
  # uninstall sendmail
  &msg_conftitle_uninstall($text{'sendmail'});
  &sendmail_module_uninstall();
  &msg_done();
 }
 
 if ($config{'amavis'})
 { 
  # uninstall amavis
  &msg_conftitle_uninstall($text{'amavis'});
  &amavis_module_uninstall();
  &msg_done();
 }

 if ($config{'imap'} == 0)
 { 
  # uninstall courier
  &msg_conftitle_uninstall($text{'courier'});
  &courier_module_uninstall();
  &msg_done();
 }
 elsif ($config{'imap'} == 1)
 { 
  # uninstall dovecot
  &msg_conftitle_uninstall($text{'dovecot'});
  &dovecot_module_uninstall();
  &msg_done();
 }
 elsif ($config{'imap'} == 2)
 { 
  # uninstall imap2002
  &msg_conftitle_uninstall($text{'imap'});
  &xinetd_module_uninstall();
  &msg_done();
 }
  
 if ($config{'procmail'})
 { 
  # uninstall procmail
  &msg_conftitle_uninstall($text{'procmail'});
  &procmail_module_uninstall();
  &msg_done();
 }
}

#---------------------------- Authentication services ----------------------

# &cyrus_module_configure()
# configure cyrus-sasl
sub cyrus_module_configure
{
 # require librarys from module cyrus-sasl
 &foreign_require('cyrus-sasl','cyrus-sasl-lib.pl');

 # check service name
 my $service_name='saslauthd';

 # configure MTA
 my $out;
 if ($config{'mta'} == 0)
 {
  # configure Postfix
  # backup cyrus-sasl config files
  my @config = ($cyrus_sasl::config{'sasl2_postfix'},$cyrus_sasl::config{'saslauthd_conf'});
  my $j;
  foreach (@config)
  {
   &write_debug_log(&text('backup','Cyrus-SASL'),'&backup_config',$_);
   $out = &backup_config($_);
   $j++;
   
   # add summary log
   &write_postinstall_log('mail','backup_cyrus'.$j, $_) if $out;
  }
  
  # configure cyrus-sasl config file
  &write_debug_log($text{'sasl_postfix'},'&mailserv_sasl_set',$cyrus_sasl::config{'sasl2_postfix'},'saslauthd');
  &mailserv_sasl_set($cyrus_sasl::config{'sasl2_postfix'},'saslauthd');
 }
 elsif ($config{'mta'} == 1)
 {
  # configure Sendmail
  # backup cyrus-sasl config files
  my @config=($cyrus_sasl::config{'sasl2_sendmail'}, $cyrus_sasl::config{'saslauthd_conf'});
  my $j;
  foreach (@config)
  {
   &write_debug_log(&text('backup','Cyrus-SASL'),'&backup_config',$_);
   $out = &backup_config($_);
   $j++;
	
   # add summary log
   &write_postinstall_log('mail','backup_cyrus'.$j,$_) if $out;
  }
  # configure cyrus-sasl config files
  &write_debug_log($text{'sasl_sendmail'},'&mailserv_sasl_set',$cyrus_sasl::config{'sasl2_sendmail'},'saslauthd');
  &mailserv_sasl_set($cyrus_sasl::config{'sasl2_sendmail'},'saslauthd');
 }
 # configure saslauthd daemon
 &saslauthd_configure();
 
 # get saslauthd service status
 &write_debug_log(&text('status', $service_name),'&cyrus_sasl::status_sasld');
 if (&cyrus_sasl::status_sasld())
 {
  # restart saslauthd
  &write_debug_log(&text('restart',$service_name),'&cyrus_sasl::restart_sasld');
  $out = &cyrus_sasl::restart_sasld();
 }
 else
 {
  # start saslauthd
  &write_debug_log(&text('start',$service_name),'&cyrus_sasl::manage_sasld',1);
  $out = &cyrus_sasl::manage_sasld(1);
 }
 &error($out) if $out;
 
 # enable saslauthd on boot
 &write_debug_log(&text('chk_en',$service_name), '&foreign_call', 'init','enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);

 # add summary log
 &write_postinstall_log('mail','saslauth',1);
}

# &cyrus_module_uninstall()
# uninstall cyrus-sasl
sub cyrus_module_uninstall
{
 # require librarys from module cyrus-sasl
 &foreign_require('cyrus-sasl','cyrus-sasl-lib.pl');

 # check service name
 my $service_name = 'saslauthd';

 # get saslauthd service status
 my $out;
 &write_debug_log(&text('status', $service_name),'&cyrus_sasl::status_sasld');
 if (&cyrus_sasl::status_sasld())
 {
  # stop saslauthd
  &write_debug_log(&text('start',$service_name),'&cyrus_sasl::manage_sasld');
  $out = &cyrus_sasl::manage_sasld();
  &error($out) if $out;
 }
 
 # disable saslauthd from boot
 &write_debug_log(&text('chk_dis',$service_name),'&foreign_call','init','disable_at_boot',$service_name);
 &foreign_call('init','disable_at_boot',$service_name);

 # get array of backup files
 my @config;
 if ($config{'mta'} == 0)
 {
  # Postfix
  @config=($cyrus_sasl::config{'sasl_postfix'},$cyrus_sasl::config{'sasl2_postfix'}, $cyrus_sasl::config{'saslauthd_conf'});
 }
 elsif ($config{'mta'} == 1)
 {
  # Sendmail
  @config=($cyrus_sasl::config{'sasl_sendmail'},$cyrus_sasl::config{'sasl2_sendmail'}, $cyrus_sasl::config{'saslauthd_conf'});
 }
 my $j;
 foreach (@config)
 {
  # restore cyrus-sasl config files
  &write_debug_log(&text('restore','Cyrus-SASL'),'&restore_config',$_);
  $out = &restore_config($_);
  $j++;
   
  # add summary log
  &write_postinstall_log('mail','restore_cyrus'.$j,$_) if $out;
 }
 
 # add summary log
 &write_postinstall_log('mail','saslauth',1);
}

#---------------------------- Spam filtering & Antivirus systems ----------------------

# &spam_module_configure()
# configure spamassassin
sub spam_module_configure
{
 # require librarys from modules spam & procmail
 &foreign_require('spam','spam-lib.pl');

 if ($config{'spam'} && $config{'razor'})
 {
  # configure spamassassin with razor calling by amavis
  # backup local.cf
  &write_debug_log(&text('backup','SA'),'&backup_config',$spam::config{'local_cf'});
  my $out = &backup_config($spam::config{'local_cf'});
  
  # add summary log
  &write_postinstall_log('mail','backup_sa',$spam::config{'local_cf'}) if $out;

  # configure razor for spamassassin
  &write_debug_log($text{'razor_conf'},'&spam::razor_setup');
  my $out = &spam::razor_setup();

  # add summary log
  &write_postinstall_log('mail','razor',1) unless $out;
 }
}

# &spam_module_uninstall()
# uninstall spamassassin
sub spam_module_uninstall
{
 # require librarys from module spamassassin
 &foreign_require('spam','spam-lib.pl');
 
 if ($config{'spam'} && $config{'razor'})
 {
  # restore local.cf
  &write_debug_log(&text('restore','SA'),'&restore_config',$spam::config{'local_cf'});
  my $out = &restore_config($spam::config{'local_cf'});

  # add summary log
  &write_postinstall_log('mail','restore_sa',$spam::config{'local_cf'}) if $out;
  &write_postinstall_log('mail','razor',1);
 }
}

# &clamav_module_configure()
# configure clamav services
sub clamav_module_configure
{
 # require librarys from module clamav
 &foreign_require('clamav','clamav-lib.pl');

 # check services name
 my $service_name = 'clamd';
 
 # backup clamav.conf
 &write_debug_log(&text('backup','ClamAV'),'&backup_config',$clamav::config{'clamav_conf'});
 my $out = &backup_config($clamav::config{'clamav_conf'});
 
 # add summary log
 &write_postinstall_log('mail','backup_clamav',$clamav::config{'clamav_conf'}) if $out;
 
 # check version
 my $ver = &clamav::get_version_number_clamav();
 my $new_config = &clamav::check_version_support($ver, '0.90.2');

 # changed clamav configuration
 my $new_config = { 'LocalSocket'=> '/var/run/clamav/clamd.sock',
						'TCPAddr' => '#',
						'TCPSocket' => '#',
		 				'TemporaryDirectory' => '/var/tmp',
		 				'PidFile' => '/var/run/clamav/clamd.pid',
						'LogFile' => '/var/log/clamav/clamd.log',
						};
 if ($new_config){
	# version >= 0.90.2
	$new_config->{'LogVerbose'} = $new_config->{'FixStaleSocket'} = $new_config->{'DetectBrokenExecutables'} = $new_config->{'ScanMail'} = $new_config->{'ScanArchive'} = $new_config->{'ScanOLE2'} = $new_config->{'ScanHTML'} = $new_config->{'ScanPDF'} = 'yes';
	}
 else {
	# old versions
	$new_config->{'LogVerbose'} = $new_config->{'FixStaleSocket'} = $new_config->{'DetectBrokenExecutables'} = $new_config->{'ScanMail'} = $new_config->{'ScanArchive'} = $new_config->{'ScanOLE2'} = $new_config->{'ScanHTML'} = $new_config->{'ScanRAR'} = undef;
	}
						
 # advanced ClamAV services 
 if ($config{'clamav'} == 1)
 {
  # running clamav from amavis
  $new_config->{'User'} = 'amavis';
  $new_config->{'DatabaseDirectory'} = '/var/lib/clamav';
  
  # freshclam service configure
  &freshclam_module_configure();
 }
 elsif ($config{'clamav'} == 2)
 {
  # running clamav from clamsmtpd
  $new_config->{'User'} = 'clamav';
  $new_config->{'DatabaseDirectory'} = '/var/clamav';
  
  # clamsmtpd service configure
  &clamsmtpd_module_configure();
 }
 
 # modify clamav configuration
 &write_debug_log(&text('modify','ClamAV'),'&clamav_save_directive', $new_config, $clamav::config{'clamav_conf'});
 &clamav_save_directive($new_config, $clamav::config{'clamav_conf'});

 # get service clamd status
 &write_debug_log(&text('status',$service_name),'&clamav::status_clamav_daemon');
 if (&clamav::status_clamav_daemon())
 {
  # restart clamd
  &write_debug_log(&text('restart',$service_name),'&clamav::restart_clamav_daemon');
  &clamav::restart_clamav_daemon();
 }
 else
 {
  # start clamd
  &write_debug_log(&text('start',$service_name),'&clamav::start_clamav_daemon');
  &clamav::start_clamav_daemon();
 }
 
 # enable clamd on boot
 &write_debug_log(&text('chk_en',$service_name), '&foreign_call', 'init','enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);

 # add summary log
 &write_postinstall_log('mail','clamav',1);
}

# &freshclam_module_configure()
# configure freshclam service
sub freshclam_module_configure
{
 my $service_name = 'freshclam';
 
 # start or restart freshclamav
 &write_debug_log(&text('status', $service_name),'&clamav::status_freshclamav_daemon');
 if (&clamav::status_freshclamav_daemon())
 {
  # restart freshclam
  &write_debug_log(&text('restart', $service_name),'&clamav::restart_freshclamav_daemon');
  &clamav::restart_freshclamav_daemon();
 }
 else
 {
  # start freshclam
  &write_debug_log(&text('start', $service_name),'&clamav::start_freshclamav_daemon');
  &clamav::start_freshclamav_daemon();
 }
 
 # enable freshclamav on boot
 &write_debug_log(&text('chk_en',$service_name), '&foreign_call', 'init','enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);
  
 # add summary log
 &write_postinstall_log('mail', $service_name, 1);
}

# &clamsmtpd_module_configure()
# configure clamsmtpd service
sub clamsmtpd_module_configure
{
 my $service_name = 'clamsmtpd';
 
 # backup clamavsmtpd.conf
 &write_debug_log(&text('backup','ClamAV'),'&backup_config',$clamav::config{'clamsmtpd_conf'});
 $out = &backup_config($clamav::config{'clamsmtpd_conf'});
  
 # add summary log
 &write_postinstall_log('mail','backup_clamsmtpd',$clamav::config{'clamsmtpd_conf'}) if $out;
 
 # modify clamsmtpd configuration
 my $new_config = { 'OutAddress:'=> '10026',
 						'Listen:'=> '0.0.0.0:10025',
						'ClamAddress:' => '/var/run/clamav/clamd.sock',
						'Quarantine:' => 'on',
						'User:' => 'clamav' };
 &write_debug_log(&text('modify','ClamSMTPd'),'&clamav_save_directive', $new_config, $clamav::config{'clamsmtpd_conf'});
 &clamav_save_directive($new_config, $clamav::config{'clamsmtpd_conf'});
 
 # start or restart clamsmtpd
 &write_debug_log(&text('status', $service_name),'&clamav::status_clamsmtpd_daemon');
 if (&clamav::status_clamsmtpd_daemon())
 {
  # restart clamsmtpd
  &write_debug_log(&text('restart', $service_name),'&clamav::restart_clamsmtpd_daemon');
  &clamav::restart_clamsmtpd_daemon();
 }
 else
 {
  # start clamsmtpd
  &write_debug_log(&text('start', $service_name),'&clamav::start_clamsmtpd_daemon');
  &clamav::start_clamsmtpd_daemon();
 }
 
 # enable freshclamav on boot
 &write_debug_log(&text('chk_en',$service_name), '&foreign_call', 'init','enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);
  
 # add summary log
 &write_postinstall_log('mail', $service_name, 1);
}

# &clamav_module_uninstall()
# uninstall clamav services
sub clamav_module_uninstall
{
 # require librarys from module clamav
 &foreign_require('clamav','clamav-lib.pl');

 # check service name
 my $service_name = 'clamd';

 # get service clamd status
 &write_debug_log(&text('status', $service_name),'&clamav::status_clamav_daemon');
 unless (&clamav::status_clamav_daemon())
 {
  # stop clamd
  &write_debug_log(&text('stop', $service_name),'&clamav::stop_clamav_daemon');
  &clamav::stop_clamav_daemon();
 }
 
 # disable clamd from boot
 &write_debug_log(&text('chk_dis', $service_name),'&foreign_call','init','disable_at_boot', $service_name);
 &foreign_call('init','disable_at_boot',$service_name);

 # add summary log
 &write_postinstall_log('mail','clamav',1);

 # restore clamd.conf
 &write_debug_log(&text('restore','ClamAV'),'&restore_config',$clamav::config{'clamav_conf'});
 my $out = &restore_config($clamav::config{'clamav_conf'});

 # add summary log
 &write_postinstall_log('mail','restore_clamav', $clamav::config{'clamav_conf'}) if $out;
 
 # advanced services
 if ($config{'clamav'} == 1)
 {
  # freshclam service configure
  &freshclam_module_uninstall();
 }
 elsif ($config{'clamav'} == 2)
 {
  # clamsmtpd service configure
  &clamsmtpd_module_uninstall();
 }
}

# &freshclam_module_uninstall()
# uninstall freshclam service
sub freshclam_module_uninstall
{
 # check service name
 my $service_name = 'freshclam';
 
 # get service freshclam status
 &write_debug_log(&text('status',$service_name),'&clamav::status_freshclamav_daemon');
 unless (&clamav::status_freshclamav_daemon())
 {
  # stop freshclam
  &write_debug_log(&text('stop',$service_name),'&clamav::stop_freshclamav_daemon');
  &clamav::stop_freshclamav_daemon;
 }
  
 # disable freshclam from boot
 &write_debug_log(&text('chk_dis',$service_name),'&foreign_call','init','disable_at_boot',$service_name);
 &foreign_call('init', 'disable_at_boot', $service_name);
 
 # add summary log
 &write_postinstall_log('mail', $service_name, 1);
}

# &clamsmtpd_module_uninstall()
# uninstall clamsmtpd service
sub clamsmtpd_module_uninstall
{
 # check services name
 my $service_name = 'clamsmtpd';
 
 # get service clamsmtpd status
 &write_debug_log(&text('status', $service_name),'&clamav::status_clamsmtpd_daemon');
 unless (&clamav::status_clamsmtpd_daemon())
 {
  # stop clamsmtpd
  &write_debug_log(&text('stop', $service_name),'&clamav::stop_clamsmtpd_daemon');
  &clamav::stop_clamsmtpd_daemon;
 }
 
 # restore clamsmtpd.conf
 &write_debug_log(&text('restore','ClamAV'),'&restore_config',$clamav::config{'clamsmtpd_conf'});
 my $out = &restore_config($clamav::config{'clamsmtpd_conf'});

 # add summary log
 &write_postinstall_log('mail','restore_clamsmtpd', $clamav::config{'clamsmtpd_conf'}) if $out;
  
 # disable clamsmtpd from boot
 &write_debug_log(&text('chk_dis', $service_name),'&foreign_call','init','disable_at_boot',$service_name1);
 &foreign_call('init','disable_at_boot', $service_name);
 
 # add summary log
 &write_postinstall_log('mail', $service_name, 1);
}

#---------------------------- Filtering system ----------------------

# &amavis_module_configure()
# configure amavisd-new
sub amavis_module_configure
{
 # require librarys from module amavis
 &foreign_require('amavis', 'amavis-lib.pl');

 # check service name
 my $service_name = 'amavisd';

 # backup amavisd.conf
 &write_debug_log(&text('backup','Amavis'),'&backup_config',$amavis::config{'amavisd_conf'});
 my $out = &backup_config($amavis::config{'amavisd_conf'});
 
 # add summary log
 &write_postinstall_log('mail','backup_amavis',$amavis::config{'amavisd_conf'}) if $out;

 # get $mydomain option from postfix
 &write_debug_log($text{'postfix_get'},'&postfix::get_current_value','myhostname');
 my $mydomain = &postfix::get_current_value('myhostname');

 # modify amavis configuration
 my $new_config = {'$mydomain'=>"\'$mydomain\';",
 						'$daemon_user'=>'\'amavis\';',
						'$daemon_group'=>'\'amavis\';',
						'$forward_method'=>'\'smtp:[127.0.0.1]:10025\';',
						'$notify_method'=>'$forward_method;',
						'$inet_socket_port'=>'10024;',
						'$inet_socket_bind'=>'\'127.0.0.1\';',
						'$child_timeout'=>'8*60;',
						'@bypass_spam_checks_maps'=>'(1);',
						#'$final_spam_destiny'=>'D_PASS;',
						'$warnvirussender' => '1;',
						#'$warnspamsender' =>'0;',
						'$warnbannedsender' => '0;',
						'$warnbadhsender' => '0;',
						'$virus_admin'=>'"root\@$mydomain";',
						#'$spam_admin'=>'undef;',
						'$warnvirusrecip' => '0;',
						'$warnbannedrecip' => '0;',
						'$X_HEADER_LINE'=>'"by $myproduct_name using ClamAv at $mydomain";',
						#'$sa_local_tests_only'=>'0;',
						'$QUARANTINEDIR'=>"\'$amavis::config{'home_dir'}/virusmails\';"};
 &write_debug_log(&text('modify','Amavis'),'&amavis::save_config_file',$new_config);
 &amavis::save_config_file($new_config);

 # get service amavisd status
 &write_debug_log(&text('status',$service_name),'&amavis::status_amavisd');
 if (&amavis::status_amavisd)
 {
  # restart amavisd
  &write_debug_log(&text('restart',$service_name),'&amavis::restart_amavisd');
  &amavis::restart_amavisd();
 }
 else
 {
  # start amavisd
  &write_debug_log(&text('start',$service_name),'&amavis::start_amavisd');
  &amavis::start_amavisd();
 }
 
 # enable amavisd on boot
 &write_debug_log(&text('chk_en',$service_name), '&foreign_call', 'init','enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);

 # add summary log
 &write_postinstall_log('mail','amavis',1);
}

# &amavis_module_uninstall()
# uninstall amavis
sub amavis_module_uninstall
{
 # require librarys from module amavis
 &foreign_require('amavis', 'amavis-lib.pl');

 # check service name
 my $service_name = 'amavisd';

 # get amavisd service status
 &write_debug_log(&text('status',$service_name),'&amavis::status_amavisd');
 if (&amavis::status_amavisd())
 {
  # stop amavisd
  &write_debug_log(&text('stop',$service_name),'&amavis::stop_amavisd');
  &amavis::stop_amavisd();
 }
 
 # disable amavisd from boot
 &write_debug_log(&text('chk_dis',$service_name),'&foreign_call','init','disable_at_boot', $service_name);
 &foreign_call('init','disable_at_boot',$service_name);

 # restore amavisd.conf
 &write_debug_log(&text('restore','Amavis'),'&restore_config',$amavis::config{'amavisd_conf'});
 my $out = &restore_config($amavis::config{'amavisd_conf'});

 # add summary log
 &write_postinstall_log('mail','restore_amavis',$amavis::config{'amavisd_conf'}) if $out;
 &write_postinstall_log('mail','amavis',1);
}

# &procmail_module_configure()
# configure procmail
sub procmail_module_configure
{
 # require librarys from module procmail
 &foreign_require('procmail', 'procmail-lib.pl');

 # set mboxes format
 my ($action, $file, $mailbox) = &has_command($spam::config{'spamassassin'});
 if ($config{'imap'} == 0)
 {
  # for courier-imap
  $file = "\$HOME/spam/";
  $mailbox = "\$HOME/Maildir/";
  #$action= &has_command($spam::config{'spamassassin'})." --local";
 }
 else
 {
  # for imap2002, dovecot
  $file = "\$HOME/spam";
  $mailbox = "\$MAIL";
 }
 # environment variable to drop privileges
 my @recipe;
 push (@recipe,{ 'name' => 'DROPPRIVS','value' => 'yes' });
 if ($spam::config{'call_spam'} || !$spam::module_info{'usermin'})
 {
  # recipe to call spamassassin
  push (@recipe,{'flags' => [ 'f', 'w' ],'conds' => [ ],'type' => '|','action' => $action});
 }
 if ($file)
 {
  # recipe to perform actual filtering to a file
  push (@recipe,{'flags' => [ ],'conds' => [ [ '', '^X-Spam-Status: Yes' ] ],'action' => $file});
 }
 # recipe to perform message to a file
 push (@recipe,{'flags' => [ ],'conds' => [ ],'action' => $mailbox});
 
 # save new procmail directives
 &write_debug_log(&text('modify','Procmail'),'&procmail_save_directive',\@recipe);
 &procmail_save_directive(\@recipe);

 # add summary log
 &write_postinstall_log('mail','procmail',1);
}

# &procmail_module_uninstall()
# uninstall procmail
sub procmail_module_uninstall
{
 # require librarys from module procmail
 &foreign_require('procmail', 'procmail-lib.pl');

 # get procmail config file
 my @pmrc = &spam::get_procmailrc();
 my $pmrc = $pmrc[$#pmrc];

 # restore procmail.rc
 &write_debug_log(&text('restore','Procmail'),'&restore_config',$pmrc);
 my $out = &restore_config($pmrc);
 
 # add summary log
 &write_postinstall_log('mail','restore_procmail',$pmrc) if $out;
 &write_postinstall_log('mail','procmail',1);
}

#---------------------------- Mail lists system ----------------------

# &mailman_module_configure()
# configure mailman
sub mailman_module_configure
{
 # require library from module mailman
 &foreign_require('mailman', 'mailman-lib.pl');
 
 # service name
 my $service_name = 'mailman';
 
 # get hostname
 my $hostname = &get_system_hostname();
 
  # get random password
 my @rand=("A".."Z","a".."z",0..9);
 my $passwd = join("",@rand[map {rand @rand}(1..8)]);
  
 # mailman apache support setup
 &write_debug_log($text{'apache_mailman'},'&mailman_apache_setup');
 my $out = &mailman_apache_setup($hostname);
 
 unless ($out)
 {
  # set site password
  $out = &mailman::set_site_password($passwd);
  &write_postinstall_log('mail','site_passwd', $passwd) unless $out;
 }
   
 # get MTA
 my $mta = ($config{'mta'} == 0) ? 'Postfix' : 'Sendmail';
 
 # backup mailman config file
 my $conf_file = $mailman::config{'mailman_conf'};
 &write_debug_log(&text('backup','Mailman'),'&backup_config', $conf_file);
 $out = &backup_config($conf_file);
 
 # add summary log
 &write_postinstall_log('mail','backup_mailman', $conf_file) if $out;
 
 # modify mailman config file
 &mailman::remove_mailman_str(['^DEFAULT_HOST_NAME\b', '^DEFAULT_URL\b']);
 my $conf = {  'DEFAULT_URL_HOST' => "'www.".$hostname."'", 'DEFAULT_EMAIL_HOST' => "'".$hostname."'" };
 $conf->{'MTA'} = "'".$mta."'" if $mta;
 &write_debug_log(&text('modify', 'Mailman'),'&mailman::save_new_options', $conf);
 &mailman::save_new_options($conf);
  
 # get mailman lists
 &write_debug_log($text{'mailman_list'},'&mailman::list_lists');
 my @out = &mailman::list_lists();
 my $new = grep {$_->{'list'} eq 'mailman'} @out;
 unless ($new)
 {
  # add new list - 'mailman'
  $passwd = join("",@rand[map {rand @rand}(1..8)]);
  my $list = { 'name' => 'mailman',
					'email' => "root\@$hostname",
					'pass' => $passwd
				 };
  &write_debug_log($text{'mailman_pass'},'&mailman::create_list', $list);
  $out = &mailman::create_list($list);
  &error($out) if $out;
  
  # add summary log
  &write_postinstall_log('mail','passwd', $passwd);
 }
 else 
 {
  # regenerate mailman aliases
  &write_debug_log($text{'mailman_aliases'},'&mailman::regenerate_aliases');
  $out = &mailman::regenerate_aliases();
  &error($out) if $out;
 }
 
 # get mailman service status
 &write_debug_log(&text('status',$service_name),'&mailman::is_mailman_running');
 if (&mailman::is_mailman_running)
 {
  # restart mailman
  &write_debug_log(&text('restart',$service_name),'&mailman::restart_mailman');
  $out = &mailman::restart_mailman();
 }
 else
 {
  # start mailman
  &write_debug_log(&text('start',$service_name),'&mailman::start_mailman');
  $out = &mailman::start_mailman();
 }
 &error($out) if $out;
 
 # enable mailman on boot
 &write_debug_log(&text('chk_en',$service_name), '&foreign_call', 'init','enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);

 # add summary log
 &write_postinstall_log('mail','mailman',1);
}

# &mailman_module_uninstall()
# uninstall mailman
sub mailman_module_uninstall
{
 # require library from module mailman
 &foreign_require('mailman', 'mailman-lib.pl');
 
 # service name
 my $service_name = 'mailman';
 
 # get mailman lists
 my @lists = &mailman::list_lists();
 foreach (@lists)
 {
  # delete all lists
  &mailman::delete_list($_, 1);
 }
 
 # regenerate aliases
 &mailman::regenerate_aliases();
  
 my $out;
 &write_debug_log(&text('status',$service_name),'&mailman::is_mailman_running');
 if (&mailman::is_mailman_running)
 {
  # stop mailman
  &write_debug_log(&text('stop',$service_name),'&mailman::stop_mailman');
  $out = &mailman::stop_mailman();
  &error($out) if $out;
 }
 
 # disable mailman from boot
 &write_debug_log(&text('chk_dis',$service_name),'&foreign_call','init','disable_at_boot', $service_name);
 &foreign_call('init', 'disable_at_boot', $service_name);
 
 # restore mailman config file
 my $conf_file = $mailman::config{'mailman_conf'};
 &write_debug_log(&text('restore','Mailman'),'&restore_config', $conf_file);
 $out = &restore_config($conf_file);
 
 # add summary log
 &write_postinstall_log('mail','restore_mailman', $conf_file) if $out;

 # desable apache mailman support
 &mailman_apache_uninstall();
 
 # add summary log
 &write_postinstall_log('mail','mailman',1);
}

#------------------------------------- MTA ------------------------------------

# &postfix_module_configure()
# configure postfix
sub postfix_module_configure
{
 # require librarys from module virtual-server
 &foreign_require('virtual-server','virtual-server-lib.pl');
 
 # check services names
 my $service_name = 'postfix';
 my $service_name1 = 'sendmail';
 my $service_name2 = 'exim';

 #--- set Postfix as main system MTA ---
 
 my $out;
 if (&configured('sendmail'))
 {
  # uninstall sendmail
  # require librarys from module sendmail
  &foreign_require('sendmail','sendmail-lib.pl');

  # get sendmail service status
  &write_debug_log(&text('status',$service_name1),'&sendmail::is_sendmail_running');
  if (&sendmail::is_sendmail_running())
  {
   # stop sendmail
   &write_debug_log(&text('stop',$service_name1),'&sendmail::stop_sendmail');
   $out = &sendmail::stop_sendmail();
   &error($out) if $out;
  }
  
  # disable sendmail from boot
  &write_debug_log(&text('chk_dis',$service_name1),'&foreign_call','init','disable_at_boot', $service_name1);
  &foreign_call('init','disable_at_boot',$service_name1);
 }
 
 # uninstall exim
 # get sendmail service status
 &write_debug_log(&text('status',$service_name2),'&is_exim_running');
 if (&is_exim_running())
 {
  # stop exim
  &write_debug_log(&text('stop',$service_name2),'&stop_exim');
  $out = &stop_exim();
  # &error($out) if $out;
  
  # disable exim from boot
  &write_debug_log(&text('chk_dis',$service_name2),'&foreign_call','init','disable_at_boot', $service_name2);
  &foreign_call('init','disable_at_boot',$service_name2);
 }
 
 # check smtp link
 &switch_smtp_link('postfix');
 
 #--- backup postfix files ---
 
 # backup main.cf
 &write_debug_log(&text('backup','Postfix'),'&backup_config',$postfix::config{'postfix_config_file'});
 $out = &backup_config($postfix::config{'postfix_config_file'});
  
 # add summary log
 &write_postinstall_log('mail','backup_postfix1',$postfix::config{'postfix_config_file'}) if $out;
 
 if ($config{'amavis'} || $config{'clamav'})
 {
  # backup main.cf
  &write_debug_log(&text('backup','Postfix'),'&backup_config',$postfix::config{'postfix_master_file'});
  $out = &backup_config($postfix::config{'postfix_master_file'});
  
  # add summary log
  &write_postinstall_log('mail','backup_postfix2',$postfix::config{'postfix_master_file'}) if $out; 
 }
  
#--- configure master.cf ---

 my @new_conf;
 if ($config{'amavis'})
 {
  # for amavis
  push (@new_conf, {'service'=>'smtp-amavis',
							'type'=>'unix',
							'private'=>'-',
							'unpriv'=>'-', 
							'chroot'=>'y',
							'wakeup'=>'-',
							'maxproc'=>'2',
							'command'=>'smtp',
							'args'=>["\t-o smtp_data_done_timeout=1200",
										"\t-o disable_dns_lookups=yes"]});
   push (@new_conf, {'service'=>'127.0.0.1:10025',
							'type'=>'inet',
							'private'=>'n',
							'unpriv'=>'-', 
							'chroot'=>'n',
							'wakeup'=>'-',
							'maxproc'=>'-',
							'command'=>'smtpd',
							'args'=> ["\t-o content_filter=",
										"\t-o local_recipient_maps=",
										"\t-o helo_rectrictions=",
										"\t-o smtpd_client_restrictions=",
										"\t-o smtrp_sender_restrictions=permit_mynetworks,permit_sasl_authenticated,reject",
										"\t-o mynetworks=127.0.0.0/8"]});
 }
 elsif ($config{'clamav'} == 2)
 {
  # for clamsmtpd
  push (@new_conf, {'service'=>'scan',
							'type'=>'unix',
							'private'=>'-',
							'unpriv'=>'-', 
							'chroot'=>'n', 
							'wakeup'=>'-', 
							'maxproc'=>'16', 
							'command'=>'smtp', 
							'args'=>["\t-o smtp_send_xforward_command=yes"]});
  push (@new_conf, {'service'=>'127.0.0.1:10026',
							'type'=>'inet',
							'private'=>'n',
							'unpriv'=>'-', 
							'chroot'=>'n',
							'wakeup'=>'-',
							'maxproc'=>'16',
							'command'=>'smtpd',
							'args'=> ["\t-o content_filter=",
										"\t-o receive_override_options=no_unknown_recipient_checks,no_header_body_checks",
										"\t-o smtpd_helo_restrictions=",
										"\t-o smtpd_client_restrictions=",
										"\t-o smtrp_sender_restrictions=",
										"\t-o smtpd_recipient_restrictions=permit_mynetworks,permit_sasl_authenticated,reject",
										"\t-o mynetworks_style=host",
										"\t-o smtpd_authorized_xforward_hosts=127.0.0.0/8"]});
 }
 if (@new_conf)
 {
  &write_debug_log(&text('modify','Postfix master'),'&postfix_master_save',\@new_conf);
  &postfix_master_save(\@new_conf);
 }
  
 #--- configure main.cf ---
 
 # get main.cf value
 &write_debug_log($text{'postfix_get'},'&postfix::get_current_value','config_directory');
 my $dir = &postfix::get_current_value('config_directory');
 
 # configure base options
 my $hostname = &get_system_hostname();
 my $origin = &postfix::get_default_value('myorigin');
 my $dest = &postfix::get_default_value('mydestination');
 my $net = &postfix::get_default_value('mynetworks');
 
 # configure virtual maps
 my $virtual = "hash:$dir/virtual";
 $virtual .= ", hash:$mailman::config{'mailman_aliases'}/virtual-mailman" if $config{'mailman'}; 
 
 # configure alias maps
 my $aliases = "hash:/etc/aliases";
 $aliases .= ", hash:$mailman::config{'mailman_aliases'}/aliases" if $config{'mailman'};
  
 # base configuration
 my $options = { 'myhostname_def' => $hostname,
 					'mydomain_def' => $hostname,
 					'myorigin_def' => $origin,
					'mydestination_def' => $dest,
					'mynetworks_def' => $net,
 					"$postfix::virtual_maps\_def" => $virtual,
 					'alias_maps_def' => $aliases,
					'alias_database_def' => $aliases,
					'inet_interfaces_def' => 'all',
					'home_mailbox_def' => '',
					'mailbox_size_limit_def' => 11240000,
					'message_size_limit_def' => 10240000,
					'smtpd_recipient_restrictions_def' => 'permit_mynetworks reject_unauth_destination'
					};
					
 if ($config{'sasl'}){
 	# configure sasl
	$options->{'smtpd_sasl_auth_enable_def'} = 'yes';
	$options->{'smtpd_sasl_security_options_def'} = 'noanonymous';
	$options->{'broken_sasl_auth_clients_def'} = 'yes';
	$options->{'smtpd_recipient_restrictions_def'} = 'permit_mynetworks permit_sasl_authenticated reject_unauth_destination';
	}
					
 # configure home mailboxes
 if ($config{'imap'} == 0) {
 	# maildir
	$options->{'home_mailbox_def'} = 'Maildir/';
	}

 # configure filtering
 if ($config{'amavis'}){
 	# configure amavis filter
	$options->{'content_filter_def'} = 'smtp-amavis:[127.0.0.1]:10024';
	}
 elsif ($config{'clamav'} == 2){
 	# configure clamsmtpd filter
	$options->{'content_filter_def'} = 'scan:127.0.0.1:10025';
	$options->{'receive_override_options'} = 'no_address_mappings';
	}
 
 if ($config{'procmail'}){
 	# configure procmail command
	$options->{'mailbox_command_def'} = '/usr/bin/procmail -t';
	}
	
  if ($config{'mailman'}){
 	# configure mailman specific options
	$options->{'owner_request_special_def'} = 'no';
	$options->{'recipient_delimiter_def'} = '+';
	}
 &write_debug_log(&text('modify','Postfix main'),'&postfix_main_save',$options);
 &postfix_main_save($options);

 #--- system configuration ---

 # add new system user 'postmaster'
 &write_debug_log($text{'create_user'},'&user_add','postmaster');
 &user_add('postmaster');
 
 # add alias from 'root' to user 'postmaster'
 &write_debug_log($text{'aliases_create'},'&postfix_alias_root',$aliases,'postmaster');
 &postfix_alias_root($aliases,'postmaster');

 # add summary log
 &write_postinstall_log('mail','mail_root_name','postmaster');
  
 # regenerate aliases
 &write_debug_log($text{'postfix_aliases_reg'},'&postfix::regenerate_aliases');
 &postfix::regenerate_aliases();
 
 # regenerate virtual map
 &write_debug_log($text{'postfix_virtual_reg'},'&postfix::regenerate_virtual_table');
 &postfix::regenerate_virtual_table();

 # get postfix service status
 &write_debug_log(&text('status',$service_name),'&postfix::is_postfix_running');
 if (&postfix::is_postfix_running())
 {
  # restart postfix
  &write_debug_log(&text('stop',$service_name),'&postfix::stop_postfix');
  $out = &postfix::stop_postfix();
  &error($out) if $out;
  sleep (3);
  &write_debug_log(&text('start',$service_name),'&postfix::start_postfix');
  $out = &postfix::start_postfix();
 }
 else
 {
  # start postfix
  &write_debug_log(&text('start',$service_name),'&postfix::start_postfix');
  $out = &postfix::start_postfix();
 }
 &error($out) if $out;
 
 # enable postfix on boot
 &write_debug_log(&text('chk_en',$service_name), '&foreign_call', 'init','enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);

 #--- configure mailboxes module ---
 
 # require librarys from module mailboxes
 &foreign_require('mailboxes','mailboxes-lib.pl');
 
 # set mail system type
 $mailboxes::config{'mail_system'} = 0;
 
 # save mailboxes config
 &write_debug_log(&text('modify','Mailboxes'), '&save_module_config', \%mailboxes::config, 'mailboxes');
 &save_module_config(\%mailboxes::config,'mailboxes');
 
 #--- configure endusers mailbox module ---
 
 # require librarys from module enduser_cf
 &foreign_require('enduser_cf','enduser_cf-lib.pl');
 
 # get enduser_cf config
 &read_file("$enduser_cf::config{'usermin_dir'}/mailbox/config", \%mconfig);
 
 # set virtual domain map options
 $mconfig{'from_map'}="$dir/virtual";
 $mconfig{'from_format'} = 1;
 
 if ($config{'imap'} == 0){
 	# set Maildir format
	$mconfig{'mail_system'} = 1;
	$mconfig{'mailbox_dir'}='Maildir';
	}
 else {
 	# set mailbox format
	$mconfig{'mail_system'} = 0;
	}
 
 # save enduser_cf config
 &write_debug_log(&text('modify','Enduser mailbox'),'&write_file', "$enduser_cf::config{'usermin_dir'}/mailbox/config",\%mconfig);
 &write_file("$enduser_cf::config{'usermin_dir'}/mailbox/config", \%mconfig);

 # add summary log
 &write_postinstall_log('mail','postfix',1);
}

# &postfix_module_uninstall()
# uninstall postfix
sub postfix_module_uninstall
{
 # require librarys from module postfix
 &foreign_require('postfix','postfix-lib.pl');

 # check service name
 my $service_name = 'postfix';
 
 #--- service configuration ---
 
 # regenerate postfix virtual map
 &postfix::regenerate_virtual_table();

 # get postfix service status
 &write_debug_log(&text('status',$service_name),'&postfix::is_postfix_running');
 my $out;
 if (&postfix::is_postfix_running())
 {
  # stop postfix
  &write_debug_log(&text('stop',$service_name),'&postfix::stop_postfix');
  $out = &postfix::stop_postfix();
  &error($out) if $out;
 }
 
 # disable postfix from boot
 &write_debug_log(&text('chk_dis',$service_name),'&foreign_call','init','disable_at_boot', $service_name);
 &foreign_call('init','disable_at_boot',$service_name);

 #--- restore config files ---
 
 # restore main.cf
 &write_debug_log(&text('restore','Postfix'),'&restore_config',$postfix::config{'postfix_config_file'});
 $out = &restore_config($postfix::config{'postfix_config_file'});
  
 # add summary log
 &write_postinstall_log('mail','restore_postfix1',$postfix::config{'postfix_config_file'}) if $out;
 
 if ($config{'amavis'} || $config{'clamav'})
 {
  # restore master.cf
  &write_debug_log(&text('restore','Postfix'),'&restore_config',$postfix::config{'postfix_master_file'});
  $out = &restore_config($postfix::config{'postfix_master_file'});
  
  # add summary log
  &write_postinstall_log('mail','restore_postfix2',$postfix::config{'postfix_master_file'}) if $out;
 }
 
 # add summary log
 &write_postinstall_log('mail','postfix',1);
}

# &sendmail_module_configure()
# configure sendmail
sub sendmail_module_configure
{
 # require librarys from module sendmail
 &foreign_require('sendmail','sendmail-lib.pl');
 &foreign_require('sendmail','features-lib.pl');

 # check service name
 my $service_name='sendmail';
 my $service_name1='postfix';
 my $service_name2='exim';

 #--- set Sendmail as main system MTA ---
  
 my $out;
 if (&configured('postfix'))
 {
  # uninstall postfix
  # get service postfix status
  &write_debug_log(&text('status',$service_name1),'&postfix::is_postfix_running');
  if (&postfix::is_postfix_running())
  {
   # stop postfix
   &write_debug_log(&text('stop',$service_name1),'&postfix::stop_postfix');
   $out = &postfix::stop_postfix();
   &error($out) if $out;
  }
  
  # disable postfix from boot
  &write_debug_log($text{'chk_dis'},'&foreign_call','init','disable_at_boot','postfix');
  &foreign_call('init','disable_at_boot','postfix');
 }
 
 # uninstall exim
 # get sendmail service status
 &write_debug_log(&text('status',$service_name2),'&is_exim_running');
 if (&is_exim_running())
 {
  # stop exim
  &write_debug_log(&text('stop',$service_name2),'&stop_exim');
  $out = &stop_exim();
  # &error($out) if $out;
  
  # disable exim from boot
  &write_debug_log(&text('chk_dis',$service_name2),'&foreign_call','init','disable_at_boot', $service_name2);
  &foreign_call('init','disable_at_boot',$service_name2);
 }
 
 # check smtp link
 &switch_smtp_link('sendmail');
 
 #--- backup sendmail files ---
 
 # backup sendmail.mc
 &write_debug_log(&text('backup','Sendmail'),'&backup_config',$sendmail::config{'sendmail_mc'});
 my $out = &backup_config($sendmail::config{'sendmail_mc'});
 
 # add summary log
 &write_postinstall_log('mail','backup_sendmail',$sendmail::config{'sendmail_mc'}) if $out;

 #--- configure sendmail files ---
 
 # modify sendmail.mc
 my @new_config=( {'type'=>0,
 						 'text'=>'DAEMON_OPTIONS(`Port=smtp, Name=MTA\')',
						 'name'=>'DAEMON_OPTIONS'} );
 if ($config{'sasl'}){
 	# sasl directives
	push (@new_config, {'type'=>0,
							'text'=>'TRUST_AUTH_MECH(`LOGIN PLAIN\')',
							'name'=>'TRUST_AUTH_MECH'},
 							 {'type'=>2,
							'text'=>'define(`confAUTH_MECHANISMS\', `LOGIN PLAIN\')',
							'name'=>'confAUTH_MECHANISMS', 
							'value'=>['confAUTH_MECHANISMS','LOGIN PLAIN']});
	}
 if ($config{'procmail'}){
 	# procmail directives
 	push (@new_config, {'type'=>1,
							'text'=>'FEATURE(`local_procmail\',`\',`procmail -t -Y -a \$h -d \$u\')',
							'name'=>'local_procmail', 
							'value'=>['','procmail -t -Y -a $h -d $u']} ,
 							 {'type'=>4,
							'mailer'=>'procmail',
							'text'=>'MAILER(`procmail\')'});
	}
 &write_debug_log(&text('modify','Sendmail'),'&sendmail_save_directive',$new_config);
 &sendmail_save_directive(\@new_config);

 # rebuild sendmail.cf
 &lock_file($sendmail::config{'sendmail_cf'});
 my $cmd = "cd $sendmail::config{'sendmail_features'}/m4; m4 $sendmail::config{'sendmail_features'}/m4/cf.m4 $sendmail::config{'sendmail_mc'}";
 &write_debug_log($text{'sendmail_rebuild'},'system',"$cmd 2>/dev/null >$sendmail::config{'sendmail_cf'} </dev/null 2>&1");
 system("$cmd 2>/dev/null >$sendmail::config{'sendmail_cf'} </dev/null 2>&1");
 &unlock_file($sendmail::config{'sendmail_cf'});

 #--- system configuration ---
 
 # get service sendmail status
 &write_debug_log(&text('status',$service_name),'&sendmail::is_sendmail_running');
 if (&sendmail::is_sendmail_running())
 {
  # restart sendmail
  &write_debug_log(&text('restart',$service_name),'&sendmail::restart_sendmail');
  $out = &sendmail::restart_sendmail();
 }
 else
 {
  # start sendmail
  &write_debug_log(&text('start',$service_name),'&sendmail::start_sendmail');
  $out = &sendmail::start_sendmail();
 }
 &error($out) if $out;

 # enable sendmail on boot
 &write_debug_log(&text('chk_en',$service_name), '&foreign_call', 'init','enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);

 #--- configure mailboxes module ---
 
 # require librarys from module mailboxes
 &foreign_require('mailboxes','mailboxes-lib.pl');
 
 # set mail system type
 $mailboxes::config{'mail_system'} = 1;
 
 # save mailboxes config
 &write_debug_log(&text('modify','Mailboxes'), '&save_module_config', \%mailboxes::config, 'mailboxes');
 &save_module_config(\%mailboxes::config,'mailboxes');
 
 #--- configure endusers mailbox module ---
 
 # require librarys from module enduser_cf
 &foreign_require('enduser_cf','enduser_cf-lib.pl');
 
 # get enduser_cf module config
 &read_file("$enduser_cf::config{'usermin_dir'}/mailbox/config", \%mconfig);
 
 # set virtual domain map options
 $mconfig{'from_map'} = '/etc/mail/virtusertable';
 $mconfig{'from_format'} = 1;
 
 # set mailbox options
 $mconfig{'mail_system'} = 0;
 
 # save enduser_cf module config
 &write_debug_log(&text('modify','Enduser mailbox'),'&write_file',"$enduser_cf::config{'usermin_dir'}/mailbox/config", \%mconfig);
 &write_file("$enduser_cf::config{'usermin_dir'}/mailbox/config", \%mconfig);

 # add summary log
 &write_postinstall_log('mail','sendmail',1);
}

# &sendmail_module_uninstall()
# uninstall sendmail
sub sendmail_module_uninstall
{
 # require librarys from module sendmail
 &foreign_require('sendmail','sendmail-lib.pl');

 # check service name
 my $service_name='sendmail';
 
 # get sendmail service status
 my $out;
 &write_debug_log(&text('status',$service_name),'&sendmail::is_sendmail_running');
 if (&sendmail::is_sendmail_running())
 {
  # stop sendmail
  &write_debug_log(&text('stop',$service_name),'&sendmail::stop_sendmail');
  $out = &sendmail::stop_sendmail();
  &error($out) if $out;
 }
 
 # disable sendmail from boot
 &write_debug_log(&text('chk_dis',$service_name),'&foreign_call','init','disable_at_boot',$service_name);
 &foreign_call('init','disable_at_boot',$service_name);

 # restore sendmail.mc
 &write_debug_log(&text('restore','Sendmail'),'&restore_config',$sendmail::config{'sendmail_mc'});
 my $out = &restore_config($sendmail::config{'sendmail_mc'});
 
 # add summary log
 &write_postinstall_log('mail','restore_sendmail',$sendmail::config{'sendmail_mc'}) if $out;

 # rebuild sendmail.cf
 &lock_file($sendmail::config{'sendmail_cf'});
 my $cmd = "cd $sendmail::config{'sendmail_features'}/m4; m4 $sendmail::config{'sendmail_features'}/m4/cf.m4 $sendmail::config{'sendmail_mc'}";
 &write_debug_log($text{'sendmail_rebuild'},'system',"$cmd 2>/dev/null >$sendmail::config{'sendmail_cf'} </dev/null 2>&1");
 system("$cmd 2>/dev/null >$sendmail::config{'sendmail_cf'} </dev/null 2>&1");
 &unlock_file($sendmail::config{'sendmail_cf'});

 # add summary log
 &write_postinstall_log('mail','sendmail',1);
}

#-------------------------------- IMAP Servers --------------------------

# &courier_module_configure()
# configure courier-imap
sub courier_module_configure
{
 # require librarys from module courier-imap
 &foreign_require('courier-imap','courier-imap-lib.pl');

 # generate TLS certificate
 &write_debug_log($text{'TLS'},'&courier_imap::generate_certificates_safe',1);
 &courier_imap::generate_certificates_safe(1);
 
 # start or restart all courier daemons
 &write_debug_log(&text('start','all'),'&courier_imap::restart_daemons_safe','all',1,1);
 &courier_imap::restart_daemons_safe('all',1,1);
 
 # enable courier-imapd daemons on boot
 foreach ($courier_imap::config{'imap_init'}, $courier_imap::config{'pop3_init'}, $courier_imap::config{'authlib_init'})
 {
  if ($_)
  {
   &write_debug_log(&text('chk_en',$_), '&foreign_call', 'init','enable_at_boot', $_);
   &foreign_call('init','enable_at_boot', $_);  
  }
 }
 
 # add summary log
 &write_postinstall_log('mail','courier_imap',1);
 &write_postinstall_log('mail','courier_pop3',1);
}

# &courier_module_uninstall()
# uninstall courier-imap
sub courier_module_uninstall
{
 # require librarys from module courier-imap
 &foreign_require('courier-imap','courier-imap-lib.pl');

 # stop all courier-imapd daemons
 &write_debug_log(&text('stop','all'),'&courier_imap::stop_daemons_safe','all',1,1);
 &courier_imap::stop_daemons_safe('all',1,1);
 
 foreach ($courier_imap::config{'imap_init'}, $courier_imap::config{'pop3_init'}, $courier_imap::config{'authlib_init'})
 {
  if ($_)
  {
   &write_debug_log(&text('chk_dis',$_),'&foreign_call','init','disable_at_boot',$_);
   &foreign_call('init','disable_at_boot',$_)
  }
 }
 
 # add summary log
 &write_postinstall_log('mail','courier_imap',1);
 &write_postinstall_log('mail','courier_pop3',1);
}

# &xinetd_module_configure()
# configure imap2002d
sub xinetd_module_configure
{
 # require librarys from module xinetd
 &foreign_require('xinetd','xinetd-lib.pl');

 # check service name
 my $service_name = 'xinetd';

 # enable ipop3 & imap
 &write_debug_log(&text('modify','Xinetd'),'&xinetd_imap_enable');
 my $timer = &xinetd_imap_enable();

 if ($timer == 0){&error(&text('index_xinetd', '/xinetd/index.cgi'));}
 else
 {
  # get xinetd service status
  &write_debug_log(&text('status',$service_name),'&xinetd::is_xinetd_running');
  my $pid = &xinetd::is_xinetd_running();
  if ($pid)
  {
   # restart xinetd
   &write_debug_log(&text('kill',$service_name),'&kill_logged','USR2', $pid);
   &kill_logged('USR2', $pid);
   &xinetd::webmin_log("restart");
  }
  else
  {
   # start xinetd
   &write_debug_log(&text('start',$service_name),'&backquote_logged',"$xinetd::config{'start_cmd'} 2>&1 </dev/null");
   my $out = &backquote_logged("$xinetd::config{'start_cmd'} 2>&1 </dev/null");
   &error($out) if $?;
   &xinetd::webmin_log("start");
  }
 }
 
 # add summary log
 &write_postinstall_log('mail','imap',1);
}

# &xinetd_module_uninstall()
# uninstall imap2002
sub xinetd_module_uninstall
{
 # require librarys from module xinetd
 &foreign_require('xinetd','xinetd-lib.pl');

 # check service name
 my $service_name='xinetd';

 # disable ipop3 & imap
 &write_debug_log(&text('modify','Xinetd'),'&xinetd_imap_disable');
 my $timer = &xinetd_imap_disable();

 unless ($timer == 0)
 {
  # get xinetd service status
  &write_debug_log(&text('status',$service_name),'&xinetd::is_xinetd_running');
  my $pid = &xinetd::is_xinetd_running();
  if ($pid)
  {
   # restart xinetd
   &write_debug_log(&text('kill',$service_name),'&kill_logged','USR2', $pid);
   &kill_logged('USR2', $pid);
   &xinetd::webmin_log("restart");
  }
 }
 
 # add summary log
 &write_postinstall_log('mail','imap',1);
}

# &dovecot_module_configure()
# configure dovecot
sub dovecot_module_configure
{
 # require librarys from module dovecot
 &foreign_require('dovecot','dovecot-lib.pl');

 # check service name
 my $service_name = 'dovecot';

 # backup dovecot.conf
 &write_debug_log(&text('backup','Dovecot'),'&backup_config',$dovecot::config{'dovecot_config'});
 my $out = &backup_config($dovecot::config{'dovecot_config'});
 
 # add summary log
 &write_postinstall_log('mail','backup_dovecot',$dovecot::config{'dovecot_config'}) if $out;

 # modify dovecot configuration
 my $new_config = {'protocols'=>'imap imaps pop3 pop3s',
 						'ssl_disable'=>'no',
						'auth'=>'default',
						'auth_mechanisms'=>'plain',
						'auth_userdb'=>'passwd',
						'auth_passdb'=>'pam',
						'auth_user'=>'root'};
 &write_debug_log(&text('modify','Dovecot'),'&dovecot_save_directive',$new_config);
 &dovecot_save_directive($new_config);

 # get service dovecot status
 &write_debug_log(&text('status',$service_name),'&dovecot::is_dovecot_running');
 if (&dovecot::is_dovecot_running())
 {
  # restart dovecot
  &write_debug_log(&text('stop',$service_name),'&dovecot::stop_dovecot');
  &dovecot::stop_dovecot();
  &write_debug_log(&text('start',$service_name),'&dovecot::start_dovecot');
  &dovecot::start_dovecot();
 }
 else
 {
  # start dovecot
  &write_debug_log(&text('start',$service_name),'&dovecot::start_dovecot');
  &dovecot::start_dovecot();
 }
 
 # enable dovecot on boot
 &write_debug_log(&text('chk_en',$service_name), '&foreign_call', 'init','enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);

 # add summary log
 &write_postinstall_log('mail','dovecot',1);
}

# &dovecot_module_uninstall()
# uninstall dovecot
sub dovecot_module_uninstall
{
 # require librarys from module dovecot
 &foreign_require('dovecot','dovecot-lib.pl');

 # check service name
 my $service_name = 'dovecot';

 # get service dovecot status
 &write_debug_log(&text('status',$service_name),'&dovecot::is_dovecot_running');
 if (&dovecot::is_dovecot_running())
 {
  # stop dovecot
  &write_debug_log(&text('stop',$service_name),'&dovecot::stop_dovecot');
  &dovecot::stop_dovecot();
 }
 
 # disable dovecot from boot
 &write_debug_log(&text('chk_dis',$service_name),'&foreign_call','init','disable_at_boot', $service_name);
 &foreign_call('init','disable_at_boot',$service_name);

 # restore dovecot.conf
 &write_debug_log(&text('restore','Dovecot'),'&restore_config',$dovecot::config{'dovecot_config'});
 my $out = &restore_config($dovecot::config{'dovecot_config'});

 # add summary log
 &write_postinstall_log('mail','restore_dovecot',$dovecot::config{'dovecot_config'}) if $out;
 &write_postinstall_log('mail','dovecot',1);
}

#------------------------------------- Additional config save funcs ------------------------------------

# &mailserv_sasl_set ($mta, $auth_method)
# configure configuration SASLv2 file for Sendmail or Postfix
sub mailserv_sasl_set
{
 unless (-e $_[0]){
 	# touch smtpd.conf file 
	my $out &backquote_logged("touch $_[0]");
	&error($out) if $?;
 	}
 # SASLv2 file configure
 my $conf = &cyrus_sasl::parse_config_file($_[0]);
 &cyrus_sasl::set_config_value('pwcheck_method',$_[1], $conf);
 &cyrus_sasl::set_config_value('mech_list','plain login', $conf);
 &cyrus_sasl::save_config_file($_[0], $conf);
}

# &saslauthd_configure()
# configure configuration file for daemon saslauthd 
sub saslauthd_configure
{
 return unless (-e $cyrus_sasl::config{'saslauthd_conf'});
 
 # parse saslauthd file
 my $conf = &cyrus_sasl::parse_config_file($cyrus_sasl::config{'saslauthd_conf'},'=');
 
 # set daemon start options
 my $mech = $cyrus_sasl::config{'MECH'} || 'MECH';
 my $flags = $cyrus_sasl::config{'FLAGS'}  || 'FLAGS';
 &cyrus_sasl::set_config_value($mech,'pam', $conf);
 if (($gconfig{'os_type'} eq 'debian-linux')){
   # debian options
 	&cyrus_sasl::set_config_value('START','yes', $conf);
	
	if ($config{'mta'} == 0){
		# Postfix
		&backquote_logged('adduser postfix sasl');
		my $err++ if $?;
		&backquote_logged("mkdir -p $postfix::config{'mailq_dir'}/$cyrus_sasl::config{'socketdir'}");
		$err++ if $?;
		&backquote_logged("dpkg-statoverride --add root sasl 710 $postfix::config{'mailq_dir'}/$cyrus_sasl::config{'socketdir'}");
		$err++ if $?;
		unless ($err){
			# advanced options for chroot mode
			&cyrus_sasl::set_config_value($flags,"\"-m $postfix::config{'mailq_dir'}/var/run/saslauthd\"", $conf);
			&cyrus_sasl::set_config_value('PIDFILE',"\"$postfix::config{'mailq_dir'}/var/run/saslauthd/saslauthd.pid\"", $conf);
			#&cyrus_sasl::set_config_value('PWDIR',"$postfix::config{'mailq_dir'}/var/run/saslauthd/", $conf);
		
			# changed cyrus-sasl module configuration
			$cyrus_sasl::config{'socketdir'} = "$postfix::config{'mailq_dir'}/var/run/saslauthd/";
			$cyrus_sasl::config{'pid_file'} = "$postfix::config{'mailq_dir'}/var/run/saslauthd/saslauthd.pid";
			&write_debug_log(&text('modify','Cyrus SASL'), '&save_module_config', \%cyrus_sasl::config,  $cyrus_sasl::module_name);
 			&save_module_config(\%cyrus_sasl::config, $cyrus_sasl::module_name);
			}
		}
   }
 # save saslauthd file
 &cyrus_sasl::save_config_file($cyrus_sasl::config{'saslauthd_conf'}, $conf, '=');
}

# &clamav_save_directive(\%hash, $file)
# save new clamav config directives from hash
sub clamav_save_directive
{
 my $conf_file = $_[1] ? $_[1] : $clamav::config{'clamav_conf'};
 
 # get clamd.conf
 my $conf = &clamav::parse_config_file($conf_file);

 # set directives
 foreach (keys %{$_[0]})
 {
  &clamav::set_config_value($_, $_[0]->{$_}, $conf);
 }
 
 # save clamav configuration
 &clamav::save_config_file($conf, $conf_file);
}

# &amavis_save_directive(\%hash)
# save new amavis config directives from hash
sub amavis_save_directive
{
 # get amavisd.conf
 my $conf = &amavis::parse_config_file($amavis::config{'amavisd_conf'});

 # set directives
 foreach (keys %{$_[0]})
 {
  &amavis::set_config_value($_,$_[0]->{$_},$conf);
 }
 # save amavis configuration
 &amavis::save_config_file($amavis::config{'amavisd_conf'},$conf);
}

# &procmail_save_directive(\@array)
# save new procmail config directives from array
sub procmail_save_directive
{
 # get procmail config file
 my @pmrc = &spam::get_procmailrc();
 my $pmrc = $pmrc[$#pmrc];

 # backup procmail.rc
 &write_debug_log(&text('backup','Procmail'),'&backup_config',$pmrc);
 my $out = &backup_config($pmrc);
 
 # add summary log
 &write_postinstall_log('mail','backup_procmail',$pmrc) if $out;
 
 if (-e $pmrc)
 {
  # del procmailrc
  (undef, $out) = &unlink_file($pmrc);
  &error($out) if $out;
 }
 # set new directives
 foreach (@{$_[0]}) { &procmail::create_recipe($_, $pmrc); }
 
 # set permissions
 chmod(0644, $pmrc);
}

# &postfix_master_save(\@array)
# save new postfix master.cf config directives from array
sub postfix_master_save
{
 # get postfix master.cf
 my $conf = &postfix::parse_master_file($postfix::config{'postfix_master_file'});

 # set directives
 foreach (@{$_[0]})
 {
  &postfix::set_master_value($_,$conf);
 }
 # save postfix master.cf
 &postfix::save_master_file($postfix::config{'postfix_master_file'},$conf);
}

# &postfix_main_save(\%hash)
# save main postfix configuration from hash
sub postfix_main_save
{
 # coping Postfix main.cf to a backup file
 &postfix::before_save();

 # save main options
 &postfix::save_options($_[0]);

 # creating map files if need
 &postfix::ensure_map($postfix::virtual_maps);
 &postfix::ensure_map("alias_maps");

 # checking post-save fails
 &postfix::after_save();
}

# &sendmail_save_directive(\@array)
# save sendmail directives from array
sub sendmail_save_directive
{
 my $new_config = $_[0];

 &lock_file($sendmail::config{'sendmail_mc'});
 my @features = &sendmail::list_features() if (-r $sendmail::config{'sendmail_mc'} && -r "$sendmail::config{'sendmail_features'}/feature");

 # modify or create new features
 foreach (@{$new_config})
 {
  my $f;
  foreach my $i (@features)
  {
   if (($i->{'type'} eq $_->{'type'}) && ($_->{'type'} == 0) && ($i->{'text'}=~/^$_->{name}/))
   {
    if ($i->{'text'} ne $_->{'text'})
    {
     $i->{'text'} = $_->{'text'};
     &sendmail::modify_feature($i);
    }
    $f=1;
    last;
   }
   elsif (($i->{'type'} eq $_->{'type'}) && ($_->{'type'} == 1) && ($i->{'name'} eq $_->{'name'}))
   {
    $i->{'text'} = $_->{'text'};
    &sendmail::modify_feature($i);
    $f=1;
    last;
   }
   elsif (($i->{'type'} eq $_->{'type'})  && ($_->{'type'} == 2) && ($i->{'name'} eq $_->{'name'}))
   {
    $i->{'text'} = $_->{'text'};
    &sendmail::modify_feature($i);
    $f=1;
    last;
   }
   elsif (($i->{'type'} eq $_->{'type'}) && ($_->{'type'} == 4) && ($i->{'mailer'} eq $_->{'mailer'}))
   {
    $f=1;
    last;
   }
  }
  unless ($f) {&sendmail::create_feature({'text' => "$_->{'text'}"});}
 }
 &unlock_file($sendmail::config{'sendmail_mc'});
}

# &dovecot_save_directive(\%hash)
# save new dovecot config directives from hash
sub dovecot_save_directive
{
 # get dovecot config file
 &lock_file($dovecot::config{'dovecot_config'});
 my $conf = &dovecot::get_config();

 # set directives
 my $save;
 foreach (@{$conf})
 {
  foreach my $i(keys %{$_[0]})
  {
   &dovecot::save_directive($conf,$i,$_[0]->{$i}) if ($_->{'name'} eq $i && $_->{'value'} ne $_[0]->{$i});
  }
 }
 # save dovecot configuration
 &flush_file_lines();
 &unlock_file($dovecot::config{'dovecot_config'});
}

#------------------------------------- Other Additional funcs ------------------------------------
# &postfix_alias_root($alias-file, $name)
# add new postfix alias record: root -> $name
# delete postfix alias record: $name
sub postfix_alias_root
{
 # get aliases file
 my @afiles = &postfix::get_aliases_files($_[0]);

 # list aliases
 my @aliases = &postfix::list_aliases(\@afiles);

 # modify or add new alias
 my %newa=('name'=>'root', 'enabled'=>1, 'values'=>[$_[1]]);
 my $modify;
 foreach (@aliases)
 {
  if ($_->{'name'} eq 'root')
  {
   # modify alias record
   &postfix::modify_alias($_, \%newa, \@afiles);
   $modify = 1;
  }
  if ($_->{'name'} eq $_[1])
  {
   # delete alias record
   &postfix::delete_alias($_,\@afiles);
  }
 }
 # create alias record
 &postfix::create_alias(\%newa,\@afiles) if (!$modify);
}

# &user_add($user)
# add new UNIX user
sub user_add
{
 # require librarys from module useradmin
 &foreign_require('useradmin','user-lib.pl');

 # check user
 my $user = $_[0];
 my $old = grep { $_->{'user'} eq  $user }&useradmin::list_users();
 
 if (!$old)
 {
  # add new system user
  # build list of used UIDs and GIDs
  &useradmin::build_user_used(\%used);
  &useradmin::build_group_used(\%gused);

  # set user options
  my %user = ('user' => $user,'shell' => $virtual_server::config{'shell'});
  $user{'home'} = &useradmin::auto_home_dir($useradmin::config{'home_base'}, $user{'user'});
  my @root = getpwnam ("root");
  $user{'pass'} = $root[1];
  $user{'uid'} = int($useradmin::config{'base_uid'});
  while($used{$user{'uid'}})
  { 
   $user{'uid'}++;
  }
  $user{'gid'}=$user{'uid'};

  # create home directory
  &useradmin::lock_user_files();
  &useradmin::create_home_directory(\%user);
  &useradmin::set_ownership_permissions($user{'uid'}, $user{'gid'},undef, $user{'home'});

  # set group options
  my %group=('group' => $user{'user'},'gid' => $user{'gid'});

  # add user & group
  &useradmin::create_group(\%group);
  &useradmin::create_user(\%user);
  &useradmin::unlock_user_files();
  &useradmin::made_changes();

  # create an empty mail file
  # &virtual_server::create_mail_file(\%user);
  my ($mf, $md);
  if ($config{'mta'} == 1) {
  		# Sendmail always uses mail files
		$mf = &sendmail::user_mail_file($user);
		}
  elsif ($config{'mta'} == 0) {
  		# Postfix user
		my ($s, $d) = &postfix::postfix_mail_system();
		if ($s == 0 || $s == 1) {
			# A mail file
			$mf = &postfix::postfix_mail_file($user);
			}
   	elsif ($s == 2) {
			# A mail directory
			$md = &postfix::postfix_mail_file($user);
			}
		}
  
  if ($mf && !-r $mf) 
  {
	# Create the mailbox, owned by the user
	&open_tempfile(MF, ">$mf");
	&close_tempfile(MF);
	&set_ownership_permissions($user{'uid'}, $user{'gid'}, undef, $mf);
  }
  elsif ($md && !-d $md) 
  {
	 # Create the Maildir, owned by the user
	 foreach my $d ($md, "$md/cur", "$md/tmp", "$md/new") 
	 {
	  &make_dir($d, 0700, 1);
	  &set_ownership_permissions($user{'uid'}, $user{'gid'}, undef, $d);
	 }
  }

  if (-d $user{'home'}) 
  {
	# Create Usermin ~/mail directory
	my $umail = "$user{'home'}/mail";
	if (!-d $umail) 
	{
	 &make_dir($umail, 0755);
	 &set_ownership_permissions($user{'uid'}, $user{'gid'}, undef, $umail);
	}
  }
 }
}

# &xinetd_imap_enable()
# enable imap & pop3 services in the xinetd config files
# return undef if imap & pop3 files not found
sub xinetd_imap_enable
{
 # get xinetd configuration
 &lock_file($xinetd::config{'xinetd_conf'});
 my @conf = &xinetd::get_xinetd_config();

 # enable ipop3 & imap
 my $timer;
 foreach (@conf)
 {
  if ($_->{'file'} eq "/etc/xinetd.d/ipop3" || $_->{'file'} eq "/etc/xinetd.d/imap")
  {
   $timer++;
   if ($_->{'quick'}->{'disable'}->[0] eq 'yes')
   {
    &lock_file($_->{'file'});
    &xinetd::set_member_value($_,'disable','no');
    &xinetd::modify_xinet($_);
    &unlock_file($_->{'file'});
   }
  }
 }
 &unlock_file($xinetd::config{'xinetd_conf'});
 return $timer;
}

# xinetd_imap_disable()
# disable imap & pop3 services from the xinetd config files
# return undef if imap & pop3 files not found
sub xinetd_imap_disable
{
 # get xinetd configuration
 &lock_file($xinetd::config{'xinetd_conf'});
 my @conf = &xinetd::get_xinetd_config();

 # disable ipop3 & imap
 my $timer;
 foreach (@conf)
 {
  if ($_->{'file'} eq "/etc/xinetd.d/ipop3" || $_->{'file'} eq "/etc/xinetd.d/imap")
  {
   $timer++;
   if ($_->{'quick'}->{'disable'}->[0] eq 'no')
   {
    &lock_file($_->{'file'});
    &xinetd::set_member_value($_,'disable','yes');
    &xinetd::modify_xinet($_);
    &unlock_file($_->{'file'});
   }
  }
 }
 &unlock_file($xinetd::config{'xinetd_conf'});
 return $timer;
}

# &switch_smtp_link(MTA)
# check smtp link for PAM MTA lib
sub switch_smtp_link
{
 # require librarys from module pam
 &foreign_require('pam','pam-lib.pl');
 
 # rebuild symlink smtp.MTA 
 my $mta = $_[0];
 &backquote_logged("alternatives --set mta /usr/sbin/sendmail.$_[0] 2>&1 </dev/null");
 if (-l "$pam::config{'pam_dir'}/smtp")
 {
  my $link = &backquote_logged("readlink -f $pam::config{'pam_dir'}/smtp");
  unless ($link eq "$pam::config{'pam_dir'}/smtp.$mta")
  {
   # change symlink to smtp.MTA
   &unlink_file("$pam::config{'pam_dir'}/smtp");
   &symlink_logged("$pam::config{'pam_dir'}/smtp.$mta","$pam::config{'pam_dir'}/smtp");
  }
 }
 else
 {
  # add symlink to smtp.MTA
  &symlink_logged("$pam::config{'pam_dir'}/smtp.$mta","$pam::config{'pam_dir'}/smtp");
 }
}

#------------------------------------- Mailman Additional funcs ------------------------------------
# &mailman_apache_setup()
# apache mailman support configure
# return undef or 1 if error
sub mailman_apache_setup
{
 # require library from module apache
 &foreign_require('apache', 'apache-lib.pl');
 
 # mailman.conf copy
 my $out = &copy_source_dest("$module_root_directory/addons/web/mailman.conf","$apache::config{'httpd_dir'}/conf.d/");
 return 1 unless $out;
 
 # get mailman.conf options
 my $hostname = $_[0];
 my ($cgi, $arch) = ($mailman::config{'mailman_cgi'}, $mailman::config{'mailman_arch'});
 
 # apache config files
 my $conf_file = "$apache::config{'httpd_dir'}/conf.d/mailman.conf";
 my $apache_conf = "$apache::config{'httpd_dir'}/conf/httpd.conf";
 
 # modify mailman.conf
 &lock_file($conf_file);
 my $lref = &read_file_lines($conf_file);
 foreach (@{$lref})
 {
  $_ =~ s/\$CGI_PATH/$cgi/;
  $_ =~ s/\$ARCH_PATH/$arch/;
  $_ =~ s/\$HOSTNAME/$hostname/;
 }
 &flush_file_lines();
 &unlock_file($conf_file);
 
 # get apache config
 undef (@apache::get_config_cache);
 my $conf = &apache::get_config();
 
 # get ip and port
 my $port = &apache::find_directive("Port", $conf) || 80;
 my @int = &lan_check(1);
 my $ip = $int[0]->{'address'};

 # search Virtual Hosts
 my %hosts;
 foreach my $i (@{$conf})
 {
  # $name_virt = 1 if ($i->{'name'} eq 'NameVirtualHost');
  foreach my $j (@int)
  {
   next unless &check_ipaddress ($j->{'address'});
   $ip = $j->{'address'};
   if (($i->{'name'} eq 'VirtualHost') && ($i->{'value'} eq "$ip:$port"))
   {
	 $hosts{"$ip"} = grep {($_->{'name'} eq 'ServerName') && ($_->{'value'} eq $hostname)} @{$i->{'members'}};
   }  
  }
 }
 
 # change httpd.conf
 &lock_file($apache_conf);
 foreach (@int)
 {
  # for all ip
  next unless &check_ipaddress ($_->{'address'});
  $ip = $_->{'address'};
 
  # add NameVirtualHost
  # &apache::save_directive("NameVirtualHost", [ "$ip:$port" ], $conf, $conf) if !$name_virt;
  
  unless ($hosts{"$ip"})
  {
   # add VirtualHost
   my $lref = &read_file_lines($apache_conf);
   push(@$lref, "<VirtualHost $ip:$port>");
   push(@$lref, "ServerName $hostname");
   push(@$lref, "ServerAlias www.$hostname");
   push(@$lref, "</VirtualHost>"); 
  }
 }
 &flush_file_lines();
 &unlock_file($apache_conf);
 
 if (&apache::is_apache_running())
 {
  # restart apache
  &write_debug_log(&text('restart','Apache'),'&apache::restart_apache');
  my $out = &apache::restart_apache();
  return 1 if $out;
 }
 return 0;
}

# &mailman_apache_uninstall()
# apache mailman support disable
# return undef or 1 if error
sub mailman_apache_uninstall
{
 # require library from module apache
 &foreign_require('apache', 'apache-lib.pl');
 
 # mailman.conf remove
 my $out = &unlink_file("$apache::config{'httpd_dir'}/conf.d/mailman.conf") if (-e "$apache::config{'httpd_dir'}/conf.d/mailman.conf");
 return 1 unless $out;
 
 # get apache service status 
 if (&apache::is_apache_running())
 {
  # restart apache
  &write_debug_log(&text('restart','Apache'),'&apache::restart_apache');
  $out = &apache::restart_apache();
  return 1 if $out;
 }
 return undef;
}

#------------------------------------- Exim Additional funcs ------------------------------------
# is_exim_running()
# returns 1 or PIDs if running, 0 if stopped
sub is_exim_running
{
 if ($config{'exim_status_command'}){
 	# status cmd
 	my $out = &backquote_logged("$config{'exim_status_command'} </dev/null 2>&1");
	return 1 if $?;
	}
 else{
 	# search running process
	return &find_byname($config{'vsftpd_cmd'});
 	}
}

# &stop_exim()
# stops the exim process, returning undef on success 
# or an error message upon failure.
sub stop_exim
{
 my $out = &backquote_logged("$config{'exim_stop_command'} </dev/null 2>&1");
 return "<pre>$out</pre>" if $?;
 return undef;
}

1;
