# server-mysql.pl
# configure MySQL Server
 
#---------------------------- Global ----------------------

# &configure_mysql()
# call configure functions for all mysql services
sub configure_mysql
{
 if (&configured('mysql'))
 {
  &msg_conftitle($text{'mysql'});
  &mysql_module_configure();
  &msg_done();
 }
 else {&error(&text('index_mysql','/mysql/index.cgi'));}
}

# &uninstall_mysql()
# call uninstall functions for all mysql services
sub uninstall_mysql
{
 &msg_conftitle_uninstall($text{'mysql'});
 &mysql_module_uninstall();
 &msg_done();
}

# &get_modules_mysql()
# return all name modules for search postinstall scripts
sub get_modules_mysql
{
 return ('mysql');
}

# &get_websites_configure_mysql()
# return %rv with new options for virtual-server config
sub get_websites_configure_mysql
{
 local %rv = ();
 $rv{'mysql'} = &configured('mysql') ? 1 : 0;
 return %rv;
}

# &uninstall_websites_configure_mysql()
# return %rv with zero options for virtual-server config
sub uninstall_websites_configure_mysql
{
 local %rv = ();
 $rv{'mysql'} = 0;
 return %rv;
}

#---------------------------- MySQL configure ----------------------

# &mysql_module_configure()
# configure mysql server
sub mysql_module_configure
{
 # require librarys from modules mysql & init
 &foreign_require('mysql','mysql-lib.pl');
 &foreign_require('init','init-lib.pl');

 # check service name
 my $service_name = 'mysql';
 my @services = &init::list_actions();
 foreach (@services) {
 	$service_name=$1 if ($_=~/^(mysqld)\b/);
	}
 
 # get mysql service  status
 my $out;
 &write_debug_log(&text('status',$service_name),'&mysql::is_mysql_running');
 if (&mysql::is_mysql_running())
 {
  # restart mysql service
  &write_debug_log(&text('stop',$service_name),'&mysql::stop_mysql');
  $out = &mysql::stop_mysql();
  &error($out) if $out;
  &write_debug_log(&text('start',$service_name),'&mysql::start_mysql');
  $out = &mysql::start_mysql();
 }
 else
 {
  # start mysql service
  &write_debug_log(&text('start',$service_name),'&mysql::start_mysql');
  $out = &mysql::start_mysql();
 }
 &error($out) if $out;
 sleep(1);
 
 # enable mysql service on boot
 &write_debug_log(&text('chk_en',$service_name), '&foreign_call', 'init', 'enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);

 # generate random password
 my @rand=("A".."Z","a".."z",0..9);
 my $passwd=join("",@rand[map {rand @rand}(1..8)]);
 $passwd = &mysql::escapestr("$passwd");
 
 # set mysql root password
 &write_debug_log($text{'mysql_pass'},'&backquote_logged',"$mysql::config{'mysqladmin'} -u root password $passwd");
 my $out = &backquote_logged ("$mysql::config{'mysqladmin'} -u root password $passwd");
 unless ($?)
 {
  # set password for the Webmin login
  $mysql::config{'login'} = 'root';
  $mysql::config{'pass'} = $passwd;
  &write_file("$mysql::module_config_directory/config", \%mysql::config);
    
  # add confmaster log
  &write_postinstall_log('mysql','passwd',$passwd);
 }
 # add confmaster log
 &write_postinstall_log('mysql','mysql',1);
}

# &mysql_module_uninstall()
# uninstall mysql server 
sub mysql_module_uninstall
{
 # require librarys from modules mysql & init
 &foreign_require('mysql','mysql-lib.pl');
 &foreign_require('init','init-lib.pl');

 # check mysql service name
 my $service_name = 'mysql';
 @services = &init::list_actions();
 foreach (@services) {
 	$service_name=$1 if ($_=~/^(mysqld)\b/);
	}

 # get mysql service status
 &write_debug_log(&text('status',$service_name),'&mysql::is_mysql_running');
 if (&mysql::is_mysql_running())
 {
  # stop mysql service
  &write_debug_log(&text('stop',$service_name),'&mysql::stop_mysql');
  my $out = &mysql::stop_mysql();
  &error($out) if $out;
 }
 
 # disable mysql from boot
 &write_debug_log(&text('chk_dis',$service_name),'&foreign_call','init','disable_at_boot', $service_name);
 &foreign_call('init','disable_at_boot',$service_name);

 # add confmaster log
 &write_postinstall_log('mysql','mysql',1);
}

1;
