# server-postgresql.pl
# configures PostgreSQL Server

#---------------------------- Global ----------------------

# &configure_postgres()
# call configure functions for all postgresql services
sub configure_postgres
{
 if (&configured('postgresql'))
 {
  &msg_conftitle($text{'postgresql'});
  &postgres_module_configure();
  &msg_done();
 }
 else {&error(&text('index_postgresql','/postgresql/index.cgi'));}
}

# &uninstall_postgres()
# call uninstall functions for all postgresql services
sub uninstall_postgres
{
 &msg_conftitle_uninstall($text{'postgresql'});
 &postgresql_module_uninstall();
 &msg_done();
}

# &get_modules_postgres()
# return all name modules for search postinstall scripts
sub get_modules_postgres
{
 return ('postgresql');
}

# &get_websites_configure_postgres()
# return %rv with new options for virtual-server config
sub get_websites_configure_postgres
{
 local %rv = ();
 $rv{'postgres'} = &configured('postgresql') ? 1 : 0;
 return %rv;
}

# &uninstall_websites_configure_postgres()
# return %rv with zero options for virtual-server config
sub uninstall_websites_configure_postgres
{
 local %rv = ();
 $rv{'postgres'} = 0;
 return %rv;
}

#---------------------------- PostgreSQL configure ----------------------

# &postgres_module_configure()
# configure postgresql
sub postgres_module_configure
{
 # require librarys from modules postgresql & init
 &foreign_require('postgresql','postgresql-lib.pl');
 &foreign_require('init','init-lib.pl');

 # check service name
 my $service_name = 'postgresql';
 @services = &init::list_actions();
 foreach (@services) { 
 	$service_name = $1 if ($_=~/^(rhdb)\b/); 
	}
 
 # postgres mode
 $postgresql::force_nodbi = 1;
 
 # get postgresql service status
 my $out;
 &write_debug_log(&text('status',$service_name),'&postgresql::is_postgresql_running');
 if (&postgresql::is_postgresql_running())
 {
  # restart postgresql
  &write_debug_log(&text('restart',$service_name),'&restart_pgsql');
  $out = &restart_pgsql();
 }
 else
 {
  # start postgresql
  &write_debug_log(&text('start',$service_name),'&postgresql::start_postgresql');
  $out = &postgresql::start_postgresql();
 }
 &error($out) if $out;
 sleep(1);
 
 # generate random password
 my @rand = ("A".."Z","a".."z",0..9);
 my $passwd = join("",@rand[map {rand @rand}(1..8)]);
 
 # set postgres password
 &write_debug_log($text{'postgres_pass'}, '&postgresql::execute_sql_logged', "$postgresql::config{'basedb'}, alter user $postgresql::postgres_login with password $passwd createdb createuser");
 &postgresql::execute_sql_logged($postgresql::config{'basedb'}, "alter user $postgresql::postgres_login with password '$passwd' createdb createuser");
  
 # set password for the Webmin login
 $postgresql::config{'pass'} = $postgresql::postgres_pass = $passwd;
 &write_file("$postgresql::module_config_directory/config", \%postgresql::config);
    
 # add confmaster log
 &write_postinstall_log('postgres','passwd',$passwd);
 
 # backup postgresql files
 my @config = ($postgresql::config{'hba_conf'}, $postgresql::config{'postgresql_conf'});
 my $j;
 foreach (@config)
 {
  &write_debug_log(&text('backup','PostgreSQL'),'&backup_config', $_);
  $out = &backup_config($_);
	
  # add summary log
  &write_postinstall_log('postgres','backup_postgres'.$j, $_) if $out;
  $j++;
 }
 
 # enable tcpip_socket option in the postgresql configuration 
 &write_debug_log(&text('modify', $service_name),'&tcpip_enable');
 &tcpip_enable();
   
 # modify postgresql hba config
 my $hba = [{'type' => 'local',
					'db' => 'all',
					'user' => 'all',
					'auth' => 'password' },
				  {'address' => '127.0.0.1',
					'netmask' => '255.255.255.255',
					'type' => 'host',
					'db' =>  'all',
					'user' => 'all',
					'auth' => 'password' }];
 &write_debug_log(&text('modify', 'PostgreSQL'),'&hosts_configure', $hba);
 &hosts_configure($hba);

 # restart postgresql
 &write_debug_log(&text('restart',$service_name),'&restart_pgsql');
 $out = &restart_pgsql();
 &error($out) if $out; 
  
 # enable postgresql on boot
 &write_debug_log(&text('chk_en',$service_name), '&foreign_call', 'init','enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);

 # add summary log
 &write_postinstall_log('postgres', 'postgres', 1);
}

# &postgresql_module_uninstall()
# uninstall postgrsql
sub postgresql_module_uninstall
{
 # require librarys from modules postgresql & init
 &foreign_require('postgresql','postgresql-lib.pl');
 &foreign_require('init','init-lib.pl');

 # check service name
 my $service_name = 'postgresql';
 @services = &init::list_actions();
 foreach (@services) { 
 	$service_name = $1 if ($_=~/^(rhdb)\b/); 
	}
 
 # postgres mode
 $postgresql::force_nodbi = 1;
 
 # service postgresql status
 my $out;
 &write_debug_log(&text('status',$service_name),'&postgresql::is_postgresql_running');
 if (&postgresql::is_postgresql_running())
 {
  # stop mysql service
  &write_debug_log(&text('stop',$service_name),'&postgresql::stop_postgresql');
  $out = &postgresql::stop_postgresql();
  &error($out) if $out;
 }

 # disable postgresql service from boot
 &write_debug_log(&text('chk_dis',$service_name),'&foreign_call','init','disable_at_boot',$service_name);
 &foreign_call('init','disable_at_boot',$service_name);
 
 # restore postgresql files
 my @config = ($postgresql::config{'hba_conf'}, $postgresql::config{'postgresql_conf'});
 my $j;
 foreach (@config)
 {
  &write_debug_log(&text('restore','PostgreSQL'), '&restore_config', $_);
  $out = &restore_config($_);
	
  # add summary log
  &write_postinstall_log('postgres','restore_postgres'.$j, $_) if $out;
  $j++;
 }
 
 # add summary log
 &write_postinstall_log('postgres','postgres',1);
}

# &tcpip_enable()
# modify tcpip_socket option 
# in the postgresql configuration file
sub tcpip_enable
{
 my $modify;
 my $conf = &read_file_lines($postgresql::config{'postgresql_conf'});
 foreach (@{$conf})
 {
  if ($_ =~ /^\s*?#?\s*?tcpip_socket/)
  {
	# modify option
	$_ = 'tcpip_socket = true';
	$modify = 1;
	last;
  }
  elsif ($_ =~ /^\s*?#?\s*?listen_addresses/)
  {
	# modify option
	$_ = 'listen_addresses = \'localhost\'';
	$modify = 1;
	last;
  }
 }
 push (@{$conf}, 'tcpip_socket = true') unless $modify;
 &flush_file_lines();
}

# &hosts_configure(\@hba)
# add new records from @hba to postgresql hba config
sub hosts_configure
{
 # get postgresql hba config
 my $v = &postgresql::get_postgresql_version();
 my $lref = &postgresql::read_file_lines($postgresql::config{'hba_conf'});
 foreach (@$lref) 
 {
  # delete all hba records
  undef $_ unless ($_ =~ /^\s*?#/);
 }
 &write_debug_log(&text('modify', 'PostgreSQL'),'&flush_file_lines');
 &flush_file_lines();
	
 # create new records
 foreach (@{$_[0]}) { &postgresql::create_hba($_, $v); }
}

# &restart_pgsql()
# restart postgresql service
sub restart_pgsql
{
 $out = &postgresql::stop_postgresql();
 return $out if $out;
 $out = &postgresql::start_postgresql();
 return $out if $out;
}

1;
