# server-quota.pl
# configures quota
 
#---------------------------- Global ----------------------

# &update_rpms_quota($rpms)
# return undef for / vfs
sub update_rpms_quota
{
 &foreign_require('quota','quota-lib.pl');
 foreach (&quota::list_filesystems())
 {
  if ($_->[0] eq '/') { 
  	 return undef if ($_->[2] eq 'vfs');
	 }
 }
 return $_[0];
}

# &configure_quota()
# call configure functions for all quota services
sub configure_quota
{
 if (&configured('quota'))
 {
  &msg_conftitle($text{'quota'});
  &quota_module_configure();
  &msg_done();
 }
 else {&error(&text('index_quota','/quota/index.cgi'));}
}

# &uninstall_quota()
# call uninstall functions for all quota services
sub uninstall_quota
{
 &msg_conftitle_uninstall($text{'quota'});
 &quota_module_uninstall();
 &msg_done();
}

# &get_modules_quota()
# return all name modules for search postinstall scripts
sub get_modules_quota
{
 return ('quota');
}

# &get_websites_configure_quota()
# return %rv with new options for virtual-server config
sub get_websites_configure_quota
{
 my %rv = ();
 $rv{'quotas'} = &configured('quota') ? 1 : 0;
 $rv{'hard_quotas'} = 0;
 return %rv;
}

# &uninstall_websites_configure_quota()
# return %rv with zero options for virtual-server config
sub uninstall_websites_configure_quota
{
 local %rv = ();
 $rv{'quotas'} = 0;
 $rv{'hard_quotas'} = 0;
 return %rv;
}

#---------------------------- Quotas configure ----------------------

# &quota_module_configure()
# configure quota
sub quota_module_configure
{
 # require librarys from modules quota & mount
 &foreign_require('quota','quota-lib.pl');
 &foreign_require('mount','mount-lib.pl');

 # change quota
 my ($st, $i, $num, @temp);
 foreach (&quota::list_filesystems())
 {
  if (($_->[2] eq 'ext3') || ($_->[2] eq 'ext2'))
  {
   if ($_->[4]==3)
   {
    unless ($_->[5]==3)
    {
	  # remount and quotaon
     &enable_quota($_->[0])
    }
   }
   else
   {
    $st=0;
    foreach $i (&mount::list_mounts())
    {
     if ($_->[0] eq $i->[0])
     {
	   # change fstab file
      &mount::parse_options(undef,$i->[3]);
      unless (%mount::options){$mount::options{'defaults'} = "";}
      $mount::options{'grpquota'} = "";
      $mount::options{'usrquota'} = "";
      $options=&mount::join_options();
      @temp=@{$_};
      $num=$st;
      &write_debug_log($text{'quota_modify'},'&mount::change_mount',$num,$i->[0],$i->[1],$i->[2], 								$options,$i->[4],$i->[5]);
      &mount::change_mount($num,$i->[0],$i->[1],$i->[2],$options,$i->[4],$i->[5]);
      @{$_}=@temp;
      &write_debug_log($text{'quota_quotaoff'},'&quota::quotaoff',$_->[0],$_->[5]);
      &quota::quotaoff($_->[0],$_->[5]);
		
		# remount and quotaon
		&enable_quota($_->[0])
     }
     $st++;
    }
   }
	
   # set config flag - email_virtualmin_
   $quota::config{"email_virtualmin_$_->[0]"} = 1;
   &quota::save_module_config();
  }
 }
 
 # add new quota
 my ($j,@quota_en,@quota_type);
 @quota_en=&quota::list_filesystems();
 foreach (@quota_en){push (@quota_type,$_->[0]);}
 $st=0;
 foreach $j (&mount::list_mounts())
 {
  unless (grep {$j->[0] eq $_} @quota_type)
  {
   if (($j->[2] eq 'ext3') || ($j->[2] eq 'ext2'))
   {
    &mount::parse_options(undef,$j->[3]);
    unless (%mount::options){$mount::options{'defaults'} = "";}
    $mount::options{'grpquota'} = "";
    $mount::options{'usrquota'} = "";
    $options=&mount::join_options();
    @temp=@{$j};
    $num=$st;
    &write_debug_log($text{'quota_modify'},'&mount::change_mount',$num,$j->[0],$j->[1],$j->[2],$options,$j->[4],$j->[5]);
    &mount::change_mount($num,$j->[0],$j->[1],$j->[2],$options,$j->[4],$j->[5]);
    @{$j}=@temp;
	 
	 # remount and quotaon
	 &enable_quota($j->[0]);
    
    # set config flag - email_virtualmin_
    $quota::config{"email_virtualmin_$j->[0]"} = 1;
    &quota::save_module_config();
   }
  }
  $st++;
 }
  
 # add confmaster log
 if ($config{'ztl_mode'}) {&write_postinstall_log('web','quota',1);}
 else {&write_postinstall_log('quota','quota',1);}
}

# &quota_module_uninstall()
# uninstall quota
sub quota_module_uninstall
{
 # require librarys from modules quota & mount
 &foreign_require('quota','quota-lib.pl');
 &foreign_require('mount','mount-lib.pl');

 # disable quota
 my ($st,$i,$num,@temp);
 foreach (&quota::list_filesystems())
 {
  if (($_->[2] eq 'ext3') || ($_->[2] eq 'ext2'))
  {
   $st=0;
   foreach $i (&mount::list_mounts())
   {
    if ($_->[0] eq $i->[0])
    {
     &mount::parse_options(undef,$i->[3]);
     %mount::options=('defaults'=>"");
     $options=&mount::join_options();
     @temp=@{$_};
     $num=$st;
     &write_debug_log($text{'quota_modify'},'&mount::change_mount',$num,$i->[0],$i->[1],$i->[2],$options,$i->[4],$i->[5]);
     &mount::change_mount($num,$i->[0],$i->[1],$i->[2],$options,$i->[4],$i->[5]);
     @{$_}=@temp;
     &write_debug_log($text{'quota_quotaoff'},'&quota::quotaoff',$_->[0],$_->[5]);
     &quota::quotaoff($_->[0],$_->[5]);
     &write_debug_log($text{'quota_remount'},'&system_logged',"mount -o remount $_->[0] </dev/null 2>&1");
     &system_logged("mount -o remount $_->[0] </dev/null 2>&1");
    }
    $st++;
   }
  }
 }
 
 # add confmaster log
 if ($config{'ztl_mode'}) {&write_postinstall_log('web','quota',1);}
 else {&write_postinstall_log('quota','quota',1);}
}

# &enable_quota($filesystem)
# remount filesystem
# quotachek, quotaon
sub enable_quota
{
 &write_debug_log($text{'quota_remount'},'&system_logged',"mount -o remount $_[0] </dev/null 2>&1");
 &system_logged("mount -o remount $_[0] </dev/null 2>&1");
 &write_debug_log($text{'quota_quotacheck'},'&quota::quotacheck',$_[0],3);
 &quota::quotacheck($_[0],3);
 &write_debug_log($text{'quota_quotaon'},'&quota::quotaon',$_[0],3);
 &quota::quotaon($_[0],3);
}

1;
