# server-web.pl
# configure Apache Web Server
 
#---------------------------- Global ----------------------

# &update_rpms_web($rpms)
# only needed packages
sub update_rpms_web
{
 foreach (@{$_[0]}){
 	# remove webalizer
 	push (@rpms, $_) unless (!$config{'web_system'} && $_=~/^webalizer/);
	}
 return \@rpms;
}

# &configure_web()
# call configure functions for all web services
sub configure_web
{
 if ($config{'web_system'} == 1)
 {
  # configure Webalizer
  if (&configured('webalizer'))
  {
   # configure
   &msg_conftitle($text{'webalizer'});
   &webalizer_module_configure();
   &msg_done();
  }
  else { &error(&text('index_webalizer','/webalizer/index.cgi')); }
 }
 if (&configured('apache'))
 {
  # configure Apache
  &msg_conftitle($text{'apache'});
  &apache_module_configure();
  &msg_done();
 }
 else { &error(&text('index_apache','/apache/index.cgi')); }
}

# &uninstall_web()
# call uninstall functions for all web services
sub uninstall_web
{
 if ($config{'web_system'} == 1)
 {
  # uninstall Webalizer
  &msg_conftitle_uninstall($text{'webalizer'});
  &webalizer_module_uninstall();
  &msg_done();
 }
 
 # uninstall Apache
 &msg_conftitle_uninstall($text{'apache'});
 &apache_module_uninstall();
 &msg_done();
}

# &get_modules_web()
# return all name modules for search postinstall scripts
sub get_modules_web
{
 my @mods = ('apache');
 push (@mods, 'webalizer') if $config{'web_system'};
 return @mods;
}

# &get_websites_configure_web()
# return %rv with new options for virtual-server config
sub get_websites_configure_web
{
 local %rv = ();
 
 # web feature enable
 $rv{'web'} = &configured('web') ? 1 : 0;
 
 # webalizer feature enable
 $rv{'webalizer'} = &configured('webalizer') ? 1 : 0;
 
 # ssl support search
 $rv{'ssl'} = 0;
 if ($rv{'web'} && &has_command("openssl"))
 {
  undef (@apache::get_config_cache);
  local $conf = &apache::get_config();
  local @loads = &apache::find_directive_struct("LoadModule", $conf);
  local ($l, $hasmod);
  foreach $l (@loads) { 
  	 $hasmod++ if ($l->{'words'}->[1] =~ /mod_ssl/);
	 }
  local ($aver, $amods) = &apache::httpd_info($httpd = &apache::find_httpd());
  $hasmod++ if (&indexof("mod_ssl", @$amods) >= 0);
  $hasmod++ if ($apache::httpd_modules{'mod_ssl'});
  if ($hasmod)
  {
   local $default_web_sslport = 443;
   local @listens = &apache::find_directive_struct("Listen", $conf);
   foreach $l (@listens)
   {
    $haslisten++ if ($l->{'words'}->[0] =~ /^(\S+:)?$default_web_sslport$/);
   }
   local @ports = &apache::find_directive_struct("Port", $conf);
   foreach $l (@ports) {
		$haslisten++ if ($l->{'words'}->[0] == $default_web_sslport);
		}
	# ssl feature enable
   $rv{'ssl'} = 1 if $haslisten;
  }
 }
 return %rv;
}

# &uninstall_websites_configure_web()
# return %rv with zero options for virtual-server config
sub uninstall_websites_configure_web
{
 local %rv = ();
 $rv{'web'} = 0;
 $rv{'webalizer'} = 0;
 $rv{'ssl'} = 0;
 return %rv;
}

#---------------------------- Webalizer configuration ----------------------

# &webalizer_module_configure()
# configure webilizer
sub webalizer_module_configure
{
 # require librarys from module webalizer
 &foreign_require('webalizer', 'webalizer-lib.pl');

 # backup webalizer.conf
 &write_debug_log(&text('backup','Webalizer'),'&backup_config', $webalizer::config{'webalizer_conf'});
 my $out = &backup_config($webalizer::config{'webalizer_conf'});
 
 # add confmaster log
 &write_postinstall_log('web','backup_webalizer',$webalizer::config{'webalizer_conf'}) if $out;

 # get config webalizer
 my $cfile = $webalizer::config{'webalizer_conf'};
 &lock_file($cfile);
 my $conf = &webalizer::get_config();
 my $history = &webalizer::find_value("HistoryName", $conf);
 my $current = &webalizer::find_value("IncrementalName", $conf);
 my $cache = &webalizer::find_value("DNSCache", $conf);

 # get values
 if ($history =~ /^\//) {
 	my @history = split(/\//,$history);
	$history = pop(@history);
	}
 else { $history = 'webalizer.hist'; }
 if ($current =~ /^\//) {
 	my @current = split(/\//,$current);
	$current = pop(@current);
	}
 else { $current = 'webalizer.current'; }
 if ($cache=~/^\//) {
 	my @cache = split(/\//,$cache);
	$cache = pop(@cache);
	}
 else { $cache = 'dns_cache.db'; }
 
 # save values
 &webalizer::save_directive($conf, "HistoryName", $history);
 &webalizer::save_directive($conf, "IncrementalName", $current);
 &webalizer::save_directive($conf, "DNSCache", $cache);
 &write_debug_log(&text('modify','Webalizer'),'&flush_file_lines');
 &flush_file_lines();
 &unlock_file($cfile);

 # add confmaster log
 &write_postinstall_log('web','webalizer',1);
}

# &webalizer_module_uninstall()
# uninstall webilizer
sub webalizer_module_uninstall
{
 # restore webalizer.conf
 &write_debug_log(&text('restore','Webalizer'),'&restore_config', $webalizer::config{'webalizer_conf'});
 my $out = &restore_config($webalizer::config{'webalizer_conf'});

 # add confmaster log
 &write_postinstall_log('web','restore_webalizer',$webalizer::config{'webalizer_conf'}) if $out;
 &write_postinstall_log('web','webalizer',1);
}

#---------------------------- Apache configuration ----------------------

# &apache_module_configure()
# configure apache
sub apache_module_configure
{
 # require librarys from modules apache & init
 &foreign_require('apache','apache-lib.pl');
 &foreign_require('init','init-lib.pl');
 
 # check service name
 my $service_name = 'httpd';
 my @services = &init::list_actions();
 foreach (@services) {
 	$service_name = $1 if ($_=~/^(apache)\b/);
	}
 
 # SSL & PHP setup only for TSL
 my $out;
 if ($gconfig{'os_type'} eq 'trustix-linux') #&& ($gconfig{'os_version'} == 2.2)))
 {
  # backup apache config files
  my @config=($apache::config{'defines_file'},"$apache::config{'httpd_dir'}/conf/httpd.conf", "$apache::config{'httpd_dir'}/conf/ssl.conf","$apache::config{'httpd_dir'}/conf.d/httpd-php.conf");
  my $j;
  foreach (@config)
  {
   &write_debug_log(&text('backup','Apache'),'&backup_config',$_);
   $out=&backup_config($_);
   $j++;
  
   # add confmaster log
   &write_postinstall_log('web','backup_apache'.$j,$_) if $out;
  }
  
  # ssl setup
  &write_debug_log($text{'apache_ssl'},'&ssl_setup');
  &ssl_setup();
  
  # add confmaster log
  &write_postinstall_log('web','ssl',1);
  
  # php setup
  &write_debug_log($text{'apache_php'},'&php_setup');
  &php_setup();
  
  # add confmaster log
  &write_postinstall_log('web','php',1);
 }
 
 # start or restart httpd
 &write_debug_log(&text('status',$service_name),'&apache::is_apache_running');
 if (&apache::is_apache_running())
 {
  # restart httpd
  &write_debug_log(&text('restart',$service_name),'&apache::restart_apache');
  $out = &apache::restart_apache();
 }
 else
 {
  # start httpd
  &write_debug_log(&text('start',$service_name),'&apache::start_apache');
  $out = &apache::start_apache();
 }
 &error($out) if $out;
 
 # enable httpd on boot
 &write_debug_log(&text('chk_en', $service_name), '&foreign_call', 'init', 'enable_at_boot', $service_name);
 &foreign_call('init','enable_at_boot', $service_name);
 
 # add confmaster log
 &write_postinstall_log('web','apache',1);
}

# &apache_module_uninstall()
# uninstall apache
sub apache_module_uninstall
{
 # require librarys from modules apache & init
 &foreign_require('apache','apache-lib.pl');
 &foreign_require('init','init-lib.pl');
 
 # check service name
 my $service_name='httpd';
 @services=&init::list_actions();
 foreach (@services) {
 	$service_name=$1 if ($_=~/^(apache)\b/); 
	}
  
 # stop httpd
 my $out;
 &write_debug_log(&text('status',$service_name),'&apache::is_apache_running');
 if (&apache::is_apache_running())
 {
  &write_debug_log(&text('stop',$service_name),'&apache::stop_apache');
  $out = &apache::stop_apache();
  &error($out) if $out;
 }
 
 # disable httpd from boot
 &write_debug_log(&text('chk_dis', $service_name),'&foreign_call','init','disable_at_boot', $service_name);
 &foreign_call('init','disable_at_boot', $service_name);
  
 # only for TSL
 if ($gconfig{'os_type'} eq 'trustix-linux') #&& ($gconfig{'os_version'} == 2.2))
 {
  # restore apache config files
  my @config=($apache::config{'defines_file'},"$apache::config{'httpd_dir'}/conf/ssl.conf", "$apache::config{'httpd_dir'}/conf.d/httpd-php.conf");
  my $j;
  foreach (@config)
  {
   &write_debug_log(&text('restore','Apache'),'&restore_config', $_);
   my $out = &restore_config($_);
   $j++;
  
   # add confmaster log
   &write_postinstall_log('web','restore_apache'.$j,$_) if $out;
  }
 }
 
 # disable Traffic Monitoring
 # require library from module virtual-server
 &foreign_require('virtual-server','virtual-server-lib.pl');
 if ($virtual_server::config{'bw_active'})
 {
  # save configuration
  $virtual_server::config{'bw_active'} = 0;
  &lock_file($virtual_server::module_config_file);
  &write_debug_log(&text('modify','Virtual Server'),'&virtual_server::save_module_config');
  &virtual_server::save_module_config();
  &unlock_file($virtual_server::module_config_file);

  # find bandwidth cron job
  # require library from module cron
  &foreign_require("cron", "cron-lib.pl");
  my @jobs = &cron::list_cron_jobs();
  my ($job) = grep { $_->{'user'} eq 'root' && $_->{'command'} eq $virtual_server::bw_cron_cmd; } @jobs;
  if ($job) 
  {
   # delete bandwidth cron job
   &lock_file(&cron::cron_file($job));
	&write_debug_log(&text('modify','Cron'),'&cron::delete_cron_job',$job);
   &cron::delete_cron_job($job);
	&unlock_file(&cron::cron_file($job));
  }
  # add confmaster log
  &write_postinstall_log('web','bw',1);
 }
 
 # add confmaster log
 &write_postinstall_log('web','apache',1);
}

# &ssl_setup()
# generate SSL certificate, modify sysconfig file, modify httpd file, copy ssl file
sub ssl_setup
{
 # generate_certificate
 &generate_certificate();
 
 unless (-e "$apache::config{'httpd_dir'}/conf/ssl.conf")
 {
  # ssl.conf copy
  my (undef, $out) = &copy_source_dest("$module_root_directory/addons/web/ssl.conf","$apache::config{'httpd_dir'}/conf/");
  &error($out) if $out;
 }
 # sysconfig file configuration
 my $conf = &parse_sysconfig_file();
 my $value = &get_sysconfig_value('HTTPDARGS',$conf);
 unless ($value =~ /^-[Dd]SSL$/)
 {
  &set_sysconfig_value('HTTPDARGS','"-DSSL"',$conf);
  &save_new_options($conf);
 }
 # apache configuration
 &http_configuration();
}

# &http_configuration()
# add ssl module call into httpd configuration file
sub http_configuration
{
 # clear apache module file
 my $site_file = ($apache::config{'webmin_apache'} || $apache::module_config_directory)."/site";
 my (undef, $out) = &unlink_file($site_file);
 &error($out) if $out;
 
 # re-require apache package
 undef (@apache::get_config_cache);
 undef ($main::done_foreign_require{'apache','apache-lib.pl'});
 &foreign_require('apache','apache-lib.pl');
 undef (@apache::get_config_cache);
 
 # get httpd configuration file
 my $conf = &apache::get_config();
 my ($load, $include);
 foreach (@{$conf})
 {
  # search ssl containers
  if (($_->{'name'} eq 'LoadModule') && ($_->{'value'} eq 'ssl_module modules/mod_ssl.so'))
  {
   $load = 1;
   next;
  }
  elsif (($_->{'name'} eq 'Include') && ($_->{'value'} eq "$apache::config{'httpd_dir'}/conf/ssl.conf"))
  {
   $include = 1;
   next;
  }
  last if ($load && $include);
 }
 if (!$load || !$include)
 {
  # add ssl containers into httpd configuration file
  &lock_file("$apache::config{'httpd_dir'}/conf/httpd.conf");
  my $lref = &read_file_lines("$apache::config{'httpd_dir'}/conf/httpd.conf");
  unless ($load)
  {
   push (@{$lref},"<IfDefine SSL>");
   push (@{$lref},"LoadModule ssl_module modules/mod_ssl.so");
   push (@{$lref},"</IfDefine>");
  }
  unless ($include)
  {
   push (@{$lref},"<IfModule mod_ssl.c>");
   push (@{$lref},"Include $apache::config{'httpd_dir'}/conf/ssl.conf");
   push (@{$lref},"</IfModule>");
  }
  &flush_file_lines();
  &unlock_file("$apache::config{'httpd_dir'}/conf/httpd.conf"); 
 }
}

# &generate_certificate()
# generates new self-signed SSL certificate using OpenSSL
sub generate_certificate 
{
 return undef if (-e "$apache::config{'httpd_dir'}/conf/certs/certprivkey.pem" && -e "$apache::config{'httpd_dir'}/conf/certs/cert.cert");

 # get hostname
 my $hostname = &get_system_hostname();
 
 # generate new certificate
 &open_execute_command ('CA', "openssl req -newkey rsa:512 -x509 -nodes -out cert.cert -keyout certprivkey.pem -days 1825 >/dev/null 2>&1");
 print CA ".\n";
 print CA ".\n";
 print CA ".\n";
 print CA " Apache Web Server\n";
 print CA ".\n";
 print CA "$hostname\n";
 print CA "postmaster\@$hostname\n";
 close(CA);	
  
 # chmod & move certificate
 chmod(0755, 'cert.cert', 'certprivkey.pem');
 my (undef, $out) = &copy_source_dest ('certprivkey.pem',"$apache::config{'httpd_dir'}/conf/certs/");
 &error($out) if $out;
 (undef, $out) = &unlink_file('certprivkey.pem');
 &error($out) if $out;
 (undef, $out) = &copy_source_dest ('cert.cert',"$apache::config{'httpd_dir'}/conf/certs/");
 &error($out) if $out;
 (undef, $out) = &unlink_file('cert.cert');
 &error($out) if $out;
 
 return undef;
}

# &php_setup()
# apache php support configure
sub php_setup
{
 undef (@apache::get_config_cache);
 my ($conf, $no_conf) = &apache::get_config();
 my $no_conf = 1;
 foreach (@{$conf})
 {
  # check include containers
  if (($_->{'name'} eq 'Include') && ($_->{'value'} eq "$apache::config{'httpd_dir'}/conf.d/*.conf"))
  {
   $no_conf = 0;
   last;
  }
 }
 if ($no_conf)
 {
  # add include containers into httpd configuration file
  &lock_file("$apache::config{'httpd_dir'}/conf/httpd.conf");
  my $lref = &read_file_lines("$apache::config{'httpd_dir'}/conf/httpd.conf");
  push (@{$lref},"Include $apache::config{'httpd_dir'}/conf.d/*.conf");
 }
 &flush_file_lines();
 &unlock_file("$apache::config{'httpd_dir'}/conf/httpd.conf"); 
 
 # httpd-php.conf copy
 my (undef, $out) = &copy_source_dest("$module_root_directory/addons/web/httpd-php.conf", "$apache::config{'httpd_dir'}/conf.d/");
 &error($out) if $out;
}

#------------------------------------- Additional funcs ------------------------------------

# &parse_sysconfig_file([file])
# return hash with name=value pairs from httpd sysconfig file or call &error
sub parse_sysconfig_file
{
 my $conf_file = $_[0] ? $_[0] : $apache::config{'defines_file'};
 open (INFL,$conf_file) || &error($text{'error_open'});
 my $conf;
 while (<INFL>)
 {
  chomp;
  &trunc_space($_);
  unless ($_=~/^#/ || $_=~/^$/)
  {
   my @str=split (/=/,$_);
   $str[0]=&trunc_space($str[0]);
   $str[1]=&trunc_space($str[1]);
   $conf -> {$str[0]} = $str[1];
  }
 }
 close INFL;
 return $conf;
}

# &get_sysconfig_value(name,\%hash)
# return value for name from hash
sub get_sysconfig_value
{
 return $_[1]->{$_[0]};
}

# &set_sysconfig_value(name,value,\%hash)
# set new value for name from hash
sub set_sysconfig_value
{
 $_[2]->{$_[0]}=$_[1];
}

# &save_new_options(\%hash,[file])
# save all new options from hash to sysconfig httpd file, change union options value, delete all other options
sub save_new_options
{
 # get config file
 my $conf_file = $_[1] ? $_[1] : $apache::config{'defines_file'};
  
 # parse config file
 &lock_file($conf_file);
 my $old_conf = &parse_sysconfig_file($conf_file);
 
 # get new options
 my @new;
 foreach (keys %{$_[0]})
 {
  unless (exists $old_conf->{$_}) { push (@new,$_); }
 }
 
 # get del & union options
 my (@del,@union);
 foreach (keys %{$old_conf})
 {
  if (exists $_[0]->{$_}) { push (@union,$_); }
  else { push (@del,$_); }
 }
 
 # read config file
 my $old_file= &read_file_lines($conf_file);
 
 # modify options
 foreach (@union)
 {
  foreach my $i(@{$old_file})
  {
   my @str_old=split (/=/,$i);
   my $str=&trunc_space($str_old[0]);
   $i="$_=$_[0]->{$_}" if ($str eq $_);
  }
 }
 
 # delete options
 foreach (@del)
 {
  my @temp;
  foreach my $i(@{$old_file})
  {
   my @str_old=split (/=/,$i);
   my $str=&trunc_space($str_old[0]);
   push (@temp,$i) unless ($str eq $_);
  }
  @{$old_file}=@temp;
 }
 
 # add new options
 foreach (@new)
 {
  my (@temp,$comment);
  foreach my $i (@{$old_file})
  {
   # add new options under comment
   push (@temp,$i);
   if (!$comment && $i=~/^#\s*$_\s*=/)
   {
    push (@temp,"$_=$_[0]->{$_}");
    $comment=1;
   }
  }
  # add new options in the end of file
  push (@temp,"$_=$_[0]->{$_}") if (!$comment);
  @{$old_file}=@temp;
 } 
 # save config file
 &flush_file_lines();
 &unlock_file($conf_file);
}

# &trunc_space(string)
# delete spaces from string
sub trunc_space
{
 local $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}

1;
