# courier-imap-lib.pl
# Common functions for courier-imap configuration

do '../web-lib.pl';
&init_config();
require '../ui-lib.pl';
# use Data::Dumper;

@mail_daemons_list = ('imap', 'imapssl', 'pop3', 'pop3ssl');
@daemons_list = (@mail_daemons_list, 'authlib');

@tls_protocols_list = ('SSL2', 'SSL3', 'TLS1');
@verifypeer_list = (
	'NONE', # - do not verify anything
	'PEER', # - verify the client certificate, if one's presented
	'REQUIREPEER' # - require a client certificate, fail if one's not presented
);

# get_courier_version()
# Returns Courier IMAP version
sub get_courier_version {
	local $out;
	local $rpm_opts = "-q courier-imap --queryformat \"\%\{VERSION\}\"";
	if (-x '/bin/rpm') {
		$out = `/bin/rpm $rpm_opts 2>/dev/null`;
		return $out if !$? && $out =~ /^[\d\.]+$/;
	}
	elsif (&has_command('rpm')) {
		$out = `rpm $rpm_opts 2>/dev/null`;
		return $out if !$? && $out =~ /^[\d\.]+$/;
	}
	return undef;
}

## --- Manage config file functions ---

# get_config(conf_file, [daemon])
# Returns a list of config file entries
sub get_config {
	local $key = $_[1] ? $_[1] : $_[0];
 if (!defined(@{$get_config_cache{"$key"}})) {
 	local $conf = ();
	local $lnum = 0;
	local ($sheader);
	open(CONF, $_[0]);
	while(<CONF>) {
		s/\r|\n//g;
		if (/^\s*##\s*NAME:\s*([a-zA-Z0-9\_]+)\s*:(\d+)\s*$/) {
			# Start of a section
			local $dir = { 'name' => $1,
				       'value' => undef,
				       'enabled' => 0,
				       'sectionname' => $1,
				       'sectionvalue' => $2,
				       'line' => $lnum };
			push(@$conf, $dir);
			$sheader = $dir;
			}
#  		elsif (/^\s*(#?)\s*([a-zA-Z0-9\_]+)\s*=\s*(.*)/) {
 		elsif (/^\s*(#?)\s*([a-zA-Z0-9\_]+)\s*=\s*(\"([^\"]*)\"|([^\"]*))/) {
			local $dir = { 'name' => $2,
				       'value' => $4 || $5,
				       'enabled' => !$1,
# 						 'sep' => $4 ? '"' : '',
				       'line' => $lnum };
# 			push(@get_config_cache, $dir);
			push(@$conf, $dir);
			}
		if ($sheader && ! /^\s*#/) {
			$sheader->{'eline'} = $lnum - 1;
			$sheader = undef;
		}
		$lnum++;
		}
	close(CONF);
	$get_config_cache{"$key"} = $conf;
 }
# return \@get_config_cache;
# return $conf;
return $get_config_cache{"$key"};
}

# find(name, &config, [disabled-mode], [sectionname-mode])
# Mode 0=enabled, 1=disabled, 2=both
sub find {
local ($name, $conf, $mode, $sname) = @_;
local @rv = grep { $_->{'name'} eq $name &&
		   ($mode == 0 && $_->{'enabled'} ||
		    $mode == 1 && !$_->{'enabled'} || $mode == 2) } @$conf;
# if (defined($sname)) {
# 	# If a section was requested, limit to it
# 	@rv = grep { $_->{'sectionname'} eq $name } @rv;
# 	}
@rv = grep { $_->{'sectionname'} eq $sname } @rv;
return wantarray ? @rv : $rv[0];
}

# get_value(name, &config, [disabled-mode])
# Mode 0=enabled, 1=disabled, 2=both
sub get_value {
local @rv = &find(@_);
if (wantarray) {
	return map { $_->{'value'} } @rv;
	}
else {
	return $rv[0]->{'value'};
	}
}

# get_real_value(name, &config, [disabled-mode])
# Returns the value of a parameter, with $ substitions done
sub get_real_value {
local $name = shift;
# print "GET REAL VALUE: @_<br>";
my $v = &get_value($name, @_);
$v =~ s/\$(\{([^\}]+)\}|([A-Za-z0-9\.\-\_]+))/get_real_value($2 || $3, @_)/ge;
# print "NEW VALUE: $v<br>";
return $v;
}

# get_packed_value(value, &variables)
# Returns replaced value to $ from variables list
sub get_packed_value {
	local ($value, $vars) = @_;
	local @values = split(/\s+/, $value);
	foreach $v (@{$vars}) {
		local $name = $v->[0];
		local @v = split(/\s+/, $v->[1]);
		local %found = ();
		map { $found{$_}++ } @v;
		local @orig = @values;
		@values = grep { !$found{$_} } @values;
		if (scalar(@values) == scalar(@orig) - scalar(@v)) {
			@values = ("\$$name", @values);
		}
		else {
			@values = @orig;
		}
	}
	return join(" ", @values);
}

# save_directive($conf_file, &conf, name|&dir, value)
# Updates one directive in the config file
sub save_directive {
local ($conf_file, $conf, $name, $value) = @_;
local $lref = &read_file_lines($conf_file);
local $dir = ref($name) ? $name : &find($name, $conf, 0);
local $sep = $value =~ /\s+|\$/ ? '"' : '';
local $newline = ref($name) ? "$name->{'name'}=$sep$value$sep" : "$name=$sep$value$sep";
if ($sname) {
	$newline = "  ".$newline;
	}
if ($dir && defined($value)) {
	# Updating some directive
	$lref->[$dir->{'line'}] = $newline;
	$dir->{'value'} = $value;
	}
elsif ($dir && !defined($value)) {
	# Deleting some directive
	splice(@$lref, $dir->{'line'}, 1);
	&renumber($conf, $dir->{'line'}, -1);
	@$conf = grep { $_ ne $dir } @$conf;
	}
elsif (!$dir && defined($value)) {
	# Adding some directive .. put it after the commented version, if any
	local $cmt = &find($name, $conf, 1, $name);
	if ($cmt) {
		# After comment
		splice(@$lref, $cmt->{'eline'}+1, 0, $newline);
		&renumber($conf, $cmt->{'eline'}+1, 1);
		push(@$conf, { 'name' => $name,
			       'value' => $value,
			       'line' => $cmt->{'eline'}+1,
			       'sectionname' => $sname,
			       'sectionvalue' => $svalue });
		}
	elsif ($sname) {
		# Put at end of section
		local @insect = grep { $_->{'sectionname'} eq $sname &&
				       $_->{'sectionvalue'} eq $svalue } @$conf;
		@insect || &error("Failed to find section $sname $svalue !");
		local $line = $insect[$#insect]->{'line'}+1;
		splice(@$lref, $line, 0, $newline);
		&renumber($conf, $line, 1);
		push(@$conf, { 'name' => $name,
			       'value' => $value,
			       'line' => $line,
			       'sectionname' => $sname,
			       'sectionvalue' => $svalue });
		}
	else {
		# Need to put at end
		push(@$lref, $newline);
		push(@$conf, { 'name' => $name,
			       'value' => $value,
			       'line' => scalar(@$lref)-1,
			       'sectionname' => $sname,
			       'sectionvalue' => $svalue });
		}
	}
}

# renumber(&conf, line, offset)
sub renumber {
local ($conf, $line, $offset) = @_;
foreach my $c (@$conf) {
	$c->{'line'} += $offset if ($c->{'line'} >= $line);
	}
}

# int_to_bool(value)
# Converts 0 to 'NO' and not zero to 'YES'
sub int_to_bool {
	return $_[0] ? 'YES' : 'NO';
}

# bool_to_int(value)
# Converts 'YES' to 1 and 'NO' 0
sub bool_to_int {
	return $_[0] =~ /^yes$/i ? 1 : 0;
}
## 

# get_daemon_internal_name(daemon, [ssl])
# daemon - imap imapssl, pop3, pop3ssl, authlib
# By default daemon_type = imap
sub get_daemon_internal_name {
	local $daemon = $_[0] ? $_[0] : 'imap';
	$daemon .= "ssl" if $_[1] && $daemon !~ /ssl$/;
	return $daemon;
}

# get_daemon_title(daemon)
# Returns daemon title
sub get_daemon_title {
	local $daemon = &get_daemon_internal_name($_[0]);
	return $text{"index_$daemon"};
}

# get_daemon_config_file(daemon)
# Returns a daemon config file name
sub get_daemon_config_file {
	local $daemon = &get_daemon_internal_name($_[0]);
	return $config{$daemon . "_conf"};
}

# get_daemon_exec_file(daemon)
# Returns a daemon executable file name
sub get_daemon_exec_file {
	local $daemon = &get_daemon_internal_name($_[0]);
	return $config{$daemon . "_daemon"};
}

# get_daemon_config(daemon)
# Returns a list of daemon config file entries
sub get_daemon_config {
	local $daemon = &get_daemon_internal_name($_[0]);
	local $conf_file = &get_daemon_config_file($daemon);
	($conf_file && -r $conf_file) || &error(&text('index_eopenconfig', &get_daemon_title($daemon), $conf_file));
	return &get_config($conf_file, $daemon);
}

# get_daemon_pid_file(daemon)
# Returns the PID if the server process is active, undef if not
sub get_daemon_pid_file {
	local $daemon = &get_daemon_internal_name($_[0]);
	local $pid_file = $config{$daemon . "_pid_file"};
	local ($dir, $name, $conf);
	unless ($pid_file) {
		$conf = &get_daemon_config($daemon);
		$conf || &error($text{'index_egetconfig'});
		if ($daemon eq 'authlib') {
			$dir = &get_value('authdaemonvar', $conf);
			$pid_file = "$dir/pid";
		}
		else {
			$name = $daemon =~ /ssl$/ ? 'SSLPIDFILE' : 'PIDFILE';
			$pid_file = &get_value($name, $conf);
		}
	}
	return $pid_file;
}

# is_daemon_running(daemon)
# Returns the PID if the server process is active, undef if not
sub is_daemon_running {
	# Try the configured PID file first
	local $pid =&check_pid_file(&get_daemon_pid_file($_[0]));
	return $pid;
}

# are_used_daemons_running()
# Returns 1 when ALL used daemons are running
sub are_used_daemons_running {
	foreach my $dmn (&get_used_daemons()) {
		return 0 unless &is_daemon_running($dmn);
	}
	return 1;
}

# get_used_daemons()
# Returns list of used daemons (what start independently)
sub get_used_daemons {
	local @rv = ();
	foreach my $daemon (@daemons_list) {
		push(@rv, $daemon) if $config{"$daemon\_start"};
	}
	return @rv;
}

# get_daemon_init(daemon)
# Returns the daemon's init script name from module config
sub get_daemon_init {
	local $daemon = &get_daemon_internal_name($_[0]);
	return $config{"$daemon\_init"};
}

# get_initscript(daemon)
# Returns the full path to the daemon init script
sub get_initscript {
	local $init = &get_daemon_init($_[0]);
	if ($init) {
		&foreign_require("init", "init-lib.pl");
		if ($init::init_mode eq "init") {
			return &init::action_filename($init);
			}
		}
	return undef;
}

# stop_daemon(daemon)
# Attempts to stop the daemon process, returning an error message or undef if successful
sub stop_daemon {
local $daemon = &get_daemon_internal_name($_[0]);
local $script = &get_initscript($daemon);
local $daemon_cmd = $config{"$daemon\_daemon"};
local $cmd = $script ? "$script stop" : -x "$daemon_cmd" ? "$daemon_cmd stop" : undef;
if ($cmd) {
	local $out = &backquote_logged("$cmd 2>&1");
	return $? ? "<pre>$out</pre>" : undef;
	}
else {
	local $pid = &is_daemon_running($daemon);
	if ($pid && kill('TERM', $pid)) {
		return undef;
		}
	else {
		return &text('stop_erunning', &get_daemon_title($daemon));
		}
	}
}

# stop_daemons_safe(daemon, [nosafe], [nostop])
# Attempts to stop the defined daemon process or all daemon processes, returning undef if successful,
# an error message or calling error function if nosafe flag is on
# If nostop is enabled it does not stop already not running daemons.
# if daemon eq 'all' or undef it stops all daemons
sub stop_daemons_safe {
	&error_setup($text{'stop_err'});
	local ($daemon, $nosafe, $nostop) = ($_[0], $_[1], $_[2]);
	local @rv = ();
	local @dlist = $daemon eq 'all' || ! $daemon ? &get_used_daemons() : (&get_daemon_internal_name($daemon));
	foreach my $dmn (@dlist) {
		local $err;
		$err = &stop_daemon($dmn) if !$nostop || &is_daemon_running($dmn);
		if ($err) {
			&error($err) if $nosafe;
			push(@rv, $err);
		}
	}
	return join("\n", @rv);
}

# start_daemon(daemon)
# Attempts to start the daemon process, returning an error message or undef if successful
# if daemon eq 'all' it starts all daemons
sub start_daemon {
local $daemon = &get_daemon_internal_name($_[0]);
local $script = &get_initscript($daemon);
local $daemon_cmd = $config{$daemon . "_daemon"};
local $cmd = $script ? "$script start" : 
			-x "$daemon_cmd" ? "$daemon_cmd start" : &error(&text('index_estart', &get_daemon_title($daemon)));
local $out = &backquote_logged("$cmd 2>&1");
return $? || $out =~/ERROR|FAILED/ ? "<pre>$out</pre>" : undef;
}

# start_daemons_safe(daemon, [nosafe], [nostart])
# Attempts to start the defined daemon process or all daemon processes, returning undef if successful,
# an error message or calling error function if nosafe flag is on.
# If nostart is enabled it does not start already running daemons.
# if daemon eq 'all' or undef it starts all daemons
sub start_daemons_safe {
	&error_setup($text{'start_err'});
	local ($daemon, $nosafe, $nostart) = ($_[0], $_[1], $_[2]);
	local @rv = ();
	local @dlist = $daemon eq 'all' || ! $daemon ? &get_used_daemons() : (&get_daemon_internal_name($daemon));
	foreach my $dmn (@dlist) {
		local $err;
		$err = &start_daemon($dmn) if !$nostart || !&is_daemon_running($dmn);
		if ($err) {
			&error($err) if $nosafe;
			push(@rv, $err);
		}
	}
	return join("\n", @rv);
}

# restart_daemon(daemon, [force])
# Stop and restart the daemon.
# Daemon will be started if it is not running and force is enabled 
sub restart_daemon {
local $daemon = &get_daemon_internal_name($_[0]);
local $pid = &is_daemon_running($daemon);
if ($pid) {
	local $err = &stop_daemon($daemon);
	return $err || &start_daemon($daemon);
	}
elsif ($_[1]) {
	return &start_daemon($daemon);
	}
else {
	return $text{'stop_erunning'};
	}
}

# restart_daemons_safe(daemon, [nosafe], [force])
# Attempts to restart the defined daemon process or all daemon processes, returning undef if successful,
# an error message or calling error function if nosafe flag is on
# if daemon eq 'all' or undef it restarts all daemons
sub restart_daemons_safe {
	&error_setup($text{'apply_err'});
	local ($daemon, $nosafe, $force) = ($_[0], $_[1], $_[2]);
	local @rv = ();
	local @dlist = $daemon eq 'all' || ! $daemon ? &get_used_daemons() : (&get_daemon_internal_name($daemon));
	foreach my $dmn (@dlist) {
		my $err = &restart_daemon($dmn, $force);
		if ($err) {
			&error($err) if $nosafe;
			push(@rv, $err);
		}
	}
	return join("\n", @rv);
}

# --- Cerificates management functions ---

# get_certificate_file(daemon)
# Returns certificate file path from daemon config file
sub get_certificate_file {
	local $daemon = &get_daemon_internal_name($_[0], 1);
	local $conf = &get_daemon_config($daemon);
	local $cert = &get_value('TLS_CERTFILE', $conf);
	return $cert;
}

# is_certificate_file(daemon)
# Returns certificate file path if it is present or undef
sub is_certificate_file {
	local $cert = &get_certificate_file($_[0]);
	if (-r $cert || $_[1]) {
		return $cert;
	}
	else {
		return undef;
	}
}

# get_certificate_info(cert_file)
# Returns a hash of details of a certificate file
sub get_certificate_info {
	local %rv;
	return undef if !&has_command("openssl");
	open(OUT, "openssl x509 -in $_[0] -issuer -subject -enddate |");
	while(<OUT>) {
		s/\r|\n//g;
		if (/subject=.*CN=([^\/]+)/) {
			$rv{'cn'} = $1;
		}
		if (/subject=.*O=([^\/]+)/) {
			$rv{'o'} = $1;
		}
		if (/issuer=.*CN=([^\/]+)/) {
			$rv{'issuer_cn'} = $1;
		}
		if (/issuer=.*O=([^\/]+)/) {
			$rv{'issuer_o'} = $1;
		}
		if (/notAfter=(.*)/) {
			$rv{'notafter'} = $1;
		}
	}
	close(OUT);
	$rv{'type'} = $rv{'o'} eq $rv{'issuer_o'} ? $text{'cert_typeself'} : $text{'cert_typeca'};
	return \%rv;
}

# get_gencert_script(daemon)
# Returns path to script what generates new SSL certificates
sub get_gencert_script {
	local $daemon = &get_daemon_internal_name($_[0], 1);
	return $config{$daemon . "_gencert"};
}

# get_gencert_script_pem(daemon)
# Returns path to pem file to be created by script what generates new SSL certificates
sub get_gencert_script_pem {
	local $gencert = &get_gencert_script($daemon);
	local $data = &read_file_contents($gencert);
	if ($data =~ /-keyout\s+([^\s]+)\s+/m) {
		return $1;
	}
	else {
		return undef;
	}
}

# generate_certificate(daemon)
# Generates new self-signed SSL certificate using internal script
# Returns undef if successful or an error message
sub generate_certificate {
	local $daemon = &get_daemon_internal_name($_[0], 1);
	local $gencert = &get_gencert_script($daemon);
	return $text{'cert_enogencert'} if !$gencert || !-x $gencert;
	local $data=".\n.\n.\nCourier Mail Server\n.\nlocalhost\npostmaster\@localhost\n";
	local ($rv, $out);
	$rv = &execute_command("$gencert", \$data, \$out, undef);
# 	local $out = &backquote_logged("$gencert 2>&1");
	return "<pre>$out</pre>" if $rv;
	if ($config{$daemon . "_gencp"}) {
		local $pemfile = &get_gencert_script_pem($daemon);
		local $certfile = &get_certificate_file($daemon);
		if ($pemfile && $certfile && $pemfile ne $certfile) {
			($rv, $out) = &copy_source_dest($pemfile, $certfile);
			&unlink_file($pemfile) if -f $pemfile; # always delete generated pem file as in some case we have an error
			return $rv ? undef : "<pre>$out</pre>";
		}
	}
	return undef;
}

# generate_certificates_safe([nosafe])
# Generates new self-signed SSL certificates for POP3 and IMAP using internal scripts,
# if ones have not been created yet.
# Returns undef if successful, an error message or calling error function if nosafe flag is on
sub generate_certificates_safe {
	&error_setup($text{'gencert_err'});
	local ($nosafe, $dmn, $err) = ($_[0]);
	local @rv = ();
	foreach $dmn ('pop3', 'imap') {
		if ( ! &is_certificate_file($dmn) ) {
			$err = &generate_certificate($dmn);
			if ($err) {
				&error($err) if $nosafe;
				push(@rv, $err);
			}
		}
	}
	return join("\n", @rv);
}

# --- UI functions ---

# default_anything_input(name, value, label, input)
# Returns HTML for inputing default from any input
sub default_anything_input {
	local ($name, $value, $label, $input) = @_;
	local $name_def = "$name\_def";
	return 
		&ui_oneradio($name_def, 1, $label, !$value) .
		&ui_oneradio($name_def, 0, '', $value) .
		$input;
}

# default_text_input(name, value, label, size)
sub default_text_input {
	local ($name, $value, $label, $size) = @_;
	local $name_def = "$name\_def";
	return 
		&ui_oneradio($name_def, 1, $label, !$value) .
		&ui_oneradio($name_def, 0, '', $value) .
		&ui_textbox($name, $value ? $value : '', $size);
}

# get_daemon_edit_link(daemon)
# Returns url to edit daemon page
sub get_daemon_edit_link {
	if ($_[0] =~ /^pop3/) {
		return "edit_pop3.cgi?dmn=$_[0]";
	}
	elsif ($_[0] =~ /^imap/) {
		return "edit_imap.cgi?dmn=$_[0]";
	}
	elsif ($_[0] =~ /^authlib/) {
		return "edit_authlib.cgi";
	}
	else {
		return "";
	}
}


1;
1;
