#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Saves POP3 / POP3 SSL daemon options

&ModuleLoader::preload_module('courier_imap_class', "courier_imap::CourierimapLib");
$self = new courier_imap::CourierimapLib:: 'courier_imap_class',%$self;

$self->ReadParse();

$self->{'dmn'} = $self->{'in_hash'}->{'dmn'};
$self->{'dmn'} eq 'pop3' || $self->{'dmn'} eq 'pop3ssl' || $self->error($self->{'text_hash'}->{'pop3_ein'});
$self->{'ssl'} = $self->{'dmn'} =~ /ssl/;

$self->{'cfile_pop3'} = $self->get_daemon_config_file('pop3');
$self->{'cfile_ssl'} = $self->get_daemon_config_file('pop3ssl');
$self->{'conf_pop3'} = $self->get_daemon_config('pop3');
$self->{'conf_ssl'} = $self->get_daemon_config('pop3ssl');

if ($self->{'ssl'}) {
	$self->{'conf'} = $self->{'conf_ssl'};
	$self->{'cfile'} = $self->{'cfile_ssl'};
}
else {
	$self->{'conf'} = $self->{'conf_pop3'};
	$self->{'cfile'} = $self->{'cfile_pop3'};
}

$self->error_setup($self->text('save_err', $self->get_daemon_title($self->{'dmn'})));

$self->lock_file($self->{'cfile_pop3'});
$self->lock_file($self->{'cfile_ssl'});

# Checks IP address
$self->{'in_hash'}->{'addr_def'} || $self->check_ipaddress($self->{'in_hash'}->{'addr'}) || $self->error($self->{'text_hash'}->{'pop3_eaddr'});
$self->{'address'} = $self->{'in_hash'}->{'addr_def'} ? '0' : $self->{'in_hash'}->{'addr'};
$self->save_directive($self->{'cfile'}, $self->{'conf'}, $self->{'ssl'} ? "SSLADDRESS" : "ADDRESS", $self->{'address'});

# Checks port
$self->{'port'} = $self->{'in_hash'}->{'port'};
($self->{'port'} =~ /^\d+$/ && $self->{'port'} >= 1 && $self->{'port'} <= 65535) || $self->error($self->{'text_hash'}->{'pop3_eport'});
$self->save_directive($self->{'cfile'}, $self->{'conf'}, $self->{'ssl'} ? "SSLPORT" : "PORT", $self->{'port'});

# Checks maximum servers started
$self->{'in_hash'}->{'maxd_def'} || $self->{'in_hash'}->{'maxd'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'pop3_emaxd'});
$self->{'maxd'} = $self->{'in_hash'}->{'maxd_def'} ? undef : $self->{'in_hash'}->{'maxd'};
$self->save_directive($self->{'cfile'}, $self->{'conf'}, "MAXDAEMONS", $self->{'maxd'});

# Checks maximum connections per IP
$self->{'in_hash'}->{'maxip_def'} || $self->{'in_hash'}->{'maxip'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'pop3_emaxip'});
$self->{'maxip'} = $self->{'in_hash'}->{'maxip_def'} ? undef : $self->{'in_hash'}->{'maxip'};
$self->save_directive($self->{'cfile'}, $self->{'conf'}, "MAXPERIP", $self->{'maxip'});

# Maildir directory name
$self->{'in_hash'}->{'maildir_def'} || $self->{'in_hash'}->{'maildir'} || $self->error($self->{'text_hash'}->{'pop3_emaildir'});
$self->{'maildir'} = $self->{'in_hash'}->{'maildir_def'} ? undef : $self->{'in_hash'}->{'maildir'};
$self->save_directive($self->{'cfile'}, $self->{'conf'}, "MAILDIRPATH", $self->{'maildir'});

if ($self->{'ssl'}) {
	# TSL Protocol
	$self->save_directive($self->{'cfile'}, $self->{'conf'}, "TLS_PROTOCOL", $self->{'in_hash'}->{'protocol'});

	# Certificate file path
	$self->save_directive($self->{'cfile'}, $self->{'conf'}, "TLS_CERTFILE", $self->{'in_hash'}->{'cert'});

	# Trusted Certificate file path / directory
	$self->save_directive($self->{'cfile'}, $self->{'conf'}, "TLS_TRUSTCERTS", $self->{'in_hash'}->{'trust'});

	# Verify client certificates
	$self->save_directive($self->{'cfile'}, $self->{'conf'}, "TLS_VERIFYPEER", $self->{'in_hash'}->{'vpeer'});

	# TLS cache file name
	$self->{'in_hash'}->{'cachesize'} || $self->error('pop3_ecache');
	$self->save_directive($self->{'cfile'}, $self->{'conf'}, "TLS_CACHEFILE", $self->{'in_hash'}->{'cache'});

	# TLS cache file size
	($self->{'in_hash'}->{'cachesize'} && $self->{'in_hash'}->{'cachesize'} =~ /^\d+$/) || $self->error('pop3_ecachesize');
	$self->save_directive($self->{'cfile'}, $self->{'conf'}, "TLS_CACHESIZE", $self->{'in_hash'}->{'cachesize'} * $self->{'in_hash'}->{'cachesize_units'});
}
else {
	# SASL capability
	$self->save_directive($self->{'cfile'}, $self->{'conf'}, "POP3AUTH", join(" ", split(/\0/, $self->{'in_hash'}->{'auth'})));

	# SASL SSL capability
	$self->save_directive($self->{'cfile'}, $self->{'conf'}, "POP3AUTH_TLS", join(" ", split(/\0/, $self->{'in_hash'}->{'authtls'})));

	# POP3 STLS Extenstion
	$self->save_directive($self->{'cfile_ssl'}, $self->{'conf_ssl'}, "POP3_STARTTLS", $self->int_to_bool($self->{'in_hash'}->{'tlsstart'}));

	# TSL Protocol
	$self->save_directive($self->{'cfile_ssl'}, $self->{'conf_ssl'}, "TLS_STARTTLS_PROTOCOL", $self->{'in_hash'}->{'protocol'});
}

$self->flush_file_lines();
$self->unlock_file($self->{'cfile_pop3'});
$self->unlock_file($self->{'cfile_ssl'});
$self->webmin_log("save", $self->{'dmn'});

if ($self->{'config_hash'}->{'auto_apply'}) {
	if ($self->is_daemon_running('pop3')) {
		$self->restart_daemon('pop3');
		$self->webmin_log('apply', 'pop3');
	}
	if ($self->is_daemon_running('pop3ssl')) {
		$self->restart_daemon('pop3ssl');
		$self->webmin_log('apply', 'pop3ssl');
	}
}
$self->redirect("");

1;
		
}


1;

