#!/usr/bin/perl
package cpan::CpanLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'cpan::CpanLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# cpan-lib.pl
# Functions for getting information about perl modules

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
use Config;
$self->{'packages_file'} = "$self->{'module_config_directory'}/packages.txt.gz";
$self->{'available_packages_cache'} = "$self->{'module_config_directory'}/available-cache";

# Get the paths to perl and perldoc
$self->{'perl_path'} = $self->get_perl_path();
if ($self->has_command("perldoc")) {
	$self->{'perl_doc'} = $self->has_command("perldoc");
	}
else {
	$self->{'perl_path'} =~ /^(.*)\/[^\/]+$/;
	if (-x "$1/perldoc") {
		$self->{'perl_doc'} = "$1/perldoc";
		}
	}
if ($self->{'perl_doc'} && $] >= 5.006) {
	$self->{'perl_doc'} = "$self->{'perl_path'} -T $self->{'perl_doc'}";
	}

1;

1;
		return $self;
}

# list_perl_modules([master-name])
# Returns a list of all installed perl modules, by reading .packlist files
sub list_perl_modules
{
	my $self = shift;

local ($limit) = @_;
local (@rv, %done, $d, %donedir, %donemod);
foreach $d ($self->expand_usr64($Config{'privlib'}),
	    $self->expand_usr64($Config{'sitelib_stem'} ? $Config{'sitelib_stem'} :
				      		    $Config{'sitelib'}),
	    $self->expand_usr64($Config{'sitearch_stem'} ? $Config{'sitearch_stem'} :
				      		    $Config{'sitearch'}),
	    $self->expand_usr64($Config{'vendorlib_stem'} ? $Config{'vendorlib_stem'} :
				        	      $Config{'vendorlib'}),
	    $self->expand_usr64($Config{'installprivlib'})) {
	next if (!$d);
	next if ($donedir{$d});
	local $f;
	open(FIND, "find '$d' -name .packlist -print |");
	while($f = <FIND>) {
		chop($f);
		local @st = stat($f);
		next if ($done{$st[0],$st[1]}++);
		local @st = stat($f);
		local $mod = { 'date' => scalar(localtime($st[9])),
			       'time' => $st[9],
			       'packfile' => $f,
			       'index' => scalar(@rv) };
		$f =~ /\/(([A-Z][^\/]*\/)*[^\/]+)\/.packlist$/;
		$mod->{'name'} = $1;
		$mod->{'name'} =~ s/\//::/g;
		next if ($limit && $mod->{'name'} ne $limit);
		next if ($donemod{$mod->{'name'}}++);

		# Add the files in the .packlist
		local (%donefile, $l);
		open(FILE, $f);
		while($l = <FILE>) {
			chop($l);
			$l =~ s/^\/tmp\/[^\/]+//;
			$l =~ s/^\/var\/tmp\/[^\/]+//;
			next if ($donefile{$l}++);
			if ($l =~ /\/((([A-Z][^\/]*\/)([^\/]+\/)?)?[^\/]+)\.pm$/) {
				local $mn = $1;
				$mn =~ s/\//::/g;
				push(@{$mod->{'mods'}}, $mn);
				push(@{$mod->{'files'}}, $l);
				}
			elsif ($l =~ /^([^\/]+)\.pm$/) {
				# Module name only, with no path! Damn redhat..
				local @rpath;
				open(FIND2, "find '$d' -name '$l' -print |");
				while(<FIND2>) {
					chop;
					push(@rpath, $_);
					}
				close(FIND2);
				@rpath = sort { length($a) cmp length($b) } @rpath;
				if (@rpath) {
					$rpath[0] =~ /\/(([A-Z][^\/]*\/)*[^\/]+)\.pm$/;
					local $mn = $1;
					$mn =~ s/\//::/g;
					push(@{$mod->{'mods'}}, $mn);
					push(@{$mod->{'files'}}, $rpath[0]);
					$mod->{'noremove'} = 1;
					$mod->{'noupgrade'} = 1;
					}
				}
			push(@{$mod->{'packlist'}}, $l);
			}
		close(FILE);
		local $mi = $self->indexof($mod->{'name'}, @{$mod->{'mods'}});
		$mod->{'master'} = $mi < 0 ? 0 : $mi;
		push(@rv, $mod) if (@{$mod->{'mods'}});
		}
	close(FIND);
	}

# Look for RPMs or Debs for Perl modules
if ($self->foreign_check("software") && $self->{'config_hash'}->{'incpackages'}) {
	$self->foreign_require("software", "software-lib.pl");
	if ($main::software_object->{'config_hash'}->{'package_system'} eq "rpm") {
		local $n = $main::software_object->list_packages();
		local $i;
		for($i=0; $i<$n; $i++) {
			# Create the module object
			next if ($main::software_object->{'packages_hash'}->{$i,'name'} !~
				 /^perl-([A-Z].*)$/ &&
				 $main::software_object->{'packages_hash'}->{$i,'name'} !~
				 /^([A-Z].*)-[pP]erl$/i);
			local $mod = { 'index' => scalar(@rv),
				       'pkg' => $main::software_object->{'packages_hash'}->{$i,'name'},
				       'pkgtype' => 'rpm',
				       'noupgrade' => 1,
				       'version' =>
					  $main::software_object->{'packages_hash'}->{$i,'version'} };
			$mod->{'name'} = $1;
			$mod->{'name'} =~ s/\-/::/g;
			next if ($limit && $mod->{'name'} ne $limit);
			next if ($donemod{$mod->{'name'}}++);

			# Add the files in the RPM
			local $fn = $main::software_object->check_files(
				$main::software_object->{'packages_hash'}->{$i,'name'},
				$main::software_object->{'packages_hash'}->{$i,'version'});
			local $fi;
			for($fi=0; $fi<$fn; $fi++) {
				local $l = $main::software_object->{'files_hash'}->{$fi,'path'};
				if ($l =~ /\/((([A-Z][^\/]*\/)([^\/]+\/)?)?[^\/]+)\.pm$/) {
					local $mn = $1;
					$mn =~ s/\//::/g;
					push(@{$mod->{'mods'}}, $mn);
					push(@{$mod->{'files'}}, $l);
					}
				push(@{$mod->{'packlist'}}, $l);
				if (!$mod->{'date'}) {
					local @st = stat($l);
					$mod->{'date'} = scalar(localtime($st[9]));
					$mod->{'time'} = $st[9];
					}
				}

			local $mi = $self->indexof($mod->{'name'}, @{$mod->{'mods'}});
			$mod->{'master'} = $mi < 0 ? 0 : $mi;
			push(@rv, $mod) if (@{$mod->{'mods'}});
			}
		}
	elsif ($main::software_object->{'config_hash'}->{'package_system'} eq "debian") {
		# Look for Debian packages of Perl modules
		local $n = $main::software_object->list_packages();
		local $i;
		for($i=0; $i<$n; $i++) {
			# Create the module object
			next if ($main::software_object->{'packages_hash'}->{$i,'name'} !~
				 /^lib(\S+)-perl$/);
			local $mod = { 'index' => scalar(@rv),
				       'pkg' => $main::software_object->{'packages_hash'}->{$i,'name'},
				       'pkgtype' => 'debian',
				       'noupgrade' => 1,
				       'version' =>
					  $main::software_object->{'packages_hash'}->{$i,'version'} };

			# Add the files in the RPM
			local $fn = $main::software_object->check_files(
				$main::software_object->{'packages_hash'}->{$i,'name'},
				$main::software_object->{'packages_hash'}->{$i,'version'});
			local $fi;
			for($fi=0; $fi<$fn; $fi++) {
				local $l = $main::software_object->{'files_hash'}->{$fi,'path'};
				if ($l =~ /\/((([A-Z][^\/]*\/)([^\/]+\/)?)?[^\/]+)\.pm$/) {
					local $mn = $1;
					$mn =~ s/\//::/g;
					push(@{$mod->{'mods'}}, $mn);
					push(@{$mod->{'files'}}, $l);
					}
				push(@{$mod->{'packlist'}}, $l);
				if (!$mod->{'date'}) {
					local @st = stat($l);
					$mod->{'date'} = scalar(localtime($st[9]));
					$mod->{'time'} = $st[9];
					}
				}
			next if (!@{$mod->{'mods'}});

			# Work out the name
			foreach my $m (@{$mod->{'mods'}}) {
				local $pn = lc($m);
				$pn =~ s/::/-/g;
				$pn = "lib".$pn."-perl";
				if ($pn eq $mod->{'pkg'}) {
					$mod->{'name'} = $m;
					last;
					}
				}
			$mod->{'name'} ||= $mod->{'mods'}->[0];
			next if ($limit && $mod->{'name'} ne $limit);
			next if ($donemod{$mod->{'name'}}++);

			local $mi = $self->indexof($mod->{'name'}, @{$mod->{'mods'}});
			$mod->{'master'} = $mi < 0 ? 0 : $mi;
			push(@rv, $mod) if (@{$mod->{'mods'}});
			}

		}
	}

return @rv;
}

# expand_usr64(dir)
# If a directory is like /usr/lib and /usr/lib64 exists, return them both
sub expand_usr64
{
	my $self = shift;

if ($_[0] =~ /^(\/usr\/lib\/|\/usr\/local\/lib\/)(.*)$/) {
	local ($dir, $dir64, $rest) = ($1, $1, $2);
	$dir64 =~ s/\/lib\//\/lib64\//;
	return -d $dir64 ? ( $dir.$rest, $dir64.$rest ) : ( $dir.$rest );
	}
else {
	return ( $_[0] );
	}
}

# module_desc(&mod, index)
# Returns a one-line description for some module, and a version number
sub module_desc
{
	my $self = shift;

local ($in_name, $desc);
local $f = $_[0]->{'files'}->[$_[1]];
local $pf = $f;
local $ver = $_[0]->{'version'};
$pf =~ s/\.pm$/\.pod/;
local ($got_version, $got_name);
open(MOD, $pf) || open(MOD, $f);
while(<MOD>) {
	if (/^=head1\s+name/i && !$got_name) {
		$in_name = 1;
		}
	elsif (/^=/ && $in_name) {
		$got_name++;
		$in_name = 0;
		}
	elsif ($in_name) {
		$desc .= $_;
		}
	if (/^\s*(our\s+)?\$VERSION\s*=\s*"([0-9\.]+)"/ ||
	    /^\s*(our\s+)?\$VERSION\s*=\s*'([0-9\.]+)'/ ||
	    /^\s*(our\s+)?\$VERSION\s*=\s*([0-9\.]+)/) {
		$ver = $2;
		$got_version++;
		}
	last if ($got_version && $got_name);
	}
close(MOD);
local $name = $_[0]->{'mods'}->[$_[1]];
$desc =~ s/^\s*$name\s+\-\s+// ||
	$desc =~ s/^\s*\S*<$name>\s+\-\s+//;
$desc =~ s/\$Id:.*\$//;
return wantarray ? ($desc, $ver) : $desc;
}

# download_packages_file(&callback)
sub download_packages_file
{
	my $self = shift;

$self->{'config_hash'}->{'packages'} =~ /^http:\/\/([^\/]+)(\/.*)$/ ||
	$self->error($self->{'text_hash'}->{'download_epackages'});
local ($host, $page, $port) = ($1, $2, 80);
if ($host =~ /^(.*):(\d+)$/) { $host = $1; $port = $2; }
$self->http_download($host, $port, $page, $self->{'packages_file'}, undef, $_[0]);
}

# remove_module(&module)
# Delete some perl module, and all sub-modules
sub remove_module
{
	my $self = shift;

local ($mod) = @_;
if ($mod->{'pkg'}) {
	$self->foreign_require("software", "software-lib.pl");
	return $main::software_object->delete_package($mod->{'pkg'});
	}
else {
	unlink(@{$mod->{'packlist'}});
	unlink($mod->{'packfile'});
	return undef;
	}
}

# get_recommended_modules()
# Returns a list of Perl modules used by other Webmin modules
sub get_recommended_modules
{
	my $self = shift;

local (@rv, %done);
foreach my $m ($self->get_all_module_infos()) {
	next if (!$m->{'cpan'});
	next if (!$self->foreign_installed($m->{'dir'}));
	local $mdir = $self->module_root_directory($m->{'dir'});
	next if (!-r "$mdir/cpan_modules.pl");
	$self->foreign_require($m->{'dir'}, "cpan_modules.pl");
	foreach my $c ($self->foreign_call($m->{'dir'}, "cpan_recommended")) {
		if (!$done{$c}++) {
			push(@rv, [ $c, $m ]);
			}
		}
	}
return sort { $a->[0] cmp $b->[0] } @rv;
}

# list_packaged_modules([refresh])
# Returns a list of Perl modules that can be installed from the system's
# package update service (ie YUM or APT).
sub list_packaged_modules
{
	my $self = shift;

local ($refresh) = @_;
return ( ) if (!$self->foreign_check("software") || !$self->{'config_hash'}->{'incpackages'});
$self->foreign_require("software", "software-lib.pl");
return (  ) if (!$main::software_object->{'update_system'});
local @avail;
local @st = stat($self->{'available_packages_cache'});
if ($refresh || !@st || $st[9] < time()-24*60*60) {	# Keep cache for a day
	# Need to refresh
	@avail = $main::software_object->update_system_available();
	open(CACHE, ">$self->{'available_packages_cache'}");
	print CACHE $self->serialise_variable(\@avail);
	close(CACHE);
	}
else {
	# Can use cache
	local $avail = $self->unserialise_variable(
			$self->read_file_contents($self->{'available_packages_cache'}));
	@avail = @$avail;
	}
local @rv;
foreach my $a (@avail) {
	if ($a->{'name'} =~ /^lib(\S+)-perl$/ ||	# Debian
	    $a->{'name'} =~ /^perl-(\S+)$/) {		# Redhat
		local $mod = $1;
		$mod =~ s/-/::/g;
		push(@rv, { 'mod' => $mod,
			    'package' => $a->{'name'},
			    'version' => $a->{'version'}, });
		}
	}
return @rv;
}


1;

