#!/usr/bin/perl
# Upgrade CP+ if possible

require './cpplus-lib.pl';
require './gnupg-lib.pl';

# Save this CGI from being killed by the upgrade
$SIG{'TERM'} = 'IGNORE';

#%in
&ReadParseMime();
my $version=$in{'version'};
my $dir=$in{'dir'};

#print header

$| = 1;
$theme_no_table = 1;
$need_unlink = 1;
&ui_print_header(undef, $text{'upgrade_title'}, "");

#source mode
if ($in{'source'} == 0)
{
 # from local file
 &error_setup(&text('upgrade_err1', $in{'file'}));
 $file = $in{'file'};
 if (!(-r $file)) { &inst_error($text{'upgrade_efile'}); }
 print "$text{'upgrade_nocheck'}<p>\n";
}
if ($in{'source'} == 2)
{
 #check upgrade version
 if (&base_version($version) eq &get_webmin_base_version())
 {
  &inst_error(&text('upgrade_elatest'));
 }
 elsif (&base_version($version) lt &get_webmin_base_version())
 {
  &inst_error(&text('upgrade_eversion', $version));
 }
 #check mode
 $file = &tempname();
 if ($in{'mode'} eq 'rpm')
 {
  # Downloading RPM
  my $upg_page = $tconfig{'upgrade_page'} ? $tconfig{'upgrade_page'} : "/download/rpm";
  $progress_callback_url = "http://$update_host$upg_page/cpplus-$version-1.noarch.rpm";
  &http_download($update_host, $update_port,"$upg_page/cpplus-$version-1.noarch.rpm", $file,
	  							\$error, \&progress_callback);
 }
 else
 {
  # Downloading tar.gz file
  my $upg_page = $tconfig{'upgrade_page'} ? $tconfig{'upgrade_page'} : "/download";
  $progress_callback_url = "http://$update_host$upg_page/cpplus-$version.tar.gz";
  &http_download($update_host, $update_port, "$upg_page/cpplus-$version.tar.gz", $file,
								\$error, \&progress_callback);
 }
 #print errors
 $error && &inst_error($error);

 # Check the package signature
 ($ec, $emsg) = &gnupg_setup();
 if (!$ec)
 {
  if ($mode eq 'rpm')
  {
   # Use rpm's gpg signature verification
   local $out = `rpm --checksig $file 2>&1`;
   if ($?)
   {
    $ec = 3;
    $emsg = &text('upgrade_echecksig',"<pre>$out</pre>");
   }
  }
  else
  {
   # Do a manual signature check
   # Download the key for this tar.gz
   local ($sigtemp, $sigerror);
   my $upg_page = $tconfig{'upgrade_page'} ? $tconfig{'upgrade_page'} : "/download";
   &http_download($update_host, $update_port, "$upg_page/sigs/cpplus-$version.tar.gz-sig.asc", 										\$sigtemp, \$sigerror);
   if ($sigerror)
   {
    $ec = 4;
    $emsg = &text('upgrade_edownsig', $sigerror);
   }
   else
   {
    local $data = `cat $file`;
    local ($vc, $vmsg) = &verify_data($data, $sigtemp);
    if ($vc > 1)
    {
     $ec = 3;
     $emsg = &text("upgrade_everify$vc", &html_escape($vmsg));
    }
   }
  }
 }
 # Tell the user about any GnuPG error
 if ($ec) {&inst_error($emsg);}
 elsif ($emsg) { print "$emsg<p>\n";}
 else { print "$text{'upgrade_sigok'}<p>\n";}
}
#install package
if ($in{'mode'} eq 'rpm') { &install_rpm(); }
else { &install_package(); }
&webmin_log("upgrade", undef, undef, { 'version' => $version, 'mode' => $in{'mode'} });

#Find out about any updates for this new version.
if ($updates_error) { print "<br>",&text('upgrade_eupdates', $updates_error),"<p>\n"; }

#print return url
&ui_print_footer($text{'index_mainpage_url'}, $text{'index_mainpage'});


sub install_package
{

 #gunzip the file if needed
 open(FILE, $file);
 read(FILE, $two, 2);
 close(FILE);
 if ($two eq "\037\213")
 {
  if (!&has_command("gunzip"))
  {
   &inst_error($text{'upgrade_egunzip'});
  }
  $newfile = &tempname();
  $out = `gunzip -c $file 2>&1 >$newfile`;
  if ($?)
  {
   unlink($newfile);
   &inst_error(&text('upgrade_egzip', "<tt>$out</tt>"));
  }
  unlink($file) if ($need_unlink);
  $need_unlink = 1;
  $file = $newfile;
 }
 # Check if it is a cpplus tarfile
 open(TAR, "tar tf $file 2>&1 |");
 while(<TAR>)
 {
  s/\r|\n//g;
  if (/^cpplus-([0-9\.]+)\//)
  {
   $version = $1;
  }
  if (/^usermin-([0-9\.]+)\//)
  {
   $usermin_version = $1;
  }
  if (/^[^\/]+\/(\S+)$/)
  {
   $hasfile{$1}++;
  }
  if (/^(cpplus-([0-9\.]+)\/[^\/]+)$/)
  {
   push(@topfiles, $1);
  }
  elsif (/^cpplus-[0-9\.]+\/([^\/]+)\//)
  {
   $intar{$1}++;
  }
 }
 close(TAR);
 if ($usermin_version)
 {
  &inst_error(&text('upgrade_eusermin', $usermin_version));
 }
 if (!$version)
 {
  if ($hasfile{'module.info'})
  {
   &inst_error(&text('upgrade_emod', 'edit_mods.cgi'));
  }
  else
  {
   &inst_error($text{'upgrade_etar'});
  }
 }
 if (!$force)
 {
  if (&base_version($version) eq &get_webmin_base_version())
  {
   &inst_error(&text('upgrade_elatest', $version));
  }
  elsif (&base_version($version) lt &get_webmin_base_version())
  {
   &inst_error(&text('upgrade_eversion', $version));
  }
 }
 # Work out where to extract
 if ($dir)
 {
  # Since we are currently installed in a fixed directory,
  # just extract to a temporary location
  $extract = &tempname();
  mkdir($extract, 0755);
 }
 else
 {
  # Next to the current directory
  $extract = "../..";
 }
 # Do the extraction of the tar file, and run setup.sh
 $| = 1;
 # Extract the whole file
 $out = `cd $extract ; tar xf $file 2>&1 >/dev/null`;
 if ($?) {&inst_error(&text('upgrade_euntar', "<tt>$out</tt>"));}
 unlink($file) if ($need_unlink);
 $ENV{'config_dir'} = $config_directory;
 $ENV{'webmin_upgrade'} = 1;
 $ENV{'autothird'} = 1;
 $ENV{'tempdir'} = $gconfig{'tempdir'};
 $ENV{'deletedold'} = 1 if ($in{'delete'});
 print "<p>",$text{'upgrade_setup'},"<p>\n";
 print "<pre>";
 $setup = $dir ? "./setup.sh '$dir'" : "./setup.sh";
 &proc::safe_process_exec("cd $extract/cpplus-$version && $setup", 0, 0, STDOUT, undef, 1, 1);
 print "</pre>\n";
 if (!$?)
 {
  if ($delete)
  {
   # Can delete the old root directory
   system("rm -rf \"$root_directory\"");
  }
  elsif ($dir)
  {
   # Can delete the temporary source directory
   system("rm -rf \"$extract\"");
  }
 }
}
sub install_rpm
{
 # Check if it is an RPM package
 $out = `rpm -qp $file`;
 $out =~ /(^|\n)cpplus-([0-9\.]+)/ || &inst_error($text{'upgrade_erpm'});
 $version = $2;
 if (!$force)
 {
  if (&base_version($version) eq &get_webmin_base_version())
  {
   &inst_error(&text('upgrade_elatest', $version));
  }
  elsif (&base_version($version) lt &get_webmin_base_version())
  {
   &inst_error(&text('upgrade_eversion', $version));
  }
 }
 # Install the RPM
 $ENV{'tempdir'} = $gconfig{'tempdir'};
 print "<p>",$text{'upgrade_setuprpm'},"<p>\n";
 print "<pre>";
 &proc::safe_process_exec("rpm -U --ignoreos --ignorearch '$file'", 0, 0, STDOUT, undef, 1, 1);
 unlink($file) if ($need_unlink);
 print "</pre>\n";
}

sub inst_error
{
 unlink($file) if ($need_unlink);
 unlink($updatestemp);
 if ($whatfailed) {print "<b>$whatfailed : $_[0]</b> <p>\n";}
 else {print "<b>$_[0]</b> <p>\n";}
 &ui_print_footer($text{'index_mainpage_url'}, $text{'index_mainpage'});
 exit;
}

