#!/usr/bin/perl
package cppluslog::CppluslogLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'cppluslog::CppluslogLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# cppluslog-lib.pl
# XXX file rollback
#	XXX should we capture all files, or just those changed?
#	XXX what about commands?

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();
%{$self->{'access_mods_hash'}} = map { $_, 1 } split(/\s+/, $self->{'access_hash'}->{'mods'});
%{$self->{'access_users_hash'}} = map { $_, 1 } split(/\s+/, $self->{'access_hash'}->{'users'});

1;

1;
		return $self;
}

sub parse_logline
{
	my $self = shift;

if ($_[0] =~ /^(\d+)\.(\S+)\s+\[.*\]\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+"([^"]+)"\s+"([^"]+)"\s+"([^"]+)"(.*)/ ||
    $_[0] =~ /^(\d+)\.(\S+)\s+\[.*\]\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)(.*)/) {
	local $rv = { 'time' => $1, 'id' => "$1.$2",
		      'user' => $3, 'sid' => $4,
		      'ip' => $5, 'module' => $6,
		      'script' => $7, 'action' => $8,
		      'type' => $9, 'object' => $10 };
	local %param;
	local $p = $11;
	while($p =~ /^\s*([^=\s]+)='([^']*)'(.*)$/) {
		if (defined($param{$1})) {
			$param{$1} .= "\0".$2;
			}
		else {
			$param{$1} = $2;
			}
		$p = $3;
		}
	foreach $k (keys %param) {
		$param{$k} =~ s/%(..)/pack("c",hex($1))/ge;
		}
	$rv->{'param'} = \%param;
	if ($rv->{'script'} =~ /^(\S+):(\S+)$/) {
		$rv->{'script'} = $2;
		$rv->{'webmin'} = $1;
		}
	return $rv;
	}
else {
	return undef;
	}
}

# list_diffs(&action)
# Returns details of file changes made by this action
sub list_diffs
{
	my $self = shift;

local $i = 0;
local @rv;
local $base = "$ENV{'WEBMIN_VAR'}/diffs/$self->{'act'}->{'id'}";
local @files = $self->expand_base_dir($base);

# Read the diff files
foreach my $file (@files) {
        local ($type, $object, $diff, $input);
	open(DIFF, $file);
        local $line = <DIFF>;
        while(<DIFF>) { $diff .= $_; }
        close(DIFF);
	if ($line =~ /^(\/.*)/) {
                $type = 'modify'; $object = $1;
                }
        elsif ($line =~ /^(\S+)\s+(.*)/) {
                $type = $1; $object = $2;
                }
	if ($type eq "exec") {
		open(INPUT, $file.".input");
		while(<INPUT>) {
			$input .= $_;
			}
		close(INPUT);
		}
	push(@rv, { 'type' => $type,
		    'object' => $object,
		    'diff' => $diff,
		    'input' => $input } );
	$i++;
	}
return @rv;
}

# list_files(&action)
# Returns details of original files before this action was taken
sub list_files
{
	my $self = shift;

local $i = 0;
local @rv;
local $base = "$ENV{'WEBMIN_VAR'}/files/$self->{'act'}->{'id'}";
local @files = $self->expand_base_dir($base);
foreach my $file (@files) {
        local ($type, $object, $data);
	open(FILE, $file);
        local $line = <FILE>;
	$line =~ s/\r|\n//g;
        while(<FILE>) { $data .= $_; }
        close(FILE);
	if ($line =~ /^(\S+)\s+(.*)/) {
		$type = $1;
		$file = $2;
		}
	elsif ($line =~ /^\s+(.*)/) {
		$type = -1;
		$file = $1;
		}
	else {
		next;
		}
	push(@rv, { 'type' => $type,
		    'file' => $file,
		    'data' => $data });
	$i++;
	}
return @rv;
}

# expand_base_dir(base)
# Finds files either under some dir, or starting with some path
sub expand_base_dir
{
	my $self = shift;

local ($base) = @_;
local @files;
if (-d $base) {
	# Find files in the dir
	opendir(DIR, $base);
	@files = map { "$base/$_" } sort { $a <=> $b }
			grep { $_ =~ /^\d+$/ } readdir(DIR);
	closedir(DIR);
	}
else {
	# Files are those that start with id
	$self->{'i'} = 0;
	while(-r "$base.$self->{'i'}") {
		push(@files, "$base.$self->{'i'}");
		$self->{'i'}++;
		}
	}
return @files;
}

# can_user(username)
sub can_user
{
	my $self = shift;

return $self->{'access_users_hash'}->{'*'} || $self->{'access_users_hash'}->{$_[0]};
}

# can_mod(module)
sub can_mod
{
	my $self = shift;

return $self->{'access_mods_hash'}->{'*'} || $self->{'access_mods_hash'}->{$_[0]};
}

# get_action(id)
# Returns the structure for some action
sub get_action
{
	my $self = shift;

local %index;
$self->build_log_index(\%index);
local $act;
open(LOG, $self->{'webmin_logfile'});
local @idx = split(/\s+/, $index{$_[0]});
seek(LOG, $idx[0], 0);
local $line = <LOG>;
local $act = $self->parse_logline($line);
close(LOG);
return $act->{'id'} eq $_[0] ? $act : undef;
}

# build_log_index(&index)
# Updates the given hash with mappings between action IDs and file positions
sub build_log_index
{
	my $self = shift;

local ($index) = @_;
local $ifile = "$self->{'module_config_directory'}/logindex";
dbmopen(%$index, $ifile, 0600);
local @st = stat($self->{'webmin_logfile'});
if ($st[9] > $index->{'lastchange'}) {
	# Log has changed .. perhaps need to rebuild
	open(LOG, $self->{'webmin_logfile'});
	if ($index->{'lastsize'} && $st[7] >= $index->{'lastsize'}) {
		# Gotten bigger .. just add new lines
		seek(LOG, $index->{'lastpos'}, 0);
		}
	else {
		# Smaller! Need to rebuild from start
		%$index = ( 'lastpos' => 0 );
		}
	while(<LOG>) {
		if ($self->{'act'} = $self->parse_logline($_)) {
			$index->{$self->{'act'}->{'id'}} = $index->{'lastpos'}." ".
						 $self->{'act'}->{'time'}." ".
						 $self->{'act'}->{'user'}." ".
						 $self->{'act'}->{'module'}." ".
						 $self->{'act'}->{'sid'};
			}
		$index->{'lastpos'} += length($_);
		}
	close(LOG);
	$index->{'lastsize'} = $st[7];
	$index->{'lastchange'} = $st[9];
	}
}


1;

