#!/usr/bin/perl
package cron::CronLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'cron::CronLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# cron-lib.pl
# Common crontab functions
# XXX support for envs in /etc/crontab and /etc/cron.d (impossible!)

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();
$self->{'env_support'} = $self->{'config_hash'}->{'vixie_cron'};
if ($self->{'module_info_hash'}->{'usermin'}) {
	$self->{'single_user'} = $self->{'remote_user'};
	$self->switch_to_remote_user();
	$self->create_user_config_dirs();
	$self->{'range_cmd'} = "$self->{'user_module_config_directory'}/range.pl";
	}
else {
	$self->{'range_cmd'} = "$self->{'module_config_directory'}/range.pl";
	}
$self->{'cron_temp_file'} = $self->transname();
use Time::Local;

@{$self->{'cron_month_array'}} = ( 'jan', 'feb', 'mar', 'apr', 'may', 'jun',
		'jul', 'aug', 'sep', 'oct', 'nov', 'dec' );
@{$self->{'cron_weekday_array'}} = ( 'sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat' );

1;

1;
		return $self;
}

# list_cron_jobs()
# Returns a lists of structures of all cron jobs
sub list_cron_jobs
{
	my $self = shift;

local (@rv, $lnum, $f);
if (defined(@{$self->{'cron_jobs_cache_array'}})) {
	return @{$self->{'cron_jobs_cache_array'}};
	}

# read the master crontab file
if ($self->{'config_hash'}->{'system_crontab'}) {
	$lnum = 0;
	$self->open_readfile(*TAB, $self->{'config_hash'}->{'system_crontab'});
	while(<TAB>) {
		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(\S+)\s+(.*)/i) {
			# A normal h m s d w time
			push(@rv, { 'file' => $self->{'config_hash'}->{'system_crontab'},
				    'line' => $lnum,
				    'type' => 1,
				    'nolog' => $2,
				    'active' => !$1,
				    'mins' => $3, 'hours' => $4,
				    'days' => $5, 'months' => $6,
				    'weekdays' => $8, 'user' => $10,
				    'command' => $11,
				    'index' => scalar(@rv) });
			if ($rv[$#rv]->{'user'} =~ /^\//) {
				# missing the user, as in redhat 7 !
				$rv[$#rv]->{'command'} = $rv[$#rv]->{'user'}.
					' '.$rv[$#rv]->{'command'};
				$rv[$#rv]->{'user'} = 'root';
				}
			$self->fix_names($rv[$#rv]);
			}
		elsif (/^(#+)?\s*@([a-z]+)\s+(\S+)\s+(.*)/i) {
			# An @ time
			push(@rv, { 'file' => $self->{'config_hash'}->{'system_crontab'},
				    'line' => $lnum,
				    'type' => 1,
				    'active' => !$1,
				    'special' => $2,
				    'user' => $3,
				    'command' => $4,
				    'index' => scalar(@rv) });
			}
		$lnum++;
		}
	close(TAB);
	}

# read package-specific cron files
opendir(DIR, $self->translate_filename($self->{'config_hash'}->{'cronfiles_dir'}));
while($f = readdir(DIR)) {
	next if ($f =~ /^\./);
	$lnum = 0;
	$self->open_readfile(*TAB, "$self->{'config_hash'}->{'cronfiles_dir'}/$f");
	while(<TAB>) {
		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(\S+)\s+(.*)/i) {
			push(@rv, { 'file' => "$self->{'config_hash'}->{'cronfiles_dir'}/$f",
				    'line' => $lnum,
				    'type' => 2,
				    'active' => !$1,
				    'nolog' => $2,
				    'mins' => $3, 'hours' => $4,
				    'days' => $5, 'months' => $6,
				    'weekdays' => $8, 'user' => $10,
				    'command' => $11,
				    'index' => scalar(@rv) });
			$self->fix_names($rv[$#rv]);
			}
		elsif (/^(#+)?\s*@([a-z]+)\s+(\S+)\s+(.*)/i) {
			push(@rv, { 'file' => "$self->{'config_hash'}->{'cronfiles_dir'}/$f",
				    'line' => $lnum,
				    'type' => 2,
				    'active' => !$1,
				    'special' => $2,
				    'user' => $3,
				    'command' => $4,
				    'index' => scalar(@rv) });
			}
		$lnum++;
		}
	close(TAB);
	}
closedir(DIR);

# Read a single user's crontab file
if ($self->{'config_hash'}->{'single_file'}) {
	$self->open_readfile(*TAB, $self->{'config_hash'}->{'single_file'});
	$lnum = 0;
	while(<TAB>) {
		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(.*)/i) {
			# A normal m h d m wd time
			push(@rv, { 'file' => $self->{'config_hash'}->{'single_file'},
				    'line' => $lnum,
				    'type' => 3,
				    'active' => !$1, 'nolog' => $2,
				    'mins' => $3, 'hours' => $4,
				    'days' => $5, 'months' => $6,
				    'weekdays' => $8, 'user' => "NONE",
				    'command' => $10,
				    'index' => scalar(@rv) });
			$self->fix_names($rv[$#rv]);
			}
		elsif (/^(#+)?\s*([a-zA-Z0-9\_]+)\s*=\s*'([^']*)'/ ||
		       /^(#+)?\s*([a-zA-Z0-9\_]+)\s*=\s*"([^']*)"/ ||
		       /^(#+)?\s*([a-zA-Z0-9\_]+)\s*=\s*(\S+)/) {
			# An environment variable
			push(@rv, { 'file' => $self->{'config_hash'}->{'single_file'},
				    'line' => $lnum,
				    'active' => !$1,
				    'name' => $2,
				    'value' => $3,
				    'user' => "NONE",
				    'index' => scalar(@rv) });
			}
		$lnum++;
		}
	close(TAB);
	}


# read per-user cron files
local $fcron = ($self->{'config_hash'}->{'cron_dir'} =~ /\/fcron$/);
local @users;
if ($self->{'single_user'}) {
	@users = ( $self->{'single_user'} );
	}
else {
	opendir(DIR, $self->translate_filename($self->{'config_hash'}->{'cron_dir'}));
	@users = grep { !/^\./ } readdir(DIR);
	closedir(DIR);
	}
foreach $f (@users) {
  if (!$self->{'config_hash'}->{'file_suf'} || $f =~/.*$self->{'config_hash'}->{'file_suf'}$/)
  {
	# for Trustix
	$f =~ s/$self->{'config_hash'}->{'file_suf'}// if ($self->{'config_hash'}->{'file_suf'} && $f =~/.*$self->{'config_hash'}->{'file_suf'}$/);
	next if (!(@{$self->{'uinfo_array'}} = getpwnam($f)));
	$lnum = 0;
	if ($self->{'single_user'}) {
		$self->open_execute_command(*TAB, $self->{'config_hash'}->{'cron_user_get_command'}, 1);
		}
	elsif ($fcron) {
		$self->open_execute_command(*TAB,
			$self->user_sub($self->{'config_hash'}->{'cron_get_command'}, $f), 1);
		}
	else {
		$self->open_readfile(*TAB, "$self->{'config_hash'}->{'cron_dir'}/$f");
		}
	while(<TAB>) {
		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(.*)/i) {
			# A normal m h d m wd time
			push(@rv, { 'file' => "$self->{'config_hash'}->{'cron_dir'}/$f",
				    'line' => $lnum,
				    'type' => 0,
				    'active' => !$1, 'nolog' => $2,
				    'mins' => $3, 'hours' => $4,
				    'days' => $5, 'months' => $6,
				    'weekdays' => $8, 'user' => $f,
				    'command' => $10,
				    'index' => scalar(@rv) });
			$rv[$#rv]->{'file'} =~ s/\s+\|$//;
			$self->fix_names($rv[$#rv]);
			}
		elsif (/^(#+)?\s*@([a-z]+)\s+(.*)/i) {
			# An @ time
			push(@rv, { 'file' => "$self->{'config_hash'}->{'cron_dir'}/$f",
				    'line' => $lnum,
				    'type' => 0,
				    'active' => !$1,
				    'special' => $2,
				    'user' => $f,
				    'command' => $3,
				    'index' => scalar(@rv) });
			}
		elsif (/^(#+)?\s*([a-zA-Z0-9\_]+)\s*=\s*'([^']*)'/ ||
		       /^(#+)?\s*([a-zA-Z0-9\_]+)\s*=\s*"([^']*)"/ ||
		       /^(#+)?\s*([a-zA-Z0-9\_]+)\s*=\s*(\S+)/) {
			# An environment variable
			push(@rv, { 'file' => "$self->{'config_hash'}->{'cron_dir'}/$f",
				    'line' => $lnum,
				    'active' => !$1,
				    'name' => $2,
				    'value' => $3,
				    'user' => $f,
				    'index' => scalar(@rv) });
			}
		$lnum++;
		}
	close(TAB);
	}
}
closedir(DIR);
@{$self->{'cron_jobs_cache_array'}} = @rv;
return @{$self->{'cron_jobs_cache_array'}};
}

sub cron_job_line
{
	my $self = shift;

local @c;
push(@c, "#") if (!$_[0]->{'active'});
if ($_[0]->{'name'}) {
	push(@c, $_[0]->{'name'});
	push(@c, "=");
	push(@c, $_[0]->{'value'} =~ /'/ ? "\"$_[0]->{'value'}\"" :
		 $_[0]->{'value'} =~ /"/ ? "'$_[0]->{'value'}'" :
		 $_[0]->{'value'} !~ /^\S+$/ ? "\"$_[0]->{'value'}\""
					  : $_[0]->{'value'});
	}
else {
	if ($_[0]->{'special'}) {
		push(@c, ($_[0]->{'nolog'} ? '-' : '').'@'.$_[0]->{'special'});
		}
	else {
		push(@c, ($_[0]->{'nolog'} ? '-' : '').$_[0]->{'mins'},
			 $_[0]->{'hours'}, $_[0]->{'days'},
			 $_[0]->{'months'}, $_[0]->{'weekdays'});
		}
	push(@c, $_[0]->{'user'}) if ($_[0]->{'type'} != 0 &&
				      $_[0]->{'type'} != 3);
	push(@c, $_[0]->{'command'});
	}
return join(" ", @c);
}

# copy_cron_temp(&job)
# Copies a job's user's current cron configuration to the temp file
sub copy_cron_temp
{
	my $self = shift;

local $fcron = ($self->{'config_hash'}->{'cron_dir'} =~ /\/fcron$/);
unlink($self->{'cron_temp_file'});
if ($self->{'single_user'}) {
	$self->execute_command($self->{'config_hash'}->{'cron_user_get_command'},
			 undef, $self->{'cron_temp_file'}, undef);
	}
elsif ($fcron) {
	$self->execute_command($self->user_sub($self->{'config_hash'}->{'cron_get_command'},$_[0]->{'user'}),
			 undef, $self->{'cron_temp_file'}, undef);
	}
else {
	system("cp ".$self->translate_filename("$self->{'config_hash'}->{'cron_dir'}/$_[0]->{'user'}")."$self->{'config_hash'}->{'file_suf'} $self->{'cron_temp_file'} 2>/dev/null");
	}
}

# create_cron_job(&job)
# Add a Cron job to a user's file
sub create_cron_job
{
	my $self = shift;

$self->list_cron_jobs();	# init cache
if ($self->{'config_hash'}->{'single_file'} && !$self->{'config_hash'}->{'cron_dir'}) {
	# Add to the single file
	$_[0]->{'type'} = 3;
	local $lref = $self->read_file_lines($self->{'config_hash'}->{'single_file'});
	push(@$lref, $self->cron_job_line($_[0]));
	$self->flush_file_lines($self->{'config_hash'}->{'single_file'});
	}
else {
	# Add to the specified user's crontab
	$self->copy_cron_temp($_[0]);
	local $lref = $self->read_file_lines($self->{'cron_temp_file'});
	$_[0]->{'line'} = scalar(@$lref);
	push(@$lref, $self->cron_job_line($_[0]));
	$self->flush_file_lines();
	system("chown $_[0]->{'user'} $self->{'cron_temp_file'}");
	$self->copy_crontab($_[0]->{'user'});
	$_[0]->{'file'} = "$self->{'config_hash'}->{'cron_dir'}/$_[0]->{'user'}";
	$_[0]->{'index'} = scalar(@{$self->{'cron_jobs_cache_array'}});
	push(@{$self->{'cron_jobs_cache_array'}}, $_[0]);
	}
}

# insert_cron_job(&job)
# Add a Cron job at the top of the user's file
sub insert_cron_job
{
	my $self = shift;

$self->list_cron_jobs();	# init cache
if ($self->{'config_hash'}->{'single_file'} && !$self->{'config_hash'}->{'cron_dir'}) {
	# Insert into single file
	$_[0]->{'type'} = 3;
	local $lref = $self->read_file_lines($self->{'config_hash'}->{'single_file'});
	splice(@$lref, 0, 0, $self->cron_job_line($_[0]));
	$self->flush_file_lines($self->{'config_hash'}->{'single_file'});
	}
else {
	# Insert into the user's crontab
	$self->copy_cron_temp($_[0]);
	local $lref = $self->read_file_lines($self->{'cron_temp_file'});
	$_[0]->{'line'} = 0;
	splice(@$lref, 0, 0, $self->cron_job_line($_[0]));
	$self->flush_file_lines();
	system("chown $_[0]->{'user'} $self->{'cron_temp_file'}");
	$self->copy_crontab($_[0]->{'user'});
	$_[0]->{'file'} = "$self->{'config_hash'}->{'cron_dir'}/$_[0]->{'user'}";
	$_[0]->{'index'} = scalar(@{$self->{'cron_jobs_cache_array'}});
	$self->renumber($_[0]->{'file'}, $_[0]->{'line'}, 1);
	push(@{$self->{'cron_jobs_cache_array'}}, $_[0]);
	}
}

# renumber(file, line, offset)
# All jobs in this file whose line is at or after the given one will be
# incremented by the offset
sub renumber
{
	my $self = shift;

local $j;
foreach $j (@{$self->{'cron_jobs_cache_array'}}) {
	if ($j->{'line'} >= $_[1] &&
	    $j->{'file'} eq $_[0]) {
		$j->{'line'} += $_[2];
		}
	}
}

# renumber_index(index, offset)
sub renumber_index
{
	my $self = shift;

local $j;
foreach $j (@{$self->{'cron_jobs_cache_array'}}) {
	if ($j->{'index'} >= $_[0]) {
		$j->{'index'} += $_[1];
		}
	}
}

# change_cron_job(&job)
sub change_cron_job
{
	my $self = shift;

if ($_[0]->{'type'} == 0) {
	$self->copy_cron_temp($_[0]);
	$self->replace_file_line($self->{'cron_temp_file'}, $_[0]->{'line'},
			   $self->cron_job_line($_[0])."\n");
	$self->copy_crontab($_[0]->{'user'});
	}
else {
	$self->replace_file_line($_[0]->{'file'}, $_[0]->{'line'},
			   $self->cron_job_line($_[0])."\n");
	}
}

# delete_cron_job(&job)
sub delete_cron_job
{
	my $self = shift;

if ($_[0]->{'type'} == 0) {
	$self->copy_cron_temp($_[0]);
	$self->replace_file_line($self->{'cron_temp_file'}, $_[0]->{'line'});
	$self->copy_crontab($_[0]->{'user'});
	}
else {
	$self->replace_file_line($_[0]->{'file'}, $_[0]->{'line'});
	}
@{$self->{'cron_jobs_cache_array'}} = grep { $_ ne $_[0] } @{$self->{'cron_jobs_cache_array'}};
$self->renumber($_[0]->{'file'}, $_[0]->{'line'}, -1);
$self->renumber_index($_[0]->{'index'}, -1);
}

# read_crontab(user)
# Return an array containing the lines of the cron table for some user
sub read_crontab
{
	my $self = shift;

local(@tab);
$self->open_readfile(*TAB, "$self->{'config_hash'}->{cron_dir}/$_[0]");
@tab = <TAB>;
close(TAB);
if (@tab >= 3 && $tab[0] =~ /DO NOT EDIT/ &&
    $tab[1] =~ /^\s*#/ && $tab[2] =~ /^\s*#/) {
	@tab = @tab[3..$#tab];
	}
return @tab;
}

# copy_crontab(user)
# Copy the cron temp file to that for this user
sub copy_crontab
{
	my $self = shift;

if ($self->is_readonly_mode()) {
	# Do nothing
	return undef;
	}
local($pwd);
if (`cat $self->{'cron_temp_file'}` =~ /\S/) {
	local $temp = $self->transname();
	local $rv;
	if ($self->{'config_hash'}->{'cron_edit_command'}) {
		# fake being an editor
		# XXX does not work in translated command mode!
		local $notemp = $self->transname();
		$self->open_tempfile(NO, ">$notemp");
		$self->print_tempfile(NO, "No\n");
		$self->print_tempfile(NO, "N\n");
		$self->print_tempfile(NO, "no\n");
		$self->close_tempfile(NO);
		$ENV{"VISUAL"} = $ENV{"EDITOR"} =
			"$self->{'module_root_directory'}/cron_editor.pl";
		$ENV{"CRON_EDITOR_COPY"} = $self->{'cron_temp_file'};
		system("chown $_[0] $self->{'cron_temp_file'}");
		local $oldpwd = $self->get_current_dir();
		chdir("/");
		if ($self->{'single_user'}) {
			$rv = system($self->{'config_hash'}->{'cron_user_edit_command'}.
				     " >$temp 2>&1 <$notemp");
			}
		else {
			$rv = system(
				$self->user_sub($self->{'config_hash'}->{'cron_edit_command'},$_[0]).
				" >$temp 2>&1 <$notemp");
			}
		unlink($notemp);
		chdir($oldpwd);
		}
	else {
		# use the cron copy command
		if ($self->{'single_user'}) {
			$rv = $self->execute_command(
				$self->{'config_hash'}->{'cron_user_copy_command'},
				$self->{'cron_temp_file'}, $temp, $temp);
			}
		else {
			$rv = $self->execute_command(
				$self->user_sub($self->{'config_hash'}->{'cron_copy_command'}, $_[0]),
				$self->{'cron_temp_file'}, $temp, $temp);
			}
		}
	local $out = `cat $temp`;
	unlink($temp);
	if ($rv || $out =~ /error/i) {
		local $cronin = `cat $self->{'cron_temp_file'}`;
		$self->error($self->text('ecopy', "<pre>$out</pre>", "<pre>$cronin</pre>"));
		}
	}
else {
	# No more cron jobs left, so just delete
	if ($self->{'single_user'}) {
		$self->execute_command($self->{'config_hash'}->{'cron_user_delete_command'});
		}
	else {
		$self->execute_command($self->user_sub(
			$self->{'config_hash'}->{'cron_delete_command'}, $_[0]));
		}
	}
unlink($self->{'cron_temp_file'});
}

# parse_job(job)
# Parse a crontab line into an array containing:
#  active, mins, hrs, days, mons, weekdays, command
sub parse_job
{
	my $self = shift;

local($job, $active) = ($_[0], 1);
if ($job =~ /^#+\s*(.*)$/) {
	$active = 0;
	$job = $1;
	}
$job =~ /^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)$/;
return ($active, $1, $2, $3, $4, $5, $6);
}

# user_sub(command, user)
# Replace the string 'USER' in the command with the user name
sub user_sub
{
	my $self = shift;

local($tmp);
$tmp = $_[0];
$tmp =~ s/USER/$_[1]/g;
return $tmp;
}

# list_allowed()
# Returns a list of all users in the cron allow file
sub list_allowed
{
	my $self = shift;

local(@rv, $_);
$self->open_readfile(*ALLOW, $self->{'config_hash'}->{cron_allow_file});
while(<ALLOW>) {
	next if (/^\s*#/);
	chop; push(@rv, $_) if (/\S/);
	}
close(ALLOW);
return @rv;
}

# list_denied()
# Return a list of users from the cron deny file
sub list_denied
{
	my $self = shift;

local(@rv, $_);
$self->open_readfile(*DENY, $self->{'config_hash'}->{cron_deny_file});
while(<DENY>) {
	next if (/^\s*#/);
	chop; push(@rv, $_) if (/\S/);
	}
close(DENY);
return @rv;
}

# save_allowed(user, user, ...)
# Save the list of allowed users
sub save_allowed
{
	my $self = shift;

local($_);
$self->open_tempfile(ALLOW, ">$self->{'config_hash'}->{cron_allow_file}");
foreach (@_) {
	$self->print_tempfile(ALLOW, $_,"\n");
	}
$self->close_tempfile(ALLOW);
chmod(0444, $self->{'config_hash'}->{cron_allow_file});
}

# save_denied(user, user, ...)
# Save the list of denied users
sub save_denied
{
	my $self = shift;

local($_);
$self->open_tempfile(DENY, "> $self->{'config_hash'}->{cron_deny_file}");
foreach (@_) {
	$self->print_tempfile(DENY, $_,"\n");
	}
$self->close_tempfile(DENY);
chmod(0444, $self->{'config_hash'}->{cron_deny_file});
}

# read_envs(user)
# Returns an array of name,value pairs containing the environment settings
# from the crontab for some user
sub read_envs
{
	my $self = shift;

local(@tab, @rv, $_);
@tab = $self->read_crontab($_[0]);
foreach (@tab) {
	chop; s/#.*$//g;
	if (/^\s*(\S+)\s*=\s*(.*)$/) { push(@rv, "$1 $2"); }
	}
return @rv;
}

# save_envs(user, [name, value]*)
# Updates the cron file for some user with the given list of environment
# variables. All others in the file are removed
sub save_envs
{
	my $self = shift;

local($i, @tab, $line);
@tab = $self->read_crontab($_[0]);
open(TAB, ">$self->{'cron_temp_file'}");
for($i=1; $i<@_; $i+=2) {
	print TAB "$_[$i]=$_[$i+1]\n";
	}
foreach (@tab) {
	chop($line = $_); $line =~ s/#.*$//g;
	if ($line !~ /^\s*(\S+)\s*=\s*(.*)$/) { print TAB $_; }
	}
close(TAB);
$self->copy_crontab($_[0]);
}

# expand_run_parts(directory)
sub expand_run_parts
{
	my $self = shift;

local $dir = $_[0];
$dir = "$self->{'config_hash'}->{'run_parts_dir'}/$dir"
	if ($self->{'config_hash'}->{'run_parts_dir'} && $dir !~ /^\//);
opendir(DIR, $self->translate_filename($dir));
local @rv = readdir(DIR);
closedir(DIR);
@rv = grep { !/^\./ } @rv;
@rv = map { $dir."/".$_ } @rv;
return @rv;
}

# is_run_parts(command)
sub is_run_parts
{
	my $self = shift;

local $rp = $self->{'config_hash'}->{'run_parts'};
return $rp && $_[0] =~ /$rp(.*)\s+(\S+)$/ ? $2 : undef;
}

# can_edit_user(&access, user)
sub can_edit_user
{
	my $self = shift;

local %umap;
map { $umap{$_}++; } split(/\s+/, $_[0]->{'users'})
	if ($_[0]->{'mode'} == 1 || $_[0]->{'mode'} == 2);
if ($_[0]->{'mode'} == 1 && !$umap{$_[1]} ||
    $_[0]->{'mode'} == 2 && $umap{$_[1]}) { return 0; }
elsif ($_[0]->{'mode'} == 3) {
	return $self->{'remote_user'} eq $_[1];
	}
elsif ($_[0]->{'mode'} == 4) {
	local @u = getpwnam($_[1]);
	return (!$_[0]->{'uidmin'} || $u[2] >= $_[0]->{'uidmin'}) &&
	       (!$_[0]->{'uidmax'} || $u[2] <= $_[0]->{'uidmax'});
	}
elsif ($_[0]->{'mode'} == 5) {
	local @u = getpwnam($_[1]);
	return $u[3] == $_[0]->{'users'};
	}
else {
	return 1;
	}
}

# show_times_input(&job, [nospecial])
sub show_times_input
{
	my $self = shift;

return $self->theme_show_times_input(@_) if ($self->UNIVERSAL::can('theme_show_times_input'));
local $job = $_[0];
if ($self->{'config_hash'}->{'vixie_cron'} && (!$_[1] || $_[0]->{'special'})) {
	# Allow selection of special @ times
	print "<tr $self->{'cb'}> <td colspan=6>\n";
	printf "<input type=radio name=special_def value=1 %s> %s\n",
		$job->{'special'} ? "checked" : "", $self->{'text_hash'}->{'edit_special1'};
	print "<select name=special>\n";
	local $s;
	local $sp = $job->{'special'} eq 'midnight' ? 'daily' :
	    $job->{'special'} eq 'annually' ? 'yearly' : $job->{'special'};
	foreach $s ('hourly', 'daily', 'weekly', 'monthly', 'yearly', 'reboot'){
		printf "<option value=%s %s>%s\n",
		    $s, $sp eq $s ? "selected" : "", $self->{'text_hash'}->{'edit_special_'.$s};
		}
	print "</select>\n";
	printf "<input type=radio name=special_def value=0 %s> %s\n",
		$job->{'special'} ? "" : "checked", $self->{'text_hash'}->{'edit_special0'};
	print "</td></tr>\n";
	}

print "<tr $self->{'tb'}>\n";
print "<td><b>$self->{'text_hash'}->{'edit_mins'}</b></td> <td><b>$self->{'text_hash'}->{'edit_hours'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'edit_days'}</b></td> <td><b>$self->{'text_hash'}->{'edit_months'}</b></td>",
      "<td><b>$self->{'text_hash'}->{'edit_weekdays'}</b></td> </tr> <tr $self->{'cb'}>\n";

local @mins = (0..59);
local @hours = (0..23);
local @days = (1..31);
local @months = map { $self->{'text_hash'}->{"month_$_"}."=".$_ } (1 .. 12);
local @weekdays = map { $self->{'text_hash'}->{"day_$_"}."=".$_ } (0 .. 6);

foreach $arr ("mins", "hours", "days", "months", "weekdays") {
	# Find out which ones are being used
	local %inuse;
	local $min = ($arr =~ /days|months/ ? 1 : 0);
	local $max = $min+scalar(@$arr)-1;
	foreach $w (split(/,/ , $job->{$arr})) {
		if ($w eq "*") {
			# all values
			for($j=$min; $j<=$max; $j++) { $inuse{$j}++; }
			}
		elsif ($w =~ /^\*\/(\d+)$/) {
			# only every Nth
			for($j=$min; $j<=$max; $j+=$1) { $inuse{$j}++; }
			}
		elsif ($w =~ /^(\d+)-(\d+)\/(\d+)$/) {
			# only every Nth of some range
			for($j=$1; $j<=$2; $j+=$3) { $inuse{int($j)}++; }
			}
		elsif ($w =~ /^(\d+)-(\d+)$/) {
			# all of some range
			for($j=$1; $j<=$2; $j++) { $inuse{int($j)}++; }
			}
		else {
			# One value
			$inuse{int($w)}++;
			}
		}
	if ($job->{$arr} eq "*") { undef(%inuse); }

	# Output selection list
	print "<td valign=top>\n";
	printf "<input type=radio name=all_$arr value=1 %s> $self->{'text_hash'}->{'edit_all'}<br>\n",
		$job->{$arr} eq "*" ? "checked" : "";
	printf "<input type=radio name=all_$arr value=0 %s> $self->{'text_hash'}->{'edit_selected'}<br>\n",
		$job->{$arr} ne "*" ? "checked" : "";
	print "<table> <tr>\n";
	for($j=0; $j<@$arr; $j+=12) {
		$self->{'jj'} = $j+11;
		if ($self->{'jj'} >= @$arr) { $self->{'jj'} = @$arr - 1; }
		@{$self->{'sec_array'}} = @$arr[$j .. $self->{'jj'}];
		printf "<td valign=top><select multiple size=%d name=$arr>\n",
			@{$self->{'sec_array'}} > 12 ? 12 : scalar(@{$self->{'sec_array'}});
		foreach $v (@{$self->{'sec_array'}}) {
			if ($v =~ /^(.*)=(.*)$/) { $self->{'disp'} = $1; $self->{'code'} = $2; }
			else { $self->{'disp'} = $self->{'code'} = $v; }
			printf "<option value=\"$self->{'code'}\" %s>$self->{'disp'}\n",
				$inuse{$self->{'code'}} ? "selected" : "";
			}
		print "</select></td>\n";
		}
	print "</tr></table></td>\n";
	}
print "</tr> <tr $self->{'cb'}> <td colspan=5>$self->{'text_hash'}->{'edit_ctrl'}</td> </tr>\n";
}

# parse_times_input(&job, &in)
sub parse_times_input
{
	my $self = shift;

local $job = $_[0];
local %in = %{$_[1]};
local @pers = ("mins", "hours", "days", "months", "weekdays");
local $arr;
if ($in{'special_def'}) {
	# Job time is a special period
	foreach $arr (@pers) {
		delete($job->{$arr});
		}
	$job->{'special'} = $in{'special'};
	}
else {
	# User selection of times
	foreach $arr (@pers) {
		if ($in{"all_$arr"}) {
			# All mins/hrs/etc.. chosen
			$job->{$arr} = "*";
			}
		elsif (defined($in{$arr})) {
			# Need to work out and simplify ranges selected
			local (@range, @newrange, $i);
			@range = split(/\0/, $in{$arr});
			@range = sort { $a <=> $b } @range;
			local $start = -1;
			for($i=0; $i<@range; $i++) {
				if ($i && $range[$i]-1 == $range[$i-1]) {
					# ok.. looks like a range
					if ($start < 0) { $start = $i-1; }
					}
				elsif ($start < 0) {
					# Not in a range at all
					push(@newrange, $range[$i]);
					}
				else {
					# End of the range.. add it
					$newrange[@newrange - 1] =
						"$range[$start]-".$range[$i-1];
					push(@newrange, $range[$i]);
					$start = -1;
					}
				}
			if ($start >= 0) {
				# Reached the end while in a range
				$newrange[@newrange - 1] =
					"$range[$start]-".$range[$i-1];
				}
			$job->{$arr} = join(',' , @newrange);
			}
		else {
			$self->error($self->text('save_enone', $self->{'text_hash'}->{"edit_$arr"}));
			}
		}
	delete($job->{'special'});
	}
}

# show_range_input(&job)
# Given a cron job, prints fields for selecting it's run date range
sub show_range_input
{
	my $self = shift;

local ($job) = @_;
print $self->ui_oneradio("range_def", 1, $self->{'text_hash'}->{'range_all'}, !$job->{'start'}),
      "<br>\n";
print $self->ui_oneradio("range_def", 0, "", $job->{'start'}),"\n";
print $self->text('range_start', $self->ui_date_input(
	$job->{'start'}->[0], $job->{'start'}->[1], $job->{'start'}->[2],
	"range_start_day", "range_start_month", "range_start_year")),"\n",
      $self->date_chooser_button(
	"range_start_day", "range_start_month", "range_start_year"),"\n";
print $self->text('range_end', $self->ui_date_input(
	$job->{'end'}->[0], $job->{'end'}->[1], $job->{'end'}->[2],
	"range_end_day", "range_end_month", "range_end_year")),"\n",
      $self->date_chooser_button(
	"range_end_day", "range_end_month", "range_end_year"),"\n";
}

# parse_range_input(&job, &in)
# Updates the job object with the specified date range. May call &error
# for invalid inputs
sub parse_range_input
{
	my $self = shift;

local ($job, $in) = @_;
if ($in->{'range_def'}) {
	# No range used
	delete($job->{'start'});
	delete($job->{'end'});
	}
else {
	# Validate and store range
	foreach my $r ("start", "end") {
		eval { timelocal(0, 0, 0, $self->{'in_hash'}->{'range_'.$r.'_day'},
					  $self->{'in_hash'}->{'range_'.$r.'_month'}-1,
					  $self->{'in_hash'}->{'range_'.$r.'_year'}-1900) };
		if ($@) {
			$self->error($self->{'text_hash'}->{'range_e'.$r}." ".$@);
			}
		$job->{$r} = [ $self->{'in_hash'}->{'range_'.$r.'_day'},
			       $self->{'in_hash'}->{'range_'.$r.'_month'},
			       $self->{'in_hash'}->{'range_'.$r.'_year'} ];
		}
	}
}

# fix_names(&cron)
# Convert day and month names to numbers
sub fix_names
{
	my $self = shift;

local ($m, $w);

local @mts = split(/,/, $_[0]->{'months'});
foreach $m (@mts) {
	local $mi = $self->indexof(lc($m), @{$self->{'cron_month_array'}});
	$m = $mi+1 if ($mi >= 0);
	}
$_[0]->{'months'} = join(",", @mts);

local @wds = split(/,/, $_[0]->{'weekdays'});
foreach $w (@wds) {
	local $di = $self->indexof(lc($w), @{$self->{'cron_weekday_array'}});
	$w = $di if ($di >= 0);
	$w = 0 if ($w == 7);
	}
$_[0]->{'weekdays'} = join(",", @wds);
}

# create_wrapper(wrapper-path, module, script)
# Creates a wrapper script which calls a script in some module's directory
# with the proper webmin environment variables set. This should always be used
# when setting up a cron job, instead of attempting to run a command in the
# module directory directly.
sub create_wrapper
{
	my $self = shift;

local $perl_path = $self->get_perl_path();
$self->open_tempfile(CMD, ">$_[0]");
$self->print_tempfile(CMD, <<EOF
#!$perl_path
open(CONF, "$self->{'config_directory'}/miniserv.conf");
while(<CONF>) {
        \$root = \$1 if (/^root=(.*)/);
        }
close(CONF);
\$ENV{'WEBMIN_CONFIG'} = "$ENV{'WEBMIN_CONFIG'}";
\$ENV{'WEBMIN_VAR'} = "$ENV{'WEBMIN_VAR'}";
EOF
	);
if ($self->{'gconfig_hash'}->{'os_type'} eq 'windows') {
	# On windows, we need to chdir to the drive first, and use system
	$self->print_tempfile(CMD, "if (\$root =~ /^([a-z]:)/i) {\n");
	$self->print_tempfile(CMD, "       chdir(\"\$1\");\n");
	$self->print_tempfile(CMD, "       }\n");
	$self->print_tempfile(CMD, "chdir(\"\$root/$_[1]\");\n");
	$self->print_tempfile(CMD, "exit(system(\"\$root/$_[1]/$_[2]\", \@ARGV));\n");
	}
else {
	# Can use exec on Unix systems
	$self->print_tempfile(CMD, "chdir(\"\$root/$_[1]\");\n");
	$self->print_tempfile(CMD, "exec(\"\$root/$_[1]/$_[2]\", \@ARGV) || die \"Failed to run \$root/$_[1]/$_[2] : \$!\";\n");
	}
$self->close_tempfile(CMD);
chmod(0755, $_[0]);
}

# cron_file(&job)
# Returns the file that a cron job is in, or would be in
sub cron_file
{
	my $self = shift;

return $_[0]->{'file'} || "$self->{'config_hash'}->{'cron_dir'}/$_[0]->{'user'}";
}

# when_text(&job, [upper-case-first])
# Returns a text string describing when a cron job is run
sub when_text
{
	my $self = shift;

local $pfx = $_[1] ? "uc" : "";
if ($_[0]->{'special'}) {
	$pfx = $_[1] ? "" : "lc";
	return $self->{'text_hash'}->{$pfx.'edit_special_'.$_[0]->{'special'}};
	}
elsif ($_[0]->{'mins'} eq '*' && $_[0]->{'hours'} eq '*' && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
	return $self->{'text_hash'}->{$pfx.'when_min'};
	}
elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} eq '*' && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
	return $self->text($pfx.'when_hour', $_[0]->{'mins'});
	}
elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} =~ /^\d+$/ && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
	return $self->text($pfx.'when_day', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'});
	}
elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} =~ /^\d+$/ && $_[0]->{'days'} =~ /^\d+$/ && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
	return $self->text($pfx.'when_month', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'}, $_[0]->{'days'});
	}
elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} =~ /^\d+$/ && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} =~ /^\d+$/) {
	return $self->text($pfx.'when_weekday', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'}, $self->{'text_hash'}->{"day_".$_[0]->{'weekdays'}});
	}
else {
	return $self->text($pfx.'when_cron', join(" ", $_[0]->{'mins'}, $_[0]->{'hours'}, $_[0]->{'days'}, $_[0]->{'months'}, $_[0]->{'weekdays'}));
	}
}

# can_use_cron(user)
# Returns 1 if some user is allowed to use cron, based on cron.allow and
# cron.deny files
sub can_use_cron
{
	my $self = shift;

local $err;
if (-r $self->{'config_hash'}->{cron_allow_file}) {
	local @allowed = $self->list_allowed();
	if ($self->indexof($_[0], @allowed) < 0 &&
	    $self->indexof("all", @allowed) < 0) { $err = 1; }
	}
elsif (-r $self->{'config_hash'}->{cron_deny_file}) {
	local @denied = $self->list_denied();
	if ($self->indexof($_[0], @denied) >= 0 ||
	    $self->indexof("all", @denied) >= 0) { $err = 1; }
	}
elsif ($self->{'config_hash'}->{cron_deny_all} == 0) { $err = 1; }
elsif ($self->{'config_hash'}->{cron_deny_all} == 1) {
	if ($self->{'in_hash'}->{user} ne "root") { $err = 1; }
	}
return !$err;
}

# swap_cron_jobs(&job1, &job2)
# Swaps two Cron jobs, which must be in the same file
sub swap_cron_jobs
{
	my $self = shift;

if ($_[0]->{'type'} == 0) {
	$self->copy_cron_temp($_[0]);
	local $lref = $self->read_file_lines($self->{'cron_temp_file'});
	($lref->[$_[0]->{'line'}], $lref->[$_[1]->{'line'}]) =
		($lref->[$_[1]->{'line'}], $lref->[$_[0]->{'line'}]);
	$self->flush_file_lines();
	$self->copy_crontab($_[0]->{'user'});
	}
else {
	local $lref = $self->read_file_lines($_[0]->{'file'});
	($lref->[$_[0]->{'line'}], $lref->[$_[1]->{'line'}]) =
		($lref->[$_[1]->{'line'}], $lref->[$_[0]->{'line'}]);
	$self->flush_file_lines();
	}
}

# find_cron_process(&job, [&procs])
sub find_cron_process
{
	my $self = shift;

local @procs;
if ($_[1]) {
	@procs = @{$_[1]};
	}
else {
	$self->foreign_require("proc", "proc-lib.pl");
	@procs = $main::proc_object->list_processes();
	}
local $rpd = $self->is_run_parts($_[0]->{'command'});
local @exp = $rpd ? $self->expand_run_parts($rpd) : ();
local $cmd = $exp[0] || $_[0]->{'command'};
$cmd =~ s/^\s*\[.*\]\s+\&\&\s+//;
$cmd =~ s/^\s*\[.*\]\s+\|\|\s+//;
while($cmd =~ s/(\d*)(<|>)((\/\S+)|&\d+)\s*$//) { }
$cmd =~ s/^\((.*)\)\s*$/$1/;
$cmd =~ s/\s+$//;
if ($self->{'config_hash'}->{'match_mode'} == 1) {
	$cmd =~ s/\s.*$//;
	}
($self->{'proc'}) = grep { $_->{'args'} =~ /\Q$cmd\E/ &&
		 (!$self->{'config_hash'}->{'match_user'} || $_->{'user'} eq $_[0]->{'user'}) }
		@procs;
if (!$self->{'proc'} && $cmd =~ /^$self->{'config_directory'}\/(.*\.pl)(.*)$/) {
	# Must be a Webmin wrapper
	$cmd = "$self->{'root_directory'}/$1$2";
	($self->{'proc'}) = grep { $_->{'args'} =~ /\Q$cmd\E/ &&
			 (!$self->{'config_hash'}->{'match_user'} ||
			  $_->{'user'} eq $_[0]->{'user'}) }
			@procs;
	}
return $self->{'proc'};
}

# extract_input(command)
# Given a command line cmd%input , returns the command and input parts
sub extract_input
{
	my $self = shift;

local ($cmd) = @_;
$cmd =~ s/\\%/\0/g;
local ($cmd, $input) = split(/\%/, $cmd, 2);
$cmd =~ s/\0/%/g;
$input =~ s/\0/%/g;
return ($cmd, $input);
}

# convert_range(&job)
# Given a cron job that uses range.pl , work out the date range and update
# the job object command
sub convert_range
{
	my $self = shift;

local ($job) = @_;
local ($cmd, $input) = $self->extract_input($job->{'command'});
if ($cmd =~ /^\Q$self->{'range_cmd'}\E\s+(\d+)\-(\d+)\-(\d+)\s+(\d+)\-(\d+)\-(\d+)\s+(.*)$/) {
	# Looks like a range command
	$job->{'start'} = [ $1, $2, $3 ];
	$job->{'end'} = [ $4, $5, $6 ];
	$job->{'command'} = $7;
	$job->{'command'} =~ s/\\(.)/$1/g;
	if ($input) {
		$job->{'command'} .= '%'.$input;
		}
	return 1;
	}
return 0;
}

# unconvert_range(&job)
sub unconvert_range
{
	my $self = shift;

if ($self->{'job'}->{'start'}) {
	# Need to add range command
	local ($cmd, $input) = $self->extract_input($self->{'job'}->{'command'});
	$self->{'job'}->{'command'} = $self->{'range_cmd'}." ".join("-", @{$self->{'job'}->{'start'}})." ".
					   join("-", @{$self->{'job'}->{'end'}})." ".
					   quotemeta($cmd);
	if ($input) {
		$self->{'job'}->{'command'} .= '%'.$input;
		}
	delete($self->{'job'}->{'start'});
	delete($self->{'job'}->{'end'});
	$self->create_wrapper($self->{'range_cmd'}, $self->{'module_name'}, "range.pl");
	$self->set_ownership_permissions(undef, undef, 0755, $self->{'range_cmd'});
	return 1;
	}
return 0;
}

# convert_comment(&job)
# Given a cron job with a # command after the command, sets the comment field
sub convert_comment
{
	my $self = shift;

if ($self->{'job'}->{'command'} =~ /^(.*)\s*#([^#]*)$/) {
	$self->{'job'}->{'command'} = $1;
	$self->{'job'}->{'comment'} = $2;
	return 1;
	}
return 0;
}

# unconvert_comment(&job)
# Adds an comment back to the command in a cron job
sub unconvert_comment
{
	my $self = shift;

if ($self->{'job'}->{'comment'} =~ /\S/) {
	$self->{'job'}->{'command'} .= " #".$self->{'job'}->{'comment'};
	return 1;
	}
return 0;
}


1;

