#!/usr/bin/perl
package cyrus_sasl::CyrussaslLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'cyrus_sasl::CyrussaslLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# cyrus-sasl-lib.pl
# common functions for cyrus_sasl configuration

# main part
&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

1;
1;
		return $self;
}

# Return version for daemon saslauthd
sub get_version_sasld
{
	my $self = shift;

return $self->backquote_command("$self->{'config_hash'}->{'saslauthd_cmd'} -v 2>&1");
}

# Check status of service
# 1 or @PIDs - service running; 
# 0 - service stopped or dead
sub status_sasld
{
	my $self = shift;

 my $out;
 if ($self->{'config_hash'}->{'status_cmd'}) {
 	# status cmd
 	$out = $self->backquote_command("$self->{'config_hash'}->{'status_cmd'} 2>&1");
	return $? ? 0: 1;
	}
 else {
 	# search running process
	return $self->find_byname($self->{'config_hash'}->{'saslauthd_cmd'});
 	}
}

# Start or stop daemon saslauthd 
# 1 -start service; 0 -stop service
# return undef on success or an error message
sub manage_sasld
{
	my $self = shift;

my($out, $command, %mech);
if( ($_[0] && $self->{'config_hash'}->{'start_cmd'}) || (!$_[0] && $self->{'config_hash'}->{'stop_cmd'})) {
    if($_[0]) { $command = $self->{'config_hash'}->{'start_cmd'};}
    else { $command = $self->{'config_hash'}->{'stop_cmd'};} 
    $out = $self->backquote_logged("$command 2>&1");
    return $? ? "<pre>$out</pre>" : undef;
    }
else {
    if($_[0]) {
	%mech = $self->get_auth_id();
	$command = "$self->{'config_hash'}->{'saslauthd_cmd'} -a $mech{'0'}";
        $out = $self->backquote_logged("$command 2>&1");
        return $? ? "<pre>$out</pre>" : undef;
	}
    else {
	$out = $self->kill_logged('TERM', $self->check_pid_file($self->{'config_hash'}->{'pid_file'}));	
        return $? ? "<pre>$out</pre>" : undef;
	}
    }
}

# Restart daemon saslauthd
# return undef on success or an error message
sub restart_sasld
{
	my $self = shift;

my($out, $var);
if($self->{'config_hash'}->{'restart_cmd'}) {
    $out = $self->backquote_logged("$self->{'config_hash'}->{'restart_cmd'} 2>&1");
    if(!( $var = $? )  && $self->status_sasld()) { return undef; }
    elsif(! $var) { return $self->{'text_hash'}->{'process_erestart'}; }
    else { return "<pre>$out</pre>" };
    }
else {
    $self->manage_sasld(0);
    $out = $self->manage_sasld(1);
    $self->webmin_log("restart");
    return $? ? "<pre>$out</pre>" : undef;
    }
}

# Get script name without pathname
sub get_script_name
{
	my $self = shift;

my($var) = $0;
$var =~ /^.*\/([^\/]+)$/;
return $1;
}

# Get hash for auth mechanisms
# 0 - default auth mechanism 
# [1-6] - available methods in system
sub get_auth_id
{
	my $self = shift;

my ($key, $value, @var);
my %authmech = ( 1 => 'getpwent', 2 => 'kerberos5', 3 => 'pam',
4 => 'rimap', 5 => 'shadow', 6 => 'ldap');

# Check what auth type used in system
# and remove unused from hash
@var=`$self->{'config_hash'}->{'saslauthd_cmd'} -v 2>&1`;
while( ($key, $value) =  each(%authmech)) {
    $var[1] =~ /\b$value\b/ && next;
    delete($authmech{$key});
    }

# Added in hash default mech
$authmech{'0'} = $authmech{$self->{'config_hash'}->{'authmech'}};

return %authmech;
}

# Show form with content of some file
# $1 - name of file
# $2 - name of script
# $3 - number of rows in textarea
# $4 - number of columns in textarea
# $5 - title of site
# $6 - reference to hash with hidden fields
# 	it will be added into the form
# $7 - server will be restarted (Save and Apply)
sub show_manually_edit
{
	my $self = shift;

my ($file, $script, $row, $col, $title, $hfield, $but_name) = @_;
if($but_name) { $but_name = "Save and Apply";}
else { $but_name =  "Save";}
$row ||= 10; $col ||= 50; $title ||= "Edit file";

$self->ui_print_header( $title, $title);
unless( -r $file) { $self->error("$self->{'text_hash'}->{'error_open'}: $file'}"); }

print $self->ui_form_start("$script","form-data");
print $self->ui_table_start($self->text('form_editfile_header', $file), 
    "width='80%' align='center'");

print $self->ui_columns_row([ $self->ui_textarea("data", $self->read_file_contents($file), 
    $row, $col) . "<br>$self->{'text_hash'}->{'edit_warning'}" . $self->ui_hidden("sysreq",3) ]);

if( $hfield) {
    $title = '';
    foreach (keys %{$hfield}) {	$title .= $self->ui_hidden($_, $$hfield{$_});}
    print $self->ui_columns_row([ $title ]);
}	

print $self->ui_table_hr();
print $self->ui_columns_row( [ $self->ui_submit($but_name) ],
    [ "align='left'"]);
print $self->ui_table_end() . $self->ui_form_end();
$self->ui_print_footer( $ENV{'HTTP_REFERER'}, $self->{'text_hash'}->{'error_previous'});
}

# Save content from form to some file
# $1 -name of file
# $2 -if defined, then restart daemon
# $3 -title
sub save_manually_edit
{
	my $self = shift;

my($var, $title, @new_content);
$self->{'in_hash'}->{'data'} =~ s/\r//g;
@new_content = split('\n', $self->{'in_hash'}->{'data'});
$main::file_cache{$self->translate_filename($_[0])} = \@new_content;
$self->flush_file_lines($_[0]);

# Restart server
if($_[1]) {
    $title = $_[2];
    $title ||= "Save file";
    $var = $self->restart_sasld();
    unless(defined($var)) { $self->redirect("index.cgi"); return 1; }
    $self->ui_print_header( $title, $title);
    print $self->text('form_save_action', $_[0]);
    print "<br><br>$var<br>";
    $self->ui_print_footer($ENV{'HTTP_REFERER'}, $self->{'text_hash'}->{'error_previous'});
    return 1;
    }    
$self->redirect("index.cgi");
}

# Delete spaces from string
# $1 -some string
sub trunc_space 
{
	my $self = shift;

 my $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}

############################### OLD FUNCTION #############################
# parse_config_file(file,[spliter])
# parses cyrus_sasl SMTP file and returns an array of hashes
sub parse_config_file
{
	my $self = shift;

 # set parser
 my $parse = $_[1] ? $_[1] : ':'; 
 
 # parse file
 open (INFL, $_[0]) || $self->error($self->{'text_hash'}->{'error_open'});
 my ($number, @conf_file);
 while (<INFL>)
 {
  $number++;
  chomp;
  $self->trunc_space($_);
  unless ($_=~/^#/ || $_=~/^$/)
  {
   my @str = split (/$parse/,$_);
   $str[0] = $self->trunc_space($str[0]);
   $str[1] = $self->trunc_space($str[1]);
   push (@conf_file,{'name'=> $str[0],'value'=> $str[1]});
  }
 }
 close INFL;
 return \@conf_file;
}

# &set_config_value (name, value, @array)
# set value for name from array
sub set_config_value
{
	my $self = shift;

 my $str_old;
 foreach (@{$_[2]})
 {
  if ($_->{'name'} eq $_[0])
  {
   $_->{'value'}=$_[1];
   $str_old=1;
  }
 }
 push (@{$_[2]},{'name'=>$_[0],'value'=>$_[1]}) if ($str_old!=1);
 return @{$_[2]};
}

# &save_config_file(file,\@array,[spliter])
# save all new options from array into Cyrus-SASL SMTP file, change union options values
sub save_config_file
{
	my $self = shift;
 
 # set parser
 my $parse = $_[2] ? $_[2] : ':'; 
   
 # read config file
 $self->lock_file($_[0]);
 my $old_file = $self->read_file_lines($_[0]);
 
 my %key;
 # modify options
 foreach (@{$old_file})
 {
  my @str_old = split (/$parse/, $_);
  my $str = $self->trunc_space($str_old[0]);
  foreach my $i (@{$_[1]})
  {
	if (($str eq $i->{'name'}))
	{
    $_ = $i->{'name'}.$parse.$i->{'value'};
	 $key{$i->{'name'}} = 1;
	 last;
	}
  }
 }
 
 # add new options
 foreach my $i (@{$_[1]})
 {
  push (@{$old_file}, $i->{'name'}.$parse.$i->{'value'}) unless ($key{$i->{'name'}});
 }
 
 # save config file
 $self->flush_file_lines();
 $self->unlock_file($_[0]);
}


1;

