# cyrus-sasl-lib.pl
# common functions for cyrus-sasl configuration

# main part
do '../web-lib.pl';
&init_config();
require '../ui-lib.pl';

# Return version for daemon saslauthd
sub get_version_sasld
{
return &backquote_command("$config{'saslauthd_cmd'} -v 2>&1");
}

# Check status of service
# 1 or @PIDs - service running; 
# 0 - service stopped or dead
sub status_sasld
{
 my $out;
 if ($config{'status_cmd'}) {
 	# status cmd
 	$out = &backquote_command("$config{'status_cmd'} 2>&1");
	return $? ? 0: 1;
	}
 else {
 	# search running process
	return &find_byname($config{'saslauthd_cmd'});
 	}
}

# Start or stop daemon saslauthd 
# 1 -start service; 0 -stop service
# return undef on success or an error message
sub manage_sasld
{
my($out, $command, %mech);
if( ($_[0] && $config{'start_cmd'}) || (!$_[0] && $config{'stop_cmd'})) {
    if($_[0]) { $command = $config{'start_cmd'};}
    else { $command = $config{'stop_cmd'};} 
    $out = &backquote_logged("$command 2>&1");
    return $? ? "<pre>$out</pre>" : undef;
    }
else {
    if($_[0]) {
	%mech = &get_auth_id();
	$command = "$config{'saslauthd_cmd'} -a $mech{'0'}";
        $out = &backquote_logged("$command 2>&1");
        return $? ? "<pre>$out</pre>" : undef;
	}
    else {
	$out = &kill_logged('TERM', &check_pid_file($config{'pid_file'}));	
        return $? ? "<pre>$out</pre>" : undef;
	}
    }
}

# Restart daemon saslauthd
# return undef on success or an error message
sub restart_sasld
{
my($out, $var);
if($config{'restart_cmd'}) {
    $out = &backquote_logged("$config{'restart_cmd'} 2>&1");
    if(!( $var = $? )  && &status_sasld()) { return undef; }
    elsif(! $var) { return $text{'process_erestart'}; }
    else { return "<pre>$out</pre>" };
    }
else {
    &manage_sasld(0);
    $out = &manage_sasld(1);
    &webmin_log("restart");
    return $? ? "<pre>$out</pre>" : undef;
    }
}

# Get script name without pathname
sub get_script_name
{
my($var) = $0;
$var =~ /^.*\/([^\/]+)$/;
return $1;
}
 
# Get hash for auth mechanisms
# 0 - default auth mechanism 
# [1-6] - available methods in system
sub get_auth_id
{
my ($key, $value, @var);
my %authmech = ( 1 => 'getpwent', 2 => 'kerberos5', 3 => 'pam',
4 => 'rimap', 5 => 'shadow', 6 => 'ldap');

# Check what auth type used in system
# and remove unused from hash
@var=`$config{'saslauthd_cmd'} -v 2>&1`;
while( ($key, $value) =  each(%authmech)) {
    $var[1] =~ /\b$value\b/ && next;
    delete($authmech{$key});
    }

# Added in hash default mech
$authmech{'0'} = $authmech{$config{'authmech'}};

return %authmech;
}

# Show form with content of some file
# $1 - name of file
# $2 - name of script
# $3 - number of rows in textarea
# $4 - number of columns in textarea
# $5 - title of site
# $6 - reference to hash with hidden fields
# 	it will be added into the form
# $7 - server will be restarted (Save and Apply)
sub show_manually_edit
{
my ($file, $script, $row, $col, $title, $hfield, $but_name) = @_;
if($but_name) { $but_name = "Save and Apply";}
else { $but_name =  "Save";}
$row ||= 10; $col ||= 50; $title ||= "Edit file";

&ui_print_header( $title, $title);
unless( -r $file) { &error("$text{'error_open'}: $file'}"); }

print &ui_form_start("$script","form-data");
print &ui_table_start(&text('form_editfile_header', $file), 
    "width='80%' align='center'");

print &ui_columns_row([ &ui_textarea("data", &read_file_contents($file), 
    $row, $col) . "<br>$text{'edit_warning'}" . &ui_hidden("sysreq",3) ]);

if( $hfield) {
    $title = '';
    foreach (keys %{$hfield}) {	$title .= &ui_hidden($_, $$hfield{$_});}
    print &ui_columns_row([ $title ]);
}	

print &ui_table_hr();
print &ui_columns_row( [ &ui_submit($but_name) ],
    [ "align='left'"]);
print &ui_table_end() . &ui_form_end();
&ui_print_footer( $ENV{'HTTP_REFERER'}, $text{'error_previous'});
}

# Save content from form to some file
# $1 -name of file
# $2 -if defined, then restart daemon
# $3 -title
sub save_manually_edit
{
my($var, $title, @new_content);
$in{'data'} =~ s/\r//g;
@new_content = split('\n', $in{'data'});
$main::file_cache{&translate_filename($_[0])} = \@new_content;
&flush_file_lines($_[0]);

# Restart server
if($_[1]) {
    $title = $_[2];
    $title ||= "Save file";
    $var = &restart_sasld();
    unless(defined($var)) { &redirect("index.cgi"); return 1; }
    &ui_print_header( $title, $title);
    print &text('form_save_action', $_[0]);
    print "<br><br>$var<br>";
    &ui_print_footer($ENV{'HTTP_REFERER'}, $text{'error_previous'});
    return 1;
    }    
&redirect("index.cgi");
}

# Delete spaces from string
# $1 -some string
sub trunc_space 
{
 my $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}

############################### OLD FUNCTION #############################
# parse_config_file(file,[spliter])
# parses cyrus-sasl SMTP file and returns an array of hashes
sub parse_config_file
{
 # set parser
 my $parse = $_[1] ? $_[1] : ':'; 
 
 # parse file
 open (INFL, $_[0]) || &error($text{'error_open'});
 my ($number, @conf_file);
 while (<INFL>)
 {
  $number++;
  chomp;
  &trunc_space($_);
  unless ($_=~/^#/ || $_=~/^$/)
  {
   my @str = split (/$parse/,$_);
   $str[0] = &trunc_space($str[0]);
   $str[1] = &trunc_space($str[1]);
   push (@conf_file,{'name'=> $str[0],'value'=> $str[1]});
  }
 }
 close INFL;
 return \@conf_file;
}

# &set_config_value (name, value, @array)
# set value for name from array
sub set_config_value
{
 my $str_old;
 foreach (@{$_[2]})
 {
  if ($_->{'name'} eq $_[0])
  {
   $_->{'value'}=$_[1];
   $str_old=1;
  }
 }
 push (@{$_[2]},{'name'=>$_[0],'value'=>$_[1]}) if ($str_old!=1);
 return @{$_[2]};
}

# &save_config_file(file,\@array,[spliter])
# save all new options from array into Cyrus-SASL SMTP file, change union options values
sub save_config_file
{ 
 # set parser
 my $parse = $_[2] ? $_[2] : ':'; 
   
 # read config file
 &lock_file($_[0]);
 my $old_file = &read_file_lines($_[0]);
 
 my %key;
 # modify options
 foreach (@{$old_file})
 {
  my @str_old = split (/$parse/, $_);
  my $str = &trunc_space($str_old[0]);
  foreach my $i (@{$_[1]})
  {
	if (($str eq $i->{'name'}))
	{
    $_ = $i->{'name'}.$parse.$i->{'value'};
	 $key{$i->{'name'}} = 1;
	 last;
	}
  }
 }
 
 # add new options
 foreach my $i (@{$_[1]})
 {
  push (@{$old_file}, $i->{'name'}.$parse.$i->{'value'}) unless ($key{$i->{'name'}});
 }
 
 # save config file
 &flush_file_lines();
 &unlock_file($_[0]);
}

1;
1;
