#!/usr/bin/perl
package dfsadmin::DfsLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'dfsadmin::DfsLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# dfs-lib.pl
# Common functions for managing dfstab files

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();

$self->{'default_type'} = 'nfs';
if ($self->{'config_hash'}->{'fstypes_file'} && open(TYPES, $self->{'config_hash'}->{'fstypes_file'})) {
	if (<TYPES> =~ /^(\S+)/) {
		$self->{'default_type'} = $1;
		}
	close(TYPES);
	}
%{$self->{'access_hash'}} = $self->get_module_acl();

1;

1;
		return $self;
}

# list_shares()
# Returns a list of structures containing share details
sub list_shares
{
	my $self = shift;

local $lnum = 0;
local @rv;
open(DFS, $self->{'config_hash'}->{'dfstab_file'});
while(<DFS>) {
	s/\r|\n//g; s/#.*$//;
	if (/^\s*\S*share\s+(.*)/) {
		# Found a share line
		local $share = { 'line' => $lnum,
				 'index' => scalar(@rv) };
		local $line = $1;
		while($line =~ /\\$/) {
			$_ = <DFS>;
			s/\r|\n//g; s/#.*$//;
			$line =~ s/\\$//;
			$line .= $_;
			$lnum++;
			}
		$share->{'eline'} = $lnum;
		if ($line =~ /(\/\S+)/) {
			$share->{'dir'} = $1;
			}
		if ($line =~ /-d\s+"([^"]+)"/) { $share->{'desc'} = $1; }
		elsif ($line =~ /-d\s+(\S+)/) { $share->{'desc'} = $1; }
		if ($line =~ /-o\s+"([^"]+)"/) { $share->{'opts'} = $1; }
		elsif ($line =~ /-o\s+(\S+)/) { $share->{'opts'} = $1; }
		if ($line =~ /\s-F\s+(\S+)/) { $share->{'type'} = $1; }
		else { $share->{'type'} = $self->{'default_type'}; }
		push(@rv, $share);
		}
	$lnum++;
	}
close(DFS);
return @rv;
}

# create_share(&share)
# Add a new share to the dfstab file
sub create_share
{
	my $self = shift;

$self->open_tempfile(DFS, ">> $self->{'config_hash'}->{dfstab_file}");
$self->print_tempfile(DFS, $self->share_line($_[0]),"\n");
$self->close_tempfile(DFS);
}

# modify_share(&share)
# Modify an existing share
sub modify_share
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'dfstab_file'});
splice(@$lref, $_[0]->{'line'}, $_[0]->{'eline'} - $_[0]->{'line'} + 1,
       $self->share_line($_[0]));
$self->flush_file_lines();
}

# share_line(&share)
sub share_line
{
	my $self = shift;

local $s = "share";
$s .= " -d \"$_[0]->{'desc'}\"" if ($_[0]->{'desc'});
$s .= " -o $_[0]->{'opts'}" if ($_[0]->{'opts'});
$s .= " -F $_[0]->{'type'}" if ($_[0]->{'type'} &&
				$_[0]->{'type'} ne $self->{'default_type'});
$s .= " $_[0]->{'dir'}";
return $s;
}

# delete_share(&share)
# Delete the share for a particular directory
sub delete_share
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'dfstab_file'});
splice(@$lref, $_[0]->{'line'}, $_[0]->{'eline'} - $_[0]->{'line'} + 1);
$self->flush_file_lines();
}

# parse_options(string)
# Parse a mount options string like rw=foo,nosuid,... into the associative
# array %{$self->{'options_hash'}}. Parts with no value are given an empty string as the value
sub parse_options
{
	my $self = shift;

local($opt);
undef(%{$self->{'options_hash'}});
foreach $opt (split(/,/, $_[0])) {
	if ($opt =~ /^([^=]+)=(.*)$/) {
		$self->{'options_hash'}->{$1} = $2;
		}
	else {
		$self->{'options_hash'}->{$opt} = "";
		}
	}
return \%{$self->{'options_hash'}};
}

# join_options([&options])
# Returns a list of options from the %{$self->{'options_hash'}} array, in the form used in
# the dfstab file
sub join_options
{
	my $self = shift;

local $o = $_[0] ? $_[0] : \%{$self->{'options_hash'}};
local(@list, $k);
foreach $k (keys %$o) {
	if ($o->{$k} eq "") {
		push(@list, $k);
		}
	else {
		push(@list, "$k=$o->{$k}");
		}
	}
return join(',', @list);
}

# apply_configuration()
# Apply the NFS configuration, returning undef on success or an error message
# on failure
sub apply_configuration
{
	my $self = shift;

local $temp = $self->transname();
$self->system_logged("$self->{'config_hash'}->{unshare_all_command} >/dev/null 2>&1");
$self->system_logged("$self->{'config_hash'}->{share_all_command} >/dev/null 2>$temp");
local $why = `/bin/cat $temp`;
unlink($temp);
if ($why =~ /\S+/) {
	return $why;
	}
return undef;
}


1;

