#!/usr/bin/perl
package dg::DgLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'dg::DgLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# dg-lib.pl
# Common functions for DansGuardian configuration

# main part
&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

#--- System funcs ---

#--- File operation funcs ---

#--- DansGuardian options funcs ---

#--- Filter groups funcs ---

#--- Download & Upload funcs ---

#--- Interface funcs ---

1;
		return $self;
}

# &get_version()
# returns the dansguardian version number, or 0 if not available
sub get_version
{
	my $self = shift;

 # from rpm
 return $self->get_rpm_version();
 
 return 0;
}

# &get_rpm_version()
# returns dansguardian rpm version or 0
sub get_rpm_version 
{
	my $self = shift;

 my $out;
 my $rpm_opts = "-q dansguardian --queryformat \"\%\{VERSION\}\"";
 if (-x '/bin/rpm') 
 {
  $out = `/bin/rpm $rpm_opts 2>/dev/null`;
  return $out if !$? && $out =~ /^[\d\.]+$/;
 }
 elsif ($self->has_command('rpm')) 
 {
  $out = `rpm $rpm_opts 2>/dev/null`;
  return $out if !$? && $out =~ /^[\d\.]+$/;
 }
 return 0;
}

# &status_dansguardian()
# return 1 if daemon start, return 0 if daemon stop 
sub status_dansguardian
{
	my $self = shift;

 my $out = $self->backquote_command("$self->{'config_hash'}->{'status_cmd'} 2>&1 </dev/null");
 if ($? || $out=~/stopped/ || $out=~/not\s+?running/)
 {
  # service stoped
  return 0;
 }
 else
 {
  # service started
  return 1;
 }
}

# &start_dansguardian()
# start daemon dansguardian or call &error
sub start_dansguardian
{
	my $self = shift;

 unless ($self->status_dansguardian())
 {
  # service stoped
  if ($self->{'config_hash'}->{'start_cmd'})
  {
   # start with script
	my $out = $self->backquote_command("$self->{'config_hash'}->{'start_cmd'} 2>&1 </dev/null");
   if ($?){ $self->error($self->text('error_start', $out)); }
   else { $self->webmin_log("start"); }
  }
  else { $self->error($self->{'text_hash'}->{'error_start_cmd'}); }
 }
 else
 {
  # service started
  $self->error($self->{'text_hash'}->{'proces_status'});
 }
}

# &stop_dansguardian()
# stop daemon dansguardian or call &error
sub stop_dansguardian
{
	my $self = shift;

 if ($self->status_dansguardian())
 {
  # service started
  if ($self->{'config_hash'}->{'stop_cmd'})
  {
   # stop with script
   my $out = $self->backquote_command("$self->{'config_hash'}->{'stop_cmd'} 2>&1 </dev/null");
   if ($?){ $self->error($self->text('error_stop',$out)); }
   else { $self->webmin_log("stop"); }
  }
  else
  {
   # send TERM signal
   $self->kill_logged('TERM', $self->find_byname("dansguardian"));
   $self->webmin_log("stop");
  }
 }
 else
 {
  # service stoped
  $self->error($self->{'text_hash'}->{'proces_status2'});
 }
}

# &restart_dansguardian()
# restart daemon dansguardian or call &error
sub restart_dansguardian
{
	my $self = shift;

 if ($self->status_dansguardian())
 {
  # service started
  if ($self->{'config_hash'}->{'restart_cmd'})
  {
   # restart with script
   my $out = $self->backquote_command("$self->{'config_hash'}->{'restart_cmd'} 2>&1 </dev/null");
   if ($?){ $self->error($self->text('error_restart',$out)); }
   else {$self->webmin_log("restart");}
  }
  else
  {
   # send HUP signal
   $self->kill_logged('HUP', $self->find_byname("dansguardian"));
   $self->webmin_log("restart");
  }
 }
 else
 {
  # service stoped
  $self->error($self->{'text_hash'}->{'proces_status2'});
 }
}

# &parse_config_file([file])
# return hash with name=value pairs from dansguardian configuration file or call &error
sub parse_config_file
{
	my $self = shift;

 my $conf_file = $_[0] ? $_[0] : $self->{'config_hash'}->{'dansguardian_conf'};
 $self->lock_file($conf_file);
 my $conf;
 open (INFL,$conf_file) || $self->error($self->text('error_open', $conf_file));
 while (<INFL>)
 {
  chomp;
  $self->trunc_space($_);
  unless ($_=~/^#/ || $_=~/^$/)
  {
   my @options = split (/=/,$_);
   $options[0] = $self->trunc_space($options[0]);
	$options[1] =~ s/#(.*?)$//;
   $options[1] = $self->trunc_space($options[1]);
   $conf -> {$options[0]} = $options[1];
  }
 }
 close INFL;
 $self->unlock_file($conf_file);
 return $conf;
}

# &get_config_value(name,\%hash)
# return value for name from hash
sub get_config_value
{
	my $self = shift;

 return $_[1]->{$_[0]};
}

# &set_config_value(name,value,\%hash)
# set new value for name from hash
sub set_config_value
{
	my $self = shift;

 $_[2]->{$_[0]}=$_[1];
}

# &del_config_value(name,\%hash)
# del name from hash
sub del_config_value
{
	my $self = shift;

 delete $_[1]->{$_[0]};
}

# &save_new_options(\%hash,[file])
# save all new options from hash to dansguardian configuration file, 
# change union options value, delete all other options
sub save_new_options
{
	my $self = shift;

 # get config file
 my $conf_file = $_[1] ? $_[1] : $self->{'config_hash'}->{'dansguardian_conf'};
  
 # parse config file
 $self->lock_file($conf_file);
 my $old_conf = $self->parse_config_file($conf_file);
 
 # get new options
 my @new;
 foreach (keys %{$_[0]})
 {
  unless (exists $old_conf->{$_}) { push (@new,$_); }
 }
 
 # get del & union options
 my (@del, @union);
 foreach (keys %{$old_conf})
 {
  if (exists $_[0]->{$_}) { push (@union,$_); }
  else { push (@del,$_); }
 }
 
 # read config file
 my $old_file = $self->read_file_lines($conf_file);
 
 foreach (@{$old_file})
 {
  my @str_old = split (/=/, $_);
  my $str = $self->trunc_space($str_old[0]);
  foreach my $i (@union)
  {
   # modify options
   $_ = "$i = $_[0]->{$i}" if ($str eq $i);
  }
  foreach my $j (@del)
  {
   # delete options
   undef $_ if ($str eq $j);
  }
 }
 
 # add new options
 foreach (@new)
 {
  my (@temp,$comment);
  foreach my $i (@{$old_file})
  {
   # add new options under comment
   push (@temp,$i);
   if (!$comment && $i=~/^#\s*$_\s*=/)
   {
    push (@temp,"$_=$_[0]->{$_}");
    $comment=1;
   }
  }
  # add new options in the end of file
  push (@temp,"$_=$_[0]->{$_}") if (!$comment);
  @{$old_file}=@temp;
 } 
 
 # save config file
 $self->flush_file_lines();
 $self->unlock_file($conf_file);
}

# &check_option_version($op_ver, $pr_ver)
# return 1 if option supported in this version
sub check_option_version
{
	my $self = shift;

 my $version = $_[1];
 my @version = split (/\./,$version);
 my $v1 = $version[0]*1000000+$version[1]*1000+$version[2];
 my @vers_opt = split (/\./,$_[0]);
 return 1 if ($v1 >= ($vers_opt[0]*1000000+$vers_opt[1]*1000+$vers_opt[2]));
 return 0;
}

# &get_option_list()
# return array of configuration options (section, name, type, version) for installed dansguardian version
# type: 0 - (text,default), 1 - (on,off,default), 2 - (0,1,default), 3 - (long text,default), 4 - text
sub get_option_list
{
	my $self = shift;

 my @option = ( 
					#---Logging & Reporting Settings---
					['log', 'reportinglevel', 0,0],
					['log', 'accessdeniedaddress', 3,0],
					['log', 'nonstandarddelimiter', 1,0],
					['log', 'loglevel', 0,0],
					['log', 'logexceptionhits', 1,0],
					['log', 'logfileformat', 0,0],
					['log', 'loglocation', 3,0],	
					
 					#---Network Settings---
					['network', 'filterip', 0,0],
					['network', 'filterport', 0,0],
					['network', 'proxyip', 0,0],
					['network', 'proxyport', 0,0],
					
					#---Image & Langs---
					['image', 'languagedir', 3,0],
					['image', 'language', 3,0],
					['image', 'usecustombannedimage', 2,0],
					['image', 'custombannedimagefile', 3,0],
					
					#---Filter groups options & Weighted phrase mode---
					['filtergroup', 'showweightedfound', 1,0],
					['filtergroup', 'weightedphrasemode', 0,0],
					['filtergroup', 'phrasefiltermode', 0,0],
					['filtergroup', 'filtergroups', 0,0],
					['filtergroup', 'filtergroupslist', 4,0],
					
					#---Authentication files location---
					['auth', 'bannediplist', 4,0],
					['auth', 'exceptioniplist', 4,0],
					['auth', 'banneduserlist', 4,0],
					['auth', 'exceptionuserlist', 4,0],
					
					#---Misc settings---
					['misc', 'urlcachenumber', 0,0],
					['misc', 'urlcacheage', 0,0],
					['misc', 'preservecase', 2,0],
					['misc', 'hexdecodecontent', 2,0],
					['misc', 'forcequicksearch', 2,0],
					['misc', 'reverseaddresslookups', 1,0],
					['misc', 'reverseclientiplookups', 1,0],
					['misc', 'createlistcachefiles', 1,0],
					['misc', 'maxuploadsize', 0,0],
					['misc', 'maxcontentfiltersize', 0,0],
					['misc', 'usernameidmethodproxyauth', 1,0],
					['misc', 'usernameidmethodntlm', 1,0],
					['misc', 'usernameidmethodident', 1,0],
					['misc', 'preemptivebanning', 1,0],
					['misc', 'forwardedfor', 1,0],
					['misc', 'usexforwardedfor', 1,0],
					['misc', 'logconnectionhandlingerrors', 1,0],
					
					#---Process & Forkoptions
					['process', 'maxchildren', 0,0],
					['process', 'minchildren', 0,0],
					['process', 'minsparechildren', 0,0],
					['process', 'preforkchildren', 0,0],
					['process', 'maxsparechildren', 0,0],
					['process', 'ipcfilename', 3,0],
					['process', 'urlipcfilename', 3,0],
					['process', 'pidfilename', 0,0],
					#['process', 'nodaemon', 1,0], # error in the TSL
					['process', 'nologger', 1,0],
					['process', 'daemonuser', 0,0],
					['process', 'daemongroup', 0,0],
					['process', 'softrestart', 1,0]);
 my @option_ver;
 my $version = $self->get_version();
 foreach (@option)
 {
  push (@option_ver,$_) if $self->check_option_version($_->[3],$version);
 }
 return \@option_ver;
}

# &get_filter_option_list()
# return array of filter configuration options (name, type, version) for installed dansguardian version
# type: 0 - text, 1 - (text,default)
sub get_filter_option_list
{
	my $self = shift;

 my @option = ( 
					#--- Content filtering files location
					['bannedphraselist', 0,0],
					['weightedphraselist', 0,0],
					['exceptionphraselist', 0,0],
					['bannedsitelist', 0,0],
					['greysitelist', 0,0],
					['exceptionsitelist', 0,0],
					['bannedurllist', 0,0],
					['greyurllist', 0,0],
					['exceptionurllist', 0,0],
					['bannedregexpurllist', 0,0],
					['bannedextensionlist', 0,0],
					['bannedmimetypelist', 0,0],
					['picsfile', 0,0],
					['contentregexplist', 0,0],
					
					#--- Other
					['naughtynesslimit', 1,0],
					['bypass', 1,0],
					['bypasskey', 1,0]);
 my @option_ver;
 my $version = $self->get_version();
 foreach (@option)
 {
  push (@option_ver,$_) if $self->check_option_version($_->[2],$version);
 }
 return \@option_ver;
}

# &list_filter_files()
# return array of filter groups config hash
sub list_filter_files
{
	my $self = shift;

 my ($idx, @filters, $f) = 0;
 my $dir = $self->{'config_hash'}->{'dansguardian_dir'};
 opendir (DIR, $dir) || $self->error($self->text('error_open_dir', $dir));
 while (defined ($f = readdir (DIR)))
 {
  next if $f =~ /^\.\.?$/;
  if ($f =~ /^dansguardianf(\d+?)\.conf$/)
  {
   my $name = $1;
   my $conf = $self->parse_config_file("$dir/$f");
	push (@filters,{'idx' => $idx,'file' => "$dir/$f",'name' => $name,'conf' => $conf});
	$idx++;
  }
 }
 closedir (DIR);
 return \@filters;
}

# &get_filter_group(idx)
# return conf hash for idx filter group
sub get_filter_group
{
	my $self = shift;

 my $filters = $self->list_filter_files();
 foreach (@{$filters})
 {
  return $_ if ($_->{'idx'} eq $_[0])
 }
 return 0;
}

# &check_number_valid()
# return valid filter group number
sub check_number_valid
{
	my $self = shift;

 my $filters = $self->list_filter_files();
 my @n;
 foreach (@$filters) { push (@n, $_->{'name'}); }
 my $i = 1;
 foreach (sort @n)
 {
  if ($_ != $i)
  {
	return $i;
  }
  $i++;
 }
 return $i;
}

# &add_filter_group()
# add new filter group file
sub add_filter_group
{
	my $self = shift;

 return unless(-e "$self->{'config_hash'}->{'dansguardian_dir'}/dansguardianf1.conf");
 
 # add filter group file
 my $n = $self->check_number_valid();
 $self->copy_source_dest("$self->{'config_hash'}->{'dansguardian_dir'}/dansguardianf1.conf", "$self->{'config_hash'}->{'dansguardian_dir'}/dansguardianf$n.conf");
 
 # change config option
 my $conf = $self->parse_config_file();
 $self->set_config_value('filtergroups', $n, $conf);
 $self->save_new_options($conf);
 
 if ($self->{'config_hash'}->{'save_apply'} && $self->status_dansguardian())
 {
  # restart dansguardian
  $self->restart_dansguardian();
 }
}

# &delete_filter_group($file, $name)
# delete filter group
sub delete_filter_group
{
	my $self = shift;

 my ($file, $name) = @_;
 my $n = $self->check_number_valid();
 
 # change config option
 my $conf = $self->parse_config_file();
 if (($name + 1) <= $n)
 {
  $name--;
  $self->set_config_value('filtergroups', $name, $conf);
 }
 else
 {
  $n--;
  $self->set_config_value('filtergroups', $n, $conf);
 }
 $self->save_new_options($conf);
 
 # delete file
 my (undef, $out) = $self->unlink_file($file);
 $self->error($self->text('error_open', $file)) if $out;
}

# &download_blacklist($url, $path);
# download file from url
sub download_blacklist
{
	my $self = shift;

 # print download header
 print "<tr $self->{'tb'}><td><b>$self->{'text_hash'}->{'downloading_title'}</b></td></tr>\n";
 print "<tr $self->{'cb'}><td><pre>";

 # get templete file to download
 my $file = $_[1];
 
 # downloading tar.gz file
 my $error;
 my $progress_callback_url = $self->convert_osdn_url($_[0]);
 my ($host, $port, $page, $ssl) = $self->parse_http_url($progress_callback_url);
 $self->http_download($host, $port, $page, $file, \$error,'progress_callback', $ssl);
 $self->error($error) if $error; 
 print "</pre></td></tr>\n";
 
 return $file;
}

# &install_blacklist($path)
# untar download file to DG config dir
sub install_blacklist
{
	my $self = shift;

 # print install header
 print "<tr $self->{'tb'}><td><b>$self->{'text_hash'}->{'installing_title'}</b></td></tr>\n";
 print "<tr $self->{'cb'}><td><pre>";

 my $path = $_[0];
 $path =~ /^(\S*\/)/;
 my $dir = $1;
 my $qdir = quotemeta($dir);
 my $qpath = quotemeta($path);
 
 # untar file
 if (!$self->has_command("tar")) 
 { 
  # error tar
  $self->error($self->text('upload_ecmd', "tar")); 
 }
 if ($path =~ /\.(tar\.gz|tgz)$/i) 
 {
  if (!$self->has_command("gunzip")) 
  { 
   # error gunzip
   $self->error($self->text('upload_ecmd', "gunzip")); 
  }
  else 
  {
   # untar and gzip command
   open(OUT,"(cd $qdir && gunzip -c $qpath | tar xvf -) 2>&1 </dev/null |");
	while(<OUT>) 
	{
	 # print output
	 print "$_";
	}
	close(OUT);
	$self->error($self->text('error_untar',$?)) if ($?);
  }
 }
 elsif ($path =~ /\.tar$/i) 
 {
  # untar command
  open(OUT,"(cd $qdir && gunzip -c $qpath | tar xvf -) 2>&1 </dev/null |");
  while(<OUT>) 
  {
	# print output
	print "$_";
  }
  close(OUT);
  $self->error($self->text('error_untar',$?)) if ($?);
 }
 else 
 {
  # doesn't look possible
  $self->error($self->{'text_hash'}->{'upload_notcomp'});
 }
 
 # delete tar file
 $self->unlink_file($path);
 
 # set owners & permissions
 if (-d "$self->{'config_hash'}->{'dansguardian_dir'}/blacklists")
 {
  $self->{'out'} = $self->backquote_command("chown -R root.root $self->{'config_hash'}->{'dansguardian_dir'}/blacklists 2>&1 </dev/null");
  $self->error($self->{'out'}) if $?;
  $self->{'out'} = $self->backquote_command("chmod -R 755 $self->{'config_hash'}->{'dansguardian_dir'}/blacklists 2>&1 </dev/null");
  $self->error($self->{'out'}) if $?;
 }
 print "</pre></td></tr>\n";
}

# &print_filter_file($file)
# print file string (for textarea)
sub print_filter_file
{
	my $self = shift;

 open(INFL, $_[0]);
 while(<INFL>)
 {
  print $self->html_escape($_);
 }
 close(INFL);
}

# &trunc_space($string)
# delete spaces from string
sub trunc_space
{
	my $self = shift;

 local $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}


1;

