#!/usr/bin/perl
package dovecot::DovecotLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'dovecot::DovecotLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for parsing the dovecot config file

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

@{$self->{'supported_protocols_array'}} = ( "imap", "pop3", "imaps", "pop3s" );
@{$self->{'supported_auths_array'}} = ( "anonymous", "plain", "digest-md5", "cram-md5", "apop" );
@{$self->{'mail_envs_array'}} = ( undef, "maildir:~/Maildir", "mbox:~/mail/:INBOX=/var/mail/%u",
	       "maildir:~/Maildir:mbox:~/mail/" );

1;

1;
		return $self;
}

# get_config()
# Returns a list of dovecot config entries
sub get_config
{
	my $self = shift;

if (!defined(@{$self->{'get_config_cache_array'}})) {
	@{$self->{'get_config_cache_array'}} = ( );
	local $lnum = 0;
	local ($section, @sections);
	open(CONF, $self->{'config_hash'}->{'dovecot_config'});
	while(<CONF>) {
		s/\r|\n//g;
		if (/^\s*(#?)\s*([a-z0-9\_]+)\s*(\S*)\s*\{\s*$/) {
			# Start of a section .. add this as a value too
			local $oldsection = $section;
			if ($section) {
				push(@sections, $section);	# save old
				}
			$section = { 'name' => $2,
				     'value' => $3,
				     'enabled' => !$1,
				     'section' => 1,
				     'members' => [ ],
				     'indent' => scalar(@sections),
				     'line' => $lnum,
				     'eline' => $lnum };
			if ($oldsection) {
				$section->{'sectionname'} =
					$oldsection->{'name'};
				$section->{'sectionvalue'} =
					$oldsection->{'value'};
				}
			push(@{$self->{'get_config_cache_array'}}, $section);
			}
		elsif (/^\s*(#?)\s*}\s*$/ && $section) {
			# End of a section
			$section->{'eline'} = $lnum;
			$section->{'eline'} = $lnum;
			if (@sections) {
				$section = pop(@sections);
				}
			else {
				$section = undef;
				}
			}
		elsif (/^\s*(#?)([a-z0-9\_]+)\s+=\s*(.*)/) {
			# A directive inside a section
			local $dir = { 'name' => $2,
				       'value' => $3,
				       'enabled' => !$1,
				       'line' => $lnum };
			if ($section) {
				$dir->{'sectionname'} = $section->{'name'};
				$dir->{'sectionvalue'} = $section->{'value'};
				push(@{$section->{'members'}}, $dir);
				$section->{'eline'} = $lnum;
				}
			push(@{$self->{'get_config_cache_array'}}, $dir);
			}
		$lnum++;
		}
	close(CONF);
	}
return \@{$self->{'get_config_cache_array'}};
}

# find(name, &config, [disabled-mode], [sectionname], [sectionvalue])
# Mode 0=enabled, 1=disabled, 2=both
sub find
{
	my $self = shift;

local ($name, $conf, $mode, $sname, $svalue) = @_;
local @rv = grep { !$_->{'section'} &&
		   $_->{'name'} eq $name &&
		   ($mode == 0 && $_->{'enabled'} ||
		    $mode == 1 && !$_->{'enabled'} || $mode == 2) } @$conf;
if (defined($sname)) {
	# If a section was requested, limit to it
	@rv = grep { $_->{'sectionname'} eq $sname &&
		     $_->{'sectionvalue'} eq $svalue } @rv;
	}
return wantarray ? @rv : $rv[0];
}

# find_value(name, &config, [disabled-mode], [sectionname], [sectionvalue])
# Mode 0=enabled, 1=disabled, 2=both
sub find_value
{
	my $self = shift;

local @rv = $self->find(@_);
if (wantarray) {
	return map { $_->{'value'} } @rv;
	}
else {
	return $rv[0]->{'value'};
	}
}

# find_section(name, &config, [disabled-mode], [sectionname], [sectionvalue])
# Returns a Dovecot config section object
sub find_section
{
	my $self = shift;

local ($name, $conf, $mode, $sname, $svalue) = @_;
local @rv = grep { $_->{'section'} &&
		   $_->{'name'} eq $name &&
		   ($mode == 0 && $_->{'enabled'} ||
		    $mode == 1 && !$_->{'enabled'} || $mode == 2) } @$conf;
if (defined($sname)) {
	# If a section was requested, limit to it
	@rv = grep { $_->{'sectionname'} eq $sname &&
		     $_->{'sectionvalue'} eq $svalue } @rv;
	}
return wantarray ? @rv : $rv[0];
}

# save_directive(&conf, name|&dir, value, [sectionname], [sectionvalue])
# Updates one directive in the config file
sub save_directive
{
	my $self = shift;

local ($conf, $name, $value, $sname, $svalue) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'dovecot_config'});
local $dir = ref($name) ? $name : $self->find($name, $conf, 0, $sname, $svalue);
local $newline = ref($name) ? "$name->{'name'} = $value" : "$name = $value";
if ($sname) {
	$newline = "  ".$newline;
	}
if ($dir && defined($value)) {
	# Updating some directive
	$lref->[$dir->{'line'}] = $newline;
	$dir->{'value'} = $value;
	}
elsif ($dir && !defined($value)) {
	# Deleting some directive
	splice(@$lref, $dir->{'line'}, 1);
	$self->renumber($conf, $dir->{'line'}, -1);
	@$conf = grep { $_ ne $dir } @$conf;
	}
elsif (!$dir && defined($value)) {
	# Adding some directive .. put it after the commented version, if any
	local $cmt = $self->find($name, $conf, 1, $sname, $svalue);
	if ($cmt) {
		# After comment
		splice(@$lref, $cmt->{'line'}+1, 0, $newline);
		$self->renumber($conf, $cmt->{'line'}+1, 1);
		push(@$conf, { 'name' => $name,
			       'value' => $value,
			       'line' => $cmt->{'line'}+1,
			       'sectionname' => $sname,
			       'sectionvalue' => $svalue });
		}
	elsif ($sname) {
		# Put at end of section
		local @insect = grep { $_->{'sectionname'} eq $sname &&
				       $_->{'sectionvalue'} eq $svalue } @$conf;
		@insect || $self->error("Failed to find section $sname $svalue !");
		local $line = $insect[$#insect]->{'line'}+1;
		splice(@$lref, $line, 0, $newline);
		$self->renumber($conf, $line, 1);
		push(@$conf, { 'name' => $name,
			       'value' => $value,
			       'line' => $line,
			       'sectionname' => $sname,
			       'sectionvalue' => $svalue });
		}
	else {
		# Need to put at end
		push(@$lref, $newline);
		push(@$conf, { 'name' => $name,
			       'value' => $value,
			       'line' => scalar(@$lref)-1,
			       'sectionname' => $sname,
			       'sectionvalue' => $svalue });
		}
	}
}

# save_section(&conf, &section)
# Updates one section in the config file
sub save_section
{
	my $self = shift;

local ($conf, $section) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'dovecot_config'});
local $indent = "  " x $section->{'indent'};
local @newlines;
push(@newlines, $indent.$section->{'name'}." ".$section->{'value'}." {");
foreach my $m (@{$section->{'members'}}) {
	push(@newlines, $indent."  ".$m->{'name'}." = ".$m->{'value'});
	}
push(@newlines, $indent."}");
local $oldlen = $section->{'eline'} - $section->{'line'} + 1;
splice(@$lref, $section->{'line'}, $oldlen, @newlines);
$self->renumber($conf, $section->{'eline'}, scalar(@newlines)-$oldlen);
$section->{'eline'} = $section->{'line'} + scalar(@newlines) - 1;
}

# renumber(&conf, line, offset)
sub renumber
{
	my $self = shift;

local ($conf, $line, $offset) = @_;
foreach my $c (@$conf) {
	$c->{'line'} += $offset if ($c->{'line'} >= $line);
	$c->{'eline'} += $offset if ($c->{'eline'} >= $line);
	}
}

# is_dovecot_running()
# Returns the PID if the server process is active, undef if not
sub is_dovecot_running
{
	my $self = shift;

# Try the configured PID file first
local $pid =$self->check_pid_file($self->{'config_hash'}->{'pid_file'});
return $pid if ($pid);

# Look in the base dir
local $base = $self->find_value("base_dir", $self->get_config(), 2);
return $self->check_pid_file("$base/master.pid");
}

# get_initscript()
# Returns the full path to the Dovecot init script
sub get_initscript
{
	my $self = shift;

if ($self->{'config_hash'}->{'init_script'}) {
	$self->foreign_require("init", "init-lib.pl");
	if ($main::init_object->{'init_mode'} eq "init") {
		return $main::init_object->action_filename($self->{'config_hash'}->{'init_script'});
		}
	}
return undef;
}

# stop_dovecot()
# Attempts to stop the dovecot server process, returning an error message or
# undef if successful
sub stop_dovecot
{
	my $self = shift;

local $script = $self->get_initscript();
if ($script) {
	local $out = $self->backquote_logged("$script stop 2>&1");
	return $? ? "<pre>$out</pre>" : undef;
	}
else {
	local $pid = $self->is_dovecot_running();
	if ($pid && kill('TERM', $pid)) {
		return undef;
		}
	else {
		return $self->{'text_hash'}->{'stop_erunning'};
		}
	}
}

# start_dovecot()
# Attempts to start the dovecot server process, returning an error message or
# undef if successful
sub start_dovecot
{
	my $self = shift;

local $script = $self->get_initscript();
local $cmd = $script ? "$script start" : $self->{'config_hash'}->{'dovecot'};
local $out = $self->backquote_logged("$cmd 2>&1");
return $? ? "<pre>$out</pre>" : undef;
}

# apply_configration()
# Stop and re-start the Dovecot server
sub apply_configuration
{
	my $self = shift;

local $pid = $self->is_dovecot_running();
if ($pid) {
	$self->stop_dovecot();
	local $err;
	for(my $i=0; $i<5; $i++) {
		$err = $self->start_dovecot();
		last if (!$err);
		sleep(1);
		}
	return $err;
	}
else {
	return $self->{'text_hash'}->{'stop_erunning'};
	}
}

# getdef(name, [&mapping])
# Returns 'Default (value)' for some config
sub getdef
{
	my $self = shift;

local $def = $self->find_value($_[0], $self->get_config(), 1);
if (defined($def)) {
	local $map;
	if ($_[1]) {
		($map) = grep { $_->[0] eq $def } @{$_[1]};
		}
	if (defined($map)) {
		return "$self->{'text_hash'}->{'default'} ($map->[1])";
		}
	else {
		return "$self->{'text_hash'}->{'default'} ($def)";
		}
	}
else {
	return $self->{'text_hash'}->{'default'};
	}
}

# get_dovecot_version()
# Returns the dovecot version number, or undef if not available
sub get_dovecot_version
{
	my $self = shift;

local $out = `$self->{'config_hash'}->{'dovecot'} --version 2>&1`;
return $out =~ /([0-9\.]+)/ ? $1 : undef;
}


1;

