#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Update login options

&ModuleLoader::preload_module('dovecot_class', "dovecot::DovecotLib");
$self = new dovecot::DovecotLib:: 'dovecot_class',%$self;

$self->ReadParse();
$self->error_setup($self->{'text_hash'}->{'login_err'});
$self->lock_file($self->{'config_hash'}->{'dovecot_config'});
$self->{'conf'} = $self->get_config();

# Allowed and default realm
$self->save_directive($self->{'conf'}, "auth_realms",
		$self->{'in_hash'}->{'realms_def'} ? undef : $self->{'in_hash'}->{'realms'});
$self->save_directive($self->{'conf'}, "auth_default_realm",
		$self->{'in_hash'}->{'realm_def'} ? undef : $self->{'in_hash'}->{'realm'});

# Authentication mechanisms
if ($self->find("auth_mechanisms", $self->{'conf'}, 2)) {
	$self->save_directive($self->{'conf'}, "auth_mechanisms",
			join(" ", split(/\0/, $self->{'in_hash'}->{'mechs'})));
	}
else {
	$self->save_directive($self->{'conf'}, "mechanisms",
			join(" ", split(/\0/, $self->{'in_hash'}->{'mechs'})), "auth","default");
	}

# User database
$self->{'userdb'} = $self->{'in_hash'}->{'usermode'};
if ($self->{'in_hash'}->{'usermode'} eq 'passwd-file') {
	-r $self->{'in_hash'}->{'passwdfile'} || $self->error($self->{'text_hash'}->{'login_epasswdfile'});
	$self->{'userdb'} .= " ".$self->{'in_hash'}->{'passwdfile'};
	}
elsif ($self->{'in_hash'}->{'usermode'} eq 'static') {
	$self->{'in_hash'}->{'uid'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'login_euid'});
	$self->{'in_hash'}->{'gid'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'login_egid'});
	$self->{'in_hash'}->{'home'} || $self->error($self->{'text_hash'}->{'login_ehome'});
	$self->{'userdb'} .= " uid=".$self->{'in_hash'}->{'uid'}." gid=".$self->{'in_hash'}->{'gid'}.
		   " home=".$self->{'in_hash'}->{'home'};
	}
elsif ($self->{'in_hash'}->{'usermode'} eq 'ldap') {
	-r $self->{'in_hash'}->{'ldap'} || $self->error($self->{'text_hash'}->{'login_eldap'});
	$self->{'userdb'} .= " ".$self->{'in_hash'}->{'ldap'};
	}
elsif ($self->{'in_hash'}->{'usermode'} eq 'pgsql') {
	-r $self->{'in_hash'}->{'pgsql'} || $self->error($self->{'text_hash'}->{'login_epgsql'});
	$self->{'userdb'} .= " ".$self->{'in_hash'}->{'pgsql'};
	}
elsif ($self->{'in_hash'}->{'usermode'} eq 'sql') {
	-r $self->{'in_hash'}->{'sql'} || $self->error($self->{'text_hash'}->{'login_esql'});
	$self->{'userdb'} .= " ".$self->{'in_hash'}->{'sql'};
	}
elsif ($self->{'in_hash'}->{'usermode'} eq '') {
	$self->{'userdb'} = $self->{'in_hash'}->{'other'};
	}
if ($self->{'usec'} = $self->find_section("userdb", $self->{'conf'}, undef, "auth", "default")) {
	# Version 1.0.alpha format, which has a userdb *section*
	($self->{'svalue'}, $self->{'args'}) = split(/\s+/, $self->{'userdb'}, 2);
	$self->{'usec'}->{'value'} = $self->{'svalue'};
	$self->{'usec'}->{'members'} = [ grep { $_->{'name'} ne 'args' }
				    @{$self->{'usec'}->{'members'}} ];
	if ($self->{'args'}) {
		$self->{'usec'}->{'members'} = [ { 'name' => 'args',
					 'value' => $self->{'args'} } ];
		}
	$self->save_section($self->{'conf'}, $self->{'usec'});
	}
elsif ($self->find("auth_userdb", $self->{'conf'}, 2)) {
	# Version 0.99 format
	$self->save_directive($self->{'conf'}, "auth_userdb", $self->{'userdb'});
	}
else {
	# Version 1.0 format
	$self->save_directive($self->{'conf'}, "userdb", $self->{'userdb'}, "auth", "default");
	}

# Password mode
$self->{'passdb'} = $self->{'in_hash'}->{'passmode'};
if ($self->{'in_hash'}->{'passmode'} eq 'dpam') {
	$self->{'passdb'} = "pam";
	}
elsif ($self->{'in_hash'}->{'passmode'} eq 'pam') {
	$self->{'in_hash'}->{'ppam'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'login_edpam'});
	if (defined($self->{'in_hash'}->{'ppam_ckey'}) && !$self->{'in_hash'}->{'ppam_ckey_def'}) {
		$self->{'ckey'} = $self->{'in_hash'}->{'ppam_ckey'};
		$self->{'ckey'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'login_eckey'});
		}
	$self->{'passdb'} .= ($self->{'in_hash'}->{'ppam_session'} ? " -session" : "").
		   ($self->{'ckey'} ? " cache_key=$self->{'ckey'}" : "").
		   " ".$self->{'in_hash'}->{'ppam'};
	}
elsif ($self->{'in_hash'}->{'passmode'} eq 'passwd-file') {
	-r $self->{'in_hash'}->{'ppasswdfile'} || $self->error($self->{'text_hash'}->{'login_epasswdfile'});
	$self->{'passdb'} .= " ".$self->{'in_hash'}->{'ppasswdfile'};
	}
elsif ($self->{'in_hash'}->{'passmode'} eq 'ldap') {
	-r $self->{'in_hash'}->{'pldap'} || $self->error($self->{'text_hash'}->{'login_eldap'});
	$self->{'passdb'} .= " ".$self->{'in_hash'}->{'pldap'};
	}
elsif ($self->{'in_hash'}->{'passmode'} eq 'pgsql') {
	-r $self->{'in_hash'}->{'ppgsql'} || $self->error($self->{'text_hash'}->{'login_epgsql'});
	$self->{'passdb'} .= " ".$self->{'in_hash'}->{'ppgsql'};
	}
elsif ($self->{'in_hash'}->{'passmode'} eq 'sql') {
	-r $self->{'in_hash'}->{'psql'} || $self->error($self->{'text_hash'}->{'login_esql'});
	$self->{'passdb'} .= " ".$self->{'in_hash'}->{'psql'};
	}
elsif ($self->{'in_hash'}->{'passmode'} eq 'bsdauth') {
	$self->{'in_hash'}->{'bsdauth_ckey_def'} || $self->{'in_hash'}->{'bsdauth_ckey'} =~ /^\S+$/ ||
		$self->error($self->{'text_hash'}->{'login_eckey'});
	$self->{'passdb'} .= " cache_key=$self->{'in_hash'}->{'bsdauth_ckey'}"
		if (!$self->{'in_hash'}->{'bsdauth_ckey_def'});
	}
elsif ($self->{'in_hash'}->{'passmode'} eq 'checkpassword') {
	-x $self->{'in_hash'}->{'checkpassword'} || $self->error($self->{'text_hash'}->{'login_echeckpassword'});
	$self->{'passdb'} .= " ".$self->{'in_hash'}->{'checkpassword'};
	}
elsif ($self->{'in_hash'}->{'passmode'} eq '') {
	$self->{'passdb'} = $self->{'in_hash'}->{'other'};
	}
# XXX other modes
if ($self->{'psec'} = $self->find_section("passdb", $self->{'conf'}, undef, "auth", "default")) {
	# Version 1.0.alpha format
	($self->{'svalue'}, $self->{'args'}) = split(/\s+/, $self->{'passdb'}, 2);
	$self->{'psec'}->{'value'} = $self->{'svalue'};
	$self->{'psec'}->{'members'} = [ grep { $_->{'name'} ne 'args' }
				    @{$self->{'psec'}->{'members'}} ];
	if ($self->{'args'}) {
		$self->{'psec'}->{'members'} = [ { 'name' => 'args',
					 'value' => $self->{'args'} } ];
		}
	$self->save_section($self->{'conf'}, $self->{'psec'});
	}
elsif ($self->find("auth_passdb", $self->{'conf'}, 2)) {
	# Version 0.99 format
	$self->save_directive($self->{'conf'}, "auth_passdb", $self->{'passdb'});
	}
else {
	# Version 1.0 format
	$self->save_directive($self->{'conf'}, "passdb", $self->{'passdb'}, "auth", "default");
	}

# Allowed UIDs and GIDs
$self->{'in_hash'}->{'fuid_def'} || $self->{'in_hash'}->{'fuid'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'login_efuid'});
$self->save_directive($self->{'conf'}, "first_valid_uid",
		$self->{'in_hash'}->{'fuid_def'} ? undef : $self->{'in_hash'}->{'fuid'});

$self->{'in_hash'}->{'luid_def'} || $self->{'in_hash'}->{'luid'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'login_eluid'});
$self->save_directive($self->{'conf'}, "last_valid_uid",
		$self->{'in_hash'}->{'luid_def'} ? undef : $self->{'in_hash'}->{'luid'});

$self->{'in_hash'}->{'fgid_def'} || $self->{'in_hash'}->{'fgid'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'login_efgid'});
$self->save_directive($self->{'conf'}, "first_valid_gid",
		$self->{'in_hash'}->{'fgid_def'} ? undef : $self->{'in_hash'}->{'fgid'});

$self->{'in_hash'}->{'lgid_def'} || $self->{'in_hash'}->{'lgid'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'login_elgid'});
$self->save_directive($self->{'conf'}, "last_valid_gid",
		$self->{'in_hash'}->{'lgid_def'} ? undef : $self->{'in_hash'}->{'lgid'});

$self->save_directive($self->{'conf'}, "mail_extra_groups",
		$self->{'in_hash'}->{'extra_def'} ? undef : $self->{'in_hash'}->{'extra'});

$self->{'in_hash'}->{'chroot_def'} || -d $self->{'in_hash'}->{'chroot'} || $self->error($self->{'text_hash'}->{'login_echroot'});
$self->save_directive($self->{'conf'}, "mail_chroot",
		$self->{'in_hash'}->{'chroot_def'} ? undef : $self->{'in_hash'}->{'chroot'});

if ($self->find("login_max_processes_count", $self->{'conf'}, 2)) {
	$self->{'in_hash'}->{'procs_def'} || $self->{'in_hash'}->{'procs'} =~ /^\d+$/ ||
		$self->error($self->{'text_hash'}->{'login_eprocs'});
	$self->save_directive($self->{'conf'}, "login_max_processes_count",
			$self->{'in_hash'}->{'procs_def'} ? undef : $self->{'in_hash'}->{'procs'});
	}

if ($self->find("login_processes_count", $self->{'conf'}, 2)) {
	$self->{'in_hash'}->{'count_def'} || $self->{'in_hash'}->{'count'} =~ /^\d+$/ ||
		$self->error($self->{'text_hash'}->{'login_ecount'});
	$self->save_directive($self->{'conf'}, "login_processes_count",
			$self->{'in_hash'}->{'count_def'} ? undef : $self->{'in_hash'}->{'count'});
	}

$self->flush_file_lines();
$self->unlock_file($self->{'config_hash'}->{'dovecot_config'});
$self->webmin_log("login");
$self->redirect("");

1;
		
}


1;

