#!/usr/bin/perl
package PostfixLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'PostfixLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
%{$self->{'postfix_config_hash'}} = ();
$self->read_file("$self->{'config_directory'}/../postfix/config", \%{$self->{'postfix_config_hash'}});

# Get the saved version number
if (open(VERSION, "$self->{'config_directory'}/../postfix/version")) {
	chop($self->{'postfix_version'} = <VERSION>);
	close(VERSION);
	}
if ($self->{'postfix_version'} >= 2) {
	$self->{'postfix_virtual_maps'} = "virtual_alias_maps";
	$self->{'postfix_ldap_timeout'} = "ldap_timeout";
	}
else {
	$self->{'postfix_virtual_maps'} = "virtual_maps";
	$self->{'postfix_ldap_timeout'} = "ldap_lookup_timeout";
	}

$self->{'config_dir'} = $self->postfix_guess_config_dir();

$self->{'virtual_type'} = $self->{'postfix_virtual_maps'} || "virtual_maps";
$self->{'virtual_maps'} = $self->postfix_get_real_value($self->{'virtual_type'});
@{$self->{'virtual_map_files_array'}} = $self->postfix_get_maps_files($self->{'virtual_maps'});
$self->{'postfix_afiles'} = [ $self->postfix_get_aliases_files($self->postfix_get_real_value("alias_maps")) ];

############################################################################
# maps [canonical, virtual, transport] support

############################################################################
# aliases support    [too lazy to create a aliases-lib.pl :-)]

1;
1;
		return $self;
}

sub postfix_guess_config_dir
{
	my $self = shift;

    my $answ = $self->{'postfix_config_hash'}->{'postfix_config_file'};
    $answ =~ /(.*)\/[^\/]*/;
    return $1;
}

# get_current_value(parameter_name)
# returns a scalar corresponding to the value of the parameter
sub postfix_get_current_value
{
	my $self = shift;

	my $out = `$self->{'postfix_config_hash'}->{'postfix_config_command'} -c $self->{'config_dir'} -h $_[0] 2>&1`;
#	if ($?) { &error(&text('postfix_query_get_efailed', $_[0], $out) . $step++); }
	return '' if $?;
	chop($out);
	return $out;
}

# get_real_value(parameter_name)
# Returns the value of a parameter, with $ substitions done
sub postfix_get_real_value
{
	my $self = shift;

	 #&error("VVV: $_[0]") if $_[1];
my $v = $self->postfix_get_current_value($_[0]);
$v =~ s/\$(\{([^\}]+)\}|([A-Za-z0-9\.\-\_]+))/$self->postfix_get_real_value($2 || $3)/ge;
return $v;
}

# get_maps_files($maps_param) : @maps_files
# parses its argument to extract the filenames of the mapping files
# supports multiple maps-files
sub postfix_get_maps_files
{
	my $self = shift;

    $_[0] =~ /:(\/[^,\s]*)(.*)/;
    (my $returnvalue, my $recurse) = ( $1, $2 );

    return ( $returnvalue,
	     ($recurse =~ /:\/[^,\s]*/) ?
	         $self->postfix_get_maps_files($recurse)
	     :
	         ()
           )
}

# get_maps($maps_name) : \@maps
# construct the mappings database taken from the map files given from the parameter
sub postfix_get_maps
{
	my $self = shift;

    if (!defined($self->{'maps_cache_hash'}->{$_[0]}))
    {
	my @maps_files = $self->postfix_get_maps_files($self->postfix_get_real_value($_[0]));
	my $number = 0;
	foreach $maps_file (@maps_files)
	{
	    open(MAPS, $maps_file);
	    my $i = 0;
	    while (<MAPS>)
	    {
		s/^#.*$//g;	# remove comments
		s/\r|\n//g;	# remove newlines
		if (/^\s*([^\s]+)\s+([^#]*)/)
		{
		    $number++;
		    my %map;
		    $map{'name'} = $1;
		    $map{'value'} = $2;
		    $map{'line'} = $i;
		    $map{'map_file'} = $maps_file;
		    $map{'file'} = $maps_file;
		    $map{'number'} = $number;
		    push(@{$self->{'maps_cache_hash'}->{$_[0]}}, \%map);
		}
		$i++;
	    }
	    close(MAPS);
	}
    }
    return $self->{'maps_cache_hash'}->{$_[0]};
}

# get_aliases_files($alias_maps) : @aliases_files
# parses its argument to extract the filenames of the aliases files
# supports multiple alias-files
sub postfix_get_aliases_files
{
	my $self = shift;

    $_[0] =~ /:(\/[^,\s]*)(.*)/;
    (my $returnvalue, my $recurse) = ( $1, $2 );

    # Yes, Perl is also a functional language -> I construct a list, and no problem, lists are flattened in Perl
    return ( $returnvalue,
	     ($recurse =~ /:\/[^,\s]*/) ?
	         $self->postfix_get_aliases_files($recurse)
	     :
	         ()
           )
}

# get_aliases() : \@aliases
# construct the aliases database taken from the aliases files given in the "alias_maps" parameter
sub postfix_get_aliases
{
	my $self = shift;

    if (!@{$self->{'aliases_cache_array'}})
    {
	my @aliases_files = $self->postfix_get_aliases_files($self->postfix_get_current_value("alias_maps"));
	my $number = 0;
	foreach $aliases_file (@aliases_files)
	{
	    open(ALIASES, $aliases_file);
	    my $i = 0;
	    while (<ALIASES>)
	    {
		s/^#.*$//g;	# remove comments
		s/\r|\n//g;	# remove newlines
		if ((/^\s*\"([^\"]*)[^:]*:\s*([^#]*)/) ||      # names with double quotes (") are special, as seen in `man aliases(5)`
		    (/^\s*([^\s:]*)[^:]*:\s*([^#]*)/))         # other names
		{
		    $number++;
		    my %alias;
		    $alias{'name'} = $1;
		    $alias{'value'} = $2;
		    $alias{'line'} = $i;
		    $alias{'alias_file'} = $aliases_file;
		    $alias{'number'} = $number;
		    push(@{$self->{'aliases_cache_array'}}, \%alias);
		}
		$i++;
	    }
	    close(ALIASES);
	}
    }
    return \@{$self->{'aliases_cache_array'}};
}

# list_aliases(files)
# Returns an array of data structures, each containing information about
# one sendmail alias
sub postfix_list_aliases
{
	my $self = shift;

local $jfiles = join(",", @{$_[0]});
local $c = $self->{'list_aliases_cache_hash'}->{$jfiles};
if (!defined($c)) {
	$c = $self->{'list_aliases_cache_hash'}->{$jfiles} = [ ];
	local $file;
	local @skip = split(/\s+/, $self->{'postfix_config_hash'}->{'alias_skip'});
	foreach $file (@{$_[0]}) {
		local $lalias;
		local $lnum = 0;
		open(AFILE, $file);
		while(<AFILE>) {
			s/\r|\n//g;	# remove newlines
			if (/^(#*)\s*([^:$ \t]+)\s*:\s*(.*)$/) {
				local(%alias, @values, $v);
				$alias{'eline'} = $alias{'line'} = $lnum;
				$alias{'file'} = $file;
				$alias{'enabled'} = $1 ? 0 : 1;
				$alias{'name'} = $2;
				$v = $alias{'value'} = $3;
				while($v =~ /^\s*,?\s*()"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*(\|)"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*()([^,\s]+)(.*)$/) {
					push(@values, $1.$2); $v = $3;
					}
				$alias{'values'} = \@values;
				$alias{'num'} = scalar(@$c);
				if ($self->indexof($alias{'name'}, @skip) < 0) {
					push(@$c, \%alias);
					$lalias = \%alias;
					}
				}
			elsif (/^(#*)\s+(\S.*)$/ && $lalias &&
			       ($1 && !$lalias->{'enabled'} ||
				!$1 && $lalias->{'enabled'})) {
				# continuation of last alias
				$lalias->{'eline'} = $lnum;
				local $v = $2;
				$lalias->{'value'} .= $v;
				while($v =~ /^\s*,?\s*()"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*(\|)"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*()([^,\s]+)(.*)$/) {
					push(@{$lalias->{'values'}}, $1.$2); $v = $3;
					}
				}
			else { $lalias = undef; }
			$lnum++;
			}
		close(AFILE);
		}
	}
return @$c;
}


1;

