#!/usr/bin/perl
package SambaLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'SambaLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# samba-lib.pl
# Common functions for editing the samba config file
# XXX privileges for groups with 'net' command
%{$self->{'config_hash'}} = ();
$self->read_file("$self->{'config_directory'}/../samba/config", \%{$self->{'config_hash'}});

if (open(VERSION, "$self->{'module_config_directory'}/version")) {
        chop($self->{'samba_version'} = <VERSION>);
        close(VERSION);
        }
else
{
 my $out;
 $self->{'samba_version'}=$self->get_samba_version(\$out);
}

$self->{'has_pdbedit'} = ( $self->{'samba_version'} >= 3 && $self->has_command($self->{'config_hash'}->{'pdbedit'}) );
$self->{'has_smbgroupedit'} = 1 if ($self->has_command($self->{'config_hash'}->{'smbgroupedit'}));
$self->{'has_net'} = 1 if ($self->{'config_hash'}->{'net'} =~ /^(\S+)/ && $self->has_command("$1"));
$self->{'has_groups'} = ( $self->{'samba_version'} >= 3 && ($self->{'has_smbgroupedit'} || $self->{'has_net'}) );

# The list of synonyms used by samba for parameter names
@{$self->{'synon_array'}} = (	"writeable,write ok,writable",
		"public,guest ok",
		"printable,print ok",
		"allow hosts,hosts allow",
		"deny hosts,hosts deny",
		"create mode,create mask",
		"directory mode,directory mask",
		"path,directory",
		"exec,preexec",
		"group,force group",
		"only guest,guest only",
		"user,username,users",
		"default,default service",
		"auto services,preload",
		"lock directory,lock dir",
		"max xmit,max packet",
		"root directory,root dir,root",
		"case sensitive,case sig names",
		"idmap uid,winbind uid",
		"idmap gid,winbind gid",
	 );
foreach $s (@{$self->{'synon_array'}}) {
	foreach $ss (split(/,/ , $s)) {
		$self->{'synon_hash'}->{$ss} = $s;
		}
	}

# Default values for samba configuration parameters
%{$self->{'default_values_hash'}} = (	"allow hosts",undef,
			"alternate permissions","no",
			"available","yes",
			"browseable","yes",
			"comment",undef,
			"create","yes",
			"create mode","755",
			"directory mode","755",
			"default case","lower",
			"case sensitive","no",
			"mangle case","no",
			"preserve case","yes",
			"short preserve case","yes",
			"delete readonly","no",
			"deny hosts",undef,
			"dont descend",undef,
			"force group",undef,
			"force user",undef,
			"force create mode","000",
			"force directory mode","000",
			"guest account","nobody",	# depends
			"guest only","no",
			"hide dot files","yes",
			"invalid users",undef,
			"locking","yes",
			"lppause command",undef,	# depends
			"lpq command",undef,		# depends
			"lpresume command",undef,	# depends
			"lprm command",undef,		#depends
			"magic output",undef,		# odd..
			"magic script",undef,
			"mangled map",undef,
			"mangled names","yes",
			"mangling char","~",
			"map archive","yes",
			"map system","no",
			"map hidden","no",
			"max connections",0,
			"only user","no",
			"fake oplocks","no",
			"oplocks","yes",
			"os level",20,
			"level2 oplocks","no",
			"load printers","yes",
			"min print space",0,
			"path",undef,
			"postscript","no",
			"preexec",undef,
			"print command",undef,
#			"print command","lpr -r -P %p %s",
			"printer",undef,
			"printer driver",undef,
			"public","no",
			"read list",undef,
			"revalidate","no",
			"root preexec",undef,
			"root postexec",undef,
			"set directory","no",
			"share modes","yes",
			"socket options","TCP_NODELAY",
			"strict locking","no",
			"sync always","no",
			"unix password sync","no",
			"user",undef,
			"valid chars",undef,
			"volume",undef,		# depends
			"wide links","yes",
			"wins support","no",
			"writable","no",
			"write list",undef,
			"winbind cache time",300,
			"winbind enable local accounts","yes",
			"winbind trusted domains only","no",
			"winbind enum users","yes",
			"winbind enum groups","yes",
		);
$self->{'default_values_hash'}->{'encrypt passwords'} = 'yes' if ($self->{'samba_version'} >= 3);

@{$self->{'sock_opts_array'}} = ("SO_KEEPALIVE", "SO_REUSEADDR", "SO_BROADCAST", "TCP_NODELAY", 
	      "IPTOS_LOWDELAY", "IPTOS_THROUGHPUT", "SO_SNDBUF*", "SO_RCVBUF*",
	      "SO_SNDLOWAT*", "SO_RCVLOWAT*");

@{$self->{'protocols_array'}} = ("CORE", "COREPLUS", "LANMAN1", "LANMAN2", "NT1");

1;

1;
		return $self;
}

# list_locks()
# Returns a list of locked files as an array, in the form:
#  pid, mode, rw, oplock, file, date
sub list_locks
{
	my $self = shift;

local($l, $started, @rv);
local $out;
$self->execute_command("$self->{'config_hash'}->{samba_status_program} -L", undef, \$out, undef);
foreach $l (split(/\n/, $out)) {
	if ($l =~ /^----/) { $started = 1; }
	if ($started && $l =~ /^(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)\s+(\S+\s+\S+\s+\d+\s+\d+:\d+:\d+\s+\d+)/) {
		push(@rv, [ $1, $2, $3, $4, $5, $6 ]);
		}
	}
return @rv;
}

# get_samba_version(&out)
sub get_samba_version
{
	my $self = shift;

local $flag;
foreach $flag ("-V", "-v") {
        $self->execute_command("$self->{'config_hash'}->{'samba_server'} $flag", undef, $_[0], $_[0]);
        if (${$_[0]} =~ /(Version|Samba)\s+(CVS\s+)?[^0-9 ]*(\d+)\.(\S+)/i) {
                local $v1 = $3;
                local $v2 = $4;
                $v2 =~ s/[^0-9]//g;
                return "$v1.$v2";
                }
        }
return undef;
}

# istrue(key)
# Checks if the value of this key (or it's synonyms) in %share is true
sub istrue
{
	my $self = shift;

return $self->getval($_[0]) =~ /yes|true|1/i;
}

# isfalse(key)
# Checks if the value of this key (or it's synonyms) in %share is false
sub isfalse
{
	my $self = shift;

return $self->getval($_[0]) =~ /no|false|0/i;
}

# $self->getval(name, [&hash])
# Given the name of a key in %{$self->{'share_hash'}}, return the value. Also looks for synonyms.
# If the value is not found, a default is looked for.. this can come from
# a copied section, the [global] configuration section, or from the SAMBA
# defaults. This means that $self->getval() always returns something..
sub getval
{
	my $self = shift;

local $hash = $_[1] || \%{$self->{'share_hash'}};
local($_, $copy);
if ($self->{'synon_hash'}->{$_[0]}) {
	foreach (split(/,/, $self->{'synon_hash'}->{$_[0]})) {
		if (defined($hash->{$_})) { return $hash->{$_}; }
		}
	}
if (defined($hash->{$_[0]})) {
	return $hash->{$_[0]};
	}
elsif ($_[0] ne "copy" && ($copy = $hash->{"copy"})) {
	# this share is a copy.. get the value from the source
	$self->get_share($copy);
	return $self->getval($_[0]);
	}
else {
	# return the default value...
	return $self->default_value($_[0]);
	}
return undef;
}

# user_list(list)
# Convert a samba unix user list into a more readable form
sub user_list
{
	my $self = shift;

local($u, @rv);
foreach $u (split(/[ \t,]+/ , $_[0])) {
	if ($u =~ /^\@(.*)$/) {
		push(@rv, "group <tt>".$self->html_escape($1)."</tt>");
		}
	else {
		push(@rv, "<tt>".$self->html_escape($u)."</tt>");
		}
	}
return join("," , @rv);
}

sub yesno_input
{
	my $self = shift;

($self->{'n'} = $_[0]) =~ s/ /_/g;
return sprintf "<input type=radio name=$self->{'n'} value=yes %s> $self->{'text_hash'}->{'yes'}\n".
	       "<input type=radio name=$self->{'n'} value=no %s> $self->{'text_hash'}->{'no'}\n",
		$self->istrue($_[0]) ? "checked" : "",
		$self->isfalse($_[0]) ? "checked" : "";
}

# username_input(name)
# Outputs HTML for an username field
sub username_input
{
	my $self = shift;

($self->{'n'} = $_[0]) =~ s/ /_/g;
$self->{'v'} = $self->getval($_[0]);
print "<td><input name=$self->{'n'} size=8 value=\"$self->{'v'}\"> ",
	$self->user_chooser_button($self->{'n'}, 0),"</td>\n";
}

# username_input(name, default)
sub groupname_input
{
	my $self = shift;

($self->{'n'} = $_[0]) =~ s/ /_/g;
$self->{'v'} = $self->getval($_[0]);
print "<td><input name=$self->{'n'} size=8 value=\"$self->{'v'}\"> ",
	$self->group_chooser_button($self->{'n'}, 0),"</td>\n";
}

# list_users()
# Returns an array of all the users from the samba password file
sub list_users
{
	my $self = shift;

local(@rv, @b, $_, $lnum);
if ($self->{'has_pdbedit'}) {
	# Get list of users from the pdbedit command, which uses a configurable
	# back-end for storage
	$self->open_execute_command(*PASS, "cd / && $self->{'config_hash'}->{'pdbedit'} -L -w -s $self->{'config_hash'}->{'smb_conf'}", 1);
	}
else {
	# Read the password file directly
	$self->open_readfile(*PASS, $self->{'config_hash'}->{'smb_passwd'});
	}
while(<PASS>) {
	$lnum++;
	chop;
	s/#.*$//g;
	local @b = split(/:/, $_);
	next if (@b < 4);
	local $u = { 'name' => $b[0],  'uid' => $b[1],
		     'pass1' => $b[2], 'pass2' => $b[3],
		     'oldname' => $b[0] };
	if ($self->{'samba_version'} >= 2 && $b[4] =~ /^\[/) {
		$b[4] =~ s/[\[\] ]//g;
		$u->{'opts'} = [ split(//, $b[4]) ];
		$u->{'change'} = $b[5];
		}
	else {
		$u->{'real'} = $b[4];
		$u->{'home'} = $b[5];
		$u->{'shell'} = $b[6];
		}
	$u->{'index'} = scalar(@rv);
	$u->{'line'} = $lnum-1;
	push(@rv, $u);
	}
close(PASS);
return @rv;
}

# modify_user(&user)
# Change an existing samba user
sub modify_user
{
	my $self = shift;

if ($self->{'has_pdbedit'}) {
	# Use the pdbedit command
	if ($_[0]->{'oldname'} ne "" && $_[0]->{'oldname'} ne $_[0]->{'name'}) {
		# Username changed! Have to delete and re-create
		local $out = $self->backquote_logged(
		    "cd / && $self->{'config_hash'}->{'pdbedit'} -x -s $self->{'config_hash'}->{'smb_conf'} -u ".
		    quotemeta($_[0]->{'oldname'}));
		$? && $self->error("$self->{'config_hash'}->{'pdbedit'} failed : <pre>$out</pre>");
		$self->create_user($_[0]);
		}
	else {
		# Update update user
		local @opts = grep { $_ ne "U" } @{$_[0]->{'opts'}};
		$self->indexof("W", @{$_[0]->{'opts'}}) >= 0 && $self->error($self->{'text_hash'}->{'saveuser_ews'});
		$out = $self->backquote_logged(
			"cd / && $self->{'config_hash'}->{'pdbedit'} -r -s $self->{'config_hash'}->{'smb_conf'} -u ".
			quotemeta($_[0]->{'name'}).
			" -c '[".join("", @opts)."]' 2>&1");
		$? && $self->error("$self->{'config_hash'}->{'pdbedit'} failed : <pre>$out</pre>");
		}
	}
else {
	# Directly update the Samba password file
	$self->replace_file_line($self->{'config_hash'}->{'smb_passwd'}, $_[0]->{'line'}, $self->user_string($_[0]));
	}
}

sub user_string
{
	my $self = shift;

local @u = ($_[0]->{'name'}, $_[0]->{'uid'},
	    $_[0]->{'pass1'}, $_[0]->{'pass2'});
if ($_[0]->{'opts'}) {
	push(@u, sprintf "[%-11s]", join("", @{$_[0]->{'opts'}}));
	push(@u, sprintf "LCT-%X", time());
	push(@u, "");
	}
else {
	push(@u, $_[0]->{'real'}, $_[0]->{'home'}, $_[0]->{'shell'});
	}
return join(":", @u)."\n";
}

# set_password(user, password)
# Changes the password of a user in the encrypted password file
sub set_password
{
	my $self = shift;

local $qu = quotemeta($_[0]);
local $qp = quotemeta($_[1]);
if ($self->{'samba_version'} >= 2) {
	local $passin = "$_[1]\n$_[1]\n";
	local $ex = $self->execute_command("$self->{'config_hash'}->{'samba_password_program'} -c $self->{'config_hash'}->{'smb_conf'} -s $qu", \$passin, undef, undef);
	if ($ex) {
		# Try without -c
		$self->{'rv'} = $self->execute_command("$self->{'config_hash'}->{'samba_password_program'} -s $qu", \$passin, undef, undef);
		}
	unlink($self->{'temp'});
	return !$self->{'rv'};
	}
else {
	local $out;
	$self->execute_command("$self->{'config_hash'}->{'samba_password_program'} $qu $qp",
			 undef, \$out, \$out);
	return $out =~ /changed/i;
	}
}

# can($permissions_string, \%access, [$sname])
# check global and per-share permissions:
#
# $permissions_string = any exists permissions except 'c' (creation).
# \%access = ref on what get_module_acl() returns.
sub can
{
	my $self = shift;

local ($acl, $stype, @perm);
local ($perm, $acc, $sname) = @_;
@perm  = split(//, $perm);
$sname = $self->{'in_hash'}->{'old_name'} || $self->{'in_hash'}->{'share'} unless $sname;

{	local %share;
	$self->get_share($sname); # use local %share
	$stype = $self->istrue('printable') ? 'ps' : 'fs';
	}

# check global acl (r,w)
foreach (@perm) {
	next if ($_ ne 'r') && ($_ ne 'w');
	return 0 unless $acc->{$_ . '_' . $stype};
	}

# check per-share acl
if ($acc->{'per_' . $stype . '_acls'}) {
    $acl = $acc->{'ACL' . $stype . '_' . $sname};
    foreach (@perm) {
#        next if $_ eq 'c'; # skip creation perms for per-share acls
		return 0 if index($acl, $_) == -1;
		}
	}
return 1;	
}

# list_groups()
# Returns an array containing details of Samba groups
sub list_groups
{
	my $self = shift;

local (@rv, $group, $cmd);
if ($self->{'has_smbgroupedit'}) {
	$cmd = "$self->{'config_hash'}->{'smbgroupedit'} -v -l";
	}
else {
	$cmd = "$self->{'config_hash'}->{'net'} -s $self->{'config_hash'}->{'smb_conf'} groupmap list verbose";
	}
$self->open_execute_command(*GROUPS, $cmd, 1);
while(<GROUPS>) {
	s/\r|\n//g;
	if (/^(\S.*)/) {
		$group = { 'name' => $1,
			   'index' => scalar(@rv) };
		push(@rv, $group);
		}
	elsif (/^\s+SID\s*:\s+(.*)/i) {
		$group->{'sid'} = $1;
		}
	elsif (/^\s+Unix group\s*:\s*(.*)/i) {
		$group->{'unix'} = $1;
		}
	elsif (/^\s+Group type\s*:\s*(.*)/i) {
		$group->{'type'} = lc($1) eq 'domain group' ? 'd' :
				   lc($1) eq 'nt builtin' ? 'b' :
				   lc($1) eq 'unknown type' ? 'u' :
				   lc($1) eq 'local group' ? 'l' : $1;
		}
	elsif (/^\s+Comment\s*:\s*(.*)/i) {
		$group->{'desc'} = $1;
		}
	elsif (/^\s+Privilege\s*:\s*(.*)/i) {
		$group->{'priv'} = lc($1) eq 'no privilege' ? undef : $1;
		}
	}
close(GROUPS);
return @rv;
}

sub check_user_enabled
{
	my $self = shift;

local %share;
$self->get_share("global");
if (!$self->istrue("encrypt passwords")) {
	$self->{'err'} = $self->text('check_user1', $_[0], "conf_pass.cgi");
	}
elsif (!$self->{'config_hash'}->{'smb_passwd'} && !$self->{'has_pdbedit'}) {
	$self->{'err'} = $self->text('check_user2', $_[0], "../config.cgi?$self->{'module_name'}");
	}
if ($self->{'err'}) {
	print "<p>$self->{'err'}<p>\n";
	print "<hr>\n";
	$self->footer("", $self->{'text_hash'}->{'index_sharelist'});
	exit;
	}
}

sub check_group_enabled
{
	my $self = shift;

if ($self->{'samba_version'} < 3) {
	$self->{'err'} = $self->text('check_groups1', $_[0]);
	}
elsif (!$self->{'has_groups'}) {
	$self->{'err'} = $self->text('check_groups2', $_[0], "../config.cgi?$self->{'module_name'}");
	}
if ($self->{'err'}) {
	print "<p>$self->{'err'}<p>\n";
	print "<hr>\n";
	$self->footer("", $self->{'text_hash'}->{'index_sharelist'});
	exit;
	}
}


1;

