#!/usr/bin/perl
package SendmailLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'SendmailLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
%{$self->{'sendmail_config_hash'}} = ();
$self->read_file("$self->{'config_directory'}/../sendmail/config", \%{$self->{'sendmail_config_hash'}});

$self->{'sendmail_conf'} = $self->get_sendmailcf();
$self->{'sendmail_vfile'} = $self->sendmail_virtusers_file($self->{'sendmail_conf'});
($self->{'sendmail_vdbm'}, $self->{'sendmail_vdbmtype'}) = $self->sendmail_virtusers_dbm($self->{'sendmail_conf'});
#$sendmail_afiles = &sendmail_aliases_file($self->{'sendmail_conf'});

1;
1;
		return $self;
}

# get_sendmailcf()
# Parses sendmail.cf and return a reference to an array of options.
# Each line is a single character directive, followed by a list of values?
sub get_sendmailcf
{
	my $self = shift;

if (!@{$self->{'sendmailcf_cache_array'}}) {
	local($lnum, $i);
	$lnum = 0; $i = 0;
#	open(CF, $_[0);
	open(CF, $self->{'sendmail_config_hash'}->{'sendmail_cf'});
	while(<CF>) {
		s/^#.*$//g;	# remove comments
		s/\r|\n//g;	# remove newlines
		if (/^(\S)(\s*(.*))$/) {
			local(%opt);
			$opt{'type'} = $1;
			$opt{'value'} = $3;
			$opt{'values'} = [ split(/\s+/, $2) ];
			$opt{'line'} = $lnum;
			$opt{'eline'} = $opt{'line'};
			$opt{'pos'} = $i++;
			push(@{$self->{'sendmailcf_cache_array'}}, \%opt);
			}
		$lnum++;
		}
	close(CF);
	}
return \@{$self->{'sendmailcf_cache_array'}};
}

# find_type(name, &config)
# Returns an array of config directives of some type
sub find_type
{
	my $self = shift;

local($c, @rv);
foreach $c (@{$_[1]}) {
	if ($c->{'type'} eq $_[0]) {
		push(@rv, $c);
		}
	}
return @rv ? wantarray ? @rv : $rv[0]
           : wantarray ? () : undef;
}

# find_textfile(config, dbm)
sub find_textfile
{
	my $self = shift;

local($conf, $dbm) = @_;
if ($conf) { return $conf; }
elsif (!$dbm) { return undef; }
elsif ($dbm =~ /^(.*)\.(db|dbm|pag|dir|hash)$/i && -r $1) {
	# Database is like /etc/virtusertable.db, text is /etc/virtusertable
	return $1;
	}
elsif ($dbm =~ /^(.*)\.(db|dbm|pag|dir|hash)$/i && -r "$1.txt") {
	# Database is like /etc/virtusertable.db, text is /etc/virtusertable.txt
	return "$1.txt";
	}
elsif (-r "$dbm.txt") {
	# Database is like /etc/virtusertable, text is /etc/virtusertable.txt
	return "$dbm.txt";
	}
else {
	# Text and database have same name
	return $dbm;
	}
}

# virtusers_dbm(&config)
# Returns the filename and type of the virtusers database, or undef if none
sub sendmail_virtusers_dbm
{
	my $self = shift;

foreach $f ($self->find_type("K", $_[0])) {
        if ($f->{'value'} =~ /^virtuser\s+(\S+)[^\/]+(\S+)$/) {
		return ($2, $1);
                }
	}
return undef;
}

# virtusers_file(&config)
# Returns the filename of the text virtusers file, or undef if none
sub sendmail_virtusers_file
{
	my $self = shift;

return $self->find_textfile($self->{'sendmail_config_hash'}->{'virtusers_file'}, $self->sendmail_virtusers_dbm($_[0]));
}

# list_virtusers(textfile)
sub sendmail_list_virtusers
{
	my $self = shift;

if (!defined(@{$self->{'list_virtusers_cache_array'}})) {
	@{$self->{'list_virtusers_cache_array'}} = ( );
	local $lnum = 0;
	open(VIRT, $_[0]);
	while(<VIRT>) {
		s/\r|\n//g;     # remove newlines
		s/#.*$//g;	# remove comments
		if (/^(\S+)\s+(.*)/) {
			local(%virt);
			$virt{'from'} = $1;
			$virt{'to'} = $2;
			$virt{'line'} = $lnum;
			$virt{'file'} = $_[0];
			$virt{'num'} = scalar(@{$self->{'list_virtusers_cache_array'}});
			push(@{$self->{'list_virtusers_cache_array'}}, \%virt);
			}
		$lnum++;
		}
	close(VIRT);
	}
return @{$self->{'list_virtusers_cache_array'}};
}

# aliases_file(&config)
# Returns the alias filenames
sub sendmail_aliases_file
{
	my $self = shift;

if ($self->{'sendmail_config_hash'}->{'alias_file'}) { return [ $self->{'sendmail_config_hash'}->{'alias_file'} ]; }
else {
	local(@afiles, $o);
	foreach $o ($self->find_type("O", $_[0])) {
		if ($o->{'value'} =~ /^\s*AliasFile=(.*)$/) {
			push(@afiles, split(/,/, $1));
			}
		}
	map { s/dbm:// } @afiles;
	return \@afiles;
	}
}

# list_aliases(files)
# Returns an array of data structures, each containing information about
# one sendmail alias
sub sendmail_list_aliases
{
	my $self = shift;

local $jfiles = join(",", @{$_[0]});
local $c = $self->{'list_aliases_cache_hash'}->{$jfiles};
if (!defined($c)) {
	$c = $self->{'list_aliases_cache_hash'}->{$jfiles} = [ ];
	local $file;
	local @skip = split(/\s+/, $self->{'sendmail_config_hash'}->{'alias_skip'});
	foreach $file (@{$_[0]}) {
		local $lalias;
		local $lnum = 0;
		open(AFILE, $file);
		while(<AFILE>) {
			s/\r|\n//g;	# remove newlines
			if (/^(#*)\s*([^:$ \t]+)\s*:\s*(.*)$/) {
				local(%alias, @values, $v);
				$alias{'eline'} = $alias{'line'} = $lnum;
				$alias{'file'} = $file;
				$alias{'enabled'} = $1 ? 0 : 1;
				$alias{'name'} = $2;
				$v = $alias{'value'} = $3;
				while($v =~ /^\s*,?\s*()"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*(\|)"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*()([^,\s]+)(.*)$/) {
					push(@values, $1.$2); $v = $3;
					}
				$alias{'values'} = \@values;
				$alias{'num'} = scalar(@$c);
				if ($self->indexof($alias{'name'}, @skip) < 0) {
					push(@$c, \%alias);
					$lalias = \%alias;
					}
				}
			elsif (/^(#*)\s+(\S.*)$/ && $lalias &&
			       ($1 && !$lalias->{'enabled'} ||
				!$1 && $lalias->{'enabled'})) {
				# continuation of last alias
				$lalias->{'eline'} = $lnum;
				local $v = $2;
				$lalias->{'value'} .= $v;
				while($v =~ /^\s*,?\s*()"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*(\|)"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*()([^,\s]+)(.*)$/) {
					push(@{$lalias->{'values'}}, $1.$2); $v = $3;
					}
				}
			else { $lalias = undef; }
			$lnum++;
			}
		close(AFILE);
		}
	}
return @$c;
}


1;

