#!/usr/bin/perl
package filter::FilterLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'filter::FilterLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for creating simple mail filtering rules
# XXX use same virtualmin spam detection trick for spam module

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
&ModuleLoader::preload_module(ref($self), "filter::AliasesLib");
$self = new filter::AliasesLib:: ref($self), %$self; 
&ModuleLoader::preload_module(ref($self), "filter::AutoreplyfileLib");
$self = new filter::AutoreplyfileLib:: ref($self), %$self; 

if ($self->get_product_name() eq 'enduser_cp') {
	# If configured, check if this user has virtualmin spam filtering
	# enabled before switching away from root
	if ($self->{'config_hash'}->{'virtualmin_spam'} && -x $self->{'config_hash'}->{'virtualmin_spam'}) {
		 $self->{'out'} = `$self->{'config_hash'}->{'virtualmin_spam'} $self->{'remote_user'} </dev/null 2>/dev/null`;
		if ($self->{'out'} =~ /\d/) {
			# Yes - we can show the user this
			$self->{'global_spamassassin'} = 2;
			}
		}

	$self->switch_to_remote_user();
	$self->create_user_config_dirs();
	$self->foreign_require("procmail", "procmail-lib.pl");
	$self->foreign_require("mailbox", "mailbox-lib.pl");
	$self->foreign_require("spam", "spam-lib.pl");
	$self->{'autoreply_cmd'} = "$self->{'config_directory'}/forward/autoreply.pl";
	}
else {
	# Running under Webmin, so different modules are used
	$self->foreign_require("procmail", "procmail-lib.pl");
	$self->foreign_require("mailboxes", "mailboxes-lib.pl");
	$self->foreign_require("enduser_cf", "enduser_cf-lib.pl");
	$self->{'mail_system_module'} =
		$main::mailboxes_object->{'config_hash'}->{'mail_system'} == 1 ? "postfix" :
		$main::mailboxes_object->{'config_hash'}->{'mail_system'} == 2 ? "qmailadmin" :
							 "sendmail";
	$self->{'autoreply_cmd'} = "$self->{'config_directory'}/$self->{'mail_system_module'}/autoreply.pl";
	$self->{'user_autoreply_cmd'} = "$main::enduser_cf_object->{'config_hash'}->{'usermin_dir'}/forward/autoreply.pl";
	}

1;

1;
		return $self;
}

# list_filters([file])
# Returns a list of filter objects, which have a 1-to-1 correlation with
# procmail recipes. Any recipes too complex for parsing are not included.
sub list_filters
{
	my $self = shift;

local ($file) = @_;
local @rv;
local @pmrc = $main::procmail_object->parse_procmail_file($file || $main::procmail_object->{'procmailrc'});
foreach my $r (@pmrc) {
	# Check for un-supported recipes
	if (@{$r->{'conds'}} > 1 ||
	    $r->{'block'} ||
	    $r->{'name'}) {
		next;
		}

	# Check for flags
	local %flags = map { $_, 1 } @{$r->{'flags'}};

	# Work out condition type
	local ($condtype, $cond);
	if (@{$r->{'conds'}}) {
		($condtype, $cond) = @{$r->{'conds'}->[0]};
		if ($condtype && $condtype ne "<" && $condtype ne ">") {
			# Unsupported conditon type
			next;
			}
		}

	# Work out action type
	local ($actionspam, $actionreply);
	if ($r->{'type'} eq '|' &&
	    $r->{'action'} =~ /spamassassin|spamc/) {
		$actionspam = 1;
		}
	elsif ($r->{'type'} eq '|' &&
	       ($r->{'action'} =~ /^\Q$self->{'autoreply_cmd'}\E\s+(\S+)/ ||
		$self->{'user_autoreply_cmd'} &&
		$self->{'user_autoreply_cmd'} &&
		  $r->{'action'} =~ /^\Q$self->{'user_autoreply_cmd'}\E\s+(\S+)/)) {
		$actionreply = $1;
		}
	elsif ($r->{'type'} && $r->{'type'} ne '!') {
		# Unsupported action type
		next;
		}

	# Finally create the simple object
	local $simple = { 'condtype' => $condtype,
			  'cond' => $cond,
			  'body' => $flags{'B'},
			  'continue' => $flags{'c'},
			  'actiontype' => $r->{'type'},
			  'action' => $r->{'action'},
			  'index' => scalar(@rv),
			  'recipe' => $r };

	# Set spam flag
	if ($actionspam) {
		$simple->{'actionspam'} = 1;
		delete($simple->{'actiontype'});
		delete($simple->{'action'});
		}

	# Check for throw away
	if ($simple->{'actiontype'} eq '' &&
	    $simple->{'action'} eq '/dev/null') {
		$simple->{'actionthrow'} = 1;
		delete($simple->{'actiontype'});
		delete($simple->{'action'});
		}

	# Check for default delivery
	if ($simple->{'actiontype'} eq '' &&
	    $simple->{'action'} eq '$DEFAULT') {
		$simple->{'actiondefault'} = 1;
		delete($simple->{'actiontype'});
		delete($simple->{'action'});
		}

	# Read autoreply file
	if ($actionreply) {
		$simple->{'actionreply'} = $actionreply;
		$simple->{'reply'} = { };
		$self->read_autoreply($actionreply, $simple->{'reply'});
		delete($simple->{'actiontype'});
		delete($simple->{'action'});
		}

	# Split condition regexp into header and value, if possible
	if ($simple->{'condtype'} ne '<' && $simple->{'condtype'} ne '>' &&
	    !$simple->{'body'} &&
	    $simple->{'cond'} =~ /^\^?([a-zA-Z0-9\-]+):\s*(.*)/) {
		if ($1 eq "X-Spam-Status" && $2 eq "Yes") {
			# Special case for spam detection
			$simple->{'condspam'} = 1;
			}
		else {
			# Match on some header
			$simple->{'condheader'} = $1;
			$simple->{'condvalue'} = $2;
			}
		delete($simple->{'cond'});
		}

	push(@rv, $simple);
	}
return @rv;
}

# create_filter(&filter)
# Create a new filter by adding a procmail recipe
sub create_filter
{
	my $self = shift;

local ($filter) = @_;
local $recipe = { };
$self->update_filter_recipe($filter, $recipe);
$main::procmail_object->create_recipe($recipe);
}

# modify_filter(&filter)
# Change a filter by modifying the underlying procmail recipe
sub modify_filter
{
	my $self = shift;

local ($filter) = @_;
$self->update_filter_recipe($filter, $filter->{'recipe'});
$main::procmail_object->modify_recipe($filter->{'recipe'});
}

# update_filter_recipe(&filter, &recipe)
# Update a procmail recipe based on some filter
sub update_filter_recipe
{
	my $self = shift;

local ($filter, $recipe) = @_;

# Set condition section
local @conds;
local @flags;
if ($filter->{'condspam'}) {
	@conds = ( [ "", "X-Spam-Status: Yes" ] );
	}
elsif ($filter->{'condheader'}) {
	@conds = ( [ "", $filter->{'condheader'}.": ".
			 $filter->{'condvalue'} ] );
	}
elsif ($filter->{'condtype'} eq '<' || $filter->{'condtype'} eq '>') {
	@conds = ( [ $filter->{'condtype'}, $filter->{'cond'} ] );
	}
elsif ($filter->{'cond'}) {
	@conds = ( [ "", $filter->{'cond'} ] );
	}
$recipe->{'conds'} = \@conds;

# Set action section
if ($filter->{'actionspam'}) {
	$recipe->{'type'} = '|';
	$recipe->{'action'} = $main::spam_object->get_procmail_command();
	push(@flags, "f", "w");
	}
elsif ($filter->{'actionthrow'}) {
	$recipe->{'type'} = '';
	$recipe->{'action'} = '/dev/null';
	}
elsif ($filter->{'actiondefault'}) {
	$recipe->{'type'} = '';
	$recipe->{'action'} = '$DEFAULT';
	}
elsif ($filter->{'actionreply'}) {
	$recipe->{'type'} = '|';
	$recipe->{'action'} =
		"$self->{'autoreply_cmd'} $filter->{'reply'}->{'autoreply'} $self->{'remote_user'}";
	$self->write_autoreply($filter->{'reply'}->{'autoreply'},
			 $filter->{'reply'});
	}
else {
	$recipe->{'type'} = $filter->{'actiontype'};
	$recipe->{'action'} = $filter->{'action'};
	}

# Set flags
push(@flags, "B") if ($filter->{'body'});
push(@flags, "c") if ($filter->{'continue'});
$recipe->{'flags'} = [ $self->unique(@flags) ];
}

# delete_filter(&filter)
# Delete a filter by removing the underlying procmail rule
sub delete_filter
{
	my $self = shift;

local ($filter) = @_;
$main::procmail_object->delete_recipe($filter->{'recipe'});
}

# swap_filters(&filter1, &filter2)
# Swap two filters in the config file
sub swap_filters
{
	my $self = shift;

local ($filter1, $filter2) = @_;
$main::procmail_object->swap_recipes($filter1->{'recipe'}, $filter2->{'recipe'});
}

# file_to_folder(file, &folders, [homedir])
# Given a path like mail/foo or ~/mail/foo or $HOME/mail/foo or
# /home/bob/mail/foo, returns the folder object for it.
sub file_to_folder
{
	my $self = shift;

local ($file, $folders, $home) = @_;
$home ||= $self->{'remote_user_info_array'}->[7];
$file =~ s/^\~/$home/;
$file =~ s/^\$HOME/$home/;
if ($file !~ /^\//) {
	$file = "$home/$file";
	}
local ($folder) = grep { $_->{'file'} eq $file ||
			 $_->{'file'}.'/' eq $file } @$folders;
return $folder;
}

# get_global_spamassassin()
# Returns true if spamasassin is run globally
sub get_global_spamassassin
{
	my $self = shift;

return $self->{'global_spamassassin'} if ($self->{'global_spamassassin'});
local @recipes = $main::procmail_object->parse_procmail_file(
	$main::spam_object->{'config_hash'}->{'global_procmailrc'});
return $main::spam_object->find_spam_recipe(\@recipes) ? 1 : 0;
}

sub has_spamassassin
{
	my $self = shift;

return $self->foreign_installed("spam");
}

# get_override_alias()
# Check for any mail alias matching this user, which is defined in /etc/aliases
# as an entry matching his username.
sub get_override_alias
{
	my $self = shift;

local @afiles = split(/\t+/, $self->{'config_hash'}->{'alias_files'});
foreach my $alias ($self->list_aliases(\@afiles)) {
	if ($alias->{'name'} eq $self->{'remote_user'} && $alias->{'enabled'}) {
		return $alias;
		}
	}
return undef;
}

# describe_alias_dest(&values)
# Returns a text description of some alias destination
sub describe_alias_dest
{
	my $self = shift;

local ($values) = @_;
local @rv;
foreach my $v (@$values) {
	local ($atype, $adesc) = $self->alias_type($v);
	if ($atype == 1 && $adesc eq "\\$self->{'remote_user'}") {
		push(@rv, $self->{'text_hash'}->{'aliases_your'});
		}
	elsif ($atype == 1 && $adesc =~ /^\\(\S+)$/) {
		push(@rv, $self->text('aliases_other', "<tt>$1</tt>"));
		}
	elsif ($atype == 3 && $adesc eq "/dev/null") {
		push(@rv, $self->{'text_hash'}->{'aliases_delete'});
		}
	elsif ($atype == 4 && $adesc =~ /^(.*)\/autoreply.pl\s+(\S+)/) {
		# Autoreply from file .. check contents
		local $auto = $self->read_file_contents("$2");
		if ($auto) {
			local @lines = grep { !/^(\S+):/} split(/\r?\n/, $auto);
			local $msg = join(" ", @lines);
			$msg = substr($msg, 0, 100)." ..."
				if (length($msg) > 100);
			push(@rv, $self->text('aliases_auto', "<i>$msg</i>"));
			}
		else {
			push(@rv, $self->text('aliases_type5', "<tt>$2</tt>"));
			}
		}
	elsif ($atype == 4 && $adesc =~ /^(.*)\/filter.pl\s+(\S+)/) {
		# Apply filter file
		push(@rv, $self->text('aliases_type6', "<tt>$2</tt>"));
		}
	else {
		push(@rv, $self->text('aliases_type'.$atype, $adesc));
		}
	}
return @rv;
}

# is_table_comment(line, [force-prefix])
# Returns the comment text if a line contains a comment, like # foo. This is
# defined only because functions in aliases-lib.pl call it.
sub is_table_comment
{
	my $self = shift;

local ($line, $force) = @_;
if ($force) {
	return $line =~ /^\s*#+\s*Webmin:\s*(.*)/ ? $1 : undef;
	}
else {
	return $line =~ /^\s*#+\s*(.*)/ ? $1 : undef;
	}
}

# describe_condition(&filter)
# Returns a human-readable description of the filter condition, and a flag
# indicating if this is an 'always' condition.
sub describe_condition
{
	my $self = shift;

local ($f) = @_;
local $cond;
local $lastalways = 0;
if ($f->{'condspam'}) {
	$cond = $self->{'text_hash'}->{'index_cspam'};
	}
elsif ($f->{'condheader'}) {
	$cond = $self->text('index_cheader',
		"<tt>$f->{'condheader'}</tt>",
		"<tt>$f->{'condvalue'}</tt>");
	}
elsif ($f->{'condtype'} eq '<' || $f->{'condtype'} eq '>') {
	$cond = $self->text('index_csize'.$f->{'condtype'},
		      $self->nice_size($f->{'cond'}));
	}
elsif ($f->{'cond'}) {
	$cond = $self->text($f->{'body'} ? 'index_cre2' : 'index_cre',
		       "<tt>$f->{'cond'}</tt>");
	}
else {
	$cond = $self->{'text_hash'}->{'index_calways'};
	if (!$f->{'continue'} && !$f->{'actionspam'}) {
		$lastalways = 1;
		}
	}
return wantarray ? ( $cond, $lastalways ) : $cond;
}

# describe_action(&filter, &folder, [homedir])
# Returns a human-readable description for the delivery action for some folder
sub describe_action
{
	my $self = shift;

local ($f, $folders, $home) = @_;
local $action;
if ($f->{'actionspam'}) {
	$action = $self->{'text_hash'}->{'index_aspam'};
	}
elsif ($f->{'actionthrow'}) {
	$action = $self->{'text_hash'}->{'index_athrow'};
	}
elsif ($f->{'actiondefault'}) {
	$action = $self->{'text_hash'}->{'index_adefault'};
	}
elsif ($f->{'actiontype'} eq '!') {
	$action = $self->text('index_aforward',
		"<tt>$f->{'action'}</tt>");
	}
elsif ($f->{'actionreply'}) {
	$action = $self->text('index_areply',
	    "<i>".$self->html_escape(substr(
		$f->{'reply'}->{'autotext'}, 0, 50))."</i>");
	}
else {
	# Work out what folder
	local $folder = $self->file_to_folder($f->{'action'}, $folders, $home);
	if ($folder) {
		if ($self->get_product_name() eq 'usermin') {
			local $id = $main::mailbox_object->folder_name($folder);
			$action = $self->text('index_afolder',
			   "<a href='../mailbox/index.cgi?id=$id'>".
			   "$folder->{'name'}</a>");
			}
		else {
			local $id = $main::mailboxes_object->folder_name($folder);
			if ($self->foreign_available("mailboxes")) {
				$action = $self->text('index_afolder',
				  "<a href='../mailboxes/list_mail.cgi?user=".
				  $self->urlize($folder->{'user'})."&folder=".
				  $folder->{'index'}."'>$folder->{'name'}</a>");
				}
			else {
				$action = $self->text('index_afolder',
						$folder->{'name'});
				}
			}
		}
	else {
		$action = $self->text('index_afile',
				"<tt>$f->{'action'}</tt>");
		}
	}
if ($f->{'continue'}) {
	$action = $self->text('index_acontinue', $action);
	}
return $action;
}


1;

