#!/usr/bin/perl
package gnupg::GnupgLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'gnupg::GnupgLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# gnupg-lib.pl
# Functions for managing gnupg keys, signing, encrypting and so on

if (!$self->{'module_name'}) {
	# Only do this if we are the primary library for the usermin gnupg mod
	&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
	$self->init_config();
	&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
	$self->switch_to_remote_user();
	$self->create_user_config_dirs();
	}
$self->foreign_require("proc", "proc-lib.pl");

$self->{'gpgpath'} = $self->{'config_hash'}->{'gpg'} || "gpg";

1;

1;
		return $self;
}

# list_keys()
# Returns an array of all GnuPG keys
sub list_keys
{
	my $self = shift;

local (@rv, %kmap);
open(GPG, "$self->{'gpgpath'} --list-keys 2>/dev/null |");
while(<GPG>) {
	if (/^pub\s+(\S+)\/(\S+)\s+(\S+)\s+(.*)\s+<(\S+)>/ ||
	    /^pub\s+(\S+)\/(\S+)\s+(\S+)\s+(.*)/) {
		local $k = { 'size' => $1,
			     'key' => $2,
			     'date' => $3,
			     'name' => $4 ? [ $4 ] : [ ],
			     'email' => $5 ? [ $5 ] : $4 ? [ "" ] : [ ],
			     'index' => scalar(@rv) };
		$kmap{$k->{'key'}} = $k;
		while(1) {
			$_ = <GPG>;
			last if ($_ !~ /\S/);
			if (/^sub\s+(\S+)\/(\S+)\s+/) {
				push(@{$k->{'key2'}}, $2);
				}
			elsif (/^uid\s+(.*)\s+<(\S+)>/ ||
			       /^uid\s+(.*)/) {
				push(@{$k->{'name'}}, $1);
				push(@{$k->{'email'}}, $2);
				}
			}
		push(@rv, $k);
		}
	}
close(GPG);
open(GPG, "$self->{'gpgpath'} --list-secret-keys 2>/dev/null |");
while(<GPG>) {
	if (/^sec\s+(\S+)\/(\S+)\s+(\S+)\s+(.*)/ && $kmap{$2}) {
		$kmap{$2}->{'secret'}++;
		}
	}
close(GPG);
return @rv;
}

# list_secret_keys()
# List list_keys, but only returns secret ones
sub list_secret_keys
{
	my $self = shift;

return grep { $_->{'secret'} } $self->list_keys();
}

# key_fingerprint(&key)
sub key_fingerprint
{
	my $self = shift;

local $fp;
local $_;
open(GPG, "$self->{'gpgpath'} --fingerprint \"$_[0]->{'name'}->[0]\" |");
while(<GPG>) {
	if (/fingerprint\s+=\s+(.*)/) {
		$fp = $1;
		}
	}
close(GPG);
return $fp;
}

# get_passphrase(&key)
sub get_passphrase
{
	my $self = shift;

open(PASS, "$self->{'user_module_config_directory'}/pass.$_[0]->{'key'}") ||
  open(PASS, "$self->{'user_module_config_directory'}/pass") || return undef;
local $pass = <PASS>;
close(PASS);
chop($pass);
return $pass;
}

# put_passphrase(pass, &key)
sub put_passphrase
{
	my $self = shift;

$self->open_tempfile(PASS, ">$self->{'user_module_config_directory'}/pass.$_[1]->{'key'}");
$self->print_tempfile(PASS, $_[0],"\n");
$self->close_tempfile(PASS);
chmod(0700, "$self->{'user_module_config_directory'}/pass.$_[1]->{'key'}");
}

# encrypt_data(data, &result, &key|&keys, ascii)
# Encrypts some data with the given public key and returns the result, and
# returns an error message or undef on failure
sub encrypt_data
{
	my $self = shift;

local $srcfile = $self->transname();
local @keys = ref($_[2]) eq 'ARRAY' ? @{$_[2]} : ( $_[2] );
local $rcpt = join(" ", map { "--recipient \"$_->{'name'}->[0]\"" } @keys);
$self->write_entire_file($srcfile, $_[0]);
local $dstfile = $self->transname();
local $ascii = $_[3] ? "--armor" : "";
local $comp = $self->{'config_hash'}->{'compress'} eq '' ? "" :
		" --compress-algo $self->{'config_hash'}->{'compress'}";
local $cmd = "$self->{'gpgpath'} --output $dstfile $rcpt $ascii $comp --encrypt $srcfile";
local ($fh, $fpid) = $self->foreign_call("proc", "pty_process_exec", $cmd);
while(1) {
	$self->{'rv'} = $self->wait_for($fh, "anyway");
	if ($self->{'rv'} == 0) {
		syswrite($fh, "yes\n", length("yes\n"));
		}
	elsif ($self->{'rv'} < 0) {
		last;
		}
	}
close($fh);
unlink($srcfile);
local $dst = $self->read_entire_file($dstfile);
unlink($dstfile);
if ($dst) {
	${$_[1]} = $dst;
	return undef;
	}
else {
	return $self->{'wait_for_input'};
	}
}

# decrypt_data(data, &result)
# Decrypts some data encrypted for the current GnuPG user, and puts the results
# into &result. Returns an error message or undef on success.
sub decrypt_data
{
	my $self = shift;

local $srcfile = $self->transname();
$self->write_entire_file($srcfile, $_[0]);
local $dstfile = $self->transname();
local $cmd = "$self->{'gpgpath'} --output $dstfile --decrypt $srcfile";
local ($fh, $fpid) = $self->foreign_call("proc", "pty_process_exec", $cmd);
local ($error, $seen_pass, $pass, $key, $keyid);
while(1) {
	local $rv = $self->wait_for($fh, "passphrase:", "key,\\s+ID\\s+(\\S+),", "failed.*\\n", "error.*\\n", "invalid.*\\n", "signal caught.*\\n");
	if ($rv == 0) {
		last if ($seen_pass++);
		sleep(1);
		syswrite($fh, "$pass\n", length("$pass\n"));
		}
	elsif ($rv == 1) {
		$keyid = $self->{'matches_array'}->[1];
		($key) = grep { $self->indexof($self->{'matches_array'}->[1], @{$_->{'key2'}}) >= 0 }
			      $self->list_secret_keys();
		$pass = $self->get_passphrase($key) if ($key);
		}
	elsif ($rv > 1) {
		$error++;
		last;
		}
	elsif ($rv < 0) {
		last;
		}
	}
close($fh);
unlink($srcfile);
local $dst = $self->read_entire_file($dstfile);
unlink($dstfile);
if (!$keyid) {
	return $self->{'text_hash'}->{'gnupg_ecryptid'};
	}
elsif (!$key) {
	return $self->text('gnupg_ecryptkey', "<tt>$keyid</tt>");
	}
elsif (!defined($pass)) {
	return $self->text('gnupg_ecryptpass', $key->{'name'}->[0]).". ".
	    $self->text('gnupg_canset', "/gnupg/edit_key.cgi?key=$key->{'key'}").".";
	}
elsif ($error || $seen_pass > 1) {
	return "<pre>$self->{'wait_for_input'}</pre>";
	}
else {
	${$_[1]} = $dst;
	return undef;
	}
}

# sign_data(data, \&result, &key, mode)
# Signs the given data and returns the result. Mode 0 = binary signature
# mode 1 = ascii signature at end, mode 2 = ascii signature only
sub sign_data
{
	my $self = shift;

local $srcfile = $self->transname();
$self->write_entire_file($srcfile, $_[0]);
local $dstfile = $self->transname();
local $cmd;
if ($_[3] == 0) {
	$cmd = "$self->{'gpgpath'} --output $dstfile --default-key $_[2]->{'key'} --sign $srcfile";
	}
elsif ($_[3] == 1) {
	$cmd = "$self->{'gpgpath'} --output $dstfile --default-key $_[2]->{'key'} --clearsign $srcfile";
	}
elsif ($_[3] == 2) {
	$cmd = "$self->{'gpgpath'} --armor --output $dstfile --default-key $_[2]->{'key'} --detach-sig $srcfile";
	}
local ($fh, $fpid) = $self->foreign_call("proc", "pty_process_exec", $cmd);
local ($error, $seen_pass);
local $pass = $self->get_passphrase($_[2]);
if (!defined($pass)) {
	return $self->{'text_hash'}->{'gnupg_esignpass'}.". ".
	    $self->text('gnupg_canset', "/gnupg/edit_key.cgi?key=$_[2]->{'key'}").".";
	}
while(1) {
	$self->{'rv'} = $self->wait_for($fh, "passphrase:", "failed", "error");
	if ($self->{'rv'} == 0) {
		last if ($seen_pass++);
		sleep(1);
		syswrite($fh, "$pass\n", length("$pass\n"));
		}
	elsif ($self->{'rv'} > 0) {
		$error++;
		last;
		}
	elsif ($self->{'rv'} < 0) {
		last;
		}
	}
close($fh);
unlink($srcfile);
local $dst = $self->read_entire_file($dstfile);
unlink($dstfile);
if ($error || $seen_pass > 1) {
	return "<pre>$self->{'wait_for_input'}</pre>";
	}
else {
	${$_[1]} = $dst;
	return undef;
	}
}

# verify_data(data, [signature])
# Verifies the signature on some data, and returns a status code and a message
# code 0 = verified successfully, message contains signer
# code 1 = verified successfully but no trust chain, message contains signer
# code 2 = failed to verify, message contains signer
# code 3 = do not have signers public key, message contains ID
# code 4 = verification totally failed, message contains reason
sub verify_data
{
	my $self = shift;

local $datafile = $self->transname();
$self->write_entire_file($datafile, $_[0]);
local $cmd;
local $sigfile;
if (!$_[1]) {
	$cmd = "$self->{'gpgpath'} --verify $datafile";
	}
else {
	$sigfile = $self->transname();
	$self->write_entire_file($sigfile, $_[1]);
	$cmd = "$self->{'gpgpath'} --verify $sigfile $datafile";
	}
#local ($fh, $fpid) = &foreign_call("proc", "pty_process_exec", $cmd);
#&wait_for($fh);
#close($fh);
#local $out = $wait_for_input;
local $out = `$cmd 2>&1 </dev/null`;
unlink($datafile);
unlink($sigfile) if ($sigfile);
if ($out =~ /BAD signature from "(.*)"/i) {
	return (2, $1);
	}
elsif ($out =~ /key ID (\S+).*\n.*not found/i) {
	return (3, $1);
	}
elsif ($out =~ /Good signature from "(.*)"/i) {
	local $signer = $1;
	if ($out =~ /warning/) {
		return (1, $signer);
		}
	else {
		return (0, $signer);
		}
	}
else {
	return (4, $out);
	}
}

# read_entire_file(file)
sub read_entire_file
{
	my $self = shift;

local ($rv, $buf);
open(FILE, $_[0]) || return undef;
while(read(FILE, $buf, 1024) > 0) {
	$rv .= $buf;
	}
close(FILE);
return $rv;
}

# write_entire_file(file, data)
sub write_entire_file
{
	my $self = shift;

$self->open_tempfile(FILE, ">$_[0]");
$self->print_tempfile(FILE, $_[1]);
$self->close_tempfile(FILE);
}

# get_trust_level(&key)
# Returns the trust level of a key
sub get_trust_level
{
	my $self = shift;

local $cmd = "$self->{'gpgpath'} --edit-key \"$_[0]->{'name'}->[0]\"";
local ($fh, $fpid) = $self->foreign_call("proc", "pty_process_exec", $cmd);
local $rv = $self->wait_for($fh, "trust:\\s+(.)", "command>");
local $tr;
if ($rv == 0) {
	$tr = $matches[1] eq "q" ? 1 : $matches[1] eq "n" ? 2 :
	      $matches[1] eq "m" ? 3 : $matches[1] eq "f" ? 4 : 0;
	}
else {
	$tr = -1;
	}
syswrite($fh, "quit\n", length("quit\n"));
close($fh);
return $tr;
}

# delete_key(&key)
# Delete one public or secret key
sub delete_key
{
	my $self = shift;

local ($key) = @_;
if ($key->{'secret'}) {
	local $cmd = "$self->{'gpgpath'} --delete-secret-key \"$key->{'name'}->[0]\"";
	local ($fh, $fpid) = $self->foreign_call("proc", "pty_process_exec", $cmd);
	$self->wait_for($fh, "\\?");
	syswrite($fh, "y\n");
	$self->wait_for($fh, "\\?");
	syswrite($fh, "y\n");
	sleep(1);
	close($fh);
	}
local $cmd = "$self->{'gpgpath'} --delete-key \"$key->{'name'}->[0]\"";
local ($fh, $fpid) = $self->foreign_call("proc", "pty_process_exec", $cmd);
$self->wait_for($fh, "\\?");
syswrite($fh, "y\n");
sleep(1);
close($fh);
}


1;

