#!/usr/bin/perl

&MAIN();

 sub MAIN {
# setup.sh
# Call gnupg to setup a key initially

&ModuleLoader::preload_module('gnupg_class', "gnupg::GnupgLib");
$self = new gnupg::GnupgLib:: 'gnupg_class',%$self;

$self->ReadParse();
$self->{'pfx'} = $self->{'in_hash'}->{'newkey'} ? 'secret' : 'setup';
$self->error_setup($self->{'text_hash'}->{$self->{'pfx'}.'_err'});

# Validate inputs
$self->{'in_hash'}->{'name'} || $self->error($self->{'text_hash'}->{'setup_ename'});
length($self->{'in_hash'}->{'name'}) >= 5 || $self->error($self->text('setup_enamelen', 5));
#$self->{'in_hash'}->{'email'} || $self->error($self->{'text_hash'}->{'setup_eemail'});

# Run gpg --list-keys to create the directory
$self->list_keys();
$self->list_keys();

# Create .gnupg directory if missing
if (!-d "$self->{'remote_user_info_array'}->[7]/.gnupg") {
	mkdir("$self->{'remote_user_info_array'}->[7]/.gnupg", 0700) ||
		$self->error($self->text('setup_emkdir', $!));
	}

$self->{'pid'} = fork();
if (!$self->{'pid'}) {
	# Run the find command in a subprocess to generate some entropy
	untie(*STDOUT);
	untie(*STDERR);
	close(STDOUT);
	close(STDERR);
	exec("find / -type f");
	exit(0);
	}

foreach $k ($self->list_secret_keys()) {
	$self->{'oldid_hash'}->{$k->{'key'}}++;
	}

# Run the gpg --gen-key command
$SIG{ALRM} = "gpg_timeout";
alarm(90);
$self->foreign_require("proc", "proc-lib.pl");
($fh, $self->{'fpid'}) = $self->foreign_call("proc", "pty_process_exec", "$self->{'gpgpath'} --gen-key");
$self->wait_for($fh, "Your selection");
syswrite($fh, "\n");
$self->wait_for($fh, "you want");
if ($self->{'in_hash'}->{'keysize'}) {
	syswrite($fh, "$self->{'in_hash'}->{'keysize'}\n");
	}
else {
	syswrite($fh, "\n");
	}
$self->wait_for($fh, "valid for");
syswrite($fh, "\n");
$self->wait_for($fh, "correct");
syswrite($fh, "y\n");
$self->wait_for($fh, "Real name");
syswrite($fh, "$self->{'in_hash'}->{'name'}\n");
$self->wait_for($fh, "Email address");
syswrite($fh, "$self->{'in_hash'}->{'email'}\n");
$self->wait_for($fh, "Comment");
syswrite($fh, "$self->{'in_hash'}->{'comment'}\n");
$self->wait_for($fh, "Change");
syswrite($fh, "O\n");
$self->wait_for($fh, "passphrase");
sleep(1);
syswrite($fh, "$self->{'in_hash'}->{'pass'}\n");
$self->wait_for($fh, "passphrase");
sleep(1);
syswrite($fh, "$self->{'in_hash'}->{'pass'}\n");
$self->wait_for($fh);
close($fh);
$self->{'err'} = $?;
alarm(0);
kill('TERM', $self->{'pid'});

$self->ui_print_header(undef, $self->{'text_hash'}->{$self->{'pfx'}.'_title'}, "");

if ($self->{'err'} || $self->{'timed_out'}) {
	print "<p>",$self->text('setup_failed', "<pre>$self->{'wait_for_input'}</pre>"),"<p>\n";
	}
else {
	print "<p>$self->{'text_hash'}->{$self->{'pfx'}.'_ok'}<p>\n";
	@{$self->{'keys_array'}} = $self->list_secret_keys();
	($self->{'key'}) = grep { !$self->{'oldid_hash'}->{$_->{'key'}} } @{$self->{'keys_array'}};
	$self->put_passphrase($self->{'in_hash'}->{'pass'}, $self->{'key'});
	}

$self->ui_print_footer("", $self->{'text_hash'}->{'index_return'});

1;
		
}

sub gpg_timeout
{
	shift if ($self == $_[0]);

kill('KILL', $self->{'fpid'}) if ($self->{'fpid'});
$self->{'timed_out'}++;
}


1;

