#!/usr/bin/perl
package htaccess_htpasswd::HtpasswdfileLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'htaccess_htpasswd::HtpasswdfileLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# htpasswd-file-lib.pl
# Functions for reading and writing a .htpasswd format file
# XXX md5 and old password

if (!$self->{'module_name'}) {
	&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
	$self->init_config();
	&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
	%{$self->{'access_hash'}} = $self->get_module_acl();
	}
&ModuleLoader::preload_module(ref($self), "htaccess_htpasswd::Md5Lib");
$self = new htaccess_htpasswd::Md5Lib:: ref($self), %$self; 
$self->{'htdigest_command'} = $self->has_command("htdigest") || $self->has_command("htdigest2");

1;

1;
		return $self;
}

# list_users([file])
# Returns an array of user and password details from the given file
sub list_users
{
	my $self = shift;

local $file = $_[0] || $self->{'config_hash'}->{'file'};
if (!defined($self->{'list_authusers_cache_hash'}->{$file})) {
	$self->{'list_authusers_cache_hash'}->{$file} = [ ];
	local $_;
	local $lnum = 0;
	local $count = 0;
	open(HTPASSWD, $file);
	while(<HTPASSWD>) {
		if (/^(#?)\s*(\S+):(\S*)/) {
			push(@{$self->{'list_authusers_cache_hash'}->{$file}},
				  { 'user' => $2,
				    'pass' => $3,
				    'enabled' => !$1,
				    'file' => $file,
				    'line' => $lnum,
				    'index' => $count++ });
			}
		$lnum++;
		}
	close(HTPASSWD);
	}
return $self->{'list_authusers_cache_hash'}->{$file};
}

# list_digest_users([file])
# Returns an array of user, domain and password details from the given file
sub list_digest_users
{
	my $self = shift;

local $file = $_[0] || $self->{'config_hash'}->{'file'};
if (!defined($self->{'list_authusers_cache_hash'}->{$file})) {
	$self->{'list_authusers_cache_hash'}->{$file} = [ ];
	local $_;
	local $lnum = 0;
	local $count = 0;
	open(HTPASSWD, $file);
	while(<HTPASSWD>) {
		if (/^(#?)\s*(\S+):(\S+):(\S*)/) {
			push(@{$self->{'list_authusers_cache_hash'}->{$file}},
				  { 'user' => $2,
				    'dom' => $3,
				    'pass' => $4,
				    'enabled' => !$1,
				    'digest' => 1,
				    'file' => $file,
				    'line' => $lnum,
				    'index' => $count++ });
			}
		$lnum++;
		}
	close(HTPASSWD);
	}
return $self->{'list_authusers_cache_hash'}->{$file};
}

# modify_user(&user)
sub modify_user
{
	my $self = shift;

local $lref = $self->read_file_lines($_[0]->{'file'});
if ($_[0]->{'digest'}) {
	$lref->[$_[0]->{'line'}] = ($_[0]->{'enabled'} ? "" : "#").
			   "$_[0]->{'user'}:$_[0]->{'dom'}:$_[0]->{'pass'}";
	}
else {
	$lref->[$_[0]->{'line'}] = ($_[0]->{'enabled'} ? "" : "#").
				   "$_[0]->{'user'}:$_[0]->{'pass'}";
	}
$self->flush_file_lines($_[0]->{'file'});
}

# create_user(&user, [file])
sub create_user
{
	my $self = shift;

$_[0]->{'file'} = $_[1] || $self->{'config_hash'}->{'file'};
local $lref = $self->read_file_lines($_[0]->{'file'});
$_[0]->{'line'} = @$lref;
if ($_[0]->{'digest'}) {
	push(@$lref, ($_[0]->{'enabled'} ? "" : "#").
		     "$_[0]->{'user'}:$_[0]->{'dom'}:$_[0]->{'pass'}");
	}
else {
	push(@$lref, ($_[0]->{'enabled'} ? "" : "#").
		     "$_[0]->{'user'}:$_[0]->{'pass'}");
	}
$self->flush_file_lines($_[0]->{'file'});
$_[0]->{'index'} = @{$self->{'list_authusers_cache_hash'}->{$_[0]->{'file'}}};
push(@{$self->{'list_authusers_cache_hash'}->{$_[0]->{'file'}}}, $_[0]);
}

# delete_user(&user)
sub delete_user
{
	my $self = shift;

local $lref = $self->read_file_lines($_[0]->{'file'});
splice(@$lref, $_[0]->{'line'}, 1);
$self->flush_file_lines($_[0]->{'file'});
splice(@{$self->{'list_authusers_cache_hash'}->{$_[0]->{'file'}}}, $_[0]->{'index'}, 1);
map { $_->{'line'}-- if ($_->{'line'} > $_[0]->{'line'}) }
    @{$self->{'list_authusers_cache_hash'}->{$_[0]->{'file'}}};
}

# encrypt_password(string, [old], md5mode)
sub encrypt_password
{
	my $self = shift;

$self->seed_random();
if ($_[2] == 1) {
	# MD5
	return $self->encrypt_md5($_[0], $_[1]);
	}
elsif ($_[2] == 2) {
	# SHA1
	return $self->encrypt_sha1($_[0]);
	}
elsif ($_[2] == 3) {
	# Digest
	return $self->digest_password(undef, undef, $_[0]);
	}
else {
	# Crypt
	if ($self->{'gconfig_hash'}->{'os_type'} eq 'windows' && $self->has_command("htpasswd")) {
		# Call htpasswd program
		local $qp = quotemeta($_[0]);
		local $out = `htpasswd -n -b foo $qp 2>&1 <$self->{'null_file'}`;
		if ($out =~ /^foo:(\S+)/) {
			return $1;
			}
		else {
			$self->error("htpasswd failed : $out");
			}
		}
	else {
		# Use built-in encryption code
		local $salt = $_[1] ||
			      chr(int(rand(26))+65).chr(int(rand(26))+65);
		return $self->unix_crypt($_[0], $salt);
		}
	}
}

# digest_password(user, realm, pass)
# Encrypts a password in the format used by htdigest
sub digest_password
{
	my $self = shift;

local ($user, $dom, $pass) = @_;
local $temp = $self->tempname();
$self->foreign_require("proc", "proc-lib.pl");
local ($fh, $fpid) = $main::proc_object->pty_process_exec("$self->{'htdigest_command'} -c $temp ".quotemeta($dom)." ".quotemeta($user));
$self->wait_for($fh, "password:");
$self->sysprint($fh, "$pass\n");
$self->wait_for($fh, "password:");
$self->sysprint($fh, "$pass\n");
$self->wait_for($fh);
close($fh);
local $tempusers = $self->list_digest_users($temp);
#unlink($temp);
return $tempusers->[0]->{'pass'};
}

# list_groups(file)
# Returns an array of group details from the given file
sub list_groups
{
	my $self = shift;

local $file = $_[0];
if (!defined($self->{'list_authgroups_cache_hash'}->{$file})) {
	$self->{'list_authgroups_cache_hash'}->{$file} = [ ];
	local $_;
	local $lnum = 0;
	local $count = 0;
	open(HTPASSWD, $file);
	while(<HTPASSWD>) {
		if (/^(#?)\s*(\S+):\s*(.*)/) {
			push(@{$self->{'list_authgroups_cache_hash'}->{$file}},
				  { 'group' => $2,
				    'enabled' => !$1,
				    'members' => [ split(/\s+/, $3) ],
				    'file' => $file,
				    'line' => $lnum,
				    'index' => $count++ });
			}
		$lnum++;
		}
	close(HTPASSWD);
	}
return $self->{'list_authgroups_cache_hash'}->{$file};
}

# modify_group(&group)
sub modify_group
{
	my $self = shift;

local $lref = $self->read_file_lines($_[0]->{'file'});
$lref->[$_[0]->{'line'}] = ($_[0]->{'enabled'} ? "" : "#").
			   "$_[0]->{'group'}: ".
			   join(" ", @{$_[0]->{'members'}});
$self->flush_file_lines();
}

# create_group(&group, [file])
sub create_group
{
	my $self = shift;

$_[0]->{'file'} = $_[1] || $self->{'config_hash'}->{'file'};
local $lref = $self->read_file_lines($_[0]->{'file'});
$_[0]->{'line'} = @$lref;
push(@$lref, ($_[0]->{'enabled'} ? "" : "#").
	      "$_[0]->{'group'}: ".
	      join(" ", @{$_[0]->{'members'}}));
$self->flush_file_lines();
$_[0]->{'index'} = @{$self->{'list_authgroups_cache_hash'}->{$_[0]->{'file'}}};
push(@{$self->{'list_authgroups_cache_hash'}->{$_[0]->{'file'}}}, $_[0]);
}

# delete_group(&group)
sub delete_group
{
	my $self = shift;

local $lref = $self->read_file_lines($_[0]->{'file'});
splice(@$lref, $_[0]->{'line'}, 1);
$self->flush_file_lines();
splice(@{$self->{'list_authgroups_cache_hash'}->{$_[0]->{'file'}}}, $_[0]->{'index'}, 1);
map { $_->{'line'}-- if ($_->{'line'} > $_[0]->{'line'}) }
    @{$self->{'list_authgroups_cache_hash'}->{$_[0]->{'file'}}};
}


1;

