#!/usr/bin/perl
package mailcap::MailcapLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'mailcap::MailcapLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for editing /etc/mailcap

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

if ($self->{'module_info_hash'}->{'usermin'}) {
	$self->switch_to_remote_user();
	$self->{'mailcap_file'} = $self->{'userconfig_hash'}->{'mailcap'};
	if ($self->{'mailcap_file'} !~ /^\//) {
		$self->{'mailcap_file'} = "$self->{'remote_user_info_array'}->[7]/$self->{'mailcap_file'}";
		}
	}
else {
	$self->{'mailcap_file'} = $self->{'config_hash'}->{'mailcap'};
	}

1;

1;
		return $self;
}

# list_mailcap()
# Returns a list of /etc/mailcap entries
sub list_mailcap
{
	my $self = shift;

if (!defined(@{$self->{'list_mailcap_cache_array'}})) {
  @{$self->{'list_mailcap_cache_array'}} = ( );
  open(CAP, $self->{'mailcap_file'});
  local $lnum = 0;
  while(<CAP>) {
	  local ($slnum, $elnum) = ($lnum, $lnum);
	  s/\r|\n//g;
	  while (/^(.*)\\/) {
		# Continuation line! Read the next one and append it
		local $before = $1;
		local $nxt = <CAP>;
		$nxt =~ s/\r|\n//g;
		$_ = $before.$nxt;
		$elnum++;
		}
	  if (/^(#*)\s*([a-z0-9\-]+)\/([a-z0-9\-\*]+);\s*([^;]*)(;(.*))?/) {
		  # Found a line
		  local @cmtlines = split(/\n/, $self->{'cmt'});
		  local $cap = { 'type' => $2."/".$3,
				 'program' => $4,
				 'enabled' => !$1,
				 'cmt' => $self->{'cmt'},
				 'args' => { },
				 'line' => $slnum-scalar(@cmtlines),
				 'eline' => $elnum,
				 'index' => scalar(@{$self->{'list_mailcap_cache_array'}}),
			       };
		  # Parse ; separated args after the command
		  local $args = $6;
		  local @args = split(/\s*;\s*/, $args);
		  foreach my $arg (@args) {
			if ($arg =~ /^\s*(\S+)\s*=\s*(.*)/) {
				# A name-value arg
				$cap->{'args'}->{$1} = $2;
				}
			elsif ($arg =~ /^\s*(\S+)\s*$/) {
				$cap->{'args'}->{$1} = "";
				}
			}
		  push(@{$self->{'list_mailcap_cache_array'}}, $cap);
		  $self->{'cmt'} = undef;
		  }
	  elsif (/^#+(.*)/) {
		  # Found a comment before a rule
		  if ($self->{'cmt'}) {
			$self->{'cmt'} .= "\n".$1;
			}
		  else {
			$self->{'cmt'} = $1;
			}
		  }
	  else {
		  $self->{'cmt'} = undef;
		  }
	  $lnum++;
	  }
  close(CAP);
  }
return @{$self->{'list_mailcap_cache_array'}};
}

# create_mailcap(&mailcap)
# Adds a mailcap entry
sub create_mailcap
{
	my $self = shift;

local ($mailcap) = @_;
$self->list_mailcap();  # init cache
local $lref = $self->read_file_lines($self->{'mailcap_file'});
local @lines = $self->mailcap_lines($mailcap);
$mailcap->{'line'} = scalar(@$lref);
$mailcap->{'eline'} = scalar(@$lref)+scalar(@lines)-1;
$mailcap->{'index'} = scalar(@{$self->{'list_mailcap_cache_array'}});
push(@$lref, @lines);
$self->flush_file_lines($self->{'mailcap_file'});
push(@{$self->{'list_mailcap_cache_array'}}, $mailcap);
}

# modify_mailcap(&mailcap)
# Updates one mailcap entry in the file
sub modify_mailcap
{
	my $self = shift;

local ($mailcap) = @_;
local $lref = $self->read_file_lines($self->{'mailcap_file'});
local @lines = $self->mailcap_lines($mailcap);
local $oldlen = $mailcap->{'eline'} - $mailcap->{'line'} + 1;
splice(@$lref, $mailcap->{'line'}, $oldlen, @lines);
$self->flush_file_lines($self->{'mailcap_file'});
local $diff = scalar(@lines)-$oldlen;
foreach my $c (grep { $c ne $mailcap } @{$self->{'list_mailcap_cache_array'}}) {
  $c->{'line'} += $diff if ($c->{'line'} > $mailcap->{'line'});
  $c->{'eline'} += $diff if ($c->{'eline'} > $mailcap->{'line'});
  }
}

# delete_mailcap(&mailcap)
# Removes one mailcap entry from the file
sub delete_mailcap
{
	my $self = shift;

local ($mailcap) = @_;
local $lref = $self->read_file_lines($self->{'mailcap_file'});
local $len = $mailcap->{'eline'} - $mailcap->{'line'} + 1;
splice(@$lref, $mailcap->{'line'}, $len);
$self->flush_file_lines($self->{'mailcap_file'});
@list_mailcap_cache = grep { $_ ne $mailcap } @list_mailcap_cache;
foreach my $c (@list_mailcap_cache) {
  $c->{'line'} -= $len if ($c->{'line'} > $mailcap->{'line'});
  $c->{'eline'} -= $len if ($c->{'eline'} > $mailcap->{'line'});
  $c->{'index'}-- if ($c->{'index'} > $mailcap->{'index'});
  }
}

# mailcap_lines(&mailcap)
# Returns an array of lines for a mailcap entry
sub mailcap_lines
{
	my $self = shift;

local ($mailcap) = @_;
local @rv;
local $args;
foreach my $a (keys %{$mailcap->{'args'}}) {
	local $v = $mailcap->{'args'}->{$a};
	if ($v eq '') {
		$args .= "; $a";
		}
	else {
		$args .= "; $a=$v";
		}
	}
foreach my $l (split(/\n/, $mailcap->{'cmt'})) {
	push(@rv, "#$l");
	}
push(@rv, ($mailcap->{'enabled'} ? "" : "#").
          "$mailcap->{'type'}; $mailcap->{'program'}".$args);
return @rv;
}


1;

