%postfix_config = ();
&read_file("$config_directory/../postfix/config", \%postfix_config);

# Get the saved version number
if (open(VERSION, "$config_directory/../postfix/version")) {
	chop($postfix_version = <VERSION>);
	close(VERSION);
	}
if ($postfix_version >= 2) {
	$postfix_virtual_maps = "virtual_alias_maps";
	$postfix_ldap_timeout = "ldap_timeout";
	}
else {
	$postfix_virtual_maps = "virtual_maps";
	$postfix_ldap_timeout = "ldap_lookup_timeout";
	}

sub postfix_guess_config_dir
{
    my $answ = $postfix_config{'postfix_config_file'};
    $answ =~ /(.*)\/[^\/]*/;
    return $1;
}

$config_dir = &postfix_guess_config_dir();

$virtual_type = $postfix_virtual_maps || "virtual_maps";
$virtual_maps = &postfix_get_real_value($virtual_type);
@virtual_map_files = &postfix_get_maps_files($virtual_maps);
$postfix_afiles = [ &postfix_get_aliases_files(&postfix_get_real_value("alias_maps")) ];

# get_current_value(parameter_name)
# returns a scalar corresponding to the value of the parameter
sub postfix_get_current_value
{
	my $out = `$postfix_config{'postfix_config_command'} -c $config_dir -h $_[0] 2>&1`;
#	if ($?) { &error(&text('postfix_query_get_efailed', $_[0], $out) . $step++); }
	return '' if $?;
	chop($out);
	return $out;
}

# get_real_value(parameter_name)
# Returns the value of a parameter, with $ substitions done
sub postfix_get_real_value
{
	 #&error("VVV: $_[0]") if $_[1];
my $v = &postfix_get_current_value($_[0]);
$v =~ s/\$(\{([^\}]+)\}|([A-Za-z0-9\.\-\_]+))/postfix_get_real_value($2 || $3)/ge;
return $v;
}

############################################################################
# maps [canonical, virtual, transport] support

# get_maps_files($maps_param) : @maps_files
# parses its argument to extract the filenames of the mapping files
# supports multiple maps-files
sub postfix_get_maps_files
{
    $_[0] =~ /:(\/[^,\s]*)(.*)/;
    (my $returnvalue, my $recurse) = ( $1, $2 );

    return ( $returnvalue,
	     ($recurse =~ /:\/[^,\s]*/) ?
	         &postfix_get_maps_files($recurse)
	     :
	         ()
           )
}


# get_maps($maps_name) : \@maps
# construct the mappings database taken from the map files given from the parameter
sub postfix_get_maps
{
    if (!defined($maps_cache{$_[0]}))
    {
	my @maps_files = &postfix_get_maps_files(&postfix_get_real_value($_[0]));
	my $number = 0;
	foreach $maps_file (@maps_files)
	{
	    open(MAPS, $maps_file);
	    my $i = 0;
	    while (<MAPS>)
	    {
		s/^#.*$//g;	# remove comments
		s/\r|\n//g;	# remove newlines
		if (/^\s*([^\s]+)\s+([^#]*)/)
		{
		    $number++;
		    my %map;
		    $map{'name'} = $1;
		    $map{'value'} = $2;
		    $map{'line'} = $i;
		    $map{'map_file'} = $maps_file;
		    $map{'file'} = $maps_file;
		    $map{'number'} = $number;
		    push(@{$maps_cache{$_[0]}}, \%map);
		}
		$i++;
	    }
	    close(MAPS);
	}
    }
    return $maps_cache{$_[0]};
}

############################################################################
# aliases support    [too lazy to create a aliases-lib.pl :-)]

# get_aliases_files($alias_maps) : @aliases_files
# parses its argument to extract the filenames of the aliases files
# supports multiple alias-files
sub postfix_get_aliases_files
{
    $_[0] =~ /:(\/[^,\s]*)(.*)/;
    (my $returnvalue, my $recurse) = ( $1, $2 );

    # Yes, Perl is also a functional language -> I construct a list, and no problem, lists are flattened in Perl
    return ( $returnvalue,
	     ($recurse =~ /:\/[^,\s]*/) ?
	         &postfix_get_aliases_files($recurse)
	     :
	         ()
           )
}


# get_aliases() : \@aliases
# construct the aliases database taken from the aliases files given in the "alias_maps" parameter
sub postfix_get_aliases
{
    if (!@aliases_cache)
    {
	my @aliases_files = &postfix_get_aliases_files(&postfix_get_current_value("alias_maps"));
	my $number = 0;
	foreach $aliases_file (@aliases_files)
	{
	    open(ALIASES, $aliases_file);
	    my $i = 0;
	    while (<ALIASES>)
	    {
		s/^#.*$//g;	# remove comments
		s/\r|\n//g;	# remove newlines
		if ((/^\s*\"([^\"]*)[^:]*:\s*([^#]*)/) ||      # names with double quotes (") are special, as seen in `man aliases(5)`
		    (/^\s*([^\s:]*)[^:]*:\s*([^#]*)/))         # other names
		{
		    $number++;
		    my %alias;
		    $alias{'name'} = $1;
		    $alias{'value'} = $2;
		    $alias{'line'} = $i;
		    $alias{'alias_file'} = $aliases_file;
		    $alias{'number'} = $number;
		    push(@aliases_cache, \%alias);
		}
		$i++;
	    }
	    close(ALIASES);
	}
    }
    return \@aliases_cache;
}

# list_aliases(files)
# Returns an array of data structures, each containing information about
# one sendmail alias
sub postfix_list_aliases
{
local $jfiles = join(",", @{$_[0]});
local $c = $list_aliases_cache{$jfiles};
if (!defined($c)) {
	$c = $list_aliases_cache{$jfiles} = [ ];
	local $file;
	local @skip = split(/\s+/, $postfix_config{'alias_skip'});
	foreach $file (@{$_[0]}) {
		local $lalias;
		local $lnum = 0;
		open(AFILE, $file);
		while(<AFILE>) {
			s/\r|\n//g;	# remove newlines
			if (/^(#*)\s*([^:$ \t]+)\s*:\s*(.*)$/) {
				local(%alias, @values, $v);
				$alias{'eline'} = $alias{'line'} = $lnum;
				$alias{'file'} = $file;
				$alias{'enabled'} = $1 ? 0 : 1;
				$alias{'name'} = $2;
				$v = $alias{'value'} = $3;
				while($v =~ /^\s*,?\s*()"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*(\|)"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*()([^,\s]+)(.*)$/) {
					push(@values, $1.$2); $v = $3;
					}
				$alias{'values'} = \@values;
				$alias{'num'} = scalar(@$c);
				if (&indexof($alias{'name'}, @skip) < 0) {
					push(@$c, \%alias);
					$lalias = \%alias;
					}
				}
			elsif (/^(#*)\s+(\S.*)$/ && $lalias &&
			       ($1 && !$lalias->{'enabled'} ||
				!$1 && $lalias->{'enabled'})) {
				# continuation of last alias
				$lalias->{'eline'} = $lnum;
				local $v = $2;
				$lalias->{'value'} .= $v;
				while($v =~ /^\s*,?\s*()"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*(\|)"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*()([^,\s]+)(.*)$/) {
					push(@{$lalias->{'values'}}, $1.$2); $v = $3;
					}
				}
			else { $lalias = undef; }
			$lnum++;
			}
		close(AFILE);
		}
	}
return @$c;
}

1;
1;
