#!/usr/bin/perl
package quota::LinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'quota::LinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# linux-lib.pl
# Quota functions for all linux version

# Tell the mount module not to check which filesystems are supported,
# as we don't care for the calls made by this module
$main::mount_object->{'no_check_support'} = 1;

# Pass UIDs and GIDs to edquota instead of names
$self->{'edquota_use_ids'} = 1;

%{$self->{'name_to_unit_hash'}} = ( "second", 0, "seconds", 0,
		  "minute", 1, "minutes", 1,
		  "hour", 2, "hours", 2,
		  "day", 3, "days", 3,
		);
foreach $k (keys %{$self->{'name_to_unit_hash'}}) {
	$self->{'unit_to_name_hash'}->{$self->{'name_to_unit_hash'}->{$k}} = $k;
	}

1;

1;
		return $self;
}

# quotas_init()
sub quotas_init
{
	my $self = shift;

if ($self->has_command("quotaon") && $self->has_command("quotaoff")) {
	return undef;
	}
else {
	return "The quotas package does not appear to be installed on ".
	       "your system\n";
	}
}

# quotas_supported()
# Returns 1 for user quotas, 2 for group quotas or 3 for both
sub quotas_supported
{
	my $self = shift;

return 3;
}

# free_space(filesystem, [blocksize])
# Returns an array containing  btotal, bfree, ftotal, ffree
sub free_space
{
	my $self = shift;

local(@out, @rv);
$self->{'out'} = `df -k $_[0]`;
$self->{'out'} =~ /Mounted on\n\S+\s+(\d+)\s+\d+\s+(\d+)/;
if ($_[1]) {
	push(@rv, int($1*1024/$_[1]), int($2*1024/$_[1]));
	}
else {
	push(@rv, $1, $2);
	}
$self->{'out'} = `df -i $_[0]`;
$self->{'out'} =~ /Mounted on\n\S+\s+(\d+)\s+\d+\s+(\d+)/;
push(@rv, $1, $2);
return @rv;
}

# quota_can(&mnttab, &fstab)
# Can this filesystem type support quotas?
#  0 = No quota support (or not turned on in /etc/fstab)
#  1 = User quotas only
#  2 = Group quotas only
#  3 = User and group quotas
sub quota_can
{
	my $self = shift;

return ($_[1]->[3] =~ /usrquota/ ? 1 : 0) +
       ($_[1]->[3] =~ /grpquota/ ? 2 : 0);
}

# quota_now(&mnttab, &fstab)
# Are quotas currently active?
#  0 = Not active
#  1 = User quotas active
#  2 = Group quotas active
#  3 = Both active
# Adding 4 means they cannot be turned off (such as for XFS)
sub quota_now
{
	my $self = shift;

local $rv = 0;
local $dir = $_[0]->[0];
local %opts = map { $_, 1 } split(/,/, $_[0]->[3]);
if ($_[0]->[2] eq "xfs") {
	# For XFS, assume enabled if setup in fstab
	$rv += 1 if ($opts{'quota'} || $opts{'usrquota'} ||
		     $opts{'uqnoenforce'});
	$rv += 2 if ($opts{'grpquota'} || $opts{'gqnoenforce'});
	return $rv + 4;
	}
if ($_[0]->[4]%2 == 1) {
	# test user quotas
	if (-r "$dir/quota.user" || -r "$dir/aquota.user") {
		local $stout = $self->supports_status($dir, "user");
		if ($stout =~ /is\s+(on|off)/) {
			# Can use output from -p mode
			if ($stout =~ /is\s+on/) {
				$rv += 1;
				}
			}
		else {
			# Fall back to testing by running quotaon
			$self->{'out'} = `$self->{'config_hash'}->{'user_quotaon_command'} $dir 2>&1`;
			if ($self->{'out'} =~ /Device or resource busy/i) {
				# already on..
				$rv += 1;
				}
			elsif ($self->{'out'} =~ /Package not installed/i) {
				# No quota support!
				return 0;
				}
			else {
				# was off.. need to turn on again
				`$self->{'config_hash'}->{'user_quotaoff_command'} $dir 2>&1`;
				}
			}
		}
	}
if ($_[0]->[4] > 1) {
	# test group quotas
	if (-r "$dir/quota.group" || -r "$dir/aquota.group") {
		local $stout = $self->supports_status($dir, "group");
		if ($stout =~ /is\s+(on|off)/) {
			# Can use output from -p mode
			if ($stout =~ /is\s+on/) {
				$rv += 2;
				}
			}
		else {
			# Fall back to testing by running quotaon
			$self->{'out'} = `$self->{'config_hash'}->{'group_quotaon_command'} $dir 2>&1`;
			if ($self->{'out'} =~ /Device or resource busy/i) {
				# already on..
				$rv += 2;
				}
			elsif ($self->{'out'} =~ /Package not installed/i) {
				# No quota support!
				return 0;
				}
			else {
				# was off.. need to turn on again
				`$self->{'config_hash'}->{'group_quotaoff_command'} $dir 2>&1`;
				}
			}
		}
	}
return $rv;
}

# supports_status(dir, mode)
sub supports_status
{
	my $self = shift;

if (!defined($self->{'supports_status_cache_hash'}->{$_[0],$_[1]})) {
	local $stout = `$self->{'config_hash'}->{$_[1].'_quotaon_command'} -p $_[0] 2>&1`;
	$self->{'supports_status_cache_hash'}->{$_[0],$_[1]} =
		$stout =~ /is\s+(on|off)/ ? $stout : 0;
	}
return $self->{'supports_status_cache_hash'}->{$_[0],$_[1]};
}

# quotaon(filesystem, mode)
# Activate quotas and create quota files for some filesystem. The mode can
# be 1 for user only, 2 for group only or 3 for user and group
sub quotaon
{
	my $self = shift;

local($out, $qf, @qfile, $flags, $version);
return if ($self->is_readonly_mode());

# Check which version of quota is being used
$out = `quota -V 2>&1`;
if ($out =~ /\s(\d+\.\d+)/) {
	$version = $1;
	}

# Force load of quota kernel modules
$self->system_logged("modprobe quota_v2 >/dev/null 2>&1");

if ($_[1]%2 == 1) {
	# turn on user quotas
	local $qf = $version >= 2 ? "aquota.user" : "quota.user";
	if (!-s "$_[0]/$qf") {
		# Setting up for the first time
		local $ok = 0;
		if ($self->has_command("convertquota") && $version >= 2) {
			# Try creating a quota.user file and converting it
			$self->open_tempfile(QUOTAFILE, ">>$_[0]/quota.user", 0, 1);
			$self->close_tempfile(QUOTAFILE);
			$self->set_ownership_permissions(undef, undef, 0600,
						   "$_[0]/quota.user");
			$self->system_logged("convertquota -u / 2>&1");
			$ok = 1 if (!$?);
			$self->unlink_file("$_[0]/quota.user");
			}
		if (!$ok) {
			# Try to create an [a]quota.user file
			$self->open_tempfile(QUOTAFILE, ">>$_[0]/$qf", 0, 1);
			$self->close_tempfile(QUOTAFILE);
			$self->set_ownership_permissions(undef, undef, 0600,
						   "$_[0]/$qf");
			$self->run_quotacheck($_[0]) || $self->run_quotacheck($_[0], "-u -f") || $self->run_quotacheck($_[0], "-u -f -m") || $self->run_quotacheck($_[0], "-u -f -m -c");
			}
		}
	$out = $self->backquote_logged("$self->{'config_hash'}->{'user_quotaon_command'} $_[0] 2>&1");
	if ($?) { return $out; }
	}
if ($_[1] > 1) {
	# turn on group quotas
	local $qf = $version >= 2 ? "aquota.group" : "quota.group";
	if (!-s "$_[0]/$qf") {
		# Setting up for the first time
		local $ok = 0;
		if (!$ok && $self->has_command("convertquota") && $version >= 2) {
			# Try creating a quota.group file and converting it
			$self->open_tempfile(QUOTAFILE, ">>$_[0]/quota.group", 0, 1);
			$self->close_tempfile(QUOTAFILE);
			$self->set_ownership_permissions(undef, undef, 0600,
						   "$_[0]/quota.group");
			$self->system_logged("convertquota -g / 2>&1");
			$ok = 1 if (!$?);
			$self->unlink_file("$_[0]/quota.group");
			}
		if (!$ok) {
			# Try to create an [a]quota.group file
			$self->open_tempfile(QUOTAFILE, ">>$_[0]/$qf", 0, 1);
			$self->close_tempfile(QUOTAFILE);
			$self->set_ownership_permissions(undef, undef, 0600,
						   "$_[0]/$qf");
			$self->run_quotacheck($_[0]) || $self->run_quotacheck($_[0], "-u -f") || $self->run_quotacheck($_[0], "-u -f -m") || $self->run_quotacheck($_[0], "-u -f -m -c");
			}
		}
	$out = $self->backquote_logged("$self->{'config_hash'}->{'group_quotaon_command'} $_[0] 2>&1");
	if ($?) { return $out; }
	}
return undef;
}

# run_quotacheck(filesys, args)
sub run_quotacheck
{
	my $self = shift;

local $out =$self->backquote_logged("$self->{'config_hash'}->{'quotacheck_command'} $_[1] $_[0] 2>&1");
return $? || $out =~ /cannot remount|please stop/i ? 0 : 1;
}

# quotaoff(filesystem, mode)
# Turn off quotas for some filesystem
sub quotaoff
{
	my $self = shift;

return if ($self->is_readonly_mode());
local($out);
if ($_[1]%2 == 1) {
	$out = $self->backquote_logged("$self->{'config_hash'}->{'user_quotaoff_command'} $_[0] 2>&1");
	if ($?) { return $out; }
	}
if ($_[1] > 1) {
	$out = $self->backquote_logged("$self->{'config_hash'}->{'group_quotaoff_command'} $_[0] 2>&1");
	if ($?) { return $out; }
	}
return undef;
}

# user_filesystems(user)
# Fills the array %filesys with details of all filesystem some user has
# quotas on
sub user_filesystems
{
	my $self = shift;

return $self->parse_quota_output("$self->{'config_hash'}->{'user_quota_command'} ".quotemeta($_[0]));
}

# group_filesystems(user)
# Fills the array %filesys with details of all filesystem some group has
# quotas on
sub group_filesystems
{
	my $self = shift;

return $self->parse_quota_output("$self->{'config_hash'}->{'group_quota_command'} ".quotemeta($_[0]));
}

sub parse_quota_output
{
	my $self = shift;

local($n, $_, %mtab);
%mtab = $self->get_mtab_map();
open(QUOTA, "$_[0] |");
$n=0; while(<QUOTA>) {
	chop;
	if (/^(Disk|\s+Filesystem)/) { next; }
	if (/^(\S+)$/) {
		# Bogus wrapped line
		$self->{'filesys_hash'}->{$n,'filesys'} = $mtab{$1};
		local $nl = <QUOTA>;
		$nl =~/^\s+(\S+)\s+(\S+)\s+(\S+)(.{8}\s+)(\S+)\s+(\S+)\s+(\S+)(.*)/ ||
		      /^.{15}.(.{7}).(.{7}).(.{7})(.{8}.)(.{7}).(.{7}).(.{7})(.*)/;
		$self->{'filesys_hash'}->{$n,'ublocks'} = int($1);
		$self->{'filesys_hash'}->{$n,'sblocks'} = int($2);
		$self->{'filesys_hash'}->{$n,'hblocks'} = int($3);
		$self->{'filesys_hash'}->{$n,'gblocks'} = $4;
		$self->{'filesys_hash'}->{$n,'ufiles'} = int($5);
		$self->{'filesys_hash'}->{$n,'sfiles'} = int($6);
		$self->{'filesys_hash'}->{$n,'hfiles'} = int($7);
		$self->{'filesys_hash'}->{$n,'gfiles'} = $8;
		$self->{'filesys_hash'}->{$n,'gblocks'} = $self->trunc_space($self->{'filesys_hash'}->{$n,'gblocks'});
		$self->{'filesys_hash'}->{$n,'gfiles'} = $self->trunc_space($self->{'filesys_hash'}->{$n,'gfiles'});
		$n++;
		}
	elsif (/^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)(.{8}\s+)(\S+)\s+(\S+)\s+(\S+)(.*)/ ||
	       /^(.{15}).(.{7}).(.{7}).(.{7})(.{8}.)(.{7}).(.{7}).(.{7})(.*)/) {
		# Single quota line
		$self->{'filesys_hash'}->{$n,'ublocks'} = int($2);
		$self->{'filesys_hash'}->{$n,'sblocks'} = int($3);
		$self->{'filesys_hash'}->{$n,'hblocks'} = int($4);
		$self->{'filesys_hash'}->{$n,'gblocks'} = $5;
		$self->{'filesys_hash'}->{$n,'ufiles'} = int($6);
		$self->{'filesys_hash'}->{$n,'sfiles'} = int($7);
		$self->{'filesys_hash'}->{$n,'hfiles'} = int($8);
		$self->{'filesys_hash'}->{$n,'gfiles'} = $9;
		$self->{'dev'} = $1; $self->{'dev'} =~ s/\s+$//g; $self->{'dev'} =~ s/^\s+//g;
		$self->{'filesys_hash'}->{$n,'filesys'} = $mtab{$self->{'dev'}};
		$self->{'filesys_hash'}->{$n,'gblocks'} = $self->trunc_space($self->{'filesys_hash'}->{$n,'gblocks'});
		$self->{'filesys_hash'}->{$n,'gfiles'} = $self->trunc_space($self->{'filesys_hash'}->{$n,'gfiles'});
		$n++;
		}
	}
close(QUOTA);
return $n;
}

# filesystem_users(filesystem)
# Fills the array %{$self->{'user_hash'}} with information about all users with quotas
# on this filesystem. This may not be all users on the system..
sub filesystem_users
{
	my $self = shift;

%{$self->{'user_hash'}} = ();
return $self->parse_repquota_output(
	"$self->{'config_hash'}->{'user_repquota_command'} $_[0]", "user");
}

sub filesystem_groups
{
	my $self = shift;

%{$self->{'group_hash'}} = ();
return $self->parse_repquota_output(
	"$self->{'config_hash'}->{'group_repquota_command'} $_[0]", "group");
}

sub parse_repquota_output
{
	my $self = shift;

local($rep, @rep, $n, $what, $u, @uinfo);
$what = $_[1];
$rep = `$_[0] 2>&1`;
if ($?) { return -1; }
local $st = $self->supports_status($_[0], $what);
if (!$st) {
	# Older system, need to build username map to identify truncation
	if ($what eq 'user') {
		setpwent();
		while(@uinfo = getpwent()) {
			$self->{'hasu_hash'}->{$uinfo[0]}++;
			}
		endpwent();
		}
	else {
		setgrent();
		while(@uinfo = getgrent()) {
			$self->{'hasu_hash'}->{$uinfo[0]}++;
			}
		endgrent();
		}
	}
@rep = split(/\n/, $rep); @rep = @rep[3..$#rep];
local $nn = 0;
local %already;
for($n=0; $n<@rep; $n++) {
	if ($rep[$n] =~ /^\s*(\S.*\S|\S)\s+[\-\+]{2}\s+(\S+)\s+(\S+)\s+(\S+)(.{7})\s+(\S+)\s+(\S+)\s+(\S+)(.*)/ || $rep[$n] =~ /([^\-\s]\S*)\s*[\-\+]{2}(.{8})(.{8})(.{8})(.{7})(.{8})(.{6})(.{6})(.*)/) {
		$self->{$what."_hash"}->{$nn,$what} = $1;
		$self->{$what."_hash"}->{$nn,'ublocks'} = int($2);
		$self->{$what."_hash"}->{$nn,'sblocks'} = int($3);
		$self->{$what."_hash"}->{$nn,'hblocks'} = int($4);
		$self->{$what."_hash"}->{$nn,'gblocks'} = $5;
		$self->{$what."_hash"}->{$nn,'ufiles'} = int($6);
		$self->{$what."_hash"}->{$nn,'sfiles'} = int($7);
		$self->{$what."_hash"}->{$nn,'hfiles'} = int($8);
		$self->{$what."_hash"}->{$nn,'gfiles'} = $9;
		if (!$st && $self->{$what."_hash"}->{$nn,$what} !~ /^\d+$/ &&
			    !$self->{'hasu_hash'}->{$self->{$what."_hash"}->{$nn,$what}}) {
			# User/group name was truncated! Try to find him..
			foreach $u (keys %{$self->{'hasu_hash'}}) {
				if (substr($u, 0, length($self->{$what."_hash"}->{$nn,$what})) eq
				    $self->{$what."_hash"}->{$nn,$what}) {
					# found him..
					$self->{$what."_hash"}->{$nn,$what} = $u;
					last;
					}
				}
			}
		next if ($already{$self->{$what."_hash"}->{$nn,$what}}++); # skip dupe users
		$self->{$what."_hash"}->{$nn,'gblocks'} = $self->trunc_space($self->{$what."_hash"}->{$nn,'gblocks'});
		$self->{$what."_hash"}->{$nn,'gfiles'} = $self->trunc_space($self->{$what."_hash"}->{$nn,'gfiles'});
		$nn++;
		}
	}
return $nn;
}

# edit_quota_file(data, filesys, sblocks, hblocks, sfiles, hfiles)
sub edit_quota_file
{
	my $self = shift;

local($rv, $line, %mtab, @m, @line);
%mtab = $self->get_mtab_map();
@line = split(/\n/, $_[0]);
for(my $i=0; $i<@line; $i++) {
	if ($line[$i] =~ /^(\S+): blocks in use: (\d+), limits \(soft = (\d+), hard = (\d+)\)$/ && $mtab{$self->resolve_links("$1")} eq $_[1]) {
		# Found old-style lines to change
		$rv .= "$1: blocks in use: $2, limits (soft = $_[2], hard = $_[3])\n";
		$line[++$i] =~ /^\s*inodes in use: (\d+), limits \(soft = (\d+), hard = (\d+)\)$/;
		$rv .= "\tinodes in use: $1, limits (soft = $_[4], hard = $_[5])\n";
		}
	elsif ($line[$i] =~ /^device\s+(\S+)\s+\((\S+)\):/i && $2 eq $_[1]) {
		# Even newer-style line to change
		$rv .= "$line[$i]\n";
		$line[++$i] =~ /^used\s+(\S+),\s+limits:\s+soft=(\d+)\s+hard=(\d+)/i;
		$rv .= "Used $1, limits: soft=$_[2] hard=$_[3]\n";
		$line[++$i] =~ /^used\s+(\S+) inodes,\s+limits:\s+soft=(\d+)\s+hard=(\d+)/i;
		$rv .= "Used $1 inodes, limits: soft=$_[4] hard=$_[5]\n";
		}
	elsif ($line[$i] =~ /^\s+(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)$/ && $mtab{$self->resolve_links("$1")} eq $_[1]) {
		# New-style line to change
		$rv .= "  $1 $2 $_[2] $_[3] $5 $_[4] $_[5]\n";
		}
	else {
		# Leave this line alone
		$rv .= "$line[$i]\n";
		}
	}
return $rv;
}

# quotacheck(filesystem, mode(1=users, 2=group))
# Runs quotacheck on some filesystem
sub quotacheck
{
	my $self = shift;

local $out;
local $cmd = $self->{'config_hash'}->{'quotacheck_command'};
$cmd =~ s/\s+-[ug]//g;
local $flag = $_[1] == 1 ? "-u" : $_[1] == 2 ? "-g" : "-u -g";
$out = $self->backquote_logged("$cmd $flag $_[0] 2>&1");
if ($?) {
	# Try with the -f and -m options
	$out = $self->backquote_logged("$cmd $flag -f -m $_[0] 2>&1");
	if ($?) {
		# Try with the -F option
		$out = $self->backquote_logged("$self->{'config_hash'}->{'quotacheck_command'} $flag -F $_[0] 2>&1");
		}
	return $out if ($?);
	}
return undef;
}

# copy_user_quota(user, [user]+)
# Copy the quotas for some user to many others
sub copy_user_quota
{
	my $self = shift;

for($i=1; $i<@_; $i++) {
	$self->{'out'} = $self->backquote_logged("$self->{'config_hash'}->{'user_copy_command'} ".
				quotemeta($_[0])." ".quotemeta($_[$i])." 2>&1");
	if ($?) { return $self->{'out'}; }
	}
return undef;
}

# copy_group_quota(group, [group]+)
# Copy the quotas for some group to many others
sub copy_group_quota
{
	my $self = shift;

for($i=1; $i<@_; $i++) {
	$self->{'out'} = $self->backquote_logged("$self->{'config_hash'}->{'group_copy_command'} ".
				quotemeta($_[0])." ".quotemeta($_[$i])." 2>&1");
	if ($?) { return $self->{'out'}; }
	}
return undef;
}

# get_user_grace(filesystem)
# Returns an array containing  btime, bunits, ftime, funits
# The units can be 0=sec, 1=min, 2=hour, 3=day
sub get_user_grace
{
	my $self = shift;

return $self->parse_grace_output($self->{'config_hash'}->{'user_grace_command'}, $_[0]);
}

# get_group_grace(filesystem)
# Returns an array containing  btime, bunits, ftime, funits
# The units can be 0=sec, 1=min, 2=hour, 3=day
sub get_group_grace
{
	my $self = shift;

return $self->parse_grace_output($self->{'config_hash'}->{'group_grace_command'}, $_[0]);
}

# default_grace()
# Returns 0 if grace time can be 0, 1 if zero grace means default
sub default_grace
{
	my $self = shift;

return 0;
}

sub parse_grace_output
{
	my $self = shift;

local(@rv, %mtab, @m);
%mtab = $self->get_mtab_map();
$ENV{'EDITOR'} = $ENV{'VISUAL'} = "cat";
open(GRACE, "$_[0] 2>&1 |");
while(<GRACE>) {
	if (/^(\S+): block grace period: (\d+) (\S+), file grace period: (\d+) (\S+)/ && $mtab{$self->resolve_links("$1")} eq $_[1]) {
		@rv = ($2, $self->{'name_to_unit_hash'}->{$3}, $4, $self->{'name_to_unit_hash'}->{$5});
		}
	elsif (/^\s+(\S+)\s+(\d+)(\S+)\s+(\d+)(\S+)/ && $mtab{$self->resolve_links("$1")} eq $_[1]) {
		@rv = ($2, $self->{'name_to_unit_hash'}->{$3}, $4, $self->{'name_to_unit_hash'}->{$5});
		}
	elsif (/^device\s+(\S+)\s+\((\S+)\):/i && $2 eq $_[1]) {
		if (<GRACE> =~ /^block\s+grace:\s+(\S+)\s+(\S+)\s+inode\s+grace:\s+(\S+)\s+(\S+)/i) {
			@rv = ($1, $self->{'name_to_unit_hash'}->{$2}, $3, $self->{'name_to_unit_hash'}->{$4});
			last;
			}
		}
	}
close(GRACE);
return @rv;
}

# edit_grace_file(data, filesystem, btime, bunits, ftime, funits)
sub edit_grace_file
{
	my $self = shift;

local($rv, $line, @m, %mtab, @line);
%mtab = $self->get_mtab_map();
@line = split(/\n/, $_[0]);
for(my $i=0; $i<@line; $i++) {
	$line = $line[$i];
	print STDERR "line = $line\n";
	if ($line =~ /^(\S+): block grace period: (\d+) (\S+), file grace period: (\d+) (\S+)/ && $mtab{$self->resolve_links("$1")} eq $_[1]) {
		# replace this line
		$line = "$1: block grace period: $_[2] $self->{'unit_to_name_hash'}->{$_[3]}, file grace period: $_[4] $self->{'unit_to_name_hash'}->{$_[5]}";
		}
	elsif ($line =~ /^\s+(\S+)\s+(\d+)(\S+)\s+(\d+)(\S+)/ && $mtab{$self->resolve_links("$1")} eq $_[1]) {
		# replace new-style line
		$line = "  $1 $_[2]$self->{'unit_to_name_hash'}->{$_[3]} $_[4]$self->{'unit_to_name_hash'}->{$_[5]}";
		}
	elsif ($line =~ /^device\s+(\S+)\s+\((\S+)\):/i && $2 eq $_[1]) {
		# replace even newer-style line
		$rv .= "$line\n";
		$line = "Block grace: $_[2] $self->{'unit_to_name_hash'}->{$_[3]} Inode grace: $_[4] $self->{'unit_to_name_hash'}->{$_[5]}";
		$i++;
		}
	print STDERR "new = $line\n";
	$rv .= "$line\n";
	}
return $rv;
}

# grace_units()
# Returns an array of possible units for grace periods
sub grace_units
{
	my $self = shift;

return ($self->{'text_hash'}->{'grace_seconds'}, $self->{'text_hash'}->{'grace_minutes'}, $self->{'text_hash'}->{'grace_hours'},
	$self->{'text_hash'}->{'grace_days'});
}

# fs_block_size(dir, device, filesystem)
# Returns the size of blocks on some filesystem, or undef if unknown.
# Consult the dumpe2fs command where possible.
sub fs_block_size
{
	my $self = shift;

if ($_[2] eq "ext2" || $_[2] eq "ext3" || $_[2] eq "vfs") {
	return 1024;
	# This code isn't needed, because the quota block size is
	# not the same as the filesystem block size!!
	#if (&has_command("dumpe2fs")) {
	#	local $out = `dumpe2fs -h $_[1] 2>&1`;
	#	if (!$? && $out =~ /block size:\s+(\d+)/i) {
	#		return $1;
	#		}
	#	}
	}
elsif ($_[0] eq "xfs") {
	return 1024;
	}
return undef;
}

# Returns a hash mapping mount points to devices
sub get_mtab_map
{
	my $self = shift;

local $mm = $self->{'module_info_hash'}->{'usermin'} ? "usermount" : "mount";
$self->foreign_require($mm, "$mm-lib.pl");
local ($m, %mtab);
foreach $m ($self->foreign_call($mm, "list_mounted", 1)) {
	if ($m->[3] =~ /loop=([^,]+)/) {
		$mtab{$self->resolve_links("$1")} = $m->[0];
		}
	else {
		$mtab{$self->resolve_links($m->[1])} = $m->[0];
		}
	}
return %mtab;
}


1;

