#!/usr/bin/perl
package quota::MacosLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'quota::MacosLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# macos-lib.pl
# Quota functions for OSX
# XXX checking if on/off

%{$self->{'name_to_unit_hash'}} = ( "second", 0, "seconds", 0,
		  "minute", 1, "minutes", 1,
		  "hour", 2, "hours", 2,
		  "day", 3, "days", 3,
		);
foreach $k (keys %{$self->{'name_to_unit_hash'}}) {
	$self->{'unit_to_name_hash'}->{$self->{'name_to_unit_hash'}->{$k}} = $k;
	}

1;

1;
		return $self;
}

# quotas_init()
sub quotas_init
{
	my $self = shift;

if ($self->has_command("quotaon") && $self->has_command("quotaoff")) {
	return undef;
	}
else {
	return "The quotas programs do not appear to be installed on ".
	       "your system\n";
	}
}

# quotas_supported()
# Returns 1 for user quotas, 2 for group quotas or 3 for both
sub quotas_supported
{
	my $self = shift;

return 3;
}

# free_space(filesystem)
# Returns an array containing  btotal, bfree, ftotal, ffree
sub free_space
{
	my $self = shift;

local(@out, @rv);
$ENV{'BLOCKSIZE'} = 1024;
`df -i $_[0]` =~ /Mounted on\n\S+\s+(\d+)\s+\d+\s+(\d+)\s+\S+\s+(\d+)\s+(\d+)/;
return ($1, $2, $3+$4, $4);
}

# quota_can(&mnttab, &fstab)
# Can this filesystem type support quotas?
#  0 = No quota support (or not turned on in /etc/fstab)
#  1 = User quotas only
#  2 = Group quotas only
#  3 = User and group quotas
sub quota_can
{
	my $self = shift;

return 0 if ($_[0]->[2] ne 'ufs' && $_[0]->[2] ne 'hfs');
return $self->quota_now($_[0], $_[1]) || 3;	# use the current mode if active
}

# quota_now(&mnttab, &fstab)
# Are quotas currently active?
#  0 = Not active
#  1 = User quotas active
#  2 = Group quotas active
#  3 = Both active
sub quota_now
{
	my $self = shift;

local $rv;
$rv += 1 if (-r "$_[0]->[0]/.quota.ops.user" &&
	     $self->big_enough("$_[0]->[0]/.quota.user"));
$rv += 2 if (-r "$_[0]->[0]/.quota.ops.group" &&
	     $self->big_enough("$_[0]->[0]/.quota.group"));
return $rv;
}

# quotaon(filesystem, mode)
# Activate quotas and create quota files for some filesystem. The mode can
# be 1 for user only, 2 for group only or 3 for user and group
sub quotaon
{
	my $self = shift;

return if ($self->is_readonly_mode());
local($out, $qf, @qfile, $flags);
if ($_[1]%2 == 1) {
	# turn on user quotas
	$qf = "$_[0]/.quota.ops.user";
	$self->open_tempfile(QUOTAFILE, ">$qf", 0, 1);
	$self->close_tempfile(QUOTAFILE);
	$qf = "$_[0]/.quota.user";
	if (!$self->big_enough($qf)) {
		$self->unlink_file($qf);
		$self->system_logged("$self->{'config_hash'}->{'quotacheck_command'} $_[0]");
		}
	$out = $self->backquote_logged("$self->{'config_hash'}->{'user_quotaon_command'} $_[0] 2>&1");
	if ($?) { return $out; }
	}
if ($_[1] > 1) {
	# turn on group quotas
	$qf = "$_[0]/.quota.ops.group";
	$self->open_tempfile(QUOTAFILE, ">$qf", 0, 1);
	$self->close_tempfile(QUOTAFILE);
	$qf = "$_[0]/quota.group";
	if (!$self->big_enough($qf)) {
		$self->unlink_file($qf);
		$self->system_logged("$self->{'config_hash'}->{'quotacheck_command'} $_[0]");
		}
	$out = $self->backquote_logged("$self->{'config_hash'}->{'group_quotaon_command'} $_[0] 2>&1");
	if ($?) { return $out; }
	}
return undef;
}

sub big_enough
{
	my $self = shift;

local @st = stat($_[0]);
return $st[7] >= 1024;
}

# quotaoff(filesystem, mode)
# Turn off quotas for some filesystem
sub quotaoff
{
	my $self = shift;

return if ($self->is_readonly_mode());
local($out);
if ($_[1]%2 == 1) {
	$out = $self->backquote_logged("$self->{'config_hash'}->{'user_quotaoff_command'} $_[0] 2>&1");
	if ($?) { return $out; }
	$self->unlink_file("$_[0]/.quota.ops.user");
	}
if ($_[1] > 1) {
	$out = $self->backquote_logged("$self->{'config_hash'}->{'group_quotaoff_command'} $_[0] 2>&1");
	if ($?) { return $out; }
	$self->unlink_file("$_[0]/.quota.ops.group");
	}
return undef;
}

# user_filesystems(user)
# Fills the array %{$self->{'filesys_hash'}} with details of all filesystem some user has
# quotas on
sub user_filesystems
{
	my $self = shift;

local($n, $_, %mtab);
open(QUOTA, "$self->{'config_hash'}->{'user_quota_command'} ".quotemeta($_[0])." |");
$n=0; while(<QUOTA>) {
	chop;
	if (/^(Disk|\s+Filesystem)/) { next; }
	if (/^(\S+)\s+(\S+)\s+(\S+)\s+(\S+).(.{9})\s+(\S+)\s+(\S+)\s+(\S+)/) {
		$self->{'filesys_hash'}->{$n,'filesys'} = $1;
		$self->{'filesys_hash'}->{$n,'ublocks'} = int($2);
		$self->{'filesys_hash'}->{$n,'sblocks'} = int($3);
		$self->{'filesys_hash'}->{$n,'hblocks'} = int($4);
		$self->{'filesys_hash'}->{$n,'ufiles'} = int($6);
		$self->{'filesys_hash'}->{$n,'sfiles'} = int($7);
		$self->{'filesys_hash'}->{$n,'hfiles'} = int($8);
		$n++;
		}
	}
close(QUOTA);
return $n;
}

# group_filesystems(group)
# Fills the array %{$self->{'filesys_hash'}} with details of all filesystem some group has
# quotas on
sub group_filesystems
{
	my $self = shift;

local($n, $_, %mtab);
open(QUOTA, "$self->{'config_hash'}->{'group_quota_command'} ".quotemeta($_[0])." |");
$n=0; while(<QUOTA>) {
	chop;
	if (/^(Disk|\s+Filesystem)/) { next; }
	if (/^(\S+)\s+(\S+)\s+(\S+)\s+(\S+).(.{9})\s+(\S+)\s+(\S+)\s+(\S+)/) {
		$self->{'filesys_hash'}->{$n,'filesys'} = $1;
		$self->{'filesys_hash'}->{$n,'ublocks'} = int($2);
		$self->{'filesys_hash'}->{$n,'sblocks'} = int($3);
		$self->{'filesys_hash'}->{$n,'hblocks'} = int($4);
		$self->{'filesys_hash'}->{$n,'ufiles'} = int($6);
		$self->{'filesys_hash'}->{$n,'sfiles'} = int($7);
		$self->{'filesys_hash'}->{$n,'hfiles'} = int($8);
		$n++;
		}
	}
close(QUOTA);
return $n;
}

# filesystem_users(filesystem)
# Fills the array %{$self->{'user_hash'}} with information about all users with quotas
# on this filesystem. This may not be all users on the system..
sub filesystem_users
{
	my $self = shift;

local($rep, @rep, $n, $what);
$rep = `$self->{'config_hash'}->{'user_repquota_command'} $_[0] 2>&1`;
if ($?) { return -1; }
@rep = split(/\n/, $rep);
@rep = grep { !/^root\s/ } @rep[3..$#rep];
for($n=0; $n<@rep; $n++) {
	if ($rep[$n] =~ /(\S+)\s*[\-\+]{2}\s+(\d+)\s+(\d+)\s+(\d+)\s.{0,15}\s(\d+)\s+(\d+)\s+(\d+)/ || $rep[$n] =~ /(\S+)\s+..(.{8})(.{8})(.{8}).{7}(.{8})(.{8})(.{8})/) {
		$self->{'user_hash'}->{$n,'user'} = $1;
		$self->{'user_hash'}->{$n,'ublocks'} = int($2);
		$self->{'user_hash'}->{$n,'sblocks'} = int($3);
		$self->{'user_hash'}->{$n,'hblocks'} = int($4);
		$self->{'user_hash'}->{$n,'ufiles'} = int($5);
		$self->{'user_hash'}->{$n,'sfiles'} = int($6);
		$self->{'user_hash'}->{$n,'hfiles'} = int($7);
		}
	}
return $n;
}

# filesystem_groups(filesystem)
# Fills the array %{$self->{'group_hash'}} with information about all groups with quotas
# on this filesystem. This may not be all groups on the system..
sub filesystem_groups
{
	my $self = shift;

local($rep, @rep, $n, $what);
$rep = `$self->{'config_hash'}->{'group_repquota_command'} $_[0] 2>&1`;
if ($?) { return -1; }
@rep = split(/\n/, $rep);
@rep = @rep[3..$#rep];
for($n=0; $n<@rep; $n++) {
	if ($rep[$n] =~ /(\S+)\s*[\-\+]{2}\s+(\d+)\s+(\d+)\s+(\d+)\s.{0,15}\s(\d+)\s+(\d+)\s+(\d+)/ || $rep[$n] =~ /(\S+)\s+..(.{8})(.{8})(.{8}).{7}(.{8})(.{8})(.{8})/) {
		$self->{'group_hash'}->{$n,'group'} = $1;
		$self->{'group_hash'}->{$n,'ublocks'} = int($2);
		$self->{'group_hash'}->{$n,'sblocks'} = int($3);
		$self->{'group_hash'}->{$n,'hblocks'} = int($4);
		$self->{'group_hash'}->{$n,'ufiles'} = int($5);
		$self->{'group_hash'}->{$n,'sfiles'} = int($6);
		$self->{'group_hash'}->{$n,'hfiles'} = int($7);
		}
	}
return $n;
}

# edit_quota_file(data, filesys, sblocks, hblocks, sfiles, hfiles)
sub edit_quota_file
{
	my $self = shift;

local($rv, $line, %mtab, @m);
@{$self->{'line_array'}} = split(/\n/, $_[0]);
for($i=0; $i<@{$self->{'line_array'}}; $i++) {
	if ($self->{'line_array'}->[$i] =~ /^(\S+): (.*) in use: (\d+), limits \(soft = (\d+), hard = (\d+)\)$/ && $1 eq $_[1]) {
		# found lines to change
		$rv .= "$1: $2 in use: $3, limits (soft = $_[2], hard = $_[3])\n";
		$self->{'line_array'}->[++$i] =~ /^\s*inodes in use: (\d+), limits \(soft = (\d+), hard = (\d+)\)$/;
		$rv .= "\tinodes in use: $1, limits (soft = $_[4], hard = $_[5])\n";
		}
	else { $rv .= "$self->{'line_array'}->[$i]\n"; }
	}
return $rv;
}

# quotacheck(filesystem, mode)
# Runs quotacheck on some filesystem
sub quotacheck
{
	my $self = shift;

$self->{'out'} = $self->backquote_logged("$self->{'config_hash'}->{'quotacheck_command'} $_[0] 2>&1");
if ($?) { return $self->{'out'}; }
return undef;
}

# copy_user_quota(user, [user]+)
# Copy the quotas for some user to many others
sub copy_user_quota
{
	my $self = shift;

for($i=1; $i<@_; $i++) {
	$self->{'out'} = $self->backquote_logged("$self->{'config_hash'}->{'user_copy_command'} ".
				quotemeta($_[0])." ".quotemeta($_[$i])." 2>&1");
	if ($?) { return $self->{'out'}; }
	}
return undef;
}

# copy_group_quota(group, [group]+)
# Copy the quotas for some group to many others
sub copy_group_quota
{
	my $self = shift;

for($i=1; $i<@_; $i++) {
	$self->{'out'} = $self->backquote_logged("$self->{'config_hash'}->{'group_copy_command'} ".
				quotemeta($_[0])." ".quotemeta($_[$i])." 2>&1");
	if ($?) { return $self->{'out'}; }
	}
return undef;
}

# default_grace()
# Returns 0 if grace time can be 0, 1 if zero grace means default
sub default_grace
{
	my $self = shift;

return 0;
}

# get_user_grace(filesystem)
# Returns an array containing  btime, bunits, ftime, funits
# The units can be 0=sec, 1=min, 2=hour, 3=day
sub get_user_grace
{
	my $self = shift;

local(@rv, %mtab, @m);
$ENV{'EDITOR'} = $ENV{'VISUAL'} = "cat";
open(GRACE, "$self->{'config_hash'}->{'user_grace_command'} $_[0] |");
while(<GRACE>) {
	if (/^(\S+): block grace period: (\d+) (\S+), file grace period: (\d+) (\S+)/ && $1 eq $_[0]) {
		@rv = ($2, $self->{'name_to_unit_hash'}->{$3}, $4, $self->{'name_to_unit_hash'}->{$5});
		}
	}
close(GRACE);
return @rv;
}

# get_group_grace(filesystem)
# Returns an array containing  btime, bunits, ftime, funits
# The units can be 0=sec, 1=min, 2=hour, 3=day
sub get_group_grace
{
	my $self = shift;

local(@rv, %mtab, @m);
$ENV{'EDITOR'} = $ENV{'VISUAL'} = "cat";
open(GRACE, "$self->{'config_hash'}->{'group_grace_command'} $_[0] |");
while(<GRACE>) {
	if (/^(\S+): block grace period: (\d+) (\S+), file grace period: (\d+) (\S+)/ && $1 eq $_[0]) {
		@rv = ($2, $self->{'name_to_unit_hash'}->{$3}, $4, $self->{'name_to_unit_hash'}->{$5});
		}
	}
close(GRACE);
return @rv;
}

# edit_grace_file(data, filesystem, btime, bunits, ftime, funits)
sub edit_grace_file
{
	my $self = shift;

local($rv, $line, @m, %mtab);
foreach $line (split(/\n/, $_[0])) {
	if ($line =~ /^(\S+): block grace period: (\d+) (\S+), file grace period: (\d+) (\S+)/ && $1 eq $_[1]) {
		# replace this line
		$line = "$1: block grace period: $_[2] $self->{'unit_to_name_hash'}->{$_[3]}, file grace period: $_[4] $self->{'unit_to_name_hash'}->{$_[5]}";
		}
	$rv .= "$line\n";
	}
return $rv;
}

# grace_units()
# Returns an array of possible units for grace periods
sub grace_units
{
	my $self = shift;

return ($self->{'text_hash'}->{'grace_seconds'}, $self->{'text_hash'}->{'grace_minutes'}, $self->{'text_hash'}->{'grace_hours'},
	$self->{'text_hash'}->{'grace_days'});
}


1;

