#!/usr/bin/perl
package quota::QuotaLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'quota::QuotaLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# quota-lib.pl
# Common functions for quota management.

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
if ($self->{'gconfig_hash'}->{'os_type'} =~ /^\S+\-linux$/) {
	&ModuleLoader::preload_module(ref($self), "quota::LinuxLib");
$self = new quota::LinuxLib:: ref($self), %$self; 
	}
else {
	{
    my $pkg = $self->lib_to_pkg("$self->{gconfig_hash}->{os_type}-lib.pl", "quota");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
	}
if ($self->{'module_info_hash'}->{'usermin'}) {
	$self->foreign_require("usermount", "usermount-lib.pl");
	$self->switch_to_remote_user();
	}
else {
	%{$self->{'access_hash'}} = $self->get_module_acl();
	$self->foreign_require("mount", "mount-lib.pl");
	}

$self->{'email_cmd'} = "$self->{'module_config_directory'}/email.pl";

1;

1;
		return $self;
}

# list_filesystems()
# Returns a list of details of local filesystems on which quotas are supported
#  directory device type options quotacan quotanow
sub list_filesystems
{
	my $self = shift;

local $f;
local @mtab = $main::mount_object->list_mounted();
foreach $f ($main::mount_object->list_mounts()) {
	$self->{'fmap_hash'}->{$f->[0],$f->[1]} = $f;
	}
map { $_->[4] = $self->quota_can($_, $self->{'fmap_hash'}->{$_->[0],$_->[1]}) } @mtab;
map { $_->[5] = $self->quota_now($_, $self->{'fmap_hash'}->{$_->[0],$_->[1]}) } @mtab;
return grep { $_->[4] } @mtab;
}

# parse_options(type, options)
# Convert an options string for some filesystem into the associative
# array %{$self->{'options_hash'}}
sub parse_options
{
	my $self = shift;

local($_);
undef(%{$self->{'options_hash'}});
if ($_[0] ne "-") {
	foreach (split(/,/, $_[0])) {
		if (/^([^=]+)=(.*)$/) { $self->{'options_hash'}->{$1} = $2; }
		else { $self->{'options_hash'}->{$_} = ""; }
		}
	}
}

# user_quota(user, filesystem)
# Returns an array of  ublocks, sblocks, hblocks, ufiles, sfiles, hfiles
# for some user, or an empty array if no quota has been assigned
sub user_quota
{
	my $self = shift;

local (%user, $n, $i);
$n = $self->filesystem_users($_[1]);
for($i=0; $i<$n; $i++) {
	if ($user{$i,'user'} eq $_[0]) {
		return ( $user{$i,'ublocks'}, $user{$i,'sblocks'},
			 $user{$i,'hblocks'}, $user{$i,'ufiles'},
			 $user{$i,'sfiles'},  $user{$i,'hfiles'} );
		}
	}
return ();
}

# group_quota(group, filesystem)
# Returns an array of  ublocks, sblocks, hblocks, ufiles, sfiles, hfiles
# for some group, or an empty array if no quota has been assigned
sub group_quota
{
	my $self = shift;

local (%group, $n, $i);
$n = $self->filesystem_groups($_[1]);
for($i=0; $i<$n; $i++) {
	if ($group{$i,'group'} eq $_[0]) {
		return ( $group{$i,'ublocks'}, $group{$i,'sblocks'},
			 $group{$i,'hblocks'}, $group{$i,'ufiles'},
			 $group{$i,'sfiles'},  $group{$i,'hfiles'} );
		}
	}
return ();
}

# edit_user_quota(user, filesys, sblocks, hblocks, sfiles, hfiles)
# Sets the disk quota for some user
sub edit_user_quota
{
	my $self = shift;

if ($self->{'config_hash'}->{'user_setquota_command'} &&
    $self->has_command((split(/\s+/, $self->{'config_hash'}->{'user_setquota_command'}))[0])) {
	# Use quota setting command
	local $cmd = $self->{'config_hash'}->{'user_setquota_command'}." ".quotemeta($_[0])." ".
		     $_[2]." ".$_[3]." ".$_[4]." ".$_[5]." ".quotemeta($_[1]);
	local $out = $self->backquote_logged("$cmd 2>&1 </dev/null");
	$self->error("<tt>".$self->html_escape($out)."</tt>") if ($?);
	}
else {
	# Call the quota editor
	$ENV{'EDITOR'} = $ENV{'VISUAL'} = "$self->{'module_root_directory'}/edquota.pl";
	$ENV{'QUOTA_USER'} = $_[0];
	$ENV{'QUOTA_FILESYS'} = $_[1];
	$ENV{'QUOTA_SBLOCKS'} = $_[2];
	$ENV{'QUOTA_HBLOCKS'} = $_[3];
	$ENV{'QUOTA_SFILES'} = $_[4];
	$ENV{'QUOTA_HFILES'} = $_[5];
	local $user = $_[0];
	if ($self->{'edquota_use_ids'}) {
		# Use UID instead of username
		if ($user =~ /^#(\d+)$/) {
			$user = $1;
			}
		else {
			local $uid = getpwnam($user);
			$user = $uid if (defined($uid));
			}
		}
	$self->system_logged("$self->{'config_hash'}->{'user_edquota_command'} ".
		       quotemeta($user)." >/dev/null 2>&1");
	}
}

# edit_group_quota(group, filesys, sblocks, hblocks, sfiles, hfiles)
# Sets the disk quota for some group
sub edit_group_quota
{
	my $self = shift;

if ($self->{'config_hash'}->{'group_setquota_command'} &&
    $self->has_command((split(/\s+/, $self->{'config_hash'}->{'group_setquota_command'}))[0])) {
	# Use quota setting command
	local $cmd = $self->{'config_hash'}->{'group_setquota_command'}." ".quotemeta($_[0])." ".
		     $_[2]." ".$_[3]." ".$_[4]." ".$_[5]." ".quotemeta($_[1]);
	local $out = $self->backquote_logged("$cmd 2>&1 </dev/null");
	$self->error("<tt>".$self->html_escape($out)."</tt>") if ($?);
	}
else {
	# Call the editor
	$ENV{'EDITOR'} = $ENV{'VISUAL'} = "$self->{'module_root_directory'}/edquota.pl";
	$ENV{'QUOTA_USER'} = $_[0];
	$ENV{'QUOTA_FILESYS'} = $_[1];
	$ENV{'QUOTA_SBLOCKS'} = $_[2];
	$ENV{'QUOTA_HBLOCKS'} = $_[3];
	$ENV{'QUOTA_SFILES'} = $_[4];
	$ENV{'QUOTA_HFILES'} = $_[5];
	local $group = $_[0];
	if ($self->{'edquota_use_ids'}) {
		# Use GID instead of group name
		if ($group =~ /^#(\d+)$/) {
			$group = $1;
			}
		else {
			local $gid = getgrnam($group);
			$group = $gid if (defined($gid));
			}
		}
	$self->system_logged("$self->{'config_hash'}->{'group_edquota_command'} ".
		       quotemeta($group)." >/dev/null 2>&1");
	}
}

# edit_user_grace(filesystem, btime, bunits, ftime, funits)
# Change the grace times for blocks and files on some filesystem
sub edit_user_grace
{
	my $self = shift;

$ENV{'EDITOR'} = $ENV{'VISUAL'} = "$self->{'module_root_directory'}/edgrace.pl";
$ENV{'QUOTA_FILESYS'} = $_[0];
$ENV{'QUOTA_BTIME'} = $_[1];
$ENV{'QUOTA_BUNITS'} = $_[2];
$ENV{'QUOTA_FTIME'} = $_[3];
$ENV{'QUOTA_FUNITS'} = $_[4];
$self->system_logged($self->{'config_hash'}->{'user_grace_command'});
}

# edit_group_grace(filesystem, btime, bunits, ftime, funits)
# Change the grace times for blocks and files on some filesystem
sub edit_group_grace
{
	my $self = shift;

$ENV{'EDITOR'} = $ENV{'VISUAL'} = "$self->{'module_root_directory'}/edgrace.pl";
$ENV{'QUOTA_FILESYS'} = $_[0];
$ENV{'QUOTA_BTIME'} = $_[1];
$ENV{'QUOTA_BUNITS'} = $_[2];
$ENV{'QUOTA_FTIME'} = $_[3];
$ENV{'QUOTA_FUNITS'} = $_[4];
$self->system_logged($self->{'config_hash'}->{'group_grace_command'});
}

# quota_input(name, value, [blocksize])
# Prints an input for selecting a quota or unlimited, in a table
sub quota_input
{
	my $self = shift;

print "<td nowrap>",$self->ui_radio($_[0]."_def", $_[1] == 0 ? 1 : 0,
			      [ [ 1, $self->{'text_hash'}->{'quota_unlimited'} ], [ 0, " " ] ]);
print $self->quota_inputbox(@_);
print "</td> </tr>\n";
}

# quota_inputbox(name, value, [blocksize])
# Returns an input for selecting a quota
sub quota_inputbox
{
	my $self = shift;

if ($_[2]) {
	# We know the real size, so can offer units
	local $sz = $_[1]*$_[2];
	local $units = 1;
	if ($sz >= 10*1024*1024*1024) {
		$units = 1024*1024*1024;
		}
	elsif ($sz >= 10*1024*1024) {
		$units = 1024*1024;
		}
	elsif ($sz >= 10*1024) {
		$units = 1024;
		}
	else {
		$units = 1;
		}
	$sz = $sz == 0 ? "" : sprintf("%.2f", ($sz*1.0)/$units);
	return $self->ui_textbox($_[0], $sz, 8).
	       $self->ui_select($_[0]."_units", $units,
			 [ [ 1, "bytes" ], [ 1024, "kB" ], [ 1024*1024, "MB" ],
			   [ 1024*1024*1024, "GB" ] ]);
	}
else {
	# Just show blocks
	return $self->ui_textbox($_[0], $_[1], 8);
	}
}

# quota_parse(name, [bsize], [nodef])
sub quota_parse
{
	my $self = shift;

if ($self->{'in_hash'}->{$_[0]."_def"} && !$_[2]) {
	return 0;
	}
elsif ($_[1]) {
	# Include units, and covert to blocks
	return int($self->{'in_hash'}->{$_[0]}*$self->{'in_hash'}->{$_[0]."_units"}/$_[1]);
	}
else {
	# Just use blocks
	return int($self->{'in_hash'}->{$_[0]});
	}
}

# can_edit_filesys(filesys)
sub can_edit_filesys
{
	my $self = shift;

local $fs;
foreach $fs (split(/\s+/, $self->{'access_hash'}->{'filesys'})) {
	return 1 if ($fs eq "*" || $fs eq $_[0]);
	}
return 0;
}

# can_edit_user(user)
sub can_edit_user
{
	my $self = shift;

if ($self->{'access_hash'}->{'umode'} == 0) {
	return 1;
	}
elsif ($self->{'access_hash'}->{'umode'} == 3) {
	local @u = getpwnam($_[0]);
	return $self->{'access_hash'}->{'users'} == $u[3];
	}
elsif ($self->{'access_hash'}->{'umode'} == 4) {
	local @u = getpwnam($_[0]);
	return (!$self->{'access_hash'}->{'umin'} || $u[2] >= $self->{'access_hash'}->{'umin'}) &&
	       (!$self->{'access_hash'}->{'umax'} || $u[2] <= $self->{'access_hash'}->{'umax'});
	}
else {
	local ($u, %ucan);
	map { $ucan{$_}++ } split(/\s+/, $self->{'access_hash'}->{'users'});
	return $self->{'access_hash'}->{'umode'} == 1 && $ucan{$_[0]} ||
	       $self->{'access_hash'}->{'umode'} == 2 && !$ucan{$_[0]};
	}
}

# can_edit_group(group)
sub can_edit_group
{
	my $self = shift;

return 1 if ($self->{'access_hash'}->{'gmode'} == 0);
return 0 if ($self->{'access_hash'}->{'gmode'} == 3);
local ($g, %gcan);
map { $gcan{$_}++ } split(/\s+/, $self->{'access_hash'}->{'groups'});
return $self->{'access_hash'}->{'gmode'} == 1 && $gcan{$_[0]} ||
       $self->{'access_hash'}->{'gmode'} == 2 && !$gcan{$_[0]};
}

# filesystem_info(filesystem, &hash, count, [blocksize])
sub filesystem_info
{
	my $self = shift;

local @fs = $self->free_space($_[0], $_[3]);
if ($_[3]) {
	local $i;
	foreach $i (0 .. 3) {
		$fs[$i] = $i < 2 ? $self->nice_size($fs[$i]*$_[3])
				 : int($fs[$i]);
		}
	}
if ($_[1]) {
	local $bt = 0;
	local $ft = 0;
	local $i;
	for($i=0; $i<$_[2]; $i++) {
		$bt += $_[1]->{$i,'hblocks'};
		$ft += $_[1]->{$i,'hfiles'};
		}
	if ($_[3]) {
		$bt = $self->nice_size($bt*$_[3]);
		}
	return ( "$fs[0] total / $fs[1] free / $bt granted",
		 "$fs[2] total / $fs[3] free / $ft granted" );
	}
else {
	return ( "$fs[0] total / $fs[1] free",
		 "$fs[2] total / $fs[3] free" );
	}
}

# block_size(dir, [for-filesys])
# Returns the size (in bytes) of blocks on some filesystem
sub block_size
{
	my $self = shift;

return undef if (!$self->{'config_hash'}->{'block_mode'});
return undef if (!$self->UNIVERSAL::can('quota_block_size') &&
		 !$self->UNIVERSAL::can('fs_block_size'));
# local @mounts = $main::mount_object->list_mounted();
local @mounts = $self->{'module_info_hash'}->{'usermin'} ? $main::usermount_object->list_mounted() : $main::mount_object->list_mounted();
local ($mount) = grep { $_->[0] eq $_[0] } @mounts;
if ($mount) {
	if ($_[1]) {
		return $self->fs_block_size(@$mount);
		}
	else {
		if ($self->UNIVERSAL::can('quota_block_size')) {
			return $self->quota_block_size(@$mount);
			}
		else {
			return $self->fs_block_size(@$mount);
			}
		}
	}
return undef;
}

# print_limit(amount, no-blocks)
sub print_limit
{
	my $self = shift;

if ($_[0] == 0) { print "<td>$self->{'text_hash'}->{'quota_unlimited'}</td>\n"; }
elsif ($self->{'bsize'} && !$_[1]) { print "<td>",$self->nice_size($_[0]*$self->{'bsize'}),"</td>"; }
else { print "<td>$_[0]</td>\n"; }
}

# nice_limit(amount, bsize, no-blocks)
sub nice_limit
{
	my $self = shift;

local ($amount, $bsize, $noblocks) = @_;
return $amount == 0 ? $self->{'text_hash'}->{'quota_unlimited'} :
       $bsize && !$noblocks ? $self->nice_size($amount*$bsize) : $amount;
}

sub print_grace
{
	my $self = shift;

print "<td>",($_[0] || "<br>"),"</td>\n";
}

sub find_email_job
{
	my $self = shift;

$self->foreign_require("cron", "cron-lib.pl");
local @jobs = $main::cron_object->list_cron_jobs();
local ($job) = grep { $_->{'command'} eq $self->{'email_cmd'} } @jobs;
return $job;
}

# create_email_job()
# Creates the cron job for scheduled emailing
sub create_email_job
{
	my $self = shift;

$self->foreign_require("cron", "cron-lib.pl");
local $job = $self->find_email_job();
if (!$job) {
	$job = { 'user' => 'root',
		 'command' => $self->{'email_cmd'},
		 'active' => 1,
		 'mins' => '0,10,20,30,40,50',
		 'hours' => '*',
		 'days' => '*',
		 'months' => '*',
		 'weekdays' => '*' };
	$self->lock_file($main::cron_object->cron_file($job));
	$main::cron_object->create_cron_job($job);
	$main::cron_object->create_wrapper($self->{'email_cmd'}, $self->{'module_name'}, "email.pl");
	$self->unlock_file($main::cron_object->cron_file($job));
	}
}

# trunc_space(string)
# Removes spaces from the start and end of a string
sub trunc_space
{
	my $self = shift;

local $rv = $_[0];
$rv =~ s/^\s+//;
$rv =~ s/\s+$//;
return $rv;
}

# to_percent(used, total)
sub to_percent
{
	my $self = shift;

if ($_[1]) {
	return $_[0]*100/$_[1];
	}
else {
	return 0;
	}
}

# quota_show(number, filesystem)
# Returns text for the quota on some filesystem, in a human-readable format
sub quota_show
{
	my $self = shift;

local $bsize = $self->block_size($_[1]);
if ($bsize) {
        return sprintf('%0.1f MB',$_[0]*$bsize/1024/1024); #int($_[0]*$bsize/1024) . " kB";
        }
return $_[0] . " blocks";
}


1;

