#!/usr/bin/perl
package schedule::ScheduleLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'schedule::ScheduleLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for scheduled email notification

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();

$self->{'cron_cmd'} = "$self->{'module_config_directory'}/email.pl";
$self->{'atmode_cmd'} = "$self->{'module_config_directory'}/atmode.pl";
if (!-x $self->{'cron_cmd'} && $< == 0) {
	$self->create_wrapper($self->{'cron_cmd'}, $self->{'module_name'}, "email.pl");
	}
if (!-x $self->{'atmode_cmd'} && $< == 0) {
	$self->create_wrapper($self->{'atmode_cmd'}, $self->{'module_name'}, "atmode.pl");
	}

$self->switch_to_remote_user();
$self->create_user_config_dirs();
$self->foreign_require("cron", "cron-lib.pl");
$self->foreign_require("mailbox", "mailbox-lib.pl");
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
use Time::Local;

$self->{'schedules_dir'} = "$self->{'user_module_config_directory'}/schedules";
$self->{'messages_dir'} = "$self->{'user_module_config_directory'}/messages";
$self->{'files_dir'} = "$self->{'user_module_config_directory'}/files";
$self->{'old_cron_cmd'} = "$self->{'user_module_config_directory'}/email.pl";
$self->{'old_atmode_cmd'} = "$self->{'user_module_config_directory'}/atmode.pl";

1;

1;
		return $self;
}

# list_schedules()
# Returns a list of all scheduled messages
sub list_schedules
{
	my $self = shift;

local @rv;
opendir(DIR, $self->{'schedules_dir'});
foreach $f (readdir(DIR)) {
	next if ($f eq "." || $f eq "..");
	local $sched = $self->get_schedule($f);
	push(@rv, $sched) if ($sched);
	}
closedir(DIR);
return @rv;
}

# get_schedule(id)
sub get_schedule
{
	my $self = shift;

local $file = "$self->{'schedules_dir'}/$_[0]";
local %sched;
$self->read_file($file, \%sched);
$sched{'id'} = $_[0];
$sched{'file'} = $file;
$sched{'mfile'} = "$self->{'messages_dir'}/$_[0]";
open(MAIL, $sched{'mfile'});
while(<MAIL>) {
	$sched{'mail'} .= $_;
	}
close(MAIL);
return \%sched;
}

# save_schedule(&sched)
sub save_schedule
{
	my $self = shift;

mkdir($self->{'schedules_dir'}, 0700) if (!-d $self->{'schedules_dir'});
mkdir($self->{'messages_dir'}, 0700) if (!-d $self->{'messages_dir'});
$_[0]->{'id'} ||= time().$$;
local $file = $_[0]->{'file'} || "$self->{'schedules_dir'}/$_[0]->{'id'}";
local %copy = %{$_[0]};
delete($copy{'mail'});
$self->write_file($file, \%copy);
local $mfile = $_[0]->{'mfile'} || "$self->{'messages_dir'}/$_[0]->{'id'}";
$self->open_tempfile(MAIL, ">$mfile");
$self->print_tempfile(MAIL, $_[0]->{'mail'});
$self->close_tempfile(MAIL);
}

# delete_schedule(&sched)
sub delete_schedule
{
	my $self = shift;

unlink($_[0]->{'file'});
unlink($_[0]->{'mfile'});
system("rm -rf ".quotemeta("$self->{'files_dir'}/$_[0]->{'id'}"));
}

# list_schedule_files(&sched)
# Returns a list of files attached to a scheduled email
sub list_schedule_files
{
	my $self = shift;

local ($sched) = @_;
return ( ) if (!$sched->{'id'});
local @rv;
opendir(DIR, "$self->{'files_dir'}/$sched->{'id'}");
foreach my $f (readdir(DIR)) {
	next if ($f eq "." || $f eq "..");
	local $path = "$self->{'files_dir'}/$sched->{'id'}/$f";
	if (-l $path) {
		# A server-side file
		push(@rv, { 'file' => readlink($path),
			    'id' => $f,
			    'type' => 0 });
		}
	else {
		# An uploaded file
		push(@rv, { 'file' => $path,
			    'id' => $f,
			    'type' => 1 });
		}
	}
closedir(DIR);
return sort { lc($a->{'id'}) cmp lc($b->{'id'}) } @rv;
}

# create_schedule_file(&sched, path|data, uploaded-filename)
# Adds a file to a scheduled email
sub create_schedule_file
{
	my $self = shift;

local ($sched, $path, $uploaded) = @_;
mkdir($self->{'files_dir'}, 0700) if (!-d $self->{'files_dir'});
mkdir("$self->{'files_dir'}/$sched->{'id'}", 0700) if (!-d "$self->{'files_dir'}/$sched->{'id'}");
if ($uploaded) {
	$uploaded =~ s/^(.*)[\/\\]//;
	$self->open_tempfile(FILE, ">$self->{'files_dir'}/$sched->{'id'}/$uploaded");
	$self->print_tempfile(FILE, $path);
	$self->close_tempfile(FILE);
	}
else {
	local $short = $path;
	$short =~ s/^(.*)[\/\\]//;
	symlink($path, "$self->{'files_dir'}/$sched->{'id'}/$short");
	}
}

# delete_schedule_file(&sched, &file)
sub delete_schedule_file
{
	my $self = shift;

local ($sched, $file) = @_;
unlink("$self->{'files_dir'}/$sched->{'id'}/$file->{'id'}");
}

# find_cron_job(&sched)
# Finds the cron job for some scheduled email
sub find_cron_job
{
	my $self = shift;

local @jobs = $main::cron_object->list_cron_jobs();
local ($job) = grep { $_->{'user'} eq $self->{'remote_user'} &&
		      ($_->{'command'} eq "$self->{'cron_cmd'} $_[0]->{'id'}" ||
		       $_->{'command'} eq "$self->{'old_cron_cmd'} $_[0]->{'id'}") } @jobs;
return $job;
}

# my_email_address([with-name])
sub my_email_address
{
	my $self = shift;

local ($froms, $doms) = $main::mailbox_object->list_from_addresses();
if (@$froms) {
	local ($fp) = $main::mailbox_object->split_addresses($froms->[0]);
	if ($fp) {
		return $_[0] ? $fp->[2] : $fp->[0];
		}
	}
return $self->{'remote_user'}.'@'.$self->get_system_hostname();
}

# create_atmode_job()
# If any jobs exist that are scheduled at a specific time, create a cron job
# to check for them once per minute
sub create_atmode_job
{
	my $self = shift;

local @ats = grep { $_->{'at'} } $self->list_schedules();
if (@ats) {
	local @jobs = $main::cron_object->list_cron_jobs();
	local ($badjob) = grep { $_->{'user'} eq $self->{'remote_user'} &&
				 $_->{'command'} eq $self->{'old_atmode_cmd'} } @jobs;
	if ($badjob) {
		$main::cron_object->delete_cron_job($badjob);
		}
	local ($job) = grep { $_->{'user'} eq $self->{'remote_user'} &&
			      $_->{'command'} eq $self->{'atmode_cmd'} } @jobs;
	if (!$job) {
		local @mins = map { $_*5 } (0 .. 11);
		$job = { 'command' => $self->{'atmode_cmd'},
			 'user' => $self->{'remote_user'},
			 'active' => 1,
			 'mins' => join(",", @mins),
			 'hours' => '*',
			 'days' => '*',
			 'months' => '*',
			 'weekdays' => '*' };
		$main::cron_object->create_cron_job($job);
		}
	}
}

# make_email(&schedule)
sub make_email
{
	my $self = shift;

local ($sched) = @_;
local $myaddr = $self->my_email_address(1);
local $data = $sched->{'mail'};
if ($self->{'config_hash'}->{'attach'} && $sched->{'mailfile'}) {
	open(FILE, $sched->{'mailfile'});
	while(<FILE>) {
		$data .= $_;
		}
	close(FILE);
	}
local @attach;
foreach my $file ($self->list_schedule_files($sched)) {
	local $type = $self->guess_mime_type($file->{'id'},
				       "application/octet-stream").
		      "; name=\"$file->{'id'}\"";
	local $disp = "inline; filename=\"$file->{'id'}\"";
	push(@attach,
		{ 'headers' => [ [ 'Content-type', $type ],
				 [ 'Content-Disposition', $disp ],
				 [ 'Content-Transfer-Encoding', 'base64' ] ],
		  'data' => $self->read_file_contents($file->{'file'}) });
	}
return { 'headers' => [ [ 'From' => $sched->{'from'} || $myaddr ],
			[ 'To' => $main::mailbox_object->expand_to($sched->{'to'}) ||
				  $myaddr ],
			[ 'Cc' => $main::mailbox_object->expand_to($sched->{'cc'}) ],
			[ 'Bcc' => $main::mailbox_object->expand_to($sched->{'bcc'}) ],
			[ 'Subject' => $sched->{'subject'} ] ],
	  'attach' => [ { 'headers' => [ [ 'Content-type', 'text/plain' ] ],
			  'data' => $data },
			@attach ]
	};
}

# create_wrapper(wrapper-path, module, script)
# Creates a wrapper script which calls a script in some module's directory
# with the proper webmin environment variables set. Copied from the cron module,
# so that it can be called before requireing cron-lib.pl
sub create_wrapper
{
	my $self = shift;

local $perl_path = $self->get_perl_path();
$self->open_tempfile(CMD, ">$_[0]");
$self->print_tempfile(CMD, <<EOF
#!$perl_path
open(CONF, "$self->{'config_directory'}/miniserv.conf");
while(<CONF>) {
\$root = \$1 if (/^root=(.*)/);
}
close(CONF);
\$ENV{'WEBMIN_CONFIG'} = "$ENV{'WEBMIN_CONFIG'}";
\$ENV{'WEBMIN_VAR'} = "$ENV{'WEBMIN_VAR'}";
chdir("\$root/$_[1]");
exec("\$root/$_[1]/$_[2]", \@ARGV) || die "Failed to run \$root/$_[1]/$_[2] : \$!";
EOF
	);
$self->close_tempfile(CMD);
chmod(0755, $_[0]);
}

# next_run(&sched)
# Returns the unix time at which the given scheduled email will next be sent
sub next_run
{
	my $self = shift;

local ($sched) = @_;
if ($sched->{'at'}) {
	return $sched->{'at'};
	}
else {
	local $now = time();
	local @tm = localtime($now);

	local @mins = $self->cron_all_ranges($sched->{'mins'}, 0, 59);
	local @hours = $self->cron_all_ranges($sched->{'hours'}, 0, 23);
	local @days = $self->cron_all_ranges($sched->{'days'}, 1, 31);
	local @months = $self->cron_all_ranges($sched->{'months'}, 1, 12);
	local @weekdays = $self->cron_all_ranges($sched->{'weekdays'}, 0, 6);
	local ($min, $hour, $day, $month, $year);
	local @possible;
	foreach $min (@mins) {
		foreach $hour (@hours) {
			foreach $day (@days) {
				foreach $month (@months) {
					foreach $year ($tm[5] .. $tm[5]+7) {
						local $tt;
						eval { $tt = timelocal(0, $min, $hour, $day, $month-1, $year) };
						next if ($tt < $now);
						local @ttm = localtime($tt);
						next if ($self->indexof($ttm[6], @weekdays) < 0);
						push(@possible, $tt);
						last;
						}
					}
				}
			}
		}
	@possible = sort { $a <=> $b } @possible;
	return $possible[0];
	}
}

# cron_first(list, min, max, current)
# Returns the lowest value from expanding a cron range >= current
sub cron_first
{
	my $self = shift;

local @r = $self->cron_range($_[0], $_[1], $_[2]);
local $r;
foreach $r (sort { $a <=> $b } @r) {
	if ($r >= $_[3]) {
		return $r;
		}
	}
return undef;
}

# cron_range(range, min, max)
sub cron_range
{
	my $self = shift;

local ($w, $min, $max) = @_;
local $j;
local %inuse;
if ($w eq "*") {
	# all values
	for($j=$min; $j<=$max; $j++) { $inuse{$j}++; }
	}
elsif ($w =~ /^\*\/(\d+)$/) {
	# only every Nth
	for($j=$min; $j<=$max; $j+=$1) { $inuse{$j}++; }
	}
elsif ($w =~ /^(\d+)-(\d+)\/(\d+)$/) {
	# only every Nth of some range
	for($j=$1; $j<=$2; $j+=$3) { $inuse{int($j)}++; }
	}
elsif ($w =~ /^(\d+)-(\d+)$/) {
	# all of some range
	for($j=$1; $j<=$2; $j++) { $inuse{int($j)}++; }
	}
else {
	# One value
	$inuse{int($w)}++;
	}
return sort { $a <=> $b } (keys %inuse);
}

# cron_all_ranges(comma-list, min, max)
sub cron_all_ranges
{
	my $self = shift;

local @rv;
foreach $r (split(/,/, $_[0])) {
	push(@rv, $self->cron_range($r, $_[1], $_[2]));
	}
return sort { $a <=> $b } @rv;
}


1;

