#!/usr/bin/perl
package spam::SpamLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'spam::SpamLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# spam-lib.pl
# Common functions for parsing and editing the spamassassin config file
# XXX online help
# XXX whitelist editing?

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

$self->{'local_cf'} = $self->{'config_hash'}->{'local_cf'};
$self->{'warn_procmail'} = $self->{'config_hash'}->{'warn_procmail'};
if ($self->{'module_info_hash'}->{'usermin'}) {
	# Running under Usermin, editing user's personal config file
	$self->switch_to_remote_user();
	$self->create_user_config_dirs();
	if ($self->{'local_cf'} !~ /^\//) {
		$self->{'local_cf'} = "$self->{'remote_user_info_array'}->[7]/$self->{'local_cf'}";
		if ($self->{'local_cf'} =~ /^(.*)\// && !-d $1) {
			mkdir($1, 0700);
			}
		}
	}
else {
	# Running under Webmin, typically editing global config file
	%{$self->{'access_hash'}} = $self->get_module_acl();
	if ($self->{'access_hash'}->{'file'}) {
		$self->{'local_cf'} = $self->{'access_hash'}->{'file'};
		}
	if ($self->{'access_hash'}->{'nocheck'}) {
		$self->{'warn_procmail'} = 0;
		}
	}
$self->{'add_cf'} = !-d $self->{'local_cf'} ? $self->{'local_cf'} :
	  $self->{'module_info_hash'}->{'usermin'} ? "$self->{'local_cf'}/user_prefs" :
				    "$self->{'local_cf'}/local.cf";

### UI functions ###

1;

1;
		return $self;
}

# $self->get_config([file])
# Return a structure containing the contents of the spamassassin config file
sub get_config
{
	my $self = shift;

local @rv;
local $lnum = 0;
local $file = $_[0] || $self->{'local_cf'};
if (-d $file) {
	# A directory of files - read them all
	opendir(DIR, $file);
	local @files = sort { $a cmp $b } readdir(DIR);
	closedir(DIR);
	local $f;
	foreach $f (@files) {
		if ($f =~ /\.cf$/) {
			local $add = $self->get_config("$file/$f");
			map { $_->{'index'} += scalar(@rv) } @$add;
			push(@rv, @$add);
			}
		}
	}
else {
	# A single file that can be read right here
	open(FILE, $file);
	while(<FILE>) {
		s/\r|\n//g;
		s/^#.*$//;
		if (/^(\S+)\s*(.*)$/) {
			local $dir = { 'name' => $1,
				       'value' => $2,
				       'index' => scalar(@rv),
				       'file' => $file,
				       'line' => $lnum };
			$dir->{'words'} = [ split(/\s+/, $dir->{'value'}) ];
			push(@rv, $dir);
			}
		$lnum++;
		}
	close(FILE);
	}
return \@rv;
}

# find(name, &config)
sub find
{
	my $self = shift;

local @rv;
foreach $c (@{$_[1]}) {
	push(@rv, $c) if (lc($c->{'name'}) eq lc($_[0]));
	}
return wantarray ? @rv : $rv[0];
}

# find_value(name, &config)
sub find_value
{
	my $self = shift;

local @rv = map { $_->{'value'} } $self->find(@_);
return wantarray ? @rv : $rv[0];
}

# save_directives(&config, name|&old, &new, valuesonly)
# Update the config file with some directives
sub save_directives
{
	my $self = shift;

local @old = ref($_[1]) ? @{$_[1]} : $self->find($_[1], $_[0]);
local @new = $_[3] ? $self->make_directives($_[1], $_[2]) : @{$_[2]};
local $i;
for($i=0; $i<@old || $i<@new; $i++) {
	local $line;
	if ($new[$i]) {
		$line = $new[$i]->{'name'};
		$line .= " ".$new[$i]->{'value'} if ($new[$i]->{'value'} ne '');
		}
	if ($old[$i] && $new[$i]) {
		# Replacing a directive
		local $lref = $self->read_file_lines($old[$i]->{'file'});
		$lref->[$old[$i]->{'line'}] = $line;
		$_[0]->[$old[$i]->{'index'}] = $new[$i];
		}
	elsif ($old[$i]) {
		# Deleting a directive
		local $lref = $self->read_file_lines($old[$i]->{'file'});
		splice(@$lref, $old[$i]->{'line'}, 1);
		splice(@{$_[0]}, $old[$i]->{'index'}, 1);
		foreach $c (@{$_[0]}) {
			$c->{'line'}-- if ($c->{'line'} > $old[$i]->{'line'} &&
					   $c->{'file'} eq $old[$i]->{'file'});
			$c->{'index'}-- if ($c->{'index'} > $old[$i]->{'index'});
			}
		}
	elsif ($new[$i]) {
		# Adding a directive
		local $lref = $self->read_file_lines($self->{'add_cf'});
		$new[$i]->{'line'} = @$lref;
		$new[$i]->{'index'} = @{$_[0]};
		push(@$lref, $line);
		push(@{$_[0]}, $new[$i]);
		}
	}
}

# make_directives(name, &values)
sub make_directives
{
	my $self = shift;

return map { { 'name' => $_[0],
	       'value' => $_ } } @{$_[1]};
}

# edit_table(name, &headings, &&values, &sizes, [&convfunc], blankrows)
# Display a table of values for editing, with one blank row
sub edit_table
{
	my $self = shift;

local ($h, $v);
print "<table border>\n";
print "<tr $self->{'tb'}>\n";
foreach $h (@{$_[1]}) {
	print "<td><b>$h</b></td>\n";
	}
print "</tr>\n";
local $i = 0;
local $cfunc = $_[4] || \&default_convfunc;
local $blanks = $_[5] || 1;
foreach $v (@{$_[2]}, map { [ ] } (1 .. $blanks)) {
	print "<tr $self->{'cb'}>\n";
	for($j=0; $j<@{$_[1]}; $j++) {
		print "<td>",$self->$cfunc($j, "$_[0]_${i}_${j}", $_[3]->[$j],
				     $v->[$j], $v),"</td>";
		}
	print "</tr>\n";
	$i++;
	}
print "</table>\n";
}

# default_convfunc(column, name, size, value)
sub default_convfunc
{
	my $self = shift;

return "<input name=$_[1] size=$_[2] value='".$self->html_escape($_[3])."'>";
}

# parse_table(name, &parser)
# Parse the inputs from a table and return an array of results
sub parse_table
{
	my $self = shift;

local ($i, @rv);
local $pfunc = $_[1] || \&default_parsefunc;
for($i=0; defined($self->{'in_hash'}->{"$_[0]_${i}_0"}); $i++) {
	local ($j, $v, @vals);
	for($j=0; defined($v = $self->{'in_hash'}->{"$_[0]_${i}_${j}"}); $j++) {
		push(@vals, $v);
		}
	local $p = $self->$pfunc("$_[0]_${i}", @vals);
	push(@rv, $p) if (defined($p));
	}
return @rv;
}

# default_parsefunc(rowname, value, ...)
# Returns a value or undef if empty, or calls &error if invalid
sub default_parsefunc
{
	my $self = shift;

return $_[1] ? join(" ", @_[1..$#_]) : undef;
}

# start_form(cgi, header)
sub start_form
{
	my $self = shift;

print "<form action=$_[0] method=post>\n";
print "<table border width=100%>\n";
print "<tr $self->{'tb'}> <td><b>$_[1]</b></td> </tr>\n";
print "<tr $self->{'cb'}> <td><table width=100%>\n";
}

# end_form(buttonname, buttonvalue, ...)
sub end_form
{
	my $self = shift;

print "</table></td></tr></table>\n";
if (@_) {
	local $p = int(200 / scalar(@_));
	print "<table width=100%><tr>\n";
	local $i;
	for($i=0; $i<@_; $i+=2 ) {
		local $al = $i == 0 ? "align=left" :
			    $i == @_-2 ? "align=right" : "align=center";
		local $n = $_[$i] ? "name='$_[$i]'" : "";
		local $v = $self->html_escape($_[$i+1]);
		print "<td width=$p% $al><input type=submit $n value='$v'></td>\n";
		}
	print "</table>\n";
	}
print "</form>\n";
}

# yes_no_field(name, value, default)
sub yes_no_field
{
	my $self = shift;

local $v = !$_[1] ? -1 : $_[1]->{'value'};
local $def = $self->find_default($_[0], $_[2]) ? $self->{'text_hash'}->{'yes'} : $self->{'text_hash'}->{'no'};
printf "<input type=radio name=$_[0] value=1 %s> %s\n",
	$v == 1 ? "checked" : "", $self->{'text_hash'}->{'yes'};
printf "<input type=radio name=$_[0] value=0 %s> %s\n",
	$v == 0 ? "checked" : "", $self->{'text_hash'}->{'no'};
printf "<input type=radio name=$_[0] value=-1 %s> %s (%s)\n",
	$v == -1 ? "checked" : "", $self->{'text_hash'}->{'default'}, $def;
}

# parse_yes_no(&config, name)
sub parse_yes_no
{
	my $self = shift;

$self->save_directives($_[0], $_[1], $self->{'in_hash'}->{$_[1]} == 1 ? [ 1 ] :
			       $self->{'in_hash'}->{$_[1]} == 0 ? [ 0 ] : [ ], 1);
}

# option_field(name, value, default, &opts)
sub option_field
{
	my $self = shift;

local $v = !$_[1] ? -1 : $_[1]->{'value'};
local $def = $self->find_default($_[0], $_[2]);
local ($defopt) = grep { $_->[0] eq $def } @{$_[3]};
print $self->ui_radio($_[0], $v,
		[ @{$_[3]}, [ -1, "$self->{'text_hash'}->{'default'} ($defopt->[1])" ] ]);
}

sub parse_option
{
	my $self = shift;

$self->save_directives($_[0], $_[1], $self->{'in_hash'}->{$_[1]} == -1 ? [ ] : [ $self->{'in_hash'}->{$_[1]} ], 1);
}

# opt_field(name, value, size, default)
sub opt_field
{
	my $self = shift;

local $def = $self->find_default($_[0], $_[3]) if ($_[3]);
printf "<input type=radio name=$_[0]_def value=1 %s> %s %s\n",
	$_[1] ? "" : "checked", $self->{'text_hash'}->{'default'}, $_[3] ? " ($def)" : "";
printf "<input type=radio name=$_[0]_def value=0 %s>\n",
	$_[1] ? "checked" : "";
printf "<input name=$_[0] size=$_[2] value='%s'>\n",
	$_[1] ? $self->html_escape(ref($_[1]) ? $_[1]->{'value'} : $_[1]) : "";
}

# parse_opt(&config, name, [&checkfunc])
sub parse_opt
{
	my $self = shift;

if (defined($self->{'in_hash'}->{"$_[1]_default"}) && $self->{'in_hash'}->{"$_[1]_default"} eq $self->{'in_hash'}->{$_[1]} ||
    !defined($self->{'in_hash'}->{"$_[1]_default"}) && $self->{'in_hash'}->{"$_[1]_def"}) {
	$self->save_directives($_[0], $_[1], [ ], 1);
	}
else {
	&{$_[2]}($self->{'in_hash'}->{$_[1]}) if ($_[2]);
	$self->save_directives($_[0], $_[1], [ $self->{'in_hash'}->{$_[1]} ], 1);
	}
}

# edit_textbox(name, &values, width, height)
sub edit_textbox
{
	my $self = shift;

print "<textarea name=$_[0] cols=$_[2] rows=$_[3]>";
foreach $v (@{$_[1]}) {
	print "$v\n";
	}
print "</textarea>\n";
}

# parse_textbox(&config, name)
sub parse_textbox
{
	my $self = shift;

$self->{'in_hash'}->{$_[1]} =~ s/^\s+//;
$self->{'in_hash'}->{$_[1]} =~ s/\s+$//;
local @v = split(/\s+/, $self->{'in_hash'}->{$_[1]});
$self->save_directives($_[0], $_[1], \@v, 1);
}

# get_procmailrc()
# Returns the full paths to the procmail config files in use, the last one
# being the user's config
sub get_procmailrc
{
	my $self = shift;

if ($self->{'module_info_hash'}->{'usermin'}) {
	local @rv;
	push(@rv, $self->{'config_hash'}->{'global_procmailrc'});
	push(@rv, $self->{'config_hash'}->{'procmailrc'} || $main::procmail_object->{'procmailrc'});
	return @rv;
	}
else {
	return ( $self->{'access_hash'}->{'procmailrc'} || $self->{'config_hash'}->{'procmailrc'} || $main::procmail_object->{'procmailrc'} );
	}
}

# find_default(name, compiled-in-default)
sub find_default
{
	my $self = shift;

if ($self->{'config_hash'}->{'global_cf'}) {
	local $gconf = $self->get_config($self->{'config_hash'}->{'global_cf'});
	local $v = $self->find_value($_[0], $gconf);
	return $v if (defined($v));
	}
return $_[1];
}

# can_use_page(page)
# Returns 1 if some page can be used, 0 if not
sub can_use_page
{
	my $self = shift;

local %avail_icons;
if ($self->{'module_info_hash'}->{'usermin'}) {
	%avail_icons = map { $_, 1 } split(/,/, $self->{'config_hash'}->{'avail_icons'});
	}
else {
	%avail_icons = map { $_, 1 } split(/,/, $self->{'access_hash'}->{'avail'});
	}
local $p = $_[0] eq "simple" ? "header" : $_[0];
return $avail_icons{$p};
}

# can_use_check(page)
# Calls error if some page cannot be used
sub can_use_check
{
	my $self = shift;

$self->can_use_page($_[0]) || $self->error($self->{'text_hash'}->{'ecannot'});
}

# get_spamassassin_version(&out)
sub get_spamassassin_version
{
	my $self = shift;

local $out;
$self->execute_command("$self->{'config_hash'}->{'spamassassin'} -V", undef, \$out, \$out, 0, 1);
${$_[0]} = $out if ($_[0]);
return $out =~ /(version|Version:)\s+(\S+)/ ? $2 : undef;
}

# version_atleast(num)
sub version_atleast
{
	my $self = shift;

if (!$self->{'version_cache'}) {
	$self->{'version_cache'} = $self->get_spamassassin_version();
	}
return $self->{'version_cache'} >= $_[0];
}

# spam_file_folder()
sub spam_file_folder
{
	my $self = shift;

$self->foreign_require("mailbox", "mailbox-lib.pl");
local ($sf) = grep { $_->{'spam'} } $main::mailbox_object->list_folders();
return $sf;
}

# disable_indexing(&folder)
sub disable_indexing
{
	my $self = shift;

if (!$self->{'config_hash'}->{'index_spam'}) {
	$main::mailbox_object->{'config_hash'}->{'index_min'} = 1000000000;
	unlink($main::mailbox_object->user_index_file($_[0]->{'file'}));
	}
}

# get_process_pids()
# Returns the PIDs and names of SpamAssassin daemon processes like spamd
sub get_process_pids
{
	my $self = shift;

local ($pn, @pids);
foreach $pn (split(/\s+/, $self->{'config_hash'}->{'processes'})) {
	push(@pids, map { [ $_, $pn ] } $self->find_byname($pn));
	}
return @pids;
}

sub lock_spam_files
{
	my $self = shift;

local $conf = $self->get_config();
@{$self->{'spam_files_array'}} = $self->unique(map { $_->{'file'} } @$conf);
local $f;
foreach $f (@{$self->{'spam_files_array'}}) {
	$self->lock_file($f);
	}
}

sub unlock_spam_files
{
	my $self = shift;

local $f;
foreach $f (@{$self->{'spam_files_array'}}) {
	$self->unlock_file($f);
	}
}

# show_buttons(number)
sub show_buttons
{
	my $self = shift;

print "<table width=100%> <tr>\n";
local $onclick = "onClick='return check_clicks(form)'"
	if ($self->UNIVERSAL::can('check_clicks_function'));
print "<td align=left><input type=submit name=inbox value=\"$self->{'text_hash'}->{'mail_inbox'}\" $onclick></td>\n";
print "<td align=left><input type=submit name=whitelist value=\"$self->{'text_hash'}->{'mail_whitelist2'}\" $onclick></td>\n";
if ($self->has_command($self->{'config_hash'}->{'sa_learn'})) {
	print "<td align=center><input type=submit name=ham value=\"$self->{'text_hash'}->{'mail_ham'}\" $onclick></td>\n";
	}
print "<td align=right><input type=submit name=delete value=\"$self->{'text_hash'}->{'mail_delete'}\" $onclick></td>\n";
print "<td align=right><input type=submit name=razor value=\"$self->{'text_hash'}->{'mail_razor'}\" $onclick></td>\n";
print "</tr></table>\n";
}

# restart_spamd()
# Re-start all SpamAssassin processes, or return an error message
sub restart_spamd
{
	my $self = shift;

if ($self->{'config_hash'}->{'restart_cmd'}) {
	local $out = $self->backquote_logged(
		"$self->{'config_hash'}->{'restart_cmd'} 2>&1 </dev/null");
	if ($? || $out =~ /error|failed/i) {
		return "<pre>$out</pre>";
		}
	}
else {
	local @pids = $self->get_process_pids();
	@pids || return $self->{'text_hash'}->{'apply_none'};
	local $p;
	foreach $p (@pids) {
		$self->kill_logged("HUP", $p->[0]);
		}
	}
return undef;
}

# find_spam_recipe(&recipes)
# Returns the recipe that runs spamassassin
sub find_spam_recipe
{
	my $self = shift;

local $r;
foreach $r (@{$_[0]}) {
	if ($r->{'action'} =~ /spamassassin/i ||
	    $r->{'action'} =~ /spamc/i) {
		return $r;
		}
	}
return undef;
}

# find_file_recipe(&recipes)
# Returns the recipe for delivering mail based on the X-Spam-Status header
sub find_file_recipe
{
	my $self = shift;

local ($r, $c);
foreach $r (@{$_[0]}) {
	foreach $c (@{$r->{'conds'}}) {
		if ($c->[1] =~ /X-Spam-Status/i) {
			return $r;
			}
		}
	}
return undef;
}

# find_virtualmin_recipe(&recipes)
# Returns the recipe that runs the Virtualmin lookup command
sub find_virtualmin_recipe
{
	my $self = shift;

local ($r, $c);
foreach $r (@{$_[0]}) {
	if ($r->{'action'} =~ /^VIRTUALMIN=/) {
		return $r;
		}
	}
return undef;
}

# find_force_default_receipe(&recipes)
# Returns the recipe that forces delivery to $DEFAULT, used by Virtualmin and
# others to prevent per-user .procmailrc settings
sub find_force_default_receipe
{
	my $self = shift;

local ($r, $c);
foreach $r (@{$_[0]}) {
	if ($r->{'action'} eq '$DEFAULT' && !@{$r->{'conds'}}) {
		return $r;
		}
	}
return undef;
}

# get_simple_tests(&conf)
sub get_simple_tests
{
	my $self = shift;

local ($conf) = @_;
local (@simple, %simple);
foreach my $h ($self->find("header", $conf)) {
	if ($h->{'value'} =~ /^(\S+)\s+(\S+)\s+=~\s+\/(.*)\/(\S*)\s*$/) {
		push(@{$self->{'simples_array'}}, { 'header_dir' => $h,
				 'name' => $1,
				 'header' => lc($2),
			 	 'regexp' => $3,
				 'flags' => $4, });
		$self->{'simples_hash'}->{$1} = $self->{'simples_array'}->[$#{$self->{'simples_array'}}];
		}
	}
foreach my $b ($self->find("body", $conf), $self->find("full", $conf),
	       $self->find("uri", $conf)) {
	if ($b->{'value'} =~ /^(\S+)\s+\/(.*)\/(\S*)\s*$/) {
		push(@{$self->{'simples_array'}}, { $b->{'name'}.'_dir' => $b,
				 'name' => $1,
				 'header' => $b->{'name'},
			 	 'regexp' => $2,
				 'flags' => $3, });
		$self->{'simples_hash'}->{$1} = $self->{'simples_array'}->[$#{$self->{'simples_array'}}];
		}
	}
foreach my $s ($self->find("score", $conf)) {
	if ($s->{'value'} =~ /^(\S+)\s+(\S+)/ && $self->{'simples_hash'}->{$1}) {
		$self->{'simples_hash'}->{$1}->{'score_dir'} = $s;
		$self->{'simples_hash'}->{$1}->{'score'} = $2;
		}
	}
foreach my $d ($self->find("describe", $conf)) {
	if ($d->{'value'} =~ /^(\S+)\s+(\S.*)/ && $self->{'simples_hash'}->{$1}) {
		$self->{'simples_hash'}->{$1}->{'describe_dir'} = $d;
		$self->{'simples_hash'}->{$1}->{'describe'} = $2;
		}
	}
return @{$self->{'simples_array'}};
}

# get_procmail_command()
# Returns the command that should be used in /etc/procmailrc to call
# spamassassin, such as spamc or the full spamassassin path
sub get_procmail_command
{
	my $self = shift;

if ($self->{'config_hash'}->{'procmail_cmd'} eq '*') {
	# Is spamd running?
	if ($self->get_process_pids()) {
		local $spamc = $self->has_command("spamc");
		return $spamc if ($spamc);
		}
	return $self->has_command($self->{'config_hash'}->{'spamassassin'});
	}
elsif ($self->{'config_hash'}->{'procmail_cmd'}) {
	return $self->{'config_hash'}->{'procmail_cmd'};
	}
else {
	return $self->has_command($self->{'config_hash'}->{'spamassassin'});
	}
}

# execute_before(section)
# If a before-change command is configured, run it. If it fails, call error
sub execute_before
{
	my $self = shift;

local ($section) = @_;
if ($self->{'config_hash'}->{'before_cmd'}) {
	$ENV{'SPAM_SECTION'} = $section;
	local $out;
	local $rv = $self->execute_command(
			$self->{'config_hash'}->{'before_cmd'}, undef, \$out, \$out);
	$rv && $self->error($self->text('before_ecmd',
			    "<pre>".$self->html_escape($out)."</pre>"));
	}
}

# execute_after(section)
# If a after-change command is configured, run it. If it fails, call error
sub execute_after
{
	my $self = shift;

local ($section) = @_;
if ($self->{'config_hash'}->{'after_cmd'}) {
	$ENV{'SPAM_SECTION'} = $section;
	local $out;
	local $rv = $self->execute_command(
			$self->{'config_hash'}->{'after_cmd'}, undef, \$out, \$out);
	$rv && $self->error($self->text('after_ecmd',
			    "<pre>".$self->html_escape($out)."</pre>"));
	}
}


1;

