#!/usr/bin/perl
package ssh::SshLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'ssh::SshLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# ssh-lib.pl
# Common functions for manipulating the users .ssh directory files

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
$self->switch_to_remote_user();

$self->{'ssh_directory'} = "$self->{'remote_user_info_array'}->[7]/.ssh";
$self->{'ssh_client_config'} = "$self->{'ssh_directory'}/config";
$self->{'authorized_keys'} = "$self->{'ssh_directory'}/authorized_keys";
$self->{'known_hosts'} = "$self->{'ssh_directory'}/known_hosts";

1;

1;
		return $self;
}

# find_value(name, &config)
sub find_value
{
	my $self = shift;

foreach $c (@{$_[1]}) {
	if (lc($c->{'name'}) eq lc($_[0])) {
		return wantarray ? @{$c->{'values'}} : $c->{'values'}->[0];
		}
	}
return wantarray ? ( ) : undef;
}

# find(value, &config)
sub find
{
	my $self = shift;

local @rv;
foreach $c (@{$_[1]}) {
	if (lc($c->{'name'}) eq lc($_[0])) {
		push(@rv, $c);
		}
	}
return wantarray ? @rv : $rv[0];
}

# save_directive(name, &config, [value*])
sub save_directive
{
	my $self = shift;

local @o = $self->find($_[0], $_[1]);
local @n = grep { defined($_) } @_[2..@_-1];
local $lref = $self->read_file_lines($_[1]->[0]->{'file'});
local $id = ("\t" x $_[1]->[0]->{'indent'});
local $i;
for($i=0; $i<@o || $i<@n; $i++) {
	if ($o[$i] && $n[$i]) {
		# Replacing a line
		$lref->[$o[$i]->{'line'}] = "$id$_[0] $n[$i]";
		}
	elsif ($o[$i]) {
		# Removing a line
		splice(@$lref, $o[$i]->{'line'}, 1);
		foreach $c (@{$_[1]}) {
			if ($c->{'line'} > $o[$i]->{'line'}) {
				$c->{'line'}--;
				}
			}
		}
	elsif ($n[$i]) {
		# Adding a line
		local $ll = $_[1]->[@{$_[1]}-1]->{'line'};
		splice(@$lref, $ll+1, 0, "$id$_[0] $n[$i]");
		}
	}
}

# scmd(double)
sub scmd
{
	my $self = shift;

if ($self->{'cmd_count'} % 2 == 0) {
	print "<tr>\n";
	}
elsif ($_[0]) {
	print "<td colspan=2></td> </tr>\n";
	print "<tr>\n";
	$self->{'cmd_count'} = 0;
	}
$self->{'cmd_count'} += ($_[0] ? 2 : 1);
}

# ecmd()
sub ecmd
{
	my $self = shift;

if ($self->{'cmd_count'} % 2 == 0) {
	print "</tr>\n";
	}
}

# get_client_config()
# Returns a list of structures, one for each host
sub get_client_config
{
	my $self = shift;

local @rv = ( { 'dummy' => 1,
		'indent' => 0,
		'file' => $self->{'ssh_client_config'},
		'line' => -1,
		'eline' => -1 } );
local $host;
local $lnum = 0;
open(CLIENT, $self->{'ssh_client_config'});
while(<CLIENT>) {
	s/\r|\n//g;
	s/^\s*#.*$//g;
	s/^\s*//g;
	local ($name, @values) = split(/\s+/, $_);
	if (lc($name) eq 'host') {
		# Start of new host
		$host = { 'name' => $name,
			  'values' => \@values,
			  'file' => $self->{'ssh_client_config'},
			  'line' => $lnum,
			  'eline' => $lnum,
			  'members' => [ { 'dummy' => 1,
					   'indent' => 1,
					   'file' => $self->{'ssh_client_config'},
					   'line' => $lnum } ] };
		push(@rv, $host);
		}
	elsif ($name) {
		# A directive inside a host
		local $dir = { 'name' => $name,
			       'values' => \@values,
			       'file' => $self->{'ssh_client_config'},
			       'line' => $lnum };
		push(@{$host->{'members'}}, $dir);
		$host->{'eline'} = $lnum;
		}
	$lnum++;
	}
close(CLIENT);
return \@rv;
}

# create_host(&host)
sub create_host
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'ssh_client_config'});
$_[0]->{'line'} = $_[0]->{'eline'} = scalar(@$lref);
push(@$lref, "Host ".join(" ", @{$_[0]->{'values'}}));
$_[0]->{'members'} = [ { 'dummy' => 1,
			 'indent' => 1,
			 'file' => $self->{'ssh_client_config'},
			 'line' => $_[0]->{'line'} } ];
}

# modify_host(&host)
sub modify_host
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'ssh_client_config'});
$lref->[$_[0]->{'line'}] = "Host ".join(" ", @{$_[0]->{'values'}});
}

# delete_host(&host)
sub delete_host
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'ssh_client_config'});
splice(@$lref, $_[0]->{'line'}, $_[0]->{'eline'} - $_[0]->{'line'} + 1);
}

# list_auths()
# Returns a list of authorized key structures
sub list_auths
{
	my $self = shift;

local @rv;
local $lnum = 0;
open(AUTHS, $self->{'authorized_keys'});
while(<AUTHS>) {
	s/\r|\n//g;
	s/#.*$//;
	if (/^((.*)\s+)?(\d+)\s+(\d+)\s+(\d+)\s+(\S+)$/) {
		# SSH1 style line
		local $auth = { 'name' => $6,
				'key' => $5,
				'exp' => $4,
				'bits' => $3,
				'opts' => $2,
				'type' => 1,
				'line' => $lnum,
				'index' => scalar(@rv) };
		$auth->{'opts'} =~ s/\s+$//;
		push(@rv, $auth);
		}
	elsif (/^((.*)\s+)?([a-z]\S+)\s+(\S+)\s+(\S+)/) {
		# SSH2 style line
		local $auth = { 'name' => $5,
				'key' => $4,
				'opts' => $2,
				'keytype' => $3,
				'type' => 2,
				'line' => $lnum,
				'index' => scalar(@rv) };
		$auth->{'opts'} =~ s/\s+$//;
		push(@rv, $auth);
		}
	$lnum++;
	}
close(AUTHS);
return @rv;
}

# create_auth(&auth)
sub create_auth
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'authorized_keys'});
push(@$lref, $self->auth_line($_[0]));
$self->flush_file_lines();
}

# delete_auth(&auth)
sub delete_auth
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'authorized_keys'});
splice(@$lref, $_[0]->{'line'}, 1);
$self->flush_file_lines();
}

# modify_auth(&auth)
sub modify_auth
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'authorized_keys'});
$lref->[$_[0]->{'line'}] = $self->auth_line($_[0]);
$self->flush_file_lines();
}

# parse_options(string, &opts)
sub parse_options
{
	my $self = shift;

local $opts = $_[0];
while($opts =~ /^([^=\s,]+)=\"([^\"]*)\",?(.*)$/ ||
      $opts =~ /^([^=\s,]+)=([^=\s,]+),?(.*)$/ ||
      $opts =~ /^([^=\s,]+)(),?(.*)$/) {
	push(@{$_[1]->{$1}}, $2);
	$opts = $3;
	}
}

# join_options(&opts)
sub join_options
{
	my $self = shift;

local @rv;
foreach $o (keys %{$_[0]}) {
	foreach $ov (@{$_[0]->{$o}}) {
		if (defined($ov)) {
			push(@rv, "$o=\"$ov\"");
			}
		else {
			push(@rv, $o);
			}
		}
	}
return join(",", @rv);
}

sub auth_line
{
	my $self = shift;

if ($_[0]->{'type'} == 2) {
	# SSH 2 format line
	return join(" ", $_[0]->{'opts'} ? ( $_[0]->{'opts'} ) : ( ),
			 $_[0]->{'keytype'}, $_[0]->{'key'}, $_[0]->{'name'} );
	}
else {
	# SSH 1 format line
	return join(" ", $_[0]->{'opts'} ? ( $_[0]->{'opts'} ) : ( ),
			 $_[0]->{'bits'}, $_[0]->{'exp'}, $_[0]->{'key'}, $_[0]->{'name'} );
	}
}

# list_knowns()
# Returns a list of known host structures
sub list_knowns
{
	my $self = shift;

local @rv;
local $lnum = 0;
open(KNOWNS, $self->{'known_hosts'});
while(<KNOWNS>) {
	s/\r|\n//g;
	s/#.*$//;
	if (/^(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s*(.*)$/) {
		local $known = { 'comment' => $5,
				 'key' => $4,
				 'exp' => $3,
				 'bits' => $2,
				 'hosts' => [ split(/,/, $1) ],
				 'line' => $lnum,
				 'index' => scalar(@rv) };
		push(@rv, $known);
		}
	$lnum++;
	}
close(KNOWNS);
return @rv;
}

# create_known(&known)
sub create_known
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'known_hosts'});
push(@$lref, $self->known_line($_[0]));
$self->flush_file_lines();
}

# delete_known(&known)
sub delete_known
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'known_hosts'});
splice(@$lref, $_[0]->{'line'}, 1);
$self->flush_file_lines();
}

# modify_known(&known)
sub modify_known
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'known_hosts'});
$lref->[$_[0]->{'line'}] = $self->known_line($_[0]);
$self->flush_file_lines();
}

sub known_line
{
	my $self = shift;

return join(" ", join(",", @{$_[0]->{'hosts'}}), $_[0]->{'bits'}, $_[0]->{'exp'},
	    $_[0]->{'key'}, $_[0]->{'comment'} ? ( $_[0]->{'comment'} ) : ( ) );
}

# get_ssh_type()
# Returns either 'openssh' or 'ssh'
sub get_ssh_type
{
	my $self = shift;

local $out = `ssh -V 2>&1 </dev/null`;
return $out =~ /openssh/i ? 'openssh' :
       $out =~ /Sun_SSH/i ? 'openssh' :
			    'ssh';
}

# get_ssh_version()
# Returns the SSH version number
sub get_ssh_version
{
	my $self = shift;

local $out = `ssh -V 2>&1 </dev/null`;
if ($out =~ /(sshd\s+version\s+([0-9\.]+))/i ||
    $out =~ /(ssh\s+secure\s+shell\s+([0-9\.]+))/i) {
	# Classic commercial SSH
	return $2;
	}
elsif ($out =~ /(OpenSSH.([0-9\.]+))/i) {
	# OpenSSH .. assume all versions are supported
	return $2;
	}
elsif ($out =~ /(Sun_SSH_([0-9\.]+))/i) {
	# Solaris 9 SSH is actually OpenSSH 2.x
	return 2.0;
	}
elsif (($out = $self->{'config_hash'}->{'sshd_version'}) && ($out =~ /(Sun_SSH_([0-9\.]+))/i)) {
	# Probably Solaris 10 SSHD that didn't display version.  Use it.
	return 2.0;
	}
else {
	return undef;
	}
}


1;

