#!/usr/bin/perl
package usermount::LinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'usermount::LinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# linux-lib.pl
# Mount table functions for linux

if (!$self->{'no_check_support'}) {
	if ($self->has_command("amd")) {
		 $self->{'amd'} = $self->read_amd_conf();
		$self->{'amd_support'} = $self->{'amd'} =~ /\[\s*global\s*\]/i ? 2 : 1;
		}
	$self->{'autofs_support'} = $self->has_command("automount");
	if ($self->has_command("mount.cifs")) {
		$self->{'cifs_support'} = 4;
		}
	if ($self->has_command("mount.smbfs")) {
		$self->{'smbfs_support'} = $self->backquote_command("mount.smbfs -v", 1) =~ /username=/i ? 4 : 3;
		$self->{'smbfs_fs'} = "smbfs";
		}
	elsif ($self->has_command("mount.smb")) {
		$self->{'smbfs_support'} = $self->backquote_command("mount.smb -v", 1) =~ /username=/i ? 4 : 3;
		$self->{'smbfs_fs'} = "smb";
		}
	elsif ($self->has_command("smbmount")) {
		$self->{'smbfs_support'} = $self->backquote_command("smbmount -v", 1) =~ /Version\s+2/i ? 2 : 1;
		$self->{'smbfs_fs'} = "smbfs";
		}
	$self->{'swaps_support'} = -r "/proc/swaps";
	if (`uname -r` =~ /^(\d+\.\d+)/ && $1 >= 2.4) {
		$self->{'tmpfs_support'} = 1;
		$self->{'ext3_support'} = 1;
		$self->{'no_mount_check'} = 1;
		$self->{'bind_support'} = 1;	# XXX which version?
		}
	if ($self->has_command("mkfs.xfs")) {
		$self->{'xfs_support'} = 1;
		}
	if ($self->has_command("mkfs.jfs")) {
		$self->{'jfs_support'} = 1;
		}
	}

# We always need to check this, to fix up LABEL= mounts
if ($self->has_command("e2label")) {
	$self->{'has_e2label'} = 1;
	}
if ($self->has_command("xfs_db")) {
	$self->{'has_xfs_db'} = 1;
	}
if ($self->has_command("vol_id")) {
	$self->{'has_volid'} = 1;
	}

1;

1;
		return $self;
}

# Return information about a filesystem, in the form:
#  directory, device, type, options, fsck_order, mount_at_boot
# If a field is unused or ignored, a - appears instead of the value.
# Swap-filesystems (devices or files mounted for VM) have a type of 'swap',
# and 'swap' in the directory field
sub list_mounts
{
	my $self = shift;

return @{$self->{'list_mounts_cache_array'}} if (@{$self->{'list_mounts_cache_array'}});
local(@rv, @p, @o, $_, $i, $j); $i = 0;

# Get /etc/fstab mounts
open(FSTAB, $self->{'config_hash'}->{fstab_file});
while(<FSTAB>) {
	local(@o, $at_boot);
	chop; s/#.*$//g;
	if (!/\S/ || /\signore\s/) { next; }
	@p = split(/\s+/, $_);
	if ($p[2] eq "proc") { $p[0] = "proc"; }
	elsif ($p[2] eq "auto") { $p[2] = "*"; }
	elsif ($p[2] eq "swap") { $p[1] = "swap"; }
	elsif ($p[2] eq $self->{'smbfs_fs'} || $p[2] eq "cifs") {
		$p[0] =~ s/\\040/ /g;
		$p[0] =~ s/\//\\/g;
		}
	$rv[$i] = [ $p[1], $p[0], $p[2] ];
	$rv[$i]->[5] = "yes";
	@o = split(/,/ , $p[3] eq "defaults" ? "" : $p[3]);
	if (($j = $self->indexof("noauto", @o)) >= 0) {
		# filesytem is not mounted at boot
		splice(@o, $j, 1);
		$rv[$i]->[5] = "no";
		}
	if (($j = $self->indexof("bind", @o)) >= 0) {
		# Special bind option, which indicates a loopback filesystem
		splice(@o, $j, 1);
		$p[0] = "bind";
		}
	$rv[$i]->[3] = (@o ? join(',' , @o) : "-");
	$rv[$i]->[4] = (@p >= 5 ? $p[5] : 0);
	$i++;
	}
close(FSTAB);

if ($self->{'amd_support'} == 1) {
	# Get old automounter configuration, as used by redhat
	local $amd = $self->read_amd_conf();
	if ($amd =~ /MOUNTPTS='(.*)'/) {
		@p = split(/\s+/, $1);
		for($j=0; $j<@p; $j+=2) {
			$rv[$i++] = [ $p[$j], $p[$j+1], "auto",
				      "-", 0, "yes" ];
			}
		}
	}
elsif ($self->{'amd_support'} == 2) {
	# Guess what? There's now a *new* amd config file format, introduced
	# in redhat 6.1 and caldera 2.3
	local @amd = $self->parse_amd_conf();
	local @sp = split(/:/, $amd[0]->{'opts'}->{'search_path'});
	local ($am, $sp);
	foreach $am (@amd) {
		local $mn = $am->{'opts'}->{'map_name'};
		if ($mn !~ /^\//) {
			foreach $sp (@sp) {
				if (-r "$sp/$mn") {
					$mn = "$sp/$mn";
					last;
					}
				}
			}
		$rv[$i++] = [ $am->{'dir'}, $mn,
			      "auto", $am->{'opts'}, 0, "yes" ]
			if ($am->{'dir'} ne 'global');
		}
	}

# Get kernel automounter configuration
if ($self->{'autofs_support'}) {
	open(AUTO, $self->{'config_hash'}->{'autofs_file'});
	while(<AUTO>) {
		chop;
		s/#.*$//g;
		if (/^\s*(\S+)\s+(\S+)\s*(.*)$/) {
			$rv[$i++] = [ $1, $2, "autofs",
				      ($3 ? $self->autofs_options($3) : "-"),
				      0, "yes" ];
			}
		}
	close(AUTO);
	}

@{$self->{'list_mounts_cache_array'}} = @rv;
return @rv;
}

# create_mount(directory, device, type, options, fsck_order, mount_at_boot)
# Add a new entry to the fstab file, old or new automounter file
sub create_mount
{
	my $self = shift;

local(@mlist, @amd, $_); local($opts);

if ($_[2] eq "auto") {
	if ($self->{'amd_support'} == 1) {
		# Adding an old automounter mount
		local $amd = $self->read_amd_conf();
		local $m = "$_[0] $_[1]";
		if ($amd =~ /MOUNTPTS=''/) {
			$amd =~ s/MOUNTPTS=''/MOUNTPTS='$m'/;
			}
		else {
			$amd =~ s/MOUNTPTS='(.*)'/MOUNTPTS='$1 $m'/;
			}
		$self->write_amd_conf($amd);
		}
	elsif ($self->{'amd_support'} == 2) {
		# Adding a new automounter mount
		local @amfs = split(/\s+/, $self->{'config_hash'}->{'auto_file'});
		$self->open_tempfile(AMD, ">>$amfs[0]");
		$self->print_tempfile(AMD, "\n");
		$self->print_tempfile(AMD, "[ $_[0] ]\n");
		$self->print_tempfile(AMD, "map_name = $_[1]\n");
		$self->close_tempfile(AMD);
		}
	}
elsif ($_[2] eq "autofs") {
	# Adding a new automounter mount
	$self->open_tempfile(AUTO, ">> $self->{'config_hash'}->{'autofs_file'}");
	$self->print_tempfile(AUTO, "$_[0]  $_[1]");
	if ($_[3]) {
		$self->print_tempfile(AUTO, "  ",$self->autofs_args($_[3]));
		}
	$self->print_tempfile(AUTO, "\n");
	$self->close_tempfile(AUTO);
	}
else {
	# Adding a normal mount to the fstab file
	local $dev = $_[1];
	if ($_[2] eq $self->{'smbfs_fs'} || $_[2] eq "cifs") {
		$dev =~ s/\\/\//g;
		$dev =~ s/ /\\040/g;
		}
	$self->open_tempfile(FSTAB, ">> $self->{'config_hash'}->{fstab_file}");
	$self->print_tempfile(FSTAB, "$dev  $_[0]  $_[2]");
	local @opts = $_[3] eq "-" ? ( ) : split(/,/, $_[3]);
	if ($_[5] eq "no") {
		push(@opts, "noauto");
		}
	else {
		@opts = grep { $_ !~ /^(auto|noauto)$/ } @opts;
		}
	if ($_[2] eq "bind") {
		push(@opts, "bind");
		}
	if (!@opts) { $self->print_tempfile(FSTAB, "  defaults"); }
	else { $self->print_tempfile(FSTAB, "  ".join(",", @opts)); }
	$self->print_tempfile(FSTAB, "  0  ");
	$self->print_tempfile(FSTAB, $_[4] eq "-" ? "0\n" : "$_[4]\n");
	$self->close_tempfile(FSTAB);
	}
undef(@{$self->{'list_mounts_cache_array'}});
}

# change_mount(num, directory, device, type, options, fsck_order, mount_at_boot)
# Change an existing permanent mount
sub change_mount
{
	my $self = shift;

local($i, @fstab, $line, $opts, $j, @amd);
$i = 0;

# Update fstab file
open(FSTAB, $self->{'config_hash'}->{fstab_file});
@fstab = <FSTAB>;
close(FSTAB);
$self->open_tempfile(FSTAB, "> $self->{'config_hash'}->{fstab_file}");
foreach (@fstab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $line !~ /\signore\s/ && $i++ == $_[0]) {
		# Found the line to replace
		local $dev = $_[2];
		$dev =~ s/ /\\040/g;
		$self->print_tempfile(FSTAB, "$dev  $_[1]  $_[3]");
		local @opts = $_[4] eq "-" ? ( ) : split(/,/, $_[4]);
		if ($_[6] eq "no") {
			push(@opts, "noauto");
			}
		else {
			@opts = grep { $_ !~ /^(auto|noauto)$/ } @opts;
			}
		if ($_[3] eq "bind") {
			push(@opts, "bind");
			}
		if (!@opts) { $self->print_tempfile(FSTAB, "  defaults"); }
		else { $self->print_tempfile(FSTAB, "  ".join(",", @opts)); }
		$self->print_tempfile(FSTAB, "  0  ");
		$self->print_tempfile(FSTAB, $_[5] eq "-" ? "0\n" : "$_[5]\n");
		}
	else { $self->print_tempfile(FSTAB, $_,"\n"); }
	}
$self->close_tempfile(FSTAB);

if ($self->{'amd_support'} == 1) {
	# Update older amd configuration
	local $amd = $self->read_amd_conf();
	if ($amd =~ /MOUNTPTS='(.*)'/) {
		# found mount points line..
		local @mpts = split(/\s+/, $1);
		for($j=0; $j<@mpts; $j+=2) {
			if ($i++ == $_[0]) {
				$mpts[$j] = $_[1];
				$mpts[$j+1] = $_[2];
				}
			}
		local $mpts = join(" ", @mpts);
		$amd =~ s/MOUNTPTS='(.*)'/MOUNTPTS='$mpts'/;
		}
	$self->write_amd_conf($amd);
	}
elsif ($self->{'amd_support'} == 2) {
	# Update new amd configuration
	local @amd = $self->parse_amd_conf();
	foreach $am (@amd) {
		next if ($am->{'dir'} eq 'global');
		if ($i++ == $_[0]) {
			local $lref = $self->read_file_lines($am->{'file'});
			local @nl = ( "[ $_[1] ]" );
			local %opts = %{$am->{'opts'}};
			$opts{'map_name'} = $_[2];
			foreach $o (keys %opts) {
				push(@nl, "$o = $opts{$o}");
				}
			splice(@$lref, $am->{'line'},
			       $am->{'eline'} - $am->{'line'} + 1, @nl);
			$self->flush_file_lines();
			}
		}
	}

# Update autofs configuration
if ($self->{'autofs_support'}) {
	open(AUTO, $self->{'config_hash'}->{'autofs_file'});
	@{$self->{'auto_array'}} = <AUTO>;
	close(AUTO);
	$self->open_tempfile(AUTO, "> $self->{'config_hash'}->{'autofs_file'}");
	foreach (@{$self->{'auto_array'}}) {
		chop; ($line = $_) =~ s/#.*$//g;
		if ($line =~ /\S/ && $i++ == $_[0]) {
			$self->print_tempfile(AUTO, "$_[1]  $_[2]");
			if ($_[4]) {
				$self->print_tempfile(AUTO, "  ",$self->autofs_args($_[4]));
				}
			$self->print_tempfile(AUTO, "\n");
			}
		else {
			$self->print_tempfile(AUTO, $_,"\n");
			}
		}
	$self->close_tempfile(AUTO);
	}
undef(@{$self->{'list_mounts_cache_array'}});
}

# delete_mount(index)
# Delete an existing permanent mount
sub delete_mount
{
	my $self = shift;

local($i, @fstab, $line, $opts, $j, @amd);
$i = 0;

# Update fstab file
open(FSTAB, $self->{'config_hash'}->{fstab_file});
@fstab = <FSTAB>;
close(FSTAB);
$self->open_tempfile(FSTAB, ">$self->{'config_hash'}->{fstab_file}");
foreach (@fstab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line !~ /\S/ || $line =~ /\signore\s/ || $i++ != $_[0]) {
		# Don't delete this line
		$self->print_tempfile(FSTAB, $_,"\n");
		}
	}
$self->close_tempfile(FSTAB);

if ($self->{'amd_support'} == 1) {
	# Update older amd configuration
	local $foundamd = 0;
	local $amd = $self->read_amd_conf();
	if ($amd =~ /MOUNTPTS='(.*)'/) {
		# found mount points line..
		local @mpts = split(/\s+/, $1);
		for($j=0; $j<@mpts; $j+=2) {
			if ($i++ == $_[0]) {
				splice(@mpts, $j, 2);
				$foundamd = 1;
				}
			}
		local $mpts = join(" ", @mpts);
		$amd =~ s/MOUNTPTS='(.*)'/MOUNTPTS='$mpts'/;
		}
	$self->write_amd_conf($amd) if ($foundamd);
	}
elsif ($self->{'amd_support'} == 2) {
	# Update new amd configuration
	local @amd = $self->parse_amd_conf();
	foreach $am (@amd) {
		next if ($am->{'dir'} eq 'global');
		if ($i++ == $_[0]) {
			local $lref = $self->read_file_lines($am->{'file'});
			splice(@$lref, $am->{'line'},
			       $am->{'eline'} - $am->{'line'} + 1);
			$self->flush_file_lines();
			}
		}
	}

# Update AMD file
if ($self->{'amd_support'}) {
	open(AMD, $self->{'config_hash'}->{auto_file});
	@amd = <AMD>;
	close(AMD);
	$self->open_tempfile(AMD, ">$self->{'config_hash'}->{auto_file}");
	foreach (@amd) {
		if (/MOUNTPTS='(.*)'/) {
			# found mount points line..
			@mpts = split(/\s+/, $1);
			for($j=0; $j<@mpts; $j+=2) {
				if ($i++ != $_[0]) {
					push(@{$self->{'nmpts_array'}}, $mpts[$j]);
					push(@{$self->{'nmpts_array'}}, $mpts[$j+1]);
					}
				}
			$self->print_tempfile(AMD, "MOUNTPTS='".join(' ', @{$self->{'nmpts_array'}})."'\n");
			}
		else { $self->print_tempfile(AMD, $_); }
		}
	$self->close_tempfile(AMD);
	}

# Update autofs file
if ($self->{'autofs_support'}) {
	open(AUTO, $self->{'config_hash'}->{'autofs_file'});
	@{$self->{'auto_array'}} = <AUTO>;
	close(AUTO);
	$self->open_tempfile(AUTO, ">$self->{'config_hash'}->{'autofs_file'}");
	foreach (@{$self->{'auto_array'}}) {
		chop; ($line = $_) =~ s/#.*$//g;
		if ($line !~ /\S/ || $i++ != $_[0]) {
			# keep this line
			$self->print_tempfile(AUTO, $_,"\n");
			}
		}
	$self->close_tempfile(AUTO);
	}
undef(@{$self->{'list_mounts_cache_array'}});
}

# list_mounted([no-label])
# Return a list of all the currently mounted filesystems and swap files.
# The list is in the form:
#  directory device type options
sub list_mounted
{
	my $self = shift;

return @{$self->{'list_mounted_cache_array'}}
	if (@{$self->{'list_mounted_cache_array'}} && $self->{'list_mounted_cache_mode'} == $_[0]);
local(@rv, @p, @o, $mo, $_, %smbopts);
local @mounts = $self->list_mounts();

$self->read_smbopts();
open(MTAB, "/etc/mtab");
while(<MTAB>) {
	chop;
	s/#.*$//g; if (!/\S/) { next; }
	@p = split(/\s+/, $_);
	if ($p[2] eq "auto" || $p[0] =~ /^\S+:\(pid\d+\)$/) {
		# Automounter map.. turn the map= option into the device
		@o = split(/,/ , $p[3]);
		($mo) = grep {/^map=/} (@o);
		$mo =~ /^map=(.*)$/; $p[0] = $1;
		$p[3] = join(',' , grep {!/^map=/} (@o));
		$p[2] = "auto";
		}
	elsif ($p[2] eq "autofs") {
		# Kernel automounter map.. use the pid to find the map
		$p[0] =~ /automount\(pid(\d+)\)/ || next;
		$out = $self->backquote_command("ps hwwww $1", 1);
		$out =~ /automount\s+(.*)\s*(\S+)\s+(file|program|yp)(,\S+)?\s+(\S+)/ || next;
		$p[0] = $5;
		$p[3] = $1 ? $self->autofs_options($1) : "-";
		}
	elsif ($p[2] eq $self->{'smbfs_fs'} || $p[2] eq "cifs") {
		# Change from //FOO/BAR to \\foo\bar
		$p[0] =~ s/\//\\/g;
		$p[0] = lc($p[0]);
		$p[3] = $smbopts{$p[1]};
		}
	elsif ($p[2] eq "proc") {
		# The source for proc mounts is always proc
		$p[0] = "proc";
		}
	if (!$_[0] && ($p[2] =~ /^ext[23]$/ && $self->{'has_e2label'} ||
	    	       $p[2] eq "xfs" && $self->{'has_xfs_db'})) {
		# Check for a label on this partition, and there is one
		# and this filesystem is in fstab with the label, change
		# the device.
		local $label;
		if ($p[2] eq "xfs") {
			local $out = $self->backquote_command("xfs_db -x -p xfs_admin -c label -r $p[0] 2>&1", 1);
			$label = $1 if ($out =~ /label\s*=\s*"(.*)"/ &&
					$1 ne '(null)');
			}
		else {
			$label = $self->backquote_command("e2label $p[0] 2>&1", 1);
			chop($label);
			}
		if (!$?) {
			foreach $m (@mounts) {
				if ($m->[0] eq $p[1] &&
				    $m->[1] eq "LABEL=$label") {
					$p[0] = "LABEL=$label";
					last;
					}
				}
			}
		}
	if (!$_[0] && $self->{'has_volid'} && $p[0] =~ /^\/dev\//) {
		# Check for a UUID on this partition, and if there is one
		# and the filesystem is in fstab with the label, change
		# the device.
		local $out = $self->backquote_command("vol_id $p[0] 2>&1", 1);
		if ($out =~ /ID_FS_UUID=(\S+)/) {
			local $uuid = $1;
			foreach $m (@mounts) {
				if ($m->[0] eq $p[1] &&
				    $m->[1] eq "UUID=$uuid") {
					$p[0] = "UUID=$uuid";
					last;
					}
				}
			}
		}

	# check fstab for a mount on the same dir which is a symlink
	# to the device
	local @st = stat($p[0]);
	foreach $m (@mounts) {
		if ($m->[0] eq $p[1]) {
			local @fst = stat($m->[1]);
			if ($fst[0] == $st[0] && $fst[1] == $st[1]) {
				# symlink to the same place!
				$p[0] = $m->[1];
				last;
				}
			}
		}
	push(@rv, [ $p[1], $p[0], $p[2], $p[3] ]);
	}
close(MTAB);
open(SWAPS, "/proc/swaps");
while(<SWAPS>) {
	chop;
	if (/^(\/\S+)\s+/) {
		local $sf = $1;
		if ($sf =~ /^\/dev\/ide\// || $sf =~ /^\/dev\/mapper\//) {
			# check fstab for a mount on a device which is a symlink
			local @st = stat($sf);
			foreach $m (@mounts) {
				local @fst = stat($m->[1]);
				if ($m->[2] eq 'swap' && $fst[0] == $st[0] &&
				    $fst[1] == $st[1]) {
				    	$sf = $m->[1];
					last;
					}
				}
			}
		if (!$_[0] && $self->{'has_volid'} && $sf =~ /^\/dev\//) {
			# Similarly, fix up UUID= entrys for swap in /etc/fstab
			local $out = $self->backquote_command("vol_id $sf 2>&1", 1);
			if ($out =~ /ID_FS_UUID=(\S+)/) {
				local $uuid = $1;
				foreach $m (@mounts) {
					if ($m->[0] eq "swap" &&
					    $m->[1] eq "UUID=$uuid") {
						$sf = "UUID=$uuid";
						last;
						}
					}
				}
			}
		push(@rv, [ "swap", $sf, "swap", "-" ]);
		}
	}
close(SWAPS);
@{$self->{'list_mounted_cache_array'}} = @rv;
$self->{'list_mounted_cache_mode'} = $_[0];
return @rv;
}

# mount_dir(directory, device, type, options)
# Mount a new directory from some device, with some options. Returns 0 if ok,
# or an error string if failed
sub mount_dir
{
	my $self = shift;

local($out, $opts, $shar, %options, %smbopts);
local @opts = $_[3] eq "-" || $_[3] eq "" ? ( ) :
		grep { $_ ne "noauto" } split(/,/, $_[3]);
if ($_[2] eq "bind") {
	push(@opts, "bind");
	}
$opts = @opts ? "-o ".quotemeta(join(",", @opts)) : "";
$self->parse_options($_[2], $_[3]);
if ($_[2] eq "swap") {
	# Use swapon to add the swap space..
	local $priarg = $options{'pri'} ne "" ? "-p $options{'pri'}" : "";
	$out = $self->backquote_logged("swapon $priarg $_[1] 2>&1");
	if ($out =~ /Invalid argument/) {
		# looks like this swap partition isn't ready yet.. set it up
		$out = $self->backquote_logged("mkswap $_[1] 2>&1");
		if ($?) { return "mkswap failed : <pre>$out</pre>"; }
		$out = $self->backquote_logged("swapon $_[1] 2>&1");
		}
	if ($?) { return "<pre>$out</pre>"; }
	}
elsif ($_[2] eq "auto") {
	# Old automounter filesystem
	$out = $self->backquote_logged("amd $_[0] $_[1] >/dev/null 2>/dev/null");
	if ($?) { return $self->{'text_hash'}->{'linux_eamd'}; }
	}
elsif ($_[2] eq "autofs") {
	# New automounter filesystem
	$opts = $self->autofs_args($_[3]);
	$self->{'type'} = $_[1] !~ /^\// ? "yp" :
		(-x $_[1]) ? "program" : "file";
	$out = $self->backquote_logged("automount $opts $_[0] $self->{'type'} $_[1] 2>&1");
	if ($?) { return $self->text('linux_eauto', "<pre>$out</pre>"); }
	}
elsif ($_[2] eq $self->{'smbfs_fs'} || $_[2] eq "cifs") {
	local $shar = $_[1];
	$shar =~ s/\\/\//g if ($shar =~ /^\\/);
	local $support = $_[2] eq $self->{'smbfs_fs'} ? $self->{'smbfs_support'} : $self->{'cifs_support'};
	return uc($_[2])." not supported" if (!$support);
	local $qshar = quotemeta($shar);
	if ($support >= 3) {
		# SMB filesystem mounted with mount command
		local $temp = $self->transname();
		local $ex = $self->system_logged("mount -t $_[2] $opts $qshar $_[0] >$temp 2>&1 </dev/null");
		local $out = `cat $temp`;
		unlink($temp);
		if ($ex || $out =~ /failed|error/i) {
			$self->system_logged("umount $_[0] >/dev/null 2>&1");
			return "<pre>$out</pre>";
			}
		}
	elsif ($support == 2) {
		# SMB filesystem mounted with version 2.x smbmount
		$opts =
		    ($options{'user'} ? "-U $options{'user'} " : "").
		    ($options{'passwd'} ? "" : "-N ").
		    ($options{'workgroup'} ? "-W $options{'workgroup'} " : "").
		    ($options{'clientname'} ? "-n $options{'clientname'} " : "").
		    ($options{'machinename'} ? "-I $options{'machinename'} " : "");
		$self->foreign_require("proc", "proc-lib.pl");
		local ($fh, $fpid) = $self->foreign_call(
			"proc", "pty_process_exec_logged",
			"sh -c 'smbmount $shar $_[0] -d 0 $opts'");
		if ($options{'passwd'}) {
			local $w = $self->wait_for($fh, "word:");
			if ($w < 0) {
				$self->system_logged("umount $_[0] >/dev/null 2>&1");
				return $self->{'text_hash'}->{'linux_esmbconn'};
				}
			local $p = "$options{'passwd'}\n";
			syswrite($fh, $p, length($p));
			}
		local $got;
		while(<$fh>) {
			$got .= $_;
			}
		if ($got =~ /failed/) {
			$self->system_logged("umount $_[0] >/dev/null 2>&1");
			return "<pre>$got</pre>\n";
			}
		close($fh);
		}
	elsif ($support == 1) {
		# SMB filesystem mounted with older smbmount
		$self->{'shortname'} = $self->get_system_hostname();
		if ($self->{'shortname'} =~ /^([^\.]+)\.(.+)$/) { $self->{'shortname'} = $1; }
		$opts =
		   ($options{servername} ? "-s $options{servername} " : "").
		   ($options{clientname} ? "-c $options{clientname} "
					 : "-c $self->{'shortname'} ").
		   ($options{machinename} ? "-I $options{machinename} " : "").
		   ($options{user} ? "-U $options{user} " : "").
		   ($options{passwd} ? "-P $options{passwd} " : "-n ").
		   ($options{uid} ? "-u $options{uid} " : "").
		   ($options{gid} ? "-g $options{gid} " : "").
		   ($options{fmode} ? "-f $options{fmode} " : "").
		   ($options{dmode} ? "-d $options{dmode} " : "");
		$out = $self->backquote_logged("smbmount $shar $_[0] $opts 2>&1 </dev/null");
		if ($out) {
			$self->system_logged("umount $_[0] >/dev/null 2>&1");
			return "<pre>$out</pre>";
			}
		}
	$self->read_smbopts();
	$smbopts{$_[0]} = $_[3] eq "-" ? "dummy=1" : $_[3];
	$self->write_smbopts();
	}
else {
	# some filesystem supported by mount
	local $fs = $_[2] eq "*" ? "auto" : $_[2];
	if ($_[1] =~ /LABEL=(.*)/) {
		$self->{'cmd'} = "mount -t $fs -L ".quotemeta($1)." $opts ".quotemeta($_[0]);
		}
	elsif ($_[1] =~ /UUID=(\S+)/) {
		$self->{'cmd'} = "mount -t $fs -U ".quotemeta($1)." $opts ".quotemeta($_[0]);
		}
	else {
		$self->{'cmd'} = "mount -t $fs $opts ".quotemeta($_[1])." ".quotemeta($_[0]);
		}
	$out = $self->backquote_logged("$self->{'cmd'} 2>&1 </dev/null");
	if ($?) { return "<pre>$out</pre>"; }
	}
undef(@{$self->{'list_mounted_cache_array'}});
return 0;
}

# os_remount_dir(directory, device, type, options)
# Adjusts the options for some mounted filesystem, by re-mounting
sub os_remount_dir
{
	my $self = shift;

if ($_[2] eq "swap" || $_[2] eq "auto" || $_[2] eq "autofs" ||
    $_[2] eq $self->{'smbfs_fs'} || $_[2] eq "cifs") {
	# Cannot use remount
	local $err = $self->unmount_dir(@_);
	return $err if ($err);
	return $self->mount_dir(@_);
	}
else {
	# Attempt to use remount
	local @opts = $_[3] eq "-" || $_[3] eq "" ? ( ) :
			grep { $_ ne "noauto" } split(/,/, $_[3]);
	push(@opts, "remount");
	local $opts = @opts ? "-o ".quotemeta(join(",", @opts)) : "";
	local $fs = $_[2] eq "*" ? "auto" : $_[2];
	if ($_[1] =~ /LABEL=(.*)/) {
		$self->{'cmd'} = "mount -t $fs -L $1 $opts $_[0]";
		}
	elsif ($_[1] =~ /UUID=(\S+)/) {
		$self->{'cmd'} = "mount -t $fs -U $1 $opts $_[0]";
		}
	else {
		$self->{'cmd'} = "mount -t $fs $opts $_[1] $_[0]";
		}
	$self->{'out'} = $self->backquote_logged("$self->{'cmd'} 2>&1 </dev/null");
	if ($?) { return "<pre>$self->{'out'}</pre>"; }
	undef(@{$self->{'list_mounted_cache_array'}});
	return undef;
	}
}

# unmount_dir(directory, device, type, options, [force])
# Unmount a directory that is currently mounted. Returns 0 if ok,
# or an error string if failed
sub unmount_dir
{
	my $self = shift;

local($out, %smbopts, $dir);
if ($_[2] eq "swap") {
	# Use swapoff to remove the swap space..
	$out = $self->backquote_logged("swapoff $_[1]");
	if ($?) { return "<pre>$out</pre>"; }
	}
elsif ($_[2] eq "auto") {
	# Kill the amd process
	$dir = $_[0];
	if ($self->backquote_command("cat /etc/mtab") =~ /:\(pid([0-9]+)\)\s+$dir\s+(auto|nfs)\s+/) {
		$self->kill_logged('TERM', $1) || return $self->{'text_hash'}->{'linux_ekillamd'};
		}
	sleep(2);
	}
elsif ($_[2] eq "autofs") {
	# Kill the automount process
	$dir = $_[0];
	$self->backquote_command("cat /etc/mtab") =~ /automount\(pid([0-9]+)\)\s+$dir\s+autofs\s+/;
	$self->kill_logged('TERM', $1) || return $self->{'text_hash'}->{'linux_ekillauto'};
	sleep(2);
	}
else {
	local $fflag = $_[4] ? "-l" : "";
	$out = $self->backquote_logged("umount $fflag $_[0] 2>&1");
	if ($?) { return "<pre>$out</pre>"; }
	if ($_[2] eq $self->{'smbfs_fs'} || $_[2] eq "cifs") {
		# remove options from list
		$self->read_smbopts();
		delete($smbopts{$_[0]});
		$self->write_smbopts();
		$self->execute_command("rmmod smbfs");
		$self->execute_command("rmmod cifs");
		}
	}
undef(@{$self->{'list_mounted_cache_array'}});
return 0;
}

# can_force_unmount_dir(directory, device, type)
# Returns 1 if some directory can be forcibly un-mounted
sub can_force_unmount_dir
{
	my $self = shift;

if ($_[2] ne "swap" && $_[2] ne "auto" && $_[2] ne "autofs") {
	# All filesystems using the normal 'mount' command can be
	return 1;
	}
else {
	return 0;
	}
}

# mount_modes(type)
# Given a filesystem type, returns 4 numbers that determine how the file
# system can be mounted, and whether it can be fsck'd
#  0 - cannot be permanently recorded
#	(smbfs under linux before 2.2)
#  1 - can be permanently recorded, and is always mounted at boot
#	(swap under linux)
#  2 - can be permanently recorded, and may or may not be mounted at boot
#	(most normal filesystems)
# The second is:
#  0 - mount is always permanent => mounted when saved
#	(swap under linux before 2.2, or any filesystem that is always
#	 mounted at boot by some script)
#  1 - doesn't have to be permanent
#	(normal fs types)
# The third is:
#  0 - cannot be fsck'd at boot time
#  1 - can be be fsck'd at boot
# The fourth is:
#  0 - can be unmounted
#  1 - cannot be unmounted
# The (optional) fourth is:
#  0 - can be edited
#  1 - cannot be edited (because is always mounted at boot by some script)
sub mount_modes
{
	my $self = shift;

if ($_[0] eq "swap")
	{ return (1, $self->{'swaps_support'} ? 1 : 0, 0, 0); }
elsif ($_[0] eq "auto" || $_[0] eq "autofs")
	{ return (1, 1, 0, 0); }
elsif ($_[0] eq $self->{'smbfs_fs'})
	{ return ($self->{'smbfs_support'} >= 3 ? 2 : 0, 1, 0, 0); }
elsif ($_[0] eq "cifs") { return (2, 1, 0, 0); }
elsif ($_[0] eq "ext2" || $_[0] eq "ext3" || $_[0] eq "minix" ||
       $_[0] eq "xiafs" || $_[0] eq "xfs")
	{ return (2, 1, 1, 0); }
else
	{ return (2, 1, 0, 0); }
}

# disk_space(type, directory)
# Returns the amount of total and free space for some filesystem, or an
# empty array if not appropriate.
sub disk_space
{
	my $self = shift;

if ($self->get_mounted($_[1], "*") < 0) { return (); }
if ($_[0] eq "proc" || $_[0] eq "swap" ||
    $_[0] eq "auto" || $_[0] eq "autofs") { return (); }
local $out = $self->backquote_command("LC_ALL='' LANG='' df -k $_[1]", 1);
if ($out =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/) {
	return ($1, $2);
	}
return ( );
}

# inode_space(type, directory)
# Returns the total and free number of inodes for some filesystem.
sub inode_space
{
	my $self = shift;

if ($self->get_mounted($_[1], "*") < 0) { return (); }
if ($self->backquote_command("LC_ALL='' LANG='' df -i $_[1]", 1) =~
    /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/) {
	return ($1, $2);
	}
return ( );
}

# list_fstypes()
# Returns an array of all the supported filesystem types. If a filesystem is
# found that is not one of the supported types, generate_location() and
# generate_options() will not be called for it.
sub list_fstypes
{
	my $self = shift;

local @sup = ("ext2", "minix", "msdos", "nfs", "nfs4", "iso9660", "ext",
	      "xiafs", "hpfs", "fat", "vfat", "umsdos", "sysv", "reiserfs",
	      "ntfs", "hfs", "fatx");
push(@sup, $self->{'smbfs_fs'}) if ($self->{'smbfs_support'});
push(@sup, "cifs") if ($self->{'cifs_support'});
push(@sup, "auto") if ($self->{'amd_support'});
push(@sup, "autofs") if ($self->{'autofs_support'});
push(@sup, "tmpfs") if ($self->{'tmpfs_support'});
push(@sup, "ext3") if ($self->{'ext3_support'});
push(@sup, "xfs") if ($self->{'xfs_support'});
push(@sup, "jfs") if ($self->{'jfs_support'});
push(@sup, "bind") if ($self->{'bind_support'});
push(@sup, "swap");
return @sup;
}

# fstype_name(type)
# Given a short filesystem type, return a human-readable name for it
sub fstype_name
{
	my $self = shift;

local(%fsmap);
%fsmap = ("ext2","Linux Native Filesystem",
	  "ext3","New Linux Native Filesystem",
	  "minix","Minix Filesystem",
	  "msdos","MS-DOS Filesystem",
	  "nfs","Network Filesystem",
	  "nfs4","Network Filesystem v4",
	  $self->{'smbfs_fs'},"Windows Networking Filesystem",
	  "cifs","Common Internet Filesystem",
	  "iso9660","ISO9660 CD-ROM",
	  "ext","Old EXT Linux Filesystem",
	  "xiafs","Old XIAFS Linux Filesystem",
	  "hpfs","OS/2 Filesystem",
	  "fat","MS-DOS Filesystem",
	  "vfat","Windows Filesystem",
	  "umsdos","Linux on top of MS-DOS Filesystem",
	  "sysv","System V Filesystem",
	  "swap","Virtual Memory",
	  "proc","Kernel Filesystem",
	  "devpts","PTS Filesystem",
	  "auto",($self->{'autofs_support'} ? "Old " : "")."Automounter Filesystem",
	  "reiserfs","Reiser Filesystem",
	  "autofs","New Automounter Filesystem",
	  "usbdevfs","USB Devices",
	  "shm","SysV Shared Memory",
	  "tmpfs","RAM Disk",
	  "xfs","SGI Filesystem",
	  "jfs","IBM Journalling Filesystem",
	  "ntfs","Windows NT Filesystem",
	  "bind","Loopback Filesystem",
	  "hfs","Apple Filesystem",
	  "fatx","XBOX Filesystem",
	  );
return $self->{'config_hash'}->{long_fstypes} && $fsmap{$_[0]} ? $fsmap{$_[0]} : uc($_[0]);
}

# multiple_mount(type)
# Returns 1 if filesystems of this type can be mounted multiple times, 0 if not
sub multiple_mount
{
	my $self = shift;

return ($_[0] eq "nfs" || $_[0] eq "nfs4" || $_[0] eq "auto" || $_[0] eq "autofs" ||
	$_[0] eq "bind");
}

# generate_location(type, location)
# Output HTML for editing the mount location of some filesystem.
sub generate_location
{
	my $self = shift;

if (($_[0] eq "nfs") || ($_[0] eq "nfs4")) {
	# NFS mount from some host and directory
	$_[1] =~ /^([^:]+):(.*)$/;
	print "<tr> <td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_nfshost'}</b>", "nfshost"), "</td>\n";
	print "<td><input name=nfs_host size=20 value=\"$1\">\n";
	$self->nfs_server_chooser_button("nfs_host");
	print "&nbsp;", $self->hlink("<b>$self->{'text_hash'}->{'linux_nfsdir'}</b>", "nfsdir"), "\n";
	printf "<input name=nfs_dir size=20 value=%s>\n",
	       ($_[0] eq "nfs4") && ($2 eq "") ? "/" : $2;
	$self->nfs_export_chooser_button("nfs_host", "nfs_dir");
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "auto") {
	# Using some automounter map
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_map'}</b></td>\n";
	print "<td><input name=auto_map size=20 value=\"$_[1]\">\n";
	print $self->file_chooser_button("auto_map", 0);
	print "</td> <td colspan=2></td> </tr>\n";
	}
elsif ($_[0] eq "autofs") {
	# Using some kernel automounter map
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_map'}</b></td>\n";
	print "<td><input name=autofs_map size=20 value=\"$_[1]\">\n";
	print $self->file_chooser_button("autofs_map", 0);
	print "</td> <td colspan=2></td> </tr>\n";
	}
elsif ($_[0] eq "swap") {
	# Swap file or device
	$self->foreign_require("fdisk", "fdisk-lib.pl");
	printf "<tr> <td valign=top><b>$self->{'text_hash'}->{'linux_swapfile'}</b></td>\n";
	print "<td colspan=3>\n";
	local ($found, $ufound);

	# Show partitions input
	local $sel = $self->foreign_call("fdisk", "partition_select", "lnx_disk",
				   $_[1], 3, \$found);
	printf "<input type=radio name=lnx_dev value=0 %s> %s %s<br>\n",
		$found ? "checked" : "", $self->{'text_hash'}->{'linux_disk'}, $sel;

	# Show UUID input
	if ($self->{'has_volid'}) {
		local $u = $_[1] =~ /UUID=(\S+)/ ? $1 : undef;
		local $usel = $main::fdisk_object->volid_select("lnx_volid", $u, \$ufound);
		if ($usel) {
			printf "<input type=radio name=lnx_dev value=5 %s> %s %s<br>\n", $ufound ? "checked" : "", $self->{'text_hash'}->{'linux_usel'}, $usel;
			}
		}

	# Show other file input
	printf "<input type=radio name=lnx_dev value=1 %s> %s\n",
		$found || $ufound ? "" : "checked", $self->{'text_hash'}->{'linux_swapfile'};
	printf "<input name=lnx_other size=35 value='%s'> %s<br>\n",
		$found || $ufound ? "" : $_[1],
		$self->file_chooser_button("lnx_other");
	print "</td> </tr>\n";
	}
elsif ($_[0] eq $self->{'smbfs_fs'} || $_[0] eq "cifs") {
	# Windows filesystem
	$_[1] =~ /^\\\\(.*)\\(.*)$/;
	local ($server, $share) = ($1, $2);
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_smbserver'}</b></td>\n";
	print "<td><input name=smbfs_server value=\"$server\" size=20>\n";
	$self->smb_server_chooser_button("smbfs_server");
	print "</td>\n";
	print "<td><b>$self->{'text_hash'}->{'linux_smbshare'}</b></td>\n";
	print "<td><input name=smbfs_share value=\"$share\" size=20>\n";
	$self->smb_share_chooser_button("smbfs_server", "smbfs_share");
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "tmpfs") {
	# RAM disk (no location needed)
	}
elsif ($_[0] eq "bind") {
	# Loopback filesystem, mounted from some other directory
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_bind'}</b></td> <td colspan=3>\n";
	printf "<input name=bind_dir value='%s' size=35> %s</td> </tr>\n",
		$_[1], $self->file_chooser_button("bind_dir", 1);
	}
else {
	# This is some linux disk-based filesystem
	$self->foreign_require("fdisk", "fdisk-lib.pl");
	printf "<tr> <td valign=top><b>%s</b></td>\n", $self->fstype_name($_[0]);
	print "<td colspan=3>\n";
	local ($found, $rfound, $lfound, $vfound, $ufound, $rsel, $c);

	# Show regular partition input
	local $sel = $self->foreign_call("fdisk", "partition_select", "lnx_disk",
				   $_[1], 0, \$found);
	printf "<input type=radio name=lnx_dev value=0 %s> %s %s<br>\n",
		$found ? "checked" : "", $self->{'text_hash'}->{'linux_disk'}, $sel;

	# Show RAID input
	if ($self->foreign_check("raid")) {
		$self->foreign_require("raid", "raid-lib.pl");
		local $conf = $self->foreign_call("raid", "get_raidtab");
		foreach $c (@$conf) {
			if ($c->{'active'}) {
				$rsel .= sprintf "<option value=%s %s>%s\n",
					$c->{'value'},
					$_[1] eq $c->{'value'} ? 'selected' : '',
					$self->text('linux_rdev', substr($c->{'value'}, -1));
				$rfound++ if ($_[1] eq $c->{'value'});
				}
			}
		if ($rsel) {
			printf "<input type=radio name=lnx_dev value=2 %s> %s\n",
				$rfound ? "checked" : " ", $self->{'text_hash'}->{'linux_raid'};
			print "<select name=lnx_raid>\n",$rsel,"</select><br>\n";
			}
		}

	# Show LVM input
	if ($self->foreign_check("lvm")) {
		$self->foreign_require("lvm", "lvm-lib.pl");
		local @vgs = $self->foreign_call("lvm", "list_volume_groups");
		local @lvs;
		foreach $v (@vgs) {
			push(@lvs, $self->foreign_call("lvm",
					"list_logical_volumes", $v->{'name'}));
			}
		if (@lvs) {
			local $lsel;
			foreach $l (@lvs) {
				local $sf = $self->same_file($_[1], $l->{'device'});
				$lsel .= sprintf "<option value=%s %s>%s\n",
					$l->{'device'},
					$sf ? 'selected' : '',
					$self->text('linux_ldev', $l->{'vg'},
							    $l->{'name'});
				$vfound++ if ($sf);
				}
			printf "<input type=radio name=lnx_dev value=4 %s> %s\n",
				$vfound ? "checked" : " ", $self->{'text_hash'}->{'linux_lvm'};
			print "<select name=lnx_lvm>\n",$lsel,"</select><br>\n";
			}
		}

	# Show label input
	if ($self->{'has_e2label'} || $self->{'has_xfs_db'}) {
		local $l = $_[1] =~ /LABEL=(.*)/ ? $1 : undef;
		local $esel = $main::fdisk_object->label_select("lnx_label", $l, \$lfound);
		if ($esel) {
			printf "<input type=radio name=lnx_dev value=3 %s> %s %s<br>\n", $lfound ? "checked" : "", $self->{'text_hash'}->{'linux_lsel'}, $esel;
			}
		}

	# Show UUID input
	if ($self->{'has_volid'}) {
		local $u = $_[1] =~ /UUID=(\S+)/ ? $1 : undef;
		local $usel = $main::fdisk_object->volid_select("lnx_uuid", $u, \$ufound);
		if ($usel) {
			printf "<input type=radio name=lnx_dev value=5 %s> %s %s<br>\n", $ufound ? "checked" : "", $self->{'text_hash'}->{'linux_usel'}, $usel;
			}
		}

	# Show other device input
	local $anyfound = $found || $rfound || $lfound || $vfound || $ufound;
	printf "<input type=radio name=lnx_dev value=1 %s> %s\n",
		$anyfound ? "" : "checked",
		$self->{'text_hash'}->{'linux_other'};
	printf "<input name=lnx_other size=35 value='%s'> %s<br>\n",
		$anyfound ? "" : $_[1],
		$self->file_chooser_button("lnx_other");
	print "</td> </tr>\n";
	}
}

# generate_options(type, newmount)
# Output HTML for editing mount options for a particular filesystem 
# under this OS
sub generate_options
{
	my $self = shift;

if ($_[0] ne "swap" && $_[0] ne "auto" &&
    $_[0] ne "autofs" && $_[0] ne $self->{'smbfs_fs'} && $_[0] ne "cifs") {
	# Lots of options are common to all linux filesystems
        print "<tr $self->{'tb'}> <td colspan=4><b>$self->{'text_hash'}->{'edit_comm_opt'}</b></td> </tr>\n";
	print "<tr> <td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_ro'}</b>", "linux_ro"),"</td>\n";
	printf "<td nowrap><input type=radio name=lnx_ro value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
	printf "<input type=radio name=lnx_ro value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";

	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_sync'}</b>", "linux_sync"), "</td>\n";
	printf"<td nowrap><input type=radio name=lnx_sync value=0 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"sync"}) ? "" : "checked";
	printf "<input type=radio name=lnx_sync value=1 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"sync"}) ? "checked" : "";

	local $nodev = defined($self->{'options_hash'}->{"nodev"}) ||
		       defined($self->{'options_hash'}->{"user"}) && !defined($self->{'options_hash'}->{"dev"});
	print "<tr> <td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_nodev'}</b>", "linux_nodev"), "</td>\n";
	printf "<td nowrap><input type=radio name=lnx_nodev value=0 %s> $self->{'text_hash'}->{'yes'}\n",
		$nodev ? "" : "checked";
	printf "<input type=radio name=lnx_nodev value=1 %s> $self->{'text_hash'}->{'no'}</td>\n",
		$nodev ? "checked" : "";

	local $noexec = defined($self->{'options_hash'}->{"noexec"}) ||
		       defined($self->{'options_hash'}->{"user"}) && !defined($self->{'options_hash'}->{"exec"});
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_noexec'}</b>", "linux_noexec"), "</td>\n";
	printf"<td nowrap><input type=radio name=lnx_noexec value=0 %s> $self->{'text_hash'}->{'yes'}\n",
		$noexec ? "" : "checked";
	printf "<input type=radio name=lnx_noexec value=1 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		$noexec ? "checked" : "";

	local $nosuid = defined($self->{'options_hash'}->{"nosuid"}) ||
		       defined($self->{'options_hash'}->{"user"}) && !defined($self->{'options_hash'}->{"suid"});
	print "<tr> <td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_nosuid'}</b>", "linux_nosuid"), "</td>\n";
	printf "<td nowrap><input type=radio name=lnx_nosuid value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		$nosuid ? "checked" : "";
	printf "<input type=radio name=lnx_nosuid value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		$nosuid ? "" : "checked";

	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_user'}</b>", "linux_user"), "</td>\n";
	printf"<td nowrap><input type=radio name=lnx_user value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"user"}) ? "checked" : "";
	printf "<input type=radio name=lnx_user value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"user"}) ? "" : "checked";
	}
	
if ($_[0] eq "ext2" || $_[0] eq "ext3") {
	# Ext2 has lots more options..
        print "<tr $self->{'tb'}> <td colspan=4><b>$self->{'text_hash'}->{'edit_ext_opt'}</b></td> </tr>\n";
	if ($self->{'no_mount_check'}) {
		print "<tr> <td><b>$self->{'text_hash'}->{'linux_df'}</b></td>\n";
		printf "<td><input type=radio name=ext2_df value=1 %s> %s\n",
		    defined($self->{'options_hash'}->{"minixdf"}) ? "checked" : "", $self->{'text_hash'}->{'yes'};
		printf "<input type=radio name=ext2_df value=0 %s> %s</td>\n",
		    defined($self->{'options_hash'}->{"minixdf"}) ? "" : "checked", $self->{'text_hash'}->{'no'};
		}
	else {
		print "<tr> <td><b>$self->{'text_hash'}->{'linux_check'}</b></td>\n";
		print "<td><select name=ext2_check>\n";
		printf "<option value=normal %s> $self->{'text_hash'}->{'linux_normal'}\n",
			$self->{'options_hash'}->{"check"} eq "" ||
			$self->{'options_hash'}->{"check"} eq "normal" ?  "selected" : "";
		printf "<option value=strict %s> $self->{'text_hash'}->{'linux_strict'}\n",
			$self->{'options_hash'}->{"check"} eq "strict" ? "selected" : "";
		printf "<option value=none %s> $self->{'text_hash'}->{'linux_none'}\n",
			$self->{'options_hash'}->{"check"} eq "none" ||
			defined($self->{'options_hash'}->{"nocheck"}) ?  "selected" : "";
		print "</select></td>\n";
		}

	print "<td><b>$self->{'text_hash'}->{'linux_errors'}</b></td>\n";
	print "<td><select name=ext2_errors>\n";
	printf "<option value=default %s> $self->{'text_hash'}->{'default'}\n",
		!defined($self->{'options_hash'}->{"errors"}) ? "selected" : "";
	printf "<option value=continue %s> $self->{'text_hash'}->{'linux_continue'}\n",
		$self->{'options_hash'}->{"error"} eq "continue" ? "selected" : "";
	printf "<option value=remount-ro %s> $self->{'text_hash'}->{'linux_remount_ro'}\n",
		$self->{'options_hash'}->{"error"} eq "remount-ro" ? "selected" : "";
	printf "<option value=panic %s> $self->{'text_hash'}->{'linux_panic'}\n",
		$self->{'options_hash'}->{"error"} eq "panic" ? "selected" : "";
	print "</select></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'linux_grpid'}</b></td>\n";
	printf "<td nowrap><input type=radio name=ext2_grpid value=1 %s> %s\n",
		defined($self->{'options_hash'}->{"grpid"}) || defined($self->{'options_hash'}->{"bsdgroups"}) ?
			"checked" : "", $self->{'text_hash'}->{'yes'};
	printf "<input type=radio name=ext2_grpid value=0 %s> %s</td>\n",
		defined($self->{'options_hash'}->{"grpid"}) || defined($self->{'options_hash'}->{"bsdgroups"}) ?
			"" : "checked", $self->{'text_hash'}->{'no'};

	print "<td><b>$self->{'text_hash'}->{'linux_quotas'}</b></td>\n";
	local $usrquota = defined($self->{'options_hash'}->{"usrquota"});
	local $grpquota = defined($self->{'options_hash'}->{"grpquota"});
	print "<td nowrap><select name=ext2_quota>\n";
	printf "<option value=0 %s> $self->{'text_hash'}->{'no'}\n",
		$usrquota || $grpquota ? "" : "selected";
	printf "<option value=1 %s> $self->{'text_hash'}->{'linux_usrquota'}\n",
		$usrquota && !$grpquota ? "selected" : "";
	printf "<option value=2 %s> $self->{'text_hash'}->{'linux_grpquota'}\n",
		$grpquota && !$usrquota ? "selected" : "";
	if (`uname -r` =~ /^2\.0\.(\d+)/ && $1 < 31) {
		printf "<option value=3 %s> $self->{'text_hash'}->{'linux_usrgrpquota2'}\n",
			$usrquota && $grpquota ? "selected" : "";
		}
	else {
		printf "<option value=3 %s> $self->{'text_hash'}->{'linux_usrgrpquota'}\n",
			$usrquota && $grpquota ? "selected" : "";
		}
	print "</select></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'linux_resuid'}</b></td>\n";
	printf "<td><input name=ext2_resuid size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{"resuid"}) ? getpwuid($self->{'options_hash'}->{"resuid"}) : "";
	print $self->user_chooser_button("ext2_resuid", 0),"</td>\n";

	print "<td><b>$self->{'text_hash'}->{'linux_resgid'}</b></td>\n";
	printf "<td><input name=ext2_resgid size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{"resgid"}) ? getgrgid($self->{'options_hash'}->{"resgid"}) : "";
	print $self->group_chooser_button("ext2_resgid", 0),"</td> </tr>\n";
	}
elsif (($_[0] eq "nfs") || ($_[0] eq "nfs4")) {
	# Linux nfs has some more options...
        print "<tr $self->{'tb'}> <td colspan=4><b>$self->{'text_hash'}->{'edit_nfs_opt'}</b></td> </tr>\n";
	
	print "<tr> <td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_vers'}</b>", "linux_vers"), "</td>\n";
	if ($_[0] eq "nfs4") {
	    print "<td nowrap><input type=radio name=nfs_vers value=4 checked> 4</td>\n";
	} else {
	    print "<td nowrap><input type=radio name=nfs_vers value=3 checked> $self->{'text_hash'}->{'edit_3_or_lower'}</td>\n";
	}
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_port'}</b>", "linux_port"), "</td>\n";
	printf "<td nowrap><input type=radio name=nfs_port_def value=1 %s> $self->{'text_hash'}->{'default'}\n",
		defined($self->{'options_hash'}->{"port"}) ? "" : "checked";
	printf "<input type=radio name=nfs_port_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"port"}) ? "checked" : "";
	print "<input size=5 name=nfs_port value=$self->{'options_hash'}->{port}></td> </tr>\n";

	print "<tr> <td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_bg'}</b>", "linux_bg"), "</td>\n";
	printf "<td nowrap><input type=radio name=nfs_bg value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"bg"}) ? "checked" : "";
	printf "<input type=radio name=nfs_bg value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"bg"}) ? "" : "checked";

	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_soft'}</b>", "linux_soft"), "</td>\n";
	printf "<td nowrap><input type=radio name=nfs_soft value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"soft"}) ? "checked" : "";
	printf "<input type=radio name=nfs_soft value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"soft"}) ? "" : "checked";

	print "<tr> <td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_timeo'}</b>", "linux_timeo"), "</td>\n";
	printf "<td nowrap><input type=radio name=nfs_timeo_def value=1 %s> $self->{'text_hash'}->{'default'}\n",
		defined($self->{'options_hash'}->{"timeo"}) ? "" : "checked";
	printf "<input type=radio name=nfs_timeo_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"timeo"}) ? "checked" : "";
	printf "<input size=5 name=nfs_timeo value=$self->{'options_hash'}->{timeo}></td>\n";

	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_retrans'}</b>", "linux_retrans"), "</td>\n";
	printf "<td nowrap><input type=radio name=nfs_retrans_def value=1 %s> $self->{'text_hash'}->{'default'}\n",
		defined($self->{'options_hash'}->{"retrans"}) ? "" : "checked";
	printf "<input type=radio name=nfs_retrans_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"retrans"}) ? "checked" : "";
	print "<input size=5 name=nfs_retrans value=$self->{'options_hash'}->{retrans}></td> </tr>\n";

	print "<tr> <td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_intr'}</b>", "linux_intr"), "</td>\n";
	printf "<td nowrap><input type=radio name=nfs_intr value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"intr"}) ? "checked" : "";
	printf "<input type=radio name=nfs_intr value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"intr"}) ? "" : "checked";

	local $proto = defined($self->{'options_hash'}->{"udp"}) ? "udp" :
		       defined($self->{'options_hash'}->{"tcp"}) ? "tcp" :
		       $_[0] eq "nfs" ? "udp" : "tcp";
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_transfert'}</b>", "linux_transfert"), "</td>\n";
	print "<td nowrap><select name=nfs_transfert>\n";
	printf "<option value=tcp %s>TCP</option>\n",
		$proto eq "tcp" ? "selected" : "";
	printf "<option value=udp %s %s>UDP</option>\n",
	        $proto eq "udp" ? "selected" : "", ($_[0] eq "nfs4") ? "disabled" : "";
	print "</select></td> </tr>\n";

	print "<tr> <td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_rsize'}</b>", "linux_rsize"), "</td>\n";
	printf "<td nowrap><input type=radio name=nfs_rsize_def value=1 %s> $self->{'text_hash'}->{'default'}\n",
		defined($self->{'options_hash'}->{"rsize"}) ? "" : "checked";
	printf "<input type=radio name=nfs_rsize_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"rsize"}) ? "checked" : "";
	printf "<input size=5 name=nfs_rsize value=$self->{'options_hash'}->{rsize}></td>\n";

	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_wsize'}</b>", "linux_wsize"), "</td>\n";
	printf "<td nowrap><input type=radio name=nfs_wsize_def value=1 %s> $self->{'text_hash'}->{'default'}\n",
		defined($self->{'options_hash'}->{"wsize"}) ? "" : "checked";
	printf "<input type=radio name=nfs_wsize_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"wsize"}) ? "checked" : "";
	print "<input size=5 name=nfs_wsize value=$self->{'options_hash'}->{wsize}></td> </tr>\n";
	print "<tr> <td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_auth'}</b>", "linux_auth"), "</td>\n";
	printf "<td nowrap colspan=2><input type=radio name=nfs_auth value=0 %s onclick=enable_sec(0)> sys\n",
		!defined($self->{'options_hash'}->{"sec"}) ? "checked" : "";
	printf "<input type=radio name=nfs_auth value=1 %s onclick=enable_sec(1)> krb5\n",
		($self->{'options_hash'}->{"sec"} =~ /krb5/) ? "checked" : "";
	printf "<input type=radio name=nfs_auth value=2 %s disabled> lipkey\n",
		($self->{'options_hash'}->{"sec"} =~ /lipkey/) ? "checked" : "";
	printf "<input type=radio name=nfs_auth value=3 %s disabled> spkm-3</td></tr>\n",
		($self->{'options_hash'}->{"sec"} =~ /spkm/) ? "checked" : "";

	print "<tr> <td>", $self->hlink("<b>$self->{'text_hash'}->{'linux_sec'}</b>", "linux_sec"), "</td>\n";
	printf "<td nowrap colspan=2><input type=radio name=nfs_sec value=0 %s> $self->{'text_hash'}->{'config_none'}\n",
		(!defined($self->{'options_hash'}->{"sec"}) || ($self->{'options_hash'}->{"sec"} !~ /[ip]$/)) ? "checked" : "";
	printf "<input type=radio name=nfs_sec value=1 %s> $self->{'text_hash'}->{'linux_integrity'}\n",
		($self->{'options_hash'}->{"sec"} =~ /i$/) ? "checked" : "";
	printf "<input type=radio name=nfs_sec value=2 %s> $self->{'text_hash'}->{'linux_privacy'}</td></tr>\n",
		($self->{'options_hash'}->{"sec"} =~ /p$/) ? "checked" : "";
	
	print "<script type=\"text/javascript\">\n";
	print "function enable_sec(level) {\n";
	print " if (level) {\n";
	print "   document.forms[0].nfs_sec[1].disabled=0;\n";
	print "   document.forms[0].nfs_sec[2].disabled=0;\n";
	print "   } else {\n";
	print "   document.forms[0].nfs_sec[1].disabled=1;\n";
	print "   document.forms[0].nfs_sec[2].disabled=1;\n";
	print "   document.forms[0].nfs_sec[0].checked=1;\n";
	print " } }\n";
	print "window.onload = function() {;\n";
	printf "   enable_sec(%d);\n", !defined($self->{'options_hash'}->{"sec"}) ? 0 : 1;
	print "   }\n";
	print "</script>\n";
	}
elsif ($_[0] eq "fat" || $_[0] eq "vfat" || $_[0] eq "msdos" ||
       $_[0] eq "umsdos" || $_[0] eq "fatx"){
	# All dos-based filesystems share some options
        print "<tr $self->{'tb'}> <td colspan=4><b>$self->{'text_hash'}->{'edit_dos_opt'}</b></td> </tr>\n";
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_uid'}</b></td>\n";
	printf "<td><input name=fat_uid size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{'uid'}) ? getpwuid($self->{'options_hash'}->{'uid'}) : "";
	print $self->user_chooser_button("fat_uid", 0),"</td>\n";

	print "<td><b>$self->{'text_hash'}->{'linux_gid'}</b></td>\n";
	printf "<td><input name=fat_gid size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{'gid'}) ? getgrgid($self->{'options_hash'}->{'gid'}) : "";
	print $self->group_chooser_button("fat_gid", 0),"</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'linux_rules'}</b></td>\n";
	print "<td><select name=fat_check>\n";
	printf "<option value=\"\" %s> $self->{'text_hash'}->{'default'}\n",
		defined($self->{'options_hash'}->{"check"}) ? "" : "selected";
	printf "<option value=r %s> $self->{'text_hash'}->{'linux_relaxed'}\n",
		$self->{'options_hash'}->{"check"} =~ /^r/ ? "selected" : "";
	printf "<option value=n %s> $self->{'text_hash'}->{'linux_normal'}\n",
		$self->{'options_hash'}->{"check"} =~ /^n/ ? "selected" : "";
	printf "<option value=s %s> $self->{'text_hash'}->{'linux_strict'}\n",
		$self->{'options_hash'}->{"check"} =~ /^s/ ? "selected" : "";
	print "</select></td>\n";

	print "<td><b>$self->{'text_hash'}->{'linux_conv'}</b></td>\n";
	print "<td><select name=fat_conv>\n";
	printf "<option value=\"\" %s> $self->{'text_hash'}->{'linux_none'}\n",
		$self->{'options_hash'}->{"conv"} =~ /^b/ || !defined($self->{'options_hash'}->{"conv"}) ?
		"selected" : "";
	printf "<option value=t %s> $self->{'text_hash'}->{'linux_allfiles'}\n",
		$self->{'options_hash'}->{"conv"} =~ /^t/ ? "selected" : "";
	printf "<option value=a %s> $self->{'text_hash'}->{'linux_textfiles'}\n",
		$self->{'options_hash'}->{"conv"} =~ /^a/ ? "selected" : "";
	print "</select></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'linux_umask'}</b></td>\n";
	printf "<td><input type=radio name=fat_umask_def value=1 %s> $self->{'text_hash'}->{'default'}\n",
		defined($self->{'options_hash'}->{"umask"}) ? "" : "checked";
	printf "<input type=radio name=fat_umask_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"umask"}) ? "checked" : "";
	print "<input size=5 name=fat_umask value=\"$self->{'options_hash'}->{umask}\"></td>\n";

	print "<td><b>$self->{'text_hash'}->{'linux_quiet'}</b></td>\n";
	printf "<td nowrap><input type=radio name=fat_quiet value=0 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"quiet"}) ? "" : "checked";
	printf "<input type=radio name=fat_quiet value=1 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"quiet"}) ? "checked" : "";

	if ($_[0] eq "vfat") {
		# vfat has some extra options beyond fat
		print "<tr> <td><b>$self->{'text_hash'}->{'linux_uni_xlate'}</b></td>\n";
		printf "<td><input type=radio name=fat_uni_xlate value=1 %s> $self->{'text_hash'}->{'yes'}\n",
			defined($self->{'options_hash'}->{"uni_xlate"}) ? "checked" : "";
		printf "<input type=radio name=fat_uni_xlate value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
			defined($self->{'options_hash'}->{"uni_xlate"}) ? "" : "checked";

		print "<td><b>$self->{'text_hash'}->{'linux_posix'}</b></td>\n";
		printf "<td nowrap><input type=radio name=fat_posix value=1 %s> $self->{'text_hash'}->{'yes'}\n",
			defined($self->{'options_hash'}->{"posix"}) ? "checked" : "";
		printf "<input type=radio name=fat_posix value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
			defined($self->{'options_hash'}->{"posix"}) ? "" : "checked";
		}
	}
elsif ($_[0] eq "hpfs") {
	# OS/2 filesystems has some more options..
        print "<tr $self->{'tb'}> <td colspan=4><b>$self->{'text_hash'}->{'edit_hpfs_opt'}</b></td> </tr>\n";
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_uid'}</b></td>\n";
	printf "<td><input name=hpfs_uid size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{"uid"}) ? getpwuid($self->{'options_hash'}->{"uid"}) : "";
	print $self->user_chooser_button("hpfs_uid", 0),"</td>\n";

	print "<td><b>$self->{'text_hash'}->{'linux_gid'}</b></td>\n";
	printf "<td><input name=hpfs_gid size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{"gid"}) ? getgrgid($self->{'options_hash'}->{"gid"}) : "";
	print $self->group_chooser_button("hpfs_gid", 0),"</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'linux_umask'}</b></td>\n";
	printf"<td><input type=radio name=hpfs_umask_def value=1 %s> Default\n",
		defined($self->{'options_hash'}->{"umask"}) ? "" : "checked";
	printf "<input type=radio name=hpfs_umask_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"umask"}) ? "checked" : "";
	print "<input size=5 name=hpfs_umask value=\"$self->{'options_hash'}->{umask}\"></td>\n";

	print "<td><b>$self->{'text_hash'}->{'linux_conv2'}</b></td>\n";
	print "<td><select name=hpfs_conv>\n";
	printf "<option value=b %s> $self->{'text_hash'}->{'linux_none'}\n",
		$self->{'options_hash'}->{"conv"} =~ /^b/ || !defined($self->{'options_hash'}->{"conv"}) ?
		"selected" : "";
	printf "<option value=t %s> $self->{'text_hash'}->{'linux_allfiles'}\n",
		$self->{'options_hash'}->{"conv"} =~ /^t/ ? "selected" : "";
	printf "<option value=a %s> $self->{'text_hash'}->{'linux_textfiles'}\n",
		$self->{'options_hash'}->{"conv"} =~ /^a/ ? "selected" : "";
	print "</select></td> </tr>\n";
	}
elsif ($_[0] eq "iso9660") {
	# CD-ROM filesystems have some more options..
        print "<tr $self->{'tb'}> <td colspan=4><b>$self->{'text_hash'}->{'edit_iso9660_opt'}</b></td> </tr>\n";
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_uid'}</b></td>\n";
	printf "<td><input name=iso9660_uid size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{"uid"}) ? getpwuid($self->{'options_hash'}->{"uid"}) : "";
	print $self->user_chooser_button("iso9660_uid", 0),"</td>\n";

	print "<td><b>$self->{'text_hash'}->{'linux_gid'}</b></td>\n";
	printf "<td><input name=iso9660_gid size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{"gid"}) ? getgrgid($self->{'options_hash'}->{"gid"}) : "";
	print $self->group_chooser_button("iso9660_gid", 0),"</td>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'linux_rock'}</b></td>\n";
	printf "<td><input type=radio name=iso9660_norock value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"norock"}) ? "checked" : "";
	printf "<input type=radio name=iso9660_norock value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"norock"}) ? "" : "checked";

	print "<td><b>$self->{'text_hash'}->{'linux_mode'}</b></td>\n";
	printf"<td><input size=10 name=iso9660_mode value=\"%s\"></td> </tr>\n",
		defined($self->{'options_hash'}->{"mode"}) ? $self->{'options_hash'}->{"mode"} : "444";
	}
elsif ($_[0] eq "auto") {
	# Don't know how to set options for auto filesystems yet..
	print "<tr> <td><i>$self->{'text_hash'}->{'linux_noopts'}</i></td> </tr>\n";
	}
elsif ($_[0] eq "autofs") {
        print "<tr $self->{'tb'}> <td colspan=4><b>$self->{'text_hash'}->{'edit_autofs_opt'}</b></td> </tr>\n";
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_timeout'}</b></td> <td>\n";
	printf"<input type=radio name=autofs_timeout_def value=1 %s> $self->{'text_hash'}->{'default'}\n",
		defined($self->{'options_hash'}->{'timeout'}) ? "" : "checked";
	printf "<input type=radio name=autofs_timeout_def value=0 %s>\n",
		defined($self->{'options_hash'}->{'timeout'}) ? "checked" : "";
	printf "<input name=autofs_timeout size=5 value=\"%s\"> $self->{'text_hash'}->{'linux_secs'}</td>\n",
		$self->{'options_hash'}->{'timeout'};

	print "<td><b>$self->{'text_hash'}->{'linux_pid_file'}</b>?</td>\n";
	printf"<td><input type=radio name=autofs_pid-file_def value=1 %s> $self->{'text_hash'}->{'no'}\n",
		defined($self->{'options_hash'}->{'pid-file'}) ? "" : "checked";
	printf "<input type=radio name=autofs_pid-file_def value=0 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{'pid-file'}) ? "checked" : "";
	printf "<input name=autofs_pid-file size=20 value=\"%s\">\n",
		$self->{'options_hash'}->{'pid-file'};
	print $self->file_chooser_button("autofs_pid-file", 1);
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "swap") {
	# Swap has no options..
	print "<tr> <td width=25%><b>$self->{'text_hash'}->{'linux_swappri'}</b></td>\n";
	print "<td width=25%>",$self->ui_opt_textbox("swap_pri", $self->{'options_hash'}->{'pri'}, 6,
				     $self->{'text_hash'}->{'default'}),"</td>\n";

	print "<td width=50% colspan=2></td>\n";
	print "</tr>\n";
	}
elsif ($_[0] eq $self->{'smbfs_fs'} || $_[0] eq "cifs") {
	# SMB filesystems have a few options..
        print "<tr $self->{'tb'}> <td colspan=4><b>$self->{'text_hash'}->{'edit_smbfs_opt'}</b></td> </tr>\n";
	$self->{'support'} = $_[0] eq $self->{'smbfs_fs'} ? $self->{'smbfs_support'} : $self->{'cifs_support'};
	if (keys(%{$self->{'options_hash'}}) == 0 && !$_[1]) {
		print "<tr> <td colspan=4><i>$self->{'text_hash'}->{'linux_smbwarn'}</i></td> </tr>\n";
		}

	print "<tr> <td><b>$self->{'text_hash'}->{'linux_username'}</b></td>\n";
	printf "<td><input name=smbfs_user size=15 value=\"%s\"></td>\n",
		$self->{'support'} == 4 ? $self->{'options_hash'}->{'username'} : $self->{'options_hash'}->{'user'};

	print "<td><b>$self->{'text_hash'}->{'linux_password'}</b></td>\n";
	printf "<td><input type=password name=smbfs_passwd size=15 value=\"%s\"></td> </tr>\n",
		$self->{'support'} == 4 ? $self->{'options_hash'}->{'password'} : $self->{'options_hash'}->{'passwd'};

	if (!$self->{'access_hash'}->{'simopts'}) {
		if ($self->{'support'} != 2) {
			print "<tr> <td><b>$self->{'text_hash'}->{'linux_uid'}</b></td>\n";
			printf "<td><input name=smbfs_uid size=8 value=\"%s\">\n",
				defined($self->{'options_hash'}->{"uid"}) ? getpwuid($self->{'options_hash'}->{"uid"}) : "";
			print $self->user_chooser_button("smbfs_uid", 0),"</td>\n";

			print "<td><b>$self->{'text_hash'}->{'linux_gid'}</b></td>\n";
			printf "<td><input name=smbfs_gid size=8 value=\"%s\">\n",
				defined($self->{'options_hash'}->{"gid"}) ? getgrgid($self->{'options_hash'}->{"gid"}) : "";
			print $self->group_chooser_button("smbfs_gid", 0),"</td>\n";
			}

		if ($self->{'support'} == 1) {
			print "<tr> <td><b>$self->{'text_hash'}->{'linux_sname'}</b></td>\n";
			printf "<td><input type=radio name=smbfs_sname_def value=1 %s> $self->{'text_hash'}->{'linux_auto'}\n",
				defined($self->{'options_hash'}->{"servername"}) ? "" : "checked";
			printf "<input type=radio name=smbfs_sname_def value=0 %s>\n",
				defined($self->{'options_hash'}->{"servername"}) ? "checked" : "";
			print "<input size=10 name=smbfs_sname value=\"$self->{'options_hash'}->{servername}\"></td>\n";
			}
		elsif ($self->{'support'} == 2) {
			print "<tr> <td><b>$self->{'text_hash'}->{'linux_wg'}</b></td>\n";
			printf "<td><input type=radio name=smbfs_wg_def value=1 %s> $self->{'text_hash'}->{'linux_auto'}\n",
				defined($self->{'options_hash'}->{"workgroup"}) ? "" : "checked";
			printf "<input type=radio name=smbfs_wg_def value=0 %s>\n",
				defined($self->{'options_hash'}->{"workgroup"}) ? "checked" : "";
			print "<input size=10 name=smbfs_wg value=\"$self->{'options_hash'}->{'workgroup'}\"></td>\n";
			}

		if ($self->{'support'} < 3) {
			print "<td><b>$self->{'text_hash'}->{'linux_cname'}</b></td>\n";
			printf "<td><input type=radio name=smbfs_cname_def value=1 %s> $self->{'text_hash'}->{'linux_auto'}\n",
				defined($self->{'options_hash'}->{"clientname"}) ? "" : "checked";
			printf "<input type=radio name=smbfs_cname_def value=0 %s>\n",
				defined($self->{'options_hash'}->{"clientname"}) ? "checked" : "";
			print "<input size=10 name=smbfs_cname value=\"$self->{'options_hash'}->{clientname}\"></td> </tr>\n";

			print "<tr> <td><b>$self->{'text_hash'}->{'linux_mname'}</b></td>\n";
			printf "<td colspan=3><input type=radio name=smbfs_mname_def value=1 %s> %s\n",
				defined($self->{'options_hash'}->{"machinename"}) ? "" : "checked", $self->{'text_hash'}->{'linux_auto'};
			printf "<input type=radio name=smbfs_mname_def value=0 %s>\n",
				defined($self->{'options_hash'}->{"machinename"}) ? "checked" : "";
			print "<input size=30 name=smbfs_mname value=\"$self->{'options_hash'}->{machinename}\"></td> </tr>\n";
			}
		
		if ($self->{'support'} == 1) {
			print "<tr> <td><b>$self->{'text_hash'}->{'linux_fmode'}</b></td>\n";
			printf
			    "<td><input name=smbfs_fmode size=5 value=\"%s\"></td>\n",
			    defined($self->{'options_hash'}->{'fmode'}) ? $self->{'options_hash'}->{'fmode'} : "755";

			print "<td><b>$self->{'text_hash'}->{'linux_dmode'}</b></td>\n";
			printf
			    "<td><input name=smbfs_dmode size=5 value=\"%s\"></td>\n",
			    defined($self->{'options_hash'}->{'dmode'}) ? $self->{'options_hash'}->{'dmode'} : "755";
			print "</tr>\n";
			}
		elsif ($self->{'support'} >= 3) {
			print "<tr> <td><b>$self->{'text_hash'}->{'linux_fmode'}</b></td> <td>\n";
			printf"<input type=radio name=smbfs_fmask_def value=1 %s> %s\n",
				defined($self->{'options_hash'}->{'fmask'}) ? "" : "checked",
				$self->{'text_hash'}->{'default'};
			printf"<input type=radio name=smbfs_fmask_def value=0 %s>\n",
				defined($self->{'options_hash'}->{'fmask'}) ? "checked" : "";
			printf "<input name=smbfs_fmask size=5 value='%s'></td>\n",
				$self->{'options_hash'}->{'fmask'};

			print "<td><b>$self->{'text_hash'}->{'linux_dmode'}</b></td> <td>\n";
			printf"<input type=radio name=smbfs_dmask_def value=1 %s> %s\n",
				defined($self->{'options_hash'}->{'dmask'}) ? "" : "checked",
				$self->{'text_hash'}->{'default'};
			printf"<input type=radio name=smbfs_dmask_def value=0 %s>\n",
				defined($self->{'options_hash'}->{'dmask'}) ? "checked" : "";
			printf "<input name=smbfs_dmask size=5 value='%s'></td></tr>\n",
				$self->{'options_hash'}->{'dmask'};

			print "<tr> <td><b>$self->{'text_hash'}->{'linux_ro'}</b></td>\n";
			printf "<td><input type=radio name=smbfs_ro value=1 %s> $self->{'text_hash'}->{'yes'}\n",
				defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
			printf "<input type=radio name=smbfs_ro value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
				defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";

			print "<td><b>$self->{'text_hash'}->{'linux_user'}</b></td>\n";
			printf "<td><input type=radio name=smbfs_user2 value=1 %s> $self->{'text_hash'}->{'yes'}\n",
				defined($self->{'options_hash'}->{"user"}) ? "checked" : "";
			printf "<input type=radio name=smbfs_user2 value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
				defined($self->{'options_hash'}->{"user"}) ? "" : "checked";
			}
		if ($self->{'support'} == 4) {
			print "<tr> <td><b>$self->{'text_hash'}->{'linux_cname'}</b></td>\n";
			printf "<td colspan=3><input type=radio name=smbfs_cname_def value=1 %s> $self->{'text_hash'}->{'linux_auto'}\n",
				defined($self->{'options_hash'}->{"netbiosname"}) ? "" : "checked";
			printf "<input type=radio name=smbfs_cname_def value=0 %s>\n",
				defined($self->{'options_hash'}->{"netbiosname"}) ? "checked" : "";
			print "<input size=40 name=smbfs_cname value=\"$self->{'options_hash'}->{netbiosname}\"></td> </tr>\n";

			print "<tr> <td><b>$self->{'text_hash'}->{'linux_mname'}</b></td>\n";
			printf "<td colspan=3><input type=radio name=smbfs_mname_def value=1 %s> %s\n",
				defined($self->{'options_hash'}->{"ip"}) ? "" : "checked", $self->{'text_hash'}->{'linux_auto'};
			printf "<input type=radio name=smbfs_mname_def value=0 %s>\n",
				defined($self->{'options_hash'}->{"ip"}) ? "checked" : "";
			print "<input size=40 name=smbfs_mname value=\"$self->{'options_hash'}->{ip}\"></td> </tr>\n";

			print "<tr> <td><b>$self->{'text_hash'}->{'linux_wg'}</b></td>\n";
			printf "<td><input type=radio name=smbfs_wg_def value=1 %s> $self->{'text_hash'}->{'linux_auto'}\n",
				defined($self->{'options_hash'}->{"workgroup"}) ? "" : "checked";
			printf "<input type=radio name=smbfs_wg_def value=0 %s>\n",
				defined($self->{'options_hash'}->{"workgroup"}) ? "checked" : "";
			print "<input size=10 name=smbfs_wg value=\"$self->{'options_hash'}->{'workgroup'}\"></td>\n";

			print "</tr>\n";
			}
		}

	if ($_[0] eq "cifs") {
		# Show cifs-only options
		print "<tr> <td><b>$self->{'text_hash'}->{'linux_codepage'}</b></td>\n";
		print "<td>",$self->ui_opt_textbox("smbfs_codepage",
		    $self->{'options_hash'}->{'codepage'}, 10, $self->{'text_hash'}->{'default'}),"</td>\n";

		print "<td><b>$self->{'text_hash'}->{'linux_iocharset'}</b></td>\n";
		print "<td>",$self->ui_opt_textbox("smbfs_iocharset",
		    $self->{'options_hash'}->{'iocharset'}, 10, $self->{'text_hash'}->{'default'}),"</td> </tr>\n";
		}
	}
elsif ($_[0] eq "reiserfs") {
	# Reiserfs is a new super-efficient filesystem
        print "<tr $self->{'tb'}> <td colspan=4><b>$self->{'text_hash'}->{'edit_reiserfs_opt'}</b></td> </tr>\n";
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_notail'}</b></td>\n";
	printf "<td><input type=radio name=lnx_notail value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"notail"}) ? "checked" : "";
	printf "<input type=radio name=lnx_notail value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"notail"}) ? "" : "checked";
	}
elsif ($_[0] eq "tmpfs") {
	# Tmpfs has some size options
        print "<tr $self->{'tb'}> <td colspan=4><b>$self->{'text_hash'}->{'edit_tmpfs_opt'}</b></td> </tr>\n";
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_tmpsize'}</b></td>\n";
	printf "<td><input type=radio name=lnx_tmpsize_def value=1 %s> %s\n",
		!defined($self->{'options_hash'}->{"size"}) ? "checked" : "",
		$self->{'text_hash'}->{'linux_unlimited'};
	printf "<input type=radio name=lnx_tmpsize_def value=0 %s>\n",
		!defined($self->{'options_hash'}->{"size"}) ? "" : "checked";
	printf "<input name=lnx_tmpsize size=6 value='%s'></td>\n",
		$self->{'options_hash'}->{"size"};

	print "<td><b>$self->{'text_hash'}->{'linux_nr_blocks'}</b></td>\n";
	printf "<td><input type=radio name=lnx_nr_blocks_def value=1 %s> %s\n",
		!defined($self->{'options_hash'}->{"nr_blocks"}) ? "checked" : "",
		$self->{'text_hash'}->{'linux_unlimited'};
	printf "<input type=radio name=lnx_nr_blocks_def value=0 %s>\n",
		!defined($self->{'options_hash'}->{"nr_blocks"}) ? "" : "checked";
	printf "<input name=lnx_nr_blocks size=6 value='%s'></td> </tr>\n",
		$self->{'options_hash'}->{"nr_blocks"};

	print "<tr> <td><b>$self->{'text_hash'}->{'linux_nr_inodes'}</b></td>\n";
	printf "<td><input type=radio name=lnx_nr_inodes_def value=1 %s> %s\n",
		!defined($self->{'options_hash'}->{"nr_inodes"}) ? "checked" : "",
		$self->{'text_hash'}->{'linux_unlimited'};
	printf "<input type=radio name=lnx_nr_inodes_def value=0 %s>\n",
		!defined($self->{'options_hash'}->{"nr_inodes"}) ? "" : "checked";
	printf "<input name=lnx_nr_inodes size=6 value='%s'></td>\n",
		$self->{'options_hash'}->{"nr_inodes"};

	print "<td><b>$self->{'text_hash'}->{'linux_tmpmode'}</b></td>\n";
	printf "<td><input type=radio name=lnx_tmpmode_def value=1 %s> %s\n",
		!defined($self->{'options_hash'}->{"mode"}) ? "checked" : "", $self->{'text_hash'}->{'default'};
	printf "<input type=radio name=lnx_tmpmode_def value=0 %s>\n",
		!defined($self->{'options_hash'}->{"mode"}) ? "" : "checked";
	printf "<input name=lnx_tmpmode size=3 value='%s'></td> </tr>\n",
		$self->{'options_hash'}->{"mode"};
	}
elsif ($_[0] eq "xfs") {
	# Show options for XFS
        print "<tr $self->{'tb'}> <td colspan=4><b>$self->{'text_hash'}->{'edit_xfs_opt'}</b></td> </tr>\n";
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_usrquotas'}</b></td>\n";
	print "<td colspan=3>\n";
	printf "<input type=radio name=xfs_usrquota value=1 %s> %s\n",
		defined($self->{'options_hash'}->{"quota"}) || defined($self->{'options_hash'}->{"usrquota"}) ?
		"checked" : "", $self->{'text_hash'}->{'yes'};
	printf "<input type=radio name=xfs_usrquota value=2 %s> %s\n",
		defined($self->{'options_hash'}->{"uqnoenforce"}) ? "checked" : "",
		$self->{'text_hash'}->{'linux_noenforce'};
	printf "<input type=radio name=xfs_usrquota value=0 %s> %s</td></tr>\n",
		defined($self->{'options_hash'}->{"quota"}) || defined($self->{'options_hash'}->{"usrquota"}) ||
		defined($self->{'options_hash'}->{"uqnoenforce"}) ? "" : "checked", $self->{'text_hash'}->{'no'};

	print "<tr> <td><b>$self->{'text_hash'}->{'linux_grpquotas'}</b></td>\n";
	print "<td colspan=3>\n";
	printf "<input type=radio name=xfs_grpquota value=1 %s> %s\n",
		defined($self->{'options_hash'}->{"grpquota"}) ?  "checked" : "", $self->{'text_hash'}->{'yes'};
	printf "<input type=radio name=xfs_grpquota value=2 %s> %s\n",
		defined($self->{'options_hash'}->{"gqnoenforce"}) ? "checked" : "",
		$self->{'text_hash'}->{'linux_noenforce'};
	printf "<input type=radio name=xfs_grpquota value=0 %s> %s</td></tr>\n",
		defined($self->{'options_hash'}->{"grpquota"}) ||
		defined($self->{'options_hash'}->{"gqnoenforce"}) ? "" : "checked", $self->{'text_hash'}->{'no'};
	}
elsif ($_[0] eq "jfs") {
	# No other JFS options yet!
	}
elsif ($_[0] eq "ntfs") {
	# Windows NT/XP/2000 filesystem
        print "<tr $self->{'tb'}> <td colspan=4><b>$self->{'text_hash'}->{'edit_ntfs_opt'}</b></td> </tr>\n";
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_uid'}</b></td>\n";
	printf "<td><input name=ntfs_uid size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{"uid"}) ? getpwuid($self->{'options_hash'}->{"uid"}) : "";
	print $self->user_chooser_button("ntfs_uid", 0),"</td>\n";

	print "<td><b>$self->{'text_hash'}->{'linux_gid'}</b></td>\n";
	printf "<td><input name=ntfs_gid size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{"gid"}) ? getgrgid($self->{'options_hash'}->{"gid"}) : "";
	print $self->group_chooser_button("ntfs_gid", 0),"</td>\n";
	}
}

# check_location(type)
# Parse and check inputs from %{$self->{'in_hash'}}, calling $self->error() if something is wrong.
# Returns the location string for storing in the fstab file
sub check_location
{
	my $self = shift;

if (($_[0] eq "nfs") || ($_[0] eq "nfs4")) {
	local($out, $temp, $mout, $dirlist);

	if ($self->has_command("showmount")) {
		# Use ping and showmount to see if the host exists and is up
		if ($self->{'in_hash'}->{nfs_host} !~ /^\S+$/) {
			$self->error($self->text('linux_ehost', $self->{'in_hash'}->{'nfs_host'}));
			}
		$out = $self->backquote_command("ping -c 1 '$self->{'in_hash'}->{nfs_host}' 2>&1");
		if ($out =~ /unknown host/) {
			$self->error($self->text('linux_ehost2', $self->{'in_hash'}->{'nfs_host'}));
			}
		elsif ($out =~ /100\% packet loss/) {
			$self->error($self->text('linux_edown', $self->{'in_hash'}->{'nfs_host'}));
			}
		$out = $self->backquote_command("showmount -e '$self->{'in_hash'}->{nfs_host}' 2>&1");
		if ($out =~ /Unable to receive/) {
			$self->error($self->text('linux_enfs', $self->{'in_hash'}->{'nfs_host'}));
			}
		elsif ($?) {
			$self->error($self->text('linux_elist', $out));
			}

		# Validate directory name for NFSv3 (in v4 '/' exists)
		foreach (split(/\n/, $out)) {
			if (/^(\/\S+)/) { $dirlist .= "$1\n"; }
			}
		
		if (($_[0] ne "nfs4") && ($self->{'in_hash'}->{nfs_dir} !~ /^\/.*$/)) {
			$self->error($self->text('linux_enfsdir', $self->{'in_hash'}->{'nfs_dir'},
				     $self->{'in_hash'}->{'nfs_host'}, "<pre>$dirlist</pre>"));
		    }
	    }

	# Try a test mount to see if filesystem is available
	$temp = $self->tempname();
	$self->make_dir($temp, 0755);
	$self->execute_command("mount -t $_[0] ".
			 quotemeta("$self->{'in_hash'}->{nfs_host}:$self->{'in_hash'}->{nfs_dir}")." ".
			 quotemeta($temp),
			 undef, \$mout, \$mout);
	if ($mout =~ /No such file or directory/i) {
		$self->error($self->text('linux_enfsdir', $self->{'in_hash'}->{'nfs_dir'},
			     $self->{'in_hash'}->{'nfs_host'}, "<pre>$dirlist</pre>"));
		}
	elsif ($mout =~ /Permission denied/i) {
		$self->error($self->text('linux_enfsperm', $self->{'in_hash'}->{'nfs_dir'}, $self->{'in_hash'}->{'nfs_host'}));
		}
	elsif ($?) {
		$self->error($self->text('linux_enfsmount', "<tt>$mout</tt>"));
		}
	# It worked! unmount
	local $umout;
	$self->execute_command("umount ".quotemeta($temp), undef, \$umout, \$umout);
	if ($?) {
		$self->error($self->text('linux_enfsmount', "<tt>$umout</tt>"));
		}
	rmdir($self->translate_filename($temp));	# Don't delete mounted files!

	return "$self->{'in_hash'}->{nfs_host}:$self->{'in_hash'}->{nfs_dir}";
	}
elsif ($_[0] eq "auto") {
	# Check if the automounter map exists..
	(-r $self->{'in_hash'}->{auto_map}) || $self->error($self->text('linux_eautomap', $self->{'in_hash'}->{'auto_map'}));
	return $self->{'in_hash'}->{auto_map};
	}
elsif ($_[0] eq "autofs") {
	# Check if the map exists (if it is a file)
	if ($self->{'in_hash'}->{'autofs_map'} =~ /^\// && !(-r $self->{'in_hash'}->{'autofs_map'})) {
		$self->error($self->text('linux_eautomap', $self->{'in_hash'}->{'autofs_map'}));
		}
	return $self->{'in_hash'}->{autofs_map};
	}
elsif ($_[0] eq $self->{'smbfs_fs'} || $_[0] eq "cifs") {
	# No real checking done
	$self->{'in_hash'}->{'smbfs_server'} =~ /\S/ || $self->error($self->{'text_hash'}->{'linux_eserver'});
	$self->{'in_hash'}->{'smbfs_share'} =~ /\S/ || $self->error($self->{'text_hash'}->{'linux_eshare'});
	return "\\\\".lc($self->{'in_hash'}->{'smbfs_server'})."\\".lc($self->{'in_hash'}->{'smbfs_share'});
	}
elsif ($_[0] eq "tmpfs") {
	# No location needed
	return "tmpfs";
	}
elsif ($_[0] eq "bind") {
	# Just check the directory
	-d $self->{'in_hash'}->{'bind_dir'} || $self->error($self->{'text_hash'}->{'linux_ebind'});
	return $self->{'in_hash'}->{'bind_dir'};
	}
else {
	# This is some kind of disk-based linux filesystem.. get the device name
	if ($self->{'in_hash'}->{'lnx_dev'} == 0) {
		$self->{'dv'} = $self->{'in_hash'}->{'lnx_disk'};
		}
	elsif ($self->{'in_hash'}->{'lnx_dev'} == 2) {
		$self->{'dv'} = $self->{'in_hash'}->{'lnx_raid'};
		}
	elsif ($self->{'in_hash'}->{'lnx_dev'} == 3) {
		$self->{'dv'} = "LABEL=".$self->{'in_hash'}->{'lnx_label'};
		}
	elsif ($self->{'in_hash'}->{'lnx_dev'} == 4) {
		$self->{'dv'} = $self->{'in_hash'}->{'lnx_lvm'};
		}
	elsif ($self->{'in_hash'}->{'lnx_dev'} == 5) {
		$self->{'dv'} = "UUID=".$self->{'in_hash'}->{'lnx_uuid'};
		}
	else {
		$self->{'dv'} = $self->{'in_hash'}->{'lnx_other'};
		$self->{'dv'} || $self->error($self->{'text_hash'}->{'linux_edev'});
		}

	# If the device entered is a symlink, follow it
	#if ($dvlink = readlink($self->{'dv'})) {
	#	if ($dvlink =~ /^\//) { $self->{'dv'} = $dvlink; }
	#	else {	$self->{'dv'} =~ /^(.*\/)[^\/]+$/;
	#		$self->{'dv'} = $1.$dvlink;
	#		}
	#	}

	# Check if the device actually exists and uses the right filesystem
	if (!-r $self->{'dv'} && $self->{'dv'} !~ /LABEL=/ && $self->{'dv'} !~ /UUID=/) {
		if ($_[0] eq "swap" && $self->{'dv'} !~ /\/dev/) {
			$self->swap_form($self->{'dv'});
			}
		else {
			$self->error($self->text('linux_edevfile', $self->{'dv'}));
			}
		}
	return $self->{'dv'};
	}
}

# check_options(type, device, directory)
# Read options for some filesystem from %{$self->{'in_hash'}}, and use them to update the
# %{$self->{'options_hash'}} array. Options handled by the user interface will be set or
# removed, while unknown options will be left untouched.
sub check_options
{
	my $self = shift;

local($k, @rv);

# Parse the common options first..
if ($_[0] ne "swap" && $_[0] ne "auto" &&
    $_[0] ne "autofs" && $_[0] ne $self->{'smbfs_fs'} && $_[0] ne "cifs") {
	delete($self->{'options_hash'}->{"ro"}); delete($self->{'options_hash'}->{"rw"});
	if ($self->{'in_hash'}->{lnx_ro}) { $self->{'options_hash'}->{"ro"} = ""; }

	delete($self->{'options_hash'}->{"sync"}); delete($self->{'options_hash'}->{"async"});
	if ($self->{'in_hash'}->{lnx_sync}) { $self->{'options_hash'}->{"sync"} = ""; }

	delete($self->{'options_hash'}->{"dev"}); delete($self->{'options_hash'}->{"nodev"});
	if ($self->{'in_hash'}->{lnx_nodev}) { $self->{'options_hash'}->{"nodev"} = ""; }
	else { $self->{'options_hash'}->{"dev"} = ""; }

	delete($self->{'options_hash'}->{"exec"}); delete($self->{'options_hash'}->{"noexec"});
	if ($self->{'in_hash'}->{lnx_noexec}) { $self->{'options_hash'}->{"noexec"} = ""; }
	else { $self->{'options_hash'}->{"exec"} = ""; }

	delete($self->{'options_hash'}->{"suid"}); delete($self->{'options_hash'}->{"nosuid"});
	if ($self->{'in_hash'}->{lnx_nosuid}) { $self->{'options_hash'}->{"nosuid"} = ""; }
	else { $self->{'options_hash'}->{"suid"} = ""; }

	delete($self->{'options_hash'}->{"user"}); delete($self->{'options_hash'}->{"nouser"});
	if ($self->{'in_hash'}->{lnx_user}) { $self->{'options_hash'}->{"user"} = ""; }
	}

if (($_[0] eq "nfs") || ($_[0] eq "nfs4")) {
	# NFS has a few specific options..
	delete($self->{'options_hash'}->{"bg"}); delete($self->{'options_hash'}->{"fg"});
	if ($self->{'in_hash'}->{nfs_bg}) { $self->{'options_hash'}->{"bg"} = ""; }

	delete($self->{'options_hash'}->{"soft"}); delete($self->{'options_hash'}->{"hard"});
	if ($self->{'in_hash'}->{nfs_soft}) { $self->{'options_hash'}->{"soft"} = ""; }

	delete($self->{'options_hash'}->{"timeo"});
	if (!$self->{'in_hash'}->{nfs_timeo_def}) { $self->{'options_hash'}->{"timeo"} = $self->{'in_hash'}->{nfs_timeo}; }

	delete($self->{'options_hash'}->{"port"});
	if (!$self->{'in_hash'}->{nfs_port_def}) { $self->{'options_hash'}->{"port"} = $self->{'in_hash'}->{nfs_port}; }

	delete($self->{'options_hash'}->{"intr"}); delete($self->{'options_hash'}->{"nointr"});
	if ($self->{'in_hash'}->{nfs_intr}) { $self->{'options_hash'}->{"intr"} = ""; }

	delete($self->{'options_hash'}->{"tcp"}); delete($self->{'options_hash'}->{"udp"});
	local $dproto = $_[0] eq "nfs" ? "udp" : "tcp";
	if ($self->{'in_hash'}->{nfs_transfert} eq "tcp" && $dproto ne "tcp") {
		$self->{'options_hash'}->{"tcp"} = "";
		}
	elsif ($self->{'in_hash'}->{nfs_transfert} eq "udp" && $dproto ne "udp" && $_[0] ne "nfs4") {
		$self->{'options_hash'}->{"udp"} = "";
		}

	delete($self->{'options_hash'}->{"wsize"});
	if (!$self->{'in_hash'}->{nfs_wsize_def}) { $self->{'options_hash'}->{"wsize"} = $self->{'in_hash'}->{nfs_wsize}; }

	delete($self->{'options_hash'}->{"rsize"});
	if (!$self->{'in_hash'}->{nfs_rsize_def}) { $self->{'options_hash'}->{"rsize"} = $self->{'in_hash'}->{nfs_rsize}; }

	delete($self->{'options_hash'}->{"sec"});
	# Only sys and krb5 for the moment
	if ($self->{'in_hash'}->{nfs_auth}) {
	    if ($self->{'in_hash'}->{nfs_sec} == 0) { $self->{'options_hash'}->{"sec"} = "krb5"; }
	    if ($self->{'in_hash'}->{nfs_sec} == 1) { $self->{'options_hash'}->{"sec"} = "krb5i"; }
	    if ($self->{'in_hash'}->{nfs_sec} == 2) { $self->{'options_hash'}->{"sec"} = "krb5p"; }
	}
    }
elsif ($_[0] eq "ext2" || $_[0] eq "ext3") {
	# More options for ext2..
	if ($self->{'no_mount_check'}) {
		delete($self->{'options_hash'}->{"bsddf"}); delete($self->{'options_hash'}->{"minixdf"});
		$self->{'options_hash'}->{"minixdf"} = "" if ($self->{'in_hash'}->{'ext2_df'});
		}
	else {
		delete($self->{'options_hash'}->{"check"}); delete($self->{'options_hash'}->{"nocheck"});
		if ($self->{'in_hash'}->{ext2_check} ne "normal") {
			$self->{'options_hash'}->{"check"} = $self->{'in_hash'}->{ext2_check};
			}
		}

	delete($self->{'options_hash'}->{"errors"});
	if ($self->{'in_hash'}->{ext2_errors} ne "default") {
		$self->{'options_hash'}->{"errors"} = $self->{'in_hash'}->{ext2_errors};
		}

	delete($self->{'options_hash'}->{"grpid"}); delete($self->{'options_hash'}->{"bsdgroups"});
	delete($self->{'options_hash'}->{"sysvgroups"}); delete($self->{'options_hash'}->{"nogrpid"});
	if ($self->{'in_hash'}->{ext2_grpid}) {
		$self->{'options_hash'}->{"grpid"} = "";
		}

	delete($self->{'options_hash'}->{"resuid"}); delete($self->{'options_hash'}->{"resgid"});
	if ($self->{'in_hash'}->{'ext2_resuid'})
		{ $self->{'options_hash'}->{"resuid"} = getpwnam($self->{'in_hash'}->{'ext2_resuid'}); }
	if ($self->{'in_hash'}->{'ext2_resgid'})
		{ $self->{'options_hash'}->{"resgid"} = getgrnam($self->{'in_hash'}->{'ext2_resgid'}); }

	delete($self->{'options_hash'}->{"quota"}); delete($self->{'options_hash'}->{"noquota"});
	delete($self->{'options_hash'}->{"usrquota"}); delete($self->{'options_hash'}->{"grpquota"});
	if ($self->{'in_hash'}->{'ext2_quota'} == 1) { $self->{'options_hash'}->{'usrquota'} = ""; }
	elsif ($self->{'in_hash'}->{'ext2_quota'} == 2) { $self->{'options_hash'}->{'grpquota'} = ""; }
	elsif ($self->{'in_hash'}->{'ext2_quota'} == 3)
		{ $self->{'options_hash'}->{'usrquota'} = $self->{'options_hash'}->{'grpquota'} = ""; }
	}
elsif ($_[0] eq "fat" || $_[0] eq "vfat" ||
       $_[0] eq "msdos" || $_[0] eq "umsdos" || $_[0] eq "fatx") {
	# All dos-based filesystems have similar options
	delete($self->{'options_hash'}->{"uid"}); delete($self->{'options_hash'}->{"gid"});
	if ($self->{'in_hash'}->{fat_uid} ne "") { $self->{'options_hash'}->{"uid"} = getpwnam($self->{'in_hash'}->{'fat_uid'}); }
	if ($self->{'in_hash'}->{fat_gid} ne "") { $self->{'options_hash'}->{"gid"} = getgrnam($self->{'in_hash'}->{'fat_gid'}); }

	delete($self->{'options_hash'}->{"check"});
	if ($self->{'in_hash'}->{fat_check} ne "") { $self->{'options_hash'}->{"check"} = $self->{'in_hash'}->{fat_check}; }

	delete($self->{'options_hash'}->{"conv"});
	if ($self->{'in_hash'}->{fat_conv} ne "") { $self->{'options_hash'}->{"conv"} = $self->{'in_hash'}->{fat_conv}; }

	delete($self->{'options_hash'}->{"umask"});
	if (!$self->{'in_hash'}->{fat_umask_def}) {
		$self->{'in_hash'}->{fat_umask} =~ /^[0-7]{3}$/ ||
			$self->error($self->text('linux_emask', $self->{'in_hash'}->{'fat_umask'}));
		$self->{'options_hash'}->{"umask"} = $self->{'in_hash'}->{fat_umask};
		}

	delete($self->{'options_hash'}->{"quiet"});
	if ($self->{'in_hash'}->{fat_quiet}) {
		$self->{'options_hash'}->{"quiet"} = "";
		}

	if ($_[0] eq "vfat") {
		# Parse extra vfat options..
		delete($self->{'options_hash'}->{"uni_xlate"});
		if ($self->{'in_hash'}->{fat_uni_xlate}) { $self->{'options_hash'}->{"uni_xlate"} = ""; }

		delete($self->{'options_hash'}->{"posix"});
		if ($self->{'in_hash'}->{fat_posix}) { $self->{'options_hash'}->{"posix"} = ""; }
		}
	}
elsif ($_[0] eq "hpfs") {
	# OS/2 filesystem options..
	delete($self->{'options_hash'}->{"uid"}); delete($self->{'options_hash'}->{"gid"});
	if ($self->{'in_hash'}->{hpfs_uid} ne "") { $self->{'options_hash'}->{"uid"} = getpwnam($self->{'in_hash'}->{hpfs_uid}); }
	if ($self->{'in_hash'}->{hpfs_gid} ne "") { $self->{'options_hash'}->{"gid"} = getgrnam($self->{'in_hash'}->{hpfs_gid}); }

	delete($self->{'options_hash'}->{"umask"});
	if (!$self->{'in_hash'}->{hpfs_umask_def}) {
		$self->{'in_hash'}->{hpfs_umask} =~ /^[0-7]{3}$/ ||
			$self->error($self->text('linux_emask', $self->{'in_hash'}->{'hpfs_umask'}));
		$self->{'options_hash'}->{"umask"} = $self->{'in_hash'}->{hpfs_umask};
		}

	delete($self->{'options_hash'}->{"conv"});
	if ($self->{'in_hash'}->{hpfs_conv} ne "") { $self->{'options_hash'}->{"conv"} = $self->{'in_hash'}->{hpfs_conv}; }
	}
elsif ($_[0] eq "iso9660") {
	# Options for iso9660 cd-roms
	delete($self->{'options_hash'}->{"uid"}); delete($self->{'options_hash'}->{"gid"});
	if ($self->{'in_hash'}->{iso9660_uid} ne "")
		{ $self->{'options_hash'}->{"uid"} = getpwnam($self->{'in_hash'}->{iso9660_uid}); }
	if ($self->{'in_hash'}->{iso9660_gid} ne "")
		{ $self->{'options_hash'}->{"gid"} = getgrnam($self->{'in_hash'}->{iso9660_gid}); }

	delete($self->{'options_hash'}->{"norock"});
	if ($self->{'in_hash'}->{iso9660_norock}) { $self->{'options_hash'}->{"norock"} = ""; }

	delete($self->{'options_hash'}->{"mode"});
	$self->{'in_hash'}->{iso9660_mode} =~ /^[0-7]{3}$/ ||
		$self->error($self->text('linux_emask', $self->{'in_hash'}->{'iso9660_mode'}));
	$self->{'options_hash'}->{"mode"} = $self->{'in_hash'}->{iso9660_mode};
	}
elsif ($_[0] eq "autofs") {
	# Options for automounter filesystems
	delete($self->{'options_hash'}->{'timeout'});
	if (!$self->{'in_hash'}->{'autofs_timeout_def'}) {
		$self->{'in_hash'}->{'autofs_timeout'} =~ /^\d+$/ ||
			$self->error($self->text('linux_etimeout', $self->{'in_hash'}->{'autofs_timeout'}));
		$self->{'options_hash'}->{'timeout'} = $self->{'in_hash'}->{'autofs_timeout'};
		}
	delete($self->{'options_hash'}->{'pid-file'});
	if (!$self->{'in_hash'}->{'autofs_pid-file_def'}) {
		$self->{'in_hash'}->{'autofs_pid-file'} =~ /^\/\S+$/ ||
		       $self->error($self->text('linux_epidfile', $self->{'in_hash'}->{'autofs_pid-file'}));
		$self->{'options_hash'}->{'pid-file'} = $self->{'in_hash'}->{'autofs_pid-file'};
		}
	}
elsif ($_[0] eq $self->{'smbfs_fs'} || $_[0] eq "cifs") {
	# Options for smb filesystems..
	local $support = $_[0] eq $self->{'smbfs_fs'} ? $self->{'smbfs_support'} : $self->{'cifs_support'};
	delete($self->{'options_hash'}->{'user'}); delete($self->{'options_hash'}->{'username'});
	if ($self->{'in_hash'}->{smbfs_user}) {
		$self->{'options_hash'}->{$support == 4 ? 'username' : 'user'} = $self->{'in_hash'}->{smbfs_user};
		}

	delete($self->{'options_hash'}->{'passwd'}); delete($self->{'options_hash'}->{'password'});
	if ($self->{'in_hash'}->{smbfs_passwd}) {
		$self->{'options_hash'}->{$support == 4 ? 'password' : 'passwd'} = $self->{'in_hash'}->{smbfs_passwd};
		}

	if (!$self->{'access_hash'}->{'simopts'}) {
		if ($support != 2) {
			delete($self->{'options_hash'}->{uid});
			if ($self->{'in_hash'}->{smbfs_uid} ne "") { $self->{'options_hash'}->{uid} = getpwnam($self->{'in_hash'}->{smbfs_uid}); }

			delete($self->{'options_hash'}->{gid});
			if ($self->{'in_hash'}->{smbfs_gid} ne "") { $self->{'options_hash'}->{gid} = getgrnam($self->{'in_hash'}->{smbfs_gid}); }
			}

		if ($support == 1) {
			delete($self->{'options_hash'}->{servername});
			if (!$self->{'in_hash'}->{smbfs_sname_def})
				{ $self->{'options_hash'}->{servername} = $self->{'in_hash'}->{smbfs_sname}; }
			}
		elsif ($support == 2 || $support == 4) {
			delete($self->{'options_hash'}->{workgroup});
			if (!$self->{'in_hash'}->{smbfs_wg_def})
				{ $self->{'options_hash'}->{workgroup} = $self->{'in_hash'}->{smbfs_wg}; }
			}

		if ($support < 3) {
			delete($self->{'options_hash'}->{clientname});
			if (!$self->{'in_hash'}->{smbfs_cname_def})
				{ $self->{'options_hash'}->{clientname} = $self->{'in_hash'}->{smbfs_cname}; }

			delete($self->{'options_hash'}->{machinename});
			if (!$self->{'in_hash'}->{smbfs_mname_def})
				{ $self->{'options_hash'}->{machinename} = $self->{'in_hash'}->{smbfs_mname}; }
			elsif (!gethostbyname($self->{'in_hash'}->{'smbfs_server'})) {
				# No hostname found for the server.. try to guess
				local($out, $sname);
				$sname = $self->{'in_hash'}->{'smbfs_server'};
				$out = $self->backquote_command("$self->{'config_hash'}->{'nmblookup_path'} -d 0 $sname 2>&1");
				if (!$? && $out =~ /^([0-9\.]+)\s+$sname\n/) {
					$self->{'options_hash'}->{machinename} = $1;
					}
				}
			}
		elsif ($support == 4) {
			delete($self->{'options_hash'}->{"netbiosname"});
			if (!$self->{'in_hash'}->{"smbfs_cname_def"}) {
				$self->{'in_hash'}->{"smbfs_cname"} =~ /^\S+$/ ||
					$self->error($self->{'text_hash'}->{'linux_ecname'});
				$self->{'options_hash'}->{"netbiosname"} = $self->{'in_hash'}->{"smbfs_cname"};
				}
			delete($self->{'options_hash'}->{"ip"});
			if (!$self->{'in_hash'}->{"smbfs_mname_def"}) {
				gethostbyname($self->{'in_hash'}->{"smbfs_mname"}) ||
				    $self->check_ipaddress($self->{'in_hash'}->{"smbfs_mname"}) ||
					$self->error($self->{'text_hash'}->{'linux_emname'});
				$self->{'options_hash'}->{"ip"} = $self->{'in_hash'}->{"smbfs_mname"};
				}
			}

		if ($support == 1) {
			delete($self->{'options_hash'}->{fmode});
			if ($self->{'in_hash'}->{smbfs_fmode} !~ /^[0-7]{3}$/) {
				$self->error($self->text('linux_efmode', $self->{'in_hash'}->{'smbfs_fmode'}));
				}
			elsif ($self->{'in_hash'}->{smbfs_fmode} ne "755")
				{ $self->{'options_hash'}->{fmode} = $self->{'in_hash'}->{smbfs_fmode}; }

			delete($self->{'options_hash'}->{dmode});
			if ($self->{'in_hash'}->{smbfs_dmode} !~ /^[0-7]{3}$/) {
				$self->error($self->text('linux_edmode', $self->{'in_hash'}->{'smbfs_dmode'}));
				}
			elsif ($self->{'in_hash'}->{smbfs_dmode} ne "755")
				{ $self->{'options_hash'}->{dmode} = $self->{'in_hash'}->{smbfs_dmode}; }
			}
		elsif ($support >= 3) {
			if ($self->{'in_hash'}->{'smbfs_fmask_def'}) {
				delete($self->{'options_hash'}->{'fmask'});
				}
			else {
				$self->{'in_hash'}->{'smbfs_fmask'} =~ /^[0-7]{3}$/ ||
				    $self->error($self->text('linux_efmode', $self->{'in_hash'}->{'smbfs_fmask'}));
				$self->{'options_hash'}->{'fmask'} = $self->{'in_hash'}->{'smbfs_fmask'};
				}

			if ($self->{'in_hash'}->{'smbfs_dmask_def'}) {
				delete($self->{'options_hash'}->{'dmask'});
				}
			else {
				$self->{'in_hash'}->{'smbfs_dmask'} =~ /^[0-7]{3}$/ ||
				    $self->error($self->text('linux_edmode', $self->{'in_hash'}->{'smbfs_dmask'}));
				$self->{'options_hash'}->{'dmask'} = $self->{'in_hash'}->{'smbfs_dmask'};
				}

			delete($self->{'options_hash'}->{'ro'}); delete($self->{'options_hash'}->{'rw'});
			if ($self->{'in_hash'}->{'smbfs_ro'}) { $self->{'options_hash'}->{'ro'} = ''; }

			delete($self->{'options_hash'}->{'user'});
			if ($self->{'in_hash'}->{'smbfs_user2'}) { $self->{'options_hash'}->{'user'} = ''; }
			}
		}

	if ($_[0] eq "cifs") {
		# Parse CIFS-specific options
		delete($self->{'options_hash'}->{'codepage'});
		if (!$self->{'in_hash'}->{'smbfs_codepage_def'}) {
			$self->{'in_hash'}->{'smbfs_codepage'} =~ /^\S+$/ ||
				$self->error($self->{'text_hash'}->{'linux_ecodepage'});
			$self->{'options_hash'}->{'codepage'} = $self->{'in_hash'}->{'smbfs_codepage'};
			}

		delete($self->{'options_hash'}->{'iocharset'});
		if (!$self->{'in_hash'}->{'smbfs_iocharset_def'}) {
			$self->{'in_hash'}->{'smbfs_iocharset'} =~ /^\S+$/ ||
				$self->error($self->{'text_hash'}->{'linux_eiocharset'});
			$self->{'options_hash'}->{'iocharset'} = $self->{'in_hash'}->{'smbfs_iocharset'};
			}
		}
	}
elsif ($_[0] eq "reiserfs") {
	# Save reiserfs options
	delete($self->{'options_hash'}->{'notail'});
	$self->{'options_hash'}->{'notail'} = "" if ($self->{'in_hash'}->{'lnx_notail'});

	if ($self->{'in_hash'}->{'lnx_user'} && !$self->{'in_hash'}->{'lnx_noexec'}) {
		# Have to force on the exec option
		$self->{'options_hash'}->{"exec"} = "";
		}
	}
elsif ($_[0] eq "tmpfs") {
	# Save tmpfs options
	if ($self->{'in_hash'}->{'lnx_tmpsize_def'}) {
		delete($self->{'options_hash'}->{'size'});
		}
	else {
		$self->{'in_hash'}->{'lnx_tmpsize'} =~ /^(\d+)([kmg]?)$/ ||
			$self->error($self->{'text_hash'}->{'lnx_etmpsize'});
		$self->{'options_hash'}->{'size'} = $self->{'in_hash'}->{'lnx_tmpsize'};
		}

	if ($self->{'in_hash'}->{'lnx_nr_blocks_def'}) {
		delete($self->{'options_hash'}->{'nr_blocks'});
		}
	else {
		$self->{'in_hash'}->{'lnx_nr_blocks'} =~ /^\d+$/ ||
			$self->error($self->{'text_hash'}->{'lnx_enr_blocks'});
		$self->{'options_hash'}->{'nr_blocks'} = $self->{'in_hash'}->{'lnx_nr_blocks'};
		}

	if ($self->{'in_hash'}->{'lnx_nr_inodes_def'}) {
		delete($self->{'options_hash'}->{'nr_inodes'});
		}
	else {
		$self->{'in_hash'}->{'lnx_nr_inodes'} =~ /^\d+$/ ||
			$self->error($self->{'text_hash'}->{'lnx_enr_inodes'});
		$self->{'options_hash'}->{'nr_inodes'} = $self->{'in_hash'}->{'lnx_nr_inodes'};
		}

	if ($self->{'in_hash'}->{'lnx_tmpmode_def'}) {
		delete($self->{'options_hash'}->{'mode'});
		}
	else {
		$self->{'in_hash'}->{'lnx_tmpmode'} =~ /^[0-7]{3,4}$/ ||
			$self->error($self->{'text_hash'}->{'lnx_etmpmode'});
		$self->{'options_hash'}->{'mode'} = $self->{'in_hash'}->{'lnx_tmpmode'};
		}
	}
elsif ($_[0] eq "xfs") {
	# Save XFS options
	delete($self->{'options_hash'}->{'quota'});
	delete($self->{'options_hash'}->{'usrquota'});
	delete($self->{'options_hash'}->{'uqnoenforce'});
	$self->{'options_hash'}->{'usrquota'} = "" if ($self->{'in_hash'}->{'xfs_usrquota'} == 1);
	$self->{'options_hash'}->{'uqnoenforce'} = "" if ($self->{'in_hash'}->{'xfs_usrquota'} == 2);

	delete($self->{'options_hash'}->{'grpquota'});
	delete($self->{'options_hash'}->{'gqnoenforce'});
	$self->{'options_hash'}->{'grpquota'} = "" if ($self->{'in_hash'}->{'xfs_grpquota'} == 1);
	$self->{'options_hash'}->{'gqnoenforce'} = "" if ($self->{'in_hash'}->{'xfs_grpquota'} == 2);
	}
elsif ($_[0] eq "ntfs") {
	# Save NTFS options
	delete($self->{'options_hash'}->{"uid"}); delete($self->{'options_hash'}->{"gid"});
	if ($self->{'in_hash'}->{ntfs_uid} ne "")
		{ $self->{'options_hash'}->{"uid"} = getpwnam($self->{'in_hash'}->{ntfs_uid}); }
	if ($self->{'in_hash'}->{ntfs_gid} ne "")
		{ $self->{'options_hash'}->{"gid"} = getgrnam($self->{'in_hash'}->{ntfs_gid}); }
	}
elsif ($_[0] eq "swap") {
	# Save SWAP options
	if ($self->{'in_hash'}->{'swap_pri_def'}) {
		delete($self->{'options_hash'}->{'pri'});
		}
	else {
		$self->{'in_hash'}->{'swap_pri'} =~ /^\d+$/ && $self->{'in_hash'}->{'swap_pri'} <= 32767 ||
			$self->error($self->{'text_hash'}->{'linux_eswappri'});
		$self->{'options_hash'}->{'pri'} = $self->{'in_hash'}->{'swap_pri'};
		}
	}

# Add loop option if mounting a normal file
if ($_[0] ne "swap" && $_[0] ne "auto" && $_[0] ne "autofs" &&
    $_[0] ne $self->{'smbfs_fs'} && $_[0] ne "cifs" && $_[0] ne "nfs" &&
    $_[0] ne "nfs4" && $_[0] ne "tmpfs") {
	local(@st);
	@st = stat($_[1]);
	if (@st && ($st[2] & 0xF000) == 0x8000) {
		# a regular file.. add the loop option
		if (!$self->{'options_hash'}->{'loop'}) {
			$self->{'options_hash'}->{'loop'} = "";
			}
		}
	}

# Return options string
foreach $k (sort { ($b eq "user" ? 1 : 0) <=> ($a eq "user" ? 1 : 0) } (keys %{$self->{'options_hash'}})) {
	if ($self->{'options_hash'}->{$k} eq "") { push(@rv, $k); }
	else { push(@rv, "$k=$self->{'options_hash'}->{$k}"); }
	}
return @rv ? join(',' , @rv) : "-";
}

# Get the smbfs options from 'smbfs_opts' file in the current directory. This
# is sadly necessary because there is no way to get the current options for
# an existing smbfs mount... so webmin has to save them in a file when
# mounting. Blech.
sub read_smbopts
{
	my $self = shift;

local($_);
open(SMBOPTS, "$self->{'module_config_directory'}/smbfs");
while(<SMBOPTS>) {
	/^(\S+)\s+(\S+)$/;
	$self->{'smbopts_hash'}->{$1} = $2;
	}
close(SMBOPTS);
}

sub write_smbopts
{
	my $self = shift;

local($_);
$self->open_tempfile(SMBOPTS, "> $self->{'module_config_directory'}/smbfs");
foreach (keys %{$self->{'smbopts_hash'}}) {
	$self->print_tempfile(SMBOPTS, "$_\t$self->{'smbopts_hash'}->{$_}\n");
	}
$self->close_tempfile(SMBOPTS);
}

# create_swap(file, size, units)
# Calls dd and mkswap to setup a swap file
sub create_swap
{
	my $self = shift;

local($out, $bl);
$bl = $_[1] * ($_[2] eq "g" ? 1024*1024 :
	       $_[2] eq "m" ? 1024 : 1);
$out = $self->backquote_logged("dd if=/dev/zero of=$_[0] bs=1024 count=$bl 2>&1");
if ($?) { return "dd failed : $out"; }
$out = $self->backquote_logged("mkswap $_[0] $bl 2>&1");
if ($?) { return "mkswap failed : $out"; }
$self->system_logged("sync >/dev/null 2>&1");
return 0;
}

# exports_list(host, dirarray, clientarray)
# Fills the directory and client array references with exports from some
# host. Returns an error string if something went wrong
sub exports_list
{
	my $self = shift;

local($dref, $cref, $out, $_);
$dref = $_[1]; $cref = $_[2];
$out = $self->backquote_command("showmount -e ".quotemeta($_[0])." 2>&1", 1);
if ($?) { return $out; }

# Add '/' if the server is in NFSv4
if ($self->nfs_max_version($_[0]) >= 4) {
    push(@$dref, "/"); push(@$cref, "*"); }

foreach (split(/\n/, $out)) {
	if (/^(\/\S*)\s+(.*)$/) {
		push(@$dref, $1); push(@$cref, $2);
		}
	}
return undef;
}

# nfs_max_version(host)
# Return the max NFS version allowed on a server
sub nfs_max_version
{
	my $self = shift;

    local($_, $max, $out);
    $max = 0;
    $out = $self->backquote_command("/usr/sbin/rpcinfo -p ".quotemeta($_[0])." 2>&1", 1);
    if ($?) { return $out; }
    foreach (split(/\n/, $out)) {
	if ((/ +(\d) +.*nfs/) && ($1 > $max)) {
	    $max = $1; }
    }
    return $max;
}

# broadcast_addr()
# Returns a useable broadcast address for finding NFS servers
sub broadcast_addr
{
	my $self = shift;

local($out);
$out = $self->backquote_command("ifconfig -a 2>&1", 1);
if ($out =~ /(eth|tr)\d\s+.*\n.*Bcast:(\S+)\s+/) { return $2; }
return "255.255.255.255";
}

# autofs_options(string)
# Converts a string of options line --timeout 60 to something like timeout=60
sub autofs_options
{
	my $self = shift;

local(@options);
if ($_[0] =~ /--timeout\s+(\d+)/ || $_[0] =~ /-t\s+(\d+)/) {
	push(@options, "timeout=$1");
	}
if ($_[0] =~ /--pid-file\s+(\S+)/ || $_[0] =~ /-p\s+(\d+)/) {
	push(@options, "pid-file=$1");
	}
return join(',', @options);
}

# autofs_args(string)
# Convert a comma-separated options string into args for automount
sub autofs_args
{
	my $self = shift;

local(%options, $args);
$self->parse_options("autofs", $_[0]);
if (defined($options{'timeout'})) {
	$args .= " --timeout $options{'timeout'}";
	}
if (defined($options{'pid-file'})) {
	$args .= " --pid-file $options{'pid-file'}";
	}
return $args;
}

# read_amd_conf()
# Returns the entire amd config file as a string
sub read_amd_conf
{
	my $self = shift;

local $sl = $/;
$/ = undef;
local $rv;
foreach $f (split(/\s+/, $self->{'config_hash'}->{'auto_file'})) {
	open(AMD, $f);
	$rv .= <AMD>;
	close(AMD);
	}
$/ = $sl;
return $rv;
}

# write_amd_conf(text)
sub write_amd_conf
{
	my $self = shift;

local @af = split(/\s+/, $self->{'config_hash'}->{'auto_file'});
$self->open_tempfile(AMD, ">$self->{'config_hash'}->{'auto_file'}");
$self->print_tempfile(AMD, $_[0]);
$self->close_tempfile(AMD);
}

# parse_amd_conf()
# Parses a new-style amd.conf file into a hashtable
sub parse_amd_conf
{
	my $self = shift;

local (@rv, $str);
foreach $f (split(/\s+/, $self->{'config_hash'}->{'auto_file'})) {
	local $lnum = 0;
	open(AMD, $f);
	while(<AMD>) {
		s/\r|\n//g;
		s/#.*$//g;
		if (/\[\s*(\S+)\s*\]/) {
			$str = { 'dir' => $1,
				 'line' => $lnum,
				 'eline' => $lnum,
				 'file' => $f };
			push(@rv, $str);
			}
		elsif (/(\S+)\s*=\s*"(.*)"/ || /(\S+)\s*=\s*(\S+)/) {
			$str->{'opts'}->{$1} = $2;
			$str->{'eline'} = $lnum;
			}
		$lnum++;
		}
	close(AMD);
	}
return @rv;
}

# device_name(device)
# Converts a device name to a human-readable form
sub device_name
{
	my $self = shift;

if (!$self->{'text_hash'}->{'select_part'}) {
	local %flang = $self->load_language('fdisk');
	foreach $k (keys %flang) {
		$self->{'text_hash'}->{$k} = $flang{$k} if ($k =~ /^select_/);
		}
	}
return $_[0] =~ /^\/dev\/(s|h)d([a-z])(\d+)$/ ?
	$self->text('select_part', $1 eq 's' ? 'SCSI' : 'IDE', uc($2), "$3") :
       $_[0] =~ /^\/dev\/(s|h)d([a-z])$/ ?
	$self->text('select_device', $1 eq 's' ? 'SCSI' : 'IDE', uc($2)) :
       $_[0] =~ /rd\/c(\d+)d(\d+)p(\d+)$/ ?
	$self->text('select_mpart', "$1", "$2", "$3") :
       $_[0] =~ /ida\/c(\d+)d(\d+)p(\d+)$/ ?
	$self->text('select_cpart', "$1", "$2", "$3") :
       $_[0] =~ /cciss\/c(\d+)d(\d+)p(\d+)$/ ?
	$self->text('select_smartpart', "$1", "$2", "$3") :
       $_[0] =~ /scsi\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/part(\d+)/ ?
	$self->text('select_spart', "$1", "$2", "$3", "$4", "$5") :
       $_[0] =~ /scsi\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/disc/ ?
	$self->text('select_scsi', "$1", "$2", "$3", "$4") :
       $_[0] =~ /ide\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/part(\d+)/ ?
	$self->text('select_snewide', "$1", "$2", "$3", "$4", "$5") :
       $_[0] =~ /ide\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/disc/ ?
	$self->text('select_newide', "$1", "$2", "$3", "$4") :
       $_[0] =~ /ataraid\/disc(\d+)\/part(\d+)$/ ?
	$self->text('select_ppart', "$1", "$2") :
       $_[0] =~ /fd(\d+)$/ ?
	$self->text('select_fd', "$1") :
       $_[0] =~ /md(\d+)$/ ?
	$self->text('linux_rdev', "$1") :
       $_[0] =~ /\/dev\/([^\/]+)\/([^\/]+)$/ && $1 ne "cdroms" ?
	$self->text('linux_ldev', "$1", "$2") :
       $_[0] =~ /LABEL=(\S+)/i ?
	$self->text('linux_label', "$1") :
       $_[0] =~ /UUID=(\S+)/i ?
	$self->text('linux_uuid', "$1") :
       $_[0] eq '/dev/cdrom' ?
	$self->{'text_hash'}->{'linux_cddev'} :
       $_[0] eq '/dev/burner' ?
	$self->{'text_hash'}->{'linux_burnerdev'} :
       $_[0] =~ /cdroms\/cdrom(\d+)$/ ?
	$self->text('linux_cddev2', "$1") :
	$_[0];
}

sub files_to_lock
{
	my $self = shift;

return ( $self->{'config_hash'}->{'fstab_file'}, $self->{'config_hash'}->{'autofs_file'},
	 split(/\s+/, $self->{'config_hash'}->{'auto_file'}) );
}


1;

