#!/usr/bin/perl
package fdisk::FdiskLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'fdisk::FdiskLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# fdisk-lib.pl
# Functions for disk management under linux

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
$self->foreign_require("mount", "mount-lib.pl");
if ($self->foreign_check("raid")) {
	$self->foreign_require("raid", "raid-lib.pl");
	$self->{'raid_module'}++;
	}
if ($self->foreign_check("lvm")) {
	$self->foreign_require("lvm", "lvm-lib.pl");
	$self->{'lvm_module'}++;
	}
$self->foreign_require("proc", "proc-lib.pl");
%{$self->{'access_hash'}} = $self->get_module_acl();
$self->{'has_e2label'} = $self->has_command("e2label");
$self->{'has_xfs_db'} = $self->has_command("xfs_db");
$self->{'has_volid'} = $self->has_command("vol_id");
$| = 1;

%{$self->{'tags_hash'}} = ('0', 'Empty',
	 '1', 'FAT12',
	 '2', 'XENIX root',
	 '3', 'XENIX usr',
	 '4', 'FAT16 <32M',
	 '6', 'FAT16',
	 '7', 'NTFS',
	 '8', 'AIX',
	 '9', 'AIX bootable',
	 'a', 'OS/2 boot manager',
	 'b', 'Win95 FAT32',
	 'c', 'Win95 FAT32 LBA',
	 'e', 'Win95 FAT16 LBA',
	'10', 'OPUS',
	'11', 'Hidden FAT12',
	'12', 'Compaq diagnostic',
	'14', 'Hidden FAT16 < 32M',
	'16', 'Hidden FAT16',
	'17', 'Hidden NTFS',
	'18', 'AST Windows swapfile',
	'1b', 'Hidden Win95 FAT (1b)',
	'1c', 'Hidden Win95 FAT (1c)',
	'1e', 'Hidden Win95 FAT (1e)',
	'24', 'NEC DOS',
	'3c', 'PartitionMagic recovery',
	'40', 'Venix 80286',
	'41', 'PPC PReP boot',
	'42', 'SFS',
	'4d', 'QNX 4.x',
	'4e', 'QNX 4.x 2nd partition',
	'4f', 'QNX 4.x 3rd partition',
	'50', 'OnTrack DM',
	'51', 'OnTrack DM6 Aux1',
	'52', 'CP/M',
	'53', 'OnTrack DM6 Aux3',
	'54', 'OnTrack DM6',
	'55', 'EZ-Drive',
	'56', 'Golden Bow',
	'5c', 'Priam Edisk',
	'61', 'SpeedStor',
	'63', 'GNU HURD or SysV',
	'64', 'Novell Netware 286',
	'65', 'Novell Netware 386',
	'70', 'DiskSecure Multi-Boot',
	'75', 'PC/IX',
	'80', 'Old Minix',
	'81', 'Minix / Old Linux / Solaris',
	'82', 'Linux swap',
	'83', 'Linux',
	'84', 'OS/2 hidden C: drive',
	'85', 'Linux extended',
	'86', 'NTFS volume set (86)',
	'87', 'NTFS volume set (87)',
	'8e', 'Linux LVM',
	'93', 'Amoeba',
	'94', 'Amoeba BBT',
	'a0', 'IBM Thinkpad hibernation',
	'a5', 'BSD/386',
	'a6', 'OpenBSD',
	'a7', 'NeXTSTEP',
	'b7', 'BSDI filesystem',
	'b8', 'BSDI swap',
	'c1', 'DRDOS/sec FAT12',
	'c4', 'DRDOS/sec FAT16 <32M',
	'c6', 'DRDOS/sec FAT16',
	'c7', 'Syrinx',
	'db', 'CP/M / CTOS',
	'e1', 'DOS access',
	'e3', 'DOS read-only',
	'e4', 'SpeedStor',
	'eb', 'BeOS',
	'f1', 'SpeedStor',
	'f4', 'SpeedStor large partition',
	'f2', 'DOS secondary',
	'fd', 'Linux RAID',
	'fe', 'LANstep',
	'ff', 'BBT'
	);

%{$self->{'hidden_tags_hash'}} = (
	 '5', 'Extended',
	 'f', 'Win95 extended LBA',
	);
	
@{$self->{'space_type_array'}} = ( '1', '4', '5', '6', 'b', 'c', 'e', '83' );

1;
		return $self;
}

# list_disks_partitions([include-cds])
# Returns a structure containing the details of all disks and partitions
sub list_disks_partitions
{
	my $self = shift;

if (defined(@{$self->{'list_disks_partitions_cache_array'}})) {
	return @{$self->{'list_disks_partitions_cache_array'}};
	}

local (@pscsi, @dscsi, $dscsi_mode);
if (-r "/proc/scsi/sg/devices" && -r "/proc/scsi/sg/device_strs") {
	# Get device info from various /proc/scsi files
	open(DEVICES, "/proc/scsi/sg/devices");
	while(<DEVICES>) {
		s/\r|\n//g;
		local @l = split(/\t+/, $_);
		push(@dscsi, { 'host' => $l[0],
			       'bus' => $l[1],
			       'target' => $l[2],
			       'lun' => $l[3],
			       'type' => $l[4] });
		}
	close(DEVICES);
	local $i = 0;
	open(DEVNAMES, "/proc/scsi/sg/device_strs");
	while(<DEVNAMES>) {
		s/\r|\n//g;
		local @l = split(/\t+/, $_);
		$dscsi[$i]->{'make'} = $l[0];
		$dscsi[$i]->{'model'} = $l[1];
		$i++;
		}
	close(DEVNAMES);
	$dscsi_mode = 1;
	@dscsi = grep { $_->{'type'} == 0 } @dscsi;
	}
else {
	# Check /proc/scsi/scsi for SCSI disk models
	open(SCSI, "/proc/scsi/scsi");
	local @lines = <SCSI>;
	close(SCSI);
	if ($lines[0] =~ /^Attached\s+domains/i) {
		# New domains format
		local $dscsi;
		foreach (@lines) {
			s/\s/ /g;
			if (/Device:\s+(.*)(sd[a-z]+)\s+usage/) {
				$dscsi = { 'dev' => $2 };
				push(@dscsi, $dscsi);
				}
			elsif (/Device:/) {
				$dscsi = undef;
				}
			elsif (/Vendor:\s+(\S+)\s+Model:\s+(\S+)/ && $dscsi) {
				$dscsi->{'make'} = $1;
				$dscsi->{'model'} = $2;
				}
			elsif (/Host:\s+scsi(\d+)\s+Channel:\s+(\d+)\s+Id:\s+(\d+)\s+Lun:\s+(\d+)/ && $dscsi) {
				$dscsi->{'host'} = $1;
				$dscsi->{'bus'} = $2;
				$dscsi->{'target'} = $3;
				$dscsi->{'lun'} = $4;
				}
			}
		$dscsi_mode = 1;
		}
	else {
		# Standard format
		foreach (@lines) {
			s/\s/ /g;
			if (/^Host:/) {
				push(@pscsi, $_);
				}
			elsif (/^\s+\S/ && @pscsi) {
				$pscsi[$#pscsi] .= $_;
				}
			}
		@pscsi = grep { /Type:\s+Direct-Access/i } @pscsi;
		$dscsi_mode = 0;
		}
	}

local (@disks, @devs, $d);
if (open(PARTS, "/proc/partitions")) {
	# The list of all disks can come from the kernel
	local $sc = 0;
	while(<PARTS>) {
		if (/\d+\s+\d+\s+\d+\s+sd([a-z]+)\s/ ||
		    /\d+\s+\d+\s+\d+\s+(scsi\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/disc)\s+/) {
			# New or old style SCSI device
			local $d = $1;
			local ($host, $bus, $target, $lun) = ($2, $3, $4, $5);
			if (!$dscsi_mode && $pscsi[$sc] =~ /USB-FDU/) {
				# USB floppy with scsi emulation!
				splice(@pscsi, $sc, 1);
				next;
				}
			if ($host ne '') {
				local $scsidev = "/dev/$d";
				if (!-r $scsidev) {
					push(@devs, "/dev/".
						  $self->number_to_device("sd", $sc));
					}
				else {
					push(@devs, $scsidev);
					}
				}
			else {
				push(@devs, "/dev/sd$d");
				}
			$sc++;
			}
		elsif (/\d+\s+\d+\s+\d+\s+hd([a-z]+)\s/) {
			# IDE disk (but skip CDs)
			local $n = $1;
			if (open(MEDIA, "/proc/ide/hd$n/media")) {
				local $media = <MEDIA>;
				close(MEDIA);
				if ($media =~ /^disk/ && !$_[0]) {
					push(@devs, "/dev/hd$n");
					}
				}
			}
		elsif (/\d+\s+\d+\s+\d+\s+(ide\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/disc)\s+/) {
			# New-style IDE disk
			local $idedev = "/dev/$1";
			local ($host, $bus, $target, $lun) = ($2, $3, $4, $5);
			if (!-r $idedev) {
				push(@devs, "/dev/".
				    $self->hbt_to_device($host, $bus, $target));
				}
			else {
				push(@devs, "/dev/$1");
				}
			}
		elsif (/\d+\s+\d+\s+\d+\s+(rd\/c(\d+)d\d+)\s/) {
			# Mylex raid device
			push(@devs, "/dev/$1");
			}
		elsif (/\d+\s+\d+\s+\d+\s+(ida\/c(\d+)d\d+)\s/) {
			# Compaq raid device
			push(@devs, "/dev/$1");
			}
		elsif (/\d+\s+\d+\s+\d+\s+(cciss\/c(\d+)d\d+)\s/) {
			# Compaq Smart Array RAID
			push(@devs, "/dev/$1");
			}
		elsif (/\d+\s+\d+\s+\d+\s+(ataraid\/disc(\d+)\/disc)\s+/) {
			# Promise raid controller
			push(@devs, "/dev/$1");
			}
		}
	close(PARTS);
	@devs = sort { ($b =~ /\/hd[a-z]+$/ ? 1 : 0) <=>
		       ($a =~ /\/hd[a-z]+$/ ? 1 : 0) } @devs;
	}

# Skip cd-rom drive, identified from symlink. Don't do this if we can identify
# cds by their media type though
if (!-d "/proc/ide") {
	local @cdstat = stat("/dev/cdrom");
	if (@cdstat && !$_[0]) {
		@devs = grep { (stat($_))[1] != $cdstat[1] } @devs;
		}
	}

# Call fdisk to get partition and geometry information
local $devs = join(" ", @devs);
local ($disk, $m2);
open(FDISK, "fdisk -l $devs 2>/dev/null |");
while(<FDISK>) {
	if (/Disk\s+([^ :]+):\s+(\d+)\s+\S+\s+(\d+)\s+\S+\s+(\d+)/ ||
	    ($m2 = ($_ =~ /Disk\s+([^ :]+):\s+(.*)\s+bytes/))) {
		# New disk section
		if ($m2) {
			$disk = { 'device' => $1,
				  'prefix' => $1 };
			<FDISK> =~ /(\d+)\s+\S+\s+(\d+)\s+\S+\s+(\d+)/ || next;
			$disk->{'heads'} = $1;
			$disk->{'sectors'} = $2;
			$disk->{'cylinders'} = $3;
			}
		else {
			$disk = { 'device' => $1,
				  'prefix' => $1,
				  'heads' => $2,
				  'sectors' => $3,
				  'cylinders' => $4 };
			}
		$disk->{'index'} = scalar(@disks);
		$disk->{'parts'} = [ ];
		local @st = stat($disk->{'device'});
		next if (@cdstat && $st[1] == $cdstat[1]);
		if ($disk->{'device'} =~ /\/sd([a-z]+)$/) {
			# Old-style SCSI disk
			$disk->{'desc'} = $self->text('select_device', 'SCSI',uc($1));
			local ($dscsi) = grep { $_->{'dev'} eq "sd$1" } @dscsi;
			$disk->{'scsi'} = $dscsi ? $self->indexof($dscsi, @dscsi)
						 : ord(uc($1))-65;
			$disk->{'type'} = 'scsi';
			}
		elsif ($disk->{'device'} =~ /\/hd([a-z]+)$/) {
			# IDE disk
			$disk->{'desc'} = $self->text('select_device', 'IDE', uc($1));
			$disk->{'type'} = 'ide';
			}
		elsif ($disk->{'device'} =~ /\/(scsi\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/disc)/) {
			# New complete SCSI disk specification
			$disk->{'host'} = $2;
			$disk->{'bus'} = $3;
			$disk->{'target'} = $4;
			$disk->{'lun'} = $5;
			$disk->{'desc'} = $self->text('select_scsi',
						"$2", "$3", "$4", "$5");

			# Work out the SCSI index for this disk
			local $j;
			if ($dscsi_mode) {
				for($j=0; $j<@dscsi; $j++) {
					if ($dscsi[$j]->{'host'} == $disk->{'host'} && $dscsi[$j]->{'bus'} == $disk->{'bus'} && $dscsi[$j]->{'target'} == $disk->{'target'} && $dscsi[$j]->{'lnun'} == $disk->{'lun'}) {
						$disk->{'scsi'} = $j;
						last;
						}
					}
				}
			else {
				for($j=0; $j<@pscsi; $j++) {
					if ($pscsi[$j] =~ /Host:\s+scsi(\d+).*Id:\s+(\d+)/i && $disk->{'host'} == $1 && $disk->{'target'} == $2) {
						$disk->{'scsi'} = $j;
						last;
						}
					}
				}
			$disk->{'type'} = 'scsi';
			$disk->{'prefix'} =~ s/disc$/part/g;
			}
		elsif ($disk->{'device'} =~ /\/(ide\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/disc)/) {
			# New-style IDE specification
			$disk->{'host'} = $2;
			$disk->{'bus'} = $3;
			$disk->{'target'} = $4;
			$disk->{'lun'} = $5;
			$disk->{'desc'} = $self->text('select_newide',
						"$2", "$3", "$4", "$5");
			$disk->{'type'} = 'ide';
			$disk->{'prefix'} =~ s/disc$/part/g;
			}
		elsif ($disk->{'device'} =~ /\/(rd\/c(\d+)d(\d+))/) {
			# Mylex raid device
			local ($mc, $md) = ($2, $3);
			$disk->{'desc'} = $self->text('select_mylex', $mc, $md);
			open(RD, "/proc/rd/c$mc/current_status");
			while(<RD>) {
				if (/^Configuring\s+(.*)/i) {
					$disk->{'model'} = $1;
					}
				elsif (/\s+(\S+):\s+([^, ]+)/ &&
				       $1 eq $disk->{'device'}) {
					$disk->{'raid'} = $2;
					}
				}
			close(RD);
			$disk->{'type'} = 'raid';
			$disk->{'prefix'} = $disk->{'device'}.'p';
			}
		elsif ($disk->{'device'} =~ /\/(ida\/c(\d+)d(\d+))/) {
			# Compaq RAID device
			local ($ic, $id) = ($2, $3);
			$disk->{'desc'} = $self->text('select_cpq', $ic, $id);
			open(IDA, -d "/proc/driver/array" ? "/proc/driver/array/ida$ic" : "/proc/driver/cpqarray/ida$ic");
			while(<IDA>) {
				if (/^(\S+):\s+(.*)/ && $1 eq "ida$ic") {
					$disk->{'model'} = $2;
					}
				}
			close(IDA);
			$disk->{'type'} = 'raid';
			$disk->{'prefix'} = $disk->{'device'}.'p';
			}
		elsif ($disk->{'device'} =~ /\/(cciss\/c(\d+)d(\d+))/) {
			# Compaq Smart Array RAID
			local ($ic, $id) = ($2, $3);
			$disk->{'desc'} = $self->text('select_smart', $ic, $id);
			open(CCI, "/proc/driver/cciss/cciss$ic");
			while(<CCI>) {
				if (/^\s*(\S+):\s*(.*)/ && $1 eq "cciss$ic") {
					$disk->{'model'} = $2;
					}
				}
			close(CCI);
			$disk->{'type'} = 'raid';
			$disk->{'prefix'} = $disk->{'device'}.'p';
			}
		elsif ($disk->{'device'} =~ /\/(ataraid\/disc(\d+)\/disc)/) {
			# Promise RAID controller
			local $dd = $2;
			$disk->{'desc'} = $self->text('select_promise', $dd);
			$disk->{'type'} = 'raid';
			$disk->{'prefix'} =~ s/disc$/part/g;
			}
		push(@disks, $disk);
		}
	elsif (/^Units\s+=\s+cylinders\s+of\s+(\d+)\s+\*\s+(\d+)/) {
		# Unit size for disk
		$disk->{'bytes'} = $2;
		$disk->{'cylsize'} = $disk->{'heads'} * $disk->{'sectors'} *
				     $disk->{'bytes'};
		}
	elsif (/(\/dev\/\S+?(\d+))[ \t*]+\d+\s+(\d+)\s+(\d+)\s+(\S+)\s+(\S{1,2})\s+(.*)/ || /(\/dev\/\S+?(\d+))[ \t*]+(\d+)\s+(\d+)\s+(\S+)\s+(\S{1,2})\s+(.*)/) {
		# Partition within the current disk
		local $part = { 'number' => $2,
				'device' => $1,
				'type' => $6,
				'start' => $3,
				'end' => $4,
				'blocks' => int($5),
				'extended' => $6 eq '5' || $6 eq 'f' ? 1 : 0,
				'index' => scalar(@{$disk->{'parts'}}) };
		$part->{'desc'} =
			$part->{'device'} =~ /(.)d(\S)(\d+)$/ ?
			 $self->text('select_part', $1 eq 's' ? 'SCSI' : 'IDE', uc($2), "$3") :
			$part->{'device'} =~ /scsi\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/part(\d+)/ ?
			 $self->text('select_spart', "$1", "$2", "$3", "$4", "$5") :
			$part->{'device'} =~ /ide\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/part(\d+)/ ?
			 $self->text('select_snewide', "$1", "$2", "$3", "$4", "$5") :
			$part->{'device'} =~ /rd\/c(\d+)d(\d+)p(\d+)$/ ? 
			 $self->text('select_mpart', "$1", "$2", "$3") :
			$part->{'device'} =~ /ida\/c(\d+)d(\d+)p(\d+)$/ ? 
			 $self->text('select_cpart', "$1", "$2", "$3") :
			$part->{'device'} =~ /cciss\/c(\d+)d(\d+)p(\d+)$/ ? 
			 $self->text('select_smartpart', "$1", "$2", "$3") :
			$part->{'device'} =~ /ataraid\/disc(\d+)\/part(\d+)$/ ?
			 $self->text('select_ppart', "$1", "$2") :
			"???",
		push(@{$disk->{'parts'}}, $part);
		}
	}
close(FDISK);

# Check /proc/ide for IDE disk models
foreach $d (@disks) {
	if ($d->{'type'} eq 'ide') {
		local $short;
		if (defined($d->{'host'})) {
			$short = $self->hbt_to_device($d->{'host'},
						$d->{'bus'},
						$d->{'target'});
			}
		else {
			$short = $d->{'device'};
			$short =~ s/^.*\///g;
			}
		if (open(MODEL, "/proc/ide/$short/model")) {
			($d->{'model'} = <MODEL>) =~ s/\r|\n//g;
			close(MODEL);
			}
		if (open(MEDIA, "/proc/ide/$short/media")) {
			($d->{'media'} = <MEDIA>) =~ s/\r|\n//g;
			close(MEDIA);
			}
		}
	}

# Fill in SCSI information
foreach $d (@disks) {
	if ($d->{'type'} eq 'scsi') {
		local $s = $d->{'scsi'};
		if ($dscsi_mode) {
			# From other scsi files
			$d->{'model'} = "$dscsi[$s]->{'make'} $dscsi[$s]->{'model'}";
			$d->{'controller'} = $dscsi[$s]->{'host'};
			$d->{'scsiid'} = $dscsi[$s]->{'target'};
			}
		else {
			# From /proc/scsi/scsi lines
			if ($pscsi[$s] =~ /Vendor:\s+(\S+).*Model:\s+(.*)\s+Rev:/i) {
				$d->{'model'} = "$1 $2";
				}
			if ($pscsi[$s] =~ /Host:\s+scsi(\d+).*Id:\s+(\d+)/i) {
				$d->{'controller'} = int($1);
				$d->{'scsiid'} = int($2);
				}
			}
		}
	}

@{$self->{'list_disks_partitions_cache_array'}} = @disks;
return @disks;
}

# hbt_to_device(host, bus, target)
# Converts an IDE device specified as a host, bus and target to an hdX device
sub hbt_to_device
{
	my $self = shift;

local ($host, $bus, $target) = @_;
local $num = $host*4 + $bus*2 + $target;
return $self->number_to_device("hd", $num);
}

# number_to_device(suffix, number)
sub number_to_device
{
	my $self = shift;

local ($suffix, $num) = @_;
if ($num < 26) {
	# Just a single letter
	return $suffix.(('a' .. 'z')[$num]);
	}
else {
	# Two-letter format
	local $first = int($num / 26);
	local $second = $num % 26;
	return $suffix.(('a' .. 'z')[$first]).(('a' .. 'z')[$second]);
	}
}

# change_type(disk, partition, type)
# Changes the type of an existing partition
sub change_type
{
	my $self = shift;

$self->open_fdisk("$_[0]");
$self->wprint("t\n");
local $rv = $self->wait_for($fh, 'Partition.*:', 'Selected partition');
$self->wprint("$_[1]\n") if ($rv == 0);
$self->wait_for($fh, 'Hex.*:');
$self->wprint("$_[2]\n");
$self->wait_for($fh, 'Command.*:');
$self->wprint("w\n"); sleep(1);
$self->close_fdisk();
undef(@{$self->{'list_disks_partitions_cache_array'}});
}

# delete_partition(disk, partition)
# Delete an existing partition
sub delete_partition
{
	my $self = shift;

$self->open_fdisk("$_[0]");
$self->wprint("d\n");
local $rv = $self->wait_for($fh, 'Partition.*:', 'Selected partition');
$self->wprint("$_[1]\n") if ($rv == 0);
$self->wait_for($fh, 'Command.*:');
$self->wprint("w\n");
$self->wait_for($fh, 'Syncing'); sleep(3);
$self->close_fdisk();
undef(@{$self->{'list_disks_partitions_cache_array'}});
}

# create_partition(disk, partition, start, end, type)
# Create a new partition with the given extent and type
sub create_partition
{
	my $self = shift;

$self->open_fdisk("$_[0]");
$self->wprint("n\n");
local $wf = $self->wait_for($fh, 'primary.*\r\n', 'First.*:');
if ($_[1] > 4) {
	$self->wprint("l\n");
	}
else {
	$self->wprint("p\n");
	local $wf2 = $self->wait_for($fh, 'Partition.*:', 'Selected partition');
	$self->wprint("$_[1]\n") if ($wf2 == 0);
	}
$self->wait_for($fh, 'First.*:') if ($wf != 1);
$self->wprint("$_[2]\n");
$self->wait_for($fh, 'Last.*:');
$self->wprint("$_[3]\n");
$self->wait_for($fh, 'Command.*:');

$self->wprint("t\n");
local $rv = $self->wait_for($fh, 'Partition.*:', 'Selected partition');
$self->wprint("$_[1]\n") if ($rv == 0);
$self->wait_for($fh, 'Hex.*:');
$self->wprint("$_[4]\n");
$self->wait_for($fh, 'Command.*:');
$self->wprint("w\n");
$self->wait_for($fh, 'Syncing'); sleep(3);
$self->close_fdisk();
undef(@{$self->{'list_disks_partitions_cache_array'}});
}

# create_extended(disk, partition, start, end)
sub create_extended
{
	my $self = shift;

$self->open_fdisk("$_[0]");
$self->wprint("n\n");
$self->wait_for($fh, 'primary.*\r\n');
$self->wprint("e\n");
$self->wait_for($fh, 'Partition.*:');
$self->wprint("$_[1]\n");
$self->wait_for($fh, 'First.*:');
$self->wprint("$_[2]\n");
$self->wait_for($fh, 'Last.*:');
$self->wprint("$_[3]\n");
$self->wait_for($fh, 'Command.*:');

$self->wprint("w\n");
$self->wait_for($fh, 'Syncing'); sleep(3);
$self->close_fdisk();
undef(@{$self->{'list_disks_partitions_cache_array'}});
}

# list_tags()
# Returns a list of known partition tag numbers
sub list_tags
{
	my $self = shift;

return sort { hex($a) <=> hex($b) } (keys %{$self->{'tags_hash'}});
}

# tag_name(tag)
# Returns a human-readable version of a tag
sub tag_name
{
	my $self = shift;

return $self->{'tags_hash'}->{$_[0]} ? $self->{'tags_hash'}->{$_[0]}
		    : $self->{'hidden_tags_hash'}->{$_[0]};
}

# conv_type(tag)
# Given a partition tag, returns the filesystem type (assuming it is supported)
sub conv_type
{
	my $self = shift;

local @rv;
if ($_[0] eq "4" || $_[0] eq "6" ||
    $_[0] eq "1" || $_[0] eq "e") { @rv = ( "msdos" ); }
elsif ($_[0] eq "b" || $_[0] eq "c") { @rv = ( "vfat" ); }
elsif ($_[0] eq "83") { @rv = ( "ext2", "ext3" ); }
elsif ($_[0] eq "82") { @rv = ( "swap" ); }
elsif ($_[0] eq "81") { @rv = ( "minix" ); }
else { return ( ); }
local %supp = map { $_, 1 } $main::mount_object->list_fstypes();
@rv = grep { $supp{$_} } @rv;
return wantarray ? @rv : $rv[0];
}

# fstype_name(type)
# Returns a readable name for a filesystem type
sub fstype_name
{
	my $self = shift;

return $self->{'text_hash'}->{"fs_".$_[0]};
}

sub mkfs_options
{
	my $self = shift;

if ($_[0] eq "ext2") {
	$self->opt_input("ext2_b", $self->{'text_hash'}->{'bytes'}, 1);
	$self->opt_input("ext2_f", $self->{'text_hash'}->{'bytes'}, 0);
	$self->opt_input("ext2_i", "", 1);
	$self->opt_input("ext2_m", "%", 0);
	$self->opt_input("ext2_g", "", 1);
	print "<td align=right><b>$self->{'text_hash'}->{'ext2_c'}</b></td>\n";
	print "<td><input type=radio name=ext2_c value=1> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=ext2_c value=0 checked> $self->{'text_hash'}->{'no'}",
	      "</td> </tr>\n";
	}
elsif ($_[0] eq "msdos" || $_[0] eq "vfat") {
	$self->opt_input("msdos_ff", "", 1);
	print "<td align=right><b>$self->{'text_hash'}->{'msdos_F'}</b></td>\n";
	print "<td>",$self->ui_select("msdos_F", undef,
			[ [ undef, $self->{'text_hash'}->{'default'} ],
			  [ 12 ], [ 16 ], [ 32 ],
			  [ "*", $self->{'text_hash'}->{'msdos_F_other'} ] ])," ",
		     $self->ui_textbox("msdos_F_other", undef, 4),"</td> </tr>\n";
	$self->opt_input("msdos_i", "", 1);
	$self->opt_input("msdos_n", "", 0);
	$self->opt_input("msdos_r", "", 1);
	$self->opt_input("msdos_s", "sectors", 0);
	print "<tr> <td align=right><b>$self->{'text_hash'}->{'msdos_c'}</b></td>\n";
	print "<td><input type=radio name=msdos_c value=1> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=msdos_c value=0 checked> $self->{'text_hash'}->{'no'}",
	      "</td> </tr>\n";
	}
elsif ($_[0] eq "minix") {
	$self->opt_input("minix_n", "", 1);
	$self->opt_input("minix_i", "", 0);
	$self->opt_input("minix_b", "", 1);
	print "<td align=right><b>$self->{'text_hash'}->{'minix_c'}</b></td>\n";
	print "<td><input type=radio name=msdos_c value=1> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=msdos_c value=0 checked> $self->{'text_hash'}->{'no'}",
	      "</td> </tr>\n";
	}
elsif ($_[0] eq "reiserfs") {
	print "<tr> <td><b>$self->{'text_hash'}->{'reiserfs_force'}</b></td>\n";
	print "<td><input type=radio name=reiserfs_f value=1> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=reiserfs_f value=0 checked> ",
	      "$self->{'text_hash'}->{'no'}</td>\n";

	print "<td><b>$self->{'text_hash'}->{'reiserfs_hash'}</b></td>\n";
	print "<td><select name=reiserfs_h>\n";
	print "<option value='' checked>$self->{'text_hash'}->{'default'}\n";
	print "<option>rupasov\n";
	print "<option>tea\n";
	print "</select></td> </tr>\n";
	}
elsif ($_[0] eq "ext3") {
	$self->opt_input("ext2_b", $self->{'text_hash'}->{'bytes'}, 1);
	$self->opt_input("ext2_f", $self->{'text_hash'}->{'bytes'}, 0);
	$self->opt_input("ext2_i", "", 1);
	$self->opt_input("ext2_m", "%", 0);
	$self->opt_input("ext3_j", "MB", 1);
	print "<td align=right><b>$self->{'text_hash'}->{'ext2_c'}</b></td>\n";
	print "<td><input type=radio name=ext2_c value=1> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=ext2_c value=0 checked> $self->{'text_hash'}->{'no'}",
	      "</td> </tr>\n";
	}
elsif ($_[0] eq "xfs") {
	print "<tr> <td><b>$self->{'text_hash'}->{'xfs_force'}</b></td>\n";
	print "<td><input type=radio name=xfs_f value=1> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=xfs_f value=0 checked> ",
	      "$self->{'text_hash'}->{'no'}</td>\n";

	$self->opt_input("xfs_b", $self->{'text_hash'}->{'bytes'}, 0);
	}
elsif ($_[0] eq "jfs") {
	$self->opt_input("jfs_s", $self->{'text_hash'}->{'megabytes'}, 1);
	print "<td align=right><b>$self->{'text_hash'}->{'jfs_c'}</b></td>\n";
	print "<td><input type=radio name=jfs_c value=1> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=jfs_c value=0 checked> $self->{'text_hash'}->{'no'}",
	      "</td> </tr>\n";
	}
elsif ($_[0] eq "fatx") {
	# Has no options!
	print "<tr> <td colspan=4>$self->{'text_hash'}->{'fatx_none'}</td> </tr>\n";
	}
}

# mkfs_parse(type, device)
# Returns a command to build a new filesystem of the given type on the
# given device. Options are taken from %{$self->{'in_hash'}}.
sub mkfs_parse
{
	my $self = shift;

local($cmd);
if ($_[0] eq "ext2") {
	$cmd = "mkfs -t ext2";
	$cmd .= $self->opt_check("ext2_b", '\d+', "-b");
	$cmd .= $self->opt_check("ext2_f", '\d+', "-f");
	$cmd .= $self->opt_check("ext2_i", '\d{4,}', "-i");
	$cmd .= $self->opt_check("ext2_m", '\d+', "-m");
	$cmd .= $self->opt_check("ext2_g", '\d+', "-g");
	$cmd .= $self->{'in_hash'}->{'ext2_c'} ? " -c" : "";
	$cmd .= " $_[1]";
	}
elsif ($_[0] eq "msdos" || $_[0] eq "vfat") {
	$cmd = "mkfs -t $_[0]";
	$cmd .= $self->opt_check("msdos_ff", '[1-2]', "-f");
	if ($self->{'in_hash'}->{'msdos_F'} eq '*') {
		$self->{'in_hash'}->{'msdos_F_other'} =~ /^\d+$/ ||
			$self->error($self->text('opt_error', $self->{'in_hash'}->{'msdos_F_other'},
						  $self->{'text_hash'}->{'msdos_F'}));
		$cmd .= " -F ".$self->{'in_hash'}->{'msdos_F_other'};
		}
	elsif ($self->{'in_hash'}->{'msdos_F'}) {
		$cmd .= " -F ".$self->{'in_hash'}->{'msdos_F'};
		}
	$cmd .= $self->opt_check("msdos_i", '[0-9a-f]{8}', "-i");
	$cmd .= $self->opt_check("msdos_n", '\S{1,11}', "-n");
	$cmd .= $self->opt_check("msdos_r", '\d+', "-r");
	$cmd .= $self->opt_check("msdos_s", '\d+', "-s");
	$cmd .= $self->{'in_hash'}->{'msdos_c'} ? " -c" : "";
	$cmd .= " $_[1]";
	}
elsif ($_[0] eq "minix") {
	local(@plist, $disk, $part, $i, @pinfo);
	$cmd = "mkfs -t minix";
	$cmd .= $self->opt_check("minix_n", '14|30', "-n ");
	$cmd .= $self->opt_check("minix_i", '\d+', "-i ");
	$cmd .= $self->{'in_hash'}->{'minix_c'} ? " -c" : "";
	$cmd .= $self->opt_check("minix_b", '\d+', " ");
	$cmd .= " $_[1]";
	}
elsif ($_[0] eq "reiserfs") {
	$cmd = "yes | mkreiserfs";
	$cmd .= " -f" if ($self->{'in_hash'}->{'reiserfs_f'});
	$cmd .= " -h $self->{'in_hash'}->{'reiserfs_h'}" if ($self->{'in_hash'}->{'reiserfs_h'});
	$cmd .= " $_[1]";
	}
elsif ($_[0] eq "ext3") {
	if ($self->has_command("mkfs.ext3")) {
		$cmd = "mkfs -t ext3";
		$cmd .= $self->opt_check("ext3_j", '\d+', "-j");
		}
	elsif ($self->has_command("mke3fs")) {
		$cmd = "mke3fs";
		$cmd .= $self->opt_check("ext3_j", '\d+', "-j");
		}
	else {
		$cmd = "mkfs.ext2 -j";
		if (!$self->{'in_hash'}->{'ext3_j_def'}) {
			$self->{'in_hash'}->{'ext3_j'} =~ /^\d+$/ ||
				$self->error($self->text('opt_error', $self->{'in_hash'}->{'ext3_j'},
					     $self->{'text_hash'}->{'ext3_j'}));
			$cmd .= " -J size=$self->{'in_hash'}->{'ext3_j'}";
			}
		}
	$cmd .= $self->opt_check("ext2_b", '\d+', "-b");
	$cmd .= $self->opt_check("ext2_f", '\d+', "-f");
	$cmd .= $self->opt_check("ext2_i", '\d{4,}', "-i");
	$cmd .= $self->opt_check("ext2_m", '\d+', "-m");
	$cmd .= $self->{'in_hash'}->{'ext2_c'} ? " -c" : "";
	$cmd .= " $_[1]";
	}
elsif ($_[0] eq "xfs") {
	$cmd = "mkfs -t $_[0]";
	$cmd .= " -f" if ($self->{'in_hash'}->{'xfs_f'});
	$cmd .= " -b size=$self->{'in_hash'}->{'xfs_b'}" if (!$self->{'in_hash'}->{'xfs_b_def'});
	$cmd .= " $_[1]";
	}
elsif ($_[0] eq "jfs") {
	$cmd = "mkfs -t $_[0] -q";
	$cmd .= $self->opt_check("jfs_s", '\d+', "-s");
	$cmd .= " -c" if ($self->{'in_hash'}->{'jfs_c'});
	$cmd .= " $_[1]";
	}
elsif ($_[0] eq "fatx") {
	$cmd = "mkfs -t $_[0] $_[1]";
	}
return $cmd;
}

# can_tune(type)
# Returns 1 if this filesystem type can be tuned
sub can_tune
{
	my $self = shift;

return $_[0] eq "ext2" || $_[0] eq "ext3";
}

# tunefs_options(type)
# Output HTML for tuning options for some filesystem type
sub tunefs_options
{
	my $self = shift;

if ($_[0] eq "ext2" || $_[0] eq "ext3") {
	$self->opt_input("tunefs_c", "", 1);

	print "<td align=right><b>$self->{'text_hash'}->{'tunefs_e'}</b></td> <td>\n";
	print "<input type=radio name=tunefs_e_def value=1 checked> ",
	      "$self->{'text_hash'}->{'opt_default'}\n";
	print "&nbsp; <input type=radio name=tunefs_e_def value=0>\n";
	print "<select name=tunefs_e>\n";
	print "<option value=continue> $self->{'text_hash'}->{'tunefs_continue'}\n";
	print "<option value=remount-ro> $self->{'text_hash'}->{'tunefs_remount'}\n";
	print "<option value=panic> $self->{'text_hash'}->{'tunefs_panic'}\n";
	print "</select></td> </tr>\n";

	print "<tr> <td align=right><b>$self->{'text_hash'}->{'tunefs_u'}</b></td> <td>\n";
	print "<input type=radio name=tunefs_u_def value=1 checked> ",
	      "$self->{'text_hash'}->{'opt_default'}\n";
	print "&nbsp; <input type=radio name=tunefs_u_def value=0>\n";
	print "<input name=tunefs_u size=8> ",
	      $self->user_chooser_button("tunefs_u", 0),"</td>\n";

	print "<td align=right><b>$self->{'text_hash'}->{'tunefs_g'}</b></td> <td>\n";
	print "<input type=radio name=tunefs_g_def value=1 checked> ",
	      "$self->{'text_hash'}->{'opt_default'}\n";
	print "&nbsp; <input type=radio name=tunefs_g_def value=0>\n";
	print "<input name=tunefs_g size=8> ",
	      $self->group_chooser_button("tunefs_g", 0),"</td> </tr>\n";

	$self->opt_input("tunefs_m", "%", 1);
	$self->{'tsel'} = "<select name=tunefs_i_unit>\n".
		"<option value=d> $self->{'text_hash'}->{'tunefs_days'}\n".
		"<option value=w> $self->{'text_hash'}->{'tunefs_weeks'}\n".
		"<option value=m> $self->{'text_hash'}->{'tunefs_months'}\n".
		"</select>\n";
	$self->opt_input("tunefs_i", $self->{'tsel'}, 0);
	}
}

# tunefs_parse(type, device)
# Returns the tuning command based on user inputs
sub tunefs_parse
{
	my $self = shift;

if ($_[0] eq "ext2" || $_[0] eq "ext3") {
	$self->{'cmd'} = "tune2fs";
	$self->{'cmd'} .= $self->opt_check("tunefs_c", '\d+', "-c");
	$self->{'cmd'} .= $self->{'in_hash'}->{'tunefs_e_def'} ? "" : " -e$self->{'in_hash'}->{'tunefs_e'}";
	$self->{'cmd'} .= $self->{'in_hash'}->{'tunefs_u_def'} ? "" : " -u".getpwnam($self->{'in_hash'}->{'tunefs_u'});
	$self->{'cmd'} .= $self->{'in_hash'}->{'tunefs_g_def'} ? "" : " -g".getgrnam($self->{'in_hash'}->{'tunefs_g'});
	$self->{'cmd'} .= $self->opt_check("tunefs_m",'\d+',"-m");
	$self->{'cmd'} .= $self->opt_check("tunefs_i", '\d+', "-i").
		($self->{'in_hash'}->{'tunefs_i_def'} ? "" : $self->{'in_hash'}->{'tunefs_i_unit'});
	$self->{'cmd'} .= " $_[1]";
	}
return $self->{'cmd'};
}

# need_reboot(disk)
# Returns 1 if a reboot is needed after changing the partitions on some disk
sub need_reboot
{
	my $self = shift;

local $un = `uname -r`;
return $un =~ /^2\.0\./ || $un =~ /^1\./ || $un =~ /^0\./;
}

# device_status(device)
# Returns an array of  directory, type, mounted
sub device_status
{
	my $self = shift;

@{$self->{'mounted_array'}} = $self->foreign_call("mount", "list_mounted") if (!@{$self->{'mounted_array'}});
@{$self->{'mounts_array'}} = $self->foreign_call("mount", "list_mounts") if (!@{$self->{'mounts_array'}});
local $label = $self->get_label($_[0]);
local $volid = $self->get_volid($_[0]);

local ($mounted) = grep { $self->same_file($_->[1], $_[0]) ||
			  $_->[1] eq "LABEL=$label" ||
			  $_->[1] eq "UUID=$volid" } @{$self->{'mounted_array'}};
local ($mount) = grep { $self->same_file($_->[1], $_[0]) ||
			$_->[1] eq "LABEL=$label" ||
			$_->[1] eq "UUID=$volid" } @{$self->{'mounts_array'}};
if ($mounted) { return ($mounted->[0], $mounted->[2], 1,
			$self->indexof($mount, @{$self->{'mounts_array'}}),
			$self->indexof($mounted, @{$self->{'mounted_array'}})); }
elsif ($mount) { return ($mount->[0], $mount->[2], 0,
			 $self->indexof($mount, @{$self->{'mounts_array'}})); }
if ($self->{'raid_module'}) {
	$self->{'raidconf'} = $self->foreign_call("raid", "get_raidtab") if (!$self->{'raidconf'});
	foreach $c (@{$self->{'raidconf'}}) {
		foreach $d ($main::raid_object->find_value('device', $c->{'members'})) {
			return ( $c->{'value'}, "raid", 1 ) if ($d eq $_[0]);
			}
		}
	}
if ($self->{'lvm_module'}) {
	if (!defined(@{$self->{'physical_volumes_array'}})) {
		@{$self->{'physical_volumes_array'}} = ();
		foreach $vg ($self->foreign_call("lvm", "list_volume_groups")) {
			push(@{$self->{'physical_volumes_array'}},
				$self->foreign_call("lvm", "list_physical_volumes",
						     $vg->{'name'}));
			}
		}
	foreach $pv (@{$self->{'physical_volumes_array'}}) {
		return ( $pv->{'vg'}, "lvm", 1)
			if ($pv->{'device'} eq $_[0]);
		}
	}
return ();
}

# can_fsck(type)
# Returns 1 if some filesystem type can fsck'd
sub can_fsck
{
	my $self = shift;

return ($_[0] eq "ext2" && $self->has_command("fsck.ext2") ||
	$_[0] eq "ext3" && $self->has_command("fsck.ext3") ||
	$_[0] eq "minix" && $self->has_command("fsck.minix"));
}

# fsck_command(type, device)
# Returns the fsck command to unconditionally check a filesystem
sub fsck_command
{
	my $self = shift;

if ($_[0] eq "ext2" || $_[0] eq "ext3") {
	return "fsck -t $_[0] -p $_[1]";
	}
elsif ($_[0] eq "minix") {
	return "fsck -t minix -a $_[1]";
	}
}

# fsck_error(code)
# Returns a description of an exit code from fsck
sub fsck_error
{
	my $self = shift;

return $self->{'text_hash'}->{"fsck_err$_[0]"} ? $self->{'text_hash'}->{"fsck_err$_[0]"}
			      : $self->text("fsck_unknown", $_[0]);
}

# partition_select(name, value, mode, [&found], [disk_regexp])
# Returns HTML for selecting a disk or partition
# mode 0 = floppies and disk partitions
#      1 = disks
#      2 = floppies and disks and disk partitions
#      3 = disk partitions
sub partition_select
{
	my $self = shift;

local $rv = "<select name=$_[0]>\n";
local ($found, $d, $p);
if ($_[2] == 0 || $_[2] == 2) {
	$rv .= sprintf "<option %s value=/dev/fd0>%s\n",
		$_[1] eq "/dev/fd0" ? "selected" : "",
		$self->text('select_fd', 0) if (!$_[4] || "/dev/fd0" =~ /$_[4]/);
	$rv .= sprintf "<option %s value=/dev/fd1>%s\n",
		$_[1] eq "/dev/fd1" ? "selected" : "",
		$self->text('select_fd', 1) if (!$_[4] || "/dev/fd1" =~ /$_[4]/);
	$found++ if ($_[1] =~ /^\/dev\/fd[01]$/);
	}
local @dlist = $self->list_disks_partitions();
foreach $d (@dlist) {
	local $dev = $d->{'device'};
	next if ($_[4] && $dev !~ /$_[4]/);
	if ($_[2] == 1 || $_[2] == 2) {
		local $name = $d->{'desc'};
		$name .= " ($d->{'model'})" if ($d->{'model'});
		$rv .= sprintf "<option value=%s %s>%s\n",
			$dev, $_[1] eq $dev ? "selected" : "", $name;
		$found++ if ($dev eq $_[1]);
		}
	if ($_[2] == 0 || $_[2] == 2 || $_[2] == 3) {
		foreach $p (@{$d->{'parts'}}) {
			next if ($p->{'extended'});
			local $name = $p->{'desc'};
			$name .= " (".$self->tag_name($p->{'type'}).")"
				if ($self->tag_name($p->{'type'}));
			$rv .= sprintf "<option %s value=%s>%s\n",
				  $_[1] eq $p->{'device'} ? "selected" : "",
				  $p->{'device'}, $name;
			$found++ if ($_[1] eq $p->{'device'});
			}
		}
	}
if (!$found && $_[1] && !$_[3]) {
	$rv .= "<option selected>$_[1]\n";
	}
if ($_[3]) {
	${$_[3]} = $found;
	}
$rv .= "</select>\n";
return $rv;
}

# label_select(name, value, &found)
# Returns HTML for selecting a filesystem label
sub label_select
{
	my $self = shift;

local $rv = "<select name=$_[0]>\n";
local @dlist = $self->list_disks_partitions();
local $any;
foreach $d (@dlist) {
	local $dev = $d->{'device'};
	foreach $p (@{$d->{'parts'}}) {
		next if ($p->{'type'} ne '83');
		local $label = $self->get_label($p->{'device'});
		next if (!$label);
		$rv .= sprintf "<option %s value=%s>%s (%s)\n",
			  $_[1] eq $label ? "selected" : "",
			  $label, $label, $p->{'desc'};
		${$_[2]}++ if ($_[1] eq $label);
		$any++;
		}
	}
if (!$self->{'found'} && $_[1] && !$_[2]) {
	$rv .= "<option selected>$_[1]\n";
	}
$rv .= "</select>\n";
return $any ? $rv : undef;
}

# volid_select(name, value, &found)
# Returns HTML for selecting a filesystem UUID
sub volid_select
{
	my $self = shift;

local ($name, $value, $found) = @_;
local @dlist = $self->list_disks_partitions();
local @opts;
foreach my $d (@dlist) {
	local $dev = $d->{'device'};
	foreach $p (@{$d->{'parts'}}) {
		next if ($p->{'type'} ne '83' && $p->{'type'} ne '82' &&
			 $p->{'type'} ne 'b' && $p->{'type'} ne 'c');
		local $volid = $self->get_volid($p->{'device'});
		next if (!$volid);
		push(@opts, [ $volid, "$volid ($p->{'desc'})" ]);
		$$found++ if ($value eq $volid);
		}
	}
if (@opts) {
	return $self->ui_select($name, $value, \@opts, 1, 0, $value ? 1 : 0);
	}
else {
	return undef;
	}
}

#############################################################################
# Internal functions
#############################################################################
sub open_fdisk
{
	my $self = shift;

local $fpath = $self->check_fdisk();
($self->{'fh'}, $self->{'fpid'}) = $self->foreign_call("proc", "pty_process_exec", join(" ",$fpath, @_));
}

sub open_sfdisk
{
	my $self = shift;

local $sfpath = $self->has_command("sfdisk");
($self->{'fh'}, $self->{'fpid'}) = $self->foreign_call("proc", "pty_process_exec", join(" ",$sfpath, @_));
}

sub check_fdisk
{
	my $self = shift;

local $fpath = $self->has_command("fdisk");
$self->error($self->text('open_error', "<tt>fdisk</tt>")) if (!$fpath);
return $fpath;
}

sub close_fdisk
{
	my $self = shift;

close($self->{'fh'}); kill('TERM', $self->{'fpid'});
}

sub wprint
{
	my $self = shift;

syswrite($self->{'fh'}, $_[0], length($_[0]));
}

sub opt_input
{
	my $self = shift;

print $_[2] ? "<tr>" : "";
print "<td align=right><b>$self->{'text_hash'}->{$_[0]}</b></td>\n";
print "<td nowrap><input type=radio name=$_[0]_def value=1 checked> ",
      $self->{'text_hash'}->{'opt_default'},"\n";
print "&nbsp; <input type=radio name=$_[0]_def value=0>\n";
print "<input name=$_[0] size=6> $_[1]</td>";
print $_[2] ? "\n" : "</tr>\n";
}

sub opt_check
{
	my $self = shift;

if ($self->{'in_hash'}->{"$_[0]_def"}) { return ""; }
elsif ($self->{'in_hash'}->{$_[0]} !~ /^$_[1]$/) {
	$self->error($self->text('opt_error', $self->{'in_hash'}->{$_[0]}, $self->{'text_hash'}->{$_[0]}));
	}
else { return " $_[2] $self->{'in_hash'}->{$_[0]}"; }
}

# can_edit_disk(device)
sub can_edit_disk
{
	my $self = shift;

foreach (split(/\s+/, $self->{'access_hash'}->{'disks'})) {
        return 1 if ($_ eq "*" || $_ eq $_[0]);
        }
return 0;
}

# disk_space(device, [mountpoint])
# Returns the amount of total and free space for some filesystem, or an
# empty array if not appropriate.
sub disk_space
{
	my $self = shift;

local $w = $_[1] || $_[0];
local $out = `df -k '$w'`;
if ($out =~ /Mounted on\s*\n\s*\S+\s+(\S+)\s+\S+\s+(\S+)/i) {
	return ($1, $2);
	}
elsif ($out =~ /Mounted on\s*\n\S+\s*\n\s+(\S+)\s+\S+\s+(\S+)/i) {
	return ($1, $2);
	}
else {
	return ( );
	}
}

# supported_filesystems()
# Returns a list of filesystem types that can have mkfs_options called on them
sub supported_filesystems
{
	my $self = shift;

local @fstypes = ( "ext2" );
push(@fstypes, "ext3") if ($self->has_command("mkfs.ext3") ||
			   $self->has_command("mke3fs") ||
			   `mkfs.ext2 -h 2>&1` =~ /\[-j\]/);
push(@fstypes, "reiserfs") if ($self->has_command("mkreiserfs"));
push(@fstypes, "xfs") if ($self->has_command("mkfs.xfs"));
push(@fstypes, "jfs") if ($self->has_command("mkfs.jfs"));
push(@fstypes, "fatx") if ($self->has_command("mkfs.fatx"));
push(@fstypes, "msdos");
push(@fstypes, "vfat");
push(@fstypes, "minix");
return @fstypes;
}

# get_label(device, [type])
# Returns the XFS or EXT label for some device's filesystem
sub get_label
{
	my $self = shift;

local $label;
if ($self->{'has_e2label'}) {
	$label = `e2label $_[0] 2>&1`;
	chop($label);
	}
if (($? || $label !~ /\S/) && $self->{'has_xfs_db'}) {
	$label = undef;
	local $out = $self->backquote_with_timeout("xfs_db -x -p xfs_admin -c label -r $_[0] 2>&1", 5);
	$label = $1 if ($out =~ /label\s*=\s*"(.*)"/ &&
			$1 ne '(null)');
	}
return $? || $label !~ /\S/ ? undef : $label;
}

# get_volid(device)
# Returns the UUID for some device's filesystem
sub get_volid
{
	my $self = shift;

if ($self->{'has_volid'}) {
	local $out = $self->backquote_command("vol_id $_[0] 2>&1");
	if ($out =~ /ID_FS_UUID=(\S+)/) {
		return $1;
		}
	}
return undef;
}

# set_label(device, label, [type])
# Tries to set the label for some device's filesystem
sub set_label
{
	my $self = shift;

if ($self->{'has_e2label'} && ($_[2] =~ /^ext[23]$/ || !$_[2])) {
	$self->system_logged("e2label '$_[0]' '$_[1]' >/dev/null 2>&1");
	return 1 if (!$?);
	}
if ($self->{'has_xfs_db'} && ($_[2] eq "xfs" || !$_[2])) {
	$self->system_logged("xfs_db -x -p xfs_admin -c \"label $_[1]\" $_[0] >/dev/null 2>&1");
	return 1 if (!$?);
	}
return 0;
}


1;

