#!/usr/bin/perl
package firewall::DebianlinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'firewall::DebianlinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# debians-linux-lib.pl
# Deal with debian's iptables save file and startup script

if ($self->{'gconfig_hash'}->{'os_version'} >= 3.1 &&
    !-r "/etc/init.d/iptables" &&
    !-r "/etc/init.d/webmin-iptables") {
	# In newer Debians, IPtable is started by the network init script
	$self->{'has_new_debian_iptables'} = 1;
	$self->{'iptables_save_file'} = "/etc/iptables.up.rules";
	}
else {
	# Older Debians use an init script
	$self->{'has_debian_iptables'} = -r "/etc/init.d/iptables";
	$self->{'debian_iptables_dir'} = "/var/lib/iptables";
	if ($self->{'has_debian_iptables'}) {
		mkdir($self->{'debian_iptables_dir'}, 0755) if (!-d $self->{'debian_iptables_dir'});
		$self->{'iptables_save_file'} = "$self->{'debian_iptables_dir'}/active";
		}
	}

1;

1;
		return $self;
}

# apply_iptables()
# Applies the current iptables configuration from the save file
sub apply_iptables
{
	my $self = shift;

if ($self->{'has_debian_iptables'}) {
	local $out = $self->backquote_logged("cd / ; /etc/init.d/iptables start 2>&1");
	return $? ? "<pre>$out</pre>" : undef;
	}
else {
	return $self->iptables_restore();
	}
}

# unapply_iptables()
# Writes the current iptables configuration to the save file
sub unapply_iptables
{
	my $self = shift;

if ($self->{'has_debian_iptables'}) {
	$self->{'out'} = $self->backquote_logged("cd / ; /etc/init.d/iptables save active 2>&1 </dev/null");
	return $? ? "<pre>$self->{'out'}</pre>" : undef;
	}
else {
	return $self->iptables_save();
	}
}

# started_at_boot()
sub started_at_boot
{
	my $self = shift;

$self->foreign_require("init", "init-lib.pl");
if ($self->{'has_debian_iptables'}) {
	# Check Debian init script
	return $main::init_object->action_status("iptables") == 2;
	
	}
elsif ($self->{'has_new_debian_iptables'}) {
	# Check network interface config
	local $pri = $self->get_primary_network_interface();
	local ($debpri) = grep { $_->[0] eq $pri->{'fullname'} }
			       $main::net_object->get_interface_defs();
	foreach my $o (@{$debpri->[3]}) {
		if ($o->[0] eq "pre-up" &&
		    $o->[1] =~ /\S*iptables-restore\s+<\s+(\S+)/ &&
		    $1 eq $self->{'iptables_save_file'}) {
			return 1;
			}
		}
	}
else {
	# Check Webmin init script
	return $main::init_object->action_status("$self->{'gconfig_hash'}->{'product'}-iptables") == 2;
	}
}

sub enable_at_boot
{
	my $self = shift;

$self->foreign_require("init", "init-lib.pl");
if ($self->{'has_debian_iptables'}) {
	$main::init_object->enable_at_boot("iptables");	 # Assumes init script exists
	}
elsif ($self->{'has_new_debian_iptables'}) {
	# Add to network interface config
	local $pri = $self->get_primary_network_interface();
	local ($debpri) = grep { $_->[0] eq $pri->{'fullname'} }
			       $main::net_object->get_interface_defs();
	push(@{$debpri->[3]},
	     [ "pre-up", "iptables-restore < $self->{'iptables_save_file'}" ]);
	$main::net_object->modify_interface_def(@$debpri);
	}
else {
	$self->create_webmin_init();
	}
}

sub disable_at_boot
{
	my $self = shift;

$self->foreign_require("init", "init-lib.pl");
if ($self->{'has_debian_iptables'}) {
	$main::init_object->disable_at_boot("iptables");
	}
elsif ($self->{'has_new_debian_iptables'}) {
	# Remove from network interface config
	local $pri = $self->get_primary_network_interface();
	local ($debpri) = grep { $_->[0] eq $pri->{'fullname'} }
			       $main::net_object->get_interface_defs();
	@{$debpri->[3]} = grep { $_->[0] ne "pre-up" ||
				 $_->[1] !~ /^\S*iptables/ } @{$debpri->[3]};
	$main::net_object->modify_interface_def(@$debpri);
	}
else {
	$main::init_object->disable_at_boot("$self->{'gconfig_hash'}->{'product'}-iptables");
	}
}

sub get_primary_network_interface
{
	my $self = shift;

$self->foreign_require("net", "net-lib.pl");
local @boot = sort { $a->{'fullname'} cmp $b->{'fullname'} }
		   $main::net_object->boot_interfaces();
local ($eth) = grep { $_->{'fullname'} =~ /^eth\d+$/ } @boot;
local ($ppp) = grep { $_->{'fullname'} =~ /^ppp\d+$/ } @boot;
return $eth || $ppp || $boot[0];
}


1;

