#!/usr/bin/perl
package firewall::RedhatlinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'firewall::RedhatlinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# redhat-linux-lib.pl
# Deal with redhat's /etc/sysconfig/iptables save file and startup script

$self->foreign_require("init", "init-lib.pl");
$self->{'init_script'} = "$main::init_object->{'config_hash'}->{'init_dir'}/iptables";

$self->{'iptables_save_file'} = "/etc/sysconfig/iptables";

1;

1;
		return $self;
}

# check_iptables()
# Returns an error message if something is wrong with iptables on this system
sub check_iptables
{
	my $self = shift;

if (!-r $self->{'init_script'}) {
	return $self->text('redhat_escript', "<tt>$self->{'init_script'}</tt>");
	}
return undef if ($self->{'gconfig_hash'}->{'os_type'} eq 'trustix-linux');
return undef if ($self->{'gconfig_hash'}->{'os_type'} eq 'redhat-linux' &&
		 $self->{'gconfig_hash'}->{'os_version'} > 10);
if (!$self->{'config_hash'}->{'done_check_iptables'}) {
	local $out = `$self->{'init_script'} status 2>&1`;
	if ($out !~ /table:|INPUT|FORWARD|OUTPUT|is\s+stopped|firewall\s+stopped/) {
		return $self->text('redhat_eoutput',
			     "<tt>$self->{'init_script'} status</tt>");
		}
	$self->{'config_hash'}->{'done_check_iptables'} = 1;
	$self->save_module_config();
	}
return undef;
}

# apply_iptables()
# Applies the current iptables configuration from the save file
sub apply_iptables
{
	my $self = shift;

local $out = $self->backquote_logged("cd / ; $self->{'init_script'} restart 2>&1");
$out =~ s/\033[^m]+m//g;
return $? || $out =~ /FAILED/ ? "<pre>$out</pre>" : undef;
}

# unapply_iptables()
# Writes the current iptables configuration to the save file
sub unapply_iptables
{
	my $self = shift;

$self->{'out'} = $self->backquote_logged("cd / ; $self->{'init_script'} save 2>&1 </dev/null");
$self->{'out'} =~ s/\033[^m]+m//g;
if ($? && $self->{'out'} =~ /usage/i) {
	# 'save' argument not supported .. call iptables-save manually
	return $self->iptables_save();
	}
return $? || $self->{'out'} =~ /FAILED/ ? "<pre>$self->{'out'}</pre>" : undef;
}

# started_at_boot()
sub started_at_boot
{
	my $self = shift;

return $main::init_object->action_status("iptables") == 2;
}

sub enable_at_boot
{
	my $self = shift;

$main::init_object->enable_at_boot("iptables");	 # Assumes init script exists
}

sub disable_at_boot
{
	my $self = shift;

$main::init_object->disable_at_boot("iptables");
}

# &save_rules_sh(%hash, [$internet_interface])
# save iptables rules in to .sh script
sub save_rules_sh
{
	my $self = shift;

 my $rules_file = $self->read_file_lines($self->{'config_hash'}->{'rules_file'});
 @{$rules_file} = ("# Configured by ZTL #\n#####################");
 push (@{$rules_file},"# (wan $_[1])") if $_[1];
 push (@{$rules_file},@{$_[0]});
 $self->flush_file_lines();
 chmod(0744, $self->{'config_hash'}->{'rules_file'});
 
 # load rules 
 my $out = $self->backquote_logged("$self->{'config_hash'}->{'rules_file'} 2>&1");
 return $out if $?;
 
 # save rules
 $out = $self->iptables_save();
 return $out ? $out : undef;
}


1;

