#!/usr/bin/perl
package firewall::TrustixlinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'firewall::TrustixlinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# redhat-linux-lib.pl
# Deal with redhat's /etc/sysconfig/iptables save file and startup script

$self->foreign_require("init", "init-lib.pl");
$self->{'init_script'} = "$main::init_object->{'config_hash'}->{'init_dir'}/iptables";

$self->{'iptables_save_file'} = "/etc/sysconfig/iptables-ipv4.d/start";

#-------- new functions ---------
@{$self->{'services_ports_array'}} = (['http',{'http'=>'tcp'}],
						 ['https',{'https'=>'tcp'}],
						 ['ftp',{'ftp'=>'tcp'}],
						 ['ssh',{'ssh'=>'tcp'}],
						 ['telnet',{'telnet'=>'tcp'}],
						 ['smtp',{'smtp'=>'tcp'}],
						 ['pop3',{'pop3'=>'tcp'}],
						 ['pop3s',{'pop3s'=>'tcp'}],
						 ['imap',{'imap'=>'tcp'}], 
						 ['imaps',{'imaps'=>'tcp'}],
						 ['dns',{'domain'=>'udp'}],
						 ['dhcp_server',{'bootps'=>'udp'}],
						 ['dhcp_client',{'bootpc'=>'udp'}],
						 ['gopher',{'gopher'=>'tcp'}],
						 ['finger',{'finger'=>'tcp'}],
						 ['ntp',{'ntp'=>'udp'}],
						 ['samba',{'netbios-ns'=>'udp',
						 						 'netbios-dgm'=>'udp',
												 'netbios-ssn'=>'tcp',
												 'microsoft-ds'=>'tcp'}],
						 ['squid',{'3128'=>'tcp'}],
						 ['dansguardian',{'8080'=>'tcp'}],
						 ['ztl_admin',{'10000'=>'tcp'}],
						 ['ztl_user',{'20000'=>'tcp'}]);

1;

1;
		return $self;
}

# check_iptables()
# Returns an error message if something is wrong with iptables on this system
sub check_iptables
{
	my $self = shift;

if (!-r $self->{'init_script'}) {
	return $self->text('redhat_escript', "<tt>$self->{'init_script'}</tt>");
	}
return undef if ($self->{'gconfig_hash'}->{'os_type'} eq 'trustix-linux');
return undef if ($self->{'gconfig_hash'}->{'os_type'} eq 'redhat-linux' &&
		 $self->{'gconfig_hash'}->{'os_version'} > 10);
if (!$self->{'config_hash'}->{'done_check_iptables'}) {
	local $out = `$self->{'init_script'} status 2>&1`;
	if ($out !~ /table:|INPUT|FORWARD|OUTPUT|is\s+stopped|firewall\s+stopped/) {
		return $self->text('redhat_eoutput',
			     "<tt>$self->{'init_script'} status</tt>");
		}
	$self->{'config_hash'}->{'done_check_iptables'} = 1;
	$self->save_module_config();
	}
return undef;
}

# apply_iptables()
# Applies the current iptables configuration from the save file
sub apply_iptables
{
	my $self = shift;

local $out = $self->backquote_logged("cd / ; $self->{'init_script'} restart 2>&1");
$out =~ s/\033[^m]+m//g;
return $? || $out =~ /FAILED/ ? "<pre>$out</pre>" : undef;
}

# unapply_iptables()
# Writes the current iptables configuration to the save file
sub unapply_iptables
{
	my $self = shift;

$self->{'out'} = $self->backquote_logged("cd / ; $self->{'init_script'} save 2>&1 </dev/null");
$self->{'out'} =~ s/\033[^m]+m//g;
if ($? && $self->{'out'} =~ /usage/i) {
	# 'save' argument not supported .. call iptables-save manually
	return $self->iptables_save();
	}
return $? || $self->{'out'} =~ /FAILED/ ? "<pre>$self->{'out'}</pre>" : undef;
}

# started_at_boot()
sub started_at_boot
{
	my $self = shift;

return $main::init_object->action_status("iptables") == 2;
}

sub enable_at_boot
{
	my $self = shift;

$main::init_object->enable_at_boot("iptables");	 # Assumes init script exists
}

sub disable_at_boot
{
	my $self = shift;

$main::init_object->disable_at_boot("iptables");
}

# &generate_clear_rules(\%hash)
# add iptables rules to %hash:
# clear existing rules and chains
sub generate_clear_rules
{
	my $self = shift;

 # flush existing rules
 push (@{$_[0]},"\n# Flush existing rules");
 push (@{$_[0]},'iptables -t filter -F');
 push (@{$_[0]},'iptables -t nat -F');
 push (@{$_[0]},'iptables -t mangle -F');

 # delete custom chains
 push (@{$_[0]},"\n# Delete custom chains");
 push (@{$_[0]},'iptables -X');
}

# &generate_default_policies(\%hash,$input,$forward,$output)
# add iptables rules to %hash:
# set default policies
sub generate_default_policies
{
	my $self = shift;

 my ($rule, $input, $forward, $output) = @_;
 
 # set default policies
 push (@{$rule},"\n# Set default policies");
 if ($input eq 'ACCEPT' && $forward eq 'ACCEPT' && $output eq 'ACCEPT') { push (@{$rule},"# (policies accept)"); }
 else { push (@{$rule},"# (policies default)"); }
 push (@{$rule},"iptables -P INPUT $input");
 push (@{$rule},"iptables -P FORWARD $forward");
 push (@{$rule},"iptables -P OUTPUT $output");
}

# &generate_prepare_rules(\%hash)
# add iptables rules to %hash:
# allow established, drop all invalid packets, accept all from the loopback device
sub generate_prepare_rules
{
	my $self = shift;

 # allow established
 push (@{$_[0]},"\n# Allow input and forward established packets");
 push (@{$_[0]},'iptables -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT');
 push (@{$_[0]},'iptables -A FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT');
 
 # drop all invalid packets
 push (@{$_[0]},"\n# Drop input, output and forward all invalid packets");
 push (@{$_[0]},'iptables -A INPUT -m state --state INVALID -j DROP');
 push (@{$_[0]},'iptables -A FORWARD -m state --state INVALID -j DROP');
 push (@{$_[0]},'iptables -A OUTPUT -m state --state INVALID -j DROP');
 
 # accept all from the loopback device
 push (@{$_[0]},"\n# Allow input all packets from lo");
 push (@{$_[0]},'iptables -A INPUT -i lo -j ACCEPT');
 push (@{$_[0]},'iptables -A INPUT -s 127.0.0.1/255.0.0.0 ! -i lo -j DROP'); 
}

# &generate_icmp_rules(\%hash)
# add iptables rules to %hash:
# allow input and forward some ICMP packets
sub generate_icmp_rules
{
	my $self = shift;

 my @icmp = (0,3,8,11);
 push (@{$_[0]},"\n# Allow ICMP packets types: echo reply, network unreachable, echo request, time-exceeded");
 foreach (@icmp)
 {
  push (@{$_[0]},"iptables -A INPUT -p icmp --icmp-type $_ -j ACCEPT");
  push (@{$_[0]},"iptables -A FORWARD -p icmp --icmp-type $_ -j ACCEPT");
 }
}

# &generate_private_rules(\%hash,$internet_interface)
# add iptables rules to %hash:
# drop all packets from private reserved networks on Internet interface
sub generate_private_rules
{
	my $self = shift;

 my $wan_interface = $_[1];
 my @private_nets = ('10.0.0.0/8','192.168.0.0/16','172.16.0.0/12');
 push (@{$_[0]},"\n# Drop all packets from private reserved networks on the Internet interface");
 push (@{$_[0]},"# (priv $wan_interface)");
 foreach (@private_nets)
 {
  push (@{$_[0]},"iptables -A INPUT -i $wan_interface -s $_ -j DROP");
  push (@{$_[0]},"iptables -A FORWARD -o $wan_interface -d $_ -j DROP");
  push (@{$_[0]},"iptables -A OUTPUT -o $wan_interface -d $_ -j DROP");
 }
}

# &generate_lan_server_accept(\%hash,\@lan)
# add iptables rules to %hash:
# accept all packets from LANs to Server
sub generate_lan_server_accept
{
	my $self = shift;

 my $lan = $_[1];
 push (@{$_[0]},"\n# Allow input all packets from LANs");
 push (@{$_[0]},"# (lan server any)");
 foreach (@{$lan})
 {
  push (@{$_[0]},"iptables -A INPUT -i $_ -j ACCEPT");
 }
}

# &generate_lan_internet_accept(\%hash,\@lan, [$wan], [1])
# add iptables rules to %hash:
# accept all packets from LANs to Internet
# if $_[3] -- without advanced comments
sub generate_lan_internet_accept
{
	my $self = shift;

 my ($lan, $wan_interface) = ($_[1],$_[2]);
 if ($_[3])
 {
  if ($wan_interface) { push (@{$_[0]},"\n# Allow forward all packets from LANs, but not to Internet"); }
  else { push (@{$_[0]},"\n# Allow forward all packets from LANs"); }
 }
 else 
 { 
  push (@{$_[0]},"\n# Allow forward all packets from LANs"); 
  push (@{$_[0]},"# (lan internet any)");
 }
 foreach (@{$lan})
 {
  if ($wan_interface) { push (@{$_[0]},"iptables -A FORWARD -i $_ ! -o $wan_interface -j ACCEPT"); }
  else { push (@{$_[0]},"iptables -A FORWARD -i $_ -j ACCEPT"); }
 }
}

# &generate_internet_server_accept(\%hash,$wan)
# add iptables rules to %hash:
# accept all packets from Internet to Server
sub generate_internet_server_accept
{
	my $self = shift;

 push (@{$_[0]},"\n# Allow input all packets from Internet");
 push (@{$_[0]},"# (internet server any)");
 push (@{$_[0]},"iptables -A INPUT -i $_[1] -j ACCEPT");
}

# &generate_service_rules(\%hash,\@lan, $internet_interface, $sorce, $dest, \%port, $service, [$protocol])
# sorce/dest: lan, server, internet
# protocol: tcp, udp
# add iptables rules to %hash:
# allow access for some services
sub generate_service_rules
{
	my $self = shift;

 my $unpriports="1024:65535";
 my ($rules, $lan, $wan_interface, $sorce, $dest, $port, $service, $protocol) = @_;
 if ($sorce eq 'lan' && $dest eq 'internet')
 {
  push (@{$_[0]},"\n# Allow forward packets from LANs to $service Internet services");
  push (@{$_[0]},"# (lan internet $service $protocol)");
  foreach my $i (keys %{$port})
  {
   foreach (@{$lan})
   {
    push (@{$_[0]},"iptables -A FORWARD -p $port->{$i} -m $port->{$i} -i $_ -o $wan_interface --dport $i -j ACCEPT");
   }
  }
 }
 elsif ($sorce eq 'lan' && $dest eq 'server')
 {
  push (@{$_[0]},"\n# Allow forward packets from LANs to $service host services");
  push (@{$_[0]},"# (lan server $service $protocol)");
  foreach my $i (keys %{$port})
  {
   foreach (@{$lan})
   {
    push (@{$_[0]},"iptables -A INPUT -p $port->{$i} -m $port->{$i} -i $_ --dport $i -j ACCEPT");
   }
  }
 }
 elsif ($sorce eq 'internet' && $dest eq 'server')
 {
  push (@{$_[0]},"\n# Allow forward packets from Internet to $service host services");
  push (@{$_[0]},"# (internet server $service $protocol)");
  foreach my $i (keys %{$port})
  {
   push (@{$_[0]},"iptables -A INPUT -p $port->{$i} -m $port->{$i} -i $wan_interface --dport $i -j ACCEPT");
  }
 }
}

# &generate_proxy_rules(\%hash,\@lan)
# add iptables rules to %hash:
# redirect all query to Squid through DansGuardian
sub generate_proxy_rules
{
	my $self = shift;

 my ($rules, $lan, $squid, $dg) = @_;
 
 # get Squid port
 $self->foreign_require('squid','squid-lib.pl');
 my $sq_port = $main::squid_object->get_squid_port();
 
 if ($squid && !$dg)
 {
  push (@{$_[0]},"\n# Redirect all WEB packets to Squid");
  push (@{$_[0]},"# (squid $sq_port)");
  foreach (@{$lan})
  {
   push (@{$_[0]},"iptables -t nat -A PREROUTING -i $_ -p tcp --dport http -j REDIRECT --to-port $sq_port");
  }
 } 
 elsif ($dg)
 {
  # get DG port
  $self->foreign_require('dg','dg-lib.pl');
  my $conf = $main::dg_object->parse_config_file();
  my $port = $main::dg_object->get_config_value('filterport', $conf); 
  
  push (@{$_[0]},"\n# Redirect all LAN to Squid packets to DansGuardian");
  push (@{$_[0]},"# (dg $port)");
  foreach (@{$lan})
  {
   push (@{$_[0]},"iptables -t nat -A PREROUTING -i $_ -p tcp --dport $sq_port -j REDIRECT --to-port $port");
  }
  if ($squid)
  {
   push (@{$_[0]},"\n# Redirect all WEB packets to DansGuardian");
   push (@{$_[0]},"# (squid $sq_port)");
   foreach (@{$lan})
   {
    push (@{$_[0]},"iptables -t nat -A PREROUTING -i $_ -p tcp --dport http -j REDIRECT --to-port $port");
   }
  }  
 }
}

# &generate_masquerade_rules(\%hash, internet_interface)
# add iptables rules to %hash:
# enable ip masquerade on Internet interface
sub generate_masquerade_rules
{
	my $self = shift;

 my $wan_interface = $_[1];
 push (@{$_[0]},"\n# Enable MASQUERADE on the Internet interface");
 push (@{$_[0]},"# (nat $wan_interface)");
 push (@{$_[0]},"iptables -t nat -A POSTROUTING -o $wan_interface -j MASQUERADE");
}

# lan_check()
# return array refs of lan and wan interfaces
sub lan_check
{
	my $self = shift;

 # require librarys from module net
 $self->foreign_require('net','net-lib.pl');

 # list of interfaces brought up at boot time
 my (@boot, @lan, $wan) = $main::net_object->boot_interfaces();
 foreach (@boot)
 {
  if ($_->{'name'} ne 'lo')
  {
   if ($_->{'address'}=~/^192\.168\./ || $_->{'address'}=~/^172\.[16..31]\./ || $_->{'address'}=~/^10\./ || $wan) { push (@lan,$_->{'fullname'}); }
	else { $wan = $_->{'fullname'}; }
  }
 }
 return (\@lan, $wan);
}

# &save_rules_sh(%hash, [$internet_interface])
# save iptables rules in to .sh script
sub save_rules_sh
{
	my $self = shift;

 my $rules_file = $self->read_file_lines($self->{'config_hash'}->{'rules_file'});
 @{$rules_file} = ("# Configured by ZTL #\n#####################");
 push (@{$rules_file},"# (wan $_[1])") if $_[1];
 push (@{$rules_file},@{$_[0]});
 $self->flush_file_lines();
 chmod(0744, $self->{'config_hash'}->{'rules_file'});
 
 # restart iptables
 my $out = $self->apply_iptables();
 return $out ? $out : undef;
}

# &parse_rules()
# parse iptables rules from .sh script
sub parse_rules
{
	my $self = shift;

 my $rules_file = $self->read_file_lines($self->{'config_hash'}->{'rules_file'});
 my $conf;
 foreach (@{$rules_file})
 {
  if ($_ =~ /^#\s+\((.+)\)/)
  {
   my @temp = split (/\s+/, $1);
	if ($temp[0] eq 'lan' || $temp[0] eq 'internet')
	{
	 push (@{$conf}, {'src' => $temp[0],
	 						'dst' => $temp[1],
							'port' => $temp[2],
							'protocol' => $temp[3]});
	}
	else { push (@{$conf}, {$temp[0] => $temp[1]}) }
  }
 }
 return $conf;
}

# status_iptables()
# return 1 if all iptables rules flushed
sub status_iptables
{
	my $self = shift;

 my (@ch, @out) = ('INPUT','FORWARD','OUTPUT', 'PREROUTING', 'POSTROUTING');
 foreach (@ch)
 {
  @out = $self->backquote_logged("iptables -t nat -L $_ 2>&1");
  return 0 if ($#out > 1);
 }
 return 1;
}

# stop_iptables()
# stop iptables service
sub stop_iptables
{
	my $self = shift;

 local $out = $self->backquote_logged("cd / ; $self->{'init_script'} stop 2>&1");
 $out =~ s/\033[^m]+m//g;
 return $? || $out =~ /FAILED/ ? "<pre>$out</pre>" : undef;
}


1;

