# firewall-lib.pl
# Functions for parsing iptables-save format files
# - help pages

do '../web-lib.pl';
&init_config();
do '../ui-lib.pl';
if ($config{'save_file'}) {
	# Force use of a different save file, and webmin's functions
	$iptables_save_file = $config{'save_file'};
	}
else {
	if (-r "$module_root_directory/$gconfig{'os_type'}-lib.pl") {
		# Use the operating system's save file and functions
		do "$gconfig{'os_type'}-lib.pl";
		}

	if (!$iptables_save_file) {
		# Use webmin's own save file
		$iptables_save_file = "$module_config_directory/iptables.save";
		}
	}

%access = &get_module_acl();

@known_tables = ( "filter", "mangle", "nat" );
@known_args =   ('-p', '-m', '-s', '-d', '-i', '-o', '-f',
		 '--dport', '--sport', '--tcp-flags', '--tcp-option',
		 '--icmp-type', '--mac-source', '--limit', '--limit-burst',
		 '--ports', '--uid-owner', '--gid-owner',
		 '--pid-owner', '--sid-owner', '--state', '--tos', '-j',
		 '--to-ports', '--to-destination', '--to-source',
		 '--reject-with', '--dports', '--sports',
		 '--comment');

# get_iptables_save([file])
# Parse the iptables save file into a list of tables 
# format seems to be:
#  *table
#  :chain defaultpolicy
#  -A chain options
#  -N chain
#  COMMIT
sub get_iptables_save
{
local (@rv, $table, %got);
local $lnum = 0;
open(FILE, $_[0] || ($config{'direct'} ? "iptables-save |"
				       : $iptables_save_file));
local $cmt;
while(<FILE>) {
        local $read_comment;
	s/\r|\n//g;
	if (s/#\s*(.*)$//) {
		$cmt .= " " if ($cmt);
		$cmt .= $1;
		$read_comment=1;
		}
	if (/^\*(\S+)/) {
		# Start of a new table
		$got{$1}++;
		push(@rv, $table = { 'line' => $lnum,
				     'eline' => $lnum,
				     'name' => $1,
				     'rules' => [ ],
				     'defaults' => { } });
		}
	elsif (/^:(\S+)\s+(\S+)/) {
		# Default policy definition
		$table->{'defaults'}->{$1} = $2;
		}
	elsif (/^(\[[^\]]*\]\s+)?-N\s+(\S+)(.*)/) {
		# New chain definition
		$table->{'defaults'}->{$2} = '-';
		}
	elsif (/^(\[[^\]]*\]\s+)?-A\s+(\S+)(.*)/) {
		# Rule definition
		local $rule = { 'line' => $lnum,
				'eline' => $lnum,
				'index' => scalar(@{$table->{'rules'}}),
				'cmt' => $cmt,
				'chain' => $2,
				'args' => $3 };
		push(@{$table->{'rules'}}, $rule);

		# Parse arguments
		foreach $a (@known_args) {
			local @vl;
			while($rule->{'args'} =~ s/\s+(!?)\s*($a)\s+(!?)\s*(([^ \-!]\S*(\s+|$))+)/ / || $rule->{'args'} =~ s/\s+(!?)\s*($a)()(\s+|$)/ /) {
				push(@vl, [ $1 || $3, &split_quoted_string($4) ]);
				}
			local ($aa = $a); $aa =~ s/^-+//;
			if ($a eq '-m') {
				$rule->{$aa} = \@vl if (@vl);
				}
			else {
				$rule->{$aa} = $vl[0];
				}
			}
		}
	elsif (/^COMMIT/) {
		# Marks end of a table
		$table->{'eline'} = $lnum;
		}
	elsif (/\S/) {
		&error(&text('eiptables', "<tt>$_</tt>"));
		}
	$lnum++;
	if (! defined($read_comment)) { $cmt=undef; }
	}
close(FILE);
@rv = sort { $a->{'name'} cmp $b->{'name'} } @rv;
local $i;
map { $_->{'index'} = $i++ } @rv;
return @rv;
}

# save_table(&table)
# Updates an existing IPtable in the save file
sub save_table
{
local $lref;
if ($config{'direct'}) {
	# Read in the current iptables-save output
	$lref = &read_file_lines("iptables-save |");
	}
else {
	# Updating the save file
	$lref = &read_file_lines($iptables_save_file);
	}
local @lines = ( "*$_[0]->{'name'}" );
local ($d, $r);
foreach $d (keys %{$_[0]->{'defaults'}}) {
	push(@lines, ":$d $_[0]->{'defaults'}->{$d} [0:0]");
	}
foreach $r (@{$_[0]->{'rules'}}) {
	local $line;
	$line = "# $r->{'cmt'}\n" if ($r->{'cmt'});
	$line .= "-A $r->{'chain'}";
	foreach $a (@known_args) {
		local ($aa = $a); $aa =~ s/^-+//;
		if ($r->{$aa}) {
			local @al = ref($r->{$aa}->[0]) ?
					@{$r->{$aa}} : ( $r->{$aa} );
			foreach $ag (@al) {
				local $n = shift(@$ag);
				local @w = ( $n ? ( $n ) : (), $a, @$ag );
				@w = map { $_ =~ /\s/ ? "\"$_\"" : $_ } @w;
				$line .= " ".join(" ", @w);
				}
			}
		}
	$line .= " $r->{'args'}" if ($r->{'args'} =~ /\S/);
	push(@lines, $line);
	}
push(@lines, "COMMIT");
if (defined($_[0]->{'line'})) {
	# Update in file
	splice(@$lref, $_[0]->{'line'}, $_[0]->{'eline'} - $_[0]->{'line'} + 1,
	       @lines);
	}
else {
	# Append new table to file
	push(@$lref, "# Generated by webmin", @lines, "# Completed");
	}
if ($config{'direct'}) {
	# Pass new lines to iptables-restore
	open(SAVE, "| iptables-restore");
	print SAVE map { $_."\n" } @$lref;
	close(SAVE);
	}
else {
	# Just save the file
	&flush_file_lines();
	}
}

# describe_rule(&rule)
sub describe_rule
{
local (@c, $d);
foreach $d ('p', 's', 'd', 'i', 'o', 'f', 'dport',
	    'sport', 'tcp-flags', 'tcp-option',
	    'icmp-type', 'mac-source', 'limit', 'limit-burst',
	    'ports', 'uid-owner', 'gid-owner',
	    'pid-owner', 'sid-owner', 'state', 'tos',
	    'dports', 'sports') {
	if ($_[0]->{$d}) {
		local ($n, @v) = @{$_[0]->{$d}};
		@v = map { uc($_) } @v if ($d eq 'p');
		local $txt = &text("desc_$d$n", map { "<b>$_</b>" } @v);
		push(@c, $txt) if ($txt);
		}
	}
local $rv;
if (@c) {
	$rv = &text('desc_conds', join(" $text{'desc_and'} ", @c));
	}
else {
	$rv = $text{'desc_always'};
	}
return $rv;
}

# create_firewall_init()
# Do whatever is needed to have the firewall started at boot time
sub create_firewall_init
{
if (defined(&enable_at_boot)) {
	# Use distro's function
	&enable_at_boot();
	}
else {
	# May need to create init script
	&create_webmin_init();
	}
}

# create_webmin_init()
# Create (if necessary) the Webmin iptables init script
sub create_webmin_init
{
local $res = &has_command("iptables-restore");
local $ipt = &has_command("iptables");
local $start = "$res <$iptables_save_file";
local $stop = "$ipt -t filter -F\n".
	      "$ipt -t nat -F\n".
	      "$ipt -t mangle -F\n".
	      "$ipt -t filter -P INPUT ACCEPT\n".
	      "$ipt -t filter -P OUTPUT ACCEPT\n".
	      "$ipt -t filter -P FORWARD ACCEPT\n".
	      "$ipt -t nat -P PREROUTING ACCEPT\n".
	      "$ipt -t nat -P POSTROUTING ACCEPT\n".
	      "$ipt -t nat -P OUTPUT ACCEPT\n".
	      "$ipt -t mangle -P PREROUTING ACCEPT\n".
	      "$ipt -t mangle -P OUTPUT ACCEPT";
&foreign_require("init", "init-lib.pl");
&init::enable_at_boot("$gconfig{'product'}-iptables", "Load IPtables save file",
		      $start, $stop);
}

# interface_choice(name, value)
sub interface_choice
{
local @ifaces;
if (&foreign_check("net")) {
	&foreign_require("net", "net-lib.pl");
	return &net::interface_choice($_[0], $_[1]);
	}
else {
	return "<input name=$_[0] size=6 value='$_[1]'>";
	}
}

sub check_previous
{
	my (@p,$max,$n)=@_;
	for ($i=0;$i<$max;$i++)
	{
		if ($n eq $p[$i]){return 1}
	}
	return -1;
}
 
sub by_string_for_iptables
{
	my @p=("PREROUTING","INPUT","FORWARD","OUTPUT","POSTROUTING");

	for ($i=0;$i<@p;$i++)
	{
		if ($a eq $p[$i]){
			if (&check_previous(@p,$i,$b)){return -1;}
			else{ return 1;}}
		if ($b eq $p[$i]){
			if (&check_previous(@p,$i,$b)){return 1;}
			else{ return -1;}}
	}

	return $a cmp $b;
}

sub missing_firewall_commands
{
local $c;
foreach $c ("iptables", "iptables-restore", "iptables-save") {
	return $c if (!&has_command($c));
	}
return undef;
}

# iptables_restore()
# Activates the current firewall rules, and returns any error
sub iptables_restore
{
local $out = &backquote_logged("cd / ; iptables-restore <$iptables_save_file 2>&1");
return $? ? "<pre>$out</pre>" : undef;
}

# iptables_save()
# Saves the active firewall rules, and returns any error
sub iptables_save
{
local $out = &backquote_logged("iptables-save >$iptables_save_file 2>&1");
return $? ? "<pre>$out</pre>" : undef;
}

# can_edit_table(name)
sub can_edit_table
{
return $access{$_[0]};
}

# can_jump(jump|&rule)
sub can_jump
{
return 1 if (!$access{'jumps'});
if (!defined(%can_jumps_cache)) {
	%can_jumps_cache = map { lc($_), 1 } split(/\s+/, $access{'jumps'});
	}
local $j = ref($_[0]) ? $_[0]->{'j'}->[1] : $_[0];
return 1 if (!$j);	# always allow 'do nothing'
return $can_jumps_cache{lc($j)};
}

# run_before_command()
# Runs the before-saving command, if any
sub run_before_command
{
if ($config{'before_cmd'}) {
	&system_logged("($config{'before_cmd'}) </dev/null >/dev/null 2>&1");
	}
}

# run_after_command()
# Runs the after-saving command, if any
sub run_after_command
{
if ($config{'after_cmd'}) {
	&system_logged("($config{'after_cmd'}) </dev/null >/dev/null 2>&1");
	}
}

# run_before_apply_command()
# Runs the before-applying command, if any. If it failes, returns the error
# message output
sub run_before_apply_command
{
if ($config{'before_apply_cmd'}) {
	local $out = &backquote_logged("($config{'before_apply_cmd'}) </dev/null 2>&1");
	return $out if ($?);
	}
return undef;
}

# run_after_apply_command()
# Runs the after-applying command, if any
sub run_after_apply_command
{
if ($config{'after_apply_cmd'}) {
	&system_logged("($config{'after_apply_cmd'}) </dev/null >/dev/null 2>&1");
	}
}

# apply_configuration()
# Calls all the appropriate apply functions and programs, and returns an error
# message if anything fails
sub apply_configuration
{
local $err = &run_before_apply_command();
return $err if ($err);
if (defined(&apply_iptables)) {
	# Call distro's apply command
	$err = &apply_iptables();
	}
else {
	# Manually run iptables-restore
	$err = &iptables_restore();
	}
return $err if ($err);
&run_after_apply_command();
return undef;
}

# list_cluster_servers()
# Returns a list of servers on which the firewall is managed
sub list_cluster_servers
{
&foreign_require("servers", "servers-lib.pl");
local %ids = map { $_, 1 } split(/\s+/, $config{'servers'});
return grep { $ids{$_->{'id'}} } &servers::list_servers();
}

# add_cluster_server(&server)
sub add_cluster_server
{
local @sids = split(/\s+/, $config{'servers'});
$config{'servers'} = join(" ", @sids, $_[0]->{'id'});
&save_module_config();
}

# delete_cluster_server(&server)
sub delete_cluster_server
{
local @sids = split(/\s+/, $config{'servers'});
$config{'servers'} = join(" ", grep { $_ != $_[0]->{'id'} } @sids);
&save_module_config();
}

# server_name(&server)
sub server_name
{
return $_[0]->{'desc'} ? $_[0]->{'desc'} : $_[0]->{'host'};
}

# copy_to_cluster([force])
# Copy all firewall rules from this server to those in the cluster
sub copy_to_cluster
{
return if (!$config{'servers'});		# no servers defined
return if (!$_[0] && $config{'cluster_mode'});	# only push out when applying
local $s;
local $ltemp;
if ($config{'direct'}) {
	# Dump current configuration
	$ltemp = &transname();
	system("iptables-save >$ltemp 2>/dev/null");
	}
foreach $s (&list_cluster_servers()) {
	&remote_foreign_require($s, "firewall", "firewall-lib.pl");
	if ($config{'direct'}) {
		# Directly activate on remote server!
		local $rtemp = &remote_write($s, $ltemp);
		unlink($ltemp);
		local $err = &remote_eval($s, "firewall",
		  "\$out = `iptables-restore <$rtemp 2>&1`; [ \$out, \$? ]"); 
		&remote_eval($s, "firewall", "unlink('$rtemp')");
		&error(&text('apply_remote', $s->{'host'}, $err->[0]))
			if ($err->[1]);
		}
	else {
		# Can just copy across save file
		local $rfile = &remote_eval($s, "firewall",
					    "\$iptables_save_file");
		&remote_write($s, $iptables_save_file, $rfile);
		}
	}
}

# apply_cluster_configuration()
# Activate the current configuration on all servers in the cluster
sub apply_cluster_configuration
{
return undef if (!$config{'servers'});
if ($config{'cluster_mode'}) {
	&copy_to_cluster(1);
	}
local $s;
foreach $s (&list_cluster_servers()) {
	&remote_foreign_require($s->{'host'}, "firewall", "firewall-lib.pl");
	local $err = &remote_foreign_call($s->{'host'}, "firewall", "apply_configuration");
	if ($err) {
		return &text('apply_remote', $s->{'host'}, $err);
		}
	}
return undef;
}

#-------- forwarding functions --------

# check_forwarding()
# return net.ipv4.ip_forward option
sub check_forwarding
{
 return &backquote_command("sysctl -n net.ipv4.ip_forward 2>&1 </dev/null");
}

# enable_forwarding()
# set net.ipv4.ip_forward option
sub enable_forwarding
{
 # check forwarding
 my $out = &check_forwarding();
 if ($out != 1)
 {
  # enable forwarding
  $out = &backquote_command("sysctl -w net.ipv4.ip_forward=1 2>&1 </dev/null");
  &error ($out) if $?;
 }
 # change sysctl.conf if needed
 my $rules_file;
 my $rules_file = &read_file_lines($config{'sysctl_file'});
 foreach (@{$rules_file})
 {
  if ($_ =~ /^\s*net.ipv4.ip_forward\b/)
  {
   $_ = "net.ipv4.ip_forward = 1";
	last;
  }
 }
 &flush_file_lines();
}

# disable_forwarding()
# disable net.ipv4.ip_forward option
sub disable_forwarding
{
 # check forwarding
 my $out = &check_forwarding();
 if ($out != 0)
 {
  # disable forwarding
  $out = &backquote_command("sysctl -w net.ipv4.ip_forward=0 2>&1 </dev/null");
  &error ($out) if $?;
 }
 # change sysctl.conf if needed
 my $rules_file;
 my $rules_file = &read_file_lines($config{'sysctl_file'});
 foreach (@{$rules_file})
 {
  if ($_ =~ /^\s*net.ipv4.ip_forward\b/)
  {
   $_ = "net.ipv4.ip_forward = 0";
	last;
  }
 }
 &flush_file_lines();
}

#-------- new functions ---------
@services_ports = (['http',{'http'=>'tcp'}],
						 ['https',{'https'=>'tcp'}],
						 ['ftp',{'ftp'=>'tcp'}],
						 ['ssh',{'ssh'=>'tcp'}],
						 ['telnet',{'telnet'=>'tcp'}],
						 ['smtp',{'smtp'=>'tcp'}],
						 ['pop3',{'pop3'=>'tcp'}],
						 ['pop3s',{'pop3s'=>'tcp'}],
						 ['imap',{'imap'=>'tcp'}], 
						 ['imaps',{'imaps'=>'tcp'}],
						 ['dns',{'domain'=>'udp'}],
						 ['dhcp_server',{'bootps'=>'udp'}],
						 ['dhcp_client',{'bootpc'=>'udp'}],
						 ['gopher',{'gopher'=>'tcp'}],
						 ['finger',{'finger'=>'tcp'}],
						 ['ntp',{'ntp'=>'udp'}],
						 ['samba',{'netbios-ns'=>'udp',
						 						 'netbios-dgm'=>'udp',
												 'netbios-ssn'=>'tcp',
												 'microsoft-ds'=>'tcp'}],
						 ['squid',{'3128'=>'tcp'}],
						 ['dansguardian',{'8080'=>'tcp'}],
						 ['ztl_admin',{'10000'=>'tcp'}],
						 ['ztl_user',{'20000'=>'tcp'}]);

# &generate_clear_rules(\%hash)
# add iptables rules to %hash:
# clear existing rules and chains
sub generate_clear_rules
{
 # flush existing rules
 push (@{$_[0]},"\n# Flush existing rules");
 push (@{$_[0]},'iptables -t filter -F');
 push (@{$_[0]},'iptables -t nat -F');
 push (@{$_[0]},'iptables -t mangle -F');

 # delete custom chains
 push (@{$_[0]},"\n# Delete custom chains");
 push (@{$_[0]},'iptables -X');
}

# &generate_default_policies(\%hash,$input,$forward,$output)
# add iptables rules to %hash:
# set default policies
sub generate_default_policies
{
 my ($rule, $input, $forward, $output) = @_;
 
 # set default policies
 push (@{$rule},"\n# Set default policies");
 if ($input eq 'ACCEPT' && $forward eq 'ACCEPT' && $output eq 'ACCEPT') { push (@{$rule},"# (policies accept)"); }
 else { push (@{$rule},"# (policies default)"); }
 push (@{$rule},"iptables -P INPUT $input");
 push (@{$rule},"iptables -P FORWARD $forward");
 push (@{$rule},"iptables -P OUTPUT $output");
}

# &generate_prepare_rules(\%hash)
# add iptables rules to %hash:
# allow established, drop all invalid packets, accept all from the loopback device
sub generate_prepare_rules
{
 # allow established
 push (@{$_[0]},"\n# Allow input and forward established packets");
 push (@{$_[0]},'iptables -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT');
 push (@{$_[0]},'iptables -A FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT');
 
 # drop all invalid packets
 push (@{$_[0]},"\n# Drop input, output and forward all invalid packets");
 push (@{$_[0]},'iptables -A INPUT -m state --state INVALID -j DROP');
 push (@{$_[0]},'iptables -A FORWARD -m state --state INVALID -j DROP');
 push (@{$_[0]},'iptables -A OUTPUT -m state --state INVALID -j DROP');
 
 # accept all from the loopback device
 push (@{$_[0]},"\n# Allow input all packets from lo");
 push (@{$_[0]},'iptables -A INPUT -i lo -j ACCEPT');
 push (@{$_[0]},'iptables -A INPUT -s 127.0.0.1/255.0.0.0 ! -i lo -j DROP'); 
}

# &generate_icmp_rules(\%hash)
# add iptables rules to %hash:
# allow input and forward some ICMP packets
sub generate_icmp_rules
{
 my @icmp = (0,3,8,11);
 push (@{$_[0]},"\n# Allow ICMP packets types: echo reply, network unreachable, echo request, time-exceeded");
 foreach (@icmp)
 {
  push (@{$_[0]},"iptables -A INPUT -p icmp --icmp-type $_ -j ACCEPT");
  push (@{$_[0]},"iptables -A FORWARD -p icmp --icmp-type $_ -j ACCEPT");
 }
}

# &generate_private_rules(\%hash,$internet_interface)
# add iptables rules to %hash:
# drop all packets from private reserved networks on Internet interface
sub generate_private_rules
{
 my $wan_interface = $_[1];
 my @private_nets = ('10.0.0.0/8','192.168.0.0/16','172.16.0.0/12');
 push (@{$_[0]},"\n# Drop all packets from private reserved networks on the Internet interface");
 push (@{$_[0]},"# (priv $wan_interface)");
 foreach (@private_nets)
 {
  push (@{$_[0]},"iptables -A INPUT -i $wan_interface -s $_ -j DROP");
  push (@{$_[0]},"iptables -A FORWARD -o $wan_interface -d $_ -j DROP");
  push (@{$_[0]},"iptables -A OUTPUT -o $wan_interface -d $_ -j DROP");
 }
}

# &generate_lan_server_accept(\%hash,\@lan)
# add iptables rules to %hash:
# accept all packets from LANs to Server
sub generate_lan_server_accept
{
 my $lan = $_[1];
 push (@{$_[0]},"\n# Allow input all packets from LANs");
 push (@{$_[0]},"# (lan server any)");
 foreach (@{$lan})
 {
  push (@{$_[0]},"iptables -A INPUT -i $_ -j ACCEPT");
 }
}

# &generate_lan_internet_accept(\%hash,\@lan, [$wan], [1])
# add iptables rules to %hash:
# accept all packets from LANs to Internet
# if $_[3] -- without advanced comments
sub generate_lan_internet_accept
{
 my ($lan, $wan_interface) = ($_[1],$_[2]);
 if ($_[3])
 {
  if ($wan_interface) { push (@{$_[0]},"\n# Allow forward all packets from LANs, but not to Internet"); }
  else { push (@{$_[0]},"\n# Allow forward all packets from LANs"); }
 }
 else 
 { 
  push (@{$_[0]},"\n# Allow forward all packets from LANs"); 
  push (@{$_[0]},"# (lan internet any)");
 }
 foreach (@{$lan})
 {
  if ($wan_interface) { push (@{$_[0]},"iptables -A FORWARD -i $_ ! -o $wan_interface -j ACCEPT"); }
  else { push (@{$_[0]},"iptables -A FORWARD -i $_ -j ACCEPT"); }
 }
}

# &generate_internet_server_accept(\%hash,$wan)
# add iptables rules to %hash:
# accept all packets from Internet to Server
sub generate_internet_server_accept
{
 push (@{$_[0]},"\n# Allow input all packets from Internet");
 push (@{$_[0]},"# (internet server any)");
 push (@{$_[0]},"iptables -A INPUT -i $_[1] -j ACCEPT");
}

# &generate_service_rules(\%hash,\@lan, $internet_interface, $sorce, $dest, \%port, $service, [$protocol])
# sorce/dest: lan, server, internet
# protocol: tcp, udp
# add iptables rules to %hash:
# allow access for some services
sub generate_service_rules
{
 my $unpriports="1024:65535";
 my ($rules, $lan, $wan_interface, $sorce, $dest, $port, $service, $protocol) = @_;
 if ($sorce eq 'lan' && $dest eq 'internet')
 {
  push (@{$_[0]},"\n# Allow forward packets from LANs to $service Internet services");
  push (@{$_[0]},"# (lan internet $service $protocol)");
  foreach my $i (keys %{$port})
  {
   foreach (@{$lan})
   {
    push (@{$_[0]},"iptables -A FORWARD -p $port->{$i} -m $port->{$i} -i $_ -o $wan_interface --dport $i -j ACCEPT");
   }
  }
 }
 elsif ($sorce eq 'lan' && $dest eq 'server')
 {
  push (@{$_[0]},"\n# Allow forward packets from LANs to $service host services");
  push (@{$_[0]},"# (lan server $service $protocol)");
  foreach my $i (keys %{$port})
  {
   foreach (@{$lan})
   {
    push (@{$_[0]},"iptables -A INPUT -p $port->{$i} -m $port->{$i} -i $_ --dport $i -j ACCEPT");
   }
  }
 }
 elsif ($sorce eq 'internet' && $dest eq 'server')
 {
  push (@{$_[0]},"\n# Allow forward packets from Internet to $service host services");
  push (@{$_[0]},"# (internet server $service $protocol)");
  foreach my $i (keys %{$port})
  {
   push (@{$_[0]},"iptables -A INPUT -p $port->{$i} -m $port->{$i} -i $wan_interface --dport $i -j ACCEPT");
  }
 }
}

# &generate_proxy_rules(\%hash,\@lan)
# add iptables rules to %hash:
# redirect all query to Squid through DansGuardian
sub generate_proxy_rules
{
 my ($rules, $lan, $squid, $dg) = @_;
 
 # get Squid port
 &foreign_require('squid','squid-lib.pl');
 my $sq_port = &squid::get_squid_port();
 
 if ($squid && !$dg)
 {
  push (@{$_[0]},"\n# Redirect all WEB packets to Squid");
  push (@{$_[0]},"# (squid $sq_port)");
  foreach (@{$lan})
  {
   push (@{$_[0]},"iptables -t nat -A PREROUTING -i $_ -p tcp --dport http -j REDIRECT --to-port $sq_port");
  }
 } 
 elsif ($dg)
 {
  # get DG port
  &foreign_require('dg','dg-lib.pl');
  my $conf = &dg::parse_config_file();
  my $port = &dg::get_config_value('filterport', $conf); 
  
  push (@{$_[0]},"\n# Redirect all LAN to Squid packets to DansGuardian");
  push (@{$_[0]},"# (dg $port)");
  foreach (@{$lan})
  {
   push (@{$_[0]},"iptables -t nat -A PREROUTING -i $_ -p tcp --dport $sq_port -j REDIRECT --to-port $port");
  }
  if ($squid)
  {
   push (@{$_[0]},"\n# Redirect all WEB packets to DansGuardian");
   push (@{$_[0]},"# (squid $sq_port)");
   foreach (@{$lan})
   {
    push (@{$_[0]},"iptables -t nat -A PREROUTING -i $_ -p tcp --dport http -j REDIRECT --to-port $port");
   }
  }  
 }
}

# &generate_masquerade_rules(\%hash, internet_interface)
# add iptables rules to %hash:
# enable ip masquerade on Internet interface
sub generate_masquerade_rules
{
 my $wan_interface = $_[1];
 push (@{$_[0]},"\n# Enable MASQUERADE on the Internet interface");
 push (@{$_[0]},"# (nat $wan_interface)");
 push (@{$_[0]},"iptables -t nat -A POSTROUTING -o $wan_interface -j MASQUERADE");
}

# lan_check()
# return array refs of lan and wan interfaces
sub lan_check
{
 # require librarys from module net
 &foreign_require('net','net-lib.pl');

 # list of interfaces brought up at boot time
 my (@boot, @lan, $wan) = &net::boot_interfaces();
 foreach (@boot)
 {
  if ($_->{'name'} ne 'lo')
  {
   if ($_->{'address'}=~/^192\.168\./ || $_->{'address'}=~/^172\.[16..31]\./ || $_->{'address'}=~/^10\./ || $wan) { push (@lan,$_->{'fullname'}); }
	else { $wan = $_->{'fullname'}; }
  }
 }
 return (\@lan, $wan);
}

1;

1;
