#!/usr/bin/perl

&MAIN();

 sub MAIN {
# index.cgi
# Display current iptables firewall configuration from save file

&ModuleLoader::preload_module('firewall_class', "firewall::FirewallLib");
$self = new firewall::FirewallLib:: 'firewall_class',%$self;


$self->redirect("simple.cgi") if ($self->{'gconfig_hash'}->{'ztl_mode'} && ($self->{'gconfig_hash'}->{'os_type'} eq 'trustix-linux'));

$self->ReadParse();
$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, undef, "intro", 1, 1, 0,
	$self->help_search_link("iptables", "man", "doc"));

# Check for iptables and iptables-restore commands
if ($c = $self->missing_firewall_commands()) {
	print "<p>",$self->text('index_ecommand', "<tt>$c</tt>"),"<p>\n";
	$self->ui_print_footer("/", $self->{'text_hash'}->{'index'});
	exit;
	}

# Check if the kernel supports iptables
$self->{'out'} = `iptables -n -t filter -L OUTPUT 2>&1`;
if ($?) {
	print "<p>",$self->text('index_ekernel', "<pre>$self->{'out'}</pre>"),"<p>\n";
	$self->ui_print_footer("/", $self->{'text_hash'}->{'index'});
	exit;
	}

# Check if the distro supports iptables
if (!$self->{'config_hash'}->{'direct'} && $self->UNIVERSAL::can('check_iptables') &&
    ($self->{'err'} = $self->check_iptables())) {
	print "<p>$self->{'err'}</p>\n";
	$self->ui_print_footer("/", $self->{'text_hash'}->{'index'});
	exit;
	}

# Check if firewall is being started at boot
if (!$self->{'config_hash'}->{'direct'} && $self->foreign_check("init")) {
	$self->{'init_support'}++;
	if ($self->UNIVERSAL::can('started_at_boot')) {
		$self->{'atboot'} = $self->started_at_boot();
		}
	else {
		$self->foreign_require("init", "init-lib.pl");
		$self->{'atboot'} = $main::init_object->action_status("$self->{'gconfig_hash'}->{'product'}-iptables") == 2;
		}
	}

# Check if the save file exists. If not, check for any existing firewall
# rules, and offer to create a save file from them
@{$self->{'livetables_array'}} = $self->get_iptables_save("iptables-save |");
&shorewall_message(\@{$self->{'livetables_array'}});
if (!$self->{'config_hash'}->{'direct'} &&
    (!-s $self->{'iptables_save_file'} || $self->{'in_hash'}->{'reset'}) && $self->{'access_hash'}->{'setup'}) {
	@{$self->{'tables_array'}} = @{$self->{'livetables_array'}};
	foreach $t (@{$self->{'tables_array'}}) {
		$rules++ if (@{$t->{'rules'}});
		foreach $c (keys %{$t->{'defaults'}}) {
			$self->{'chains'}++ if ($t->{'defaults'}->{$c} ne 'ACCEPT');
			}
		$self->{'hastable_hash'}->{$t->{'name'}}++;
		}
	foreach $t (@{$self->{'known_tables_array'}}) {
		system("iptables -t $t -n -L >/dev/null") if (!$self->{'hastable_hash'}->{$t});
		}
	if (!$self->{'in_hash'}->{'reset'} && ($rules || $self->{'chains'})) {
		# Offer to save the current rules
		print $self->text('index_existing', $rules,
			    "<tt>$self->{'iptables_save_file'}</tt>"),"<p>\n";
		print "<form action=convert.cgi>\n";
		print "<center><input type=submit ",
		      "value='$self->{'text_hash'}->{'index_saveex'}'><p>\n";
		if ($self->{'init_support'} && !$self->{'atboot'}) {
			print "<input type=checkbox name=atboot value=1> ",
			      "$self->{'text_hash'}->{'index_atboot'}\n";
			}
		print "</center></form><p>\n";

		print "<table border width=100%>\n";
		print "<tr $self->{'tb'}><td><b>$self->{'text_hash'}->{'index_headerex'}</b></td></tr>\n";
		print "<tr $self->{'cb'}> <td><pre>";
		open(OUT, "iptables-save |");
		while(<OUT>) {
			print $self->html_escape($_);
			}
		close(OUT);
		print "</pre></td> </tr></table>\n";
		}
	else {
		# Offer to set up a firewall
		print $self->text($self->{'in_hash'}->{'reset'} ? 'index_rsetup' : 'index_setup',
			    "<tt>$self->{'iptables_save_file'}</tt>"),"<p>\n";
		print "<form action=setup.cgi>\n";
		print "<input type=hidden name=reset value='$self->{'in_hash'}->{'reset'}'>\n";
		print "<center><table><tr><td>\n";
		print "<input type=radio name=auto value=0 checked> ",
		      "$self->{'text_hash'}->{'index_auto0'}<p>\n";
		foreach $a (1 .. 4) {
			print "<input type=radio name=auto value=$a> ",
			      "$self->{'text_hash'}->{'index_auto'.$a} ",
			      $self->interface_choice("iface".$a),"<p>\n";
			}
		print "</td></tr></table>\n";
		print "<input type=submit value='$self->{'text_hash'}->{'index_auto'}'><p>\n";
		if ($self->{'init_support'} && !$self->{'atboot'}) {
			print "<input type=checkbox name=atboot value=1> ",
			      "$self->{'text_hash'}->{'index_atboot'}\n";
			}
		print "</center></form>\n";
		}
	}
else {
	$self->{'form'} = 0;
	@{$self->{'tables_array'}} = $self->get_iptables_save();
	if (!$self->{'config_hash'}->{'direct'}) {
		# Verify that all known tables exist, and if not add them to the
		# save file
		foreach $t (@{$self->{'tables_array'}}) {
			$self->{'hastable_hash'}->{$t->{'name'}}++;
			}
		foreach $t (@{$self->{'known_tables_array'}}) {
			if (!$self->{'hastable_hash'}->{$t}) {
				local ($missing) = $self->get_iptables_save(
						"iptables-save --table $t |");
				if ($missing) {
					delete($missing->{'line'});
					$self->save_table($missing);
					}
				$self->{'need_reload'}++;
				}
			}
		@{$self->{'tables_array'}} = $self->get_iptables_save() if ($self->{'need_reload'});
		}

	# Work out the default table
	if (!defined($self->{'in_hash'}->{'table'})) {
		foreach $t (@{$self->{'tables_array'}}) {
			if (@{$t->{'rules'}} && $self->can_edit_table($t->{'name'})) {
				$self->{'in_hash'}->{'table'} = $t->{'index'};
				last;
				}
			}
		}
	if (!defined($self->{'in_hash'}->{'table'})) {
		foreach $t (@{$self->{'tables_array'}}) {
			if ($self->can_edit_table($t->{'name'})) {
				$self->{'in_hash'}->{'table'} = $t->{'index'};
				last;
				}
			}
		}
	$self->{'table'} = $self->{'tables_array'}->[$self->{'in_hash'}->{'table'}];

	# Allow selection of a table
	print "<table width=100%><tr>\n";
	print "<form action=index.cgi>\n";
	print "<td><input type=submit value='$self->{'text_hash'}->{'index_change'}'>\n";
	print "<select name=table>\n";
	foreach $t (@{$self->{'tables_array'}}) {
		if ($self->can_edit_table($t->{'name'})) {
			printf "<option value=%s %s>%s\n",
			    $t->{'index'}, $t eq $self->{'table'} ? "selected" : "",
			    $self->text('index_table_'.$t->{'name'}) || $t->{'name'};
			}
		}
	print "</select></td></form>\n";
	$self->{'form'}++;

	if ($self->{'access_hash'}->{'newchain'}) {
		# Show form to create a chain
		print "<form action=newchain.cgi>\n";
		print "<td align=right><input type=hidden name=table ",
		      "value='$self->{'in_hash'}->{'table'}'>\n";
		print "<input type=submit value='$self->{'text_hash'}->{'index_cadd'}'>\n";
		print "<input name=chain size=20></td></form>\n";
		print "</tr></table>\n";
		$self->{'form'}++;
		}

	# Display a table of rules for each chain
	foreach $c (sort { $self->{'a'} = $a; $self->{'b'} = $b; $self->by_string_for_iptables } keys %{$self->{'table'}->{'defaults'}}) {
		print "<hr>\n";
		@{$self->{'rules_array'}} = grep { lc($_->{'chain'}) eq lc($c) }
			      @{$self->{'table'}->{'rules'}};
		print "<b>",$self->{'text_hash'}->{"index_chain_".lc($c)} ||
			    $self->text('index_chain', "<tt>$c</tt>"),"</b><br>\n";
		print "<form action=save_policy.cgi>\n";
		print "<input type=hidden name=table value='$self->{'in_hash'}->{'table'}'>\n";
		print "<input type=hidden name=chain value='$c'>\n";
		if (@{$self->{'rules_array'}}) {
			@{$self->{'links_array'}} = ( $self->select_all_link("d", $self->{'form'}),
				   $self->select_invert_link("d", $self->{'form'}) );
			print $self->ui_links_row(\@{$self->{'links_array'}});

			# Generate the header
			local (@hcols, @tds);
			push(@hcols, "", $self->{'text_hash'}->{'index_action'});
			push(@tds, "width=5", "width=10% nowrap");
			if ($self->{'config_hash'}->{'view_condition'}) {
				push(@hcols, $self->{'text_hash'}->{'index_desc'});
				push(@tds, "");
				}
			if ($self->{'config_hash'}->{'view_comment'}) {
				push(@hcols, $self->{'text_hash'}->{'index_comm'});
				push(@tds, "");
				}
			push(@hcols, $self->{'text_hash'}->{'index_move'}, $self->{'text_hash'}->{'index_add'});
			push(@tds, "width=32", "width=32");
			print $self->ui_columns_start(\@hcols, 100, 0, \@tds);

			# Generate a row for each rule
			foreach $r (@{$self->{'rules_array'}}) {
				$self->{'edit'} = $self->can_jump($r);
				local @cols;
				local $act =
				  $self->{'text_hash'}->{"index_jump_".lc($r->{'j'}->[1])} ||
				  $self->text('index_jump', $r->{'j'}->[1]);
				if ($self->{'edit'}) {
					push(@cols, "<a href='edit_rule.cgi?table=$self->{'in_hash'}->{'table'}&idx=$r->{'index'}'>$act</a>");
					}
				else {
					push(@cols, $act);
					}
				if ($self->{'config_hash'}->{'view_condition'}) {
					push(@cols, $self->describe_rule($r));
					}
				if ($self->{'config_hash'}->{'view_comment'}) {
					$self->{'cmt'} = $self->{'config_hash'}->{'comment_mod'} ?
					    $r->{'comment'}->[1] : $r->{'cmt'};
					push(@cols, $self->{'cmt'});
					}

				# Up/down mover
				local $mover;
				if ($r eq $self->{'rules_array'}->[@{$self->{'rules_array'}}-1]) {
					$mover .= "<img src=images/gap.gif>";
					}
				else {
					$mover .= "<a href='move.cgi?table=".
					      "$self->{'in_hash'}->{'table'}&idx=$r->{'index'}&".
					      "down=1'><img src=".
					      "images/down.gif border=0></a>";
					}
				if ($r eq $self->{'rules_array'}->[0]) {
					$mover .= "<img src=images/gap.gif>";
					}
				else {
					$mover .= "<a href='move.cgi?table=".
					      "$self->{'in_hash'}->{'table'}&idx=$r->{'index'}&".
					      "up=1'><img src=images/up.gif ".
					      "border=0></a>";
					}
				push(@cols, $mover);

				# Before / after adder
				local $adder;
				$adder .= "<a href='edit_rule.cgi?table=".
				      "$self->{'in_hash'}->{'table'}&chain=$c&new=1&".
				      "after=$r->{'index'}'><img src=".
				      "images/after.gif border=0></a>";
				$adder .= "<a href='edit_rule.cgi?table=".
				      "$self->{'in_hash'}->{'table'}&chain=$c&new=1&".
				      "before=$r->{'index'}'><img src=".
				      "images/before.gif border=0></a>";
				push(@cols, $adder);

				if ($self->{'edit'}) {
					print $self->ui_checked_columns_row(
					    \@cols, \@tds, "d", $r->{'index'});
					}
				else {
					print $self->ui_columns_row(\@cols, \@tds);
					}
				}
			print $self->ui_columns_end();
			print $self->ui_links_row(\@{$self->{'links_array'}});
			}
		else {
			print "<b>$self->{'text_hash'}->{'index_none'}</b><br>\n";
			}

		# Show policy changing button for chains that support it,
		# and rule-adding button
		print "<table width=100%><tr>\n";
		local $d = $self->{'table'}->{'defaults'}->{$c};
		if ($d ne '-') {
			if ($self->{'access_hash'}->{'policy'}) {
				# Change default button
				print "<td width=33% nowrap>",
				      $self->ui_submit($self->{'text_hash'}->{'index_policy'}),"\n";
				print "<select name=policy>\n";
				foreach $t ('ACCEPT','DROP','QUEUE','RETURN') {
					printf "<option value=%s %s>%s\n",
						$t, $d eq $t ? "selected" : "",
						$self->{'text_hash'}->{"index_policy_".lc($t)};
					}
				print "</select></td>\n";
				}
			else {
				print "<td width=33%></td>\n";
				}
			print "<td align=center width=33%>\n";
			if (@{$self->{'rules_array'}}) {
				# Delete selected button
				print $self->ui_submit($self->{'text_hash'}->{'index_cdeletesel'},
						 "delsel"),"\n";
				}
			print "</td>\n";
			}
		else {
			# Delete chain and delete selected buttons
			if ($self->{'access_hash'}->{'delchain'}) {
				print "<td width=33%>",
				   $self->ui_submit($self->{'text_hash'}->{'index_cdelete'}, "delete"),
				   "</td>\n";
				}
			print "<td align=center width=33%>\n";
			if (@{$self->{'rules_array'}}) {
				if ($self->{'access_hash'}->{'delchain'}) {
					print $self->ui_submit($self->{'text_hash'}->{'index_cclear'},
							 "clear"),"\n";
					print "&nbsp;" x 5;
					}
				print $self->ui_submit($self->{'text_hash'}->{'index_cdeletesel'},
						 "delsel"),"\n";
				}
			print "</td>\n";
			}
		print "<td align=right width=33%>",
		      $self->ui_submit($self->{'text_hash'}->{'index_radd'}, "add"),"</td>\n";
		print "</tr></table></form>\n";
		$self->{'form'}++;
		}

	# Display buttons for applying and un-applying the configuration,
	# and for creating an init script if possible
	print "<hr>\n";
	print "<table width=100%>\n";

	if (!$self->{'config_hash'}->{'direct'}) {
		if ($self->foreign_check("servers")) {
			@{$self->{'servers_array'}} = $self->list_cluster_servers();
			}
		if ($self->{'access_hash'}->{'apply'}) {
			print "<tr><form action=apply.cgi>\n";
			print "<input type=hidden name=table value='$self->{'in_hash'}->{'table'}'>\n";
			print "<td><input type=submit ",
			      "value='$self->{'text_hash'}->{'index_apply'}'></td>\n";
			if (@{$self->{'servers_array'}}) {
				print "<td>$self->{'text_hash'}->{'index_applydesc2'}</td>\n";
				}
			else {
				print "<td>$self->{'text_hash'}->{'index_applydesc'}</td>\n";
				}
			print "</form></tr>\n";
			}

		if ($self->{'access_hash'}->{'unapply'}) {
			print "<tr><form action=unapply.cgi>\n";
			print "<input type=hidden name=table value='$self->{'in_hash'}->{'table'}'>\n";
			print "<td><input type=submit ",
			      "value='$self->{'text_hash'}->{'index_unapply'}'></td>\n";
			print "<td>$self->{'text_hash'}->{'index_unapplydesc'}</td>\n";
			print "</form></tr>\n";
			}

		if ($self->{'init_support'} && $self->{'access_hash'}->{'bootup'}) {
			print "<tr><form action=bootup.cgi>\n";
			print "<input type=hidden name=table ",
			      "value='$self->{'in_hash'}->{'table'}'>\n";
			print "<td nowrap><input type=submit ",
			      "value='$self->{'text_hash'}->{'index_bootup'}'>\n";
			printf "<input type=radio name=boot value=1 %s> %s\n",
				$self->{'atboot'} ? "checked" : "", $self->{'text_hash'}->{'yes'};
			printf "<input type=radio name=boot value=0 %s> %s\n",
				$self->{'atboot'} ? "" : "checked", $self->{'text_hash'}->{'no'};
			print "</td> <td>$self->{'text_hash'}->{'index_bootupdesc'}</td>\n";
			print "</form></tr>\n";
			}

		if ($self->{'access_hash'}->{'setup'}) {
			print "<tr><form action=index.cgi>\n";
			print "<input type=hidden name=reset value=1>\n";
			print "<td><input type=submit ",
			      "value='$self->{'text_hash'}->{'index_reset'}'></td>\n";
			print "<td>$self->{'text_hash'}->{'index_resetdesc'}</td>\n";
			print "</form></tr>\n";
			}
		}

	# Show button for cluster page
	if ($self->foreign_check("servers")) {
		$self->foreign_require("servers", "servers-lib.pl");
		@{$self->{'allservers_array'}} = grep { $_->{'user'} }
				$main::servers_object->list_servers();
		}
	if ($self->{'access_hash'}->{'cluster'} && @{$self->{'allservers_array'}}) {
		print "<tr><form action=cluster.cgi>\n";
		print "<td><input type=submit ",
		      "value='$self->{'text_hash'}->{'index_cluster'}'></td>\n";
		print "<td>$self->{'text_hash'}->{'index_clusterdesc'}</td>\n";
		print "</form></tr>\n";
		}

	print "</table>\n";
	}

$self->ui_print_footer("/", $self->{'text_hash'}->{'index'});

1;
		
}

sub shorewall_message
{
	shift if ($self == $_[0]);

local ($filter) = grep { $_->{'name'} eq 'filter' } @{$_[0]};
if ($filter->{'defaults'}->{'shorewall'}) {
	print "<b><center>",
	      $self->text('index_shorewall', "$self->{'gconfig_hash'}->{'webprefix'}/shorewall/"),
	      "</b></center><p>\n";
	}
}


1;

