#!/usr/bin/perl

&MAIN();

 sub MAIN {
# setup.cgi
# Setup an initial save file

&ModuleLoader::preload_module('firewall_class', "firewall::FirewallLib");
$self = new firewall::FirewallLib:: 'firewall_class',%$self;

$self->ReadParse();
$self->{'access_hash'}->{'setup'} || $self->error($self->{'text_hash'}->{'setup_ecannot'});

$self->lock_file($self->{'iptables_save_file'});
if ($self->{'in_hash'}->{'reset'}) {
	# Clear out all rules
	foreach $t ("filter", "nat", "mangle") {
		$self->system_logged("iptables -t $t -P INPUT ACCEPT >/dev/null 2>&1");
		$self->system_logged("iptables -t $t -P OUTPUT ACCEPT >/dev/null 2>&1");
		$self->system_logged("iptables -t $t -P FORWARD ACCEPT >/dev/null 2>&1");
		$self->system_logged("iptables -t $t -P PREROUTING ACCEPT >/dev/null 2>&1");
		$self->system_logged("iptables -t $t -P POSTROUTING ACCEPT >/dev/null 2>&1");
		$self->system_logged("iptables -t $t -F >/dev/null 2>&1");
		$self->system_logged("iptables -t $t -X >/dev/null 2>&1");
		}
	}

# Save all existing active rules
if ($self->UNIVERSAL::can('unapply_iptables')) {
	$self->unapply_iptables();
	}
else {
	$self->backquote_logged("iptables-save >$self->{'iptables_save_file'} 2>&1");
	}

if ($self->{'in_hash'}->{'auto'}) {
	@{$self->{'tables_array'}} = $self->get_iptables_save();
	if ($self->{'in_hash'}->{'auto'} == 1) {
		# Add a single rule to the nat table for masquerading
		$self->{'iface'} = $self->{'in_hash'}->{'iface1'} || $self->{'in_hash'}->{'iface1_other'};
		$self->{'iface'} || $self->error($self->{'text_hash'}->{'setup_eiface'});
		($self->{'table'}) = grep { $_->{'name'} eq 'nat' } @{$self->{'tables_array'}};
		push(@{$self->{'table'}->{'rules'}},
		     	{ 'chain' => 'POSTROUTING',
			  'o' => [ "", $self->{'iface'} ],
			  'j' => [ "", 'MASQUERADE' ] } );
		}
	elsif ($self->{'in_hash'}->{'auto'} >= 2) {
		# Block all incoming traffic, except for established
		# connections, DNS replies and safe ICMP types
		# In mode 3 allow ssh and ident too
		# In mode 4 allow ftp, echo-request and high ports too
		$self->{'iface'} = $self->{'in_hash'}->{'iface'.$self->{'in_hash'}->{'auto'}} ||
			 $self->{'in_hash'}->{'iface'.$self->{'in_hash'}->{'auto'}.'_other'};
		$self->{'iface'} || $self->error($self->{'text_hash'}->{'setup_eiface'});
		($self->{'table'}) = grep { $_->{'name'} eq 'filter' } @{$self->{'tables_array'}};
		$self->{'table'}->{'defaults'}->{'INPUT'} = 'DROP';
		push(@{$self->{'table'}->{'rules'}},
		     { 'chain' => 'INPUT',
		       'i' => [ "!", $self->{'iface'} ],
		       'j' => [ "", 'ACCEPT' ],
		       'cmt' => 'Accept traffic from internal interfaces' },
		     { 'chain' => 'INPUT',
		       'm' => [ [ "", "tcp" ] ],
		       'p' => [ "", "tcp" ],
		       'tcp-flags' => [ "", "ACK", "ACK" ],
		       'j' => [ "", 'ACCEPT' ],
		       'cmt' => 'Accept traffic with the ACK flag set' },
		     { 'chain' => 'INPUT',
		       'm' => [ [ "", "state" ] ],
		       'state' => [ "", "ESTABLISHED" ],
		       'j' => [ "", 'ACCEPT' ],
		       'cmt' => 'Allow incoming data that is part of a connection we established' },
		     { 'chain' => 'INPUT',
		       'm' => [ [ "", "state" ] ],
		       'state' => [ "", "RELATED" ],
		       'j' => [ "", 'ACCEPT' ],
		       'cmt' => 'Allow data that is related to existing connections' },
		     { 'chain' => 'INPUT',
		       'm' => [ [ "", "udp" ] ],
		       'p' => [ "", "udp" ],
		       'sport' => [ "", 53 ],
		       'dport' => [ "", "1024:65535" ],
		       'j' => [ "", 'ACCEPT' ],
		       'cmt' => 'Accept responses to DNS queries' },
		     { 'chain' => 'INPUT',
		       'm' => [ [ "", "icmp" ] ],
		       'p' => [ [ "", "icmp" ] ],
		       'icmp-type' => [ "", "echo-reply" ],
		       'j' => [ "", 'ACCEPT' ],
		       'cmt' => 'Accept responses to our pings' },
		     { 'chain' => 'INPUT',
		       'm' => [ [ "", "icmp" ] ],
		       'p' => [ [ "", "icmp" ] ],
		       'icmp-type' => [ "", "destination-unreachable" ],
		       'j' => [ "", 'ACCEPT' ],
		       'cmt' => 'Accept notifications of unreachable hosts' },
		     { 'chain' => 'INPUT',
		       'm' => [ [ "", "icmp" ] ],
		       'p' => [ [ "", "icmp" ] ],
		       'icmp-type' => [ "", "source-quench" ],
		       'j' => [ "", 'ACCEPT' ],
		       'cmt' => 'Accept notifications to reduce sending speed' },
		     { 'chain' => 'INPUT',
		       'm' => [ [ "", "icmp" ] ],
		       'p' => [ [ "", "icmp" ] ],
		       'icmp-type' => [ "", "time-exceeded" ],
		       'j' => [ "", 'ACCEPT' ],
		       'cmt' => 'Accept notifications of lost packets' },
		     { 'chain' => 'INPUT',
		       'm' => [ [ "", "icmp" ] ],
		       'p' => [ [ "", "icmp" ] ],
		       'icmp-type' => [ "", "parameter-problem" ],
		       'j' => [ "", 'ACCEPT' ],
		       'cmt' => 'Accept notifications of protocol problems' }
			);
		if ($self->{'in_hash'}->{'auto'} >= 3) {
			# Allow ssh and ident
			push(@{$self->{'table'}->{'rules'}},
			     { 'chain' => 'INPUT',
			       'm' => [ [ "", "tcp" ] ],
			       'p' => [ "", "tcp" ],
			       'dport' => [ "", "ssh" ],
			       'j' => [ "", 'ACCEPT' ],
			       'cmt' => 'Allow connections to our SSH server' },
			     { 'chain' => 'INPUT',
			       'm' => [ [ "", "tcp" ] ],
			       'p' => [ "", "tcp" ],
			       'dport' => [ "", "auth" ],
			       'j' => [ "", 'ACCEPT' ],
			       'cmt' => 'Allow connections to our IDENT server'}
				);
			}
		if ($self->{'in_hash'}->{'auto'} == 4) {
			# Allow pings and most high ports
			push(@{$self->{'table'}->{'rules'}},
			     { 'chain' => 'INPUT',
			       'm' => [ [ "", "icmp" ] ],
			       'p' => [ [ "", "icmp" ] ],
			       'icmp-type' => [ "", "echo-request" ],
			       'j' => [ "", 'ACCEPT' ],
			       'cmt' => 'Respond to pings' },
			     { 'chain' => 'INPUT',
			       'm' => [ [ "", "tcp" ] ],
			       'p' => [ "", "tcp" ],
			       'dport' => [ "", "2049:2050" ],
			       'j' => [ "", 'DROP' ],
			       'cmt' => 'Protect our NFS server' },
			     { 'chain' => 'INPUT',
			       'm' => [ [ "", "tcp" ] ],
			       'p' => [ "", "tcp" ],
			       'dport' => [ "", "6000:6063" ],
			       'j' => [ "", 'DROP' ],
			       'cmt' => 'Protect our X11 display server' },
			     { 'chain' => 'INPUT',
			       'm' => [ [ "", "tcp" ] ],
			       'p' => [ "", "tcp" ],
			       'dport' => [ "", "7000:7010" ],
			       'j' => [ "", 'DROP' ],
			       'cmt' => 'Protect our X font server' },
			     { 'chain' => 'INPUT',
			       'm' => [ [ "", "tcp" ] ],
			       'p' => [ "", "tcp" ],
			       'dport' => [ "", "1024:65535" ],
			       'j' => [ "", 'ACCEPT' ],
			       'cmt' => 'Allow connections to unprivileged ports' },
				);
			}
		}
	$self->run_before_command();
	$self->save_table($self->{'table'});
	$self->run_after_command();
	$self->copy_to_cluster();
	}

if ($self->{'in_hash'}->{'atboot'}) {
	$self->create_firewall_init();
	}
$self->unlock_file($self->{'iptables_save_file'});

$self->webmin_log("setup");
$self->redirect("");

1;
		
}


1;

