#!/usr/bin/perl
package frox::FroxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'frox::FroxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Common functions for editing the Frox config file

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
@{$self->{'ui_tds_array'}} = ( undef, "nowrap" );

1;

1;
		return $self;
}

# get_config()
# Returns an array reference containing the contents of the Frox config file
sub get_config
{
	my $self = shift;

    if (!defined(@{$self->{'get_config_cache_array'}})) {
	    local $lnum = 0;
	    open(CONF, $self->{'config_hash'}->{'frox_conf'});
	    while(<CONF>) {
		s/\r|\n//g;
		s/^\s*#.$//g;
		if (/^\s*(\S+)\s*(.*)/) {
		    push(@{$self->{'get_config_cache_array'}},
			      { 'name' => $1,
				'value' => $2,
				'words' => [ split(/\s+/, $2) ],
				'line' => $lnum,
				'index' => scalar(@{$self->{'get_config_cache_array'}}) });
		}
		$lnum++;
	    }
	    close(CONF);
    }
    return \@{$self->{'get_config_cache_array'}};
}

# find(name, &config)
sub find
{
	my $self = shift;

    local @rv;
    foreach $c (@{$_[1]}) {
	if (lc($c->{'name'}) eq lc($_[0])) {
	    push(@rv, $c);
	}
    }
    return wantarray ? @rv : $rv[0];
}

# find_value(name, &config)
sub find_value
{
	my $self = shift;

    local @rv = $self->find($_[0], $_[1]);
    if (wantarray) {
	return map { $_->{'value'} } @rv;
	}
    else {
	return $rv[0]->{'value'};
	}
}

# save_directive(&config, name, &values)
# Update the config file with some new values for an option
sub save_directive
{
	my $self = shift;

local ($conf, $name, $values) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'frox_conf'});
local @old = $self->find($name, $conf);
local @new = @$values;
local $i;
local $changed;
for($i=0; $i<@old || $i<@new; $i++) {
	if ($i<@old && $i<@new) {
		# Updating some directive
		$lref->[$old[$i]->{'line'}] = $name." ".$new[$i];
		$old[$i]->{'value'} = $new[$i];
		$changed = $old[$i];
		}
	elsif ($i<@old) {
		# Removing some directive (need to renumber)
		splice(@$lref, $old[$i]->{'line'}, 1);
		splice(@$conf, $old[$i]->{'index'}, 1);
		$self->renumber($conf, 'line', $old[$i]->{'line'}, -1);
		$self->renumber($conf, 'index', $old[$i]->{'index'}, -1);
		}
	elsif ($i<@new) {
		# Adding some directive (perhaps after same)
		local $nd = { 'name' => $name,
			      'value' => $new[$i] };
		local ($j, $cmtline);
		for($j=0; $j<@$lref; $j++) {
			if ($lref->[$j] =~ /^\s*#+\s*(\S+)/ && $1 eq $name) {
				$cmtline = $j;
				}
			}
		if ($changed) {
			# Adding after same
			$nd->{'line'} = $changed->{'line'}+1;
			$nd->{'index'} = $changed->{'index'}+1;
			$self->renumber($conf, 'line', $changed->{'line'}, 1);
			$self->renumber($conf, 'index', $changed->{'index'}, 1);
			splice(@$lref, $changed->{'line'}+1, 0,
			       $name." ".$new[$i]);
			splice(@$conf, $changed->{'index'}+1, 0, $nd);
			$changed = $nd;
			}
		elsif (defined($cmtline)) {
			# Adding after comment of same directive
			local ($aftercmt) = grep { $_->{'line'} > $cmtline } @$conf;
			$nd->{'line'} = $cmtline;
			$nd->{'index'} = $aftercmt ? $aftercmt->{'index'} : 0;
			$self->renumber($conf, 'line', $nd->{'line'}-1, 1);
			$self->renumber($conf, 'index', $nd->{'index'}-1, 1);
			splice(@$lref, $cmtline+1, 0,
			       $name." ".$new[$i]);
			splice(@$conf, $nd->{'index'}, 0, $nd);
			$changed = $nd;
			}
		else {
			# Adding at end
			$nd->{'line'} = scalar(@$lref);
			$nd->{'index'} = scalar(@$conf);
			push(@$lref, $name." ".$new[$i]);
			push(@$conf, $nd);
			}
		}
	}
}

# renumber(&conf, field, pos, offset)
sub renumber
{
	my $self = shift;

local ($conf, $field, $pos, $offset) = @_;
local $c;
foreach $c (@$conf) {
	if ($c->{$field} > $pos) {
		$c->{$field} += $offset;
		}
	}
}

# config_textbox(&config, name, size, cols, [prefix])
# Returns HTML for a table row for some text field
sub config_textbox
{
	my $self = shift;

local ($conf, $name, $size, $cols, $prefix) = @_;
$cols ||= 1;
local $val = join(" ", $self->find_value($name, $conf));
$val =~ s/\s*'(.*)'\s*$/$1/;
return $self->ui_table_row($self->{'text_hash'}->{'edit_'.$name},
		     $self->ui_textbox($name, $val, $size)." ".$prefix,
		     $cols, \@{$self->{'ui_tds_array'}});
}

# save_textbox(config, name, [function], [split])
sub save_textbox
{
	my $self = shift;

local ($conf, $name, $func, $split) = @_;
local @vals = $split ? split(/\s+/, $self->{'in_hash'}->{$name}) : ( $self->{'in_hash'}->{$name} );
local $v;
foreach $v (@vals) {
	$self->error_check($name, $func, $v);
	$v = "'$v'" if ($v =~ /\s/);
	}
$self->save_directive($conf, $name, \@vals);
}

# config_opt_textbox(&config, name, size, cols, [default])
# Returns HTML for a table row for some optional text field
sub config_opt_textbox
{
	my $self = shift;

local ($conf, $name, $size, $cols, $default) = @_;
$cols ||= 1;
local $val = join(" ", $self->find_value($name, $conf));
$val =~ s/\s*'(.*)'\s*$/$1/;
return $self->ui_table_row($self->{'text_hash'}->{'edit_'.$name},
		     $self->ui_radio($name."_def", $val ? 0 : 1,
			       [ [ 1, $default || $self->{'text_hash'}->{'default'} ],
				 [ 0, " " ] ])." ".
		     $self->ui_textbox($name, $val, $size), $cols, \@{$self->{'ui_tds_array'}});
}

# save_opt_textbox(config, name, [function], [split])
sub save_opt_textbox
{
	my $self = shift;

local ($conf, $name, $func, $split) = @_;
if ($self->{'in_hash'}->{$name."_def"}) {
	$self->save_directive($conf, $name, [ ]);
	}
else {
	$self->save_textbox(@_);
	}
}

# config_yesno(&config, name, [yes], [no], [default])
# Returns HTML for a table row for yes/no checkboxes
sub config_yesno
{
	my $self = shift;

local ($conf, $name, $yes, $no, $default) = @_;
local $val = $self->find_value($name, $conf);
$val ||= $default;
return $self->ui_table_row($self->{'text_hash'}->{'edit_'.$name},
		     $self->ui_radio($name, $val =~ /yes|on|true/i ? 1 : 0,
			       [ [ 1, $yes || $self->{'text_hash'}->{'yes'} ],
				 [ 0, $no || $self->{'text_hash'}->{'no'} ] ]), 1, \@{$self->{'ui_tds_array'}}).
       $self->ui_hidden($name."_default", $default);
}

# save_yesno(&config, name, default)
sub save_yesno
{
	my $self = shift;

local ($conf, $name, $default) = @_;
local $val = $self->find_value($name, $conf);
$val ||= $self->{'in_hash'}->{$name."_default"};
local $curr = $val =~ /yes|on|true/i ? 1 : 0;
if ($curr ne $self->{'in_hash'}->{$name}) {
	$self->save_directive($conf, $name, [ $self->{'in_hash'}->{$name} ? "yes" : "no" ]);
	}
}

# config_exists(&config, name, [yes], [no])
# Returns HTML for a table row for yes/no checkboxes, based on the existance
# of some config item
sub config_exists
{
	my $self = shift;

local ($conf, $name, $yes, $no) = @_;
local $exists = $self->find($name, $conf);
return $self->ui_table_row($self->{'text_hash'}->{'edit_'.$name},
		     $self->ui_radio($name, $exists ? 1 : 0,
			       [ [ 1, $yes || $self->{'text_hash'}->{'yes'} ],
				 [ 0, $no || $self->{'text_hash'}->{'no'} ] ]), 1, \@{$self->{'ui_tds_array'}});
}

# save_exists(&config, name)
sub save_exists
{
	my $self = shift;

local ($conf, $name) = @_;
$self->save_directive($conf, $name, $self->{'in_hash'}->{$name} ? [ "" ] : [ ]);
}

# config_user(&config, name)
# Returns HTML for a table row for a username field
sub config_user
{
	my $self = shift;

local ($conf, $name) = @_;
local $val = $self->find_value($name, $conf);
return $self->ui_table_row($self->{'text_hash'}->{'edit_'.$name},
		     $self->ui_user_textbox($name, $val, $self->{'size'}), 1, \@{$self->{'ui_tds_array'}});
}

# save_user(&config, name)
# Saves a username field
sub save_user
{
	my $self = shift;

local ($conf, $name) = @_;
defined(getpwnam($self->{'in_hash'}->{$name})) ||
	$self->error($self->{'text_hash'}->{'edit_'.$name}." : ".$self->{'text_hash'}->{'edit_euser'});
$self->save_directive($conf, $name, [ $self->{'in_hash'}->{$name} ]);
}

# config_group(&config, name)
# Returns HTML for a table row for a groupname field
sub config_group
{
	my $self = shift;

local ($conf, $name) = @_;
local $val = $self->find_value($name, $conf);
return $self->ui_table_row($self->{'text_hash'}->{'edit_'.$name},
		     $self->ui_group_textbox($name, $val, $self->{'size'}), 1, \@{$self->{'ui_tds_array'}});
}

# save_group(&config, name)
# Saves a group name field
sub save_group
{
	my $self = shift;

local ($conf, $name) = @_;
defined(getgrnam($self->{'in_hash'}->{$name})) ||
	$self->error($self->{'text_hash'}->{'edit_'.$name}." : ".$self->{'text_hash'}->{'edit_egroup'});
$self->save_directive($conf, $name, [ $self->{'in_hash'}->{$name} ]);
}

# config_opt_range(&config, name, cols, default)
# Returns HTML for a row containing a two-part range input field
sub config_opt_range
{
	my $self = shift;

local ($conf, $name, $cols, $default) = @_;
$cols ||= 1;
local $val = $self->find_value($name, $conf);
local ($v1, $v2) = split(/\-/, $val);
return $self->ui_table_row($self->{'text_hash'}->{'edit_'.$name},
		     $self->ui_radio($name."_def", $val ? 0 : 1,
			       [ [ 1, $default || $self->{'text_hash'}->{'default'} ],
				 [ 0, " " ] ])." ".
		     $self->{'text_hash'}->{'edit_from'}." ".
		     $self->ui_textbox($name."_from", $v1, 6)." ".
		     $self->{'text_hash'}->{'edit_to'}." ".
		     $self->ui_textbox($name."_to", $v2, 6), $cols, \@{$self->{'ui_tds_array'}});
}

# save_opt_range(config, name)
sub save_opt_range
{
	my $self = shift;

local ($conf, $name, $func, $split) = @_;
if ($self->{'in_hash'}->{$name."_def"}) {
	$self->save_directive($conf, $name, [ ]);
	}
else {
	local $v1 = $self->{'in_hash'}->{$name."_from"};
	local $v2 = $self->{'in_hash'}->{$name."_to"};
	$v1 =~ /^\d+$/ && $v1 > 0 && $v2 < 65535 ||
		$self->error($self->{'text_hash'}->{'edit_'.$name}." : ".$self->{'text_hash'}->{'edit_efrom'});
	$v2 =~ /^\d+$/ && $v2 > 0 && $v2 < 65536 ||
		$self->error($self->{'text_hash'}->{'edit_'.$name}." : ".$self->{'text_hash'}->{'edit_eto'});
	$v1 < $v2 || $self->error($self->{'text_hash'}->{'edit_'.$name}." : ".$self->{'text_hash'}->{'edit_erange'});
	$self->save_directive($conf, $name, [ $v1."-".$v2 ]);
	}
}

# error_check(name, func, value)
sub error_check
{
	my $self = shift;

local ($name, $func, $val) = @_;
return if (!$func);
local $err = $self->$func($val);
$self->error($self->{'text_hash'}->{'edit_'.$name}." : ".$err) if ($err);
}

# is_frox_running()
# Returns the PID if Frox is running, 0 if not
sub is_frox_running
{
	my $self = shift;

local $conf = $self->get_config();
local $pidfile = $self->find_value("PidFile", $conf);
if ($pidfile) {
	open(PID, $pidfile) || return 0;
	chop($pid = <PID>);
	close(PID);
	return $pid && kill(0, $pid) ? $pid : 0;
	}
else {
	local ($pid) = $self->find_byname("frox");
	return $pid;
	}
}

# restart_frox()
# Apply the current frox configuration, or return an error message
sub restart_frox
{
	my $self = shift;

if ($self->{'config_hash'}->{'apply_cmd'}) {
	$self->{'out'} = $self->backquote_logged("($self->{'config_hash'}->{'apply_cmd'}) 2>&1 </dev/null");
	if ($?) {
		return "<pre>$self->{'out'}</pre>";
		}
	}
else {
	$self->{'pid'} = $self->is_frox_running();
	$self->{'pid'} || return $self->{'text_hash'}->{'stop_egone'};
	$self->kill_logged('HUP', $self->{'pid'}) || return $self->{'text_hash'}->{'stop_egone'};
	}
return undef;
}


1;

