#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Enable or disable the iptables rule

&ModuleLoader::preload_module('frox_class', "frox::FroxLib");
$self = new frox::FroxLib:: 'frox_class',%$self;

$self->foreign_require("firewall", "firewall-lib.pl");
$self->{'conf'} = $self->get_config();
$self->{'port'} = $self->find_value("Port", $self->{'conf'});
$self->error_setup($self->{'text_hash'}->{'iptables_err'});
$self->ReadParse();

# Validate inputs
if ($self->{'in_hash'}->{'enabled'} == 1) {
	gethostbyname($self->{'in_hash'}->{'net'}) ||
	    ($self->{'in_hash'}->{'net'} =~ /^([0-9\.]+)\/(\d+)$/ &&
	     $self->check_ipaddress($1) && $2 > 0 && $2 <= 32) ||
		$self->error($self->{'text_hash'}->{'iptables_enet'});
	}
elsif ($self->{'in_hash'}->{'enabled'} == 2) {
	$self->{'iface'} = $self->{'in_hash'}->{'iface'} eq 'other' ? $self->{'in_hash'}->{'iface_other'} : $self->{'in_hash'}->{'iface'};
	$self->{'iface'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'iptables_eiface'});
	}

# Get the old rule
@{$self->{'tables_array'}} = $main::firewall_object->get_iptables_save();
($self->{'nat'}) = grep { $_->{'name'} eq 'nat'} @{$self->{'tables_array'}};
if ($self->{'in_hash'}->{'rule'} ne "") {
	($self->{'rule'}) = $self->{'nat'}->{'rules'}->[$self->{'in_hash'}->{'rule'}];
	}

if ($self->{'in_hash'}->{'enabled'} && !$self->{'rule'}) {
	# Need to create
	$self->{'rule'} = { 'chain' => 'PREROUTING',
		  'j' => [ '', 'REDIRECT' ],
		  'p' => [ '', 'tcp' ],
		  'm' => [ '', 'tcp' ],
		  'dport' => [ '', 21 ],
		  'to-ports' => [ '', $self->{'port'} ],
		  ($self->{'iface'} ? ( 'i' => [ '', $self->{'iface'} ] )
			  : ( 's' => [ '', $self->{'in_hash'}->{'net'} ] ) ),
		  'cmt' => 'Forward FTP connections to Frox proxy' };
	push(@{$self->{'nat'}->{'rules'}}, $self->{'rule'});
	$self->{'apply'} = 1;
	}
elsif ($self->{'in_hash'}->{'enabled'} && $self->{'rule'}) {
	# Need to update rule
	if ($self->{'iface'}) {
		delete($self->{'rule'}->{'s'});
		$self->{'rule'}->{'i'} = [ '', $self->{'iface'} ];
		}
	else {
		delete($self->{'rule'}->{'i'});
		$self->{'rule'}->{'s'} = [ '', $self->{'in_hash'}->{'net'} ];
		}
	$self->{'apply'} = 1;
	}
elsif (!$self->{'in_hash'}->{'enabled'} && $self->{'rule'}) {
	# Need to delete
	splice(@{$self->{'nat'}->{'rules'}}, $self->{'in_hash'}->{'rule'}, 1);
	$self->{'apply'} = 2;
	}
else {
	$self->{'apply'} = 0;
	}
if ($self->{'apply'}) {
	# Save and apply
	$self->lock_file($main::firewall_object->{'iptables_save_file'});
	$main::firewall_object->save_table($self->{'nat'});
	$self->unlock_file($main::firewall_object->{'iptables_save_file'});
	$self->{'err'} = $main::firewall_object->apply_configuration();
	$self->error($self->text('iptables_eapply', $self->{'err'})) if ($self->{'err'});
	$self->webmin_log("iptables", $self->{'apply'});
	}

$self->redirect("");

1;
		
}


1;

