#!/usr/bin/perl
package fsdump::FsdumpLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'fsdump::FsdumpLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# fsdump-lib.pl
# Common functions for doing filesystem backups with dump
# XXX multi-volume tar restore

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
if ($self->{'gconfig_hash'}->{'os_type'} =~ /^\S+\-linux$/) {
	&ModuleLoader::preload_module(ref($self), "fsdump::LinuxLib");
$self = new fsdump::LinuxLib:: ref($self), %$self; 
	}
else {
	{
    my $pkg = $self->lib_to_pkg("$self->{gconfig_hash}->{os_type}-lib.pl", "fsdump");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
	}
$self->foreign_require("mount", "mount-lib.pl");
%{$self->{'access_hash'}} = $self->get_module_acl();

$self->{'cron_cmd'} = "$self->{'module_config_directory'}/backup.pl";
$self->{'newtape_cmd'} = "$self->{'module_config_directory'}/newtape.pl";
$self->{'notape_cmd'} = "$self->{'module_config_directory'}/notape.pl";
$self->{'multi_cmd'} = "$self->{'module_config_directory'}/multi.pl";
$self->{'rmulti_cmd'} = "$self->{'module_config_directory'}/rmulti.pl";
$self->{'ftp_cmd'} = "$self->{'module_config_directory'}/ftp.pl";

1;

1;
		return $self;
}

# list_dumps()
# Returns a list of all scheduled dumps
sub list_dumps
{
	my $self = shift;

local (@rv, $f);
opendir(DIR, $self->{'module_config_directory'});
foreach $f (sort { $a cmp $b } readdir(DIR)) {
	next if ($f !~ /^(\S+)\.dump$/);
	push(@rv, $self->get_dump($1));
	}
closedir(DIR);
return @rv;
}

# get_dump(id)
sub get_dump
{
	my $self = shift;

local %dump;
$self->read_file("$self->{'module_config_directory'}/$_[0].dump", \%dump) || return undef;
$dump{'id'} = $_[0];
return \%dump;
}

# save_dump(&dump)
sub save_dump
{
	my $self = shift;

$_[0]->{'id'} = $$.time() if (!$_[0]->{'id'});
$self->lock_file("$self->{'module_config_directory'}/$_[0]->{'id'}.dump");
$self->write_file("$self->{'module_config_directory'}/$_[0]->{'id'}.dump", $_[0]);
$self->unlock_file("$self->{'module_config_directory'}/$_[0]->{'id'}.dump");
}

# delete_dump(&dump)
sub delete_dump
{
	my $self = shift;

$self->lock_file("$self->{'module_config_directory'}/$_[0]->{'id'}.dump");
unlink("$self->{'module_config_directory'}/$_[0]->{'id'}.dump");
$self->unlock_file("$self->{'module_config_directory'}/$_[0]->{'id'}.dump");
}

# directory_filesystem(dir)
# Returns the filesystem type of some directory , or the full details
# if requesting an array
sub directory_filesystem
{
	my $self = shift;

local $fs;
foreach $m (sort { length($a->[0]) <=> length($b->[0]) }
	         $self->foreign_call("mount", "list_mounted")) {
	local $l = length($m->[0]);
	if ($m->[0] eq $_[0] || $m->[0] eq "/" ||
	    (length($_[0]) >= $l && substr($_[0], 0, $l+1) eq $m->[0]."/")) {
		$fs = $m;
		}
	}
return wantarray ? @$fs : $fs->[2];
}

# same_filesystem(fs1, fs2)
# Returns 1 if type filesystem types are the same
sub same_filesystem
{
	my $self = shift;

local ($fs1, $fs2) = @_;
$fs1 = "ext2" if ($fs1 eq "ext3");
$fs2 = "ext2" if ($fs2 eq "ext3");
return lc($fs1) eq lc($fs2);
}

# date_subs(string)
sub date_subs
{
	my $self = shift;

if ($self->{'config_hash'}->{'date_subs'}) {
	eval "use POSIX";
	eval "use posix" if ($@);
	local @tm = localtime(time());
	return strftime($_[0], @tm);
	}
else {
	return $_[0];
	}
}

# execute_before(&dump, handle, escape)
# Executes the before-dump command, and prints the output. Returns 1 on success
# or 0 on failure
sub execute_before
{
	my $self = shift;

if ($_[0]->{'before'}) {
	local $h = $_[1];
	$self->open_execute_command(*before, "($_[0]->{'before'}) 2>&1 </dev/null", 1);
	while(<before>) {
		print $h $_[2] ? $self->html_escape($_) : $_;
		}
	close(before);
	return !$?;
	}
return 1;
}

# execute_after(&dump, handle, escape)
sub execute_after
{
	my $self = shift;

if ($_[0]->{'after'}) {
	local $h = $_[1];
	$self->open_execute_command(*after, "($_[0]->{'after'}) 2>&1 </dev/null", 1);
	while(<after>) {
		print $h $_[2] ? $self->html_escape($_) : $_;
		}
	close(after);
	return !$?;
	}
return 1;
}

# running_dumps(&procs)
# Returns a list of backup jobs currently in progress, and their statuses
sub running_dumps
{
	my $self = shift;

local ($p, @rv, %got);
foreach $p (@{$_[0]}) {
	if (($p->{'args'} =~ /$self->{'cron_cmd'}\s+(\S+)/ ||
	    $p->{'args'} =~ /$self->{'module_root_directory'}\/backup.pl\s+(\S+)/) &&
	    $p->{'args'} !~ /^\/bin\/(sh|bash|csh|tcsh)/) {
		local $backup = $self->get_dump($1);
		local $sfile = "$self->{'module_config_directory'}/$1.$p->{'pid'}.status";
		local %status;
		$self->read_file($sfile, \%status);
		$backup->{'status'} = \%status;
		$backup->{'pid'} = $p->{'pid'};
		push(@rv, $backup);
		$got{$sfile} = 1 if (!$status{'end'});
		}
	}
# Remove any left over .status files
opendir(DIR, $self->{'module_config_directory'});
local $f;
foreach $f (readdir(DIR)) {
	local $path = "$self->{'module_config_directory'}/$f";
	unlink($path) if ($path =~ /\.status$/ && !$got{$path});
	}
closedir(DIR);
return @rv;
}

# can_edit_dir(dir)
# Returns 1 if some backup can be used or edited
sub can_edit_dir
{
	my $self = shift;

return 1 if ($self->{'access_hash'}->{'dirs'} eq '*');
local ($d, $dd);
local @ddirs = !ref($_[0]) ? ( $_[0] ) :
	       $self->{'supports_multiple'} ? split(/\s+/, $_[0]->{'dir'}) :
				    ( $_[0]->{'dir'} );
foreach $dd (@ddirs) {
	local $anyok = 0;
	foreach $d (split(/\t+/, $self->{'access_hash'}->{'dirs'})) {
		$anyok = 1 if ($self->is_under_directory($d, $dd));
		}
	return 0 if (!$anyok);
	}
return 1;
}

sub create_wrappers
{
	my $self = shift;

$self->foreign_require("cron", "cron-lib.pl");
$main::cron_object->create_wrapper($self->{'notape_cmd'}, $self->{'module_name'}, "notape.pl");
$main::cron_object->create_wrapper($self->{'newtape_cmd'}, $self->{'module_name'}, "newtape.pl");
$main::cron_object->create_wrapper($self->{'multi_cmd'}, $self->{'module_name'}, "multi.pl");
$main::cron_object->create_wrapper($self->{'rmulti_cmd'}, $self->{'module_name'}, "rmulti.pl");
}

# new_header(title)
sub new_header
{
	my $self = shift;

print "</table></td></tr></table><br>\n";
print "<table border width=100%>\n";
print "<tr $self->{'tb'}> <td><b>$_[0]</b></td> </tr>\n";
print "<tr $self->{'cb'}> <td><table width=100%>\n";
}

# dump_directories(&dump)
sub dump_directories
{
	my $self = shift;

if (!$self->multiple_directory_support($_[0]->{'fs'})) {
	return $_[0]->{'dir'};
	}
elsif ($_[0]->{'tabs'}) {
	return split(/\t+/, $_[0]->{'dir'});
	}
else {
	return split(/\s+/, $_[0]->{'dir'});
	}
}

# run_ssh_command(command, output-fh, output-mode, password)
# Run some command and display it's output, possibly providing a password
# if one is requested
sub run_ssh_command
{
	my $self = shift;

local ($cmd, $fh, $fhmode, $pass) = @_;
$self->foreign_require("proc", "proc-lib.pl");
local ($cfh, $fpid) = $main::proc_object->pty_process_exec_logged($cmd);
local ($wrong_password, $got_login, $connect_failed);
local $out;
while(1) {
	local $rv = $self->wait_for($cfh, "password:", "yes\\/no", "(^|\\n)\\s*Permission denied.*\n", "ssh: connect.*\n", ".*\n");
	if ($self->{'wait_for_input'} !~ /^\s*DUMP:\s+ACLs\s+in\s+inode/i) {
		if ($fhmode) {
			print $fh $self->html_escape($self->{'wait_for_input'});
			}
		else {
			print $fh $self->{'wait_for_input'};
			}
		}
	if ($rv == 0) {
		syswrite($cfh, "$pass\n");
		}
	elsif ($rv == 1) {
		syswrite($cfh, "yes\n");
		}
	elsif ($rv == 2) {
		$wrong_password++;
		last;
		}
	elsif ($rv == 3) {
		$connect_failed++;
		}
	elsif ($rv < 0) {
		last;
		}
	}
close($cfh);
local $got = waitpid($fpid, 0);
return $?;
}

# rsh_command_input(selname, textname, value)
# Returns HTML for selecting an rsh command
sub rsh_command_input
{
	my $self = shift;

local ($selname, $textname, $rsh) = @_;
local $ssh = $self->has_command("ssh");
local $r = $ssh && $rsh eq $ssh ? 1 :
	   $rsh eq $self->{'ftp_cmd'} ? 3 :
	   $rsh ? 2 : 0;
local @opts = ( [ 0, $self->{'text_hash'}->{'dump_rsh0'} ],
		[ 1, $self->{'text_hash'}->{'dump_rsh1'} ],
		[ 3, $self->{'text_hash'}->{'dump_rsh3'} ] );
if ($r == 2) {
	push(@opts, [ 2, $self->{'text_hash'}->{'dump_rsh2'}." ".
			 $self->ui_textbox($textname, $rsh, 30) ]);
	}
return $self->ui_radio($selname, $r, \@opts);
}

# rsh_command_parse(selname, textname)
# Returns the rsh command to use for a backup/restore, based on %{$self->{'in_hash'}}
sub rsh_command_parse
{
	my $self = shift;

local ($selname, $textname) = @_;
if ($self->{'in_hash'}->{$selname} == 0) {
	return undef;
	}
elsif ($self->{'in_hash'}->{$selname} == 1) {
	local $ssh = $self->has_command("ssh");
	$ssh || $self->error($self->{'text_hash'}->{'dump_essh'});
	return $ssh;
	}
elsif ($self->{'in_hash'}->{$selname} == 3) {
	return $self->{'ftp_cmd'};
	}
else {
	$self->{'in_hash'}->{$textname} =~ /^(\S+)/ && $self->has_command("$1") ||
		$self->error($self->{'text_hash'}->{'dump_ersh'});
	return $self->{'in_hash'}->{$textname};
	}
}


1;

