#!/usr/bin/perl
package fsdump::LinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'fsdump::LinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# linux-lib.pl

 $self->{'out'} = $self->backquote_command("dump -v 2>&1", 1);
if ($self->{'out'} =~ /dump\s+([0-9\.]+)b(\d+)/) {
	$self->{'dump_version'} = "$1$2";
	}
else {
	$self->{'out'} = $self->backquote_command("dump --version 2>&1", 1);
	if ($self->{'out'} =~ /dump\s+([0-9\.]+)b(\d+)/) {
		$self->{'dump_version'} = "$1$2";
		}
	}

$self->{'supports_tar'} = 1;

1;

1;
		return $self;
}

# supported_filesystems()
# Returns a list of filesystem types on which dumping is supported
sub supported_filesystems
{
	my $self = shift;

local @rv;
push(@rv, "ext2", "ext3") if ($self->has_command("dump"));
push(@rv, "xfs") if ($self->has_command("xfsdump"));
return @rv;
}

# multiple_directory_support(fs)
# Returns 1 if some filesystem dump supports multiple directories
sub multiple_directory_support
{
	my $self = shift;

return $_[0] eq "tar";
}

# dump_form(&dump)
sub dump_form
{
	my $self = shift;

# Display destination options
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_dest'}, "dest"),
   $self->ui_radio("mode", $_[0]->{'host'} ? 1 : 0,
	[ [ 0, $self->{'text_hash'}->{'dump_file'}." ".
	       $self->ui_textbox("file", $_[0]->{'file'}, 50).
	       " ".$self->file_chooser_button("file")."<br>" ],
	  [ 1, $self->text('dump_host',
		     $self->ui_textbox("host", $_[0]->{'host'}, 15),
		     $self->ui_textbox("huser", $_[0]->{'huser'}, 8),
		     $self->ui_textbox("hfile", $_[0]->{'hfile'}, 20)) ] ]), 3);

if ($_[0]->{'fs'} ne 'xfs') {
	# Display remote target options
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_rsh'},"rsh"),
		      $self->rsh_command_input("rsh_def", "rsh", $_[0]->{'rsh'}), 3);

	# Password option for SSH
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_pass2'},"pass2"),
		      $self->ui_password("pass", $_[0]->{'pass'}, 20), 3);
	}
}

sub dump_options_form
{
	my $self = shift;

local ($dump, $tds) = @_;
if ($_[0]->{'fs'} eq 'tar') {
	# Display gnutar options
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_label'},"label"),
			    $self->ui_textbox("label", $_[0]->{'label'}, 15),
			    1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_blocks'},"blocks"),
			    $self->ui_opt_textbox("blocks", $_[0]->{'blocks'}, 8,
					    $self->{'text_hash'}->{'dump_auto'})." kB",
			    3, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_gzip'},"gzip"),
			    $self->ui_select("gzip", int($_[0]->{'gzip'}),
				[ [ 0, $self->{'text_hash'}->{'no'} ],
				  [ 1, $self->{'text_hash'}->{'dump_gzip1'} ],
				  [ 2, $self->{'text_hash'}->{'dump_gzip2'} ] ]), 1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_multi'},"multi"),
			    $self->ui_yesno_radio("multi", int($_[0]->{'multi'})),
			    1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_links'},"links"),
			    $self->ui_yesno_radio("links", int($_[0]->{'links'})),
			    1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_xdev'},"xdev"),
			    $self->ui_yesno_radio("xdev", int($_[0]->{'xdev'})),
			    1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_notape'},"notape"),
			    $self->ui_radio("notape", int($_[0]->{'notape'}),
				  [ [ 0, $self->{'text_hash'}->{'yes'} ], [ 1, $self->{'text_hash'}->{'no'} ] ]),
			    1, $tds);
	}
elsif ($_[0]->{'fs'} eq 'xfs') {
	# Display xfs dump options
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_level'},"level"),
			    $self->ui_select("level", int($_[0]->{'level'}),
				[ map { [ $_, $self->{'text_hash'}->{'dump_level_'.$_} ] }
				      (0 .. 9) ]), 1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_label'},"label"),
			    $self->ui_textbox("label", $_[0]->{'label'}, 15),
			    1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_max'},"max"),
	    $self->ui_opt_textbox("max", $_[0]->{'max'}, 8,
			    $self->{'text_hash'}->{'dump_unlimited'})." kB", 1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_attribs'},"attribs"),
			    $self->ui_yesno_radio("attribs", int($_[0]->{'attribs'})),
			    1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_over'},"over"),
			    $self->ui_yesno_radio("over", int($_[0]->{'over'})),
			    1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_invent'},"invent"),
			    $self->ui_radio("noinvent", int($_[0]->{'noinvent'}),
			      [ [ 0, $self->{'text_hash'}->{'yes'} ], [ 1, $self->{'text_hash'}->{'no'} ] ]),
			    1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_overwrite'},"overwrite"),
		    $self->ui_yesno_radio("overwrite", int($_[0]->{'overwrite'})),
		    1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_erase'},"erase"),
			    $self->ui_yesno_radio("erase", int($_[0]->{'erase'})),
			    1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_bsize'},"bsize"),
	    $self->ui_opt_textbox("bsize", $_[0]->{'bsize'}, 8,
			    $self->{'text_hash'}->{'default'})." kB", 1, $tds);
	}
else {
	# Display ext2/3 filesystem dump options
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_update'},"update"),
			    $self->ui_yesno_radio("update", int($_[0]->{'update'})),
			    1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_multi'},"multi"),
			    $self->ui_yesno_radio("multi", int($_[0]->{'multi'})),
			    1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_level'},"level"),
			    $self->ui_select("level", int($_[0]->{'level'}),
				[ map { [ $_, $self->{'text_hash'}->{'dump_level_'.$_} ] }
				      (0 .. 9) ]), 1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_label'},"label"),
			    $self->ui_textbox("label", $_[0]->{'label'}, 15),
			    1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_blocks'},"blocks"),
			    $self->ui_opt_textbox("blocks", $_[0]->{'blocks'}, 8,
					    $self->{'text_hash'}->{'dump_auto'})." kB",
			    3, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_bsize'},"bsize"),
	    $self->ui_opt_textbox("bsize", $_[0]->{'bsize'}, 8,
			    $self->{'text_hash'}->{'default'})." kB", 1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_honour'},"honour"),
			    $self->ui_yesno_radio("honour", int($_[0]->{'honour'})),
			    1, $tds);

	if ($self->{'dump_version'} >= 0.424) {
		print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_comp'},"comp"),
			$self->ui_opt_textbox("comp", $_[0]->{'comp'}, 4,
					$self->{'text_hash'}->{'dump_complvl'}, $self->{'text_hash'}->{'no'}),
			3, $tds);
		}
	}

# Re-mount option
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_remount'},"remount"),
	$self->ui_yesno_radio("remount", int($_[0]->{'remount'})), 1, $tds);

if ($_[0]->{'fs'} eq 'tar') {
	# rmt path option
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_rmt'},"rmt"),
		$self->ui_opt_textbox("rmt", $_[0]->{'rmt'}, 30, $self->{'text_hash'}->{'default'}),
		3, $tds);
	}
}

# parse_dump(&dump)
sub parse_dump
{
	my $self = shift;

# Parse destination options
if ($self->{'in_hash'}->{'mode'} == 0) {
	$self->{'in_hash'}->{'file'} =~ /\S/ || $self->error($self->{'text_hash'}->{'dump_efile'});
	$_[0]->{'file'} = $self->{'in_hash'}->{'file'};
	delete($_[0]->{'host'});
	delete($_[0]->{'huser'});
	delete($_[0]->{'hfile'});
	}
else {
	gethostbyname($self->{'in_hash'}->{'host'}) || $self->check_ipaddress($self->{'in_hash'}->{'host'}) ||
		$self->error($self->{'text_hash'}->{'dump_ehost'});
	$_[0]->{'host'} = $self->{'in_hash'}->{'host'};
	$self->{'in_hash'}->{'huser'} =~ /^\S*$/ || $self->error($self->{'text_hash'}->{'dump_ehuser'});
	$_[0]->{'huser'} = $self->{'in_hash'}->{'huser'};
	$self->{'in_hash'}->{'hfile'} || $self->error($self->{'text_hash'}->{'dump_ehfile'});
	$_[0]->{'hfile'} = $self->{'in_hash'}->{'hfile'};
	delete($_[0]->{'file'});
	}

if ($_[0]->{'fs'} eq 'tar') {
	# Parse tar options
	$_[0]->{'rsh'} = $self->rsh_command_parse("rsh_def", "rsh");
	$_[0]->{'pass'} = $self->{'in_hash'}->{'pass'};
	$self->{'in_hash'}->{'label'} =~ /^\S*$/ && length($self->{'in_hash'}->{'label'}) < 16 ||
		$self->error($self->{'text_hash'}->{'dump_elabel'});
	$_[0]->{'label'} = $self->{'in_hash'}->{'label'};
	if ($self->{'in_hash'}->{'blocks_def'}) {
		delete($_[0]->{'blocks'});
		}
	else {
		$self->{'in_hash'}->{'blocks'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'dump_eblocks'});
		$_[0]->{'blocks'} = $self->{'in_hash'}->{'blocks'};
		$self->{'in_hash'}->{'gzip'} && $self->error($self->{'text_hash'}->{'dump_egzip'});
		}
	$_[0]->{'gzip'} = $self->{'in_hash'}->{'gzip'};
	$_[0]->{'multi'} = $self->{'in_hash'}->{'multi'};
	$_[0]->{'links'} = $self->{'in_hash'}->{'links'};
	$_[0]->{'xdev'} = $self->{'in_hash'}->{'xdev'};
	if ($self->{'in_hash'}->{'multi'}) {
		!-c $self->{'in_hash'}->{'file'} && !-b $self->{'in_hash'}->{'file'} ||
			$self->error($self->{'text_hash'}->{'dump_emulti'});
		$self->{'in_hash'}->{'gzip'} && $self->error($self->{'text_hash'}->{'dump_egzip2'});
		$self->{'in_hash'}->{'mode'} == 0 || $self->error($self->{'text_hash'}->{'dump_emulti2'});
		}
	$_[0]->{'notape'} = $self->{'in_hash'}->{'notape'};
	if ($self->{'in_hash'}->{'rmt_def'}) {
		delete($_[0]->{'rmt'});
		}
	else {
		$self->{'in_hash'}->{'rmt'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'dump_ermt'});
		$_[0]->{'rmt'} = $self->{'in_hash'}->{'rmt'};
		}
	}
elsif ($_[0]->{'fs'} eq 'xfs') {
	# Parse xfs options
	local $mp;
	foreach $m ($self->foreign_call("mount", "list_mounted")) {
		$mp++ if ($m->[0] eq $self->{'in_hash'}->{'dir'});
		}
	$mp || $self->error($self->{'text_hash'}->{'dump_emp'});
	$self->{'in_hash'}->{'label'} =~ /^\S*$/ && length($self->{'in_hash'}->{'label'}) < 256 ||
		$self->error($self->{'text_hash'}->{'dump_elabel2'});
	$_[0]->{'label'} = $self->{'in_hash'}->{'label'};
	$_[0]->{'level'} = $self->{'in_hash'}->{'level'};
	if ($self->{'in_hash'}->{'max_def'}) {
		delete($_[0]->{'max'});
		}
	else {
		$self->{'in_hash'}->{'max'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'dump_emax'});
		$_[0]->{'max'} = $self->{'in_hash'}->{'max'};
		}
	$_[0]->{'noattribs'} = $self->{'in_hash'}->{'noattribs'};
	$_[0]->{'over'} = $self->{'in_hash'}->{'over'};
	$_[0]->{'noinvent'} = $self->{'in_hash'}->{'noinvent'};
	$_[0]->{'overwrite'} = $self->{'in_hash'}->{'overwrite'};
	$_[0]->{'erase'} = $self->{'in_hash'}->{'erase'};
	if ($self->{'in_hash'}->{'bsize_def'}) {
		delete($_[0]->{'bsize'});
		}
	else {
		$self->{'in_hash'}->{'bsize'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'dump_ebsize'});
		$_[0]->{'bsize'} = $self->{'in_hash'}->{'bsize'};
		}
	}
else {
	# Parse ext2/3 options
	$_[0]->{'rsh'} = $self->rsh_command_parse("rsh_def", "rsh");
	$_[0]->{'pass'} = $self->{'in_hash'}->{'pass'};
	$_[0]->{'update'} = $self->{'in_hash'}->{'update'};
	$_[0]->{'multi'} = $self->{'in_hash'}->{'multi'};
	$_[0]->{'level'} = $self->{'in_hash'}->{'level'};
	$self->{'in_hash'}->{'label'} =~ /^\S*$/ && length($self->{'in_hash'}->{'label'}) < 16 ||
		$self->error($self->{'text_hash'}->{'dump_elabel'});
	$_[0]->{'label'} = $self->{'in_hash'}->{'label'};
	if ($self->{'in_hash'}->{'blocks_def'}) {
		delete($_[0]->{'blocks'});
		}
	else {
		$self->{'in_hash'}->{'blocks'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'dump_eblocks'});
		$_[0]->{'blocks'} = $self->{'in_hash'}->{'blocks'};
		}
	if ($self->{'in_hash'}->{'bsize_def'}) {
		delete($_[0]->{'bsize'});
		}
	else {
		$self->{'in_hash'}->{'bsize'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'dump_ebsize'});
		$_[0]->{'bsize'} = $self->{'in_hash'}->{'bsize'};
		}
	$_[0]->{'honour'} = $self->{'in_hash'}->{'honour'};
	if ($self->{'in_hash'}->{'comp_def'} || !defined($self->{'in_hash'}->{'comp'})) {
		delete($_[0]->{'comp'});
		}
	else {
		$self->{'in_hash'}->{'comp'} =~ /^[1-9]\d*$/ || $self->error($self->{'text_hash'}->{'dump_ecomp'});
		$_[0]->{'comp'} = $self->{'in_hash'}->{'comp'};
		}
	}
$_[0]->{'remount'} = $self->{'in_hash'}->{'remount'};
}

# execute_dump(&dump, filehandle, escape, background-mode)
# Executes a dump and displays the output
sub execute_dump
{
	my $self = shift;

local $fh = $_[1];
local ($cmd);
($self->{'flag'}, $self->{'hfile'}) = $self->dump_flag($_[0]);
local $tapecmd = $_[0]->{'multi'} && $_[0]->{'fs'} eq 'tar' ? $self->{'multi_cmd'} :
		 $_[0]->{'notape'} ? undef :
		 $_[0]->{'multi'} ? undef :
		 $_[3] && !$self->{'config_hash'}->{'nonewtape'} ? $self->{'newtape_cmd'} : $self->{'notape_cmd'};
local @dirs = split(/\s+/, $_[0]->{'dir'});
if ($_[0]->{'fs'} eq 'tar') {
	# tar format backup
	$cmd = "tar -c $self->{'flag'}";
	$cmd .= " -V '$_[0]->{'label'}'" if ($_[0]->{'label'});
	$cmd .= " -L $_[0]->{'blocks'}" if ($_[0]->{'blocks'});
	$cmd .= " -z" if ($_[0]->{'gzip'} == 1);
	$cmd .= " --bzip" if ($_[0]->{'gzip'} == 2);
	$cmd .= " -M" if ($_[0]->{'multi'});
	$cmd .= " -h" if ($_[0]->{'links'});
	$cmd .= " -l" if ($_[0]->{'xdev'});
	$cmd .= " -F \"$tapecmd $_[0]->{'id'}\"" if (!$_[0]->{'gzip'} && $tapecmd);
	$cmd .= " --rsh-command=".quotemeta($_[0]->{'rsh'}) if ($_[0]->{'rsh'});
	$cmd .= " --rmt-command=".quotemeta($_[0]->{'rmt'}) if ($_[0]->{'rmt'});
	$cmd .= " $_[0]->{'extra'}" if ($_[0]->{'extra'});
	$cmd .= " ".join(" ", map { "'$_'" } @dirs);
	}
elsif ($_[0]->{'fs'} eq 'xfs') {
	# xfs backup
	$cmd = "xfsdump -l $_[0]->{'level'}";
	$cmd .= $self->{'flag'};
	$cmd .= " -L '$_[0]->{'label'}'" if ($_[0]->{'label'});
	$cmd .= " -M '$_[0]->{'label'}'" if ($_[0]->{'label'});
	$cmd .= " -z '$_[0]->{'max'}'" if ($_[0]->{'max'});
	$cmd .= " -A" if ($_[0]->{'noattribs'});
	$cmd .= " -F" if ($_[0]->{'over'});
	$cmd .= " -J" if ($_[0]->{'noinvent'});
	$cmd .= " -o" if ($_[0]->{'overwrite'});
	$cmd .= " -E -F" if ($_[0]->{'erase'});
	$cmd .= " -b $_[0]->{'bsize'}" if ($_[0]->{'bsize'});
	$cmd .= " $_[0]->{'extra'}" if ($_[0]->{'extra'});
	$cmd .= " ".join(" ", map { "'$_'" } @dirs);
	}
else {
	# ext2/3 backup
	$cmd = "dump -$_[0]->{'level'}";
	$cmd .= $self->{'flag'};
	$cmd .= " -u" if ($_[0]->{'update'});
	$cmd .= " -M" if ($_[0]->{'multi'});
	$cmd .= " -L '$_[0]->{'label'}'" if ($_[0]->{'label'});
	$cmd .= " -B $_[0]->{'blocks'}" if ($_[0]->{'blocks'});
	$cmd .= " -b $_[0]->{'bsize'}" if ($_[0]->{'bsize'});
	$cmd .= " -h0" if ($_[0]->{'honour'});
	$cmd .= " -j$_[0]->{'comp'}" if ($_[0]->{'comp'});
	$cmd .= " -F \"$tapecmd $_[0]->{'id'}\"" if ($tapecmd);
	$cmd .= " $_[0]->{'extra'}" if ($_[0]->{'extra'});
	$cmd .= " '$_[0]->{'dir'}'";
	if ($_[0]->{'rsh'}) {
		$cmd = "RSH=\"$_[0]->{'rsh'}\" RMT=\"touch '$self->{'hfile'}'; /etc/rmt\" $cmd";
		}
	else {
		$cmd = "RMT=\"touch '$self->{'hfile'}'; /etc/rmt\" $cmd";
		}
	}

$self->system_logged("sync");
sleep(1);

# Remount with noatime, if needed
if ($_[0]->{'remount'}) {
	local @fs = $self->directory_filesystem($dirs[0]);
	$main::mount_object->parse_options($fs[2], $fs[3]);
	$main::mount_object->{'options_hash'}->{'noatime'} = '';
	$fs[3] = $main::mount_object->join_options($fs[2]);
	local $err = $main::mount_object->remount_dir(@fs);
	if ($err) {
		$err =~ s/<[^>]*>//g;
		print $fh "Failed to re-mount with noatime option : $err\n";
		return 0;
		}
	}

# Run the command, which may call SSH to do a remote login
$ENV{'DUMP_PASSWORD'} = $_[0]->{'pass'};
local $got = $self->run_ssh_command($cmd, $fh, $_[2], $_[0]->{'pass'});
if ($_[0]->{'multi'} && $_[0]->{'fs'} eq 'tar') {
	# Run multi-file switch command one last time
	$self->execute_command("$self->{'multi_cmd'} $_[0]->{'id'} >/dev/null 2>&1");
	}

# Remount with atime option
if ($_[0]->{'remount'}) {
	local @fs = $self->directory_filesystem($dirs[0]);
	$main::mount_object->parse_options($fs[2], $fs[3]);
	delete($main::mount_object->{'options_hash'}->{'noatime'});
	$main::mount_object->{'options_hash'}->{'atime'} = '';
	$fs[3] = $main::mount_object->join_options($fs[2]);
	local $err = $main::mount_object->remount_dir(@fs);
	}

return $got ? 0 : 1;
}

# dump_flag(&dump)
# Given a dump, returns the -f flag and server-side file
sub dump_flag
{
	my $self = shift;

local ($flag, $hfile);
if ($_[0]->{'huser'}) {
	$hfile = $self->date_subs($_[0]->{'hfile'});
	$flag = " -f '$_[0]->{'huser'}\@$_[0]->{'host'}:$hfile'";
	}
elsif ($_[0]->{'host'}) {
	$hfile = $self->date_subs($_[0]->{'hfile'});
	$flag = " -f '$_[0]->{'host'}:$hfile'";
	}
else {
	$flag = " -f '".$self->date_subs($_[0]->{'file'})."'";
	}
return ($flag, $hfile);
}

# verify_dump(&dump, filehandle, escape, background-mode)
# Verifies a dump, returning 1 if OK and 0 if not
sub verify_dump
{
	my $self = shift;

# Build verify command
local $fh = $_[1];
local $vcmd;
local ($flag, $hfile) = $self->dump_flag($_[0]);
if ($_[0]->{'fs'} eq "tar") {
	$vcmd = "tar -t -v";
	$vcmd .= " -z" if ($_[0]->{'gzip'} == 1);
	$vcmd .= " --bzip" if ($_[0]->{'gzip'} == 2);
	$vcmd .= " -M" if ($_[0]->{'multi'});
	}
elsif ($_[0]->{'fs'} eq "xfs") {
	$vcmd = "xfsrestore -t";
	}
else {
	$vcmd = "restore -t";
	$vcmd .= " -M" if ($_[0]->{'multi'});
	}
$vcmd .= $flag;
if ($_[0]->{'fs'} eq "tar") {
	$vcmd .= " --rsh-command=$_[0]->{'rsh'}" if ($_[0]->{'rsh'});
	}
elsif ($_[0]->{'fs'} ne "xfs") {
	if ($_[0]->{'rsh'}) {
		$vcmd = "RSH=\"$_[0]->{'rsh'}\" RMT=\"touch '$hfile'; /etc/rmt\" $vcmd";
		}
	else {
		$vcmd = "RMT=\"touch '$hfile'; /etc/rmt\" $vcmd";
		}
	}

# Run it
$vcmd .= " >/dev/null";
local $vgot = $self->run_ssh_command($vcmd, $fh, $_[2], $_[0]->{'pass'});
return $vgot ? 0 : 1;
}

# dump_dest(&dump)
sub dump_dest
{
	my $self = shift;

if ($_[0]->{'file'}) {
	return "<tt>".$self->html_escape($_[0]->{'file'})."</tt>";
	}
else {
	return "<tt>".$self->html_escape("$_[0]->{'host'}:$_[0]->{'hfile'}")."</tt>";
	}
}

# missing_restore_command(filesystem)
sub missing_restore_command
{
	my $self = shift;

local $cmd = $_[0] eq 'xfs' ? 'xfsrestore' : 'restore';
return $self->has_command($cmd) ? undef : $cmd;
}

# restore_form(filesystem, [&dump], &tds)
sub restore_form
{
	my $self = shift;

local ($fs, $dump, $tds) = @_;

# common options
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_src'}, "rsrc"),
   $self->ui_radio("mode", $_[1]->{'host'} ? 1 : 0,
	[ [ 0, $self->{'text_hash'}->{'dump_file'}." ".
	       $self->ui_textbox("file", $_[1]->{'file'}, 50).
	       " ".$self->file_chooser_button("file")."<br>" ],
	  [ 1, $self->text('dump_host',
		     $self->ui_textbox("host", $_[1]->{'host'}, 15),
		     $self->ui_textbox("huser", $_[1]->{'huser'}, 8),
		     $self->ui_textbox("hfile", $_[1]->{'hfile'}, 20)) ] ]), 3, $tds);

if ($_[0] eq 'tar') {
	# tar restore options
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_rsh'},"rrsh"),
		      $self->rsh_command_input("rsh_def", "rsh", $_[1]->{'rsh'}),
		      3, $tds);

	# Password option for SSH
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_pass2'},"passs"),
		      $self->ui_password("pass", $_[1]->{'pass'}, 20),
		      3, $tds);

	# Files to restore
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_files'},"rfiles"),
		      $self->ui_opt_textbox("files", undef, 40, $self->{'text_hash'}->{'restore_all'},
				      $self->{'text_hash'}->{'restore_sel'}), 3, $tds);

	# Target dir
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_dir'},"rdir"),
		      $self->ui_textbox("dir", undef, 50)." ".
		      $self->file_chooser_button("dir", 1), 3, $tds);

	# Restore permissions?
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_perms'},"perms"),
		      $self->ui_yesno_radio("perms", 1), 1, $tds);

	# Uncompress?
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_gzip'},"rgzip"),
		      $self->ui_select("gzip", $_[1]->{'gzip'},
				[ [ 0, $self->{'text_hash'}->{'no'} ],
				  [ 1, $self->{'text_hash'}->{'dump_gzip1'} ],
				  [ 2, $self->{'text_hash'}->{'dump_gzip2'} ] ]), 1, $tds);

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_keep'},"keep"),
		      $self->ui_yesno_radio("keep", 0), 1, $tds);

	# Multiple files
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_multi'},"rmulti"),
		      $self->ui_yesno_radio("multi", 0), 1, $tds);
	}
elsif ($_[0] eq 'xfs') {
	# xfs restore options

	# Target dir
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_dir'},"rdir"),
		      $self->ui_textbox("dir", undef, 50)." ".
		      $self->file_chooser_button("dir", 1), 3, $tds);

	# Overwrite
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_over'},"rover"),
		$self->ui_radio("over", 0, [ [ 0, $self->{'text_hash'}->{'restore_over0'} ],
				       [ 1, $self->{'text_hash'}->{'restore_over1'} ],
				       [ 2, $self->{'text_hash'}->{'restore_over2'} ] ]),
		3, $tds);

	# Attributes?
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_noattribs'},"rnoattribs"),
		$self->ui_radio("noattribs", 0, [ [ 0, $self->{'text_hash'}->{'yes'} ],
					    [ 1, $self->{'text_hash'}->{'no'} ] ]), 1, $tds);

	# Label to restore from
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_label'},"rlabel"),
		$self->ui_textbox("label", undef, 20), 1, $tds);
	}
else {
	# ext2/3 restore options
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_rsh'},"rrsh"),
		      $self->rsh_command_input("rsh_def", "rsh", $_[1]->{'rsh'}),
		      3, $tds);

	# Password option for SSH
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_pass2'},"passs"),
		      $self->ui_password("pass", $_[1]->{'pass'}, 20),
		      3, $tds);

	# Files to restore
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_files'},"rfiles"),
		      $self->ui_opt_textbox("files", undef, 40, $self->{'text_hash'}->{'restore_all'},
				      $self->{'text_hash'}->{'restore_sel'}), 3, $tds);

	# Target dir
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_dir'},"rdir"),
		      $self->ui_textbox("dir", undef, 50)." ".
		      $self->file_chooser_button("dir", 1), 3, $tds);

	# Multiple files
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_multi'},"rmulti"),
		      $self->ui_yesno_radio("multi", 0), 1, $tds);
	}

# Show only
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'restore_test'},"rtest"),
	      $self->ui_yesno_radio("test", 1), 1, $tds);
}

# parse_restore(filesystem)
# Parses inputs from restore_form() and returns a command to be passed to
# restore_backup()
sub parse_restore
{
	my $self = shift;

local $cmd;
if ($_[0] eq 'tar') {
	$cmd = "tar";
	if ($self->{'in_hash'}->{'test'}) {
		$cmd .= " -t -v";
		}
	else {
		$cmd .= " -x";
		}
	}
elsif ($_[0] eq 'xfs') {
	$cmd = "xfsrestore";
	$cmd .= " -t" if ($self->{'in_hash'}->{'test'});
	}
else {
	$cmd = "restore";
	$cmd .= ($self->{'in_hash'}->{'test'} ? " -t" : " -x");
	}
if ($self->{'in_hash'}->{'mode'} == 0) {
	$self->{'in_hash'}->{'file'} || $self->error($self->{'text_hash'}->{'restore_efile'});
	$cmd .= " -f '$self->{'in_hash'}->{'file'}'";
	}
else {
	gethostbyname($self->{'in_hash'}->{'host'}) || $self->check_ipaddress($self->{'in_hash'}->{'host'}) ||
		$self->error($self->{'text_hash'}->{'restore_ehost'});
	$self->{'in_hash'}->{'huser'} =~ /^\S*$/ || $self->error($self->{'text_hash'}->{'restore_ehuser'});
	$self->{'in_hash'}->{'hfile'} || $self->error($self->{'text_hash'}->{'restore_ehfile'});
	if ($self->{'in_hash'}->{'huser'}) {
		$cmd .= " -f '$self->{'in_hash'}->{'huser'}\@$self->{'in_hash'}->{'host'}:$self->{'in_hash'}->{'hfile'}'";
		}
	else {
		$cmd .= " -f '$self->{'in_hash'}->{'host'}:$self->{'in_hash'}->{'hfile'}'";
		}
	}
if ($_[0] eq 'tar') {
	# parse tar options
	$cmd .= " -p" if ($self->{'in_hash'}->{'perms'});
	$cmd .= " -z" if ($self->{'in_hash'}->{'gzip'} == 1);
	$cmd .= " --bzip" if ($self->{'in_hash'}->{'gzip'} == 2);
	$cmd .= " -k" if ($self->{'in_hash'}->{'keep'});
	if ($self->{'in_hash'}->{'multi'}) {
		!-c $self->{'in_hash'}->{'file'} && !-b $self->{'in_hash'}->{'file'} ||
			$self->error($self->{'text_hash'}->{'restore_emulti'});
		$self->{'in_hash'}->{'mode'} == 0 || $self->error($self->{'text_hash'}->{'restore_emulti2'});
		$cmd .= " -M -F \"$self->{'rmulti_cmd'} $self->{'in_hash'}->{'file'}\"";
		}
	local $rsh = $self->rsh_command_parse("rsh_def", "rsh");
	if ($rsh) {
		$cmd .= " --rsh-command=".quotemeta($rsh);
		}
	$cmd .= " $self->{'in_hash'}->{'extra'}" if ($self->{'in_hash'}->{'extra'});
	if (!$self->{'in_hash'}->{'files_def'}) {
		$self->{'in_hash'}->{'files'} || $self->error($self->{'text_hash'}->{'restore_efiles'});
		$cmd .= " $self->{'in_hash'}->{'files'}";
		}
	-d $self->{'in_hash'}->{'dir'} || $self->error($self->{'text_hash'}->{'restore_edir'});
	$cmd = "cd '$self->{'in_hash'}->{'dir'}' && $cmd";
	if ($self->{'in_hash'}->{'multi'}) {
		$cmd = "$self->{'rmulti_cmd'} $self->{'in_hash'}->{'file'} 1 && $cmd";
		}
	}
elsif ($_[0] eq 'xfs') {
	# parse xfs options
	$cmd .= " -E" if ($self->{'in_hash'}->{'over'} == 1);
	$cmd .= " -e" if ($self->{'in_hash'}->{'over'} == 2);
	$cmd .= " -A" if ($self->{'in_hash'}->{'noattribs'});
	$cmd .= " -L '$self->{'in_hash'}->{'label'}'" if ($self->{'in_hash'}->{'label'});
	$cmd .= " -F";
	$cmd .= " $self->{'in_hash'}->{'extra'}" if ($self->{'in_hash'}->{'extra'});
	if (!$self->{'in_hash'}->{'test'}) {
		-d $self->{'in_hash'}->{'dir'} || $self->error($self->{'text_hash'}->{'restore_edir'});
		$cmd .= " '$self->{'in_hash'}->{'dir'}'";
		}
	}
else {
	# parse ext2/3 options
	local $rsh = $self->rsh_command_parse("rsh_def", "rsh");
	if ($rsh) {
		$cmd = "RSH=\"$rsh\" $cmd";
		}

	if ($self->{'in_hash'}->{'multi'}) {
		$cmd .= " -M";
		if ($self->{'dump_version'} >= 0.428 && $self->{'in_hash'}->{'extra'} !~ /-a/) {
			$cmd .= " -a";
			}
		}
	$cmd .= " -u";		# force overwrite
	$cmd .= " $self->{'in_hash'}->{'extra'}" if ($self->{'in_hash'}->{'extra'});
	if (!$self->{'in_hash'}->{'files_def'}) {
		$self->{'in_hash'}->{'files'} || $self->error($self->{'text_hash'}->{'restore_efiles'});
		$cmd .= " $self->{'in_hash'}->{'files'}";
		}
	-d $self->{'in_hash'}->{'dir'} || $self->error($self->{'text_hash'}->{'restore_edir'});
	}
return $cmd;
}

# restore_backup(filesystem, command)
# Restores a backup based on inputs from restore_form(), and displays the results
sub restore_backup
{
	my $self = shift;

$self->additional_log('exec', undef, $_[1]);
$ENV{'DUMP_PASSWORD'} = $self->{'in_hash'}->{'pass'};
if ($_[0] eq 'xfs') {
	# Just run the xfsrestore command
	$self->open_execute_command(*CMD, "$_[1] 2>&1 </dev/null", 1);
	while(<CMD>) {
		print $self->html_escape($_);
		}
	close(CMD);
	return $? || undef;
	}
else {
	# Need to supply prompts
	$self->foreign_require("proc", "proc-lib.pl");
	local ($fh, $fpid) = $self->foreign_call("proc", "pty_process_exec", "cd '$self->{'in_hash'}->{'dir'}' ; $_[1]");
	local $donevolume;
	while(1) {
		local $rv = $self->wait_for($fh, "(.*next volume #)", "(.*set owner.mode for.*\\[yn\\])", "((.*)\\[yn\\])", "(.*enter volume name)", "password:", "yes\\/no", "(.*\\n)");
		last if ($rv < 0);
		print $self->html_escape($self->{'matches_array'}->[1]);
		if ($rv == 0) {
			if ($donevolume++) {
				return $self->{'text_hash'}->{'restore_evolume'};
				}
			else {
				syswrite($fh, "1\n", 2);
				}
			}
		elsif ($rv == 1) {
			syswrite($fh, "n\n", 2);
			}
		elsif ($rv == 3) {
			syswrite($fh, "\n", 1);
			}
		elsif ($rv == 2) {
			return $self->text('restore_equestion',
				     "<tt>$self->{'matches_array'}->[2]</tt>");
			}
		elsif ($rv == 4) {
			syswrite($fh, "$self->{'in_hash'}->{'pass'}\n");
			}
		elsif ($rv == 5) {
			syswrite($fh, "yes\n");
			}
		}
	close($fh);
	waitpid($fpid, 0);
	return $? || undef;
	}
}


1;

