#!/usr/bin/perl
package fsdump::SolarisLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'fsdump::SolarisLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# solaris-lib.pl

1;

1;
		return $self;
}

# supported_filesystems()
# Returns a list of filesystem types on which dumping is supported
sub supported_filesystems
{
	my $self = shift;

local @rv;
push(@rv, "ufs") if ($self->has_command("ufsdump"));
return @rv;
}

# multiple_directory_support(fs)
# Returns 1 if some filesystem dump supports multiple directories
sub multiple_directory_support
{
	my $self = shift;

return $_[0] eq "ufs";
}

# dump_form(&dump)
sub dump_form
{
	my $self = shift;

# Display common options
print "<tr> <td valign=top><b>",$self->hlink($self->{'text_hash'}->{'dump_dest'}, "dest"),
      "</b></td> <td colspan=3>\n";
printf "<input type=radio name=mode value=0 %s> %s\n",
	$_[0]->{'host'} ? '' : 'checked', $self->{'text_hash'}->{'dump_file'};
printf "<input name=file size=50 value='%s'> %s<br>\n",
	$_[0]->{'host'} ? '' : $_[0]->{'file'},
	$self->file_chooser_button("file");
printf "<input type=radio name=mode value=1 %s>\n",
	$_[0]->{'host'} ? 'checked' : '';
print $self->text('dump_host',
	    "<input name=host size=15 value='$_[0]->{'host'}'>",
	    "<input name=huser size=8 value='$_[0]->{'huser'}'>",
	    "<input name=hfile size=20 value='$_[0]->{'hfile'}'>"),
      "</td> </tr>\n";
}

sub dump_options_form
{
	my $self = shift;

print "<tr> <td><b>",$self->hlink($self->{'text_hash'}->{'dump_update'},"update"),
      "</b></td>\n";
printf "<td><input name=update type=radio value=1 %s> %s\n",
	$_[0]->{'update'} ? 'checked' : '', $self->{'text_hash'}->{'yes'};
printf "<input name=update type=radio value=0 %s> %s</td>\n",
	$_[0]->{'update'} ? '' : 'checked', $self->{'text_hash'}->{'no'};

print "<td><b>",$self->hlink($self->{'text_hash'}->{'dump_verify'},"verify"),"</b></td>\n";
printf "<td><input name=verify type=radio value=1 %s> %s\n",
	$_[0]->{'verify'} ? 'checked' : '', $self->{'text_hash'}->{'yes'};
printf "<input name=verify type=radio value=0 %s> %s</td> </tr>\n",
	$_[0]->{'verify'} ? '' : 'checked', $self->{'text_hash'}->{'no'};

print "<tr> <td><b>",$self->hlink($self->{'text_hash'}->{'dump_level'},"level"),"</b></td>\n";
print "<td><select name=level>\n";
foreach $l (0 .. 9) {
	printf "<option value=%d %s>%d %s\n",
		$l, $_[0]->{'level'} == $l ? "selected" : "", $l,
		$self->{'text_hash'}->{'dump_level_'.$l};
	}
print "</select></td>\n";

print "<td><b>",$self->hlink($self->{'text_hash'}->{'dump_offline'},"offline"),"</b></td>\n";
printf "<td><input name=offline type=radio value=1 %s> %s\n",
	$_[0]->{'offline'} ? 'checked' : '', $self->{'text_hash'}->{'yes'};
printf "<input name=offline type=radio value=0 %s> %s</td> </tr>\n",
	$_[0]->{'offline'} ? '' : 'checked', $self->{'text_hash'}->{'no'};
}

# parse_dump(&dump)
sub parse_dump
{
	my $self = shift;

# Parse common options
if ($self->{'in_hash'}->{'mode'} == 0) {
	$self->{'in_hash'}->{'file'} =~ /\S/ || $self->error($self->{'text_hash'}->{'dump_efile'});
	$_[0]->{'file'} = $self->{'in_hash'}->{'file'};
	delete($_[0]->{'host'});
	delete($_[0]->{'huser'});
	delete($_[0]->{'hfile'});
	}
else {
	gethostbyname($self->{'in_hash'}->{'host'}) || $self->check_ipaddress($self->{'in_hash'}->{'host'}) ||
		$self->error($self->{'text_hash'}->{'dump_ehost'});
	$_[0]->{'host'} = $self->{'in_hash'}->{'host'};
	$self->{'in_hash'}->{'huser'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'dump_ehuser'});
	$_[0]->{'huser'} = $self->{'in_hash'}->{'huser'};
	$self->{'in_hash'}->{'hfile'} || $self->error($self->{'text_hash'}->{'dump_ehfile'});
	$_[0]->{'hfile'} = $self->{'in_hash'}->{'hfile'};
	delete($_[0]->{'file'});
	}

$_[0]->{'update'} = $self->{'in_hash'}->{'update'};
$_[0]->{'verify'} = $self->{'in_hash'}->{'verify'};
$_[0]->{'level'} = $self->{'in_hash'}->{'level'};
$_[0]->{'offline'} = $self->{'in_hash'}->{'offline'};
}

# execute_dump(&dump, filehandle, escape)
# Executes a dump and displays the output
sub execute_dump
{
	my $self = shift;

local $fh = $_[1];
local $cmd = "ufsdump $_[0]->{'level'}";
$cmd .= "u" if ($_[0]->{'update'});
$cmd .= "v" if ($_[0]->{'verify'});
$cmd .= "o" if ($_[0]->{'offline'});
if ($_[0]->{'huser'}) {
	$cmd .= "f '$_[0]->{'huser'}\@$_[0]->{'host'}:".
		$self->date_subs($_[0]->{'hfile'})."'";
	}
elsif ($_[0]->{'host'}) {
	$cmd .= "f '$_[0]->{'host'}:".$self->date_subs($_[0]->{'hfile'})."'";
	}
else {
	$cmd .= "f '".$self->date_subs($_[0]->{'file'})."'";
	}
$cmd .= " $_[0]->{'extra'}" if ($_[0]->{'extra'});
$cmd .= " ".join(" ", map { "'$_'" } split(/\s+/, $_[0]->{'dir'}));

$self->system_logged("sync");
sleep(1);
$self->additional_log('exec', undef, $cmd);
$self->open_execute_command(*CMD, "$cmd 2>&1 </dev/null", 1);
while(<CMD>) {
	if ($_[2]) {
		print $fh $self->html_escape($_);
		}
	else {
		print $fh $_;
		}
	}
close(CMD);
return $? ? 0 : 1;
}

# dump_dest(&dump)
sub dump_dest
{
	my $self = shift;

if ($_[0]->{'file'}) {
	return "<tt>".$self->html_escape($_[0]->{'file'})."</tt>";
	}
elsif ($_[0]->{'huser'}) {
	return "<tt>".$self->html_escape("$_[0]->{'huser'}\@$_[0]->{'host'}:$_[0]->{'hfile'}")."</tt>";
	}
else {
	return "<tt>".$self->html_escape("$_[0]->{'host'}:$_[0]->{'hfile'}")."</tt>";
	}
}

# missing_restore_command(filesystem)
sub missing_restore_command
{
	my $self = shift;

return $self->has_command("ufsrestore") ? undef : "ufsrestore";
}

# restore_form(filesystem)
sub restore_form
{
	my $self = shift;

# common options
print "<tr> <td valign=top><b>",$self->hlink($self->{'text_hash'}->{'restore_src'}, "rsrc"),
      "</b></td>\n";
printf "<td colspan=3><input type=radio name=mode value=0 %s> %s\n",
        $_[1]->{'host'} ? "" : "checked", $self->{'text_hash'}->{'dump_file'};
printf "<input name=file size=50 value='%s'> %s<br>\n",
        $_[1]->{'host'} ? "" : $_[0]->{'file'}, $self->file_chooser_button("file");
printf "<input type=radio name=mode value=1 %s>\n",
        $_[1]->{'host'} ? "checked" : "";
print $self->text('dump_host',
            "<input name=host size=15 value='$_[1]->{'host'}'>",
            "<input name=huser size=8 value='$_[1]->{'huser'}'>",
            "<input name=hfile size=20 value='$_[1]->{'hfile'}'>"),
      "</td> </tr>\n";

print "<tr> <td><b>",$self->hlink($self->{'text_hash'}->{'restore_files'},"rfiles"),
      "</b></td>\n";
print "<td colspan=3><input type=radio name=files_def value=1 checked> ",
      "$self->{'text_hash'}->{'restore_all'}\n";
print "<input type=radio name=files_def value=0> $self->{'text_hash'}->{'restore_sel'}\n";
print "<input name=files size=40></td> </tr>\n";

print "<tr> <td><b>",$self->hlink($self->{'text_hash'}->{'restore_dir'},"rdir"),"</td>\n";
print "<td><input name=dir size=40> ",$self->file_chooser_button("dir", 1),
      "</td> </tr>\n";

print "<tr> <td><b>",$self->hlink($self->{'text_hash'}->{'restore_test'},"rtest"),"</td>\n";
print "<td><input type=radio name=test value=1> $self->{'text_hash'}->{'yes'}\n";
print "<input type=radio name=test value=0 checked> $self->{'text_hash'}->{'no'}</td> </tr>\n";
}

# parse_restore(filesystem)
# Parses inputs from restore_form() and returns a command to be passed to
# restore_backup()
sub parse_restore
{
	my $self = shift;

local $cmd;
$cmd = "ufsrestore";
$cmd .= ($self->{'in_hash'}->{'test'} ? " t" : " x");
if ($self->{'in_hash'}->{'mode'} == 0) {
	$self->{'in_hash'}->{'file'} || $self->error($self->{'text_hash'}->{'restore_efile'});
	$cmd .= "f '$self->{'in_hash'}->{'file'}'";
	}
else {
	gethostbyname($self->{'in_hash'}->{'host'}) || $self->check_ipaddress($self->{'in_hash'}->{'host'}) ||
		$self->error($self->{'text_hash'}->{'restore_ehost'});
	$self->{'in_hash'}->{'huser'} =~ /^\S*$/ || $self->error($self->{'text_hash'}->{'restore_ehuser'});
	$self->{'in_hash'}->{'hfile'} || $self->error($self->{'text_hash'}->{'restore_ehfile'});
	if ($self->{'in_hash'}->{'huser'}) {
		$cmd .= "f '$self->{'in_hash'}->{'huser'}\@$self->{'in_hash'}->{'host'}:$self->{'in_hash'}->{'hfile'}'";
		}
	else {
		$cmd .= "f '$self->{'in_hash'}->{'host'}:$self->{'in_hash'}->{'hfile'}'";
		}
	}

$cmd .= " $self->{'in_hash'}->{'extra'}" if ($self->{'in_hash'}->{'extra'});
if (!$self->{'in_hash'}->{'files_def'}) {
	$self->{'in_hash'}->{'files'} || $self->error($self->{'text_hash'}->{'restore_efiles'});
	$cmd .= " $self->{'in_hash'}->{'files'}";
	}
-d $self->{'in_hash'}->{'dir'} || $self->error($self->{'text_hash'}->{'restore_edir'});

return $cmd;
}

# restore_backup(filesystem, command)
# Restores a backup based on inputs from restore_form(), and displays the results
sub restore_backup
{
	my $self = shift;

$self->additional_log('exec', undef, $_[1]);

$self->foreign_require("proc", "proc-lib.pl");
local ($fh, $fpid) = $self->foreign_call("proc", "pty_process_exec", "cd '$self->{'in_hash'}->{'dir'}' ; $_[1]");
local $donevolume;
while(1) {
	local $rv = $self->wait_for($fh, "(next volume #)", "(set owner.mode for.*\\[yn\\])", "(Directories already exist, set modes anyway. \\[yn\\])", "((.*)\\[yn\\])", "(.*\\n)");
	last if ($rv < 0);
	print $self->html_escape($self->{'matches_array'}->[1]);
	if ($rv == 0) {
		if ($donevolume++) {
			return $self->{'text_hash'}->{'restore_evolume'};
			}
		else {
			syswrite($fh, "1\n", 2);
			}
		}
	elsif ($rv == 1 || $rv == 2) {
		syswrite($fh, "n\n", 2);
		}
	elsif ($rv == 3) {
		return $self->text('restore_equestion',
			     "<tt>$self->{'matches_array'}->[2]</tt>");
		}
	}
close($fh);
waitpid($fpid, 0);
return $? || undef;
}


1;

