#!/usr/bin/perl

&MAIN();

 sub MAIN {
# edit_dump.cgi
# Edit or create a filesystem backup

&ModuleLoader::preload_module('fsdump_class', "fsdump::FsdumpLib");
$self = new fsdump::FsdumpLib:: 'fsdump_class',%$self;

$self->foreign_require("cron", "cron-lib.pl");
$self->ReadParse();

if (!$self->{'in_hash'}->{'id'}) {
	# Adding a new backup of some type
	$self->{'access_hash'}->{'edit'} || $self->error($self->{'text_hash'}->{'dump_ecannot1'});
	$self->error_setup($self->{'text_hash'}->{'edit_err'});
	$self->{'in_hash'}->{'dir'} || $self->error($self->{'text_hash'}->{'edit_edir'});
	if ($self->{'supports_tar'} && ($self->{'config_hash'}->{'always_tar'} || $self->{'in_hash'}->{'forcetar'})) {
		# Always use tar format
		$self->{'fs'} = "tar";
		}
	else {
		# Work out filesystem type
		$self->{'fs'} = $self->directory_filesystem($self->{'in_hash'}->{'dir'});
		@{$self->{'supp_array'}} = $self->supported_filesystems();
		if ($self->indexof($self->{'fs'}, @{$self->{'supp_array'}}) < 0) {
			if ($self->{'supports_tar'}) {
				$self->{'fs'} = "tar";		# fall back to tar mode
				}
			else {
				$self->error($self->text('edit_efs', uc($self->{'fs'})));
				}
			}
		}
	$self->ui_print_header(undef, $self->{'text_hash'}->{'edit_title'}, "", "edit");
	$self->{'dump'} = { 'dir' => $self->{'in_hash'}->{'dir'},
		  'fs' => $self->{'fs'},
		  'rsh' => $self->has_command("ssh"),
		   $self->{'config_hash'}->{'simple_sched'} ?
			( 'special' => 'daily' ) :
			( 'mins' => '0',
			  'hours' => '0',
			  'days' => '*',
			  'months' => '*',
			  'weekdays' => '*' ) };
	}
else {
	# Editing an existing backup
	$self->{'dump'} = $self->get_dump($self->{'in_hash'}->{'id'});
	$self->{'access_hash'}->{'edit'} && $self->can_edit_dir($self->{'dump'}) ||
		$self->error($self->{'text_hash'}->{'dump_ecannot2'});
	$self->ui_print_header(undef, $self->{'text_hash'}->{'edit_title2'}, "", "create");
	}

@{$self->{'tds_array'}} = ( "width=30%" );
print $self->ui_form_start("save_dump.cgi", "post");
print $self->ui_hidden("id", $self->{'in_hash'}->{'id'}),"\n";
print $self->ui_hidden("fs", $self->{'dump'}->{'fs'}),"\n";
print $self->ui_table_start($self->text('edit_header', uc($self->{'dump'}->{'fs'})),
		      "width=100%", 2);

print $self->ui_table_row($self->{'text_hash'}->{'dump_format'},
	$self->{'dump'}->{'fs'} eq 'tar' ? $self->{'text_hash'}->{'dump_tar'}
		    : $self->text('dump_dumpfs', uc($self->{'dump'}->{'fs'})),
	undef, \@{$self->{'tds_array'}});

if (!$self->multiple_directory_support($self->{'dump'}->{'fs'})) {
	# One directory
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_dir'}, "dir"),
			    $self->ui_textbox("dir", $self->{'dump'}->{'dir'}, 50)."\n".
			    $self->file_chooser_button("dir", 1),
			    undef, \@{$self->{'tds_array'}});
	}
else {
	# Multiple directories
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_dirs'}, "dirs"),
		    $self->ui_textarea("dir", join("\n", $self->dump_directories($self->{'dump'})),
				 3, 50),
		    undef, \@{$self->{'tds_array'}});
	}

$self->dump_form($self->{'dump'}, \@{$self->{'tds_array'}});
print $self->ui_table_end();

print $self->ui_table_start($self->{'text_hash'}->{'edit_header3'}, "width=100%", 4);
if ($self->UNIVERSAL::can('dump_options_form')) {
	$self->dump_options_form($self->{'dump'}, \@{$self->{'tds_array'}});
	}

if ($self->UNIVERSAL::can('verify_dump')) {
	# Add option to verify, if supported
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_reverify'},"reverify"),
			    $self->ui_yesno_radio("reverify",
					    int($self->{'dump'}->{'reverify'})),
			    \@{$self->{'tds_array'}});
	}

# Extra command-line parameters
if ($self->{'access_hash'}->{'extra'}) {
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_extra'}, "extra"),
			    $self->ui_textbox("extra", $self->{'dump'}->{'extra'}, 60), 3,
			    \@{$self->{'tds_array'}});
	}

# Before and after commands
if ($self->{'access_hash'}->{'cmds'}) {
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_before'},"before"),
			    $self->ui_textbox("before", $self->{'dump'}->{'before'}, 60), 3,
			    \@{$self->{'tds_array'}});

	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'dump_after'},"after"),
			    $self->ui_textbox("after", $self->{'dump'}->{'after'}, 60), 3,
			    \@{$self->{'tds_array'}});
	}
print $self->ui_table_end();

print $self->ui_table_start($self->{'text_hash'}->{'edit_header2'}, "width=100%", 4);

# Show input for selecting when to run a dump, which can be never, on schedule
# or after some other dump
@{$self->{'dlist_array'}} = grep { $_->{'id'} ne $self->{'in_hash'}->{'id'} } $self->list_dumps();
if (@{$self->{'dlist_array'}}) {
	$self->{'follow'} = $self->ui_select("follow", $self->{'dump'}->{'follow'},
	    [ map { [ $_->{'id'}, &follow_desc($_) ] } @{$self->{'dlist_array'}} ]);
	}
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'edit_enabled'}, "enabled"),
	$self->ui_radio("enabled", $self->{'dump'}->{'follow'} ? 2 :
				$self->{'dump'}->{'enabled'} ? 1 : 0,
	     [ [ 0, $self->{'text_hash'}->{'edit_enabled_no'}."<br>" ],
	       @{$self->{'dlist_array'}} ? 
		( [ 2, $self->{'text_hash'}->{'edit_enabled_af'}." ".$self->{'follow'}."<br>" ] ) : ( ),
	       [ 1, $self->{'text_hash'}->{'edit_enabled_yes'} ] ]), 3, \@{$self->{'tds_array'}});

# Email address to send output to
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'edit_email'}, "email"),
		    $self->ui_textbox("email", $self->{'dump'}->{'email'}, 30), 3, \@{$self->{'tds_array'}});

# Subject line for email message
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'edit_subject'}, "subject"),
		    $self->ui_opt_textbox("subject", $self->{'dump'}->{'subject'},
				    40, $self->{'text_hash'}->{'default'}), 3, \@{$self->{'tds_array'}});

if (!$self->{'config_hash'}->{'simple_sched'} || ($self->{'dump'} && !$self->{'dump'}->{'special'})) {
	# Complex Cron time input
	print $self->ui_table_row(undef,
		"<table border width=100%>".
		$self->capture_function_output(\$main::cron_object->show_times_input, $self->{'dump'}).
		"</table>",
		4);
	}
else {
	# Simple input
	print $self->ui_hidden("special_def", 1),"\n";
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'edit_special'}, "special"),
		$self->ui_select("special", $self->{'dump'}->{'special'},
		    [ map { [ $_, $main::cron_object->{'text_hash'}->{'edit_special_'.$_} ] }
			  ('hourly', 'daily', 'weekly', 'monthly', 'yearly') ]),
		3, \@{$self->{'tds_array'}});
	}
print $self->ui_table_end();

if ($self->{'in_hash'}->{'id'}) {
	print $self->ui_form_end([ [ "save", $self->{'text_hash'}->{'save'} ],
			     [ "savenow", $self->{'text_hash'}->{'edit_savenow'} ],
			     [ "restore", $self->{'text_hash'}->{'edit_restore'} ],
			     [ "delete", $self->{'text_hash'}->{'delete'} ] ]);
	}
else {
	print $self->ui_form_end([ [ "create", $self->{'text_hash'}->{'create'} ],
			     [ "savenow", $self->{'text_hash'}->{'edit_createnow'} ] ]);
	}

$self->ui_print_footer("", $self->{'text_hash'}->{'index_return'});

1;
		
}

# follow_desc(&dump)
sub follow_desc
{
	shift if ($self == $_[0]);

local @dirs = $self->dump_directories($_[0]);
return $self->text(defined($_[0]->{'level'}) ? 'edit_tolevel' : 'edit_to',
	     $dirs[0], $self->dump_dest($_[0]), $_[0]->{'level'});
}


1;

