#!/usr/bin/perl
package htaccess_htpasswd::HtaccessLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'htaccess_htpasswd::HtaccessLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# htaccess-lib.pl
# Common functions for the htaccess and htpasswd file management module
# XXX check if MD5 is installed, if offered
# XXX same for SHA1

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
&ModuleLoader::preload_module(ref($self), "htaccess_htpasswd::HtpasswdfileLib");
$self = new htaccess_htpasswd::HtpasswdfileLib:: ref($self), %$self; 

@{$self->{'accessdirs_array'}} = ( );
if ($self->{'module_info_hash'}->{'usermin'}) {
	# Allowed directories are in module configuration
	$self->switch_to_remote_user();
	$self->create_user_config_dirs();
	$self->{'default_dir'} = $self->resolve_links($self->{'remote_user_info_array'}->[7]);
	push(@{$self->{'accessdirs_array'}}, $self->{'default_dir'}) if ($self->{'config_hash'}->{'home'});
	 $d;
	foreach $d (split(/\t+/, $self->{'config_hash'}->{'dirs'})) {
		push(@{$self->{'accessdirs_array'}}, $d =~ /^\// ? $d : "$self->{'default_dir'}/$d");
		}
	$self->{'directories_file'} = "$self->{'user_module_config_directory'}/directories";
	$self->{'apachemod'} = "htaccess";
	$self->{'can_htpasswd'} = $self->{'config_hash'}->{'can_htpasswd'};
	$self->{'can_htgroups'} = $self->{'config_hash'}->{'can_htgroups'};
	}
else {
	# Allowed directories come from ACL
	%{$self->{'access_hash'}} = $self->get_module_acl();
	 @{$self->{'uinfo_array'}};
	if ($self->supports_users()) {
		# Include user home
		@{$self->{'uinfo_array'}} = getpwnam($self->{'remote_user'});
		if ($self->{'access_hash'}->{'home'} && defined(@{$self->{'uinfo_array'}})) {
			push(@{$self->{'accessdirs_array'}}, $self->resolve_links($self->{'uinfo_array'}->[7]));
			}
		}
	 $d;
	foreach $d (split(/\t+/, $self->{'access_hash'}->{'dirs'})) {
		push(@{$self->{'accessdirs_array'}}, $d =~ /^\// || !@{$self->{'uinfo_array'}} ?
			$d : $self->resolve_links("$self->{'uinfo_array'}->[7]/$d"));
		}
	$self->{'directories_file'} = "$self->{'module_config_directory'}/directories";
	$self->{'directories_file'} .= ".".$self->{'remote_user'} if ($self->{'access_hash'}->{'userdirs'});
	$self->{'apachemod'} = "apache";
	$self->{'can_htpasswd'} = 1;
	$self->{'can_htgroups'} = 1;
	$self->{'default_dir'} = $self->{'accessdirs_array'}->[0];
	$self->{'can_sync'} = $self->{'access_hash'}->{'sync'};
	}

1;

1;
		return $self;
}

# list_directories([even-if-missing])
# Returns a list of protected directories known to this module, and the
# users file, encryption mode, sync mode and groups file for each
sub list_directories
{
	my $self = shift;

local @rv;
open(DIRS, $self->{'directories_file'});
while(<DIRS>) {
	s/\r|\n//g;
	local @dir = split(/\t+/, $_);
	next if (!@dir);
	if ($_[0] || -d $dir[0] && -r "$dir[0]/$self->{'config_hash'}->{'htaccess'}") {
		push(@rv, \@dir);
		}
	}
closedir(DIRS);
return @rv;
}

# save_directories(&dirs)
sub save_directories
{
	my $self = shift;

local $d;
$self->open_tempfile(DIRS, ">$self->{'directories_file'}");
foreach $d (@{$_[0]}) {
	$self->print_tempfile(DIRS, join("\t", @$d),"\n");
	}
$self->close_tempfile(DIRS);
}

# can_access_dir(dir)
# Returns 1 if files can be created under some directory, 0 if not
sub can_access_dir
{
	my $self = shift;

return 1 if (!$ENV{'GATEWAY_INTERFACE'});
local $d;
foreach $d (@{$self->{'accessdirs_array'}}) {
	return 1 if ($self->is_under_directory($self->resolve_links($d),
					 $self->resolve_links($_[0])));
	}
return 0;
}

# switch_user()
# Switch to the Unix user that files are accessed as.
# No need to do anything for Usermin, because the switch was done above.
sub switch_user
{
	my $self = shift;

if (!$self->{'module_info_hash'}->{'usermin'} &&
    $self->{'access_hash'}->{'user'} ne 'root' && !defined($self->{'old_uid'}) && $self->supports_users()) {
	local @uinfo = getpwnam($self->{'access_hash'}->{'user'} eq "*" ? $self->{'remote_user'}
						       : $self->{'access_hash'}->{'user'});
	$self->{'old_uid'} = $>;
	$self->{'old_gid'} = $);
	$) = "$uinfo[3] $uinfo[3]";
	$> = $uinfo[2];
	}
}

sub switch_back
{
	my $self = shift;

if (defined($self->{'old_uid'})) {
	$> = $self->{'old_uid'};
	$) = $self->{'old_gid'};
	$self->{'old_uid'} = $self->{'old_gid'} = undef;
	}
}


1;

