#!/usr/bin/perl

&MAIN();

 sub MAIN {
# save_dir.cgi
# Create or update a .htaccess file

&ModuleLoader::preload_module('htaccess_htpasswd_class', "htaccess_htpasswd::HtaccessLib");
$self = new htaccess_htpasswd::HtaccessLib:: 'htaccess_htpasswd_class',%$self;

$self->ReadParse();
@{$self->{'dirs_array'}} = $self->list_directories();
$self->error_setup($self->{'text_hash'}->{'dir_err'});
$self->foreign_require($self->{'apachemod'}, "apache-lib.pl");

# Work out what .htaccess file to use
$self->{'in_hash'}->{'dir'} =~ s/\\/\//g;	# use forward slashes
if ($self->{'in_hash'}->{'new'} && $self->{'in_hash'}->{'dir'} !~ /^([a-z]:)?\//i && $self->{'default_dir'} ne "/") {
	# Make path absolute
	$self->{'in_hash'}->{'dir'} = "$self->{'default_dir'}/$self->{'in_hash'}->{'dir'}";
	}
($self->{'dir'}) = grep { $_->[0] eq $self->{'in_hash'}->{'dir'} } @{$self->{'dirs_array'}};
if ($self->{'in_hash'}->{'new'}) {
	$self->{'dir'} && $self->error($self->{'text_hash'}->{'dir_eclash'});
	$self->{'htaccess'} = "$self->{'in_hash'}->{'dir'}/$self->{'config_hash'}->{'htaccess'}";
	}
else {
	$self->{'htaccess'} = "$self->{'dir'}->[0]/$self->{'config_hash'}->{'htaccess'}";
	}

$self->lock_file($self->{'htaccess'});
$self->lock_file($self->{'directories_file'});

# Get the apache directives for the .htaccess file, if any
$self->{'authz'} = $main::apache_object->{'httpd_modules_hash'}->{'mod_auth_digest'} >= 2.2;
$self->{'auf'} = $self->{'in_hash'}->{'crypt'} == 3 && !$self->{'authz'} ? "AuthDigestFile" : "AuthUserFile";
$self->{'agf'} = $self->{'in_hash'}->{'crypt'} == 3 && !$self->{'authz'} ? "AuthDigestGroupFile" : "AuthGroupFile";
$self->{'conf'} = $self->foreign_call($self->{'apachemod'}, "get_htaccess_config", $self->{'htaccess'});
$self->{'currfile'} = $self->foreign_call($self->{'apachemod'}, "find_directive",
			  $self->{'auf'}, $self->{'conf'}, 1);
$self->{'currgfile'} = $self->foreign_call($self->{'apachemod'}, "find_directive",
			   $self->{'agf'}, $self->{'conf'}, 1);
$self->lock_file($self->{'currfile'}) if ($self->{'currfile'});

# Make sure it is allowed
$self->switch_user();
$self->can_access_dir($self->{'htaccess'}) || $self->error($self->{'text_hash'}->{'dir_ecannot'});
$self->open_tempfile(TEST, ">>$self->{'htaccess'}", 1) || $self->error($self->text('dir_ehtaccess', $self->{'htaccess'}, $!));
$self->close_tempfile(TEST);

if ($self->{'in_hash'}->{'delete'}) {
	if ($self->{'in_hash'}->{'remove'}) {
		# Blow away .htaccess, htpasswd and htgroups
		$self->unlink_logged($self->{'htaccess'});
		$self->unlink_logged($self->{'currfile'}) if ($self->{'currfile'});
		$self->unlink_logged($self->{'currgfile'}) if ($self->{'currgfile'});
		@{$self->{'dirs_array'}} = grep { $_ ne $self->{'dir'} } @{$self->{'dirs_array'}};
		}
	else {
		# Take the authentication directives out of the .htaccess
		@{$self->{'dirs_array'}} = grep { $_ ne $self->{'dir'} } @{$self->{'dirs_array'}};
		$self->foreign_call($self->{'apachemod'}, "save_directive",
			      "require", [ ], $self->{'conf'}, $self->{'conf'});
		}
	}
else {
	# Validate inputs
	if ($self->{'in_hash'}->{'new'}) {
		$self->{'in_hash'}->{'dir'} =~ /^([a-z]:)?\// && -d $self->{'in_hash'}->{'dir'} ||
			$self->error($self->{'text_hash'}->{'dir_edir'});
		}

	# Parse users file option
	if (!$self->{'can_htpasswd'}) {
		# Users file is always automatic
		$self->{'file'} = $self->{'in_hash'}->{'new'} ? "$self->{'in_hash'}->{'dir'}/$self->{'config_hash'}->{'htpasswd'}"
				   : $self->{'dir'}->[1];
		}
	elsif ($self->{'in_hash'}->{'auto'}) {
		# User choose for it to be automatic
		$self->{'file'} = "$self->{'in_hash'}->{'dir'}/$self->{'config_hash'}->{'htpasswd'}";
		}
	else {
		# Entered by user
		$self->{'in_hash'}->{'file'} || $self->error($self->{'text_hash'}->{'dir_efile'});
		if ($self->{'in_hash'}->{'file'} !~ /^([a-z]:)?\//) {
			$self->{'file'} = "$self->{'in_hash'}->{'dir'}/$self->{'in_hash'}->{'file'}";
			}
		else {
			$self->{'file'} = $self->{'in_hash'}->{'file'};
			}
		}

	# Parse groups file option
	if (!$self->{'can_htgroups'}) {
		# Groups file is always fixed, or none
		$self->{'gfile'} = $self->{'in_hash'}->{'new'} ? undef : $self->{'dir'}->[3];
		}
	elsif ($self->{'in_hash'}->{'gauto'} == 2) {
		# No groups file
		$self->{'gfile'} = undef;
		}
	elsif ($self->{'in_hash'}->{'gauto'} == 1) {
		# User choose for groups file to be automatic
		$self->{'gfile'} = "$self->{'in_hash'}->{'dir'}/$self->{'config_hash'}->{'htgroups'}";
		}
	else {
		# Groups file was entered by user
		$self->{'in_hash'}->{'file'} || $self->error($self->{'text_hash'}->{'dir_egfile'});
		if ($self->{'in_hash'}->{'gfile'} !~ /^([a-z]:)?\//) {
			$self->{'gfile'} = "$self->{'in_hash'}->{'dir'}/$self->{'in_hash'}->{'gfile'}";
			}
		else {
			$self->{'gfile'} = $self->{'in_hash'}->{'gfile'};
			}
		}

	# Parse require option
	@{$self->{'require_array'}} = ( $self->{'in_hash'}->{'require_mode'} );
	if ($self->{'in_hash'}->{'require_mode'} eq "user") {
		@{$self->{'users_array'}} = split(/\s+/, $self->{'in_hash'}->{'require_user'});
		@{$self->{'users_array'}} || $self->error($self->{'text_hash'}->{'dir_erequire_user'});
		push(@{$self->{'require_array'}}, @{$self->{'users_array'}});
		}
	elsif ($self->{'in_hash'}->{'require_mode'} eq "group") {
		@{$self->{'groups_array'}} = split(/\s+/, $self->{'in_hash'}->{'require_group'});
		@{$self->{'groups_array'}} || $self->error($self->{'text_hash'}->{'dir_erequire_group'});
		push(@{$self->{'require_array'}}, @{$self->{'groups_array'}});
		}

	# Parse Webmin sync
	$self->{'sync'} = join(",", grep { $self->{'in_hash'}->{'sync_'.$_} }
			       ('create', 'update', 'delete'));
	$self->{'sync'} ||= "-";

	if ($self->{'in_hash'}->{'new'}) {
		# Either update an existing .htaccess to ensure that all
		# needed directives exist, or create from scratch

		# Use the existing users path if there is one, otherwise add
		$self->{'currfile'} = $self->foreign_call($self->{'apachemod'}, "find_directive",
					  $self->{'auf'}, $self->{'conf'}, 1);
		if ($self->{'currfile'}) {
			$self->{'file'} = $self->{'currfile'};
			}
		else {
			$self->foreign_call($self->{'apachemod'}, "save_directive",
				      $self->{'auf'}, [ "\"$self->{'file'}\"" ], $self->{'conf'}, $self->{'conf'});
			}

		# Use the existing groups path if there is one, otherwise add
		$self->{'currgfile'} = $self->foreign_call($self->{'apachemod'}, "find_directive",
					   $self->{'agf'}, $self->{'conf'}, 1);
		if ($self->{'currgfile'}) {
			$self->{'gfile'} = $self->{'currgfile'};
			}
		elsif ($self->{'gfile'}) {
			$self->foreign_call($self->{'apachemod'}, "save_directive",
				      $self->{'agf'}, [ "\"$self->{'gfile'}\"" ], $self->{'conf'},$self->{'conf'});
			}

		# Add an auth type if needed
		$self->{'currtype'} = $self->foreign_call($self->{'apachemod'}, "find_directive",
					  "AuthType", $self->{'conf'}, 1);
		if (!$self->{'currtype'}) {
			$self->foreign_call($self->{'apachemod'}, "save_directive",
				     "AuthType",
				     [ $self->{'in_hash'}->{'crypt'} == 3 ? "Digest" : "Basic" ],
				     $self->{'conf'}, $self->{'conf'});
			}

		# Add a realm if needed
		$self->{'currrealm'} = $self->foreign_call($self->{'apachemod'}, "find_directive",
					   "AuthName", $self->{'conf'}, 1);
		if (!$self->{'currrealm'}) {
			$self->{'in_hash'}->{'realm'} || $self->error($self->{'text_hash'}->{'dir_erealm'});
			$self->foreign_call($self->{'apachemod'}, "save_directive", "AuthName",
				      [ "\"$self->{'in_hash'}->{'realm'}\"" ], $self->{'conf'}, $self->{'conf'});
			}

		# Add a require if needed
		$self->{'currrequire'} = $self->foreign_call($self->{'apachemod'}, "find_directive",
					     "require", $self->{'conf'}, 1);
		if (!$self->{'currrequire'}) {
			$self->foreign_call($self->{'apachemod'}, "save_directive",
				      "require", [ join(" ", @{$self->{'require_array'}}) ],
						$self->{'conf'}, $self->{'conf'});
			}

		# Add AuthDigestProvider if needed
		if ($self->{'authz'} && $self->{'in_hash'}->{'crypt'} == 3) {
			$self->foreign_call($self->{'apachemod'}, "save_directive",
				      "AuthDigestProvider",
				      [ "file" ], $self->{'conf'}, $self->{'conf'});
			}

                # Add 'extra directives' if needed
                local $edline;
                foreach $edline (split(/\t+/, $self->{'config_hash'}->{'extra_directives'})) {
                        local ($ed, $edval);
                        $edline =~ m/(.*?)\s+(.*)/;
                        ($ed, $edval) = ($1, $2);
                        $self->{'curred'} = $self->foreign_call($self->{'apachemod'}, "find_directive",
                                                         $ed, $self->{'conf'});
                        if (!$self->{'curred'}) {
                                $self->foreign_call($self->{'apachemod'}, "save_directive",
					  $ed, [$edval], $self->{'conf'}, $self->{'conf'});
                                }
                        }

		# Add to the known directories list
		$self->{'sync'} = "-" if (!$self->{'can_sync'});
		$self->{'dir'} = [ $self->{'in_hash'}->{'dir'}, $self->{'file'}, $self->{'in_hash'}->{'crypt'}, $self->{'sync'}, $self->{'gfile'} ];
		push(@{$self->{'dirs_array'}}, $self->{'dir'});
		}
	else {
		# Just update the users and groups file paths, realm and
		# require directive
		$self->foreign_call($self->{'apachemod'}, "save_directive",
			      $self->{'auf'}, [ $self->{'file'} ],
					$self->{'conf'}, $self->{'conf'});
		$self->foreign_call($self->{'apachemod'}, "save_directive",
			      $self->{'agf'}, $self->{'gfile'} ? [ $self->{'gfile'} ] : [ ],
					$self->{'conf'}, $self->{'conf'});
		$self->foreign_call($self->{'apachemod'}, "save_directive",
			      "AuthName", [ "\"$self->{'in_hash'}->{'realm'}\"" ],
					$self->{'conf'}, $self->{'conf'});
		$self->foreign_call($self->{'apachemod'}, "save_directive",
			      "require", [ join(" ", @{$self->{'require_array'}}) ],
					$self->{'conf'}, $self->{'conf'});

		# Update the known directories list
		$self->{'dir'}->[1] = $self->{'file'};
		$self->{'dir'}->[2] = $self->{'in_hash'}->{'crypt'};
		$self->{'dir'}->[3] = $self->{'sync'} if ($self->{'can_sync'});
		}

	# Create an empty users file if needed
	if (!-r $self->{'file'}) {
		$self->lock_file($self->{'file'});
		$self->open_tempfile(FILE, ">$self->{'file'}", 1, 1) ||
			$self->error($self->text('dir_ehtpasswd', $self->{'file'}, $!));
		$self->close_tempfile(FILE) ||
			$self->error($self->text('dir_ehtpasswd', $self->{'file'}, $!));
		$self->unlock_file($self->{'file'});
		}

	# Create an empty groups file if needed
	if ($self->{'gfile'} && !-r $self->{'gfile'}) {
		$self->lock_file($self->{'gfile'});
		$self->open_tempfile(FILE, ">$self->{'gfile'}", 1, 1) ||
			$self->error($self->text('dir_ehtgroup', $self->{'gfile'}, $!));
		$self->close_tempfile(FILE) ||
			$self->error($self->text('dir_ehtgroup', $self->{'gfile'}, $!));
		$self->unlock_file($self->{'gfile'});
		}
	}

$self->flush_file_lines();
$self->switch_back();

$self->save_directories(\@{$self->{'dirs_array'}});
$self->unlock_all_files();
$self->webmin_log($self->{'in_hash'}->{'delete'} ? "delete" : $self->{'in_hash'}->{'new'} ? "create" : "modify",
	    "dir", $self->{'dir'}->[0]);
$self->redirect("");

1;
		
}


1;

