#!/usr/bin/perl
package inetd::InetdLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'inetd::InetdLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# inetd-lib.pl
# Common functions for managing inetd.conf and services files

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

$self->{'lib'} = $self->get_mod_lib();
if ($self->{'lib'}) {
	{
    my $pkg = $self->lib_to_pkg($self->{lib}, "inetd");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
	}

1;
1;
		return $self;
}

# list_inets_files()
# Returns a list of inetd configuration files
sub list_inets_files
{
	my $self = shift;

local @files = ( $self->{'config_hash'}->{'inetd_conf_file'} );
if ($self->{'config_hash'}->{'inetd_dir'}) {
	opendir(DIR, $self->{'config_hash'}->{'inetd_dir'});
	local $f;
	foreach $f (readdir(DIR)) {
		next if ($f =~ /^\./);
		push(@files, "$self->{'config_hash'}->{'inetd_dir'}/$f");
		}
	closedir(DIR);
	}
return @files;
}

# list_rpcs()
# Returns a list of rpc services, in the format
#  line name number aliases index
sub list_rpcs
{
	my $self = shift;

local(@rv, $l);
$l = 0;
open(RPC, $self->{'config_hash'}->{rpc_file});
while(<RPC>) {
	chop; s/#.*$//g;
	if (/^(\S+)\s+(\d+)\s*(.*)$/) {
		push(@rv, [ $l, $1, $2, $3, scalar(@rv) ]);
		}
	$l++;
	}
close(RPC);
return @rv;
}

# create_rpc(name, number, aliases)
# Create a new rpc file entry
sub create_rpc
{
	my $self = shift;

$self->open_tempfile(RPC, ">> $self->{'config_hash'}->{rpc_file}");
$self->print_tempfile(RPC, "$_[0]\t$_[1]",($_[2] ? "\t$_[2]\n" : "\n"));
$self->close_tempfile(RPC);
}

# modify_rpc(line, name, number, aliases)
# Change an existing rpc program
sub modify_rpc
{
	my $self = shift;

local(@rpcs);
open(RPC, $self->{'config_hash'}->{rpc_file});
@rpcs = <RPC>;
close(RPC);
$rpcs[$_[0]] = "$_[1]\t$_[2]".($_[3] ? "\t$_[3]\n" : "\n");
$self->open_tempfile(RPC, "> $self->{'config_hash'}->{rpc_file}");
$self->print_tempfile(RPC, @rpcs);
$self->close_tempfile(RPC);
}

# delete_rpc(line)
# Delete an entry from the rpc file
sub delete_rpc
{
	my $self = shift;

local(@rpcs);
open(RPC, $self->{'config_hash'}->{rpc_file});
@rpcs = <RPC>;
close(RPC);
splice(@rpcs, $_[0], 1);
$self->open_tempfile(RPC, "> $self->{'config_hash'}->{rpc_file}");
$self->print_tempfile(RPC, @rpcs);
$self->close_tempfile(RPC);
}

sub lock_inetd_files
{
	my $self = shift;

$self->lock_file($self->{'config_hash'}->{'inetd_conf_file'}, 0, 1);
$self->lock_file($self->{'config_hash'}->{'services_file'}, 0, 1);
$self->lock_file($self->{'config_hash'}->{'protocols_file'}, 0, 1);
$self->lock_file($self->{'config_hash'}->{'rpc_file'}, 0, 1);
}

sub unlock_inetd_files
{
	my $self = shift;

$self->unlock_file($self->{'config_hash'}->{'inetd_conf_file'});
$self->unlock_file($self->{'config_hash'}->{'services_file'});
$self->unlock_file($self->{'config_hash'}->{'protocols_file'});
$self->unlock_file($self->{'config_hash'}->{'rpc_file'});
}


1;

