#!/usr/bin/perl
package ipfilter::IpfilterLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'ipfilter::IpfilterLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for parsing ipf.conf

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
$self->foreign_require("net", "net-lib.pl");

# Get the detected ipf version
if (open(VERSION, "$self->{'module_config_directory'}/version")) {
	chop($self->{'ipf_version'} = <VERSION>);
	close(VERSION);
	}

@{$self->{'actions_array'}} = ( "block", "pass", "log", "count", "skip", "auth", "preauth", "call" );
@{$self->{'compare_ops_array'}} = ( "=", "!=", "<", ">", "<=", ">=", "eq", "ne", "lt", "gt", "le", "ge" );
@{$self->{'icmp_codes_array'}} = ( "net-unr" , "host-unr" , "proto-unr" , "port-unr" ,
            "needfrag" , "srcfail" , "net-unk" , "host-unk" , "isolate" ,
            "net-prohib" , "host-prohib" , "net-tos" , "host-tos" ,
            "filter-prohib" , "host-preced" , "cutoff-preced" );
@{$self->{'log_priorities_array'}} = ( "emerg" , "alert" , "crit" , "err" , "warn" , "notice" , "info" , "debug" );
@{$self->{'log_facilities_array'}} = ( "kern" , "user" , "mail" , "daemon" , "auth" , "syslog" ,
            "lpr" , "news" , "uucp" , "cron" , "ftp" , "authpriv" ,
            "audit" , "logalert" , "local0" , "local1" , "local2" ,
            "local3" , "local4" , "local5" , "local6" , "local7" );
@{$self->{'icmp_types_array'}} = ( "unreach" , "echo" , "echorep" , "squench" , "redir" ,
            "timex" , "paramprob" , "timest" , "timestrep" , "inforeq" ,
            "inforep" , "maskreq" , "maskrep" );

$self->{'init_script'} = $self->{'config_hash'}->{'init'} || "webmin-$self->{'module_name'}";

1;

1;
		return $self;
}

sub missing_firewall_commands
{
	my $self = shift;

local $cmd;
foreach $cmd ("ipf", "ipfstat", "ipnat") {
	if (!$self->has_command($self->{'config_hash'}->{$cmd})) {
		return $self->{'config_hash'}->{$cmd};
		}
	}
return undef;
}

# get_config([file])
# Parse the ipfilter config file
sub get_config
{
	my $self = shift;

local $file = $_[0] || $self->{'config_hash'}->{'ipf_conf'};
return $self->{'get_config_cache_hash'}->{$file} if ($self->{'get_config_cache_hash'}->{$file});
local @rv;
local $lnum = 0;
open(FILE, $file);
while(<FILE>) {
	# Read each line, splitting into words
	s/\r|\n//g;
	if (/^\s*(#*)\s*((block|pass|log|count|skip|auth|preauth|call|\@).*)$/) {
		# A rule, perhaps commented
		local $nocmt = $2;
		local @w = split(/\s+/, $nocmt);
		local @cmts = split(/\n/, $self->{'cmt'});
		local $rule = { 'index' => scalar(@rv),
				'type' => 'ipf',
				'line' => $lnum-scalar(@cmts),
				'eline' => $lnum,
				'file' => $file,
				'text' => $_,
				'cmt' => $self->{'cmt'},
				'active' => !$1 };
		$self->{'cmt'} = undef;

		# There can be a special insert prefix
		if ($w[0] eq "\@") {
			shift(@w);
			$rule->{'insert'} = shift(@w);
			}

		# First word is the action, possibly with an arg
		$rule->{'action'} = shift(@w);
		if ($rule->{'action'} eq "block") {
			# Block can have ICMP return type parameter
			if ($w[0] eq "return-rst") {
				shift(@w);
				$rule->{'block-return'} = "rst";
				}
			elsif ($w[0] eq "return-icmp" ||
			       $w[0] eq "return-icmp-as-dest") {
				# XXX is this correct? ie.
				# return-icmp ( srcfail )
				$rule->{'block-return-dest'} = 1
					if ($w[0] eq "return-icmp-as-dest");
				shift(@w);
				shift(@w);	# skip (
				$rule->{'block-return'} = shift(@w);
				shift(@w);	# skip )
				}
			}
		elsif ($rule->{'action'} eq "log") {
			# Log action can have several options
			$self->parse_log("log");
			}
		elsif ($rule->{'action'} eq "skip") {
			# Skip action specifies rule number
			$rule->{'skip'} = shift(@w);
			}
		elsif ($rule->{'action'} eq "call") {
			# Call action has a function name
			if ($w[0] eq "now") {
				$rule->{'call-now'} = shift(@w);
				}
			$rule->{'call'} = shift(@w);
			}

		# Second is 'in' or 'out'
		$rule->{'dir'} = shift(@w);

		# Parse options
		while(1) {
			if ($w[0] eq "log") {
				# Log option has several sub-options!
				shift(@w);
				$rule->{'olog'} = 1;
				$self->parse_log("olog");
				}
			elsif ($w[0] eq "tag") {
				# Tag has ID option
				shift(@w);
				$rule->{'tag'} = shift(@w);
				}
			elsif ($w[0] eq "quick") {
				shift(@w);
				$rule->{'quick'} = 1;
				}
			elsif ($w[0] eq "on") {
				# On some interface name
				shift(@w);
				$rule->{'on'} = shift(@w);
				}
			elsif ($w[0] eq "dup-to") {
				# Has interface:ip option
				shift(@w);
				$rule->{'dup-to'} = shift(@w);
				}
			elsif ($w[0] eq "fastroute" || $w[0] eq "to") {
				# Fastroute option has interface name (and IP)
				shift(@w);
				$rule->{'fastroute'} = shift(@w);
				if ($w[0] eq ":") {
					shift(@w);
					$rule->{'fastroute-ip'} = shift(@w);
					}
				}
			elsif ($w[0] eq "reply-to") {
				# Replyto option has interface name (and IP)
				shift(@w);
				$rule->{'reply-to'} = shift(@w);
				if ($w[0] eq ":") {
					shift(@w);
					$rule->{'reply-to-ip'} = shift(@w);
					}
				}
			else {
				last;
				}
			}

		# Parse TOS
		if ($w[0] eq "tos") {
			shift(@w);
			$rule->{'tos'} = shift(@w);
			}

		# Parse TTL
		if ($w[0] eq "ttl") {
			shift(@w);
			$rule->{'ttl'} = shift(@w);
			}

		# Parse protocol
		if ($w[0] eq "proto") {
			shift(@w);
			$rule->{'proto'} = shift(@w);
			}

		# Parse from/to section
		if ($w[0] eq "all") {
			shift(@w);
			$rule->{'all'} = 1;
			}
		elsif ($w[0] eq "from") {
			shift(@w);	# skip 'from'
			$self->parse_object("from");
			shift(@w);	# skip 'to'
			$self->parse_object("to");
			}
		else {
			if (!$rule->{'active'}) {
				# Must actually be a comment!
				$self->{'cmt'} .= "\n" if ($self->{'cmt'});
				$self->{'cmt'} .= $nocmt;
				goto nextline;
				}
			$self->error("error parsing IPF line $_ at $w[0] line $lnum");
			}

		# Parse ip options
		if ($w[0] eq "flags") {
			shift(@w);
			$rule->{'flags1'} = shift(@w);
			if ($w[0] eq "/") {
				shift(@w);
				$rule->{'flags2'} = shift(@w);
				}
			}
		if ($w[0] eq "with" || $w[0] eq "and") {
			# Just store keywords till end of section
			shift(@w);
			local @with;
			while(@w && $w[0] ne "keep" && $w[0] ne "icmp-type" &&
			      $w[0] ne "head" && $w[0] ne "group") {
				push(@with, shift(@w));
				}
			$rule->{'with'} = \@with;
			}
		if ($w[0] eq "icmp-type") {
			shift(@w);
			$rule->{'icmp-type'} = shift(@w);
			if ($w[0] eq "code") {
				shift(@w);
				$rule->{'icmp-type-code'} = shift(@w);
				}
			}
		if ($w[0] eq "keep") {
			shift(@w);
			$rule->{'keep'} = shift(@w);
			}

		# Parse group section
		if ($w[0] eq "head") {
			shift(@w);
			$rule->{'head'} = shift(@w);
			}
		elsif ($w[0] eq "group") {
			shift(@w);
			$rule->{'group'} = shift(@w);
			}

		push(@rv, $rule);
		}
	elsif (/^\s*#\s*(.*)$/) {
		# A comment line
		$self->{'cmt'} .= "\n" if ($self->{'cmt'});
		$self->{'cmt'} .= $1;
		}
	nextline:
	$lnum++;
	}
close(FILE);
$self->{'get_config_cache_hash'}->{$file} = \@rv;
return $self->{'get_config_cache_hash'}->{$file};
}

# get_live_config()
# Returns all active firewall rules
sub get_live_config
{
	my $self = shift;

local $livein = $self->get_config("$self->{'config_hash'}->{'ipfstat'} -i |");
local $liveout = $self->get_config("$self->{'config_hash'}->{'ipfstat'} -o |");
return [ @$livein, @$liveout ];
}

# parse_log(suffix)
sub parse_log
{
	my $self = shift;

local $pfx = $_[0];
while(1) {
	if ($self->{'w_array'}->[0] eq "body") {
		shift(@{$self->{'w_array'}});
		$self->{'rule'}->{$pfx.'-body'} = 1;
		}
	elsif ($self->{'w_array'}->[0] eq "first") {
		shift(@{$self->{'w_array'}});
		$self->{'rule'}->{$pfx.'-first'} = 1;
		}
	elsif ($self->{'w_array'}->[0] eq "or-block") {
		shift(@{$self->{'w_array'}});
		$self->{'rule'}->{$pfx.'-or-block'} = 1;
		}
	elsif ($self->{'w_array'}->[0] eq "level") {
		shift(@{$self->{'w_array'}});
		$self->{'rule'}->{$pfx.'-level'} = shift(@{$self->{'w_array'}});
		}
	else {
		last;
		}
	}
}

# parse_object(direction)
sub parse_object
{
	my $self = shift;

local $dir = $_[0];

# Parse ! prefix
if ($self->{'w_array'}->[0] eq "!") {
	shift(@{$self->{'w_array'}});
	$self->{'rule'}->{$dir."-not"} = 1;
	}

# Parse addr section
local $addr = shift(@{$self->{'w_array'}});
if ($addr eq "any") {
	$self->{'rule'}->{$dir."-any"} = 1;
	}
elsif ($addr eq "<thishost>") {
	$self->{'rule'}->{$dir."-thishost"} = 1;
	}
elsif ($addr =~ /^(\S+)\/(\S+)$/) {
	# host-name/number
	$self->{'rule'}->{$dir."-numhost"} = $1;
	$self->{'rule'}->{$dir."-nummask"} = $2;
	}
elsif (@{$self->{'w_array'}} > 0 && $self->{'w_array'}->[0] eq "/") {
	# host-name [ "/" decnumber ]
	$self->{'rule'}->{$dir."-numhost"} = $addr;
	shift(@{$self->{'w_array'}});
	$self->{'rule'}->{$dir."-nummask"} = shift(@{$self->{'w_array'}});
	}
else {
	# host-name [ "mask" ipaddr | "mask" hexnumber ]
	$self->{'rule'}->{$dir."-host"} = $addr;
	if (@{$self->{'w_array'}} && $self->{'w_array'}->[0] eq "mask") {
		shift(@{$self->{'w_array'}});
		$self->{'rule'}->{$dir."-mask"} = shift(@{$self->{'w_array'}});
		}
	}

# Parse port-comp or port-range
if ($self->{'w_array'}->[0] eq "port") {
	shift(@{$self->{'w_array'}});
	if ($self->indexof($self->{'w_array'}->[0], @{$self->{'compare_ops_array'}}) >= 0) {
		# Must be port-comp
		$self->{'rule'}->{$dir.'-port-comp'} = shift(@{$self->{'w_array'}});
		$self->{'rule'}->{$dir.'-port-num'} = shift(@{$self->{'w_array'}});
		}
	else {
		# Must be port-range
		$self->{'rule'}->{$dir.'-port-start'} = shift(@{$self->{'w_array'}});
		$self->{'rule'}->{$dir.'-port-range'} = shift(@{$self->{'w_array'}});
		$self->{'rule'}->{$dir.'-port-end'} = shift(@{$self->{'w_array'}});
		}
	}
}

# interface_choice(name, value, noignored, disabled?)
sub interface_choice
{
	my $self = shift;

local @ifaces;
if ($self->foreign_check("net")) {
	$self->foreign_require("net", "net-lib.pl");
	return $main::net_object->interface_choice($_[0], $_[1],
			      $_[2] ? undef : $self->{'text_hash'}->{'edit_anyiface'}, $_[3]);
	}
else {
	return $self->ui_textbox($_[0], $_[1], 6, $_[3]);
	}
}

# parse_interface_choice(name, error)
sub parse_interface_choice
{
	my $self = shift;

local $rv = $self->{'in_hash'}->{$_[0]} eq "other" ? $self->{'in_hash'}->{$_[0]."_other"} : $self->{'in_hash'}->{$_[0]};
$rv =~ /^[a-z]+\d*/ || $self->error($_[1]);
return $rv;
}

# describe_rule(&rule, where-mode)
# Returns a human-readable description for the conditions for some rule
sub describe_rule
{
	my $self = shift;

local $r = $_[0];
local @rv;
if ($r->{'proto'}) {
	push(@rv, $self->text('desc_proto', "<b>".uc($r->{'proto'})."</b>"));
	}
local $f;
push(@rv, $self->describe_object($r, "from"));
push(@rv, $self->describe_object($r, "to"));
if ($r->{'on'}) {
	push(@rv, $self->text('desc_on', "<b>".$r->{'on'}."</b>"));
	}
return $self->describe_join(\@rv, $_[1]);
}

# describe_object(&rule, prefix)
sub describe_object
{
	my $self = shift;

local ($r, $f) = @_;
local @rv;
if ($r->{$f.'-thishost'}) {
	push(@rv, $self->text('desc_'.$f.'_thishost'));
	}
elsif ($r->{$f.'-numhost'} && $r->{$f.'-nummask'} == 32) {
	push(@rv, $self->text('desc_'.$f, "<b>".$r->{$f.'-numhost'}."</b>"));
	}
elsif ($r->{$f.'-numhost'} && $r->{$f.'-nummask'}) {
	push(@rv, $self->text('desc_'.$f, "<b>".$r->{$f.'-numhost'}."/".
				    $r->{$f.'-nummask'}."</b>"));
	}
elsif ($r->{$f.'-numhost'}) {
	push(@rv, $self->text('desc_'.$f, "<b>".$r->{$f.'-numhost'}."</b>"));
	}
elsif ($r->{$f.'-host'} && $r->{$f.'-mask'} eq "255.255.255.255") {
	push(@rv, $self->text('desc_'.$f, "<b>".$r->{$f.'-host'}."</b>"));
	}
elsif ($r->{$f.'-host'} && $r->{$f.'-mask'}) {
	push(@rv, $self->text('desc_'.$f, "<b>".$r->{$f.'-host'}."/".
				    $r->{$f.'-mask'}."</b>"));
	}
elsif ($r->{$f.'-host'}) {
	push(@rv, $self->text('desc_'.$f, "<b>".$r->{$f.'-host'}."</b>"));
	}
if ($r->{$f.'-port-comp'}) {
	push(@rv, $self->text('desc_portcomp_'.$f,
		$r->{$f.'-port-comp'},
		"<b>".$r->{$f.'-port-num'}."</b>"));
	}
elsif ($r->{$f.'-port-range'}) {
	push(@rv, $self->text('desc_portrange_'.$f,
		"<b>".$r->{$f.'-port-start'}."</b>",
		"<b>".$r->{$f.'-port-end'}."</b>"));
	}
return @rv;
}

# describe_from(&rule, where-mode)
# Returns a human-readable description of the match part of a NAT rule
sub describe_from
{
	my $self = shift;

local ($r) = @_;
local @rv;
push(@rv, $self->text('desc_on', "<b>".$r->{'iface'}."</b>"));
if ($r->{'from'}) {
	push(@rv, $self->describe_object($r, "from"), $self->describe_object($r, "fromto"));
	}
elsif ($r->{'frommask'} eq '255.255.255.255' || $r->{'frommask'} eq "32") {
	push(@rv, $self->text('desc_natfromh', "<b>$r->{'fromip'}</b>"));
	}
elsif ($r->{'frommask'} ne '0.0.0.0' && $r->{'frommask'} ne "0") {
	push(@rv, $self->text('desc_natfrom', "<b>$r->{'fromip'}</b>",
				        "<b>$r->{'frommask'}</b>"));
	}
if ($r->{'dport1'} && $r->{'dport2'}) {
	push(@rv, $self->text('desc_dport2', "<b>$r->{'dport1'}</b>",
				    "<b>$r->{'dport2'}</b>"));
	}
elsif ($r->{'dport1'}) {
	push(@rv, $self->text('desc_dport1', "<b>$r->{'dport1'}</b>"));
	}
return $self->describe_join(\@rv, $_[1]);
}

# describe_to(&rule, where-mode)
# Returns a human-readable description of the translation part of a NAT rule
sub describe_to
{
	my $self = shift;

local ($r, $where) = @_;
local @rv;
if ($r->{'rdrip'}) {
	push(@rv, $self->text('desc_rdr', "<b>".join(" ", @{$r->{'rdrip'}})."</b>",
				    "<b>".$r->{'rdrport'}."</b>"));
	}
elsif ($r->{'tostart'}) {
	push(@rv, $self->text('desc_natrange', "<b>$r->{'tostart'}</b>",
				         "<b>$r->{'toend'}</b>"));
	}
elsif ($r->{'toip'} eq '0.0.0.0' && $r->{'tomask'} eq "32") {
	push(@rv, $self->{'text_hash'}->{'desc_nattoiface'});
	}
elsif ($r->{'tomask'} eq '255.255.255.255' || $r->{'tomask'} eq "32") {
	push(@rv, $self->text('desc_nattoh', "<b>$r->{'toip'}</b>"));
	}
else {
	push(@rv, $self->text('desc_natto', "<b>$r->{'toip'}</b>",
				      "<b>$r->{'tomask'}</b>"));
	}
return $self->text($where ? 'desc_tolc' : 'desc_touc',
	     join(" $self->{'text_hash'}->{'desc_and'} ", @rv));
}

# describe_join(&array, where-mode)
sub describe_join
{
	my $self = shift;

local ($rv, $where) = @_;
return @$rv ? $self->text($where ? 'desc_where' : 'desc_if',
		   join(" $self->{'text_hash'}->{'desc_and'} ", @$rv))
	    : $self->{'text_hash'}->{$where ? 'desc_all' : 'desc_always'};
}

# create_rule(&rule)
# Add one rule to the config file, at the end
sub create_rule
{
	my $self = shift;

# Add to file
if ($_[0]->{'type'} eq 'ipnat') {
	$_[0]->{'file'} ||= $config{'ipnat_conf'};
	}
else {
	$_[0]->{'file'} ||= $config{'ipf_conf'};
	}
local $lref = $self->read_file_lines($_[0]->{'file'});
$_[0]->{'line'} = scalar(@$lref);
local @lines = $self->rule_lines($_[0]);
$_[0]->{'eline'} = $_[0]->{'line'} + scalar(@lines) - 1;
push(@$lref, @lines);
$self->flush_file_lines();

# Add to config cache
local $conf = $self->config_for_rule($_[0]);
$_[0]->{'index'} = scalar(@$conf);
push(@$conf, $_[0]);
}

# insert_rule(&rule, &before)
# Adds one rule to the config file, before some other
sub insert_rule
{
	my $self = shift;

# Add to file
$_[0]->{'file'} ||= $_[1]->{'file'};
$_[0]->{'type'} ||= $_[1]->{'type'};
local $lref = $self->read_file_lines($_[0]->{'file'});
$_[0]->{'line'} = $_[1]->{'line'};
local @lines = $self->rule_lines($_[0]);
$_[0]->{'eline'} = $_[0]->{'line'} + scalar(@lines) - 1;
splice(@$lref, $_[0]->{'line'}, 0, @lines);

# Update config cache
local $conf = $self->config_for_rule($_[0]);
$_[0]->{'index'} = $_[1]->{'index'};
foreach $c (@$conf) {
	$c->{'index'}++ if ($c->{'index'} >= $_[0]->{'index'});
	$c->{'line'} += scalar(@lines) if ($c->{'line'} >= $_[0]->{'line'});
	$c->{'eline'} += scalar(@lines) if ($c->{'eline'} >= $_[0]->{'eline'});
	}
splice(@$conf, $_[0]->{'index'}, 0, $_[0]);
}

# delete_rule(&rule)
# Remove one rule
sub delete_rule
{
	my $self = shift;

# Update file
local $lref = $self->read_file_lines($_[0]->{'file'});
local $len = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
splice(@$lref, $_[0]->{'line'}, $len);

# Update config cache
local $conf = $self->config_for_rule($_[0]);
splice(@$conf, $_[0]->{'index'}, 1);
local $c;
foreach $c (@$conf) {
	$c->{'index'}-- if ($c->{'index'} > $_[0]->{'index'});
	$c->{'line'} -= $len if ($c->{'line'} > $_[0]->{'line'});
	$c->{'eline'} -= $len if ($c->{'eline'} > $_[0]->{'eline'});
	}
}

# modify_rule(&rule)
# Update one rule
sub modify_rule
{
	my $self = shift;

# Update file
local $lref = $self->read_file_lines($_[0]->{'file'});
local @lines = $self->rule_lines($_[0]);
local $len = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
local $newlen = scalar(@lines);
splice(@$lref, $_[0]->{'line'}, $len, @lines);

# Update config cache
$_[0]->{'eline'} = $_[0]->{'line'} + $newlen - 1;
local $conf = $self->config_for_rule($_[0]);
local $c;
foreach $c (@$conf) {
	next if ($c eq $_[0]);
	$c->{'line'} += $newlen - $len if ($c->{'line'} > $_[0]->{'line'});
	$c->{'eline'} += $newlen - $len if ($c->{'eline'} > $_[0]->{'eline'});
	}
}

# swap_rules(&rule1, &rule2)
# Swap two rules in the config file, which MUST be next to each other
sub swap_rules
{
	my $self = shift;

local $lref = $self->read_file_lines($_[0]->{'file'});
local @lines0 = @$lref[$_[0]->{'line'} .. $_[0]->{'eline'}];
local @lines1 = @$lref[$_[1]->{'line'} .. $_[1]->{'eline'}];
if ($_[0]->{'line'} < $_[1]->{'line'}) {
	splice(@$lref, $_[0]->{'line'}, $_[1]->{'eline'}-$_[0]->{'line'}+1,
	       (@lines1, @lines0));
	$_[1]->{'line'} = $_[0]->{'line'};
	$_[1]->{'eline'} = $_[1]->{'line'} + scalar(@lines1) - 1;
	$_[0]->{'line'} = $_[1]->{'eline'}+1;
	$_[0]->{'eline'} = $_[0]->{'line'} + scalar(@lines0) - 1;
	}
else {
	splice(@$lref, $_[1]->{'line'}, $_[0]->{'eline'}-$_[1]->{'line'}+1,
	       (@lines0, @lines1));
	$_[0]->{'line'} = $_[1]->{'line'};
	$_[0]->{'eline'} = $_[0]->{'line'} + scalar(@lines0) - 1;
	$_[1]->{'line'} = $_[0]->{'eline'}+1;
	$_[1]->{'eline'} = $_[1]->{'line'} + scalar(@lines1) - 1;
	}

# Update config cache
($_[0]->{'index'}, $_[1]->{'index'}) = ($_[1]->{'index'}, $_[0]->{'index'});
local $conf = $self->config_for_rule($_[0]);
$conf->[$_[0]->{'index'}] = $_[0];
$conf->[$_[1]->{'index'}] = $_[1];
}

sub config_for_rule
{
	my $self = shift;

return $_[0]->{'type'} eq 'ipnat' ? $self->get_ipnat_config($_[0]->{'file'})
				  : $self->get_config($_[0]->{'file'});
}

# save_config(&rules, [file], type)
# Write out the entire config file
sub save_config
{
	my $self = shift;

local $file = $_[1] || $_[0]->[0]->{'file'} ||
	      ($_[2] eq "ipnat" ? $self->{'config_hash'}->{'ipnat_conf'} : $self->{'config_hash'}->{'ipf_conf'});
$self->open_tempfile(FILE, ">$file");
local $r;
local $idx = 0;
local $lnum = 0;
foreach $r (@{$_[0]}) {
	$r->{'file'} = $file;
	$r->{'index'} = $idx++;
	$r->{'line'} = $lnum;
	local @lines = $self->rule_lines($r);
	$lnum += scalar(@lines);
	$r->{'eline'} = $lnum - 1;
	$self->print_tempfile(FILE, map { "$_\n" } $self->rule_lines($r));
	}
$self->close_tempfile(FILE);
if ($_[2] eq "ipnat") {
	$self->{'get_ipnat_config_cache_hash'}->{$file} = $_[0];
	}
else {
	$self->{'get_config_cache_hash'}->{$file} = $_[0];
	}
}

# rule_lines(&rule)
# Returns the lines of text that make up a rule
sub rule_lines
{
	my $self = shift;

local ($rule) = @_;
local @rv = map { "# $_" } split(/\n/, $rule->{'cmt'});
local @w;
push(@w, "#") if (!$_[0]->{'active'});

if ($rule->{'type'} ne 'ipnat') {
	# Standard firewall rule
	# Add insert prefix
	push(@w, "\@", $rule->{'insert'}) if ($rule->{'insert'} ne "");

	# Add action and args
	push(@w, $rule->{'action'});
	if ($rule->{'action'} eq "block") {
		# Add block action options
		if ($rule->{'block-return'} eq "rst") {
			push(@w, "return-rst");
			}
		elsif ($rule->{'block-return'} ne "") {
			# XXX may be wrong
			push(@w, $rule->{'block-return-dest'} ? "return-icmp-as-dest"
							      : "return-icmp");
			push(@w, "(", $rule->{'block-return'}, ")");
			}
		}
	elsif ($rule->{'action'} eq "log") {
		# Add log action options
		$self->print_log("log");
		}
	elsif ($rule->{'action'} eq "skip") {
		push(@w, $rule->{'skip'});
		}
	elsif ($rule->{'action'} eq "call") {
		push(@w, "now") if ($rule->{'call-now'});
		push(@w, $rule->{'call'});
		}

	# Add in or out
	push(@w, $rule->{'dir'});

	# Add options
	if ($rule->{'olog'}) {
		push(@w, "log");
		$self->print_log("olog");
		}
	if ($rule->{'tag'} ne "") {
		push(@w, "tag", $rule->{'tag'});
		}
	if ($rule->{'quick'}) {
		push(@w, "quick");
		}
	if ($rule->{'on'} ne "") {
		push(@w, "on", $rule->{'on'});
		}
	if ($rule->{'dup-to'} ne "") {
		push(@w, "dup-to", $rule->{'dup-to'});
		}
	if ($rule->{'fastroute'} ne "") {
		push(@w, "fastroute", $rule->{'fastroute'});
		if ($rule->{'fastroute-ip'} ne "") {
			push(@w, ":", $rule->{'fastroute-ip'});
			}
		}
	if ($rule->{'reply-to'} ne "") {
		push(@w, "reply-to", $rule->{'reply-to'});
		if ($rule->{'reply-to-ip'} ne "") {
			push(@w, ":", $rule->{'reply-to-ip'});
			}
		}

	# Add TOS
	if ($rule->{'tos'} ne "") {
		push(@w, "tos", $rule->{'tos'});
		}

	# Add TTL
	if ($rule->{'ttl'} ne "") {
		push(@w, "ttl", $rule->{'ttl'});
		}

	# Add protocol
	if ($rule->{'proto'} ne "") {
		push(@w, "proto", $rule->{'proto'});
		}

	# Add to/from section
	if ($rule->{'all'}) {
		push(@w, "all");
		}
	else {
		push(@w, "from");
		$self->print_object("from");
		push(@w, "to");
		$self->print_object("to");
		}

	# Add IP options
	if ($rule->{'flags1'} ne "") {
		push(@w, "flags", $rule->{'flags1'});
		if ($rule->{'flags2'} ne "") {
			push(@w, "/", $rule->{'flags2'});
			}
		}
	if ($rule->{'with'}) {
		push(@w, "with", @{$rule->{'with'}});
		}
	if ($rule->{'icmp-type'} ne "") {
		push(@w, "icmp-type", $rule->{'icmp-type'});
		if ($rule->{'icmp-type-code'} ne "") {
			push(@w, "code", $rule->{'icmp-type-code'});
			}
		}
	if ($rule->{'keep'} ne "") {
		push(@w, "keep", $rule->{'keep'});
		}

	# Add group section
	if ($rule->{'head'} ne "") {
		push(@w, "head", $rule->{'head'});
		}
	if ($rule->{'group'} ne "") {
		push(@w, "group", $rule->{'group'});
		}
	}
else {
	# A NAT rule
	push(@w, $rule->{'action'});
	push(@w, $rule->{'iface'});
	if ($rule->{'action'} ne 'rdr') {
		# Mapping rule .. add the source
		if ($rule->{'from'}) {
			push(@w, "from");
			$self->print_object("from");
			push(@w, "to");
			$self->print_object("fromto");
			}
		else {
			$self->print_ipmask("from");
			}

		push(@w, "->");

		# Destination address
		if ($rule->{'tostart'}) {
			push(@w, "range", $rule->{'tostart'}, "-",
					  $rule->{'toend'});
			}
		else {
			$self->print_ipmask("to");
			}

		# Port mapping
		if ($rule->{'portmap'}) {
			push(@w, "portmap", $rule->{'portmap'});
			push(@w, $rule->{'portauto'} ? "auto" :
				   $rule->{'portmapfrom'}.":".
				   $rule->{'portmapto'});
			}

		# Proxy mapping
		if ($rule->{'proxyport'}) {
			$self->print_proxy("proxy");
			}

		# Add options
		if ($rule->{'proto'}) {
			push(@w, $rule->{'proto'});
			}
		if ($rule->{'frag'}) {
			push(@w, "frag");
			}
		if ($rule->{'age1'}) {
			push(@w, "age", $rule->{'age1'});
			if ($rule->{'age2'}) {
				push(@w, "/", $rule->{'age2'});
				}
			}
		if ($rule->{'mssclamp'}) {
			push(@w, "mssclamp", $rule->{'mssclamp'});
			}
		if ($rule->{'oproxyport'}) {
			$self->print_proxy("oproxy");
			}
		}
	else {
		# Redirect rule .. add source
		$self->print_ipmask("from");

		# Add destination ports
		push(@w, "port");
		push(@w, $rule->{'dport1'});
		if ($rule->{'dport2'}) {
			push(@w, "-", $rule->{'dport2'});
			}

		push(@w, "->");

		# Add destination IPs
		push(@w, join(" , ", @{$rule->{'rdrip'}}));

		# Add destination port and protocol
		push(@w, "port", $rule->{'rdrport'}, $rule->{'rdrproto'});

		# Add options
		if ($rule->{'round-robin'}) {
			push(@w, "round-robin");
			}
		if ($rule->{'frag'}) {
			push(@w, "frag");
			}
		if ($rule->{'age1'}) {
			push(@w, "age", $rule->{'age1'});
			if ($rule->{'age2'}) {
				push(@w, "/", $rule->{'age2'});
				}
			}
		if ($rule->{'mssclamp'}) {
			push(@w, "mssclamp", $rule->{'mssclamp'});
			}
		if ($rule->{'proxy'}) {
			push(@w, "proxy", $rule->{'proxy'});
			}
		}
	}

push(@rv, join(" ", @w));
return @rv;
}

# print_log(prefix)
sub print_log
{
	my $self = shift;

local $pfx = $_[0];
push(@{$self->{'w_array'}}, "body") if ($self->{'rule'}->{$pfx.'-body'});
push(@{$self->{'w_array'}}, "first") if ($self->{'rule'}->{$pfx.'-first'});
push(@{$self->{'w_array'}}, "or-block") if ($self->{'rule'}->{$pfx.'-or-block'});
push(@{$self->{'w_array'}}, "level", $self->{'rule'}->{$pfx.'-level'}) if($self->{'rule'}->{$pfx.'-level'} ne "");
}

# print_object(dir)
sub print_object
{
	my $self = shift;

local $dir = $_[0];
if ($self->{'rule'}->{$dir."-any"}) {
	push(@{$self->{'w_array'}}, "any");
	}
elsif ($self->{'rule'}->{$dir."-thishost"}) {
	push(@{$self->{'w_array'}}, "<thishost>");
	}
elsif ($self->{'rule'}->{$dir."-numhost"}) {
	push(@{$self->{'w_array'}}, $self->{'rule'}->{$dir."-numhost"}."/".$self->{'rule'}->{$dir."-nummask"});
	}
else {
	push(@{$self->{'w_array'}}, $self->{'rule'}->{$dir."-host"});
	if ($self->{'rule'}->{$dir."-mask"} ne "") {
		push(@{$self->{'w_array'}}, "mask", $self->{'rule'}->{$dir."-mask"});
		}
	}

if ($self->{'rule'}->{$dir."-port-comp"}) {
	push(@{$self->{'w_array'}}, "port", $self->{'rule'}->{$dir.'-port-comp'},
			 $self->{'rule'}->{$dir.'-port-num'});
	}
elsif ($self->{'rule'}->{$dir."-port-range"}) {
	push(@{$self->{'w_array'}}, "port", $self->{'rule'}->{$dir.'-port-start'},
			 $self->{'rule'}->{$dir.'-port-range'},
			 $self->{'rule'}->{$dir.'-port-end'});
	}
}

# get_ipnat_config([file])
# Returns an array reference of ipnat rules
sub get_ipnat_config
{
	my $self = shift;

local $file = $_[0] || $self->{'config_hash'}->{'ipnat_conf'};
return $self->{'get_ipnat_config_cache_hash'}->{$file} if ($self->{'get_ipnat_config_cache_hash'}->{$file});
open(FILE, $file);
while(<FILE>) {
	# Read each line, splitting into words
	s/\r|\n//g;
	if (/^\s*(#*)\s*((map-block|map|bimap|mapit|rdr).*)$/) {
		# A NAT rule, perhaps commented
		local $nocmt = $2;
		local @w = split(/\s+/, $nocmt);
		local @cmts = split(/\n/, $self->{'cmt'});
		local $rule = { 'index' => scalar(@{$self->{'rv_array'}}),
				'type' => 'ipnat',
				'line' => $self->{'lnum'}-scalar(@cmts),
				'eline' => $self->{'lnum'},
				'file' => $file,
				'text' => $_,
				'cmt' => $self->{'cmt'},
				'active' => !$1 };
		$self->{'cmt'} = undef;

		# Parse action
		$rule->{'action'} = shift(@w);

		if ($rule->{'action'} eq 'map' ||
		    $rule->{'action'} eq 'bimap' ||
		    $rule->{'action'} eq 'map-block') {
			# Parse interface
			$rule->{'iface'} = shift(@w);

			if ($w[0] eq "from") {
				# A full from XXX to YYY block
				shift(@w);	# skip 'from'
				$rule->{'from'} = 1;
				$self->parse_object("from");
				shift(@w);	# skip 'to'
				$self->parse_object("fromto");
				}
			else {
				# IP and netmask only
				$self->parse_ipmask("from");
				}

			local $arrow = shift(@w);
			$arrow eq "->" ||
				$self->error("error parsing IPNAT line $_ at ".
				       "$arrow line $self->{'lnum'}");

			if ($w[0] eq "range") {
				# A destination IP range
				shift(@w);	# skip 'range'
				$rule->{'tostart'} = shift(@w);
				if ($rule->{'tostart'} =~ /^(\S+)\-(\S+)$/) {
					$rule->{'tostart'} = $1;
					$rule->{'toend'} = $2;
					}
				else {
					shift(@w);	# skip '-'
					$rule->{'toend'} = shift(@w);
					}
				}
			else {
				# Destination IP and netmask only
				$self->parse_ipmask("to");
				}

			if ($w[0] eq "portmap") {
				# Parse port mapping
				shift(@w);	# skip 'portmap'
				$rule->{'portmap'} = shift(@w);
				if ($w[0] eq "auto") {
					shift(@w);	# skip 'auto'
					$rule->{'portauto'} = 1;
					}
				else {
					($rule->{'portmapfrom'},
				         $rule->{'portmapto'}) =
						split(/:/, shift(@w));
					}
				}

			if ($w[0] eq "proxy") {
				# Parse proxy
				$self->parse_proxy("proxy");
				}

			# Parse options
			if ($w[0] eq "tcp/udp" ||
			    getprotobyname($w[0]) || $w[0] =~ /^\d+$/) {
				$rule->{'proto'} = shift(@w);
				}
			if ($w[0] eq "frag") {
				$rule->{'frag'} = shift(@w);
				}
			if ($w[0] eq "age") {
				shift(@w);	# skip 'age'
				$rule->{'age1'} = shift(@w);
				if ($w[0] eq "/") {
					shift(@w);	# skip '/'
					$rule->{'age2'} = shift(@w);
					}
				}
			if ($w[0] eq "mssclamp") {
				shift(@w);	# skip 'mssclamp'
				$rule->{'mssclamp'} = shift(@w);
				}
			if ($w[0] eq "proxy") {
				# Parse proxy option
				$self->parse_proxy("oproxy");
				}
			}
		elsif ($rule->{'action'} eq 'rdr') {
			# Parse redirect
			$rule->{'iface'} = shift(@w);

			# Parse IP and netmask
			$self->parse_ipmask("from");

			# Parse destination ports
			local $ports = shift(@w);
			$ports eq "port" ||
				$self->error("error parsing IPNAT line $_ at ".
				       "$ports line $self->{'lnum'}");
			$rule->{'dport1'} = shift(@w);
			if ($w[0] eq "-") {
				shift(@w);	# skip '-'
				$rule->{'dport2'} = shift(@w);
				}

			local $arrow = shift(@w);
			$arrow eq "->" ||
				$self->error("error parsing IPNAT line $_ at ".
				       "$arrow line $self->{'lnum'}");

			# Parse destination IPs
			$rule->{'rdrip'} = [ shift(@w) ];
			while($w[0] eq ",") {
				shift(@w);	# skip ,
				push(@{$rule->{'rdrip'}}, shift(@w));
				}

			# Parse destination port
			shift(@w);	# skip 'port'
			$rule->{'rdrport'} = shift(@w);

			# Parse protocol
			$rule->{'rdrproto'} = shift(@w);

			# Parse other options
			if ($w[0] eq "round-robin") {
				$rule->{'round-robin'} = shift(@w);
				}
			if ($w[0] eq "frag") {
				$rule->{'frag'} = shift(@w);
				}
			if ($w[0] eq "age") {
				shift(@w);	# skip 'age'
				$rule->{'age1'} = shift(@w);
				if ($w[0] eq "/") {
					shift(@w);	# skip '/'
					$rule->{'age2'} = shift(@w);
					}
				}
			if ($w[0] eq "mssclamp") {
				shift(@w);	# skip 'mssclamp'
				$rule->{'mssclamp'} = shift(@w);
				}
			if ($w[0] eq "proxy") {
				shift(@w);	# skip 'proxy'
				$rule->{'proxy'} = shift(@w);
				}
			}
		push(@{$self->{'rv_array'}}, $rule);
		}
	elsif (/^\s*#\s*(.*)$/) {
		# A comment line
		$self->{'cmt'} .= "\n" if ($self->{'cmt'});
		$self->{'cmt'} .= $1;
		}
	nextline:
	$self->{'lnum'}++;
	}
close(FILE);
$self->{'get_ipnat_config_cache_hash'}->{$file} = \@{$self->{'rv_array'}};
return $self->{'get_ipnat_config_cache_hash'}->{$file};
}

# parse_ipmask(prefix)
sub parse_ipmask
{
	my $self = shift;

local ($pfx) = @_;
local $ip = shift(@{$self->{'w_array'}});
if ($ip =~ /^(\S+)\/(\S+)$/) {
	$self->{'rule'}->{$pfx."ip"} = $1;
	$self->{'rule'}->{$pfx."mask"} = $2;
	}
else {
	$self->{'rule'}->{$pfx."ip"} = $ip;
	shift(@{$self->{'w_array'}});	# skip '/' or 'netmask'
	$self->{'rule'}->{$pfx."mask"} = shift(@{$self->{'w_array'}});
	}
}

# print_ipmask(prefix)
sub print_ipmask
{
	my $self = shift;

local ($pfx) = @_;
push(@{$self->{'w_array'}}, $self->{'rule'}->{$pfx."ip"}."/".$self->{'rule'}->{$pfx."mask"});
}

# parse_proxy(prefix)
sub parse_proxy
{
	my $self = shift;

local ($pfx) = @_;
shift(@{$self->{'w_array'}});	# skip 'proxy'
shift(@{$self->{'w_array'}});	# skip 'port'
$self->{'rule'}->{$pfx.'port'} = shift(@{$self->{'w_array'}});
$self->{'rule'}->{$pfx.'name'} = shift(@{$self->{'w_array'}});
if ($self->{'rule'}->{$pfx.'name'} =~ /^(\S+)\/(\S+)$/) {
	$self->{'rule'}->{$pfx.'name'} = $1;
	$self->{'rule'}->{$pfx.'proto'} = $2;
	}
else {
	shift(@{$self->{'w_array'}});	# skip '/'
	$self->{'rule'}->{$pfx.'proto'} = shift(@{$self->{'w_array'}});
	}
}

# print_proxy(prefix)
sub print_proxy
{
	my $self = shift;

local ($pfx) = @_;
push(@{$self->{'w_array'}}, "proxy", "port", $self->{'rule'}->{$pfx.'port'},
         $self->{'rule'}->{$pfx.'name'}."/".$self->{'rule'}->{$pfx.'proto'});
}

# list_protocols()
# Returns a list of IP protocols
sub list_protocols
{
	my $self = shift;

local @stdprotos = ( 'tcp', 'udp', 'icmp' );
local @otherprotos;
open(PROTOS, "/etc/protocols");
while(<PROTOS>) {
	s/\r|\n//g;
	s/#.*$//;
	push(@otherprotos, $1) if (/^(\S+)\s+(\d+)/);
	}
close(PROTOS);
@otherprotos = sort { lc($a) cmp lc($b) } @otherprotos;
return $self->unique(@stdprotos, @otherprotos);
}

# apply_configuration()
# Activates the IPfilter configuration, and returns undef on success or an
# error message on failure.
sub apply_configuration
{
	my $self = shift;

local $out;
if ($self->{'config_hash'}->{'apply_cmd'} && !$self->{'config_hash'}->{'smf'}) {
	$out = $self->backquote_logged("$self->{'config_hash'}->{'apply_cmd'} 2>&1 </dev/null");
	}
else {
	$self->system_logged("$self->{'config_hash'}->{'ipf'} -F a >/dev/null 2>&1");
	$out = $self->backquote_logged("$self->{'config_hash'}->{'ipf'} -f $self->{'config_hash'}->{'ipf_conf'} 2>&1 </dev/null");
	if (-r $self->{'config_hash'}->{'ipnat_conf'} && !$?) {
		$self->system_logged("$self->{'config_hash'}->{'ipnat'} -C -F >/dev/null 2>&1");
		$out = $self->backquote_logged("$self->{'config_hash'}->{'ipnat'} -f $self->{'config_hash'}->{'ipnat_conf'} 2>&1 </dev/null");
		}
	}
return $? || $out =~ /error|failed|unknown/i ? "<pre>$out</pre>" : undef;
}

# check_firewall_init()
# Returns 2 if started at boot, 1 or 0 if not
sub check_firewall_init
{
	my $self = shift;

if ($self->{'config_hash'}->{'smf'}) {
	# Look for SMF service
	$self->foreign_require("smf", "smf-lib.pl");
	local $state = $main::smf_object->svc_get_state_cmd($self->{'config_hash'}->{'smf'});
	return $state eq 'online' ? 2 :
	       $state eq 'disabled' || $state eq 'offline' ||
	        $state eq 'maintenance' ? 1 : 0;
	}
else {
	# Look at init script
	$self->foreign_require("init", "init-lib.pl");
	return $main::init_object->action_status($self->{'init_script'});
	}
}

# create_firewall_init()
# Create (if necessary) the ipfilter init script, and enable it
sub create_firewall_init
{
	my $self = shift;

if ($self->{'config_hash'}->{'smf'}) {
	# Enable SMF service
	$self->foreign_require("smf", "smf-lib.pl");
	local $atboot = $self->check_firewall_init();
	$atboot || $self->error($self->text('boot_esmf', "<tt>$self->{'config_hash'}->{'smf'}</tt>"));
	if ($atboot != 2) {
		$main::smf_object->svc_state_cmd($main::smf_object->{'text_hash'}->{'state_enable'},
				    [ $self->{'config_hash'}->{'smf'} ]);
		}
	}
else {
	# Create or enable init script
	local $ipf = $self->has_command($self->{'config_hash'}->{'ipf'});
	local $ipfstat = $self->has_command($self->{'config_hash'}->{'ipfstat'});
	local $start = "$ipf -F a\n".
		       "$ipf -f $self->{'config_hash'}->{'ipf_conf'}";
	local $stop = "$ipf -F a".
	$self->foreign_require("init", "init-lib.pl");
	$main::init_object->enable_at_boot($self->{'init_script'}, "Activate IPfilter firewall",
			      $start, $stop);
	}
}

# delete_firewall_init()
# Turn off the firewall at boot time
sub delete_firewall_init
{
	my $self = shift;

if ($self->{'config_hash'}->{'smf'}) {
	# Disable SMF service
	$self->foreign_require("smf", "smf-lib.pl");
	local $atboot = $self->check_firewall_init();
	$atboot || $self->error($self->text('boot_esmf', "<tt>$self->{'config_hash'}->{'smf'}</tt>"));
	if ($atboot == 2) {
		$main::smf_object->svc_state_cmd($main::smf_object->{'text_hash'}->{'state_disable'},
				    [ $self->{'config_hash'}->{'smf'} ]);
		}
	}
else {
	# Disable init script
	$self->foreign_require("init", "init-lib.pl");
	$main::init_object->disable_at_boot($self->{'init_script'});
	}
}

# unapply_configuration()
# Replace the IPfilter configuration file with active rules
sub unapply_configuration
{
	my $self = shift;

local $inrules = `$self->{'config_hash'}->{'ipfstat'} -i 2>/dev/null </dev/null`;
return $self->{'text_hash'}->{'unapply_ein'} if ($?);
local $outrules = `$self->{'config_hash'}->{'ipfstat'} -o 2>/dev/null </dev/null`;
return $self->{'text_hash'}->{'unapply_eout'} if ($?);

$self->open_lock_tempfile(OUT, ">$self->{'config_hash'}->{'ipf_conf'}");
$self->print_tempfile(OUT, $inrules);
$self->print_tempfile(OUT, $outrules);
$self->close_tempfile(OUT);

return undef;
}

# list_cluster_servers()
# Returns a list of servers on which the firewall is managed
sub list_cluster_servers
{
	my $self = shift;

$self->foreign_require("servers", "servers-lib.pl");
local %ids = map { $_, 1 } split(/\s+/, $self->{'config_hash'}->{'servers'});
return grep { $ids{$_->{'id'}} } $main::servers_object->list_servers();
}

# add_cluster_server(&server)
sub add_cluster_server
{
	my $self = shift;

local @sids = split(/\s+/, $self->{'config_hash'}->{'servers'});
$self->{'config_hash'}->{'servers'} = join(" ", @sids, $_[0]->{'id'});
$self->save_module_config();
}

# delete_cluster_server(&server)
sub delete_cluster_server
{
	my $self = shift;

local @sids = split(/\s+/, $self->{'config_hash'}->{'servers'});
$self->{'config_hash'}->{'servers'} = join(" ", grep { $_ != $_[0]->{'id'} } @sids);
$self->save_module_config();
}

# server_name(&server)
sub server_name
{
	my $self = shift;

return $_[0]->{'desc'} ? $_[0]->{'desc'} : $_[0]->{'host'};
}

# copy_to_cluster([force])
# Copy all firewall rules from this server to those in the cluster
sub copy_to_cluster
{
	my $self = shift;

return if (!$self->{'config_hash'}->{'servers'});		# no servers defined
return if (!$_[0] && $self->{'config_hash'}->{'cluster_mode'});	# only push out when applying
local $s;
foreach $s ($self->list_cluster_servers()) {
	$self->remote_foreign_require($s, "ipfilter", "ipfilter-lib.pl");
	local $iconfig = $self->remote_foreign_config($s, "ipfilter");
	$self->remote_write($s, $self->{'config_hash'}->{'ipf_conf'}, $iconfig->{'ipf_conf'});
	if ($self->{'iconfig_hash'}->{'ipnat_conf'} && -r $self->{'config_hash'}->{'ipnat_conf'} &&
	    $self->{'config_hash'}->{'cluster_nat'}) {
		$self->remote_write($s, $self->{'config_hash'}->{'ipnat_conf'},
				  $iconfig->{'ipnat_conf'});
		}
	}
}

# apply_cluster_configuration()
# Activate the current configuration on all servers in the cluster
sub apply_cluster_configuration
{
	my $self = shift;

return undef if (!$self->{'config_hash'}->{'servers'});
if ($self->{'config_hash'}->{'cluster_mode'}) {
	$self->copy_to_cluster(1);
	}
local $s;
foreach $s ($self->list_cluster_servers()) {
	$self->remote_foreign_require($s, "ipfilter", "ipfilter-lib.pl");
	local $err = $self->remote_foreign_call($s, "ipfilter", "apply_configuration");
	if ($err) {
		return $self->text('apply_remote', $s->{'host'}, $err);
		}
	}
return undef;
}

# object_input(rule, prefix, [no-thishost])
# Returns HTML for selecting an address and HTML for selecting a port
sub object_input
{
	my $self = shift;

local ($rule, $f, $nothis) = @_;

# Address part
local $ft;
$ft .= $self->ui_oneradio($f, "any", $self->{'text_hash'}->{'edit_any'},
		    $rule->{$f."-any"} || $rule->{'all'})."\n";
if ($nothis) {
	$ft .= "<br>\n";
	}
else {
	$ft .= $self->ui_oneradio($f, "thishost", $self->{'text_hash'}->{'edit_thishost'},
			    $rule->{$f."-thishost"})."<br>\n";
	}

$ft .= $self->ui_oneradio($f, "host", $self->{'text_hash'}->{'edit_host'},
		    $rule->{$f."-host"})."\n";
$ft .= $self->ui_textbox($f."_host", $rule->{$f."-host"}, 20)."\n";
$ft .= "$self->{'text_hash'}->{'edit_mask'}\n";
$ft .= $self->ui_textbox($f."_mask", $rule->{$f."-mask"}, 13)."<br>\n";

$ft .= $self->ui_oneradio($f, "numhost", $self->{'text_hash'}->{'edit_host'},
		    $rule->{$f."-numhost"}),"\n";
$ft .= $self->ui_textbox($f."_numhost", $rule->{$f."-numhost"}, 20)."\n";
$ft .= "$self->{'text_hash'}->{'edit_nummask'}\n";
$ft .= $self->ui_textbox($f."_nummask", $rule->{$f."-nummask"}, 6)."\n".
       "$self->{'text_hash'}->{'edit_opt'}<br>\n";

# Ports part
local $pt;

$pt .= $self->ui_oneradio($f."_port", "any", $self->{'text_hash'}->{'edit_anyport'},
		    !$rule->{$f."-port-comp"} &&
		    !$rule->{$f."-port-range"})."\n";
$pt .= $self->ui_oneradio($f."_port", "comp",
    $self->text('edit_portcomp',
	  $self->ui_select($f."_portcomp", $rule->{$f."-port-comp"},
		     [ map { [ $_ ] } @{$self->{'compare_ops_array'}} ],
		     0, 0, $rule->{$f."-port-comp"} ? 1 : 0),
	  $self->ui_textbox($f."_portnum", $rule->{$f."-port-num"}, 6)),
    $rule->{$f."-port-comp"})."<br>\n";
$pt .= $self->ui_oneradio($f."_port", "range",
    $self->text('edit_portrange',
	  $self->ui_textbox($f."_portstart", $rule->{$f."-port-start"}, 6),
	  $self->ui_textbox($f."_portend", $rule->{$f."-port-end"}, 6)),
    $rule->{$f."-port-range"})."<br>\n";

return ($ft, $pt);
}

# parse_object_input(rule, prefix)
sub parse_object_input
{
	my $self = shift;

local ($rule, $f) = @_;
delete($rule->{$f."-any"});
delete($rule->{$f."-thishost"});
delete($rule->{$f."-host"});
delete($rule->{$f."-numhost"});
if ($self->{'in_hash'}->{$f} eq "any") {
	$rule->{$f."-any"} = 1;
	}
elsif ($self->{'in_hash'}->{$f} eq "thishost") {
	$rule->{$f."-thishost"} = 1;
	}
elsif ($self->{'in_hash'}->{$f} eq "host") {
	gethostbyname($self->{'in_hash'}->{$f."_host"}) ||
		$self->error($self->{'text_hash'}->{'save_ehost'.$f});
	$rule->{$f."-host"} = $self->{'in_hash'}->{$f."_host"};
	$self->check_ipaddress($self->{'in_hash'}->{$f."_mask"}) ||
		$self->error($self->{'text_hash'}->{'save_emask'.$f});
	$rule->{$f."-mask"} = $self->{'in_hash'}->{$f."_mask"};
	}
elsif ($self->{'in_hash'}->{$f} eq "numhost") {
	gethostbyname($self->{'in_hash'}->{$f."_numhost"}) ||
		$self->error($self->{'text_hash'}->{'save_ehost'.$f});
	$rule->{$f."-numhost"} = $self->{'in_hash'}->{$f."_numhost"};
	$self->{'in_hash'}->{$f."_nummask"} = "32" if ($self->{'in_hash'}->{$f."_nummask"} eq "");
	$self->{'in_hash'}->{$f."_nummask"} =~ /^\d+$/ &&
	    $self->{'in_hash'}->{$f."_nummask"} <= 32 ||
		$self->error($self->{'text_hash'}->{'save_enummask'.$f});
	$rule->{$f."-nummask"} = $self->{'in_hash'}->{$f."_nummask"};
	}

# Parse port section
delete($rule->{$f."-port-comp"});
delete($rule->{$f."-port-range"});
if ($self->{'in_hash'}->{$f."_port"} eq "comp") {
	$self->valid_port($self->{'in_hash'}->{$f."_portnum"}) ||
		$self->error($self->{'text_hash'}->{'save_eportnum'.$f});
	$rule->{$f."-port-num"} = $self->{'in_hash'}->{$f."_portnum"};
	$rule->{$f."-port-comp"} = $self->{'in_hash'}->{$f."_portcomp"};
	}
elsif ($self->{'in_hash'}->{$f."_port"} eq "range") {
	$self->valid_port($self->{'in_hash'}->{$f."_portstart"}) ||
		$self->error($self->{'text_hash'}->{'save_eportstart'.$f});
	$self->valid_port($self->{'in_hash'}->{$f."_portend"}) ||
		$self->error($self->{'text_hash'}->{'save_eportend'.$f});
	$rule->{$f."-port-range"} ||= "<>";
	$rule->{$f."-port-start"} = $self->{'in_hash'}->{$f."_portstart"};
	$rule->{$f."-port-end"} = $self->{'in_hash'}->{$f."_portend"};
	}
}

# protocol_input(name, value, add-any, add-tcp-udp)
sub protocol_input
{
	my $self = shift;

local ($name, $value, $any, $tcpudp) = @_;
return $self->ui_select($name, $value,
	       [ $any ? ( [ "", $self->{'text_hash'}->{'edit_protoany'} ] ) : ( ),
		 $tcpudp ? ( [ "tcp/udp", $self->{'text_hash'}->{'edit_prototcpudp'} ] ) : ( ),
		 map { [ $_, uc($_) ] } $self->list_protocols() ],
	       0, 0, $value ? 1 : 0);
}

# valid_port(name|number)
# Returns 1 if give a valid port number or TCP or UDP name
sub valid_port
{
	my $self = shift;

local $n = $_[0];
return getservbyname($n, "tcp") ||
       getservbyname($n, "udp") ||
       ($n =~ /^\d+$/ && $n > 0 && $n < 65536);
}

sub valid_hexdec
{
	my $self = shift;

return $_[0] =~ /^\d+$/ || $_[0] =~ /^0x([0-9a-f]+)$/;
}


1;

