#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Show all firewall rules

&ModuleLoader::preload_module('ipfilter_class', "ipfilter::IpfilterLib");
$self = new ipfilter::IpfilterLib:: 'ipfilter_class',%$self;

$self->ReadParse();

# Make sure the ipf command is installed
$self->{'cmd'} = $self->missing_firewall_commands();
if ($self->{'cmd'}) {
	$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", undef, 1, 1);
	$self->ui_print_endpage(
		$self->ui_config_link('index_ecmd',
			[ "<tt>$self->{'cmd'}</tt>", undef ]));
	}

# Get the version number
$self->{'vout'} = `$self->{'config_hash'}->{'ipf'} -V 2>&1`;
if ($self->{'vout'} =~ /IP\s+Filter:\s+v?(\S+)/i) {
	$self->{'ipf_version'} = $1;
	}
open(VERSION, ">$self->{'module_config_directory'}/version");
print VERSION $self->{'ipf_version'},"\n";
close(VERSION);
$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", undef, 1, 1, 0,
		 $self->help_search_link("ipf", "man", "doc", "google"),
		 undef, undef,
		 $self->text('index_version', $self->{'ipf_version'}));

# See if enabled at boot
$self->{'atboot'} = $self->check_firewall_init();

# Get the saved and active rules
$self->{'live'} = $self->get_live_config();
$self->{'rules'} = $self->get_config();

if (!-r $self->{'config_hash'}->{'ipf_conf'} && @{$self->{'live'}}) {
	# Offer to save existing rules
	print $self->text('index_existing', scalar(@{$self->{'live'}}),
		    "<tt>$self->{'config_hash'}->{'ipf_conf'}</tt>"),"<p>\n";
	print $self->ui_form_start("convert.cgi");
	print "<center>",$self->ui_submit($self->{'text_hash'}->{'index_saveex'}),"<p>\n";
	print $self->ui_checkbox("atboot", 1, $self->{'text_hash'}->{'index_atboot'},
			   $self->{'atboot'} == 2),"\n";
	print "</center>",$self->ui_form_end(),"<p>\n";

	print "<table border width=100%>\n";
	print "<tr $self->{'tb'}><td><b>$self->{'text_hash'}->{'index_headerex'}</b></td></tr>\n";
	print "<tr $self->{'cb'}> <td><pre>";
	foreach $dir ("i", "o") {
		open(OUT, "$self->{'config_hash'}->{'ipfstat'} -$dir |");
		while(<OUT>) {
			print $self->html_escape($_);
			}
		close(OUT);
		}
	print "</pre></td> </tr></table>\n";
	}
elsif (@{$self->{'rules'}} && !$self->{'in_hash'}->{'reset'}) {
	# Show the rules
	print $self->ui_form_start("edit_rule.cgi");
	local @widths = ( "width=10", "width=5%", "width=10%", undef );
	push(@widths, undef) if ($self->{'config_hash'}->{'view_condition'});
	push(@widths, undef) if ($self->{'config_hash'}->{'view_comment'});
	push(@widths, "width=5%", "width=5%");
	@{$self->{'links_array'}} = ( $self->select_all_link("d", 0),
		   $self->select_invert_link("d", 0) );
	print $self->ui_links_row(\@{$self->{'links_array'}});
	print $self->ui_columns_start([ "",
				  $self->{'text_hash'}->{'index_active'},
				  $self->{'text_hash'}->{'index_action'},
				  $self->{'text_hash'}->{'index_dir'},
			    	  $self->{'config_hash'}->{'view_condition'} ?
					( $self->{'text_hash'}->{'index_desc'} ) : ( ),
			    	  $self->{'config_hash'}->{'view_comment'} ?
					( $self->{'text_hash'}->{'index_cmt'} ) : ( ),
			    	  $self->{'text_hash'}->{'index_move'},
			    	  $self->{'text_hash'}->{'index_radd'} ], 100, 0,
				\@widths);
	foreach $r (@{$self->{'rules'}}) {
		local ($mover, $adder);
		if ($r eq $self->{'rules'}->[@{$self->{'rules'}}-1]) {
			$mover .= "<img src=images/gap.gif>";
			}
		else {
			$mover .= "<a href='move.cgi?idx=$r->{'index'}&".
			      	  "down=1'><img src=".
			      	  "images/down.gif border=0></a>";
			}
		if ($r eq $self->{'rules'}->[0]) {
			$mover .= "<img src=images/gap.gif>";
			}
		else {
			$mover .= "<a href='move.cgi?idx=$r->{'index'}&".
			          "up=1'><img src=images/up.gif ".
			          "border=0></a>";
			}
		$adder .= "<a href='edit_rule.cgi?new=1&".
			  "after=$r->{'index'}'>".
			  "<img src=images/after.gif border=0></a>";
		$adder .= "<a href='edit_rule.cgi?new=1&".
			  "before=$r->{'index'}'>".
			  "<img src=images/before.gif border=0></a>";

		local $ls = "<a href='edit_rule.cgi?idx=$r->{'index'}'>";
		local $le = "</a>";
		local $act = $self->{'amap_hash'}->{int($r->{'num'})};
		print $self->ui_checked_columns_row(
			[ $ls.($r->{'active'} ? $self->{'text_hash'}->{'yes'} :
			       "<font color=#ff0000>$self->{'text_hash'}->{'no'}</font>").$le,
			  $ls.($self->{'text_hash'}->{'action_'.$r->{'action'}} ||
			   uc($r->{'action'})).$le,
			  $ls.$self->{'text_hash'}->{'dir_'.$r->{'dir'}}.$le,
			  $self->{'config_hash'}->{'view_condition'} ?
				( $self->describe_rule($r) ) : ( ),
			  $self->{'config_hash'}->{'view_comment'} ?
				( $r->{'cmt'} || "<br>" ) : ( ),
			  $mover,
			  $adder ],
			\@widths, "d", $r->{'index'});
		}
	print $self->ui_columns_end();
	print $self->ui_links_row(\@{$self->{'links_array'}});

	# Buttons to delete and add
	print "<table width=100%><tr>\n";
	print "<td align=left>",
	      $self->ui_submit($self->{'text_hash'}->{'index_delete'}, "delsel"),"</td>\n";
	print "<td align=right>",
	      $self->ui_submit($self->{'text_hash'}->{'index_add2'}, "new"),"</td>\n";
	print "</tr></table>\n";
	print $self->ui_form_end();

	# Show NAT rules
	print "<hr>\n";
	$self->{'natrules'} = $self->get_ipnat_config();
	print $self->ui_form_start("edit_nat.cgi");
	if (@{$self->{'natrules'}}) {
		local @widths = ( "width=10", "width=5%", "width=10%", undef );
		push(@widths, undef) if ($self->{'config_hash'}->{'view_condition'});
		push(@widths, undef) if ($self->{'config_hash'}->{'view_comment'});
		push(@widths, "width=5%", "width=5%");
		print $self->select_all_link("d", 1),"\n";
		print $self->select_invert_link("d", 1),"<br>\n";
		print $self->ui_columns_start([ "",
					  $self->{'text_hash'}->{'index_active'},
					  $self->{'text_hash'}->{'index_nataction'},
					  $self->{'config_hash'}->{'view_condition'} ?
						( $self->{'text_hash'}->{'index_natfrom'},
						  $self->{'text_hash'}->{'index_natto'} ) : ( ),
					  $self->{'config_hash'}->{'view_comment'} ?
						( $self->{'text_hash'}->{'index_cmt'} ) : ( ),
					  $self->{'text_hash'}->{'index_move'} ], 100, 0,
					\@widths);
		foreach $r (@{$self->{'natrules'}}) {
			local ($mover, $adder);
			if ($r eq $self->{'natrules'}->[@{$self->{'natrules'}}-1]) {
				$mover .= "<img src=images/gap.gif>";
				}
			else {
				$mover .= "<a href='natmove.cgi?idx=$r->{'index'}&".
					  "down=1'><img src=".
					  "images/down.gif border=0></a>";
				}
			if ($r eq $self->{'natrules'}->[0]) {
				$mover .= "<img src=images/gap.gif>";
				}
			else {
				$mover .= "<a href='natmove.cgi?idx=$r->{'index'}&".
					  "up=1'><img src=images/up.gif ".
					  "border=0></a>";
				}

			local $ls = "<a href='edit_nat.cgi?idx=$r->{'index'}'>";
			local $le = "</a>";
			print $self->ui_columns_row(
				[ $self->ui_checkbox("d", $r->{'index'}, "", 0),
				  $ls.($r->{'active'} ? $self->{'text_hash'}->{'yes'} :
				       "<font color=#ff0000>$self->{'text_hash'}->{'no'}</font>").$le,
				  $ls.($self->{'text_hash'}->{'action_'.$r->{'action'}} ||
				   uc($r->{'action'})).$le,
				  $self->{'config_hash'}->{'view_condition'} ?
					( $self->describe_from($r), $self->describe_to($r) ) : ( ),
				  $self->{'config_hash'}->{'view_comment'} ?
					( $r->{'cmt'} || "<br>" ) : ( ),
				  $mover ],
				\@widths);
			}
		print $self->ui_columns_end();
		print $self->select_all_link("d", 1),"\n";
		print $self->select_invert_link("d", 1),"<br>\n";
		print "<table width=100%><tr>\n";
		print "<td align=left>",
		      $self->ui_submit($self->{'text_hash'}->{'index_delete'}, "delsel"),"</td>\n";
		print "<td align=right>",
		      $self->ui_submit($self->{'text_hash'}->{'index_add3'}, "newmap"),"\n",
		      $self->ui_submit($self->{'text_hash'}->{'index_add4'}, "newrdr"),"</td>\n";
		print "</tr></table>\n";
		}
	else {
		print "<b>$self->{'text_hash'}->{'index_natnone'}</b><p>\n";
		print "<table width=100%><tr>\n";
		print "<td align=right>",
		      $self->ui_submit($self->{'text_hash'}->{'index_add3'}, "newmap"),"\n",
		      $self->ui_submit($self->{'text_hash'}->{'index_add4'}, "newrdr"),"</td>\n";
		print "</tr></table>\n";
		}
	print $self->ui_form_end();

	# Show buttons to apply configuration and start at boot
	print "<hr>\n";

	print $self->ui_buttons_start();
	if ($self->foreign_check("servers")) {
		@{$self->{'servers_array'}} = $self->list_cluster_servers();
		}
	print $self->ui_buttons_row("apply.cgi", $self->{'text_hash'}->{'index_apply'},
			      @{$self->{'servers_array'}} ? $self->{'text_hash'}->{'index_applydesc2'}
				       : $self->{'text_hash'}->{'index_applydesc'});
	print $self->ui_buttons_row("unapply.cgi", $self->{'text_hash'}->{'index_unapply'},
			      $self->{'text_hash'}->{'index_unapplydesc'});
	print $self->ui_buttons_row("bootup.cgi", $self->{'text_hash'}->{'index_boot'},
			      $self->{'text_hash'}->{'index_bootdesc'}, undef,
			      $self->ui_radio("boot", $self->{'atboot'} == 2 ? 1 : 0,
					[ [ 1, $self->{'text_hash'}->{'yes'} ],
					  [ 0, $self->{'text_hash'}->{'no'} ] ]));
	print $self->ui_buttons_row("index.cgi", $self->{'text_hash'}->{'index_reset'},
			      $self->{'text_hash'}->{'index_resetdesc'}, undef,
			      $self->ui_hidden("reset", 1));
	# Show button for cluster page
	if ($self->foreign_check("servers")) {
		$self->foreign_require("servers", "servers-lib.pl");
		@{$self->{'allservers_array'}} = grep { $_->{'user'} }
				$main::servers_object->list_servers();
		}
	if (@{$self->{'allservers_array'}}) {
		print $self->ui_buttons_row("cluster.cgi", $self->{'text_hash'}->{'index_cluster'},
				      $self->{'text_hash'}->{'index_clusterdesc'});
		}
	print $self->ui_buttons_end();
	}
else {
	# Offer to setup simple firewall
	print $self->text($self->{'in_hash'}->{'reset'} ? 'index_rsetup' : 'index_setup',
		    "<tt>$self->{'ipfw_file'}</tt>"),"<p>\n";
	print $self->ui_form_start("setup.cgi");
	print $self->ui_hidden("reset", $self->{'in_hash'}->{'reset'});
	print "<center><table><tr><td>\n";
	print $self->ui_oneradio("auto", 0, $self->{'text_hash'}->{'index_auto0'}, 1),"<p>\n";
	foreach $a (1 .. 4) {
		print $self->ui_oneradio("auto", $a, $self->{'text_hash'}->{'index_auto'.$a}, 0)," ",
		      $self->interface_choice("iface".$a, undef, 1),"<p>\n";
		}
	print "</td></tr></table>\n";
	print $self->ui_submit($self->{'text_hash'}->{'index_auto'}),"<p>\n";
	print $self->ui_checkbox("atboot", 1, $self->{'text_hash'}->{'index_atboot'},
			   $self->{'atboot'} == 2),"\n";
	print "</center>",$self->ui_form_end(),"\n";
	}

$self->ui_print_footer("/", $self->{'text_hash'}->{'index'});

1;
		
}


1;

