#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Update one NAT rule

&ModuleLoader::preload_module('ipfilter_class', "ipfilter::IpfilterLib");
$self = new ipfilter::IpfilterLib:: 'ipfilter_class',%$self;

$self->ReadParse();
$self->{'rules'} = $self->get_ipnat_config();
if (!$self->{'in_hash'}->{'new'}) {
	# Get the rule
	$self->{'rule'} = $self->{'rules'}->[$self->{'in_hash'}->{'idx'}];
	}
else {
	$self->{'rule'} = { 'file' => $self->{'config_hash'}->{'ipnat_conf'},
		  'type' => 'ipnat' };
	}

if ($self->{'in_hash'}->{'delete'}) {
	# Just deleting
	$self->lock_file($self->{'rule'}->{'file'});
	$self->delete_rule($self->{'rule'});
	$self->flush_file_lines();
	$self->unlock_file($self->{'rule'}->{'file'});
	$self->webmin_log("delete", "nat", undef, $self->{'rule'});
	$self->redirect("");
	exit;
	}

# Validate and store inputs, starting with action
$self->error_setup($self->{'text_hash'}->{'nat_err'});
$self->{'rule'}->{'cmt'} = $self->{'in_hash'}->{'cmt'};
$self->{'rule'}->{'active'} = $self->{'in_hash'}->{'active'};
$self->{'rule'}->{'action'} = $self->{'in_hash'}->{'action'};

if ($self->{'rule'}->{'action'} ne 'rdr') {
	# Parse source options
	$self->{'rule'}->{'iface'} = $self->parse_interface_choice("iface", $self->{'text_hash'}->{'nat_eiface'});
	if ($self->{'in_hash'}->{'frommode'} == 0) {
		delete($self->{'rule'}->{'from'});
		&parse_ipmask_input("from");
		}
	else {
		$self->{'in_hash'}->{'action'} eq 'map-block' && $self->error($self->{'text_hash'}->{'nat_emapblock1'});
		$self->{'rule'}->{'from'} = 1;
		$self->parse_object_input($self->{'rule'}, "from");
		$self->parse_object_input($self->{'rule'}, "fromto");
		}

	# Parse destination
	delete($self->{'rule'}->{'tostart'});
	if ($self->{'in_hash'}->{'tomode'} == 0) {
		&parse_ipmask_input("to");
		}
	elsif ($self->{'in_hash'}->{'tomode'} == 2) {
		$self->{'rule'}->{'toip'} = '0.0.0.0';
		$self->{'rule'}->{'tomask'} = 32;
		}
	else {
		$self->{'in_hash'}->{'action'} eq 'map-block' && $self->error($self->{'text_hash'}->{'nat_emapblock2'});
		$self->check_ipaddress($self->{'in_hash'}->{'tostart'}) ||
			$self->error($self->{'text_hash'}->{'nat_etostart'});
		$self->check_ipaddress($self->{'in_hash'}->{'toend'}) ||
			$self->error($self->{'text_hash'}->{'nat_etoend'});
		$self->{'rule'}->{'tostart'} = $self->{'in_hash'}->{'tostart'};
		$self->{'rule'}->{'toend'} = $self->{'in_hash'}->{'toend'};
		}

	# Parse port mapping
	if ($self->{'in_hash'}->{'portmapmode'} == 0) {
		delete($self->{'rule'}->{'portmap'});
		}
	else {
		$self->{'rule'}->{'portmap'} = $self->{'in_hash'}->{'portmap'};
		if ($self->{'in_hash'}->{'portmapnoauto'}) {
			$self->{'rule'}->{'portauto'} = 0;
			$self->valid_port($self->{'in_hash'}->{'portmapfrom'}) ||
				$self->error($self->{'text_hash'}->{'nat_eportmapfrom'});
			$self->valid_port($self->{'in_hash'}->{'portmapto'}) ||
				$self->error($self->{'text_hash'}->{'nat_eportmapto'});
			$self->{'rule'}->{'portmapfrom'} = $self->{'in_hash'}->{'portmapfrom'};
			$self->{'rule'}->{'portmapto'} = $self->{'in_hash'}->{'portmapto'};
			}
		else {
			$self->{'rule'}->{'portauto'} = 1;
			}
		}

	# Parse application proxy
	if ($self->{'in_hash'}->{'proxymode'} == 0) {
		delete($self->{'rule'}->{'proxyport'});
		}
	else {
		&parse_proxy_input("proxy");
		}

	# Parse other options
	if ($self->{'in_hash'}->{'proto'}) {
		$self->{'rule'}->{'proto'} = $self->{'in_hash'}->{'protoproto'};
		}
	else {
		delete($self->{'rule'}->{'proto'});
		}
	$self->{'rule'}->{'frag'} = $self->{'in_hash'}->{'frag'};
	if ($self->{'in_hash'}->{'mssclamp'}) {
		$self->{'in_hash'}->{'mss'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'nat_emss'});
		$self->{'rule'}->{'mssclamp'} = $self->{'in_hash'}->{'mss'};
		}
	else {
		delete($self->{'rule'}->{'mssclamp'});
		}
	if ($self->{'in_hash'}->{'oproxy'}) {
		&parse_proxy_input("oproxy");
		}
	else {
		delete($self->{'rule'}->{'oproxyport'});
		}
	}
else {
	# Validate and store redirect inputs
	$self->{'rule'}->{'iface'} = $self->parse_interface_choice("iface", $self->{'text_hash'}->{'nat_eiface'});

	# Save redirect address
	&parse_ipmask_input("from");

	# Save destination ports
	if ($self->{'in_hash'}->{'dportsmode'} == 0) {
		$self->valid_port($self->{'in_hash'}->{'dport'}) || $self->error($self->{'text_hash'}->{'nat_edport'});
		$self->{'rule'}->{'dport1'} = $self->{'in_hash'}->{'dport'};
		delete($self->{'rule'}->{'dport2'});
		}
	else {
		$self->valid_port($self->{'in_hash'}->{'dport1'}) || $self->error($self->{'text_hash'}->{'nat_edport1'});
		$self->valid_port($self->{'in_hash'}->{'dport2'}) || $self->error($self->{'text_hash'}->{'nat_edport2'});
		$self->{'rule'}->{'dport1'} = $self->{'in_hash'}->{'dport1'};
		$self->{'rule'}->{'dport2'} = $self->{'in_hash'}->{'dport2'};
		}

	# Save protocol
	$self->{'rule'}->{'rdrproto'} = $self->{'in_hash'}->{'rdrproto'};

	# Save redirect IPs
	@{$self->{'ips_array'}} = split(/\s+/, $self->{'in_hash'}->{'rdrip'});
	foreach $ip (@{$self->{'ips_array'}}) {
		$self->check_ipaddress($ip) || $self->error($self->text('net_erdrip', $ip));
		}
	@{$self->{'ips_array'}} || $self->error($self->{'text_hash'}->{'nat_erdrips'});
	$self->{'rule'}->{'rdrip'} = \@{$self->{'ips_array'}};

	# Save redirect port
	$self->valid_port($self->{'in_hash'}->{'rdrport'}) || $self->error($self->{'text_hash'}->{'nat_erdrport'});
	$self->{'rule'}->{'rdrport'} = $self->{'in_hash'}->{'rdrport'};

	# Save options
	$self->{'rule'}->{'round-robin'} = $self->{'in_hash'}->{'round-robin'};
	$self->{'rule'}->{'frag'} = $self->{'in_hash'}->{'frag'};
	if ($self->{'in_hash'}->{'mssclamp'}) {
		$self->{'in_hash'}->{'mss'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'nat_emss'});
		$self->{'rule'}->{'mssclamp'} = $self->{'in_hash'}->{'mss'};
		}
	else {
		delete($self->{'rule'}->{'mssclamp'});
		}
	}

$self->lock_file($self->{'rule'}->{'file'});
if ($self->{'in_hash'}->{'new'}) {
	if ($self->{'in_hash'}->{'before'} ne '') {
		# Insert before some rule
		$self->{'before'} = $self->{'rules'}->[$self->{'in_hash'}->{'before'}];
		$self->insert_rule($self->{'rule'}, $self->{'before'});
		}
	elsif ($self->{'in_hash'}->{'after'} ne '') {
		if ($self->{'in_hash'}->{'after'} == @{$self->{'rules'}} - 1) {
			$self->create_rule($self->{'rule'});	# at end anyway
			}
		else {
			# Insert after some rule
			$self->{'before'} = $self->{'rules'}->[$self->{'in_hash'}->{'after'}+1];
			$self->insert_rule($self->{'rule'}, $self->{'before'});
			}
		}
	else {
		# Append to end
		$self->create_rule($self->{'rule'});
		}
	}
else {
	$self->modify_rule($self->{'rule'});
	}
$self->flush_file_lines();
$self->unlock_file($self->{'rule'}->{'file'});
$self->copy_to_cluster();
$self->webmin_log($self->{'in_hash'}->{'new'} ? "create" : "modify", "nat", undef, $self->{'rule'});

$self->redirect("");

1;
		
}

# parse_ipmask_input(prefix)
sub parse_ipmask_input
{
	shift if ($self == $_[0]);

local ($pfx) = @_;
$self->check_ipaddress($self->{'in_hash'}->{$pfx."ip"}) || $self->error($self->{'text_hash'}->{'nat_e'.$pfx.'ip'});
$self->check_ipaddress($self->{'in_hash'}->{$pfx."mask"}) ||
    $self->{'in_hash'}->{$pfx."mask"} =~ /^\d+$/ &&
    $self->{'in_hash'}->{$pfx."mask"} >= 0 && $self->{'in_hash'}->{$pfx."mask"} <= 32 ||
	$self->error($self->{'text_hash'}->{'nat_e'.$pfx.'ip'});
$self->{'rule'}->{$pfx."ip"} = $self->{'in_hash'}->{$pfx."ip"};
$self->{'rule'}->{$pfx."mask"} = $self->{'in_hash'}->{$pfx."mask"};
}

sub parse_proxy_input
{
	shift if ($self == $_[0]);

local ($pfx) = @_;
$self->valid_port($self->{'in_hash'}->{$pfx."port"}) || $self->error($self->{'text_hash'}->{'nat_e'.$pfx.'port'});
$self->valid_port($self->{'in_hash'}->{$pfx."name"}) || $self->error($self->{'text_hash'}->{'nat_e'.$pfx.'name'});
$self->{'rule'}->{$pfx."port"} = $self->{'in_hash'}->{$pfx."port"};
$self->{'rule'}->{$pfx."name"} = $self->{'in_hash'}->{$pfx."name"};
$self->{'rule'}->{$pfx."proto"} = $self->{'in_hash'}->{$pfx."proto"};
}


1;

