#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Update one firewall rule

&ModuleLoader::preload_module('ipfilter_class', "ipfilter::IpfilterLib");
$self = new ipfilter::IpfilterLib:: 'ipfilter_class',%$self;

$self->ReadParse();
$self->{'rules'} = $self->get_config();
if (!$self->{'in_hash'}->{'new'}) {
	# Get the rule
	$self->{'rule'} = $self->{'rules'}->[$self->{'in_hash'}->{'idx'}];
	}
else {
	$self->{'rule'} = { 'file' => $self->{'config_hash'}->{'ipf_conf'},
		  'type' => 'ipf' };
	}

if ($self->{'in_hash'}->{'delete'}) {
	# Just deleting
	$self->lock_file($self->{'rule'}->{'file'});
	$self->delete_rule($self->{'rule'});
	$self->flush_file_lines();
	$self->unlock_file($self->{'rule'}->{'file'});
	$self->webmin_log("delete", "rule", undef, $self->{'rule'});
	$self->redirect("");
	exit;
	}

# Validate and store inputs, starting with action
$self->{'rule'}->{'cmt'} = $self->{'in_hash'}->{'cmt'};
$self->{'rule'}->{'active'} = $self->{'in_hash'}->{'active'};
$self->{'rule'}->{'action'} = $self->{'in_hash'}->{'action'};
if ($self->{'rule'}->{'action'} eq "block") {
	# Parse ICMP block options
	if ($self->{'in_hash'}->{'block_return'}) {
		$self->{'rule'}->{'block-return'} = $self->{'in_hash'}->{'block_return'};
		$self->{'rule'}->{'block-return-dest'} = $self->{'in_hash'}->{'block_return_dest'};
		}
	else {
		$self->{'rule'}->{'block-return'} = undef;
		}
	}
elsif ($self->{'rule'}->{'action'} eq "log") {
	# Parse logging options
	&parse_logging_options("log");
	}
elsif ($self->{'rule'}->{'action'} eq "skip") {
	# Save rule to skip to
	$self->{'in_hash'}->{'skip'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'save_eskip'});
	$self->{'rule'}->{'skip'} = $self->{'in_hash'}->{'skip'};
	}
elsif ($self->{'rule'}->{'action'} eq "call") {
	# Save function to call
	$self->{'in_hash'}->{'call'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'save_ecall'});
	$self->{'rule'}->{'call'} = $self->{'in_hash'}->{'call'};
	$self->{'rule'}->{'call-now'} = $self->{'in_hash'}->{'call_now'};
	}

# Parse source and destination
$self->{'rule'}->{'all'} = $self->{'in_hash'}->{'all'};
if (!$self->{'in_hash'}->{'all'}) {
	$self->parse_object_input($self->{'rule'}, "from");
	$self->parse_object_input($self->{'rule'}, "to");
	}

# Parse other conditions
$self->{'rule'}->{'dir'} = $self->{'in_hash'}->{'dir'};
$self->{'rule'}->{'proto'} = $self->{'in_hash'}->{'proto'};
if ($self->{'in_hash'}->{'tos_def'}) {
	delete($self->{'rule'}->{'tos'});
	}
else {
	$self->valid_hexdec($self->{'in_hash'}->{'tos'}) || $self->error($self->{'text_hash'}->{'save_etos'});
	$self->{'rule'}->{'tos'} = $self->{'in_hash'}->{'tos'};
	}
if ($self->{'in_hash'}->{'ttl_def'}) {
	delete($self->{'rule'}->{'ttl'});
	}
else {
	$self->valid_hexdec($self->{'in_hash'}->{'ttl'}) || $self->error($self->{'text_hash'}->{'save_ettl'});
	$self->{'rule'}->{'ttl'} = $self->{'in_hash'}->{'ttl'};
	}
if ($self->{'in_hash'}->{'on'} eq "") {
	delete($self->{'rule'}->{'on'});
	}
else {
	$self->{'rule'}->{'on'} = $self->parse_interface_choice("on", $self->{'text_hash'}->{'save_eon'});
	}
if ($self->{'in_hash'}->{'flags1_def'}) {
	delete($self->{'rule'}->{'flags1'});
	delete($self->{'rule'}->{'flags2'});
	}
else {
	$self->{'in_hash'}->{'flags1'} =~ /^[FSRPAU]+$/ || $self->error($self->{'text_hash'}->{'save_eflags1'});
	$self->{'in_hash'}->{'flags2'} =~ /^[FSRPAU]*$/ || $self->error($self->{'text_hash'}->{'save_eflags2'});
	$self->{'rule'}->{'flags1'} = $self->{'in_hash'}->{'flags1'};
	$self->{'rule'}->{'flags2'} = $self->{'in_hash'}->{'flags2'};
	}
if (!$self->{'in_hash'}->{'icmptype'}) {
	delete($self->{'rule'}->{'icmp-type'});
	}
else {
	lc($self->{'rule'}->{'proto'}) eq "icmp" || $self->error($self->{'text_hash'}->{'save_eicmp'});
	$self->{'rule'}->{'icmp-type'} = $self->{'in_hash'}->{'icmptype'};
	$self->{'rule'}->{'icmp-type-code'} = $self->{'in_hash'}->{'icmpcode'};
	}

# Parse action options
$self->{'rule'}->{'quick'} = $self->{'in_hash'}->{'quick'};
$self->{'rule'}->{'olog'} = $self->{'in_hash'}->{'olog'};
if ($self->{'in_hash'}->{'olog'}) {
	&parse_logging_options("olog");
	}
if ($self->{'in_hash'}->{'tag'}) {
	$self->valid_hexdec($self->{'in_hash'}->{'tagid'}) || $self->error($self->{'text_hash'}->{'save_etag'});
	$self->{'rule'}->{'tag'} = $self->{'in_hash'}->{'tagid'};
	}
else {
	delete($self->{'rule'}->{'tag'});
	}
if ($self->{'in_hash'}->{'dup_to'}) {
	$self->{'rule'}->{'dup-to'} = $self->parse_interface_choice("dup_toiface",
						    $self->{'text_hash'}->{'save_edupto'});
	if ($self->{'in_hash'}->{'dup_toip'}) {
		$self->check_ipaddress($self->{'in_hash'}->{'dup_toip'}) ||
			$self->error($self->{'text_hash'}->{'save_eduptoip'});
		$self->{'rule'}->{'dup-to'} .= ":".$self->{'in_hash'}->{'dup_toip'};
		}
	}
else {
	delete($self->{'rule'}->{'dup-to'});
	}
if ($self->{'in_hash'}->{'fastroute'}) {
	$self->{'rule'}->{'fastroute'} = $self->parse_interface_choice("fastrouteiface",
						       $self->{'text_hash'}->{'save_eto'});
	if ($self->{'in_hash'}->{'fastrouteip'}) {
		$self->check_ipaddress($self->{'in_hash'}->{'fastrouteip'}) ||
			$self->error($self->{'text_hash'}->{'save_etoip'});
		}
	$self->{'rule'}->{'fastroute-ip'} = $self->{'in_hash'}->{'fastrouteip'};
	}
else {
	delete($self->{'rule'}->{'fastroute'});
	}
if ($self->{'in_hash'}->{'reply_to'}) {
	$self->{'rule'}->{'reply-to'} = $self->parse_interface_choice("reply_toiface",
					   $self->{'text_hash'}->{'save_ereplyto'});
	$self->{'rule'}->{'reply-to'} =~ /^[a-z]+\d*/ || $self->error($self->{'text_hash'}->{'save_ereply_to'});
	if ($self->{'in_hash'}->{'reply_toip'}) {
		$self->check_ipaddress($self->{'in_hash'}->{'reply_toip'}) ||
			$self->error($self->{'text_hash'}->{'save_ereplytoip'});
		}
	$self->{'rule'}->{'reply-to-ip'} = $self->{'in_hash'}->{'reply_toip'};
	}
else {
	delete($self->{'rule'}->{'reply-to'});
	}
if ($self->{'in_hash'}->{'keep'}) {
	$self->{'rule'}->{'keep'} = $self->{'in_hash'}->{'keepmode'};
	}
else {
	delete($self->{'rule'}->{'keep'});
	}

$self->lock_file($self->{'rule'}->{'file'});
if ($self->{'in_hash'}->{'new'}) {
	if ($self->{'in_hash'}->{'before'} ne '') {
		# Insert before some rule
		$self->{'before'} = $self->{'rules'}->[$self->{'in_hash'}->{'before'}];
		$self->insert_rule($self->{'rule'}, $self->{'before'});
		}
	elsif ($self->{'in_hash'}->{'after'} ne '') {
		if ($self->{'in_hash'}->{'after'} == @{$self->{'rules'}} - 1) {
			$self->create_rule($self->{'rule'});	# at end anyway
			}
		else {
			# Insert after some rule
			$self->{'before'} = $self->{'rules'}->[$self->{'in_hash'}->{'after'}+1];
			$self->insert_rule($self->{'rule'}, $self->{'before'});
			}
		}
	else {
		# Append to end
		$self->create_rule($self->{'rule'});
		}
	}
else {
	$self->modify_rule($self->{'rule'});
	}
$self->flush_file_lines();
$self->unlock_file($self->{'rule'}->{'file'});
$self->copy_to_cluster();
$self->webmin_log($self->{'in_hash'}->{'new'} ? "create" : "modify", "rule", undef, $self->{'rule'});

$self->redirect("");

1;
		
}

# parse_logging_options(prefix)
sub parse_logging_options
{
	shift if ($self == $_[0]);

local $pfx = $_[0];
if ($self->{'in_hash'}->{$pfx."_pri"}) {
	if ($self->{'in_hash'}->{$pfx."_fac"}) {
		$self->{'rule'}->{$pfx."-level"} = $self->{'in_hash'}->{$pfx."_fac"}.".".$self->{'in_hash'}->{$pfx."_pri"};
		}
	else {
		$self->{'rule'}->{$pfx."-level"} = $self->{'in_hash'}->{$pfx."_pri"};
		}
	}
else {
	delete($self->{'rule'}->{$pfx."-level"});
	}
$self->{'rule'}->{$pfx."-body"} = $self->{'in_hash'}->{$pfx."_body"};
$self->{'rule'}->{$pfx."-first"} = $self->{'in_hash'}->{$pfx."_first"};
$self->{'rule'}->{$pfx."-or-block"} = $self->{'in_hash'}->{$pfx."_orblock"};
}


1;

