#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Show all firewall rules

&ModuleLoader::preload_module('ipfw_class', "ipfw::IpfwLib");
$self = new ipfw::IpfwLib:: 'ipfw_class',%$self;

$self->ReadParse();

# Make sure the ipfw command is installed
if (!$self->has_command($self->{'config_hash'}->{'ipfw'})) {
	$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", undef, 1, 1);
	$self->ui_print_endpage(
		$self->ui_config_link('index_eipfw',
			[ "<tt>$self->{'config_hash'}->{'ipfw'}</tt>", undef ]));
	}

# Make sure ipfw works
$self->{'rules'} = $self->get_config();
$self->{'active'} = $self->get_config("$self->{'config_hash'}->{'ipfw'} show |", \$self->{'out'});
if ($?) {
	$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", undef, 1, 1);
	$self->ui_print_endpage(
		$self->ui_config_link('index_elist',
			[ "<tt>$self->{'config_hash'}->{'ipfw'} list</tt>",
			  "<pre>$self->{'out'}</pre>", undef ]));
	}

# Get the version number
$self->{'vout'} = `$self->{'config_hash'}->{'ipfw'} 2>&1`;
$self->{'vouth'} = `$self->{'config_hash'}->{'ipfw'} -h 2>&1`;
if ($self->{'vout'} =~ /preproc/ || $self->{'vouth'} =~ /preproc/) {
	$self->{'ipfw_version'} = 2;
	}
else {
	$self->{'ipfw_version'} = 1;
	}
open(VERSION, ">$self->{'module_config_directory'}/version");
print VERSION $self->{'ipfw_version'},"\n";
close(VERSION);
$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", undef, 1, 1, 0,
		 $self->help_search_link("ipfw", "man", "doc", "google"),
		 undef, undef,
		 $self->text('index_version', $self->{'ipfw_version'}));

# Check for an active firewall that is not managed by this module
if (!@{$self->{'rules'}} && @{$self->{'active'}} > 1) {
	# Yes .. offer to convert
	print $self->text('index_existing', scalar(@{$self->{'active'}}),
		    "<tt>$self->{'ipfw_file'}</tt>"),"<p>\n";
	print $self->ui_form_start("convert.cgi");
	print "<center>",$self->ui_submit($self->{'text_hash'}->{'index_saveex'}),"<p>\n";
	print "</center>\n";
	print $self->ui_form_end();

	print "<table border width=100%>\n";
	print "<tr $self->{'tb'}><td><b>$self->{'text_hash'}->{'index_headerex'}</b></td></tr>\n";
	print "<tr $self->{'cb'}> <td><pre>";
	print $self->{'out'};
	print "</pre></td> </tr></table>\n";
	}
elsif (@{$self->{'rules'}} && !$self->{'in_hash'}->{'reset'}) {
	# Find last editable rule
	if ($self->{'rules'}->[@{$self->{'rules'}}-1]->{'num'} == 65535 &&
	    @{$self->{'rules'}} > 1) {
		$self->{'lastidx'} = $self->{'rules'}->[@{$self->{'rules'}}-2]->{'index'};
		}
	else {
		$self->{'lastidx'} = $self->{'rules'}->[@{$self->{'rules'}}-1]->{'index'};
		}

	# Build map of active rules
	local %amap = map { int($_->{'num'}), $_ } @{$self->{'active'}};

	# Show the rules
	print $self->ui_form_start("edit_rule.cgi");
	local @widths = ( "width=10", "width=5%", undef );
	push(@widths, undef) if ($self->{'config_hash'}->{'view_condition'});
	push(@widths, undef) if ($self->{'config_hash'}->{'view_comment'});
	push(@widths, "width=5%", "width=5% nowrap") if ($self->{'config_hash'}->{'view_counters'});
	push(@widths, "width=5%", "width=5%");
	@{$self->{'links_array'}} = ( $self->select_all_link("d", 0),
		   $self->select_invert_link("d", 0) );
	print $self->ui_links_row(\@{$self->{'links_array'}});
	print $self->ui_columns_start([ "",
				  $self->{'text_hash'}->{'index_num'},
				  $self->{'text_hash'}->{'index_action'},
			    	  $self->{'config_hash'}->{'view_condition'} ?
					( $self->{'text_hash'}->{'index_desc'} ) : ( ),
			    	  $self->{'config_hash'}->{'view_comment'} ?
					( $self->{'text_hash'}->{'index_cmt'} ) : ( ),
			    	  $self->{'config_hash'}->{'view_counters'} ?
					( $self->{'text_hash'}->{'index_count1'},
					  $self->{'text_hash'}->{'index_count2'} ) : ( ),
			    	  $self->{'text_hash'}->{'index_move'},
			    	  $self->{'text_hash'}->{'index_radd'} ], 100, 0,
				\@widths);
	foreach $r (@{$self->{'rules'}}) {
		local ($mover, $adder);
		if ($r->{'index'} >= $self->{'lastidx'}) {
			$mover .= "<img src=images/gap.gif>";
			}
		else {
			$mover .= "<a href='move.cgi?idx=$r->{'index'}&".
			      	  "down=1'><img src=".
			      	  "images/down.gif border=0></a>";
			}
		if ($r->{'index'} == 0 || $r->{'index'} > $self->{'lastidx'}) {
			$mover .= "<img src=images/gap.gif>";
			}
		else {
			$mover .= "<a href='move.cgi?idx=$r->{'index'}&".
			          "up=1'><img src=images/up.gif ".
			          "border=0></a>";
			}
		if ($r->{'index'} <= $self->{'lastidx'}) {
			$adder .= "<a href='edit_rule.cgi?new=1&".
				  "after=$r->{'index'}'>".
				  "<img src=images/after.gif border=0></a>";
			$adder .= "<a href='edit_rule.cgi?new=1&".
				  "before=$r->{'index'}'>".
				  "<img src=images/before.gif border=0></a>";
			}

		local ($ls, $le);
		if ($r->{'index'} <= $self->{'lastidx'}) {
			$ls = "<a href='edit_rule.cgi?idx=$r->{'index'}'>";
			$le = "</a>";
			}
		local $act = $amap{int($r->{'num'})};
		print $self->ui_checked_columns_row(
			[ $ls.$r->{'num'}.$le,
			  $ls.($self->{'text_hash'}->{'action_'.$self->real_action($r->{'action'})} ||
			   uc($r->{'action'})).
			  (defined($r->{'aarg'}) ? " $r->{'aarg'}" : "").$le,
			  $self->{'config_hash'}->{'view_condition'} ? ( $self->describe_rule($r) ) : ( ),
			  $self->{'config_hash'}->{'view_comment'} ? ( $r->{'cmt'} || "<br>" ) : ( ),
			  $self->{'config_hash'}->{'view_counters'} ? ( $act->{'count1'}, $self->nice_size($act->{'count2'}) ) : ( ),
			  $mover,
			  $adder ],
			\@widths, "d", $r->{'num'});
		}
	print $self->ui_columns_end();
	print $self->ui_links_row(\@{$self->{'links_array'}});

	# Show delete and add buttons
	print "<table width=100%><tr>\n";
	print "<td align=left>",
	      $self->ui_submit($self->{'text_hash'}->{'index_delete'}, "delsel"),"</td>\n";
	print "<td align=right>",
	      $self->ui_submit($self->{'text_hash'}->{'index_add2'}, "new"),"</td>\n";
	print "</tr></table>\n";
	print $self->ui_form_end();

	# Show buttons to apply configuration and start at boot
	print "<hr>\n";

	$self->foreign_require("init", "init-lib.pl");
	$self->{'atboot'} = $main::init_object->action_status($self->{'module_name'});
	print $self->ui_buttons_start();
	if ($self->foreign_check("servers")) {
		@{$self->{'servers_array'}} = $self->list_cluster_servers();
		}
	print $self->ui_buttons_row("apply.cgi", $self->{'text_hash'}->{'index_apply'},
			      @{$self->{'servers_array'}} ? $self->{'text_hash'}->{'index_applydesc2'}
				       : $self->{'text_hash'}->{'index_applydesc'});
	print $self->ui_buttons_row("unapply.cgi", $self->{'text_hash'}->{'index_unapply'},
			      $self->{'text_hash'}->{'index_unapplydesc'});
	print $self->ui_buttons_row("bootup.cgi", $self->{'text_hash'}->{'index_boot'},
			      $self->{'text_hash'}->{'index_bootdesc'}, undef,
			      $self->ui_radio("boot", $self->{'atboot'} == 2 ? 1 : 0,
					[ [ 1, $self->{'text_hash'}->{'yes'} ],
					  [ 0, $self->{'text_hash'}->{'no'} ] ]));
	print $self->ui_buttons_row("index.cgi", $self->{'text_hash'}->{'index_reset'},
			      $self->{'text_hash'}->{'index_resetdesc'}, undef,
			      $self->ui_hidden("reset", 1));
	# Show button for cluster page
	if ($self->foreign_check("servers")) {
		$self->foreign_require("servers", "servers-lib.pl");
		@{$self->{'allservers_array'}} = grep { $_->{'user'} }
				$main::servers_object->list_servers();
		}
	if (@{$self->{'allservers_array'}}) {
		print $self->ui_buttons_row("cluster.cgi", $self->{'text_hash'}->{'index_cluster'},
				      $self->{'text_hash'}->{'index_clusterdesc'});
		}
	print $self->ui_buttons_end();
	}
else {
	# Offer to setup simple firewall
	print $self->text($self->{'in_hash'}->{'reset'} ? 'index_rsetup' : 'index_setup',
		    "<tt>$self->{'ipfw_file'}</tt>"),"<p>\n";
	print "<form action=setup.cgi>\n";
	print $self->ui_hidden("reset", $self->{'in_hash'}->{'reset'});
	print "<center><table><tr><td>\n";
	print "<input type=radio name=auto value=0 checked> ",
	      "$self->{'text_hash'}->{'index_auto0'}<p>\n";
	foreach $a (2 .. 4) {
		print "<input type=radio name=auto value=$a> ",
		      "$self->{'text_hash'}->{'index_auto'.$a} ",
		      $self->interface_choice("iface".$a, undef, 1),"<p>\n";
		}
	print "</td></tr></table>\n";
	print "<input type=submit value='$self->{'text_hash'}->{'index_auto'}'><p>\n";
	print "<input type=checkbox name=atboot value=1> ",
	      "$self->{'text_hash'}->{'index_atboot'}\n";
	print "</center></form>\n";
	}

$self->ui_print_footer("/", $self->{'text_hash'}->{'index'});

1;
		
}


1;

