#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Create, update or delete a firewall rule

&ModuleLoader::preload_module('ipfw_class', "ipfw::IpfwLib");
$self = new ipfw::IpfwLib:: 'ipfw_class',%$self;

$self->ReadParse();
$self->error_setup($self->{'text_hash'}->{'save_err'});

$self->{'rules'} = $self->get_config();
if ($self->{'in_hash'}->{'new'}) {
	# Find the last editable rule
	if ($self->{'rules'}->[@{$self->{'rules'}}-1]->{'num'} == 65535 &&
	    @{$self->{'rules'}} > 1) {
		$self->{'lastidx'} = $self->{'rules'}->[@{$self->{'rules'}}-2]->{'index'};
		}
	else {
		$self->{'lastidx'} = $self->{'rules'}->[@{$self->{'rules'}}-1]->{'index'};
		}

	# Work out where to insert, and what number to use
	if ($self->{'in_hash'}->{'before'} ne '') {
		# Adding before some rule
		local $pn = $self->{'in_hash'}->{'before'} == 0 ? 0 :
			    $self->{'rules'}->[$self->{'in_hash'}->{'before'}-1]->{'num'};
		$self->{'rule'} = { 'num' => ($self->{'rules'}->[$self->{'in_hash'}->{'before'}]->{'num'}+$pn)/2 };
		splice(@{$self->{'rules'}}, $self->{'in_hash'}->{'before'}, 0, $self->{'rule'});
		}
	elsif ($self->{'in_hash'}->{'after'} ne '') {
		# Adding after some rule
		local $nn = $self->{'in_hash'}->{'after'} == $self->{'lastidx'} ?
				$self->{'rules'}->[$self->{'in_hash'}->{'after'}]->{'num'}+200 :
				$self->{'rules'}->[$self->{'in_hash'}->{'after'}+1]->{'num'};
		$self->{'rule'} = { 'num' => ($self->{'rules'}->[$self->{'in_hash'}->{'after'}]->{'num'}+$nn)/2 };
		splice(@{$self->{'rules'}}, $self->{'in_hash'}->{'after'}+1, 0, $self->{'rule'});
		}
	elsif (!$self->{'in_hash'}->{'num_def'}) {
		# At specified number
		$self->{'in_hash'}->{'num'} =~ /^\d+$/ && $self->{'in_hash'}->{'num'} >= 0 && $self->{'in_hash'}->{'num'} < 65536
			|| $self->error($self->{'text_hash'}->{'save_enum'});
		$self->{'rule'} = { 'num' => $self->{'in_hash'}->{'num'} };
		local $found = 0;
		for(local $i=0; $i<@{$self->{'rules'}}; $i++) {
			if ($self->{'rules'}->[$i]->{'num'} >= $self->{'in_hash'}->{'num'}) {
				splice(@{$self->{'rules'}}, $i, 0, $self->{'rule'});
				$found++;
				last;
				}
			}
		push(@{$self->{'rules'}}, $self->{'rule'}) if (!$found);
		}
	elsif (!@{$self->{'rules'}}) {
		# First rule
		$self->{'rule'} = { 'num' => '00100' };
		push(@{$self->{'rules'}}, $self->{'rule'});
		}
	else {
		# At end or before last deny-all rule
		$self->{'rule'} = { 'num' => $self->{'rules'}->[$self->{'lastidx'}]->{'num'}+100 };
		splice(@{$self->{'rules'}}, $self->{'lastidx'}+1, 0, $self->{'rule'});
		}
	$self->{'rule'}->{'num'} = sprintf "%5.5d", $self->{'rule'}->{'num'};
	}
else {
	$self->{'rule'} = $self->{'rules'}->[$self->{'in_hash'}->{'idx'}];
	delete($self->{'rule'}->{'text'});
	}

if ($self->{'in_hash'}->{'delete'}) {
	# Just remove this rule
	splice(@{$self->{'rules'}}, $self->{'in_hash'}->{'idx'}, 1);
	}
else {
	# Validate inputs and contruct the rule object
	$self->{'in_hash'}->{'cmt'} =~ s/\r//g;
	$self->{'rule'}->{'cmt'} = $self->{'in_hash'}->{'cmt'};

	# Parse rule action and arg
	$self->{'rule'}->{'action'} = $self->{'in_hash'}->{'action'};
	if ($self->{'in_hash'}->{'action'} eq "skipto") {
		$self->{'in_hash'}->{'action_skipto'} =~ /^\d+$/ ||
			$self->error($self->{'text_hash'}->{'save_eskipto'});
		$self->{'rule'}->{'aarg'} = $self->{'in_hash'}->{'action_skipto'};
		}
	elsif ($self->{'in_hash'}->{'action'} eq "fwd") {
		$self->check_ipaddress($self->{'in_hash'}->{'action_fwdip'}) ||
			$self->error($self->{'text_hash'}->{'save_efwdip'});
		if ($self->{'in_hash'}->{'action_fwdport'} eq "") {
			$self->{'rule'}->{'aarg'} = $self->{'in_hash'}->{'action_fwdip'};
			}
		else {
			$self->{'in_hash'}->{'action_fwdport'} =~ /^\d+$/ ||
				$self->error($self->{'text_hash'}->{'save_efwdport'});
			$self->{'rule'}->{'aarg'} = $self->{'in_hash'}->{'action_fwdip'}.",".
					  $self->{'in_hash'}->{'action_fwdport'};
			}
		}
	elsif ($self->{'in_hash'}->{'action'} eq "divert" || $self->{'in_hash'}->{'action'} eq "pipe" ||
	       $self->{'in_hash'}->{'action'} eq "queue" || $self->{'in_hash'}->{'action'} eq "tee") {
		$self->{'in_hash'}->{'action_port'} =~ /^\d+$/ ||
			$self->error($self->{'text_hash'}->{'save_eteeport'});
		$self->{'rule'}->{'aarg'} = $self->{'in_hash'}->{'action_port'};
		}
	elsif ($self->{'in_hash'}->{'action'} eq "unreach") {
		$self->{'rule'}->{'aarg'} = $self->{'in_hash'}->{'action_unreach'};
		}
	else {
		delete($self->{'rule'}->{'aarg'});
		}

	# Parse protocol
	if ($self->{'in_hash'}->{'proto_orblock'}) {
		$self->{'rule'}->{'proto'} = &parse_orblock("proto");
		}
	else {
		$self->{'rule'}->{'proto'} = $self->{'in_hash'}->{'proto'};
		}

	# Parse in/out option
	delete($self->{'rule'}->{'in'});
	delete($self->{'rule'}->{'out'});
	delete($self->{'rule'}->{'in_not'});
	delete($self->{'rule'}->{'out_not'});
	if ($self->{'in_hash'}->{'inout'} == 1) {
		$self->{'rule'}->{'in'} = 1;
		}
	elsif ($self->{'in_hash'}->{'inout'} == 2) {
		$self->{'rule'}->{'out'} = 1;
		}

	# Parse via interface
	$self->{'rule'}->{'via'} = &parse_interface("via");

	# Parse logging level
	if ($self->{'in_hash'}->{'log'}) {
		$self->{'rule'}->{'log'} = 1;
		if ($self->{'in_hash'}->{'logamount'} ne "") {
			$self->{'in_hash'}->{'logamount'} =~ /^\d+$/ ||
				$self->error($self->{'text_hash'}->{'save_elogamount'});
			$self->{'rule'}->{'logamount'} = $self->{'in_hash'}->{'logamount'};
			}
		else {
			delete($self->{'rule'}->{'logamount'});
			}
		}
	else {
		$self->{'rule'}->{'log'} = 0;
		}

	# Parse source and destination
	foreach $s ("from", "to") {
		# IP address
		if ($self->{'in_hash'}->{$s."_orblock"}) {
			$self->{'rule'}->{$s} = &parse_orblock($s);
			}
		elsif ($self->{'in_hash'}->{$s."_mode"} == 0) {
			$self->{'rule'}->{$s} = "any";
			}
		elsif ($self->{'in_hash'}->{$s."_mode"} == 1) {
			$self->{'rule'}->{$s} = "me";
			}
		else {
			gethostbyname($self->{'in_hash'}->{$s}) ||
			    $self->check_ipaddress($self->{'in_hash'}->{$s}) ||
			    ($self->{'in_hash'}->{$s} =~ /^([0-9\.]+)\/(\d+)$/ &&
			     $self->check_ipaddress($1)) ||
			    ($self->{'in_hash'}->{$s} =~ /^([0-9\.]+)\/(\d+)\{([0-9,]+)\}$/ &&
			     $self->check_ipaddress($1) &&
			     $self->{'ipfw_version'} >= 2) ||
				$self->error($self->{'text_hash'}->{'save_e'.$s});
			$self->{'rule'}->{$s} = $self->{'in_hash'}->{$s};
			}

		# Port numbers
		if ($self->{'in_hash'}->{$s."_ports_orblock"}) {
			# XXX could be optional?
			$self->{'rule'}->{$s."_ports"} = &parse_orblock($s."_ports");
			}
		elsif ($self->{'in_hash'}->{$s."_ports_mode"} == 0) {
			delete($self->{'rule'}->{$s."_ports"});
			}
		else {
			local $p = $self->{'rule'}->{'proto'};
			$p eq "tcp" || $p eq "udp" || $p eq "ip" ||
			    $self->{'ipfw_version'} >= 2 ||
				$self->error($self->{'text_hash'}->{'save_eportsproto'.$s});
			$self->{'in_hash'}->{$s."_ports"} =~ /^\d+$/ ||
			  getservbyname($self->{'in_hash'}->{$s."_ports"}, $p) ||
			  $self->{'in_hash'}->{$s."_ports"} =~ /^\d+\-\d+$/ ||
			  ($self->{'in_hash'}->{$s."_ports"} =~ /^([a-z0-9]+)\-([a-z0-9]+)$/i &&
			   getservbyname($1, $p) && getservbyname($2, $p)) ||
			  $self->{'in_hash'}->{$s."_ports"} =~ /^([a-z0-9]+)(,[a-z0-9]+)*$/ ||
			  ($self->{'in_hash'}->{$s."_ports"} =~ /^([a-z0-9]+|([a-z0-9]+)\-([a-z0-9]+))(,[a-z0-9]+|,([a-z0-9]+)\-([a-z0-9]+))*$/ &&
			   $self->{'ipfw_version'} >= 2) ||
				$self->error($self->{'text_hash'}->{'save_eports'.$s});
			$self->{'rule'}->{$s."_ports"} = $self->{'in_hash'}->{$s."_ports"};
			$self->{'rule'}->{$s."_ports_not"} = $self->{'in_hash'}->{$s."_ports_not"}
				if ($self->{'ipfw_version'} >= 2);
			}
		}
	$self->{'rule'}->{'xmit'} = &parse_interface("xmit");
	$self->{'rule'}->{'recv'} = &parse_interface("recv");

	# XXX multiple options

	# Parse various options
	&parse_yes_no_ignored("established");
	&parse_yes_no_ignored("keep-state");
	&parse_yes_no_ignored("bridged");
	&parse_yes_no_ignored("frag");
	&parse_yes_no_ignored("setup");

	# Parse MAC address
	if ($self->{'ipfw_version'} >= 2) {
		if ($self->{'in_hash'}->{'mac1_def'} && $self->{'in_hash'}->{'mac2_def'}) {
			delete($self->{'rule'}->{'mac'});
			}
		else {
			local @mac;
			if ($self->{'in_hash'}->{'mac2_def'}) {
				push(@mac, "any");
				}
			else {
				$self->{'in_hash'}->{'mac2'} =~ /^[0-9a-f]{2}(:[0-9a-f]{2}){5}(\/\d+)?$/ || $self->error($self->{'text_hash'}->{'save_emac2'});
				push(@mac, $self->{'in_hash'}->{'mac2'});
				}
			if ($self->{'in_hash'}->{'mac1_def'}) {
				push(@mac, "any");
				}
			else {
				$self->{'in_hash'}->{'mac1'} =~ /^[0-9a-f]{2}(:[0-9a-f]{2}){5}(\/\d+)?$/ || $self->error($self->{'text_hash'}->{'save_emac1'});
				push(@mac, $self->{'in_hash'}->{'mac1'});
				}
			$self->{'rule'}->{'mac'} = \@mac;
			}
		}

	# Parse UID and GID
	if ($self->{'in_hash'}->{'uid_def'}) {
		delete($self->{'rule'}->{'uid'});
		}
	elsif ($self->{'in_hash'}->{'uid'} =~ /^#(\d+)$/) {
		$self->{'rule'}->{'uid'} = $1;
		}
	else {
		defined($self->{'rule'}->{'uid'} = getpwnam($self->{'in_hash'}->{'uid'})) ||
			$self->error($self->{'text_hash'}->{'save_euid'});
		}
	if ($self->{'in_hash'}->{'gid_def'}) {
		delete($self->{'rule'}->{'gid'});
		}
	elsif ($self->{'in_hash'}->{'gid'} =~ /^#(\d+)$/) {
		$self->{'rule'}->{'gid'} = $1;
		}
	else {
		defined($self->{'rule'}->{'gid'} = getgrnam($self->{'in_hash'}->{'gid'})) ||
			$self->error($self->{'text_hash'}->{'save_egid'});
		}

	# Parse ICMP types
	if ($self->{'in_hash'}->{'icmptypes'}) {
		$self->{'rule'}->{'proto'} eq 'icmp' || $self->error($self->{'text_hash'}->{'save_eicmptypes'});
		$self->{'rule'}->{'icmptypes'} = join(",", split(/\0/, $self->{'in_hash'}->{'icmptypes'}));
		}
	else {
		delete($self->{'rule'}->{'icmptypes'});
		}

	# Parse tcp flags
	if ($self->{'in_hash'}->{'tcpflags'}) {
		$self->{'rule'}->{'proto'} eq 'tcp' || $self->error($self->{'text_hash'}->{'save_etcpflags'});
		$self->{'rule'}->{'tcpflags'} = join(",", split(/\0/, $self->{'in_hash'}->{'tcpflags'}));
		}
	else {
		delete($self->{'rule'}->{'tcpflags'});
		}

	# Parse limit directive
	if ($self->{'in_hash'}->{'limit'}) {
		$self->{'in_hash'}->{'limit2'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'save_elimit'});
		$self->{'rule'}->{'limit'} = [ $self->{'in_hash'}->{'limit'}, $self->{'in_hash'}->{'limit2'} ];
		}
	else {
		delete($self->{'rule'}->{'limit'});
		}

	# Parse dst-port and src-port directive
	foreach $ds ('dst', 'src') {
		if (!$self->{'in_hash'}->{$ds.'port_def'}) {
			local @dstports = split(/[ ,]+/, $self->{'in_hash'}->{$ds.'port'});
			foreach $p (@dstports) {
				$self->valid_port($p, $self->{'rule'}->{'proto'}) ||
					$self->error($self->{'text_hash'}->{'save_e'.$ds.'port'});
				}
			$self->{'rule'}->{$ds.'-port'} = \@dstports;
			}
		else {
			delete($self->{'rule'}->{$ds.'-port'});
			}
		}
	}

# Save all rules
$self->lock_file($self->{'ipfw_file'});
$self->save_config($self->{'rules'});
$self->unlock_file($self->{'ipfw_file'});
$self->copy_to_cluster();
$self->webmin_log($self->{'in_hash'}->{'delete'} ? "delete" : $self->{'in_hash'}->{'new'} ? "create" : "modify",
	    "rule", $self->{'rule'}->{'action'}, $self->{'rule'});
$self->redirect("");

1;
		
}

# parse_interface(name)
sub parse_interface
{
	shift if ($self == $_[0]);

local $iface = $self->{'in_hash'}->{$_[0]} eq "other" ? $self->{'in_hash'}->{$_[0]."_other"} : $self->{'in_hash'}->{$_[0]};
return undef if (!$iface);
$iface =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'save_e'.$_[0]});
return $iface;
}

# parse_orblock(name)
sub parse_orblock
{
	shift if ($self == $_[0]);

$self->{'in_hash'}->{$_[0]} =~ /\S/ || $self->error($self->text('save_eorblock'.$_[0])); 
return [ split(/\s+/, $self->{'in_hash'}->{$_[0]}) ];
}

# parse_yes_no_ignored(name)
sub parse_yes_no_ignored
{
	shift if ($self == $_[0]);

if ($self->{'in_hash'}->{$_[0]} == 0) {
	delete($self->{'rule'}->{$_[0]});
	}
elsif ($self->{'in_hash'}->{$_[0]} == 1) {
	$self->{'rule'}->{$_[0]} = 1;
	$self->{'rule'}->{$_[0]."_not"} = 0;
	}
elsif ($self->{'in_hash'}->{$_[0]} == 2) {
	$self->{'rule'}->{$_[0]} = 1;
	$self->{'rule'}->{$_[0]."_not"} = 1;
	}
}


1;

