#!/usr/bin/perl
package ipsec::IpsecLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'ipsec::IpsecLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# ipsec-lib.pl
# Common functions for managing the freeswan config file
# XXX check for new errors in log after applying
# XXX option to download connection as .conf file, and upload an existing
#     .conf file for addition

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

@{$self->{'rsa_attribs_array'}} = ( "Modulus", "PublicExponent", "PrivateExponent",
		 "Prime1", "Prime2", "Exponent1", "Exponent2", "Coefficient" );

1;

1;
		return $self;
}

# $self->get_config([file])
# Returns an array of configured connections
sub get_config
{
	my $self = shift;

local $file = $_[0] || $self->{'config_hash'}->{'file'};
local (@rv, $sect);
local $lnum = 0;
local $fh = "CONF".$self->{'get_config_fh'}++;
open($fh, $file);
while(<$fh>) {
	s/\r|\n//g;
	s/#.*$//;
	if (/^\s*([^= ]+)\s*=\s*"([^"]*)"/ ||
	    /^\s*([^= ]+)\s*=\s*'([^"]*)'/ ||
	    /^\s*([^= ]+)\s*=\s*(\S+)/) {
		# Directive within a section
		if ($sect) {
			if ($sect->{'values'}->{lc($1)}) {
				$sect->{'values'}->{lc($1)} .= "\0".$2;
				}
			else {
				$sect->{'values'}->{lc($1)} = $2;
				}
			$sect->{'eline'} = $lnum;
			}
		}
	elsif (/^\s*include\s+(\S+)/) {
		# Including possibly multiple files
		local $inc = $1;
		if ($inc !~ /^\//) {
			$file =~ /^(.*)\//;
			$inc = "$1/$inc";
			}
		local $g;
		foreach $g (glob($inc)) {
			local @inc = $self->get_config($g);
			map { $_->{'index'} += scalar(@rv) } @inc;
			push(@rv, @inc);
			}
		}
	elsif (/^\s*(\S+)\s+(\S+)/) {
		# Start of a section
		$sect = { 'name' => $1,
			  'value' => $2,
			  'line' => $lnum,
			  'eline' => $lnum,
			  'file' => $file,
			  'index' => scalar(@rv),
			  'values' => { } };
		push(@rv, $sect);
		}
	$lnum++;
	}
close($fh);
return @rv;
}

# create_conn(&conn)
# Add a new connection to the config file
sub create_conn
{
	my $self = shift;

local $lref = $self->read_file_lines($_[0]->{'file'} || $self->{'config_hash'}->{'file'});
push(@$lref, "", $self->conn_lines($_[0]));
$self->flush_file_lines();
}

# modify_conn(&conn)
sub modify_conn
{
	my $self = shift;

local $lref = $self->read_file_lines($_[0]->{'file'});
splice(@$lref, $_[0]->{'line'}, $_[0]->{'eline'} - $_[0]->{'line'} + 1,
       $self->conn_lines($_[0]));
$self->flush_file_lines();
}

# delete_conn(&conn)
sub delete_conn
{
	my $self = shift;

local $lref = $self->read_file_lines($_[0]->{'file'});
splice(@$lref, $_[0]->{'line'}, $_[0]->{'eline'} - $_[0]->{'line'} + 1);
$self->flush_file_lines();
}

# swap_conns(&conn1, &conn2)
# Swaps two connections in the config file
sub swap_conns
{
	my $self = shift;

local ($first, $second) = @_;
if ($first->{'line'} > $second->{'line'}) {
	($first, $second) = ($second, $first);
	}
local $lref1 = $self->read_file_lines($first->{'file'});
local $lref2 = $self->read_file_lines($second->{'file'});
splice(@$lref2, $second->{'line'}, $second->{'eline'} - $second->{'line'} + 1,
       @$lref1[$first->{'line'} .. $first->{'eline'}]);
splice(@$lref2, $first->{'line'}, $first->{'eline'} - $first->{'line'} + 1,
       @$lref1[$second->{'line'} .. $second->{'eline'}]);
$self->flush_file_lines();
}

# conn_lines(&conn)
sub conn_lines
{
	my $self = shift;

local @rv;
push(@rv, $_[0]->{'name'}." ".$_[0]->{'value'});
foreach $o (sort { $a cmp $b } keys %{$_[0]->{'values'}}) {
	local $v = $_[0]->{'values'}->{$o};
	local $vv;
	foreach $vv (split(/\0/, $v)) {
		if ($vv =~ /\s|=/) {
			push(@rv, "\t".$o."=\"".$vv."\"");
			}
		else {
			push(@rv, "\t".$o."=".$vv);
			}
		}
	}
return @rv;
}

# is_ipsec_running()
sub is_ipsec_running
{
	my $self = shift;

local $out = `$self->{'config_hash'}->{'ipsec'} auto --status 2>&1`;
return $? || $out =~ /not running/i ? 0 : 1;
}

# get_public_key()
# Returns this system's public key
sub get_public_key
{
	my $self = shift;

local $out = `$self->{'config_hash'}->{'ipsec'} showhostkey --file '$self->{'config_hash'}->{'secrets'}' --left 2>&1`;
if ($out =~ /leftrsasigkey=(\S+)/) {
	return $1;
	}
return undef;
}

# get_public_key_dns()
# Returns the flags, protocol, algorithm and key data for the public key,
# suitable for creating a DNS KEY record
sub get_public_key_dns
{
	my $self = shift;

local $out = `$self->{'config_hash'}->{'ipsec'} showhostkey --file '$self->{'config_hash'}->{'secrets'}' 2>&1`;
if ($out =~ /KEY\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/) {
	return ($1, $2, $3, $4);
	}
else {
	# Try with new --key argument
	$out = `$self->{'config_hash'}->{'ipsec'} showhostkey --key --file '$self->{'config_hash'}->{'secrets'}' 2>&1`;
	if ($out =~ /KEY\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/) {
		return ($1, $2, $3, $4);
		}
	}
return ();
}

# list_policies()
# Returns a list of all policy files
sub list_policies
{
	my $self = shift;

local ($f, @rv);
opendir(DIR, $self->{'config_hash'}->{'policies_dir'});
while($f = readdir(DIR)) {
	push(@rv, $f) if ($f !~ /^\./ && $f !~ /\.rpmsave$/);
	}
closedir(DIR);
return @rv;
}

# read_policy(name)
sub read_policy
{
	my $self = shift;

local @rv;
open(FILE, "$self->{'config_hash'}->{'policies_dir'}/$_[0]");
while(<FILE>) {
	push(@rv, "$1/$2") if (/^\s*([0-9\.]+)\/(\d+)/);
	}
close(FILE);
return @rv;
}

# write_policy(name, &nets)
sub write_policy
{
	my $self = shift;

local $lref = $self->read_file_lines("$self->{'config_hash'}->{'policies_dir'}/$_[0]");
local $l = 0;
foreach $p (@{$_[1]}) {
	while($l < @$lref && $lref->[$l] !~ /^\s*([0-9\.]+)\/(\d+)/) {
		$l++;
		}
	if ($l < @$lref) {
		# Found line to replace
		$lref->[$l] = $p;
		}
	else {
		# Add at end
		push(@$lref, $p);
		}
	$l++;
	}
while($l < @$lref) {
	if ($lref->[$l] =~ /^\s*([0-9\.]+)\/(\d+)/) {
		splice(@$lref, $l, 1);
		}
	else { $l++; }
	}
$self->flush_file_lines();
}

# wrap_lines(text, width)
# Given a multi-line string, return an array of lines wrapped to
# the given width
sub wrap_lines
{
	my $self = shift;

local $rest = $_[0];
local @rv;
while(length($rest) > $_[1]) {
	push(@rv, substr($rest, 0, $_[1]));
	$rest = substr($rest, $_[1]);
	}
push(@rv, $rest) if ($rest ne '');
return @rv;
}

# before_start()
# Work out which log file IPsec messages go to, and record the size
sub before_start
{
	my $self = shift;

@{$self->{'ipsec_logfiles_array'}} = ( $self->{'config_hash'}->{'logfile'} );
if ($self->foreign_check("syslog")) {
	# Find all syslog logfiles
	$self->foreign_require("syslog", "syslog-lib.pl");
	local $conf = $main::syslog_object->get_config();
	foreach $c (@$conf) {
		push(@{$self->{'ipsec_logfiles_array'}}, $c->{'file'}) if ($c->{'file'} &&
							-f $c->{'file'});
		}
	}
@{$self->{'ipsec_logfiles_array'}} = $self->unique(@{$self->{'ipsec_logfiles_array'}});
@{$self->{'ipsec_logfile_sizes_array'}} = map { local @st = stat($_); $st[7] } @{$self->{'ipsec_logfiles_array'}};
}

# after_start()
# Check any new IPsec-related messages in the log for errors
sub after_start
{
	my $self = shift;

# Give the server a chance to start
sleep(5);

# Look for new error messages
local $i;
for($i=0; $i<@{$self->{'ipsec_logfiles_array'}}; $i++) {
	open(LOG, $self->{'ipsec_logfiles_array'}->[$i]) || next;
	seek(LOG, $self->{'ipsec_logfile_sizes_array'}->[$i], 0);
	while(<LOG>) {
		s/\r|\n//g;
		if (/ipsec/i && /error/i) {
			s/^(\S+)\s+(\d+)\s+(\d+:\d+:\d+)\s+(\S+)\s+//;
			push(@{$self->{'errs_array'}}, $_);
			}
		}
	close(LOG);
	}

# Fail if there were any
if (@{$self->{'errs_array'}}) {
	$self->error($self->text('start_elog', "<p><tt>".join("<br>", @{$self->{'errs_array'}})."</tt><br>"));
	}
}

# get_ipsec_version(&out)
sub get_ipsec_version
{
	my $self = shift;

local $out = `$self->{'config_hash'}->{'ipsec'} --version 2>&1`;
${$_[0]} = $out;
return $out =~ /(FreeS\/WAN|Openswan|StrongSWAN)\s+([^ \n\(]+)/i ? ($2,$1) : (undef);
}

# got_secret()
# Returns 1 if a valid secret key file exists, 0 if not
sub got_secret
{
	my $self = shift;

local $gotkey;
open(SEC, $self->{'config_hash'}->{'secrets'}) || return 0;
while(<SEC>) {
	s/\r|\n//g;
	s/#.*$//;
	if (/Modulus:\s*(\S+)/) {
		$gotkey = 1;
		}
	}
close(SEC);
return $gotkey;
}

# expand_conf(&config)
sub expand_conf
{
	my $self = shift;

my $conf = shift;
for my $n (0..scalar(@$conf)-1) {
	$conn = @$conf[$n];
	foreach my $key (keys(%{$conn->{'values'}})) {
		$expanded{$conn->{'value'}}->{$key} = $conn->{'values'}->{$key};
		}
	}
# now go through and expand alsos
foreach my $k (keys(%expanded)) {
	$conn = \%{$expanded{$k}};
	# XXX - only supporing a single level of redirection
	#     - this should be moved into a function that could be called
	#       recursively
	if ($self->{$conn."_hash"}->{'also'}) {
		foreach my $also (split(/\000/, $self->{$conn."_hash"}->{'also'})) {
			foreach my $i (keys(%{$expanded{$also}})) {
				$self->{$conn."_hash"}->{$i} = $expanded{$also}{$i};
				}
			}
		# there is only one also key
		next;
		}
	}
return %expanded;
}

# restart_ipsec()
# Apply the current configuration, and return an error message on failure or
# undef on success
sub restart_ipsec
{
	my $self = shift;

local $cmd = $self->{'config_hash'}->{'restart_cmd'} ||
       "($self->{'config_hash'}->{'stop_cmd'} && $self->{'config_hash'}->{'start_cmd'})";
$self->before_start();
local $out = $self->backquote_logged("$cmd 2>&1");
if ($?) {
	return "<pre>$out</pre>";
	}
$self->after_start();
return undef;
}

# list_secrets()
# Returns a list of IPsec secret keys
sub list_secrets
{
	my $self = shift;

if (!defined(@{$self->{'list_secrets_cache_array'}})) {
	local (@lines);
	local $lnum = 0;
	open(SEC, $self->{'config_hash'}->{'secrets'});
	while(<SEC>) {
		s/\r|\n//g;
		s/^\s*#.*$//;
		if (/^(\S.*)$/) {
			push(@lines, { 'value' => $1,
				       'line' => $lnum,
				       'eline' => $lnum });
			}
		elsif (/^\s+(.*)/ && @lines) {
			$lines[$#{$self->{'rv_array'}}]->{'value'} .= "\n".$1;
			$lines[$#{$self->{'rv_array'}}]->{'eline'} = $lnum;
			}
		$lnum++;
		}
	close(SEC);

	# Turn joined lines into secrets
	local $l;
	foreach $l (@lines) {
		$l->{'value'} =~ /^([^:]*)\s*:\s+(\S+)\s+((.|\n)*)$/ || next;
		local $sec = { 'type' => $2,
			       'name' => $1,
			       'value' => $3,
			       'line' => $l->{'line'},
			       'eline' => $l->{'eline'},
			       'idx' => scalar(@{$self->{'list_secrets_cache_array'}}),
			      };
		$sec->{'name'} =~ s/\n/ /g;
		$sec->{'name'} =~ s/\s+$//;
		push(@{$self->{'list_secrets_cache_array'}}, $sec);
		}
	}
return @{$self->{'list_secrets_cache_array'}};
}

# delete_secret(&sec)
# Removes one secret from the file
sub delete_secret
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'secrets'});
local $lines = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
splice(@$lref, $_[0]->{'line'}, $lines);
$self->flush_file_lines();
local $s;
splice(@{$self->{'list_secrets_cache_array'}}, $_[0]->{'idx'}, 1);
foreach $s (@{$self->{'list_secrets_cache_array'}}) {
	if ($s->{'line'} > $_[0]->{'line'}) {
		$s->{'line'} -= $lines;
		$s->{'eline'} -= $lines;
		}
	if ($s->{'idx'} > $_[0]->{'idx'}) {
		$s->{'idx'}--;
		}
	}
}

# create_secret(&sec)
# Add one secret to the file
sub create_secret
{
	my $self = shift;

$self->list_secrets();	# force cache init
local $lref = $self->read_file_lines($self->{'config_hash'}->{'secrets'});
$_[0]->{'line'} = scalar(@$lref);
local @lines = $self->secret_lines($_[0]);
push(@$lref, @lines);
$self->flush_file_lines();
$_[0]->{'eline'} = scalar(@$lref)-1;
$_[0]->{'idx'} = scalar(@{$self->{'list_secrets_cache_array'}});
push(@{$self->{'list_secrets_cache_array'}}, $_[0]);
}

# modify_secret(&sec)
# Update one secret in the file
sub modify_secret
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'secrets'});
local @newlines = $self->secret_lines($_[0]);
local $oldlines = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
splice(@$lref, $_[0]->{'line'}, $oldlines, @newlines);
$self->flush_file_lines();
local $s;
foreach $s (@{$self->{'list_secrets_cache_array'}}) {
	if ($s ne $_[0] && $s->{'line'} > $_[0]->{'line'}) {
		$s->{'line'} += @newlines - $oldlines;
		$s->{'eline'} += @newlines - $oldlines;
		}
	}
$_[0]->{'eline'} += @newlines - $oldlines;
}

sub secret_lines
{
	my $self = shift;

local $str = $_[0]->{'name'} ? $_[0]->{'name'}." : " : ": ";
$str .= uc($_[0]->{'type'});
$str .= " ".$_[0]->{'value'};
return split(/\n/, $str);
}


1;

