#!/usr/bin/perl
package jabber::JabberLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'jabber::JabberLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# jabber-lib.pl
# Common functions for editing the jabber config files
#
# XXX - http://prdownloads.sourceforge.net/expat/expat-1.95.2-1.i686.rpm
#     - XML::Parser  XML::Generator
# XXX - admin <read> and <write> - what do they mean?
use locale;
&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

if ($self->{'config_hash'}->{'jabber_lib'}) {
	$ENV{$self->{'gconfig_hash'}->{'ld_env'}} .= ':' if ($ENV{$self->{'gconfig_hash'}->{'ld_env'}});
	$ENV{$self->{'gconfig_hash'}->{'ld_env'}} .= $self->{'config_hash'}->{'jabber_lib'};
	}

eval "use XML::Parser; \$got_xml_parser++";
eval "use XML::Generator; \$got_xml_generator++";

@{$self->{'register_fields_array'}} = ( 'name', 'email' );

@{$self->{'karma_presets_array'}} = ( { 'heartbeat' => 2,	'init' => 10,
		     'max' => 10,	'inc' => 1,
		     'dec' => 1,	'penalty' => -6,
		     'restore' => 10 },
		   { 'heartbeat' => 2,	'init' => 50,
		     'max' => 50,	'inc' => 4,
		     'dec' => 1,	'penalty' => -5,
		     'restore' => 50 },
		   { 'heartbeat' => 2,	'init' => 64,
		     'max' => 64,	'inc' => 6,
		     'dec' => 1,	'penalty' => -3,
		     'restore' => 64 }
		  );

@{$self->{'filter_conds_array'}} = ( "ns", "unavailable", "from", "resource", "subject", "body",
		  "show", "type", "roster", "group" );

@{$self->{'filter_acts_array'}} =  ( "error", "offline", "forward", "reply", "continue",
		  "settype" );

1;

1;
		return $self;
}

# get_jabber_config()
# Parse the jabber XML config file
sub get_jabber_config
{
	my $self = shift;

return $self->{'get_jabber_config_cache'} if (defined($self->{'get_jabber_config_cache'}));
local $xml = new XML::Parser('Style' => 'Tree');
eval { $self->{'get_jabber_config_cache'} = $xml->parsefile($self->{'config_hash'}->{'jabber_config'}); };
if ($@) {
	return $@;
	}
return $self->{'get_jabber_config_cache'};
}

# find(name, &config)
sub find
{
	my $self = shift;

local (@rv, $i);
local $list = $_[1]->[1];
for($i=1; $i<@$list; $i+=2) {
	if (lc($list->[$i]) eq lc($_[0])) {
		push(@rv, [ $list->[$i], $list->[$i+1], $i ]);
		}
	}
return wantarray ? @rv : $rv[0];
}

# find_value(name, &config)
sub find_value
{
	my $self = shift;

local @rv = map { $self->value_in($_) } $self->find($_[0], $_[1]);
return wantarray ? @rv : $rv[0];
}

# jabber_pid_file()
# Returns the PID file used by jabber
sub jabber_pid_file
{
	my $self = shift;

local $conf = $self->get_jabber_config();
local $pidfile = $self->find_value("pidfile", $conf);
if ($pidfile =~ /^\//) {
	return $pidfile;
	}
elsif ($pidfile) {
	return "$self->{'config_hash'}->{'jabber_dir'}/$pidfile";
	}
else {
	return "$self->{'config_hash'}->{'jabber_dir'}/jabber.pid";
	}
}

# value_in(&tag)
sub value_in
{
	my $self = shift;

return undef if (!$_[0]);
local $zero = $self->find("0", $_[0]);
return $zero ? $zero->[1] : undef;
}

# $self->generate_config(&tree, &gen)
# Returns an XML::Generator object created from a config tree structure
sub generate_config
{
	my $self = shift;

local $gen = $_[1] ? $_[1] : XML::Generator->new(escape => 'always');
local $list = $_[0]->[1];
local (@mems, $i);
for($i=1; $i<@$list-1; $i+=2) {
	if ($list->[$i] eq '0') {
		push(@mems, $list->[$i+1]);
		}
	else {
		push(@mems, $self->generate_config(
				[ $list->[$i], $list->[$i+1] ], $gen));
		}
	}
local $name = $_[0]->[0];
return $gen->$name($list->[0], @mems);
}

# save_jabber_config(&config)
sub save_jabber_config
{
	my $self = shift;

$self->open_tempfile(CONFIG, ">$self->{'config_hash'}->{'jabber_config'}");
local $xml = $self->generate_config($_[0]);
$self->print_tempfile(CONFIG, $xml);
$self->print_tempfile(CONFIG, "\n");
$self->close_tempfile(CONFIG);
}

# save_directive(&config, name|&old, &new)
# Replaces all directives of some name with new values
sub save_directive
{
	my $self = shift;

local @ov = ref($_[1]) ? @{$_[1]} : $self->find($_[1], $_[0]);
local @nv = @{$_[2]};
local ($i, $j);
for($i=0; $i<@ov || $i<@nv; $i++) {
	local $idx = $ov[$i]->[2] if ($ov[$i]);
	if ($ov[$i] && $nv[$i]) {
		# Updating an existing value
		$_[0]->[1]->[$idx] = $nv[$i]->[0];
		$_[0]->[1]->[$idx+1] = $nv[$i]->[1];
		}
	elsif ($ov[$i]) {
		# Deleting an old value
		splice(@{$_[0]->[1]}, $idx, 2);
		map { $_->[2] -= 2 if ($_->[2] >= $idx) } @ov;
		}
	else {
		# Adding a new value after the last non-text one
		local $nt = -1;
		for($j=1; $j<@{$_[0]->[1]}; $j+=2) {
			$nt = $j if ($_[0]->[1]->[$j] ne '0');
			}
		splice(@{$_[0]->[1]}, $nt+2, 0, $nv[$i]->[0], $nv[$i]->[1]);
		}
	}
}

# find_by_tag(name, tag, value, &config)
sub find_by_tag
{
	my $self = shift;

local @m = $self->find($_[0], $_[3]);
@m = grep { lc($_->[1]->[0]->{lc($_[1])}) eq lc($_[2]) } @m;
return wantarray ? @m : $m[0];
}

# xml_string(name, &tree, ...)
# Converts a list of XML structures into text
sub xml_string
{
	my $self = shift;

local $rv = "";
for($self->{'i'}=0; $self->{'i'}<@_; $self->{'i'}+=2) {
	local $xml = $self->generate_config([ $_[$self->{'i'}], $_[$self->{'i'}+1] ]);
	if ($xml =~ /\S/) {
		$rv .= $xml."\n";
		}
	}
return $rv;
}

# get_jabberd_version(&out)
sub get_jabberd_version
{
	my $self = shift;

local $jabberd = $self->{'config_hash'}->{'jabber_daemon'} ? $self->{'config_hash'}->{'jabber_daemon'}
				    : "$self->{'config_hash'}->{'jabber_dir'}/bin/jabberd";
local $out = `$jabberd -v 2>&1`;
${$_[0]} = $out;
return $out =~ /\s(1\.4\S*)/ ? $1 : undef;
}

# stop_jabber()
# Stops jabber, and returns undef on success or an error message on failure
sub stop_jabber
{
	my $self = shift;

if ($self->{'config_hash'}->{'stop_cmd'}) {
        $self->system_logged("$self->{'config_hash'}->{'stop_cmd'} </dev/null >/dev/null 2>&1");
        }
else {
	local $pid = $self->check_pid_file($self->jabber_pid_file());
	if ($pid) {
		$self->kill_logged('TERM', $pid) || return $!;
		}
	else {
		return $self->{'text_hash'}->{'stop_epid'};
		}
        }
unlink($self->jabber_pid_file());
return undef;
}

# start_jabber()
# Starts jabber, and returns undef on success or an error message on failure
sub start_jabber
{
	my $self = shift;

$self->system_logged("$self->{'config_hash'}->{'start_cmd'} </dev/null >/tmp/err 2>&1");
return undef;
}


1;

