# jakarta-tomcat-lib.pl
# Common functions for jakarta-tomcat configuration

#main part
do '../web-lib.pl';
&init_config();
require '../ui-lib.pl';

eval "use XML::Parser;";
$got_xml_parser++ unless $@;

#get version of jakarta-tomcat
sub get_version_tomcat
{
 my $version=`JAVA_HOME=$config{java_home} $config{tomcat_cmd} version | grep -i 'version'`;
 $version =~ m/\/(.+)/;
 $version = $1;
 return $version;
}

#status jakarta-tomcat
#return 'start' if Tomcat stop & return 'stop' if Tomcat start
sub status_tomcat
{
 @pids = &find_byname("tomcat");
 if (@pids){return $status='Stop';}
 else {return $status='Start';}
}

#start jakarta-tomcat
sub start_tomcat
{
 my $status=&status_tomcat();
 if ($status eq 'Start')
 {
  my $out = &backquote_logged("JAVA_HOME=$config{java_home} $config{tomcat_cmd} start 2>&1 </dev/null");
  &error($out) if ($?);
  &webmin_log("start");
 }
 else {$text{'proces_status'};}
}

#stop jakarta-tomcat
sub stop_tomcat
{
 my $status=&status_tomcat();
 if ($status eq 'Stop')
 {
  while ($status eq 'Stop')
  {
   my $out = &backquote_logged("JAVA_HOME=$config{java_home} $config{tomcat_cmd} stop 2>&1 </dev/null");
   &error($out) if ($?);
   &webmin_log("stop");
   $status=&status_tomcat();
  }
 }
 else {$text{'proces_status2'};
}

#restart jakarta-tomcat
sub restart_tomcat
{
 my $status=&status_tomcat();
 if ($status eq 'Stop')
 {
  while ($status eq 'Stop')
  {
   my $out = &backquote_logged("JAVA_HOME=$config{java_home} $config{tomcat_cmd} stop 2>&1 </dev/null");
   &error($out) if ($?);
   &webmin_log("stop");
   $status=&status_tomcat();
  }
  my $out = &backquote_logged("JAVA_HOME=$config{java_home} $config{tomcat_cmd} start 2>&1 </dev/null");
  &error($out) if ($?);
  &webmin_log("restart");}
 }
}

#delete configuration cache
sub delete_cache
{
 if (-e "$config{'tomcat_conf_dir'}/$_[0]")
 {
  `rm -rf $config{'tomcat_conf_dir'}/$_[0]`;
 }
}

#get hosts configuration
sub get_index_conf
{
 my ($port,$hosts)=&read_conf_host();
 my (@paths,@links,@icons);
 foreach (@{$hosts})
 {
  $name=@{$_}[1]->{'name'};
  $base=@{$_}[1]->{'appBase'};
  push (@paths,$name);
  push (@links,"virt.cgi?name=$name");
  push(@icons, "images/virt.gif");
  }
 return ($port,$hosts,\@paths,\@links,\@icons);
}

#get host $_[0] configuration
sub get_virt_conf
{
 my ($port,$hosts)=&read_conf_host();
 my ($name,$base,$alias,@paths,@links,@icons);
 foreach (@{$hosts})
 {
  if (@{$_}[1]->{'name'} eq $_[0])
  {
   $name=@{$_}[1]->{'name'};
   $base=@{$_}[1]->{'appBase'};
   foreach $i (@{$_})
   {
    if (ref ($i) eq "ARRAY")
    {
     if (@{$i}[0] eq 'Alias')
     {
      $alias=@{$i}[2];
     }
     if (@{$i}[0] eq 'Context')
     {
      push (@paths,@{$i}[1]->{'docBase'});
      push (@links,"dir.cgi?name=$name&path=@{$i}[1]->{'path'}");
      push(@icons, "images/dir.gif");
     }
    }
   }
  }
 }
 return ($port,$hosts,$name,$base,$alias,\@paths,\@links,\@icons);
}

#get directory $_[1] configuration for server $_[0]
sub get_dir_conf
{
 ($port,$hosts)=&read_conf_host();
 my ($name,$dir,$path,$reload,$debug);
 foreach (@{$hosts})
 {
  if (@{$_}[1]->{'name'} eq $_[0])
  {
   $name=@{$_}[1]->{'name'};
   foreach $i (@{$_})
   {
    if (ref ($i) eq "ARRAY")
    {
     if (@{$i}[0] eq 'Context')
     {
      if (@{$i}[1]->{'path'} eq $_[1])
      {
       $dir=@{$i}[1]->{'docBase'};
       $path=@{$i}[1]->{'path'};
       $reload=@{$i}[1]->{'reloadable'};
       $debug=@{$i}[1]->{'debug'};
      }
     }
    }
   }
  }
 }
 return ($name,$dir,$path,$reload,$debug);
}

#parse server.xml
sub read_conf_host
{
 &error(&text('jabber_eparser', "<tt>XML::Parser</tt>")) unless $got_xml_parser;

 $level=0;
 my $parser=new XML::Parser(Handlers =>
 					{
					 Start=>\&handle_start,
					 End=>\&handle_end,
					 Char=>\&handle_char
 					});
 eval {$parser->parsefile($config{tomcat_conf})};
 return $@ if ($@);
 return ($expat->{'port'},$expat->{'hosts'});
}

sub handle_start
{
 my (undef,$name,%attr)=@_;
 $expat->{'elem'}=[$name,\%attr] if ($expat->{'level'}==1);
 if ($name eq 'Host')
 {
  $expat->{'host'}=[$name,\%attr];
  $expat->{'level'}=1;
 }
 $expat->{'port'}=$attr{'port'} if ($name eq 'Connector' && !$attr{'protocol'});
}

sub handle_char
{
 if ($expat->{'level'}==1)
 {
  $text=$_[1];
  push (@{$expat->{'elem'}},$text);
 }
}

sub handle_end
{
 my (undef,$name)=@_;
 if ($name eq 'Host')
 {
  push (@{$expat->{'hosts'}},$expat->{'host'});
  $expat->{'level'}=0;
 }
 else
 {
  push (@{$expat->{'host'}},$expat->{'elem'});
  $expat->{'elem'}=();
 }
}
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#add new host $_[0] into server.xml
sub add_new_host
{
 &error(&text('jakarta_eparser', "<tt>XML::Parser</tt>"))  unless $got_xml_parser;
 my $file=&tempname();
 open (OUT,">$file");
 my $parser=new XML::Parser(Handlers =>
 					{
					 Start=>\&add_host_start,
					 End=>\&add_host_end,
					 Char=>\&add_host_char,
					 Comment=>\&add_host_comment
 					});
 ($expat->{'name'},$expat->{'alias'},$expat->{'dir'})=@_;
 eval {
       $parser->parsefile($config{tomcat_conf});
       close (OUT);
       `mv -f $file $config{tomcat_conf}`;
      };
 return $@ if ($@);
}

sub add_host_start
{
 my (undef,$name,%attr)=@_;
 print OUT ("<$name");
 while (my ($key,$value)=each(%attr))
 {
  print OUT (" $key=\"$value\"");
 }
 print OUT (">");
 if ($name eq 'Engine')
 {
  print OUT ("<Host name=\"$expat->{'name'}\" appBase=\"$expat->{'dir'}\" xmlValidation=\"false\" unpackWARs=\"true\" xmlNamespaceAware=\"false\" autoDeploy=\"true\"  debug=\"0\">\n");
  print OUT ("<Alias>$expat->{'alias'}</Alias>\n");
  print OUT ("<Logger suffix=\".txt\" timestamp=\"true\" className=\"org.apache.catalina.logger.FileLogger\" prefix=\"$expat->{'name'}\.\" directory=\"logs\">\n</Logger>\n</Host>");
 }
}

sub add_host_char
{
 my (undef,$text)=@_;
 print OUT ("$text");
}

sub add_host_comment
{
 my (undef,$data)=@_;
 print OUT ("<!--$data-->");
}

sub add_host_end
{
 my (undef,$name)=@_;
 print OUT ("</$name>");
}
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#save host $_[0] configuration into server.xml
sub save_conf_virt
{
 &error(&text('jabber_eparser', "<tt>XML::Parser</tt>"))  unless $got_xml_parser;
 my $file=&tempname();
 open (OUT,">$file");
 my $parser=new XML::Parser(Handlers =>
 					{
					 Start=>\&write_virt_start,
					 End=>\&write_virt_end,
					 Char=>\&write_virt_char,
					 Comment=>\&write_virt_comment
 					});
 ($expat->{'name'},$expat->{'newname'},$expat->{'port'},$expat->{'alias'}, $expat->{'base'})=@_;
 eval {
       $parser->parsefile($config{tomcat_conf});
       close (OUT);
       `mv -f $file $config{tomcat_conf}`;
      };
 return $@ if ($@);
}

sub write_virt_start
{
 my (undef,$name,%attr)=@_;
 $attr{'port'}=$expat->{'port'} if ($name eq 'Connector' && !$attr{'protocol'});
 if ($name eq 'Host' && $attr{'name'} eq $expat->{'name'})
 {
  $attr{'name'}=$expat->{'newname'};
  $attr{'appBase'}=$expat->{'base'};
  $expat->{'virt'}=1;
 }
 $expat->{'als'}=1 if ($name eq 'Alias');
 print OUT ("<$name");
 while (my ($key,$value)=each(%attr))
 {
  print OUT (" $key=\"$value\"");
 }
 print OUT (">");
}

sub write_virt_char
{
 my (undef,$text)=@_;
 $text=$expat->{'alias'} if ($expat->{'virt'}==1 && $expat->{'als'}==1);
 print OUT ("$text");
}

sub write_virt_comment
{
 my (undef,$data)=@_;
 print OUT ("<!--$data-->");
}

sub write_virt_end
{
 my (undef,$name)=@_;
 $expat->{'virt'}=0 if ($name eq 'Host');
 $expat->{'als'}=0 if ($name eq 'Alias');
 print OUT ("</$name>");
}
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#delete host $_[0] configuration into server.xml
sub del_virt
{
 &error(&text('jabber_eparser', "<tt>XML::Parser</tt>"))  unless $got_xml_parser;
 my $file=&tempname();
 open (OUT,">$file");
 my $parser=new XML::Parser(Handlers =>
 					{
					 Start=>\&del_virt_start,
					 End=>\&del_virt_end,
					 Char=>\&del_virt_char,
					 Comment=>\&del_virt_comment
 					});
 $expat->{'name'}=$_[0];
 eval {
       $parser->parsefile($config{tomcat_conf});
       close (OUT);
       `mv -f $file $config{tomcat_conf}`;
      };
 return $@ if ($@);
}

sub del_virt_start
{
 my (undef,$name,%attr)=@_;
 unless ($name eq 'Host' && $attr{'name'} eq $expat->{'name'})
 {
  unless ($expat->{'del'}==1)
  {
   print OUT ("<$name");
   while (my ($key,$value)=each(%attr))
   {
    print OUT (" $key=\"$value\"");
   }
   print OUT (">");
  }
 }
 else {$expat->{'del'}=1;}
}

sub del_virt_char
{
 my (undef,$text)=@_;
 print OUT ("$text") unless ($expat->{'del'}==1);
}

sub del_virt_comment
{
 my (undef,$data)=@_;
 print OUT ("<!--$data-->") unless ($expat->{'del'}==1);
}

sub del_virt_end
{
 my (undef,$name)=@_;
 print OUT ("</$name>") unless ($expat->{'del'}==1);
 $expat->{'del'}=0 if ($name eq 'Host');
}
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#save context configuration into server.xml for host $_[0]
sub add_new_dir
{
 &error(&text('jabber_eparser', "<tt>XML::Parser</tt>"))  unless $got_xml_parser;
 my $file=&tempname();
 open (OUT,">$file");
 my $parser=new XML::Parser(Handlers =>
 					{
					 Start=>\&add_dir_start,
					 End=>\&add_dir_end,
					 Char=>\&add_dir_char,
					 Comment=>\&add_dir_comment
 					});
 ($expat->{'name'},$expat->{'dir'},$expat->{'url'},$expat->{'reload'}, $expat->{'debug'})=@_;
 eval {
       $parser->parsefile($config{tomcat_conf});
       close (OUT);
       `mv -f $file $config{tomcat_conf}`;
      };
 return $@ if ($@);
}

sub add_dir_start
{
 my (undef,$name,%attr)=@_;
 print OUT ("<$name");
 while (my ($key,$value)=each(%attr))
 {
  print OUT (" $key=\"$value\"");
 }
 print OUT (">");
 if ($name eq 'Host' && $attr{'name'} eq $expat->{'name'})
 {
  my $r;
  if ($expat->{'reload'} eq 'on') {$r='true';}
  else {$r='false';}
  print OUT ("<Context path=\"$expat->{'url'}\" docBase=\"$expat->{'dir'}\" reloadable=\"$r\" debug=\"$expat->{'debug'}\"></Context>");
 }
}

sub add_dir_char
{
 my (undef,$text)=@_;
 print OUT ("$text");
}

sub add_dir_comment
{
 my (undef,$data)=@_;
 print OUT ("<!--$data-->");
}

sub add_dir_end
{
 my (undef,$name)=@_;
 print OUT ("</$name>");
}
#++++++++++++++++++++++++++++++++++++++++++++++

#save context configuration into server.xml for host $_[0]
sub save_conf_dir
{
 &error(&text('jabber_eparser', "<tt>XML::Parser</tt>"))  unless $got_xml_parser;
 my $file=&tempname();
 open (OUT,">$file");
 my $parser=new XML::Parser(Handlers =>
 					{
					 Start=>\&write_start,
					 End=>\&write_end,
					 Char=>\&write_char,
					 Comment=>\&write_comment
 					});
 ($expat->{'name'},$expat->{'path'},$expat->{'dir'},$expat->{'url'},$expat->{'reload'}, $expat->{'debug'})=@_;
 eval {
       $parser->parsefile($config{tomcat_conf});
       close (OUT);
       `mv -f $file $config{tomcat_conf}`;
      };
 return $@ if ($@);
}

sub write_start
{
 my (undef,$name,%attr)=@_;
 $expat->{'virt'}=1 if ($name eq 'Host' && $attr{'name'} eq $expat->{'name'});
 if ($expat->{'virt'}==1 && $name eq 'Context' && $attr{'path'} eq $expat->{'path'})
 {
  $attr{'path'}=$expat->{'url'};
  $attr{'docBase'}=$expat->{'dir'};
  if ($expat->{'reload'} eq 'on') {$attr{'reloadable'}='true';}
  else {$attr{'reloadable'}='false';}
  $attr{'debug'}=$expat->{'debug'};
 }
 print OUT ("<$name");
 while (my ($key,$value)=each(%attr))
 {
  print OUT (" $key=\"$value\"");
 }
 print OUT (">");
}

sub write_char
{
 my (undef,$text)=@_;
 print OUT ("$text");
}

sub write_comment
{
 my (undef,$data)=@_;
 print OUT ("<!--$data-->");
}

sub write_end
{
 my (undef,$name)=@_;
 $expat->{'virt'}=0 if ($name eq 'Host');
 print OUT ("</$name>");
}
#++++++++++++++++++++++++++++++++++++++++++++++++++++

#delete context configuration into server.xml for host $_[0]
sub del_dir
{
 &error(&text('jabber_eparser', "<tt>XML::Parser</tt>"))  unless $got_xml_parser;
 my $file=&tempname();
 open (OUT,">$file");
 my $parser=new XML::Parser(Handlers =>
 					{
					 Start=>\&del_dir_start,
					 End=>\&del_dir_end,
					 Char=>\&del_dir_char,
					 Comment=>\&del_dir_comment
 					});
 ($expat->{'name'},$expat->{'path'})=@_;
 eval {
       $parser->parsefile($config{tomcat_conf});
       close (OUT);
       `mv -f $file $config{tomcat_conf}`;
      };
 return $@ if ($@);
}

sub del_dir_start
{
 my (undef,$name,%attr)=@_;
 $expat->{'virt'}=1 if ($name eq 'Host' && $attr{'name'} eq $expat->{'name'});
 unless ($expat->{'virt'}==1 && $name eq 'Context' && $attr{'path'} eq $expat->{'path'})
 {
  print OUT ("<$name");
  while (my ($key,$value)=each(%attr))
  {
   print OUT (" $key=\"$value\"");
  }
  print OUT (">");
 }
 else {$expat->{'del'}=1;}
}

sub del_dir_char
{
 my (undef,$text)=@_;
 print OUT ("$text") unless ($expat->{'del'}==1);
}

sub del_dir_comment
{
 my (undef,$data)=@_;
 print OUT ("<!--$data-->") unless ($expat->{'del'}==1);
}

sub del_dir_end
{
 my (undef,$name)=@_;
 print OUT ("</$name>") unless ($expat->{'del'}==1);
 $expat->{'del'}=0 if ($name eq 'Context');
 $expat->{'virt'}=0 if ($name eq 'Host');
}

1;
1;
