#!/usr/bin/perl
package ldap_client::LdapclientLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'ldap_client::LdapclientLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for parsing and updating the LDAP config file

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

@{$self->{'base_types_array'}} = ("passwd", "shadow", "group", "hosts", "networks", "netmasks",
	       "services", "protocols", "aliases", "netgroup");

1;

1;
		return $self;
}

# get_config()
# Parses the NSS LDAP config file into a list of names and values
sub get_config
{
	my $self = shift;

local $file = $_[0] || $self->{'config_hash'}->{'auth_ldap'};
if (!defined(@{$self->{'get_config_cache_array'}})) {
	local $lnum = 0;
	@{$self->{'get_config_cache_array'}} = ( );
	$self->open_readfile(*CONF, $file);
	while(<CONF>) {
		s/\r|\n//g;
                s/#.*$//;
		if (/^(#?)(\S+)\s*(.*)/) {
			push(@{$self->{'get_config_cache_array'}}, { 'name' => lc($2),
						  'value' => $3,
						  'enabled' => !$1,
						  'line' => $lnum,
						  'file' => $file });
			}
		$lnum++;
		}
	close(CONF);
	}
return \@{$self->{'get_config_cache_array'}};
}

# find(name, &conf, disabled-mode)
sub find
{
	my $self = shift;

local ($name, $conf, $dis) = @_;
local @rv = grep { $_->{'name'} eq $name } @$conf;
if ($dis == 0) {
	# Enabled only
	@rv = grep { $_->{'enabled'} } @rv;
	}
elsif ($dis == 1) {
	# Disabled only
	@rv = grep { !$_->{'enabled'} } @rv;
	}
return wantarray ? @rv : $rv[0];
}

# find_value(name, &conf)
sub find_value
{
	my $self = shift;

local ($name, $conf, $dis) = @_;
local @rv = map { $_->{'value'} } $self->find($name, $conf, $dis);
return wantarray ? @rv : $rv[0];
}

sub find_svalue
{
	my $self = shift;

local $rv = $self->find_value(@_);
return $rv;
}

# save_directive(&conf, name, [value])
sub save_directive
{
	my $self = shift;

local ($conf, $name, $value) = @_;
local $old = $self->find($name, $conf);
local $oldcmt = $self->find($name, $conf, 1);
local $lref = $self->read_file_lines($old ? $old->{'file'} :
			       $oldcmt ? $oldcmt->{'file'} :
				         $self->{'config_hash'}->{'auth_ldap'});
if (defined($value) && $old) {
	# Just update value
	$old->{'value'} = $value;
	$lref->[$old->{'line'}] = "$name $value";
	}
elsif (defined($value) && $oldcmt) {
	# Add value after commented version
	splice(@$lref, $oldcmt->{'line'}+1, 0, "$name $value");
	$self->renumber($conf, $oldcmt->{'line'}+1, $oldcmt->{'file'}, 1);
	push(@$conf, { 'name' => $name,
		       'value' => $value,
		       'enabled' => 1,
		       'line' => $oldcmt->{'line'}+1,
		       'file' => $oldcmt->{'file'} });
	}
elsif (!defined($value) && $old) {
	# Delete current value
	splice(@$lref, $old->{'line'}, 1);
	$self->renumber($conf, $old->{'line'}, $old->{'file'}, -1);
	@$conf = grep { $_ ne $old } @$conf;
	}
elsif ($value) {
	# Add value at end of file
	push(@$conf, { 'name' => $name,
		       'value' => $value,
		       'enabled' => 1,
		       'line' => scalar(@$lref),
		       'file' => $self->{'config_hash'}->{'auth_ldap'} });
	push(@$lref, "$name $value");
	}
}

sub renumber
{
	my $self = shift;

local ($conf, $line, $file, $offset) = @_;
foreach my $c (@$conf) {
	if ($c->{'line'} >= $line && $c->{'file'} eq $file) {
		$c->{'line'} += $offset;
		}
	}
}

# get_rootbinddn_secret()
# Returns the password used when the root user connects to the LDAP server
sub get_rootbinddn_secret
{
	my $self = shift;

$self->open_readfile(*SECRET, $self->{'config_hash'}->{'secret'}) || return undef;
local $secret = <SECRET>;
close(SECRET);
$secret =~ s/\r|\n//g;
return $secret;
}

# save_rootbinddn_secret(secret)
# Saves the password used when the root user connects to the LDAP server
sub save_rootbinddn_secret
{
	my $self = shift;

if (defined($_[0])) {
	$self->open_tempfile(SECRET, ">$self->{'config_hash'}->{'secret'}");
	$self->print_tempfile(SECRET, $_[0],"\n");
	$self->close_tempfile(SECRET);
	$self->set_ownership_permissions(0, 0, 0600, $self->{'config_hash'}->{'secret'});
	}
else {
	$self->unlink_file($self->{'config_hash'}->{'secret'});
	}
}

# ldap_connect(return-error, [&host])
# Connect to the LDAP server and return a handle to the Net::LDAP object
sub ldap_connect
{
	my $self = shift;

# Load the LDAP module
eval "use Net::LDAP";
if ($@) {
	local $err = $self->text('ldap_emodule', "<tt>Net::LDAP</tt>",
		   "../cpan/download.cgi?source=3&".
		   "cpan=Convert::ASN1%20Net::LDAP&mode=2&".
		   "return=../$self->{'module_name'}/&".
		   "returndesc=".$self->urlize($self->{'module_info_hash'}->{'desc'}));
	if ($_[0]) { return $err; }
	else { $self->error($err); }
	}

# Get the host and port
local $conf = $self->get_config();
local $uri = $self->find_svalue("uri", $conf);
local ($ldap, $use_ssl, $err);
if ($self->{'config_hash'}->{'ldap_hosts'}) {
	# Using hosts from module config
	local @hosts = split(/\s+/, $self->{'config_hash'}->{'ldap_hosts'});
	$use_ssl = $self->{'config_hash'}->{'ldap_tls'} eq '' ?
			$self->find_svalue("ssl", $conf) eq "start_tls" :
			$self->{'config_hash'}->{'ldap_tls'};
	local $port = $self->{'config_hash'}->{'ldap_port'} ||
		      $self->find_svalue("port", $conf) ||
		      ($use_ssl ? 636 : 389);
	foreach $host (@hosts) {
		$ldap = Net::LDAP->new($host, port => $port);
		${$_[1]} = $host if ($_[1]);
		if (!$ldap) {
			$err = $self->text('ldap_econn',
				     "<tt>$host</tt>", "<tt>$port</tt>");
			}
		else {
			$err = undef;
			last;
			}
		}
	}
elsif ($uri) {
	# Using uri directive
	foreach $u (split(/\s+/, $uri)) {
		if ($u =~ /^(ldap|ldaps):\/\/([a-z0-9\_\-\.]+)(:(\d+))?/) {
			($self->{'proto'}, $host, $port) = ($1, $2, $4);
			if (!$port && $self->{'proto'} eq "ldap") {
				$port = 389;
				}
			elsif (!$port && $self->{'proto'} eq "ldaps") {
				$port = 636;
				}
			$ldap = Net::LDAP->new($host, port => $port);
			${$_[1]} = $host if ($_[1]);
			if (!$ldap) {
				$err = $self->text('ldap_econn',
					     "<tt>$host</tt>","<tt>$port</tt>");
				}
			else {
				$err = undef;
				$use_ssl = $self->{'proto'} eq "ldaps" ? 1 : 0;
				last;
				}
			}
		}
	}
else {
	# Using host and port directives
	if ($self->find_svalue("ssl", $conf) eq "start_tls") {
		$use_ssl = 1;
		}
	local @hosts = split(/[ ,]+/, $self->find_svalue("host", $conf));
	local $port = $self->find_svalue("port", $conf) ||
		      $use_ssl ? 636 : 389;
	@hosts = ( "localhost" ) if (!@hosts);

	foreach $host (@hosts) {
		$ldap = Net::LDAP->new($host, port => $port);
		${$_[1]} = $host if ($_[1]);
		if (!$ldap) {
			$err = $self->text('ldap_econn',
				     "<tt>$host</tt>", "<tt>$port</tt>");
			}
		else {
			$err = undef;
			last;
			}
		}
	}
if ($err) {
	if ($_[0]) { return $err; }
	else { $self->error($err); }
	}

# Start TLS if configured
if ($use_ssl) {
	$ldap->start_tls;
	}

local ($dn, $password);
local $rootbinddn = $self->find_svalue("rootbinddn", $conf);
if ($self->{'config_hash'}->{'ldap_user'}) {
	# Use login from config
	$dn = $self->{'config_hash'}->{'ldap_user'};
	$password = $self->{'config_hash'}->{'ldap_pass'};
	}
elsif ($rootbinddn) {
	# Use the root login if we have one
	$dn = $rootbinddn;
	$password = $self->get_rootbinddn_secret();
	}
else {
	# Use the normal login
	$dn = $self->find_svalue("binddn", $conf);
	$password = $self->find_svalue("bindpw", $conf);
	}
local $mesg;
if ($dn) {
	$mesg = $ldap->bind(dn => $dn, password => $password);
	}
else {
	$mesg = $ldap->bind(anonymous => 1);
	}
if (!$mesg || $mesg->code) {
	local $err = $self->text('ldap_elogin', "<tt>$host</tt>",
		     	   $dn || $self->{'text_hash'}->{'ldap_anon'},
			   $mesg ? $mesg->error : "Unknown error");
	if ($_[0]) { return $err; }
	else { $self->error($err); }
	}
return $ldap;
}

# base_chooser_button(field, node, form)
# Returns HTML for a popup LDAP base chooser button
sub base_chooser_button
{
	my $self = shift;

local ($field, $node, $form) = @_;
$form ||= 0;
local $w = 500;
local $h = 500;
if ($self->{'gconfig_hash'}->{'db_sizeusers'}) {
	($w, $h) = split(/x/, $self->{'gconfig_hash'}->{'db_sizeusers'});
	}
return "<input type=button onClick='ifield = document.forms[$form].$field; chooser = window.open(\"popup_browser.cgi?node=$node&base=\"+escape(ifield.value), \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=$w,height=$h\"); chooser.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
}


1;

