#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Save the LDAP server to connect to

&ModuleLoader::preload_module('ldap_client_class', "ldap_client::LdapclientLib");
$self = new ldap_client::LdapclientLib:: 'ldap_client_class',%$self;

$self->error_setup($self->{'text_hash'}->{'server_err'});
$self->ReadParse();

$self->lock_file($self->{'config_hash'}->{'auth_ldap'});
$self->lock_file($self->{'config_hash'}->{'secret'});
$self->{'conf'} = $self->get_config();
$self->{'uri'} = $self->find_svalue("uri", $self->{'conf'});

# Validate and save inputs
if ($self->{'uri'}) {
	# Save uri directive
	for($i=0; defined($self->{'host'} = $self->{'in_hash'}->{'uhost_'.$i}); $i++) {
		next if (!$self->{'host'});
		$self->{'port'} = $self->{'in_hash'}->{'uport_'.$i.'_def'} ? undef : $self->{'in_hash'}->{'uport_'.$i};
		$self->{'proto'} = $self->{'in_hash'}->{'uproto_'.$i};
		!defined($self->{'port'}) ||
		    $self->{'port'} =~ /^\d+$/ && $self->{'port'} > 0 && $self->{'port'} < 65536 ||
		    $self->error($self->text('server_euport', $self->{'host'}));
		$self->{'uri'} = $self->{'proto'}."://".$self->{'host'}.($self->{'port'} ? ":$self->{'port'}" : "");
		push(@{$self->{'uris_array'}}, $self->{'uri'});
		}
	@{$self->{'uris_array'}} || $self->error($self->{'text_hash'}->{'server_euri'});
	$self->save_directive($self->{'conf'}, "uri", join(" ", @{$self->{'uris_array'}}));
	}
else {
	# Set host and port directives
	@{$self->{'hosts_array'}} = split(/\s+/, $self->{'in_hash'}->{'host'});
	foreach $h (@{$self->{'hosts_array'}}) {
		gethostbyname($h) || $self->error($self->text('server_ehost', $h));
		}
	@{$self->{'hosts_array'}} || $self->error($self->{'text_hash'}->{'server_ehosts'});
	$self->save_directive($self->{'conf'}, "host", join(" ", @{$self->{'hosts_array'}}));

	# Save server port
	if ($self->{'in_hash'}->{'port_def'}) {
		$self->save_directive($self->{'conf'}, "port", undef);
		}
	else {
		$self->{'in_hash'}->{'port'} =~ /^\d+$/ &&
		    $self->{'in_hash'}->{'port'} > 0 && $self->{'in_hash'}->{'port'} < 65536 ||
			$self->error($self->{'text_hash'}->{'server_eport'});
		$self->save_directive($self->{'conf'}, "port", $self->{'in_hash'}->{'port'});
		}
	}

# Save LDAP protocol version
$self->save_directive($self->{'conf'}, "ldap_version", $self->{'in_hash'}->{'version'} || undef);

# Save time limit
if ($self->{'in_hash'}->{'timelimit_def'}) {
	$self->save_directive($self->{'conf'}, "bind_timelimit", undef);
	}
else {
	$self->{'in_hash'}->{'timelimit'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'server_etimelimit'});
	$self->save_directive($self->{'conf'}, "bind_timelimit", $self->{'in_hash'}->{'timelimit'});
	}

# Save non-root login
if ($self->{'in_hash'}->{'binddn_def'}) {
	$self->save_directive($self->{'conf'}, "binddn", undef);
	}
else {
	$self->{'in_hash'}->{'binddn'} =~ /\S/ || $self->error($self->{'text_hash'}->{'server_ebinddn'});
	$self->save_directive($self->{'conf'}, "binddn", $self->{'in_hash'}->{'binddn'});
	}

# Save non-root password
if ($self->{'in_hash'}->{'bindpw_def'}) {
	$self->save_directive($self->{'conf'}, "bindpw", undef);
	}
else {
	$self->{'in_hash'}->{'bindpw'} =~ /\S/ || $self->error($self->{'text_hash'}->{'server_ebindpw'});
	$self->save_directive($self->{'conf'}, "bindpw", $self->{'in_hash'}->{'bindpw'});
	}

# Save root login
if ($self->{'in_hash'}->{'rootbinddn_def'}) {
	$self->save_directive($self->{'conf'}, "rootbinddn", undef);
	}
else {
	$self->{'in_hash'}->{'rootbinddn'} =~ /\S/ || $self->error($self->{'text_hash'}->{'server_erootbinddn'});
	$self->save_directive($self->{'conf'}, "rootbinddn", $self->{'in_hash'}->{'rootbinddn'});
	}

# Save root password
if ($self->{'in_hash'}->{'rootbindpw_def'}) {
	$self->save_rootbinddn_secret(undef);
	}
else {
	$self->{'in_hash'}->{'rootbindpw'} =~ /\S/ || $self->error($self->{'text_hash'}->{'server_erootbindpw'});
	$self->save_rootbinddn_secret($self->{'in_hash'}->{'rootbindpw'});
	}

if (!$self->{'uri'}) {
	# SSL mode
	$self->save_directive($self->{'conf'}, "ssl", $self->{'in_hash'}->{'ssl'} || undef);
	}

# Check server SSL cert
$self->save_directive($self->{'conf'}, "tls_checkpeer", $self->{'in_hash'}->{'peer'} || undef);

# CA cert file for server
if ($self->{'in_hash'}->{'cacert_def'}) {
	$self->save_directive($self->{'conf'}, "tls_cacertfile", undef);
	}
else {
	$self->{'in_hash'}->{'cacert'} =~ /^\// && -r $self->{'in_hash'}->{'cacert'} && !-d $self->{'in_hash'}->{'cacert'} ||
		$self->error($self->{'text_hash'}->{'server_ecacert'});
	$self->save_directive($self->{'conf'}, "tls_cacertfile", $self->{'in_hash'}->{'cacert'});
	}

# Write out config
$self->flush_file_lines();
$self->unlock_file($self->{'config_hash'}->{'auth_ldap'});
$self->unlock_file($self->{'config_hash'}->{'secret'});

$self->webmin_log("server");
$self->redirect("");

1;
		
}


1;

