#include <string>

namespace CTLicense {
  /**
   * Reload all license certificates from the disk store. This funcion
   * is called internally at library load time, so it's not necessary
   * to call it unless you suspect that the contents of the certificate
   * may have changed.
   */
  extern void reloadLicenses(const std::string& dirLic);

  /**
   * Returns the 'system key' for this host. The system key is a slightly
   * modified MAC address string.
   */
//  extern std::string& getSystemKey();

  /**
   * Check if there is a valid license for the named module
   */
  extern bool checkModuleLicense(const std::string& moduleName);

  /**
   * Get the expiredate for the named module. Returns (time_t)-1 if the
   * named module is not found (to be valid)
   */
  extern time_t getModuleBeginDate(const std::string& moduleName);
  extern time_t getModuleExpireDate(const std::string& moduleName);

  /**
   * Get the expiredate for the named module. Returns the string "No valid license"
   * if the named module is not found (to be valid)
   */
  extern std::string getModuleBeginDateString(const std::string& moduleName);
  extern std::string getModuleExpireDateString(const std::string& moduleName);

  /*
   * NOTE: Before using the numeric term lookup function below, you must 
   * verify that there is a valid license for the module itself.
   */

  extern std::string getModuleLicenseDescription(const std::string& moduleName);

//   extern std::string checkModuleString(const std::string& moduleName, const std::string& termName); 
  extern std::string getModuleStringTerm(const std::string& moduleName, const std::string& termName); 
  
  /**
   * Check the state of the named boolean term for the named module. Returns
   * false if there's no valid license for the module, or if the term is not 
   * found in any of the valid licenses.
   */
  extern bool checkModuleBoolTerm(const std::string& moduleName, const std::string& termName); 

  /**
   * Returns the value of the named numeric term for the named module. Returns
   * -1 if term doesn't exist, or isn't numeric, or if there's no valid license
   * for the module.
   */
  extern int getModuleNumericTerm(const std::string& moduleName, const std::string& termName);

  /* 
   * Utility functions
   */

//  extern std::string getIPaddress();   // Returns the IP of eth0
//  extern std::string getMACaddress();  // Returns the MAC of eth0 with all ':' changed to '-'
  extern void dumpLicense(int i);
}
