
#define VERBOSE_TEST 1
#define USE_TEST_XML_FILE 1
#define DUMP_XML_STREAM 1

#include "license.hpp"
#include "licensetag.hpp"
/*
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include <string>
#include <vector>
#include <stack>
#include <map>
#include <time.h>
*/
#include <expat.h>

namespace CTLicense {

////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class LicenseTag
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  void LicenseTag::dump(char *ind) {
    
    printf("%s%s",ind,name.c_str());
    if (tags.size() == 0) {
      struct tm t;
      
      if ((strcmp("NotAfter",name.c_str()) == 0) ||
	  (strcmp("NotBefore",name.c_str()) == 0) ||
	  (strcmp("IssueDate",name.c_str()) == 0)) {
	strptime(data.c_str(),"%Y-%m-%dT%H:%M:%SZ",&t);
	time_t sec = mktime(&t);
	time_t now = time(NULL);
	printf("%s", (sec < now) ? "<" : ">");
	printf(" [%04d-%02d-%02d %02d:%02d:%02d]",t.tm_year+1900,t.tm_mon+1,t.tm_mday,t.tm_hour,t.tm_min,t.tm_sec);
      }
      printf(" (%s)\n",data.c_str());
    }
    else { 
      printf("\n");
      char buf[80]; sprintf(buf,"  %s",ind);
      for (int i = 0; i < tags.size(); i++) tags[i]->dump(buf);
    }
  }

  static time_t blockIssueDate;

  void LicenseTag::registerLicenses(std::vector<License *> &lics)
  {
    //printf("Register %s?\n", name.c_str());
 
    if (name == "IssueDate") {
      struct tm tim;
      strptime(data.c_str(),"%Y-%m-%dT%H:%M:%SZ",&tim);
      // IssueDate is only in the preamble of a Licenses block, so doing
      // this will work.
      blockIssueDate = mktime(&tim);
    }
    if (name == "Licence") {
      License *newLic = new License(this);
      // BlockIssueDate will always be the right at this point
      newLic->issuedate = blockIssueDate;
      bool inserted = false;
      for (int i = 0; i < lics.size(); i++) {
	if (lics[i]->prodID == newLic->prodID) {
	  if (lics[i]->issuedate < newLic->issuedate) {
	    delete lics[i];
	    lics[i] = newLic;
	  }
	  else
	    delete newLic;
	  inserted = true;
	  break;
	}
      }
      if (!inserted) lics.push_back(newLic);
    }
    else for (int i = 0; i < tags.size(); i++) tags[i]->registerLicenses(lics);
  }
  
  LicenseTag::~LicenseTag() {
    for (int i = 0; i < tags.size(); i++) delete tags[i];
  }
  

  //static char eth0Address[24];
  //static char macAddress[24];


////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class License
////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  License::License(LicenseTag *lic) {
    issuedate = 0;
    notbefore = 0;
    notafter = 0x7FFFFFFF;
	 
  for (int i = 0; i < lic->tags.size(); i++) {
	if (lic->tags[i]->name == "ProductID") {
		prodID = lic->tags[i]->data;
		continue;
	}
	if (lic->tags[i]->name == "ProductName") {
		prodName = lic->tags[i]->data;
		continue;
	}
	if (lic->tags[i]->name == "Description") {
		prodDescription = lic->tags[i]->data;
		continue;
	}
      // This isn't likely ever to happen, but...
      if (lic->tags[i]->name == "IssueDate") {
	struct tm tim;
	strptime(lic->tags[i]->data.c_str(),"%Y-%m-%dT%H:%M:%SZ",&tim);
	issuedate = mktime(&tim);
	continue;
      }
      if (lic->tags[i]->name == "Terms") {
	for (int t = 0; t < lic->tags[i]->tags.size(); t++) {
	  if (lic->tags[i]->tags[t]->name == "NotAfter" || lic->tags[i]->tags[t]->name == "NotBefore") {
	    struct tm tim;
	    strptime(lic->tags[i]->tags[t]->data.c_str(),"%Y-%m-%dT%H:%M:%SZ",&tim);
	    time_t sec = mktime(&tim);
	    if (lic->tags[i]->tags[t]->name == "NotAfter") notafter = sec;
	    else                                          notbefore = sec;
	    continue;
	  }
	  if (lic->tags[i]->tags[t]->name == "Host") {
	    hostAddr = lic->tags[i]->tags[t]->data;
	    continue;
	  }
	  if (lic->tags[i]->tags[t]->name == "MACAddress") {
	    hostMAC = lic->tags[i]->tags[t]->data;
	    continue;
	  }
	  terms[lic->tags[i]->tags[t]->name] = lic->tags[i]->tags[t]->data;
	}
      }
    }
  }
  
  const std::string License::getProduct() { return prodID; }
  
  bool License::checkTime() {
    time_t now = time(NULL);
    return (now >= notbefore) && ( now <= notafter);
  }
  bool License::checkHost(const std::string &compareTo) { 
    if (hostAddr.size() == 0) return true;
    return hostAddr == compareTo;
  }
  bool License::checkMAC(const std::string &compareTo) { 
    if (hostMAC.size() == 0) return true;
    //printf("%s %s\n", hostMAC.c_str(), compareTo.c_str());
    return hostMAC == compareTo;
  }
  
  bool License::validForID(const std::string &module) {
    const char *me  = prodID.c_str();
    const char *him = module.c_str();
    return (strncmp(me,him,strlen(me)) && ((him[strlen(me)] == 0) || (him[strlen(me)]) == '/'));
  }
  
  std::string License::getStringTerm(const std::string &term) {
    if (terms.find(term) != terms.end())
      return std::string(terms[term]);
  }
  
  bool License::getBoolTerm(const std::string &term) {
    if (terms.find(term) != terms.end())
      return terms[term] == "Yes" || terms[term] == "On" || terms[term] == "1";
    else return false;
  }

  int License::getNumericTerm(const std::string &term) {
    if (terms.find(term) != terms.end()) {
      char *end;
      int val = strtol(terms[term].c_str(),&end,10);
      return (*end == 0) ? val : -1;
    }
    else return -1;
  }
  
  std::string License::getDescription() {
        return prodDescription;
  }
  
  time_t License::getBeginDate() {
    return notbefore;
  }
  
  time_t License::getExpireDate() {
    return notafter;
  }

  std::string License::getBeginDateString() {
        return std::string(ctime(&notbefore));
  }
  
  std::string License::getExpireDateString() {
        return std::string(ctime(&notafter));
  }

  void License::dump() {
    printf("ProductID :  %s\n", prodID.c_str());
    printf("ProductName :  %s\n", prodName.c_str());
    printf("Description :  %s\n", prodDescription.c_str());
    printf(" IssueDate:  %s"  , ctime(&issuedate));
    printf(" Notbefore:  %s"  , ctime(&notbefore));
    printf(" Notafter :  %s"  , ctime(&notafter));
	struct tm *t = localtime(&issuedate);
	printf(" IssueDate(short): [%04d-%02d-%02d]\n", t->tm_year+1900, t->tm_mon+1, t->tm_mday);
	t = localtime(&notbefore);
	printf(" Notbefore(short): [%04d-%02d-%02d]\n", t->tm_year+1900, t->tm_mon+1, t->tm_mday);
	t = localtime(&notafter);
	printf(" Notafter(short): [%04d-%02d-%02d]\n", t->tm_year+1900, t->tm_mon+1, t->tm_mday);
    printf(" HostAddr :  %s\n", hostAddr.c_str());
    printf(" HostMAC  :  %s\n", hostMAC.c_str());
    for (std::map<std::string,std::string>::const_iterator i = terms.begin();  i != terms.end(); i++) {
      printf("  term %s = %s\n", 
	     (*i).first.c_str(),
	     (*i).second.c_str());
    }
  }
  
}
