/* apps/pkcs7.c */
/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
 * All rights reserved.
 *
 * This package is an SSL implementation written
 * by Eric Young (eay@cryptsoft.com).
 * The implementation was written so as to conform with Netscapes SSL.
 * 
 * This library is free for commercial and non-commercial use as long as
 * the following conditions are aheared to.  The following conditions
 * apply to all code found in this distribution, be it the RC4, RSA,
 * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
 * included with this distribution is covered by the same copyright terms
 * except that the holder is Tim Hudson (tjh@cryptsoft.com).
 * 
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.
 * If this package is used in a product, Eric Young should be given attribution
 * as the author of the parts of the library used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    "This product includes cryptographic software written by
 *     Eric Young (eay@cryptsoft.com)"
 *    The word 'cryptographic' can be left out if the rouines from the library
 *    being used are not cryptographic related :-).
 * 4. If you include any Windows specific code (or a derivative thereof) from 
 *    the apps directory (application code) you must include an acknowledgement:
 *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
 * 
 * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 */

#define VERBOSE_TEST 0
#define USE_TEST_XML_FILE 0
#define DUMP_XML_STREAM 0
#define DEBUG 0
#define FULL_PANIC_ALERTS 0

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
/*#include "apps.h"*/
#include <openssl/err.h>
#include <openssl/objects.h>
#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/pkcs7.h>
#include <openssl/pem.h>
#include <openssl/x509v3.h>
#include <openssl/conf.h>
/*#include "cryptlib.h"*/
#include "X509Common.h"
#include <string>
#include <vector>
#include <stack>
#include <map>
#include <time.h>
#include <iostream.h>
#include <expat.h>

#include "license.hpp"      //  class License
#include "licensetag.hpp"   //  class LIcenseTag
#include "ctlicense.hpp"    //  'Published' library funcitons.

#undef PROG
#define PROG	pkcs7_main

/* -inform arg	- input format - default PEM (DER or PEM)
 * -outform arg - output format - default PEM
 * -in arg	- input file - default stdin
 * -out arg	- output file - default stdout
 * -print_certs
 */

namespace CTLicense {
  std::string dirLicenses;

  static std::string eth0Address;
  static std::string macAddress;
  
  //extern "C" {
  static int cb( int ok, X509_STORE_CTX *ctx);
  //}
  
  static char *xml = NULL;

  extern void scanxml(const char *xml, std::vector<License *> &licenses);


static int print_extensions(BIO *, char *, STACK_OF(X509_EXTENSION) *, unsigned long, int);

#define DUMP_EXTENSIONS 0
static void scan_X509_exts(BIO *bp, X509 *x)
{
#if DUMP_EXTENSIONS
	printf("Extenstions=======================================\n");
#endif
	unsigned long cflag=X509_FLAG_COMPAT;
	X509_CINF *ci = x->cert_info;
	print_extensions(bp, "X509v3 extensions",
			 ci->extensions, cflag, 8);
#if DUMP_EXTENSIONS
	printf("Extenstions end====================================\n");
#endif
}
                                     
#define LICENSE_OID "1.3.6.1.4.1.6449.1.1.5.2"

static int print_extensions(BIO *bp, char *title, STACK_OF(X509_EXTENSION) *exts, unsigned long flag, int indent)
{
        int i, j;
 
        if(sk_X509_EXTENSION_num(exts) <= 0) return 1;
#if DUMP_EXTENSIONS
        if(title)
                {
                BIO_printf(bp,"%*s%s:\n",indent, "", title);
                indent += 4;
                }
#endif
        for (i=0; i<sk_X509_EXTENSION_num(exts); i++)
                {
                ASN1_OBJECT *obj;
                X509_EXTENSION *ex;
		BIO *mem = BIO_new(BIO_s_mem());
                ex=sk_X509_EXTENSION_value(exts, i);
		ASN1_OBJECT *o2 = X509_EXTENSION_get_object(ex);
		i2a_ASN1_OBJECT(mem,o2);
		char buf[256];
		BIO_gets(mem,buf,256);
		//printf("object: %s\n", buf);
		if (strcmp(LICENSE_OID,buf) == 0) {
			char *data;
			M_ASN1_OCTET_STRING_print(mem,ex->value);
			int len = BIO_get_mem_data(mem,&data);
			xml = (char*)malloc(len+1);
			strncpy(xml,data,len);
			xml[len] = 0;
		}
#if DUMP_EXTENSIONS
                if (indent && BIO_printf(bp,"%*s",indent, "") <= 0) return 0;
                obj=X509_EXTENSION_get_object(ex);
                
		i2a_ASN1_OBJECT(bp,obj);
                j=X509_EXTENSION_get_critical(ex);
                if (BIO_printf(bp,": %s\n",j?"critical":"","") <= 0)
                        return 0;
                if(!X509V3_EXT_print(bp, ex, flag, indent + 4))
                        {
                        BIO_printf(bp, "%*s", indent + 4, "");
                        M_ASN1_OCTET_STRING_print(bp,ex->value);
                        }
                if (BIO_write(bp,"\n",1) <= 0) return 0;
#endif
		BIO_free(mem);
                }
        return 1;
}
                        
static int check(X509_STORE *ctx, char *certfile, STACK_OF(X509) *uchain, int purpose);

static int validate_cert(const char *certfile, const char *CApath, const char *CAfile)
{
    X509_STORE *ctx = NULL;
    STACK_OF(X509) *untrusted = NULL;
    X509_LOOKUP *lookup = NULL;
 
    //char *untfile = NULL;
 
    if ((ctx = X509_STORE_new()) == NULL)
    {
      //puts("validate failed at first attempt");
      exit(1);
    }
    
    X509_STORE_set_verify_cb_func(ctx, cb);
     
    // Probably uneccessary
    //ERR_load_crypto_strings();
    ERR_clear_error();
    apps_startup();
 
    OpenSSL_add_all_algorithms();
 
    if ((lookup = X509_STORE_add_lookup(ctx, X509_LOOKUP_file())) == NULL)
    {
      //puts("Lookup failed at first attempt");
      exit(1);
    }
 
    ERR_clear_error(); // Don't touch! This is vital for the developer's sanity
 
    if (CAfile) {
        if (!X509_LOOKUP_load_file(lookup,CAfile,X509_FILETYPE_PEM))
        {
	  //puts("Lookup failed at second attempt");
	  exit(1);
        }
    }
    else
        X509_LOOKUP_load_file(lookup, NULL, X509_FILETYPE_DEFAULT);
 
    //ERR_clear_error();
    // Set lookup to use hash names
    if ((lookup = X509_STORE_add_lookup(ctx,X509_LOOKUP_hash_dir())) == NULL)
    {
      //puts("Lookup failed at third attempt");
      exit(1);
    }
 
    //ERR_clear_error();
    if (CApath) {
        if (!X509_LOOKUP_add_dir(lookup, CApath, X509_FILETYPE_PEM))
        {
	  //puts("Lookup failed at second attempt");
	  exit(1);
        }
    }
    else
        X509_LOOKUP_add_dir(lookup,NULL,X509_FILETYPE_DEFAULT);
 
    ERR_clear_error();
    int ret = check(ctx, (char *)certfile, untrusted, 0);
    sk_X509_pop_free(untrusted, X509_free);
    return ret;
}
 
int check(X509_STORE *ctx, char *certfile, STACK_OF(X509) *uchain, int purpose)
{
  X509 *x509 = NULL;
  BIO *in = NULL;
  int ret=0;
  X509_STORE_CTX *csc;
  //char *crldir=NULL;
  
  in=BIO_new(BIO_s_file());

  if (in == NULL) {
    // print some errors and return
    return 0;
  }
  if (certfile==NULL) {
    BIO_set_fp(in,stdin,BIO_NOCLOSE);
  }
  else {
    if (BIO_read_filename(in,certfile) <= 0) {
      // print some errors and return
      return 0;
    }
  }
  x509=PEM_read_bio_X509(in, NULL, NULL, NULL);

  if (x509 == NULL) {
    // print some errors and return
    return 0;
  }
  
  csc = X509_STORE_CTX_new();

  if (csc == NULL) {
    // print some errors and return
    fprintf(stderr, "Cannot create certificate store\n");
    return 0;
  }
  X509_STORE_CTX_init(csc, ctx, x509, uchain);
  
  if (purpose >= 0) X509_STORE_CTX_set_purpose(csc, purpose);
  ret=X509_verify_cert(csc);
  
  //if (ret && crldir!=NULL)
  //    ret=crl_check(csc, crldir);
  X509_STORE_CTX_free(csc);
  
  return(ret);
}

#define LICENSE_CERT "/tmp/___crt.pem"
#define CA_CERTS     "/tmp/___cas.pem"

#ifndef OPENSSL_NO_ENGINE
#define OPENSSL_NO_ENGINE
#endif

static void loadLicenseFile(char *licfile, std::vector<License *> &licenses);
static void loadLicenseFile2(char *licfile, std::vector<License *> &licenses);
static void loadTestLicenseFile(char *licfile, std::vector<License *> &licenses);


void _reloadLicenses(std::vector<License *> &licenses) {
#if VERBOSE_TEST
  cout <<" Inside ReloadLicenses "<<endl;
#endif
  licenses.clear();

  char buf[256];
#if USE_TEST_XML_FILE
  strcpy(buf,"/opt/xsentry/etc/licenses");
#else
//  strcpy(buf,"/home/vladimir/comodo-certs/lics");
  strcpy(buf, dirLicenses.c_str());
#endif
  DIR *dir = opendir(buf);
  if (dir) {
    strcat(buf,"/");
    char *ptr = buf + strlen(buf);
    struct dirent *dp;
    while (dp = readdir(dir)) {
      strcpy(ptr,dp->d_name);
      struct stat st;
      if ((stat(buf,&st) == 0) && (st.st_mode & S_IFREG)) {
#if USE_TEST_XML_FILE
	loadTestLicenseFile(buf,licenses);
#else
	loadLicenseFile2(buf,licenses);
#endif
      }
    }
    closedir(dir);
  }
  else {
    loadLicenseFile2(buf,licenses);
  }
  
#if VERBOSE_TEST
  printf ("Has %d licenses\n", licenses.size());
  for (int i = 0; i < licenses.size(); i++) {
    printf("HAS LICENSE FOR: %s\n", licenses[i]->getProduct().c_str());
    licenses[i]->dump();
  }
#endif
  
//  eth0Address = getIPaddress();
//  macAddress  = getMACaddress();
}   

void loadTestLicenseFile(char *infile, std::vector<License*> &licenses)
{
#if VERBOSE_TEST
  cout<<"Inside Load Test License File"<<endl;
  printf("\n>>> Trying test license file %s\n",infile);
#endif
  char *xml = NULL;
  char buf[256];
  int sz = 0;
  
  FILE *f = fopen(infile,"r");

  if (f) {
    while (fgets(buf,256,f)) {
      if (strlen(buf)) {
	if (sz) {
	  xml = (char*)realloc(xml,sz + strlen(buf) + 1);
	  strcat(xml,buf);
	}
	else {
	  xml = (char*) malloc(strlen(buf) + 1);
	  strcpy(xml,buf);
	}
	sz += strlen(buf) + 1;
      }
    }
    fclose(f);
    scanxml(xml, licenses);
    if (xml) free(xml);
  }
}


void loadLicenseFile(char *infile, std::vector<License*> &licenses)
{
#if VERBOSE_TEST
  cout<<"Inside Load License file"<<endl;
  printf("\n>>> Trying file %s\n",infile);
#endif
  X509               *x509  = NULL;
  
  BIO *in = NULL, *out = NULL;
  char *prog;
  int ret = 1;
  
  if (bio_err == NULL)
    if ((bio_err=BIO_new(BIO_s_file())) != NULL)
      BIO_set_fp(bio_err,stderr,BIO_NOCLOSE|BIO_FP_TEXT);
  
  ERR_load_crypto_strings();
  
  in=BIO_new(BIO_s_file());
  out=BIO_new(BIO_s_file());
  if ((in == NULL) || (out == NULL))
    {
      ERR_print_errors(bio_err);
      goto end;
    }
  BIO_set_fp(out,stdout,BIO_NOCLOSE);
  
  if (infile == NULL)
    BIO_set_fp(in,stdin,BIO_NOCLOSE);
  else
    {
      if (BIO_read_filename(in,infile) <= 0)
	if (in == NULL)
	  {
	    perror(infile);
	    goto end;
	  }
    }
  x509=PEM_read_bio_X509(in,NULL,NULL,NULL); // Try PEM format first
  if (x509 == NULL) {
#ifdef FULL_PANIC_ALERTS
      BIO_printf(bio_err,"unable to load X509 object\n");
      ERR_print_errors(bio_err);
#endif
      goto end;
    }
//  BIO *cert
//  PEM_write_bio_X509(in, xx);
//  scan_X509_exts(out,x);
  scan_X509_exts(in, x509);

end:
  //puts("\n\nVERIFICATION\n\n");
//  int valid = validate_cert(LICENSE_CERT, NULL, CA_CERTS);
  int valid = validate_cert(infile, NULL, CA_CERTS);
#if VERBOSE_TEST
  printf("Verification was %s\n", (valid)? "OK" : "not OK"); 
#endif

  if (xml)
  {
#if VERBOSE_TEST && DUMP_XML_STREAM
    printf("XML of license:\n%s\n",xml);
    printf("---------------------------------------------------------\n");
#endif
    if (valid) scanxml(xml, licenses);
    free(xml);
  }
#if VERBOSE_TEST && DUMP_XML_STREAM
  else
    printf("No XML of license!\n");
#endif

#if VERBOSE_TEST
  printf ("==============================================================\n");
#endif
  
#if VERBOSE_TEST
  for (int i = 0; i < licenses.size(); i++) {
    printf("HAS LICENSE FOR: %s\n", licenses[i]->getProduct().c_str());
    licenses[i]->dump();
  }
#endif

  if (x509 != NULL) X509_free(x509);
  if (in != NULL) BIO_free(in);
  if (out != NULL) BIO_free_all(out);
#if VERBOSE_TEST
  printf ("===end of loadLicenseFile======================================\n");
#endif
}

void loadLicenseFile2(char *infile, std::vector<License*> &licenses)
{
#if VERBOSE_TEST
  cout<<"Inside Load License file"<<endl;
  printf("\n>>> Trying file %s\n",infile);
#endif
  STACK_OF(X509)     *certs = NULL;
  STACK_OF(X509_CRL) *crls  = NULL;
  X509               *x509  = NULL;
  //int i,badops=0;
  BIO *in = NULL, *out = NULL;
  char *prog;
  int ret = 1;
  
  if (bio_err == NULL)
    if ((bio_err=BIO_new(BIO_s_file())) != NULL)
      BIO_set_fp(bio_err,stderr,BIO_NOCLOSE|BIO_FP_TEXT);
  
  ERR_load_crypto_strings();
  
  in=BIO_new(BIO_s_file());
  out=BIO_new(BIO_s_file());
  if ((in == NULL) || (out == NULL))
    {
      ERR_print_errors(bio_err);
      goto end;
    }
  BIO_set_fp(out,stdout,BIO_NOCLOSE);
  
  if (infile == NULL)
    BIO_set_fp(in,stdin,BIO_NOCLOSE);
  else
    {
      if (BIO_read_filename(in,infile) <= 0)
	if (in == NULL)
	  {
	    perror(infile);
	    goto end;
	  }
    }
	 certs = sk_X509_new_null();
	 while (x509 = PEM_read_bio_X509(in,NULL,NULL,NULL))
		sk_X509_push(certs, x509); // Try PEM format first
#if VERBOSE_TEST
	 printf("HAS BEEN LOADED %d certificates\n", sk_X509_num(certs));
#endif
  
  if (sk_X509_num(certs) == 0)
    {
#ifdef FULL_PANIC_ALERTS
      BIO_printf(bio_err,"unable to load X509 objects\n");
      ERR_print_errors(bio_err);
#endif
      goto end;
    }
  if (certs != NULL)
    {
      X509 *x,*xx;
      BIO *cert, *cas;
      cert = BIO_new_file(LICENSE_CERT,"w");
      cas  = BIO_new_file(CA_CERTS,"w");
      
      for (int i = 0; i < sk_X509_num(certs); i++)
	{
	  x=sk_X509_value(certs,i);
	  xx=sk_X509_value(certs,i);
	  PEM_write_bio_X509((i < 2) ? cas : cert, xx);
	  scan_X509_exts(out,x);
#if DUMP_EXTENSIONS
	  BIO_puts(out,"\n");
#endif
	}
      BIO_flush(cert); BIO_free(cert);
      BIO_flush(cas);  BIO_free(cas);
    }
  
  if (crls != NULL)
    {
      X509_CRL *crl;
      for (int i = 0; i < sk_X509_CRL_num(crls); i++)
	{
	  crl=sk_X509_CRL_value(crls,i);
	  
	  X509_CRL_print(out, crl);
	  
	  BIO_puts(out,"\n");
	}
    }
  ret=0;
 end:
  
  int valid = validate_cert(LICENSE_CERT, NULL, CA_CERTS);
#if VERBOSE_TEST
  printf("Verification was %s\n", (valid)? "OK" : "not OK"); 
#endif
  remove(LICENSE_CERT);
  remove(CA_CERTS);
  
  if (xml)
  {
#if VERBOSE_TEST && DUMP_XML_STREAM
    printf("XML of license:\n%s\n",xml);
    printf("---------------------------------------------------------\n");
#endif
    if (valid) scanxml(xml, licenses);
    free(xml);
  }
#if VERBOSE_TEST && DUMP_XML_STREAM
  else
    printf("No XML of license!\n");
#endif

#if VERBOSE_TEST
  printf ("==============================================================\n");
#endif
  
#if VERBOSE_TEST
  for (int i = 0; i < licenses.size(); i++) {
    printf("HAS LICENSE FOR: %s\n", licenses[i]->getProduct().c_str());
    licenses[i]->dump();
  }
#endif
  if (in != NULL) BIO_free(in);
  if (out != NULL) BIO_free_all(out);
  if (certs != NULL) sk_X509_pop_free(certs, X509_free);
}

static int cb( int ok, X509_STORE_CTX *ctx)
{
#ifdef DEBUG
    char buf[256];
#endif
 
    if (!ok)
    {
#ifdef DEBUG
        cerr << "Verify Cert caused callback activation, status <" << ((ok)? "OK" : "NOT OK")
             << "> errorcode <" << ctx->error << ">\n";
#endif
        {
#ifdef DEBUG
            X509_NAME_oneline(X509_get_subject_name(ctx->current_cert),buf,sizeof(buf));
            fprintf(stderr,"%s\n",buf);
            fprintf(stderr,"error %d at %d depth lookup:%s\n",ctx->error,
                    ctx->error_depth, X509_verify_cert_error_string(ctx->error));
#endif
             
        }
        /*
         * TODO: Expired certs and selfsigned certs should perhaps not be accepted here
         *       Also, the same may be the case for extension errors
         */
        if (ctx->error == X509_V_ERR_CERT_HAS_EXPIRED) ok=1;
 
        /*
         * If this block is not defined in, only a self-signed cert which also is one
         * of our accepted CA certs, will pass validation. If this block is defined
         * in, all self-signed certs will pass validation, even if the cert is unknown
         * to us.
         */
#ifdef XSENTRY_ACCEPTS_SELFSIGNED_CERTS
        if (ctx->error == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT)
            /* Check to see if it is a CA certificate, ie with
             * basicConstraints, and perhaps other checks.
             * For the moment, we just let it slide */
            ok = 1;
#endif
        /* Continue after extensions error too */
        if (ctx->error == X509_V_ERR_INVALID_CA) ok=1;
        if (ctx->error == X509_V_ERR_PATH_LENGTH_EXCEEDED) ok=1;
        if (ctx->error == X509_V_ERR_INVALID_PURPOSE) ok=1;
    }
    return(ok);
}
 
}
