/* scanxml.cpp
 * 
 * Frontend into expat to parse xml streams into class License objects.
 * The actual License object creation is done by the LicenseTag helper class.
 *
 * There is one externally visible function, scanxml(xml, licenses), which
 * expect its in args to be
 *    xml      - should point to a buffer containing an xml stream
 *    licenses - should reference a std::vector<License*> where any
 *               licenses found in the xml stream will be appended.
 */

#define VERBOSE_TEST 0


#include <stack>
#include <expat.h>
#include <iostream.h>
#include "licensetag.hpp"

namespace CTLicense {

  static int depth = 0;
  static LicenseTag *root = NULL;
  
  static std::stack<LicenseTag*> licensestack;
  
  static void startX(void *data, const char *el, const char **attr);
  static void startX(void *data, const char *el, const char **attr) {
    LicenseTag *now = new LicenseTag();
    now->name = el;
    licensestack.top()->tags.push_back(now);
    licensestack.push(now);
    depth++;
  }

  static void endX(void *data, const XML_Char *name);
  static void endX(void *data, const XML_Char *name) {
    depth--;
    licensestack.pop();
  }
  
  static int cdata = 0;
  static void startC(void*data) { cdata++; printf("<"); }
  static void endC  (void*data) { cdata++; printf("<"); }
  
  static void dataX(void *userData,const XML_Char *s,int len);
  static void dataX(void *userData,const XML_Char *s,int len) {
    
    char buf[1024];
    strncpy(buf, s, len); buf[len] = 0;
    if (0) printf (" (%s)", buf);
    licensestack.top()->data = buf;
  }
  
  static void extractLicenses(std::vector<License *> &l) { root->registerLicenses(l); }
  
  void scanxml(const char *xml, std::vector<License *> &licenses) {
#if VERBOSE_TEST
    cout<<" Inside Scan XML"<<endl;
#endif
    int skip = 0;
    root = new LicenseTag();
    licensestack.push(root);
    XML_Parser parser;
    parser = XML_ParserCreate(NULL);
    XML_SetElementHandler(parser,startX,endX);
    XML_SetCharacterDataHandler(parser,dataX);
    XML_SetCdataSectionHandler(parser,startC,endC);
    while (xml[skip] != '<') skip++;
    if (XML_Parse(parser,&xml[skip],strlen(xml)-skip,1)) {
#if VERBOSE_TEST
      printf("XML looked fine\n");
#endif
      extractLicenses(licenses);
      if (root) delete root;
    }
#if VERBOSE_TEST
    else
      printf("XML looked bad\n");
#endif
  }
}
