#!/usr/bin/perl
package lpadmin::Cupsdriver;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'lpadmin::Cupsdriver');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# cups-driver.pl
# Functions for CUPS printer drivers

$self->{'webmin_windows_driver'} = 0;
$self->{'cups_ppd_dir'} = "/etc/cups/ppd";
$self->{'lpoptions'} = $self->has_command("lpoptions.cups") ? "lpoptions.cups" : "lpoptions";

1;

1;
		return $self;
}

# is_windows_driver(path, &printer)
# Returns the server, share, username, password, workgroup, program
# if path is a webmin windows driver
sub is_windows_driver
{
	my $self = shift;

if ($_[1]->{'dev'} =~ /^smb:\/\/(\S*):(\S*)\@(\S*)\/(\S*)\/(\S*)$/) {
	return { 'user' => $1,
		 'pass' => $2,
		 'workgroup' => $3,
		 'server' => $4,
		 'share' => $5,
		 'program' => $_[0] };
	}
elsif ($_[1]->{'dev'} =~ /^smb:\/\/(\S*):(\S*)\@(\S*)\/(\S*)$/) {
	return { 'user' => $1,
		 'pass' => $2,
		 'server' => $3,
		 'share' => $4,
		 'program' => $_[0] };
	}
elsif ($_[1]->{'dev'} =~ /^smb:\/\/(\S*)\/(\S*)\/(\S*)$/) {
	return { 'workgroup' => $1,
		 'server' => $2,
		 'share' => $3,
		 'program' => $_[0] };
	}
elsif ($_[1]->{'dev'} =~ /^smb:\/\/(\S*)\/(\S*)$/) {
	return { 'server' => $1,
		 'share' => $2,
		 'program' => $_[0] };
	}
else {
	return undef;
	}
}

# is_driver(path, &printer)
# Returns the driver name if some path is a CUPS driver, or undef
sub is_driver
{
	my $self = shift;

if (!$_[0] || !-r $_[0]) {
	return { 'mode' => 0,
		 'desc' => $self->{'text_hash'}->{'cups_none'} };
	}
local $ppd = $self->parse_cups_ppd($_[0]);
if ($ppd->{'NickName'}) {
	# Looks like a CUPS PPD file!
	return { 'mode' => 1,
		 'manuf' => $ppd->{'Manufacturer'},
		 'model' => $ppd->{'ModelName'},
		 'nick' => $ppd->{'NickName'},
		 'desc' => "$ppd->{'Manufacturer'} $ppd->{'ModelName'}" };
	}
else {
	# Some other kind of interface file
	return { 'mode' => 2,
		 'file' => $_[0],
		 'desc' => $_[0] };
	}
}

# create_windows_driver(&printer, &driver)
sub create_windows_driver
{
	my $self = shift;

if ($_[1]->{'workgroup'} && $_[1]->{'user'}) {
	$_[0]->{'dev'} = "smb://$_[1]->{'user'}:$_[1]->{'pass'}\@$_[1]->{'workgroup'}/$_[1]->{'server'}/$_[1]->{'share'}";
	}
elsif ($_[1]->{'workgroup'}) {
	$_[0]->{'dev'} = "smb://$_[1]->{'workgroup'}/$_[1]->{'server'}/$_[1]->{'share'}";
	}
elsif ($_[1]->{'user'}) {
	$_[0]->{'dev'} = "smb://$_[1]->{'user'}:$_[1]->{'pass'}\@$_[1]->{'server'}/$_[1]->{'share'}";
	}
else {
	$_[0]->{'dev'} = "smb://$_[1]->{'server'}/$_[1]->{'share'}";
	}
return $_[1]->{'program'};
}

# create_driver(&printer, &driver)
sub create_driver
{
	my $self = shift;

local $drv = "$self->{'cups_ppd_dir'}/$_[0]->{'name'}.ppd";
undef($self->{'cups_driver_options'});
if ($_[1]->{'mode'} == 0) {
	$self->system_logged("rm -f \"$drv\"");
	return undef;
	}
elsif ($_[1]->{'mode'} == 2) {
	$self->system_logged("rm -f \"$drv\"");
	return $_[1]->{'file'};
	}
else {
	# Copy the driver into place
	if ($_[1]->{'ppd'} =~ /\.gz$/) {
		$self->system_logged("gunzip -c '$_[1]->{'ppd'}' >$drv");
		}
	else {
		$self->system_logged("cp $_[1]->{'ppd'} $drv");
		}
	chmod(0777, $drv);
	$self->{'cups_driver_options'} = $_[1]->{'opts'};	# for modify_printer
	return $drv;
	}
}

# delete_driver(name)
sub delete_driver
{
	my $self = shift;

$self->system_logged("rm -f \"$self->{'cups_ppd_dir'}/$_[0].ppd\"");
}

# driver_input(&printer, &driver)
sub driver_input
{
	my $self = shift;

printf "<tr> <td><input type=radio name=mode value=0 %s> %s</td>\n",
	$_[1]->{'mode'} == 0 ? 'checked' : '', $self->{'text_hash'}->{'cups_none'};
print "<td>($self->{'text_hash'}->{'cups_nonemsg'})</td> </tr>\n";
printf "<tr> <td><input type=radio name=mode value=2 %s> %s</td>",
	$_[1]->{'mode'} == 2 ? 'checked' : '', $self->{'text_hash'}->{'cups_prog'};
printf "<td><input name=program size=40 value='%s'></td> </tr>\n",
	$_[1]->{'mode'} == 2 ? $_[0]->{'iface'} : '';

# Display all the CUPS drivers
printf "<tr> <td valign=top><input type=radio name=mode value=1 %s> %s</td>\n",
	$_[1]->{'mode'} == 1 ? 'checked' : '', $self->{'text_hash'}->{'cups_driver'};
print "<td><select name=ppd size=10>\n";
local (@ppds, $d, $f, $ppd, %cache, $outofdate, @files, %donefile);
open(FIND, "find '$self->{'config_hash'}->{'model_path'}' -type f -print |");
while(<FIND>) {
	chop;
	/([^\/]+)$/;
	next if ($donefile{$1}++);
	push(@files, $_);
	}
close(FIND);
$self->read_file("$self->{'module_config_directory'}/ppd-cache", \%cache);
foreach $f (@files) {
	if (!defined($cache{$f})) {
		$outofdate = $f;
		last;
		}
	}
if ($outofdate || scalar(keys %cache) != scalar(@files)) {
	# Cache is out of date
	undef(%cache);
	local %donecache;
	foreach $f (@files) {
		local $ppd = $self->parse_cups_ppd($f);
		local $nn = $ppd->{'NickName'};
		$cache{$f} = $donecache{$nn} ? "duplicate" : $ppd->{'NickName'};
		$donecache{$nn}++;
		}
	$self->write_file("$self->{'module_config_directory'}/ppd-cache", \%cache);
	}
local %done;
foreach $f (sort { $cache{$a} cmp $cache{$b} } keys %cache) {
	if ($cache{$f} && $cache{$f} ne "duplicate" &&
	    !$done{$cache{$f}}++) {
		printf "<option value=%s %s>%s\n",
			$f, $_[1]->{'nick'} eq $cache{$f} ? 'selected' : '',
			$cache{$f};
		$self->{'currppd'} = $f if ($_[1]->{'nick'} eq $cache{$f});
		}
	}
print "</select>\n";

# Display driver option inputs
if ($self->{'currppd'}) {
	local $ppd = $self->parse_cups_ppd($self->{'currppd'});
	print "<br><b>",$self->text('cups_opts', $ppd->{'NickName'}),
	      "</b><table>\n";
	open(OPTS, "$self->{'lpoptions'} -p '$_[0]->{'name'}' -l 2>/dev/null |");
	while(<OPTS>) {
		if (/^(\S+)\/([^:]+):\s*(.*)/ && $1 ne "PageRegion") {
			print "<tr>\n" if ($self->{'i'}%2 == 0);
			local $code = $1;
			local $disp = $2;
			local @opts = split(/\s+/, $3);
			print "<td><b>$disp:</b></td>\n";
			print "<td><select name=ppd_$code>\n";
			foreach $o (@opts) {
				local $sel = ($o =~ s/^\*//);
				printf "<option value='%s' %s>%s\n",
					$o, $sel ? "selected" : "",
					$ppd->{$code}->{$o} ?
					  $ppd->{$code}->{$o} : $o;
				}
			print "</select></td>\n";
			print "<tr>\n" if ($self->{'i'}%2 == 1);
			}
		}
	close(OPTS);
	print "</table>\n";
	}

print "</td> </tr>\n";
return undef;
}

# parse_driver()
# Parse driver selection from %{$self->{'in_hash'}} and return a driver structure
sub parse_driver
{
	my $self = shift;

if ($self->{'in_hash'}->{'mode'} == 0) {
	return { 'mode' => 0 };
	}
elsif ($self->{'in_hash'}->{'mode'} == 2) {
	$self->{'in_hash'}->{'program'} =~ /^(\S+)/ && -x $1 ||
		$self->error($self->text('cups_eprog', $self->{'in_hash'}->{'program'}));
	return { 'mode' => 2,
		 'file' => $self->{'in_hash'}->{'program'} };
	}
elsif ($self->{'in_hash'}->{'mode'} == 1) {
	# CUPS printer driver
	local $ppd = $self->parse_cups_ppd($self->{'in_hash'}->{'ppd'});
	local $rv = { 'mode' => 1,
		      'ppd' => $self->{'in_hash'}->{'ppd'},
		      'nick' => $ppd->{'NickName'},
		      'manuf' => $ppd->{'Manufacturer'},
		      'model' => $ppd->{'ModelName'} };
	foreach $i (keys %{$self->{'in_hash'}}) {
		$rv->{'opts'}->{$1} = $self->{'in_hash'}->{$i} if ($i =~ /^ppd_(.*)$/);
		}
	return $rv;
	}
}


1;

