#!/usr/bin/perl
package lpadmin::FreebsdLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'lpadmin::FreebsdLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# freebsd-lib.pl
# Functions for printcap-style printer management

if ($self->{'gconfig_hash'}->{'os_type'} eq 'openbsd') {
	@{$self->{'device_files_array'}} = ("/dev/lpt0", "/dev/cua00", "/dev/cua01");
	}
else {
	@{$self->{'device_files_array'}} = ("/dev/lpt0", "/dev/cuaa0", "/dev/cuaa1");
	}
@{$self->{'device_names_array'}} = ($self->{'text_hash'}->{'freebsd_paralel'}, $self->text('freebsd_serial', "1"),
		 $self->text('freebsd_serial', "2"));

1;
		return $self;
}

# list_printers()
# Returns a list of known printer names
sub list_printers
{
	my $self = shift;

local($l, @rv);
foreach $l ($self->list_printcap()) {
	$l->{'name'} =~ /^([^\|]+)/;
	push(@rv, $1);
	}
return @rv;
}

# get_printer(name, [nostatus])
sub get_printer
{
	my $self = shift;

local($l, %prn, @w, @n, $w, %cap, @jobs);
foreach $l ($self->list_printcap()) {
	@n = split(/\|/, $l->{'name'});
	if ($n[0] eq $_[0]) {
		# found the printer.. get info from printcap
		$prn{'name'} = $n[0];
		if (@n > 2) { $prn{'alias'} = [ @n[1..$#n-1] ]; }
		if (@n > 1) { $prn{'desc'} = $n[$#n]; }
		$prn{'iface'} = $l->{'if'};
		$prn{'banner'} = !defined($l->{'sh'});
		$prn{'dev'} = $l->{'lp'};
		$prn{'rhost'} = $l->{'rm'};
		$prn{'rqueue'} = $l->{'rp'};
		$prn{'msize'} = $l->{'mx'};
		$prn{'ro'} = $l->{'file'} eq $self->{'config_hash'}->{'ro_printcap_file'};

		if (!$_[1]) {
			# call lpc to get status
			local $esc = quotemeta($prn{'name'});
			$self->{'out'} = `lpc status $esc 2>&1`;
			$prn{'accepting'} = ($self->{'out'} =~ /queuing is enabled/);
			$prn{'enabled'} = ($self->{'out'} =~ /printing is enabled/);
			}

		return \%prn;
		}
	}
return undef;
}

sub get_jobs
{
	my $self = shift;

local @jobs;
local $esc = quotemeta($_[0]);
open(LPQ, "lpq -P$esc |");
while(<LPQ>) {
	chop;
	if (/^Rank\s+Owner\s+/) { $self->{'doneheader'}++; }
	elsif ($self->{'doneheader'} && /^(\S+)\s+(\S+)\s+(\d+)\s+(.*\S)\s+(\d+)\s+(\S+)$/) {
		local(%job);
		$job{'id'} = $3;
		$job{'user'} = $2;
		$job{'size'} = $5;
		$job{'file'} = $4;
		$job{'printing'} = ($1 eq "active");
		push(@jobs, \%job);
		}
	}
close(LPQ);
return @jobs;
}

sub printer_support
{
	my $self = shift;

return $_[0] !~ /^(why|allow|default|ctype|riface|sysv|direct|rnoqueue|ipp)$/;
}

# create_printer(&details)
# Create a new printer in /etc/printcap
sub create_printer
{
	my $self = shift;

local(%cap);
$cap{'sd'} = "$self->{'config_hash'}->{'spool_dir'}/$_[0]->{'name'}";
$self->lock_file($cap{'sd'});
mkdir($cap{'sd'}, 0755);
$self->unlock_file($cap{'sd'});

$self->lock_file($self->{'config_hash'}->{'printcap_file'});
local $lref = $self->read_file_lines($self->{'config_hash'}->{'printcap_file'});
push(@$lref, $self->make_printcap($_[0], \%cap));
$self->flush_file_lines($self->{'config_hash'}->{'printcap_file'});
$self->unlock_file($self->{'config_hash'}->{'printcap_file'});
$self->apply_status($_[0]);
}

# modify_printer(&details)
sub modify_printer
{
	my $self = shift;

local(@old, $o, $old, @cap);
$self->lock_file($self->{'config_hash'}->{'printcap_file'});
@old = $self->list_printcap();
foreach $o (@old) {
	$o->{'name'} =~ /^([^\|]+)/;
	if ($1 eq $_[0]->{'name'}) {
		# found current details
		$old = $o;
		last;
		}
	}
if (!$old) { $self->error("Printer '$_[0]->{'name'}' no longer exists"); }
local $lref = $self->read_file_lines($self->{'config_hash'}->{'printcap_file'});
splice(@$lref, $old->{'line'}, $old->{'eline'} - $old->{'line'} + 1,
	$self->make_printcap($_[0], $old));
$self->flush_file_lines($self->{'config_hash'}->{'printcap_file'});
$self->unlock_file($self->{'config_hash'}->{'printcap_file'});
$self->apply_status($_[0]);
}

# delete_printer(name)
sub delete_printer
{
	my $self = shift;

local(@old, $o, $old, @cap);
$self->lock_file($self->{'config_hash'}->{'printcap_file'});
@old = $self->list_printcap();
foreach $o (@old) {
	$o->{'name'} =~ /^([^\|]+)/;
	if ($1 eq $_[0]) {
		# found current details
		$old = $o;
		last;
		}
	}
if (!$old) { $self->error("Printer '$_[0]' no longer exists"); }
local $lref = $self->read_file_lines($self->{'config_hash'}->{'printcap_file'});
splice(@$lref, $old->{'line'}, $old->{'eline'} - $old->{'line'} + 1);
$self->flush_file_lines($self->{'config_hash'}->{'printcap_file'});
$self->unlock_file($self->{'config_hash'}->{'printcap_file'});
}

# cancel_job(printer, job)
# Calls lprm to remove some job
sub cancel_job
{
	my $self = shift;

local($out);
local $esc = quotemeta($_[0]);
local $iesc = quotemeta($_[1]);
$out = $self->backquote_logged("lprm -P$esc $iesc 2>&1");
if ($?) { $self->error("lprm failed : $out"); }
}

# make_printcap(&details, &old)
# Updates or creates a printcap line
sub make_printcap
{
	my $self = shift;

local(%prn, %cap, $a, $rv, $c);
%prn = %{$_[0]}; %cap = %{$_[1]};
$cap{'if'} = $prn{'iface'} ? $prn{'iface'} : undef;
$cap{'sh'} = $prn{'banner'} ? undef : "";
$cap{'lp'} = $prn{'dev'} ? $prn{'dev'} : undef;
$cap{'rm'} = $prn{'rhost'} ? $prn{'rhost'} : undef;
$cap{'rp'} = $prn{'rqueue'} ? $prn{'rqueue'} : undef;
$cap{'mx'} = defined($prn{'msize'}) ? $prn{'msize'} : undef;
$rv = $prn{'name'};
foreach $a (@{$prn{'alias'}}) { $rv .= "|$a"; }
$rv .= "|$prn{'desc'}";
foreach $c (keys %cap) {
	if (length($c) == 2 && defined($cap{$c})) {
		if ($cap{$c} eq "") { $rv .= ":$c"; }
		elsif ($cap{$c} =~ /^\d+$/) { $rv .= ":$c#$cap{$c}"; }
		else { $rv .= ":$c=$cap{$c}"; }
		}
	}
$rv .= ":";
return $rv;
}

# list_printcap()
# Returns an array of associative arrays containing printcap fields
sub list_printcap
{
	my $self = shift;

return @{$self->{'list_printcap_cache_array'}} if (defined(@{$self->{'list_printcap_cache_array'}}));
local(@rv, @line, $line, $cont, $lnum, $i, %done, $capfile);
foreach $capfile ($self->{'config_hash'}->{'printcap_file'}, $self->{'config_hash'}->{'ro_printcap_file'}) {
	next if (!$capfile || $done{$capfile}++);
	open(CAP, $capfile);
	$lnum = 0;
	while($line = <CAP>) {
		$line =~ s/^#.*$//g;	# remove comments
		$line =~ s/\s+$//g;	# remove trailing spaces
		$line =~ s/^\s+//g;	# remove leading spaces
		if ($line =~ /\S/) {
			$self->{'ncont'} = ($line =~ s/\\$//g);
			if ($cont) {
				$line[$#line] .= $line;
				$self->{'eline_array'}->[@line - 1] = $lnum;
				}
			else {
				push(@line, $line);
				$self->{'eline_array'}->[@line - 1] = $self->{'sline_array'}->[@line - 1] = $lnum;
				}
			$cont = $self->{'ncont'};
			}
		$lnum++;
		}
	close(CAP);
	for($i=0; $i<@line; $i++) {
		local(%cap);
		@w = split(/:+/, $line[$i]);
		$cap{'name'} = $w[0];
		$cap{'line'} = $self->{'sline_array'}->[$i];
		$cap{'eline'} = $self->{'eline_array'}->[$i];
		$cap{'file'} = $capfile;
		foreach $w (@w[1..$#w]) {
			if ($w =~ /^([A-z0-9]+)[=#](.*)$/) { $cap{$1} = $2; }
			elsif ($w =~ /^([A-z0-9]+)$/) { $cap{$w} = ""; }
			}
		push(@rv, \%cap);
		}
	}
@{$self->{'list_printcap_cache_array'}} = @rv;
return @rv;
}

# apply_status(&details)
# Calls lpc to enable or disable a printer.
# Restarting lpd doesn't seem to be necessary?
sub apply_status
{
	my $self = shift;

local($out);
local $esc = quotemeta($_[0]->{'name'});
$out = `lpc status $esc 2>&1`;
if ($_[0]->{'enabled'} && $out !~ /printing is enabled/)
	{ $self->backquote_logged("lpc up $esc"); }
elsif (!$_[0]->{'enabled'} && $out =~ /printing is enabled/)
	{ $self->backquote_logged("lpc down $esc"); }
if ($_[0]->{'accepting'} && $out !~ /queuing is enabled/)
	{ $self->backquote_logged("lpc enable $esc"); }
elsif (!$_[0]->{'accepting'} && $out =~ /queuing is enabled/)
	{ $self->backquote_logged("lpc disable $esc"); }
}

# sched_running()
# Returns the pid if lpsched is running, 0 if not, -1 if cannot be stopped
sub sched_running
{
	my $self = shift;

@{$self->{'pid_array'}} = $self->find_byname("lpd");
if (@{$self->{'pid_array'}}) { return $self->{'pid_array'}->[0]; }
return 0;
}

# start_sched()
# Start lpsched
sub start_sched
{
	my $self = shift;

local $out = $self->backquote_logged("lpd 2>&1");
if ($?) { $self->error("failed to start lpd : <tt>$out</tt>"); }
}

# stop_sched(pid)
# Stop the running lpsched process
sub stop_sched
{
	my $self = shift;

local @pid = $self->find_byname("lpd");
$self->kill_logged('TERM', @pid) || $self->error("Failed to stop lpd : $!");
}

# print_command(printer, file)
# Returns the command to print some file on some printer
sub print_command
{
	my $self = shift;

local $esc = quotemeta($_[0]);
local $fesc = quotemeta($_[1]);
return "lpr -P$esc $fesc";
}

# check_print_system()
sub check_print_system
{
	my $self = shift;

$self->has_command("lpr") || return $self->text('freebsd_ecmd', "<tt>lpr</tt>");
-d $self->{'config_hash'}->{'spool_dir'} || return $self->text('freebsd_espool', "<tt>$self->{'config_hash'}->{'spool_dir'}</tt>");
return undef;
}


1;

