#!/usr/bin/perl
package lpadmin::HpuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'lpadmin::HpuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# hpux-lib.pl
# Functions for hpux-style printer management

$self->{'hpux_iface_path'} = "/usr/spool/lp/interface";

@{$self->{'device_files_array'}} = ("/dev/c1t0d0_lp", "/dev/c2t0d0_lp",
		 "/dev/c0p0_lp", "/dev/c0p1_lp",
		 "/dev/c0p2_lp", "/dev/c0p3_lp",
		 "/dev/c0p4_lp", "/dev/c0p4_lp",
		 "/dev/c0p5_lp", "/dev/c0p5_lp",
		 "/dev/c1p0_lp",
		 "/dev/null");
@{$self->{'device_names_array'}} = ($self->text('hpux_paralel', "c1t0d0"), $self->text('hpux_paralel', "c2t0d0"),
		 $self->text('hpux_serial', "c0p0"), $self->text('hpux_serial', "c0p1"),
		 $self->text('hpux_serial', "c0p2"), $self->text('hpux_serial', "c0p3"),
		 $self->text('hpux_serial', "c0p4"), $self->text('hpux_serial', "c0p5"),
		 $self->text('hpux_serial', "c0p6"), $self->text('hpux_serial', "c0p7"),
		 $self->text('hpux_serial', "c1p0"), $self->{'text_hash'}->{'hpux_null'});
1;
		return $self;
}

# list_printers()
# Returns an array of known printer names
sub list_printers
{
	my $self = shift;

local(@rv, $_);
open(STAT, "lpstat -p |");
while(<STAT>) {
	if (/^printer\s+(\S+)/) { push(@rv, $1); }
	}
close(STAT);

# make models executable
`chmod 755 $self->{'config_hash'}->{'model_path'}/*`;

return $self->unique(@rv);
}

# get_printer(name, [nostatus])
# Returns a reference to an associative array of printer details
sub get_printer
{
	my $self = shift;

local($stat, @rv, $body1, $body2, $avl, $con, $sys, %prn, $_, $out);
local $esc = quotemeta($_[0]);
$out = `lpstat -p$esc`;
if ($out =~ /^printer\s+(\S+)\s+(.*)\n(.*)\n(.*)$/) {
	# printer exists
	$prn{'name'} = $1;
	if ($2 =~ /enabled/) {$prn{'enabled'} = "enabled"};
        $body1 = $3;
        $body2 = $4;
	}
else {
	# no printer found
	return undef;
	}

# Interface
$prn{'iface'} = "$self->{'hpux_iface_path'}/$prn{'name'}";

# Description
local $wdrv = $self->is_windows_driver($prn{'iface'});
local $hdrv = $self->is_hpnp_driver($prn{'iface'});
$prn{'desc'} = $wdrv ? $self->grep_interface($wdrv->{'program'}) :
	       $hdrv ? $self->grep_interface($hdrv->{'program'}) :
		       $self->grep_interface($prn{'iface'});

# printer enabled?
if (!$prn{'enabled'} && $body1 =~ /^\s+(.*)/) {
	$prn{'enabled_why'} = $1 eq "reason unknown" ? "" : $1;
	}

if (!$_[1]) {
	# request availability
	$avl = `lpstat -a$esc 2>&1`;
	if ($avl =~ /^\S+\s+not accepting.*\n\s+(.*)/) {
		$prn{'accepting'} = 0;
		$prn{'accepting_why'} = $1;
		if ($prn{'accepting_why'} eq "reason unknown") {
			$prn{'accepting_why'} = "";
			}
		}
	else { $prn{'accepting'} = 1; }
	}

# request connection
$con = `lpstat -v$esc 2>&1`;
if ($con =~ /^device for \S+:\s+(\S+)\n\s+(remote to:)\s+(\S+)\s+(on)\s+(\S+)/) {
	$prn{'rhost'} = $5;
	$prn{'rqueue'} = $3;
	}
elsif ($con =~ /^device for \S+:\s+(\S+)/) { $prn{'dev'} = $1; }

# Check if this is the default printer
`lpstat -d 2>&1` =~ /destination: (\S+)/;
if ($1 eq $prn{'name'}) { $prn{'default'} = 1; }

return \%prn;
}

sub get_jobs
{
	my $self = shift;

local $esc = quotemeta($_[0]);
open(STAT, "lpstat -o$esc |");
local($id, $user, $prio, $when, $printing);
while(<STAT>) {
	if (/^(\S+-\d+)\s+(\S+)\s+priority\s+(\S+)\s+(\S+.*\d+ \d+:\d+)\s+(.*)/) {
		$id = $1;
		$user = $2;
		$prio = $3;
 		$when = $4;
		$printing = ($5 =~ /^on /);
		}
	elsif (/^(\S+-\d+)\s+(\S+)\s+priority\s+(\S+)\s+from\s+(\S+)\s+(.*)/) {
		$id = $1;
		$user = "$2\@$4";
		$prio = $3;
		$when = "?";
		$printing = ($5 =~ /^on /);
		}
	elsif (/^(.*)\s+(\d+) bytes/) {
		local(%job);
		$job{'id'} = $id;
		$job{'user'} = $user;
		$job{'prio'} = $prio;
	 	$job{'when'} = $when;
		$job{'printing'} = $printing;
		$job{'file'} = $1;
 		$job{'size'} = $2;
		push(@{$self->{'jobs_array'}}, \%job);
		}
	}
close(STAT);
return @{$self->{'jobs_array'}};
}

# grep_interface($iface)
# try to find a description in the interface/model
sub grep_interface
{
	my $self = shift;

local($iface, $desc, $out);
$iface = $_[0];

local $drv = $self->is_driver($iface);
if ($drv->{'mode'} == 1) {
	$desc = $drv->{'desc'};
	}
elsif ($drv->{'mode'} == 2) {
	$out = `head $drv->{'prog'} | grep -e interface  -e Printer -e /model/`;
	if ($out =~ /interface for\s+(.*)/) { $desc = $1; }
	elsif ($out =~ /\s+(\S.*)interface/) { $desc = $1; }
	elsif ($out =~ /Printer Command Language level\s+(\S+)/) { $desc = "PCL$1"; }
	elsif ($out =~ /\/model\/(\S+).*/) { $desc = $1; }
	elsif ($out =~ /^#\s+(.*)/) { $desc = $1; }
	else { $desc = "None"; }
	}
else {
	$desc = "None";
	}
return $desc;
}

# printer_support(option)
sub printer_support
{
	my $self = shift;

return $_[0] !~ /^(allow|alias|ctype|banner|desc|editdest|msize|direct|rnoqueue|ipp)$/;
}

# list_classes()
# Returns an associative array of print classes
sub list_classes
{
	my $self = shift;

local($stat, %rv);
$stat = `lpstat -c 2>&1`;
while($stat =~ /^members of class (\S+):\n((\s+\S+\n)+)([\000-\377]*)$/) {
	$stat = $4;
	$rv{$1} = [ grep { $_ ne "" } split(/\s+/, $2) ];
	}
return \%rv;
}

# create_printer(&details)
# Create a new printer
sub create_printer
{
	my $self = shift;

local(%prn, $cmd, $out, $model, $dummy, $scheduler);
%prn = %{$_[0]};
local $wdrv = $self->is_windows_driver($prn{'iface'});
local $hdrv = $self->is_hpnp_driver($prn{'iface'});
$scheduler = $self->sched_running();
$dummy = "webmin.tmp"; `touch $self->{'config_hash'}->{'model_path'}/$dummy`;

# create lpadmin command
local $esc = quotemeta($prn{'name'});
$cmd = "lpadmin -p$esc";

## remote unix printer
if ($prn{'rhost'}) {
	if ($prn{'iface'}) {
		$self->error("lpadmin failed : <pre>No model allowed for remote unix printer.</pre>");
	}
	$cmd .= " -orm".quotemeta($prn{'rhost'});
	$cmd .= " -orp".quotemeta($prn{'rqueue'});
	$cmd .= " -mrmodel";
	$cmd .= " -ocmrcmodel";
	$cmd .= " -osmrsmodel";
	$cmd .= " -v/dev/null";
	$cmd .= " -orc";
	if ($prn{'rtype'} =~ /^BSD$/) {
		$cmd .= " -ob3";
		}
	}

## remote windows printer
elsif ($wdrv) {
	$cmd .= " -m$dummy";
	$cmd .= " -v/dev/null";
	$cmd .= " -g0";
	}

## remote HPNP printer
elsif ($hdrv) {
	$cmd .= " -m$dummy";
	$cmd .= " -v/dev/null";
	$cmd .= " -g0";
	}

## local printer with webmin driver
elsif ($prn{'iface'} eq "$self->{'drivers_directory'}/$prn{'name'}") {
	$cmd .= " -m$dummy";
	$cmd .= " -v".quotemeta($prn{'dev'});
	$cmd .= " -g0";
	}

## local printer with HP-UX model
elsif ($prn{'iface'} =~ $self->{'config_hash'}->{'model_path'}) {
	$model = substr($prn{'iface'}, length($self->{'config_hash'}->{'model_path'}) + 1);
	$cmd .= " -m$model";
	$cmd .= " -v".quotemeta($prn{'dev'});
	$cmd .= " -g0";
	}
else {
	$self->error("lpadmin failed : <pre>Action not supported.</pre>");
	}

# stop scheduler
$out = $self->backquote_logged("lpshut 2>&1");

# call lpadmin
$out = $self->backquote_logged("$cmd 2>&1");
if ($?) { $self->error("lpsched failed : <pre>$out</pre>"); }

## Link to windows webmin driver
$self->lock_file("$self->{'hpux_iface_path'}/$prn{'name'}");
if ($wdrv) {
	`rm $self->{'hpux_iface_path'}/$esc`;
	`ln -s $self->{'drivers_directory'}/$esc.smb $self->{'hpux_iface_path'}/$esc`;
	}

## Link to webmin hpnp driver
if ($hdrv) {
	`rm $self->{'hpux_iface_path'}/$esc`;
	`ln -s $self->{'drivers_directory'}/$esc.hpnp $self->{'hpux_iface_path'}/$esc`;
	}

## Link to webmin driver
if ($prn{'iface'} eq "$self->{'drivers_directory'}/$prn{'name'}" && !$wdrv) {
	`rm $self->{'hpux_iface_path'}/$esc`;
	`ln -s $self->{'drivers_directory'}/$esc $self->{'hpux_iface_path'}/$esc`;
	}
$self->unlock_file("$self->{'hpux_iface_path'}/$prn{'name'}");
$self->lock_file("$self->{'config_hash'}->{'model_path'}/$dummy");
`rm $self->{'config_hash'}->{'model_path'}/$dummy`;
$self->unlock_file("$self->{'config_hash'}->{'model_path'}/$dummy");

# start scheduler
if ($scheduler) {
	$out = $self->backquote_logged("lpsched 2>&1");
	if ($?) { $self->error("lpsched failed : <pre>$out</pre>"); }
	}

$self->modify_printer($_[0]);
}

# modify_printer(&details)
# Change an existing printer
sub modify_printer
{
	my $self = shift;

local(%prn, $cmd, $out);
%prn = %{$_[0]};

# make the default
local $esc = quotemeta($prn{'name'});
if ($prn{'default'}) {
	$out = $self->backquote_logged("lpadmin -d$esc 2>&1");
	if ($?) { $self->error("Failed to set default : <pre>$out</pre>"); }
	}

# enable or disable
if ($prn{'enabled'}) { $cmd = "enable $esc"; }
elsif ($prn{'enabled_why'}) {
	local $wesc = quotemeta($prn{'enabled_why'});
	$cmd = "enable $esc ; disable -r$wesc $esc";
	}
else { $cmd = "enable $esc ; disable $esc"; }
$out = $self->backquote_logged("$cmd 2>&1");

# accepting or rejecting requests
if ($prn{'accepting'}) { $cmd = "accept $esc"; }
elsif ($prn{'accepting_why'}) {
	local $wesc = quotemeta($prn{'accepting_why'});
	$cmd = "accept $esc ; reject -r$wesc $esc";
	}
else { $cmd = "accept $esc ; reject $esc"; }
$out = $self->backquote_logged("$cmd 2>&1");
}

# delete_printer(name)
# Deletes some existing printer
sub delete_printer
{
	my $self = shift;

local($out, $scheduler);
$scheduler = $self->sched_running();

# delete print jobs
local $esc = quotemeta($_[0]);
$out = $self->backquote_logged("cancel $esc -a 2>1");
if ($?) { $self->error("cancel failed : <pre>$out</pre>"); }

# stop scheduler
$out = $self->backquote_logged("lpshut 2>&1");

# call lpadmin
$out = $self->backquote_logged("lpadmin -x$esc 2>&1");
if ($?) { $self->error("lpadmin failed : <pre>$out</pre>"); }

# start scheduler
if ($scheduler) {
	$out = $self->backquote_logged("lpsched 2>&1");
	if ($?) { $self->error("lpsched failed : <pre>$out</pre>"); }
	}
}

# cancel_job(printer, id)
# Cancels some print job
sub cancel_job
{
	my $self = shift;

local($out);
local $esc = quotemeta($_[1]);
$out = $self->backquote_logged("cancel $esc 2>&1");
if ($?) { $self->error("cancel failed : <pre>$out</pre>"); }
sleep(1);
}

# sched_running()
# Returns 1 if running and 0 if not running
sub sched_running
{
	my $self = shift;

local $out = `lpstat -r 2>&1`;
if ($out =~ /not/) { return 0; }
else { return 1; }
}

# start_sched()
# Start lpsched
sub start_sched
{
	my $self = shift;

local $out = $self->backquote_logged("lpsched 2>&1");
if ($?) { $self->error("failed to start lpsched : <tt>$out</tt>"); }
}

# stop_sched(pid)
# Stop the running lpsched process
sub stop_sched
{
	my $self = shift;

local $out = $self->backquote_logged("lpshut 2>&1");
if ($?) { $self->error("lpshut failed : <tt>$out</tt>"); }
}

# print_command(printer, file)
# Returns the command to print some file on some printer
sub print_command
{
	my $self = shift;

local $esc = quotemeta($_[0]);
local $iesc = quotemeta($_[1]);
return "lp -d $esc $iesc";
}

# check_print_system()
sub check_print_system
{
	my $self = shift;

$self->has_command("lpstat") || return $self->text('hpux_ecmd', "<tt>lpstat</tt>");
-d $self->{'hpux_iface_path'} || return $self->text('hpux_eiface',"<tt>$self->{'hpux_iface_path'}</tt>");
return undef;
}


1;

