#!/usr/bin/perl
package lpadmin::Printconfdriver;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'lpadmin::Printconfdriver');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# printconf-driver.pl
# Functions for printer drivers as created by redhat 7.1

$self->{'mf_wrapper'} = "/usr/share/printconf/mf_wrapper";
@{$self->{'overview_files_array'}} = ( "/usr/share/printconf/foomatic/data/00-overview.foo",
		    "/usr/share/printconf/foomatic/rh-data/00-overview.foo" );
$self->{'mfomatic'} = "/usr/share/printconf/foomatic/mfomatic";
$self->{'ps_wrapper'}= "/usr/share/printconf/util/mf_postscript_wrapper";
$self->{'text_filter'} = "pipe/postscript/ /usr/bin/mpage -b ifdef(`PAGEsize', PAGEsize, Letter) -1 -o -P- -";
$self->{'default_filter'} = "cat/text/";
$self->{'smb_driver'} = "/usr/share/printconf/smbprint";

@{$self->{'paper_sizes_array'}} = ( 'Letter', 'Legal', 'Executive',
		 'A5', 'A4', 'A3' );

1;
		return $self;
}

# is_windows_driver(path, &printer)
# Returns a driver structure if some path is a windows driver
sub is_windows_driver
{
	my $self = shift;

local $sd = "$self->{'config_hash'}->{'spool_dir'}/$_[1]->{'name'}";
if ($_[1]->{'dev'} eq "|$self->{'smb_driver'}" && -r "$sd/script.cfg") {
	# Looks like a redhat SMB driver
	local %sconfig;
	$self->read_env_file("$sd/script.cfg", \%sconfig);
	$sconfig{'share'} =~ /^\\\\(.*)\\(.*)$/;
	return { 'server' => $1,
		 'share' => $2,
		 'user' => $sconfig{'user'},
		 'pass' => $sconfig{'password'},
		 'workgroup' => $sconfig{'workgroup'},
		 'program' => $_[0] };
	}
return undef;

}

# is_driver(path, &printer)
# Returns a structure containing the details of a driver
# XXX do we need to support old redhat drivers?
sub is_driver
{
	my $self = shift;

if (!$_[0]) {
	return { 'mode' => 0,
		 'desc' => "$self->{'text_hash'}->{'redhat_none'}" };
	}
local $sd = "$self->{'config_hash'}->{'spool_dir'}/$_[1]->{'name'}";
if ($_[0] eq $self->{'mf_wrapper'} && -r "$sd/mf.cfg") {
	# Looks like a new printconf driver
	local %define;
	$self->read_m4_file("$sd/mf.cfg", \%define);
	if ($define{'PSfilter'} =~
	    /^filter\s+$self->{'ps_wrapper'}.*\s(\S+)-(\d+)\.foo$/) {
		# Has printer driver
		return { 'mode' => 1,
			 'driver' => $1,
			 'id' => $2,
			 'paper' => $define{'PAGEsize'},
			 'make' => $define{'MAKE'},
			 'model' => $define{'MODEL'},
			 'desc' => "$define{'MAKE'} $define{'MODEL'}" };
		}
	elsif ($define{'PSfilter'} =~ /^filter\s+$self->{'ps_wrapper'}\s*$/) {
		# Postscript printer
		return { 'mode' => 1,
			 'postscript' => 1,
			 'paper' => $define{'PAGEsize'},
			 'desc' => 'Postscript printer' };
		}
	elsif ($define{'PSfilter'} eq 'text' &&
	       $define{'TEXTfilter'} eq 'text') {
		# Text only printer
		return { 'mode' => 1,
			 'text' => 1,
			 'desc' => 'Text printer' };
		}
	}

# Some other kind of driver
return { 'mode' => 2,
	 'file' => $_[0],
	 'desc' => $_[0] };
}

# read_m4_file(file, &hash)
sub read_m4_file
{
	my $self = shift;

open(CFG, $_[0]);
while(<CFG>) {
	s/#.*$//; s/\r|\n//g;
	if (/^\s*define\(([A-Za-z0-9\_]+)\s*,\s*`([^']*)'\)/) {
		$_[1]->{$1} = $2;
		}
	elsif (/^\s*define\(([A-Za-z0-9\_]+)\s*,\s*([^\s\)]+)\)/) {
		$_[1]->{$1} = $2;
		}
	}
close(CFG);
}

# create_windows_driver(&printer, &driver)
# Creates a new windows printer driver
sub create_windows_driver
{
	my $self = shift;

local $sd = "$self->{'config_hash'}->{'spool_dir'}/$_[0]->{'name'}";

# Create the config file and driver
local %sconfig;
$self->read_file("$sd/script.cfg", \%sconfig);
$sconfig{'share'} = "\\\\$_[1]->{'server'}\\$_[1]->{'share'}";
$sconfig{'hostip'} = $_[1]->{'server'} eq $self->{'config_hash'}->{'hostip'} ?
			$sconfig{'hostip'} : undef;
$sconfig{'user'} = $_[1]->{'user'};
$sconfig{'password'} = $_[1]->{'pass'};
$sconfig{'workgroup'} = $_[1]->{'workgroup'};
$self->lock_file($sd);
mkdir($sd, 0755);
$self->unlock_file($sd);
$self->open_lock_tempfile(ENV, ">$sd/script.cfg");
$self->print_tempfile(ENV, "share='$sconfig{'share'}'\n");
$self->print_tempfile(ENV, "hostip=$sconfig{'hostip'}\n");
$self->print_tempfile(ENV, "user='$sconfig{'user'}'\n");
$self->print_tempfile(ENV, "password='$sconfig{'password'}'\n");
$self->print_tempfile(ENV, "workgroup='$sconfig{'workgroup'}'\n");
$self->close_tempfile(ENV);

# Set the print device
$_[0]->{'dev'} = "|$self->{'smb_driver'}";
return $_[1]->{'program'};
}

# create_driver(&printer, &driver)
# Creates a new local printer driver and returns the path
sub create_driver
{
	my $self = shift;

local ($prn, $drv) = @_;
if ($drv->{'mode'} == 0) {
	return undef;
	}
elsif ($drv->{'mode'} == 2) {
	return $drv->{'file'};
	}
else {
	# Create the config file
	local $sd = "$self->{'config_hash'}->{'spool_dir'}/$_[0]->{'name'}";
	mkdir($sd, 0755);
	$self->lock_file("$sd/mf.cfg");
	if ($drv->{'text'}) {
		%{$self->{'driver_hash'}} = ( 'MAKE', '',
			    'MODEL', '',
			    'COLOR', '',
			    'PAGEsize', $_[1]->{'paper'},
			    'TEXTfilter', 'text',
			    'PSfilter', 'text',
			    'PCLfilter', 'reject',
			    'PJLfilter', 'reject',
			    'DEFAULTfilter', $self->{'default_filter'} );
		}
	elsif ($drv->{'postscript'}) {
		%{$self->{'driver_hash'}} = ( 'MAKE', '',
			    'MODEL', '',
			    'COLOR', '',
			    'PAGEsize', $_[1]->{'paper'},
			    'TEXTfilter', $self->{'text_filter'},
			    'PSfilter', "filter $self->{'ps_wrapper'} ",
			    'PCLfilter', 'cat',
			    'PJLfilter', 'cat',
			    'DEFAULTfilter', $self->{'default_filter'} );
		}
	else {
		local $base = "$drv->{'driver'}-$drv->{'id'}.foo";
		local $path;
		foreach $f (@{$self->{'overview_files_array'}}) {
			local $d = $f; $d =~ s/\/[^\/]+$//;
			$path = "$d/$base" if (-r "$d/$base");
			}
		$self->system_logged("cp $path $sd");
		$self->read_m4_file("$sd/mf.cfg", \%{$self->{'driver_hash'}});
		$self->{'driver_hash'}->{'MAKE'} = $drv->{'make'};
		$self->{'driver_hash'}->{'MODEL'} = $drv->{'model'};
		$self->{'driver_hash'}->{'COLOR'} = 'true';	# ???
		$self->{'driver_hash'}->{'PSfilter'} = "filter $self->{'ps_wrapper'} --mfomatic -d $base";
		$self->{'driver_hash'}->{'TEXTfilter'} = $self->{'text_filter'};
		$self->{'driver_hash'}->{'PCLfilter'} = 'cat';
		$self->{'driver_hash'}->{'PJLfilter'} = 'cat';
		$self->{'driver_hash'}->{'PAGEsize'} = $_[1]->{'paper'};
		$self->{'driver_hash'}->{'DEFAULTfilter'} = $self->{'default_filter'};
		}
	$self->open_tempfile(CFG, ">$sd/mf.cfg");
	foreach $d (keys %{$self->{'driver_hash'}}) {
		if ($self->{'driver_hash'}->{$d} =~ /`|'/) {
			$self->print_tempfile(CFG, "define($d, $self->{'driver_hash'}->{$d})dnl\n");
			}
		else {
			$self->print_tempfile(CFG, "define($d, `$self->{'driver_hash'}->{$d}')dnl\n");
			}
		}
	$self->close_tempfile(CFG);
	$self->unlock_file("$sd/mf.cfg");
	return $self->{'mf_wrapper'};
	}
}

# delete_driver(name)
sub delete_driver
{
	my $self = shift;

local $sd = "$self->{'config_hash'}->{'spool_dir'}/$_[0]";
unlink("$sd/mf.cfg", "$sd/account");
}

# driver_input(&printer, &driver)
sub driver_input
{
	my $self = shift;

local ($prn, $drv) = @_;

printf "<tr> <td><input type=radio name=mode value=0 %s> %s</td>\n",
	$drv->{'mode'} == 0 ? "checked" : "", $self->{'text_hash'}->{'redhat_none'};
print "<td>($self->{'text_hash'}->{'redhat_nonemsg'})</td> </tr>\n";

printf "<tr> <td><input type=radio name=mode value=2 %s> %s</td>\n",
	$drv->{'mode'} == 2 ? "checked" : "", $self->{'text_hash'}->{'redhat_prog'};
printf "<td><input name=iface value=\"%s\" size=35></td> </tr>\n",
	$drv->{'mode'} == 2 ? $drv->{'file'} : "";

printf "<tr> <td valign=top><input type=radio name=mode value=1 %s>\n",
	$drv->{'mode'} == 1 ? "checked" : "";
print "$self->{'text_hash'}->{'redhat_driver'}</td> <td><table width=100%>";

print "<tr> <td valign=top><b>$self->{'text_hash'}->{'redhat_printer2'}</b></td>\n";
print "<td><select name=printer size=10 onChange='setdriver(0)'>\n";
printf "<option value=%s %s>%s\n",
	'text', $drv->{'text'} ? 'selected' : '', "Text printer";
printf "<option value=%s %s>%s\n",
	'postscript', $drv->{'postscript'} ? 'selected' : '',
	"Postscript printer";

local $list = $self->list_printconf_drivers();
local $select_driver = 0;
foreach $k (sort { lc($a) cmp lc($b) } keys %$list) {
	foreach $m (@{$list->{$k}}) {
		next if (!@{$m->{'drivers'}});
		printf "<option value='%s' %s>%s\n",
			join(";", $m->{'id'}, @{$m->{'drivers'}}),
			$drv->{'make'} eq $k &&
			$drv->{'model'} eq $m->{'model'} ? 'selected' : '',
			"$k $m->{'model'}";
		if ($drv->{'make'} eq $k && $drv->{'model'} eq $m->{'model'}) {
			$select_driver = $self->indexof($drv->{'driver'},
						  @{$m->{'drivers'}});
			}
		}
	}
print "</select><select name=driver size=10 width=100></select></td> </tr>\n";

local $foundp = 0;
print "<tr> <td><b>$self->{'text_hash'}->{'redhat_paper'}</b></td> <td><select name=paper>\n";
print "<option value='' selected>$self->{'text_hash'}->{'redhat_none'}\n" if (!$drv->{'paper'});
foreach $p (@{$self->{'paper_sizes_array'}}) {
	printf "<option %s>%s\n",
		$drv->{'paper'} eq $p ? 'selected' : '', $p;
	$foundp++ if ($drv->{'paper'} eq $p);
	}
printf "<option selected>$drv->{'paper'}\n" if (!$foundp && $drv->{'paper'});
print "</select></td> </tr>\n";

print "</table></td></tr>\n";
return <<EOF;
<script>
function setdriver(sel)
{
var idx = document.forms[0].printer.selectedIndex;
var v = new String(document.forms[0].printer.options[idx].value);
var vv = v.split(";");
var driver = document.forms[0].driver;
driver.length = 0;
for(var i=1; i<vv.length; i++) {
	driver.options[i-1] = new Option(vv[i], vv[i]);
	}
if (driver.length > 0) {
	driver.options[sel].selected = true;
	}
}
setdriver($select_driver);
</script>
EOF
}

# parse_driver()
# Parse driver selection from %{$self->{'in_hash'}} and return a driver structure
sub parse_driver
{
	my $self = shift;

if ($self->{'in_hash'}->{'mode'} == 0) {
	return { 'mode' => 0 };
	}
elsif ($self->{'in_hash'}->{'mode'} == 2) {
	(-x $self->{'in_hash'}->{'iface'}) || $self->error($self->text('redhat_eprog', $self->{'in_hash'}->{'iface'}));
	return { 'mode' => 2,
		 'file' => $self->{'in_hash'}->{'iface'} };
	}
else {
	$self->{'in_hash'}->{'printer'} || $self->error($self->{'text_hash'}->{'redhat_edriver'});
	if ($self->{'in_hash'}->{'printer'} eq 'text') {
		return { 'mode' => 1,
			 'paper' => $self->{'in_hash'}->{'paper'},
			 'text' => 1 };
		}
	elsif ($self->{'in_hash'}->{'printer'} eq 'postscript') {
		return { 'mode' => 1,
			 'paper' => $self->{'in_hash'}->{'paper'},
			 'postscript' => 1 };
		}
	else {
		$self->{'in_hash'}->{'driver'} || $self->error($self->{'text_hash'}->{'redhat_edriver'});
		local @p = split(/;/, $self->{'in_hash'}->{'printer'});
		local $list = $self->list_printconf_drivers();
		local ($make, $model);
		foreach $k (keys %$list) {
			foreach $m (@{$list->{$k}}) {
				if ($m->{'id'} == $p[0]) {
					$make = $k;
					$model = $m->{'model'};
					}
				}
			}
		return { 'mode' => 1,
			 'paper' => $self->{'in_hash'}->{'paper'},
			 'id' => $p[0],
			 'make' => $make,
			 'model' => $model,
			 'driver' => $self->{'in_hash'}->{'driver'} };
		}
	}
}

# list_printconf_drivers()
sub list_printconf_drivers
{
	my $self = shift;

local %rv;
foreach $ov (@{$self->{'overview_files_array'}}) {
	next if (!-r $ov);
	local $VAR1;
	eval { {
    my $pkg = $self->lib_to_pkg($ov , "lpadmin");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}};
	next if ($@);
	foreach $k (keys %$VAR1) {
		foreach $m (@{$VAR1->{$k}}) {
			push(@{$rv{$k}}, $m);
			}
		}
	}
return \%rv;
}


1;

