#!/usr/bin/perl
package lpadmin::Solarisdriver;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'lpadmin::Solarisdriver');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# solaris-driver.pl
# Functions for Solaris 10 printer drivers

$self->{'webmin_windows_driver'} = 1;
$self->{'cups_ppd_dir'} = "/etc/lp/ppd";
$self->{'lpoptions'} = $self->has_command("lpoptions.cups") ? "lpoptions.cups" : "lpoptions";

1;

1;
		return $self;
}

# is_windows_driver(path, &printer)
# Returns the server, share, username, password, workgroup, program
# if path is a webmin windows driver
sub is_windows_driver
{
	my $self = shift;

return $self->is_webmin_windows_driver(@_);
}

# is_driver(path, &printer)
# Returns the driver name if some path is a Solaris driver, or undef
sub is_driver
{
	my $self = shift;

local $ppd = $_[1]->{'ppd'} ? $self->parse_cups_ppd($_[1]->{'ppd'}) : undef;
if ($ppd && $ppd->{'NickName'}) {
	# Printer has a PPD file
	return { 'mode' => 1,
		 'manuf' => $ppd->{'Manufacturer'},
		 'model' => $ppd->{'ModelName'},
		 'nick' => $ppd->{'NickName'},
		 'desc' => $ppd->{'NickName'} };
	}
elsif ($_[1]->{'iface'}) {
	# Some other kind of interface file
	return { 'mode' => 2,
		 'file' => $_[1]->{'iface'},
		 'desc' => $_[1]->{'iface'} };
	}
else {
	# No driver
	return { 'mode' => 0,
		 'desc' => $self->{'text_hash'}->{'cups_none'} };
	}
}

# create_windows_driver(&printer, &driver)
sub create_windows_driver
{
	my $self = shift;

return $self->create_webmin_windows_driver(@_);
}

# create_driver(&printer, &driver)
sub create_driver
{
	my $self = shift;

local $drv = "$self->{'cups_ppd_dir'}/$_[0]->{'name'}.ppd";
if ($_[1]->{'mode'} == 0) {
	# No driver
	$self->system_logged("rm -f \"$drv\"");
	$_[0]->{'ppd'} = undef;
	return undef;
	}
elsif ($_[1]->{'mode'} == 2) {
	# A separate interface program
	$self->system_logged("rm -f \"$drv\"");
	$_[0]->{'ppd'} = undef;
	return $_[1]->{'file'};
	}
else {
	# A PPD driver, which replaces any interface program
	$_[0]->{'ppd'} = $_[1]->{'ppd'};
	return undef;
	}
}

# delete_driver(name)
sub delete_driver
{
	my $self = shift;

$self->system_logged("rm -f \"$self->{'cups_ppd_dir'}/$_[0].ppd\"");
}

# driver_input(&printer, &driver)
sub driver_input
{
	my $self = shift;

printf "<tr> <td><input type=radio name=mode value=0 %s> %s</td>\n",
	$_[1]->{'mode'} == 0 ? 'checked' : '', $self->{'text_hash'}->{'cups_none'};
print "<td>($self->{'text_hash'}->{'cups_nonemsg'})</td> </tr>\n";
printf "<tr> <td><input type=radio name=mode value=2 %s> %s</td>",
	$_[1]->{'mode'} == 2 ? 'checked' : '', $self->{'text_hash'}->{'cups_prog'};
printf "<td><input name=program size=40 value='%s'></td> </tr>\n",
	$_[1]->{'mode'} == 2 ? $_[0]->{'iface'} : '';

# Display all the CUPS drivers
printf "<tr> <td valign=top><input type=radio name=mode value=1 %s> %s</td>\n",
	$_[1]->{'mode'} == 1 ? 'checked' : '', $self->{'text_hash'}->{'cups_driver'};
print "<td><select name=ppd size=10>\n";
local (@ppds, $d, $f, $ppd, %cache, $outofdate, @files, %donefile);
open(FIND, "find '$self->{'config_hash'}->{'model_path'}' -type f -print |");
while(<FIND>) {
	chop;
	/([^\/]+)$/;
	next if ($donefile{$1}++);
	push(@files, $_);
	}
close(FIND);
$self->read_file("$self->{'module_config_directory'}/ppd-cache", \%cache);
foreach $f (@files) {
	if (!defined($cache{$f})) {
		$outofdate = $f;
		last;
		}
	}
if ($outofdate || scalar(keys %cache) != scalar(@files)) {
	# Cache is out of date
	undef(%cache);
	local %donecache;
	foreach $f (@files) {
		local $ppd = $self->parse_cups_ppd($f);
		$cache{$f} = $ppd->{'NickName'}
			if (!$donecache{$ppd->{'NickName'}}++);
		}
	$self->write_file("$self->{'module_config_directory'}/ppd-cache", \%cache);
	}
local %done;
foreach $f (sort { $cache{$a} cmp $cache{$b} } keys %cache) {
	if ($cache{$f} && !$done{$cache{$f}}++) {
		printf "<option value=%s %s>%s\n",
			$f, $_[1]->{'nick'} eq $cache{$f} ? 'selected' : '',
			$cache{$f};
		$self->{'currppd'} = $f if ($_[1]->{'nick'} eq $cache{$f});
		}
	}
print "</select>\n";
print "</td> </tr>\n";
return undef;
}

# parse_driver()
# Parse driver selection from %{$self->{'in_hash'}} and return a driver structure
sub parse_driver
{
	my $self = shift;

if ($self->{'in_hash'}->{'mode'} == 0) {
	return { 'mode' => 0 };
	}
elsif ($self->{'in_hash'}->{'mode'} == 2) {
	$self->{'in_hash'}->{'program'} =~ /^(\S+)/ && -x $1 ||
		$self->error($self->text('cups_eprog', $self->{'in_hash'}->{'program'}));
	return { 'mode' => 2,
		 'file' => $self->{'in_hash'}->{'program'} };
	}
elsif ($self->{'in_hash'}->{'mode'} == 1) {
	# CUPS printer driver
	local $ppd = $self->parse_cups_ppd($self->{'in_hash'}->{'ppd'});
	local $rv = { 'mode' => 1,
		      'ppd' => $self->{'in_hash'}->{'ppd'},
		      'nick' => $ppd->{'NickName'},
		      'manuf' => $ppd->{'Manufacturer'},
		      'model' => $ppd->{'ModelName'} };
	return $rv;
	}
}


1;

