#!/usr/bin/perl
package mailboxes::MailboxesLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'mailboxes::MailboxesLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# mailboxes-lib.pl
# Common functions for reading user mailboxes

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
&ModuleLoader::preload_module(ref($self), "mailboxes::BoxesLib");
$self = new mailboxes::BoxesLib:: ref($self), %$self; 
&ModuleLoader::preload_module(ref($self), "mailboxes::FoldersLib");
$self = new mailboxes::FoldersLib:: ref($self), %$self; 
$self->init_config();
%{$self->{'access_hash'}} = $self->get_module_acl();
$self->{'config_hash'}->{'perpage'} ||= 20;
$self->{'config_hash'}->{'column_count'} ||= 4;
$self->{'gconfig_hash'}->{'logfiles'} = 0;	# file change logging never needs to be done

# Always detect the mail system if not set
if ($self->{'config_hash'}->{'mail_system'} == 3) {
	$self->{'config_hash'}->{'mail_system'} = $self->detect_mail_system();
	$self->save_module_config() if ($self->{'config_hash'}->{'mail_system'} != 3);
	}

@{$self->{'mail_system_modules_array'}} = (
			 [ undef, 4, \&check_qmail_ldap, \&test_qmail_ldap ],
			 [ undef, 5, \&check_vpopmail ],
			 [ "qmailadmin", 2 ],
			 [ "postfix", 0 ],
			 [ "sendmail", 1 ],
			);

1;

1;
		return $self;
}

# send_mail_program(from, to)
# Returns the command for injecting email, based on the mail system in use
sub send_mail_program
{
	my $self = shift;

if ($self->{'config_hash'}->{'mail_system'} == 1 || $self->{'config_hash'}->{'mail_system'} == 0) {
	# Use sendmail executable, or postfix wrapper
	local %sconfig = $self->foreign_check("sendmail") &&
			 $self->{'config_hash'}->{'mail_system'} == 1 ?
				$self->foreign_config("sendmail") : ( );
	local $cmd = $self->has_command($sconfig{'sendmail_path'} || "sendmail");
	return "$cmd -t -f".quotemeta($_[0]) if ($cmd);
	}
elsif ($self->{'config_hash'}->{'mail_system'} == 2) {
	# Use qmail injector
	local %qconfig = $self->foreign_check("qmailadmin") ?
				$self->foreign_config("qmailadmin") : ( );
	local $cmd = ($qconfig{'qmail_dir'} || "/var/qmail").
		     "/bin/qmail-inject";
	return $cmd if (-x $cmd);
	}
else {
	# Fallback - use sendmail command
	local $cmd = $self->has_command("sendmail");
	return "$cmd -t -f".quotemeta($_[0]) if ($cmd);
	}
return undef;
}

# list_folders()
# Returns a list of all mailboxes for all users
sub list_folders
{
	my $self = shift;

local (@rv, $uinfo);
foreach $uinfo ($self->list_mail_users()) {
	push(@rv, $self->list_user_folders(@$uinfo));
	}
return @rv;
}

# list_user_folders(user, [other info])
# Returns a list of folders for mailboxes belonging to some user
sub list_user_folders
{
	my $self = shift;

if ($_[0] =~ /^\//) {
	# A path .. return a folder just for it
	return ( { 'name' => $_[0],
		   'file' => $_[0],
		   'type' => $self->folder_type($_[0]),
		   'mode' => 1,
		   'index' => 0 } );
	}
else {
	# Getting folders for a user
	local @uinfo = @_ > 1 ? @_ : $self->get_mail_user($_[0]);
	return ( ) if (!@uinfo);
	local ($dir, $style, $mailbox, $maildir) = $self->get_mail_style();
	local @rv;

	# Check for user-specified mail store
	if ($uinfo[10]) {
		push(@rv, { 'name' => $uinfo[10],
			    'file' => $uinfo[10],
			    'type' => $self->folder_type($uinfo[10]),
			    'mode' => 0,
			    'index' => scalar(@rv) } );
		}

	# Check for /var/mail/USERNAME file
	if ($dir) {
		local $mf = $self->mail_file_style($uinfo[0], $dir, $style);
		push(@rv, { 'type' => 0,
			    'name' => $mf,
			    'file' => $mf,
			    'user' => $uinfo[0],
			    'index' => scalar(@rv) });
		}

	# Check for file in home dir
	if ($mailbox) {
		local $mf = "$uinfo[7]/$mailbox";
		if (-r $mf || !@rv) {
			push(@rv, { 'type' => 0,
				    'name' => "~$uinfo[0]/$mailbox",
				    'file' => $mf,
				    'user' => $uinfo[0],
				    'index' => scalar(@rv) });
			}
		}

	# Check for directory in home dir
	if ($maildir) {
		local $mf = "$uinfo[7]/$maildir";
		if (-d $mf || !@rv) {
			push(@rv, { 'type' => 1,
				    'name' => "~$uinfo[0]/$maildir/",
				    'file' => $mf,
				    'user' => $uinfo[0],
				    'index' => scalar(@rv) });
			}
		}

	# Add any ~/mail files
	if ($self->{'config_hash'}->{'mail_usermin'}) {
		local $folders_dir = "$uinfo[7]/$self->{'config_hash'}->{'mail_usermin'}";
		foreach $p ($self->recursive_files($folders_dir, 1)) {
			local $f = $p;
			$f =~ s/^\Q$folders_dir\E\///;
			push(@rv, { 'file' => $p,
				    'name' => "~$uinfo[0]/$self->{'config_hash'}->{'mail_usermin'}/$f",
				    'type' => $self->folder_type($p),
				    'mode' => 0,
				    'sent' => $f eq "sentmail",
				    'index' => scalar(@rv) } );
			}
		}

	# Add any Usermin external mail files
	if ($self->{'config_hash'}->{'mailbox_user'}) {
		local %userconfig;
		$self->read_file_cached("$uinfo[7]/$self->{'config_hash'}->{'mailbox_user'}/config",
				  \%userconfig);
		local $o;
		foreach $o (split(/\t+/, $userconfig{'mailboxes'})) {
			$o =~ /\/([^\/]+)$/ || next;
			push(@rv, { 'name' => $o,
				    'file' => $o,
				    'type' => $self->folder_type($o),
				    'mode' => 1,
				    'index' => scalar(@rv) } );
			}
		}

	foreach my $f (@rv) {
		$f->{'user'} = $_[0];
		}
	return @rv;
	}
}

sub list_user_folders_sorted
{
	my $self = shift;

return $self->list_user_folders(@_);
}

# get_mail_style()
# Returns a list containing the mail base directory, directory style,
# mail file in home dir, and maildir in home dir
sub get_mail_style
{
	my $self = shift;

if (!defined(@{$self->{'mail_style_cache_array'}})) {
	if ($self->{'config_hash'}->{'auto'}) {
		# Based on mail server
		if ($self->{'config_hash'}->{'mail_system'} == 1) {
			# Can get paths from Sendmail module config
			local %sconfig = $self->foreign_config("sendmail");
			if ($sconfig{'mail_dir'}) {
				return ($sconfig{'mail_dir'}, $sconfig{'mail_style'}, undef, undef);
				}
			else {
				return (undef, $sconfig{'mail_style'}, $sconfig{'mail_file'}, undef);
				}
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 0) {
			# Need to query Postfix module for paths
			$self->foreign_require("postfix", "postfix-lib.pl");
			local @s = $main::postfix_object->postfix_mail_system();
			if ($s[0] == 0) {
				return ($s[1], 0, undef, undef);
				}
			elsif ($s[0] == 1) {
				return (undef, 0, $s[1], undef);
				}
			elsif ($s[0] == 2) {
				return (undef, 0, undef, $s[1]);
				}
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 2 ||
		       $self->{'config_hash'}->{'mail_system'} == 4) {
			# Need to check qmail module config for paths
			local %qconfig = $self->foreign_config("qmailadmin");
			if ($qconfig{'mail_system'} == 1) {
				return (undef, 0, undef,
					$qconfig{'mail_dir_qmail'});
				}
			elsif ($qconfig{'mail_dir'}) {
				return ($qconfig{'mail_dir'}, $qconfig{'mail_style'}, undef, undef);
				}
			else {
				return (undef, $qconfig{'mail_style'}, $qconfig{'mail_file'}, undef);
				}
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 5) {
			# vpopmail always uses ~/Maildir
			return ( undef, 0, undef, "Maildir" );
			}
		else {
			# No mail server set yet!
			return (undef, undef, undef, undef);
			}
		}
	else {
		# Use config settings
		@{$self->{'mail_style_cache_array'}} = ($self->{'config_hash'}->{'mail_dir'}, $self->{'config_hash'}->{'mail_style'},
				     $self->{'config_hash'}->{'mail_file'}, $self->{'config_hash'}->{'mail_sub'});
		}
	}
return @{$self->{'mail_style_cache_array'}};
}

# $self->can_user(username, [other details])
sub can_user
{
	my $self = shift;

if (!$self->is_user($_[0])) {
	# For external files, check if the file is under an allowed
	# directory, or owned by an allowed user.
	local @st = stat($_[0]);
	local @uinfo = $self->get_mail_uid($st[4]);
	return 1 if (@uinfo && $self->can_user(@uinfo));
	local $dir = $self->allowed_directory();
	return defined($dir) && $self->is_under_directory($dir, $_[0]);
	}
local @u = @_ > 1 ? @_ : $self->get_mail_user($_[0]);
return 1 if ($_[0] && $self->{'access_hash'}->{'sent'} eq $_[0]);
return 1 if ($self->{'access_hash'}->{'mmode'} == 1);
return 0 if (!@u);
return 0 if ($_[0] =~ /\.\./);
return 0 if ($self->{'access_hash'}->{'mmode'} == 0);
local $u;
if ($self->{'access_hash'}->{'mmode'} == 2) {
	# Is user in list of users?
	foreach $u (split(/\s+/, $self->{'access_hash'}->{'musers'})) {
		return 1 if ($u eq $_[0]);
		}
	return 0;
	}
elsif ($self->{'access_hash'}->{'mmode'} == 4) {
	# Is user the current Webmin user?
	return 1 if ($_[0] eq $self->{'remote_user'});
	}
elsif ($self->{'access_hash'}->{'mmode'} == 5) {
	# Is the user's gid in the list of groups?
	local $gid;
	foreach $gid (split(/\s+/, $self->{'access_hash'}->{'musers'})) {
		return 1 if ($u[3] == $gid);
		if ($self->{'access_hash'}->{'msec'}) {
			# Check user's secondary groups too
			local @ginfo = getgrgid($gid);
			local @m = split(/\s+/, $ginfo[3]);
			return 1 if ($self->indexof($_[0], @m) >= 0);
			}
		}
	}
elsif ($self->{'access_hash'}->{'mmode'} == 3) {
	# Is the user not in the list of denied users
	foreach $u (split(/\s+/, $self->{'access_hash'}->{'musers'})) {
		return 0 if ($u eq $_[0]);
		}
	return 1;
	}
elsif ($self->{'access_hash'}->{'mmode'} == 6) {
	# Does the user match a regexp?
	return ($_[0] =~ /^$self->{'access_hash'}->{'musers'}$/);
	}
elsif ($self->{'access_hash'}->{'mmode'} == 7) {
	# Is the user's UID within the allowed range?
	return (!$self->{'access_hash'}->{'musers'} || $u[2] >= $self->{'access_hash'}->{'musers'}) &&
	       (!$self->{'access_hash'}->{'musers2'} || $u[2] <= $self->{'access_hash'}->{'musers2'});
	}
return 0;	# can't happen!
}

# movecopy_user_select(number, folders, folder, form-no)
# Returns HTML for entering a username to copy mail to
sub movecopy_user_select
{
	my $self = shift;

local $rv;
$rv .= "<input type=submit name=move$_[0] value=\"$self->{'text_hash'}->{'mail_move'}\" ".
       "onClick='return check_clicks(form)'>";
$rv .= "<input type=submit name=copy$_[0] value=\"$self->{'text_hash'}->{'mail_copy'}\" ".
       "onClick='return check_clicks(form)'>";
$rv .= $self->ui_user_textbox("mfolder$_[0]", undef, $_[3]);
return $rv;
}

# need_delete_warn(&folder)
sub need_delete_warn
{
	my $self = shift;

return 1 if ($self->{'config_hash'}->{'delete_warn'} eq 'y');
return 0 if ($self->{'config_hash'}->{'delete_warn'} eq 'n');
local $mf;
return $_[0]->{'type'} == 0 &&
       ($mf = $self->folder_file($_[0])) &&
       $self->disk_usage_kb($mf)*1024 > $self->{'config_hash'}->{'delete_warn'};
}

# detect_mail_system()
# Works out which mail server is installed
sub detect_mail_system
{
	my $self = shift;

foreach $m (@{$self->{'mail_system_modules_array'}}) {
	return $m->[1] if ($self->check_mail_system($m));
	}
return 3;
}

# check_mail_system(&mailsystem)
sub check_mail_system
{
	my $self = shift;

if ($_[0]->[0]) {
	# Just check module
	return $self->foreign_installed($_[0]->[0]);
	}
else {
	# Call function
	local $func = $_[0]->[2];
	return $self->$func($_[0]);
	}
}

# test_mail_system([&mailsystem])
# Returns an error message if the mail system is invalid
sub test_mail_system
{
	my $self = shift;

local $ms;
if (!$ms) {
	($ms) = grep { $_->[1] == $self->{'config_hash'}->{'mail_system'} } @{$self->{'mail_system_modules_array'}};
	}
if ($ms->[3]) {
	local $func = $ms->[3];
	return $self->$func();
	}
return undef;
}

# check_qmail_ldap()
# Make sure Qmail with LDAP extensions is installed
sub check_qmail_ldap
{
	my $self = shift;

return 0 if ($self->foreign_installed("qmailadmin", 1) != 2);
local %qconfig = $self->foreign_config("qmailadmin");
return 0 if (!-r "$qconfig{'qmail_dir'}/control/ldapserver");
return 1;
}

# check_vpopmail()
# Make sure Qmail with VPopMail extensions is installed
sub check_vpopmail
{
	my $self = shift;

return 0 if ($self->foreign_installed("qmailadmin", 1) != 2);
return -x "$self->{'config_hash'}->{'vpopmail_dir'}/bin/vadddomain";
}

# test_qmail_ldap()
# Returns undef the Qmail+LDAP database can be contacted OK, or an error message
sub test_qmail_ldap
{
	my $self = shift;

$self->{'config_hash'}->{'ldap_host'} || return $self->{'text_hash'}->{'ldap_ehost'};
$self->{'config_hash'}->{'ldap_port'} =~ /^\d+$/ || return $self->{'text_hash'}->{'ldap_eport'};
$self->{'config_hash'}->{'ldap_login'} || return $self->{'text_hash'}->{'ldap_euser'};
$self->{'config_hash'}->{'ldap_base'} || return $self->{'text_hash'}->{'ldap_ebase'};
local $err = $self->connect_qmail_ldap(1);
return ref($err) ? undef : $err;
}

# show_users_table(&users-list, [only-with-mail])
# Outputs HTML for a table of users, with the appropriate sorting and mode
sub show_users_table
{
	my $self = shift;

local @users = @{$_[0]};
local ($u, %size, %incount, %sentcount, %foldercount);
if ($self->{'config_hash'}->{'sort_mode'} == 2 || $self->{'config_hash'}->{'show_size'} > 0 || $_[1] ||
    $self->{'config_hash'}->{'show_count'} || $self->{'config_hash'}->{'show_sent'}) {
	# Need to check folders
	foreach $u (@users) {
    		next if ($self->{'config_hash'}->{'ignore_users_enabled'} == 1 &&
			 $self->{'config_hash'}->{'ignore_users'} =~ /$u->[0]/);
		local @folders = $self->list_user_folders(@$u);
		$foldercount{$u->[0]} = scalar(@folders);
		if ($self->{'config_hash'}->{'sort_mode'} == 2 ||
		    $self->{'config_hash'}->{'show_size'} > 0 || $_[1]) {
			# Compute size of folders
			$size{$u->[0]} = $self->{'config_hash'}->{'size_mode'} ?
						$self->folder_size(@folders) :
						$self->folder_size($folders[0]);
			}
		if ($self->{'config_hash'}->{'show_count'}) {
			# Get number of mails in inbox
			$incount{$u->[0]} = $self->mailbox_folder_size($folders[0]);
			}
		if ($self->{'config_hash'}->{'show_sent'}) {
			# Count number of messages in sent mail
			local ($sent) = grep { $_->{'sent'} } @folders;
			$sentcount{$u->[0]} = $self->mailbox_folder_size($sent)
				if ($sent);
			}
		}
	}

# Sort by chosen mode
if ($self->{'config_hash'}->{'sort_mode'} == 2) {
	@users = sort { $size{$b->[0]} <=> $size{$a->[0]} } @users;
	}
elsif ($self->{'config_hash'}->{'sort_mode'} == 1) {
	@users = sort { lc($a->[0]) cmp lc($b->[0]) } @users;
	}

local @allusers = @users;
if ($_[1]) {
	# Limit to those with mail
	@users = grep { $size{$_->[0]} } @users;
	}

# Show table of users
if (!@allusers) {
	print "<b>$self->{'text_hash'}->{'index_nousers'}</b><p>\n";
	}
elsif (!@users) {
	print "<b>$self->{'text_hash'}->{'index_nousersmail'}</b><p>\n";
	}
elsif ($self->{'config_hash'}->{'show_size'} == 2) {
	# Show full user details
	local %uconfig = $self->foreign_config("useradmin");
	local @ccols;
	push(@ccols, $self->{'text_hash'}->{'find_incount'}) if ($self->{'config_hash'}->{'show_count'});
	push(@ccols, $self->{'text_hash'}->{'find_sentcount'}) if ($self->{'config_hash'}->{'show_sent'});
	push(@ccols, $self->{'text_hash'}->{'find_fcount'}) if (%foldercount);
	print $self->ui_columns_start( [ $self->{'text_hash'}->{'find_user'}, $self->{'text_hash'}->{'find_real'},
				   $self->{'text_hash'}->{'find_group'}, $self->{'text_hash'}->{'find_home'},
				   $self->{'text_hash'}->{'find_size'}, @ccols ], 100);
	foreach $u (@users) {
		local $g = getgrgid($u->[3]);
	        next if ($self->{'config_hash'}->{'ignore_users_enabled'} == 1 &&
			 $self->{'config_hash'}->{'ignore_users'} =~ /$u->[0]/);
		$u->[6] =~ s/,.*$// if ($uconfig{'extra_real'});
		local $home = $u->[7];
		if (length($home) > 30) {
			$home = "...".substr($home, -30);
			}
		local @ccols;
		if ($self->{'config_hash'}->{'show_count'}) {
			push(@ccols, int($incount{$u->[0]}))
			}
		if ($self->{'config_hash'}->{'show_sent'}) {
			push(@ccols, int($sentcount{$u->[0]}))
			}
		if (%foldercount) {
			push(@ccols, int($foldercount{$u->[0]}))
			}
		print $self->ui_columns_row(
			[ "<a href='list_mail.cgi?user=$u->[0]'>$u->[0]</a>",
			  $u->[6], $g, $home,
			  $size{$u->[0]} == 0 ? $self->{'text_hash'}->{'index_empty'} :
				$self->nice_size($size{$u->[0]}),
			  @ccols ],
			[ undef, undef, undef, undef, "nowrap" ]);
		}
	print $self->ui_columns_end();
	}
else {
	# Just showing username (and maybe size)
	print $self->ui_table_start($self->{'text_hash'}->{'index_header'}, "width=100%", $self->{'config_hash'}->{'column_count'});
	local $i = 0;
	foreach $u (@users) {
	        next if ($self->{'config_hash'}->{'ignore_users_enabled'} == 1 &&
			 $self->{'config_hash'}->{'ignore_users'} =~ /$u->[0]/);
		print "<tr>\n" if ($i % $self->{'config_hash'}->{'column_count'} == 0);
		print "<td width=", int(100/$self->{'config_hash'}->{'column_count'}), "%><a href='list_mail.cgi?user=$u->[0]'>";
		print $u->[0];
		if ($self->{'config_hash'}->{'show_size'} == 1) {
			local @folders = $self->list_user_folders(@$u);
			local $total = $self->folder_size(@folders);
			if ($size{$u->[0]} > 0) {
				print $self->{'config_hash'}->{'show_size_below'} ? '<BR>' : ' ';
				print "(";
				if (%foldercount) {
					print $self->text('find_in',
						$self->nice_size($size{$u->[0]}),
						$foldercount{$u->[0]});
					}
				else {
					print $self->nice_size($size{$u->[0]});
					}
				print ")";
				}
			}
		print "</a></td>\n";
		print "</tr>\n" if ($i % $self->{'config_hash'}->{'column_count'} == ($self->{'config_hash'}->{'column_count'} - 1));
		$i++;
		}
	if ($i % $self->{'config_hash'}->{'column_count'}) {
		while($i++ % $self->{'config_hash'}->{'column_count'}) { print "<td width=", int(100/$self->{'config_hash'}->{'column_count'}), "%></td>\n"; }
		print "</tr>\n";
		}
	print $self->ui_table_end();
	}
}

# switch_to_user(user)
# Switch to the Unix user that files are accessed as.
sub switch_to_user
{
	my $self = shift;

if (!defined($self->{'old_uid'})) {
        local @uinfo = $self->get_mail_user($_[0]);
        $self->{'old_uid'} = $>;
        $self->{'old_gid'} = $);
        $) = "$uinfo[3] $uinfo[3]";
        $> = $uinfo[2];
        }
}

sub switch_user_back
{
	my $self = shift;

if (defined($self->{'old_uid'})) {
        $> = $self->{'old_uid'};
        $) = $self->{'old_gid'};
        $self->{'old_uid'} = $self->{'old_gid'} = undef;
        }
}

sub folder_link
{
	my $self = shift;

return "<a href='list_mail.cgi?user=$_[0]&folder=$_[1]->{'index'}'>$self->{'text_hash'}->{'mail_return2'}</a>";
}

# get_from_address()
# Returns the address to use when sending email from a script
sub get_from_address
{
	my $self = shift;

local $host = $self->get_from_domain();
if ($self->{'config_hash'}->{'webmin_from'} =~ /\@/) {
	return $self->{'config_hash'}->{'webmin_from'};
	}
elsif (!$self->{'config_hash'}->{'webmin_from'}) {
	return "cpplus\@$host";
	}
else {
	return "$self->{'config_hash'}->{'webmin_from'}\@$host";
	}
}

# get_from_domain()
# Returns the default domain for From: addresses
sub get_from_domain
{
	my $self = shift;

return $self->{'config_hash'}->{'from_dom'} || $self->get_display_hostname();
}

# get_user_from_address(&uinfo)
# Returns the default From: address for mail sent from some user's mailbox
sub get_user_from_address
{
	my $self = shift;

local $uinfo = $_[0];
if ($self->{'config_hash'}->{'from_addr'}) {
	return $self->{'config_hash'}->{'from_addr'};
	}
elsif ($uinfo->[11]) {
	return $uinfo->[11];
	}
elsif ($self->{'config_hash'}->{'from_virtualmin'} && $self->foreign_check("virtual_server")) {
	# Does Virtualmin manage this user?
	$self->foreign_require("virtual_server", "virtual-server-lib.pl");
	local $d;
	foreach $d ($main::virtual_server_object->list_domains()) {
		local @users = $main::virtual_server_object->list_domain_users($d, 0, 0, 1);
		local $u;
		foreach $u (@users) {
			if ($u->{'user'} eq $uinfo->[0] && $u->{'email'}) {
				# Found him!
				return $u->{'email'};
				}
			}
		}
	}
if ($uinfo->[0] =~ /\@/) {
	return $uinfo->[0];
	}
else {
	return $uinfo->[0].'@'.$self->get_from_domain();
	}
}

# check_modification(&folder)
# Display an error message if a folder has been modified since the time
# in $self->{'in_hash'}->{'mod'}
sub check_modification
{
	my $self = shift;

local $newmod = $self->modification_time($_[0]);
if ($self->{'in_hash'}->{'mod'} && $self->{'in_hash'}->{'mod'} != $newmod && $self->{'config_hash'}->{'check_mod'}) {
	# Changed!
	$self->error($self->text('emodified', "list_mail.cgi?user=$self->{'in_hash'}->{'user'}&folder=$_[0]->{'index'}"));
	}
}

# spam_report_cmd(user)
# Returns a command for reporting spam, or undef if none
sub spam_report_cmd
{
	my $self = shift;

local ($user) = @_;
local %sconfig = $self->foreign_config("spam");
local $cmd;
if ($self->{'config_hash'}->{'spam_report'} eq 'sa_learn') {
        $cmd = $self->has_command($sconfig{'sa_learn'}) ? "$sconfig{'sa_learn'} --spam --mbox" : undef;
        }
elsif ($self->{'config_hash'}->{'spam_report'} eq 'spamassassin') {
        $cmd = $self->has_command($sconfig{'spamassassin'}) ? "$sconfig{'spamassassin'} --r" : undef;
        }
else {
        $cmd = $self->has_command($sconfig{'sa_learn'}) ?
                "$sconfig{'sa_learn'} --spam --mbox" :
               $self->has_command($sconfig{'spamassassin'}) ?
                "$sconfig{'spamassassin'} --r" : undef;
        }
return $user eq "root" ? $cmd :
       $cmd ? $self->command_as_user($user, 0, $cmd) : undef;
}

# allowed_directory()
# Returns base directory for mail files, or undef if not allowed
sub allowed_directory
{
	my $self = shift;

if ($self->{'access_hash'}->{'dir'}) {
	return $self->{'access_hash'}->{'dir'};
	}
else {
	return $self->{'access_hash'}->{'mmode'} == 1 ? "/" : undef;
	}
}

# is_user(user)
sub is_user
{
	my $self = shift;

return $_[0] !~ /^\//;
}

# list_mail_users([max], [filterfunc])
# Returns getpw* style structures for all users who can recieve mail. Those with
# duplicate info are skipped.
sub list_mail_users
{
	my $self = shift;

local ($max, $filter) = @_;
local @rv;
if ($self->{'config_hash'}->{'mail_system'} < 3) {
	# Postfix, Sendmail and Qmail all use Unix users
	local %found;
	local %hfound;
	local ($dir, $style, $mailbox, $maildir) = $self->get_mail_style();
	setpwent();
	while(my (@uinfo) = getpwent()) {
		next if ($found{$uinfo[0]}++);	# done this username already
		next if (!$dir && $hfound{$uinfo[7]}++);
		next if ($filter && !$self->$filter(@uinfo));
		push(@rv, \@uinfo);
		last if ($max && @rv > $max);
		}
	endpwent();
	}
elsif ($self->{'config_hash'}->{'mail_system'} == 4) {
	# Qmail+LDAP uses an LDAP db
	local $ldap = $self->connect_qmail_ldap();
	local $rv = $ldap->search(base => $self->{'config_hash'}->{'ldap_base'},
			  filter => "(objectClass=qmailUser)");
	$self->error($rv->error) if ($rv->code);
	local $u;
	foreach $u ($rv->all_entries) {
		local @uinfo = $self->qmail_dn_to_user($u);
		next if (!$uinfo[10]);	# alias only
		next if ($filter && !$self->$filter(@uinfo));
		push(@rv, \@uinfo);
		last if ($max && @rv > $max);
		}
	$ldap->unbind();
	}
elsif ($self->{'config_hash'}->{'mail_system'} == 5) {
	# Get vpopmail user list for all domains
	opendir(DOMS, "$self->{'config_hash'}->{'vpopmail_dir'}/domains");
	foreach my $d (readdir(DOMS)) {
		next if ($d =~ /^\./);
		local @uinfos = $self->parse_vpopmail_users("-D $d", $d);
		if ($filter) {
			@uinfos = grep { $self->$filter(@$_) } @uinfos;
			}
		push(@rv, @uinfos);
		last if ($max && @rv > $max);
		}
	closedir(DOMS);
	}
return @rv;
}

# parse_vpopmail_users(command, domain)
sub parse_vpopmail_users
{
	my $self = shift;

local %attr_map = ( "passwd" => 1,
		    "gecos" => 5,
		    "dir" => 7 );
local (@rv, $user);
open(UINFO, "$self->{'config_hash'}->{'vpopmail_dir'}/bin/vuserinfo $_[0] |");
while(<UINFO>) {
	s/\r|\n//g;
	if (/^([^:]+):\s+(.*)$/) {
		local ($attr, $value) = ($1, $2);
		if ($attr eq "name") {
			# Start of a new user
			$user = [ "$value\@$_[1]" ];
			$user->[11] = $user->[0];
			push(@rv, $user);
			}
		local $amapped = $attr_map{$attr};
		$user->[$amapped] = $value if ($amapped);
		}
	}
close(UINFO);
return @rv;
}

# get_mail_user(name)
# Looks up a user by name
sub get_mail_user
{
	my $self = shift;

if ($self->{'config_hash'}->{'mail_system'} < 3) {
	# Just find Unix user
	return getpwnam($_[0]);
	}
elsif ($self->{'config_hash'}->{'mail_system'} == 4) {
	# Lookup in LDAP DB
	local $ldap = $self->connect_qmail_ldap();
	local $rv = $ldap->search(base => $self->{'config_hash'}->{'ldap_base'},
			  filter => "(&(objectClass=qmailUser)(uid=$_[0]))");
	$self->error($rv->error) if ($rv->code);
	local ($u) = $rv->all_entries;
	if ($u) {
		# Found in LDAP
		local @uinfo = $self->qmail_dn_to_user($u);
		$ldap->unbind();
		return @uinfo;
		}
	else {
		# Fall back to Unix user
		return getpwnam($_[0]);
		}
	}
elsif ($self->{'config_hash'}->{'mail_system'} == 5) {
	# Find in vpopmail
	local ($box, $dom) = split(/\@/, $_[0]);
	local @users = $self->parse_vpopmail_users($_[0], $dom);
	return @{$users[0]};
	}
}

# get_mail_uid(name)
# Looks up a user by UID
sub get_mail_uid
{
	my $self = shift;

if ($self->{'config_hash'}->{'mail_system'} < 3) {
	# Just find Unix user
	return getpwuid($_[0]);
	}
elsif ($self->{'config_hash'}->{'mail_system'} == 4) {
	# Lookup in LDAP DB
	local $ldap = $self->connect_qmail_ldap();
	local $rv = $ldap->search(base => $self->{'config_hash'}->{'ldap_base'},
		  filter => "(&(objectClass=qmailUser)(uidNumber=$_[0]))");
	$self->error($rv->error) if ($rv->code);
	local ($u) = $rv->all_entries;
	local @uinfo = $self->qmail_dn_to_user($u);
	$ldap->unbind();
	return @uinfo;
	}
elsif ($self->{'config_hash'}->{'mail_system'} == 5) {
	# Find in vpopmail
	return ( );	# not possible, since UIDs aren't used!
	}
}

# connect_qmail_ldap([return-error])
# Connect to the LDAP server used for Qmail. Returns an LDAP handle on success,
# or an error message on failure.
sub connect_qmail_ldap
{
	my $self = shift;

eval "use Net::LDAP";
if ($@) {
	local $err = $self->text('ldap_emod', "<tt>Net::LDAP</tt>");
	if ($_[0]) { return $err; }
	else { $self->error($err); }
	}

# Connect to server
local $port = $self->{'config_hash'}->{'ldap_port'} || 389;
local $ldap = Net::LDAP->new($self->{'config_hash'}->{'ldap_host'}, port => $port);
if (!$ldap) {
	local $err = $self->text('ldap_econn',
			   "<tt>$self->{'config_hash'}->{'ldap_host'}</tt>","<tt>$port</tt>");
	if ($_[0]) { return $err; }
	else { $self->error($err); }
	}

# Start TLS if configured
if ($self->{'config_hash'}->{'ldap_tls'}) {
	$ldap->start_tls();
	}

# Login
local $mesg;
if ($self->{'config_hash'}->{'ldap_login'}) {
	$mesg = $ldap->bind(dn => $self->{'config_hash'}->{'ldap_login'},
			    password => $self->{'config_hash'}->{'ldap_pass'});
	}
else {
	$mesg = $ldap->bind(anonymous => 1);
	}
if (!$mesg || $mesg->code) {
	local $err = $self->text('ldap_elogin', "<tt>$self->{'config_hash'}->{'ldap_host'}</tt>",
		     $self->{'dn'}, $mesg ? $mesg->error : "Unknown error");
	if ($_[0]) { return $err; }
	else { $self->error($err); }
	}
return $ldap;
}

# qmail_dn_to_user(&dn)
sub qmail_dn_to_user
{
	my $self = shift;

local $mms = $self->add_ldapmessagestore(
	scalar($_[0]->get_value("mailMessageStore")));
if (-d "$mms/Maildir") {
	$mms .= "/" if ($mms !~ /\/$/);
	$mms .= "Maildir";
	}
return ( scalar($_[0]->get_value("uid")),
	 scalar($_[0]->get_value("userPassword")),
	 scalar($_[0]->get_value("uidNumber")),
	 scalar($_[0]->get_value("gidNumber")),
	 scalar($_[0]->get_value("mailQuotaSize")),
	 scalar($_[0]->get_value("cn")),
	 scalar($_[0]->get_value("cn")),
	 scalar($_[0]->get_value("homeDirectory")),
	 scalar($_[0]->get_value("loginShell")),
	 undef,
	 $mms,
	 scalar($_[0]->get_value("mail")),
	 );
}

# add_ldapmessagestore(path)
sub add_ldapmessagestore
{
	my $self = shift;

if (!$_[0]) {
	return $_[0];
	}
elsif ($_[0] =~ /^\//) {
	return $_[0];
	}
else {
	$self->foreign_require("qmailadmin", "qmail-lib.pl");
	local $pfx = $main::qmailadmin_object->get_control_file("ldapmessagestore");
	return $pfx."/".$_[0];
	}
}

# show_buttons(number, &folders, current-folder, &mail, user, search-mode)
sub show_buttons
{
	my $self = shift;

local ($num, $folders, $folder, $mail, $user, $search) = @_;
local $uuser = $self->urlize($user);
local $spacer = "&nbsp;\n";
if (@$mail) {

	# Mark as
	print "<input type=submit name=mark$_[0] value=\"$self->{'text_hash'}->{'mail_mark'}\">";
	print "<select name=mode$_[0]>\n";
	print "<option value=1 checked>$self->{'text_hash'}->{'mail_mark1'}\n";
	print "<option value=0>$self->{'text_hash'}->{'mail_mark0'}\n";
	print "<option value=2>$self->{'text_hash'}->{'mail_mark2'}\n";
	print "</select>";
	print $spacer;

	if ($self->is_user($user)) {
		# Forward
		if ($self->{'config_hash'}->{'open_mode'}) {
			# Forward messages in a separate window
			print "<input type=submit name=forward value=\"$self->{'text_hash'}->{'mail_forward'}\" onClick='args = \"user=$uuser&folder=$folder->{'index'}\"; for(i=0; i<form.d.length; i++) { if (form.d[i].checked) { args += \"&mailforward=\"+form.d[i].value; } } window.open(\"reply_mail.cgi?\"+args, \"compose\", \"toolbar=no,menubar=no,scrollbars=yes,width=1024,height=768\"); return false'>";

			}
		else {
			print "<input type=submit name=forward value=\"$self->{'text_hash'}->{'mail_forward'}\">";
			}
		print $spacer;
		}
	# Delete
	print "<input type=submit name=delete value=\"$self->{'text_hash'}->{'mail_delete'}\" ",
	      "onClick='return check_clicks(form)'>";
	if ($self->{'config_hash'}->{'show_delall'} && !$search) {
		print "<input type=submit name=deleteall value=\"$self->{'text_hash'}->{'mail_deleteall'}\">";
		}
	print $spacer;

	# Move/copy
	print "<br><hr>\n";
	print $self->movecopy_user_select($_[0], $folders, $folder, 1);
	print $spacer;

	# Show spam report buttons
	local @modules = $self->get_available_module_infos(1);
	local ($hasspam) = grep { $_->{'dir'} eq "spam" } @modules;
	if ($self->foreign_installed("spam") &&
		$self->{'config_hash'}->{'spam_buttons'} =~ /list/ &&
		$self->spam_report_cmd($user)) {
		print "<br><hr>\n";
		if ($hasspam) {
			print "<input type=submit value=\"$self->{'text_hash'}->{'mail_black'}\" name=black>";
			}
		if ($self->{'config_hash'}->{'spam_del'}) {
			print "<input type=submit value=\"$self->{'text_hash'}->{'view_razordel'}\" name=razor>\n";
			}
		else {
			print "<input type=submit value=\"$self->{'text_hash'}->{'view_razor'}\" name=razor>";
			}
		print $spacer;
		}
	}

if ($self->{'config_hash'}->{'open_mode'}) {
        # Show mass open button
        print "<input type=submit name=new value=\"$self->{'text_hash'}->{'mail_open'}\" onClick='for(i=0; i<form.d.length; i++) { if (form.d[i].checked) { window.open(\"view_mail.cgi?user=$uuser&folder=$folder->{'index'}&idx=\"+form.d[i].value, \"view\"+i, \"toolbar=no,menubar=no,scrollbars=yes,width=1024,height=768\"); } }
 return false'>";
        print $spacer;
        }

# Compose
if ($self->is_user($user)) {
	if ($self->{'config_hash'}->{'open_mode'}) {
		# In a separate window
		print "<input type=submit name=new value=\"$self->{'text_hash'}->{'mail_compose'}\" onClick='window.open(\"reply_mail.cgi?user=$user&new=1\", \"compose\", \"toolbar=no,menubar=no,scrollbars=yes,width=1024,height=768\"); return false'>";
		}
	else {
		print "<br><hr><input type=submit name=new value=\"$self->{'text_hash'}->{'mail_compose'}\">";
		}
	}
print "<br>\n";
}

# get_signature(user)
# Returns the users signature, if any
sub get_signature
{
	my $self = shift;

local $sf = $self->get_signature_file($_[0]);
$sf || return undef;
local $sig;
open(SIG, $sf) || return undef;
while(<SIG>) {
	$sig .= $_;
	}
close(SIG);
return $sig;
}

# get_signature_file(user)
# Returns the full path to the file that should contain the user's signature,
# or undef if none is defined
sub get_signature_file
{
	my $self = shift;

return undef if ($self->{'config_hash'}->{'sig_file'} eq '*' || $self->{'config_hash'}->{'sig_file'} eq '');
local $sf = $self->{'config_hash'}->{'sig_file'};
if ($sf !~ /^\//) {
	local @uinfo = getpwnam($_[0]);
	$sf = "$uinfo[7]/$sf";
	}
return $sf;
}

# view_mail_link(user, &folder, index, from-to-text)
sub view_mail_link
{
	my $self = shift;

local ($user, $folder, $idx, $txt) = @_;
local $uuser = $self->urlize($user);
local $url = "view_mail.cgi?user=$uuser&idx=$idx&folder=$folder->{'index'}";
if ($self->{'config_hash'}->{'open_mode'}) {
        return "<a href='' onClick='window.open(\"$url\", \"viewmail\", \"toolbar=no,menubar=no,scrollbars=yes,width=1024,height=768\"); return false'>".
               $self->simplify_from($txt)."</a>";
        }
else {
        return "<a href='$url'>".$self->simplify_from($txt)."</a>";
        }
}

# mail_page_header(title, headstuff, bodystuff, rightstuff)
sub mail_page_header
{
	my $self = shift;

if ($self->{'config_hash'}->{'open_mode'}) {
        $self->popup_header($_[0], $_[1], $_[2]);
        }
else {
        $self->ui_print_header(undef, $_[0], "", undef, 0, 0, 0, $_[3], $_[1], $_[2]);
        }
}

# mail_page_footer(link, text, ...)
sub mail_page_footer
{
	my $self = shift;

if ($self->{'config_hash'}->{'open_mode'}) {
        $self->popup_footer();
        }
else {
        $self->ui_print_footer(@_);
        }
}


1;

