#!/usr/bin/perl
package mailman::MailmanLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'mailman::MailmanLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# mailman-lib.pl
# mailman functions library

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

# require virtual_server library
$self->foreign_require("virtual_server", "virtual-server-lib.pl");

 @{$self->{'pre_gl_array'}} = ( "", "admin", "bounces", "confirm", "join","leave", "owner", "request", "subscribe", "unsubscribe" );

1;
1;
		return $self;
}

#--- Service funcs ---
# &get_mailman_version()
# Returns the mailman version number, or undef if not available
sub get_mailman_version
{
	my $self = shift;

 my $out = `$self->{'config_hash'}->{'mailman_scripts'}/version 2>&1 </dev/null </dev/null`;
 if ($out =~ /version\s+(\S+)/i || $out =~ /version:\s+(\S+)/i) { return $1; } 
 else { return undef; }
}

# &mailman_check()
# is mailman installed and module configured
sub mailman_check
{
	my $self = shift;

 # check scripts directory
 return $self->text('feat_emailman', "<tt>$self->{'config_hash'}->{'mailman_scripts'}</tt>") if (!-d $self->{'config_hash'}->{'mailman_scripts'});
 
 # check lists directory
 return $self->text('feat_emailman2', "<tt>$self->{'config_hash'}->{'mailman_lists'}</tt>") if (!-d $self->{'config_hash'}->{'mailman_lists'}); 
 return undef;
}

# &is_mailman_running()
# Returns the PID if the server process is active, undef if not
sub is_mailman_running
{
	my $self = shift;

 # Try the configured PID file first
 my $pid = $self->check_pid_file($self->{'config_hash'}->{'pid_file'});
 return $pid if ($pid);
 
 my $script = $self->{'config_hash'}->{'init_script'};
 if ($script) 
 {
  # by script
  my $out = `$script status 2>&1`;
  if ($out =~ /pid:*?\s*?(\d+)/) { return $1; } 
  else { return undef; }
  return $? ? "<pre>$out</pre>" : undef;
 } 
 else
 {
  # Look in the base dir
  my $out = `$self->{'config_hash'}->{'mailman_scripts'}/mailmanctl status`;
  if ($out =~ /pid:*?\s*?(\d+)/) { return $1; } 
  else { return undef; }
 }
}

# &stop_mailman()
# Attempts to stop the mailman process, returning an error message or
# undef if successful
sub stop_mailman
{
	my $self = shift;

 my $script = $self->{'config_hash'}->{'init_script'};
 if ($script) 
 {
  # by script
  my $out = `$script stop 2>&1`;
  return $? ? "<pre>$out</pre>" : undef;
 } 
 else 
 {
  # by PID
  my $pid = $self->is_mailman_running();
  if ($pid && kill('TERM', $pid)) { return undef; } 
  else { return $self->{'text_hash'}->{'pid_erunning'}; }
 }
}

# &start_mailman()
# Attempts to start the mailman process, returning an error message or
# undef if successful
sub start_mailman
{
	my $self = shift;

 my $script = $self->{'config_hash'}->{'init_script'};
 my $cmd = $script ? "$script start" : "$self->{'config_hash'}->{'mailman_scripts'}/mailmanctl start";
 my $out = `$cmd 2>&1`;
 return $? ? "<pre>$out</pre>" : undef;
}

# &restart_mailman()
# Attempts to restart the mailman process, returning an error message or
# undef if successful
sub restart_mailman
{
	my $self = shift;

 my $script = $self->{'config_hash'}->{'init_script'};
 my $cmd = $script ? "$script restart" : "$self->{'config_hash'}->{'mailman_scripts'}/mailmanctl restart";
 my $out = `$cmd 2>&1`;
 return $? ? "<pre>$out</pre>" : undef;
}

#--- List funcs ---
# &list_lists([$dom])
# Returns a list of mailing lists and domains and descriptions
sub list_lists
{
	my $self = shift;

 # get command
 my $cmd = "$self->{'config_hash'}->{'mailman_scripts'}/list_lists -b"; 
 # $cmd .= " -V $_[0]" if $_[0];

 # parse command output
 my $out = `$cmd 2>/dev/null`;
 my @rv;
 foreach my $domain (split(/\n|\r/, $out)) 
 {
  my $fullname = $self->trunc_space($domain);
  my ($name, $dom, $desc, $email) = $self->get_list_options($fullname);
  
  if ($_[0]) 
  { 
	 push(@rv, {	'name' => $name,
						'dom' => $dom,
						'list' => $fullname,
						'desc' => $desc,
						'email' => $email }) if ($dom eq $_[0]); 
  }
  else 
  { 
   $dom = (grep { $_->{'dom'} eq $dom } $main::virtual_server_object->list_domains()) ? $dom : undef;
	push(@rv, { 	'name' => $name,
						'dom' => $dom,
						'list' => $fullname,
						'desc' => $desc,
						'email' => $email }); 
  }
 }
 return sort { $a->{'dom'} cmp $b->{'dom'} } @rv;    
}

# &create_list(\%maillist, [$no_reg_aliases], [$no_rel_postfix])
# Create a new mailing list, and returns undef on success or an error
# message on failure.
sub create_list
{
	my $self = shift;

 # list options
 my $maillist = $_[0];
 my $name = $maillist->{'name'};
 my $dom = $maillist->{'dom'};
 my $fullname = $dom ? $name.".$dom" : $name;
 my $lang = $maillist->{'lang'};
 my $email = $maillist->{'email'};
 my $pass = $maillist->{'pass'};
    
 # Call the list creation program
 my $cmd = "$self->{'config_hash'}->{'mailman_scripts'}/newlist";
 $cmd .= " -l $lang" if $lang;
 my $listname = $dom ? "$fullname\@$dom" : $fullname;
 my $out = $self->backquote_logged("$cmd -q $listname $email $pass 2>&1 </dev/null");
 return "<pre>$out</pre>" if $?;
  
 unless ($_[1])
 {
  # regenerate aliases
  $out = $self->regenerate_aliases();
  return "<pre>$out</pre>" if $out;
 }

 if ($dom)
 {
  # set web_page_url option
  my (undef, $url_host) = $self->get_host_options();
  $self->set_list_options($maillist, $url_host);
  
  # add a new virtual map
  if ($self->{'config_hash'}->{'mta'} == 0)
  {
   # Postfix
   $self->create_postfix_map($name, $fullname, $dom, $_[1]);
  }
  elsif ($self->{'config_hash'}->{'mta'} == 1)
  {
   # Sendmail
	$self->create_sendmail_map($name, $fullname, $dom);
  }
 }
 else { $self->set_list_desc($maillist) if $maillist->{'desc'}; }
 return undef;
}

# &create_postfix_map($name, $fullname, $dom, [$no-reload])
# create postfix mapping
sub create_postfix_map
{
	my $self = shift;

 my ($name, $fullname, $dom) = @_;
 $self->foreign_require("postfix", "postfix-lib.pl");
 my $maps = $main::postfix_object->get_maps("$main::postfix_object->{'virtual_maps'}");
 my @pre = @{$self->{'pre_gl_array'}};
 foreach my $i (@pre)
 {
  my $pre_name = $i ? $name."-".$i : $name;
  my $pre_fullname = $i ? $fullname."-".$i : $fullname;
  my $old = grep {$_->{'name'} eq "$pre_name\@$dom"} @{$maps};
  unless ($old)
  { 
	my %newmap = ( 'name' => "$pre_name\@$dom", 'value' => "$pre_fullname\@$dom" );
	$main::postfix_object->create_mapping("$main::postfix_object->{'virtual_maps'}", \%newmap);
  }
 }
 unless ($_[3])
 {
  # regenerate virtual table
  $main::postfix_object->regenerate_virtual_table();
 }
}

# &create_sendmail_map($name, $fullname, $dom)
# create sendmail mapping
sub create_sendmail_map
{
	my $self = shift;

 my ($name, $fullname, $dom) = @_;
 $self->foreign_require("sendmail", "sendmail-lib.pl");
	
 # get sendmail options
 my $conf = $main::sendmail_object->get_sendmailcf();
 my $vfile = $main::sendmail_object->virtusers_file($conf);
 $self->lock_file($vfile);
 my ($vdbm, $vdbmtype) = $main::sendmail_object->virtusers_dbm($conf);
	
 # get sendmail virtusers table
 my @virts = $main::sendmail_object->list_virtusers($vfile);
 my @pre = @{$self->{'pre_gl_array'}};
 foreach my $i (@pre)
 {
  my $pre_name = $i ? $name."-".$i : $name;
  my $pre_fullname = $i ? $fullname."-".$i : $fullname;
  my $old = grep {$_->{'from'} eq "$pre_name\@$dom"} @virts;
  unless ($old)
  { 
   my %newv = ( 'from' => "$pre_name\@$dom", 'to' => "$pre_fullname\@$dom");
   $main::sendmail_object->create_virtuser(\%newv, $vfile, $vdbm, $vdbmtype);
  }
  $self->unlock_file($vfile);
 }
}

# &delete_list(\%maillist, [$no_reg_aliases], [$no_map])
# Delete mailing list, and returns undef on success or an error
# message on failure.
sub delete_list
{
	my $self = shift;

 # list options
 my $name = $_[0]->{'name'};
 my $dom = $_[0]->{'dom'};
 my $fullname = $dom ? $name.".$dom" : $name;

 # Run the remove command
 my $out = $self->backquote_logged("$self->{'config_hash'}->{'mailman_scripts'}/rmlist -a $fullname 2>&1 </dev/null");
 return "<pre>$out</pre>" if $?;
 
 unless ($_[1])
 {
  # regenerate aliases
  $out = $self->regenerate_aliases();
  return "<pre>$out</pre>" if $out;
 }
 
 if ($dom && !$_[2])
 {
  # delete virtual map
  if ($self->{'config_hash'}->{'mta'} == 0)
  {
   # Postfix
	$self->delete_postfix_map($name, $fullname, $dom, $_[2]);
  }
  elsif ($self->{'config_hash'}->{'mta'} == 1)
  {
   # Sendmail
	$self->delete_sendmail_map($name, $fullname, $dom);
  }
 }
 return undef;
}

# &delete_postfix_map($name, $fullname, $dom, [$no-reload])
# delete postfix mapping
sub delete_postfix_map
{
	my $self = shift;

 my ($name, $fullname, $dom) = @_;
 $self->foreign_require("postfix", "postfix-lib.pl");
 my $maps = $main::postfix_object->get_maps("$main::postfix_object->{'virtual_maps'}");
 my @pre = @{$self->{'pre_gl_array'}};
 foreach my $i (@pre)
 {
  my $pre_name = $i ? $name."-".$i : $name;
  my $pre_fullname = $i ? $fullname."-".$i : $fullname;
  foreach (@{$maps})
  {
   $main::postfix_object->delete_mapping("$main::postfix_object->{'virtual_maps'}", $_) if ($_->{'name'} eq "$pre_name\@$dom" && $_->{'value'} eq "$pre_fullname\@$dom");
  }
 }
 unless ($_[2])
 {
  # regenerate virtual table
  $main::postfix_object->regenerate_virtual_table();
 }
}

# &delete_sendmail_map($name, $fullname, $dom)
# delete sendmail mapping
sub delete_sendmail_map
{
	my $self = shift;

 my ($name, $fullname, $dom) = @_;
 $self->foreign_require("sendmail", "sendmail-lib.pl");
	
 # get sendmail options
 my $conf = $main::sendmail_object->get_sendmailcf();
 my $vfile = $main::sendmail_object->virtusers_file($conf);
 $self->lock_file($vfile);
 my ($vdbm, $vdbmtype) = $main::sendmail_object->virtusers_dbm($conf);
	
 # get sendmail virtusers table
 my @virts = $main::sendmail_object->list_virtusers($vfile);
 my @pre = @{$self->{'pre_gl_array'}};
 foreach my $i (@pre)
 {
  my $pre_name = $i ? $name."-".$i : $name;
  my $pre_fullname = $i ? $fullname."-".$i : $fullname;
  foreach (@virts)
  {
   $main::sendmail_object->delete_virtuser($_, $vfile, $vdbm, $vdbmtype) if ($_->{'from'} eq "$pre_name\@$dom" && $_->{'to'} eq "$pre_fullname\@$dom");
  }
 }
 $self->unlock_file($vfile);
}

# &get_list_config($list_name)
# Returns the config file name on success or an error
# message on failure.
sub get_list_config
{
	my $self = shift;

 my $temp = $self->transname();
 my $out = $self->backquote_logged("$self->{'config_hash'}->{'mailman_scripts'}/config_list -o $temp $_[0]");
 return $? ? $out : $temp;
}

# &save_list_config ($conf_file, $list_name);
# save mail list configuration from config file
sub save_list_config
{
	my $self = shift;
    
 my ($file, $list) = @_;
 
 # save list config
 my $out = $self->backquote_logged("$self->{'config_hash'}->{'mailman_scripts'}/config_list -i $file $list");
 return $? ? $out : undef;
 $self->unlink_file($file);
}

# &get_list_options($list_name);
# returns name, hostname and description for list
sub get_list_options
{
	my $self = shift;

 my ($name, $dom, $desc);
 my $conf_file = $self->get_list_config($_[0]);
 my $conf = $self->read_file_lines($conf_file);
 foreach (@{$conf})
 {
  if ($_ =~ /^\s*?real_name\s*?=\s*?'(.+?)'/)
  {
   # get name
   $name = lc($1);
  }
  elsif ($_ =~ /^\s*?host_name\s*?=\s*?'(.+?)'/)
  {
   # get domain
	$dom = $1;
  }
  elsif ($_ =~ /^\s*?description\s*?=\s*?'(.+?)'/)
  {
   # get domain
   $desc = $1;
  }
  elsif ($_ =~ /^\s*?owner\s*?=\s*?\['(.+?)'\]/)
  {
   # get domain
   $self->{'email'} = $1;
  }
 }
 $name = $1 if ($name =~ /(.+?)\.$dom$/);
 $self->unlink_file($conf_file);
 return ($name, $dom, $desc, $self->{'email'});
}

# &set_list_options(\%maillist, $web_page_url)
# add web_page_url option in to the mailman list
# change description, subject_prefix & reply_to_address
sub set_list_options
{
	my $self = shift;

 my $fullname = $_[0]->{'dom'} ? $_[0]->{'name'}.".$_[0]->{'dom'}" : $_[0]->{'name'};
 my $conf_file = $self->get_list_config($fullname);
 my $conf = $self->read_file_lines($conf_file);
 foreach (@{$conf})
 {
  # change description, subject_prefix & reply_to_address
  $_ = "description = \'$_[0]->{'desc'}\'" if ($_ =~ /^\s*?description\b/);
  $_ = "subject_prefix = \'[$_[0]->{'name'}]\'" if ($_ =~ /^\s*?subject_prefix\b/);
  $_ = "reply_to_address = \'$_[0]->{'name'}\@$_[0]->{'dom'}\'" if ($_ =~ /^\s*?reply_to_address\b/);
 }
 # add web_page_url
 push(@{$conf}, "web_page_url = \'$_[1]\'");
 $self->flush_file_lines();
 $self->save_list_config($conf_file, $fullname);
}

# &set_list_desc(\%maillist)
# change description, subject_prefix & reply_to_address
sub set_list_desc
{
	my $self = shift;

 my $fullname = $_[0]->{'dom'} ? $_[0]->{'name'}.".$_[0]->{'dom'}" : $_[0]->{'name'};
 my $conf_file = $self->get_list_config($fullname);
 my $conf = $self->read_file_lines($conf_file);
 foreach (@{$conf})
 {
  # change change description
  $_ = "description = \'$_[0]->{'desc'}\'" if ($_ =~ /^\s*?description\b/);
 }
 $self->flush_file_lines();
 $self->save_list_config($conf_file, $fullname);
}

#--- Members funcs ---
# &list_members(\%maillist)
# Returns an array of user structures for some list
sub list_members
{
	my $self = shift;

 my $fullname = $_[0]->{'dom'} ? $_[0]->{'name'}.".$_[0]->{'dom'}" : $_[0]->{'name'};
 
 # get command
 my $cmd = "$self->{'config_hash'}->{'mailman_scripts'}/list_members -r $fullname";
 my $out = `$cmd 2>&1 </dev/null`;
 
 # parse command output
 my @rv;
 foreach my $member (split(/\n|\r/,$out)) 
 {
  push(@rv, { 'email' => $member, 'digest' => 'n' });
 }
 
 # get command
 $cmd = "$self->{'config_hash'}->{'mailman_scripts'}/list_members -d $fullname";
 my $out = `$cmd 2>&1 </dev/null`;
 
 # parse command output
 foreach my $member (split(/\n|\r/,$out)) 
 {
  push(@rv, { 'email' => $member, 'digest' => 'y' });
 }
 return sort { $a->{'email'} cmp $b->{'email'} } @rv;
}

# &add_member(\%member, $list_name)
# Add one subscriber to a list
sub add_member
{
	my $self = shift;

 # random temporary file
 my $temp = $self->transname();
 
 # get command
 my $cmd = "$self->{'config_hash'}->{'mailman_scripts'}/add_members";
 if ($_[0]->{'digest'} eq 'y') { 
 	$cmd .= " -d $temp"; 
	} 
 else { 
 	$cmd .= " -r $temp"; 
	}  
 if ($_[0]->{'welcome'}) {
	$cmd .= " -w ".$_[0]->{'welcome'};
	}
 if ($_[0]->{'admin'}) {
	$cmd .= " -a ".$_[0]->{'admin'};
	}
 $cmd .= ' '.lc($_[1]);
  
 # write e-mail into temp file   
 open(TEMP, ">$temp");
 print TEMP $_[0]->{'email'};
 close(TEMP);
 
 # add member
 my $out = $self->backquote_logged("$cmd 2>&1");
 return $? ? $out : undef;
}

# &add_member(\@members, $list_name, [nodigest])
# Add subscribers to a list
sub add_members
{
	my $self = shift;

 # random temporary file
 my $temp = $self->transname();
  
 # write e-mail into temp file   
 open(TEMP, ">$temp");
 foreach (@{$_[0]}) { print TEMP "$_\n"; }
 close(TEMP);
 
 # add members
 my $out;
 unless ($_[2])
 {
  $out = $self->backquote_logged("$self->{'config_hash'}->{'mailman_scripts'}/add_members -d $temp $_[1] 2>&1");
 }
 else
 {
  $out = $self->backquote_logged("$self->{'config_hash'}->{'mailman_scripts'}/add_members -r $temp $_[1] 2>&1");
 }
 return $? ? $out : undef;
}

# &remove_member($email, $list_name)
# Deletes one person from a mailing list
sub delete_member
{
	my $self = shift;

 # random temporary file
 my $temp = $self->transname();
 
 # write e-mail into temp file   
 open(TEMP, ">$temp");
 print TEMP $_[0];
 close(TEMP);
 
 # delete member
 my $cmd = "$self->{'config_hash'}->{'mailman_scripts'}/remove_members -f $temp $_[1]";
 my $out = $self->backquote_logged("$cmd 2>&1");
 return $? ? $out : undef;
}

#--- Others funcs ---
# &set_site_password($passwd)
# change mailman site password and returns undef on success or an error
# message on failure.
sub set_site_password
{
	my $self = shift;

 my $out = $self->backquote_logged("$self->{'config_hash'}->{'mailman_scripts'}/mmsitepass $_[0] 2>&1 </dev/null");
 return "<pre>$out</pre>" if $?;
 return undef;
}

# &get_host_options()
# return 'DEFAULT_URL_HOST' & 'DEFAULT_EMAIL_HOST' mailman options 
sub get_host_options
{
	my $self = shift;

 my $conf = $self->parse_config_file();
 $conf->{'DEFAULT_URL_HOST'} =~ /'(.*?)'/;
 my $url_host = $1;
 $conf->{'DEFAULT_EMAIL_HOST'} =~ /'(.*?)'/;
 my $email_host = $1;
 return ($email_host, $url_host);
}

# &chpass_list($list_name)
# change mail lists password and returns undef on success or an error
# message on failure.
sub chpass_list
{
	my $self = shift;

 my $out = $self->backquote_logged("$self->{'config_hash'}->{'mailman_scripts'}/change_pw -l $_[0] 2>&1 </dev/null");
 return "<pre>$out</pre>" if $?;
 return undef;
}

# &list_mailman_languages()
# list all languages for mailman
sub list_mailman_languages
{
	my $self = shift;

 opendir(DIR, "$self->{'config_hash'}->{'mailman_templates'}");
 my @rv = grep { $_ !~ /^\./ && $_ !~ /\.(txt|html)$/i } readdir(DIR);
 closedir(DIR);
 return sort { $a cmp $b } @rv;
}

# &get_list_owner($list_name)
# return mail list owner aon success or unsef on failure
sub get_list_owner
{
	my $self = shift;

 my $out = $self->backquote_logged("$self->{'config_hash'}->{'mailman_scripts'}/list_owners $_[0] 2>&1 </dev/null");
 return $? ? undef : $out;
}

# &regenerate_aliases()
# regenerate aliases and returns undef on success or an error
# message on failure.
sub regenerate_aliases
{
	my $self = shift;

 my $out = $self->backquote_logged("$self->{'config_hash'}->{'mailman_scripts'}/genaliases 2>&1 </dev/null");
 return "<pre>$out</pre>" if $?;
 $out = $self->backquote_logged("chown mailman:mailman $self->{'config_hash'}->{'mailman_aliases'}/aliases* 2>&1 </dev/null");
 return "<pre>$out</pre>" if $?;
 if (-e "$self->{'config_hash'}->{'mailman_aliases'}/virtual-mailman")
 {
  $out = $self->backquote_logged("chown mailman:mailman $self->{'config_hash'}->{'mailman_aliases'}/virtual-mailman* 2>&1 </dev/null");
  return "<pre>$out</pre>" if $?;
 }
 return undef;
}

# &parse_config_file()
# return \%conf (option => value)
sub parse_config_file
{
	my $self = shift;

 my $conf_file = $self->{'config_hash'}->{'mailman_conf'};
 $self->lock_file($conf_file);
 my $conf;
 open (INFL, $conf_file) || $self->error($self->{'text_hash'}->{'error_open'});
 while (<INFL>)
 {
  chomp;
  $self->trunc_space($_);
  unless ($_=~/^#/ || $_=~/^$/)
  {
   my @options=split (/=/,$_);
   $options[0]=$self->trunc_space($options[0]);
   $options[1]=$self->trunc_space($options[1]);
   $conf -> {$options[0]} = $options[1];
  }
 }
 close INFL;
 $self->unlock_file($conf_file);
 return $conf;
}

# &remove_mailman_str($str)
# remome string from mailman configuration file
sub remove_mailman_str
{
	my $self = shift;

 my $str_reg = $_[0];
 
 # parse config file
 my $conf_file = $self->{'config_hash'}->{'mailman_conf'};
 $self->lock_file($conf_file);
 my $conf = $self->read_file_lines($conf_file);
 my @new;
 foreach my $i (@{$conf})
 {
  push (@new, $i) unless grep { $i =~ /$_/ } @{$_[0]};
 }
 # save options
 @{$conf} = @new;
 $self->flush_file_lines();
 $self->unlock_file($conf_file);
}

# &save_new_options(\%hash,[file])
# save all new options from hash to mailman configuration file
sub save_new_options
{
	my $self = shift;

 # get config file
 my $conf_file = $_[1] ? $_[1] : $self->{'config_hash'}->{'mailman_conf'};
  
 # parse config file
 $self->lock_file($conf_file);
 my $conf = $self->read_file_lines($conf_file);
 
 # change old options
 my @used;
 foreach (@{$conf})
 {
  if (/^\s*(\S+)\s*=\s*('.*')/ || /^\s*(\S+)\s*=\s*(".*")/ || /^\s*(\S+)\s*=\s*(\S+)/)
  {
   foreach my $i (keys %{$_[0]})
   {
	 if (($1 eq $i))
	 {
     $_ = "$i = $_[0]->{$i}" if ($2 ne $_[0]->{$i});
	  push (@used, $i);
	 }
   }
  }
 }
 
 # add new options
 foreach my $i (keys %{$_[0]})
 {
  unless (grep {$i eq $_} @used)
  {
   $_[0]->{$i} ? push (@{$conf}, "$i = $_[0]->{$i}") : push (@{$conf}, $i);
  }
 }
 
 # save options
 $self->flush_file_lines();
 $self->unlock_file($conf_file);
}

# trunc_space(string)
# delete spaces from string
sub trunc_space
{
	my $self = shift;

 local $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}

#--- ACL ---
# &check_dom_acl($domain)
# if no ACL return undef
sub check_dom_acl
{
	my $self = shift;

 my @access_dom = split(/\s/,$self->{'access_hash'}->{'virts'});
 my $ok = grep { $_ eq $_[0] } @access_dom;
 return $ok; 
}


1;

