#!/usr/bin/perl
package majordomo::MajordomoLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'majordomo::MajordomoLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# majordomo-lib.pl
# Common majordomo functions

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

%{$self->{'MAJOR_ENV_hash'}} = ( 'HOME', $self->{'config_hash'}->{'program_dir'} );

1;

1;
		return $self;
}

# get_config()
# Parse and return the majordomo config file
sub get_config
{
	my $self = shift;

local(@rv, $line);
open(CONF, $self->{'config_hash'}->{'majordomo_cf'});
while(<CONF>) {
	s/\r|\n//g;
	if (/^\s*\$(\S+)\s*=\s*"(.*)";\s*$/) {
		# static config option
		push(@rv, { 'name' => $1,
			    'value' => $self->perl_unescape($2),
			    'line' => $line,
			    'eline' => $line });
		}
	elsif (/^\s*\$(\S+)\s*=\s*<<\s*'(\S+)';\s*$/) {
		# multiline config option
		local $o = { 'name' => $1,
			     'line' => $2 };
		local $end = $3;
		while(<CONF>) {
			$line++;
			last if ($_ =~ /^$end[\r\n]+$/);
			$o->{'value'} .= $_;
			}
		$o->{'eline'} = $line;
		push(@rv, $o);
		}
#	elsif (/^\s*\$(\S+)\s*=\s*\$ENV\{['"]([^'"]+)['"]\};\s*$/) {
#		# from majordomo environment variable
#		push(@rv, { 'name' => $1,
#			    'value' => $MAJOR_ENV{$2},
#			    'line' => $line,
#			    'eline' => $line });
#		}
	elsif (/^\s*\$(\S+)\s*=\s*(.*);\s*$/) {
		# computed config option
		push(@rv, { 'name' => $1,
			    'value' => $2,
			    'computed' => 1,
			    'line' => $line,
			    'eline' => $line });
		}
	$line++;
	}
close(CONF);
return \@rv;
}

# save_directive(&config, name, value)
# Update some directive in the global config file
sub save_directive
{
	my $self = shift;

local $old = $self->find($_[1], $_[0]);
return if (!$old);
local $lref = $self->read_file_lines($self->{'config_hash'}->{'majordomo_cf'});
local $olen = $old->{'eline'} - $old->{'line'} + 1;
local $v = $_[2];
$v =~ s/\n$//;
if ($v =~ /\n/) {
	splice(@$lref, $old->{'line'}, $olen,
	       ( "\$$_[1] = <<'END';", split(/\n/, $v, -1), "END" ));
	}
else {
	$v =~ s/\@/\\@/g;
	splice(@$lref, $old->{'line'}, $olen, "\$$_[1] = \"$v\";");
	}
}

# find(name, &array)
sub find
{
	my $self = shift;

local($c, @rv);
foreach $c (@{$_[1]}) {
	if ($c->{'name'} eq $_[0]) {
		push(@rv, $c);
		}
	}
return @rv ? wantarray ? @rv : $rv[0]
           : wantarray ? () : undef;
}

# find_value(name, &array)
sub find_value
{
	my $self = shift;

local(@v);
@v = $self->find($_[0], $_[1]);
@v = grep { !$_->{'computed'} } @v;
if (!@v) { return undef; }
elsif (wantarray) { return map { $_->{'value'} } @v; }
else { return $v[0]->{'value'}; }
}

# list_lists(&config)
# Returns a list of mailing list names
sub list_lists
{
	my $self = shift;

local ($l, @rv);
local $ldir = $self->perl_var_replace($self->find_value("listdir", $_[0]), $_[0]);
opendir(DIR, $ldir);
while($l = readdir(DIR)) {
	if ($l =~ /^(\S+)\.config$/ && $1 !~ /\.old$/) {
		push(@rv, $1);
		}
	}
closedir(DIR);
return @rv;
}

# get_list(name, &config)
# Returns the details of some list
sub get_list
{
	my $self = shift;

local $ldir = $self->perl_var_replace($self->find_value("listdir", $_[1]), $_[1]);
local %list;
return undef if (!-r "$ldir/$_[0].config");
$list{'name'} = $_[0];
$list{'members'} = "$ldir/$_[0]";
$list{'config'} = "$ldir/$_[0].config";
$list{'info'} = "$ldir/$_[0].info";
$list{'intro'} = "$ldir/$_[0].intro";
return \%list;
}

# get_list_config(file)
sub get_list_config
{
	my $self = shift;

local(@rv, $line);
$self->{'lnum'} = 0;
open(CONF, $_[0]);
while(<CONF>) {
	s/\r|\n//g;
	s/#.*$//g;
	if (/^\s*(\S+)\s*=\s*(.*)$/) {
		# single value
		push(@rv, { 'name' => $1,
			    'value' => $2,
			    'index' => scalar(@rv),
			    'line' => $self->{'lnum'},
			    'eline' => $self->{'lnum'} });
		}
	elsif (/^\s*(\S+)\s*<<\s*(\S+)/) {
		# multi-line value
		local $c = { 'name' => $1,
			     'index' => scalar(@rv),
			     'line' => $self->{'lnum'} };
		local $end = $2;
		while(<CONF>) {
			$self->{'lnum'}++;
			last if (/^$end[\r\n]+$/);
			s/^--/-/;
			s/^-\n/\n/;
			$c->{'value'} .= $_;
			}
		$c->{'eline'} = $self->{'lnum'};
		push(@rv, $c);
		}
	$self->{'lnum'}++;
	}
return \@rv;
}

# save_list_directive(&config, file, name, value, multiline)
sub save_list_directive
{
	my $self = shift;

local $old = $self->find($_[2], $_[0]);
local $lref = $self->read_file_lines($_[1]);
local ($pos, $olen, $nlen);
if ($old) {
	$olen = $old->{'eline'} - $old->{'line'} + 1;
	$pos = $old->{'line'};
	}
else {
	$olen = 0;
	$pos = @$lref;
	}
if ($_[4]) {
	local $ov = $old->{'value'};
	$ov =~ s/\n$//;
	local $v = $_[3];
	$v =~ s/\n$//;
	local @lines = split(/\n/, $v, -1);
	@lines = map { s/^-/--/; s/^$/-/; $_ } @lines;
	splice(@$lref, $pos, $olen, ("$_[2]        <<   END", @lines, "END"))
		if (!$old || $v ne $ov);
	$nlen = (!$old || $v ne $ov) ? @lines + 2 : $olen;
	}
else {
	splice(@$lref, $pos, $olen, "$_[2] = $_[3]")
		if (!$old || $_[3] ne $old->{'value'});
	$nlen = 1;
	}
if ($old && $nlen != $olen) {
	foreach $c (@{$_[0]}) {
		if ($c->{'line'} > $old->{'eline'}) {
			$c->{'line'} += ($nlen - $olen);
			$c->{'eline'} += ($nlen - $olen);
			}
		}
	}
}

# get_aliases_file()
# Returns the paths to the sendmail-style aliases files
sub get_aliases_file
{
	my $self = shift;

if ($self->{'config_hash'}->{'aliases_file'} eq 'postfix') {
	# Get from Postfix config
	$self->foreign_require("postfix", "postfix-lib.pl");
	local @afiles = $main::postfix_object->get_aliases_files(
				$main::postfix_object->get_current_value("alias_maps"));
	$self->{'aliases_module'} = "postfix";
	return \@afiles;
	}
else {
	$self->foreign_require("sendmail", "sendmail-lib.pl");
	$self->foreign_require("sendmail", "aliases-lib.pl");
	$self->{'aliases_module'} = "sendmail";
	if ($self->{'config_hash'}->{'aliases_file'} eq '') {
		# Get from Sendmail
		local $sm_conf = $main::sendmail_object->get_sendmailcf();
		return $main::sendmail_object->aliases_file($sm_conf);
		}
	else {
		# Use fixed file
		return [ $self->{'config_hash'}->{'aliases_file'} ];
		}
	}
}

# perl_unescape(string)
# Converts a string like "hello\@there\\foo" to "hello@there\foo"
sub perl_unescape
{
	my $self = shift;

local $v = $_[0];
$v =~ s/\\(.)/$1/g;
return $v;
}

# perl_var_replace(string, &config)
# Replaces variables like $foo in a string with their value from
# the config file
sub perl_var_replace
{
	my $self = shift;

local $str = $_[0];
local %donevar;
while($str =~ /\$([A-z0-9\_]+)/ && !$donevar{$1}) {
	$donevar{$1}++;
	local $val = $self->find_value($1, $_[1]);
	$str =~ s/\$([A-z0-9\_]+)/$val/;
	}
return $str;
}

# set_permissions(file)
# Sets the ownership and permissions on some file or directory,
# based on the ownership of the lists directory
sub set_permissions
{
	my $self = shift;

local $conf = $self->get_config();
local $ldir = $self->perl_var_replace($self->find_value("listdir", $conf), $conf);
local @ldir = stat($ldir);
chown($ldir[4], $ldir[5], $_[0]);
if ($self->{'config_hash'}->{'perms'}) {
	chmod(-d $_[0] ? 0755 : 0644, $_[0]);
	}
else {
	chmod(-d $_[0] ? 0775 : 0664, $_[0]);
	}
}

# choice_input(name, text, &config, [opt, display]+)
sub choice_input
{
	my $self = shift;

local $v = $self->find_value($_[0], $_[2]);
local $rv = "<td><b>$_[1]</b></td> <td nowrap>";
for($self->{'i'}=3; $self->{'i'}<@_; $self->{'i'}+=2) {
	local $ch = $v eq $_[$self->{'i'}] ? "checked" : "";
	$rv .= "<input name=$_[0] type=radio value='$_[$self->{'i'}]' $ch> ".$_[$self->{'i'}+1];
	}
$rv .= "</td>\n";
return $rv;
}

# save_choice(&config, file, name)
sub save_choice
{
	my $self = shift;

$self->save_list_directive($_[0], $_[1], $_[2], $self->{'in_hash'}->{$_[2]});
}

# opt_input(name, text, &config, default, size, [units])
sub opt_input
{
	my $self = shift;

local $v = $self->find_value($_[0], $_[2]);
local $rv = "<td><b>$_[1]</b></td> <td nowrap ".
	    ($_[4] > 30 ? "colspan=3" : "").">";
$rv .= sprintf "<input type=radio name=$_[0]_def value=1 %s> $_[3]\n",
		$v eq "" ? "checked" : "";
$rv .= sprintf "<input type=radio name=$_[0]_def value=0 %s>\n",
		$v eq "" ? "" : "checked";
local $passwd = $_[0] =~ /passwd/ ? "type=password" : "";
$rv .= "<input $passwd name=$_[0] size=$_[4] value=\"$v\"> $_[5]</td>\n";
return $rv;
}

# save_opt(&config, file, name, [&func])
sub save_opt
{
	my $self = shift;

if ($self->{'in_hash'}->{"$_[2]_def"}) { $self->save_list_directive($_[0], $_[1], $_[2], ""); }
elsif ($_[3] && ($self->{'err'} = &{$_[3]}($self->{'in_hash'}->{$_[2]}))) { $self->error($self->{'err'}); }
else { $self->save_list_directive($_[0], $_[1], $_[2], $self->{'in_hash'}->{$_[2]}); }
}

# select_input(name, text, &config, [opt, display]+)
sub select_input
{
	my $self = shift;

local $v = $self->find_value($_[0], $_[2]);
local $rv = "<td><b>$_[1]</b></td> <td nowrap><select name=$_[0]>";
for($self->{'i'}=3; $self->{'i'}<@_; $self->{'i'}+=2) {
	local $ch = $v eq $_[$self->{'i'}] ? "selected" : "";
	$rv .= "<option value='$_[$self->{'i'}]' $ch> ".$_[$self->{'i'}+1];
	}
$rv .= "</select></td>\n";
return $rv;
}

# save_select(&config, file, name)
sub save_select
{
	my $self = shift;

$self->save_list_directive($_[0], $_[1], $_[2], $self->{'in_hash'}->{$_[2]});
}

# multi_input(name, text, &config)
sub multi_input
{
	my $self = shift;

local $v = $self->find_value($_[0], $_[2]);
local $rv = "<td valign=top><b>$_[1]</b></td> <td colspan=3>".
	    "<textarea rows=4 cols=80 name=$_[0]>\n$v</textarea></td>\n";
return $rv;
}

# save_multi(&config, file, name)
sub save_multi
{
	my $self = shift;

$self->{'in_hash'}->{$_[2]} =~ s/\r//g;
$self->save_list_directive($_[0], $_[1], $_[2], $self->{'in_hash'}->{$_[2]}, 1);
}

# can_edit_list(&access, name)
sub can_edit_list
{
	my $self = shift;

foreach (split(/\s+/, $_[0]->{'lists'})) {
	return 1 if ($_ eq "*" || $_ eq $_[1]);
	}
return 0;
}

# homedir_valid(&config)
sub homedir_valid
{
	my $self = shift;

local $homedir = $self->find_value("homedir", $_[0]);
if (!-d $homedir) {
	local $homeused;
	foreach $c (@{$self->{'conf'}}) {
		$homeused++ if ($c->{'value'} =~ /\$homedir/);
		}
	if ($homeused) {
		return 0;
		}
	}
return 1;
}


1;

