#!/usr/bin/perl
package mount::FreebsdLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'mount::FreebsdLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# freebsd-lib.pl
# Mount table functions for freebsd

$self->{'uname_release'} = `uname -r`;
if ($self->has_command("mount_smbfs")) {
	$self->{'smbfs_support'} = 1;
	$self->{'nsmb_conf'} = "/etc/nsmb.conf";
	}

1;

1;
		return $self;
}

# Return information about a filesystem, in the form:
#  directory, device, type, options, fsck_order, mount_at_boot
# If a field is unused or ignored, a - appears instead of the value.
# Swap-filesystems (devices or files mounted for VM) have a type of 'swap',
# and 'swap' in the directory field
sub list_mounts
{
	my $self = shift;

local(@rv, @p, @o, $_, $i, $j); $i = 0;

# Get /etc/fstab mounts
open(FSTAB, $self->{'config_hash'}->{'fstab_file'});
while(<FSTAB>) {
	local(@o, $at_boot);
	chop; s/#.*$//g;
	if (!/\S/) { next; }
	@p = split(/\s+/, $_);
	if ($p[2] eq "proc" || $p[2] eq "procfs") { $p[2] = $p[0] = "proc"; }
	elsif ($p[2] eq "swap") { $p[1] = "swap"; }
	elsif ($p[2] eq "smbfs") {
		# Need to get nsmb.conf options, covert share and extract user
		$p[0] = lc($p[0]);
		local $noptions = $self->read_nsmb($p[0]);
		local %options;
		$self->parse_options($p[3]);
		if ($p[0] =~ /^\/\/(\S+)\@(\S+)\/(.*)$/) {
			$p[0] = "\\\\$2\\$3";
			$options{'user'} = $1;
			}
		elsif ($p[0] =~ /\/\/(\S+)\/(.*)$/) {
			$p[0] = "\\\\$1\\$2";
			}
		%options = ( %options, %$noptions );
		$p[3] = $self->join_options();
		}
	$rv[$i] = [ $p[1], $p[0], $p[2] ];
	$rv[$i]->[5] = "yes";
	@o = split(/,/ , $p[3] eq "defaults" ? "" : $p[3]);
	if (($j = $self->indexof("noauto", @o)) >= 0) {
		# filesytem is not mounted at boot
		splice(@o, $j, 1);
		$rv[$i]->[5] = "no";
		}
	$rv[$i]->[3] = (@o ? join(',' , @o) : "-");
	$rv[$i]->[4] = (@p >= 5 ? $p[5] : 0);
	$i++;
	}
close(FSTAB);
return @rv;
}

# create_mount(directory, device, type, options, fsck_order, mount_at_boot)
# Add a new entry to the fstab file, old or new automounter file
sub create_mount
{
	my $self = shift;

local(@mlist, @amd, $_, $opts);

$self->open_tempfile(FSTAB, ">> $self->{'config_hash'}->{'fstab_file'}");
$self->print_mount_line(@_);
$self->close_tempfile(FSTAB);
}

# print_mount_line(directory, device, type, options, fsck_order, mount_at_boot)
sub print_mount_line
{
	my $self = shift;

if ($_[2] eq "smbfs") {
	# Adding an SMB mount, which needs special handling
	local %options;
	$self->parse_options("smbfs", $_[3]);
	local $share;
	if ($options{'user'} && $_[1] =~ /^\\\\(.*)\\(.*)$/) {
		$share = "//$options{'user'}\@$1/$2";
		}
	else {
		($share = $_[1]) =~ s/\\/\//g;
		}
	$self->print_tempfile(FSTAB, "$share  $_[0]  $_[2]");
	local $roptions = $self->update_nsmb($share, \%options);
	delete($roptions->{'user'});
	$self->{'opts'} = $self->join_options($_[2], $roptions);
	}
else {
	# Adding a normal mount to the fstab file
	$self->print_tempfile(FSTAB, "$_[1]  $_[0]  $_[2]");
	$self->{'opts'} = $_[3] eq "-" ? "" : $_[3];
	}
if ($_[5] eq "no") {
	$self->{'opts'} = join(',' , (split(/,/ , $self->{'opts'}) , "noauto"));
	}
if ($self->{'opts'} eq "") {
	$self->print_tempfile(FSTAB, "  defaults");
	}
else {
	$self->print_tempfile(FSTAB, "  $self->{'opts'}");
	}
$self->print_tempfile(FSTAB, "  0  ");
$self->print_tempfile(FSTAB, $_[4] eq "-" ? "0\n" : "$_[4]\n");
}

# change_mount(num, directory, device, type, options, fsck_order, mount_at_boot)
# Change an existing permanent mount
sub change_mount
{
	my $self = shift;

local($i, @fstab, $line, $opts, $j, @amd);
$i = 0;

# Update fstab file
open(FSTAB, $self->{'config_hash'}->{fstab_file});
@fstab = <FSTAB>;
close(FSTAB);
$self->open_tempfile(FSTAB, "> $self->{'config_hash'}->{fstab_file}");
foreach (@fstab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $i++ == $_[0]) {
		# Found the line to replace
		$self->print_mount_line(@_[1..$#_]);
		}
	else {
		$self->print_tempfile(FSTAB, $_,"\n");
		}
	}
$self->close_tempfile(FSTAB);
}

# delete_mount(index)
# Delete an existing permanent mount
sub delete_mount
{
	my $self = shift;

local($i, @fstab, $line, $opts, $j, @amd);
$i = 0;

# Update fstab file
open(FSTAB, $self->{'config_hash'}->{fstab_file});
@fstab = <FSTAB>;
close(FSTAB);
$self->open_tempfile(FSTAB, "> $self->{'config_hash'}->{fstab_file}");
foreach (@fstab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line !~ /\S/ || $i++ != $_[0]) {
		# Don't delete this line
		$self->print_tempfile(FSTAB, $_,"\n");
		}
	}
$self->close_tempfile(FSTAB);
}

# list_mounted()
# Return a list of all the currently mounted filesystems and swap files.
# The list is in the form:
#  directory device type options
# Under FreeBSD, there seems to be no way to get additional mount options
# used by filesystems like NFS etc. Even getting the full details of mounted
# filesystems requires C code! So we have to call a specially-written external
# program to get the mount list
sub list_mounted
{
	my $self = shift;

# get the list of mounted filesystems
local(@rv, $_);
local $cmd = $self->{'uname_release'} =~ /^5\.[0-9]/ ? "freebsd-mounts-5" :
	     $self->{'uname_release'} =~ /^4\.[0-9]/ ? "freebsd-mounts-4" :
	     $self->{'uname_release'} =~ /^3\.[1-9]/ ? "freebsd-mounts-3" :
				             "freebsd-mounts-2";
$self->compile_program($cmd, '.*86');
open(CMD, "$self->{'module_config_directory'}/$cmd |");
while(<CMD>) {
	local @p = split(/\t/, $_);
	if ($p[2] eq "procfs" || $p[1] eq "procfs") { $p[1] = $p[2] = "proc"; }
	elsif ($p[2] eq "mfs") { $p[1] =~ s/:.*$//; }
	elsif ($p[2] eq "smbfs") {
		# Need to get nsmb.conf options, covert share and extract user
		$p[1] = lc($p[1]);
		local $noptions = $self->read_nsmb($p[1]);
		local %options;
		$self->parse_options($p[3]);
		if ($p[1] =~ /^\/\/(\S+)\@(\S+)\/(.*)$/) {
			$p[1] = "\\\\$2\\$3";
			$options{'user'} = $1;
			}
		elsif ($p[1] =~ /\/\/(\S+)\/(.*)$/) {
			$p[1] = "\\\\$1\\$2";
			}
		%options = ( %options, %$noptions );
		$p[3] = $self->join_options();
		}
	push(@rv, \@p);
	}
close(CMD);

# add output from swapinfo
local $out;
$self->execute_command("swapinfo", undef, \$out, undef, 0, 1);
foreach (split(/\n/, $out)) {
	if (/^(\/\S+)\s+\d+\s+\d+/) {
		push(@rv, [ "swap", $1, "swap", "-" ]);
		}
	}
return @rv;
}

# mount_dir(directory, device, type, options)
# Mount a new directory from some device, with some options. Returns 0 if ok,
# or an error string if failed
sub mount_dir
{
	my $self = shift;

local($out, $opts, $shar, %options, %smbopts);
if ($_[2] eq "swap") {
	# Use swapon to add the swap space..
	$out = $self->backquote_logged("swapon $_[1] 2>&1");
	if ($?) { return "<pre>$out</pre>"; }
	}
elsif ($_[2] eq "smbfs") {
	# Need special handling for SMB mounts
	local %options;
	$self->parse_options($_[2], $_[3]);
	local $share;
	if ($options{'user'} && $_[1] =~ /^\\\\(.*)\\(.*)$/) {
		$share = "//$options{'user'}\@$1/$2";
		}
	else {
		($share = $_[1]) =~ s/\\/\//g;
		}
	local $roptions = $self->update_nsmb($share, \%options);
	delete($roptions->{'user'});
	local $opts = $self->join_options($_[2], $roptions);
	$opts = $opts ne "-" ? " -o $opts" : "";
	$self->foreign_require("proc", "proc-lib.pl");
	local ($fh, $fpid) = $main::proc_object->pty_process_exec_logged(
		"mount -t $_[2] $opts $share $_[0]");
	local $got;
	local $rv = $self->wait_for($fh, "Password:");
	$got .= $self->{'wait_for_input'};
	if ($rv == 0) {
		print $fh $options{'nsmb_password'},"\n";
		}
	$rv = $self->wait_for($fh);
	$got .= $self->{'wait_for_input'};
	close($fh);
	if ($? || $got =~ /failed|error|syserr|usage:/i) { return "<pre>$out</pre>"; }
	}
else {
	# Can use mount command for this filesystem
	$opts = $_[3] eq "-" ? "" :
			"-o ".join(',', grep { !/quota/ } split(/,/, $_[3]));
	$out = $self->backquote_logged("mount -t $_[2] $opts $_[1] $_[0] 2>&1");
	if ($?) { return "<pre>$out</pre>"; }
	}
return 0;
}

# unmount_dir(directory, device, type)
# Unmount a directory that is currently mounted. Returns 0 if ok,
# or an error string if failed
sub unmount_dir
{
	my $self = shift;

local($out, %smbopts, $dir);
if ($_[2] eq "swap") {
	# Not possible!
	$self->error("Swap space cannot be removed");
	}
else {
	$out = $self->backquote_logged("umount $_[0] 2>&1");
	if ($?) { return "<pre>$out</pre>"; }
	}
return 0;
}

# mount_modes(type)
# Given a filesystem type, returns 4 numbers that determine how the file
# system can be mounted, and whether it can be fsck'd
# The first is:
#  0 - cannot be permanently recorded
#	(smbfs under linux)
#  1 - can be permanently recorded, and is always mounted at boot
#	(swap under linux)
#  2 - can be permanently recorded, and may or may not be mounted at boot
#	(most normal filesystems)
# The second is:
#  0 - mount is always permanent => mounted when saved
#	(swap under linux)
#  1 - doesn't have to be permanent
#	(normal fs types)
# The third is:
#  0 - cannot be fsck'd at boot time
#  1 - can be be fsck'd at boot
# The fourth is:
#  0 - can be unmounted
#  1 - cannot be unmounted
sub mount_modes
{
	my $self = shift;

if ($_[0] eq "swap")
	{ return (2, 1, 0, 1); }
elsif ($_[0] eq "ufs")
	{ return (2, 1, 1, 0); }
else
	{ return (2, 1, 0, 0); }
}

# disk_space(type, directory)
# Returns the amount of total and free space for some filesystem, or an
# empty array if not appropriate.
sub disk_space
{
	my $self = shift;

if ($self->get_mounted($_[1], "*") < 0) { return (); }
if ($_[0] eq "proc" || $_[0] eq "swap") { return (); }
$self->execute_command("df -k $_[1]", undef, \$self->{'out'}, undef, 0, 1);
if ($self->{'out'} =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/) {
	return ($1, $2);
	}
else {
	return ( );
	}
}

# list_fstypes()
# Returns an array of all the supported filesystem types. If a filesystem is
# found that is not one of the supported types, generate_location() and
# generate_options() will not be called for it.
sub list_fstypes
{
	my $self = shift;

local @rv = ("ufs", "nfs", "cd9660", "msdos", "swap");
push(@rv, "ext2fs") if ($self->has_command("mount_ext2fs"));
push(@rv, "ntfs") if ($self->has_command("mount_ntfs"));
push(@rv, "smbfs") if ($self->{'smbfs_support'});
return @rv;
}

# fstype_name(type)
# Given a short filesystem type, return a human-readable name for it
sub fstype_name
{
	my $self = shift;

local(%fsmap);
%fsmap = ("ufs", "FreeBSD Unix Filesystem",
	  "nfs","Network Filesystem",
	  "cd9660","ISO9660 CD-ROM",
	  "msdos","MS-DOS Filesystem",
	  "ext2fs","Linux Filesystem",
	  "ntfs","Windows NT Filesystem",
	  "swap","Virtual Memory",
	  "proc","Process Image Filesystem",
	  "smbfs","Windows Networking Filesystem");
return $self->{'config_hash'}->{long_fstypes} && $fsmap{$_[0]} ? $fsmap{$_[0]} : uc($_[0]);
}

# multiple_mount(type)
# Returns 1 if filesystems of this type can be mounted multiple times, 0 if not
sub multiple_mount
{
	my $self = shift;

return $_[0] eq "nfs" || $_[0] eq "smbfs";
}

# generate_location(type, location)
# Output HTML for editing the mount location of some filesystem.
sub generate_location
{
	my $self = shift;

if ($_[0] eq "nfs") {
	# NFS mount from some host and directory
	$_[1] =~ /^([^:]+):(.*)$/;
	print "<tr> <td><b>NFS Hostname</b></td>\n";
	print "<td><input name=nfs_host size=20 value=\"$1\">\n";
	$self->nfs_server_chooser_button("nfs_host");
	print "</td>\n";
	print "<td><b>NFS Directory</b></td>\n";
	print "<td><input name=nfs_dir size=20 value=\"$2\">\n";
	$self->nfs_export_chooser_button("nfs_host", "nfs_dir");
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "smbfs") {
	# SMB mount from some server and share
	$_[1] =~ /^\\\\(.*)\\(.*)$/;
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_smbserver'}</b></td>\n";
	print "<td><input name=smbfs_server value=\"$1\" size=20>\n";
	$self->smb_server_chooser_button("smbfs_server");
	print "</td>\n";
	print "<td><b>$self->{'text_hash'}->{'linux_smbshare'}</b></td>\n";
	print "<td><input name=smbfs_share value=\"$2\" size=20>\n";
	$self->smb_share_chooser_button("smbfs_server", "smbfs_share");
	print "</td> </tr>\n";
	}
else {
	if ($_[0] eq "swap") {
		# Swap file or device
		printf "<tr> <td valign=top><b>Swap File</b></td>\n";
		}
	else {
		# Disk-based filesystem
		printf "<tr> <td valign=top><b>%s Disk</b></td>\n",
			$self->fstype_name($_[0]);
		}
	print "<td colspan=3>\n";
	if ($_[1] =~ /^\/dev\/ad(\d)s(\d)([a-z]*)$/) {
		$self->{'disk_dev'} = 0; $self->{'ide_t'} = $1; $self->{'ide_s'} = $2; $self->{'ide_p'} = $3;
		}
	elsif ($_[1] =~ /^\/dev\/da(\d)s(\d)([a-z]*)$/) {
		$self->{'disk_dev'} = 1; $self->{'scsi_t'} = $1; $self->{'scsi_s'} = $2; $self->{'scsi_p'} = $3;
		}
	else { $self->{'disk_dev'} = 2; }

	printf "<input type=radio name=disk_dev value=0 %s> IDE Hard Disk:\n",
		$self->{'disk_dev'} == 0 ? "checked" : "";
	print "Device <input name=ide_t size=3 value=\"$self->{'ide_t'}\">\n";
	print "Slice <input name=ide_s size=3 value=\"$self->{'ide_s'}\">\n";
	print "Partition <input name=ide_p size=3 value=\"$self->{'ide_p'}\"><br>\n";

	printf "<input type=radio name=disk_dev value=1 %s> SCSI Disk:\n",
		$self->{'disk_dev'} == 1 ? "checked" : "";
	print "Device <input name=scsi_t size=3 value=\"$self->{'scsi_t'}\">\n";
	print "Slice <input name=scsi_s size=3 value=\"$self->{'scsi_s'}\">\n";
	print "Partition <input name=scsi_p size=3 value=\"$self->{'scsi_p'}\"><br>\n";

	printf "<input type=radio name=disk_dev value=2 %s> Other Device:\n",
		$self->{'disk_dev'} == 2 ? "checked" : "";
	printf "<input size=20 name=dev_path value=\"%s\"><br>\n",
		$self->{'disk_dev'} == 2 ? $_[1] : "";
	print "</td> </tr>\n";
	}
}

# generate_options(type, newmount)
# Output HTML for editing mount options for a particular filesystem 
# under this OS
sub generate_options
{
	my $self = shift;

if ($_[0] ne "swap") {
	# These options are common to all filesystems
	print "<tr> <td><b>Read-only?</b></td>\n";
	printf "<td nowrap><input type=radio name=bsd_ro value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"rdonly"}) || defined($self->{'options_hash'}->{"ro"})
			? "checked" : "";
	printf "<input type=radio name=bsd_ro value=0 %s> No</td>\n",
		defined($self->{'options_hash'}->{"rdonly"}) || defined($self->{'options_hash'}->{"ro"})
			? "" : "checked";

	print "<td><b>Buffer writes to filesystem?</b></td>\n";
	printf"<td nowrap><input type=radio name=bsd_sync value=0 %s> Yes\n",
		defined($self->{'options_hash'}->{"sync"}) ? "" : "checked";
	printf "<input type=radio name=bsd_sync value=1 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"sync"}) ? "checked" : "";

	print "<tr> <td><b>Allow device files?</b></td>\n";
	printf "<td nowrap><input type=radio name=bsd_nodev value=0 %s> Yes\n",
		defined($self->{'options_hash'}->{"nodev"}) ? "" : "checked";
	printf "<input type=radio name=bsd_nodev value=1 %s> No</td>\n",
		defined($self->{'options_hash'}->{"nodev"}) ? "checked" : "";

	print "<td><b>Allow execution of binaries?</b></td>\n";
	printf"<td nowrap><input type=radio name=bsd_noexec value=0 %s> Yes\n",
		defined($self->{'options_hash'}->{"noexec"}) ? "" : "checked";
	printf "<input type=radio name=bsd_noexec value=1 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"noexec"}) ? "checked" : "";

	print "<tr> <td><b>Disallow setuid programs?</b></td>\n";
	printf "<td nowrap><input type=radio name=bsd_nosuid value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=bsd_nosuid value=0 %s> No</td>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";

	print "<td><b>Update access times?</b></td>\n";
	printf"<td nowrap><input type=radio name=bsd_noatime value=0 %s> Yes\n",
		defined($self->{'options_hash'}->{"noatime"}) ? "" : "checked";
	printf "<input type=radio name=bsd_noatime value=1 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"noatime"}) ? "checked" : "";

	if ($self->{'uname_release'} =~ /^[34]\./) {
		# FreeBSD 3.x has some more options
		print "<tr> <td><b>Follow symbolic links?</b></td>\n";
		printf "<td nowrap><input type=radio name=bsd_nosymfollow value=0 %s> Yes\n",
			defined($self->{'options_hash'}->{"nosymfollow"}) ? "" : "checked";
		printf "<input type=radio name=bsd_nosymfollow value=1 %s> No</td>\n",
			defined($self->{'options_hash'}->{"nosymfollow"}) ? "checked" : "";

		print "<td><b>Files inherit group from SUID directories?</b></td>\n";
		printf"<td nowrap><input type=radio name=bsd_suiddir value=1 %s> Yes\n",
			defined($self->{'options_hash'}->{"suiddir"}) ? "checked" : "";
		printf "<input type=radio name=bsd_suiddir value=0 %s> No</td> </tr>\n",
			defined($self->{'options_hash'}->{"suiddir"}) ? "" : "checked";
		}
	}

if ($_[0] eq "ufs") {
	# UFS filesystems support quotas
	print "<tr> <td><b>User quotas at boot</b></td> <td colspan=3>\n";
	printf "<input type=radio name=ufs_userquota value=0 %s> Disabled\n",
		defined($self->{'options_hash'}->{'userquota'}) ? "" : "checked";
	printf "<input type=radio name=ufs_userquota value=1 %s> Enabled\n",
		defined($self->{'options_hash'}->{'userquota'}) && $self->{'options_hash'}->{'userquota'} eq ""
			? "checked" : "";
	printf "<input type=radio name=ufs_userquota value=2 %s>\n",
		$self->{'options_hash'}->{'userquota'} ? "checked" : "";
	print "Enabled, use file\n";
	printf "<input name=ufs_userquota_file size=30 value=\"%s\">\n",
		$self->{'options_hash'}->{'userquota'};
	print "</td> </tr>\n";
		
	print "<tr> <td><b>Group quotas at boot</b></td> <td colspan=3>\n";
	printf "<input type=radio name=ufs_groupquota value=0 %s> Disabled\n",
		defined($self->{'options_hash'}->{'groupquota'}) ? "" : "checked";
	printf "<input type=radio name=ufs_groupquota value=1 %s> Enabled\n",
		defined($self->{'options_hash'}->{'groupquota'}) && $self->{'options_hash'}->{'groupquota'} eq ""
			? "checked" : "";
	printf "<input type=radio name=ufs_groupquota value=2 %s>\n",
		$self->{'options_hash'}->{'groupquota'} ? "checked" : "";
	print "Enabled, use file\n";
	printf "<input name=ufs_groupquota_file size=30 value=\"%s\">\n",
		$self->{'options_hash'}->{'groupquota'};
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "nfs") {
	# NFS filesystems have lots more options
	print "<tr> <td><b>Retry mounts in background?</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_b value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"-b"}) ? "checked" : "";
	printf "<input type=radio name=nfs_b value=0 %s> No</td>\n",
		defined($self->{'options_hash'}->{"-b"}) ? "" : "checked";

	print "<td><b>Return error on timeouts?</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_s value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"-s"}) ? "checked" : "";
	printf "<input type=radio name=nfs_s value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"-s"}) ? "" : "checked";

	print "<tr> <td><b>Timeout</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_t_def value=1 %s> Default\n",
		defined($self->{'options_hash'}->{"-t"}) ? "" : "checked";
	printf "<input type=radio name=nfs_t_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"-t"}) ? "checked" : "";
	printf "<input size=5 name=nfs_t value=\"$self->{'options_hash'}->{'-t'}\"></td>\n";

	print "<td><b>Number of Retransmissions</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_x_def value=1 %s> Default\n",
		defined($self->{'options_hash'}->{"-x"}) ? "" : "checked";
	printf "<input type=radio name=nfs_x_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"-x"}) ? "checked" : "";
	print "<input size=5 name=nfs_x value=\"$self->{'options_hash'}->{'-x'}\"></td> </tr>\n";

	print "<tr> <td><b>NFS version</b></td> <td nowrap>\n";
	local $v = defined($self->{'options_hash'}->{"-2"}) ? 2 :
		   defined($self->{'options_hash'}->{"-3"}) ? 3 : 0;
	printf "<input type=radio name=nfs_ver value=0 %s> Auto\n",
		$v ? "" : "checked";
	printf "<input type=radio name=nfs_ver value=2 %s> V2\n",
		$v == 2 ? "checked" : "";
	printf "<input type=radio name=nfs_ver value=3 %s> V3</td>\n",
		$v == 3 ? "checked" : "";

	print "<td><b>Mount retries</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_r_def value=1 %s> Default\n",
		defined($self->{'options_hash'}->{"-R"}) ? "" : "checked";
	printf "<input type=radio name=nfs_r_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"-R"}) ? "checked" : "";
	print "<input size=5 name=nfs_r value=\"$self->{'options_hash'}->{'-R'}\"></td> </tr>\n";

	print "<tr> <td><b>Read-ahead blocks</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_a_def value=1 %s> Default\n",
		defined($self->{'options_hash'}->{"-a"}) ? "" : "checked";
	printf "<input type=radio name=nfs_a_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"-a"}) ? "checked" : "";
	print "<input size=5 name=nfs_a value=\"$self->{'options_hash'}->{'-a'}\"></td>\n";

	print "<td><b>RPC Protocol</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_t2 value=1 %s> TCP\n",
		defined($self->{'options_hash'}->{"-T"}) ? "checked" : "";
	printf "<input type=radio name=nfs_t2 value=0 %s> UDP</td> </tr>\n",
		defined($self->{'options_hash'}->{"-T"}) ? "" : "checked";
	}
elsif ($_[0] eq "msdos"){
	# MS-DOS filesystems options deal with filling in
	# missing unix functionality
	print "<tr> <td><b>User files are owned by</b></td>\n";
	printf "<td><input name=msdos_u size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{"-u"}) ? getpwuid($self->{'options_hash'}->{"-u"}) : "";
	print $self->user_chooser_button("msdos_u", 0),"</td>\n";

	print "<td><b>Group files are owned by</b></td>\n";
	printf "<td><input name=msdos_g size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{"-g"}) ? getgrgid($self->{'options_hash'}->{"-g"}) : "";
	print $self->group_chooser_button("msdos_g", 0),"</td>\n";

	print "<tr> <td><b>File permissions mask</b></td>\n";
	printf "<td><input type=radio name=msdos_m_def value=1 %s> Default\n",
		defined($self->{'options_hash'}->{"-m"}) ? "" : "checked";
	printf "<input type=radio name=msdos_m_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"-m"}) ? "checked" : "";
	print "<input size=5 name=msdos_m value=\"$self->{'options_hash'}->{'-m'}\"></td>\n";
	}
elsif ($_[0] eq "cd9660") {
	# CDROM filesystem
	print "<tr> <td><b>Ignore Unix Attributes?</b></td>\n";
	printf "<td><input type=radio name=cd9660_r value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"-r"}) ? "checked" : "";
	printf "<input type=radio name=cd9660_r value=0 %s> No</td>\n",
		defined($self->{'options_hash'}->{"-r"}) ? "" : "checked";

	print "<td><b>Show version numbers?</b></td>\n";
	printf "<td><input type=radio name=cd9660_g value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"-g"}) ? "checked" : "";
	printf "<input type=radio name=cd9660_g value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"-g"}) ? "" : "checked";

	print "<tr> <td><b>Use extended attributes?</b></td>\n";
	printf "<td><input type=radio name=cd9660_e value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"-e"}) ? "checked" : "";
	printf "<input type=radio name=cd9660_e value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"-e"}) ? "" : "checked";
	}
elsif ($_[0] eq "ntfs") {
	# Windows NT filesystem
	print "<tr> <td><b>Display MSDOS 8.3 filenames?</b></td>\n";
	printf "<td><input type=radio name=ntfs_a value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"-a"}) ? "checked" : "";
	printf "<input type=radio name=ntfs_a value=0 %s> No</td>\n",
		defined($self->{'options_hash'}->{"-a"}) ? "" : "checked";

	print "<td><b>Case sensitive filenames?</b></td>\n";
	printf "<td><input type=radio name=ntfs_i value=0 %s> Yes\n",
		defined($self->{'options_hash'}->{"-i"}) ? "" : "checked";
	printf "<input type=radio name=ntfs_i value=1 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"-i"}) ? "checked" : "";

	print "<tr> <td><b>User files are owned by</b></td>\n";
	printf "<td><input type=radio name=ntfs_u_def value=1 %s> Default\n",
		defined($self->{'options_hash'}->{"-u"}) ? "" : "checked";
	printf "<input type=radio name=ntfs_u_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"-u"}) ? "checked" : "";
	printf "<input name=ntfs_u size=8 value='%s'> %s</td>\n",
		defined($self->{'options_hash'}->{"-u"}) ? scalar(getpwuid($self->{'options_hash'}->{"-u"})) : "",
		$self->user_chooser_button("ntfs_u");

	print "<td><b>Group files are owned by</b></td>\n";
	printf "<td><input type=radio name=ntfs_g_def value=1 %s> Default\n",
		defined($self->{'options_hash'}->{"-u"}) ? "" : "checked";
	printf "<input type=radio name=ntfs_g_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"-u"}) ? "checked" : "";
	printf "<input name=ntfs_g size=8 value='%s'> %s</td> </tr>\n",
		defined($self->{'options_hash'}->{"-g"}) ? scalar(getgrgid($self->{'options_hash'}->{"-g"})) : "",
		$self->group_chooser_button("ntfs_g");
	}
elsif ($_[0] eq "swap") {
	# Swap has no options..
	print "<tr> <td><i>No Options Available</i></td> </tr>\n";
	}
elsif ($_[0] eq "smbfs") {
	# SMBFS has some special options
	print "<tr> <td><b>$self->{'text_hash'}->{'linux_username'}</b></td>\n";
	printf "<td><input name=smbfs_user size=15 value=\"%s\"></td>\n",
		$self->{'options_hash'}->{"user"};

	print "<td><b>$self->{'text_hash'}->{'linux_password'}</b></td>\n";
	printf "<td><input type=password name=smbfs_password size=15 value=\"%s\"></td> </tr>\n",
		$self->{'options_hash'}->{"nsmb_password"};

	print "<tr> <td><b>$self->{'text_hash'}->{'linux_wg'}</b></td>\n";
	printf "<td colspan=3><input type=radio name=smbfs_workgroup_def value=1 %s> $self->{'text_hash'}->{'linux_auto'}\n",
		defined($self->{'options_hash'}->{"nsmb_workgroup"}) ? "" : "checked";
	printf "<input type=radio name=smbfs_workgroup_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"nsmb_workgroup"}) ? "checked" : "";
	print "<input size=10 name=smbfs_workgroup value=\"$self->{'options_hash'}->{'nsmb_workgroup'}\"></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'linux_mname'}</b></td>\n";
	printf "<td colspan=3><input type=radio name=smbfs_addr_def value=1 %s> %s\n",
		defined($self->{'options_hash'}->{"nsmb_addr"}) ? "" : "checked", $self->{'text_hash'}->{'linux_auto'};
	printf "<input type=radio name=smbfs_addr_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"nsmb_addr"}) ? "checked" : "";
	print "<input size=30 name=smbfs_addr value=\"$self->{'options_hash'}->{'nsmb_addr'}\"></td> </tr>\n";
	}
}

# check_location(type)
# Parse and check inputs from %{$self->{'in_hash'}}, calling $self->error() if something is wrong.
# Returns the location string for storing in the fstab file
sub check_location
{
	my $self = shift;

if ($_[0] eq "nfs") {
	local($out, $temp, $mout, $dirlist);

	# Use ping and showmount to see if the host exists and is up
	if ($self->{'in_hash'}->{nfs_host} !~ /^\S+$/) {
		$self->error("'$self->{'in_hash'}->{nfs_host}' is not a valid hostname");
		}
	$self->execute_command("ping -c 1 '$self->{'in_hash'}->{nfs_host}'", undef, \$out, \$out);
	if ($out =~ /unknown host/i) {
		$self->error("The host '$self->{'in_hash'}->{nfs_host}' does not exist");
		}
	elsif ($out =~ /100\% packet loss/) {
		$self->error("The host '$self->{'in_hash'}->{nfs_host}' is down");
		}
	$self->execute_command("showmount -e '$self->{'in_hash'}->{nfs_host}'", undef, \$out, \$out);
	if ($out =~ /Unable to receive/) {
		$self->error("The host '$self->{'in_hash'}->{nfs_host}' does not support NFS");
		}
	elsif ($?) {
		$self->error("Failed to get mount list : $out");
		}

	# Validate directory name
	foreach (split(/\n/, $out)) {
		if (/^(\/\S+)/) { $dirlist .= "$1\n"; }
		}
	if ($self->{'in_hash'}->{nfs_dir} !~ /^\/\S+$/) {
		$self->error("'$self->{'in_hash'}->{nfs_dir}' is not a valid directory name. The ".
		       "available directories on $self->{'in_hash'}->{nfs_host} are:".
		       "<pre>$dirlist</pre>");
		}

	# Try a test mount to see if filesystem is available
	$temp = $self->transname();
	$self->make_dir($temp, 0755);
	$self->execute_command("mount $self->{'in_hash'}->{nfs_host}:$self->{'in_hash'}->{nfs_dir} $temp",
			 undef, \$mout, \$mout);
	if ($mout =~ /No such file or directory/) {
		$self->error("The directory '$self->{'in_hash'}->{nfs_dir}' does not exist on the ".
		       "host $self->{'in_hash'}->{nfs_host}. The available directories are:".
		       "<pre>$dirlist</pre>");
		}
	elsif ($mout =~ /Permission denied/) {
		$self->error("This host is not allowed to mount the directory ".
		       "$self->{'in_hash'}->{nfs_dir} from $self->{'in_hash'}->{nfs_host}");
		}
	elsif ($?) {
		$self->error("NFS Error - $mout");
		}
	# It worked! unmount
	$self->execute_command("umount $temp");
	$self->unlink_file($temp);
	return "$self->{'in_hash'}->{nfs_host}:$self->{'in_hash'}->{nfs_dir}";
	}
elsif ($_[0] eq "smbfs") {
	# A windows server filesystem .. check the server and share
	$self->{'in_hash'}->{'smbfs_server'} =~ /\S/ || $self->error($self->{'text_hash'}->{'linux_eserver'});
	$self->{'in_hash'}->{'smbfs_share'} =~ /\S/ || $self->error($self->{'text_hash'}->{'linux_eshare'});
	return "\\\\".lc($self->{'in_hash'}->{'smbfs_server'})."\\".lc($self->{'in_hash'}->{'smbfs_share'});
	}
else {
	# This is some kind of disk-based filesystem.. get the device name
	if ($self->{'in_hash'}->{'disk_dev'} == 0) {
		$self->{'in_hash'}->{'ide_t'} =~ /^\d+$/ ||
			$self->error("'$self->{'in_hash'}->{ide_t}' is not a valid device number");
		$self->{'in_hash'}->{'ide_s'} =~ /^\d+$/ ||
			$self->error("'$self->{'in_hash'}->{ide_s}' is not a valid slice number");
		$self->{'in_hash'}->{'ide_p'} =~ /^[a-z]*$/ ||
			$self->error("'$self->{'in_hash'}->{ide_p}' is not a valid partition letter");
		$self->{'dv'} = "/dev/ad$self->{'in_hash'}->{ide_t}s$self->{'in_hash'}->{ide_s}$self->{'in_hash'}->{ide_p}";
		}
	elsif ($self->{'in_hash'}->{'disk_dev'} == 1) {
		$self->{'in_hash'}->{'scsi_t'} =~ /^\d+$/ ||
			$self->error("'$self->{'in_hash'}->{scsi_t}' is not a valid device number");
		$self->{'in_hash'}->{'scsi_s'} =~ /^\d+$/ ||
			$self->error("'$self->{'in_hash'}->{scsi_s}' is not a valid slice number");
		$self->{'in_hash'}->{'scsi_p'} =~ /^[a-z]*$/ ||
			$self->error("'$self->{'in_hash'}->{scsi_p}' is not a valid partition letter");
		$self->{'dv'} = "/dev/da$self->{'in_hash'}->{scsi_t}s$self->{'in_hash'}->{scsi_s}$self->{'in_hash'}->{scsi_p}";
		}
	else {
		$self->{'dv'} = $self->{'in_hash'}->{'dev_path'};
		}

	# If the device entered is a symlink, follow it
	if ($self->{'dvlink'} = readlink($self->{'dv'})) {
		if ($self->{'dvlink'} =~ /^\//) { $self->{'dv'} = $self->{'dvlink'}; }
		else {	$self->{'dv'} =~ /^(.*\/)[^\/]+$/;
			$self->{'dv'} = $1.$self->{'dvlink'};
			}
		}

	# Check if the device actually exists and uses the right filesystem
	(-r $self->{'dv'}) || $self->error("The device file '$self->{'dv'}' does not exist");
	return $self->{'dv'};
	}
}

# check_options(type, device, directory)
# Read options for some filesystem from %{$self->{'in_hash'}}, and use them to update the
# %{$self->{'options_hash'}} array. Options handled by the user interface will be set or
# removed, while unknown options will be left untouched.
sub check_options
{
	my $self = shift;

local($k, @rv);

# Parse the common options first..
if ($_[0] ne "swap") {
	delete($self->{'options_hash'}->{"ro"}); delete($self->{'options_hash'}->{"rw"});
	delete($self->{'options_hash'}->{"rdonly"});
	if ($self->{'in_hash'}->{'bsd_ro'}) { $self->{'options_hash'}->{'ro'} = ''; }
	else { $self->{'options_hash'}->{'rw'} = ""; }

	delete($self->{'options_hash'}->{"sync"}); delete($self->{'options_hash'}->{"async"});
	if ($self->{'in_hash'}->{'bsd_sync'}) { $self->{'options_hash'}->{'sync'} = ''; }

	delete($self->{'options_hash'}->{'nodev'});
	if ($self->{'in_hash'}->{'bsd_nodev'}) { $self->{'options_hash'}->{'nodev'} = ''; }

	delete($self->{'options_hash'}->{'noexec'});
	if ($self->{'in_hash'}->{'bsd_noexec'}) { $self->{'options_hash'}->{'noexec'} = ''; }

	delete($self->{'options_hash'}->{'nosuid'});
	if ($self->{'in_hash'}->{'bsd_nosuid'}) { $self->{'options_hash'}->{'nosuid'} = ''; }

	delete($self->{'options_hash'}->{'noatime'});
	if ($self->{'in_hash'}->{'bsd_noatime'}) { $self->{'options_hash'}->{'noatime'} = ''; }

	if ($self->{'uname_release'} =~ /^[34]\./) {
		delete($self->{'options_hash'}->{'nosymfollow'});
		$self->{'options_hash'}->{'nosymfollow'} = '' if ($self->{'in_hash'}->{'bsd_nosymfollow'});

		delete($self->{'options_hash'}->{'suiddir'});
		$self->{'options_hash'}->{'suiddir'} = '' if ($self->{'in_hash'}->{'bsd_suiddir'});
		}
	}
else {
	# Swap always has the sw option
	$self->{'options_hash'}->{'sw'} = "";
	}

if ($_[0] eq "ufs") {
	# Parse UFS quota options
	delete($self->{'options_hash'}->{'userquota'}) if ($self->{'in_hash'}->{'ufs_userquota'} == 0);
	$self->{'options_hash'}->{'userquota'} = "" if ($self->{'in_hash'}->{'ufs_userquota'} == 1);
	$self->{'options_hash'}->{'userquota'} = $self->{'in_hash'}->{'ufs_groupquota_file'}
		if ($self->{'in_hash'}->{'ufs_userquota'} == 2);

	delete($self->{'options_hash'}->{'groupquota'}) if ($self->{'in_hash'}->{'ufs_groupquota'} == 0);
	$self->{'options_hash'}->{'groupquota'} = "" if ($self->{'in_hash'}->{'ufs_groupquota'} == 1);
	$self->{'options_hash'}->{'groupquota'} = $self->{'in_hash'}->{'ufs_groupquota_file'}
		if ($self->{'in_hash'}->{'ufs_groupquota'} == 2);
	}
elsif ($_[0] eq "nfs") {
	# NFS has a few specific options..
	delete($self->{'options_hash'}->{'-b'});
	$self->{'options_hash'}->{'-b'} = "" if ($self->{'in_hash'}->{'nfs_b'});

	delete($self->{'options_hash'}->{'-s'});
	$self->{'options_hash'}->{'-s'} = "" if ($self->{'in_hash'}->{'nfs_s'});

	delete($self->{'options_hash'}->{'-t'});
	$self->{'options_hash'}->{'-t'} = $self->{'in_hash'}->{'nfs_t'} if (!$self->{'in_hash'}->{'nfs_t_def'});

	delete($self->{'options_hash'}->{'-x'});
	$self->{'options_hash'}->{'-x'} = $self->{'in_hash'}->{'nfs_x'} if (!$self->{'in_hash'}->{'nfs_x_def'});

	delete($self->{'options_hash'}->{'-2'}); delete($self->{'options_hash'}->{'-3'});
	$self->{'options_hash'}->{'-2'} = "" if ($self->{'in_hash'}->{'nfs_ver'} == 2);
	$self->{'options_hash'}->{'-3'} = "" if ($self->{'in_hash'}->{'nfs_ver'} == 3);

	delete($self->{'options_hash'}->{'-R'});
	$self->{'options_hash'}->{'-R'} = $self->{'in_hash'}->{'nfs_r'} if (!$self->{'in_hash'}->{'nfs_r_def'});

	delete($self->{'options_hash'}->{'-a'});
	$self->{'options_hash'}->{'-a'} = $self->{'in_hash'}->{'nfs_a'} if (!$self->{'in_hash'}->{'nfs_a_def'});

	delete($self->{'options_hash'}->{'-T'});
	$self->{'options_hash'}->{'-T'} = "" if ($self->{'in_hash'}->{'nfs_t2'});
	}
elsif ($_[0] eq "msdos") {
	# MSDOS options for file ownership/perms
	delete($self->{'options_hash'}->{"-u"}); delete($self->{'options_hash'}->{"-g"});
	if ($self->{'in_hash'}->{'msdos_u'} ne "") { $self->{'options_hash'}->{'-u'} = getpwnam($self->{'in_hash'}->{'msdos_u'}); }
	if ($self->{'in_hash'}->{'msdos_g'} ne "") { $self->{'options_hash'}->{'-g'} = getgrnam($self->{'in_hash'}->{'msdos_g'}); }

	delete($self->{'options_hash'}->{"-m"});
	if (!$self->{'in_hash'}->{'msdos_m_def'}) {
		$self->{'in_hash'}->{'msdos_m'} =~ /^[0-7]{3}$/ ||
			$self->error("'$self->{'in_hash'}->{'msdos_m'}' is not a valid octal mask");
		$self->{'options_hash'}->{'-m'} = $self->{'in_hash'}->{'msdos_m'};
		}
	}
elsif ($_[0] eq "cd9660") {
	# Options for iso9660 cd-roms
	delete($self->{'options_hash'}->{'-r'});
	$self->{'options_hash'}->{'-r'} = "" if ($self->{'in_hash'}->{'cd9660_r'});

	delete($self->{'options_hash'}->{'-g'});
	$self->{'options_hash'}->{'-g'} = "" if ($self->{'in_hash'}->{'cd9660_g'});

	delete($self->{'options_hash'}->{'-e'});
	$self->{'options_hash'}->{'-e'} = "" if ($self->{'in_hash'}->{'cd9660_e'});
	}
elsif ($_[0] eq "ntfs") {
	delete($self->{'options_hash'}->{"-u"}); delete($self->{'options_hash'}->{"-g"});
	if ($self->{'in_hash'}->{'ntfs_u'} ne "") { $self->{'options_hash'}->{'-u'} = getpwnam($self->{'in_hash'}->{'ntfs_u'}); }
	if ($self->{'in_hash'}->{'ntfs_g'} ne "") { $self->{'options_hash'}->{'-g'} = getgrnam($self->{'in_hash'}->{'ntfs_g'}); }

	delete($self->{'options_hash'}->{"-a"});
	$self->{'options_hash'}->{"-a"} = '' if ($self->{'in_hash'}->{'ntfs_a'});

	delete($self->{'options_hash'}->{"-i"});
	$self->{'options_hash'}->{"-i"} = '' if ($self->{'in_hash'}->{'ntfs_i'});
	}
elsif ($_[0] eq "smbfs") {
	# Parse SMBFS options
	delete($self->{'options_hash'}->{'user'});
	$self->{'options_hash'}->{'user'} = $self->{'in_hash'}->{'smbfs_user'} if ($self->{'in_hash'}->{'smbfs_user'});

	delete($self->{'options_hash'}->{'nsmb_password'});
	$self->{'options_hash'}->{'nsmb_password'} = $self->{'in_hash'}->{'smbfs_password'}
		if ($self->{'in_hash'}->{'smbfs_password'});

	delete($self->{'options_hash'}->{'nsmb_addr'});
	if (!$self->{'in_hash'}->{"smbfs_addr_def"}) {
		$self->check_ipaddress($self->{'in_hash'}->{"smbfs_addr"}) ||
			$self->error($self->{'text_hash'}->{'freebsd_eaddr'});
		$self->{'options_hash'}->{'nsmb_addr'} = $self->{'in_hash'}->{"smbfs_addr"};
		}

	delete($self->{'options_hash'}->{'nsmb_workgroup'});
	if (!$self->{'in_hash'}->{"smbfs_workgroup_def"}) {
		$self->{'in_hash'}->{"smbfs_workgroup"} =~ /^\S+$/ ||
			$self->error($self->{'text_hash'}->{'freebsd_eworkgroup'});
		$self->{'options_hash'}->{'nsmb_workgroup'} = $self->{'in_hash'}->{"smbfs_workgroup"};
		}
	}

# Return options string
return $self->join_options();
}

# create_swap(file, size, units)
# Calls dd and mkswap to setup a swap file
sub create_swap
{
	my $self = shift;

local($out, $bl);
$bl = $_[1] * ($_[2] eq "g" ? 1024*1024 :
	       $_[2] eq "m" ? 1024 : 1);
$out = $self->backquote_logged("dd if=/dev/zero of=$_[0] bs=1024 count=$bl 2>&1");
if ($?) { return "dd failed : $out"; }
$out = $self->backquote_logged("mkswap $_[0] $bl 2>&1");
if ($?) { return "mkswap failed : $out"; }
$self->system_logged("sync >/dev/null 2>&1");
return 0;
}

# exports_list(host, dirarray, clientarray)
# Fills the directory and client array references with exports from some
# host. Returns an error string if something went wrong
sub exports_list
{
	my $self = shift;

local($dref, $cref, $out, $_);
$dref = $_[1]; $cref = $_[2];
$self->execute_command("showmount -e ".quotemeta($_[0]), undef, \$out, \$out, 0, 1);
if ($?) { return $out; }
foreach (split(/\n/, $out)) {
	if (/^(\/\S*)\s+(.*)$/) {
		push(@$dref, $1); push(@$cref, $2);
		}
	}
return undef;
}

# broadcast_addr()
# Returns a useable broadcast address for finding NFS servers
sub broadcast_addr
{
	my $self = shift;

local($out);
$self->execute_command("ifconfig -a", undef, \$out, \$out, 0, 1);
if ($out =~ /broadcast\s+(\S+)\s+/) { return $1; }
return "255.255.255.255";
}

sub device_name
{
	my $self = shift;

return $_[0];
}

sub files_to_lock
{
	my $self = shift;

return ( $self->{'config_hash'}->{'fstab_file'}, $self->{'nsmb_conf'} );
}

# get_nsmb_conf(server, share, user)
# Finds a single nsmb.conf section
sub get_nsmb_conf
{
	my $self = shift;

local $conf;
local $insection = 0;
local $lnum = 0;
open(CONF, $self->{'nsmb_conf'});
while(<CONF>) {
	s/\r|\n//g;
	s/^\s*#.*$//;
	if (/^\s*\[([^:]+):([^:]+):([^:]+)\]/ &&
	    lc($1) eq lc($_[0]) && lc($2) eq lc($_[2]) && lc($3) eq lc($_[1])) {
		# Start of section
		$insection = 1;
		$conf = { 'line' => $lnum,
			  'eline' => $lnum,
			  'server' => lc($_[0]),
			  'user' => lc($_[2]),
			  'share' => lc($_[1]) };
		}
	elsif (/^\s*\[.*\]/) {
		# Start of another section
		$insection = 0;
		}
	elsif (/^\s*(\S+)\s*=\s*(\S+)/ && $insection) {
		$conf->{'values'}->{lc($1)} = $2;
		$conf->{'eline'} = $lnum;
		}
	$lnum++;
	}
close(CONF);
return $conf;
}

# save_nsmb_conf(&conf)
# Updates or creates a single nsmb.conf section
sub save_nsmb_conf
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'nsmb_conf'});
local @lines = ( "[$_[0]->{'server'}:$_[0]->{'user'}:$_[0]->{'share'}]" );
foreach $k (keys %{$_[0]->{'values'}}) {
	push(@lines, $k."=".$_[0]->{'values'}->{$k});
	}
if (defined($_[0]->{'line'})) {
	# Modifying
	splice(@$lref, $_[0]->{'line'}, $_[0]->{'eline'} - $_[0]->{'line'} + 1,
	       @lines);
	}
else {
	# Adding
	push(@$lref, @lines);
	}
$self->flush_file_lines();
}

# update_nsmb(share, &options)
sub update_nsmb
{
	my $self = shift;

local ($server, $share, $user);
if ($_[0] =~ /^[\\\/]{2}(\S+)\@(\S+)[\\\/](\S+)$/) {
	($user, $server, $share) = ($1, $2, $3);
	}
elsif ($_[0] =~ /^[\\\/]{2}(\S+)[\\\/](\S+)$/) {
	($user, $server, $share) = ("root", $1, $2);
	}
else {
	$self->error("Invalid share $_[0]");
	}
local $conf = $self->get_nsmb_conf($server, $share, $user);
$conf ||= { "server" => $server,
	    "share" => $share,
	    "user" => $user };
$conf->{'values'} = { };
local %others;
foreach $k (keys %{$_[1]}) {
	if ($k =~ /^nsmb_(.*)$/) {
		$conf->{'values'}->{$1} = $_[1]->{$k};
		}
	else {
		$others{$k} = $_[1]->{$k};
		}
	}
$self->save_nsmb_conf($conf);
return \%others;
}

# read_nsmb(share)
# Returns a hash reference containing options for some share
sub read_nsmb
{
	my $self = shift;

local ($server, $share, $user);
if ($_[0] =~ /^[\\\/]{2}(\S+)\@(\S+)[\\\/](\S+)$/) {
	($user, $server, $share) = ($1, $2, $3);
	}
elsif ($_[0] =~ /^[\\\/]{2}(\S+)[\\\/](\S+)$/) {
	($user, $server, $share) = ("root", $1, $2);
	}
else {
	$self->error("Invalid share $_[0]");
	}
local $conf = $self->get_nsmb_conf($server, $share, $user);
if ($conf) {
	local (%rv, $k);
	foreach $k (keys %{$conf->{'values'}}) {
		$rv{"nsmb_".$k} = $conf->{'values'}->{$k};
		}
	return \%rv;
	}
return undef;
}


1;

