#!/usr/bin/perl
package mount::HpuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'mount::HpuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# hpux-lib.pl
# Filesystem functions for HP-UX (works for me on 10.xx and 11.00)

1;
1;
		return $self;
}

# Return information about a filesystem, in the form:
#  directory, device, type, options, fsck_order, mount_at_boot
# If a field is unused or ignored, a - appears instead of the value.
# Swap-filesystems (devices or files mounted for VM) have a type of 'swap',
# and 'swap' in the directory field
sub list_mounts
{
	my $self = shift;

local(@rv, @p, $_, $i); $i = 0;

# List normal filesystem mounts
open(FSTAB, $self->{'config_hash'}->{fstab_file});
while(<FSTAB>) {
	chop; s/#.*$//g;
	if (!/\S/) { next; }
	@p = split(/\s+/, $_);
	if ($p[2] eq "ignore") { next; }
	if ($p[2] eq "swap") { $p[1] = "swap"; }
	$rv[$i++] = [ $p[1], $p[0], $p[2], $p[3], $p[5], "yes" ];
	}
close(FSTAB);

# List automount points
open(AUTOTAB, $self->{'config_hash'}->{autofs_file});
while(<AUTOTAB>) {
	chop; s/#.*$//g;
	if (!/\S/ || /^[+\-]/) { next; }
	@p = split(/\s+/, $_);
	if ($p[2] eq "") { $p[2] = "-"; }
	else { $p[2] =~ s/^-//g; }
	$rv[$i++] = [ $p[0], $p[1], "autofs", $p[2], "-", "yes" ];
	}
close(AUTOTAB);

return @rv;
}

# create_mount(directory, device, type, options, fsck_order, mount_at_boot)
# Add a new entry to the fstab file, and return the index of the new entry
sub create_mount
{
	my $self = shift;

local($len, @mlist, $fsck, $dir);
if ($_[2] eq "autofs") {
	# An autofs mount.. add to /etc/auto_master
	$len = grep { $_->[2] eq "autofs" } ($self->list_mounts());
	$self->open_tempfile(AUTOTAB, ">> $self->{'config_hash'}->{autofs_file}");
	$self->print_tempfile(AUTOTAB, "$_[0] $_[1]",($_[3] eq "-" ? "" : " -$_[3]"),"\n");
	$self->close_tempfile(AUTOTAB);
	}
else {
	# Add to the fstab file
	$len = grep { $_->[2] ne "autofs" } ($self->list_mounts());
        if ($_[4] eq "-") { 
		$fsck = "0";
		}
        else {
		$fsck = $_[4];
		}
	$self->open_tempfile(FSTAB, ">> $self->{'config_hash'}->{fstab_file}");
	$self->print_tempfile(FSTAB, "$_[1] $_[0] $_[2] $_[3] 0 $fsck\n");
	$self->close_tempfile(FSTAB);
	}
return $len;
}

# delete_mount(index)
# Delete some mount from the table
sub delete_mount
{
	my $self = shift;

local(@fstab, $i, $line, $_);
open(FSTAB, $self->{'config_hash'}->{fstab_file});
@fstab = <FSTAB>;
close(FSTAB);
$i = 0;

$self->open_tempfile(FSTAB, "> $self->{'config_hash'}->{fstab_file}");
foreach (@fstab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $i++ == $_[0]) {
		# found the line not to include
		}
	else {
		$self->print_tempfile(FSTAB, $_,"\n");
		}
	}
$self->close_tempfile(FSTAB);

open(AUTOTAB, $self->{'config_hash'}->{autofs_file});
@{$self->{'autotab_array'}} = <AUTOTAB>;
close(AUTOTAB);
$self->open_tempfile(AUTOTAB, "> $self->{'config_hash'}->{autofs_file}");
foreach (@{$self->{'autotab_array'}}) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $line !~ /^[+\-]/ && $i++ == $_[0]) {
		# found line not to include..
		}
	else {
		$self->print_tempfile(AUTOTAB, $_,"\n");
		}
	}
$self->close_tempfile(AUTOTAB);
}

# change_mount(num, directory, device, type, options, fsck_order, mount_at_boot)
# Change an existing permanent mount
sub change_mount
{
	my $self = shift;

local(@fstab, @autotab, $i, $line, $fsck, $dir, $_);
$i = 0;

open(FSTAB, $self->{'config_hash'}->{fstab_file});
@fstab = <FSTAB>;
close(FSTAB);
$self->open_tempfile(FSTAB, "> $self->{'config_hash'}->{fstab_file}");
foreach (@fstab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $i++ == $_[0]) {
	        if ($_[5] eq "-") {
       	        	$fsck = "0";
                	}
        	else {
                	$fsck = $_[5];
                	}
		# Found the line to replace
		$self->print_tempfile(FSTAB, "$_[2] $_[1] $_[3] $_[4] 0 $fsck\n");
		}
	else {
		$self->print_tempfile(FSTAB, $_,"\n");
		}
	}
$self->close_tempfile(FSTAB);

open(AUTOTAB, $self->{'config_hash'}->{autofs_file});
@autotab = <AUTOTAB>;
close(AUTOTAB);
$self->open_tempfile(AUTOTAB, "> $self->{'config_hash'}->{autofs_file}");
foreach (@autotab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $line !~ /^[+\-]/ && $i++ == $_[0]) {
		# Found the line to replace
		$self->print_tempfile(AUTOTAB, "$_[1]  $_[2]  ",
				($_[4] eq "-" ? "" : "-$_[4]"),"\n");
		}
	else {
		$self->print_tempfile(AUTOTAB, $_,"\n");
		}
	}
$self->close_tempfile(AUTOTAB);
}

# list_mounted()
# Return a list of all the currently mounted filesystems and swap files.
# The list is in the form: directory device type options
# For swap files, the directory will be 'swap'
sub list_mounted
{
	my $self = shift;

local(@rv, @p, $_, $i, $r);
$self->open_execute_command(*SWAP, "swapinfo -a", 1, 1);
while(<SWAP>) {
	if (/^dev\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/) { push(@rv, [ "swap", $8, "swap", "pri=$7" ]); }
	}
close(SWAP);
$self->open_tempfile(MNTTAB, "/etc/mnttab");
while(<MNTTAB>) {
	s/#.*$//g; if (!/\S/) { next; }
	@p = split(/\s+/, $_);
	if ($p[2] eq "ignore") { next; }
	push(@rv, [ $p[1], $p[0], $p[2], $p[3] ]);
	}
close(MNTTAB);
return @rv;
}

# mount_dir(directory, device, type, options)
# Mount a new directory from some device, with some options. Returns 0 if ok,
# or an error string if failed. If the directory is 'swap', then mount as
# virtual memory.
sub mount_dir
{
	my $self = shift;

local($out, $opts);
if ($_[0] eq "swap") {
	# Adding a swap device
        local(%options, $opts);
        $self->parse_options("swap", $_[3]);
        if (defined($options{"pri"})) {
		$opts = "-p $options{'pri'}";
		}
	$out = $self->backquote_logged("swapon $opts $_[1] 2>&1");
	if ($? && !($out =~ /already enabled for paging/)) { return $out; }
	}
else {
	$opts = $_[3] eq "-" ? "" : "-o \"$_[3]\"";
	$out = $self->backquote_logged("mount -F $_[2] $opts -- $_[1] $_[0] 2>&1");
	if ($?) { return $out; }
	}
return 0;
}

# unmount_dir(directory, device, type)
# Unmount a directory (or swap device) that is currently mounted. Returns 0 if
# ok, or an error string if failed
sub unmount_dir
{
	my $self = shift;

if ($_[0] eq "swap") {
        # Not possible!
        $self->error("Swap space cannot be removed");
        }
else {
	$self->{'out'} = $self->backquote_logged("umount $_[0] 2>&1");
	}
if ($?) { return $self->{'out'}; }
return 0;
}

# disk_space(type, directory)
# Returns the amount of total and free space for some filesystem, or an
# empty array if not appropriate.
sub disk_space
{
	my $self = shift;

if ($self->get_mounted($_[1], "*") < 0) { return (); }
if ($_[0] eq "swap") {
	return ();
	}
local $out;
$self->execute_command("bdf $_[1]", undef, \$out, \$out, 0, 1);
if ($out =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/) {
	return ($1, $2);
	}
else {
	return ( );
	}
}

# list_fstypes()
# Returns an array of all the supported filesystem types. If a filesystem is
# found that is not one of the supported types, generate_location() and
# generate_options() will not be called for it.
sub list_fstypes
{
	my $self = shift;

return ("hfs", "vxfs", "swap", "cdfs", "nfs", "lofs");
}

# fstype_name(type)
# Given a short filesystem type, return a human-readable name for it
sub fstype_name
{
	my $self = shift;

local(%fsmap);
%fsmap = ("hfs","HP Unix Filesystem",
	  "vxfs","HP Journaled Unix Filesystem",
	  "nfs","Network Filesystem",
	  "cdfs","ISO9660 CD-ROM",
	  "lofs","Loopback Filesystem",
	  "swapfs","Filesystem Swap Space",
	  "swap","Virtual Memory",
	  "autofs","Automounter Filesystem");
return $self->{'config_hash'}->{long_fstypes} && $fsmap{$_[0]} ? $fsmap{$_[0]} : uc($_[0]);
}

# mount_modes(type)
# Given a filesystem type, returns 4 numbers that determine how the file
# system can be mounted, and whether it can be fsck'd
#  0 - cannot be permanently recorded
#  1 - can be permanently recorded, and is always mounted at boot
#  2 - can be permanently recorded, and may or may not be mounted at boot
# The second is:
#  0 - mount is always permanent => mounted when saved
#  1 - doesn't have to be permanent
# The third is:
#  0 - cannot be fsck'd at boot time
#  1 - can be be fsck'd at boot time
# The fourth is:
#  0 - can be unmounted
#  1 - cannot be unmounted
sub mount_modes
{
	my $self = shift;

if ($_[0] eq "hfs" || $_[0] eq "vxfs") {
	return (1, 1, 1, 0);
	}
elsif ($_[0] eq "swap") {
	return (1, 1, 0, 0);
	}
else { return (1, 1, 0, 0); }
}

# multiple_mount(type)
# Returns 1 if filesystems of this type can be mounted multiple times, 0 if not
sub multiple_mount
{
	my $self = shift;

return ($_[0] eq "nfs" || $_[0] eq "lofs");
}

# generate_location(type, location)
# Output HTML for editing the mount location of some filesystem.
sub generate_location
{
	my $self = shift;

if ($_[0] eq "nfs") {
	# NFS mount from some host and directory
	$self->{'onenfs'} = !$_[1] || $_[1] =~ /^([A-z0-9\-\.]+):([^,]+)$/;
	print "<tr> <td><b>NFS Hostname</b></td>\n";
	print "<td><input name=nfs_host size=20 value=\"$1\">\n";
	$self->nfs_server_chooser_button("nfs_host");
	print "</td>\n";
	print "<td><b>NFS Directory</b></td>\n";
	print "<td><input name=nfs_dir size=20 value=\"$2\">\n";
	$self->nfs_export_chooser_button("nfs_host", "nfs_dir");
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "hfs") {
	# Mounted from a normal disk, LVM device or from
	# somewhere else
	print "<tr> <td valign=top><b>HFS Device</b></td>\n";
	print "<td colspan=3>\n";
	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/) {
		$self->{'hfs_dev'} = 0;
		$self->{'scsi_c'} = $1; $self->{'scsi_t'} = $2; $self->{'scsi_d'} = $3; $self->{'scsi_s'} = $4;
		}
	elsif ($_[1] eq "") {
		$self->{'hfs_dev'} = 0; $self->{'scsi_c'} = $self->{'scsi_t'} = $self->{'scsi_s'} = $self->{'scsi_d'} = 0;
		}
	elsif ($_[1] =~ /^\/dev\/vg([0-9]+)\/(\S+)/) {
		$self->{'hfs_dev'} = 1; $self->{'scsi_vg'} = $1; $self->{'scsi_lv'} = $2;
		}
	else {
		$self->{'hfs_dev'} = 2; $self->{'scsi_path'} = $_[1];
		}
	$self->{'scsi_path'} = $_[1];

	printf "<input type=radio name=hfs_dev value=0 %s> SCSI Disk:\n",
		$self->{'hfs_dev'} == 0 ? "checked" : "";
	print "Controller <input name=hfs_c size=3 value=\"$self->{'scsi_c'}\">\n";
	print "Target <input name=hfs_t size=3 value=\"$self->{'scsi_t'}\">\n";
	print "Unit <input name=hfs_d size=3 value=\"$self->{'scsi_d'}\">\n";
	print "Partition <input name=hfs_s size=3 value=\"$self->{'scsi_s'}\"><br>\n";

	printf "<input type=radio name=hfs_dev value=1 %s> LVM Device:\n",
		$self->{'hfs_dev'} == 1 ? "checked" : "";
	print "Volume Group <input name=hfs_vg size=2 value=\"$self->{'scsi_vg'}\">\n";
	print "Logical Volume <input name=hfs_lv size=20 value=\"$self->{'scsi_lv'}\"><br>\n";

	printf "<input type=radio name=hfs_dev value=2 %s> Other Device:\n",
		$self->{'hfs_dev'} == 2 ? "checked" : "";
	print "<input name=hfs_path size=20 value=\"$self->{'scsi_path'}\">";
        print $self->file_chooser_button("hfs_path", 0);
        print "<br>\n";
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "vxfs") {
	# Mounted from a normal disk, LVM device or from
	# somewhere else
	print "<tr> <td valign=top><b>VXFS Device</b></td>\n";
	print "<td colspan=3>\n";
	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/) {
		$self->{'jfs_dev'} = 0;
		$self->{'scsi_c'} = $1; $self->{'scsi_t'} = $2; $self->{'scsi_d'} = $3; $self->{'scsi_s'} = $4;
		}
	elsif ($_[1] eq "") {
		$self->{'jfs_dev'} = 0; $self->{'scsi_c'} = $self->{'scsi_t'} = $self->{'scsi_s'} = $self->{'scsi_d'} = 0;
		}
	elsif ($_[1] =~ /^\/dev\/vg([0-9]+)\/(\S+)/) {
		$self->{'jfs_dev'} = 1; $self->{'scsi_vg'} = $1; $self->{'scsi_lv'} = $2;
		}
	else {
		$self->{'jfs_dev'} = 2; $self->{'scsi_path'} = $_[1];
		}
	$self->{'scsi_path'} = $_[1];

	printf "<input type=radio name=jfs_dev value=0 %s> SCSI Disk:\n",
		$self->{'jfs_dev'} == 0 ? "checked" : "";
	print "Controller <input name=jfs_c size=3 value=\"$self->{'scsi_c'}\">\n";
	print "Target <input name=jfs_t size=3 value=\"$self->{'scsi_t'}\">\n";
	print "Unit <input name=jfs_d size=3 value=\"$self->{'scsi_d'}\">\n";
	print "Partition <input name=jfs_s size=3 value=\"$self->{'scsi_s'}\"><br>\n";

	printf "<input type=radio name=jfs_dev value=1 %s> LVM Device:\n",
		$self->{'jfs_dev'} == 1 ? "checked" : "";
	print "Volume Group <input name=jfs_vg size=2 value=\"$self->{'scsi_vg'}\">\n";
	print "Logical Volume <input name=jfs_lv size=20 value=\"$self->{'scsi_lv'}\"><br>\n";

	printf "<input type=radio name=jfs_dev value=2 %s> Other Device:\n",
		$self->{'jfs_dev'} == 2 ? "checked" : "";
	print "<input name=jfs_path size=20 value=\"$self->{'scsi_path'}\">";
        print $self->file_chooser_button("jfs_path", 0);
        print "<br>\n";
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "swap") {
	# Swapping to a disk partition or a file
	print "<tr> <td valign=top><b>Swap File</b></td>\n";
	print "<td colspan=3>\n";
	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/) {
		$self->{'swap_dev'} = 0;
		$self->{'scsi_c'} = $1; $self->{'scsi_t'} = $2; $self->{'scsi_d'} = $3; $self->{'scsi_s'} = $4;
		}
	elsif ($_[1] =~ /^\/dev\/vg([0-9]+)\/(\S+)/) {
		$self->{'swap_dev'} = 1; $self->{'scsi_vg'} = $1; $self->{'scsi_lv'} = $2;
		}
	else {
		$self->{'swap_dev'} = 2; $self->{'scsi_path'} = $_[1];
		}
	$self->{'scsi_path'} = $_[1];
	printf "<input type=radio name=swap_dev value=0 %s> SCSI Disk:\n",
		$self->{'swap_dev'} == 0 ? "checked" : "";
	print "Controller <input name=swap_c size=3 value=\"$self->{'scsi_c'}\">\n";
	print "Target <input name=swap_t size=3 value=\"$self->{'scsi_t'}\">\n";
	print "Unit <input name=swap_d size=3 value=\"$self->{'scsi_d'}\">\n";
	print "Partition <input name=swap_s size=3 value=\"$self->{'scsi_s'}\"><br>\n";

	printf "<input type=radio name=swap_dev value=1 %s> LVM Device:\n",
		$self->{'swap_dev'} == 1 ? "checked" : "";
	print "Volume Group <input name=swap_vg size=2 value=\"$self->{'scsi_vg'}\">\n";
	print "Logical Volume <input name=swap_lv size=20 value=\"$self->{'scsi_lv'}\"><br>\n";

	printf "<input type=radio name=swap_dev value=2 %s> File:\n",
		$self->{'swap_dev'} == 2 ? "checked" : "";
	print "<input name=swap_path size=20 value=\"$self->{'scsi_path'}\">";
        print $self->file_chooser_button("swap_path", 0);
	print "<br>\n";
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "cdfs") {
	# Mounting a SCSI cdrom
	print "<tr> <td valign=top><b>CDROM Disk</b></td>\n";
	print "<td colspan=3>\n";
	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)$/) {
		$self->{'cdfs_dev'} = 0;
		$self->{'scsi_c'} = $1; $self->{'scsi_t'} = $2; $self->{'scsi_d'} = $3;
		}
	else {
		$self->{'cdfs_dev'} = 1; $self->{'scsi_path'} = $_[1];
		}
	$self->{'scsi_path'} = $_[1];
	printf "<input type=radio name=cdfs_dev value=0 %s> SCSI Device:\n",
		$self->{'cdfs_dev'} == 0 ? "checked" : "";
	print "Controller <input name=cdfs_c size=3 value=\"$self->{'scsi_c'}\">\n";
	print "Target <input name=cdfs_t size=3 value=\"$self->{'scsi_t'}\">\n";
	print "Unit <input name=cdfs_d size=3 value=\"$self->{'scsi_d'}\">\n<BR>";

	printf "<input type=radio name=cdfs_dev value=1 %s> Other Device:\n",
		$self->{'cdfs_dev'} == 1 ? "checked" : "";
	print "<input name=cdfs_path size=20 value=\"$self->{'scsi_path'}\">";
        print $self->file_chooser_button("cdfs_path", 0);
	print "<br>\n";
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "lofs") {
	# Mounting some directory to another location
	print "<tr> <td><b>Original Directory</b></td>\n";
	print "<td><input name=lofs_src size=30 value=\"$_[1]\">\n";
	print $self->file_chooser_button("lofs_src", 1);
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "swapfs") {
	# Mounting a cached filesystem of some type.. need a location for
	# the source of the mount
	print "<tr> <td><b>Cache Source</b></td>\n";
	print "<td><input name=cfs_src size=20 value=\"$_[1]\"></td> </tr>\n";
	}
elsif ($_[0] eq "autofs") {
	# An automounter entry.. can be -hosts, -xfn or from some mapping
	print "<tr> <td valign=top><b>Automounter map</b></td>\n";
	printf "<td><input type=radio name=autofs_type value=0 %s>\n",
		$_[1] eq "-hosts" || $_[1] eq "-xfn" ? "" : "checked";
	printf "Use map <input name=autofs_map size=20 value=\"%s\"><br>\n",
		$_[1] eq "-hosts" || $_[1] eq "-xfn" ? "" : $_[1];
	printf "<input type=radio name=autofs_type value=1 %s>\n",
		$_[1] eq "-hosts" ? "checked" : "";
	print "All NFS exports map<br>\n";
	printf "<input type=radio name=autofs_type value=2 %s>\n",
		$_[1] eq "-xfn" ? "checked" : "";
	print "Federated  Naming  Service map</td> </tr>\n";
	}
}

# $self->generate_options(type, newmount)
# Output HTML for editing mount options for a partilcar filesystem 
# under this OS
sub generate_options
{
	my $self = shift;

if ($_[0] eq "nfs") {
	# NFS has many options, not all of which are editable here
	print "<tr> <td><b>Read-Only?</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_ro value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
	printf "<input type=radio name=nfs_ro value=0 %s> No</td>\n",
		defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";

	print "<td><b>Disallow setuid programs?</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_nosuid value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=nfs_nosuid value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";

	print "<tr> <td><b>Return error on timeouts?</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_soft value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"soft"}) ? "checked" : "";
	printf "<input type=radio name=nfs_soft value=0 %s> No</td>\n",
		defined($self->{'options_hash'}->{"soft"}) ? "" : "checked";

	print "<td><b>Retry mounts in background?</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_bg value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"bg"}) ? "checked" : "";
	printf "<input type=radio name=nfs_bg value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"bg"}) ? "" : "checked";

	print "<tr> <td><b>Allow interrupts?</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_nointr value=0 %s> Yes\n",
		defined($self->{'options_hash'}->{"nointr"}) ? "" : "checked";
	printf "<input type=radio name=nfs_nointr value=1 %s> No</td>\n",
		defined($self->{'options_hash'}->{"nointr"}) ? "checked" : "";

	print "<td><b>Allow Access to Local Devices?</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_nodevs value=0 %s> Yes\n",
		defined($self->{'options_hash'}->{"nodevs"}) ? "" : "checked";
	printf "<input type=radio name=nfs_nodevs value=1 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"nodevs"}) ? "checked" : "";
	}
if ($_[0] eq "hfs") {
	print "<tr> <td><b>Read-Only?</b></td>\n";
	printf "<td nowrap><input type=radio name=hfs_ro value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
	printf "<input type=radio name=hfs_ro value=0 %s> No</td>\n",
		defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";

	print "<td><b>Disallow setuid programs?</b></td>\n";
	printf "<td nowrap><input type=radio name=hfs_nosuid value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=hfs_nosuid value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";

        print "<tr> <td><b>Enable quotas at boot time?</b></td>\n";
        printf "<td nowrap><input type=radio name=hfs_quota value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"quota"}) ? "checked" : "";
        printf "<input type=radio name=hfs_quota value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"quota"}) ? "" : "checked";
	}
if ($_[0] eq "vxfs") {
	print "<tr> <td><b>Read-Only?</b></td>\n";
	printf "<td nowrap><input type=radio name=jfs_ro value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
	printf "<input type=radio name=jfs_ro value=0 %s> No</td>\n",
		defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";

	print "<td><b>Disallow setuid programs?</b></td>\n";
	printf "<td nowrap><input type=radio name=jfs_nosuid value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=jfs_nosuid value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";

	print "<tr> <td><b>Full integrity for all Metadata?</b></td>\n";
	printf "<td nowrap><input type=radio name=jfs_log value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"log"}) ? "checked" : "";
	printf "<input type=radio name=jfs_log value=0 %s> No</td>\n",
		defined($self->{'options_hash'}->{"log"}) ? "" : "checked";

	print "<td><b>Synchronous-write data logging?</b></td>\n";
	printf "<td nowrap><input type=radio name=jfs_syncw value=1 %s> Yes\n",
		!defined($self->{'options_hash'}->{"nodatainlog"}) ? "checked" : "";
	printf "<input type=radio name=jfs_syncw value=0 %s> No</td> </tr>\n",
		!defined($self->{'options_hash'}->{"nodatainlog"}) ? "" : "checked";

        print "<tr> <td><b>Enable quotas at boot time?</b></td>\n";
        printf "<td nowrap><input type=radio name=jfs_quota value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"quota"}) ? "checked" : "";
        printf "<input type=radio name=jfs_quota value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"quota"}) ? "" : "checked";
	}
if ($_[0] eq "cdfs") {
	print "<tr> <td><b>Disallow setuid programs?</b></td>\n";
	printf"<td nowrap><input type=radio name=cdfs_nosuid value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=cdfs_nosuid value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";
	}
if ($_[0] eq "lofs") {
	print "<tr> <td><b>Read-Only?</b></td>\n";
	printf "<td nowrap><input type=radio name=lofs_ro value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
	printf "<input type=radio name=lofs_ro value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";
	}
if ($_[0] eq "swap") {
	local($i);
	print "<tr> <td><b>Priority</b></td>\n";
	print "<td><select name=swap_pri>\n";
	for ($i = 0; $i < 11; ++$i) {
		printf "<option value=\"%s\" %s>%s\n",
			$i, $self->{'options_hash'}->{"pri"} == $i ? "selected" : "", $i;
		}
	print "</select></td> </tr>\n";
	}
if ($_[0] eq "swapfs") {
	# The caching filesystem has lots of options.. cachefs mounts can
	# be of an existing 'manually' mounted back filesystem, or of a
	# back-filesystem that has been automatically mounted by the cache.
	# The user should never see the automatic mountings made by cachefs.
	print "<tr> <td><b>Real filesystem type</b></td>\n";
	print "<td nowrap><select name=cfs_backfstype>\n";
	if (!defined($self->{'options_hash'}->{backfstype})) { $self->{'options_hash'}->{backfstype} = "nfs"; }
	foreach ($self->list_fstypes()) {
		if ($_ eq "cachefs") { next; }
		printf "<option value=\"$_\" %s>$_\n",
			$_ eq $self->{'options_hash'}->{backfstype} ? "selected" : "";
		}
	print "</select></td>\n";

	print "<td><b>Real mount point</b></td>\n";
	printf"<td nowrap><input type=radio name=cfs_noback value=1 %s> Automatic\n",
		defined($self->{'options_hash'}->{"backpath"}) ? "" : "checked";
	printf "<input type=radio name=cfs_noback value=0 %s>\n",
		defined($self->{'options_hash'}->{"backpath"}) ? "checked" : "";
	print "<input size=10 name=cfs_backpath value=\"$self->{'options_hash'}->{backpath}\"></td> </tr>\n";

	print "<tr> <td><b>Cache directory</b></td>\n";
	printf "<td nowrap><input size=10 name=cfs_cachedir value=\"%s\"></td>\n",
		defined($self->{'options_hash'}->{"cachedir"}) ? $self->{'options_hash'}->{"cachedir"} : "/cache";

	print "<td><b>Write mode</b></td>\n";
	printf"<td nowrap><input type=radio name=cfs_wmode value=0 %s> Write-around\n",
		defined($self->{'options_hash'}->{"non-shared"}) ? "" : "checked";
	printf "<input type=radio name=cfs_wmode value=1 %s> Non-shared\n",
		defined($self->{'options_hash'}->{"non-shared"}) ? "checked" : "";
	print "</td> </tr>\n";

	print "<tr> <td><b>Consistency check</b></td>\n";
	print "<td><select name=cfs_con>\n";
	print "<option value=1> Periodically\n";
	printf "<option value=0 %s> Never\n",
		defined($self->{'options_hash'}->{"noconst"}) ? "selected" : "";
	printf "<option value=2 %s> On demand\n",
		defined($self->{'options_hash'}->{"demandconst"}) ? "selected" : "";
	print "</select></td>\n";

	print "<td><b>Check permissions in cache?</b></td>\n";
	printf "<td nowrap><input type=radio name=cfs_local value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"local-access"}) ? "checked" : "";
	printf "<input type=radio name=cfs_local value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"local-access"}) ? "" : "checked";

	print "<tr> <td><b>Read-Only?</b></td>\n";
	printf "<td nowrap><input type=radio name=cfs_ro value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
	printf "<input type=radio name=cfs_ro value=0 %s> No</td>\n",
		defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";

	print "<td><b>Disallow setuid programs?</b></td>\n";
	printf "<td nowrap><input type=radio name=cfs_nosuid value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=cfs_nosuid value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";
	}
if ($_[0] eq "autofs") {
	# Autofs has lots of options, depending on the type of file
	# system being automounted.. the fstype options determines this
	local($fstype);
	$fstype = $self->{'options_hash'}->{fstype} eq "" ? "nfs" : $self->{'options_hash'}->{fstype};
	$self->generate_options($fstype);
	print "<input type=hidden name=autofs_fstype value=\"$fstype\">\n";
	}
}

# check_location(type)
# Parse and check inputs from %{$self->{'in_hash'}}, calling $self->error() if something is wrong.
# Returns the location string for storing in the fstab file
sub check_location
{
	my $self = shift;

if ($_[0] eq "nfs") {
	local($out, $temp, $mout, $dirlist);

        # Use ping and showmount to see if the host exists and is up
        if ($self->{'in_hash'}->{nfs_host} !~ /^\S+$/) {
                $self->error("'$self->{'in_hash'}->{nfs_host}' is not a valid hostname");
                }
	$self->execute_command("ping -c 1 '$self->{'in_hash'}->{nfs_host}'", undef, \$out, \$out);
        if ($out =~ /unknown host/i) {
                $self->error("The host '$self->{'in_hash'}->{nfs_host}' does not exist");
                }
        elsif ($out =~ /100\% packet loss/) {
                $self->error("The host '$self->{'in_hash'}->{nfs_host}' is down");
                }
	$self->execute_command("showmount -e '$self->{'in_hash'}->{nfs_host}'", undef, \$out, \$out);
        if ($out =~ /Unable to receive/) {
                $self->error("The host '$self->{'in_hash'}->{nfs_host}' does not support NFS");
                }
        elsif ($?) {
                $self->error("Failed to get mount list : $out");
                }

        # Validate directory name
        foreach (split(/\n/, $out)) {
                if (/^(\/\S+)/) { $dirlist .= "$1\n"; }
                }
        if ($self->{'in_hash'}->{nfs_dir} !~ /^\/\S+$/) {
                $self->error("'$self->{'in_hash'}->{nfs_dir}' is not a valid directory name. The ".
                       "available directories on $self->{'in_hash'}->{nfs_host} are:".
                       "<pre>$dirlist</pre>");
                }

	# Try a test mount to see if filesystem is available
	$temp = $self->transname();
	$self->make_dir($temp, 0755);
	$self->execute_command("mount $self->{'in_hash'}->{nfs_host}:$self->{'in_hash'}->{nfs_dir} $temp",
			 undef, \$mout, \$mout);
	if ($mout =~ /No such file or directory/) {
		$self->error("The directory '$self->{'in_hash'}->{nfs_dir}' does not exist on the ".
		       "host $self->{'in_hash'}->{nfs_host}. The available directories are:".
		       "<pre>$dirlist</pre>");
		}
	elsif ($mout =~ /Permission denied/) {
		$self->error("This host is not allowed to mount the directory ".
		       "$self->{'in_hash'}->{nfs_dir} from $self->{'in_hash'}->{nfs_host}");
		}
	elsif ($?) {
		$self->error("NFS Error - $mout");
		}
	# It worked! unmount
	$self->execute_command("umount $temp");
	$self->unlink_file($temp);
	return "$self->{'in_hash'}->{nfs_host}:$self->{'in_hash'}->{nfs_dir}";
	}
elsif ($_[0] eq "hfs") {
	# Get the device name
	if ($self->{'in_hash'}->{hfs_dev} == 0) {
		$self->{'in_hash'}->{hfs_c} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{hfs_c}' is not a valid SCSI controller");
		$self->{'in_hash'}->{hfs_t} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{hfs_t}' is not a valid SCSI target");
		$self->{'in_hash'}->{hfs_d} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{hfs_d}' is not a valid SCSI unit");
		$self->{'in_hash'}->{hfs_s} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{hfs_s}' is not a valid SCSI partition");
		$self->{'dv'} = "/dev/dsk/c$self->{'in_hash'}->{hfs_c}t$self->{'in_hash'}->{hfs_t}d$self->{'in_hash'}->{hfs_d}s$self->{'in_hash'}->{hfs_s}";
		}
	elsif ($self->{'in_hash'}->{hfs_dev} == 1) {
		$self->{'in_hash'}->{hfs_vg} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{hfs_vg}' is not a valid Volume Group");
		$self->{'in_hash'}->{hfs_lv} =~ /^\S+$/ ||
			$self->error("'$self->{'in_hash'}->{hfs_lv}' is not a valid Logical Volume");
		$self->{'dv'} = "/dev/vg$self->{'in_hash'}->{hfs_vg}/$self->{'in_hash'}->{hfs_lv}";
		}
	else {
		$self->{'in_hash'}->{hfs_path} =~ /^\/\S+$/ ||
			$self->error("'$self->{'in_hash'}->{hfs_path}' is not a valid pathname");
		$self->{'dv'} = $self->{'in_hash'}->{hfs_path};
		}

	$self->fstyp_check($self->{'dv'}, "hfs");
	return $self->{'dv'};
	}
elsif ($_[0] eq "vxfs") {
	# Get the device name
	if ($self->{'in_hash'}->{jfs_dev} == 0) {
		$self->{'in_hash'}->{jfs_c} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{jfs_c}' is not a valid SCSI controller");
		$self->{'in_hash'}->{jfs_t} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{jfs_t}' is not a valid SCSI target");
		$self->{'in_hash'}->{jfs_d} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{jfs_d}' is not a valid SCSI unit");
		$self->{'in_hash'}->{jfs_s} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{jfs_s}' is not a valid SCSI partition");
		$self->{'dv'} = "/dev/dsk/c$self->{'in_hash'}->{jfs_c}t$self->{'in_hash'}->{jfs_t}d$self->{'in_hash'}->{jfs_d}s$self->{'in_hash'}->{jfs_s}";
		}
	elsif ($self->{'in_hash'}->{jfs_dev} == 1) {
		$self->{'in_hash'}->{jfs_vg} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{jfs_vg}' is not a valid Volume Group");
		$self->{'in_hash'}->{jfs_lv} =~ /^\S+$/ ||
			$self->error("'$self->{'in_hash'}->{jfs_lv}' is not a valid Logical Volume");
		$self->{'dv'} = "/dev/vg$self->{'in_hash'}->{jfs_vg}/$self->{'in_hash'}->{jfs_lv}";
		}
	else {
		$self->{'in_hash'}->{jfs_path} =~ /^\/\S+$/ ||
			$self->error("'$self->{'in_hash'}->{jfs_path}' is not a valid pathname");
		$self->{'dv'} = $self->{'in_hash'}->{jfs_path};
		}

	$self->fstyp_check($self->{'dv'}, "vxfs");
	return $self->{'dv'};
	}
elsif ($_[0] eq "lofs") {
	# Get and check the original directory
	$self->{'dv'} = $self->{'in_hash'}->{'lofs_src'};
	if (!(-r $self->{'dv'})) { $self->error("'$self->{'in_hash'}->{lofs_src}' does not exist"); }
	if (!(-d $self->{'dv'})) { $self->error("'$self->{'in_hash'}->{lofs_src}' is not a directory"); }
	return $self->{'dv'};
	}
elsif ($_[0] eq "swap") {
	if ($self->{'in_hash'}->{swap_dev} == 0) {
		$self->{'in_hash'}->{swap_c} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{swap_c}' is not a valid SCSI controller");
		$self->{'in_hash'}->{swap_t} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{swap_t}' is not a valid SCSI target");
		$self->{'in_hash'}->{swap_d} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{swap_d}' is not a valid SCSI unit");
		$self->{'in_hash'}->{swap_s} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{swap_s}' is not a valid SCSI partition");
		$self->{'dv'}="/dev/dsk/c$self->{'in_hash'}->{swap_c}t$self->{'in_hash'}->{swap_t}d$self->{'in_hash'}->{swap_d}s$self->{'in_hash'}->{swap_s}";
		}
	elsif ($self->{'in_hash'}->{swap_dev} == 1) {
		$self->{'in_hash'}->{swap_vg} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{swap_vg}' is not a valid Volume Group");
		$self->{'in_hash'}->{swap_lv} =~ /^\S+$/ ||
			$self->error("'$self->{'in_hash'}->{swap_lv}' is not a valid Logical Volume");
		$self->{'dv'} = "/dev/vg$self->{'in_hash'}->{swap_vg}/$self->{'in_hash'}->{swap_lv}";
		}
	else {
		$self->{'in_hash'}->{swap_path} =~ /^\/\S+$/ ||
			$self->error("'$self->{'in_hash'}->{swap_path}' is not a valid pathname");
		$self->{'dv'} = $self->{'in_hash'}->{swap_path};
		}
	$self->fstyp_check($self->{'dv'}, "swap");
	return $self->{'dv'};
	}
elsif ($_[0] eq "cdfs") {
	# Get the device name
	if ($self->{'in_hash'}->{cdfs_dev} == 0) {
		$self->{'in_hash'}->{cdfs_c} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{cdfs_c}' is not a valid SCSI controller");
		$self->{'in_hash'}->{cdfs_t} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{cdfs_t}' is not a valid SCSI target");
		$self->{'in_hash'}->{cdfs_d} =~ /^[0-9]+$/ ||
			$self->error("'$self->{'in_hash'}->{cdfs_d}' is not a valid SCSI unit");
		$self->{'dv'} = "/dev/dsk/c$self->{'in_hash'}->{cdfs_c}t$self->{'in_hash'}->{cdfs_t}d$self->{'in_hash'}->{cdfs_d}";
		}
	else {
		$self->{'in_hash'}->{cdfs_path} =~ /^\/\S+$/ ||
			$self->error("'$self->{'in_hash'}->{cdfs_path}' is not a valid pathname");
		$self->{'dv'} = $self->{'in_hash'}->{cdfs_path};
		}

	$self->fstyp_check($self->{'dv'}, "cdfs");
	return $self->{'dv'};
	}
elsif ($_[0] eq "swapfs") {
	# In order to check the location for the caching filesystem, we need
	# to check the back filesystem
	if (!$self->{'in_hash'}->{cfs_noback}) {
		# The back filesystem is manually mounted.. hopefully
		local($bidx, @mlist, @binfo);
		$bidx = $self->get_mounted($self->{'in_hash'}->{cfs_backpath}, "*");
		if ($bidx < 0) {
			$self->error("The back filesystem '$self->{'in_hash'}->{cfs_backpath}' is ".
			       "not mounted");
			}
		@mlist = $self->list_mounted();
		@binfo = @{$mlist[$bidx]};
		if ($binfo[2] ne $self->{'in_hash'}->{cfs_backfstype}) {
			$self->error("The back filesystem is '$binfo[2]', not ".
			       "'$self->{'in_hash'}->{cfs_backfstype}'");
			}
		}
	else {
		# Need to automatically mount the back filesystem.. check
		# it for sanity first.
		# But HOW?
		$self->{'in_hash'}->{cfs_src} =~ /^\S+$/ ||
			$self->error("'$self->{'in_hash'}->{cfs_src}' is not a valid cache source");
		}
	return $self->{'in_hash'}->{cfs_src};
	}
elsif ($_[0] eq "autofs") {
	# An autofs filesystem can be either mounted from the special
	# -hosts and -xfn maps, or from a normal map. The map can be a file
	# name (if it starts with /), or an NIS map (if it doesn't)
	if ($self->{'in_hash'}->{autofs_type} == 0) {
		# Normal map
		$self->{'in_hash'}->{autofs_map} =~ /\S/ ||
			$self->error("You did not enter an automount map name");
		if ($self->{'in_hash'}->{autofs_map} =~ /^\// && !(-r $self->{'in_hash'}->{autofs_map})) {
			$self->error("The map file '$self->{'in_hash'}->{autofs_map}' does not exist");
			}
		return $self->{'in_hash'}->{autofs_map};
		}
	elsif ($self->{'in_hash'}->{autofs_type} == 1) {
		# Special hosts map (automount all shares from some host)
		return "-hosts";
		}
	else {
		# Special FNS map (not sure what this does)
		return "-xfn";
		}
	}
}

# fstyp_check(device, type)
# Check if some device exists, and contains a filesystem of the given type,
# using the fstyp command.
sub fstyp_check
{
	my $self = shift;

local($out, $part, $found);

# Check if the device/partition actually exists
if ($_[0] =~ /^\/dev\/dsk\/c(.)t(.)d(.)s(.)$/) {
	# a normal scsi device..
	if (!$self->open_tempfile(DEV, $_[0], 0, 1)) {
		if ($! =~ /No such file or directory/) {
                	$self->error("The SCSI target for '$_[0]' does not exist");
			}
		elsif ($! =~ /No such device or address/) {
                	$self->error("The SCSI target for '$_[0]' does not exist");
			}
		}
	$self->close_tempfile(DEV);
	}
elsif ($_[0] =~ /^\/dev\/vg([0-9]+)\/(\S+)$/) {
	# Logical Volume device..
	$out = $self->backquote_command("lvdisplay -v $_[0] 2>&1");
	if ($out =~ /No such file or directory/) {
		$self->error("The Logical Volume device for '$_[0]' does not exist");
		}
	}
else {
	# Some other device
	if (!$self->open_tempfile(DEV, $_[0], 0, 1)) {
		if ($! =~ /No such file or directory/) {
			$self->error("The device file '$_[0]' does not exist");
			}
		elsif ($! =~ /No such device or address/) {
			$self->error("The device for '$_[0]' does not exist");
			}
		}
	$self->close_tempfile(DEV);
	}

# Check the filesystem type
if ($_[1] ne "cdfs" && $_[1] ne "swap") {
	$out = $self->backquote_command("fstyp $_[0] 2>&1");
	if ($out =~ /^([A-z0-9]+)\n$/) {
		if ($1 eq $_[1]) { return; }
		else {
			# Wrong filesystem type
			$self->error("The device '$_[0]' is formatted as a ".
			       $self->fstype_name($1));
			}
		}
	else {
		$self->error("Failed to check filesystem type : $out");
		}
	}
}

# $self->check_options(type)
# Read options for some filesystem from %{$self->{'in_hash'}}, and use them to update the
# %{$self->{'options_hash'}} array. Options handled by the user interface will be set or
# removed, while unknown options will be left untouched.
sub check_options
{
	my $self = shift;

local($k, @rv);
delete($self->{'options_hash'}->{"defaults"});

if ($_[0] eq "nfs") {
	# NFS has lots of options to parse
	if ($self->{'in_hash'}->{'nfs_ro'}) {
		# Read-only
		$self->{'options_hash'}->{"ro"} = ""; delete($self->{'options_hash'}->{"rw"});
		}
	else {
		# Read-write
		$self->{'options_hash'}->{"rw"} = ""; delete($self->{'options_hash'}->{"ro"});
		}

	if ($self->{'in_hash'}->{hfs_nosuid}) {
		# nosuid
		$self->{'options_hash'}->{"nosuid"} = ""; delete($self->{'options_hash'}->{"suid"});
		}
	else {
		# suid
		$self->{'options_hash'}->{"suid"} = ""; delete($self->{'options_hash'}->{"nosuid"});
		}

	delete($self->{'options_hash'}->{"soft"}); delete($self->{'options_hash'}->{"hard"});
	if ($self->{'in_hash'}->{nfs_soft}) { $self->{'options_hash'}->{"soft"} = ""; }

	delete($self->{'options_hash'}->{"bg"}); delete($self->{'options_hash'}->{"fg"});
	if ($self->{'in_hash'}->{nfs_bg}) { $self->{'options_hash'}->{"bg"} = ""; }

	delete($self->{'options_hash'}->{"intr"}); delete($self->{'options_hash'}->{"nointr"});
	if ($self->{'in_hash'}->{nfs_nointr}) { $self->{'options_hash'}->{"nointr"} = ""; }

	delete($self->{'options_hash'}->{"devs"}); delete($self->{'options_hash'}->{"nodevs"});
	if ($self->{'in_hash'}->{nfs_nodevs}) { $self->{'options_hash'}->{"nodevs"} = ""; }
	}
elsif ($_[0] eq "hfs") {
	if ($self->{'in_hash'}->{hfs_ro}) {
		# read-only
		$self->{'options_hash'}->{"ro"} = ""; delete($self->{'options_hash'}->{"rw"});
		}
	else {
		# read-write
		$self->{'options_hash'}->{"rw"} = ""; delete($self->{'options_hash'}->{"ro"});
		}
	if ($self->{'in_hash'}->{hfs_nosuid}) {
		# nosuid
		$self->{'options_hash'}->{"nosuid"} = ""; delete($self->{'options_hash'}->{"suid"});
		}
	else {
		# suid
		$self->{'options_hash'}->{"suid"} = ""; delete($self->{'options_hash'}->{"nosuid"});
		}
	if ($self->{'in_hash'}->{hfs_quota}) {
		# quota
		$self->{'options_hash'}->{"quota"} = "";
		}
	else {
		# noquota
		delete($self->{'options_hash'}->{"quota"});
		}
	}
elsif ($_[0] eq "vxfs") {
	if ($self->{'in_hash'}->{jfs_ro}) {
		# read-only
		$self->{'options_hash'}->{"ro"} = ""; delete($self->{'options_hash'}->{"rw"});
		}
	else {
		# read-write
		$self->{'options_hash'}->{"rw"} = ""; delete($self->{'options_hash'}->{"ro"});
		}
	if ($self->{'in_hash'}->{jfs_nosuid}) {
		# nosuid
		$self->{'options_hash'}->{"nosuid"} = ""; delete($self->{'options_hash'}->{"suid"});
		}
	else {
		# suid
		$self->{'options_hash'}->{"suid"} = ""; delete($self->{'options_hash'}->{"nosuid"});
		}
	if ($self->{'in_hash'}->{jfs_log}) {
		# log
		$self->{'options_hash'}->{"log"} = ""; delete($self->{'options_hash'}->{"delaylog"});
		}
	else {
		# delaylog
		$self->{'options_hash'}->{"delaylog"} = ""; delete($self->{'options_hash'}->{"log"});
		}
	if ($self->{'in_hash'}->{jfs_syncw}) {
		# datainlog
		$self->{'options_hash'}->{"datainlog"} = ""; delete($self->{'options_hash'}->{"nodatainlog"});
		}
	else {
		# nodatainlog
		$self->{'options_hash'}->{"nodatainlog"} = ""; delete($self->{'options_hash'}->{"datainlog"});
		}
	if ($self->{'in_hash'}->{jfs_quota}) {
		# quota
		$self->{'options_hash'}->{"quota"} = "";
		}
	else {
		# noquota
		delete($self->{'options_hash'}->{"quota"});
		}
	}
elsif ($_[0] eq "lofs") {
	if ($self->{'in_hash'}->{lofs_ro}) {
		# read-only
		$self->{'options_hash'}->{"ro"} = "";
		}
	else {
		# read-write
		$self->{'options_hash'}->{"defaults"} = "";
		}
	}
elsif ($_[0] eq "swap") {
	$self->{'options_hash'}->{"pri"} = $self->{'in_hash'}->{swap_pri};
	}
elsif ($_[0] eq "cdfs") {
	# read-only
	$self->{'options_hash'}->{"ro"} = "";
	if ($self->{'in_hash'}->{cdfs_nosuid}) {
		# nosuid
		$self->{'options_hash'}->{"nosuid"} = ""; delete($self->{'options_hash'}->{"suid"});
		}
	else {
		# suid
		$self->{'options_hash'}->{"suid"} = ""; delete($self->{'options_hash'}->{"nosuid"});
		}
	}
elsif ($_[0] eq "tmpfs") {
	# Ram-disk filesystems have only one option
	delete($self->{'options_hash'}->{"size"});
	if (!$self->{'in_hash'}->{"tmpfs_size_def"}) {
		$self->{'options_hash'}->{"size"} = "$self->{'in_hash'}->{tmpfs_size}$self->{'in_hash'}->{tmpfs_unit}";
		}
	}
elsif ($_[0] eq "swapfs") {
	# The caching filesystem has lots of options
	$self->{'options_hash'}->{"backfstype"} = $self->{'in_hash'}->{"cfs_backfstype"};

	delete($self->{'options_hash'}->{"backpath"});
	if (!$self->{'in_hash'}->{"cfs_noback"}) {
		# A back filesystem was given..  (alreadys checked)
		$self->{'options_hash'}->{"backpath"} = $self->{'in_hash'}->{"cfs_backpath"};
		}

	if ($self->{'in_hash'}->{"cfs_cachedir"} !~ /^\/\S+/) {
		$self->error("'$self->{'in_hash'}->{cfs_cachedir}' is not a valid cache directory");
		}
	$self->{'options_hash'}->{"cachedir"} = $self->{'in_hash'}->{"cfs_cachedir"};

	delete($self->{'options_hash'}->{"write-around"}); delete($self->{'options_hash'}->{"non-shared"});
	if ($self->{'in_hash'}->{"cfs_wmode"}) {
		$self->{'options_hash'}->{"non-shared"} = "";
		}

	delete($self->{'options_hash'}->{"noconst"}); delete($self->{'options_hash'}->{"demandconst"});
	if ($self->{'in_hash'}->{"cfs_con"} == 0) { $self->{'options_hash'}->{"noconst"} = ""; }
	elsif ($self->{'in_hash'}->{"cfs_con"} == 2) { $self->{'options_hash'}->{"demandconst"} = ""; }

	delete($self->{'options_hash'}->{"ro"}); delete($self->{'options_hash'}->{"rw"});
	if ($self->{'in_hash'}->{"cfs_ro"}) { $self->{'options_hash'}->{"ro"} = ""; }

	delete($self->{'options_hash'}->{"suid"}); delete($self->{'options_hash'}->{"nosuid"});
	if ($self->{'in_hash'}->{"cfs_nosuid"}) { $self->{'options_hash'}->{"nosuid"} = ""; }
	}
elsif ($_[0] eq "autofs") {
	# The options for autofs depend on the type of the automounted
	# filesystem.. 
	$self->{'options_hash'}->{"fstype"} = $self->{'in_hash'}->{"autofs_fstype"};
	return $self->check_options($self->{'options_hash'}->{"fstype"});
	}

# Return options string
foreach $k (keys %{$self->{'options_hash'}}) {
	if ($self->{'options_hash'}->{$k} eq "") { push(@rv, $k); }
	else { push(@rv, "$k=$self->{'options_hash'}->{$k}"); }
	}
return @rv ? join(',' , @rv) : "-";
}

# create_swap(path, size, units)
# Attempt to create a swap file 
sub create_swap
{
	my $self = shift;

local($out);
$out = $self->backquote_logged("mkfile $_[1]$_[2] $_[0] 2>&1");
if ($?) {
	$self->unlink_file($_[0]);
	return "mkfile failed : $out";
	}
return 0;
}

# exports_list(host, dirarray, clientarray)
# Fills the directory and client array references with exports from some
# host. Returns an error string if something went wrong
sub exports_list
{
	my $self = shift;

local($dref, $cref, $out, $_);
$dref = $_[1]; $cref = $_[2];
$out = $self->backquote_command("showmount -e ".quotemeta($_[0])." 2>&1", 1);
if ($?) { return $out; }
foreach (split(/\n/, $out)) {
	if (/^(\/\S*)\s+(.*)$/) {
		push(@$dref, $1); push(@$cref, $2);
		}
	}
return undef;
}

# broadcast_addr()
# Returns a useable broadcast address for finding NFS servers
sub broadcast_addr
{
	my $self = shift;

local($out, $hostname, $broadcast, @tmp);
$hostname = $self->get_system_hostname();
$out = $self->backquote_command("netstat -i 2>&1 | grep $hostname", 1);
if ($out =~ /\s+(\S*)\s+(\S*)\s+(.*)/) {
	$broadcast = "$2.255.255.255";
	@tmp = split(/\./,$broadcast);
	$broadcast = "$tmp[0].$tmp[1].$tmp[2].$tmp[3]";
	return $broadcast;
	}
return "255.255.255.255";
}

sub device_name
{
	my $self = shift;

return $_[0];
}

sub files_to_lock
{
	my $self = shift;

return ( $self->{'config_hash'}->{'fstab_file'} );
}


1;

