#!/usr/bin/perl
package mount::IrixLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'mount::IrixLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# irix-lib.pl
# Filesystem functions for IRIX
# XXX logging and locking?

1;
1;
		return $self;
}

# Return information about a filesystem, in the form:
#  directory, device, type, options, fsck_order, mount_at_boot
# If a field is unused or ignored, a - appears instead of the value.
# Swap-filesystems (devices or files mounted for VM) have a type of 'swap',
# and 'swap' in the directory field
sub list_mounts
{
	my $self = shift;

local(@rv, @p, $_, $i); $i = 0;
open(FSTAB, $self->{'config_hash'}->{'fstab_file'});
while(<FSTAB>) {
	chop; s/#.*$//g;
	if (!/\S/) { next; }
	@p = split(/\s+/, $_);
	$rv[$i] = [ $p[1], $p[0], $p[2] ];
	$rv[$i]->[5] = "yes";
	local @o = split(/,/ , $p[3] eq "defaults" ? "" : $p[3]);
	if (($self->{'j'} = $self->indexof("noauto", @o)) >= 0) {
		# filesytem is not mounted at boot
		splice(@o, $self->{'j'}, 1);
		$rv[$i]->[5] = "no";
		}
	$rv[$i]->[3] = (@o ? join(',' , @o) : "-");
	$rv[$i]->[4] = (@p >= 5 ? $p[5] : 0);
	$i++;
	}
close(FSTAB);

# List automount points
open(AUTOTAB, $self->{'config_hash'}->{'autofs_file'});
while(<AUTOTAB>) {
	chop; s/#.*$//g;
	if (!/\S/ || /^[+\-]/) { next; }
	@p = split(/\s+/, $_);
	if ($p[2] eq "") { $p[2] = "-"; }
	else { $p[2] =~ s/^-//g; }
	$rv[$i++] = [ $p[0], $p[1], "autofs", $p[2], "-", "yes" ];
	}
close(AUTOTAB);

return @rv;
}

# create_mount(directory, device, type, options, fsck_order, mount_at_boot)
# Add a new entry to the fstab file, and return the index of the new entry
sub create_mount
{
	my $self = shift;

local($len, @mlist, $fcsk, $dir);
if ($_[2] eq "autofs") {
	# An autofs mount.. add to /etc/auto_master
	$len = grep { $_->[2] eq "autofs" } ($self->list_mounts());
	$self->open_tempfile(AUTOTAB, ">> $self->{'config_hash'}->{autofs_file}");
	$self->print_tempfile(AUTOTAB, "$_[0] $_[1]",($_[3] eq "-" ? "" : " -$_[3]"),"\n");
	$self->close_tempfile(AUTOTAB);
	$self->system_logged("autofs -r >/dev/null 2>&1 </dev/null");
	}
else {
	# Add to the fstab file
	$len = grep { $_->[2] ne "autofs" } ($self->list_mounts());
	$self->open_tempfile(FSTAB, ">> $self->{'config_hash'}->{fstab_file}");
	$self->print_tempfile(FSTAB, "$_[1]  $_[0]  $_[2]");
	local $opts = $_[3] eq "-" ? "" : $_[3];
	if ($_[5] eq "no") {
		$opts = join(',' , (split(/,/ , $opts) , "noauto"));
		}
	if ($opts eq "") {
		$self->print_tempfile(FSTAB, "  defaults");
		}
	else {
		$self->print_tempfile(FSTAB, "  $opts");
		}
	$self->print_tempfile(FSTAB, "  0  ");
	$self->print_tempfile(FSTAB, $_[4] eq "-" ? "0\n" : "$_[4]\n");
	$self->close_tempfile(FSTAB);
	}
return $len;
}

# delete_mount(index)
# Delete some mount from the table
sub delete_mount
{
	my $self = shift;

local(@fstab, @autotab, $i, $line, $_);
open(FSTAB, $self->{'config_hash'}->{fstab_file});
@fstab = <FSTAB>;
close(FSTAB);
$i = 0;

$self->open_tempfile(FSTAB, "> $self->{'config_hash'}->{fstab_file}");
foreach (@fstab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $i++ == $_[0]) {
		# found the line not to include
		}
	else {
		$self->print_tempfile(FSTAB, $_,"\n");
		}
	}
$self->close_tempfile(FSTAB);

local $found;
open(AUTOTAB, $self->{'config_hash'}->{autofs_file});
@autotab = <AUTOTAB>;
close(AUTOTAB);
$self->open_tempfile(AUTOTAB, "> $self->{'config_hash'}->{autofs_file}");
foreach (@autotab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $line !~ /^[+\-]/ && $i++ == $_[0]) {
		# found line not to include..
		$line =~ /^(\S+)/;
		$found = $1;
		}
	else {
		$self->print_tempfile(AUTOTAB, $_,"\n");
		}
	}
$self->close_tempfile(AUTOTAB);
if ($found) {
	$self->system_logged("autofs -r >/dev/null 2>&1 </dev/null");
	$self->system_logged("umount $found >/dev/null 2>&1 </dev/null");
	}
}

# change_mount(num, directory, device, type, options, fsck_order, mount_at_boot)
# Change an existing permanent mount
sub change_mount
{
	my $self = shift;

local(@fstab, @autotab, $i, $line, $fsck, $dir, $_);
$i = 0;

open(FSTAB, $self->{'config_hash'}->{fstab_file});
@fstab = <FSTAB>;
close(FSTAB);
$self->open_tempfile(FSTAB, "> $self->{'config_hash'}->{fstab_file}");
foreach (@fstab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $i++ == $_[0]) {
		# Found the line to replace
		$self->print_tempfile(FSTAB, "$_[2]  $_[1]  $_[3]");
		local $opts = $_[4] eq "-" ? "" : $_[4];
		if ($_[6] eq "no") {
			$opts = join(',' , (split(/,/ , $opts) , "noauto"));
			}
		if ($opts eq "") {
			$self->print_tempfile(FSTAB, "  defaults");
			}
		else {
			$self->print_tempfile(FSTAB, "  $opts");
			}
		$self->print_tempfile(FSTAB, "  0  ");
		$self->print_tempfile(FSTAB, $_[5] eq "-" ? "0\n" : "$_[5]\n");
		}
	else {
		$self->print_tempfile(FSTAB, $_,"\n");
		}
	}
$self->close_tempfile(FSTAB);

local $found;
open(AUTOTAB, $self->{'config_hash'}->{autofs_file});
@autotab = <AUTOTAB>;
close(AUTOTAB);
$self->open_tempfile(AUTOTAB, "> $self->{'config_hash'}->{autofs_file}");
foreach (@autotab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $line !~ /^[+\-]/ && $i++ == $_[0]) {
		# Found the line to replace
		$self->print_tempfile(AUTOTAB, "$_[1]  $_[2]  ",
				($_[4] eq "-" ? "" : "-$_[4]"),"\n");
		$found++;
		}
	else {
		$self->print_tempfile(AUTOTAB, $_,"\n");
		}
	}
$self->close_tempfile(AUTOTAB);
if ($found) {
	$self->system_logged("autofs -r >/dev/null 2>&1 </dev/null");
	}
}

# list_mounted()
# Return a list of all the currently mounted filesystems and swap files.
# The list is in the form:
#  directory device type options
# For swap files, the directory will be 'swap'
sub list_mounted
{
	my $self = shift;

local(@rv, @p, $_, $i, $r);
$self->open_execute_command(*SWAP, "swap -l 2>/dev/null", 1, 1);
while(<SWAP>) {
	if (/^\s*\d+\s+(\/\S+)/) {
		push(@rv, [ "swap", $1, "swap", "-" ]);
		}
	}
close(SWAP);
open(MTAB, "/etc/mtab");
while(<MTAB>) {
	s/#.*$//g; if (!/\S/) { next; }
	@p = split(/\s+/, $_);
	push(@rv, [ $p[1], $p[0], $p[2], $p[3] ]);
	}
close(MTAB);
return @rv;
}

# mount_dir(directory, device, type, options)
# Mount a new directory from some device, with some options. Returns 0 if ok,
# or an error string if failed. If the directory is 'swap', then mount as
# virtual memory.
sub mount_dir
{
	my $self = shift;

local($out, $opts);
if ($_[2] eq "swap") {
	# Adding a swap device
	$out = $self->backquote_logged("swap -a $_[1] 2>&1");
	if ($?) { return $out; }
	}
elsif ($_[2] eq "autofs") {
	# Do nothing, because autofs -r will be run after the auto_master file
	# is updated.
	}
else {
	# Mounting a directory
	$opts = $_[3] eq "-" ? "" : "-o \"$_[3]\"";
	$out = $self->backquote_logged("mount -t $_[2] $opts $_[1] $_[0] 2>&1");
	if ($?) { return $out; }
	}
return 0;
}

# unmount_dir(directory, device, type)
# Unmount a directory (or swap device) that is currently mounted. Returns 0 if
# ok, or an error string if failed
sub unmount_dir
{
	my $self = shift;

if ($_[2] eq "swap") {
	$self->{'out'} = $self->backquote_logged("swap -d $_[1] 2>&1");
	}
elsif ($_[2] eq "autofs") {
	# Do nothing - see the comment in mount_dir
	return 0;
	}
else {
	$self->{'out'} = $self->backquote_logged("umount $_[0] 2>&1");
	}
if ($?) { return $self->{'out'}; }
return 0;
}

# disk_space(type, directory)
# Returns the amount of total and free space for some filesystem, or an
# empty array if not appropriate.
sub disk_space
{
	my $self = shift;

if ($self->get_mounted($_[1], "*") < 0) { return (); }
if ($_[0] eq "fd" || $_[0] eq "proc" || $_[0] eq "swap" || $_[0] eq "autofs") {
	return ();
	}
if ($self->backquote_command("df -k $_[1]", 1) =~
    /Mounted on\n\S+\s+\S+\s+(\S+)\s+\S+\s+(\S+)/) {
	return ($1, $2);
	}
else {
	return ( );
	}
}

# list_fstypes()
# Returns an array of all the supported filesystem types. If a filesystem is
# found that is not one of the supported types, generate_location() and
# generate_options() will not be called for it.
sub list_fstypes
{
	my $self = shift;

return ("xfs","efs","nfs","iso9660","dos","autofs","swap");
}

# fstype_name(type)
# Given a short filesystem type, return a human-readable name for it
sub fstype_name
{
	my $self = shift;

local(%fsmap);
%fsmap = ("xfs","SGI Filesystem",
	  "nfs","Network Filesystem",
	  "cachefs","Caching Filesystem",
	  "iso9660","ISO9660 CD-ROM",
	  "dos","MS-DOS Filesystem",
	  "efs","Old SGI Filesystem",
	  "fd","File Descriptor Filesystem",
	  "hfs","Macintosh Filesystem",
	  "kfs","AppleShare Filesystem",
	  "proc","Process Image Filesystem",
	  "hwgfs","Hardware Configuration Filesystem",
	  "autofs","Automounter Filesystem",
	  "nfs3","NIS Maps Filesystem",
	  "swap","Virtual Memory");
return $self->{'config_hash'}->{long_fstypes} && $fsmap{$_[0]} ? $fsmap{$_[0]} : uc($_[0]);
}

# mount_modes(type, dir, device)
# Given a filesystem type, returns 5 numbers that determine how the file
# system can be mounted, and whether it can be fsck'd
sub mount_modes
{
	my $self = shift;

if ($_[0] eq "swap") {
	return $_[2] eq "/dev/swap" ? (1, 0, 0, 0, 1) : (1, 1, 0, 0);
	}
elsif ($_[0] eq "fd" || $_[0] eq "proc" || $_[0] eq "hwgfs" ||
       $_[0] eq "nfs3") {
	return (1, 0, 0, 1);
	}
elsif ($_[0] eq "xfs" || $_[0] eq "efs" || $_[0] eq "cachefs") {
	return (2, 1, 1, 0);
	}
elsif ($_[0] eq "autofs") {
	return (1, 0, 0, 0);
	}
else {
	return (2, 1, 0, 0);
	}
}

# multiple_mount(type)
# Returns 1 if filesystems of this type can be mounted multiple times, 0 if not
sub multiple_mount
{
	my $self = shift;

return ($_[0] eq "nfs" || $_[0] eq "kfs" || $_[0] eq "cachefs");
}

# generate_location(type, location)
# Output HTML for editing the mount location of some filesystem.
sub generate_location
{
	my $self = shift;

if ($_[0] eq "nfs") {
	# NFS mount from some host and directory
	local ($nfshost, $nfspath) = $_[1] =~ /^([^:]+):(.*)/ ? ( $1, $2 ) : ();
	print "<tr> <td><b>$self->{'text_hash'}->{'irix_nhost'}</b></td>\n";
	print "<td><input name=nfs_host size=20 value=\"$nfshost\">\n";
	$self->nfs_server_chooser_button("nfs_host");
	print "</td>\n";
	print "<td><b>$self->{'text_hash'}->{'irix_ndir'}</b></td>\n";
	print "<td><input name=nfs_dir size=20 value=\"$nfspath\">\n";
	$self->nfs_export_chooser_button("nfs_host", "nfs_dir");
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "autofs") {
	# Selecting an automounter map
	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'irix_auto'}</b></td>\n";
	print "<td colspan=3>\n";
	printf "<input type=radio name=auto_mode value=0 %s> %s<br>\n",
		$_[1] eq "-hosts" ? "checked" : "", $self->{'text_hash'}->{'irix_autohosts'};
	printf "<input type=radio name=auto_mode value=1 %s> %s\n",
		$_[1] eq "-hosts" ? "" : "checked", $self->{'text_hash'}->{'irix_automap'};
	printf "<input name=auto_map size=30 value='%s'><br>\n",
		$_[1] eq "-hosts" ? "" : $_[1];
	}
elsif ($_[0] eq "iso9660") {
	# Selecting an entire SCSI disk
	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'irix_cd'}</b></td>\n";
	print "<td colspan=3>\n";
	local ($mode, $ctrlr, $drive, $other);
	if ($_[1] =~ /^\/dev\/rdsk\/dks(\d+)d(\d+)vol$/) {
		$mode = 0;
		$ctrlr = $1;
		$drive = $2;
		}
	elsif ($_[1]) {
		$mode = 1;
		$other = $_[1];
		}
	printf "<input type=radio name=irix_mode value=0 %s>\n",
		$mode == 0 ? "checked" : "";
	print $self->text('irix_mode0cd',
		    "<input name=irix_ctrlr size=2 value='$ctrlr'>",
		    "<input name=irix_drive size=2 value='$drive'>"),"<br>\n";
	printf "<input type=radio name=irix_mode value=1 %s> %s\n",
		$mode == 1 ? "checked" : "", $self->{'text_hash'}->{'irix_mode1'};
	printf "<input name=irix_other size=30 value='%s'><br>\n",
		$mode == 1 ? $other : "";
	print "</td> </tr>\n";

	}
else {
	# Mounting some local disk (or file)
	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'irix_part'}</b></td>\n";
	print "<td colspan=3>\n";
	local ($mode, $ctrlr, $drive, $part, $other);
	if ($_[1] =~ /^\/dev\/dsk\/dks(\d+)d(\d+)s(\d+)$/) {
		$mode = 0;
		$ctrlr = $1;
		$drive = $2;
		$part = $3;
		}
	elsif ($_[1]) {
		$mode = 1;
		$other = $_[1];
		}
	printf "<input type=radio name=irix_mode value=0 %s>\n",
		$mode == 0 ? "checked" : "";
	local ($sel, $currctrlr, $currdrive);
	open(VTOC, "prtvtoc -a |");
	while(<VTOC>) {
		if (/^\/dev\/rdsk\/dks(\d+)d(\d+)vh/) {
			$currctrlr = $1;
			$currdrive = $2;
			}
		elsif (/^\/dev/) {
			undef($currctrlr);
			}
		elsif (/^\s*(\d+)\s+(\d+)\s+(\d+)/ && defined($currctrlr)) {
			$sel .= sprintf "<option value=%s %s>%s\n",
				"/dev/dsk/dks${currctrlr}d${currdrive}s$1",
				$currctrlr == $ctrlr && $currdrive == $drive &&
				$1 == $part ? "selected" : "",
				$self->text('irix_mode0', $currctrlr,$currdrive,"$1");
			}
		}
	close(VTOC);
	if ($sel) {
		print "<select name=irix_sel>\n";
		print $sel;
		print "</select>\n";
		}
	else {
		print $self->text('irix_mode0',
			    "<input name=irix_ctrlr size=2 value='$ctrlr'>",
			    "<input name=irix_drive size=2 value='$drive'>",
			    "<input name=irix_part size=2 value='$part'>"),"\n";
		}
	print "<br>\n";
	printf "<input type=radio name=irix_mode value=1 %s> %s\n",
		$mode == 1 ? "checked" : "", $self->{'text_hash'}->{'irix_mode1'};
	printf "<input name=irix_other size=30 value='%s'><br>\n",
		$mode == 1 ? $other : "";
	print "</td> </tr>\n";
	}
}

# generate_options(type, newmount)
# Output HTML for editing mount options for a partilcar filesystem 
# under this OS
sub generate_options
{
	my $self = shift;

if ($_[0] ne "swap") {
	# All filesystem types have these options
	print "<tr> <td><b>$self->{'text_hash'}->{'irix_ro'}</b></td>\n";
	printf "<td nowrap><input type=radio name=irix_ro value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
	printf "<input type=radio name=irix_ro value=0 %s> No</td>\n",
		defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";

	print "<td><b>$self->{'text_hash'}->{'irix_nosuid'}</b></td>\n";
	printf "<td nowrap><input type=radio name=irix_nosuid value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=irix_nosuid value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";

	print "<tr> <td><b>$self->{'text_hash'}->{'irix_grpid'}</b></td>\n";
	printf "<td nowrap><input type=radio name=irix_grpid value=0 %s> Yes\n",
		defined($self->{'options_hash'}->{"grpid"}) ? "" : "checked";
	printf "<input type=radio name=irix_grpid value=1 %s> No</td>\n",
		defined($self->{'options_hash'}->{"grpid"}) ? "checked" : "";

	print "<td><b>$self->{'text_hash'}->{'irix_nodev'}</b></td>\n";
	printf "<td nowrap><input type=radio name=irix_nodev value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"nodev"}) ? "checked" : "";
	printf "<input type=radio name=irix_nodev value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"nodev"}) ? "" : "checked";
	}
else {
	# No options available for swap
	print "<tr> <td><i>$self->{'text_hash'}->{'irix_noopts'}</i></td> </tr>\n";
	}

if ($_[0] eq "nfs") {
	# Irix NFS
	print "<tr> <td><b>$self->{'text_hash'}->{'irix_bg'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_bg value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"bg"}) ? "checked" : "";
	printf "<input type=radio name=nfs_bg value=0 %s> No</td>\n",
		defined($self->{'options_hash'}->{"bg"}) ? "" : "checked";

	print "<td><b>$self->{'text_hash'}->{'irix_soft'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_soft value=1 %s> Yes\n",
		defined($self->{'options_hash'}->{"soft"}) ? "checked" : "";
	printf "<input type=radio name=nfs_soft value=0 %s> No</td> </tr>\n",
		defined($self->{'options_hash'}->{"soft"}) ? "" : "checked";

	print "<tr> <td><b>$self->{'text_hash'}->{'irix_nointr'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_nointr value=0 %s> Yes\n",
		defined($self->{'options_hash'}->{"nointr"}) ? "" : "checked";
	printf "<input type=radio name=nfs_nointr value=1 %s> No</td>\n",
		defined($self->{'options_hash'}->{"nointr"}) ? "checked" : "";

	print "<td><b>$self->{'text_hash'}->{'irix_version'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_vers_def value=1 %s> %s\n",
		defined($self->{'options_hash'}->{"vers"}) ? "" : "checked", $self->{'text_hash'}->{'default'};
	printf "<input type=radio name=nfs_vers_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"vers"}) ? "checked" : "";
	print "<input size=2 name=nfs_vers value='$self->{'options_hash'}->{vers}'></td></tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'irix_proto'}</b></td>\n";
	print "<td nowrap><select name=proto>\n";
	printf "<option value=\"\" %s> Default\n",
		defined($self->{'options_hash'}->{"proto"}) ? "" : "selected";
	printf "<option value=tcp %s> TCP\n",
		$self->{'options_hash'}->{"proto"} eq "tcp" ? "selected" : "";
	printf "<option value=udp %s> UDP\n",
		$self->{'options_hash'}->{"proto"} eq "udp" ? "selected" : "";
	print "</select></td>\n";

	print "<td><b>$self->{'text_hash'}->{'irix_port'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_port_def value=1 %s> %s\n",
		defined($self->{'options_hash'}->{"port"}) ? "" : "checked", $self->{'text_hash'}->{'default'};
	printf "<input type=radio name=nfs_port_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"port"}) ? "checked" : "";
	print "<input size=5 name=nfs_port value='$self->{'options_hash'}->{port}'></td></tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'irix_timeo'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_timeo_def value=1 %s> %s\n",
		defined($self->{'options_hash'}->{"timeo"}) ? "" : "checked", $self->{'text_hash'}->{'default'};
	printf "<input type=radio name=nfs_timeo_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"timeo"}) ? "checked" : "";
	printf "<input size=5 name=nfs_timeo value='$self->{'options_hash'}->{timeo}'></td>\n";

	print "<td><b>$self->{'text_hash'}->{'irix_retrans'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_retrans_def value=1 %s> %s\n",
		defined($self->{'options_hash'}->{"retrans"}) ? "" : "checked", $self->{'text_hash'}->{'default'};
	printf "<input type=radio name=nfs_retrans_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"retrans"}) ? "checked" : "";
	print "<input size=5 name=nfs_retrans value='$self->{'options_hash'}->{retrans}'></td> </tr>\n";

	print "</tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'irix_quota'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_quota value=1 %s> %s\n",
		defined($self->{'options_hash'}->{"quota"}) ? "checked" : "", $self->{'text_hash'}->{'yes'};
	printf "<input type=radio name=nfs_quota value=0 %s> %s</td>\n",
		defined($self->{'options_hash'}->{"quota"}) ? "" : "checked", $self->{'text_hash'}->{'no'};

	print "</tr>\n";
	}
elsif ($_[0] eq "xfs") {
	# Irix XFS options
	print "<tr> <td><b>$self->{'text_hash'}->{'irix_noatime'}</b></td>\n";
	printf "<td nowrap><input type=radio name=xfs_noatime value=0 %s> %s\n",
		defined($self->{'options_hash'}->{"noatime"}) ? "" : "checked", $self->{'text_hash'}->{'yes'};
	printf "<input type=radio name=xfs_noatime value=1 %s> %s</td>\n",
		defined($self->{'options_hash'}->{"noatime"}) ? "checked" : "", $self->{'text_hash'}->{'no'};

	print "<td><b>$self->{'text_hash'}->{'irix_wsync'}</b></td>\n";
	printf "<td nowrap><input type=radio name=xfs_wsync value=1 %s> %s\n",
		defined($self->{'options_hash'}->{"wsync"}) ? "checked" : "", $self->{'text_hash'}->{'yes'};
	printf "<input type=radio name=xfs_wsync value=0 %s> %s</td> </tr>\n",
		defined($self->{'options_hash'}->{"wsync"}) ? "" : "checked", $self->{'text_hash'}->{'no'};

	local $qm = defined($self->{'options_hash'}->{"qnoenforce"}) ? 2 :
		    defined($self->{'options_hash'}->{"quota"}) ? 1 : 0;
	print "<tr> <td><b>$self->{'text_hash'}->{'irix_quota'}</b></td> <td colspan=3>\n";
	printf "<input type=radio name=xfs_quota value=1 %s> %s\n",
		$qm == 1 ? "checked" : "", $self->{'text_hash'}->{'yes'};
	printf "<input type=radio name=xfs_quota value=2 %s> %s\n",
		$qm == 2 ? "checked" : "", $self->{'text_hash'}->{'irix_quotano'};
	printf "<input type=radio name=xfs_quota value=0 %s> %s</td> </tr>\n",
		$qm == 0 ? "checked" : "", $self->{'text_hash'}->{'no'};
	}
elsif ($_[0] eq "efs") {
	# Irix EFS (old filesystem) options
	print "<tr> <td><b>$self->{'text_hash'}->{'irix_quota'}</b></td>\n";
	printf "<td nowrap><input type=radio name=efs_quota value=1 %s> %s\n",
		defined($self->{'options_hash'}->{"quota"}) ? "checked" : "", $self->{'text_hash'}->{'yes'};
	printf "<input type=radio name=efs_quota value=0 %s> %s</td>\n",
		defined($self->{'options_hash'}->{"quota"}) ? "" : "checked", $self->{'text_hash'}->{'no'};

	print "<td><b>$self->{'text_hash'}->{'irix_nofsck'}</b></td>\n";
	printf "<td nowrap><input type=radio name=efs_nofsck value=0 %s> %s\n",
		defined($self->{'options_hash'}->{"nofsck"}) ? "" : "checked", $self->{'text_hash'}->{'yes'};
	printf "<input type=radio name=efs_nofsck value=1 %s> %s</td> </tr>\n",
		defined($self->{'options_hash'}->{"nofsck"}) ? "checked" : "", $self->{'text_hash'}->{'no'};
	}
elsif ($_[0] eq "dos") {
	# There are no additional dos vfat options
	}
elsif ($_[0] eq "iso9660") {
	# Irix CD-ROM filesystem options
	print "<tr> <td><b>$self->{'text_hash'}->{'irix_setx'}</b></td>\n";
	printf "<td nowrap><input type=radio name=iso_setx value=1 %s> %s\n",
		defined($self->{'options_hash'}->{"setx"}) ? "checked" : "", $self->{'text_hash'}->{'yes'};
	printf "<input type=radio name=iso_setx value=0 %s> %s</td>\n",
		defined($self->{'options_hash'}->{"setx"}) ? "" : "checked", $self->{'text_hash'}->{'no'};

	local $rock = !defined($self->{'options_hash'}->{"norrip"}) &&
		      !defined($self->{'options_hash'}->{"noext"});
	print "<td><b>$self->{'text_hash'}->{'irix_rock'}</b></td>\n";
	printf "<td nowrap><input type=radio name=iso_rock value=1 %s> %s\n",
		$rock ? "checked" : "", $self->{'text_hash'}->{'yes'};
	printf "<input type=radio name=iso_rock value=0 %s> %s</td> </tr>\n",
		$rock ? "" : "checked", $self->{'text_hash'}->{'no'};
	}
elsif ($_[0] eq "autofs") {
	# Irix automounter options
	}
}

# check_location(type)
# Parse and check inputs from %{$self->{'in_hash'}}, calling $self->error() if something is wrong.
# Returns the location string for storing in the fstab file
sub check_location
{
	my $self = shift;

if ($_[0] eq "nfs") {
	# Use ping and showmount to see if the host exists and is up
	local $out = $self->backquote_command("ping -c 1 '$self->{'in_hash'}->{nfs_host}' 2>&1");
	if ($out =~ /unknown host/i) {
		$self->error($self->text('irix_ehost', $self->{'in_hash'}->{'nfs_host'}));
		}
	elsif ($out =~ /100\% packet loss/) {
		$self->error($self->text('irix_eloss', $self->{'in_hash'}->{'nfs_host'}));
		}
	$out = $self->backquote_command("showmount -e '$self->{'in_hash'}->{nfs_host}' 2>&1");
	if ($out =~ /port mapper failure/i) {
		$self->error($self->text('irix_eshowmount', $self->{'in_hash'}->{'nfs_host'}));
		}
	elsif ($?) {
		$self->error($self->text('irix_emountlist', "<pre>$out</pre>"));
		}

	# Validate directory name
	foreach (split(/\n/, $out)) {
		if (/^(\/\S+)/) { $self->{'dirlist'} .= "$1\n"; }
		}
	if ($self->{'in_hash'}->{'nfs_dir'} !~ /^\/\S+$/) {
		$self->error($self->text('irix_edirlist', $self->{'in_hash'}->{'nfs_dir'}, $self->{'in_hash'}->{'nfs_host'},
					      "<pre>$self->{'dirlist'}</pre>"));
		}

	# Try a test mount to see if filesystem is available
	local $temp = $self->transname();
	$self->make_dir($temp, 0755);
	local $mout;
	$self->execute_command("mount $self->{'in_hash'}->{'nfs_host'}:$self->{'in_hash'}->{'nfs_dir'} $temp 2>&1",
			 undef, \$mout, \$mout);
	if ($mout =~ /No such file or directory/i) {
		$self->error($self->text('irix_enotexist', $self->{'in_hash'}->{'nfs_dir'}, $self->{'in_hash'}->{'nfs_host'},
			     "<pre>$self->{'dirlist'}</pre>"));
		}
	elsif ($mout =~ /Permission denied/i) {
		$self->error($self->text('irix_epermission', $self->{'in_hash'}->{'nfs_dir'},
			     $self->{'in_hash'}->{'nfs_host'}));
		}
	elsif ($?) {
		$self->error($self->text('irix_enfserr', $mout));
		}
	# It worked! unmount
	$self->execute_command("umount $temp");
	$self->unlink_file($temp);
	return "$self->{'in_hash'}->{nfs_host}:$self->{'in_hash'}->{nfs_dir}";
	}
elsif ($_[0] eq "autofs") {
	# Check automounter map
	if ($self->{'in_hash'}->{'auto_mode'} == 0) {
		return "-hosts";
		}
	else {
		$self->{'in_hash'}->{'auto_map'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'irix_eautomap2'});
		$self->{'in_hash'}->{'auto_map'} =~ /^\// && !-r $self->{'in_hash'}->{'auto_map'} &&
			$self->error($self->text('irix_eautomap', $self->{'in_hash'}->{'auto_map'}));
		return $self->{'in_hash'}->{'auto_map'};
		}
	}
else {
	# Get the device name
	local $dv;
	if ($self->{'in_hash'}->{'irix_mode'} == 0 && defined($self->{'in_hash'}->{'irix_sel'})) {
		$dv = $self->{'in_hash'}->{'irix_sel'};
		}
	elsif ($self->{'in_hash'}->{'irix_mode'} == 0) {
		$self->{'in_hash'}->{'irix_ctrlr'} =~ /^\d+$/ ||
			$self->error($self->text('irix_ectrlr', $self->{'in_hash'}->{'irix_ctrlr'}));
		$self->{'in_hash'}->{'irix_drive'} =~ /^\d+$/ ||
			$self->error($self->text('irix_edrive', $self->{'in_hash'}->{'irix_drive'}));
		if ($_[0] eq "iso9660") {
			$dv = "/dev/rdsk/dks$self->{'in_hash'}->{'irix_ctrlr'}d$self->{'in_hash'}->{'irix_drive'}vol";
			}
		else {
			$self->{'in_hash'}->{'irix_part'} =~ /^\d+$/ ||
				$self->error($self->text('irix_epart', $self->{'in_hash'}->{'irix_part'}));
			$dv = "/dev/dsk/dks$self->{'in_hash'}->{'irix_ctrlr'}d$self->{'in_hash'}->{'irix_drive'}s$self->{'in_hash'}->{'irix_part'}";
			}
		}
	else {
		$dv = $self->{'in_hash'}->{'irix_other'};
		}

	# Check the filesystem type
	if ($_[0] eq "xfs" || $_[0] eq "efs") {
		local $out = $self->backquote_command("fstyp '$dv' 2>&1");
		$out =~ s/[^A-Za-z0-9]//g;
		if ($?) {
			$self->error($self->text('irix_efstyp2', $dv, $out));
			}
		elsif ($out ne $_[0]) {
			$self->error($self->text('irix_efstyp', $dv, $out, $_[0]));
			}
		}

	# Check if the file exists
	if (!-r $dv) {
		if ($_[0] eq "swap") {
			if ($dv =~ /^\/dev/) {
				$self->error($self->text('solaris_eswapfile', $dv));
				}
			else {
				$self->swap_form($dv);
				}
			}
		else {
			$self->error($self->text('irix_edevice', $dv));
			}
		}
	return $dv;
	}
}

# check_options(type)
# Read options for some filesystem from %{$self->{'in_hash'}}, and use them to update the
# %{$self->{'options_hash'}} array. Options handled by the user interface will be set or
# removed, while unknown options will be left untouched.
sub check_options
{
	my $self = shift;

local($k, @rv);

# Parse options common to all Irix filesystems
if ($_[0] ne "swap") {
	if ($self->{'in_hash'}->{'irix_ro'}) {
		# Read-only
		$self->{'options_hash'}->{"ro"} = "";
		delete($self->{'options_hash'}->{"rw"});
		}
	else {
		# Read-write
		$self->{'options_hash'}->{"rw"} = "";
		delete($self->{'options_hash'}->{"ro"});
		}

	delete($self->{'options_hash'}->{"nosuid"}); delete($self->{'options_hash'}->{"suid"});
	if ($self->{'in_hash'}->{'irix_nosuid'}) { $self->{'options_hash'}->{"nosuid"} = ""; }

	delete($self->{'options_hash'}->{"grpid"});
	if ($self->{'in_hash'}->{'irix_grpid'}) { $self->{'options_hash'}->{"grpid"} = ""; }

	delete($self->{'options_hash'}->{"nodev"}); delete($self->{'options_hash'}->{"dev"});
	if ($self->{'in_hash'}->{'irix_nodev'}) { $self->{'options_hash'}->{"nodev"} = ""; }
	}

if ($_[0] eq "nfs") {
	# Parse Irix NFS options
	delete($self->{'options_hash'}->{"bg"}); delete($self->{'options_hash'}->{"fg"});
	if ($self->{'in_hash'}->{nfs_bg}) { $self->{'options_hash'}->{"bg"} = ""; }

	delete($self->{'options_hash'}->{"soft"}); delete($self->{'options_hash'}->{"hard"});
	if ($self->{'in_hash'}->{nfs_soft}) { $self->{'options_hash'}->{"soft"} = ""; }

	delete($self->{'options_hash'}->{"intr"}); delete($self->{'options_hash'}->{"nointr"});
	if ($self->{'in_hash'}->{nfs_nointr}) { $self->{'options_hash'}->{"nointr"} = ""; }

	delete($self->{'options_hash'}->{"vers"});
	if (!$self->{'in_hash'}->{nfs_vers_def}) { $self->{'options_hash'}->{"vers"} = $self->{'in_hash'}->{nfs_vers}; }

	delete($self->{'options_hash'}->{"proto"});
	if ($self->{'in_hash'}->{nfs_proto} ne "") { $self->{'options_hash'}->{"proto"} = $self->{'in_hash'}->{nfs_proto}; }

	delete($self->{'options_hash'}->{"port"});
	if (!$self->{'in_hash'}->{nfs_port_def}) { $self->{'options_hash'}->{"port"} = $self->{'in_hash'}->{nfs_port}; }

	delete($self->{'options_hash'}->{"timeo"});
	if (!$self->{'in_hash'}->{nfs_timeo_def}) { $self->{'options_hash'}->{"timeo"} = $self->{'in_hash'}->{nfs_timeo}; }

	delete($self->{'options_hash'}->{"retrans"});
	if (!$self->{'in_hash'}->{nfs_retrans_def}) { $self->{'options_hash'}->{"retrans"} = $self->{'in_hash'}->{nfs_retrans};}

	delete($self->{'options_hash'}->{"quota"});
	if ($self->{'in_hash'}->{'nfs_quota'}) { $self->{'options_hash'}->{"quota"} = ""; }
	}
elsif ($_[0] eq "xfs") {
	# Parse Irix XFS options
	delete($self->{'options_hash'}->{"noatime"});
	if ($self->{'in_hash'}->{'xfs_noatime'}) { $self->{'options_hash'}->{"noatime"} = ""; }

	delete($self->{'options_hash'}->{"wsync"});
	if ($self->{'in_hash'}->{'xfs_wsync'}) { $self->{'options_hash'}->{"wsync"} = ""; }

	delete($self->{'options_hash'}->{"quota"}); delete($self->{'options_hash'}->{"qnoenforce"});
	if ($self->{'in_hash'}->{'xfs_quota'} == 2) { $self->{'options_hash'}->{"qnoenforce"} = ""; }
	elsif ($self->{'in_hash'}->{'xfs_quota'} == 1) { $self->{'options_hash'}->{"quota"} = ""; }
	}
elsif ($_[0] eq "iso9660") {
	# Parse Irix ISO9660 options
	delete($self->{'options_hash'}->{"setx"});
	if ($self->{'in_hash'}->{'iso_setx'}) { $self->{'options_hash'}->{"setx"} = ""; }

	delete($self->{'options_hash'}->{"norrip"}); delete($self->{'options_hash'}->{"rrip"});
	delete($self->{'options_hash'}->{"noext"});
	if (!$self->{'in_hash'}->{'iso_rock'}) { $self->{'options_hash'}->{"noext"} = ""; }
	}

# Return options string
foreach $k (keys %{$self->{'options_hash'}}) {
	if ($self->{'options_hash'}->{$k} eq "") { push(@rv, $k); }
	else { push(@rv, "$k=$self->{'options_hash'}->{$k}"); }
	}
return @rv ? join(',' , @rv) : "-";
}

# create_swap(path, size, units)
# Attempt to create a swap file 
sub create_swap
{
	my $self = shift;

local($out);
$out = $self->backquote_logged("mkfile $_[1]$_[2] $_[0] 2>&1");
if ($?) {
	$self->unlink_file($_[0]);
	return "mkfile failed : $out";
	}
return 0;
}

# exports_list(host, dirarray, clientarray)
# Fills the directory and client array references with exports from some
# host. Returns an error string if something went wrong
sub exports_list
{
	my $self = shift;

local($dref, $cref, $out, $_);
$dref = $_[1]; $cref = $_[2];
$out = $self->backquote_command("showmount -e ".quotemeta($_[0]), 1);
if ($?) { return $out; }
foreach (split(/\n/, $out)) {
	if (/^(\/\S*)\s+(.*)$/) {
		push(@$dref, $1); push(@$cref, $2);
		}
	}
return undef;
}

# broadcast_addr()
# Returns a useable broadcast address for finding NFS servers
sub broadcast_addr
{
	my $self = shift;

local($out);
$out = $self->backquote_command("ifconfig -a 2>&1", 1);
if ($out =~ /broadcast\s+(\S+)/) { return $1; }
return "255.255.255.255";
}

sub device_name
{
	my $self = shift;

return $_[0] eq "/dev/root" ? $self->{'text_hash'}->{'irix_devroot'} :
       $_[0] eq "/dev/swap" ? $self->{'text_hash'}->{'irix_devswap'} :
       $_[0] eq "-hosts" ? $self->{'text_hash'}->{'irix_autohosts'} :
       $_[0] =~ /^\/dev\/r?dsk\/dks(\d+)d(\d+)s(\d+)$/ ?
	$self->text('irix_mode0', "$1", "$2", "$3") :
       $_[0] =~ /^\/dev\/r?dsk\/dks(\d+)d(\d+)vol$/ ?
	$self->text('irix_mode0cd', "$1", "$2", "$3") :
	$_[0];
}

sub files_to_lock
{
	my $self = shift;

return ( $self->{'config_hash'}->{'fstab_file'} );
}


1;

