#!/usr/bin/perl
package mount::MountLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'mount::MountLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# mount-lib.pl
# Functions for handling the /etc/[v]fstab file. Some functions are defined in
# here, and some in OS-specific files named <os_type>-lib.pl

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();
$self->{'filesystem_users_file'} = "$self->{'module_config_directory'}/filesystem-users";
@access_fs = split(/\s+/, $self->{'access_hash'}->{'fs'});

# Include the correct OS-specific functions file
if ($self->{'gconfig_hash'}->{'os_type'} =~ /^\S+\-linux$/) {
	&ModuleLoader::preload_module(ref($self), "mount::LinuxLib");
$self = new mount::LinuxLib:: ref($self), %$self; 
	}
else {
	{
    my $pkg = $self->lib_to_pkg("$self->{gconfig_hash}->{os_type}-lib.pl", "mount");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
	}

1;

1;
		return $self;
}

# get_mount(directory|'swap', device)
# Returns the index of this mount, or -1 if not known
sub get_mount
{
	my $self = shift;

local(@mlist, $p, $d, $i);
@mlist = $self->list_mounts();
for($i=0; $i<@mlist; $i++) {
	$p = $mlist[$i];
	if ($_[0] eq "*" && $p->[1] eq $_[1]) {
		# found by match on device
		return $i;
		}
	elsif ($_[1] eq "*" && $p->[0] eq $_[0]) {
		# found by match on directory
		return $i;
		}
	elsif ($p->[0] eq $_[0] && $p->[1] eq $_[1]) {
		# found by match on both
		return $i;
		}
	}
return -1;
}

# get_mounted(directory|'swap', device)
# Returns the index of this current mount, or -1 if not known
sub get_mounted
{
	my $self = shift;

local(@mlist, $p, $d, $i);
@mlist = $self->list_mounted();
for($i=0; $i<@mlist; $i++) {
	$p = $mlist[$i];
	if ($_[0] eq "*" && $p->[1] eq $_[1]) {
		# found by match on device
		return $i;
		}
	elsif ($_[1] eq "*" && $p->[0] eq $_[0]) {
		# found by match on directory
		return $i;
		}
	elsif ($p->[0] eq $_[0] && $p->[1] eq $_[1]) {
		# found by match on both
		return $i;
		}
	}
return -1;
}

# parse_options(type, options)
# Convert an options string for some filesystem into the associative
# array %{$self->{'options_hash'}}
sub parse_options
{
	my $self = shift;

local($_);
undef(%{$self->{'options_hash'}});
if ($_[1] ne "-") {
	foreach (split(/,/, $_[1])) {
		if (/^([^=]+)=(.*)$/) { $self->{'options_hash'}->{$1} = $2; }
		else { $self->{'options_hash'}->{$_} = ""; }
		}
	}
return \%{$self->{'options_hash'}};
}

# join_options(type, [&hash])
# Returns a string constructed from the %{$self->{'options_hash'}} hash
sub join_options
{
	my $self = shift;

local $h = $_[1] || \%{$self->{'options_hash'}};
local (@rv, $k);
foreach $k (keys %$h) {
	push(@rv, $h->{$k} eq "" ? $k : $k."=".$h->{$k});
	}
return @rv ? join(",", @rv) : "-";
}

# swap_form(path)
# This function should be called by os-specific code to display a form
# asking for the size of a swap file to create. The form will be submitted
# to a creation program, and then redirected back to the original mount cgi
sub swap_form
{
	my $self = shift;

$self->ui_print_header(undef, "Create Swap File", "");
print "<form action=create_swap.cgi>\n";
foreach $k (keys %{$self->{'in_hash'}}) {
	print "<input type=hidden name=\"$k\" value=\"$self->{'in_hash'}->{$k}\">\n";
	}
print "<input type=hidden name=cswap_file value=\"$_[0]\">\n";
print "The swap file <tt>$_[0]</tt> does not exist.<p>\n";
print "Create and mount a swap file with size <input name=cswap_size size=6>\n";
print "<select name=cswap_units>\n";
print "<option value=k> kB\n";
print "<option value=m selected> MB\n";
print "<option value=g> GB\n";
print "</select>\n";
print "<input type=submit value=\"Create\"></form>\n";
$self->ui_print_footer("", "mount list");
exit;
}

# nfs_server_chooser_button(input, [form])
sub nfs_server_chooser_button
{
	my $self = shift;

local($form);
$form = @_ > 1 ? $_[1] : 0;
if ($self->{'access_hash'}->{'browse'}) {
	print "<input type=button onClick='ifield = document.forms[$form].$_[0]; nfs_server = window.open(\"../$self->{'module_name'}/nfs_server.cgi\", \"nfs_server\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); nfs_server.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
	}
}

# nfs_export_chooser_button(serverinput, exportinput, [form])
sub nfs_export_chooser_button
{
	my $self = shift;

local($form);
$form = @_ > 2 ? $_[2] : 0;
if ($self->{'access_hash'}->{'browse'}) {
	print "<input type=button onClick='if (document.forms[$form].$_[0].value != \"\") { ifield = document.forms[$form].$_[1]; nfs_export = window.open(\"../$self->{'module_name'}/nfs_export.cgi?server=\"+document.forms[$form].$_[0].value, \"nfs_export\", \"toolbar=no,menubar=no,scrollbars=yes,width=500,height=200\"); nfs_export.ifield = ifield; window.ifield = ifield }' value=\"...\">\n";
	}
}

# smb_server_chooser_button(serverinput, [form])
sub smb_server_chooser_button
{
	my $self = shift;

local($form);
$form = @_ > 1 ? $_[1] : 0;
if ($self->has_command($self->{'config_hash'}->{'smbclient_path'}) && $self->{'access_hash'}->{'browse'}) {
	print "<input type=button onClick='ifield = document.forms[$form].$_[0]; smb_server = window.open(\"../$self->{'module_name'}/smb_server.cgi\", \"smb_server\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); smb_server.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
	}
}

# smb_share_chooser_button(serverinput, shareinput, [form])
sub smb_share_chooser_button
{
	my $self = shift;

local($form);
$form = @_ > 2 ? $_[2] : 0;
if ($self->has_command($self->{'config_hash'}->{'smbclient_path'}) && $self->{'access_hash'}->{'browse'}) {
	print "<input type=button onClick='if (document.forms[$form].$_[0].value != \"\") { ifield = document.forms[$form].$_[1]; smb_share = window.open(\"../$self->{'module_name'}/smb_share.cgi?server=\"+document.forms[$form].$_[0].value, \"smb_share\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); smb_share.ifield = ifield; window.ifield = ifield }' value=\"...\">\n";
	}
}

# can_edit_fs(dir, device, type, options, [is-new])
# Returns 1 if a filesystem can be edited, 0 otherwise
sub can_edit_fs
{
	my $self = shift;

local $ok = 1;
if (@{$self->{'access_fs_array'}}) {
	local $d;
	foreach $d (@{$self->{'access_fs_array'}}) {
		$ok = 0 if (!$self->is_under_directory($d, $_[0]));
		}
	}
$ok = 0 if (!$self->can_fstype($_[2]));
if ($self->{'access_hash'}->{'user'} && !$_[4]) {
	local $users = $self->get_filesystem_users();
	$ok = 0 if ($users->{$_[0]} ne $self->{'remote_user'});
	}
return $ok;
}

# can_fstype(type)
sub can_fstype
{
	my $self = shift;

return 1 if (!$self->{'access_hash'}->{'types'});
local @types = split(/\s+/, $self->{'access_hash'}->{'types'});
return $self->indexof($_[0], @types) >= 0;
}

# compile_program(name, default-arch)
# Ensures that some C program is compiled and copied to /etc/webmin . Uses the
# supplied native versions if possible
sub compile_program
{
	my $self = shift;

return if (-r "$self->{'module_config_directory'}/$_[0]" &&
   $self->execute_command("$self->{'module_config_directory'}/$_[0]", undef, undef, undef, 0, 1) == 0);
local $arch = $self->backquote_command("uname -m");
$arch =~ s/\r|\n//g;
local $re = $_[1];
if ($re && $arch =~ /^$re$/i && -r "$self->{'module_root_directory'}/$_[0]" &&
    $self->execute_command("$self->{'module_root_directory'}/$_[0]", undef, undef, undef, 0, 1) == 0) {
	# Compiled program for this architecture already exists and is working,
	# so can just copy
	$self->execute_command("cp $self->{'module_root_directory'}/$_[0] $self->{'module_config_directory'}/$_[0]");
	}
else {
	# Need to compile
	local ($cc) = ($self->has_command("gcc") || $self->has_command("cc") ||
		       $self->has_command("gcc-4.0") || $self->has_command("gcc-3.3"));
	$cc || $self->error($self->{'text_hash'}->{'egcc'});
	local $out = $self->backquote_logged("$cc -o $self->{'module_config_directory'}/$_[0] $self->{'module_root_directory'}/$_[0].c 2>&1");
	if ($?) {
		$self->error($self->text('ecompile', "<pre>$out</pre>"));
		}
	}
}

# get_filesystem_users()
# Returns a mapping between filesystems and their owners
sub get_filesystem_users
{
	my $self = shift;

local %users;
$self->read_file($self->{'filesystem_users_file'}, \%users);
return \%users;
}

# save_filesystem_users(&usermap)
# Saves the filesystem owner mapping
sub save_filesystem_users
{
	my $self = shift;

$self->write_file($self->{'filesystem_users_file'}, $_[0]);
}

# can_delete_directory(mount-point)
# Returns 1 if some directory should be deleted when un-mounting
sub can_delete_directory
{
	my $self = shift;

local @dirs = split(/\s+/, $self->{'config_hash'}->{'delete_under'});
return 0 if (!@dirs);
return 0 if ($_[0] eq "swap");
local $d;
foreach $d (@dirs) {
	return 1 if ($self->is_under_directory($d, $_[0]));
	}
return 0;
}

# delete_unmounted(dir, device)
# If some directory is no longer in the permanent mount list, delete it if it
# is under the list of dirs to auto-delete
sub delete_unmounted
{
	my $self = shift;

if ($self->can_delete_directory($_[0]) &&
    $self->get_mount($_[0], $_[1]) < 0) {
	$self->system_logged("rmdir ".quotemeta($_[0]));
	}
}

# remount_dir(directory, device, type, options)
# Adjusts the options for some mounted filesystem
sub remount_dir
{
	my $self = shift;

if ($self->UNIVERSAL::can('os_remount_dir')) {
	return $self->os_remount_dir(@_);
	}
else {
	local $err = $self->unmount_dir(@_);
	return $err if ($err);
	return $self->mount_dir(@_);
	}
}

# filesystem_for_dir(dir)
# Give a directory, returns the details filesystem it is on (dir, device,
# type, options)
sub filesystem_for_dir
{
	my $self = shift;

local @stdir = stat($_[0]);
foreach my $m ($self->list_mounted()) {
	local @stm = stat($m->[0]);
	if ($stm[0] == $stdir[0]) {
		# Save device number!
		return @$m;
		}
	}
return ( );
}


1;

