#!/usr/bin/perl
package mount::SolarisLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'mount::SolarisLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# solaris-lib.pl
# Filesystem functions for Solaris (works for me on 2.5)

$self->{'smbmount'} = $self->has_command("rumba") ? "rumba" :
	    $self->has_command("shlight") ? "shlight" : undef;

1;
1;
		return $self;
}

# Return information about a filesystem, in the form:
#  directory, device, type, options, fsck_order, mount_at_boot
# If a field is unused or ignored, a - appears instead of the value.
# Swap-filesystems (devices or files mounted for VM) have a type of 'swap',
# and 'swap' in the directory field
sub list_mounts
{
	my $self = shift;

return @{$self->{'list_mounts_cache_array'}} if (@{$self->{'list_mounts_cache_array'}});
local(@rv, @p, $_, $i); $i = 0;

# List normal filesystem mounts
open(FSTAB, $self->{'config_hash'}->{fstab_file});
while(<FSTAB>) {
	chop; s/#.*$//g;
	if (!/\S/) { next; }
	@p = split(/\s+/, $_);
	if ($p[3] eq "swap") { $p[2] = "swap"; }
	$rv[$i++] = [ $p[2], $p[0], $p[3], $p[6], $p[4], $p[5] ];
	}
close(FSTAB);

# List automount points
open(AUTOTAB, $self->{'config_hash'}->{autofs_file});
while(<AUTOTAB>) {
	chop; s/#.*$//g;
	if (!/\S/ || /^[+\-]/) { next; }
	@p = split(/\s+/, $_);
	if ($p[2] eq "") { $p[2] = "-"; }
	else { $p[2] =~ s/^-//g; }
	$rv[$i++] = [ $p[0], $p[1], "autofs", $p[2], "-", "yes" ];
	}
close(AUTOTAB);

@{$self->{'list_mounts_cache_array'}} = @rv;
return @rv;
}

# create_mount(directory, device, type, options, fsck_order, mount_at_boot)
# Add a new entry to the fstab file, and return the index of the new entry
sub create_mount
{
	my $self = shift;

local($len, @mlist, $fcsk, $dir);
if ($_[2] eq "autofs") {
	# An autofs mount.. add to /etc/auto_master
	$len = grep { $_->[2] eq "autofs" } ($self->list_mounts());
	$self->open_tempfile(AUTOTAB, ">> $self->{'config_hash'}->{autofs_file}");
	$self->print_tempfile(AUTOTAB, "$_[0] $_[1]",($_[3] eq "-" ? "" : " -$_[3]"),"\n");
	$self->close_tempfile(AUTOTAB);
	}
else {
	# Add to the fstab file
	$len = grep { $_->[2] ne "autofs" } ($self->list_mounts());
	$self->open_tempfile(FSTAB, ">> $self->{'config_hash'}->{fstab_file}");
	if ($_[2] eq "ufs" || $_[2] eq "s5fs") {
		($self->{'fsck'} = $_[1]) =~ s/\/dsk\//\/rdsk\//g;
		}
	else { $self->{'fsck'} = "-"; }
	if ($_[2] eq "swap") { $dir = "-"; }
	else { $dir = $_[0]; }
	$self->print_tempfile(FSTAB, "$_[1]  $self->{'fsck'}  $dir  $_[2]  $_[4]  $_[5]  $_[3]\n");
	$self->close_tempfile(FSTAB);
	}
undef(@{$self->{'list_mounts_cache_array'}});
return $len;
}

# delete_mount(index)
# Delete some mount from the table
sub delete_mount
{
	my $self = shift;

local(@fstab, $i, $line, $_);
open(FSTAB, $self->{'config_hash'}->{fstab_file});
@fstab = <FSTAB>;
close(FSTAB);
$i = 0;

$self->open_tempfile(FSTAB, "> $self->{'config_hash'}->{fstab_file}");
foreach (@fstab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $i++ == $_[0]) {
		# found the line not to include
		}
	else { $self->print_tempfile(FSTAB, $_,"\n"); }
	}
$self->close_tempfile(FSTAB);

open(AUTOTAB, $self->{'config_hash'}->{autofs_file});
@{$self->{'autotab_array'}} = <AUTOTAB>;
close(AUTOTAB);
$self->open_tempfile(AUTOTAB, "> $self->{'config_hash'}->{autofs_file}");
foreach (@{$self->{'autotab_array'}}) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $line !~ /^[+\-]/ && $i++ == $_[0]) {
		# found line not to include..
		}
	else { $self->print_tempfile(AUTOTAB, $_,"\n"); }
	}
$self->close_tempfile(AUTOTAB);
undef(@{$self->{'list_mounts_cache_array'}});
}

# change_mount(num, directory, device, type, options, fsck_order, mount_at_boot)
# Change an existing permanent mount
sub change_mount
{
	my $self = shift;

local(@fstab, @autotab, $i, $line, $fsck, $dir, $_);
$i = 0;

open(FSTAB, $self->{'config_hash'}->{fstab_file});
@fstab = <FSTAB>;
close(FSTAB);
$self->open_tempfile(FSTAB, "> $self->{'config_hash'}->{fstab_file}");
foreach (@fstab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $i++ == $_[0]) {
		# Found the line to replace
		if ($_[3] eq "ufs" || $_[3] eq "s5fs") {
			($fsck = $_[2]) =~ s/\/dsk\//\/rdsk\//g;
			}
		else { $fsck = "-"; }
		if ($_[3] eq "swap") { $dir = "-"; }
		else { $dir = $_[1]; }
		$self->print_tempfile(FSTAB, "$_[2]  $fsck  $dir  $_[3]  $_[5]  $_[6]  $_[4]\n");
		}
	else { $self->print_tempfile(FSTAB, $_,"\n"); }
	}
$self->close_tempfile(FSTAB);

open(AUTOTAB, $self->{'config_hash'}->{autofs_file});
@autotab = <AUTOTAB>;
close(AUTOTAB);
$self->open_tempfile(AUTOTAB, "> $self->{'config_hash'}->{autofs_file}");
foreach (@autotab) {
	chop; ($line = $_) =~ s/#.*$//g;
	if ($line =~ /\S/ && $line !~ /^[+\-]/ && $i++ == $_[0]) {
		# Found the line to replace
		$self->print_tempfile(AUTOTAB, "$_[1]  $_[2]  ",
				($_[4] eq "-" ? "" : "-$_[4]"),"\n");
		}
	else { $self->print_tempfile(AUTOTAB, $_,"\n"); }
	}
$self->close_tempfile(AUTOTAB);
undef(@{$self->{'list_mounts_cache_array'}});
}

# list_mounted()
# Return a list of all the currently mounted filesystems and swap files.
# The list is in the form:
#  directory device type options
# For swap files, the directory will be 'swap'
sub list_mounted
{
	my $self = shift;

return @{$self->{'list_mounted_cache_array'}} if (@{$self->{'list_mounted_cache_array'}});
local(@rv, @p, $_, $i, $r);
$self->open_execute_command(*SWAP, "swap -l 2>/dev/null", 1, 1);
while(<SWAP>) {
	if (/^(\/\S+)\s+/) { push(@rv, [ "swap", $1, "swap", "-" ]); }
	}
close(SWAP);
$self->open_tempfile(MNTTAB, "/etc/mnttab");
while(<MNTTAB>) {
	s/#.*$//g; if (!/\S/) { next; }
	@p = split(/\s+/, $_);
	if ($p[0] =~ /:vold/) { next; }
	if ($p[0] =~ /^(rumba|shlight)-(\d+)$/) {
		# rumba smb mount
		local($args, $ps); $p[3] = "pid=$2";
		$ps = (-x "/usr/ucb/ps") ? "/usr/ucb/ps auwwwwx $2"
				 	 : "ps -o args -f $2";
		$self->backquote_command($ps, 1) =~ /(rumba|shlight)\s+\/\/([^\/]+)\/(.*\S)\s+(\/\S+)(.*)/ || next;
		$self->{'serv'} = $2; $self->{'shar'} = $3; $p[2] = "rumba"; $args = $5;
		if ($args =~ /\s+-s\s+(\S+)/ && $1 ne $self->{'serv'}) {
			$p[0] = "\\\\$1\\$self->{'shar'}";
			$p[3] .= ",machinename=$self->{'serv'}";
			}
		else { $p[0] = "\\\\$self->{'serv'}\\$self->{'shar'}"; }
		if ($args =~ /\s+-c\s+(\S+)/) { $p[3] .= ",clientname=$1"; }
		if ($args =~ /\s+-U\s+(\S+)/) { $p[3] .= ",username=$1"; }
		if ($args =~ /\s+-u\s+(\S+)/) { $p[3] .= ",uid=$1"; }
		if ($args =~ /\s+-g\s+(\S+)/) { $p[3] .= ",gid=$1"; }
		if ($args =~ /\s+-f\s+(\S+)/) { $p[3] .= ",fmode=$1"; }
		if ($args =~ /\s+-d\s+(\S+)/) { $p[3] .= ",dmode=$1"; }
		if ($args =~ /\s+-C/) { $p[3] .= ",noupper"; }
		if ($args =~ /\s+-P\s+(\S+)/) { $p[3] .= ",password=$1"; }
		if ($args =~ /\s+-S/) { $p[3] .= ",readwrite"; }
		if ($args =~ /\s+-w/) { $p[3] .= ",readonly"; }
		if ($args =~ /\s+-e/) { $p[3] .= ",attr"; }
		}
	else { $p[3] = join(',' , (grep {!/^dev=/} split(/,/ , $p[3]))); }
	push(@rv, [ $p[1], $p[0], $p[2], $p[3] ]);
	}
$self->close_tempfile(MNTTAB);
foreach $r (@rv) {
	if ($r->[2] eq "cachefs" && $r->[1] =~ /\.cfs_mnt_points/) {
		# Oh no.. a caching filesystem mount. Fiddle things so that
		# it looks right.
		for($i=0; $i<@rv; $i++) {
			if ($rv[$i]->[0] eq $r->[1]) {
				# Found the automatically mounted entry. lose it
				$r->[1] = $rv[$i]->[1];
				splice(@rv, $i, 1);
				last;
				}
			}
		}
	}
@{$self->{'list_mounted_cache_array'}} = @rv;
return @rv;
}

# mount_dir(directory, device, type, options)
# Mount a new directory from some device, with some options. Returns 0 if ok,
# or an error string if failed. If the directory is 'swap', then mount as
# virtual memory.
sub mount_dir
{
	my $self = shift;

local($out, $opts);
if ($_[0] eq "swap") {
	# Adding a swap device
	$out = $self->backquote_logged("swap -a $_[1] 2>&1");
	if ($?) { return $out; }
	}
else {
	# Mounting a directory
	if ($_[2] eq "cachefs") {
		# Mounting a caching filesystem.. need to create cache first
		local(%options);
		$self->parse_options("cachefs", $_[3]);
		if (!(-r "$options{cachedir}/.cfs_resource")) {
			# The cache directory does not exist.. set it up
			if (-d $options{cachedir} &&
			    !rmdir($options{"cachedir"})) {
				return $self->text('solaris_ecacheexists',
					     $options{'cachedir'});
				}
			$out = $self->backquote_logged("cfsadmin -c $options{cachedir} 2>&1");
			if ($?) { return $out; }
			}
		}
	if ($_[2] eq "rumba") {
		# call 'rumba' to mount
		local(%options, $shortname, $shar, $opts, $rv);
		$self->parse_options("rumba", $_[3]);
		$shortname = $self->get_system_hostname();
		if ($shortname =~ /^([^\.]+)\.(.+)$/) { $shortname = $1; }
		$_[1] =~ /^\\\\(.+)\\(.+)$/;
		$shar = "//".($options{machinename} ?$options{machinename} :$1).
			"/$2";
		$opts = ("-s $1 ").
		 (defined($options{'clientname'}) ?
			"-c $options{'clientname'} " : "-c $shortname ").
		 (defined($options{'username'}) ?
			"-U $options{'username'} " : "").
		 (defined($options{'uid'}) ? "-u $options{'uid'} " : "").
		 (defined($options{'gid'}) ? "-g $options{'gid'} " : "").
		 (defined($options{'fmode'}) ? "-f $options{'fmode'} " : "").
		 (defined($options{'dmode'}) ? "-d $options{'dmode'} " : "").
		 (defined($options{'noupper'}) ? "-C " : "").
		 (defined($options{'password'}) ?
			"-P $options{'password'} " : "-n ").
		 (defined($options{'readwrite'}) ? "-S " : "").
		 (defined($options{'readonly'}) ? "-w " : "").
		 (defined($options{'attr'}) ? "-e " : "");
		local $rtemp = $self->transname();
		$rv = $self->system_logged("rumba \"$shar\" $_[0] $opts >$rtemp 2>&1 </dev/null");
		$out = `cat $rtemp`; unlink($rtemp);
		if ($rv) { return "<pre>$out</pre> : rumba \"$shar\" $_[0] $opts"; }
		}
	else {
		$opts = $_[3] eq "-" ? "" : "-o \"$_[3]\"";
		$out = $self->backquote_logged("mount -F $_[2] $opts -- $_[1] $_[0] 2>&1");
		if ($?) { return $out; }
		}
	}
undef(@{$self->{'list_mounted_cache_array'}});
return 0;
}

# unmount_dir(directory, device, type)
# Unmount a directory (or swap device) that is currently mounted. Returns 0 if
# ok, or an error string if failed
sub unmount_dir
{
	my $self = shift;

if ($_[0] eq "swap") {
	$self->{'out'} = $self->backquote_logged("swap -d $_[1] 2>&1");
	}
elsif ($_[2] eq "rumba") {
	# kill the process (if nobody is in the directory)
	$self->{'dir'} = $_[0];
	if ($self->backquote_command("fuser -c $_[0] 2>/dev/null", 1) =~ /\d/) {
		return $self->text('solaris_ebusy', $_[0]);
		}
	if ($self->backquote_command("cat /etc/mnttab", 1) =~
	    /(rumba|shlight)-(\d+)\s+$self->{'dir'}\s+nfs/) {
		$self->kill_logged('TERM', $2) || return $self->{'text_hash'}->{'solaris_ekill'};
		}
	else {
		return $self->{'text_hash'}->{'solaris_epid'};
		}
	sleep(1);
	}
else {
	$self->{'out'} = $self->backquote_logged("umount $_[0] 2>&1");
	}
undef(@{$self->{'list_mounted_cache_array'}});
if ($?) { return $self->{'out'}; }
return 0;
}

# disk_space(type, directory)
# Returns the amount of total and free space for some filesystem, or an
# empty array if not appropriate.
sub disk_space
{
	my $self = shift;

if ($self->get_mounted($_[1], "*") < 0) { return (); }
if ($_[0] eq "fd" || $_[0] eq "proc" || $_[0] eq "swap" || $_[0] eq "autofs") {
	return ();
	}
if ($self->backquote_command("df -k $_[1]", 1) =~
    /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/) {
	return ($1, $2);
	}
return ( );
}

# list_fstypes()
# Returns an array of all the supported filesystem types. If a filesystem is
# found that is not one of the supported types, generate_location() and
# generate_options() will not be called for it.
sub list_fstypes
{
	my $self = shift;

local @fs = ("ufs", "nfs", "hsfs", "pcfs", "lofs", "cachefs",
	     "swap", "tmpfs", "autofs");
if ($self->running_in_zone()) {
	# Only the filesystems are available in zones
	@fs = ( "tmpfs", "autofs", "nfs" );
	}
push(@fs, $self->{'smbmount'}) if ($self->{'smbmount'});
push(@fs, "udfs", "xmemfs") if ($self->{'gconfig_hash'}->{'os_version'} >= 8);
return @fs;
}

# fstype_name(type)
# Given a short filesystem type, return a human-readable name for it
sub fstype_name
{
	my $self = shift;

local(%fsmap);
%fsmap = ("ufs","Solaris Unix Filesystem",
	  "nfs","Network Filesystem",
	  "hsfs","ISO9660 CD-ROM",
	  "pcfs","MS-DOS Filesystem",
	  "lofs","Loopback Filesystem",
	  "cachefs","Caching Filesystem",
	  "swap","Virtual Memory",
	  "tmpfs","RAM Disk",
	  "xmemfs","Large RAM Disk",
	  "autofs","Automounter Filesystem",
	  "proc","Process Image Filesystem",
	  "fd","File Descriptor Filesystem",
	  "mntfs","Filesystems List",
	  "udfs","DVD Filesystem",
	  "rumba","Windows Networking Filesystem");
return $self->{'config_hash'}->{long_fstypes} && $fsmap{$_[0]} ? $fsmap{$_[0]} : uc($_[0]);
}

# mount_modes(type)
# Given a filesystem type, returns 4 numbers that determine how the file
# system can be mounted, and whether it can be fsck'd
sub mount_modes
{
	my $self = shift;

if ($_[0] eq "ufs" || $_[0] eq "cachefs" || $_[0] eq "s5fs") {
	return (2, 1, 1, 0);
	}
elsif ($_[0] eq "rumba") { return (0, 1, 0, 0); }
else { return (2, 1, 0, 0); }
}

# multiple_mount(type)
# Returns 1 if filesystems of this type can be mounted multiple times, 0 if not
sub multiple_mount
{
	my $self = shift;

return ($_[0] eq "nfs" || $_[0] eq "tmpfs" || $_[0] eq "cachefs" ||
        $_[0] eq "autofs" || $_[0] eq "lofs" || $_[0] eq "rumba" || $_[0] eq "xmemfs");
}

# generate_location(type, location)
# Output HTML for editing the mount location of some filesystem.
sub generate_location
{
	my $self = shift;

if ($_[0] eq "nfs") {
	# NFS mount from some host and directory
	if ($_[1] =~ /^nfs:/) { $self->{'nfsmode'} = 2; }
	elsif (!$_[1] || $_[1] =~ /^([A-z0-9\-\.]+):([^,]+)$/) {
		$self->{'nfsmode'} = 0; $self->{'nfshost'} = $1; $self->{'nfspath'} = $2;
		}
	else { $self->{'nfsmode'} = 1; }
	if ($self->{'gconfig_hash'}->{'os_version'} >= 2.6) {
		# Solaris 2.6 can list multiple NFS servers in mount
		print "<tr> <td><b>$self->{'text_hash'}->{'solaris_nsource'}</b></td>\n";
		printf "<td><input type=radio name=nfs_serv value=0 %s>\n",
			$self->{'nfsmode'} == 0 ? "checked" : "";
		print "<b>$self->{'text_hash'}->{'solaris_nhost'}</b></td>\n";
		print "<td><input name=nfs_host size=20 value=\"$self->{'nfshost'}\">\n";
		$self->nfs_server_chooser_button("nfs_host");
		print "&nbsp;<b>$self->{'text_hash'}->{'solaris_ndir'}</b>\n";
		print "<input name=nfs_dir size=20 value=\"$self->{'nfspath'}\">\n";
		$self->nfs_export_chooser_button("nfs_host", "nfs_dir");
		print "</td> </tr>\n";

		print "<tr> <td></td>\n";
		printf "<td><input type=radio name=nfs_serv value=1 %s>\n",
			$self->{'nfsmode'} == 1 ? "checked" : "";
		print "<b>$self->{'text_hash'}->{'solaris_nmult'}</b></td>\n";
		printf "<td><input name=nfs_list size=40 value=\"%s\">\n",
			$self->{'nfsmode'} == 1 ? $_[1] : "";
		print "</td> </tr>\n";

		if ($self->{'gconfig_hash'}->{'os_version'} >= 7) {
			print "<tr> <td></td> <td>\n";
			printf "<input type=radio name=nfs_serv value=2 %s>\n",
				$self->{'nfsmode'} == 2 ? "checked" : "";
			print "<b>$self->{'text_hash'}->{'solaris_webnfs'}</b></td> <td>\n";
			printf "<input name=nfs_url size=40 value=\"%s\">\n",
				$self->{'nfsmode'} == 2 ? $_[1] : "";
			print "</td> </tr>\n";
			}
		}
	else {
		print "<tr> <td><b>$self->{'text_hash'}->{'solaris_nhost'}</b></td>\n";
		print "<td><input name=nfs_host size=20 value=\"$self->{'nfshost'}\">\n";
		$self->nfs_server_chooser_button("nfs_host");
		print "</td>\n";
		print "<td><b>$self->{'text_hash'}->{'solaris_ndir'}</b></td>\n";
		print "<td><input name=nfs_dir size=20 value=\"$self->{'nfspath'}\">\n";
		$self->nfs_export_chooser_button("nfs_host", "nfs_dir");
		print "</td> </tr>\n";
		}
	}
elsif ($_[0] eq "tmpfs" || $_[0] eq "xmemfs") {
	# Location is irrelevant for tmpfs and xmemfs filesystems
	}
elsif ($_[0] eq "ufs") {
	# Mounted from a normal disk, raid (MD) device or from
	# somewhere else
	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'solaris_ufs'}</b></td>\n";
	print "<td colspan=3>\n";
	$self->foreign_require("format", "format-lib.pl");

	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/ ||
	    $_[1] =~ /^\/dev\/dsk\/c([0-9]+)d([0-9]+)s([0-9]+)$/) {
		$self->{'ufs_dev'} = 0;
		}
	elsif ($_[1] eq "") {
		$self->{'ufs_dev'} = 0;
		}
	elsif ($_[1] =~ /^\/dev\/md\/dsk\/d([0-9]+)$/) {
		$self->{'ufs_dev'} = 1;
		}
	else {
		$self->{'ufs_dev'} = 2;
		}

	local $found;
	local $sel = $self->foreign_call("format", "partition_select",
				   "ufs_disk", $_[1], 0,
				   $self->{'ufs_dev'} ? \$found : undef);
	printf "<input type=radio name=ufs_dev value=0 %s> %s : %s<br>\n",
		$self->{'ufs_dev'} == 0 ? "checked" : "", $self->{'text_hash'}->{'solaris_scsi'}, $sel;

	printf "<input type=radio name=ufs_dev value=1 %s> %s :\n",
		$self->{'ufs_dev'} == 1 ? "checked" : "",
		$self->{'text_hash'}->{'solaris_raid'};
	printf "%s <input name=ufs_md size=3 value=\"%s\"><br>\n",
		$self->{'text_hash'}->{'solaris_unit'},
		 $_[1] =~ /^\/dev\/md\/dsk\/d([0-9]+)$/ ? $1 : "";

	printf "<input type=radio name=ufs_dev value=2 %s> %s :\n",
		$self->{'ufs_dev'} == 2 ? "checked" : "", $self->{'text_hash'}->{'solaris_otherdev'};
	printf "<input name=ufs_path size=20 value=\"%s\"><br>\n",
		$self->{'ufs_dev'} == 2 ? $_[1] : "";
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "swap") {
	# Swapping to a disk partition or a file
	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'solaris_swapfile'}</b></td>\n";
	print "<td colspan=3>\n";
	$self->foreign_require("format", "format-lib.pl");

	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/ ||
	    $_[1] =~ /^\/dev\/dsk\/c([0-9]+)d([0-9]+)s([0-9]+)$/) {
		$self->{'swap_dev'} = 0;
		}
	elsif ($_[1] eq "") {
		$self->{'swap_dev'} = 0;
		}
	else {
		$self->{'swap_dev'} = 1;
		}

	local $found;
	local $sel = $self->foreign_call("format", "partition_select",
				   "swap_disk", $_[1], 0,
				   $self->{'swap_dev'} ? \$found : undef);
	printf "<input type=radio name=swap_dev value=0 %s> %s : %s<br>\n",
		$self->{'swap_dev'} == 0 ? "checked" : "", $self->{'text_hash'}->{'solaris_scsi'}, $sel;

	printf "<input type=radio name=swap_dev value=1 %s> %s :\n",
		$self->{'swap_dev'} == 1 ? "checked" : "", $self->{'text_hash'}->{'solaris_file'};
	printf "<input name=swap_path size=20 value=\"%s\"><br>\n",
		$self->{'swap_dev'} == 1 ? $_[1] : "";
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "hsfs" || $_[0] eq "udfs") {
	# Mounting a SCSI cdrom or DVD
	if ($_[0] eq "hsfs") {
		print "<tr> <td valign=top><b>$self->{'text_hash'}->{'solaris_cdrom'}</b></td>\n";
		}
	else {
		print "<tr> <td valign=top><b>$self->{'text_hash'}->{'solaris_dvd'}</b></td>\n";
		}
	print "<td colspan=3>\n";
	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/) {
		$self->{'hsfs_dev'} = 0;
		$self->{'scsi_c'} = $1; $self->{'scsi_t'} = $2; $self->{'scsi_d'} = $3; $self->{'scsi_s'} = $4;
		}
	elsif ($_[1] eq "") {
		$self->{'hsfs_dev'} = 0;
		$self->{'scsi_c'} = 0; $self->{'scsi_t'} = 6; $self->{'scsi_d'} = 0; $self->{'scsi_s'} = 0;
		}
	else {
		$self->{'hsfs_dev'} = 2; $self->{'scsi_path'} = $_[1];
		}
	printf "<input type=radio name=ufs_dev value=0 %s> $self->{'text_hash'}->{'solaris_scsi'}:\n",
		$self->{'hsfs_dev'} == 0 ? "checked" : "";
	print "$self->{'text_hash'}->{'solaris_ctrlr'} <input name=ufs_c size=3 value=\"$self->{'scsi_c'}\">\n";
	print "$self->{'text_hash'}->{'solaris_target'} <input name=ufs_t size=3 value=\"$self->{'scsi_t'}\">\n";
	print "$self->{'text_hash'}->{'solaris_unit'} <input name=ufs_d size=3 value=\"$self->{'scsi_d'}\">\n";
	print "$self->{'text_hash'}->{'solaris_part'} <input name=ufs_s size=3 value=\"$self->{'scsi_s'}\"><br>\n";

	printf "<input type=radio name=ufs_dev value=2 %s> $self->{'text_hash'}->{'solaris_otherdev'}:\n",
		$self->{'hsfs_dev'} == 2 ? "checked" : "";
	print "<input name=ufs_path size=20 value=\"$self->{'scsi_path'}\"><br>\n";
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "pcfs") {
	# Mounting a SCSI msdos filesystem
	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'solaris_msdos'}</b></td>\n";
	print "<td colspan=3>\n";
	$self->foreign_require("format", "format-lib.pl");

	if ($_[1] =~ /^\/dev\/dsk\/c([0-9]+)t([0-9]+)d([0-9]+)s([0-9]+)$/ ||
	    $_[1] =~ /^\/dev\/dsk\/c([0-9]+)d([0-9]+)s([0-9]+)$/) {
		$self->{'pcfs_dev'} = 0;
		}
	elsif ($_[1] eq "") {
		$self->{'pcfs_dev'} = 0;
		}
	else {
		$self->{'pcfs_dev'} = 2;
		}

	local $found;
	local $sel = $self->foreign_call("format", "partition_select",
				   "ufs_disk", $_[1], 0,
				   $self->{'pcfs_dev'} ? \$found : undef);
	printf "<input type=radio name=ufs_dev value=0 %s> %s : %s<br>\n",
		$self->{'pcfs_dev'} == 0 ? "checked" : "", $self->{'text_hash'}->{'solaris_scsi'}, $sel;

	printf "<input type=radio name=ufs_dev value=2 %s> %s :\n",
		$self->{'pcfs_dev'} == 2 ? "checked" : "", $self->{'text_hash'}->{'solaris_file'};
	printf "<input name=ufs_path size=20 value=\"%s\"><br>\n",
		$self->{'pcfs_dev'} == 2 ? $_[1] : "";
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "lofs") {
	# Mounting some directory to another location
	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_orig'}</b></td>\n";
	print "<td><input name=lofs_src size=30 value=\"$_[1]\">\n";
	print $self->file_chooser_button("lofs_src", 1);
	print "</td> </tr>\n";
	}
elsif ($_[0] eq "cachefs") {
	# Mounting a cached filesystem of some type.. need a location for
	# the source of the mount
	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_cache'}</b></td>\n";
	print "<td><input name=cfs_src size=20 value=\"$_[1]\"></td> </tr>\n";
	}
elsif ($_[0] eq "autofs") {
	# An automounter entry.. can be -hosts, -xfn or from some mapping
	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'solaris_automap'}</b></td>\n";
	printf "<td><input type=radio name=autofs_type value=0 %s>\n",
		$_[1] eq "-hosts" || $_[1] eq "-xfn" ? "" : "checked";
	printf "Use map <input name=autofs_map size=20 value=\"%s\"><br>\n",
		$_[1] eq "-hosts" || $_[1] eq "-xfn" ? "" : $_[1];
	printf "<input type=radio name=autofs_type value=1 %s>\n",
		$_[1] eq "-hosts" ? "checked" : "";
	print "$self->{'text_hash'}->{'solaris_autohosts'}<br>\n";
	printf "<input type=radio name=autofs_type value=2 %s>\n",
		$_[1] eq "-xfn" ? "checked" : "";
	print "$self->{'text_hash'}->{'solaris_autoxfn'}</td> </tr>\n";
	}
elsif ($_[0] eq "rumba") {
	# Windows filesystem
	$_[1] =~ /^\\\\(.*)\\(.*)$/;
	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_server'}</b></td>\n";
	print "<td><input name=rumba_server value=\"$1\" size=20>\n";
	$self->smb_server_chooser_button("rumba_server");
	print "</td>\n";
	print "<td><b>$self->{'text_hash'}->{'solaris_share'}</b></td>\n";
	print "<td><input name=rumba_share value=\"$2\" size=20>\n";
	$self->smb_share_chooser_button("rumba_server", "rumba_share");
	print "</td> </tr>\n";
	}
}

# $self->generate_options(type, newmount)
# Output HTML for editing mount options for a partilcar filesystem 
# under this OS
sub generate_options
{
	my $self = shift;

if ($_[0] eq "nfs") {
	# Solaris NFS has many options, not all of which are editable here
	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_ro'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_ro value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
	printf "<input type=radio name=nfs_ro value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";

	print "<td><b>$self->{'text_hash'}->{'solaris_nosuid'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_nosuid value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=nfs_nosuid value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_grpid'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_grpid value=0 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"grpid"}) ? "" : "checked";
	printf "<input type=radio name=nfs_grpid value=1 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"grpid"}) ? "checked" : "";

	print "<td><b>$self->{'text_hash'}->{'solaris_soft'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_soft value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"soft"}) ? "checked" : "";
	printf "<input type=radio name=nfs_soft value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"soft"}) ? "" : "checked";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_bg'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_bg value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"bg"}) ? "checked" : "";
	printf "<input type=radio name=nfs_bg value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"bg"}) ? "" : "checked";

	print "<td><b>$self->{'text_hash'}->{'solaris_quota'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_quota value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"quota"}) ? "checked" : "";
	printf "<input type=radio name=nfs_quota value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"quota"}) ? "" : "checked";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_nointr'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_nointr value=0 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"nointr"}) ? "" : "checked";
	printf "<input type=radio name=nfs_nointr value=1 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"nointr"}) ? "checked" : "";

	print "<td><b>$self->{'text_hash'}->{'solaris_nfsver'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_vers_def value=1 %s> $self->{'text_hash'}->{'solaris_highest'}\n",
		defined($self->{'options_hash'}->{"vers"}) ? "" : "checked";
	printf "<input type=radio name=nfs_vers_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"vers"}) ? "checked" : "";
	print "<input size=2 name=nfs_vers value=$self->{'options_hash'}->{vers}></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_proto'}</b></td>\n";
	print "<td nowrap><select name=proto>\n";
	printf "<option value=\"\" %s> $self->{'text_hash'}->{'default'}\n",
		defined($self->{'options_hash'}->{"proto"}) ? "" : "selected";
	$self->open_tempfile(NETCONFIG, "/etc/netconfig");
	while(<NETCONFIG>) {
		if (!/^([A-z0-9\_\-]+)\s/) { next; }
		printf "<option value=\"$1\" %s> $1\n",
			$self->{'options_hash'}->{"proto"} eq $1 ? "selected" : "";
		}
	$self->close_tempfile(NETCONFIG);
	print "</select></td>\n";

	print "<td><b>$self->{'text_hash'}->{'solaris_port'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_port_def value=1 %s> $self->{'text_hash'}->{'default'}\n",
		defined($self->{'options_hash'}->{"port"}) ? "" : "checked";
	printf "<input type=radio name=nfs_port_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"port"}) ? "checked" : "";
	print "<input size=5 name=nfs_port value=$self->{'options_hash'}->{port}></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_timeo'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_timeo_def value=1 %s> $self->{'text_hash'}->{'default'}\n",
		defined($self->{'options_hash'}->{"timeo"}) ? "" : "checked";
	printf "<input type=radio name=nfs_timeo_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"timeo"}) ? "checked" : "";
	printf "<input size=5 name=nfs_timeo value=$self->{'options_hash'}->{timeo}></td>\n";

	print "<td><b>$self->{'text_hash'}->{'solaris_retrans'}</b></td>\n";
	printf "<td nowrap><input type=radio name=nfs_retrans_def value=1 %s> $self->{'text_hash'}->{'default'}\n",
		defined($self->{'options_hash'}->{"retrans"}) ? "" : "checked";
	printf "<input type=radio name=nfs_retrans_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"retrans"}) ? "checked" : "";
	print "<input size=5 name=nfs_retrans value=$self->{'options_hash'}->{retrans}></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_auth'}</b></td>\n";
	$self->{'nfs_auth'} = $self->{'options_hash'}->{'sec'} ? $self->{'options_hash'}->{'sec'} :
		    defined($self->{'options_hash'}->{"secure"}) ? "dh" :
		    defined($self->{'options_hash'}->{"kerberos"}) ? "krb" : "";
	print "<td><select name=nfs_auth>\n";
	printf "<option value=\"\" %s> $self->{'text_hash'}->{'solaris_none'}\n",
		$self->{'nfs_auth'} eq "" ? "selected" : "";
	printf "<option value=dh %s> $self->{'text_hash'}->{'solaris_des'}\n",
		$self->{'nfs_auth'} eq "dh" ? "selected" : "";
	printf "<option value=krb %s> $self->{'text_hash'}->{'solaris_krb'}\n",
		$self->{'nfs_auth'} eq "krb" ? "selected" : "";
	print "</select></td>\n";

	if ($self->{'gconfig_hash'}->{'os_version'} >= 7) {
		print "<td><b>$self->{'text_hash'}->{'solaris_public'}</b></td> <td>\n";
		printf "<input type=radio name=nfs_public value=1 %s> $self->{'text_hash'}->{'yes'}\n",
			defined($self->{'options_hash'}->{'public'}) ? "checked" : "";
		printf "<input type=radio name=nfs_public value=0 %s> $self->{'text_hash'}->{'no'}\n",
			defined($self->{'options_hash'}->{'public'}) ? "" : "checked";
		print "</td>\n";
		}
	print "</tr>\n";
	}
if ($_[0] eq "ufs") {
	# Solaris UFS also has many options, not all of which are here
	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_ro'}</b></td>\n";
	printf "<td nowrap><input type=radio name=ufs_ro value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
	printf "<input type=radio name=ufs_ro value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";

	print "<td><b>$self->{'text_hash'}->{'solaris_nosuid'}</b></td>\n";
	printf "<td nowrap><input type=radio name=ufs_nosuid value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=ufs_nosuid value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_nointr'}</b></td>\n";
	printf "<td nowrap><input type=radio name=ufs_nointr value=0 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"nointr"}) ? "" : "checked";
	printf "<input type=radio name=ufs_nointr value=1 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"nointr"}) ? "checked" : "";

	print "<td><b>$self->{'text_hash'}->{'solaris_quotab'}</b></td>\n";
	printf "<td nowrap><input type=radio name=ufs_quota value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"quota"}) || defined($self->{'options_hash'}->{"rq"}) ?
			"checked" : "";
	printf "<input type=radio name=ufs_quota value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"quota"}) || defined($self->{'options_hash'}->{"rq"}) ?
			"" : "checked";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_onerror'}</b></td>\n";
	print "<td><select name=ufs_onerror>\n";
	foreach ('panic', 'lock', 'umount', 'repair') {
		next if ($_ eq "repair" && $self->{'gconfig_hash'}->{'os_version'} >= 10);
		printf "<option value=\"$_\" %s> $_\n",
		 $self->{'options_hash'}->{onerror} eq $_ ||
		 !defined($self->{'options_hash'}->{onerror}) && $_ eq "panic" ? "selected" : "";
		}
	print "</select></td>\n";

	if ($self->{'gconfig_hash'}->{'os_version'} >= 7) {
		print "<td><b>$self->{'text_hash'}->{'solaris_noatime'}</b></td> <td>\n";
		if ($self->{'gconfig_hash'}->{'os_version'} >= 8) {
			printf "<input type=radio name=ufs_noatime value=0 %s> $self->{'text_hash'}->{'solaris_immed'}\n",
				defined($self->{'options_hash'}->{'nodfratime'}) ? "checked" : "";
			printf "<input type=radio name=ufs_noatime value=1 %s> $self->{'text_hash'}->{'solaris_defer'}\n",
				!defined($self->{'options_hash'}->{'nodfratime'}) &&
				!defined($self->{'options_hash'}->{'noatime'}) ? "checked" : "";
			printf "<input type=radio name=ufs_noatime value=2 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
				defined($self->{'options_hash'}->{'noatime'}) ? "checked" : "";
			}
		else {
			printf "<input type=radio name=ufs_noatime value=0 %s> $self->{'text_hash'}->{'yes'}\n",
				defined($self->{'options_hash'}->{'noatime'}) ? "" : "checked";
			printf "<input type=radio name=ufs_noatime value=1 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
				defined($self->{'options_hash'}->{'noatime'}) ? "checked" : "";
			}

		print "<tr> <td><b>$self->{'text_hash'}->{'solaris_force'}</b></td> <td>\n";
		printf "<input type=radio name=ufs_force value=1 %s> $self->{'text_hash'}->{'yes'}\n",
			defined($self->{'options_hash'}->{'forcedirectio'}) ? "checked" : "";
		printf "<input type=radio name=ufs_force value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
			defined($self->{'options_hash'}->{'forcedirectio'}) ? "" : "checked";

		print "<td><b>$self->{'text_hash'}->{'solaris_nolarge'}</td> <td>\n";
		printf "<input type=radio name=ufs_nolarge value=0 %s> $self->{'text_hash'}->{'yes'}\n",
			defined($self->{'options_hash'}->{'nolargefiles'}) ? "" : "checked";
		printf "<input type=radio name=ufs_nolarge value=1 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
			defined($self->{'options_hash'}->{'nolargefiles'}) ? "checked" : "";

		print "<tr> <td><b>$self->{'text_hash'}->{'solaris_logging'}</td> <td>\n";
		printf "<input type=radio name=ufs_logging value=1 %s> $self->{'text_hash'}->{'yes'}\n",
			defined($self->{'options_hash'}->{'logging'}) ? "checked" : "";
		printf "<input type=radio name=ufs_logging value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
			defined($self->{'options_hash'}->{'logging'}) ? "" : "checked";
		}
	else {
		print "<td><b>$self->{'text_hash'}->{'solaris_toosoon'}</b></td>\n";
		$self->{'options_hash'}->{toosoon} =~ /([0-9]+)([A-z])/;
		print "<td nowrap><input size=5 name=ufs_toosoon_time value='$1'>\n";
		print "<select name=ufs_toosoon_units>\n";
		foreach $u ('s', 'm', 'h', 'd', 'w', 'y') {
			printf "<option value=%s %s> %s\n",
				$u, $2 eq $u ? "selected" : "", $self->{'text_hash'}->{"solaris_time_$u"};
			}
		print "</select></td> </tr>\n";
		}
	}
if ($_[0] eq "hsfs") {
	# Solaris hsfs is used for CDROMs
	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_nrr'}</b></td>\n";
	printf "<td nowrap><input type=radio name=hsfs_nrr value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"nrr"}) ? "checked" : "";
	printf "<input type=radio name=hsfs_nrr value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"nrr"}) ? "" : "checked";

	print "<td><b>$self->{'text_hash'}->{'solaris_notraildot'}</b></td>\n";
	printf "<td nowrap><input type=radio name=hsfs_notraildot value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"notraildot"}) ? "checked" : "";
	printf "<input type=radio name=hsfs_notraildot value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"notraildot"}) ? "" : "checked";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_nomaplcase'}</b></td>\n";
	printf "<td nowrap><input type=radio name=hsfs_nomaplcase value=0 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"nomaplcase"}) ? "" : "checked";
	printf "<input type=radio name=hsfs_nomaplcase value=1 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"nomaplcase"}) ? "checked" : "";

	print "<td><b>$self->{'text_hash'}->{'solaris_nosuid'}</b></td>\n";
	printf"<td nowrap><input type=radio name=hsfs_nosuid value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=hsfs_nosuid value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";
	}
if ($_[0] eq "pcfs") {
	# Solaris pcfs for for FAT filesystems. It doesn't have many options
	print "<tr> <td width=25%><b>$self->{'text_hash'}->{'solaris_ro'}</b></td> <td width=25%>\n";
	printf "<input type=radio name=pcfs_ro value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
	printf "<input type=radio name=pcfs_ro value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";

	if ($self->{'gconfig_hash'}->{'os_version'} >= 7) {
		print "<td><b>$self->{'text_hash'}->{'solaris_foldcase'}</b></td> <td>\n";
		printf "<input type=radio name=pcfs_foldcase value=1 %s> $self->{'text_hash'}->{'yes'}\n",
			defined($self->{'options_hash'}->{'foldcase'}) ? "checked" : "";
		printf "<input type=radio name=pcfs_foldcase value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
			defined($self->{'options_hash'}->{'foldcase'}) ? "" : "checked";
		}
	else {
		print "<td colspan=2></td> </tr>\n";
		}
	}
if ($_[0] eq "lofs") {
	# LOFS is a loopback filesystem
	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_ro'}</b></td>\n";
	printf "<td nowrap><input type=radio name=ufs_ro value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
	printf "<input type=radio name=ufs_ro value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";

	print "<td><b>$self->{'text_hash'}->{'solaris_nosuid'}</b></td>\n";
	printf "<td nowrap><input type=radio name=ufs_nosuid value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=ufs_nosuid value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";
	}
if ($_[0] eq "tmpfs") {
	# Solaris tmpfs (virtual memory) filesystem.
	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_size'}</b>&nbsp;&nbsp;&nbsp;</td>\n";
	printf"<td><input type=radio name=tmpfs_size_def value=1 %s> $self->{'text_hash'}->{'solaris_max'}\n",
		defined($self->{'options_hash'}->{"size"}) ? "" : "checked";
	printf"&nbsp;&nbsp;<input type=radio name=tmpfs_size_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"size"}) ? "checked" : "";
	($self->{'tmpsz'} = $self->{'options_hash'}->{size}) =~ s/[A-z]+$//g;
	print "<input name=tmpfs_size size=6 value=\"$self->{'tmpsz'}\">\n";
	print "<select name=tmpfs_unit>\n";
	printf "<option value=m %s> MB\n",
		$self->{'options_hash'}->{"size"} =~ /m$/ ? "selected" : "";
	printf "<option value=k %s> kB\n",
		$self->{'options_hash'}->{"size"} =~ /k$/ ? "selected" : "";
	printf "<option value=b %s> bytes\n",
		$self->{'options_hash'}->{"size"} !~ /(k|m)$/ ? "selected" : "";
	print "</select></td>\n";

	print "<td><b>$self->{'text_hash'}->{'solaris_nosuid'}</b></td> <td nowrap>\n";
	printf "<input type=radio name=tmpfs_nosuid value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=tmpfs_nosuid value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";
	print "</tr>\n";
	}
if ($_[0] eq "swap") {
	# Solaris swap has no options
	print "<tr> <td><i>$self->{'text_hash'}->{'solaris_noopts'}</i></td> </tr>\n";
	}
if ($_[0] eq "cachefs") {
	# The caching filesystem has lots of options.. cachefs mounts can
	# be of an existing 'manually' mounted back filesystem, or of a
	# back-filesystem that has been automatically mounted by the cache.
	# The user should never see the automatic mountings made by cachefs.
	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_backfs'}</b></td>\n";
	print "<td nowrap><select name=cfs_backfstype>\n";
	if (!defined($self->{'options_hash'}->{backfstype})) { $self->{'options_hash'}->{backfstype} = "nfs"; }
	foreach ($self->list_fstypes()) {
		if ($_ eq "cachefs") { next; }
		printf "<option value=\"$_\" %s>$_\n",
			$_ eq $self->{'options_hash'}->{backfstype} ? "selected" : "";
		}
	print "</select></td>\n";

	print "<td><b>$self->{'text_hash'}->{'solaris_backpath'}</b></td>\n";
	printf"<td nowrap><input type=radio name=cfs_noback value=1 %s> $self->{'text_hash'}->{'solaris_auto'}\n",
		defined($self->{'options_hash'}->{"backpath"}) ? "" : "checked";
	printf "<input type=radio name=cfs_noback value=0 %s>\n",
		defined($self->{'options_hash'}->{"backpath"}) ? "checked" : "";
	print "<input size=10 name=cfs_backpath value=\"$self->{'options_hash'}->{backpath}\"></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_cachedir'}</b></td>\n";
	printf "<td nowrap><input size=10 name=cfs_cachedir value=\"%s\"></td>\n",
		defined($self->{'options_hash'}->{"cachedir"}) ? $self->{'options_hash'}->{"cachedir"} : "/cache";

	print "<td><b>$self->{'text_hash'}->{'solaris_wmode'}</b></td>\n";
	printf"<td nowrap><input type=radio name=cfs_wmode value=0 %s> $self->{'text_hash'}->{'solaris_waround'}\n",
		defined($self->{'options_hash'}->{"non-shared"}) ? "" : "checked";
	printf "<input type=radio name=cfs_wmode value=1 %s> $self->{'text_hash'}->{'solaris_nshared'}\n",
		defined($self->{'options_hash'}->{"non-shared"}) ? "checked" : "";
	print "</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_con'}</b></td>\n";
	print "<td><select name=cfs_con>\n";
	print "<option value=1> $self->{'text_hash'}->{'solaris_period'}\n";
	printf "<option value=0 %s> $self->{'text_hash'}->{'solaris_never'}\n",
		defined($self->{'options_hash'}->{"noconst"}) ? "selected" : "";
	printf "<option value=2 %s> $self->{'text_hash'}->{'solaris_demand'}\n",
		defined($self->{'options_hash'}->{"demandconst"}) ? "selected" : "";
	print "</select></td>\n";

	print "<td><b>$self->{'text_hash'}->{'solaris_local'}</b></td>\n";
	printf "<td nowrap><input type=radio name=cfs_local value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"local-access"}) ? "checked" : "";
	printf "<input type=radio name=cfs_local value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"local-access"}) ? "" : "checked";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_ro'}</b></td>\n";
	printf "<td nowrap><input type=radio name=cfs_ro value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
	printf "<input type=radio name=cfs_ro value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";

	print "<td><b>$self->{'text_hash'}->{'solaris_nosuid'}</b></td>\n";
	printf "<td nowrap><input type=radio name=cfs_nosuid value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=cfs_nosuid value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";
	}
if ($_[0] eq "autofs") {
	# Autofs has lots of options, depending on the type of file
	# system being automounted.. the fstype options determines this
	local($fstype);
	$fstype = $self->{'options_hash'}->{fstype} eq "" ? "nfs" : $self->{'options_hash'}->{fstype};
	if ($self->{'gconfig_hash'}->{'os_version'} >= 2.6) {
		print "<tr> <td><b>$self->{'text_hash'}->{'solaris_nobrowse'}</b></td> <td>\n";
		printf "<input type=radio name=auto_nobrowse value=0 %s> $self->{'text_hash'}->{'yes'}\n",
			defined($self->{'options_hash'}->{'nobrowse'}) ? "" : "checked";
		printf "<input type=radio name=auto_nobrowse value=1 %s> $self->{'text_hash'}->{'no'}\n",
			defined($self->{'options_hash'}->{'nobrowse'}) ? "checked" : "";
		print "</td> <td colspan=2></td> </tr>\n";
		}
	$self->generate_options($fstype);
	print "<input type=hidden name=autofs_fstype value=\"$fstype\">\n";
	}
if ($_[0] eq "rumba") {
	# SMB filesystems have a few options..
	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_mname'}</b></td>\n";
	printf "<td><input type=radio name=rumba_mname_def value=1 %s> $self->{'text_hash'}->{'solaris_auto'}\n",
		defined($self->{'options_hash'}->{"machinename"}) ? "" : "checked";
	printf "<input type=radio name=rumba_mname_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"machinename"}) ? "checked" : "";
	print "<input size=10 name=rumba_mname value=\"$self->{'options_hash'}->{machinename}\"></td>\n";

	print "<td><b>$self->{'text_hash'}->{'solaris_cname'}</b></td>\n";
	printf "<td><input type=radio name=rumba_cname_def value=1 %s> $self->{'text_hash'}->{'solaris_auto'}\n",
		defined($self->{'options_hash'}->{"clientname"}) ? "" : "checked";
	printf "<input type=radio name=rumba_cname_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"clientname"}) ? "checked" : "";
	print "<input size=10 name=rumba_cname value=\"$self->{'options_hash'}->{clientname}\"></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_username'}</b></td>\n";
	print "<td><input name=rumba_username size=15 value=\"$self->{'options_hash'}->{username}\"></td>\n";

	print "<td><b>$self->{'text_hash'}->{'solaris_password'}</b></td>\n";
	print "<td><input type=password name=rumba_password size=15 value=\"$self->{'options_hash'}->{password}\"></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_uid'}</b></td>\n";
	printf "<td><input name=rumba_uid size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{'uid'}) ? getpwuid($self->{'options_hash'}->{'uid'}) : "";
	print $self->user_chooser_button("rumba_uid", 0),"</td>\n";

	print "<td><b>$self->{'text_hash'}->{'solaris_gid'}</b></td>\n";
	printf "<td><input name=rumba_gid size=8 value=\"%s\">\n",
		defined($self->{'options_hash'}->{'gid'}) ? getgrgid($self->{'options_hash'}->{'gid'}) : "";
	print $self->group_chooser_button("rumba_gid", 0),"</td>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_fmode'}</b></td>\n";
	printf "<td><input name=rumba_fmode size=5 value=\"%s\"></td>\n",
		defined($self->{'options_hash'}->{fmode}) ? $self->{'options_hash'}->{fmode} : "755";

	print "<td><b>$self->{'text_hash'}->{'solaris_dmode'}</b></td>\n";
	printf "<td><input name=rumba_dmode size=5 value=\"%s\"></td> </tr>\n",
		defined($self->{'options_hash'}->{dmode}) ? $self->{'options_hash'}->{dmode} : "755";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_readwrite'}</b></td>\n";
	printf"<td nowrap><input type=radio name=rumba_readwrite value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"readwrite"}) ? "checked" : "";
	printf "<input type=radio name=rumba_readwrite value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"readwrite"}) ? "" : "checked";

	print "<td><b>$self->{'text_hash'}->{'solaris_readonly'}</b></td>\n";
	printf"<td nowrap><input type=radio name=rumba_readonly value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"readonly"}) ? "checked" : "";
	printf "<input type=radio name=rumba_readonly value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"readonly"}) ? "" : "checked";

	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_noupper'}</b></td>\n";
	printf"<td nowrap><input type=radio name=rumba_noupper value=0 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"noupper"}) ? "" : "checked";
	printf "<input type=radio name=rumba_noupper value=1 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"noupper"}) ? "checked" : "";

	print "<td><b>$self->{'text_hash'}->{'solaris_attr'}</b></td>\n";
	printf"<td nowrap><input type=radio name=rumba_attr value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"attr"}) ? "checked" : "";
	printf "<input type=radio name=rumba_attr value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"attr"}) ? "" : "checked";
	}
if ($_[0] eq "udfs") {
	# Solaris udfs is used for DVDs
	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_ro'}</b></td>\n";
	printf "<td nowrap><input type=radio name=udfs_ro value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"ro"}) ? "checked" : "";
	printf "<input type=radio name=udfs_ro value=0 %s> $self->{'text_hash'}->{'no'}</td>\n",
		defined($self->{'options_hash'}->{"ro"}) ? "" : "checked";

	print "<td><b>$self->{'text_hash'}->{'solaris_nosuid'}</b></td>\n";
	printf "<td nowrap><input type=radio name=udfs_nosuid value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "checked" : "";
	printf "<input type=radio name=udfs_nosuid value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"nosuid"}) ? "" : "checked";
	}
if ($_[0] eq "xmemfs") {
	# Solaris xmemfs (virtual memory) filesystem.
	print "<tr> <td><b>$self->{'text_hash'}->{'solaris_size'}</b>&nbsp;&nbsp;&nbsp;</td>\n";
	printf"<td><input type=radio name=xmemfs_size_def value=1 %s> $self->{'text_hash'}->{'solaris_max'}\n",
		defined($self->{'options_hash'}->{"size"}) ? "" : "checked";
	printf"&nbsp;&nbsp;<input type=radio name=xmemfs_size_def value=0 %s>\n",
		defined($self->{'options_hash'}->{"size"}) ? "checked" : "";
	($self->{'tmpsz'} = $self->{'options_hash'}->{size}) =~ s/[A-z]+$//g;
	print "<input name=xmemfs_size size=6 value=\"$self->{'tmpsz'}\">\n";
	print "<select name=xmemfs_unit>\n";
	printf "<option value=m %s> MB\n",
		$self->{'options_hash'}->{"size"} =~ /m$/ ? "selected" : "";
	printf "<option value=k %s> kB\n",
		$self->{'options_hash'}->{"size"} =~ /k$/ ? "selected" : "";
	printf "<option value=b %s> bytes\n",
		$self->{'options_hash'}->{"size"} !~ /(k|m)$/ ? "selected" : "";
	print "</select></td>\n";

	print "<td><b>$self->{'text_hash'}->{'solaris_largebsize'}</b></td>\n";
	printf "<td nowrap><input type=radio name=xmemfs_largebsize value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		defined($self->{'options_hash'}->{"largebsize"}) ? "checked" : "";
	printf "<input type=radio name=xmemfs_largebsize value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		defined($self->{'options_hash'}->{"largebsize"}) ? "" : "checked";
	}
}

# check_location(type)
# Parse and check inputs from %{$self->{'in_hash'}}, calling $self->error() if something is wrong.
# Returns the location string for storing in the fstab file
sub check_location
{
	my $self = shift;

if ($_[0] eq "nfs") {
	local($out, $temp, $mout, $dirlist);

	if ($self->{'in_hash'}->{'nfs_serv'} == 1) {
		# multiple servers listed.. assume the user has a brain
		return $self->{'in_hash'}->{'nfs_list'};
		}
	elsif ($self->{'in_hash'}->{'nfs_serv'} == 2) {
		# NFS url.. check syntax
		if ($self->{'in_hash'}->{'nfs_url'} !~ /^nfs:\/\/([^\/ ]+)(\/([^\/ ]*))?$/) {
			$self->error($self->text('solaris_eurl', $self->{'in_hash'}->{'nfs_url'}));
			}
		return $self->{'in_hash'}->{'nfs_url'};
		}

	# Use dfshares to see if the host exists and is up
	if ($self->{'in_hash'}->{nfs_host} !~ /^\S+$/) {
		$self->error($self->text('solaris_ehost', $self->{'in_hash'}->{'nfs_host'}));
		}
	$out = $self->backquote_command("dfshares '$self->{'in_hash'}->{nfs_host}' 2>&1");
	if ($out =~ /Unknown host/) {
		$self->error($self->text('solaris_ehost2', $self->{'in_hash'}->{'nfs_host'}));
		}
	elsif ($out =~ /Timed out/) {
		$self->error($self->text('solaris_edown', $self->{'in_hash'}->{'nfs_host'}));
		}
	elsif ($out =~ /Program not registered/) {
		$self->error($self->text('solaris_enfs', $self->{'in_hash'}->{'nfs_host'}));
		}

	# Try a test mount to see if filesystem is available
	foreach (split(/\n/, $out)) {
		if (/^\s*([^ :]+):(\/\S+)\s+/) { $dirlist .= "$2\n"; }
		}
	if ($self->{'in_hash'}->{nfs_dir} !~ /^\S+$/) {
		$self->error($self->text('solaris_enfsdir', $self->{'in_hash'}->{'nfs_dir'},
			     $self->{'in_hash'}->{'nfs_host'}, "<pre>$dirlist</pre>"));
		}
	$temp = $self->transname();
	$self->make_dir($temp, 0755);
	$mout = $self->backquote_command("mount $self->{'in_hash'}->{nfs_host}:$self->{'in_hash'}->{nfs_dir} $temp 2>&1");
	if ($mout =~ /No such file or directory/) {
		rmdir($temp);
		$self->error($self->text('solaris_enfsdir', $self->{'in_hash'}->{'nfs_dir'},
			     $self->{'in_hash'}->{'nfs_host'}, "<pre>$dirlist</pre>"));
		}
	elsif ($mout =~ /Permission denied/) {
		rmdir($temp);
		$self->error($self->text('solaris_enfsperm', $self->{'in_hash'}->{'nfs_dir'}, $self->{'in_hash'}->{'nfs_host'}));
		}
	elsif ($?) {
		rmdir($temp);
		$self->error($self->text('solaris_enfsmount', "<tt>$mout</tt>"));
		}
	# It worked! unmount
	$self->execute_command("umount $temp");
	$self->unlink_file($temp);
	return "$self->{'in_hash'}->{nfs_host}:$self->{'in_hash'}->{nfs_dir}";
	}
elsif ($_[0] eq "ufs" || $_[0] eq "pcfs") {
	# Get the device name
	if ($self->{'in_hash'}->{ufs_dev} == 0) {
		$self->{'dv'} = $self->{'in_hash'}->{'ufs_disk'};
		}
	elsif ($self->{'in_hash'}->{ufs_dev} == 1) {
		$self->{'in_hash'}->{ufs_md} =~ /^[0-9]+$/ ||
			$self->error($self->text('solaris_eraid', $self->{'in_hash'}->{'ufs_md'}));
		$self->{'dv'} = "/dev/md/dsk/d$self->{'in_hash'}->{ufs_md}";
		}
	else {
		$self->{'in_hash'}->{ufs_path} =~ /^\/\S+$/ ||
			$self->error($self->text('solaris_epath', $self->{'in_hash'}->{'ufs_path'}));
		$self->{'dv'} = $self->{'in_hash'}->{ufs_path};
		}

	$self->fstyp_check($self->{'dv'}, $_[0]);
	return $self->{'dv'};
	}
elsif ($_[0] eq "hsfs" || $_[0] eq "udfs") {
	# Get the device name
	if ($self->{'in_hash'}->{ufs_dev} == 0) {
		$self->{'in_hash'}->{ufs_c} =~ /^[0-9]+$/ ||
			$self->error($self->text('solaris_ectrlr', $self->{'in_hash'}->{'ufs_c'}));
		$self->{'in_hash'}->{ufs_t} =~ /^[0-9]+$/ ||
			$self->error($self->text('solaris_etarget', $self->{'in_hash'}->{'ufs_t'}));
		$self->{'in_hash'}->{ufs_d} =~ /^[0-9]+$/ ||
			$self->error($self->text('solaris_eunit', $self->{'in_hash'}->{'ufs_d'}));
		$self->{'in_hash'}->{ufs_s} =~ /^[0-9]+$/ ||
			$self->error($self->text('solaris_epart', $self->{'in_hash'}->{'ufs_s'}));
		$self->{'dv'} = "/dev/dsk/c$self->{'in_hash'}->{ufs_c}t$self->{'in_hash'}->{ufs_t}d$self->{'in_hash'}->{ufs_d}s$self->{'in_hash'}->{ufs_s}";
		}
	else {
		$self->{'in_hash'}->{ufs_path} =~ /^\/\S+$/ ||
			$self->error($self->text('solaris_epath', $self->{'in_hash'}->{'ufs_path'}));
		$self->{'dv'} = $self->{'in_hash'}->{ufs_path};
		}

	return $self->{'dv'};
	}
elsif ($_[0] eq "lofs") {
	# Get and check the original directory
	$self->{'dv'} = $self->{'in_hash'}->{'lofs_src'};
	if (!(-r $self->{'dv'})) { $self->error($self->text('solaris_eexist', $self->{'dv'})); }
	if (!(-d $self->{'dv'})) { $self->error($self->text('solaris_edir', $self->{'dv'})); }
	return $self->{'dv'};
	}
elsif ($_[0] eq "swap") {
	if ($self->{'in_hash'}->{swap_dev} == 0) {
		$self->{'dv'} = $self->{'in_hash'}->{'swap_disk'};
		}
	else { $self->{'dv'} = $self->{'in_hash'}->{swap_path}; }

	if (!open(SWAPFILE, $self->{'dv'})) {
		if ($! =~ /No such file/ && $self->{'in_hash'}->{swap_dev}) {
			if ($self->{'dv'} !~ /^\/dev/) {
				$self->swap_form($self->{'dv'});
				}
			else {
				$self->error($self->text('solaris_eswapfile', $self->{'dv'}));
				}
			}
		elsif ($! =~ /No such file/) {
			$self->error($self->text('solaris_etarget', $self->{'in_hash'}->{'swap_t'}));
			}
		elsif ($! =~ /No such device or address/) {
			$self->error($self->text('solaris_epart', $self->{'in_hash'}->{'swap_s'}));
			}
		else {
			$self->error($self->text('solaris_eopen', $self->{'dv'}, $!));
			}
		}
	close(SWAPFILE);
	return $self->{'dv'};
	}
elsif ($_[0] eq "tmpfs") {
	# Ram-disk filesystems have no location
	return "swap";
	}
elsif ($_[0] eq "xmemfs") {
	# Large ram-disk filesystems have no location
	return "xmem";
	}
elsif ($_[0] eq "cachefs") {
	# In order to check the location for the caching filesystem, we need
	# to check the back filesystem
	if (!$self->{'in_hash'}->{cfs_noback}) {
		# The back filesystem is manually mounted.. hopefully
		local($bidx, @mlist, @binfo);
		$bidx = $self->get_mounted($self->{'in_hash'}->{cfs_backpath}, "*");
		if ($bidx < 0) {
			$self->error($self->text('solaris_ebackfs', $self->{'in_hash'}->{'cfs_backpath'}));
			}
		@mlist = $self->list_mounted();
		@binfo = @{$mlist[$bidx]};
		if ($binfo[2] ne $self->{'in_hash'}->{cfs_backfstype}) {
			$self->error($self->text('solaris_ebacktype', $binfo[2], $self->{'in_hash'}->{'cfs_backfstype'}));
			}
		}
	else {
		# Need to automatically mount the back filesystem.. check
		# it for sanity first.
		# But HOW?
		$self->{'in_hash'}->{cfs_src} =~ /^\S+$/ ||
			$self->error($self->text('solaris_ecsrc', $self->{'in_hash'}->{'cfs_src'}));
		}
	return $self->{'in_hash'}->{cfs_src};
	}
elsif ($_[0] eq "autofs") {
	# An autofs filesystem can be either mounted from the special
	# -hosts and -xfn maps, or from a normal map. The map can be a file
	# name (if it starts with /), or an NIS map (if it doesn't)
	if ($self->{'in_hash'}->{autofs_type} == 0) {
		# Normal map
		$self->{'in_hash'}->{autofs_map} =~ /\S/ ||
			$self->error($self->{'text_hash'}->{'solaris_eautomap'});
		if ($self->{'in_hash'}->{autofs_map} =~ /^\// && !(-r $self->{'in_hash'}->{autofs_map})) {
			$self->error($self->text('solaris_eautofile', $self->{'in_hash'}->{'autofs_map'}));
			}
		return $self->{'in_hash'}->{autofs_map};
		}
	elsif ($self->{'in_hash'}->{autofs_type} == 1) {
		# Special hosts map (automount all shares from some host)
		return "-hosts";
		}
	else {
		# Special FNS map (not sure what this does)
		return "-xfn";
		}
	}
elsif ($_[0] eq "rumba") {
	# Cannot check much here..
	return "\\\\$self->{'in_hash'}->{rumba_server}\\$self->{'in_hash'}->{rumba_share}";
	}
}

# fstyp_check(device, type)
# Check if some device exists, and contains a filesystem of the given type,
# using the fstyp command.
sub fstyp_check
{
	my $self = shift;

local($out, $part, $found);

# Check if the device/partition actually exists
if ($_[0] =~ /^\/dev\/dsk\/c(.)t(.)d(.)s(.)$/) {
	# mounting a normal scsi device..
	$out = $self->backquote_command("prtvtoc -h $_[0] 2>&1");
	if ($out =~ /No such file or directory|No such device or address/) {
		$self->error($self->text('solaris_etarget2', $_[0]));
		}
	$part = $4;
	foreach (split(/\n/, $out)) {
		/^\s+([0-9]+)\s+([0-9]+)/;
		if ($1 == $part) {
			$found = 1; last;
			}
		}
	if (!$found) {
		$self->error($self->text('solaris_epart2', $_[0]));
		}
	}
elsif ($_[0] =~ /^\/dev\/md\/dsk\/d(.)$/) {
	# mounting a multi-disk (raid) device..
	$out = $self->backquote_command("prtvtoc -h $_[0] 2>&1");
	if ($out =~ /No such file or directory|No such device or address/) {
		$self->error($self->text('solaris_eraid2', $_[0]));
		}
	if ($out !~ /\S/) {
		$self->error($self->text('solaris_enopart', $_[0]));
		}
	}
else {
	# Some other device
	if (!open(DEV, $_[0])) {
		if ($! =~ /No such file or directory/) {
			$self->error($self->text('solaris_edevfile', $_[0]));
			}
		elsif ($! =~ /No such device or address/) {
			$self->error($self->text('solaris_edevice', $_[0]));
			}
		}
	close(DEV);
	}

# Check the filesystem type
$out = $self->backquote_command("fstyp $_[0] 2>&1");
if ($out =~ /^([A-Za-z0-9]+)\n$/) {
	if ($1 eq $_[1]) { return; }
	else {
		# Wrong filesystem type
		$self->error($self->text('solaris_efstyp', $_[0], $self->fstype_name($1)));
		}
	}
else {
	$self->error($self->text('solaris_efstyp2', $out));
	}
}

# $self->check_options(type)
# Read options for some filesystem from %{$self->{'in_hash'}}, and use them to update the
# %{$self->{'options_hash'}} array. Options handled by the user interface will be set or
# removed, while unknown options will be left untouched.
sub check_options
{
	my $self = shift;

local($k, @rv);
if ($_[0] eq "nfs") {
	# NFS has lots of options to parse
	if ($self->{'in_hash'}->{'nfs_ro'}) {
		# Read-only
		$self->{'options_hash'}->{"ro"} = ""; delete($self->{'options_hash'}->{"rw"});
		}
	else {
		# Read-write
		$self->{'options_hash'}->{"rw"} = ""; delete($self->{'options_hash'}->{"ro"});
		}

	delete($self->{'options_hash'}->{'quota'}); delete($self->{'options_hash'}->{'noquota'});
	if ($self->{'in_hash'}->{'nfs_quota'}) { $self->{'options_hash'}->{'quota'} = ""; }

	delete($self->{'options_hash'}->{"nosuid"}); delete($self->{'options_hash'}->{"suid"});
	if ($self->{'in_hash'}->{nfs_nosuid}) { $self->{'options_hash'}->{"nosuid"} = ""; }

	delete($self->{'options_hash'}->{"grpid"});
	if ($self->{'in_hash'}->{nfs_grpid}) { $self->{'options_hash'}->{"grpid"} = ""; }

	delete($self->{'options_hash'}->{"soft"}); delete($self->{'options_hash'}->{"hard"});
	if ($self->{'in_hash'}->{nfs_soft}) { $self->{'options_hash'}->{"soft"} = ""; }

	delete($self->{'options_hash'}->{"bg"}); delete($self->{'options_hash'}->{"fg"});
	if ($self->{'in_hash'}->{nfs_bg}) { $self->{'options_hash'}->{"bg"} = ""; }

	delete($self->{'options_hash'}->{"intr"}); delete($self->{'options_hash'}->{"nointr"});
	if ($self->{'in_hash'}->{nfs_nointr}) { $self->{'options_hash'}->{"nointr"} = ""; }

	delete($self->{'options_hash'}->{"vers"});
	if (!$self->{'in_hash'}->{nfs_vers_def}) { $self->{'options_hash'}->{"vers"} = $self->{'in_hash'}->{nfs_vers}; }

	delete($self->{'options_hash'}->{"proto"});
	if ($self->{'in_hash'}->{nfs_proto} ne "") { $self->{'options_hash'}->{"proto"} = $self->{'in_hash'}->{nfs_proto}; }

	delete($self->{'options_hash'}->{"port"});
	if (!$self->{'in_hash'}->{nfs_port_def}) { $self->{'options_hash'}->{"port"} = $self->{'in_hash'}->{nfs_port}; }

	delete($self->{'options_hash'}->{"timeo"});
	if (!$self->{'in_hash'}->{nfs_timeo_def}) { $self->{'options_hash'}->{"timeo"} = $self->{'in_hash'}->{nfs_timeo}; }

	delete($self->{'options_hash'}->{"secure"}); delete($self->{'options_hash'}->{"kerberos"});
	delete($self->{'options_hash'}->{"sec"});
	if ($self->{'gconfig_hash'}->{'os_version'} >= 2.6) {
		if ($self->{'in_hash'}->{'nfs_auth'}) { $self->{'options_hash'}->{'sec'} = $self->{'in_hash'}->{'nfs_auth'}; }
		}
	else {
		if ($self->{'in_hash'}->{'nfs_auth'} eq "dh") { $self->{'options_hash'}->{"secure"} = ""; }
		elsif ($self->{'in_hash'}->{'nfs_auth'} eq "krb") { $self->{'options_hash'}->{"kerberos"} = ""; }
		}

	if ($self->{'gconfig_hash'}->{'os_version'} >= 7) {
		delete($self->{'options_hash'}->{'public'});
		$self->{'options_hash'}->{'public'} = "" if ($self->{'in_hash'}->{'nfs_public'});
		}
	}
elsif ($_[0] eq "ufs") {
	# UFS also has lots of options..
	if ($self->{'in_hash'}->{ufs_ro}) {
		# read-only (and thus no quota)
		$self->{'options_hash'}->{"ro"} = ""; delete($self->{'options_hash'}->{"rw"});
		delete($self->{'options_hash'}->{"rq"}); delete($self->{'options_hash'}->{"quota"});
		}
	elsif ($self->{'in_hash'}->{ufs_quota}) {
		# read-write, with quota
		delete($self->{'options_hash'}->{"ro"}); $self->{'options_hash'}->{"rw"} = "";
		$self->{'options_hash'}->{"quota"} = "";
		}
	else {
		# read-write, without quota
		delete($self->{'options_hash'}->{"ro"}); $self->{'options_hash'}->{"rw"} = "";
		delete($self->{'options_hash'}->{"quota"});
		}

	delete($self->{'options_hash'}->{"nosuid"});
	if ($self->{'in_hash'}->{ufs_nosuid}) { $self->{'options_hash'}->{"nosuid"} = ""; }

	delete($self->{'options_hash'}->{"intr"}); delete($self->{'options_hash'}->{"nointr"});
	if ($self->{'in_hash'}->{ufs_nointr}) { $self->{'options_hash'}->{"nointr"} = ""; }

	delete($self->{'options_hash'}->{"onerror"});
	if ($self->{'in_hash'}->{ufs_onerror} ne "panic") {
		$self->{'options_hash'}->{"onerror"} = $self->{'in_hash'}->{ufs_onerror};
		}

	if ($self->{'gconfig_hash'}->{'os_version'} >= 7) {
		if ($self->{'gconfig_hash'}->{'os_version'} >= 8) {
			delete($self->{'options_hash'}->{'noatime'});
			delete($self->{'options_hash'}->{'dfratime'}); delete($self->{'options_hash'}->{'nodfratime'});
			if ($self->{'in_hash'}->{'ufs_noatime'} == 0) { $self->{'options_hash'}->{'nodfratime'} = ""; }
			elsif ($self->{'in_hash'}->{'ufs_noatime'} == 2) { $self->{'options_hash'}->{'noatime'} = ""; }
			}
		else {
			delete($self->{'options_hash'}->{'noatime'});
			$self->{'options_hash'}->{'noatime'} = "" if ($self->{'in_hash'}->{'ufs_noatime'});
			}

		delete($self->{'options_hash'}->{'forcedirectio'});
		delete($self->{'options_hash'}->{'noforcedirectio'});
		$self->{'options_hash'}->{'forcedirectio'} = "" if ($self->{'in_hash'}->{'ufs_force'});

		delete($self->{'options_hash'}->{'nolargefiles'});delete($self->{'options_hash'}->{'largefiles'});
		$self->{'options_hash'}->{'nolargefiles'} = "" if ($self->{'in_hash'}->{'ufs_nolarge'});

		delete($self->{'options_hash'}->{'logging'}); delete($self->{'options_hash'}->{'nologging'});
		$self->{'options_hash'}->{'logging'} = "" if ($self->{'in_hash'}->{'ufs_logging'});
		}
	else {
		delete($self->{'options_hash'}->{"toosoon"});
		if ($self->{'in_hash'}->{ufs_toosoon_time}) {
			$self->{'options_hash'}->{"toosoon"} = $self->{'in_hash'}->{ufs_toosoon_time}.
					      $self->{'in_hash'}->{ufs_toosoon_units};
			}
		}
	}
elsif ($_[0] eq "lofs") {
	# LOFS has a few options
	if ($self->{'in_hash'}->{'nfs_ro'}) {
		# Read-only
		$self->{'options_hash'}->{"ro"} = ""; delete($self->{'options_hash'}->{"rw"});
		}
	else {
		# Read-write
		$self->{'options_hash'}->{"rw"} = ""; delete($self->{'options_hash'}->{"ro"});
		}

	delete($self->{'options_hash'}->{"nosuid"});
	if ($self->{'in_hash'}->{ufs_nosuid}) { $self->{'options_hash'}->{"nosuid"} = ""; }
	}
elsif ($_[0] eq "swap") {
	# Swap has no options to parse
	}
elsif ($_[0] eq "pcfs") {
	# PCFS has only 2 options
	delete($self->{'options_hash'}->{'ro'}); delete($self->{'options_hash'}->{'rw'});
	$self->{'options_hash'}->{'ro'} = "" if ($self->{'in_hash'}->{'pcfs_ro'});

	delete($self->{'options_hash'}->{'foldcase'}); delete($self->{'options_hash'}->{'nofoldcase'});
	$self->{'options_hash'}->{'foldcase'} = "" if ($self->{'in_hash'}->{'pcfs_foldcase'});
	}
elsif ($_[0] eq "hsfs") {
	# Options for ISO-9660 filesystems
	delete($self->{'options_hash'}->{'nrr'});
	$self->{'options_hash'}->{'nrr'} = "" if ($self->{'in_hash'}->{'hsfs_nrr'});

	delete($self->{'options_hash'}->{"notraildot"});
	$self->{'options_hash'}->{"notraildot"} = "" if ($self->{'in_hash'}->{'hsfs_notraildot'});

	delete($self->{'options_hash'}->{'nomaplcase'});
	$self->{'options_hash'}->{'nomaplcase'} = "" if ($self->{'in_hash'}->{'hsfs_nomaplcase'});

	delete($self->{'options_hash'}->{'nosuid'});
	$self->{'options_hash'}->{'nosuid'} = "" if ($self->{'in_hash'}->{'hsfs_nosuid'});
	}
elsif ($_[0] eq "tmpfs") {
	# Ram-disk filesystems have only two options
	delete($self->{'options_hash'}->{"size"});
	if (!$self->{'in_hash'}->{"tmpfs_size_def"}) {
		$self->{'options_hash'}->{"size"} = "$self->{'in_hash'}->{tmpfs_size}$self->{'in_hash'}->{tmpfs_unit}";
		}

	delete($self->{'options_hash'}->{"nosuid"});
	if ($self->{'in_hash'}->{'tmpfs_nosuid'}) { $self->{'options_hash'}->{"nosuid"} = ""; }
	}
elsif ($_[0] eq "xmemfs") {
	# Large ram-disk filesystems have only two options
	delete($self->{'options_hash'}->{"size"});
	if (!$self->{'in_hash'}->{"xmemfs_size_def"}) {
		$self->{'options_hash'}->{"size"} = "$self->{'in_hash'}->{xmemfs_size}$self->{'in_hash'}->{xmemfs_unit}";
		}

	delete($self->{'options_hash'}->{"largebsize"});
	if ($self->{'in_hash'}->{'xmemfs_largebsize'}) { $self->{'options_hash'}->{"largebsize"} = ""; }

	delete($self->{'options_hash'}->{'rw'});
	}
elsif ($_[0] eq "cachefs") {
	# The caching filesystem has lots of options
	$self->{'options_hash'}->{"backfstype"} = $self->{'in_hash'}->{"cfs_backfstype"};

	delete($self->{'options_hash'}->{"backpath"});
	if (!$self->{'in_hash'}->{"cfs_noback"}) {
		# A back filesystem was given..  (alreadys checked)
		$self->{'options_hash'}->{"backpath"} = $self->{'in_hash'}->{"cfs_backpath"};
		}

	if ($self->{'in_hash'}->{"cfs_cachedir"} !~ /^\/\S+/) {
		$self->error($self->text('solaris_ecachedir', $self->{'in_hash'}->{'cfs_cachedir'}));
		}
	$self->{'options_hash'}->{"cachedir"} = $self->{'in_hash'}->{"cfs_cachedir"};

	delete($self->{'options_hash'}->{"write-around"}); delete($self->{'options_hash'}->{"non-shared"});
	if ($self->{'in_hash'}->{"cfs_wmode"}) {
		$self->{'options_hash'}->{"non-shared"} = "";
		}

	delete($self->{'options_hash'}->{"noconst"}); delete($self->{'options_hash'}->{"demandconst"});
	if ($self->{'in_hash'}->{"cfs_con"} == 0) { $self->{'options_hash'}->{"noconst"} = ""; }
	elsif ($self->{'in_hash'}->{"cfs_con"} == 2) { $self->{'options_hash'}->{"demandconst"} = ""; }

	delete($self->{'options_hash'}->{"ro"}); delete($self->{'options_hash'}->{"rw"});
	if ($self->{'in_hash'}->{"cfs_ro"}) { $self->{'options_hash'}->{"ro"} = ""; }

	delete($self->{'options_hash'}->{"suid"}); delete($self->{'options_hash'}->{"nosuid"});
	if ($self->{'in_hash'}->{"cfs_nosuid"}) { $self->{'options_hash'}->{"nosuid"} = ""; }
	}
elsif ($_[0] eq "autofs") {
	# The options for autofs depend on the type of the automounted
	# filesystem.. 
	$self->{'options_hash'}->{"fstype"} = $self->{'in_hash'}->{"autofs_fstype"};
	if ($self->{'gconfig_hash'}->{'os_version'} >= 2.6) {
		delete($self->{'options_hash'}->{'nobrowse'}); delete($self->{'options_hash'}->{'browse'});
		$self->{'options_hash'}->{'nobrowse'} = "" if ($self->{'in_hash'}->{'auto_nobrowse'});
		}
	return $self->check_options($self->{'options_hash'}->{"fstype"});
	}
elsif ($_[0] eq "rumba") {
	# Options for smb filesystems..
	delete($self->{'options_hash'}->{machinename});
	if (!$self->{'in_hash'}->{rumba_mname_def}) { $self->{'options_hash'}->{machinename} = $self->{'in_hash'}->{rumba_mname}; }

	delete($self->{'options_hash'}->{clientname});
	if (!$self->{'in_hash'}->{rumba_cname_def}) { $self->{'options_hash'}->{clientname} = $self->{'in_hash'}->{rumba_cname}; }

	delete($self->{'options_hash'}->{username});
	if ($self->{'in_hash'}->{rumba_username}) { $self->{'options_hash'}->{username} = $self->{'in_hash'}->{rumba_username}; }

	delete($self->{'options_hash'}->{password});
	if ($self->{'in_hash'}->{rumba_password}) { $self->{'options_hash'}->{password} = $self->{'in_hash'}->{rumba_password}; }

	delete($self->{'options_hash'}->{uid});
	if ($self->{'in_hash'}->{rumba_uid} ne "") { $self->{'options_hash'}->{uid} = getpwnam($self->{'in_hash'}->{rumba_uid}); }

	delete($self->{'options_hash'}->{gid});
	if ($self->{'in_hash'}->{rumba_gid} ne "") { $self->{'options_hash'}->{gid} = getgrnam($self->{'in_hash'}->{rumba_gid}); }

	delete($self->{'options_hash'}->{fmode});
	if ($self->{'in_hash'}->{rumba_fmode} !~ /^[0-7]{3}$/) {
		$self->error($self->text('solaris_efmode', $self->{'in_hash'}->{'rumba_fmode'}));
		}
	elsif ($self->{'in_hash'}->{rumba_fmode} ne "755") { $self->{'options_hash'}->{fmode} = $self->{'in_hash'}->{rumba_fmode}; }

	delete($self->{'options_hash'}->{dmode});
	if ($self->{'in_hash'}->{rumba_dmode} !~ /^[0-7]{3}$/) {
		$self->error($self->text('solaris_edmode', $self->{'in_hash'}->{'rumba_dmode'}));
		}
	elsif ($self->{'in_hash'}->{rumba_dmode} ne "755") { $self->{'options_hash'}->{dmode} = $self->{'in_hash'}->{rumba_dmode}; }

	delete($self->{'options_hash'}->{'readwrite'});
	if ($self->{'in_hash'}->{'rumba_readwrite'}) { $self->{'options_hash'}->{'readwrite'} = ""; }

	delete($self->{'options_hash'}->{'readonly'});
	if ($self->{'in_hash'}->{'rumba_readonly'}) { $self->{'options_hash'}->{'readonly'} = ""; }

	delete($self->{'options_hash'}->{'attr'});
	if ($self->{'in_hash'}->{'rumba_attr'}) { $self->{'options_hash'}->{'attr'} = ""; }

	delete($self->{'options_hash'}->{'noupper'});
	if ($self->{'in_hash'}->{'rumba_noupper'}) { $self->{'options_hash'}->{'noupper'} = ""; }
	}
elsif ($_[0] eq "udfs") {
	# The DVD filesystem has only 2 options
	delete($self->{'options_hash'}->{'ro'}); delete($self->{'options_hash'}->{'rw'});
	$self->{'options_hash'}->{'ro'} = "" if ($self->{'in_hash'}->{'udfs_ro'});

	delete($self->{'options_hash'}->{"nosuid"});
	if ($self->{'in_hash'}->{'udfs_nosuid'}) { $self->{'options_hash'}->{"nosuid"} = ""; }
	}

# Return options string
foreach $k (keys %{$self->{'options_hash'}}) {
	if ($self->{'options_hash'}->{$k} eq "") { push(@rv, $k); }
	else { push(@rv, "$k=$self->{'options_hash'}->{$k}"); }
	}
return @rv ? join(',' , @rv) : "-";
}

# create_swap(path, size, units)
# Attempt to create a swap file 
sub create_swap
{
	my $self = shift;

local($out);
$out = $self->backquote_logged("mkfile $_[1]$_[2] $_[0] 2>&1");
if ($?) {
	$self->unlink_file($_[0]);
	return "mkfile failed : $out";
	}
return 0;
}

# exports_list(host, dirarray, clientarray)
# Fills the directory and client array references with exports from some
# host. Returns an error string if something went wrong
sub exports_list
{
	my $self = shift;

local($dref, $cref, $out, $_);
$dref = $_[1]; $cref = $_[2];
$out = $self->backquote_command("showmount -e ".quotemeta($_[0])." 2>&1", 1);
if ($?) { return $out; }
foreach (split(/\n/, $out)) {
	if (/^(\/\S*)\s+(.*)$/) {
		push(@$dref, $1); push(@$cref, $2);
		}
	}
return undef;
}

# broadcast_addr()
# Returns a useable broadcast address for finding NFS servers
sub broadcast_addr
{
	my $self = shift;

local($out);
$out = $self->backquote_command("ifconfig -a 2>&1", 1);
if ($out =~ /broadcast\s+(\S+)/) { return $1; }
return "255.255.255.255";
}

# device_name(device)
# Converts a device name to a human-readable form
sub device_name
{
	my $self = shift;

return $_[0] =~ /\/dev\/dsk\/c(\d+)t(\d+)d(\d+)s(\d+)$/ ?
	$self->text('solaris_scsidev', "$1", "$2", "$3", "$4") :
       $_[0] =~ /^\/dev\/md\/dsk\/d([0-9]+)$/ ?
	$self->text('solaris_mddev', "$1") :
       $_[0] =~ /^\/dev\/dsk\/c(\d+)d(\d+)s(\d+)$/ ?
	$self->text('solaris_idedev', "$1", "$2", "$3") :
       $_[0] eq "-hosts" ?
	$self->{'text_hash'}->{'solaris_autohosts'} :
       $_[0] eq "-xfn" ?
	$self->{'text_hash'}->{'solaris_autoxfn'} :
	$_[0];
}

sub files_to_lock
{
	my $self = shift;

return ( $self->{'config_hash'}->{'fstab_file'}, $self->{'config_hash'}->{'autofs_file'} );
}


1;

