#!/usr/bin/perl
package net::GentoolinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'net::GentoolinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# gentoo-linux-lib.pl
# Networking functions for gentoo linux

use Data::Dumper;

$self->{'net_config'} = "/etc/conf.d/net";
$self->{'routes_config'} = "/etc/conf.d/routes";
$self->{'sysctl_config'} = "/etc/sysctl.conf";

## This is a template for the net startscripts used for starting interfaces at boot time
#  I'm not quite sure but I think the one for the first Interface comes with gentoo.
$self->{'net_template'} = "/etc/init.d/net.eth0";

## Since Gentoo starts and stops interfaces like daemons these directories are needed.
$self->{'init_scripts'} = "/etc/init.d";
$self->{'boot_scripts'} = "/etc/runlevels/default";

&ModuleLoader::preload_module(ref($self), "net::LinuxLib");
$self = new net::LinuxLib:: ref($self), %$self; 

1;

1;
		return $self;
}

# boot_interfaces()
# Returns a list of interfaces brought up at boot time
sub boot_interfaces
{
	my $self = shift;

my %nconfig;
$self->read_env_file($self->{'net_config'}, \%nconfig);
foreach $f (sort keys %nconfig) {
	local (%conf, $b);
	## [Gentoo] If we've got a physical Interface
	if ($f =~ /^iface_([a-z]+\d*(\.\d+)?)$/) {

		## [Gentoo] Set IP, Netmask, Broadcast.Interface is to be
		#  started at boot-time if start stript is found in the default
		#  runlevel directoriy (etc/runlevels/default)
		$b->{'name'} = $1;
		$b->{'fullname'} = $b->{'name'};

		## [Gentoo] Push the configuration string into an
		#  array to take care of wich arg is which.
		local @config_string = split(/\s+/,$nconfig{$f});
		if($config_string[0] eq 'dhcp') {
			$b->{'dhcp'} = 'dhcp';
		}
		$b->{'address'} = $config_string[0];
		$b->{$config_string[1]} = $config_string[2];
		$b->{$config_string[3]} = $config_string[4];
		if(-e $self->{'boot_scripts'} . '/net.' . $b->{'name'}) {
			$b->{'up'} = 'onboot';
		} else {
			$b->{'up'} = '';
		}
		$b->{'edit'} = ($b->{'name'} !~ /^ppp|irlan/);
	        $b->{'index'} = scalar(@{$self->{'rv_array'}});
		$b->{'virtual'} = '';
		$b->{'file'} = $self->{'net_config'};

	## [Gentoo] For Aliases. This is a bit difficult. they are stored space-seperated.
	#  So we'll push them as arrays into a href per interface
	} elsif ($f =~ /^alias_([a-z]+\d*(\.\d+)?)$/) {
		@{$self->{'addresses_hash'}->{$1}} =  split(/\s+/,$nconfig{$f});
		next;
	} elsif ($f =~ /^broadcast_([a-z]+\d*(\.\d+)?)$/) {
		@{$self->{'broadcasts_hash'}->{$1}} = split(/\s+/,$nconfig{$f});
		next;
	} elsif ($f =~ /^netmask_([a-z]+\d*(\.\d+)?)$/) {
		@{$self->{'netmasks_hash'}->{$1}} = split(/\s+/,$nconfig{$f});
		next;
	} else {
		next;
	}
push(@{$self->{'rv_array'}}, $b);
}

	# [Gentoo] Push the Alias Interfaces into the array as well.
	foreach (sort keys %{$self->{'addresses_hash'}}) {
		for $i (0 .. (scalar(@{$self->{'addresses_hash'}->{$_}})-1)) {
			local $p;
			$p->{'name'} = $_;
			$p->{'fullname'} = $_ . ":" . ($i+1);
			$p->{'broadcast'} = @{$self->{'broadcasts_hash'}->{$_}}[$i];
			$p->{'netmask'} = @{$self->{'netmasks_hash'}->{$_}}[$i];
			$p->{'address'} = @{$self->{'addresses_hash'}->{$_}}[$i];
			$p->{'virtual'} = $i;
			if(-e $self->{'boot_scripts'} . '/net.' . $p->{'name'}) {
				$p->{'up'} = 'onboot';
			} else {
				$p->{'up'} = '';
			}
			$p->{'edit'} = 1;
			$p->{'index'} = scalar(@{$self->{'rv_array'}});
			$p->{'file'} = $self->{'net_config'};
			push(@{$self->{'rv_array'}}, $p);
		}
	}
return @{$self->{'rv_array'}};
}

# save_interface(&details)
# Create or update a boot-time interface
sub save_interface
{
	my $self = shift;

local(%conf);
$self->lock_file("$self->{'net_config'}");
$self->read_env_file("$self->{'net_config'}", \%conf);

## [Gentoo] If we have a normal Interface, we'll just write out the config line.
if($_[0]->{'virtual'} eq "") {
	if($_[0]->{'dhcp'}) {
		$conf{'iface_' . $_[0]->{'name'}} = 'dhcp';
	} else {
		$conf{'iface_' . $_[0]->{'name'}} = $_[0]->{'address'} . 
				    " broadcast " .
				    $_[0]->{'broadcast'} . 
				    " netmask " .
				    $_[0]->{'netmask'};
	}
} else {
	
## [Gentoo] We don't need to check if an alias of the given address already exists, since
#  save_iface.cgi does this already.
	$conf{'alias_' . $_[0]->{'name'}} .= " " . $_[0]->{'address'};
	$conf{'broadcast_' . $_[0]->{'name'}} .= " " . $_[0]->{'broadcast'};
	$conf{'netmask_' . $_[0]->{'name'}} .= " " .$_[0]->{'netmask'};
}

## [Gentoo] Need some Quotes and get rid of leading or ending spaces ..
foreach(sort keys %conf) {
	$conf{$_} =~ s/^\s*//g;
	$conf{$_} =~ s/\s*$//g;
	if($conf{$_}) {
		$self->{'write_hash'}->{$_} = '"' . $conf{$_} . '"';
	}

}

$self->write_file("$self->{'net_config'}", \%{$self->{'write_hash'}});
$self->unlock_file("$self->{'net_config'}");

## [Gentoo] If Interface is to be started at boot time,
#  create a link in the default runlevel directory. Else delete it if exists.
if(($_[0]->{'up'} == 1) && (!-e $self->{'boot_scripts'} . "/net." . $_[0]->{'name'})) {
	
	## [Gentoo] Some time we'll need to create a start script first.
	if(!-e $self->{'init_scripts'} . "/net." . $_[0]->{'name'}) {
		$self->lock_file($self->{'init_scripts'} . "/net." . $_[0]->{'name'});
		$self->open_readfile(*IN, $self->{'net_template'});
		local $tofile = $self->{'init_scripts'}."/net.".$_[0]->{'name'};
		$self->open_tempfile(OUT, ">$tofile");
		$self->copydata(\*IN, \*OUT) || $self->error($self->text("efilewrite", $tofile, $!));
		close IN;
		$self->close_tempfile(OUT);
		$self->unlock_file($self->{'init_scripts'} . "/net." . $_[0]->{'name'});
		chmod oct("0755"), $self->{'init_scripts'} . "/net." . $_[0]->{'name'};
	}
	$self->symlink_logged($self->{'init_scripts'} . "/net." . $_[0]->{'name'}, $self->{'boot_scripts'} . "/net." . $_[0]->{'name'});
} elsif (($_[0]->{'up'} == 0) && (-e $self->{'boot_scripts'} . "/net." . $_[0]->{'name'})) {
	$self->unlink_logged($self->{'boot_scripts'} . "/net." . $_[0]->{'name'});
}

}

# delete_interface(&details)
# Delete a boot-time interface
sub delete_interface
{
	my $self = shift;

local(%conf);
$self->lock_file("$self->{'net_config'}");
$self->read_env_file("$self->{'net_config'}", \%conf);

## [Gentoo] If we have a normal Interface, delete the line.
if($_[0]->{'virtual'} eq "") {
	undef($conf{'iface_' . $_[0]->{'name'}});
} else {

	## [Gentoo] First push Data into array, undef given array_seq, set it again for wrtiting
	#  Do this for the alias-, the netmask- and the broadcastline.
	foreach my $t('alias', 'netmask', 'broadcast') {
		local @ta = split(/\s+/, $conf{$t . '_' . $_[0]->{'name'}});
		undef(@ta[$_[0]->{'virtual'}]);
		$conf{$t . '_' . $_[0]->{'name'}} = join(' ', @ta);
	}
}

## [Gentoo] Need some Quotes ..
foreach(sort keys %conf) {
	$conf{$_} =~ s/^\s*//g;
	$conf{$_} =~ s/\s*$//g;
	if($conf{$_}) {
                $self->{'write_hash'}->{$_} = '"' . $conf{$_} . '"';
        }
}

$self->write_file("$self->{'net_config'}", \%{$self->{'write_hash'}});
$self->unlock_file("$self->{'net_config'}");

## [Gentoo] Now delete the boot script
$self->unlink_logged($self->{'boot_scripts'} . "/net." . $_[0]->{'name'});
}

# can_edit(what)
# Can some boot-time interface parameter be edited?
sub can_edit
{
	my $self = shift;

return $_[0] ne "mtu" && $_[0] ne "bootp";
}

# valid_boot_address(address)
# Is some address valid for a bootup interface
sub valid_boot_address
{
	my $self = shift;

return $self->check_ipaddress($_[0]);
}

# get_hostname()
sub get_hostname
{
	my $self = shift;

return $self->get_system_hostname(1);
}

# save_hostname(name)
sub save_hostname
{
	my $self = shift;

local %conf;
$self->system_logged("hostname $_[0] >/dev/null 2>&1");
$self->open_lock_tempfile(HOST, ">/etc/hostname");
$self->print_tempfile(HOST, $_[0],"\n");
$self->close_tempfile(HOST);
undef(@main::get_system_hostname);      # clear cache
}

# get_domainname()
sub get_domainname
{
	my $self = shift;

local $d;
$self->execute_command("domainname", undef, \$d, undef);
chop($d);
return $d;
}

# save_domainname(domain)
sub save_domainname
{
	my $self = shift;

local %conf;
$self->execute_command("domainname ".quotemeta($_[0]));
}

sub routing_config_files
{
	my $self = shift;

return ( $self->{'routes_config'}, $self->{'net_config'}, $self->{'sysctl_config'} );
}

sub routing_input
{
	my $self = shift;

local (@routes, $i);
$self->open_readfile(*ROUTES, $self->{'routes_config'});
while(<ROUTES>) {
	s/#.*$//;
	s/\r|\n//g;
	local @r = map { $_ eq '-' ? undef : $_ } split(/\s+/, $_);
	push(@routes, \@r) if (@r);
	}
close(ROUTES);

## [Gentoo] This is a workaround since Gentoo stores gateway Information in the net_config file

$self->lock_file($self->{'net_config'});
$self->read_env_file($self->{'net_config'}, \%{$self->{'gwif_hash'}});
$self->unlock_file($self->{'net_config'});

## [Gentoo] Push the standard gateway Information into @routes using the given format.

local @gwifad = split(/\//, $self->{'gwif_hash'}->{'gateway'});
push(local @gwline, 'default', $gwifad[1], undef, $gwifad[0]);
push(@routes, \@gwline) if (@gwline);

# show default router and device
local ($def) = grep { $_->[0] eq "default" } @routes;
print "<tr> <td><b>$self->{'text_hash'}->{'routes_default'}</b></td> <td>\n";
printf "<input type=radio name=gateway_def value=1 %s> $self->{'text_hash'}->{'routes_none'}\n",
	$def->[1] ? "" : "checked";
printf "<input type=radio name=gateway_def value=0 %s>\n",
	$def->[1] ? "checked" : "";
printf "<input name=gateway size=15 value=\"%s\"></td> </tr>\n",
	$def->[1];

print "<tr> <td><b>$self->{'text_hash'}->{'routes_device2'}</b></td> <td>\n";
printf "<input type=radio name=gatewaydev_def value=1 %s> $self->{'text_hash'}->{'routes_none'}\n",
	$def->[3] ? "" : "checked";
printf "<input type=radio name=gatewaydev_def value=0 %s>\n",
	$def->[3] ? "checked" : "";
printf "<input name=gatewaydev size=6 value=\"%s\"></td> </tr>\n",
	$def->[3];

$self->read_env_file($self->{'sysctl_config'}, \%{$self->{'sysctl_hash'}});
print "<tr> <td><b>$self->{'text_hash'}->{'routes_forward'}</b></td> <td>\n";
printf "<input type=radio name=forward value=1 %s> $self->{'text_hash'}->{'yes'}\n",
	$self->{'sysctl_hash'}->{'net.ipv4.ip_forward'} eq '1' ? "checked" : "";
printf "<input type=radio name=forward value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
	$self->{'sysctl_hash'}->{'net.ipv4.ip_forward'} eq '1' ? "" : "checked";

# show static network routes
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'routes_static'}</b></td>\n";
print "<td><table border>\n";
print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'routes_ifc'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_net'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_mask'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_gateway'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_type'}</b></td> </tr>\n";
local ($r, $i = 0);
foreach $r (@routes, [ ]) {
	next if ($r eq $def);
	print "<tr $self->{'cb'}>\n";
	print "<td><input name=dev_$i size=6 value='$r->[3]'></td>\n";
	print "<td><input name=net_$i size=15 value='$r->[0]'></td>\n";
	print "<td><input name=netmask_$i size=15 value='$r->[2]'></td>\n";
	print "<td><input name=gw_$i size=15 value='$r->[1]'></td>\n";
	print "<td><input name=type_$i size=10 value='$r->[4]'></td>\n";
	print "</tr>\n";
	$i++;
	}
print "</table></td> </tr>\n";
}

sub parse_routing
{
	my $self = shift;

# Parse route inputs
local (@routes, $r, $i);
if (!$self->{'in_hash'}->{'gateway_def'}) {
	gethostbyname($self->{'in_hash'}->{'gateway'}) ||
		$self->error($self->text('routes_edefault', $self->{'in_hash'}->{'gateway'}));
	local @def = ( "default", $self->{'in_hash'}->{'gateway'}, undef, undef );
	if (!$self->{'in_hash'}->{'gatewaydev_def'}) {
		$self->{'in_hash'}->{'gatewaydev'} =~ /^\S+$/ ||
			$self->error($self->text('routes_edevice', $self->{'in_hash'}->{'gatewaydev'}));
		$def[3] = $self->{'in_hash'}->{'gatewaydev'};
		}
	push(@routes, \@def);
	}
for($i=0; defined($self->{'in_hash'}->{"dev_$i"}); $i++) {
	next if (!$self->{'in_hash'}->{"net_$i"});
	$self->check_ipaddress($self->{'in_hash'}->{"net_$i"}) ||
		$self->{'in_hash'}->{"net_$i"} =~ /^(\S+)\/(\d+)$/ && $self->check_ipaddress($1) ||
		$self->error($self->text('routes_enet', $self->{'in_hash'}->{"net_$i"}));
	$self->{'in_hash'}->{"dev_$i"} =~ /^\S*$/ || $self->error($self->text('routes_edevice', $self->{'dev'}));
	!$self->{'in_hash'}->{"netmask_$i"} || $self->check_ipaddress($self->{'in_hash'}->{"netmask_$i"}) ||
		$self->error($self->text('routes_emask', $self->{'in_hash'}->{"netmask_$i"}));
	!$self->{'in_hash'}->{"gw_$i"} || $self->check_ipaddress($self->{'in_hash'}->{"gw_$i"}) ||
		$self->error($self->text('routes_egateway', $self->{'in_hash'}->{"gw_$i"}));
	$self->{'in_hash'}->{"type_$i"} =~ /^\S*$/ ||
		$self->error($self->text('routes_etype', $self->{'in_hash'}->{"type_$i"}));
	push(@routes, [ $self->{'in_hash'}->{"net_$i"}, $self->{'in_hash'}->{"gw_$i"}, $self->{'in_hash'}->{"netmask_$i"},
			$self->{'in_hash'}->{"dev_$i"}, $self->{'in_hash'}->{"type_$i"} ] );
	}

# Save routes and routing option
$self->open_tempfile(ROUTES, ">$self->{'routes_config'}");
foreach $r (@routes) {
	
	## [Gentoo] Get the default gw's entry and write it to net_config file instead of routes_config
	if(grep { $_ eq "default" } @$r) {
		local $gwif;
		local %nconfig;
		if(($$r[3] eq "") && ($$r[1] ne "")) {
			
			## If a gateway but no device is provided, get the device.
			$gwif = $self->get_gw_if($$r[1]);
		} elsif(($$r[4] eq "") && ($$r[1] eq "")) {

			## If neither gateway nor device is provided, don't care bout it.
			next;
		} else {
			$gwif = $$r[3];
		}

		## Now write it all into the file
		$self->lock_file($self->{'net_config'});
		$self->read_env_file($self->{'net_config'}, \%nconfig);
		$nconfig{'gateway'} = $gwif . "/" . $$r[1];
		foreach(sort keys %nconfig) {
		        $nconfig{$_} =~ s/^\s*//g;
	       		$nconfig{$_} =~ s/\s*$//g;
	      		if($nconfig{$_}) {
               			$self->{'write_hash'}->{$_} = '"' . $nconfig{$_} . '"';
			}
       		}
		$self->write_file($self->{'net_config'}, \%{$self->{'write_hash'}});
		$self->unlock_file($self->{'net_config'});

	## The rest is as it was.
	} else {
		$self->print_tempfile(ROUTES, join(" ", map { $_ eq '' ? "-" : $_ } @$r),"\n");
	}
}
$self->close_tempfile(ROUTES);
local $lref = $self->read_file_lines($self->{'sysctl_config'});
for($i=0; $i<@$lref; $i++) {
	if ($lref->[$i] =~ /^\s*net\.ipv4\.ip_forward\s*=/) {
		$lref->[$i] = "net.ipv4.ip_forward = ".($self->{'in_hash'}->{'forward'} ? 1 : 0);
		}
	}
$self->flush_file_lines();
}

sub os_feedback_files
{
	my $self = shift;

opendir(DIR, $self->{'net_scripts_dir'});
local @f = readdir(DIR);
closedir(DIR);
return ( (map { "$self->{'net_scripts_dir'}/$_" } grep { /^ifcfg-/ } @f),
	 $self->{'network_config'}, $self->{'static_route_config'}, "/etc/resolv.conf",
	 "/etc/nsswitch.conf", "/etc/hostname" );
}

## [Gentoo] Gentoo needs to know the interface through which to reach the gateway
#  (Uses the dev option with route)
sub get_gw_if
{
	my $self = shift;

	local $ip = shift;
	local $gwdel = 'route del default';
	$self->backquote_logged("$gwdel 2>&1");
	local $gwadd = 'route add default gw ' . $ip;
	$self->backquote_logged("$gwadd 2>&1") ;
	$self->open_execute_command(*ROUTE, "route -n", 1);
	while(<ROUTE>) {
		s/\r|\n//g;
		if($_ =~ /\d+/) {
			local($dest, $gw, $mask, $flags, $met, $ref, $use, $if) = split(/\s+/, $_);
			if($gw eq $ip) {
				return $if;
			}
		}
	}
	close ROUTE;
}


1;

