#!/usr/bin/perl
package net::NetLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'net::NetLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# net-lib.pl
# Common  networking functions

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();
$self->{'access_hash'}->{'ipnodes'} = $self->{'access_hash'}->{'hosts'};

if (-r "$self->{'module_root_directory'}/$self->{'gconfig_hash'}->{'os_type'}-$self->{'gconfig_hash'}->{'os_version'}-lib.pl") {
	{
    my $pkg = $self->lib_to_pkg("$self->{gconfig_hash}->{os_type}-$self->{gconfig_hash}->{os_version}-lib.pl", "net");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
	}
else {
	{
    my $pkg = $self->lib_to_pkg("$self->{gconfig_hash}->{os_type}-lib.pl", "net");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
	}

1;

1;
		return $self;
}

# list_hosts()
# Parse hosts from /etc/hosts into a data structure
sub list_hosts
{
	my $self = shift;

local @rv;
local $lnum = 0;
$self->open_readfile(*HOSTS, $self->{'config_hash'}->{'hosts_file'});
while(<HOSTS>) {
	s/\r|\n//g;
	s/#.*$//g;
	s/\s+$//g;
	if (/(\d+\.\d+\.\d+\.\d+)\s+(.*)$/) {
		push(@rv, { 'address' => $1,
			    'hosts' => [ split(/\s+/, $2) ],
			    'line', $lnum,
			    'index', scalar(@rv) });
		}
	$lnum++;
	}
close(HOSTS);
return @rv;
}

# create_host(&host)
# Add a new host to /etc/hosts
sub create_host
{
	my $self = shift;

$self->open_tempfile(HOSTS, ">>$self->{'config_hash'}->{'hosts_file'}");
$self->print_tempfile(HOSTS, $_[0]->{'address'},"\t",join(" ",@{$_[0]->{'hosts'}}),"\n");
$self->close_tempfile(HOSTS);
}

# modify_host(&host)
# Update the address and hosts of a line in /etc/hosts
sub modify_host
{
	my $self = shift;

$self->replace_file_line($self->{'config_hash'}->{'hosts_file'},
		   $_[0]->{'line'},
		   $_[0]->{'address'}."\t".join(" ",@{$_[0]->{'hosts'}})."\n");
}

# delete_host(&host)
# Delete a host from /etc/hosts
sub delete_host
{
	my $self = shift;

$self->replace_file_line($self->{'config_hash'}->{'hosts_file'}, $_[0]->{'line'});
}

# list_ipnodes()
# Parse ipnodes from /etc/ipnodes into a data structure
sub list_ipnodes
{
	my $self = shift;

local @rv;
local $lnum = 0;
$self->open_readfile(*HOSTS, $self->{'config_hash'}->{'ipnodes_file'});
while(<HOSTS>) {
	s/\r|\n//g;
	s/#.*$//g;
	s/\s+$//g;
	if (/([0-9a-f:]+|[0-9\.]+)\s+(.*)$/) {
		push(@rv, { 'address' => $1,
			    'ipnodes' => [ split(/\s+/, $2) ],
			    'line', $lnum,
			    'index', scalar(@rv) });
		}
	$lnum++;
	}
close(HOSTS);
return @rv;
}

# create_ipnode(&ipnode)
# Add a new ipnode to /etc/ipnodes
sub create_ipnode
{
	my $self = shift;

$self->open_tempfile(HOSTS, ">>$self->{'config_hash'}->{'ipnodes_file'}");
$self->print_tempfile(HOSTS, $_[0]->{'address'},"\t",join(" ",@{$_[0]->{'ipnodes'}}),"\n");
$self->close_tempfile(HOSTS);
}

# modify_ipnode(&ipnode)
# Update the address and ipnodes of a line in /etc/ipnodes
sub modify_ipnode
{
	my $self = shift;

$self->replace_file_line($self->{'config_hash'}->{'ipnodes_file'},
		   $_[0]->{'line'},
		   $_[0]->{'address'}."\t".join(" ",@{$_[0]->{'ipnodes'}})."\n");
}

# delete_ipnode(&ipnode)
# Delete a ipnode from /etc/ipnodes
sub delete_ipnode
{
	my $self = shift;

$self->replace_file_line($self->{'config_hash'}->{'ipnodes_file'}, $_[0]->{'line'});
}

# parse_hex(hex)
# Convert an address like ff000000 into 255.0.0.0
sub parse_hex
{
	my $self = shift;

$_[0] =~ /(..)(..)(..)(..)$/;
return join(".", (hex($1), hex($2), hex($3), hex($4)));
}

# interfaces_chooser_button(field, multiple, [form])
# Returns HTML for a javascript button for choosing an interface or interfaces
sub interfaces_chooser_button
{
	my $self = shift;

  local $form = @_ > 2 ? $_[2] : 0;
  local $w = $_[1] ? 500 : 300;
  return "<input type=button onClick='ifield = document.forms[$form].$_[0]; chooser = window.open(\"$self->{'gconfig_hash'}->{'webprefix'}/net/interface_chooser.cgi?multi=$_[1]&interface=\"+escape(ifield.value), \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=$w,height=200\"); chooser.ifield = ifield' value=\"...\">\n";
}

# prefix_to_mask(prefix)
# Converts a number like 24 to a mask like 255.255.255.0
sub prefix_to_mask
{
	my $self = shift;

return $_[0] >= 24 ? "255.255.255.".(256-(2 ** (32-$_[0]))) :
       $_[0] >= 16 ? "255.255.".(256-(2 ** (24-$_[0]))).".0" :
       $_[0] >= 16 ? "255.".(256-(2 ** (16-$_[0]))).".0.0" :
		     (256-(2 ** (8-$_[0]))).".0.0.0";
}

# mask_to_prefix(mask)
# Converts a mask like 255.255.255.0 to a prefix like 24
sub mask_to_prefix
{
	my $self = shift;

return $_[0] =~ /^255\.255\.255\.(\d+)$/ ? 32-$self->log2(256-$1) :
       $_[0] =~ /^255\.255\.(\d+)\.0$/ ? 24-$self->log2(256-$1) :
       $_[0] =~ /^255\.(\d+)\.0\.0$/ ? 16-$self->log2(256-$1) :
       $_[0] =~ /^(\d+)\.0\.0\.0$/ ? 8-$self->log2(256-$1) : 32;
}

sub log2
{
	my $self = shift;

return int(log($_[0])/log(2));
}

# module_for_interface(&interface)
# Returns a structure containing details of some other module that manages
# some active interface
sub module_for_interface
{
	my $self = shift;

if ($self->foreign_check("zones") && $_[0]->{'zone'}) {
	# Zones virtual interface
	return { 'module' => 'zones',
		 'desc' => $self->text('mod_zones', $_[0]->{'zone'}) };
	}
if ($self->foreign_check("virtual_server") && $_[0]->{'virtual'} ne '') {
	# Check for a Virtualmin interface
	$self->foreign_require("virtual_server", "virtual-server-lib.pl");
	local ($d) = $main::virtual_server_object->get_domain_by("ip", $_[0]->{'address'});
	if ($d) {
		return { 'module' => 'virtual_server',
			 'desc' => $self->text('mod_virtualmin', $d->{'dom'}) };
		}
	if ($main::virtual_server_object->UNIVERSAL::can("list_resellers")) {
		($self->{'resel'}) = grep { $_->{'acl'}->{'defip'} eq $_[0]->{'address'} }
				$main::virtual_server_object->list_resellers();
		if ($self->{'resel'}) {
			return { 'module' => 'virtual_server',
				 'desc' => $self->text('mod_reseller',
						 $self->{'resel'}->{'name'}) };
			}
		}
	}
return undef if ($_[0]->{'name'} !~ /^ppp/);	# only for PPP
if ($self->foreign_check("ppp-client")) {
	# Dialup PPP connection
	$self->foreign_require("ppp-client", "ppp-client-lib.pl");
	local ($ip, $pid, $sect) = &ppp_client::get_connect_details();
	if ($ip eq $_[0]->{'address'}) {
		return { 'module' => 'ppp-client',
			 'desc' => $self->text('mod_ppp', $sect) };
		}
	}
if ($self->foreign_check("adsl-client")) {
	# ADSL PPP connection
	$self->foreign_require("adsl-client", "adsl-client-lib.pl");
	local ($dev, $ip) = &adsl_client::get_adsl_ip();
	if ("ppp$dev" eq $_[0]->{'fullname'}) {
		return { 'module' => 'adsl-client',
			 'desc' => $self->text('mod_adsl') };
		}
	}
if ($self->foreign_check("pap")) {
	# Dialin PPP connection
	# XXX not handled yet
	}
if ($self->foreign_check("pptp_client")) {
	# PPTP client connection
	$self->foreign_require("pptp_client", "pptp-client-lib.pl");
	local @tunnels = $main::pptp_client_object->list_tunnels();
	local %tunnels = map { $_->{'name'}, 1 } @tunnels;
	local @conns = $main::pptp_client_object->list_connected();
	foreach $c (@conns) {
		if ($c->[2] eq $_[0]->{'fullname'}) {
			return { 'module' => 'pptp_client',
				 'desc' => $self->text('mod_pptpc', "<i>$c->[0]</i>") };
			}
		}
	}
if ($self->foreign_check("pptp_server")) {
	# PPTP server connection
	$self->foreign_require("pptp_server", "pptp-server-lib.pl");
	local @conns = $main::pptp_server_object->list_connections();
	local $c;
	foreach $c (@conns) {
		if ($c->[3] eq $_[0]->{'fullname'} ||
		    $c->[4] eq $_[0]->{'address'}) {
			return { 'module' => 'pptp_server',
				 'desc' => $self->text('mod_pptps', $c->[2]) };
			}
		}
	}
return undef;
}

# can_iface(name)
sub can_iface
{
	my $self = shift;

local $name = ref($_[0]) && $_[0]->{'fullname'} ? $_[0]->{'fullname'} :
	      ref($_[0]) ? $_[0]->{'name'}.
		   ($_[0]->{'virtual'} ne "" ? ":$_[0]->{'virtual'}" : "") :
		   $_[0];
return 0 if ($self->{'access_hash'}->{'ifcs'} == 0 || $self->{'access_hash'}->{'ifcs'} == 1);
return 1 if ($self->{'access_hash'}->{'ifcs'} == 2);
local %can = map { $_, 1 } split(/\s+/, $self->{'access_hash'}->{'interfaces'});
if ($self->{'access_hash'}->{'ifcs'} == 3) {
	return $can{$name};
	}
else {
	return !$can{$name};
	}
}

sub can_create_iface
{
	my $self = shift;

return $self->{'access_hash'}->{'ifcs'} == 2;
}

# interface_choice(name, value, blankmode-text, [disabled?])
# Returns HTML for an interface chooser menu
sub interface_choice
{
	my $self = shift;

local @ifaces = map { $_->{'fullname'} } ($main::net_object->active_interfaces(), $main::net_object->boot_interfaces());
@ifaces = sort { $a cmp $b } $self->unique(@ifaces);
local $rv = "<select name=$_[0] ".($_[3] ? " disabled=true" : "").">\n";
local ($i, $found);
if ($_[2]) {
	$rv .= sprintf "<option value='' %s> %s\n",
			$_[1] eq "" ? "checked" : "", $_[2];
	}
$found++ if ($_[1] eq "");
foreach $i (@ifaces) {
	$rv .= sprintf "<option value=%s %s>%s\n",
		$i, $_[1] eq $i ? "selected" : "", $i;
	$found++ if ($_[1] eq $i);
	}
#$rv .= "<option value=$_[1] selected>$_[1]\n" if (!$found && $_[1]);
$rv .= sprintf "<option value=other %s> %s\n",
		!$found && $_[1] ? "selected" : "", $self->{'text_hash'}->{'chooser_other'};
$rv .= "</select>";
$rv .= sprintf "<input name=$_[0]_other size=6 value='%s' %s>\n",
		!$found ? $_[1] : "", $_[3] ? " disabled=true" : "";
return $rv;
}

# compute_broadcast(ip, netmask)
# Returns a computed broadcast address (ip ^ ~netmask)
sub compute_broadcast
{
	my $self = shift;

local $ipnum = $self->ip_to_integer($_[0]);
local $nmnum = $self->ip_to_integer($_[1]);
return $self->integer_to_ip($ipnum | (~$nmnum));
}

# compute_network(ip, netmask)
# Returns a computed network address (ip & netmask)
sub compute_network
{
	my $self = shift;

local $ipnum = $self->ip_to_integer($_[0]);
local $nmnum = $self->ip_to_integer($_[1]);
return $self->integer_to_ip($ipnum & $nmnum);
}

# ip_to_integer(ip)
# Given an IP address, returns a 32-bit number
sub ip_to_integer
{
	my $self = shift;

local @ip = split(/\./, $_[0]);
return ($ip[0]<<24) + ($ip[1]<<16) + ($ip[2]<<8) + ($ip[3]<<0);
}

# ip_to_integer(integer)
# Given a 32-bit number, converts it to an IP
sub integer_to_ip
{
	my $self = shift;

return sprintf "%d.%d.%d.%d",
		($_[0]>>24)&0xff,
		($_[0]>>16)&0xff,
		($_[0]>>8)&0xff,
		($_[0]>>0)&0xff;
}

# all_interfaces()
# Returns a list of all active and boot-time interfaces
sub all_interfaces
{
	my $self = shift;

local @rv;
foreach my $a ($self->active_interfaces()) {
	$a->{'active'} = 1;
	push(@rv, $a);
	}
foreach my $a ($self->boot_interfaces()) {
	$a->{'boot'} = 1;
	push(@rv, $a);
	}
return @rv;
}


1;

