#!/usr/bin/perl
package net::OpenlinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'net::OpenlinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# open-linux.pl
# Networking functions for openlinux

$self->{'net_scripts_dir'} = "/etc/sysconfig/network-scripts";
$self->{'network_config'} = "/etc/sysconfig/network";
$self->{'static_route_config'} = "/etc/sysconfig/network-scripts/ifcfg-routes";
$self->{'nis_conf'} = "/etc/nis.conf";

&ModuleLoader::preload_module(ref($self), "net::LinuxLib");
$self = new net::LinuxLib:: ref($self), %$self; 

1;

1;
		return $self;
}

# boot_interfaces()
# Returns a list of interfaces brought up at boot time
sub boot_interfaces
{
	my $self = shift;

local(@rv, $f);
opendir(CONF, $self->translate_filename($self->{'net_scripts_dir'}));
while($f = readdir(CONF)) {
	next if ($f !~ /^ifcfg-(\S+)/ || $f eq 'ifcfg-routes' ||
		 $f =~ /\.sample$/);
	local (%conf, $b);
	$self->read_env_file("$self->{'net_scripts_dir'}/$f", \%conf);
	$b->{'fullname'} = $conf{'DEVICE'} ? $conf{'DEVICE'} : $1;
	if ($b->{'fullname'} =~ /(\S+):(\d+)/) {
		$b->{'name'} = $1;
		$b->{'virtual'} = $2;
		}
	else { $b->{'name'} = $b->{'fullname'}; }
	$b->{'up'} = ($conf{'ONBOOT'} eq 'yes');
	$b->{'address'} = $conf{'IPADDR'} ? $conf{'IPADDR'} : "Automatic";
	$b->{'netmask'} = $conf{'NETMASK'} ? $conf{'NETMASK'} : "Automatic";
	$b->{'broadcast'} = $conf{'BROADCAST'} ? $conf{'BROADCAST'}
					       : "Automatic";
	$b->{'dhcp'} = $conf{'DYNAMIC'} eq 'dhcp';
	$b->{'edit'} = ($b->{'name'} !~ /^ppp|plip/);
	$b->{'index'} = scalar(@rv);
	$b->{'file'} = "$self->{'net_scripts_dir'}/$f";
	push(@rv, $b);
	}
closedir(CONF);
return @rv;
}

# save_interface(&details)
# Create or update a boot-time interface
sub save_interface
{
	my $self = shift;

local(%conf);
local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
				       : $_[0]->{'name'};
$self->lock_file("$self->{'net_scripts_dir'}/ifcfg-$name");
$self->read_env_file("$self->{'net_scripts_dir'}/ifcfg-$name", \%conf);
$conf{'DEVICE'} = $name;
if ($_[0]->{'dhcp'}) {
	$conf{'DYNAMIC'} = 'dhcp';
	}
else {
	$conf{'IPADDR'} = $_[0]->{'address'};
	delete($conf{'DYNAMIC'});
	}
local($ip1, $ip2, $ip3, $ip4) = split(/\./, $_[0]->{'address'});
$conf{'NETMASK'} = $_[0]->{'netmask'};
local($nm1, $nm2, $nm3, $nm4) = split(/\./, $_[0]->{'netmask'});
$conf{'NETWORK'} = sprintf "%d.%d.%d.%d",
			($ip1 & int($nm1))&0xff,
			($ip2 & int($nm2))&0xff,
			($ip3 & int($nm3))&0xff,
			($ip4 & int($nm4))&0xff;
$conf{'BROADCAST'} = $_[0]->{'broadcast'};
$conf{'ONBOOT'} = $_[0]->{'up'} ? "yes" : "no";
$self->write_env_file("$self->{'net_scripts_dir'}/ifcfg-$name", \%conf);
$self->unlock_file("$self->{'net_scripts_dir'}/ifcfg-$name");
}

# delete_interface(&details)
# Delete a boot-time interface
sub delete_interface
{
	my $self = shift;

local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
				       : $_[0]->{'name'};
$self->unlink_logged("$self->{'net_scripts_dir'}/ifcfg-$name");
}

# can_edit(what)
# Can some boot-time interface parameter be edited?
sub can_edit
{
	my $self = shift;

return $_[0] ne "bootp" && $_[0] ne "mtu";
}

# valid_boot_address(address)
# Is some address valid for a bootup interface
sub valid_boot_address
{
	my $self = shift;

return $self->check_ipaddress($_[0]);
}

sub get_hostname
{
	my $self = shift;

return $self->get_system_hostname(1);
}

# save_hostname(name)
sub save_hostname
{
	my $self = shift;

local %conf;
$self->system_logged("hostname $_[0] >/dev/null 2>&1");
$self->open_lock_tempfile(HOST, ">/etc/HOSTNAME");
$self->print_tempfile(HOST, $_[0],"\n");
$self->close_tempfile(HOST);
$self->lock_file($self->{'network_config'});
$self->read_file($self->{'network_config'}, \%conf);
$conf{'HOSTNAME'} = $_[0];
$self->write_file($self->{'network_config'}, \%conf);
$self->unlock_file($self->{'network_config'});
undef(@main::get_system_hostname);      # clear cache
}

# get_domainname()
sub get_domainname
{
	my $self = shift;

local $d;
$self->execute_command("domainname", undef, \$d, undef);
chop($d);
return $d;
}

# save_domainname(domain)
sub save_domainname
{
	my $self = shift;

local %conf;
$self->execute_command("domainname ".quotemeta($_[0]));
$self->read_env_file($self->{'network_config'}, \%conf);
if ($_[0]) {
	$conf{'NISDOMAIN'} = $_[0];
	}
else {
	delete($conf{'NISDOMAIN'});
	}
$self->write_env_file($self->{'network_config'}, \%conf);
}

sub routing_config_files
{
	my $self = shift;

return ( $self->{'network_config'},
	 map { $_->{'file'} } $self->boot_interfaces() );
}

sub routing_input
{
	my $self = shift;

local (%conf, %ifc, $f, $gateway, $gatewaydev);
$self->read_file($self->{'network_config'}, \%conf);
local ($gateway, $gatewaydev) = $self->get_default_gateway();

print "<tr> <td><b>$self->{'text_hash'}->{'routes_default'}</b></td> <td>\n";
printf "<input type=radio name=gateway_def value=1 %s> %s\n",
	$gateway ? "" : "checked", $self->{'text_hash'}->{'routes_none'};
printf "<input type=radio name=gateway_def value=0 %s> %s\n",
	$gateway ? "checked" : "", $self->{'text_hash'}->{'routes_gateway'};
printf "<input name=gateway size=15 value=\"%s\"> %s\n",
	$gateway, $self->{'text_hash'}->{'routes_device'};
printf "<input name=gatewaydev size=6 value=\"%s\"></td> </tr>\n",
	$gatewaydev;

print "<tr> <td><b>$self->{'text_hash'}->{'routes_forward'}</b></td> <td>\n";
printf "<input type=radio name=forward value=1 %s> $self->{'text_hash'}->{'yes'}\n",
	$conf{'IPFORWARDING'} =~ /yes|true/i ? "checked" : "";
printf "<input type=radio name=forward value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
	$conf{'IPFORWARDING'} =~ /yes|true/i ? "" : "checked";

print "<tr> <td valign=top><b>$self->{'text_hash'}->{'routes_script'}</b></td> <td>\n";
print "<textarea name=script rows=4 cols=60>\n";
$self->open_readfile(*SCRIPT, $self->{'static_route_config'});
while(<SCRIPT>) { print; }
close(SCRIPT);
print "</textarea></td> </tr>\n";
}

sub parse_routing
{
	my $self = shift;

local %conf;
$self->lock_file($self->{'network_config'});
$self->read_file($self->{'network_config'}, \%conf);
if ($self->{'in_hash'}->{'forward'}) { $conf{'IPFORWARDING'} = 'yes'; }
else { delete($conf{'IPFORWARDING'}); }
local %ifcs = map { $_->{'fullname'}, 1 } $self->all_interfaces();

if (!$self->{'in_hash'}->{'gateway_def'}) {
	gethostbyname($self->{'in_hash'}->{'gateway'}) ||
		$self->error($self->text('routes_edefault', $self->{'in_hash'}->{'gateway'}));
	$ifcs{$self->{'in_hash'}->{'gatewaydev'}} ||
		$self->error($self->text('routes_edevice', $self->{'in_hash'}->{'gatewaydev'}));
	}

$self->set_default_gateway($self->{'in_hash'}->{'gateway_def'} ? ( ) :
			( $self->{'in_hash'}->{'gateway'}, $self->{'in_hash'}->{'gatewaydev'} ) );

$self->write_file($self->{'network_config'}, \%conf);
$self->unlock_file($self->{'network_config'});

$self->open_lock_tempfile(SCRIPT, ">$self->{'static_route_config'}");
$self->{'in_hash'}->{'script'} =~ s/\r//g;
$self->print_tempfile(SCRIPT, $self->{'in_hash'}->{'script'});
$self->close_tempfile(SCRIPT);
$self->system_logged("chmod +x $self->{'static_route_config'}");
}

sub os_feedback_files
{
	my $self = shift;

opendir(DIR, $self->{'net_scripts_dir'});
local @f = readdir(DIR);
closedir(DIR);
return ( (map { "$self->{'net_scripts_dir'}/$_" } grep { /^ifcfg-/ } @f),
	 $self->{'network_config'}, $self->{'static_route_config'}, $self->{'nis_conf'}, "/etc/resolv.conf",
	 "/etc/nsswitch.conf", "/etc/HOSTNAME" );
}

# apply_network()
# Apply the interface and routing settings
sub apply_network
{
	my $self = shift;

$self->system_logged("(cd / ; /etc/rc.d/init.d/network stop ; /etc/rc.d/init.d/network start) >/dev/null 2>&1");
}

# apply_interface(&iface)
# Calls an OS-specific function to make a boot-time interface active
sub apply_interface
{
	my $self = shift;

local $out = $self->backquote_logged("cd / ; ifup '$_[0]->{'fullname'}' 2>&1 </dev/null");
return $? ? $out : undef;
}

# get_default_gateway()
# Returns the default gateway IP (if one is set) and device (if set) boot time
# settings.
sub get_default_gateway
{
	my $self = shift;

$self->read_file($self->{'network_config'}, \%{$self->{'conf_hash'}});
opendir(CONF, $self->translate_filename($self->{'net_scripts_dir'}));
local $f;
while($f = readdir(CONF)) {
	next if ($f !~ /^ifcfg-(\S+)/);
	local %ifc;
	$self->read_file("$self->{'net_scripts_dir'}/$f", \%ifc);
	if ($self->check_ipaddress($ifc{'GATEWAY'})) {
		return ( $ifc{'GATEWAY'}, $ifc{'DEVICE'} );
		}
	}
closedir(CONF);
return ( );
}

# set_default_gateway([gateway, device])
# Sets the default gateway to the given IP accessible via the given device,
# in the boot time settings.
sub set_default_gateway
{
	my $self = shift;

opendir(CONF, $self->translate_filename($self->{'net_scripts_dir'}));
local $f;
while($f = readdir(CONF)) {
	next if ($f !~ /^ifcfg-(\S+)/);
	local %ifc;
	$self->lock_file("$self->{'net_scripts_dir'}/$f");
	$self->read_file("$self->{'net_scripts_dir'}/$f", \%ifc);
	if (!$_[0] || $ifc{'DEVICE'} ne $_[1]) {
		delete($ifc{'GATEWAY'});
		}
	else {
		$ifc{'GATEWAY'} = $_[0];
		}
	$self->write_file("$self->{'net_scripts_dir'}/$f", \%ifc);
	$self->unlock_file("$self->{'net_scripts_dir'}/$f");
	}
closedir(CONF);
}


1;

