#!/usr/bin/perl
package net::RedhatlinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'net::RedhatlinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# redhat-linux-lib.pl
# Networking functions for redhat linux

$self->{'net_scripts_dir'} = "/etc/sysconfig/network-scripts";
$self->{'network_config'} = "/etc/sysconfig/network";
$self->{'static_route_config'} = "/etc/sysconfig/static-routes";
$self->{'sysctl_config'} = "/etc/sysctl.conf";
$self->{'devices_dir'} = "/etc/sysconfig/networking/devices";

# Redhat 7.2+ and Mandrake 9.1+ support separate gateways in each interface file
$self->{'supports_dev_gateway'} = ($self->{'gconfig_hash'}->{'os_type'} eq 'redhat-linux' &&
			 $self->{'gconfig_hash'}->{'os_version'} >= 7.2) ||
			($self->{'gconfig_hash'}->{'os_type'} eq 'mandrake-linux' &&
			 $self->{'gconfig_hash'}->{'os_version'} >= 9.1) ||
			($self->{'gconfig_hash'}->{'os_type'} eq 'coherant-linux' &&
			 $self->{'gconfig_hash'}->{'os_version'} >= 3.0) ||
			($self->{'gconfig_hash'}->{'os_type'} eq 'trustix-linux');

# Redhat 8.0+ and Mandrake 9.1+ have a separate file for static routes for
# each interface
$self->{'supports_dev_routes'} = ($self->{'gconfig_hash'}->{'os_type'} eq 'redhat-linux' &&
		        $self->{'gconfig_hash'}->{'os_version'} >= 8.0) ||
			($self->{'gconfig_hash'}->{'os_type'} eq 'mandrake-linux' &&
			 $self->{'gconfig_hash'}->{'os_version'} >= 9.1) ||
			($self->{'gconfig_hash'}->{'os_type'} eq 'coherant-linux' &&
			 $self->{'gconfig_hash'}->{'os_version'} >= 3.0) ||
			($self->{'gconfig_hash'}->{'os_type'} eq 'trustix-linux');

# Redhat 10 (ES/AS 3) uses route-$dev instead of $dev.route
$self->{'supports_route_dev'} = ($self->{'gconfig_hash'}->{'os_type'} eq 'redhat-linux' &&
		       $self->{'gconfig_hash'}->{'os_version'} >= 10.0) ||
		      ($self->{'gconfig_hash'}->{'os_type'} eq 'coherant-linux' &&
		       $self->{'gconfig_hash'}->{'os_version'} >= 3.0);

# Redhat 9.0+ uses the ONPARENT variable for virtual interfaces
$self->{'uses_on_parent'} = ($self->{'gconfig_hash'}->{'os_type'} eq 'redhat-linux' &&
		   $self->{'gconfig_hash'}->{'os_version'} >= 9.0) ||
		  ($self->{'gconfig_hash'}->{'os_type'} eq 'mandrake-linux' &&
		   $self->{'gconfig_hash'}->{'os_version'} >= 9.1) ||
		  ($self->{'gconfig_hash'}->{'os_type'} eq 'coherant-linux' &&
		   $self->{'gconfig_hash'}->{'os_version'} >= 3.0);

# Redhat versions 7.2 and above allow the MTU to be set at boot time
$self->{'supports_mtu'} = ($self->{'gconfig_hash'}->{'os_type'} eq 'redhat-linux' &&
		 $self->{'gconfig_hash'}->{'os_version'} >= 7.2) ||
		($self->{'gconfig_hash'}->{'os_type'} eq 'coherant-linux' &&
 		 $self->{'gconfig_hash'}->{'os_version'} >= 3.0);

&ModuleLoader::preload_module(ref($self), "net::LinuxLib");
$self = new net::LinuxLib:: ref($self), %$self; 

# unapply_interface(&iface)
# Calls an OS-specific function to make a boot-time interface inactive
#sub unapply_interface
#{
# $out = &backquote_logged("cd / ; ifdown '$_[0]->{'fullname'}' 2>&1 </dev/null");
#return $? ? $out : undef;
#}

1;

1;
		return $self;
}

# boot_interfaces()
# Returns a list of interfaces brought up at boot time
sub boot_interfaces
{
	my $self = shift;

local(@rv, $f);
opendir(CONF, $self->translate_filename($self->{'net_scripts_dir'}));
while($f = readdir(CONF)) {
	local (%conf, $b);
	if ($f =~ /^ifcfg-([a-z0-9:\.]+)\-range([a-z0-9\.\_]+)$/) {
		# A range of addresses
		$self->read_env_file("$self->{'net_scripts_dir'}/$f", \%conf);
		$b->{'fullname'} = "$1-range$2";
		$b->{'name'} = $1;
		$b->{'range'} = $2;
		$b->{'start'} = $conf{'IPADDR_START'};
		$b->{'end'} = $conf{'IPADDR_END'};
		$b->{'num'} = $conf{'CLONENUM_START'};
		$b->{'up'} = 1;
		$b->{'edit'} = 1;
		$b->{'index'} = scalar(@rv);
		$b->{'file'} = "$self->{'net_scripts_dir'}/$f";
		push(@rv, $b);
		}
	elsif ($f =~ /^ifcfg-[a-z0-9:\.]+$/ && $f !~ /\.(bak|old)$/i) {
		# Normal interface
		$self->read_env_file("$self->{'net_scripts_dir'}/$f", \%conf);
		$b->{'fullname'} = $conf{'DEVICE'};
		if ($b->{'fullname'} =~ /(\S+):(\d+)/) {
			$b->{'name'} = $1;
			$b->{'virtual'} = $2;
			}
		else { $b->{'name'} = $b->{'fullname'}; }
		$b->{'up'} = defined($conf{'ONPARENT'}) && $b->{'virtual'} ne '' ?
				($conf{'ONPARENT'} eq 'yes') :
				($conf{'ONBOOT'} eq 'yes');
		$b->{'address'} = $conf{'IPADDR'};
		$b->{'netmask'} = $conf{'NETMASK'};
		$b->{'broadcast'} = $conf{'BROADCAST'};
		$b->{'gateway'} = $conf{'GATEWAY'};
		$b->{'mtu'} = $conf{'MTU'};
		$b->{'dhcp'} = ($conf{'BOOTPROTO'} eq 'dhcp');
		$b->{'bootp'} = ($conf{'BOOTPROTO'} eq 'bootp');
		$b->{'edit'} = ($b->{'name'} !~ /^ppp|irlan/);
		$b->{'index'} = scalar(@rv);
		$b->{'file'} = "$self->{'net_scripts_dir'}/$f";
		push(@rv, $b);
		}
	}
closedir(CONF);
return @rv;
}

# save_interface(&details)
# Create or update a boot-time interface
sub save_interface
{
	my $self = shift;

local(%conf);
local $name = $_[0]->{'range'} ne "" ? $_[0]->{'name'}."-range".$_[0]->{'range'} :
	      $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
				       : $_[0]->{'name'};
$self->lock_file("$self->{'net_scripts_dir'}/ifcfg-$name");
$self->read_env_file("$self->{'net_scripts_dir'}/ifcfg-$name", \%conf);
if ($_[0]->{'range'} ne "") {
	# Special case - saving a range
	$conf{'IPADDR_START'} = $_[0]->{'start'};
	$conf{'IPADDR_END'} = $_[0]->{'end'};
	$conf{'CLONENUM_START'} = $_[0]->{'num'};
	}
else {
	# Saving a normal interface
	$conf{'DEVICE'} = $name;
	$conf{'IPADDR'} = $_[0]->{'address'};
	local($ip1, $ip2, $ip3, $ip4) = split(/\./, $_[0]->{'address'});
	$conf{'NETMASK'} = $_[0]->{'netmask'};
	local($nm1, $nm2, $nm3, $nm4) = split(/\./, $_[0]->{'netmask'});
	if ($_[0]->{'address'} && $_[0]->{'netmask'}) {
		$conf{'NETWORK'} = sprintf "%d.%d.%d.%d",
					($ip1 & int($nm1))&0xff,
					($ip2 & int($nm2))&0xff,
					($ip3 & int($nm3))&0xff,
					($ip4 & int($nm4))&0xff;
		}
	else {
		$conf{'NETWORK'} = '';
		}
	$conf{'BROADCAST'} = $_[0]->{'broadcast'};
	if ($_[0]->{'gateway'}) {
		$conf{'GATEWAY'} = $_[0]->{'gateway'};
		}
	else {
		delete($conf{'GATEWAY'});
		}
	$conf{'MTU'} = $_[0]->{'mtu'};
	$conf{'ONBOOT'} = $_[0]->{'up'} ? "yes" : "no";
	$conf{'ONPARENT'} = $_[0]->{'up'} ? "yes" : "no" if ($_[0]->{'virtual'} ne '');
	$conf{'BOOTPROTO'} = $_[0]->{'bootp'} ? "bootp" :
			     $_[0]->{'dhcp'} ? "dhcp" : "none";
	}
$self->write_env_file("$self->{'net_scripts_dir'}/ifcfg-$name", \%conf);
if (-d $self->translate_filename($self->{'devices_dir'})) {
	$self->link_file("$self->{'net_scripts_dir'}/ifcfg-$name",
		   "$self->{'devices_dir'}/ifcfg-$name");
	}
$self->unlock_file("$self->{'net_scripts_dir'}/ifcfg-$name");
}

# delete_interface(&details)
# Delete a boot-time interface
sub delete_interface
{
	my $self = shift;

local $name = $_[0]->{'range'} ne "" ? $_[0]->{'name'}."-range".$_[0]->{'range'} :
	      $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
				       : $_[0]->{'name'};
$self->lock_file("$self->{'net_scripts_dir'}/ifcfg-$name");
$self->unlink_file("$self->{'net_scripts_dir'}/ifcfg-$name");
if (-d $self->translate_filename($self->{'devices_dir'})) {
	$self->unlink_file("$self->{'devices_dir'}/ifcfg-$name");
	}
$self->unlock_file("$self->{'net_scripts_dir'}/ifcfg-$name");
}

# can_edit(what)
# Can some boot-time interface parameter be edited?
sub can_edit
{
	my $self = shift;

if ($self->{'supports_mtu'}) {
	return 1;
	}
else {
	return $_[0] ne "mtu";
	}
}

# valid_boot_address(address)
# Is some address valid for a bootup interface
sub valid_boot_address
{
	my $self = shift;

return $self->check_ipaddress($_[0]);
}

# get_hostname()
sub get_hostname
{
	my $self = shift;

return $self->get_system_hostname(1);
}

# save_hostname(name)
sub save_hostname
{
	my $self = shift;

local $old = $self->get_hostname();
local %conf;
$self->system_logged("hostname $_[0] >/dev/null 2>&1");
$self->open_lock_tempfile(HOST, ">/etc/HOSTNAME");
$self->print_tempfile(HOST, $_[0],"\n");
$self->close_tempfile(HOST);
$self->lock_file($self->{'network_config'});
$self->read_env_file($self->{'network_config'}, \%conf);
$conf{'HOSTNAME'} = $_[0];
$self->write_env_file($self->{'network_config'}, \%conf);
$self->unlock_file($self->{'network_config'});

# If any ifcfg-XXX files have the old hostname in DHCP_HOSTNAME, fix it
foreach my $b ($self->boot_interfaces()) {
	local %ifc;
	$self->read_env_file($b->{'file'}, \%ifc);
	if ($ifc{'DHCP_HOSTNAME'} eq $old) {
		$ifc{'DHCP_HOSTNAME'} = $_[0];
		$self->lock_file($b->{'file'});
		$self->write_env_file($b->{'file'}, \%ifc);
		$self->unlock_file($b->{'file'});
		}
	}
undef(@main::get_system_hostname);	# clear cache
}

# get_domainname()
sub get_domainname
{
	my $self = shift;

local $d;
$self->execute_command("domainname", undef, \$d, undef);
chop($d);
return $d;
}

# save_domainname(domain)
sub save_domainname
{
	my $self = shift;

local %conf;
$self->execute_command("domainname ".quotemeta($_[0]));
$self->read_env_file($self->{'network_config'}, \%conf);
if ($_[0]) {
	$conf{'NISDOMAIN'} = $_[0];
	}
else {
	delete($conf{'NISDOMAIN'});
	}
$self->write_env_file($self->{'network_config'}, \%conf);
}

sub routing_config_files
{
	my $self = shift;

local @rv = ( $self->{'network_config'}, $self->{'sysctl_config'} );
if (!$self->{'supports_dev_routes'}) {
	push(@rv, $self->{'static_route_config'});
	}
else {
	local $f;
	opendir(DIR, $self->translate_filename($self->{'devices_dir'}));
	while($f = readdir(DIR)) {
		if ($f =~ /^([a-z]+\d*(\.\d+)?(:\d+)?)\.route$/ ||
		    $f =~ /^route\-([a-z]+\d*(\.\d+)?(:\d+)?)$/) {
			push(@rv, "$self->{'devices_dir'}/$f");
			}
		}
	closedir(DIR);
	}
return @rv;
}

sub routing_input
{
	my $self = shift;

local (%conf, @st, @hr, %sysctl);
$self->read_env_file($self->{'network_config'}, \%conf);
if (!$self->{'supports_dev_gateway'}) {
	# show default router and device
	print "<tr> <td><b>$self->{'text_hash'}->{'routes_default'}</b></td> <td>\n";
	printf "<input type=radio name=gateway_def value=1 %s> $self->{'text_hash'}->{'routes_none'}\n",
		$conf{'GATEWAY'} ? "" : "checked";
	printf "<input type=radio name=gateway_def value=0 %s>\n",
		$conf{'GATEWAY'} ? "checked" : "";
	printf "<input name=gateway size=15 value=\"%s\"></td> </tr>\n",
		$conf{'GATEWAY'};

	print "<tr> <td><b>$self->{'text_hash'}->{'routes_device2'}</b></td> <td>\n";
	printf "<input type=radio name=gatewaydev_def value=1 %s> $self->{'text_hash'}->{'routes_none'}\n",
		$conf{'GATEWAYDEV'} ? "" : "checked";
	printf "<input type=radio name=gatewaydev_def value=0 %s>\n",
		$conf{'GATEWAYDEV'} ? "checked" : "";
	printf "<input name=gatewaydev size=6 value=\"%s\"></td> </tr>\n",
		$conf{'GATEWAYDEV'};
	}
else {
	# multiple default routers can exist!
	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'routes_default2'}</b></td>\n";
	print "<td><table border>\n";
	print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'routes_ifc'}</b></td> ",
	      "<td><b>$self->{'text_hash'}->{'routes_gateway'}</b></td> </tr>\n";
	local $r = 0;
	if ($conf{'GATEWAY'}) {
		print "<tr $self->{'cb'}>\n";
		print "<td>",$self->interface_sel("gatewaydev$r",
				    $conf{'GATEWAYDEV'} || "*"),"</td>\n";
		printf "<td><input name=gateway$r size=15 value='%s'></td>\n",
			$conf{'GATEWAY'};
		print "</tr>\n";
		$r++;
		}
	local @boot = $self->boot_interfaces();
	foreach $b (grep { $_->{'gateway'} && $_->{'virtual'} eq '' } @boot) {
		print "<tr $self->{'cb'}>\n";
		print "<td>",$self->interface_sel("gatewaydev$r",
				    $b->{'name'}),"</td>\n";
		printf "<td><input name=gateway$r size=15 value='%s'></td>\n",
			$b->{'gateway'};
		print "</tr>\n";
		$r++;
		}
	print "<tr $self->{'cb'}>\n";
	print "<td>",$self->interface_sel("gatewaydev$r"),"</td>\n";
	print "<td><input name=gateway$r size=15></td>\n";
	print "</tr>\n";
	print "</table></td> </tr>\n";
	}

# show routing
if ($self->{'gconfig_hash'}->{'os_version'} < 7.0) {
	print "<tr> <td><b>$self->{'text_hash'}->{'routes_forward'}</b></td> <td>\n";
	printf "<input type=radio name=forward value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		$conf{'FORWARD_IPV4'} eq "yes" ? "checked" : "";
	printf "<input type=radio name=forward value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		$conf{'FORWARD_IPV4'} eq "yes" ? "" : "checked";
	}
else {
	$self->read_env_file($self->{'sysctl_config'}, \%sysctl);
	print "<tr> <td><b>$self->{'text_hash'}->{'routes_forward'}</b></td> <td>\n";
	printf "<input type=radio name=forward value=1 %s> $self->{'text_hash'}->{'yes'}\n",
		$sysctl{'net.ipv4.ip_forward'} ? "checked" : "";
	printf "<input type=radio name=forward value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
		$sysctl{'net.ipv4.ip_forward'} ? "" : "checked";
	}

if (!$self->{'supports_dev_routes'}) {
	# get static routes from single file
	$self->open_readfile(*STATIC, $self->{'static_route_config'});
	while(<STATIC>) {
		if (/(\S+)\s+net\s+(\S+)\s+netmask\s+(\S+)\s+gw\s+(\S+)/) {
			push(@st, [ $1, $2, $3, $4 ]);
			}
		elsif (/(\S+)\s+host\s+(\S+)\s+gw\s+(\S+)/) {
			push(@st, [ $1, $2, '255.255.255.255', $3 ]);
			}
		elsif (/(\S+)\s+net\s+(\S+)\s+netmask\s+(\S+)/) {
			push(@hr, [ $1, $2, $3 ]);
			}
		elsif (/(\S+)\s+host\s+(\S+)/) {
			push(@hr, [ $1, $2, '255.255.255.255' ]);
			}
		}
	close(STATIC);
	}
else {
	# get static routes from per-interface files
	local $f;
	opendir(DIR, $self->translate_filename($self->{'devices_dir'}));
	while($f = readdir(DIR)) {
		if ($f =~ /^([a-z]+\d*(\.\d+)?(:\d+)?)\.route$/ ||
		    $f =~ /^route\-([a-z]+\d*(\.\d+)?(:\d+)?)$/) {
			local $dev = $1;
			local (%rfile, $i);
			$self->read_env_file("$self->{'devices_dir'}/$f", \%rfile);
			for($i=0; defined($rfile{"ADDRESS$i"}); $i++) {
				if ($rfile{"GATEWAY$i"}) {
					push(@st, [ $dev, $rfile{"ADDRESS$i"},
							  $rfile{"NETMASK$i"},
							  $rfile{"GATEWAY$i"} ]);
					}
				else {
					push(@hr, [ $dev, $rfile{"ADDRESS$i"},
							  $rfile{"NETMASK$i"} ||
							  "255.255.255.255" ]);
					}
				}
			}
		}
	closedir(DIR);
	}

# show static network routes
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'routes_static'}</b></td>\n";
print "<td><table border>\n";
print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'routes_ifc'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_net'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_mask'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_gateway'}</b></td> </tr>\n";
for($i=0; $i<=@st; $i++) {
	local $st = $st[$i];
	print "<tr $self->{'cb'}>\n";
	print "<td><input name=dev_$i size=6 value='$st->[0]'></td>\n";
	print "<td><input name=net_$i size=15 value='$st->[1]'></td>\n";
	print "<td><input name=netmask_$i size=15 value='$st->[2]'></td>\n";
	print "<td><input name=gw_$i size=15 value='$st->[3]'></td>\n";
	print "</tr>\n";
	}
print "</table></td> </tr>\n";

# Show static host routes
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'routes_local'}</b></td>\n";
print "<td><table border>\n";
print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'routes_ifc'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_net'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_mask'}</b></td> </tr>\n";
for($i=0; $i<=@hr; $i++) {
	local $st = $hr[$i];
	print "<tr $self->{'cb'}>\n";
	print "<td><input name=ldev_$i size=6 value='$st->[0]'></td>\n";
	print "<td><input name=lnet_$i size=15 value='$st->[1]'></td>\n";
	print "<td><input name=lnetmask_$i size=15 value='$st->[2]'></td>\n";
	print "</tr>\n";
	}
print "</table></td> </tr>\n";
}

sub parse_routing
{
	my $self = shift;

local (%conf, @st, %sysctl, %st, @boot);
$self->lock_file($self->{'network_config'});
$self->read_env_file($self->{'network_config'}, \%conf);
if (!$self->{'supports_dev_gateway'}) {
	# Just update a single file
	if ($self->{'in_hash'}->{'gateway_def'}) { delete($conf{'GATEWAY'}); }
	elsif (!gethostbyname($self->{'in_hash'}->{'gateway'})) {
		$self->error($self->text('routes_edefault', $self->{'in_hash'}->{'gateway'}));
		}
	else { $conf{'GATEWAY'} = $self->{'in_hash'}->{'gateway'}; }

	if ($self->{'in_hash'}->{'gatewaydev_def'}) { delete($conf{'GATEWAYDEV'}); }
	elsif ($self->{'in_hash'}->{'gatewaydev'} !~ /^\S+$/) {
		$self->error($self->text('routes_edevice', $self->{'in_hash'}->{'gatewaydev'}));
		}
	else { $conf{'GATEWAYDEV'} = $self->{'in_hash'}->{'gatewaydev'}; }
	}
else {
	# Multiple defaults can be specified!
	local ($r, $b);
	@boot = grep { $->{'virtual'} eq '' } $self->boot_interfaces();
	foreach $b (@boot) {
		delete($b->{'gateway'});
		}
	delete($conf{'GATEWAY'});
	delete($conf{'GATEWAYDEV'});

	for($r=0; defined($self->{'in_hash'}->{"gatewaydev$r"}); $r++) {
		next if (!$self->{'in_hash'}->{"gatewaydev$r"});
		$self->check_ipaddress($self->{'in_hash'}->{"gateway$r"}) ||
			$self->error($self->text('routes_edefault2', $r+1));
		if ($self->{'in_hash'}->{"gatewaydev$r"} eq "*") {
			# For any interface
			$conf{'GATEWAY'} && $self->error($self->text('routes_eclash'));
			$conf{'GATEWAY'} = $self->{'in_hash'}->{"gateway$r"};
			}
		else {
			# For a specific interface
			local ($b) = grep { $_->{'fullname'} eq $self->{'in_hash'}->{"gatewaydev$r"} } @boot;
			$b->{'gateway'} && $self->error($self->text('routes_eclash2',
							$self->{'in_hash'}->{"gatewaydev$r"}));
			$b->{'gateway'} = $self->{'in_hash'}->{"gateway$r"};
			}
		}
	}

if ($self->{'gconfig_hash'}->{'os_version'} < 7.0) {
	if ($self->{'in_hash'}->{'forward'}) { $conf{'FORWARD_IPV4'} = 'yes'; }
	else { $conf{'FORWARD_IPV4'} = 'no'; }
	}
else {
	$self->lock_file($self->{'sysctl_config'});
	$self->read_env_file($self->{'sysctl_config'}, \%sysctl);
	$sysctl{'net.ipv4.ip_forward'} = $self->{'in_hash'}->{'forward'};
	}

# Parse static and local routes
for($i=0; defined($dev = $self->{'in_hash'}->{"dev_$i"}); $i++) {
	next if (!$dev);
	$self->{'net'} = $self->{'in_hash'}->{"net_$i"}; $self->{'netmask'} = $self->{'in_hash'}->{"netmask_$i"}; $self->{'gw'} = $self->{'in_hash'}->{"gw_$i"};
	$dev =~ /^\S+$/ || $self->error($self->text('routes_edevice', $dev));
	gethostbyname($self->{'net'}) || $self->error($self->text('routes_enet', $self->{'net'}));
	$self->check_ipaddress($self->{'netmask'}) || $self->error($self->text('routes_emask', $self->{'netmask'}));
	gethostbyname($self->{'gw'}) || $self->error($self->text('routes_egateway', $self->{'gw'}));
	if ($self->{'netmask'} eq "255.255.255.255") {
		push(@st, "$dev host $self->{'net'} gw $self->{'gw'}\n");
		}
	else {
		push(@st, "$dev net $self->{'net'} netmask $self->{'netmask'} gw $self->{'gw'}\n");
		}
	push(@{$st{$dev}}, [ $self->{'net'}, $self->{'netmask'}, $self->{'gw'} ]);
	}
for($i=0; defined($dev = $self->{'in_hash'}->{"ldev_$i"}); $i++) {
	$self->{'net'} = $self->{'in_hash'}->{"lnet_$i"}; $self->{'netmask'} = $self->{'in_hash'}->{"lnetmask_$i"};
	next if (!$dev && !$self->{'net'});
	$dev =~ /^\S+$/ || $self->error($self->text('routes_edevice', $dev));
	gethostbyname($self->{'net'}) || $self->{'net'} =~ /^(\S+)\/(\d+)$/ && gethostbyname($1) ||
		$self->error($self->text('routes_enet', $self->{'net'}));
	$self->check_ipaddress($self->{'netmask'}) || $self->error($self->text('routes_emask', $self->{'netmask'}));
	if ($self->{'netmask'} eq "255.255.255.255") {
		push(@st, "$dev host $self->{'net'}\n");
		}
	else {
		push(@st, "$dev net $self->{'net'} netmask $self->{'netmask'}\n");
		}
	push(@{$st{$dev}}, [ $self->{'net'}, $self->{'netmask'} ]);
	}
if (!$self->{'supports_dev_routes'}) {
	# Write to a single file
	$self->open_lock_tempfile(STATIC, ">$self->{'static_route_config'}");
	$self->print_tempfile(STATIC, @st);
	$self->close_tempfile(STATIC);
	}
else {
	# Write to one file per interface (delete old, then save new/updated)
	local $f;
	opendir(DIR, $self->translate_filename($self->{'devices_dir'}));
	while($f = readdir(DIR)) {
		if (($f =~ /^([a-z]+\d*(\.\d+)?(:\d+)?)\.route$/ ||
		     $f =~ /^route\-([a-z]+\d*(\.\d+)?(:\d+)?)$/) && !$st{$1}) {
			$self->unlink_logged("$self->{'devices_dir'}/$f");
			$self->unlink_logged("$self->{'net_scripts_dir'}/$f");
			}
		}
	closedir(DIR);
	foreach $dev (keys %st) {
		$f = $self->{'supports_route_dev'} ? "route-$dev" : "$dev.route";
		local (%rfile, $i);
		for($i=0; $i<@{$st{$dev}}; $i++) {
			$rfile{"ADDRESS$i"} = $st{$dev}->[$i]->[0];
			$rfile{"NETMASK$i"} = $st{$dev}->[$i]->[1];
			$rfile{"GATEWAY$i"} = $st{$dev}->[$i]->[2];
			}
		$self->lock_file("$self->{'devices_dir'}/$f");
		$self->write_env_file("$self->{'devices_dir'}/$f", \%rfile);
		$self->unlock_file("$self->{'devices_dir'}/$f");
		$self->lock_file("$self->{'net_scripts_dir'}/$f");
		$self->link_file("$self->{'devices_dir'}/$f", "$self->{'net_scripts_dir'}/$f");
		$self->unlock_file("$self->{'net_scripts_dir'}/$f");
		}
	}
$self->write_env_file($self->{'network_config'}, \%conf);
$self->unlock_file($self->{'network_config'});
if (%sysctl) {
	$self->write_env_file($self->{'sysctl_config'}, \%sysctl);
	$self->unlock_file($self->{'sysctl_config'});
	}
if (@boot) {
	local $b;
	foreach $b (@boot) {
		$self->save_interface($b);
		}
	}
}

sub os_feedback_files
{
	my $self = shift;

opendir(DIR, $self->{'net_scripts_dir'});
local @f = readdir(DIR);
closedir(DIR);
return ( (map { "$self->{'net_scripts_dir'}/$_" } grep { /^ifcfg-/ } @f),
	 $self->{'network_config'}, $self->{'static_route_config'}, "/etc/resolv.conf",
	 "/etc/nsswitch.conf", "/etc/HOSTNAME" );
}

# interface_sel(name, value)
# Returns a menu for all boot-time interfaces
sub interface_sel
{
	my $self = shift;

local $rv = "<select name=$_[0]>\n";
$rv .= sprintf "<option value='' %s>&nbsp;\n",
	$_[1] eq "" ? "selected" : "";
$rv .= sprintf "<option value='*' %s>%s\n",
	$_[1] eq "*" ? "selected" : "", $self->{'text_hash'}->{'routes_any'};
@{$self->{'boot_interfaces_cache_array'}} = $self->boot_interfaces() if (!@{$self->{'boot_interfaces_cache_array'}});
foreach $b (@{$self->{'boot_interfaces_cache_array'}}) {
	next if ($b->{'virtual'} ne '');
	$rv .= sprintf "<option value=%s %s>%s\n",
	    $b->{'name'}, $b->{'name'} eq $_[1] ? "selected" : "", $b->{'name'};
	}
$rv .= "</select>\n";
return $rv;
}

# apply_network()
# Apply the interface and routing settings
sub apply_network
{
	my $self = shift;

$self->system_logged("(cd / ; /etc/init.d/network stop ; /etc/init.d/network start) >/dev/null 2>&1");
}

# apply_interface(&iface)
# Calls an OS-specific function to make a boot-time interface active
sub apply_interface
{
	my $self = shift;

local $out = $self->backquote_logged("cd / ; ifdown '$_[0]->{'fullname'}' >/dev/null 2>&1 </dev/null ; ifup '$_[0]->{'fullname'}' 2>&1 </dev/null");
return $? || $out =~ /error/i ? $out : undef;
}

# get_default_gateway()
# Returns the default gateway IP (if one is set) and device (if set) boot time
# settings.
sub get_default_gateway
{
	my $self = shift;

local %conf;
$self->read_env_file($self->{'network_config'}, \%conf);
local @boot = $self->boot_interfaces();
local ($gifc) = grep { $_->{'gateway'} && $_->{'virtual'} eq '' } @boot;
return ( $gifc->{'gateway'}, $gifc->{'fullname'} ) if ($gifc);

return $conf{'GATEWAY'} ? ( $conf{'GATEWAY'}, $conf{'GATEWAYDEV'} ) : ( );
}

# set_default_gateway(gateway, device)
# Sets the default gateway to the given IP accessible via the given device,
# in the boot time settings.
sub set_default_gateway
{
	my $self = shift;

$self->lock_file($self->{'network_config'});
$self->read_env_file($self->{'network_config'}, \%conf);
if (!$self->{'supports_dev_gateway'}) {
	# Just update the network config file
	local %conf;
	if ($_[0]) {
		$conf{'GATEWAY'} = $_[0];
		$conf{'GATEWAYDEV'} = $_[1];
		}
	else {
		delete($conf{'GATEWAY'});
		delete($conf{'GATEWAYDEV'});
		}
	}
else {
	# Set the gateway in the specified interface file, and clear the rest
	local @boot = grep { $->{'virtual'} eq '' } $self->boot_interfaces();
	foreach $b (@boot) {
		delete($b->{'gateway'});
		if ($_[0] && $b->{'fullname'} eq $_[1]) {
			$b->{'gateway'} = $_[0];
			$self->save_interface($b);
			}
		}
	delete($conf{'GATEWAY'});
	delete($conf{'GATEWAYDEV'});
	}
$self->write_env_file($self->{'network_config'}, \%conf);
$self->unlock_file($self->{'network_config'});
}

# supports_ranges()
# Returns 1 for newer redhat versions
sub supports_ranges
{
	my $self = shift;

return ($self->{'gconfig_hash'}->{'os_type'} eq 'redhat-linux' &&
	$self->{'gconfig_hash'}->{'os_version'} >= 7.3) ||
       ($self->{'gconfig_hash'}->{'os_type'} eq 'mandrake-linux' &&
	$self->{'gconfig_hash'}->{'os_version'} >= 8.0) ||
       ($self->{'gconfig_hash'}->{'os_type'} eq 'coherant-linux' &&
	$self->{'gconfig_hash'}->{'os_version'} >= 3.0);
}

# range_input([&interface])
# Print HTML for a IP range interface
sub range_input
{
	my $self = shift;

local $new = !$_[0];

# Base interface
print "<tr> <td><b>$self->{'text_hash'}->{'range_iface'}</b></td>\n";
if ($new) {
	print "<td><select name=iface>\n";
	local $b;
	foreach $b ($self->boot_interfaces()) {
		if ($b->{'virtual'} eq '') {
			printf "<option %s>%s\n",
			 $b->{'fullname'} eq $_[0]->{'name'} ?  "selected" : "",
			 $b->{'fullname'};
			}
		}
	print "</select></td>\n";
	}
else {
	print "<td><tt>$_[0]->{'name'}</tt></td>\n";
	}

# Name for this range
print "<td><b>$self->{'text_hash'}->{'range_name'}</b></td>\n";
if ($new) {
	print "<td><input name=range size=10></td> </tr>\n";
	}
else {
	print "<td><tt>$_[0]->{'range'}</tt></td> </tr>\n";
	}

# Start
print "<tr> <td><b>$self->{'text_hash'}->{'range_start'}</b></td>\n";
printf "<td><input name=start size=15 value='%s'></td>\n",
	$_[0]->{'start'};

# Stop
print "<td><b>$self->{'text_hash'}->{'range_end'}</b></td>\n";
printf "<td><input name=end size=15 value='%s'></td> </tr>\n",
	$_[0]->{'end'};

# Base number
print "<tr> <td><b>$self->{'text_hash'}->{'range_num'}</b></td>\n";
printf "<td><input name=num size=5 value='%s'></td> </tr>\n",
	$_[0]->{'num'};
}

# parse_range(&range, &in)
sub parse_range
{
	my $self = shift;

local %in = %{$_[1]};
if ($in{'new'}) {
	$_[0]->{'name'} = $in{'iface'};
	$in{'range'} =~ /^[a-z0-9\.\_]+$/ || $self->error($self->{'text_hash'}->{'range_ename'});
	$_[0]->{'range'} = $in{'range'};
	$_[0]->{'fullname'} = $in{'iface'}."-range".$in{'range'};
	}

$self->check_ipaddress($in{'start'}) || $self->error($self->{'text_hash'}->{'range_estart'});
$_[0]->{'start'} = $in{'start'};

$self->check_ipaddress($in{'end'}) || $self->error($self->{'text_hash'}->{'range_eend'});
$_[0]->{'end'} = $in{'end'};

local @sip = split(/\./, $in{'start'});
local @eip = split(/\./, $in{'end'});
$sip[0] == $eip[0] && $sip[1] == $eip[1] && $sip[2] == $eip[2] ||
	$self->error($self->{'text_hash'}->{'range_eclass'});
$sip[3] <= $eip[3] || $self->error($self->{'text_hash'}->{'range_ebefore'});

$in{'num'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'range_enum'});
$_[0]->{'num'} = $in{'num'};
}

# get_dhcp_hostname()
# Returns 0 if the hostname is not set by DHCP, 1 if it is, or -1 if this
# feature is not supported on this OS.
sub get_dhcp_hostname
{
	my $self = shift;

return -1 if ($self->{'gconfig_hash'}->{'os_type'} ne 'redhat-linux' ||
	      $self->{'gconfig_hash'}->{'os_version'} < 11);
local @boot = $self->boot_interfaces();
local ($eth) = grep { $_->{'fullname'} =~ /^eth\d+$/ } @boot;
return -1 if (!$eth);
local %eth;
$self->read_env_file($eth->{'file'}, \%eth);
return $eth{'DHCP_HOSTNAME'} ne $self->get_system_hostname();
}

# save_dhcp_hostname(set)
# If called with a parameter of 0, the hostname is fixed and not set by
# DHCP. If called with 1, the hostname is chosen by DHCP.
sub save_dhcp_hostname
{
	my $self = shift;

}


1;

