#!/usr/bin/perl
package net::SlackwarelinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'net::SlackwarelinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# slackware-linux-lib.pl
# Networking functions for slackware linux
# To support boot-time interfaces, ifconfig commands are added to rc.local so
# that additional virtual interfaces can be created

&ModuleLoader::preload_module(ref($self), "net::LinuxLib");
$self = new net::LinuxLib:: ref($self), %$self; 
%{$self->{'iconfig_hash'}} = $self->foreign_config("init");
$self->{'interfaces_file'} = $self->{'iconfig_hash'}->{'local_script'} || $self->{'iconfig_hash'}->{'extra_init'};
$self->{'rc_init'} = "/etc/rc.d/rc.inet1";
$self->{'dhcp_init'} = "/etc/rc.d/rc.dhcpd";

1;

1;
		return $self;
}

# boot_interfaces()
# Returns a list of interfaces brought up at boot time
sub boot_interfaces
{
	my $self = shift;

local @rv;

# Look in rc.init1 file for master interface
local $iface = { 'up' => 1,
		 'edit' => 1,
		 'index' => 0,
		 'init' => 1,
		 'name' => 'eth0',
		 'fullname' => 'eth0',
		 'file' => $self->{'rc_init'} };
local $gotdevice;
$self->open_readfile(*INIT, $self->{'rc_init'});
while(<INIT>) {
	s/\r|\n//g;
	s/#.*$//;
	if (/^\s*IPADDR\s*=\s*["']?([0-9\.]+)/) {
		$iface->{'address'} = $1;
		}
	elsif (/^\s*DEVICE\s*=\s*["']?([0-9\.]+)/) {
		$iface->{'name'} = $iface->{'fullname'} = $1;
		$gotdevice++;
		}
	elsif (/^\s*NETMASK\s*=\s*["']?([0-9\.]+)/) {
		$iface->{'netmask'} = $1;
		}
	elsif (/^\s*BROADCAST\s*=\s*["']?([0-9\.]+)/) {
		$iface->{'broadcast'} = $1;
		}
	elsif (/^\s*DHCP\s*=\s*["']?([0-9\.]+)/) {
		$iface->{'dhcp'} = ($1 eq "yes");
		}
	elsif (/^\s*ifconfig\s+(\S+)\s+.*IPADDR.*/ && !$gotdevice) {
		$iface->{'name'} = $iface->{'fullname'} = $1;
		}
	}
close(INIT);
local @st1 = stat($self->{'rc_init'});
local @st2 = stat($self->{'dhcp_init'});
if ($st1[7] == $st2[7]) {
	# Looks like rc.dhcpd script has been copied to rc.inet1 - assume DHCP
	$iface->{'dhcp'} = 1;
	}
push(@rv, $iface) if ($iface->{'address'} || $iface->{'dhcp'});

# Read extra init script for virtual interfaces
local $lnum = 0;
$self->open_readfile(*IFACES, $self->{'interfaces_file'});
while(<IFACES>) {
	s/\r|\n//g;
	if (/^(#*)\s*(\S*ifconfig)\s+(\S+)\s+(\S+)(\s+netmask\s+(\S+))?(\s+broadcast\s+(\S+))?(\s+mtu\s+(\d+))?\s+up$/) {
		# Found a usable interface line
		local $b = { 'fullname' => $3,
			     'up' => !$1,
			     'address' => $4,
			     'netmask' => $6,
			     'broadcast' => $8,
			     'mtu' => $10,
			     'edit' => 1,
			     'line' => $lnum,
			     'file' => $self->{'interfaces_file'},
			     'index' => scalar(@rv) };
		if ($b->{'fullname'} =~ /(\S+):(\d+)/) {
			$b->{'name'} = $1;
			$b->{'virtual'} = $2;
			}
		else {
			$b->{'name'} = $b->{'fullname'};
			}
		push(@rv, $b);
		}
	$lnum++;
	}
close(IFACES);
return @rv;
}

# save_interface(&details)
# Create or update a boot-time interface's ifconfig command
sub save_interface
{
	my $self = shift;

if ($_[0]->{'index'} == 0 && $_[0]->{'fullname'} eq 'eth0') {
	# Modifying the primary interface
	$self->lock_file($self->{'rc_init'});
	if ($_[0]->{'dhcp'} && -r $self->{'dhcp_init'}) {
		# Just copy rc.dhcpd to rc.inet1
		$self->system_logged("cp $self->{'dhcp_init'} $self->{'rc_init'}");
		}
	else {
		# Is the current file rc.dhcpd?
		if (!$_[0]->{'dhcp'}) {
			local @st1 = stat($self->{'rc_init'});
			local @st2 = stat($self->{'dhcp_init'});
			if ($st1[7] == $st2[7]) {
				# Yes! Use built-in static IP version
				$self->system_logged("cp $self->{'module_root_directory'}/rc.inet1 $self->{'rc_init'}");
				}
			}

		# Update init script with new settings
		local $lref = $self->read_file_lines($self->{'rc_init'});
		foreach $l (@$lref) {
			if ($l =~ /^(\s*)IPADDR\s*=\s*(\S+)(.*)/) {
				$l = $1."IPADDR=\"".$_[0]->{'address'}."\"".$3;
				}
			elsif ($l =~ /^(\s*)NETMASK\s*=\s*(\S+)(.*)/) {
				$l = $1."NETMASK=\"".$_[0]->{'netmask'}."\"".$3;
				}
			elsif ($l =~ /^(\s*)BROADCAST\s*=\s*(\S+)(.*)/) {
				$l = $1."BROADCAST=\"".$_[0]->{'broadcast'}."\"".$3;
				}
			if ($l =~ /^(\s*)DHCP\s*=\s*(\S+)(.*)/) {
				$l = $1."DHCP=\"".($_[0]->{'dhcp'} ? "yes" : "no")."\"".$3;
				}
			}
		$self->flush_file_lines();
		}
	$self->unlock_file($self->{'rc_init'});
	}
else {
	# Modifying or adding some other interface
	$_[0]->{'dhcp'} && $self->error($self->{'text_hash'}->{'bifc_edhcpmain'});
	$self->lock_file($self->{'interfaces_file'});
	local $lref = $self->read_file_lines($self->{'interfaces_file'});
	local $lnum = defined($_[0]->{'line'}) ? $_[0]->{'line'}
					       : $self->interface_lnum($_[0]);
	if (defined($lnum)) {
		$lref->[$lnum] = $self->interface_line($_[0]);
		}
	else {
		push(@$lref, $self->interface_line($_[0]));
		}
	$self->flush_file_lines();
	$self->unlock_file($self->{'interfaces_file'});
	}
}

# delete_interface(&details)
# Delete a boot-time interface's ifconfig command
sub delete_interface
{
	my $self = shift;

if ($_[0]->{'init'}) {
	$self->error("The primary network interface cannot be deleted");
	}
else {
	$self->lock_file($self->{'interfaces_file'});
	local $lref = $self->read_file_lines($self->{'interfaces_file'});
	local $lnum = defined($_[0]->{'line'}) ? $_[0]->{'line'}
					       : $self->interface_lnum($_[0]);
	if (defined($lnum)) {
		splice(@$lref, $lnum, 1);
		}
	$self->flush_file_lines();
	$self->unlock_file($self->{'interfaces_file'});
	}
}

sub interface_lnum
{
	my $self = shift;

local @boot = $self->boot_interfaces();
local ($found) = grep { $_->{'fullname'} eq $_[0]->{'fullname'} } @boot;
return $found ? $found->{'line'} : undef;
}

sub interface_line
{
	my $self = shift;

local $str;
$str .= "# " if (!$_[0]->{'up'});
$str .= $self->has_command("ifconfig");
if (!$_[0]->{'fullname'}) {
	$_[0]->{'fullname'} = $_[0]->{'virtual'} ne "" ?
		$_[0]->{'name'}.":".$_[0]->{'virtual'} : $_[0]->{'name'};
	}
$str .= " $_[0]->{'fullname'} $_[0]->{'address'}";
if ($_[0]->{'netmask'}) {
	$str .= " netmask $_[0]->{'netmask'}";
	}
if ($_[0]->{'broadcast'}) {
	$str .= " broadcast $_[0]->{'broadcast'}";
	}
if ($_[0]->{'mtu'}) {
	$str .= " mtu $_[0]->{'mtu'}";
	}
$str .= " up";
return $str;
}

# can_edit(what)
# Can some boot-time interface parameter be edited?
sub can_edit
{
	my $self = shift;

return $_[0] ne "bootp";
}

# valid_boot_address(address)
# Is some address valid for a bootup interface
sub valid_boot_address
{
	my $self = shift;

return $self->check_ipaddress($_[0]);
}

# get_hostname()
sub get_hostname
{
	my $self = shift;

return $self->get_system_hostname(1);
}

# save_hostname(name)
sub save_hostname
{
	my $self = shift;

$self->system_logged("hostname $_[0] >/dev/null 2>&1");
$self->open_lock_tempfile(HOST, ">/etc/HOSTNAME");
$self->print_tempfile(HOST, $_[0],"\n");
$self->close_tempfile(HOST);
undef(@main::get_system_hostname);      # clear cache
}

sub routing_config_files
{
	my $self = shift;

return ( $self->{'rc_init'} );
}

sub routing_input
{
	my $self = shift;

$self->open_readfile(*INIT, $self->{'rc_init'});
while(<INIT>) {
	s/\r|\n//g;
        s/#.*$//;
	if (/^\s*GATEWAY\s*=\s*["']?([0-9\.]+)/) {
		$self->{'gw'} = $1;
		}
	}
close(INIT);
print "<tr> <td><b>$self->{'text_hash'}->{'routes_default'}</b></td> <td>\n";
printf "<input type=radio name=default value=1 %s> %s\n",
	$self->{'gw'} ? "" : "checked", $self->{'text_hash'}->{'routes_none'};
printf "<input type=radio name=default value=0 %s> %s\n",
	$self->{'gw'} ? "checked" : "", $self->{'text_hash'}->{'routes_gateway'};
printf "<input name=gw size=20 value='%s'></td> </tr>\n", $self->{'gw'};
}

sub parse_routing
{
	my $self = shift;

local $gw = "";
if (!$self->{'in_hash'}->{'default'}) {
	$self->check_ipaddress($self->{'in_hash'}->{'gw'}) ||
		$self->error($self->text('routes_edefault', $self->{'in_hash'}->{'gw'}));
	$gw = $self->{'in_hash'}->{'gw'};
	}
$self->lock_file($self->{'rc_init'});
local $lref = $self->read_file_lines($self->{'rc_init'});
foreach $l (@$lref) {
	if ($l =~ /^(\s*)GATEWAY\s*=\s*(\S+)(.*)/) {
		$l = $1."GATEWAY=\"".$gw."\"".$3;
		}
	}
$self->flush_file_lines();
$self->unlock_file($self->{'rc_init'});
}


1;

