#!/usr/bin/perl
package net::Suselinux80Lib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'net::Suselinux80Lib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# united-linux-lib.pl
# Networking functions for united linux

$self->{'net_scripts_dir'} = "/etc/sysconfig/network";
$self->{'routes_config'} = "/etc/sysconfig/network/routes";
$self->{'sysctl_config'} = "/etc/sysconfig/sysctl";

&ModuleLoader::preload_module(ref($self), "net::LinuxLib");
$self = new net::LinuxLib:: ref($self), %$self; 

1;

1;
		return $self;
}

# boot_interfaces()
# Returns a list of interfaces brought up at boot time
sub boot_interfaces
{
	my $self = shift;

local(@rv, $f);
opendir(CONF, $self->translate_filename($self->{'net_scripts_dir'}));
while($f = readdir(CONF)) {
	next if ($f !~ /^ifcfg-([a-z0-9:\.]+)$/);
	local (%conf, $b);
	$b->{'fullname'} = $1;
	$self->read_env_file("$self->{'net_scripts_dir'}/$f", \%conf);
	if ($b->{'fullname'} =~ /(\S+):(\d+)/) {
		$b->{'name'} = $1;
		$b->{'virtual'} = $2;
		}
	else { $b->{'name'} = $b->{'fullname'}; }
	$b->{'up'} = ($conf{'STARTMODE'} eq 'onboot');
	local $pfx;
	if ($conf{'IPADDR'} =~ /^(\S+)\/(\d+)$/) {
		$b->{'address'} = $1;
		$pfx = $2;
		}
	else {
		$b->{'address'} = $conf{'IPADDR'};
		}
	$pfx = $conf{'PREFIXLEN'} if (!$pfx);
	if ($pfx) {
		$b->{'netmask'} = $self->prefix_to_mask($pfx);
		}
	else {
		$b->{'netmask'} = $conf{'NETMASK'};
		}
	$b->{'broadcast'} = $conf{'BROADCAST'};
	$b->{'dhcp'} = ($conf{'BOOTPROTO'} eq 'dhcp');
	$b->{'edit'} = ($b->{'name'} !~ /^ppp|irlan/);
	$b->{'index'} = scalar(@rv);
	$b->{'file'} = "$self->{'net_scripts_dir'}/$f";
	push(@rv, $b);
	}
closedir(CONF);
return @rv;
}

# save_interface(&details)
# Create or update a boot-time interface
sub save_interface
{
	my $self = shift;

local(%conf);
local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
				       : $_[0]->{'name'};
$self->lock_file("$self->{'net_scripts_dir'}/ifcfg-$name");
$self->read_env_file("$self->{'net_scripts_dir'}/ifcfg-$name", \%conf);
$conf{'IPADDR'} = $_[0]->{'address'};
local($ip1, $ip2, $ip3, $ip4) = split(/\./, $_[0]->{'address'});
$conf{'NETMASK'} = $_[0]->{'netmask'};
local($nm1, $nm2, $nm3, $nm4) = split(/\./, $_[0]->{'netmask'});
if ($_[0]->{'address'} && $_[0]->{'netmask'}) {
	$conf{'NETWORK'} = sprintf "%d.%d.%d.%d",
				($ip1 & int($nm1))&0xff,
				($ip2 & int($nm2))&0xff,
				($ip3 & int($nm3))&0xff,
				($ip4 & int($nm4))&0xff;
	}
else {
	$conf{'NETWORK'} = '';
	}
delete($conf{'PREFIXLEN'});
$conf{'BROADCAST'} = $_[0]->{'broadcast'};
$conf{'STARTMODE'} = $_[0]->{'up'} ? "onboot" :
		     $conf{'STARTMODE'} eq "onboot" ? "manual" :
						      $conf{'STARTMODE'};
$conf{'BOOTPROTO'} = $_[0]->{'dhcp'} ? "dhcp" : "static";
$conf{'UNIQUE'} = time();
$self->write_env_file("$self->{'net_scripts_dir'}/ifcfg-$name", \%conf);
$self->unlock_file("$self->{'net_scripts_dir'}/ifcfg-$name");
}

# delete_interface(&details)
# Delete a boot-time interface
sub delete_interface
{
	my $self = shift;

local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
				       : $_[0]->{'name'};
$self->unlink_logged("$self->{'net_scripts_dir'}/ifcfg-$name");
}

# can_edit(what)
# Can some boot-time interface parameter be edited?
sub can_edit
{
	my $self = shift;

return $_[0] ne "mtu" && $_[0] ne "bootp";
}

# valid_boot_address(address)
# Is some address valid for a bootup interface
sub valid_boot_address
{
	my $self = shift;

return $self->check_ipaddress($_[0]);
}

# get_hostname()
sub get_hostname
{
	my $self = shift;

return $self->get_system_hostname(1);
}

# save_hostname(name)
sub save_hostname
{
	my $self = shift;

local %conf;
$self->system_logged("hostname $_[0] >/dev/null 2>&1");
$self->open_lock_tempfile(HOST, ">/etc/HOSTNAME");
$self->print_tempfile(HOST, $_[0],"\n");
$self->close_tempfile(HOST);
$self->lock_file($self->{'network_config'});
$self->read_env_file($self->{'network_config'}, \%conf);
$conf{'HOSTNAME'} = $_[0];
$self->write_env_file($self->{'network_config'}, \%conf);
$self->unlock_file($self->{'network_config'});
undef(@main::get_system_hostname);      # clear cache
}

# get_domainname()
sub get_domainname
{
	my $self = shift;

local $d;
$self->execute_command("domainname", undef, \$d, undef);
chop($d);
return $d;
}

# save_domainname(domain)
sub save_domainname
{
	my $self = shift;

local %conf;
$self->execute_command("domainname ".quotemeta($_[0]));
$self->read_env_file($self->{'network_config'}, \%conf);
if ($_[0]) {
	$conf{'NISDOMAIN'} = $_[0];
	}
else {
	delete($conf{'NISDOMAIN'});
	}
$self->write_env_file($self->{'network_config'}, \%conf);
}

sub routing_config_files
{
	my $self = shift;

return ( $self->{'routes_config'}, $self->{'sysctl_config'} );
}

sub routing_input
{
	my $self = shift;

local (@routes, $i);
$self->open_readfile(*ROUTES, $self->{'routes_config'});
while(<ROUTES>) {
	s/#.*$//;
	s/\r|\n//g;
	local @r = map { $_ eq '-' ? undef : $_ } split(/\s+/, $_);
	push(@routes, \@r) if (@r);
	}
close(ROUTES);

# show default router and device
local ($def) = grep { $_->[0] eq "default" } @routes;
print "<tr> <td><b>$self->{'text_hash'}->{'routes_default'}</b></td> <td>\n";
printf "<input type=radio name=gateway_def value=1 %s> $self->{'text_hash'}->{'routes_none'}\n",
	$def->[1] ? "" : "checked";
printf "<input type=radio name=gateway_def value=0 %s>\n",
	$def->[1] ? "checked" : "";
printf "<input name=gateway size=15 value=\"%s\"></td> </tr>\n",
	$def->[1];

print "<tr> <td><b>$self->{'text_hash'}->{'routes_device2'}</b></td> <td>\n";
printf "<input type=radio name=gatewaydev_def value=1 %s> $self->{'text_hash'}->{'routes_none'}\n",
	$def->[3] ? "" : "checked";
printf "<input type=radio name=gatewaydev_def value=0 %s>\n",
	$def->[3] ? "checked" : "";
printf "<input name=gatewaydev size=6 value=\"%s\"></td> </tr>\n",
	$def->[3];

$self->read_env_file($self->{'sysctl_config'}, \%{$self->{'sysctl_hash'}});
print "<tr> <td><b>$self->{'text_hash'}->{'routes_forward'}</b></td> <td>\n";
printf "<input type=radio name=forward value=1 %s> $self->{'text_hash'}->{'yes'}\n",
	$self->{'sysctl_hash'}->{'IP_FORWARD'} eq 'yes' ? "checked" : "";
printf "<input type=radio name=forward value=0 %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
	$self->{'sysctl_hash'}->{'IP_FORWARD'} eq 'yes' ? "" : "checked";

# show static network routes
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'routes_static'}</b></td>\n";
print "<td><table border>\n";
print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'routes_ifc'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_net'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_mask'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_gateway'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_type'}</b></td> </tr>\n";
local ($r, $i = 0);
foreach $r (@routes, [ ]) {
	next if ($r eq $def);
	print "<tr $self->{'cb'}>\n";
	print "<td><input name=dev_$i size=6 value='$r->[3]'></td>\n";
	print "<td><input name=net_$i size=15 value='$r->[0]'></td>\n";
	print "<td><input name=netmask_$i size=15 value='$r->[2]'></td>\n";
	print "<td><input name=gw_$i size=15 value='$r->[1]'></td>\n";
	print "<td><input name=type_$i size=10 value='$r->[4]'></td>\n";
	print "</tr>\n";
	$i++;
	}
print "</table></td> </tr>\n";
}

sub parse_routing
{
	my $self = shift;

# Parse route inputs
local (@routes, $r, $i);
if (!$self->{'in_hash'}->{'gateway_def'}) {
	gethostbyname($self->{'in_hash'}->{'gateway'}) ||
		$self->error($self->text('routes_edefault', $self->{'in_hash'}->{'gateway'}));
	local @def = ( "default", $self->{'in_hash'}->{'gateway'}, undef, undef );
	if (!$self->{'in_hash'}->{'gatewaydev_def'}) {
		$self->{'in_hash'}->{'gatewaydev'} =~ /^\S+$/ ||
			$self->error($self->text('routes_edevice', $self->{'in_hash'}->{'gatewaydev'}));
		$def[3] = $self->{'in_hash'}->{'gatewaydev'};
		}
	push(@routes, \@def);
	}
for($i=0; defined($self->{'in_hash'}->{"dev_$i"}); $i++) {
	next if (!$self->{'in_hash'}->{"net_$i"});
	$self->check_ipaddress($self->{'in_hash'}->{"net_$i"}) ||
		$self->{'in_hash'}->{"net_$i"} =~ /^(\S+)\/(\d+)$/ && $self->check_ipaddress($1) ||
		$self->error($self->text('routes_enet', $self->{'in_hash'}->{"net_$i"}));
	$self->{'in_hash'}->{"dev_$i"} =~ /^\S*$/ || $self->error($self->text('routes_edevice', $self->{'dev'}));
	!$self->{'in_hash'}->{"netmask_$i"} || $self->check_ipaddress($self->{'in_hash'}->{"netmask_$i"}) ||
		$self->error($self->text('routes_emask', $self->{'in_hash'}->{"netmask_$i"}));
	!$self->{'in_hash'}->{"gw_$i"} || $self->check_ipaddress($self->{'in_hash'}->{"gw_$i"}) ||
		$self->error($self->text('routes_egateway', $self->{'in_hash'}->{"gw_$i"}));
	$self->{'in_hash'}->{"type_$i"} =~ /^\S*$/ ||
		$self->error($self->text('routes_etype', $self->{'in_hash'}->{"type_$i"}));
	push(@routes, [ $self->{'in_hash'}->{"net_$i"}, $self->{'in_hash'}->{"gw_$i"}, $self->{'in_hash'}->{"netmask_$i"},
			$self->{'in_hash'}->{"dev_$i"}, $self->{'in_hash'}->{"type_$i"} ] );
	}

# Save routes and routing option
$self->open_tempfile(ROUTES, ">$self->{'routes_config'}");
foreach $r (@routes) {
	$self->print_tempfile(ROUTES,join(" ", map { $_ eq '' ? "-" : $_ } @$r),"\n");
	}
$self->close_tempfile(ROUTES);
local $lref = $self->read_file_lines($self->{'sysctl_config'});
for($i=0; $i<@$lref; $i++) {
	if ($lref->[$i] =~ /^\s*IP_FORWARD\s*=/) {
		$lref->[$i] = "IP_FORWARD=".($self->{'in_hash'}->{'forward'} ? "yes" : "no");
		}
	}
$self->flush_file_lines();
}

sub os_feedback_files
{
	my $self = shift;

opendir(DIR, $self->{'net_scripts_dir'});
local @f = readdir(DIR);
closedir(DIR);
return ( (map { "$self->{'net_scripts_dir'}/$_" } grep { /^ifcfg-/ } @f),
	 $self->{'network_config'}, $self->{'static_route_config'}, "/etc/resolv.conf",
	 "/etc/nsswitch.conf", "/etc/HOSTNAME" );
}

# apply_network()
# Apply the interface and routing settings
sub apply_network
{
	my $self = shift;

$self->system_logged("(cd / ; /etc/init.d/network stop ; /etc/init.d/network start) >/dev/null 2>&1");
}


1;

