#!/usr/bin/perl
package net::SuselinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'net::SuselinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# suse-linux-lib.pl
# Networking functions for SuSE linux

$self->{'rc_config'} = "/etc/rc.config";
$self->{'route_conf'} = "/etc/route.conf";

$self->{'use_suse_dns'} = 1;
&ModuleLoader::preload_module(ref($self), "net::LinuxLib");
$self = new net::LinuxLib:: ref($self), %$self; 

1;

1;
		return $self;
}

# boot_interfaces()
# Returns a list of interfaces brought up at boot time
sub boot_interfaces
{
	my $self = shift;

local $rc = $self->parse_rc_config();
local @rv;
push(@rv, { 'fullname' => 'lo',
	    'name' => 'lo',
	    'up' => $rc->{'START_LOOPBACK'}->{'value'} =~ /yes/,
	    'address' => '127.0.0.1',
	    'netmask' => '255.0.0.0',
	    'broadcast' => '127.255.255.255',
	    'edit' => 0,
	    'file' => $self->{'rc_config'},
	    'index' => scalar(@rv) });
local @nc = split(/\s+/, $rc->{'NETCONFIG'}->{'value'});
foreach $nc (@nc) {
	local $ip = $rc->{"IPADDR$nc"}->{'value'};
	local $dev = $rc->{"NETDEV$nc"}->{'value'};
	local $conf = $rc->{"IFCONFIG$nc"}->{'value'};
	if ($dev) {
		local $b;
		$b->{'fullname'} = $dev;
		if ($b->{'fullname'} =~ /(\S+):(\d+)/) {
			$b->{'name'} = $1;
			$b->{'virtual'} = $2;
			}
		else { $b->{'name'} = $b->{'fullname'}; }
		if ($conf =~ /^([0-9\.]+)/) {
			$b->{'address'} = $1;
			if ($conf =~ /broadcast\s+(\S+)/) {
				$b->{'broadcast'} = $1;
				}
			if ($conf =~ /netmask\s+(\S+)/) {
				$b->{'netmask'} = $1;
				}
			if ($conf =~ /\s+up/ || $self->{'gconfig_hash'}->{'os_version'} >= 7.1) {
				$b->{'up'} = 1;
				}
			}
		elsif ($conf =~ /bootp/) {
			$b->{'bootp'} = 1;
			$b->{'netmask'} = 'Automatic';
			$b->{'broadcast'} = 'Automatic';
			$b->{'up'}++;
			}
		elsif ($conf =~ /dhcpclient/) {
			$b->{'dhcp'} = 1;
			$b->{'netmask'} = 'Automatic';
			$b->{'broadcast'} = 'Automatic';
			$b->{'up'}++;
			}
		$b->{'edit'} = 1;
		$b->{'index'} = scalar(@rv);
		$b->{'nc'} = $nc;
		$b->{'file'} => $self->{'rc_config'},
		push(@rv, $b);
		}
	}
return @rv;
}

# save_interface(&details)
# Create or update a boot-time interface
sub save_interface
{
	my $self = shift;

# Find existing interface, if any
$self->lock_file($self->{'rc_config'});
local $rc = $self->parse_rc_config();
local @boot = $self->boot_interfaces();
local ($o, $old, $found);
foreach $o (@boot) {
	if ($o->{'name'} eq $_[0]->{'name'} &&
	    $o->{'virtual'} eq $_[0]->{'virtual'}) {
		$found++; $old = $o;
		last;
		}
	}

# build interface detail strings
local $fullname = $_[0]->{'name'};
$fullname .= ":".$_[0]->{'virtual'} if (defined($_[0]->{'virtual'}));
local $ifconfig;
if ($_[0]->{'bootp'}) {
	$ifconfig = 'bootp';
	}
elsif ($_[0]->{'dhcp'}) {
	$ifconfig = 'dhcpclient';
	}
else {
	$ifconfig = $_[0]->{'address'};
	$ifconfig .= " broadcast $_[0]->{'broadcast'}"
		if ($_[0]->{'broadcast'});
	$ifconfig .= " netmask $_[0]->{'netmask'}"
		if ($_[0]->{'netmask'});
	$ifconfig .= " up" if ($_[0]->{'up'});
	}

if ($found) {
	# Updating an existing interface
	local $nnc = $old->{'nc'};
	$self->save_rc_config($rc, "IPADDR$nnc", $_[0]->{'address'});
	$self->save_rc_config($rc, "NETDEV$nnc", $fullname);
	$self->save_rc_config($rc, "IFCONFIG$nnc", $ifconfig);
	}
else {
	# Adding a new interface
	local @nc = split(/\s+/, $rc->{'NETCONFIG'}->{'value'});
	local $nnc = $nc[@nc-1] =~ /_(\d+)/ ? "_".($1+1) : "_0";
	$self->save_rc_config($rc, "NETCONFIG", join(" ", @nc, $nnc));
	$self->save_rc_config($rc, "IPADDR$nnc", $_[0]->{'address'});
	$self->save_rc_config($rc, "NETDEV$nnc", $fullname);
	$self->save_rc_config($rc, "IFCONFIG$nnc", $ifconfig);
	}
$self->unlock_file($self->{'rc_config'});
}

# delete_interface(&details)
# Delete a boot-time interface
sub delete_interface
{
	my $self = shift;

$self->lock_file($self->{'rc_config'});
local $rc = $self->parse_rc_config();
local @boot = $self->boot_interfaces();
local $old;
foreach $old (@boot) {
	if ($old->{'name'} eq $_[0]->{'name'} &&
	    $old->{'virtual'} eq $_[0]->{'virtual'}) {
		# found it .. remove
		local $nnc = $old->{'nc'};
		local @nc = split(/\s+/, $rc->{'NETCONFIG'}->{'value'});
		@nc = grep { $_ ne $nnc } @nc;
		$self->save_rc_config($rc, "NETCONFIG", join(" ", @nc));
		$self->save_rc_config($rc, "IPADDR$nnc", "");
		$self->save_rc_config($rc, "NETDEV$nnc", "");
		$self->save_rc_config($rc, "IFCONFIG$nnc", "");
		}
	}
$self->unlock_file($self->{'rc_config'});
}

# can_edit(what, &details)
# Can some boot-time interface parameter be edited?
sub can_edit
{
	my $self = shift;

if ($self->{'gconfig_hash'}->{'os_version'} >= 7.1) {
	if ($_[1] && ($_[1]->{'bootp'} || $_[1]->{'dhcp'})) {
		return $_[0] ne "mtu" && $_[0] ne "netmask" &&
		       $_[0] ne "broadcast" && $_[0] ne "up";
		}
	return $_[0] ne "mtu" && $_[0] ne "up";
	}
else {
	if ($_[1] && $_[1]->{'bootp'}) {
		return $_[0] ne "mtu" && $_[0] ne "dhcp" &&
		       $_[0] ne "netmask" && $_[0] ne "broadcast" &&
		       $_[0] ne "up";
		}
	return $_[0] ne "mtu" && $_[0] ne "dhcp";
	}
}

# valid_boot_address(address)
# Is some address valid for a bootup interface
sub valid_boot_address
{
	my $self = shift;

return $self->check_ipaddress($_[0]);
}

# get_hostname()
sub get_hostname
{
	my $self = shift;

local $rc = $self->parse_rc_config();
return $rc->{'FQHOSTNAME'}->{'value'};
}

# save_hostname(name)
sub save_hostname
{
	my $self = shift;

$self->system_logged("hostname $_[0] >/dev/null 2>&1");
$self->lock_file($self->{'rc_config'});
local $rc = $self->parse_rc_config();
$self->save_rc_config($rc, "FQHOSTNAME", $_[0]);
$self->unlock_file($self->{'rc_config'});

# run SuSEconfig, as this function is called last
$self->system_logged("SuSEconfig -quick >/dev/null 2>&1");

undef(@main::get_system_hostname);      # clear cache
}

sub routing_config_files
{
	my $self = shift;

return ( $self->{'route_conf'}, $self->{'rc_config'} );
}

sub routing_input
{
	my $self = shift;

# read route.conf
local ($default, @sr, @lr);
$self->open_readfile(*ROUTE, $self->{'route_conf'});
while(<ROUTE>) {
	s/\r|\n//g;
	s/#.*$//g;
	local @r = split(/\s+/, $_);
	if ($r[0] eq 'default' || $r[0] eq '0.0.0.0') { $default = $r[1]; }
	elsif (@r == 4) { push(@lr, \@r); }
	elsif (@r == 3) { push(@sr, \@r); }
	}
close(ROUTE);

# input for routing
local $rc = $self->parse_rc_config();
local $ipf = $rc->{'IP_FORWARD'}->{'value'};
print "<tr> <td><b>$self->{'text_hash'}->{'routes_forward'}</b></td> <td>\n";
printf "<input type=radio name=forward value=yes %s> $self->{'text_hash'}->{'yes'}\n",
	$ipf eq "yes" ? "checked" : "";
printf "<input type=radio name=forward value=no %s> $self->{'text_hash'}->{'no'}</td> </tr>\n",
	$ipf eq "yes" ? "" : "checked";

# input for default route
print "<tr> <td><b>$self->{'text_hash'}->{'routes_default'}</b></td> <td>\n";
printf "<input type=radio name=default_def value=1 %s> $self->{'text_hash'}->{'routes_none'}\n",
	$default ? "" : "checked";
printf "<input type=radio name=default_def value=0 %s>\n",
	$default ? "checked" : "";
printf "<input name=default size=15 value=\"%s\"></td> </tr>\n",
	$default;

# table for local routes
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'routes_local'}</b></td>\n";
print "<td><table border>\n";
print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'routes_ifc'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_net'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_mask'}</b></td> </tr>\n";
for($i=0; $i<=@lr; $i++) {
	local $lr = $lr[$i];
	print "<tr $self->{'cb'}>\n";
	print "<td><input name=lr_dev_$i size=6 value='$lr->[3]'></td>\n";
	print "<td><input name=lr_net_$i size=15 value='$lr->[0]'></td>\n";
	print "<td><input name=lr_mask_$i size=15 value='$lr->[2]'></td>\n";
	print "</tr>\n";
	}
print "</table></td> </tr>\n";

# table for static routes
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'routes_static'}</b></td>\n";
print "<td><table border>\n";
print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'routes_net'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_gateway'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'routes_mask'}</b></td> </tr>\n";
for($i=0; $i<=@sr; $i++) {
	local $sr = $sr[$i];
	print "<tr $self->{'cb'}>\n";
	print "<td><input name=sr_net_$i size=15 value='$sr->[0]'></td>\n";
	print "<td><input name=sr_gw_$i size=15 value='$sr->[1]'></td>\n";
	print "<td><input name=sr_mask_$i size=15 value='$sr->[2]'></td>\n";
	print "</tr>\n";
	}
print "</table></td> </tr>\n";

}

sub parse_routing
{
	my $self = shift;

$self->lock_file($self->{'rc_config'});
local $rc = $self->parse_rc_config();
$self->save_rc_config($rc, IP_FORWARD, $self->{'in_hash'}->{'forward'});
$self->unlock_file($self->{'rc_config'});
$self->lock_file($self->{'route_conf'});
local $route = "# Generated by Webmin\n";
for($i=0; defined($self->{'dev'} = $self->{'in_hash'}->{"lr_dev_$i"}); $i++) {
	$self->{'net'} = $self->{'in_hash'}->{"lr_net_$i"}; $self->{'mask'} = $self->{'in_hash'}->{"lr_mask_$i"};
	next if (!$self->{'dev'} && !$self->{'net'} && !$self->{'mask'});
	gethostbyname($self->{'net'}) ||
		$self->error($self->text('routes_enet', $self->{'net'}));
	$self->check_ipaddress($self->{'mask'}) ||
		$self->error($self->text('routes_emask', $self->{'mask'}));
	$route .= "$self->{'net'}\t\t0.0.0.0\t\t$self->{'mask'}\t\t$self->{'dev'}\n";
	}
for($i=0; defined($self->{'gw'} = $self->{'in_hash'}->{"sr_gw_$i"}); $i++) {
	$self->{'net'} = $self->{'in_hash'}->{"sr_net_$i"}; $self->{'mask'} = $self->{'in_hash'}->{"sr_mask_$i"};
	next if (!$self->{'gw'} && !$self->{'net'} && !$self->{'mask'});
	gethostbyname($self->{'gw'}) ||
		$self->error($self->text('routes_egateway', $self->{'gw'}));
	gethostbyname($self->{'net'}) ||
		$self->error($self->text('routes_enet', $self->{'net'}));
	$self->check_ipaddress($self->{'mask'}) ||
		$self->error($self->text('routes_emask', $self->{'mask'}));
	$route .= "$self->{'net'}\t\t$self->{'gw'}\t\t$self->{'mask'}\n";
	}
if (!$self->{'in_hash'}->{'default_def'}) {
	gethostbyname($self->{'in_hash'}->{'default'}) ||
		$self->error($self->text('routes_edefault', $self->{'in_hash'}->{'default'}));
	$route .= "default\t\t$self->{'in_hash'}->{'default'}\n";
	}
$self->open_tempfile(ROUTE, ">$self->{'route_conf'}");
$self->print_tempfile(ROUTE, $route);
$self->close_tempfile(ROUTE);
$self->unlock_file($self->{'route_conf'});
}

# parse_rc_config()
sub parse_rc_config
{
	my $self = shift;

local $rc;
local $lnum = 0;
$self->open_readfile(*CONF, $self->{'rc_config'});
while(<CONF>) {
	s/\r|\n//g;
	s/#.*$//g;
	if (/([^=\s]+)="(.*)"/) {
		$rc->{$1} = { 'value' => $2,
			      'line' => $lnum };
		}
	elsif (/([^=\s]+)=(\S+)/) {
		$rc->{$1} = { 'value' => $2,
			      'line' => $lnum };
		}
	$lnum++;
	}
close(CONF);
return $rc;
}

# save_rc_config(&config, directive, value)
sub save_rc_config
{
	my $self = shift;

local $old = $_[0]->{$_[1]};
local $line = "$_[1]=\"$_[2]\"\n";
if ($old) {
	$self->replace_file_line($self->{'rc_config'}, $old->{'line'}, $line);
	}
else {
	$self->open_tempfile(RC, ">>$self->{'rc_config'}");
	$self->print_tempfile(RC, $line);
	$self->close_tempfile(RC);
	}
}

sub os_feedback_files
{
	my $self = shift;

return ( $self->{'rc_config'}, $self->{'route_conf'}, "/etc/resolv.conf", "/etc/nsswitch.conf" );
}


1;

