#!/usr/bin/perl
package net::UnixwareLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'net::UnixwareLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# unixware-lib.pl
# Networking functions for UnixWare

$self->{'max_dns_servers'} = 3;

1;

1;
		return $self;
}

# active_interfaces()
# Returns a list of currently ifconfig'd interfaces
sub active_interfaces
{
	my $self = shift;

local(@rv, @lines, $l);
$self->open_execute_command(*IFC, "ifconfig -a", 1, 1);
while(<IFC>) {
	s/\r|\n//g;
	if (/^\S+:/) { push(@lines, $_); }
	else { $lines[$#lines] .= $_; }
	}
close(IFC);
foreach $l (@lines) {
	local %ifc;
	$l =~ /^([^:\s]+):/; $ifc{'name'} = $1;
	$l =~ /^(\S+):/; $ifc{'fullname'} = $1;
	if ($l =~ /^(\S+):(\d+):\s/) { $ifc{'virtual'} = $2; }
	if ($l =~ /inet\s+(\S+)/) { $ifc{'address'} = $1; }
	if ($l =~ /netmask\s+(\S+)/) { $ifc{'netmask'} = $self->parse_hex($1); }
	if ($l =~ /broadcast\s+(\S+)/) { $ifc{'broadcast'} = $1; }
	if ($l =~ /ether\s+(\S+)/) { $ifc{'ether'} = $1; }
	if ($l =~ /mtu\s+(\S+)/) { $ifc{'mtu'} = $1; }
	$ifc{'up'}++ if ($l =~ /\<UP/);
	$ifc{'edit'} = ($ifc{'name'} !~ /ipdptp|ppp/);
	$ifc{'index'} = scalar(@rv);
	if ($ifc{'ether'}) {
		$ifc{'ether'} = join(":", map { sprintf "%2.2d", $_ }
					      split(/:/, $ifc{'ether'}));
		}
	push(@rv, \%ifc);
	}
return @rv;
}

# activate_interface(&details)
# Create or modify an interface
sub activate_interface
{
	my $self = shift;

local $a = $_[0];
if ($a->{'virtual'} eq "") {
	local $out = $self->backquote_logged("ifconfig $a->{'name'} plumb 2>&1");
	if ($out) { $self->error("Interface '$a->{'name'}' does not exist"); }
	}
local $cmd = "ifconfig $a->{'name'}";
if ($a->{'virtual'} ne "") { $cmd .= ":$a->{'virtual'}"; }
$cmd .= " $a->{'address'}";
if ($a->{'netmask'}) { $cmd .= " netmask $a->{'netmask'}"; }
else { $cmd .= " netmask +"; }
if ($a->{'broadcast'}) { $cmd .= " broadcast $a->{'broadcast'}"; }
else { $cmd .= " broadcast +"; }
if ($a->{'mtu'}) { $cmd .= " mtu $a->{'mtu'}"; }
if ($a->{'up'}) { $cmd .= " up"; }
else { $cmd .= " down"; }
local $out = $self->backquote_logged("$cmd 2>&1");
if ($?) { $self->error($out); }
if ($a->{'ether'}) {
	$out = $self->backquote_logged(
		"ifconfig $a->{'name'} ether $a->{'ether'} 2>&1");
	if ($? && $out !~ /Device busy/) { $self->error($out); }
	}
}

# deactivate_interface(&details)
# Deactive an interface
sub deactivate_interface
{
	my $self = shift;

local $cmd;
if ($self->{'a'}->{'virtual'} eq "") {
	$cmd = "ifconfig $self->{'a'}->{'name'} unplumb";
	}
else {
	$cmd = "ifconfig $self->{'a'}->{'name'}:$self->{'a'}->{'virtual'} 0.0.0.0 down";
	}
local $out = $self->backquote_logged("$cmd 2>&1");
if ($?) { $self->error($out); }
}

# boot_interfaces()
# Returns a list of interfaces brought up at boot time
sub boot_interfaces
{
	my $self = shift;

local (@rv, $f, %mask);
push(@rv, { 'name' => 'lo0',
	    'fullname' => 'lo0',
	    'address' => '127.0.0.1',
	    'netmask' => '255.0.0.0',
	    'up' => 1,
	    'edit' => 0 });
open(MASK, "/etc/netmasks");
while(<MASK>) {
	s/\r|\n//g;
	s/#.*$//g;
	if (/([0-9\.]+)\s+([0-9\.]+)/) {
		$mask{$1} = $2;
		}
	}
close(MASK);
opendir(ETC, "/etc");
while($f = readdir(ETC)) {
	if ($f =~ /^hostname.(\S+):(\d+)$/ || $f =~ /^hostname.(\S+)/) {
		local %ifc;
		$ifc{'fullname'} = $ifc{'name'} = $1;
		$ifc{'virtual'} = $2 if (defined($2));
		$ifc{'fullname'} .= ":$2" if (defined($2));
		$ifc{'index'} = scalar(@rv);
		$ifc{'edit'}++;
		$ifc{'file'} = "/etc/$f";
		open(FILE, "/etc/$f");
		chop($ifc{'address'} = <FILE>);
		close(FILE);
		if ($ifc{'address'}) {
			$self->to_ipaddress($ifc{'address'})
				=~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/;
			if ($mask{"$1.$2.$3.0"})
				{ $ifc{'netmask'} = $mask{"$1.$2.$3.0"}; }
			elsif ($mask{"$1.$2.0.0"})
				{ $ifc{'netmask'} = $mask{"$1.$2.0.0"}; }
			elsif ($mask{"$1.0.0.0"})
				{ $ifc{'netmask'} = $mask{"$1.0.0.0"}; }
			else
				{ $ifc{'netmask'} = "255.255.255.0"; }
			local ($a1, $a2, $a3, $a4) = ($1, $2, $3, $4);
			$ifc{'netmask'} =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/;
			$ifc{'broadcast'} = sprintf "%d.%d.%d.%d",
						($a1 | ~int($1))&0xff,
						($a2 | ~int($2))&0xff,
						($a3 | ~int($3))&0xff,
						($a4 | ~int($4))&0xff;
			}
		else {
			$ifc{'netmask'} = "Automatic";
			$ifc{'broadcast'} = "Automatic";
			$ifc{'dhcp'}++;
			}
		$ifc{'up'}++;
		push(@rv, \%ifc);
		}
	}
closedir(ETC);
return @rv;
}

# save_interface(&details)
# Create or update a boot-time interface
sub save_interface
{
	my $self = shift;

local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
				       : $_[0]->{'name'};
$self->open_lock_tempfile(IFACE, ">/etc/hostname.$name");
if (!$_[0]->{'dhcp'}) {
	$self->print_tempfile(IFACE, $_[0]->{'address'},"\n");
	}
$self->close_tempfile(IFACE);
}

# delete_interface(&details)
# Delete a boot-time interface
sub delete_interface
{
	my $self = shift;

local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
				       : $_[0]->{'name'};
$self->unlink_logged("/etc/hostname.$name");
}

# iface_type(name)
# Returns a human-readable interface type name
sub iface_type
{
	my $self = shift;

return "Fast Ethernet" if ($_[0] =~ /^hme/);
return "Loopback" if ($_[0] =~ /^lo/);
return "Token Ring" if ($_[0] =~ /^tr/);
return "PPP" if ($_[0] =~ /^ipdptp/ || $_[0] =~ /^ppp/);
return "Ethernet";
}

# iface_hardware(name)
# Does some interface have an editable hardware address
sub iface_hardware
{
	my $self = shift;

return $_[0] !~ /^(lo|ipdptp|ppp)/;
}

# can_edit(what)
# Can some boot-time interface parameter be edited?
sub can_edit
{
	my $self = shift;

return $_[0] eq "dhcp";
}

# valid_boot_address(address)
# Is some address valid for a bootup interface
sub valid_boot_address
{
	my $self = shift;

return 1 if ($self->check_ipaddress($_[0]));
return gethostbyname($_[0]) ? 1 : 0;
}

# get_dns_config()
# Returns a hashtable containing keys nameserver, domain, search & order
sub get_dns_config
{
	my $self = shift;

local $dns;
open(RESOLV, "/etc/resolv.conf");
while(<RESOLV>) {
	s/\r|\n//g;
	s/#.*$//g;
	if (/nameserver\s+(.*)/) {
		push(@{$dns->{'nameserver'}}, split(/\s+/, $1));
		}
	elsif (/domain\s+(\S+)/) {
		$dns->{'domain'} = [ $1 ];
		}
	elsif (/search\s+(.*)/) {
		$dns->{'domain'} = [ split(/\s+/, $1) ];
		}
	}
close(RESOLV);
open(SWITCH, "/etc/nsswitch.conf");
while(<SWITCH>) {
	s/\r|\n//g;
	if (/hosts:\s+(.*)/) {
		$dns->{'order'} = $1;
		}
	}
close(SWITCH);
$dns->{'files'} = [ "/etc/resolv.conf", "/etc/nsswitch.conf" ];
return $dns;
}

# save_dns_config(&config)
# Writes out the resolv.conf and nsswitch.conf files
sub save_dns_config
{
	my $self = shift;

$self->lock_file("/etc/resolv.conf");
open(RESOLV, "/etc/resolv.conf");
local @resolv = <RESOLV>;
close(RESOLV);
$self->open_tempfile(RESOLV, ">/etc/resolv.conf");
foreach (@{$_[0]->{'nameserver'}}) {
	$self->print_tempfile(RESOLV, "nameserver $_\n");
	}
if ($_[0]->{'domain'}) {
	if ($_[0]->{'domain'}->[1]) {
		$self->print_tempfile(RESOLV, "search ",join(" ", @{$_[0]->{'domain'}}),"\n");
		}
	else {
		$self->print_tempfile(RESOLV, "domain $_[0]->{'domain'}->[0]\n");
		}
	}
foreach (@resolv) {
	$self->print_tempfile(RESOLV, $_) if (!/^\s*(nameserver|domain|search)\s+/);
	}
$self->close_tempfile(RESOLV);
$self->unlock_file("/etc/resolv.conf");

$self->lock_file("/etc/nsswitch.conf");
open(SWITCH, "/etc/nsswitch.conf");
local @switch = <SWITCH>;
close(SWITCH);
$self->open_tempfile(SWITCH, ">/etc/nsswitch.conf");
foreach (@switch) {
	if (/hosts:\s+/) {
		$self->print_tempfile(SWITCH, "hosts:\t$_[0]->{'order'}\n");
		}
	else {
		$self->print_tempfile(SWITCH, $_);
		}
	}
$self->close_tempfile(SWITCH);
$self->unlock_file("/etc/nsswitch.conf");
}

# order_input(&dns)
# Returns HTML for selecting the name resolution order
sub order_input
{
	my $self = shift;

if ($_[0]->{'order'} =~ /\[/) {
	# Using a complex resolve list
	return "<input name=order size=45 value=\"$_[0]->{'order'}\">\n";
	}
else {
	# Can select by menus
	local @o = split(/\s+/, $_[0]->{'order'});
	local ($rv, $i, $j);
	local @srcs = ( "", "files", "dns", "nis", "nisplus" );
	local @srcn = ( "", "Hosts", "DNS", "NIS", "NIS+" );
	for($i=1; $i<@srcs; $i++) {
		local $ii = $i-1;
		$rv .= "<select name=order_$ii>\n";
		for($j=0; $j<@srcs; $j++) {
			$rv .= sprintf "<option value=\"%s\" %s>%s\n",
					$srcs[$j],
					$o[$ii] eq $srcs[$j] ? "selected" : "",
					$srcn[$j] ? $srcn[$j] : "&nbsp;";
			}
		$rv .= "</select>\n";
		}
	return $rv;
	}
}

# parse_order(&dns)
# Parses the form created by order_input()
sub parse_order
{
	my $self = shift;

if (defined($self->{'in_hash'}->{'order'})) {
	$self->{'in_hash'}->{'order'} =~ /\S/ || $self->error($self->{'text_hash'}->{'dns_eorder'});
	$_[0]->{'order'} = $self->{'in_hash'}->{'order'};
	}
else {
	local($i, @order);
	for($i=0; defined($self->{'in_hash'}->{"order_$i"}); $i++) {
		push(@order, $self->{'in_hash'}->{"order_$i"}) if ($self->{'in_hash'}->{"order_$i"});
		}
	$_[0]->{'order'} = join(" ", @order);
	}
}

sub get_hostname
{
	my $self = shift;

return $self->get_system_hostname();
}

# save_hostname(name)
sub save_hostname
{
	my $self = shift;

$self->system_logged("hostname $_[0] >/dev/null 2>&1");
undef(@main::get_system_hostname);      # clear cache
}

# get_domainname()
sub get_domainname
{
	my $self = shift;

local $d;
$self->execute_command("domainname", undef, \$d, undef);
chop($d);
return $d;
}

# save_domainname(domain)
sub save_domainname
{
	my $self = shift;

$self->system_logged("domainname ".quotemeta($_[0]));
$self->lock_file("/etc/defaultdomain");
if ($_[0]) {
	$self->open_tempfile(DOMAIN, ">/etc/defaultdomain");
	$self->print_tempfile(DOMAIN, $_[0],"\n");
	$self->close_tempfile(DOMAIN);
	}
else {
	$self->unlink_file("/etc/defaultdomain");
	}
$self->unlock_file("/etc/defaultdomain");
}

sub routing_config_files
{
	my $self = shift;

return ( "/etc/defaultrouter", "/etc/notrouter", "/etc/gateways" );
}

sub routing_input
{
	my $self = shift;

# show default router(s) input
local(@defrt);
$self->open_readfile(*DEFRT, "/etc/defaultrouter");
while(<DEFRT>) {
	s/#.*$//g;
	if (/(\S+)/) { push(@defrt, $1); }
	}
close(DEFRT);
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'routes_defaults'}</b></td>\n";
print "<td><textarea name=defrt rows=3 cols=40>",
	join("\n", @defrt),"</textarea></td> </tr>\n";

# show router input
local $notrt = (-r "/etc/notrouter");
local $gatew = (-r "/etc/gateways");
print "<tr> <td><b>Act as router?</b></td> <td>\n";
printf "<input type=radio name=router value=0 %s> $self->{'text_hash'}->{'yes'}\n",
	$gatew && !$notrt ? "checked" : "";
printf "<input type=radio name=router value=1 %s> $self->{'text_hash'}->{'routes_possible'}\n",
	!$gatew && !$notrt ? "checked" : "";
printf "<input type=radio name=router value=2 %s> $self->{'text_hash'}->{'no'}\n",
	$notrt ? "checked" : "";
print "</td> </tr>\n";
}

sub parse_routing
{
	my $self = shift;

local @defrt = split(/\s+/, $self->{'in_hash'}->{'defrt'});
foreach $d (@defrt) {
	gethostbyname($d) || $self->error($self->text('routes_edefault', $d));
	}
$self->lock_file("/etc/defaultrouter");
if (@defrt) {
	$self->open_tempfile(DEFRT, ">/etc/defaultrouter");
	foreach $d (@defrt) { $self->print_tempfile(DEFRT, $d,"\n"); }
	$self->close_tempfile(DEFRT);
	}
else {
	$self->unlink_file("/etc/defaultrouter");
	}
$self->unlock_file("/etc/defaultrouter");

$self->lock_file("/etc/gateways");
$self->lock_file("/etc/notrouter");
if ($self->{'in_hash'}->{'router'} == 0) {
	$self->create_empty_file("/etc/gateways");
	$self->unlink_file("/etc/notrouter");
	}
elsif ($self->{'in_hash'}->{'router'} == 2) {
	$self->create_empty_file("/etc/notrouter");
	$self->unlink_file("/etc/gateways");
	}
else {
	$self->unlink_file("/etc/gateways");
	$self->unlink_file("/etc/notrouter");
	}
$self->unlock_file("/etc/gateways");
$self->unlock_file("/etc/notrouter");
}

# create_empty_file(filename)
sub create_empty_file
{
	my $self = shift;

if (!-r $_[0]) {
	$self->open_tempfile(EMPTY,">$_[0]");
	$self->close_tempfile(EMPTY);
	}
}

sub os_feedback_files
{
	my $self = shift;

opendir(DIR, "/etc");
local @f = map { "/etc/$_" } grep { /^hostname\./ } readdir(DIR);
closedir(DIR);
return ( @f, "/etc/netmasks", "/etc/resolv.conf", "/etc/nsswitch.conf",
	 "/etc/defaultrouter", "/etc/notrouter", "/etc/gateways" );
}


1;

