#!/usr/bin/perl
package nis::LinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'nis::LinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Common NIS server functions for Linux

1;
		return $self;
}

# show_server_security()
# Show NIS server security-related options
sub show_server_security
{
	my $self = shift;

local ($opts, $hosts) = $self->parse_ypserv_conf();

# Show port checking option
local $port = $opts->{'xfr_check_port'} ? $opts->{'xfr_check_port'}->{'value'}
					: 1;
print "<tr> <td><b>$self->{'text_hash'}->{'security_port'}</b></td>\n";
printf "<td><input type=radio name=port value=1 %s> %s\n",
	$port ? 'checked' : '', $self->{'text_hash'}->{'yes'};
printf "<input type=radio name=port value=0 %s> %s</td> </tr>\n",
	$port ? '' : 'checked', $self->{'text_hash'}->{'no'};

# Work out if the new (with domains) or old format is in use
local ($newfmt) = grep { $_->{'domain'} } @$hosts;
print $self->ui_hidden("format", $newfmt ? 1 : 0),"\n";
local $table;
if ($newfmt) {
	# Generate new format table
	$table .= $self->ui_columns_start([ $self->{'text_hash'}->{'security_hosts'},
				      $self->{'text_hash'}->{'security_domain'},
				      $self->{'text_hash'}->{'security_map'},
				      $self->{'text_hash'}->{'security_sec'},
				      $self->{'text_hash'}->{'security_mangle'} ]);
	local $i = 0;
	foreach $h (@$hosts, { 'map' => '*' }) {
		local @cols;
		push(@cols, $self->ui_radio("host_def_$i", $h->{'host'} eq '*' ? 2 :
						     $h->{'host'} ? 0 : 1,
			[ [ 1, $self->{'text_hash'}->{'security_none'} ],
			  [ 2, $self->{'text_hash'}->{'security_any'} ],
			  [ 0, $self->ui_textbox("host_$i",
			$h->{'host'} eq '*' ? undef : $h->{'host'}, 20) ] ]));
		push(@cols, $self->ui_opt_textbox("domain_$i",
				$h->{'domain'} eq '*' ? undef : $h->{'domain'},
				20, $self->{'text_hash'}->{'security_tall'}));
		push(@cols, $self->ui_radio("map_def_$i", $h->{'map'} eq '*' ? 1 : 0,
		      [ [ 1, $self->{'text_hash'}->{'security_tall'} ],
			[ 0, $self->ui_textbox("map_$i", $h->{'map'} eq '*' ? undef :
							$h->{'map'}, 20) ] ]));
		push(@cols, $self->ui_select("sec_$i", $h->{'sec'},
			[ [ "none", $self->{'text_hash'}->{'security_sec_none'} ],
			  [ "port", $self->{'text_hash'}->{'security_sec_port'} ],
			  [ "deny", $self->{'text_hash'}->{'security_sec_deny'} ] ]));
		push(@cols, $self->ui_opt_textbox("mangle_$i",
				!$h->{'mangle'} ? undef :
				  $h->{'field'} ? $h->{'field'} : 2,
				5, $self->{'text_hash'}->{'security_none'}));
		$table .= $self->ui_columns_row(\@cols);
                $i++;
		}
	}
else {
	# Generate old format table
	$table .= $self->ui_columns_start([ $self->{'text_hash'}->{'security_hosts'},
				      $self->{'text_hash'}->{'security_map'},
				      $self->{'text_hash'}->{'security_sec'},
				      $self->{'text_hash'}->{'security_mangle'} ]);
	local $i = 0;
	foreach $h (@$hosts, { 'map' => '*' }) {
		local @cols;
		push(@cols, $self->ui_radio("host_def_$i", $h->{'host'} eq '*' ? 2 :
						     $h->{'host'} ? 0 : 1,
			[ [ 1, $self->{'text_hash'}->{'security_none'} ],
			  [ 2, $self->{'text_hash'}->{'security_any'} ],
			  [ 0, $self->ui_textbox("host_$i",
			$h->{'host'} eq '*' ? undef : $h->{'host'}, 20) ] ]));
		push(@cols, $self->ui_radio("map_def_$i", $h->{'map'} eq '*' ? 1 : 0,
		      [ [ 1, $self->{'text_hash'}->{'security_tall'} ],
			[ 0, $self->ui_textbox("map_$i", $h->{'map'} eq '*' ? undef :
							$h->{'map'}, 20) ] ]));
		push(@cols, $self->ui_select("sec_$i", $h->{'sec'},
			[ [ "none", $self->{'text_hash'}->{'security_sec_none'} ],
			  [ "port", $self->{'text_hash'}->{'security_sec_port'} ],
			  [ "deny", $self->{'text_hash'}->{'security_sec_deny'} ],
			  [ "des", $self->{'text_hash'}->{'security_sec_des'} ] ]));
		push(@cols, $self->ui_radio("mangle_$i", $h->{'mangle'} ? 1 : 0,
			[ [ 0, $self->{'text_hash'}->{'security_none'} ],
			  [ 1, $self->ui_textbox("field_$i", $h->{'field'}, 4) ] ]));
		$table .= $self->ui_columns_row(\@cols);
		$i++;
		}
	$table .= $self->ui_columns_end();
	}

print "<tr> <td colspan=2><b>$self->{'text_hash'}->{'security_maps'}</b><br>\n";
print "$table</td> </tr>\n";

print "</table></td> </tr>\n";
}

# parse_server_security()
# Save and apply server security options
sub parse_server_security
{
	my $self = shift;

# Save security settings
local ($opts, $hosts) = $self->parse_ypserv_conf();
local $lref = $self->read_file_lines($self->{'ypserv_conf'});
local $xfr = $opts->{'xfr_check_port'};
local $line = $self->{'in_hash'}->{'port'} ? 'xfr_check_port: yes' : 'xfr_check_port: no';
if ($xfr) {
	$lref->[$xfr->{'line'}] = $line;
	}
else {
	push(@$lref, $line);
	}

# Save host restrictions
local ($i, $j, $offset);
for($i=0; defined($self->{'in_hash'}->{"host_$i"}); $i++) {
	local @line;
	next if ($self->{'in_hash'}->{"host_def_$i"} == 1);
	$self->{'in_hash'}->{"host_def_$i"} == 2 || $self->{'in_hash'}->{"host_$i"} =~ /^[^:\s]+$/ ||
		 $self->error($self->text('security_ehost', $self->{'in_hash'}->{"host_$i"}));
	$self->{'in_hash'}->{"map_def_$i"} || $self->{'in_hash'}->{"map_$i"} =~ /^[^:\s]+$/ ||
		$self->error($self->text('security_emap', $self->{'in_hash'}->{"map_$i"}));
	if ($self->{'in_hash'}->{'format'} == 1) {
		# New format, including domain
		$self->{'in_hash'}->{"domain_${i}_def"} || $self->{'in_hash'}->{"domain_$i"} =~ /^[^:\s]+$/ ||
			$self->error($self->text('security_edomain', $self->{'in_hash'}->{"domain_$i"}));
		$self->{'in_hash'}->{"mangle_${i}_def"} || $self->{'in_hash'}->{"mangle_$i"} =~ /^\d+$/ ||
			$self->error($self->text('security_efield', $self->{'in_hash'}->{"mangle_$i"}));
		@line = ( $self->{'in_hash'}->{"host_def_$i"} == 2 ? "*" : $self->{'in_hash'}->{"host_$i"},
			  $self->{'in_hash'}->{"domain_${i}_def"} ? "*" : $self->{'in_hash'}->{"domain_$i"},
			  $self->{'in_hash'}->{"map_def_$i"} ? "*" : $self->{'in_hash'}->{"map_$i"},
			  $self->{'in_hash'}->{"sec_$i"}.
			    ($self->{'in_hash'}->{"mangle_${i}_def"} ? "" :
			     "/mangle:".$self->{'in_hash'}->{"mangle_$i"}) );
		}
	else {
		# Old format
		$self->{'in_hash'}->{"field_$i"} =~ /^\d*$/ ||
			$self->error($self->text('security_efield', $self->{'in_hash'}->{"field_$i"}));
		@line = ( $self->{'in_hash'}->{"host_def_$i"} == 2 ? "*" : $self->{'in_hash'}->{"host_$i"},
			  $self->{'in_hash'}->{"map_def_$i"} ? "*" : $self->{'in_hash'}->{"map_$i"},
			  $self->{'in_hash'}->{"sec_$i"},
			  $self->{'in_hash'}->{"mangle_$i"} ? "yes" : "no" );
		push(@line, $self->{'in_hash'}->{"field_$i"})
			if ($self->{'in_hash'}->{"field_$i"} && $self->{'in_hash'}->{"field_$i"} != 2);
		}
	local $old = $hosts->[$j++];
	if ($old) {
		$lref->[$old->{'line'}] = join(":", @line);
		}
	else {
		push(@$lref, join(":", @line));
		}
	}
while($hosts->[$j]) {
	splice(@$lref, $hosts->[$j]->{'line'}-$offset, 1);
	$j++; $offset++;
	}
$self->flush_file_lines();

# Apply the changes
local $pid = $self->check_pid_file($self->{'pid_file'});
$self->kill_logged('HUP', $pid) if ($pid);
}


1;

