#!/usr/bin/perl
package nis::SlackwarelinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'nis::SlackwarelinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# redhat-linux-lib.pl
# NIS functions for redhat linux NIS client and server

$self->{'yp_makefile'} = "/var/yp/Makefile";
$self->{'ypserv_conf'} = "/etc/ypserv.conf";
$self->{'pid_file'} = "/var/run/ypserv.pid";
$self->{'default_domain'} = "/etc/defaultdomain";

@{$self->{'nis_files_array'}} = ( "passwd", "shadow", "group", "gshadow", "adjunct",
	       "aliases", "ethers", "bootparams", "hosts", "networks",
	       "printcap", "protocols", "publickeys", "rpc", "services",
	       "netgroup", "netid", "auto_master", "auto_home", "auto_local" );

@{$self->{'nis_tables_array'}} = ( "passwd", "group", "hosts", "rpc", "services", "netid",
		"protocols", "netgrp", "mail", "shadow", "publickey", "networks",
		"ethers", "bootparams", "printcap", "amd.home", "auto.master",
		"auto.home", "auto.local", "passwd.adjunct" );

1;

1;
		return $self;
}

# get_nis_support()
# Returns 0 for no support, 1 for client only, 2 for server and 3 for both
sub get_nis_support
{
	my $self = shift;

local $rv;
$rv += 1 if ($self->has_command("ypbind"));
$rv += 2 if ($self->has_command("ypserv"));
return $rv;
}

# get_client_config()
# Returns a hash ref containg details of the client's NIS settings
sub get_client_config
{
	my $self = shift;

local $nis;
open(CONF, $self->{'config_hash'}->{'client_conf'});
while(<CONF>) {
	s/\r|\n//g;
	s/#.*$//g;
	if (/^\s*domain\s*(\S+)\s*broadcast/i) {
		$nis->{'broadcast'}++;
		}
	elsif (/^\s*domain\s*(\S+)\s*server\s*(\S+)/i) {
		push(@{$nis->{'servers'}}, $2);
		}
	elsif (/^\s*ypserver\s*(\S+)/) {
		push(@{$nis->{'servers'}}, $1);
		}
	}
close(CONF);
if (open(DOMAIN, $self->{'default_domain'})) {
	chop($nis->{'domain'} = <DOMAIN>);
	close(DOMAIN);
	}
return $nis;
}

# save_client_config(&config)
# Saves and applies the NIS client configuration in the give hash.
# Returns an error message if any, or undef on success.
sub save_client_config
{
	my $self = shift;

# Save the config file
$self->open_tempfile(CONF, ">$self->{'config_hash'}->{'client_conf'}");
if ($_[0]->{'domain'}) {
	if ($_[0]->{'broadcast'}) {
		$self->print_tempfile(CONF, "domain $_[0]->{'domain'} broadcast\n");
		}
	else {
		local @s = @{$_[0]->{'servers'}};
		$self->print_tempfile(CONF,"domain $_[0]->{'domain'} server $s[0]\n");
		foreach $s (@s) {
			$self->print_tempfile(CONF, "ypserver $s\n");
			}
		}
	}
$self->close_tempfile(CONF);
if ($_[0]->{'domain'}) {
	$self->open_tempfile(DOMAIN, ">$self->{'default_domain'}");
	$self->print_tempfile(DOMAIN, $_[0]->{'domain'},"\n");
	$self->close_tempfile(DOMAIN);
	}
else {
	unlink($self->{'default_domain'});
	}

# Apply by setting the domainname and running the init script
$self->system_logged("nisdomainname '$_[0]->{'domain'}' >/dev/null 2>&1");
if ($_[0]->{'domain'}) {
	local $out = $self->backquote_logged("ypbind 2>&1");
	if ($?) { return "<pre>$out</pre>"; }
	}
return undef;
}

# show_server_config()
# Display a form for editing NIS server options
sub show_server_config
{
	my $self = shift;

local ($var, $rule) = $self->parse_yp_makefile();

local @pid = $self->find_byname("ypserv");
local $boot = @pid > 0;
print "<tr> <td><b>$self->{'text_hash'}->{'server_boot'}</b></td>\n";
printf "<td><input type=radio name=boot value=1 %s> %s\n",
	$boot ? 'checked' : '', $self->{'text_hash'}->{'yes'};
printf "<input type=radio name=boot value=0 %s> %s</td>\n",
	$boot ? '' : 'checked', $self->{'text_hash'}->{'no'};

local $dom = $var->{'LOCALDOMAIN'}->{'value'};
print "<td><b>$self->{'text_hash'}->{'server_domain'}</b></td>\n";
printf "<td><input type=radio name=domain_auto value=1 %s> %s\n",
	$dom =~ /`.*domainname`/ ? 'checked' : '', $self->{'text_hash'}->{'server_domain_auto'};
printf "<input type=radio name=domain_auto value=0 %s>\n",
	$dom =~ /`.*domainname`/ ? '' : 'checked';
printf "<input name=domain size=20 value='%s'></td> </tr>\n",
	$dom =~ /`.*domainname`/ ? '' : $dom;

print "<tr> <td><b>$self->{'text_hash'}->{'server_type'}</b></td>\n";
printf "<td colspan=3><input type=radio name=type value=1 %s> %s\n",
	$self->{'config_hash'}->{'slave'} ? '' : 'checked', $self->{'text_hash'}->{'server_master'};
printf "<input type=radio name=type value=0 %s> %s\n",
	$self->{'config_hash'}->{'slave'} ? 'checked' : '', $self->{'text_hash'}->{'server_slave'};
printf "<input name=slave size=30 value='%s'></td> </tr>\n", $self->{'config_hash'}->{'slave'};

print "</table></td></tr></table><p>\n";
print "<table border width=100%>\n";
print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'server_mheader'}</b></td> </tr>\n";
print "<tr $self->{'cb'}> <td><table width=100%>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'server_dns'}</b></td>\n";
printf "<td><input type=radio name=b value='-b' %s> %s\n",
	$var->{'B'}->{'value'} eq '-b' ? 'checked' : '', $self->{'text_hash'}->{'yes'};
printf "<input type=radio name=b value='' %s> %s</td>\n",
	$var->{'B'}->{'value'} eq '-b' ? '' : 'checked', $self->{'text_hash'}->{'no'};

print "<td><b>$self->{'text_hash'}->{'server_push'}</b></td>\n";
printf "<td><input type=radio name=nopush value=false %s> %s\n",
	$var->{'NOPUSH'}->{'value'} eq 'true' ? '' : 'checked', $self->{'text_hash'}->{'yes'};
printf "<input type=radio name=nopush value=true %s> %s</td> </tr>\n",
	$var->{'NOPUSH'}->{'value'} eq 'true' ? 'checked' : '', $self->{'text_hash'}->{'no'};

local %inall;
map { $inall{$_}++ } split(/\s+/, $rule->{'all'}->{'value'});
print "<tr> <td rowspan=2 valign=top><b>$self->{'text_hash'}->{'server_tables'}</b></td>\n";
print "<td rowspan=2><select multiple size=5 name=tables>\n";
foreach $t (grep { $rule->{$_} } @{$self->{'nis_tables_array'}}) {
	printf "<option value=%s %s>%s\n",
		$t, $inall{$t} ? 'selected' : '', $t;
	}
print "</select></td>\n";

print "<td><b>$self->{'text_hash'}->{'server_minuid'}</b></td>\n";
printf "<td><input name=minuid size=10 value='%s'></td> </tr>\n",
	$var->{'MINUID'}->{'value'};

print "<td><b>$self->{'text_hash'}->{'server_mingid'}</b></td>\n";
printf "<td><input name=mingid size=10 value='%s'></td> </tr>\n",
	$var->{'MINGID'}->{'value'};

print "<tr> <td><b>$self->{'text_hash'}->{'server_slaves'}</b></td>\n";
open(SLAVES, "/var/yp/ypservers");
while(<SLAVES>) {
	s/\s//g;
	push(@{$self->{'slaves_array'}}, $_) if ($_);
	}
close(SLAVES);
printf "<td colspan=3><input name=slaves size=60 value='%s'></td> </tr>\n",
	join(" ", @{$self->{'slaves_array'}});

print "</table></td></tr></table><p>\n";
print "<table border width=100%>\n";
print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'server_fheader'}</b></td> </tr>\n";
print "<tr $self->{'cb'}> <td><table width=100%>\n";

local $i = 0;
foreach $t (@{$self->{'nis_files_array'}}) {
	local $f = $self->expand_vars($var->{uc($t)}->{'value'}, $var);
	next if (!$f);
	print "<tr>\n" if ($i%2 == 0);
	print "<td><b>",$self->text('server_file', $self->{'text_hash'}->{"desc_$t"} ? $self->{'text_hash'}->{"desc_$t"}
							      : $t),"</b></td>\n";
	print "<td><input name=$t size=30 value='$f'></td>\n";
	print "</tr>\n" if ($i++%2 == 1);
	}
}

# parse_server_config()
# Parse and save the NIS server options
sub parse_server_config
{
	my $self = shift;

local ($var, $rule) = $self->parse_yp_makefile();
$self->{'in_hash'}->{'minuid'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'server_eminuid'});
$self->{'in_hash'}->{'mingid'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'server_emingid'});
$self->{'in_hash'}->{'domain_auto'} || $self->{'in_hash'}->{'domain'} =~ /^[A-Za-z0-9\.\-]+$/ ||
	$self->error($self->{'text_hash'}->{'server_edomain'});
$self->{'in_hash'}->{'type'} || gethostbyname($self->{'in_hash'}->{'slave'}) || $self->error($self->{'text_hash'}->{'server_eslave'});
$self->update_makefile($var->{'MINUID'}, $self->{'in_hash'}->{'minuid'});
$self->update_makefile($var->{'MINGID'}, $self->{'in_hash'}->{'mingid'});
$self->update_makefile($var->{'NOPUSH'}, $self->{'in_hash'}->{'nopush'});
$self->update_makefile($var->{'B'}, $self->{'in_hash'}->{'b'});
$self->update_makefile($var->{'LOCALDOMAIN'}, $self->{'in_hash'}->{'domain_auto'} ? "`domainname`"
							   : $self->{'in_hash'}->{'domain'});
$self->update_makefile($rule->{'all'}, join(" ", split(/\0/, $self->{'in_hash'}->{'tables'})), "");

foreach $t (@{$self->{'nis_files_array'}}) {
	local $old = $self->expand_vars($var->{uc($t)}->{'value'}, $var);
	next if (!$old);
	if ($old ne $self->{'in_hash'}->{$t}) {
		$self->{'in_hash'}->{$t} =~ /\S/ || $self->error($self->text('server_efile', $self->{'text_hash'}->{"desc_$t"}));
		$self->update_makefile($var->{uc($t)}, $self->{'in_hash'}->{$t});
		}
	}

$self->open_tempfile(SLAVES, ">/var/yp/ypservers");
foreach $s (split(/\s+/, $self->{'in_hash'}->{'slaves'})) {
	$self->print_tempfile(SLAVES, "$s\n");
	}
$self->close_tempfile(SLAVES);

local @pid = $self->find_byname("ypserv");
if ($self->{'in_hash'}->{'boot'}) {
	# Start ypserv if not running
	if (@pid == 0) {
		$self->system_logged("/usr/sbin/ypserv >/dev/null 2>&1");
		}
	}
else {
	# Stop if running
	if (@pid > 0) {
		$self->kill_logged('TERM', @pid);
		}
	}
$self->flush_file_lines();
if ($self->{'in_hash'}->{'type'}) {
	# Master server
	delete($self->{'config_hash'}->{'slave'});
	$self->apply_table_changes() if ($self->{'in_hash'}->{'boot'});
	}
else {
	$self->{'out'} = $self->backquote_logged("/usr/lib/yp/ypinit -s $self->{'in_hash'}->{'slave'} 2>&1");
	if ($?) { $self->error("<tt>$self->{'out'}</tt>"); }
	$self->{'config_hash'}->{'slave'} = $self->{'in_hash'}->{'slave'};
	}
$self->write_file("$self->{'module_config_directory'}/config", \%{$self->{'config_hash'}});
}

# get_server_mode()
# Returns 0 if the NIS server is inactive, 1 if active as a master, or 2 if
# active as a slave.
sub get_server_mode
{
	my $self = shift;

local @pid = $self->find_byname("ypserv");
if (@pid == 0) {
	return 0;
	}
elsif ($self->{'config_hash'}->{'slave'}) {
	return 2;
	}
else {
	return 1;
	}
}

# list_nis_tables()
# Returns a list of structures of all NIS tables
sub list_nis_tables
{
	my $self = shift;

local ($var, $rule) = $self->parse_yp_makefile();
local @rv;
local $dom = $var->{'LOCALDOMAIN'}->{'value'};
chop($dom = `domainname`) if ($dom =~ /`.*domainname`/);
local %file;
map { $file{uc($_)} = $self->expand_vars($var->{uc($_)}->{'value'}, $var) } @{$self->{'nis_files_array'}};
local @all = split(/\s+/, $rule->{'all'}->{'value'});
foreach $t (@all) {
	local $table = { 'table' => $t,
		         'index' => scalar(@rv),
		         'domain' => $dom };
	if ($t eq "passwd") {
		if ($var->{'MERGE_PASSWD'}->{'value'} eq 'true') {
			$table->{'type'} = 'passwd_shadow';
			$table->{'files'} = [ $file{'PASSWD'}, $file{'SHADOW'} ];
			}
		elsif ($self->indexof('shadow', @all) >= 0) {
			# Show separate shadow and passwd tables as one table
			$table->{'type'} = 'passwd_shadow_full';
			$table->{'files'} = [ $file{'PASSWD'}, $file{'SHADOW'} ];
			@all = grep { $_ ne 'shadow' } @all;
			}
		else {
			$table->{'type'} = 'passwd';
			$table->{'files'} = [ $file{'PASSWD'} ];
			}
		}
	elsif ($t eq "group") {
		if ($var->{'MERGE_GROUP'}->{'value'} eq 'true') {
			$table->{'type'} = 'group_shadow';
			$table->{'files'} = [ $file{'GROUP'}, $file{'GSHADOW'} ];
			}
		else {
			$table->{'type'} = 'group';
			$table->{'files'} = [ $file{'GROUP'} ];
			}
		}
	elsif ($t eq "netgrp") {
		$table->{'type'} = "netgroup";
		$table->{'files'} = [ $file{'NETGROUP'} ];
		}
	elsif ($t eq "mail") {
		$table->{'type'} = "aliases";
		$table->{'files'} = [ $file{'ALIASES'} ];
		}
	else {
		$table->{'type'} = $t;
		$table->{'files'} = [ $file{uc($t)} ];
		}
	push(@rv, $table);
	}
return @rv;
}

# apply_table_changes()
# Do whatever is necessary for the table text files to be loaded into
# the NIS server
sub apply_table_changes
{
	my $self = shift;

$self->system_logged("(cd /var/yp ; make) >/dev/null 2>&1 </dev/null");
}

sub extra_config_files
{
	my $self = shift;

return ( "/etc/defaultdomain", "/var/yp/ypservers" );
}


1;

