#!/usr/bin/perl
package nis::SolarisLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'nis::SolarisLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# solaris-lib.pl
# NIS functions for solaris

$self->{'binding_dir'} = "/var/yp/binding";
$self->{'yp_makefile'} = "/var/yp/Makefile";

@{$self->{'nis_tables_array'}} = ( "passwd", "group", "hosts", "ethers", "networks", "rpc",
	        "services", "protocols", "netgroup", "bootparams", "aliases",
		"publickey", "netid", "netmasks", "c2secure", "timezone",
		"auto.master", "auto.home" );

1;

1;
		return $self;
}

# get_nis_support()
# Returns 0 for no support, 1 for client only, 2 for server and 3 for both
sub get_nis_support
{
	my $self = shift;

local $rv;
$rv += 1 if (-x "/usr/lib/netsvc/yp/ypbind");
$rv += 2 if (-x "/usr/lib/netsvc/yp/ypserv");
return $rv;
}

# get_client_config()
# Returns a hash ref containg details of the client's NIS settings
sub get_client_config
{
	my $self = shift;

local $nis;
open(DOM, "/etc/defaultdomain");
chop($nis->{'domain'} = <DOM>);
close(DOM);
if ($nis->{'domain'}) {
	if (open(SRV, "$self->{'binding_dir'}/$nis->{'domain'}/ypservers")) {
		while(<SRV>) {
			s/\r|\n//g;
			push(@{$nis->{'servers'}}, $_);
			}
		close(SRV);
		}
	else {
		$nis->{'broadcast'} = 1;
		}
	}
return $nis;
}

# save_client_config(&config)
# Saves and applies the NIS client configuration in the give hash.
# Returns an error message if any, or undef on success.
sub save_client_config
{
	my $self = shift;

if ($_[0]->{'domain'}) {
	# Check if the servers are in /etc/hosts
	local @s = @{$_[0]->{'servers'}};
	$self->foreign_require("net", "net-lib.pl");
	foreach $s (@s) {
		local $found = 0;
		foreach $h ($self->foreign_call("net", "list_hosts")) {
			$found++ if ($self->indexof($s, @{$h->{'hosts'}}) >= 0);
			}
		return $self->text("client_ehosts", $s) if (!$found);
		}

	# Write the files
	$self->open_tempfile(DOM, ">/etc/defaultdomain");
	$self->print_tempfile(DOM, $_[0]->{'domain'},"\n");
	$self->close_tempfile(DOM);
	mkdir("$self->{'binding_dir'}/$_[0]->{'domain'}", 0755);
	if (@s) {
		$self->open_tempfile(SRV,">$self->{'binding_dir'}/$_[0]->{'domain'}/ypservers");
		foreach $s (@s) {
			$self->print_tempfile(SRV, "$s\n");
			}
		$self->close_tempfile(SRV);
		}
	else {
		unlink("$self->{'binding_dir'}/$_[0]->{'domain'}/ypservers");
		}
	}
else {
	unlink("/etc/defaultdomain");
	}

# Apply by running ypstop and ypstart
$self->system_logged("(/usr/lib/netsvc/yp/ypstop || /etc/init.d/yp stop) >/dev/null 2>&1");
$self->system_logged("domainname \"$_[0]->{'domain'}\" >/dev/null 2>&1");
$self->system_logged("(/usr/lib/netsvc/yp/ypstart || /etc/init.d/yp start) >/dev/null 2>&1");
sleep(2);
if ($_[0]->{'domain'}) {
	local $out = `ypwhich 2>&1`;
	if ($? || $out =~ /not\s+bound/i || $out =~ /can't\s+communicate/i) {
		system("(/usr/lib/netsvc/yp/ypstop || /etc/init.d/yp stop) >/dev/null 2>&1");
		return $self->{'text_hash'}->{'client_eypwhich'};
		}
	}
}

# show_server_config()
# Display a form for editing NIS server options
sub show_server_config
{
	my $self = shift;

local ($var, $rule) = $self->parse_yp_makefile();
local $dom = `domainname`; chop($dom);

print "<tr> <td><b>$self->{'text_hash'}->{'server_boot'}</b></td> <td>\n";
if ($dom && -d "/var/yp/$dom") {
	print "<i>$self->{'text_hash'}->{'server_already'}</i>\n";
	}
else {
	print "<input type=radio name=boot value=1 > $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=boot value=0 checked > $self->{'text_hash'}->{'no'}\n";
	}
print "</td>\n";

print "<td><b>$self->{'text_hash'}->{'server_domain'}</b></td>\n";
printf "<td><input type=radio name=domain_def value=1 %s> %s\n",
	$dom ? '' : 'checked', $self->{'text_hash'}->{'server_none'};
printf "<input type=radio name=domain_def value=0 %s>\n",
	$dom ? 'checked' : '';
printf "<input name=domain size=35 value='%s'></td> </tr>\n", $dom;

print "<tr> <td><b>$self->{'text_hash'}->{'server_type'}</b></td>\n";
printf "<td colspan=3><input type=radio name=type value=1 %s> %s\n",
	$self->{'config_hash'}->{'slave'} ? '' : 'checked', $self->{'text_hash'}->{'server_master'};
printf "<input type=radio name=type value=0 %s> %s\n",
	$self->{'config_hash'}->{'slave'} ? 'checked' : '', $self->{'text_hash'}->{'server_slave'};
printf "<input name=slave size=30 value='%s'></td> </tr>\n", $self->{'config_hash'}->{'slave'};

print "<tr> <td colspan=4><i>$self->{'text_hash'}->{'server_solaris'}</i></td> </tr>\n";

print "</table></td></tr></table><p>\n";
print "<table border width=100%>\n";
print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'server_mheader'}</b></td> </tr>\n";
print "<tr $self->{'cb'}> <td><table width=100%>\n";

local %inall;
local @all = split(/\s+/, $rule->{'all'}->{'value'});
map { $inall{$_}++ } @all;
print "<tr> <td rowspan=4 valign=top><b>$self->{'text_hash'}->{'server_tables'}</b></td>\n";
print "<td rowspan=4><select multiple size=6 name=tables>\n";
foreach $t ($self->unique(@{$self->{'nis_tables_array'}}, @all)) {
	printf "<option value=%s %s>%s\n",
		$t, $inall{$t} ? 'selected' : '', $t;
	}
print "</select></td>\n";

print "<td><b>$self->{'text_hash'}->{'server_dns'}</b></td>\n";
printf "<td><input type=radio name=b value='-b' %s> %s\n",
	$var->{'B'}->{'value'} eq '-b' ? 'checked' : '', $self->{'text_hash'}->{'yes'};
printf "<input type=radio name=b value='' %s> %s</td> </tr>\n",
	$var->{'B'}->{'value'} eq '-b' ? '' : 'checked', $self->{'text_hash'}->{'no'};

print "<tr> <td><b>$self->{'text_hash'}->{'server_push'}</b></td>\n";
printf "<td><input type=radio name=nopush value='\"\"' %s> %s\n",
	$var->{'NOPUSH'}->{'value'} eq 'true' ? '' : 'checked', $self->{'text_hash'}->{'yes'};
printf "<input type=radio name=nopush value=true %s> %s</td> </tr>\n",
	$var->{'NOPUSH'}->{'value'} eq 'true' ? 'checked' : '', $self->{'text_hash'}->{'no'};

print "<tr> <td><b>$self->{'text_hash'}->{'server_dir'}</b></td>\n";
printf "<td><input name=dir size=30 value='%s'> %s</td> </tr>\n",
	$var->{'DIR'}->{'value'}, $self->file_chooser_button("dir", 0);

print "<tr> <td><b>$self->{'text_hash'}->{'server_pwdir'}</b></td>\n";
printf "<td><input name=pwdir size=30 value='%s'> %s</td> </tr>\n",
	$var->{'PWDIR'}->{'value'}, $self->file_chooser_button("pwdir", 0);
}

# parse_server_config()
# Parse and save the NIS server options
sub parse_server_config
{
	my $self = shift;

local ($var, $rule) = $self->parse_yp_makefile();
$self->{'in_hash'}->{'domain_def'} || $self->{'in_hash'}->{'domain'} =~ /^[A-Za-z0-9\.\-\_]+$/ ||
	$self->error($self->text('server_edomain', $self->{'in_hash'}->{'domain'}));
if ($self->{'in_hash'}->{'boot'} && $self->{'in_hash'}->{'domain_def'}) {
	$self->error($self->{'text_hash'}->{'server_ebootdom'});
	}
$self->{'in_hash'}->{'type'} || gethostbyname($self->{'in_hash'}->{'slave'}) || $self->error($self->{'text_hash'}->{'server_eslave'});
-d $self->{'in_hash'}->{'dir'} || $self->error($self->{'text_hash'}->{'server_edir'});
-d $self->{'in_hash'}->{'pwdir'} || $self->error($self->{'text_hash'}->{'server_epwdir'});
$self->update_makefile($var->{'NOPUSH'}, $self->{'in_hash'}->{'nopush'});
$self->update_makefile($var->{'B'}, $self->{'in_hash'}->{'b'});
$self->update_makefile($rule->{'all'}, join(" ", split(/\0/, $self->{'in_hash'}->{'tables'})), "");
$self->update_makefile($var->{'DIR'}, $self->{'in_hash'}->{'dir'});
$self->update_makefile($var->{'PWDIR'}, $self->{'in_hash'}->{'pwdir'});
$self->flush_file_lines();

if ($self->{'in_hash'}->{'domain_def'}) {
	unlink("/etc/defaultdomain");
	$self->system_logged("domainname \"\" >/dev/null 2>&1");
	}
else {
	local $old = `domainname`; chop($old);
	$self->open_tempfile(DOM, ">/etc/defaultdomain");
	$self->print_tempfile(DOM, "$self->{'in_hash'}->{'domain'}\n");
	$self->close_tempfile(DOM);
	$self->system_logged("domainname \"$self->{'in_hash'}->{'domain'}\" >/dev/null 2>&1");
	if ($self->{'in_hash'}->{'boot'}) {
		# Create the domain directory
		mkdir("/var/yp/$self->{'in_hash'}->{'domain'}", 0755);
		$self->system_logged("rm -f /var/yp/*.time");	# force a remake
		}
	}

if ($self->{'in_hash'}->{'type'}) {
	# Master server
	delete($self->{'config_hash'}->{'slave'});
	$self->apply_table_changes()
		if (!$self->{'in_hash'}->{'domain_def'} && -d "/var/yp/$self->{'in_hash'}->{'domain'}");
	}
else {
	local $temp = $self->transname();
	open(TEMP, ">$temp");
	print TEMP "n\ny\n";
	close(TEMP);
	$self->{'out'} = $self->backquote_logged("/usr/sbin/ypinit -s $self->{'in_hash'}->{'slave'} <$temp 2>&1");
	unlink($temp);
	if ($?) { $self->error("<tt>$self->{'out'}</tt>"); }
	$self->{'config_hash'}->{'slave'} = $self->{'in_hash'}->{'slave'};
	}
$self->write_file("$self->{'module_config_directory'}/config", \%{$self->{'config_hash'}});
$self->system_logged("(/usr/lib/netsvc/yp/ypstop || /etc/init.d/yp stop) >/dev/null 2>&1");
$self->system_logged("(/usr/lib/netsvc/yp/ypstart || /etc/init.d/yp start) >/dev/null 2>&1");
}

# get_server_mode()
# Returns 0 if the NIS server is inactive, 1 if active as a master, or 2 if
# active as a slave.
sub get_server_mode
{
	my $self = shift;

local $dom = `domainname`; chop($dom);
return !$dom ? 0 : $self->{'config_hash'}->{'slave'} ? 2 : 1;
}

# parse_yp_makefile()
# Returns hashes of makefile variables and rules
sub parse_yp_makefile
{
	my $self = shift;

# First parse joined lines
local $lnum = 0;
local (@lines, $llast);
open(MAKE, $self->{'yp_makefile'});
while(<MAKE>) {
	s/\r|\n//g;
	local $slash = (s/\\$//);
	s/#.*$//;
	if ($llast) {
		$llast->{'value'} .= " $_";
		$llast->{'eline'} = $lnum;
		}
	else {
		push(@lines, { 'value' => $_,
			       'line' => $lnum,
			       'eline' => $lnum });
		}
	$llast = $slash ? $lines[$#lines] : undef;
	$lnum++;
	}
close(MAKE);

# Then look for variables and rules
local ($i, %var, %rule);
for($i=0; $i<@lines; $i++) {
	if ($lines[$i]->{'value'} =~ /^\s*(\S+)\s*=\s*(.*)/) {
		# Found a variable
		$var{$1} = { 'name' => $1,
			     'value' => $2,
			     'type' => 0,
			     'line' => $lines[$i]->{'line'},
			     'eline' => $lines[$i]->{'eline'} };
		}
	elsif ($lines[$i]->{'value'} =~ /^\s*(\S+):\s*(.*)/) {
		# Found a makefile rule
		$rule{$1} = { 'name' => $1,
			      'value' => $2,
			      'type' => 1,
			      'code' => $lines[$i+1]->{'value'},
			      'line' => $lines[$i]->{'line'},
			      'eline' => $lines[$i+1]->{'eline'} };
		$i++;
		}
	}
return ( \%var, \%rule );
}

# expand_vars(string, &vars)
sub expand_vars
{
	my $self = shift;

local $rv = $_[0];
while($rv =~ /^(.*)\$\(([A-Za-z0-9_]+)\)(.*)$/) {
	$rv = $1.$_[1]->{$2}->{'value'}.$3;
	}
return $rv;
}

# update_makefile(&old, value, [value]);
sub update_makefile
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'yp_makefile'});
local @n;
if ($_[0]->{'type'} == 0) {
	@n = ( "$_[0]->{'name'} = $_[1]" );
	}
else {
	@n = ( "$_[0]->{'name'}: $_[1]", $_[2] );
	}
splice(@$lref, $_[0]->{'line'}, $_[0]->{'eline'} - $_[0]->{'line'} + 1, @n);
}

# apply_table_changes()
# Do whatever is necessary for the table text files to be loaded into
# the NIS server
sub apply_table_changes
{
	my $self = shift;

$self->system_logged("(cd /var/yp ; /usr/ccs/bin/make) >/dev/null 2>&1 </dev/null");
}

# list_nis_tables()
# Returns a list of structures of all NIS tables
sub list_nis_tables
{
	my $self = shift;

local @rv;
local ($var, $rule) = $self->parse_yp_makefile();
local $dom = `domainname`; chop($dom);
local @all = split(/\s+/, $rule->{'all'}->{'value'});
foreach $t (@all) {
	local $table = { 'table' => $t,
		         'index' => scalar(@rv),
		         'domain' => $dom };
	local $rt = $rule->{"$t.time"};
	local @files = split(/\s+/, $rt->{'value'});
	@files = map { $self->expand_vars($_, $var) } @files;
	$table->{'files'} = \@files;
	$table->{'type'} = $t eq 'passwd' && @files > 1 ? 'passwd_shadow' :
			   $t;
	push(@rv, $table);
	}
return @rv;
}

sub show_server_security
{
	my $self = shift;

}

sub parse_server_security
{
	my $self = shift;

$self->system_logged("(/usr/lib/netsvc/yp/ypstop || /etc/init.d/yp stop) >/dev/null 2>&1");
$self->system_logged("(/usr/lib/netsvc/yp/ypstart || /etc/init.d/yp start) >/dev/null 2>&1");
}

sub extra_config_files
{
	my $self = shift;

local @rv = ( "/etc/defaultdomain" );
local $conf = $self->get_client_config();
if ($conf->{'domain'}) {
	push(@rv, "$self->{'binding_dir'}/$conf->{'domain'}/ypservers");
	}
return @rv;
}


1;

