#!/usr/bin/perl
package nis::SwitchLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'nis::SwitchLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for parsing nsswitch.conf

$self->{'nsswitch_config_file'} = $self->{'config_hash'}->{'nsswitch_conf'} || "/etc/nsswitch.conf";

1;

1;
		return $self;
}

# get_nsswitch_config()
# Returns an array ref of information from nsswitch.conf
sub get_nsswitch_config
{
	my $self = shift;

if (!defined(@{$self->{'get_nsswitch_cache_array'}})) {
	@{$self->{'get_nsswitch_cache_array'}} = ( );
	local $lnum = 0;
	open(CONF, $self->{'nsswitch_config_file'});
	while(<CONF>) {
		s/\r|\n//g;
		s/#.*$//;
		if (/^\s*(\S+)\s*:\s*(.*)/) {
			# Found a switch config file
			local $switch = { 'name' => $1,
					  'line' => $lnum };
			local $servs = $2;
			local @srcs = ( );
			while($servs =~ /\S/) {
				if ($servs =~ /^\s*\[([^\]]*)\](.*)/) {
					# Actions for some source
					$servs = $2;
					foreach $av (split(/\s+/, $1)) {
						local ($a, $v) = split(/=/,$av);
						$srcs[$#srcs]->{lc($a)} =lc($v);
						}
					}
				elsif ($servs =~ /^\s*(\S+)(.*)/) {
					# A source
					push(@srcs, { 'src' => $1 });
					$servs = $2;
					}
				}
			$switch->{'srcs'} = \@srcs;
			push(@{$self->{'get_nsswitch_cache_array'}}, $switch);
			}
		$lnum++;
		}
	close(CONF);
	}
return \@{$self->{'get_nsswitch_cache_array'}};
}

# save_nsswitch_config(&switch)
# Update one service
sub save_nsswitch_config
{
	my $self = shift;

local ($switch) = @_;
local $lref = $self->read_file_lines($self->{'nsswitch_config_file'});
local $line = "$switch->{'name'}:";
foreach my $s (@{$switch->{'srcs'}}) {
	$line .= " ".$s->{'src'};
	local @acts;
	foreach my $st (keys %$s) {
		if ($st ne "src") {
			push(@acts, uc($st)."=".$s->{$st});
			}
		}
	if (@acts) {
		$line .= " [".join(" ", @acts)."]";
		}
	}
$lref->[$switch->{'line'}] = $line;
$self->flush_file_lines($self->{'nsswitch_config_file'});
}

# list_switch_sources()
# Returns a list of valid nsswitch.conf sources for this OS, and a map from
# sources to allowed services
sub list_switch_sources
{
	my $self = shift;

if ($self->{'gconfig_hash'}->{'os_type'} =~ /-linux$/) {
	# All Linux variants
	return ( [ 'files', 'nisplus', 'nis', 'compat', 'dns', 'db',
		   'hesiod', 'ldap' ],
		 { 'dns' => [ 'hosts' ],
		   'compat' => [ 'passwd', 'shadow', 'group' ] } );
	}
elsif ($self->{'gconfig_hash'}->{'os_type'} eq 'solaris' && $self->{'gconfig_hash'}->{'os_version'} < 8) {
	# Older Solaris
	return ( [ 'files', 'nis', 'nisplus', 'compat', 'dns' ],
		 { 'dns' => [ 'hosts' ],
		   'compat' => [ 'passwd', 'group' ] } );
	}
elsif ($self->{'gconfig_hash'}->{'os_type'} eq 'solaris' && $self->{'gconfig_hash'}->{'os_version'} >= 8) {
	# Newer Solaris
	return ( [ 'files', 'nis', 'nisplus', 'compat', 'dns', 'ldap',
		   'user', 'xfn' ],
		 { 'dns' => [ 'hosts' ],
		   'compat' => [ 'passwd', 'group' ],
		   'user' => [ 'printers' ],
		   'xfn' => [ 'printers' ] } );
	}
elsif ($self->{'gconfig_hash'}->{'os_type'} eq 'aix') {
	# IBM AIX
	return ( [ 'files', 'nis', 'nisplus', 'compat', 'dns', 'ldap',
		   'user', 'xfn' ],
		 { 'dns' => [ 'hosts' ],
		   'compat' => [ 'passwd', 'group' ],
		   'user' => [ 'printers' ],
		   'xfn' => [ 'printers' ] } );
	}
elsif ($self->{'gconfig_hash'}->{'os_type'} eq 'unixware') {
	# All Linux variants
	return ( [ 'files', 'dns', 'nis', 'nisplus' ],
		 { 'dns' => [ 'hosts' ] } );
	}
else {
	# Punt!
	return ( [ 'files', 'dns', 'nis', 'nisplus' ] );
	}
}

sub list_switch_statuses
{
	my $self = shift;

return ( 'success', 'notfound', 'unavail', 'tryagain' );
}

sub list_switch_actions
{
	my $self = shift;

return ( 'return', 'continue' );
}


1;

