#!/usr/bin/perl
package phpini::PhpiniLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'phpini::PhpiniLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for managing the PHP configuration file
# XXX install_check.pl
# XXX backup_config.pl

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();

1;

1;
		return $self;
}

# get_config([file])
# Returns an array ref of PHP configuration directives from some file 
sub get_config
{
	my $self = shift;

local ($file) = @_;
$file ||= $config{'php_ini'};
if (!defined($self->{'get_config_cache_hash'}->{$file})) {
	local @rv = ( );
	local $lnum = 0;
	local $section;
	open(CONFIG, $file);
	while(<CONFIG>) {
		s/\r|\n//g;
		s/\s+$//;
		if (/^(;?)\s*(\S+)\s*=\s*"(.*)"/ ||
		    /^(;?)\s*(\S+)\s*=\s*'(.*)'/ ||
		    /^(;?)\s*(\S+)\s*=\s*(.*)/) {
			# Found a variable
			push(@rv, { 'name' => $2,
				    'value' => $3,
				    'enabled' => !$1,
				    'line' => $lnum,
				    'file' => $file,
				    'section' => $section,
				  });
			}
		elsif (/^\[(.*)\]/) {
			# A new section
			$section = $1;
			}
		$lnum++;
		}
	close(CONFIG);
	$self->{'get_config_cache_hash'}->{$file} = \@rv;
	}
return $self->{'get_config_cache_hash'}->{$file};
}

# find(name, &config, [disabled-mode])
sub find
{
	my $self = shift;

local ($name, $conf, $mode) = @_;
local @rv = grep { lc($_->{'name'}) eq lc($name) &&
		   ($mode == 0 && $_->{'enabled'} ||
		    $mode == 1 && !$_->{'enabled'} ||
		    $mode == 2) } @$conf;
return wantarray ? @rv : $rv[0];
}

sub find_value
{
	my $self = shift;

local @rv = map { $_->{'value'} } $self->find(@_);
return $rv[0];
}

# save_directive(&config, name, [value], [newsection])
# Updates a single entry in the PHP config file
sub save_directive
{
	my $self = shift;

local ($conf, $name, $value, $newsection) = @_;
$newsection ||= "PHP";
local $old = $self->find($name, $conf, 0);
local $cmt = $self->find($name, $conf, 1);
local $lref;
local $newline = $name." = ".
		 ($value !~ /\s/ ? $value :
		  $value =~ /"/ ? "'$value'" : "\"$value\"");
if (defined($value) && $old) {
	# Update existing value
	$lref = $self->read_file_lines($old->{'file'});
	$lref->[$old->{'line'}] = $newline;
	$old->{'value'} = $value;
	}
elsif (defined($value) && !$old && $cmt) {
	# Update existing commented value
	$lref = $self->read_file_lines($cmt->{'file'});
	$lref->[$cmt->{'line'}] = $newline;
	$cmt->{'value'} = $value;
	$cmt->{'enabled'} = 1;
	}
elsif (defined($value) && !$old && !$cmt) {
	# Add a new value, at the end of the section
	local $last;
	foreach my $c (@$conf) {
		if ($c->{'section'} eq $newsection) {
			$last = $c;
			}
		}
	if ($last) {
		# Found last value in the section - add after it
		$lref = $self->read_file_lines($last->{'file'});
		splice(@$lref, $last->{'line'}+1, 0, $newline);
		$self->renumber($conf, $last->{'line'}, 1);
		push(@$conf, { 'name' => $name,
			       'value' => $value,
			       'enabled' => 1,
			       'file' => $last->{'file'},
			       'line' => $last->{'line'}+1,
			       'section' => $newsection,
			     });
		}
	else {
		$self->error("Could not find any values in section $newsection");
		}
	}
elsif (!defined($value) && $old && $cmt) {
	# Totally remove a value
	$lref = $self->read_file_lines($old->{'file'});
	splice(@$lref, $old->{'line'}, 1);
	@$conf = grep { $_ ne $old } @$conf;
	$self->renumber($conf, $old->{'line'}, -1);
	}
elsif (!defined($value) && $old && !$cmt) {
	# Turn a value into a comment
	$lref = $self->read_file_lines($old->{'file'});
	$old->{'enabled'} = 0;
	$lref->[$old->{'line'}] = "; ".$lref->[$old->{'line'}];
	}
}

sub renumber
{
	my $self = shift;

local ($conf, $line, $oset) = @_;
foreach my $c (@$conf) {
	$c->{'line'} += $oset if ($c->{'line'} > $line);
	}
}

# can_php_config(file)
# Returns 1 if some config file can be edited
sub can_php_config
{
	my $self = shift;

local ($file) = @_;
return $self->indexof($file, map { $_->[0] } $self->list_php_configs()) >= 0 ||
       $self->{'access_hash'}->{'anyfile'};
}

# list_php_configs()
# Returns 1 list of allowed config files and descriptions
sub list_php_configs
{
	my $self = shift;

local @rv;
if ($self->{'access_hash'}->{'global'}) {
	foreach my $ai (split(/\t+/, $self->{'config_hash'}->{'php_ini'})) {
		local ($f, $d) = split(/=/, $ai);
		push(@rv, [ $f, $d || $self->{'text_hash'}->{'file_global'} ]);
		}
	}
foreach my $ai (split(/\t+/, $self->{'access_hash'}->{'php_inis'})) {
	local ($f, $d) = split(/=/, $ai);
	push(@rv, [ $f, $d || $f ]);
	}
return @rv;
}

# onoff_radio(name)
# Returns a field for editing a binary configuration value
sub onoff_radio
{
	my $self = shift;

local ($name) = @_;
local $v = $self->find_value($name, $self->{'conf'});
return $self->ui_radio($name, lc($v) eq "on" || lc($v) eq "true" ||
			lc($v) eq "yes" || $v eq "1" ? "On" : $v ? "Off" : "",
		 [ !$v ? ( [ "", $self->{'text_hash'}->{'default'} ] ) : ( ),
		   [ "On", $self->{'text_hash'}->{'yes'} ],
		   [ "Off", $self->{'text_hash'}->{'no'} ] ]);
}

# graceful_apache_restart()
# Signal a graceful Apache restart, to pick up new php.ini settings
sub graceful_apache_restart
{
	my $self = shift;

if ($self->foreign_installed("apache")) {
	$self->foreign_require("apache", "apache-lib.pl");
	if ($main::apache_object->is_apache_running() &&
	    $main::apache_object->{'httpd_modules_hash'}->{'core'} >= 2 &&
	    $self->has_command($main::apache_object->{'config_hash'}->{'apachectl_path'})) {
		$self->clean_environment();
		$self->system_logged("$main::apache_object->{'config_hash'}->{'apachectl_path'} graceful >/dev/null 2>&1");
		$self->reset_environment();
		}
	}
}


1;

