#!/usr/bin/perl
package plugins::Pluginphpmyadmin;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'plugins::Pluginphpmyadmin');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# plugin-phpmyadmin.pl
# Functions for phpMyAdmin configure
 
1;
1;
		return $self;
}

# &phpmyadmin_depends()
# return hash of phpmyadmin depends
sub phpmyadmin_depends
{
	my $self = shift;

 my @deps;
 return {'apache' => [$self->configured('apache'), 'apache'], 'php' => [$self->check_php_apache(), 'apache'], 'mysql' => [$self->configured('mysql'), 'mysql']};
}

# &configure_phpmyadmin(tar, version)
# configure SquirrelMail plugin
sub configure_phpmyadmin
{
	my $self = shift;

 # print configure message
 $self->msg_conftitle($self->{'text_hash'}->{'plugin_phpmyadmin'});
 
 # tar file name and version
 my ($file, $version) = @_;
 
 # get plugin directorys
 my ($dest, $subdir, $plugin_dir, $apache_user, $apache_group) = $self->get_install_dir('phpmyadmin');
 
 # untar file
 my $out = $self->backquote_logged("tar -zxf $file -C $dest 2>&1 </dev/null");
 $self->error($out) if $?;
 (undef, $out) = $self->unlink_file($file);
 $self->error($out) if $out;
 
 # prepare plugin directory
 my $f;
 opendir (DIR, $dest) || $self->error($self->text('error_open_dir',$dest));
 while (defined ($f = readdir (DIR)))
 {
  next if $f =~ /^\.\.?$/;
  if (-d "$dest/$f" && $f =~ /^phpmyadmin-$version/i)
  {
   $out = $self->backquote_logged("mv $dest/$f $plugin_dir 2>&1 </dev/null");
   $self->error($out) if $?;
   last;
  }
 }
 closedir (DIR);
 
 # make config directory
 $out = $self->backquote_logged("mkdir $plugin_dir/config 2>&1 </dev/null");
 
 # set owners & permissions
 $out = $self->backquote_logged("chown -R $apache_user.$apache_group $plugin_dir 2>&1 </dev/null");
 $self->error($out) if $?;
 $out = $self->backquote_logged("chmod -R 755 $plugin_dir 2>&1 </dev/null");
 $self->error($out) if $?;
 
 # set mysql extension
 $self->set_mysql_extension() if (($self->{'gconfig_hash'}->{'os_type'} eq 'trustix-linux') || $self->{'config_hash'}->{'ztl_mode'});
 
 # add apache per-directory and alias
 $self->apache_install_conf($subdir, $plugin_dir); 
 
 if ($main::apache_object->is_apache_running())
 {
  # restart apache
  my $out = $main::apache_object->restart_apache();
  $self->error($out) if ($out);
 }
 
 # configure endusers mailboxes
 $self->foreign_require('enduser_cf','enduser_cf-lib.pl');
 $self->read_file("$main::enduser_cf_object->{'config_hash'}->{'usermin_dir'}/config", \%{$self->{'gconfig_hash'}});
 $self->{'gconfig_hash'}->{'phpmyadmin'} = 1;
 $self->write_file("$main::enduser_cf_object->{'config_hash'}->{'usermin_dir'}/config", \%{$self->{'gconfig_hash'}});
 
 # print done message
 $self->msg_done();
}

# &update_phpmyadmin(tar,version)
# update SquirrelMail plugin
sub update_phpmyadmin
{
	my $self = shift;

 # tar file name and version
 my ($file,$version) = @_;
 
 # get plugin directorys
 my ($dest, $subdir, $plugin_dir, $apache_user, $apache_group) = $self->get_install_dir('phpmyadmin');
 
 # log file name
 my ($log_file, $log) = "$self->{'module_config_directory'}\/phpmyadmin.log";
 
 # read log
 $self->read_file($log_file, $log);
 
 my $out;
 if (-d $log->{'per-dir'})
 {
  # update plugin
  # print configure message
  $self->msg_conftitle($self->{'text_hash'}->{'plugin_phpmyadmin'});
  
  # untar file
  $out = $self->backquote_logged("tar -zxf $file -C $dest 2>&1 </dev/null");
  $self->error($out) if $?;
  (undef, $out) = $self->unlink_file($file);
  print $self->text('update_error',$out) if $out;
  
  # prepare plugin directory
  my $f;
  opendir (DIR, $dest) || $self->error($self->text('error_open_dir',$dest));
  while (defined ($f = readdir (DIR)))
  {
   next if $f =~ /^\.\.?$/;
   if (-d "$dest/$f" && $f =~ /^phpmyadmin-([0-9\.]+)/i) #/^$subdir/
   {
    # restore old data
    # restore config.php
    $out = $self->backquote_logged("cp --reply=yes $log->{'per-dir'}/config.inc.php $dest/$f/ 2>&1 </dev/null");
	 if ($?)
	 {
	  $self->unlink_file("$dest/$f");
     $self->error($out);
	 }
    
    # delete old directory
	 $out = $self->unlink_file($log->{'per-dir'});
	 if ($?)
	 {
	  $self->unlink_file("$dest/$f");
     $self->error($out);
	 }
	 
	 # rename directory
    $out = $self->backquote_logged("mv $dest/$f $plugin_dir 2>&1 </dev/null");
	 if ($?)
	 {
	  $self->unlink_file("$dest/$f");
     $self->error($out);
	 }
	 
	 # delete directory
    $self->unlink_file("$dest/$f");
    last;
   }
  }
  closedir (DIR);
   
  # make config directory
  $out = $self->backquote_logged("mkdir $plugin_dir/config 2>&1 </dev/null");
 
  # set owners & permissions
  $out=$self->backquote_logged("chown -R $apache_user.$apache_group $plugin_dir 2>&1 </dev/null");
  print $self->text('update_error',$out) if $?;
  $out=$self->backquote_logged("chmod -R 755 $plugin_dir 2>&1 </dev/null");
  print $self->text('update_error',$out) if $?;
  
  # update apache per-directory and alias, restart apache
  if ($plugin_dir ne $log->{'per-dir'})
  {
   # del apache per-dir & aliases
   $self->apache_uninstall_conf($log->{'per-dir'}, $log->{'alias'});
   $self->apache_install_conf($subdir, $plugin_dir);
  }
  
  if ($main::apache_object->is_apache_running())
  {
   # restart apache
   my $out = $main::apache_object->restart_apache();
   $self->error($out) if ($out);
  }
  
  # print done message
  $self->msg_done();
 }
 else 
 {
  # install plugin
  $self->configure_phpmyadmin($file, $version);
 }
}

# &configure_phpmyadmin(tar)
# uninstall SquirrelMail plugin
sub uninstall_phpmyadmin
{
	my $self = shift;

 # print uninstall message
 $self->msg_conftitle_uninstall($self->{'text_hash'}->{'plugin_phpmyadmin'});
 
 # log file name
 my ($log_file, $log) = "$self->{'module_config_directory'}\/phpmyadmin.log";
 
 # read log
 $self->read_file($log_file, $log);
 
 # delete plugin directory
 my $out = $self->unlink_file("$log->{'per-dir'}") if (-d $log->{'per-dir'});
 print $self->text('uninstall_error',$out) if $?;
 
 # del apache per-dir & aliases
 $self->apache_uninstall_conf($log->{'per-dir'}, $log->{'alias'});
 
 if ($main::apache_object->is_apache_running())
 {
  # restart apache
  my $out = $main::apache_object->restart_apache();
  $self->error($out) if ($out);
 }
   
 # add confmaster log
 $self->write_postinstall_log('phpmyadmin','uninstall',1,1);
 
 # configure endusers mailboxes
 $self->foreign_require('enduser_cf','enduser_cf-lib.pl');
 $self->read_file("$main::enduser_cf_object->{'config_hash'}->{'usermin_dir'}/config", \%{$self->{'gconfig_hash'}});
 $self->{'gconfig_hash'}->{'phpmyadmin'} = 0;
 $self->write_file("$main::enduser_cf_object->{'config_hash'}->{'usermin_dir'}/config", \%{$self->{'gconfig_hash'}});
 
 # print done message
 $self->msg_done();
}

# &get_phpmyadmin_version()
# return plugin version
sub get_phpmyadmin_version
{
	my $self = shift;

 # log file name
 my ($log_file, $log, $f)="$self->{'module_config_directory'}\/phpmyadmin.log";
 
 # read log
 $self->read_file($log_file,$log);
 
 if (-d $log->{'per-dir'})
 {
  opendir (DIR, $log->{'per-dir'}) || $self->error($self->text('error_open_dir', $log->{'per-dir'}));
  while (defined ($f = readdir (DIR)))
  {
   next if $f =~ /^\.\.?$/;
   if (-e "$log->{'per-dir'}/$f" && $f =~ /^release-date-([0-9\.]+)/i) { return $1; }
  }
 }
 return 0;
}

# &set_mysql_extension()
# enable mysql extension in the php.ini
sub set_mysql_extension
{
	my $self = shift;

 my $conf_file = "$main::apache_object->{'config_hash'}->{'httpd_dir'}/php.ini";
 $self->lock_file($conf_file);
 my $conf = $self->read_file_lines($conf_file);
 foreach (@{$conf})
 { 
  $_ =~ s/;// if ($_ =~ /^\s*;+?\s*extension\s*=\s*mysqli?.so/)
 }
 $self->flush_file_lines();
 $self->unlock_file($conf_file);
}


1;

