#!/usr/bin/perl
package plugins::PluginsLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'plugins::PluginsLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# plugins-lib.pl
# Common functions for plugins module

# main part
&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

# supported plugins list
@{$self->{'plugin_list_array'}} = ('squirrelmail','phpmyadmin');

# plugin funcs require
foreach (@{$self->{'plugin_list_array'}})
{
 {
    my $pkg = $self->lib_to_pkg("plugin-$_.pl", "plugins");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
}

1;
1;
		return $self;
}

# &install_plugin (plugin, [upgrade])
# install or upgrade plugin
sub install_plugin
{
	my $self = shift;

 # plugin name
 my $pl = $_[0];
 
 # base update url
 my $update_url = $self->{'config_hash'}->{'base_update_url'};
 
 # print install header
 if ($_[1]) { print "<tr $self->{'tb'}><td colspan=2><b>$self->{'text_hash'}->{'updating_title'}</b></td></tr>\n"; }
 else { print "<tr $self->{'tb'}><td colspan=2><b>$self->{'text_hash'}->{'install_title'}</b></td></tr>\n"; }
 print "<tr $self->{'cb'}><td colspan=2><pre>";

 # get templete file to download
 my $file = $self->transname();
 
 # find latest version of plugin
 my $progress_callback_url = $self->convert_osdn_url($update_url);
 my ($host, $port, $page, $error, $ssl) = $self->parse_http_url($progress_callback_url);
 $self->http_download($host, $port, $page, $file, \$error, undef, $ssl);
 $self->inst_error($error, $file) if $error;
 
 # get tar version
 my $version = $self->get_tar_version($file,$pl); 
 unless ($version)
 {
  # can't get new version
  $self->inst_error($self->text('get_version_error', $pl));
 }

 # get current  version
 $self->{'func'} = "get_".$pl."_version";
 my $dur_version = do { my $fn = $self->{'func'}; $self->$fn()};
 
 # compare versions
 my @ver1=split(/\./,$version);
 my $v1=$ver1[0]*1000000+$ver1[1]*1000+$ver1[2];
 my @ver2=split(/\./,$dur_version);
 my $v2 = $ver2[0]*1000000+$ver2[1]*1000+$ver2[2];
 if ($v1 == $v2 || $v2 >= $v1)
 {
  # use last version
  $self->inst_error($self->text('upgrade_elatest',$dur_version, $pl));
 }
 
 # downloading tar.gz file
 $update_url = "$update_url/$pl-$version.tar.gz"; 
 my $progress_callback_url = $self->convert_osdn_url($update_url);
 ($host, $port, $page, $ssl) = $self->parse_http_url($progress_callback_url);
 $self->http_download($host, $port, $page, $file, \$error,'progress_callback', $ssl);
 $self->inst_error($error, $file) if $error; 
 print "</pre></td></tr>\n";
 
 # print configure header
 print "<tr $self->{'tb'}><td colspan=2><b>$self->{'text_hash'}->{'configure_title'}</b></td></tr>\n";
 print "<tr $self->{'cb'}><td colspan=2><pre>";
 
 # add confmaster log
 $self->write_postinstall_log($pl,'download',1);
 
 if ($_[1]) 
 {
  # update plugin
  $self->{'func'} = "update_".$pl;
 }
 else
 {
  # configure plugin
  $self->{'func'} = "configure_".$pl;
 }
 do { my $fn = $self->{'func'}; $self->$fn($file, $version)};
 
 # add confmaster log
 $self->write_postinstall_log($pl,'configure',1);
 $self->write_postinstall_log($pl,'complete',1);
 print "</pre></td></tr>\n";
}

# &uninstall_plugin (plugin)
# uninstall_plugin
sub uninstall_plugin
{
	my $self = shift;

 # plugin name
 my $pl = $_[0];

 # print uninstall header
 print "<tr $self->{'tb'}><td colspan=2><b>$self->{'text_hash'}->{'uninstalling_title'}</b></td></tr>\n";
 print "<tr $self->{'cb'}><td colspan=2><pre>";
 
 # uninstall plugin
 $self->{'func'} = "uninstall_".$pl;
 do { my $fn = $self->{'func'}; $self->$fn($self->{'file'})};
 
 # add confmaster log
 $self->write_postinstall_log($pl,'complete',1,1);
 print "</pre></td></tr>\n";
}

# &get_tar_version(tar,plugin)
# return plugin version in tar & delete temp file
sub get_tar_version
{
	my $self = shift;

 my ($file,$pl,$version) = @_;
 open(FILE, $_[0]);
 while(<FILE>) 
 {
  if (/$pl-([0-9\.]+)\.tar\.gz/i) 
  {
   $version = $1;
   last;
  }
 }
 close(FILE);
 $self->unlink_file($file);
 return $version;
}

# &print_servers_table();
# print plugin icons
sub print_plugins_table
{
	my $self = shift;

 # get ACL
 my %access = $self->get_module_acl();
 
 print "<table cellpadding=10>\n";
 foreach (@{$self->{'plugin_list_array'}})
 {
  my $flag = $self->get_configured_flag($_);
  my $descr = $self->{'text_hash'}->{'descr_'.$_};
  if ($flag)
  {
   # plugin was installed
	my $icon = "$_.gif";
	my $actions = "<a href=\"configure.cgi?pl=".$_."&confirm_up=1\" class='menu'>$self->{'text_hash'}->{'plugin_upgrade'}</a> | <a href=\"configure.cgi?pl=".$_."&confirm_un=1\" class='menu'>$self->{'text_hash'}->{'plugin_uninstall'}</a>" if $access{'install'};
	
	# get plugin link
   $self->read_file("$self->{'module_config_directory'}\/$_.log", $self->{'log'});
	my $pl_link = "http://$ENV{'SERVER_NAME'}/$self->{'log'}->{'alias'}/";
	
	print "<tr><td class=\"infotitle\" width=60><a href=\"$pl_link\"><img width=48 height=48 src=\"images/$icon\" border=\"0\" />&nbsp; <font size=+1>".$self->{'text_hash'}->{"plugin_$_"}."</font></a><td>$descr</td>\n";
	print "<td width=30%>$actions</td></tr>\n";
  }
  elsif ($access{'install'})
  {
   # plugin was't installed
	my $icon = "${_}_off.gif";
	my $actions = "<a href=\"configure.cgi?pl=".$_."&confirm=1\" class='menu'>$self->{'text_hash'}->{'plugin_install'}</a>";
	
	print "<tr><td class=\"infotitle\" width=60><img width=48 height=48 src=\"images/$icon\" border=\"0\" />&nbsp; <font size=+1>".$self->{'text_hash'}->{"plugin_$_"}."</font><td>$descr</td>\n";
	print "<td width=30%>$actions</td></tr>\n";
  }
 }
 print "</table>\n";
}

# &set_configured_flag(plugin)
# set config value in 1 for plugin
sub set_configured_flag
{
	my $self = shift;

 $self->{'config_hash'}->{"configured_$_[0]"}=1;
 $self->save_module_config();
}

# &del_configured_flag(plugin)
# set config value in 0 for plugin
sub del_configured_flag
{
	my $self = shift;

 $self->{'config_hash'}->{"configured_$_[0]"} = 0;
 $self->save_module_config();
}

# &get_configured_flag(plugin)
# get config value for plugin
sub get_configured_flag
{
	my $self = shift;

 return $self->{'config_hash'}->{"configured_$_[0]"};
}

# &conf_title($name)
# print name title message
sub msg_conftitle
{
	my $self = shift;

 print $self->text('setup_configure', $_[0]) . "<br>\n";
}

# &conf_title($name)
# print name uninstall message
sub msg_conftitle_uninstall
{
	my $self = shift;

 print $self->text('uninstall_configure', $_[0]) . "<br>\n";
}

# &msg_done();
# print done message
sub msg_done
{
	my $self = shift;

 print "$self->{'text_hash'}->{'index_done'}<br>\n";
}

# &write_postinstall_log(plugin, name, value, [uninstall mode])
# write postinstall log file for plugin in the plugins module directory
sub write_postinstall_log
{
	my $self = shift;

 my $log_file;
 if ($_[3]) { $log_file = "$self->{'module_config_directory'}\/$_[0]_un.log"; }
 else { $log_file = "$self->{'module_config_directory'}\/$_[0].log"; }
 my $log;
 $self->read_file($log_file,$log);
 $log->{$_[1]}=$_[2];
 $self->write_file($log_file,$log);
}

# &print_postinstall_log(plugin,[1])
# print summary information for install or uninstall plugin
# if $_[1] = 1 - uninstall mode, else - install
sub print_postinstall_log
{
	my $self = shift;

 # plugin name
 my $pl = $_[0];
 my ($log_file,$old_log,$log,$temp_file,$temp,@temp1,$out);
 
 # install or uninstall templete & log
 if ($_[1]) 
 { 
  # uninstall templete
  $log_file="$self->{'module_config_directory'}\/$_[0]_un.log";
  $old_log="$self->{'module_config_directory'}\/$_[0].log";
  $temp_file="$self->{'root_directory'}\/$self->{'module_name'}\/template.uninstall"; 
 }
 else 
 { 
  # install templete
  $log_file="$self->{'module_config_directory'}\/$_[0].log";
  $old_log="$self->{'module_config_directory'}\/$_[0]_un.log";
  $temp_file="$self->{'root_directory'}\/$self->{'module_name'}\/template.install"; 
 }
 
 if (-e $temp_file && -e $log_file)
 {
  # print summary header
  print "<tr $self->{'tb'}><td colspan=2 width=70%><b>$self->{'text_hash'}->{'summary_log'}</b></td><tr>\n";
  
  # read log file
  $self->read_file($log_file,$log);
  
  # read templete file
  my $start;
  open(TPL, $temp_file);
  while($_=<TPL>)
  {
   chomp;
   if ($_ =~ /\[$pl\]/) 
   { 
    # start tag
    $start = 1; 
    next;
   }
   elsif ($_ =~ /\[\/$pl\]/) 
   { 
    # end tag
    $start = 0; 
    last;
   }
   if ($start)
   {
    # print summary
    my @rang=split(/:/,$_);
    $rang[1] = $self->trunc_space($rang[1]);
    $rang[2] = $self->trunc_space($rang[2]);
    if ($log->{$rang[1]} == 1 && !$rang[2])
    {
     # main done
     if ($rang[1] eq 'complete')
     {
      print "<tr $self->{'cb'}><td><b>$rang[0]</b></td><td><font color=green><b>DONE</b></font></td></tr>\n";
     }
     else { print "<tr $self->{'cb'}><td>$rang[0]</td><td><font color=green><b>DONE</b></font></td></tr>\n"; }
    }
    elsif (exists $log->{$rang[1]} && $rang[2])
    {
     # advanced log
     print "<tr $self->{'cb'}><td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$rang[0]</td><td><font color=blue>$log->{$rang[1]}</font></td></tr>\n";
    }
    elsif(!exists $log->{$rang[1]} && !$rang[2])
    {
     # main failed
     if ($rang[1] eq 'complete')
     {
      print "<tr $self->{'cb'}><td>$rang[0]</td><td><font color=red><b>FAILED</b></font></td></tr>\n";
     }
     else { print "<tr $self->{'cb'}><td>$rang[0]</td><td><font color=red><b>FAILED</b></font></td></tr>\n"; }
    }
   }
  }
  close (TPL);
 }
 (undef, $out) = $self->unlink_file($old_log) if (-e $old_log);
 $self->error($out) if ($out);
 
 if ($self->{'text_hash'}->{"summary_$pl"} && !$_[1])
 {
  # print advancrd postinstall log
  my $pl_url = "http://$ENV{'SERVER_NAME'}/$log->{'alias'}";
  print "<tr $self->{'cb'}><td colspan=2>";
  print $self->text("summary_$pl", $pl_url, $log->{'per-dir'});
  print "</td></tr>";
 }
}

# &check php apache()
# return 1 if php enable, else return 0
sub check_php_apache
{
	my $self = shift;

 $self->foreign_require('apache', 'apache-lib.pl');
 undef (@{$main::apache_object->{'get_config_cache_array'}});
 my $conf = $main::apache_object->get_config();
 my $conf_dir = "$main::apache_object->{'config_hash'}->{'httpd_dir'}/conf.d";
 foreach (@{$conf})
 {
  return 1 if (($_->{'name'} eq 'LoadModule') && ($_->{'words'}->[0]=~/^php[45]_module$/) && ($_->{'words'}->[1]=~/^.*?libphp[45].so$/) && ($_->{'file'}=~/$conf_dir\/.*?php.conf/));
 }
 return 0;
}

# &is_apache_dir($subdir, $plugin_dir)
# check apache per-directory, apache alias
sub is_apache_dir
{
	my $self = shift;

 my ($subdir, $plugin_dir) = @_;
 
 # get httpd configuration file
 undef (@{$main::apache_object->{'get_config_cache_array'}});
 my $conf = $main::apache_object->get_config();

 # search Directory and Alias option
 my ($dir, $alias);
 foreach (@{$conf})
 {
  if (($_->{'name'} eq 'Directory') && ($_->{'words'}->[0] eq $plugin_dir))
  {
   $dir = 1;
   if (!$alias) { next; }
   else { last ;}
  }
  elsif (($_->{'name'} eq 'Alias') && ($_->{'words'}->[0] eq $subdir))
  {
   if ($_->{'words'}->[1] eq $plugin_dir)
   {
    $alias = 1;
    if (!$dir) { next; }
    else { last ;}
   }
   else { $self->error($self->text('alias_error', $subdir)); }
  }
 }
 return ($dir, $alias);
}

# &apache_install_conf($subdir, $plugin_dir)
# add apache per-directory and alias
# restart apache
sub apache_install_conf
{
	my $self = shift;

 my ($subdir, $plugin_dir) = @_;
 
 # is apache directory?
 my ($per_dir, $alias) = $self->is_apache_dir($subdir, $plugin_dir);
 if (!$per_dir || !$alias)
 {
  # add apache per-directory and alias
  my ($vconf, $v) = $main::apache_object->get_virtual_config();
  $main::apache_object->can_edit_virt($v) || $self->error($main::apache_object->{'text_hash'}->{'virt_ecannot'});
  my $f = $vconf->[0]->{'file'};
  for($self->{'j'}=0; $vconf->[$self->{'j'}]->{'file'} eq $f; $self->{'j'}++) { }
  $self->lock_file($f);
  $main::apache_object->before_changing();
  my $lref = $self->read_file_lines($f);
  my $l = $vconf->[$self->{'j'}-1]->{'eline'};
  unless ($alias)
  {
   # add apache alias
   $l++;
   splice(@$lref, $l, 0, "Alias \/$subdir \"$plugin_dir\"");
   
   # add confmaster log
   $self->write_postinstall_log($subdir, 'alias', $subdir);
  }
  unless ($per_dir)
  {
   # add apache per-directory
   $l++;
   my $newdir = "<Directory \"$plugin_dir\">";
   my $enddir = "</Directory>";
   splice(@$lref, $l, 0, ($newdir, $enddir));
   
   # add confmaster log
   $self->write_postinstall_log($subdir, 'per-dir', $plugin_dir);
  }
  $self->flush_file_lines();
  $self->unlock_file($f);
  $main::apache_object->after_changing(); 
  sleep(1);
 }
}

# &apache_reconfigure($per_dir, $alias)
# del apache per-directory and alias
# restart apache
sub apache_uninstall_conf
{
	my $self = shift;

 my ($per_dir, $alias) = @_;  

 # get apache configuration
 my ($vconf, $v) = $main::apache_object->get_virtual_config();
 $main::apache_object->can_edit_virt($v) || $self->error($main::apache_object->{'text_hash'}->{'virt_ecannot'});
 my $f = $vconf->[0]->{'file'};
 $self->lock_file($f);
 $main::apache_object->before_changing();
 my $lref = $self->read_file_lines($f);
 my $del_per_dir;
 foreach (@{$lref})
 {
  if ($_ =~ /Alias\s+?\/$alias\s+?"$per_dir"/)
  {
   # delete alias
   undef $_;
  
   # add confmaster log
   $self->write_postinstall_log($alias, 'alias', $alias, 1);
	last;
  }
 }
 foreach (@{$lref})
 {
  if ($_ =~ /<\s*?Directory\s+?"$per_dir"\s*?>/)
  {
   # delete per_dir
   $del_per_dir = 1;
   undef $_;
  }
  if ($del_per_dir && $_ =~ /<\s*?\/Directory\s*?>/)
  {
   # delete per_dir end tag
   undef $_;
   
   # add confmaster log
   $self->write_postinstall_log($alias, 'per-dir', $per_dir, 1);
	last;
  }
 }
 $self->flush_file_lines();
 $self->unlock_file($f);
 $main::apache_object->after_changing();
 sleep(1);
}

# &get_install_dir($plugin)
# return array (dest_dir, sub_dir, plugin_dir, user)
sub get_install_dir
{
	my $self = shift;

 my $plugin = $_[0];
 
 # get httpd configuration file
 undef (@{$main::apache_object->{'get_config_cache_array'}});
 my $conf = $main::apache_object->get_config();
 my $apache_user = $main::apache_object->find_directive('User', $conf);
 my $apache_group = $main::apache_object->find_directive('Group', $conf);
 
 my $dest;
 if ($self->{'config_hash'}->{'install_'.$plugin.'_dir'})
 {
  # from module configuration
  $dest = $self->{'config_hash'}->{'install_'.$plugin.'_dir'};
 }
 else
 {
  # search DocumentRoot option
  $dest = $main::apache_object->find_directive('DocumentRoot', $conf);
  $dest =~ /"(.+)"/;
  $dest = $1;
 }
 $self->error($self->text('error_open_dir',$dest)) unless (-d $dest);
 
 $dest =~ s/\/+/\//g;
 my $subdir = $plugin;
 my $plugin_dir = $dest."/".$subdir."/";
 $plugin_dir =~ s/\/+/\//g;
 return ($dest, $subdir, $plugin_dir, $apache_user, $apache_group); 
}

# &inst_error(error)
# delete temp file & print error message
sub inst_error
{
	my $self = shift;

 $self->unlink_file($_[1]) if ($_[1]);
 print "<b>$_[0]</b><br></table>\n";
 $self->ui_print_footer("", $self->{'text_hash'}->{'index_return'});
 exit;
}

# &check_error (warning)
# print warning messages
sub check_error
{
	my $self = shift;

 print "<p class=warn>$_[0]</p>\n";
}

# &trunc_space(string)
# delete spaces from string
sub trunc_space
{
	my $self = shift;

 my $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}


1;

