#!/usr/bin/perl
package plugins::Pluginsquirrelmail;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'plugins::Pluginsquirrelmail');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# plugin-squirrelmail.pl
# Functions for SquirrelMail configure
 
1;
1;
		return $self;
}

# &squirrelmail_depends()
# return hash of squirrelmail depends
sub squirrelmail_depends
{
	my $self = shift;

 my @deps;
 return {'apache' => [$self->configured('apache'), 'apache'], 'php' => [$self->check_php_apache(), 'apache']};
}

# &configure_squirrelmail(tar, version)
# configure SquirrelMail plugin
sub configure_squirrelmail
{
	my $self = shift;

 # print configure message
 $self->msg_conftitle($self->{'text_hash'}->{'plugin_squirrelmail'});
 
 # tar file name and version
 my ($file, $version) = @_;
 
 # get plugin directorys
 my ($dest, $subdir, $plugin_dir, $apache_user, $apache_group) = $self->get_install_dir('squirrelmail'); # &get_squirrelmail_dir();
 
 # untar file
 my $out = $self->backquote_logged("tar -zxf $file -C $dest 2>&1 </dev/null");
 $self->error($out) if $?;
 (undef, $out) = $self->unlink_file($file);
 $self->error($out) if $out;
 
 # prepare plugin directory
 opendir (DIR, $dest) || $self->error($self->text('error_open_dir',$dest));
 while (defined ($self->{'f'} = readdir (DIR)))
 {
  next if $self->{'f'} =~ /^\.\.?$/;
  if (-d "$dest/$self->{'f'}" && $self->{'f'} =~ /^squirrelmail-$version/)
  {
   $out = $self->backquote_logged("mv $dest/$self->{'f'} $plugin_dir 2>&1 </dev/null");
   $self->error($out) if $?;
   last;
  }
 }
 closedir (DIR);
  
 unless (-e "$plugin_dir/config/config.php")
 {
  # copy config.php
  my $source = "$self->{'module_root_directory'}/addons/squirrelmail/config.php";
    
  # get hostname
  my $hostname = $self->get_system_hostname();
  
  # edit config.php
  my $conf = $self->parse_squirrelmail_config($source);
  $self->set_squirrelmail_value('$domain',"'".$hostname."';",$conf);
  $self->set_squirrelmail_value('$config_version',"'".$version."';",$conf);
  $self->save_squirrelmail_options($conf,$source);
  
  # config.php copy
  (undef, $out) = $self->copy_source_dest($source,"$plugin_dir/config/"); 
  $self->error($out) if $out;
 }
 
 unless (-d "$plugin_dir/data/")
 {
  # make data dir
  $out = $self->backquote_logged("mkdir $plugin_dir/data 2>&1 </dev/null");
  $self->error($out) if $?;
 }
 
 # set owners & permissions
 $out = $self->backquote_logged("chown -R $apache_user.$apache_group $plugin_dir 2>&1 </dev/null");
 $self->error($out) if $?;
 $out = $self->backquote_logged("chmod -R 755 $plugin_dir 2>&1 </dev/null");
 $self->error($out) if $?;
 
 # add apache per-directory and alias
 $self->apache_install_conf($subdir, $plugin_dir); 
 
 if ($main::apache_object->is_apache_running())
 {
  # restart apache
  my $out = $main::apache_object->restart_apache();
  $self->error($out) if ($out);
 }
 
 # configure endusers mailboxes
 $self->foreign_require('enduser_cf','enduser_cf-lib.pl');
 $self->read_file("$main::enduser_cf_object->{'config_hash'}->{'usermin_dir'}/config", \%{$self->{'gconfig_hash'}});
 $self->{'gconfig_hash'}->{'squirrelmail'} = 1;
 $self->write_file("$main::enduser_cf_object->{'config_hash'}->{'usermin_dir'}/config", \%{$self->{'gconfig_hash'}});
 
 # print done message
 $self->msg_done();
}

# &update_squirrelmail(tar,version)
# update SquirrelMail plugin
sub update_squirrelmail
{
	my $self = shift;

 # tar file name and version
 my ($file,$version) = @_;
 
 # get plugin directorys
 my ($dest, $subdir, $plugin_dir, $apache_user, $apache_group) = $self->get_install_dir('squirrelmail'); # &get_squirrelmail_dir();
 
 # log file name
 my ($log_file, $log) = "$self->{'module_config_directory'}\/squirrelmail.log";
 
 # read log
 $self->read_file($log_file, $log);
 
 my $out;
 if (-d $log->{'per-dir'})
 {
  # update plugin
  # print configure message
  $self->msg_conftitle($self->{'text_hash'}->{'plugin_squirrelmail'});
  
  # untar file
  $out = $self->backquote_logged("tar -zxf $file -C $dest 2>&1 </dev/null");
  $self->error($out) if $?;
  (undef, $out) = $self->unlink_file($file);
  print $self->text('update_error',$out) if $out;
  
  # prepare plugin directory
  opendir (DIR, $dest) || $self->error($self->text('error_open_dir',$dest));
  while (defined ($self->{'f'} = readdir (DIR)))
  {
   next if $self->{'f'} =~ /^\.\.?$/;
   if (-d "$dest/$self->{'f'}" && $self->{'f'} =~ /^squirrelmail-([0-9\.]+)/) #/^$subdir/
   {
    # restore old data
    # restore config.php
    $out = $self->backquote_logged("cp --reply=yes $log->{'per-dir'}/config/config.php $dest/$self->{'f'}/config/ 2>&1 </dev/null");
	 if ($?)
	 {
	  $self->unlink_file("$dest/$self->{'f'}");
     $self->error($out);
	 }
	 
	 # restore data
	 my @b_dir = ('data','plugins','themes');
	 foreach (@b_dir)
	 {
	  unless (-d "$dest/$self->{'f'}/$_/")
     {
      # make directory if needed
      $out = $self->backquote_logged("mkdir $dest/$self->{'f'}/$_/ 2>&1 </dev/null");
	   if ($?)
	   {
	    $self->unlink_file("$dest/$self->{'f'}");
       $self->error($out);
	   }
     }
     $out = $self->backquote_logged("cp -r --reply=yes $log->{'per-dir'}/$_/* $dest/$self->{'f'}/$_/ 2>&1 </dev/null") if (-d "$log->{'per-dir'}/$_/" && -d "$dest/$self->{'f'}/$_/");
	  if ($?)
	  {
	   $self->unlink_file("$dest/$self->{'f'}");
      $self->error($out);
	  }
	 }
    
    # copy new data to old directory
    $out = $self->backquote_logged("cp -r --reply=yes $dest/$self->{'f'}/* $log->{'per-dir'} 2>&1 </dev/null");
	 if ($?)
	 {
	  $self->unlink_file("$dest/$self->{'f'}");
     $self->error($out);
	 }
    
    # rename directory
    $out = $self->backquote_logged("mv $log->{'per-dir'} $plugin_dir 2>&1 </dev/null") if ($log->{'per-dir'} ne $plugin_dir);
	 if ($?)
	 {
	  $self->unlink_file("$dest/$self->{'f'}");
     $self->error($out);
	 }
	 
	 # delete directory
    $self->unlink_file("$dest/$self->{'f'}");
    last;
   }
  }
  closedir (DIR);
   
  # set owners & permissions
  $out=$self->backquote_logged("chown -R $apache_user.$apache_group $plugin_dir 2>&1 </dev/null");
  print $self->text('update_error',$out) if $?;
  $out=$self->backquote_logged("chmod -R 755 $plugin_dir 2>&1 </dev/null");
  print $self->text('update_error',$out) if $?;
  
  # update apache per-directory and alias, restart apache
  if ($plugin_dir ne $log->{'per-dir'})
  {
   # del apache per-dir & aliases
   $self->apache_uninstall_conf($log->{'per-dir'}, $log->{'alias'});
   $self->apache_install_conf($subdir, $plugin_dir);
	if ($main::apache_object->is_apache_running())
   {
    # restart apache
    my $out = $main::apache_object->restart_apache();
    $self->error($out) if ($out);
   }
  }
  # print done message
  $self->msg_done();
 }
 else 
 {
  # install plugin
  $self->configure_squirrelmail($file, $version);
 }
}

# &configure_squirrelmail(tar)
# uninstall SquirrelMail plugin
sub uninstall_squirrelmail
{
	my $self = shift;

 # print uninstall message
 $self->msg_conftitle_uninstall($self->{'text_hash'}->{'plugin_squirrelmail'});
 
 # log file name
 my ($log_file, $log) = "$self->{'module_config_directory'}\/squirrelmail.log";
 
 # read log
 $self->read_file($log_file, $log);
 
 # delete plugin directory
 my $out = $self->unlink_file("$log->{'per-dir'}") if (-d $log->{'per-dir'});
 print $self->text('uninstall_error',$out) if $?;
 
 # del apache per-dir & aliases
 $self->apache_uninstall_conf($log->{'per-dir'}, $log->{'alias'});
 if ($main::apache_object->is_apache_running())
 {
  # restart apache
  my $out = $main::apache_object->restart_apache();
  $self->error($out) if ($out);
 }
   
 # add confmaster log
 $self->write_postinstall_log('squirrelmail','uninstall',1,1);
 
 # configure endusers mailboxes
 $self->foreign_require('enduser_cf','enduser_cf-lib.pl');
 $self->read_file("$main::enduser_cf_object->{'config_hash'}->{'usermin_dir'}/config", \%{$self->{'gconfig_hash'}});
 $self->{'gconfig_hash'}->{'squirrelmail'} = 0;
 $self->write_file("$main::enduser_cf_object->{'config_hash'}->{'usermin_dir'}/config", \%{$self->{'gconfig_hash'}});
 
 # print done message
 $self->msg_done();
}

# &get_squirrelmail_version()
# return plugin version
sub get_squirrelmail_version
{
	my $self = shift;

 # log file name
 my ($log_file, $log)="$self->{'module_config_directory'}\/squirrelmail.log";
 
 # read log
 $self->read_file($log_file,$log);
 
 if (-e "$log->{'per-dir'}/functions/strings.php")
 {
  # get squirrelmail version
  my $conf = $self->parse_squirrelmail_config("$log->{'per-dir'}/functions/strings.php");
  $self->get_squirrelmail_value('$version', $conf) =~ /'([0-9]\.[0-9]\.[0-9])'/;
  return $1;
 }
 return 0;
}

# &parse_squirrelmail_file($file)
# return hash with name=value pairs from SM configuration file or call &error
# only for $name = value
sub parse_squirrelmail_config
{
	my $self = shift;

 my $conf_file = $_[0];
 open (INFL,$conf_file) || $self->error($self->text('error_open',$conf_file));
 my $conf;
 while (<INFL>)
 {
  chomp;
  $self->trunc_space($_);
  if ($_=~/^\$/)
  {
   my @str=split (/=/,$_);
   $str[0]=$self->trunc_space($str[0]);
   $str[1]=$self->trunc_space($str[1]);
   $conf -> {$str[0]} = $str[1];
  }
 }
 close INFL;
 return $conf;
}

# &get_squirrelmail_value(name,\%hash)
# return value for name from hash
sub get_squirrelmail_value
{
	my $self = shift;

 return $_[1]->{$_[0]};
}

# &set_squirrelmail_value(name,value,\%hash)
# set new value for name from hash
sub set_squirrelmail_value
{
	my $self = shift;

 $_[2]->{$_[0]}=$_[1];
}

# &save_squirrelmail_options(\%hash, $file)
# save all new options from hash to php config file, change union options value, delete all other options
sub save_squirrelmail_options
{
	my $self = shift;

 # get config file
 my $conf_file = $_[1];
  
 # parse config file
 $self->lock_file($conf_file);
 my $old_conf = $self->parse_squirrelmail_config($conf_file);
 
 # get new options
 my @new;
 foreach (keys %{$_[0]})
 {
  unless (exists $old_conf->{$_}) { push (@new,$_); }
 }
 # get del & union options
 my (@del,@union);
 foreach (keys %{$old_conf})
 {
  if (exists $_[0]->{$_}) { push (@union,$_); }
  else { push (@del,$_); }
 }
 # read config file
 my $old_file= $self->read_file_lines($conf_file);
 
 # modify options
 foreach (@union)
 {
  foreach my $i(@{$old_file})
  {
   my @str_old=split (/=/,$i);
   my $str=$self->trunc_space($str_old[0]);
   $i="$_=$_[0]->{$_}" if ($str eq $_);
  }
 }
 # delete options
 foreach (@del)
 {
  my @temp;
  foreach my $i(@{$old_file})
  {
   my @str_old=split (/=/,$i);
   my $str=$self->trunc_space($str_old[0]);
   push (@temp,$i) unless ($str eq $_);
  }
  @{$old_file}=@temp;
 }
 # add new options
 foreach (@new)
 {
  my (@temp,$comment);
  foreach my $i (@{$old_file})
  {
   # add new options under comment
   push (@temp,$i);
   if (!$comment && $i=~/^#\s*$_\s*=/)
   {
    push (@temp,"$_=$_[0]->{$_}");
    $comment=1;
   }
  }
  # add new options in the end of file
  push (@temp,"$_=$_[0]->{$_}") if (!$comment);
  @{$old_file}=@temp;
 } 
 # save config file
 $self->flush_file_lines();
 $self->unlock_file($conf_file);
}


1;

