# plugin-phpmyadmin.pl
# Functions for phpMyAdmin configure
 
# &phpmyadmin_depends()
# return hash of phpmyadmin depends
sub phpmyadmin_depends
{
 my @deps;
 return {'apache' => [&configured('apache'), 'apache'], 'php' => [&check_php_apache(), 'apache'], 'mysql' => [&configured('mysql'), 'mysql']};
}

# &configure_phpmyadmin(tar, version)
# configure SquirrelMail plugin
sub configure_phpmyadmin
{
 # print configure message
 &msg_conftitle($text{'plugin_phpmyadmin'});
 
 # tar file name and version
 my ($file, $version) = @_;
 
 # get plugin directorys
 my ($dest, $subdir, $plugin_dir, $apache_user, $apache_group) = &get_install_dir('phpmyadmin');
 
 # untar file
 my $out = &backquote_logged ("tar -zxf $file -C $dest 2>&1 </dev/null");
 &error($out) if $?;
 (undef, $out) = &unlink_file($file);
 &error($out) if $out;
 
 # prepare plugin directory
 my $f;
 opendir (DIR, $dest) || &error(&text('error_open_dir',$dest));
 while (defined ($f = readdir (DIR)))
 {
  next if $f =~ /^\.\.?$/;
  if (-d "$dest/$f" && $f =~ /^phpmyadmin-$version/i)
  {
   $out = &backquote_logged("mv $dest/$f $plugin_dir 2>&1 </dev/null");
   &error($out) if $?;
   last;
  }
 }
 closedir (DIR);
 
 # make config directory
 $out = &backquote_logged("mkdir $plugin_dir/config 2>&1 </dev/null");
 
 # set owners & permissions
 $out = &backquote_logged("chown -R $apache_user.$apache_group $plugin_dir 2>&1 </dev/null");
 &error($out) if $?;
 $out = &backquote_logged("chmod -R 755 $plugin_dir 2>&1 </dev/null");
 &error($out) if $?;
 
 # set mysql extension
 &set_mysql_extension() if (($gconfig{'os_type'} eq 'trustix-linux') || $config{'ztl_mode'});
 
 # add apache per-directory and alias
 &apache_install_conf($subdir, $plugin_dir); 
 
 if (&apache::is_apache_running())
 {
  # restart apache
  my $out = &apache::restart_apache();
  &error($out) if ($out);
 }
 
 # configure endusers mailboxes
 &foreign_require('enduser_cf','enduser_cf-lib.pl');
 &read_file("$enduser_cf::config{'usermin_dir'}/config", \%gconfig);
 $gconfig{'phpmyadmin'} = 1;
 &write_file("$enduser_cf::config{'usermin_dir'}/config", \%gconfig);
 
 # print done message
 &msg_done();
}

# &update_phpmyadmin(tar,version)
# update SquirrelMail plugin
sub update_phpmyadmin
{
 # tar file name and version
 my ($file,$version) = @_;
 
 # get plugin directorys
 my ($dest, $subdir, $plugin_dir, $apache_user, $apache_group) = &get_install_dir('phpmyadmin');
 
 # log file name
 my ($log_file, $log) = "$module_config_directory\/phpmyadmin.log";
 
 # read log
 &read_file($log_file, $log);
 
 my $out;
 if (-d $log->{'per-dir'})
 {
  # update plugin
  # print configure message
  &msg_conftitle($text{'plugin_phpmyadmin'});
  
  # untar file
  $out = &backquote_logged ("tar -zxf $file -C $dest 2>&1 </dev/null");
  &error($out) if $?;
  (undef, $out) = &unlink_file($file);
  print &text('update_error',$out) if $out;
  
  # prepare plugin directory
  my $f;
  opendir (DIR, $dest) || &error(&text('error_open_dir',$dest));
  while (defined ($f = readdir (DIR)))
  {
   next if $f =~ /^\.\.?$/;
   if (-d "$dest/$f" && $f =~ /^phpmyadmin-([0-9\.]+)/i) #/^$subdir/
   {
    # restore old data
    # restore config.php
    $out = &backquote_logged("cp --reply=yes $log->{'per-dir'}/config.inc.php $dest/$f/ 2>&1 </dev/null");
	 if ($?)
	 {
	  &unlink_file("$dest/$f");
     &error($out);
	 }
    
    # delete old directory
	 $out = &unlink_file($log->{'per-dir'});
	 if ($?)
	 {
	  &unlink_file("$dest/$f");
     &error($out);
	 }
	 
	 # rename directory
    $out = &backquote_logged("mv $dest/$f $plugin_dir 2>&1 </dev/null");
	 if ($?)
	 {
	  &unlink_file("$dest/$f");
     &error($out);
	 }
	 
	 # delete directory
    &unlink_file("$dest/$f");
    last;
   }
  }
  closedir (DIR);
   
  # make config directory
  $out = &backquote_logged("mkdir $plugin_dir/config 2>&1 </dev/null");
 
  # set owners & permissions
  $out=&backquote_logged("chown -R $apache_user.$apache_group $plugin_dir 2>&1 </dev/null");
  print &text('update_error',$out) if $?;
  $out=&backquote_logged("chmod -R 755 $plugin_dir 2>&1 </dev/null");
  print &text('update_error',$out) if $?;
  
  # update apache per-directory and alias, restart apache
  if ($plugin_dir ne $log->{'per-dir'})
  {
   # del apache per-dir & aliases
   &apache_uninstall_conf($log->{'per-dir'}, $log->{'alias'});
   &apache_install_conf($subdir, $plugin_dir);
  }
  
  if (&apache::is_apache_running())
  {
   # restart apache
   my $out = &apache::restart_apache();
   &error($out) if ($out);
  }
  
  # print done message
  &msg_done();
 }
 else 
 {
  # install plugin
  &configure_phpmyadmin ($file, $version);
 }
}

# &configure_phpmyadmin(tar)
# uninstall SquirrelMail plugin
sub uninstall_phpmyadmin
{
 # print uninstall message
 &msg_conftitle_uninstall($text{'plugin_phpmyadmin'});
 
 # log file name
 my ($log_file, $log) = "$module_config_directory\/phpmyadmin.log";
 
 # read log
 &read_file($log_file, $log);
 
 # delete plugin directory
 my $out = &unlink_file("$log->{'per-dir'}") if (-d $log->{'per-dir'});
 print &text('uninstall_error',$out) if $?;
 
 # del apache per-dir & aliases
 &apache_uninstall_conf($log->{'per-dir'}, $log->{'alias'});
 
 if (&apache::is_apache_running())
 {
  # restart apache
  my $out = &apache::restart_apache();
  &error($out) if ($out);
 }
   
 # add confmaster log
 &write_postinstall_log('phpmyadmin','uninstall',1,1);
 
 # configure endusers mailboxes
 &foreign_require('enduser_cf','enduser_cf-lib.pl');
 &read_file("$enduser_cf::config{'usermin_dir'}/config", \%gconfig);
 $gconfig{'phpmyadmin'} = 0;
 &write_file("$enduser_cf::config{'usermin_dir'}/config", \%gconfig);
 
 # print done message
 &msg_done();
}

# &get_phpmyadmin_version()
# return plugin version
sub get_phpmyadmin_version
{
 # log file name
 my ($log_file, $log, $f)="$module_config_directory\/phpmyadmin.log";
 
 # read log
 &read_file($log_file,$log);
 
 if (-d $log->{'per-dir'})
 {
  opendir (DIR, $log->{'per-dir'}) || &error(&text('error_open_dir', $log->{'per-dir'}));
  while (defined ($f = readdir (DIR)))
  {
   next if $f =~ /^\.\.?$/;
   if (-e "$log->{'per-dir'}/$f" && $f =~ /^release-date-([0-9\.]+)/i) { return $1; }
  }
 }
 return 0;
}

# &set_mysql_extension()
# enable mysql extension in the php.ini
sub set_mysql_extension
{
 my $conf_file = "$apache::config{'httpd_dir'}/php.ini";
 &lock_file($conf_file);
 my $conf = &read_file_lines($conf_file);
 foreach (@{$conf})
 { 
  $_ =~ s/;// if ($_ =~ /^\s*;+?\s*extension\s*=\s*mysqli?.so/)
 }
 &flush_file_lines();
 &unlock_file($conf_file);
}


1;
1;
