# plugin-squirrelmail.pl
# Functions for SquirrelMail configure
 
# &squirrelmail_depends()
# return hash of squirrelmail depends
sub squirrelmail_depends
{
 my @deps;
 return {'apache' => [&configured('apache'), 'apache'], 'php' => [&check_php_apache(), 'apache']};
}

# &configure_squirrelmail(tar, version)
# configure SquirrelMail plugin
sub configure_squirrelmail
{
 # print configure message
 &msg_conftitle($text{'plugin_squirrelmail'});
 
 # tar file name and version
 my ($file, $version) = @_;
 
 # get plugin directorys
 my ($dest, $subdir, $plugin_dir, $apache_user, $apache_group) = &get_install_dir('squirrelmail'); # &get_squirrelmail_dir();
 
 # untar file
 my $out = &backquote_logged ("tar -zxf $file -C $dest 2>&1 </dev/null");
 &error($out) if $?;
 (undef, $out) = &unlink_file($file);
 &error($out) if $out;
 
 # prepare plugin directory
 opendir (DIR, $dest) || &error(&text('error_open_dir',$dest));
 while (defined ($f = readdir (DIR)))
 {
  next if $f =~ /^\.\.?$/;
  if (-d "$dest/$f" && $f =~ /^squirrelmail-$version/)
  {
   $out = &backquote_logged("mv $dest/$f $plugin_dir 2>&1 </dev/null");
   &error($out) if $?;
   last;
  }
 }
 closedir (DIR);
  
 unless (-e "$plugin_dir/config/config.php")
 {
  # copy config.php
  my $source = "$module_root_directory/addons/squirrelmail/config.php";
    
  # get hostname
  my $hostname = &get_system_hostname();
  
  # edit config.php
  my $conf = &parse_squirrelmail_config ($source);
  &set_squirrelmail_value('$domain',"'".$hostname."';",$conf);
  &set_squirrelmail_value('$config_version',"'".$version."';",$conf);
  &save_squirrelmail_options($conf,$source);
  
  # config.php copy
  (undef, $out) = &copy_source_dest($source,"$plugin_dir/config/"); 
  &error($out) if $out;
 }
 
 unless (-d "$plugin_dir/data/")
 {
  # make data dir
  $out = &backquote_logged("mkdir $plugin_dir/data 2>&1 </dev/null");
  &error($out) if $?;
 }
 
 # set owners & permissions
 $out = &backquote_logged("chown -R $apache_user.$apache_group $plugin_dir 2>&1 </dev/null");
 &error($out) if $?;
 $out = &backquote_logged("chmod -R 755 $plugin_dir 2>&1 </dev/null");
 &error($out) if $?;
 
 # add apache per-directory and alias
 &apache_install_conf($subdir, $plugin_dir); 
 
 if (&apache::is_apache_running())
 {
  # restart apache
  my $out = &apache::restart_apache();
  &error($out) if ($out);
 }
 
 # configure endusers mailboxes
 &foreign_require('enduser_cf','enduser_cf-lib.pl');
 &read_file("$enduser_cf::config{'usermin_dir'}/config", \%gconfig);
 $gconfig{'squirrelmail'} = 1;
 &write_file("$enduser_cf::config{'usermin_dir'}/config", \%gconfig);
 
 # print done message
 &msg_done();
}

# &update_squirrelmail(tar,version)
# update SquirrelMail plugin
sub update_squirrelmail
{
 # tar file name and version
 my ($file,$version) = @_;
 
 # get plugin directorys
 my ($dest, $subdir, $plugin_dir, $apache_user, $apache_group) = &get_install_dir('squirrelmail'); # &get_squirrelmail_dir();
 
 # log file name
 my ($log_file, $log) = "$module_config_directory\/squirrelmail.log";
 
 # read log
 &read_file($log_file, $log);
 
 my $out;
 if (-d $log->{'per-dir'})
 {
  # update plugin
  # print configure message
  &msg_conftitle($text{'plugin_squirrelmail'});
  
  # untar file
  $out = &backquote_logged ("tar -zxf $file -C $dest 2>&1 </dev/null");
  &error($out) if $?;
  (undef, $out) = &unlink_file($file);
  print &text('update_error',$out) if $out;
  
  # prepare plugin directory
  opendir (DIR, $dest) || &error(&text('error_open_dir',$dest));
  while (defined ($f = readdir (DIR)))
  {
   next if $f =~ /^\.\.?$/;
   if (-d "$dest/$f" && $f =~ /^squirrelmail-([0-9\.]+)/) #/^$subdir/
   {
    # restore old data
    # restore config.php
    $out = &backquote_logged("cp --reply=yes $log->{'per-dir'}/config/config.php $dest/$f/config/ 2>&1 </dev/null");
	 if ($?)
	 {
	  &unlink_file("$dest/$f");
     &error($out);
	 }
	 
	 # restore data
	 my @b_dir = ('data','plugins','themes');
	 foreach (@b_dir)
	 {
	  unless (-d "$dest/$f/$_/")
     {
      # make directory if needed
      $out = &backquote_logged("mkdir $dest/$f/$_/ 2>&1 </dev/null");
	   if ($?)
	   {
	    &unlink_file("$dest/$f");
       &error($out);
	   }
     }
     $out = &backquote_logged("cp -r --reply=yes $log->{'per-dir'}/$_/* $dest/$f/$_/ 2>&1 </dev/null") if (-d "$log->{'per-dir'}/$_/" && -d "$dest/$f/$_/");
	  if ($?)
	  {
	   &unlink_file("$dest/$f");
      &error($out);
	  }
	 }
    
    # copy new data to old directory
    $out = &backquote_logged("cp -r --reply=yes $dest/$f/* $log->{'per-dir'} 2>&1 </dev/null");
	 if ($?)
	 {
	  &unlink_file("$dest/$f");
     &error($out);
	 }
    
    # rename directory
    $out = &backquote_logged("mv $log->{'per-dir'} $plugin_dir 2>&1 </dev/null") if ($log->{'per-dir'} ne $plugin_dir);
	 if ($?)
	 {
	  &unlink_file("$dest/$f");
     &error($out);
	 }
	 
	 # delete directory
    &unlink_file("$dest/$f");
    last;
   }
  }
  closedir (DIR);
   
  # set owners & permissions
  $out=&backquote_logged("chown -R $apache_user.$apache_group $plugin_dir 2>&1 </dev/null");
  print &text('update_error',$out) if $?;
  $out=&backquote_logged("chmod -R 755 $plugin_dir 2>&1 </dev/null");
  print &text('update_error',$out) if $?;
  
  # update apache per-directory and alias, restart apache
  if ($plugin_dir ne $log->{'per-dir'})
  {
   # del apache per-dir & aliases
   &apache_uninstall_conf($log->{'per-dir'}, $log->{'alias'});
   &apache_install_conf($subdir, $plugin_dir);
	if (&apache::is_apache_running())
   {
    # restart apache
    my $out = &apache::restart_apache();
    &error($out) if ($out);
   }
  }
  # print done message
  &msg_done();
 }
 else 
 {
  # install plugin
  &configure_squirrelmail ($file, $version);
 }
}

# &configure_squirrelmail(tar)
# uninstall SquirrelMail plugin
sub uninstall_squirrelmail
{
 # print uninstall message
 &msg_conftitle_uninstall($text{'plugin_squirrelmail'});
 
 # log file name
 my ($log_file, $log) = "$module_config_directory\/squirrelmail.log";
 
 # read log
 &read_file($log_file, $log);
 
 # delete plugin directory
 my $out = &unlink_file("$log->{'per-dir'}") if (-d $log->{'per-dir'});
 print &text('uninstall_error',$out) if $?;
 
 # del apache per-dir & aliases
 &apache_uninstall_conf($log->{'per-dir'}, $log->{'alias'});
 if (&apache::is_apache_running())
 {
  # restart apache
  my $out = &apache::restart_apache();
  &error($out) if ($out);
 }
   
 # add confmaster log
 &write_postinstall_log('squirrelmail','uninstall',1,1);
 
 # configure endusers mailboxes
 &foreign_require('enduser_cf','enduser_cf-lib.pl');
 &read_file("$enduser_cf::config{'usermin_dir'}/config", \%gconfig);
 $gconfig{'squirrelmail'} = 0;
 &write_file("$enduser_cf::config{'usermin_dir'}/config", \%gconfig);
 
 # print done message
 &msg_done();
}

# &get_squirrelmail_version()
# return plugin version
sub get_squirrelmail_version
{
 # log file name
 my ($log_file, $log)="$module_config_directory\/squirrelmail.log";
 
 # read log
 &read_file($log_file,$log);
 
 if (-e "$log->{'per-dir'}/functions/strings.php")
 {
  # get squirrelmail version
  my $conf = &parse_squirrelmail_config("$log->{'per-dir'}/functions/strings.php");
  &get_squirrelmail_value('$version', $conf) =~ /'([0-9]\.[0-9]\.[0-9])'/;
  return $1;
 }
 return 0;
}

# &parse_squirrelmail_file($file)
# return hash with name=value pairs from SM configuration file or call &error
# only for $name = value
sub parse_squirrelmail_config
{
 my $conf_file = $_[0];
 open (INFL,$conf_file) || &error(&text('error_open',$conf_file));
 my $conf;
 while (<INFL>)
 {
  chomp;
  &trunc_space($_);
  if ($_=~/^\$/)
  {
   my @str=split (/=/,$_);
   $str[0]=&trunc_space($str[0]);
   $str[1]=&trunc_space($str[1]);
   $conf -> {$str[0]} = $str[1];
  }
 }
 close INFL;
 return $conf;
}

# &get_squirrelmail_value(name,\%hash)
# return value for name from hash
sub get_squirrelmail_value
{
 return $_[1]->{$_[0]};
}

# &set_squirrelmail_value(name,value,\%hash)
# set new value for name from hash
sub set_squirrelmail_value
{
 $_[2]->{$_[0]}=$_[1];
}

# &save_squirrelmail_options(\%hash, $file)
# save all new options from hash to php config file, change union options value, delete all other options
sub save_squirrelmail_options
{
 # get config file
 my $conf_file = $_[1];
  
 # parse config file
 &lock_file($conf_file);
 my $old_conf = &parse_squirrelmail_config($conf_file);
 
 # get new options
 my @new;
 foreach (keys %{$_[0]})
 {
  unless (exists $old_conf->{$_}) { push (@new,$_); }
 }
 # get del & union options
 my (@del,@union);
 foreach (keys %{$old_conf})
 {
  if (exists $_[0]->{$_}) { push (@union,$_); }
  else { push (@del,$_); }
 }
 # read config file
 my $old_file= &read_file_lines($conf_file);
 
 # modify options
 foreach (@union)
 {
  foreach my $i(@{$old_file})
  {
   my @str_old=split (/=/,$i);
   my $str=&trunc_space($str_old[0]);
   $i="$_=$_[0]->{$_}" if ($str eq $_);
  }
 }
 # delete options
 foreach (@del)
 {
  my @temp;
  foreach my $i(@{$old_file})
  {
   my @str_old=split (/=/,$i);
   my $str=&trunc_space($str_old[0]);
   push (@temp,$i) unless ($str eq $_);
  }
  @{$old_file}=@temp;
 }
 # add new options
 foreach (@new)
 {
  my (@temp,$comment);
  foreach my $i (@{$old_file})
  {
   # add new options under comment
   push (@temp,$i);
   if (!$comment && $i=~/^#\s*$_\s*=/)
   {
    push (@temp,"$_=$_[0]->{$_}");
    $comment=1;
   }
  }
  # add new options in the end of file
  push (@temp,"$_=$_[0]->{$_}") if (!$comment);
  @{$old_file}=@temp;
 } 
 # save config file
 &flush_file_lines();
 &unlock_file($conf_file);
}


1;
1;
