# plugins-lib.pl
# Common functions for plugins module

# main part
do '../web-lib.pl';
&init_config();
require '../ui-lib.pl';

# supported plugins list
@plugin_list = ('squirrelmail','phpmyadmin');

# plugin funcs require
foreach (@plugin_list)
{
 require "plugin-$_.pl";
}

# &install_plugin (plugin, [upgrade])
# install or upgrade plugin
sub install_plugin
{
 # plugin name
 my $pl = $_[0];
 
 # base update url
 my $update_url = $config{'base_update_url'};
 
 # print install header
 if ($_[1]) { print "<tr $tb><td colspan=2><b>$text{'updating_title'}</b></td></tr>\n"; }
 else { print "<tr $tb><td colspan=2><b>$text{'install_title'}</b></td></tr>\n"; }
 print "<tr $cb><td colspan=2><pre>";

 # get templete file to download
 my $file = &transname();
 
 # find latest version of plugin
 my $progress_callback_url = &convert_osdn_url($update_url);
 my ($host, $port, $page, $error, $ssl) = &parse_http_url($progress_callback_url);
 &http_download($host, $port, $page, $file, \$error, undef, $ssl);
 &inst_error($error, $file) if $error;
 
 # get tar version
 my $version = &get_tar_version($file,$pl); 
 unless ($version)
 {
  # can't get new version
  &inst_error(&text('get_version_error', $pl));
 }

 # get current  version
 $func = "get_".$pl."_version";
 my $dur_version = &$func();
 
 # compare versions
 my @ver1=split(/\./,$version);
 my $v1=$ver1[0]*1000000+$ver1[1]*1000+$ver1[2];
 my @ver2=split(/\./,$dur_version);
 my $v2 = $ver2[0]*1000000+$ver2[1]*1000+$ver2[2];
 if ($v1 == $v2 || $v2 >= $v1)
 {
  # use last version
  &inst_error(&text('upgrade_elatest',$dur_version, $pl));
 }
 
 # downloading tar.gz file
 $update_url = "$update_url/$pl-$version.tar.gz"; 
 my $progress_callback_url = &convert_osdn_url($update_url);
 ($host, $port, $page, $ssl) = &parse_http_url($progress_callback_url);
 &http_download($host, $port, $page, $file, \$error,\&progress_callback, $ssl);
 &inst_error($error, $file) if $error; 
 print "</pre></td></tr>\n";
 
 # print configure header
 print "<tr $tb><td colspan=2><b>$text{'configure_title'}</b></td></tr>\n";
 print "<tr $cb><td colspan=2><pre>";
 
 # add confmaster log
 &write_postinstall_log($pl,'download',1);
 
 if ($_[1]) 
 {
  # update plugin
  $func = "update_".$pl;
 }
 else
 {
  # configure plugin
  $func = "configure_".$pl;
 }
 &$func($file, $version);
 
 # add confmaster log
 &write_postinstall_log($pl,'configure',1);
 &write_postinstall_log($pl,'complete',1);
 print "</pre></td></tr>\n";
}

# &uninstall_plugin (plugin)
# uninstall_plugin
sub uninstall_plugin
{
 # plugin name
 my $pl = $_[0];

 # print uninstall header
 print "<tr $tb><td colspan=2><b>$text{'uninstalling_title'}</b></td></tr>\n";
 print "<tr $cb><td colspan=2><pre>";
 
 # uninstall plugin
 $func = "uninstall_".$pl;
 &$func($file);
 
 # add confmaster log
 &write_postinstall_log($pl,'complete',1,1);
 print "</pre></td></tr>\n";
}

# &get_tar_version(tar,plugin)
# return plugin version in tar & delete temp file
sub get_tar_version
{
 my ($file,$pl,$version) = @_;
 open(FILE, $_[0]);
 while(<FILE>) 
 {
  if (/$pl-([0-9\.]+)\.tar\.gz/i) 
  {
   $version = $1;
   last;
  }
 }
 close(FILE);
 &unlink_file($file);
 return $version;
}

# &print_servers_table();
# print plugin icons
sub print_plugins_table
{
 # get ACL
 my %access = &get_module_acl();
 
 print "<table cellpadding=10>\n";
 foreach (@plugin_list)
 {
  my $flag = &get_configured_flag($_);
  my $descr = $text{'descr_'.$_};
  if ($flag)
  {
   # plugin was installed
	my $icon = "$_.gif";
	my $actions = "<a href=\"configure.cgi?pl=".$_."&confirm_up=1\" class='menu'>$text{'plugin_upgrade'}</a> | <a href=\"configure.cgi?pl=".$_."&confirm_un=1\" class='menu'>$text{'plugin_uninstall'}</a>" if $access{'install'};
	
	# get plugin link
   &read_file("$module_config_directory\/$_.log", $log);
	my $pl_link = "http://$ENV{'SERVER_NAME'}/$log->{'alias'}/";
	
	print "<tr><td class=\"infotitle\" width=60><a href=\"$pl_link\"><img width=48 height=48 src=\"images/$icon\" border=\"0\" />&nbsp; <font size=+1>".$text{"plugin_$_"}."</font></a><td>$descr</td>\n";
	print "<td width=30%>$actions</td></tr>\n";
  }
  elsif ($access{'install'})
  {
   # plugin was't installed
	my $icon = "${_}_off.gif";
	my $actions = "<a href=\"configure.cgi?pl=".$_."&confirm=1\" class='menu'>$text{'plugin_install'}</a>";
	
	print "<tr><td class=\"infotitle\" width=60><img width=48 height=48 src=\"images/$icon\" border=\"0\" />&nbsp; <font size=+1>".$text{"plugin_$_"}."</font><td>$descr</td>\n";
	print "<td width=30%>$actions</td></tr>\n";
  }
 }
 print "</table>\n";
}

# &set_configured_flag(plugin)
# set config value in 1 for plugin
sub set_configured_flag
{
 $config{"configured_$_[0]"}=1;
 &save_module_config();
}

# &del_configured_flag(plugin)
# set config value in 0 for plugin
sub del_configured_flag
{
 $config{"configured_$_[0]"} = 0;
 &save_module_config();
}

# &get_configured_flag(plugin)
# get config value for plugin
sub get_configured_flag
{
 return $config{"configured_$_[0]"};
}

# &conf_title($name)
# print name title message
sub msg_conftitle
{
 print &text('setup_configure', $_[0]) . "<br>\n";
}

# &conf_title($name)
# print name uninstall message
sub msg_conftitle_uninstall
{
 print &text('uninstall_configure', $_[0]) . "<br>\n";
}

# &msg_done();
# print done message
sub msg_done
{
 print "$text{'index_done'}<br>\n";
}

# &write_postinstall_log(plugin, name, value, [uninstall mode])
# write postinstall log file for plugin in the plugins module directory
sub write_postinstall_log
{
 my $log_file;
 if ($_[3]) { $log_file = "$module_config_directory\/$_[0]_un.log"; }
 else { $log_file = "$module_config_directory\/$_[0].log"; }
 my $log;
 &read_file($log_file,$log);
 $log->{$_[1]}=$_[2];
 &write_file($log_file,$log);
}

# &print_postinstall_log(plugin,[1])
# print summary information for install or uninstall plugin
# if $_[1] = 1 - uninstall mode, else - install
sub print_postinstall_log
{
 # plugin name
 my $pl = $_[0];
 my ($log_file,$old_log,$log,$temp_file,$temp,@temp1,$out);
 
 # install or uninstall templete & log
 if ($_[1]) 
 { 
  # uninstall templete
  $log_file="$module_config_directory\/$_[0]_un.log";
  $old_log="$module_config_directory\/$_[0].log";
  $temp_file="$root_directory\/$module_name\/template.uninstall"; 
 }
 else 
 { 
  # install templete
  $log_file="$module_config_directory\/$_[0].log";
  $old_log="$module_config_directory\/$_[0]_un.log";
  $temp_file="$root_directory\/$module_name\/template.install"; 
 }
 
 if (-e $temp_file && -e $log_file)
 {
  # print summary header
  print "<tr $tb><td colspan=2 width=70%><b>$text{'summary_log'}</b></td><tr>\n";
  
  # read log file
  &read_file($log_file,$log);
  
  # read templete file
  my $start;
  open(TPL, $temp_file);
  while($_=<TPL>)
  {
   chomp;
   if ($_ =~ /\[$pl\]/) 
   { 
    # start tag
    $start = 1; 
    next;
   }
   elsif ($_ =~ /\[\/$pl\]/) 
   { 
    # end tag
    $start = 0; 
    last;
   }
   if ($start)
   {
    # print summary
    my @rang=split(/:/,$_);
    $rang[1] = &trunc_space($rang[1]);
    $rang[2] = &trunc_space($rang[2]);
    if ($log->{$rang[1]} == 1 && !$rang[2])
    {
     # main done
     if ($rang[1] eq 'complete')
     {
      print "<tr $cb><td><b>$rang[0]</b></td><td><font color=green><b>DONE</b></font></td></tr>\n";
     }
     else { print "<tr $cb><td>$rang[0]</td><td><font color=green><b>DONE</b></font></td></tr>\n"; }
    }
    elsif (exists $log->{$rang[1]} && $rang[2])
    {
     # advanced log
     print "<tr $cb><td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$rang[0]</td><td><font color=blue>$log->{$rang[1]}</font></td></tr>\n";
    }
    elsif(!exists $log->{$rang[1]} && !$rang[2])
    {
     # main failed
     if ($rang[1] eq 'complete')
     {
      print "<tr $cb><td>$rang[0]</td><td><font color=red><b>FAILED</b></font></td></tr>\n";
     }
     else { print "<tr $cb><td>$rang[0]</td><td><font color=red><b>FAILED</b></font></td></tr>\n"; }
    }
   }
  }
  close (TPL);
 }
 (undef, $out) = &unlink_file($old_log) if (-e $old_log);
 &error($out) if ($out);
 
 if ($text{"summary_$pl"} && !$_[1])
 {
  # print advancrd postinstall log
  my $pl_url = "http://$ENV{'SERVER_NAME'}/$log->{'alias'}";
  print "<tr $cb><td colspan=2>";
  print &text("summary_$pl", $pl_url, $log->{'per-dir'});
  print "</td></tr>";
 }
}

# &check php apache()
# return 1 if php enable, else return 0
sub check_php_apache
{
 &foreign_require('apache', 'apache-lib.pl');
 undef (@apache::get_config_cache);
 my $conf = &apache::get_config();
 my $conf_dir = "$apache::config{'httpd_dir'}/conf.d";
 foreach (@{$conf})
 {
  return 1 if (($_->{'name'} eq 'LoadModule') && ($_->{'words'}->[0]=~/^php[45]_module$/) && ($_->{'words'}->[1]=~/^.*?libphp[45].so$/) && ($_->{'file'}=~/$conf_dir\/.*?php.conf/));
 }
 return 0;
}

# &is_apache_dir($subdir, $plugin_dir)
# check apache per-directory, apache alias
sub is_apache_dir
{
 my ($subdir, $plugin_dir) = @_;
 
 # get httpd configuration file
 undef (@apache::get_config_cache);
 my $conf = &apache::get_config();

 # search Directory and Alias option
 my ($dir, $alias);
 foreach (@{$conf})
 {
  if (($_->{'name'} eq 'Directory') && ($_->{'words'}->[0] eq $plugin_dir))
  {
   $dir = 1;
   if (!$alias) { next; }
   else { last ;}
  }
  elsif (($_->{'name'} eq 'Alias') && ($_->{'words'}->[0] eq $subdir))
  {
   if ($_->{'words'}->[1] eq $plugin_dir)
   {
    $alias = 1;
    if (!$dir) { next; }
    else { last ;}
   }
   else { &error(&text('alias_error', $subdir)); }
  }
 }
 return ($dir, $alias);
}

# &apache_install_conf($subdir, $plugin_dir)
# add apache per-directory and alias
# restart apache
sub apache_install_conf
{
 my ($subdir, $plugin_dir) = @_;
 
 # is apache directory?
 my ($per_dir, $alias) = &is_apache_dir($subdir, $plugin_dir);
 if (!$per_dir || !$alias)
 {
  # add apache per-directory and alias
  my ($vconf, $v) = &apache::get_virtual_config();
  &apache::can_edit_virt($v) || &error($apache::text{'virt_ecannot'});
  my $f = $vconf->[0]->{'file'};
  for($j=0; $vconf->[$j]->{'file'} eq $f; $j++) { }
  &lock_file($f);
  &apache::before_changing();
  my $lref = &read_file_lines($f);
  my $l = $vconf->[$j-1]->{'eline'};
  unless ($alias)
  {
   # add apache alias
   $l++;
   splice(@$lref, $l, 0, "Alias \/$subdir \"$plugin_dir\"");
   
   # add confmaster log
   &write_postinstall_log($subdir, 'alias', $subdir);
  }
  unless ($per_dir)
  {
   # add apache per-directory
   $l++;
   my $newdir = "<Directory \"$plugin_dir\">";
   my $enddir = "</Directory>";
   splice(@$lref, $l, 0, ($newdir, $enddir));
   
   # add confmaster log
   &write_postinstall_log($subdir, 'per-dir', $plugin_dir);
  }
  &flush_file_lines();
  &unlock_file($f);
  &apache::after_changing(); 
  sleep(1);
 }
}

# &apache_reconfigure($per_dir, $alias)
# del apache per-directory and alias
# restart apache
sub apache_uninstall_conf
{
 my ($per_dir, $alias) = @_;  

 # get apache configuration
 my ($vconf, $v) = &apache::get_virtual_config();
 &apache::can_edit_virt($v) || &error($apache::text{'virt_ecannot'});
 my $f = $vconf->[0]->{'file'};
 &lock_file($f);
 &apache::before_changing();
 my $lref = &read_file_lines($f);
 my $del_per_dir;
 foreach (@{$lref})
 {
  if ($_ =~ /Alias\s+?\/$alias\s+?"$per_dir"/)
  {
   # delete alias
   undef $_;
  
   # add confmaster log
   &write_postinstall_log($alias, 'alias', $alias, 1);
	last;
  }
 }
 foreach (@{$lref})
 {
  if ($_ =~ /<\s*?Directory\s+?"$per_dir"\s*?>/)
  {
   # delete per_dir
   $del_per_dir = 1;
   undef $_;
  }
  if ($del_per_dir && $_ =~ /<\s*?\/Directory\s*?>/)
  {
   # delete per_dir end tag
   undef $_;
   
   # add confmaster log
   &write_postinstall_log($alias, 'per-dir', $per_dir, 1);
	last;
  }
 }
 &flush_file_lines();
 &unlock_file($f);
 &apache::after_changing();
 sleep(1);
}

# &get_install_dir($plugin)
# return array (dest_dir, sub_dir, plugin_dir, user)
sub get_install_dir
{
 my $plugin = $_[0];
 
 # get httpd configuration file
 undef (@apache::get_config_cache);
 my $conf = &apache::get_config();
 my $apache_user = &apache::find_directive('User', $conf);
 my $apache_group = &apache::find_directive('Group', $conf);
 
 my $dest;
 if ($config{'install_'.$plugin.'_dir'})
 {
  # from module configuration
  $dest = $config{'install_'.$plugin.'_dir'};
 }
 else
 {
  # search DocumentRoot option
  $dest = &apache::find_directive('DocumentRoot', $conf);
  $dest =~ /"(.+)"/;
  $dest = $1;
 }
 &error(&text('error_open_dir',$dest)) unless (-d $dest);
 
 $dest =~ s/\/+/\//g;
 my $subdir = $plugin;
 my $plugin_dir = $dest."/".$subdir."/";
 $plugin_dir =~ s/\/+/\//g;
 return ($dest, $subdir, $plugin_dir, $apache_user, $apache_group); 
}

# &inst_error(error)
# delete temp file & print error message
sub inst_error
{
 &unlink_file($_[1]) if ($_[1]);
 print "<b>$_[0]</b><br></table>\n";
 &ui_print_footer("", $text{'index_return'});
 exit;
}

# &check_error (warning)
# print warning messages
sub check_error
{
 print "<p class=warn>$_[0]</p>\n";
}

# &trunc_space(string)
# delete spaces from string
sub trunc_space
{
 my $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}

1;
1;
