#!/usr/bin/perl
package postfix::AliasesLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'postfix::AliasesLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# aliases-lib.pl
# Alias file functions

1;

1;
		return $self;
}

# aliases_file(&config)
# Returns the alias filenames
sub aliases_file
{
	my $self = shift;

if ($self->{'config_hash'}->{'alias_file'}) { return [ $self->{'config_hash'}->{'alias_file'} ]; }
else {
	local(@afiles, $o);
	foreach $o ($self->find_type("O", $_[0])) {
		if ($o->{'value'} =~ /^\s*AliasFile=(.*)$/) {
			push(@afiles, split(/,/, $1));
			}
		}
	map { s/dbm:// } @afiles;
	return \@afiles;
	}
}

# list_aliases(files)
# Returns an array of data structures, each containing information about
# one sendmail alias
sub list_aliases
{
	my $self = shift;

local $jfiles = join(",", @{$_[0]});
local $c = $self->{'list_aliases_cache_hash'}->{$jfiles};
if (!defined($c)) {
	$c = $self->{'list_aliases_cache_hash'}->{$jfiles} = [ ];
	local $file;
	local @skip = split(/\s+/, $self->{'config_hash'}->{'alias_skip'});
	foreach $file (@{$_[0]}) {
		local $lalias;
		local $lnum = 0;
		local $cmt;
		$self->open_readfile(*AFILE, $file);
		while(<AFILE>) {
			s/\r|\n//g;	# remove newlines
			if (/^\s*#+\s*(.*)/ && $self->is_table_comment($_, 1)) {
				# A comment line
				$cmt = $self->is_table_comment($_, 1);
				}
			elsif (/^(#*)\s*([^:$ \t]+)\s*:\s*(.*)$/) {
				local(%alias, @values, $v);
				$alias{'line'} = $cmt ? $lnum-1 : $lnum;
				$alias{'eline'} = $lnum;
				$alias{'file'} = $file;
				$alias{'files'} = $_[0];
				$alias{'enabled'} = $1 ? 0 : 1;
				$alias{'name'} = $2;
				$alias{'cmt'} = $cmt;
				$v = $3;
				$alias{'value'} = $v;
				while($v =~ /^\s*,?\s*()"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*(\|)"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*()([^,\s]+)(.*)$/) {
					push(@values, $1.$2); $v = $3;
					}
				$alias{'values'} = \@values;
				$alias{'num'} = scalar(@$c);
				if ($self->indexof($alias{'name'}, @skip) < 0) {
					push(@$c, \%alias);
					$lalias = \%alias;
					}
				$cmt = undef;
				}
			elsif (/^(#*)\s+(\S.*)$/ && $lalias &&
			       ($1 && !$lalias->{'enabled'} ||
				!$1 && $lalias->{'enabled'})) {
				# continuation of last alias
				$lalias->{'eline'} = $lnum;
				local $v = $2;
				$lalias->{'value'} .= $v;
				while($v =~ /^\s*,?\s*()"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*(\|)"([^"]+)"(.*)$/ ||
				      $v =~ /^\s*,?\s*()([^,\s]+)(.*)$/) {
					push(@{$lalias->{'values'}}, $1.$2); $v = $3;
					}
				$cmt = undef;
				}
			else {
				# Some other line
				$lalias = undef;
				$cmt = undef;
				}
			$lnum++;
			}
		close(AFILE);
		}
	}
return @$c;
}

# alias_form([alias])
# Display a form for editing or creating an alias. Each alias can map to
# 1 or more programs, files, lists or users
sub alias_form
{
	my $self = shift;

local($a, @values, $v, $type, $val, @typenames);
$a = $_[0];
if ($a) { @values = @{$a->{'values'}}; }
@typenames = map { $self->{'text_hash'}->{"aform_type$_"} } (0 .. 6);
$typenames[0] = "&lt;$typenames[0]&gt;";

# Start of form and table
print $self->ui_form_start("save_alias.cgi", "post");
if ($a) {
	print $self->ui_hidden("num", $a->{'num'}),"\n";
	}
else {
	print $self->ui_hidden("new", 1),"\n";
	}
print $self->ui_table_start($a ? $self->{'text_hash'}->{'aform_edit'}
			 : $self->{'text_hash'}->{'aform_create'}, undef, 2);

# Description
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'aform_cmt'},"alias_cmt"),
	$self->ui_textbox("cmt", $a ? $a->{'cmt'} : undef, 50));


# Alias name
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'aform_name'},"alias_name"),
		    $self->ui_textbox("name", $a ? $a->{'name'} : "", 20));

# Enabled flag
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'aform_enabled'}, "alias_enabled"),
		    $self->ui_yesno_radio("enabled", !$a || $a->{'enabled'} ? 1 : 0));

# Destinations
local @typeopts;
for($j=0; $j<@typenames; $j++) {
	if (!$j || $self->{'access_hash'}->{"aedit_$j"}) {
		push(@typeopts, [ $j, $typenames[$j] ]);
		}
	}
for($self->{'i'}=0; $self->{'i'}<=@values; $self->{'i'}++) {
	($type, $val) = $values[$self->{'i'}] ? $self->alias_type($values[$self->{'i'}]) : (0, "");

	local $typesel = $self->ui_select("type_$self->{'i'}", $type, \@typeopts);
	local $valtxt = $self->ui_textbox("val_$self->{'i'}", $val, 30);
	local $edlnk;
	if ($type == 2 && $a) {
		$edlnk = "<a href='edit_afile.cgi?file=$val&num=$a->{'num'}'>".
		      	 "$self->{'text_hash'}->{'aform_afile'}</a>\n";
		}
	elsif ($type == 5 && $a) {
		$edlnk = "<a href='edit_rfile.cgi?file=$val&num=$a->{'num'}'>".
		      	 "$self->{'text_hash'}->{'aform_afile'}</a>\n";
		}
	elsif ($type == 6 && $a) {
		$edlnk = "<a href='edit_ffile.cgi?file=$val&num=$a->{'num'}'>".
		         "$self->{'text_hash'}->{'aform_afile'}</a>\n";
		}
	print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'aform_val'},"alias_to"),
			      $typesel."\n".$valtxt."\n".$edlnk);
	}

# Table and form end
print $self->ui_table_end();
if ($a) {
	print $self->ui_form_end([ [ "save", $self->{'text_hash'}->{'save'} ],
			     [ "delete", $self->{'text_hash'}->{'delete'} ] ]);
	}
else {
	print $self->ui_form_end([ [ "create", $self->{'text_hash'}->{'create'} ] ]);
	}
}

# create_alias(&details, &files, [norebuild])
# Create a new alias
sub create_alias
{
	my $self = shift;

$self->list_aliases($_[1]);	# force cache init

# Update the config fi;e
local(%aliases);
local $lref = $self->read_file_lines($_[1]->[0]);
$_[0]->{'line'} = scalar(@$lref);
push(@$lref, $self->make_table_comment($_[0]->{'cmt'}, 1));
local $str = ($_[0]->{'enabled'} ? "" : "# ") . $_[0]->{'name'} . ": " .
	     join(',', map { /\s/ ? "\"$_\"" : $_ } @{$_[0]->{'values'}});
push(@$lref, $str);
$_[0]->{'eline'} = scalar(@$lref)-1;
$self->flush_file_lines($_[1]->[0]);
$self->system_logged("newaliases >/dev/null 2>&1") if (!$_[2]);

# Add to the cache
local $jfiles = join(",", @{$_[1]});
local $c = $self->{'list_aliases_cache_hash'}->{$jfiles};
$_[0]->{'file'} = $_[1]->[0];
$_[0]->{'num'} = scalar(@$c);
push(@$c, $_[0]);
}

# delete_alias(&details, [norebuild])
# Deletes one mail alias
sub delete_alias
{
	my $self = shift;

# Remove from the file
local $lref = $self->read_file_lines($_[0]->{'file'});
local $len = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
splice(@$lref, $_[0]->{'line'}, $len);
$self->flush_file_lines($_[0]->{'file'});
$self->system_logged("newaliases >/dev/null 2>&1") if (!$_[1]);

# Remove from the cache
local $jfiles = join(",", @{$_[0]->{'files'}});
local $c = $self->{'list_aliases_cache_hash'}->{$jfiles};
local $idx = $self->indexof($_[0], @$c);
splice(@$c, $idx, 1) if ($idx != -1);
$self->renumber_list($c, $_[0], -$len);
}

# modify_alias(&old, &details, [norebuild])
# Update some existing alias
sub modify_alias
{
	my $self = shift;

# Add to the file
local @newlines;
push(@newlines, $self->make_table_comment($_[1]->{'cmt'}, 1));
local $str = ($_[1]->{'enabled'} ? "" : "# ") . $_[1]->{'name'} . ": " .
	     join(',', map { /\s/ ? "\"$_\"" : $_ } @{$_[1]->{'values'}});
push(@newlines, $str);
local $lref = $self->read_file_lines($_[0]->{'file'});
local $len = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
splice(@$lref, $_[0]->{'line'}, $len, @newlines);
$self->flush_file_lines($_[0]->{'file'});
$self->system_logged("newaliases >/dev/null 2>&1") if (!$_[2]);

local $jfiles = join(",", @{$_[0]->{'files'}});
local $c = $self->{'list_aliases_cache_hash'}->{$jfiles};
local $idx = $self->indexof($_[0], @$c);
$_[1]->{'file'} = $_[0]->{'file'};
$_[1]->{'line'} = $_[0]->{'line'};
$_[1]->{'eline'} = $_[1]->{'line'}+scalar(@newlines)-1;
$c->[$idx] = $_[1] if ($idx != -1);
$self->renumber_list($c, $_[0], scalar(@newlines) - $len);
}

# alias_type(string)
# Return the type and destination of some alias string
sub alias_type
{
	my $self = shift;

local @rv;
if ($_[0] =~ /^\|$self->{'module_config_directory'}\/autoreply.pl\s+(\S+)/) {
	@rv = (5, $1);
	}
elsif ($_[0] =~ /^\|$self->{'module_config_directory'}\/filter.pl\s+(\S+)/) {
	@rv = (6, $1);
	}
elsif ($_[0] =~ /^\|(.*)$/) {
	@rv = (4, $1);
	}
elsif ($_[0] =~ /^(\/.*)$/) {
	@rv = (3, $1);
	}
elsif ($_[0] =~ /^:include:(.*)$/) {
	@rv = (2, $1);
	}
else {
	@rv = (1, $_[0]);
	}
return wantarray ? @rv : $rv[0];
}

# lock_alias_files(&files)
sub lock_alias_files
{
	my $self = shift;

foreach $f (@{$_[0]}) {
	$self->lock_file($f);
	}
}

# unlock_alias_files(&files)
sub unlock_alias_files
{
	my $self = shift;

foreach $f (@{$_[0]}) {
	$self->unlock_file($f);
	}
}

# can_edit_alias(&alias)
sub can_edit_alias
{
	my $self = shift;

local ($a) = @_;
foreach my $v (@{$a->{'values'}}) {
	$self->{'access_hash'}->{"aedit_".$self->alias_type($v)} || return 0;
	}
if ($self->{'access_hash'}->{'amode'} == 2) {
	$a->{'name'} =~ /$self->{'access_hash'}->{'aliases'}/ || return 0;
	}
elsif ($self->{'access_hash'}->{'amode'} == 3) {
	$a->{'name'} eq $self->{'remote_user'} || return 0;
	}
return 1;
}


1;

