#!/usr/bin/perl
package postfix::PostfixLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'postfix::PostfixLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# postfix-lib.pl
# XXX ldap support and multiple servers (mentioned by Joe)
# XXX virtual mail boxes and read mail

#
# postfix-module by Guillaume Cottenceau <gc@mandrakesoft.com>,
# for webmin by Jamie Cameron
#
# Copyright (c) 2000 by Mandrakesoft
#
# Permission to use, copy, modify, and distribute this software and its
# documentation under the terms of the GNU General Public License is hereby 
# granted. No representations are made about the suitability of this software 
# for any purpose. It is provided "as is" without express or implied warranty.
# See the GNU General Public License for more details.
#
#
# Functions for managing the postfix module for Webmin.
#
# Written by G. Cottenceau for MandrakeSoft <gc@mandrakesoft.com>
# This is free software under GPL license.
#

$self->{'POSTFIX_MODULE_VERSION'} = 5;

#
#
# -------------------------------------------------------------------------

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();
$self->{'access_hash'}->{'postfinger'} = 0 if ($self->is_readonly_mode());
&ModuleLoader::preload_module(ref($self), "postfix::AliasesLib");
$self = new postfix::AliasesLib:: ref($self), %$self; 

$self->{'config_hash'}->{'perpage'} ||= 20;      # a value of 0 can cause problems

# Get the saved version number
if ($self->open_readfile(*VERSION, "$self->{'module_config_directory'}/version")) {
	chop($self->{'postfix_version'} = <VERSION>);
	close(VERSION);
	}
else {
	# Not there .. work it out
	if ($self->has_command($self->{'config_hash'}->{'postfix_config_command'}) &&
	    $self->backquote_command("$self->{'config_hash'}->{'postfix_config_command'} mail_version 2>&1", 1) =~ /mail_version\s*=\s*(.*)/) {
		# Got the version
		$self->{'postfix_version'} = $1;
		}
	}

if ($self->{'postfix_version'} >= 2) {
	$self->{'virtual_maps'} = "virtual_alias_maps";
	$self->{'ldap_timeout'} = "ldap_timeout";
	}
else {
	$self->{'virtual_maps'} = "virtual_maps";
	$self->{'ldap_timeout'} = "ldap_lookup_timeout";
	}

$self->{'config_dir'} = $self->guess_config_dir();

## DOC: compared to other webmin modules, here we don't need to parse
##      the config file, because a config command is provided by
##      postfix to read and write the config parameters

############################################################################
# aliases support    [too lazy to create a aliases-lib.pl :-)]

############################################################################
# maps [canonical, virtual, transport] support

@{$self->{'header_checks_actions_array'}} = ( "REJECT", "HOLD", "REDIRECT", "DUNNO", "IGNORE",
			   "DISCARD", "FILTER",
			   "PREPEND", "REPLACE", "WARN" );

@{$self->{'check_sender_actions_array'}} = ( "OK", "REJECT", "DISCARD", "FILTER", "PREPEND", 
        "REDIRECT", "WARN", "DUNNO" );			   

1;

1;
		return $self;
}

sub guess_config_dir
{
	my $self = shift;

    my $answ = $self->{'config_hash'}->{'postfix_config_file'};
    $answ =~ /(.*)\/[^\/]*/;
    return $1;
}

# postfix_module_version()
# returns the version of the postfix module
sub postfix_module_version
{
	my $self = shift;

    return $self->{'POSTFIX_MODULE_VERSION'};
}

# is_postfix_running()
# returns 1 if running, 0 if stopped, calls error() if problem
sub is_postfix_running
{
	my $self = shift;

    my $queuedir = $self->get_current_value("queue_directory");
    my $processid = $self->get_current_value("process_id_directory");

    my $pid_file = $queuedir."/".$processid."/master.pid";
    my $pid = $self->check_pid_file($pid_file);
    return $pid ? 1 : 0;
}

sub is_existing_parameter
{
	my $self = shift;

    my $out = $self->backquote_command("$self->{'config_hash'}->{'postfix_config_command'} -c $self->{'config_dir'} $_[0] 2>&1", 1);
    return !($out =~ /unknown parameter/);
}

# get_current_value(parameter_name)
# returns a scalar corresponding to the value of the parameter
## modified to allow main_parameter:subparameter 
sub get_current_value
{
	my $self = shift;

# First try to get the value from main.cf directly
my ($name,$key)=split /:/,$_[0];
my $lref = $self->read_file_lines($self->{'config_hash'}->{'postfix_config_file'});
my $out;
foreach my $l (@$lref) {
	if ($l =~ /^\s*([a-z0-9\_]+)\s*=\s*(.*)/ &&
	    $1 eq $name) {
		# Found it! Return ..
		$out = $2;
		last;
		}
	}
if (!defined($out)) {
	# Fall back to asking Postfix
	$out = $self->backquote_command("$self->{'config_hash'}->{'postfix_config_command'} -c $self->{'config_dir'} -h $name 2>&1", 1);  # -h tells postconf not to output the name of the parameter
	if ($?) { $self->error($self->text('query_get_efailed', $name, $out)); }
	elsif ($out =~ /warning:.*unknown\s+parameter/) {
		return undef;
		}
	chop($out);
	}
if ($key) {
	my @res=();
	foreach (split /,/,$out)
	    { push @res, $1 if /$key\s+(.+)/; }
	if ($#res>0) {$out=join ", ",@res}
	else {$out=$res[0]}
	}
return $out;
}

# if_default_value(parameter_name)
# returns if the value is the default value
sub if_default_value
{
	my $self = shift;

    my $out = $self->backquote_command("$self->{'config_hash'}->{'postfix_config_command'} -c $self->{'config_dir'} -n $_[0] 2>&1", 1);
    if ($?) { $self->error($self->text('query_get_efailed', $_[0], $out)); }
    return ($out eq "");
}

# get_default_value(parameter_name)
# returns the default value of the parameter
sub get_default_value
{
	my $self = shift;

    my $out = $self->backquote_command("$self->{'config_hash'}->{'postfix_config_command'} -c $self->{'config_dir'} -dh $_[0] 2>&1", 1);  # -h tells postconf not to output the name of the parameter
    if ($?) { $self->error($self->text('query_get_efailed', $_[0], $out)); }
    chop($out);
    return $out;
}

# set_current_value(parameter_name, parameter_value)
# 
sub set_current_value
{
	my $self = shift;

    my $value = $_[1];
#    print "--".$value."--<br>";
    if (($value eq "__DEFAULT_VALUE_IE_NOT_IN_CONFIG_FILE__") || ($value eq $self->get_default_value($_[0])))
    {
	# there is a special case in which there is no static default value ;
	# postfix will handle it correctly if I remove the line in `main.cf'
	my $all_lines = $self->read_file_lines($self->{'config_hash'}->{'postfix_config_file'});
	my $line_of_parameter = -1;
	my $i = 0;

	foreach (@{$all_lines})
	{
	    if (/^\s*$_[0]\s*=/)
	    {
		$line_of_parameter = $i;
	    }
	    $i++;
	}

	if ($line_of_parameter != -1) {
	    splice(@{$all_lines}, $line_of_parameter, 1);
	    $self->flush_file_lines($self->{'config_hash'}->{'postfix_config_file'});
	} else {
	    $self->unflush_file_lines($self->{'config_hash'}->{'postfix_config_file'});
	}
    }
    else
    {
	$value =~ s/\$/\\\$/g;     # prepend a \ in front of every $ to protect from shell substitution
        local ($out, $ex);
	$ex = $self->execute_command("$self->{'config_hash'}->{'postfix_config_command'} -c $self->{'config_dir'} -e $_[0]=\"$value\"", undef, \$out, \$out);
	$ex && $self->error($self->text('query_set_efailed', $_[0], $_[1], $out)."<br> $self->{'config_hash'}->{'postfix_config_command'} -c $self->{'config_dir'} -e $_[0]=\"$value\" 2>&1");
    }
}

# check_postfix()
#
sub check_postfix
{
	my $self = shift;

	my $cmd = "$self->{'config_hash'}->{'postfix_control_command'} -c $self->{'config_dir'} check";
	my $out = $self->backquote_command("$cmd 2>&1 </dev/null", 1);
	my $ex = $?;
	if ($ex && $self->foreign_check("proc")) {
		# Get a better error message
		$self->foreign_require("proc", "proc-lib.pl");
		$out = $main::proc_object->pty_backquote("$cmd 2>&1 </dev/null");
		}
	return $ex ? ($out || "$cmd failed") : undef;
}

# reload_postfix()
#
sub reload_postfix
{
	my $self = shift;

    $self->{'access_hash'}->{'startstop'} || $self->error($self->{'text_hash'}->{'reload_ecannot'});
    if ($self->is_postfix_running())
    {
	if ($self->check_postfix()) { $self->error("$self->{'text_hash'}->{'check_error'}"); }
	my $out = $self->system_logged("$self->{'config_hash'}->{'postfix_control_command'} -c $self->{'config_dir'} reload >/dev/null 2>&1");
	if ($out) { $self->error($self->{'text_hash'}->{'reload_efailed'}); }
    }
}

# stop_postfix()
# Attempts to stop postfix, returning undef on success or an error message
sub stop_postfix
{
	my $self = shift;

local $out = $self->backquote_logged("$self->{'config_hash'}->{'postfix_control_command'} -c $self->{'config_dir'} stop 2>&1");
return $? ? "<tt>$out</tt>" : undef;
}

# start_postfix()
# Attempts to start postfix, returning undef on success or an error message
sub start_postfix
{
	my $self = shift;

local $out = $self->backquote_logged("$self->{'config_hash'}->{'postfix_control_command'} -c $self->{'config_dir'} start 2>&1");
return $? ? "<tt>$out</tt>" : undef;
}

# option_radios_freefield(name_of_option, length_of_free_field, [name_of_radiobutton, text_of_radiobutton]+)
# builds an option with variable number of radiobuttons and a free field
# WARNING: *FIRST* RADIO BUTTON *MUST* BE THE DEFAULT VALUE OF POSTFIX
sub option_radios_freefield
{
	my $self = shift;

    my ($name, $length) = ($_[0], $_[1]);

    my $v = $self->get_current_value($name);
    my $key = 'opts_'.$name;

    my $check_free_field = 1;
    
    printf "<td>".$self->hlink("<b>$self->{'text_hash'}->{$key}</b>", "opt_".$name)."</td> <td %s nowrap>\n",
    $length > 20 ? "colspan=3" : "";

    # first radio button (must be default value!!)
    
    print $self->ui_oneradio($name."_def", "__DEFAULT_VALUE_IE_NOT_IN_CONFIG_FILE__",
		       $_[2], $self->if_default_value($name));

    $check_free_field = 0 if $self->if_default_value($name);
    shift;
    
    # other radio buttons
    while (defined($_[2]))
    {
	print $self->ui_oneradio($name."_def", $_[2], $_[3], $v eq $_[2]);
	if ($v eq $_[2]) { $check_free_field = 0; }
	shift;
	shift;
    }

    # the free field
    print $self->ui_oneradio($name."_def", "__USE_FREE_FIELD__", undef,
		       $check_free_field == 1);
    print $self->ui_textbox($name, $check_free_field == 1 ? $v : undef, $length);
    print "</td>\n";
}

# option_freefield(name_of_option, length_of_free_field)
# builds an option with free field
sub option_freefield
{
	my $self = shift;

    my ($name, $length) = ($_[0], $_[1]);

    my $v = $self->get_current_value($name);
    my $key = 'opts_'.$name;
    
    printf "<td>".$self->hlink("<b>$self->{'text_hash'}->{$key}</b>", "opt_".$name)."</td> <td %s nowrap>\n",
    $length > 20 ? "colspan=3" : "";
    
    print $self->ui_textbox($name."_def", $v, $length),"</td>\n";
}

# option_yesno(name_of_option, [help])
# if help is provided, displays help link
sub option_yesno
{
	my $self = shift;

    my $name = $_[0];
    my $v = $self->get_current_value($name);
    my $key = 'opts_'.$name;

    defined($_[1]) ?
	print "<td>".$self->hlink("<b>$self->{'text_hash'}->{$key}</b>", "opt_".$name)."</td> <td nowrap>\n"
    :
	print "<td><b>$self->{'text_hash'}->{$key}</b></td> <td nowrap>\n";
    
    print $self->ui_radio($name."_def", lc($v),
		    [ [ "yes", $self->{'text_hash'}->{'yes'} ], [ "no", $self->{'text_hash'}->{'no'} ] ]);
    print "</td>\n";
}

# option_select(name_of_option, &options, [help])
# Shows a drop-down menu of options
sub option_select
{
	my $self = shift;

    my $name = $_[0];
    my $v = $self->get_current_value($name);
    my $key = 'opts_'.$name;

    defined($_[2]) ?
	print "<td>".$self->hlink("<b>$self->{'text_hash'}->{$key}</b>", "opt_".$name)."</td> <td nowrap>\n"
    :
	print "<td><b>$self->{'text_hash'}->{$key}</b></td> <td nowrap>\n";
    
    print $self->ui_select($name."_def", lc($v), $_[1]);
    print "</td>\n";
}

# $self->get_aliases_files($alias_maps) : @aliases_files
# parses its argument to extract the filenames of the aliases files
# supports multiple alias-files
sub get_aliases_files
{
	my $self = shift;

    $_[0] =~ /:(\/[^,\s]*)(.*)/;
    (my $returnvalue, my $recurse) = ( $1, $2 );

    # Yes, Perl is also a functional language -> I construct a list, and no problem, lists are flattened in Perl
    return ( $returnvalue,
	     ($recurse =~ /:\/[^,\s]*/) ?
	         $self->get_aliases_files($recurse)
	     :
	         ()
           )
}

# get_aliases() : \@aliases
# construct the aliases database taken from the aliases files given in the "alias_maps" parameter
sub get_aliases
{
	my $self = shift;

    if (!@{$self->{'aliases_cache_array'}})
    {
	my @aliases_files = $self->get_aliases_files($self->get_current_value("alias_maps"));
	my $number = 0;
	foreach $aliases_file (@aliases_files)
	{
	    $self->open_readfile(*ALIASES, $aliases_file);
	    my $i = 0;
	    while (<ALIASES>)
	    {
		s/^#.*$//g;	# remove comments
		s/\r|\n//g;	# remove newlines
		if ((/^\s*\"([^\"]*)[^:]*:\s*([^#]*)/) ||      # names with double quotes (") are special, as seen in `man aliases(5)`
		    (/^\s*([^\s:]*)[^:]*:\s*([^#]*)/))         # other names
		{
		    $number++;
		    my %alias;
		    $alias{'name'} = $1;
		    $alias{'value'} = $2;
		    $alias{'line'} = $i;
		    $alias{'alias_file'} = $aliases_file;
		    $alias{'number'} = $number;
		    push(@{$self->{'aliases_cache_array'}}, \%alias);
		}
		$i++;
	    }
	    close(ALIASES);
	}
    }
    return \@{$self->{'aliases_cache_array'}};
}

# init_new_alias() : $number
# gives a new number of alias
sub init_new_alias
{
	my $self = shift;

    $self->{'aliases'} = $self->get_aliases();

    my $max_number = 0;

    foreach $trans (@{$self->{'aliases'}})
    {
	if ($trans->{'number'} > $max_number) { $max_number = $trans->{'number'}; }
    }
    
    return $max_number+1;
}

# renumber_list(&list, &position-object, lines-offset)
sub renumber_list
{
	my $self = shift;

return if (!$_[2]);
local $e;
foreach $e (@{$_[0]}) {
	next if (defined($e->{'alias_file'}) &&
	         $e->{'alias_file'} ne $_[1]->{'alias_file'});
	next if (defined($e->{'map_file'}) &&
	         $e->{'map_file'} ne $_[1]->{'map_file'});
	$e->{'line'} += $_[2] if ($e->{'line'} > $_[1]->{'line'});
	$e->{'eline'} += $_[2] if (defined($e->{'eline'}) &&
				   $e->{'eline'} > $_[1]->{'eline'});
	}
}

# save_options(%options)
#
sub save_options
{
	my $self = shift;

    if ($self->check_postfix()) { $self->error("$self->{'text_hash'}->{'check_error'}"); }

    my %options = %{$_[0]};

    foreach $key (keys %options)
    {
	if ($key =~ /_def/)
	{
	    (my $param = $key) =~ s/_def//;
	    my $value = $options{$key} eq "__USE_FREE_FIELD__" ?
			$options{$param} : $options{$key};
            if ($value =~ /^(\S+):/ && $self->{'access_hash'}->{'dir'} ne '/') {
		foreach my $f ($self->get_maps_files($value)) {
		   if (!$self->is_under_directory($self->{'access_hash'}->{'dir'}, $f)) {
			$self->error($self->text('opts_edir', $self->{'access_hash'}->{'dir'}));
		   }
		}
            }
	    $self->set_current_value($param, $value);
	}
    }
}

# regenerate_aliases
#
sub regenerate_aliases
{
	my $self = shift;

    local $out;
    $self->{'access_hash'}->{'aliases'} || $self->error($self->{'text_hash'}->{'regenerate_ecannot'});
    if ($self->get_current_value("alias_maps") eq "")
    {
	$out = $self->backquote_logged("$self->{'config_hash'}->{'postfix_newaliases_command'} 2>&1");
	if ($?) { $self->error($self->text('regenerate_alias_efailed', $out)); }
    }
    else
    {
	local $map;
	foreach $map ($self->get_maps_files($self->get_real_value("alias_maps")))
	{
	    $out = $self->backquote_logged("$self->{'config_hash'}->{'postfix_aliases_table_command'} -c $self->{'config_dir'} $map 2>&1");
	    if ($?) { $self->error($self->text('regenerate_table_efailed', $map, $out)); }
	}
    }
}

# regenerate_relocated_table()
sub regenerate_relocated_table
{
	my $self = shift;

    $self->regenerate_any_table("relocated_maps");
}

# regenerate_virtual_table()
sub regenerate_virtual_table
{
	my $self = shift;

    $self->regenerate_any_table($self->{'virtual_maps'});
}

# regenerate_header_table()
sub regenerate_header_table
{
	my $self = shift;

    $self->regenerate_any_table("header_checks");
}

# regenerate_body_table()
sub regenerate_body_table
{
	my $self = shift;

    $self->regenerate_any_table("body_checks");
}

# regenerate_canonical_table
#
sub regenerate_canonical_table
{
	my $self = shift;

    $self->regenerate_any_table("canonical_maps");
    $self->regenerate_any_table("recipient_canonical_maps");
    $self->regenerate_any_table("sender_canonical_maps");
}

# regenerate_transport_table
#
sub regenerate_transport_table
{
	my $self = shift;

    $self->regenerate_any_table("transport_maps");
}

# regenerate_any_table($parameter_where_to_find_the_table_names,
#		       [ &force-files ])
#
sub regenerate_any_table
{
	my $self = shift;

    local @files;
    if ($_[1]) {
	@files = @{$_[1]};
    } elsif ($self->get_current_value($_[0]) ne "") {
	@files = $self->get_maps_files($self->get_real_value($_[0]));
    }
    foreach $map (@files)
    {
        next unless $map;
        local $out = $self->backquote_logged("$self->{'config_hash'}->{'postfix_lookup_table_command'} -c $self->{'config_dir'} $map 2>&1");
        if ($out) { $self->error($self->text('regenerate_table_efailed', $map, $out)); }
    }
}

# $self->get_maps_files($maps_param) : @maps_files
# parses its argument to extract the filenames of the mapping files
# supports multiple maps-files
sub get_maps_files
{
	my $self = shift;

    $_[0] =~ /:(\/[^,\s]*)(.*)/ || return ( );
    (my $returnvalue, my $recurse) = ( $1, $2 );

    return ( $returnvalue,
	     ($recurse =~ /:\/[^,\s]*/) ?
	         $self->get_maps_files($recurse)
	     :
	         ()
           )
}

# get_maps($maps_name, [&force-files]) : \@maps
# Construct the mappings database taken from the map files given from the
# parameters.
sub get_maps
{
	my $self = shift;

    if (!defined($self->{'maps_cache_hash'}->{$_[0]}))
    {
	my @maps_files = $_[1] ? @{$_[1]} : $self->get_maps_files($self->get_real_value($_[0]));
	my $number = 0;
	foreach $maps_file (@maps_files)
	{
	    $self->open_readfile(*MAPS, $maps_file);
	    my $i = 0;
	    my $cmt;
	    while (<MAPS>)
	    {
		s/\r|\n//g;	# remove newlines
		if (/^\s*#+\s*(.*)/) {
		    # A comment line
		    $cmt = $self->is_table_comment($_);
		    }
		elsif (/^\s*(\/[^\/]*\/[a-z]*)\s+([^#]*)/ ||
		       /^\s*([^\s]+)\s+([^#]*)/) {
		    # An actual map
		    $number++;
		    my %map;
		    $map{'name'} = $1;
		    $map{'value'} = $2;
		    $map{'line'} = $cmt ? $i-1 : $i;
		    $map{'eline'} = $i;
		    $map{'map_file'} = $maps_file;
		    $map{'file'} = $maps_file;
		    $map{'number'} = $number;
		    $map{'cmt'} = $cmt;
		    push(@{$self->{'maps_cache_hash'}->{$_[0]}}, \%map);
		    $cmt = undef;
		    }
		else {
		    $cmt = undef;
		    }
		$i++;
	    }
	    close(MAPS);
	}
    }
    return $self->{'maps_cache_hash'}->{$_[0]};
}

# generate_map_edit(name, desc, [wide], [nametitle], [valuetitle])
sub generate_map_edit
{
	my $self = shift;

    if ($self->get_current_value($_[0]) eq "")
    {
	print ("<h2>$self->{'text_hash'}->{'no_map2'}</h2><br>");
	print "<hr>\n";
	$self->footer("", $self->{'text_hash'}->{'index_return'});
	exit;
    }


    my $mappings = $self->get_maps($_[0]);
    my $nt = $_[3] || $self->{'text_hash'}->{'mapping_name'};
    my $vt = $_[4] || $self->{'text_hash'}->{'mapping_value'};

    if ($#{$mappings} ne -1)
    {
	print $_[1];
	
	print $self->ui_form_start("delete_mappings.cgi", "post");
	print $self->ui_hidden("map_name", $_[0]),"\n";
	local @links = ( $self->select_all_link("d", 1),
			 $self->select_invert_link("d", 1) );
	print $self->ui_links_row(\@links);
	print "<table width=100%> <tr><td width=50% valign=top>\n";
	
	local @tds = ( "width=5" );
	print $self->ui_columns_start(
			[ "", $nt, $vt,
			  $self->{'config_hash'}->{'show_cmts'} ? ( $self->{'text_hash'}->{'mapping_cmt'} ) : ( )
			], 100, 0, \@tds);
	my $split_index = int(($#{$mappings})/2);
	my $i = -1;
	
	if ($self->{'config_hash'}->{'sort_mode'} == 1) {
		if ($_[0] eq $self->{'virtual_maps'}) {
			@{$mappings} = sort sort_by_domain @{$mappings};
			}
		else {
			@{$mappings} = sort { $a->{'name'} cmp $b->{'name'} }
					    @{$mappings};
			}
		}
	foreach $map (@{$mappings})
	{
	    local @cols = ( "<a href=\"edit_mapping.cgi?num=$map->{'number'}&map_name=$_[0]\">$map->{'name'}</a>",
			    $map->{'value'} );
	    push(@cols, $self->html_escape($map->{'cmt'})) if ($self->{'config_hash'}->{'show_cmts'});
	    print $self->ui_checked_columns_row(\@cols, \@tds, "d", $map->{'name'});
	    $i++;
	    if ($i == $split_index && !$_[2] && $self->{'config_hash'}->{'columns'} == 2)
	    {
		# Switch to second table
		print $self->ui_columns_end();
		print "</td><td width=50% valign=top>\n";
		if ($i == @$mappings -1) {
			# No more to show!
			print $self->ui_columns_start([ ]);
			}
		else {
			print $self->ui_columns_start([ "", $nt, $vt ], 100, 0,\@tds);
			}
	    }
	}
	
	print $self->ui_columns_end();
	print "</td></tr></table>\n";
	print $self->ui_links_row(\@links);
	print $self->ui_form_end([ [ "delete", $self->{'text_hash'}->{'mapping_delete'} ] ]);
    }


    # new form
    print $self->ui_buttons_start();
    print $self->ui_buttons_row("edit_mapping.cgi", $self->{'text_hash'}->{'new_mapping'},
			  $self->{'text_hash'}->{'new_mappingmsg'},
			  $self->ui_hidden("map_name", $_[0]));
    if ($self->{'access_hash'}->{'manual'}) {
	    print $self->ui_buttons_row("edit_manual.cgi", $self->{'text_hash'}->{'new_manual'},
				  $self->{'text_hash'}->{'new_manualmsg'},
				  $self->ui_hidden("map_name", $_[0]));
	    }
    print $self->ui_buttons_end();

}

# create_mapping(map, &mapping, [&force-files])
sub create_mapping
{
	my $self = shift;

$self->get_maps($_[0], $_[2]);	# force cache init
my @maps_files = $_[2] ? @{$_[2]} : $self->get_maps_files($self->get_real_value($_[0]));
local $lref = $self->read_file_lines($maps_files[0]);
$_[1]->{'line'} = scalar(@$lref);
push(@$lref, $self->make_table_comment($_[1]->{'cmt'}));
push(@$lref, "$_[1]->{'name'}\t$_[1]->{'value'}");
$_[1]->{'eline'} = scalar(@$lref)-1;
$self->flush_file_lines();

$_[1]->{'map_file'} = $maps_files[0];
$_[1]->{'file'} = $maps_files[0];
$_[1]->{'number'} = scalar(@{$self->{'maps_cache_hash'}->{$_[0]}});
push(@{$self->{'maps_cache_hash'}->{$_[0]}}, $_[1]);
}

# delete_mapping(map, &mapping)
sub delete_mapping
{
	my $self = shift;

local $lref = $self->read_file_lines($_[1]->{'map_file'});
local $len = $_[1]->{'eline'} - $_[1]->{'line'} + 1;
splice(@$lref, $_[1]->{'line'}, $len);
$self->flush_file_lines();

local $idx = $self->indexof($_[1], @{$self->{'maps_cache_hash'}->{$_[0]}});
splice(@{$self->{'maps_cache_hash'}->{$_[0]}}, $idx, 1) if ($idx != -1);
$self->renumber_list($self->{'maps_cache_hash'}->{$_[0]}, $_[1], -$len);
}

# modify_mapping(map, &oldmapping, &newmapping)
sub modify_mapping
{
	my $self = shift;

local $lref = $self->read_file_lines($_[1]->{'map_file'});
local $oldlen = $_[1]->{'eline'} - $_[1]->{'line'} + 1;
local @newlines;
push(@newlines, $self->make_table_comment($_[2]->{'cmt'}));
push(@newlines, "$_[2]->{'name'}\t$_[2]->{'value'}");
splice(@$lref, $_[1]->{'line'}, $oldlen, @newlines);
$self->flush_file_lines();

local $idx = $self->indexof($_[1], @{$self->{'maps_cache_hash'}->{$_[0]}});
$_[2]->{'map_file'} = $_[1]->{'map_file'};
$_[2]->{'file'} = $_[1]->{'file'};
$_[2]->{'line'} = $_[1]->{'line'};
$_[2]->{'eline'} = $_[2]->{'cmt'} ? $_[1]->{'line'}+1 : $_[1]->{'line'};
$self->{'maps_cache_hash'}->{$_[0]}->[$idx] = $_[2] if ($idx != -1);
$self->renumber_list($self->{'maps_cache_hash'}->{$_[0]}, $_[1], scalar(@newlines)-$oldlen);
}

# init_new_mapping($maps_parameter) : $number
# gives a new number of mapping
sub init_new_mapping
{
	my $self = shift;

    $self->{'maps'} = $self->get_maps($_[0]);

    my $max_number = 0;

    foreach $trans (@{$self->{'maps'}})
    {
	if ($trans->{'number'} > $max_number) { $max_number = $trans->{'number'}; }
    }
    
    return $max_number+1;
}

# postfix_mail_file(user)
sub postfix_mail_file
{
	my $self = shift;

local @s = $self->postfix_mail_system();
if ($s[0] == 0) {
	return "$s[1]/$_[0]";
	}
elsif (@_ > 1) {
	return "$_[7]/$s[1]";
	}
else {
	local @u = getpwnam($_[0]);
	return "$u[7]/$s[1]";
	}
}

# postfix_mail_system()
# Returns 0 and the spool dir for sendmail style,
#         1 and the mbox filename for ~/Mailbox style
#         2 and the maildir name for ~/Maildir style
sub postfix_mail_system
{
	my $self = shift;

if (!defined(@{$self->{'mail_system_cache_array'}})) {
	local $home_mailbox = $self->get_current_value("home_mailbox");
	local $mail_spool_directory =$self->get_current_value("mail_spool_directory");
	if ($home_mailbox) {
		@{$self->{'mail_system_cache_array'}} = $home_mailbox =~ /^(.*)\/$/ ?
			(2, $1) : (1, $home_mailbox);
		}
	else {
		@{$self->{'mail_system_cache_array'}} = (0, $mail_spool_directory);
		}
	}
return wantarray ? @{$self->{'mail_system_cache_array'}} : $self->{'mail_system_cache_array'}->[0];
}

# list_queue()
# Returns a list of strutures, each containing details of one queued message
sub list_queue
{
	my $self = shift;

local @qfiles;
$self->open_execute_command(*MAILQ, $self->{'config_hash'}->{'mailq_cmd'}, 1, 1);
while(<MAILQ>) {
	next if (/^(\S+)\s+is\s+empty/i || /^\s+Total\s+requests:/i);
	if (/^([^\s\*\!]+)[\*\!]?\s*(\d+)\s+(\S+\s+\S+\s+\d+\s+\d+:\d+:\d+)\s+(.*)/) {
		push(@qfiles, { 'id' => $1,
			        'size' => $2,
				'date' => $3,
				'from' => $4 });
		}
	elsif (/\((.*)\)/ && @qfiles) {
		$qfiles[$#qfiles]->{'status'} = $1;
		}
	elsif (/^\s+(\S+)/ && @qfiles) {
		$qfiles[$#qfiles]->{'to'} .= "$1 ";
		}
	}
close(MAILQ);
return @qfiles;
}

# parse_queue_file(id)
# Parses a postfix mail queue file into a standard mail structure
sub parse_queue_file
{
	my $self = shift;

local @qfiles = ( $self->recurse_files("$self->{'config_hash'}->{'mailq_dir'}/active"),
		  $self->recurse_files("$self->{'config_hash'}->{'mailq_dir'}/incoming"),
		  $self->recurse_files("$self->{'config_hash'}->{'mailq_dir'}/deferred"),
		  $self->recurse_files("$self->{'config_hash'}->{'mailq_dir'}/corrupt"),
		  $self->recurse_files("$self->{'config_hash'}->{'mailq_dir'}/hold"),
		);
local $f = $_[0];
local ($file) = grep { $_ =~ /\/$f$/ } @qfiles;
return undef if (!$file);
local $mode = 0;
local ($mail, @headers);
$self->open_execute_command(*QUEUE, "$self->{'config_hash'}->{'postcat_cmd'} ".quotemeta($file), 1, 1);
while(<QUEUE>) {
	if (/^\*\*\*\s+MESSAGE\s+CONTENTS/ && !$mode) {	   # Start of headers
		$mode = 1;
		}
	elsif (/^\*\*\*\s+HEADER\s+EXTRACTED/ && $mode) {  # End of email
		last;
		}
	elsif ($mode == 1 && /^\s*$/) {			   # End of headers
		$mode = 2;
		}
	elsif ($mode == 1 && /^(\S+):\s*(.*)/) {	   # Found a header
		push(@headers, [ $1, $2 ]);
		}
	elsif ($mode == 1 && /^(\s+.*)/) {		   # Header continuation
		$headers[$#headers]->[1] .= $1 unless($#headers < 0);
		}
	elsif ($mode == 2) {				   # Part of body
		$mail->{'size'} += length($_);
		$mail->{'body'} .= $_;
		}
	}
close(QUEUE);
$mail->{'headers'} = \@headers;
foreach $h (@headers) {
	$mail->{'header'}->{lc($h->[0])} = $h->[1];
	}
return $mail;
}

# $self->recurse_files(dir)
sub recurse_files
{
	my $self = shift;

opendir(DIR, $self->translate_filename($_[0])) || return ( $_[0] );
local @dir = readdir(DIR);
closedir(DIR);
local ($f, @rv);
foreach $f (@dir) {
	push(@rv, $self->recurse_files("$_[0]/$f")) if ($f !~ /^\./);
	}
return @rv;
}

sub sort_by_domain
{
	my $self = shift;

local ($a1, $a2, $b1, $b2);
if ($self->{'a'}->{'name'} =~ /^(.*)\@(.*)$/ && (($a1, $a2) = ($1, $2)) &&
    $self->{'b'}->{'name'} =~ /^(.*)\@(.*)$/ && (($b1, $b2) = ($1, $2))) {
	return $a2 cmp $b2 ? $a2 cmp $b2 : $a1 cmp $b1;
	}
else {
	return $self->{'a'}->{'name'} cmp $self->{'b'}->{'name'};
	}
}

# before_save()
# Copy the postfix config file to a backup file, for reversion if
# a post-save check fails
sub before_save
{
	my $self = shift;

if ($self->{'config_hash'}->{'check_config'} && !defined($self->{'save_file'})) {
	$self->{'save_file'} = $self->transname();
	$self->execute_command("cp $self->{'config_hash'}->{'postfix_config_file'} $self->{'save_file'}");
	}
}

sub after_save
{
	my $self = shift;

if (defined($self->{'save_file'})) {
	local $err = $self->check_postfix();
	if ($err) {
		$self->execute_command("mv $self->{'save_file'} $self->{'config_hash'}->{'postfix_config_file'}");
		$self->error($self->text('after_err', "<pre>$err</pre>"));
		}
	else {
		unlink($self->{'save_file'});
		$self->{'save_file'} = undef;
		}
	}
}

# $self->get_real_value(parameter_name)
# Returns the value of a parameter, with $ substitions done
sub get_real_value
{
	my $self = shift;

my $v = $self->get_current_value($_[0]);
$v =~ s/\$(\{([^\}]+)\}|([A-Za-z0-9\.\-\_]+))/$self->get_real_value($2 || $3)/ge;
return $v;
}

# ensure_map(name)
# Create some map text file, if needed
sub ensure_map
{
	my $self = shift;

foreach my $mf ($self->get_maps_files($self->get_real_value($_[0]))) {
	if ($mf =~ /^\// && !-e $mf) {
		$self->open_lock_tempfile(TOUCH, ">$mf", 1) ||
			$self->error($self->text("efilewrite", $mf, $!));
		$self->close_tempfile(TOUCH);
		$self->set_ownership_permissions(undef, undef, 0755, $mf);
		}
	}
}

# Functions for editing the header_checks map nicely
sub edit_name_header_checks
{
	my $self = shift;

return $self->ui_table_row($self->{'text_hash'}->{'header_name'},
		     $self->ui_textbox("name", $_[0]->{'name'}, 60));
}

sub parse_name_header_checks
{
	my $self = shift;

$_[1]->{'name'} =~ /^\/\S+\/[a-z]*$/ || $self->error($self->{'text_hash'}->{'header_ename'});
return $_[1]->{'name'};
}

sub edit_value_header_checks
{
	my $self = shift;

local ($act, $dest) = split(/\s+/, $_[0]->{'value'}, 2);
return $self->ui_table_row($self->{'text_hash'}->{'header_value'},
              $self->ui_select("action", $act,
			 [ map { [ $_, $self->{'text_hash'}->{'header_'.lc($_)} ] }
			       @{$self->{'header_checks_actions_array'}} ], 0, 0, $act)."\n".
	      $self->ui_textbox("value", $dest, 40));
}

sub parse_value_header_checks
{
	my $self = shift;

local $rv = $_[1]->{'action'};
if ($_[1]->{'value'}) {
	$rv .= " ".$_[1]->{'value'};
	}
return $rv;
}

# Functions for editing the body_checks map (same as header_checks)
sub edit_name_body_checks
{
	my $self = shift;

return $self->edit_name_header_checks(@_);
}

sub parse_name_body_checks
{
	my $self = shift;

return $self->parse_name_header_checks(@_);
}

sub edit_value_body_checks
{
	my $self = shift;

return $self->edit_value_header_checks(@_);
}

sub parse_value_body_checks
{
	my $self = shift;

return $self->parse_value_header_checks(@_);
}

## added function for sender_access_maps
## added function for client_access_maps
sub edit_name_check_sender_access
{
	my $self = shift;

return "<td><b>$self->{'text_hash'}->{'access_addresses'}</b></td>\n".
       "<td>".$self->ui_textbox("name", $_[0]->{'name'},40)."</td>\n";
}

sub edit_value_check_sender_access
{
	my $self = shift;

local ($act, $dest) = split(/\s+/, $_[0]->{'value'}, 2);
return "<td><b>$self->{'text_hash'}->{'header_value'}</b></td>\n".
       "<td>".$self->ui_select("action", $act,
			 [ map { [ $_, $self->{'text_hash'}->{'header_'.lc($_)} ] }
			       @{$self->{'check_sender_actions_array'}} ], 0, 0, $act)."\n".
	      $self->ui_textbox("value", $dest, 40)."</td>\n";
}

sub parse_value_check_sender_access
{
	my $self = shift;

return $self->parse_value_header_checks(@_);
}

# get_master_config()
# Returns an array reference of entries from the Postfix master.cf file
sub get_master_config
{
	my $self = shift;

if (!defined(@{$self->{'master_config_cache_array'}})) {
	@{$self->{'master_config_cache_array'}} = ( );
	local $lnum = 0;
	local $prog;
	open(MASTER, $self->{'config_hash'}->{'postfix_master'});
	while(<MASTER>) {
		s/\r|\n//g;
		if (/^(#?)\s*(\S+)\s+(inet|unix|fifo)\s+(y|n|\-)\s+(y|n|\-)\s+(y|n|\-)\s+(\S+)\s+(\S+)\s+(.*)$/) {
			# A program line
			$prog = { 'enabled' => !$1,
				  'name' => $2,
				  'type' => $3,
				  'private' => $4,
				  'unpriv' => $5,
				  'chroot' => $6,
				  'wakeup' => $7,
				  'maxprocs' => $8,
				  'command' => $9,
				  'line' => $lnum,
				  'eline' => $lnum,
				 };
			push(@{$self->{'master_config_cache_array'}}, $prog);
			}
		elsif (/^(#?)\s+(.*)$/ && $prog &&
		       $prog->{'eline'} == $lnum-1 &&
		       $prog->{'enabled'} == !$1) {
			# Continuation line
			$prog->{'command'} .= " ".$2;
			$prog->{'eline'} = $lnum;
			}
		$lnum++;
		}
	close(MASTER);
	}
return \@{$self->{'master_config_cache_array'}};
}

# create_master(&master)
# Adds a new Postfix server process
sub create_master
{
	my $self = shift;

local ($master) = @_;
local $conf = $self->get_master_config();
local $lref = $self->read_file_lines($self->{'config_hash'}->{'postfix_master'});
push(@$lref, $self->master_line($master));
$self->flush_file_lines($self->{'config_hash'}->{'postfix_master'});
$master->{'line'} = scalar(@$lref)-1;
$master->{'eline'} = scalar(@$lref)-1;
push(@$conf, $master);
}

# delete_master(&master)
# Removes one Postfix server process
sub delete_master
{
	my $self = shift;

local ($master) = @_;
local $conf = $self->get_master_config();
local $lref = $self->read_file_lines($self->{'config_hash'}->{'postfix_master'});
local $lines = $master->{'eline'} - $master->{'line'} + 1;
splice(@$lref, $master->{'line'}, $lines);
$self->flush_file_lines($self->{'config_hash'}->{'postfix_master'});
@$conf = grep { $_ ne $master } @$conf;
foreach my $c (@$conf) {
	if ($c->{'line'} > $master->{'eline'}) {
		$c->{'line'} -= $lines;
		$c->{'eline'} -= $lines;
		}
	}
}

# modify_master(&master)
# Updates one Postfix server process
sub modify_master
{
	my $self = shift;

local ($master) = @_;
local $conf = $self->get_master_config();
local $lref = $self->read_file_lines($self->{'config_hash'}->{'postfix_master'});
local $lines = $master->{'eline'} - $master->{'line'} + 1;
splice(@$lref, $master->{'line'}, $lines,
       $self->master_line($master));
$self->flush_file_lines($self->{'config_hash'}->{'postfix_master'});
foreach my $c (@$conf) {
	if ($c->{'line'} > $master->{'eline'}) {
		$c->{'line'} -= $lines-1;
		$c->{'eline'} -= $lines-1;
		}
	}
}

# master_line(&master)
sub master_line
{
	my $self = shift;

local ($prog) = @_;
return ($prog->{'enabled'} ? "" : "#").
       join("\t", $prog->{'name'}, $prog->{'type'}, $prog->{'private'},
		  $prog->{'unpriv'}, $prog->{'chroot'}, $prog->{'wakeup'},
		  $prog->{'maxprocs'}, $prog->{'command'});
}

sub redirect_to_map_list
{
	my $self = shift;

local ($map_name) = @_;
if ($map_name =~ /transport/) { $self->redirect("transport.cgi"); }
elsif ($map_name =~ /canonical/) { $self->redirect("canonical.cgi"); }
elsif ($map_name =~ /virtual/) { $self->redirect("virtual.cgi"); }
elsif ($map_name =~ /relocated/) { $self->redirect("relocated.cgi"); }
elsif ($map_name =~ /header/) { $self->redirect("header.cgi"); }
elsif ($map_name =~ /body/) { $self->redirect("body.cgi"); }
elsif ($map_name =~ /sender_access/) { $self->redirect("edit_access.cgi?name=smtpd_client_restrictions:check_sender_access&title=Check+sender+access+mapping+table"); }
else { $self->redirect(""); }
}

sub regenerate_map_table
{
	my $self = shift;

local ($map_name) = @_;
if ($map_name =~ /canonical/) { $self->regenerate_canonical_table(); }
if ($map_name =~ /relocated/) { $self->regenerate_relocated_table(); }
if ($map_name =~ /virtual/) { $self->regenerate_virtual_table(); }
if ($map_name =~ /transport/) { $self->regenerate_transport_table(); }
if ($map_name =~ /sender_access/) { $self->regenerate_any_table($map_name); }
}

# mailq_table(&qfiles)
# Print a table of queued mail messages
sub mailq_table
{
	my $self = shift;

local ($qfiles) = @_;

# Show queued messages
print "<form action=delete_queues.cgi method=post>\n";
local @links = ( $self->select_all_link("file", 0),
		 $self->select_invert_link("file", 0) );
print $self->ui_links_row(\@links);

# Table header
local @tds = ( "width=5" );
print $self->ui_columns_start([
	"",
	$self->{'text_hash'}->{'mailq_id'},
	$self->{'text_hash'}->{'mailq_date'},
	$self->{'text_hash'}->{'mailq_from'},
	$self->{'text_hash'}->{'mailq_to'},
	$self->{'text_hash'}->{'mailq_size'},
	$self->{'text_hash'}->{'mailq_status'} ], 100, 0, \@tds);
foreach my $q (@$qfiles) {
	local @cols;
	push(@cols, "<a href='view_mailq.cgi?id=$q->{'id'}'>$q->{'id'}</a>");
	local $size = $self->nice_size($q->{'size'});
	push(@cols, "<font size=1>$q->{'date'}</font>");
	push(@cols, "<font size=1>".$self->html_escape($q->{'from'})."</font>");
	push(@cols, "<font size=1>".$self->html_escape($q->{'to'})."</font>");
	push(@cols, "<font size=1>$size</font>");
	push(@cols, "<font size=1>".$self->html_escape($q->{'status'})."</font>");
	print $self->ui_checked_columns_row(\@cols, \@tds, "file", $q->{'id'});
	}
print $self->ui_columns_end();
print $self->ui_links_row(\@links);
print "<input type=submit value='$self->{'text_hash'}->{'mailq_delete'}'>\n";
if ($self->{'postfix_version'} >= 1.1) {
	# Show button to re-queue
	print "&nbsp;\n";
	print $self->ui_submit($self->{'text_hash'}->{'mailq_move'}, "move");
	}
if ($self->{'postfix_version'} >= 2) {
	# Show button to hold and un-hold
	print "&nbsp;\n";
	print $self->ui_submit($self->{'text_hash'}->{'mailq_hold'}, "hold");
	print $self->ui_submit($self->{'text_hash'}->{'mailq_unhold'}, "unhold");
	}
print "<p>\n";
print "</form>\n";
}

# is_table_comment(line, [force-prefix])
# Returns the comment text if a line contains a comment, like # foo
sub is_table_comment
{
	my $self = shift;

local ($line, $force) = @_;
if ($self->{'config_hash'}->{'prefix_cmts'} || $force) {
	return $line =~ /^\s*#+\s*Webmin:\s*(.*)/ ? $1 : undef;
	}
else {
	return $line =~ /^\s*#+\s*(.*)/ ? $1 : undef;
	}
}

# make_table_comment(comment, [force-tag])
# Returns an array of lines for a comment in a map file, like # foo
sub make_table_comment
{
	my $self = shift;

local ($cmt, $force) = @_;
if (!$cmt) {
	return ( );
	}
elsif ($self->{'config_hash'}->{'prefix_cmts'} || $force) {
	return ( "# Webmin: $cmt" );
	}
else {
	return ( "# $cmt" );
	}
}

# lock_postfix_files()
# Lock all Postfix config files
sub lock_postfix_files
{
	my $self = shift;

$self->lock_file($self->{'config_hash'}->{'postfix_config_file'});
$self->lock_file($self->{'config_hash'}->{'postfix_master'});
}

# unlock_postfix_files()
# Un-lock all Postfix config files
sub unlock_postfix_files
{
	my $self = shift;

$self->unlock_file($self->{'config_hash'}->{'postfix_config_file'});
$self->unlock_file($self->{'config_hash'}->{'postfix_master'});
}

#parses file $_[0] and returns a reference to an array of hashes (spliter - \s)
sub parse_master_file
{
	my $self = shift;

 my $number,@options,$arg,$conf_file;
 open (INFL,$_[0]) || $self->error($self->text('ffile_efile',"$_[0]"));
 while (<INFL>)
 {
  $number++;
  chomp;
  unless ($_=~/^#/ || $_=~/^$/)
  {
   if (!($_=~/^\s/ or $_=~/^\t/))
   {
    @options=split (/\s+/,$_);
    push (@{$conf_file},{'service'=>$options[0],
	                 'type'=>$options[1],
			 'private'=>$options[2],
			 'unpriv'=>$options[3],
			 'chroot'=>$options[4],
			 'wakeup'=>$options[5],
			 'maxproc'=>$options[6],
			 'command'=>$options[7],});
   }
   else
   {
    $arg=pop(@{$conf_file});
    push (@{$arg->{'args'}},$_);
    push (@{$conf_file},$arg);
   }
  }
 }
 close INFL;
 return $conf_file;
}

#set new value $_[0] of service with name $_[0]->{'service'} from array $_[1]
#or add new string to array $_[1]
sub set_master_value
{
	my $self = shift;

 my $str_old;
 foreach (@{$_[1]})
 {
  if ($_->{'service'} eq $_[0]->{'service'})
  {
   $_->{'type'}=$_[0]->{'type'};
   $_->{'private'}=$_[0]->{'private'};
   $_->{'unpriv'}=$_[0]->{'unpriv'};
   $_->{'chroot'}=$_[0]->{'chroot'};
   $_->{'wakeup'}=$_[0]->{'wakeup'};
   $_->{'maxproc'}=$_[0]->{'maxproc'};
   $_->{'command'}=$_[0]->{'command'};
   $_->{'args'}=$_[0]->{'args'};
   $str_old=1;
  }
 }
 push (@{$_[1]},{'service' => $_[0]->{'service'},
		 'type' => $_[0]->{'type'},
		 'private' => $_[0]->{'private'},
		 'unpriv' => $_[0]->{'unpriv'},
		 'chroot' => $_[0]->{'chroot'},
		 'wakeup' => $_[0]->{'wakeup'},
		 'maxproc' => $_[0]->{'maxproc'},
		 'command' => $_[0]->{'command'},
		 'args' => $_[0]->{'args'}}) if ($str_old!=1);
}

#save array $_[1] to file $_[0]
sub save_master_file
{
	my $self = shift;

 my $i,$m,$k,$ch,$old,$temp,$str_new,@str_old,@new;
 $self->lock_file($_[0]);
 my $old_file= $self->read_file_lines($_[0]);

 foreach (@{$_[1]})
 {
  $old=0;
  foreach $i (@{$old_file})
  {
   unless ($i=~/^#/ || $i=~/^$/)
   {
    @str_old=split (/\s+/,$i);
    if (($_->{'service'} eq $str_old[0]) && ($_->{'command'} eq $str_old[7]))
    { $i="$_->{'service'}\t$_->{'type'}\t$_->{'private'}\t$_->{'unpriv'}\t$_->{'chroot'}\t$_->{'wakeup'}\t$_->{'maxproc'}\t$_->{'command'}";
     $temp=0;
     $ch=1;
     $old=1;
    }
    elsif (($i=~/^\s+/ || $i=~/^\t+/) && $ch)
    {
     $i="$_->{'args'}[$temp]";
     $temp++;
    }
    else {$ch=0;}
   }
  }
  unless ($old)
  {
   push(@new,$_);
  }
 }

 foreach (@new)
 { $str_new="$_->{'service'}\t$_->{'type'}\t$_->{'private'}\t$_->{'unpriv'}\t$_->{'chroot'}\t$_->{'wakeup'}\t$_->{'maxproc'}\t$_->{'command'}";
  push (@{$old_file},$str_new);
  foreach $i (@{$_->{'args'}})
  {
   push (@{$old_file},$i);
  }
 }
 $self->flush_file_lines();
 $self->unlock_file($_[0]);
}


1;

