#!/usr/bin/perl

&MAIN();

 sub MAIN {
#
# postfix-module by Guillaume Cottenceau <gc@mandrakesoft.com>,
# for webmin by Jamie Cameron
#
# Copyright (c) 2000 by Mandrakesoft
#
# Permission to use, copy, modify, and distribute this software and its
# documentation under the terms of the GNU General Public License is hereby 
# granted. No representations are made about the suitability of this software 
# for any purpose. It is provided "as is" without express or implied warranty.
# See the GNU General Public License for more details.
#
#
# A word about this module.
#
# Postfix provides a command to control its parameters: `postconf'.
# That's the reason why I don't parse and set the values manually.
# It's much better because it can resist to changes of the Postfix
# config files.
#
# However, to `set back to default' an already changed parameter,
# there is no way to do it in the case of dynamic parameters.
# [example: I mean that for `static' parameters, which defaults to
# `0', I can set the parameter to `0' ; but for `dynamic'
# parameters such as domainname [which comes from a system call]
# I have no way]
# So for this special case, I parse the config file, and delete
# manually the correct line.
#
# gc.
#

&ModuleLoader::preload_module('postfix_class', "postfix::PostfixLib");
$self = new postfix::PostfixLib:: 'postfix_class',%$self;


if ($self->has_command($self->{'config_hash'}->{'postfix_config_command'}) &&
    $self->backquote_command("$self->{'config_hash'}->{'postfix_config_command'} mail_version 2>&1", 1) =~ /mail_version\s*=\s*(.*)/) {
	# Got the version
	$self->{'postfix_version'} = $1;
	}
$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", "intro", 1, 1, 0,
	$self->help_search_link("postfix", "man", "doc", "google"),
	undef, undef, $self->{'postfix_version'} ?
		$self->text('index_version', $self->{'postfix_version'}) : undef);

# Save the version for use by other CGIs
$self->open_tempfile(VERSION, ">$self->{'module_config_directory'}/version", 0, 1);
$self->print_tempfile(VERSION, "$self->{'postfix_version'}\n");
$self->close_tempfile(VERSION);

# Verify the postfix control command
if (!-x $self->{'config_hash'}->{'postfix_control_command'}) {
	print $self->text('index_epath',
		"<tt>$self->{'config_hash'}->{'postfix_control_command'}</tt>",
		"../config.cgi?$self->{'module_name'}"),"<p>\n";

	$self->foreign_require("software", "software-lib.pl");
	$self->{'lnk'} = $main::software_object->missing_install_link(
			"postfix", $self->{'text_hash'}->{'index_postfix'},
			"../$self->{'module_name'}/", $self->{'text_hash'}->{'index_title'});
	print $self->{'lnk'},"<p>\n" if ($self->{'lnk'});

	$self->ui_print_footer("/", $self->{'text_hash'}->{'index'});
	exit;
	}

# Verify the postfix config command
if (!-x $self->{'config_hash'}->{'postfix_config_command'}) {
	print $self->text('index_econfig',
		"<tt>$self->{'config_hash'}->{'postfix_config_command'}</tt>",
		"../config.cgi?$self->{'module_name'}"),"<p>\n";
	$self->ui_print_footer("/", $self->{'text_hash'}->{'index'});
	exit;
	}

# Verify the postsuper command
if (!-x $self->{'config_hash'}->{'postfix_super_command'}) {
	print $self->text('index_esuper',
		"<tt>$self->{'config_hash'}->{'postfix_super_command'}</tt>",
		"../config.cgi?$self->{'module_name'}"),"<p>\n";
	$self->ui_print_footer("/", $self->{'text_hash'}->{'index'});
	exit;
	}

# Verify that current configuration is valid
if ($self->{'config_hash'}->{'index_check'} && ($self->{'err'} = $self->check_postfix())) {
	print $self->text('check_error'),"<p>\n";
	print "<pre>$self->{'err'}</pre>\n";
	$self->ui_print_footer("/", $self->{'text_hash'}->{'index'});
	exit;
	}

@{$self->{'onames_array'}} =  ( "general", "address_rewriting", "aliases", "canonical", "virtual", "transport", "relocated", "header", "body",
	     "local_delivery", "resource",
	     "smtpd", "smtp",
	     "rate", "debug", $self->{'postfix_version'} > 2 ? ( ) : ( "ldap" ),
	     "master", "mailq", "postfinger", "boxes", "manual" );

$self->{'access_hash'}->{'boxes'} = $self->foreign_available("mailboxes");
foreach $oitem (@{$self->{'onames_array'}})
{
	if ($self->{'access_hash'}->{$oitem}) {
		push (@{$self->{'olinks_array'}}, $oitem eq "boxes" ? "../mailboxes/"
						 : $oitem . ".cgi");
		push (@{$self->{'otitles_array'}}, $oitem eq 'manual' ? $self->{'text_hash'}->{'cmanual_title'}
						   : $self->{'text_hash'}->{$oitem . "_title"});
		if ($oitem eq 'mailq' && !$self->{'config_hash'}->{'mailq_count'}) {
			# Count the queue
			local @mqueue = $self->list_queue();
			local $mcount = scalar(@mqueue);
			$self->{'otitles_array'}->[$#{$self->{'otitles_array'}}] .=
				"<br>".$self->text('mailq_count', $mcount);
			}
		push (@{$self->{'oicons_array'}}, "images/" . $oitem . ".gif");
	}
}

$self->icons_table(\@{$self->{'olinks_array'}}, \@{$self->{'otitles_array'}}, \@{$self->{'oicons_array'}});

if ($self->{'access_hash'}->{'startstop'})
{
    print "<hr>\n";

    if ($self->is_postfix_running())
    {
	print "<table cellpadding=5 width=100%><tr><td>\n";
	print "<form action=stop.cgi>\n";
	print "<input type=submit value=\"$self->{'text_hash'}->{'index_stop'}\">\n";
	print "</td> <td>$self->{'text_hash'}->{'index_stopmsg'}\n";
    }
    else
    {
	print "<table cellpadding=5 width=100%><tr><td>\n";
	print "<form action=start.cgi>\n";
	print "<input type=submit value=\"$self->{'text_hash'}->{'index_start'}\">\n";
	print "</td> <td>$self->{'text_hash'}->{'index_startmsg'}\n";
    }
    print "</td></tr></table></form>\n";
}

$self->ui_print_footer("/", $self->{'text_hash'}->{'index'});

1;
		
}


1;

