#!/usr/bin/perl
package postgresql::PostgresqlLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'postgresql::PostgresqlLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# postgresql-lib.pl
# Common PostgreSQL functions
# XXX updating date field
# XXX access control and schema tables

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
if ($self->{'config_hash'}->{'plib'}) {
	$ENV{$self->{'gconfig_hash'}->{'ld_env'}} .= ':' if ($ENV{$self->{'gconfig_hash'}->{'ld_env'}});
	$ENV{$self->{'gconfig_hash'}->{'ld_env'}} .= $self->{'config_hash'}->{'plib'};
	}
if ($self->{'config_hash'}->{'psql'} =~ /^(.*)\/bin\/psql$/ && $1 ne '' && $1 ne '/usr') {
	$ENV{$self->{'gconfig_hash'}->{'ld_env'}} .= ':' if ($ENV{$self->{'gconfig_hash'}->{'ld_env'}});
	$ENV{$self->{'gconfig_hash'}->{'ld_env'}} .= "$1/lib";
	}

if ($self->{'module_info_hash'}->{'usermin'}) {
	# Login and password is set by user in Usermin, and the module always
	# runs as the Usermin user
	$self->switch_to_remote_user();
	$self->create_user_config_dirs();
	$self->{'postgres_login'} = $self->{'userconfig_hash'}->{'login'};
	$self->{'postgres_pass'} = $self->{'userconfig_hash'}->{'pass'};
	$self->{'postgres_sameunix'} = 0;
	%{$self->{'access_hash'}} = ( 'backup' => 1,
		    'restore' => 1,
		    'tables' => 1,
		    'cmds' => 1, );
	$self->{'max_dbs'} = $self->{'userconfig_hash'}->{'max_dbs'};
	}
else {
	# Login and password is determined by ACL in Webmin
	%{$self->{'access_hash'}} = $self->get_module_acl();
	if ($self->{'access_hash'}->{'user'} && !$self->{'use_global_login'}) {
		$self->{'postgres_login'} = $self->{'access_hash'}->{'user'};
		$self->{'postgres_pass'} = $self->{'access_hash'}->{'pass'};
		$self->{'postgres_sameunix'} = $self->{'access_hash'}->{'sameunix'};
		}
	else {
		$self->{'postgres_login'} = $self->{'config_hash'}->{'login'};
		$self->{'postgres_pass'} = $self->{'config_hash'}->{'pass'};
		$self->{'postgres_sameunix'} = $self->{'config_hash'}->{'sameunix'};
		}
	$self->{'max_dbs'} = $self->{'config_hash'}->{'max_dbs'};
	}
$self->{'cron_cmd'} = "$self->{'module_config_directory'}/backup.pl";

if (!$self->{'config_hash'}->{'nodbi'}) {
	# Check if we have DBD::Pg
	eval <<EOF;
use DBI;
\$self->{'driver_handle'} = DBI->install_driver("Pg");
EOF
	}

1;

1;
		return $self;
}

# is_postgresql_running()
# Returns 1 if yes, 0 if no, -1 if the login is invalid, -2 if there
# is a library problem. When called in an array context, returns the full error
# message too.
sub is_postgresql_running
{
	my $self = shift;

local $temp = $self->transname();
local $host = $self->{'config_hash'}->{'host'} ? "-h $self->{'config_hash'}->{'host'}" : "";
$host .= " -p $self->{'config_hash'}->{'port'}" if ($self->{'config_hash'}->{'port'});
local $cmd = $self->quote_path($self->{'config_hash'}->{'psql'}).
	     (!$self->supports_pgpass() ? " -u" : " -U $self->{'postgres_login'}").
	     " -c '' $host $self->{'config_hash'}->{'basedb'}";
if ($self->{'postgres_sameunix'} && defined(getpwnam($self->{'postgres_login'}))) {
	$cmd = "su $self->{'postgres_login'} -c ".quotemeta($cmd);
	}
$cmd = $self->command_with_login($cmd);
if ($self->foreign_check("proc")) {
	$self->foreign_require("proc", "proc-lib.pl");
	if ($main::proc_object->UNIVERSAL::can("close_controlling_pty")) {
		# Detach from tty if possible, so that the psql
		# command doesn't prompt for a login
		$main::proc_object->close_controlling_pty();
		}
	}
open(OUT, "$cmd 2>&1 |");
while(<OUT>) { $self->{'out'} .= $_; }
close(OUT);
unlink($temp);
local $rv;
if ($self->{'out'} =~ /setuserid:/i || $self->{'out'} =~ /no\s+password\s+supplied/i ||
    $self->{'out'} =~ /no\s+postgres\s+username/i || $self->{'out'} =~ /authentication\s+failed/i ||
    $self->{'out'} =~ /password:.*password:/i || $self->{'out'} =~ /database.*does.*not/i ||
    $self->{'out'} =~ /user.*does.*not/i) {
	$rv = -1;
	}
elsif ($self->{'out'} =~ /connect.*failed/i || $self->{'out'} =~ /could not connect to server:/) {
	$rv = 0;
	}
elsif ($self->{'out'} =~ /lib\S+\.so/i) {
	$rv = -2;
	}
else {
	$rv = 1;
	}
return wantarray ? ($rv, $self->{'out'}) : $rv;
}

# get_postgresql_version([from-command])
sub get_postgresql_version
{
	my $self = shift;

local ($fromcmd) = @_;
local $main::error_must_die = 1;
return $self->{'postgresql_version_cache'} if (defined($self->{'postgresql_version_cache'}));
local $rv;
if (!$fromcmd) {
	eval {
		local $v = $self->execute_sql_safe($self->{'config_hash'}->{'basedb'},
					     'select version()');
		$v = $v->{'data'}->[0]->[0];
		if ($v =~ /postgresql\s+([0-9\.]+)/i) {
			$rv = $1;
			}
		};
	}
if (!$rv || $@) {
	local $out = $self->backquote_command($self->quote_path($self->{'config_hash'}->{'psql'})." -V 2>&1 <$self->{'null_file'}");
	$rv = $out =~ /\s([0-9\.]+)/ ? $1 : undef;
	}
$self->{'postgresql_version_cache'} = $rv;
return $rv;
}

sub can_drop_fields
{
	my $self = shift;

return $self->get_postgresql_version() >= 7.3;
}

# list_databases()
# Returns a list of all databases
sub list_databases
{
	my $self = shift;

$self->{'force_nodbi'} = 1;
local $t = $self->execute_sql_safe($self->{'config_hash'}->{'basedb'}, 'select * from pg_database order by datname');
return map { $_->[0] } @{$t->{'data'}};
}

# supports_schemas(database)
# Returns 1 if schemas are supported
sub supports_schemas
{
	my $self = shift;

local $t = $self->execute_sql_safe($_[0], "select a.attname FROM pg_class c, pg_attribute a, pg_type t WHERE c.relname = 'pg_tables' and a.attnum > 0 and a.attrelid = c.oid and a.atttypid = t.oid and a.attname = 'schemaname' order by attnum");
return $t->{'data'}->[0]->[0] ? 1 : 0;
}

# list_tables(database)
# Returns a list of tables in some database
sub list_tables
{
	my $self = shift;

if ($self->supports_schemas($_[0])) {
	local $t = $self->execute_sql_safe($_[0], 'select schemaname,tablename from pg_tables where tablename not like \'pg_%\' and tablename not like \'sql_%\' order by tablename');
	return map { ($_->[0] eq "public" ? "" : $_->[0].".").$_->[1] } @{$t->{'data'}};
	}
else {
	local $t = $self->execute_sql_safe($_[0], 'select tablename from pg_tables where tablename not like \'pg_%\' and tablename not like \'sql_%\' order by tablename');
	return map { $_->[0] } @{$t->{'data'}};
	}
}

# list_types()
# Returns a list of all available field types
sub list_types
{
	my $self = shift;

local $t = $self->execute_sql_safe($self->{'config_hash'}->{'basedb'}, 'select typname from pg_type where typrelid = 0 and typname !~ \'^_.*\' order by typname');
local @types = map { $_->[0] } @{$t->{'data'}};
push(@types, "serial", "bigserial") if ($self->get_postgresql_version() >= 7.4);
return sort { $a cmp $b } $self->unique(@types);
}

# table_structure(database, table)
# Returns a list of hashes detailing the structure of a table
sub table_structure
{
	my $self = shift;

if ($self->supports_schemas($_[0])) {
	# Find the schema and table
	local ($tn, $ns);
	if ($_[1] =~ /^(\S+)\.(\S+)$/) {
		$ns = $1;
		$tn = $2;
		}
	else {
		$ns = "public";
		$tn = $_[1];
		}
	$tn =~ s/^([^\.]+)\.//;
	local $t = $self->execute_sql_safe($_[0], "select a.attnum, a.attname, t.typname, a.attlen, a.atttypmod, a.attnotnull, a.atthasdef FROM pg_class c, pg_attribute a, pg_type t, pg_namespace ns WHERE c.relname = '$tn' and ns.nspname = '$ns' and a.attnum > 0 and a.attrelid = c.oid and a.atttypid = t.oid and a.attname not like '%pg.dropped%' and c.relnamespace = ns.oid order by attnum");
	local (@rv, $r);
	foreach $r (@{$t->{'data'}}) {
		local $arr;
		$arr++ if ($r->[2] =~ s/^_//);
		local $sz = $r->[4] - 4;
		if ($sz >= 65536 && $r->[2] =~ /numeric/i) {
			$sz = int($sz/65536).",".($sz%65536);
			}
		push(@rv, { 'field' => $r->[1],
			    'arr' => $arr ? 'YES' : 'NO',
			    'type' => $r->[4] < 0 ? $r->[2]
						  : $r->[2]."($sz)",
			    'null' => $r->[5] =~ /f|0/ ? 'YES' : 'NO' } );
		}

	# Work out which fields are the primary key
	if ($self->supports_indexes()) {
		local ($keyidx) = grep { $_ eq $_[1]."_pkey" }
				       $self->list_indexes($_[0]);
		if ($keyidx) {
			local $istr = $self->index_structure($_[0], $keyidx);
			foreach my $r (@rv) {
				if ($self->indexof($r->{'field'},
					     @{$istr->{'cols'}}) >= 0) {
					$r->{'key'} = 'PRI';
					}
				}
			}
		}

	return @rv;
	}
else {
	# Just look by table name
	local $t = $self->execute_sql_safe($_[0], "select a.attnum, a.attname, t.typname, a.attlen, a.atttypmod, a.attnotnull, a.atthasdef FROM pg_class c, pg_attribute a, pg_type t WHERE c.relname = '$_[1]' and a.attnum > 0 and a.attrelid = c.oid     and a.atttypid = t.oid order by attnum");
	local (@rv, $r);
	foreach $r (@{$t->{'data'}}) {
		local $arr;
		$arr++ if ($r->[2] =~ s/^_//);
		local $sz = $r->[4] - 4;
		if ($sz >= 65536 && $r->[2] =~ /numeric/i) {
			$sz = int($sz/65536).",".($sz%65536);
			}
		push(@rv, { 'field' => $r->[1],
			    'arr' => $arr ? 'YES' : 'NO',
			    'type' => $r->[4] < 0 ? $r->[2]
						  : $r->[2]."($sz)",
			    'null' => $r->[5] =~ /f|0/ ? 'YES' : 'NO' } );
		}
	return @rv;
	}
}

# execute_sql(database, sql, [param, ...])
sub execute_sql
{
	my $self = shift;

if ($self->is_readonly_mode()) {
	return { };
	}
$self->execute_sql_safe(@_);
}

# execute_sql_safe(database, sql, [param, ...])
sub execute_sql_safe
{
	my $self = shift;

local $sql = $_[1];
local @params = @_[2..$#_];
if ($self->{'driver_handle'} &&
    $sql !~ /^\s*(create|drop)\s+database/ && $sql !~ /^\s*\\/ &&
    !$self->{'force_nodbi'}) {
	# Use the DBI interface
	local $pid;
	local $cstr = "dbname=$_[0]";
	$cstr .= ";host=$self->{'config_hash'}->{'host'}" if ($self->{'config_hash'}->{'host'});
	$cstr .= ";port=$self->{'config_hash'}->{'port'}" if ($self->{'config_hash'}->{'port'});
	local @uinfo;
	if ($self->{'postgres_sameunix'} &&
	    defined(@uinfo = getpwnam($self->{'postgres_login'}))) {
		# DBI call which must run in subprocess
		pipe(OUTr, OUTw);
		if (!($pid = fork())) {
			($(, $)) = ( $uinfo[3], $uinfo[3] );
			($>, $<) = ( $uinfo[2], $uinfo[2] );
			close(OUTr);
			local $dbh = $self->{'driver_handle'}->connect($cstr,
					$self->{'postgres_login'}, $self->{'postgres_pass'});
			if (!$dbh) {
				print OUTw $self->serialise_variable(
				    "DBI connect failed : ".$DBI::errstr);
				exit(0);
				}
			$dbh->{'AutoCommit'} = 0;
			local $cmd = $dbh->prepare($sql);
			#foreach (@params) {	# XXX dbd quoting is broken!
			#	s/\\/\\\\/g;
			#	}
			if (!$cmd->execute(@params)) {
				print OUTw $self->serialise_variable($self->text('esql',
				    "<tt>".$self->html_escape($sql)."</tt>",
				    "<tt>".$self->html_escape($dbh->errstr)."</tt>"));
				$dbh->disconnect();
				exit(0);
				}
			local (@data, @row);
			local @titles = @{$cmd->{'NAME'}};
			while(@row = $cmd->fetchrow()) {
				push(@data, [ @row ]);
				}
			$cmd->finish();
			$dbh->commit();
			$dbh->disconnect();
			print OUTw $self->serialise_variable(
					      { 'titles' => \@titles,
						'data' => \@data });
			exit(0);
			}
		close(OUTw);
		local $line = <OUTr>;
		local $rv = $self->unserialise_variable($line);
		if (ref($rv)) {
			return $rv;
			}
		else {
			$self->error($rv || "$sql : Unknown DBI error");
			}
		}
	else {
		# Just normal DBI call
		local $dbh = $self->{'driver_handle'}->connect($cstr,
				$self->{'postgres_login'}, $self->{'postgres_pass'});
		$dbh || $self->error("DBI connect failed : ",$DBI::errstr);
		$dbh->{'AutoCommit'} = 0;
		local $cmd = $dbh->prepare($sql);
		if (!$cmd->execute(@params)) {
			$self->error($self->text('esql', "<tt>".$self->html_escape($sql)."</tt>",
				     "<tt>".$self->html_escape($dbh->errstr)."</tt>"));
			}
		local (@data, @row);
		local @titles = @{$cmd->{'NAME'}};
		while(@row = $cmd->fetchrow()) {
			push(@data, [ @row ]);
			}
		$cmd->finish();
		$dbh->commit();
		$dbh->disconnect();
		return { 'titles' => \@titles,
			 'data' => \@data };
		}
	}
else {
	# Check for a \ command
        my $break_f = 0 ;
        if ( $sql =~ /^\s*\\/ ) {
            $break_f = 1 ;
            if ( $sql !~ /^\s*\\copy\s+/ &&
                 $sql !~ /^\s*\\i\s+/ ) {
                $self->error( $self->text( 'r_command', ) ) ;
            }
        }

	if (@params) {
		# Sub in ? parameters
		local $p;
		local $pos = -1;
		foreach $p (@params) {
			$pos = index($sql, '?', $pos+1);
			$self->error("Incorrect number of parameters in $_[1] (".scalar(@params).")") if ($pos < 0);
			local $qp = $p;
			if ($qp !~ /^[bB]'\d+'$/) {
				# Quote value, except for bits
				$qp =~ s/\\/\\\\/g;
				$qp =~ s/'/''/g;
				$qp =~ s/\$/\\\$/g;
				$qp =~ s/\n/\\n/g;
				$qp = $qp eq '' ? "NULL" : "'$qp'";
				}
			$sql = substr($sql, 0, $pos).$qp.substr($sql, $pos+1);
			$pos += length($qp)-1;
			}
		}

	# Call the psql program
	local $host = $self->{'config_hash'}->{'host'} ? "-h $self->{'config_hash'}->{'host'}" : "";
	$host .= " -p $self->{'config_hash'}->{'port'}" if ($self->{'config_hash'}->{'port'});
	local $cmd = $self->quote_path($self->{'config_hash'}->{'psql'}).
		     (!$self->supports_pgpass() ? " -u" : " -U $self->{'postgres_login'}").
		     " -c ".$self->quote_path($sql)." $host $_[0]";
	if ($self->{'postgres_sameunix'} && defined(getpwnam($self->{'postgres_login'}))) {
		$cmd = "su $self->{'postgres_login'} -c ".quotemeta($cmd);
		}
	$cmd = $self->command_with_login($cmd);

	delete($ENV{'LANG'});		# to force output to english
	delete($ENV{'LANGUAGE'});
        if ( $break_f == 0 ) {
		# Running a normal SQL command, not one with a \
		#$ENV{'PAGER'} = "cat";
		if ($self->foreign_check("proc")) {
			$self->foreign_require("proc", "proc-lib.pl");
			if ($main::proc_object->UNIVERSAL::can("close_controlling_pty")) {
				# Detach from tty if possible, so that the psql
				# command doesn't prompt for a login
				$main::proc_object->close_controlling_pty();
				}
			}
		open(OUT, "$cmd 2>&1 |");
		local ($line, $rv, @data);
		do {
			$line = <OUT>;
			last if (!defined($line));
			} while($line =~ /^(username|password|user name):/i ||
				$line =~ /(warning|notice):/i || $line !~ /\S/);
		unlink($self->{'temp'});
		if ($line =~ /^ERROR:\s+(.*)/ || $line =~ /FATAL.*:\s+(.*)/) {
			$self->error($self->text('esql', "<tt>$sql</tt>", "<tt>$1</tt>"));
			}
		elsif (!defined($line)) {
			# Un-expected end of output ..
			$self->error($self->text('esql', "<tt>$sql</tt>",
				     "<tt>$self->{'config_hash'}->{'psql'} failed</tt>"));
			}
		else {
			local $dash = <OUT>;
			if ($dash =~ /^\s*\+\-/) {
				# mysql-style output
				$line = <OUT>;
				$line =~ s/^[\s\|]+//; $line =~ s/[\s\|]+$//;
				local @titles = split(/\|/, $line);
				map { s/^\s+//; s/\s+$// } @titles;
				$line = <OUT>;	# skip useless dashes
				while(1) {
					$line = <OUT>;
					last if (!$line || $line =~ /^\s*\+/);
					$line =~ s/^[\s\|]+//;
					$line =~ s/[\s\|]+$//;
					local @row = split(/\|/, $line);
					map { s/^\s+//; s/\s+$// } @row;
					push(@data, \@row);
					}
				$rv = { 'titles' => \@titles, 'data' => \@data };
				}
			elsif ($dash !~ /^-/) {
				# no output, such as from an insert
				$rv = undef;
				}
			else {
				# psql-style output
				local @titles = split(/\|/, $line);
				map { s/^\s+//; s/\s+$// } @titles;
				while(1) {
					$line = <OUT>;
					last if (!$line ||
						 $line =~ /^\(\d+\s+\S+\)/);
					local @row = split(/\|/, $line);
					map { s/^\s+//; s/\s+$// } @row;
					push(@data, \@row);
					}
				$rv = { 'titles' => \@titles,
					'data' => \@data };
				}
			}
		close(OUT);
		return $rv;
		}
	else {
		# Running a special \ command
		local ( @titles, @row, @data, $rc, $emsgf, $emsg ) ;

		$emsgf = $self->transname();
		$rc = $self->system_logged( "$cmd >$emsgf 2>&1");
		$emsg  = `cat $emsgf` ;
		unlink ( $emsgf ) ;
		if ($rc) {
			$self->error( "<pre>$emsg</pre>" );
			}
		else {
			@titles = ( "     Command Invocation      " ) ;
			@row    = ( "   Done ( return code : $rc )" ) ;
			map { s/^\s+//; s/\s+$// } @row ;
			push ( @data, \@row ) ;
			return { 'titles' => \@titles, 'data' => \@data } ;
			}
		}
	}
}

# execute_sql_logged(database, command)
sub execute_sql_logged
{
	my $self = shift;

$self->additional_log('sql', $_[0], $_[1]);
return $self->execute_sql(@_);
}

sub can_edit_db
{
	my $self = shift;

if ($self->{'module_info_hash'}->{'usermin'}) {
	# Check access control list in configuration
	local $rv;
	DB: foreach $l (split(/\t/, $self->{'config_hash'}->{'access'})) {
		if ($l =~ /^(\S+):\s*(.*)$/ &&
		    ($1 eq $self->{'remote_user'} || $1 eq '*')) {
			local @dbs = split(/\s+/, $2);
			foreach $d (@dbs) {
				$d =~ s/\$REMOTE_USER/$self->{'remote_user'}/g;
				if ($d eq '*' || $_[0] =~ /^$d$/) {
					$rv = 1;
					last DB;
					}
				}
			$rv = 0;
			last DB;
			}
		}
	if ($rv && $self->{'config_hash'}->{'access_own'}) {
		# Check ownership on DB - first get login ID
		if (!defined($self->{'postgres_login_id'})) {
			local $d = $self->execute_sql($self->{'config_hash'}->{'basedb'}, "select usesysid from pg_user where usename = ?", $self->{'postgres_login'});
			$self->{'postgres_login_id'} = $d->{'data'}->[0]->[0];
			}
		# Get database owner
		local $d = $self->execute_sql($self->{'config_hash'}->{'basedb'}, "select datdba from pg_database where datname = ?", $_[0]);
		if ($d->{'data'}->[0]->[0] != $self->{'postgres_login_id'}) {
			$rv = 0;
			}
		}
	return $rv;
	}
else {
	# Check Webmin ACL
	local $d;
	return 1 if ($self->{'access_hash'}->{'dbs'} eq '*');
	foreach $d (split(/\s+/, $self->{'access_hash'}->{'dbs'})) {
		return 1 if ($d && $d eq $_[0]);
		}
	return 0;
	}
}

# get_hba_config(version)
# Parses the postgres host access config file
sub get_hba_config
{
	my $self = shift;

local $lnum = 0;
open(HBA, $self->{'config_hash'}->{'hba_conf'});
while(<HBA>) {
	s/\r|\n//g;
	s/^\s*#.*$//g;
	if ($_[0] >= 7.3) {
		# New file format
		if (/^\s*(host|hostssl)\s+(\S+)\s+(\S+)\s+(\S+)\/(\S+)\s+(\S+)(\s+(\S+))?/) {
			# Host/cidr format
			push(@{$self->{'rv_array'}}, { 'type' => $1,
				    'index' => scalar(@{$self->{'rv_array'}}),
				    'line' => $lnum,
				    'db' => $2,
				    'user' => $3,
				    'address' => $4,
				    'cidr' => $5,
				    'auth' => $6,
				    'arg' => $8 } );
			}
		elsif (/^\s*(host|hostssl)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)(\s+(\S+))?/) {
			# Host netmask format
			push(@{$self->{'rv_array'}}, { 'type' => $1,
				    'index' => scalar(@{$self->{'rv_array'}}),
				    'line' => $lnum,
				    'db' => $2,
				    'user' => $3,
				    'address' => $4,
				    'netmask' => $5,
				    'auth' => $6,
				    'arg' => $8 } );
			}
		elsif (/^\s*local\s+(\S+)\s+(\S+)\s+(\S+)(\s+(\S+))?/) {
			push(@{$self->{'rv_array'}}, { 'type' => 'local',
				    'index' => scalar(@{$self->{'rv_array'}}),
				    'line' => $lnum,
				    'db' => $1,
				    'user' => $2,
				    'auth' => $3,
				    'arg' => $5 } );
			}
		}
	else {
		# Old file format
		if (/^\s*host\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)(\s+(\S+))?/) {
			push(@{$self->{'rv_array'}}, { 'type' => 'host',
				    'index' => scalar(@{$self->{'rv_array'}}),
				    'line' => $lnum,
				    'db' => $1,
				    'address' => $2,
				    'netmask' => $3,
				    'auth' => $4,
				    'arg' => $6 } );
			}
		elsif (/^\s*local\s+(\S+)\s+(\S+)(\s+(\S+))?/) {
			push(@{$self->{'rv_array'}}, { 'type' => 'local',
				    'index' => scalar(@{$self->{'rv_array'}}),
				    'line' => $lnum,
				    'db' => $1,
				    'auth' => $2,
				    'arg' => $4 } );
			}
		}
	$lnum++;
	}
close(HBA);
return @{$self->{'rv_array'}};
}

# create_hba(&hba, version)
sub create_hba
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'hba_conf'});
push(@$lref, $self->hba_line($_[0], $_[1]));
$self->flush_file_lines();
}

# delete_hba(&hba, version)
sub delete_hba
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'hba_conf'});
splice(@$lref, $_[0]->{'line'}, 1);
$self->flush_file_lines();
}

# modify_hba(&hba, version)
sub modify_hba
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'hba_conf'});
splice(@$lref, $_[0]->{'line'}, 1, $self->hba_line($_[0], $_[1]));
$self->flush_file_lines();
}

# swap_hba(&hba1, &hba2)
sub swap_hba
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'hba_conf'});
local $line0 = $lref->[$_[0]->{'line'}];
local $line1 = $lref->[$_[1]->{'line'}];
$lref->[$_[1]->{'line'}] = $line0;
$lref->[$_[0]->{'line'}] = $line1;
$self->flush_file_lines();
}

# hba_line(&hba, version)
sub hba_line
{
	my $self = shift;

if ($_[0]->{'type'} eq 'host' || $_[0]->{'type'} eq 'hostssl') {
	return join(" ", $_[0]->{'type'}, $_[0]->{'db'},
			 ( $_[1] >= 7.3 ? ( $_[0]->{'user'} ) : ( ) ),
			 ($_[0]->{'cidr'} eq '' ? 
				 ( $_[0]->{'address'},
				   $_[0]->{'netmask'} ) :
				 ( "$_[0]->{'address'}/$_[0]->{'cidr'}" )),
			 $_[0]->{'auth'},
			 $_[0]->{'arg'} ? ( $_[0]->{'arg'} ) : () );
	}
else {
	return join(" ", 'local', $_[0]->{'db'},
			 ( $_[1] >= 7.3 ? ( $_[0]->{'user'} ) : ( ) ),
			 $_[0]->{'auth'},
			 $_[0]->{'arg'} ? ( $_[0]->{'arg'} ) : () );
	}
}

# split_array(value)
sub split_array
{
	my $self = shift;

if ($_[0] =~ /^\{(.*)\}$/) {
	local @a = split(/,/, $1);
	return @a;
	}
else {
	return ( $_[0] );
	}
}

# join_array(values ..)
sub join_array
{
	my $self = shift;

local $alpha;
map { $alpha++ if (!/^-?[0-9\.]+/) } @_;
return $alpha ? '{'.join(',', map { "'$_'" } @_).'}'
	      : '{'.join(',', @_).'}';
}

sub is_blob
{
	my $self = shift;

return $_[0]->{'type'} eq 'text' || $_[0]->{'type'} eq 'bytea';
}

# restart_postgresql()
# HUP postmaster if running, so that hosts file changes take effect
sub restart_postgresql
{
	my $self = shift;

if (open(PID, $self->{'config_hash'}->{'pid_file'})) {
	($self->{'pid'} = <PID>) =~ s/\r|\n//g;
	close(PID);
	$self->kill_logged('HUP', $self->{'pid'}) if ($self->{'pid'});
	}
}

# date_subs(filename)
# Does strftime-style date substitutions on a filename, if enabled
sub date_subs
{
	my $self = shift;

if ($self->{'config_hash'}->{'date_subs'}) {
        eval "use POSIX";
	eval "use posix" if ($@);
        local @tm = localtime(time());
        return strftime($_[0], @tm);
        }
else {
        return $_[0];
        }
}

# execute_before(db, handle, escape, path, db-for-config)
sub execute_before
{
	my $self = shift;

local $cmd = $self->{'config_hash'}->{'backup_before_'.$_[4]};
if ($cmd) {
	$ENV{'BACKUP_FILE'} = $_[3];
	local $h = $_[1];
	local $out;
	local $rv = $self->execute_command($cmd, undef, \$out, \$out);
	if ($h && $out) {
		print $h $_[2] ? "<pre>".$self->html_escape($out)."</pre>" : $out;
		}
	return !$rv;
	}
return 1;
}

# execute_after(db, handle, escape, path, db-for-config)
sub execute_after
{
	my $self = shift;

local $cmd = $self->{'config_hash'}->{'backup_after_'.$_[4]};
if ($cmd) {
	$ENV{'BACKUP_FILE'} = $_[3];
	local $h = $_[1];
	local $out;
	local $rv = $self->execute_command($cmd, undef, \$out, \$out);
	if ($h && $out) {
		print $h $_[2] ? "<pre>".$self->html_escape($out)."</pre>" : $out;
		}
	return !$rv;
	}
return 1;
}

sub quote_table
{
	my $self = shift;

local @tn = split(/\./, $_[0]);
return join(".", map { "\"$_\"" } @tn);
}

sub quotestr
{
	my $self = shift;

return "\"$_[0]\"";
}

# execute_sql_file(database, file, [user, pass])
# Executes some file of SQL statements, and returns the exit status and output
sub execute_sql_file
{
	my $self = shift;

if ($self->is_readonly_mode()) {
	return (0, undef);
	}
local ($db, $file, $user, $pass) = @_;
if (!defined($user)) {
	$user = $self->{'postgres_login'};
	$pass = $self->{'postgres_pass'};
	}
local $cmd = $self->quote_path($self->{'config_hash'}->{'psql'})." -f ".$self->quote_path($file).
	     ($self->supports_pgpass() ? " -U $user" : " -u").
	     ($self->{'config_hash'}->{'host'} ? " -h $self->{'config_hash'}->{'host'}" : "").
	     ($self->{'config_hash'}->{'port'} ? " -h $self->{'config_hash'}->{'port'}" : "").
	     " $db";
$cmd = $self->command_with_login($cmd, $user, $pass);
local $out = $self->backquote_logged("$cmd 2>&1");
return ($out =~ /ERROR/i ? 1 : 0, $out);
}

# split_table(&titles, &checkboxes, &links, &col1, &col2, ...)
# Outputs a table that is split into two parts
sub split_table
{
	my $self = shift;

local $mid = int((@{$_[1]}+1) / 2);
local ($i, $j);
print "<table width=100%><tr>\n";
foreach $s ([0, $mid-1], [$mid, @{$_[1]}-1]) {
	print "<td width=50% valign=top>\n";

	# Header
	local @tds = ( "width=5" );
	if ($s->[0] <= $s->[1]) {
		local @hcols;
		foreach $t (@{$_[0]}) {
			push(@hcols, $t);
			}
		print $self->ui_columns_start(\@hcols, 100, 0, \@tds);
		}

	for($i=$s->[0]; $i<=$s->[1]; $i++) {
		local @cols;
		push(@cols, "<a href='$_[2]->[$i]'>$_[3]->[$i]</a>");
		for($j=4; $j<@_; $j++) {
			push(@cols, $_[$j]->[$i]);
			}
		print $self->ui_checked_columns_row(\@cols, \@tds, "d", $_[1]->[$i]);
		}
	if ($s->[0] <= $s->[1]) {
		print $self->ui_columns_end();
		}
	print "</td>\n";
	}
print "</tr></table>\n";
}

# accepting_connections(db)
# Returns 1 if some database is accepting connections, 0 if not
sub accepting_connections
{
	my $self = shift;

if (!defined($self->{'has_connections'})) {
	$self->{'has_connections'} = 0;
	local @str = $self->table_structure($self->{'config_hash'}->{'basedb'},
				      "pg_catalog.pg_database");
	foreach my $f (@str) {
		$self->{'has_connections'} = 1 if ($f->{'field'} eq 'datallowconn');
		}
	}
if ($self->{'has_connections'}) {
	$self->{'rv'} = $self->execute_sql_safe($self->{'config_hash'}->{'basedb'}, "select datallowconn from pg_database where datname = '$_[0]'");
	if ($self->{'rv'}->{'data'}->[0]->[0] !~ /^(t|1)/i) {
		return 0;
		}
	}
return 1;
}

# start_postgresql()
# Starts the PostgreSQL database server. Returns an error message on failure
# or undef on success.
sub start_postgresql
{
	my $self = shift;

if ($self->{'gconfig_hash'}->{'os_type'} eq 'windows' && $self->foreign_check("init")) {
	# On Windows, always try to sc start the pgsql- service
	$self->foreign_require("init", "init-lib.pl");
	local ($pg) = grep { $_->{'name'} =~ /^pgsql-/ }
			   $main::init_object->list_win32_services();
	if ($pg) {
		return $main::init_object->start_win32_service($pg->{'name'});
		}
	}
local $temp = $self->transname();
local $rv = $self->system_logged("($self->{'config_hash'}->{'start_cmd'}) >$temp 2>&1");
local $out = `cat $temp`; unlink($temp);
unlink($temp);
if ($rv || $out =~ /failed|error/i) {
	return "<pre>$out</pre>";
	}
return undef;
}

# stop_postgresql()
# Stops the PostgreSQL database server. Returns an error message on failure
# or undef on success.
sub stop_postgresql
{
	my $self = shift;

if ($self->{'gconfig_hash'}->{'os_type'} eq 'windows' && $self->foreign_check("init")) {
	# On Windows, always try to sc stop the pgsql- service
	$self->foreign_require("init", "init-lib.pl");
	local ($pg) = grep { $_->{'name'} =~ /^pgsql-/ }
			   $main::init_object->list_win32_services();
	if ($pg) {
		return $main::init_object->stop_win32_service($pg->{'name'});
		}
	}
if ($self->{'config_hash'}->{'stop_cmd'}) {
	local $out = $self->backquote_logged("$self->{'config_hash'}->{'stop_cmd'} 2>&1");
	if ($? || $out =~ /failed|error/i) {
		return "<pre>$?\n$out</pre>";
		}
	}
else {
	local $pid;
	open(PID, $self->{'config_hash'}->{'pid_file'});
	($pid = <PID>) =~ s/\r|\n//g;
	close(PID);
	$pid || return $self->text('stop_epidfile', "<tt>$self->{'config_hash'}->{'pid_file'}</tt>");
	$self->kill_logged('TERM', $pid) ||
		return $self->text('stop_ekill', "<tt>$pid</tt>", "<tt>$!</tt>");
	}
return undef;
}

# setup_postgresql()
# Performs initial postgreSQL configuration. Returns an error message on failure
# or undef on success.
sub setup_postgresql
{
	my $self = shift;

return undef if (!$self->{'config_hash'}->{'setup_cmd'});
local $temp = $self->transname();
local $rv = $self->system_logged("($self->{'config_hash'}->{'setup_cmd'}) >$temp 2>&1");
local $out = `cat $temp`;
unlink($temp);
if ($rv) {
	return "<pre>$out</pre>";
	}
return undef;
}

# list_indexes(db)
# Returns the names of all indexes in some database
sub list_indexes
{
	my $self = shift;

local ($db) = @_;
local (@rv, $r);
local %tables = map { $_, 1 } $self->list_tables($db);
if ($self->supports_schemas($db)) {
	local $t = $self->execute_sql_safe($db, "select schemaname,indexname,tablename from pg_indexes");
	return map { ($_->[0] eq "public" ? "" : $_->[0].".").$_->[1] }
		grep { $tables{($_->[0] eq "public" ? "" : $_->[0].".").$_->[2]} }
		   @{$t->{'data'}};
	}
else {
	local $t = $self->execute_sql_safe($db, "select indexname,tablename from pg_indexes");
	return map { $_->[0] } grep { $tables{$t->[1]} } @{$t->{'data'}};
	}
}

# index_structure(db, indexname)
# Returns information on an index
sub index_structure
{
	my $self = shift;

local ($db, $index) = @_;
local $info = { 'name' => $index };
if ($self->supports_schemas($db)) {
	local ($sn, $in);
	if ($index =~ /^(\S+)\.(\S+)$/) {
		($sn, $in) = ($1, $2);
		}
	else {
		($sn, $in) = ("public", $index);
		}
	local $t = $self->execute_sql_safe($db, "select schemaname,tablename,indexdef from pg_indexes where indexname = '$in' and schemaname = '$sn'");
	local $r = $t->{'data'}->[0];
	if ($r->[0] eq "public") {
		$info->{'table'} = $r->[1];
		}
	else {
		$info->{'table'} = $r->[0].".".$r->[1];
		}
	$info->{'create'} = $r->[2];
	}
else {
	local $t = $self->execute_sql_safe($db, "select tablename,indexdef from pg_indexes where indexname = '$index'");
	local $r = $t->{'data'}->[0];
	$info->{'table'} = $r->[0];
	$info->{'create'} = $r->[1];
	}

# Parse create expression
if ($info->{'create'} =~ /^create\s+unique/i) {
	$info->{'type'} = 'unique';
	}
if ($info->{'create'} =~ /using\s+(\S+)\s/) {
	$info->{'using'} = lc($1);
	}
if ($info->{'create'} =~ /\((.*)\)/) {
	$info->{'cols'} = [ split(/\s*,\s*/, $1) ];
	}

return $info;
}

sub supports_indexes
{
	my $self = shift;

return $self->get_postgresql_version() >= 7.3;
}

# list_views(db)
# Returns the names of all views in some database
sub list_views
{
	my $self = shift;

local ($db) = @_;
local (@rv, $r);
if ($self->supports_schemas($db)) {
	local $t = $self->execute_sql_safe($db, "select schemaname,viewname from pg_views where schemaname != 'pg_catalog' and schemaname != 'information_schema'");
	return map { ($_->[0] eq "public" ? "" : $_->[0].".").$_->[1] }
		   @{$t->{'data'}};
	}
else {
	local $t = $self->execute_sql_safe($db, "select viewname from pg_indexes");
	return map { $_->[0] } @{$t->{'data'}};
	}
}

# view_structure(db, viewname)
# Returns information about a view
sub view_structure
{
	my $self = shift;

local ($db, $view) = @_;
local $info = { 'name' => $view };
if ($self->supports_schemas($db)) {
	local ($sn, $in);
	if ($view =~ /^(\S+)\.(\S+)$/) {
		($sn, $in) = ($1, $2);
		}
	else {
		($sn, $in) = ("public", $view);
		}
	local $t = $self->execute_sql_safe($db, "select schemaname,viewname,definition from pg_views where viewname = '$in' and schemaname = '$sn'");
	local $r = $t->{'data'}->[0];
	$info->{'query'} = $r->[2];
	}
else {
	local $t = $self->execute_sql_safe($db, "select viewname,definition from pg_views where viewname = '$self->{'index'}'");
	local $r = $t->{'data'}->[0];
	$info->{'query'} = $r->[1];
	}

$info->{'query'} =~ s/;$//;

return $info;
}

sub supports_views
{
	my $self = shift;

return $self->get_postgresql_version() >= 7.3;
}

# list_sequences(db)
# Returns the names of all sequences in some database
sub list_sequences
{
	my $self = shift;

local ($db) = @_;
local (@rv, $r);
if ($self->supports_schemas($db)) {
	local $t = $self->execute_sql_safe($db, "select schemaname,relname from pg_statio_user_sequences");
	return map { ($_->[0] eq "public" ? "" : $_->[0].".").$_->[1] }
		   @{$t->{'data'}};
	}
else {
	local $t = $self->execute_sql_safe($db, "select relname from pg_statio_user_sequences");
	return map { $_->[0] } @{$t->{'data'}};
	}
}

# sequence_structure(db, name)
# Returns details of a sequence
sub sequence_structure
{
	my $self = shift;

local ($db, $seq) = @_;
local $info = { 'name' => $seq };

local $t = $self->execute_sql_safe($db, "select * from ".$self->quote_table($seq));
local $r = $t->{'data'}->[0];
local $i = 0;
foreach my $c (@{$t->{'titles'}}) {
	$info->{$c} = $r->[$i++];
	}

return $info;
}

sub supports_sequences
{
	my $self = shift;

return $self->get_postgresql_version() >= 7.4 ? 1 :
       $self->get_postgresql_version() >= 7.3 ? 2 : 0;
}

# Returns 1 if the postgresql server being managed is on this system
sub is_postgresql_local
{
	my $self = shift;

return $self->{'config_hash'}->{'host'} eq '' || $self->{'config_hash'}->{'host'} eq 'localhost' ||
       $self->{'config_hash'}->{'host'} eq $self->get_system_hostname() ||
       $self->to_ipaddress($self->{'config_hash'}->{'host'}) eq $self->to_ipaddress($self->get_system_hostname());
}

# backup_database(database, dest-path, format, [&only-tables])
# Executes the pg_dump command to backup the specified database to the
# given destination path. Returns undef on success, or an error message
# on failure.
sub backup_database
{
	my $self = shift;

local ($db, $path, $format, $tables) = @_;
local $tablesarg = join(" ", map { " -t ".quotemeta($_) } @$tables);
local $cmd = $self->quote_path($self->{'config_hash'}->{'dump_cmd'}).
	     (!$self->{'postgres_login'} ? "" :
	      $self->supports_pgpass() ? " -U $self->{'postgres_login'}" : " -u").
	     ($self->{'config_hash'}->{'host'} ? " -h $self->{'config_hash'}->{'host'}" : "").
	     ($format eq 'p' ? "" : " -b").
	     $tablesarg.
	     " -F$format -f ".$self->quote_path($path)." $db";
if ($self->{'postgres_sameunix'} && defined(getpwnam($self->{'postgres_login'}))) {
	$cmd = $self->command_as_user($self->{'postgres_login'}, 0, $cmd);
	}
$cmd = $self->command_with_login($cmd);
local $out = $self->backquote_logged("$cmd 2>&1");
if ($? || $out =~ /could not|error|failed/i) {
	return $out;
	}
return undef;
}

# restore_database(database, source-path, only-data, clear-db)
# Restores the contents of a PostgreSQL backup into the specified database.
# Returns undef on success, or an error message on failure.
sub restore_database
{
	my $self = shift;

local ($db, $path, $only, $clean) = @_;
local $cmd = $self->quote_path($self->{'config_hash'}->{'rstr_cmd'}).
	     (!$self->{'postgres_login'} ? "" :
	      $self->supports_pgpass() ? " -U $self->{'postgres_login'}" : " -u").
	     ($self->{'config_hash'}->{'host'} ? " -h $self->{'config_hash'}->{'host'}" : "").
	     ($only ? " -a" : "").
	     ($clean ? " -c" : "").
	     " -d $db ".$self->quote_path($path);
if ($self->{'postgres_sameunix'} && defined(getpwnam($self->{'postgres_login'}))) {
	$cmd = $self->command_as_user($self->{'postgres_login'}, 0, $cmd);
	}
$cmd = $self->command_with_login($cmd);
local $out = $self->backquote_logged("$cmd 2>&1");
if ($? || $out =~ /could not|error|failed/i) {
	return $out;
	}
return undef;
}

# PostgreSQL versions below 7.3 don't support .pgpass, and version 8.0.*
# don't allow it to be located via $HOME or $PGPASSFILE.
sub supports_pgpass
{
	my $self = shift;

local $ver = $self->get_postgresql_version(1);
return $ver >= 7.3 && $ver < 8.0 ||
       $ver >= 8.1;
}

# command_with_login(command, [user, pass])
# Given a command that talks to postgresql (like psql or pg_dump), sets up
# the environment so that it can login to the database. Returns a modified
# command to execute.
sub command_with_login
{
	my $self = shift;

local ($cmd, $user, $pass) = @_;
if (!defined($user)) {
	$user = $self->{'postgres_login'};
	$pass = $self->{'postgres_pass'};
	}
local $loginfile;
if ($self->supports_pgpass()) {
	# Can use .pgpass file
	local $pgpass;
	if ($self->{'gconfig_hash'}->{'os_type'} eq 'windows') {
		# On Windows, the file is under ~/application data
		local $appdata = "$ENV{'HOME'}/application data";
		$self->make_dir($appdata, 0755);
		local $postgresql = "$appdata/postgresql";
		$self->make_dir($postgresql, 0755);
		$pgpass = "$postgresql/pgpass.conf";
		}
	else {
		my @user = $self->{'postgres_sameunix'} ? getpwnam($user) : getpwnam('root');
		$pgpass = "$user[7]/.pgpass";
		}
	open(PGPASS, ">$pgpass");
	print PGPASS "*:*:*:$user:$pass\n";
	close(PGPASS);
	$self->set_ownership_permissions(
		$self->{'postgres_sameunix'} ? $user : undef,
		undef, 0600, $pgpass);
	}
else {
	# Need to put login and password in temp file
	$loginfile = $self->transname();
	open(TEMP, ">$loginfile");
	print TEMP "$user\n$pass\n";
	close(TEMP);
	$cmd .= " <$loginfile";
	}
return $cmd;
}


1;

