#!/usr/bin/perl
package postgresql::ViewLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'postgresql::ViewLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions used by view_table.cgi and siblings

1;

1;
		return $self;
}

# get_search_args(&in)
# Returns the search SQL, search URL arguments and search hidden fields
sub get_search_args
{
	my $self = shift;

local %in = %{$_[0]};
local ($search, $searchhids, $searchargs, @adv);
if ($in{'field'}) {
	# A simple search
	$search = "where ".$self->quotestr($in{'field'})." ".
		   $self->make_like($in{'match'}, $in{'for'});
	$searchargs = "&field=".$self->urlize($in{'field'}).
		      "&for=".$self->urlize($in{'for'}).
		      "&match=".$self->urlize($in{'match'});
	$searchhids = $self->ui_hidden("field", $in{'field'})."\n".
		      $self->ui_hidden("for", $in{'for'})."\n".
		      $self->ui_hidden("match", $in{'match'})."\n";
	}
elsif ($in{'advanced'}) {
	# An advanced search
	for(my $i=0; defined($in{"field_$i"}); $i++) {
		if ($in{"field_$i"}) {
			push(@adv, $self->quotestr($in{"field_$i"})." ".
				   $self->make_like($in{"match_$i"}, $in{"for_$i"}));
			$searchargs .= "&field_$i=".$self->urlize($in{"field_$i"}).
				       "&for_$i=".$self->urlize($in{"for_$i"}).
				       "&match_$i=".$self->urlize($in{"match_$i"});
			$searchhids .=
				$self->ui_hidden("field_$i", $in{"field_$i"})."\n".
				$self->ui_hidden("for_$i", $in{"for_$i"})."\n".
				$self->ui_hidden("match_$i", $in{"match_$i"})."\n";
			}
		}
	if (@adv) {
		$search = "where (".join($in{'and'} ? " and " : " or ",
					@adv).")";
		$searchhids .= $self->ui_hidden("and", $in{'and'})."\n".
			       $self->ui_hidden("advanced", 1)."\n";
		$searchargs .= "&and=".$in{'and'}.
			       "&advanced=1";
		}
	}
return ($search, $searchhids, $searchargs, scalar(@adv));
}

# get_search_limit(&in)
# Build limiting expression
sub get_search_limit
{
	my $self = shift;

local %in = %{$_[0]};
$in{'start'} ||= 0;
if ($self->{'module_name'} eq "mysql") {
	return "limit $in{'start'},$self->{'config_hash'}->{'perpage'}";
	}
else {
	return "limit $self->{'config_hash'}->{'perpage'} offset $in{'start'}";
	}
}

# get_search_limit(&in)
# Returns an SQL expression for sorting, hidden fields for the current sort order, and
# URL args for the current order
sub get_search_sort
{
	my $self = shift;

local %in = %{$_[0]};
if ($in{'sortfield'}) {
	local ($sort, $sorthids, $sortargs);
	$sort = "order by ".$self->quotestr($in{'sortfield'})." ".($in{'sortdir'} ? "asc" : "desc");
	$sorthids = $self->ui_hidden("sortfield", $in{'sortfield'})."\n".
		    $self->ui_hidden("sortdir", $in{'sortdir'})."\n";
	$sortargs = "&sortfield=".$self->urlize($in{'sortfield'}).
		    "&sortdir=$in{'sortdir'}";
	return ($sort, $sorthids, $sortargs);
	}
else {
	return ( undef, undef, undef );
	}
}

# make_like(mode, for)
sub make_like
{
	my $self = shift;

local ($match, $for) = @_;
local $qu = $self->{'module_name'} eq "mysql" ? '"' : "'";
return $match == 0 ? "like $qu%$for%$qu" :
       $match == 1 ? "like $qu$for$qu" :
       $match == 2 ? "not like $qu%$for%$qu" :
       $match == 3 ? "not like $qu$for$qu" : " = \"\"";
}


1;

