#!/usr/bin/perl

&MAIN();

 sub MAIN {
# save_host.cgi
# Create, modify or delete an allowed host record

&ModuleLoader::preload_module('postgresql_class', "postgresql::PostgresqlLib");
$self = new postgresql::PostgresqlLib:: 'postgresql_class',%$self;

$self->ReadParse();
$self->lock_file($self->{'config_hash'}->{'hba_conf'});
$self->{'v'} = $self->get_postgresql_version();
@{$self->{'all_array'}} = $self->get_hba_config($self->{'v'});
$self->{'host'} = $self->{'all_array'}->[$self->{'in_hash'}->{'idx'}] if (!$self->{'in_hash'}->{'new'});
$self->error_setup($self->{'text_hash'}->{'host_err'});

if ($self->{'in_hash'}->{'delete'}) {
	# delete one host
	$self->delete_hba($self->{'host'}, $self->{'v'});
	}
else {
	# validate and parse inputs
	if ($self->{'in_hash'}->{'addr_mode'} == 0) {
		$self->{'host'}->{'address'} = '0.0.0.0' if (!$self->{'host'}->{'address'});
		$self->{'object'} = $self->{'host'}->{'netmask'} = '0.0.0.0';
		$self->{'host'}->{'type'} = $self->{'in_hash'}->{'ssl'} ? 'hostssl' : 'host';
		}
	elsif ($self->{'in_hash'}->{'addr_mode'} == 1) {
		$self->check_ipaddress($self->{'in_hash'}->{'host'}) ||
		  $self->check_ip6address($self->{'in_hash'}->{'host'}) ||
			$self->error($self->{'text_hash'}->{'host_ehost'});
		$self->{'object'} = $self->{'host'}->{'address'} = $self->{'in_hash'}->{'host'};
		$self->{'host'}->{'netmask'} = '255.255.255.255';
		$self->{'host'}->{'type'} = $self->{'in_hash'}->{'ssl'} ? 'hostssl' : 'host';
		}
	elsif ($self->{'in_hash'}->{'addr_mode'} == 2) {
		# Parse address / netmask
		$self->check_ipaddress($self->{'in_hash'}->{'network'}) ||
		   $self->check_ip6address($self->{'in_hash'}->{'network'}) ||
			$self->error($self->{'text_hash'}->{'host_enetwork'});
		$self->check_ipaddress($self->{'in_hash'}->{'netmask'}) ||
			$self->error($self->{'text_hash'}->{'host_enetmask'});
		$self->{'host'}->{'address'} = $self->{'in_hash'}->{'network'};
		$self->{'host'}->{'netmask'} = $self->{'in_hash'}->{'netmask'};
		delete($self->{'host'}->{'cidr'});
		$self->{'host'}->{'type'} = $self->{'in_hash'}->{'ssl'} ? 'hostssl' : 'host';
		$self->{'object'} = "$self->{'in_hash'}->{'network'}/$self->{'in_hash'}->{'netmask'}";
		}
	elsif ($self->{'in_hash'}->{'addr_mode'} == 4) {
		# Parse address / CIDR
		$self->check_ipaddress($self->{'in_hash'}->{'network2'}) ||
		    $self->check_ip6address($self->{'in_hash'}->{'network2'}) ||
			$self->error($self->{'text_hash'}->{'host_enetwork'});
		$self->{'in_hash'}->{'cidr'} =~ /^\d+$/ ||
			$self->error($self->{'text_hash'}->{'host_ecidr'});
		$self->{'host'}->{'address'} = $self->{'in_hash'}->{'network2'};
		$self->{'host'}->{'cidr'} = $self->{'in_hash'}->{'cidr'};
		delete($self->{'host'}->{'netmask'});
		$self->{'host'}->{'type'} = $self->{'in_hash'}->{'ssl'} ? 'hostssl' : 'host';
		$self->{'object'} = "$self->{'in_hash'}->{'network2'}/$self->{'in_hash'}->{'cidr'}";
		}
	else {
		$self->{'object'} = $self->{'host'}->{'type'} = 'local';
		}
	if ($self->{'in_hash'}->{'db'}) {
		$self->{'host'}->{'db'} = $self->{'in_hash'}->{'db'};
		}
	else {
		$self->{'in_hash'}->{'dbother'} || $self->error($self->{'text_hash'}->{'host_edb'});
		$self->{'host'}->{'db'} = join(",", split(/\s+/, $self->{'in_hash'}->{'dbother'}));
		}
	if ($self->{'v'} >= 7.3) {
		$self->{'in_hash'}->{'user_def'} || $self->{'in_hash'}->{'user'} || $self->error($self->{'text_hash'}->{'host_euser'});
		$self->{'host'}->{'user'} = $self->{'in_hash'}->{'user_def'} ? 'all' :
					join(",", split(/\s+/, $self->{'in_hash'}->{'user'}));
		}
	$self->{'host'}->{'auth'} = $self->{'in_hash'}->{'auth'};
	if ($self->{'in_hash'}->{'auth'} eq 'password' && $self->{'in_hash'}->{'passwordarg'}) {
		$self->{'in_hash'}->{'password'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'host_epassword'});
		$self->{'host'}->{'arg'} = $self->{'in_hash'}->{'password'};
		}
	elsif ($self->{'in_hash'}->{'auth'} eq 'ident' && $self->{'in_hash'}->{'identarg'}) {
		$self->{'in_hash'}->{'ident'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'host_eident'});
		$self->{'host'}->{'arg'} = $self->{'in_hash'}->{'ident'};
		}
	elsif ($self->{'in_hash'}->{'auth'} eq 'pam' && $self->{'in_hash'}->{'pamarg'}) {
		$self->{'in_hash'}->{'pam'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'host_epam'});
		$self->{'host'}->{'arg'} = $self->{'in_hash'}->{'pam'};
		}
	else {
		$self->{'host'}->{'arg'} = undef;
		}

	if ($self->{'in_hash'}->{'new'}) {
		$self->create_hba($self->{'host'}, $self->{'v'});
		}
	else {
		$self->modify_hba($self->{'host'}, $self->{'v'});
		}
	}
$self->unlock_file($self->{'config_hash'}->{'hba_conf'});
$self->restart_postgresql();
$self->webmin_log($self->{'in_hash'}->{'new'} ? 'create' : $self->{'in_hash'}->{'delete'} ? 'delete' : 'modify',
	    'hba', $self->{'host'}->{'type'} eq 'local' ? 'local' :
		   $self->{'host'}->{'netmask'} eq '0.0.0.0' ? 'all' :
		   $self->{'host'}->{'netmask'} eq '255.255.255.255' ? $self->{'host'}->{'address'}:
		   "$self->{'host'}->{'address'}/$self->{'host'}->{'netmask'}", $self->{'host'});
$self->redirect("list_hosts.cgi");

1;
		
}


1;

