#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Create, modify or delete a sequence

&ModuleLoader::preload_module('postgresql_class', "postgresql::PostgresqlLib");
$self = new postgresql::PostgresqlLib:: 'postgresql_class',%$self;

$self->ReadParse();
$self->error_setup($self->{'text_hash'}->{'seq_err'});
$self->can_edit_db($self->{'in_hash'}->{'db'}) || $self->error($self->{'text_hash'}->{'dbase_ecannot'});
$self->{'access_hash'}->{'edonly'} && $self->error($self->{'text_hash'}->{'dbase_ecannot'});
$self->{'access_hash'}->{'seqs'} || $self->error($self->{'text_hash'}->{'seq_ecannot'});

if ($self->{'in_hash'}->{'delete'}) {
	# Just drop the sequence
	$self->{'sql'} = "drop sequence ".$self->quotestr($self->{'in_hash'}->{'old'});
	$self->execute_sql_logged($self->{'in_hash'}->{'db'}, $self->{'sql'});
	$self->webmin_log("delete", "seq", $self->{'in_hash'}->{'old'}, \%{$self->{'in_hash'}});
	}
else {
	# Validate inputs
	$self->{'in_hash'}->{'old'} || $self->{'in_hash'}->{'name'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'seq_ename'});
	if (!$self->{'in_hash'}->{'old'}) {
		@{$self->{'seqs_array'}} = $self->list_sequences($self->{'in_hash'}->{'db'});
		$self->indexof($self->{'in_hash'}->{'name'}, @{$self->{'seqs_array'}}) >= 0 &&
			$self->error($self->{'text_hash'}->{'seq_eclash'});
		}
	$self->{'in_hash'}->{'min_def'} || $self->{'in_hash'}->{'min'} =~ /^\d+$/ ||
		$self->error($self->{'text_hash'}->{'seq_emin'});
	$self->{'in_hash'}->{'max_def'} || $self->{'in_hash'}->{'max'} =~ /^\d+$/ ||
		$self->error($self->{'text_hash'}->{'seq_emax'});
	$self->{'in_hash'}->{'inc'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'seq_einc'});
	$self->{'in_hash'}->{'cache_def'} || $self->{'in_hash'}->{'cache'} =~ /^\d+$/ ||
		$self->error($self->{'text_hash'}->{'seq_ecache'});

	if ($self->supports_sequences() == 2 && $self->{'in_hash'}->{'old'}) {
		# Need to drop and re-create
		if ($self->indexof($self->{'in_hash'}->{'old'}, $self->list_sequences($self->{'in_hash'}->{'db'})) >= 0) {
			$self->{'sql'} = "drop sequence ".$self->quotestr($self->{'in_hash'}->{'old'});
			$self->execute_sql_logged($self->{'in_hash'}->{'db'}, $self->{'sql'});
			}
		$self->{'sql'} = "create sequence ".$self->quote_table($self->{'in_hash'}->{'old'}).
		       " increment ".$self->{'in_hash'}->{'inc'}.
		       ($self->{'in_hash'}->{'min_def'} ? "" : " minvalue ".$self->{'in_hash'}->{'min'}).
		       ($self->{'in_hash'}->{'max_def'} ? "" : " maxvalue ".$self->{'in_hash'}->{'max'}).
		       " start ".$self->{'in_hash'}->{'last'}.
		       ($self->{'in_hash'}->{'cache_def'} ? "" : " cache ".$self->{'in_hash'}->{'cache'}).
		       ($self->{'in_hash'}->{'cycle'} ? " cycle" : "");
		}
	elsif ($self->{'in_hash'}->{'old'} && !$self->{'recreate'}) {
		# Alter the current sequence
		$self->{'sql'} = "alter sequence ".$self->quote_table($self->{'in_hash'}->{'old'}).
		       " increment ".$self->{'in_hash'}->{'inc'}.
		       ($self->{'in_hash'}->{'min_def'} ? " no minvalue"
				       : " minvalue ".$self->{'in_hash'}->{'min'}).
		       ($self->{'in_hash'}->{'max_def'} ? " no maxvalue"
				       : " maxvalue ".$self->{'in_hash'}->{'max'}).
		       ($self->{'in_hash'}->{'last_def'} ? "" : " restart ".$self->{'in_hash'}->{'last'}).
		       " cache ".$self->{'in_hash'}->{'cache'}.
		       ($self->{'in_hash'}->{'cycle'} ? " cycle" : " no cycle");
		}
	else {
		# Create a new one
		$self->{'sql'} = "create sequence ".$self->quote_table($self->{'in_hash'}->{'name'}).
		       " increment ".$self->{'in_hash'}->{'inc'}.
		       ($self->{'in_hash'}->{'min_def'} ? "" : " minvalue ".$self->{'in_hash'}->{'min'}).
		       ($self->{'in_hash'}->{'max_def'} ? "" : " maxvalue ".$self->{'in_hash'}->{'max'}).
		       " start ".$self->{'in_hash'}->{'last'}.
		       ($self->{'in_hash'}->{'cache_def'} ? "" : " cache ".$self->{'in_hash'}->{'cache'}).
		       ($self->{'in_hash'}->{'cycle'} ? " cycle" : "");
		}
	$self->execute_sql_logged($self->{'in_hash'}->{'db'}, $self->{'sql'});

	if ($self->{'in_hash'}->{'old'}) {
		$self->webmin_log("modify", "view", $self->{'in_hash'}->{'old'}, \%{$self->{'in_hash'}});
		}
	else {
		$self->webmin_log("create", "view", $self->{'in_hash'}->{'name'}, \%{$self->{'in_hash'}});
		}
	}
$self->redirect("edit_dbase.cgi?db=$self->{'in_hash'}->{'db'}");

1;
		
}


1;

