#!/usr/bin/perl
package pptp_client::PptpclientLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'pptp_client::PptpclientLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# pptp-client-lib.pl
# XXX help page

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
&ModuleLoader::preload_module(ref($self), "pptp_client::SecretsLib");
$self = new pptp_client::SecretsLib:: ref($self), %$self; 

@{$self->{'old_mppe_array'}} = ( 'mppe-40', 'mppe-128', 'mppe-stateless' );
@{$self->{'new_mppe_array'}} = ( [ 'mppe', 0 ], [ 'mppe-40', 1 ], [ 'mppe-128', 1 ],
	      [ 'mppe-stateful', 0 ] );

1;

1;
		return $self;
}

# list_tunnels()
# Returns a list of the details of configured tunnels, in the format used
# by the pptp-command script
sub list_tunnels
{
	my $self = shift;

local ($f, @rv);
opendir(DIR, $self->{'config_hash'}->{'peers_dir'});
while($f = readdir(DIR)) {
	next if ($f =~ /^\./ || $f eq "__default");
	local @opts = $self->parse_ppp_options("$self->{'config_hash'}->{'peers_dir'}/$f");
	local ($pptp) = grep { $_->{'comment'} =~ /^PPTP/ } @opts;
	if ($pptp) {
		# Is a tunnel config .. add it
		push(@rv, { 'name' => $f,
			    'file' => "$self->{'config_hash'}->{'peers_dir'}/$f",
			    'opts' => \@opts });
		}
	}
closedir(DIR);
return @rv;
}

# parse_ppp_options(file)
sub parse_ppp_options
{
	my $self = shift;

local @rv;
local $lnum = 0;
open(OPTS, $_[0]);
while(<OPTS>) {
	s/\r|\n//g;
	if (/^#\s*(.*)/) {
		# A comment, used to store meta-information
		push(@rv, { 'comment' => $1,
			    'file' => $_[0],
			    'line' => $lnum,
			    'index' => scalar(@rv) });
		}
	elsif (/^([0-9\.]+):([0-9\.]+)/) {
		# A local/remote IP specification
		push(@rv, { 'local' => $1,
			    'remote' => $2,
			    'file' => $_[0],
			    'line' => $lnum,
			    'index' => scalar(@rv) });
		}
	elsif (/^([^# ]*)\s*([^#]*)/) {
		# A PPP options directive
		push(@rv, { 'name' => $1,
			    'value' => $2,
			    'file' => $_[0],
			    'line' => $lnum,
			    'index' => scalar(@rv) });
		}
	$lnum++;
	}
close(OPTS);
return @rv;
}

# find(name, &config)
sub find
{
	my $self = shift;

local @rv = grep { lc($_->{'name'}) eq lc($_[0]) } @{$_[1]};
return wantarray ? @rv : $rv[0];
}

# save_ppp_option(&config, file, &old|name, &new)
sub save_ppp_option
{
	my $self = shift;

local $ol = ref($_[2]) || !defined($_[2]) ? $_[2] : $self->find($_[2], $_[0]);
local $nw = $_[3];
local $lref = $self->read_file_lines($_[1]);
local $line;
if ($nw) {
	if ($nw->{'local'}) {
		$line = $nw->{'local'}.":".$nw->{'remote'};
		}
	elsif ($nw->{'comment'}) {
		$line = "# ".$nw->{'comment'};
		}
	else {
		$line = $nw->{'name'};
		$line .= " $nw->{'value'}" if ($nw->{'value'} ne "");
		}
	}
if ($ol && $nw) {
	$lref->[$ol->{'line'}] = $line;
	}
elsif ($ol) {
	splice(@$lref, $ol->{'line'}, 1);
	local $c;
	foreach $c (@{$_[0]}) {
		$c->{'line'}-- if ($c->{'line'} > $ol->{'line'});
		}
	}
elsif ($nw) {
	push(@$lref, $line);
	}
}

# list_connected()
# Returns a list of the names of tunnels that appear to be active. May include
# other ppp calls as well
sub list_connected
{
	my $self = shift;

$self->foreign_require("proc", "proc-lib.pl");
local @rv;
foreach $p ($main::proc_object->list_processes()) {
	if ($p->{'args'} =~ /pppd\s.*call\s+(.*\S+)/) {
		push(@rv, [ $1, $p->{'pid'} ]);
		if ($1 eq $self->{'config_hash'}->{'tunnel'}) {
			$rv[$#rv]->[2] = $self->{'config_hash'}->{'iface'};
			}
		}
	}
return @rv;
}

# parse_comments(&tunnel)
sub parse_comments
{
	my $self = shift;

foreach $c (@{$_[0]->{'opts'}}) {
	if ($c->{'comment'} =~ /Server IP: (\S+)/) {
		$_[0]->{'server'} = $1;
		$_[0]->{'server_c'} = $c;
		}
	elsif ($c->{'comment'} =~ /Route: (.*)/) {
		push(@{$_[0]->{'routes'}}, $1);
		push(@{$_[0]->{'routes_c'}}, $c);
		}
	}
}

# mppe_options_form(&opts)
# Show a form for editing MPPE-related PPP options
sub mppe_options_form
{
	my $self = shift;

# Get the PPPd version. Only those above 2.4.2 have built-in MPPE support
local $out = `pppd --help 2>&1`;
local $mppe_mode = $self->mppe_support();
print "<input type=hidden name=mppe_mode value='$mppe_mode'>\n";

local $opts = $_[0];
if ($mppe_mode) {
	# Show new MPPE options
	local $o;
	foreach $o (@{$self->{'new_mppe_array'}}) {
		local $o0 = $self->find("require-".$o->[0], $opts);
		local $o1 = $self->find("no".$o->[0], $opts);
		local $mode = $o0 ? 2 : $o1 ? 0 : 1;
		print "<tr> <td><b>",$self->{'text_hash'}->{'mppe_'.$o->[0]},"</b></td>\n";
		print "<td colspan=3>\n";
		printf "<input type=radio name=%s value=2 %s> %s\n",
			$o->[0], $mode == 2 ? "checked" : "", $self->{'text_hash'}->{'mppe_m2'};
		printf "<input type=radio name=%s value=1 %s> %s (%s)\n",
			$o->[0], $mode == 1 ? "checked" : "", $self->{'text_hash'}->{'default'},
			$o->[1] ? $self->{'text_hash'}->{'mppe_d1'} : $self->{'text_hash'}->{'mppe_d0'};
		printf "<input type=radio name=%s value=0 %s> %s\n",
			$o->[0], $mode == 0 ? "checked" : "", $self->{'text_hash'}->{'mppe_m0'};
		print "</td> </tr>\n";
		}
	local @anyold = grep { $self->find($_, $opts) } @{$self->{'old_mppe_array'}};
	if (@anyold) {
		print "<tr> <td colspan=4 align=center>",$self->text('mppe_old',
			"<tt>".join(" ", @anyold)."</tt>"),"</td> </tr>\n";
		}
	}
else {
	# Show old MPPE options
	$self->{'i'} = 0;
	foreach $o (@{$self->{'old_mppe_array'}}) {
		print "<tr>\n" if ($self->{'i'}%2 == 0);
		local $v = $self->find($o, $opts);
		print "<td><b>",$self->{'text_hash'}->{'mppe_'.$o},"</b></td> <td>\n";
		printf "<input type=radio name=$o value=1 %s> %s\n",
			$v ? "checked" : "", $self->{'text_hash'}->{'yes'};
		printf "<input type=radio name=$o value=0 %s> %s</td>\n",
			$v ? "" : "checked", $self->{'text_hash'}->{'no'};
		print "</tr>\n" if ($self->{'i'}%2 == 1);
		$self->{'i'}++;
		}
	local @anynew = grep { $self->find($_, $opts) }
		( map { 'require-'.$_->[0] } @{$self->{'new_mppe_array'}} ),
		( map { 'no'.$_->[0] } @{$self->{'new_mppe_array'}} );
	if (@anynew) {
		print "<tr> <td colspan=4 align=center>",$self->text('mppe_new',
			"<tt>".join(" ", @anynew)."</tt>"),"</td> </tr>\n";
		}
	print "</tr>\n";
	}
}

# parse_mppe_options(&config, file)
sub parse_mppe_options
{
	my $self = shift;

local $o;
if ($self->{'in_hash'}->{'mppe_mode'}) {
	# Parse new-style options
	foreach $o (map { $_->[0] } @{$self->{'new_mppe_array'}}) {
		if ($self->{'in_hash'}->{$o} == 2) {
			$self->save_ppp_option($_[0], $_[1], "require-$o",
					 { 'name' => "require-$o" });
			$self->save_ppp_option($_[0], $_[1], "no$o", undef);
			}
		elsif ($self->{'in_hash'}->{$o} == 1) {
			$self->save_ppp_option($_[0], $_[1], "require-$o", undef);
			$self->save_ppp_option($_[0], $_[1], "no$o", undef);
			}
		else {
			$self->save_ppp_option($_[0], $_[1], "require-$o", undef);
			$self->save_ppp_option($_[0], $_[1], "no$o",
					 { 'name' => "no$o" });
			}
		}
	}
else {
	# Parse old-style options
	foreach $o (@{$self->{'old_mppe_array'}}) {
		$self->save_ppp_option($_[0], $_[1], $o,
				 $self->{'in_hash'}->{$o} ? { 'name' => $o } : undef);
		}
	}
}

# mppe_support()
# Returns 1 if the PPP daemon supports new-style MPPE options (version 2.4.2+,
# 0 if might only support the old options)
sub mppe_support
{
	my $self = shift;

local $out = `pppd --help 2>&1`;
local $vers;
if ($out =~ /version\s+(\S+)/i) {
	$vers = $1;
	}
if ($vers =~ /^(\d+)/ && $1 > 2 ||
    $vers =~ /^(\d+)\.(\d+)/ && $1 == 2 && $2 > 4 ||
    $vers =~ /^(\d+)\.(\d+)\.(\d+)/ && $1 == 2 && $2 == 4 && $3 >= 2) {
	return 1;
	}
return 0;
}

# get_pppd_version(&out)
sub get_pppd_version
{
	my $self = shift;

local $out = `pppd --help 2>&1`;
${$_[0]} = $out;
return $out =~ /version\s+(\S+)/i ? $1 : undef;
}

# connect_tunnel(&tunnel)
# Attempts to open some tunnel. Returns either :
# 1, iface-name, iface-address, iface-ptp
# 0, error-message
sub connect_tunnel
{
	my $self = shift;

local $tunnel = $_[0];
$self->foreign_require("net", "net-lib.pl");

# Run the PPTP command, and wait for a new pppN interface to come up
local %sifaces = map { $_->{'fullname'}, $_->{'address'} } $self->get_ppp_ifaces();
local $start = time();
local $temp = $self->tempname();
$self->system_logged("modprobe ip_gre >/dev/null 2>&1");
$self->system_logged("$self->{'config_hash'}->{'pptp'} ".quotemeta($tunnel->{'server'})." call ".
	       quotemeta($tunnel->{'name'})." >$temp 2>&1 </dev/null &");
local $newiface;
LOOP: while(time() - $start < $self->{'config_hash'}->{'timeout'}) {
	sleep(1);
	local @nifaces = $self->get_ppp_ifaces();
	local $i;
	foreach $i (@nifaces) {
		if (!$sifaces{$i->{'fullname'}}) {
			$newiface = $i;
			last LOOP;
			}
		}
	}
local $tempout = `cat $temp`;
unlink($temp);

# Find out if we were connected, or if it failed
if ($newiface) {
	# It worked! Add the routes
	local (@rout, @rcmd);
	if (@{$tunnel->{'routes'}}) {
		local @routes = $main::net_object->list_routes();
		local ($defroute) = grep { $_->{'dest'} eq '0.0.0.0' } @routes;
		local $oldgw = $defroute->{'gateway'} if ($defroute);
		foreach $r (@{$tunnel->{'routes'}}) {
			$self->{'cmd'} = "route $r";
			$self->{'cmd'} =~ s/TUNNEL_DEV/$newiface->{'fullname'}/g;
			$self->{'cmd'} =~ s/DEF_GW/$oldgw/g;
			$self->{'cmd'} =~ s/GW/$newiface->{'ptp'}/g;
			push(@rcmd, $self->{'cmd'});
			$self->{'out'} = $self->backquote_logged("$self->{'cmd'} 2>&1 </dev/null");
			push(@rout, $self->{'out'});
			}
		}

	return (1, $newiface->{'fullname'}, $newiface->{'address'},
		   $newiface->{'ptp'}, \@rcmd, \@rout);
	}
else {
	# Must have timed out due to a failure
	$self->foreign_require("syslog", "syslog-lib.pl");
	local $sysconf = $main::syslog_object->get_config();
	local $c;
	local $logs;
	foreach $c (@$sysconf) {
		next if ($c->{'tag'} || !$c->{'file'} || !-f $c->{'file'});
		local @st = stat($c->{'file'});
		if ($st[9] > $start) {
			# Was modified since start .. but by ppp or pptp?
			local $tail = `tail -10 '$c->{'file'}'`;
			if ($tail =~ /ppp|pptp/) {
				$logs = $tail;
				last;
				}
			}
		}
	return (0, $tempout.$logs || "No logged error messages found");
	}
}

sub get_ppp_ifaces
{
	my $self = shift;

return grep { $_->{'fullname'} =~ /^ppp(\d+)$/ &&
	      $_->{'up'} && $_->{'address'} } $main::net_object->active_interfaces();
}


1;

