#!/usr/bin/perl

&MAIN();

 sub MAIN {
# save.cgi
# Create, update or delete a PPTP tunnel

&ModuleLoader::preload_module('pptp_client_class', "pptp_client::PptpclientLib");
$self = new pptp_client::PptpclientLib:: 'pptp_client_class',%$self;

$self->ReadParse();
@{$self->{'tunnels_array'}} = $self->list_tunnels();
@{$self->{'secs_array'}} = $self->list_secrets();
if (!$self->{'in_hash'}->{'new'}) {
	($self->{'tunnel'}) = grep { $_->{'name'} eq $self->{'in_hash'}->{'old'} } @{$self->{'tunnels_array'}};
	$self->parse_comments($self->{'tunnel'});
	$self->{'name'} = $self->find("name", $self->{'tunnel'}->{'opts'});
	$self->{'remote'} = $self->find("remotename", $self->{'tunnel'}->{'opts'});
	$self->{'sec'} = &find_secret($self->{'name'} ? $self->{'name'}->{'value'} : $self->get_system_hostname(1),
			    $self->{'remote'} ? $self->{'remote'}->{'value'} : undef);
	$self->lock_file($self->{'tunnel'}->{'file'});
	}
else {
	$self->{'tunnel'} = { 'opts' => [ ] };
	}
$self->error_setup($self->{'text_hash'}->{'save_err'});
$self->lock_file($self->{'config_hash'}->{'pap_file'});

if ($self->{'in_hash'}->{'delete'}) {
	# Just delete this tunnel and it's secret (if not used by any other)
	unlink($self->{'tunnel'}->{'file'});
	}
else {
	# Validate inputs
	$self->{'in_hash'}->{'tunnel'} =~ /\S/ || $self->error($self->{'text_hash'}->{'save_ename'});
	gethostbyname($self->{'in_hash'}->{'server'}) || $self->error($self->{'text_hash'}->{'save_eserver'});
	$self->{'in_hash'}->{'login_def'} || $self->{'in_hash'}->{'login'} =~ /^\S+$/ ||
		$self->error($self->{'text_hash'}->{'save_elogin'});
	$self->{'in_hash'}->{'remote_def'} || $self->{'in_hash'}->{'remote'} =~ /^\S+$/ ||
		$self->error($self->{'text_hash'}->{'save_eremote'});
	$self->{'in_hash'}->{'file_def'} < 2 || -r $self->{'in_hash'}->{'file'} ||
		$self->error($self->{'text_hash'}->{'save_efile'});

	# Add default route changes
	if ($self->{'in_hash'}->{'deldef'}) {
		push(@{$self->{'routes_array'}}, "delete default");
		}
	if ($self->{'in_hash'}->{'adddef'} == 1) {
		push(@{$self->{'routes_array'}}, "add default dev TUNNEL_DEV");
		}
	elsif ($self->{'in_hash'}->{'adddef'} == 2) {
		gethostbyname($self->{'in_hash'}->{'def'}) || $self->error($self->{'text_hash'}->{'save_edef'});
		push(@{$self->{'routes_array'}}, "add default gw ".$self->{'in_hash'}->{'def'});
		}

	# Parse and add extra route commands
	$self->{'in_hash'}->{'unknown'} =~ s/\r//g;
	push(@{$self->{'routes_array'}}, grep { /\S/ } split(/\n/, $self->{'in_hash'}->{'unknown'}));

	# Parse and add static routes
	for($i=0; defined($self->{'t'} = $self->{'in_hash'}->{"type_$i"}); $i++) {
		next if (!$self->{'t'});
		if ($self->{'t'} == 1) {
			$self->check_ipaddress($self->{'in_hash'}->{"net_$i"}) ||
				$self->error($self->text('save_enet', $i+1));
			$self->check_ipaddress($self->{'in_hash'}->{"mask_$i"}) ||
				$self->error($self->text('save_emask', $i+1));
			$self->{'in_hash'}->{"gw_def_$i"} || $self->check_ipaddress($self->{'in_hash'}->{"gw_$i"}) ||
				$self->error($self->text('save_egw', $i+1));
			if ($self->{'in_hash'}->{"gw_def_$i"}) {
				push(@{$self->{'routes_array'}},
				    sprintf("add -net %s dev %s netmask %s",
					$self->{'in_hash'}->{"net_$i"}, 'TUNNEL_DEV',
					$self->{'in_hash'}->{"mask_$i"}));
				}
			else {
				push(@{$self->{'routes_array'}},
				    sprintf("add -net %s gw %s netmask %s",
					$self->{'in_hash'}->{"net_$i"}, $self->{'in_hash'}->{"gw_$i"},
					$self->{'in_hash'}->{"mask_$i"}));
				}
			}
		else {
			$self->check_ipaddress($self->{'in_hash'}->{"net_$i"}) ||
				$self->error($self->text('save_ehost', $i+1));
			$self->{'in_hash'}->{"mask_$i"} && $self->error($self->text('save_emask2', $i+1));
			$self->{'in_hash'}->{"gw_def_$i"} || $self->check_ipaddress($self->{'in_hash'}->{"gw_$i"}) ||
				$self->error($self->text('save_egw2', $i+1));
			if ($self->{'in_hash'}->{"gw_def_$i"}) {
				push(@{$self->{'routes_array'}},
				    sprintf("add -host %s dev %s",
					$self->{'in_hash'}->{"net_$i"}, 'TUNNEL_DEV'));
				}
			else {
				push(@{$self->{'routes_array'}},
				    sprintf("add -host %s gw %s",
					$self->{'in_hash'}->{"net_$i"}, $self->{'in_hash'}->{"gw_$i"}));
				}
			}
		}

	mkdir($self->{'config_hash'}->{'peers_dir'}, 0755);
	if ($self->{'in_hash'}->{'new'}) {
		&check_clash();

		# Create file and set default options
		$self->{'tunnel'}->{'name'} = $self->{'in_hash'}->{'tunnel'};
		$self->{'tunnel'}->{'file'} = "$self->{'config_hash'}->{'peers_dir'}/$self->{'in_hash'}->{'tunnel'}";
		$self->lock_file($self->{'tunnel'}->{'file'});
		$self->save_ppp_option($self->{'tunnel'}->{'opts'}, $self->{'tunnel'}->{'file'}, undef,
				 { 'comment' => "PPTP Tunnel configuration for tunnel $self->{'in_hash'}->{'tunnel'}" });
		}
	else {
		# Check for a re-name
		if ($self->{'in_hash'}->{'tunnel'} ne $self->{'tunnel'}->{'name'}) {
			&check_clash();
			$self->{'tunnel'}->{'name'} = $self->{'in_hash'}->{'tunnel'};
			$self->{'nf'} = "$self->{'config_hash'}->{'peers_dir'}/$self->{'in_hash'}->{'tunnel'}";
			rename($self->{'tunnel'}->{'file'}, $self->{'nf'}) ||
				$self->error($self->{'text_hash'}->{'save_erename'});
			$self->{'tunnel'}->{'file'} = $self->{'nf'};
			}
		}

	# Save server IP
	$self->save_ppp_option($self->{'tunnel'}->{'opts'}, $self->{'tunnel'}->{'file'},
			 $self->{'tunnel'}->{'server_c'},
			 { 'comment' => "Server IP: $self->{'in_hash'}->{'server'}" } );

	# Save all routes
	@{$self->{'or_array'}} = @{$self->{'tunnel'}->{'routes_c'}};
	for($i=0; $i<@{$self->{'routes_array'}} || $i<@{$self->{'or_array'}}; $i++) {
		$self->save_ppp_option($self->{'tunnel'}->{'opts'}, $self->{'tunnel'}->{'file'},
				 $self->{'or_array'}->[$i], $self->{'routes_array'}->[$i] ?
					{ 'comment' => "Route: $self->{'routes_array'}->[$i]" } : undef);
		}

	# Save PPP options
	$self->save_ppp_option($self->{'tunnel'}->{'opts'}, $self->{'tunnel'}->{'file'}, "name",
			 $self->{'in_hash'}->{'login_def'} ? undef :
			 { 'name' => 'name', 'value' => $self->{'in_hash'}->{'login'} } );
	$self->save_ppp_option($self->{'tunnel'}->{'opts'}, $self->{'tunnel'}->{'file'}, "remotename",
			 $self->{'in_hash'}->{'remote_def'} ? undef :
			 { 'name' => 'remotename','value' => $self->{'in_hash'}->{'remote'} } );
	$self->save_ppp_option($self->{'tunnel'}->{'opts'}, $self->{'tunnel'}->{'file'}, "file",
			 $self->{'in_hash'}->{'file_def'} == 0 ? undef :
			 $self->{'in_hash'}->{'file_def'} == 1 ? 
				{ 'name' => 'file',
				  'value' => $self->{'config_hash'}->{'pptp_options'} } :
				{ 'name' => 'file',
				  'value' => $self->{'in_hash'}->{'file'} });
	$self->parse_mppe_options($self->{'tunnel'}->{'opts'}, $self->{'tunnel'}->{'file'});

	# Update or add to the secrets file
	$self->{'newname'} = $self->{'in_hash'}->{'login_def'} ? $self->get_system_hostname(1) : $self->{'in_hash'}->{'login'};
	$self->{'newremote'} = $self->{'in_hash'}->{'remote_def'} ? "*" : $self->{'in_hash'}->{'remote'};
	if (!$self->{'sec'}) {
		# No old secret was found, so look for one matching the new
		# details
		$self->{'sec'} = &find_secret($self->{'newname'}, $self->{'newremote'});
		}
	if ($self->{'sec'}) {
		# Just update the secret for our login name with the new login
		# and password. This can happen when re-naming, or if a secret
		# for the name already exists
		$self->{'sec'}->{'client'} = $self->{'newname'};
		if ($self->{'sec'}->{'server'} ne '*' && $self->{'newremote'} ne '*') {
			$self->{'sec'}->{'server'} = $self->{'newremote'};
			}
		$self->{'sec'}->{'secret'} = $self->{'in_hash'}->{'pass'};
		$self->change_secret($self->{'sec'});
		}
	else {
		# Need to create a new secret
		$self->{'sec'} = { 'client' => $self->{'newname'},
			 'secret' => $self->{'in_hash'}->{'pass'},
			 'server' => $self->{'newremote'} };
		$self->create_secret($self->{'sec'});
		}

	$self->flush_file_lines();
	}
$self->unlock_file($self->{'tunnel'}->{'file'});
$self->unlock_file($self->{'config_hash'}->{'pap_file'});

$self->webmin_log($self->{'in_hash'}->{'new'} ? "create" : $self->{'in_hash'}->{'delete'} ? "delete" : "update",
	    "tunnel", $self->{'tunnel'}->{'name'});
$self->redirect("");

1;
		
}

sub check_clash
{
	shift if ($self == $_[0]);

-r "$self->{'config_hash'}->{'peers_dir'}/$self->{'in_hash'}->{'tunnel'}" && $self->error($self->{'text_hash'}->{'save_eclash'});
}

# find_secret(client, server)
# Returns the best matching secret with the given details
sub find_secret
{
	shift if ($self == $_[0]);

local ($exact) = grep { $_->{'client'} eq $_[0] &&
			$_->{'server'} eq $_[1] } @{$self->{'secs_array'}};
return $exact if ($exact);
local ($client) = grep { $_->{'client'} eq $_[0] } @{$self->{'secs_array'}};
return $client;
}


1;

