#!/usr/bin/perl
package proc::SysvLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'proc::SysvLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# sysv-lib.pl
# Functions for parsing sysv-style ps output

$self->{'has_stime'} = $self->{'gconfig_hash'}->{'os_type'} eq 'solaris';
$self->{'has_task'} = $self->{'gconfig_hash'}->{'os_type'} eq 'solaris' && $self->{'gconfig_hash'}->{'os_version'} >= 10;
$self->{'has_zone'} = $self->{'gconfig_hash'}->{'os_type'} eq 'solaris' && $self->{'gconfig_hash'}->{'os_version'} >= 10;

$self->{'has_trace_command'} = $self->{'gconfig_hash'}->{'os_type'} eq 'solaris' &&
		     $self->has_command("truss");

foreach $ia (keys %{$self->{'text_hash'}}) {
	if ($ia =~ /^sysv(_\S+)/) {
		$self->{'info_arg_map_hash'}->{$1} = $self->{'text_hash'}->{$ia};
		}
	}
delete($self->{'info_arg_map_hash'}->{'_stime'}) if (!$self->{'has_stime'});

@{$self->{'nice_range_array'}} = (-20 .. 19);

$self->{'has_fuser_command'} = 1;

1;

1;
		return $self;
}

# list_processes([pid]*)
sub list_processes
{
	my $self = shift;

local($line, $dummy, @w, $i, $_, $pcmd, @plist);
foreach (@_) { $pcmd .= " -p $_"; }
if (!$pcmd) { $pcmd = " -e"; }
$ENV{'COLUMNS'} = 10000;	# needed on AIX
local @cols = ( "user","ruser","group","rgroup","pid","ppid","pgid","pcpu","vsz",
		"nice","etime","time",
		($self->{'has_stime'} ? ("stime") : ( )),
		($self->{'has_task'} ? ("taskid") : ( )),
		($self->{'has_zone'} ? ("zone") : ( )),
		"tty","args" );
open(PS, "ps -o ".join(",", @cols)." $pcmd |");
$dummy = <PS>;
for($i=0; $line=<PS>; $i++) {
	chop($line);
	$line =~ s/^\s+//g;
	@w = split(/\s+/, $line);
	if ($line =~ /ps -o user,ruser/) {
		# Skip ps command
		$i--; next;
		}
	$plist[$i]->{"pid"} = $w[4];
	$plist[$i]->{"ppid"} = $w[5];
	$plist[$i]->{"user"} = $w[0];
	$plist[$i]->{"cpu"} = "$w[7] %";
	$plist[$i]->{"size"} = "$w[8] kB";
	local $ofs = 0;
	if ($self->{'has_stime'}) {
		$plist[$i]->{"_stime"} = $w[12+$ofs];
		$plist[$i]->{"_stime"} =~ s/_/ /g;
		$ofs++;
		}
	if ($self->{'has_task'}) {
		$plist[$i]->{"_task"} = $w[12+$ofs];
		$ofs++;
		}
	if ($self->{'has_zone'}) {
		$plist[$i]->{"_zone"} = $w[12+$ofs];
		$ofs++;
		}
	$plist[$i]->{"time"} = $w[11];
	$plist[$i]->{"nice"} = $w[9] =~ /\d+/ ? $w[9]-20 : $w[9];
	$plist[$i]->{"args"} = @w<14+$ofs ? "defunct"
					  : join(' ', @w[13+$ofs..$#w]);
	$plist[$i]->{"_group"} = $w[2];
	$plist[$i]->{"_ruser"} = $w[1];
	$plist[$i]->{"_rgroup"} = $w[3];
	$plist[$i]->{"_pgid"} = $w[6];
	$plist[$i]->{"_tty"} = $w[12+$ofs] =~ /\?/ ? $self->{'text_hash'}->{'edit_none'}
					           : "/dev/$w[12+$ofs]";
	}
close(PS);
return @plist;
}

# find_mount_processes(mountpoint)
# Find all processes under some mount point
sub find_mount_processes
{
	my $self = shift;

local($out);
$out = `fuser -c $_[0] 2>/dev/null`;
$out =~ s/^\s+//g; $out =~ s/\s+$//g;
return split(/\s+/, $out);
}

# find_file_processes([file]+)
# Find all processes with some file open
sub find_file_processes
{
	my $self = shift;

local($out, $files);
$files = join(' ', @_);
$out = `fuser $files 2>/dev/null`;
$out =~ s/^\s+//g; $out =~ s/\s+$//g;
return split(/\s+/, $out);
}

# renice_proc(pid, nice)
sub renice_proc
{
	my $self = shift;

return undef if ($self->is_readonly_mode());
$self->{'out'} = `renice $_[1] -p $_[0] 2>&1`;
if ($?) { return $self->{'out'}; }
return undef;
}

# get_new_pty()
# Returns the filehandles and names for a pty and tty
sub get_new_pty
{
	my $self = shift;

if (!-e "/dev/ptyp0") {
	# Must use IO::Pty :(
	$self->error("IO::Pty Perl module is not installed");
	}
else {
	# Need to search through pty files
	opendir(DEV, "/dev");
	local @ptys = map { "/dev/$_" } (grep { /^pty/ } readdir(DEV));
	closedir(DEV);
	local ($pty, $tty);
	foreach $pty (@ptys) {
		open(PTY, "+>$pty") || next;
		local $tty = $pty; $tty =~ s/pty/tty/;
		open(TTY, "+>$tty") || next;
		local $old = select(PTY); $| = 1;
		select(TTY); $| = 1; select($old);
		return (*PTY, *TTY, $pty, $tty);
		}
	return ();
	}
}

# open_process_trace(pid, [&syscalls])
# Starts tracing on some process, and returns a trace object
sub open_process_trace
{
	my $self = shift;

local $fh = time().$$;
local $sc;
if (@{$_[1]}) {
	$sc = "-t ".join(",", @{$_[1]});
	}
local $tpid = open($fh, "truss $sc -i -p $_[0] 2>&1 |");
$self->{'line'} = <$fh>;
return { 'pid' => $_[0],
	 'tpid' => $tpid,
	 'fh' => $fh };
}

# close_process_trace(&trace)
# Halts tracing on some trace object
sub close_process_trace
{
	my $self = shift;

kill('TERM', $_[0]->{'tpid'}) if ($_[0]->{'tpid'});
close($_[0]->{'fh'});
}

# read_process_trace(&trace)
# Returns an action structure representing one action by traced process, or
# undef if an error occurred
sub read_process_trace
{
	my $self = shift;

local $fh = $_[0]->{'fh'};
local @tm = localtime(time());
while(1) {
	local $line = <$fh>;
	return undef if (!$line);
	if ($line =~ /^([^\(]+)\((.*)\)(\s*=\s*(\-?\d+)|\s+(Err\S+))?/) {
		local $action = { 'time' => time(),
				  'call' => $1,
				  'rv' => $4 ne "" ? $4 : $5 };
		local $args = $2;
		local @args;
		while(1) {
			if ($args =~ /^[ ,]*(\{[^}]*\})(.*)$/) {
				# A structure in { }
				push(@args, $1);
				$args = $2;
				}
			elsif ($args =~ /^[ ,]*"([^"]*)"\.*(.*)$/) {
				# A quoted string
				push(@args, $1);
				$args = $2;
				}
			elsif ($args =~ /^[ ,]*\[([^\]]*)\](.*)$/) {
				# A square-bracket number
				push(@args, $1);
				$args = $2;
				}
			elsif ($args =~ /^[ ,]*\<([^\>]*)\>(.*)$/) {
				# An angle-bracketed string
				push(@args, $1);
				$args = $2;
				}
			elsif ($args =~ /[ ,]*([^, ]+)(.*)$/) {
				# Just a number
				push(@args, $1);
				$args = $2;
				}
			else {
				last;
				}
			}
		$action->{'args'} = \@args;
		return $action;
		}
	}
}

# get_memory_info()
# Returns a list containing the real mem, free real mem, swap and free swap
sub get_memory_info
{
	my $self = shift;

return ( ) if (!$self->has_command("top"));
local @rv;
open(TOP, "top 2>&1 |");
while(<TOP>) {
	if (/Memory:\s+(\S+)\s+real,\s+(\S+)\s+free,\s+(\S+)\s+swap\s+in\s+use,\s+(\S+)\s+swap\s+free/) {
		local ($real, $free, $swapused, $swapfree) = ($1, $2, $3, $4);
		@rv = ( $self->fix_suffix($real),
			$self->fix_suffix($free),
			$self->fix_suffix($swapused) + $self->fix_suffix($swapfree),
			$self->fix_suffix($swapfree) );
		}
	}
close(TOP);
return @rv;
}

sub fix_suffix
{
	my $self = shift;

if ($_[0] =~ /^(\d+)K/i) {
	return $1;
	}
elsif ($_[0] =~ /^(\d+)M/i) {
	return $1*1024;
	}
elsif ($_[0] =~ /^(\d+)G/i) {
	return $1*1024*1024;
	}
else {
	return int($1);
	}
}

# os_get_cpu_info()
# Returns a list containing the 5, 10 and 15 minute load averages
sub os_get_cpu_info
{
	my $self = shift;

local $out = `uptime 2>&1`;
if ($out =~ /load average:\s+(\S+),\s+(\S+),\s+(\S+)/) {
	return ($1, $2, $3);
	}
else {
	return ( );
	}
}


1;

