#!/usr/bin/perl
package proftpd::Log_parser;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'proftpd::Log_parser');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# log_parser.pl
# Functions for parsing this module's logs

&ModuleLoader::preload_module(ref($self), "proftpd::ProftpdLib");
$self = new proftpd::ProftpdLib:: ref($self), %$self; 

1;
		return $self;
}

# parse_webmin_log(user, script, action, type, object, &params, long)
# Converts logged information from this module into human-readable form
sub parse_webmin_log
{
	my $self = shift;

local ($user, $script, $action, $type, $object, $p, $long) = @_;
local $typestr = "<i>".$self->{'text_hash'}->{"type_$type"}."</i>";
if ($action eq 'global') {
	return $self->text('log_global', $typestr);
	}
elsif ($action eq 'ftpusers') {
	return $self->{'text_hash'}->{'log_ftpusers'};
	}
elsif ($action eq 'virt') {
	$object = $object eq '-' ? $self->{'text_hash'}->{'index_defserv'}
				 : $self->html_escape($object);
	if ($type eq 'create') {
		return $self->text('log_virtc', "<tt>$object</tt>");
		}
	elsif ($type eq 'save') {
		return $self->text('log_virts', "<tt>$object</tt>");
		}
	elsif ($type eq 'delete') {
		return $self->text('log_virtd', "<tt>$object</tt>");
		}
	elsif ($type eq 'manual') {
		return $self->text('log_virtm', "<tt>$object</tt>");
		}
	else {
		return $self->text('log_virt', $typestr, "<tt>$object</tt>");
		}
	}
elsif ($action eq 'dir') {
	local ($virt, $dir) = split(/:/, $object);
	$virt = $virt ? $self->html_escape($virt) : $self->{'text_hash'}->{'index_defserv'};
	$dir = $self->html_escape($dir);
	if ($type eq 'create') {
		return $self->text($long ? 'log_dirc_l' : 'log_dirc',
			     "<tt>$dir</tt>", "<tt>$virt</tt>");
		}
	elsif ($type eq 'save') {
		return $self->text($long ? 'log_dirs_l' : 'log_dirs',
			     "<tt>$dir</tt>", "<tt>$virt</tt>");
		}
	elsif ($type eq 'delete') {
		return $self->text($long ? 'log_dird_l' : 'log_dird',
			     "<tt>$dir</tt>", "<tt>$virt</tt>");
		}
	elsif ($type eq 'manual') {
		return $self->text($long ? 'log_dirm_l' : 'log_dirm',
			     "<tt>$dir</tt>", "<tt>$virt</tt>");
		}
	else {
		return $self->text($long ? 'log_dir_l' : 'log_dir', $typestr,
			     "<tt>$dir</tt>", "<tt>$virt</tt>");
		}
	}
elsif ($action eq 'limit') {
	if ($type eq 'create') {
		return $self->text('log_limitc', "<tt>$object</tt>");
		}
	elsif ($type eq 'save') {
		return $self->text('log_limits', "<tt>$object</tt>");
		}
	elsif ($type eq 'delete') {
		return $self->text('log_limitd', "<tt>$object</tt>");
		}
	elsif ($type eq 'manual') {
		return $self->text('log_limitm', "<tt>$object</tt>");
		}
	else {
		return $self->text('log_limit', $typestr, "<tt>$object</tt>");
		}
	}
elsif ($action eq 'ftpaccess') {
	if ($type eq 'create') {
		return $self->text('log_ftpaccessc', "<tt>$object</tt>");
		}
	elsif ($type eq 'save') {
		return $self->text('log_ftpaccesss', "<tt>$object</tt>");
		}
	elsif ($type eq 'delete') {
		return $self->text('log_ftpaccessd', "<tt>$object</tt>");
		}
	elsif ($type eq 'manual') {
		return $self->text('log_ftpaccessm', "<tt>$object</tt>");
		}
	else {
		return $self->text('log_ftpaccess', $typestr, "<tt>$object</tt>");
		}
	}
elsif ($action eq 'anon') {
	$object = $object eq '-' ? $self->{'text_hash'}->{'index_defserv'}
				 : $self->html_escape($object);
	if ($type eq 'create') {
		return $self->text('log_anonc', "<tt>$object</tt>");
		}
	elsif ($type eq 'save') {
		return $self->text('log_anons', "<tt>$object</tt>");
		}
	elsif ($type eq 'delete') {
		return $self->text('log_anond', "<tt>$object</tt>");
		}
	elsif ($type eq 'manual') {
		return $self->text('log_anonm', "<tt>$object</tt>");
		}
	else {
		return $self->text('log_anon', $typestr, "<tt>$object</tt>");
		}
	}
else {
	return $self->{'text_hash'}->{"log_$action"};
	}
}


1;

