#!/usr/bin/perl
package proftpd::Mod_core;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'proftpd::Mod_core');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# mod_core.pl
# Core proftpd directives

1;
		return $self;
}

# mod_core_directives(version)
# Returns a directive structure, like the one user by Apache. Types are :
#	0 - Networking
#	1 - Logging
#	2 - Files
#	3 - Access control
#	4 - Misc
#	5 - User and Group
#	6 - Authentication
sub mod_core_directives
{
	my $self = shift;

local $rv = [
	[ 'AccessDenyMsg', 0, 3, 'virtual anon global', 1.202 ],
	[ 'AccessGrantMsg', 0, 3, 'virtual anon global', 0.99 ],
	[ 'Allow Deny Order', 1, 3, 'limit', 0.99 ],
	[ 'AllowAll DenyAll', 0, 3, 'directory anon limit ftpaccess', 0.99 ],
	[ 'AllowFilter', 0, 3, 'virtual anon global', 1.20 ],
	[ 'AllowForeignAddress', 0, 0, 'virtual anon global', 1.17 ],
	[ 'AllowGroup', 1, 3, 'limit', 1.11 ],
	[ 'AllowOverwrite', 0, 3, 'virtual anon directory ftpaccess global', 0.99 ],
	[ 'AllowUser', 1, 3, 'limit', 1.17 ],
	[ 'AllowRetrieveRestart', 0, 0, 'virtual anon directory global ftpaccess', 0.99 ],
	[ 'AllowStoreRestart', 0, 0, 'virtual anon directory global ftpaccess', 0.99 ],
	[ 'AnonRequirePassword', 0, 6, 'anon', 0.99 ],
	[ 'AnonymousGroup', 0, 6, 'virtual global', 1.13 ],
	[ 'AuthAliasOnly', 0, 6, 'virtual anon global', 1.13 ],
	[ 'AuthUsingAlias', 0, 6, 'anon', 1.20 ],
	[ 'Bind', 0, 0, 'virtual', 1.16 ],
	[ 'CDPath', 1, 2, 'virtual anon global', 1.20 ],
	[ 'Class Classes', 1, 3, 'virtual', 1.20 ],
	[ 'CommandBufferSize', 0, 0, 'virtual global', 1.20 ],
	[ 'DefaultServer', 0, 0, 'virtual', undef, 0.99, 8 ],
	[ 'DefaultTransferMode', 0, 0, 'virtual global', 1.20 ],
	[ 'DeferWelcome', 0, 0, 'virtual global', 0.99 ],
	[ 'DeleteAbortedStores', 0, 2, 'virtual directory anon global ftpaccess', 1.20 ],
	[ 'DenyFilter', 0, 3, 'virtual anon global', 1.20 ],
	[ 'DenyGroup', 1, 3, 'limit', 1.11 ],
	[ 'DenyUser', 1, 3, 'limit', 1.17 ],
	[ 'DisplayConnect', 0, 6, 'virtual global', 1.20 ],
	[ 'DisplayFirstChdir', 0, 2, 'virtual anon directory global', 0.99 ],
	[ 'DisplayGoAway', 0, 6, 'virtual anon global', 1.20 ],
	[ 'DisplayLogin', 0, 6, 'virtual anon global', 0.99 ],
	[ 'DisplayQuit', 0, 6, 'virtual anon global', 1.20 ],
	[ 'Group', 0, 5, 'virtual anon', undef, 0.99, 9 ],
	[ 'GroupOwner', 0, 5, 'anon directory ftpaccess', 0.99 ],
	[ 'GroupPassword', 1, 6, 'virtual anon global', 0.99 ],
	[ 'HiddenStor', 0, 2, 'virtual anon directory global', 1.20 ],
	[ 'HideGroup', 1, 2, 'directory anon', 0.99 ],
	[ 'HideNoAccess', 0, 2, 'directory anon', 0.99 ],
	[ 'HideUser', 1, 2, 'directory anon', 0.99 ],
	[ 'IdentLookups', 0, 0, 'virtual global', 1.15 ],
	[ 'IgnoreHidden', 0, 2, 'limit', 0.99 ],
	[ 'MasqueradeAddress', 0, 0, 'virtual', 1.202 ],
	[ 'MaxClients', 0, 0, 'virtual anon global', 0.99 ],
	[ 'MaxClientsPerHost', 0, 0, 'virtual anon global', 1.17 ],
#	[ 'MaxClientsPerUser', 0, 0, 'virtual anon global', 1.20 ],
	[ 'MaxInstances', 0, 0, 'root', undef, 1.16, 8 ],
	[ 'MaxLoginAttempts', 0, 6, 'virtual global', 0.99 ],
	[ 'MultilineRFC2228', 0, 0, 'root', 1.20 ],
	[ 'PassivePorts', 0, 0, 'virtual global', 1.20 ],
	[ 'PathAllowFilter', 0, 2, 'virtual anon global', 1.17 ],
	[ 'PathDenyFilter', 0, 2, 'virtual anon global', 1.17 ],
	[ 'PidFile', 0, 4, 'root', 1.20 ],
	[ 'Port', 0, 0, 'virtual', 0.99 ],
	[ 'RequireValidShell', 0, 6, 'virtual anon global', 0.99 ],
	[ 'RLimitCPU', 0, 4, 'root', 1.202 ],
	[ 'RLimitMemory', 0, 4, 'root', 1.202 ],
	[ 'RLimitOpenFiles', 0, 4, 'root', 1.202 ],
	[ 'ScoreboardPath', 0, 4, 'root', 1.16 ],
	[ 'ServerAdmin', 0, 4, 'virtual', 0.99 ],
	[ 'ServerIdent', 0, 0, 'virtual global', 1.20 ],
	[ 'ServerName', 0, 4, 'virtual', undef, 0.99, 11 ],
	[ 'ServerType', 0, 0, 'root', undef, 0.99, 10 ],
	[ 'ShowSymlinks', 0, 2, 'virtual anon global', 0.99 ],
	[ 'SocketBindTight', 0, 0, 'root', 0.99 ],
	[ 'SyslogFacility', 0, 1, 'root', 1.16 ],
	[ 'SyslogLevel', 0, 1, 'virtual anon global', 1.20 ],
	[ 'tcpBackLog', 0, 0, 'root', 0.99 ],
	[ 'tcpNoDelay', 0, 0, 'virtual global', 1.20 ],
	[ 'tcpReceiveWindow', 0, 0, 'virtual', 0.99 ],
	[ 'tcpSendWindow', 0, 0, 'virtual', 0.99 ],
	[ 'TimesGMT', 0, 4, 'root', 1.20 ],
	[ 'TimeoutIdle', 0, 0, 'root', 0.99 ],
	[ 'TimeoutLogin', 0, 0, 'root', 0.99 ],
	[ 'TimeoutNoTransfer', 0, 0, 'root', 0.99 ],
	[ 'TimeoutStalled', 0, 0, 'root', 1.16 ],
	[ 'TransferLog', 0, 1, 'virtual anon global', undef, 1.14, 10 ],
	[ 'Umask', 0, 2, 'virtual directory ftpaccess', undef, 0.99, 3 ],
	[ 'UseFtpUsers', 0, 6, 'virtual anon global', 0.99 ],
	[ 'UseHostsAllowFile', 0, 3, 'virtual anon', 1.20 ],
	[ 'UseHostsDenyFile', 0, 3, 'virtual anon', 1.20 ],
	[ 'UseReverseDNS', 0, 0, 'root', 1.17 ],
	[ 'User', 0, 5, 'virtual anon', undef, 0.99, 10 ],
	[ 'UserDirRoot', 0, 2, 'anon', 1.20 ],
	[ 'UserAlias', 1, 6, 'virtual anon global', 0.99 ],
	[ 'UserOwner', 0, 5, 'anon directory', 1.20 ],
	[ 'UserPassword', 1, 6, 'virtual anon global', 0.99 ],
	[ 'WtmpLog', 0, 4, 'virtual anon global', 1.17 ],
	];
return $self->make_directives($rv, $_[0], "mod_core");
}

sub edit_AccessDenyMsg
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_accessdeny'},
	$self->opt_input($_[0]->{'words'}->[0], "AccessDenyMsg", $self->{'text_hash'}->{'default'}, 20));
}

sub save_AccessDenyMsg
{
	my $self = shift;

return $self->parse_opt("AccessDenyMsg");
}

sub edit_AccessGrantMsg
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_accessgrant'},
	$self->opt_input($_[0]->{'words'}->[0], "AccessGrantMsg", $self->{'text_hash'}->{'default'}, 20));
}

sub save_AccessGrantMsg
{
	my $self = shift;

return $self->parse_opt("AccessGrantMsg");
}

sub edit_Allow_Deny_Order
{
	my $self = shift;

local (@type, @what, @mode, $i);
foreach $d (@{$_[0]}, @{$_[1]}) {
	local @w = @{$d->{'words'}};
	shift(@w) if (lc($w[0]) eq 'from');
	for($i=0; $i<@w; $i++) {
		push(@type, lc($d->{'name'}) eq "allow" ? 1 : 2);
		push(@what, $w[$i] eq 'all' || $w[$i] eq 'none' ? undef
								: $w[$i]);
		if ($w[$i] eq 'all') { push(@mode, 0); }
		elsif ($w[$i] eq 'none') { push(@mode, 1); }
		elsif ($w[$i] =~ /^\d+\.\d+\.\d+\.\d+$/) { push(@mode, 2); }
		elsif ($w[$i] =~ /^[0-9\.\/]+$/) { push(@mode, 3); }
		else { push(@mode, 4); }
		}
	}
push(@type, ""); push(@what, ""); push(@mode, 0);
$self->{'rv'} = "<i>$self->{'text_hash'}->{'mod_core_order'}</i>\n".
      $self->choice_input($_[2]->[0]->{'value'}, "order", "",
      		    "$self->{'text_hash'}->{'mod_core_denyallow'},deny,allow", 
      		    "$self->{'text_hash'}->{'mod_core_allowdeny'},allow,deny", 
      		    "$self->{'text_hash'}->{'default'},")."<br>\n";
$self->{'rv'} .= "<table border>\n".
       "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'mod_core_action'}</b></td> ".
       "<td><b>$self->{'text_hash'}->{'mod_core_cond'}</b></td> </tr>\n";
@{$self->{'sels_array'}} = map { $self->{'text_hash'}->{"mod_core_mode_$_"}.','.$_ } (0 .. 4);
for($i=0; $i<@type; $i++) {
	$self->{'rv'} .= "<tr $self->{'cb'}> <td>".$self->select_input($type[$i], "allow_type_$i", "0",
		"&nbsp;,0", "$self->{'text_hash'}->{'mod_core_allow'},1",
		"$self->{'text_hash'}->{'mod_core_deny'},2")."</td>\n";
	$self->{'rv'} .= "<td>".$self->select_input($mode[$i], "allow_mode_$i", "0", @{$self->{'sels_array'}});
	$self->{'rv'} .= sprintf "<input name=allow_what_$i size=20 value=\"%s\"></td>\n",
		 $mode[$i] ? $what[$i] : "";
	$self->{'rv'} .= "</tr>\n";
	}
$self->{'rv'} .= "</table>\n";
return (2, $self->{'text_hash'}->{'mod_core_allow_deny'}, $self->{'rv'});
}

sub save_Allow_Deny_Order
{
	my $self = shift;

local ($i, $type, $mode, $what, @allow, @deny);
for($i=0; defined($type = $self->{'in_hash'}->{"allow_type_$i"}); $i++) {
	$mode = $self->{'in_hash'}->{"allow_mode_$i"}; $what = $self->{'in_hash'}->{"allow_what_$i"};
	next if (!$type);
	if ($mode == 0) { $what = "all"; }
	elsif ($mode == 1) { $what = "none"; }
	elsif ($mode == 2) {
		$self->check_ipaddress($what) || $self->error($self->text('mod_core_eip', $what));
		}
	elsif ($mode == 3) {
		$what =~ /^[0-9\.]+\.$/ ||
			$what =~ /^([0-9\.]+)\/\d+$/ && $self->check_ipaddress($1) ||
				$self->error($self->text('mod_core_enet', $what));
		}
	elsif ($mode == 4) {
		$what =~ /^[A-Za-z0-9\.\-]+$/ ||
			$self->error($self->text('mod_core_ehost', $what));
		}
	if ($type == 1) { push(@allow, $what); }
	else { push(@deny, $what); }
	}
return ( \@allow, \@deny, $self->parse_choice("order", ""));
}

sub edit_AllowAll_DenyAll
{
	my $self = shift;

#local $a = @{$_[0]}, $d = @{$_[1]};
local $a = $_[0], $d = $_[1];
local $rv = sprintf "<input type=radio name=AllowAll value=0 %s> %s\n",
	$a || $d ? "" : "checked", $self->{'text_hash'}->{'mod_core_addefault'};
$rv .= sprintf "<input type=radio name=AllowAll value=1 %s> %s\n",
	$a ? "checked" : "", $self->{'text_hash'}->{'mod_core_allowall'};
$rv .= sprintf "<input type=radio name=AllowAll value=2 %s> %s\n",
	$d ? "checked" : "", $self->{'text_hash'}->{'mod_core_denyall'};
return (1, $self->{'text_hash'}->{'mod_core_adall'}, $rv);
}

sub save_AllowAll_DenyAll
{
	my $self = shift;

return $self->{'in_hash'}->{'AllowAll'} == 0 ? ( [ ], [ ] ) :
       $self->{'in_hash'}->{'AllowAll'} == 1 ? ( [ "" ], [ ] ) : ( [ ], [ "" ] );
}

sub edit_AllowFilter
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_filter'},
	$self->opt_input($_[0]->{'value'}, "AllowFilter", $self->{'text_hash'}->{'default'}, 15));
}

sub save_AllowFilter
{
	my $self = shift;

return $self->parse_opt("AllowFilter");
}

sub edit_AllowForeignAddress
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_foreign'},
	$self->choice_input($_[0]->{'value'}, "AllowForeignAddress", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_AllowForeignAddress
{
	my $self = shift;

return $self->parse_choice("AllowForeignAddress", "");
}

sub edit_AllowGroup
{
	my $self = shift;

local $v = @{$_[0]} ? join(" ", (map { $_->{'value'} } @{$_[0]})) : undef;
return (2, $self->{'text_hash'}->{'mod_core_agroup'},
	$self->opt_input($v, "AllowGroup", $self->{'text_hash'}->{'mod_core_all'}, 50));
}

sub save_AllowGroup
{
	my $self = shift;

return ( $self->{'in_hash'}->{'AllowGroup_def'} ? [ ] : [ split(/\s+/, $self->{'in_hash'}->{'AllowGroup'}) ] );
}

sub edit_AllowOverwrite
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_overwrite'},
	$self->choice_input($_[0]->{'value'}, "AllowOverwrite", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_AllowOverwrite
{
	my $self = shift;

return $self->parse_choice("AllowOverwrite", "");
}

sub edit_AllowRetrieveRestart
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_restart'},
	$self->choice_input($_[0]->{'value'}, "AllowRetrieveRestart", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_AllowRetrieveRestart
{
	my $self = shift;

return $self->parse_choice("AllowRetrieveRestart", "");
}

sub edit_AllowStoreRestart
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_restart2'},
	$self->choice_input($_[0]->{'value'}, "AllowStoreRestart", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_AllowStoreRestart
{
	my $self = shift;

return $self->parse_choice("AllowStoreRestart", "");
}

sub edit_AllowUser
{
	my $self = shift;

local $v = @{$_[0]} ? join(" ", (map { $_->{'value'} } @{$_[0]})) : undef;
return (2, $self->{'text_hash'}->{'mod_core_auser'},
	$self->opt_input($v, "AllowUser", $self->{'text_hash'}->{'mod_core_all'}, 50));
}

sub save_AllowUser
{
	my $self = shift;

return ( $self->{'in_hash'}->{'AllowUser_def'} ? [ ] : [ split(/\s+/, $self->{'in_hash'}->{'AllowUser'}) ] );
}

sub edit_AnonRequirePassword
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_require'},
	$self->choice_input($_[0]->{'value'}, "AnonRequirePassword", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_AnonRequirePassword
{
	my $self = shift;

return $self->parse_choice("AnonRequirePassword", "");
}

sub edit_AnonymousGroup
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_anongroup'},
	$self->opt_input($_[0]->{'value'}, "AnonymousGroup", $self->{'text_hash'}->{'default'}, 50));
	
}

sub save_AnonymousGroup
{
	my $self = shift;

return $self->parse_opt("AnonymousGroup", '\S', $self->{'text_hash'}->{'mod_core_eanongroup'});
}

sub edit_AuthAliasOnly
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_authalias'},
	$self->choice_input($_[0]->{'value'}, "AuthAliasOnly", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_AuthAliasOnly
{
	my $self = shift;

return $self->parse_choice("AuthAliasOnly", "");
}

sub edit_AuthUsingAlias
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_authusingalias'},
	$self->choice_input($_[0]->{'value'}, "AuthUsingAlias", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_AuthUsingAlias
{
	my $self = shift;

return $self->parse_choice("AuthUsingAlias", "");
}

sub edit_Bind
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_bind'},
	$self->opt_input($_[0]->{'value'}, "Bind", $self->{'text_hash'}->{'mod_core_bind_all'}, 15));
}

sub save_Bind
{
	my $self = shift;

return $self->parse_opt("Bind", '^(\d+)\.(\d+)\.(\d+)\.(\d+)|([0-9:]+)$',
		  $self->{'text_hash'}->{'mod_core_ebind'});
}

sub edit_CDPath
{
	my $self = shift;

local $rv = "<textarea rows=3 cols=50 name=CDPath>";
foreach $p (@{$_[0]}) {
	$rv .= "$p->{'value'}\n";
	}
$rv .= "</textarea>\n";
return (2, $self->{'text_hash'}->{'mod_core_cdpath'}, $rv);
}

sub save_CDPath
{
	my $self = shift;

$self->{'in_hash'}->{'CDPath'} =~ s/\r//g;
return ( [ split(/\s+/, $self->{'in_hash'}->{'CDPath'}) ] );
}

sub edit_Class_Classes
{
	my $self = shift;

local $rv = $self->{'text_hash'}->{'mod_core_classes'}.
	    $self->choice_input($_[1]->[0]->{'value'}, "Classes", "",
		          "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      	  "$self->{'text_hash'}->{'default'},")."<br>\n";
$rv .= "<table border>\n".
       "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'mod_core_cname'}</b></td> ".
       "<td><b>$self->{'text_hash'}->{'mod_core_ctype'}</b></td> </tr>\n";
local $i = 0;
foreach $c (@{$_[0]}, { }) {
	local @w = @{$c->{'words'}};
	$rv .= "<tr $self->{'cb'}>\n";
	$rv .= "<td><input name=Class_n_$i size=10 value='$w[0]'></td>\n";
	$rv .= "<td><select name=Class_t_$i>\n";
	$rv .= sprintf "<option value=limit %s>%s\n",
		$w[1] eq 'limit' ? 'selected' : '', $self->{'text_hash'}->{'mod_core_climit'};
	$rv .= sprintf "<option value=regex %s>%s\n",
		$w[1] eq 'regex' ? 'selected' : '', $self->{'text_hash'}->{'mod_core_cregex'};
	$rv .= sprintf "<option value=ip %s>%s\n",
		$w[1] eq 'ip' ? 'selected' : '', $self->{'text_hash'}->{'mod_core_cip'};
	$rv .= "</select>\n";
	$rv .= "<input name=Class_v_$i size=20 value='$w[2]'></td>\n";
	$rv .= "</tr>\n";
	$i++;
	}
$rv .= "</table>\n";
return (2, $self->{'text_hash'}->{'mod_core_cls'}, $rv);
}

sub save_Class_Classes
{
	my $self = shift;

local ($i, @rv);
for($i=0; defined($self->{'in_hash'}->{"Class_n_$i"}); $i++) {
	next if (!$self->{'in_hash'}->{"Class_n_$i"});
	$self->{'in_hash'}->{"Class_t_$i"} ne 'limit' ||
		$self->{'in_hash'}->{"Class_v_$i"} =~ /^\d+$/ ||
			$self->error($self->{'text_hash'}->{'mod_core_eclimit'});
	$self->{'in_hash'}->{"Class_t_$i"} ne 'regex' ||
		$self->{'in_hash'}->{"Class_v_$i"} =~ /\S/ ||
			$self->error($self->{'text_hash'}->{'mod_core_ecregex'});
	$self->{'in_hash'}->{"Class_t_$i"} ne 'ip' ||
		$self->{'in_hash'}->{"Class_v_$i"} =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)\/(\d+)$/ ||
			$self->error($self->{'text_hash'}->{'mod_core_ecip'});
	push(@rv, join(" ", $self->{'in_hash'}->{"Class_n_$i"}, $self->{'in_hash'}->{"Class_t_$i"},
			    $self->{'in_hash'}->{"Class_v_$i"}));
	}
return ( \@rv, $self->{'in_hash'}->{'Classes'} eq 'on' ? [ 'on' ] :
	       $self->{'in_hash'}->{'Classes'} eq 'off' ? [ 'off' ] : [ ] );
}

sub edit_CommandBufferSize
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_buffer'},
	$self->opt_input($_[0]->{'value'}, "CommandBufferSize", $self->{'text_hash'}->{'default'}, 5));
}

sub save_CommandBufferSize
{
	my $self = shift;

return $self->parse_opt("CommandBufferSize", '^\d+$', $self->{'text_hash'}->{'mod_core_ebuffer'});
}

sub edit_DefaultServer
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_defaultserver'},
	$self->choice_input($_[0]->{'value'}, "DefaultServer", "off",
		      "$self->{'text_hash'}->{'yes'},on",
		      "$self->{'text_hash'}->{'no'},off"));
}

sub save_DefaultServer
{
	my $self = shift;

return $self->parse_choice("DefaultServer", "off");
}

sub edit_DefaultTransferMode
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_transfer'},
	$self->select_input($_[0]->{'value'}, "DefaultTransferMode", "",
		      "$self->{'text_hash'}->{'mod_core_ascii'},ascii",
		      "$self->{'text_hash'}->{'mod_core_binary'},binary",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_DefaultTransferMode
{
	my $self = shift;

return $self->parse_choice("DefaultTransferMode", "");
}

sub edit_DeferWelcome
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_defer'},
	$self->choice_input($_[0]->{'value'}, "DeferWelcome", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_DeferWelcome
{
	my $self = shift;

return $self->parse_choice("DeferWelcome", "");
}

sub edit_DeleteAbortedStores
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_aborted'},
	$self->choice_input($_[0]->{'value'}, "DeleteAbortedStores", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_DeleteAbortedStores
{
	my $self = shift;

return $self->parse_choice("DeleteAbortedStores", "");
}

sub edit_DenyFilter
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_dfilter'},
	$self->opt_input($_[0]->{'value'}, "DenyFilter", $self->{'text_hash'}->{'default'}, 15));
}

sub save_DenyFilter
{
	my $self = shift;

return $self->parse_opt("DenyFilter");
}

sub edit_DenyGroup
{
	my $self = shift;

local $v = @{$_[0]} ? join(" ", (map { $_->{'value'} } @{$_[0]})) : undef;
return (2, $self->{'text_hash'}->{'mod_core_dgroup'},
	$self->opt_input($v, "DenyGroup", $self->{'text_hash'}->{'mod_core_none'}, 50));
}

sub save_DenyGroup
{
	my $self = shift;

return ( $self->{'in_hash'}->{'DenyGroup_def'} ? [ ] : [ split(/\s+/, $self->{'in_hash'}->{'DenyGroup'}) ] );
}

sub edit_DenyUser
{
	my $self = shift;

local $v = @{$_[0]} ? join(" ", (map { $_->{'value'} } @{$_[0]})) : undef;
return (2, $self->{'text_hash'}->{'mod_core_duser'},
	$self->opt_input($v, "DenyUser", $self->{'text_hash'}->{'mod_core_none'}, 50));
}

sub save_DenyUser
{
	my $self = shift;

return ( $self->{'in_hash'}->{'DenyUser_def'} ? [ ] : [ split(/\s+/, $self->{'in_hash'}->{'DenyUser'}) ] );
}

sub edit_DisplayConnect
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_display'},
	$self->opt_input($_[0]->{'value'}, "DisplayConnect",
		   $self->{'text_hash'}->{'mod_core_none'}, 50));
}

sub save_DisplayConnect
{
	my $self = shift;

return $self->parse_opt("DisplayConnect", '\S', $self->{'text_hash'}->{'mod_core_edisplay'});
}

sub edit_DisplayFirstChdir
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_firstcd'},
	$self->opt_input($_[0]->{'value'}, "DisplayFirstChdir",
		   $self->{'text_hash'}->{'mod_core_none'}, 15));
}

sub save_DisplayFirstChdir
{
	my $self = shift;

return $self->parse_opt("DisplayFirstChdir", '^\S+$', $self->{'text_hash'}->{'mod_core_efirstcd'});
}

sub edit_DisplayGoAway
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_goaway'},
	$self->opt_input($_[0]->{'value'}, "DisplayGoAway",
		   $self->{'text_hash'}->{'mod_core_none'}, 50));
}

sub save_DisplayGoAway
{
	my $self = shift;

return $self->parse_opt("DisplayGoAway", '\S', $self->{'text_hash'}->{'mod_core_egoaway'});
}

sub edit_DisplayLogin
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_login'},
	$self->opt_input($_[0]->{'value'}, "DisplayLogin",
		   $self->{'text_hash'}->{'mod_core_none'}, 50));
}

sub save_DisplayLogin
{
	my $self = shift;

return $self->parse_opt("DisplayLogin", '\S', $self->{'text_hash'}->{'mod_core_elogin'});
}

sub edit_DisplayQuit
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_quit'},
	$self->opt_input($_[0]->{'value'}, "DisplayQuit",
		   $self->{'text_hash'}->{'mod_core_none'}, 50));
}

sub save_DisplayQuit
{
	my $self = shift;

return $self->parse_opt("DisplayQuit", '\S', $self->{'text_hash'}->{'mod_core_equit'});
}

sub edit_Group
{
	my $self = shift;

local($rv, @ginfo);
$rv = sprintf "<input type=radio name=Group value=0 %s> $self->{'text_hash'}->{'default'}\n",
       $_[0] ? "" : "checked";
$rv .= sprintf "<input type=radio name=Group value=1 %s> %s\n",
        $_[0] && $_[0]->{'value'} !~ /^#/ ? "checked" : "",
	$self->{'text_hash'}->{'mod_core_gname'};
$rv .= sprintf "<input name=Group_name size=8 value=\"%s\"> %s\n",
	$_[0]->{'value'} !~ /^#/ ? $_[0]->{'value'} : "",
	$self->group_chooser_button("Group_name", 0);
$rv .= sprintf "<input type=radio name=Group value=2 %s> %s\n",
        $_[0]->{'value'} =~ /^#/ ? "checked" : "",
	$self->{'text_hash'}->{'mod_core_gid'};
$rv .= sprintf "<input name=Group_id size=6 value=\"%s\">\n",
	 $_[0]->{'value'} =~ /^#(.*)$/ ? $1 : "";
return (2, $self->{'text_hash'}->{'mod_core_group'}, $rv);
}

sub save_Group
{
	my $self = shift;

if ($self->{'in_hash'}->{'Group'} == 0) { return ( [ ] ); }
elsif ($self->{'in_hash'}->{'Group'} == 1) { return ( [ $self->{'in_hash'}->{'Group_name'} ] ); }
elsif ($self->{'in_hash'}->{'Group_id'} !~ /^\-?\d+$/) {
	$self->error($self->text('core_euid', $self->{'in_hash'}->{'Group_id'}));
	}
else { return ( [ "#$self->{'in_hash'}->{'Group_id'}" ] ); }
}

sub edit_GroupOwner
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_gowner'},
	$self->opt_input($_[0]->{'value'}, "GroupOwner", $self->{'text_hash'}->{'default'}, 13,
		   $self->group_chooser_button("GroupOwner")));
}

sub save_GroupOwner
{
	my $self = shift;

if ($self->{'in_hash'}->{'GroupOwner_def'}) { return ( [ ] ); }
else {
	defined(getgrnam($self->{'in_hash'}->{'GroupOwner'})) || $self->error($self->{'text_hash'}->{'mod_core_egowner'});
	return ( [ $self->{'in_hash'}->{'GroupOwner'} ] );
	}
}

sub edit_GroupPassword
{
	my $self = shift;

local $rv = "<table border>\n";
$rv .= "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'mod_core_gpname'}</b></td> ".
       "<td><b>$self->{'text_hash'}->{'mod_core_gppass'}</b></td> </tr>\n";
local $i = 0;
foreach $g (@{$_[0]}) {
	local @v = @{$g->{'words'}};
	$rv .= "<tr $self->{'cb'}>\n";
	$rv .= "<td><input name=GroupPassword_n_$i size=13 value='$v[0]'></td>\n";
	$rv .= "<td><input type=radio name=GroupPassword_d_$i value='$v[1]' checked> $self->{'text_hash'}->{'mod_core_gpdef'}\n";
	$rv .= "<input type=radio name=GroupPassword_d_$i value=0>\n";
	$rv .= "<input name=GroupPassword_p_$i size=25></td> </tr>\n";
	$i++;
	}
$rv .= "<tr $self->{'cb'}>\n".
       "<td><input name=GroupPassword_n_$i size=13></td>\n".
       "<td><input name=GroupPassword_p_$i size=35></td>\n".
       "</tr> </table>\n";
return (2, $self->{'text_hash'}->{'mod_core_grouppassword'}, $rv);
}

sub save_GroupPassword
{
	my $self = shift;

local @rv;
for($i=0; defined($self->{'in_hash'}->{"GroupPassword_n_$i"}); $i++) {
	next if (!$self->{'in_hash'}->{"GroupPassword_n_$i"});
	scalar(getgrnam($self->{'in_hash'}->{"GroupPassword_n_$i"})) ||
		$self->error($self->{'text_hash'}->{'mod_core_egpname'});
	if ($self->{'in_hash'}->{"GroupPassword_d_$i"}) {
		push(@rv, $self->{'in_hash'}->{"GroupPassword_n_$i"}.' '.
			  $self->{'in_hash'}->{"GroupPassword_d_$i"});
		}
	else {
		$self->{'salt'} = substr(time(), 0, 2);
		push(@rv, $self->{'in_hash'}->{"GroupPassword_n_$i"}.' '.
			  $self->unix_crypt($self->{'in_hash'}->{"GroupPassword_p_$i"}, $self->{'salt'}));
		}
	}
return ( \@rv );
}

sub edit_HiddenStor
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_hstor'},
	$self->choice_input($_[0]->{'value'}, "HiddenStor", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_HiddenStor
{
	my $self = shift;

return $self->parse_choice("HiddenStor", "");
}

sub edit_HideGroup
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_hgroup'},
	sprintf "<input name=HideGroup size=50 value='%s'>",
	 join(" ", map { $_->{'value'} } @{$_[0]}));
}

sub save_HideGroup
{
	my $self = shift;

local @hg = split(/\s+/, $self->{'in_hash'}->{'HideGroup'});
foreach $g (@hg) {
	scalar(getgrnam($g)) || $self->error($self->{'text_hash'}->{'mod_core_ehgroup'});
	}
return ( \@hg );
}

sub edit_HideNoAccess
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_hnoaccess'},
	$self->choice_input($_[0]->{'value'}, "HideNoAccess", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_HideNoAccess
{
	my $self = shift;

return $self->parse_choice("HideNoAccess", "");
}

sub edit_HideUser
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_huser'},
	sprintf "<input name=HideUser size=50 value='%s'>",
	 join(" ", map { $_->{'value'} } @{$_[0]}));
}

sub save_HideUser
{
	my $self = shift;

local @hu = split(/\s+/, $self->{'in_hash'}->{'HideUser'});
foreach $u (@hu) {
	defined(getpwnam($u)) || $self->error($self->{'text_hash'}->{'mod_core_ehuser'});
	}
return ( \@hu );
}

sub edit_IdentLookups
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_ident'},
	$self->choice_input($_[0]->{'value'}, "IdentLookups", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_IdentLookups
{
	my $self = shift;

return $self->parse_choice("IdentLookups", "");
}

sub edit_IgnoreHidden
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_ihidden'},
	$self->choice_input($_[0]->{'value'}, "IgnoreHidden", "off",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off"));
}

sub save_IgnoreHidden
{
	my $self = shift;

return $self->parse_choice("IgnoreHidden", "off");
}

sub edit_MasqueradeAddress
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_masq'},
	$self->opt_input($_[0]->{'value'}, "MasqueradeAddress",
		   $self->{'text_hash'}->{'mod_core_masq_def'}, 30));
}

sub save_MasqueradeAddress
{
	my $self = shift;

$self->{'in_hash'}->{'MasqueradeAddress_def'} || $self->to_ipaddress($self->{'in_hash'}->{'MasqueradeAddress'}) ||
	$self->error($self->{'text_hash'}->{'mod_core_emasq'});
return $self->parse_opt("MasqueradeAddress");
}

sub edit_MaxClients
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_maxc'}, $self->edit_max($_[0], "MaxClients"));
}

sub save_MaxClients
{
	my $self = shift;

return $self->save_max("MaxClients");
}

sub edit_MaxClientsPerHost
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_maxch'}, $self->edit_max($_[0], "MaxClientsPerHost"));
}

sub save_MaxClientsPerHost
{
	my $self = shift;

return $self->save_max("MaxClientsPerHost");
}

sub edit_MaxClientsPerUser
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_maxcu'}, $self->edit_max($_[0], "MaxClientsPerUser"));
}

sub save_MaxClientsPerUser
{
	my $self = shift;

return $self->save_max("MaxClientsPerUser");
}

sub edit_max
{
	my $self = shift;

local $m = !$_[0] ? 0 :
	   $_[0]->{'words'}->[0] eq 'none' ? 1 : 2;
local $rv = sprintf "<input type=radio name=$_[1]_m value=0 %s> %s\n",
		$m == 0 ? "checked" : "", $self->{'text_hash'}->{'default'};
$rv .= sprintf "<input type=radio name=$_[1]_m value=1 %s> %s\n",
		$m == 1 ? "checked" : "", $self->{'text_hash'}->{'mod_core_maxc1'};
$rv .= sprintf "<input type=radio name=$_[1]_m value=2 %s>\n",
		$m == 2 ? "checked" : "";
$rv .= sprintf "<input name=$_[1] size=6 value='%s'><br>\n",
		$m == 2 ? $_[0]->{'words'}->[0] : "";
$rv .= sprintf "%s <input name=$_[1]_t size=40 value='%s'>\n",
	$self->{'text_hash'}->{'mod_core_maxcmsg'}, $_[0]->{'words'}->[1];
return $rv;
}

sub save_max
{
	my $self = shift;

if ($self->{'in_hash'}->{"$_[0]_m"} == 0) {
	return ( [ ] );
	}
else {
	local $n;
	if ($self->{'in_hash'}->{"$_[0]_m"} == 1) {
		$n = "none";
		}
	else {
		$self->{'in_hash'}->{$_[0]} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'mod_core_emaxc'});
		$n = $self->{'in_hash'}->{$_[0]};
		}
	if ($self->{'in_hash'}->{"$_[0]_t"}) {
		return ( [ "$n \"".$self->{'in_hash'}->{"$_[0]_t"}."\"" ] );
		}
	else {
		return ( [ $n ] );
		}
	}
}

sub edit_MaxInstances
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_instances'},
	$self->opt_input($_[0]->{'value'}, "MaxInstances", $self->{'text_hash'}->{'default'}, 4));
}

sub save_MaxInstances
{
	my $self = shift;

return $self->parse_opt("MaxInstances", '^\d+$', $self->{'text_hash'}->{'mod_core_einstances'});
}

sub edit_MaxLoginAttempts
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_logins'},
	$self->opt_input($_[0]->{'value'}, "MaxLoginAttempts", $self->{'text_hash'}->{'default'}, 4));
}

sub save_MaxLoginAttempts
{
	my $self = shift;

return $self->parse_opt("MaxLoginAttempts", '^\d+$', $self->{'text_hash'}->{'mod_core_elogins'});
}

sub edit_MultilineRFC2228
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_rfc2228'},
	$self->choice_input($_[0]->{'value'}, "MultilineRFC2228", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_MultilineRFC2228
{
	my $self = shift;

return $self->parse_choice("MultilineRFC2228", "");
}

sub edit_PassivePorts
{
	my $self = shift;

local $rv = sprintf "<input type=radio name=PassivePorts_def value=1 %s> %s\n",
		$_[0] ? "" : "checked", $self->{'text_hash'}->{'default'};
$rv .= sprintf "<input type=radio name=PassivePorts_def value=0 %s> %s\n",
		$_[0] ? "checked" : "", $self->{'text_hash'}->{'mod_core_pasvr'};
$rv .= sprintf "<input name=PassivePorts_f size=5 value='%s'> -\n",
		$_[0]->{'words'}->[0];
$rv .= sprintf "<input name=PassivePorts_t size=5 value='%s'>\n",
		$_[0]->{'words'}->[1];
return (1, $self->{'text_hash'}->{'mod_core_pasv'}, $rv);
}

sub save_PassivePorts
{
	my $self = shift;

if ($self->{'in_hash'}->{'PassivePorts_def'}) {
	return ( [ ] );
	}
else {
	$self->{'in_hash'}->{'PassivePorts_f'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'mod_core_epasv'});
	$self->{'in_hash'}->{'PassivePorts_t'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'mod_core_epasv'});
	return ( [ "$self->{'in_hash'}->{'PassivePorts_f'} $self->{'in_hash'}->{'PassivePorts_t'}" ] );
	}
}

sub edit_PathAllowFilter
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_pathallow'},
	$self->opt_input($_[0]->{'words'}->[0], "PathAllowFilter",
		   $self->{'text_hash'}->{'mod_core_any'}, 20));
}

sub save_PathAllowFilter
{
	my $self = shift;

return $self->parse_opt("PathAllowFilter");
}

sub edit_PathDenyFilter
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_pathdeny'},
	$self->opt_input($_[0]->{'words'}->[0], "PathDenyFilter",
		   $self->{'text_hash'}->{'mod_core_none'}, 20));
}

sub save_PathDenyFilter
{
	my $self = shift;

return $self->parse_opt("PathDenyFilter");
}

sub edit_PidFile
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_pidfile'},
	$self->opt_input($_[0]->{'words'}->[0], "PidFile", $self->{'text_hash'}->{'default'}, 50,
		   $self->file_chooser_button("PidFile")));
}

sub save_PidFile
{
	my $self = shift;

return $self->parse_opt("PidFile", '^\/\S+$', $self->{'text_hash'}->{'mod_core_epidfile'});
}

sub edit_Port
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_port'},
	$self->opt_input($_[0]->{'value'}, "Port", $self->{'text_hash'}->{'default'}, 6));
}

sub save_Port
{
	my $self = shift;

return $self->parse_opt("Port", '^\d+$', $self->{'text_hash'}->{'mod_core_eport'});
}

sub edit_RequireValidShell
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_shell'},
	$self->choice_input($_[0]->{'value'}, "RequireValidShell", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_RequireValidShell
{
	my $self = shift;

return $self->parse_choice("RequireValidShell", "");
}

sub edit_RLimitCPU
{
	my $self = shift;

return $self->rlimit_input("RLimitCPU", $self->{'text_hash'}->{'mod_core_cpulimit'}, $_[0]);
}

sub save_RLimitCPU
{
	my $self = shift;

return $self->parse_rlimit("RLimitCPU", $self->{'text_hash'}->{'mod_core_ecpulimit'});
}

sub edit_RLimitMemory
{
	my $self = shift;

return $self->rlimit_input("RLimitMemory", $self->{'text_hash'}->{'mod_core_memlimit'}, $_[0]);
}

sub save_RLimitMemory
{
	my $self = shift;

return $self->parse_rlimit("RLimitMemory", $self->{'text_hash'}->{'mod_core_ememlimit'});
}

sub edit_RLimitOpenFiles
{
	my $self = shift;

return $self->rlimit_input("RLimitOpenFiles", $self->{'text_hash'}->{'mod_core_filelimit'}, $_[0]);
}

sub save_RLimitOpenFiles
{
	my $self = shift;

return $self->parse_rlimit("RLimitOpenFiles", $self->{'text_hash'}->{'mod_core_efilelimit'});
}

# rlimit_input(name, desc, value)
sub rlimit_input
{
	my $self = shift;

local @w = @{$_[2]->{'words'}};
local $rv;
$rv .= sprintf "<b>%s</b> <input type=radio name=%s_smax value=2 %s> %s\n",
		$self->{'text_hash'}->{'mod_core_soft'}, $_[0], $w[0] ? "" : "checked",
		$self->{'text_hash'}->{'default'};
$rv .= sprintf "<input type=radio name=%s_smax value=1 %s> %s\n",
		$_[0], $w[0] eq 'max' ? "checked" : "", $self->{'text_hash'}->{'mod_core_max'};
$rv .= sprintf "<input type=radio name=%s_smax value=0 %s>\n",
		$_[0], !$w[0] || $w[0] eq 'max' ? "" : "checked";
$rv .= sprintf "<input name=%s_soft size=6 value='%s'>\n",
		$_[0], $w[0] eq 'max' ? '' : $w[0];
$rv .= "&nbsp;&nbsp;&nbsp;";

$rv .= sprintf "<b>%s</b> <input type=radio name=%s_hmax value=2 %s> %s\n",
		$self->{'text_hash'}->{'mod_core_hard'}, $_[0], $w[1] ? "" : "checked",
		$self->{'text_hash'}->{'default'};
$rv .= sprintf "<input type=radio name=%s_hmax value=1 %s> %s\n",
		$_[0], $w[1] eq 'max' ? "checked" : "", $self->{'text_hash'}->{'mod_core_max'};
$rv .= sprintf "<input type=radio name=%s_hmax value=0 %s>\n",
		$_[0], !$w[1] || $w[1] eq 'max' ? "" : "checked";
$rv .= sprintf "<input name=%s_hard size=6 value='%s'>\n",
		$_[0], $w[1] eq 'max' ? '' : $w[1];
return (2, $_[1], $rv);
}

# parse_rlimit(name, desc)
sub parse_rlimit
{
	my $self = shift;

if ($self->{'in_hash'}->{"$_[0]_smax"} == 2) {
	return ( [ ] );
	}
local @v;
if ($self->{'in_hash'}->{"$_[0]_smax"} == 1) {
	push(@v, "max");
	}
else {
	$self->{'in_hash'}->{"$_[0]_soft"} =~ /^(\d+)(G|M|K|B)?$/i ||
		$self->error($self->text('mod_core_esoft', $_[1]));
	push(@v, $self->{'in_hash'}->{"$_[0]_soft"});
	}
if ($self->{'in_hash'}->{"$_[0]_hmax"} == 1) {
	push(@v, "max");
	}
elsif ($self->{'in_hash'}->{"$_[0]_hmax"} == 0) {
	$self->{'in_hash'}->{"$_[0]_hard"} =~ /^(\d+)(G|M|K|B)?$/i ||
		$self->error($self->text('mod_core_ehard', $_[1]));
	push(@v, $self->{'in_hash'}->{"$_[0]_hard"});
	}
return ( [ join(" ", @v) ] );
}

sub edit_ScoreboardPath
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_score'},
	$self->opt_input($_[0]->{'words'}->[0], "ScoreboardPath", $self->{'text_hash'}->{'default'},
		   50, $self->file_chooser_button("ScoreboardPath")));
}

sub save_ScoreboardPath
{
	my $self = shift;

return $self->parse_opt("ScoreboardPath", '^\/\S+$', $self->{'text_hash'}->{'mod_core_escore'});
}

sub edit_ServerAdmin
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_admin'},
	$self->opt_input($_[0]->{'words'}->[0], "ServerAdmin", $self->{'text_hash'}->{'default'}, 40));
}

sub save_ServerAdmin
{
	my $self = shift;

return $self->parse_opt("ServerAdmin", '^\S+\@\S+$', $self->{'text_hash'}->{'mod_core_eadmin'});
}

sub edit_ServerIdent
{
	my $self = shift;

local @w = @{$_[0]->{'words'}};
local $rv = sprintf "<input type=radio name=ServerIdent_m value=0 %s> %s\n",
	$_[0] ? "" : "checked", $self->{'text_hash'}->{'default'};
$rv .= sprintf "<input type=radio name=ServerIdent_m value=1 %s> %s\n",
	lc($w[0]) eq 'off' ? "checked" : "", $self->{'text_hash'}->{'mod_core_none'};
$rv .= sprintf "<input type=radio name=ServerIdent_m value=2 %s> %s\n",
	lc($w[0]) eq 'on' && !$w[1] ? "checked" : "",
	$self->{'text_hash'}->{'mod_core_identmsg_def'};
$rv .= sprintf "<input type=radio name=ServerIdent_m value=3 %s>\n",
	lc($w[0]) eq 'on' && $w[1] ? "checked" : "";
$rv .= sprintf "<input name=ServerIdent size=30 value='%s'>\n",
	lc($w[0]) eq 'on' ? $w[1] : "";
return (2, $self->{'text_hash'}->{'mod_core_identmsg'}, $rv);
}

sub save_ServerIdent
{
	my $self = shift;

if ($self->{'in_hash'}->{'ServerIdent_m'} == 0) {
	return ( [ ] );
	}
elsif ($self->{'in_hash'}->{'ServerIdent_m'} == 1) {
	return ( [ "off" ] );
	}
elsif ($self->{'in_hash'}->{'ServerIdent_m'} == 2) {
	return ( [ "on" ] );
	}
else {
	return ( [ "on \"$self->{'in_hash'}->{'ServerIdent'}\"" ] );
	}
}

sub edit_ServerName
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_servername'},
	$self->opt_input($_[0]->{'words'}->[0], "ServerName", $self->{'text_hash'}->{'default'}, 50));
}

sub save_ServerName
{
	my $self = shift;

return $self->parse_opt("ServerName", '\S', $self->{'text_hash'}->{'mod_core_eservername'});
}

sub edit_ServerType
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_type'},
	$self->select_input($_[0]->{'value'}, "ServerType", "",
		      "$self->{'text_hash'}->{'mod_core_inetd'},inetd",
		      "$self->{'text_hash'}->{'mod_core_stand'},standalone",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_ServerType
{
	my $self = shift;

return $self->parse_choice("ServerType", "");
}

sub edit_ShowSymlinks
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_links'},
	$self->choice_input($_[0]->{'value'}, "ShowSymlinks", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_ShowSymlinks
{
	my $self = shift;

return $self->parse_choice("ShowSymlinks", "");
}

sub edit_SocketBindTight
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_tight'},
	$self->choice_input($_[0]->{'value'}, "SocketBindTight", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_SocketBindTight
{
	my $self = shift;

return $self->parse_choice("SocketBindTight", "");
}

sub edit_SyslogFacility
{
	my $self = shift;

local @facils = map { "$_,$_" } ( 'auth', 'authpriv', 'cron', 'daemon', 'kern', 'lpr', 'mail', 'news', 'user', 'uucp', 'local0', 'local1', 'local2', 'local3', 'local4', 'local5', 'local6', 'local7' );
return (1, $self->{'text_hash'}->{'mod_core_facility'},
	$self->select_input($_[0]->{'value'}, "SyslogFacility", "",
		      "$self->{'text_hash'}->{'default'},", @facils));
}

sub save_SyslogFacility
{
	my $self = shift;

return $self->parse_select("SyslogFacility", "");
}

sub edit_SyslogLevel
{
	my $self = shift;

local @levels = map { "$_,$_" } ( 'emerg', 'alert', 'crit', 'error', 'warn', 'notice', 'info', 'debug' );
return (1, $self->{'text_hash'}->{'mod_core_level'},
	$self->select_input($_[0]->{'value'}, "SyslogLevel", "",
		      "$self->{'text_hash'}->{'default'},", @levels));
}

sub save_SyslogLevel
{
	my $self = shift;

return $self->parse_select("SyslogLevel", "");
}

sub edit_TransferLog
{
	my $self = shift;

local $mode = $_[0]->{'value'} eq 'NONE' ? 2 :
	      $_[0]->{'value'} ? 0 : 1;
local $rv = sprintf "<input type=radio name=TransferLog_def value=1 %s> %s\n",
		$mode == 1 ? "checked" : "", $self->{'text_hash'}->{'default'};
if ($_[1]->{'version'} >= 1.17) {
	$rv .= sprintf"<input type=radio name=TransferLog_def value=2 %s> %s\n",
		$mode == 2 ? "checked" : "", $self->{'text_hash'}->{'mod_core_nowhere'};
	}
$rv .= sprintf "<input type=radio name=TransferLog_def value=0 %s>\n",
		$mode == 0 ? "checked" : "";
$rv .= sprintf "<input name=TransferLog size=50 value='%s'>\n",
		$mode == 0 ? $_[0]->{'value'} : "";
return (2, $self->{'text_hash'}->{'mod_core_tlog'}, $rv);
}

sub save_TransferLog
{
	my $self = shift;

if ($self->{'in_hash'}->{'TransferLog_def'} == 1) {
	return ( [ ] );
	}
elsif ($self->{'in_hash'}->{'TransferLog_def'} == 2) {
	return ( [ 'NONE' ] );
	}
else {
	$self->{'in_hash'}->{'TransferLog'} =~ /^\/\S+$/ || $self->error($self->{'text_hash'}->{'mod_core_etlog'}); 
	return ( [ $self->{'in_hash'}->{'TransferLog'} ] );
	}
}

sub edit_tcpBackLog
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_backlog'},
	$self->opt_input($_[0]->{'value'}, "tcpBackLog", $self->{'text_hash'}->{'default'}, 6));
}

sub save_tcpBackLog
{
	my $self = shift;

return $self->parse_opt("tcpBackLog", '^\d+$', $self->{'text_hash'}->{'mod_core_ebacklog'});
}

sub edit_tcpNoDelay
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_nodelay'},
	$self->choice_input($_[0]->{'value'}, "tcpNoDelay", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_tcpNoDelay
{
	my $self = shift;

return $self->parse_choice("tcpNoDelay", "");
}

sub edit_tcpReceiveWindow
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_rwindow'},
	$self->opt_input($_[0]->{'value'}, "tcpReceiveWindow", $self->{'text_hash'}->{'default'}, 6));
}

sub save_tcpReceiveWindow
{
	my $self = shift;

return $self->parse_opt("tcpReceiveWindow", '^\d+$', $self->{'text_hash'}->{'mod_core_erwindow'});
}

sub edit_tcpSendWindow
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_swindow'},
	$self->opt_input($_[0]->{'value'}, "tcpSendWindow", $self->{'text_hash'}->{'default'}, 6));
}

sub save_tcpSendWindow
{
	my $self = shift;

return $self->parse_opt("tcpSendWindow", '^\d+$', $self->{'text_hash'}->{'mod_core_eswindow'});
}

sub edit_TimesGMT
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_gmt'},
	$self->choice_input($_[0]->{'value'}, "TimesGMT", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_TimesGMT
{
	my $self = shift;

return $self->parse_choice("TimesGMT", "");
}

sub edit_TimeoutIdle
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_tidle'},
	$self->opt_input($_[0]->{'value'}, "TimeoutIdle", $self->{'text_hash'}->{'default'}, 6,
		   $self->{'text_hash'}->{'mod_core_secs'}));
}

sub save_TimeoutIdle
{
	my $self = shift;

return $self->parse_opt("TimeoutIdle", '^\d+$', $self->{'text_hash'}->{'mod_core_etidle'});
}

sub edit_TimeoutLogin
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_tlogin'},
	$self->opt_input($_[0]->{'value'}, "TimeoutLogin", $self->{'text_hash'}->{'default'}, 6,
		   $self->{'text_hash'}->{'mod_core_secs'}));
}

sub save_TimeoutLogin
{
	my $self = shift;

return $self->parse_opt("TimeoutLogin", '^\d+$', $self->{'text_hash'}->{'mod_core_etlogin'});
}

sub edit_TimeoutNoTransfer
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_ttransfer'},
	$self->opt_input($_[0]->{'value'}, "TimeoutNoTransfer", $self->{'text_hash'}->{'default'}, 6,
		   $self->{'text_hash'}->{'mod_core_secs'}));
}

sub save_TimeoutNoTransfer
{
	my $self = shift;

return $self->parse_opt("TimeoutNoTransfer", '^\d+$', $self->{'text_hash'}->{'mod_core_ettransfer'});
}

sub edit_TimeoutStalled
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_tstalled'},
	$self->opt_input($_[0]->{'value'}, "TimeoutStalled", $self->{'text_hash'}->{'default'}, 6,
		   $self->{'text_hash'}->{'mod_core_secs'}));
}

sub save_TimeoutStalled
{
	my $self = shift;

return $self->parse_opt("TimeoutStalled", '^\d+$', $self->{'text_hash'}->{'mod_core_etstalled'});
}

sub edit_Umask
{
	my $self = shift;

local $rv;
$rv .= sprintf "<input type=radio name=Umask_def value=1 %s> %s\n",
	$_[0]->{'words'}->[0] ? "" : "checked", $self->{'text_hash'}->{'default'};
$rv .= sprintf "<input type=radio name=Umask_def value=0 %s> %s\n",
	$_[0]->{'words'}->[0] ? "checked" : "", $self->{'text_hash'}->{'mod_core_octal'};
$rv .= sprintf "<input name=Umask size=5 value='%s'>\n",
	$_[0]->{'words'}->[0];

$rv .= "&nbsp;&nbsp;&nbsp;<b>$self->{'text_hash'}->{'mod_core_umask_d'}</b>\n";
$rv .= sprintf "<input type=radio name=Umask_d_def value=1 %s> %s\n",
	$_[0]->{'words'}->[1] ? "" : "checked", $self->{'text_hash'}->{'default'};
$rv .= sprintf "<input type=radio name=Umask_d_def value=0 %s> %s\n",
	$_[0]->{'words'}->[1] ? "checked" : "", $self->{'text_hash'}->{'mod_core_octal'};
$rv .= sprintf "<input name=Umask_d size=5 value='%s'>\n",
	$_[0]->{'words'}->[1];

return (2, $self->{'text_hash'}->{'mod_core_umask'}, $rv);
}

sub save_Umask
{
	my $self = shift;

if ($self->{'in_hash'}->{'Umask_def'}) {
	return ( [ ] );
	}
else {
	$self->{'in_hash'}->{'Umask'} =~ /^[0-7]{3}$/ || $self->error($self->{'text_hash'}->{'mod_core_eumask'});
	if ($self->{'in_hash'}->{'Umask_d_def'}) {
		return ( [ $self->{'in_hash'}->{'Umask'} ] );
		}
	else {
		$self->{'in_hash'}->{'Umask_d'} =~ /^[0-7]{3}$/ || $self->error($self->{'text_hash'}->{'mod_core_eumask'});
		return ( [ $self->{'in_hash'}->{'Umask'}." ".$self->{'in_hash'}->{'Umask_d'} ] );
		}
	}
}

sub edit_UseFtpUsers
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_ftpusers'},
	$self->choice_input($_[0]->{'value'}, "UseFtpUsers", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_UseFtpUsers
{
	my $self = shift;

return $self->parse_choice("UseFtpUsers", "");
}

sub edit_UseHostsAllowFile
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_hostsallow'},
	$self->opt_input($_[0]->{'value'}, "UseHostsAllowFile", $self->{'text_hash'}->{'default'}, 50,
		   $self->file_chooser_button("UseHostsAllowFile")));
}

sub save_UseHostsAllowFile
{
	my $self = shift;

$self->{'in_hash'}->{'UseHostsAllowFile_def'} || -r $self->{'in_hash'}->{'UseHostsAllowFile'} ||
	$self->error($self->{'text_hash'}->{'mod_core_ehostsallow'});
return $self->parse_opt("UseHostsAllowFile");
}

sub edit_UseHostsDenyFile
{
	my $self = shift;

return (2, $self->{'text_hash'}->{'mod_core_hostsdeny'},
	$self->opt_input($_[0]->{'value'}, "UseHostsDenyFile", $self->{'text_hash'}->{'default'}, 50,
		   $self->file_chooser_button("UseHostsDenyFile")));
}

sub save_UseHostsDenyFile
{
	my $self = shift;

$self->{'in_hash'}->{'UseHostsDenyFile_def'} || -r $self->{'in_hash'}->{'UseHostsDenyFile'} ||
	$self->error($self->{'text_hash'}->{'mod_core_ehostsdeny'});
return $self->parse_opt("UseHostsDenyFile");
}

sub edit_UseReverseDNS
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_revdns'},
	$self->choice_input($_[0]->{'value'}, "UseReverseDNS", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_UseReverseDNS
{
	my $self = shift;

return $self->parse_choice("UseReverseDNS", "");
}

sub edit_UserDirRoot
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_userdir'},
	$self->choice_input($_[0]->{'value'}, "UserDirRoot", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'default'},"));
}

sub save_UserDirRoot
{
	my $self = shift;

return $self->parse_choice("UserDirRoot", "");
}

sub edit_User
{
	my $self = shift;

local($rv, @uinfo);
$rv = sprintf "<input type=radio name=User value=0 %s> $self->{'text_hash'}->{'default'}\n",
       $_[0] ? "" : "checked";
$rv .= sprintf "<input type=radio name=User value=1 %s> %s\n",
        $_[0] && $_[0]->{'value'} !~ /^#/ ? "checked" : "",
	$self->{'text_hash'}->{'mod_core_uname'};
$rv .= sprintf "<input name=User_name size=8 value=\"%s\"> %s&nbsp;\n",
	$_[0]->{'value'} !~ /^#/ ? $_[0]->{'value'} : "",
	$self->user_chooser_button("User_name", 0);
$rv .= sprintf "<input type=radio name=User value=2 %s> %s\n",
        $_[0]->{'value'} =~ /^#/ ? "checked" : "",
	$self->{'text_hash'}->{'mod_core_uid'};
$rv .= sprintf "<input name=User_id size=6 value=\"%s\">\n",
	 $_[0]->{'value'} =~ /^#(.*)$/ ? $1 : "";
return (2, $self->{'text_hash'}->{'mod_core_user'}, $rv);
}

sub save_User
{
	my $self = shift;

if ($self->{'in_hash'}->{'User'} == 0) { return ( [ ] ); }
elsif ($self->{'in_hash'}->{'User'} == 1) { return ( [ $self->{'in_hash'}->{'User_name'} ] ); }
elsif ($self->{'in_hash'}->{'User_id'} !~ /^\-?\d+$/) {
	$self->error($self->text('core_egid', $self->{'in_hash'}->{'User_id'}));
	}
else { return ( [ "#$self->{'in_hash'}->{'User_id'}" ] ); }
}

sub edit_UserAlias
{
	my $self = shift;

local $rv = "<table border>\n".
	    "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'mod_core_afrom'}</b></td> ".
	    "<td><b>$self->{'text_hash'}->{'mod_core_ato'}</b></td> </tr>\n";
local $i = 0;
foreach $u (@{$_[0]}, { }) {
	local @w = @{$u->{'words'}};
	$rv .= "<tr $self->{'cb'}>\n";
	$rv .= "<td><input name=UserAlias_f_$i size=15 value='$w[0]'></td>\n";
	$rv .= "<td><input name=UserAlias_t_$i size=15 value='$w[1]'></td>\n";
	$rv .= "</tr>\n";
	$i++;
	}
$rv .= "</table>\n";
return (2, $self->{'text_hash'}->{'mod_core_ualias'}, $rv);
}

sub save_UserAlias
{
	my $self = shift;

local @rv;
for($i=0; defined($self->{'in_hash'}->{"UserAlias_f_$i"}); $i++) {
	next if (!$self->{'in_hash'}->{"UserAlias_f_$i"});
	$self->{'in_hash'}->{"UserAlias_f_$i"} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'mod_core_eafrom'});
	$self->{'in_hash'}->{"UserAlias_t_$i"} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'mod_core_eato'});
	push(@rv, $self->{'in_hash'}->{"UserAlias_f_$i"}.' '.$self->{'in_hash'}->{"UserAlias_t_$i"});
	}
return ( \@rv );
}

sub edit_UserOwner
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_uowner'},
	$self->opt_input($_[0]->{'value'}, "UserOwner", $self->{'text_hash'}->{'default'}, 13,
		   $self->user_chooser_button("UserOwner")));
}

sub save_UserOwner
{
	my $self = shift;

if ($self->{'in_hash'}->{'UserOwner_def'}) { return ( [ ] ); }
else {
	getpwnam($self->{'in_hash'}->{'UserOwner'}) || $self->error($self->{'text_hash'}->{'mod_core_euowner'});
	return ( [ $self->{'in_hash'}->{'UserOwner'} ] );
	}
}

sub edit_UserPassword
{
	my $self = shift;

local $rv = "<table border>\n";
$rv .= "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'mod_core_upname'}</b></td> ".
       "<td><b>$self->{'text_hash'}->{'mod_core_uppass'}</b></td> </tr>\n";
local $i = 0;
foreach $u (@{$_[0]}) {
	local @v = @{$u->{'words'}};
	$rv .= "<tr $self->{'cb'}>\n";
	$rv .= "<td><input name=UserPassword_n_$i size=13 value='$v[0]'></td>\n";
	$rv .= "<td><input type=radio name=UserPassword_d_$i value='$v[1]' checked> $self->{'text_hash'}->{'mod_core_updef'}\n";
	$rv .= "<input type=radio name=UserPassword_d_$i value=0>\n";
	$rv .= "<input name=UserPassword_p_$i size=25></td> </tr>\n";
	$i++;
	}
$rv .= "<tr $self->{'cb'}>\n".
       "<td><input name=UserPassword_n_$i size=13></td>\n".
       "<td><input name=UserPassword_p_$i size=35></td>\n".
       "</tr> </table>\n";
return (2, $self->{'text_hash'}->{'mod_core_userpassword'}, $rv);
}

sub save_UserPassword
{
	my $self = shift;

local @rv;
for($i=0; defined($self->{'in_hash'}->{"UserPassword_n_$i"}); $i++) {
	next if (!$self->{'in_hash'}->{"UserPassword_n_$i"});
	scalar(getpwnam($self->{'in_hash'}->{"UserPassword_n_$i"})) ||
		$self->error($self->{'text_hash'}->{'mod_core_eupname'});
	if ($self->{'in_hash'}->{"UserPassword_d_$i"}) {
		push(@rv, $self->{'in_hash'}->{"UserPassword_n_$i"}.' '.
			  $self->{'in_hash'}->{"UserPassword_d_$i"});
		}
	else {
		$self->{'salt'} = substr(time(), 0, 2);
		push(@rv, $self->{'in_hash'}->{"UserPassword_n_$i"}.' '.
			  $self->unix_crypt($self->{'in_hash'}->{"UserPassword_p_$i"}, $self->{'salt'}));
		}
	}
return ( \@rv );
}

sub edit_WtmpLog
{
	my $self = shift;

return (1, $self->{'text_hash'}->{'mod_core_wtmp'},
	$self->choice_input($_[0]->{'value'}, "WtmpLog", "",
		      "$self->{'text_hash'}->{'yes'},on", "$self->{'text_hash'}->{'no'},off",
		      "$self->{'text_hash'}->{'mod_core_none'},NONE", "$self->{'text_hash'}->{'default'},"));
}

sub save_WtmpLog
{
	my $self = shift;

return $self->parse_choice("WtmpLog", "");
}


1;

