#!/usr/bin/perl
package proftpd::ProftpdLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'proftpd::ProftpdLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# proftpd-lib.pl
# Common functions for the proftpd server config file

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

# Load the site-specific information on the server executable
$self->read_file("$self->{'module_config_directory'}/site", \%{$self->{'site_hash'}});
@ftpaccess_files = split(/\s+/, $self->{'site_hash'}->{'ftpaccess'});
opendir(DIR, ".");
foreach $f (readdir(DIR)) {
	if ($f =~ /^(mod_\S+)\.pl$/) {
		push(@{$self->{'module_files_array'}}, $1);
		{
    my $pkg = $self->lib_to_pkg($f, "proftpd");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
		}
	}
closedir(DIR);

1;

1;
		return $self;
}

# get_config()
# Returns the entire proftpd config structure
sub get_config
{
	my $self = shift;

if (@{$self->{'get_config_cache_array'}}) {
	return \@{$self->{'get_config_cache_array'}};
	}
@{$self->{'get_config_cache_array'}} = $self->get_config_file($self->{'config_hash'}->{'proftpd_conf'});
return \@{$self->{'get_config_cache_array'}};
}

# $self->get_config_file(filename)
sub get_config_file
{
	my $self = shift;

local @rv;
local $fn = $_[0];
if ($fn !~ /^\//) {
	$self->{'config_hash'}->{'proftpd_conf'} =~ /^(.*)\//;
	$fn = "$1/$fn";
	}
if (opendir(DIR, $fn)) {
	# Is a directory .. parse all files!
	local @files = readdir(DIR);
	closedir(DIR);
	foreach $f (@files) {
		next if ($f =~ /^\./);
		push(@rv, $self->get_config_file("$fn/$f"));
		}
	}
else {
	# Just a normal config file
	local $lnum = 0;
	if (open(CONF, $fn)) {
		@rv = $self->parse_config_file(CONF, $lnum, $fn);
		close(CONF);
		foreach $inc ($self->find_directive("Include", \@rv)) {
			push(@rv, $self->get_config_file($inc));
			}
		}
	}
return @rv;
}

# $self->parse_config_file(handle, lines, file)
# Parses lines of text from some config file into a data structure. The
# return value is an array of references, one for each directive in the file.
# Each reference points to an associative array containing
#  line -	The line number this directive is at
#  eline -	The line number this directive ends at
#  file -	The file this directive is from
#  type -	0 for a normal directive, 1 for a container directive
#  name -	The name of this directive
#  value -	Value (possibly with spaces)
#  members -	For type 1, a reference to the array of members
sub parse_config_file
{
	my $self = shift;

local($fh, @rv, $line, %dummy);
$fh = $_[0];
$dummy{'line'} = $dummy{'eline'} = $_[1]-1;
$dummy{'file'} = $_[2];
$dummy{'type'} = 0;
$dummy{'name'} = "dummy";
@rv = (\%dummy);
while($line = <$fh>) {
	chop;
	$line =~ s/^\s*#.*$//g;
	if ($line =~ /^\s*<\/(\S+)\s*(.*)>/) {
		# end of a container directive. This can only happen in a
		# recursive call to this function
		$_[1]++;
		last;
		}
	elsif ($line =~ /^\s*<IfModule\s+(\!?)(\S+)\.c>/i) {
		# start of an IfModule block. Read it, and if the module
		# exists put the directives in this section.
		local ($not, $mod) = ($1, $2);
		local $oldline = $_[1];
		$_[1]++;
		local @dirs = $self->parse_config_file($fh, $_[1], $_[2]);
		if (!$not && $self->{'httpd_modules_hash'}->{$mod} ||
		    $not && !$self->{'httpd_modules_hash'}->{$mod}) {
			# use the directives..
			push(@rv, { 'line', $oldline,
				    'eline', $oldline,
				    'file', $_[2],
				    'name', "<IfModule $not$mod>" });
			push(@rv, @dirs);
			push(@rv, { 'line', $_[1]-1,
				    'eline', $_[1]-1,
				    'file', $_[2],
				    'name', "</IfModule>" });
			}
		}
	elsif ($line =~ /^\s*<IfDefine\s+(\!?)(\S+)>/i) {
		# start of an IfDefine block. Read it, and if the define
		# exists put the directives in this section
		local ($not, $def) = ($1, $2);
		local $oldline = $_[1];
		$_[1]++;
		local @dirs = $self->parse_config_file($fh, $_[1], $_[2]);
		if (!$not && $self->{'httpd_defines_hash'}->{$def} ||
		    $not && !$self->{'httpd_defines_hash'}->{$def}) {
			# use the directives..
			push(@rv, { 'line', $oldline,
				    'eline', $oldline,
				    'file', $_[2],
				    'name', "<IfDefine $not$def>" });
			push(@rv, @dirs);
			push(@rv, { 'line', $_[1]-1,
				    'eline', $_[1]-1,
				    'file', $_[2],
				    'name', "</IfDefine>" });
			}
		}
	elsif ($line =~ /^\s*<(\S+)\s*(.*)>/) {
		# start of a container directive. The first member is a dummy
		# directive at the same line as the container
		local(%dir, @members);
		%dir = ('line', $_[1],
			'file', $_[2],
			'type', 1,
			'name', $1,
			'value', $2);
		$dir{'value'} =~ s/\s+$//g;
		$dir{'words'} = $self->wsplit($dir{'value'});
		$_[1]++;
		@members = $self->parse_config_file($fh, $_[1], $_[2]);
		$dir{'members'} = \@members;
		$dir{'eline'} = $_[1]-1;
		push(@rv, \%dir);
		}
	elsif ($line =~ /^\s*(\S+)\s*(.*)$/) {
		# normal directive
		local(%dir);
		%dir = ('line', $_[1],
			'eline', $_[1],
			'file', $_[2],
			'type', 0,
			'name', $1,
			'value', $2);
		if ($dir{'value'} =~ s/\\$//g) {
			# multi-line directive!
			while($line = <$fh>) {
				chop($line);
				$self->{'cont'} = ($line =~ s/\\$//g);
				$dir{'value'} .= $line;
				$dir{'eline'} = ++$_[1];
				if (!$self->{'cont'}) { last; }
				}
			}
		$dir{'value'} =~ s/\s+$//g;
		$dir{'words'} = $self->wsplit($dir{'value'});
		push(@rv, \%dir);
		$_[1]++;
		}
	else {
		# blank or comment line
		$_[1]++;
		}
	}
return @rv;
}

# wsplit(string)
# Splits a string like  foo "foo \"bar\"" bazzz  into an array of words
sub wsplit
{
	my $self = shift;

local($s, @rv); $s = $_[0];
$s =~ s/\\\"/\0/g;
while($s =~ /^"([^"]*)"\s*(.*)$/ || $s =~ /^(\S+)\s*(.*)$/) {
	$self->{'w'} = $1; $s = $2;
	$self->{'w'} =~ s/\0/"/g; push(@rv, $self->{'w'});
	}
return \@rv;
}

# wjoin(word, word, ...)
sub wjoin
{
	my $self = shift;

local(@rv, $w);
foreach $w (@_) {
	if ($w =~ /^\S+$/) { push(@rv, $w); }
	else { push(@rv, "\"$w\""); }
	}
return join(' ', @rv);
}

# find_directive(name, &directives)
# Returns the values of directives matching some name
sub find_directive
{
	my $self = shift;

local(@rv, $i, @vals, $dref);
foreach $ref (@{$_[1]}) {
	if (lc($ref->{'name'}) eq lc($_[0])) {
		push(@vals, $ref->{'words'}->[0]);
		}
	}
return wantarray ? @vals : !@vals ? undef : $vals[$#vals];
}

# find_directive_struct(name, &directives)
# Returns references to directives maching some name
sub find_directive_struct
{
	my $self = shift;

local(@rv, $i, @vals);
foreach $ref (@{$_[1]}) {
	if (lc($ref->{'name'}) eq lc($_[0])) {
		push(@vals, $ref);
		}
	}
return wantarray ? @vals : !@vals ? undef : $vals[$#vals];
}

# find_vdirective(name, &virtualdirectives, &directives)
# Looks for some directive in a <VirtualHost> section, and then in the 
# main section
sub find_vdirective
{
	my $self = shift;

if ($_[1]) {
	$self->{'rv'} = $self->find_directive($_[0], $_[1]);
	if ($self->{'rv'}) { return $self->{'rv'}; }
	}
return $self->find_directive($_[0], $_[2]);
}

# make_directives(ref, version, module)
sub make_directives
{
	my $self = shift;

local @rv;
local $ver = $_[1];
if ($ver =~ /^(1)\.(2)(\d+)$/) {
	$ver = sprintf "%d.%d%2.2d", $1, $2, $3;
	}
foreach $d (@{$_[0]}) {
	local(%dir);
	$dir{'name'} = $d->[0];
	$dir{'multiple'} = $d->[1];
	$dir{'type'} = $d->[2];
	$dir{'module'} = $_[2];
	$dir{'version'} = $_[1];
	$dir{'priority'} = $d->[5];
	foreach $c (split(/\s+/, $d->[3])) { $dir{$c}++; }
	if (!$d->[4]) { push(@rv, \%dir); }
	elsif ($d->[4] =~ /^-([\d\.]+)$/ && $ver < $1) { push(@rv, \%dir); }
	elsif ($d->[4] =~ /^([\d\.]+)$/ && $ver >= $1) { push(@rv, \%dir); }
	elsif ($d->[4] =~ /^([\d\.]+)-([\d\.]+)$/ && $ver >= $1 && $ver < $2)
		{ push(@rv, \%dir); }
	}
return @rv;
}

# editable_directives(type, context)
# Returns an array of references to associative arrays, one for each 
# directive of the given type that can be used in the given context
sub editable_directives
{
	my $self = shift;

local($m, $func, @rv);
local @mods = split(/\s+/, $self->{'site_hash'}->{'modules'});
foreach $m (@{$self->{'module_files_array'}}) {
	if ($self->indexof($m, @mods) != -1) {
		$func = $m."_directives";
		push(@rv, $self->$func($self->{'site_hash'}->{'version'}));
		}
	}
@rv = grep { $_->{'type'} == $_[0] && $_->{$_[1]} } @rv;
@rv = sort { $pd = $b->{'priority'} - $a->{'priority'};
	     $md = $a->{'module'} cmp $b->{'module'};
	     $pd == 0 ? ($md == 0 ? $a->{'name'} cmp $b->{'name'} : $md) : $pd }
		@rv;
return @rv;
}

# generate_inputs(&editors, &directives)
# Displays a 2-column list of options, for use inside a table
sub generate_inputs
{
	my $self = shift;

local($e, $sw, @args, @rv, $func);
foreach $e (@{$_[0]}) {
	if (!$sw) { print "<tr>\n"; }

	# Build arg list for the editing function. Each arg can be a single
	# directive struct, or a reference to an array of structures.
	$func = "edit";
	undef(@args);
	foreach $ed (split(/\s+/, $e->{'name'})) {
		local(@vals);
		$func .= "_$ed";
		@vals = $self->find_directive_struct($ed, $_[1]);
		if ($e->{'multiple'}) { push(@args, \@vals); }
		elsif (!@vals) { push(@args, undef); }
		else { push(@args, $vals[$#vals]); }
		}
	push(@args, $e);

	# call the function
	@rv = $self->$func(@args);
	if ($rv[0] == 2) {
		# spans 2 columns..
		if ($sw) {
			# need to end this row
			print "<td colspan=2></td> </tr><tr>\n";
			}
		else { $sw = !$sw; }
		print "<td valign=top align=right width=25%><b>$rv[1]</b></td>\n";
		print "<td nowrap valign=top colspan=3 width=75%>$rv[2]</td>\n";
		}
	else {
		# only spans one column
		print "<td valign=top align=right width=25%><b>$rv[1]</b></td>\n";
		print "<td nowrap valign=top width=25%>$rv[2]</td>\n";
		}

	if ($sw) { print "</tr>\n"; }
	$sw = !$sw;
	}
}

# parse_inputs(&editors, &directives, &config)
# Reads user choices from a form and update the directives and config files.
sub parse_inputs
{
	my $self = shift;

# First call editor functions to get new values. Each function returns
# an array of references to arrays containing the new values for the directive.
local ($i, @chname, @chval);
$self->before_changing();
foreach $e (@{$_[0]}) {
	local @dirs = split(/\s+/, $e->{'name'});
	local $func = "save_".join('_', @dirs);
	local @rv = $self->$func($e);
	for($i=0; $i<@dirs; $i++) {
		push(@chname, $dirs[$i]);
		push(@chval, $rv[$i]);
		}
	}

# Assuming everything went OK, update the configuration
for($i=0; $i<@chname; $i++) {
	$self->save_directive($chname[$i], $chval[$i], $_[1], $_[2]);
	}
$self->flush_file_lines();
$self->after_changing();
}

# opt_input(value, name, default, size, [units])
sub opt_input
{
	my $self = shift;

return sprintf "<input type=radio name=$_[1]_def value=1 %s> $_[2]\n".
	       "<input type=radio name=$_[1]_def value=0 %s>\n".
	       "<input name=$_[1] size=$_[3] value='%s'> %s\n",
	defined($_[0]) ? "" : "checked",
	defined($_[0]) ? "checked" : "",
	$_[0], $_[4];
}

# parse_opt(name, regexp, error)
sub parse_opt
{
	my $self = shift;

local($i, $re);
if ($self->{'in_hash'}->{"$_[0]_def"}) { return ( [ ] ); }
for($i=1; $i<@_; $i+=2) {
	$re = $_[$i];
	if ($self->{'in_hash'}->{$_[0]} !~ /$re/) { $self->error($_[$i+1]); }
	}
return ( [ $self->{'in_hash'}->{$_[0]} =~ /^\S+$/ ? $self->{'in_hash'}->{$_[0]} : '"'.$self->{'in_hash'}->{$_[0]}.'"' ] );
}

# choice_input(value, name, default, [choice]+)
# Each choice is a display,value pair
sub choice_input
{
	my $self = shift;

local($i, $rv);
for($i=3; $i<@_; $i++) {
	$_[$i] =~ /^([^,]*),(.*)$/;
	$rv .= sprintf "<input type=radio name=$_[1] value=\"$2\" %s> $1\n",
		lc($2) eq lc($_[0]) ||
		lc($2) eq 'on' && lc($_[0]) eq 'yes' ||
		lc($2) eq 'off' && lc($_[0]) eq 'no' ||
		!defined($_[0]) && lc($2) eq lc($_[2]) ? "checked" : "";
	}
return $rv;
}

# choice_input_vert(value, name, default, [choice]+)
# Each choice is a display,value pair
sub choice_input_vert
{
	my $self = shift;

local($i, $rv);
for($i=3; $i<@_; $i++) {
	$_[$i] =~ /^([^,]*),(.*)$/;
	$rv .= sprintf "<input type=radio name=$_[1] value=\"$2\" %s> $1<br>\n",
		lc($2) eq lc($_[0]) || !defined($_[0]) &&
				       lc($2) eq lc($_[2]) ? "checked" : "";
	}
return $rv;
}

# parse_choice(name, default)
sub parse_choice
{
	my $self = shift;

if (lc($self->{'in_hash'}->{$_[0]}) eq lc($_[1])) { return ( [ ] ); }
else { return ( [ $self->{'in_hash'}->{$_[0]} ] ); }
}

# select_input(value, name, default, [choice]+)
sub select_input
{
	my $self = shift;

local($i, $rv);
$rv = "<select name=\"$_[1]\">\n";
for($i=3; $i<@_; $i++) {
	$_[$i] =~ /^([^,]*),(.*)$/;
	$rv .= sprintf "<option value=\"$2\" %s> $1\n",
		lc($2) eq lc($_[0]) || !defined($_[0]) && lc($2) eq lc($_[2]) ? "selected" : "";
	}
$rv .= "</select>\n";
return $rv;
}

# $self->parse_choice(name, default)
sub parse_select
{
	my $self = shift;

return $self->parse_choice(@_);
}

# config_icons(contexts, program)
# Displays up to 17 icons, one for each type of configuration directive, for
# some context (global, virtual, directory or htaccess)
sub config_icons
{
	my $self = shift;

local($m, $func, $e, %etype, $i, $c);
local @mods = split(/\s+/, $self->{'site_hash'}->{'modules'});
local @ctx = split(/\s+/, $_[0]);
foreach $m (sort { $a cmp $b } (@{$self->{'module_files_array'}})) {
	if ($self->indexof($m, @mods) != -1) {
		$func = $m."_directives";
		foreach $e ($self->$func($self->{'site_hash'}->{'version'})) {
			foreach $c (@ctx) {
				$etype{$e->{'type'}}++ if ($e->{$c});
				}
			}
		}
        }
local (@titles, @links, @icons);
for($i=0; $self->{'text_hash'}->{"type_$i"}; $i++) {
	if ($etype{$i}) {
		push(@links, $_[1]."type=$i");
		push(@titles, $self->{'text_hash'}->{"type_$i"});
		push(@icons, "images/type_icon_$i.gif");
		}
	}
for($i=2; $i<@_; $i++) {
	push(@links, $_[$i]->{'link'});
	push(@titles, $_[$i]->{'name'});
	push(@icons, $_[$i]->{'icon'});
	}
$self->icons_table(\@links, \@titles, \@icons, 5);
print "<p>\n";
}

sub lock_proftpd_files
{
	my $self = shift;

local $conf = $self->get_config();
foreach $f ($self->unique(map { $_->{'file'} } @$conf)) {
	$self->lock_file($f);
	}
}

sub unlock_proftpd_files
{
	my $self = shift;

local $conf = $self->get_config();
foreach $f ($self->unique(map { $_->{'file'} } @$conf)) {
	$self->unlock_file($f);
	}
}

# save_directive(name, &values, &directives, &config)
# Updates the config file(s) and the directives structure with new values
# for the given directives.
# If a directive's value is merely being changed, then its value only needs
# to be updated in the directives array and in the file.
sub save_directive
{
	my $self = shift;

local($i, @old, $lref, $change, $len, $v);
@old = $self->find_directive_struct($_[0], $_[2]);
for($i=0; $i<@old || $i<@{$_[1]}; $i++) {
	$v = ${$_[1]}[$i];
	if ($i >= @old) {
		# a new directive is being added. If other directives of this
		# type exist, add it after them. Otherwise, put it at the end of
		# the first file in the section
		if ($change) {
			# Have changed some old directive.. add this new one
			# after it, and update change
			local(%v, $j);
			%v = (	"line", $change->{'line'}+1,
				"eline", $change->{'line'}+1,
				"file", $change->{'file'},
				"type", 0,
				"name", $_[0],
				"value", $v);
			$j = $self->indexof($change, @{$_[2]})+1;
			$self->renumber($_[3], $v{'line'}, $v{'file'}, 1);
			splice(@{$_[2]}, $j, 0, \%v);
			$lref = $self->read_file_lines($v{'file'});
			splice(@$lref, $v{'line'}, 0, "$_[0] $v");
			$change = \%v;
			}
		else {
			# Adding a new directive to the end of the list
			# in this section
			local($f, %v, $j, $l);
			$f = $_[2]->[0]->{'file'};
			for($j=0; $_[2]->[$j]->{'file'} eq $f; $j++) { }
			$l = $_[2]->[$j-1]->{'eline'}+1;
			%v = (	"line", $l,
				"eline", $l,
				"file", $f,
				"type", 0,
				"name", $_[0],
				"value", $v);
			$self->renumber($_[3], $l, $f, 1);
			splice(@{$_[2]}, $j, 0, \%v);
			$lref = $self->read_file_lines($f);
			splice(@$lref, $l, 0, "$_[0] $v");
			}
		}
	elsif ($i >= @{$_[1]}) {
		# a directive was deleted
		$lref = $self->read_file_lines($old[$i]->{'file'});
		$self->{'idx'} = $self->indexof($old[$i], @{$_[2]});
		splice(@{$_[2]}, $self->{'idx'}, 1);
		$len = $old[$i]->{'eline'} - $old[$i]->{'line'} + 1;
		splice(@$lref, $old[$i]->{'line'}, $len);
		$self->renumber($_[3], $old[$i]->{'line'}, $old[$i]->{'file'}, -$len);
		}
	else {
		# just changing the value
		$lref = $self->read_file_lines($old[$i]->{'file'});
		$len = $old[$i]->{'eline'} - $old[$i]->{'line'} + 1;
		$self->renumber($_[3], $old[$i]->{'eline'}+1,
			  $old[$i]->{'file'},1-$len);
		$old[$i]->{'value'} = $v;
		$old[$i]->{'eline'} = $old[$i]->{'line'};
		splice(@$lref, $old[$i]->{'line'}, $len, "$_[0] $v");
		$change = $old[$i];
		}
	}
}

# $self->renumber(&config, line, file, offset)
# Recursively changes the line number of all directives from some file 
# beyond the given line.
sub renumber
{
	my $self = shift;

local($d);
if (!$_[3]) { return; }
foreach $d (@{$_[0]}) {
	if ($d->{'file'} eq $_[2] && $d->{'line'} >= $_[1]) {
		$d->{'line'} += $_[3];
		}
	if ($d->{'file'} eq $_[2] && $d->{'eline'} >= $_[1]) {
		$d->{'eline'} += $_[3];
		}
	if ($d->{'type'}) {
		$self->renumber($d->{'members'}, $_[1], $_[2], $_[3]);
		}
	}
}

sub def
{
	my $self = shift;

return $_[0] ? $_[0] : $_[1];
}

# get_virtual_config(index)
sub get_virtual_config
{
	my $self = shift;

local($conf, $c, $v);
$conf = $self->get_config();
if (!$_[0]) { $c = $conf; $v = undef; }
else {
	$c = $conf->[$_[0]]->{'members'};
	$v = $conf->[$_[0]];
	}
return wantarray ? ($c, $v) : $c;
}

# get_ftpaccess_config(file)
sub get_ftpaccess_config
{
	my $self = shift;

local($lnum, @conf);
open(FTPACCESS, $_[0]);
@conf = $self->parse_config_file(FTPACCESS, $lnum, $_[0]);
close(FTPACCESS);
return \@conf;
}

# get_or_create_global(&config)
# Returns an array ref of members of the <Global> section, creating if necessary
sub get_or_create_global
{
	my $self = shift;

local $global = $self->find_directive_struct("Global", $self->{'conf'});
if ($global) {
	# Already exists .. just return member list
	return $global->{'members'};
	}
else {
	# Need to add it!
	local $lref = $self->read_file_lines($self->{'config_hash'}->{'proftpd_conf'});
	local $olen = @$lref;
	push(@$lref, "<Global>", "</Global>");
	$self->flush_file_lines();
	$global = { 'name' => 'Global',
		    'members' => [ { 'line' => $olen,
				     'eline' => $olen,
				     'file' => $self->{'config_hash'}->{'proftpd_conf'},
				     'type' => 0,
				     'name' => 'dummy' } ],
		    'line' => $olen,
		    'eline' => $olen+1,
		    'file' => $self->{'config_hash'}->{'proftpd_conf'},
		    'type' => 1,
		    'value' => undef,
		    'words' => [ ] };
	push(@{$_[0]}, $global);
	return $global->{'members'};
	}
}

# test_config()
# If possible, test the current configuration and return an error message,
# or undef.
sub test_config
{
	my $self = shift;

if ($self->{'site_hash'}->{'version'} >= 1.2) {
	# Test the configuration with -t flag
	local $cmd = "$self->{'config_hash'}->{'proftpd_path'} -t -c $self->{'config_hash'}->{'proftpd_conf'}";
	local $out = `$cmd 2>&1 </dev/null`;
	return $out if ($?);
	}
return undef;
}

# before_changing()
# If testing all changes, backup the config files so they can be reverted
# if necessary.
sub before_changing
{
	my $self = shift;

if ($self->{'config_hash'}->{'test_always'}) {
	local $conf = $self->get_config();
	local @files = $self->unique(map { $_->{'file'} } @$conf);
	local $/ = undef;
	foreach $f (@files) {
		if (open(BEFORE, $f)) {
			$self->{'before_changing_hash'}->{$f} = <BEFORE>;
			close(BEFORE);
			}
		}
	}
}

# after_changing()
# If testing all changes, test now and revert the configs and show an error
# message if a problem was found.
sub after_changing
{
	my $self = shift;

if ($self->{'config_hash'}->{'test_always'}) {
	local $err = $self->test_config();
	if ($err) {
		# Something failed .. revert all files
		local $f;
		foreach $f (keys %{$self->{'before_changing_hash'}}) {
			$self->open_tempfile(AFTER, ">$f");
			$self->print_tempfie(AFTER, $self->{'before_changing_hash'}->{$f});
			$self->close_tempfile(AFTER);
			}
		$self->error($self->text('eafter', "<pre>$err</pre>"));
		}
	}
}

# restart_button()
# Returns HTML for a link to put in the top-right corner of every page
sub restart_button
{
	my $self = shift;

local $conf = $self->get_config();
local $st = $self->find_directive("ServerType", $conf);
return undef if (lc($st) eq "inetd");
local $rv;
local $pidfile = $self->{'config_hash'}->{'pid_file'};
$self->{'args'} = "redir=".$self->urlize($self->this_url());
if (open(PID, $pidfile) && <PID> =~ /(\d+)/ && kill(0, $1)) {
	$rv .= "<a href=\"apply.cgi?$self->{'args'}&pid=$1\">$self->{'text_hash'}->{'proftpd_apply'}</a><br>\n";
	$rv .= "<a href=\"stop.cgi?$self->{'args'}&pid=$1\">$self->{'text_hash'}->{'proftpd_stop'}</a>\n";
	}
else {
	$rv = "<a href=\"start.cgi?$self->{'args'}\">$self->{'text_hash'}->{'proftpd_start'}</a><br>\n";
	}
close(PID);
return $rv;
}

# this_url()
# Returns the URL in the apache directory of the current script
sub this_url
{
	my $self = shift;

local($url);
$url = $ENV{'SCRIPT_NAME'};
if ($ENV{'QUERY_STRING'} ne "") { $url .= "?$ENV{'QUERY_STRING'}"; }
return $url;
}

# running_under_inetd()
# Returns the inetd/xinetd object and program if ProFTPd is running under one
sub running_under_inetd
{
	my $self = shift;

local ($inet, $inet_mod);
if ($self->foreign_check('inetd')) {
        # Check if proftpd is in inetd
        $self->foreign_require('inetd', 'inetd-lib.pl');
	local $i;
        foreach $i ($self->foreign_call('inetd', 'list_inets')) {
                if ($i->[1] && $i->[3] eq 'ftp') {
                        $inet = $i;
                        last;
                        }
                }
        $inet_mod = 'inetd';
        }
elsif ($self->foreign_check('xinetd')) {
        # Check if proftpd is in xinetd
        $self->foreign_require('xinetd', 'xinetd-lib.pl');
	local $xi;
        foreach $xi ($self->foreign_call("xinetd", "get_xinetd_config")) {
                if ($xi->{'quick'}->{'disable'}->[0] ne 'yes' &&
                    $xi->{'value'} eq 'ftp') {
                        $inet = $xi;
                        last;
                        }
                }
        $inet_mod = 'xinetd';
        }
else {
        # Not supported on this OS .. assume so
        $inet = 1;
	}
return ($inet, $inet_mod);
}

# get_proftpd_pid()
sub get_proftpd_pid
{
	my $self = shift;

if ($self->{'config_hash'}->{'pid_file'}) {
	return $self->check_pid_file($self->{'config_hash'}->{'pid_file'});
	}
else {
	local ($pid) = $self->find_byname("proftpd");
	return $pid;
	}
}

sub get_proftpd_version
{
	my $self = shift;

local $out = `$self->{'config_hash'}->{'proftpd_path'} -v 2>&1`;
${$_[0]} = $out if ($_[0]);
if ($out =~ /ProFTPD\s+Version\s+(\d+)\.([0-9\.]+)/i ||
    $out =~ /ProFTPD\s+(\d+)\.([0-9\.]+)/i) {
	local ($v1, $v2) = ($1, $2);
	$v2 =~ s/\.//g;
	return "$v1.$v2";
	}
return undef;
}

# apply_configuration()
# Activate the ProFTPd configuration, either by sending a HUP signal or
# by stopping and starting
sub apply_configuration
{
	my $self = shift;

# Check if running from inetd
local $conf = $self->get_config();
local $st = $self->find_directive("ServerType", $conf);
if ($st eq 'inetd') {
	return $self->{'text_hash'}->{'stop_einetd'};
	}
if ($self->get_proftpd_version() > 1.22) {
	# Stop and re-start
	local $err = $self->stop_proftpd();
	return $err if ($err);
	return $self->start_proftpd();
	}
else {
	# Can just HUP
	local $pid = $self->get_proftpd_pid();
	$pid || return $self->{'text_hash'}->{'apply_egone'};
	$self->kill_logged('HUP', $pid);
	return undef;
	}
}

# stop_proftpd()
# Halts the running ProFTPd process, and returns undef on success or any error
# message on failure.
sub stop_proftpd
{
	my $self = shift;

# Check if running from inetd
local $conf = $self->get_config();
local $st = $self->find_directive("ServerType", $conf);
if ($st eq 'inetd') {
	return $self->{'text_hash'}->{'stop_einetd'};
	}
if ($self->{'config_hash'}->{'stop_cmd'}) {
	local $out = $self->backquote_logged("$self->{'config_hash'}->{'stop_cmd'} 2>&1 </dev/null");
	if ($?) {
		return "<pre>$out</pre>";
		}
	}
else {
	local $pid = $self->get_proftpd_pid();
	$pid && $self->kill_logged('TERM', $pid) ||
		return $self->{'text_hash'}->{'stop_erun'};
	}
return undef;
}

# start_proftpd()
# Attempt to start the FTP server, and return undef on success or an error
# messsage on failure.
sub start_proftpd
{
	my $self = shift;

local $conf = $self->get_config();
local $st = $self->find_directive("ServerType", $conf);
if ($st eq 'inetd') {
	return $self->{'text_hash'}->{'start_einetd'};
	}
local $out;
if ($self->{'config_hash'}->{'start_cmd'}) {
	$out = $self->backquote_logged("$self->{'config_hash'}->{'start_cmd'} 2>&1 </dev/null");
	}
else {
	$out = $self->backquote_logged("$self->{'config_hash'}->{'proftpd_path'} 2>&1 </dev/null");
	}
return $? ? "<pre>$out</pre>" : undef;
}


1;

