#!/usr/bin/perl

&MAIN();

 sub MAIN {
# userpermissions_form.cgi
# Display a the list of users and their permissions
# Author: Mattias Gaertner
#
# Abstract:
#   - Allows editing the user permissions for a directory with an 
#     .ftpaccess file.
#   - It has a select field to easily add a user to the .ftpaccess file.
#   - Shows a list of users with their permissions.
#   - Provides minimum allowed commands (at the moment hardcoded in
#     $MiniumCommands).
#     These commands will applied to any new and changed permissions.
#   - Shows names instead of the hard to remember FTP abbreviations
#     (e.g. PBSZ).
#   - Commands can be combined. For example: RNFR and RNTO are shown 
#     as only one permission.
#   - adds automatically a DenyAll All limit, so the default is to allow
#     nothing.
#   
# ToDos:
#   - multi language support 
#   - a page to config the minimum commands
#   - a page to config the tuples (combined commands)
#   - Probably some functions already exists in webmin and can be replaced

&ModuleLoader::preload_module('proftpd_class', "proftpd::ProftpdLib");
$self = new proftpd::ProftpdLib:: 'proftpd_class',%$self;


$self->ReadParse();

# read .ftpaccess file
$self->{'file'} = $self->{'in_hash'}->{'file'};
$self->{'title'} = $self->text('ftpindex_header', "<tt>$self->{'in_hash'}->{'file'}</tt>");
$self->{'return'} = "ftpaccess_index.cgi";
$self->{'rmsg'} = $self->{'text_hash'}->{'ftpindex_return'};

$self->ui_print_header($self->{'title'}, "Edit User Permissions", "",
	undef, undef, undef, undef, $self->restart_button());

#########################################
# Navigation parameters
foreach $h ('virt', 'idx', 'file', 'limit', 'anon', 'global') {
    if (defined($self->{'in_hash'}->{$h})) {
	$self->{'NavigationData'}.="<input type=hidden name=$h value='$self->{'in_hash'}->{$h}'>\n";
	push(@{$self->{'args_array'}}, "$h=$self->{'in_hash'}->{$h}");
    }
}
$self->{'args'} = join('&', @{$self->{'args_array'}});

# These are the FTP Commands, that any user have
$self->{'MinimumCommands'}="CWD XCWD CDUP XCUP PORT PASS PASV EPRT EPSV"
  ." PWD XPWD SIZE HELP NOOP AUTH ABORT USER LIST TYPE PROT QUIT PBSZ MDTM MODE";

$self->{'Commands_hash'}->{"CWD"}="Change working directory";
$self->{'Commands_hash'}->{"XCWD"}=""; 
$self->{'Commands_hash'}->{"CDUP"}="";
$self->{'Commands_hash'}->{"XCUP"}="";
$self->{'Commands_hash'}->{"PORT"}="";
$self->{'Commands_hash'}->{"PASV"}="enter passive mode";
$self->{'Commands_hash'}->{"EPRT"}="";
$self->{'Commands_hash'}->{"EPSV"}="";
$self->{'Commands_hash'}->{"RNFR"}="Rename From";
$self->{'Commands_hash'}->{"RNTO"}="Rename To";
$self->{'Commands_hash'}->{"DELE"}="Delete File";
$self->{'Commands_hash'}->{"RMD"}="Remove Directory";
$self->{'Commands_hash'}->{"XRMD"}="X Remove Directory";
$self->{'Commands_hash'}->{"MKD"}="Create Directory";
$self->{'Commands_hash'}->{"XMKD"}="X Create Directory";
$self->{'Commands_hash'}->{"MODE"}="";
$self->{'Commands_hash'}->{"PWD"}="";
$self->{'Commands_hash'}->{"XPWD"}="";
$self->{'Commands_hash'}->{"SIZE"}="";
$self->{'Commands_hash'}->{"SITE_CHMOD"}="Change Unix File Permissions";
$self->{'Commands_hash'}->{"STAT"}="Return Server Status";
$self->{'Commands_hash'}->{"SYST"}="Prints System Info";
$self->{'Commands_hash'}->{"HELP"}="";
$self->{'Commands_hash'}->{"NOOP"}="";
$self->{'Commands_hash'}->{"AUTH"}="";
$self->{'Commands_hash'}->{"PBSZ"}="";
$self->{'Commands_hash'}->{"PROT"}="";
$self->{'Commands_hash'}->{"TYPE"}="Set Transfer Type";
$self->{'Commands_hash'}->{"MODE"}="Set Transfer Mode";
$self->{'Commands_hash'}->{"MDTM"}="List Modification Time";
$self->{'Commands_hash'}->{"RETR"}="Retrieve (Read)";
$self->{'Commands_hash'}->{"STOR"}="Store (Write)";
$self->{'Commands_hash'}->{"STOU"}="Store Unique";
$self->{'Commands_hash'}->{"APPE"}="Append";
$self->{'Commands_hash'}->{"REST"}="Restart Write";
$self->{'Commands_hash'}->{"ABOR"}="Abort";
$self->{'Commands_hash'}->{"USER"}="";
$self->{'Commands_hash'}->{"PASS"}="";
$self->{'Commands_hash'}->{"LIST"}="List remote files";
$self->{'Commands_hash'}->{"QUIT"}="";
$self->{'Commands_hash'}->{"TupleRMD"} = "Remove Directory";
$self->{'Commands_hash'}->{"TupleMKD"} = "Make Directory";
$self->{'Commands_hash'}->{"TupleRN"} = "Rename";
$self->{'Commands_hash'}->{"TuplePWD"} = "Print Working Directory";

# Not implemented by proftpd:
#$self->{'Commands_hash'}->{"STRU"}="Specify File Structure";

# Here you can group commands
$self->{'CommandTuples_hash'}->{"TupleRMD"} = "RMD XRMD";
$self->{'CommandTuples_hash'}->{"TupleMKD"} = "MKD XMKD";
$self->{'CommandTuples_hash'}->{"TupleRN"} = "RNFR RNTO";
$self->{'CommandTuples_hash'}->{"TuplePWD"} = "PWD XPWD";

# Create CommandToTuple array
foreach $TupleName(sort keys %{$self->{'CommandTuples_hash'}}){
    foreach $Command(split (" ",$self->{'CommandTuples_hash'}->{$TupleName})){
	next unless ($Command);
	$self->{'CommandToTuple_hash'}->{$Command}=$TupleName;
    } 
}

#########################################
# Get user list and read old permissions
&GetUsers();
&GetFTPAccessUserPerms($self->{'file'});

#########################################
# Parse Input and update .ftpaccess file

foreach $ParamName(keys %{$self->{'in_hash'}}){
    #print "Name=\"$ParamName\" Value=\"".$self->{'in_hash'}->{$ParamName}."\"<br>\n";
    if($ParamName eq "AddUser"){
	$Username=$self->{'in_hash'}->{$ParamName};
	if($Username =~ /^[a-zA-Z_]+$/){
	    &AddUser($Username,$self->{'file'});
	}
    }
    if($ParamName eq "DeleteUser"){
	$Username=$self->{'in_hash'}->{$ParamName};
	if($Username =~ /^[a-zA-Z_]+$/){
	    if($self->{'in_hash'}->{"Confirm Delete User"} eq "on"){
		&DeleteUser($Username,$self->{'file'});
		#print "New used usernames: $self->{'UsedUsernames'}<br>\n";
	    } else {
		print "<H2>To really delete a user, please check the confim checkbox.</H2>\n";
	    }
	}
    }
    if($ParamName eq "ChangePermissions"){
	$Username=$self->{'in_hash'}->{$ParamName};
	if($Username =~ /^[a-zA-Z_]+$/){
	    &ChangePermissions($Username,$self->{'file'});
	}
    }
}

#########################################
# select box and button for add user
print "<form action=userpermissions_form.cgi method=get>\n";
print $self->{'NavigationData'};
print "<H3>Add an User to the permission table</H3>\n";
print "<select name=\"AddUser\">\n";
foreach $Username (sort split(" ",$self->{'Usernames'})){
    print "<option value=\"$Username\">$Username</option>\n";
}
print "</select>\n";
print "<input type=submit value=\"Add User\"><br>\n";
print "</form>\n";

#########################################
# Print Permissions

$self->{'MaxColumns'}=4;
foreach $Username(sort split (" ",$self->{'UsedUsernames'})){
    #print "User: $Username  Allowed=\"".$self->{'UserAllowedCommands_hash'}->{$Username}."\" Denied=\"".$self->{'UserDeniedCommands_hash'}->{$Username}."\"\n";
    print "<form action=userpermissions_form.cgi method=get>\n";
    print $self->{'NavigationData'};
    print "<input type=hidden name=\"ChangePermissions\" value=\"".$Username."\">\n";
    print "<HR WIDTH=\"100%\">\n";
    print "<H2>User: $Username</H2>\n";
    print "<table border=1>\n";
    $self->{'Column'}=0;
    $self->{'Row'}=0;
    foreach $Command(sort keys %{$self->{'Commands_hash'}}){
	if($self->{'MinimumCommands'} =~ /$Command/i){
	    # skip minimum permissions, that all users are allowed to
	    next;
	}
	if($self->{'CommandToTuple_hash'}->{$Command}){
	    # skip commands that belong to a tuple
	    next;
	}
	$self->{'FTPCommands'}=$Command;
	if($self->{'CommandTuples_hash'}->{$self->{'FTPCommands'}}){
	    $self->{'FTPCommands'} = $self->{'CommandTuples_hash'}->{$self->{'FTPCommands'}};
	}

	if($self->{'Column'} == 0){
	    if($self->{'Row'}==0){
		print "  <tr>\n";
		for ($i=0; $i<$self->{'MaxColumns'}; $i++){
		    print "    <td>Command</td><td>Allow/Deny/Default</td>\n";
		}
		print "  </tr>\n";
	    }
	    print "  <tr>\n";
	}
	$self->{'CommandDesc'}=$self->{'Commands_hash'}->{$Command};
	if(!$self->{'CommandDesc'}){
	    $self->{'CommandDesc'} = $Command;
	}
	print "    <td>$self->{'CommandDesc'}</td><td>\n";
	if(&CommandContains($self->{'UserAllowedCommands_hash'}->{$Username},$self->{'FTPCommands'})){
	    $self->{'AllowChecked'}=" checked";
	} else {
	    $self->{'AllowChecked'}="";
	}
	if(&CommandContains($self->{'UserDeniedCommands_hash'}->{$Username},$self->{'FTPCommands'})){
	    $self->{'DenyChecked'}=" checked";
	} else {
	    $self->{'DenyChecked'}="";
	}
	if($self->{'AllowChecked'} || $self->{'DenyChecked'}){
	    $self->{'DefaultChecked'} = "";
	} else {
	    $self->{'DefaultChecked'} = " checked";
	}
	print "      <input type=\"radio\" name=\"".$Command."\" value=\"allow\"".$self->{'AllowChecked'}.">\n";
	print "      <input type=\"radio\" name=\"".$Command."\" value=\"deny\"".$self->{'DenyChecked'}.">\n";
	print "      <input type=\"radio\" name=\"".$Command."\" value=\"default\"".$self->{'DefaultChecked'}.">\n";
	print "    </td>";
	$self->{'Column'}++;
	if($self->{'Column'} == $self->{'MaxColumns'}){
	    print "  </tr>\n";
	    $self->{'Column'}=0;
	    $self->{'Row'}++;
	}
    }
    if($self->{'Column'} > 0){
	print "  </tr>\n";
    }
    print "</table>\n";
    print "<input type=submit value=\"Change Permissions\">\n";
    print "</form><br>\n";

    print "<form action=userpermissions_form.cgi method=get>\n";
    print $self->{'NavigationData'};
    print "<input type=hidden name=\"DeleteUser\" value=\"".$Username."\">\n";
    print "<input type=submit value=\"Delete User Permissions\">\n";
    print "<input type=checkbox name=\"Confirm Delete User\">I'm sure<br>\n";
    print "</form>\n";
}

#########################################
# print textarea

print "<HR WIDTH=100%>\n";
print $self->text('manual_header', "<tt>$self->{'file'}</tt>"),"<p>\n";

print "<form action=manual_save.cgi method=post enctype=multipart/form-data>\n";
print $self->{'NavigationData'};

print "<br><textarea rows=15 cols=80 name=directives>\n";
$self->{'lref'} = $self->read_file_lines($self->{'file'});
if (!defined($self->{'start'})) {
	$self->{'start'} = 0;
	$self->{'end'} = @{$self->{'lref'}} - 1;
	}
for($i=$self->{'start'}; $i<=$self->{'end'}; $i++) {
	print $self->html_escape($self->{'lref'}->[$i]),"\n";
	}
print "</textarea><br><input type=submit value=\"$self->{'text_hash'}->{'save'}\"></form>\n";

#########################################
# print footer

$self->ui_print_footer("$self->{'return'}?$self->{'args'}", $self->{'rmsg'});

exit;

#########################################################

# end.
1;
		
}

sub GetUsers(){
    my $UserCount=0;
    setpwent();
    while(my @uinfo = getpwent()) {
	if ($uinfo[2] > 100) {
		$UserCount++;
                $self->{'Users_array'}->[$UserCount]=$uinfo[0];
		$self->{'Usernames'}.=" ".$uinfo[0];
	}
    }
    endpwent();
}

sub GetFTPAccessUserPerms(){
    # Fills global variables:
    # $self->{'UsedUsernames'}, %{$self->{'UserAllowedCommands_hash'}}, %{$self->{'UserDeniedCommands_hash'}}

    my ($FTPAccessFile) = @_;

    ##################################################
    # Read .ftpaccess file
    my $Commands = "";

    open FTPACCESS, "$FTPAccessFile" or $self->error("Can't open $FTPAccessFile: $!");
    while (my $line=<FTPACCESS>){
        chomp $line;
        #print $line."\n";
        if($line =~ /<Limit (.*)>/i){
            $Commands = $1;
            #print "Limit $Commands\n";
        }
        if($line =~ /<\/Limit(.*)>/i){
            $Commands = "";
            #print "End Limit $Commands\n";
        }
        if($Commands){
            #print "$line\n";
            if($line =~ /AllowUser (.+)/i){
                my $AllowedUsernames = $1;
                #print "AllowUser $AllowedUsernames\n";
                foreach $AllowedUsername (split (" ",$AllowedUsernames)){
                    next unless ($AllowedUsername);
                    $self->{'UserAllowedCommands_hash'}->{$AllowedUsername}.=" ".$Commands;
                    #print "AllowUser $AllowedUsername\n";
                }
            }
            if($line =~ /DenyUser (.+)/i){
                my $DeniedUsernames = $1;
                foreach $DeniedUsername (split (" ",$DeniedUsernames)){
                    next unless ($DeniedUsername);
                    $self->{'UserDeniedCommands_hash'}->{$DeniedUsername}.=" ".$Commands;
                }
            }
        }
    }
    close FTPACCESS;

    ##################################################
    # collect all mentioned users in table
    $self->{'UsedUsernames'}="";
    foreach $Username(keys %{$self->{'UserAllowedCommands_hash'}}){
        #print "Adding $Username\n";
	$self->{'UserAllowedCommands_hash'}->{$Username}=
	    &UnifyAndExpandCommands($self->{'UserAllowedCommands_hash'}->{$Username}." ".$Commands);
        if($self->{'UsedUsernames'} !~ /\b$Username\b/){
            $self->{'UsedUsernames'}.=$Username." ";
        }
    }
    foreach $Username(keys %{$self->{'UserDeniedCommands_hash'}}){
	$self->{'UserDeniedCommands_hash'}->{$Username}=
	    &UnifyAndExpandCommands($self->{'UserDeniedCommands_hash'}->{$Username}." ".$Commands);
        if($self->{'UsedUsernames'} !~ /\b$Username\b/){
            $self->{'UsedUsernames'}.=$Username." ";
        }
    }
}

sub UnifyAndExpandCommands(){
    (my $Commands) = @_;
    my $NewCommands = "";
    foreach $Command(split(" ",$Commands)){
	next unless($Command);
	if($self->{'CommandTuples_hash'}->{$Command}){
	    $NewCommands.=" ".$self->{'CommandTuples_hash'}->{$Command};
	} else {
	    $NewCommands.=" ".$Command;
	}
    }
    return &UnifyCommands($NewCommands);
}

sub UnifyCommands(){
    (my $Commands) = @_;
    my $NewCommands = "";
    foreach $Command(split(" ",$Commands)){
	next unless($Command);
	next if($NewCommands =~ /\b$Command\b/i);
	if($NewCommands){
	    $NewCommands.=" ";
	}
	$NewCommands.=$Command;
    }
    return $NewCommands;
}

sub AddUser(){
    (my $Username, $FTPAccessFile) = @_;

    if($self->{'Usernames'} =~ /\b$self->{'Usernames'}\b/){
	print "<H2>Username $Username does not exist.</H2>\n";
	return;
    }

    if ($self->{'UserAllowedCommands_hash'}->{$Username} || $self->{'UserDeniedCommands_hash'}->{$Username}){
	# user already exists
	print "<H2>Username $Username already exists.</H2>\n";
	return;
    }
    $self->{'UserAllowedCommands_hash'}->{$Username}=$self->{'MinimumCommands'};
    $self->{'UserDeniedCommands_hash'}->{$Username}="";
    if($self->{'UsedUsernames'} !~ /\b$Username\b/){
	$self->{'UsedUsernames'}.=$Username." ";
    }

    &WritePermissions($FTPAccessFile);
}

sub DeleteUser(){
    (my $Username, $FTPAccessFile) = @_;

    if($self->{'UsedUsernames'} =~ /\b$self->{'Usernames'}\b/){
	print "<H2>Username $Username does not exist in table.</H2>\n";
	return;
    }

    if ((!$self->{'UserAllowedCommands_hash'}->{$Username}) && (!$self->{'UserDeniedCommands_hash'}->{$Username})){
	# user already deleted
	print "<H2>Username $Username is already not in table.</H2>\n";
	return;
    }
    $self->{'UserAllowedCommands_hash'}->{$Username}="";
    $self->{'UserDeniedCommands_hash'}->{$Username}="";
    $self->{'UsedUsernames'} =~ s/\b$Username\b *//;

    &WritePermissions($FTPAccessFile);
}

sub ChangePermissions(){
    (my $Username, $FTPAccessFile) = @_;

    if($self->{'UsedUsernames'} =~ /\b$self->{'Usernames'}\b/){
	print "<H2>Username $Username does not exist in table.</H2>\n";
	return;
    }

    foreach $Command(keys %{$self->{'Commands_hash'}}){
	#print "$Command value=".$self->{'in_hash'}->{$Command}."<br>\n";

	if($self->{'CommandToTuple_hash'}->{$Command}){
	    # skip commands in tuples
	    next;
	}

	my $FTPCommands=$Command;
	if($self->{'CommandTuples_hash'}->{$FTPCommands}){
	    $FTPCommands = $self->{'CommandTuples_hash'}->{$FTPCommands};
	}

	if ($self->{'in_hash'}->{$Command} eq "allow"){
	    $self->{'UserAllowedCommands_hash'}->{$Username}.=" ".$FTPCommands;
	    #print "Allow $Username $Command<br>\n";
	} else {
	    $self->{'UserAllowedCommands_hash'}->{$Username} =
		&RemoveCommands($self->{'UserAllowedCommands_hash'}->{$Username},$FTPCommands);
	}
	if ($self->{'in_hash'}->{$Command} eq "deny"){
	    $self->{'UserDeniedCommands_hash'}->{$Username}.=" ".$FTPCommands;
	    #print "Deny $Username $Command<br>\n";
	} else {
	    $self->{'UserDeniedCommands_hash'}->{$Username} =
		&RemoveCommands($self->{'UserDeniedCommands_hash'}->{$Username},$FTPCommands);
	}
    }
    $self->{'UserAllowedCommands_hash'}->{$Username}=
	&UnifyCommands($self->{'MinimumCommands'}." ".$self->{'UserAllowedCommands_hash'}->{$Username});
    $self->{'UserDeniedCommands_hash'}->{$Username}=
	&UnifyCommands($self->{'UserDeniedCommands_hash'}->{$Username});

    &WritePermissions($FTPAccessFile);
}

sub WritePermissions(){
    # Read .ftpaccess file, remove all user command permissions
    # and add new set of user permissions
    (my $FTPAccessFile) = @_;
    my $NewConfig = "";
    my $OldCommands = "";
    my $Username;

    # Lock .ftpaccess file
    $self->lock_file($FTPAccessFile);
    $self->lock_file($FTPAccessFile);


    # Read old .ftpaccess file
    open FTPACCESS, "$FTPAccessFile" or die "Can't read $FTPAccessFile: $!";
    $self->{'DenyAllBlockFound'} = 0;
    while(my $line = <FTPACCESS>){
	my $ShortLine = $line;
	chomp $ShortLine;
        #print $ShortLine."\n";
        if($ShortLine =~ /<Limit (.*)>/i){
	    # start of Limit block
            $OldCommands = $1;
            #print "Limit $OldCommands\n";
	    $self->{'LimitBlock'} = $line;
	    $self->{'ImportantLimitLineFound'} = 0;
	    $self->{'DenyAllFound'} = 0;
        } elsif($ShortLine =~ /<\/Limit(.*)>/i){
	    # end of Limit block
            #print "End Limit $OldCommands\n";
	    $self->{'LimitBlock'} .= $line;
	    if($self->{'ImportantLimitLineFound'}){
		$NewConfig .= $self->{'LimitBlock'};
	    }
	    if(($OldCommands =~ /\bALL\b/i) && ($self->{'DenyAllFound'})){
		# this was a DenyAll for All commands block
		$self->{'DenyAllBlockFound'} = 1;
	    }
            $OldCommands = "";
        } elsif($OldCommands){
            #print "$ShortLine\n";
            if($ShortLine =~ /AllowUser (.*)/i){
		# AllowUser line -> will be replaced, not important
            } elsif($ShortLine =~ /DenyUser (.*)/i){
		# DenyUser line -> will be replaced, not important
            } elsif($ShortLine =~ /^ +$/){
		# empty line -> not important, but keep it for readability
		$self->{'LimitBlock'} .= $line;
	    } else {
		# other limit directive -> important
		$self->{'LimitBlock'} .= $line;
		$self->{'ImportantLimitLineFound'} = 1;
		if($ShortLine =~ /\bDenyAll\b/i){
		    $self->{'DenyAllFound'} = 1;
		}
	    }
        } else {
	    # other directives -> keep
	    $NewConfig .= $line;
	}
    }
    close FTPACCESS;

    # Append new directives

    # Append DenyAll block if not already there
    if(!$self->{'DenyAllBlockFound'}){
	$NewConfig.="<Limit All>\n";
	$NewConfig.="  DenyAll\n";
	$NewConfig.="</Limit>\n";
    }

    # Append Limit blocks for users
    foreach $Username (sort split(" ",$self->{'Usernames'})){
	my $CurAllow = $self->{'UserAllowedCommands_hash'}->{$Username};
	if ($CurAllow){
	    $NewConfig.="<Limit ".$CurAllow.">\n";
	    $NewConfig.="  AllowUser ".$Username."\n";
	    $NewConfig.="</Limit>\n";
	}
	my $CurDeny = $self->{'UserDeniedCommands_hash'}->{$Username};
	if ($CurDeny){
	    $NewConfig.="<Limit ".$CurDeny.">\n";
	    $NewConfig.="  DenyUser ".$Username."\n";
	    $NewConfig.="</Limit>\n";
	}
    }
    #print "<br>\n".$NewConfig."<br>\n";

    # Write new .ftpaccess file
    open FTPACCESS, "> $FTPAccessFile" or die "Can't append to $FTPAccessFile: $!";
    print FTPACCESS $NewConfig;
    close FTPACCESS;

    # Unlock .ftpaccess file
    $self->unlock_file($FTPAccessFile);

    $self->{'logtype'} = 'ftpaccess'; 
    $self->{'logname'} = $self->{'in_hash'}->{'file'};
    $self->webmin_log($self->{'logtype'}, "user permissions", $self->{'logname'}, \%{$self->{'in_hash'}});
}

sub CommandContains(){
    (my $Commands, my $SubSet) = @_;
    foreach my $Command(split(" ",$SubSet)){
	next unless($Command);
	if($Commands =~ /\b$Command\b/i){
	    return 1;
	}
    }
    return 0;
}

sub RemoveCommands(){
    (my $Commands, my $SubSet) = @_;
    foreach my $Command(split(" ",$SubSet)){
	next unless($Command);
	$Commands =~ s/\b$Command\b *//gi;
    }
    return $Commands;
}


1;

