#!/usr/bin/perl
package pserver::PserverLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'pserver::PserverLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# pserver-lib.pl

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

$self->{'cvs_path'} = $self->has_command($self->{'config_hash'}->{'cvs'});
$self->{'cvs_port'} = 2401;
$self->{'cvs_inet_name'} = "cvspserver";
$self->{'has_xinetd'} = $self->foreign_check("xinetd");
$self->{'has_inetd'} = $self->foreign_check("inetd");
$self->{'passwd_file'} = "$self->{'config_hash'}->{'cvsroot'}/CVSROOT/passwd";
$self->{'readers_file'} = "$self->{'config_hash'}->{'cvsroot'}/CVSROOT/readers";
$self->{'writers_file'} = "$self->{'config_hash'}->{'cvsroot'}/CVSROOT/writers";
$self->{'cvs_config_file'} = "$self->{'config_hash'}->{'cvsroot'}/CVSROOT/config";

@{$self->{'features_array'}} = ('passwd', 'access', 'config', 'cvsweb');
%{$self->{'access_hash'}} = $self->get_module_acl();
%{$self->{'featureprog_hash'}} = ( 'passwd' => 'list_passwd.cgi',
		 'access' => 'edit_access.cgi',
		 'config' => 'edit_config.cgi',
		 'cvsweb' => 'cvsweb.cgi' );

@{$self->{'hist_chars_array'}} = ( "F", "O", "E", "T", "C", "G", "U", "W", "A", "M", "R" );

1;

1;
		return $self;
}

# check_inetd()
# Find out if cvs is being run from inetd or xinetd
sub check_inetd
{
	my $self = shift;

if ($self->{'has_xinetd'}) {
	# Find an xinetd service on the CVS port, with the CVS command, or
	# with the CVS name
	$self->foreign_require("xinetd", "xinetd-lib.pl");
	local @xic = $main::xinetd_object->get_xinetd_config();
	local $x;
	foreach $x (@xic) {
		next if ($x->{'name'} ne 'service');
		local $q = $x->{'quick'};
		if ($q->{'server'}->[0] eq $self->{'cvs_path'} ||
		    $q->{'server'}->[0] eq $self->{'config_hash'}->{'cvs'} ||
		    $self->{'port'} == $self->{'cvs_port'} ||
		    $x->{'value'} eq $self->{'cvs_inet_name'}) {
			# Found the entry
			return { 'type' => 'xinetd',
				 'user' => $q->{'user'}->[0],
				 'command' => $q->{'server'}->[0],
				 'args' => $q->{'server'}->[0]." ".
					   $q->{'server_args'}->[0],
				 'active' => $q->{'disable'}->[0] ne 'yes',
				 'xinetd' => $x };
			}
		}
	}
if ($self->{'has_inetd'}) {
	# Find an inetd service on the CVS port, with the CVS command, or
	# with the CVS name
	local (%portmap, $s, $a, $i);
	$self->foreign_require("inetd", "inetd-lib.pl");
	foreach $s ($main::inetd_object->list_services()) {
		$portmap{$s->[1]} = $s;
		foreach $a (split(/\s+/, $s->[4])) {
			$portmap{$a} = $s;
			}
		}
	foreach $i ($main::inetd_object->list_inets()) {
		if ($i->[8] eq $self->{'cvs_path'} || $i->[8] eq $self->{'config_hash'}->{'cvs'} ||
		    $portmap{$i->[3]}->[2] == $self->{'cvs_port'} ||
		    $i->[3] eq $self->{'cvs_inet_name'}) {
			# Found the entry
			return { 'type' => 'inetd',
				 'user' => $i->[7],
				 'command' => $i->[8],
				 'args' => $i->[9],
				 'active' => $i->[1],
				 'inetd' => $i };
			}
		}
	}
return undef;
}

# list_passwords()
# List all CVS users
sub list_passwords
{
	my $self = shift;

local @rv;
local $lnum = 0;
open(PASSWD, $self->{'passwd_file'});
while(<PASSWD>) {
	s/\r|\n//g;
	s/#.*$//;
	local @p = split(/:/, $_);
	if (@p) {
		push(@rv, { 'user' => $p[0],
			    'pass' => $p[1],
			    'unix' => $p[2],
			    'line' => $lnum,
			    'index' => scalar(@rv) });
		}
	$lnum++;
	}
close(PASSWD);
return @rv;
}

sub create_password
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'passwd_file'});
push(@$lref, join(":", $_[0]->{'user'}, $_[0]->{'pass'}, $_[0]->{'unix'}));
$self->flush_file_lines();
}

sub modify_password
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'passwd_file'});
$lref->[$_[0]->{'line'}] =
	join(":", $_[0]->{'user'}, $_[0]->{'pass'}, $_[0]->{'unix'});
$self->flush_file_lines();
}

sub delete_password
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'passwd_file'});
splice(@$lref, $_[0]->{'line'}, 1);
$self->flush_file_lines();
}

# get_cvs_config()
# Returns a list of values from the CVSROOT/config file
sub get_cvs_config
{
	my $self = shift;

local @rv;
local $lnum = 0;
open(CONFIG, $self->{'cvs_config_file'});
while(<CONFIG>) {
	s/\s+$//;
	s/^\s*#.*$//;
	if (/^\s*([^\s=]+)\s*=\s*(.*)/) {
		push(@rv, { 'name' => $1,
			    'value' => $2,
			    'line' => $lnum,
			    'index' => scalar(@{$self->{'rc_array'}}) } );
		}
	$lnum++;
	}
close(CONFIG);
return @rv;
}

# find(name, &config)
sub find
{
	my $self = shift;

local ($c, @rv);
foreach $c (@{$_[1]}) {
	push(@rv, $c) if (lc($c->{'name'}) eq lc($_[0]));
	}
return wantarray ? @rv : $rv[0];
}

# save_cvs_config(&config, name, value, [default])
sub save_cvs_config
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'cvs_config_file'});
local $old = $self->find($_[1], $_[0]);
if ($old && $_[2]) {
	# Replacing an existing config line
	$lref->[$old->{'line'}] = "$_[1]=$_[2]";
	}
elsif ($old) {
	# Deleting a config line (unless it already exists with the default)
	if (!$_[3] || $old->{'value'} ne $_[3]) {
		splice(@$lref, $old->{'line'}, 1);
		local $c;
		foreach $c (@{$_[0]}) {
			$c->{'line'}-- if ($c->{'line'} > $old->{'line'});
			}
		}
	}
elsif ($_[2]) {
	# Adding a config line
	push(@$lref, "$_[1]=$_[2]");
	}
}

# get_cvs_version(&out)
# Returns the cvs command version number, or undef
sub get_cvs_version
{
	my $self = shift;

local $out = `$self->{'config_hash'}->{'cvs'} -v`;
${$_[0]} = $out;
if ($out =~ /CVS[^0-9\.]*([0-9\.]+)/) {
	return $1;
	}
else {
	return undef;
	}
}


1;

