#!/usr/bin/perl
package qmailadmin::Autoreply;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'qmailadmin::Autoreply');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# autoreply.pl
# Simple autoreply script

# read qmail module config
$self->{'p'} = -l $0 ? readlink($0) : $0;
$self->{'p'} =~ /^(.*)\/[^\/]+$/;
open(CONF, "$1/config") || die "Failed to open $1/config";
while(<CONF>) {
	if (/^(\S+)=(.*)/) {
		$self->{'config_hash'}->{$1} = $2;
		}
	}
close(CONF);

# read headers and body
while(<STDIN>) {
	s/\r|\n//g;
	if (/^(\S+):\s+(.*)/) {
		$self->{'header_hash'}->{lc($1)} = $2;
		}
	elsif (!$_) { last; }
	}
while(<STDIN>) {
	$self->{'body'} .= $_;
	}
if ($self->{'header_hash'}->{'x-webmin-autoreply'}) {
	print STDERR "Cancelling autoreply to an autoreply\n";
	exit 1;
	}
if ($self->{'header_hash'}->{'x-mailing-list'} ||
    $self->{'header_hash'}->{'list-id'} ||
    $self->{'header_hash'}->{'precedence'} =~ /junk|bulk|list/i ||
    $self->{'header_hash'}->{'to'} =~ /Multiple recipients of/i) {
	# Do nothing if post is from a mailing list
	exit 0;
	}
if ($self->{'header_hash'}->{'from'} =~ /postmaster|mailer-daemon/i) {
	# Do nothing if post is a bounce
	exit 0;
	}

# work out the correct to address
@{$self->{'to_array'}} = ( $self->split_addresses($self->{'header_hash'}->{'to'}),
	$self->split_addresses($self->{'header_hash'}->{'cc'}),
	$self->split_addresses($self->{'header_hash'}->{'bcc'}) );
$self->{'to'} = $self->{'to_array'}->[0]->[0];
foreach $t (@{$self->{'to_array'}}) {
	if ($t->[0] =~ /^([^\@\s]+)/ && $1 eq $ARGV[1]) {
		$self->{'to'} = $t->[0];
		}
	}

# build list of default reply headers
$self->{'rheader_hash'}->{'From'} = $self->{'to'};
$self->{'rheader_hash'}->{'To'} = $self->{'header_hash'}->{'reply-to'} ? $self->{'header_hash'}->{'reply-to'}
				     : $self->{'header_hash'}->{'from'};
$self->{'rheader_hash'}->{'Subject'} = "Autoreply to $self->{'header_hash'}->{'subject'}";
$self->{'rheader_hash'}->{'X-Webmin-Autoreply'} = 1;
$self->{'rheader_hash'}->{'X-Originally-To'} = $self->{'header_hash'}->{'to'};

# read the autoreply file
if (open(AUTO, $ARGV[0])) {
	while(<AUTO>) {
		s/\$SUBJECT/$self->{'header_hash'}->{'subject'}/g;
		s/\$FROM/$self->{'header_hash'}->{'from'}/g;
		s/\$TO/$self->{'to'}/g;
		s/\$DATE/$self->{'header_hash'}->{'date'}/g;
		s/\$BODY/$self->{'body'}/g;
		if (/^(\S+):\s*(.*)/ && !$self->{'doneheaders'}) {
			$self->{'rheader_hash'}->{$1} = $2;
			}
		else {
			$self->{'rbody'} .= $_;
			$self->{'doneheaders'} = 1;
			}
		}
	close(AUTO);
	}
else {
	$self->{'rbody'} = "Failed to open autoreply file $ARGV[0] : $!";
	}

# Open the replies tracking DBM, if one was set
if ($self->{'rheader_hash'}->{'Reply-Tracking'}) {
	$self->{'track_replies'} = dbmopen(%{$self->{'replies_hash'}}, $self->{'rheader_hash'}->{'Reply-Tracking'}, 0700);
	}
if ($self->{'track_replies'}) {
	# See if we have replied to this address before
	$self->{'period'} = $self->{'rheader_hash'}->{'Reply-Period'} || 60*60;
	($self->{'from'}) = $self->split_addresses($self->{'header_hash'}->{'from'});
	if ($self->{'from'}) {
		$self->{'lasttime'} = $self->{'replies_hash'}->{$self->{'from'}->[0]};
		$self->{'now'} = time();
		if ($self->{'now'} < $self->{'lasttime'}+$self->{'period'}) {
			# Autoreplied already in this period .. just halt
			exit(0);
			}
		$self->{'replies_hash'}->{$self->{'from'}->[0]} = $self->{'now'};
		}
	}
delete($self->{'rheader_hash'}->{'Reply-Tracking'});
delete($self->{'rheader_hash'}->{'Reply-Period'});

# run qmail and feed it the reply
open(MAIL, "|$self->{'config_hash'}->{'qmail_dir'}/bin/qmail-inject");
foreach $h (keys %{$self->{'rheader_hash'}}) {
	print MAIL "$h: $self->{'rheader_hash'}->{$h}\n";
	}
print MAIL "\n";
print MAIL $self->{'rbody'};
close(MAIL);

1;
		return $self;
}

# split_addresses(string)
# Splits a comma-separated list of addresses into [ email, real-name ] pairs
sub split_addresses
{
	my $self = shift;

local (@rv, $str = $_[0]);
while(1) {
	if ($str =~ /^[\s,]*(([^<>\(\)\s]+)\s+\(([^\(\)]+)\))(.*)$/) {
		push(@rv, [ $2, $3, $1 ]);
		$str = $4;
		}
	elsif ($str =~ /^[\s,]*("([^"]+)"\s+<([^\s<>]+)>)(.*)$/ ||
	       $str =~ /^[\s,]*(([^<>]+)\s+<([^\s<>]+)>)(.*)$/ ||
	       $str =~ /^[\s,]*(([^<>\[\]]+)\s+\[mailto:([^\s\[\]]+)\])(.*)$/||
	       $str =~ /^[\s,]*(()<([^\s<>]+)>)(.*)/ ||
	       $str =~ /^[\s,]*(()([^\s<>,]+))(.*)/) {
		push(@rv, [ $3, $2, $1 ]);
		$str = $4;
		}
	else {
		last;
		}
	}
return @rv;
}


1;

