#!/usr/bin/perl
package quota::SolarisLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'quota::SolarisLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# solaris-lib.pl
# Quota functions for solaris 2.5+

%{$self->{'name_to_unit_hash'}} = ( "sec", 0, "secs", 0,
		  "min", 1, "mins", 1,
		  "hour", 2, "hours", 2,
		  "day", 3, "days", 3,
		  "week", 4, "weeks", 4,
		  "month", 5, "months", 5
		);
foreach $k (keys %{$self->{'name_to_unit_hash'}}) {
	$self->{'unit_to_name_hash'}->{$self->{'name_to_unit_hash'}->{$k}} = $k;
	}

1;

1;
		return $self;
}

# quotas_init()
sub quotas_init
{
	my $self = shift;

return undef;
}

# quotas_supported()
# Returns 1 for user quotas, 2 for group quotas or 3 for both
sub quotas_supported
{
	my $self = shift;

return 1;
}

# free_space(filesystem)
# Returns an array containing  btotal, bfree, ftotal, ffree
sub free_space
{
	my $self = shift;

local($out);
$out = `df -t $_[0]`;
$out =~ /(\d+) blocks\s+(\d+) files\n.*\s+(\d+) blocks\s+(\d+) files/;
return ($3, $1, $4, $2);
}

# quota_can(&mnttab, &fstab)
# Can this filesystem type support quotas?
#  0 = No quota support (or not turned on in /etc/fstab)
#  1 = User quotas only
#  2 = Group quotas only
#  3 = User and group quotas
sub quota_can
{
	my $self = shift;

return $_[0]->[2] eq "ufs" && $_[0]->[1] ne $_[0]->[0] ? 1 : 0;
}

# quota_now(&mnttab, &fstab)
# Are quotas currently active?
#  0 = Not active
#  1 = User quotas active
#  2 = Group quotas active
#  3 = Both active
sub quota_now
{
	my $self = shift;

return $_[0]->[3] =~ /,quota/ || $_[0]->[3] =~ /^quota/ ? 1 : 0;
}

# filesystem_users(filesystem)
# Fills the array %{$self->{'user_hash'}} with information about all users with quotas
# on this filesystem. This may not be all users on the system..
sub filesystem_users
{
	my $self = shift;

local($rep, @rep, $n, %hasu, $u);
$rep = `$self->{'config_hash'}->{'user_repquota_command'} $_[0] 2>&1`;
if ($?) { return -1; }
setpwent();
while(@{$self->{'uinfo_array'}} = getpwent()) {
	$hasu{$self->{'uinfo_array'}->[0]}++;
	}
endpwent() if ($self->{'gconfig_hash'}->{'os_type'} ne 'hpux');
@rep = split(/\n/, $rep); @rep = @rep[3..$#rep];
for($n=0; $n<@rep; $n++) {
	if ($rep[$n] =~ /(\S+)\s+[\-\+]{2}\s+(\d+)\s+(\d+)\s+(\d+)\s+(.{0,15})\s+(\d+)\s+(\d+)\s+(\d+)(.*)/ || $rep[$n] =~ /(\S+)\s+[\-\+]{2}(.{7})(.{7})(.{7})(.{13})(.{7})(.{7})(.{7})(.*)/) {
		$self->{'user_hash'}->{$n,'user'} = $1;
		$self->{'user_hash'}->{$n,'ublocks'} = int($2);
		$self->{'user_hash'}->{$n,'sblocks'} = int($3);
		$self->{'user_hash'}->{$n,'hblocks'} = int($4);
		$self->{'user_hash'}->{$n,'gblocks'} = $5;
		$self->{'user_hash'}->{$n,'ufiles'} = int($6);
		$self->{'user_hash'}->{$n,'sfiles'} = int($7);
		$self->{'user_hash'}->{$n,'hfiles'} = int($8);
		$self->{'user_hash'}->{$n,'gfiles'} = $9;
		$self->{'user_hash'}->{$n,'user'} =~ s/^#//g;
		if ($self->{'user_hash'}->{$n,'user'} !~ /^\d+$/ && !$hasu{$self->{'user_hash'}->{$n,'user'}}) {
			# Username was truncated! Try to find him..
			foreach $u (keys %hasu) {
				if (substr($u, 0, length($self->{'user_hash'}->{$n,'user'})) eq
				    $self->{'user_hash'}->{$n,'user'}) {
					# found him..
					$self->{'user_hash'}->{$n,'user'} = $u;
					last;
					}
				}
			}
		$self->{'user_hash'}->{$self->{'nn'},'gblocks'} = $self->trunc_space($self->{'user_hash'}->{$self->{'nn'},'gblocks'});
		$self->{'user_hash'}->{$self->{'nn'},'gfiles'} = $self->trunc_space($self->{'user_hash'}->{$self->{'nn'},'gfiles'});
		}
	}
return $n;
}

# edit_quota_file(data, filesys, sblocks, hblocks, sfiles, hfiles)
sub edit_quota_file
{
	my $self = shift;

local($rv, $line);
foreach $line (split(/\n/, $_[0])) {
	if ($line =~ /^fs (\S+) blocks \(soft = (\d+), hard = (\d+)\) inodes \(soft = (\d+), hard = (\d+)\)$/ && $1 eq $_[1]) {
		# found line to change
		$line = "fs $_[1] blocks (soft = $_[2], hard = $_[3]) inodes (soft = $_[4], hard = $_[5])";
		}
	$rv .= "$line\n";
	}
return $rv;
}

# quotaon(filesystem, mode)
# Activate quotas and create quota file for some filesystem. The mode can
# be 1 for user only, 2 for group only or 3 for user and group
sub quotaon
{
	my $self = shift;

return if ($self->is_readonly_mode());
local($qf, $out);
$qf = "$_[0]/quotas";
if (!(-r $qf)) {
	$self->open_tempfile(QUOTAFILE, ">$qf", 0, 1);
	$self->close_tempfile(QUOTAFILE);
	$self->set_ownership_permissions(undef, undef, 0600, $qf);
	}
$out = $self->backquote_logged("$self->{'config_hash'}->{'user_quotaon_command'} $_[0] 2>&1");
if ($?) { return $out; }
return undef;
}

# quotaoff(filesystem, mode)
# Turn off quotas for some filesystem
sub quotaoff
{
	my $self = shift;

return if ($self->is_readonly_mode());
local($out);
$out = $self->backquote_logged("$self->{'config_hash'}->{'user_quotaoff_command'} $_[0] 2>&1");
if ($?) { return $out; }
return undef;
}

# quotacheck(filesystem, mode)
# Runs quotacheck on some filesystem
sub quotacheck
{
	my $self = shift;

$self->{'out'} = $self->backquote_logged("$self->{'config_hash'}->{'quotacheck_command'} $_[0] 2>&1");
if ($?) { return $self->{'out'}; }
return undef;
}

# copy_user_quota(user, [user]+)
# Copy the quotas for some user to many others
sub copy_user_quota
{
	my $self = shift;

for($i=1; $i<@_; $i++) {
	$self->{'out'} = $self->backquote_logged("$self->{'config_hash'}->{'user_copy_command'} ".
				quotemeta($_[0])." ".quotemeta($_[$i])." 2>&1");
	if ($?) { return $self->{'out'}; }
	}
return undef;
}

# user_filesystems(user)
# Fills the array %{$self->{'filesys_hash'}} with details of all filesystems some user has
# quotas on
sub user_filesystems
{
	my $self = shift;

local($n, $_);
open(QUOTA, "$self->{'config_hash'}->{'user_quota_command'} ".quotemeta($_[0])." |");
$n=0; while(<QUOTA>) {
	chop;
	if (/^(Disk|Filesystem)/i) { next; }
	if (/^(\S+)$/) {
		# Bogus wrapped line!
		$self->{'filesys_hash'}->{$n,'filesys'} = $1;
		local $nl = <QUOTA>;
		if ($nl =~ /^\s+(\d+)\s+(\d+)\s+(\d+)\s(.{0,15})\s(\d+)\s+(\d+)\s+(\d+)(.*)/ || $nl =~ /^.{13}(.{7})(.{7})(.{7})(.{12})(.{7})(.{7})(.{7})(.*)/) {
			$self->{'filesys_hash'}->{$n,'ublocks'} = int($1);
			$self->{'filesys_hash'}->{$n,'sblocks'} = int($2);
			$self->{'filesys_hash'}->{$n,'hblocks'} = int($3);
			$self->{'filesys_hash'}->{$n,'gblocks'} = $4;
			$self->{'filesys_hash'}->{$n,'ufiles'} = int($5);
			$self->{'filesys_hash'}->{$n,'sfiles'} = int($6);
			$self->{'filesys_hash'}->{$n,'hfiles'} = int($7);
			$self->{'filesys_hash'}->{$n,'gfiles'} = $8;
			$self->{'filesys_hash'}->{$n,'gblocks'} = $self->trunc_space($self->{'filesys_hash'}->{$n,'gblocks'});
			$self->{'filesys_hash'}->{$n,'gfiles'} = $self->trunc_space($self->{'filesys_hash'}->{$n,'gfiles'});
			$n++;
			}
		}
	elsif (/^(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s(.{0,15})\s(\d+)\s+(\d+)\s+(\d+)(.*)/ || /^(.{13})(.{7})(.{7})(.{7})(.{12})(.{7})(.{7})(.{7})(.*)/) {
		$self->{'filesys_hash'}->{$n,'filesys'} = $1;
		$self->{'filesys_hash'}->{$n,'ublocks'} = int($2);
		$self->{'filesys_hash'}->{$n,'sblocks'} = int($3);
		$self->{'filesys_hash'}->{$n,'hblocks'} = int($4);
		$self->{'filesys_hash'}->{$n,'gblocks'} = $5;
		$self->{'filesys_hash'}->{$n,'ufiles'} = int($6);
		$self->{'filesys_hash'}->{$n,'sfiles'} = int($7);
		$self->{'filesys_hash'}->{$n,'hfiles'} = int($8);
		$self->{'filesys_hash'}->{$n,'gfiles'} = $9;
		$self->{'filesys_hash'}->{$n,'filesys'} =~ s/\s+$//g;
		$self->{'filesys_hash'}->{$n,'gblocks'} = $self->trunc_space($self->{'filesys_hash'}->{$n,'gblocks'});
		$self->{'filesys_hash'}->{$n,'gfiles'} = $self->trunc_space($self->{'filesys_hash'}->{$n,'gfiles'});
		$n++;
		}
	}
close(QUOTA);
return $n;
}

# get_user_grace(filesystem)
# Returns an array containing  btime, bunits, ftime, funits
# The units can be 0=sec, 1=min, 2=hour, 3=day, 4=week, 5=month
sub get_user_grace
{
	my $self = shift;

local(@rv);
$ENV{'EDITOR'} = $ENV{'VISUAL'} = "cat";
open(GRACE, "$self->{'config_hash'}->{'user_grace_command'} |");
while(<GRACE>) {
	if (/^fs (\S+) blocks time limit = ([0-9\.]+) (\S+), files time limit = ([0-9\.]+) (\S+)/ && $1 eq $_[0]) {
		if ($2 == 0) { push(@rv, 0, 0); }
		else { push(@rv, $2, $self->{'name_to_unit_hash'}->{$3}); }
		if ($4 == 0) { push(@rv, 0, 0); }
		else { push(@rv, $4, $self->{'name_to_unit_hash'}->{$5}); }
		}
	}
close(GRACE);
return @rv;
}

# default_grace()
# Returns 0 if grace time can be 0, 1 if zero grace means default
sub default_grace
{
	my $self = shift;

return 1;
}

# edit_grace_file(data, filesystem, btime, bunits, ftime, funits)
sub edit_grace_file
{
	my $self = shift;

local($rv, $line);
foreach $line (split(/\n/, $_[0])) {
	if ($line =~ /^fs (\S+) blocks time limit = ([0-9\.]+) (\S+), files time limit = ([0-9\.]+) (\S+)/ && $1 eq $_[1]) {
		# replace this line
		$line = "fs $_[1] blocks time limit = $_[2] $self->{'unit_to_name_hash'}->{$_[3]}, files time limit = $_[4] $self->{'unit_to_name_hash'}->{$_[5]}";
		}
	$rv .= "$line\n";
	}
return $rv;
}

# grace_units()
# Returns an array of possible units for grace periods
sub grace_units
{
	my $self = shift;

return ($self->{'text_hash'}->{'grace_seconds'}, $self->{'text_hash'}->{'grace_minutes'}, $self->{'text_hash'}->{'grace_hours'},
	$self->{'text_hash'}->{'grace_days'}, $self->{'text_hash'}->{'grace_weeks'}, $self->{'text_hash'}->{'grace_months'});
}

# quota_block_size(dir, device, filesystem)
# Returns the size of quota blocks on some filesystem, or undef if unknown.
# Always 1024, even though *filesystem* blocks are not!
sub quota_block_size
{
	my $self = shift;

return 1024;
}

# fs_block_size(dir, device, filesystem)
# Returns the size of blocks on some filesystem, or undef if unknown.
sub fs_block_size
{
	my $self = shift;

if ($_[2] eq "ufs") {
	local $kout = `df -k $_[0]`;
	local $bout = `df -t $_[0]`;
	if ($kout =~ /\n\Q$_[1]\E\s+(\d+)/) {
		local $ks = $1;
		if ($bout =~ /total\s*:\s*(\d+)\s+blocks/) {
			local $bs = $1;
			return ($ks*1024) / $bs;
			}
		}
	}
return 1024;
}


1;

