#!/usr/bin/perl
package rbac::RbacLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'rbac::RbacLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
#, $in{'mode'} == 1 Functions for parsing the various RBAC configuration files

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();

####################### functions for users #######################

####################### functions for profiles #######################

####################### functions for authorizations #######################

####################### functions for exec attributes #######################

####################### policy.conf functions #######################

####################### functions for projects #######################

####################### miscellaneous functions #######################

1;

1;
		return $self;
}

# list_user_attrs()
# Returns a list of user attribute objects
sub list_user_attrs
{
	my $self = shift;

if (!defined(@{$self->{'list_user_attrs_cache_array'}})) {
	@{$self->{'list_user_attrs_cache_array'}} = ( );
	local $lnum = 0;
	open(ATTR, $self->{'config_hash'}->{'user_attr'});
	while(<ATTR>) {
		s/\r|\n//g;
		s/#.*$//;
		local @w = split(/:/, $_, -1);
		if (@w == 5) {
			local $attr = { 'user' => $w[0],
					'qualifier' => $w[1],
					'res1' => $w[2],
					'res2' => $w[3],
					'type' => 'user',
					'line' => $lnum,
					'index' => scalar(@{$self->{'list_user_attrs_cache_array'}}) };
			local $a;
			foreach $a (split(/;/, $w[4])) {
				local ($an, $av) = split(/=/, $a, 2);
				$attr->{'attr'}->{$an} = $av;
				}
			push(@{$self->{'list_user_attrs_cache_array'}}, $attr);
			}
		$lnum++;
		}
	close(ATTR);
	}
return \@{$self->{'list_user_attrs_cache_array'}};
}

# create_user_attr(&attr)
# Add a new user attribute
sub create_user_attr
{
	my $self = shift;

local ($attr) = @_;
$self->list_user_attrs();	# init cache
local $lref = $self->read_file_lines($self->{'config_hash'}->{'user_attr'});
$attr->{'line'} = scalar(@$lref);
push(@$lref, $self->user_attr_line($attr));
$attr->{'index'} = scalar(@{$self->{'list_user_attrs_cache_array'}});
push(@{$self->{'list_user_attrs_cache_array'}}, $attr);
$self->flush_file_lines();
}

# modify_user_attr(&attr)
# Updates an existing user attribute in the config file
sub modify_user_attr
{
	my $self = shift;

local ($attr) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'user_attr'});
$lref->[$attr->{'line'}] = $self->user_attr_line($attr);
$self->flush_file_lines();
}

# delete_user_attr(&attr)
# Removes one user attribute entry
sub delete_user_attr
{
	my $self = shift;

local ($attr) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'user_attr'});
splice(@$lref, $attr->{'line'}, 1);
splice(@{$self->{'list_user_attrs_cache_array'}}, $attr->{'index'}, 1);
local $c;
foreach $c (@{$self->{'list_user_attrs_cache_array'}}) {
	$c->{'line'}-- if ($c->{'line'} > $attr->{'line'});
	$c->{'index'}-- if ($c->{'index'} > $attr->{'index'});
	}
$self->flush_file_lines();
}

# user_attr_line(&attr)
# Returns the text for a user attribute line
sub user_attr_line
{
	my $self = shift;

local ($attr) = @_;
local $rv = $attr->{'user'}.":".$attr->{'qualifier'}.":".$attr->{'res1'}.":".
	    $attr->{'res2'}.":";
$rv .= join(";", map { $_."=".$attr->{'attr'}->{$_} }
		     keys %{$attr->{'attr'}});
return $rv;
}

# attr_input(name, value, [type], [acl-restrict])
# Returns HTML for selecting one or more user attrs
sub attr_input
{
	my $self = shift;

local ($name, $value, $type, $restrict) = @_;
local @values = split(/,+/, $value);
local $users = $self->list_user_attrs();
local ($u, @sel);
foreach $u (@$users) {
	local $utype = $u->{'attr'}->{'type'} || "normal";
	if ((!$type || $utype eq $type) &&
	    (!$restrict || $self->can_assign_role($u))) {
		push(@sel, [ $u->{'user'} ]);
		}
	}
if (@sel) {
	return $self->ui_select($name, \@values, \@sel, 5, 1, 1);
	}
else {
	return $self->{'text_hash'}->{'attr_none'.$type};
	}
}

# attr_parse(name)
# Returns a comma-separated list of values from an attrs field
sub attr_parse
{
	my $self = shift;

return join(",", split(/\0/, $self->{'in_hash'}->{$_[0]}));
}

# $self->all_recursive_roles(username)
# Returns all roles and sub-roles for some user
sub all_recursive_roles
{
	my $self = shift;

local $users = $self->list_user_attrs();
local ($user) = grep { $_->{'user'} eq $_[0] } @$users;
local (@rv, $r);
foreach $r (split(/,/, $user->{'attr'}->{'roles'})) {
	push(@rv, $r, $self->all_recursive_roles($r));
	}
return @rv;
}

# list_prof_attrs()
# Returns a list of all profiles
sub list_prof_attrs
{
	my $self = shift;

if (!defined(@{$self->{'list_prof_attrs_cache_array'}})) {
	@{$self->{'list_prof_attrs_cache_array'}} = ( );
	local $lnum = 0;
	open(ATTR, $self->{'config_hash'}->{'prof_attr'});
	while(<ATTR>) {
		s/\r|\n//g;
		s/#.*$//;
		local @w = split(/:/, $_, -1);
		if (@w == 5) {
			local $attr = { 'name' => $w[0],
					'res1' => $w[1],
					'res2' => $w[2],
					'desc' => $w[3],
					'line' => $lnum,
					'type' => 'prof',
					'index' => scalar(@{$self->{'list_prof_attrs_cache_array'}}) };
			local $a;
			foreach $a (split(/;/, $w[4])) {
				local ($an, $av) = split(/=/, $a, 2);
				$attr->{'attr'}->{$an} = $av;
				}
			push(@{$self->{'list_prof_attrs_cache_array'}}, $attr);
			}
		$lnum++;
		}
	close(ATTR);
	$lnum++;
	}
return \@{$self->{'list_prof_attrs_cache_array'}};
}

# create_prof_attr(&attr)
# Add a new profile
sub create_prof_attr
{
	my $self = shift;

local ($attr) = @_;
$self->list_prof_attrs();	# init cache
local $lref = $self->read_file_lines($self->{'config_hash'}->{'prof_attr'});
$attr->{'line'} = scalar(@$lref);
push(@$lref, $self->prof_attr_line($attr));
$attr->{'index'} = scalar(@{$self->{'list_prof_attrs_cache_array'}});
push(@{$self->{'list_prof_attrs_cache_array'}}, $attr);
$self->flush_file_lines();
}

# modify_prof_attr(&attr)
# Updates an existing profile in the config file
sub modify_prof_attr
{
	my $self = shift;

local ($attr) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'prof_attr'});
$lref->[$attr->{'line'}] = $self->prof_attr_line($attr);
$self->flush_file_lines();
}

# delete_prof_attr(&attr)
# Removes one profile
sub delete_prof_attr
{
	my $self = shift;

local ($attr) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'prof_attr'});
splice(@$lref, $attr->{'line'}, 1);
splice(@{$self->{'list_prof_attrs_cache_array'}}, $attr->{'index'}, 1);
local $c;
foreach $c (@{$self->{'list_prof_attrs_cache_array'}}) {
	$c->{'line'}-- if ($c->{'line'} > $attr->{'line'});
	$c->{'index'}-- if ($c->{'index'} > $attr->{'index'});
	}
$self->flush_file_lines();
}

# prof_attr_line(&attr)
# Returns the text for a prof attribute line
sub prof_attr_line
{
	my $self = shift;

local ($attr) = @_;
local $rv = $attr->{'name'}.":".$attr->{'res1'}.":".$attr->{'res2'}.":".
	    $attr->{'desc'}.":";
$rv .= join(";", map { $_."=".$attr->{'attr'}->{$_} }
		     keys %{$attr->{'attr'}});
return $rv;
}

# profiles_input(name, comma-sep-value, acl-restrict)
# Returns HTML for a field for selecting multiple profiles
sub profiles_input
{
	my $self = shift;

local ($name, $value, $restrict) = @_;
local @values = split(/,/, $value);
local $profs = $self->list_prof_attrs();
local @canprofs = $restrict ? grep { $self->can_assign_profile($_) } @$profs
			    : @$profs;
if (@canprofs) {
	return $self->ui_select($name, \@values,
			  [ map { [ $_->{'name'}, "$_->{'name'} ($_->{'desc'})" ] }
				sort { $a->{'name'} cmp $b->{'name'} }
				@canprofs ], 10, 1, 1);
	}
else {
	return $self->{'text_hash'}->{'prof_none'};
	}
}

# profiles_parse(name)
# Returns a comma-separated list of values from a profiles field
sub profiles_parse
{
	my $self = shift;

return join(",", split(/\0/, $self->{'in_hash'}->{$_[0]}));
}

# $self->all_recursive_profs(username|profilename)
# Returns all profiles and sub-profiles for some user
sub all_recursive_profs
{
	my $self = shift;

local $users = $self->list_user_attrs();
local ($user) = grep { $_->{'user'} eq $_[0] } @$users;
local $profs = $self->list_prof_attrs();
local ($prof) = grep { $_->{'name'} eq $_[0] } @$profs;
local (@rv, $r);
if ($user) {
	# Return all profiles from roles, direct profiles and sub-profiles
	foreach $r (split(/,/, $user->{'attr'}->{'roles'})) {
		push(@rv, $self->all_recursive_profs($r));
		}
	foreach $r (split(/,/, $user->{'attr'}->{'profiles'})) {
		push(@rv, $r, $self->all_recursive_profs($r));
		}
	}
elsif ($prof) {
	# Return all sub-profiles
	foreach $r (split(/,/, $prof->{'attr'}->{'profs'})) {
		push(@rv, $self->all_recursive_profs($r));
		}
	}
return @rv;
}

# list_auth_attrs()
# Returns a user of all authorizations
sub list_auth_attrs
{
	my $self = shift;

if (!defined(@{$self->{'list_auth_attrs_cache_array'}})) {
	@{$self->{'list_auth_attrs_cache_array'}} = ( );
	local $lnum = 0;
	open(ATTR, $self->{'config_hash'}->{'auth_attr'});
	while(<ATTR>) {
		s/\r|\n//g;
		s/#.*$//;
		local @w = split(/:/, $_, -1);
		if (@w == 6) {
			local $attr = { 'name' => $w[0],
					'res1' => $w[1],
					'res2' => $w[2],
					'short' => $w[3],
					'desc' => $w[4],
					'line' => $lnum,
					'type' => 'auth',
					'index' => scalar(@{$self->{'list_auth_attrs_cache_array'}}) };
			local $a;
			foreach $a (split(/;/, $w[5])) {
				local ($an, $av) = split(/=/, $a, 2);
				$attr->{'attr'}->{$an} = $av;
				}
			push(@{$self->{'list_auth_attrs_cache_array'}}, $attr);
			}
		$lnum++;
		}
	close(ATTR);
	$lnum++;
	}
return \@{$self->{'list_auth_attrs_cache_array'}};
}

# create_auth_attr(&attr)
# Add a new authorization
sub create_auth_attr
{
	my $self = shift;

local ($attr) = @_;
$self->list_auth_attrs();	# init cache
local $lref = $self->read_file_lines($self->{'config_hash'}->{'auth_attr'});
$attr->{'line'} = scalar(@$lref);
push(@$lref, $self->auth_attr_line($attr));
$attr->{'index'} = scalar(@{$self->{'list_auth_attrs_cache_array'}});
push(@{$self->{'list_auth_attrs_cache_array'}}, $attr);
$self->flush_file_lines();
}

# modify_auth_attr(&attr)
# Updates an existing authorization in the config file
sub modify_auth_attr
{
	my $self = shift;

local ($attr) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'auth_attr'});
$lref->[$attr->{'line'}] = $self->auth_attr_line($attr);
$self->flush_file_lines();
}

# delete_auth_attr(&attr)
# Removes one authorization
sub delete_auth_attr
{
	my $self = shift;

local ($attr) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'auth_attr'});
splice(@$lref, $attr->{'line'}, 1);
splice(@{$self->{'list_auth_attrs_cache_array'}}, $attr->{'index'}, 1);
local $c;
foreach $c (@{$self->{'list_auth_attrs_cache_array'}}) {
	$c->{'line'}-- if ($c->{'line'} > $attr->{'line'});
	$c->{'index'}-- if ($c->{'index'} > $attr->{'index'});
	}
$self->flush_file_lines();
}

# auth_attr_line(&attr)
# Returns the text for a auth attribute line
sub auth_attr_line
{
	my $self = shift;

local ($attr) = @_;
local $rv = $attr->{'name'}.":".$attr->{'res1'}.":".$attr->{'res2'}.":".
	    $attr->{'short'}.":".$attr->{'desc'}.":";
$rv .= join(";", map { $_."=".$attr->{'attr'}->{$_} }
		     keys %{$attr->{'attr'}});
return $rv;
}

# auths_input(name, value)
# Returns HTML for a text area for entering and choosing authorizations
sub auths_input
{
	my $self = shift;

local ($name, $value) = @_;
return "<table cellpadding=0 cellspacing=0><tr><td>".
       $self->ui_textarea($name, join("\n", split(/,/, $value)), 3, 50).
       "</td><td valign=top>&nbsp;".
       $self->auth_chooser($name).
       "</td></tr></table>";
}

# auth_chooser(field)
# Returns HTML for a button that pops up an authorization chooser window
sub auth_chooser
{
	my $self = shift;

return "<input type=button onClick='ifield = form.$_[0]; chooser = window.open(\"auth_chooser.cgi\", \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=500,height=400\"); chooser.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
}

# auths_parse(name)
# Returns a comma-separated list of auths
sub auths_parse
{
	my $self = shift;

local @auths = split(/[\r\n]+/, $self->{'in_hash'}->{$_[0]});
local $a;
foreach $a (@auths) {
	$a =~ /^\S+$/ || $self->error($self->text('user_eauth', $a));
	}
return join(",", @auths);
}

# list_exec_attrs()
# Returns a user of all execorizations
sub list_exec_attrs
{
	my $self = shift;

if (!defined(@{$self->{'list_exec_attrs_cache_array'}})) {
	@{$self->{'list_exec_attrs_cache_array'}} = ( );
	local $lnum = 0;
	open(ATTR, $self->{'config_hash'}->{'exec_attr'});
	while(<ATTR>) {
		s/\r|\n//g;
		s/#.*$//;
		local @w = split(/:/, $_, -1);
		if (@w == 7) {
			local $attr = { 'name' => $w[0],
					'policy' => $w[1],
					'cmd' => $w[2],
					'res1' => $w[3],
					'res2' => $w[4],
					'id' => $w[5],
					'type' => 'exec',
					'index' => scalar(@{$self->{'list_exec_attrs_cache_array'}}) };
			local $a;
			foreach $a (split(/;/, $w[6])) {
				local ($an, $av) = split(/=/, $a, 2);
				$attr->{'attr'}->{$an} = $av;
				}
			push(@{$self->{'list_exec_attrs_cache_array'}}, $attr);
			}
		$lnum++;
		}
	close(ATTR);
	$lnum++;
	}
return \@{$self->{'list_exec_attrs_cache_array'}};
}

# create_exec_attr(&attr)
# Add a new execorization
sub create_exec_attr
{
	my $self = shift;

local ($attr) = @_;
$self->list_exec_attrs();	# init cache
local $lref = $self->read_file_lines($self->{'config_hash'}->{'exec_attr'});
$attr->{'line'} = scalar(@$lref);
push(@$lref, $self->exec_attr_line($attr));
$attr->{'index'} = scalar(@{$self->{'list_exec_attrs_cache_array'}});
push(@{$self->{'list_exec_attrs_cache_array'}}, $attr);
$self->flush_file_lines();
}

# modify_exec_attr(&attr)
# Updates an existing execorization in the config file
sub modify_exec_attr
{
	my $self = shift;

local ($attr) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'exec_attr'});
$lref->[$attr->{'line'}] = $self->exec_attr_line($attr);
$self->flush_file_lines();
}

# delete_exec_attr(&attr)
# Removes one execorization
sub delete_exec_attr
{
	my $self = shift;

local ($attr) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'exec_attr'});
splice(@$lref, $attr->{'line'}, 1);
splice(@{$self->{'list_exec_attrs_cache_array'}}, $attr->{'index'}, 1);
local $c;
foreach $c (@{$self->{'list_exec_attrs_cache_array'}}) {
	$c->{'line'}-- if ($c->{'line'} > $attr->{'line'});
	$c->{'index'}-- if ($c->{'index'} > $attr->{'index'});
	}
$self->flush_file_lines();
}

# exec_attr_line(&attr)
# Returns the text for a exec attribute line
sub exec_attr_line
{
	my $self = shift;

local ($attr) = @_;
local $rv = $attr->{'name'}.":".$attr->{'policy'}.":".$attr->{'cmd'}.":".
	    $attr->{'res1'}.":".$attr->{'res2'}.":".$attr->{'id'}.":";
$rv .= join(";", map { $_."=".$attr->{'attr'}->{$_} }
		     keys %{$attr->{'attr'}});
return $rv;
}

# get_policy_config()
# Returns a list of policy config file directives
sub get_policy_config
{
	my $self = shift;

if (!defined(@{$self->{'policy_conf_cache_array'}})) {
	@{$self->{'policy_conf_cache_array'}} = ( );
	local $lnum = 0;
	open(ATTR, $self->{'config_hash'}->{'policy_conf'});
	while(<ATTR>) {
		s/\r|\n//g;
		s/\s+$//;
		if (/^\s*(#*)\s*([^= ]+)\s*=\s*(\S.*)$/) {
			local $pol = { 'name' => $2,
				       'value' => $3,
				       'enabled' => !$1,
				       'line' => $lnum,
				       'index' => scalar(@{$self->{'policy_conf_cache_array'}}) };
			push(@{$self->{'policy_conf_cache_array'}}, $pol);
			}
		$lnum++;
		}
	close(ATTR);
	$lnum++;
	}
return \@{$self->{'policy_conf_cache_array'}};
}

# find_policy(name, &conf, [enabled])
sub find_policy
{
	my $self = shift;

local ($name, $conf, $ena) = @_;
local ($rv) = grep { lc($_->{'name'}) eq lc($name) &&
		     (!defined($ena) ||
		      defined($ena) && $ena == $_->{'enabled'}) } @$conf;
return $rv;
}

# find_policy_value(name, &conf);
sub find_policy_value
{
	my $self = shift;

local ($name, $conf) = @_;
local $rv = $self->find_policy($name, $conf, 1);
return $rv ? $rv->{'value'} : undef;
}

# save_policy(&conf, name, [value])
# Update or delete an entry in policy.conf
sub save_policy
{
	my $self = shift;

local ($conf, $name, $value) = @_;
local $old = $self->find_policy($name, $conf);
local $lref = $self->read_file_lines($self->{'config_hash'}->{'policy_conf'});
if (!$old && $value) {
	# Need to add
	push(@$lref, "$name=$value");
	push(@$conf, { 'name' => $name,
		       'value' => $value,
		       'index' => scalar(@$conf),
		       'line' => scalar(@$lref) - 1 });
	}
elsif ($old && $value) {
	# Need to update (and perhaps comment in)
	$old->{'value'} = $value;
	$old->{'enabled'} = 1;
	$lref->[$old->{'line'}] = "$name=$value";
	}
elsif ($old && $old->{'enabled'} && !$value) {
	# Need to comment out
	$old->{'enabled'} = 0;
	$lref->[$old->{'line'}] = "#$name=$old->{'value'}";
	}
}

# list_projects()
# Returns a list of project objects
sub list_projects
{
	my $self = shift;

if (!defined(@{$self->{'list_projects_cache_array'}})) {
	@{$self->{'list_projects_cache_array'}} = ( );
	local $lnum = 0;
	open(ATTR, $self->{'config_hash'}->{'project'});
	while(<ATTR>) {
		s/\r|\n//g;
		s/#.*$//;
		local @w = split(/:/, $_, -1);
		if (@w >= 2) {
			local $attr = { 'name' => $w[0],
					'id' => $w[1],
					'desc' => $w[2],
					'users' => $w[3],
					'groups' => $w[4],
					'line' => $lnum,
					'type' => 'project',
					'index' => scalar(@{$self->{'list_projects_cache_array'}}) };
			local $a;
			foreach $a (split(/;/, $w[5])) {
				local ($an, $av) = split(/=/, $a, 2);
				$attr->{'attr'}->{$an} = $av;
				}
			push(@{$self->{'list_projects_cache_array'}}, $attr);
			}
		$lnum++;
		}
	close(ATTR);
	}
return \@{$self->{'list_projects_cache_array'}};
}

# create_project(&attr)
# Add a new project
sub create_project
{
	my $self = shift;

local ($attr) = @_;
$self->list_projects();	# init cache
local $lref = $self->read_file_lines($self->{'config_hash'}->{'project'});
$attr->{'line'} = scalar(@$lref);
push(@$lref, $self->project_line($attr));
$attr->{'index'} = scalar(@{$self->{'list_projects_cache_array'}});
push(@{$self->{'list_projects_cache_array'}}, $attr);
$self->flush_file_lines();
}

# modify_project(&attr)
# Updates an existing project in the config file
sub modify_project
{
	my $self = shift;

local ($attr) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'project'});
$lref->[$attr->{'line'}] = $self->project_line($attr);
$self->flush_file_lines();
}

# delete_project(&attr)
# Removes one project entry
sub delete_project
{
	my $self = shift;

local ($attr) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'project'});
splice(@$lref, $attr->{'line'}, 1);
splice(@{$self->{'list_projects_cache_array'}}, $attr->{'index'}, 1);
local $c;
foreach $c (@{$self->{'list_projects_cache_array'}}) {
	$c->{'line'}-- if ($c->{'line'} > $attr->{'line'});
	$c->{'index'}-- if ($c->{'index'} > $attr->{'index'});
	}
$self->flush_file_lines();
}

# project_line(&attr)
# Returns the text for a project file line
sub project_line
{
	my $self = shift;

local ($attr) = @_;
local $rv = $attr->{'name'}.":".$attr->{'id'}.":".$attr->{'desc'}.":".
	    $attr->{'users'}.":".$attr->{'groups'}.":";
$rv .= join(";", map { defined($attr->{'attr'}->{$_}) ?
			$_."=".$attr->{'attr'}->{$_} : $_ }
		     keys %{$attr->{'attr'}});
return $rv;
}

# project_input(name, value)
# Returns HTML for selecting one project
sub project_input
{
	my $self = shift;

local ($name, $value) = @_;
local $projects = $self->list_projects();
return $self->ui_select($name, $value,
		  [ map { [ $_->{'name'},
			    $_->{'name'}.($_->{'desc'} ? " ($_->{'desc'})"
						       : "") ] } @$projects ],
		  0, 0, $value ? 1 : 0);
}

# project_members_input(name, comma-separated-members)
# Returns HTML for selecting users or groups in a project. These may be all,
# none, all except or only some
sub project_members_input
{
	my $self = shift;

local ($name, $users) = @_;
local @users = split(/,/, $users);
local %users = map { $_, 1 } @users;
local (@canusers, @cannotuser, $mode);
if ($users{'*'} && @users == 1) {
	$mode = 0;
	}
elsif (@users == 0 || $users{'!*'} && @users == 1) {
	$mode = 1;
	}
elsif ($users{'*'}) {
	# All except some
	$mode = 3;
	@{$self->{'cannotusers_array'}} = map { /^\!(.*)/; $1 } grep { /^\!/ } @users[1..$#users];
	}
elsif ($users{'!*'}) {
	# Only some
	$mode = 2;
	@canusers = grep { !/^\!/ } @users[1..$#users];
	}
else {
	# Only listed
	$mode = 2;
	@canusers = @users;
	}
local $cb = $name =~ /user/ ? 'user_chooser_button' : 'group_chooser_button';
return $self->ui_radio($name."_mode", $mode,
	[ [ 0, $self->{'text_hash'}->{'projects_all'.$name} ],
	  [ 1, $self->{'text_hash'}->{'projects_none'.$name}."<br>" ],
	  [ 2, $self->text('projects_only'.$name,
	     $self->ui_textbox($name."_can", join(" ", @canusers), 40)." ".
	     $self->$cb($name."_can", 1))."<br>" ],
	  [ 3, $self->text('projects_except'.$name,
	     $self->ui_textbox($name."_cannot", join(" ", @{$self->{'cannotusers_array'}}), 40)." ".
	     $self->$cb($name."_cannot", 1)) ]
	]);
}

# parse_project_members(name)
sub parse_project_members
{
	my $self = shift;

local ($name) = @_;
if ($self->{'in_hash'}->{$name."_mode"} == 0) {
	return "*";
	}
elsif ($self->{'in_hash'}->{$name."_mode"} == 1) {
	return "";
	}
elsif ($self->{'in_hash'}->{$name."_mode"} == 2) {
	$self->{'in_hash'}->{$name."_can"} || $self->error($self->{'text_hash'}->{'project_e'.$name.'can'});
	return join(",", split(/\s+/, $self->{'in_hash'}->{$name."_can"}));
	}
elsif ($self->{'in_hash'}->{$name."_mode"} == 3) {
	$self->{'in_hash'}->{$name."_cannot"} || $self->error($self->{'text_hash'}->{'project_e'.$name.'cannot'});
	return join(",", "*", map { "!$_" } split(/\s+/, $self->{'in_hash'}->{$name."_cannot"}));
	}
}

# nice_comma_list(comma-separated-value)
# Nicely formats and shortens a comma-separated string
sub nice_comma_list
{
	my $self = shift;

local @l = split(/,/, $_[0]);
if (@l > 2) {
	return join(" , ", @l[0..1], "...");
	}
else {
	return join(" , ", @l);
	}
}

# rbac_help_file(&object)
sub rbac_help_file
{
	my $self = shift;

local $hf = $self->{'config_hash'}->{$_[0]->{'type'}.'_help_dir'}."/$_[0]->{'attr'}->{'help'}";
return -r $hf && !-d $hf ? $hf : undef;
}

# rbac_help_link(&object, desc)
sub rbac_help_link
{
	my $self = shift;

local $hf = $self->rbac_help_file($_[0]);
local $rv = "<table cellpadding=0 cellspacing=0 width=100%><tr><td>$_[1]</td>";
if ($hf) {
	$hf = $self->urlize($hf);
	$rv .= "<td align=right><a onClick='window.open(\"rbac_help.cgi?help=$hf\", \"help\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300,resizable=yes\"); return false' href=\"rbac_help.cgi?help=$hf\"><img src=images/help.gif border=0></a></td>";
	}
$rv .= "</tr></table>";
return $rv;
}

# rbac_config_files()
# Returns a list of all config files managed by this module
sub rbac_config_files
{
	my $self = shift;

return map { $self->{'config_hash'}->{$_} } ('user_attr', 'prof_attr', 'auth_attr',
			    'exec_attr', 'policy_conf', 'project');
}

# missing_rbac_config_file()
# Returns the path to any missing config file
sub missing_rbac_config_file
{
	my $self = shift;

foreach $c ($self->rbac_config_files()) {
	return $c if (!-r $c);
	}
return undef;
}

# lock_rbac_files()
# Lock all config files used by RBAC
sub lock_rbac_files
{
	my $self = shift;

local $c;
foreach $c ($self->rbac_config_files()) {
	$self->lock_file($c);
	}
}

# unlock_rbac_files()
# Unlock all config files used by RBAC
sub unlock_rbac_files
{
	my $self = shift;

local $c;
foreach $c ($self->rbac_config_files()) {
	$self->unlock_file($c);
	}
}

# list_crypt_algorithms()
# Returns 1 list of all encryption algorithms, including the internal __unix__
sub list_crypt_algorithms
{
	my $self = shift;

if (!defined(@{$self->{'list_crypt_algorithms_cache_array'}})) {
	push(@{$self->{'list_crypt_algorithms_cache_array'}}, { 'name' => '__unix__' } );
	local $lnum = 0;
	open(CRYPT, $self->{'config_hash'}->{'crypt_conf'});
	while(<CRYPT>) {
		s/\r|\n//g;
		s/#.*$//;
		if (/^\s*(\S+)\s+(\S+)/) {
			push(@{$self->{'list_crypt_algorithms_cache_array'}},
			   { 'name' => $1,
			     'lib' => $2,
			     'line' => $lnum,
			     'index' => scalar(@{$self->{'list_crypt_algorithms_cache_array'}}) });
			}
		$lnum++;
		}
	close(CRYPT);
	}
return \@{$self->{'list_crypt_algorithms_cache_array'}};
}

# crypt_algorithms_input(name, value, multiple)
# Returns HTML for selecting one or many crypt algorithms
sub crypt_algorithms_input
{
	my $self = shift;

local ($name, $value, $multiple) = @_;
local @values = split(/,/, $value);
local $crypts = $self->list_crypt_algorithms();
return $self->ui_select($name, \@values,
		  [ map { [ $_->{'name'}, $self->{'text_hash'}->{'crypt_'.$_->{'name'}} ] }
			@$crypts ], $multiple ? scalar(@$crypts) : undef,
			$multiple, 1);
}

# can_edit_user(&user)
# Returns 1 if some user can be edited
sub can_edit_user
{
	my $self = shift;

local ($user) = @_;
return 0 if ($user->{'attr'}->{'type'} eq 'role' && !$self->{'access_hash'}->{'roles'});
return 0 if ($user->{'attr'}->{'type'} ne 'role' && !$self->{'access_hash'}->{'users'});
return 1;
}

# can_assign_role(&role|rolename)
# Returns 1 if some role can be assigned
sub can_assign_role
{
	my $self = shift;

local ($role) = @_;
local $rolename = ref($role) ? $role->{'user'} : $role;
if ($self->{'access_hash'}->{'roleassign'} eq '*') {
	return 1;
	}
else {
	local @canroles;
	if ($self->{'access_hash'}->{'roleassign'} eq 'x') {
		# Work out Webmin user's roles
		@canroles = $self->all_recursive_roles($self->{'remote_user'});
		}
	else {
		@canroles = split(/,/, $self->{'access_hash'}->{'roleassign'});
		}
	return $self->indexof($rolename, @canroles) != -1;
	}
}

# can_assign_profile(&profile|profilename)
# Returns 1 if some profile can be assigned
sub can_assign_profile
{
	my $self = shift;

local ($prof) = @_;
local $profname = ref($prof) ? $prof->{'name'} : $prof;
if ($self->{'access_hash'}->{'profassign'} eq '*') {
	return 1;
	}
else {
	local @canprofs;
	if ($self->{'access_hash'}->{'profassign'} eq 'x') {
		# Work out Webmin user's profs
		@canprofs = $self->all_recursive_profs($self->{'remote_user'});
		}
	else {
		@canprofs = split(/,/, $self->{'access_hash'}->{'profassign'});
		}
	return $self->indexof($profname, @canprofs) != -1;
	}
}

# list_rctls()
# Returns a list of possible resource control names
sub list_rctls
{
	my $self = shift;

local @rv;
open(RCTL, "rctladm -l |");
while(<RCTL>) {
	if (/^(\S+)\s+(\S+)=(\S+)/) {
		push(@rv, $1);
		}
	}
close(RCTL);
return @rv;
}

# list_rctl_signals()
# Returns a list of allowed signals for rctl actions
sub list_rctl_signals
{
	my $self = shift;

return ( [ "SIGABRT", "Abort the process" ],
	 [ "SIGHUP", "Send a hangup signal" ],
	 [ "SIGTERM", "Terminate the process" ],
	 [ "SIGKILL", "Kill the process" ],
	 [ "SIGSTOP", "Stop the process" ],
	 [ "SIGXRES", "Resource control limit exceeded" ],
	 [ "SIGXFSZ", "File size limit exceeded" ],
	 [ "SIGXCPU", "CPU time limit exceeded" ] );
}

# list_resource_controls(type, id)
# Returns a list of resource controls for some project, zone or process
sub list_resource_controls
{
	my $self = shift;

local ($type, $id) = @_;
$self->open_execute_command(*PRCTL,
		      "prctl -i ".quotemeta($type)." ".quotemeta($id), 1);
local (@rv, $res);
while(<PRCTL>) {
	s/\r|\n//g;
	next if (/^NAME/);	# skip header
	if (/^(\S+)\s*$/) {
		# Start of a new resource
		$res = $1;
		}
	elsif (/^\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/ && $res) {
		# A limit within a resource
		push(@rv, { 'res' => $res,
			    'priv' => $1,
			    'limit' => $2,
			    'flag' => $3 eq "-" ? undef : $3,
			    'action' => $4 });
		}
	}
close(PRCTL);
return @rv;
}


1;

