#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Create, update or delete one RBAC profile

&ModuleLoader::preload_module('rbac_class', "rbac::RbacLib");
$self = new rbac::RbacLib:: 'rbac_class',%$self;

$self->ReadParse();
$self->{'access_hash'}->{'profs'} == 1 || $self->error($self->{'text_hash'}->{'profs_ecannot'});
$self->error_setup($self->{'text_hash'}->{'prof_err'});

$self->lock_rbac_files();
$self->{'profs'} = $self->list_prof_attrs();
if (!$self->{'in_hash'}->{'new'}) {
	$self->{'prof'} = $self->{'profs'}->[$self->{'in_hash'}->{'idx'}];
	$self->{'logname'} = $self->{'prof'}->{'name'};
	}
else {
	$self->{'prof'} = { 'attr' => { } };
	$self->{'logname'} = $self->{'in_hash'}->{'name'};
	}

if (!$self->{'in_hash'}->{'new'}) {
	# Find users of this profile
	$self->{'users'} = $self->list_user_attrs();
	foreach $u (@{$self->{'users'}}) {
		local @profiles =
		    split(/,/, $u->{'attr'}->{'profiles'});
		$self->{'idx'} = $self->indexof($self->{'logname'}, @profiles);
		if ($self->{'idx'} >= 0) {
			push(@{$self->{'profusers_array'}}, [ $u, $self->{'idx'}, \@profiles ]);
			}
		}
	foreach $p (@{$self->{'profs'}}) {
		local @profiles =
		    split(/,/, $p->{'attr'}->{'profs'});
		$self->{'idx'} = $self->indexof($self->{'logname'}, @profiles);
		if ($self->{'idx'} >= 0) {
			push(@{$self->{'profprofs_array'}}, [ $p, $self->{'idx'}, \@profiles ]);
			}
		}
	$self->{'execs'} = $self->list_exec_attrs();
	foreach $e (@{$self->{'execs'}}) {
		if ($e->{'name'} eq $self->{'logname'}) {
			push(@{$self->{'profexecs_array'}}, [ $e ]);
			}
		}
	}

if ($self->{'in_hash'}->{'delete'}) {
	# Just delete this prof
	@{$self->{'profusers_array'}} && $self->error($self->text('prof_einuseu',
				   $self->{'profusers_array'}->[0]->[0]->{'user'}));
	@{$self->{'profprofs_array'}} && $self->error($self->text('prof_einusep',
				   $self->{'profprofs_array'}->[0]->[0]->{'name'}));
	@{$self->{'profexecs_array'}} && $self->error($self->text('prof_einusee', scalar(@{$self->{'profexecs_array'}})));
	$self->delete_prof_attr($self->{'prof'});
	}
else {
	# Check for clash
	if ($self->{'in_hash'}->{'new'} || $self->{'logname'} ne $self->{'in_hash'}->{'name'}) {
		($self->{'clash'}) = grep { $_->{'name'} eq $self->{'in_hash'}->{'name'} } @{$self->{'profs'}};
		$self->{'clash'} && $self->error($self->{'text_hash'}->{'prof_eclash'});
		}

	# Validate and store inputs
	$self->{'in_hash'}->{'name'} =~ /^[^:,]+$/ || $self->error($self->{'text_hash'}->{'prof_ename'});
	$self->{'prof'}->{'name'} = $self->{'in_hash'}->{'name'};
	$self->{'in_hash'}->{'desc'} =~ /^[^:]*$/ || $self->error($self->{'text_hash'}->{'prof_edesc'});
	$self->{'prof'}->{'desc'} = $self->{'in_hash'}->{'desc'};
	$self->{'profiles'} = $self->profiles_parse("profiles");
	if ($self->{'profiles'}) {
		@profiles = split(/,/, $self->{'profiles'});
		$self->indexof($self->{'in_hash'}->{'name'}, @profiles) < 0 ||
			$self->error($self->{'text_hash'}->{'prof_esub'});
		$self->{'prof'}->{'attr'}->{'profs'} = $self->{'profiles'};
		}
	else {
		delete($self->{'prof'}->{'attr'}->{'profs'});
		}
	$self->{'auths'} = $self->auths_parse("auths");
	if ($self->{'auths'}) {
		$self->{'prof'}->{'attr'}->{'auths'} = $self->{'auths'};
		}
	else {
		delete($self->{'prof'}->{'attr'}->{'auths'});
		}

	# Save or update profile
	if ($self->{'in_hash'}->{'new'}) {
		$self->create_prof_attr($self->{'prof'});
		}
	else {
		$self->modify_prof_attr($self->{'prof'});

		# Update other users of this profile, if renamed
		if ($self->{'logname'} ne $self->{'in_hash'}->{'name'}) {
			foreach $pu (@{$self->{'profusers_array'}}) {
				$pu->[2]->[$pu->[1]] = $self->{'in_hash'}->{'name'};
				$pu->[0]->{'attr'}->{'profiles'} =
					join(",", @{$pu->[2]});
				$self->modify_user_attr($pu->[0]);
				}
			foreach $pp (@{$self->{'profprofs_array'}}) {
				$pp->[2]->[$pp->[1]] = $self->{'in_hash'}->{'name'};
				$pp->[0]->{'attr'}->{'profiles'} =
					join(",", @{$pp->[2]});
				$self->modify_prof_attr($pp->[0]);
				}
			foreach $pe (@{$self->{'profexecs_array'}}) {
				$pe->[0]->{'name'} = $self->{'in_hash'}->{'name'};
				$self->modify_exec_attr($pe->[0]);
				}
			}
		}
	}

$self->unlock_rbac_files();
$self->webmin_log($self->{'in_hash'}->{'delete'} ? "delete" : $self->{'in_hash'}->{'new'} ? "create" : "modify",
	    "prof", $self->{'logname'}, $self->{'prof'});
$self->redirect("list_profs.cgi");

1;
		
}


1;

