#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Create, update or delete one project

&ModuleLoader::preload_module('rbac_class', "rbac::RbacLib");
$self = new rbac::RbacLib:: 'rbac_class',%$self;

$self->{'access_hash'}->{'projects'} || $self->error($self->{'text_hash'}->{'projects_ecannot'});
$self->ReadParse();
$self->error_setup($self->{'text_hash'}->{'project_err'});

$self->lock_rbac_files();
$self->{'projects'} = $self->list_projects();
if (!$self->{'in_hash'}->{'new'}) {
	$self->{'project'} = $self->{'projects'}->[$self->{'in_hash'}->{'idx'}];
	$self->{'logname'} = $self->{'project'}->{'name'};
	}
else {
	$self->{'project'} = { 'attr' => { } };
	$self->{'logname'} = $self->{'in_hash'}->{'name'};
	}

if ($self->{'in_hash'}->{'delete'}) {
	# Just delete this project
	$self->delete_project($self->{'project'});
	}
else {
	# Check for clash
	if ($self->{'in_hash'}->{'new'} || $self->{'logname'} ne $self->{'in_hash'}->{'name'}) {
		($self->{'clash'}) = grep { $_->{'name'} eq $self->{'in_hash'}->{'name'} } @{$self->{'projects'}};
		$self->{'clash'} && $self->error($self->{'text_hash'}->{'project_eclash'});
		}

	# Validate and store inputs
	$self->{'in_hash'}->{'name'} =~ /^[^: ]+$/ || $self->error($self->{'text_hash'}->{'project_ename'});
	$self->{'project'}->{'name'} = $self->{'in_hash'}->{'name'};
	$self->{'in_hash'}->{'id'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'project_eid'});
	if ($self->{'in_hash'}->{'new'} || $self->{'in_hash'}->{'id'} != $self->{'project'}->{'id'}) {
		($self->{'clash'}) = grep { $_->{'id'} == $self->{'in_hash'}->{'id'} } @{$self->{'projects'}};
		$self->{'clash'} && $self->error($self->{'text_hash'}->{'project_eidclash'});
		}
	$self->{'project'}->{'id'} = $self->{'in_hash'}->{'id'};
	$self->{'in_hash'}->{'desc'} =~ /^[^:]*$/ || $self->error($self->{'text_hash'}->{'project_edesc'});
	$self->{'project'}->{'desc'} = $self->{'in_hash'}->{'desc'};
	$self->{'project'}->{'users'} = $self->parse_project_members("users");
	$self->{'project'}->{'groups'} = $self->parse_project_members("groups");

	# Validate resources, and build list of selected rctls
	for($i=0; defined($rctl = $self->{'in_hash'}->{"rctl_$i"}); $i++) {
		next if (!$rctl);
		$self->{'priv'} = $self->{'in_hash'}->{"priv_$i"};
		$self->{'limit'} = $self->{'in_hash'}->{"limit_$i"};
		$self->{'action'} = $self->{'in_hash'}->{"action_$i"};
		if ($self->{'rctls_hash'}->{$rctl}) {
			$self->{'rctls_hash'}->{$rctl} .= ",";
			}
		if ($self->{'priv'}) {
			$self->{'limit'} =~ /^\d+/ ||
				$self->error($self->text('project_elimit', $i+1));
			$self->{'rctls_hash'}->{$rctl} .= "($self->{'priv'},$self->{'limit'},$self->{'action'})";
			}
		else {
			$self->{'rctls_hash'}->{$rctl} = undef;
			}
		}
	foreach $k (keys %{$self->{'project'}->{'attr'}}) {
		delete($self->{'project'}->{'attr'}->{$k}) if ($k =~ /\./);
		}
	foreach $rctl (keys %{$self->{'rctls_hash'}}) {
		$self->{'project'}->{'attr'}->{$rctl} = $self->{'rctls_hash'}->{$rctl};
		}

	# Save pool and max-rss attributes
	delete($self->{'project'}->{'attr'}->{'project.pool'});
	if (!$self->{'in_hash'}->{'pool_def'}) {
		$self->{'in_hash'}->{'pool'} =~ /^[a-z0-9\.\-]+$/i ||
			$self->error($self->{'text_hash'}->{'project_epool'});
		$self->{'project'}->{'attr'}->{'project.pool'} = $self->{'in_hash'}->{'pool'};
		}
	delete($self->{'project'}->{'attr'}->{'rcap.max-rss'});
	if (!$self->{'in_hash'}->{'maxrss_def'}) {
		$self->{'in_hash'}->{'maxrss'} =~ /^\d+$/ ||
			$self->error($self->{'text_hash'}->{'project_emaxrss'});
		$self->{'project'}->{'attr'}->{'rcap.max-rss'} =
			$self->{'in_hash'}->{'maxrss'}*$self->{'in_hash'}->{'maxrss_units'};
		}

	# Save or update project
	if ($self->{'in_hash'}->{'new'}) {
		$self->create_project($self->{'project'});
		}
	else {
		$self->modify_project($self->{'project'});
		}
	}

$self->unlock_rbac_files();
$self->webmin_log($self->{'in_hash'}->{'delete'} ? "delete" : $self->{'in_hash'}->{'new'} ? "create" : "modify",
	    "project", $self->{'logname'}, $self->{'project'});
$self->redirect("list_projects.cgi");

1;
		
}


1;

