#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Create, update or delete one RBAC user

&ModuleLoader::preload_module('rbac_class', "rbac::RbacLib");
$self = new rbac::RbacLib:: 'rbac_class',%$self;

$self->ReadParse();
$self->error_setup($self->{'text_hash'}->{'user_err'});

$self->lock_rbac_files();
$self->{'users'} = $self->list_user_attrs();
if (!$self->{'in_hash'}->{'new'}) {
	$self->{'user'} = $self->{'users'}->[$self->{'in_hash'}->{'idx'}];
	$self->can_edit_user($self->{'user'}) || $self->error($self->{'text_hash'}->{'user_ecannot'});
	$self->{'loguser'} = $self->{'user'}->{'user'};
	@{$self->{'oldroles_array'}} = split(/,/, $self->{'user'}->{'attr'}->{'roles'});
	@{$self->{'oldprofs_array'}} = split(/,/, $self->{'user'}->{'attr'}->{'profiles'});
	}
else {
	$self->{'access_hash'}->{'users'} || $self->{'access_hash'}->{'roles'} || $self->error($self->{'text_hash'}->{'user_ecannot'});
	$self->{'user'} = { 'attr' => { } };
	$self->{'loguser'} = $self->{'in_hash'}->{'user'};
	}

if (!$self->{'in_hash'}->{'new'}) {
	# Find users of this role
	foreach $u (@{$self->{'users'}}) {
		local @roles =
		    split(/,/, $u->{'attr'}->{'roles'});
		$self->{'idx'} = $self->indexof($self->{'loguser'}, @roles);
		if ($self->{'idx'} >= 0) {
			push(@{$self->{'roleusers_array'}}, [ $u, $self->{'idx'}, \@roles ]);
			}
		}
	}

if ($self->{'in_hash'}->{'delete'}) {
	# Just delete this user
	@{$self->{'roleusers_array'}} && $self->error($self->text('user_einuse',
				   $self->{'roleusers_array'}->[0]->[0]->{'user'}));
	$self->delete_user_attr($self->{'user'});
	}
else {
	# Check for clash
	if ($self->{'in_hash'}->{'new'} || $self->{'loguser'} ne $self->{'in_hash'}->{'user'}) {
		($self->{'clash'}) = grep { $_->{'user'} eq $self->{'in_hash'}->{'user'} } @{$self->{'users'}};
		$self->{'clash'} && $self->error($self->{'text_hash'}->{'user_eclash'});
		}

	# Validate and store inputs
	$self->{'in_hash'}->{'user'} =~ /^[^ :]+$/ || $self->error($self->{'text_hash'}->{'user_euser'});
	$self->{'user'}->{'user'} = $self->{'in_hash'}->{'user'};
	if (!$self->{'access_hash'}->{'users'}) {
		# Type must be role
		$self->{'user'}->{'attr'}->{'type'} = 'role';
		}
	elsif (!$self->{'access_hash'}->{'roles'}) {
		# Type must be user
		$self->{'user'}->{'attr'}->{'type'} = 'normal';
		}
	elsif ($self->{'in_hash'}->{'type'}) {
		# A type was selected
		$self->{'user'}->{'attr'}->{'type'} = $self->{'in_hash'}->{'type'};
		}
	else {
		# Default type chosen
		delete($self->{'user'}->{'attr'}->{'type'});
		}
	$self->{'profiles'} = $self->profiles_parse("profiles");
	if ($self->{'profiles'}) {
		@{$self->{'profiles_array'}} = split(/,/, $self->{'profiles'});
		foreach $p (@{$self->{'profiles_array'}}) {
			if (!$self->can_assign_profile($p) &&
			    $self->indexof($p, @{$self->{'oldprofs_array'}}) == -1) {
				$self->error($self->text('user_eprof', $p));
				}
			}
		$self->{'user'}->{'attr'}->{'profiles'} = $self->{'profiles'};
		}
	else {
		delete($self->{'user'}->{'attr'}->{'profiles'});
		}
	if ($self->{'access_hash'}->{'authassign'}) {
		$self->{'auths'} = $self->auths_parse("auths");
		if ($self->{'auths'}) {
			$self->{'user'}->{'attr'}->{'auths'} = $self->{'auths'};
			}
		else {
			delete($self->{'user'}->{'attr'}->{'auths'});
			}
		}
	$self->{'roles'} = $self->attr_parse("roles");
	if ($self->{'roles'}) {
		@roles = split(/,/, $self->{'roles'});
		$self->indexof($self->{'in_hash'}->{'user'}, @roles) < 0 ||
			$self->error($self->{'text_hash'}->{'user_esub'});
		foreach $r (@roles) {
			if (!$self->can_assign_role($r) &&
			    $self->indexof($r, @{$self->{'oldroles_array'}}) == -1) {
				$self->error($self->text('user_erole', $r));
				}
			}
		$self->{'user'}->{'attr'}->{'roles'} = $self->{'roles'};
		}
	else {
		delete($self->{'user'}->{'attr'}->{'roles'});
		}
	if ($self->{'in_hash'}->{'project_def'}) {
		delete($self->{'user'}->{'attr'}->{'project'});
		}
	else {
		$self->{'user'}->{'attr'}->{'project'} = $self->{'in_hash'}->{'project'};
		}
	if ($self->{'in_hash'}->{'lock'}) {
		$self->{'user'}->{'attr'}->{'lock_after_retries'} = $self->{'in_hash'}->{'lock'};
		}
	else {
		delete($self->{'user'}->{'attr'}->{'lock_after_retries'});
		}

	# Save or update user
	if ($self->{'in_hash'}->{'new'}) {
		$self->create_user_attr($self->{'user'});
		}
	else {
		$self->modify_user_attr($self->{'user'});

		# Update other users of this role, if renamed
		if ($self->{'loguser'} ne $self->{'in_hash'}->{'user'}) {
			foreach $ru (@{$self->{'roleusers_array'}}) {
				$ru->[2]->[$ru->[1]] = $self->{'in_hash'}->{'user'};
				$ru->[0]->{'attr'}->{'roles'} =
					join(",", @{$ru->[2]});
				$self->modify_user_attr($ru->[0]);
				}
			}
		}
	}

$self->unlock_rbac_files();
$self->webmin_log($self->{'in_hash'}->{'delete'} ? "delete" : $self->{'in_hash'}->{'new'} ? "create" : "modify",
	    "user", $self->{'loguser'}, $self->{'user'});
$self->redirect("list_users.cgi");

1;
		
}


1;

