#!/usr/bin/perl
package samba::Useradmin_update;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'samba::Useradmin_update');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;

&ModuleLoader::preload_module(ref($self), "samba::SambaLib");
$self = new samba::SambaLib:: ref($self), %$self; 

# When running Samba 3.x, these functions update the Samba groups file to
# match Unix groups

1;

1;
		return $self;
}

# useradmin_create_user(&details)
# Create a new samba user if sync is enabled
sub useradmin_create_user
{
	my $self = shift;

$self->get_share("global");
if ($self->istrue("encrypt passwords") && ($self->{'config_hash'}->{'smb_passwd'} || $self->{'has_pdbedit'}) &&
    $self->{'config_hash'}->{'sync_add'} && !$self->get_user($_[0]->{'user'})) {
	# Add a user to smbpasswd
	$self->lock_file($self->{'config_hash'}->{'smb_passwd'});
	local $u = { 'name' => $_[0]->{'user'},
		     'uid' => $_[0]->{'uid'} };
	if ($self->{'samba_version'} >= 2) {
		local @opts = ("U");
		push(@opts, "N") if ($_[0]->{'passmode'} == 0);
		push(@opts, "D") if ($_[0]->{'passmode'} == 1);
		$u->{'opts'} = \@opts;
		}
	else {
		$u->{'real'} = $_[0]->{'real'};
		$u->{'home'} = $_[0]->{'home'};
		$u->{'shell'} = $_[0]->{'shell'};
		}
	$u->{'pass1'} = $u->{'pass2'} = ("X" x 32);
	if ($_[0]->{'passmode'} == 0) {
		$u->{'pass1'} = "NO PASSWORDXXXXXXXXXXXXXXXXXXXXX";
		$u->{'pass2'} = $u->{'pass1'};
		}
	$self->create_user($u);
	if ($_[0]->{'passmode'} == 3) {
		$self->set_password($_[0]->{'user'}, $_[0]->{'plainpass'});
		}
	$self->unlock_file($self->{'config_hash'}->{'smb_passwd'});
	}
}

# useradmin_delete_user(&details)
# Delete a samba user
sub useradmin_delete_user
{
	my $self = shift;

$self->get_share("global");
if ($self->istrue("encrypt passwords") && ($self->{'config_hash'}->{'smb_passwd'} || $self->{'has_pdbedit'}) &&
    $self->{'config_hash'}->{'sync_delete'} && ($self->{'u'} = $self->get_user($_[0]->{'user'}))) {
	# Delete the user
	$self->lock_file($self->{'config_hash'}->{'smb_passwd'});
	$self->delete_user($self->{'u'});
	$self->unlock_file($self->{'config_hash'}->{'smb_passwd'});
	}

if ($self->{'config_hash'}->{'sync_delete_profile'}) {
	# Delete his roaming profile, if any
	if ($self->get_share("Profiles")) {
		local $ppath = $self->getval("path");
		if ($ppath) {
			local $upath = "$ppath/$_[0]->{'user'}";
			if (-d $upath) {
				$self->system_logged("rm -rf ".quotemeta($upath));
				}
			else {
				$self->lock_file($upath);
				unlink($upath);
				$self->unlock_file($upath);
				}
			}
		}
	}

}

# useradmin_modify_user(&details)
# Update a samba user
sub useradmin_modify_user
{
	my $self = shift;

$self->get_share("global");
if ($self->istrue("encrypt passwords") && ($self->{'config_hash'}->{'smb_passwd'} || $self->{'has_pdbedit'}) &&
    $self->{'config_hash'}->{'sync_change'} && ($self->{'u'} = $self->get_user($_[0]->{'olduser'}))) {
	# Update details
	$self->lock_file($self->{'config_hash'}->{'smb_passwd'});
	$self->{'u'}->{'uid'} = $_[0]->{'uid'};
	$self->{'u'}->{'name'} = $_[0]->{'user'};
	if ($self->{'u'}->{'opts'}) {
		local @opts = grep { !/[ND]/ } @{$self->{'u'}->{'opts'}};
		push(@opts, "N") if ($_[0]->{'passmode'} == 0);
		push(@opts, "D") if ($_[0]->{'passmode'} == 1);
		$self->{'u'}->{'opts'} = \@opts;
		}
	else {
		$self->{'u'}->{'real'} = $_[0]->{'real'};
		$self->{'u'}->{'home'} = $_[0]->{'home'};
		$self->{'u'}->{'shell'} = $_[0]->{'shell'};
		}
	if ($_[0]->{'passmode'} == 0) {
		$self->{'u'}->{'pass1'} = "NO PASSWORDXXXXXXXXXXXXXXXXXXXXX";
		$self->{'u'}->{'pass2'} = $self->{'u'}->{'pass1'};
		}
	elsif ($_[0]->{'passmode'} == 1) {
		$self->{'u'}->{'pass1'} = $self->{'u'}->{'pass2'} = ("X" x 32);
		}
	$self->modify_user($self->{'u'});
	if ($_[0]->{'passmode'} == 3) {
		$self->set_password($_[0]->{'user'}, $_[0]->{'plainpass'});
		}
	$self->unlock_file($self->{'config_hash'}->{'smb_passwd'});
	}

if ($self->{'config_hash'}->{'sync_change_profile'}) {
	# Rename his roaming profile, if any
	if ($self->get_share("Profiles")) {
		local $ppath = $self->getval("path");
		if ($ppath) {
			local $upath = "$ppath/$_[0]->{'olduser'}";
			local $newupath = "$ppath/$_[0]->{'user'}";
			if (-e $upath) {
				$self->rename_logged($upath, $newupath);
				}
			}
		}
	}

}

sub get_user
{
	my $self = shift;

local @ulist = $self->list_users();
local $u;
foreach $u (@ulist) {
	return $u if ($u->{'name'} eq $_[0]);
	}
return undef;
}

# useradmin_create_group(&group)
sub useradmin_create_group
{
	my $self = shift;

return if (!$self->{'config_hash'}->{'gsync_add'});
return if ($self->{'samba_version'} < 3 || 
	   (!$self->{'has_smbgroupedit'} && !$self->{'has_net'}));
local $clash = $self->get_group($_[0]->{'group'});
return if ($clash);

local $group = { 'name' => $_[0]->{'group'},
		 'unix' => $_[0]->{'group'},
		 'type' => $self->{'config_hash'}->{'gsync_type'} };
if ($group->{'type'} eq 'l') {
	$group->{'priv'} = $self->{'config_hash'}->{'gsync_priv'};
	}
$self->create_group($group);
}

# useradmin_delete_group(&group)
sub useradmin_delete_group
{
	my $self = shift;

return if (!$self->{'config_hash'}->{'gsync_delete'});
return if ($self->{'samba_version'} < 3 || 
	   (!$self->{'has_smbgroupedit'} && !$self->{'has_net'}));
local $group = $self->get_group($_[0]->{'group'});
return if (!$group);

$self->delete_group($group);
}

# useradmin_modify_group(&group, &oldgroup)
sub useradmin_modify_group
{
	my $self = shift;

return if (!$self->{'config_hash'}->{'gsync_change'});
return if ($_[0]->{'group'} eq $_[1]->{'group'});
return if ($self->{'samba_version'} < 3 || 
	   (!$self->{'has_smbgroupedit'} && !$self->{'has_net'}));
local $group = $self->get_group($_[1]->{'group'});
return if (!$group);

$group->{'name'} = $_[0]->{'group'};
$self->modify_group($group);
# XXX clash?
}

sub get_group
{
	my $self = shift;

local @glist = $self->list_groups();
local $g;
foreach $g (@glist) {
	return $g if ($g->{'name'} eq $_[0]);
	}
return undef;
}


1;

