#!/usr/bin/perl
package sarg::SargLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'sarg::SargLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for reading and writing sarg.conf

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

$self->{'cron_cmd'} = "$self->{'module_config_directory'}/generate.pl";
%{$self->{'needs_quotes_hash'}} = map { $_, 1 } ("title", "logo_text", "user_invalid_char",
			       "privacy_string", "include_users",
			       "exclude_string", "datafile_delimiter");

1;

1;
		return $self;
}

# get_config()
# Parses the sarg config file into directives
sub get_config
{
	my $self = shift;

if (!defined(@{$self->{'get_config_cache_array'}})) {
	local $lnum = 0;
	open(CONF, $self->{'config_hash'}->{'sarg_conf'});
	while(<CONF>) {
		s/\r|\n//g;
		if (/^\s*(#*)\s*(\S+)\s+"([^"]*)"/ ||
		    /^\s*(#*)\s*(\S+)\s+'([^"]*)'/ ||
		    /^\s*(#*)\s*(\S+)\s+(.*\S)/) {
			push(@{$self->{'get_config_cache_array'}},
				  { 'name' => $2,
				    'value' => $3,
				    'enabled' => !$1,
				    'line' => $lnum });
			}
		$lnum++;
		}
	close(CONF);
	}
return \@{$self->{'get_config_cache_array'}};
}

# find(name, &config, [and-disabled])
sub find
{
	my $self = shift;

local @rv;
local $c;
foreach $c (@{$_[1]}) {
	if (lc($c->{'name'}) eq lc($_[0])) {
		if ($c->{'enabled'} && $_[2] == 0 ||
		    !$c->{'enabled'} && $_[2] == 2 ||
				       $_[2] == 1) {
			push(@rv, $c);
			}
		}
	}
return wantarray ? @rv : $rv[0];
}

# find_value(name, &config)
sub find_value
{
	my $self = shift;

local @rv = map { $_->{'value'} } $self->find(@_);
return wantarray ? @rv : $rv[0];
}

# save_directive(&config, name, &values)
# Update the config file with some new values for an option
sub save_directive
{
	my $self = shift;

local ($conf, $name, $values) = @_;
local $lref = $self->read_file_lines($self->{'config_hash'}->{'sarg_conf'});
local @old = $self->find($name, $conf);
local @new = @$values;
local $i;
local $changed;
for($i=0; $i<@old || $i<@new; $i++) {
	local $newline;
	if ($i<@new) {
		if ($self->{'needs_quotes_hash'}->{$name}) {
			$newline = $name." \"".$new[$i]."\"";
			}
		else {
			$newline = $name." ".$new[$i];
			}
		}

	if ($i<@old && $i<@new) {
		# Updating some directive
		$lref->[$old[$i]->{'line'}] = $newline;
		$old[$i]->{'value'} = $new[$i];
		$changed = $old[$i];
		}
	elsif ($i<@old) {
		# Removing some directive (need to renumber)
		splice(@$lref, $old[$i]->{'line'}, 1);
		splice(@$conf, $old[$i]->{'index'}, 1);
		$self->renumber($conf, 'line', $old[$i]->{'line'}, -1);
		$self->renumber($conf, 'index', $old[$i]->{'index'}, -1);
		}
	elsif ($i<@new) {
		# Adding some directive (perhaps after comment)
		local $nd = { 'name' => $name,
			      'value' => $new[$i] };
		local $cmt = $self->find($name, $conf, 1);
		if ($cmt && !$cmt->{'enabled'}) {
			# Add after comment line
			$nd->{'line'} = $cmt->{'line'} + 1;
			$nd->{'index'} = $cmt->{'index'} + 1;
			$self->renumber($conf, 'line', $nd->{'line'}-1, 1);
			$self->renumber($conf, 'index', $nd->{'index'}-1, 1);
			splice(@$lref, $nd->{'line'}, 0, $newline);
			splice(@$conf, $nd->{'index'}, 0, $nd);
			}
		else {
			# Add to end
			$nd->{'line'} = scalar(@$lref);
			$nd->{'index'} = scalar(@$conf);
			push(@$lref, $newline);
			push(@$conf, $nd);
			}
		}
	}
}

# renumber(&conf, field, pos, offset)
sub renumber
{
	my $self = shift;

local ($conf, $field, $pos, $offset) = @_;
local $c;
foreach $c (@$conf) {
	if ($c->{$field} > $pos) {
		$c->{$field} += $offset;
		}
	}
}

# config_textbox(&config, name, size, cols, [prefix])
# Returns HTML for a table row for some text field
sub config_textbox
{
	my $self = shift;

local ($conf, $name, $size, $cols, $prefix) = @_;
$cols ||= 1;
local $val = join(" ", $self->find_value($name, $conf));
return $self->ui_table_row($self->{'text_hash'}->{$self->{'config_prefix'}.$name},
		     $self->ui_textbox($name, $val, $size)." ".$prefix,
		     $cols, \@{$self->{'ui_tds_array'}});
}

# save_textbox(config, name, [function])
sub save_textbox
{
	my $self = shift;

local ($conf, $name, $func) = @_;
local @vals = $self->{'split'} ? split(/\s+/, $self->{'in_hash'}->{$name}) : ( $self->{'in_hash'}->{$name} );
local $v;
foreach $v (@vals) {
	$self->error_check($name, $func, $v);
	}
$self->save_directive($conf, $name, \@vals);
}

# error_check(name, func, value)
sub error_check
{
	my $self = shift;

local ($name, $func, $val) = @_;
return if (!$func);
local $err = $self->$func($val);
$self->error($self->{'text_hash'}->{$self->{'config_prefix'}.$name}." : ".$err) if ($err);
}

# config_opt_textbox(&config, name, size, cols, [default])
# Returns HTML for a table row for some optional text field
sub config_opt_textbox
{
	my $self = shift;

local ($conf, $name, $size, $cols, $default) = @_;
$cols ||= 1;
local $val = $self->find_value($name, $conf);
local $defstr = $self->find($name, $conf, 2);
local $def = $defstr && $defstr->{'value'} ? " ($defstr->{'value'})" : undef;
return $self->ui_table_row($self->{'text_hash'}->{$self->{'config_prefix'}.($name eq "title" ? "title2" :$name)},
		     $self->ui_radio($name."_def", $val ? 0 : 1,
			       [ [ 1, $default || $self->{'text_hash'}->{'default'}.$def ],
				 [ 0, " " ] ])." ".
		     $self->ui_textbox($name, $val, $size), $cols, \@{$self->{'ui_tds_array'}});
}

# save_opt_textbox(config, name, [function], [split])
sub save_opt_textbox
{
	my $self = shift;

local ($conf, $name, $func, $split) = @_;
if ($self->{'in_hash'}->{$name."_def"}) {
	$self->save_directive($conf, $name, [ ]);
	}
else {
	$self->save_textbox(@_);
	}
}

# config_yesno(&config, name, [yes], [no], [&others])
# Returns HTML for a table row for yes/no checkboxes
sub config_yesno
{
	my $self = shift;

local ($conf, $name, $yes, $no, $others, $width) = @_;
local $val = $self->find_value($name, $conf);
local $default = $self->find($name, $conf, 2);
#local $defstr = $default ? " ($default->{'value'})" : "";
return $self->ui_table_row($self->{'text_hash'}->{$self->{'config_prefix'}.$name},
		     $self->ui_radio($name, $val,
			       [ [ "yes", $yes || $self->{'text_hash'}->{'yes'} ],
				 [ "no", $no || $self->{'text_hash'}->{'no'} ],
				 @$others,
				 [ "", $self->{'text_hash'}->{'default'}.$defstr ] ]),
		     $width, \@{$self->{'ui_tds_array'}});
}

# save_yesno(&config, name)
sub save_yesno
{
	my $self = shift;

local ($conf, $name) = @_;
if (!$self->{'in_hash'}->{$name}) {
	$self->save_directive($conf, $name, [ ]);
	}
else {
	$self->save_directive($conf, $name, [ $self->{'in_hash'}->{$name} ]);
	}
}

# config_radio(&config, name, [&options], [width])
# Returns HTML for a table row for a bunch of checkboxes
sub config_radio
{
	my $self = shift;

local ($conf, $name, $others, $width) = @_;
local $val = $self->find_value($name, $conf);
local $default = $self->find($name, $conf, 2);
#local $defstr = $default ? " ($default->{'value'})" : "";
return $self->ui_table_row($self->{'text_hash'}->{$self->{'config_prefix'}.$name},
		     $self->ui_radio($name, $val,
			       [ @$others,
				 [ "", $self->{'text_hash'}->{'default'}.$defstr ] ]),
		     $width, \@{$self->{'ui_tds_array'}});
}

sub save_radio
{
	my $self = shift;

$self->save_yesno(@_);	# same logic works
}

# config_sortfield(&config, name, &opts, [&sort-opts])
# Returns HTML for a table row for selecting a sort mode
sub config_sortfield
{
	my $self = shift;

local ($conf, $name, $opts, $sort) = @_;
local $val = $self->find_value($name, $conf);
local $default = $self->find($name, $conf, 2);
#local $defstr = $default ? " ($default->{'value'})" : "";
local @vals = split(/\s+/, $val);
$sort ||= [ [ "reverse", $self->{'text_hash'}->{'report_reverse'} ],
	    [ "forward", $self->{'text_hash'}->{'report_forward'} ] ];
return $self->ui_table_row($self->{'text_hash'}->{$self->{'config_prefix'}.$name},
	$self->ui_oneradio($name."_def", 1, $self->{'text_hash'}->{'default'}.$defstr, !$val)."\n".
	$self->ui_oneradio($name."_def", 0, $self->{'text_hash'}->{'report_field'}, $val)."\n".
	$self->ui_select($name."_field", $vals[0],
		   [ map { [ $_, $self->{'text_hash'}->{'report_by_'.lc($_)} ] } @$opts ])."\n".
	$self->ui_select($name."_order", $vals[1], $sort), 3);
}

sub save_sortfield
{
	my $self = shift;

local ($conf, $name) = @_;
if ($self->{'in_hash'}->{$name."_def"}) {
	$self->save_directive($conf, $name, [ ]);
	}
else {
	$self->save_directive($conf, $name, [ $self->{'in_hash'}->{$name."_field"}." ".
					$self->{'in_hash'}->{$name."_order"} ]);
	}
}

# config_language(&config, name, width, file)
sub config_language
{
	my $self = shift;

local ($conf, $name, $width, $file) = @_;
local $val = $self->find_value($name, $conf);
local $default = $self->find($name, $conf, 2);
local $defstr = $default ? " ($default->{'value'})" : "";
local $found;
local @langs = ( [ "", $self->{'text_hash'}->{'default'}.$defstr ] );
open(LANGS, $file);
while(<LANGS>) {
	chop;
	if (/^(\S+)\t+(\S.*)/) {
		push(@langs, [ $1, "$1 - $2" ]);
		}
	elsif (/^(\S+)/) {
		push(@langs, [ $1 ]);
		}
	$found++ if ($val eq $_);
	}
close(LANGS);
push(@langs, [ $val ]) if (!$found && $val);
return $self->ui_table_row($self->{'text_hash'}->{$self->{'config_prefix'}.$name},
		     $self->ui_select($name, $val, \@langs), $width);
}

# save_language(&config, name)
sub save_language
{
	my $self = shift;

$self->save_yesno(@_);	# same logic works
}

# config_select(&config, name, &options, default, width)
sub config_select
{
	my $self = shift;

local ($conf, $name, $others, $default, $width) = @_;
local $val = $self->find_value($name, $conf);
local @vals = split(/\s+/, $val);
return $self->ui_table_row($self->{'text_hash'}->{$self->{'config_prefix'}.$name},
		     $self->ui_radio($name."_def", $val ? 0 : 1,
			       [ [ 1, $default || $self->{'text_hash'}->{'default'} ],
				 [ 0, $self->{'text_hash'}->{'report_below'} ] ])."<br>".
		     $self->ui_select($name, \@vals, $others, "5 ", 1),
		     $width, \@{$self->{'ui_tds_array'}});

}

sub save_select
{
	my $self = shift;

local ($conf, $name) = @_;
$self->{'in_hash'}->{$name} =~ s/\0/ /g;
$self->{'in_hash'}->{$name."_def"} || $self->{'in_hash'}->{$name} || $self->error($self->{'text_hash'}->{'report_eselect'});
$self->save_opt_textbox(@_);	# same logic works
}

# config_colons(&config, name, separator, default, width)
sub config_colons
{
	my $self = shift;

local ($conf, $name, $sep, $default, $width) = @_;
local $val = $self->find_value($name, $conf);
return $self->ui_table_row($self->{'text_hash'}->{$self->{'config_prefix'}.$name},
		     $self->ui_radio($name."_def", $val ? 0 : 1,
			       [ [ 1, $default || $self->{'text_hash'}->{'default'} ],
				 [ 0, $self->{'text_hash'}->{'report_below2'} ] ])."<br>".
		     $self->ui_textarea($name, join("\n", split($sep, $val)),
				  3, 30),
		     $width, \@{$self->{'ui_tds_array'}});
}

# save_colons(&config, name, separator)
sub save_colons
{
	my $self = shift;

local ($conf, $name, $sep) = @_;
$self->{'in_hash'}->{$name} =~ s/\r//g;
$self->{'in_hash'}->{$name} =~ s/\s+$//;
$self->{'in_hash'}->{$name} =~ s/^\s+//;
$self->{'in_hash'}->{$name} =~ s/\n/$sep/g;
$self->{'in_hash'}->{$name."_def"} || $self->{'in_hash'}->{$name} || $self->error($self->{'text_hash'}->{'report_eenter'});
$self->save_opt_textbox($conf, $name);	# same logic works
}

# config_range(&config, name, start, end, width)
sub config_range
{
	my $self = shift;

local ($conf, $name, $start, $end, $width) = @_;
local $val = $self->find_value($name, $conf);
local (%sel, $v, $vn);
foreach $v (split(/,/, $val)) {
	if ($v =~ /^(\d+)\-(\d+)$/) {
		foreach $vn ($1 .. $2) { $sel{$vn}++; }
		}
	else { $sel{$v}++; }
	}
local $table = "<table>\n";
local $i;
for($i=$start; $i<=$end; $i++) {
	$table .= "<tr>\n" if ($i%12 == 0);
	$table .= "<td>".$self->ui_checkbox($name, $i,
		$self->{'text_hash'}->{$self->{'config_prefix'}.$name.$i} || $i, $sel{$i})."</td>\n";
	$table .= "</tr>\n" if ($i%12 == 11);
	}
$table .= "</tr>\n" if ($i%12 != 0);
$table .= "</table>\n";
return $self->ui_table_row($self->{'text_hash'}->{$self->{'config_prefix'}.$name},
		     $self->ui_radio($name."_def", $val ? 0 : 1,
			       [ [ 1, $self->{'text_hash'}->{'default'} ],
				 [ 0, $self->{'text_hash'}->{'report_below'} ] ]).
		     "<br>".$table, $width);
}

# save_range(&config, name)
sub save_range
{
	my $self = shift;

local ($conf, $name) = @_;
if ($self->{'in_hash'}->{$name."_def"}) {
	$self->save_directive($conf, $name, [ ]);
	}
else {
	local @vals = split(/\0/, $self->{'in_hash'}->{$name});
	$self->save_directive($conf, $name, [ join(",", @vals) ]);
	}
}

# generate_report(handle, escape, clear-output, from, to)
# Immediately generate a report
sub generate_report
{
	my $self = shift;

local ($h, $esc, $clear, $from, $to) = @_;
local $conf = $self->get_config();
local $odir = $self->find_value("output_dir", $conf);
$odir ||= $self->find_value("output_dir", $conf, 1);

if ($clear) {
	# Delete all old files
	unlink("$odir/index.html");
	opendir(DIR, $odir);
	while($self->{'f'} = readdir(DIR)) {
		if (-r "$odir/$self->{'f'}/usuarios") {
			$self->system_logged("rm -rf ".quotemeta("$odir/$self->{'f'}"));
			}
		}
	closedir(DIR);
	}

# Work out date range
local $rangearg;
if ($from || $to) {
	if ($from < $to) {
		($from, $to) = ($to, $from);
		}
	local $now = time();
	local @fromtm = localtime($now - $from*24*60*60);
	local @totm = localtime($now - $to*24*60*60);
	$rangearg = sprintf "-d %2.2d/%2.2d/%4.4d-%2.2d/%2.2d/%4.4d",
			$fromtm[3], $fromtm[4]+1, $fromtm[5]+1900,
			$totm[3], $totm[4]+1, $totm[5]+1900;
	}

local $sfile = $self->find_value("access_log", $conf);
local @all = $self->all_log_files($sfile);
foreach $file (@all) {
	local $cmd = "$self->{'config_hash'}->{'sarg'} -l $file $rangearg";
	print $h $cmd,"\n";
	open(OUT, "$cmd 2>&1 |");
	while(<OUT>) {
		print $h $esc ? $self->html_escape($_) : $_;
		}
	close(OUT);
	return 0 if ($?);
	$self->additional_log("exec", undef, $cmd);
	}
return 1;
}

# all_log_files(file)
sub all_log_files
{
	my $self = shift;

$_[0] =~ /^(.*)\/([^\/]+)$/;
local $dir = $1;
local $base = $2;
local ($f, @rv);
opendir(DIR, $dir);
foreach $f (readdir(DIR)) {
	if ($f =~ /^\Q$base\E/ && -f "$dir/$f") {
		push(@rv, "$dir/$f");
		}
	}
closedir(DIR);
return @rv;
}

sub gen_clear_input
{
	my $self = shift;

return $self->ui_radio("clear", int($self->{'config_hash'}->{'clear'}),
		      [ [ 1, $self->{'text_hash'}->{'yes'} ],
			[ 0, $self->{'text_hash'}->{'no'} ] ]);
}

sub gen_range_input
{
	my $self = shift;

local ($rfrom, $rto) = split(/\s+/, $self->{'config_hash'}->{'range'});
local $range = $self->text('sched_rsel',
	       $self->ui_textbox("rfrom", $rfrom, 3),
	       $self->ui_textbox("rto", $rto, 3));
return $self->ui_radio("range_def", $self->{'config_hash'}->{'range'} ? 0 : 1,
		      [ [ 1, $self->{'text_hash'}->{'sched_rall'} ],
			[ 0, $range ] ]);
}

sub lock_sarg_files
{
	my $self = shift;

$self->lock_file($self->{'config_hash'}->{'sarg_conf'});
}

sub unlock_sarg_files
{
	my $self = shift;

$self->unlock_file($self->{'config_hash'}->{'sarg_conf'});
}


1;

