#!/usr/bin/perl
package sendmail::AccessLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'sendmail::AccessLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# access-lib.pl
# Functions for the access_db table

1;

1;
		return $self;
}

# access_dbm(&config)
# Returns the filename and type of the access database, or undef if none
sub access_dbm
{
	my $self = shift;

foreach $f ($self->find_type("K", $_[0])) {
        if ($f->{'value'} =~ /^access\s+(\S+)[^\/]+(\S+)$/) {
		return ($2, $1);
                }
	}
return undef;
}

# access_file(&config)
# Returns the filename of the text access file, or undef if none
sub access_file
{
	my $self = shift;

return $self->find_textfile($self->{'config_hash'}->{'access_file'}, $self->access_dbm($_[0]));
}

# list_access(textfile)
sub list_access
{
	my $self = shift;

if (!defined(@{$self->{'list_access_cache_array'}})) {
	@{$self->{'list_access_cache_array'}} = ( );
	local $lnum = 0;
	open(ACC, $_[0]);
	while(<ACC>) {
		s/\r|\n//g;     # remove newlines
		if (/^\s*#+\s*(.*)/) {
			# A comment line
			$self->{'cmt'} = $self->is_table_comment($_);
			}
		elsif (/^(\S+)\s+(.*)/) {
			local(%acc);
			$acc{'from'} = $1;
			$acc{'action'} = $2;
			$acc{'line'} = $self->{'cmt'} ? $lnum-1 : $lnum;
			$acc{'eline'} = $lnum;
			$acc{'num'} = scalar(@{$self->{'list_access_cache_array'}});
			if ($acc{'from'} =~ /^(Connect|From|To):(.*)/i) {
				$acc{'tag'} = $1;
				$acc{'from'} = $2;
				}
			$acc{'cmt'} = $self->{'cmt'};
			push(@{$self->{'list_access_cache_array'}}, \%acc);
			$self->{'cmt'} = undef;
			}
		else {
			$self->{'cmt'} = undef;
			}
		$lnum++;
		}
	close(ACC);
	}
return @{$self->{'list_access_cache_array'}};
}

# create_access(&details, textfile, dbmfile, dbmtype)
# Create a new access database entry
sub create_access
{
	my $self = shift;

$self->list_access($_[1]);	 # force cache init
local(%acc);

# Write to the file
local $lref = $self->read_file_lines($_[1]);
$_[0]->{'line'} = scalar(@$lref);
push(@$lref, $self->make_table_comment($_[0]->{'cmt'}));
local $from = $_[0]->{'tag'} ? "$_[0]->{'tag'}:$_[0]->{'from'}"
			     : $_[0]->{'from'};
push(@$lref, "$from\t$_[0]->{'action'}");
$_[0]->{'eline'} = scalar(@$lref)-1;
$self->flush_file_lines($_[1]);

# Add to DBM
if ($_[3] eq "dbm") {
	dbmopen(%acc, $_[2], 0644);
	$acc{$from} = $_[0]->{'action'};
	dbmclose(%acc);
	}
else { $self->run_makemap($_[1], $_[2], $_[3]); }

# Add to cache
$_[0]->{'num'} = scalar(@{$self->{'list_access_cache_array'}});
$_[0]->{'file'} = $_[1];
push(@{$self->{'list_access_cache_array'}}, $_[0]);
}

# delete_access(&details, textfile, dbmfile, dbmtype)
# Delete an existing access entry
sub delete_access
{
	my $self = shift;

local(%acc);

# Delete form file
local $lref = $self->read_file_lines($_[1]);
local $len = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
splice(@$lref, $_[0]->{'line'}, $len);
$self->flush_file_lines($_[1]);

# Delete from DBM
local $oldfrom = $_[0]->{'tag'} ? "$_[0]->{'tag'}:$_[0]->{'from'}"
			        : $_[0]->{'from'};
if ($_[3] eq "dbm") {
	dbmopen(%acc, $_[2], 0644);
	delete($acc{$oldfrom});
	dbmclose(%acc);
	}
else { $self->run_makemap($_[1], $_[2], $_[3]); }

# Delete from cache
local $idx = $self->indexof($_[0], @{$self->{'list_access_cache_array'}});
splice(@{$self->{'list_access_cache_array'}}, $idx, 1) if ($idx != -1);
$self->renumber_list(\@{$self->{'list_access_cache_array'}}, $_[0], -$len);
}

# modify_access(&old, &details, textfile, dbmfile, dbmtype)
# Change an existing access entry
sub modify_access
{
	my $self = shift;

local %acc;
local $oldfrom = $_[0]->{'tag'} ? "$_[0]->{'tag'}:$_[0]->{'from'}"
			     : $_[0]->{'from'};
local $from = $_[1]->{'tag'} ? "$_[1]->{'tag'}:$_[1]->{'from'}"
			     : $_[1]->{'from'};

# Update in file
local $lref = $self->read_file_lines($_[2]);
local $oldlen = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
local @newlines;
push(@newlines, $self->make_table_comment($_[1]->{'cmt'}));
push(@newlines, "$from\t$_[1]->{'action'}");
splice(@$lref, $_[0]->{'line'}, $oldlen, @newlines);
$self->flush_file_lines($_[2]);

# Update DBM
if ($_[4] eq "dbm") {
	dbmopen(%{$self->{'virt_hash'}}, $_[3], 0644);
	delete($self->{'virt_hash'}->{$oldfrom});
	$self->{'virt_hash'}->{$from} = $_[1]->{'action'};
	dbmclose(%{$self->{'virt_hash'}});
	}
else { $self->run_makemap($_[2], $_[3], $_[4]); }

# Update cache
local $idx = $self->indexof($_[0], @{$self->{'list_generics_cache_array'}});
$_[1]->{'line'} = $_[0]->{'line'};
$_[1]->{'eline'} = $_[1]->{'cmt'} ? $_[0]->{'line'}+1 : $_[0]->{'line'};
$self->{'list_generics_cache_array'}->[$idx] = $_[1] if ($idx != -1);
$self->renumber_list(\@{$self->{'list_generics_cache_array'}}, $_[0], scalar(@newlines)-$oldlen);
}

# access_form([&details])
sub access_form
{
	my $self = shift;

local ($v) = @_;
local ($mode, $addr);

print $self->ui_form_start("save_access.cgi", "post");
if ($v) {
	print $self->ui_hidden("num", $v->{'num'}),"\n";
	}
else {
	print $self->ui_hidden("new", 1),"\n";
	}
print $self->ui_table_start($v ? $self->{'text_hash'}->{'sform_edit'} : $self->{'text_hash'}->{'sform_create'},
		      undef, 2);

# Comment
print $self->ui_table_row($self->{'text_hash'}->{'vform_cmt'},
		    $self->ui_textbox("cmt", $v->{'cmt'}, 50));

# Mail source
local $src = $v->{'from'} =~ /^\S+\@\S+$/ ? 0 :
	     $v->{'from'} =~ /^[0-9\.]+$/ ? 1 :
	     $v->{'from'} =~ /^\S+\@$/ ? 2 :
	     $v->{'from'} =~ /^[A-z0-9\-\.]+$/ ? 3 : 0;
print $self->ui_table_row($self->{'text_hash'}->{'sform_source'},
    $self->ui_select("from_type", $src,
	       [ map { [ $_, $self->{'text_hash'}->{"sform_type$_"} ] } (0 .. 3) ])."\n".
    $self->ui_textbox("from", $v->{'from'}, 25));

# Match against tag
local $ver = $self->get_sendmail_version();
if ($v->{'tag'} || $ver >= 8.10) {
	print $self->ui_table_row($self->{'text_hash'}->{'sform_tag'},
	    $self->ui_select("tag", $v->{'tag'},
		       [ [ "", $self->{'text_hash'}->{'sform_tag_'} ],
		         [ "From", $self->{'text_hash'}->{'sform_tag_from'} ],
		         [ "To", $self->{'text_hash'}->{'sform_tag_to'} ],
		         [ "Connect", $self->{'text_hash'}->{'sform_tag_connect'} ],
		         [ "Spam", $self->{'text_hash'}->{'sform_tag_spam'} ] ]));
	}

# Action
local $atable = "<table>\n";
$atable .= "<tr>";
$atable .= "<td>".$self->ui_oneradio("action", "OK", $self->{'text_hash'}->{'sform_ok'},
		       $v->{'action'} eq "OK" || !$v->{'action'})."</td>\n";
$atable .= "<td>".$self->ui_oneradio("action", "RELAY", $self->{'text_hash'}->{'sform_relay'},
			       $v->{'action'} eq "RELAY")."</td>\n";
$atable .= "</tr>";
$atable .= "<tr>";
$atable .= "<td>".$self->ui_oneradio("action", "REJECT", $self->{'text_hash'}->{'sform_reject'},
			       $v->{'action'} eq "REJECT")."</td>\n";
$atable .= "<td>".$self->ui_oneradio("action", "DISCARD", $self->{'text_hash'}->{'sform_discard'},
			       $v->{'action'} eq "DISCARD")."</td>\n";
$atable .= "</tr>";
$atable .= "<tr>";
local ($err, $msg) = $v->{'action'} =~ /(\d+)\s*(.*)$/ ? ($1, $2) : ( );
$atable .= "<td colspan=2>".$self->ui_oneradio("action", 0,  $self->{'text_hash'}->{'sform_err'},
					 $err)."\n";
$atable .= $self->ui_textbox("err", $err, 4)." ".$self->{'text_hash'}->{'sform_msg'}."\n";
$atable .= $self->ui_textbox("msg", $msg, 20)."</td>\n";
$atable .= "</tr>";
$atable .= "</table>\n";
print $self->ui_table_row($self->{'text_hash'}->{'sform_action'}, $atable);

print $self->ui_table_end();
print $self->ui_form_end($_[0] ? [ [ "save", $self->{'text_hash'}->{'save'} ],
			     [ "delete", $self->{'text_hash'}->{'delete'} ] ]
		         : [ [ "create", $self->{'text_hash'}->{'create'} ] ]);
}

sub can_edit_access
{
	my $self = shift;

local ($g) = @_;
return $self->{'access_hash'}->{'smode'} == 1 ||
       $self->{'access_hash'}->{'smode'} == 2 && $g->{'from'} =~ /$self->{'access_hash'}->{'saddrs'}/;
}


1;

