#!/usr/bin/perl
package sendmail::FeaturesLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'sendmail::FeaturesLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# features-lib.pl

1;

1;
		return $self;
}

# list_features()
# Returns a list of entries in the sendmail.mc file, each of which may be a
# feature or some other unrecognized line
sub list_features
{
	my $self = shift;

local (@rv, $lnum = 0);
open(MC, $self->{'config_hash'}->{'sendmail_mc'});
while(<MC>) {
	s/\r|\n//g;
	local $f;
	if (/^FEATURE\((.*)\)/i) {
		local ($name, @v) = $self->split_m4_params($1);
		$f = { 'type' => 1,
		       'name' => $name,
		       'values' => \@v };
		}
	elsif (/^(define|undefine)\((.*)\)/i) {
		local @v = $self->split_m4_params($2);
		$f = { 'type' => ($1 eq 'define' ? 2 : 3),
		       'name' => $v[0],
		       'value' => $v[1] };
		}
	elsif (/^MAILER\((.*)\)/i) {
		local ($mailer) = $self->split_m4_params($1);
		$f = { 'type' => 4,
		       'mailer' => $mailer };
		}
	elsif (/^OSTYPE\((.*)\)/i) {
		local ($ostype) = $self->split_m4_params($1);
		$f = { 'type' => 5,
		       'line' => $lnum,
		       'index' => scalar(@rv),
		       'ostype' => $ostype };
		}
	else {
		# Unrecognized line
		$f = { 'type' => 0 };
		}
	if ($f) {
		$f->{'line'} = $lnum;
		$f->{'index'} = scalar(@rv);
		$f->{'text'} = $_;
		push(@rv, $f);
		}
	$lnum++;
	}
close(MC);
return @rv;
}

# split_m4_params(string)
sub split_m4_params
{
	my $self = shift;

local @p;
local $str = $_[0];
while($str =~ /^`([^']*)'\s*,?\s*(.*)$/ ||
      $str =~ /^([^\s,]+)\s*,?\s*(.*)$/) {
	push(@p, $1);
	$str = $2;
	}
return @p;
}

# list_feature_types()
sub list_feature_types
{
	my $self = shift;

local (@rv, $f);
opendir(DIR, "$self->{'config_hash'}->{'sendmail_features'}/feature");
while($f = readdir(DIR)) {
	if ($f =~ /^(\S+)\.m4$/) {
		local $t = $self->{'text_hash'}->{'feat_'.lc($1)};
		push(@rv, [ $1, $t ? "$1 ($t)" : $1 ] );
		}
	}
close(DIR);
return @rv;
}

# list_define_types()
# Returns a list of known define types. Some (but not all) will have human-readable
# descriptions
sub list_define_types
{
	my $self = shift;

local (@rv, $d);
open(DEFINES, "$self->{'module_root_directory'}/defines");
while($d = <DEFINES>) {
	$d =~ s/\r|\n//g;
	local $t = $self->{'text_hash'}->{'def_'.lc($d)};
	push(@rv, [ $d, $t ? "$d ($t)" : $d ]);
	}
close(DEFINES);
return @rv;
}

# list_mailer_types()
sub list_mailer_types
{
	my $self = shift;

local (@rv, $f);
opendir(DIR, "$self->{'config_hash'}->{'sendmail_features'}/mailer");
while($f = readdir(DIR)) {
	if ($f =~ /^(\S+)\.m4$/) {
		local $t = $self->{'text_hash'}->{'mailer_'.lc($1)};
		push(@rv, [ $1, $t ? "$1 ($t)" : $1 ] );
		}
	}
close(DIR);
return @rv;
}

# list_ostype_types()
sub list_ostype_types
{
	my $self = shift;

local (@rv, $f);
opendir(DIR, "$self->{'config_hash'}->{'sendmail_features'}/ostype");
while($f = readdir(DIR)) {
	if ($f =~ /^(\S+)\.m4$/) {
		local $t = $self->{'text_hash'}->{'ostype_'.lc($1)};
		push(@rv, [ $1, $t ? "$1 ($t)" : $1 ] );
		}
	}
close(DIR);
return @rv;
}

# create_feature(&feature)
# Adds an entry to the end of the M4 config file
sub create_feature
{
	my $self = shift;

$self->open_tempfile(MC, ">>$self->{'config_hash'}->{'sendmail_mc'}");
$self->print_tempfile(MC, $self->feature_line($_[0]),"\n");
$self->close_tempfile(MC);
$_[0]->{'text'} = $self->feature_line($_[0]);
}

# delete_feature(&feature)
# Deletes one entry from the M4 config file
sub delete_feature
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'sendmail_mc'});
splice(@$lref, $_[0]->{'line'}, 1);
$self->flush_file_lines();
}

# modify_feature(&feature)
# Updates an entry in the M4 config file
sub modify_feature
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'sendmail_mc'});
splice(@$lref, $_[0]->{'line'}, 1, $self->feature_line($_[0]));
$self->flush_file_lines();
$_[0]->{'text'} = $self->feature_line($_[0]);
}

# swap_features(&feature1, &feature2)
sub swap_features
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'sendmail_mc'});
splice(@$lref, $_[0]->{'line'}, 1, $_[1]->{'text'});
splice(@$lref, $_[1]->{'line'}, 1, $_[0]->{'text'});
$self->flush_file_lines();
}

# feature_line(&feature)
sub feature_line
{
	my $self = shift;

if ($_[0]->{'type'} == 0) {
	return $_[0]->{'text'};
	}
elsif ($_[0]->{'type'} == 1) {
	return "FEATURE(".$self->join_m4_params($_[0]->{'name'}, @{$_[0]->{'values'}}).")";
	}
elsif ($_[0]->{'type'} == 2) {
	if ($_[0]->{'value'} eq '') {
		return "define(".$self->join_m4_params($_[0]->{'name'}).")";
		}
	else {
		return "define(".$self->join_m4_params($_[0]->{'name'},
						$_[0]->{'value'}).")";
		}
	}
elsif ($_[0]->{'type'} == 3) {
	return "undefine(".$self->join_m4_params($_[0]->{'name'}).")";
	}
elsif ($_[0]->{'type'} == 4) {
	return "MAILER(".$self->join_m4_params($_[0]->{'mailer'}).")";
	}
elsif ($_[0]->{'type'} == 5) {
	return "OSTYPE(".$self->join_m4_params($_[0]->{'ostype'}).")";
	}
}

sub join_m4_params
{
	my $self = shift;

local @rv = map { $_ =~ /^\d+$/ ? $_ : "`$_'" } @_;
return join(",", @rv);
}


1;

