#!/usr/bin/perl
package sendmail::GenericsLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'sendmail::GenericsLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# generics-lib.pl
# Functions for the generics table

1;

1;
		return $self;
}

# generics_dbm(&config)
# Returns the filename and type of the generics database, or undef if none
sub generics_dbm
{
	my $self = shift;

foreach $f ($self->find_type("K", $_[0])) {
        if ($f->{'value'} =~ /^generics\s+(\S+)[^\/]+(\S+)$/) {
		return ($2, $1);
                }
	}
return undef;
}

# generics_file(&config)
# Returns the filename of the text generics file, or undef if none
sub generics_file
{
	my $self = shift;

return $self->find_textfile($self->{'config_hash'}->{'generics_file'}, $self->generics_dbm($_[0]));
}

# list_generics(textfile)
sub list_generics
{
	my $self = shift;

if (!defined(@{$self->{'list_generics_cache_array'}})) {
	@{$self->{'list_generics_cache_array'}} = ( );
	local $lnum = 0;
	local $cmt;
	open(GEN, $_[0]);
	while(<GEN>) {
		s/\r|\n//g;     # remove newlines
		if (/^\s*#+\s*(.*)/) {
			# A comment line
			$cmt = $self->is_table_comment($_);
			}
		elsif (/^(\S+)\s+(.*)/) {
			local(%gen);
			$gen{'from'} = $1;
			$gen{'to'} = $2;
			$gen{'line'} = $cmt ? $lnum-1 : $lnum;
			$gen{'eline'} = $lnum;
			$gen{'file'} = $_[0];
			$gen{'num'} = scalar(@{$self->{'list_generics_cache_array'}});
			$gen{'cmt'} = $cmt;
			push(@{$self->{'list_generics_cache_array'}}, \%gen);
			$cmt = undef;
			}
		else {
			$cmt = undef;
			}
		$lnum++;
		}
	close(GEN);
	}
return @{$self->{'list_generics_cache_array'}};
}

# create_generic(&details, textfile, dbmfile, dbmtype)
# Create a new generic mapping
sub create_generic
{
	my $self = shift;

$self->list_generics($_[1]);	# force cache init
local(%virt);

# Write to the file
local $lref = $self->read_file_lines($_[1]);
$_[0]->{'line'} = scalar(@$lref);
push(@$lref, $self->make_table_comment($_[0]->{'cmt'}));
push(@$lref, "$_[0]->{'from'}\t$_[0]->{'to'}");
$_[0]->{'eline'} = scalar(@$lref)-1;
$self->flush_file_lines($_[1]);

# Add to DBM
if ($_[3] eq "dbm") {
	dbmopen(%virt, $_[2], 0644);
	$virt{$_[0]->{'from'}} = $_[0]->{'to'};
	dbmclose(%virt);
	}
else { $self->run_makemap($_[1], $_[2], $_[3]); }

# Add to cache
$_[0]->{'num'} = scalar(@{$self->{'list_generics_cache_array'}});
$_[0]->{'file'} = $_[1];
push(@{$self->{'list_generics_cache_array'}}, $_[0]);
}

# delete_generic(&details, textfile, dbmfile, dbmtype)
# Delete an existing generic mapping
sub delete_generic
{
	my $self = shift;

local(%virt);

# Delete from file
local $lref = $self->read_file_lines($_[1]);
local $len = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
splice(@$lref, $_[0]->{'line'}, $len);
$self->flush_file_lines($_[1]);

# Delete from DBM
if ($_[3] eq "dbm") {
	dbmopen(%virt, $_[2], 0644);
	delete($virt{$_[0]->{'from'}});
	dbmclose(%virt);
	}
else { $self->run_makemap($_[1], $_[2], $_[3]); }

# Delete from cache
local $idx = $self->indexof($_[0], @{$self->{'list_generics_cache_array'}});
splice(@{$self->{'list_generics_cache_array'}}, $idx, 1) if ($idx != -1);
$self->renumber_list(\@{$self->{'list_generics_cache_array'}}, $_[0], -$len);
}

# modify_generic(&old, &details, textfile, dbmfile, dbmtype)
# Change an existing generic
sub modify_generic
{
	my $self = shift;

local(%virt);

# Update in file
local $lref = $self->read_file_lines($_[2]);
local $oldlen = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
local @newlines;
push(@newlines, $self->make_table_comment($_[1]->{'cmt'}));
push(@newlines, "$_[1]->{'from'}\t$_[1]->{'to'}");
splice(@$lref, $_[0]->{'line'}, $oldlen, @newlines);
$self->flush_file_lines($_[2]);

# Update DBM
if ($_[4] eq "dbm") {
	dbmopen(%virt, $_[3], 0644);
	delete($virt{$_[0]->{'from'}});
	$virt{$_[1]->{'from'}} = $_[1]->{'to'};
	dbmclose(%virt);
	}
else { $self->run_makemap($_[2], $_[3], $_[4]); }

# Update cache
local $idx = $self->indexof($_[0], @{$self->{'list_generics_cache_array'}});
$_[1]->{'line'} = $_[0]->{'line'};
$_[1]->{'eline'} = $_[1]->{'cmt'} ? $_[0]->{'line'}+1 : $_[0]->{'line'};
$self->{'list_generics_cache_array'}->[$idx] = $_[1] if ($idx != -1);
$self->renumber_list(\@{$self->{'list_generics_cache_array'}}, $_[0], scalar(@newlines)-$oldlen);
}

sub generic_form([&details])
{
	my $self = shift;

local $g = $_[0];

print $self->ui_form_start("save_generic.cgi", "post");
if ($g) {
	print $self->ui_hidden("num", $g->{'num'}),"\n";
	}
else {
	print $self->ui_hidden("new", 1),"\n";
	}
print $self->ui_table_start($g ? $self->{'text_hash'}->{'gform_edit'} : $self->{'text_hash'}->{'gform_create'},
		      undef, 2);

print $self->ui_table_row($self->{'text_hash'}->{'vform_cmt'},
		    $self->ui_textbox("cmt", $g->{'cmt'}, 50));

print $self->ui_table_row($self->{'text_hash'}->{'gform_from'},
		    $self->ui_textbox("from", $g->{'from'}, 30));

print $self->ui_table_row($self->{'text_hash'}->{'gform_to'},
		    $self->ui_textbox("to", $g->{'to'}, 30));

print $self->ui_table_end();
print $self->ui_form_end($_[0] ? [ [ "save", $self->{'text_hash'}->{'save'} ],
		    	     [ "delete", $self->{'text_hash'}->{'delete'} ] ]
		         : [ [ "create", $self->{'text_hash'}->{'create'} ] ]);
}

sub can_edit_generic
{
	my $self = shift;

local ($g) = @_;
return $self->{'access_hash'}->{'omode'} == 1 ||
       $self->{'access_hash'}->{'omode'} == 2 && ($g->{'from'} =~ /$self->{'access_hash'}->{'oaddrs'}/ ||
				 $g->{'to'} =~ /$self->{'access_hash'}->{'oaddrs'}/);
}


1;

