#!/usr/bin/perl
package sendmail::MailersLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'sendmail::MailersLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# mailers-lib.pl
# Functions for editing the mailertable database

1;

1;
		return $self;
}

# mailers_dbm(&config)
# Returns the filename of the mailertable database and type, or undef if none
sub mailers_dbm
{
	my $self = shift;

foreach $f ($self->find_type("K", $_[0])) {
        if ($f->{'value'} =~ /^mailertable\s+(\S+)[^\/]+(\/\S+)$/) {
		return ($2, $1);
                }
	}
return undef;
}

# mailers_file(&config)
# Returns the filename of the text mailertable file, or undef if none
sub mailers_file
{
	my $self = shift;

return $self->find_textfile($self->{'config_hash'}->{'mailers_file'}, $self->mailers_dbm($_[0]));
}

# list_mailers(textfile)
sub list_mailers
{
	my $self = shift;

if (!defined(@{$self->{'list_mailers_cache_array'}})) {
	local $lnum = 0;
	@{$self->{'list_mailers_cache_array'}} = ( );
	local $cmt;
	open(MAILER, $_[0]);
	while(<MAILER>) {
		s/\r|\n//g;     # remove newlines
		if (/^\s*#+\s*(.*)/) {
			# A comment line
			$cmt = $self->is_table_comment($_);
			}
		elsif (/^(\S+)\s+([^: ]+):(.*)/) {
			# An actual mailer line
			local(%virt);
			$virt{'domain'} = $1;
			$virt{'mailer'} = $2;
			$virt{'dest'} = $3;
			$virt{'line'} = $cmt ? $lnum-1 : $lnum;
			$virt{'eline'} = $lnum;
			$virt{'num'} = scalar(@{$self->{'list_mailers_cache_array'}});
			$virt{'cmt'} = $cmt;
			push(@{$self->{'list_mailers_cache_array'}}, \%virt);
			$cmt = undef;
			}
		else {
			$cmt = undef;
			}
		$lnum++;
		}
	close(MAILER);
	}
return @{$self->{'list_mailers_cache_array'}};
}

# create_mailer(&details, textfile, dbmfile, dbmtype)
sub create_mailer
{
	my $self = shift;

local(%mailer);
$self->list_mailers($_[1]);	# force cache init

# Write to the file
local $lref = $self->read_file_lines($_[1]);
$_[0]->{'line'} = scalar(@$lref);
push(@$lref, $self->make_table_comment($_[0]->{'cmt'}));
push(@$lref, "$_[0]->{'domain'}\t$_[0]->{'mailer'}:$_[0]->{'dest'}");
$_[0]->{'eline'} = scalar(@$lref)-1;
$self->flush_file_lines();

# Write to the DBM
if ($_[3] eq "dbm") {
	dbmopen(%mailer, $_[2], 0644);
	$mailer{$_[0]->{'domain'}} = "$_[0]->{'mailer'}:$_[0]->{'dest'}";
	dbmclose(%mailer);
	}
else { $self->run_makemap($_[1], $_[2], $_[3]); }

# Update the cache
$_[0]->{'num'} = scalar(@{$self->{'list_mailers_cache_array'}});
$_[0]->{'file'} = $_[1];
push(@{$self->{'list_mailers_cache_array'}}, $_[0]);
}

# modify_mailer(&old, &details, textfile, dbmfile, dbmtype)
sub modify_mailer
{
	my $self = shift;

local(@mailer, %mailer);

# Update the file
local $lref = $self->read_file_lines($_[2]);
local $oldlen = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
local @newlines;
push(@newlines, $self->make_table_comment($_[1]->{'cmt'}));
push(@newlines, "$_[1]->{'domain'}\t$_[1]->{'mailer'}:$_[1]->{'dest'}");
splice(@$lref, $_[0]->{'line'}, $oldlen, @newlines);
$self->flush_file_lines($_[2]);

# Update the DBM
if ($_[3] eq "dbm") {
	dbmopen(%mailer, $_[3], 0644);
	delete($mailer{$_[0]->{'domain'}});
	$mailer{$_[1]->{'domain'}} = "$_[1]->{'mailer'}:$_[1]->{'dest'}";
	dbmclose(%mailer);
	}
else { $self->run_makemap($_[2], $_[3], $_[4]); }

local $idx = $self->indexof($_[0], @{$self->{'list_mailers_cache_array'}});
$_[1]->{'line'} = $_[0]->{'line'};
$_[1]->{'eline'} = $_[1]->{'cmt'} ? $_[0]->{'line'}+1 : $_[0]->{'line'};
$self->{'list_mailers_cache_array'}->[$idx] = $_[1] if ($idx != -1);
$self->renumber_list(\@{$self->{'list_mailers_cache_array'}}, $_[0], scalar(@newlines)-$oldlen);
}

# delete_mailer(&old, textfile, dbmfile, dbmtype)
sub delete_mailer
{
	my $self = shift;

local(@mailer, %mailer);

# Delete from the file
local $len = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
local $lref = $self->read_file_lines($_[1]);
splice(@$lref, $_[0]->{'line'}, $len);
$self->flush_file_lines($_[1]);

# Delete f rom the DBM
if ($_[3] eq "dbm") {
	dbmopen(%mailer, $_[2], 0644);
	delete($mailer{$_[0]->{'domain'}});
	dbmclose(%mailer);
	}
else { $self->run_makemap($_[1], $_[2], $_[3]); }

# Update the cache
local $idx = $self->indexof($_[0], @{$self->{'list_mailers_cache_array'}});
splice(@{$self->{'list_mailers_cache_array'}}, $idx, 1) if ($idx != -1);
$self->renumber_list(\@{$self->{'list_mailers_cache_array'}}, $_[0], -$len);
}

# mailer_form([&details])
sub mailer_form
{
	my $self = shift;

local ($m) = @_;
local ($mode, $addr, $conf, $ml, $dest, $nomx);

print $self->ui_form_start("save_mailer.cgi", "post");
if ($m) {
	print $self->ui_hidden("num", $m->{'num'});
	}
else {
	print $self->ui_hidden("new", 1);
	}
print $self->ui_table_start($m ? $self->{'text_hash'}->{'mform_edit'} : $self->{'text_hash'}->{'mform_create'},
		      undef, 2);

# Description
print $self->ui_table_row($self->{'text_hash'}->{'vform_cmt'},
		    $self->ui_textbox("cmt", $m ? $m->{'cmt'} : undef, 50));

# Show 'mail for' input
local $dom = $m && $m->{'domain'} =~ /^\.(\S+)$/ ? $1 : undef;
print $self->ui_table_row($self->{'text_hash'}->{'mform_for'},
    $self->ui_radio_table("from_type", $dom ? 1 : 0,
	[ [ 0, $self->{'text_hash'}->{'mform_host2'},
	       $self->ui_textbox("from_host", $dom || !$m ? "" : $m->{'domain'},
			   20) ],
	  [ 1, $self->{'text_hash'}->{'mform_domain2'},
	       $self->ui_textbox("from_dom", $dom, 20) ],
	  $m ? ( ) : ( [ 2, $self->{'text_hash'}->{'mform_domain3'},
			    $self->ui_textbox("from_all", undef ,20) ] ) ]));

# Show delivery input
$conf = $self->get_sendmailcf();
local @mailers = ( { 'value' => 'error' }, $self->find_type("M", $conf) );
print $self->ui_table_row($self->{'text_hash'}->{'mform_delivery'},
    $self->ui_select("mailer",
	       $m ? $m->{'mailer'} : "smtp",
	       [ map { $_->{'value'} =~ /^([^ ,]+)/;
		       [ $1, $self->{'text_hash'}->{"mform_$1"} ] } @mailers ]));

# Show send to input
if ($m->{'dest'} =~ /^\[(\S+)\]$/) { $dest = $1; $nomx = 1; }
else { $dest = $m->{'dest'}; }
print $self->ui_table_row($self->{'text_hash'}->{'mform_to'},
    $self->ui_textbox("dest", $dest, 30)."<br>".
    $self->ui_checkbox("nomx", 1, $self->{'text_hash'}->{'mform_ignore'}, $nomx));

print $self->ui_table_end();
print $self->ui_form_end($_[0] ? [ [ "save", $self->{'text_hash'}->{'save'} ],
			     [ "delete", $self->{'text_hash'}->{'delete'} ] ]
		         : [ [ "create", $self->{'text_hash'}->{'create'} ] ]);
}


1;

