#!/usr/bin/perl
package sendmail::SendmailLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'sendmail::SendmailLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# sendmail-lib.pl
# Functions for managing sendmail aliases, domains and mappings.
# Only sendmail versions 8.8 and above are supported

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();
$self->{'features_access'} = $self->{'access_hash'}->{'opts'} && $self->{'access_hash'}->{'cws'} && $self->{'access_hash'}->{'masq'} && $self->{'access_hash'}->{'trusts'} && $self->{'access_hash'}->{'vmode'} && $self->{'access_hash'}->{'amode'} && $self->{'access_hash'}->{'omode'} && $self->{'access_hash'}->{'cgs'} && $self->{'access_hash'}->{'relay'} && $self->{'access_hash'}->{'mailers'} && $self->{'access_hash'}->{'access'} && $self->{'access_hash'}->{'domains'};
$self->{'config_hash'}->{'perpage'} ||= 20;	# a value of 0 can cause problems

1;

1;
		return $self;
}

# get_sendmailcf()
# Parses sendmail.cf and return a reference to an array of options.
# Each line is a single character directive, followed by a list of values?
sub get_sendmailcf
{
	my $self = shift;

if (!@{$self->{'sendmailcf_cache_array'}}) {
	local($lnum, $i);
	$lnum = 0; $i = 0;
	open(CF, $self->{'config_hash'}->{'sendmail_cf'});
	while(<CF>) {
		s/^#.*$//g;	# remove comments
		s/\r|\n//g;	# remove newlines
		if (/^(\S)(\s*(.*))$/) {
			local(%opt);
			$opt{'type'} = $1;
			$opt{'value'} = $3;
			$opt{'values'} = [ split(/\s+/, $2) ];
			$opt{'line'} = $lnum;
			$opt{'eline'} = $opt{'line'};
			$opt{'pos'} = $i++;
			push(@{$self->{'sendmailcf_cache_array'}}, \%opt);
			}
		$lnum++;
		}
	close(CF);
	}
return \@{$self->{'sendmailcf_cache_array'}};
}

# check_sendmail_version(&config)
# Is the sendmail config file a usable version?
sub check_sendmail_version
{
	my $self = shift;

local $ver = $self->find_type("V", $_[0]);
return $ver && $ver->{'value'} =~ /^(\d+)/ && $1 >= 7 ? $1 : undef;
}

# get_sendmail_version(&out)
# Returns the actual sendmail executable version, if it is available
sub get_sendmail_version
{
	my $self = shift;

local $out = $self->backquote_with_timeout("$self->{'config_hash'}->{'sendmail_path'} -d0 -bv 2>&1",
				     2, undef, 1);
local $version;
if ($out =~ /version\s+(\S+)/i) {
	$version = $1;
	}
${$_[0]} = $out if ($_[0]);
return $version;
}

# save_directives(&config, &oldvalues, &newvalues)
# Given 2 arrays of directive structures, this function will replace the
# old ones with the new. If the old list is empty, new directives are added
# to the end of the config file. If the new list is empty, all old directives
# are removed. If both exist, new ones replace old..
sub save_directives
{
	my $self = shift;

local(@old) = @{$_[1]};
local(@new) = @{$_[2]};
$self->{'lref'} = $self->read_file_lines($self->{'config_hash'}->{'sendmail_cf'});
for($i=0; $i<@old || $i<@new; $i++) {
	if ($i >= @old) {
		# A new directive has been added.. put it at the end of the file
		$new[$i]->{'line'} = scalar(@{$self->{'lref'}});
		$new[$i]->{'eline'} = $new[$i]->{'line'}+1;
		push(@{$self->{'lref'}}, $self->directive_line($new[$i]));
		push(@{$_[0]}, $new[$i]);
		}
	elsif ($i >= @new) {
		# A directive was deleted
		$self->{'ol'} = $old[$i]->{'eline'} - $old[$i]->{'line'} + 1;
		splice(@{$self->{'lref'}}, $old[$i]->{'line'}, $self->{'ol'});
		$self->renumber_list($_[0], $old[$i], -$self->{'ol'});
		splice(@{$_[0]}, $self->indexof($old[$i], @{$_[0]}), 1);
		}
	else {
		# A directive was changed
		$self->{'ol'} = $old[$i]->{'eline'} - $old[$i]->{'line'} + 1;
		splice(@{$self->{'lref'}}, $old[$i]->{'line'}, $self->{'ol'},
		       $self->directive_line($new[$i]));
		$new[$i]->{'line'} = $new[$i]->{'eline'} = $old[$i]->{'line'};
		$self->renumber_list($_[0], $old[$i], 1-$self->{'ol'});
		$_[0]->[$self->indexof($old[$i], @{$_[0]})] = $new[$i];
		}
	}
}

# directive_line(&details)
sub directive_line
{
	my $self = shift;

return $_[0]->{'type'}.join(' ', @{$_[0]->{'values'}});
}

# find_type(name, &config)
# Returns an array of config directives of some type
sub find_type
{
	my $self = shift;

local($c, @rv);
foreach $c (@{$_[1]}) {
	if ($c->{'type'} eq $_[0]) {
		push(@rv, $c);
		}
	}
return @rv ? wantarray ? @rv : $rv[0]
           : wantarray ? () : undef;
}

# find_option(name, &config)
# Returns the structure and value of some option directive
sub find_option
{
	my $self = shift;

local(@opts, $o);
@opts = $self->find_type("O", $_[1]);
foreach $o (@opts) {
	if ($o->{'value'} =~ /^\s*([^=]+)=(.*)$/ && $1 eq $_[0]) {
		# found it.. return
		return wantarray ? ($o, $2) : $2;
		}
	}
return undef;
}

# find_optionss(name, &config)
# Returns the structures and values of some option directive
sub find_options
{
	my $self = shift;

local(@opts, $o);
@opts = $self->find_type("O", $_[1]);
foreach $o (@opts) {
	if ($o->{'value'} =~ /^\s*([^=]+)=(.*)$/ && $1 eq $_[0]) {
		push(@{$self->{'rv_array'}}, [ $o, $2 ]);
		}
	}
return wantarray ? @{$self->{'rv_array'}} : $self->{'rv_array'}->[0];
}

# find_type2(type1, type2, &config)
# Returns the structure and value of some directive
sub find_type2
{
	my $self = shift;

local @types = $self->find_type($_[0], $_[2]);
local $t;
foreach $t (@types) {
	if ($t->{'value'} =~ /^(\S)(.*)$/ && $1 eq $_[1]) {
		return ($t, $2);
		}
	}
return undef;
}

# restart_sendmail()
# Send a SIGHUP to sendmail
sub restart_sendmail
{
	my $self = shift;

if ($self->{'config_hash'}->{'sendmail_restart_command'}) {
	# Use the restart command
	local $out = $self->backquote_logged("$self->{'config_hash'}->{'sendmail_restart_command'} 2>&1 </dev/null");
	return $? || $out =~ /failed|error/i ? "<pre>$out</pre>" : undef;
	}
else {
	# Just HUP the process
	local ($pid, $any);
	foreach my $pidfile (split(/\t+/, $self->{'config_hash'}->{'sendmail_pid'})) {
		if (open(PID, $pidfile)) {
			chop($pid = <PID>);
			close(PID);
			if ($pid) { $self->kill_logged('HUP', $pid); }
			$any++;
			}
		}
	if (!$any) {
		local @pids = $self->find_byname("sendmail");
		@pids || return $self->{'text_hash'}->{'restart_epids'};
		$self->kill_logged('HUP', @pids) ||
			return $self->text('restart_ekill', $!);
		}
	return undef;
	}
}

# run_makemap(textfile, dbmfile, type)
sub run_makemap
{
	my $self = shift;

local($out);
$out = $self->backquote_logged("$self->{'config_hash'}->{'makemap_path'} $_[2] $_[1] <\"$_[0]\" 2>&1");
if ($?) { $self->error("makemap failed : <pre>$out</pre>"); }
}

# find_textfile(config, dbm)
sub find_textfile
{
	my $self = shift;

local($conf, $dbm) = @_;
if ($conf) { return $conf; }
elsif (!$dbm) { return undef; }
elsif ($dbm =~ /^(.*)\.(db|dbm|pag|dir|hash)$/i && -r $1) {
	# Database is like /etc/virtusertable.db, text is /etc/virtusertable
	return $1;
	}
elsif ($dbm =~ /^(.*)\.(db|dbm|pag|dir|hash)$/i && -r "$1.txt") {
	# Database is like /etc/virtusertable.db, text is /etc/virtusertable.txt
	return "$1.txt";
	}
elsif (-r "$dbm.txt") {
	# Database is like /etc/virtusertable, text is /etc/virtusertable.txt
	return "$dbm.txt";
	}
else {
	# Text and database have same name
	return $dbm;
	}
}

# mailq_dir($conf)
sub mailq_dir
{
	my $self = shift;

local ($opt, $mqueue) = $self->find_option("QueueDirectory", $_[0]);
local @rv;
if (!$mqueue) { @rv = ( "/var/spool/mqueue" ); }
elsif ($mqueue =~ /\*|\?/) {
	@rv = split(/\s+/, `echo $mqueue`);
	}
else {
	@rv = ( $mqueue );
	}
push(@rv, split(/\s+/, $self->{'config_hash'}->{'queue_dirs'}));
return @rv;
}

sub sort_by_domain
{
	my $self = shift;

local ($a1, $a2, $b1, $b2);
if ($self->{'a'}->{'from'} =~ /^(.*)\@(.*)$/ && (($a1, $a2) = ($1, $2)) &&
    $self->{'b'}->{'from'} =~ /^(.*)\@(.*)$/ && (($b1, $b2) = ($1, $2))) {
	return $a2 cmp $b2 ? $a2 cmp $b2 : $a1 cmp $b1;
	}
else {
	return $self->{'a'}->{'from'} cmp $self->{'b'}->{'from'};
	}
}

# can_view_qfile(&mail)
# Returns 1 if some queued message can be viewed, 0 if not
sub can_view_qfile
{
	my $self = shift;

return 1 if (!$self->{'access_hash'}->{'qdoms'});
local $re = $self->{'access_hash'}->{'qdoms'};
if ($self->{'access_hash'}->{'qdomsmode'} == 0) {
	return $_[0]->{'header'}->{'from'} =~ /$re/i;
	}
elsif ($self->{'access_hash'}->{'qdomsmode'} == 0) {
	return $_[0]->{'header'}->{'to'} =~ /$re/i;
	}
else {
	return $_[0]->{'header'}->{'from'} =~ /$re/i ||
	       $_[0]->{'header'}->{'to'} =~ /$re/i;
	}
}

# renumber_list(&list, &position-object, lines-offset)
sub renumber_list
{
	my $self = shift;

return if (!$_[2]);
local $e;
foreach $e (@{$_[0]}) {
	if (!defined($e->{'file'}) || $e->{'file'} eq $_[1]->{'file'}) {
		$e->{'line'} += $_[2] if ($e->{'line'} > $_[1]->{'line'});
		$e->{'eline'} += $_[2] if (defined($e->{'eline'}) &&
					   $e->{'eline'} > $_[1]->{'eline'});
		}
	}
}

# get_file_or_config(&config, suffix, [additional-conf], [&cwfile])
# Returns all values for some config file entries, which may be in sendmail.cf
# (like Cw) or externally (like Fw)
sub get_file_or_config
{
	my $self = shift;

local ($conf, $suffix, $addit, $cwref) = @_;
local ($cwfile, $f);
foreach $f ($self->find_type("F", $conf)) {
	if ($f->{'value'} =~ /^${suffix}[^\/]*(\/\S+)/ ||
	    $f->{'value'} =~ /^\{${suffix}\}[^\/]*(\/\S+)/) {
		$cwfile = $1;
		}
	}
local @rv;
if ($cwfile) {
	# get entries listed in a separate file
	$$cwref = $cwfile if ($cwref);
	open(CW, $cwfile);
	while(<CW>) {
		s/\r|\n//g;
		s/#.*$//g;
		if (/\S/) { push(@rv, $_); }
		}
	close(CW);
	}
else {
	$$cwref = undef if ($cwref);
	}
# Add entries from sendmail.cf
foreach $f ($self->find_type("C", $conf)) {
	if ($f->{'value'} =~ /^${suffix}\s*(.*)$/ ||
	    $f->{'value'} =~ /^\{${suffix}\}\s*(.*)$/) {
		push(@rv, split(/\s+/, $1));
		}
	}
if ($addit) {
	push(@rv, map { $_->{'value'} } $self->find_type($addit, $conf));
	}
return $self->unique(@rv);
}

# save_file_or_config(&conf, suffix, &values, [additional-conf])
# Updates the values in some external file or in sendmail.cf
sub save_file_or_config
{
	my $self = shift;

local ($conf, $suffix, $values, $addit) = @_;
local ($cwfile, $f);
foreach $f ($self->find_type("F", $conf)) {
	if ($f->{'value'} =~ /^${suffix}[^\/]*(\/\S+)/ ||
	    $f->{'value'} =~ /^\{${suffix}\}[^\/]*(\/\S+)/) {
		$cwfile = $1;
		}
	}
local @old = grep { $_->{'value'} =~ /^${suffix}/ ||
		    $_->{'value'} =~ /^\{${suffix}\}/ } $self->find_type("C", $conf);
if ($addit) {
	push(@old, $self->find_type($addit, $conf));
	}
local @new;
local $d;
if ($cwfile) {
	# If there is a .cw file, write all entries to it and take any
	# out of sendmail.cf
	$self->open_tempfile(CW, ">$cwfile");
	foreach $d (@$values) {
		$self->print_tempfile(CW, $d,"\n");
		}
	$self->close_tempfile(CW);
	}
else {
	# Stick all entries in sendmail.cf
	foreach $d (@$values) {
		push(@new, { 'type' => 'C',
			     'values' => [ $suffix.$d ] });
		}
	}
$self->save_directives($conf, \@old, \@new);
}

# add_file_or_config(&config, suffix, value)
# Adds an entry to sendmail.cf or an external file
sub add_file_or_config
{
	my $self = shift;

local ($conf, $suffix, $value) = @_;
local ($cwfile, $f);
foreach $f ($self->find_type("F", $conf)) {
	if ($f->{'value'} =~ /^${suffix}[^\/]*(\/\S+)/ ||
	    $f->{'value'} =~ /^\{${suffix}\}[^\/]*(\/\S+)/) {
		$cwfile = $1;
		}
	}
local @old = grep { $_->{'value'} =~ /^${suffix}/ ||
		    $_->{'value'} =~ /^\{${suffix}\}/ } $self->find_type("C", $conf);
if ($cwfile) {
	# Add to external file
	$self->open_tempfile(CW, ">>$cwfile");
	$self->print_tempfile(CW, $value,"\n");
	$self->close_tempfile(CW);
	}
else {
	# Add to sendmail.cf
	local @new = ( @old, { 'type' => 'C',
			       'values' => [ $suffix.$value ] });
	$self->save_directives($conf, \@old, \@new);
	}
}

# delete_file_or_config(&config, suffix, value)
# Removes an entry from sendmail.cf or an external file
sub delete_file_or_config
{
	my $self = shift;

local ($conf, $suffix, $value) = @_;
local ($cwfile, $f);
foreach $f ($self->find_type("F", $conf)) {
	if ($f->{'value'} =~ /^${suffix}[^\/]*(\/\S+)/ ||
	    $f->{'value'} =~ /^\{${suffix}\}[^\/]*(\/\S+)/) {
		$cwfile = $1;
		}
	}
local @old = grep { $_->{'value'} =~ /^${suffix}/ ||
		    $_->{'value'} =~ /^\{${suffix}\}/ } $self->find_type("C", $conf);
if ($cwfile) {
	# Remove from external file
	local $lref = $self->read_file_lines($cwfile);
	@$lref = grep { $_ !~ /^\s*\Q$value\E\s*$/i } @$lref;
	$self->flush_file_lines($cwfile);
	}
else {
	# Remove from sendmail.cf
	local @new = grep { $_->{'values'}->[0] ne $suffix.$value } @old;
	$self->save_directives($conf, \@old, \@new);
	}
}

# list_mail_queue([&conf])
# Returns a list of all files in the mail queue
sub list_mail_queue
{
	my $self = shift;

local ($mqueue, @qfiles);
local $conf = $_[0] || $self->get_sendmailcf();
foreach $mqueue ($self->mailq_dir($conf)) {
	opendir(QDIR, $mqueue);
	push(@qfiles, map { "$mqueue/$_" } grep { /^(qf|hf|Qf)/ } readdir(QDIR));
	closedir(QDIR);
	}
return @qfiles;
}

# list_dontblames()
# Returns an array of valid options for the DontBlameSendmail option
sub list_dontblames
{
	my $self = shift;

local @rv;
open(BLAME, "$self->{'module_root_directory'}/dontblames");
while(<BLAME>) {
	s/\r|\n//g;
	s/^\s*#.*$//;
	if (/^(\S+)\s+(\S.*)$/) {
		push(@rv, [ $1, $2 ]);
		}
	}
close(BLAME);
return @rv;
}

# stop_sendmail()
# Stops the sendmail process, returning undef on success or an error message
# upon failure.
sub stop_sendmail
{
	my $self = shift;

if ($self->{'config_hash'}->{'sendmail_stop_command'}) {
	local $out = $self->backquote_logged("$self->{'config_hash'}->{'sendmail_stop_command'} </dev/null 2>&1");
	if ($?) {
		return "<pre>$out</pre>";
		}
	}
else {
	foreach my $pidfile (split(/\t+/, $self->{'config_hash'}->{'sendmail_pid'})) {
		local $pid = $self->check_pid_file($pidfile);
		if ($pid && $self->kill_logged('KILL', $pid)) {
			unlink($pidfile);
			}
		else {
			return $self->{'text_hash'}->{'stop_epid'};
			}
		}
	}
return undef;
}

# start_sendmail()
# Starts the sendmail server, returning undef on success or an error message
# upon failure.
sub start_sendmail
{
	my $self = shift;

if ($self->{'config_hash'}->{'sendmail_stop_command'}) {
	# Make sure any init script lock files are gone
	$self->backquote_logged("$self->{'config_hash'}->{'sendmail_stop_command'} </dev/null 2>&1");
	}
local $out = $self->backquote_logged("$self->{'config_hash'}->{'sendmail_command'} </dev/null 2>&1");
return $? ? "<pre>$out</pre>" : undef;
}

sub is_sendmail_running
{
	my $self = shift;

local @pidfiles = split(/\t+/, $self->{'config_hash'}->{'sendmail_pid'});
return $self->check_pid_file($pidfiles[0]);
}

# mailq_table(&qfiles)
# Print a table showing queued emails
sub mailq_table
{
	my $self = shift;

local ($qfiles, $qmails) = @_;

# Show buttons to flush and delete
print "<form action=del_mailqs.cgi method=post>\n";
local @links = ( $self->select_all_link("file", 0),
	         $self->select_invert_link("file", 0) );
if ($self->{'config_hash'}->{'top_buttons'}) {
	if ($self->{'access_hash'}->{'mailq'} == 2) {
		print "<input type=submit value='$self->{'text_hash'}->{'mailq_delete'}'>\n";
		print "<input type=checkbox name=locked value=1> $self->{'text_hash'}->{'mailq_locked'}\n";
		print "&nbsp;&nbsp;\n";
		print "<input type=submit name=flush value='$self->{'text_hash'}->{'mailq_flushsel'}'>\n";
		print "<p>\n";
		print $self->ui_links_row(\@links);
		}
	}

# Generate table header
local (@hcols, @tds);
if ($self->{'access_hash'}->{'mailq'} == 2) {
	push(@hcols, "");
	push(@tds, "width=5");
	}
local %show;
foreach my $s (split(/,/, $self->{'config_hash'}->{'mailq_show'})) {
	$show{$s}++;
	}
push(@hcols, $self->{'text_hash'}->{'mailq_id'});
push(@hcols, $self->{'text_hash'}->{'mailq_sent'}) if ($show{'Date'});
push(@hcols, $self->{'text_hash'}->{'mailq_from'}) if ($show{'From'});
push(@hcols, $self->{'text_hash'}->{'mailq_to'}) if ($show{'To'});
push(@hcols, $self->{'text_hash'}->{'mailq_cc'}) if ($show{'Cc'});
push(@hcols, $self->{'text_hash'}->{'mailq_subject'}) if ($show{'Subject'});
push(@hcols, $self->{'text_hash'}->{'mailq_size'}) if ($show{'Size'});
push(@hcols, $self->{'text_hash'}->{'mailq_status'}) if ($show{'Status'});
print $self->ui_columns_start(\@hcols, 100, 0, \@tds);

# Show table rows for emails
foreach my $f (@$qfiles) {
	local $n;
	($n = $f) =~ s/^.*\///;
	local $mail = $qmails->{$f} || $self->mail_from_queue($f);
	next if (!$mail);

	$mail->{'header'}->{'from'} ||= $self->{'text_hash'}->{'mailq_unknown'};
	$mail->{'header'}->{'to'} ||= $self->{'text_hash'}->{'mailq_unknown'};
	$mail->{'header'}->{'date'} ||= $self->{'text_hash'}->{'mailq_unknown'};
	$mail->{'header'}->{'subject'} ||= $self->{'text_hash'}->{'mailq_unknown'};
	$mail->{'header'}->{'cc'} ||= "&nbsp;";
	if ($mail->{'quar'}) {
		$mail->{'status'} = $self->{'text_hash'}->{'mailq_quar'};
		$self->{'quarcount'}++;
		}
	$mail->{'status'} ||= $self->{'text_hash'}->{'mailq_sending'};

	$mail->{'header'}->{'from'} =
		$self->html_escape($mail->{'header'}->{'from'});
	$mail->{'header'}->{'to'} =
		$self->html_escape($mail->{'header'}->{'to'});
	$mail->{'header'}->{'date'} =~ s/\+.*//g;

	local @cols;
	$self->{'size'} = $self->nice_size($mail->{'size'});
	if ($self->{'access_hash'}->{'mailq'} == 2) {
		push(@cols, "<a href=\"view_mailq.cgi?".
			    "file=$f\">$n</a>");
		}
	else {
		push(@cols, $n);
		}
	push(@cols, "<font size=1>$mail->{'header'}->{'date'}</font>") if ($show{'Date'});
	push(@cols, "<font size=1>$mail->{'header'}->{'from'}</font>") if ($show{'From'});
	push(@cols, "<font size=1>$mail->{'header'}->{'to'}</font>") if ($show{'To'});
	push(@cols, "<font size=1>$mail->{'header'}->{'cc'}</font>") if ($show{'Cc'});
	push(@cols, "<font size=1>$mail->{'header'}->{'subject'}</font>") if ($show{'Subject'});
	push(@cols, "<font size=1>$self->{'size'}</font>") if ($show{'Size'});
	push(@cols, "<font size=1>$mail->{'status'}</font>") if ($show{'Status'});
	print "</tr>\n";
	if ($self->{'access_hash'}->{'mailq'} == 2) {
		print $self->ui_checked_columns_row(\@cols, \@tds, "file",$f);
		}
	else {
		print $self->ui_columns_row(\@cols, \@tds);
		}
	}
print $self->ui_columns_end();
if ($self->{'access_hash'}->{'mailq'} == 2) {
	print $self->ui_links_row(\@links);
	print "<input type=submit value='$self->{'text_hash'}->{'mailq_delete'}'>\n";
	print "<input type=checkbox name=locked value=1> $self->{'text_hash'}->{'mailq_locked'}\n";

	print "&nbsp;&nbsp;\n";
	print "<input type=submit name=flush value='$self->{'text_hash'}->{'mailq_flushsel'}'>\n";
	print "<p>\n";
	}
print "</form>\n";
}

# is_table_comment(line, [force-prefix])
# Returns the comment text if a line contains a comment, like # foo
sub is_table_comment
{
	my $self = shift;

local ($line, $force) = @_;
if ($self->{'config_hash'}->{'prefix_cmts'} || $force) {
	return $line =~ /^\s*#+\s*Webmin:\s*(.*)/ ? $1 : undef;
	}
else {
	return $line =~ /^\s*#+\s*(.*)/ ? $1 : undef;
	}
}

# make_table_comment(comment, [force-tag])
# Returns an array of lines for a comment in a map file, like # foo
sub make_table_comment
{
	my $self = shift;

local ($cmt, $force) = @_;
if (!$cmt) {
	return ( );
	}
elsif ($self->{'config_hash'}->{'prefix_cmts'} || $force) {
	return ( "# Webmin: $cmt" );
	}
else {
	return ( "# $cmt" );
	}
}


1;

