#!/usr/bin/perl
package sendmail::VirtusersLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'sendmail::VirtusersLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# virtusers-lib.pl
# Functions for the virtusers table

1;

1;
		return $self;
}

# virtusers_dbm(&config)
# Returns the filename and type of the virtusers database, or undef if none
sub virtusers_dbm
{
	my $self = shift;

foreach $f ($self->find_type("K", $_[0])) {
        if ($f->{'value'} =~ /^virtuser\s+(\S+)[^\/]+(\S+)$/) {
		return ($2, $1);
                }
	}
return undef;
}

# virtusers_file(&config)
# Returns the filename of the text virtusers file, or undef if none
sub virtusers_file
{
	my $self = shift;

return $self->find_textfile($self->{'config_hash'}->{'virtusers_file'}, $self->virtusers_dbm($_[0]));
}

# list_virtusers(textfile)
sub list_virtusers
{
	my $self = shift;

if (!defined(@{$self->{'list_virtusers_cache_array'}})) {
	@{$self->{'list_virtusers_cache_array'}} = ( );
	local $lnum = 0;
	local $cmt;
	open(VIRT, $_[0]);
	while(<VIRT>) {
		s/\r|\n//g;     # remove newlines
		if (/^\s*#+\s*(.*)/) {
			# A comment line
			$cmt = $self->is_table_comment($_);
			}
		elsif (/^(\S+)\s+(.*)/) {
			# An actual virtuser line
			local(%virt);
			$virt{'from'} = $1;
			$virt{'to'} = $2;
			$virt{'line'} = $cmt ? $lnum-1 : $lnum;
			$virt{'eline'} = $lnum;
			$virt{'file'} = $_[0];
			$virt{'num'} = scalar(@{$self->{'list_virtusers_cache_array'}});
			$virt{'cmt'} = $cmt;
			push(@{$self->{'list_virtusers_cache_array'}}, \%virt);
			$cmt = undef;
			}
		else {
			$cmt = undef;
			}
		$lnum++;
		}
	close(VIRT);
	}
return @{$self->{'list_virtusers_cache_array'}};
}

# create_virtuser(&details, textfile, dbmfile, dbmtype)
# Create a new virtuser mapping
sub create_virtuser
{
	my $self = shift;

$self->list_virtusers($_[1]);	# force cache init
local(%virt);

# Add to file
local $lref = $self->read_file_lines($_[1]);
$_[0]->{'line'} = scalar(@$lref);
push(@$lref, $self->make_table_comment($_[0]->{'cmt'}));
push(@$lref, "$_[0]->{'from'}\t$_[0]->{'to'}");
$_[0]->{'eline'} = scalar(@$lref)-1;
$self->flush_file_lines($_[1]);

# Add to DBM
if ($_[3] eq "dbm") {
	dbmopen(%virt, $_[2], 0644);
	$virt{$_[0]->{'from'}} = $_[0]->{'to'};
	dbmclose(%virt);
	}
else { $self->run_makemap($_[1], $_[2], $_[3]); }

# Add to cache
$_[0]->{'num'} = scalar(@{$self->{'list_virtusers_cache_array'}});
$_[0]->{'file'} = $_[1];
push(@{$self->{'list_virtusers_cache_array'}}, $_[0]);
}

# delete_virtuser(&details, textfile, dbmfile, dbmtype)
# Delete an existing virtuser mapping
sub delete_virtuser
{
	my $self = shift;

local %virt;

# Delete  from file
local $lref = $self->read_file_lines($_[1]);
local $len = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
splice(@$lref, $_[0]->{'line'}, $len);
$self->flush_file_lines($_[1]);

# Delete from DBM
if ($_[3] eq "dbm") {
	dbmopen(%virt, $_[2], 0644);
	delete($virt{$_[0]->{'from'}});
	dbmclose(%virt);
	}
else { $self->run_makemap($_[1], $_[2], $_[3]); }

# Delete from cache
local $idx = $self->indexof($_[0], @{$self->{'list_virtusers_cache_array'}});
splice(@{$self->{'list_virtusers_cache_array'}}, $idx, 1) if ($idx != -1);
$self->renumber_list(\@{$self->{'list_virtusers_cache_array'}}, $_[0], -$len);
}

# modify_virtuser(&old, &details, textfile, dbmfile, dbmtype)
# Change an existing virtuser
sub modify_virtuser
{
	my $self = shift;

local %virt;

# Update in file
local $lref = $self->read_file_lines($_[2]);
local $oldlen = $_[0]->{'eline'} - $_[0]->{'line'} + 1;
local @newlines;
push(@newlines, $self->make_table_comment($_[1]->{'cmt'}));
push(@newlines, "$_[1]->{'from'}\t$_[1]->{'to'}");
splice(@$lref, $_[0]->{'line'}, $oldlen, @newlines);
$self->flush_file_lines($_[2]);

# Update DBM
if ($_[4] eq "dbm") {
	dbmopen(%virt, $_[3], 0644);
	delete($virt{$_[0]->{'from'}});
	$virt{$_[1]->{'from'}} = $_[1]->{'to'};
	dbmclose(%virt);
	}
else { $self->run_makemap($_[2], $_[3], $_[4]); }

# Update cache
local $idx = $self->indexof($_[0], @{$self->{'list_virtusers_cache_array'}});
$_[1]->{'line'} = $_[0]->{'line'};
$_[1]->{'eline'} = $_[1]->{'cmt'} ? $_[0]->{'line'}+1 : $_[0]->{'line'};
$self->{'list_virtusers_cache_array'}->[$idx] = $_[1] if ($idx != -1);
$self->renumber_list(\@{$self->{'list_virtusers_cache_array'}}, $_[0], scalar(@newlines)-$oldlen);
}

# virtuser_form([&details])
sub virtuser_form
{
	my $self = shift;

local ($v) = @_;
local ($mode, $addr);

print $self->ui_form_start("save_virtuser.cgi", "post");
if ($v) {
	print $self->ui_hidden("num", $v->{'num'});
	}
else {
	print $self->ui_hidden("new", 1);
	}
print $self->ui_table_start($v ? $self->{'text_hash'}->{'vform_edit'} : $self->{'text_hash'}->{'vform_create'},
		      undef, 2);

# Description
print $self->ui_table_row($self->{'text_hash'}->{'vform_cmt'},
	$self->ui_textbox("cmt", $v ? $v->{'cmt'} : undef, 50));

# Source address
$addr = !$v || $v->{'from'} =~ /^(\S+)\@(\S+)$/;
print $self->ui_table_row($self->{'text_hash'}->{'vform_for'},
	$self->ui_radio_table("from_type", $addr ? 0 : 1,
	  [ [ 0, $self->{'text_hash'}->{'vform_address'},
		 $self->ui_textbox("from_addr", $addr ? $v->{'from'} : "", 20) ],
	    [ 1, $self->{'text_hash'}->{'vform_domain'},
		 $self->ui_textbox("from_dom",
			     $addr ? "" : substr($v->{'from'}, 1), 20) ] ]));
		
# Virtuser destination
$mode = !$v ? 2 :
	$v->{'to'} =~ /^error:(\S+)\s*(.*)$/ ? 0 :
	$v->{'to'} =~ /^\%1\@(\S+)$/ ? 1 :
	$v->{'to'} =~ /^(.*)$/ ? 2 : 2;
local ($one, $two) = ($1, $2);
local @opts;
if ($self->{'access_hash'}->{'vedit_2'}) {
	# Some address
	push(@opts, [ 2, $self->{'text_hash'}->{'vform_address'},
		      $self->ui_textbox("to_addr", $mode == 2 ? $one : "", 20) ]);
	}
if ($self->{'access_hash'}->{'vedit_1'}) {
	# Another domain
	push(@opts, [ 1, $self->{'text_hash'}->{'vform_domain'},
		      $self->ui_textbox("to_dom", $mode == 1 ? $one : "", 15) ]);
	}
if ($self->{'access_hash'}->{'vedit_0'}) {
	# Return an error
	push(@opts, [ 0, $self->{'text_hash'}->{'vform_error'},
		      $self->ui_select("to_code", $one,
			[ map { [ $_, $self->{'text_hash'}->{'vform_err_'.$_} ] }
			      ( "nouser", "nohost", "unavailable",
				"tempfail", "protocol" ) ])." ".
		      $self->ui_textbox("to_error", $mode == 0 ? $two : "", 15)
		      ]);
	}
print $self->ui_table_row($self->{'text_hash'}->{'vform_to'},
	$self->ui_radio_table("to_type", $mode, \@opts));

print $self->ui_table_end();
print $self->ui_form_end($_[0] ? [ [ "save", $self->{'text_hash'}->{'save'} ],
			     [ "delete", $self->{'text_hash'}->{'delete'} ] ]
		         : [ [ "create", $self->{'text_hash'}->{'create'} ] ]);
}

# virt_type(string)
# Return the type and destination of some virtuser target
sub virt_type
{
	my $self = shift;

local @rv;
if ($_[0] =~ /^error:(.*)$/) {
	@rv = (0, $1);
	}
elsif ($_[0] =~ /^\%1\@(\S+)$/) {
	@rv = (1, $1);
	}
else {
	@rv = (2, $_[0]);
	}
return wantarray ? @rv : $rv[0];
}

sub can_edit_virtuser
{
	my $self = shift;

local ($v) = @_;
return $self->{'access_hash'}->{'vmode'} == 1 ||
       $self->{'access_hash'}->{'vmode'} == 2 && $v->{'from'} =~ /$self->{'access_hash'}->{'vaddrs'}/ ||
       $self->{'access_hash'}->{'vmode'} == 3 && $v->{'from'} =~ /^$self->{'remote_user'}\@/;
}


1;

