#!/usr/bin/perl
package sentry::SentryLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'sentry::SentryLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# sentry-lib.pl
# Functions for configuring portsentry, hostsentry and logcheck

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

1;

1;
		return $self;
}

# get_portsentry_config()
# Parses the portsentry.conf file
sub get_portsentry_config
{
	my $self = shift;

return $self->get_config($self->{'config_hash'}->{'portsentry_config'});
}

# get_hostsentry_config()
# Parses the hostsentry.conf file
sub get_hostsentry_config
{
	my $self = shift;

return $self->get_config($self->{'config_hash'}->{'hostsentry_config'});
}

# get_logcheck_config()
# Parses the logcheck.sh program script
sub get_logcheck_config
{
	my $self = shift;

return $self->get_config($self->{'config_hash'}->{'logcheck'});
}

# lock_config_files(&config)
sub lock_config_files
{
	my $self = shift;

foreach $f ($self->unique(map { $_->{'file'} } @{$_[0]})) {
	$self->lock_file($f);
	}
}

# unlock_config_files(&config)
sub unlock_config_files
{
	my $self = shift;

foreach $f ($self->unique(map { $_->{'file'} } @{$_[0]})) {
	$self->unlock_file($f);
	}
}

# $self->get_config(file)
sub get_config
{
	my $self = shift;

local (@rv, $lnum = 0);
open(CONF, $_[0]);
local @lines = <CONF>;
close(CONF);
foreach (@lines) {
	s/\r|\n//g;
	s/#.*$//;
	if (/^([^=\s]+)\s*=\s*"(.*)"/ || /^([^=\s]+)\s*=\s*(\S+)/) {
		push(@rv, { 'name' => $1,
			    'value' => $2,
			    'file' => $_[0],
			    'line' => $lnum });
		}
	elsif (/^\.\s+(\S+)/) {
		# Included file!
		local $inc = $self->get_config("$1");
		push(@rv, @$inc);
		}
	$lnum++;
	}
return \@rv;
}

# save_config(&conf, name, value)
sub save_config
{
	my $self = shift;

local $old = $self->find($_[1], $_[0]);
local $lref = $self->read_file_lines($old ? $old->{'file'} : $_[0]->[0]->{'file'});
local $nl = "$_[1]=\"$_[2]\"";
if ($old) {
	$lref->[$old->{'line'}] = $nl;
	}
else {
	push(@$lref, $nl);
	}
}

# find(name, &config)
sub find
{
	my $self = shift;

foreach $c (@{$_[1]}) {
	if (lc($c->{'name'}) eq lc($_[0])) {
		return $c;
		}
	}
return undef;
}

# $self->find_value(name, &config, subs)
sub find_value
{
	my $self = shift;

local $rv = $self->find($_[0], $_[1]);
return undef if (!defined($rv));
local $str = $rv->{'value'};
if ($_[2]) {
	local %donevar;
	while($str =~ /\$([A-z0-9\_]+)/ && !$donevar{$1}) {
		$donevar{$1}++;
		local $val = $self->find_value($1, $_[1]);
		$str =~ s/\$([A-z0-9\_]+)/$val/;
		}
	}
return $str;
}

# get_portsentry_pids()
sub get_portsentry_pids
{
	my $self = shift;

if ($self->{'config_hash'}->{'portsentry_pid'}) {
	# Get from pid file
	local $pid;
	if (open(PID, $self->{'config_hash'}->{'portsentry_pid'}) && chop($pid = <PID>) &&
	    kill(0, $pid)) {
		close(PID);
		return ( $pid );
		}
	else {
		return ();
		}
	}
else {
	# Just see if the process is running
	return grep { $_ != $$ } $self->find_byname("portsentry");
	}
}

# portsentry_start_cmd()
sub portsentry_start_cmd
{
	my $self = shift;

return $self->{'config_hash'}->{'portsentry_start'} ? $self->{'config_hash'}->{'portsentry_start'} :
	"$self->{'config_hash'}->{'portsentry'} -$self->{'config_hash'}->{'portsentry_tmode'} && $self->{'config_hash'}->{'portsentry'} -$self->{'config_hash'}->{'portsentry_umode'}";
}

# stop_portsentry()
# Stops portsentry
sub stop_portsentry
{
	my $self = shift;

if ($self->{'config_hash'}->{'portsentry_stop'}) {
	local $out = $self->backquote_logged("($self->{'config_hash'}->{'portsentry_stop'}) 2>&1 </dev/null");
	return "<tt>$out</tt>" if ($out =~ /error|failed/i);
	}
else {
	local @pids = $self->get_portsentry_pids();
	if (@pids) {
		$self->kill_logged("TERM", @pids) ||
			return $self->text('portsentry_ekill', join(" ", @pids), $!);
		}
	else {
		return $self->{'text_hash'}->{'portsentry_estopped'};
		}
	}
return undef;
}

# start_portsentry()
# Starts portsentry, and returns an error message on failure, or undef 
sub start_portsentry
{
	my $self = shift;

local $cmd = $self->portsentry_start_cmd();
local $out = $self->backquote_logged("$cmd 2>&1 </dev/null");
return "<tt>$out</tt>" if ($out =~ /failed|error/i);
return undef;
}

# list_hostsentry_modules($conf)
# Returns a list of all hostsentry python modules
sub list_hostsentry_modules
{
	my $self = shift;

local $dir = $self->find_value("MODULE_PATH", $_[0]);
opendir(DIR, $dir);
local @rv = map { /^(\S+)\.py$/; $1 }
	        grep { /\.py$/ && !/^moduleExample/ } readdir(DIR);
closedir(DIR);
return @rv;
}

# hostsentry_start_cmd()
sub hostsentry_start_cmd
{
	my $self = shift;

return $self->{'config_hash'}->{'hostsentry_start'} ? $self->{'config_hash'}->{'hostsentry_start'}
				   : "python $self->{'config_hash'}->{'hostsentry'}";
}

# start_hostsentry()
# Start hostsentry, or return an error message
sub start_hostsentry
{
	my $self = shift;

local $cmd = $self->hostsentry_start_cmd();
local $temp = $self->tempname();
$self->system_logged("$cmd >$temp 2>&1 </dev/null");
local $out;
open(TEMP, $temp);
while(<TEMP>) { $out .= $_; }
close(TEMP);
unlink($temp);
return "<tt>$out</tt>" if ($out =~ /failed|error/i);
return undef;
}

# stop_hostsentry()
# Stop hostsentry, or return an error message
sub stop_hostsentry
{
	my $self = shift;

if ($self->{'config_hash'}->{'hostsentry_stop'}) {
        local $out = $self->backquote_logged("($self->{'config_hash'}->{'hostsentry_stop'}) 2>&1 </dev/null")
;
        return "<tt>$out</tt>" if ($out =~ /error|failed/i);
        }
else {
        local $pid = $self->get_hostsentry_pid();
        if ($pid) {
                $self->kill_logged("TERM", $pid) ||
                        return $self->text('hostsentry_ekill', $pid, $!);
                }
        else {
                return $self->{'text_hash'}->{'hostsentry_estopped'};
                }
        }
return undef;
}

# get_hostsentry_pid()
sub get_hostsentry_pid
{
	my $self = shift;

local ($pid) = grep { $_ != $$ } $self->find_byname("python.*hostsentry");
return $pid;
}

# get_hostsentry_dir()
sub get_hostsentry_dir
{
	my $self = shift;

$self->{'config_hash'}->{'hostsentry_config'} =~ /^(\S+)\//;
return $1;
}


1;

