#!/usr/bin/perl
package servers::ServersLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'servers::ServersLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# servers-lib.pl
# Common functions for managing servers

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();
$self->{'cron_cmd'} = "$self->{'module_config_directory'}/auto.pl";

@{$self->{'cluster_modules_array'}} = ( "cluster-software" );

@{$self->{'server_types_array'}} = (
		  [ 'asianux', 'Asianux', undef, 'Asianux' ],
		  [ 'centos', 'CentOS Linux', undef, 'CentOS' ],
		  [ 'cobalt', 'Cobalt Linux', 'cobalt-linux' ],
		  [ 'debian', 'Debian Linux', 'debian-linux' ],
		  [ 'fedora', 'Fedora Linux', undef, 'Fedora' ],
		  [ 'caldera', 'OpenLinux', 'open-linux' ],
		  [ 'mandrake', 'Mandrake Linux', 'mandrake-linux' ],
		  [ 'msc', 'MSC.Linux', 'msc-linux' ],
		  [ 'redhat', 'Redhat Linux', 'redhat-linux' ],
		  [ 'redflag', 'Red Flag Linux', undef, 'RedFlag' ],
		  [ 'slackware', 'Slackware Linux', 'slackware-linux' ],
		  [ 'suse', 'SuSE Linux', 'suse-linux' ],
		  [ 'turbo', 'TurboLinux', 'turbo-linux' ],
		  [ 'ubuntu', 'Ubuntu Linux', undef, 'Ubuntu' ],
		  [ 'linux', 'Linux', '.*-linux' ],

		  [ 'freebsd', 'FreeBSD', 'freebsd' ],
		  [ 'solaris', 'Solaris', 'solaris' ],
		  [ 'hpux', 'HP/UX', 'hpux' ],
		  [ 'sco', 'SCO', '(openserver|unixware)' ],
		  [ 'mac', 'Macintosh', 'macos' ],
		  [ 'irix', 'IRIX', 'irix' ],
		  [ 'windows', 'Windows', 'windows' ],
		  [ 'unknown', $self->{'text_hash'}->{'lib_other'} ] );

1;

1;
		return $self;
}

sub list_servers
{
	my $self = shift;

local ($f, @rv);
opendir(DIR, $self->{'module_config_directory'});
while($f = readdir(DIR)) {
	if ($f =~ /^(\d+)\.serv$/) {
		push(@rv, $self->get_server($1));
		}
	}
closedir(DIR);
return @rv;
}

# list_servers_sorted(applyacl)
# Returns a list of servers, sorted according to the module configuration
sub list_servers_sorted
{
	my $self = shift;

local @servers = $self->list_servers();
if ($_[0]) {
	@servers = grep { $self->can_use_server($_) } @servers;
	}
if ($self->{'config_hash'}->{'sort_mode'} == 1) {
	@servers = sort { $a->{'host'} cmp $b->{'host'} } @servers;
	}
elsif ($self->{'config_hash'}->{'sort_mode'} == 2) {
	@servers = sort { lc($a->{'desc'} ? $a->{'desc'} : $a->{'host'}) cmp
		   lc($b->{'desc'} ? $b->{'desc'} : $b->{'host'}) } @servers;
	}
elsif ($self->{'config_hash'}->{'sort_mode'} == 3) {
	@servers = sort { $a->{'type'} cmp $b->{'type'} } @servers;
	}
elsif ($self->{'config_hash'}->{'sort_mode'} == 4) {
	@servers = sort { &to_ipaddress($a->{'host'}) cmp
			  &to_ipaddress($b->{'host'}) } @servers;
	}
elsif ($self->{'config_hash'}->{'sort_mode'} == 5) {
	@servers = sort { $a->{'group'} cmp $b->{'group'} } @servers;
	}
return @servers;
}

# get_server(id)
sub get_server
{
	my $self = shift;

local $serv;
$serv->{'id'} = $_[0];
$self->read_file("$self->{'module_config_directory'}/$_[0].serv", $serv) || return undef;
return $serv;
}

# save_server(&server)
sub save_server
{
	my $self = shift;

$self->lock_file("$self->{'module_config_directory'}/$_[0]->{'id'}.serv");
$self->write_file("$self->{'module_config_directory'}/$_[0]->{'id'}.serv", $_[0]);
chmod(0600, "$self->{'module_config_directory'}/$_[0]->{'id'}.serv");
$self->unlock_file("$self->{'module_config_directory'}/$_[0]->{'id'}.serv");
undef(%main::remote_servers_cache);
}

# delete_server(id)
sub delete_server
{
	my $self = shift;

$self->lock_file("$self->{'module_config_directory'}/$_[0].serv");
unlink("$self->{'module_config_directory'}/$_[0].serv");
$self->unlock_file("$self->{'module_config_directory'}/$_[0].serv");
undef(%main::remote_servers_cache);
}

# can_use_server(&server)
sub can_use_server
{
	my $self = shift;

return 1 if ($self->{'access_hash'}->{'servers'} eq '*');
foreach $s (split(/\s+/, $self->{'access_hash'}->{'servers'})) {
	return 1 if ($_[0]->{'host'} eq $s ||
		     $_[0]->{'id'} eq $s);
	}
return 0;
}

# list_all_groups([&servers])
# Returns a list of all webmin and MSC groups and their members
sub list_all_groups
{
	my $self = shift;

local (@rv, %gmap, $s, $f, $gn);

# Add webmin servers groups
foreach $s ($_[0] ? @{$_[0]} : $self->list_servers()) {
	foreach $gn (split(/\t+/, $s->{'group'})) {
		local $grp = $gmap{$gn};
		if (!$grp) {
			$gmap{$gn} = $grp = { 'name' => $gn, 'type' => 0 };
			push(@rv, $grp);
			}
		push(@{$grp->{'members'}}, $s->{'host'});
		}
	}

# Add MSC cluster groups
opendir(DIR, $self->{'config_hash'}->{'groups_dir'});
foreach $f (readdir(DIR)) {
	next if ($f eq '.' || $f eq '..');
	local $grp = $gmap{$f};
	if (!$grp) {
		$gmap{$f} = $grp = { 'name' => $f, 'type' => 1 };
		push(@rv, $grp);
		}
	open(GROUP, "$self->{'config_hash'}->{'groups_dir'}/$f");
	while(<GROUP>) {
		s/\r|\n//g;
		s/#.*$//;
		if (/(\S*)\[(\d)-(\d+)\](\S*)/) {
			# Expands to multiple hosts
			push(@{$grp->{'members'}}, map { $1.$_.$4 } ($2 .. $3));
			}
		elsif (/(\S+)/) {
			push(@{$grp->{'members'}}, $1);
			}
		}
	close(GROUP);
	}
closedir(DIR);

# Fix up MSC groups that include other groups
while(1) {
	local ($grp, $any);
	foreach $grp (@rv) {
		local @mems;
		foreach $m (@{$grp->{'members'}}) {
			if ($m =~ /^:(.*)$/) {
				push(@mems, @{$gmap{$1}->{'members'}});
				$any++;
				}
			else {
				push(@mems, $m);
				}
			}
		$grp->{'members'} = \@mems;
		}
	last if (!$any);
	}

return @rv;
}

# logged_in(&serv)
sub logged_in
{
	my $self = shift;

local $id = $_[0]->{'id'};
if ($ENV{'HTTP_COOKIE'} =~ /$id=([A-Za-z0-9=]+)/) {
	return split(/:/, $self->decode_base64("$1"));
	}
else {
	return ();
	}
}

# this_server()
# Returns a fake servers-list entry for this server
sub this_server
{
	my $self = shift;

local $type = 'unknown';
foreach $s (@{$self->{'server_types_array'}}) {
	if ($s->[2] && $self->{'gconfig_hash'}->{'os_type'} =~ /^$s->[2]$/ ||
	    $s->[3] && $self->{'gconfig_hash'}->{'real_os_type'} =~ /$s->[3]/) {
		$type = $s->[0];
		last;
		}
	}
return { 'id' => 0, 'desc' => $self->{'text_hash'}->{'this_server'}, 'type' => $type };
}

# get_my_address()
# Returns the system's IP address, or undef
sub get_my_address
{
	my $self = shift;

local $myip;
if ($self->foreign_check("net")) {
	# Try to get ethernet interface
	$self->foreign_require("net", "net-lib.pl");
	local @act = $main::net_object->active_interfaces();
	local @ifaces = grep { $main::net_object->iface_type($_->{'fullname'}) =~ /ether/i }
			      @act;
	@ifaces = ( $act[0] ) if (!@ifaces && @act);
	if (@ifaces) {
		return wantarray ? ( map { $_->{'address'} } @ifaces )
				 : $ifaces[0]->{'address'};
		}
	}
$myip = $self->to_ipaddress($self->get_system_hostname());
if ($myip) {
	# Can resolve hostname .. use that
	return wantarray ? ( $myip ) : $myip;
	}
return wantarray ? ( ) : undef;
}

# address_to_broadcast(address, net-mode)
sub address_to_broadcast
{
	my $self = shift;

local $end = $_[1] ? "0" : "255";
local @ip = split(/\./, $_[0]);
return $ip[0] >= 192 ? "$ip[0].$ip[1].$ip[2].$end" :
       $ip[0] >= 128 ? "$ip[0].$ip[1].$end.$end" :
		       "$ip[0].$end.$end.$end";
}

# test_server(host)
# Returns undef if some server can be connected to OK, or an error message
sub test_server
{
	my $self = shift;

local $main::error_must_die = 1;
eval {
	$SIG{'ALRM'} = sub { die "Timeout\n" };
	alarm(10);
	$self->remote_foreign_require($_[0], "webmin", "webmin-lib.pl");
	alarm(0);
	};
local $rv = $@;
$rv =~ s/\s+at\s+(\S+)\s+line\s+\d+.*$//;
return $rv;
}

sub find_cron_job
{
	my $self = shift;

$self->foreign_require("cron", "cron-lib.pl");
local ($job) = grep { $_->{'command'} eq $self->{'cron_cmd'} } $main::cron_object->list_cron_jobs();
return $job;
}

# find_servers(&addresses, limit, no-print, defuser, defpass, deftype,
#	       &cluster-modules, find-self, port)
sub find_servers
{
	my $self = shift;

local ($broad, $limit, $noprint, $defuser, $defpass, $deftype, $mods, $tmpself,
       $port) = @_;
local (@found, @already, @foundme, %addmods);

my %server;
foreach my $s ($self->list_servers()) {
	$server{$self->to_ipaddress($s->{'host'})} = $s;
	}

# create the broadcast socket
local %miniserv;
$self->get_miniserv_config(\%miniserv);
$port ||= $self->{'config_hash'}->{'listen'} || $miniserv{'listen'} || 10000;
socket(BROAD, PF_INET, SOCK_DGRAM, getprotobyname("udp")) ||
	$self->error("socket failed : $!");
setsockopt(BROAD, SOL_SOCKET, SO_BROADCAST, pack("l", 1));

# Ignore primary IP address
my $myip = $self->get_my_address();
my %myaddr;
if ($myip && !$tmpself) {
	$myaddr{$self->inet_aton($myip)}++;
	}

# Find all our IPs
my %me = map { $_, 1 } $self->get_my_address();

# Ignore configured IPs
local %skip;
foreach my $skip (split(/\t+/, $self->{'config_hash'}->{'skipips'})) {
	$skip{$self->to_ipaddress($skip)} = 1;
	}

# Ignore our own IP addresses
if ($self->foreign_check("net")) {
	$self->foreign_require("net", "net-lib.pl");
	local @active = $main::net_object->active_interfaces();
	foreach my $a (@active) {
		if ($a->{'address'} && (!$tmpself || $a->{'virtual'} ne '')) {
			$myaddr{$self->inet_aton($a->{'address'})}++;
			}
		}

	# Adds IPs of interfaces to skip
	foreach my $skip (split(/\s+/, $self->{'config_hash'}->{'skipifaces'})) {
		local ($iface) = grep { $_->{'fullname'} eq $skip } @active;
		if ($iface) {
			$skip{$iface->{'address'}} = 1;
			}
		}
	}

# send out the packets
@broad = $self->unique(@broad);
foreach my $b (@broad) {
	send(BROAD, "webmin", 0, $self->pack_sockaddr_in($port, $self->inet_aton($b)));
	}

my $id = time();
my $tmstart = time();
my $found;
my %already;
while(time()-$tmstart < $limit) {
	local $rin;
	vec($rin, fileno(BROAD), 1) = 1;
	if (select($rin, undef, undef, 1)) {
		local $buf;
		local $from = recv(BROAD, $buf, 1024, 0);
		next if (!$from);
		local ($fromport, $fromaddr) = $self->unpack_sockaddr_in($from);
		local $fromip = $self->inet_ntoa($fromaddr);
		if ($fromip !~ /\.(255|0)$/ && !$already{$fromip}++) {
			# Got a response .. parse it
			local ($host, $port, $ssl, $realhost) =split(/:/, $buf);
			if ($self->{'config_hash'}->{'resolve'}) {
				local $byname = gethostbyaddr($fromaddr,
							      AF_INET);
				$host = !$host && $byname ? $byname :
					!$host && !$byname ? $fromip :
							     $host;
				}
			else {
				$host = $fromip;
				}
			if ($host eq "0.0.0.0") {
				# Remote doesn't know it's IP or name
				local $byname = gethostbyaddr($fromaddr,
							      AF_INET);
				$host = $byname || $fromip;
				}
			local $url = ($ssl ? 'https' : 'http').
				     "://$host:$port/";

			# Hack for OC to use real hostname if we found
			# ourselves
			if ($self->{'config_hash'}->{'selfrealhost'} &&
			    $me{$fromip}) {
				$realhost = $self->get_system_hostname();
				}

			# See if we have already found this server
			if ($skip{$fromip}) {
				# On skip list
				print $self->text('find_skip',
				    "<tt>$url</tt>"),"<br>\n" if (!$noprint);
				}
			elsif ($server{$fromip}) {
				# Already got it (but update real hostname)
				print $self->text('find_already',
				    "<tt>$url</tt>"),"<br>\n" if (!$noprint);
				push(@already, $server{$fromip});
				if ($server{$fromip}->{'realhost'} ne $realhost) {
					$server{$fromip}->{'realhost'} = $realhost;
					$self->save_server($server{$fromip});
					}
				}
			elsif ($myaddr{$fromaddr}) {
				# This server
				print $self->text('find_me',
				    "<tt>$url</tt>"),"<br>\n" if (!$noprint);
				push(@foundme, $fromaddr);
				}
			else {
				# Found a new one!
				local $fast = $self->{'config_hash'}->{'deffast'} == 1 ? 1 : 0;
				local $serv = {	'id' => $id++,
						'ssl' => $ssl,
						'type' => $deftype || 'unknown',
					 	'fast' => $fast,
						'port' => $port,
						'host' => $host,
						'realhost' => $realhost,
						'user' => $defuser,
						'pass' => $defpass, };
				$self->save_server($serv);

				local $err;
				if ($defuser) {
					# See if the login was OK
					$err = $self->test_server($host);
					}
				if (!$noprint) {
					if ($err) {
						print $self->text('find_but',
						    "<tt>$url</tt>", $err),"<br>\n";
						}
					else {
						print $self->text('find_new',
							    "<tt>$url</tt>"),"<br>\n";
						}
					}
				push(@found, $serv);

				if ($defuser && !$err) {
					# Add in all the cluster modules too
					foreach my $m (@$mods) {
						$self->foreign_require($m, "$m-lib.pl");
						($self->{'ok'}, $self->{'out'}) = $self->foreign_call($m, "add_managed_host", $serv);
						push(@{$addmods{$serv->{'id'}}}, [ $m, $self->{'ok'}, $self->{'out'} ]);
						}
					}

				if ($defuser && !$err) {
					# Get the OS type
					if ($self->remote_foreign_check(
					     $serv, "servers")) {
						$self->remote_foreign_require(
							$serv, "servers",
							"servers-lib.pl");
						local $rt =
							$self->remote_foreign_call(
							$serv, "servers",
							"this_server");
						$serv->{'type'} = $rt->{'type'};
						$self->save_server($serv);
						}
					}

				$self->webmin_log("find", "server", $host, $serv);
				$server{$fromip} = $serv;
				}
			$found++;
			}
		}
	}
print "$self->{'text_hash'}->{'find_none'}<p>\n" if (!$found && !$noprint);
return ( \@found, \@already, \@foundme, \%addmods );
}


1;

