#!/usr/bin/perl
package shorewall::ShorewallLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'shorewall::ShorewallLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# shorewall-lib.pl
# Common functions for the shorewall configuration files
# XXX rule sections

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

# Get the version
$self->{'shorewall_version'} = $self->get_shorewall_version(0);

# get access permissions
%{$self->{'access_hash'}} = $self->get_module_acl();

@{$self->{'shorewall_files_array'}} = ( 'zones', 'interfaces', 'policy', 'rules', 'tos',
	   	     'masq', 'nat', 'proxyarp', 'routestopped',
	   	     'tunnels', 'hosts', 'blacklist',
		     ( $self->version_atleast(2, 3) ? ( 'providers' ) : ( ) ) );
@{$self->{'comment_tables_array'}} = ( 'masq', 'nat', 'rules', 'tcrules' );

################################# zones #######################################

################################# interfaces ###################################

@{$self->{'interfaces_opts_array'}} = ( 'dhcp', 'noping', 'filterping', 'routestopped', 'norfc1918',
		     'multi', 'routefilter', 'dropunclean', 'logunclean',
		     'blacklist', 'maclist', 'tcpflags', 'proxyarp' );
if ($self->version_atleast(3)) {
	push(@{$self->{'interfaces_opts_array'}}, "logmartians", "routeback", "arp_filter",
			       "arp_ignore", "nosmurfs", "detectnets", "upnp");
	}

################################# policy #######################################

@{$self->{'policy_list_array'}} = ( "ACCEPT", "DROP", "REJECT", "CONTINUE" );

################################# rules #######################################

@{$self->{'rules_actions_array'}} = ( 'ACCEPT', 'DROP', 'REJECT', 'DNAT', 'DNAT-', 'REDIRECT' );
if ($self->version_atleast(2, 0, 0)) {
	push(@{$self->{'rules_actions_array'}}, 'CONTINUE');
	push(@{$self->{'rules_actions_array'}}, 'ACCEPT+');
	push(@{$self->{'rules_actions_array'}}, 'NONAT');
	push(@{$self->{'rules_actions_array'}}, 'REDIRECT-');
	push(@{$self->{'rules_actions_array'}}, 'LOG');
	}
if ($self->version_atleast(3)) {
	push(@{$self->{'rules_actions_array'}}, 'DNAT-');
	push(@{$self->{'rules_actions_array'}}, 'SAME');
	push(@{$self->{'rules_actions_array'}}, 'SAME-');
	push(@{$self->{'rules_actions_array'}}, 'QUEUE');
	}
@{$self->{'rules_protos_array'}} = ( 'all', 'related', 'tcp', 'udp', 'icmp' );

################################# tos #########################################

%{$self->{'tos_map_hash'}} = ( 0, 'Normal-Service',
	     2, 'Minimize-Cost',
	     4, 'Maximize-Reliability',
	     8, 'Maximize-Throughput',
	     16, 'Minimize-Delay' );
@{$self->{'tos_protos_array'}} = ( 'tcp', 'udp', 'icmp' );

################################# masq #########################################

################################# nat #########################################

################################# proxyarp #######################################

################################ routestopped ##################################

@{$self->{'routestopped_options_array'}} = ( "routeback", "source", "dest", "critical" );

################################ tunnels ##################################

################################ hosts ##################################

@{$self->{'host_options_array'}} = ( "maclist", "routeback" );
if ($self->version_atleast(3)) {
	push(@{$self->{'host_options_array'}}, "norfc1918", "blacklist", "tcpflags",
			    "nosmurfs", "ipsec");
	}

################################ blacklist ##################################

@{$self->{'blacklist_protos_array'}} = ( undef, 'tcp', 'udp', 'icmp' );

################################ providers ##################################

@{$self->{'providers_opts_array'}} = ( "track", "balance", "loose" );

#############################################################################

1;

1;
		return $self;
}

# version_atleast(v1, v2, v3)
sub version_atleast
{
	my $self = shift;

local @vsp = split(/\./, $self->{'shorewall_version'});
local $i;
for($i=0; $i<@vsp || $i<@_; $i++) {
	return 0 if ($vsp[$i] < $_[$i]);
	return 1 if ($vsp[$i] > $_[$i]);
	}
return 1;	# same!
}

# read_table_file(table, &parserfunc)
sub read_table_file
{
	my $self = shift;

local @rv;
local $func = $_[1];
open(FILE, "$self->{'config_hash'}->{'config_dir'}/$_[0]");
while(<FILE>) {
	s/\r|\n//g;
	local $l = $self->$func($_);
	push(@rv, $l) if ($l);
	}
close(FILE);
return @rv;
}

# read_table_struct(table, &parserfunc)
sub read_table_struct
{
	my $self = shift;

if (!defined($self->{'read_table_cache_hash'}->{$_[0]})) {
	local @rv;
	local $func = $_[1];
	open(FILE, "$self->{'config_hash'}->{'config_dir'}/$_[0]");
	local $lnum = 0;
	while(<FILE>) {
		s/\r|\n//g;
		local $cmt;
		if (s/#\s*(.*)$//) {
			$cmt = $1;
			}
		local $l = $self->$func($_);
		if ($l) {
			push(@rv, { 'line' => $lnum,
				    'file' => "$self->{'config_hash'}->{'config_dir'}/$_[0]",
				    'table' => $_[0],
				    'index' => scalar(@rv),
				    'values' => $l,
				    'comment' => $cmt });
			}
		$lnum++;
		}
	close(FILE);
	$self->{'read_table_cache_hash'}->{$_[0]} = \@rv;
	}
return $self->{'read_table_cache_hash'}->{$_[0]};
}

# find_line_num(&lref, &parserfunc, index)
sub find_line_num
{
	my $self = shift;

local $lref = $_[0];
local $func = $_[1];
local $idx = 0;
for($i=0; $i<@$lref; $i++) {
	if ($self->$func($lref->[$i])) {
		if ($idx++ == $_[2]) {
			return $i;
			}
		}
	}
return undef;
}

# delete_table_row(table, &parserfunc, index)
sub delete_table_row
{
	my $self = shift;

local $lref = $self->read_file_lines("$self->{'config_hash'}->{'config_dir'}/$_[0]");
local $lnum = $self->find_line_num($lref, $_[1], $_[2]);
splice(@$lref, $lnum, 1) if (defined($lnum));
$self->flush_file_lines();
}

# delete_table_struct(&struct)
sub delete_table_struct
{
	my $self = shift;

local $lref = $self->read_file_lines($_[0]->{'file'});
splice(@$lref, $_[0]->{'line'}, 1);
$self->flush_file_lines();
local $cache = $self->{'read_table_cache_hash'}->{$_[0]->{'table'}};
local $idx = $self->indexof($_[0], @$cache);
if ($idx >= 0) {
	splice(@$cache, $idx, 1);
	}
local $c;
foreach $c (@$cache) {
	$c->{'line'}-- if ($c->{'line'} > $_[0]->{'line'});
	$c->{'index'}-- if ($c->{'index'} > $_[0]->{'index'});
	}
}

# create_table_row(table, &parserfunc, line, [insert-index])
sub create_table_row
{
	my $self = shift;

local $lref = $self->read_file_lines("$self->{'config_hash'}->{'config_dir'}/$_[0]");
local ($i, $idx);
for($i=0; $i<@$lref; $i++) {
	if ($lref->[$i] =~ /^#+\s*LAST\s+LINE/) {
		$idx = $i;
		last;
		}
	}
if (defined($_[3])) {
	local $lnum = $self->find_line_num($lref, $_[1], $_[3]);
	$lnum = $idx if (!defined($lnum));
	splice(@$lref, $lnum, 0, $self->simplify_line($_[2]));
	}
else {
	splice(@$lref, $idx, 0, $self->simplify_line($_[2]));
	}
$self->flush_file_lines();
}

# create_table_struct(&struct, parserfunc, [&insert-before])
sub create_table_struct
{
	my $self = shift;

local $lref = $self->read_file_lines("$self->{'config_hash'}->{'config_dir'}/$_[0]->{'table'}");
local ($i, $idx);
for($i=0; $i<@$lref; $i++) {
	if ($lref->[$i] =~ /^#+\s*LAST\s+LINE/) {
		$idx = $i;
		last;
		}
	}
if (!defined($idx)) {
	$idx = @$lref;
	}
local $cache = $self->read_table_struct($_[0]->{'table'}, $_[1]);
if ($_[2]) {
	# Insert into file
	splice(@$lref, $_[2]->{'line'}, 0, $self->make_struct($_[0]));
	$_[0]->{'file'} = "$self->{'config_hash'}->{'config_dir'}/$_[0]->{'table'}";
	$_[0]->{'line'} = $_[2]->{'line'};
	$_[0]->{'index'} = $_[2]->{'index'};
	local $c;
	foreach $c (@$cache) {
		$_[0]->{'line'}++ if ($c->{'line'} >= $_[2]->{'line'});
		$_[0]->{'index'}++ if ($c->{'index'} >= $_[2]->{'index'});
		}
	local $iidx = $self->indexof($_[2], @$cache);
	splice(@$cache, $iidx, 0, $_[0]);
	}
else {
	# Append to file
	splice(@$lref, $idx, 0, $self->make_struct($_[0]));
	$_[0]->{'file'} = "$self->{'config_hash'}->{'config_dir'}/$_[0]->{'table'}";
	$_[0]->{'line'} = $idx;
	$_[0]->{'index'} = @$cache;
	push(@$cache, $_[0]->{'index'});
	}
$self->flush_file_lines();
}

# modify_table_row(table, &parserfunc, index, line)
sub modify_table_row
{
	my $self = shift;

local $lref = $self->read_file_lines("$self->{'config_hash'}->{'config_dir'}/$_[0]");
local $lnum = $self->find_line_num($lref, $_[1], $_[2]);
$lref->[$lnum] = $self->simplify_line($_[3]) if (defined($lnum));
$self->flush_file_lines();
}

# modify_table_struct(&newstruct, &oldstruct)
sub modify_table_struct
{
	my $self = shift;

local $lref = $self->read_file_lines("$self->{'config_hash'}->{'config_dir'}/$_[1]->{'table'}");
$lref->[$_[1]->{'line'}] = $self->make_struct($_[0]);
if ($_[0] ne $_[1]) {
	$_[0]->{'line'} = $_[1]->{'line'};
	$_[0]->{'index'} = $_[1]->{'index'};
	local $cache = $self->{'read_table_cache_hash'}->{$_[1]->{'table'}};
	local $idx = $self->indexof($_[1], @$cache);
	$cache->[$idx] = $_[0];
	}
$self->flush_file_lines();
}

# swap_table_rows(table, &parserfunc, index1, index2)
sub swap_table_rows
{
	my $self = shift;

local $lref = $self->read_file_lines("$self->{'config_hash'}->{'config_dir'}/$_[0]");
local $lnum1 = $self->find_line_num($lref, $_[1], $_[2]);
local $lnum2 = $self->find_line_num($lref, $_[1], $_[3]);
($lref->[$lnum1], $lref->[$lnum2]) = ($lref->[$lnum2], $lref->[$lnum1]);
$self->flush_file_lines();
}

# make_struct(&struct)
sub make_struct
{
	my $self = shift;

local $line = join("\t", @{$_[0]->{'values'}});
if ($_[0]->{'comment'}) {
	$line .= "\t# $_[0]->{'comment'}";
	}
return $self->simplify_line($line);
}

# simplify_line(line)
# Removes blank fields from the end of a line
sub simplify_line
{
	my $self = shift;

local $rv = $_[0];
while($rv =~ s/\s+$// || $rv =~ s/\-$//) { }
return $rv;
}

sub lock_table
{
	my $self = shift;

$self->lock_file("$self->{'config_hash'}->{'config_dir'}/$_[0]");
}

sub unlock_table
{
	my $self = shift;

$self->unlock_file("$self->{'config_hash'}->{'config_dir'}/$_[0]");
}

sub standard_parser
{
	my $self = shift;

local $l = $_[0];
$l =~ s/#.*$//;
local @sp = split(/\s+/, $l);
return undef if ($sp[0] eq "SECTION");
return @sp ? \@sp : undef;
}

# zone_field(name, value, othermode, simplemode)
sub zone_field
{
	my $self = shift;

local @ztable = $self->read_table_file("zones", \&zones_parser);
local $found = 0;

print "<select name=$_[0]>\n";
if ($_[3] == 2) {
	$found = !$_[1];
	}
elsif ($_[3] == 1) {
	printf "<option value=- %s>%s\n",
		$_[1] eq '-' ? "selected" : "", "&lt;$self->{'text_hash'}->{'list_any'}&gt;";
	$found = !$_[1] || $_[1] eq '-';
	}
elsif ($_[3] == 0) {
	printf "<option value=all %s>%s\n",
		$_[1] eq 'all' ? "selected" : "", "&lt;$self->{'text_hash'}->{'list_any'}&gt;";
	printf "<option value=\$FW %s>%s\n",
		$self->is_fw($_[1]) ? "selected" : "", "&lt;$self->{'text_hash'}->{'list_fw'}&gt;";
	$found = !$_[1] || $_[1] eq 'all' || $self->is_fw($_[1]);
	}
foreach $z (@ztable) {
	if ($self->version_atleast(3)) {
		printf "<option value=%s %s>%s\n",
			$z->[0], $_[1] eq $z->[0] ? "selected" : "", $z->[0];
		}
	else {
		# Old zone type has a human-readable name
		printf "<option value=%s %s>%s\n",
			$z->[0], $_[1] eq $z->[0] ? "selected" : "", $z->[1];
		}
	$found++ if ($_[1] eq $z->[0]);
	}
if ($_[2]) {
	printf "<option value='' %s>%s\n",
		$found ? "" : "selected", $self->{'text_hash'}->{'list_other'};
	}
else {
	print "<option value=$_[1] selected>$_[1]\n" if (!$found);
	}
print "</select>\n";
return $found;
}

# iface_field(name, value)
sub iface_field
{
	my $self = shift;

local @itable = $self->read_table_file("interfaces", \&standard_parser);
print "<select name=$_[0]>\n";
local $found = !$_[1];
foreach $i (@itable) {
	printf "<option value=%s %s>%s\n",
		$i->[1], $_[1] eq $i->[1] ? "selected" : "", $i->[1];
	$found++ if ($_[1] eq $i->[1]);
	}
print "<option value=$_[1] selected>$_[1]\n" if (!$found);
print "</select>\n";
}

# convert_zone(name)
# Given a zone name, returns a description
sub convert_zone
{
	my $self = shift;

if ($self->version_atleast(3)) {
	# No descriptions in shorewall 3
	return $_[0];
	}
local @ztable = $self->read_table_file("zones", \&zones_parser);
foreach $z (@ztable) {
	if ($_[0] eq $z->[0]) {
	     $self->{'ret'} = $z->[1];
	     }
	}
if ($self->is_fw($_[0])) {
	$self->{'ret'} = $self->{'text_hash'}->{'list_fw'};
	}
return $self->{'ret'} || $_[0];
}

# nice_host_list(list)
# Convert a comma-separate host string to space-separated
sub nice_host_list
{
	my $self = shift;

local @hosts = split(/,/, $_[0]);
if (@{$self->{'host_array'}} > 5) {
	return join(", ", @hosts[0..5]).", ...";
	}
else {
	return join(", ", @hosts);
	}
}

# is_fw(zone)
sub is_fw
{
	my $self = shift;

return $_[0] eq '$FW' || $_[0] eq 'fw';
}

sub zones_parser
{
	my $self = shift;

if ($self->version_atleast(3)) {
	# Zones table is in new format in Shorewall 3
	local $l = $_[0];
	$l =~ s/#.*$//;
	local @r = split(/\s+/, $l);
	return scalar(@r) ? \@r : undef;
	}
else {
	# Old format
	local $l = $_[0];
	$l =~ s/#.*$//;
	if ($l =~ /^(\S+)\s+(\S+)\s*(.*)/) {
		return [ $1, $2, $3 ];
		}
	else {
		return undef;
		}
	}
}

sub zones_columns
{
	my $self = shift;

return $self->version_atleast(3) ? 2 : 3;
}

sub zones_row
{
	my $self = shift;

if ($self->version_atleast(3)) {
	return ( $_[0], $self->{'text_hash'}->{'zones_'.$_[1]} || $_[1] );
	}
else {
	return @_;
	}
}

sub zones_colnames
{
	my $self = shift;

if ($self->version_atleast(3)) {
	return ( $self->{'text_hash'}->{'zones_0'}, $self->{'text_hash'}->{'zones_1new'} );
	}
else {
	return ( $self->{'text_hash'}->{'zones_0'}, $self->{'text_hash'}->{'zones_1'}, $self->{'text_hash'}->{'zones_2'} );
	}
}

sub zones_form
{
	my $self = shift;

if ($self->version_atleast(3)) {
	# Shorewall 3 zones format
	print "<tr> <td><b>$self->{'text_hash'}->{'zones_0'}</b></td>\n";
	print "<td>",$self->ui_textbox("id", $_[0], 8),"</td>\n";

	print "<td><b>$self->{'text_hash'}->{'zones_1new'}</b></td>\n";
	print "<td>",$self->ui_select("type", $_[1],
		[ [ "ipv4", $self->{'text_hash'}->{'zones_ipv4'} ],
		  [ "ipsec", $self->{'text_hash'}->{'zones_ipsec'} ],
		  [ "firewall", $self->{'text_hash'}->{'zones_firewall'} ] ]),"</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'zones_2new'}</b></td>\n";
	print "<td>",$self->ui_textbox("opts", $_[2], 50),"</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'zones_3new'}</b></td>\n";
	print "<td>",$self->ui_textbox("opts_in", $_[3], 50),"</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'zones_4new'}</b></td>\n";
	print "<td>",$self->ui_textbox("opts_out", $_[4], 50),"</td> </tr>\n";
	}
else {
	# Shorewall 2 zones format
	print "<tr> <td><b>$self->{'text_hash'}->{'zones_0'}</b></td>\n";
	print "<td><input name=id size=8 value='$_[0]'></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'zones_1'}</b></td>\n";
	print "<td><input name=name size=15 value='$_[1]'></td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'zones_2'}</b></td>\n";
	print "<td><input name=desc size=70 value='$_[2]'></td> </tr>\n";
	}
}

sub zones_validate
{
	my $self = shift;

$self->{'in_hash'}->{'id'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'zones_eid'});
$self->is_fw($self->{'in_hash'}->{'id'}) && $self->error($self->{'text_hash'}->{'zones_efwid'});
if ($self->version_atleast(3)) {
	# Parse new format
	$self->{'in_hash'}->{'opts'} =~ /^\S*$/ || $self->error($self->{'text_hash'}->{'zones_eopts'});
	$self->{'in_hash'}->{'opts_in'} =~ /^\S*$/ || $self->error($self->{'text_hash'}->{'zones_eopts_in'});
	$self->{'in_hash'}->{'opts_out'} =~ /^\S*$/ || $self->error($self->{'text_hash'}->{'zones_eopts_out'});
	return ( $self->{'in_hash'}->{'id'}, $self->{'in_hash'}->{'type'}, $self->{'in_hash'}->{'opts'},
		 $self->{'in_hash'}->{'opts_in'}, $self->{'in_hash'}->{'opts_out'} );
	}
else {
	# Parse old format
	$self->{'in_hash'}->{'name'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'zones_ename'});
	$self->{'in_hash'}->{'desc'} =~ /\S/ || $self->error($self->{'text_hash'}->{'zones_edesc'});
	return ( $self->{'in_hash'}->{'id'}, $self->{'in_hash'}->{'name'}, $self->{'in_hash'}->{'desc'} );
	}
}

sub interfaces_row
{
	my $self = shift;

return ( $_[1],
	 $_[0] eq '-' ? $self->{'text_hash'}->{'list_any'} : $_[0],
	 $_[2] eq 'detect' ? $self->{'text_hash'}->{'list_auto'} :
	  $_[2] eq '-' || $_[2] eq '' ? $self->{'text_hash'}->{'list_none'} : $_[2],
	 $_[3] ? $_[3] : $self->{'text_hash'}->{'list_none'} );
}

sub interfaces_form
{
	my $self = shift;

print "<tr> <td><b>$self->{'text_hash'}->{'interfaces_0'}</b></td>\n";
print "<td><input name=iface size=6 value='$_[1]'></td>\n";

local @ztable = $self->read_table_file("zones", \&zones_parser);
print "<td><b>$self->{'text_hash'}->{'interfaces_1'}</b></td>\n";
print "<td>\n";
$self->zone_field("zone", $_[0], 0, 1);
print "</td> </tr>\n";

local $bmode = $_[2] eq 'detect' ? 2 :
	       $_[2] eq '-' || $_[2] eq '' ? 1 : 0;
print "<tr> <td><b>$self->{'text_hash'}->{'interfaces_2'}</b></td> <td colspan=3>\n";
printf "<input type=radio name=broad_mode value=1 %s> %s\n",
	$bmode == 1 ? "checked" : "", $self->{'text_hash'}->{'list_none'};
printf "<input type=radio name=broad_mode value=2 %s> %s\n",
	$bmode == 2 ? "checked" : "", $self->{'text_hash'}->{'list_auto'};
printf "<input type=radio name=broad_mode value=0 %s>\n",
	$bmode == 0 ? "checked" : "";
printf "<input name=broad size=50 value='%s'></td> </tr>\n",
	$bmode == 0 ? $_[2] : "";

# options
local %opts = map { $_, 1 } split(/,/, $_[3]);
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'interfaces_3'}</b></td> <td colspan=3>\n";
$self->options_input("opts", $_[3], \@{$self->{'interfaces_opts_array'}});
print "</td> </tr>\n";
}

sub interfaces_validate
{
	my $self = shift;

$self->{'in_hash'}->{'iface'} =~ /^[a-z]+\d*(\.\d+)?$/ ||
	$self->{'in_hash'}->{'iface'} =~ /^[a-z]+\+$/ || $self->error($self->{'text_hash'}->{'interfaces_eiface'});
$self->{'in_hash'}->{'broad_mode'} || $self->{'in_hash'}->{'broad'} =~ /^[0-9\.,]+$/ ||
	$self->error($self->{'text_hash'}->{'interfaces_ebroad'});
return ( $self->{'in_hash'}->{'zone'}, $self->{'in_hash'}->{'iface'},
	 $self->{'in_hash'}->{'broad_mode'} == 2 ? 'detect' :
	 $self->{'in_hash'}->{'broad_mode'} == 1 ? '-' : $self->{'in_hash'}->{'broad'},
	 join(",", split(/\0/, $self->{'in_hash'}->{'opts'})) );
}

sub policy_row
{
	my $self = shift;

return ( $_[0] eq 'all' ? $self->{'text_hash'}->{'list_any'} :
	  $self->is_fw($_[0]) ? $self->{'text_hash'}->{'list_fw'} : $_[0],
	 $_[1] eq 'all' ? $self->{'text_hash'}->{'list_any'} :
	  $self->is_fw($_[1]) ? $self->{'text_hash'}->{'list_fw'} : $_[1],
	 $_[2], $_[3] eq '-' || $_[3] eq '' ? $self->{'text_hash'}->{'list_none'} : $_[3],
	 $_[4] =~ /(\d+):(\d+)/ ? $self->text('policy_limit', "$1", "$2")
				: $self->{'text_hash'}->{'list_none'} );
}

sub policy_form
{
	my $self = shift;

local $found;

print "<tr> <td><b>$self->{'text_hash'}->{'policy_0'}</b></td>\n";
print "<td>\n";
$self->zone_field("source", $_[0], 0);
print "</td>\n";

print "<td><b>$self->{'text_hash'}->{'policy_1'}</b></td>\n";
print "<td>\n";
$self->zone_field("dest", $_[1], 0);
print "</td> </tr>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'policy_2'}</b></td>\n";
print "<td><select name=policy>\n";
$found = !$_[2];
foreach $p (@{$self->{'policy_list_array'}}) {
	printf "<option value=%s %s>%s\n",
		$p, lc($p) eq lc($_[2]) ? "selected" : "", $p;
	$found++ if (lc($p) eq lc($_[2]));
	}
print "<option value=$_[2] selected>$_[2]\n" if (!$found);
print "</select></td>\n";

print "<td><b>$self->{'text_hash'}->{'policy_3'}</b></td>\n";
print "<td><select name=log>\n";
printf "<option value=- %s>%s\n",
	$_[3] eq '-' || !$_[3] ? "selected" : "", "&lt;$self->{'text_hash'}->{'policy_nolog'}&gt;";
printf "<option value=ULOG %s>%s\n",
	$_[3] eq 'ULOG' ? "selected" : "", "&lt;$self->{'text_hash'}->{'policy_ulog'}&gt;";
$found = !$_[3] || $_[3] eq '-' || $_[3] eq 'ULOG';
$self->foreign_require("syslog", "syslog-lib.pl");
foreach $l ($main::syslog_object->list_priorities()) {
	printf "<option value=%s %s>%s\n",
		$l, $_[3] eq $l ? "selected" : "", $l;
	$found++ if ($_[3] eq $l);
	}
print "<option value=$_[3] selected>$_[3]\n" if (!$found);
print "</select></td> </tr>\n";

local ($l, $b) = $_[4] =~ /(\d+):(\d+)/ ? ($1, $2) : ( );
print "<tr> <td><b>$self->{'text_hash'}->{'policy_4'}</b></td> <td colspan=3>\n";
printf "<input type=radio name=limit_def value=1 %s> %s\n",
	$l eq '' ? "checked" : "", $self->{'text_hash'}->{'list_none'};
printf "<input type=radio name=limit_def value=0 %s>\n",
	$l eq '' ? "" : "checked";
print $self->text('policy_limit',
	    "<input name=limit size=5 value='$l'>",
	    "<input name=burst size=5 value='$b'>"),"</td> </tr>\n";
}

sub policy_validate
{
	my $self = shift;

$self->is_fw($self->{'in_hash'}->{'source'}) && $self->is_fw($self->{'in_hash'}->{'dest'}) && $self->error($self->{'text_hash'}->{'policy_efw'});
if (!$self->{'in_hash'}->{'limit_def'}) {
	$self->{'in_hash'}->{'limit'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'policy_elimit'});
	$self->{'in_hash'}->{'burst'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'policy_eburst'});
	}
return ( $self->{'in_hash'}->{'source'}, $self->{'in_hash'}->{'dest'}, $self->{'in_hash'}->{'policy'}, $self->{'in_hash'}->{'log'}, 
	 $self->{'in_hash'}->{'limit_def'} ? ( ) : ( "$self->{'in_hash'}->{'limit'}:$self->{'in_hash'}->{'burst'}" ) );
}

sub rules_row
{
	my $self = shift;

return ( $_[0] =~ /^(\S+):/ ? "$1" : $_[0],
	 $self->is_fw($_[1]) ? $self->{'text_hash'}->{'list_fw'} :
	  $_[1] eq 'all' ? $self->{'text_hash'}->{'list_any'} :
	  $_[1] =~ /^([^:]+):(\S+)$/ ?
	  $self->text('rules_hosts', $self->convert_zone("$1"), $self->nice_host_list("$2")) :
	  $self->text('rules_zone', $self->convert_zone($_[1])),
	 $self->is_fw($_[2]) ? $self->{'text_hash'}->{'list_fw'} :
	  $_[2] eq 'all' ? $self->{'text_hash'}->{'list_any'} :
	  $_[2] =~ /^\d+$/ ? $self->text('rules_rport', $_[2]) :
	  $_[2] =~ /^([^:]+):(\S+)$/ ?
	  $self->text('rules_hosts', $self->convert_zone("$1"), $self->nice_host_list("$2")) :
	  $self->text('rules_zone', $self->convert_zone($_[2])),
	 $_[3] eq 'all' ? $self->{'text_hash'}->{'list_any'} :
	  $_[3] eq 'related' ? $self->{'text_hash'}->{'rules_related'} : uc($_[3]),
	 $_[3] eq 'all' || $_[3] eq 'related' ? "" :
	  $_[5] eq '-' || $_[5] eq '' ? $self->{'text_hash'}->{'list_any'} : $_[5],
	 $_[4] eq '-' || $_[4] eq '' ? "" : $_[4],
	 $self->version_atleast(1, 4, 7) ? (
		$_[7] eq "-" ? "" : $_[7],
		$_[8] eq "-" ? "" : $_[8] ) :
		( )
	);
}

sub rules_form
{
	my $self = shift;

local $found;
local @ztable = $self->read_table_file("zones", \&zones_parser);

local ($action, $log) = split(/:/, $_[0]);
local $macroarg;
if ($action =~ /^(.*)\/(.*)$/) {
	$action = $1;
	$macroarg = $2;
	}

# Rule action
print "<tr> <td><b>$self->{'text_hash'}->{'rules_0'}</b></td>\n";
print "<td colspan=3><select name=action>\n";
$found = !$_[0];
foreach $a ((sort { $a cmp $b } @{$self->{'rules_actions_array'}}),
	    "-------- Actions --------",
	    $self->list_standard_actions(),
	    ($self->version_atleast(3) ? ( "-------- Macros --------",
				     $self->list_standard_macros() ) : ( ) )) {
	printf "<option value=%s %s>%s\n",
		$a, $action eq $a ? "selected" : "", $a;
	$found++ if ($action eq $a);
	}
print "<option value=$action selected>$action\n" if (!$found);
print "</select>\n";

# Logging level
print "<b>$self->{'text_hash'}->{'rules_log'}</b> <select name=log>\n";
printf "<option value='' %s>%s\n",
	!$log ? "selected" : "", "&lt;$self->{'text_hash'}->{'rules_nolog'}&gt;";
printf "<option value=ULOG %s>%s\n",
	$log eq 'ULOG' ? "selected" : "", "&lt;$self->{'text_hash'}->{'policy_ulog'}&gt;";
$found = !$log || $log eq '-' || $log eq 'ULOG';
$self->foreign_require("syslog", "syslog-lib.pl");
foreach $l ($main::syslog_object->list_priorities()) {
	printf "<option value=%s %s>%s\n",
		$l, $log eq $l ? "selected" : "", $l;
	$found++ if ($log eq $l);
	}
print "<option value=$log selected>$log\n" if (!$found);
print "</select></td> </tr>\n";

if ($self->version_atleast(3)) {
	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'rules_macro'}</b></td>\n";
	print "<td colspan=3 nowrap>\n";
	print $self->ui_select("macro", $macroarg,
		[ [ "", "&lt;$self->{'text_hash'}->{'rules_none2'}&gt;" ],
		  map { [ $_ ] } (sort { $a cmp $b } @{$self->{'rules_actions_array'}}) ],
		1, 0, $macroarg);
	print "</td> </tr>\n";
	}

# Source zone and hosts
local ($zone, $host) = split(/:/, $_[1], 2);
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'rules_1z'}</b></td>\n";
print "<td colspan=3 nowrap>\n";
$found = $self->zone_field("source", $zone, 1);
printf "<input name=sother size=10 value='%s'>\n",
	$found ? "" : $zone;

print "<br><b>$self->{'text_hash'}->{'rules_inzone'}</b>\n";
printf "<input type=checkbox name=sinzone_def value=1 %s> %s\n",
	$host ? "checked" : "", $self->{'text_hash'}->{'rules_addr'};
printf "<input name=sinzone size=50 value='%s'></td> </tr>\n",
	join(" ", split(/,/, $host));

($zone, $host) = split(/:/, $_[2], 2);
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'rules_2z'}</b></td>\n";
print "<td colspan=3 nowrap>\n";
$found = $self->zone_field("dest", $zone, 1);
printf "<input name=dother size=10 value='%s'>\n",
	$found ? "" : $zone;

print "<br><b>$self->{'text_hash'}->{'rules_inzone'}</b>\n";
printf "<input type=checkbox name=dinzone_def value=1 %s> %s\n",
	$host ? "checked" : "", $self->{'text_hash'}->{'rules_addr'};
printf "<input name=dinzone size=50 value='%s'>\n",
	join(" ", split(/,/, $host));
print "<br>$self->{'text_hash'}->{'rules_dnat_dest'}</td> </tr>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'rules_3'}</b></td>\n";
print "<td colspan=3><select name=proto>\n";
$found = !$_[3];
foreach $p (@{$self->{'rules_protos_array'}}) {
	printf "<option value=%s %s>%s\n",
		$p, $p eq $_[3] ? "selected" : "",
		$p eq 'all' ? "&lt;$self->{'text_hash'}->{'list_any'}&gt;" :
		 $p eq 'related' ? "&lt;$self->{'text_hash'}->{'rules_related'}&gt;" : uc($p);
	$found++ if ($p eq $_[3]);
	}
printf "<option value='' %s>%s\n",
	$found ? "" : "selected", $self->{'text_hash'}->{'list_other'};
print "</select>\n";
printf "<input name=pother size=5 value='%s'></td> </tr>\n",
	$found ? "" : $_[3];

print "<tr> <td><b>$self->{'text_hash'}->{'rules_4'}</b></td> <td colspan=3>\n";
printf "<input type=radio name=sport_def value=1 %s> %s\n",
	$_[5] eq '' || $_[5] eq '-' ? "checked" : "", $self->{'text_hash'}->{'list_any'};
printf "<input type=radio name=sport_def value=0 %s> %s\n",
	$_[5] eq '' || $_[5] eq '-' ? "" : "checked", $self->{'text_hash'}->{'rules_ranges'};
printf "<input name=sport size=30 value='%s'></td> </tr>\n",
	$_[5] eq '' || $_[5] eq '-' ? "" : join(" ", split(/,/, $_[5]));

print "<tr> <td><b>$self->{'text_hash'}->{'rules_5'}</b></td> <td colspan=3>\n";
printf "<input type=radio name=dport_def value=1 %s> %s\n",
	$_[4] eq '' || $_[4] eq '-' ? "checked" : "", $self->{'text_hash'}->{'list_any'};
printf "<input type=radio name=dport_def value=0 %s> %s\n",
	$_[4] eq '' || $_[4] eq '-' ? "" : "checked", $self->{'text_hash'}->{'rules_ranges'};
printf "<input name=dport size=30 value='%s'>\n",
	$_[4] eq '' || $_[4] eq '-' ? "" : join(" ", split(/,/, $_[4]));
print "<br>$self->{'text_hash'}->{'rules_dnat_port'}</td> </tr>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'rules_dnat'}</b></td> <td colspan=3>\n";
printf "<input type=radio name=dnat_def value=1 %s> %s\n",
	$_[6] eq '' || $_[6] eq '-' ? "checked" : "", $self->{'text_hash'}->{'list_none'};
printf "<input type=radio name=dnat_def value=0 %s>\n",
	$_[6] eq '' || $_[6] eq '-' ? "" : "checked";
printf "<input name=dnat size=30 value='%s'></td> </tr>\n",
	$_[6] eq '' || $_[6] eq '-' ? "" : $_[6];

if ($self->version_atleast(1, 4, 7)) {
	print "<tr> <td><b>$self->{'text_hash'}->{'rules_rate'}</b></td> <td colspan=3>\n";
	printf "<input type=radio name=rate_def value=1 %s> %s\n",
		$_[7] eq "-" || !$_[7] ? "checked" : "", $self->{'text_hash'}->{'rules_norate'};
	printf "<input type=radio name=rate_def value=0 %s>\n",
		$_[7] eq "-" || !$_[7] ? "" : "checked";
	printf "<input name=rate size=15 value='%s'></td> </tr>\n",
		$_[7] eq "-" ? "" : $_[7];

	print "<tr> <td><b>$self->{'text_hash'}->{'rules_set'}</b></td> <td colspan=3>\n";
	printf "<input type=radio name=set_def value=1 %s> %s\n",
		$_[8] eq "-" || !$_[8] ? "checked" : "", $self->{'text_hash'}->{'rules_noset'};
	printf "<input type=radio name=set_def value=0 %s>\n",
		$_[8] eq "-" || !$_[8] ? "" : "checked";
	printf "<input name=set size=15 value='%s'></td> </tr>\n",
		$_[8] eq "-" ? "" : $_[8];
	}
}

sub rules_validate
{
	my $self = shift;

$self->{'in_hash'}->{'action'} !~ /----/ || $self->error($self->{'text_hash'}->{'rules_eaction'});
$self->{'in_hash'}->{'source'} || $self->{'in_hash'}->{'sother'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'rules_esother'});
!$self->{'in_hash'}->{'sinzone_def'} || $self->{'in_hash'}->{'sinzone'} =~ /\S/ || $self->error($self->{'text_hash'}->{'rules_esinzone'});
$self->{'in_hash'}->{'dest'} || $self->{'in_hash'}->{'dother'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'rules_edother'});
!$self->{'in_hash'}->{'dinzone_def'} || $self->{'in_hash'}->{'dinzone'} =~ /\S/ || $self->error($self->{'text_hash'}->{'rules_edinzone'});
$self->{'in_hash'}->{'proto'} || $self->{'in_hash'}->{'pother'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'rules_epother'});
$self->{'in_hash'}->{'sport_def'} || $self->{'in_hash'}->{'sport'} =~ /\S/ || $self->error($self->{'text_hash'}->{'rules_esport'});
$self->{'in_hash'}->{'dport_def'} || $self->{'in_hash'}->{'dport'} =~ /\S/ || $self->error($self->{'text_hash'}->{'rules_edport'});
$self->{'in_hash'}->{'dnat_def'} || $self->check_ipaddress($self->{'in_hash'}->{'dnat'}) ||
	($self->{'in_hash'}->{'dnat'} =~ /^([0-9\.]+):([0-9\.]+)$/ &&
	 $self->check_ipaddress("$1") && $self->check_ipaddress("$2")) ||
	($self->{'in_hash'}->{'dnat'} =~ /^\!([0-9\.]+)$/ && $self->check_ipaddress("$1")) ||
	($self->{'in_hash'}->{'dnat'} =~ /^\!([0-9\.]+),([0-9\.]+)(\/\d+)?$/ &&
	 $self->check_ipaddress("$1") && $self->check_ipaddress("$2")) ||
	($self->{'in_hash'}->{'dnat'} =~ /^\!([0-9\.,]+)$/ &&
         scalar(grep { $self->check_ipaddress($_) } split(/,/, $1))) ||
	$self->error($self->{'text_hash'}->{'rules_ednat'});
$self->{'in_hash'}->{'action'} ne 'DNAT' && $self->{'in_hash'}->{'action'} ne 'REDIRECT' && $self->{'in_hash'}->{'action'} ne 'DNAT-' && 
	!$self->{'in_hash'}->{'dnat_def'} && $self->error($self->{'text_hash'}->{'rules_ednat2'});

$self->{'in_hash'}->{'sinzone'} =~ s/\s+/,/g;
$self->{'in_hash'}->{'dinzone'} =~ s/\s+/,/g;
$self->{'in_hash'}->{'sport'} =~ s/\s+/,/g;
$self->{'in_hash'}->{'dport'} =~ s/\s+/,/g;
if ($self->version_atleast(1, 4, 7)) {
	$self->{'in_hash'}->{'rate_def'} || $self->{'in_hash'}->{'rate'} =~ /^\S+$/ ||
		$self->error($self->{'text_hash'}->{'rules_erate'});
	$self->{'in_hash'}->{'set_def'} || $self->{'in_hash'}->{'set'} =~ /^\S+$/ ||
		$self->error($self->{'text_hash'}->{'rules_eset'});
	}
if ($self->{'in_hash'}->{'macro'} && $self->indexof($self->{'in_hash'}->{'action'}, $self->list_standard_macros()) >= 0) {
	$self->{'in_hash'}->{'action'} .= "/".$self->{'in_hash'}->{'macro'};
	$self->{'in_hash'}->{'proto'} = $self->{'in_hash'}->{'pother'} = undef;
	}
return ( $self->{'in_hash'}->{'log'} ? "$self->{'in_hash'}->{'action'}:$self->{'in_hash'}->{'log'}" : $self->{'in_hash'}->{'action'},
	 ($self->{'in_hash'}->{'source'} || $self->{'in_hash'}->{'sother'}).
	  ($self->{'in_hash'}->{'sinzone_def'} ? ":$self->{'in_hash'}->{'sinzone'}" : ""),
	 ($self->{'in_hash'}->{'dest'} || $self->{'in_hash'}->{'dother'}).
	  ($self->{'in_hash'}->{'dinzone_def'} ? ":$self->{'in_hash'}->{'dinzone'}" : ""),
	 $self->{'in_hash'}->{'proto'} || $self->{'in_hash'}->{'pother'},
	 $self->{'in_hash'}->{'dport_def'} ? "-" : $self->{'in_hash'}->{'dport'},
	 $self->{'in_hash'}->{'sport_def'} ? "-" : $self->{'in_hash'}->{'sport'},
	 $self->{'in_hash'}->{'dnat_def'} ? ( "-" ) : ( $self->{'in_hash'}->{'dnat'} ),
	 $self->version_atleast(1, 4, 7) ? (
		( $self->{'in_hash'}->{'rate_def'} ? "-" : $self->{'in_hash'}->{'rate'} ),
		( $self->{'in_hash'}->{'set_def'} ? "-" : $self->{'in_hash'}->{'set'} )
		) : ( )
	);
}

sub rules_columns
{
	my $self = shift;

return $self->version_atleast(1, 4, 7) ? 6 : 8;
}

sub tos_row
{
	my $self = shift;

return ( $self->is_fw($_[0]) ? $self->{'text_hash'}->{'list_fw'} :
	  $_[0] eq 'all' ? $self->{'text_hash'}->{'list_any'} :
	  $_[0] =~ /^([^:]+):(\S+)$/ ? $self->text('rules_hosts', "$1", "$2") :
	  $self->text('rules_zone', $_[0]),
	 $self->is_fw($_[1]) ? $self->{'text_hash'}->{'list_fw'} :
	  $_[1] eq 'all' ? $self->{'text_hash'}->{'list_any'} :
	  $_[1] =~ /^([^:]+):(\S+)$/ ? $self->text('rules_hosts', "$1", "$2") :
	  $self->text('rules_zone', $_[1]),
	 uc($_[2]),
	 $_[3] eq '-' || $_[3] eq '' ? $self->{'text_hash'}->{'list_any'} : $_[3],
	 $_[4] eq '-' || $_[4] eq '' ? $self->{'text_hash'}->{'list_any'} : $_[4],
	 $self->{'tos_map_hash'}->{$_[5]} || $_[5] );
}

sub tos_form
{
	my $self = shift;

local ($zone, $host) = split(/:/, $_[0], 2);
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'tos_0z'}</b></td>\n";
print "<td colspan=3 nowrap>\n";
$self->{'found'} = $self->zone_field("source", $zone, 1);
printf "<input name=sother size=10 value='%s'>\n",
	$self->{'found'} ? "" : $zone;

print "<br><b>$self->{'text_hash'}->{'rules_inzone'}</b>\n";
printf "<input type=checkbox name=sinzone_def value=1 %s> %s\n",
	$host ? "checked" : "", $self->{'text_hash'}->{'rules_addr'};
printf "<input name=sinzone size=50 value='%s'></td> </tr>\n",
	join(" ", split(/,/, $host));

($zone, $host) = split(/:/, $_[1], 2);
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'tos_1z'}</b></td>\n";
print "<td colspan=3 nowrap>\n";
$self->{'found'} = $self->zone_field("dest", $zone, 1);
printf "<input name=dother size=10 value='%s'>\n",
	$self->{'found'} ? "" : $zone;

print "<br><b>$self->{'text_hash'}->{'rules_inzone'}</b>\n";
printf "<input type=checkbox name=dinzone_def value=1 %s> %s\n",
	$host ? "checked" : "", $self->{'text_hash'}->{'rules_addr'};
printf "<input name=dinzone size=50 value='%s'></td> </tr>\n",
	join(" ", split(/,/, $host));

print "<tr> <td><b>$self->{'text_hash'}->{'tos_2'}</b></td>\n";
print "<td><select name=proto>\n";
$self->{'found'} = !$_[2];
foreach $p (@{$self->{'tos_protos_array'}}) {
	printf "<option value=%s %s>%s\n",
		$p, $p eq $_[2] ? "selected" : "", uc($p);
	$self->{'found'}++ if ($p eq $_[2]);
	}
printf "<option value='' %s>%s\n",
	$self->{'found'} ? "" : "selected", $self->{'text_hash'}->{'list_other'};
print "</select>\n";
printf "<input name=pother size=5 value='%s'></td> </tr>\n",
	$self->{'found'} ? "" : $_[2];

print "<tr> <td><b>$self->{'text_hash'}->{'tos_3'}</b></td> <td colspan=3>\n";
printf "<input type=radio name=sport_def value=1 %s> %s\n",
	$_[3] eq '' || $_[3] eq '-' ? "checked" : "", $self->{'text_hash'}->{'list_any'};
printf "<input type=radio name=sport_def value=0 %s> %s\n",
	$_[3] eq '' || $_[3] eq '-' ? "" : "checked", $self->{'text_hash'}->{'rules_ranges'};
printf "<input name=sport size=30 value='%s'></td> </tr>\n",
	$_[3] eq '' || $_[3] eq '-' ? "" : join(" ", split(/,/, $_[3]));

print "<tr> <td><b>$self->{'text_hash'}->{'tos_4'}</b></td> <td colspan=3>\n";
printf "<input type=radio name=dport_def value=1 %s> %s\n",
	$_[4] eq '' || $_[4] eq '-' ? "checked" : "", $self->{'text_hash'}->{'list_any'};
printf "<input type=radio name=dport_def value=0 %s> %s\n",
	$_[4] eq '' || $_[4] eq '-' ? "" : "checked", $self->{'text_hash'}->{'rules_ranges'};
printf "<input name=dport size=30 value='%s'></td> </tr>\n",
	$_[4] eq '' || $_[4] eq '-' ? "" : join(" ", split(/,/, $_[4]));

print "<tr> <td><b>$self->{'text_hash'}->{'tos_5'}</b></td>\n";
print "<td><select name=tos>\n";
$self->{'found'} = !$_[5];
foreach $t (sort { $a <=> $b } keys %{$self->{'tos_map_hash'}}) {
	printf "<option value=%s %s>%s\n",
		$t, $_[5] == $t ? "selected" : "", $self->{'tos_map_hash'}->{$t};
	$self->{'found'}++ if ($_[5] == $t);
	}
print "<option value=$_[5] selected>$_[5]\n" if (!$self->{'found'});
print "</select></td> </tr>\n";
}

sub tos_validate
{
	my $self = shift;

$self->{'in_hash'}->{'source'} || $self->{'in_hash'}->{'sother'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'rules_esother'});
!$self->{'in_hash'}->{'sinzone_def'} || $self->{'in_hash'}->{'sinzone'} =~ /\S/ || $self->error($self->{'text_hash'}->{'rules_esinzone'});
$self->{'in_hash'}->{'dest'} || $self->{'in_hash'}->{'dother'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'rules_edother'});
!$self->{'in_hash'}->{'dinzone_def'} || $self->{'in_hash'}->{'dinzone'} =~ /\S/ || $self->error($self->{'text_hash'}->{'rules_edinzone'});
$self->{'in_hash'}->{'proto'} || $self->{'in_hash'}->{'pother'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'rules_epother'});
$self->{'in_hash'}->{'sport_def'} || $self->{'in_hash'}->{'sport'} =~ /\S/ || $self->error($self->{'text_hash'}->{'rules_esport'});
$self->{'in_hash'}->{'dport_def'} || $self->{'in_hash'}->{'dport'} =~ /\S/ || $self->error($self->{'text_hash'}->{'rules_edport'});
return ( ($self->{'in_hash'}->{'source'} || $self->{'in_hash'}->{'sother'}).
	  ($self->{'in_hash'}->{'sinzone_def'} ? ":$self->{'in_hash'}->{'sinzone'}" : ""),
	 ($self->{'in_hash'}->{'dest'} || $self->{'in_hash'}->{'dother'}).
	  ($self->{'in_hash'}->{'dinzone_def'} ? ":$self->{'in_hash'}->{'dinzone'}" : ""),
	 $self->{'in_hash'}->{'proto'} || $self->{'in_hash'}->{'pother'},
	 $self->{'in_hash'}->{'sport_def'} ? "-" : join(",", split(/\s+/, $self->{'in_hash'}->{'sport'})),
	 $self->{'in_hash'}->{'dport_def'} ? "-" : join(",", split(/\s+/, $self->{'in_hash'}->{'dport'})),
	 $self->{'in_hash'}->{'tos'} );
}

sub masq_row
{
	my $self = shift;

return ( $_[0] =~ /^(\S+):(\S+)$/ ? $self->text('masq_in', "$1", "$2") : $_[0],
	 $_[1] =~ /^[0-9\.\/]+$/ ? $_[1] :
	  $_[1] =~ /^([a-z]+\d*)\!(\S+)$/ ? $self->text('masq_ex', "$1", "$2") :
	  $self->text('masq_iface', $_[1]),
	 $_[2] eq "" ? "" : $_[2] );
}

sub masq_columns
{
	my $self = shift;

return 3;
}

sub masq_form
{
	my $self = shift;

local ($iface, $net) = split(/:/, $_[0], 2);
print "<tr> <td><b>$self->{'text_hash'}->{'masq_0'}</b></td> <td colspan=3>\n";
$self->iface_field("iface", $iface);

printf "<input type=checkbox name=net_def value=1 %s> %s\n",
	$net ? "checked" : "", $self->{'text_hash'}->{'masq_net'};
print "<input name=net size=20 value='$net'></td> </tr>\n";

local ($mnet, $miface, $mode);
if ($_[1] =~ /^[0-9\.\/]+$/) {
	$mnet = $_[1];
	$mode = 0;
	}
elsif ($_[1] =~ /^([a-z]+\d*)\!(\S+)$/) {
	$miface = $1;
	$mnet = $2;
	$mode = 1;
	}
else {
	$miface = $_[1];
	$mode = 1;
	}
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'masq_1'}</b></td> <td colspan=3>\n";
printf "<input type=radio name=mode value=0 %s> %s\n",
	$mode == 0 ? "checked" : "", $self->{'text_hash'}->{'masq_mode0'};
printf "<input name=mnet size=20 value='%s'><br>\n",
	$mode == 0 ? $mnet : "";
printf "<input type=radio name=mode value=1 %s> %s\n",
	$mode == 1 ? "checked" : "", $self->{'text_hash'}->{'masq_mode1'};
$self->iface_field("miface", $mode == 1 ? $miface : undef);
printf "<input type=checkbox name=mnet_def value=1 %s> %s\n",
	$mode == 1 && $mnet ? "checked" : "", $self->{'text_hash'}->{'masq_except'};
printf "<input name=mnete size=20 value='%s'>\n",
	$mode == 1 ? join(" ", split(/,/, $mnet)) : "";
print "</td> </tr>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'masq_2'}</b></td> <td colspan=3>\n";
printf "<input type=radio name=snat_def value=1 %s> %s\n",
	$_[2] eq '' || $_[2] eq '-' ? "checked" : "", $self->{'text_hash'}->{'list_none'};
printf "<input type=radio name=snat_def value=0 %s>\n",
	$_[2] eq '' || $_[2] eq '-' ? "" : "checked";
printf "<input name=snat size=15 value='%s'></td> </tr>\n",
	$_[2] eq '' || $_[2] eq '-' ? "" : $_[2];

if ($self->version_atleast(3)) {
	print "<tr> <td><b>$self->{'text_hash'}->{'masq_3'}</b></td> <td colspan=3>\n";
	print $self->ui_radio("proto_def", $_[3] ? 0 : 1,
			[ [ 1, $self->{'text_hash'}->{'masq_any'} ],
			  [ 0, " " ] ]),"\n",
	      $self->ui_select("proto", $_[3],
			 [ map { [ $_, uc($_) ] } $self->list_protocols() ],
			 1, 0, $_[3] ? 1 : 0),"</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'masq_4'}</b></td> <td colspan=3>\n";
	print $self->ui_opt_textbox("ports", $_[4], 40, $self->{'text_hash'}->{'masq_all'}),
	      "</td> </tr>\n";

	print "<tr> <td><b>$self->{'text_hash'}->{'masq_5'}</b></td> <td colspan=3>\n";
	print $self->ui_opt_textbox("ipsec", $_[5], 40, $self->{'text_hash'}->{'default'}),
	      "</td> </tr>\n";
	}
}

sub masq_validate
{
	my $self = shift;

!$self->{'in_hash'}->{'net_def'} || $self->{'in_hash'}->{'net'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'masq_enet'});
if ($self->{'in_hash'}->{'mode'} == 0) {
	$self->{'in_hash'}->{'mnet'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'masq_emnet'});
	}
else {
	!$self->{'in_hash'}->{'mnet_def'} || $self->{'in_hash'}->{'mnete'} =~ /\S/ ||$self->error($self->{'text_hash'}->{'masq_emnete'});
	}
$self->{'in_hash'}->{'snat_def'} || $self->check_ipaddress($self->{'in_hash'}->{'snat'}) || $self->error($self->{'text_hash'}->{'masq_esnat'});

local @rv = ( $self->{'in_hash'}->{'iface'}.(!$self->{'in_hash'}->{'net_def'} ? "" : ":$self->{'in_hash'}->{'net'}"),
	 $self->{'in_hash'}->{'mode'} == 0 ? $self->{'in_hash'}->{'mnet'} :
	  $self->{'in_hash'}->{'miface'}.(!$self->{'in_hash'}->{'mnet_def'} ? "" :
			 "!".join(",", split(/\s+/, $self->{'in_hash'}->{'mnete'}))),
	 $self->{'in_hash'}->{'snat_def'} ? ( "" ) : ( $self->{'in_hash'}->{'snat'} ) );
if ($self->version_atleast(3)) {
	push(@rv, $self->{'in_hash'}->{'proto_def'} ? "" : $self->{'in_hash'}->{'proto'});
	if ($self->{'in_hash'}->{'ports_def'}) {
		push(@rv, "");
		}
	else {
		$self->{'in_hash'}->{'ports'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'masq_eports'});
		push(@rv, $self->{'in_hash'}->{'ports'});
		}
	if ($self->{'in_hash'}->{'ipsec_def'}) {
		push(@rv, "");
		}
	else {
		$self->{'in_hash'}->{'ipsec'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'masq_eipsec'});
		push(@rv, $self->{'in_hash'}->{'ipsec'});
		}
	}
return @rv;
}

sub nat_form
{
	my $self = shift;

print "<tr> <td><b>$self->{'text_hash'}->{'nat_0'}</b></td>\n";
print "<td><input name=ext size=15 value='$_[0]'></td>\n";

print "<td><b>$self->{'text_hash'}->{'nat_1'}</b></td>\n";
print "<td>";
if ($self->version_atleast(1, 3, 14)) {
	local ($iface, $virt) = split(/:/, $_[1]);
	$self->iface_field("iface", $iface);
	print "<b>$self->{'text_hash'}->{'nat_virt'}</b>\n";
	print "<input name=virt size=3 value='$virt'>\n";
	print "</td> </tr>\n";
	}
else {
	$self->iface_field("iface", $_[1]);
	print "</td> </tr>\n";
	}

print "<tr> <td><b>$self->{'text_hash'}->{'nat_2'}</b></td>\n";
print "<td><input name=int size=15 value='$_[2]'></td> </tr>\n";

local $all = $_[3] eq '-' || $_[3] eq '' || $_[3] =~ /yes/i;
print "<tr> <td><b>$self->{'text_hash'}->{'nat_all'}</b></td>\n";
printf "<td><input type=radio name=all value=1 %s> %s\n",
	$all ? "checked" : "", $self->{'text_hash'}->{'yes'};
printf "<input type=radio name=all value=0 %s> %s</td>\n",
	$all ? "" : "checked", $self->{'text_hash'}->{'no'};

local $local = $_[4] =~ /yes/i;
print "<td><b>$self->{'text_hash'}->{'nat_local'}</b></td>\n";
printf "<td><input type=radio name=local value=1 %s> %s\n",
	$local ? "checked" : "", $self->{'text_hash'}->{'yes'};
printf "<input type=radio name=local value=0 %s> %s</td> </tr>\n",
	$local ? "" : "checked", $self->{'text_hash'}->{'no'};
}

sub nat_validate
{
	my $self = shift;

$self->check_ipaddress($self->{'in_hash'}->{'ext'}) || $self->error($self->{'text_hash'}->{'nat_eext'});
$self->check_ipaddress($self->{'in_hash'}->{'int'}) || $self->error($self->{'text_hash'}->{'nat_eint'});
$self->{'in_hash'}->{'virt'} =~ /^\d*$/ || $self->error($self->{'text_hash'}->{'nat_evirt'});
return ( $self->{'in_hash'}->{'ext'},
	 $self->{'in_hash'}->{'virt'} ne '' ? $self->{'in_hash'}->{'iface'}.":".$self->{'in_hash'}->{'virt'} : $self->{'in_hash'}->{'iface'},
	 $self->{'in_hash'}->{'int'},
	 $self->{'in_hash'}->{'all'} ? "yes" : "no",
	 $self->{'in_hash'}->{'local'} ? "yes" : "no" );
}

sub proxyarp_row
{
	my $self = shift;

return ( $_[0],
	 $_[1] eq '-' || $_[1] eq '' ? $self->{'text_hash'}->{'list_auto'} : $_[1],
	 $_[2],
	 $self->version_atleast(2, 0, 0) ?
		( $_[4] =~ /yes/i ? $self->{'text_hash'}->{'yes'} : $self->{'text_hash'}->{'no'} ) : ( ) );
}

sub proxyarp_form
{
	my $self = shift;

print "<tr> <td><b>$self->{'text_hash'}->{'proxyarp_0'}</b></td>\n";
print "<td><input name=addr size=15 value='$_[0]'></td>\n";

print "<td><b>$self->{'text_hash'}->{'proxyarp_1'}</b></td>\n";
printf "<td><input type=radio name=int_def value=1 %s> %s\n",
	$_[1] eq '-' || $_[1] eq '' ? "checked" : "", $self->{'text_hash'}->{'list_auto'};
printf "<input type=radio name=int_def value=0 %s>\n",
	$_[1] eq '-' || $_[1] eq '' ? "" : "checked";
$self->iface_field("int", $_[1] eq '-' ? undef : $_[1]);
print "</td> </tr>";

local $have = $_[3] =~ /yes/i;
print "<tr> <td><b>$self->{'text_hash'}->{'proxyarp_have'}</b></td>\n";
printf "<td><input type=radio name=have value=1 %s> %s\n",
	$have ? "checked" : "", $self->{'text_hash'}->{'yes'};
printf "<input type=radio name=have value=0 %s> %s</td>\n",
	$have ? "" : "checked", $self->{'text_hash'}->{'no'};

print "<td><b>$self->{'text_hash'}->{'proxyarp_2'}</b></td>\n";
print "<td>";
$self->iface_field("ext", $_[2]);
print "</td> </tr>";

if ($self->version_atleast(2, 0, 0)) {
	local $pers = $_[4] =~ /yes/i;
	print "<tr> <td><b>$self->{'text_hash'}->{'proxyarp_pers'}</b></td>\n";
	printf "<td><input type=radio name=pers value=1 %s> %s\n",
		$pers ? "checked" : "", $self->{'text_hash'}->{'yes'};
	printf "<input type=radio name=pers value=0 %s> %s</td>\n",
		$pers ? "" : "checked", $self->{'text_hash'}->{'no'};
	}
}

sub proxyarp_validate
{
	my $self = shift;

$self->check_ipaddress($self->{'in_hash'}->{'addr'}) || $self->error($self->{'text_hash'}->{'proxyarp_eaddr'});
return ( $self->{'in_hash'}->{'addr'},
	 $self->{'in_hash'}->{'int_def'} ? "-" : $self->{'in_hash'}->{'int'},
	 $self->{'in_hash'}->{'ext'},
	 $self->{'in_hash'}->{'have'} ? "yes" : "no",
	 $self->version_atleast(2, 0, 0) ? ( $self->{'in_hash'}->{'pers'} ? "yes" : "no" ) : ( )
	); 
	 
}

sub proxyarp_columns
{
	my $self = shift;

return $self->version_atleast(2, 0, 0) ? 4 : 3;
}

sub routestopped_row
{
	my $self = shift;

return ( $_[0], $_[1],
	 $_[2] eq '-' || $_[2] eq '' ? $self->{'text_hash'}->{'default'} : $_[2],
	 $_[3] eq '-' || $_[3] eq '' ? $self->{'text_hash'}->{'tunnels_gnone'} : $_[3] );
}

sub routestopped_columns
{
	my $self = shift;

return 2;
}

sub routestopped_form
{
	my $self = shift;

print "<tr> <td valign=top><b>$self->{'text_hash'}->{'routestopped_0'}</b></td>\n";
print "<td valign=top>";
$self->iface_field("iface", $_[0]);
print "</td>\n";

local $none = $_[1] eq '' || $_[1] eq '-' || $_[1] eq '0.0.0.0/0';
print "<td valign=top><b>$self->{'text_hash'}->{'routestopped_1'}</b></td>\n";
printf "<td><input type=radio name=addr_def value=1 %s> %s<br>\n",
	$none ? "checked" : "", $self->{'text_hash'}->{'routestopped_all'};
printf "<input type=radio name=addr_def value=0 %s> %s<br>\n",
	$none ? "" : "checked", $self->{'text_hash'}->{'routestopped_list'};
print "<textarea name=addr rows=5 cols=20>",
	$none ? "" : join("\n", split(/,/, $_[1])),
	"</textarea></td> </tr>\n";

if ($self->version_atleast(3)) {
	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'routestopped_2'}</b></td>\n";
	print "<td colspan=3>\n";
	$self->options_input("opts", $_[2], \@{$self->{'routestopped_options_array'}});
	print "</td> </tr>\n";
	}
}

sub routestopped_validate
{
	my $self = shift;

$self->{'in_hash'}->{'addr_def'} || $self->{'in_hash'}->{'addr'} =~ /\S/ || $self->error($self->{'text_hash'}->{'routestopped_eaddr'});
return ( $self->{'in_hash'}->{'iface'},
	 $self->{'in_hash'}->{'addr_def'} ? "-" : join(",", split(/\s+/, $self->{'in_hash'}->{'addr'})),
	 join(",", split(/\0/, $self->{'in_hash'}->{'opts'})) );
}

sub tunnels_row
{
	my $self = shift;

local $tt = $_[0];
$tt =~ s/^(openvpn|generic):.*$/$1/;
return ( $self->{'text_hash'}->{'tunnels_'.$tt} || $tt,
	 $_[1] eq '-' || $_[1] eq '' ? $self->{'text_hash'}->{'routestopped_all'} : $_[1],
	 $_[2], $_[3] );
}

sub tunnels_form
{
	my $self = shift;

print "<tr> <td><b>$self->{'text_hash'}->{'tunnels_0'}</b></td>\n";
print "<td><select name=type>\n";
local $tt;
local $found = !$_[0];
local $ttype = $_[0];
local $tport;
if ($ttype =~ s/^(openvpn|generic):(.*)$/$1/) {
	$tport = $2;
	}
foreach $tt ('ipsec', 'ipsecnat',
	     ($self->version_atleast(2, 0, 0) ? ( 'ipsec:noah', 'ipsecnat:noah' )
				       : ( )),
	     'ip', 'gre', 'pptpclient', 'pptpserver', 'generic',
	     ($self->version_atleast(1, 3, 14) ? ( 'openvpn' ) : ( )) ) {
	printf "<option value=%s %s>%s\n",
		$tt, $ttype eq $tt ? "selected" : "",
		$self->{'text_hash'}->{'tunnels_'.$tt.'_l'} || $self->{'text_hash'}->{'tunnels_'.$tt};
	$found++ if ($ttype eq $tt);
	}
print "<option value=$ttype selected>",uc($ttype),"\n" if (!$found);
print "</select>\n";
print "<input name=tport size=10 value='$tport'>\n";
print "</td>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'tunnels_1'}</b></td>\n";
print "<td>";
$self->zone_field("zone", $_[1], 0, 0);
print "</td> </tr>\n";

local $none = $_[2] eq '' || $_[2] eq '-';
print "<tr> <td><b>$self->{'text_hash'}->{'tunnels_2'}</b></td> <td valign=top>\n";
printf "<input type=radio name=gateway_def value=1 %s> %s\n",
	$none ? "checked" : "", $self->{'text_hash'}->{'default'};
printf "<input type=radio name=gateway_def value=0 %s> %s\n",
	$none ? "" : "checked", $self->{'text_hash'}->{'tunnels_sel'};
printf "<input name=gateway size=20 value='%s'></td> </tr>\n", $_[2];

local $none = $_[2] eq '' || $_[2] eq '-';
print "<tr> <td><b>$self->{'text_hash'}->{'tunnels_3'}</b></td> <td valign=top>\n";
printf "<input type=radio name=gzones_def value=1 %s> %s\n",
	$none ? "checked" : "", $self->{'text_hash'}->{'tunnels_gnone'};
printf "<input type=radio name=gzones_def value=0 %s> %s\n",
	$none ? "" : "checked", $self->{'text_hash'}->{'tunnels_gsel'};
printf "<input name=gzones size=50 value='%s'></td> </tr>\n",
	join(" ", split(/,/, $_[3]));
}

sub tunnels_validate
{
	my $self = shift;

$self->{'in_hash'}->{'gateway_def'} || $self->check_ipaddress($self->{'in_hash'}->{'gateway'}) ||
	($self->{'in_hash'}->{'gateway'} =~ /^(\S+)\/(\d+)$/ && $self->check_ipaddress($1)) ||
		$self->error($self->{'text_hash'}->{'tunnels_egateway'});
if ($self->{'in_hash'}->{'type'} eq "openvpn") {
	$self->{'in_hash'}->{'tport'} =~ /^\d*$/ || $self->error($self->{'text_hash'}->{'tunnels_eopenvpn'});
	$self->{'in_hash'}->{'type'} .= ":".$self->{'in_hash'}->{'tport'} if ($self->{'in_hash'}->{'tport'});
	}
elsif ($self->{'in_hash'}->{'type'} eq 'generic') {
	$self->{'in_hash'}->{'tport'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'tunnels_egeneric'});
	$self->{'in_hash'}->{'type'} .= ":".$self->{'in_hash'}->{'tport'};
	}
return ( $self->{'in_hash'}->{'type'}, $self->{'in_hash'}->{'zone'},
	 $self->{'in_hash'}->{'gateway_def'} ? '-' : $self->{'in_hash'}->{'gateway'},
	 $self->{'in_hash'}->{'gzones_def'} ? '-' : join(",", split(/\s+/, $self->{'in_hash'}->{'gzones'})) );
}

sub hosts_row
{
	my $self = shift;

return ( $_[0], $_[1] =~ /^(\S+):(\S+)$/ ? ( $1, $2 ) : ( undef, undef ) );
}

sub hosts_form
{
	my $self = shift;

print "<tr> <td><b>$self->{'text_hash'}->{'hosts_0'}</b></td>\n";
print "<td>";
$self->zone_field("zone", $_[0], 0, 2);
print "</td> </tr>\n";

local ($iface, $net) = split(/:/, $_[1]);
print "<tr> <td><b>$self->{'text_hash'}->{'hosts_1'}</b></td>\n";
print "<td>";
$self->iface_field("iface", $iface);
print "</td> </tr>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'hosts_2'}</b></td>\n";
print "<td><input name=net size=50 value='$net'></td> </tr>\n";

print "<tr> <td valign=top><b>$self->{'text_hash'}->{'hosts_opts'}</b></td> <td>\n";
$self->options_input("opts", $_[2], \@{$self->{'host_options_array'}});
print "</td> </tr>\n";
}

sub hosts_validate
{
	my $self = shift;

$self->check_ipaddress($self->{'in_hash'}->{'net'}) ||
	$self->{'in_hash'}->{'net'} =~ /^(\S+)\/(\d+)$/ && $self->check_ipaddress($1) ||
	$self->error($self->{'text_hash'}->{'hosts_enet'});
return ( $self->{'in_hash'}->{'zone'}, $self->{'in_hash'}->{'iface'}.":".$self->{'in_hash'}->{'net'},
	 join(",", split(/\0/, $self->{'in_hash'}->{'opts'})) );
}

sub blacklist_row
{
	my $self = shift;

return ( $_[0], uc($_[1]) || $self->{'text_hash'}->{'blacklist_any'},
		$_[2] || $self->{'text_hash'}->{'blacklist_any'} );
}

sub blacklist_form
{
	my $self = shift;

print "<tr> <td valign=top><b>$self->{'text_hash'}->{'blacklist_host'}</b></td> <td colspan=3>\n";
local ($mode, $ipset, $mac, $ip);
if ($_[0] =~ /^\+(.*)/) {
	$mode = 2; $ipset = $1;
	}
elsif ($_[0] =~ /^\~(.*)$/) {
	$mode = 1; $mac = $1;
	}
else {
	$mode = 0; $ip = $_[0];
	}
print $self->ui_radio("host_def", $mode,
    [ [ 0, $self->text('hosts_ip', $self->ui_textbox("host", $ip, 30))."<br>" ],
      [ 1, $self->text('hosts_mac', $self->ui_textbox("mac", $mac, 30))."<br>" ],
      $self->version_atleast(3) ?
       ( [ 2, $self->text('hosts_ipset', $self->ui_textbox("ipset", $ipset, 15)) ] ) : ( ),
	    ]);
print "</td> </tr>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'blacklist_proto'}</b></td>\n";
print "<td colspan=3><select name=proto>\n";
$self->{'found'} = !$_[1];
foreach $p (@{$self->{'blacklist_protos_array'}}) {
	printf "<option value='%s' %s>%s\n",
		$p, $p eq $_[1] ? "selected" : "",
		$p eq '' ? "&lt;$self->{'text_hash'}->{'list_any'}&gt;" : uc($p);
	$self->{'found'}++ if ($p eq $_[1]);
	}
printf "<option value='*' %s>%s\n",
	$self->{'found'} ? "" : "selected", $self->{'text_hash'}->{'list_other'};
print "</select>\n";
printf "<input name=pother size=5 value='%s'></td> </tr>\n",
	$self->{'found'} ? "" : $_[1];

print "<tr> <td><b>$self->{'text_hash'}->{'blacklist_ports'}</b></td>\n";
print "<td colspan=3><input name=ports size=20 value='$_[2]'></td> </tr>\n";
}

sub blacklist_validate
{
	my $self = shift;

local $host;
if ($self->{'in_hash'}->{'host_def'} == 0) {
	$self->check_ipaddress($self->{'in_hash'}->{'host'}) ||
		$self->{'in_hash'}->{'host'} =~ /^(\S+)\/(\d+)$/ && $self->check_ipaddress($1) ||
		$self->error($self->{'text_hash'}->{'blacklist_ehost'});
	$host = $self->{'in_hash'}->{'host'};
	}
elsif ($self->{'in_hash'}->{'host_def'} == 1) {
	$self->{'in_hash'}->{'mac'} =~ s/:/-/g;
	$self->{'in_hash'}->{'mac'} =~ /^[0-9a-f]{2}(\-[0-9a-f]{2}){5}$/ ||
		$self->error($self->{'text_hash'}->{'blacklist_emac'});
	$host = "~".$self->{'in_hash'}->{'mac'};
	}
elsif ($self->{'in_hash'}->{'host_def'} == 2) {
	$self->{'in_hash'}->{'ipset'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'blacklist_eipset'});
	$host = "+".$self->{'in_hash'}->{'ipset'};
	}
local $proto;
if ($self->{'in_hash'}->{'proto'} eq '*') {
	$self->{'in_hash'}->{'pother'} =~ /^\d+$/ ||
	    defined(getprotobyname($self->{'in_hash'}->{'pother'})) ||
		$self->error($self->{'text_hash'}->{'blacklist_eproto'});
	$proto = lc($self->{'in_hash'}->{'pother'});
	}
else {
	$proto = lc($self->{'in_hash'}->{'proto'});
	}
if ($proto eq "tcp" || $proto eq "udp") {
	$self->{'in_hash'}->{'ports'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'blacklist_eports'});
	}
elsif ($self->{'in_hash'}->{'ports'}) {
	$self->error($self->{'text_hash'}->{'blacklist_eports2'});
	}
return ( $host, $proto, $self->{'in_hash'}->{'ports'} );
}

sub providers_row
{
	my $self = shift;

return ( $_[0], $_[1], $_[2], $_[4], $_[5] );
}

sub providers_form
{
	my $self = shift;

print "<tr> <td><b>$self->{'text_hash'}->{'providers_name'}</b></td>\n";
print "<td><input name=name size=20 value='$_[0]'></td>\n";

print "<td><b>$self->{'text_hash'}->{'providers_number'}</b></td>\n";
print "<td><input name=number size=4 value='$_[1]'></td> </tr>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'providers_iface'}</b></td>\n";
print "<td>";
$self->iface_field("iface", $_[4]);
print "</td>\n";

print "<td><b>$self->{'text_hash'}->{'providers_mark'}</b></td>\n";
print "<td><input name=mark size=4 value='$_[2]'></td> </tr>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'providers_gateway'}</b></td>\n";
print "<td><input name=gateway size=15 value='$_[5]'></td>\n";

local $ddef = $_[3] eq "-" || $_[3] eq "" ? 0 : $_[3] eq "main" ? 1 : 2;
print "<td><b>$self->{'text_hash'}->{'providers_dup'}</b></td>\n";
print "<td>",$self->ui_radio("dup_def", $ddef,
		[ [ 0, $self->{'text_hash'}->{'default'} ],
		  [ 1, $self->{'text_hash'}->{'providers_main'} ],
		  [ 2, $self->ui_textbox("dup", $ddef == 2 ? $_[3] : "", 5) ] ]),
      "</td> </tr>\n";

local %opts = map { $_, 1 } split(/,/, $_[6]);
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'providers_opts'}</b></td> <td>\n";
foreach my $o (@providers_opts) {
	print $self->ui_checkbox("opts", $o, $self->{'text_hash'}->{'providers_'.$o}, $opts{$o})."<br>\n";
	delete($opts{$o});
	}
foreach my $o (keys %opts) {
	print $self->ui_hidden("opts", $o),"\n";
	}
print "</td>\n";

print "<td valign=top><b>$self->{'text_hash'}->{'providers_copy'}</b></td>\n";
print "<td valign=top><input name=copy size=15 value='$_[7]'></td> </tr>\n";
}

sub providers_validate
{
	my $self = shift;

$self->{'in_hash'}->{'name'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'providers_ename'});
$self->{'in_hash'}->{'number'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'providers_enumber'});
$self->{'in_hash'}->{'mark'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'providers_emark'});
$self->{'in_hash'}->{'dup_def'} < 2 || $self->{'in_hash'}->{'dup'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'providers_edup'});
$self->check_ipaddress($self->{'in_hash'}->{'gateway'}) || $self->error($self->{'text_hash'}->{'providers_egateway'});
return ( $self->{'in_hash'}->{'name'}, $self->{'in_hash'}->{'number'}, $self->{'in_hash'}->{'mark'},
	 $self->{'in_hash'}->{'dup_def'} == 0 ? '-' : $self->{'in_hash'}->{'dup_def'} == 1 ? 'main' : $self->{'in_hash'}->{'dup'},
	 $self->{'in_hash'}->{'iface'}, $self->{'in_hash'}->{'gateway'},
	 join(",", split(/\0/, $self->{'in_hash'}->{'opts'})) || "-",
	 $self->{'in_hash'}->{'copy'} || "-" );
}

# can_access(file)
sub can_access
{
	my $self = shift;

if ($self->{'access_hash'}->{'files'} eq '*') {
	return 1;
	}
else {
	local @acc = split(/\s+/, $self->{'access_hash'}->{'files'});
	return $self->indexof($_[0], @acc) >= 0;
	}
}

# run_before_apply_command()
# Runs the before-applying command, if any. If it failes, returns the error
# message output
sub run_before_apply_command
{
	my $self = shift;

if ($self->{'config_hash'}->{'before_apply_cmd'}) {
	local $out = $self->backquote_logged("($self->{'config_hash'}->{'before_apply_cmd'}) </dev/null 2>&1");
	return $out if ($?);
	}
return undef;
}

# run_after_apply_command()
# Runs the after-applying command, if any
sub run_after_apply_command
{
	my $self = shift;

if ($self->{'config_hash'}->{'after_apply_cmd'}) {
	$self->system_logged("($self->{'config_hash'}->{'after_apply_cmd'}) </dev/null >/dev/null 2>&1");
	}
}

# list_standard_actions()
# Returns a list of standard Shorewall actions
sub list_standard_actions
{
	my $self = shift;

local @rv;
foreach my $a (split(/\t+/, $self->{'config_hash'}->{'actions'})) {
	open(ACTIONS, $a);
	while(<ACTIONS>) {
		s/\r|\n//g;
		s/#.*$//;
		s/\s+$//;
		if (/\S/) {
			push(@rv, $_);
			}
		}
	close(ACTIONS);
	}
if ($self->version_atleast(3)) {
	# Add built-in actions
	push(@rv, "allowBcast", "dropBcast", "dropNotSyn", "rejNotSyn",
		  "dropInvalid", "allowInvalid", "allowoutUPnP", "allowinUPnP",
		  "forwardUPnP");
	}
return $self->unique(@rv);
}

# list_standard_macros()
# Returns a list of all macro. actions
sub list_standard_macros
{
	my $self = shift;

local @rv;
foreach my $a ($self->{'config_hash'}->{'config_dir'}, $self->{'config_hash'}->{'macros'}) {
	opendir(DIR, $a);
	foreach my $f (readdir(DIR)) {
		push(@rv, $1) if ($f =~ /^macro\.(.*)$/);
		}
	closedir(DIR);
	}
return $self->unique(@rv);
}

# get_shorewall_version(nocache)
sub get_shorewall_version
{
	my $self = shift;

local ($nocache) = @_;
local $version;
if (!$nocache && open(VERSION, "$self->{'module_config_directory'}/version")) {
	chop($version = <VERSION>);
	close(VERSION);
	}
else {
	local $out = `$self->{'config_hash'}->{'shorewall'} version 2>&1`;
	$out =~ s/\r//g;
	if ($out =~ /(\n|^)([0-9\.]+)\n/) {
		$version = $2;
		}
	}
return $version;
}

sub list_protocols
{
	my $self = shift;

local @stdprotos = ( 'tcp', 'udp', 'icmp' );
local @otherprotos;
open(PROTOS, "/etc/protocols");
while(<PROTOS>) {
	s/\r|\n//g;
	s/#.*$//;
	push(@otherprotos, $1) if (/^(\S+)\s+(\d+)/);
	}
close(PROTOS);
@otherprotos = sort { lc($a) cmp lc($b) } @otherprotos;
return $self->unique(@stdprotos, @otherprotos);
}

# options_input(name, value, &opts)
sub options_input
{
	my $self = shift;

local ($name, $value, $opts) = @_;
local %opts = map { $_, 1 } split(/,/, $value);
print "<table width=100%>\n";
local $i = 0;
foreach my $o (@$opts) {
	print "<tr>\n" if ($i%3 == 0);
	printf "<td><input type=checkbox name=$name value=%s %s> %s</td>\n",
		$o, $opts{$o} ? "checked" : "", $self->{'text_hash'}->{'opts_'.$o} || $o;
	print "</tr>\n" if ($i%3 == 2);
	delete($opts{$o});
	$i++;
	}
foreach $o (keys %opts) {
	print "<input type=hidden name=opts value=$o>\n";
	}
print "</table>\n";
}


1;

