#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Show all drives and their SMART status

&ModuleLoader::preload_module('smart_status_class', "smart_status::SmartstatusLib");
$self = new smart_status::SmartstatusLib:: 'smart_status_class',%$self;

$self->ReadParse();

# Make sure SMART commands are installed
if (!$self->has_command($self->{'config_hash'}->{'smartctl'})) {
	$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", undef, 1, 1);
	$self->ui_print_endpage(
		$self->ui_config_link('index_ecmd',
		        [ "<tt>$self->{'config_hash'}->{'smartctl'}</tt>", undef ]));
	}

# Show the version
$self->{'ver'} = $self->get_smart_version();
if (!$self->{'ver'}) {
	$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", undef, 1, 1);
	$self->ui_print_endpage(
		$self->ui_config_link('index_ecmd2',
		        [ "<tt>$self->{'config_hash'}->{'smartctl'}</tt>", undef ]));
	}
$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", undef, 1, 1, 0,
		 $self->help_search_link("smartctl", "man", "doc", "google"),
		 undef, undef,
		 $self->text('index_version', $self->{'ver'}));

# Get list of drives
@{$self->{'drives_array'}} = grep { $_->{'type'} eq 'ide' ||
		 $_->{'type'} eq 'scsi' } $main::fdisk_object->list_disks_partitions();
@{$self->{'drives_array'}} = sort { $a->{'device'} cmp $b->{'device'} } @{$self->{'drives_array'}};
if (!@{$self->{'drives_array'}}) {
	$self->ui_print_endpage($self->{'text_hash'}->{'index_eidescsi'});
	}

if ($self->{'config_hash'}->{'mode'} == 1 || $self->{'in_hash'}->{'drive'}) {
	# Just show one drive, selected from menu
	print $self->ui_form_start("index.cgi");
	print "<b>$self->{'text_hash'}->{'index_show'}</b>\n";
	print $self->ui_select("drive", $self->{'in_hash'}->{'drive'},
			 [ map { [ $_->{'device'},
				   $_->{'desc'}.($_->{'model'} ? " ($_->{'model'})" : "") ] } @{$self->{'drives_array'}} ]);
	print $self->ui_submit($self->{'text_hash'}->{'index_ok'}),"\n";
	print $self->ui_form_end();

	if ($self->{'in_hash'}->{'drive'}) {
		($d) = grep { $_->{'device'} eq $self->{'in_hash'}->{'drive'} } @{$self->{'drives_array'}};
		&show_drive($d);
		}
	}
else {
	# Show all IDE drives
	foreach $d (@{$self->{'drives_array'}}) {
		&show_drive($d);
		}
	}

$self->ui_print_footer("/", $self->{'text_hash'}->{'index'});

1;
		
}

# show_drive(&drive)
sub show_drive
{
	shift if ($self == $_[0]);

print $self->ui_form_start("action.cgi");
print $self->ui_hidden("drive", $_[0]->{'device'});
print $self->ui_table_start($self->text('index_drive', "<tt>$_[0]->{'device'}</tt>"),
		      undef, 2);
local $st = $self->get_drive_status($_[0]->{'device'});
print $self->ui_table_row($self->{'text_hash'}->{'index_desc'},
		    $_[0]->{'desc'});
if ($_[0]->{'cylsize'}) {
	print $self->ui_table_row($self->{'text_hash'}->{'index_size'},
		    $self->nice_size($_[0]->{'cylinders'}*$_[0]->{'cylsize'}));
	}
if ($_[0]->{'model'}) {
	print $self->ui_table_row($self->{'text_hash'}->{'index_model'},
			    $_[0]->{'model'});
	}
print $self->ui_table_row($self->{'text_hash'}->{'index_support'},
		    $st->{'support'} ? $self->{'text_hash'}->{'yes'} : $self->{'text_hash'}->{'no'});
print $self->ui_table_row($self->{'text_hash'}->{'index_enabled'},
		    $st->{'enabled'} ? $self->{'text_hash'}->{'yes'} : $self->{'text_hash'}->{'no'});
if ($st->{'support'} && $st->{'enabled'}) {
	if ($st->{'errors'}) {
		print $self->ui_table_row($self->{'text_hash'}->{'index_errors'},
				    "<font color=#ff0000>".
				    $self->text('index_ecount', $st->{'errors'}).
				    "</font>");
		}
	print $self->ui_table_row($self->{'text_hash'}->{'index_check'},
			    $st->{'check'} ? $self->{'text_hash'}->{'yes'} : "<font color=#ff0000>$self->{'text_hash'}->{'no'}</font>");
	if ($self->{'config_hash'}->{'attribs'}) {
		print $self->ui_table_hr();
		local $a;
		foreach $a (@{$st->{'attribs'}}) {
			next if ($a->[0] =~ /UDMA CRC Error Count/i); # too long
			print $self->ui_table_row($a->[0],
				$a->[2] =~ /^\s*(seconds|minutes|hours|days|months|years|weeks)\s*/i || !$a->[2] ? $a->[1]." ".$a->[2] : $a->[2]);
			}
		}
	}
print $self->ui_table_end();
if ($st->{'support'} && $st->{'enabled'}) {
	print $self->ui_form_end([ [ "short", $self->{'text_hash'}->{'index_short'} ],
			     [ "ext", $self->{'text_hash'}->{'index_ext'} ],
			     [ "data", $self->{'text_hash'}->{'index_data'} ] ]);
	}
else {
	print $self->ui_form_end();
	}
}


1;

