#!/usr/bin/perl
package smf::SmfLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'smf::SmfLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
#
# Common functions for managing smf services

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
require '../javascript-lib.pl';

&ModuleLoader::preload_module(ref($self), "smf::Wizard");
$self = new smf::Wizard:: ref($self), %$self; 

$self->{'lib'} = $self->get_mod_lib();
if ($self->{'lib'}) {
	{
    my $pkg = $self->lib_to_pkg("$self->{lib}", "smf");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
	}

# Hash matching svc type to prefix
%{$self->{'svc_types_hash'}} = ("Network", "svc:/network/",
	"System", "svc:/system/",
	"Legacy", "lrc:/",
	"Milestone", "svc:/milestone/",
	"Platform-specific", "svc:/platform/",
	"Application", "svc:/application/",
	"Device-specific", "svc:/device/",
	"Site-specific", "svc:/site/",
	"All", "svc:/");

# Hash matching svc option to description
%{$self->{'svc_options_hash'}} = ("FMRI", "FMRI",
	"STATE", "State",
	"DESC", "Description",
	"STIME", "StartTime",
	"NSTATE", "NextState");

# defaults for main svc page
$self->{'default_svc_options'} = "FMRI,STATE,DESC";
$self->{'default_sortopt'} = "-SSTATE";
$self->{'default_include_disabled'} = 0;

# this hash associates smf states with colors
%{$self->{'state_colors_hash'}} = ("online", "green",
	"offline", "yellow",
	"maintenance", "red",
	"legacy_run", "gray",
	"disabled", "gray");

# this variable is used to record commands run. these are then
# displayed at bottom of page, in msg "this page was generated using
# commands $self->{'cmds_run'}".

$self->{'cmds_run'} = "";

$self->{'maxint'} = 65535;

# this array constitutes the specification of the smf service
# creation wizard
$self->{'wizard'} = "page=smfwizard_service.cgi,min=1,max=1; \
	page=smfwizard_instance.cgi,min=1,max=$self->{'maxint'}; \
	page=smfwizard_restarter.cgi,min=0,max=1; \
	page=smfwizard_dependency.cgi,min=0,max=$self->{'maxint'}; \
	page=smfwizard_dependent.cgi,min=0,max=$self->{'maxint'}; \
	page=smfwizard_exec.cgi,min=0,max=$self->{'maxint'}; \
	page=smfwizard_property_group.cgi,min=0,max=$self->{'maxint'}; \
	page=smfwizard_property.cgi,min=0,max=$self->{'maxint'}; \
	page=smfwizard_template.cgi,min=0,max=1; \
	page=smfwizard_manpage.cgi,min=0,max=$self->{'maxint'}; \
	submit=smfwizard_process_submit.cgi";

# Array of boolean values
@{$self->{'boolean_values_array'}} = ("true", "false");

# Array of dependency/dependent types
@{$self->{'dep_types_array'}} = ("service", "path");
# Array of restart_on values
@{$self->{'restart_on_values_array'}} = ("none", "fault", "restart", "refresh", "error");
# Array of grouping values
@{$self->{'grouping_values_array'}} = ("require_all", "require_any", "optional_all",
        "exclude_all");
# Array of property value types
@{$self->{'propval_type_values_array'}} = ("count", "integer", "opaque", "host", "hostname",
"net_address_v4", "net_address_v6", "time", "astring", "ustring", "boolean",
"fmri", "uri");

# Array of stability values
@{$self->{'stability_values_array'}} = ("Evolving", "Unstable", "External", "-");

# Array of number of lines choices for logfile viewer
@{$self->{'logfile_numlines_values_array'}} = ("20", "40", "80", "160", "all");

# subroutines used to run/check/log smf commands.

# add/delete/list propgroup/prop commands
#
# 

# general subroutines: whats this links, converting links in text etc.
#

# subroutines used by index page

# subroutines used in wizard-based service creation

# subfunction to create smf manifest from wizard datafiles.
#
# Parameters:

# subroutines used in properties page

#
# subroutines used by dep_viewer page
#

1;
		return $self;
}

# Run all smf commands (separated by ";", check if expected retvals match
# (if needed) and return response(s)
#
# Paramters: cmdlist, [expected_retval]
#
sub run_smf_cmds()
{
	my $self = shift;

local ($cmdlist, $expected_retval, @cmds, $errmsg, $response, $cmd,
	$newresponse, $retval);
($cmdlist, $expected_retval) = @_;
@cmds = split(/;/, "$cmdlist");
$errmsg = "";
$response = "";
foreach $cmd (@cmds) {
	$newresponse = $self->backquote_logged("$cmd");
	$retval = $?;
	if ($response =~ /.+/) {
		# append to responses so far...
		$response = "$response ; $newresponse";
	} else {
		$response = "$newresponse";
		}
	# check retval
	if (($expected_retval =~ /.+/) &&
	    ($expected_retval != $retval)) {
		$errmsg =
"${errmsg}Unexpected return value $retval from $cmd: $newresponse .";
		}
	if ($self->{'cmds_run'} =~ /.+/) {
		$self->{'cmds_run'} = "$self->{'cmds_run'} ; $cmd";
	} else {
		$self->{'cmds_run'} = "$cmd";
		}
	}
if ($errmsg =~ /.+/) {
	$self->error("$errmsg");
	}
return $response;
}

# Import supplied manifest into repository, expect success
#
# Paramters: manifest
#
sub svc_import()
{
	my $self = shift;

$self->{'manifest'} = $_[0];
$self->run_smf_cmds("/usr/sbin/svccfg import $self->{'manifest'}", 0);
}

# get dependencies/dependent listing for fmri
#
# Parameters: dependency_or_dependent, fmri
#
sub svc_dep_cmd()
{
	my $self = shift;

local ($dependency_or_dependent, $fmri, $cmd);
($dependency_or_dependent, $fmri) = @_;

if ($dependency_or_dependent eq "dependent") {
	$cmd = "/usr/bin/svcs -H -oFMRI,STATE -D $fmri";
} else {
	$cmd = "/usr/bin/svcs -H -oFMRI,STATE -d $fmri";
	}
return ($self->run_smf_cmds("$cmd"));
}

# get grouping for child dependency on parent fmri using svcs -l
#
# Parameters: parent_fmri child_fmri
#
sub svc_grouping_cmd
{
	my $self = shift;

local ($parent_fmri, $child_fmri, $output, @output_lines, $line, $grouping);
($parent_fmri, $child_fmri) = @_;
$output = $self->run_smf_cmds("/usr/bin/svcs -l $parent_fmri");
@output_lines = split(/\n/, $output);
foreach $line (@output_lines) {
	chomp($line);
	if ($line =~ /dependency\s+([^\/]+)\/\S+\s+$child_fmri/) {
		# found dependency at instance level
		$grouping = $1;
		return $grouping;
		}
	}
# if no match at instance level, we try service level...
if ($child_fmri =~ /(svc:\/[^:]*):.*/) {
	$self->{'child_fmri_svc'} = $1;
	return ($self->svc_grouping_cmd($parent_fmri, $self->{'child_fmri_svc'}));
} else {	
	return "";
	}
}

# apply svc command to elts in fmri list
#
# Parameters: cmd, fmri
sub svc_state_cmd()
{
	my $self = shift;

local ($cmd_name, $fmrilist_ref, $cmd, $cmdlist, $fmri);
($cmd_name, $fmrilist_ref) = @_;
if ($cmd_name eq "$self->{'text_hash'}->{'state_enable'}") {
	$cmd = "/usr/sbin/svcadm enable";
} elsif ($cmd_name eq "$self->{'text_hash'}->{'state_disable'}") {
	$cmd = "/usr/sbin/svcadm disable";
} elsif ($cmd_name eq "$self->{'text_hash'}->{'state_refresh'}") {
	$cmd = "/usr/sbin/svcadm refresh";
} elsif ($cmd_name eq "$self->{'text_hash'}->{'state_restart'}") {
	$cmd = "/usr/sbin/svcadm restart";
} elsif ($cmd_name eq "$self->{'text_hash'}->{'state_maintenance'}") {
	$cmd = "/usr/sbin/svcadm mark maintenance";
} elsif ($cmd_name eq "$self->{'text_hash'}->{'state_degraded'}") {
	$cmd = "/usr/sbin/svcadm mark degraded";
} elsif ($cmd_name eq "$self->{'text_hash'}->{'state_clear'}") {
	$cmd = "/usr/sbin/svcadm clear";
} elsif ($cmd_name eq "$self->{'text_hash'}->{'index_delete'}") {
	$cmd = "/usr/sbin/svccfg delete -f";
} else {
	$self->error("Unknown command $cmd_name!");
	}
$cmdlist = "";
foreach $fmri (@$fmrilist_ref) {
	$cmdlist = "${cmdlist}$cmd $fmri;";
	}
return ($self->run_smf_cmds("$cmdlist", 0));
}

# get state for instance
#
# Parameters: fmri
#
sub svc_get_state_cmd()
{
	my $self = shift;

local ($fmri, $state);
$fmri = $_[0];
$state = $self->run_smf_cmds("/usr/bin/svcs -H -oSTATE $fmri");
chomp($state);
return $state;
}

# Add property group
#
# Parameters: fmri, service_or_instance, name, type
#
sub svc_addpg()
{
	my $self = shift;

local($fmri, $sinst, $name, $type, $entity);
($fmri, $sinst, $name, $type) = @_;
$entity = ($sinst eq "service") ? "$fmri" : "$fmri:$sinst";
$self->run_smf_cmds("/usr/bin/echo \"select $entity\naddpg $name $type\nquit\n\"  | /usr/sbin/svccfg", 0);
}

# Delete property group
#
# Parameters: fmri, service_or_instance, name
#
sub svc_delpg()
{
	my $self = shift;

local($fmri, $sinst, $name, $entity);
($fmri, $sinst, $name) = @_;
$entity = ($sinst eq "service") ? "$fmri" : "$fmri:$sinst";
$self->run_smf_cmds("/usr/bin/echo \"select $entity\ndelpg $name\nquit\n\"  | /usr/sbin/svccfg",
	0);
}

# list property groups. we also determine if property group is at service
# or instance level and include this in listing.
#
# Parameters: fmri, service_or_instance
#
sub svc_listpg()
{
	my $self = shift;

local ($fmri, $sinst, $entity, $pgroups, @pgroup_list, $i, $pgroup_info,
	$name, $type, @listing);
($fmri, $sinst) = @_;
$entity = ($sinst eq "service") ? "$fmri" : "$fmri:$sinst";
$pgroups = $self->run_smf_cmds("/usr/bin/echo \"select $entity\nlistpg\nquit\n\"  | /usr/sbin/svccfg", 0);
@pgroup_list = split(/\n/, $pgroups);
$i = 0;
foreach $pgroup_info (@pgroup_list) {
	$pgroup_info =~ /([^\s]*)\s+(.*)/;
	$name = $1;
	$type = $2;
	$listing[$i]{'pgroup_name'} = $1;
	$listing[$i]{'pgroup_type'} = $2;
	$listing[$i]{'sinst'} = $sinst;
	$i = $i + 1;
	}
return @listing;
}

# Set property (creates if doesnt yet exist)
#
# Parameters: fmri, service_or_instance, pgname, name, type, value
#
sub svc_setprop()
{
	my $self = shift;

local($fmri, $sinst, $pgname, $pname, $type, $value, $entity);
($fmri, $sinst, $pgname, $pname, $type, $value) = @_;
$entity = ($sinst eq "service") ? "$fmri" : "$fmri:$sinst";
$self->run_smf_cmds("/usr/bin/echo \"select $entity\nsetprop $pgname/$pname=$type:\\\"$value\\\"\nquit\n\" | /usr/sbin/svccfg", 0);
}

# Delete property 
#
# Parameters: fmri, service_or_instance, pgroup_name, name
#
sub svc_delprop()
{
	my $self = shift;

local($fmri, $sinst, $pgname, $name, $entity);
($fmri, $sinst, $pgname, $name) = @_;
$entity = ($sinst eq "service") ? "$fmri" : "$fmri:$sinst";
$self->run_smf_cmds("/usr/bin/echo \"select $entity\ndelprop $pgname/$name\nquit\n\" | /usr/sbin/svccfg", 0);
}

# list properties
#
# Parameters: fmri, service_or_instance, pgroup_name
#
sub svc_listprop()
{
	my $self = shift;

local ($fmri, $sinst, $entity, $pgroup_name, $props, @prop_list, $i,
	$prop_info, $name, $type, $value, @listing);
($fmri, $sinst, $pgroup_name) = @_;
$entity = ($sinst eq "service") ? "$fmri" : "$fmri:$sinst";
$props = $self->run_smf_cmds("/usr/bin/echo \"select $entity\nlistprop $pgroup_name/\*\nquit\n\"  | /usr/sbin/svccfg", 0);
@prop_list = split(/\n/, $props);
$i = 0;
foreach $prop_info (@prop_list) {
	$prop_info =~ /[^\/]*\/([^\s]+)\s+([^\s]*)\s+(.*)/;
	$listing[$i]{'prop_name'} = $1;
	$listing[$i]{'prop_type'} = $2;
	$listing[$i]{'prop_value'} = $3;
	$i = $i + 1;
	}
return @listing;
}

# wrapper for printing text and associated whats this link if required
#
# Parameters: text
#
sub text_and_whats_this()
{
	my $self = shift;

local ($textname);
$textname = $_[0];
if ($self->{'config_hash'}->{'enable_whats_this'} == 1) {
	print "$self->{'text_hash'}->{$textname}&nbsp;", $self->hlink($self->{'text_hash'}->{'whats_this'}, $textname);
	}
else {
	print "$self->{'text_hash'}->{$textname}";
	}
}

# Show what`s this? link, which opens new page with tip info. We pass in
# pagename and tipname, and the appropriate localized string, representing
# a concatenation of the two, is displayed.
#
# Parameters: tiparea, tipname
#
sub print_whats_this_link()
{
	my $self = shift;

local($tipname);
$tipname = $_[0];
if ($self->{'config_hash'}->{'enable_whats_this'} == 1) {
	print "<a href=\"whats_this.cgi?tipname=$tipname\" target=\"new\">$self->{'text_hash'}->{'whats_this'}</a>";
	}
}

# convert manpage references to links, and http references to actual links
#
# Parameters: text
#
sub convert_links_in_text()
{
	my $self = shift;

local($text);
$text = $_[0];
# convert http links,fmris,manpages and logfiles to links...
$text =~ s/See:\s+([^\(]*)\(([^\)]*)\)/See: <a href=\"\/man\/view_man.cgi?page=$1&sec=$2\">$1\($2\)<\/a>/g;
$text =~ s/http:\/\/(\S+)/<a href=\"http:\/\/$1\">http:\/\/$1<\/a>/g;
$text =~ s/(svc:\/\S+)/<a href=\"instance_viewer.cgi?fmri='$1'\">$1<\/a>/g;
$text =~
  s/(\/var\/svc\/log\/\S+)/<a href=\"log_viewer.cgi?logfile='$1'\">$1<\/a>/g;
$text =~
s/(\/etc\/svc\/volatile\/\S+)/<a href=\"log_viewer.cgi?logfile='$1'\">$1<\/a>/g;
return $text;
}

# show selection
#
# Parameters: name, selected, selection_arrayref
#
sub print_selection()
{
	my $self = shift;

local($name, $selected, $selection_arrayref, @array, $elt, $select);
($name, $selected, $selection_arrayref) = @_;
print "<select name=\"$name\" size=1>\n";
@array = @$selection_arrayref;
foreach $elt (@array) {
        $select = "";
        if ($elt eq $selected) {
                $select = "selected";
                }
        print "<option $select>$elt</option>\n";
        }
print "</select>\n";
}

# shows commands run in generating page...
#
# Parameters:
#
sub print_cmds_run()
{
	my $self = shift;

local ($cmds_run_clipped);
$cmds_run_clipped = substr($self->{'cmds_run'}, 0, 240);
print "<hr>\n";
print "<p>$self->{'text_hash'}->{'cmds_run'}:<\p>\n";
print "<p>${cmds_run_clipped}...</p>\n";
}

# show buttons to enable/disable etc
#
# Parameters:
#
sub print_state_buttons()
{
	my $self = shift;

print "<input type=submit name=\"change_state\" value=\"$self->{'text_hash'}->{'state_enable'}\">&nbsp;\n";
print "<input type=submit name=\"change_state\" value=\"$self->{'text_hash'}->{'state_disable'}\">&nbsp;\n";
print "<input type=submit name=\"change_state\" value=\"$self->{'text_hash'}->{'state_refresh'}\">&nbsp;\n";
print "<input type=submit name=\"change_state\" value=\"$self->{'text_hash'}->{'state_restart'}\">&nbsp;\n";
print "<input type=submit name=\"change_state\" value=\"$self->{'text_hash'}->{'state_maintenance'}\">&nbsp;\n";
print "<input type=submit name=\"change_state\" value=\"$self->{'text_hash'}->{'state_degraded'}\">&nbsp;\n";
print "<input type=submit name=\"change_state\" value=\"$self->{'text_hash'}->{'state_clear'}\">&nbsp;\n";
}

# show svc chooser button
#
# Parameters: prop_name, form_index, button_value, type, add
#
sub print_svc_chooser()
{
	my $self = shift;

($self->{'prop_name'}, $self->{'form_index'}, $self->{'button_value'}, $self->{'type'}, $self->{'add'}) = @_;
print "<input type=button onClick='ifield = document.forms[$self->{'form_index'}].$self->{'prop_name'}; chooser= window.open(\"svc_chooser.cgi?type=$self->{'type'}&add=$self->{'add'}\", \"chooser\", \"toolbar=no,menubar=no,scrollbar=no,width=500,height=400\"); chooser.ifield = ifield; window.ifield = ifield' value=\"$self->{'button_value'}\">\n";
}

# show path chooser button
#
# Parameters: prop_name, form_index, button_value, add
#
sub print_path_chooser()
{
	my $self = shift;

($self->{'prop_name'}, $self->{'form_index'}, $self->{'button_value'}, $self->{'add'}) = @_;
print "<input type=button onClick='ifield = document.forms[$self->{'form_index'}].$self->{'prop_name'}; chooser= window.open(\"path_chooser.cgi?add=$self->{'add'}\", \"chooser\", \"toolbar=no,menubar=no,scrollbar=no,width=500,height=400\"); chooser.ifield = ifield; window.ifield = ifield' value=\"$self->{'button_value'}\">\n";
}

# gather required information from svcs, storing as
# array of hashes indexed by property...
# 
# Parameters: filter, comma_separated_optlist
#
sub svcs_listing()
{
	my $self = shift;

local ($filter, $opts, $opt, @optlist, @sinfo, $sinf, @sinf_array, $i);
($filter, $self->{'sortopt'}) = @_;
$filter = "$filter\*";
$self->{'allopts'} = "FMRI,STATE,NSTATE,STIME,DESC";
$self->{'sinfo'} = $self->run_smf_cmds("/usr/bin/svcs -H -o $self->{'allopts'} $self->{'sortopt'} $filter");
@{$self->{'sinfo_list_array'}} = split(/\n/, $self->{'sinfo'});
foreach $sinf (@{$self->{'sinfo_list_array'}}) {
	chomp($sinf);
	if ($sinf =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)/) {
		$self->{'listing_array'}->[$i]{'FMRI'} = $1;
		$self->{'listing_array'}->[$i]{'STATE'} = $2;
		$self->{'listing_array'}->[$i]{'NSTATE'} = $3;
		$self->{'listing_array'}->[$i]{'STIME'} = $4;
		$self->{'listing_array'}->[$i]{'DESC'} = $5;
		$i = $i + 1;
		}
	}
return @{$self->{'listing_array'}};
}

sub create_smf_manifest()
{
	my $self = shift;

local (%service_info, $manifest, @find_replace_array, @datafiles, $i);
$manifest = $_[0];
unlink($manifest);
# add service information to manifest
%service_info = $self->wizard_get_data("smfwizard_service.cgi", "0");
@find_replace_array = ("SERVICE_NAME/$service_info{'service_name'}",
	"SERVICE_VERSION/$service_info{'service_version'}");
$self->fill_manifest_template("service_template.xml", $manifest,
	\@find_replace_array);
# restarter
%service_info = $self->wizard_get_data("smfwizard_restarter.cgi", "0");
if ($service_info{'restarter_fmri'} =~ /.+/) {
	@find_replace_array =
  ("RESTARTER_FMRI/<service_fmri value=\'$service_info{'restarter_fmri'}\' />");
	$self->fill_manifest_template("restarter_template.xml", $manifest,
		\@find_replace_array);
	}
$self->add_service_or_instance_info($manifest, "service");
@datafiles = $self->list_wizard_datafiles("smfwizard_instance.cgi");
for ($i = 0; $i < @datafiles; $i++) {
	%service_info = $self->wizard_get_data("smfwizard_instance.cgi", $i);
	@find_replace_array = ("INSTANCE_NAME/$service_info{'instance_name'}",
		"INSTANCE_ENABLED/$service_info{'instance_enabled'}");
	$self->fill_manifest_template("instance_template.xml", $manifest,
		\@find_replace_array);
	$self->add_service_or_instance_info($manifest,
		"$service_info{'instance_name'}");
	@find_replace_array = ();
	$self->fill_manifest_template("instance_template_end.xml", $manifest,
		\@find_replace_array);
	}
# template info
%service_info = $self->wizard_get_data("smfwizard_template.cgi", "0");
@find_replace_array =
	("TEMPLATE_COMMON_NAME/$service_info{'template_common_name'}",
	"TEMPLATE_DESCRIPTION/$service_info{'template_description'}");
if (($service_info{'template_common_name'} =~ /.+/) &&
    ($service_info{'template_description'} =~ /.+/)) {
	$self->fill_manifest_template("template_template.xml", $manifest,
		\@find_replace_array);
	@datafiles = $self->list_wizard_datafiles("smfwizard_manpage.cgi");
	for ($i = 0; $i < @datafiles; $i++) {
		%service_info = $self->wizard_get_data("smfwizard_manpage.cgi", $i);
		@find_replace_array =
		("MANPAGE_TITLE/$service_info{'manpage_title'}",
		"MANPAGE_SECTION/$service_info{'manpage_section'}",
		"MANPAGE_MANPATH/$service_info{'manpage_manpath'}");
		$self->fill_manifest_template("manpage_template.xml", $manifest,
			\@find_replace_array);
		}
	@find_replace_array = ();
	$self->fill_manifest_template("template_template_end.xml", $manifest,
		\@find_replace_array);
	}
@find_replace_array = ();
$self->fill_manifest_template("service_template_end.xml", $manifest,
	\@find_replace_array);
}

# subfunction to add dependecy/dependent/exec method/propgroup/prop
# information to manifest. if at service level, we look for sinst=service,
# otherwise, look for matches to instance name
#
sub add_service_or_instance_info()
{
	my $self = shift;

local ($manifest, $sinst, @datafiles, $i, %service_info, $stability,
	@find_replace_array, $stability, $cred, $pgroup_name, $pgroup_type,
	$j, @propfiles, %prop_info);
($manifest, $sinst) = @_;
@datafiles = $self->list_wizard_datafiles("smfwizard_dependency.cgi");
for ($i = 0; $i < @datafiles; $i++) {
	%service_info = $self->wizard_get_data("smfwizard_dependency.cgi", $i);
	# does sinst match?
	if ("$service_info{'sinst'}" eq "$sinst") {
		# add dep'y
		if ($service_info{'dependency_stability'} eq "-") {
			$stability = "";
		} else {
			$stability =
		"<stability value='$service_info{'dependency_stability'}'/>";
			}
		@find_replace_array =
		    ("DEP_NAME/$service_info{'dependency_name'}",
		     "DEP_TYPE/$service_info{'dependency_type'}",
		     "DEP_GROUPING/$service_info{'dependency_grouping'}",
		     "DEP_RESTART_ON/$service_info{'dependency_restart_on'}",
	"DEP_FMRI/<service_fmri value=\'$service_info{'dependency_fmri'}\' />",
		     "DEP_STABILITY/$stability");
		$self->fill_manifest_template("dependency_template.xml", $manifest,
			\@find_replace_array);
		}
	}
@datafiles = $self->list_wizard_datafiles("smfwizard_dependent.cgi");
for ($i = 0; $i < @datafiles; $i++) {
	%service_info = $self->wizard_get_data("smfwizard_dependent.cgi", $i);
	# does sinst match?
	if ("$service_info{'sinst'}" eq "$sinst") {
		# add dep't
		if ($service_info{'dependent_stability'} eq "-") {
			$stability = "";
		} else {
			$stability =
		"<stability value='$service_info{'dependent_stability'}'/>";
			}
		@find_replace_array =
		    ("DEP_NAME/$service_info{'dependent_name'}",
		     "DEP_TYPE/$service_info{'dependent_type'}",
		     "DEP_GROUPING/$service_info{'dependent_grouping'}",
		     "DEP_RESTART_ON/$service_info{'dependent_restart_on'}",
	"DEP_FMRI/<service_fmri value=\'$service_info{'dependent_fmri'}\' />",
			"DEP_STABILITY/$stability");
		$self->fill_manifest_template("dependent_template.xml", $manifest,
			\@find_replace_array);
		}
	}
@datafiles = $self->list_wizard_datafiles("smfwizard_exec.cgi");
for ($i = 0; $i < @datafiles; $i++) {
	%service_info = $self->wizard_get_data("smfwizard_exec.cgi", $i);
	# does sinst match?
	if ("$service_info{'sinst'}" eq "$sinst") {
		# add exec method
		# method context
		if (("$service_info{'exec_user'}" =~ /.+/) ||
		    ("$service_info{'exec_group'}" =~ /.+/) ||
		    ("$service_info{'exec_privileges'}" =~ /.+/)) {
			$cred = "<method_context><method_credential ";
			if ("$service_info{'exec_user'}" =~ /.+/) {
				$cred =
				   "$cred user=\'$service_info{'exec_user'}\' ";
				}
			if ("$service_info{'exec_group'}" =~ /.+/) {
				$cred =
				 "$cred group=\'$service_info{'exec_group'}\' ";
				}
			if ("$service_info{'exec_privileges'}" =~ /.+/) {
				$cred =
			"$cred privileges=\'$service_info{'exec_privileges'}\' ";
				}
			$cred = "$cred /> </method_context>";
		} else {
			$cred = "";
			}
		@find_replace_array =
		    ("EXEC_NAME/$service_info{'exec_name'}",
		     "EXEC_TIMEOUT_SECONDS/$service_info{'exec_timeout'}",
		     "EXEC_EXEC/$service_info{'exec_exec'}",
		     "EXEC_METHOD_CREDENTIAL/$cred");
		$self->fill_manifest_template("exec_template.xml", $manifest,
			\@find_replace_array);
		}
	}
# for property group/properties, we cycle through property groups, then
# for each pgroup we see if each property is a member of the group AND
# applies at the appropriate service/instance level

@datafiles = $self->list_wizard_datafiles("smfwizard_property_group.cgi");
for ($i = 0; $i < @datafiles; $i++) {
	%service_info = $self->wizard_get_data("smfwizard_property_group.cgi", $i);
	$pgroup_name = "$service_info{'property_group_name'}";
	$pgroup_type = "$service_info{'property_group_type'}";
	if ($service_info{'property_group_stability'} eq "-") {
		$stability = "";
	} else {
		$stability =
	"<stability value='$service_info{'property_group_stability'}'/>";
		}
	@find_replace_array =
		("PGROUP_NAME/$pgroup_name", "PGROUP_TYPE/$pgroup_type",
		 "PGROUP_STABILITY/$stability");
	$self->fill_manifest_template("property_group_template.xml",
		$manifest, \@find_replace_array);
	@propfiles = $self->list_wizard_datafiles("smfwizard_property.cgi");
	for ($j = 0; $j < @propfiles; $j++) {
		# does sinst/pgroup match??
		%prop_info = $self->wizard_get_data("smfwizard_property.cgi", $j);
		if (("$sinst" eq "$prop_info{'sinst'}") &&
		    ("$pgroup_name" eq "$prop_info{'pgroup'}")) {
			# matched sinst/pgroup, display prop...
			@find_replace_array =
			    ("PROP_NAME/$prop_info{'property_name'}",
			     "PROP_TYPE/$prop_info{'property_type'}",
			     "PROP_VALUE/$prop_info{'property_value'}");
			$self->fill_manifest_template("property_template.xml",
				 $manifest, \@find_replace_array);
			}
		}
	@find_replace_array = ();
	$self->fill_manifest_template("property_group_template_end.xml", $manifest,
		\@find_replace_array);
	}
}

# subfunction to read in manifest template (for exec method, dependency
# etc), and replace keywords with desired values. the keywords/desired
# values are passed in as an array, each elt is a find/replace string.
# template values may be replace multiple times (e.g. service_fmris
# for deps
#
# Parameters: template_file target_file find_replace_arrayref
#
sub fill_manifest_template()
{
	my $self = shift;

local ($template, $target, $find_replace_arrayref, @template_data,
	$find_replace_expr, @tdata, $find, $replace, $linenum, $line,
	@replace_data, $i);
($template, $target, $find_replace_arrayref) = @_;
open(TEMPLATE, "$self->{'module_root_directory'}/$template");
@template_data = <TEMPLATE>;
close(TEMPLATE);
foreach $find_replace_expr (@$find_replace_arrayref) {
	@tdata = @template_data;
	# split find/replace into components...
	$find_replace_expr =~ /([^\/]*)\/(.*)/;
	$find = $1;
	$replace = $2;
	$linenum = 0;
	foreach $line (@tdata) {
		if ($line =~ s/$find/$replace/) {
			$replace_data[$linenum] =
				"$replace_data[$linenum]$line\n";
			}
		$linenum++;
		}
	}
# now append to target file
if (-e $target) {
	open(TARGET, ">>$target");
} else {
	open(TARGET, ">$target");
	}
for ($i = 0; $i < @template_data; $i++) {
	# if we have a "replace" line, write it, otherwise write original
	# template line
	if ($replace_data[$i] =~ /.+/) {
		print TARGET "$replace_data[$i]";
	} else {
		print TARGET "$template_data[$i]";
		}
	}
close(TARGET);
}

# Builds dependency/dependent tree starting from fmri.
# Uses hash with fmris as keys to contain tree info. This way,
# we don't expand a particular branch multiple times when we recurse...
#
# Parameters: dependency_or_dependent, tree_ref, fmri, level, expand_listref
#
sub build_dep_tree
{
	my $self = shift;

local ($dependency_or_dependent, $tree_ref, $fmri, $level,
	$expand_listref, $state, $grouping, $allow_expand, $expand_fmri, $deps,
	@dep_info, $dep_inf, $dep_fmri, $dep_state, $parent, $child,
	$dep_grouping);
($dependency_or_dependent, $tree_ref, $fmri, $level, $expand_listref,
$state, $grouping) = @_;
$allow_expand = 0;
# if fmri is on expand list passed in, we allow expansion.
foreach $expand_fmri (@$expand_listref) {
	if ($expand_fmri eq $fmri) {
		$allow_expand = 1;
		break;
		}
	}
if ($tree_ref->{"$fmri"}->{'exists'} != 1) {
	# new hash element
	$tree_ref->{"$fmri"}->{'exists'} = 1;
	# state information, if passed in...
	if ($state =~ /.+/) {
		$tree_ref->{"$fmri"}->{'state'} = $state;
		}
	if ($grouping =~ /.+/) {
		$tree_ref->{"$fmri"}->{'grouping'} = $grouping;
		}
	# do we expand tree here (i.e. recurse?)
	$tree_ref->{"$fmri"}->{'expand'} = $allow_expand;
	@{$tree_ref->{"$fmri"}->{'children'}} = ();
	$tree_ref->{"$fmri"}->{'haschildren'} = 0;
	$deps = $self->svc_dep_cmd($dependency_or_dependent, $fmri);
	@dep_info = split(/\n/, $deps);
	foreach $dep_inf (@dep_info) {
		# dependency info consists of fmri and state
		$dep_inf =~ /(\S+)\s+(\S+)/;
		$dep_fmri = $1;
		$dep_state = $2;
		# we need to find out what grouping is (using svcs -l).
		# for dep'y we call svcs -l $fmri, looking for line
		# specifying $dep_fmri. for dep't we call svcs -l
		# $dep_fmri, looking for line specifying $fmri.
		if ($dependency_or_dependent eq "dependency") {
			$parent = $fmri;
			$child = $dep_fmri;
		} else {
			$parent = $dep_fmri;
			$child = $fmri;
			}
		$dep_grouping = $self->svc_grouping_cmd($parent, $child);
		# we need to determine if dep has children, in order
		# to disable the expand button if not. however, we
		# dont recurse if it does unless it was on expand list
		$tree_ref->{"$fmri"}->{'haschildren'} = 1;
		chomp($dep_fmri);
		push(@{$tree_ref->{"$fmri"}->{'children'}},
			$dep_fmri);
		if ($allow_expand == 1) {
			$self->build_dep_tree($dependency_or_dependent, $tree_ref,
				$dep_fmri, $level + 1, $expand_listref,
				$dep_state, $dep_grouping);
			}
		}	
	}
}

# Displays dependency/dependent tree using depth-first recursion:
# get fmris children, the print each childs children etc.
# However, if a fmri has expand == 0, we don`t recurse for it.
# For expanded fmris, we show the contract (-) button, and for
# unexpanded, we show the expand button (if they have children).
#
# Parameters: dependency_or_dependent, tree_ref, fmri, level
#
sub print_dep_tree
{
	my $self = shift;

local ($dependency_or_dependent, $tree_ref, $fmri, $level, $fmriinfo,
	$indent, $child_fmri, $tab, $i, $color);
($dependency_or_dependent, $tree_ref, $fmri, $level) = @_;
$fmriinfo = $fmri;
# indent according to level of tree
$indent = 8*$level;
$tab = "";
for ($i = 0; $i < $indent; $i++) {
	$tab="$tab&nbsp;";
	}
print "<p>$tab";
if ($tree_ref->{"$fmri"}->{'expand'} == 1) {
	$fmriinfo = $self->contractbutton($dependency_or_dependent, $tree_ref,
		$fmri);
} else {
	$fmriinfo = $self->expandbutton($dependency_or_dependent, $tree_ref,
		$fmri);
	}
# show expand/contract button...
if ($level > 0) {
	print "$fmriinfo";
	}
# apply-to checkbox
print "&nbsp;<input type=checkbox name=\"applyto\" value=\"$fmri\">\n";
# fmri, link to state editor page for fmri
print "&nbsp;<a href=\"dep_viewer.cgi?fmri='$fmri'\">$fmri</a>";
if ($tree_ref->{"$fmri"}->{'state'} =~ /.+/) {
	$color = "$self->{'state_colors_hash'}->{$tree_ref->{$fmri}->{'state'}}";
	print
	  "&nbsp;<font color=$color>$tree_ref->{$fmri}->{'state'}</font>";
	}
if ($tree_ref->{"$fmri"}->{'grouping'} =~ /.+/) {
	print "&nbsp;($tree_ref->{$fmri}->{'grouping'})";
	}
print "</p>\n";
if ($tree_ref->{"$fmri"}->{'expand'} == 1) {
	# we can expand
	foreach $child_fmri (@{$tree_ref->{"$fmri"}->{'children'}}) {
		$self->print_dep_tree($dependency_or_dependent, $tree_ref,
			$child_fmri, $level + 1);
		}
	}
}

sub contractbutton()
{
	my $self = shift;

local ($dependency_or_dependent, $tree_ref, $fmri, $disable);
($dependency_or_dependent, $tree_ref, $fmri) = @_;
$disable = "";
# for elements with no children, disable button...
if ($tree_ref->{"$fmri"}->{'haschildren'} == 0) {
	$disable = "disabled=\"disabled\"";
	}
return "<input type=submit name=\"${dependency_or_dependent}_contract_$fmri\" value=\"-\" $disable><input type=\"hidden\" name=\"${dependency_or_dependent}_expandlist\" value=\"$fmri\">";
}

sub expandbutton()
{
	my $self = shift;

local ($dependency_or_dependent, $tree_ref, $fmri, $disable);
($dependency_or_dependent, $tree_ref, $fmri) = @_;
$disable = "";
if ($tree_ref->{"$fmri"}->{'haschildren'} == 0) {
	$disable = "disabled=\"disabled\"";
	}
return "<input type=submit name=\"${dependency_or_dependent}_expand_$fmri\" value=\"+\" $disable>";
}


1;

