#!/usr/bin/perl
package smf::Wizard;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'smf::Wizard');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
#
# Code for a generic cgi wizard. The wizard save
# for each stage in the process, provides validation of fields via 
# regular expressions, and manages button presses etc. It allows pages
# to be used to enter multiple sets of values, all of which are stored.
#
#	1) create a wizard array. Each array element specifies
# 	a page in the wizard. Each element should be a comma-separated
#	string, containing the following values:
#		- "page": the .cgi file for the page
#		- "min": the minimum valid number of datafiles
#		required for this page
#		- "max": the maximum valid number of datafiles
#		required for this page.
#	An example of a valid entry would be "page=mywizpage.cgi,min=1,max=1".
#
#	The wizard allows pages to be used to enter multiple sets of
#	properties. Each wizard page has an associated data file, and
#	where multiple datafiles are valid (i.e. max_datafiles>1),
#	they are numbered monotonically.
#	2) Create a .cgi file for each wizard page you specified in the
#	wizard array. Each wizard page will call wizard_header(),
#	wizard_[inputtype]() for each data field and wizard_footer().
#	3) Create a submit .cgi script that processes the set of datafiles
#	created.
#
# Appearance:
#
# Wizard consists of a main table holding 3 subtables
#       - the description/image table, taking up the LHS
#       - the data table, where fields appear, on the RHS
#       - the button table on the bottom
# The data table is opened, and rows are added by calls
# to wizard_input(). wizard_footer() adds the button table.

# This variable is set to 0 whenever an element in the form fails to
# validate against its regular expression. This prevents progress, and the
# highlighted elements are marked in red.
$self->{'validation_success'} = 1;

# subfns used for wizard form inputs...

1;
		return $self;
}

# Creates tables/subtables, show image etc.
#
# Parameters: ref_to_array_of_wizard_element_hashes, pagename, heading,
# description, imagename
#
sub wizard_header()
{
	my $self = shift;

local ($wizinfo, %wizard, $wizref, $pagename, $heading, $description, $image);
($wizinfo, $pagename, $heading, $description, $image) = @_;
%wizard = $self->read_in_wizard("$wizinfo");
$wizref = \%wizard;
$self->{'pageinst'} = $self->wizard_get_pageinst();
$self->{'submit'} = $self->get_wizard_info_by_name($wizref, "submit");
print "<form method=\"POST\" action=\"$self->{'submit'}\">\n";
print "<table border cellpadding=0 cellspacing=0 width=100%>\n";
print "<tr $self->{'cb'}><td>\n";
print "<table noborder cellpadding=0 cellspacing=0 width=100% height=15%>\n";
print "<tr $self->{'tb'}><td>$heading</td></tr></table></td></tr>\n";
print "<tr $self->{'cb'}><td>\n";
print "<table border cellpadding=0 cellspacing=0 width=100% height=70%>\n";

# LHS consists description + optional image.
print "<tr $self->{'cb'}><td>\n";
print "<table noborder cellpadding=0 cellspacing=0>\n";
if ($image =~ /.+/) {
	print "<tr $self->{'cb'}><td><img src=$image alt=\"\"></td>\n";
	}
print "<td><p>$description</p></td></tr></table></td>\n";

# RHS contains information supplied by calls to wizard_input(), i.e.
# it`s where data is entered. We open this table and close it with
# wizard_footer() below...

print "<td><table noborder cellpadding=0 cellspacing=0>\n";
}

sub wizard_footer()
{
	my $self = shift;

local ($wizinfo, $pagename, %wizard, $wizref, $min_datafiles, $max_datafiles,
	$pageinst, $pagenum, $datafile, $back_disabled, $next_disable,
	$add_disabled, $remove_disabled, $finish_disabled, @wizard_pages,
	$page, @datafiles);
($wizinfo, $pagename) = @_;
%wizard = $self->read_in_wizard("$wizinfo");
$wizref = \%wizard;
$pageinst = $self->wizard_get_pageinst();

$min_datafiles = $self->get_wizard_info_by_name($wizref, "min_datafiles", $pagename);
$max_datafiles = $self->get_wizard_info_by_name($wizref, "max_datafiles", $pagename);
$pagenum = $self->get_wizard_info_by_name($wizref, "pagenum", $pagename);
$datafile = $self->wizard_datafile_name($pagename, $pageinst);

# which buttons do we show?

$back_disabled = "";
# first page, first instance, so no back button
if (($pagenum == 0) && ($pageinst == 0)) {
	$back_disabled="disabled=\"disabled\"";
	}

$self->{'next_disabled'} = "disabled=\"disabled\"";
# if we have met or exceeded minimum requirement of datafiles for
# this page, and are not on last page, or there are subsequent
# instances, enable next button
@datafiles = $self->list_wizard_datafiles($pagename);
@{$self->{'num_pages_array'}} = $self->get_wizard_info_by_name($wizref, "pages");
@{$self->{'num_pageinsts_array'}} = $self->list_wizard_datafiles($pagename);
if ((($pagenum < (@{$self->{'num_pages_array'}} - 1)) && (@datafiles >= $min_datafiles)) ||
	(($pageinst < (@{$self->{'num_pageinsts_array'}} - 1)))) {
	$self->{'next_disabled'} = "";
	}

$add_disabled = "";
$self->{'update_disabled'} = "disabled=\"disabled\"";
$self->{'new_disabled'} = "disabled=\"disabled\"";
$remove_disabled="disabled=\"disabled\"";
# if we have data already for page, dont disable remove/clear, disable add,
# enable update
@{$self->{'thispage_datafiles_array'}} = $self->list_wizard_datafiles("$pagename.$pageinst");
if (@{$self->{'thispage_datafiles_array'}} > 0) {
	$remove_disabled = "";
	$add_disabled = "disabled=\"disabled\"";
	$self->{'update_disabled'} = "";
	# enable new if we can have more instances....
	if ($pageinst < ($max_datafiles - 1)) {
		$self->{'new_disabled'} = "";
		}
	}

# can we show finish button? if each page has exceeded it`s minimum
# amount of datafiles, and each datafile has validation_success == 1, yes.
$finish_disabled = "";
@wizard_pages = $self->get_wizard_info_by_name($wizref, "pages");
foreach $page (@wizard_pages) {
	@datafiles = $self->list_wizard_datafiles("$page->{'pagename'}");
	foreach $datafile (@datafiles) {
		# get instance number...
		$datafile =~ /$page->{'pagename'}\.([0-9]*)/;
		$self->{'inst'} = $1;
		$self->{'validated'} = $self->wizard_get_data_value($page->{'pagename'},
			$self->{'inst'}, "validation_success");
		if ($self->{'validated'} != 1) {
			# instance not valid!
			$finish_disabled = "disabled=\"disabled\"";
			}
		}
	$self->{'page_min_datafiles'} = $self->get_wizard_info_by_name($wizref, "min_datafiles",
		"$page->{'pagename'}");
	if (@datafiles < $self->{'page_min_datafiles'}) {
		# haven't got minimum number of instances!
		$finish_disabled = "disabled=\"disabled\"";
		}
	}
# close data table
print "</table></td></tr></table></td></tr>\n";

# include hidden parameters here. These are:
# 	- pagename: current pagename
# 	- pageinst: current pageinst
print "<input type=\"hidden\" name=\"pagename\" value=\"$pagename\">";
print "<input type=\"hidden\" name=\"pageinst\" value=\"$pageinst\">";

# open button window
print "<tr $self->{'cb'}><td>\n";
print "<table noborder cellpadding=0 cellspacing=0 width=100% height=15%>";
print "<tr $self->{'cb'}><td>\n";
print
  "<input type=\"submit\" name=\"submit\" value=\"$self->{'text_hash'}->{'wizard_back'}\" $back_disabled>\n";
print "</td><td>\n";
print
  "<input type=\"submit\" name=\"submit\" value=\"$self->{'text_hash'}->{'wizard_new'}\" $self->{'new_disabled'}>\n";
print "</td><td>\n";
print
  "<input type=\"submit\" name=\"submit\" value=\"$self->{'text_hash'}->{'wizard_add'}\" $add_disabled>\n";
print "</td><td>\n";
print
  "<input type=\"submit\" name=\"submit\" value=\"$self->{'text_hash'}->{'wizard_update'}\" $self->{'update_disabled'}>\n";
print "</td><td>\n";
print
   "<input type=\"submit\" name=\"submit\" value=\"$self->{'text_hash'}->{'wizard_remove'}\" $remove_disabled>\n";
print "</td><td>\n";
print
  "<input type=\"submit\" name=\"submit\" value=\"$self->{'text_hash'}->{'wizard_next'}\" $self->{'next_disabled'}>\n";
print "</td><td>\n";
print
 "<input type=\"submit\" name=\"submit\" value=\"$self->{'text_hash'}->{'wizard_finish'}\" $finish_disabled>\n";
print "</td></tr></table></td></tr></table></form>\n";

# if we are validating page, ensure that we redirect!
if ($self->{'validation_success'} == 1) {
	# do redirect if next page/inst are defined
	$self->wizard_redirect_next($pagename, $pageinst);
	}
}

sub wizard_input()
{
	my $self = shift;

local ($pagename, $description, $name, $size, $validation_regexp,
	$validation_err, $pageinst, $value);
($pagename, $description, $name, $size, $validation_regexp, $validation_err)
	= @_;
$value = $self->wizard_form_input_header($pagename, $description, $name,
	$validation_regexp, $validation_err);
print "<input size=$size name=\"$name\" value=\"$value\">\n";
$self->wizard_form_input_footer();
}

sub wizard_textarea()
{
	my $self = shift;

local ($pagename, $description, $name, $rows, $cols, $validation_regexp,
	$validation_err, $pageinst, $value);
($pagename, $description, $name, $rows, $cols,  $validation_regexp,
	$validation_err) = @_;
$value = $self->wizard_form_input_header($pagename, $description, $name,
	$validation_regexp, $validation_err);
print "<textarea rows=$rows cols=$cols name=\"$name\" wrap=\"virtual\">\n";
print "$value";
print "</textarea>\n";
$self->wizard_form_input_footer();
}

sub wizard_select()
{
	my $self = shift;

local ($pagename, $description, $name, $selection_arrayref, $validation_regexp,
	$validation_err, $pageinst, $value, @array, $elt, $select);
($pagename, $description, $name, $selection_arrayref, $validation_regexp,
	$validation_err) = @_;
$value = $self->wizard_form_input_header($pagename, $description, $name,
	$validation_regexp, $validation_err);
print "<select name=\"$name\" size=1>\n";
@array = @$selection_arrayref;
foreach $elt (@array) {
	$select = "";
	if ("$elt" eq "$value") {
		$select = "selected";
		}
	print "<option $select>$elt</option>\n";
	}
print "</select>\n";
$self->wizard_form_input_footer();
}

sub wizard_form_input_header()
{
	my $self = shift;

local ($pagename, $description, $name, $validation_regexp,
	$validation_err, $pageinst, $value);
($pagename, $description, $name, $validation_regexp, $validation_err) = @_;
$pageinst = $self->wizard_get_pageinst();
$value = $self->wizard_get_data_value($pagename, $pageinst, $name);
$self->wizard_validate_input($value, $validation_regexp, $validation_err);
print "<tr $self->{'cb'}><td><p>$description</p></td></tr><tr $self->{'cb'}><td>\n";
return $value;
}

sub wizard_form_input_footer()
{
	my $self = shift;

print "</td></tr>\n";
}

# Deals with page submission
#
# Parameters: ref_to_array_of_wizard_element_hashes, pagename, heading,
# description, imagename
#
sub wizard_process_submit()
{
	my $self = shift;

local ($wizinfo, %wizard, $wizref, $pagename, $heading, $description, $image);
($wizinfo, $pagename, $heading, $description, $image) = @_;
%wizard = $self->read_in_wizard("$wizinfo");
$wizref = \%wizard;
$self->{'pageinst'} = $self->wizard_get_pageinst();
if (defined($self->{'in_hash'}->{'submit'})) {
	$self->{'button_name'} = $self->{'in_hash'}->{'submit'};
} else {
	$self->error("No button name passed to wizard_process_submit!");
	}
if ($self->{'button_name'} eq "$self->{'text_hash'}->{'wizard_finish'}") {
	# finish is the submit scripts problem!
	return;
	}
if (defined($self->{'in_hash'}->{'pagename'})) {
	$pagename = $self->{'in_hash'}->{'pagename'};
} else {
	$self->error("No page name passed to wizard_process_submit!");
	}
if (defined($self->{'in_hash'}->{'pageinst'})) {
	$self->{'pageinst'} = $self->{'in_hash'}->{'pageinst'};
} else {
	$self->error("No page instance passed to wizard_process_submit!");
	}
if ($self->{'button_name'} eq "$self->{'text_hash'}->{'wizard_back'}") {
	# find previous page
	$self->{'previous_pagename'} = $pagename;
	$self->{'previous_pageinst'} = $self->{'pageinst'};
	$self->{'pagenum'} = $self->get_wizard_info_by_name($wizref, "pagenum",
		$pagename);
	if (($self->{'pageinst'} == 0) && ($self->{'pagenum'} > 0)) {
		$self->{'previous_pagenum'} = $self->{'pagenum'} - 1;
		$self->{'previous_pagename'} = $self->get_wizard_info_by_page($wizref,
			"pagename", $self->{'previous_pagenum'});
		@{$self->{'num_pageinsts_array'}} = $self->list_wizard_datafiles($self->{'previous_pagename'});
		if (@{$self->{'num_pageinsts_array'}} > 0) {
			$self->{'previous_pageinst'} = @{$self->{'num_pageinsts_array'}} - 1;
			}
	} elsif ($self->{'pageinst'} > 0) {
		$self->{'previous_pageinst'} = $self->{'pageinst'} - 1;
		}
	# validation needed? does data already exist for page?
	$self->{'validate_option'} = "";
	$self->{'previous_datafile'} = "$self->{'previous_pagename'}.$self->{'previous_pageinst'}";
	@{$self->{'datafiles_array'}} =  $self->list_wizard_datafiles($self->{'previous_datafile'});
	if (@{$self->{'datafiles_array'}} > 0) {
		$self->{'validate_option'} = "&validate=1";
		}
	$self->redirect("$self->{'previous_pagename'}?pageinst=$self->{'previous_pageinst'}$self->{'validate_option'}");
	}
if ($self->{'button_name'} eq "$self->{'text_hash'}->{'wizard_next'}") {
	# find next page
	$self->{'next_pagename'} = $pagename;
	$self->{'next_pageinst'} = $self->{'pageinst'};
	@{$self->{'num_pageinsts_array'}} = $self->list_wizard_datafiles($pagename);
	$self->{'lastinst'} = 0;
	if (@{$self->{'num_pageinsts_array'}} > 0) {
		$self->{'lastinst'} = @{$self->{'num_pageinsts_array'}} - 1;
		}
	if ($self->{'pageinst'} >= $self->{'lastinst'}) {
		$self->{'pagenum'} = $self->get_wizard_info_by_name($wizref, "pagenum",
			$pagename);
		@{$self->{'num_pages_array'}} = $self->get_wizard_info_by_name($wizref, "pages");
		if ($self->{'pagenum'} < (@{$self->{'num_pages_array'}} - 1)) {
			$self->{'next_pagenum'} = $self->{'pagenum'} + 1;
			$self->{'next_pagename'} = $self->get_wizard_info_by_page($wizref,
				"pagename", $self->{'next_pagenum'});
			$self->{'next_pageinst'} = 0;
			}
	} else {
		$self->{'next_pageinst'} = $self->{'pageinst'} + 1;
		}
	# validation needed? does data already exist for page?
	$self->{'next_datafile'} = "$self->{'next_pagename'}.$self->{'next_pageinst'}";
	@{$self->{'datafiles_array'}} = $self->list_wizard_datafiles($self->{'next_datafile'});
	$self->{'validate_option'}= "";
	if (@{$self->{'datafiles_array'}} > 0) {
		$self->{'validate_option'} = "&validate=1";
		}
	$self->redirect("$self->{'next_pagename'}?pageinst=$self->{'next_pageinst'}$self->{'validate_option'}");
	}
if ($self->{'button_name'} eq "$self->{'text_hash'}->{'wizard_new'}") {
	# for new, if we have less than full complement of instances,
	# use the last unused instance, otherwise stay here
	$self->{'next_pagename'} = $pagename;
	$self->{'next_pageinst'} = $self->{'pageinst'};
	@{$self->{'num_pageinsts_array'}} = $self->list_wizard_datafiles($pagename);
	$self->{'max_inst'} = $self->get_wizard_info_by_name($wizref, "max_datafiles",
		$pagename);
	if (@{$self->{'num_pageinsts_array'}} <= $self->{'max_inst'}) {
		$self->{'next_pageinst'} = @{$self->{'num_pageinsts_array'}};
		}
	$self->redirect("$self->{'next_pagename'}?pageinst=$self->{'next_pageinst'}");
	}
if ($self->{'button_name'} eq "$self->{'text_hash'}->{'wizard_remove'}") {
	$self->wizard_clear_data($pagename, $self->{'pageinst'});
	$self->redirect("$pagename?pageinst=$self->{'pageinst'}");
	}
if (($self->{'button_name'} eq "$self->{'text_hash'}->{'wizard_add'}") ||
	($self->{'button_name'} eq "$self->{'text_hash'}->{'wizard_update'}")) { 
	# find next page
	$self->{'next_pagename'} = $pagename;
	$self->{'next_pageinst'} = $self->{'pageinst'};
	$self->{'max_inst'} = $self->get_wizard_info_by_name($wizref, "max_datafiles",
		$pagename);
	if ($self->{'pageinst'} < ($self->{'max_inst'} - 1)) {
		# in this case, add can move to another instance
		$self->{'next_pageinst'} = $self->{'pageinst'} + 1;
	} else {
		# in this case, add moves to next page if possible
		$self->{'pagenum'} = $self->get_wizard_info_by_name($wizref, "pagenum",
			$pagename);
		@{$self->{'num_pages_array'}} = $self->get_wizard_info_by_name($wizref, "pages");
		if ($self->{'pagenum'} < (@{$self->{'num_pages_array'}} - 1)) {
			$self->{'next_pagenum'} = $self->{'pagenum'} + 1;
			$self->{'next_pagename'} = $self->get_wizard_info_by_page($wizref,
				"pagename", $self->{'next_pagenum'});
			$self->{'next_pageinst'} = 0;
			}
		}
	# save data and reload validation stage (which includes automatic
	# redirect if no validation errors occur)
	$self->wizard_save_data($pagename, $self->{'pageinst'});
	$self->{'validation_success'} = 1;
	$self->redirect("$pagename?pageinst=$self->{'pageinst'}&validate=1&next_pagename=$self->{'next_pagename'}&next_pageinst=$self->{'next_pageinst'}");
	}

# Shouldnt be here!
$self->error("Invalid button name $self->{'button_name'}!");
}

# Return pageinst if defined, 0 otherwise.
#
# Parameters:
#
sub wizard_get_pageinst()
{
	my $self = shift;

local ($pageinst);
if (defined($self->{'in_hash'}->{'pageinst'})) {
	$pageinst = $self->{'in_hash'}->{'pageinst'};
} else {
	$pageinst = 0;
	}
return $pageinst;
}

# Redirect to next target pagename/pageinst are defined...
#
# Parameters:
#
sub wizard_redirect_next()
{
	my $self = shift;

local ($pagename, $pageinst);
($pagename, $pageinst) = @_;
if ((defined($self->{'in_hash'}->{'next_pagename'})) && (defined($self->{'in_hash'}->{'next_pageinst'}))) {
	# add validation success value to datafile
	# add validation result to datafile - used to assess if we can finish
	$self->wizard_save_validation_value($pagename, $pageinst);
	# if validation succeeded, move on...
	if ($self->{'validation_success'} == 1) {
		print <<EOF;
<script>
location.href = '$self->{'in_hash'}->{'next_pagename'}?pageinst=$self->{'in_hash'}->{'next_pageinst'}';
</script>
EOF
		}
	}
}

sub wizard_validate_input()
{
	my $self = shift;

local ($value, $validation_regexp, $validation_err);
($value, $validation_regexp, $validation_err) = @_;
# do we do validation here?
if (defined($self->{'in_hash'}->{'validate'})) {
	if ($self->{'in_hash'}->{'validate'} == 1) {
		if ($value =~ /^$validation_regexp$/) {
			# ok, so we can redirect
		} else {
			print "<tr $self->{'cb'}><td>\n";
			print
			    "<i><font color=#FF0000>$validation_err</font></i>";
			print "</td></tr>\n";
			# flag global var to indicate validation failed - no
			# redirect will occur as a result...
			$self->{'validation_success'} = 0;
			}
		}
	}
}

# Give appropriate full path to datafile when given pagename/inst
#
# Parameters: pagename, pageinst
#
sub wizard_datafile_name()
{
	my $self = shift;

local ($pagename, $pageinst);
($pagename, $pageinst) = @_;
return "$self->{'module_config_directory'}/${pagename}.${pageinst}.wizdata";
}

# Saves data to datafile for page/inst
#
# Parameters: pagename, pageinst
#
sub wizard_save_data()
{
	my $self = shift;

local ($pagename, $pageinst, @inkeys, $datafile);
($pagename, $pageinst) = @_;
$datafile = $self->wizard_datafile_name($pagename, $pageinst);
# save webmin's "in" hash to file
@inkeys = keys(%{$self->{'in_hash'}});
open(DATAFILE,">$datafile");
foreach $inkey (@inkeys) {
	print DATAFILE "$inkey=$self->{'in_hash'}->{$inkey}\n";
	}
close(DATAFILE);
}

sub wizard_save_validation_value()
{
	my $self = shift;

local ($pagename, $pageinst, $datafile);
($pagename, $pageinst) = @_;
$datafile = $self->wizard_datafile_name($pagename, $pageinst);
open(DATAFILE,">>$datafile");
print DATAFILE "validation_success=$self->{'validation_success'}";
close(DATAFILE);
}

# Removes all datafiles for wizard
#
# Parameters: wizard_info
#
sub wizard_clear_all_data()
{
	my $self = shift;

local ($wizinfo, %wizard, $wizref, @pages, $page, @allinsts, $inst);
$wizinfo = $_[0];
%wizard = $self->read_in_wizard("$wizinfo");
$wizref = \%wizard;
@pages = $self->get_wizard_info_by_name($wizref, "pages");
foreach $page (@pages) {
	@allinsts = $self->list_wizard_datafiles("$page->{'pagename'}");
	foreach $inst (@allinsts) {
		unlink($inst);
		}
	}
}

# Removes datafile, moves those above down one instance
#
# Parameters: pagename, pageinst
#
sub wizard_clear_data()
{
	my $self = shift;

local ($pagename, $pageinst, $datafile, @num_inst, $instnum);
($pagename, $pageinst) = @_;
$datafile = $self->wizard_datafile_name($pagename, $pageinst);
@num_inst = $self->list_wizard_datafiles($pagename);
unlink($datafile);
# move all instances above deleted instance down one to fill gap....
for ($instnum = $pageinst + 1; $instnum < @num_inst; $instnum++) {
	$self->{'datafile_old'} = $self->wizard_datafile_name($pagename,
		$instnum);
	$self->{'datafile_new'} = $self->wizard_datafile_name($pagename,
		$instnum -1);
	rename($self->{'datafile_old'}, $self->{'datafile_new'});
	}
}

# Reads wizard data file into key-value hash
#
# Parameters: pagename, pageinst
#
sub wizard_get_data()
{
	my $self = shift;

local ($pagename, $pageinst, $datafile, @data, $line, $key, $value, %hash);
($pagename, $pageinst) = @_;
$datafile = $self->wizard_datafile_name($pagename, $pageinst);
open(DATAFILE, $datafile);
@data = <DATAFILE>;
foreach $line (@data) {
	if ($line =~ /^\s*\#/) {
		next;
		}
	if ($line =~ /\s*([^=]+)=(.*)/) {
		$key = $1;
		$value = $2;
		$hash{"$key"} = "$value";
		}
        }
return %hash;
}

# Gets data value for particular page/inst
#
# Parameters: pagename, pageinst, datakey
#
sub wizard_get_data_value()
{
	my $self = shift;

local ($pagename, $pageinst, $datakey, %hash);
($pagename, $pageinst, $datakey) = @_;
%hash = $self->wizard_get_data($pagename, $pageinst);
return ($hash{"$datakey"});
}

# Gets data values for all page insts associated with a pagename
#
# Parameters: pagename, datakey
#
sub wizard_get_data_values()
{
	my $self = shift;

local ($pagename, $datakey, @allinsts, $instfile, $pageinst, $value);
($pagename, $datakey) = @_;
# get all matching instance files...
@allinsts = $self->list_wizard_datafiles($pagename);
@{$self->{'values_array'}} = ();
foreach $instfile (@allinsts) {
	$instfile =~ /${pagename}\.([^\.]*)/;
	$pageinst = $1;
	$value = $self->wizard_get_data_value($pagename, $pageinst, $datakey);
	push (@{$self->{'values_array'}}, $value);
	}
return @{$self->{'values_array'}};
}

# Read in wizard string, create internal representation (wizard hash).
#
# Parameters: wizard_string;
sub read_in_wizard()
{
	my $self = shift;

local ($wizard_string, @wizard_array, $pagenum, %wizard, $wiz_elt);
$wizard_string = $_[0];
@wizard_array = split(/;/, $wizard_string);
$pagenum = 0;
foreach $wiz_elt (@wizard_array) {
	if ($wiz_elt =~ /\s*page\s*=\s*([^,\s]*)\s*,\s*min\s*=\s*([^,\s]*)\s*,\s*max\s*=\s*(.+)/) {
		# standard wizard element 
		$wizard{'pages'}->[$pagenum]->{'pagename'} = $1;
		$wizard{'pages'}->[$pagenum]->{'min_datafiles'} = $2;
		$wizard{'pages'}->[$pagenum]->{'max_datafiles'} = $3;
		$wizard{'pages'}->[$pagenum]->{'pagenum'} = $pagenum;
		$pagenum = $pagenum + 1;
	} elsif ($wiz_elt =~ /\s*submit\s*=\s*(\S+)/) {
		# submit cgi script name
		$wizard{'submit'} = $1;
	} else {
		$self->error("Syntax error in wizard specification: $wiz_elt");
		}
	}
return %wizard;
}

sub get_wizard_info_by_name()
{
	my $self = shift;

local ($wizref, $pagename, @wizard_pages, $page);
($wizref, $self->{'data'}, $pagename) = @_;
@wizard_pages = @{$wizref->{'pages'}};
if ($self->{'data'} eq "pages") {
	return @wizard_pages;
} elsif ($self->{'data'} eq "submit") {
	return $wizref->{$self->{'data'}};
	}
foreach $page (@wizard_pages) {
	if ($pagename eq "$page->{'pagename'}") {
		return $page->{$self->{'data'}};
		}
	}
# not found
$self->error("Data $self->{'data'} not found for wizard. Pagename: $pagename");
}

sub get_wizard_info_by_page()
{
	my $self = shift;

local ($wizref, $data, $pagenum);
($wizref, $data, $pagenum) = @_;
@{$self->{'wizard_pages_array'}} = @{$wizref->{'pages'}};
return $self->{'wizard_pages_array'}->[$pagenum]->{$data};
}

# list all files matching pattern and ending in .wizdata
#
# Parameters: filematch_pattern
#
sub list_wizard_datafiles()
{
	my $self = shift;

local($filematch, $f,@filelist);
$filematch = $_[0];
opendir(DIR, $self->{'module_config_directory'});
foreach $f (readdir(DIR)) {
	if ($f =~ /^$filematch.*\.wizdata/) {
		push(@filelist, "$self->{'module_config_directory'}/$f");
		}
	}
return @filelist;
}


1;

