#!/usr/bin/perl

&MAIN();

 sub MAIN {
# dependency/dependent tree viewer
# Display a form for displaying/editing SMF service states in dependency
# and dependent trees.

&ModuleLoader::preload_module('smf_class', "smf::SmfLib");
$self = new smf::SmfLib:: 'smf_class',%$self;

$self->ReadParse();

# get instance fmri
if (defined($self->{'in_hash'}->{'fmri'})) {
	$self->{'fmri'} = $self->{'in_hash'}->{'fmri'};
	# remove quotes...
	$self->{'fmri'} =~ /\'([^\']*)\'/;
	$self->{'fmri'} = $1;
} else {
	$self->error("No fmri supplied to dep viewer!");
	}

# deal with application of state changes first. this way
# dep lists will show new states...
if (defined($self->{'in_hash'}->{'change_state'})) {
	$self->{'cmd'} = "$self->{'in_hash'}->{'change_state'}";
	# get update fmri list
	@{$self->{'update_fmris_array'}} = split(/\0/, $self->{'in_hash'}->{'applyto'});
	$self->svc_state_cmd("$self->{'cmd'}", \@{$self->{'update_fmris_array'}});
	}

@{$self->{'depy_expand_list_array'}} = ();
# which dependency fmris do we expand?
if (defined($self->{'in_hash'}->{'dependency_expandlist'})) {
	# get required expansions. each is fmri
	@{$self->{'depy_expand_list_array'}} = split(/\0/, $self->{'in_hash'}->{'dependency_expandlist'});
	}
# Expand list - list of fmris to expand. By default, expand top fmri.
push(@{$self->{'depy_expand_list_array'}}, $self->{'fmri'});

@{$self->{'dept_expand_list_array'}} = ();
# which dependent fmris do we expand?
if (defined($self->{'in_hash'}->{'dependent_expandlist'})) {
	# get required expansions. each is fmri
	@{$self->{'dept_expand_list_array'}} = split(/\0/, $self->{'in_hash'}->{'dependent_expandlist'});
	}
# Expand list - list of fmris to expand. By default, expand top fmri.
push(@{$self->{'dept_expand_list_array'}}, $self->{'fmri'});

# get new expansion/contractions submitted...
@{$self->{'inkeys_array'}} = keys(%{$self->{'in_hash'}});
foreach $inkey (@{$self->{'inkeys_array'}}) {
	if ($inkey =~ /dependency_expand_(.*)$/) {
		$self->{'expand_fmri'} = $1;
		push(@{$self->{'depy_expand_list_array'}}, $self->{'expand_fmri'});
		}
	if ($inkey =~ /dependency_contract_(.*)$/) {
		$self->{'contract_fmri'} = $1;
		foreach $e (@{$self->{'depy_expand_list_array'}}) {
			$self->{'old'} = shift(@{$self->{'depy_expand_list_array'}});
			if ($self->{'old'} eq $self->{'contract_fmri'}) {
				next;
				}
			push(@{$self->{'depy_expand_list_array'}}, $self->{'old'});
			}
		}
	if ($inkey =~ /dependent_expand_(.*)$/) {
		$self->{'expand_fmri'} = $1;
		push(@{$self->{'dept_expand_list_array'}}, $self->{'expand_fmri'});
		}
	if ($inkey =~ /dependent_contract_(.*)$/) {
		$self->{'contract_fmri'} = $1;
		foreach $e (@{$self->{'dept_expand_list_array'}}) {
			$self->{'old'} = shift(@{$self->{'dept_expand_list_array'}});
			if ($self->{'old'} eq $self->{'contract_fmri'}) {
				next;
				}
			push(@{$self->{'dept_expand_list_array'}}, $self->{'old'});
			}
		}
	}

# Gather service description, and state info
$self->{'description'} = $self->run_smf_cmds("/usr/bin/svcs -H -oDESC $self->{'fmri'}", 0);

%{$self->{'depy_tree_hash'}} = ();
%{$self->{'dept_tree_hash'}} = ();
$self->{'depy_treeref'} = \%{$self->{'depy_tree_hash'}};
$self->{'dept_treeref'}= \%{$self->{'dept_tree_hash'}};
$self->{'instance_state'} = $self->svc_get_state_cmd($self->{'fmri'});
$self->build_dep_tree("dependent", $self->{'dept_treeref'}, $self->{'fmri'}, 0, \@{$self->{'dept_expand_list_array'}},
	$self->{'instance_state'});
$self->build_dep_tree("dependency", $self->{'depy_treeref'}, $self->{'fmri'}, 0, \@{$self->{'depy_expand_list_array'}},
	$self->{'instance_state'});

$self->ui_print_header(undef, $self->{'text_hash'}->{'dep_viewer_title'}, "", undef);

print "<form method=\"POST\" action=\"dep_viewer.cgi?fmri='$self->{'fmri'}'\">\n";

print "<h2>$self->{'description'}</h2>\n";
print "<h2>";
$self->text_and_whats_this("dep_viewer_detail");
print "</h2>";

print "<h2>$self->{'text_hash'}->{'dep_viewer_depy_info'}</h2>\n";
print "<p>$self->{'text_hash'}->{'dep_viewer_apply'}:&nbsp\n";
$self->print_state_buttons();
print "</p>\n";
$self->print_dep_tree("dependency", $self->{'depy_treeref'}, $self->{'fmri'}, 0, \@{$self->{'depy_expand_list_array'}});

print "<h2>$self->{'text_hash'}->{'dep_viewer_dept_info'}</h2>\n";
print "<p>$self->{'text_hash'}->{'dep_viewer_apply'}:&nbsp\n";
$self->print_state_buttons();
print "</p>\n";
$self->print_dep_tree("dependent", $self->{'dept_treeref'}, $self->{'fmri'}, 0, \@{$self->{'dept_expand_list_array'}});

print "</form>\n";

$self->print_cmds_run();

$self->ui_print_footer("instance_viewer.cgi?fmri='$self->{'fmri'}'",
        $self->{'text_hash'}->{'dep_viewer_back'});
1;
		
}


1;

