#!/usr/bin/perl
package software::AixLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'software::AixLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# aix-lib.pl
# Functions for aix package management

1;

1;
		return $self;
}

# list_packages([package]*)
# Fills the array %{$self->{'packages_hash'}} with a list of all packages
sub list_packages
{
	my $self = shift;

	local($_, $list, $i, $file, %fileset);
	$i = 0;
	$list = join(' ', @_);
	$self->open_execute_command(*LSLPP, "lslpp -L -c $list 2>&1 | grep -v '#'", 1, 1);
	while($file = <LSLPP>) {
                %fileset = $self->fileset_info($file); 
		if ($fileset{"type"} eq "R") {
                        $self->{'packages_hash'}->{$i,'name'} = join("-", $fileset{"package_name"}, $fileset{"level"});
	                $self->{'packages_hash'}->{$i,'class'} = $fileset{"package_name"};
	                $self->{'packages_hash'}->{$i,'desc'} = $fileset{"description"};
                }
 		else {
	                $self->{'packages_hash'}->{$i,'name'} = $fileset{"fileset"};
	                $self->{'packages_hash'}->{$i,'class'} = $fileset{"class"};
	                $self->{'packages_hash'}->{$i,'desc'} = $fileset{"description"};
	 	}
		$i++;
	}
	close(LSLPP);
	return $i;
}

# package_info(package)
# Returns an array of package information in the order
#  name, class, description, arch, version, vendor, installtime
sub package_info
{
	my $self = shift;

	local(@rv, @tmp, $d, $out, $archout, %fileset);

	local $qm = quotemeta($_[0]);
	$out = $self->backquote_command("lslpp -L -c $qm 2>&1 | grep -v '#'", 1);
	%fileset = $self->fileset_info($out);
	if ($out =~ /^lslpp:/) {
		$self->open_execute_command(*RPM, "rpm -q $qm --queryformat \"%{NAME}-%{VERSION}\\n%{GROUP}\\n%{ARCH}\\n%{VERSION}\\n%{VENDOR}\\n%{INSTALLTIME}\\n\" 2>/dev/null", 1, 1);
		@tmp = <RPM>;
		chop(@tmp);
		if (!@tmp) { return (); }
		close(RPM);
		$self->open_execute_command(*RPM, "rpm -q $qm --queryformat \"%{DESCRIPTION}\"", 1, 1);
		while(<RPM>) { $d .= $_; }
		close(RPM);
		return ($tmp[0], $tmp[1], $d, $tmp[2], $tmp[3], $tmp[4], $self->make_date($tmp[5]));
     	}
	else {
		$archout = `uname -s`; 
		push(@rv, $_[0]);
		push(@rv, $fileset{"class"});
		push(@rv, $fileset{"description"});
		push(@rv, $archout);   
		push(@rv, $fileset{"level"});
		push(@rv, $self->{'text_hash'}->{'aix_unknown'});
		push(@rv, $self->{'text_hash'}->{'aix_unknown'});
		return @rv;
     	}
}

# is_package(file)
# Tests if some file is a valid package file
sub is_package
{
	my $self = shift;

	local($out, $name, $filetype);
	local $qm = quotemeta($_[0]);
	$out = $self->backquote_command("file $qm 2>&1", 1);
	($name, $filetype) = split(/:/, $out);
	if ($filetype =~ /backup\/restore format file/i) { 
	 	$self->{'fileset_hash'}->{"filetype"} = "AIX";
		return 1;	
  	}
        elsif ($filetype =~ /RPM v3 bin PowerPC/i) {
		$self->{'fileset_hash'}->{"filetype"} = "RPM";
		return 1;
        }
        else {  return 0; }
}

# file_packages(file)
# Returns a list of all packages in the given file, in the form
#  package description
sub file_packages
{
	my $self = shift;

        local($_, $line, $i, $j, $file, $out, $continue, @token, $firstline);
	local(@rv, @output, $description, @vrmf, @field, $k, $l, $stub);
	local $qm = quotemeta($_[0]);
	local $real = $self->translate_filename($_[0]);
        if ($self->is_package($_[0])) {
                @token = split(/\//, $_[0]);
                $i = @token;
                $file = $token[$i - 1];
		if ($self->{'fileset_hash'}->{"filetype"} eq "RPM") {
			$self->{'fileset_hash'}->{"fileset_name"} = "R:$file";
			if (-d $real) {
		       		local @rv;
			        $self->open_execute_file(RPM, "cd $qm ; rpm -q -p *.rpm --queryformat \"%{NAME}-%{VERSION} %{SUMMARY}\\n\" 2>&1", 1, 1);
			        while(<RPM>) {
			                chop;
		        	        push(@rv, $_) if (!/does not appear|query of.*failed/);
		                }
			        close(RPM);
			        return @rv;
		        }
			else {
			        local($out);
		        	$out = $self->backquote_command("rpm -q -p $qm --queryformat \"%{NAME}-%{VERSION} %{SUMMARY}\" 2>&1", 1);
			        return ($out);
		        }
		}
		elsif ($self->{'fileset_hash'}->{"filetype"} eq "AIX") {
			$self->open_tempfile(FILE, $_[0]);
			$line = <FILE>;
			$continue = 1;
        	       	while ($continue) {
				if ($line =~ /^\}/) { $continue = 0; }
	                	elsif (($line =~ /lpp_name/) && ($line =~ / \{/)) { 
					$firstline = $line; 
        	       	        	$line = <FILE>;
               			    	$out = $out . $line;
	       	        	}
	                        elsif ($line =~ /^\]/) {
        	       	               	$line = <FILE>;
					if ($line =~ /^\}/) { $continue = 0; }
					else { $out = $out . $line; }
		               	}	
				$line = <FILE>; 
       			}
			close(FILE);
			@field = split(/\s/, $firstline);
			$j = @field;
			for ($i = 0; $i < $j; $i++) {
				if ($field[$i] =~ /\{/) {
					$self->{'fileset_hash'}->{"fileset_name"} = $field[$i - 1] =~ /\>|\s+/ ? $file : 
                       	                	                   $field[$i - 1];
				}
			}
			@output = split(/\n/, $out);
		        $j = @output;
			for ($i = 0; $i < $j; $i++) {
				if ($output[$i] !~ /\{|\}|\*/) {
					@field = split(/\s/, $output[$i]);
                                        $out = undef;
                                        $out = "$self->{'fileset_hash'}->{fileset_name}  ";
					$description = undef;
					@vrmf = split(/\./, $field[1]);
					$vrmf[0] =~ s/^0+//;
					$vrmf[1] =~ s/^0+//; if ($vrmf[1] == "") { $vrmf[1] = "0"; }
					$vrmf[2] =~ s/^0+//; if ($vrmf[2] == "") { $vrmf[2] = "0"; } 
					$vrmf[3] =~ s/^0+//; if ($vrmf[3] == "") { $vrmf[3] = "0"; }
					if ($vrmf[0] != "") {
						$k = @field;
						for ($l = 6; $l <= $k; $l++) {
							$description = $description . " " . $field[$l];
						}
						$out = $out . "$field[0], $vrmf[0].$vrmf[1].$vrmf[2].$vrmf[3], $description";
					}	
				}
                                push(@rv, $out);
			}
			return @rv;
		}
	}
	else {  return undef; }
}

# install_options(file, package)
# Outputs HTML for choosing install options
sub install_options
{
	my $self = shift;

        local(@token, $command, $i, $j, $file, $directory);
        @token = split(/\//, $_[0]);
        $i = @token;
        for ( $j = 1; $j < $i - 1; $j++) {
                $directory = join("/", $directory, $token[$j]);
        }

	$file = $token[$i-1];

	print "<script language=\"JavaScript\">\n";
	print "   function changeRadio(formName, radiobutton, disable) {\n";
        print "      for( var i=0; i<formName.elements.length; i++) {\n";
        print "         if (formName.elements[i].name == radiobutton) {\n";
        print "            formName.elements[i].checked = disable;\n";
        print "         }\n";
        print "      }\n";
        print "   }\n";
	print "</script>\n";

        print "<tr>\n";
        print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_device'}</b>", "aix_device"), "</td>\n";
        print "<td>$directory</td>\n";
        print "</tr>\n";

        print "<tr>\n";
        print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_software'}</b>", "aix_software"), "</td>\n";
        print "<td>$file</td>\n";
        print "</tr>\n";

	print "<tr>\n";
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_preview'}</b>", "aix_preview"), "</td>\n";
	print "<td><input type=radio name=preview value=1> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=preview value=0 checked> $self->{'text_hash'}->{'no'}</td>\n";
	print "</tr>\n";

	print "<tr>\n";
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_commit'}</b>", "aix_commit"), "</td>\n";
	print "<td><input type=radio name=commit value=1 checked> $self->{'text_hash'}->{'yes'}\n"; 
	print "<input type=radio name=commit value=0 \n";
	print "     onClick=\"changeRadio(this.form, 'save', true)\"> $self->{'text_hash'}->{'no'}</td>\n";
	print "</tr>\n";

	print "<tr>\n";
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_save'}</b>", "aix_save"), "</td>\n";
	print "<td><input type=radio name=save value=1> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=save value=0 checked> $self->{'text_hash'}->{'no'}</td>\n";
	print "</tr>\n";

	print "<tr>\n";
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_auto'}</b>", "aix_auto"), "</td>\n";
	print "<td><input type=radio name=auto value=1 \n";
	print "     onClick=\"changeRadio(this.form, 'overwrite', false)\" checked> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=auto value=0> $self->{'text_hash'}->{'no'}</td>\n";
	print "</tr>\n";

	print "<tr>\n";
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_extend'}</b>", "aix_extend"), "</td>\n";
	print "<td><input type=radio name=extend value=1 checked> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=extend value=0> $self->{'text_hash'}->{'no'}</td>\n";
	print "</tr>\n";

	print "<tr>\n";
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_overwrite'}</b>", "aix_overwrite"), "</td>\n";
	print "<td><input type=radio name=overwrite value=1 \n";
	print "     onClick=\"changeRadio(this.form, 'auto', false)\"> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=overwrite value=0 checked > $self->{'text_hash'}->{'no'}</td>\n";
	print "</tr>\n";

	print "<tr>\n";
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_verify'}</b>", "aix_verify"), "</td>\n";
	print "<td><input type=radio name=verify value=1> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=verify value=0 checked> $self->{'text_hash'}->{'no'}</td>\n";
	print "</tr>\n";

#        print "<tr>\n";
#        print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_include'}</b>", "aix_include"), "</td>\n";
#        print "<td><input type=radio name=include value=1> $self->{'text_hash'}->{'yes'}\n";
#        print "<input type=radio name=include value=0 checked> $self->{'text_hash'}->{'no'}</td>\n";
#        print "</tr>\n";

	print "<tr>\n";
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_detail'}</b>", "aix_detail"), "</td>\n";
	print "<td><input type=radio name=detail value=1> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=detail value=0 checked> $self->{'text_hash'}->{'no'}</td>\n";
	print "</tr>\n";

	print "<tr>\n";
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_process'}</b>", "aix_process"), "</td>\n";
	print "<td><input type=radio name=process value=1 checked> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=process value=0> $self->{'text_hash'}->{'no'}</td>\n";
	print "</tr>\n";

	print "<tr>\n";
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_accept'}</b>", "aix_accept"), "</td>\n";
	print "<td><input type=radio name=accept value=1> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=accept value=0 checked> $self->{'text_hash'}->{'no'}</td>\n";
	print "</tr>\n";

	print "<tr>\n";
	print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_license'}</b>", "aix_license"), "</td>\n";
	print "<td><input type=radio name=license value=1> $self->{'text_hash'}->{'yes'}\n";
	print "<input type=radio name=license value=0 checked> $self->{'text_hash'}->{'no'}</td>\n";
	print "</tr>\n";

        print "<tr>\n";
        print "<td>", $self->hlink("<b>$self->{'text_hash'}->{'aix_clean'}</b>", "aix_clean"), "</td>\n";
        print "<td><input type=radio name=clean value=1 \n";
	print "     onClick=\"changeRadio(this.form, 'preview', false),\n";
	print "		      changeRadio(this.form, 'commit', false),\n";
	print "               changeRadio(this.form, 'save', false),\n";
	print "               changeRadio(this.form, 'auto', false),\n";
	print "               changeRadio(this.form, 'extend', false),\n";
	print "               changeRadio(this.form, 'overwrite', false),\n";
	print "               changeRadio(this.form, 'verify', false),\n";
#	print "               changeRadio(this.form, 'include', false),\n";
	print "               changeRadio(this.form, 'detail', false),\n";
	print "               changeRadio(this.form, 'process', false),\n";
	print "               changeRadio(this.form, 'accept', false),\n";
	print "               changeRadio(this.form, 'license', false)\"> $self->{'text_hash'}->{'yes'}\n";
        print "<input type=radio name=clean value=0\n";
	print "     onClick=\"reset()\" checked> $self->{'text_hash'}->{'no'}</td>\n";
        print "</tr>\n";
}

# install_package(file, package)
# Installs the package in the given file, with options from %{$self->{'in_hash'}}
sub install_package
{
	my $self = shift;

	local(@token, $command, $directory, $out);
	@token = split(/\//, $_[0]);
	$self->{'i'} = @token;
	for ( $self->{'j'} = 1; $self->{'j'} < $self->{'i'} - 1; $self->{'j'}++) {
		$directory = join("/", $directory, $token[$self->{'j'}]);
	}

	local $args = ($self->{'in_hash'}->{"preview"}   ? "p"  : "")   .
        	      ($self->{'in_hash'}->{"commit"}    ? "c"  : "")   .
 	              ($self->{'in_hash'}->{"save"}      ? ""   : "N")  .
		      ($self->{'in_hash'}->{"auto"}      ? "g"  : "")   .
		      ($self->{'in_hash'}->{"extend"}    ? "X"  : "")   .
        	      ($self->{'in_hash'}->{"overwrite"} ? "F"  : "")   .
	              ($self->{'in_hash'}->{"verify"}    ? "v"  : "")   .
#        	      ($self->{'in_hash'}->{"include"}   ? "G"  : "")   .
	              ($self->{'in_hash'}->{"detail"}    ? "V2" : "")   .
	              ($self->{'in_hash'}->{"process"}   ? ""   : "S")  .
	              ($self->{'in_hash'}->{"accept"}    ? "Y"  : "")   .
	              ($self->{'in_hash'}->{"license"}   ? "E"  : "");

	$command = "geninstall -I \"-a$args\" -d '$directory' '$self->{'fileset_hash'}->{fileset_name}' 2>&1";
	if ($self->{'in_hash'}->{"clean"}) {
		$args = "-C";
		$command = "installp $args 2>&1";
	}

	local $out = $self->backquote_logged($command);
        	if (($?) || ($self->{'in_hash'}->{"preview"}) || ($self->{'in_hash'}->{"clean"})) { 
        		return "<pre>$command<br>$out</pre>";
        }
	return undef;
}

# check_files(package)
# Fills in the %{$self->{'files_hash'}} array with information about the files belonging
# to some package. Values in %{$self->{'files_hash'}} are path type user group mode size error link
sub check_files
{
	my $self = shift;

	local($_, $list, $i, $_, @w, %errs, %myfile, $epath, $path, $fileset, $file);
	local $qm = quotemeta($_[0]);
	$i = 0;
	$list = join(' ', @_);
	$self->open_execute_command(*LSLPP, "lslpp -f -c $qm 2>&1 | grep -v '#'", 1, 1);
	$self->{'out'} = <LSLPP>;
	if ($self->{'out'} =~ /lslpp:/) { 
		$self->open_execute_command(*RPM, "rpm -V $qm", 1, 1);
		while(<RPM>) {
       			/^(.{8}) (.) (.*)$/;
	        	if ($1 eq "missing ") { $errs{$3} = "Missing"; }
        		else {
                		$epath = $3;
	                	@w = grep { $_ ne "." } split(//, $1);
        	        	$errs{$epath} =
                	        	join("\n", map { "Failed $self->{'etype_hash'}->{$_} check" } @w);
	             	}
        	}
		close(RPM);
		$self->open_execute_command(*RPM, "rpm -q $qm -l --dump", 1, 1);
		while(<RPM>) {
        		chop;
                	if ($_ =~ /(contains no files)/) { return $i; }
	        	@w = split(/ /);
       			$self->{'files_hash'}->{$i,'path'} = $w[0];
        		if ($w[10] ne "X") { $self->{'files_hash'}->{$i,'link'} = $w[10]; }
        		$self->{'files_hash'}->{$i,'type'} = $w[10] ne "X" ? 3 :
					    (-d $self->translate_filename($w[0]))? 1 :
	                	            $w[7]         ? 5 : 0;
	        	$self->{'files_hash'}->{$i,'user'} = $w[5];
        		$self->{'files_hash'}->{$i,'group'} = $w[6];
        		$self->{'files_hash'}->{$i,'size'} = $w[1];
	       		$self->{'files_hash'}->{$i,'error'} = $w[7]        ? "" : $errs{$w[0]};
        		$i++;
       		}
		close(RPM);
	}
	else {
		while($self->{'fileinfo'} = <LSLPP>) {
			chop($self->{'fileinfo'});
			#Path:Fileset:File
			($path, $fileset, $file) = split(/:/, $self->{'fileinfo'});
			%myfile = $self->file_info($file);
                	$self->{'files_hash'}->{$i,'path'}  = $myfile{"path"};
	                $self->{'files_hash'}->{$i,'type'}  = $myfile{"type"}; 
	                $self->{'files_hash'}->{$i,'user'}  = $myfile{"uid"}; 
        	        $self->{'files_hash'}->{$i,'group'} = $myfile{"gid"};
                	$self->{'files_hash'}->{$i,'mode'}  = $myfile{"mode"}; 
	                $self->{'files_hash'}->{$i,'size'}  = $myfile{"size"}; 
			$self->{'files_hash'}->{$i,'link'}  = $myfile{"link"};
	                $self->{'files_hash'}->{$i,'error'} = $myfile{"err"};
	                $i++;
	        }
	}
	close(LSLPP);
	return $i;
}

# installed_file(file)
# Given a filename, fills %{$self->{'file_hash'}} with details of the given file and returns 1.
# If the file is not known to the package system, returns 0
# Usable values in %{$self->{'file_hash'}} are  path type user group mode size packages
sub installed_file
{
	my $self = shift;

	local(%myfile);
	local $qm = quotemeta($_[0]);
	$self->{'out'} = $self->backquote_command("lslpp -wc $qm 2>&1 | grep -v '#'", 1);
	if ($?) {
		local($pkg, @w, $_);
		undef(%{$self->{'file_hash'}});
		$pkg = $self->backquote_command("rpm -q -f $qm --queryformat \"%{NAME}-%{VERSION}\\n\" 2>&1", 1);

		if ($pkg =~ /not owned/ || $?) { return 0; }
		@{$self->{'pkgs_array'}} = split(/\n/, $pkg);
		$self->open_execute_command(*RPM, "rpm -q $self->{'pkgs_array'}->[0] -l --dump", 1);
		while(<RPM>) {
		        chop;
	        	@w = split(/ /);
	        	if ($w[0] eq $_[0]) {
	                	$self->{'file_hash'}->{'packages'} = join(' ', @{$self->{'pkgs_array'}});
        	        	$self->{'file_hash'}->{'path'} = $w[0];
                		if ($w[10] ne "X") { $self->{'files_hash'}->{$self->{'i'},'link'} = $w[10]; }
             			$self->{'file_hash'}->{'type'} = $w[10] ne "X" ? 3 :
					(-d $self->translate_filename($w[0])) ? 1 :
                                		$w[7]         ? 5 : 0;
                		$self->{'file_hash'}->{'user'} = $w[5];
                		$self->{'file_hash'}->{'group'}= $w[6];
                		$self->{'file_hash'}->{'mode'} = substr($w[4], -4);
                		$self->{'file_hash'}->{'size'} = $w[1];
                		last;
                	}
        	}
		close(RPM);
	}
	else {
		%myfile = $self->file_info($_[0]);
		$self->{'file_hash'}->{'path'}     = $myfile{"path"};
		$self->{'file_hash'}->{'type'}     = $myfile{"type"};
		$self->{'file_hash'}->{'user'}     = $myfile{"uid"};
		$self->{'file_hash'}->{'group'}    = $myfile{"gid"};
		$self->{'file_hash'}->{'mode'}     = $myfile{"mode"};
		$self->{'file_hash'}->{'size'}     = $myfile{"size"};
		$self->{'file_hash'}->{'link'}     = $myfile{"link"};
		$self->{'file_hash'}->{'packages'} = $myfile{"package"};
	}
	return 1;
}

# delete_package(package)
# Totally remove some package
sub delete_package
{
	my $self = shift;

        local(%fileset, $file, $out, $rv);
	local $temp = $self->transname();
	local $qm = quotemeta($_[0]);

	$file = $self->backquote_command("lslpp -L -c $qm 2>&1 | grep -v '#'", 1);
	%fileset = $self->fileset_info($file);

	$rv = $self->system_logged("geninstall -u $_[0] >$temp 2>&1");
	local $out = $self->backquote_command("cat $temp");
	$self->unlink_file($temp);
	
	if ($rv) { return "<pre>$out</pre>"; }
	return undef; 
}

sub fileset_info
{
	my $self = shift;

	local($_, $out, %fileset, $package_name, $fileset, $level, $state, $ptf_id, 
	      $fix_state, $type, $description, $destination_dir, $uninstaller, 
	      $msg_cat, $msg_set, $msg_num, $parent, $class);
	%fileset = ();
	($package_name, $fileset, $level, $state, $ptf_id, $fix_state,
	 $type, $description, $destination_dir, $uninstaller, $msg_cat,
	 $msg_set, $msg_num, $parent) = split(/:/, $_[0]);
	($class, $self->{'stub'})             = split(/\./, $package_name);
	$fileset{"class"}           = $class;
	$fileset{"package_name"}    = $package_name;
	$fileset{"fileset"}         = $fileset;
	$fileset{"level"}           = $level;
	$fileset{"state"}           = $state;
	$fileset{"ptf_id"}          = $ptf_id;
	$fileset{"fix_state"}       = $fix_state;
	$fileset{"type"}            = $type;
	$fileset{"description"}     = $description;
	$fileset{"destination_dir"} = $destination_dir;
	$fileset{"uninstaller"}     = $uninstaller;
	$fileset{"msg_cat"}         = $msg_cat;
        $fileset{"msg_set"}         = $msg_set;
	$fileset{"msg_num"}         = $msg_num;
	$fileset{"parent"}          = $parent;
	return(%fileset);
}

sub file_info
{
	my $self = shift;

	local($_, %file, @out, $filename, $fileset, $type,  
	      $dev, $ino, $mode, $nlink, $uid, $gid, @args, $rdev,
	      $size, $atime, $mtime, $ctime, $blksize, $blocks);
  	%file = ();	
	local $real;
	
        if ($_[0] =~ /->/) {
		($filename, $self->{'symlink'}) = split(/->/, $_[0]);
		$real = $self->translate_filename($filename);
		chop($filename);
                ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size,
                 $atime, $mtime, $ctime, $blksize, $blocks) = lstat($real);
	}
	else {
		$filename = $_[0];
		$real = $self->translate_filename($filename);
                ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size,
                 $atime, $mtime, $ctime, $blksize, $blocks) = stat($real);
	}
	
	local $qm = quotemeta($filename);
	@out = $self->backquote_command("lslpp -wc $qm 2>&1", 1);
	($self->{'stub'}, $fileset, $type) = split(/:/, $out[1]);

        $file{"path"}    = $filename;	
	$file{"dev"}     = sprintf "%d,%d", $dev/256, $dev%256;
	$file{"ino"}     = $ino;
	$file{"mode"}    = sprintf "%o", $mode & 07777;
	$file{"nlink"}   = $nlink;
	$file{"uid"}     = getpwuid($uid);
	$file{"gid"}     = getgrgid($gid);
	$file{"rdev"}    = readlink($rdev);
	$file{"size"}    = $size        ? $size : 0;
	$file{"atime"}   = scalar(localtime($atime));
	$file{"mtime"}   = scalar(localtime($mtime));
	$file{"ctime"}   = scalar(localtime($ctime));
	$file{"blksize"} = $blksize;
	$file{"blocks"}  = $blocks;
	$file{"link"}    = readlink($real);
	$file{"package"} = $fileset;
	$file{"err"}     = -e $real  ? 0 : "Error";
	$file{"type"}    = -l $real ? 3 :
                           -d $real ? 1 :
			   0;
	return(%file);
}

sub package_system
{
	my $self = shift;

	return $self->{'text_hash'}->{'aix_manager'}; 
}

sub package_help
{
	my $self = shift;

        return "installp lslpp";
}


1;

