#!/usr/bin/perl
package software::DebianLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'software::DebianLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# debian-lib.pl
# Functions for debian DPKG package management

1;

1;
		return $self;
}

# list_packages([package]*)
# Fills the array %{$self->{'packages_hash'}} with a list of all packages
sub list_packages
{
	my $self = shift;

local $i = 0;
local $arg = @_ ? join(" ", map { quotemeta($_) } @_) : "";
$self->open_execute_command(*PKGINFO, "COLUMNS=200 dpkg --list $arg", 1, 1);
while(<PKGINFO>) {
	next if (/^\|/ || /^\+/);
	if (/^[ri]i..(\S+)\s+(\S+)\s+(.*)/) {
		$self->{'packages_hash'}->{$i,'name'} = $1;
		$self->{'packages_hash'}->{$i,'class'} = $self->alphabet_name($1);
		$self->{'packages_hash'}->{$i,'version'} = $2;
		$self->{'packages_hash'}->{$i,'desc'} = $3;
		$i++;
		}
	}
close(PKGINFO);
return $i;
}

sub alphabet_name
{
	my $self = shift;

return lc($_[0]) =~ /^[a-e]/ ? "A-E" :
       lc($_[0]) =~ /^[f-j]/ ? "F-J" :
       lc($_[0]) =~ /^[k-o]/ ? "K-O" :
       lc($_[0]) =~ /^[p-t]/ ? "P-T" :
       lc($_[0]) =~ /^[u-z]/ ? "U-Z" : "Other";
}

# package_info(package)
# Returns an array of package information in the order
#  name, class, description, arch, version, vendor, installtime
sub package_info
{
	my $self = shift;

local $qm = quotemeta($_[0]);
local $out = $self->backquote_command("dpkg --print-avail $qm 2>&1", 1);
return () if ($? || $out =~ /Package .* is not available/i);
local @rv = ( $_[0], $self->alphabet_name($_[0]) );
push(@rv, $out =~ /Description:\s+([\0-\177]*\S)/i ? $1
						   : $self->{'text_hash'}->{'debian_unknown'});
push(@rv, $out =~ /Architecture:\s+(\S+)/i ? $1 : $self->{'text_hash'}->{'debian_unknown'});
push(@rv, $out =~ /Version:\s+(\S+)/i ? $1 : $self->{'text_hash'}->{'debian_unknown'});
push(@rv, $out =~ /Maintainer:\s+(.*)/i ? $self->html_escape($1)
					 : $self->{'text_hash'}->{'debian_unknown'});
push(@rv, $self->{'text_hash'}->{'debian_unknown'});
return @rv;
}

# check_files(package)
# Fills in the %{$self->{'files_hash'}} array with information about the files belonging
# to some package. Values in %{$self->{'files_hash'}} are  path type user group mode size error
sub check_files
{
	my $self = shift;

local $i = 0;
local $file;
local $qm = quotemeta($_[0]);
$self->open_execute_command(*PKGINFO, "dpkg --listfiles $qm", 1, 1);
while($file = <PKGINFO>) {
	$file =~ s/\r|\n//g;
	next if ($file !~ /^\/[^\.]/);
	local @st = stat($file);
	$self->{'files_hash'}->{$i,'path'} = $file;
	$self->{'files_hash'}->{$i,'type'} = -l $file ? 3 :
			    -d $file ? 1 : 0;
	$self->{'files_hash'}->{$i,'user'} = getpwuid($st[4]);
	$self->{'files_hash'}->{$i,'group'} = getgrgid($st[5]);
	$self->{'files_hash'}->{$i,'mode'} = sprintf "%o", $st[2] & 07777;
	$self->{'files_hash'}->{$i,'size'} = $st[7];
	$self->{'files_hash'}->{$i,'link'} = readlink($file);
	$i++;
	}
return $i;
}

# installed_file(file)
# Given a filename, fills %{$self->{'file_hash'}} with details of the given file and returns 1.
# If the file is not known to the package system, returns 0
# Usable values in %{$self->{'file_hash'}} are  path type user group mode size packages
sub installed_file
{
	my $self = shift;

local $qm = quotemeta($_[0]);
local $out = $self->backquote_command("dpkg --search $qm 2>&1", 1);
return 0 if ($out =~ /not found/i);
$out =~ s/:\s+\S+\n$//;
local @pkgin = split(/[\s,]+/, $out);
local $real = $self->translate_filename($_[0]);
local @st = stat($real);
$self->{'file_hash'}->{'path'} = $_[0];
$self->{'file_hash'}->{'type'} = -l $real ? 3 :
		-d $real ? 1 : 0;
$self->{'file_hash'}->{'user'} = getpwuid($st[4]);
$self->{'file_hash'}->{'group'} = getgrgid($st[5]);
$self->{'file_hash'}->{'mode'} = sprintf "%o", $st[2] & 07777;
$self->{'file_hash'}->{'size'} = $st[7];
$self->{'file_hash'}->{'link'} = readlink($real);
$self->{'file_hash'}->{'packages'} = join(" ", @pkgin);
return 1;
}

# is_package(file)
sub is_package
{
	my $self = shift;

local $qm = quotemeta($_[0]);
local $out = $self->backquote_command("dpkg --info $qm 2>&1", 1);
return $? || $out !~ /Package:/ ? 0 : 1;
}

# file_packages(file)
# Returns a list of all packages in the given file, in the form
#  package description
sub file_packages
{
	my $self = shift;

local $qm = quotemeta($_[0]);
local $out = $self->backquote_command("dpkg --info $qm 2>&1", 1);
local $name;
if ($out =~ /Package:\s+(\S+)/i && ($name=$1) &&
    $out =~ /Description:\s+(.*)/i) {
	return ( "$name $1" );
	}
return ();
}

# install_options(file, package)
# Outputs HTML for choosing install options
sub install_options
{
	my $self = shift;

print "<tr> <td><b>$self->{'text_hash'}->{'debian_depends'}</b></td>\n";
print "<td><input type=radio name=depends value=1> $self->{'text_hash'}->{'yes'}\n";
print "<input type=radio name=depends value=0 checked> $self->{'text_hash'}->{'no'}</td> </tr>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'debian_conflicts'}</b></td>\n";
print "<td><input type=radio name=conflicts value=1> $self->{'text_hash'}->{'yes'}\n";
print "<input type=radio name=conflicts value=0 checked> $self->{'text_hash'}->{'no'}</td> </tr>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'debian_overwrite'}</b></td>\n";
print "<td><input type=radio name=overwrite value=1> $self->{'text_hash'}->{'yes'}\n";
print "<input type=radio name=overwrite value=0 checked> $self->{'text_hash'}->{'no'}</td> </tr>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'debian_downgrade'}</b></td>\n";
print "<td><input type=radio name=downgrade value=1> $self->{'text_hash'}->{'yes'}\n";
print "<input type=radio name=downgrade value=0 checked> $self->{'text_hash'}->{'no'}</td> </tr>\n";
}

# install_package(file, package)
# Installs the package in the given file, with options from %{$self->{'in_hash'}}
sub install_package
{
	my $self = shift;

local $in = $_[2] ? $_[2] : \%{$self->{'in_hash'}};
local $args = ($in->{'depends'} ? " --force-depends" : "").
	      ($in->{'conflicts'} ? " --force-conflicts" : "").
	      ($in->{'overwrite'} ? " --force-overwrite" : "").
	      ($in->{'downgrade'} ? " --force-downgrade" : "");
local $qm = quotemeta($_[0]);
local $out = $self->backquote_logged("dpkg --install $args $qm 2>&1 </dev/null");
if ($?) {
	return "<pre>$out</pre>";
	}
return undef;
}

# delete_package(package)
# Totally remove some package
sub delete_package
{
	my $self = shift;

local $qm = quotemeta($_[0]);
local $out = $self->backquote_logged("dpkg --remove $qm 2>&1 </dev/null");
if ($? || $out =~ /which isn.t installed/i) {
	return "<pre>$out</pre>";
	}
return undef;
}

sub package_system
{
	my $self = shift;

return $self->{'text_hash'}->{'debian_manager'};
}

sub package_help
{
	my $self = shift;

return "dpkg";
}


1;

