#!/usr/bin/perl
package software::EmergeLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'software::EmergeLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# emerge-lib.pl
# Functions for gentoo package management

chop($self->{'system_arch'} = `uname -m`);
$self->{'pkg_dir'} = "/var/db/pkg";
$self->{'portage_bin'} = "/usr/lib/portage/bin";
$ENV{'TERM'} = "dumb";

$self->{'wide_install_options'} = 0;

$self->{'show_install_progress'} = 1;

$self->{'has_update_system'} = 1;

1;

1;
		return $self;
}

# list_packages([package]*)
# Fills the array %{$self->{'packages_hash'}} with all or listed packages
sub list_packages
{
	my $self = shift;

local $i = 0;
$self->open_execute_command(*LIST, "$self->{'portage_bin'}/pkglist", 1, 1);
while(<LIST>) {
	if (/^([^\/]+)\/([^0-9]+)-(\d\S+)$/ &&
	    !@_ || $self->indexof($2, @_) >= 0) {
		$self->{'packages_hash'}->{$i,'name'} = $2;
		$self->{'packages_hash'}->{$i,'class'} = $1;
		$self->{'packages_hash'}->{$i,'version'} = $3;
		$self->open_readfile(*BUILD, "$self->{'pkg_dir'}/$1/$2-$3/$2-$3.ebuild");
		while(<BUILD>) {
			if (/DESCRIPTION="([^"]+)"/ || /DESCRIPTION='([^']+)'/) {
				$self->{'packages_hash'}->{$i,'desc'} = $1;
				last;
				}
			}
		close(BUILD);
		$i++;
		}
	}
return $i;
}

# package_search(string, [allavailable])
# Searches the package database for packages matching some string and puts
# them into %{$self->{'packages_hash'}}
sub package_search
{
	my $self = shift;

local $n = 0;
local $qm = quotemeta($_[0]);
$self->open_execute_command(*SEARCH, "emerge search $qm", 1, 1);
while(<SEARCH>) {
	s/\r|\n//g;
	s/\033[^m]+m//g;
	if (/^\*\s+([^\/]+)\/(\S+)/) {
		$self->{'packages_hash'}->{$n,'name'} = $2;
		$self->{'packages_hash'}->{$n,'class'} = $1;
		$self->{'packages_hash'}->{$n,'missing'} = 0;
		}
	elsif (/version\s+Available:\s+(\S+)/i) {
		$self->{'packages_hash'}->{$n,'version'} = $1;
		}
	elsif (/version\s+Installed:\s+\[\s+Not/i && !$_[1]) {
		$self->{'packages_hash'}->{$n,'missing'} = 1;
		}
	elsif (/\s+Description:\s*(.*)/i) {
		$self->{'packages_hash'}->{$n,'desc'} = $1;
		local $nl = <SEARCH>;
		chop($nl);
		if ($nl =~ /\S/) {
			$self->{'packages_hash'}->{$n,'desc'} .= " " if ($self->{'packages_hash'}->{$n,'desc'});
			$self->{'packages_hash'}->{$n,'desc'} .= $nl;
			}
		$n++ if (!$self->{'packages_hash'}->{$n,'missing'} || $_[1]);
		}
	}
close(SEARCH);
return $n;
}

# package_info(package)
# Returns an array of package information in the order
#  name, class, description, arch, version, vendor, installtime
sub package_info
{
	my $self = shift;

local %packages;
local $n = $self->list_packages($_[0]);
$n || return ();
local @st = stat("$self->{'pkg_dir'}/$packages{0,'class'}/$packages{0,'name'}-$packages{0,'version'}");
return ( $packages{0,'name'}, $packages{0,'class'}, $packages{0,'desc'},
	 $self->{'system_arch'}, $packages{0,'version'}, "Gentoo", $self->make_date($st[9]) );
}

# is_package(file)
# Check if some file is a package file
sub is_package
{
	my $self = shift;

local $qm = quotemeta($_[0]);
local $out = $self->backquote_command("emerge --pretend $qm 2>&1", 1);
return $? ? 0 : 1;
}

# file_packages(file)
# Returns a list of all packages in the given file, in the form
#  package description
sub file_packages
{
	my $self = shift;

local @rv;
local $qm = quotemeta($_[0]);
$self->open_execute_command(*EMERGE, "emerge --pretend $qm", 1, 1);
while(<EMERGE>) {
	s/\r|\n//g;
	s/\033[^m]+m//g;
	if (/\s+[NRU]\s+\]\s+([^\/]+)\/([^0-9]+)\-(\d\S+)/) {
		push(@rv, $2);
		}
	}
close(EMERGE);
return @rv;
}

# install_options(file, package)
# Outputs HTML for choosing install options for some package
sub install_options
{
	my $self = shift;

print "<tr> <td><b>$self->{'text_hash'}->{'emerge_noreplace'}</b></td>\n";
print "<td><input type=radio name=noreplace value=0 checked> $self->{'text_hash'}->{'yes'}\n";
print "<input type=radio name=noreplace value=1> $self->{'text_hash'}->{'no'}</td> </tr>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'emerge_onlydeps'}</b></td>\n";
print "<td><input type=radio name=onlydeps value=1> $self->{'text_hash'}->{'yes'}\n";
print "<input type=radio name=onlydeps value=0 checked> $self->{'text_hash'}->{'no'}</td> </tr>\n";
}

# install_package(file, package, [&inputs], [show])
# Install the given package from the given file, using options from %{$self->{'in_hash'}}
sub install_package
{
	my $self = shift;

local $file = $_[0];
local $in = $_[2] ? $_[2] : \%{$self->{'in_hash'}};
local $cmd = "emerge";
$cmd .= " --noreplace" if ($self->{'in_hash'}->{'noreplace'});
$cmd .= " --onlydeps" if ($self->{'in_hash'}->{'onlydeps'});
$cmd .= " ".quotemeta($_[1]);
if ($_[3]) {
	$self->open_execute_command(*OUT, "$cmd 2>&1", 1);
	while(<OUT>) {
		print $self->html_escape($_);
		}
	close(OUT);
	return $? ? "Emerge error" : undef;
	}
else {
	local $out;
	$self->open_execute_command(*OUT, "$cmd 2>&1 | tail -10", 1);
	while(<OUT>) {
		$out .= $_;
		}
	close(OUT);
	return $? ? "<pre>$out</pre>" : undef;
	}
}

# check_files(package)
# Fills in the %{$self->{'files_hash'}} array with information about the files belonging
# to some package. Values in %{$self->{'files_hash'}} are  path type user group size error
sub check_files
{
	my $self = shift;

local $i = 0;
local (@files, %filesmap);
local %packages;
$self->list_packages($_[0]);
$self->open_readfile(*CONTENTS, "$self->{'pkg_dir'}/$packages{0,'class'}/$packages{0,'name'}-$packages{0,'version'}/CONTENTS");
while(<CONTENTS>) {
	s/\r|\n//g;
	local @l = split(/\s+/);
	$self->{'files_hash'}->{$i,'path'} = $l[1];
	$self->{'files_hash'}->{$i,'type'} = $l[0] eq 'dir' ? 1 :
			    $l[0] eq 'sym' ? 3 : 0;
	local $real = $self->translate_filename($l[1]);
	local @st = stat($real);
	$self->{'files_hash'}->{$i,'user'} = getpwuid($st[4]);
	$self->{'files_hash'}->{$i,'group'} = getgrgid($st[5]);
	$self->{'files_hash'}->{$i,'size'} = $st[7];
	if (!-e $l[1]) {
		$self->{'files_hash'}->{$i,'error'} = "Does not exist";
		}
	elsif ($l[0] eq 'sym') {
		$self->{'files_hash'}->{$i,'link'} = $l[3];
		local $lnk = readlink($real);
		$self->{'files_hash'}->{$i,'error'} = "Incorrect link" if ($l[3] ne $lnk);
		}
	elsif ($l[0] eq 'obj') {
		push(@files, $l[1]);
		$filesmap{$l[1]} = $i;
		$self->{'files_hash'}->{$i,'md5'} = $l[2];
		}
	$i++;
	}
close(CONTENTS);
if ($self->has_command("md5sum")) {
	$self->open_execute_command(*MD5, "md5sum ".join(" ", @files), 1, 1);
	while(<MD5>) {
		local ($md, $fn) = split(/\s+/);
		local $n = $filesmap{$fn};
		if ($md ne $self->{'files_hash'}->{$n,'md5'}) {
			$self->{'files_hash'}->{$n,'error'} = "Checksum failed";
			}
		}
	close(MD5);
	}
return $i;
}

# installed_file(file)
# Given a filename, fills %{$self->{'file_hash'}} with details of the given file and returns 1.
# If the file is not known to the package system, returns 0
# Usable values in %{$self->{'file_hash'}} are  path type user group mode size packages
sub installed_file
{
	my $self = shift;

local ($cf, $type, @packs);
local $real_dir = $self->translate_filename($self->{'pkg_dir'});
while($cf = <$real_dir/*/*/CONTENTS>) {
	open(FILE, $cf);
	while(<FILE>) {
		local @l = split(/\s+/);
		if ($l[1] eq $_[0]) {
			# Found it!
			$cf =~ /\/([^0-9\/]+)-(\d[^\s\/]+)\/CONTENTS$/;
			push(@packs, $1);
			$type = $l[0] if (!$type);
			}
		}
	close(FILE);
	}
return 0 if (!@packs);

local $real = $self->translate_filename($_[0]);
local @st = stat($real);
$self->{'file_hash'}->{'packages'} = join(' ', @packs);
$self->{'file_hash'}->{'path'} = $_[0];
$self->{'file_hash'}->{'user'} = getpwuid($st[4]);
$self->{'file_hash'}->{'group'} = getgrgid($st[5]);
$self->{'file_hash'}->{'mode'} = sprintf "%o", $st[2] & 07777;
$self->{'file_hash'}->{'size'} = $st[7];
$self->{'file_hash'}->{'link'} = readlink($real);
$self->{'file_hash'}->{'type'} = $type eq 'dir' ? 1 :
		$type eq 'sym' ? 3 : 0;
return 1;
}

# delete_package(package, [&options])
# Attempt to remove some package
sub delete_package
{
	my $self = shift;

local $out = $self->backquote_logged("emerge -u ".quotemeta($_[0])." 2>&1");
return $? ? "<pre>$out</pre>" : undef;
}

sub package_system
{
	my $self = shift;

return "Gentoo Ebuild";
}

sub package_help
{
	my $self = shift;

return "emerge";
}

# update_system_input()
# Returns HTML for entering a package to install
sub update_system_input
{
	my $self = shift;

return "$self->{'text_hash'}->{'emerge_input'} <input name=update size=20> <input type=button onClick='window.ifield = form.update; chooser = window.open(\"../$self->{'module_name'}/emerge_find.cgi\", \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=600,height=500\")' value=\"$self->{'text_hash'}->{'emerge_find'}\">";
}

# update_system_install([package])
# Install some package with emerge
sub update_system_install
{
	my $self = shift;

local $update = $_[0] || $self->{'in_hash'}->{'update'};
local $cmd = "emerge ".quotemeta($update);
local @rv;
print "<b>",$self->text('emerge_install', "<tt>$cmd</tt>"),"</b><p>\n";
print "<pre>\n";
$self->additional_log('exec', undef, $cmd);
$self->open_execute_command(*CMD, "$cmd 2>&1 </dev/null", 1);
while(<CMD>) {
	print $self->html_escape($_);
	if (/^\>\>\>\s+([^\/]+)\/([^0-9]+)-(\d\S+)\s+merged\./i) {
		push(@rv, $2);
		}
	}
close(CMD);
print "</pre>\n";
if ($?) { print "<b>$self->{'text_hash'}->{'emerge_failed'}</b><p>\n"; }
else { print "<b>$self->{'text_hash'}->{'emerge_ok'}</b><p>\n"; }
return @rv;
}


1;

