#!/usr/bin/perl
package software::FreebsdLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'software::FreebsdLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# bsd-lib.pl
# Functions for FreeBSD package management

use POSIX;
chop($self->{'system_arch'} = `uname -m`);
$self->{'package_dir'} = "/var/db/pkg";

1;
1;
		return $self;
}

# list_packages([package]*)
# Fills the array %{$self->{'packages_hash'}} with a list of all packages
sub list_packages
{
	my $self = shift;

local $i = 0;
local $arg = @_ ? join(" ", map { quotemeta($_) } @_) : "-a";
$self->open_execute_command(*PKGINFO, "pkg_info -I $arg", 1, 1);
while(<PKGINFO>) {
	if (/^(\S+)\s+(.*)/) {
		$self->{'packages_hash'}->{$i,'name'} = $1;
		$self->{'packages_hash'}->{$i,'class'} = "";
		$self->{'packages_hash'}->{$i,'desc'} = $2;
		$i++;
		}
	}
close(PKGINFO);
return $i;
}

# package_info(package)
# Returns an array of package information in the order
#  name, class, description, arch, version, vendor, installtime
sub package_info
{
	my $self = shift;

local $qm = quotemeta($_[0]);
local $out = $self->backquote_command("pkg_info $qm 2>&1", 1);
return () if ($?);
local @rv = ( $_[0] );
push(@rv, "");
push(@rv, $out =~ /Description:\n([\0-\177]*\S)/i ? $1 : $self->{'text_hash'}->{'bsd_unknown'});
push(@rv, $self->{'system_arch'});
push(@rv, $_[0] =~ /-([^\-]+)$/ ? $1 : $self->{'text_hash'}->{'bsd_unknown'});
push(@rv, "FreeBSD");
local @st = stat($self->translate_filename("$self->{'package_dir'}/$_[0]"));
push(@rv, @st ? ctime($st[9]) : $self->{'text_hash'}->{'bsd_unknown'});
return @rv;
}

# check_files(package)
# Fills in the %{$self->{'files_hash'}} array with information about the files belonging
# to some package. Values in %{$self->{'files_hash'}} are  path type user group mode size error
sub check_files
{
	my $self = shift;

local $i = 0;
local $file;
local $qm = quotemeta($_[0]);
$self->open_execute_command(*PKGINFO, "pkg_info -L $qm", 1, 1);
while($file = <PKGINFO>) {
	$file =~ s/\r|\n//g;
	next if ($file !~ /^\//);
	local $real = $self->translate_filename($file);
	local @st = stat($real);
	$self->{'files_hash'}->{$i,'path'} = $file;
	$self->{'files_hash'}->{$i,'type'} = -l $real ? 3 :
			    -d $real ? 1 : 0;
	$self->{'files_hash'}->{$i,'user'} = getpwuid($st[4]);
	$self->{'files_hash'}->{$i,'group'} = getgrgid($st[5]);
	$self->{'files_hash'}->{$i,'mode'} = sprintf "%o", $st[2] & 07777;
	$self->{'files_hash'}->{$i,'size'} = $st[7];
	$self->{'files_hash'}->{$i,'link'} = readlink($real);
	$i++;
	}
return $i;
}

# installed_file(file)
# Given a filename, fills %{$self->{'file_hash'}} with details of the given file and returns 1.
# If the file is not known to the package system, returns 0
# Usable values in %{$self->{'file_hash'}} are  path type user group mode size packages
sub installed_file
{
	my $self = shift;

local (%packages, $file, $i, @pkgin);
local $n = $self->list_packages();
for($i=0; $i<$n; $i++) {
	$self->open_execute_command(*PKGINFO, "pkg_info -L $packages{$i,'name'}", 1, 1);
	while($file = <PKGINFO>) {
		$file =~ s/\r|\n//g;
		if ($file eq $_[0]) {
			# found it
			push(@pkgin, $packages{$i,'name'});
			}
		}
	close(PKGINFO);
	}
if (@pkgin) {
	local $real = $self->translate_filename($_[0]);
	local @st = stat($real);
	$self->{'file_hash'}->{'path'} = $_[0];
	$self->{'file_hash'}->{'type'} = -l $real ? 3 :
			-d $real ? 1 : 0;
	$self->{'file_hash'}->{'user'} = getpwuid($st[4]);
	$self->{'file_hash'}->{'group'} = getgrgid($st[5]);
	$self->{'file_hash'}->{'mode'} = sprintf "%o", $st[2] & 07777;
	$self->{'file_hash'}->{'size'} = $st[7];
	$self->{'file_hash'}->{'link'} = readlink($real);
	$self->{'file_hash'}->{'packages'} = join(" ", @pkgin);
	return 1;
	}
else {
	return 0;
	}
}

# is_package(file)
sub is_package
{
	my $self = shift;

local $real = $self->translate_filename($_[0]);
local $qm = quotemeta($_[0]);
if (-d $_[0]) {
	# A directory .. see if it contains any tgz files
	opendir(DIR, $real);
	local @list = grep { /\.tgz$/ || /\.tbz$/ } readdir(DIR);
	closedir(DIR);
	return @list ? 1 : 0;
	}
elsif ($_[0] =~ /\*|\?/) {
	# a wildcard .. see what it matches
	local @list = glob($real);
	return @list ? 1 : 0;
	}
else {
	# just a normal file - see if it is a package
	local $cmd;
	foreach $cmd ('gunzip', 'bunzip2') {
		next if (!$self->has_command($cmd));
		local ($desc, $contents);
		$self->open_execute_command(*TAR, "$cmd -c $qm | tar tf -", 1, 1);
		while(<TAR>) {
			$desc++ if (/^\+DESC/);
			$contents++ if (/^\+CONTENTS/);
			}
		close(TAR);
		return 1 if ($desc && $contents);
		}
	return 0;
	}
}

# $self->file_packages(file)
# Returns a list of all packages in the given file, in the form
#  package description
sub file_packages
{
	my $self = shift;

local $real = $self->translate_filename($_[0]);
local $qm = quotemeta($_[0]);
if (-d $real) {
	# Scan directory for packages
	local ($f, @rv);
	opendir(DIR, $real);
	while($f = readdir(DIR)) {
		if ($f =~ /\.tgz$/i || $f =~ /\.tbz$/i) {
			local @pkg = $self->file_packages("$_[0]/$f");
			push(@rv, @pkg);
			}
		}
	closedir(DIR);
	return @rv;
	}
elsif ($real =~ /\*|\?/) {
	# Expand glob of packages
	# XXX won't work in translation
	local ($f, @rv);
	foreach $f (glob($real)) {
		local @pkg = $self->file_packages($f);
		push(@rv, @pkg);
		}
	return @rv;
	}
else {
	# Just one file
	local $cmd;
	foreach $cmd ('gunzip', 'bunzip2') {
		next if (!$self->has_command($cmd));
		local $temp = $self->transname();
		$self->make_dir($temp, 0700);
		local $rv = $self->execute_command("cd $temp && $cmd -c $qm | tar xf - +CONTENTS +COMMENT");
		if ($rv) {
			$self->unlink_file($temp);
			next;
			}
		local ($comment, $name);
		$self->open_readfile(*COMMENT, "$temp/+COMMENT");
		($comment = <COMMENT>) =~ s/\r|\n//g;
		close(COMMENT);
		$self->open_readfile(*CONTENTS, "$temp/+CONTENTS");
		while(<CONTENTS>) {
			$name = $1 if (/^\@name\s+(\S+)/);
			}
		close(CONTENTS);
		$self->unlink_file($temp);
		return ( "$name $comment" );
		}
	return ( );
	}
}

# install_options(file, package)
# Outputs HTML for choosing install options
sub install_options
{
	my $self = shift;

print "<tr> <td><b>$self->{'text_hash'}->{'bsd_scripts'}</b></td>\n";
print "<td><input type=radio name=scripts value=0 checked> $self->{'text_hash'}->{'yes'}\n";
print "<input type=radio name=scripts value=1> $self->{'text_hash'}->{'no'}</td> </tr>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'bsd_force'}</b></td>\n";
print "<td><input type=radio name=force value=1> $self->{'text_hash'}->{'yes'}\n";
print "<input type=radio name=force value=0 checked> $self->{'text_hash'}->{'no'}</td> </tr>\n";
}

# install_package(file, package)
# Installs the package in the given file, with options from %{$self->{'in_hash'}}
sub install_package
{
	my $self = shift;

local $in = $_[2] ? $_[2] : \%{$self->{'in_hash'}};
local $args = ($in->{"scripts"} ? " -I" : "").
	      ($in->{"force"} ? " -f" : "");
local $out = $self->backquote_logged("pkg_add $args $_[0] 2>&1");
if ($?) {
	return "<pre>$out</pre>";
	}
return undef;
}

# install_packages(file, [&inputs])
# Installs all the packages in the given file or glob
sub install_packages
{
	my $self = shift;

local $in = $_[2] ? $_[2] : \%{$self->{'in_hash'}};
local $args = ($in->{"scripts"} ? " -I" : "").
	      ($in->{"force"} ? " -f" : "");
local $file;
if (-d $_[0]) {
	$file = "$_[0]/*.tgz";
	}
else {
	$file = $_[0];
	}
local $out = $self->backquote_logged("pkg_add $args $file 2>&1");
if ($?) {
	return "<pre>$out</pre>";
	}
return undef;
}

# &install_remote_package_list(\@pkg_list)
# install all the packages from ftp://ftp.freebsd.org
sub install_remote_package_list
{
	my $self = shift;

 my $pkgs = $_[0];
 if (@{$pkgs}) 
 {
  $ENV{'FTP_PASSIVE_MODE'} = 1;
  print "<tr $self->{'tb'}><td colspan=2><b>$self->{'text_hash'}->{'info_installing'}</b></td></tr>\n";
  print "<tr $self->{'cb'}><td colspan=2><pre>";
  foreach my $i (@{$pkgs})
  {
	open(CMD, "pkg_add -r -v $i 2>&1 </dev/null |");
	while(<CMD>)
	{
    print $self->html_escape($_);
	}
	close(CMD);
  }
  $ENV{'FTP_PASSIVE_MODE'} = 0;
  
  # reload unix command hash
  my $out = $self->backquote_logged("csh -c rehash 2>&1");
  print $out if $out;
  print "</pre></td></tr>\n";
 }
 return undef;
}

# delete_package(package)
# Totally remove some package
sub delete_package
{
	my $self = shift;

local $out = $self->backquote_logged("pkg_delete $_[0] 2>&1");
if ($?) { return "<pre>$out</pre>"; }
return undef;
}

sub package_system
{
	my $self = shift;

return $self->text('bsd_manager', "FreeBSD");
}

sub package_help
{
	my $self = shift;

return "pkg_add pkg_info pkg_delete";
}


1;

