#!/usr/bin/perl
package software::MsiLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'software::MsiLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
#) msi-lib.pl
# Functions for Windows package management

$self->{'msi_package_logdir'} = "$self->{'module_config_directory'}/msis";
$self->{'sw_discovery_cmd'} = "$self->{'root_directory'}/SW_Discovery/SW_Discovery.exe";
$self->{'has_sw_discovery'} = -x $self->{'sw_discovery_cmd'};
$self->{'no_package_filesearch'} = 1;

1;

1;
		return $self;
}

# list_packages([package]*)
# Fills the array %{$self->{'packages_hash'}} with all or listed packages
sub list_packages
{
	my $self = shift;

if ($self->{'has_sw_discovery'}) {
	local %want = map { $_, 1 } @_;
	open(OUT, "$self->{'sw_discovery_cmd'} |");
	local $n = -1;
	while(<OUT>) {
		s/\r|\n//g;
		if (/ID\s*=\s*\{(\S+)\}\s*(\S*)/ ||
		    /ID\s*=\s*(.*)/) {
			$n++;
			$self->{'packages_hash'}->{$n,'name'} = $1;
			$self->{'packages_hash'}->{$n,'arch'} = 'x86';
			$self->{'packages_hash'}->{$n,'version'} = $2;
			if (/ID\s*=\s*\{(\S+)\}/) {
				$self->{'packages_hash'}->{$n,'code'} = $1;
				}
			else {
				$self->{'packages_hash'}->{$n,'nouninstall'} = 1;
				}
			$self->{'packages_hash'}->{$n,'nolist'} = 1;
			}
		elsif (/Package\s+Name\s*:\s*(.*)/) {
			if ($self->{'packages_hash'}->{$n,'code'}) {
				$self->{'packages_hash'}->{$n,'name'} = $1;
				}
			else {
				$self->{'packages_hash'}->{$n,'desc'} = $1;
				}
			}
		elsif (/Version\s*:\s*(.*)/) {
			$self->{'packages_hash'}->{$n,'version'} = $1;
			}
		elsif (/Publisher\s*:\s*(.*)/) {
			$self->{'packages_hash'}->{$n,'vendor'} = $1;
			$self->{'packages_hash'}->{$n,'class'} = $1;
			$self->{'packages_hash'}->{$n,'class'} =~ s/(http|https):\/\///g;
			}
		elsif (/Location\s*:\s*(\S.*)/) {
			$self->{'packages_hash'}->{$n,'desc'} ||= $1;
			local @st = stat($1);
			$self->{'packages_hash'}->{$n,'installed'} = $st[9];
			}
		}
	close(OUT);
	$n++;

	# Filter out un-wanted packages, or those with no name
	for(my $i=0; $i<$n; $i++) {
		if (%want && !$want{$self->{'packages_hash'}->{$i,'name'}} ||
		    !$self->{'packages_hash'}->{$i,'name'}) {
			if ($i != $n-1) {
				foreach my $f ('name', 'code', 'arch',
					       'version', 'class',
					       'vendor', 'installed') {
					$self->{'packages_hash'}->{$i,$f} = 
						$self->{'packages_hash'}->{$i+1,$f};
					}
				}
			$i--;
			$n--;
			}
		}

	return $n;
	}

# Query built-in list of those we know were installed
local @pkgs = @_;
@pkgs = $self->list_msi_packages() if (!@pkgs);
local $i = 0;
foreach my $p (@pkgs) {
	local $msi = $self->get_msi_package($p);
	$self->{'packages_hash'}->{$i,'name'} = $msi->{'name'};
	$self->{'packages_hash'}->{$i,'version'} = $msi->{'version'};
	$self->{'packages_hash'}->{$i,'desc'} = $msi->{'desc'};
	$self->{'packages_hash'}->{$i,'nolist'} = 0;
	$i++;
	}
return $i;
}

# list_msi_packages()
# Returns the names of installed MSI packages (using Webmin)
sub list_msi_packages
{
	my $self = shift;

opendir(DIR, $self->{'msi_package_logdir'});
local @rv = grep { $_ ne "." && $_ ne ".." } readdir(DIR);
closedir(DIR);
return @rv;
}

# get_msi_package(name)
# Returns a hash containing details of an MSI package
sub get_msi_package
{
	my $self = shift;

local ($pkg) = @_;
local %msi;
$self->read_file("$self->{'msi_package_logdir'}/".lc($pkg), \%msi);
$msi{'name'} = $pkg;
return \%msi;
}

# save_msi_package(&package)
# Updates the local details of an MSI package
sub save_msi_package
{
	my $self = shift;

local ($msi) = @_;
$self->make_dir($self->{'msi_package_logdir'}, 0700);
$self->write_file("$self->{'msi_package_logdir'}/".lc($msi->{'name'}), $msi);
}

# delete_msi_package(name)
# Removes the local details of an MSI package
sub delete_msi_package
{
	my $self = shift;

local ($name) = @_;
unlink("$self->{'msi_package_logdir'}/".lc($name));
}

# package_info(package, [version])
# Returns an array of package information in the order
#  name, class, description, arch, version, vendor, installtime,
#  no-uninstall, no-list
sub package_info
{
	my $self = shift;

local ($name, $ver) = @_;
if ($self->{'has_sw_discovery'}) {
	# Get from total list
	local %packages;
	local $n = $self->list_packages();
	for(my $i=0; $i<$n; $i++) {
		if ($packages{$i,'name'} eq $name &&
		    (!$ver || $ver eq $packages{$i,'version'})) {
			return ($packages{$i,'name'},
				$packages{$i,'vendor'},
				$packages{$i,'desc'},
				$packages{$i,'arch'},
				$packages{$i,'version'},
				$packages{$i,'vendor'},
				$packages{$i,'installed'} ?
				   $self->make_date($packages{$i,'installed'}) :
				   "Unknown",
				$packages{$i,'nouninstall'},
				$packages{$i,'nolist'},
				);
			}
		}
	return ( );
	}
else {
	# Query our savedd DB
	local $msi = $self->get_msi_package($name);
	return ( $msi->{'name'}, $msi->{'class'}, $msi->{'desc'},
		 $msi->{'arch'}, $msi->{'version'}, $msi->{'vendor'},
		 $self->make_date($msi->{'installed'}),
		 $msi->{'code'} ? 1 : 0,
		 0 );
	}
}

# is_package(file)
# Check if some file is a package file
sub is_package
{
	my $self = shift;

local ($file) = @_;
return $file =~ /\.msi$/i ? 1 : 0;
}

# file_packages(file)
# Returns a list of all packages in the given file, in the form
#  package-version description
# XXX how to get proper description?
sub file_packages
{
	my $self = shift;

local ($file) = @_;
if ($file =~ /([^\/\\]+)\.msi$/i) {
	local $suffix = $1;
	$suffix =~ s/_\d.*$//;	# For files named like apache_2.x.y...
	return $suffix;
	}
return ( );
}

# install_options(file, package)
# Outputs HTML for choosing install options for some package
sub install_options
{
	my $self = shift;

local ($file, $pkg) = @_;

print "<tr>\n";
print "<td><b>$self->{'text_hash'}->{'msi_users'}</b></td>\n";
print "<td>",$self->ui_radio("users", 2, [ [ 0, $self->{'text_hash'}->{'msi_users0'} ],
				     [ 1, $self->{'text_hash'}->{'msi_users1'} ],
				     [ 2, $self->{'text_hash'}->{'msi_users2'} ] ]),"</td>\n";
print "</tr>\n";
}

# install_package(file, package, [&inputs])
# Install the given package from the given file, using options from %in. Returns
# undef on success or an error message on failure.
sub install_package
{
	my $self = shift;

local ($file, $pkg, $in) = @_;

# Run the msiexec command
local $temp = $self->tempname();
$file =~ s/\//\\/g;
$temp =~ s/\//\\/g;
local @st = stat($file);
system("msiexec /i ".$self->quote_path($file)." /quiet /norestart ".
       "/l*vx ".$self->quote_path($temp)." >$self->{'null_file'}");
$self->wait_till_stopped_logging($temp);

# Read output from log file
local $ok;
local %msi = ( 'name' => $pkg,
	       'arch' => 'x86',			# Wrong!
	       'installed' => time() );
local $fc = 0;
open(OUT, $temp);
while(<OUT>) {
	s/\r|\n//g;
	s/\0//g;		# Strip unicode, primitively
	if (/Product:\s*(.*)\s\-\-.*success/i) {
		$ok = 1;
		}
	elsif (/ProductVersion\s*=\s*(.*)/) {
		$msi{'version'} = $1;
		}
	elsif (/ProductName\s*=\s*(.*)/) {
		$msi{'desc'} = $1;
		}
	elsif (/ProductCode\s*=\s*(.*)/) {
		$msi{'code'} = $1;
		}
	elsif (/Manufacturer\s*=\s*(.*)/) {
		$msi{'vendor'} = $1;
		}
	elsif (/Note:\s+\d+:\s+\d+\s+\d+:\s+([a-z]:.*)/) {
		$msi{'files_'.$fc} = $1;
		local @st = stat($1);
		$msi{'sizes_'.$fc} = $st[7];
		$fc++;
		}
	}
close(OUT);
if (!$ok) {
	return "MSI install failed!";
	}
$self->save_msi_package(\%msi);

# Make available to users, if requested
if ($in->{'users'}) {
	local $flag = $in->{'users'} == 1 ? "u" : "m";
	system("msiexec /j$flag ".$self->quote_path($file)." /quiet /norestart ".
	       ">$self->{'null_file'}");
	}

return undef;
}

# check_files(package, version)
# Fills in the %{$self->{'files_hash'}} array with information about the files belonging
# to some package. Values in %{$self->{'files_hash'}} are  path type user group size error
sub check_files
{
	my $self = shift;

local ($name, $ver) = @_;
local $msi = $self->get_msi_package($name);
local $i;
for($i=0; defined($msi->{'files_'.$i}); $i++) {
	$self->{'files_hash'}->{$i,'path'} = $msi->{'files_'.$i};
	local @st = stat($self->{'files_hash'}->{$i,'path'});
	$self->{'files_hash'}->{$i,'type'} = -d $self->{'files_hash'}->{$i,'path'} ? 1 : 0;
	$self->{'files_hash'}->{$i,'size'} = $msi->{'sizes_'.$i};
	if (!@st) {
		$self->{'files_hash'}->{$i,'error'} = $self->{'text_hash'}->{'msi_missing'};
		}
	elsif ($self->{'files_hash'}->{$i,'size'} ne $st[7]) {
		$self->{'files_hash'}->{$i,'error'} = $self->{'text_hash'}->{'msi_size'};
		}
	}
return $i;
}

# delete_options(package)
# Outputs HTML for package uninstall options
sub delete_options
{
	my $self = shift;

local ($name) = @_;
# None!
}

# delete_package(package, [&options], version)
# Attempt to remove some package
sub delete_package
{
	my $self = shift;

local ($name, $opts, $ver) = @_;

# Call the uninstall command
local $code;
if ($self->{'has_sw_discovery'}) {
	# Get code form package list
	local %packages;
	local $n = $self->list_packages();
	for(my $i=0; $i<$n; $i++) {
		if ($packages{$i,'name'} eq $name &&
		    !$ver || $ver eq $packages{$i,'version'}) {
			$code = $packages{$i,'code'};
			}
		}
	$code || $self->error("Couldn't find install code for $name");
	}
else {
	local $msi = $self->get_msi_package($name);
	$code = $msi->{'code'};
	}
local $temp = $self->tempname();
$temp =~ s/\//\\/g;
system("msiexec /x \"$code\" /quiet /norestart ".
       "/l*vx ".$self->quote_path($temp)." >$self->{'null_file'}");
$self->wait_till_stopped_logging($temp);

# Check log for success
local $ok = 0;
open(LOG, $temp);
while(<LOG>) {
	s/\r|\n//g;
	s/\0//g;		# Strip unicode, primitively
	if (/Product:\s*(.*)\s\-\-.*success/i) {
		$ok = 1;
		}
	}
close(LOG);
if (!$ok) {
	return "MSI uninstall failed";
	}

# Remove from local info
$self->delete_msi_package($name);

return undef;
}

sub package_system
{
	my $self = shift;

return "MSI";
}

sub package_help
{
	my $self = shift;

return "msi";
}

# wait_till_stopped_logging(file)
# Spin until some file has remained untouched for 5 seconds
sub wait_till_stopped_logging
{
	my $self = shift;

local ($file) = @_;
while(1) {
	local @before = stat($file);
	sleep(5);
	local @after = stat($file);
	last if ($before[9] == $after[9]);
	}
}


1;

