#!/usr/bin/perl
package software::OpenbsdLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'software::OpenbsdLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# openbsd-lib.pl
# Functions for OpenBSD package management

use POSIX;
chop($self->{'system_arch'} = `uname -m`);
$self->{'package_dir'} = "/var/db/pkg";

1;
1;
		return $self;
}

# list_packages([package]*)
# Fills the array %{$self->{'packages_hash'}} with a list of all packages
sub list_packages
{
	my $self = shift;

local $i = 0;
local $arg = @_ ? join(" ", map { quotemeta($_) } @_) : "-a";
$self->open_execute_command(*PKGINFO, "pkg_info -I $arg", 1, 1);
while(<PKGINFO>) {
	if (/^(\S+)\s+(.*)/) {
		$self->{'packages_hash'}->{$i,'name'} = $1;
		$self->{'packages_hash'}->{$i,'class'} = "";
		$self->{'packages_hash'}->{$i,'desc'} = $2;
		$i++;
		}
	}
close(PKGINFO);
return $i;
}

# package_info(package)
# Returns an array of package information in the order
#  name, class, description, arch, version, vendor, installtime
sub package_info
{
	my $self = shift;

local $qm = quotemeta($_[0]);
local $out = $self->backquote_command("pkg_info $_[0] 2>&1", 1);
return () if ($?);
local @rv = ( $_[0] );
push(@rv, "");
push(@rv, $out =~ /Description:\n([\0-\177]*\S)/i ? $1 : $self->{'text_hash'}->{'bsd_unknown'});
push(@rv, $self->{'system_arch'});
push(@rv, $_[0] =~ /-([^\-]+)$/ ? $1 : $self->{'text_hash'}->{'bsd_unknown'});
push(@rv, "OpenBSD");
local @st = stat($self->translate_filename("$self->{'package_dir'}/$_[0]"));
push(@rv, @st ? ctime($st[9]) : $self->{'text_hash'}->{'bsd_unknown'});
return @rv;
}

# check_files(package)
# Fills in the %{$self->{'files_hash'}} array with information about the files belonging
# to some package. Values in %{$self->{'files_hash'}} are  path type user group mode size error
sub check_files
{
	my $self = shift;

local $i = 0;
local $file;
local $qm = quotemeta($_[0]);
$self->open_execute_command(*PKGINFO, "pkg_info -L $qm", 1, 1);
while($file = <PKGINFO>) {
	$file =~ s/\r|\n//g;
	next if ($file !~ /^\//);
	local $real = $self->translate_filename($file);
	local @st = stat($real);
	$self->{'files_hash'}->{$i,'path'} = $file;
	$self->{'files_hash'}->{$i,'type'} = -l $real ? 3 :
			    -d $real ? 1 : 0;
	$self->{'files_hash'}->{$i,'user'} = getpwuid($st[4]);
	$self->{'files_hash'}->{$i,'group'} = getgrgid($st[5]);
	$self->{'files_hash'}->{$i,'mode'} = sprintf "%o", $st[2] & 07777;
	$self->{'files_hash'}->{$i,'size'} = $st[7];
	$self->{'files_hash'}->{$i,'link'} = readlink($real);
	$i++;
	}
return $i;
}

# installed_file(file)
# Given a filename, fills %{$self->{'file_hash'}} with details of the given file and returns 1.
# If the file is not known to the package system, returns 0
# Usable values in %{$self->{'file_hash'}} are  path type user group mode size packages
sub installed_file
{
	my $self = shift;

local (%packages, $file, $i, @pkgin);
local $n = $self->list_packages();
for($i=0; $i<$n; $i++) {
	$self->open_execute_command(*PKGINFO, "pkg_info -L $packages{$i,'name'}", 1,1);
	while($file = <PKGINFO>) {
		$file =~ s/\r|\n//g;
		if ($file eq $_[0]) {
			# found it
			push(@pkgin, $packages{$i,'name'});
			}
		}
	close(PKGINFO);
	}
if (@pkgin) {
	local $real = $self->translate_filename($_[0]);
	local @st = stat($real);
	$self->{'file_hash'}->{'path'} = $_[0];
	$self->{'file_hash'}->{'type'} = -l $real ? 3 :
			-d $real ? 1 : 0;
	$self->{'file_hash'}->{'user'} = getpwuid($st[4]);
	$self->{'file_hash'}->{'group'} = getgrgid($st[5]);
	$self->{'file_hash'}->{'mode'} = sprintf "%o", $st[2] & 07777;
	$self->{'file_hash'}->{'size'} = $st[7];
	$self->{'file_hash'}->{'link'} = readlink($real);
	$self->{'file_hash'}->{'packages'} = join(" ", @pkgin);
	return 1;
	}
else {
	return 0;
	}
}

# is_package(file)
sub is_package
{
	my $self = shift;

local ($desc, $contents);
local $qm = quotemeta($_[0]);
$self->open_execute_command(*TAR, "gunzip -c $qm | tar tf -", 1, 1);
while(<TAR>) {
	$desc++ if (/^\+DESC/);
	$contents++ if (/^\+CONTENTS/);
	}
close(TAR);
return $desc && $contents;
}

# file_packages(file)
# Returns a list of all packages in the given file, in the form
#  package description
sub file_packages
{
	my $self = shift;

local $temp = $self->transname();
$self->make_dir($temp, 0700);
local $qm = quotemeta($_[0]);
$self->execute_command("cd $temp && gunzip -c $qm | tar xf - +CONTENTS +COMMENT");
local ($comment, $name);
$self->open_readfile(*COMMENT, "$temp/+COMMENT");
($comment = <COMMENT>) =~ s/\r|\n//g;
close(COMMENT);
$self->open_readfile(*CONTENTS, "$temp/+CONTENTS");
while(<CONTENTS>) {
	$name = $1 if (/^\@name\s+(\S+)/);
	}
close(CONTENTS);
$self->unlink_file($temp);
return ( "$name $comment" );
}

# install_options(file, package)
# Outputs HTML for choosing install options
sub install_options
{
	my $self = shift;

print "<tr> <td><b>$self->{'text_hash'}->{'bsd_scripts'}</b></td>\n";
print "<td><input type=radio name=scripts value=0 checked> $self->{'text_hash'}->{'yes'}\n";
print "<input type=radio name=scripts value=1> $self->{'text_hash'}->{'no'}</td>\n";

print "<td><b>$self->{'text_hash'}->{'bsd_force'}</b></td>\n";
print "<td><input type=radio name=force value=1> $self->{'text_hash'}->{'yes'}\n";
print "<input type=radio name=force value=0 checked> $self->{'text_hash'}->{'no'}</td> </tr>\n";
}

# install_package(file, package)
# Installs the package in the given file, with options from %{$self->{'in_hash'}}
sub install_package
{
	my $self = shift;

local $in = $_[2] ? $_[2] : \%{$self->{'in_hash'}};
local $args = ($in->{"scripts"} ? " -I" : "").
	      ($in->{"force"} ? " -f" : "");
local $out = $self->backquote_logged("pkg_add $args $_[0] 2>&1");
if ($?) {
	return "<pre>$out</pre>";
	}
return undef;
}

# delete_package(package)
# Totally remove some package
sub delete_package
{
	my $self = shift;

local $out = $self->backquote_logged("pkg_delete $_[0] 2>&1");
if ($?) { return "<pre>$out</pre>"; }
return undef;
}

sub package_system
{
	my $self = shift;

return $self->text('bsd_manager', "OpenBSD");
}

sub package_help
{
	my $self = shift;

return "pkg_add pkg_info pkg_delete";
}


1;

