#!/usr/bin/perl
package software::RpmLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'software::RpmLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# rpm-lib.pl
# Functions for redhat linux package management

$self->{'wide_install_options'} = 1;

%{$self->{'etype_hash'}} = (	"5", $self->{'text_hash'}->{'rpm_md5'},	"S", $self->{'text_hash'}->{'rpm_fsize'},
		"L", $self->{'text_hash'}->{'rpm_sym'},	"T", $self->{'text_hash'}->{'rpm_mtime'},
		"D", $self->{'text_hash'}->{'rpm_dev'},	"U", $self->{'text_hash'}->{'rpm_user'},
		"M", $self->{'text_hash'}->{'rpm_perm'},	"G", $self->{'text_hash'}->{'rpm_group'} );

$self->{'has_search_system'} = 1;

1;

1;
		return $self;
}

# list_packages([package]*)
# Fills the array %{$self->{'packages_hash'}} with all or listed packages
sub list_packages
{
	my $self = shift;

local($i, $list); $i = 0;
$list = @_ ? join(' ', map { quotemeta($_) } @_) : "-a";
$self->open_execute_command(*RPM, "rpm -q $list --queryformat \"%{NAME}\\n%{VERSION}-%{RELEASE}\\n%{EPOCH}\\n%{GROUP}\\n%{SUMMARY}\\n\\n\"", 1, 1);
while($self->{'packages_hash'}->{$i,'name'} = <RPM>) {
	chop($self->{'packages_hash'}->{$i,'name'});
	next if ($self->{'packages_hash'}->{$i,'name'} eq 'gpg-pubkey');
	chop($self->{'packages_hash'}->{$i,'version'} = <RPM>);
	chop($self->{'packages_hash'}->{$i,'epoch'} = <RPM>);
	$self->{'packages_hash'}->{$i,'epoch'} = undef if ($self->{'packages_hash'}->{$i,'epoch'} eq '(none)');
	chop($self->{'packages_hash'}->{$i,'class'} = <RPM>);
	while(<RPM>) {
		s/\r|\n/ /g;
		last if (!/\S/);
		$self->{'packages_hash'}->{$i,'desc'} .= $_;
		}
	$i++;
	}
close(RPM);
return 0 if ($?);	# couldn't find the package
return $i;
}

# package_info(package, [version])
# Returns an array of package information in the order
#  name, class, description, arch, version, vendor, installtime
sub package_info
{
	my $self = shift;

local(@rv, @tmp, $d);
local $n = $_[1] ? "$_[0]-$_[1]" : $_[0];
$self->open_execute_command(*RPM, "rpm -q $n --queryformat \"%{NAME}\\n%{GROUP}\\n%{ARCH}\\n%{VERSION}-%{RELEASE}\\n%{VENDOR}\\n%{INSTALLTIME}\\n\" 2>/dev/null", 1, 1);
@tmp = <RPM>;
chop(@tmp);
if (!@tmp) { return (); }
close(RPM);
$self->open_execute_command(*RPM, "rpm -q $n --queryformat \"%{DESCRIPTION}\"", 1, 1);
while(<RPM>) { $d .= $_; }
close(RPM);
return ($tmp[0], $tmp[1], $d, $tmp[2], $tmp[3], $tmp[4], $self->make_date($tmp[5]));
}

# is_package(file)
# Check if some file is a package file
sub is_package
{
	my $self = shift;

local($out);
local $real = $self->translate_filename($_[0]);
if (-d $real) {
	# a directory .. see if it contains any .rpm files
	opendir(DIR, $real);
	local @list = grep { /\.rpm$/ } readdir(DIR);
	closedir(DIR);
	return @list ? 1 : 0;
	}
elsif ($real =~ /\*|\?/) {
	# a wildcard .. see what it matches
	local @list = glob($real);
	return @list ? 1 : 0;
	}
else {
	# just a normal file .. check if it is an RPM and not an SRPM
	local $qm = quotemeta($_[0]);
	$out = $self->backquote_command("rpm -q -p $qm 2>&1", 1);
	if ($out =~ /does not appear|No such file|with major numbers|not an rpm/i || $?) {
		return 0;
		}
	$self->open_execute_command(*OUT, "rpm -q -p -l $qm 2>&1", 1, 1);
	while(<OUT>) {
		return 0 if (/^([^\/\s]+)\.spec$/);
		}
	close(OUT);
	return 1;
	}
}

# file_packages(file)
# Returns a list of all packages in the given file, in the form
#  package-version description
sub file_packages
{
	my $self = shift;

local $real = $self->translate_filename($_[0]);
local $qm = quotemeta($_[0]);
if (-d $real) {
	local @rv;
	$self->open_execute_command(*RPM, "cd $qm ; rpm -q -p *.rpm --queryformat \"%{NAME} %{SUMMARY}\\n\" 2>&1", 1, 1);
	while(<RPM>) {
		chop;
		push(@rv, $_) if (!/does not appear|query of.*failed|warning:/);
		}
	close(RPM);
	return @rv;
	}
elsif ($_[0] =~ /\*|\?/) {
	local @rv;
	$self->open_execute_command(*RPM, "rpm -q -p $_[0] --queryformat \"%{NAME} %{SUMMARY}\\n\" 2>&1", 1);
	while(<RPM>) {
		chop;
		push(@rv, $_) if (!/does not appear|query of.*failed|warning:/);
		}
	close(RPM);
	return @rv;
	}
else {
	local($out);
	$out = $self->backquote_command("rpm -q -p $qm --queryformat \"%{NAME} %{SUMMARY}\\n\" 2>&1", 1);
	$out =~ s/warning:.*\n//;
	$out =~ s/\n//g;
	return ($out);
	}
}

# install_options(file, package)
# Outputs HTML for choosing install options for some package
sub install_options
{
	my $self = shift;

print "<tr>\n";
print $self->yesno_input($self->{'text_hash'}->{'rpm_upgrade'}, "upgrade", 1, 0, 1);
print $self->yesno_input($self->{'text_hash'}->{'rpm_replacepkgs'}, "replacepkgs", 1, 0);
print "</tr>\n";

print "<tr>\n";
print $self->yesno_input($self->{'text_hash'}->{'rpm_nodeps'}, "nodeps", 1, 0);
print $self->yesno_input($self->{'text_hash'}->{'rpm_oldpackage'}, "oldpackage", 1, 0);
print "</tr>\n";

print "<tr>\n";
print $self->yesno_input($self->{'text_hash'}->{'rpm_noscripts'}, "noscripts", 0, 1);
print $self->yesno_input($self->{'text_hash'}->{'rpm_excludedocs'}, "excludedocs", 0, 1);
print "</tr>\n";

print "<tr>\n";
print $self->yesno_input($self->{'text_hash'}->{'rpm_notriggers'}, "notriggers", 0, 1);
print $self->yesno_input($self->{'text_hash'}->{'rpm_ignoresize'}, "ignoresize", 0, 1);
print "</tr>\n";

print "<tr>\n";
print $self->yesno_input($self->{'text_hash'}->{'rpm_replacefiles'}, "replacefiles", 1, 0);
print "<td align=right>",$self->hlink("<b>$self->{'text_hash'}->{'rpm_root'}</b>","root"),"</td>\n";
print "<td><input name=root size=25 value=\"/\">\n";
print $self->file_chooser_button("root", 1),"</td> </tr>\n";
}

sub yesno_input
{
	my $self = shift;

local $cy = $_[2] ^ $_[4] ? "" : "checked";
local $cn = $_[3] ^ $_[4] ? "" : "checked";
return  "<td align=right>".$self->hlink("<b>$_[0]</b>", $_[1])."</td> <td>\n".
	"<input type=radio name=$_[1] value=$_[2] $cy> $self->{'text_hash'}->{'yes'}\n".
	"<input type=radio name=$_[1] value=$_[3] $cn> $self->{'text_hash'}->{'no'}</td>\n";
}

# install_package(file, package, [&inputs])
# Install the given package from the given file, using options from %{$self->{'in_hash'}}
sub install_package
{
	my $self = shift;

local $file = $_[0];
local $real = $self->translate_filename($file);
local $in = $_[2] ? $_[2] : \%{$self->{'in_hash'}};
local $opts;
foreach $o ('oldpackage', 'replacefiles', 'replacepkgs', 'noscripts',
	    'excludedocs', 'nodeps', 'upgrade', 'notriggers', 'ignoresize') {
	if ($in->{$o}) { $opts .= " --$o"; }
	}
if ($in->{'root'} =~ /^\/.+/) {
	if (!(-d $self->{'in_hash'}->{'root'})) {
		return $self->text('rpm_eroot', $in->{'root'});
		}
	$opts .= " --root $in->{'root'}";
	}
if (-d $real) {
	# Find the package in the directory
	local ($f, $out);
	opendir(DIR, $real);
	while($f = readdir(DIR)) {
		next if ($f !~ /\.rpm$/);
		$out = $self->backquote_command("rpm -q -p $file/$f --queryformat \"%{NAME}\\n\" 2>&1", 1);
		$out =~ s/warning:.*\n//;
		$out =~ s/\n//;
		if ($out eq $_[1]) {
			$file = "$file/$f";
			last;
			}
		}
	closedir(DIR);
	$self->error($self->text('rpm_erpm', $_[1], $out)) if ($file eq $_[0]);
	}
elsif ($file =~ /\*|\?/) {
	# Find the package in the glob
	# XXX won't work when translation is in effect
	local ($f, $out);
	foreach $f (glob($real)) {
		$out = $self->backquote_command("rpm -q -p $f --queryformat \"%{NAME}\\n\" 2>&1", 1);
		$out =~ s/warning:.*\n//;
		$out =~ s/\n//;
		if ($out eq $_[1]) {
			$file = $f;
			last;
			}
		}
	$self->error($self->text('rpm_erpm', $_[1], $out)) if ($file eq $_[0]);
	}
local $temp = $self->transname();
local $rv = $self->system_logged("rpm -i $opts ".quotemeta($file)." >$temp 2>&1");
local $out = $self->backquote_command("cat $temp");
$out =~ s/warning:.*\n//;
$self->unlink_file($temp);
if ($rv) {
	return "<pre>$out</pre>";
	}
return undef;
}

# install_packages(file, [&inputs])
# Installs all the packages in the given file or glob
sub install_packages
{
	my $self = shift;

local $file = $_[0];
local $in = $_[1] ? $_[1] : \%{$self->{'in_hash'}};
local $opts;
foreach $o ('oldpackage', 'replacefiles', 'replacepkgs', 'noscripts',
	    'excludedocs', 'nodeps', 'upgrade', 'notriggers', 'ignoresize') {
	if ($in->{$o}) { $opts .= " --$o"; }
	}
if ($in->{'root'} =~ /^\/.+/) {
	if (!(-d $self->{'in_hash'}->{'root'})) {
		return $self->text('rpm_eroot', $in->{'root'});
		}
	$opts .= " --root $in->{'root'}";
	}
if (-d $self->translate_filename($file)) {
	# Install everything in a directory
	$file = "$file/*.rpm";
	}
else {
	# Install packages matching a glob (no need for any special action)
	}
local $temp = $self->transname();
local $rv = $self->system_logged("rpm -i $opts $file >$temp 2>&1");
local $out = $self->backquote_command("cat $temp");
$out =~ s/warning:.*\n//;
unlink($temp);
if ($rv) {
	return "<pre>$out</pre>";
	}
return undef;
}

# check_files(package, version)
# Fills in the %{$self->{'files_hash'}} array with information about the files belonging
# to some package. Values in %{$self->{'files_hash'}} are  path type user group size error
sub check_files
{
	my $self = shift;

local($i, $_, @w, %errs, $epath); $i = 0;
local $n = $_[1] ? "$_[0]-$_[1]" : $_[0];
local $qn = quotemeta($n);
$self->open_execute_command(*RPM, "rpm -V $qn", 1, 1);
while(<RPM>) {
	/^(.{8}) (.) (.*)$/;
	if ($1 eq "missing ") {
		$errs{$3} = $self->{'text_hash'}->{'rpm_missing'};
		}
	else {
		$epath = $3;
		@w = grep { $_ ne "." } split(//, $1);
		$errs{$epath} =
			join("\n", map { $self->text('rpm_checkfail', $self->{'etype_hash'}->{$_}) } @w);
		}
	}
close(RPM);
$self->open_execute_command(*RPM, "rpm -q $qn -l --dump", 1, 1);
while(<RPM>) {
	chop;
	@w = split(/ /);
	$self->{'files_hash'}->{$i,'path'} = $w[0];
	if ($w[10] ne "X") { $self->{'files_hash'}->{$i,'link'} = $w[10]; }
	$self->{'files_hash'}->{$i,'type'} = $w[10] ne "X" ? 3 :
			    (-d $self->translate_filename($w[0])) ? 1 :
			    $w[7] ? 5 : 0;
	$self->{'files_hash'}->{$i,'user'} = $w[5];
	$self->{'files_hash'}->{$i,'group'} = $w[6];
	$self->{'files_hash'}->{$i,'size'} = $w[1];
	$self->{'files_hash'}->{$i,'error'} = $w[7] ? "" : $errs{$w[0]};
	$i++;
	}
close(RPM);
return $i;
}

# installed_file(file)
# Given a filename, fills %{$self->{'file_hash'}} with details of the given file and returns 1.
# If the file is not known to the package system, returns 0
# Usable values in %{$self->{'file_hash'}} are  path type user group mode size packages
sub installed_file
{
	my $self = shift;

local($pkg, @w, $_, @pkgs, @vers);
undef(%{$self->{'file_hash'}});
local $qm = quotemeta($_[0]);
$pkg = $self->backquote_command("rpm -q -f $qm --queryformat \"%{NAME}\\n\" 2>&1", 1);
if ($pkg =~ /not owned/ || $?) { return 0; }
@pkgs = split(/\n/, $pkg);
$pkg = $self->backquote_command("rpm -q -f $qm --queryformat \"%{VERSION}-%{RELEASE}\\n\" 2>&1", 1);
@vers = split(/\n/, $pkg);
$self->open_execute_command(*RPM, "rpm -q $pkgs[0] -l --dump", 1, 1);
while(<RPM>) {
	chop;
	@w = split(/ /);
	if ($w[0] eq $_[0]) {
		$self->{'file_hash'}->{'packages'} = join(' ', @pkgs);
		$self->{'file_hash'}->{'versions'} = join(' ', @vers);
		$self->{'file_hash'}->{'path'} = $w[0];
		if ($w[10] ne "X") { $self->{'files_hash'}->{$self->{'i'},'link'} = $w[10]; }
		$self->{'file_hash'}->{'type'} = $w[10] ne "X" ? 3 :
				(-d $self->translate_filename($w[0])) ? 1 :
				$w[7] ? 5 : 0;
		$self->{'file_hash'}->{'user'} = $w[5];
		$self->{'file_hash'}->{'group'} = $w[6];
		$self->{'file_hash'}->{'mode'} = substr($w[4], -4);
		$self->{'file_hash'}->{'size'} = $w[1];
		last;
		}
	}
close(RPM);
return 1;
}

# delete_options(package)
# Outputs HTML for package uninstall options
sub delete_options
{
	my $self = shift;

print "<b>$self->{'text_hash'}->{'delete_nodeps'}</b>\n";
print "<input type=radio name=nodeps value=1> $self->{'text_hash'}->{'yes'}\n";
print "<input type=radio name=nodeps value=0 checked> $self->{'text_hash'}->{'no'}<br>\n";

print "<b>$self->{'text_hash'}->{'delete_noscripts'}</b>\n";
print "<input type=radio name=noscripts value=1> $self->{'text_hash'}->{'yes'}\n";
print "<input type=radio name=noscripts value=0 checked> $self->{'text_hash'}->{'no'}<br>\n";
}

# delete_package(package, [&options], version)
# Attempt to remove some package
sub delete_package
{
	my $self = shift;

local $opts;
$opts .= $_[1]->{'nodeps'} ? "--nodeps " : "";
$opts .= $_[1]->{'noscripts'} ? "--noscripts " : "";
local $n = $_[2] ? "$_[0]-$_[2]" : $_[0];
local $qm = quotemeta($n);
local $out = $self->backquote_logged("rpm -e $opts $qm 2>&1");
if ($? || $out =~ /error:/) { return "<pre>$out</pre>"; }
return undef;
}

# delete_packages(&packages, [&options], &versions)
# Attempt to remove multiple packages at once
sub delete_packages
{
	my $self = shift;

local $opts;
$opts .= $_[1]->{'nodeps'} ? "--nodeps " : "";
$opts .= $_[1]->{'noscripts'} ? "--noscripts " : "";
local $cmd = "rpm -e $opts";
local $i;
for($i=0; $i<@{$_[0]}; $i++) {
	if ($_[2]->[$i]) {
		$cmd .= " ".quotemeta($_[0]->[$i]."-".$_[2]->[$i]);
		}
	else {
		$cmd .= " ".quotemeta($_[0]->[$i]);
		}
	}
local $out = $self->backquote_logged("$cmd 2>&1");
if ($? || $out =~ /error:/) { return "<pre>$out</pre>"; }
return undef;
}

sub package_system
{
	my $self = shift;

return "RPM";
}

sub package_help
{
	my $self = shift;

return "rpm";
}

sub search_system_input
{
	my $self = shift;

print "<input type=button onClick='window.ifield = document.forms[2].url; chooser = window.open(\"rpmfind.cgi\", \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=600,height=500\")' value=\"$self->{'text_hash'}->{'rpm_find'}\">";
}


1;

