#!/usr/bin/perl
package software::SlackwareLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'software::SlackwareLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# slackware-lib.pl
# Functions for slackware package management

$self->{'package_dir'} = "/var/log/packages";
%{$self->{'class_map_hash'}} = (	'a',	'Base Linux system',
		'ap',	'Applications',
		'd',	'Program development',
		'des',	'Crypt library',
		'e',	'GNU Emacs',
		'f',	'FAQs and documentation',
		'gtk',	'GTK+ and GNOME programs',
		'k',	'Linux kernel source',
		'kde',	'KDE desktop and programs',
		'n',	'Networking',
		't',	'TeX',
		'tcl',	'TcL/Tk',
		'xap',	'X applications',
		'xd',	'X server development',
		'xv',	'XView programs',
		'y',	'Games' );
use POSIX;
chop($self->{'system_arch'} = `uname -m`);

1;

1;
		return $self;
}

# list_packages([package]*)
# Fills the array %{$self->{'packages_hash'}} with a list of all packages
sub list_packages
{
	my $self = shift;

local ($i, $f, @list);
opendir(DIR, $self->translate_filename($self->{'package_dir'}));
local @list = @_ ? @_ : grep { !/^\./ } readdir(DIR);
$i = 0;
foreach $f (@list) {
	$self->{'packages_hash'}->{$i,'name'} = $f;
	$self->{'packages_hash'}->{$i,'class'} = $self->{'text_hash'}->{'slack_unclass'};
	$self->open_tempfile(PKG, "$self->{'package_dir'}/$f");
	while(<PKG>) {
		if (/^PACKAGE LOCATION:\s+disk([a-z]+)\d+/i) {
			$self->{'packages_hash'}->{$i,'class'} = $self->{'class_map_hash'}->{$1};
			}
		elsif (/^PACKAGE DESCRIPTION:/i) {
			local $desc = <PKG>;
			$desc =~ s/^\S+:\s+//;
			$desc =~ s/\n//;
			$self->{'packages_hash'}->{$i,'desc'} = $desc;
			}
		}
	close(PKG);
	$i++;
	}
closedir(DIR);
return $i;
}

# package_info(package)
# Returns an array of package information in the order
#  name, class, description, arch, version, vendor, installtime
sub package_info
{
	my $self = shift;

local @rv = ( $_[0], $self->{'text_hash'}->{'slack_unclass'}, $self->{'text_hash'}->{'slack_unknown'},
	      $self->{'system_arch'}, $self->{'text_hash'}->{'slack_unknown'}, "Slackware" );
local @st = stat($self->translate_filename("$self->{'package_dir'}/$_[0]"));
$rv[6] = ctime($st[9]);
$self->open_readfile(*PKG, "$self->{'package_dir'}/$_[0]");
while(<PKG>) {
	if (/^PACKAGE LOCATION:\s+disk([a-z]+)\d+/i) {
		$rv[1] = $self->{'class_map_hash'}->{$1};
		}
	elsif (/^PACKAGE DESCRIPTION:/i) {
		$rv[2] = "";
		while(<PKG>) {
			last if (/^FILE LIST/i);
			s/^\S+: *//;
			if (!$rv[2] && /([0-9][0-9\.]*)/) {
				$rv[4] = $1;
				}
			$rv[2] .= $_;
			}
		$rv[2] =~ s/\s+$//;
		}
	}
close(PKG);
return @rv;
}

# check_files(package)
# Fills in the %{$self->{'files_hash'}} array with information about the files belonging
# to some package. Values in %{$self->{'files_hash'}} are  path type user group mode size error
sub check_files
{
	my $self = shift;

local $i = 0;
local $file;
$self->open_readfile(*PKG, "$self->{'package_dir'}/$_[0]");
while(<PKG>) {
	last if (/^FILE LIST:/i);
	}
while($file = <PKG>) {
	$file =~ s/\r|\n//g;
	next if ($file eq "./");
	$file = '/'.$file;
	local $real = $self->translate_filename($file);
	$self->{'files_hash'}->{$i,'path'} = $file;
	local @st = stat($real);
	if (@st) {
		$self->{'files_hash'}->{$i,'type'} = -l $real ? 3 :
				    -d $real ? 1 : 0;
		$self->{'files_hash'}->{$i,'user'} = getpwuid($st[4]);
		$self->{'files_hash'}->{$i,'group'} = getgrgid($st[5]);
		$self->{'files_hash'}->{$i,'mode'} = sprintf "%o", $st[2] & 07777;
		$self->{'files_hash'}->{$i,'size'} = $st[7];
		$self->{'files_hash'}->{$i,'link'} = readlink($file);
		}
	else {
		$self->{'files_hash'}->{$i,'type'} = $file =~ /\// ? 1 : 0;
		$self->{'files_hash'}->{$i,'user'} = $self->{'files_hash'}->{$i,'group'} =
		 $self->{'files_hash'}->{$i,'mode'} = $self->{'files_hash'}->{$i,'size'} = $self->{'text_hash'}->{'slack_unknown'};
		$self->{'files_hash'}->{$i,'error'} = $self->{'text_hash'}->{'slack_missing'};
		}
	$i++;
	}
return $i;
}

# installed_file(file)
# Given a filename, fills %{$self->{'file_hash'}} with details of the given file and returns 1.
# If the file is not known to the package system, returns 0
# Usable values in %{$self->{'file_hash'}} are  path type user group mode size packages
sub installed_file
{
	my $self = shift;

local ($f, $file, @pkgin);
opendir(DIR, $self->translate_filename($self->{'package_dir'}));
while($f = readdir(DIR)) {
	next if ($f =~ /^\./);
	$self->open_readfile(*PKG, "$self->{'package_dir'}/$f");
	while(<PKG>) {
		last if (/^FILE LIST:/);
		}
	while($file = <PKG>) {
		next if ($file eq "./");
		$file =~ s/[\/\r\n]+$//;
		$file = '/'.$file;
		if ($_[0] eq $file) {
			# found it!
			push(@pkgin, $f);
			last;
			}
		}
	close(PKG);
	}
closedir(DIR);
if (@pkgin) {
	local $real = $self->translate_filename($_[0]);
	local @st = stat($real);
	$self->{'file_hash'}->{'path'} = $_[0];
	$self->{'file_hash'}->{'type'} = -l $real ? 3 :
			-d $real ? 1 : 0;
	$self->{'file_hash'}->{'user'} = getpwuid($st[4]);
	$self->{'file_hash'}->{'group'} = getgrgid($st[5]);
	$self->{'file_hash'}->{'mode'} = sprintf "%o", $st[2] & 07777;
	$self->{'file_hash'}->{'size'} = $st[7];
	$self->{'file_hash'}->{'link'} = readlink($real);
	$self->{'file_hash'}->{'packages'} = join(" ", @pkgin);
	return 1;
	}
else {
	return 0;
	}
}

# is_package(file)
sub is_package
{
	my $self = shift;

local $count;
local $qm = quotemeta($_[0]);
$self->open_execute_command(*TAR, "gunzip -c $qm | tar tf - 2>&1", 1, 1);
while(<TAR>) {
	$count++ if (/^[^\/\s]\S+/);
	}
close(TAR);
return $count < 2 ? 0 : 1;
}

# file_packages(file)
# Returns a list of all packages in the given file, in the form
#  package description
sub file_packages
{
	my $self = shift;

if ($_[0] !~ /^(.*)\/(([^\/]+)(\.tgz|\.tar\.gz))$/) {
	return "$_[0] $self->{'text_hash'}->{'slack_unknown'}";
	}
local ($dir, $file, $base) = ($1, $2, $3);
local $diskfile;
opendir(DIR, $self->translate_filename($dir));
while($self->{'f'} = readdir(DIR)) {
	if ($self->{'f'} =~ /^disk\S+\d+$/ || $self->{'f'} eq 'package_descriptions') {
		# found the slackware disk file
		$diskfile = "$dir/$self->{'f'}";
		last;
		}
	}
closedir(DIR);
return "$base $self->{'text_hash'}->{'slack_unknown'}" if (!$diskfile);

# read the disk file
local $desc;
$self->open_readfile(*DISK, $diskfile);
while(<DISK>) {
	if (/^$base:\s*(.*)/) {
		$desc = $1;
		last;
		}
	}
close(DISK);
return $desc ? "$base $desc" : "$base $self->{'text_hash'}->{'slack_unknown'}";
}

# install_options(file, package)
# Outputs HTML for choosing install options
sub install_options
{
	my $self = shift;

print "<tr> <td><b>$self->{'text_hash'}->{'slack_root'}</b></td>\n";
print "<td colspan=3><input name=root size=30 value='/'> ",
	$self->file_chooser_button("root", 1),"</td> </tr>\n";
}

# install_package(file, package)
# Installs the package in the given file, with options from %{$self->{'in_hash'}}
sub install_package
{
	my $self = shift;

local $in = $_[2] ? $_[2] : \%{$self->{'in_hash'}};
return $self->{'text_hash'}->{'slack_eroot'} if (!-d $in->{'root'});
$ENV{'ROOT'} = $in->{'root'};
local $out;
local $qm = quotemeta($_[0]);
if ($self->has_command("upgradepkg") &&
    -r $self->translate_filename("$self->{'package_dir'}/$_[1]")) {
	# Try to upgrade properly
	$out = $self->backquote_logged("upgradepkg $qm 2>&1");
	}
else {
	# Just install
	$out = $self->backquote_logged("installpkg $qm 2>&1");
	}
if ($?) {
	return "<pre>$out</pre>";
	}
return undef;
}

# delete_package(package)
# Totally remove some package
sub delete_package
{
	my $self = shift;

local $qm = quotemeta($_[0]);
local $out = $self->backquote_logged("removepkg $qm 2>&1");
if ($?) { return "<pre>$out</pre>"; }
return undef;
}

sub package_system
{
	my $self = shift;

return $self->{'text_hash'}->{'slack_manager'};
}

sub package_help
{
	my $self = shift;

return "installpkg removepkg";
}


1;

