#!/usr/bin/perl
package software::SoftwareLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'software::SoftwareLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# software-lib.pl
# A generalized system for package management on solaris, linux, etc..

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
$self->{'heiropen_file'} = "$self->{'module_config_directory'}/heiropen";

# Use the appropriate function set for whatever package management system
# we are using.
{
    my $pkg = $self->lib_to_pkg("$self->{config_hash}->{package_system}-lib.pl", "software");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}

if ($self->{'config_hash'}->{'update_system'}) {
	# User-specified system
	$self->{'update_system'} = $self->{'config_hash'}->{'update_system'};
	}
else {
	# Guess which update system we are using
	if ($self->has_command($self->{'config_hash'}->{'apt_mode'} ? "aptitude" : "apt-get")) {
		$self->{'update_system'} = "apt";
		}
	elsif ($self->has_command("yum") && -r "/etc/yum.conf") {
		$self->{'update_system'} = "yum";
		}
	elsif ($self->has_command("up2date") && $self->has_command("rhn_check")) {
		$self->{'update_system'} = "rhn";
		}
	elsif (-x "/opt/csw/bin/pkg-get" || $self->has_command("pkg-get")) {
		$self->{'update_system'} = "csw";
		}
	elsif ($self->has_command("urpmi")) {
		$self->{'update_system'} = "urpmi";
		}
	elsif ($self->has_command("emerge")) {
		$self->{'update_system'} = "emerge";
		}
	elsif ($self->has_command("cupdate")) {
		# not done yet!
		}
	}
if ($self->{'update_system'}) {
	{
    my $pkg = $self->lib_to_pkg($self->{update_system}."-lib.pl", "software");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
	$self->{'has_update_system'} = 1;
	}

@{$self->{'type_map_array'}} = (	$self->{'text_hash'}->{'soft_reg'}, $self->{'text_hash'}->{'soft_dir'},  $self->{'text_hash'}->{'soft_spec'},
		$self->{'text_hash'}->{'soft_sym'}, $self->{'text_hash'}->{'soft_hard'}, $self->{'text_hash'}->{'soft_edit'} );

1;

1;
		return $self;
}

# uncompress_if_needed(file, disposable)
# If some file needs to be uncompressed or ungzipped, do it and return the
# new temp file path. Otherwise, return the original path.
sub uncompress_if_needed
{
	my $self = shift;

return $_[0] if ($self->is_readonly_mode());	# don't even bother
open(PFILE, $_[0]);
read(PFILE, $self->{'two'}, 2);
close(PFILE);
if ($self->{'two'} eq "\037\235") {
	if (!$self->has_command("uncompress")) {
		$self->unlink_file($_[0]) if ($_[1]);
		$self->error($self->{'text_hash'}->{'soft_euncompress'});
		}
	local $temp = $_[0] =~ /\/([^\/]+)\.Z/i ? $self->tempname("$1")
						: $self->tempname();
	local $out = `uncompress -c $_[0] 2>&1 >$temp`;
	unlink($_[0]) if ($_[1]);
	if ($?) {
		unlink($temp);
		$self->error($self->text('soft_euncmsg', $out));
		}
	return $temp;
	}
elsif ($self->{'two'} eq "\037\213") {
	if (!$self->has_command("gunzip")) {
		unlink($_[0]) if ($_[1]);
		$self->error($self->{'text_hash'}->{'soft_egzip'});
		}
	local $temp = $_[0] =~ /\/([^\/]+)\.gz/i ? $self->tempname("$1")
						 : $self->tempname();
	local $out = `gunzip -c $_[0] 2>&1 >$temp`;
	unlink($_[0]) if ($_[1]);
	if ($?) {
		unlink($temp);
		$self->error($self->text('soft_egzmsg', $out));
		}
	return $temp;
	}
return $_[0];
}

# show_package_info(package, version)
sub show_package_info
{
	my $self = shift;

@{$self->{'pinfo_array'}} = $self->package_info($_[0], $_[1]);
return () if (!@{$self->{'pinfo_array'}});
print $self->ui_subheading($self->text('do_success', $_[0]));
print "<table border width=100%>\n";
print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'do_details'}</b></td> </tr>\n";
print "<tr $self->{'cb'}> <td><table width=100%>\n";

if ($self->{'pinfo_array'}->[2]) {
	print "<tr> <td valign=top width=20%><b>$self->{'text_hash'}->{'do_desc'}</b></td>\n";
	print "<td colspan=3><pre>",$self->html_escape($self->{'pinfo_array'}->[2]),"</pre></td> </tr>\n";
	}

print "<tr> <td width=20%><b>$self->{'text_hash'}->{'do_pack'}</b></td> <td>",
	$self->html_escape($self->{'pinfo_array'}->[0]),"</td>\n";
print "<td width=20%><b>$self->{'text_hash'}->{'do_class'}</b></td> <td>",
	$self->{'pinfo_array'}->[1] ? $self->html_escape($self->{'pinfo_array'}->[1]) : $self->{'text_hash'}->{'do_none'},"</td> </tr>\n";

print "<tr> <td width=20%><b>$self->{'text_hash'}->{'do_ver'}</b></td> <td>",
	$self->html_escape($self->{'pinfo_array'}->[4]),"</td>\n";
print "<td width=20%><b>$self->{'text_hash'}->{'do_vend'}</b></td> <td>",
	$self->html_escape($self->{'pinfo_array'}->[5]),"</td> </tr>\n";

print "<tr> <td width=20%><b>$self->{'text_hash'}->{'do_arch'}</b></td> <td>",
	$self->html_escape($self->{'pinfo_array'}->[3]),"</td>\n";
print "<td width=20%><b>$self->{'text_hash'}->{'do_inst'}</b></td> <td>",
	$self->html_escape($self->{'pinfo_array'}->[6]),"</td> </tr>\n";
print "</table></td></tr></table><p>\n";
return @{$self->{'pinfo_array'}};
}

# get_heiropen()
# Returns an array of open categories
sub get_heiropen
{
	my $self = shift;

open(HEIROPEN, $self->{'heiropen_file'});
local @heiropen = <HEIROPEN>;
chop(@heiropen);
close(HEIROPEN);
return @heiropen;
}

# save_heiropen(&heir)
sub save_heiropen
{
	my $self = shift;

$self->open_tempfile(HEIR, ">$self->{'heiropen_file'}");
foreach $h (@{$_[0]}) {
	$self->print_tempfile(HEIR, $h,"\n");
	}
$self->close_tempfile(HEIR);
}

# missing_install_link(package, description, return, return-desc)
# Returns HTML for installing some package that is missing, from the
# appropriate update system for this OS. Returns undef if automatic installation
# is not possible for some reason.
# Supported package names are :
#	apache
#	sendmail
#	postfix
#	squid
#	procmail
#	samba
#	mysql
#	postgresql
#	clamav
#	spamassassin
sub missing_install_link
{
	my $self = shift;

local ($name, $desc, $return, $returndesc) = @_;
return undef if (!$self->UNIVERSAL::can('update_system_resolve'));
return undef if (!$self->foreign_check($self->{'module_name'}));
local $pkg = $self->update_system_resolve($name);
return undef if (!$pkg);
return $self->text('missing_link', $desc, "../$self->{'module_name'}/install_pack.cgi?source=3&update=".$self->urlize($pkg)."&return=".$self->urlize($return)."&returndesc=".$self->urlize($returndesc), $self->{'text_hash'}->{$self->{'update_system'}."_name"});
}

# update_system_button(field-name, label)
# Returns HTML for a button that opens the update system search window
sub update_system_button
{
	my $self = shift;

local ($name, $label) = @_;
if ($self->UNIVERSAL::can('update_system_available') || $self->UNIVERSAL::can('update_system_search')) {
	return "<input type=button onClick='window.ifield = form.$name; chooser = window.open(\"../$self->{'module_name'}/find.cgi\", \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=600,height=500\")' value=\"$label\">";
	}
return undef;
}

# compare_versions(ver1, ver2)
# Returns -1 if ver1 is older than ver2, 1 if newer, 0 if same
sub compare_versions
{
	my $self = shift;

local @sp1 = split(/[\.\-]/, $_[0]);
local @sp2 = split(/[\.\-]/, $_[1]);
for(my $i=0; $i<@sp1 || $i<@sp2; $i++) {
	local $v1 = $sp1[$i];
	local $v2 = $sp2[$i];
	local $comp;
	if ($v1 =~ /^\d+$/ && $v2 =~ /^\d+$/) {
		$comp = $v1 <=> $v2;
		}
	else {
		$comp = $v1 cmp $v2;
		}
	return $comp if ($comp);
	}
return 0;
}


1;

