#!/usr/bin/perl

&MAIN();

 sub MAIN {
# install_pack.cgi
# Install a package from some source

&ModuleLoader::preload_module('software_class', "software::SoftwareLib");
$self = new software::SoftwareLib:: 'software_class',%$self;

if ($ENV{REQUEST_METHOD} eq "POST") {
	$self->ReadParse(\%{$self->{'getin_hash'}}, "GET");
	$self->ReadParseMime(undef, 'read_parse_mime_callback', [ $self->{'getin_hash'}->{'id'} ]);
	}
else {
	$self->ReadParse();
	$self->{'no_upload'} = 1;
	}
$self->error_setup($self->{'text_hash'}->{'install_err'});

if ($self->{'in_hash'}->{source} >= 2) {
	$self->ui_print_unbuffered_header(undef, $self->{'text_hash'}->{'install_title'}, "", "install");
	}
else {
	$self->ui_print_header(undef, $self->{'text_hash'}->{'install_title'}, "", "install");
	}

if ($self->{'in_hash'}->{source} == 0) {
	# installing from local file (or maybe directory)
	if (!$self->{'in_hash'}->{'local'})
		{ &install_error($self->{'text_hash'}->{'install_elocal'}); }
	if (!-r $self->{'in_hash'}->{'local'} && !-d $self->{'in_hash'}->{'local'} && $self->{'in_hash'}->{'local'} !~ /\*|\?/)
		{ &install_error($self->text('install_elocal2', $self->{'in_hash'}->{'local'})); }
	$self->{'source'} = $self->{'in_hash'}->{'local'};
	$self->{'pfile'} = $self->{'in_hash'}->{'local'};
	$self->{'need_unlink'} = 0;
	}
elsif ($self->{'in_hash'}->{source} == 1) {
	# installing from upload .. store file in temp location
	if ($self->{'no_upload'}) {
		&install_error($self->{'text_hash'}->{'install_eupload'});
		}
	$self->{'in_hash'}->{'upload_filename'} =~ /([^\/\\]+$)/;
	$self->{'pfile'} = $self->tempname("$1");
	$self->open_tempfile(PFILE, ">$self->{'pfile'}", 0, 1);
	$self->print_tempfile(PFILE, $self->{'in_hash'}->{'upload'});
	$self->close_tempfile(PFILE);
	$self->{'source'} = $self->{'in_hash'}->{'upload_filename'};
	$self->{'need_unlink'} = 1;
	}
elsif ($self->{'in_hash'}->{source} == 2) {
	# installing from URL.. store downloaded file in temp location
	$self->{'in_hash'}->{'url'} = $self->convert_osdn_url($self->{'in_hash'}->{'url'});
	$self->{'in_hash'}->{'url'} =~ /\/([^\/]+)\/*$/;
	$self->{'pfile'} = $self->tempname("$1");
	local $error;
	$self->{'progress_callback_url'} = $self->{'in_hash'}->{'url'};
	if ($self->{'in_hash'}->{'url'} =~ /^(http|https):\/\/([^\/]+)(\/.*)$/) {
		# Make a HTTP request
		$self->{'ssl'} = $1 eq 'https';
		$self->{'host'} = $2; $self->{'page'} = $3; $self->{'port'} = $self->{'ssl'} ? 443 : 80;
		if ($self->{'host'} =~ /^(.*):(\d+)$/) { $self->{'host'} = $1; $self->{'port'} = $2; }
		$self->http_download($self->{'host'}, $self->{'port'}, $self->{'page'}, $self->{'pfile'}, \$error,
			       'progress_callback', $self->{'ssl'});
		}
	elsif ($self->{'in_hash'}->{'url'} =~ /^ftp:\/\/([^\/]+)(:21)?(\/.*)$/) {
		$self->{'host'} = $1; $self->{'file'} = $3;
		$self->ftp_download($self->{'host'}, $self->{'file'}, $self->{'pfile'}, \$error,
			      'progress_callback');
		}
	else {
		&install_error($self->text('install_eurl', $self->{'in_hash'}->{'url'}));
		}
	&install_error($error) if ($error);
	$self->{'source'} = $self->{'in_hash'}->{'url'};
	$self->{'need_unlink'} = 1;
	}
elsif ($self->{'in_hash'}->{'source'} == 3) {
	# installing from some update system
	$self->clean_environment();
	@{$self->{'packs_array'}} = $self->update_system_install($self->{'in_hash'}->{'update'}, \%{$self->{'in_hash'}});
	$self->reset_environment();

	print "<hr>\n" if (@{$self->{'packs_array'}});
	foreach $p (@{$self->{'packs_array'}}) {
		local @pinfo = $self->show_package_info($p);
		}
	$self->webmin_log($self->{'config_hash'}->{'update_system'}, "install", undef,
		    { 'packages' => \@{$self->{'packs_array'}} } ) if (@{$self->{'packs_array'}});

	if ($self->{'in_hash'}->{'return'}) {
		$self->ui_print_footer($self->{'in_hash'}->{'return'}, $self->{'in_hash'}->{'returndesc'});
		}
	else {
		$self->ui_print_footer("", $self->{'text_hash'}->{'index_return'});
		}
	exit;
	}

# Check validity
if (!$self->is_package($self->{'pfile'})) {
	if (-d $self->{'pfile'}) {
		&install_error($self->text('install_edir', $self->package_system()));
		}
	else {
		# attempt to uncompress
		local $unc = $self->uncompress_if_needed($self->{'pfile'}, $self->{'need_unlink'});
		if ($unc ne $self->{'pfile'}) {
			# uncompressed ok..
			if (!$self->is_package($unc)) {
				$self->unlink_file($unc);
				&install_error($self->text('install_ezip',
					     $self->package_system()));
				}
			$self->{'pfile'} = $unc;
			}
		else {
			# uncompress failed.. give up
			#unlink($self->{'pfile'}) if ($self->{'need_unlink'});
			&install_error($self->text('install_efile', $self->package_system()));
			}
		}
	}

# ask for package to install and install options
@{$self->{'rv_array'}} = $self->file_packages($self->{'pfile'});

print "<form action=do_install.cgi>\n";
print "<input type=hidden name=file value=\"$self->{'pfile'}\">\n";
print "<input type=hidden name=need_unlink value=\"$self->{'need_unlink'}\">\n";
print "<table border>\n";
print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'install_header'}</b></td> </tr>\n";
print "<tr $self->{'cb'}> <td><table>\n";
print "<tr> <td valign=top><b>$self->{'text_hash'}->{'install_packs'}</b></td>\n";
print $self->{'wide_install_options'} ? "<td colspan=3>\n" : "<td>\n";
foreach (@{$self->{'rv_array'}}) {
	($p, $self->{'d'}) = split(/\s+/, $_, 2);
	if ($self->{'d'}) {
		print $self->html_escape($self->{'d'})," (",$self->html_escape($p),")<br>\n";
		}
	else {
		print $self->html_escape($p),"<br>\n";
		}
	}
$self->install_options($self->{'pfile'}, $p);
print "</table></td></tr>\n";
print "</table><input type=submit value=\"$self->{'text_hash'}->{'install_ok'}\"></form>\n";

$self->ui_print_footer("", $self->{'text_hash'}->{'index_return'});

1;
		
}

sub install_error
{
	shift if ($self == $_[0]);

print "<br><b>$self->{'whatfailed'} : $_[0]</b> <p>\n";
$self->ui_print_footer("", $self->{'text_hash'}->{'index_return'});
exit;
}


1;

