#!/usr/bin/perl

&MAIN();

 sub MAIN {
# save_header.cgi
# Save custom header and body tests

&ModuleLoader::preload_module('spam_class', "spam::SpamLib");
$self = new spam::SpamLib:: 'spam_class',%$self;

$self->can_use_check("header");
$self->error_setup($self->{'text_hash'}->{'header_err'});
$self->ReadParse();
$self->execute_before("header");
$self->lock_spam_files();
$self->{'conf'} = $self->get_config();

if (!$self->{'module_info_hash'}->{'usermin'}) {
	$self->parse_yes_no($self->{'conf'}, "allow_user_rules");
	}

@{$self->{'header_array'}} = $self->parse_table("header", \&header_parser);
$self->save_directives($self->{'conf'}, "header", \@{$self->{'header_array'}}, 1);

@{$self->{'oldbody_array'}} = ( $self->find("body", $self->{'conf'}),
	     $self->find("rawbody", $self->{'conf'}),
	     $self->find("fullbody", $self->{'conf'}),
	     $self->find("full", $self->{'conf'}) );
@{$self->{'body_array'}} = $self->parse_table("body", \&body_parser);
$self->save_directives($self->{'conf'}, \@{$self->{'oldbody_array'}}, \@{$self->{'body_array'}}, 0);

@{$self->{'uri_array'}} = $self->parse_table("uri", \&uri_parser);
$self->save_directives($self->{'conf'}, "uri", \@{$self->{'uri_array'}}, 1);

@{$self->{'meta_array'}} = $self->parse_table("meta", \&meta_parser);
$self->save_directives($self->{'conf'}, "meta", \@{$self->{'meta_array'}}, 1);

@{$self->{'score_array'}} = $self->parse_table("score", \&score_parser);
$self->save_directives($self->{'conf'}, 'score', \@{$self->{'score_array'}}, 1);

@{$self->{'describe_array'}} = $self->parse_table("describe", \&describe_parser);
$self->save_directives($self->{'conf'}, 'describe', \@{$self->{'describe_array'}}, 1);

$self->flush_file_lines();
$self->unlock_spam_files();
$self->execute_after("header");
$self->webmin_log("header");
$self->redirect("");

1;
		
}

# header_parser(rowname, value, ...)
sub header_parser
{
	shift if ($self == $_[0]);

return undef if ($_[1] eq '');
$_[1] =~ /^\S+$/ || $self->error($self->text('header_ename', $_[1]));
if ($_[3] eq 'eval') {
	$_[4] =~ /^\S+\(.*\)$/ || $self->error($self->text('header_eeval', $_[4]));
	return "$_[1] eval:$_[4]";
	}
elsif ($_[3] eq 'exists') {
	$_[2] =~ /^\S+$/ || $self->error($self->text('header_eheader', $_[2]));
	return "$_[1] exists:$_[2]";
	}
else {
	$_[2] =~ /^\S+$/ || $self->error($self->text('header_eheader', $_[2]));
	$_[4] =~ /^\/(.*)\/\S*$/ || $self->error($self->text('header_eregexp', $_[4]));
	return "$_[1] $_[2] $_[3] $_[4]".($_[5] ? " if-unset: $_[5]" : "");
	}
}

# body_parser(rowname, value, ...)
sub body_parser
{
	shift if ($self == $_[0]);

return undef if ($_[1] eq '');
$_[1] =~ /^\S+$/ || $self->error($self->text('header_ename', $_[1]));
local $v;
if ($_[3] == 0) {
	$_[4] =~ /^\/(.*)\/\S*$/ || $self->error($self->text('header_eregexp', $_[4]));
	$v = "$_[1] $_[4]";
	}
else {
	$_[4] =~ /^\S+\(.*\)$/ || $self->error($self->text('header_eeval', $_[4]));
	$v = "$_[1] eval:$_[4]";
	}
return { 'name' => $_[2] == 0 ? 'body' :
		   $_[2] == 1 ? 'rawbody' :
		   $_[2] == 2 ? 'fullbody' : 'full',
	 'value' => $v };
}

sub uri_parser
{
	shift if ($self == $_[0]);

return undef if ($_[1] eq '');
$_[1] =~ /^\S+$/ || $self->error($self->text('header_ename', $_[1]));
$_[2] =~ /^\/(.*)\/\S*$/ || $self->error($self->text('header_eregexp', $_[2]));
return "$_[1] $_[2]";
}

sub meta_parser
{
	shift if ($self == $_[0]);

return undef if ($_[1] eq '');
$_[1] =~ /^\S+$/ || $self->error($self->text('header_ename', $_[1]));
$_[2] =~ /\S/ || $self->error($self->text('header_emeta', $_[1]));
return "$_[1] $_[2]";
}

sub score_parser
{
	shift if ($self == $_[0]);

return undef if (!$_[1]);
$_[1] =~ /^\S+$/ || $self->error($self->text('score_ename', $_[1]));
$_[2] =~ /^\-?\d+(\.\d+)?$/ || $_[2] =~ /^\-?\.\d+$/ ||
	$self->error($self->text('score_epoints', $_[2]));
return "$_[1] $_[2]";
}

sub describe_parser
{
	shift if ($self == $_[0]);

return undef if (!$_[1]);
$_[1] =~ /^\S+$/ || $self->error($self->text('score_ename', $_[1]));
$_[2] =~ /\S/ || $self->error($self->text('score_edesc', $_[1]));
return "$_[1] $_[2]";
}


1;

