#!/usr/bin/perl
package sshd::SshdLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'sshd::SshdLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# sshd-lib.pl
# Common functions for the ssh daemon config file

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

# Get version information
$self->read_file("$self->{'module_config_directory'}/version", \%{$self->{'version_hash'}});

1;

1;
		return $self;
}

# get_sshd_config()
# Returns a reference to an array of SSHD config file options
sub get_sshd_config
{
	my $self = shift;

local @rv = ( { 'dummy' => 1,
		'indent' => 0,
		'file' => $self->{'config_hash'}->{'sshd_config'},
		'line' => -1,
		'eline' => -1 } );
local $lnum = 0;
open(CONF, $self->{'config_hash'}->{'sshd_config'});
while(<CONF>) {
	s/\r|\n//g;
	s/^\s*#.*$//g;
	local ($name, @values) = split(/\s+/, $_);
	if ($name) {
		local $dir = { 'name' => $name,
			       'values' => \@values,
			       'file' => $self->{'config_hash'}->{'sshd_config'},
			       'line' => $lnum };
		push(@rv, $dir);
		}
	$lnum++;
	}
close(CONF);
return \@rv;
}

# find_value(name, &config)
sub find_value
{
	my $self = shift;

foreach $c (@{$_[1]}) {
	if (lc($c->{'name'}) eq lc($_[0])) {
		return wantarray ? @{$c->{'values'}} : $c->{'values'}->[0];
		}
	}
return wantarray ? ( ) : undef;
}

# find(value, &config)
sub find
{
	my $self = shift;

local @rv;
foreach $c (@{$_[1]}) {
	if (lc($c->{'name'}) eq lc($_[0])) {
		push(@rv, $c);
		}
	}
return wantarray ? @rv : $rv[0];
}

# save_directive(name, &config, [value*|&values], [before])
sub save_directive
{
	my $self = shift;

local @o = $self->find($_[0], $_[1]);
local @n = ref($_[2]) ?
		grep { defined($_) } @{$_[2]} :
		grep { defined($_) } @_[2..@_-1];
local $lref = $self->read_file_lines($_[1]->[0]->{'file'});
local $id = ("\t" x $_[1]->[0]->{'indent'});
local $i;
local $before = $_[3] && ref($_[2]) ? $self->find($_[3], $_[1]) : undef;
for($i=0; $i<@o || $i<@n; $i++) {
	if ($o[$i] && $n[$i]) {
		# Replacing a line
		$lref->[$o[$i]->{'line'}] = "$id$_[0] $n[$i]";
		}
	elsif ($o[$i]) {
		# Removing a line
		splice(@$lref, $o[$i]->{'line'}, 1);
		foreach $c (@{$_[1]}) {
			if ($c->{'line'} > $o[$i]->{'line'}) {
				$c->{'line'}--;
				}
			}
		}
	elsif ($n[$i] && !$before) {
		# Adding a line at the end
		local $ll = $_[1]->[@{$_[1]}-1]->{'line'};
		splice(@$lref, $ll+1, 0, "$id$_[0] $n[$i]");
		}
	elsif ($n[$i] && $before) {
		# Adding a line before the first instance of some directive
		splice(@$lref, $before->{'line'}, 0, "$id$_[0] $n[$i]");
		foreach $c (@{$_[1]}) {
			if ($c->{'line'} >= $before->{'line'}) {
				$c->{'line'}--;
				}
			}
		}
	}
}

# scmd(double)
sub scmd
{
	my $self = shift;

if ($self->{'cmd_count'} % 2 == 0) {
	print "<tr>\n";
	}
elsif ($_[0]) {
	print "<td colspan=2></td> </tr>\n";
	print "<tr>\n";
	$self->{'cmd_count'} = 0;
	}
$self->{'cmd_count'} += ($_[0] ? 2 : 1);
}

# ecmd()
sub ecmd
{
	my $self = shift;

if ($self->{'cmd_count'} % 2 == 0) {
	print "</tr>\n";
	}
}

# get_client_config()
# Returns a list of structures, one for each host
sub get_client_config
{
	my $self = shift;

local @rv = ( { 'dummy' => 1,
		'indent' => 0,
		'file' => $self->{'config_hash'}->{'client_config'},
		'line' => -1,
		'eline' => -1 } );
local $host;
local $lnum = 0;
open(CLIENT, $self->{'config_hash'}->{'client_config'});
while(<CLIENT>) {
	s/\r|\n//g;
	s/^\s*#.*$//g;
	s/^\s*//g;
	local ($name, @values) = split(/\s+/, $_);
	if (lc($name) eq 'host') {
		# Start of new host
		$host = { 'name' => $name,
			  'values' => \@values,
			  'file' => $self->{'config_hash'}->{'client_config'},
			  'line' => $lnum,
			  'eline' => $lnum,
			  'members' => [ { 'dummy' => 1,
					   'indent' => 1,
					   'file' => $self->{'config_hash'}->{'client_config'},
					   'line' => $lnum } ] };
		push(@rv, $host);
		}
	elsif ($name) {
		# A directive inside a host
		local $dir = { 'name' => $name,
			       'values' => \@values,
			       'file' => $self->{'config_hash'}->{'client_config'},
			       'line' => $lnum };
		push(@{$host->{'members'}}, $dir);
		$host->{'eline'} = $lnum;
		}
	$lnum++;
	}
close(CLIENT);
return \@rv;
}

# create_host(&host)
sub create_host
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'client_config'});
$_[0]->{'line'} = $_[0]->{'eline'} = scalar(@$lref);
push(@$lref, "Host ".join(" ", @{$_[0]->{'values'}}));
$_[0]->{'members'} = [ { 'dummy' => 1,
			 'indent' => 1,
			 'file' => $self->{'config_hash'}->{'client_config'},
			 'line' => $_[0]->{'line'} } ];
}

# modify_host(&host)
sub modify_host
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'client_config'});
$lref->[$_[0]->{'line'}] = "Host ".join(" ", @{$_[0]->{'values'}});
}

# delete_host(&host)
sub delete_host
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'client_config'});
splice(@$lref, $_[0]->{'line'}, $_[0]->{'eline'} - $_[0]->{'line'} + 1);
}

# restart_sshd()
# Re-starts the SSH server, and returns an error message on failure or
# undef on success
sub restart_sshd
{
	my $self = shift;

if ($self->{'config_hash'}->{'restart_cmd'}) {
	local $out = `$self->{'config_hash'}->{'restart_cmd'} 2>&1 </dev/null`;
	return "<pre>$out</pre>" if ($?);
	}
else {
	local $pid = $self->get_sshd_pid();
	$pid || return $self->{'text_hash'}->{'apply_epid'};
	$self->kill_logged('HUP', $pid);
	}
return undef;
}

# stop_sshd()
# Kills the SSH server, and returns an error message on failure or
# undef on success
sub stop_sshd
{
	my $self = shift;

if ($self->{'config_hash'}->{'stop_cmd'}) {
	local $out = `$self->{'config_hash'}->{'stop_cmd'} 2>&1 </dev/null`;
	return "<pre>$out</pre>" if ($?);
	}
else {
	local $pid = $self->get_sshd_pid();
	$pid || return $self->{'text_hash'}->{'apply_epid'};
	$self->kill_logged('TERM', $pid);
	}
return undef;
}

# start_sshd()
# Attempts to start the SSH server, returning undef on success or an error
# message on failure.
sub start_sshd
{
	my $self = shift;

# Remove PID file if invalid
if (-f $self->{'config_hash'}->{'pid_file'} && !$self->check_pid_file($self->{'config_hash'}->{'pid_file'})) {
	$self->unlink_file($self->{'config_hash'}->{'pid_file'});
	}

if ($self->{'config_hash'}->{'start_cmd'}) {
	$self->{'out'} = $self->backquote_logged("$self->{'config_hash'}->{'start_cmd'} 2>&1 </dev/null");
	if ($?) { return "<pre>$self->{'out'}</pre>"; }
	}
else {
	$self->{'out'} = $self->backquote_logged("$self->{'config_hash'}->{'sshd_path'} 2>&1 </dev/null");
	if ($?) { return "<pre>$self->{'out'}</pre>"; }
	}
return undef;
}

# get_pid_file()
# Returns the SSH server PID file
sub get_pid_file
{
	my $self = shift;

%{$self->{'config_hash'}};
local $conf = $self->get_sshd_config();
local $pidfile = $self->find_value("PidFile", $conf);
$pidfile ||= $self->{'config_hash'}->{'pid_file'};
return $pidfile;
}

# get_sshd_pid()
# Returns the PID of the running SSHd process
sub get_sshd_pid
{
	my $self = shift;

local $file = $self->get_pid_file();
if ($file) {
	return $self->check_pid_file($file);
	}
else {
	return $self->find_byname("sshd");
	}
}


1;

