#!/usr/bin/perl

&MAIN();

 sub MAIN {
# index.cgi
# Display sshd option categories

&ModuleLoader::preload_module('sshd_class', "sshd::SshdLib");
$self = new sshd::SshdLib:: 'sshd_class',%$self;


# Check if config file exists
if (!-r $self->{'config_hash'}->{'sshd_config'}) {
	$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", "intro", 1, 1);
	print $self->text('index_econfig', "<tt>$self->{'config_hash'}->{'sshd_config'}</tt>",
		    "$self->{'gconfig_hash'}->{'webprefix'}/config.cgi?$self->{'module_name'}"),"<p>\n";
	$self->ui_print_footer("/", $self->{'text_hash'}->{"index"});
	exit;
	}

# Check if sshd exists
if (!$self->has_command($self->{'config_hash'}->{'sshd_path'})) {
	$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", "intro", 1, 1);
	print $self->text('index_esshd', "<tt>$self->{'config_hash'}->{'sshd_path'}</tt>",
		    "$self->{'gconfig_hash'}->{'webprefix'}/config.cgi?$self->{'module_name'}"),"<p>\n";
	$self->ui_print_footer("/", $self->{'text_hash'}->{"index"});
	exit;
	}

# Check if sshd is the right version
$self->{'out'} = $self->backquote_command($self->quote_path($self->{'config_hash'}->{'sshd_path'})." -h 2>&1");
if ($self->{'config_hash'}->{'sshd_version'}) {
	# Forced version
	$self->{'version_hash'}->{'type'} = 'openssh';
	$self->{'version_hash'}->{'number'} = $self->{'version_hash'}->{'full'} = $self->{'config_hash'}->{'sshd_version'};
	}
elsif ($self->{'out'} =~ /(sshd\s+version\s+([0-9\.]+))/i ||
    $self->{'out'} =~ /(ssh\s+secure\s+shell\s+([0-9\.]+))/i) {
	# Classic commercial SSH
	$self->{'version_hash'}->{'type'} = 'ssh';
	$self->{'version_hash'}->{'number'} = $2;
	$self->{'version_hash'}->{'full'} = $1;
	}
elsif ($self->{'out'} =~ /(OpenSSH.([0-9\.]+))/i) {
	# OpenSSH .. assume all versions are supported
	$self->{'version_hash'}->{'type'} = 'openssh';
	$self->{'version_hash'}->{'number'} = $2;
	$self->{'version_hash'}->{'full'} = $1;
	}
elsif ($self->{'out'} =~ /(Sun_SSH_([0-9\.]+))/i) {
	# Solaris 9 SSH is actually OpenSSH 2.x
	$self->{'version_hash'}->{'type'} = 'openssh';
	$self->{'version_hash'}->{'number'} = 2.0;
	$self->{'version_hash'}->{'full'} = $1;
	}
elsif (($self->{'out'} = $self->{'config_hash'}->{'sshd_version'}) && ($self->{'out'} =~ /(Sun_SSH_([0-9\.]+))/i)) {
	# Probably Solaris 10 SSHD that didn't display version.  Use it.
	$self->{'version_hash'}->{'type'} = 'openssh';
	$self->{'version_hash'}->{'number'} = 2.0;
	$self->{'version_hash'}->{'full'} = $1;
	}
else {
	# Unknown version
	$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", "intro", 1, 1);
	print $self->text('index_eversion', "<tt>$self->{'config_hash'}->{'sshd_path'}</tt>",
		    "$self->{'gconfig_hash'}->{'webprefix'}/config.cgi?$self->{'module_name'}",
		    "<tt>$self->{'config_hash'}->{'sshd_path'} -h</tt>",
		    "<pre>$self->{'out'}</pre>"),"<p>\n";
	$self->ui_print_footer("/", $self->{'text_hash'}->{"index"});
	exit;
	}
$self->write_file("$self->{'module_config_directory'}/version", \%{$self->{'version_hash'}});

$self->ui_print_header(undef, $self->{'text_hash'}->{'index_title'}, "", "intro", 1, 1, 0,
	$self->help_search_link("ssh", "man", "doc", "google"), undef, undef,
	$self->text('index_version', $self->{'version_hash'}->{'full'}));

# Display icons for options
foreach $i ('users', 'net', 'access', 'misc') {
	push(@{$self->{'links_array'}}, "edit_$i.cgi");
	push(@{$self->{'titles_array'}}, $self->{'text_hash'}->{$i.'_title'});
	push(@{$self->{'icons_array'}}, "images/$i.gif");
	}
if (-r $self->{'config_hash'}->{'client_config'}) {
	push(@{$self->{'links_array'}}, "list_hosts.cgi");
	push(@{$self->{'titles_array'}}, $self->{'text_hash'}->{'hosts_title'});
	push(@{$self->{'icons_array'}}, "images/hosts.gif");
	}
push(@{$self->{'links_array'}}, "edit_sync.cgi");
push(@{$self->{'titles_array'}}, $self->{'text_hash'}->{'sync_title'});
push(@{$self->{'icons_array'}}, "images/sync.gif");
push(@{$self->{'links_array'}}, "edit_manual.cgi");
push(@{$self->{'titles_array'}}, $self->{'text_hash'}->{'manual_title'});
push(@{$self->{'icons_array'}}, "images/manual.gif");
$self->icons_table(\@{$self->{'links_array'}}, \@{$self->{'titles_array'}}, \@{$self->{'icons_array'}}, scalar(@{$self->{'links_array'}}));

# Check if sshd is running
$self->{'pid'} = $self->get_sshd_pid();
print "<hr>\n";
print $self->ui_buttons_start();
if ($self->{'pid'}) {
	# Running .. offer to apply changes and stop
	print $self->ui_buttons_row("apply.cgi",
	      $self->{'text_hash'}->{'index_apply'},
	      $self->{'config_hash'}->{'restart_cmd'} ?
		$self->text('index_applymsg2', "<tt>$self->{'config_hash'}->{'restart_cmd'}</tt>") :
		$self->{'text_hash'}->{'index_applymsg'});

	print $self->ui_buttons_row("stop.cgi",
			      $self->{'text_hash'}->{'index_stop'}, $self->{'text_hash'}->{'index_stopmsg'});
	}
else {
	# Not running .. offer to start
	print $self->ui_buttons_row("start.cgi", $self->{'text_hash'}->{'index_start'},
			      $self->{'text_hash'}->{'index_startmsg'});
	}
print $self->ui_buttons_end();

$self->ui_print_footer("/", $self->{'text_hash'}->{"index"});

1;
		
}


1;

