#!/usr/bin/perl

&MAIN();

 sub MAIN {
# save_host.cgi
# Create, update or delete a client host

&ModuleLoader::preload_module('sshd_class', "sshd::SshdLib");
$self = new sshd::SshdLib:: 'sshd_class',%$self;

$self->ReadParse();
$self->lock_file($self->{'config_hash'}->{'client_config'});
$self->{'hconf'} = $self->get_client_config();
$self->error_setup($self->{'text_hash'}->{'host_err'});

if ($self->{'in_hash'}->{'delete'}) {
	# Just delete the host
	$self->{'host'} = $self->{'hconf'}->[$self->{'in_hash'}->{'idx'}];
	$self->delete_host($self->{'host'});
	}
else {
	# Saving or creating a host
	$self->{'host'} = $self->{'hconf'}->[$self->{'in_hash'}->{'idx'}] if (!$self->{'in_hash'}->{'new'});
	if ($self->{'in_hash'}->{'name_def'}) {
		$self->{'host'}->{'values'} = [ '*' ];
		}
	else {
		$self->{'in_hash'}->{'name'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'host_ename'});
		$self->{'host'}->{'values'} = [ $self->{'in_hash'}->{'name'} ];
		}
	if ($self->{'in_hash'}->{'new'}) {
		# Create empty host structure
		$self->create_host($self->{'host'});
		}
	else {
		$self->modify_host($self->{'host'});
		}
	$self->{'conf'} = $self->{'host'}->{'members'};

	# Validate and store host options
	if ($self->{'in_hash'}->{'user_def'}) {
		$self->save_directive("User", $self->{'conf'});
		}
	else {
		$self->{'in_hash'}->{'user'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'host_euser'});
		$self->save_directive("User", $self->{'conf'}, $self->{'in_hash'}->{'user'});
		}

	$self->save_directive("KeepAlive", $self->{'conf'},
		$self->{'in_hash'}->{'keep'} == 2 ? undef : $self->{'in_hash'}->{'keep'} ? 'yes' : 'no');

	if ($self->{'in_hash'}->{'hostname_def'}) {
		$self->save_directive("HostName", $self->{'conf'});
		}
	else {
		$self->check_ipaddress($self->{'in_hash'}->{'hostname'}) ||
			gethostbyname($self->{'in_hash'}->{'hostname'}) ||
			$self->error($self->{'text_hash'}->{'host_ehostname'});
		$self->save_directive("HostName", $self->{'conf'}, $self->{'in_hash'}->{'hostname'});
		}

	$self->save_directive("BatchMode", $self->{'conf'},
		$self->{'in_hash'}->{'batch'} == 2 ? undef : $self->{'in_hash'}->{'batch'} ? 'yes' : 'no');

	if ($self->{'in_hash'}->{'port_def'}) {
		$self->save_directive("Port", $self->{'conf'});
		}
	else {
		$self->{'in_hash'}->{'port'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'host_eport'});
		$self->save_directive("Port", $self->{'conf'}, $self->{'in_hash'}->{'port'});
		}

	if ($self->{'version_hash'}->{'type'} ne 'ssh' || $self->{'version_hash'}->{'number'} < 3) {
		$self->save_directive("Compression", $self->{'conf'},
			$self->{'in_hash'}->{'comp'} == 2 ? undef : $self->{'in_hash'}->{'comp'} ? 'yes' : 'no');
		}

	if ($self->{'in_hash'}->{'escape_def'} == 1) {
		$self->save_directive("EscapeChar", $self->{'conf'});
		}
	elsif ($self->{'in_hash'}->{'escape_def'} == 2) {
		$self->save_directive("EscapeChar", $self->{'conf'}, "none");
		}
	else {
		$self->{'in_hash'}->{'escape'} =~ /^\S$/ || $self->{'in_hash'}->{'escape'} =~ /^\^\S$/ ||
			 $self->error($self->{'text_hash'}->{'host_eescape'});
		$self->save_directive("EscapeChar", $self->{'conf'}, $self->{'in_hash'}->{'escape'});
		}

	if ($self->{'version_hash'}->{'type'} ne 'ssh' || $self->{'version_hash'}->{'number'} < 3) {
		if ($self->{'in_hash'}->{'clevel_def'}) {
			$self->save_directive("CompressionLevel", $self->{'conf'});
			}
		else {
			$self->save_directive("CompressionLevel", $self->{'conf'},
					$self->{'in_hash'}->{'clevel'});
			}

		if ($self->{'in_hash'}->{'attempts_def'}) {
			$self->save_directive("ConnectionAttempts", $self->{'conf'});
			}
		else {
			$self->{'in_hash'}->{'attempts'} =~ /^\d+$/ ||
				$self->error($self->{'text_hash'}->{'host_eattempts'});
			$self->save_directive("ConnectionAttempts", $self->{'conf'},
					$self->{'in_hash'}->{'attempts'});
			}

		$self->save_directive("UsePrivilegedPort", $self->{'conf'},
			$self->{'in_hash'}->{'priv'} == 2 ? undef : $self->{'in_hash'}->{'priv'} ? 'yes' : 'no');

		$self->save_directive("FallBackToRsh", $self->{'conf'},
			$self->{'in_hash'}->{'rsh'} == 2 ? undef : $self->{'in_hash'}->{'rsh'} ? 'yes' : 'no');

		$self->save_directive("UseRsh", $self->{'conf'},
		    $self->{'in_hash'}->{'usersh'} == 2 ? undef : $self->{'in_hash'}->{'usersh'} ? 'yes' : 'no');
		}

	$self->save_directive("ForwardAgent", $self->{'conf'},
		$self->{'in_hash'}->{'agent'} == 2 ? undef : $self->{'in_hash'}->{'agent'} ? 'yes' : 'no');

	$self->save_directive("ForwardX11", $self->{'conf'},
		$self->{'in_hash'}->{'x11'} == 2 ? undef : $self->{'in_hash'}->{'x11'} ? 'yes' : 'no');

	$self->save_directive("StrictHostKeyChecking", $self->{'conf'},
		$self->{'in_hash'}->{'strict'} == 2 ? undef : $self->{'in_hash'}->{'strict'} == 1 ? 'yes' :
		$self->{'in_hash'}->{'strict'} == 0 ? 'no' : 'ask');

	if ($self->{'version_hash'}->{'type'} eq 'openssh') {
		$self->save_directive("CheckHostIP", $self->{'conf'},
		  $self->{'in_hash'}->{'checkip'} == 2 ? undef : $self->{'in_hash'}->{'checkip'} ? 'yes' : 'no');

		if ($self->{'in_hash'}->{'prots_def'}) {
			$self->save_directive("Protocol", $self->{'conf'});
			}
		else {
			@{$self->{'prots_array'}} = split(/\0/, $self->{'in_hash'}->{'prots'});
			@{$self->{'prots_array'}} || $self->error($self->{'text_hash'}->{'host_prots'});
			$self->save_directive("Protocol", $self->{'conf'}, join(",", @{$self->{'prots_array'}}));
			}
		}

	for($i=0; defined($self->{'in_hash'}->{"llport_$i"}); $i++) {
		next if (!$self->{'in_hash'}->{"llport_$i"} && !$self->{'in_hash'}->{"lrhost_$i"} &&
			 !$self->{'in_hash'}->{"lrport_$i"});
		$self->{'in_hash'}->{"llport_$i"} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'host_elport'});
		$self->{'in_hash'}->{"lrhost_$i"} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'host_erhost'});
		$self->{'in_hash'}->{"lrport_$i"} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'host_erport'});
		push(@{$self->{'lforward_array'}}, sprintf("%d %s:%d", $self->{'in_hash'}->{"llport_$i"},
				 	$self->{'in_hash'}->{"lrhost_$i"}, $self->{'in_hash'}->{"lrport_$i"}));
		}
	$self->save_directive("LocalForward", $self->{'conf'}, @{$self->{'lforward_array'}});

	for($i=0; defined($self->{'in_hash'}->{"rrport_$i"}); $i++) {
		next if (!$self->{'in_hash'}->{"rrport_$i"} && !$self->{'in_hash'}->{"rlhost_$i"} &&
			 !$self->{'in_hash'}->{"rlport_$i"});
		$self->{'in_hash'}->{"rrport_$i"} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'host_erport'});
		$self->{'in_hash'}->{"rlhost_$i"} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'host_elhost'});
		$self->{'in_hash'}->{"rlport_$i"} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'host_elport'});
		push(@{$self->{'rforward_array'}}, sprintf("%d %s:%d", $self->{'in_hash'}->{"rrport_$i"},
				 	$self->{'in_hash'}->{"rlhost_$i"}, $self->{'in_hash'}->{"rlport_$i"}));
		}
	$self->save_directive("RemoteForward", $self->{'conf'}, @{$self->{'rforward_array'}});
	}

$self->flush_file_lines();
$self->unlock_file($self->{'config_hash'}->{'client_config'});
$self->webmin_log($self->{'in_hash'}->{'new'} ? "create" : $self->{'in_hash'}->{'delete'} ? "delete" : "update",
	    "host", $self->{'host'}->{'values'}->[0]);
$self->redirect("list_hosts.cgi");

1;
		
}


1;

