#!/usr/bin/perl
package status::Httpmonitor;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'status::Httpmonitor');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# http-monitor.pl
# Monitor a remote HTTP server

1;
		return $self;
}

sub get_http_status
{
	my $self = shift;

# Connect to the server
local $up=0;
local $st = time();
eval {
	local $SIG{ALRM} = sub { die "alarm\n" }; # NB: \n required
	alarm($_[0]->{'alarm'} ? $_[0]->{'alarm'} : 10);

	local $re = $_[0]->{'regexp'};
	local $method = $_[0]->{'method'} || "HEAD";
	local $con = $self->make_http_connection($_[0]->{'host'}, $_[0]->{'port'},
				   $_[0]->{'ssl'}, $method, $_[0]->{'page'});
	if (!ref($con)) {
		return { 'up' => 0 };
		}
	$self->write_http_connection($con, "Host: $_[0]->{'host'}\r\n");
	$self->write_http_connection($con, "User-agent: Webmin\r\n");
	if ($_[0]->{'user'}) {
		local $auth = $self->encode_base64("$_[0]->{'user'}:$_[0]->{'pass'}");
		$auth =~ s/\n//g;
		$self->write_http_connection($con, "Authorization: basic $auth\r\n");
		}
	$self->write_http_connection($con, "\r\n");
	local $line = $self->read_http_connection($con);
	$up = $line =~ /^HTTP\/1\..\s+(200|301|302)\s+/ ? 1 : 0;
	if ($re && $up) {
		# Read the headers
		local %header;
		while(1) {
			$line = $self->read_http_connection($con);
			$line =~ tr/\r\n//d;
			$line =~ /^(\S+):\s+(.*)$/ || last;
			$header{lc($1)} = $2;
			}

		# Read the body
		local ($buf, $data);
		while(defined($buf = $self->read_http_connection($con))) {
			$data .= $buf;
			}
		if ($data !~ /$re/i) {
			$up = 0;
			}
		}

	$self->close_http_connection($con);
	alarm(0);
	};

if ($@) {
	die unless $@ eq "alarm\n";   # propagate unexpected errors
	return { 'up' => 0 };
	}
else { 
	return { 'up' => $up, 'time' => time() - $st};
	}
}

sub show_http_dialog
{
	my $self = shift;

local $url = $_[0]->{'host'} ? ($_[0]->{'ssl'} ? "https" : "http").
	     		    "://$_[0]->{'host'}:$_[0]->{'port'}$_[0]->{'page'}"
			     : "http://";
print $self->ui_table_row($self->{'text_hash'}->{'http_url'},
	$self->ui_textbox("url", $url, 50), 3);

print $self->ui_table_row($self->{'text_hash'}->{'http_alarm'},
	$self->ui_opt_textbox("alarm", $_[0]->{'alarm'}, 5, $self->{'text_hash'}->{'default'}));

print $self->ui_table_row($self->{'text_hash'}->{'http_method'},
	$self->ui_select("method", $_[0]->{'method'},
		   [ [ "HEAD" ], [ "GET" ], [ "POST" ] ]));

print $self->ui_table_row($self->{'text_hash'}->{'http_login'},
	$self->ui_radio("user_def", $_[0]->{'user'} ? 0 : 1,
		[ [ 1, $self->{'text_hash'}->{'http_none'} ],
		  [ 0, $self->{'text_hash'}->{'http_user'}." ".
		       $self->ui_textbox("huser", $_[0]->{'user'}, 15)." ".
		       $self->{'text_hash'}->{'http_pass'}." ".
		       $self->ui_password("hpass", $_[0]->{'pass'}, 15) ] ]), 3);

print $self->ui_table_row($self->{'text_hash'}->{'http_regexp'},
	$self->ui_opt_textbox("regexp", $_[0]->{'regexp'}, 50, $self->{'text_hash'}->{'http_none2'}),
	3);
}

sub parse_http_dialog
{
	my $self = shift;

if ($self->{'in_hash'}->{'url'} =~ /^(http|https):\/\/([^:]+):(\d+)(\/.*)$/) {
	$_[0]->{'ssl'} = $1 eq "https";
	$_[0]->{'host'} = $2;
	$_[0]->{'port'} = $3;
	$_[0]->{'page'} = $4;
	}
elsif ($self->{'in_hash'}->{'url'} =~ /^(http|https):\/\/([^\/]+)(\/.*)$/) {
	$_[0]->{'ssl'} = $1 eq "https";
	$_[0]->{'host'} = $2;
	$_[0]->{'port'} = $1 eq "https" ? 443 : 80;
	$_[0]->{'page'} = $3;
	}
elsif ($self->{'in_hash'}->{'url'} =~ /^(http|https):\/\/([^:]+):(\d+)$/) {
	$_[0]->{'ssl'} = $1 eq "https";
	$_[0]->{'host'} = $2;
	$_[0]->{'port'} = $3;
	$_[0]->{'page'} = "/";
	}
elsif ($self->{'in_hash'}->{'url'} =~ /^(http|https):\/\/([^\/]+)$/) {
	$_[0]->{'ssl'} = $1 eq "https";
	$_[0]->{'host'} = $2;
	$_[0]->{'port'} = $1 eq "https" ? 443 : 80;
	$_[0]->{'page'} = "/";
	}
else {
	$self->error($self->{'text_hash'}->{'http_eurl'});
	}

if ($self->{'in_hash'}->{'alarm_def'}) {
	delete($_[0]->{'alarm'});
	}
else {
	$self->{'in_hash'}->{'alarm'} =~ /^\d+$/ || $self->error($self->{'text_hash'}->{'http_ealarm'});
	$_[0]->{'alarm'} = $self->{'in_hash'}->{'alarm'};
	}

if ($self->{'in_hash'}->{'user_def'}) {
	delete($_[0]->{'user'});
	delete($_[0]->{'pass'});
	}
else {
	$self->{'in_hash'}->{'user'} || $self->error($self->{'text_hash'}->{'http_euser'});
	$_[0]->{'user'} = $self->{'in_hash'}->{'huser'};
	$_[0]->{'pass'} = $self->{'in_hash'}->{'hpass'};
	}

$_[0]->{'method'} = $self->{'in_hash'}->{'method'};

if ($self->{'in_hash'}->{'regexp_def'}) {
	delete($_[0]->{'regexp'});
	}
else {
	$self->{'in_hash'}->{'regexp'} || $self->error($self->{'text_hash'}->{'http_eregexp'});
	$_[0]->{'regexp'} = $self->{'in_hash'}->{'regexp'};
	$self->{'in_hash'}->{'method'} eq 'HEAD' && $self->error($self->{'text_hash'}->{'http_ehead'});
	}
}


1;

