#!/usr/bin/perl
package status::Querymonitor;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'status::Querymonitor');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# query-monitor.pl
# Try an SQL query on a MySQL or PostgreSQL server

1;
		return $self;
}

sub get_query_status
{
	my $self = shift;

# Load the driver
local $drh;
eval <<EOF;
use DBI;
\$drh = DBI->install_driver(\$_[0]->{'driver'});
EOF
if ($@) {
	return { 'up' => -1,
		 'desc' => $self->text('query_edriver',
				 "<tt>DBD::$_[0]->{'driver'}</tt>") };
	}

# Connect to the database server
local $dbistr = $self->make_dbistr($_[0]->{'driver'}, $_[0]->{'db'}, $_[0]->{'host'});
local $dbh = $drh->connect($dbistr,
                           $_[0]->{'user'}, $_[0]->{'pass'}, { });
if (!$dbh) {
	return { 'up' => 0,
		 'desc' => $self->text('query_elogin', $drh->errstr) };
	}

# Execute the query
local $cmd = $dbh->prepare($_[0]->{'sql'});
if (!$cmd) {
	return { 'up' => 0,
		 'desc' => $self->text('query_eprepare', $dbh->errstr) };
	}
if (!$cmd->execute()) {
	return { 'up' => 0,
		 'desc' => $self->text('query_eexecute', $dbh->errstr) };
	}
local @r = $cmd->fetchrow();
$cmd->finish();

if ($_[0]->{'result'} && $r[0] ne $_[0]->{'result'}) {
	return { 'up' => 0,
		 'desc' => $self->text('query_ewrong', $r[0]) };
	}

return { 'up' => 1 };
}

sub show_query_dialog
{
	my $self = shift;

print $self->ui_table_row($self->{'text_hash'}->{'query_driver'},
	$self->ui_select("driver", $_[0]->{'driver'},
		   [ [ "mysql", "MySQL" ],
		     [ "Pg", "PostgreSQL" ],
		     [ "Oracle", "Oracle" ] ]));

print $self->ui_table_row($self->{'text_hash'}->{'query_db'},
	$self->ui_textbox("db", $_[0]->{'db'}, 20));

print $self->ui_table_row($self->{'text_hash'}->{'query_user'},
	$self->ui_textbox("quser", $_[0]->{'user'}, 20));

print $self->ui_table_row($self->{'text_hash'}->{'query_pass'},
	$self->ui_password("qpass", $_[0]->{'pass'}, 20));

print $self->ui_table_row($self->{'text_hash'}->{'query_host'},
	$self->ui_opt_textbox("host", $_[0]->{'host'}, 40, $self->{'text_hash'}->{'query_local'}), 3);

print $self->ui_table_row($self->{'text_hash'}->{'query_sql'},
	$self->ui_textbox("sql", $_[0]->{'sql'}, 60), 3);

print $self->ui_table_row($self->{'text_hash'}->{'query_result'},
	$self->ui_opt_textbox("result", $_[0]->{'result'}, 40,
			$self->{'text_hash'}->{'query_ignore'}), 3);
}

sub parse_query_dialog
{
	my $self = shift;

eval "use DBD::$self->{'in_hash'}->{'driver'}";
$self->error($self->text('query_edriver', "<tt>DBD::$self->{'in_hash'}->{'driver'}</tt>")) if ($@);
$_[0]->{'driver'} = $self->{'in_hash'}->{'driver'};

$self->{'in_hash'}->{'db'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'query_edb'});
$_[0]->{'db'} = $self->{'in_hash'}->{'db'};

if ($self->{'in_hash'}->{'host_def'}) {
	delete($_[0]->{'host'});
	}
else {
	gethostbyname($self->{'in_hash'}->{'host'}) || $self->error($self->{'text_hash'}->{'query_ehost'});
	$_[0]->{'host'} = $self->{'in_hash'}->{'host'};
	}

$self->{'in_hash'}->{'quser'} =~ /^\S*$/ || $self->error($self->{'text_hash'}->{'query_euser'});
$_[0]->{'user'} = $self->{'in_hash'}->{'quser'};

$self->{'in_hash'}->{'qpass'} =~ /^\S*$/ || $self->error($self->{'text_hash'}->{'query_epass'});
$_[0]->{'pass'} = $self->{'in_hash'}->{'qpass'};

$self->{'in_hash'}->{'sql'} =~ /\S/ || $self->error($self->{'text_hash'}->{'query_esql'});
$_[0]->{'sql'} = $self->{'in_hash'}->{'sql'};

if ($self->{'in_hash'}->{'result_def'}) {
	delete($_[0]->{'result'});
	}
else {
	$self->{'in_hash'}->{'result'} =~ /\S/ || $self->error($self->{'text_hash'}->{'query_eresult'});
	$_[0]->{'result'} = $self->{'in_hash'}->{'result'};
	}
}

sub make_dbistr
{
	my $self = shift;

local ($driver, $db, $host) = @_;
local $rv;
if ($driver eq "mysql") {
	$rv = "database=$db";
	}
elsif ($driver eq "Pg") {
	$rv = "dbname=$db";
	}
else {
	$rv = $db;
	}
if ($host) {
	$rv .= ";host=$host";
	}
return $rv;
}


1;

