#!/usr/bin/perl
package status::StatusLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'status::StatusLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# status-lib.pl
# Functions for getting the status of services

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();

$self->{'services_dir'} = "$self->{'module_config_directory'}/services";
$self->{'cron_cmd'} = "$self->{'module_config_directory'}/monitor.pl";

$self->{'oldstatus_file'} = "$self->{'module_config_directory'}/oldstatus";
$self->{'fails_file'} = "$self->{'module_config_directory'}/fails";

%{$self->{'monitor_os_support_hash'}} = ( 'traffic' => { 'os_support' => '*-linux freebsd' },
		      );

1;

1;
		return $self;
}

# list_services()
# Returns a list of all services this module knows how to get status on.
# If this is the first time the function is called a default set of services
# will be setup.
sub list_services
{
	my $self = shift;

my (%mod, @rv);
if (!-d $self->{'services_dir'}) {
	# setup initial services
	mkdir($self->{'module_config_directory'}, 0700);
	mkdir($self->{'services_dir'}, 0700);
	system("cp services/* $self->{'services_dir'}");
	}
map { $mod{$_}++ } $self->list_modules();
opendir(DIR, $self->{'services_dir'});
while($self->{'f'} = readdir(DIR)) {
	next if ($self->{'f'} !~ /^(.*)\.serv$/);
	local $serv = $self->get_service($1);
	next if (!$serv || !$serv->{'type'} || !$serv->{'id'});
	if ($serv->{'depends'}) {
		local $d;
		map { $d++ if (!$mod{$_}) } split(/\s+/, $serv->{'depends'});
		push(@rv, $serv) if (!$d);
		}
	else {
		push(@rv, $serv);
		}
	}
closedir(DIR);
return @rv;
}

# get_service(id)
sub get_service
{
	my $self = shift;

local %serv;
$self->read_file("$self->{'services_dir'}/$_[0].serv", \%serv);
$serv{'fails'} = 1 if (!defined($serv{'fails'}));
$serv{'_file'} = "$self->{'services_dir'}/$_[0].serv";
if (!defined($serv{'notify'})) {
	$serv{'notify'} = 'email pager snmp';
	}
$serv{'remote'} = "*" if (!$serv{'remote'} && !$serv{'groups'});
return $_[0] ne $serv{'id'} ? undef : \%serv;
}

# save_service(&serv)
sub save_service
{
	my $self = shift;

mkdir($self->{'services_dir'}, 0755) if (!-d $self->{'services_dir'});
$self->lock_file("$self->{'services_dir'}/$_[0]->{'id'}.serv");
$self->write_file("$self->{'services_dir'}/$_[0]->{'id'}.serv", $_[0]);
$self->unlock_file("$self->{'services_dir'}/$_[0]->{'id'}.serv");
}

# delete_service(serv)
sub delete_service
{
	my $self = shift;

$self->lock_file("$self->{'services_dir'}/$_[0]->{'id'}.serv");
unlink("$self->{'services_dir'}/$_[0]->{'id'}.serv");
$self->unlock_file("$self->{'services_dir'}/$_[0]->{'id'}.serv");
}

# expand_remotes(&service)
# Given a service with direct and group remote hosts, returns a list of the
# names of all actual hosts (* means local)
sub expand_remotes
{
	my $self = shift;

local @remote;
push(@remote, split(/\s+/, $_[0]->{'remote'}));
local @groupnames = split(/\s+/, $_[0]->{'groups'});
if (@groupnames) {
	$self->foreign_require("servers", "servers-lib.pl");
	local @groups = $main::servers_object->list_all_groups();
	foreach my $g (@groupnames) {
		local ($group) = grep { $_->{'name'} eq $g } @groups;
		if ($group) {
			push(@remote, @{$group->{'members'}});
			}
		}
	}
return $self->unique(@remote);
}

# service_status(&service, [from-cgi])
# Gets the status of a service, possibly on another server. If called in
# an array content, the status of all hosts for this monitor are returned.
sub service_status
{
	my $self = shift;

local $t = $_[0]->{'type'};
local @rv;
foreach $r ($self->expand_remotes($_[0])) {
	local $rv;
	local $main::error_must_die = 1;
	eval {
		local $SIG{'ALRM'} = sub { die "status alarm\n" };
		alarm(60);	# wait at most 60 secs for a result
		if ($r ne "*") {
			# Make a remote call to another webmin server
			$self->remote_error_setup(\&remote_error);
			$self->{'remote_error_msg'} = undef;
			$self->remote_foreign_require($r, 'status', 'status-lib.pl')
				if (!$self->{'done_remote_status_hash'}->{$r}++);
			local $webmindown = $self->{'s'}->{'type'} eq 'alive' ? 0 : -2;
			if ($self->{'remote_error_msg'}) {
				$rv = { 'up' => $webmindown,
					 'desc' => "$self->{'text_hash'}->{'mon_webmin'} : $self->{'remote_error_msg'}" };
				}
			else {
				local %s = %{$_[0]};
				$s{'remote'} = '*';
				$s{'groups'} = undef;
				($rv) = $self->remote_foreign_call($r, 'status',
					    'service_status', \%s, $_[1]);
				if ($self->{'remote_error_msg'}) {
					$rv = { 'up' => $webmindown, 'desc' =>
					    "$self->{'text_hash'}->{'mon_webmin'} : $self->{'remote_error_msg'}" };
					}
				}
			}
		elsif ($t =~ /^(\S+)::(\S+)$/) {
			# Call to another module
			local ($mod, $mtype) = ($1, $2);
			$self->foreign_require($mod, "status_monitor.pl");
			$rv = $self->foreign_call($mod, "status_monitor_status",
					    $mtype, $_[0], $_[1]);
			}
		else {
			# Just include and use the local monitor library
		if (!$self->{'done_monitor_hash'}->{$t}++)
		{
		 {
    my $pkg = $self->lib_to_pkg("${t}-monitor.pl", "status");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
		}
			local $func = "get_${t}_status";
			$rv = $self->$func($_[0],
				     $_[0]->{'clone'} ? $_[0]->{'clone'} : $t,
				     $_[1]);
			}
		alarm(0);
		};
	if ($@ =~ /status alarm/) {
		push(@rv, { 'up' => -3,
			    'remote' => $r });
		}
	elsif ($@) {
		# A real error happened
		die $@;
		}
	else {
		$rv->{'remote'} = $r;
		push(@rv, $rv);
		}
	}
return wantarray ? @rv : $rv[0];
}

sub remote_error
{
	my $self = shift;

$self->{'remote_error_msg'} = join("", @_);
}

# list_modules()
# Returns a list of all modules available on this system
sub list_modules
{
	my $self = shift;

return map { $_->{'dir'} } grep { $self->check_os_support($_) }
	$self->get_all_module_infos();
}

# list_handlers()
# Returns a list of the module's monitor type handlers, and those
# defined in other modules.
sub list_handlers
{
	my $self = shift;

local ($f, @rv);
opendir(DIR, ".");
while($f = readdir(DIR)) {
	if ($f =~ /^(\S+)-monitor\.pl$/) {
		local $m = $1;
		local $oss = $self->{'monitor_os_support_hash'}->{$m};
		next if ($oss && !$self->check_os_support($oss));
		push(@rv, [ $m, $self->{'text_hash'}->{"type_$m"} ]);
		}
	}
closedir(DIR);
local $m;
foreach $m ($self->get_all_module_infos()) {
	local $mdir = $self->UNIVERSAL::can('module_root_directory') ?
		$self->module_root_directory($m->{'dir'}) :
		"$self->{'root_directory'}/$m->{'dir'}";
	if (-r "$mdir/status_monitor.pl" &&
	    $self->check_os_support($m)) {
		$self->foreign_require($m->{'dir'}, "status_monitor.pl");
		local @mms = $self->foreign_call($m->{'dir'}, "status_monitor_list");
		push(@rv, map { [ $m->{'dir'}."::".$_->[0], $_->[1] ] } @mms);
		}
	}
return @rv;
}

# depends_check(&service, [module]+)
sub depends_check
{
	my $self = shift;

return if ($_[0]->{'id'});	# only check for new services
if ($_[0]->{'remote'}) {
	# Check on the remote server
	foreach $m (@_[1..$#_]) {
		$self->remote_foreign_check($_[0]->{'remote'}, $m) ||
			$self->error($self->text('depends_remote', "<tt>$m</tt>",
				     "<tt>$_[0]->{'remote'}</tt>"));
		}
	}
else {
	# Check on this server
	foreach $m (@_[1..$#_]) {
		local %minfo = $self->get_module_info($m);
		%minfo || $self->error($self->text('depends_mod', "<tt>$m</tt>"));
		$self->check_os_support(\%minfo) ||
			$self->error($self->text('depends_os', "<tt>$minfo{'desc'}</tt>"));
		}
	$_[0]->{'depends'} = join(" ", @_[1..$#_]);
	}
}

# find_named_process(regexp)
sub find_named_process
{
	my $self = shift;

foreach $p ($self->foreign_call("proc", "list_processes")) {
	$p->{'args'} =~ s/\s.*$//; $p->{'args'} =~ s/[\[\]]//g;
	if ($p->{'args'} =~ /$_[0]/) {
		return $p;
		}
	}
return undef;
}

# smtp_command(handle, command)
sub smtp_command
{
	my $self = shift;

local ($m, $c) = @_;
print $m $c;
local $r = <$m>;
if ($r !~ /^[23]\d+/) {
	$self->error($self->text('sched_esmtpcmd', "<tt>$c</tt>", "<tt>$r</tt>"));
	}
}

# setup_cron_job()
# Create a cron job based on the module's configuration
sub setup_cron_job
{
	my $self = shift;

$self->lock_file($self->{'cron_cmd'});
$self->foreign_require("cron", "cron-lib.pl");
local ($j, $job);
foreach $j ($self->foreign_call("cron", "list_cron_jobs")) {
	$job = $j if ($j->{'user'} eq 'root' && $j->{'command'} eq $self->{'cron_cmd'});
	}
if ($job) {
	$self->lock_file($main::cron_object->cron_file($job));
	$self->foreign_call("cron", "delete_cron_job", $job);
	$self->unlock_file($main::cron_object->cron_file($job));
	unlink($self->{'cron_cmd'});
	}
if ($self->{'config_hash'}->{'sched_mode'}) {
	# Create the program that cron calls
	$main::cron_object->create_wrapper($self->{'cron_cmd'}, $self->{'module_name'}, "monitor.pl");

	# Setup the actual cron job
	local $njob;
	$njob = { 'user' => 'root', 'active' => 1,
		  'hours' => '*', 'days' => '*',
		  'months' => '*', 'weekdays' => '*',
		  'command' => $self->{'cron_cmd'} };
	if ($self->{'config_hash'}->{'sched_period'} == 0) {
		$njob->{'mins'} = $self->make_interval(60);
		}
	elsif ($self->{'config_hash'}->{'sched_period'} == 1) {
		$njob->{'hours'} = $self->make_interval(24);
		$njob->{'mins'} = 0;
		}
	elsif ($self->{'config_hash'}->{'sched_period'} == 2) {
		$njob->{'days'} = $self->make_interval(31, 1);
		$njob->{'hours'} = $njob->{'mins'} = 0;
		}
	elsif ($self->{'config_hash'}->{'sched_period'} == 3) {
		$njob->{'months'} = $self->make_interval(12, 1);
		$njob->{'days'} = 1;
		$njob->{'hours'} = $njob->{'mins'} = 0;
		}
	$self->lock_file($main::cron_object->cron_file($njob));
	$self->foreign_call("cron", "create_cron_job", $njob);
	$self->unlock_file($main::cron_object->cron_file($njob));
	}
$self->unlock_file($self->{'cron_cmd'});
}

# make_interval(length, offset2)
sub make_interval
{
	my $self = shift;

local (@rv, $i);
for($i=$self->{'config_hash'}->{'sched_offset'}+$_[1]; $i<$_[0]; $i+=$self->{'config_hash'}->{'sched_int'}) {
	push(@rv,$i);
	}
return join(",", @rv);
}

# expand_oldstatus(oldstatus, &serv)
# Converts an old-status string like *=1 foo.com=2 into a hash. If the string
# contains just one number, it is assumed to be for just the first remote host
sub expand_oldstatus
{
	my $self = shift;

local ($o, $serv) = @_;
local @remotes = split(/\s+/, $serv->{'remote'});
if ($o =~ /^\-?(\d+)$/) {
	return { $remotes[0] => $o };
	}
else {
	local %rv;
	foreach my $hs (split(/\s+/, $o)) {
		local ($h, $s) = split(/=/, $hs);
		$rv{$h} = $s;
		}
	return \%rv;
	}
}

# nice_remotes(&monitor, [max])
sub nice_remotes
{
	my $self = shift;

local ($s, $max) = @_;
$max ||= 3;
local @remotes = map { $_ eq "*" ? $self->{'text_hash'}->{'index_local'}
			         : $self->html_escape($_) }
		     split(/\s+/, $s->{'remote'});
foreach my $g (split(/\s+/, $s->{'groups'})) {
	push(@remotes, $self->text('index_group', $g));
	}
return @remotes > $max ? join(", ", @remotes[0..$max]).", ..."
		       : join(", ", @remotes);
}

sub group_desc
{
	my $self = shift;

local ($group) = @_;
local $mems = scalar(@{$group->{'members'}});
return $group->{'name'}." (".
       $self->text($mems == 0 ? 'mon_empty' :
	     $mems == 1 ? 'mon_onemem' : 'mon_members', $mems).")";
}

# list_notification_modes()
# Returns a list of available notifcation modes (like email, sms, etc..)
sub list_notification_modes
{
	my $self = shift;

local @rv = ( "email" );
if ($self->{'config_hash'}->{'pager_cmd'} && $self->{'config_hash'}->{'sched_pager'}) {
	push(@rv, "pager");
	}
if ($self->{'config_hash'}->{'snmp_server'}) {
	local $gotmod = 0;
	eval "use Net::SNMP";
	$gotmod++ if (!$@);
	eval "use SNMP_Session";
	$gotmod++ if (!$@);
	push(@rv, "snmp") if ($gotmod);
	}
if ($self->{'config_hash'}->{'sched_carrier'} && $self->{'config_hash'}->{'sched_sms'}) {
	push(@rv, "sms");
	}
return @rv;
}

# list_sms_carriers()
# Returns a list of information about carriers to whom we can send SMS
sub list_sms_carriers
{
	my $self = shift;

return ( { 'id' => 'tmobile',
	   'desc' => 'T-Mobile',
	   'domain' => 'tmomail.net' },
	 { 'id' => 'cingular',
	   'desc' => 'Cingular / AT&T',
	   'domain' => 'cingularme.com' },
	 { 'id' => 'verizon',
	   'desc' => 'Verizon',
	   'domain' => 'vtext.com' },
	 { 'id' => 'sprint',
	   'desc' => 'Sprint PCS',
	   'domain' => 'messaging.sprintpcs.com' },
	 { 'id' => 'nextel',
	   'desc' => 'Nextel',
	   'domain' => 'messaging.nextel.com' },
	 { 'id' => 'alltel',
	   'desc' => 'Alltel',
	   'domain' => 'message.alltel.com' },
         { 'id' => 'boost',
	   'desc' => 'Boost Mobile',
	   'domain' => 'myboostmobile.com' },
	 { 'id' => 'virgin',
	   'desc' => 'Virgin Mobile',
	   'domain' => 'vmobl.com' },
        );
}


1;

