# swup-lib.pl
# Common functions for SWUP

do '../web-lib.pl';
&init_config();
require '../ui-lib.pl';

&foreign_require('proc', 'proc-lib.pl');

if ($config{'swup_path'} =~ /([^\/]+)$/) {
	$swup_shortname = $1;
	# get swup version
	$swup_version = &get_swup_version(\$swup_version_output);
}

if (!$config{'rpm_path'}) {
	$config{'rpm_path'} = '/bin/rpm';
}

# checkup_swup_path()
sub check_swup_path {
if (!-x $config{'swup_path'}) {
	&header($text{'index_title'}, "", "intro", 1, 1);
	print "<hr>\n";
	print &text('index_e_swup_path', "<tt>$config{'swup_path'}</tt>",
		    "$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";
	print "<hr>\n";
	&footer("/", $text{"index"});
	exit;
	}
}

# checkup_rpm_path()
sub check_rpm_path {
if (!-x $config{'rpm_path'}) {
	&header($text{'index_title'}, "", "intro", 1, 1);
	print "<hr>\n";
	print &text('index_e_rpm_path', "<tt>$config{'rpm_path'}</tt>",
		    "$gconfig{'webprefix'}/config.cgi?$module_name"),"<p>\n";
	print "<hr>\n";
	&footer("/", $text{"index"});
	exit;
	}
}

# get_swup_version(&out)
sub get_swup_version {
	local $out = `$config{'swup_path'} --version 2>&1`;
	${$_[0]} = $out;
	return $out =~ /software\s+updater\s+(\S+)/i ? $1 : undef;
}

# get_installed_rpms_list()
sub get_installed_rpms_list {
	local @out = `$config{'rpm_path'} -qa --queryformat "%{NAME}\t%{VERSION}\t%{RELEASE}\t%{SUMMARY}\t%{INSTALLTIME}\n" 2>/dev/null`;
	local @rv = ();
	foreach my $line (@out) {
		next unless $line =~ /^([^\t]+)\t([^\t]+)\t([^\t]+)\t([^\t]+)\t([^\t]+)$/;
		my $info = {
			'name' => $1,
			'version' => $2,
			'release' => $3,
			'summary' => $4,
			'installtime' => scalar(localtime($5)),
		};
		push(@rv, $info);
	}
	return @rv;
}

# print_installed_rpms_list()
sub print_installed_rpms_list {
	local @rpms = @_;
	print "<table border width=100%>\n";
	print "<tr $tb><td><b>$text{'index_name'}</b></td> ",
	      "<td><b>$text{'index_summary'}</b></td> ",
	      "<td><b>$text{'index_version'}</b></td> ",
	      "<td><b>$text{'index_installed_on'}</b></td> </tr>\n";
	foreach my $rpm (sort {$a->{name} cmp $b->{name}} @rpms) {
		print "<tr $cb>\n";
		my ($name, $summary, $ver, $date, $name_url) = (
			&html_escape($rpm->{name}),
			&html_escape($rpm->{summary}),
			&html_escape("$rpm->{'version'}-$rpm->{'release'}"),
			&html_escape($rpm->{installtime}),
			&urlize($rpm->{name})
		);
		print "<td><a href='package_info.cgi?pkg=$name_url'>$name</a></td><td>$summary</td><td>$ver</td><td>$date</td>\n";
		print "</tr>\n";
	}
	print "</table>\n";
}

# --------- upgrade ---------

# get_upgrade_pkgs_list()
sub get_upgrade_pkgs_list {
	local @out = `$config{'swup_path'} --list-upgrade --config-file $config{'swup_conf'} 2>/dev/null`;
	local @rv = ();
	foreach my $line (@out) {
		next if $line !~ /^\s*(\S+)\s*$/ && $line !~ /^\s*(\S+)\s+\(.+vs\..+\)\s*$/;
		push(@rv, {'name' => $1});
	}
	return @rv;
}

# print_upgrade_pkgs_list()
sub print_upgrade_pkgs_list {
	local @pkgs = @_;
	my ($cols, $col) = (4, 0);

	print "<form action='upgrade_pkgs.cgi' method=post>";
	print &select_all_link("pkg", 0, undef),"&nbsp;\n";
   print &select_invert_link("pkg", 0, undef),"&nbsp;\n";
	print "<table border width=100%>\n";
	print "<tr $tb><td colspan=$cols><b>$text{'upgrade_packages'}</b></td></tr>\n";
	foreach my $p (sort {$a->{'name'} cmp $b->{'name'}} @pkgs) {
		print "<tr $cb>\n" unless $col;
		my ($name, $pkg_url) = (&html_escape($p->{'name'}), &urlize($p->{'name'}));
		print "<td><input type=checkbox name=pkg value='pkg_$p->{'name'}'>&nbsp;<a href='list_upgrade.cgi?pkg=$pkg_url'>$name</a></td>";
		$col++;
		if ($col == $cols) {print "\n</tr>\n"; $col = 0;};
	}
	if ($col < $cols && $col != 0) {
		foreach ($col .. $cols-1) {
			print "<td>&nbsp;</td>";
		}
		print "</tr>";
	}
	print "<tr><td colspan=$cols></td></tr>";
	print "</table>\n";
	print &select_all_link("pkg", 0, undef),"&nbsp;\n";
   print &select_invert_link("pkg", 0, undef),"&nbsp;\n";
	print "<br><input type=submit name=do_upgrade value='$text{upgrade_upgrade}'></form>\n";
}

# --------- install ---------

# get_install_pkgs_list()
sub get_install_pkgs_list {
	local @out = `$config{'swup_path'} --list-prospect --config-file $config{'swup_conf'} 2>/dev/null`;
	local @rv = ();
	foreach my $line (@out) {
		my $info;
		if ($line =~ /^([^\s]+)\s+([^\s]+)\s+-\s+(.+)$/) {
			$info = {
				'name' => $1,
				'version' => $2,
				'summary' => $3,
			};
			push(@rv, $info);
		}
		elsif ($line =~ /^([^\s]+)\s+-\s+(.+)$/) {
			$info->{'summary'} = $2;
			next unless $1 =~ /^(\D+)-(\d+.*)$/;
			$info->{'name'} = $1;
			$info->{'version'} = $2;
			push(@rv, $info);
		}
	}
	return @rv;
}

# print_install_pkgs_list()
sub print_install_pkgs_list {
	local @pkgs = @_;

	print "<form action='install_pkgs.cgi' method=post>";
	print &select_all_link("pkg", 0, undef),"&nbsp;\n";
	print &select_invert_link("pkg", 0, undef),"&nbsp;\n";	
	print "<table border width=100%>\n";
	print "<tr $tb><td><b>$text{'install_name'}</b></td> ",
	      "<td><b>$text{'install_version'}</b></td> ",
	      "<td><b>$text{'install_summary'}</b></td></tr>\n";
	foreach my $p (sort {$a->{name} cmp $b->{name}} @pkgs) {
		print "<tr $cb>\n";
		my ($name, $summary, $ver) = (
			&html_escape($p->{name}),
			&html_escape($p->{summary}),
			&html_escape($p->{version}),
		);
		print "<td><input type=checkbox name=pkg value='pkg_$p->{'name'}'>&nbsp;$name </td><td>$ver</td><td>$summary</td>\n";
		print "</tr>\n";
	}
	print "<tr><td colspan=3></td></tr>\n";
	print "</table>\n";
	print &select_all_link("pkg", 0, undef),"&nbsp;\n";
   print &select_invert_link("pkg", 0, undef),"&nbsp;\n";
	print "<input type=submit name=do_install value='$text{install_install}'></form>\n";
}

# get_remote_package_desc(package_name);
sub get_remote_package_desc {
	local $pkg = $_[0];
	local @out = `$config{'swup_path'} --describe $pkg --config-file $config{'swup_conf'} 2>/dev/null`;
	return @out;
}

# ========== processing packages ============
# install_package(package_name);
sub install_package {
	local $pkg = $_[0];
	local $dload_first = $config{'dload_first'} ? "--download-first" : "";
	print "<tr $tb><td colspan=2><b>$text{'info_installing'}</b></td></tr>\n";
	print "<tr $cb><td colspan=2><pre>";
	&proc::safe_process_exec("$config{'swup_path'} --install $pkg $dload_first --config-file $config{'swup_conf'}", 0, 0, STDOUT, undef, 1, 1);
	print "</pre></td></tr>\n";
}

# install_package_list(\@pkg_list);
sub install_package_list {
	local $pkgs = $_[0];
	if (@{$pkgs}) {
		local $file = &transname();
		if (open(TMP, ">$file")) {
			foreach (@{$pkgs}) { print TMP "$_\n"; }
			close(TMP);
		}
		local $dload_first = $config{'dload_first'} ? "--download-first" : "";
		print "<tr $tb><td colspan=2><b>$text{'info_installing'}</b></td></tr>\n";
		print "<tr $cb><td colspan=2><pre>";
		&proc::safe_process_exec("$config{'swup_path'} --install $pkgs->[0] $dload_first --package-file '$file' --config-file $config{'swup_conf'}", 0, 0, STDOUT, undef, 1, 1);
		print "</pre>\n";
		&error($text{'install_elist'}) if $?;
		print "</td></tr>\n";

	}
}

# upgrade_package(package_name);
sub upgrade_package {
	local $pkg = $_[0];
	local $dload_first = $config{'dload_first'} ? "--download-first" : "";
	print "<tr $tb><td colspan=2><b>$text{'info_upgrading'}</b></td></tr>\n";
	print "<tr $cb><td colspan=2><pre>";
	&proc::safe_process_exec("$config{'swup_path'} --upgrade $pkg $dload_first --config-file $config{'swup_conf'}", 0, 0, STDOUT, undef, 1, 1);
	print "</pre></td></tr>\n";
}

# upgrade_package_list(\@pkg_list);
sub upgrade_package_list {
	local $pkgs = $_[0];
	if (@{$pkgs}) {
		local $file = &transname();
		if (open(TMP, ">$file")) {
			foreach (@{$pkgs}) { print TMP "$_\n"; }
			close(TMP);
		}
		local $conf_name = $config{'swup_conf'};
		local $dload_first = $config{'dload_first'} ? "--download-first" : "";
		print "<tr $tb><td colspan=2><b>$text{'info_upgrading'}</b></td></tr>\n";
		print "<tr $cb><td colspan=2><pre>";
		if ($config{'upgrade_list'}) {
			&proc::safe_process_exec("$config{'swup_path'} --upgrade $dload_first --package-file '$file' --config-file $conf_name", 0, 0, STDOUT, undef, 1, 1);
		} else {
			foreach my $p (@{$pkgs}) {
				&proc::safe_process_exec("$config{'swup_path'} --upgrade $p $dload_first --config-file $conf_name", 0, 0, STDOUT, undef, 1, 1);
			}
		}
		print "</pre>\n";
		&error($text{'upgrade_elist'}) if $?;
		print "</td></tr>\n";
	}
}

# upgrade_all_packages()
# upgrade all system packages
sub upgrade_all_packages
{
 my $conf_name = $config{'swup_conf'};
 print "<tr $tb><td colspan=2><b>$text{'info_upgrading'}</b></td></tr>\n";
 print "<tr $cb><td colspan=2><pre>";
 &proc::safe_process_exec("$config{'swup_path'} --upgrade --config-file $conf_name", 0, 0, STDOUT, undef, 1, 1);
 print "</pre>\n";
 &error($text{'upgrade_elist'}) if $?;
 print "</td></tr>\n";
}

# uninstall_package(package_name);
sub uninstall_package {
	local $pkg = $_[0];
	print "<tr $tb><td colspan=2><b>$text{'info_uninstalling'}</b></td></tr>\n";
	print "<tr $cb><td colspan=2><pre>";
	&proc::safe_process_exec("$config{'swup_path'} --remove $pkg --config-file $config{'swup_conf'}", 0, 0, STDOUT, undef, 1, 1);
	print "</pre></td></tr>\n";
}

# get_installed_rpms_list()
sub get_package_desc {
	local $pkg = $_[0];
	local @out = `$config{'rpm_path'} -qi $pkg --queryformat "%{NAME}\t%{VERSION}\t%{RELEASE}\t%{INSTALLTIME}\n" 2>/dev/null`;
	local $info = undef;
	$count = scalar(@out) - 1;
	$line = $out[$count];
	if ($line =~ /^([^\t]+)\t([^\t]+)\t([^\t]+)\t([^\t]+)$/) {
		$info = {
			'name' => $1,
			'version' => "$2-$3",
			'installtime' => scalar(localtime($4)),
		};
		pop(@out);
	}
	return ($info, join('', @out));
}

# return list all available package(s) whose name matches  a  given  pattern.
# &search_package($pattern);
sub search_package {
        local $pattern = $_[0];
        local @out = `$config{'swup_path'} --search-package $pattern --config-file $config{'swup_conf'} 2>/dev/null`;
        local @rv = ();
        foreach my $line (@out) {
                my $info;
                if ($line =~ /^(\S+)\s+\(([\d\.]+-\S+)\)$/) {
                        $info = {
                                'name' => $1,
                                'version' => $2,
                        };
                        push(@rv, $info);
                }
        }
        return @rv;
}

# parses a file and returns a reference to an array of hashes
# &parse_config_file(\&conf, $config_file_name, [$split_char]);
sub parse_config_file {
	open(INFL, $_[1]) || &error($text{'error_open'});
	local $line = 0;
	local $sp = $_[2] ? $_[2] : "=";
	local $conf = $_[0];
	while (<INFL>) {
		chomp;
		s/#.*$//;
		s/^\s*//;
		if (/^([^$sp]+)$sp(.*)$/) {
		push (@{$conf}, 
				{'line' => $line,
				'name' => &trunc_space($1),
				'value' => &trunc_space($2)});
		}
		$line++;
	}
	close INFL;
}

# returns a value of a parameter
# &get_config_value(\@conf, $variable);
sub get_config_value {
	foreach (@{$_[0]}) {
		return $_->{'value'} if $_->{'name'} eq $_[1]
	}
}

# sets a new value to the given parameter
# &set_config_value(\@conf, $variable, $value);
sub set_config_value {
	local $conf = $_[0];
	foreach (@{$conf}) {
		if ($_->{'name'} eq $_[1]) {
			$_->{'value'} = $_[2];
			return;
		}
	}
	push(@{$conf}, {'name' => $_[1], 'value' => $_[2]});
}

# saves config file
# &save_config_file(\@conf, $config_file_name, [$splitter]);
sub save_config_file {
	&lock_file($_[1]);
	local $conf = $_[0];
	local $sp = $_[2] ? $_[2] : "=";
	local $old_file = &read_file_lines($_[1]);
	local ($line, $str_new, $i) = (0, undef, 0);
	
	foreach (@{$old_file}) {
		my $l = $_;
		$l =~ s/#.*$//;
		$l =~ s/^\s*//;
		if ($l =~ /^([^$sp]+)$sp(.*)$/) {
			$str_new = $conf->[$i];
			if ($str_new->{'line'} = $line && $str_new->{'name'} eq &trunc_space($1) && $str_new->{'value'} ne &trunc_space($2)) {
				$old_file->[$line] = "$str_new->{'name'}$sp$str_new->{'value'}";
			}
			$i++;
		}
		$line++;
	}
	local ($rep_str, $m);
	$m=@{$conf};
	if ($line < $m) {
		for (my $k = $i; $k < $m; $k++) {
			$str_new=@{$conf}[$k];
			$str_new="$str_new->{'name'}$sp$str_new->{'value'}";
			foreach (@{$old_file}) {
				$rep_str = 1 if ($_ eq $str_new);
			}
			push (@{$old_file}, $str_new) if ($rep_str != 1);
		}
	}
	flush_file_lines($_[1]);
	&unlock_file($_[1]);
}

#delete spaces from string $_[0]
sub trunc_space
{
 local $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}

# &duplicate_config_file($config_file_name);
sub duplicate_config_file {
	local $tmp_file = &transname();
	local $lines = &read_file_lines($_[0]);
	&error("Failed to write to $tmp_file : $!") unless &open_tempfile(FILE, ">$tmp_file");
	&print_tempfile(FILE, join("\n", @{$lines}));
	&close_tempfile(FILE);
	return $tmp_file;
}

# --list-upgrade --config-file $config{'swup_path'}
1;
1;
