#!/usr/bin/perl
package syslog::SyslogLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'syslog::SyslogLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# syslog-lib.pl
# Functions for the syslog module

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();

1;

1;
		return $self;
}

# get_config()
sub get_config
{
	my $self = shift;

local $lnum = 0;
local ($line, $cont, @rv);
local $tag = { 'tag' => '*',
	       'index' => 0,
	       'line' => 0 };
push(@rv, $tag);
$self->open_readfile(*CONF, $self->{'config_hash'}->{'syslog_conf'});
while($line = <CONF>) {
	local $slnum = $lnum;
	$line =~ s/\r|\n//g;
	if ($line =~ /\\$/) {
		# continuation .. get the next lines
		$line =~ s/\\$//;
		while($cont = <CONF>) {
			$lnum++;
			$cont =~ s/^[#\s]+//;
			$cont =~ s/\r|\n//g;
			$line .= $cont;
			last if ($line !~ s/\\$//);
			}
		}
	if ($line =~ /^(#*)\s*([^#\s]+\.\S+)\s+(\S+)$/ ||
	    $line =~ /^(#*)\s*([^#\s]+\.\S+)\s+(\|.*)$/) {
		local $act = $3;
		local $log = { 'active' => !$1,
			       'sel' => [ split(/;/, $2) ],
			       'line' => $slnum,
			       'eline' => $lnum };
		if ($act =~ /^\-(\/\S+)$/) {
			$log->{'file'} = $1;
			$log->{'sync'} = 0;
			}
		elsif ($act =~ /^\|(.*)$/) {
			$log->{'pipe'} = $1;
			}
		elsif ($act =~ /^(\/\S+)$/) {
			$log->{'file'} = $1;
			$log->{'sync'} = 1;
			}
		elsif ($act =~ /^\@\@(\S+)$/) {
			$log->{'socket'} = $1;
			}
		elsif ($act =~ /^\@(\S+)$/) {
			$log->{'host'} = $1;
			}
		elsif ($act eq '*') {
			$log->{'all'} = 1;
			}
		else {
			$log->{'users'} = [ split(/,/, $act) ];
			}
		$log->{'index'} = scalar(@rv);
		$log->{'section'} = $tag;
		$tag->{'eline'} = $lnum;
		push(@rv, $log);
		}
	elsif ($line =~ /^(#?)!(\S+)$/) {
		push(@rv, { 'tag' => $2,
			    'index' => scalar(@rv),
			    'line' => $lnum,
			    'eline' => $lnum });
		$tag = $rv[@rv-1];
		}
	$lnum++;
	}
close(CONF);
return \@rv;
}

# create_log(&log)
sub create_log
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'syslog_conf'});
if ($self->{'config_hash'}->{'tags'}) {
	splice(@$lref, $_[0]->{'section'}->{'eline'}+1, 0, $self->log_line($_[0]));
	}
else {
	push(@$lref, $self->log_line($_[0]));
	}
$self->flush_file_lines();
}

# update_log(&old, &log)
sub update_log
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'syslog_conf'});
if ($self->{'config_hash'}->{'tags'} && $_[0]->{'section'} ne $_[1]->{'section'}) {
	if ($_[0]->{'section'}->{'line'} < $_[1]->{'section'}->{'line'}) {
		splice(@$lref, $_[1]->{'section'}->{'eline'}+1, 0,
		       $self->log_line($_[1]));
		splice(@$lref, $_[0]->{'line'},
		       $_[0]->{'eline'} - $_[0]->{'line'} + 1);
		}
	else {
		splice(@$lref, $_[0]->{'line'},
		       $_[0]->{'eline'} - $_[0]->{'line'} + 1);
		splice(@$lref, $_[1]->{'section'}->{'eline'}+1, 0,
		       $self->log_line($_[1]));
		}
	}
else {
	splice(@$lref, $_[0]->{'line'}, $_[0]->{'eline'} - $_[0]->{'line'} + 1,
	       $self->log_line($_[1]));
	}
$self->flush_file_lines();
}

# delete_log(&log)
sub delete_log
{
	my $self = shift;

local $lref = $self->read_file_lines($self->{'config_hash'}->{'syslog_conf'});
splice(@$lref, $_[0]->{'line'}, $_[0]->{'eline'} - $_[0]->{'line'} + 1);
$self->flush_file_lines();
}

sub log_line
{
	my $self = shift;

local $d;
if ($_[0]->{'file'}) {
	$d = ($_[0]->{'sync'} || !$self->{'config_hash'}->{'sync'} ? "" : "-").$_[0]->{'file'};
	}
elsif ($_[0]->{'pipe'}) {
	$d = '|'.$_[0]->{'pipe'};
	}
elsif ($_[0]->{'host'}) {
	$d = '@'.$_[0]->{'host'};
	}
elsif ($_[0]->{'users'}) {
	$d = join(",", @{$_[0]->{'users'}});
	}
elsif ($_[0]->{'socket'}) {
	$d = '@@'.$_[0]->{'socket'};
	}
else {
	$d = '*';
	}
return ($_[0]->{'active'} ? "" : "#").join(";", @{$_[0]->{'sel'}})."\t".$d;
}

# list_priorities()
# Returns a list of all priorities
sub list_priorities
{
	my $self = shift;

return ( 'debug', 'info', 'notice', 'warning',
	 'err', 'crit', 'alert', 'emerg' );
}

# can_edit_log(&log)
# Returns 1 if some log can be viewed/edited, 0 if not
sub can_edit_log
{
	my $self = shift;

return 1 if (!$self->{'access_hash'}->{'logs'});
local @files = split(/\s+/, $self->{'access_hash'}->{'logs'});
local $lf;
if (ref($_[0])) {
	$lf = $_[0]->{'file'} || $_[0]->{'pipe'} || $_[0]->{'host'} ||
	      $_[0]->{'socket'} || $_[0]->{'cmd'} ||
	      ($_[0]->{'all'} ? "*" : "users");
	}
else {
	$lf = $_[0];
	}
foreach $f (@files) {
	return 1 if ($f eq $lf || $self->is_under_directory($f, $lf));
	}
return 0;
}

sub needs_m4
{
	my $self = shift;

local $oldslash = $/;
$/ = undef;
$self->open_readfile(*CONF, $self->{'config_hash'}->{'syslog_conf'});
local $conf1 = <CONF>;
close(CONF);
$self->open_execute_command(*CONF, "$self->{'config_hash'}->{'m4_path'} $self->{'config_hash'}->{'syslog_conf'}", 1, 1);
local $conf2 = <CONF>;
close(CONF);
$/ = $oldslash;
return $conf1 ne $conf2;
}

# get_syslog_pid(pid)
# Returns the syslog PID file
sub get_syslog_pid
{
	my $self = shift;

local $pid;
if ($self->{'config_hash'}->{'pid_file'}) {
	if ($self->open_readfile(*PID, $self->{'config_hash'}->{'pid_file'}) &&
	    <PID> =~ /^(\d+)$/ && kill(0, $1)) {
		$pid = $1;
		}
	}
else {
	($pid) = $self->find_byname("syslogd");
	}
return $pid;
}

# restart_syslog()
# Stop and re-start the syslog server. Returns an error message on failure.
sub restart_syslog
{
	my $self = shift;

if ($self->{'config_hash'}->{'restart_cmd'}) {
	$self->system_logged("$self->{'config_hash'}->{'restart_cmd'} >/dev/null 2>/dev/null </dev/null");
	}
else {
	local $pid = $self->get_syslog_pid();
	$pid && $self->kill_logged('TERM', $pid) ||
		return $self->text('restart_ekill', $pid, $!);
	sleep(2);
	if ($self->{'config_hash'}->{'start_cmd'}) {
		$self->system_logged("$self->{'config_hash'}->{'start_cmd'} >/dev/null 2>/dev/null </dev/null");
		}
	else {
		$self->system_logged("cd / ; $self->{'config_hash'}->{'syslogd'} >/dev/null 2>/dev/null </dev/null &");
		}
	}
return undef;
}

# signal_syslog()
# Tell the syslog server to re-open it's log files
sub signal_syslog
{
	my $self = shift;

local $pid = $self->get_syslog_pid();
if ($pid) {
	$self->kill_logged('HUP', $pid);
	}
}

# all_log_files(file)
# Given a filename, returns all rotated versions, ordered by oldest first
sub all_log_files
{
	my $self = shift;

$_[0] =~ /^(.*)\/([^\/]+)$/;
local $dir = $1;
local $base = $2;
local ($f, @rv);
opendir(DIR, $self->translate_filename($dir));
foreach $f (readdir(DIR)) {
	local $trans = $self->translate_filename("$dir/$f");
	if ($f =~ /^\Q$base\E/ && -f $trans) {
		push(@rv, "$dir/$f");
		$self->{'mtime_hash'}->{"$dir/$f"} = [ stat($trans) ];
		}
	}
closedir(DIR);
return sort { $self->{'mtime_hash'}->{$a}->[9] <=> $self->{'mtime_hash'}->{$b}->[9] } @rv;
}

# get_other_module_logs([module])
# Returns a list of logs supplied by other modules
sub get_other_module_logs
{
	my $self = shift;

local ($mod) = @_;
local @rv;
local %done;
foreach my $minfo ($self->get_all_module_infos()) {
	next if ($mod && $minfo->{'dir'} ne $mod);
	next if (!$minfo->{'syslog'});
	next if (!$self->foreign_installed($minfo->{'dir'}));
	local $mdir = $self->module_root_directory($minfo->{'dir'});
	next if (!-r "$mdir/syslog_logs.pl");
	$self->foreign_require($minfo->{'dir'}, "syslog_logs.pl");
	local $j = 0;
	foreach my $l ($self->foreign_call($minfo->{'dir'}, "syslog_getlogs")) {
		local $fc = $l->{'file'} || $l->{'cmd'};
		next if ($done{$fc}++);
		$l->{'minfo'} = $minfo;
		$l->{'mod'} = $minfo->{'dir'};
		$l->{'mindex'} = $j++;
		push(@rv, $l);
		}
	}
@rv = sort { $a->{'minfo'}->{'desc'} cmp $b->{'minfo'}->{'desc'} } @rv;
local $i = 0;
foreach my $l (@rv) {
	$l->{'index'} = $i++;
	}
return @rv;
}

# catter_command(file)
# Given a file that may be compressed, returns the command to output it in
# plain text, or undef if impossible
sub catter_command
{
	my $self = shift;

local ($l) = @_;
local $q = quotemeta($l);
if ($l =~ /\.gz$/i) {
	return $self->has_command("gunzip") ? "gunzip -c $q" : undef;
	}
elsif ($l =~ /\.Z$/i) {
	return $self->has_command("uncompress") ? "uncompress -c $q" : undef;
	}
elsif ($l =~ /\.bz2$/i) {
	return $self->has_command("bunzip2") ? "bunzip2 -c $q" : undef;
	}
else {
	return "cat $q";
	}
}

# extra_log_files()
# Returns a list of extra log files available to the current Webmin user. No filtering
# based on allowed directory is done though!
sub extra_log_files
{
	my $self = shift;

local @rv;
foreach my $fd (split(/\t+/, $self->{'config_hash'}->{'extras'}), split(/\t+/, $self->{'access_hash'}->{'extras'})) {
	if ($fd =~ /^(\S+)\s+(\S.*)$/) {
		push(@rv, { 'file' => $1, 'desc' => $2 });
		}
	else {
		push(@rv, { 'file' => $fd });
		}
	}
return @rv;
}


1;

