#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Create, update or delete a destination

&ModuleLoader::preload_module('syslog_ng_class', "syslog_ng::SyslogngLib");
$self = new syslog_ng::SyslogngLib:: 'syslog_ng_class',%$self;

$self->ReadParse();
$self->error_setup($self->{'text_hash'}->{'destination_err'});

# Get the old destination
$self->{'conf'} = $self->get_config();
if (!$self->{'in_hash'}->{'new'}) {
	@{$self->{'dests_array'}} = $self->find("destination", $self->{'conf'});
	($self->{'dest'}) = grep { $_->{'value'} eq $self->{'in_hash'}->{'old'} } @{$self->{'dests_array'}};
	$self->{'dest'} || $self->error($self->{'text_hash'}->{'destination_egone'});
	$self->{'old'} = $self->{'dest'};
	}
else {
	$self->{'dest'} = { 'name' => 'destination',
		  'type' => 1,
		  'members' => [ ] };
	}

$self->lock_file($self->{'config_hash'}->{'syslogng_conf'});
if ($self->{'in_hash'}->{'delete'}) {
	# Just delete it!
	$self->check_dependencies('destination', $self->{'in_hash'}->{'old'}) &&
	    $self->error($self->text('ddelete_eused', $self->{'in_hash'}->{'old'}));
	$self->save_directive($self->{'conf'}, undef, $self->{'dest'}, undef, 0);
	}
else {
	# Validate inputs, and update object
	$self->{'in_hash'}->{'name'} =~ /^[a-z0-9_]+$/i || $self->error($self->{'text_hash'}->{'destination_ename'});
	if ($self->{'in_hash'}->{'new'} || $self->{'in_hash'}->{'old'} ne $self->{'in_hash'}->{'name'}) {
		($self->{'clash'}) = grep { $_->{'value'} eq $self->{'in_hash'}->{'name'} } @{$self->{'dests_array'}};
		$self->{'clash'} && $self->error($self->{'text_hash'}->{'destination_eclash'});
		}
	$self->{'dest'}->{'values'} = [ $self->{'in_hash'}->{'name'} ];

	# Clear out all existing values
	$self->{'dest'}->{'members'} = [ ];

	# Save type-specific values
	if ($self->{'in_hash'}->{'type'} == 0) {
		# Writing to a file
		$self->{'in_hash'}->{'file_name'} =~ /^\/\S/ ||
			$self->error($self->{'text_hash'}->{'destination_efile_name'});
		$self->{'file'} = { 'name' => 'file',
			  'type' => 0,
			  'values' => [ $self->{'in_hash'}->{'file_name'} ] };
		$self->save_directive($self->{'conf'}, $self->{'dest'}, undef, $self->{'file'}, 1);

		# Save owner
		if (!$self->{'in_hash'}->{'file_owner_def'}) {
			defined(getpwnam($self->{'in_hash'}->{'file_owner'})) ||
				$self->error($self->{'text_hash'}->{'destination_eowner'});
			$self->save_directive($self->{'conf'}, $self->{'file'}, "owner",
					$self->{'in_hash'}->{'file_owner'}, 1);
			}
		if (!$self->{'in_hash'}->{'file_group_def'}) {
			defined(getgrnam($self->{'in_hash'}->{'file_group'})) ||
				$self->error($self->{'text_hash'}->{'destination_egroup'});
			$self->save_directive($self->{'conf'}, $self->{'file'}, "group",
					$self->{'in_hash'}->{'file_group'}, 1);
			}

		# Save permissions
		if (!$self->{'in_hash'}->{'file_perm_def'}) {
			$self->{'in_hash'}->{'file_perm'} =~ /^[0-7]+$/ ||
				$self->error($self->{'text_hash'}->{'destination_eperm'});
			$self->save_directive($self->{'conf'}, $self->{'file'}, "perm",
					$self->{'in_hash'}->{'file_perm'}, 1);
			}

		# Save create dirs option
		if ($self->{'in_hash'}->{'file_create_dirs'}) {
			$self->save_directive($self->{'conf'}, $self->{'file'}, "create_dirs",
					$self->{'in_hash'}->{'file_create_dirs'}, 1);
			}
		if (!$self->{'in_hash'}->{'file_dir_perm_def'}) {
			$self->{'in_hash'}->{'file_dir_perm'} =~ /^[0-7]+$/ ||
				$self->error($self->{'text_hash'}->{'destination_edir_perm'});
			$self->save_directive($self->{'conf'}, $self->{'file'}, "dir_perm",
					$self->{'in_hash'}->{'file_dir_perm'}, 1);
			}

		# Save sync options
		if ($self->{'in_hash'}->{'file_fsync'}) {
			$self->save_directive($self->{'conf'}, $self->{'file'}, "fsync",
					$self->{'in_hash'}->{'file_fsync'}, 1);
			}
		if (!$self->{'in_hash'}->{'file_sync_freq_def'}) {
			$self->{'in_hash'}->{'file_sync_freq'} =~ /^\d+$/ ||
				$self->error($self->{'text_hash'}->{'destination_esync_freq'});
			$self->save_directive($self->{'conf'}, $self->{'file'}, "sync_freq",
					$self->{'in_hash'}->{'file_sync_freq'}, 1);
			}
		}

	elsif ($self->{'in_hash'}->{'type'} == 1) {
		# Sending to users
		$self->{'in_hash'}->{'usertty_user_def'} || $self->{'in_hash'}->{'usertty_user'} ||
			$self->error($self->{'text_hash'}->{'destination_euser'});
		$self->{'usertty'} = { 'name' => 'usertty',
			  'type' => 0,
			  'values' => [ $self->{'in_hash'}->{'usertty_user_def'} ? '*' :
					  $self->{'in_hash'}->{'usertty_user'} ] };
		$self->save_directive($self->{'conf'}, $self->{'dest'}, undef, $self->{'usertty'}, 1);
		}

	elsif ($self->{'in_hash'}->{'type'} == 2) {
		# Feeding to a program
		$self->{'in_hash'}->{'program_prog'} =~ /^\S/ ||
			$self->error($self->{'text_hash'}->{'destination_eprog'});
		$self->{'program'} = { 'name' => 'program',
			     'type' => 0,
			     'values' => [ $self->{'in_hash'}->{'program_prog'} ] };
		$self->save_directive($self->{'conf'}, $self->{'dest'}, undef, $self->{'program'}, 1);
		}

	elsif ($self->{'in_hash'}->{'type'} == 3) {
		# Writing to a pipe file
		$self->{'in_hash'}->{'pipe_name'} =~ /^\S/ ||
			$self->error($self->{'text_hash'}->{'destination_epipe'});
		$self->{'pipe'} = { 'name' => 'pipe',
			     'type' => 0,
			     'values' => [ $self->{'in_hash'}->{'pipe_name'} ] };
		$self->save_directive($self->{'conf'}, $self->{'dest'}, undef, $self->{'pipe'}, 1);
		}

	elsif ($self->{'in_hash'}->{'type'} == 4) {
		# Writing to a TCP or UDP socket
		$self->{'net'} = { 'name' => $self->{'in_hash'}->{'net_proto'},
			 'type' => 0,
			 'values' => [ $self->{'in_hash'}->{'net_host'} ] };
		gethostbyname($self->{'in_hash'}->{'net_host'}) ||
			$self->error($self->{'text_hash'}->{'destination_enet_host'});
		$self->save_directive($self->{'conf'}, $self->{'dest'}, undef, $self->{'net'}, 1);

		# Save other network dest options
		if (!$self->{'in_hash'}->{'net_port_def'}) {
			$self->{'in_hash'}->{'net_port'} =~ /^\d+$/ ||
				$self->error($self->{'text_hash'}->{'destination_enet_port'});
			$self->save_directive($self->{'conf'}, $self->{'net'}, "port",
					$self->{'in_hash'}->{'net_port'}, 1);
			}
		if (!$self->{'in_hash'}->{'net_localip_def'}) {
			$self->check_ipaddress($self->{'in_hash'}->{'net_localip'}) ||
				$self->error($self->{'text_hash'}->{'destination_enet_localip'});
			$self->save_directive($self->{'conf'}, $self->{'net'}, "localip",
					$self->{'in_hash'}->{'net_localip'}, 1);
			}
		if (!$self->{'in_hash'}->{'net_localport_def'}) {
			$self->{'in_hash'}->{'net_localport'} =~ /^\d+$/ ||
				$self->error($self->{'text_hash'}->{'destination_enet_localport'});
			$self->save_directive($self->{'conf'}, $self->{'net'}, "localport",
					$self->{'in_hash'}->{'net_localport'}, 1);
			}
		}

	elsif ($self->{'in_hash'}->{'type'} == 6) {
		$self->{'unix'} = { 'name' => $self->{'in_hash'}->{'unix_type'},
			 'type' => 0,
			 'values' => [ $self->{'in_hash'}->{'unix_name'} ] };
		$self->{'in_hash'}->{'unix_name'} || $self->error($self->{'text_hash'}->{'destination_eunix'});
		$self->save_directive($self->{'conf'}, $self->{'dest'}, undef, $self->{'unix'}, 1);
		}

	# Actually update the object
	$self->save_directive($self->{'conf'}, undef, $self->{'old'}, $self->{'dest'}, 0);

        # Update dependent log targets
	if (!$self->{'in_hash'}->{'new'}) {
		  $self->rename_dependencies('destination', $self->{'in_hash'}->{'old'}, $self->{'in_hash'}->{'name'});
	          }
	}

$self->unlock_file($self->{'config_hash'}->{'syslogng_conf'});
$self->webmin_log($self->{'in_hash'}->{'delete'} ? 'delete' : $self->{'in_hash'}->{'new'} ? 'create' : 'modify',
	    'destination', $self->{'in_hash'}->{'old'} || $self->{'in_hash'}->{'name'});
$self->redirect("list_destinations.cgi");

1;
		
}


1;

