#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Create, update or delete a log filter

&ModuleLoader::preload_module('syslog_ng_class', "syslog_ng::SyslogngLib");
$self = new syslog_ng::SyslogngLib:: 'syslog_ng_class',%$self;

$self->ReadParse();
$self->error_setup($self->{'text_hash'}->{'filter_err'});

# Get the old filter
$self->{'conf'} = $self->get_config();
if (!$self->{'in_hash'}->{'new'}) {
	@{$self->{'filters_array'}} = $self->find("filter", $self->{'conf'});
	($self->{'filter'}) = grep { $_->{'value'} eq $self->{'in_hash'}->{'old'} } @{$self->{'filters_array'}};
	$self->{'filter'} || $self->error($self->{'text_hash'}->{'filter_egone'});
	$self->{'old'} = $self->{'filter'};
	}
else {
	$self->{'filter'} = { 'name' => 'filter',
		  'type' => 1,
		  'members' => [ ] };
	}

$self->lock_file($self->{'config_hash'}->{'syslogng_conf'});
if ($self->{'in_hash'}->{'delete'}) {
	# Just delete it!
	$self->check_dependencies('filter', $self->{'in_hash'}->{'old'}) &&
	    $self->error($self->text('fdelete_eused', $self->{'in_hash'}->{'old'}));
	$self->save_directive($self->{'conf'}, undef, $self->{'filter'}, undef, 0);
	}
else {
	# Validate inputs, and update object
	$self->{'in_hash'}->{'name'} =~ /^[a-z0-9_]+$/i || $self->error($self->{'text_hash'}->{'filter_ename'});
	if ($self->{'in_hash'}->{'new'} || $self->{'in_hash'}->{'old'} ne $self->{'in_hash'}->{'name'}) {
		($self->{'clash'}) = grep { $_->{'value'} eq $self->{'in_hash'}->{'name'} } @{$self->{'filters_array'}};
		$self->{'clash'} && $self->error($self->{'text_hash'}->{'filter_eclash'});
		}
	$self->{'filter'}->{'values'} = [ $self->{'in_hash'}->{'name'} ];

	# Clear out current values
	$self->{'filter'}->{'members'} = [ ];

	if ($self->{'in_hash'}->{'mode'} == 0) {
		if ($self->{'in_hash'}->{'priority'}) {
			# Add selected priorities
			@{$self->{'pris_array'}} = split(/\0/, $self->{'in_hash'}->{'pri'});
			@{$self->{'pris_array'}} || $self->error($self->{'text_hash'}->{'filter_epris'});
			@{$self->{'pris_array'}} = map { (",", $_) } @{$self->{'pris_array'}};
			shift(@{$self->{'pris_array'}});	# remove first ,
			push(@{$self->{'filter'}->{'members'}}, "and",
			     { 'name' => 'priority',
			       'type' => 0,
			       'values' => \@{$self->{'pris_array'}} });
			}

		if ($self->{'in_hash'}->{'facility'}) {
			# Add selected facilities
			@{$self->{'facs_array'}} = split(/\0/, $self->{'in_hash'}->{'fac'});
			@{$self->{'facs_array'}} || $self->error($self->{'text_hash'}->{'filter_efacs'});
			@{$self->{'facs_array'}} = map { (",", $_) } @{$self->{'facs_array'}};
			shift(@{$self->{'facs_array'}});	# remove first ,
			push(@{$self->{'filter'}->{'members'}}, "and",
			     { 'name' => 'facility',
			       'type' => 0,
			       'values' => \@{$self->{'facs_array'}} });
			}

		if ($self->{'in_hash'}->{'program'}) {
			$self->{'in_hash'}->{'prog'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'filter_eprog'});
			push(@{$self->{'filter'}->{'members'}}, "and",
			     { 'name' => 'program',
			       'type' => 0,
			       'values' => [ $self->{'in_hash'}->{'prog'} ] });
			}

		if ($self->{'in_hash'}->{'match'}) {
			$self->{'in_hash'}->{'re'} =~ /\S/ || $self->error($self->{'text_hash'}->{'filter_ematch'});
			push(@{$self->{'filter'}->{'members'}}, "and",
			     { 'name' => 'match',
			       'type' => 0,
			       'values' => [ $self->{'in_hash'}->{'re'} ] });
			}

		if ($self->{'in_hash'}->{'host'}) {
			$self->{'in_hash'}->{'hn'} =~ /^\S+$/ || $self->error($self->{'text_hash'}->{'filter_ehost'});
			push(@{$self->{'filter'}->{'members'}}, "and",
			     { 'name' => 'host',
			       'type' => 0,
			       'values' => [ $self->{'in_hash'}->{'hn'} ] });
			}

		if ($self->{'in_hash'}->{'netmask'}) {
		        $self->check_ipaddress($self->{'in_hash'}->{'net'}) ||
			      $self->error($self->{'text_hash'}->{'filter_enet'});
		        $self->check_ipaddress($self->{'in_hash'}->{'mask'}) ||
			      $self->error($self->{'text_hash'}->{'filter_emask'});
			push(@{$self->{'filter'}->{'members'}}, "and",
			     { 'name' => 'netmask',
			       'type' => 0,
			       'values' => [ $self->{'in_hash'}->{'net'}."/".$self->{'in_hash'}->{'mask'} ] });
			}

		if (@{$self->{'filter'}->{'members'}}) {
			# Remove first 'and'
			shift(@{$self->{'filter'}->{'members'}});
			}
		else {
			$self->error($self->{'text_hash'}->{'filter_enone'});
			}
		}

	else {
		# Parse boolean expression (in a temp file), and add to values
		$self->{'temp'} = $self->transname();
		$self->open_tempfile(TEMP, ">$self->{'temp'}", 0, 1);
		$self->print_tempfile(TEMP, "filter xxx {\n");
		$self->print_tempfile(TEMP, $self->{'in_hash'}->{'bool'},"\n");
		$self->print_tempfile(TEMP, "};\n");
		$self->close_tempfile(TEMP);
		eval {
		  $self->{'error_must_die'} = 1;
		  ($self->{'tfilter'}) = $self->read_config_file($self->{'temp'});
		  };
		$@ && $self->error($self->{'text_hash'}->{'filter_ebool'});
		unlink($self->{'temp'});
		$self->{'filter'}->{'members'} = $self->{'tfilter'}->{'members'};
		}
	
	# Actually update the object
	$self->save_directive($self->{'conf'}, undef, $self->{'old'}, $self->{'filter'}, 0);

	# Update dependent log targets
	if (!$self->{'in_hash'}->{'new'}) {
		$self->rename_dependencies('filter', $self->{'in_hash'}->{'old'}, $self->{'in_hash'}->{'name'});
		}
	}

$self->unlock_file($self->{'config_hash'}->{'syslogng_conf'});
$self->webmin_log($self->{'in_hash'}->{'delete'} ? 'delete' : $self->{'in_hash'}->{'new'} ? 'create' : 'modify',
	    'filter', $self->{'in_hash'}->{'old'} || $self->{'in_hash'}->{'name'});
$self->redirect("list_filters.cgi");

1;
		
}


1;

