#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Create, update or delete a log source

&ModuleLoader::preload_module('syslog_ng_class', "syslog_ng::SyslogngLib");
$self = new syslog_ng::SyslogngLib:: 'syslog_ng_class',%$self;

$self->ReadParse();
$self->error_setup($self->{'text_hash'}->{'source_err'});

# Get the old source
$self->{'conf'} = $self->get_config();
if (!$self->{'in_hash'}->{'new'}) {
	@{$self->{'sources_array'}} = $self->find("source", $self->{'conf'});
	($self->{'source'}) = grep { $_->{'value'} eq $self->{'in_hash'}->{'old'} } @{$self->{'sources_array'}};
	$self->{'source'} || $self->error($self->{'text_hash'}->{'source_egone'});
	$self->{'old'} = $self->{'source'};
	}
else {
	$self->{'source'} = { 'name' => 'source',
		  'type' => 1,
		  'members' => [ ] };
	}

$self->lock_file($self->{'config_hash'}->{'syslogng_conf'});
if ($self->{'in_hash'}->{'delete'}) {
	# Just delete it!
	$self->check_dependencies('source', $self->{'in_hash'}->{'old'}) &&
	    $self->error($self->text('sdelete_eused', $self->{'in_hash'}->{'old'}));
	$self->save_directive($self->{'conf'}, undef, $self->{'source'}, undef, 0);
	}
else {
	# Validate inputs, and update object
	$self->{'in_hash'}->{'name'} =~ /^[a-z0-9_]+$/i || $self->error($self->{'text_hash'}->{'source_ename'});
	if ($self->{'in_hash'}->{'new'} || $self->{'in_hash'}->{'old'} ne $self->{'in_hash'}->{'name'}) {
		($self->{'clash'}) = grep { $_->{'value'} eq $self->{'in_hash'}->{'name'} } @{$self->{'sources_array'}};
		$self->{'clash'} && $self->error($self->{'text_hash'}->{'source_eclash'});
		}
	$self->{'source'}->{'values'} = [ $self->{'in_hash'}->{'name'} ];

	# Clear out all existing values
	$self->{'source'}->{'members'} = [ ];

	if ($self->{'in_hash'}->{'internal'}) {
		# Save internal option
		$self->{'internal'} = { 'name' => 'internal',
			      'type' => 0,
			      'values' => [ ] };
		$self->save_directive($self->{'conf'}, $self->{'source'}, undef, $self->{'internal'}, 1);
		}

	foreach $t ("unix-stream", "unix-dgram") {
		# Save Unix socket file option
		next if (!$self->{'in_hash'}->{$t});
		$self->{'in_hash'}->{$t.'_name'} =~ /^\/\S/ ||
			$self->error($self->{'text_hash'}->{'source_eunix_name'});
		$self->{'unix'} = { 'name' => $t,
			  'type' => 0,
			  'values' => [ $self->{'in_hash'}->{$t.'_name'} ] };
		$self->save_directive($self->{'conf'}, $self->{'source'}, undef, $self->{'unix'}, 1);

		# Save owner
		if (!$self->{'in_hash'}->{$t.'_owner_def'}) {
			defined(getpwnam($self->{'in_hash'}->{$t.'_owner'})) ||
				$self->error($self->{'text_hash'}->{'source_eowner'});
			$self->save_directive($self->{'conf'}, $self->{'unix'}, "owner",
					$self->{'in_hash'}->{$t.'_owner'}, 1);
			}
		if (!$self->{'in_hash'}->{$t.'_group_def'}) {
			defined(getgrnam($self->{'in_hash'}->{$t.'_group'})) ||
				$self->error($self->{'text_hash'}->{'source_egroup'});
			$self->save_directive($self->{'conf'}, $self->{'unix'}, "group",
					$self->{'in_hash'}->{$t.'_group'}, 1);
			}

		# Save permissions
		if (!$self->{'in_hash'}->{$t.'_perm_def'}) {
			$self->{'in_hash'}->{$t.'_perm'} =~ /^[0-7]+$/ ||
				$self->error($self->{'text_hash'}->{'source_eperm'});
			$self->save_directive($self->{'conf'}, $self->{'unix'}, "perm",
					$self->{'in_hash'}->{$t.'_perm'}, 1);
			}

		if ($t eq "unix-stream") {
			# Save keep-alive option
			if ($self->{'in_hash'}->{$t.'_keep'}) {
				$self->save_directive($self->{'conf'}, $self->{'unix'}, "keep-alive",
						$self->{'in_hash'}->{$t.'_keep'}, 1);
				}

			# Save max connections option
			if (!$self->{'in_hash'}->{$t.'_max_def'}) {
				$self->{'in_hash'}->{$t.'_max'} =~ /^\d+$/ ||
					$self->error($self->{'text_hash'}->{'source_emax'});
				$self->save_directive($self->{'conf'}, $self->{'unix'}, "max-connections",
						$self->{'in_hash'}->{$t.'_max'}, 1);
				}
			}
		}

	foreach $t ('tcp', 'udp') {
		# Save network socket file option
		next if (!$self->{'in_hash'}->{$t});
		$self->{'net'} = { 'name' => $t,
		         'type' => 0,
			 'values' => [ ] };
		$self->save_directive($self->{'conf'}, $self->{'source'}, undef, $self->{'net'}, 1);

		# Save local IP and port
		if (!$self->{'in_hash'}->{$t.'_ip_def'}) {
			$self->check_ipaddress($self->{'in_hash'}->{$t.'_ip'}) ||
				$self->error($self->{'text_hash'}->{'source_eip'});
			$self->save_directive($self->{'conf'}, $self->{'net'}, "ip",
					$self->{'in_hash'}->{$t.'_ip'}, 1);
			}
		if (!$self->{'in_hash'}->{$t.'_port_def'}) {
			$self->{'in_hash'}->{$t.'_port'} =~ /^\d+$/ ||
				$self->error($self->{'text_hash'}->{'source_eport'});
			$self->save_directive($self->{'conf'}, $self->{'net'}, "port",
					$self->{'in_hash'}->{$t.'_port'}, 1);
			}

		# Save TCP-specific options and max connections
		if ($t eq "tcp") {
			if ($self->{'in_hash'}->{$t.'_keep'}) {
				$self->save_directive($self->{'conf'}, $self->{'net'}, "keep-alive",
						$self->{'in_hash'}->{$t.'_keep'}, 1);
				}
			if ($self->{'in_hash'}->{$t.'_tkeep'}) {
				$self->save_directive($self->{'conf'}, $self->{'net'}, "tcp-keep-alive",
						$self->{'in_hash'}->{$t.'_tkeep'}, 1);
				}
			}
		if (!$self->{'in_hash'}->{$t.'_max_def'}) {
			$self->{'in_hash'}->{$t.'_max'} =~ /^\d+$/ ||
				$self->error($self->{'text_hash'}->{'source_emax'});
			$self->save_directive($self->{'conf'}, $self->{'net'}, "max-connections",
					$self->{'in_hash'}->{$t.'_max'}, 1);
			}
		}

	# Save kernel file option
	if ($self->{'in_hash'}->{'file'}) {
		$self->{'in_hash'}->{'file_name'} =~ /^\/\S/ ||
			$self->error($self->{'text_hash'}->{'source_efile_name'});
		$self->{'file'} = { 'name' => 'file',
		          'type' => 0,
			  'values' => [ $self->{'in_hash'}->{'file_name'} ] };
		$self->save_directive($self->{'conf'}, $self->{'source'}, undef, $self->{'file'}, 1);

		# Save log prefix
		if (!$self->{'in_hash'}->{'file_prefix_def'}) {
			$self->{'in_hash'}->{'file_prefix'} =~ /\S/ ||
			    $self->error($self->{'text_hash'}->{'source_eprefix'});
			$self->save_directive($self->{'conf'}, $self->{'file'}, "log_prefix",
					$self->{'in_hash'}->{'file_prefix'}, 1);
			}
		}

	# Save named pipe option
	if ($self->{'in_hash'}->{'pipe'}) {
		$self->{'in_hash'}->{'pipe_name'} =~ /^\/\S/ ||
			$self->error($self->{'text_hash'}->{'source_epipe_name'});
		$self->{'pipe'} = { 'name' => 'pipe',
		          'type' => 0,
			  'values' => [ $self->{'in_hash'}->{'pipe_name'} ] };
		$self->save_directive($self->{'conf'}, $self->{'source'}, undef, $self->{'pipe'}, 1);

		# Save log prefix and pad size
		if (!$self->{'in_hash'}->{'pipe_prefix_def'}) {
			$self->{'in_hash'}->{'pipe_prefix'} =~ /\S/ ||
			    $self->error($self->{'text_hash'}->{'source_eprefix'});
			$self->save_directive($self->{'conf'}, $self->{'pipe'}, "log_prefix",
					$self->{'in_hash'}->{'pipe_prefix'}, 1);
			}
		if (!$self->{'in_hash'}->{'pipe_pad_def'}) {
			$self->{'in_hash'}->{'pipe_pad'} =~ /^\d+$/ ||
			    $self->error($self->{'text_hash'}->{'source_epad'});
			$self->save_directive($self->{'conf'}, $self->{'pipe'}, "pad_size",
					$self->{'in_hash'}->{'pipe_pad'}, 1);
			}
		}
		
	# Save Solaris streams option
	if ($self->{'in_hash'}->{'sun-streams'}) {
		$self->{'in_hash'}->{'sun_streams_name'} =~ /^\/\S/ ||
			$self->error($self->{'text_hash'}->{'source_esun_streams_name'});
		$self->{'sun_streams'} = { 'name' => 'sun-streams',
				 'type' => 0,
				 'values' => [ $self->{'in_hash'}->{'sun_streams_name'} ] };
		$self->save_directive($self->{'conf'}, $self->{'source'}, undef, $self->{'sun_streams'}, 1);

		# Save door file
		$self->{'in_hash'}->{'sun_streams_door'} =~ /\S/ ||
		    $self->error($self->{'text_hash'}->{'source_edoor'});
		$self->save_directive($self->{'conf'}, $self->{'sun_streams'}, "door",
				$self->{'in_hash'}->{'sun_streams_door'}, 1);
		}

	# Actually update the object
	$self->save_directive($self->{'conf'}, undef, $self->{'old'}, $self->{'source'}, 0);

	# Update dependent log targets
	if (!$self->{'in_hash'}->{'new'}) {
		$self->rename_dependencies('source', $self->{'in_hash'}->{'old'}, $self->{'in_hash'}->{'name'});
		}
	}

$self->unlock_file($self->{'config_hash'}->{'syslogng_conf'});
$self->webmin_log($self->{'in_hash'}->{'delete'} ? 'delete' : $self->{'in_hash'}->{'new'} ? 'create' : 'modify',
	    'source', $self->{'in_hash'}->{'old'} || $self->{'in_hash'}->{'name'});
$self->redirect("list_sources.cgi");

1;
		
}


1;

