#!/usr/bin/perl
package time::LinuxLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'time::LinuxLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for getting and setting the timezone on Linux

$self->{'timezones_file'} = "/usr/share/zoneinfo/zone.tab";
$self->{'currentzone_link'} = "/etc/localtime";
$self->{'currentzone_file'} = "/etc/timezone";
$self->{'timezones_dir'} = "/usr/share/zoneinfo";
$self->{'sysclock_file'} = "/etc/sysconfig/clock";

1;

1;
		return $self;
}

# list_timezones()
sub list_timezones
{
	my $self = shift;

local @rv;
local %done;
$self->open_readfile(*ZONE, $self->{'timezones_file'}) || return ( );
while(<ZONE>) {
	s/\r|\n//g;
	s/^\s*#.*$//;
	if (/^(\S+)\s+(\S+)\s+(\S+)\s+(\S.*)/) {
		push(@rv, [ $3, $4 ]);
		$done{$3}++;
		}
	elsif (/^(\S+)\s+(\S+)\s+(\S+)/) {
		push(@rv, [ $3, undef ]);
		$done{$3}++;
		}
	}
close(ZONE);
push(@rv, [ "GMT", "GMT" ]) if (!$done{'GMT'});
return sort { $a->[0] cmp $b->[0] } @rv;
}

# get_current_timezone()
sub get_current_timezone
{
	my $self = shift;

local $lnk = readlink($self->translate_filename($self->{'currentzone_link'}));
if ($lnk) {
	# Easy - it a link
	$lnk =~ s/$self->{'timezones_dir'}\///;
	return $lnk;
	}
else {
	# Need to compare with all timezone files!
	return $self->find_same_zone($self->{'currentzone_link'});
	}
}

# set_current_timezone(zone)
sub set_current_timezone
{
	my $self = shift;

$self->lock_file($self->{'currentzone_link'});
unlink($self->translate_filename($self->{'currentzone_link'}));
symlink($self->translate_filename("$self->{'timezones_dir'}/$_[0]"),
	$self->translate_filename($self->{'currentzone_link'}));
$self->unlock_file($self->{'currentzone_link'});

if (-r $self->{'currentzone_file'}) {
	# This file is used on Debian systems
	$self->open_lock_tempfile(FILE, ">$self->{'currentzone_file'}");
	$self->print_tempfile(FILE, $_[0],"\n");
	$self->close_tempfile(FILE);
	}

local %clock;
if ($self->read_env_file($self->{'sysclock_file'}, \%clock)) {
	$clock{'ZONE'} = $_[0];
	$self->lock_file($self->{'sysclock_file'});
	$self->write_env_file($self->{'sysclock_file'}, \%clock);
	$self->unlock_file($self->{'sysclock_file'});
	}
}

sub os_has_timezones
{
	my $self = shift;

return -r $self->{'timezones_file'};
}

sub timezone_files
{
	my $self = shift;

local @rv = ( $self->{'currentzone_link'} );
push(@rv, $self->{'currentzone_file'}) if (-r $self->{'currentzone_file'});
push(@rv, $self->{'sysclock_file'}) if (-r $self->{'sysclock_file'});
return @rv;
}


1;

