#!/usr/bin/perl
package time::SolarisLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'time::SolarisLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for getting and setting the timezone on Linux

$self->{'timezone_file'} = "/etc/TIMEZONE";
$self->{'timezones_dir'} = "/usr/share/lib/zoneinfo";
$self->{'rtc_config'} = "/etc/rtc_config";

1;

1;
		return $self;
}

# list_timezones()
sub list_timezones
{
	my $self = shift;

local @rv;
local $file;
$self->open_execute_command(*FIND, "find $self->{'timezones_dir'} -type f", 1);
while($file = <FIND>) {
	chop($file);
	local $buf;
	$self->open_readfile(*INFO, $file);
	read(INFO, $buf, 2);
	close(INFO);
	if ($buf eq "TZ") {
		# A timezone file we can use!
		$file =~ s/^$self->{'timezones_dir'}\///;
		push(@rv, [ $file, undef ]);
		}
	}
close(FIND);
return sort { $a->[0] cmp $b->[0] } @rv;
}

# get_current_timezone()
sub get_current_timezone
{
	my $self = shift;

local %tz;
$self->read_env_file($self->{'timezone_file'}, \%tz);
$tz{'TZ'} =~ s/^://;
return $tz{'TZ'};
}

# set_current_timezone(zone)
sub set_current_timezone
{
	my $self = shift;

local %tz;
$self->lock_file($self->{'timezone_file'});
$self->read_env_file($self->{'timezone_file'}, \%tz);
$tz{'TZ'} = $_[0];
$self->write_env_file($self->{'timezone_file'}, \%tz);
$self->unlock_file($self->{'timezone_file'});

if (-r $self->{'rtc_config'}) {
	# Update x86 RTC timezone too
	$self->lock_file($self->{'rtc_config'});
	local %rtc;
	$self->read_env_file($self->{'rtc_config'}, \%rtc);
	$rtc{'zone_info'} = $_[0];
	$self->write_env_file($self->{'rtc_config'}, \%rtc);
	$self->unlock_file($self->{'rtc_config'});
	}
}

sub os_has_timezones
{
	my $self = shift;

return -r $self->{'timezone_file'} && -d $self->{'timezones_dir'};
}

sub timezone_files
{
	my $self = shift;

return ( $self->{'timezone_file'} );
}


1;

