#!/usr/bin/perl
package time::TimeLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'time::TimeLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
%{$self->{'access_hash'}} = $self->get_module_acl();
$self->{'cron_cmd'} = "$self->{'module_config_directory'}/sync.pl";
if ($self->{'config_hash'}->{'zone_style'}) {
	{
    my $pkg = $self->lib_to_pkg("$self->{config_hash}->{zone_style}-lib.pl", "time");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
	}

@{$self->{'weekday_names_array'}} = ( "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" );

1;

1;
		return $self;
}

sub p_link
{
	my $self = shift;

    my ( $dest, $text ) = @_;
    return "<a href=\"". $dest. "\">". $text. "</a>";
}

sub p_radio
{
	my $self = shift;

    my ( $name, $checked, @list ) = @_;
    local ($out, $size, $i);
    $size = @list; $i = 0;

    do
    {
	$out .= " <input type=radio name=".$name." value=".$list[$i];
	$out .= " checked" if( $checked eq $list[$i++] );
	$out .="> ".$list[$i++];
    } while( $i < $size );

    return $out;
}

sub p_entry
{
	my $self = shift;

    my ( $name, $value, $size ) = @_;

    $size ? return "<input name=". $name. " size=". $size." value=\"". $value."\">" : return "<input name=". $name. " value=\"". $value."\">";
}

sub p_select_wdl
{
	my $self = shift;

  my ( $name, $selected, @list ) = @_;
  local $out = "<select name=$name>";
  local $i = 0;
  local $size = @list;

  do
  {
	$out .= "<option value=$list[$i++]";
	$out .= " selected" if( $selected eq $list[$i] );
	$out .= ">$list[$i++]";
  } while( $i < $size );
  $out .= "</select>";

}

sub p_select
{
	my $self = shift;

  my ( $name, $selected, @list ) = @_;
  local (@newlist, $item);

  foreach $item ( @list )
  {
	push( @newlist, $item, $item );
  }

  $self->p_select_wdl( $name, $selected, @newlist );
}

sub find_cron_job
{
	my $self = shift;

$self->foreign_require("cron", "cron-lib.pl");
local @jobs = $main::cron_object->list_cron_jobs();
local ($job) = grep { $_->{'command'} eq $self->{'cron_cmd'} &&
		      $_->{'user'} eq 'root' } @jobs;
return $job;
}

# sync_time(server, hardware-too)
# Syncs the system and maybe hardware time with some server. Returns undef
# on success, or an error message on failure.
sub sync_time
{
	my $self = shift;

local @servs = split(/\s+/, $_[0]);
local $servs = join(" ", map { quotemeta($_) } @servs);
local $out = $self->backquote_logged("ntpdate -u $servs 2>&1");
if ($? && $self->{'config_hash'}->{'ntp_only'}) {
	# error using ntp, but nothing else is allowed
	return $self->text('error_entp', "<tt>$out</tt>");
	}
elsif ($?) {
	# error using ntp. use timeservice
	local ($err, $serv);
	foreach $serv (@servs) {
		$err = undef;
		$self->open_socket($serv, 37, SOCK, \$err);
		read(SOCK, $self->{'rawtime'}, 4);
		close(SOCK);
		last if (!$err && $self->{'rawtime'});
		}
	return $err if ($err);

	# Got a time .. set it
  	$self->{'rawtime'} = unpack("N", $self->{'rawtime'});
  	$self->{'rawtime'} -= (17 * 366 + 53 * 365) * 24 * 60 * 60;
	local $diff = abs(time() - $self->{'rawtime'});
	if ($diff > 365*24*60*60) {
		# Too big!
		return $self->text('error_ediff', int($diff/(24*60*60)));
		} 
  	@{$self->{'tm_array'}} = localtime($self->{'rawtime'});
	$self->set_system_time(@{$self->{'tm_array'}});
	}
else {
	$self->{'rawtime'} = time();
	}

if ($_[1]) {
	# Set hardware clock time to match system time (which is now correct)
	local $out = $self->backquote_logged("hwclock --systohc");
	return $? ? $out : undef;
	}

return undef;
}

sub has_timezone
{
	my $self = shift;

return 0 if (!$self->UNIVERSAL::can('list_timezones'));
if ($self->UNIVERSAL::can('os_has_timezones')) {
	return $self->os_has_timezones();
	}
else {
	local @zones = $self->list_timezones();
	return @zones ? 1 : 0;
	}
}

# find_same_zone(file)
# Finds an identical timezone file to the one specified
sub find_same_zone
{
	my $self = shift;

local @st = stat($self->translate_filename($_[0]));
local $z;
foreach $z ($self->list_timezones()) {
	local $zf = $self->translate_filename("$self->{'timezones_dir'}/$z->[0]");
	local @zst = stat($zf);
	if ($zst[7] == $st[7]) {
		local $ex = system("diff ".$self->translate_filename($self->{'currentzone_link'})." $zf >/dev/null 2>&1");
		if (!$ex) {
			return $z->[0];
			}
		}
	}
return undef;
}

# get_hwclock_flags()
# Returns command-line flags for hwclock
sub get_hwclock_flags
{
	my $self = shift;

if ($self->{'config_hash'}->{'hwclock_flags'} eq "sysconfig") {
	local %clock;
	$self->read_env_file("/etc/sysconfig/clock", \%clock);
	return $clock{'CLOCKFLAGS'};
	}
else {
	return $self->{'config_hash'}->{'hwclock_flags'};
	}
}

# get_hardware_time()
# Returns the current hardware time, in localtime format
sub get_hardware_time
{
	my $self = shift;

local $flags = $self->get_hwclock_flags();
local $out = `hwclock $flags`;
if ($out =~ /^(\S+)\s+(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(\d+)\s+/) {
	return ($6, $5, $4, $3, $self->month_to_number($2), $7-1900, $self->weekday_to_number($1));
	}
elsif ($out =~ /^(\S+)\s+(\d+)\s+(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(am|pm)\s+/i) {
	return ($7, $6, $5+($8 eq 'pm' ? 12 : 0), $2, $self->month_to_number($3), $4-1900, $self->weekday_to_number($1));
	}
else {
	return ( );
	}
}

# get_system_time()
# Returns the current time, in localtime format
sub get_system_time
{
	my $self = shift;

return localtime(time());
}

# set_hardware_time(secs, mins, hours, day, month, year)
sub set_hardware_time
{
	my $self = shift;

local ($second, $minute, $hour, $date, $month, $year) = @_;
$month++;
$year += 1900;
local $format = "--set --date=\"".quotemeta("$month/$date/$year $hour:$minute:$second")."\"";
local $flags = $self->get_hwclock_flags();
local $out = $self->backquote_logged("hwclock $flags $format");
return $? ? $out : undef;
}

# set_system_time(secs, mins, hours, day, month, year)
sub set_system_time
{
	my $self = shift;

local ($second, $minute, $hour, $date, $month, $year) = @_;
$second = $self->zeropad($second, 2);
$minute = $self->zeropad($minute, 2);
$hour = $self->zeropad($hour, 2);
$date = $self->zeropad($date, 2);
$month = $self->zeropad($month+1, 2);
$year = $self->zeropad($year+1900, 4);
local $format;
if ($self->{'config_hash'}->{'seconds'} == 2) {
	$format = $year.$month.$date.$hour.$minute.".".$second;
	}
elsif ($self->{'config_hash'}->{'seconds'} == 1) {
	$format = $month.$date.$hour.$minute.$year.".".$second;
	}
else {
	$format = $month.$date.$hour.$minute.substr($year, -2);
	}
local $out = $self->backquote_logged("echo yes | date ".quotemeta($format));
if ($self->{'gconfig_hash'}->{'os_type'} eq 'freebsd' || $self->{'gconfig_hash'}->{'os_type'} eq 'netbsd') {
	return int($?/256) == 1 ? $out : undef;
	}
else {
	return $? ? $out : undef;
	}
}

sub zeropad
{
	my $self = shift;

local ($str, $len) = @_;
while(length($str) < $len) {
	$str = "0".$str;
	}
return $str;
}

# weekday_to_number(day)
# Converts a day like Mon to a number like 1
sub weekday_to_number
{
	my $self = shift;

for($i=0; $i<@{$self->{'weekday_names_array'}}; $i++) {
	return $i if (lc(substr($self->{'weekday_names_array'}->[$i], 0, 3)) eq lc($_[0]));
	}
return undef;
}

sub number_to_weekday
{
	my $self = shift;

return ucfirst($self->{'weekday_names_array'}->[$_[0]]);
}


1;

