#!/usr/bin/perl

&MAIN();

 sub MAIN {

local $format;
local $out;

&ModuleLoader::preload_module('time_class', "time::TimeLib");
$self = new time::TimeLib:: 'time_class',%$self;

require 'timelocal.pl';

$self->ReadParse();

if (!$self->{'in_hash'}->{'action'}) {
	# user probably hit return in the time server field
	$self->{'in_hash'}->{'action'} = $self->{'text_hash'}->{ 'action_timeserver_sys' };
	}

if( $self->{'in_hash'}->{ 'action' } eq $self->{'text_hash'}->{ 'action_apply' } )
{
  # Setting the system time
  $self->error( $self->{'text_hash'}->{ 'acl_nosys' } ) if( $self->{'access_hash'}->{ 'sysdate' } );
  $self->{'err'} = $self->set_system_time($self->{'in_hash'}->{ 'second' }, $self->{'in_hash'}->{'minute'}, $self->{'in_hash'}->{'hour'},
		   $self->{'in_hash'}->{'date'}, $self->{'in_hash'}->{'month'}-1, $self->{'in_hash'}->{'year'}-1900);
  $self->error($self->{'err'}) if ($self->{'err'});
  $self->webmin_log("set", "date", time(), \%{$self->{'in_hash'}});

} elsif ( $self->{'in_hash'}->{ 'action' } eq $self->{'text_hash'}->{ 'action_save' } ) {
  # Setting the hardware time
  $self->error( $self->{'text_hash'}->{ 'acl_nohw' } ) if( $self->{'access_hash'}->{ 'hwdate' } );
  $self->{'err'} = $self->set_hardware_time($self->{'in_hash'}->{ 'second' }, $self->{'in_hash'}->{'minute'}, $self->{'in_hash'}->{'hour'},
		   $self->{'in_hash'}->{'date'}, $self->{'in_hash'}->{'month'}-1, $self->{'in_hash'}->{'year'}-1900);
  $self->error( $self->text( 'error_hw', $self->{'err'} ) ) if ($self->{'err'});
  local $hwtime = timelocal($self->{'in_hash'}->{'second'}, $self->{'in_hash'}->{'minute'}, $self->{'in_hash'}->{'hour'},
			    $self->{'in_hash'}->{'date'}, $self->{'in_hash'}->{'month'}-1, $self->{'in_hash'}->{'year'} < 200 ?
			    $self->{'in_hash'}->{'year'} : $self->{'in_hash'}->{'year'}-1900);
  $self->webmin_log("set", "hwclock", $hwtime, \%{$self->{'in_hash'}});

} elsif( $self->{'in_hash'}->{ 'action' } eq $self->{'text_hash'}->{ 'action_sync' } ) {
  # Set system time to hardware time
  $self->error( $self->{'text_hash'}->{ 'acl_nosys' } ) if( $self->{'access_hash'}->{ 'sysdate' } );
  $out = $self->backquote_logged("hwclock --hctosys");
  $self->error( $self->text( 'error_sync', $out ) ) if( $out ne "" );
  $self->webmin_log("sync");

} elsif( $self->{'in_hash'}->{ 'action' } eq $self->{'text_hash'}->{ 'action_sync_s' } ) {
  # Set hardware time to system time
  $self->error( $self->{'text_hash'}->{ 'acl_nohw' } ) if( $self->{'access_hash'}->{ 'hwdate' } && $self->{'access_hash'}->{'sysdate'} );
  $out = $self->backquote_logged("hwclock --systohc");
  $self->error( $self->text( 'error_sync', $out ) ) if( $out ne "" );
  $self->webmin_log("sync_s");

} elsif( $self->{'in_hash'}->{ 'action' } eq $self->{'text_hash'}->{ 'index_sync' }) {
  # Sync with a time server
  $self->{'access_hash'}->{'ntp'} || $self->error($self->{'text_hash'}->{'acl_nontp'});
  $self->{'err'} = $self->sync_time($self->{'in_hash'}->{'timeserver'}, $self->{'in_hash'}->{'hardware'});
  $self->error($self->{'err'}) if ($self->{'err'});

  # Save settings in module config
  $self->lock_file($self->{'module_config_file'});
  $self->{'config_hash'}->{'timeserver'} = $self->{'in_hash'}->{'timeserver'};
  $self->{'config_hash'}->{'timeserver_hardware'} = $self->{'in_hash'}->{'hardware'};
  $self->save_module_config();
  $self->unlock_file($self->{'module_config_file'});

  # Create, update or delete the syncing cron job
  $self->foreign_require("cron", "cron-lib.pl");
  $self->{'oldjob'} = $self->{'job'} = $self->find_cron_job();
  $self->{'job'} ||= { 'command' => $self->{'cron_cmd'},
	     'active' => 1,
	     'user' => 'root' };
  $main::cron_object->parse_times_input($self->{'job'}, \%{$self->{'in_hash'}}) if ($self->{'in_hash'}->{'sched'});
  $self->lock_file($main::cron_object->cron_file($self->{'job'}));
  $main::cron_object->create_wrapper($self->{'cron_cmd'}, $self->{'module_name'}, "sync.pl");
  if ($self->{'in_hash'}->{'sched'} && $self->{'oldjob'}) {
	# Update job
	$main::cron_object->change_cron_job($self->{'job'});
	}
  elsif ($self->{'in_hash'}->{'sched'} && !$self->{'oldjob'}) {
	# Create wrapper script and job
	$main::cron_object->create_cron_job($self->{'job'});
	}
  elsif (!$self->{'in_hash'}->{'sched'} && $self->{'oldjob'}) {
	# Delete job
	$main::cron_object->delete_cron_job($self->{'job'});
	}
  $self->unlock_file($main::cron_object->cron_file($self->{'job'}));

  $self->webmin_log("remote", $self->{'in_hash'}->{'action'} eq $self->{'text_hash'}->{'action_timeserver_sys'} ?  "date" : "hwclock", $self->{'rawtime'}, \%{$self->{'in_hash'}});
}

$self->redirect("");

1;
		
}


1;

