#!/usr/bin/perl
package useradmin::UserLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'useradmin::UserLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# user-lib.pl
# Common functions for Unix user management

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
if ($self->{'gconfig_hash'}->{'os_type'} =~ /-linux$/) {
	&ModuleLoader::preload_module(ref($self), "useradmin::LinuxLib");
$self = new useradmin::LinuxLib:: ref($self), %$self; 
	}
else {
	{
    my $pkg = $self->lib_to_pkg("$self->{gconfig_hash}->{os_type}-lib.pl", "useradmin");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
	}
&ModuleLoader::preload_module(ref($self), "useradmin::Md5Lib");
$self = new useradmin::Md5Lib:: ref($self), %$self; 

@{$self->{'random_password_chars_array'}} = ( 'a' .. 'z', 'A' .. 'Z', '0' .. '9' );
$self->{'disable_string'} = $self->{'config_hash'}->{'lock_prepend'} eq "" ? "!" : $self->{'config_hash'}->{'lock_prepend'};

# Functions similar to the standard password file ones, but which may
# use webmin's reading of the user/group files instead.

#################################################################

1;
1;
		return $self;
}

# password_file(file)
# Returns true if some file looks like a valid Unix password file
sub password_file
{
	my $self = shift;

if (!$_[0]) { return 0; }
elsif ($self->open_readfile(*SHTEST, $_[0])) {
	local($line);
	$line = <SHTEST>;
	close(SHTEST);
	return $line =~ /^\S+:\S*:/;
	}
else { return 0; }
}

# list_users()
# Returns an array of hashtable, each containing info about one user. Each hash
# will always contain the keys
#  user, pass, uid, gid, real, home, shell
# In addition, if the system supports shadow passwords it may also have:
#  change, min, max, warn, inactive, expire
# Or if it supports FreeBSD master.passwd info, it will also have
#  class, change, expire
sub list_users
{
	my $self = shift;

return @{$self->{'list_users_cache_array'}} if (defined(@{$self->{'list_users_cache_array'}}));

# read the password file
local (@rv, $_, %idx, $lnum, @pw, $p, $i, $j);
local $pft = $self->passfiles_type();
if ($pft == 1) {
	# read the master.passwd file only
	$lnum = 0;
	$self->open_readfile(*PASSWD, $self->{'config_hash'}->{'master_file'});
	while(<PASSWD>) {
		s/\r|\n//g;
		if (/\S/ && !/^[#\+\-]/) {
			@pw = split(/:/, $_, -1);
			push(@rv, { 'user' => $pw[0],	'pass' => $pw[1],
				    'uid' => $pw[2],	'gid' => $pw[3],
				    'class' => $pw[4],	'change' => $pw[5],
				    'expire' => $pw[6],	'real' => $pw[7],
				    'home' => $pw[8],	'shell' => $pw[9],
				    'line' => $lnum,	'num' => scalar(@rv) });
			}
		$lnum++;
		}
	close(PASSWD);
	}
elsif ($pft == 6) {
	# Read netinfo dump
	$self->open_execute_command(*PASSWD, "nidump passwd '$self->{'netinfo_domain'}'", 1);
	while(<PASSWD>) {
		s/\r|\n//g;
		if (/\S/ && !/^[#\+\-]/) {
			@pw = split(/:/, $_, -1);
			push(@rv, { 'user' => $pw[0],	'pass' => $pw[1],
				    'uid' => $pw[2],	'gid' => $pw[3],
				    'class' => $pw[4],	'change' => $pw[5],
				    'expire' => $pw[6],	'real' => $pw[7],
				    'home' => $pw[8],	'shell' => $pw[9],
				    'num' => scalar(@rv) });
			}
		}
	close(PASSWD);
	}
else {
	# start by reading /etc/passwd
	$lnum = 0;
	$self->open_readfile(*PASSWD, $self->{'config_hash'}->{'passwd_file'});
	while(<PASSWD>) {
		s/\r|\n//g;
		if (/\S/ && !/^[#\+\-]/) {
			@pw = split(/:/, $_, -1);
			push(@rv, { 'user' => $pw[0],	'pass' => $pw[1],
				    'uid' => $pw[2],	'gid' => $pw[3],
				    'real' => $pw[4],	'home' => $pw[5],
				    'shell' => $pw[6],	'line' => $lnum,
				    'num' => scalar(@rv) });
			$idx{$pw[0]} = $rv[$#rv];
			}
		$lnum++;
		}
	close(PASSWD);
	if ($pft == 2 || $pft == 5) {
		# read the shadow file data
		$lnum = 0;
		$self->open_readfile(*SHADOW, $self->{'config_hash'}->{'shadow_file'});
		while(<SHADOW>) {
			s/\r|\n//g;
			if (/\S/ && !/^[#\+\-]/) {
				@pw = split(/:/, $_, -1);
				$p = $idx{$pw[0]};
				$p->{'pass'} = $pw[1];
				$p->{'change'} = $pw[2] < 0 ? "" : $pw[2];
				$p->{'min'} = $pw[3] < 0 ? "" : $pw[3];
				$p->{'max'} = $pw[4] < 0 ? "" : $pw[4];
				$p->{'warn'} = $pw[5] < 0 ? "" : $pw[5];
				$p->{'inactive'} = $pw[6] < 0 ? "" : $pw[6];
				$p->{'expire'} = $pw[7] < 0 ? "" : $pw[7];
				$p->{'sline'} = $lnum;
				}
			$lnum++;
			}
		close(SHADOW);
		for($i=0; $i<@rv; $i++) {
			if (!defined($rv[$i]->{'sline'})) {
				# not in shadow!
				for($j=$i; $j<@rv; $j++) { $rv[$j]->{'num'}--; }
				splice(@rv, $i--, 1);
				}
			}
		}
	elsif ($pft == 4) {
		# read the AIX security passwd file
		local $lastuser;
		local $lnum = 0;
		$self->open_readfile(*SECURITY, $self->{'config_hash'}->{'shadow_file'});
		while(<SECURITY>) {
			s/\s*$//;
			if (/^\s*(\S+):/) {
				$lastuser = $idx{$1};
				$lastuser->{'sline'} = $lnum;
				}
			elsif (/^\s*([^=\s]+)\s*=\s*(.*)/) {
				if ($1 eq 'password') {
					$lastuser->{'pass'} = $2;
					}
				elsif ($1 eq 'lastupdate') {
					$lastuser->{'change'} = $2;
					}
				elsif ($1 eq 'flags') {
					map { $lastuser->{lc($_)}++ }
					    split(/[,\s]+/, $2);
					}
				$lastuser->{'seline'} = $lnum;
				}
			$lnum++;
			}
		close(SECURITY);

		# read the AIX security user file
		$self->open_readfile(*USER, $self->{'config_hash'}->{'aix_user_file'});
		while(<USER>) {
			s/\s*$//;
			if (/^\s*(\S+):/) {
				$lastuser = $idx{$1};
				}
			elsif (/^\s*([^=\s]+)\s*=\s*(.*)/) {
				if ($1 eq 'expires') {
					$lastuser->{'expire'} = $2;
					$lastuser->{'expire'} =~ s/^0$//;
					}
				elsif ($1 eq 'minage') {
					$lastuser->{'min'} = $2;
					$lastuser->{'min'} =~ s/^0$//;
					}
				elsif ($1 eq 'maxage') {
					$lastuser->{'max'} = $2;
					$lastuser->{'max'} =~ s/^0$//;
					}
				elsif ($1 eq 'pwdwarntime') {
					$lastuser->{'warn'} = $2;
					$lastuser->{'warn'} =~ s/^0$//; 
					}
				}
			}
		close(USER);
		}
	}
@{$self->{'list_users_cache_array'}} = @rv;
return @rv;
}

# create_user(&details)
# Creates a new user with the given details
sub create_user
{
	my $self = shift;

local $lref;
local $pft = $self->passfiles_type();
if ($pft == 1) {
	# just need to add to master.passwd
	$lref = $self->read_file_lines($self->{'config_hash'}->{'master_file'});
	$_[0]->{'line'} = $self->nis_index($lref);
	splice(@$lref, $_[0]->{'line'}, 0,
	       "$_[0]->{'user'}:$_[0]->{'pass'}:$_[0]->{'uid'}:".
	       "$_[0]->{'gid'}:$_[0]->{'class'}:$_[0]->{'change'}:".
	       "$_[0]->{'expire'}:$_[0]->{'real'}:$_[0]->{'home'}:".
	       "$_[0]->{'shell'}");
	if (defined(@{$self->{'list_users_cache_array'}})) {
		map { $_->{'line'}++ if ($_->{'line'} >= $_[0]->{'line'}) }
		    @{$self->{'list_users_cache_array'}};
		}
	}
elsif ($pft == 3) {
	# Just invoke the useradd command
	$self->system_logged("useradd -u $_[0]->{'uid'} -g $_[0]->{'gid'} -c \"$_[0]->{'real'}\" -d $_[0]->{'home'} -s $_[0]->{'shell'} $_[0]->{'user'}");
	# And set the password
	$self->system_logged("echo $_[0]->{'pass'} | /usr/lib/scoadmin/account/password.tcl $_[0]->{'user'} >/dev/null 2>&1");
	}
elsif ($pft == 6) {
	# Use the niutil command
	$self->system_logged("niutil -create '$self->{'netinfo_domain'}' '/users/$_[0]->{'user'}'");
	$self->set_netinfo($_[0]);
	}
else {
	# add to /etc/passwd
	$lref = $self->read_file_lines($self->{'config_hash'}->{'passwd_file'});
	$_[0]->{'line'} = $self->nis_index($lref);
	if (defined(@{$self->{'list_users_cache_array'}})) {
		map { $_->{'line'}++ if ($_->{'line'} >= $_[0]->{'line'}) }
		    @{$self->{'list_users_cache_array'}};
		}
	splice(@$lref, $_[0]->{'line'}, 0,
	       "$_[0]->{'user'}:".
	       ($pft == 2 || $pft == 5 ? "x" : $pft == 4 ? "!" :
			$_[0]->{'pass'}).
	       ":$_[0]->{'uid'}:$_[0]->{'gid'}:$_[0]->{'real'}:".
	       "$_[0]->{'home'}:$_[0]->{'shell'}");
	if ($pft == 2 || $pft == 5) {
		# Find correct place to insert in shadow file
		$lref = $self->read_file_lines($self->{'config_hash'}->{'shadow_file'});
		$_[0]->{'sline'} = $self->nis_index($lref);
		if (defined(@{$self->{'list_users_cache_array'}})) {
			map { $_->{'sline'}++
			      if ($_->{'sline'} >= $_[0]->{'sline'}) }
			    @{$self->{'list_users_cache_array'}};
			}
		}
	if ($pft == 2) {
		# add to shadow as well..
		splice(@$lref, $_[0]->{'sline'}, 0,
		       "$_[0]->{'user'}:$_[0]->{'pass'}:$_[0]->{'change'}:".
		       "$_[0]->{'min'}:$_[0]->{'max'}:$_[0]->{'warn'}:".
		       "$_[0]->{'inactive'}:$_[0]->{'expire'}:");
		}
	elsif ($pft == 5) {
		# add to SCO shadow file
		splice(@$lref, $_[0]->{'sline'}, 0,
	       	    "$_[0]->{'user'}:$_[0]->{'pass'}:$_[0]->{'change'}:".
		    "$_[0]->{'min'}:$_[0]->{'max'}");
		}
	elsif ($pft == 4) {
		# add to AIX security passwd file as well..
		local @flags;
		push(@flags, 'ADMIN') if ($_[0]->{'admin'});
		push(@flags, 'ADMCHG') if ($_[0]->{'admchg'});
		push(@flags, 'NOCHECK') if ($_[0]->{'nocheck'});
		$lref = $self->read_file_lines($self->{'config_hash'}->{'shadow_file'});
		push(@$lref, "", "$_[0]->{'user'}:",
			     "\tpassword = $_[0]->{'pass'}",
			     "\tlastupdate = $_[0]->{'change'}",
			     "\tflags = ".join(",", @flags));
		
		# add to AIX security user file as well..
		$lref = $self->read_file_lines($self->{'config_hash'}->{'aix_user_file'});
		if ($_[0]->{'expire'} || $_[0]->{'min'} ||
		    $_[0]->{'max'} || $_[0]->{'warn'} ) {
			push(@$lref, "$_[0]->{'user'}:");
			push(@$lref, "\texpires = $_[0]->{'expire'}")
				if ($_[0]->{'expire'});
			push(@$lref, "\tminage = $_[0]->{'min'}")
				if ($_[0]->{'min'});
			push(@$lref, "\tmaxage = $_[0]->{'max'}")
				if ($_[0]->{'max'});
			push(@$lref, "\tpwdwarntime = $_[0]->{'warn'}")
				if ($_[0]->{'warn'});
			push(@$lref, "");
			}
		}
	}
$self->flush_file_lines() if (!$self->{'batch_mode'});
push(@{$self->{'list_users_cache_array'}}, $_[0]) if (defined(@{$self->{'list_users_cache_array'}}));
$self->refresh_nscd() if (!$self->{'batch_mode'});
}

# modify_user(&old, &details)
sub modify_user
{
	my $self = shift;

$_[0] || $self->error("Missing parameter to modify_user");
local(@passwd, @shadow, $lref, $fullreal);
local $pft = $self->passfiles_type();

if ($pft == 1) {
	# just need to update master.passwd
	$lref = $self->read_file_lines($self->{'config_hash'}->{'master_file'});
	
	if ($lref->[$_[0]->{'line'}] =~ /.*:.*:(.*,.*):.*:.*/) {
	        $fullreal = $1;
		$fullreal =~ s/^.*?,(.*)/$_[1]->{'real'},$1/g;
		$_[1]->{'real'} = $fullreal;
		}
		
	$lref->[$_[0]->{'line'}] = 
	      "$_[1]->{'user'}:$_[1]->{'pass'}:$_[1]->{'uid'}:".
	      "$_[1]->{'gid'}:$_[1]->{'class'}:$_[1]->{'change'}:".
	      "$_[1]->{'expire'}:$_[1]->{'real'}:$_[1]->{'home'}:".
	      "$_[1]->{'shell'}";
	}
elsif ($pft == 3) {
	# Just use the usermod command
	$self->system_logged("usermod -u $_[1]->{'uid'} -g $_[1]->{'gid'} -c \"$_[1]->{'real'}\" -d $_[1]->{'home'} -s $_[1]->{'shell'} $_[1]->{'user'}");
	$self->system_logged("echo ".quotemeta($_[1]->{'pass'})." | /usr/lib/scoadmin/account/password.tcl $_[1]->{'user'}");
	}
elsif ($pft == 6) {
	# Just use the niutil command to update
	if ($_[0]->{'user'} && $_[0]->{'user'} ne $_[1]->{'user'}) {
		# Need to delete and re-create!
		$self->system_logged("niutil -destroy '$self->{'netinfo_domain'}' '/users/$_[0]->{'user'}'");
		$self->system_logged("niutil -create '$self->{'netinfo_domain'}' '/users/$_[1]->{'user'}'");
		}
	$self->set_netinfo($_[1]);
	}
else {
	# update /etc/passwd
	$lref = $self->read_file_lines($self->{'config_hash'}->{'passwd_file'});
	
	if ($lref->[$_[0]->{'line'}] =~ /.*:.*:(.*,.*):.*:.*/) {
		$fullreal = $1;
		$fullreal =~ s/^.*?,(.*)/$_[1]->{'real'},$1/g;
		$_[1]->{'real'} = $fullreal;
		}


	$lref->[$_[0]->{'line'}] =
		"$_[1]->{'user'}:".
		($pft == 2 || $pft == 5 ? "x" : $pft == 4 ? "!" :
		 $_[1]->{'pass'}).
		":$_[1]->{'uid'}:$_[1]->{'gid'}:$_[1]->{'real'}:".
		"$_[1]->{'home'}:$_[1]->{'shell'}";
	if ($pft == 2) {
		# update shadow file as well..
		$lref = $self->read_file_lines($self->{'config_hash'}->{'shadow_file'});
		$lref->[$_[0]->{'sline'}] =
			"$_[1]->{'user'}:$_[1]->{'pass'}:$_[1]->{'change'}:".
			"$_[1]->{'min'}:$_[1]->{'max'}:$_[1]->{'warn'}:".
			"$_[1]->{'inactive'}:$_[1]->{'expire'}:";
		}
	elsif ($pft == 5) {
		# update SCO shadow
		$lref = $self->read_file_lines($self->{'config_hash'}->{'shadow_file'});
		$lref->[$_[0]->{'sline'}] =
		   "$_[1]->{'user'}:$_[1]->{'pass'}:$_[1]->{'change'}:".
		   "$_[1]->{'min'}:$_[1]->{'max'}";
		}
	elsif ($pft == 4) {
		# update AIX shadow passwd file as well..
		if (defined($_[0]->{'sline'})) {
			local @flags;
			push(@flags, 'ADMIN') if ($_[1]->{'admin'});
			push(@flags, 'ADMCHG') if ($_[1]->{'admchg'});
			push(@flags, 'NOCHECK') if ($_[1]->{'nocheck'});
			local $lref = $self->read_file_lines($self->{'config_hash'}->{'shadow_file'});
			splice(@$lref, $_[0]->{'sline'},
			     $_[0]->{'seline'} - $_[0]->{'sline'} + 1,
			     "$_[1]->{'user'}:", "\tpassword = $_[1]->{'pass'}",
			     "\tlastupdate = $_[1]->{'change'}",
			     "\tflags = ".join(",", @flags));
			$self->flush_file_lines();	# have to flush on AIX
			}

		# update AIX security user file as well..
		# use chuser command because it's easier than working
		# with the complexity issues of the file.
		$_[1]->{'expire'} = '' if (! $_[1]->{'expire'}); 
		$_[1]->{'min'} = '' if (! $_[1]->{'min'}); 
		$_[1]->{'max'} = '' if (! $_[1]->{'max'}); 
		$_[1]->{'warn'} = '' if (! $_[1]->{'warn'}); 
		$self->system_logged("chuser expires=$_[1]->{'expire'} minage=$_[1]->{'min'} maxage=$_[1]->{'max'} pwdwarntime=$_[1]->{'warn'} $_[1]->{'user'}");
		}
	}
if ($_[0] ne $_[1] && $self->indexof($_[0], @{$self->{'list_users_cache_array'}}) != -1) {
	# Update old object in cache
	$_[1]->{'line'} = $_[0]->{'line'} if (defined($_[0]->{'line'}));
	$_[1]->{'sline'} = $_[0]->{'sline'} if (defined($_[0]->{'sline'}));
	$_[1]->{'seline'} = $_[0]->{'seline'} if (defined($_[0]->{'seline'}));
	%{$_[0]} = %{$_[1]};
	}
if (!$self->{'batch_mode'}) {
	$self->flush_file_lines();
	$self->refresh_nscd();
	}
}

# delete_user(&details)
sub delete_user
{
	my $self = shift;

local $lref;
$_[0] || $self->error("Missing parameter to delete_user");

# delete quota if needed
if ($self->configured('quota'))
{
 $self->foreign_require("quota", "quota-lib.pl");
 my @f = $self->foreign_call("quota", "list_filesystems");
 foreach (@f)
 {
  $self->foreign_call("quota", "edit_user_quota", $_[0]->{'user'}, $_->[0], 0, 0, 0, 0);
 }
}

local $pft = $self->passfiles_type();
if ($pft == 1) {
	# Delete from BSD master.passwd file
	$lref = $self->read_file_lines($self->{'config_hash'}->{'master_file'});
	splice(@$lref, $_[0]->{'line'}, 1);
	map { $_->{'line'}-- if ($_->{'line'} > $_[0]->{'line'}) }
	    @{$self->{'list_users_cache_array'}};
	}
elsif ($pft == 3) {
	# Just invoke the userdel command
	$self->system_logged("userdel -n0 $_[0]->{'user'}");
	}
elsif ($pft == 4) {
	# Just invoke the rmuser command
	$self->system_logged("rmuser -p $_[0]->{'user'}");
	}
elsif ($pft == 6) {
	# Just delete with the niutil command
	$self->system_logged("niutil -destroy '$self->{'netinfo_domain'}' '/users/$_[0]->{'user'}'");
	}
else {
	# XXX doesn't delete from AIX file!
	$lref = $self->read_file_lines($self->{'config_hash'}->{'passwd_file'});
	splice(@$lref, $_[0]->{'line'}, 1);
	map { $_->{'line'}-- if ($_->{'line'} > $_[0]->{'line'}) }
	    @{$self->{'list_users_cache_array'}};
	if ($pft == 2 || $pft == 5) {
		if (defined($_[0]->{'sline'})) {
			$lref = $self->read_file_lines($self->{'config_hash'}->{'shadow_file'});
			splice(@$lref, $_[0]->{'sline'}, 1);
			map { $_->{'sline'}--
				if ($_->{'sline'} > $_[0]->{'sline'}) }
			    @{$self->{'list_users_cache_array'}};
			}
		}
	}
@{$self->{'list_users_cache_array'}} = grep { $_->{'user'} ne $_[0]->{'user'} } @{$self->{'list_users_cache_array'}}
	if (defined(@{$self->{'list_users_cache_array'}}));
if (!$self->{'batch_mode'}) {
	$self->flush_file_lines();
	$self->refresh_nscd();
	}
}

# list_groups()
# Returns a list of all the local groups as an array of hashtables. Each
# will contain group, pass, gid, members
sub list_groups
{
	my $self = shift;

return @{$self->{'list_groups_cache_array'}} if (defined(@{$self->{'list_groups_cache_array'}}));

local(@rv, $lnum, $_, %idx, $g, $i, $j, @gr);
$lnum = 0;
local $gft = $self->groupfiles_type();
if ($gft == 5) {
	# Get groups from netinfo
	$self->open_execute_command(*GROUP, "nidump group '$self->{'netinfo_domain'}'", 1);
	while(<GROUP>) {
		s/\r|\n//g;
		if (/\S/ && !/^[#\+\-]/) {
			@gr = split(/:/, $_, -1);
			push(@rv, { 'group' => $gr[0],	'pass' => $gr[1],
				    'gid' => $gr[2],
				    'members' => join(",",split(/\s+/,$gr[3])),
				    'num' => scalar(@rv) });
			}
		}
	close(GROUP);
	}
else {
	# Read the standard group file
	$self->open_readfile(*GROUP, $self->{'config_hash'}->{'group_file'});
	while(<GROUP>) {
		s/\r|\n//g;
		if (/\S/ && !/^[#\+\-]/) {
			@gr = split(/:/, $_, -1);
			push(@rv, { 'group' => $gr[0],	'pass' => $gr[1],
				    'gid' => $gr[2],	'members' => $gr[3],
				    'line' => $lnum,	'num' => scalar(@rv) });
			$idx{$gr[0]} = $rv[$#rv];
			}
		$lnum++;
		}
	close(GROUP);
	}
if ($gft == 2) {
	# read the gshadow file data
	$lnum = 0;
	$self->open_readfile(*SHADOW, $self->{'config_hash'}->{'gshadow_file'});
	while(<SHADOW>) {
		s/\r|\n//g;
		if (/\S/ && !/^[#\+\-]/) {
			@gr = split(/:/, $_, -1);
			$g = $idx{$gr[0]};
			$g->{'pass'} = $gr[1];
			$g->{'sline'} = $lnum;
			}
		$lnum++;
		}
	close(SHADOW);
	#for($i=0; $i<@rv; $i++) {
	#	if (!defined($rv[$i]->{'sline'})) {
	#		# not in shadow!
	#		for($j=$i; $j<@rv; $j++) { $rv[$j]->{'num'}--; }
	#		splice(@rv, $i--, 1);
	#		}
	#	}
	}
elsif ($gft == 4) {
	# read the AIX group data
	local $lastgroup;
	local $lnum = 0;
	$self->open_readfile(*SECURITY, $self->{'config_hash'}->{'gshadow_file'});
	while(<SECURITY>) {
		s/\s*$//;
		if (/^\s*(\S+):/) {
			$lastgroup = $idx{$1};
			$lastgroup->{'sline'} = $lnum;
			$lastgroup->{'seline'} = $lnum;
			}
		elsif (/^\s*([^=\s]+)\s*=\s*(.*)/) {
			$lastgroup->{'seline'} = $lnum;
			}
		$lnum++;
		}
	close(SECURITY);
	}
@{$self->{'list_groups_cache_array'}} = @rv;
return @rv;
}

# create_group(&details)
sub create_group
{
	my $self = shift;

local $gft = $self->groupfiles_type();
if ($gft == 5) {
	# Use niutil command
	$self->system_logged("niutil -create '$self->{'netinfo_domain'}' '/groups/$_[0]->{'group'}'");
	$self->set_group_netinfo($_[0]);
	}
else {
	# Update group file(s)
	local $lref;
	$lref = $self->read_file_lines($self->{'config_hash'}->{'group_file'});
	$_[0]->{'line'} = $self->nis_index($lref);
	if (defined(@{$self->{'list_groups_cache_array'}})) {
		map { $_->{'line'}++ if ($_->{'line'} >= $_[0]->{'line'}) }
		    @{$self->{'list_groups_cache_array'}};
		}
	splice(@$lref, $_[0]->{'line'}, 0,
	       "$_[0]->{'group'}:".
	       ($self->groupfiles_type() == 2 ? "x" : $_[0]->{'pass'}).
	       ":$_[0]->{'gid'}:$_[0]->{'members'}");
	if ($gft == 2) {
		$lref = $self->read_file_lines($self->{'config_hash'}->{'gshadow_file'});
		$_[0]->{'sline'} = $self->nis_index($lref);
		if (defined(@{$self->{'list_groups_cache_array'}})) {
			map { $_->{'sline'}++
			      if ($_->{'sline'} >= $_[0]->{'sline'}) }
			    @{$self->{'list_groups_cache_array'}};
			}
		splice(@$lref, $_[0]->{'sline'}, 0,
		       "$_[0]->{'group'}:$_[0]->{'pass'}::$_[0]->{'members'}");
		}
	elsif ($gft == 4) {
		$lref = $self->read_file_lines($self->{'config_hash'}->{'gshadow_file'});
		$_[0]->{'sline'} = scalar(@$lref);
		push(@$lref, "", "$_[0]->{'group'}:", "\tadmin = false");
		}
	$self->flush_file_lines();
	}
$self->refresh_nscd();
push(@{$self->{'list_groups_cache_array'}}, $_[0]) if (defined(@{$self->{'list_groups_cache_array'}}));
}

# modify_group(&old, &details)
sub modify_group
{
	my $self = shift;

$_[0] || $self->error("Missing parameter to modify_group");
local $gft = $self->groupfiles_type();
if ($gft == 5) {
	# Call niutil to update the group
	if ($_[0]->{'group'} && $_[0]->{'group'} ne $_[1]->{'group'}) {
		# Need to delete and re-create!
		$self->system_logged("niutil -destroy '$self->{'netinfo_domain'}' '/groups/$_[0]->{'group'}'");
		$self->system_logged("niutil -create '$self->{'netinfo_domain'}' '/groups/$_[1]->{'group'}'");
		}
	$self->set_group_netinfo($_[1]);
	}
else {
	# Update in files
	local $gs = ($self->groupfiles_type() == 2 && $_[0]->{'sline'} ne '');
	$self->replace_file_line($self->{'config_hash'}->{'group_file'}, $_[0]->{'line'},
		   "$_[1]->{'group'}:".($gs ? "x" : $_[1]->{'pass'}).
		   ":$_[1]->{'gid'}:$_[1]->{'members'}\n");
	if ($gs) {
		$self->replace_file_line($self->{'config_hash'}->{'gshadow_file'}, $_[0]->{'sline'},
				   "$_[1]->{'group'}:$_[1]->{'pass'}::$_[1]->{'members'}\n");
		}
	elsif ($self->groupfiles_type() == 4) {
		$self->replace_file_line($self->{'config_hash'}->{'gshadow_file'},
				   $_[0]->{'sline'},
				   "$_[1]->{'group'}:\n");
		}
	}
if ($_[0] ne $_[1] && $self->indexof($_[0], @{$self->{'list_groups_cache_array'}}) != -1) {
	$_[1]->{'line'} = $_[0]->{'line'} if (defined($_[0]->{'line'}));
	$_[1]->{'sline'} = $_[0]->{'sline'} if (defined($_[0]->{'sline'}));
	%{$_[0]} = %{$_[1]};
	}
$self->refresh_nscd();
}

# delete_group(&details)
sub delete_group
{
	my $self = shift;

$_[0] || $self->error("Missing parameter to delete_group");

# delete quota if needed
if ($self->configured('quota'))
{
 $self->foreign_require("quota", "quota-lib.pl");
 my @f = $self->foreign_call("quota", "list_filesystems");
 foreach (@f)
 {
  $self->foreign_call("quota", "edit_group_quota", $_[0]->{'group'}, $_->[0], 0, 0, 0, 0);
 }
}

local $gft = $self->groupfiles_type();
if ($gft == 5) {
	# Call niutil to delete
	$self->system_logged("niutil -destroy '$self->{'netinfo_domain'}' '/groups/$_[0]->{'group'}'");
	}
else {
	# Remove from group file(s)
	$self->replace_file_line($self->{'config_hash'}->{'group_file'}, $_[0]->{'line'});
	map { $_->{'line'}-- if ($_->{'line'} > $_[0]->{'line'}) }
	    @{$self->{'list_groups_cache_array'}};
	if ($gft == 2 && $_[0]->{'sline'} ne '') {
		$self->replace_file_line($self->{'config_hash'}->{'gshadow_file'}, $_[0]->{'sline'});
		map { $_->{'sline'}-- if ($_->{'sline'} > $_[0]->{'sline'}) }
		    @{$self->{'list_groups_cache_array'}};
		}
	elsif ($gft == 4) {
		local $lref = $self->read_file_lines($self->{'config_hash'}->{'gshadow_file'});
		splice(@$lref, $_[0]->{'sline'},
		       $_[0]->{'seline'} - $_[0]->{'sline'} + 1);
		$self->flush_file_lines();
		}
	}
@{$self->{'list_groups_cache_array'}} = grep { $_ ne $_[0] } @{$self->{'list_groups_cache_array'}}
	if (defined(@{$self->{'list_groups_cache_array'}}));
$self->refresh_nscd();
}

############################################################################
# Misc functions
############################################################################
# $self->recursive_change(dir, olduid, oldgid, newuid, newgid)
# Change the UID or GID of a directory and all files in it, if they match the
# given UID/GID
sub recursive_change
{
	my $self = shift;

local(@list, $f, @stbuf);
local $real = $self->translate_filename($_[0]);
(@stbuf = stat($real)) || return;
(-l $real) && return;
if (($_[1] < 0 || $_[1] == $stbuf[4]) &&
    ($_[2] < 0 || $_[2] == $stbuf[5])) {
	# Found match..
	$self->set_ownership_permissions(
		$_[3] < 0 ? $stbuf[4] : $_[3],
	      	$_[4] < 0 ? $stbuf[5] : $_[4], undef, $_[0]);
	}
if (-d $real) {
	opendir(DIR, $real);
	@list = readdir(DIR);
	closedir(DIR);
	foreach $f (@list) {
		if ($f eq "." || $f eq "..") { next; }
		$self->recursive_change("$_[0]/$f", $_[1], $_[2], $_[3], $_[4]);
		}
	}
}

# making_changes()
# Called before changes are made to the password or group file
sub making_changes
{
	my $self = shift;

if ($self->{'config_hash'}->{'pre_command'} =~ /\S/) {
	local $out = $self->backquote_logged("($self->{'config_hash'}->{'pre_command'}) 2>&1 </dev/null");
	return $? ? $out : undef;
	}
return undef;
}

# made_changes()
# Called after the password or group file has been changed, to run the
# post-changes command.
sub made_changes
{
	my $self = shift;

if ($self->{'config_hash'}->{'post_command'} =~ /\S/) {
	local $out = $self->backquote_logged("($self->{'config_hash'}->{'post_command'}) 2>&1 </dev/null");
	return $? ? $out : undef;
	}
return undef;
}

# other_modules(function, arg, ...)
# Call some function in the useradmin_update.pl file in other modules
sub other_modules
{
	my $self = shift;

return if ($self->is_readonly_mode());	# don't even try other modules
local($m, %minfo);
local $func = shift(@_);
foreach $m ($self->get_all_module_infos()) {
	local $mdir = $self->module_root_directory($m->{'dir'});
	if ($self->check_os_support($m) &&
	    -r "$mdir/useradmin_update.pl") {
		$self->foreign_require($m->{'dir'}, "useradmin_update.pl");
		local $pkg = $m->{'dir'};
		$pkg =~ s/[^A-Za-z0-9]/_/g;
		local $fullfunc = "${pkg}::${func}";
		if (defined(&$fullfunc)) {
			$self->foreign_call($m->{'dir'}, $func, @_);
			}
		}
	}
}

# can_edit_user(&acl, &user)
sub can_edit_user
{
	my $self = shift;

local $m = $_[0]->{'uedit_mode'};
local %u;
if ($m == 0) { return 1; }
elsif ($m == 1) { return 0; }
elsif ($m == 2 || $m == 3 || $m == 5) {
	map { $u{$_}++ } split(/\s+/, $_[0]->{'uedit'});
	if ($m == 5 && $_[0]->{'uedit_sec'}) {
		# Check secondary groups too
		return 1 if ($u{$_[1]->{'gid'}});
		foreach $g ($self->list_groups()) {
			local @m = split(/,/, $g->{'members'});
			return 1 if ($u{$g->{'gid'}} &&
				     $self->indexof($_[1]->{'user'}, @m) >= 0);
			}
		return 0;
		}
	else {
		return $m == 2 ? $u{$_[1]->{'user'}} :
		       $m == 3 ? !$u{$_[1]->{'user'}} :
				 $u{$_[1]->{'gid'}};
		}
	}
elsif ($m == 4) {
	return (!$_[0]->{'uedit'} || $_[1]->{'uid'} >= $_[0]->{'uedit'}) &&
	       (!$_[0]->{'uedit2'} || $_[1]->{'uid'} <= $_[0]->{'uedit2'});
	}
elsif ($m == 6) {
	return $_[1]->{'user'} eq $self->{'remote_user'};
	}
elsif ($m == 7) {
	return $_[1]->{'user'} =~ /$_[0]->{'uedit_re'}/;
	}
return 0;
}

# can_edit_group(&acl, &group)
sub can_edit_group
{
	my $self = shift;

local $m = $_[0]->{'gedit_mode'};
local %g;
if ($m == 0) { return 1; }
elsif ($m == 1) { return 0; }
elsif ($m == 2 || $m == 3) {
	map { $g{$_}++ } split(/\s+/, $_[0]->{'gedit'});
	return $m == 2 ? $g{$_[1]->{'group'}}
		       : !$g{$_[1]->{'group'}};
	}
else { return (!$_[0]->{'gedit'} || $_[1]->{'gid'} >= $_[0]->{'gedit'}) &&
	      (!$_[0]->{'gedit2'} || $_[1]->{'gid'} <= $_[0]->{'gedit2'}); }
}

# nis_index(&lines)
sub nis_index
{
	my $self = shift;

local $i;
for($i=0; $i<@{$_[0]}; $i++) {
	last if ($_[0]->[$i] =~ /^[\+\-]/);
	}
return $i;
}

# get_skel_directory(&user, groupname)
# Returns the skeleton files directory for some user
sub get_skel_directory
{
	my $self = shift;

local ($user, $groupname) = @_;
local $uf = $self->{'config_hash'}->{'user_files'};
local $shell = $user->{'shell'};
$shell =~ s/^(.*)\///g;
if ($groupname ne '') {
	$uf =~ s/\$group/$groupname/g;
	}
$uf =~ s/\$gid/$user->{'gid'}/g;
$uf =~ s/\$shell/$shell/g;
return $uf;
}

# copy_skel_files(source, dest, uid, gid)
sub copy_skel_files
{
	my $self = shift;

local ($f, $df);
local @rv;
foreach $f (split(/\s+/, $_[0])) {
	if (-d $f) {
		# copy all files in a directory
		opendir(DIR, $f);
		foreach $df (readdir(DIR)) {
			if ($df eq "." || $df eq "..") { next; }
			push(@rv, $self->copy_file("$f/$df", $_[1], $_[2], $_[3]));
			}
		closedir(DIR);
		}
	elsif (-r $f) {
		# copy just one file
		push(@rv, $self->copy_file($f, $_[1], $_[2], $_[3]));
		}
	}
return @rv;
}

# copy_file(file, destdir, uid, gid)
# Copy a file or directory and chown it
sub copy_file
{
	my $self = shift;

local($base, $subs);
$_[0] =~ /\/([^\/]+)$/; $base = $1;
if ($self->{'config_hash'}->{"files_remap_$base"}) {
	$base = $self->{'config_hash'}->{"files_remap_$base"};
	}
$subs = $self->{'config_hash'}->{'files_remove'};
$base =~ s/$subs//g if ($subs);
local ($opts, $nochown);
local @rv = ( "$_[1]/$base" );
if (-b $_[0] || -c $_[0]) {
	# Looks like a device file .. re-create it
	local @st = stat($_[0]);
	local $maj = int($st[6] / 256);
	local $min = $st[6] % 256;
	local $typ = ($st[2] & 00170000) == 0020000 ? 'c' : 'b';
	$self->system_logged("mknod ".quotemeta("$_[1]/$base")." $typ $maj $min");
	$self->set_ownership_permissions($_[2], $_[3], undef, "$_[1]/$base");
	$nochown++;
	}
elsif (-l $_[0] && !$self->{'config_hash'}->{'copy_symlinks'}) {
	# A symlink .. re-create it
	local $l = readlink($_[0]);
	$self->system_logged("ln -s ".quotemeta($l)." ".quotemeta("$_[1]/$base")." >/dev/null 2>/dev/null");
	$opts = "-h";
	}
elsif (-d $_[0]) {
	# A directory .. copy it recursively
	$self->system_logged("cp -Rp ".quotemeta($_[0])." ".quotemeta("$_[1]/$base")." >/dev/null 2>/dev/null");
	@rv = glob("$_[1]/$base/*");
	}
else {
	# Just a normal file .. copy it
	local @st = stat($self->translate_filename($_[0]));
	$self->system_logged("cp ".quotemeta($_[0])." ".quotemeta("$_[1]/$base")." >/dev/null 2>/dev/null");
	$self->set_ownership_permissions($_[2], $_[3], $st[2], "$_[1]/$base");
	$nochown++;
	}
$self->system_logged("chown $opts -R $_[2]:$_[3] \"$_[1]/$base\" >/dev/null 2>/dev/null") if (!$nochown);
return @rv;
}

# lock_user_files()
# Lock all password, shadow and group files
sub lock_user_files
{
	my $self = shift;

$self->lock_file($self->{'config_hash'}->{'passwd_file'});
$self->lock_file($self->{'config_hash'}->{'group_file'});
$self->lock_file($self->{'config_hash'}->{'shadow_file'});
$self->lock_file($self->{'config_hash'}->{'gshadow_file'});
$self->lock_file($self->{'config_hash'}->{'master_file'});
}

# unlock_user_files()
# Unlock all password, shadow and group files
sub unlock_user_files
{
	my $self = shift;

$self->unlock_file($self->{'config_hash'}->{'passwd_file'});
$self->unlock_file($self->{'config_hash'}->{'group_file'});
$self->unlock_file($self->{'config_hash'}->{'shadow_file'});
$self->unlock_file($self->{'config_hash'}->{'gshadow_file'});
$self->unlock_file($self->{'config_hash'}->{'master_file'});
}

sub my_setpwent
{
	my $self = shift;

if ($self->{'config_hash'}->{'from_files'}) {
	@{$self->{'setpwent_cache_array'}} = $self->list_users();
	$self->{'setpwent_pos'} = 0;
	}
else { return setpwent(); }
}

sub my_getpwent
{
	my $self = shift;

if ($self->{'config_hash'}->{'from_files'}) {
	$self->my_setpwent() if (!@{$self->{'setpwent_cache_array'}});
	if ($self->{'setpwent_pos'} >= @{$self->{'setpwent_cache_array'}}) {
		return wantarray ? () : undef;
		}
	else {
		return $self->pw_user_rv($self->{'setpwent_cache_array'}->[$self->{'setpwent_pos'}++],
				   wantarray, 'user');
		}
	}
else { return getpwent(); }
}

sub my_endpwent
{
	my $self = shift;

if ($self->{'config_hash'}->{'from_files'}) {
	undef(@{$self->{'setpwent_cache_array'}});
	}
elsif ($self->{'gconfig_hash'}->{'os_type'} eq 'hpux') {
	# On hpux, endpwent() can crash perl!
	return 0;
	}
else { return endpwent(); }
}

sub my_getpwnam
{
	my $self = shift;

if ($self->{'config_hash'}->{'from_files'}) {
	local $u;
	foreach $u ($self->list_users()) {
		return $self->pw_user_rv($u, wantarray, 'uid')
			if ($u->{'user'} eq $_[0]);
		}
	return wantarray ? () : undef;
	}
else { return getpwnam($_[0]); }
}

sub my_getpwuid
{
	my $self = shift;

if ($self->{'config_hash'}->{'from_files'}) {
	foreach $u ($self->list_users()) {
		return $self->pw_user_rv($u, wantarray, 'user')
			if ($u->{'uid'} eq $_[0]);
		}
	return wantarray ? () : undef;
	}
else { return getpwuid($_[0]); }
}

sub pw_user_rv
{
	my $self = shift;

return $_[0] ? ( $_[0]->{'user'}, $_[0]->{'pass'}, $_[0]->{'uid'},
		 $_[0]->{'gid'}, undef, undef, $_[0]->{'real'},
		 $_[0]->{'home'}, $_[0]->{'shell'}, undef ) : $_[0]->{$_[2]};
}

sub my_setgrent
{
	my $self = shift;

if ($self->{'config_hash'}->{'from_files'}) {
	@{$self->{'setgrent_cache_array'}} = $self->list_groups();
	$self->{'setgrent_pos'} = 0;
	}
else { return setgrent(); }
}

sub my_getgrent
{
	my $self = shift;

if ($self->{'config_hash'}->{'from_files'}) {
	$self->my_setgrent() if (!@{$self->{'setgrent_cache_array'}});
	if ($self->{'setgrent_pos'} >= @{$self->{'setgrent_cache_array'}}) {
		return ();
		}
	else {
		return $self->gr_group_rv($self->{'setgrent_cache_array'}->[$self->{'setgrent_pos'}++],
				    wantarray, 'group');
		}
	}
else { return getgrent(); }
}

sub my_endgrent
{
	my $self = shift;

if ($self->{'config_hash'}->{'from_files'}) {
	undef(@{$self->{'setgrent_cache_array'}});
	}
elsif ($self->{'gconfig_hash'}->{'os_type'} eq 'hpux') {
	# On hpux, endpwent() can crash perl!
	return 0;
	}
else { return endgrent(); }
}

sub my_getgrnam
{
	my $self = shift;

if ($self->{'config_hash'}->{'from_files'}) {
	local $g;
	foreach $g ($self->list_groups()) {
		return $self->gr_group_rv($g, wantarray, 'gid')
			if ($g->{'group'} eq $_[0]);
		}
	return wantarray ? () : undef;
	}
else { return getgrnam($_[0]); }
}

sub my_getgrgid
{
	my $self = shift;

if ($self->{'config_hash'}->{'from_files'}) {
	foreach $g ($self->list_groups()) {
		return $self->gr_group_rv($g, wantarray, 'group')
			if ($g->{'gid'} eq $_[0]);
		}
	return wantarray ? () : undef;
	}
else { return getgrgid($_[0]); }
}

sub gr_group_rv
{
	my $self = shift;

return $_[1] ? ( $_[0]->{'group'}, $_[0]->{'pass'}, $_[0]->{'gid'},
		 $_[0]->{'members'} ) : $_[0]->{$_[2]};
}

# auto_home_dir(base, username, groupname)
# Returns an automatically generated home directory, and creates needed
# parent dirs
sub auto_home_dir
{
	my $self = shift;

local $pfx = $_[0] eq "/" ? "/" : $_[0]."/";
if ($self->{'config_hash'}->{'home_style'} == 0) {
	return $pfx.$_[1];
	}
elsif ($self->{'config_hash'}->{'home_style'} == 1) {
	$self->mkdir_if_needed($pfx.substr($_[1], 0, 1));
	return $pfx.substr($_[1], 0, 1)."/".$_[1];
	}
elsif ($self->{'config_hash'}->{'home_style'} == 2) {
	$self->mkdir_if_needed($pfx.substr($_[1], 0, 1));
	$self->mkdir_if_needed($pfx.substr($_[1], 0, 1)."/".
			 substr($_[1], 0, 2));
	return $pfx.substr($_[1], 0, 1)."/".
	       substr($_[1], 0, 2)."/".$_[1];
	}
elsif ($self->{'config_hash'}->{'home_style'} == 3) {
	$self->mkdir_if_needed($pfx.substr($_[1], 0, 1));
	$self->mkdir_if_needed($pfx.substr($_[1], 0, 1)."/".
			 substr($_[1], 1, 1));
	return $pfx.substr($_[1], 0, 1)."/".
	       substr($_[1], 1, 1)."/".$_[1];
	}
elsif ($self->{'config_hash'}->{'home_style'} == 4) {
	return $_[0];
	}
elsif ($self->{'config_hash'}->{'home_style'} == 5) {
	return $pfx.$_[2]."/".$_[1];
	}
}

sub mkdir_if_needed
{
	my $self = shift;

-d $_[0] || $self->make_dir($_[0], 0755);
}

sub set_netinfo
{
	my $self = shift;

local %u = %{$_[0]};
$self->system_logged("niutil -createprop '$self->{'netinfo_domain'}' '/users/$u{'user'}' passwd '$u{'pass'}'");
$self->system_logged("niutil -createprop '$self->{'netinfo_domain'}' '/users/$u{'user'}' uid '$u{'uid'}'");
$self->system_logged("niutil -createprop '$self->{'netinfo_domain'}' '/users/$u{'user'}' gid '$u{'gid'}'");
$self->system_logged("niutil -createprop '$self->{'netinfo_domain'}' '/users/$u{'user'}' class '$u{'class'}'");
$self->system_logged("niutil -createprop '$self->{'netinfo_domain'}' '/users/$u{'user'}' change '$u{'change'}'");
$self->system_logged("niutil -createprop '$self->{'netinfo_domain'}' '/users/$u{'user'}' expire '$u{'expire'}'");
$self->system_logged("niutil -createprop '$self->{'netinfo_domain'}' '/users/$u{'user'}' realname '$u{'real'}'");
$self->system_logged("niutil -createprop '$self->{'netinfo_domain'}' '/users/$u{'user'}' home '$u{'home'}'");
$self->system_logged("niutil -createprop '$self->{'netinfo_domain'}' '/users/$u{'user'}' shell '$u{'shell'}'");
}

sub set_group_netinfo
{
	my $self = shift;

local %g = %{$_[0]};
local $mems = join(" ", map { "'$_'" } split(/,/, $g{'members'}));
$self->system_logged("niutil -createprop '$self->{'netinfo_domain'}' '/groups/$g{'group'}' gid '$g{'gid'}'");
$self->system_logged("niutil -createprop '$self->{'netinfo_domain'}' '/groups/$g{'group'}' passwd '$g{'pass'}'");
$self->system_logged("niutil -createprop '$self->{'netinfo_domain'}' '/groups/$g{'group'}' users $mems");
}

# check_password_restrictions(pass, username)
# Returns an error message if the given password fails length and other
# checks, or undef if it is OK
sub check_password_restrictions
{
	my $self = shift;

return $self->text('usave_epasswd_min', $self->{'config_hash'}->{'passwd_min'})
	if (length($_[0]) < $self->{'config_hash'}->{'passwd_min'});
local $re = $self->{'config_hash'}->{'passwd_re'};
return $self->text('usave_epasswd_re', $re)
	if ($re && !eval { $_[0] =~ /^$re$/ });
if ($self->{'config_hash'}->{'passwd_same'}) {
	return $self->text('usave_epasswd_same') if ($_[0] =~ /\Q$_[1]\E/i);
	}
if ($self->{'config_hash'}->{'passwd_dict'} && $_[0] =~ /^[A-Za-z\'\-]+$/ &&
    ($self->has_command("ispell") || $self->has_command("spell"))) {
	# Call spell or ispell to check for dictionary words
	local $temp = $self->transname();
	open(TEMP, ">$temp");
	print TEMP $_[0],"\n";
	close(TEMP);
	if ($self->has_command("ispell")) {
		open(SPELL, "ispell -a <$temp |");
		while(<SPELL>) {
			if (/^(#|\&|\?)/) {
				$self->{'unknown'}++;
				}
			}
		close(SPELL);
		}
	else {
		open(SPELL, "spell <$temp |");
		local $line = <SPELL>;
		$self->{'unknown'}++ if ($line);
		close(SPELL);
		}
	unlink($temp);
	return $self->text('usave_epasswd_dict') if (!$self->{'unknown'});
	}
if ($self->{'config_hash'}->{'passwd_prog'}) {
	# Run external validation program with username and password as args
	local $qu = quotemeta($_[1]);
	local $qp = quotemeta($_[0]);
	local $out = $self->backquote_command("$self->{'config_hash'}->{'passwd_prog'} $qu $qp 2>&1 </dev/null");
	if ($?) {
		return $out;
		}
	}
return undef;
}

# check_username_restrictions(username)
# Returns an error message if a username fails some restriction, or undef
sub check_username_restrictions
{
	my $self = shift;

if ($self->{'config_hash'}->{'max_length'} && length($_[0]) > $self->{'config_hash'}->{'max_length'}) {
	return $self->text('usave_elength', $self->{'config_hash'}->{'max_length'});
	}
local $re = $self->{'config_hash'}->{'username_re'};
return $self->text('usave_ere', $re)
	if ($re && !eval { $_[0] =~ /^$re$/ });
return undef;
}

# can_use_group(&acl, group)
# Returns 1 if some group can be used as a primary or secondary, 0 if not
sub can_use_group
{
	my $self = shift;

return 1 if ($_[0]->{'ugroups'} eq '*');
local @sp = split(/\s+/, $_[0]->{'ugroups'});
if ($_[0]->{'uedit_gmode'} == 3) {
	return $self->indexof($_[1], @sp) < 0;
	}
elsif ($_[0]->{'uedit_gmode'} == 4) {
	local @ginfo = $self->my_getgrnam($_[1]);
	return (!$_[0]->{'ugroups'} || $ginfo[2] >= $_[0]->{'ugroups'}) &&
	       (!$_[0]->{'ugroups2'} || $ginfo[2] <= $_[0]->{'ugroups2'});
	}
else {
	return $self->indexof($_[1], @sp) >= 0;
	}
}

# refresh_nscd()
# Sends a HUP signal to the nscd process, so that any caches are reloaded
sub refresh_nscd
{
	my $self = shift;

return if ($self->{'nscd_not_running'});
if (!$self->find_byname("nscd")) {
	$self->{'nscd_not_running'}++;
	}
elsif ($self->{'config_hash'}->{'nscd_restart'}) {
	# Run the specified command
	$self->system_logged("$self->{'config_hash'}->{'nscd_restart'} >/dev/null 2>&1 </dev/null");
	}
elsif ($self->has_command("nscd")) {
	# Use nscd -i to reload
	$self->system_logged("nscd -i passwd -i group >/dev/null 2>&1 </dev/null");
	}
else {
	# Send HUP signal
	local $rv = $self->kill_byname_logged("nscd", "HUP");
	if (!$rv) {
		$self->{'nscd_not_running'}++;
		}
	}
sleep(1);	# Give ncsd time to react
}

# set_user_envs(&user, action, [plainpass], [secondaries])
# Sets up the USERADMIN_ environment variables for a user update of some kind,
# prior to calling making_changes or made_changes. action must be one of
# CREATE_USER, MODIFY_USER or DELETE_USER
sub set_user_envs
{
	my $self = shift;

$self->clear_envs();
$ENV{'USERADMIN_USER'} = $_[0]->{'user'};
$ENV{'USERADMIN_UID'} = $_[0]->{'uid'};
$ENV{'USERADMIN_REAL'} = $_[0]->{'real'};
$ENV{'USERADMIN_SHELL'} = $_[0]->{'shell'};
$ENV{'USERADMIN_HOME'} = $_[0]->{'home'};
$ENV{'USERADMIN_GID'} = $_[0]->{'gid'};
$ENV{'USERADMIN_PASS'} = $_[2] if (defined($_[2]));
$ENV{'USERADMIN_SECONDARY'} = join(",", @{$_[3]}) if (defined($_[3]));
$ENV{'USERADMIN_ACTION'} = $_[1];
$ENV{'USERADMIN_SOURCE'} = $self->{'module_name'};
}

# set_group_envs(&group, action)
# Sets up the USERADMIN_ environment variables for a group update of some kind,
# prior to calling making_changes or made_changes. action must be one of
# CREATE_GROUP, MODIFY_GROUP or DELETE_GROUP
sub set_group_envs
{
	my $self = shift;

$self->clear_envs();
$ENV{'USERADMIN_GROUP'} = $_[0]->{'group'};
$ENV{'USERADMIN_GID'} = $_[0]->{'gid'};
$ENV{'USERADMIN_MEMBERS'} = $_[0]->{'members'};
$ENV{'USERADMIN_ACTION'} = $_[1];
$ENV{'USERADMIN_SOURCE'} = $self->{'module_name'};
}

# clear_envs()
# Removes all variables set by set_user_envs and set_group_envs
sub clear_envs
{
	my $self = shift;

local $e;
foreach $e (keys %ENV) {
	delete($ENV{$e}) if ($e =~ /^USERADMIN_/);
	}
}

# encrypt_password(password, [salt])
# Encrypts a password using the encryption format configured for this system
sub encrypt_password
{
	my $self = shift;

local ($pass, $salt) = @_;
local $md5 = 0;
if ($self->{'config_hash'}->{'md5'} == 2) {
	# Always use MD5
	$md5 = 1;
	}
elsif ($self->{'config_hash'}->{'md5'} == 1 && !$self->{'config_hash'}->{'skip_md5'}) {
	# Up to system
	$md5 = $self->use_md5() if ($self->UNIVERSAL::can('use_md5'));
	}
if ($self->{'no_encrypt_password'}) {
	# Some operating systems don't do any encryption!
	return $pass;
	}
elsif ($md5) {
	# MD5 encryption is selected .. use it if possible
	local $err = $self->check_md5();
	if ($err) {
		$self->header($self->{'text_hash'}->{'error'}, "");
		print "<hr><p>\n";
		print $self->text('usave_edigestmd5',
		    "/config.cgi?$self->{'module_name'}",
		    "/cpan/download.cgi?source=3&cpan=$err"),
		    "<p>\n";
		print "<hr>\n";
		$self->footer("", $self->{'text_hash'}->{'index_return'});
		exit;
		}
	return $self->encrypt_md5($pass, $salt);
	}
else {
	# Just do old-style crypt() DES encryption
	$salt ||= chr(int(rand(26))+65) . chr(int(rand(26))+65);
	return $self->unix_crypt($pass, $salt);
	}
}

# build_user_used([&uid-hash], [&shell-list], [&username-hash])
# Fills in a hash with used UIDs and shells
sub build_user_used
{
	my $self = shift;

$self->my_setpwent();
local @u;
while(@u = $self->my_getpwent()) {
	$_[0]->{$u[2]}++ if ($_[0]);
	push(@{$_[1]}, $u[8]) if ($_[1] && $u[8]);
	$_[2]->{$u[0]}++ if ($_[2]);
	}
$self->my_endpwent();
local $u;
foreach $u ($self->list_users()) {
	$_[0]->{$u->{'uid'}}++ if ($_[0]);
	push(@{$_[1]}, $u->{'shell'}) if ($_[1] && $u->{'shell'});
	$_[2]->{$u->{'user'}}++ if ($_[2]);
	}
}

# build_group_used([&uid-hash], [&groupname-hash])
sub build_group_used
{
	my $self = shift;

$self->my_setgrent();
local @g;
while(@g = $self->my_getgrent()) {
	$_[0]->{$g[2]}++ if ($_[0]);
	$_[1]->{$g[0]}++ if ($_[1]);
	}
$self->my_endgrent();
local $g;
foreach $g ($self->list_groups()) {
	$_[0]->{$g->{'gid'}}++ if ($_[0]);
	$_[1]->{$g->{'group'}}++ if ($_[1]);
	}
}

# allocate_uid(&uids-used)
sub allocate_uid
{
	my $self = shift;

local $rv = int($self->{'config_hash'}->{'base_uid'} > $self->{'access_hash'}->{'lowuid'} ?
		$self->{'config_hash'}->{'base_uid'} : $self->{'access_hash'}->{'lowuid'});
while($_[0]->{$rv}) {
	$rv++;
	}
return $rv;
}

# allocate_gid(&gids-used)
sub allocate_gid
{
	my $self = shift;

local $rv = int($self->{'config_hash'}->{'base_gid'} > $self->{'access_hash'}->{'lowgid'} ?
		$self->{'config_hash'}->{'base_gid'} : $self->{'access_hash'}->{'lowgid'});
while($_[0]->{$rv}) {
	$rv++;
	}
return $rv;
}

# list_allowed_users(&access, &allusers)
# Returns a list of users to whom access is allowed
sub list_allowed_users
{
	my $self = shift;

local %access = %{$_[0]};
local @ulist = @{$_[1]};
if ($access{'uedit_mode'} == 1) {
	@ulist = ();
	}
elsif ($access{'uedit_mode'} == 2) {
	local %canu;
	map { $canu{$_}++ } split(/\s+/, $access{'uedit'});
	@ulist = grep { $canu{$_->{'user'}} } @ulist;
	}
elsif ($access{'uedit_mode'} == 3) {
	local %cannotu;
	map { $cannotu{$_}++ } split(/\s+/, $access{'uedit'});
	@ulist = grep { !$cannotu{$_->{'user'}} } @ulist;
	}
elsif ($access{'uedit_mode'} == 4) {
	@ulist = grep {
		(!$access{'uedit'} || $_->{'uid'} >= $access{'uedit'}) &&
		(!$access{'uedit2'} || $_->{'uid'} <= $access{'uedit2'})
			} @ulist;
	}
elsif ($access{'uedit_mode'} == 5) {
	local %cangid;
	map { $cangid{$_}++ } split(/\s+/, $access{'uedit'});
	if ($access{'uedit_sec'}) {
		# Match secondary groups too
		local @glist = $self->list_groups();
		local (@ucan, $g);
		foreach $g (@glist) {
			push(@ucan, split(/,/, $g->{'members'}))
				if ($cangid{$g->{'gid'}});
			}
		@ulist = grep { $cangid{$_->{'gid'}} ||
				$self->indexof($_->{'user'}, @ucan) >= 0 } @ulist;
		}
	else {
		@ulist = grep { $cangid{$_->{'gid'}} } @ulist;
		}
	}
elsif ($access{'uedit_mode'} == 6) {
	@ulist = grep { $_->{'user'} eq $self->{'remote_user'} } @ulist;
	}
elsif ($access{'uedit_mode'} == 7) {
	@ulist = grep { $_->{'user'} =~ /$access{'uedit_re'}/ } @ulist;
	}
elsif ($access{'uedit_mode'} == 8) {
	@ulist = grep {
		(!$access{'uedit'} || $_->{'gid'} >= $access{'uedit'}) &&
		(!$access{'uedit2'} || $_->{'gid'} <= $access{'uedit2'})
			} @ulist;
	}
if ($access{'view'}) {
	# Include non-editable users in results
	local @rv = @{$_[1]};
	local $u;
	foreach $u (@rv) {
		if ($self->indexof($u, @ulist) < 0) {
			$u->{'noedit'} = 1;
			}
		}
	return @rv;
	}
else {
	return @ulist;
	}
}

# list_allowed_groups(&access, &allgroups)
# Returns a list of groups to whom access is allowed
sub list_allowed_groups
{
	my $self = shift;

local %access = %{$_[0]};
local @glist = @{$_[1]};
if ($access{'gedit_mode'} == 1) {
	@glist = ();
	}
elsif ($access{'gedit_mode'} == 2) {
	local %cang;
	map { $cang{$_}++ } split(/\s+/, $access{'gedit'});
	@glist = grep { $cang{$_->{'group'}} } @glist;
	}
elsif ($access{'gedit_mode'} == 3) {
	local %cannotg;
	map { $cannotg{$_}++ } split(/\s+/, $access{'gedit'});
	@glist = grep { !$cannotg{$_->{'group'}} } @glist;
	}
elsif ($access{'gedit_mode'} == 4) {
	@glist = grep {
		(!$access{'gedit'} || $_->{'gid'} >= $access{'gedit'}) &&
		(!$access{'gedit2'} || $_->{'gid'} <= $access{'gedit2'})
			} @glist;
	}
if ($access{'view'}) {
	# Include non-editable groups in results
	local @rv = @{$_[1]};
	local $g;
	foreach $g (@rv) {
		if ($self->indexof($g, @glist) < 0) {
			$g->{'noedit'} = 1;
			}
		}
	return @rv;
	}
else {
	return @glist;
	}
}

# batch_start()
# Tells the create/modify/delete functions to only update files in memory,
# not on disk.
sub batch_start
{
	my $self = shift;

$self->{'batch_mode'} = 1;
}

# batch_end()
# Flushes any user file changes
sub batch_end
{
	my $self = shift;

$self->{'batch_mode'} = 0;
$self->flush_file_lines();
$self->refresh_nscd();
}

sub mkuid
{
	my $self = shift;

#################################################################
#### 
#### Assumptions:
#### 
#### This subroutine assumes the usernames are standardized
#### using the format of 7 characters with 3 letters followed
#### by 4 digits, or 4 letters followed by 3 digits.  If
#### uppercase letters are used in the username, they will be
#### converted to lowercase and this subroutine will generate
#### a UID number identical to the usernames lowercase
#### equivalent. 
#### 
#### 3 letters, 4 digits   Lowest possible UID (aaa0000) =   1,000,000
#### 3 letters, 4 digits Hightest possible UID (zzz9999) = 176,759,999
#### 
#### 4 letters, 3 digits   Lowest possible UID (aaaa000) = 176,760,000
#### 4 letters, 3 digits Hightest possible UID (zzzz999) = 633,735,999
#### 
#################################################################
    my ${num_let} = 0;
    foreach (split(//,$_[0])) {
      ++${num_let} if ( m/[a-z]/i );
    }
    if ( length($_[0]) ne 7 ) {
        print "ERROR: Number of characters in username $_[0] is not equal to 7\n";
        return -1;
    }
    if ( ${num_let} ne 3 && ${num_let} ne 4 ) {
        print "ERROR: Number of letters in username $_[0] is not equal to 3 or 4\n";
        return -1;
    }
    my ${mkuid_type} = 10 ** ( 7 - ${num_let} );
    my ${lowlimit} = 1000000;
    my %letters;
    my ${icnt} = -1;
    my ${lowuid};
    ${lowuid} = ( 26 ** ( ${num_let} - 1 ) * ${lowlimit}/100 ) + ${lowlimit};
    ${lowuid} = ${lowlimit} if ( ${num_let} eq 3 );
    my ${base} = 26;

#################################################################
#### 
#### Establish an associative array containing all the
#### letters of the alphabet and assign a numeric value
#### to each letter from 1 - 26.
#### 
#################################################################
    $letters{'a'} = ++${icnt};
    $letters{'b'} = ++${icnt};
    $letters{'c'} = ++${icnt};
    $letters{'d'} = ++${icnt};
    $letters{'e'} = ++${icnt};
    $letters{'f'} = ++${icnt};
    $letters{'g'} = ++${icnt};
    $letters{'h'} = ++${icnt};
    $letters{'i'} = ++${icnt};
    $letters{'j'} = ++${icnt};
    $letters{'k'} = ++${icnt};
    $letters{'l'} = ++${icnt};
    $letters{'m'} = ++${icnt};
    $letters{'n'} = ++${icnt};
    $letters{'o'} = ++${icnt};
    $letters{'p'} = ++${icnt};
    $letters{'q'} = ++${icnt};
    $letters{'r'} = ++${icnt};
    $letters{'s'} = ++${icnt};
    $letters{'t'} = ++${icnt};
    $letters{'u'} = ++${icnt};
    $letters{'v'} = ++${icnt};
    $letters{'w'} = ++${icnt};
    $letters{'x'} = ++${icnt};
    $letters{'y'} = ++${icnt};
    $letters{'z'} = ++${icnt};

#################################################################
#### 
#### Initialize variables to be use while calculating the UID
#### number associated with the login name.
#### 
#### nvalue is used to store numeric characters that occurs
####     in the login name
#### ecnt is used to keep track of the base 26 exponent for 
####     each letter character that occurs in the login name
#### subtot is the sum of the calculated value for each
####     character position in the login name
#### mult is the total of the 26 ** ecnt at each iteration of 
####     the loop
#### 
#################################################################
    my ${kstring} = '';
    my ${nvalue} = '';
    my ${lvalue} = 0;
    my ${ecnt} = 0;
    my ${subtot} = 0;
    my ${tot} = 0;
    my ${mult} = 0;
#################################################################
#### 
#### each character position of the login name is split out
#### and used as an iteration of the foreach loop
#### 
#################################################################

    foreach (split(//,$_[0])) {

#################################################################
#### 
#### If the current character of the login name is a letter,
#### convert it to lower case, and obtain it's numeric value
#### from the associative array of letters, otherwise, if the
#### current character is a number, append the number to the
#### end of a buffer and save it for later processing.
#### 
#################################################################
      if ( m/[a-z]/i ) {
        $kstring = "\L${_}";
        ${lvalue} = ${letters{${kstring}}};
      } else {
        ${lvalue} = 0;
        ${nvalue} = "${nvalue}${_}";
      }
#################################################################
#### Calculate the multiplier for a base 26 calculation using
#### each iteration through the foreach loop as an increment
#### of the exponent.  The base 26 exponent starting at 0.
#################################################################

      ${mult} = ${base} ** ${ecnt};

#################################################################
#### 
#### Multiply the numeric value of the current character by
#### the multiplier and add this result to a running subtotal
#### of all characters of the login name.
#### 
#################################################################
      ${subtot} = ${subtot} + ( ${lvalue} * ${mult} );

#################################################################
#### 
#### Increment the base 26 exponent by one before iterating for
#### the next character of the login name.
#### 
#################################################################
      ++${ecnt}
    }

#################################################################
#### 
#### After all characters of the login name have be processed,
#### multiply the result by 1,000.  This is done because the 
#### username standard is 3 letters followed by 4 digits.  So
#### each 3 letter combination can have 1,000 possible combinations.
#### Then add the numeric values saved and any value to use
#### as the lowest UID number allowed through this calculated method.
#### 
#################################################################

    ${tot} = ( ${subtot} * ${mkuid_type} ) + int(${nvalue}) + ${lowuid};

#################################################################
#### 
#### Return the calculated UID number as the result of this
#### subroutine.
#### 
#################################################################

    return ${tot};
}

################################################################
sub berkeley_cksum {
	my $self = shift;

    my($crc) = my($len) = 0;
    my($buf,$num,$i);
    my($buflen) = 4096; # buffer is "4k", you can up it if you want...

    $buf = $_[0];
    $num = length($buf);

    $len += $num;
    foreach ( unpack("C*", $buf) ) {
        $crc |= 0x10000 if ( $crc & 1 ); # get ready for rotating the 1 below
        $crc = (($crc>>1)+$_) & 0xffff; # keep to 16-bit
    }
    return sprintf("%lu",${crc});;
}

# users_table(&users, [form], [no-last], [no-boxes], [&otherlinks])
# Prints a table listing full user details, with checkboxes and buttons to
# delete or disable multiple at once.
sub users_table
{
	my $self = shift;

local ($users, $formno, $nolast, $noboxes, $links) = @_;

local (@ginfo, %gidgrp);
$self->my_setgrent();
while(@ginfo = $self->my_getgrent()) {
	$gidgrp{$ginfo[2]} = $ginfo[0];
	}
$self->my_endgrent();
local ($anyedit) = grep { !$_->{'noedit'} } @$users;
$anyedit = 0 if ($noboxes);
local $lshow = !$nolast && $self->{'config_hash'}->{'last_show'};

local $buttons;
$buttons .= $self->ui_submit($self->{'text_hash'}->{'index_mass'}, "delete") if ($self->{'access_hash'}->{'udelete'});
$buttons .= $self->ui_submit($self->{'text_hash'}->{'index_mass2'}, "disable");
$buttons .= $self->ui_submit($self->{'text_hash'}->{'index_mass3'}, "enable");
$buttons .= "<br>" if ($buttons);
local @linksrow;
if ($anyedit) {
	print $self->ui_form_start("mass_delete_user.cgi", "post");
	print $buttons;
	push(@linksrow, $self->select_all_link("d", $_[1]),
			$self->select_invert_link("d", $_[1]));
	}
push(@linksrow, @$links);
print $self->ui_links_row(\@linksrow);

local @tds = $anyedit ? ( "width=5" ) : ( );
push(@tds, "width=15%", "width=10%");
print $self->ui_columns_start([
	$anyedit ? ( "" ) : ( ),
	$self->{'text_hash'}->{'user'},
	$self->{'text_hash'}->{'uid'},
	$self->{'text_hash'}->{'gid'},
	$self->{'text_hash'}->{'real'},
	$self->{'text_hash'}->{'home'},
	$self->{'text_hash'}->{'shell'},
	$lshow ? ( $self->{'text_hash'}->{'lastlogin'} ) : ( )
	], 100, 0, \@tds);
local %llogin;
if ($lshow) {
	local $l;
	foreach $l ($self->list_last_logins()) {
		$llogin{$l->[0]} ||= $l->[3];
		}
	}
local $u;
foreach $u (@$users) {
	$u->{'real'} =~ s/,.*$// if ($self->{'config_hash'}->{'extra_real'});
	local @cols;
	push(@cols, "") if ($anyedit && $u->{'noedit'});
	push(@cols, $self->user_link($u));
	push(@cols, $u->{'uid'});
	push(@cols, $gidgrp{$u->{'gid'}}||$u->{'gid'});
	push(@cols, $u->{'real'});
	push(@cols, $u->{'home'});
	push(@cols, $u->{'shell'});
	push(@cols, $llogin{$u->{'user'}}) if ($lshow);
	if ($u->{'noedit'}) {
		print $self->ui_columns_row(\@cols, \@tds);
		}
	else {
		print $self->ui_checked_columns_row(\@cols, \@tds, "d", $u->{'user'});
		}
	}
print $self->ui_columns_end();
print $self->ui_links_row(\@linksrow);
if ($anyedit) {
	print $buttons;
	print $self->ui_form_end();
	}
}

# groups_table(&groups, [form], [no-buttons], [&otherlinks])
sub groups_table
{
	my $self = shift;

local ($groups, $formno, $noboxes, $links) = @_;

local ($anyedit) = grep { !$_->{'noedit'} } @$groups;
$anyedit = 0 if ($noboxes);

local @linksrow;
if ($anyedit && $self->{'access_hash'}->{'gdelete'}) {
	print $self->ui_form_start("mass_delete_group.cgi", "post");
	print $self->ui_submit($self->{'text_hash'}->{'index_gmass'}, "delete"),"<br>\n";
	push(@linksrow, $self->select_all_link("d", $formno),
			$self->select_invert_link("d", $formno) );
	}
push(@linksrow, @$links);
print $self->ui_links_row(\@linksrow);

local @tds = $anyedit ? ( "width=5" ) : ( );
push(@tds, "width=15%", "width=10%");
print $self->ui_columns_start([
	$anyedit ? ( "" ) : ( ),
	$self->{'text_hash'}->{'gedit_group'},
	$self->{'text_hash'}->{'gedit_gid'},
	$self->{'text_hash'}->{'gedit_members'} ], 100, 0, \@tds);
local $g;
foreach $g (@$groups) {
	local $members = join(" ", split(/,/, $g->{'members'}));
	local @cols;
	push(@cols, "") if ($anyedit && $g->{'noedit'});
	push(@cols, $self->group_link($g));
	push(@cols, $g->{'gid'});
	push(@cols, $self->ifblank($members));
	if ($g->{'noedit'} || !$self->{'access_hash'}->{'gdelete'}) {
		print $self->ui_columns_row(\@cols, \@tds);
		}
	else {
		print $self->ui_checked_columns_row(\@cols, \@tds, "d",$g->{'group'});
		}
	}
print $self->ui_columns_end();
print $self->ui_links_row(\@linksrow);
if ($anyedit && $self->{'access_hash'}->{'gdelete'}) {
	print $self->ui_submit($self->{'text_hash'}->{'index_gmass'}, "delete"),"<br>\n";
	print $self->ui_form_end();
	}
}

sub ifblank
{
	my $self = shift;

return $_[0] ? $self->html_escape($_[0]) : "&nbsp;";
}

# date_input(day, month, year, prefix)
sub date_input
{
	my $self = shift;

print "<input name=$_[3]d size=3 value='$_[0]'>";
print "/<select name=$_[3]m>\n";
local $m;
foreach $m (1..12) {
	printf "<option value=%d %s>%s\n",
		$m, $_[1] eq $m ? 'selected' : '', $self->{'text_hash'}->{"smonth_$m"};
	}
print "</select>";
print "/<input name=$_[3]y size=5 value='$_[2]'>";
print $self->date_chooser_button("$_[3]d", "$_[3]m", "$_[3]y");
}

# list_last_logins([user], [max])
# Returns a list of array references, each containing the details of a login
sub list_last_logins
{
	my $self = shift;

local @rv;
$self->open_last_command(LAST, $_[0]);
while(@{$self->{'last_array'}} = $self->read_last_line(LAST)) {
	push(@rv, [ @{$self->{'last_array'}} ]);
	if ($_[1] && scalar(@rv) >= $_[1]) {
		last;	# reached max
		}
	}
close(LAST);
return @rv;
}

# user_link(&user)
sub user_link
{
	my $self = shift;

if ($_[0]->{'pass'} =~ /^\Q$self->{'disable_string'}\E/) {
	$self->{'dis'} = "<i>".$self->html_escape($_[0]->{'user'})."</i>";
	}
else {
	$self->{'dis'} = $self->html_escape($_[0]->{'user'});
	}
if ($_[0]->{'noedit'}) {
	return $self->{'dis'};
	}
elsif ($_[0]->{'dn'}) {
	return "<a href='edit_user.cgi?dn=".$self->urlize($_[0]->{'dn'})."'>".
	       "$self->{'dis'}</a>";
	}
else {
	return "<a href='edit_user.cgi?num=$_[0]->{'num'}'>".
	       "$self->{'dis'}</a>";
	}
}

# group_link(&group)
sub group_link
{
	my $self = shift;

if ($_[0]->{'noedit'}) {
	return $self->html_escape($_[0]->{'group'});
	}
elsif ($_[0]->{'dn'}) {
	return "<a href='edit_group.cgi?dn=".$self->urlize($_[0]->{'dn'})."'>".
	       $self->html_escape($_[0]->{'group'})."</a>";
	}
else {
	return "<a href='edit_group.cgi?num=$_[0]->{'num'}'>".
	       $self->html_escape($_[0]->{'group'})."</a>";
	}
}

# sort_users(&users, mode)
# Sorts a list of users, and returns the results
sub sort_users
{
	my $self = shift;

local ($users, $mode) = @_;
local @ulist = @$users;
if ($mode == 1) {
	@ulist = sort { $a->{'user'} cmp $b->{'user'} } @ulist;
	}
elsif ($mode == 2) {
	@ulist = sort { lc($a->{'real'}) cmp lc($b->{'real'}) } @ulist;
	}
elsif ($mode == 3) {
	@ulist = sort { @wa = split(/\s+/, $a->{'real'});
			@wb = split(/\s+/, $b->{'real'});
			lc($wa[@wa-1]) cmp lc($wb[@wb-1]) } @ulist;
	}
elsif ($mode == 4) {
	@ulist = sort { $a->{'shell'} cmp $b->{'shell'} } @ulist;
	}
elsif ($mode == 5) {
	@ulist = sort { $a->{'uid'} <=> $b->{'uid'} } @ulist;
	}
elsif ($mode == 6) {
	@ulist = sort { $a->{'home'} cmp $b->{'home'} } @ulist;
	}
return @ulist;
}

# sort_groups(&groups, mode)
sub sort_groups
{
	my $self = shift;

local ($groups, $mode) = @_;
local @glist = @$groups;
if ($mode == 5) {
	@glist = sort { $a->{'gid'} <=> $b->{'gid'} } @glist;
	}
elsif ($mode == 1) {
	@glist = sort { $a->{'group'} cmp $b->{'group'} } @glist;
	}
return @glist;
}

# create_home_directory(&user, [real-dir])
# Creates and chmod's the home directory for a user, or calls error on failure
sub create_home_directory
{
	my $self = shift;

local ($user, $home) = @_;
$home ||= $user->{'home'};
$self->lock_file($home);
$self->make_dir($home, oct($self->{'config_hash'}->{'homedir_perms'}), 1) ||
	$self->error($self->text('usave_emkdir', $!));
$self->set_ownership_permissions($user->{'uid'}, $user->{'gid'},
			   oct($self->{'config_hash'}->{'homedir_perms'}), $home) ||
	$self->error($self->text('usave_echmod', $!));
$self->unlock_file($home);
}

# delete_home_directory(&user)
# Delete's some users home directory
sub delete_home_directory
{
	my $self = shift;

local ($user) = @_;
if ($user->{'home'} && -d $user->{'home'}) {
	local $realhome = $self->resolve_links($user->{'home'});
	local $qhome = quotemeta($realhome);
	if ($self->{'config_hash'}->{'delete_only'}) {
		$self->system_logged("find $qhome ! -type d -user $user->{'uid'} | xargs rm -f >/dev/null 2>&1");
		$self->system_logged("find $qhome -type d -user $user->{'uid'} | xargs rmdir >/dev/null 2>&1");
		$self->unlink_file($realhome);
		}
	else {
		$self->system_logged("rm -rf $qhome >/dev/null 2>&1");
		}
	unlink($user->{'home'});	# in case of links
	}
}


1;

