#!/usr/bin/perl
package virtual_server::Check;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'virtual_server::Check');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# check.cgi
# Make sure that all enabled features are valid, and check the status of quotas

$self->master_admin() || $self->error($self->{'text_hash'}->{'check_ecannot'});

$self->read_file("$self->{'module_config_directory'}/last-config", \%{$self->{'lastconfig_hash'}});

($self->{'module_name'} eq "virtual_server") ? $self->{'clink'} = "../config.cgi?$self->{'module_name'}" : 
	$self->{'clink'} = "../config.cgi?virtual_server&conf_file=config_websites.info&ln=1&title=config_s_1";
	
# Make sure networking is supported
if (!$self->foreign_check("net")) {
	$self->foreign_require("net", "net-lib.pl");
	if (!$main::net_object->UNIVERSAL::can("boot_interfaces")) {
		$self->check_error($self->text('index_enet'));
		}
	}

if ($self->{'config_hash'}->{'dns'}) {
	# Make sure BIND is installed
	$self->foreign_installed("bind8", 1) == 2 ||
		$self->check_error($self->text('index_ebind', "/bind8/", $self->{'clink'}));
	}

if ($self->{'config_hash'}->{'mail'}) {
	if ($self->{'config_hash'}->{'mail_system'} == 3) {
		# Work out which mail server we have
		if ($self->postfix_installed()) {
			$self->{'config_hash'}->{'mail_system'} = 0;
			}
		elsif ($self->qmail_vpopmail_installed()) {
			$self->{'config_hash'}->{'mail_system'} = 5;
			}
		elsif ($self->qmail_ldap_installed()) {
			$self->{'config_hash'}->{'mail_system'} = 4;
			}
		elsif ($self->qmail_installed()) {
			$self->{'config_hash'}->{'mail_system'} = 2;
			}
		elsif ($self->sendmail_installed()) {
			$self->{'config_hash'}->{'mail_system'} = 1;
			}
		else {
			$self->check_error($self->text('index_email'));
			}
#		print $self->text('check_detected', &mail_system_name()),"<p>\n";
		$self->save_module_config();
		}
	if ($self->{'config_hash'}->{'mail_system'} == 1) {
		# Make sure sendmail is installed
		if (!$self->sendmail_installed()) {
			$self->check_error($self->text('index_esendmail', '/sendmail/', $self->{'clink'}));
			}
		# Check that aliases and virtusers are configured
		$self->require_mail();
		@{$self->{'sendmail_afiles'}} ||
			$self->check_error($self->text('index_esaliases', '/sendmail/'));
		$self->{'sendmail_vdbm'} ||
			$self->check_error($self->text('index_esvirts', '/sendmail/'));
		if ($self->{'config_hash'}->{'generics'}) {
			$self->{'sendmail_gdbm'} ||
		    		$self->check_error($self->text('index_esgens',
					     '/sendmail/', $self->{'clink'}));
			}
		}
	elsif ($self->{'config_hash'}->{'mail_system'} == 0) {
		# Make sure postfix is installed
		if (!$self->postfix_installed()) {
			$self->check_error($self->text('index_epostfix', '/postfix/', $self->{'clink'}));
			}
		# Check that aliases and virtusers are configured
		$self->require_mail();
		@{$self->{'postfix_afiles'}} ||
			$self->check_error($self->text('index_epaliases', '/postfix/'));
		$self->{'virtual_maps'} =~ /^[a-z0-9]+:\//i ||
			$self->check_error($self->text('index_epvirts', '/postfix/'));
		if ($self->{'config_hash'}->{'generics'}) {
			$self->{'canonical_maps'} ||
				$self->check_error($self->text('index_epgens',
					    '/postfix/', $self->{'clink'}));
			}
		if ($self->{'config_hash'}->{'append_style'} == 6) {
			$self->check_error($self->text('check_estyle6', $self->{'clink'}));
			}
		}
	elsif ($self->{'config_hash'}->{'mail_system'} == 2) {
		# Make sure qmail is installed
		if (!$self->qmail_installed()) {
			$self->check_error($self->text('index_eqmail', '/qmailadmin/', $self->{'clink'}));
			}
		if ($self->{'config_hash'}->{'generics'}) {
			$self->check_error($self->text('index_eqgens', $self->{'clink'}));
			}
		if ($self->{'config_hash'}->{'append_style'} == 6) {
			$self->check_error($self->text('check_qmailmode6', $self->{'clink'}));
			}
		}
	elsif ($self->{'config_hash'}->{'mail_system'} == 4) {
		# Make sure qmail with LDAP is installed
		if (!$self->qmail_ldap_installed()) {
			$self->check_error($self->text('index_eqmailldap', '/qmailadmin/', $self->{'clink'}));
			}
		if ($self->{'config_hash'}->{'generics'}) {
			$self->check_error($self->text('index_eqgens', $self->{'clink'}));
			}
		if (!gethostbyname($self->{'config_hash'}->{'ldap_host'})) {
			$self->check_error($self->text('index_eqmailhost', $self->{'clink'}));
			}
		if (!$self->{'config_hash'}->{'ldap_base'}) {
			$self->check_error($self->text('index_eqmailbase', $self->{'clink'}));
			}
		 $self->{'lerr'} = $self->connect_qmail_ldap(1);
		if (!ref($self->{'lerr'})) {
			$self->check_error($self->text('index_eqmailconn', $self->{'lerr'}, $self->{'clink'}));
			}
		}
	elsif ($self->{'config_hash'}->{'mail_system'} == 5) {
		# Make sure qmail with VPOPMail is installed
		if (!$self->qmail_vpopmail_installed()) {
			$self->check_error($self->text('index_evpopmail', '/qmailadmin/', $self->{'clink'}));
			}
		if ($self->{'config_hash'}->{'generics'}) {
			$self->check_error($self->text('index_eqgens', $self->{'clink'}));
			}
		}
	}

if ($self->{'config_hash'}->{'mailman'}) {
	# Make sure Mailman is installed
	if (!$self->mailman_installed()) {
		$self->check_error($self->text('index_emailman', '/mailman/', $self->{'clink'}));
		}
	if ($self->{'config_hash'}->{'mail_system'} != 0) {
		$self->check_error($self->text('index_emailman2'));
		}
	}
	
if ($self->{'config_hash'}->{'web'}) {
	# Make sure Apache is installed
	$self->foreign_installed("apache", 1) == 2 ||
		$self->check_error($self->text('index_eapache', "/apache/", $self->{'clink'}));
	}

if ($self->{'config_hash'}->{'webalizer'}) {
	# Make sure Webalizer is installed, and that global directives are OK
	# XXX is the path checking really needed?
	$self->{'config_hash'}->{'web'} || $self->check_error($self->text('check_edepwebalizer', $self->{'clink'}));
	$self->foreign_installed("webalizer", 1) == 2 ||
		$self->check_error($self->text('index_ewebalizer', "/webalizer/", $self->{'clink'}));
	$self->foreign_require("webalizer", "webalizer-lib.pl");
	$self->{'conf'} = $main::webalizer_object->get_config();
	$self->{'current'} = $main::webalizer_object->find_value("IncrementalName", $self->{'conf'});
	$self->{'history'} = $main::webalizer_object->find_value("HistoryName", $self->{'conf'});
	if ($self->{'current'} =~ /^\//) {
		$self->check_error($self->text('check_current', "/webalizer/"));
		}
	elsif ($self->{'history'} =~ /^\//) {
		$self->check_error($self->text('check_history', "/webalizer/"));
		}
	}

if ($self->{'config_hash'}->{'ssl'}) {
	# Make sure openssl is installed, that Apache supports mod_ssl,
	# and that port 443 is in use
	$self->{'config_hash'}->{'web'} || $self->check_error($self->text('check_edepssl', $self->{'clink'}));
	$self->has_command("openssl") ||
	    $self->check_error($self->text('index_eopenssl', "<tt>openssl</tt>", $self->{'clink'}));

	$self->require_apache();
	 $self->{'conf'} = $main::apache_object->get_config();
	 @{$self->{'loads_array'}} = $main::apache_object->find_directive_struct("LoadModule", $self->{'conf'});
	 ($l, $self->{'hasmod'});
	foreach $l (@{$self->{'loads_array'}}) {
		$self->{'hasmod'}++ if ($l->{'words'}->[1] =~ /mod_ssl/);
		}
	 ($self->{'aver'}, $self->{'amods'}) = $main::apache_object->httpd_info();
	$self->{'hasmod'}++ if ($self->indexof("mod_ssl", @{$self->{'amods'}}) >= 0);
	$self->{'hasmod'}++ if ($main::apache_object->{'httpd_modules_hash'}->{'mod_ssl'});
	$self->{'hasmod'} ||
	    $self->check_error($self->text('index_emodssl', "<tt>mod_ssl</tt>", $self->{'clink'}));

	 @{$self->{'listens_array'}} = $main::apache_object->find_directive_struct("Listen", $self->{'conf'});
	foreach $l (@{$self->{'listens_array'}}) {
		$self->{'haslisten'}++ if ($l->{'words'}->[0] =~ /^(\S+:)?$self->{'default_web_sslport'}$/);
		}
	 @{$self->{'ports_array'}} = $main::apache_object->find_directive_struct("Port", $self->{'conf'});
	foreach $l (@{$self->{'ports_array'}}) {
		$self->{'haslisten'}++ if ($l->{'words'}->[0] == $self->{'default_web_sslport'});
		}
	$self->{'haslisten'} ||
	    $self->check_error($self->text('index_emodssl2', $self->{'default_web_sslport'}, $self->{'clink'}));
	}

if ($self->{'config_hash'}->{'mysql'}) {
	# Make sure MySQL is installed
	$self->foreign_installed("mysql", 1) == 2 ||
		$self->check_error($self->text('index_emysql', "/mysql/", $self->{'clink'}));
	}

if ($self->{'config_hash'}->{'postgres'}) {
	# Make sure PostgreSQL is installed
	$self->foreign_installed("postgresql", 1) == 2 ||
		$self->check_error($self->text('index_epostgres', "/postgresql/", $self->{'clink'}));
	}

if ($self->{'config_hash'}->{'ftp'}) {
	if ($self->{'config_hash'}->{'ftp_system'} == 0) {
		# Work out which mail server we have
		if ($self->proftpd_installed()) {
			$self->{'config_hash'}->{'ftp_system'} = 1;
			}
		elsif ($self->vsftpd_installed()) {
			$self->{'config_hash'}->{'ftp_system'} = 2;
			}
		else {
			$self->check_error($self->text('index_eftp'));
			}
		$self->save_module_config();
		}
	if ($self->{'config_hash'}->{'ftp_system'} == 1) {
		# Make sure ProFTPd is installed, and that the ftp user exists
		$self->foreign_installed("proftpd", 1) == 2 || 								$self->check_error($self->text('index_eproftpd', "/proftpd/", $self->{'clink'}));
 		$self->{'err'} = $self->check_proftpd_template();
		$self->{'err'} && $self->check_error($self->text('check_proftpd', $self->{'err'}));
		}
	}

if ($self->{'config_hash'}->{'logrotate'}) {
	# Make sure logrotate is installed
	$self->foreign_installed("logrotate", 1) == 2 ||
		$self->check_error($self->text('index_elogrotate', "/logrotate/", $self->{'clink'}));
	}

# Check all plugins
foreach $p (@{$self->{'plugins_array'}}) {
	$self->{'err'} = $self->plugin_call($p, "feature_check");
	if ($self->{'err'}) {
		$self->check_error($self->{'err'});
		}
	else {
		$self->{'pname'} = $self->plugin_call($p, "feature_name");
		}
	}

if (!$self->{'config_hash'}->{'iface'}) {
	# Work out the network interface automatically
	$self->foreign_require("net", "net-lib.pl");
	foreach $a ($main::net_object->active_interfaces()) {
		if ($a->{'up'} && $a->{'virtual'} eq '' &&
		    $main::net_object->iface_type($a->{'name'}) =~ /ethernet/i) {
			$self->{'config_hash'}->{'iface'} = $a->{'fullname'};
			last;
			}
		}
	if (!$self->{'config_hash'}->{'iface'}) {
		$self->check_error($self->text('index_eiface', $self->{'clink'}));
		}
	$self->write_file("$self->{'module_config_directory'}/config", \%{$self->{'config_hash'}});
	}

# Make sure  group exists
if ($self->{'config_hash'}->{'localgroup'} && !defined(getgrnam($self->{'config_hash'}->{'localgroup'}))) {
	$self->check_error($self->text('index_elocal', "<tt>$self->{'config_hash'}->{'localgroup'}</tt>", $self->{'clink'}));
	}

$self->{'config_hash'}->{'home_quotas'} = '';
$self->{'config_hash'}->{'mail_quotas'} = '';
$self->{'config_hash'}->{'group_quotas'} = '';
if ($self->{'config_hash'}->{'quotas'}) {
	# Make sure quotas are enabled, and work out where they are needed
	$self->require_useradmin();
	if (!$self->{'home_base'}) {
		$self->check_error($self->text('index_ehomebase'));
		}
	else {
		$self->foreign_require("mount", "mount-lib.pl");
		$self->{'mail_base'} = $self->simplify_path($self->resolve_links(
				$self->mail_system_base()));
		($self->{'home_mtab'}, $self->{'home_fstab'}) = $self->mount_point($self->{'home_base'});
		($self->{'mail_mtab'}, $self->{'mail_fstab'}) = $self->mount_point($self->{'mail_base'});
		if (!$self->{'home_mtab'}) {
			$self->check_error($self->text('index_ehomemtab',"<tt>$self->{'home_base'}</tt>"));
			}
		elsif (!$self->{'mail_mtab'}) {
			$self->check_error($self->text('index_emailmtab',"<tt>$self->{'mail_base'}</tt>"));
			}
		else {
			# Check if quotas are enabled for home filesystem
			 $self->{'nohome'};
			if (!($self->{'home_mtab'}->[4] = $main::quota_object->quota_can(
				$self->{'home_mtab'}, $self->{'home_fstab'}))%2 ||
			    !$main::quota_object->quota_now($self->{'home_mtab'}, $self->{'home_fstab'})) {
				$self->{'nohome'}++;
				}
			else {
				if ($self->{'home_mtab'}->[4] >= 2) {
					# Group quotas are active too
					$self->{'config_hash'}->{'group_quotas'} = 1;
					}
				}

			if ($self->{'home_mtab'}->[0] eq $self->{'mail_mtab'}->[0]) {
				# Home and mail are the same filesystem
				if ($self->{'nohome'}) {
					# Neither are enabled
					$self->{'qerr'} = $self->text('index_equota2',
					    "<tt>$self->{'home_mtab'}->[0]</tt>",
					    "<tt>$self->{'home_base'}</tt>",
					    "<tt>$self->{'mail_base'}</tt>");
					}
				else {
					# Both are enabled
					$self->{'config_hash'}->{'home_quotas'} =
						$self->{'home_mtab'}->[0];
					$self->{'config_hash'}->{'mail_quotas'} =
						$self->{'home_mtab'}->[0];
					}
				}
			else {
				# Different .. so check mail too
				 $self->{'nomail'};
				if (!($self->{'mail_mtab'}->[4] = $main::quota_object->quota_can(
					$self->{'mail_mtab'}, $self->{'mail_fstab'})) ||
				    !$main::quota_object->quota_now($self->{'mail_mtab'},
						       $self->{'mail_fstab'})) {
					$self->{'nomail'}++;
					}
				if ($self->{'nohome'}) {
					$self->{'qerr'} = $self->text('index_equota3',
					    "<tt>$self->{'home_mtab'}->[0]</tt>",
					    "<tt>$self->{'home_base'}</tt>");
					}
				else {
					$self->{'config_hash'}->{'home_quotas'} =
						$self->{'home_mtab'}->[0];
					}
				if ($self->{'nomail'}) {
					$self->{'qerr'} = $self->text('index_equota4',
					    "<tt>$self->{'mail_mtab'}->[0]</tt>",
					    "<tt>$self->{'mail_base'}</tt>");
					}
				else {
					$self->{'config_hash'}->{'mail_quotas'} =
						$self->{'mail_mtab'}->[0];
					}
				}
			}
		}
		if (!$self->{'qerr'} && !$self->{'config_hash'}->{'group_quotas'}) {
			$self->{'qerr'} = $self->{'text_hash'}->{'check_nogroup'};
		}
	}
if ($self->{'qerr'}) {
	$self->check_error($self->{'qerr'});
	}

# Check for FTP shells in /etc/shells
open(SHELLS, "/etc/shells");
while(<SHELLS>) {
	s/\r|\n//g;
	s/#.*$//;
	$self->{'shells_hash'}->{$_}++;
	}
close(SHELLS);
if ($self->{'shells_hash'}->{$self->{'config_hash'}->{'shell'}}) {
	$self->check_error($self->text('check_eshell', "<tt>$self->{'config_hash'}->{'shell'}</tt>",
		    "<tt>/etc/shells</tt>"));
	}
if (!$self->{'shells_hash'}->{$self->{'config_hash'}->{'ftp_shell'}}) {
	$self->check_error($self->text('check_eftpshell', "<tt>$self->{'config_hash'}->{'ftp_shell'}</tt>",
		    "<tt>/etc/shells</tt>"));
	}

# Check for problem module config settings
if ($self->{'config_hash'}->{'all_namevirtual'} && $self->{'config_hash'}->{'dns_ip'}) {
	$self->check_error($self->text('check_enamevirt', $self->{'clink'}));
	}

# Make sure LDAP module is set up, if selected
if ($self->{'config_hash'}->{'ldap'}) {
	$self->{'ldap'} = &ldap_useradmin::ldap_connect(1);
	if (!ref($self->{'ldap'})) {
		$self->check_error($self->text('check_eldap', $self->{'ldap'}, $self->{'clink'},
				   "../ldap-useradmin/"));
		}
	else {
		$self->require_useradmin();
		if (!defined(&ldap_useradmin::list_users)) {
			$self->check_error($self->text('check_eldap2', $self->{'clink'}, 1.164));
			}
		}
	}

$self->{'config_hash'}->{'last_check'} = time()+1;
$self->{'config_hash'}->{'disable'} =~ s/user/unix/g;	# changed since last release
$self->lock_file($self->{'module_config_file'});
$self->write_file($self->{'module_config_file'}, \%{$self->{'config_hash'}});
$self->unlock_file($self->{'module_config_file'});
#$self->lock_file("$self->{'module_config_directory'}/last-config");
$self->write_file("$self->{'module_config_directory'}/last-config", \%{$self->{'config_hash'}});
#$self->unlock_file("$self->{'module_config_directory'}/last-config");

#print "</ul>\n";

# See if any options effecting Webmin users have changed
$self->{'webminchanged'} = 0;
foreach $k (keys %{$self->{'config_hash'}}) {
	if ($k =~ /^avail_/ || $k eq 'leave_acl' ||
	    $self->indexof($k, @{$self->{'features_array'}}) >= 0) {
		$self->{'webminchanged'}++ if ($self->{'config_hash'}->{$k} != $self->{'lastconfig_hash'}->{$k});
		}
	}

if ($self->{'webminchanged'}) {
		# Update all Webmin users
		$self->modify_all_webmin();
	}
$self->webmin_log("check");

1;
1;
		return $self;
}

sub check_error
{
	my $self = shift;

print "<p class=warn>$_[0]</p>\n";
}

# mount_point(dir)
# Returns both the mtab and fstab details for the parent mount for a directory
sub mount_point
{
	my $self = shift;

local $dir = $self->resolve_links($_[0]);
local @mounts = $main::mount_object->list_mounts();
local @mounted = $main::mount_object->list_mounted();
@mounts = @mounted if (!@mounts);
foreach $m (sort { length($b->[0]) <=> length($a->[0]) } @mounted) {
	if ($dir eq $m->[0] || $m->[0] eq "/" ||
	    substr($dir, 0, length($m->[0])+1) eq "$m->[0]/") {
		local ($m2) = grep { $_->[0] eq $m->[0] } @mounts;
		if ($m2) {
			return ($m, $m2);
			}
		}
	}
$self->error("Failed to find mount point for $dir");
}


1;

