#!/usr/bin/perl
package virtual_server::Featuredir;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'virtual_server::Featuredir');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for managing a domain's home directory

1;

1;
		return $self;
}

# setup_dir(&domain)
# Creates the home directory
sub setup_dir
{
	my $self = shift;

local $tmpl = $self->get_template($_[0]->{'template'});
$self->require_useradmin();
local $qh = quotemeta($_[0]->{'home'});

# Get Unix user, either for this domain or its parent
local $uinfo;
if ($_[0]->{'unix'} || $_[0]->{'parent'}) {
	local @users = $self->list_all_users();
	($uinfo) = grep { $_->{'user'} eq $_[0]->{'user'} } @users;
	}
if ($_[0]->{'unix'} && !$uinfo) {
	# If we are going to have a Unix user but none has been created
	# yet, fake his details here for use in chowning and skel copying
	$uinfo ||= { 'uid' => $_[0]->{'uid'},
		     'gid' => $_[0]->{'ugid'},
		     'shell' => '/bin/sh',
		     'group' => $_[0]->{'group'} || $_[0]->{'ugroup'} };
	}

# Create and populate home directory
do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_home'})};
$self->system_logged("mkdir -p $qh");
$self->system_logged("chmod '$self->{'uconfig_hash'}->{'homedir_perms'}' $qh");
if ($uinfo) {
	$self->system_logged("chown $uinfo->{'uid'}:$uinfo->{'gid'} $qh");
	}
if ($tmpl->{'skel'} ne "none") {
	$self->copy_skel_files($tmpl->{'skel'}, $uinfo, $_[0]->{'home'},
			 $_[0]->{'group'} || $_[0]->{'ugroup'});
	}

# Setup web directories
local $d;
local $perms = $tmpl->{'web_html_perms'};
foreach $d ( [ $self->public_html_dir($_[0], 1), $perms ],
             [ 'cgi-bin', '755' ],
             [ 'logs', '755' ],
             [ $self->{'config_hash'}->{'homes_dir'}, '755' ] ) {
        $self->system_logged("mkdir -p $qh/$d->[0] 2>/dev/null");
        $self->system_logged("chmod $d->[1] $qh/$d->[0]");
	if ($uinfo) {
		$self->system_logged("chown $uinfo->{'uid'}:$uinfo->{'gid'} $qh/$d->[0]");
		}
        }
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# modify_dir(&domain, &olddomain)
# Rename home directory if needed
sub modify_dir
{
	my $self = shift;

if ($_[0]->{'home'} ne $_[1]->{'home'}) {
	# Move the home directory if changed, and if not already moved as
	# part of parent
	if (-d $_[1]->{'home'}) {
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_dirhome'})};
		$self->system_logged("mv ".quotemeta($_[1]->{'home'}).
			       " ".quotemeta($_[0]->{'home'}));
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	}
if ($_[0]->{'unix'} && !$_[1]->{'unix'}) {
	# Unix user now exists! Set ownership of home dir
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_dirchown'})};
	$self->set_home_ownership($_[0]);
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
}

# delete_dir(&domain)
# Delete the home directory
sub delete_dir
{
	my $self = shift;

# Delete homedir
if (-d $_[0]->{'home'} && $_[0]->{'home'} ne "/") {
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'delete_home'})};
	$self->system_logged("rm -rf ".quotemeta($_[0]->{'home'}));
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
}

# check_dir_clash(&domain, [field])
sub check_dir_clash
{
	my $self = shift;

# Does nothing ..?
return 0;
}

# backup_dir(&domain, file, &options)
# Backs up the server's home directory in tar format to the given file
sub backup_dir
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'backup_dirtar'})};

# Create the exclude file
local $xtemp = $self->transname();
$self->open_tempfile(XTEMP, ">$xtemp");
$self->print_tempfile(XTEMP, "domains\n");
if ($_[2]->{'dirnologs'}) {
	$self->print_tempfile(XTEMP, "logs\n");
	}
$self->close_tempfile(XTEMP);

# Do the backup
local $out = `cd '$_[0]->{'home'}' && tar cfX '$_[1]' '$xtemp' . 2>&1`;
if ($?) {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->text('backup_dirtarfailed', "<pre>$out</pre>"))};
	return 0;
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	return 1;
	}
}

# show_backup_dir(&options)
# Returns HTML for the backup logs option
sub show_backup_dir
{
	my $self = shift;

return sprintf
	"(<input type=checkbox name=dir_logs value=1 %s> %s)",
	!$self->{'opts_hash'}->{'dirnologs'} ? "checked" : "", $self->{'text_hash'}->{'backup_dirlogs'};
}

# parse_backup_dir(&in)
# Parses the inputs for directory backup options
sub parse_backup_dir
{
	my $self = shift;

local %in = %{$_[0]};
return { 'dirnologs' => !$in{'dir_logs'} };
}

# restore_dir(&domain, file, &options)
# Extracts the given tar file into server's home directory
sub restore_dir
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_dirtar'})};
local $out = `cd '$_[0]->{'home'}'; tar xf '$_[1]' 2>&1`;
if ($?) {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->text('backup_unixtarfailed', "<pre>$out</pre>"))};
	return 0;
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	if ($_[2]->{'chown'} && $_[0]->{'unix'}) {
		# Set ownership on extracted home directory, apart from
		# content of ~/homes
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_dirchowning'})};
		$self->set_home_ownership($_[0]);
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}

	return 1;
	}
}

# set_home_ownership(&domain)
# Update the ownership of all files in a server's home directory, EXCEPT
# the homes directory which is used by mail users
sub set_home_ownership
{
	my $self = shift;

local $hd = $self->{'config_hash'}->{'homes_dir'};
$hd =~ s/^\.\///;
local $gid = $_[0]->{'gid'} || $_[0]->{'ugid'};
$self->system_logged("find ".quotemeta($_[0]->{'home'})." \! -name $hd | xargs chown $_[0]->{'uid'}:$gid");
$self->system_logged("chown $_[0]->{'uid'}:$gid ".quotemeta($_[0]->{'home'})."/".$self->{'config_hash'}->{'homes_dir'});
}

# show_restore_dir(&options)
# Returns HTML for home restore option inputs
sub show_restore_dir
{
	my $self = shift;

# Offer to set file ownership
return sprintf
	"(<input type=checkbox name=dir_chown value=1 %s> %s)",
	$self->{'opts_hash'}->{'chown'} ? "checked" : "", $self->{'text_hash'}->{'restore_dirchown'};
}

# parse_restore_dir(&in)
# Parses the inputs for home restore options
sub parse_restore_dir
{
	my $self = shift;

local %in = %{$_[0]};
return { 'chown' => $in{'dir_chown'} };
}


1;

