#!/usr/bin/perl
package virtual_server::Featuredns;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'virtual_server::Featuredns');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;

$self->{'done_feature_script_hash'}->{'dns'} = 1;

1;

1;
		return $self;
}

sub require_bind
{
	my $self = shift;

return if ($self->{'require_bind'}++);
$self->foreign_require("bind8", "bind8-lib.pl");
%{$self->{'bconfig_hash'}} = $self->foreign_config("bind8");
}

# check_depends_dns(&domain)
# For a sub-domain that is being added to a parent DNS domain, make sure the
# parent zone actually exists
sub check_depends_dns
{
	my $self = shift;

if ($_[0]->{'subdom'}) {
	local $tmpl = $self->get_template($_[0]->{'template'});
	local $parent = $self->get_domain($_[0]->{'subdom'});
	if ($tmpl->{'dns_sub'} && !$parent->{'dns'}) {
		return $self->{'text_hash'}->{'setup_edepdnssub'};
		}
	}
return undef;
}

# setup_dns(&domain)
# Set up a zone for a domain
sub setup_dns
{
	my $self = shift;

$self->require_bind();
local $tmpl = $self->get_template($_[0]->{'template'});
if (!$_[0]->{'subdom'} || $tmpl->{'dns_sub'} ne 'yes') {
	# Creating a new real zone
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_bind'})};
	local $conf = $main::bind8_object->get_config();
	local $base = $self->{'bconfig_hash'}->{'master_dir'} ? $self->{'bconfig_hash'}->{'master_dir'} :
					       $main::bind8_object->base_directory($conf);
	local $file = $main::bind8_object->automatic_filename($_[0]->{'dom'}, 0, $base);
	local $dir = {
		 'name' => 'zone',
		 'values' => [ $_[0]->{'dom'} ],
		 'type' => 1,
		 'members' => [ { 'name' => 'type',
				  'values' => [ 'master' ] },
				{ 'name' => 'file',
				  'values' => [ $file ] } ]
		};
	if ($tmpl->{'namedconf'} ne 'none') {
		push(@{$dir->{'members'}},
		     $self->text_to_named_conf($tmpl->{'namedconf'}));
		}
	local $pconf;
	local $indent = 0;
	if ($tmpl->{'dns_view'}) {
		# Adding inside a view. This may use named.conf, or an include
		# file references inside the view, if any
		$pconf = $main::bind8_object->get_config_parent();
		local $view = $self->get_bind_view($conf, $tmpl->{'dns_view'});
		if ($view) {
			local $addfile = $main::bind8_object->add_to_file();
			local $addfileok;
			if ($main::bind8_object->{'config_hash'}->{'zones_file'}) {
				foreach my $vm (@{$view->{'members'}}) {
					if ($vm->{'file'} eq $addfile) {
						# Add file is OK
						$addfileok = 1;
						}
					}
				}

			if (!$addfileok) {
				# Add to named.conf
				$pconf = $view;
				$indent = 1;
				$dir->{'file'} = $view->{'file'};
				}
			else {
				# Add to the file
				$dir->{'file'} = $addfile;
				$pconf = $main::bind8_object->get_config_parent($addfile);
				}
			}
		else {
			$self->error($self->text('setup_ednsview', $tmpl->{'dns_view'}));
			}
		}
	else {
		# Adding at top level .. but perhaps in a different file
		$dir->{'file'} = $main::bind8_object->add_to_file();
		$pconf = $main::bind8_object->get_config_parent($dir->{'file'});
		}
	$self->lock_file($dir->{'file'});
	$main::bind8_object->save_directive($pconf, undef, [ $dir ], $indent);
	$self->flush_file_lines();
	$self->unlock_file($dir->{'file'});

	# Create the records file
	local %zd;
	$main::bind8_object->get_zone_defaults(\%zd);
	local $rootfile = $main::bind8_object->make_chroot($file);
	local $ip = $_[0]->{'dns_ip'} || $_[0]->{'ip'};
	local @slaves = $main::bind8_object->list_slave_servers();
	if (!-r $rootfile) {
		$self->lock_file($rootfile);
		$self->create_standard_records($file, $_[0], $ip, \%zd);
		$main::bind8_object->set_ownership($rootfile);
		$self->unlock_file($rootfile);
		}
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};

	# Create on slave servers
	local $myip = $self->{'bconfig_hash'}->{'this_ip'} ||
		      $self->to_ipaddress($self->get_system_hostname());
	if (@slaves) {
		local $slaves = join(" ", map { $_->{'host'} } @slaves);
		do { my $fn = $self->{'first_print'}; $self->$fn($self->text('setup_bindslave', $slaves))};
		local @slaveerrs = $main::bind8_object->create_on_slaves($_[0]->{'dom'},
							    $myip);
		if (@slaveerrs) {
			do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_eslaves'})};
			foreach $sr (@slaveerrs) {
				do { my $fn = $self->{'second_print'}; $self->$fn($sr->[0]->{'host'}." : ".
					       $sr->[1])};
				}
			}
		else {
			do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
			}
		$_[0]->{'dns_slave'} = $slaves;
		}

	undef(@{$main::bind8_object->{'get_config_cache_array'}});
	}
else {
	# Creating a sub-domain - add to parent's DNS zone
	local $parent = $self->get_domain($_[0]->{'subdom'});
	do { my $fn = $self->{'first_print'}; $self->$fn($self->text('setup_bindsub', $parent->{'dom'}))};
	local $z = $self->get_bind_zone($parent->{'dom'});
	if (!$z) {
		$self->error($self->text('setup_ednssub', $parent->{'dom'}));
		}
	local $file = $main::bind8_object->find("file", $z->{'members'});
	local $fn = $file->{'values'}->[0];
	$self->lock_file($main::bind8_object->make_chroot($fn));
	$_[0]->{'dns_submode'} = 1;	# So we know how this was done
	local $ipdom = $_[0]->{'virt'} ? $_[0] : $parent;
	local $ip = $ipdom->{'dns_ip'} || $ipdom->{'ip'};
	$self->create_standard_records($fn, $_[0], $ip, \%zd);
	local @recs = $main::bind8_object->read_zone_file($fn, $parent->{'dom'});
        $main::bind8_object->bump_soa_record($self->{'nfn'}, \@recs);
	$self->unlock_file($main::bind8_object->make_chroot($fn));

	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
$self->register_post_action(\&restart_bind);
}

sub slave_error_handler
{
	my $self = shift;

$self->{'slave_error'} = $_[0];
}

# delete_dns(&domain)
# Delete a domain from the BIND config
sub delete_dns
{
	my $self = shift;

$self->require_bind();
if (!$_[0]->{'dns_submode'}) {
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'delete_bind'})};
        @{$main::bind8_object->{'get_config_cache_array'}} = ();
	local $z = $self->get_bind_zone($_[0]->{'dom'});
	if ($z) {
		# Delete the records file
		local $file = $main::bind8_object->find("file", $z->{'members'});
		if ($file) {
			local $zonefile =
			    $main::bind8_object->make_chroot($file->{'values'}->[0]);
			$self->unlink_logged($zonefile);
			local $logfile = $zonefile.".log";
			if (!-r $logfile) { $logfile = $zonefile.".jnl"; }
			if (-r $logfile) {
				$self->unlink_logged($logfile);
				}
			}

		# Delete from named.conf
		local $rootfile = $main::bind8_object->make_chroot($z->{'file'});
		$self->lock_file($rootfile);
		local $lref = $self->read_file_lines($rootfile);
		splice(@$lref, $z->{'line'}, $z->{'eline'} - $z->{'line'} + 1);
		$self->flush_file_lines();
		$self->unlock_file($rootfile);
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	else {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'save_nobind'})};
		}

	local @slaves = split(/\s+/, $_[0]->{'dns_slave'});
	if (@slaves) {
		# Delete from slave servers too
		do { my $fn = $self->{'first_print'}; $self->$fn($self->text('delete_bindslave', $_[0]->{'dns_slave'}))};
		local @slaveerrs = $main::bind8_object->delete_on_slaves(
					$_[0]->{'dom'}, \@slaves);
		if (@slaveerrs) {
			do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'delete_bindeslave'})};
			foreach $sr (@slaveerrs) {
				do { my $fn = $self->{'second_print'}; $self->$fn($sr->[0]->{'host'}." : ".
					       $sr->[1])};
				}
			}
		else {
			do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
			}
		delete($_[0]->{'dns_slave'});
		}
	}
else {
	# Delete records from parent zone
	local $parent = $self->get_domain($_[0]->{'subdom'});
	do { my $fn = $self->{'first_print'}; $self->$fn($self->text('delete_bindsub', $parent->{'dom'}))};
	local $z = $self->get_bind_zone($parent->{'dom'});
	if (!$z) {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'save_nobind'})};
		return;
		}
	local $file = $main::bind8_object->find("file", $z->{'members'});
	local $fn = $file->{'values'}->[0];
	$self->lock_file($main::bind8_object->make_chroot($fn));
	local @recs = $main::bind8_object->read_zone_file($fn, $parent->{'dom'});
	foreach $r (reverse(@recs)) {
		if ($r->{'name'} =~ /$_[0]->{'dom'}/) {
			$main::bind8_object->delete_record($fn, $r);
			}
		}
        $main::bind8_object->bump_soa_record($fn, \@recs);
	$self->unlock_file($main::bind8_object->make_chroot($fn));
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	$_[0]->{'dns_submode'} = 0;
	}
$self->register_post_action(\&restart_bind);
}

# modify_dns(&domain, &olddomain)
# If the IP for this server has changed, update all records containing the old
# IP to the new.
sub modify_dns
{
	my $self = shift;

$self->require_bind();
local $tmpl = $self->get_template($_[0]->{'template'});
local $z;
local ($oldzonename, $newzonename);
if ($_[0]->{'dns_submode'}) {
	# Get parent domain
	local $parent = $self->get_domain($_[0]->{'subdom'});
	$z = $self->get_bind_zone($parent->{'dom'});
	$oldzonename = $newzonename = $parent->{'dom'};
	}
else {
	# Get this domain
	$z = $self->get_bind_zone($_[1]->{'dom'});
	$newzonename = $_[1]->{'dom'};
	$oldzonename = $_[1]->{'dom'};
	}
local $oldip = $_[1]->{'dns_ip'} || $_[1]->{'ip'};
local $newip = $_[0]->{'dns_ip'} || $_[0]->{'ip'};
local $rv = 0;
if ($_[0]->{'dom'} ne $_[1]->{'dom'}) {
	local $nfn;
	local $file = $main::bind8_object->find("file", $z->{'members'});
	if (!$_[0]->{'dns_submode'}) {
		# Domain name has changed .. rename zone file
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_dns2'})};
		$self->lock_file($main::bind8_object->make_chroot($z->{'file'}));
		local $fn = $file->{'values'}->[0];
		$nfn = $fn;
		$nfn =~ s/$_[1]->{'dom'}/$_[0]->{'dom'}/;
		if ($fn ne $nfn) {
			$self->rename_logged($main::bind8_object->make_chroot($fn),
				       $main::bind8_object->make_chroot($nfn))
			}
		$file->{'values'}->[0] = $nfn;

		# Change zone in .conf file
		$z->{'values'}->[0] = $_[0]->{'dom'};
		$main::bind8_object->save_directive($main::bind8_object->get_config_parent(),
				       [ $z ], [ $z ], 0);
		$self->flush_file_lines();
		$self->unlock_file($main::bind8_object->make_chroot($z->{'file'}));
		}
	else {
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_dns6'})};
		$nfn = $file->{'values'}->[0];
		}

	# Modify any records containing the old name
	$self->lock_file($main::bind8_object->make_chroot($nfn));
        local @recs = $main::bind8_object->read_zone_file($nfn, $oldzonename);
        foreach my $r (@recs) {
                if ($r->{'name'} =~ /$_[1]->{'dom'}/i) {
                        $r->{'name'} =~ s/$_[1]->{'dom'}/$_[0]->{'dom'}/;
			if ($r->{'type'} eq 'SPF') {
				# Fix SPF TXT record
				$r->{'values'}->[0] =~
					s/$_[1]->{'dom'}/$_[0]->{'dom'}/;
				}
			if ($r->{'type'} eq 'MX') {
				# Fix mail server in MX record
				$r->{'values'}->[1] =~
					s/$_[1]->{'dom'}/$_[0]->{'dom'}/;
				}
                        $main::bind8_object->modify_record($nfn, $r, $r->{'name'},
                                              $r->{'ttl'}, $r->{'class'},
                                              $r->{'type'},
					      $self->join_record_values($r),
                                              $r->{'comment'});
                        }
                }

        # Update SOA record
        $main::bind8_object->bump_soa_record($nfn, \@recs);
	$self->unlock_file($main::bind8_object->make_chroot($nfn));
	$rv++;
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};

	if (!$_[0]->{'dns_submode'}) {
		local @slaves = split(/\s+/, $_[0]->{'dns_slave'});
		if (@slaves) {
			# Rename on slave servers too
			do { my $fn = $self->{'first_print'}; $self->$fn($self->text('save_dns3', $_[0]->{'dns_slave'}))};
			local @slaveerrs = $main::bind8_object->rename_on_slaves(
				$_[1]->{'dom'}, $_[0]->{'dom'}, \@slaves);
			if (@slaveerrs) {
				do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'save_bindeslave'})};
				foreach $sr (@slaveerrs) {
					do { my $fn = $self->{'second_print'}; $self->$fn($sr->[0]->{'host'}." : ".
						       $sr->[1])};
					}
				}
			else {
				do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
				}
			}
		}
	}

if ($oldip ne $newip) {
	# IP address has changed .. need to update any records that use
	# the old IP
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_dns'})};
	local $file = $main::bind8_object->find("file", $z->{'members'});
	local $fn = $file->{'values'}->[0];
	local $zonefile = $main::bind8_object->make_chroot($fn);
	$self->lock_file($zonefile);
	local @recs = $main::bind8_object->read_zone_file($fn, $newzonename);
	foreach $r (@recs) {
		if ($r->{'values'}->[0] eq $oldip) {
			$main::bind8_object->modify_record($fn, $r, $r->{'name'},
					      $r->{'ttl'}, $r->{'class'},
					      $r->{'type'}, $newip,
					      $r->{'comment'});
			}
		}

	# Update SOA record
	$main::bind8_object->bump_soa_record($fn, \@recs);
	$self->unlock_file($zonefile);
	$rv++;
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}

if ($_[0]->{'mail'} && !$_[1]->{'mail'} && !$tmpl->{'dns_replace'}) {
	# Email was enabled .. add MX records
	local $file = $main::bind8_object->find("file", $z->{'members'});
	local $fn = $file->{'values'}->[0];
	local $zonefile = $main::bind8_object->make_chroot($fn);
	$self->lock_file($zonefile);
	local @recs = $main::bind8_object->read_zone_file($fn, $newzonename);
	local ($mx) = grep { $_->{'type'} eq 'MX' &&
			     $_->{'name'} eq $_[0]->{'dom'}."." ||
			     $_->{'type'} eq 'A' &&
			     $_->{'name'} eq "mail.".$_[0]->{'dom'}."." } @recs;
	if (!$mx) {
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_dns4'})};
		local $ip = $_[0]->{'dns_ip'} || $_[0]->{'ip'};
		$self->create_mx_records($fn, $_[0], $ip);
		$main::bind8_object->bump_soa_record($fn, \@recs);
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		$rv++;
		}
	$self->unlock_file($zonefile);
	}
elsif (!$_[0]->{'mail'} && $_[1]->{'mail'} && !$tmpl->{'dns_replace'}) {
	# Email was disabled .. remove MX records
	local $file = $main::bind8_object->find("file", $z->{'members'});
	local $zonefile = $main::bind8_object->make_chroot($file);
	$self->lock_file($zonefile);
	local $fn = $file->{'values'}->[0];
	local @recs = $main::bind8_object->read_zone_file($fn, $newzonename);
	local @mx = grep { $_->{'type'} eq 'MX' &&
			   $_->{'name'} eq $_[0]->{'dom'}."." ||
			   $_->{'type'} eq 'A' &&
			   $_->{'name'} eq "mail.".$_[0]->{'dom'}."." } @recs;
	if (@mx) {
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_dns5'})};
		foreach my $r (reverse(@mx)) {
			$main::bind8_object->delete_record($fn, $r);
			}
		$main::bind8_object->bump_soa_record($fn, \@recs);
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		$rv++;
		}
	$self->unlock_file($zonefile);
	}

if ($_[0]->{'mx_servers'} ne $_[1]->{'mx_servers'}) {
	# Secondary MX servers have been changed - add or remove MX records
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_dns7'})};
	local @newmxs = split(/\s+/, $_[0]->{'mx_servers'});
	local @oldmxs = split(/\s+/, $_[1]->{'mx_servers'});
	local $file = $main::bind8_object->find("file", $z->{'members'});
	local $zonefile = $main::bind8_object->make_chroot($file);
	$self->lock_file($zonefile);
	local $fn = $file->{'values'}->[0];
	local @recs = $main::bind8_object->read_zone_file($fn, $newzonename);
	$self->foreign_require("servers", "servers-lib.pl");
	local %servers = map { $_->{'id'}, $_ } $main::servers_object->list_servers();
	local $withdot = $_[0]->{'dom'}.".";

	# Add missing MX records
	foreach my $id (@newmxs) {
		if ($self->indexof($id, @oldmxs) < 0) {
			# A new MX .. add a record for it
			local $s = $servers{$id};
			local $mxhost = $s->{'mxhost'} || $s->{'host'};
			$main::bind8_object->create_record($fn, $withdot, undef,
				      "IN", "MX", "10 $mxhost.");
			}
		}

	# Remove those that are no longer needed
	local @mxs;
	foreach my $id (@oldmxs) {
		if ($self->indexof($id, @newmxs) < 0) {
			# An old MX .. remove it
			local $s = $servers{$id};
			local $mxhost = $s->{'mxhost'} || $s->{'host'};
			foreach my $r (@recs) {
				if ($r->{'type'} eq 'MX' &&
				    $r->{'name'} eq $withdot &&
				    $r->{'values'}->[1] eq $mxhost.".") {
					push(@mxs, $r);
					}
				}
			}
		}
	foreach my $r (reverse(@mxs)) {
		$main::bind8_object->delete_record($fn, $r);
		}

	$main::bind8_object->bump_soa_record($fn, \@recs);
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	$rv++;
	}

$self->register_post_action(\&restart_bind) if ($rv);
return $rv;
}

# join_record_values(&record)
# Given the values for a record, joins them into a space-separated string
# with quoting if needed
sub join_record_values
{
	my $self = shift;

local ($r) = @_;
local @rv;
foreach my $v (@{$r->{'values'}}) {
	push(@rv, $v =~ /\s/ ? "\"$v\"" : $v);
	}
return join(" ", @rv);
}

# create_mx_records(file, &domain, ip)
# Adds MX records to a DNS domain
sub create_mx_records
{
	my $self = shift;

local ($file, $d, $ip) = @_;
local $withdot = $d->{'dom'}.".";
$main::bind8_object->create_record($file, "mail.$withdot", undef,
		      "IN", "A", $ip);
$main::bind8_object->create_record($file, $withdot, undef,
		      "IN", "MX", "5 mail.$withdot");

# Add MX records for slaves
local %ids = map { $_, 1 }
	split(/\s+/, $d->{'mx_servers'});
local @servers = grep { $ids{$_->{'id'}} } $self->list_mx_servers();
local $n = 10;
foreach my $s (@servers) {
	local $mxhost = $s->{'mxhost'} || $s->{'host'};
	$main::bind8_object->create_record($file, $withdot, undef,
		      "IN", "MX", "$n $mxhost.");
	$n += 5;
	}
}

# create_standard_records(file, &domain, ip, \%zd)
# Adds to a records file the needed records for some domain
sub create_standard_records
{
	my $self = shift;

my(%defv);
local ($file, $d, $ip, $ref) = @_;
%defv = %{$ref};
local $rootfile = $main::bind8_object->make_chroot($file);
local $tmpl = $self->get_template($d->{'template'});
local $serial = $self->{'bconfig_hash'}->{'soa_style'} ?
	$main::bind8_object->date_serial().sprintf("%2.2d", $self->{'bconfig_hash'}->{'soa_start'}) :
	time();
if (!$tmpl->{'dns_replace'}) {
	# Create records that are appropriate for this domain
	if (!$d->{'dns_submode'}) {
		# Only add SOA if this is a new file, not a sub-domain
		$self->open_tempfile(RECS, ">$rootfile");
		if ($self->{'bconfig_hash'}->{'master_ttl'}) {
			$self->print_tempfile(RECS,
			    "\$ttl $defv{'minimum'}$defv{'minunit'}\n");
			}
		$self->close_tempfile(RECS);
		local $master = $self->{'bconfig_hash'}->{'default_prins'} ||
				$self->get_system_hostname();
		$master .= "." if ($master !~ /\.$/);
		local $email = $self->{'bconfig_hash'}->{'tmpl_email'} ||
			       "root\@$master";
		$email = $main::bind8_object->email_to_dotted($email);
		local $soa = "$master $email (\n".
			     "\t\t\t$serial\n".
			     "\t\t\t$defv{'refresh'}$defv{'refunit'}\n".
			     "\t\t\t$defv{'retry'}$defv{'retunit'}\n".
			     "\t\t\t$defv{'expiry'}$defv{'expunit'}\n".
			     "\t\t\t$defv{'minimum'}$defv{'minunit'} )";
		$main::bind8_object->create_record($file, "@", undef, "IN",
				      "SOA", $soa);
		$main::bind8_object->create_record($file, "@", undef, "IN",
				      "NS", $master);
		local $slave;
		foreach $slave (@{$self->{'slaves_array'}}) {
			local @bn = $slave->{'nsname'} ||
				    gethostbyname($slave->{'host'});
			local $full = "$bn[0].";
			$main::bind8_object->create_record($file, "@", undef, "IN",
					      "NS", "$bn[0].");
			}
		}
	local $withdot = $d->{'dom'}.".";
	$main::bind8_object->create_record($file, $withdot, undef,
			      "IN", "A", $ip);
	$main::bind8_object->create_record($file, "www.$withdot", undef,
			      "IN", "A", $ip);
	$main::bind8_object->create_record($file, "ftp.$withdot", undef,
			      "IN", "A", $ip);
	if ($d->{'mail'}) {
		# For mail domains, add MX to this server
		$self->create_mx_records($file, $d, $ip);
		}
	if ($tmpl->{'dns_spf'} ne "none") {
		# Add SPF record for domain
		local $str = $main::bind8_object->join_spf($self->default_domain_spf($d));
		$main::bind8_object->create_record($file, $withdot, undef,
				      "IN", "TXT", "\"$str\"");
		}
	}
if ($tmpl->{'dns'} && (!$d->{'dns_submode'} || !$tmpl->{'dns_replace'})) {
	# Add or use the user-defined template
	$self->open_tempfile(RECS, ">>$rootfile");
	local %subs = %$d;
	$subs{'serial'} = $serial;
	$subs{'dnsemail'} = $d->{'emailto'};
	$subs{'dnsemail'} =~ s/\@/./g;
	local $recs = $self->substitute_domain_template(
		join("\n", split(/\t+/, $tmpl->{'dns'}))."\n", \%subs);
	$self->print_tempfile(RECS, $recs);
	$self->close_tempfile(RECS);
	}
}

# validate_dns(&domain)
# Check for the DNS domain and records file
sub validate_dns
{
	my $self = shift;

local ($d) = @_;
local $z;
if ($d->{'dns_submode'} && $d->{'subdom'}) {
	# Records are in parent domain's file
	local $parent = $self->get_domain($d->{'subdom'});
	$z = $self->get_bind_zone($parent->{'dom'});
	}
else {
	# Domain has its own records file
	$z = $self->get_bind_zone($d->{'dom'});
	}
return $self->text('validate_edns', "<tt>$d->{'dom'}</tt>") if (!$z);
local $file = $main::bind8_object->find("file", $z->{'members'});
return $self->text('validate_ednsfile', "<tt>$d->{'dom'}</tt>") if (!$file);
local $zonefile = $main::bind8_object->make_chroot(
			$main::bind8_object->absolute_path($file->{'values'}->[0]));
return $self->text('validate_ednsfile2', "<tt>$zonefile</tt>") if (!-r $zonefile);
local @recs = $main::bind8_object->read_zone_file($file->{'values'}->[0], $d->{'dom'});
local %got;
foreach my $r (@recs) {
	$got{lc($r->{'type'})}++;
	}
$got{'SOA'} || $self->text('validate_ednssoa', "<tt>$zonefile</tt>");
$got{'A'} || $self->text('validate_ednsa', "<tt>$zonefile</tt>");
return undef;
}

# disable_dns(&domain)
# Re-names this domain in named.conf with the .disabled suffix
sub disable_dns
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'disable_bind'})};
$self->require_bind();
local $z = $self->get_bind_zone($_[0]->{'dom'});
if ($z) {
	local $rootfile = $main::bind8_object->make_chroot($z->{'file'});
	$self->lock_file($rootfile);
	$z->{'values'}->[0] = $_[0]->{'dom'}.".disabled";
	$main::bind8_object->save_directive($main::bind8_object->get_config_parent(), [ $z ], [ $z ], 0);
	$self->flush_file_lines();
	$self->unlock_file($rootfile);
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	$self->register_post_action(\&restart_bind);
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'save_nobind'})};
	}
}

# enable_dns(&domain)
# Re-names this domain in named.conf to remove the .disabled suffix
sub enable_dns
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'enable_bind'})};
$self->require_bind();
local $z = $self->get_bind_zone($_[0]->{'dom'});
if ($z) {
	local $rootfile = $main::bind8_object->make_chroot($z->{'file'});
	$self->lock_file($rootfile);
	$z->{'values'}->[0] = $_[0]->{'dom'};
	$main::bind8_object->save_directive($main::bind8_object->get_config_parent(), [ $z ], [ $z ], 0);
	$self->flush_file_lines();
	$self->unlock_file($rootfile);
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	$self->register_post_action(\&restart_bind);
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'save_nobind'})};
	}
}

# get_bind_zone(name, [&config], [file])
# Returns the zone structure for the named domain, possibly with .disabled
sub get_bind_zone
{
	my $self = shift;

$self->require_bind();
local $conf = $_[1] ? $_[1] :
	      $_[2] ? [ $main::bind8_object->read_config_file($_[2]) ] :
		      $main::bind8_object->get_config();
local @zones = $main::bind8_object->find("zone", $conf);
local ($v, $z);
foreach $v ($main::bind8_object->find("view", $conf)) {
	push(@zones, $main::bind8_object->find("zone", $v->{'members'}));
	}
local ($z) = grep { lc($_->{'value'}) eq lc($_[0]) ||
		    lc($_->{'value'}) eq lc("$_[0].disabled") } @zones;
return $z;
}

# restart_bind(&domain)
# Signal BIND to re-load its configuration
sub restart_bind
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_bindpid'})};
local $pid = $self->get_bind_pid();
if ($pid) {
	if ($self->{'bconfig_hash'}->{'restart_cmd'}) {
		$self->system_logged("$self->{'bconfig_hash'}->{'restart_cmd'} >/dev/null 2>&1 </dev/null");
		}
	else {
		$self->kill_logged('HUP', $pid);
		}
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	$self->{'rv'} = 1;
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_notrun'})};
	$self->{'rv'} = 0;
	}
if ($main::bind8_object->list_slave_servers()) {
	# Re-start on slaves too
	do { my $fn = $self->{'first_print'}; $self->$fn($self->text('setup_bindslavepids'))};
	local @slaveerrs = $main::bind8_object->restart_on_slaves();
	if (@slaveerrs) {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_bindeslave'})};
		foreach $sr (@slaveerrs) {
			do { my $fn = $self->{'second_print'}; $self->$fn($sr->[0]->{'host'}." : ".$sr->[1])};
			}
		}
	else {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	}
return $self->{'rv'};
}

# check_dns_clash(&domain, [changing])
# Returns 1 if a domain already exists in BIND
sub check_dns_clash
{
	my $self = shift;

if (!$_[1] || $_[1] eq 'dom') {
	local ($czone) = $self->get_bind_zone($_[0]->{'dom'});
	return $czone ? 1 : 0;
	}
return 0;
}

# get_bind_pid()
# Returns the BIND PID, if it is running
sub get_bind_pid
{
	my $self = shift;

$self->require_bind();
local $pidfile = $main::bind8_object->get_pid_file();
return $self->check_pid_file($main::bind8_object->make_chroot($pidfile, 1));
}

# backup_dns(&domain, file)
# Save all the virtual server's DNS records as a separate file
sub backup_dns
{
	my $self = shift;

$self->require_bind();
return 1 if ($_[0]->{'dns_submode'});	# backed up in parent
do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'backup_dnscp'})};
local $z = $self->get_bind_zone($_[0]->{'dom'});
if ($z) {
	local $file = $main::bind8_object->find("file", $z->{'members'});
	local $filename = $main::bind8_object->make_chroot(
		$main::bind8_object->absolute_path($file->{'values'}->[0]));
	$self->execute_command("cp ".quotemeta($filename)." ".quotemeta($_[1]));
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	return 1;
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'backup_dnsnozone'})};
	return 0;
	}
}

# restore_dns(&domain, file, &options)
# Update the virtual server's DNS records from the backup file, except the SOA
sub restore_dns
{
	my $self = shift;

$self->require_bind();
return 1 if ($_[0]->{'dns_submode'});	# restored in parent
do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_dnscp'})};
local $z = $self->get_bind_zone($_[0]->{'dom'});
if ($z) {
	local $file = $main::bind8_object->find("file", $z->{'members'});
	local $filename = $main::bind8_object->make_chroot(
			$main::bind8_object->absolute_path($file->{'values'}->[0]));

	if ($_[2]->{'wholefile'}) {
		# Copy whole file
		$self->lock_file($filename);
		$self->copy_source_dest($_[1], $filename);
		}
	else {
		# Only copy section after SOA
		local $srclref = $self->read_file_lines($_[1]);
		local $dstlref = $self->read_file_lines($filename);
		$self->lock_file($filename);
		local ($srcstart, $srcend) = $self->except_soa($_[0], $_[1]);
		local ($dststart, $dstend) = $self->except_soa($_[0], $filename);
		splice(@$dstlref, $dststart, $dstend - $dststart + 1,
		       @$srclref[$srcstart .. $srcend]);
		$self->flush_file_lines();
		}

	# Need to bump SOA
	local $fn = $file->{'values'}->[0];
	local @recs = $main::bind8_object->read_zone_file($fn, $_[0]->{'dom'});
	$main::bind8_object->bump_soa_record($file->{'values'}->[0], \@recs);

	if ($_[2]->{'fixip'}) {
		# Need to update IP addresses
		local $r;
		local ($baserec) = grep { $_->{'type'} eq "A" &&
					  ($_->{'name'} eq $_[0]->{'dom'}."." ||
					   $_->{'name'} eq '@') } @recs;
		foreach $r (@recs) {
			if ($r->{'type'} eq "A" &&
			    $r->{'values'}->[0] eq $baserec->{'values'}->[0]) {
				$main::bind8_object->modify_record($fn, $r, $r->{'name'},
						      $r->{'ttl'},$r->{'class'},
						      $r->{'type'},
						      $_[0]->{'ip'},
						      $r->{'comment'});
				}
			}
		}

	$self->unlock_file($filename);
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};

	$self->register_post_action(\&restart_bind);
	return 1;
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'backup_dnsnozone'})};
	return 0;
	}
}

# except_soa(&domain, file)
sub except_soa
{
	my $self = shift;

local $main::bind8_object->{'config_hash'}->{'chroot'} = "/";	# make sure path is absolute
local @recs = $main::bind8_object->read_zone_file($_[1], $_[0]->{'dom'});
local ($r, $start, $end);
foreach $r (@recs) {
	if ($r->{'type'} ne "SOA" && !$r->{'generate'} && !$r->{'defttl'} &&
	    !defined($start)) {
		$start = $r->{'line'};
		}
	$end = $r->{'eline'};
	}
return ($start, $end);
}

# get_bind_view([&conf], view)
# Returns the view object for the view to add domains to
sub get_bind_view
{
	my $self = shift;

$self->require_bind();
local $conf = $_[0] || $main::bind8_object->get_config();
local @views = $main::bind8_object->find("view", $conf);
local ($view) = grep { $_->{'values'}->[0] eq $_[1] } @views;
return $view;
}

# show_restore_dns(&options)
# Returns HTML for DNS restore option inputs
sub show_restore_dns
{
	my $self = shift;

local ($opts, $d) = @_;
return $self->ui_checkbox("dns_fixip", 1, $self->{'text_hash'}->{'restore_dnsfixip'},
		    $opts->{'fixip'})."<br>\n".
       $self->ui_checkbox("dns_wholefile", 1, $self->{'text_hash'}->{'restore_dnswholefile'},
		    $opts->{'wholefile'});
}

# parse_restore_dns(&in)
# Parses the inputs for DNS restore options
sub parse_restore_dns
{
	my $self = shift;

local ($in, $d) = @_;
return { 'fixip' => $in->{'dns_fixip'},
	 'wholefile' => $in->{'dns_wholefile'} };
}

# sysinfo_dns()
# Returns the BIND version
sub sysinfo_dns
{
	my $self = shift;

$self->require_bind();
if (!$main::bind8_object->{'bind_version'}) {
	local $out = `$main::bind8_object->{'config_hash'}->{'named_path'} -v 2>&1`;
	if ($out =~ /(bind|named)\s+([0-9\.]+)/i) {
		$main::bind8_object->{'bind_version'} = $2;
		}
	}
return ( [ $self->{'text_hash'}->{'sysinfo_bind'}, $main::bind8_object->{'bind_version'} ] );
}

# links_dns(&domain)
# Returns a link to the BIND module
sub links_dns
{
	my $self = shift;

local ($d) = @_;
if ($self->{'config_hash'}->{'avail_dns'} && !$d->{'dns_submode'}) {
	return ( { 'mod' => 'bind8',
		   'desc' => $self->{'text_hash'}->{'links_dns'},
		   'page' => "edit_master.cgi?zone=".$self->urlize($d->{'dom'}),
		   'cat' => 'services',
		 } );
	}
return ( );
}

sub startstop_dns
{
	my $self = shift;

local $bpid = $self->get_bind_pid();
if ($bpid && kill(0, $bpid)) {
	return { 'status' => 1,
		 'name' => $self->{'text_hash'}->{'index_bname'},
		 'desc' => $self->{'text_hash'}->{'index_bstop'},
		 'restartdesc' => $self->{'text_hash'}->{'index_brestart'},
		 'longdesc' => $self->{'text_hash'}->{'index_bstopdesc'} };
	}
else {
	return { 'status' => 0,
		 'name' => $self->{'text_hash'}->{'index_bname'},
		 'desc' => $self->{'text_hash'}->{'index_bstart'},
		 'longdesc' => $self->{'text_hash'}->{'index_bstartdesc'} };
	}
}

sub start_service_dns
{
	my $self = shift;

$self->require_bind();
return $main::bind8_object->start_bind();
}

sub stop_service_dns
{
	my $self = shift;

$self->require_bind();
return $main::bind8_object->stop_bind();
}

# show_template_dns(&tmpl)
# Outputs HTML for editing BIND related template options
sub show_template_dns
{
	my $self = shift;

local ($tmpl) = @_;

# DNS records
print "<tr> <td valign=top>",$self->hlink("<b>$self->{'text_hash'}->{'tmpl_dns'}</b>",
				    "template_dns"),"</td> <td>\n";
print $self->none_def_input("dns", $tmpl->{'dns'}, $self->{'text_hash'}->{'tmpl_dnsbelow'},1)."<br>\n";
print "<textarea name=dns rows=10 cols=60>";
if ($tmpl->{'dns'} ne "none") {
	print join("\n", split(/\t/, $tmpl->{'dns'}));
	}
print "</textarea>\n";

print "<table>\n";

print "<tr> <td><b>$self->{'text_hash'}->{'tmpl_replace'}</b></td> <td>\n";
printf "<input type=radio name=bind_replace value=0 %s> %s\n",
	$tmpl->{'dns_replace'} ? "" : "checked", $self->{'text_hash'}->{'tmpl_replace0'};
printf "<input type=radio name=bind_replace value=1 %s> %s</td> </tr>\n",
	$tmpl->{'dns_replace'} ? "checked" : "", $self->{'text_hash'}->{'tmpl_replace1'};

# Option for view to add to, for BIND 9
$self->require_bind();
$self->{'conf'} = $main::bind8_object->get_config();
@{$self->{'views_array'}} = $main::bind8_object->find("view", $self->{'conf'});
if (@{$self->{'views_array'}}) {
	print "<tr> <td valign=top><b>$self->{'text_hash'}->{'newdns_view'}</b></td>\n";
	print "<td><select name=view>\n";
	printf "<option value='' %s>%s\n",
		$self->{'config_hash'}->{'dns_view'} eq "" ? "selected" : "",
		$self->{'text_hash'}->{'newdns_noview'};
	foreach $v (@{$self->{'views_array'}}) {
		printf "<option value=%s %s>%s\n",
			$v->{'values'}->[0],
			$self->{'config_hash'}->{'dns_view'} eq $v->{'values'}->[0] ? "selected" : "",
			$v->{'values'}->[0];
		}
	print "</select></td> </tr>\n";
	}
print "</table>\n";
print "</td> </tr>\n";

# Option for SPF record
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'tmpl_spf'},
                           "template_dns_spf_mode"),
	$self->none_def_input("dns_spf", $tmpl->{'dns_spf'},
		        $self->{'text_hash'}->{'tmpl_spfyes'}, 0, 0, $self->{'text_hash'}->{'no'}));

# Extra SPF hosts
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'tmpl_spfhosts'},
			   "template_dns_spfhosts"),
	$self->ui_textbox("dns_spfhosts", $tmpl->{'dns_spfhosts'}, 40));

# SPF ~all mode
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'tmpl_spfall'},
			   "template_dns_spfall"),
	$self->ui_yesno_radio("dns_spfall", $tmpl->{'dns_spfall'} ? 1 : 0));

# Add sub-domains to parent domain DNS
print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'tmpl_dns_sub'},
                           "template_dns_sub"),
	$self->none_def_input("dns_sub", $tmpl->{'dns_sub'},
		        $self->{'text_hash'}->{'yes'}, 0, 0, $self->{'text_hash'}->{'no'}));

# Extra named.conf directives
print $self->ui_table_hr();

print $self->ui_table_row($self->hlink($self->{'text_hash'}->{'tmpl_namedconf'}, "namedconf"),
    $self->none_def_input("namedconf", $tmpl->{'namedconf'},
		    $self->{'text_hash'}->{'tmpl_namedconfbelow'})."<br>".
    $self->ui_textarea("namedconf",
		 $tmpl->{'namedconf'} eq 'none' ? '' :
			join("\n", split(/\t/, $tmpl->{'namedconf'})),
		 5, 60));
}

# parse_template_dns(&tmpl)
# Updates BIND related template options from %{$self->{'in_hash'}}
sub parse_template_dns
{
	my $self = shift;

local ($tmpl) = @_;

# Save DNS settings
$tmpl->{'dns'} = $self->parse_none_def("dns");
if ($self->{'in_hash'}->{"dns_mode"} == 2) {
	$tmpl->{'default'} || $tmpl->{'dns'} ||
		$self->error($self->{'text_hash'}->{'tmpl_edns'});
	$tmpl->{'dns_replace'} = $self->{'in_hash'}->{'bind_replace'};
	$tmpl->{'dns_view'} = $self->{'in_hash'}->{'view'};

	$self->require_bind();
	$self->{'fakeip'} = "1.2.3.4";
	$self->{'fakedom'} = "foo.com";
	$self->{'recs'} = $self->substitute_template(
		join("\n", split(/\t+/, $self->{'in_hash'}->{'dns'}))."\n",
		{ 'ip' => $self->{'fakeip'},
		  'dom' => $self->{'fakedom'} });
	$self->{'temp'} = $self->transname();
	$self->open_tempfile(TEMP, ">$self->{'temp'}");
	$self->print_tempfile(TEMP, $self->{'recs'});
	$self->close_tempfile(TEMP);
	$main::bind8_object->{'config_hash'}->{'short_names'} = 0;	# force canonicalization
	$main::bind8_object->{'config_hash'}->{'chroot'} = '/';		# turn off chroot for temp path
	$main::bind8_object->{'config_hash'}->{'auto_chroot'} = undef;
	@recs = $main::bind8_object->read_zone_file($self->{'temp'}, $self->{'fakedom'});
	unlink($self->{'temp'});
	foreach $r (@recs) {
		$self->{'soa'}++ if ($r->{'name'} eq $self->{'fakedom'}."." &&
			   $r->{'type'} eq "SOA");
		$self->{'ns'}++ if ($r->{'name'} eq $self->{'fakedom'}."." &&
			  $r->{'type'} eq "NS");
		$self->{'dom'}++ if ($r->{'name'} eq $self->{'fakedom'}."." &&
			   ($r->{'type'} eq "A" || $r->{'type'} eq "MX"));
		$self->{'www'}++ if ($r->{'name'} eq "www.".$self->{'fakedom'}."." &&
			   $r->{'type'} eq "A" ||
			   $r->{'type'} eq "CNAME");
		}

	if ($self->{'in_hash'}->{'bind_replace'}) {
		# Make sure an SOA and NS records exist
		$self->{'soa'} == 1 || $self->error($self->{'text_hash'}->{'newdns_esoa'});
		$self->{'ns'} || $self->error($self->{'text_hash'}->{'newdns_ens'});
		$self->{'dom'} || $self->error($self->{'text_hash'}->{'newdns_edom'});
		$self->{'www'} || $self->error($self->{'text_hash'}->{'newdns_ewww'});
		}
	else {
		# Make sure SOA doesn't exist
		$self->{'soa'} && $self->error($self->{'text_hash'}->{'newdns_esoa2'});
		}
	}

# Save SPF
$tmpl->{'dns_spf'} = $self->{'in_hash'}->{'dns_spf_mode'} == 0 ? "none" :
		     $self->{'in_hash'}->{'dns_spf_mode'} == 1 ? undef : "yes";
$tmpl->{'dns_spfhosts'} = $self->{'in_hash'}->{'dns_spfhosts'};
$tmpl->{'dns_spfall'} = $self->{'in_hash'}->{'dns_spfall'};

# Save sub-domain DNS mode
$tmpl->{'dns_sub'} = $self->{'in_hash'}->{'dns_sub_mode'} == 0 ? "none" :
		     $self->{'in_hash'}->{'dns_sub_mode'} == 1 ? undef : "yes";

# Save named.conf
$tmpl->{'namedconf'} = $self->parse_none_def("namedconf");
if ($self->{'in_hash'}->{'namedconf_mode'} == 2) {
	# Make sure the directives are valid
	local @recs = $self->text_to_named_conf($tmpl->{'namedconf'});
	if ($tmpl->{'namedconf'} =~ /\S/ && !@recs) {
		$self->error($self->{'text_hash'}->{'newdns_enamedconf'});
		}
	}
}

# get_domain_spf(&domain)
# Returns the SPF object for a domain from its DNS records, or undef.
sub get_domain_spf
{
	my $self = shift;

local ($d) = @_;
local @recs = $self->get_domain_dns_records($d);
foreach my $r (@recs) {
	if ($r->{'type'} eq 'SPF' &&
	    $r->{'name'} eq $d->{'dom'}.'.') {
		return $main::bind8_object->parse_spf($r->{'values'}->[0]);
		}
	}
return undef;
}

# save_domain_spf(&domain, &spf)
# Updates/creates/deletes a domain's SPF record.
sub save_domain_spf
{
	my $self = shift;

local ($d, $spf) = @_;
local @recs = $self->get_domain_dns_records($d);
return if (!@recs);		# Domain not found!
local ($r) = grep { $_->{'type'} eq 'SPF' &&
		    $_->{'name'} eq $d->{'dom'}.'.' } @recs;
local $str = $spf ? $main::bind8_object->join_spf($spf) : undef;
if ($r && $spf) {
	# Update record
	$main::bind8_object->modify_record($r->{'file'}, $r, $r->{'name'}, $r->{'ttl'},
			      $r->{'class'}, $r->{'type'}, "\"$str\"",
			      $r->{'comment'});
	}
elsif ($r && !$spf) {
	# Remove record
	$main::bind8_object->delete_record($r->{'file'}, $r);
	}
elsif (!$r && $spf) {
	# Add record
	$main::bind8_object->create_record($recs[0]->{'file'}, $d->{'dom'}.'.', undef,
			      "IN", "TXT", "\"$str\"");
	}
else {
	return;
	}
$main::bind8_object->bump_soa_record($recs[0]->{'file'}, \@recs);
$self->register_post_action(\&restart_bind);
}

# get_domain_dns_records(&domain)
# Returns an array of DNS records for a domain, or empty if the file couldn't
# be found.
sub get_domain_dns_records
{
	my $self = shift;

local ($d) = @_;
$self->require_bind();
local $z = $self->get_bind_zone($d->{'dom'});
return ( ) if (!$z);
local $file = $main::bind8_object->find("file", $z->{'members'});
return ( ) if (!$file);
local $fn = $file->{'values'}->[0];
return $main::bind8_object->read_zone_file($fn, $d->{'dom'});
}

# default_domain_spf(&domain)
# Returns a default SPF object for a domain, based on its template
sub default_domain_spf
{
	my $self = shift;

local ($d) = @_;
local $tmpl = $self->get_template($d->{'template'});
local $defip = $self->get_default_ip();
local $spf = { 'a' => 1, 'mx' => 1,
	       'a:' => [ $d->{'dom'} ],
	       'ip4:' => [ $defip ] };
local $hosts = $self->substitute_domain_template(
	$tmpl->{'dns_spfhosts'}, $d);
foreach my $h (split(/\s+/, $hosts)) {
	if ($self->check_ipaddress($h) ||
	    $h =~ /^(\S+)\// &&
	    $self->check_ipaddress($1)) {
		push(@{$spf->{'ip4:'}}, $h);
		}
	else {
		push(@{$spf->{'a:'}}, $h);
		}
	}
$spf->{'all'} = $tmpl->{'dns_spfall'} ? 2 : 1;
return $spf;
}

# text_to_named_conf(text)
# Converts a text string which contains zero or more BIND directives into an
# array of directive objects.
sub text_to_named_conf
{
	my $self = shift;

local ($str) = @_;
local $temp = $self->transname();
$self->open_tempfile(TEMP, ">$temp");
$self->print_tempfile(TEMP, $str);
$self->close_tempfile(TEMP);
$self->require_bind();
return $main::bind8_object->read_config_file($temp, 0);
}


1;

