#!/usr/bin/perl
package virtual_server::Featureftp;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'virtual_server::Featureftp');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for managing a virtual FTP server

$self->{'feature_depends_hash'}->{'ftp'} = [ 'dir', 'virt' ];

1;

1;
		return $self;
}

sub require_ftp
{
	my $self = shift;

 return if ($self->{'require_ftp'}++);
 if ($self->{'config_hash'}->{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  $self->foreign_require("proftpd", "proftpd-lib.pl");
 }
 elsif ($self->{'config_hash'}->{'ftp_system'} == 2)
 {
  #using VSFTPd for FTP
  $self->foreign_require("vsftpd", "vsftpd-lib.pl");
 }
}

# setup_ftp(&domain, ftp_root_dir)
# Setup a virtual FTP server for some domain
sub setup_ftp_rootdir {
	my $self = shift;

	local $ftp = $_[1];
	if (!-d $ftp) {
		$self->system_logged("mkdir '$ftp' 2>/dev/null");
		$self->system_logged("chmod 755 '$ftp'");
		$self->system_logged("chown $_[0]->{'uid'}:$_[0]->{'ugid'} '$ftp'");
	}
}

# setup_ftp(&domain)
# Setup a virtual FTP server for some domain
sub setup_ftp
{
	my $self = shift;

$self->require_ftp();
# Get the template
local $tmpl = $self->get_template($_[0]->{'template'});

# Define directory for FTP root
local ($fdir) = ($tmpl->{'ftp_dir'} || 'ftp');
local $ftp = "$_[0]->{'home'}/$fdir";
if ($self->{'config_hash'}->{'ftp_system'} == 1) {
	# Set ProFTPD up for FTP feature
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_proftpd'})};

	# Get the template
	local @dirs = $self->proftpd_template($tmpl->{'ftp'}, $_[0]);

	# Add the directives
	local $conf = $main::proftpd_object->get_config();
	local $l = $conf->[@$conf - 1];
	$self->lock_file($l->{'file'});
	local $lref = $self->read_file_lines($l->{'file'});
	local @lines = ( "<VirtualHost $_[0]->{'ip'}>" );
	push(@lines, @dirs);
	push(@lines, "</VirtualHost>");
	push(@$lref, @lines);
  
	$self->flush_file_lines();
	$self->unlock_file($l->{'file'});

	# Create directory for FTP root
	$self->setup_ftp_rootdir($_[0], $ftp);
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	$self->register_post_action(\&restart_proftpd);
	undef(@{$main::proftpd_object->{'get_config_cache_array'}});
	}
elsif ($self->{'config_hash'}->{'ftp_system'} == 2) {
	# Set VsFTPd up for FTP feature
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_vsftpd'})};
  
	# Create directory for FTP root
	$self->setup_ftp_rootdir($_[0], $ftp);

	# Define vsftpd virtual server config file
	local $conf_file = "$main::vsftpd_object->{'config_hash'}->{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  
	if ($tmpl->{'ftp_vsftpd'}) {
		# Get and use the user-defined template
		local %vars = %{$_[0]};
		$vars{'serial'} = $self->{'serial'};
		$self->open_tempfile(CONF, ">$conf_file");
		local $conf_ctx = $self->vsftpd_template($tmpl->{'ftp_vsftpd'}, \%vars);
# 		local $conf_ctx = &substitute_template($self->{'config_hash'}->{'ftp_vsftpd'}, \%vars);
# 		$self->print_tempfile(CONF, $conf_ctx);
		$self->print_tempfile(CONF, $conf_ctx);
		$self->close_tempfile(CONF);
		}
	else {
		# Copy default VSFTPd server configuration file
		$self->copy_source_dest($main::vsftpd_object->{'config_hash'}->{'vsftpd_conf'}, $conf_file);
  
		# Set new VSFTPd virtual server configuration options
		local $conf = $main::vsftpd_object->parse_config_file($conf_file);
		$main::vsftpd_object->set_config_value('listen_address', $_[0]->{'ip'}, $conf);
		$main::vsftpd_object->set_config_value('anon_root', $ftp, $conf);
		$main::vsftpd_object->set_config_value('xferlog_file',"$_[0]->{'home'}/logs/ftp.log",$conf);
		$main::vsftpd_object->set_config_value('ftpd_banner',"Welcome to $_[0]->{'dom'}",$conf);
		#$main::vsftpd_object->save_config_file($conf_file,$conf);
		$main::vsftpd_object->save_new_options($conf,$conf_file);
		}

	#restart daemon vsftpd
	$main::vsftpd_object->restart_vsftpd() if $main::vsftpd_object->status_vsftpd();
  
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
}

# delete_ftp(&domain)
sub delete_ftp
{
	my $self = shift;

 $self->require_ftp();
 if ($self->{'config_hash'}->{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # Delete the virtual server from the ProFTPd config
  local $conf = $main::proftpd_object->get_config();
  do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'delete_proftpd'})};
  local ($virt, $vconf) = $self->get_proftpd_virtual($_[0]->{'ip'});
  if ($virt) {
	$self->lock_file($virt->{'file'});
	local $lref = $self->read_file_lines($virt->{'file'});
	splice(@$lref, $virt->{'line'}, $virt->{'eline'} - $virt->{'line'} + 1);
	$self->flush_file_lines();
	$self->unlock_file($virt->{'file'});

	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	$self->register_post_action(\&restart_proftpd);
	undef(@{$main::proftpd_object->{'get_config_cache_array'}});
	}
  else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'delete_noproftpd'})};
	}
 }
 elsif ($self->{'config_hash'}->{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'delete_proftpd'})};
  local $conf_file = "$main::vsftpd_object->{'config_hash'}->{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  $conf_file = "$conf_file.disable" unless (-e $conf_file);
  if (-r $conf_file) {
	#delete the vsftpd domain config
	$self->unlink_logged($conf_file);
	
	#restart daemon vsftpd
  	$main::vsftpd_object->restart_vsftpd() if $main::vsftpd_object->status_vsftpd();
	}
  do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
 }
}

# modify_ftp(&domain, &olddomain)
sub modify_ftp
{
	my $self = shift;

 $self->require_ftp();
 if ($self->{'config_hash'}->{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # If the server has changed IP address, update the ProFTPd virtual server
  local $rv = 0;
  local $conf = $main::proftpd_object->get_config();
  local ($virt, $vconf, $anon, $aconf) = $self->get_proftpd_virtual($_[1]->{'ip'});
  return 0 if (!$virt);
  $self->lock_file($virt->{'file'});
  if ($_[0]->{'dom'} ne $_[1]->{'dom'}) {
	# Update domain name in ProFTPd virtual server
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_proftpd2'})};
	local $sname = $main::proftpd_object->find_directive_struct("ServerName", $vconf);
	if ($sname) {
		$main::proftpd_object->save_directive(
			"ServerName", [ $_[0]->{'dom'} ], $vconf, $conf);
		}
	$rv++;
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
  if ($_[0]->{'home'} ne $_[1]->{'home'} && $anon) {
	# Update anonymous FTP directory in ProFTPd virtual server
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_proftpd3'})};
	local $lref = $self->read_file_lines($anon->{'file'});
	$lref->[$anon->{'line'}] =~ s/$_[1]->{'home'}/$_[0]->{'home'}/;
	$self->flush_file_lines();
	$rv++;
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
  if ($_[0]->{'ip'} ne $_[1]->{'ip'}) {
	# Update IP address in ProFTPd virtual server
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_proftpd'})};
	local $lref = $self->read_file_lines($virt->{'file'});
	$lref->[$virt->{'line'}] = "<VirtualHost $_[0]->{'ip'}>";
	$self->flush_file_lines();
	$rv++;
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
  $self->unlock_file($virt->{'file'});
  $self->register_post_action(\&restart_proftpd) if ($rv);
  return $rv;
 }
 elsif ($self->{'config_hash'}->{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  local $conf_file = "$main::vsftpd_object->{'config_hash'}->{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  local $old_file = "$main::vsftpd_object->{'config_hash'}->{'vsftpd_dir'}/vsftpd.$_[1]->{'dom'}.conf";
  
  my $change;
  if ($_[0]->{'dom'} ne $_[1]->{'dom'}) {
	# Update domain name in VsFTPd virtual server
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_proftpd2'})};
	$self->rename_logged($old_file, $conf_file);
	
	#set new vsftpd config options
	my $conf = $main::vsftpd_object->parse_config_file($conf_file);
	$main::vsftpd_object->set_config_value('ftpd_banner',"Welcome to $_[0]->{'dom'}",$conf);
	#$main::vsftpd_object->save_config_file($conf_file, $conf);
	$main::vsftpd_object->save_new_options($conf,$conf_file);
	
	$change = 1;
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
  if ($_[0]->{'home'} ne $_[1]->{'home'}) {
	# Update anonymous FTP directory in VsFTPd virtual server
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_proftpd3'})};
	
	#set new vsftpd config options
	my $conf = $main::vsftpd_object->parse_config_file($conf_file);
	$main::vsftpd_object->set_config_value('xferlog_file',"$_[0]->{'home'}/logs/ftp.log",$conf);
	#$main::vsftpd_object->save_config_file($conf_file,$conf);
	$main::vsftpd_object->save_new_options($conf,$conf_file);
	
	$change = 1;
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
  if ($_[0]->{'ip'} ne $_[1]->{'ip'}) {
	# Update IP address in VsFTPd virtual server
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_proftpd'})};
	
	#set new vsftpd config options
	my $conf = $main::vsftpd_object->parse_config_file($conf_file);
	$main::vsftpd_object->set_config_value('listen_address',$_[0]->{'ip'},$conf);
	#$main::vsftpd_object->save_config_file($conf_file,$conf);
	$main::vsftpd_object->save_new_options($conf,$conf_file);
	
	$change = 1;
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
  #restart daemon vsftpd
  $main::vsftpd_object->restart_vsftpd() if ($change && $main::vsftpd_object->status_vsftpd());
 }
}

# disable_ftp(&domain)
sub disable_ftp
{
	my $self = shift;

 $self->require_ftp();
 if ($self->{'config_hash'}->{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # Disable FTP for this server by adding a deny directive
  do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'disable_proftpd'})};
  local ($virt, $vconf, $anon, $aconf) = $self->get_proftpd_virtual($_[0]->{'ip'});
  if ($anon) {
	$self->lock_file($anon->{'file'});
	local @limit = $main::proftpd_object->find_directive_struct("Limit", $aconf);
	local ($login) = grep { $_->{'words'}->[0] eq "LOGIN" } @limit;
	if (!$login) {
		local $lref = $self->read_file_lines($anon->{'file'});
		splice(@$lref, $anon->{'eline'}, 0,
		       "<Limit LOGIN>", "DenyAll", "</Limit>");
		$self->flush_file_lines();
		}
	$self->unlock_file($anon->{'file'});
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	$self->register_post_action(\&restart_proftpd);
	}
  else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'delete_noproftpd'})};
	}
 }
 elsif ($self->{'config_hash'}->{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'disable_proftpd'})};
  
  #rename config file
  local $conf_file = "$main::vsftpd_object->{'config_hash'}->{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  $self->rename_file($conf_file,"$conf_file.disable");
  
  #restart daemon vsftpd
  $main::vsftpd_object->restart_vsftpd() if $main::vsftpd_object->status_vsftpd();
  
  do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
 }
}

# enable_ftp(&domain)
sub enable_ftp
{
	my $self = shift;

 $self->require_ftp();
 if ($self->{'config_hash'}->{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # Enable FTP for this server by removing the deny directive
  do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'enable_proftpd'})};
  local ($virt, $vconf, $anon, $aconf) = $self->get_proftpd_virtual($_[0]->{'ip'});
  if ($virt) {
	$self->lock_file($anon->{'file'});
	local @limit = $main::proftpd_object->find_directive_struct("Limit", $aconf);
	local ($login) = grep { $_->{'words'}->[0] eq "LOGIN" } @limit;
	if ($login) {
		local $lref = $self->read_file_lines($anon->{'file'});
		splice(@$lref, $login->{'line'},
		       $login->{'eline'} - $login->{'line'} + 1);
		$self->flush_file_lines();
		}
	$self->unlock_file($anon->{'file'});
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	$self->register_post_action(\&restart_proftpd);
	}
  else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'delete_noproftpd'})};
	}
 }
 elsif ($self->{'config_hash'}->{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'enable_proftpd'})};
  
  #rename config file
  local $conf_file = "$main::vsftpd_object->{'config_hash'}->{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  $self->rename_file("$conf_file.disable",$conf_file);
  
  #restart daemon vsftpd
  $main::vsftpd_object->restart_vsftpd() if $main::vsftpd_object->status_vsftpd();
  
  do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
 }
}

# check_ftp_clash(&domain, [field])
sub check_ftp_clash
{
	my $self = shift;

 $self->require_ftp();
 if ($self->{'config_hash'}->{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # Returns 1 if a ProFTPd server already exists for some domain
  if (!$_[1] || $_[1] eq 'ip') {
	local ($cvirt, $cconf) = $self->get_proftpd_virtual($_[0]->{'ip'});
	return $cvirt ? 1 : 0;
	}
  return 0;
 }
 elsif ($self->{'config_hash'}->{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  # Returns 1 if a VsFTPd server already exists for some domain
  local $conf_file = "$main::vsftpd_object->{'config_hash'}->{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  return 1 if (-r $conf_file);
  return 0;
 }
}

# backup_ftp(&domain, file)
sub backup_ftp
{
	my $self = shift;

 $self->require_ftp();
 if ($self->{'config_hash'}->{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # Save the virtual server's ProFTPd config as a separate file
  do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'backup_proftpdcp'})};
  local ($virt, $vconf) = $self->get_proftpd_virtual($_[0]->{'ip'});
  local $lref = $self->read_file_lines($virt->{'file'});
  local $l;
  $self->open_tempfile(FILE, ">$_[1]");
  foreach $l (@$lref[$virt->{'line'} .. $virt->{'eline'}]) {
	$self->print_tempfile(FILE, "$l\n");
	}
  $self->close_tempfile(FILE);
  do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
  return 1;
 }
 elsif ($self->{'config_hash'}->{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  # Save the virtual server's VsFTPd config as a separate file
  do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'backup_proftpdcp'})};
  
  #backup domain vsftpd config
  local $conf_file = "$main::vsftpd_object->{'config_hash'}->{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  $conf_file = "$conf_file.disable" unless (-e $conf_file);
  $self->copy_source_dest($conf_file,"$_[1]");
  
  do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
  return 1;
 }
}

# restore_ftp(&domain, file)
sub restore_ftp
{
	my $self = shift;

 $self->require_ftp();
 if ($self->{'config_hash'}->{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # Update the virtual server's ProFTPd configuration from a file. Does not
  # change the actual <Virtualhost> lines!
  do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_proftpdcp'})};
  local ($virt, $vconf) = $self->get_proftpd_virtual($_[0]->{'ip'});
  local $srclref = $self->read_file_lines($_[1]);
  local $dstlref = $self->read_file_lines($virt->{'file'});
  $self->lock_file($virt->{'file'});
  splice(@$dstlref, $virt->{'line'}+1, $virt->{'eline'}-$virt->{'line'}-1,
       @$srclref[1 .. @$srclref-2]);
  $self->flush_file_lines();
  $self->unlock_file($virt->{'file'});
  do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};

  $self->register_post_action(\&restart_proftpd);
  return 1;
 }
 elsif ($self->{'config_hash'}->{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  # Restore the virtual server's VsFTPd configuration from a file
  do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_proftpdcp'})};
  
  #restore domain vsftpd config
  local $conf_file = "$main::vsftpd_object->{'config_hash'}->{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  $self->copy_source_dest($_[1],$conf_file);
  
  do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
  
  &disable_ftp if ($_[0]->{'disabled'});
  
  return 1;
 }
}

# bandwidth_ftp(&domain, start, &bw-hash)
# Searches through FTP log files for records after some date, and updates the
# day counters in the given hash
sub bandwidth_ftp
{
	my $self = shift;

local $log;
if ($self->{'config_hash'}->{'ftp_system'} == 2) {
	# Using VSFTPd for FTP
	$log = $self->get_vsftpd_log($_[0]->{'dom'});
	}
else {
	# Using ProFTPD for FTP
	$log = $self->get_proftpd_log($_[0]->{'ip'});
	}
if ($log) {
	return $self->count_ftp_bandwidth($log, $_[1], $_[2], undef, "ftp");
	}
else {
	return $_[1];
	}
}

# proftpd_template(text, &domain)
# Returns a suitably substituted ProFTPd template
sub proftpd_template
{
	my $self = shift;

local $dirs = $_[0];
$dirs =~ s/\t/\n/g;
$dirs = $self->substitute_template($dirs, $_[1]);
local @dirs = split(/\n/, $dirs);
return @dirs;
}

# vsftpd_template(text, &domain)
# Returns a suitably substituted VSFTPd template
sub vsftpd_template
{
	my $self = shift;

local $opts = $_[0];
$opts =~ s/\t/\n/g;
$opts = $self->substitute_template($opts, $_[1]);
return $opts;
}

# check_proftpd_template([directives])
# Returns an error message if the default ProFTPd directives don't look valid
sub check_proftpd_template
{
	my $self = shift;

local ($d, $gotuser, $gotgroup);
local @dirs = split(/\t+/, defined($_[0]) ? $_[0] : $self->{'config_hash'}->{'proftpd_config'});
foreach $d (@dirs) {
	$d =~ s/#.*$//;
	if ($d =~ /^\s*User\s+(\S+)$/i) {
		defined(getpwnam($1)) ||
			return $self->text('fcheck_euserex', "<tt>$1</tt>");
		$gotuser++;
		}
	elsif ($d =~ /^\s*Group\s+(\S+)$/i) {
		defined(getgrnam($1)) ||
			return $self->text('fcheck_egroupex', "<tt>$1</tt>");
		$gotgroup++;
		}
	}
$gotuser || return $self->{'text_hash'}->{'fcheck_euser'};
$gotgroup || return $self->{'text_hash'}->{'fcheck_egroup'};
return undef;
}

# restart_proftpd()
# Tell ProFTPd to re-read its config file. Does nothing if run from inetd
sub restart_proftpd
{
	my $self = shift;

$self->require_ftp();
local $conf = $main::proftpd_object->get_config();
local $st = $main::proftpd_object->find_directive("ServerType", $conf);
if (lc($st) ne "inetd") {
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_proftpdpid'})};
	local $pidfile = $main::proftpd_object->{'config_hash'}->{'pid_file'};
	local $rv = 0;
	if (open(PID, $pidfile) && <PID> =~ /(\d+)/ && kill(0, $1)) {
		$self->kill_logged('HUP', $1);
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	else {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_notrun'})};
		$rv = 1;
		}
	close(PID);
	return $rv;
	}
}

# get_proftpd_virtual(ip)
# Returns the list of configuration directives and the directive for the
# virtual server itself for some domain
sub get_proftpd_virtual
{
	my $self = shift;

$self->require_ftp();
local $conf = $main::proftpd_object->get_config();
local $v;
foreach $v ($main::proftpd_object->find_directive_struct("VirtualHost", $conf)) {
	if ($v->{'words'}->[0] eq $_[0]) {
		# Found it! Looks for 
		local $a = $main::proftpd_object->find_directive_struct("Anonymous", $v->{'members'});
		if ($a) {
			return ($v, $v->{'members'}, $a, $a->{'members'});
			}
		else {
			return ($v, $v->{'members'});
			}
		}
	}
return ();
}

# get_proftpd_log(ip)
# Given a virtual server IP, returns the path to its log file. If no IP is
# give, returns the global log file path.
sub get_proftpd_log
{
	my $self = shift;

$self->require_ftp();
if ($_[0]) {
	# Find by IP
	local ($virt, $vconf) = $self->get_proftpd_virtual($_[0]);
	if ($virt) {
		return $main::proftpd_object->find_directive("ExtendedLog", $vconf) ||
		       $main::proftpd_object->find_directive("TransferLog", $vconf);
		}
	}
else {
	# Just return global log
	local $conf = $main::proftpd_object->get_config();
	local $global = $main::proftpd_object->find_directive_struct("Global", $conf);
	return $main::proftpd_object->find_directive("ExtendedLog", $global->{'members'}) ||
	       $main::proftpd_object->find_directive("TransferLog", $global->{'members'});
	}
return undef;
}

# get_vsftpd_log(ip)
# Given a virtual server IP, returns the path to its log file. If no IP is
# give, returns the global log file path.
sub get_vsftpd_log
{
	my $self = shift;

$self->require_ftp();
 if ($_[0])
 {
  local $conf_file = "$main::vsftpd_object->{'config_hash'}->{'vsftpd_dir'}/vsftpd.$_[0].conf";
  if( -r $conf_file) {
      my $conf = $main::vsftpd_object->parse_config_file($conf_file);
      return $main::vsftpd_object->get_config_value('xferlog_file',$conf);
      }
 }
 else
 {
  my $conf = $main::vsftpd_object->parse_config_file();
  return  $main::vsftpd_object->get_config_value('xferlog_file',$conf) ||
          "/var/log/vsftpd.log"; 
 }
 return undef;
}

# proftpd_installed()
# Returns 1 if proftpd is installed
sub proftpd_installed
{
	my $self = shift;

 return $self->foreign_installed("proftpd", 1) == 2;
}

# vsftpd_installed()
# Returns 1 if postfix is installed
sub vsftpd_installed
{
	my $self = shift;

 return $self->foreign_installed("vsftpd", 1) == 2;
}


1;

