#!/usr/bin/perl
package virtual_server::Featurelogrotate;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'virtual_server::Featurelogrotate');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for managing logrotate

$self->{'feature_depends_hash'}->{'logrotate'} = [ 'web' ];

1;

1;
		return $self;
}

sub require_logrotate
{
	my $self = shift;

return if ($self->{'require_logrotate'}++);
$self->foreign_require("logrotate", "logrotate-lib.pl");
}

# setup_logrotate(&domain)
# Create logrotate entries for the server's access and error logs
sub setup_logrotate
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_logrotate'})};
$self->require_logrotate();
$self->require_apache();

local $alog = $self->get_apache_log($_[0]->{'dom'}, $_[0]->{'web_port'}, 0);
local $elog = $self->get_apache_log($_[0]->{'dom'}, $_[0]->{'web_port'}, 1);
local @logs = ( $alog, $elog );
if ($_[0]->{'ftp'} && $self->{'config_hash'}->{'ftp_system'}==1) {
	push(@logs, $self->get_proftpd_log($_[0]->{'ip'}));
	}
elsif ($_[0]->{'ftp'} && $self->{'config_hash'}->{'ftp_system'}==2) {
	push(@logs, $self->get_vsftpd_log($_[0]->{'dom'}));
	}
local @logs = $self->unique(grep { $_ } @logs);
if (@logs) {
	local $parent = $main::logrotate_object->get_config_parent();
	local $apachectl = $main::apache_object->{'config_hash'}->{'apachectl_path'} ||
			   $self->has_command("apachectl") ||
			   $self->has_command("apache2ctl");
	local $script = $main::apache_object->{'config_hash'}->{'graceful_cmd'} ||
			"$apachectl graceful";
	local $lconf = {
		       'file' => $main::logrotate_object->UNIVERSAL::can("get_add_file") ?
                                        $main::logrotate_object->get_add_file() :
                                        $parent->{'file'},
		       'name' => \@logs,
		       'members' => [
			{ 'name' => 'rotate',
			  'value' => $self->{'config_hash'}->{'logrotate_num'} || 5 },
			{ 'name' => 'weekly' },
			{ 'name' => 'compress' },
			{ 'name' => 'postrotate',
			  'script' => $script }
			]
		      };
	$self->lock_file($lconf->{'file'});
	$main::logrotate_object->save_directive($parent, undef, $lconf);
	$self->flush_file_lines();
	$self->unlock_file($lconf->{'file'});
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_nolog'})};
	}
}

# modify_logrotate(&domain, &olddomain)
# Adjust path if home directory has changed
sub modify_logrotate
{
	my $self = shift;

if ($_[0]->{'home'} ne $_[1]->{'home'}) {
	$self->require_logrotate();
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_logrotate'})};

	# Work out the *old* access log, which will have already been renamed
	local $oldalog = $self->get_apache_log($_[0]->{'dom'}, $_[0]->{'web_port'});
	$oldalog =~ s/$_[0]->{'home'}/$_[1]->{'home'}/;
	local $lconf = $self->get_logrotate_section($oldalog);

	if ($lconf) {
		local $parent = $main::logrotate_object->get_config_parent();
		local $n;
		foreach $n (@{$lconf->{'name'}}) {
			$n =~ s/$_[1]->{'home'}/$_[0]->{'home'}/g;
			}
		$main::logrotate_object->save_directive($parent, $lconf, $lconf);
		$self->flush_file_lines();
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	else {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_nologrotate'})};
		}
	}
}

# delete_logrotate(&domain)
# Remove logrotate section for this domain
sub delete_logrotate
{
	my $self = shift;

$self->require_logrotate();
do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'delete_logrotate'})};
local $lconf = $self->get_logrotate_section($_[0]);
if ($lconf) {
	local $parent = $main::logrotate_object->get_config_parent();
	$main::logrotate_object->save_directive($parent, $lconf, undef);
	$self->flush_file_lines();
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_nologrotate'})};
	}
}

# get_logrotate_section(&domain|log-file)
sub get_logrotate_section
{
	my $self = shift;

$self->require_logrotate();
$self->require_apache();
local $alog = ref($_[0]) ? $self->get_apache_log($_[0]->{'dom'}, $_[0]->{'web_port'})
			 : $_[0];
local $conf = $main::logrotate_object->get_config();
local ($c, $n);
foreach $c (@$conf) {
	foreach $n (@{$c->{'name'}}) {
		return $c if ($n eq $alog);
		}
	}
return undef;
}

# check_logrotate_clash()
# No need to check for clashes ..
sub check_logrotate_clash
{
	my $self = shift;

return 0;
}

# backup_logrotate(&domain, file)
# Saves the log rotation section for this domain to a file
sub backup_logrotate
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'backup_logrotatecp'})};
local $lconf = $self->get_logrotate_section($_[0]);
if ($lconf) {
	local $lref = $self->read_file_lines($lconf->{'file'});
	local $l;
	$self->open_tempfile(FILE, ">$_[1]");
	foreach $l (@$lref[$lconf->{'line'} .. $lconf->{'eline'}]) {
		$self->print_tempfile(FILE, "$l\n");
		}
	$self->close_tempfile(FILE);
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	return 1;
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_nologrotate'})};
	return 0;
	}
}

# restore_logrotate(&domain, file)
sub restore_logrotate
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_logrotatecp'})};
local $lconf = $self->get_logrotate_section($_[0]);
if ($lconf) {
	local $srclref = $self->read_file_lines($_[1]);
	local $dstlref = $self->read_file_lines($lconf->{'file'});
	$self->lock_file($lconf->{'file'});
	splice(@$dstlref, $lconf->{'line'}+1,
	       $lconf->{'eline'}-$lconf->{'line'}-1,
	       @$srclref[1 .. @$srclref-2]);
	$self->flush_file_lines();
	$self->unlock_file($lconf->{'file'});
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	return 1;
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_nologrotate'})};
	return 0;
	}
}


1;

