#!/usr/bin/perl
package virtual_server::Featuremailman;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'virtual_server::Featuremailman');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for managing Mailman

1;

1;
		return $self;
}

# &require_mailman()
# require mailman module library & config
sub require_mailman
{
	my $self = shift;

 return if ($self->{'require_mailman'}++);
 $self->foreign_require("mailman", "mailman-lib.pl");
}

# &mailman_installed()
# Returns 1 if mailman is installed
sub mailman_installed
{
	my $self = shift;

 $self->require_mailman();
 return 1 unless $main::mailman_object->mailman_check();
 return 0;
}

# &setup_mailman(&domain)
# Add new Mailman virtual host
sub setup_mailman
{
	my $self = shift;

 do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_mailman'})};
 $self->require_mailman();
 $self->add_mailman_virtual($_[0]->{'dom'});
 do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# &delete_mailman(&domain)
# Delete Mailman virtual host & all domain lists
sub delete_mailman
{
	my $self = shift;

 do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'delete_mailman'})};
 $self->require_mailman();
 $self->delete_domain_lists($_[0]->{'dom'}); 
 do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# &modify_mailman(&domain, &olddomain)
# Modify Mailman virtual host & all domain lists
sub modify_mailman
{
	my $self = shift;

 do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'modify_mailman'})};
 $self->require_mailman();
 if ($_[0]->{'dom'} ne $_[1]->{'dom'})
 {
  # re-crete all maillists
  $self->modify_domain_lists($_[0]->{'dom'}, $_[1]->{'dom'}); 
 }
 do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# &disable_mailman(&domain)
# Delete Mailman virtual host
sub disable_mailman
{
	my $self = shift;

 do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'disable_mailman'})};
 $self->require_mailman();
 
 # delete Mailman virtual host
 $self->delete_mailman_virtual($_[0]->{'dom'});
 
 # regenerate aliases
 $main::mailman_object->regenerate_aliases();
 
 do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# &enable_mailman(&domain)
# Add Mailman virtual host
sub enable_mailman
{
	my $self = shift;

 do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'enable_mailman'})};
 $self->require_mailman();
 
 # add Mailman virtual host
 $self->add_mailman_virtual($_[0]->{'dom'});
 
 # regenerate aliases
 $main::mailman_object->regenerate_aliases();
 do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# &backup_mailman(&domain, file, &options)
# Backup domain mailing list
sub backup_mailman
{
	my $self = shift;

 # Create dummy file
 $self->open_tempfile(FILE, ">$_[1]");
 $self->close_tempfile(FILE);
 
 do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'backup_mailman'})};
 $self->require_mailman();
 my $dom = $self->trunc_space($_[0]->{'dom'});
 my @lists = $main::mailman_object->list_lists($dom);
 $self->open_tempfile(AFILE, ">$_[1]_lists");
 foreach (@lists) 
 {
  my $name = $_->{'name'};
  $self->print_tempfile(AFILE, "$name:$_->{'dom'}:$_->{'email'}:$_->{'lang'}\n");
  my @mems = $main::mailman_object->list_members($_);
  if (@mems)
  {
   $self->open_tempfile(BFILE, ">$_[1]_members_$name");
   foreach my $i (@mems)
   {
    $self->print_tempfile(BFILE, "$i->{'email'}:$i->{'digest'}\n");
   }
   $self->close_tempfile(BFILE);
  }
 }
 $self->close_tempfile(AFILE);
 do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
 return 1;
}

# &restore_mailman(&domain, file, &options)
sub restore_mailman
{
	my $self = shift;

 do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_mailman'})};
 $self->require_mailman();
 open(UFILE, "$_[1]_lists");
 while(<UFILE>) 
 {
  chomp;
  my %maillist;
  ($maillist{'name'}, $maillist{'dom'}, $maillist{'email'}, $maillist{'lang'}) = split (/:/,$_);
  $maillist{'pass'} = 1;
  $main::mailman_object->create_list(\%maillist, 1, 1);
    
  # add members
  open(ZFILE, "$_[1]_members_$maillist{'name'}");
  my $fullname = $maillist{'name'}.".".$maillist{'dom'};
  my (%mems, @digest, @nodigest);
  while(my $f = <ZFILE>) 
  {
   chomp;
   my ($email, $digest) = split (/:/, $f);
   if ($digest eq 'y') { push (@digest, $email); }
	else { push (@nodigest, $email); }
  }
  close(ZFILE);
  $main::mailman_object->add_members(\@digest, $fullname) if @digest;
  $main::mailman_object->add_members(\@nodigest, $fullname, 1) if @nodigest;
  
  # change password
  $main::mailman_object->chpass_list($fullname);
 }
 close(UFILE);
 
 # regenerate aliases
 $main::mailman_object->regenerate_aliases();
 
 if ($self->{'config_hash'}->{'mail_system'} == 0)
 {
  # regenerate postfix virtual table
  $main::postfix_object->regenerate_virtual_table();
  #$main::postfix_object->reload_postfix();
 }

 do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
 return 1;
}

# check_mailman_clash(&domain, [changing])
# Always returns 0
sub check_mailman_clash
{
	my $self = shift;

 return 0;
}

# &add_mailman_virtual($domain)
# add new Mailman virtual host
sub add_mailman_virtual
{
	my $self = shift;

 my $dom = $self->trunc_space($_[0]);
 
 # add virtual host record into mailman configuration
 my $conf = $main::mailman_object->parse_config_file();
 my $new_conf;
 if ($self->{'config_hash'}->{'mail_system'} == 0)
 {
  # Postfix
  $conf->{'POSTFIX_STYLE_VIRTUAL_DOMAINS'} =~ /\[(.+?)\]/;
  my $domains = $1;
  my @domains = split (/,\s*?/, $domains) if $domains;
  push (@domains, "\'$dom\'");
  $domains = join (',',@domains);
 
  # change mailman config file
  $new_conf = { 'POSTFIX_STYLE_VIRTUAL_DOMAINS' => "\[$domains\]",
					"add_virtualhost(\'www.$dom\', \'$dom\')"
				}; 
 }
 elsif ($self->{'config_hash'}->{'mail_system'} == 1)
 {
  # Sendmail
  $new_conf = { "add_virtualhost(\'www.$dom\', \'$dom\')"}; 
 }
 # change mailman config file
 $main::mailman_object->save_new_options($new_conf);  
 
 if ($main::mailman_object->is_mailman_running)
 {
  # restart mailman
  $main::mailman_object->restart_mailman();
 }
}

# &modify_domain_lists($dom, $old_dom)
# modify mailing list for some domain
sub modify_domain_lists
{
	my $self = shift;

 my $dom = $self->trunc_space($_[0]);
 my $old_dom = $self->trunc_space($_[1]);
 my @lists = $main::mailman_object->list_lists($old_dom);
 foreach my $j (@lists)
 {
  # get mailman lists
  my @mems = $main::mailman_object->list_members($j);
  
  # delete old lists
  $main::mailman_object->delete_list($j, 1, 1); 
  my $fullname = $j->{'name'}.".$dom";
  $main::mailman_object->delete_postfix_map($j->{'name'}, $j->{'list'}, $dom, 1);
  
  # create new lists
  $j->{'dom'} = $dom;
  $j->{'pass'} = 1;
  $main::mailman_object->create_list($j, 1, 1);
  
  # add members
  my $fullname = $j->{'name'}.".$dom";
  my (@digest, @nodigest);
  foreach my $i (@mems)
  {
   if ($i->{'digest'} eq 'y') { push (@digest, $i->{'email'}); }
	else { push (@nodigest, $i->{'email'}); }
  }
  $main::mailman_object->add_members(\@digest, $fullname) if @digest;
  $main::mailman_object->add_members(\@nodigest, $fullname, 1) if @nodigest;
  
  # change password
  $main::mailman_object->chpass_list($fullname);
 }
 
 # modify mailman config file
 $self->modify_mailman_virtual($dom, $old_dom);
 
 if ($main::mailman_object->is_mailman_running)
 {
  # restart mailman
  $main::mailman_object->restart_mailman();
 }
 
 # regenerate aliases
 $main::mailman_object->regenerate_aliases();
 
 if ($self->{'config_hash'}->{'mail_system'} == 0)
 {
  # regenerate postfix virtual table
  $main::postfix_object->regenerate_virtual_table();
  #$main::postfix_object->reload_postfix();
 }
}

# &modify_mailman_virtual($dom, $old_dom)
# modify mailman config file
sub modify_mailman_virtual
{
	my $self = shift;

 my ($dom, $old_dom) = @_;
 
 # add virtual host record into mailman configuration
 my $conf = $main::mailman_object->parse_config_file();
 my $new_conf;
 if ($self->{'config_hash'}->{'mail_system'} == 0)
 {
  # Postfix
  $conf->{'POSTFIX_STYLE_VIRTUAL_DOMAINS'} =~ /\[(.+?)\]/;
  my $domains = $1;
  if ($domains)
  {
   my @domains = split (/,/, $domains);
	foreach (@domains)
	{
	 $_ = "\'$dom\'" if ($_ =~ /^\s*?'$old_dom'\s*?$/);
	}
   $domains = join (',',@domains);

   # modify virtual host
	$main::mailman_object->remove_mailman_str(["add_virtualhost\s*?\(.*?'$old_dom'.*?\)"]);
   $new_conf = { 'POSTFIX_STYLE_VIRTUAL_DOMAINS' => "\[$domains\]", 
					 "add_virtualhost(\'www.$dom\', \'$dom\')"
				  };
  }
 }
 elsif ($self->{'config_hash'}->{'mail_system'} == 1)
 {
  # Sendmail
  $main::mailman_object->remove_mailman_str(["add_virtualhost\s*?\(.*?'$old_dom'.*?\)"]);
  $new_conf = { "add_virtualhost(\'www.$dom\', \'$dom\')"};
 }
 # modify virtual host
 $main::mailman_object->save_new_options($new_conf);
}

# &delete_domain_lists($dom)
# Delete mailing list for some domain
sub delete_domain_lists
{
	my $self = shift;

 my $dom = $self->trunc_space($_[0]);
 my @lists = $main::mailman_object->list_lists($dom);
 foreach (@lists)
 {
  $main::mailman_object->delete_list($_, 1, 1);
 }
 
 # modify mailman config file
 $self->delete_mailman_virtual($dom);
 
 if ($main::mailman_object->is_mailman_running)
 {
  # restart mailman
  $main::mailman_object->restart_mailman();
 }
 
 # regenerate aliases
 $main::mailman_object->regenerate_aliases();
 
 return undef;
}

# &delete_mailman_virtual($domain)
# modify mailman config file
sub delete_mailman_virtual
{
	my $self = shift;

 my $dom = $_[0];
 
 # add virtual host record into mailman configuration
 my $conf = $main::mailman_object->parse_config_file();
 if ($self->{'config_hash'}->{'mail_system'} == 0)
 {
  # Postfix
  $conf->{'POSTFIX_STYLE_VIRTUAL_DOMAINS'} =~ /\[(.+?)\]/;
  my $domains = $1;
  if ($domains)
  {
   my @d_temp = split (/,/, $domains);
   my @domains = grep { !($_ =~ /^\s*?'$dom'\s*?$/) } @d_temp;
   $domains = join (',',@domains);
  }
  if ($domains)
  {
   # delete virtual host
   my $conf = { 'POSTFIX_STYLE_VIRTUAL_DOMAINS' => "\[$domains\]" };
   $main::mailman_object->save_new_options($conf);
   $main::mailman_object->remove_mailman_str(["add_virtualhost\s*?\(.*?'$dom'.*?\)"]);
  }
  else
  {
   # no any virtual hosts
   $main::mailman_object->remove_mailman_str(['POSTFIX_STYLE_VIRTUAL_DOMAINS', "add_virtualhost\s*?\(.*?'$dom'.*?\)"]);
  }
 }
 elsif ($self->{'config_hash'}->{'mail_system'} == 1)
 {
  # Sendmail
  $main::mailman_object->remove_mailman_str(["add_virtualhost\s*?\(.*?'$dom'.*?\)"]);
 }
}

# trunc_space(string)
# delete spaces from string
sub trunc_space
{
	my $self = shift;

 local $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}


1;

