#!/usr/bin/perl
package virtual_server::Featurepostgres;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'virtual_server::Featurepostgres');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;

1;

1;
		return $self;
}

sub require_postgres
{
	my $self = shift;

return if ($self->{'require_postgres'}++);
$main::postgresql_object->{'use_global_login'} = 1;
$self->foreign_require("postgresql", "postgresql-lib.pl");
%{$self->{'qconfig_hash'}} = $self->foreign_config("postgresql");
}

# check_postgres_clash(&domain, [field])
# Returns 1 if some PostgreSQL database already exists
sub check_postgres_clash
{
	my $self = shift;

if (!$_[1] || $_[1] eq 'db') {
	$self->require_postgres();
	local @dblist = $main::postgresql_object->list_databases();
	return 1 if ($self->indexof($_[0]->{'db'}, @dblist) >= 0);
	}
if (!$_[0]->{'parent'} && (!$_[1] || $_[1] eq 'db')) {
	return 1 if ($self->postgres_user_exists($_[0]) ? 1 : 0);
	}
return 0;
}

# postgres_user_exists(&domain)
# Returns 1 if some user exists in PostgreSQL
sub postgres_user_exists
{
	my $self = shift;

$self->require_postgres();
local $user = $self->postgres_user($_[0]);
local $s = $main::postgresql_object->execute_sql($self->{'qconfig_hash'}->{'basedb'}, "select * from pg_shadow where usename = '$user'");
return $s->{'data'}->[0] ? 1 : 0;
}

# setup_postgres(&domain, [no-dbs])
# Create a new PostgreSQL database and user
sub setup_postgres
{
	my $self = shift;

$self->require_postgres();
local $user = $_[0]->{'postgres_user'} = $self->postgres_user($_[0]);
if (!$_[0]->{'parent'} && !$self->list_users_db($user)) {
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_postgresuser'})};
	local $pass = $self->postgres_pass($_[0]);
	$main::postgresql_object->execute_sql_logged($self->{'qconfig_hash'}->{'basedb'}, "create user \"$user\" with password $pass nocreatedb nocreateuser");
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
if (!$_[1] && $self->{'tmpl'}->{'mysql_mkdb'}) {
	$self->create_postgres_database($_[0], $_[0]->{'db'});
	}
else {
	$_[0]->{'db_postgres'} ||= "";
	}
}

# postgres_pass(&domain, [neverquote])
sub postgres_pass
{
	my $self = shift;

local $pass = defined($_[0]->{'postgres_pass'}) ? $_[0]->{'postgres_pass'}
						: $_[0]->{'pass'};
return !$_[1] && $main::postgresql_object->get_postgresql_version() >= 7 ? "'$pass'" : $pass;
}

# modify_postgres(&domain, &olddomain)
# Change the PostgreSQL user's password if needed
sub modify_postgres
{
	my $self = shift;

$self->require_postgres();
local $user = $self->postgres_user($_[0], 1);
local $olduser = $self->postgres_user($_[1]);
if ($_[0]->{'pass'} ne $_[1]->{'pass'} &&
    !$_[0]->{'parent'} && !$self->{'config_hash'}->{'postgres_nopass'}) {
	# Change PostgreSQL password ..
	local $pass = $self->postgres_pass($_[0]);
	local $oldpass = $self->postgres_pass($_[1]);
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_postgrespass'})};
	if ($self->postgres_user_exists($_[1])) {
		$main::postgresql_object->execute_sql_logged($self->{'qconfig_hash'}->{'basedb'}, "alter user \"$olduser\" with password $pass");
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	else {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'save_nopostgres'})};
		}
	}
if ($_[0]->{'user'} ne $_[1]->{'user'} && !$_[0]->{'parent'}) {
	# Rename PostgreSQL user ..
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_postgresuser'})};
	if ($self->postgres_user_exists($_[1])) {
		if ($main::postgresql_object->get_postgresql_version() >= 7.4) {
			# Can use proper rename command
			$main::postgresql_object->execute_sql_logged($self->{'qconfig_hash'}->{'basedb'}, "alter user \"$olduser\" rename to \"$user\"");
			$_[0]->{'postgres_user'} = $user;
			do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
			}
		else {
			# Cannot
			do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'save_norename'})};
			}
		}
	else {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'save_nopostgres'}." ".$_[1]->{'user'})};
		}
	}
}

# delete_postgres(&domain)
# Delete the PostgreSQL database and user
sub delete_postgres
{
	my $self = shift;

# Delete all databases
$self->require_postgres();
$self->delete_postgres_database($_[0], split(/\s+/, $_[0]->{'db_postgres'}))
	if ($_[0]->{'db_postgres'});
local $user = $self->postgres_user($_[0]);

if (!$_[0]->{'parent'}) {
	# Delete the user
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'delete_postgresuser'})};
	if ($self->postgres_user_exists($_[0]) && !$self->list_users_db($user)) {
		$main::postgresql_object->execute_sql_logged($self->{'qconfig_hash'}->{'basedb'}, "drop user \"$user\"");
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	else {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'save_nopostgres'})};
		}
	}
}

# disable_postgres(&domain)
# Invalidate the domain's PostgreSQL user
sub disable_postgres
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'disable_postgres'})};
local $user = $self->postgres_user($_[0]);
if ($_[0]->{'parent'}) {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'save_nopostgrespar'})};
	}
elsif ($self->postgres_user_exists($_[0])) {
	$self->require_postgres();
	local $date = localtime(0);
	$main::postgresql_object->execute_sql_logged($self->{'qconfig_hash'}->{'basedb'}, "alter user \"$user\" valid until '$date'");
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'save_nopostgres'})};
	}
}

# enable_postgres(&domain)
# Validate the domain's PostgreSQL user
sub enable_postgres
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'enable_postgres'})};
local $user = $self->postgres_user($_[0]);
if ($_[0]->{'parent'}) {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'save_nopostgrespar'})};
	}
elsif ($self->postgres_user_exists($_[0])) {
	$self->require_postgres();
	$main::postgresql_object->execute_sql_logged($self->{'qconfig_hash'}->{'basedb'}, "alter user \"$user\" valid until 'Jan 1 2038'");
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'save_nopostgres'})};
	}
}

# backup_postgres(&domain, file)
# Dumps this domain's postgreSQL database to a backup file
sub backup_postgres
{
	my $self = shift;

$self->require_postgres();

# Find all the domains's databases
local @dbs = split(/\s+/, $_[0]->{'db_postgres'});

# Create empty 'base' backup file
$self->open_tempfile(EMPTY, ">$_[1]");
$self->close_tempfile(EMPTY);

# Back them all up
local $db;
foreach $db (@dbs) {
	do { my $fn = $self->{'first_print'}; $self->$fn($self->text('backup_postgresdump', $db))};
	local $dbfile = $_[1]."_".$db;
	$self->foreign_require("proc", "proc-lib.pl");
	local $pass = $self->transname();
	$self->open_tempfile(PASS, ">$pass");
	$self->print_tempfile(PASS, "$main::postgresql_object->{'postgres_pass'}\n");
	$self->close_tempfile(PASS);
	local $cmd = $main::postgresql_object->{'config_hash'}->{'dump_cmd'}.
	     ($main::postgresql_object->{'postgres_login'} ?
		" -U $main::postgresql_object->{'postgres_login'}" : "").
	     ($main::postgresql_object->{'config_hash'}->{'host'} ?
		" -h $main::postgresql_object->{'config_hash'}->{'host'}" : "").
	     " -F c -b -f ".quotemeta($dbfile)." ".quotemeta($db).
	     " 2>&1 <$pass";
	local $out = `$cmd`;
	if ($?) {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->text('backup_postgresdumpfailed', "<pre>$out</pre>"))};
		return 0;
		}
	else {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	}
return 1;
}

# restore_postgres(&domain, file)
# Restores this domain's postgresql database from a backup file, and re-creates
# the postgresql user.
sub restore_postgres
{
	my $self = shift;

$self->require_postgres();
$self->foreign_require("proc", "proc-lib.pl");
do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_postgresdrop'})};
	{
	local $first_print_tmp = $self->{'first_print'};	# supress messages
	local $second_print_tmp = $self->{'second_print'};
	$self->{'first_print'} = sub {};
	$self->{'second_print'} = sub {};
	$self->require_mysql();

	# First clear out the databases
	$self->delete_postgres($_[0]);

	# Now re-set up the user only
	$self->setup_postgres($_[0], 1);
	$self->{'first_print'} = $first_print_tmp;
	$self->{'second_print'} = $second_print_tmp;
	}
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};

# Work out which databases are in backup
local ($dbfile, @dbs);
push(@dbs, [ $_[0]->{'db'}, $_[1] ]) if (-s $_[1]);
foreach $dbfile (glob("$_[1]_*")) {
	if (-r $dbfile) {
		$dbfile =~ /\Q$_[1]\E_(.*)$/;
		push(@dbs, [ $1, $dbfile ]);
		}
	}

# Finally, import the data
local $db;
foreach $db (@dbs) {
	$self->create_postgres_database($_[0], $db->[0]);
	do { my $fn = $self->{'first_print'}; $self->$fn($self->text('restore_postgresload', $db->[0]))};
	local $pass = $self->transname();
	$self->open_tempfile(PASS, ">$pass");
	$self->print_tempfile(PASS, "$main::postgresql_object->{'postgres_pass'}\n");
	$self->close_tempfile(PASS);
	local $cmd = $main::postgresql_object->{'config_hash'}->{'rstr_cmd'}.
	     ($main::postgresql_object->{'postgres_login'} ?
		" -U $main::postgresql_object->{'postgres_login'}" : "").
	     ($main::postgresql_object->{'config_hash'}->{'host'} ?
		" -h $main::postgresql_object->{'config_hash'}->{'host'}" : "").
	     " -d ".quotemeta($db->[0])." ".quotemeta($db->[1])." <$pass 2>&1";
	local $out = $self->backquote_logged($cmd);
	if ($? || $out =~ /failed|fatal/i) {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->text('restore_mysqlloadfailed', "<pre>$out</pre>"))};
		return 0;
		}
	else {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	}
return 1;
}

# postgres_user(&domain, [always-new])
sub postgres_user
{
	my $self = shift;

return defined($_[0]->{'postgres_user'}) && !$_[1] ?
	$_[0]->{'postgres_user'} : $_[0]->{'user'}; 
}

sub postgres_size
{
	my $self = shift;

$self->require_postgres();
local $size;
local $d = $main::postgresql_object->execute_sql($_[1], "select sum(relpages) from pg_class where relname not like 'pg_%'");
$size = $d->{'data'}->[0]->[0]*1024*2;
local @tables = $main::postgresql_object->list_tables($_[1], 1);
return ($size, scalar(@tables));
}

# check_postgres_database_clash(&domain, db)
# Returns 1 if some database name is already in use
sub check_postgres_database_clash
{
	my $self = shift;

$self->require_postgres();
local @dblist = $main::postgresql_object->list_databases();
return 1 if ($self->indexof($_[1], @dblist) >= 0);
}

# create_postgres_database(&domain, db)
# Create one PostgreSQL database
sub create_postgres_database
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->text('setup_postgresdb', $_[1]))};
$self->require_postgres();
local $user = $self->postgres_user($_[0]);
local $owner = $main::postgresql_object->get_postgresql_version() >= 7 ?
		"with owner=\"$user\"" : "";
$main::postgresql_object->execute_sql_logged($self->{'qconfig_hash'}->{'basedb'}, "create database $_[1] $owner");
local @dbs = split(/\s+/, $_[0]->{'db_postgres'});
push(@dbs, $_[1]);
$_[0]->{'db_postgres'} = join(" ", @dbs);
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# delete_postgres_database(&domain, dbname, ...)
# Delete one PostgreSQL database
sub delete_postgres_database
{
	my $self = shift;

$self->require_postgres();
local @dblist = $main::postgresql_object->list_databases();
do { my $fn = $self->{'first_print'}; $self->$fn($self->text('delete_postgresdb', join(", ", @_[1..$#_])))};
local @dbs = split(/\s+/, $_[0]->{'db_postgres'});
local $db;
foreach $db (@_[1..$#_]) {
	if ($self->indexof($db, @dblist) >= 0) {
		$main::postgresql_object->execute_sql_logged($self->{'qconfig_hash'}->{'basedb'}, "drop database $db");
		}
	@dbs = grep { $_ ne $db } @dbs;
	}
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
$_[0]->{'db_postgres'} = join(" ", @dbs);
}

# &list_users_db(user)
# return array of databases for some owner
sub list_users_db
{
	my $self = shift;

 my $user = $_[0];
 my $d = $main::postgresql_object->execute_sql($main::postgresql_object->{'config_hash'}->{'basedb'}, "select usesysid from pg_user where usename = ?", $user);
 my $id = $d->{'data'}->[0]->[0];
 $d = $main::postgresql_object->execute_sql($main::postgresql_object->{'config_hash'}->{'basedb'}, "select datname from pg_database where datdba = ?", $id); 
 my @db;
 foreach (@{$d->{'data'}}) { push (@db, $_->[0]); }
 return @db;
}


1;

