#!/usr/bin/perl
package virtual_server::Featuresamba;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'virtual_server::Featuresamba');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for managing a domain's Samba client

# bandwidth_mail(&domain, start, &bw-hash)
# Searches through log files for records after some date, and updates the
# day counters in the given hash
# sub bandwidth_samba
# {
# }

1;

1;
		return $self;
}

sub require_samba {
	my $self = shift;

 return if ($self->{'require_samba'}++);
 $self->foreign_require("samba", "samba-lib.pl");
 %{$self->{'smbconfig_hash'}} = $self->foreign_config("samba");
}

# Creates a samba client for a domain owner
# setup_samba(&domain)
sub setup_samba
{
	my $self = shift;

 do { my $fn = $self->{'first_print'}; $self->$fn($self->text('setup_samba_user', $_[0]->{'user'}))};
 $self->require_samba();
 $self->create_samba_client($_[0]->{'user'}, $_[0]->{'pass'});
 do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# delete_samba(&domain)
# Removes a domain owner from the list of samba clients
sub delete_samba
{
	my $self = shift;

 do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'delete_samba'})};
 $self->require_samba();
 # removing all endusers' samba clients
 local @users = $self->list_domain_users($_[0]);
 map { $self->delete_samba_client($self->{'i'}->{'user'}) } reverse(@users);
 # removing domain owner from list of samba clients
 $self->delete_samba_client($_[0]->{'user'});
 do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# Changes the samba client's password if needed (for a domain owner)
# modify_samba(&domain, &olddomain)
sub modify_samba
{
	my $self = shift;

 if ($_[0]->{'pass_set'}) {
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_samba_user'})};
	$self->require_samba();
	$self->create_samba_client($_[0]->{'user'}, $_[0]->{'pass'});
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
 }
}

# Locks a samba client for a domain owner
# disable_samba(&domain)
sub disable_samba
{
	my $self = shift;

 $self->require_samba();
 local $u;
 if ($u = $self->get_samba_user($_[0]->{'user'}))
 {
  do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'disable_samba'})};
  $u->{'disabled'} = 1;
  $self->modify_samba_client($u);
  $self->webmin_log("disable", "sambaclient", $_[0]);
  do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
 }
}

# Unlocks a samba client for a domain owner
# enable_samba(&domain)
sub enable_samba
{
	my $self = shift;

 $self->require_samba();
 local $u;
 if ($u = $self->get_samba_user($_[0]->{'user'}))
 {
  do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'enable_samba'})};
  $u->{'disabled'} = 0;
  $self->modify_samba_client($u);
  $self->webmin_log("enable", "sambaclient", $_[0]);
  do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
 }
}

# Save all the virtual server's 
# backup_samba(&domain, file, &options)
sub backup_samba
{
	my $self = shift;

 return 1;
}

# restore_samba(&domain, file, &options, &all-options)
sub restore_samba
{
	my $self = shift;

 return 1;
}

# check_samba_clash(&domain, [changing])
# Always returns 0
sub check_samba_clash
{
	my $self = shift;

 return 0;
}

# Create samba client or modify it password if it is existed
# create_samba_client(name, passwd);
sub create_samba_client
{
	my $self = shift;

$self->require_samba();
unless ($self->get_samba_user($_[0])) {
	# adding a new samba user
	$self->lock_file($main::samba_object->{'config_hash'}->{'smb_passwd'});
	local @uinfo = getpwnam($_[0]);
 	local @flags = ("UN");
	local $nu = {
		'name' => $uinfo[0],
		'uid' => $uinfo[2],
		'pass1' => "NO PASSWORDXXXXXXXXXXXXXXXXXXXXX",
		'pass2' => "NO PASSWORDXXXXXXXXXXXXXXXXXXXXX",
	};
	if ($main::samba_object->{'samba_version'} < 2) {
		$nu->{'real'} = $uinfo[6];
		$nu->{'home'} = $uinfo[7];
		$nu->{'shell'} = $uinfo[8];
		}
	else {
		$nu->{'opts'} = \@flags;
		}
	$main::samba_object->create_user($nu);
	$self->unlock_file($main::samba_object->{'config_hash'}->{'smb_passwd'});
	$self->webmin_log("add", "sambaclient", $_[0]);
}

# updating samba password
$main::samba_object->set_password($_[0], $_[1]) || $self->error($main::samba_object->{'text_hash'}->{'mkpass_passfail'});
$self->webmin_log("modify_passwd", "sambaclient", $_[0]);
}

# modify_samba_client(&user, [nocreate])
# Update a samba user
sub modify_samba_client
{
	my $self = shift;

$self->require_samba();
local $u;
local @uinfo;
local $create;
local @opts;
$_[0]->{'name'} || $self->error($self->{'text_hash'}->{'edit_samba_noname'});
$u = $self->get_samba_user($_[0]->{'oldname'}) || 
	$self->get_samba_user($_[0]->{'name'}) || 
	($_[2] && $self->error($self->text('edit_samba_nouser', $_[1]->{'name'})));

$create++ unless $u;

if ($_[0]->{'oldname'} && $_[0]->{'name'} ne $_[0]->{'oldname'}) {
	$self->delete_samba_client($_[0]->{'oldname'});
	$create++;
	}

local $passwd = $_[0]->{'plainpass'};
@uinfo = getpwnam($_[0]->{'name'});
$_[0]->{'name'} = $uinfo[0] unless $_[0]->{'name'};
$_[0]->{'uid'} = $uinfo[2] unless $_[0]->{'uid'};
if ($main::samba_object->{'samba_version'} < 2) {
	$_[0]->{'real'} = $uinfo[6] unless $_[0]->{'real'};
	$_[0]->{'home'} = $uinfo[7] unless $_[0]->{'home'};
	$_[0]->{'shell'} = $uinfo[8] unless $_[0]->{'shell'};
}

	# Update details
	$self->lock_file($self->{'config_hash'}->{'smb_passwd'});
	$u->{'uid'} = $_[0]->{'uid'};
	$u->{'name'} = $_[0]->{'name'};
	if ($main::samba_object->{'samba_version'} >= 2) {
		$u->{'opts'} = $u->{'opts'} ? $u->{'opts'} : ["U"];
		local @opts = grep { !/[ND]/ } @{$u->{'opts'}};
		push(@opts, "N") if ($create); # || $_[0]->{'passmode'} == 0);
		push(@opts, "D") if ($_[0]->{'disabled'} || $_[0]->{'passmode'} == 1);
		$u->{'opts'} = \@opts;
		}
	else {
		$u->{'real'} = $_[0]->{'real'};
		$u->{'home'} = $_[0]->{'home'};
		$u->{'shell'} = $_[0]->{'shell'};
		}
	if (!$passwd && $_[0]->{'passmode'} == 0) {
		$u->{'pass1'} = "NO PASSWORDXXXXXXXXXXXXXXXXXXXXX";
		$u->{'pass2'} = $u->{'pass1'};
		}
	elsif (!$passwd && $_[0]->{'passmode'} == 1) {
		$u->{'pass1'} = $u->{'pass2'} = ("X" x 32);
		}
	if ($create) {
		$main::samba_object->create_user($u);
		$self->webmin_log("create", "sambaclient", $_[0]);
	} 
	else {
		$main::samba_object->modify_user($u);
		$self->webmin_log("modify", "sambaclient", $_[0]);
	}
	if ($passwd && ($create || $_[0]->{'passmode'} == 3)) {
		$main::samba_object->set_password($_[0]->{'name'}, $passwd);
		$self->webmin_log("modify_passwd", "sambaclient", $_[0]);
		}
	$self->unlock_file($self->{'config_hash'}->{'smb_passwd'});
}

# Deletes samba client
# delete_samba_client(username);
sub delete_samba_client
{
	my $self = shift;

$self->require_samba();
local $u;
if ($u = $self->get_samba_user($_[0])) {
	# Delete the user
	$self->lock_file($main::samba_object->{'config_hash'}->{'smb_passwd'});
	$main::samba_object->delete_user($u);
	$self->unlock_file($main::samba_object->{'config_hash'}->{'smb_passwd'});
	$self->webmin_log("delete", "sambaclient", $_[0]);
	}
}

# Gets Samba client
# get_samba_user(username);
sub get_samba_user
{
	my $self = shift;

$self->require_samba();
local @ulist = $main::samba_object->list_users();
local $u;
foreach $u (@ulist) {
	return $u if ($u->{'name'} eq $_[0]);
	}
return undef;
}


1;

