#!/usr/bin/perl
package virtual_server::Featureunix;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'virtual_server::Featureunix');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# Functions for managing a domain's Unix user

$self->{'feature_depends_hash'}->{'unix'} = [ 'dir' ];

1;

1;
		return $self;
}

# setup_unix(&domain)
# Creates the Unix user and group for a domain
sub setup_unix
{
	my $self = shift;

local $tmpl = $self->get_template($_[0]->{'template'});
$self->require_useradmin();
local (%uinfo, %ginfo);

if ($self->mail_system_needs_group() || $_[0]->{'gid'} == $_[0]->{'ugid'}) {
	# Create the group
	do { my $fn = $self->{'first_print'}; $self->$fn($self->text('setup_group', $_[0]->{'group'}))};
	$self->foreign_call($self->{'usermodule'}, "lock_user_files");
	%ginfo = ( 'group', $_[0]->{'group'},
		   'gid', $_[0]->{'gid'} );
	$self->foreign_call($self->{'usermodule'}, "set_group_envs", \%ginfo, 'CREATE_GROUP');
	$self->foreign_call($self->{'usermodule'}, "making_changes");
	$self->foreign_call($self->{'usermodule'}, "create_group", \%ginfo);
	$self->foreign_call($self->{'usermodule'}, "made_changes");
	if (!defined(getgrnam($_[0]->{'group'}))) {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_ecrgroup'})};
		exit;
		}
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
else {
	# Server has no group!
	delete($_[0]->{'gid'});
	delete($_[0]->{'group'});
	}

# Then the user
do { my $fn = $self->{'first_print'}; $self->$fn($self->text('setup_user', $_[0]->{'user'}))};
%uinfo = ( 'user', $_[0]->{'user'},
	   'uid', $_[0]->{'uid'},
	   'gid', $_[0]->{'ugid'},
	   'pass', $self->foreign_call($self->{'usermodule'}, "encrypt_password", $_[0]->{'pass'}),
	   'real', $_[0]->{'owner'},
	   'home', $_[0]->{'home'},
	   'shell', $self->{'config_hash'}->{'unix_shell'},
	   'mailbox', $_[0]->{'user'},
	   'dom', $_[0]->{'dom'},
	   'dom_prefix', substr($_[0]->{'dom'}, 0, 1),
	 );
$self->set_pass_change(\%uinfo);
$self->foreign_call($self->{'usermodule'}, "set_user_envs", \%uinfo, 'CREATE_USER', $_[0]->{'pass'}, [ ]);
$self->foreign_call($self->{'usermodule'}, "making_changes");
$self->foreign_call($self->{'usermodule'}, "create_user", \%uinfo);
$self->foreign_call($self->{'usermodule'}, "made_changes");
$self->foreign_call($self->{'usermodule'}, "unlock_user_files");
if ($self->{'config_hash'}->{'other_doms'}) {
	$self->foreign_call($self->{'usermodule'}, "other_modules", "useradmin_create_user",
		      \%uinfo);
	}
if (!defined(getpwnam($_[0]->{'user'}))) {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_ecruser'})};
	exit;
	}
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};

# Set the user's quota
if ($self->{'config_hash'}->{'home_quotas'}) {
	$self->set_server_quotas($_[0]);
	}

$self->create_mail_file(\%uinfo);
}

# modify_unix(&domain, &olddomain)
# Change the password and real name for a domain unix user
sub modify_unix
{
	my $self = shift;

if (!$_[0]->{'parent'}) {
	$self->require_useradmin();
	local @allusers = $self->list_domain_users($_[1]);
	local ($uinfo) = grep { $_->{'user'} eq $_[1]->{'user'} } @allusers;
	if ($uinfo) {
		$self->foreign_call($self->{'usermodule'}, "lock_user_files");
		local %old = %$uinfo;
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_user'})};
		$uinfo->{'real'} = $_[0]->{'owner'};
		if ($_[0]->{'pass_set'}) {
			# Update the Unix user's password
			local $enc = $self->foreign_call($self->{'usermodule'},
				"encrypt_password", $_[0]->{'pass'});
			if ($self->{'d'}->{'disabled'}) {
				# Just keep for later use when enabling
				$self->{'d'}->{'disabled_oldpass'} = $enc;
				}
			else {
				# Set password now
				$uinfo->{'pass'} = $enc;
				}
			$uinfo->{'plainpass'} = $_[0]->{'pass'};
			$self->set_pass_change($uinfo);
			}

		if ($_[0]->{'user'} ne $_[1]->{'user'}) {
			# Unix user was re-named
			$uinfo->{'olduser'} = $_[1]->{'user'};
			$uinfo->{'user'} = $_[0]->{'user'};
			$self->rename_mail_file($uinfo, \%old);
			}

		if ($_[0]->{'home'} ne $_[1]->{'home'}) {
			# Home directory was changed
			$uinfo->{'home'} = $_[0]->{'home'};
			}

		$self->modify_user($uinfo, \%old, undef);

		if ($self->{'config_hash'}->{'other_doms'}) {
			$self->foreign_call($self->{'usermodule'}, "other_modules",
				      "useradmin_modify_user", $uinfo, \%old);
			}

		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		$self->foreign_call($self->{'usermodule'}, "unlock_user_files");
		}

	if ($self->{'config_hash'}->{'home_quotas'} && $self->{'access_hash'}->{'edit'} == 1) {
		# Update his quotas (if changed)
		if ($_[0]->{'quota'} != $_[1]->{'quota'} ||
		    $_[0]->{'uquota'} != $_[1]->{'uquota'}) {
			do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_quota'})};
			$self->set_server_quotas($_[0]);
			do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
			}
		}
	}
}

# delete_unix(&domain)
# Delete the unix user and group for a domain
sub delete_unix
{
	my $self = shift;

$self->require_useradmin();
local @allusers = $self->foreign_call($self->{'usermodule'}, "list_users");
local ($uinfo) = grep { $_->{'user'} eq $_[0]->{'user'} } @allusers;

if (!$_[0]->{'parent'}) {
	# Delete unix user
	$self->foreign_call($self->{'usermodule'}, "lock_user_files");
	if ($uinfo) {
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'delete_user'})};
		$self->delete_user($uinfo, $_[0]);
		if ($self->{'config_hash'}->{'other_doms'}) {
			$self->foreign_call($self->{'usermodule'}, "other_modules",
				      "useradmin_delete_user", $uinfo);
			}
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}

	# Delete unix group
	local @allgroups = $self->foreign_call($self->{'usermodule'}, "list_groups");
	local ($ginfo) = grep { $_->{'group'} eq $_[0]->{'group'} } @allgroups;
	if ($ginfo) {
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'delete_group'})};
		$self->foreign_call($self->{'usermodule'}, "set_group_envs", $ginfo, 'DELETE_GROUP');
		$self->foreign_call($self->{'usermodule'}, "making_changes");
		$self->foreign_call($self->{'usermodule'}, "delete_group", $ginfo);
		$self->foreign_call($self->{'usermodule'}, "made_changes");
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	$self->foreign_call($self->{'usermodule'}, "unlock_user_files");
	}
}

# check_unix_clash(&domain, [field])
sub check_unix_clash
{
	my $self = shift;

return 0 if ($_[0]->{'parent'});	# user already exists!
if (!$_[1] || $_[1] eq 'user') {
	return 1 if (defined(getpwnam($_[0]->{'user'})));
	}
if (!$_[1] || $_[1] eq 'group') {
	return 1 if ($_[0]->{'group'} &&
		     defined(getgrnam($_[0]->{'group'})));
	}
return 0;
}

# disable_unix(&domain)
# Lock out the password of this domain's Unix user
sub disable_unix
{
	my $self = shift;

if (!$_[0]->{'parent'}) {
	$self->require_useradmin();
	$self->foreign_call($self->{'usermodule'}, "lock_user_files");
	local @allusers = $self->foreign_call($self->{'usermodule'}, "list_users");
	local ($uinfo) = grep { $_->{'user'} eq $_[0]->{'user'} } @allusers;
	if ($uinfo) {
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'disable_unix'})};
		$self->foreign_call($self->{'usermodule'}, "set_user_envs", $uinfo, 'MODIFY_USER', "");
		$self->foreign_call($self->{'usermodule'}, "making_changes");
		$_[0]->{'disabled_oldpass'} = $uinfo->{'pass'};
		$uinfo->{'pass'} = $self->{'uconfig_hash'}->{'lock_string'};
		$self->foreign_call($self->{'usermodule'}, "modify_user", $uinfo, $uinfo);
		$self->foreign_call($self->{'usermodule'}, "made_changes");
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	$self->foreign_call($self->{'usermodule'}, "unlock_user_files");
	}
}

# enable_unix(&domain)
# Re-enable this domain's Unix user
sub enable_unix
{
	my $self = shift;

if (!$_[0]->{'parent'}) {
	$self->require_useradmin();
	$self->foreign_call($self->{'usermodule'}, "lock_user_files");
	local @allusers = $self->foreign_call($self->{'usermodule'}, "list_users");
	local ($uinfo) = grep { $_->{'user'} eq $_[0]->{'user'} } @allusers;
	if ($uinfo) {
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'enable_unix'})};
		$self->foreign_call($self->{'usermodule'}, "set_user_envs", $uinfo, 'MODIFY_USER', "");
		$self->foreign_call($self->{'usermodule'}, "making_changes");
		$uinfo->{'pass'} = $_[0]->{'disabled_oldpass'};
		delete($_[0]->{'disabled_oldpass'});
		$self->foreign_call($self->{'usermodule'}, "modify_user", $uinfo, $uinfo);
		$self->foreign_call($self->{'usermodule'}, "made_changes");
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	$self->foreign_call($self->{'usermodule'}, "unlock_user_files");
	}
}

# backup_unix(&domain, file)
# Does nothing, since the domain's unix user details are already known
sub backup_unix
{
	my $self = shift;

return 1;
}

# restore_unix(&domain, file, &options)
# Resets the unix users quotas, password and real name from the backup
sub restore_unix
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_unixuser'})};

# Also re-set quotas
if ($self->{'config_hash'}->{'home_quotas'}) {
	$self->set_server_quotas($_[0]);
	}

# And update password and description
$self->require_useradmin();
local @allusers = $self->foreign_call($self->{'usermodule'}, "list_users");
local ($uinfo) = grep { $_->{'user'} eq $_[0]->{'user'} } @allusers;
if ($uinfo && !$_[0]->{'parent'}) {
	$self->foreign_call($self->{'usermodule'}, "lock_user_files");
	$uinfo->{'real'} = $_[0]->{'owner'};
	local $enc = $self->foreign_call($self->{'usermodule'}, "encrypt_password",
			$_[0]->{'pass'});
	$uinfo->{'pass'} = $enc;
	$self->set_pass_change($uinfo);
	$self->foreign_call($self->{'usermodule'}, "set_user_envs",
		$uinfo, 'MODIFY_USER', $_[0]->{'pass'});
	$self->foreign_call($self->{'usermodule'}, "making_changes");
	$self->foreign_call($self->{'usermodule'}, "modify_user", $uinfo, $uinfo);
	$self->foreign_call($self->{'usermodule'}, "made_changes");
	$self->foreign_call($self->{'usermodule'}, "lock_user_files");
	}
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};

return 1;
}

# bandwidth_unix(&domain, start, &bw-hash)
# Updates the bandwidth count for FTP traffic by the domain's unix user, and
# any mail users with FTP access
sub bandwidth_unix
{
	my $self = shift;

local $log;
if ($self->{'config_hash'}->{'ftp_system'} == 2) {
	$log = $self->{'config_hash'}->{'bw_ftplog'} || $self->get_vsftpd_log();
}
else {
	$log = $self->{'config_hash'}->{'bw_ftplog'} || $self->get_proftpd_log();
}
if ($log) {
	local @users;
	if (!$_[0]->{'parent'}) {
		# Only do the domain owner if this is the parent domain, to
		# avoid double-counting in subdomains
		push(@users, $_[0]->{'user'});
		}
	foreach $u ($self->list_domain_users($_[0], 0, 1, 1)) {
		push(@users, $u->{'user'}) if ($u->{'unix'});
		}
	return $self->count_ftp_bandwidth($log, $_[1], $_[2], \@users, "ftp",
				    $self->{'config_hash'}->{'bw_ftplog_rotated'});
	}
else {
	return $_[1];
	}
}


1;

