#!/usr/bin/perl
package virtual_server::Featurewebalizer;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'virtual_server::Featurewebalizer');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;

$self->{'feature_depends_hash'}->{'webalizer'} = [ 'web' ];

1;

1;
		return $self;
}

sub require_webalizer
{
	my $self = shift;

return if ($self->{'require_webalizer'}++);
$self->foreign_require("webalizer", "webalizer-lib.pl");
%{$self->{'wconfig_hash'}} = $self->foreign_config("webalizer");
}

# setup_webalizer(&domain)
# Setup the Webalizer module for this domain, and create a Cron job to run it
sub setup_webalizer
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_webalizer'})};
$self->require_webalizer();

local $alog = $self->get_apache_log($_[0]->{'dom'}, $_[0]->{'web_port'});
if (!$alog) {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_nolog'})};
	return;
	}

# Create directory for stats
local $tmpl = $self->get_template($_[0]->{'template'});
local $hdir = $self->public_html_dir($_[0]);
local $stats;
if ($tmpl->{'web_stats_hdir'}) {
	$stats = "$_[0]->{'home'}/$tmpl->{'web_stats_hdir'}";
	}
elsif ($tmpl->{'web_stats_dir'}) {
	$stats = "$hdir/$tmpl->{'web_stats_dir'}";
	}
else {
	$stats = "$hdir/stats";
	}
if (!-d $stats) {
	$self->system_logged("mkdir '$stats' 2>/dev/null");
	$self->system_logged("chmod 755 '$stats'");
	$self->system_logged("chown $_[0]->{'uid'}:$_[0]->{'ugid'} '$stats'");
	}

local $htaccess_file = "$stats/.htaccess";
local $passwd_file = "$stats/htpasswd";
if ($tmpl->{'web_stats_pass'} && !-r $htaccess_file) {
	# Setup .htaccess file for directory
	$self->open_tempfile(HTACCESS, ">$htaccess_file");
	$self->print_tempfile(HTACCESS, "AuthName \"$_[0]->{'dom'} statistics\"\n");
	$self->print_tempfile(HTACCESS, "AuthType Basic\n");
	$self->print_tempfile(HTACCESS, "AuthUserFile $passwd_file\n");
	$self->print_tempfile(HTACCESS, "require valid-user\n");
	$self->close_tempfile(HTACCESS);
	chown($_[0]->{'uid'}, $_[0]->{'gid'}, $htaccess_file);
	}
if ($tmpl->{'web_stats_pass'}) {
	$self->update_create_htpasswd($_[0], $passwd_file, $_[0]->{'user'});
	$_[0]->{'stats_pass'} = $passwd_file;
	}
else {
	delete($_[0]->{'stats_pass'});
	}

# Set up config for log in Webalizer module
local $lcn = $main::webalizer_object->log_config_name($alog);
$self->lock_file($lcn);
local $cfile = $main::webalizer_object->config_file_name($alog);
$self->lock_file($cfile);
if (!-r $lcn) {
	$self->{'lconf'} = { 'dir' => $stats,
		   'sched' => 1,
		   'type' => 0,
		   'over' => 0,
		   'clear' => 0,
		   'user' => $_[0]->{'user'},
		   'mins' => int(rand()*60),
		   'hours' => 0,
		   'days' => '*',
		   'months' => '*',
		   'weekdays' => '*' };
	$main::webalizer_object->save_log_config($alog, $self->{'lconf'});

	# Create a custom webalizer.conf for the site
	if ($tmpl->{'webalizer'} && -r $tmpl->{'webalizer'}) {
		# Read the specified file and do substitution
		local $wt = `cat $tmpl->{'webalizer'}`;
		local %subs = %{$_[0]};
		$subs{'stats'} = $stats;
		$subs{'log'} = $alog;
		$wt = $self->substitute_template($wt, \%subs);
		$self->open_tempfile(FILE, ">$cfile");
		$self->print_tempfile(FILE, $wt);
		$self->close_tempfile(FILE);
		}
	else {
		# Copy webalizer.conf into place for site, and update
		system("cp $self->{'wconfig_hash'}->{'webalizer_conf'} $cfile");
		local $wconf = $main::webalizer_object->get_config($alog);
		$main::webalizer_object->save_directive($wconf, "HistoryName", "$stats/webalizer.hist");
		$main::webalizer_object->save_directive($wconf, "IncrementalName", "$stats/webalizer.current");
		$main::webalizer_object->save_directive($wconf, "Incremental", "yes");
		$main::webalizer_object->save_directive($wconf, "LogFile", $alog);
		$main::webalizer_object->save_directive($wconf, "HostName", $_[0]->{'dom'});
		$main::webalizer_object->save_directive($wconf, "HideReferrer", "*.$_[0]->{'dom'}");
		$self->flush_file_lines();
		}
	local $group = $_[0]->{'group'} || $_[0]->{'ugroup'};
	$self->system_logged("chown $_[0]->{'user'}:$group $cfile");
	}
else {
	# Already exists .. but update
	$self->{'lconf'} = $main::webalizer_object->get_log_config($alog);
	$self->{'lconf'}->{'dir'} = $stats;
	$main::webalizer_object->save_log_config($alog, $self->{'lconf'});
	local $wconf = $main::webalizer_object->get_config($alog);
	$main::webalizer_object->save_directive($wconf, "HistoryName", "$stats/webalizer.hist");
	$main::webalizer_object->save_directive($wconf, "IncrementalName", "$stats/webalizer.current");
	$self->flush_file_lines();
	}
$self->unlock_file($lcn);
$self->unlock_file($cfile);

$self->foreign_require("cron", "cron-lib.pl");
local ($job) = grep { $_->{'command'} eq "$main::webalizer_object->{'cron_cmd'} $alog" }
		    $main::cron_object->list_cron_jobs();
if (!$job) {
	# Create a Cron job to process the log
	$self->setup_webalizer_cron($self->{'lconf'}, $alog);
	}
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# setup_webalizer_cron(&lconf, access-log)
sub setup_webalizer_cron
{
	my $self = shift;

local ($lconf, $alog) = @_;
local $job = { 'user' => 'root',
	       'active' => 1,
	       'mins' => $lconf->{'mins'},
	       'hours' => $lconf->{'hours'},
	       'days' => $lconf->{'days'},
	       'months' => $lconf->{'months'},
	       'weekdays' => $lconf->{'weekdays'},
	       'special' => $lconf->{'special'},
	       'command' => "$main::webalizer_object->{'cron_cmd'} $alog" };
if (!-r $main::webalizer_object->{'cron_cmd'}) {
	$self->lock_file($main::webalizer_object->{'cron_cmd'});
	$main::cron_object->create_wrapper($main::webalizer_object->{'cron_cmd'},
			      "webalizer", "webalizer.pl");
	$self->unlock_file($main::webalizer_object->{'cron_cmd'});
	}
$self->lock_file($main::cron_object->cron_file($job));
$main::cron_object->create_cron_job($job);
$self->unlock_file($main::cron_object->cron_file($job));
}

# modify_webalizer(&domain, &olddomain)
sub modify_webalizer
{
	my $self = shift;

$self->require_webalizer();
local $alog = $self->get_apache_log($_[0]->{'dom'}, $_[0]->{'web_port'});
if ($_[0]->{'home'} ne $_[1]->{'home'}) {
	# Update Webalizer configuration to use new log file
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_webalizerlog'})};
	local $oldalog = $alog;
	$oldalog =~ s/$_[0]->{'home'}/$_[1]->{'home'}/;	# because it will
							# have been renamed
	local $oldcfile = $main::webalizer_object->config_file_name($oldalog);
	local $cfile = $main::webalizer_object->config_file_name($alog);
	$self->rename_logged($oldcfile, $cfile);
	local $oldlcn = $main::webalizer_object->log_config_name($oldalog);
	local $lcn = $main::webalizer_object->log_config_name($alog);
	$self->rename_logged($oldlcn, $lcn);
	$self->foreign_require("cron", "cron-lib.pl");
	local ($job) = grep { $_->{'command'} eq "$main::webalizer_object->{'cron_cmd'} $oldalog" }
			    $main::cron_object->list_cron_jobs();
	if ($job) {
		$job->{'command'} = "$main::webalizer_object->{'cron_cmd'} $alog";
		$main::cron_object->change_cron_job($job);
		}
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
if ($_[0]->{'dom'} ne $_[1]->{'dom'}) {
	# Update hostname in Webalizer configuration
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_webalizer'})};
	local $cfile = $main::webalizer_object->config_file_name($alog);
	$self->lock_file($cfile);
	local $wconf = $main::webalizer_object->get_config($alog);
	$main::webalizer_object->save_directive($wconf, "HostName", $_[0]->{'dom'});
	$self->flush_file_lines();
	$self->unlock_file($cfile);
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
if ($_[0]->{'user'} ne $_[1]->{'user'}) {
	# Update Unix user Webliazer is run as
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_webalizeruser'})};
	local $lcn = $main::webalizer_object->log_config_name($alog);
	$self->lock_file($lcn);
	local $lconf = $main::webalizer_object->get_log_config($alog);
	$lconf->{'user'} = $_[0]->{'user'};
	$main::webalizer_object->save_log_config($alog, $lconf);
	$self->unlock_file($lcn);
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	}
if ($_[0]->{'stats_pass'}) {
	# Update password for stats dir
	$self->update_create_htpasswd($_[0], $_[0]->{'stats_pass'}, $_[1]->{'user'});
	}
}

# delete_webalizer(&domain)
# Delete the Webalizer config files and Cron job
sub delete_webalizer
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'delete_webalizer'})};
$self->require_webalizer();
local $alog = $self->get_apache_log($_[0]->{'dom'}, $_[0]->{'web_port'});
if (!$alog) {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'delete_webalizerno'})};
	return;
	}

# Delete config files
#local $lfile = $main::webalizer_object->log_config_name($alog);
#unlink($lfile);
#local $cfile = $main::webalizer_object->config_file_name($alog);
#unlink($cfile);
 
# Turn off cron job for webalizer config
$self->foreign_require("cron", "cron-lib.pl");
local ($job) = grep { $_->{'command'} eq "$main::webalizer_object->{'cron_cmd'} $alog" }
		    $main::cron_object->list_cron_jobs();
if ($job) {
	$self->lock_file($main::cron_object->cron_file($job));
	$main::cron_object->delete_cron_job($job);
	$self->unlock_file($main::cron_object->cron_file($job));
        }
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# check_webalizer_clash()
# Does nothing, because the web clash check is all that is needed
sub check_webalizer_clash
{
	my $self = shift;

return 0;
}

sub enable_webalizer
{
	my $self = shift;

# Does nothing yet
}

sub disable_webalizer
{
	my $self = shift;

# Does nothing yet
}

# backup_webalizer(&domain, file)
# Saves the server's Webalizer config file, module config file and schedule
sub backup_webalizer
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'backup_webalizercp'})};
$self->require_webalizer();
local $alog = $self->get_apache_log($_[0]->{'dom'}, $_[0]->{'web_port'});
if (!$alog) {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_nolog'})};
	return 0;
	}
else {
	local $lcn = $main::webalizer_object->log_config_name($alog);
	system("cp ".quotemeta($lcn)." ".quotemeta($_[1]));

	local $cfile = $main::webalizer_object->config_file_name($alog);
	system("cp ".quotemeta($cfile)." ".quotemeta($_[1])."_conf");
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	return 1;
	}
}

# restore_webalizer(&domain, file)
# Copies back the server's Webalizer config files, and re-sets up the Cron job
sub restore_webalizer
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_webalizercp'})};
$self->require_webalizer();
local $alog = $self->get_apache_log($_[0]->{'dom'}, $_[0]->{'web_port'});
if (!$alog) {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_nolog'})};
	return 0;
	}
else {
	local $lcn = $main::webalizer_object->log_config_name($alog);
	$self->lock_file($lcn);
	system("cp ".quotemeta($_[1])." ".quotemeta($lcn));
	$self->unlock_file($lcn);

	local $cfile = $main::webalizer_object->config_file_name($alog);
	$self->lock_file($cfile);
	system("cp ".quotemeta($_[1])."_conf"." ".quotemeta($cfile));
	$self->unlock_file($cfile);

	$self->foreign_require("cron", "cron-lib.pl");
	local ($job) = grep { $_->{'command'} eq "$main::webalizer_object->{'cron_cmd'} $alog" }
			    $main::cron_object->list_cron_jobs();
	if ($job) {
		$self->lock_file($main::cron_object->cron_file($job));
		$main::cron_object->delete_cron_job($job);
		}
	local $lcn = $main::webalizer_object->log_config_name($alog);
	local $lconf;
	if (!-r $lcn) {
		$self->lock_file($lcn);
		$lconf = { 'dir' => $self->{'stats'},
		   'sched' => 1,
		   'type' => 0,
		   'over' => 0,
		   'clear' => 0,
		   'user' => $_[0]->{'user'},
		   'mins' => int(rand()*60),
		   'hours' => 0,
		   'days' => '*',
		   'months' => '*',
		   'weekdays' => '*' };
		$main::webalizer_object->save_log_config($alog, $lconf);
		$self->unlock_file($lcn);
		}
	else {
		$lconf = $main::webalizer_object->get_log_config($alog);
		}
	$self->setup_webalizer_cron($lconf, $alog);
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	return 1;
	}
}

# update_create_htpasswd(&domain, file, old-user)
sub update_create_htpasswd
{
	my $self = shift;

local $pass;
if ($_[0]->{'parent'}) {
	$pass = $self->get_domain($_[0]->{'parent'})->{'pass'};
	}
else {
	$pass = $_[0]->{'pass'};
	}
$self->foreign_require("htaccess_htpasswd", "htaccess-lib.pl");
local $users = $main::htaccess_htpasswd_object->list_users($_[1]);
local ($user) = grep { $_->{'user'} eq $_[2] } @$users;
if ($user) {
	$user->{'user'} = $_[0]->{'user'};
	$user->{'pass'} = $main::htaccess_htpasswd_object->encrypt_password($pass);
	$main::htaccess_htpasswd_object->modify_user($user);
	}
else {
	$user = { 'enabled' => 1,
		  'user' => $_[0]->{'user'},
		  'pass' => $main::htaccess_htpasswd_object->encrypt_password($pass) };
	$main::htaccess_htpasswd_object->create_user($user, $_[1]);
	chown($_[0]->{'uid'}, $_[0]->{'gid'}, $_[1]);
	}
}


1;

