#!/usr/bin/perl
package virtual_server::Featurewebmin;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'virtual_server::Featurewebmin');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;

1;

1;
		return $self;
}

sub require_acl
{
	my $self = shift;

return if ($self->{'require_acl'}++);
$self->foreign_require("acl", "acl-lib.pl");
}

# setup_webmin(&domain)
# Creates a new user to manage this domain, with access to the appropriate
# modules with the right permissions
sub setup_webmin
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_webmin'})};
$self->require_acl();
local ($wuser) = grep { $_->{'name'} eq $_[1]->{'user'} }
		      $main::acl_object->list_users();
if ($wuser) {
	# Update the modules for this Webmin user
	$self->set_user_modules($_[0], $wuser);
	}
else {
	local @modules;
	local %wuser = ( 'name' => $_[0]->{'user'},
			 'pass' => $_[0]->{'unix'} ? 'x' :
					$self->webmin_password($_[0]),
			 'notabs' => 1,
			 'modules' => [ ],
			 'theme' => $self->{'config_hash'}->{'webmin_theme'} eq '*' ? undef :
				    $self->{'config_hash'}->{'webmin_theme'} eq '' ? '' :
				     $self->{'config_hash'}->{'webmin_theme'}
			 );
	$main::acl_object->create_user(\%wuser);
	$self->set_user_modules($_[0], \%wuser);
	}
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
$self->register_post_action(\&restart_webmin);
}

# webmin_password(&domain)
# Returns an encrypted password for a virtual server
sub webmin_password
{
	my $self = shift;

$self->require_acl();
if ($main::acl_object->UNIVERSAL::can("encrypt_password")) {
	return $main::acl_object->encrypt_password($_[0]->{'pass'});
	}
else {
	local $salt = chr(int(rand(26))+65).chr(int(rand(26))+65);
	return crypt($_[0]->{'pass'}, $salt);
	}
}

# delete_webmin(&domain)
# Delete the webmin user for the domain, and all his permissions
sub delete_webmin
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'delete_webmin'})};
$self->require_acl();
$main::acl_object->delete_user($_[0]->{'user'});
local $m;
foreach $m ($self->get_all_module_infos()) {
	unlink("$self->{'config_directory'}/$m->{'dir'}/$_[0]->{'user'}.acl");
	}
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
$self->register_post_action(\&restart_webmin);
}

# modify_webmin(&domain, &olddomain)
sub modify_webmin
{
	my $self = shift;

if (!$_[0]->{'parent'}) {
	$self->require_acl();
	local ($wuser) = grep { $_->{'name'} eq $_[1]->{'user'} }
			      $main::acl_object->list_users();
	if ($_[0]->{'unix'} ne $_[1]->{'unix'}) {
		# Turn on or off password synchronization
		$wuser->{'pass'} = $_[0]->{'unix'} ? 'x' : $self->webmin_password($_[0]);
		$main::acl_object->modify_user($_[1]->{'user'}, $wuser);
		}
	if ($_[0]->{'user'} ne $_[1]->{'user'}) {
		# Need to re-name user
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_webminuser'})};
		$wuser->{'name'} = $_[0]->{'user'};
		$main::acl_object->modify_user($_[1]->{'user'}, $wuser);
		}
	else {
		# Leave name unchanged
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'save_webmin'})};
		}
	$self->set_user_modules($_[0], $wuser) if ($wuser);
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	$self->register_post_action(\&restart_webmin);
	return 1;
	}
return 0;
}

# disable_webmin(&domain)
# Lock the password of the domains's Webmin user
sub disable_webmin
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'disable_webmin'})};
$self->require_acl();
local ($wuser) = grep { $_->{'name'} eq $_[0]->{'user'} } $main::acl_object->list_users();
if ($wuser) {
	$wuser->{'pass'} = "*LK*";
	$main::acl_object->modify_user($wuser->{'name'}, $wuser);
	$self->register_post_action(\&restart_webmin);
	}
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# enable_webmin(&domain)
# Changes the password of the domain's Webmin user back to unix auth
sub enable_webmin
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'enable_webmin'})};
$self->require_acl();
local ($wuser) = grep { $_->{'name'} eq $_[0]->{'user'} } $main::acl_object->list_users();
if ($wuser) {
	$wuser->{'pass'} = "x";
	$main::acl_object->modify_user($wuser->{'name'}, $wuser);
	$self->register_post_action(\&restart_webmin);
	}
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# restart_webmin()
sub restart_webmin
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_webminpid'})};
$self->restart_miniserv();
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# restart_usermin()
sub restart_usermin
{
	my $self = shift;

$self->foreign_require("usermin", "usermin-lib.pl");
do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_userminpid'})};
&usermin::restart_usermin_miniserv();
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
}

# set_user_modules(&domain, &webminuser)
sub set_user_modules
{
	my $self = shift;

local @mods;

# Work out which module's ACLs to leave alone
local %hasmods = map { $_, 1 } @{$_[1]->{'modules'}};
%hasmods = ( ) if (!$self->{'config_hash'}->{'leave_acl'});

# Work out which domains and features exist
local @doms = ( $_[0], $self->get_domain_by("parent", $_[0]->{'id'}) );
local %doneid;
@doms = grep { !$doneid{$_->{'id'}}++ } @doms;
local (%features, $d, $f);
foreach $d (@doms) {
	foreach $f (@{$self->{'features_array'}}) {
		$features{$f}++ if ($d->{$f});
		}
	}

# Grant access to BIND module if needed
if ($features{'dns'} && $self->{'config_hash'}->{'avail_dns'}) {
	# Allow user to manage just this domain
	push(@mods, "bind8");
	local %acl = ( 'noconfig' => 1,
		       'zones' => join(" ", map { $_->{'dom'} } @doms),
		       'dir' => $self->resolve_links($_[0]->{'home'}),
		       'master' => 0,
		       'slave' => 0,
		       'forward' => 0,
		       'delegation' => 0,
		       'defaults' => 0,
		       'reverse' => 0,
		       'multiple' => 1,
		       'ro' => 0,
		       'apply' => 2,
		       'file' => 0,
		       'params' => 1,
		       'opts' => 0,
		       'delete' => 0,
		       'gen' => 1,
		       'whois' => 1,
		       'findfree' => 1,
		       'slaves' => 0,
		       'remote' => 0,
		       'views' => 0,
		       'vlist' => '' );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "bind8")
		if (!$hasmods{'bind8'});
	}
else {
	@mods = grep { $_ ne "bind8" } @mods;
	}

# Grant access to MySQL module if needed
if ($features{'mysql'} && $self->{'config_hash'}->{'avail_mysql'}) {
	# Allow user to manage just the domain's DB
	push(@mods, "mysql");
	local %acl = ( 'noconfig' => 1,
		       'dbs' => join(" ", map { split(/\s+/, $_->{'db_mysql'}) } @doms),
		       'create' => 0,
		       'delete' => 0,
		       'stop' => 0,
		       'perms' => 0,
		       'edonly' => 0,
		       'user' => $self->mysql_user($_[0]),
		       'pass' => $self->mysql_pass($_[0]),
		       'buser' => $_[0]->{'user'},
		       'bpath' => "/" );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "mysql")
		if (!$hasmods{'mysql'});
	}
else {
	@mods = grep { $_ ne "mysql" } @mods;
	}

# Grant access to PostgreSQL module if needed
if ($features{'postgres'} && $self->{'config_hash'}->{'avail_postgres'}) {
	# Allow user to manage just the domain's DB
	push(@mods, "postgresql");
	local %acl = ( 'noconfig' => 1,
		       'dbs' => join(" ", map { split(/\s+/, $_->{'db_postgres'}) } @doms),
		       'create' => 0,
		       'delete' => 0,
		       'stop' => 0,
		       'users' => 0,
		       'user' => $self->postgres_user($_[0]),
		       'pass' => $self->postgres_pass($_[0], 1),
		       'sameunix' => 1,
		       'backup' => 1,
		       'restore' => 1 );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "postgresql")
		if (!$hasmods{'postgresql'});
	}
else {
	@mods = grep { $_ ne "postgresql" } @mods;
	}

# Grant access to Apache module if needed
if ($features{'web'} && $self->{'config_hash'}->{'avail_web'}) {
	# Allow user to manage just this website
	$self->require_apache();
	push(@mods, "apache");
	local %acl = ( 'noconfig' => 1,
		       'virts' => join(" ", map { $_->{'dom'}, "$_->{'dom'}:$_->{'web_port'}" } @doms),
		       'global' => 0,
		       'create' => 0,
		       'vuser' => 0,
		       'vaddr' => 0,
		       'pipe' => 0,
		       'stop' => 0,
		       'dir' => $self->resolve_links($_[0]->{'home'}),
		       'test_always' => 1,
		       'types' => join(" ",
				(0 .. 7, 9 .. 16,
				 18 .. $main::apache_object->{'directive_type_count'}),101..105),
                       'dirsmode' => 2,
                       'dirs' => 'ServerName ServerAlias',
                      );
	if ($_[0]->{'ssl'}) {
		$acl{'virts'} .= " ".join(" ", map { $_->{'dom'}, "$_->{'dom'}:$_->{'web_sslport'}" } @doms);
		}
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "apache")
		if (!$hasmods{'apache'});
	}
else {
	@mods = grep { $_ ne "apache" } @mods;
	}

# Grant access to VSFTPd module if needed
if ($features{'ftp'} && $self->{'config_hash'}->{'avail_ftp'} && $self->{'config_hash'}->{'ftp_system'}) {
	# Allow user to manage just this website
	$self->require_ftp();
	push(@mods, "vsftpd");
	local %acl = ( 'daemon' => 0,
		       'noconfig' => 1,
		       'address_dis' => 1,
		       'security_dis' => 1,
		       'anon_root_dis' => 1,
		       'virts' => join(" ", map { $_->{'dom'}} @doms)
		     );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "vsftpd") if (!$hasmods{'vsftpd'});
	}
else {
	@mods = grep { $_ ne "vsftpd" } @mods;
	}

# Grant access to Webalizer module if needed
if ($features{'webalizer'} && $self->{'config_hash'}->{'avail_webalizer'}) {
	push(@mods, "webalizer");
	local @logs;
	local $d;
	foreach $d (@doms) {
		push(@logs, $self->resolve_links($self->get_apache_log($d->{'dom'}, $d->{'web_port'})));
		}
	local %acl = ( 'noconfig' => 1,
		       'view' => 0,
		       'global' => 0,
		       'add' => 0,
		       'user' => $_[0]->{'user'},
		       'dir' => join(" ", @logs) );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "webalizer")
		if (!$hasmods{'webalizer'});
	}
else {
	@mods = grep { $_ ne "webalizer" } @mods;
	}

# Grant access to Mailman module if needed
if ($features{'mailman'}) {
	# Allow user to manage just this website
	$self->require_mailman();
	push(@mods, "mailman");
	local %acl = ( 'nodaemon' => 1,
						'noconfig' => 1,
						'virts' => join(" ", map { $_->{'dom'}} @doms)
					 );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "mailman") if (!$hasmods{'mailman'});
	}
else {
	@mods = grep { $_ ne "mailman" } @mods;
	}
	
# Grant access to CPAN module
push(@mods, "cpan");
local %acl = ('install' => 0,
	      'download' => 0,
	      'uninstall' => 0,
			'noconfig' => 1
	     );
$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "cpan") if (!$hasmods{'cpan'});

# Grant access to Plugins module
push(@mods, "plugins");
local %acl = ( 'install' => 0,
					'noconfig' => 1 );
$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "plugins") if (!$hasmods{'plugins'});


# Grant access to this module for managing its users and aliases, or for
# adding new domains
local $can_create = defined($_[0]->{'domslimit'}) && !$_[0]->{'no_create'} &&
		    $_[0]->{'unix'};
if ($features{'mail'} && $self->{'config_hash'}->{'avail_mail'} ||	# can manage mail
    $can_create) {
	push(@mods, $self->{'module_name'});
	local %acl = ( 'noconfig' => 1,
		       'edit' => $can_create ? 2 : 0,
		       'create' => $can_create ? 2 : 0,
		       'import' => 0,
		       'stop' => 0,
		       'local' => 0,
		       'nodbname' => $_[0]->{'nodbname'},
		       'norename' => $_[0]->{'norename'},
		       'forceunder' => $_[0]->{'forceunder'},
		       'domains' => join(" ", map { $_->{'id'} } @doms) );
	if ($_[0]->{'domslimit'}) {
		foreach $f (@{$self->{'opt_features_array'}}, @{$self->{'feature_plugins_array'}}) {
			$acl{"feature_$f"} = $_[0]->{"limit_$f"};
			}
		}
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'});
	%{$self->{'uaccess_hash'}} = %acl;
	}
else {
	@mods = grep { $_ ne $self->{'module_name'} } @mods;
	}

# Set global ACL options
local $tmpl = $self->get_template($_[0]->{'template'});
local %acl = ( 'feedback' => 0,
	       'rpc' => 0 );
$acl{'root'} = $self->resolve_links(
	$self->substitute_template($tmpl->{'gacl_root'}, $_[0]));
if ($tmpl->{'gacl_umode'} == 1) {
	$acl{'uedit_mode'} = 5;
	$acl{'uedit'} = $self->substitute_template($tmpl->{'gacl_ugroups'}, $_[0]);
	}
else {
	$acl{'uedit_mode'} = 2;
	$acl{'uedit'} = $self->substitute_template($tmpl->{'gacl_uusers'}, $_[0]);
	}
$acl{'gedit_mode'} = 2;
$acl{'gedit'} = $self->substitute_template($tmpl->{'gacl_groups'}, $_[0]);
if (!$_[0]->{'domslimit'}) {
	$acl{'desc_'.$self->{'module_name'}} = $self->{'text_hash'}->{'index_title2'};
	}
$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, ".");

# Work out which extra (non feature-related) modules are available
local @extramods = map { /^avail_(\S+)/; $1 }
		    grep { $self->{'config_hash'}->{$_} }
		       grep { /^avail_/ } (keys %{$self->{'config_hash'}});
local %extramods = map { my $m=$_; $m=~s/\-/\_/g; $m, $self->{'config_hash'}->{"avail_".$_} }
		       grep { my $m=$_; { local $_; $self->foreign_check($m) } }
			@extramods;

if ($extramods{'file'} && $_[0]->{'unix'}) {
	# Limit file manager to user's directory, as unix user
	local %acl = ( 'noconfig' => 1,
		       'uid' => -1,
		       'follow' => 0,
		       'root' => '',
		       'home' => 1,
		       'goto' => 1 );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "file")
		if (!$hasmods{'file'});
	push(@mods, "file");
	}

if ($_[0]->{'unix'}) {
	if ($extramods{'passwd'} == 1) {
		# Can only change his password
		local %acl = ( 'noconfig' => 1,
			       'mode' => 3,
			       'repeat' => 1,
			       'old' => 1,
			       'expire' => 0,
			       'others' => 1 );
		$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "passwd")
			if (!$hasmods{'passwd'});
		push(@mods, "passwd");
		}
	elsif ($extramods{'passwd'} == 2) {
		# Can change all mailbox passwords
		local %acl = ( 'noconfig' => 1,
			       'mode' => 5,
			       'users' => $_[0]->{'group'},
			       'repeat' => 1,
			       'old' => 0,
			       'expire' => 0,
			       'others' => 1 );
		$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "passwd")
			if (!$hasmods{'passwd'});
		push(@mods, "passwd");
		}
	}

if ($extramods{'proc'} && $_[0]->{'unix'}) {
	# Can only manage his own processes
	local %acl = ( 'noconfig' => 1,
		       'uid' => -1,
		       'edit' => 1,
		       'run' => 1,
		       'only' => ($extramods{'proc'} == 2) );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "proc")
		if (!$hasmods{'proc'});
	push(@mods, "proc");
	}

if ($extramods{'cron'} && $_[0]->{'unix'}) {
	# Can only manage his cron jobs
	local %acl = ( 'noconfig' => 1,
		       'mode' => 3,
		       'allow' => 0 );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "cron")
		if (!$hasmods{'cron'});
	push(@mods, "cron");
	}

if ($extramods{'at'} && $_[0]->{'unix'}) {
	# Can only manage his at jobs
	local %acl = ( 'noconfig' => 1,
		       'mode' => 3,
		       'allow' => 0 );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "at")
		if (!$hasmods{'at'});
	push(@mods, "at");
	}

if ($extramods{'telnet'} && $_[0]->{'unix'}) {
	# Cannot configure module
	local %acl = ( 'noconfig' => 1 );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "telnet")
		if (!$hasmods{'telnet'});
	push(@mods, "telnet");
	}

if ($extramods{'custom'}) {
	# Cannot edit and create commands
	local %acl = ( 'noconfig' => 1,
		       'cmd' => '*',
		       'edit' => 0 );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "custom")
		if (!$hasmods{'custom'});
	push(@mods, "custom");
	}

if ($extramods{'updown'} && $_[0]->{'unix'}) {
	# Can upload and download to home dir only
	local %acl = ( 'noconfig' => 1,
		       'dirs' => '',
		       'home' => 1,
		       'mode' => 3 );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "updown")
		if (!$hasmods{'updown'});
	push(@mods, "updown");

	# Set defaults for upload and download directories for this user
	local %udconfig;
	local $udfile = "$self->{'config_directory'}/updown/config";
	$self->lock_file($udfile);
	$self->read_file($udfile, \%udconfig);
	$self->{'udfile_hash'}->{'dir_'.$_[1]->{'name'}} ||= $self->resolve_links($_[0]->{'home'});
	$self->{'udfile_hash'}->{'ddir_'.$_[1]->{'name'}} ||= $self->resolve_links($_[0]->{'home'});
	$self->write_file($udfile, \%udconfig);
	$self->unlock_file($udfile);
	}

if ($extramods{'change-user'}) {
	# This module is always safe, so no ACL needs to be set
	push(@mods, "change_user");
	}

if ($extramods{'htaccess_htpasswd'} && $_[0]->{'unix'}) {
	# Can create .htaccess files in home dir, as user
	local %acl = ( 'noconfig' => 1,
		       'dirs' => $self->resolve_links($_[0]->{'home'}),
		       'sync' => 0,
		       'user' => '*' );
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "htaccess_htpasswd")
		if (!$hasmods{'htaccess_htpasswd'});
	push(@mods, "htaccess_htpasswd");
	}

if ($extramods{'mailboxes'} && $_[0]->{'mail'}) {
	# Can read mailboxes of users
	local %acl = ( 'noconfig' => 1,
		       'fmode' => 1,
		       'from' => join(" ", map { $_->{'dom'} } @doms),
		       'canattach' => 0,
		       'candetach' => 0,
		       'dir' => $self->mail_domain_base($_[0]) );
	if (!$self->mail_system_needs_group()) {
		# For vpopmail, mailboxes are identified by domain
		$acl{'mmode'} = 6;
		$acl{'musers'} = ".*\@(".
				 join("|", map { $_->{'dom'} } @doms).")";
		}
	else {
		# By server GID
		$acl{'mmode'} = 5;
		$acl{'musers'} = $_[0]->{'gid'};
		}
	$self->save_module_acl_logged(\%acl, $_[1]->{'name'}, "mailboxes")
		if (!$hasmods{'mailboxes'});
	push(@mods, "mailboxes");
	}
else {
	@mods = grep { $_ ne "mailboxes" } @mods;
	}
push(@mods, "home");

# Add any extra modules specified for this domain
push(@mods, split(/\s+/, $_[0]->{'webmin_modules'}));

# Add plugin-specified modules
local $p;
foreach $p (@{$self->{'feature_plugins_array'}}) {
	local @pmods = $self->plugin_call($p, "feature_webmin", $_[0], \@doms);
	local $pm;
	foreach $pm (@pmods) {
		push(@mods, $pm->[0]);
		$self->save_module_acl_logged($pm->[1], $_[1]->{'name'}, $pm->[0]);
		}
	}

$_[1]->{'modules'} = [ $self->unique(@mods) ];
$_[1]->{'readonly'} = $self->{'module_name'};
$main::acl_object->modify_user($_[1]->{'name'}, $_[1]);
}

# check_webmin_clash(&domain, [field])
# Returns 1 if a user or group with this name already exists
sub check_webmin_clash
{
	my $self = shift;

if (!$_[1] || $_[1] eq 'user') {
	$self->require_acl();
	return 1 if ($_[0]->{'user'} eq 'webmin');
	return 0 if ($_[0]->{'webmin_overwrite'});
	local $u;
	foreach $u ($main::acl_object->list_users()) {
		return 1 if ($u->{'name'} eq $_[0]->{'user'});
		}
	}
return 0;
}

# modify_all_webmin()
# Updates the Webmin users for all domains
sub modify_all_webmin
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'check_allwebmin'})};
	{
	local $first_print_tmp = $self->{'first_print'};
	local $second_print_tmp = $self->{'second_print'};
	$self->{'first_print'} = sub {};
	$self->{'second_print'} = sub {};
	local $d;
	foreach $d ($self->list_domains()) {
		if ($d->{'webmin'} && $self->{'config_hash'}->{'webmin'}) {
			$self->modify_webmin($d, $d);
			}
		}
	$self->{'first_print'} = $first_print_tmp;
	$self->{'second_print'} = $second_print_tmp;
	}
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
$self->register_post_action(\&restart_webmin);
}

# save_module_acl_logged(&acl, user, module)
sub save_module_acl_logged
{
	my $self = shift;

local $afile = "$self->{'config_directory'}/$_[2]/$_[1].acl";
$self->lock_file($afile);
$self->save_module_acl(@_);
$self->unlock_file($afile);
}


1;

