#!/usr/bin/perl
package virtual_server::Postinstall;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'virtual_server::Postinstall');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;

$self->{'no_acl_check'}++;
&ModuleLoader::preload_module(ref($self), "virtual_server::VirtualserverLib");
$self = new virtual_server::VirtualserverLib:: ref($self), %$self; 

1;
		return $self;
}

sub module_install {
	my $self = shift;


# turn off misconfigured features
my %features = (
	'apache'     => "X:httpd_path:web",
	'bind8'      => "X:named_path:dns",
	'mysql'      => "X:mysqladmin:mysql",
	'postgresql' => "X:psql:postgres",
	'webalizer'  => "C:webalizer_conf:webalizer",
	'logrotate'  => "C:logrotate_conf:logrotate",
	#'proftpd'    => "X:proftpd_path:ftp",
	'samba'      => "C:_nodefined_:samba",
);

while ( ($self->{'dir'}, $self->{'val'}) = each(%features) ) {
	my ($type, $check, $ventry) = split(/:/, $self->{'val'});
	next if "$ENV{'WEBMIN_UPGRADE'}" eq "1" && !$self->{'config_hash'}->{$ventry};

	$self->read_file("$self->{'config_directory'}/$self->{'dir'}/config", \%{$self->{'conf_hash'}});
	if ($type =~ /^X/) {
		if ( !-x $self->{'conf_hash'}->{$check} ) {
			$self->{'config_hash'}->{$ventry} = 0;
		} else {
			$self->{'config_hash'}->{$ventry} = 1;
		}
	} elsif ($type =~ /^C/) {
		if ( !-r $self->{'conf_hash'}->{$check} ) {
			$self->{'config_hash'}->{$ventry} = 0;
		} else {
			$self->{'config_hash'}->{$ventry} = 1;
		}
	}
}
$self->write_file("$self->{'module_config_directory'}/config", \%{$self->{'config_hash'}});

# check up and configure mail system
if ($self->{'config_hash'}->{'mail'}) {
	if ($self->{'config_hash'}->{'mail_system'} == 3) {
		# Work out which mail server we have
		if ($self->postfix_installed()) {
			$self->{'config_hash'}->{'mail_system'} = 0;
			}
		elsif ($self->qmail_installed()) {
			$self->{'config_hash'}->{'mail_system'} = 2;
			}
		elsif ($self->sendmail_installed()) {
			$self->{'config_hash'}->{'mail_system'} = 1;
			}
		$self->write_file("$self->{'module_config_directory'}/config", \%{$self->{'config_hash'}});
		}
	$self->require_mail();
	}

# Make sure openssl is installed, that Apache supports mod_ssl,
# and that port 443 is in use
$self->{'config_hash'}->{'ssl'} = 0;
if ($self->{'config_hash'}->{'web'} && $self->has_command("openssl")) {
	$self->require_apache();
	local $conf = $main::apache_object->get_config();
	local @loads = $main::apache_object->find_directive_struct("LoadModule", $conf);
	local ($l, $hasmod);
	foreach $l (@loads) {
		$hasmod++ if ($l->{'words'}->[1] =~ /mod_ssl/);
		}
	local ($aver, $amods) = $main::apache_object->httpd_info($main::apache_object->find_httpd());
	$hasmod++ if ($self->indexof("mod_ssl", @$amods) >= 0);
	$hasmod++ if ($main::apache_object->{'httpd_modules_hash'}->{'mod_ssl'});
	if ($hasmod) {
		local @listens = $main::apache_object->find_directive_struct("Listen", $conf);
		foreach $l (@listens) {
			$self->{'haslisten'}++ if ($l->{'words'}->[0] =~ /^(\S+:)?$self->{'default_web_sslport'}$/);
			}
		local @ports = $main::apache_object->find_directive_struct("Port", $conf);
		foreach $l (@ports) {
			$self->{'haslisten'}++ if ($l->{'words'}->[0] == $self->{'default_web_sslport'});
			}
		$self->{'config_hash'}->{'ssl'} = 1 if $self->{'haslisten'};
	}
}
$self->write_file("$self->{'module_config_directory'}/config", \%{$self->{'config_hash'}});

# update domain-owners' access
foreach my $d ($self->list_domains()) {
	my $u = $d->{'user'};
	%{$self->{'acl_hash'}} = $self->get_module_acl($u);
	my $can_create = $d->{'domslimit'} && !$d->{'no_create'} && $d->{'unix'};
	$self->{'acl_hash'}->{'edit'} = $can_create ? 2 : 0;
	$self->{'acl_hash'}->{'create'} = $can_create ? 2 : 0;
	$self->save_module_acl(\%{$self->{'acl_hash'}}, $u);
	}

 #configure file /etc/shells
 if (-e '/etc/shells')
 {
  my ($file,$add,@temp)='/etc/shells';
  $self->lock_file($file);
  my $str= $self->read_file_lines($file);
  foreach (@{$str})
  {
   unless ($_ eq $self->{'config_hash'}->{'shell'})
   {
    push (@temp,$_);
    $add=1 if ($_ eq $self->{'config_hash'}->{'ftp_shell'});
   }
  }
  if ($add!=1)
  {
   #add new string into /etc/shells
   push (@temp,$self->{'config_hash'}->{'ftp_shell'});
   @{$str}=@temp;
   $self->flush_file_lines();
  }
  $self->unlock_file($file);
 }
 $self->ensure_template("domain-template");
 $self->ensure_template("subdomain-template");
 $self->ensure_template("user-template");
 $self->ensure_template("local-template");
 $self->ensure_template("bw-template");
 $self->ensure_template("warnbw-template");
 $self->ensure_template("framefwd-template");
}


1;

