#!/usr/bin/perl

&MAIN();

 sub MAIN {
# Do an immediate virtual server backup

&ModuleLoader::preload_module('virtual_server_class', "virtual_server::VirtualserverLib");
$self = new virtual_server::VirtualserverLib:: 'virtual_server_class',%$self;

$self->master_admin() || $self->error($self->{'text_hash'}->{'backup_ecannot'});
$self->ReadParse();

# Validate inputs
$self->error_setup($self->{'text_hash'}->{'backup_err'});
if ($self->{'in_hash'}->{'all'} == 1) {
	@{$self->{'doms_array'}} = $self->list_domains();
	}
elsif ($self->{'in_hash'}->{'all'} == 2) {
	%{$self->{'exc_hash'}} = map { $_, 1 } split(/\0/, $self->{'in_hash'}->{'doms'});
	@{$self->{'doms_array'}} = grep { !$self->{'exc_hash'}->{$_->{'id'}} } $self->list_domains();
	}
else {
	foreach $d (split(/\0/, $self->{'in_hash'}->{'doms'})) {
		push(@{$self->{'doms_array'}}, $self->get_domain($d));
		}
	}
@{$self->{'doms_array'}} || $self->error($self->{'text_hash'}->{'backup_edoms'});
@{$self->{'do_features_array'}} = split(/\0/, $self->{'in_hash'}->{'feature'});
@{$self->{'do_features_array'}} || $self->error($self->{'text_hash'}->{'backup_efeatures'});
$self->{'dest'} = $self->parse_backup_destination("dest", \%{$self->{'in_hash'}});
$self->{'dest'} = $self->backup_strftime($self->{'dest'}) if ($self->{'in_hash'}->{'strftime'});

# Parse option inputs
foreach $f (@{$self->{'do_features_array'}}) {
	local $ofunc = "parse_backup_$f";
	if ($self->can($ofunc)) {
		$self->{'options_hash'}->{$f} = $self->$ofunc(\%{$self->{'in_hash'}});
		}
	}

if ($self->{'in_hash'}->{'sub_backup'}) {
	
if (defined($self->{'in_hash'}->{'dom'})) {
	# Save domain-specific backup options
	$d = $self->get_domain($self->{'in_hash'}->{'dom'});
	$d->{'backup_dest'} = $self->{'dest'};
	$d->{'backup_fmt'} = $self->{'in_hash'}->{'backup_fmt'};
	$d->{'backup_errors'} = $self->{'in_hash'}->{'backup_errors'};
	$d->{'backup_strftime'} = $self->{'in_hash'}->{'backup_strftime'};
	$self->save_domain($d);
	}

$self->ui_print_unbuffered_header(undef, $self->text('backup_title1' ,$self->{'t_websites_mail'}), "",undef, 0, 1);

print "<p>",$self->text('backup_doing', scalar(@{$self->{'doms_array'}}),
				  "<tt>$self->{'dest'}</tt>"),"<p>\n";
($self->{'ok'}, $self->{'size'}) = $self->backup_domains($self->{'dest'}, \@{$self->{'doms_array'}}, \@{$self->{'do_features_array'}},
			       $self->{'in_hash'}->{'fmt'}, $self->{'in_hash'}->{'errors'}, \%{$self->{'options_hash'}});
$self->run_post_actions();
if (!$self->{'ok'}) {
	unlink($self->{'dest'});
	print "<p>",$self->{'text_hash'}->{'backup_failed'},"<p>\n";
	}
else {
	print "<p>",$self->text('backup_done', $self->nice_size($self->{'size'})),"<p>\n";
	$self->webmin_log("backup", $self->{'dest'}, undef,
		    { 'doms' => [ map { $_->{'dom'} } @{$self->{'doms_array'}} ] });
	}

$self->ui_print_footer("$self->{'text_hash'}->{'index_mainpage_url'}", $self->{'text_hash'}->{'index_mainpage'},"",$self->text('index_return', $self->{'t_website'}));
} elsif ($self->{'in_hash'}->{'sub_sched'}) {
    # Check if the cron job exists
    $self->foreign_require("cron", "cron-lib.pl");
    local @jobs = $main::cron_object->list_cron_jobs();
    local ($job) = grep { $_->{'user'} eq 'root' &&
                          $_->{'command'} eq $self->{'backup_cron_cmd'} } @jobs;

    # Create, update or delete it
    if ($job && $self->{'in_hash'}->{'enabled'}) {
            # Update job
            $self->lock_file($main::cron_object->cron_file($job));
            $main::cron_object->parse_times_input($job, \%{$self->{'in_hash'}});
            $main::cron_object->change_cron_job($job);
            $self->unlock_file($main::cron_object->cron_file($job));
            $self->{'what'} = "modify";
            }
    elsif (!$job && $self->{'in_hash'}->{'enabled'}) {
            # Create job
            $job = { 'user' => 'root',
                     'command' => $self->{'backup_cron_cmd'},
                     'active' => 1 };
            $self->lock_file($main::cron_object->cron_file($job));
            $main::cron_object->parse_times_input($job, \%{$self->{'in_hash'}});
            $main::cron_object->create_cron_job($job);
            $self->lock_file($self->{'backup_cron_cmd'});
            $main::cron_object->create_wrapper($self->{'backup_cron_cmd'}, $self->{'module_name'}, "backup.pl");
            $self->unlock_file($self->{'backup_cron_cmd'});
            $self->unlock_file($main::cron_object->cron_file($job));
            $self->{'what'} = "create";
            }
    elsif ($job && !$self->{'in_hash'}->{'enabled'}) {
            # Delete job
            $self->lock_file($main::cron_object->cron_file($job));
            $main::cron_object->delete_cron_job($job);
            $self->unlock_file($main::cron_object->cron_file($job));
            $self->{'what'} = "delete";
            }
    else {
            $self->{'what'} = "none";
         }

    # Update module config with domains and features
    $self->{'config_hash'}->{'backup_all'} = $self->{'in_hash'}->{'all'};
    $self->{'config_hash'}->{'backup_doms'} = join(" ", split(/\0/, $self->{'in_hash'}->{'doms'}));
    %{$self->{'features_hash'}} = map { $_, 1 } @{$self->{'do_features_array'}};
    foreach $f (@{$self->{'backup_features_array'}}) {
                $self->{'config_hash'}->{'backup_feature_'.$f} = $self->{'features_hash'}->{$f} ? 1 : 0;
                }
    $self->{'config_hash'}->{'backup_dest'} = $self->{'dest'};
    $self->{'config_hash'}->{'backup_fmt'} = $self->{'in_hash'}->{'fmt'};
    $self->{'config_hash'}->{'backup_email'} = $self->{'in_hash'}->{'email'};
    $self->{'config_hash'}->{'backup_errors'} = $self->{'in_hash'}->{'errors'};
    $self->{'config_hash'}->{'backup_strftime'} = $self->{'in_hash'}->{'strftime'};
    $self->{'config_hash'}->{'last_check'} = time()+1;       # no need for check.cgi to be run
    foreach $f (keys %{$self->{'options_hash'}}) {
                $self->{'config_hash'}->{'backup_opts_'.$f} = join(",", map { $_."=".$self->{'options_hash'}->{$f}->{$_} } keys %{$self->{'options_hash'}->{$f}});
            }
    $self->lock_file($self->{'module_config_file'});
    $self->save_module_config();
    $self->unlock_file($self->{'module_config_file'});
    $self->webmin_log("sched", $self->{'what'});

    # Return to main menu
    $self->redirect("$self->{'text_hash'}->{'index_mainpage_url'}");
}

1;
		
}


1;

