#!/usr/bin/perl

&MAIN();

 sub MAIN {
# check.cgi
# Make sure that all enabled features are valid, and check the status of quotas

&ModuleLoader::preload_module('virtual_server_class', "virtual_server::VirtualserverLib");
$self = new virtual_server::VirtualserverLib:: 'virtual_server_class',%$self;

$self->master_admin() || $self->error($self->{'text_hash'}->{'check_ecannot'});

$self->ui_print_unbuffered_header(undef, $self->{'text_hash'}->{'check_title'}, "");

$self->read_file("$self->{'module_config_directory'}/last-config", \%{$self->{'lastconfig_hash'}});
print "<b>$self->{'text_hash'}->{'check_desc'}</b><p>\n";
print "<ul>\n";

$self->{'clink'} = "../config.cgi?$self->{'module_name'}";

# Make sure networking is supported
if (!$self->foreign_check("net")) {
	$self->foreign_require("net", "net-lib.pl");
	if (!$main::net_object->UNIVERSAL::can("boot_interfaces")) {
		&check_error($self->text('index_enet'));
		}
	print "$self->{'text_hash'}->{'check_netok'}<p>\n";
	}

if ($self->{'config_hash'}->{'dns'}) {
	# Make sure BIND is installed
	$self->foreign_installed("bind8", 1) == 2 ||
		&check_error($self->text('index_ebind', "/bind8/", $self->{'clink'}));
	print "$self->{'text_hash'}->{'check_bindok'}<p>\n";
	}

if ($self->{'config_hash'}->{'mail'}) {
	if ($self->{'config_hash'}->{'mail_system'} == 3) {
		# Work out which mail server we have
		if ($self->postfix_installed()) {
			$self->{'config_hash'}->{'mail_system'} = 0;
			}
		elsif ($self->qmail_vpopmail_installed()) {
			$self->{'config_hash'}->{'mail_system'} = 5;
			}
		elsif ($self->qmail_ldap_installed()) {
			$self->{'config_hash'}->{'mail_system'} = 4;
			}
		elsif ($self->qmail_installed()) {
			$self->{'config_hash'}->{'mail_system'} = 2;
			}
		elsif ($self->sendmail_installed()) {
			$self->{'config_hash'}->{'mail_system'} = 1;
			}
		else {
			&check_error($self->text('index_email'));
			}
		print $self->text('check_detected', $self->mail_system_name()),"<p>\n";
		$self->save_module_config();
		}
	if ($self->{'config_hash'}->{'mail_system'} == 1) {
		# Make sure sendmail is installed
		if (!$self->sendmail_installed()) {
			&check_error($self->text('index_esendmail', '/sendmail/',
					   "/config.cgi?$self->{'module_name'}"));
			}
		# Check that aliases and virtusers are configured
		$self->require_mail();
		@{$self->{'sendmail_afiles'}} ||
			&check_error($self->text('index_esaliases', '/sendmail/'));
		$self->{'sendmail_vdbm'} ||
			&check_error($self->text('index_esvirts', '/sendmail/'));
		if ($self->{'config_hash'}->{'generics'}) {
			$self->{'sendmail_gdbm'} ||
		    		&check_error($self->text('index_esgens',
					     '/sendmail/', $self->{'clink'}));
			}
		print "$self->{'text_hash'}->{'check_sendmailok'}<p>\n";
		}
	elsif ($self->{'config_hash'}->{'mail_system'} == 0) {
		# Make sure postfix is installed
		if (!$self->postfix_installed()) {
			&check_error($self->text('index_epostfix', '/postfix/',
					   "/config.cgi?$self->{'module_name'}"));
			}
		# Check that aliases and virtusers are configured
		$self->require_mail();
		@{$self->{'postfix_afiles'}} ||
			&check_error($self->text('index_epaliases', '/postfix/'));
		$self->{'virtual_maps'} =~ /^[a-z0-9]+:\//i ||
			&check_error($self->text('index_epvirts', '/postfix/'));
		if ($self->{'config_hash'}->{'generics'}) {
			$self->{'canonical_maps'} ||
				check_error($self->text('index_epgens',
					    '/postfix/', $self->{'clink'}));
			}
		if ($self->{'config_hash'}->{'append_style'} == 6) {
			&check_error($self->text('check_estyle6', $self->{'clink'}));
			}
		print "$self->{'text_hash'}->{'check_postfixok'}<p>\n";
		}
	elsif ($self->{'config_hash'}->{'mail_system'} == 2) {
		# Make sure qmail is installed
		if (!$self->qmail_installed()) {
			&check_error($self->text('index_eqmail', '/qmailadmin/',
					   "/config.cgi?$self->{'module_name'}"));
			}
		if ($self->{'config_hash'}->{'generics'}) {
			&check_error($self->text('index_eqgens', $self->{'clink'}));
			}
		if ($self->{'config_hash'}->{'append_style'} == 6) {
			print $self->text('check_qmailmode6', $self->{'clink'}),"<p>\n";
			}
		else {
			print "$self->{'text_hash'}->{'check_qmailok'}<p>\n";
			}
		}
	elsif ($self->{'config_hash'}->{'mail_system'} == 4) {
		# Make sure qmail with LDAP is installed
		if (!$self->qmail_ldap_installed()) {
			&check_error($self->text('index_eqmailldap', '/qmailadmin/',
					   "/config.cgi?$self->{'module_name'}"));
			}
		if ($self->{'config_hash'}->{'generics'}) {
			&check_error($self->text('index_eqgens', $self->{'clink'}));
			}
		if (!gethostbyname($self->{'config_hash'}->{'ldap_host'})) {
			&check_error($self->text('index_eqmailhost', $self->{'clink'}));
			}
		if (!$self->{'config_hash'}->{'ldap_base'}) {
			&check_error($self->text('index_eqmailbase', $self->{'clink'}));
			}
		local $lerr = $self->connect_qmail_ldap(1);
		if (!ref($lerr)) {
			&check_error($self->text('index_eqmailconn', $lerr, $self->{'clink'}));
			}
		print "$self->{'text_hash'}->{'check_qmailldapok'}<p>\n";
		}
	elsif ($self->{'config_hash'}->{'mail_system'} == 5) {
		# Make sure qmail with VPOPMail is installed
		if (!$self->qmail_vpopmail_installed()) {
			&check_error($self->text('index_evpopmail', '/qmailadmin/',
					   "/config.cgi?$self->{'module_name'}"));
			}
		if ($self->{'config_hash'}->{'generics'}) {
			&check_error($self->text('index_eqgens', $self->{'clink'}));
			}
		print "$self->{'text_hash'}->{'check_vpopmailok'}<p>\n";
		}
	}

if ($self->{'config_hash'}->{'web'}) {
	# Make sure Apache is installed
	$self->foreign_installed("apache", 1) == 2 ||
		&check_error($self->text('index_eapache', "/apache/", $self->{'clink'}));
	print "$self->{'text_hash'}->{'check_webok'}<p>\n";
	}

if ($self->{'config_hash'}->{'webalizer'}) {
	# Make sure Webalizer is installed, and that global directives are OK
	# XXX is the path checking really needed?
	$self->{'config_hash'}->{'web'} || &check_error($self->text('check_edepwebalizer', $self->{'clink'}));
	$self->foreign_installed("webalizer", 1) == 2 ||
		&check_error($self->text('index_ewebalizer', "/webalizer/", $self->{'clink'}));
	$self->foreign_require("webalizer", "webalizer-lib.pl");
	$conf = $main::webalizer_object->get_config();
	$self->{'current'} = $main::webalizer_object->find_value("IncrementalName", $conf);
	$self->{'history'} = $main::webalizer_object->find_value("HistoryName", $conf);
	if ($self->{'current'} =~ /^\//) {
		&check_error($self->text('check_current', "/webalizer/"));
		}
	elsif ($self->{'history'} =~ /^\//) {
		&check_error($self->text('check_history', "/webalizer/"));
		}
	else {
		print "$self->{'text_hash'}->{'check_webalizerok'}<p>\n";
		}
	}

if ($self->{'config_hash'}->{'ssl'}) {
	# Make sure openssl is installed, that Apache supports mod_ssl,
	# and that port 443 is in use
	$self->{'config_hash'}->{'web'} || &check_error($self->text('check_edepssl', $self->{'clink'}));
	$self->has_command("openssl") ||
	    &check_error($self->text('index_eopenssl', "<tt>openssl</tt>", $self->{'clink'}));

	$self->require_apache();
	local $conf = $main::apache_object->get_config();
	local @loads = $main::apache_object->find_directive_struct("LoadModule", $conf);
	local ($l, $hasmod);
	foreach $l (@loads) {
		$hasmod++ if ($l->{'words'}->[1] =~ /mod_ssl/);
		}
	local ($aver, $amods) = $main::apache_object->httpd_info();
	$hasmod++ if ($self->indexof("mod_ssl", @$amods) >= 0);
	$hasmod++ if ($main::apache_object->{'httpd_modules_hash'}->{'mod_ssl'});
	$hasmod ||
	    &check_error($self->text('index_emodssl', "<tt>mod_ssl</tt>", $self->{'clink'}));

	local @listens = $main::apache_object->find_directive_struct("Listen", $conf);
	foreach $l (@listens) {
		$self->{'haslisten'}++ if ($l->{'words'}->[0] =~ /^(\S+:)?$self->{'default_web_sslport'}$/);
		}
	local @ports = $main::apache_object->find_directive_struct("Port", $conf);
	foreach $l (@ports) {
		$self->{'haslisten'}++ if ($l->{'words'}->[0] == $self->{'default_web_sslport'});
		}
	$self->{'haslisten'} ||
	    &check_error($self->text('index_emodssl2', $self->{'default_web_sslport'}, $self->{'clink'}));
	print "$self->{'text_hash'}->{'check_sslok'}<p>\n";
	}

if ($self->{'config_hash'}->{'mysql'}) {
	# Make sure MySQL is installed
	$self->foreign_installed("mysql", 1) == 2 ||
		&check_error($self->text('index_emysql', "/mysql/", $self->{'clink'}));
	print "$self->{'text_hash'}->{'check_mysqlok'}<p>\n";
	}

if ($self->{'config_hash'}->{'postgres'}) {
	# Make sure PostgreSQL is installed
	$self->foreign_installed("postgresql", 1) == 2 ||
		&check_error($self->text('index_epostgres', "/postgresql/", $self->{'clink'}));
	print "$self->{'text_hash'}->{'check_postgresok'}<p>\n";
	}

if ($self->{'config_hash'}->{'ftp'}) {
	# Make sure ProFTPd is installed, and that the ftp user exists
	$self->foreign_installed("proftpd", 1) == 2 ||
		&check_error($self->text('index_eproftpd', "/proftpd/", $self->{'clink'}));
	$self->{'err'} = $self->check_proftpd_template();
	$self->{'err'} && &check_error($self->text('check_proftpd', $self->{'err'}));
	print "$self->{'text_hash'}->{'check_ftpok'}<p>\n";
	}

if ($self->{'config_hash'}->{'logrotate'}) {
	# Make sure logrotate is installed
	$self->foreign_installed("logrotate", 1) == 2 ||
		&check_error($self->text('index_elogrotate', "/logrotate/", $self->{'clink'}));
	print "$self->{'text_hash'}->{'check_logrotateok'}<p>\n";
	}

# Check all plugins
foreach $p (@{$self->{'plugins_array'}}) {
	$self->{'err'} = $self->plugin_call($p, "feature_check");
	if ($self->{'err'}) {
		&check_error($self->{'err'});
		}
	else {
		$self->{'pname'} = $self->plugin_call($p, "feature_name");
		print $self->text('check_plugin', $self->{'pname'}),"<p>\n";
		}
	}

if (!$self->{'config_hash'}->{'iface'}) {
	# Work out the network interface automatically
	$self->foreign_require("net", "net-lib.pl");
	foreach $a ($main::net_object->active_interfaces()) {
		if ($a->{'up'} && $a->{'virtual'} eq '' &&
		    $main::net_object->iface_type($a->{'name'}) =~ /ethernet/i) {
			$self->{'config_hash'}->{'iface'} = $a->{'fullname'};
			last;
			}
		}
	if (!$self->{'config_hash'}->{'iface'}) {
		&check_error($self->text('index_eiface', "/config.cgi?$self->{'module_name'}"));
		}
	$self->write_file("$self->{'module_config_directory'}/config", \%{$self->{'config_hash'}});
	}
print $self->text('check_ifaceok', "<tt>$self->{'config_hash'}->{'iface'}</tt>"),"<p>\n";

# Make sure local group exists
if ($self->{'config_hash'}->{'localgroup'} && !defined(getgrnam($self->{'config_hash'}->{'localgroup'}))) {
	&check_error($self->text('index_elocal', "<tt>$self->{'config_hash'}->{'localgroup'}</tt>",
			   "/config.cgi?$self->{'module_name'}"));
	}

$self->{'config_hash'}->{'home_quotas'} = '';
$self->{'config_hash'}->{'mail_quotas'} = '';
$self->{'config_hash'}->{'group_quotas'} = '';
if ($self->{'config_hash'}->{'quotas'}) {
	# Make sure quotas are enabled, and work out where they are needed
	$self->require_useradmin();
	if (!$self->{'home_base'}) {
		print "<p><b>",$self->text('index_ehomebase'),"</b><p>\n";
		}
	else {
		$self->foreign_require("mount", "mount-lib.pl");
		$self->{'mail_base'} = $self->simplify_path($self->resolve_links(
				$self->mail_system_base()));
		($self->{'home_mtab'}, $self->{'home_fstab'}) = &mount_point($self->{'home_base'});
		($self->{'mail_mtab'}, $self->{'mail_fstab'}) = &mount_point($self->{'mail_base'});
		if (!$self->{'home_mtab'}) {
			print "<p><b>",$self->text('index_ehomemtab',
		    		"<tt>$self->{'home_base'}</tt>"),"</b><p>\n";
			}
		elsif (!$self->{'mail_mtab'}) {
			print "<p><b>",$self->text('index_emailmtab',
				"<tt>$self->{'mail_base'}</tt>"),"</b><p>\n";
			}
		else {
			# Check if quotas are enabled for home filesystem
			local $nohome;
			if (!($self->{'home_mtab'}->[4] = $main::quota_object->quota_can(
				$self->{'home_mtab'}, $self->{'home_fstab'}))%2 ||
			    !$main::quota_object->quota_now($self->{'home_mtab'}, $self->{'home_fstab'})) {
				$nohome++;
				}
			else {
				if ($self->{'home_mtab'}->[4] >= 2) {
					# Group quotas are active too
					$self->{'config_hash'}->{'group_quotas'} = 1;
					}
				}

			if ($self->{'home_mtab'}->[0] eq $self->{'mail_mtab'}->[0]) {
				# Home and mail are the same filesystem
				if ($nohome) {
					# Neither are enabled
					$self->{'qerr'} = $self->text('index_equota2',
					    "<tt>$self->{'home_mtab'}->[0]</tt>",
					    "<tt>$self->{'home_base'}</tt>",
					    "<tt>$self->{'mail_base'}</tt>");
					}
				else {
					# Both are enabled
					$self->{'config_hash'}->{'home_quotas'} =
						$self->{'home_mtab'}->[0];
					$self->{'config_hash'}->{'mail_quotas'} =
						$self->{'home_mtab'}->[0];
					}
				}
			else {
				# Different .. so check mail too
				local $nomail;
				if (!($self->{'mail_mtab'}->[4] = $main::quota_object->quota_can(
					$self->{'mail_mtab'}, $self->{'mail_fstab'})) ||
				    !$main::quota_object->quota_now($self->{'mail_mtab'},
						       $self->{'mail_fstab'})) {
					$nomail++;
					}
				if ($nohome) {
					$self->{'qerr'} = $self->text('index_equota3',
					    "<tt>$self->{'home_mtab'}->[0]</tt>",
					    "<tt>$self->{'home_base'}</tt>");
					}
				else {
					$self->{'config_hash'}->{'home_quotas'} =
						$self->{'home_mtab'}->[0];
					}
				if ($nomail) {
					$self->{'qerr'} = $self->text('index_equota4',
					    "<tt>$self->{'mail_mtab'}->[0]</tt>",
					    "<tt>$self->{'mail_base'}</tt>");
					}
				else {
					$self->{'config_hash'}->{'mail_quotas'} =
						$self->{'mail_mtab'}->[0];
					}
				}
			}
		}
	}
if ($self->{'qerr'}) {
	print "$self->{'qerr'}<p>\n";
	}
elsif (!$self->{'config_hash'}->{'group_quotas'}) {
	print "$self->{'text_hash'}->{'check_nogroup'}<p>\n";
	}
else {
	print "$self->{'text_hash'}->{'check_group'}<p>\n";
	}

# Check for FTP shells in /etc/shells
open(SHELLS, "/etc/shells");
while(<SHELLS>) {
	s/\r|\n//g;
	s/#.*$//;
	$self->{'shells_hash'}->{$_}++;
	}
close(SHELLS);
if ($self->{'shells_hash'}->{$self->{'config_hash'}->{'shell'}}) {
	print $self->text('check_eshell', "<tt>$self->{'config_hash'}->{'shell'}</tt>",
		    "<tt>/etc/shells</tt>"),"<p>\n";
	}
if (!$self->{'shells_hash'}->{$self->{'config_hash'}->{'ftp_shell'}}) {
	print $self->text('check_eftpshell', "<tt>$self->{'config_hash'}->{'ftp_shell'}</tt>",
		    "<tt>/etc/shells</tt>"),"<p>\n";
	}

# Check for problem module config settings
if ($self->{'config_hash'}->{'all_namevirtual'} && $self->{'config_hash'}->{'dns_ip'}) {
	&check_error($self->text('check_enamevirt', $self->{'clink'}));
	}

# Make sure LDAP module is set up, if selected
if ($self->{'config_hash'}->{'ldap'}) {
	$self->{'ldap'} = &ldap_useradmin::ldap_connect(1);
	if (!ref($self->{'ldap'})) {
		&check_error($self->text('check_eldap', $self->{'ldap'}, $self->{'clink'},
				   "../ldap-useradmin/"));
		}
	else {
		$self->require_useradmin();
		if (!defined(&ldap_useradmin::list_users)) {
			&check_error($self->text('check_eldap2', $self->{'clink'}, 1.164));
			}
		else {
			print $self->text('check_ldap'),"<p>\n";
			}
		}
	}

$self->{'config_hash'}->{'last_check'} = time()+1;
$self->{'config_hash'}->{'disable'} =~ s/user/unix/g;	# changed since last release
$self->lock_file($self->{'module_config_file'});
$self->write_file($self->{'module_config_file'}, \%{$self->{'config_hash'}});
$self->unlock_file($self->{'module_config_file'});
#$self->lock_file("$self->{'module_config_directory'}/last-config");
$self->write_file("$self->{'module_config_directory'}/last-config", \%{$self->{'config_hash'}});
#$self->unlock_file("$self->{'module_config_directory'}/last-config");

print "</ul>\n";
print "<b>$self->{'text_hash'}->{'check_done'}</b><p>\n";

# See if any options effecting Webmin users have changed
$self->{'webminchanged'} = 0;
foreach $k (keys %{$self->{'config_hash'}}) {
	if ($k =~ /^avail_/ || $k eq 'leave_acl' ||
	    $self->indexof($k, @{$self->{'features_array'}}) >= 0) {
		$self->{'webminchanged'}++ if ($self->{'config_hash'}->{$k} != $self->{'lastconfig_hash'}->{$k});
		}
	}

if ($self->{'webminchanged'}) {
	if ($self->{'config_hash'}->{'post_check'}) {
		# Update all Webmin users
		$self->modify_all_webmin();
		}
	else {
		# Just offer to update
		print "<form action=all_webmin.cgi>\n";
		print "$self->{'text_hash'}->{'check_needupdate'}<p>\n";
		print "<input type=submit value='$self->{'text_hash'}->{'check_updatenow'}'>\n";
		print "</form>\n";
		}
	}
$self->webmin_log("check");

$self->ui_print_footer("", $self->{'text_hash'}->{'index_return'});

1;
		
}

sub check_error
{
	shift if ($self == $_[0]);

print "<p>$_[0]<p>\n";
print "</ul>\n";
print "<b>$self->{'text_hash'}->{'check_failed'}</b><p>\n";
$self->ui_print_footer("", $self->{'text_hash'}->{'index_return'});
exit;
}

# mount_point(dir)
# Returns both the mtab and fstab details for the parent mount for a directory
sub mount_point
{
	shift if ($self == $_[0]);

local $dir = $self->resolve_links($_[0]);
local @mounts = $main::mount_object->list_mounts();
local @mounted = $main::mount_object->list_mounted();
@mounts = @mounted if (!@mounts);
foreach $m (sort { length($b->[0]) <=> length($a->[0]) } @mounted) {
	if ($dir eq $m->[0] || $m->[0] eq "/" ||
	    substr($dir, 0, length($m->[0])+1) eq "$m->[0]/") {
		local ($m2) = grep { $_->[0] eq $m->[0] } @mounts;
		if ($m2) {
			return ($m, $m2);
			}
		}
	}
$self->error("Failed to find mount point for $dir");
}


1;

