#!/usr/bin/perl
# check.cgi
# Make sure that all enabled features are valid, and check the status of quotas

&master_admin() || &error($text{'check_ecannot'});

&read_file("$module_config_directory/last-config", \%lastconfig);

($module_name eq "virtual-server") ? $clink = "../config.cgi?$module_name" : 
	$clink = "../config.cgi?virtual-server&conf_file=config_websites.info&ln=1&title=config_s_1";
	
# Make sure networking is supported
if (!&foreign_check("net")) {
	&foreign_require("net", "net-lib.pl");
	if (!defined(&net::boot_interfaces)) {
		&check_error(&text('index_enet'));
		}
	}

if ($config{'dns'}) {
	# Make sure BIND is installed
	&foreign_installed("bind8", 1) == 2 ||
		&check_error(&text('index_ebind', "/bind8/", $clink));
	}

if ($config{'mail'}) {
	if ($config{'mail_system'} == 3) {
		# Work out which mail server we have
		if (&postfix_installed()) {
			$config{'mail_system'} = 0;
			}
		elsif (&qmail_vpopmail_installed()) {
			$config{'mail_system'} = 5;
			}
		elsif (&qmail_ldap_installed()) {
			$config{'mail_system'} = 4;
			}
		elsif (&qmail_installed()) {
			$config{'mail_system'} = 2;
			}
		elsif (&sendmail_installed()) {
			$config{'mail_system'} = 1;
			}
		else {
			&check_error(&text('index_email'));
			}
#		print &text('check_detected', &mail_system_name()),"<p>\n";
		&save_module_config();
		}
	if ($config{'mail_system'} == 1) {
		# Make sure sendmail is installed
		if (!&sendmail_installed()) {
			&check_error(&text('index_esendmail', '/sendmail/', $clink));
			}
		# Check that aliases and virtusers are configured
		&require_mail();
		@$sendmail_afiles ||
			&check_error(&text('index_esaliases', '/sendmail/'));
		$sendmail_vdbm ||
			&check_error(&text('index_esvirts', '/sendmail/'));
		if ($config{'generics'}) {
			$sendmail_gdbm ||
		    		&check_error(&text('index_esgens',
					     '/sendmail/', $clink));
			}
		}
	elsif ($config{'mail_system'} == 0) {
		# Make sure postfix is installed
		if (!&postfix_installed()) {
			&check_error(&text('index_epostfix', '/postfix/', $clink));
			}
		# Check that aliases and virtusers are configured
		&require_mail();
		@$postfix_afiles ||
			&check_error(&text('index_epaliases', '/postfix/'));
		$virtual_maps =~ /^[a-z0-9]+:\//i ||
			&check_error(&text('index_epvirts', '/postfix/'));
		if ($config{'generics'}) {
			$canonical_maps ||
				check_error(&text('index_epgens',
					    '/postfix/', $clink));
			}
		if ($config{'append_style'} == 6) {
			&check_error(&text('check_estyle6', $clink));
			}
		}
	elsif ($config{'mail_system'} == 2) {
		# Make sure qmail is installed
		if (!&qmail_installed()) {
			&check_error(&text('index_eqmail', '/qmailadmin/', $clink));
			}
		if ($config{'generics'}) {
			&check_error(&text('index_eqgens', $clink));
			}
		if ($config{'append_style'} == 6) {
			&check_error(&text('check_qmailmode6', $clink));
			}
		}
	elsif ($config{'mail_system'} == 4) {
		# Make sure qmail with LDAP is installed
		if (!&qmail_ldap_installed()) {
			&check_error(&text('index_eqmailldap', '/qmailadmin/', $clink));
			}
		if ($config{'generics'}) {
			&check_error(&text('index_eqgens', $clink));
			}
		if (!gethostbyname($config{'ldap_host'})) {
			&check_error(&text('index_eqmailhost', $clink));
			}
		if (!$config{'ldap_base'}) {
			&check_error(&text('index_eqmailbase', $clink));
			}
		local $lerr = &connect_qmail_ldap(1);
		if (!ref($lerr)) {
			&check_error(&text('index_eqmailconn', $lerr, $clink));
			}
		}
	elsif ($config{'mail_system'} == 5) {
		# Make sure qmail with VPOPMail is installed
		if (!&qmail_vpopmail_installed()) {
			&check_error(&text('index_evpopmail', '/qmailadmin/', $clink));
			}
		if ($config{'generics'}) {
			&check_error(&text('index_eqgens', $clink));
			}
		}
	}

if ($config{'mailman'}) {
	# Make sure Mailman is installed
	if (!&mailman_installed()) {
		&check_error(&text('index_emailman', '/mailman/', $clink));
		}
	if ($config{'mail_system'} != 0) {
		&check_error(&text('index_emailman2'));
		}
	}
	
if ($config{'web'}) {
	# Make sure Apache is installed
	&foreign_installed("apache", 1) == 2 ||
		&check_error(&text('index_eapache', "/apache/", $clink));
	}

if ($config{'webalizer'}) {
	# Make sure Webalizer is installed, and that global directives are OK
	# XXX is the path checking really needed?
	$config{'web'} || &check_error(&text('check_edepwebalizer', $clink));
	&foreign_installed("webalizer", 1) == 2 ||
		&check_error(&text('index_ewebalizer', "/webalizer/", $clink));
	&foreign_require("webalizer", "webalizer-lib.pl");
	$conf = &webalizer::get_config();
	$current = &webalizer::find_value("IncrementalName", $conf);
	$history = &webalizer::find_value("HistoryName", $conf);
	if ($current =~ /^\//) {
		&check_error(&text('check_current', "/webalizer/"));
		}
	elsif ($history =~ /^\//) {
		&check_error(&text('check_history', "/webalizer/"));
		}
	}

if ($config{'ssl'}) {
	# Make sure openssl is installed, that Apache supports mod_ssl,
	# and that port 443 is in use
	$config{'web'} || &check_error(&text('check_edepssl', $clink));
	&has_command("openssl") ||
	    &check_error(&text('index_eopenssl', "<tt>openssl</tt>", $clink));

	&require_apache();
	local $conf = &apache::get_config();
	local @loads = &apache::find_directive_struct("LoadModule", $conf);
	local ($l, $hasmod);
	foreach $l (@loads) {
		$hasmod++ if ($l->{'words'}->[1] =~ /mod_ssl/);
		}
	local ($aver, $amods) = &apache::httpd_info();
	$hasmod++ if (&indexof("mod_ssl", @$amods) >= 0);
	$hasmod++ if ($apache::httpd_modules{'mod_ssl'});
	$hasmod ||
	    &check_error(&text('index_emodssl', "<tt>mod_ssl</tt>", $clink));

	local @listens = &apache::find_directive_struct("Listen", $conf);
	foreach $l (@listens) {
		$haslisten++ if ($l->{'words'}->[0] =~ /^(\S+:)?$default_web_sslport$/);
		}
	local @ports = &apache::find_directive_struct("Port", $conf);
	foreach $l (@ports) {
		$haslisten++ if ($l->{'words'}->[0] == $default_web_sslport);
		}
	$haslisten ||
	    &check_error(&text('index_emodssl2', $default_web_sslport, $clink));
	}

if ($config{'mysql'}) {
	# Make sure MySQL is installed
	&foreign_installed("mysql", 1) == 2 ||
		&check_error(&text('index_emysql', "/mysql/", $clink));
	}

if ($config{'postgres'}) {
	# Make sure PostgreSQL is installed
	&foreign_installed("postgresql", 1) == 2 ||
		&check_error(&text('index_epostgres', "/postgresql/", $clink));
	}

if ($config{'ftp'}) {
	if ($config{'ftp_system'} == 0) {
		# Work out which mail server we have
		if (&proftpd_installed()) {
			$config{'ftp_system'} = 1;
			}
		elsif (&vsftpd_installed()) {
			$config{'ftp_system'} = 2;
			}
		else {
			&check_error(&text('index_eftp'));
			}
		&save_module_config();
		}
	if ($config{'ftp_system'} == 1) {
		# Make sure ProFTPd is installed, and that the ftp user exists
		&foreign_installed("proftpd", 1) == 2 || 								&check_error(&text('index_eproftpd', "/proftpd/", $clink));
 		$err = &check_proftpd_template();
		$err && &check_error(&text('check_proftpd', $err));
		}
	}

if ($config{'logrotate'}) {
	# Make sure logrotate is installed
	&foreign_installed("logrotate", 1) == 2 ||
		&check_error(&text('index_elogrotate', "/logrotate/", $clink));
	}

# Check all plugins
foreach $p (@plugins) {
	$err = &plugin_call($p, "feature_check");
	if ($err) {
		&check_error($err);
		}
	else {
		$pname = &plugin_call($p, "feature_name");
		}
	}

if (!$config{'iface'}) {
	# Work out the network interface automatically
	&foreign_require("net", "net-lib.pl");
	foreach $a (&net::active_interfaces()) {
		if ($a->{'up'} && $a->{'virtual'} eq '' &&
		    &net::iface_type($a->{'name'}) =~ /ethernet/i) {
			$config{'iface'} = $a->{'fullname'};
			last;
			}
		}
	if (!$config{'iface'}) {
		&check_error(&text('index_eiface', $clink));
		}
	&write_file("$module_config_directory/config", \%config);
	}

# Make sure local group exists
if ($config{'localgroup'} && !defined(getgrnam($config{'localgroup'}))) {
	&check_error(&text('index_elocal', "<tt>$config{'localgroup'}</tt>", $clink));
	}

$config{'home_quotas'} = '';
$config{'mail_quotas'} = '';
$config{'group_quotas'} = '';
if ($config{'quotas'}) {
	# Make sure quotas are enabled, and work out where they are needed
	&require_useradmin();
	if (!$home_base) {
		&check_error(&text('index_ehomebase'));
		}
	else {
		&foreign_require("mount", "mount-lib.pl");
		$mail_base = &simplify_path(&resolve_links(
				&mail_system_base()));
		($home_mtab, $home_fstab) = &mount_point($home_base);
		($mail_mtab, $mail_fstab) = &mount_point($mail_base);
		if (!$home_mtab) {
			&check_error(&text('index_ehomemtab',"<tt>$home_base</tt>"));
			}
		elsif (!$mail_mtab) {
			&check_error(&text('index_emailmtab',"<tt>$mail_base</tt>"));
			}
		else {
			# Check if quotas are enabled for home filesystem
			local $nohome;
			if (!($home_mtab->[4] = &quota::quota_can(
				$home_mtab, $home_fstab))%2 ||
			    !&quota::quota_now($home_mtab, $home_fstab)) {
				$nohome++;
				}
			else {
				if ($home_mtab->[4] >= 2) {
					# Group quotas are active too
					$config{'group_quotas'} = 1;
					}
				}

			if ($home_mtab->[0] eq $mail_mtab->[0]) {
				# Home and mail are the same filesystem
				if ($nohome) {
					# Neither are enabled
					$qerr = &text('index_equota2',
					    "<tt>$home_mtab->[0]</tt>",
					    "<tt>$home_base</tt>",
					    "<tt>$mail_base</tt>");
					}
				else {
					# Both are enabled
					$config{'home_quotas'} =
						$home_mtab->[0];
					$config{'mail_quotas'} =
						$home_mtab->[0];
					}
				}
			else {
				# Different .. so check mail too
				local $nomail;
				if (!($mail_mtab->[4] = &quota::quota_can(
					$mail_mtab, $mail_fstab)) ||
				    !&quota::quota_now($mail_mtab,
						       $mail_fstab)) {
					$nomail++;
					}
				if ($nohome) {
					$qerr = &text('index_equota3',
					    "<tt>$home_mtab->[0]</tt>",
					    "<tt>$home_base</tt>");
					}
				else {
					$config{'home_quotas'} =
						$home_mtab->[0];
					}
				if ($nomail) {
					$qerr = &text('index_equota4',
					    "<tt>$mail_mtab->[0]</tt>",
					    "<tt>$mail_base</tt>");
					}
				else {
					$config{'mail_quotas'} =
						$mail_mtab->[0];
					}
				}
			}
		}
		if (!$qerr && !$config{'group_quotas'}) {
			$qerr = $text{'check_nogroup'};
		}
	}
if ($qerr) {
	&check_error($qerr);
	}

# Check for FTP shells in /etc/shells
open(SHELLS, "/etc/shells");
while(<SHELLS>) {
	s/\r|\n//g;
	s/#.*$//;
	$shells{$_}++;
	}
close(SHELLS);
if ($shells{$config{'shell'}}) {
	&check_error(&text('check_eshell', "<tt>$config{'shell'}</tt>",
		    "<tt>/etc/shells</tt>"));
	}
if (!$shells{$config{'ftp_shell'}}) {
	&check_error(&text('check_eftpshell', "<tt>$config{'ftp_shell'}</tt>",
		    "<tt>/etc/shells</tt>"));
	}

# Check for problem module config settings
if ($config{'all_namevirtual'} && $config{'dns_ip'}) {
	&check_error(&text('check_enamevirt', $clink));
	}

# Make sure LDAP module is set up, if selected
if ($config{'ldap'}) {
	$ldap = &ldap_useradmin::ldap_connect(1);
	if (!ref($ldap)) {
		&check_error(&text('check_eldap', $ldap, $clink,
				   "../ldap-useradmin/"));
		}
	else {
		&require_useradmin();
		if (!defined(&ldap_useradmin::list_users)) {
			&check_error(&text('check_eldap2', $clink, 1.164));
			}
		}
	}

$config{'last_check'} = time()+1;
$config{'disable'} =~ s/user/unix/g;	# changed since last release
&lock_file($module_config_file);
&write_file($module_config_file, \%config);
&unlock_file($module_config_file);
#&lock_file("$module_config_directory/last-config");
&write_file("$module_config_directory/last-config", \%config);
#&unlock_file("$module_config_directory/last-config");

#print "</ul>\n";

# See if any options effecting Webmin users have changed
$webminchanged = 0;
foreach $k (keys %config) {
	if ($k =~ /^avail_/ || $k eq 'leave_acl' ||
	    &indexof($k, @features) >= 0) {
		$webminchanged++ if ($config{$k} != $lastconfig{$k});
		}
	}

if ($webminchanged) {
		# Update all Webmin users
		&modify_all_webmin();
	}
&webmin_log("check");

sub check_error
{
print "<p class=warn>$_[0]</p>\n";
}

# mount_point(dir)
# Returns both the mtab and fstab details for the parent mount for a directory
sub mount_point
{
local $dir = &resolve_links($_[0]);
local @mounts = &mount::list_mounts();
local @mounted = &mount::list_mounted();
@mounts = @mounted if (!@mounts);
foreach $m (sort { length($b->[0]) <=> length($a->[0]) } @mounted) {
	if ($dir eq $m->[0] || $m->[0] eq "/" ||
	    substr($dir, 0, length($m->[0])+1) eq "$m->[0]/") {
		local ($m2) = grep { $_->[0] eq $m->[0] } @mounts;
		if ($m2) {
			return ($m, $m2);
			}
		}
	}
&error("Failed to find mount point for $dir");
}

1;
1;
