# Functions for managing a virtual FTP server

$feature_depends{'ftp'} = [ 'dir', 'virt' ];

sub require_ftp
{
 return if ($require_ftp++);
 if ($config{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  &foreign_require("proftpd", "proftpd-lib.pl");
 }
 elsif ($config{'ftp_system'} == 2)
 {
  #using VSFTPd for FTP
  &foreign_require("vsftpd", "vsftpd-lib.pl");
 }
}

# setup_ftp(&domain, ftp_root_dir)
# Setup a virtual FTP server for some domain
sub setup_ftp_rootdir {
	local $ftp = $_[1];
	if (!-d $ftp) {
		&system_logged("mkdir '$ftp' 2>/dev/null");
		&system_logged("chmod 755 '$ftp'");
		&system_logged("chown $_[0]->{'uid'}:$_[0]->{'ugid'} '$ftp'");
	}
}

# setup_ftp(&domain)
# Setup a virtual FTP server for some domain
sub setup_ftp
{
&require_ftp();
# Get the template
local $tmpl = &get_template($_[0]->{'template'});

# Define directory for FTP root
local ($fdir) = ($tmpl->{'ftp_dir'} || 'ftp');
local $ftp = "$_[0]->{'home'}/$fdir";
if ($config{'ftp_system'} == 1) {
	# Set ProFTPD up for FTP feature
	&$first_print($text{'setup_proftpd'});

	# Get the template
	local @dirs = &proftpd_template($tmpl->{'ftp'}, $_[0]);

	# Add the directives
	local $conf = &proftpd::get_config();
	local $l = $conf->[@$conf - 1];
	&lock_file($l->{'file'});
	local $lref = &read_file_lines($l->{'file'});
	local @lines = ( "<VirtualHost $_[0]->{'ip'}>" );
	push(@lines, @dirs);
	push(@lines, "</VirtualHost>");
	push(@$lref, @lines);
  
	&flush_file_lines();
	&unlock_file($l->{'file'});

	# Create directory for FTP root
	&setup_ftp_rootdir($_[0], $ftp);
	&$second_print($text{'setup_done'});
	&register_post_action(\&restart_proftpd);
	undef(@proftpd::get_config_cache);
	}
elsif ($config{'ftp_system'} == 2) {
	# Set VsFTPd up for FTP feature
	&$first_print($text{'setup_vsftpd'});
  
	# Create directory for FTP root
	&setup_ftp_rootdir($_[0], $ftp);

	# Define vsftpd virtual server config file
	local $conf_file = "$vsftpd::config{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  
	if ($tmpl->{'ftp_vsftpd'}) {
		# Get and use the user-defined template
		local %vars = %{$_[0]};
		$vars{'serial'} = $serial;
		&open_tempfile(CONF, ">$conf_file");
		local $conf_ctx = &vsftpd_template($tmpl->{'ftp_vsftpd'}, \%vars);
# 		local $conf_ctx = &substitute_template($config{'ftp_vsftpd'}, \%vars);
# 		&print_tempfile(CONF, $conf_ctx);
		&print_tempfile(CONF, $conf_ctx);
		&close_tempfile(CONF);
		}
	else {
		# Copy default VSFTPd server configuration file
		&copy_source_dest($vsftpd::config{'vsftpd_conf'}, $conf_file);
  
		# Set new VSFTPd virtual server configuration options
		local $conf = &vsftpd::parse_config_file($conf_file);
		&vsftpd::set_config_value('listen_address', $_[0]->{'ip'}, $conf);
		&vsftpd::set_config_value('anon_root', $ftp, $conf);
		&vsftpd::set_config_value('xferlog_file',"$_[0]->{'home'}/logs/ftp.log",$conf);
		&vsftpd::set_config_value('ftpd_banner',"Welcome to $_[0]->{'dom'}",$conf);
		#&vsftpd::save_config_file($conf_file,$conf);
		&vsftpd::save_new_options($conf,$conf_file);
		}

	#restart daemon vsftpd
	&vsftpd::restart_vsftpd() if &vsftpd::status_vsftpd();
  
	&$second_print($text{'setup_done'});
	}
}

# delete_ftp(&domain)
sub delete_ftp
{
 &require_ftp();
 if ($config{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # Delete the virtual server from the ProFTPd config
  local $conf = &proftpd::get_config();
  &$first_print($text{'delete_proftpd'});
  local ($virt, $vconf) = &get_proftpd_virtual($_[0]->{'ip'});
  if ($virt) {
	&lock_file($virt->{'file'});
	local $lref = &read_file_lines($virt->{'file'});
	splice(@$lref, $virt->{'line'}, $virt->{'eline'} - $virt->{'line'} + 1);
	&flush_file_lines();
	&unlock_file($virt->{'file'});

	&$second_print($text{'setup_done'});
	&register_post_action(\&restart_proftpd);
	undef(@proftpd::get_config_cache);
	}
  else {
	&$second_print($text{'delete_noproftpd'});
	}
 }
 elsif ($config{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  &$first_print($text{'delete_proftpd'});
  local $conf_file = "$vsftpd::config{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  $conf_file = "$conf_file.disable" unless (-e $conf_file);
  if (-r $conf_file) {
	#delete the vsftpd domain config
	&unlink_logged($conf_file);
	
	#restart daemon vsftpd
  	&vsftpd::restart_vsftpd() if &vsftpd::status_vsftpd();
	}
  &$second_print($text{'setup_done'});
 }
}

# modify_ftp(&domain, &olddomain)
sub modify_ftp
{
 &require_ftp();
 if ($config{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # If the server has changed IP address, update the ProFTPd virtual server
  local $rv = 0;
  local $conf = &proftpd::get_config();
  local ($virt, $vconf, $anon, $aconf) = &get_proftpd_virtual($_[1]->{'ip'});
  return 0 if (!$virt);
  &lock_file($virt->{'file'});
  if ($_[0]->{'dom'} ne $_[1]->{'dom'}) {
	# Update domain name in ProFTPd virtual server
	&$first_print($text{'save_proftpd2'});
	local $sname = &proftpd::find_directive_struct("ServerName", $vconf);
	if ($sname) {
		&proftpd::save_directive(
			"ServerName", [ $_[0]->{'dom'} ], $vconf, $conf);
		}
	$rv++;
	&$second_print($text{'setup_done'});
	}
  if ($_[0]->{'home'} ne $_[1]->{'home'} && $anon) {
	# Update anonymous FTP directory in ProFTPd virtual server
	&$first_print($text{'save_proftpd3'});
	local $lref = &read_file_lines($anon->{'file'});
	$lref->[$anon->{'line'}] =~ s/$_[1]->{'home'}/$_[0]->{'home'}/;
	&flush_file_lines();
	$rv++;
	&$second_print($text{'setup_done'});
	}
  if ($_[0]->{'ip'} ne $_[1]->{'ip'}) {
	# Update IP address in ProFTPd virtual server
	&$first_print($text{'save_proftpd'});
	local $lref = &read_file_lines($virt->{'file'});
	$lref->[$virt->{'line'}] = "<VirtualHost $_[0]->{'ip'}>";
	&flush_file_lines();
	$rv++;
	&$second_print($text{'setup_done'});
	}
  &unlock_file($virt->{'file'});
  &register_post_action(\&restart_proftpd) if ($rv);
  return $rv;
 }
 elsif ($config{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  local $conf_file = "$vsftpd::config{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  local $old_file = "$vsftpd::config{'vsftpd_dir'}/vsftpd.$_[1]->{'dom'}.conf";
  
  my $change;
  if ($_[0]->{'dom'} ne $_[1]->{'dom'}) {
	# Update domain name in VsFTPd virtual server
	&$first_print($text{'save_proftpd2'});
	&rename_logged($old_file, $conf_file);
	
	#set new vsftpd config options
	my $conf = &vsftpd::parse_config_file($conf_file);
	&vsftpd::set_config_value('ftpd_banner',"Welcome to $_[0]->{'dom'}",$conf);
	#&vsftpd::save_config_file($conf_file, $conf);
	&vsftpd::save_new_options($conf,$conf_file);
	
	$change = 1;
	&$second_print($text{'setup_done'});
	}
  if ($_[0]->{'home'} ne $_[1]->{'home'}) {
	# Update anonymous FTP directory in VsFTPd virtual server
	&$first_print($text{'save_proftpd3'});
	
	#set new vsftpd config options
	my $conf = &vsftpd::parse_config_file($conf_file);
	&vsftpd::set_config_value('xferlog_file',"$_[0]->{'home'}/logs/ftp.log",$conf);
	#&vsftpd::save_config_file($conf_file,$conf);
	&vsftpd::save_new_options($conf,$conf_file);
	
	$change = 1;
	&$second_print($text{'setup_done'});
	}
  if ($_[0]->{'ip'} ne $_[1]->{'ip'}) {
	# Update IP address in VsFTPd virtual server
	&$first_print($text{'save_proftpd'});
	
	#set new vsftpd config options
	my $conf = &vsftpd::parse_config_file($conf_file);
	&vsftpd::set_config_value('listen_address',$_[0]->{'ip'},$conf);
	#&vsftpd::save_config_file($conf_file,$conf);
	&vsftpd::save_new_options($conf,$conf_file);
	
	$change = 1;
	&$second_print($text{'setup_done'});
	}
  #restart daemon vsftpd
  &vsftpd::restart_vsftpd() if ($change && &vsftpd::status_vsftpd());
 }
}

# disable_ftp(&domain)
sub disable_ftp
{
 &require_ftp();
 if ($config{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # Disable FTP for this server by adding a deny directive
  &$first_print($text{'disable_proftpd'});
  local ($virt, $vconf, $anon, $aconf) = &get_proftpd_virtual($_[0]->{'ip'});
  if ($anon) {
	&lock_file($anon->{'file'});
	local @limit = &proftpd::find_directive_struct("Limit", $aconf);
	local ($login) = grep { $_->{'words'}->[0] eq "LOGIN" } @limit;
	if (!$login) {
		local $lref = &read_file_lines($anon->{'file'});
		splice(@$lref, $anon->{'eline'}, 0,
		       "<Limit LOGIN>", "DenyAll", "</Limit>");
		&flush_file_lines();
		}
	&unlock_file($anon->{'file'});
	&$second_print($text{'setup_done'});
	&register_post_action(\&restart_proftpd);
	}
  else {
	&$second_print($text{'delete_noproftpd'});
	}
 }
 elsif ($config{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  &$first_print($text{'disable_proftpd'});
  
  #rename config file
  local $conf_file = "$vsftpd::config{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  &rename_file($conf_file,"$conf_file.disable");
  
  #restart daemon vsftpd
  &vsftpd::restart_vsftpd() if &vsftpd::status_vsftpd();
  
  &$second_print($text{'setup_done'});
 }
}

# enable_ftp(&domain)
sub enable_ftp
{
 &require_ftp();
 if ($config{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # Enable FTP for this server by removing the deny directive
  &$first_print($text{'enable_proftpd'});
  local ($virt, $vconf, $anon, $aconf) = &get_proftpd_virtual($_[0]->{'ip'});
  if ($virt) {
	&lock_file($anon->{'file'});
	local @limit = &proftpd::find_directive_struct("Limit", $aconf);
	local ($login) = grep { $_->{'words'}->[0] eq "LOGIN" } @limit;
	if ($login) {
		local $lref = &read_file_lines($anon->{'file'});
		splice(@$lref, $login->{'line'},
		       $login->{'eline'} - $login->{'line'} + 1);
		&flush_file_lines();
		}
	&unlock_file($anon->{'file'});
	&$second_print($text{'setup_done'});
	&register_post_action(\&restart_proftpd);
	}
  else {
	&$second_print($text{'delete_noproftpd'});
	}
 }
 elsif ($config{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  &$first_print($text{'enable_proftpd'});
  
  #rename config file
  local $conf_file = "$vsftpd::config{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  &rename_file("$conf_file.disable",$conf_file);
  
  #restart daemon vsftpd
  &vsftpd::restart_vsftpd() if &vsftpd::status_vsftpd();
  
  &$second_print($text{'setup_done'});
 }
}

# check_ftp_clash(&domain, [field])
sub check_ftp_clash
{
 &require_ftp();
 if ($config{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # Returns 1 if a ProFTPd server already exists for some domain
  if (!$_[1] || $_[1] eq 'ip') {
	local ($cvirt, $cconf) = &get_proftpd_virtual($_[0]->{'ip'});
	return $cvirt ? 1 : 0;
	}
  return 0;
 }
 elsif ($config{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  # Returns 1 if a VsFTPd server already exists for some domain
  local $conf_file = "$vsftpd::config{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  return 1 if (-r $conf_file);
  return 0;
 }
}

# backup_ftp(&domain, file)
sub backup_ftp
{
 &require_ftp();
 if ($config{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # Save the virtual server's ProFTPd config as a separate file
  &$first_print($text{'backup_proftpdcp'});
  local ($virt, $vconf) = &get_proftpd_virtual($_[0]->{'ip'});
  local $lref = &read_file_lines($virt->{'file'});
  local $l;
  &open_tempfile(FILE, ">$_[1]");
  foreach $l (@$lref[$virt->{'line'} .. $virt->{'eline'}]) {
	&print_tempfile(FILE, "$l\n");
	}
  &close_tempfile(FILE);
  &$second_print($text{'setup_done'});
  return 1;
 }
 elsif ($config{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  # Save the virtual server's VsFTPd config as a separate file
  &$first_print($text{'backup_proftpdcp'});
  
  #backup domain vsftpd config
  local $conf_file = "$vsftpd::config{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  $conf_file = "$conf_file.disable" unless (-e $conf_file);
  &copy_source_dest($conf_file,"$_[1]");
  
  &$second_print($text{'setup_done'});
  return 1;
 }
}

# restore_ftp(&domain, file)
sub restore_ftp
{
 &require_ftp();
 if ($config{'ftp_system'} == 1)
 {
  #using ProFTPD for FTP
  # Update the virtual server's ProFTPd configuration from a file. Does not
  # change the actual <Virtualhost> lines!
  &$first_print($text{'restore_proftpdcp'});
  local ($virt, $vconf) = &get_proftpd_virtual($_[0]->{'ip'});
  local $srclref = &read_file_lines($_[1]);
  local $dstlref = &read_file_lines($virt->{'file'});
  &lock_file($virt->{'file'});
  splice(@$dstlref, $virt->{'line'}+1, $virt->{'eline'}-$virt->{'line'}-1,
       @$srclref[1 .. @$srclref-2]);
  &flush_file_lines();
  &unlock_file($virt->{'file'});
  &$second_print($text{'setup_done'});

  &register_post_action(\&restart_proftpd);
  return 1;
 }
 elsif ($config{'ftp_system'} == 2)
 {
  #using VsFTPd for FTP
  # Restore the virtual server's VsFTPd configuration from a file
  &$first_print($text{'restore_proftpdcp'});
  
  #restore domain vsftpd config
  local $conf_file = "$vsftpd::config{'vsftpd_dir'}/vsftpd.$_[0]->{'dom'}.conf";
  &copy_source_dest($_[1],$conf_file);
  
  &$second_print($text{'setup_done'});
  
  &disable_ftp if ($_[0]->{'disabled'});
  
  return 1;
 }
}

# bandwidth_ftp(&domain, start, &bw-hash)
# Searches through FTP log files for records after some date, and updates the
# day counters in the given hash
sub bandwidth_ftp
{
local $log;
if ($config{'ftp_system'} == 2) {
	# Using VSFTPd for FTP
	$log = &get_vsftpd_log($_[0]->{'dom'});
	}
else {
	# Using ProFTPD for FTP
	$log = &get_proftpd_log($_[0]->{'ip'});
	}
if ($log) {
	return &count_ftp_bandwidth($log, $_[1], $_[2], undef, "ftp");
	}
else {
	return $_[1];
	}
}

# proftpd_template(text, &domain)
# Returns a suitably substituted ProFTPd template
sub proftpd_template
{
local $dirs = $_[0];
$dirs =~ s/\t/\n/g;
$dirs = &substitute_template($dirs, $_[1]);
local @dirs = split(/\n/, $dirs);
return @dirs;
}

# vsftpd_template(text, &domain)
# Returns a suitably substituted VSFTPd template
sub vsftpd_template
{
local $opts = $_[0];
$opts =~ s/\t/\n/g;
$opts = &substitute_template($opts, $_[1]);
return $opts;
}

# check_proftpd_template([directives])
# Returns an error message if the default ProFTPd directives don't look valid
sub check_proftpd_template
{
local ($d, $gotuser, $gotgroup);
local @dirs = split(/\t+/, defined($_[0]) ? $_[0] : $config{'proftpd_config'});
foreach $d (@dirs) {
	$d =~ s/#.*$//;
	if ($d =~ /^\s*User\s+(\S+)$/i) {
		defined(getpwnam($1)) ||
			return &text('fcheck_euserex', "<tt>$1</tt>");
		$gotuser++;
		}
	elsif ($d =~ /^\s*Group\s+(\S+)$/i) {
		defined(getgrnam($1)) ||
			return &text('fcheck_egroupex', "<tt>$1</tt>");
		$gotgroup++;
		}
	}
$gotuser || return $text{'fcheck_euser'};
$gotgroup || return $text{'fcheck_egroup'};
return undef;
}

# restart_proftpd()
# Tell ProFTPd to re-read its config file. Does nothing if run from inetd
sub restart_proftpd
{
&require_ftp();
local $conf = &proftpd::get_config();
local $st = &proftpd::find_directive("ServerType", $conf);
if (lc($st) ne "inetd") {
	&$first_print($text{'setup_proftpdpid'});
	local $pidfile = $proftpd::config{'pid_file'};
	local $rv = 0;
	if (open(PID, $pidfile) && <PID> =~ /(\d+)/ && kill(0, $1)) {
		&kill_logged('HUP', $1);
		&$second_print($text{'setup_done'});
		}
	else {
		&$second_print($text{'setup_notrun'});
		$rv = 1;
		}
	close(PID);
	return $rv;
	}
}

# get_proftpd_virtual(ip)
# Returns the list of configuration directives and the directive for the
# virtual server itself for some domain
sub get_proftpd_virtual
{
&require_ftp();
local $conf = &proftpd::get_config();
local $v;
foreach $v (&proftpd::find_directive_struct("VirtualHost", $conf)) {
	if ($v->{'words'}->[0] eq $_[0]) {
		# Found it! Looks for 
		local $a = &proftpd::find_directive_struct("Anonymous", $v->{'members'});
		if ($a) {
			return ($v, $v->{'members'}, $a, $a->{'members'});
			}
		else {
			return ($v, $v->{'members'});
			}
		}
	}
return ();
}

# get_proftpd_log(ip)
# Given a virtual server IP, returns the path to its log file. If no IP is
# give, returns the global log file path.
sub get_proftpd_log
{
&require_ftp();
if ($_[0]) {
	# Find by IP
	local ($virt, $vconf) = &get_proftpd_virtual($_[0]);
	if ($virt) {
		return &proftpd::find_directive("ExtendedLog", $vconf) ||
		       &proftpd::find_directive("TransferLog", $vconf);
		}
	}
else {
	# Just return global log
	local $conf = &proftpd::get_config();
	local $global = &proftpd::find_directive_struct("Global", $conf);
	return &proftpd::find_directive("ExtendedLog", $global->{'members'}) ||
	       &proftpd::find_directive("TransferLog", $global->{'members'});
	}
return undef;
}

# get_vsftpd_log(ip)
# Given a virtual server IP, returns the path to its log file. If no IP is
# give, returns the global log file path.
sub get_vsftpd_log
{
&require_ftp();
 if ($_[0])
 {
  local $conf_file = "$vsftpd::config{'vsftpd_dir'}/vsftpd.$_[0].conf";
  if( -r $conf_file) {
      my $conf = &vsftpd::parse_config_file($conf_file);
      return &vsftpd::get_config_value('xferlog_file',$conf);
      }
 }
 else
 {
  my $conf = &vsftpd::parse_config_file();
  return  &vsftpd::get_config_value('xferlog_file',$conf) ||
          "/var/log/vsftpd.log"; 
 }
 return undef;
}

# proftpd_installed()
# Returns 1 if proftpd is installed
sub proftpd_installed
{
 return &foreign_installed("proftpd", 1) == 2;
}

# vsftpd_installed()
# Returns 1 if postfix is installed
sub vsftpd_installed
{
 return &foreign_installed("vsftpd", 1) == 2;
}

1;

1;
