# Functions for managing Mailman

# &require_mailman()
# require mailman module library & config
sub require_mailman
{
 return if ($require_mailman++);
 &foreign_require("mailman", "mailman-lib.pl");
}

# &mailman_installed()
# Returns 1 if mailman is installed
sub mailman_installed
{
 &require_mailman();
 return 1 unless &mailman::mailman_check();
 return 0;
}

# &setup_mailman(&domain)
# Add new Mailman virtual host
sub setup_mailman
{
 &$first_print($text{'setup_mailman'});
 &require_mailman();
 &add_mailman_virtual($_[0]->{'dom'});
 &$second_print($text{'setup_done'});
}

# &delete_mailman(&domain)
# Delete Mailman virtual host & all domain lists
sub delete_mailman
{
 &$first_print($text{'delete_mailman'});
 &require_mailman();
 &delete_domain_lists($_[0]->{'dom'}); 
 &$second_print($text{'setup_done'});
}

# &modify_mailman(&domain, &olddomain)
# Modify Mailman virtual host & all domain lists
sub modify_mailman
{
 &$first_print($text{'modify_mailman'});
 &require_mailman();
 if ($_[0]->{'dom'} ne $_[1]->{'dom'})
 {
  # re-crete all maillists
  &modify_domain_lists($_[0]->{'dom'}, $_[1]->{'dom'}); 
 }
 &$second_print($text{'setup_done'});
}

# &disable_mailman(&domain)
# Delete Mailman virtual host
sub disable_mailman
{
 &$first_print($text{'disable_mailman'});
 &require_mailman();
 
 # delete Mailman virtual host
 &delete_mailman_virtual($_[0]->{'dom'});
 
 # regenerate aliases
 &mailman::regenerate_aliases();
 
 &$second_print($text{'setup_done'});
}

# &enable_mailman(&domain)
# Add Mailman virtual host
sub enable_mailman
{
 &$first_print($text{'enable_mailman'});
 &require_mailman();
 
 # add Mailman virtual host
 &add_mailman_virtual($_[0]->{'dom'});
 
 # regenerate aliases
 &mailman::regenerate_aliases();
 &$second_print($text{'setup_done'});
}

# &backup_mailman(&domain, file, &options)
# Backup domain mailing list
sub backup_mailman
{
 # Create dummy file
 &open_tempfile(FILE, ">$_[1]");
 &close_tempfile(FILE);
 
 &$first_print($text{'backup_mailman'});
 &require_mailman();
 my $dom = &trunc_space($_[0]->{'dom'});
 my @lists = &mailman::list_lists($dom);
 &open_tempfile(AFILE, ">$_[1]_lists");
 foreach (@lists) 
 {
  my $name = $_->{'name'};
  &print_tempfile(AFILE, "$name:$_->{'dom'}:$_->{'email'}:$_->{'lang'}\n");
  my @mems = &mailman::list_members($_);
  if (@mems)
  {
   &open_tempfile(BFILE, ">$_[1]_members_$name");
   foreach my $i (@mems)
   {
    &print_tempfile(BFILE, "$i->{'email'}:$i->{'digest'}\n");
   }
   &close_tempfile(BFILE);
  }
 }
 &close_tempfile(AFILE);
 &$second_print($text{'setup_done'});
 return 1;
}

# &restore_mailman(&domain, file, &options)
sub restore_mailman
{
 &$first_print($text{'restore_mailman'});
 &require_mailman();
 open(UFILE, "$_[1]_lists");
 while(<UFILE>) 
 {
  chomp;
  my %maillist;
  ($maillist{'name'}, $maillist{'dom'}, $maillist{'email'}, $maillist{'lang'}) = split (/:/,$_);
  $maillist{'pass'} = 1;
  &mailman::create_list(\%maillist, 1, 1);
    
  # add members
  open(ZFILE, "$_[1]_members_$maillist{'name'}");
  my $fullname = $maillist{'name'}.".".$maillist{'dom'};
  my (%mems, @digest, @nodigest);
  while(my $f = <ZFILE>) 
  {
   chomp;
   my ($email, $digest) = split (/:/, $f);
   if ($digest eq 'y') { push (@digest, $email); }
	else { push (@nodigest, $email); }
  }
  close(ZFILE);
  &mailman::add_members(\@digest, $fullname) if @digest;
  &mailman::add_members(\@nodigest, $fullname, 1) if @nodigest;
  
  # change password
  &mailman::chpass_list($fullname);
 }
 close(UFILE);
 
 # regenerate aliases
 &mailman::regenerate_aliases();
 
 if ($config{'mail_system'} == 0)
 {
  # regenerate postfix virtual table
  &postfix::regenerate_virtual_table();
  #&postfix::reload_postfix();
 }

 &$second_print($text{'setup_done'});
 return 1;
}


# check_mailman_clash(&domain, [changing])
# Always returns 0
sub check_mailman_clash
{
 return 0;
}

# &add_mailman_virtual($domain)
# add new Mailman virtual host
sub add_mailman_virtual
{
 my $dom = &trunc_space($_[0]);
 
 # add virtual host record into mailman configuration
 my $conf = &mailman::parse_config_file();
 my $new_conf;
 if ($config{'mail_system'} == 0)
 {
  # Postfix
  $conf->{'POSTFIX_STYLE_VIRTUAL_DOMAINS'} =~ /\[(.+?)\]/;
  my $domains = $1;
  my @domains = split (/,\s*?/, $domains) if $domains;
  push (@domains, "\'$dom\'");
  $domains = join (',',@domains);
 
  # change mailman config file
  $new_conf = { 'POSTFIX_STYLE_VIRTUAL_DOMAINS' => "\[$domains\]",
					"add_virtualhost(\'www.$dom\', \'$dom\')"
				}; 
 }
 elsif ($config{'mail_system'} == 1)
 {
  # Sendmail
  $new_conf = { "add_virtualhost(\'www.$dom\', \'$dom\')"}; 
 }
 # change mailman config file
 &mailman::save_new_options($new_conf);  
 
 if (&mailman::is_mailman_running)
 {
  # restart mailman
  &mailman::restart_mailman();
 }
}

# &modify_domain_lists($dom, $old_dom)
# modify mailing list for some domain
sub modify_domain_lists
{
 my $dom = &trunc_space($_[0]);
 my $old_dom = &trunc_space($_[1]);
 my @lists = &mailman::list_lists($old_dom);
 foreach my $j (@lists)
 {
  # get mailman lists
  my @mems = &mailman::list_members($j);
  
  # delete old lists
  &mailman::delete_list($j, 1, 1); 
  my $fullname = $j->{'name'}.".$dom";
  &mailman::delete_postfix_map($j->{'name'}, $j->{'list'}, $dom, 1);
  
  # create new lists
  $j->{'dom'} = $dom;
  $j->{'pass'} = 1;
  &mailman::create_list($j, 1, 1);
  
  # add members
  my $fullname = $j->{'name'}.".$dom";
  my (@digest, @nodigest);
  foreach my $i (@mems)
  {
   if ($i->{'digest'} eq 'y') { push (@digest, $i->{'email'}); }
	else { push (@nodigest, $i->{'email'}); }
  }
  &mailman::add_members(\@digest, $fullname) if @digest;
  &mailman::add_members(\@nodigest, $fullname, 1) if @nodigest;
  
  # change password
  &mailman::chpass_list($fullname);
 }
 
 # modify mailman config file
 &modify_mailman_virtual($dom, $old_dom);
 
 if (&mailman::is_mailman_running)
 {
  # restart mailman
  &mailman::restart_mailman();
 }
 
 # regenerate aliases
 &mailman::regenerate_aliases();
 
 if ($config{'mail_system'} == 0)
 {
  # regenerate postfix virtual table
  &postfix::regenerate_virtual_table();
  #&postfix::reload_postfix();
 }
}

# &modify_mailman_virtual($dom, $old_dom)
# modify mailman config file
sub modify_mailman_virtual
{
 my ($dom, $old_dom) = @_;
 
 # add virtual host record into mailman configuration
 my $conf = &mailman::parse_config_file();
 my $new_conf;
 if ($config{'mail_system'} == 0)
 {
  # Postfix
  $conf->{'POSTFIX_STYLE_VIRTUAL_DOMAINS'} =~ /\[(.+?)\]/;
  my $domains = $1;
  if ($domains)
  {
   my @domains = split (/,/, $domains);
	foreach (@domains)
	{
	 $_ = "\'$dom\'" if ($_ =~ /^\s*?'$old_dom'\s*?$/);
	}
   $domains = join (',',@domains);

   # modify virtual host
	&mailman::remove_mailman_str(["add_virtualhost\s*?\(.*?'$old_dom'.*?\)"]);
   $new_conf = { 'POSTFIX_STYLE_VIRTUAL_DOMAINS' => "\[$domains\]", 
					 "add_virtualhost(\'www.$dom\', \'$dom\')"
				  };
  }
 }
 elsif ($config{'mail_system'} == 1)
 {
  # Sendmail
  &mailman::remove_mailman_str(["add_virtualhost\s*?\(.*?'$old_dom'.*?\)"]);
  $new_conf = { "add_virtualhost(\'www.$dom\', \'$dom\')"};
 }
 # modify virtual host
 &mailman::save_new_options($new_conf);
}

# &delete_domain_lists($dom)
# Delete mailing list for some domain
sub delete_domain_lists
{
 my $dom = &trunc_space($_[0]);
 my @lists = &mailman::list_lists($dom);
 foreach (@lists)
 {
  &mailman::delete_list($_, 1, 1);
 }
 
 # modify mailman config file
 &delete_mailman_virtual($dom);
 
 if (&mailman::is_mailman_running)
 {
  # restart mailman
  &mailman::restart_mailman();
 }
 
 # regenerate aliases
 &mailman::regenerate_aliases();
 
 return undef;
}

# &delete_mailman_virtual($domain)
# modify mailman config file
sub delete_mailman_virtual
{
 my $dom = $_[0];
 
 # add virtual host record into mailman configuration
 my $conf = &mailman::parse_config_file();
 if ($config{'mail_system'} == 0)
 {
  # Postfix
  $conf->{'POSTFIX_STYLE_VIRTUAL_DOMAINS'} =~ /\[(.+?)\]/;
  my $domains = $1;
  if ($domains)
  {
   my @d_temp = split (/,/, $domains);
   my @domains = grep { !($_ =~ /^\s*?'$dom'\s*?$/) } @d_temp;
   $domains = join (',',@domains);
  }
  if ($domains)
  {
   # delete virtual host
   my $conf = { 'POSTFIX_STYLE_VIRTUAL_DOMAINS' => "\[$domains\]" };
   &mailman::save_new_options($conf);
   &mailman::remove_mailman_str(["add_virtualhost\s*?\(.*?'$dom'.*?\)"]);
  }
  else
  {
   # no any virtual hosts
   &mailman::remove_mailman_str(['POSTFIX_STYLE_VIRTUAL_DOMAINS', "add_virtualhost\s*?\(.*?'$dom'.*?\)"]);
  }
 }
 elsif ($config{'mail_system'} == 1)
 {
  # Sendmail
  &mailman::remove_mailman_str(["add_virtualhost\s*?\(.*?'$dom'.*?\)"]);
 }
}

# trunc_space(string)
# delete spaces from string
sub trunc_space
{
 local $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}

1;

1;
