# Functions for managing a domain's Samba client

sub require_samba {
 return if ($require_samba++);
 &foreign_require("samba", "samba-lib.pl");
 %smbconfig = &foreign_config("samba");
}

# Creates a samba client for a domain owner
# setup_samba(&domain)
sub setup_samba
{
 &$first_print(&text('setup_samba_user', $_[0]->{'user'}));
 &require_samba();
 &create_samba_client($_[0]->{'user'}, $_[0]->{'pass'});
 &$second_print($text{'setup_done'});
}

# delete_samba(&domain)
# Removes a domain owner from the list of samba clients
sub delete_samba
{
 &$first_print($text{'delete_samba'});
 &require_samba();
 # removing all endusers' samba clients
 local @users = &list_domain_users($_[0]);
 map { &delete_samba_client($i->{'user'}) } reverse(@users);
 # removing domain owner from list of samba clients
 &delete_samba_client($_[0]->{'user'});
 &$second_print($text{'setup_done'});
}

# Changes the samba client's password if needed (for a domain owner)
# modify_samba(&domain, &olddomain)
sub modify_samba
{
 if ($_[0]->{'pass_set'}) {
	&$first_print($text{'save_samba_user'});
	&require_samba();
	&create_samba_client($_[0]->{'user'}, $_[0]->{'pass'});
	&$second_print($text{'setup_done'});
 }
}

# Locks a samba client for a domain owner
# disable_samba(&domain)
sub disable_samba
{
 &require_samba();
 local $u;
 if ($u = &get_samba_user($_[0]->{'user'}))
 {
  &$first_print($text{'disable_samba'});
  $u->{'disabled'} = 1;
  &modify_samba_client($u);
  &webmin_log("disable", "sambaclient", $_[0]);
  &$second_print($text{'setup_done'});
 }
}

# Unlocks a samba client for a domain owner
# enable_samba(&domain)
sub enable_samba
{
 &require_samba();
 local $u;
 if ($u = &get_samba_user($_[0]->{'user'}))
 {
  &$first_print($text{'enable_samba'});
  $u->{'disabled'} = 0;
  &modify_samba_client($u);
  &webmin_log("enable", "sambaclient", $_[0]);
  &$second_print($text{'setup_done'});
 }
}

# Save all the virtual server's 
# backup_samba(&domain, file, &options)
sub backup_samba
{
 return 1;
}

# restore_samba(&domain, file, &options, &all-options)
sub restore_samba
{
 return 1;
}

# bandwidth_mail(&domain, start, &bw-hash)
# Searches through log files for records after some date, and updates the
# day counters in the given hash
# sub bandwidth_samba
# {
# }

# check_samba_clash(&domain, [changing])
# Always returns 0
sub check_samba_clash
{
 return 0;
}

# Create samba client or modify it password if it is existed
# create_samba_client(name, passwd);
sub create_samba_client
{
&require_samba();
unless (&get_samba_user($_[0])) {
	# adding a new samba user
	&lock_file($samba::config{'smb_passwd'});
	local @uinfo = getpwnam($_[0]);
 	local @flags = ("UN");
	local $nu = {
		'name' => $uinfo[0],
		'uid' => $uinfo[2],
		'pass1' => "NO PASSWORDXXXXXXXXXXXXXXXXXXXXX",
		'pass2' => "NO PASSWORDXXXXXXXXXXXXXXXXXXXXX",
	};
	if ($samba::samba_version < 2) {
		$nu->{'real'} = $uinfo[6];
		$nu->{'home'} = $uinfo[7];
		$nu->{'shell'} = $uinfo[8];
		}
	else {
		$nu->{'opts'} = \@flags;
		}
	&samba::create_user($nu);
	&unlock_file($samba::config{'smb_passwd'});
	&webmin_log("add", "sambaclient", $_[0]);
}

# updating samba password
&samba::set_password($_[0], $_[1]) || &error($samba::text{'mkpass_passfail'});
&webmin_log("modify_passwd", "sambaclient", $_[0]);
}

# modify_samba_client(&user, [nocreate])
# Update a samba user
sub modify_samba_client
{
&require_samba();
local $u;
local @uinfo;
local $create;
local @opts;
$_[0]->{'name'} || &error($text{'edit_samba_noname'});
$u = &get_samba_user($_[0]->{'oldname'}) || 
	&get_samba_user($_[0]->{'name'}) || 
	($_[2] && &error(&text('edit_samba_nouser', $_[1]->{'name'})));

$create++ unless $u;

if ($_[0]->{'oldname'} && $_[0]->{'name'} ne $_[0]->{'oldname'}) {
	&delete_samba_client($_[0]->{'oldname'});
	$create++;
	}

local $passwd = $_[0]->{'plainpass'};
@uinfo = getpwnam($_[0]->{'name'});
$_[0]->{'name'} = $uinfo[0] unless $_[0]->{'name'};
$_[0]->{'uid'} = $uinfo[2] unless $_[0]->{'uid'};
if ($samba::samba_version < 2) {
	$_[0]->{'real'} = $uinfo[6] unless $_[0]->{'real'};
	$_[0]->{'home'} = $uinfo[7] unless $_[0]->{'home'};
	$_[0]->{'shell'} = $uinfo[8] unless $_[0]->{'shell'};
}

	# Update details
	&lock_file($config{'smb_passwd'});
	$u->{'uid'} = $_[0]->{'uid'};
	$u->{'name'} = $_[0]->{'name'};
	if ($samba::samba_version >= 2) {
		$u->{'opts'} = $u->{'opts'} ? $u->{'opts'} : ["U"];
		local @opts = grep { !/[ND]/ } @{$u->{'opts'}};
		push(@opts, "N") if ($create); # || $_[0]->{'passmode'} == 0);
		push(@opts, "D") if ($_[0]->{'disabled'} || $_[0]->{'passmode'} == 1);
		$u->{'opts'} = \@opts;
		}
	else {
		$u->{'real'} = $_[0]->{'real'};
		$u->{'home'} = $_[0]->{'home'};
		$u->{'shell'} = $_[0]->{'shell'};
		}
	if (!$passwd && $_[0]->{'passmode'} == 0) {
		$u->{'pass1'} = "NO PASSWORDXXXXXXXXXXXXXXXXXXXXX";
		$u->{'pass2'} = $u->{'pass1'};
		}
	elsif (!$passwd && $_[0]->{'passmode'} == 1) {
		$u->{'pass1'} = $u->{'pass2'} = ("X" x 32);
		}
	if ($create) {
		&samba::create_user($u);
		&webmin_log("create", "sambaclient", $_[0]);
	} 
	else {
		&samba::modify_user($u);
		&webmin_log("modify", "sambaclient", $_[0]);
	}
	if ($passwd && ($create || $_[0]->{'passmode'} == 3)) {
		&samba::set_password($_[0]->{'name'}, $passwd);
		&webmin_log("modify_passwd", "sambaclient", $_[0]);
		}
	&unlock_file($config{'smb_passwd'});
}


# Deletes samba client
# delete_samba_client(username);
sub delete_samba_client
{
&require_samba();
local $u;
if ($u = &get_samba_user($_[0])) {
	# Delete the user
	&lock_file($samba::config{'smb_passwd'});
	&samba::delete_user($u);
	&unlock_file($samba::config{'smb_passwd'});
	&webmin_log("delete", "sambaclient", $_[0]);
	}
}

# Gets Samba client
# get_samba_user(username);
sub get_samba_user
{
&require_samba();
local @ulist = &samba::list_users();
local $u;
foreach $u (@ulist) {
	return $u if ($u->{'name'} eq $_[0]);
	}
return undef;
}


1;

1;
