#!/usr/bin/perl

$no_acl_check++;
require 'virtual-server-lib.pl';

sub module_install {

# turn off misconfigured features
my %features = (
	'apache'     => "X:httpd_path:web",
	'bind8'      => "X:named_path:dns",
	'mysql'      => "X:mysqladmin:mysql",
	'postgresql' => "X:psql:postgres",
	'webalizer'  => "C:webalizer_conf:webalizer",
	'logrotate'  => "C:logrotate_conf:logrotate",
	#'proftpd'    => "X:proftpd_path:ftp",
	'samba'      => "C:_nodefined_:samba",
);

while ( ($dir, $val) = each(%features) ) {
	my ($type, $check, $ventry) = split(/:/, $val);
	next if "$ENV{'WEBMIN_UPGRADE'}" eq "1" && !$config{$ventry};

	&read_file("$config_directory/$dir/config", \%conf);
	if ($type =~ /^X/) {
		if ( !-x $conf{$check} ) {
			$config{$ventry} = 0;
		} else {
			$config{$ventry} = 1;
		}
	} elsif ($type =~ /^C/) {
		if ( !-r $conf{$check} ) {
			$config{$ventry} = 0;
		} else {
			$config{$ventry} = 1;
		}
	}
}
&write_file("$module_config_directory/config", \%config);

# check up and configure mail system
if ($config{'mail'}) {
	if ($config{'mail_system'} == 3) {
		# Work out which mail server we have
		if (&postfix_installed()) {
			$config{'mail_system'} = 0;
			}
		elsif (&qmail_installed()) {
			$config{'mail_system'} = 2;
			}
		elsif (&sendmail_installed()) {
			$config{'mail_system'} = 1;
			}
		&write_file("$module_config_directory/config", \%config);
		}
	&require_mail();
	}

# Make sure openssl is installed, that Apache supports mod_ssl,
# and that port 443 is in use
$config{'ssl'} = 0;
if ($config{'web'} && &has_command("openssl")) {
	&require_apache();
	local $conf = &apache::get_config();
	local @loads = &apache::find_directive_struct("LoadModule", $conf);
	local ($l, $hasmod);
	foreach $l (@loads) {
		$hasmod++ if ($l->{'words'}->[1] =~ /mod_ssl/);
		}
	local ($aver, $amods) = &apache::httpd_info(&apache::find_httpd());
	$hasmod++ if (&indexof("mod_ssl", @$amods) >= 0);
	$hasmod++ if ($apache::httpd_modules{'mod_ssl'});
	if ($hasmod) {
		local @listens = &apache::find_directive_struct("Listen", $conf);
		foreach $l (@listens) {
			$haslisten++ if ($l->{'words'}->[0] =~ /^(\S+:)?$default_web_sslport$/);
			}
		local @ports = &apache::find_directive_struct("Port", $conf);
		foreach $l (@ports) {
			$haslisten++ if ($l->{'words'}->[0] == $default_web_sslport);
			}
		$config{'ssl'} = 1 if $haslisten;
	}
}
&write_file("$module_config_directory/config", \%config);

# update domain-owners' access
foreach my $d (&list_domains()) {
	my $u = $d->{'user'};
	%acl = &get_module_acl($u);
	my $can_create = $d->{'domslimit'} && !$d->{'no_create'} && $d->{'unix'};
	$acl{'edit'} = $can_create ? 2 : 0;
	$acl{'create'} = $can_create ? 2 : 0;
	&save_module_acl(\%acl, $u);
	}

 #configure file /etc/shells
 if (-e '/etc/shells')
 {
  my ($file,$add,@temp)='/etc/shells';
  &lock_file($file);
  my $str= &read_file_lines($file);
  foreach (@{$str})
  {
   unless ($_ eq $config{'shell'})
   {
    push (@temp,$_);
    $add=1 if ($_ eq $config{'ftp_shell'});
   }
  }
  if ($add!=1)
  {
   #add new string into /etc/shells
   push (@temp,$config{'ftp_shell'});
   @{$str}=@temp;
   flush_file_lines();
  }
  &unlock_file($file);
 }
 &ensure_template("domain-template");
 &ensure_template("subdomain-template");
 &ensure_template("user-template");
 &ensure_template("local-template");
 &ensure_template("bw-template");
 &ensure_template("warnbw-template");
 &ensure_template("framefwd-template");
}
1;
