#!/usr/bin/perl
package vsftpd::VsftpdLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'vsftpd::VsftpdLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# vsftpd-lib.pl
# Common functions for VsFTPd configuration

# main part
&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
%{$self->{'access_hash'}} = $self->get_module_acl();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 

1;
		return $self;
}

# get_vsftpd_version()
# returns the vsftpd version number, or 0 if not available
sub get_version_vsftpd
{
	my $self = shift;

 # with command
 my $out = $self->backquote_command("$self->{'config_hash'}->{'vsftpd_cmd'} -v 0>&1");
 $out =~ /([0-9\.]+)/;
 return $1 if $1;
 
 # from rpm
 return $self->get_rpm_version();
 
 return 0;
}

# get_rpm_version()
# returns vsftpd rpm version or 0
sub get_rpm_version 
{
	my $self = shift;

 my $out;
 my $rpm_opts = "-q vsftpd --queryformat \"\%\{VERSION\}\"";
 if (-x '/bin/rpm') 
 {
  $out = `/bin/rpm $rpm_opts 2>/dev/null`;
  return $out if !$? && $out =~ /^[\d\.]+$/;
 }
 elsif ($self->has_command('rpm')) 
 {
  $out = `rpm $rpm_opts 2>/dev/null`;
  return $out if !$? && $out =~ /^[\d\.]+$/;
 }
 return 0;
}

# status_vsftpd()
# return 1 or PIDs if daemon start, 
# return 0 if daemon stop 
sub status_vsftpd
{
	my $self = shift;

 my $out;
 if ($self->{'config_hash'}->{'status_cmd'}) {
 	# status cmd
 	$out = $self->backquote_command("$self->{'config_hash'}->{'status_cmd'} 2>&1");
	return $? ? 0 : 1;
	}
 else {
 	# search running process
	return $self->find_byname($self->{'config_hash'}->{'vsftpd_cmd'});
 	}
}

# start_vsftpd()
# start daemon vsftpd or call &error
sub start_vsftpd
{
	my $self = shift;

 # service stoped
 if ($self->{'config_hash'}->{'start_cmd'})
 {
  # start with script
  system("$self->{'config_hash'}->{'start_cmd'} 2>&1");
  $self->error($self->text('error_start', $?)) if ($? || !$self->status_vsftpd());
  $self->webmin_log("start");
 }
 else { $self->error($self->text('error_start', $self->text('error_conf', "$self->{'gconfig_hash'}->{'webprefix'}/config.cgi?$self->{'module_name'}"))); }
}

# stop_vsftpd()
# stop daemon vsftpd or call &error
sub stop_vsftpd
{
	my $self = shift;

 # service started
 if ($self->{'config_hash'}->{'stop_cmd'})
 {
  # stop with script
  my $out = $self->backquote_command("$self->{'config_hash'}->{'stop_cmd'} 2>&1");
  if ($self->status_vsftpd()){
		# kill -9
		$self->kill_byname($self->{'config_hash'}->{'vsftpd_cmd'}, 9);
		}
  $self->error($self->text('error_stop', $out)) if $self->status_vsftpd();
  $self->webmin_log("stop"); 
 }
 else { $self->error($self->text('error_stop', $self->text('error_conf', "$self->{'gconfig_hash'}->{'webprefix'}/config.cgi?$self->{'module_name'}"))); }
}

# restart_vsftpd()
# restart daemon vsftpd or call &error
sub restart_vsftpd
{
	my $self = shift;

 # service started
 if ($self->{'config_hash'}->{'restart_cmd'})
 {
  # restart with script
  system("$self->{'config_hash'}->{'restart_cmd'} 2>&1");
  if ($? && $self->status_vsftpd()){
  		# kill -9
		$self->kill_byname($self->{'config_hash'}->{'vsftpd_cmd'}, 9);
		# start with script
		system("$self->{'config_hash'}->{'start_cmd'} 2>&1");
		}
  $self->error($self->text('error_restart', $?)) if ($? || !$self->status_vsftpd());
  $self->webmin_log("restart");
 }
 else { $self->error($self->text('error_restart', $self->text('error_conf', "$self->{'gconfig_hash'}->{'webprefix'}/config.cgi?$self->{'module_name'}"))); }
}

# parse_config_file([file])
# return hash with name=value pairs from vsftpd configuration file or call &error
sub parse_config_file
{
	my $self = shift;

 my $conf_file = $_[0] ? $_[0] : $self->{'config_hash'}->{'vsftpd_conf'};
 $self->lock_file($conf_file);
 my $conf;
 open (INFL,$conf_file) || $self->error($self->{'text_hash'}->{'error_open'});
 while (<INFL>)
 {
  chomp;
  $self->trunc_space($_);
  unless ($_=~/^#/ || $_=~/^$/)
  {
   my @options=split (/=/,$_);
   $options[0]=$self->trunc_space($options[0]);
   $options[1]=$self->trunc_space($options[1]);
   $conf -> {$options[0]} = $options[1];
  }
 }
 close INFL;
 $self->unlock_file($conf_file);
 return $conf;
}

# get_config_value(name,\%hash)
# return value for name from hash
sub get_config_value
{
	my $self = shift;

 return $_[1]->{$_[0]};
}

# set_config_value(name,value,\%hash)
# set new value for name from hash
sub set_config_value
{
	my $self = shift;

 $_[2]->{$_[0]}=$_[1];
}

# del_config_value(name,\%hash)
# del name from hash
sub del_config_value
{
	my $self = shift;

 delete $_[1]->{$_[0]};
}

# save_new_options(\%hash,[file])
# save all new options from hash to vsftpd configuration file, change union options value, delete all other options
sub save_new_options
{
	my $self = shift;

 #get config file
 my $conf_file = $_[1] ? $_[1] : $self->{'config_hash'}->{'vsftpd_conf'};
  
 #parse config file
 $self->lock_file($conf_file);
 my $old_conf = $self->parse_config_file($conf_file);
 
 #get new options
 my @new;
 foreach (keys %{$_[0]})
 {
  unless (exists $old_conf->{$_}) { push (@new,$_); }
 }
 #get del & union options
 my (@del,@union);
 foreach (keys %{$old_conf})
 {
  if (exists $_[0]->{$_}) { push (@union,$_); }
  else { push (@del,$_); }
 }
 #read config file
 my $old_file= $self->read_file_lines($conf_file);
 
 #modify options
 foreach (@union)
 {
  foreach my $i(@{$old_file})
  {
   my @str_old=split (/=/,$i);
   my $str=$self->trunc_space($str_old[0]);
   $i="$_=$_[0]->{$_}" if ($str eq $_);
  }
 }
 #delete options
 foreach (@del)
 {
  my @temp;
  foreach my $i(@{$old_file})
  {
   my @str_old=split (/=/,$i);
   my $str=$self->trunc_space($str_old[0]);
   push (@temp,$i) unless ($str eq $_);
  }
  @{$old_file}=@temp;
 }
 #add new options
 foreach (@new)
 {
  my (@temp,$comment);
  foreach my $i (@{$old_file})
  {
   #add new options under comment
   push (@temp,$i);
   if (!$comment && $i=~/^#\s*$_\s*=/)
   {
    push (@temp,"$_=$_[0]->{$_}");
    $comment=1;
   }
  }
  #add new options in the end of file
  push (@temp,"$_=$_[0]->{$_}") if (!$comment);
  @{$old_file}=@temp;
 } 
 #save config file
 $self->flush_file_lines();
 $self->unlock_file($conf_file);
}

# list_virtual_server()
# return array of virtual servers configuration hash (idx,file,name,\%conf)
sub list_virtual_server
{
	my $self = shift;

 my ($file, $name, $conf, $idx, @servers);
 opendir (DIR, $self->{'config_hash'}->{'vsftpd_dir'}) || $self->error($self->{'text_hash'}->{'error_open'});
 while (defined ($file = readdir (DIR)))
 {
  next if $file =~ /^\.\.?$/;
  #get server name
  if ($file eq 'vsftpd.conf' && $self->acl_virtual('default')) 
  {
   #default server
   $name='default';
   $conf=$self->parse_config_file();
  }
  elsif ($file =~ /^vsftpd\.(.+)\.conf$/ && $self->acl_virtual($1))
  {
   #virtual server
   $name = $1;
   $conf=$self->parse_config_file("$self->{'config_hash'}->{'vsftpd_dir'}/$file");
  }
  else { next; }
  $idx++;
  push (@servers,{'idx' => $idx,'file' => $file,'name' => $name,'conf' => $conf});
 }
 closedir (DIR);
 return \@servers;
}

# acl_virtual(virtual_server)
# return 1 if user allow to virtual_server, else 0 
sub acl_virtual
{
	my $self = shift;

 return 1 if ($self->{'access_hash'}->{'virts'} eq '*');
 my @acl = split (/\s+?/,$self->{'access_hash'}->{'virts'});
 foreach (@acl)
 {
  return 1 if ($_[0] eq $_);
 }
 return 0;
}

# get_server_conf(idx)
# return file, name & configuration for virtual server with idx
# return mode = 1 if owner have only one virtual server
sub get_server_conf
{
	my $self = shift;

 my ($servers,$file,$name,$conf,$mode) = $self->list_virtual_server();
 $mode = 1 if (scalar(@{$servers}) == 1 && $self->{'access_hash'}->{'virts'} ne '*');
 foreach (@{$servers})
 {
  if ($_->{'idx'} eq $_[0])
  { 
   $conf = $_->{'conf'};
   $name = $_->{'name'};
   $file = $_->{'file'};
   last;
  }
 }
 return ($file, $name, $conf, $mode);
}

# get_section_list()
# return array of configuration sections (name,pict,version) for installed vsftpd version
sub get_section_list
{
	my $self = shift;

 my $version = $self->get_version_vsftpd();
 my @section = (['address','address.gif',],
		['limits','limit.gif',],
		['security','security.gif',],
		['ssl','ssl.gif',],
		['anon','anon.gif',],
 		['users','users.gif',],
		['virt_users','virt_users.gif','1.1.3'],
		['guest','guest.gif',],
		['userlist','userlist.gif',],
		['upload','upload.gif',],
		['permis','permis.gif',],
		['banner','banner.gif',],
		['dir_mes','dir_mes.gif',],
		['files','files.gif',],
		['logging','log.gif',],
		['timeouts','timeout.gif',],
		['passive','passive.gif',]);
 my @section_ver;
 foreach (@section)
 {
  push (@section_ver,$_) if ($version >= $_->[2]);
 }
 return \@section_ver;
}

# get_option_list(section)
# return array of configuration options (name,text,type,version) for section & installed vsftpd version
# type: 1 - (yes,no,default), 0 - (value,default)
sub get_option_list
{
	my $self = shift;

 my $section = $_[0];
 my ($version, @option) = $self->get_version_vsftpd();
 if ($section eq 'anon')
 {
  #--- Anonymous FTP Options ---
  @option = (['anonymous_enable','anon_enable',1,],
  		['anon_root','anon_root',0,'0.9.3'],
		['ftp_username','anon_ftp',0,],
		['no_anon_password','anon_no_password',1,],
		['anon_mkdir_write_enable','anon_mkdir',1,],
		['anon_other_write_enable','anon_other',1,],
		['anon_upload_enable','anon_upload',1,],
		['anon_world_readable_only','anon_world',],
		['allow_anon_ssl','anon_ssl',1,],
		['anon_umask','anon_umask',0],
		['chown_uploads','anon_chown_uploads',1],
		['chown_username','anon_chown_user',0],
		['anon_max_rate','anon_max',0],
		['secure_email_list_enable','anon_secure',1,'1.2.0'],
		['email_password_file','anon_email',0]);
 }
 elsif ($section eq 'users')
 {
  #--- Local Users ---
  @option = (['local_enable','local_enable',1],
  		['local_root','local_root',0,'0.9.3'],
		['check_shell','check_shell',1],
		['chmod_enable','chmod_enable','1,1.2'],
		['chroot_list_enable','chroot_list_enable',1],
		['chroot_local_user','chroot_local_user',1],
		['passwd_chroot_enable','passwd_chroot_enable',1],
		['chroot_list_file','chroot_list_file',0],
		['local_max_rate','local_max_rate',0],
		['local_umask','local_umask',0],
		['user_config_dir','user_config_dir',0,'1.0.1']);
 }
 elsif ($section eq 'virt_users')
 {
  #--- Virtual Users ---
  @option = (['virtual_use_local_privs','virtual_use_local_privs',1,'1.1.3'],
  		['user_sub_token','user_sub_token',0,'1.2.0']);
 }
 elsif ($section eq 'upload')
 {
  #--- Download / Upload Options ---
  @option = (['ascii_download_enable','ascii_download_enable',1],
  		['ascii_upload_enable','ascii_upload_enable',1]);
 }
 elsif ($section eq 'guest')
 {
  #--- Guest Options ---
  @option = (['guest_enable','guest_enable',1],
  		['guest_username','guest_username',0]);
 }
 elsif ($section eq 'logging')
 {
  #--- Logging ---
  @option = (['syslog_enable','syslog_en',1,'1.1.3'],
  		['vsftpd_log_file','log_file',0],
		['dual_log_enable','log_enable',1,'1.1.3'],
		['xferlog_enable','xferlog_enable',1],
		['xferlog_std_format','xferlog_format',1],
		['xferlog_file','xferlog_file',0],
		['log_ftp_protocol','log_protocol',1,'0.9.0'],
		['no_log_lock','log_lock',1,'1.2.2']);
 }
 elsif ($section eq 'ssl')
 {
  #--- SSL ---
  @option = (['ssl_enable','ssl_enable',1],
  		['ssl_sslv2','ssl_sslv2',1],
		['ssl_sslv3','ssl_sslv3',1],
		['ssl_tlsv1','ssl_tlsv1',1],
		['ssl_ciphers','ssl_ciphers',0,'2.0.2'],
		['force_local_data_ssl','force_data_ssl',1],
		['force_local_logins_ssl','force_logins_ssl',1],
		['dsa_cert_file','dsa_cert_file',0],
		['rsa_cert_file','rsa_cert_file',0]);
 }
 elsif ($section eq 'permis')
 {
  #--- Ownerships and Permissions ---
  @option = (['banned_email_file','banned_email',0],
  		['deny_email_enable','deny_email',1],
		['deny_file','deny_file',0,'1.2.0'],
		['write_enable','write_enable',1],
		['async_abor_enable','async_abor',1],
		['dirlist_enable','dirlist_enable',1,'1.1.3'],
		['download_enable','download_enable',1,'1.1.3'],
		['port_enable','port_enable',1],
		['port_promiscuous','port_promiscuous',1,'1.0.1'],
		['file_open_mode','file_mode',0,'1.1.0']);
 }
 elsif ($section eq 'banner')
 {
  #--- Banner ---
  @option = (['ftpd_banner','ftpd_banner',0],
  		['banner_file','banner_file',0,'1.0.1']);
 }
 elsif ($section eq 'userlist')
 {
  #--- User List --
  @option = (['userlist_enable','userlist_enable',1],
  		['userlist_deny','userlist_deny',1],
		['userlist_file','userlist_file',0]);
 }
 elsif ($section eq 'address')
 {
  #--- Addresses and Ports ---
  @option = (['listen','listen',1],
		['listen_ipv6','listen_ipv6',1],
		['listen_address','listen_address',0],
  		['listen_address6','listen_address6',0],
		['listen_port','listen_port',0],
		['ftp_data_port','ftp_data_port',0],
		['connect_from_port_20','connect_from_20',1]);
 }
 elsif ($section eq 'limits')
 {
  #--- Connections and Limits --
  @option = (['max_clients','max_clients',0],
		['max_per_ip','max_per_ip',0],
		['tcp_wrappers','tcp_wrappers',1],
		['trans_chunk_size','trans_chunk_size',0,'1.1.3'],
		['use_sendfile','use_sendfile',1]);
 }
 elsif ($section eq 'security')
 {
  #--- Security ---
  @option = (['background','background',1,'1.1.3'],
  		['pam_service_name','pam_service_name',0],
		['nopriv_user','nopriv_user',0,'0.0.6'],
		['cmds_allowed','cmds_allowed',0,'1.1.3'],
		['one_process_model','one_process_model',1,'0.0.11'],
		['run_as_launching_user','run_as_launching',1],
		['session_support','session_support',1],
		['setproctitle_enable','setproctitle_enable',1,'0.0.14'],
		['secure_chroot_dir','secure_chroot_dir',0]);
 }
 elsif ($section eq 'dir_mes')
 {
  #--- Directory Message --
  @option = (['dirmessage_enable','dirmessage_enable',1],
  		['message_file','message_file',0]);
 }
 elsif ($section eq 'files')
 {
  #--- Files and Directories (Listing Options) ---
  @option = (['force_dot_files','force_dot_files',1,'1.1.3'],
  		['ls_recurse_enable','ls_recurse_enable',1],
		['hide_file','hide_file',0,'1.2.0'],
		['tilde_user_enable','tilde_user_enable',1,'1.2.2'],
		['use_localtime','use_localtime',1,'1.0.1'],
		['hide_ids','hide_ids',1,'1.0.1'],
		['text_userdb_names','text_userdb_names',1]);
 }
 elsif ($section eq 'timeouts')
 {
  #--- Timeouts ---
  @option = (['accept_timeout','accept_timeout',0],
  		['connect_timeout','connect_timeout',0],
		['data_connection_timeout','data_connection_timeout',0],
		['idle_session_timeout','idle_session_timeout',0]);
 }
 elsif ($section eq 'passive')
 {
  #--- Passive Mode ---
  @option = (['pasv_enable','pasv_enable',1],
  		['pasv_promiscuous','pasv_promiscuous',1],
		['pasv_address','pasv_address',0],
		['pasv_max_port','pasv_max_port',0,'0.0.13'],
		['pasv_min_port','pasv_min_port',0,'0.0.13']);
 }
 
 my @ver1=split(/\./ , $version);
 my $v1=$ver1[0]*1000000+$ver1[1]*1000+$ver1[2];
 my @option_ver;
 my @ver2;
 foreach (@option)
 { 
  @ver2=split(/\./ , $_->[3]);
  push (@option_ver,$_) if ($v1>=($ver2[0]*1000000+$ver2[1]*1000+$ver2[2]));
 }
 return \@option_ver;
}

# choice_input(name,value)
# print checkbox (yes,no,default) 
sub choice_input
{
	my $self = shift;

 my ($value,$yes,$no,$default) = lc ($_[1]);
 if ($value eq 'yes') {$yes = 'checked';}
 elsif ($value eq 'no') {$no = 'checked';}
 else {$default = 'checked';}
 my $chekbox = "<input type=radio name=$_[0] value=0 $default>Default &nbsp;\n";
 $chekbox .= "<input type=radio name=$_[0] value=YES $yes>Yes &nbsp;\n";
 $chekbox .= "<input type=radio name=$_[0] value=NO $no>No &nbsp;\n"; 
 $chekbox .= "<input type=hidden name=$_[0]_def value=1>\n";
 return $chekbox;
}

# text_input(name,value,check)
# print checkbox & text input (value,default)
sub text_input
{
	my $self = shift;

 my ($value,$check,$yes,$default) = ($_[1],$_[2]);
 if ($check) {$yes = 'checked';}
 else {$default = 'checked';}
 my $input = "<input type=radio name=$_[0]_def value=0 $default>Default &nbsp;\n";
 $input .="<input type=radio name=$_[0]_def value=1 $yes> &nbsp;\n";
 $input .= "<input type=text name=$_[0] value=$value> &nbsp;\n";
 return $input;
}

# trunc_space(string)
# delete spaces from string
sub trunc_space
{
	my $self = shift;

 local $rv = $_[0];
 $rv =~ s/^\s+//;
 $rv =~ s/\s+$//;
 return $rv;
}


1;

