#!/usr/bin/perl

&MAIN();

 sub MAIN {
# save_acl.cgi
# Save access control options

&ModuleLoader::preload_module('wuftpd_class', "wuftpd::WuftpdLib");
$self = new wuftpd::WuftpdLib:: 'wuftpd_class',%$self;

$self->error_setup($self->{'text_hash'}->{'acl_err'});
$self->ReadParse();
$self->lock_file($self->{'config_hash'}->{'ftpaccess'});
$self->{'conf'} = $self->get_ftpaccess();

# Save deny directives
for($i=0; defined($self->{'daddrs'} = $self->{'in_hash'}->{"daddrs_$i"}); $i++) {
	$self->{'dmsg'} = $self->{'in_hash'}->{"dmsg_$i"};
	next if (!$self->{'daddrs'});
	$self->{'daddrs'} =~ /^\S+$/ || $self->error($self->text('acl_edaddr', $self->{'daddrs'}));
	$self->{'daddrs'} !~ /^\|(\S+)$/ || -r $1 || $self->error($self->text('acl_edfile', $1));
	$self->{'dmsg'} =~ /^\S+$/ && -r $self->{'dmsg'} || $self->error($self->text('acl_edmsg', $self->{'dmsg'}));
	push(@{$self->{'deny_array'}}, { 'name' => 'deny',
		      'values' => [ $self->{'daddrs'}, $self->{'dmsg'} ] } );
	}
$self->save_directive($self->{'conf'}, 'deny', \@{$self->{'deny_array'}});

# Save limit directives
for($i=0; defined($self->{'lclass'} = $self->{'in_hash'}->{"lclass_$i"}); $i++) {
	next if (!$self->{'lclass'});
	$self->{'in_hash'}->{"ln_def_$i"} || $self->{'in_hash'}->{"ln_$i"} =~ /^\d+$/ ||
		$self->error($self->text('acl_eln', $self->{'in_hash'}->{"ln_$i"}));
	$self->{'in_hash'}->{"ltimes_def_$i"} || $self->{'in_hash'}->{"ltimes_$i"} =~ /^\S+$/ ||
		$self->error($self->text('acl_etimes', $self->{'in_hash'}->{"ltimes_$i"}));
	-r $self->{'in_hash'}->{"lmsg_$i"} || $self->error($self->text('acl_elmsg', $self->{'in_hash'}->{"lmsg_$i"}));
	push(@{$self->{'limit_array'}},
	     { 'name' => 'limit',
	       'values' => [ $self->{'lclass'},
			     $self->{'in_hash'}->{"ln_def_$i"} ? -1 : $self->{'in_hash'}->{"ln_$i"},
			     $self->{'in_hash'}->{"ltimes_def_$i"} ? 'Any' : $self->{'in_hash'}->{"ltimes_$i"},
			     $self->{'in_hash'}->{"lmsg_$i"} ] } );
	}
$self->save_directive($self->{'conf'}, 'limit', \@{$self->{'limit_array'}});

# Save file-limt and data-limit directives
for($i=0; defined($self->{'fblimit'} = $self->{'in_hash'}->{"fblimit_$i"}); $i++) {
	next if (!$self->{'fblimit'});
	$self->{'in_hash'}->{"fbcount_$i"} =~ /^\d+$/ ||
		$self->error($self->text('acl_efbcount', $self->{'in_hash'}->{"fbcount_$i"}));
	local @v = $self->{'in_hash'}->{"fbraw_$i"} ? ('raw') : ();
	push(@v, $self->{'in_hash'}->{"fbinout_$i"}, $self->{'in_hash'}->{"fbcount_$i"}, $self->{'in_hash'}->{"fbclass_$i"});
	if ($self->{'fblimit'} eq 'file-limit') {
		push(@{$self->{'file_limit_array'}}, { 'name' => 'file-limit',
				    'values' => \@v } );
		}
	else {
		push(@{$self->{'data_limit_array'}}, { 'name' => 'data-limit',
				    'values' => \@v } );
		}
	}
$self->save_directive($self->{'conf'}, 'file-limit', \@{$self->{'file_limit_array'}});
$self->save_directive($self->{'conf'}, 'data-limit', \@{$self->{'data_limit_array'}});

# Save noretrieve directives
@{$self->{'class_array'}} = $self->find_value("class", $self->{'conf'});
for($i=0; defined($self->{'nfiles'} = $self->{'in_hash'}->{"nfiles_$i"}); $i++) {
	next if (!$self->{'nfiles'});
	local @v = $self->{'in_hash'}->{"nrel_$i"} ? ('relative') : ();
	local @c = split(/\0/, $self->{'in_hash'}->{"nclass_$i"});
	push(@v, map { "class=$_" } @c) if (@c != @{$self->{'class_array'}});
	push(@v, split(/\s+/, $self->{'nfiles'}));
	push(@{$self->{'noretrieve_array'}}, { 'name' => 'noretrieve',
			    'values' => \@v } );
	}
$self->save_directive($self->{'conf'}, 'noretrieve', \@{$self->{'noretrieve_array'}});

# Save allow-retrieve directives
for($i=0; defined($self->{'afiles'} = $self->{'in_hash'}->{"afiles_$i"}); $i++) {
	next if (!$self->{'afiles'});
	local @v = $self->{'in_hash'}->{"arel_$i"} ? ('relative') : ();
	local @c = split(/\0/, $self->{'in_hash'}->{"aclass_$i"});
	push(@v, map { "class=$_" } @c) if (@c != @{$self->{'class_array'}});
	push(@v, split(/\s+/, $self->{'afiles'}));
	push(@{$self->{'allow_retrieve_array'}}, { 'name' => 'allow-retrieve',
			        'values' => \@v } );
	}
$self->save_directive($self->{'conf'}, 'allow-retrieve', \@{$self->{'allow_retrieve_array'}});

# Save limit-time directives
if (!$self->{'in_hash'}->{'alimit_def'}) {
	$self->{'alimit'} = $self->{'in_hash'}->{'alimit'};
	$self->{'alimit'} =~ /^\d+$/ || $self->error($self->text('acl_elimit', $self->{'alimit'}));
	}
if (!$self->{'in_hash'}->{'glimit_def'}) {
	$self->{'glimit'} = $self->{'in_hash'}->{'glimit'};
	$self->{'glimit'} =~ /^\d+$/ || $self->error($self->text('acl_elimit', $self->{'glimit'}));
	}
if ($self->{'alimit'} && $self->{'alimit'} eq $self->{'glimit'}) {
	@{$self->{'limit_time_array'}} = ( { 'name' => 'limit-time',
			  'values' => [ '*', $self->{'alimit'} ] } );
	}
else {
	if ($self->{'alimit'}) {
		push(@{$self->{'limit_time_array'}}, ( { 'name' => 'limit-time',
				  'values' => [ 'anonymous', $self->{'alimit'} ] } ) );
		}
	if ($self->{'glimit'}) {
		push(@{$self->{'limit_time_array'}}, ( { 'name' => 'limit-time',
				  'values' => [ 'guest', $self->{'glimit'} ] } ));
		}
	}
$self->save_directive($self->{'conf'}, 'limit-time', \@{$self->{'limit_time_array'}});

# Save loginfails and private directives
if ($self->{'in_hash'}->{'fails_def'}) {
	$self->save_directive($self->{'conf'}, 'loginfails', [ ]);
	}
else {
	$self->{'in_hash'}->{'fails'} =~ /^\d+$/ || $self->error($self->text('acl_efails', $self->{'in_hash'}->{'fails'}));
	$self->save_directive($self->{'conf'}, 'loginfails',
			[ { 'name' => 'loginfails',
			    'values' => [ $self->{'in_hash'}->{'fails'} ] } ] );
	}
$self->save_directive($self->{'conf'}, 'private', 
			[ { 'name' => 'private',
			    'values' => [ $self->{'in_hash'}->{'private'} ] } ] );

$self->flush_file_lines();
$self->unlock_file($self->{'config_hash'}->{'ftpaccess'});
$self->webmin_log("acl", undef, undef, \%{$self->{'in_hash'}});
$self->redirect("");

1;
		
}


1;

