#!/usr/bin/perl
package virtual_server::VirtualserverLib;

 sub new {
 	my $invocant = shift;
 	my $class = ref($invocant) || $invocant;

	my $module_class = shift;
	my $self = { @_ };
	
	eval "push(\@$module_class\:\:ISA, 'virtual_server::VirtualserverLib');";
	
	foreach (@ISA) {
	    &ModuleLoader::preload_module($module_class, "$_");
	    eval "\$self = new $_\:\: '$module_class', \%\$self;";
	}

	bless $self, $module_class;
	    

 	# common part;
# virtual-server-lib.pl
# Common functions for Virtualmin

&ModuleLoader::preload_module(ref($self), "WebLib");
$self = new WebLib:: ref($self), %$self; 
$self->init_config();
&ModuleLoader::preload_module(ref($self), "UiLib");
$self = new UiLib:: ref($self), %$self; 
use Time::Local;

$self->ztl_get_appliances(\%{$self->{'apps_hash'}});
$self->{'t_website'} = $self->ztl_get_text(\%{$self->{'apps_hash'}}, 'text_website:$web', 'text_domain:1');
$self->{'t_websites'} = $self->ztl_get_text(\%{$self->{'apps_hash'}}, 'text_websites:$web', 'text_domains:1');
$self->{'t_website_mail'} = $self->ztl_get_text(\%{$self->{'apps_hash'}}, 'text_website_mail:$web && $mail', 'text_domain_mail:$dns && $mail', 'text_website:$web', 'text_domain:$dns', 'text_mail:$mail');
$self->{'t_websites_mail'} = $self->ztl_get_text(\%{$self->{'apps_hash'}}, 'text_websites_mail:$web && $mail', 'text_domains_mail:$dns && $mail', 'text_websites:$web', 'text_domains:$dns', 'text_mail:$mail');

%{$self->{'access_hash'}} = $self->get_module_acl();
if (!defined($self->{'access_hash'}->{'feature_unix'})) {
	$self->{'access_hash'}->{'feature_unix'} = 1;
	}
if (!defined($self->{'access_hash'}->{'feature_dir'})) {
	$self->{'access_hash'}->{'feature_dir'} = 1;
	}
$self->{'single_domain_mode'} = $self->{'access_hash'}->{'domains'} =~ /^\d+$/ &&
		      !$self->{'access_hash'}->{'edit'} && !$self->{'access_hash'}->{'create'} &&
		      !$self->{'access_hash'}->{'stop'} && !$self->{'access_hash'}->{''};
if (!$self->master_admin()) {
	# Allowed alias types are set by module config
	%{$self->{'can_alias_types_hash'}} = map { $_, 1 } split(/,/, $self->{'config_hash'}->{'alias_types'});
	}
else {
	# All types are allowed
	%{$self->{'can_alias_types_hash'}} = map { $_, 1 } (0 .. 11);
	}

$self->{'first_print'} = \&first_html_print;
$self->{'second_print'} = \&second_html_print;
$self->{'indent_print'} = \&indent_html_print;
$self->{'outdent_print'} = \&outdent_html_print;

@{$self->{'plugins_array'}} = split(/\s+/, $self->{'config_hash'}->{'plugins'});
@{$self->{'opt_features_array'}} = ( 'dir', 'unix', 'dns', 'mail', 'web', 'webalizer', 'ssl', 'mysql', 'postgres', 'ftp', 'webmin', split(/\s+/, $self->{'config_hash'}->{'additional_features'}), 'logrotate');
@{$self->{'vital_features_array'}} = ( 'dir', 'unix' );
@{$self->{'features_array'}} = ( @{$self->{'opt_features_array'}} );
@{$self->{'backup_features_array'}} = ( 'virtualmin', @{$self->{'features_array'}} );
@{$self->{'opt_alias_features_array'}} = ( 'dir', 'dns', 'mail', 'web' );
@{$self->{'alias_features_array'}} = ( @{$self->{'opt_alias_features_array'}} );
foreach $fname (@{$self->{'features_array'}}) {
	{
    my $pkg = $self->lib_to_pkg("$self->{module_root_directory}\/feature-$fname.pl", "virtual_server");
    &ModuleLoader::preload_module(ref($self), "$pkg");
    $pkg .= '::';
    eval "\$self = new $pkg ref(\$self),\%\$self;"; 
}
	}
foreach $pname (@{$self->{'plugins_array'}}) {
	$self->foreign_require($pname, "virtual_feature.pl");
	}
@{$self->{'feature_plugins_array'}} = grep { $self->plugin_defined($_, "feature_setup") } @{$self->{'plugins_array'}};
@{$self->{'mail_plugins_array'}} = grep { $self->plugin_defined($_, "mailbox_inputs") } @{$self->{'plugins_array'}};
@{$self->{'database_plugins_array'}} = grep { $self->plugin_defined($_, "database_name") } @{$self->{'plugins_array'}};

$self->{'backup_cron_cmd'} = "$self->{'module_config_directory'}/backup.pl";
$self->{'bw_cron_cmd'} = "$self->{'module_config_directory'}/bw.pl";

$self->{'custom_fields_file'} = "$self->{'module_config_directory'}/custom-fields";

$self->{'domains_dir'} = "$self->{'module_config_directory'}/domains";

$self->{'bandwidth_dir'} = "$self->{'module_config_directory'}/bandwidth";

$self->{'templates_dir'} = "$self->{'module_config_directory'}/templates";

1;
1;
		return $self;
}

sub require_useradmin
{
	my $self = shift;

return if ($self->{'require_useradmin'}++);
$self->foreign_require("useradmin", "user-lib.pl");
%{$self->{'uconfig_hash'}} = $self->foreign_config("useradmin");
$self->foreign_require("quota", "quota-lib.pl");
$self->{'home_base'} = $self->resolve_links($self->{'config_hash'}->{'home_base'} || $self->{'uconfig_hash'}->{'home_base'});
if ($self->{'config_hash'}->{'ldap'}) {
	$self->foreign_require("ldap-useradmin", "ldap-useradmin-lib.pl");
	$self->{'usermodule'} = "ldap-useradmin";
	}
else {
	$self->{'usermodule'} = "useradmin";
	}
}

# list_domains()
# Returns a list of structures containing information about hosted domains
sub list_domains
{
	my $self = shift;

local (@rv, $d);
opendir(DIR, $self->{'domains_dir'});
foreach $d (readdir(DIR)) {
	if ($d !~ /^\./ && $d !~ /\.(lock|bak|rpmsave|sav|swp)$/i) {
		push(@rv, $self->get_domain($d));
		}
	}
closedir(DIR);
return @rv;
}

# get_domain(id, [file])
# Looks up a domain object by ID
sub get_domain
{
	my $self = shift;

if ($_[0] && defined($main::get_domain_cache{$_[0]})) {
	return $main::get_domain_cache{$_[0]};
	}
local %dom;
local $file = $_[1] || "$self->{'domains_dir'}/$_[0]";
$self->read_file($file, \%dom) || return undef;
$dom{'file'} = "$self->{'domains_dir'}/$_[0]";
$dom{'id'} ||= $_[0];
$dom{'mail'} = 1 if (!defined($dom{'mail'}));	# compat - assume mail is on
if (!defined($dom{'ugid'})) {
	# compat - assume user's group is domain's group
	$dom{'ugid'} = $dom{'gid'}
	}
if (!defined($dom{'ugroup'}) && defined($dom{'ugid'})) {
	$dom{'ugroup'} = getgrgid($dom{'ugid'});
	}
if ($dom{'disabled'} eq '1') {
	# compat - assume everything was disabled
	$dom{'disabled'} = "unix,web,dns,mail,mysql,postgres";
	}
elsif ($dom{'disabled'}) {
	# compat - user disabled has changed to unix
	$dom{'disabled'} =~ s/user/unix/g;
	}
if ($dom{'disabled'}) {
	# Manually disabled
	$dom{'disabled_reason'} ||= 'manual';
	}
if (!defined($dom{'created'})) {
	# compat - creation date can be inferred from ID
        $dom{'id'} =~ /^(\d{10})/;
        $dom{'created'} = $1;
        }
if (!defined($dom{'gid'}) && defined($dom{'group'})) {
	# compat - get GID from group name
	$dom{'gid'} = getgrnam($dom{'group'});
	}
if (!defined($dom{'unix'})) {
	# compat - unix is always on
	$dom{'unix'} = 1;
	}
if (!defined($dom{'dir'})) {
	# if unix is on, so is home
	$dom{'dir'} = $dom{'unix'};
	if ($dom{'parent'}) {
		# if server has a parent, it never has a Unix user
		$dom{'unix'} = 0;
		}
	}
if (!defined($dom{'limit_unix'})) {
	# compat - unix is always available for subdomains
	$dom{'limit_unix'} = 1;
	}
if (!defined($dom{'limit_dir'})) {
	# compat - home is always available for subdomains
	$dom{'limit_dir'} = 1;
	}
if (!defined($dom{'virt'})) {
	# compat - assume virtual IP if interface assigned
	$dom{'virt'} = $dom{'iface'} ? 1 : 0;
	}
if (!defined($dom{'web_port'}) && $dom{'web'}) {
	# compat - assume web port is current setting
	$dom{'web_port'} = $self->{'default_web_port'};
	}
if (!defined($dom{'web_sslport'}) && $dom{'ssl'}) {
	# compat - assume SSL port is current setting
	$dom{'web_sslport'} = $self->{'web_sslport'};
	}
if (!defined($dom{'prefix'})) {
	# compat - assume that prefix is same as group
	$dom{'prefix'} = $dom{'group'};
	}
if (!defined($dom{'home'})) {
	local @u = getpwnam($dom{'user'});
	$dom{'home'} = $u[7];
	}
if (!defined($dom{'proxy_pass_mode'}) && $dom{'proxy_pass'}) {
	# assume that proxy pass mode is proxy-based if not set
	$dom{'proxy_pass_mode'} = 1;
	}
if (!defined($dom{'template'})) {
	# assume default parent or sub-server template
	$dom{'template'} = $dom{'parent'} ? 1 : 0;
	}
if (!defined($dom{'db_mysql'}) && $dom{'mysql'}) {
	# Assume just one MySQL DB
	$dom{'db_mysql'} = $dom{'db'};
	}
if (!defined($dom{'db_postgres'}) && $dom{'postgres'}) {
	# Assume just one PostgreSQL DB
	$dom{'db_postgres'} = $dom{'db'};
	}
delete($dom{'missing'});	# never set in a saved domain
if ($_[0]) {
	$main::get_domain_cache{$_[0]} = \%dom;
	}
return \%dom;
}

# get_domain_by(field, value, [field, value, ...])
# Looks up a domain by some field(s)
sub get_domain_by
{
	my $self = shift;

local $d;
local @rv;
foreach $d ($self->list_domains()) {
	local $i;
	local $allok = 1;
	for($i=0; $i<@_; $i+=2) {
		$allok = 0 if ($d->{$_[$i]} ne $_[$i+1]);
		}
	push(@rv, $d) if ($allok);
	}
return wantarray ? @rv : $rv[0];
}

# domain_id()
# Returns a new unique domain ID
sub domain_id
{
	my $self = shift;

return time().$$;
}

# save_domain(&domain)
sub save_domain
{
	my $self = shift;

mkdir($self->{'domains_dir'}, 0700);
$self->lock_file("$self->{'domains_dir'}/$_[0]->{'id'}");
$_[0]->{'id'} = $self->domain_id() if (!$_[0]->{'id'});
$_[0]->{'created'} = time() if (!$_[0]->{'created'});
$self->write_file("$self->{'domains_dir'}/$_[0]->{'id'}", $_[0]);
$self->unlock_file("$self->{'domains_dir'}/$_[0]->{'id'}");
$main::get_domain_cache{$_[0]->{'id'}} = $_[0];
return 1;
}

# delete_domain(&domain)
sub delete_domain
{
	my $self = shift;

$self->lock_file("$self->{'domains_dir'}/$_[0]->{'id'}");
unlink("$self->{'domains_dir'}/$_[0]->{'id'}");
$self->unlock_file("$self->{'domains_dir'}/$_[0]->{'id'}");

# And the bandwidth file
unlink("$self->{'bandwidth_dir'}/$_[0]->{'id'}");
}

# list_domain_users([&domain], [skipunix], [no-virts], [no-quotas])
# List all Unix users who are in the domain's primary group.
# If domain is omitted, returns local users.
sub list_domain_users
{
	my $self = shift;

# Get all aliases (and maybe generics) to look for those that match users
# $self->require_mail();
local (%aliases, %generics);
if ($self->{'config_hash'}->{'mail'} && !$_[2]) {
	$self->require_mail();
	if ($self->{'config_hash'}->{'mail_system'} == 1) {
		# Find Postfix aliases for users
		%aliases = map { $_->{'name'}, $_ } grep { $_->{'enabled'} }
			       $main::sendmail_object->list_aliases($self->{'sendmail_afiles'});
		if ($self->{'config_hash'}->{'generics'}) {
			%generics = map { $_->{'from'}, $_ }
				      $main::sendmail_object->list_generics($self->{'sendmail_gfile'});
			}
		}
	elsif ($self->{'config_hash'}->{'mail_system'} == 0) {
		# Find Sendmail aliases for users
		%aliases = map { $_->{'name'}, $_ }
			       $main::postfix_object->list_aliases($self->{'postfix_afiles'});
		if ($self->{'config_hash'}->{'generics'}) {
			local $cans = $main::postfix_object->get_maps($self->{'canonical_type'});
			%generics = map { $_->{'name'}, $_ } @$cans;
			}
		}
	elsif ($self->{'config_hash'}->{'mail_system'} == 5) {
		# Find VPOPMail aliases to match with users
		%{$self->{'valiases_hash'}} = map { $_->{'from'}, $_ } $self->list_virtusers();
		}
	}

# Get all virtusers to look for those for users
local @virts;
if (!$_[2]) {
	@virts = $self->list_virtusers();
	}

local @users = $self->list_all_users_quotas();

# checking samba clients
if ($self->{'config_hash'}->{'samba'}) {
	$self->require_samba();
	local %smbs = ();
	map { $smbs{$_->{'name'}}++ } $main::samba_object->list_users();
	foreach my $u (@users) {
		$u->{'samba'} = 1 if $smbs{$u->{'user'}};
		}
	}

if ($_[0]) {
	# Limit to domain users.
	@users = grep { defined($_[0]->{'gid'}) &&
			$_->{'gid'} == $_[0]->{'gid'} ||
			$_->{'user'} eq $_[0]->{'user'} } @users;
	local @subdoms;
	if ($_[0]->{'parent'}) {
		# This is a subdomain - exclude parent domain users
		@users = grep { $_->{'home'} =~ /^$_[0]->{'home'}\// } @users;
		}
	elsif (@subdoms = $self->get_domain_by("parent", $_[0]->{'id'})) {
		# This domain has subdomains - exclude their users
		@users = grep { $_->{'home'} !~ /^$_[0]->{'home'}\/domains\// } @users;
		}
	@users = grep { $_->{'user'} ne $_[0]->{'user'} } @users
		if ($_[1] || $_[0]->{'parent'});

	if ($self->{'config_hash'}->{'mail_system'} == 4) {
		# Add Qmail LDAP users (who have same GID?)
		local $ldap = $self->connect_qmail_ldap();
		local $rv = $ldap->search(base => $self->{'config_hash'}->{'ldap_base'},
				  filter => "(&(objectClass=qmailUser)(|(qmailGID=$_[0]->{'gid'})(gidNumber=$_[0]->{'gid'})))");
		$self->error($rv->error) if ($rv->code);
		local $u;
		foreach $u ($rv->all_entries) {
			local %uinfo = $self->qmail_dn_to_hash($u);
			next if (!$uinfo{'mailstore'});	# alias only
			$uinfo{'ldap'} = $u;
			if ($_[0]->{'parent'}) {
				# In sub-domain, exclude parent domain users
				next if ($_->{'home'} !~ /^$_[0]->{'home'}\//);
				}
			elsif (@subdoms) {
				# In parent domain exclude sub-domain users
				next if ($_->{'home'} =~ /^$_[0]->{'home'}\/domains\//);
				}
			@users = grep { $_->{'user'} ne $uinfo{'user'} } @users;
			push(@users, \%uinfo);
			}
		$ldap->unbind();
		}
	elsif ($self->{'config_hash'}->{'mail_system'} == 5) {
		# Add VPOPMail users for this domain
		local %attr_map = ( 'name' => 'user',
				    'passwd' => 'pass',
				    'clear passwd' => 'plainpass',
				    'comment/gecos' => 'real',
				    'dir' => 'home',
				    'quota' => 'qquota',
				   );
		local $user;
		open(UINFO, "$self->{'vpopbin'}/vuserinfo -D $_[0]->{'dom'} |");
		while(<UINFO>) {
			s/\r|\n//g;
			if (/^([^:]+):\s+(.*)$/) {
				local ($attr, $value) = ($1, $2);
				if ($attr eq "name") {
					# Start of a new user
					$user = { 'vpopmail' => 1,
						  'mailquota' => 1,
						  'person' => 1,
						  'fixedhome' => 1,
						  'noappend' => 1,
						  'noprimary' => 1 };
					push(@users, $user);
					}
				local $amapped = $attr_map{$attr};
				$user->{$amapped} = $value if ($amapped);
				if ($amapped eq "qquota") {
					if ($value eq "NOQUOTA") {
						$user->{$amapped} = 0;
						}
					else {
						$user->{$amapped} = int($value);
						}
					}
				}
			}
		close(UINFO);
		}

        # Find users with broken home dir
        foreach my $u (@users) {
                if ($u->{'home'} &&
                    $u->{'home'} !~ /^$self->{'d'}->{'home'}\/$self->{'config_hash'}->{'homes_dir'}\// &&
                    !$self->is_under_directory($self->{'d'}->{'home'}, $u->{'home'})) {
                        $u->{'brokenhome'} = 1;
                        }
                }
	}
else {
	# Limit to local users
	local @lg = getgrnam($self->{'config_hash'}->{'localgroup'});
	@users = grep { $_->{'gid'} == $lg[2] } @users;
	}
if (!$_[2]) {
	# Add email addresses and forwarding addresses to user structures
	local $u;
	foreach $u (@users) {
		next if ($u->{'qmail'});	# got from LDAP already
		$u->{'email'} = $u->{'virt'} = undef;
		$u->{'alias'} = $u->{'to'} = $u->{'generic'} = undef;
		$u->{'extraemail'} = $u->{'extravirt'} = undef;
		local ($al, $va);
		if ($al = $aliases{$self->escape_alias($u->{'user'})}) {
			$u->{'alias'} = $al;
			$u->{'to'} = $al->{'values'};
			}
		elsif ($va = $self->{'valiases_hash'}->{"$u->{'user'}\@$self->{'d'}->{'dom'}"}) {
			$u->{'valias'} = $va;
			$u->{'to'} = $va->{'to'};
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 2 ||
		       $self->{'config_hash'}->{'mail_system'} == 5) {
			# Find .qmail file
			local $alias = $self->get_dotqmail($self->dotqmail_file($u));
			if ($alias) {
				$u->{'alias'} = $alias;
				$u->{'to'} = $u->{'alias'}->{'values'};
				}
			}
		$u->{'generic'} = $generics{$u->{'user'}};
		local $pop3 = $_[0] ? $self->remove_userdom($u->{'user'}, $_[0])
				    : $u->{'user'};
		local $email = $_[0] ? "$pop3\@$_[0]->{'dom'}" : undef;
		local $escuser = $self->escape_user($u->{'user'});
		local $escalias = $self->escape_alias($u->{'user'});
		local $v;
		foreach $v (@virts) {
			if (@{$v->{'to'}} == 1 &&
			    ($v->{'to'}->[0] eq $escuser ||
			     $v->{'to'}->[0] eq $escalias ||
			     $v->{'from'} eq $email &&
			      $v->{'to'}->[0] eq "BOUNCE") &&
			    (!$_[0] || $v->{'from'} ne $_[0]->{'dom'})) {
				if ($v->{'from'} eq $email) {
					if ($v->{'to'}->[0] ne "BOUNCE") {
						$u->{'email'} = $email;
						}
					$u->{'virt'} = $v;
					}
				else {
					push(@{$u->{'extraemail'}},
					     $v->{'from'});
					push(@{$u->{'extravirt'}}, $v);
					}
				}
			}
		}
	}

local @sgroups = $self->allowed_secondary_groups($_[0]);
if (@sgroups) {
        # Add any secondary groups in the template
        local @groups = $self->list_all_groups();
        foreach my $u (@users) {
                $u->{'secs'} = [ ];
                }
        foreach my $g (@sgroups) {
                local ($group) = grep { $_->{'group'} eq $g } @groups;
                if ($group) {
                        local %mems = map { $_, 1 }
                                          split(/,/, $group->{'members'});
                        foreach my $u (@users) {
                                if ($mems{$u->{'user'}}) {
                                        push(@{$u->{'secs'}}, $g);
                                        }
                                }
                        }
                }
        }

#parse users' real names
foreach my $u (@users) {
	$u->{'real'} =~ s/^(.*?),.*/$1/g;
	}

return @users;
}

# list_all_users_quotas()
# Returns a list of all Unix users, with quota info
sub list_all_users_quotas
{
	my $self = shift;

# Get quotas for all users
$self->require_useradmin();
local $qv = $self->{'config_hash'}->{'hard_quotas'} ? 'hblocks' : 'sblocks';
if (!defined(%{$self->{'home_quotas_hash'}}) && $self->{'config_hash'}->{'home_quotas'}) {
	local $n = $main::quota_object->filesystem_users($self->{'config_hash'}->{'home_quotas'});
	local $i;
	for($i=0; $i<$n; $i++) {
		$self->{'home_quota_hash'}->{$main::quota_object->{'user_hash'}->{$i,'user'}} = $main::quota_object->{'user_hash'}->{$i,$qv};
		$self->{'used_home_quota_hash'}->{$main::quota_object->{'user_hash'}->{$i,'user'}} =
			$main::quota_object->{'user_hash'}->{$i,'ublocks'};
		}
	}
if (!defined(%{$self->{'mail_quotas_hash'}}) && $self->{'config_hash'}->{'mail_quotas'} &&
    $self->{'config_hash'}->{'mail_quotas'} ne $self->{'config_hash'}->{'home_quotas'}) {
	local $n = $main::quota_object->filesystem_users($self->{'config_hash'}->{'mail_quotas'});
	local $i;
	for($i=0; $i<$n; $i++) {
		$self->{'mail_quota_hash'}->{$main::quota_object->{'user_hash'}->{$i,'user'}} = $main::quota_object->{'user_hash'}->{$i,$qv};
		$self->{'used_mail_quota_hash'}->{$main::quota_object->{'user_hash'}->{$i,'user'}} =
			$main::quota_object->{'user_hash'}->{$i,'ublocks'};
		}
	}

# Get user list and add in quota info
local @users = $self->foreign_call($self->{'usermodule'}, "list_users");
local $u;
foreach $u (@users) {
	$u->{'quota'} = $self->{'home_quota_hash'}->{$u->{'user'}};
	$u->{'uquota'} = $self->{'used_home_quota_hash'}->{$u->{'user'}};
	$u->{'mquota'} = $self->{'mail_quota_hash'}->{$u->{'user'}};
	$u->{'umquota'} = $self->{'used_mail_quota_hash'}->{$u->{'user'}};
	$u->{'unix'} = 1;
	$u->{'person'} = 1;
	}
return @users;
}

# create_user(&user, [&domain])
# Create a mailbox or local user, his virtuser and possibly his alias
sub create_user
{
	my $self = shift;

local $pop3 = $self->remove_userdom($_[0]->{'user'}, $_[1]);
$self->require_useradmin();
$self->require_mail();

if ($_[0]->{'qmail'}) {
	# Create user in Qmail LDAP
	local $ldap = $self->connect_qmail_ldap();
	local $_[0]->{'dn'} = "uid=$_[0]->{'user'},$self->{'config_hash'}->{'ldap_base'}";
	local @oc = ( "qmailUser" );
	push(@oc, "posixAccount") if ($_[0]->{'unix'});
	push(@oc, split(/\s+/, $self->{'config_hash'}->{'ldap_classes'}));
	local $attrs = $self->qmail_user_to_dn($_[0], \@oc, $_[1]);
	push(@$attrs, "objectClass" => \@oc);
	local $rv = $ldap->add($_[0]->{'dn'}, attr => $attrs);
	$self->error($rv->error) if ($rv->code);
	$ldap->unbind();
	}
elsif ($_[0]->{'vpopmail'}) {
	# Create user in VPOPMail
	local $quser = quotemeta($_[0]->{'user'});
	local $qdom = $_[1]->{'dom'};
	local $qreal = quotemeta($_[0]->{'real'}) || '""';
	local $quota = $_[0]->{'qquota'} ? "-q $_[0]->{'qquota'}" : "-q NOQUOTA";
	local $qpass = quotemeta($_[0]->{'plainpass'});
	local $cmd = "$self->{'vpopbin'}/vadduser $quota -c $qreal $quser\@$qdom $qpass";
	local $out = $self->backquote_logged("$cmd 2>&1");
	$self->error("<tt>$cmd</tt> failed: <pre>$out</pre>") if ($?);
	$_[0]->{'home'} = "$self->{'config_hash'}->{'vpopmail_dir'}/domains/$_[1]->{'dom'}/$_[0]->{'user'}";
	}
else {
	# Add the Unix user
	$self->foreign_call($self->{'usermodule'}, "set_user_envs", $_[0], 'CREATE_USER', $_[0]->{'plainpass'}, [ ]);
	$self->foreign_call($self->{'usermodule'}, "making_changes");
	$self->foreign_call($self->{'usermodule'}, "lock_user_files");
	$self->userdom_substitutions($_[0], $_[1]);
	$self->foreign_call($self->{'usermodule'}, "create_user", $_[0]);
	$self->foreign_call($self->{'usermodule'}, "unlock_user_files");
	$self->foreign_call($self->{'usermodule'}, "made_changes");
	}

local $firstemail;
if (!$_[0]->{'qmail'}) {
	# Add his virtusers for non Qmail+LDAP users
	local $vto = $_[0]->{'to'} ? $self->escape_alias($_[0]->{'user'})
				   : $self->escape_user($_[0]->{'user'});
	if ($_[0]->{'email'}) {
		local $virt = { 'from' => $_[0]->{'email'},
				'to' => [ $vto ] };
		$self->create_virtuser($virt);
		$_[0]->{'virt'} = $virt;
		$firstemail ||= $_[0]->{'email'};
		}
	elsif ($self->{'can_alias_types_hash'}->{9} && $_[1] && !$_[0]->{'noprimary'}) {
		# Add bouncer if email disabled
		local $virt = { 'from' => "$pop3\@$_[1]->{'dom'}",
				'to' => [ "BOUNCE" ] };
		$self->create_virtuser($virt);
		$_[0]->{'virt'} = $virt;
		}
	local @extravirt;
	local $e;
	foreach $e (@{$_[0]->{'extraemail'}}) {
		local $virt = { 'from' => $e,
				'to' => [ $vto ] };
		$self->create_virtuser($virt);
		push(@extravirt, $virt);
		$firstemail ||= $e;
		}
	$_[0]->{'extravirt'} = \@extravirt;
	}

if (!$_[0]->{'qmail'}) {
	# Add his alias, if any, for non Qmail+LDAP users
	local @to = @{$_[0]->{'to'}};
	if (@to) {
		local $alias = { 'name' => $self->escape_alias($_[0]->{'user'}),
				 'enabled' => 1,
				 'values' => $_[0]->{'to'} };
		$self->check_alias_clash($_[0]->{'user'}) &&
			$self->error($self->text('alias_eclash2', $_[0]->{'user'}));
		if ($self->{'config_hash'}->{'mail_system'} == 1) {
			$main::sendmail_object->lock_alias_files($self->{'sendmail_afiles'});
			$main::sendmail_object->create_alias($alias, $self->{'sendmail_afiles'});
			$main::sendmail_object->unlock_alias_files($self->{'sendmail_afiles'});
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 0) {
			$main::postfix_object->lock_alias_files($self->{'postfix_afiles'});
			$main::postfix_object->create_alias($alias, $self->{'postfix_afiles'});
			$main::postfix_object->unlock_alias_files($self->{'postfix_afiles'});
			$main::postfix_object->regenerate_aliases();
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 2 ||
		       $self->{'config_hash'}->{'mail_system'} == 5) {
			# Set up user's .qmail file
			local $dqm = $self->dotqmail_file($_[0]);
			$self->lock_file($dqm);
			$self->save_dotqmail($alias, $dqm, $pop3);
			$self->unlock_file($dqm);
			}
		$_[0]->{'alias'} = $alias;
		}

	if ($self->{'config_hash'}->{'generics'} && $firstemail) {
		# Add genericstable entry too
		if ($self->{'config_hash'}->{'mail_system'} == 1) {
			local $gen = { 'from' => $_[0]->{'user'}, 'to' => $firstemail };
			$self->lock_file($self->{'sendmail_gfile'});
			$main::sendmail_object->create_generic($gen, $self->{'sendmail_gfile'},
						  $self->{'sendmail_gdbm'}, $self->{'sendmail_gdbmtype'});
			$self->unlock_file($self->{'sendmail_gfile'});
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 0) {
			local $gen = { 'name' => $_[0]->{'user'},
				       'value' => $firstemail };
			$self->lock_file($self->{'canonical_map_files_array'}->[0]);
			$self->create_replace_mapping($self->{'canonical_type'}, $gen);
			$self->unlock_file($self->{'canonical_map_files_array'}->[0]);
			$main::postfix_object->regenerate_canonical_table();
			}
		}
	}

# Add user to any secondary groups
local @groups;
@groups = $self->list_all_groups() if (@{$_[0]->{'secs'}});
foreach my $g (@{$_[0]->{'secs'}}) {
        local ($group) = grep { $_->{'group'} eq $g } @groups;
        if ($group) {
                local @mems = split(/,/, $group->{'members'});
                push(@mems, $_[0]->{'user'});
                $group->{'members'} = join(",", @mems);
                $self->foreign_call($self->{'usermodule'}, "modify_group", $group, $group);
                }
        }
}

# modify_user(&user, &old, &domain, [noaliases])
sub modify_user
{
	my $self = shift;

local $pop3 = $self->remove_userdom($_[0]->{'user'}, $_[2]);
if ($_[1]->{'qmail'}) {
	# Update user in Qmail LDAP
	local $ldap = $self->connect_qmail_ldap();
	local $_[0]->{'dn'} = "uid=$_[0]->{'user'},$self->{'config_hash'}->{'ldap_base'}";
	local ($attrs, $delattrs) = $self->qmail_user_to_dn($_[0],
		[ $_[1]->{'ldap'}->get_value("objectClass") ], $_[2]);
	@$delattrs = grep { defined($_[1]->{'ldap'}->get_value($_))} @$delattrs;
	local (%attrs, $i);
	for($i=0; $i<@$attrs; $i+=2) {
		$attrs{$attrs->[$i]} = $attrs->[$i+1];
		}
	local $rv = $ldap->modify($_[1]->{'dn'},
				  replace => \%attrs,
				  delete => $delattrs);
	$self->error($rv->error) if ($rv->code);
	if ($_[0]->{'dn'} ne $_[1]->{'dn'}) {
		# Re-named too!
		$rv = $ldap->moddn($_[1]->{'dn'},
				   newrdn => "uid=$_[0]->{'user'}");
		$self->error($rv->error) if ($rv->code);
		}
	$ldap->unbind();
	}
elsif ($_[1]->{'vpopmail'}) {
	# Update VPOPMail user
	local $quser = quotemeta($_[1]->{'user'});
	local $qdom = $_[2]->{'dom'};
	local $qreal = quotemeta($_[0]->{'real'}) || '""';
	local $qpass = quotemeta($_[0]->{'plainpass'});
	local $qquota = $_[0]->{'qquota'} ? $_[0]->{'qquota'} : "NOQUOTA";
	local $cmd = "$self->{'vpopbin'}/vmoduser -c $qreal ".
		     ($_[0]->{'passmode'} == 3 ? " -C $qpass" : "").
		     " -q $qquota $quser\@$qdom";
	local $out = $self->backquote_logged("$cmd 2>&1");
	if ($?) {
		$self->error("<tt>$cmd</tt> failed: <pre>$out</pre>");
		}
	if ($_[0]->{'user'} ne $_[1]->{'user'}) {
		# Need to rename manually
		local $vdomdir = "$self->{'config_hash'}->{'vpopmail_dir'}/domains/$_[2]->{'dom'}";
		$self->rename_logged("$vdomdir/$_[1]->{'user'}", "$vdomdir/$_[0]->{'user'}");
		$self->lock_file("$vdomdir/vpasswd");
		local $lref = $self->read_file_lines("$vdomdir/vpasswd");
		local $l;
		foreach $l (@$lref) {
			local @u = split(/:/, $l);
			if ($u[0] eq $_[1]->{'user'}) {
				$u[0] = $_[0]->{'user'};
				$u[5] =~ s/$_[1]->{'user'}$/$_[0]->{'user'}/;
				$l = join(":", @u);
				}
			}
		$self->flush_file_lines();
		$self->unlock_file("$vdomdir/vpasswd");
		$self->system_logged("$self->{'vpopbin'}/vmkpasswd $qdom");
		}
	}
else {
	# Modifying Unix user
	$self->require_useradmin();
	$self->require_mail();

	# Update the unix user
	$self->foreign_call($self->{'usermodule'}, "set_user_envs", $_[0], 'MODIFY_USER', $_[0]->{'plainpass'});
	$self->foreign_call($self->{'usermodule'}, "making_changes");
	$self->foreign_call($self->{'usermodule'}, "lock_user_files");
	$self->userdom_substitutions($_[0], $_[2]);
#	if($_[1]->{'real'} ne $_[0]->{'real'}){$_[1]->{'real'}=$_[0]->{'real'};}
	$self->foreign_call($self->{'usermodule'}, "modify_user", $_[1], $_[0]);
	$self->foreign_call($self->{'usermodule'}, "unlock_user_files");
	$self->foreign_call($self->{'usermodule'}, "made_changes");
	return if ($_[3]);		# no need to touch aliases and virtusers
	}

local $firstemail;
if (!$_[0]->{'qmail'}) {
	# Take away all virtusers and add new ones, for non Qmail+LDAP users
	$self->delete_virtuser($_[1]->{'virt'}) if ($_[1]->{'virt'});
	local $e;
	foreach $e (@{$_[1]->{'extravirt'}}) {
		$self->delete_virtuser($e);
		}
	local $vto = $_[0]->{'to'} ? $self->escape_alias($_[0]->{'user'})
				   : $self->escape_user($_[0]->{'user'});
	if ($_[0]->{'email'}) {
		local $virt = { 'from' => $_[0]->{'email'},
				'to' => [ $vto ] };
		$self->create_virtuser($virt);
		$_[0]->{'virt'} = $virt;
		$firstemail ||= $_[0]->{'email'};
		}
	elsif ($self->{'can_alias_types_hash'}->{9} && $_[2] && !$_[0]->{'noprimary'}) {
		# Add bouncer if email disabled
		local $virt = { 'from' => "$pop3\@$_[2]->{'dom'}",
				'to' => [ "BOUNCE" ] };
		$self->create_virtuser($virt);
		$_[0]->{'virt'} = $virt;
		}
	local @extravirt;
	foreach $e (@{$_[0]->{'extraemail'}}) {
		local $virt = { 'from' => $e,
				'to' => [ $vto ] };
		$self->create_virtuser($virt);
		push(@extravirt, $virt);
		$firstemail ||= $e;
		}
	$_[0]->{'extravirt'} = \@extravirt;
	}

local @to = @{$_[0]->{'to'}};
local @oldto = @{$_[1]->{'to'}};
if (!$_[0]->{'qmail'}) {
	# Update, create or delete alias, for non Qmail+LDAP users
	if (@to && !@oldto) {
		# Need to add alias
		local $alias = { 'name' => $self->escape_alias($_[0]->{'user'}),
				 'enabled' => 1,
				 'values' => $_[0]->{'to'} };
		$self->check_alias_clash($_[0]->{'user'}) &&
			$self->error($self->text('alias_eclash2', $_[0]->{'user'}));
		if ($self->{'config_hash'}->{'mail_system'} == 1) {
			# Create Sendmail alias with same name as user
			$main::sendmail_object->lock_alias_files($self->{'sendmail_afiles'});
			$main::sendmail_object->create_alias($alias, $self->{'sendmail_afiles'});
			$main::sendmail_object->unlock_alias_files($self->{'sendmail_afiles'});
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 0) {
			# Create Postfix alias with same name as user
			$main::postfix_object->lock_alias_files($self->{'postfix_afiles'});
			$main::postfix_object->create_alias($alias, $self->{'postfix_afiles'});
			$main::postfix_object->unlock_alias_files($self->{'postfix_afiles'});
			$main::postfix_object->regenerate_aliases();
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 2 ||
		       $self->{'config_hash'}->{'mail_system'} == 5) {
			# Set up user's .qmail file
			local $dqm = $self->dotqmail_file($_[0]);
			$self->lock_file($dqm);
			$self->save_dotqmail($alias, $dqm, $pop3);
			$self->unlock_file($dqm);
			}
		$_[0]->{'alias'} = $alias;
		}
	elsif (!@to && @oldto) {
		# Need to delete alias
		if ($self->{'config_hash'}->{'mail_system'} == 1) {
			# Delete Sendmail alias
			$self->lock_file($_[0]->{'alias'}->{'file'});
			$main::sendmail_object->delete_alias($_[0]->{'alias'});
			$self->unlock_file($_[0]->{'alias'}->{'file'});
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 0) {
			# Delete Postfix alias
			$self->lock_file($_[0]->{'alias'}->{'file'});
			$main::postfix_object->delete_alias($_[0]->{'alias'});
			$self->unlock_file($_[0]->{'alias'}->{'file'});
			$main::postfix_object->regenerate_aliases();
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 2 ||
		       $self->{'config_hash'}->{'mail_system'} == 5) {
			# Remove user's .qmail file
			local $dqm = $self->dotqmail_file($_[0]);
			$self->lock_file($dqm);
			unlink($dqm);
			$self->unlock_file($dqm);
			}
		}
	elsif (@to && @oldto) {
		# Need to update the alias
		local $alias = { 'name' => $self->escape_alias($_[0]->{'user'}),
				 'enabled' => 1,
				 'values' => $_[0]->{'to'} };
		if ($self->{'config_hash'}->{'mail_system'} == 1) {
			# Update Sendmail alias
			$self->lock_file($_[1]->{'alias'}->{'file'});
			$main::sendmail_object->modify_alias($_[1]->{'alias'}, $alias);
			$self->unlock_file($_[1]->{'alias'}->{'file'});
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 0) {
			# Update Postfix alias
			$self->lock_file($_[1]->{'alias'}->{'file'});
			$main::postfix_object->modify_alias($_[1]->{'alias'}, $alias);
			$self->unlock_file($_[1]->{'alias'}->{'file'});
			$main::postfix_object->regenerate_aliases();
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 2 ||
		       $self->{'config_hash'}->{'mail_system'} == 5) {
			# Set up user's .qmail file
			local $dqm = $self->dotqmail_file($_[0]);
			$self->lock_file($dqm);
			$self->save_dotqmail($alias, $dqm, $pop3);
			$self->unlock_file($dqm);
			}
		$_[0]->{'alias'} = $alias;
		}

	if ($self->{'config_hash'}->{'generics'} && $_[0]->{'generic'}) {
		# Update genericstable entry too
		if ($self->{'config_hash'}->{'mail_system'} == 1) {
			$self->lock_file($self->{'sendmail_gfile'});
			$main::sendmail_object->delete_generic($_[0]->{'generic'}, $self->{'sendmail_gfile'},
					$self->{'sendmail_gdbm'}, $self->{'sendmail_gdbmtype'});
			if ($firstemail) {
				local $gen = { 'from' => $_[0]->{'user'},
					       'to' => $firstemail };
				$main::sendmail_object->create_generic($gen, $self->{'sendmail_gfile'},
						  $self->{'sendmail_gdbm'}, $self->{'sendmail_gdbmtype'})
				}
			$self->unlock_file($self->{'sendmail_gfile'});
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 0) {
			$self->lock_file($self->{'canonical_map_files_array'}->[0]);
			$main::postfix_object->delete_mapping($self->{'canonical_type'}, $_[0]->{'generic'});
			if ($firstemail) {
				local $gen = { 'name' => $_[0]->{'user'},
					       'value' => $firstemail };
				$self->create_replace_mapping($self->{'canonical_type'}, $gen);
				}
			$self->unlock_file($self->{'canonical_map_files_array'}->[0]);
			$main::postfix_object->regenerate_canonical_table();
			}
		}
	}

# Rename user in secondary groups, and update membership
local @groups = $self->list_all_groups();
local %secs = map { $_, 1 } @{$_[0]->{'secs'}};
foreach my $group (@groups) {
        local @mems = split(/,/, $group->{'members'});
        local $idx = $self->indexof($_[1]->{'user'}, @mems);
        local $changed;
        if ($idx >= 0) {
                # User is currently in group
                if ($secs{$group->{'group'}}) {
                        # Just rename in group
                        if ($_[0]->{'user'} ne $_[1]->{'user'}) {
                                $changed = 1;
                                $mems[$idx] = $_[0]->{'user'};
                                }
                        }
                else {
                        # Remove from group
                        splice(@mems, $idx, 1);
                        $changed = 1;
                        }
                }
        elsif ($secs{$group->{'group'}}) {
                # User is not in group, but needs to be
                push(@mems, $_[0]->{'user'});
                $changed = 1;
                }
        if ($changed) {
                # Only save group if members were changed
                $group->{'members'} = join(",", @mems);
                $self->foreign_call($self->{'usermodule'}, "modify_group", $group, $group);
                }
        }
}

# delete_user(&user, domain)
# Delete a mailbox user and all associated virtusers and aliases
sub delete_user
{
	my $self = shift;

if ($_[0]->{'qmail'}) {
	# Delete user in Qmail LDAP
	local $ldap = $self->connect_qmail_ldap();
	local $rv = $ldap->delete($_[0]->{'dn'});
	$self->error($rv->error) if ($rv->code);
	$ldap->unbind();
	}
elsif ($_[0]->{'vpopmail'}) {
	# Call VPOPMail delete user program
	local $quser = quotemeta($_[0]->{'user'});
	local $qdom = $_[1]->{'dom'};
	local $cmd = "$self->{'vpopbin'}/vdeluser $quser\@$qdom";
	local $out = $self->backquote_logged("$cmd 2>&1");
	if ($?) {
		$self->error("<tt>$cmd</tt> failed: <pre>$out</pre>");
		}
	}
else {
	# Delete Unix user
	$_[0]->{'user'} eq 'root' && $self->error("Cannot delete root user!");
	$_[0]->{'uid'} == 0 && $self->error("Cannot delete UID 0 user!");
	$self->require_useradmin();
	$self->require_mail();

	# Delete the user
	$self->foreign_call($self->{'usermodule'}, "set_user_envs", $_[0], 'DELETE_USER');
	$self->foreign_call($self->{'usermodule'}, "making_changes");
	$self->foreign_call($self->{'usermodule'}, "lock_user_files");
	$self->foreign_call($self->{'usermodule'}, "delete_user",$_[0]);
	$self->foreign_call($self->{'usermodule'}, "unlock_user_files");
	$self->foreign_call($self->{'usermodule'}, "made_changes");
	}

if (!$_[0]->{'qmail'}) {
	# Delete any virtusers
	$self->delete_virtuser($_[0]->{'virt'}) if ($_[0]->{'virt'});
	local $e;
	foreach $e (@{$_[0]->{'extravirt'}}) {
		$self->delete_virtuser($e);
		}
	}

if (!$_[0]->{'qmail'}) {
	# Delete his alias, if any
	if ($_[0]->{'alias'}) {
		if ($self->{'config_hash'}->{'mail_system'} == 1) {
			# Delete Sendmail alias with same name as user
			$self->lock_file($_[0]->{'alias'}->{'file'});
			$main::sendmail_object->delete_alias($_[0]->{'alias'});
			$self->unlock_file($_[0]->{'alias'}->{'file'});
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 0) {
			# Delete Postfix alias with same name as user
			$self->lock_file($_[0]->{'alias'}->{'file'});
			$main::postfix_object->delete_alias($_[0]->{'alias'});
			$self->unlock_file($_[0]->{'alias'}->{'file'});
			$main::postfix_object->regenerate_aliases();
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 2 ||
		       $self->{'config_hash'}->{'mail_system'} == 5) {
			# .qmail will be deleted when user is
			}
		}

	if ($self->{'config_hash'}->{'generics'} && $_[0]->{'generic'}) {
		# Delete genericstable entry too
		if ($self->{'config_hash'}->{'mail_system'} == 1) {
			$self->lock_file($self->{'sendmail_gfile'});
			$main::sendmail_object->delete_generic($_[0]->{'generic'}, $self->{'sendmail_gfile'},
					$self->{'sendmail_gdbm'}, $self->{'sendmail_gdbmtype'});
			$self->unlock_file($self->{'sendmail_gfile'});
			}
		elsif ($self->{'config_hash'}->{'mail_system'} == 0) {
			$self->lock_file($_[0]->{'generic'}->{'file'});
			$main::postfix_object->delete_mapping($self->{'canonical_type'}, $_[0]->{'generic'});
			$self->unlock_file($_[0]->{'generic'}->{'file'});
			$main::postfix_object->regenerate_canonical_table();
			}
		}
	}

# Take the user out of any secondary groups
local @groups = $self->list_all_groups();
foreach my $group (@groups) {
        local @mems = split(/,/, $group->{'members'});
        local $idx = $self->indexof($_[0]->{'user'}, @mems);
        if ($idx >= 0) {
                splice(@mems, $idx, 1);
                $group->{'members'} = join(",", @mems);
                $self->foreign_call($self->{'usermodule'}, "modify_group", $group, $group);
                }
        }
}

# domain_title(&domain)
sub domain_title
{
	my $self = shift;

print "<center><font size=+1>",$self->domain_in($_[0]),"</font></center>\n";
}

# domain_in(&domain)
sub domain_in
{
	my $self = shift;

return $self->text('indom', "<tt>$_[0]->{'dom'}</tt>");
}

# copy_skel_files(basedir, &user, home, [group])
# Copy files to the home directory of some new user
sub copy_skel_files
{
	my $self = shift;

local $uf = $_[0];
return if (!$uf);
$self->require_useradmin();
if ($_[1]) {
	local $shell = $_[1]->{'shell'};
	$shell =~ s/^(.*)\///g;
	local $group = $_[3];
	$group = getgrgid($_[1]->{'gid'}) if (!$group);
	$uf =~ s/\$group/$group/g;
	$uf =~ s/\$gid/$_[1]->{'gid'}/g;
	$uf =~ s/\$shell/$shell/g;
	$main::useradmin_object->copy_skel_files($uf, $_[2],
				    $_[1]->{'uid'}, $_[1]->{'gid'});
	}
else {
	$uf =~ s/\$group/nogroup/g;
	$uf =~ s/\$gid/100/g;
	$uf =~ s/\$shell/\/bin\/false/g;
	$main::useradmin_object->copy_skel_files($uf, $_[2], 0, 0);
	}
}

# can_edit_domain(&domain)
# Returns 1 if the current user can edit some domain
sub can_edit_domain
{
	my $self = shift;

return 1 if ($self->{'access_hash'}->{'domains'} eq "*");
local $d;
foreach $d (split(/\s+/, $self->{'access_hash'}->{'domains'})) {
	return 1 if ($d eq $_[0]->{'id'});
	}
return 0;
}

# master_admin()
# Returns 1 if the current user is the master Virtualmin admin, and can edit
# global settings
sub master_admin
{
	my $self = shift;

return !$self->{'access_hash'}->{'noconfig'};
}

# domains_table(&domains)
# Display a list of domains in a table, with links for editing
sub domains_table
{
	my $self = shift;

local @table_features = $self->{'config_hash'}->{'show_features'} ?
    (grep { $_ ne 'webmin' && $_ ne 'mail' &&
	    $_ ne 'unix' && $_ ne 'dir' } @{$self->{'features_array'}}) : ( );
print "<table border width=100% cellpadding=3>\n";
print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'index_domain'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'index_user'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'index_owner'}</b></td> ";
local $f;
foreach $f (@table_features) {
	print "<td><b>",$self->{'text_hash'}->{'index_'.$f},"</b></td> " if ($self->{'config_hash'}->{$f});
	}
if ($self->{'config_hash'}->{'mysql'} || $self->{'config_hash'}->{'postgres'}) {
    print "<td><b>$self->{'text_hash'}->{'index_databases'}</b></td> ";
}
if ($self->{'config_hash'}->{'mail'} || $self->{'config_hash'}->{'web'} || $self->{'config_hash'}->{'ftp'} || $self->{'config_hash'}->{'samba'}) {
	print "<td><b>$self->{'text_hash'}->{'website_list_table_users'}</b></td> ";
#	print "<td><b>$self->{'text_hash'}->{'index_alias'}</b></td> ";
}
if ($self->{'config_hash'}->{'home_quotas'}) {
	print "<td><b>$self->{'text_hash'}->{'index_quota'}</b></td> ".
	      "<td><b>$self->{'text_hash'}->{'index_uquota'}</b></td> ";
	}

print "<td><b>$self->{'text_hash'}->{'website_actions'}</b></td> ";
print "</tr>\n";
local $d;
local %done;
local $sortfield = $self->{'config_hash'}->{'domains_sort'} || "user";
foreach $d (sort { $a->{$sortfield} cmp $b->{$sortfield} ||
		   $a->{'created'} <=> $b->{'created'} } @{$_[0]}) {
	$done{$d->{'id'}}++;
	print "<tr $self->{'cb'}>\n";
	local $dn = $d->{'disabled'} ? "<i>$d->{'dom'}</i>" : $d->{'dom'};
	local $pfx;
        $pfx .= "&nbsp;&nbsp;" if ($d->{'parent'} && $done{$d->{'parent'}} &&
                                   $sortfield eq "user");
        $pfx .= "&nbsp;&nbsp;" if ($d->{'alias'} && $done{$d->{'alias'}} &&
                                   $sortfield eq "user");
	local $subdomain = ($d->{'parent'} && $done{$d->{'parent'}});
	if ($self->{'access_hash'}->{'edit'}) {
		print "<td>$pfx<a class=user href='edit_domain.cgi?dom=$d->{'id'}'>",
		      "$dn</a></td>\n";
		}
	else {
		print "<td>$pfx$dn</td>\n";
		}
	print "<td>$d->{'user'}</td>\n";
	print "<td>",$d->{'owner'} eq "" ? "&nbsp;": $d->{'owner'},"\n";
	if ($d->{'alias'}) {
		local $aliasdom = $self->get_domain($d->{'alias'});
		local $of = $self->text('index_aliasof', $aliasdom->{'dom'});
		print $d->{'owner'} ? "($of)" : $of;
		}
	print "</td>\n";
	foreach $f (@table_features) {
            print "<td>",$d->{$f} ? "<img src='images/yes.gif' border=0 width=12 height=12>" : "<img src='images/no.gif' border=0 width=12 height=12>","</td>\n"
                        if ($self->{'config_hash'}->{$f});
		}
	if ($self->{'config_hash'}->{'mysql'} || $self->{'config_hash'}->{'postgres'}) {
	    local $count_db = $self->domain_databases($d);
	    print "<td><a href='list_databases.cgi?dom=$d->{'id'}'>$count_db&nbsp;<img src='images/web_dbs.gif' border=0 width=16 height=16></a></td>";
	}		
	local @users = $self->list_domain_users($d, 0, 1, 0);
	local ($duser) = grep { $_->{'user'} eq $d->{'user'} } @users;
	if ($self->{'config_hash'}->{'mail'} || $self->{'config_hash'}->{'ftp'} || $self->{'config_hash'}->{'ftp'} || $self->{'config_hash'}->{'samba'}) {
		if ($d->{'mail'} || $self->{'config_hash'}->{'ftp'} || $d->{'web'} || $d->{'samba'}) {
			local @aliases = $self->list_domain_aliases($d);
			if ($d->{'alias'}) {
				print "<td>0</td>\n";
				}
			else {
				printf "<td>%d&nbsp;<a href='list_users.cgi?dom=$d->{'id'}'>&nbsp;<img src='images/users.gif' border=0 width=16 height=16></a></td>\n", scalar(@users);
				}
			}
		else {
			print "<td colspan=1>$self->{'text_hash'}->{'index_nomail'}</td>\n";
			}
		}
	if ($self->{'config_hash'}->{'home_quotas'}) {
		if ($d->{'parent'}) {
			# Domains with parent have no quota
			# if ($done{$d->{'parent'}}) {
			#	print "<td>&nbsp;&nbsp;\"</td>\n";
			#	}
			# else {
				print "<td>$self->{'text_hash'}->{'index_samequ'}</td>\n";
			#	}
			}
		else {
			# Show quota for domain
			print "<td>",$d->{'quota'} ?
			  $self->quota_show($d->{'quota'}, $self->{'config_hash'}->{'home_quotas'}) :
			  $self->{'text_hash'}->{'form_unlimit'},"</td>\n";
			}
		if ($d->{'alias'}) {
			# Alias domains have no usage
			print "<td><br></td>\n";
			}
		else {
			# Show total usage for domain
			local $ut = $duser->{'uquota'} + $duser->{'umquota'};
			foreach $u (@users) {
				$ut += $u->{'uquota'} + $u->{'umquota'}
					if ($u->{'user'} ne $d->{'user'});
				}
			print "<td>",$self->quota_show($ut, $self->{'config_hash'}->{'home_quotas'}),"</td>\n";
			}
		}
	local @links = ();
	if (! $subdomain) {
		($self->{'dcount'}, $self->{'dmax'}) = $self->count_domains($d);
		push(@links, "<a class=menu href='domain_form.cgi?add1=1&parentuser=$d->{'user'}'>$self->{'text_hash'}->{'edit_addsubdom'}&nbsp;</a>") unless ($self->{'dmax'} && $self->{'dcount'} > $self->{'dmax'});
		}
	push(@links, sprintf("<a class=menu href='%s'>%s</a>",
			$d->{'disabled'} ? "enable_domain.cgi?dom=$d->{'id'}" : "disable_domain.cgi?dom=$d->{'id'}",
			$d->{'disabled'} ? $self->{'text_hash'}->{'edit_enabledom'} : $self->{'text_hash'}->{'edit_disabledom'}));
	push(@links, "<a class=menu href='delete_domain.cgi?dom=$d->{'id'}'>$self->{'text_hash'}->{'website_delete'}</a>") unless $d->{'user'} eq $self->{'remote_user'} && !$d->{'parent'};
	print "<td>" . join(" | ", @links) . "</td>\n";
	print "</tr>\n";
	}
print "</table>\n";
}

# userdom_name(name, &domain)
# Returns a username with the domain prefix (usually group) appended somehow
sub userdom_name
{
	my $self = shift;

if ($self->{'config_hash'}->{'append_style'} == 0) {
	return $_[0].".".$_[1]->{'prefix'};
	}
elsif ($self->{'config_hash'}->{'append_style'} == 1) {
	return $_[0]."-".$_[1]->{'prefix'};
	}
elsif ($self->{'config_hash'}->{'append_style'} == 2) {
	return $_[1]->{'prefix'}.".".$_[0];
	}
elsif ($self->{'config_hash'}->{'append_style'} == 3) {
	return $_[1]->{'prefix'}."-".$_[0];
	}
elsif ($self->{'config_hash'}->{'append_style'} == 4) {
	return $_[0]."_".$_[1]->{'prefix'};
	}
elsif ($self->{'config_hash'}->{'append_style'} == 5) {
	return $_[1]->{'prefix'}."_".$_[0];
	}
elsif ($self->{'config_hash'}->{'append_style'} == 6) {
	return $_[0]."\@".$_[1]->{'dom'};
	}
else {
	$self->error("Unknown append_style $self->{'config_hash'}->{'append_style'}!");
	}
}

# remove_userdom(name, &domain)
# Returns a username with the domain prefix (group) stripped off
sub remove_userdom
{
	my $self = shift;

return $_[0] if (!$_[1]);
local $g = $_[1]->{'prefix'};
local $d = $_[1]->{'dom'};
local $rv = $_[0];
($rv =~ s/\@(\Q$d\E)$//) || ($rv =~ s/(\.|\-|_)\Q$g\E$//) || ($rv =~ s/^\Q$g\E(\.|\-|_)//);
return $rv;
}

# too_long(name)
# Returns an error message if a username is too long for this Unix variant
sub too_long
{
	my $self = shift;

local $max = $self->max_username_length();
if ($max && length($_[0]) > $max) {
	return $self->text('user_elong', "<tt>$_[0]</tt>", $max);
	}
else {
	return undef;
	}
}

sub max_username_length
{
	my $self = shift;

$self->require_useradmin();
return $self->{'uconfig_hash'}->{'max_length'};
}

# get_default_ip()
# Returns this system's primary IP address
sub get_default_ip
{
	my $self = shift;

if ($self->{'config_hash'}->{'defip'}) {
	return $self->{'config_hash'}->{'defip'};
	}
else {
	$self->foreign_require("net", "net-lib.pl");
	local ($iface) = grep { $_->{'fullname'} eq $self->{'config_hash'}->{'iface'} }
			      $main::net_object->active_interfaces();
	return $iface->{'address'};
	}
}

# get_address_iface(address)
# Given an IP address, returns the interface name
sub get_address_iface
{
	my $self = shift;

$self->foreign_require("net", "net-lib.pl");
local ($iface) = grep { $_->{'address'} eq $_[0] } $main::net_object->active_interfaces();
return $iface ? $iface->{'fullname'} : undef;
}

# check_apache_directives([directives])
# Returns an error string if the default Apache directives don't look valid
sub check_apache_directives
{
	my $self = shift;

local ($d, $gotname, $gotdom, $gotdoc, $gotproxy);
local @dirs = split(/\t+/, defined($_[0]) ? $_[0] : $self->{'config_hash'}->{'apache_config'});
foreach $d (@dirs) {
	$d =~ s/#.*$//;
	if ($d =~ /^\s*ServerName\s+(\S+)$/i) {
		$gotname++;
		$gotdom++ if ($1 =~ /\$DOM|\$\{DOM\}/);
		}
	if ($d =~ /^\s*ServerAlias\s+(.*)$/) {
		$gotdom++ if ($1 =~ /\$DOM|\$\{DOM\}/);
		}
	$gotdoc++ if ($d =~ /^\s*DocumentRoot\s+(.*)$/);
	$gotproxy++ if ($d =~ /^\s*ProxyPass\s+(.*)$/);
	}
$gotname || return $self->{'text_hash'}->{'acheck_ename'};
$gotdom || return $self->{'text_hash'}->{'acheck_edom'};
$gotdoc || $gotproxy || return $self->{'text_hash'}->{'acheck_edoc'};
return undef;
}

# Print functions for HTML output
sub first_html_print {
	my $self = shift;
 print @_,"<br>\n"; }

sub second_html_print {
	my $self = shift;
 print @_,"<p>\n"; }

sub indent_html_print {
	my $self = shift;
 print "<ul>\n"; }

sub outdent_html_print {
	my $self = shift;
 print "</ul>\n"; }

# Print functions for text output
sub first_text_print
{
	my $self = shift;

print $self->{'indent_text'},(map { $self->entities_to_ascii($_) } @_),"\n";
}

sub second_text_print
{
	my $self = shift;

print $self->{'indent_text'},(map { $self->entities_to_ascii($_) } @_),"\n\n";
}

sub indent_text_print {
	my $self = shift;
 $self->{'indent_text'} .= "    "; }

sub outdent_text_print {
	my $self = shift;
 $self->{'indent_text'} = substr($self->{'indent_text'}, 4); }

sub null_print {
	my $self = shift;
 }

sub set_all_null_print
{
	my $self = shift;

$self->{'first_print'} = $self->{'second_print'} = $self->{'indent_print'} = $self->{'outdent_print'} = \&null_print;
}

# send_domain_email(&domain)
# Sends email to a new domain owner. Returns a pair containing a number
# (0=failed, 1=success) and an optional message. Also outputs status messages.
sub send_domain_email
{
	my $self = shift;

local $tmpl = $self->get_template($_[0]->{'template'});
local $mail = $tmpl->{'mail'};
local $subject = $tmpl->{'mail_subject'};
local $cc = $tmpl->{'mail_cc'};
return (1, undef) if ($tmpl->{'mail_on'} eq 'none');
do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_email'})};
local @erv = $self->send_template_email($mail,
				  $_[0]->{'email'} ||
				    $_[0]->{'user'}.'@'.$self->get_system_hostname(),
			    	  $_[0], $subject,
				  $cc);
if ($erv[0]) {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->text('setup_emailok', $erv[1]))};
	}
else {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->text('setup_emailfailed', $erv[1]))};
	}
}

# will_send_user_email([&domain])
# Returns 1 if a new mailbox email would be sent to a user in this domain
sub will_send_user_email
{
	my $self = shift;

local $tmode = $_[0] ? "user" : "local";
if ($self->{'config_hash'}->{$tmode.'_template'} eq 'none') {
        return 0;
        }
else {
        return 1;
        }
}

# send_user_email([&domain], &user, [to])
# Sends email to a new mailbox user. Returns a pair containing a number
# (0=failed, 1=success) and an optional message
sub send_user_email
{
	my $self = shift;

local $tmode = $_[0] ? "user" : "local";
local $subject = $_[0] ? $self->{'config_hash'}->{'newuser_subject'}
		       : $self->{'config_hash'}->{'newlocal_subject'};
local $cc = $_[0] ? $self->{'config_hash'}->{'newuser_cc'}
		  : $self->{'config_hash'}->{'newlocal_cc'};
$self->ensure_template($tmode."-template");
return (1, undef) if ($self->{'config_hash'}->{$tmode.'_template'} eq 'none');
local $tmpl = $self->{'config_hash'}->{$tmode.'_template'} eq 'default' ?
	"$self->{'module_config_directory'}/$tmode-template" :
	$self->{'config_hash'}->{$tmode.'_template'};
local %hash;
local $email;
if ($_[0]) {
	%hash = ( %{$_[0]}, %{$_[1]} );
	$hash{'mailbox'} = $self->remove_userdom($_[1]->{'user'}, $_[0]);
	$email = $hash{'mailbox'}.'@'.$hash{'dom'};
	}
else {
	%hash = ( %{$_[1]} );
	$hash{'mailbox'} = $hash{'user'};
	$email = $hash{'user'}.'@'.$self->get_system_hostname();
	}
$hash{'plainpass'} ||= "";
$self->{'user_hash'}->{'extra'} = join(" ", @{$_[1]->{'extraemail'}});
if ($_[2]) {
	$email = $_[2];
	}
$hash{'ftp'} = $_[1]->{'shell'} eq $self->{'config_hash'}->{'ftp_shell'} ||
	       $self->{'config_hash'}->{'jail_shell'} &&
		$_[1]->{'shell'} eq $self->{'config_hash'}->{'jail_shell'} ? 1 : 0;
return $self->send_template_email($self->cat_file($tmpl), $email, \%hash,
			    $subject ||
			$self->entities_to_ascii($self->{'text_hash'}->{'mail_usubject'}), $cc);
}

# ensure_template(file)
sub ensure_template
{
	my $self = shift;

$self->system_logged("cp $self->{'module_root_directory'}/$_[0] $self->{'module_config_directory'}/$_[0]")
	if (!-r "$self->{'module_config_directory'}/$_[0]");
}

# send_template_email(data, address, &substitions, subject, cc)
# Sends the given file to the specified address, with the substitions from
# a hash reference. The actual subs in the file must be like $XXX for entries
# in the hash like xxx - ie. $DOM is replaced by the domain name, and $HOME
# by the home directory
sub send_template_email
{
	my $self = shift;

local $tmpl = $_[0];
$tmpl = $self->substitute_template($tmpl, $_[2]);

# Actually send using the mailboxes module
local $subject = $self->substitute_template($_[3], $_[2]);
local $cc = $self->substitute_template($_[4], $_[2]);
$self->require_mail();
$self->foreign_require("mailboxes", "mailboxes-lib.pl");
local $mail = { 'headers' => [ [ 'From', $self->{'config_hash'}->{'from_addr'} ||
					 $main::mailboxes_object->get_from_address() ],
			       [ 'To', $_[1] ],
			       $cc ? ( [ 'Cc', $cc ] ) : ( ),
			       [ 'Subject', $subject ],
			       [ 'Content-type', 'text/plain' ] ],
		'body' => $tmpl };
$main::mailboxes_object->send_mail($mail);
return (1, $self->text('mail_ok', $_[1]));
}

# substitute_template(text, &hash)
# Given some text and a hash reference, for each ocurrance of $FOO or ${FOO} in
# the text replaces it with the value of the hash key foo
sub substitute_template
{
	my $self = shift;

# Add some extra fixed parameters to the hash
local %hash = %{$_[1]};
$hash{'hostname'} = $self->get_system_hostname();
$hash{'webmin_config'} = $self->{'config_directory'};
$hash{'webmin_etc'} = $self->{'config_directory'};
$hash{'module_config'} = $self->{'module_config_directory'};
$hash{'webmin_var'} = $self->{'var_directory'};

# Actually do the substition
local $rv = $_[0];
local $s;
foreach $s (keys %hash) {
	local $us = uc($s);
	local $sv = $hash{$s};
	$rv =~ s/\$\{\Q$us\E\}/$sv/g;
	$rv =~ s/\$\Q$us\E/$sv/g;
	if ($sv) {
		$rv =~ s/\$\{IF-\Q$us\E\}(\n?)([\000-\377]*?)\$\{ELSE-\Q$us\E\}(\n?)([\000-\377]*?)\$\{ENDIF-\Q$us\E\}(\n?)/\2/g;
		$rv =~ s/\$\{IF-\Q$us\E\}(\n?)([\000-\377]*?)\$\{ENDIF-\Q$us\E\}(\n?)/\2/g;

		$rv =~ s/\$IF-\Q$us\E(\n?)([\000-\377]*?)\$ELSE-\Q$us\E(\n?)([\000-\377]*?)\$ENDIF-\Q$us\E(\n?)/\2/g;
		$rv =~ s/\$IF-\Q$us\E(\n?)([\000-\377]*?)\$ENDIF-\Q$us\E(\n?)/\2/g;
		}
	else {
		$rv =~ s/\$\{IF-\Q$us\E\}(\n?)([\000-\377]*?)\$\{ELSE-\Q$us\E\}(\n?)([\000-\377]*?)\$\{ENDIF-\Q$us\E\}(\n?)/\4/g;
		$rv =~ s/\$\{IF-\Q$us\E\}(\n?)([\000-\377]*?)\$\{ENDIF-\Q$us\E\}(\n?)//g;

		$rv =~ s/\$IF-\Q$us\E(\n?)([\000-\377]*?)\$ELSE-\Q$us\E(\n?)([\000-\377]*?)\$ENDIF-\Q$us\E(\n?)/\4/g;
		$rv =~ s/\$IF-\Q$us\E(\n?)([\000-\377]*?)\$ENDIF-\Q$us\E(\n?)//g;
		}
	}
return $rv;
}

# userdom_substitutions(&user, &dom)
# Returns a hash reference of substitutions for a user in a domain
sub userdom_substitutions
{
	my $self = shift;

if ($_[1]) {
	$_[0]->{'mailbox'} = $self->remove_userdom($_[0]->{'user'}, $_[1]);
	$_[0]->{'dom'} = $_[1]->{'dom'};
	$_[0]->{'dom_prefix'} = substr($_[1]->{'dom'}, 0, 1);
	}
return $_[0];
}

# alias_type(string, [alias-name])
# Return the type and destination of some alias string
sub alias_type
{
	my $self = shift;

local @rv;
if ($_[0] =~ /^\|\s*$self->{'module_config_directory'}\/autoreply.pl\s+(\S+)/) {
        @rv = (5, $1);
        }
elsif ($_[0] =~ /^\|\s*$self->{'config_hash'}->{'vpopmail_auto'}\s+(\d+)\s+(\d+)\s+(\S+)\s+(\S+)(\s+(\S+)\s+(\S+))?/) {
        @rv = (12, $3, $1, $2, $4, $6, $7);
        }
elsif ($_[0] =~ /^\|\s*$self->{'module_config_directory'}\/filter.pl\s+(\S+)/) {
        @rv = (6, $1);
        }
elsif ($_[0] =~ /^\|\s*(.*)$/) {
        @rv = (4, $1);
        }
elsif ($_[0] eq "./Maildir/") {
	return (10);
	}
elsif ($_[0] eq "/dev/null") {
	return (11);
	}
elsif ($_[0] =~ /^(\/.*)$/ || $_[0] =~ /^\.\//) {
        @rv = (3, $_[0]);
        }
elsif ($_[0] =~ /^:include:(.*)$/) {
        @rv = (2, $1);
        }
elsif ($_[0] =~ /^\\(\S+)$/) {
	if ($1 eq $_[1] || $1 eq "NEWUSER") {
		return (10);
		}
	else {
		@rv = (7, $1);
		}
        }
elsif ($_[0] =~ /^\%1\@(\S+)$/) {
        @rv = (8, $1);
        }
elsif ($_[0] eq "BOUNCE") {
        @rv = (9, undef);
        }
else {
        @rv = (1, $_[0]);
        }
return wantarray ? @rv : $rv[0];
}

# set_domain_envs(&domain, action)
# Sets up VIRTUALSERVER_ environment variables for a domain update or some kind,
# prior to calling making_changes or made_changes. action must be one of
# CREATE_DOMAIN, MODIFY_DOMAIN or DELETE_DOMAIN
sub set_domain_envs
{
	my $self = shift;

local $e;
foreach $e (keys %{$self->{'ENVS_hash'}}) {
	delete($ENV{$e}) if ($e =~ /^VIRTUALSERVER_/);
	}
$ENV{'VIRTUALSERVER_ACTION'} = $_[1];
foreach $e (keys %{$_[0]}) {
	$ENV{'VIRTUALSERVER_'.uc($e)} = $_[0]->{$e};
	}
}

# making_changes()
# Called before a domain is created, modified or deleted to run the
# pre-change command
sub making_changes
{
	my $self = shift;

if ($self->{'config_hash'}->{'pre_command'} =~ /\S/) {
	$self->clean_changes_environment();
	local $out = $self->backquote_logged("($self->{'config_hash'}->{'pre_command'}) 2>&1 </dev/null");
	$self->reset_changes_environment();
	return $? ? $out : undef;
	}
return undef;
}

# made_changes()
# Called after a domain has been created, modified or deleted to run the
# post-change command
sub made_changes
{
	my $self = shift;

if ($self->{'config_hash'}->{'post_command'} =~ /\S/) {
	$self->clean_changes_environment();
	local $out = $self->backquote_logged("($self->{'config_hash'}->{'post_command'}) 2>&1 </dev/null");
	$self->reset_changes_environment();
	return $? ? $out : undef;
	}
return undef;
}

sub reset_changes_environment
{
	my $self = shift;

%ENV = %{$self->{'UNCLEAN_ENV_hash'}};
}

sub clean_changes_environment
{
	my $self = shift;

local $e;
%{$self->{'UNCLEAN_ENV_hash'}} = %ENV;
foreach $e ('SERVER_ROOT', 'SCRIPT_NAME',
	    'FOREIGN_MODULE_NAME', 'FOREIGN_ROOT_DIRECTORY',
	    'SCRIPT_FILENAME') {
	delete($ENV{$e});
	}
}

# switch_to_domain_user(&domain)
# Changes the current UID and GID to that of the domain's unix user
sub switch_to_domain_user
{
	my $self = shift;

($(, $)) = ( $_[0]->{'ugid'},
	     "$_[0]->{'ugid'} ".join(" ", $_[0]->{'ugid'},
					 $self->other_groups($_[0]->{'user'})) );
($<, $>) = ( $_[0]->{'uid'}, $_[0]->{'uid'} );
$ENV{'USER'} = $ENV{'LOGNAME'} = $_[0]->{'user'};
$ENV{'HOME'} = $_[0]->{'home'};
}

# print_subs_table(sub, ..)
sub print_subs_table
{
	my $self = shift;

print "<table>\n";
foreach $k (@_) {
	print "<tr> <td><tt><b>\${$k}</b></td>\n";
	print "<td>",$self->{'text_hash'}->{"sub_".$k},"</td> </tr>\n";
	}
print "</table>\n";
print "$self->{'text_hash'}->{'sub_if'}<p>\n";
}

# alias_form(&to, left, &domain, "user"|"alias", user|alias)
sub alias_form
{
	my $self = shift;

$self->require_mail();
local @typenames = map { $self->{'text_hash'}->{"alias_type$_"} } (0 .. 12);
$typenames[0] = "&lt;$typenames[0]&gt;";

local $left = $_[1];
local @values = @{$_[0]};
local $i;
for($i=0; $i<=@values+2; $i++) {
	print "<tr> <td>$left</td> <td>\n";
	$left = "";
	local ($type, $val) = $values[$i] ? $self->alias_type($values[$i], $_[4])
					  : (0, "");
	print "<select name=type_$i>\n";
	local $j;
	for($j=0; $j<@typenames; $j++) {
		next if ($j == 8 && $_[3] eq "user");	# to domain not valid
							# for users
		next if ($j == 10 && $_[3] ne "user");	# user's mailbox not
							# valid for aliases
		if ($j == 0 || $self->{'can_alias_types_hash'}->{$j} ||
		    $type == $j) {
			printf "<option value=$j %s>$typenames[$j]\n",
				$type == $j ? "selected" : "";
			}
		}
	print "</select>\n";
	if ($type == 7) {
		$val = $self->unescape_user($val);
		}
	print $self->ui_textbox("val_$i", $val, 30),"\n";
	if ($self->{'config_hash'}->{'edit_afiles'} || $self->master_admin()) {
		local $prog = $type == 2 ? "edit_afile.cgi" :
			      $type == 5 ? "edit_rfile.cgi" :
			      $type == 6 ? "edit_ffile.cgi" :
			      $type == 12 ? "edit_vfile.cgi" : undef;
		if ($prog && $_[2]) {
			local $di = $_[2] ? $_[2]->{'id'} : undef;
			print "<a href='$prog?dom=$di&file=$val&$_[3]=$_[4]&idx=$i'>$self->{'text_hash'}->{'alias_afile'}</a>\n";
			}
		}
	print "</td> </tr>\n";
	}
}

# parse_alias(catchall, name, &old-values, "user"|"alias", &domain)
# Returns a list of values for an alias, taken from the form generated by
# &alias_form
sub parse_alias
{
	my $self = shift;

local (@values, $i, $t, $anysame);
for($i=0; defined($t = $self->{'in_hash'}->{"type_$i"}); $i++) {
	!$t || $self->{'can_alias_types_hash'}->{$t} ||
		$self->error($self->{'text_hash'}->{'alias_etype'}." : ".$self->{'text_hash'}->{'alias_type'.$t});
	local $v = $self->{'in_hash'}->{"val_$i"};
	$v =~ s/^\s+//;
	$v =~ s/\s+$//;
	if ($t == 1 && $v !~ /^(\S+)$/) {
		$self->error($self->text('alias_etype1', $v));
		}
	elsif ($t == 3 && $v !~ /^\/(\S+)$/ && $v !~ /^\.\//) {
		$self->error($self->text('alias_etype3', $v));
		}
	elsif ($t == 4) {
		$v =~ /^(\S+)/ || $self->error($self->{'text_hash'}->{'alias_etype4none'});
		(-x $1) && $self->check_aliasfile($1, 0) ||
		   $1 eq "if" || $1 eq "export" || $self->has_command("$1") ||
			$self->error($self->text('alias_etype4', $1));
		}
	elsif ($t == 7 && !defined(getpwnam($v)) &&
	       $self->{'config_hash'}->{'mail_system'} != 4 && $self->{'config_hash'}->{'mail_system'} != 5) {
		$self->error($self->text('alias_etype7', $v));
		}
	elsif ($t == 8 && $v !~ /^[a-z0-9\.\-\_]+$/) {
		$self->error($self->text('alias_etype8', $v));
		}
	elsif ($t == 8 && !$_[0]) {
		$self->error($self->text('alias_ecatchall', $v));
		}
	if ($t == 1 || $t == 3) { push(@values, $v); }
	elsif ($t == 2) {
		$v = "$self->{'d'}->{'home'}/$v" if ($v !~ /^\//);
		push(@values, ":include:$v");
		}
	elsif ($t == 4) {
		push(@values, "|$v");
		}
	elsif ($t == 5) {
		# Setup autoreply script
		$v = "$self->{'d'}->{'home'}/$v" if ($v !~ /^\//);
		push(@values, "|$self->{'module_config_directory'}/autoreply.pl ".
			      "$v $self->{'name'}");
		$self->system_logged("cp autoreply.pl $self->{'module_config_directory'}");
		$self->system_logged("chmod 755 $self->{'module_config_directory'}/config");
		if (-d $main::sendmail_object->{'config_hash'}->{'smrsh_dir'}) {
			$self->system_logged("ln -s $self->{'module_config_directory'}/autoreply.pl $main::sendmail_object->{'config_hash'}->{'smrsh_dir'}/autoreply.pl");
			}
		}
	elsif ($t == 6) {
		# Setup filter script
		$v = "$self->{'d'}->{'home'}/$v" if ($v !~ /^\//);
		push(@values, "|$self->{'module_config_directory'}/filter.pl ".
			      "$v $self->{'name'}");
		$self->system_logged("cp filter.pl $self->{'module_config_directory'}");
		$self->system_logged("chmod 755 $self->{'module_config_directory'}/config");
		if (-d $main::sendmail_object->{'config_hash'}->{'smrsh_dir'}) {
			$self->system_logged("ln -s $self->{'module_config_directory'}/filter.pl $main::sendmail_object->{'config_hash'}->{'smrsh_dir'}/filter.pl");
			}
		}
	elsif ($t == 7) {
		push(@values, "\\".$self->escape_user($v));
		}
	elsif ($t == 8) {
		push(@values, "\%1\@$v");
		$anysame++;
		}
	elsif ($t == 9) {
		push(@values, "BOUNCE");
		}
	elsif ($t == 10) {
		push(@values, "\\$_[1]");
		}
	elsif ($t == 11) {
		push(@values, "/dev/null");
		}
	elsif ($t == 12) {
		# Setup vpopmail autoresponder script
		local @qm = getpwnam($self->{'config_hash'}->{'vpopmail_user'});
		if (!$v) {
			# Create an empty responder file
			$v = $_[3] eq "alias" ?
				"$self->{'config_hash'}->{'vpopmail_dir'}/domains/$_[4]->{'dom'}/$_[1].respond" :
				"$self->{'config_hash'}->{'vpopmail_dir'}/domains/$_[4]->{'dom'}/$_[1]/respond";
			if (!-r $v) {
				$self->open_tempfile(MSG, ">$v");
				$self->close_tempfile(MSG);
				chown($qm[2], $qm[3], $v);
				}
			}
		elsif (!$v) {
			$self->error($self->text('alias_eautorepond'));
			}
		$v = "$self->{'d'}->{'home'}/$v" if ($v !~ /^\//);
		local @av;
		if ($_[2] && $self->alias_type($_[2]->[$i]) == 12) {
			# Use old settings for delay/etc
			local @oldav = $self->alias_type($_[2]->[$i]);
			@av = ( $oldav[2], $oldav[3], $v, $oldav[4] );
			push(@av, $oldav[5]) if ($oldav[5] ne "");
			push(@av, $oldav[6]) if ($oldav[6] ne "");
			}
		else {
			# User default settings for timeouts, and create log
			# directory
			local $vdir = "$v.log";
			if (!-d $vdir) {
				mkdir($vdir, 0755);
				chown($qm[2], $qm[3], $vdir);
				}
			@av = ( 10000, 5, $v, $vdir );
			}
		push(@values, "|$self->{'config_hash'}->{'vpopmail_auto'} ".join(" ", @av));
		}
	}
if (@values > 1 && $anysame) {
	$self->error($self->text('alias_ecatchall2', $v));
	}
return @values;
}

# set_pass_change(&user)
sub set_pass_change
{
	my $self = shift;

$self->require_useradmin();
local $pft = $main::useradmin_object->passfiles_type();
if ($pft == 2 || $pft == 5 || $self->{'config_hash'}->{'ldap'}) {
	$_[0]->{'change'} = int(time() / (60*60*24));
	}
elsif ($pft == 4) {
	$_[0]->{'change'} = time();
	}
}

sub check_aliasfile
{
	my $self = shift;

return 0 if (!-r $_[0] && !$_[1]);
return 1;
}

# list_all_users()
# Returns all local and LDAP users, including those from Qmail
sub list_all_users
{
	my $self = shift;

local @rv = $main::useradmin_object->list_users();
if ($self->{'config_hash'}->{'ldap'}) {
	push(@rv, &ldap_useradmin::list_users());
	}
if ($self->{'config_hash'}->{'mail_system'} == 4) {
	local $ldap = $self->connect_qmail_ldap();
	local $rv = $ldap->search(base => $self->{'config_hash'}->{'ldap_base'},
				  filter => "(objectClass=qmailUser)");
	local $u;
	foreach $u ($rv->all_entries) {
		local %uinfo = $self->qmail_dn_to_hash($u);
		push(@rv, \%uinfo);
		}
	$ldap->unbind();
	}
return @rv;
}

# list_all_groups()
# Returns all local and LDAP groups
sub list_all_groups
{
	my $self = shift;

local @rv = $main::useradmin_object->list_groups();
if ($self->{'config_hash'}->{'ldap'}) {
	push(@rv, &ldap_useradmin::list_groups());
	}
return @rv;
}

# build_taken(&uid-taken, &username-taken, [&users])
# Fills in the the given hashes with used usernames and UIDs
sub build_taken
{
	my $self = shift;

$self->require_useradmin();

# Add Unix users
local @users = $_[2] ? @{$_[2]} : $self->list_all_users();
local $u;
foreach $u (@users) {
	$_[0]->{$u->{'uid'}} = 1;
	$_[1]->{$u->{'user'}} = 1;
	}

# Add domain users
local $d;
foreach $d ($self->list_domains()) {
	$_[0]->{$d->{'uid'}} = 1;
	$_[1]->{$d->{'user'}} = 1;
	}
}

# build_group_taken(&gid-taken, &groupname-taken, [&groups])
# Fills in the the given hashes with used group names and GIDs
sub build_group_taken
{
	my $self = shift;

$self->require_useradmin();
local @groups = $_[2] ? @{$_[2]} : $self->list_all_groups();
local $g;
foreach $g (@groups) {
	$_[0]->{$g->{'gid'}} = 1;
	$_[1]->{$g->{'group'}} = 1;
	}
local $d;
foreach $d ($self->list_domains()) {
	$_[0]->{$d->{'gid'}} = 1;
	$_[1]->{$d->{'group'}} = 1;
	}
}

# allocate_uid(&uid-taken)
sub allocate_uid
{
	my $self = shift;

local $uid = $self->{'uconfig_hash'}->{'base_uid'};
while($_[0]->{$uid}) {
	$uid++;
	}
return $uid;
}

# allocate_gid(&gid-taken)
sub allocate_gid
{
	my $self = shift;

local $gid = $self->{'uconfig_hash'}->{'base_gid'};
while($_[0]->{$gid}) {
	$gid++;
	}
return $gid;
}

# server_home_directory(&domain, [&parentdomain])
# Returns the home directory for a new virtual server user
sub server_home_directory
{
	my $self = shift;

$self->require_useradmin();
if ($_[0]->{'parent'}) {
	# Owned by some existing user, so under his home
	return "$_[1]->{'home'}/domains/$_[0]->{'dom'}";
	}
elsif ($self->{'config_hash'}->{'home_format'}) {
	# Use the template from the module config
	local $home = "$self->{'home_base'}/$self->{'config_hash'}->{'home_format'}";
	return $self->substitute_template($home, $_[0]);
	}
else {
	# Just use the Users and Groups module settings
	return $main::useradmin_object->auto_home_dir($self->{'home_base'}, $_[0]->{'user'},
						     $_[0]->{'ugroup'});
	}
}

# set_quota(user, filesystem, quota)
sub set_quota
{
	my $self = shift;

$self->require_useradmin();
if ($self->{'config_hash'}->{'hard_quotas'}) {
	$main::quota_object->edit_user_quota($_[0], $_[1],
				int($_[2]), int($_[2]), 0, 0);
	}
else {
	$main::quota_object->edit_user_quota($_[0], $_[1],
				int($_[2]), 0, 0, 0);
	}
}

# set_server_quotas(&domain)
# Set the user and possibly group quotas for a domain
sub set_server_quotas
{
	my $self = shift;

if ($self->{'config_hash'}->{'home_quotas'}) {
	$self->set_quota($_[0]->{'user'}, $self->{'config_hash'}->{'home_quotas'}, $_[0]->{'uquota'});
	}
if ($self->{'config_hash'}->{'mail_quotas'} &&
    $self->{'config_hash'}->{'mail_quotas'} ne $self->{'config_hash'}->{'home_quotas'}) {
	$self->set_quota($_[0]->{'user'}, $self->{'config_hash'}->{'mail_quotas'}, $_[0]->{'uquota'});
	}
if ($self->{'config_hash'}->{'group_quotas'} && $_[0]->{'group'}) {
	$self->require_useradmin();
	if ($self->{'config_hash'}->{'hard_quotas'}) {
		$main::quota_object->edit_group_quota(
			$_[0]->{'group'}, $self->{'config_hash'}->{'home_quotas'},
			int($_[0]->{'quota'}), int($_[0]->{'quota'}), 0, 0);
		}
	else {
		$main::quota_object->edit_group_quota(
			$_[0]->{'group'}, $self->{'config_hash'}->{'home_quotas'},
			int($_[0]->{'quota'}), 0, 0, 0);
		}
	}
}

# users_table(&users, &dom)
# Output a table of mailbox users
sub users_table
{
	my $self = shift;

local $can_quotas = $self->{'config_hash'}->{'home_quotas'} || $self->{'config_hash'}->{'mail_quotas'};
local $can_qquotas = $self->{'config_hash'}->{'mail_system'} == 4 || $self->{'config_hash'}->{'mail_system'} == 5;
local ($samb_t, $mail_t, $ftp_t, $user_t);

	if ($self->{'d'}->{'samba'}) {$samb_t = $self->{'text_hash'}->{'samba_login'}};
	if ($self->{'d'}->{'mail'}) {$mail_t = $self->{'text_hash'}->{'mail_login'}};
	if ($self->{'config_hash'}->{'ftp'} || $self->{'d'}->{'web'}) {$ftp_t = $self->{'text_hash'}->{'ftp_login'}};
		$user_t = "$mail_t/$ftp_t/$samb_t";
		$user_t =~ s/\/*$//s;
		$user_t =~ s/\/\//\//s;
		$user_t =~ s/^\/*//s;

print "<table border width=100%>\n";
print "<tr $self->{'tb'}> <td><b>$self->{'text_hash'}->{'users_name'}</b></td> ",
      "<td><b>".$self->text('users_pop3', $user_t)."</b></td> ",
      "<td><b>$self->{'text_hash'}->{'users_real'}</b></td> ",
      $can_quotas ?
	  "<td><b>$self->{'text_hash'}->{'users_quota'}</b></td> " : "",
      $can_qquotas ?
	  "<td><b>$self->{'text_hash'}->{'users_qquota'}</b></td> " : "",
      "<td><b>$self->{'text_hash'}->{'users_size'}</b></td> ",
      "<td><b>$self->{'text_hash'}->{'users_ftp'}</b></td> ";
print "<td><b>$self->{'text_hash'}->{'users_samba'}</b></td> " if $_[1]->{'samba'};
print "<td><b>$self->{'text_hash'}->{'users_act'}</b></td></tr>\n";
local $u;
local $did = $_[1] ? $_[1]->{'id'} : 0;
foreach $u (@{$_[0]}) {
	local $pop3 = $_[1] ? $self->remove_userdom($u->{'user'}, $_[1])
			    : $u->{'user'};
	local $domuser = $_[1] && $u->{'user'} eq $_[1]->{'user'} &&
			 $u->{'unix'};
	print "<tr $self->{'cb'}>\n";
	print "<td><a class=user href='edit_user.cgi?dom=$did&",
	      "user=$u->{'user'}&unix=$u->{'unix'}'>",
	      ($domuser ? "<b>$pop3</b>" : $pop3),"</a></td>\n";
	print "<td>$u->{'user'}</td>\n";
	print "<td>",($u->{'real'} || "<br>"),"</td>\n";

	# Disk quotas
	local $quota;
	$quota += $u->{'quota'} if ($self->{'config_hash'}->{'home_quotas'});
	$quota += $u->{'mquota'} if ($self->{'config_hash'}->{'mail_quotas'} &&
		      $self->{'config_hash'}->{'home_quotas'} ne $self->{'config_hash'}->{'mail_quotas'});
	if (defined($quota)) {
		print "<td>",$quota ? $self->quota_show($quota,$self->{'config_hash'}->{'home_quotas'})
				    : $self->{'text_hash'}->{'form_unlimit'},"</td>\n";
		}
	if ($u->{'mailquota'}) {
		print "<td>",$u->{'qquota'} ? $self->nice_size($u->{'qquota'}) :
			     $self->{'text_hash'}->{'form_unlimit'},"<br>","</td>\n";
		}
	elsif ($can_qquotas) {
		print "<td><br></td>\n";
		}

	printf "<td>%s</td>\n",
		$u->{email} ? "<img src='images/yes.gif' border=0 width=12 height=12>" : "<img src='images/no.gif' border=0 width=12 height=12>";
	printf "<td>%s</td>\n",
		$domuser ? "<img src='images/yes.gif' border=0 width=12 height=12> $self->{'text_hash'}->{'users_main'}" :
		!$u->{'unix'} && !$u->{'shell'} ? $self->{'text_hash'}->{'users_qmail'} :
		$u->{'shell'} eq $self->{'config_hash'}->{'ftp_shell'} ? "<img src='images/yes.gif' border=0 width=12 height=12>" :
		$self->{'config_hash'}->{'jail_shell'} &&
		 $u->{'shell'} eq $self->{'config_hash'}->{'jail_shell'} ? $self->{'text_hash'}->{'users_jail'} :
		$u->{'shell'} eq $self->{'config_hash'}->{'shell'} ? "<img src='images/no.gif' border=0 width=12 height=12>" :
			$self->text('users_shell', "<tt>$u->{'shell'}</tt>");
	if ($_[1]->{'samba'})
	{
		printf "<td>%s</td>\n",
			$u->{'samba'} ? "<img src='images/yes.gif' border=0 width=12 height=12>" : "<img src='images/no.gif' border=0 width=12 height=12>";
	}
	print "<td>",$domuser ? "&nbsp;": "<a class=menu href='/virtual_server/save_user.cgi?dom=$did&old=$u->{'user'}&delete=1&unix=$u->{'unix'}'>$self->{'text_hash'}->{'users_del'}</a>" ,"</td></tr>\n";
	}
print "</table>\n";
}

# quota_bsize(filesystem)
sub quota_bsize
{
	my $self = shift;

$self->require_useradmin();
if ($main::quota_object->UNIVERSAL::can("block_size")) {
	local $bsize;
	if (!exists($self->{'bsize_cache_hash'}->{$_[0]})) {
		$self->{'bsize_cache_hash'}->{$_[0]} = $main::quota_object->block_size($_[0]);
		}
	return $self->{'bsize_cache_hash'}->{$_[0]};
	}
return undef;
}

# quota_show(number, filesystem)
# Returns text for the quota on some filesystem, in a human-readable format
sub quota_show
{
	my $self = shift;

local $bsize = $self->quota_bsize($_[1]);
if ($bsize) {
	local $value = sprintf('%0.1f',$_[0]*$bsize/1024/1024);
	return $value." ".$self->{'text_hash'}->{'form_mb'};
	}
return $_[0]." ".$self->{'text_hash'}->{'form_b'};
}

# quota_input(name, number, filesystem)
# Returns HTML for an input for entering a quota, doing block->kb conversion
sub quota_input
{
	my $self = shift;

local $bsize = $self->quota_bsize($_[2]);
local $value = !$_[1] ? '' : $bsize ? sprintf ('%0.1f', $_[1]*$bsize/1024/1024) : sprintf ('%0.1f', $_[1]) ;
return sprintf "<input name=%s size=10 value=$value> %s",
	$_[0], $bsize ? $self->{'text_hash'}->{'form_mb'} : $self->{'text_hash'}->{'form_b'};
}

# quota_parse(name, filesystem)
# Converts an entered quota into blocks
sub quota_parse
{
	my $self = shift;

local $bsize = $self->quota_bsize($_[1]);
return $bsize ? int($self->{'in_hash'}->{$_[0]}*1024*1024/$bsize) : $self->{'in_hash'}->{$_[0]};
}

# setup_virt(&domain)
# Bring up an interface for a domain, if the IP isn't already enabled
sub setup_virt
{
	my $self = shift;

$self->foreign_require("net", "net-lib.pl");
local @boot = $main::net_object->active_interfaces();
if (!$self->{'config_hash'}->{'iface_manual'}) {
    do { my $fn = $self->{'first_print'}; $self->$fn($self->text('setup_virt', $_[0]->{'ip'}))};
    local $iface_name;
    if ($_[0]->{'iface'}) {
	$_[0]->{'iface'} =~/^([a-z0-9]+):?.*$/;
        $_[0]->{'iface'}=$iface_name=$1;
    }
    $iface_name = $self->{'config_hash'}->{'iface'} unless $iface_name;

    local ($iface) = grep { $_->{'fullname'} eq $iface_name} @boot;
    
	local $b;
	local $vmax = $self->{'config_hash'}->{'iface_base'} || int($main::net_object->{'min_virtual_number'});
	foreach $b (@boot) {
		$vmax = $b->{'virtual'}
			if ($b->{'name'} eq $iface->{'name'} &&
			    $b->{'virtual'} > $vmax);
		}
	local $virt = { 'address' => $_[0]->{'ip'},
		'netmask' => $_[0]->{'netmask'} ? $_[0]->{'netmask'} :
			$main::net_object->{'virtual_netmask'} || $iface->{'netmask'},
		'broadcast' => $_[0]->{'bcast'} ? $_[0]->{'bcast'} :
			$main::net_object->{'virtual_netmask'} eq "255.255.255.255" ? $_[0]->{'ip'} : $iface->{'broadcast'},
		'name' => $_[0]->{'iface'} ? $_[0]->{'iface'} : $iface->{'name'},
			'virtual' => $vmax+1,
			'up' => 1 };
	$virt->{'fullname'} = $virt->{'name'}.":".$virt->{'virtual'};
	$main::net_object->save_interface($virt);
	$main::net_object->activate_interface($virt);
	$_[0]->{'iface'} = $virt->{'fullname'};
	$_[0]->{'netmask'} = $virt->{'netmask'};
	$_[0]->{'bcast'} = $virt->{'broadcast'};

	do { my $fn = $self->{'second_print'}; $self->$fn($self->text('setup_virtdone', $_[0]->{'iface'}))};
	}
else {
        # Just guess the interface
        local ($virt) = grep { $_->{'address'} eq $_[0]->{'ip'} } @boot;
        $_[0]->{'iface'} = $virt ? $virt->{'fullname'} : undef;
        }
}

# delete_virt(&domain)
# Take down the network interface for a domain
sub delete_virt
{
	my $self = shift;

if (!$self->{'config_hash'}->{'iface_manual'}) {
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'delete_virt'})};
	$self->foreign_require("net", "net-lib.pl");
	local ($biface) = grep { $_->{'address'} eq $_[0]->{'ip'} }
			       $main::net_object->boot_interfaces();
	local ($aiface) = grep { $_->{'address'} eq $_[0]->{'ip'} }
			       $main::net_object->active_interfaces();
	if ($biface->{'virtual'} ne '') {
		$main::net_object->delete_interface($biface);
		$main::net_object->deactivate_interface($aiface)
			if ($aiface && $aiface->{'virtual'} ne '');
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	else {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->text('delete_novirt', $self->{'d'}->{'iface'}))};
		}
	}
delete($_[0]->{'iface'});
}

# check_virt_clash(ip)
# Returns the interface if some IP is already in use
sub check_virt_clash
{
	my $self = shift;

return undef if ($self->{'config_hash'}->{'iface_manual'});      # no clash for manual mode
$self->foreign_require("net", "net-lib.pl");
local @boot = $main::net_object->boot_interfaces();
local ($boot) = grep { $_->{'address'} eq $_[0] } @boot;
local @active = $main::net_object->active_interfaces();
local ($active) = grep { $_->{'address'} eq $_[0] } @active;
return $active || $boot;
}

# backup_domains(file, &domains, &features, dir-format, skip-errors, &options)
# Perform a backup of one or more domains into a single tar.gz file. Returns
# undef on success, or an error message.
sub backup_domains
{
	my $self = shift;

# Create a temp dir for the backup, to be tarred up later
local $backupdir = $self->transname();
mkdir($backupdir, 0700);

# Go through all the domains, and for each feature call the backup function
# to add it to the backup directory
local $d;
local $ok = 1;
local @donedoms;
local ($okcount, $errcount) = (0, 0);
DOMAIN: foreach $d (@{$_[1]}) {
	do { my $fn = $self->{'first_print'}; $self->$fn($self->text('backup_fordomain', $d->{'dom'}))};
	do { my $fn = $self->{'second_print'}; $self->$fn()};
	do { my $fn = $self->{'indent_print'}; $self->$fn()};
	local $f;
	local $dok = 1;
	foreach $f (@{$_[2]}) {
		$self->{'bfunc'} = "backup_$f";
		if ($self->UNIVERSAL::can($self->{'bfunc'}) &&
		    ($d->{$f} || $f eq "virtualmin")) {
			local $ffile = "$backupdir/$d->{'dom'}_$f";
			local $fok = do { my $fn = $self->{'bfunc'}; $self->$fn($d, $ffile, $_[5]->{$f})};
			$dok = 0 if (!$fok);
			if (!$fok && !$_[4]) {
				$ok = 0;
				$errcount++;
				last DOMAIN;
				}
			push(@donedoms, $d);
			}
		}
	do { my $fn = $self->{'outdent_print'}; $self->$fn()};
	if ($dok) { $okcount++; }
	else { $errcount++; }
	}

# Work out where to write the final tar files to
local ($mode, $user, $pass, $server, $path) = $self->parse_backup_url($_[0]);
local ($dest, @destfiles);
if ($mode >= 1) {
	# Write archive to temporary file/dir first, for later upload
	$dest = $self->transname();
	}
else {
	$dest = $path;
	}

if ($ok) {
	local $out;
	if ($_[3]) {
		# Create one tar file in the destination for each domain
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'backup_final2'})};
		mkdir($dest, 0755);
		foreach $d ($self->unique(@donedoms)) {
			if ($self->has_command("gzip")) {
				$out = `cd $backupdir ; (tar cf - $d->{'dom'}_* | gzip -c) 2>&1 >$dest/$d->{'dom'}.tar.gz`;
				push(@destfiles, "$d->{'dom'}.tar.gz");
				}
			else {
				$out = `cd $backupdir ; tar cf $dest/$d->{'dom'}.tar $d->{'dom'}_* 2>&1`;
				push(@destfiles, "$d->{'dom'}.tar");
				}
			if ($?) {
				do { my $fn = $self->{'second_print'}; $self->$fn($self->text('backup_finalfailed', "<pre>$out</pre>"))};
				$ok = 0;
				last;
				}
			}
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'}) if ($ok)};
		}
	else {
		# Tar up the directory into the final file
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'backup_final'})};
		if ($self->has_command("gzip")) {
			$out = `cd $backupdir ; (tar cf - . | gzip -c) 2>&1 >$dest`;
			}
		else {
			$out = `cd $backupdir ; tar cf $dest . 2>&1`;
			}
		if ($?) {
			do { my $fn = $self->{'second_print'}; $self->$fn($self->text('backup_finalfailed', "<pre>$out</pre>"))};
			$ok = 0;
			}
		else {
			do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
			}
		}
	}

system("rm -rf ".quotemeta($backupdir));
local $sz = $_[3] ? $self->disk_usage_kb($dest)*1024
		  : (@st=stat($dest))[7];

if ($ok && $mode == 1) {
	# Upload file(s) to FTP server
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'backup_upload'})};
	local $err;
	if ($_[3]) {
		# Need to upload entire directory .. which has to be created
		local $mkdirerr;
		$self->ftp_onecommand($server, "MKD $path", \$mkdirerr, $user, $pass);
		foreach $df (@destfiles) {
			$self->ftp_upload($server, "$path/$df", "$dest/$df", \$err,
				    undef, $user, $pass);
			if ($err) {
				do { my $fn = $self->{'second_print'}; $self->$fn(
					$self->text('backup_uploadfailed', $err))};
				$ok = 0;
				last;
				}
			}
		}
	else {
		# Just a single file
		$self->ftp_upload($server, $path, $dest, \$err, undef, $user, $pass);
		if ($err) {
			do { my $fn = $self->{'second_print'}; $self->$fn($self->text('backup_uploadfailed', $err))};
			$ok = 0;
			}
		}
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'}) if ($ok)};
	}
elsif ($ok && $mode == 2) {
	# Upload to SSH server with scp
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'backup_upload2'})};
	local $err;
	local $r = ($user ? "$user\@" : "")."$server:$path";
	if ($_[3]) {
		# Need to upload entire directory
		$self->scp_copy("$dest/*", $r, $pass, \$err);
		if ($err) {
			$err = undef;
			$self->scp_copy($dest, $r, $pass, \$err);
			}
		}
	else {
		# Just a single file
		$self->scp_copy($dest, $r, $pass, \$err);
		}
	if ($err) {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->text('backup_uploadfailed', $err))};
		$ok = 0;
		}
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'}) if ($ok)};
	}

if ($mode >= 1) {
	# Always delete the temporary destination
	system("rm -rf ".quotemeta($dest));
	}

# Show some status
if ($ok) {
	do { my $fn = $self->{'first_print'}; $self->$fn($self->text('backup_finalstatus', $okcount, $errcount))};
	}

return ($ok, $sz);
}

# backup_virtualmin(&domain, file)
# Adds a domain's configuration file to the backup
sub backup_virtualmin
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'backup_virtualmincp'})};
system("cp ".quotemeta($_[0]->{'file'})." ".$_[1]);
do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
return 1;
}

# restore_domains(file, &domains, &features, &options)
# Restore multiple domains from the given file
sub restore_domains
{
	my $self = shift;

# Work out where the backup is located
local $ok = 1;
local $backup;
local ($mode, $user, $pass, $server, $path) = $self->parse_backup_url($_[0]);
if ($mode > 0) {
	# Need to download to temp file/directory first
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_download'})};
	$backup = $self->transname();
	local $derr = $self->download_backup($_[0], $backup);
	if ($derr) {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->text('restore_downloadfailed', $derr))};
		$ok = 0;
		}
	else {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
		}
	}
else {
	$backup = $_[0];
	}

local $restoredir;
if ($ok) {
	# Create a temp dir for the backup archive contents
	$restoredir = $self->transname();
	mkdir($restoredir, 0700);

	local @files;
	if (-d $backup) {
		# Extracting a directory of backup files
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_first2'})};
		opendir(DIR, $backup);
		@files = map { "$backup/$_" }
			     grep { $_ ne "." && $_ ne ".." } readdir(DIR);
		closedir(DIR);
		}
	else {
		# Extracting one backup file
		do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_first'})};
		@files = ( $backup );
		}

	# Extract each of the files
	local $f;
	foreach $f (@files) {
		open(BACKUP, $f);
		local $two;
		read(BACKUP, $two, 2);
		close(BACKUP);
		local $out;
		local $q = quotemeta($f);
		if ($two eq "\037\213") {
			# Assume gzipped tar
			$out = `cd '$restoredir' ; (gunzip -c $q | tar xf -) 2>&1`;
			}
		else {
			# Assume normal tar
			$out = `cd '$restoredir' ; tar xf $q 2>&1`;
			}
		if ($?) {
			do { my $fn = $self->{'second_print'}; $self->$fn($self->text('restore_firstfailed', "<tt>$f</tt>", "<pre>$out</pre>"))};
			$ok = 0;
			last;
			}
		}
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'}) if ($ok)};
	}

if ($ok) {
	# Fill in missing domain details
	foreach $d (@{$_[1]}) {
		if ($d->{'missing'}) {
			$d = $self->get_domain(undef,
				"$restoredir/$d->{'dom'}_virtualmin");
			$d->{'missing'} = 1;
			}
		}

	# Now restore each of the domain/feature files
	local $d;
	DOMAIN: foreach $d (sort { $a->{'parent'} <=> $b->{'parent'} } @{$_[1]}) {
		if ($d->{'missing'}) {
			# This domain doesn't exist yet - need to re-create it
			do { my $fn = $self->{'first_print'}; $self->$fn($self->text('restore_createdomain',
				      $d->{'dom'}))};
			local $cerr = $self->virtual_server_clashes($d);
			if ($cerr) {
				do { my $fn = $self->{'second_print'}; $self->$fn($self->text('restore_eclash', $cerr))};
				$ok = 0;
				last DOMAIN;
				}
			local ($parentdom, $parentuser);
			if ($d->{'parent'}) {
				# Does the parent exist?
				$parentdom = $self->get_domain($d->{'parent'});
				if (!$parentdom) {
					do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'restore_epar'})};
					$ok = 0;
					last DOMAIN;
					}
				$parentuser = $parentdom->{'user'};
				}

			if ($parentdom) {
				# UID and GID always come from parent
				$d->{'uid'} = $parentdom->{'uid'};
				$d->{'gid'} = $parentdom->{'gid'};
				$d->{'ugid'} = $parentdom->{'ugid'};
				}
			elsif ($_[3]->{'reuid'}) {
				# Re-allocate the UID and GID
				local ($samegid) =($d->{'gid'} == $d->{'ugid'});
				local (%gtaken, %taken);
				$self->build_group_taken(\%gtaken);
				$d->{'gid'} = $self->allocate_gid(\%gtaken);
				$d->{'ugid'} = $d->{'gid'};
				$self->build_taken(\%taken);
				$d->{'uid'} = $self->allocate_uid(\%taken);
				if (!$samegid) {
					# Old ugid was custom, so set from old
					# group name
					local @ginfo = getgrnam($d->{'ugroup'});
					if (@ginfo) {
						$d->{'ugid'} = $ginfo[2];
						}
					}
				}

			# Create the domain, fixing the IP if needed
			do { my $fn = $self->{'indent_print'}; $self->$fn()};
			delete($d->{'missing'});
 			if ($d->{'alias'}) {
 				local $alias = $self->get_domain($d->{'alias'});
 				$d->{'ip'} = $alias->{'ip'};
 				}
# 			elsif (!$d->{'virt'} && !$config{'all_namevirtual'}) {
# 				$d->{'ip'} = &get_default_ip();
# 				}
			$self->create_virtual_server($d, $parentdom, $parentuser);
			do { my $fn = $self->{'outdent_print'}; $self->$fn()};
			}

		do { my $fn = $self->{'first_print'}; $self->$fn($self->text('restore_fordomain', $d->{'dom'}))};
		do { my $fn = $self->{'indent_print'}; $self->$fn()};
		local $f;
		foreach $f (@{$_[2]}) {
			# Restore features
			local $rfunc = "restore_$f";
			if ($self->can($rfunc) &&
			    ($d->{$f} || $f eq "virtualmin")) {
				local $ffile = "$restoredir/$d->{'dom'}_$f";
				if (-r $ffile) {
					local $fok = $self->$rfunc($d, $ffile,
						     $_[3]->{$f}, $_[3]);
					if (!$fok) {
						$ok = 0;
						do { my $fn = $self->{'outdent_print'}; $self->$fn()};
						last DOMAIN;
						}
					}
				}
			}
		$self->save_domain($d);

		# Re-setup Webmin user
		$self->modify_webmin($d, $d);
		do { my $fn = $self->{'outdent_print'}; $self->$fn()};
		}
	}

system("rm -rf ".quotemeta($restoredir));
if ($mode > 0) {
	# Clean up downloaded file
	system("rm -rf ".quotemeta($backup));
	}
return $ok;
}

# $self->backup_contents(file)
# Returns a hash ref of domains and features in a backup file, or an error
# string if it is invalid
sub backup_contents
{
	my $self = shift;

local $backup;
local ($mode, $user, $pass, $server, $path) = $self->parse_backup_url($_[0]);
if ($mode > 0) {
	# Need to download to temp file first
	$backup = $self->transname();
	local $derr = $self->download_backup($_[0], $backup);
	return $derr if ($derr);
	}
else {
	$backup = $_[0];
	}

if (-d $backup) {
	# A directory of backup files, one per domain
	opendir(DIR, $backup);
	local $f;
	local %rv;
	foreach $f (readdir(DIR)) {
		next if ($f eq "." || $f eq "..");
		local $cont = $self->backup_contents("$backup/$f");
		if (ref($cont)) {
			local $d;
			foreach $d (keys %$cont) {
				if ($rv{$d}) {
					$self->clean_contents_temp();
					return $self->text('restore_edup', $d);
					}
				else {
					$rv{$d} = $cont->{$d};
					}
				}
			}
		else {
			$self->clean_contents_temp();
			return $backup."/".$f." : ".$cont;
			}
		}
	closedir(DIR);
	$self->clean_contents_temp();
	return \%rv;
	}
else {
	# A single file
	local $err;
	open(BACKUP, $backup);
	local $two;
	read(BACKUP, $two, 2);
	close(BACKUP);
	local $out;
	local $q = quotemeta($backup);
	if ($two eq "\037\213") {
		# Assume gzipped tar
		$out = `(gunzip -c $q | tar tf -) 2>&1`;
		}
	else {
		# Assume normal tar
		$out = `tar tf $q 2>&1`;
		}
	if ($?) {
		$self->clean_contents_temp();
		return $self->{'text_hash'}->{'restore_etar'};
		}
	local ($l, %rv, %done);
	foreach $l (split(/\n/, $out)) {
		if ($l =~ /^(.\/)?([^_]+)_([a-z0-9]+)$/) {
			push(@{$rv{$2}}, $3) if (!$done{$2,$3}++);
			}
		}
	$self->clean_contents_temp();
	return \%rv;
	}

	sub clean_contents_temp
	{
	system("rm -rf ".quotemeta($backup)) if ($mode > 0);
	}
}


# download_backup(url, tempfile)
# Downloads a backup file or directory to a local temp file or directory.
# Returns undef on success, or an error message.
sub download_backup
{
	my $self = shift;

local ($mode, $user, $pass, $server, $path) = $self->parse_backup_url($_[0]);
if ($mode == 1) {
	# Download from FTP server
	local $cwderr;
	local $isdir = $self->ftp_onecommand($server, "CWD $path", \$cwderr,
				       $user, $pass);
	local $err;
	if ($isdir) {
		# Need to download entire directory
		mkdir($_[1], 0700);
		local $list = $self->ftp_listdir($server, $path, \$err, $user, $pass);
		return $err if (!$list);
		foreach $f (@$list) {
			$f =~ s/^$path[\\\/]//; #/
			$self->ftp_download($server, "$path/$f", "$_[1]/$f", \$err,
				      undef, $user, $pass);
			return $err if ($err);
			}
		return undef;
		}
	else {
		# Can just download a single file
		$self->ftp_download($server, $path, $_[1], \$err,
			      undef, $user, $pass);
		return $err;
		}
	}
elsif ($mode == 2) {
	# Download from SSH server
	$self->scp_copy(($user ? "$user\@" : "")."$server:$path",
		  $_[1], $pass, \$err);
	return $err;
	}
}

# restore_virtualmin(&domain, file)
# Restore the settings for a domain, such as quota, password and so on
sub restore_virtualmin
{
	my $self = shift;

do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'restore_virtualmincp'})};
local %oldd;
$self->read_file($_[1], \%oldd);
$_[0]->{'quota'} = $oldd{'quota'};
$_[0]->{'uquota'} = $oldd{'uquota'};
$_[0]->{'pass'} = $oldd{'pass'};
$_[0]->{'email'} = $oldd{'email'};
$_[0]->{'mailboxlimit'} = $oldd{'mailboxlimit'};
$_[0]->{'owner'} = $oldd{'owner'};
$self->save_domain($_[0]);

do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
return 1;
}

# backup_strftime(path)
# Replaces stftime-style % codes in a path with the current time
sub backup_strftime
{
	my $self = shift;

eval "use POSIX";
eval "use posix" if ($@);
local @tm = localtime(time());
return strftime($_[0], @tm);
}

# parse_backup_url(string)
# Converts a URL like ftp:// or a filename into its components
sub parse_backup_url
{
	my $self = shift;

if ($_[0] =~ /^ftp:\/\/([^:]*):([^\@]*)\@([^\/]+)(\/.*)$/) {
	return (1, $1, $2, $3, $4);
	}
elsif ($_[0] =~ /^ssh:\/\/([^:]*):([^\@]*)\@([^\/]+)(\/.*)$/) {
	return (2, $1, $2, $3, $4);
	}
else {
	return (0, undef, undef, undef, $_[0]);
	}
}

# show_backup_destination(name, value)
# Returns HTML for a field for selecting a local or FTP file
sub show_backup_destination
{
	my $self = shift;

local ($mode, $user, $pass, $server, $path) = $self->parse_backup_url($_[1]);
local $rv;

# Local file field
$rv .= "<table cellpadding=1 cellspacing=0>";
$rv .= sprintf "<tr> <td><input type=radio name=$_[0]_mode value=0 %s></td>\n",
	$mode == 0 ? "checked" : "";
$rv .= sprintf "<td colspan=2>%s <input name=$_[0]_file size=40 value='%s'> %s</td> </tr>\n",
	$self->{'text_hash'}->{'backup_mode0'}, $mode == 0 ? $path : "",
	$self->file_chooser_button("$_[0]_file");

# FTP file fields
$rv .= sprintf "<tr> <td><input type=radio name=$_[0]_mode value=1 %s></td>\n",
	$mode == 1 ? "checked" : "";
$rv .= sprintf "<td>%s <input name=$_[0]_server size=20 value='%s'></td>\n",
	$self->{'text_hash'}->{'backup_mode1'}, $mode == 1 ? $server : undef;
$rv .= sprintf "<td>%s <input name=$_[0]_path size=20 value='%s'></td> </tr>\n",
	$self->{'text_hash'}->{'backup_path'}, $mode == 1 ? $path : undef;
$rv .= "<tr> <td></td>\n";
$rv .= sprintf "<td>%s <input name=$_[0]_user size=15 value='%s'></td>\n",
	$self->{'text_hash'}->{'backup_login'}, $mode == 1 ? $user : undef;
$rv .= sprintf "<td>%s <input type=password name=$_[0]_pass size=15 value='%s'></td> </tr>\n",
	$self->{'text_hash'}->{'backup_pass'}, $mode == 1 ? $pass : undef;

# SCP file fields
$rv .= sprintf "<tr> <td><input type=radio name=$_[0]_mode value=2 %s></td>\n",
	$mode == 2 ? "checked" : "";
$rv .= sprintf "<td>%s <input name=$_[0]_sserver size=20 value='%s'></td>\n",
	$self->{'text_hash'}->{'backup_mode2'}, $mode == 2 ? $server : undef;
$rv .= sprintf "<td>%s <input name=$_[0]_spath size=20 value='%s'></td> </tr>\n",
	$self->{'text_hash'}->{'backup_path'}, $mode == 2 ? $path : undef;
$rv .= "<tr> <td></td>\n";
$rv .= sprintf "<td>%s <input name=$_[0]_suser size=15 value='%s'></td>\n",
	$self->{'text_hash'}->{'backup_login'}, $mode == 2 ? $user : undef;
$rv .= sprintf "<td>%s <input type=password name=$_[0]_spass size=15 value='%s'></td> </tr>\n",
	$self->{'text_hash'}->{'backup_pass'}, $mode == 2 ? $pass : undef;

$rv .= "</table>\n";
return $rv;
}

# parse_backup_destination(name, &in)
# Returns a backup destination string, or calls error
sub parse_backup_destination
{
	my $self = shift;

local %in = %{$_[1]};
local $mode = $in{"$_[0]_mode"};
if ($mode == 0) {
	$in{"$_[0]_file"} =~ /^\/\S/ || $self->error($self->{'text_hash'}->{'backup_edest'});
	return $in{"$_[0]_file"};
	}
elsif ($mode == 1) {
	gethostbyname($in{"$_[0]_server"}) || $self->error($self->{'text_hash'}->{'backup_eserver1'});
	$in{"$_[0]_path"} =~ /^\/\S/ || $self->error($self->{'text_hash'}->{'backup_epath'});
	$in{"$_[0]_user"} =~ /^[^:\@\/]*$/ || $self->error($self->{'text_hash'}->{'backup_euser'});
	$in{"$_[0]_pass"} =~ /^[^:\@\/]*$/ || $self->error($self->{'text_hash'}->{'backup_epass'});
	return "ftp://".$in{"$_[0]_user"}.":".$in{"$_[0]_pass"}."\@".
	       $in{"$_[0]_server"}.$in{"$_[0]_path"};
	}
elsif ($mode == 2) {
	gethostbyname($in{"$_[0]_sserver"}) || $self->error($self->{'text_hash'}->{'backup_eserver2'});
	$in{"$_[0]_spath"} =~ /^\/\S/ || $self->error($self->{'text_hash'}->{'backup_epath'});
	$in{"$_[0]_suser"} =~ /^[^:\@\/]*$/ || $self->error($self->{'text_hash'}->{'backup_euser'});
	return "ssh://".$in{"$_[0]_suser"}.":".$in{"$_[0]_spass"}."\@".
	       $in{"$_[0]_sserver"}.$in{"$_[0]_spath"};
	}
}

# ftp_upload(host, file, srcfile, [&error], [&callback], [user, pass])
# Download data from a local file to an FTP site
sub ftp_upload
{
	my $self = shift;

local($buf, @n);
local $cbfunc = $_[4];

$self->{'download_timed_out'} = undef;
local $SIG{ALRM} = "download_timeout";
alarm(60);

# connect to host and login
$self->open_socket($_[0], 21, "SOCK", $_[3]) || return 0;
alarm(0);
if ($self->{'download_timed_out'}) {
	if ($_[3]) { ${$_[3]} = $self->{'download_timed_out'}; return 0; }
	else { $self->error($self->{'download_timed_out'}); }
	}
$self->ftp_command("", 2, $_[3]) || return 0;
if ($_[5]) {
	# Login as supplied user
	local @urv = $self->ftp_command("USER $_[5]", [ 2, 3 ], $_[3]);
	@urv || return 0;
	if (int($urv[1]/100) == 3) {
		$self->ftp_command("PASS $_[6]", 2, $_[3]) || return 0;
		}
	}
else {
	# Login as anonymous
	local @urv = $self->ftp_command("USER anonymous", [ 2, 3 ], $_[3]);
	@urv || return 0;
	if (int($urv[1]/100) == 3) {
		$self->ftp_command("PASS root\@".$self->get_system_hostname(), 2,
			     $_[3]) || return 0;
		}
	}
$self->$cbfunc(1, 0) if ($cbfunc);

$self->ftp_command("TYPE I", 2, $_[3]) || return 0;

# get the file size and tell the callback
local @st = stat($_[2]);
if ($cbfunc) {
	$self->$cbfunc(2, $st[7]);
	}

# send the file
local $pasv = $self->ftp_command("PASV", 2, $_[3]);
defined($pasv) || return 0;
$pasv =~ /\(([0-9,]+)\)/;
@n = split(/,/ , $1);
$self->open_socket("$n[0].$n[1].$n[2].$n[3]", $n[4]*256 + $n[5], "CON", $_[3]) || return 0;
$self->ftp_command("STOR $_[1]", 1, $_[3]) || return 0;

# transfer data
local $got;
open(PFILE, $_[2]);
while(read(PFILE, $buf, 1024) > 0) {
	print CON $buf;
	$got += length($buf);
	$self->$cbfunc(3, $got) if ($cbfunc);
	}
close(PFILE);
close(CON);
if ($got != $st[7]) {
	if ($_[3]) { ${$_[3]} = "Upload incomplete"; return 0; }
	else { $self->error("Upload incomplete"); }
	}
$self->$cbfunc(4) if ($cbfunc);

# finish off..
$self->ftp_command("", 2, $_[3]) || return 0;
$self->ftp_command("QUIT", 2, $_[3]) || return 0;
close(SOCK);

return 1;
}

# ftp_onecommand(host, command, [&error], [user, pass])
# Executes one command on an FTP server, after logging in, and returns its
# exit status.
sub ftp_onecommand
{
	my $self = shift;

local($buf, @n);

$self->{'download_timed_out'} = undef;
local $SIG{ALRM} = "download_timeout";
alarm(60);

# connect to host and login
$self->open_socket($_[0], 21, "SOCK", $_[2]) || return 0;
alarm(0);
if ($self->{'download_timed_out'}) {
	if ($_[2]) { ${$_[2]} = $self->{'download_timed_out'}; return 0; }
	else { $self->error($self->{'download_timed_out'}); }
	}
$self->ftp_command("", 2, $_[2]) || return 0;
if ($_[3]) {
	# Login as supplied user
	local @urv = $self->ftp_command("USER $_[3]", [ 2, 3 ], $_[2]);
	@urv || return 0;
	if (int($urv[1]/100) == 3) {
		$self->ftp_command("PASS $_[4]", 2, $_[2]) || return 0;
		}
	}
else {
	# Login as anonymous
	local @urv = $self->ftp_command("USER anonymous", [ 2, 3 ], $_[2]);
	@urv || return 0;
	if (int($urv[1]/100) == 3) {
		$self->ftp_command("PASS root\@".$self->get_system_hostname(), 2,
			     $_[2]) || return 0;
		}
	}

# make the directory
local @rv = $self->ftp_command($_[1], 2, $_[2]);
@rv || return 0;

# finish off..
$self->ftp_command("QUIT", 2, $_[3]) || return 0;
close(SOCK);

return $rv[1];
}

# ftp_listdir(host, dir, [&error], [user, pass])
# Returns a reference to a list of filenames in a directory
sub ftp_listdir
{
	my $self = shift;

local($buf, @n);

$self->{'download_timed_out'} = undef;
local $SIG{ALRM} = "download_timeout";
alarm(60);

# connect to host and login
$self->open_socket($_[0], 21, "SOCK", $_[2]) || return 0;
alarm(0);
if ($self->{'download_timed_out'}) {
	if ($_[2]) { ${$_[2]} = $self->{'download_timed_out'}; return 0; }
	else { $self->error($self->{'download_timed_out'}); }
	}
$self->ftp_command("", 2, $_[2]) || return 0;
if ($_[3]) {
	# Login as supplied user
	local @urv = $self->ftp_command("USER $_[3]", [ 2, 3 ], $_[2]);
	@urv || return 0;
	if (int($urv[1]/100) == 3) {
		$self->ftp_command("PASS $_[4]", 2, $_[2]) || return 0;
		}
	}
else {
	# Login as anonymous
	local @urv = $self->ftp_command("USER anonymous", [ 2, 3 ], $_[2]);
	@urv || return 0;
	if (int($urv[1]/100) == 3) {
		$self->ftp_command("PASS root\@".$self->get_system_hostname(), 2,
			     $_[2]) || return 0;
		}
	}

# request the listing
local $pasv = $self->ftp_command("PASV", 2, $_[2]);
defined($pasv) || return 0;
$pasv =~ /\(([0-9,]+)\)/;
@n = split(/,/ , $1);
$self->open_socket("$n[0].$n[1].$n[2].$n[3]", $n[4]*256 + $n[5], "CON", $_[2]) || return 0;
$self->ftp_command("NLST $_[1]", 1, $_[2]) || return 0;

# transfer listing
local @list;
while(<CON>) {
	s/\r|\n//g;
	push(@list, $_);
	}
close(CON);

# finish off..
$self->ftp_command("", 2, $_[3]) || return 0;
$self->ftp_command("QUIT", 2, $_[3]) || return 0;
close(SOCK);

return \@list;
}

# scp_copy(source, dest, password, &error)
# Copies a file from some source to a destination. One or the other can be
# a server, like user@foo:/path/to/bar/
sub scp_copy
{
	my $self = shift;

$self->foreign_require("proc", "proc-lib.pl");
local $cmd = "scp -r $_[0] $_[1]";
local ($fh, $fpid) = $main::proc_object->pty_process_exec($cmd);
local $out;
while(1) {
	local $rv = $self->wait_for($fh, "password:", "yes\\/no", ".*\n");
	$out .= $self->{'wait_for_input'};
	if ($rv == 0) {
		syswrite($fh, "$_[2]\n");
		}
	elsif ($rv == 1) {
		syswrite($fh, "yes\n");
		}
	elsif ($rv < 0) {
		last;
		}
	}
close($fh);
local $got = waitpid($fpid, 0);
if ($? || $out =~ /permission\s+denied/i) {
	${$_[3]} = "scp failed : <pre>$out</pre>";
	}
}

# free_ip_address()
# Returns an IP address within the allocation range which is not currently used
sub free_ip_address
{
	my $self = shift;

$self->foreign_require("net", "net-lib.pl");
local %taken = map { $_->{'address'}, $_ } ($main::net_object->boot_interfaces(),
					    $main::net_object->active_interfaces());
local @ranges = split(/\s+/, $self->{'config_hash'}->{'ip_ranges'});
local $r;
foreach $r (@ranges) {
	$r =~ /^(\d+\.\d+\.\d+)\.(\d+)\-(\d+)$/ || next;
	local ($base, $s, $e) = ($1, $2, $3);
	local $j;
	for($j=$s; $j<=$e; $j++) {
		local $try = "$base.$j";
		return $try if (!$taken{$try});
		}
	}
return undef;
}

# setup_for_subdomain(&parent-domain, subdomain-user, &sub-domain)
# Ensures that this virtual server can host sub-domains
sub setup_for_subdomain
{
	my $self = shift;

$self->system_logged("mkdir '$_[0]->{'home'}/domains' 2>/dev/null");
$self->system_logged("chmod 755 '$_[0]->{'home'}/domains'");
local $gid = $_[0]->{'gid'} || $_[0]->{'ugid'};
$self->system_logged("chown $_[0]->{'uid'}:$gid '$_[0]->{'home'}/domains'");
}

# count_domains(&parent)
# Returns the number of domains owned by this domain, and the max allowed
# for the current user. May exclude alias domains if they do not count towards
# the max.
sub count_domains
{
	my $self = shift;

local $count = 1;
local @subs = $self->get_domain_by("parent", $_[0]->{'id'});
if ($self->{'config_hash'}->{'limitnoalias'}) {
	@subs = grep { !$_->{'alias'} } @subs;
	}
$count += scalar(@subs);
local $max;
if ($self->master_admin() || $_[0]->{'domslimit'} eq "*") {
	$max = 99999999;	# a huge number
	}
else {
	$max = $_[0]->{'domslimit'};
	}
return ($count, $max);
}

# count_mailboxes(&parent)
# Returns the number of mailboxes in this domain and all subdomains, and the
# max allowed for the current user
sub count_mailboxes
{
	my $self = shift;

local $count = 0;
local $doms = 0;
local $parent = $_[0]->{'parent'} ? $self->get_domain($_[0]->{'parent'}) : $_[0];
local $d;
foreach $d ($parent, $self->get_domain_by("parent", $parent->{'id'})) {
	local @users = $self->list_domain_users($d, 0, 1, 1);
	$count += @users;
	$doms++;
	}
return ( $count, ("x$parent->{'mailboxlimit'}" ne "x") ? $parent->{'mailboxlimit'} : undef,
	 $doms );
}

# database_name(&domain)
# Returns a suitable database name for a domain
sub database_name
{
	my $self = shift;

local $tmpl = $self->get_template($_[0]->{'template'});
local $db = $self->substitute_template($tmpl->{'mysql'}, $_[0]);
$db ||= $_[0]->{'prefix'};
$db =~ s/[\.\-]/_/g;	# mysql doesn't like . or _
$db =~ s/^0/zero/g;	# postgresql doesn't like leading numbers
$db =~ s/^1/one/g;
$db =~ s/^2/two/g;
$db =~ s/^3/three/g;
$db =~ s/^4/four/g;
$db =~ s/^5/five/g;
$db =~ s/^6/six/g;
$db =~ s/^7/seven/g;
$db =~ s/^8/eight/g;
$db =~ s/^9/nine/g;
return $db;
}

# unixuser_name(domainname)
# Returns a Unix username for soem domain, or undef if none can be found
sub unixuser_name
{
	my $self = shift;

$_[0] =~ /^([^\.]+)/;
local ($try1, $user) = ($1, $1);
if (defined(getpwnam($try1)) || $self->{'config_hash'}->{'longname'}) {
	$user = $_[0];
	$self->{'try2'} = $user;
	if (defined(getpwnam($self->{'try'}))) {
		return (undef, $try1, $self->{'try2'});
		$self->error($self->text('setup_eauto', $try1, $self->{'try2'}));
		}
	}
return ($user);
}

# virtual_server_clashes(&dom, [&features-to-check])
# Returns a clash error message if any were found for some new domain
sub virtual_server_clashes
{
	my $self = shift;

local ($dom, $check) = @_;
my $f;
foreach $f (@{$self->{'features_array'}}) {
	if ($dom->{$f} && (!$check || $check->{$f})) {
		local $cfunc = "check_${f}_clash";
		if ($self->$cfunc($dom)) {
			return $self->text('setup_e'.$f, $dom->{'dom'}, $dom->{'db'},
				     $dom->{'user'}, $dom->{'group'});
			}
		}
	}
foreach $f (@{$self->{'feature_plugins_array'}}) {
	if ($dom->{$f} && (!$check || $check->{$f})) {
		local $cerr = $self->plugin_call($f, "feature_clash", $dom);
		return $cerr if ($cerr);
		}
	}
return undef;
}

# virtual_server_depends(&dom)
# Returns an error message if any of the features in the domain depend on
# missing features
sub virtual_server_depends
{
	my $self = shift;

local $f;
DEP_BLOCK: {
foreach $f (grep { $_[0]->{$_} } @{$self->{'features_array'}}) {
	local $dfunc = "check_depends_$f";
	if ($self->can($dfunc)) {
		# Call dependecy function
		local $derr = $self->$dfunc($_[0]);
		return $derr if ($derr);
		}
	else {
		# Check fixed dependency list
		local $fd;
		foreach $fd (@{$self->{'feature_depends_hash'}->{$f}}) {
			#return &text('setup_edep'.$f) if (!$_[0]->{$fd});
			if (!$_[0]->{$fd}) {
			    delete($_[0]->{$f});
			    redo DEP_BLOCK;
			    }
			}
		}
	}
}
foreach $f (grep { $_[0]->{$_} } @{$self->{'feature_plugins_array'}}) {
	local $derr = $self->plugin_call($f, "feature_depends", $_[0]);
	return $derr if ($derr);
	}
return undef;
}

# create_virtual_server(&domain, [&parent-domain], [parent-user])
# Given a complete domain object, setup all it's features
sub create_virtual_server
{
	my $self = shift;

local ($dom, $parentdom, $parentuser) = @_;

my $need_undo = 0;
CREATE_BLOCK:
{
# Run the before command
$self->set_domain_envs($_[0], "CREATE_DOMAIN");
local $merr = $self->making_changes();
return $self->text('setup_emaking', "<tt>$merr</tt>") if (defined($merr));

# Get ready for hosting a subdomain
if ($dom->{'parent'}) {
	$self->setup_for_subdomain($parentdom, $parentuser, $dom);
	}

# Add virtual IP address, if needed
if ($dom->{'virt'}) {
	local $main::error_must_die = 1;
	eval ($self->setup_virt($dom));
	if ($@) {
		 do { my $fn = $self->{'second_print'}; $self->$fn($self->text('setup_failure',$self->{'text_hash'}->{'form_ip'}, $@))};
		 $dom->{$f} = 0;
		 $need_undo=1;
		 last CREATE_BLOCK;
		}
	else
		{
		 push (@{$self->{'create_log_array'}},{'func' => "delete_virt",'params' => [$dom]});
		}
	}
	
# Set up all the selected features
my $f;
local %vital = map { $_, 1 } @{$self->{'vital_features_array'}};
foreach $f (@{$self->{'features_array'}}) {
	if ($dom->{$f}) {
		local $sfunc = "setup_$f";
			
			# Failure can be ignored
			if (!$self->try_function($f, $sfunc, $dom)) {
				$dom->{$f} = 0;
				$need_undo=1;
				last CREATE_BLOCK;
				}
			else
				{
				 push (@{$self->{'create_log_array'}},{'func' => "delete_$f",'params' => [$dom]});
				}

		}
	}

# Set up all the selected plugins
foreach $f (@{$self->{'feature_plugins_array'}}) {
	if ($dom->{$f}) {
		# Failure can be ignored
		local $main::error_must_die = 1;
		eval { $self->plugin_call($f, "feature_setup", $dom) };
		if ($@) {
			do { my $fn = $self->{'second_print'}; $self->$fn($self->text('setup_failure',
				$self->plugin_call($f, "feature_name"), $@))};
			$dom->{$f} = 0;
			$need_undo=1;
			last CREATE_BLOCK;
			}
		else
			{
			push (@{$self->{'create_log_array'}},{'func' => "plugin_call",'params' => [$f,"feature_delete",$dom]});
			}
		}
	}

# Disable all configured features
@{$self->{'disable_array'}} = split (/,/,$_[0]->{'disabled'});
%{$self->{'disable_hash'}} = map { $_, 1 } @{$self->{'disable_array'}};
my $f;
foreach $f (@{$self->{'features_array'}}) {
	if ($dom->{$f} && $self->{'disable_hash'}->{$f}) {
		local $dfunc = "disable_$f";
		if ($self->try_function($f, $dfunc, $dom)) {
			push(@{$self->{'disabled_array'}}, $f);
			}
		}
	}
foreach $f (@{$self->{'feature_plugins_array'}}) {
	if ($dom->{$f} && $self->{'disable_hash'}->{$f}) {
		$self->plugin_call($f, "feature_disable", $dom);
		push(@{$self->{'disabled_array'}}, $f);
		}
	}	

$self->run_post_actions();

# Add a virtuser for the unix user, if requested
if ($self->{'in_hash'}->{'mailbox'}) {
	do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_mailbox'})};
	local $virt = { 'from' => $self->{'user'}."\@".$dom->{'dom'},
			'to' => [ $self->{'user'} ] };
	eval($self->create_virtuser($virt));
	do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
#	if ($@) {
#		 do { my $fn = $self->{'second_print'}; $self->$fn($self->text('setup_failure',$self->{'text_hash'}->{'feature_mail'}, $@))};
#		 $dom->{$f} = 0;
#		 $need_undo=1;
#		 last CREATE_BLOCK;
#		}
#	else
#		{
#		 do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
#		 push (@{$self->{'create_log_array'}},{'func' => "delete_virtuser",'params' => [$virt]});
#		}
	}

# Save domain details
do { my $fn = $self->{'first_print'}; $self->$fn($self->{'text_hash'}->{'setup_save'})};
local $main::error_must_die = 1;
eval ($self->save_domain($dom));
if ($@) {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->text('setup_failure',"", $@))};
	$dom->{$f} = 0;
	$need_undo=1;
        last CREATE_BLOCK;
	}
	else
	{
	 do { my $fn = $self->{'second_print'}; $self->$fn($self->{'text_hash'}->{'setup_done'})};
	 push (@{$self->{'create_log_array'}},{'func' => "delete_domain",'params' => [$dom]});
	}

if (!$dom->{'alias'}) {
	# Notify the owner via email
	$self->send_domain_email($dom);
	}

# Update the parent domain Webmin user
if ($parentdom) {
	local $main::error_must_die = 1;
	eval($self->modify_webmin($parentdom, $parentdom));
	if ($@) {
 		do { my $fn = $self->{'second_print'}; $self->$fn($self->text('setup_failure',"", $@))};
 		$dom->{$f} = 0;
		$need_undo=1;
		last CREATE_BLOCK;
		}
	}

if ($self->{'remote_user'}) {
	# Add to this user's list of domains if needed
	local %access = $self->get_module_acl();
	if (!$self->can_edit_domain($dom)) {
		$access{'domains'} = join(" ", split(/\s+/, $access{'domains'}),
					       $dom->{'id'});
			local $main::error_must_die = 1;
			eval($self->save_module_acl(\%access));
			if ($@) {
 				do { my $fn = $self->{'second_print'}; $self->$fn($self->text('setup_failure',"", $@))};
 				$dom->{$f} = 0;
				$need_undo=1;
				last CREATE_BLOCK;
			}
		}
	}

# Run the after creation command
$self->made_changes();
} # closes CREATE_BLOCK

if ($need_undo) {
	print "<hr><h3>$self->{'text_hash'}->{'setup_emessage'}</h3>";
	print "<h2>$self->{'text_hash'}->{'setup_rollback'}</h2>";
	# drop all caches
	undef(@{$main::useradmin_object->{'list_users_cache_array'}});
	undef(@{$main::useradmin_object->{'list_groups_cache_array'}});
	undef(@{$main::bind8_object->{'get_config_cache_array'}});
	undef(@{$main::apache_object->{'get_config_cache_array'}});
	# Run the before command
	$self->set_domain_envs($dom, "DELETE_DOMAIN");
	$merr = $self->making_changes();
	#&error($self->text('delete_emaking', "<tt>$merr</tt>")) if (defined($merr));
	# Delete domain file
	print $self->text('delete_domain', "<tt>$dom->{'dom'}</tt>"),"<br>\n";
	$self->delete_domain(\%{$self->{'dom_hash'}});
	print $self->{'text_hash'}->{'setup_done'},"<p>\n";

	foreach (reverse @{$self->{'create_log_array'}}) {
		undef(@{$main::useradmin_object->{'list_users_cache_array'}});
		undef(@{$main::useradmin_object->{'list_groups_cache_array'}});
		#undef(@{$main::bind8_object->{'get_config_cache_array'}});
		#undef(@{$main::apache_object->{'get_config_cache_array'}});
		undef(%main::file_cache);
		$self->{'name_fun'} = $_->{'func'};
		#do { my $fn = $self->{'name_fun'}; $self->$fn(@{$_->{'params'}})};
		@{$self->{'name_param_array'}} = @{$_->{'params'}};
		foreach $n (@{$self->{'name_param_array'}})
		    {do { my $fn = $self->{'name_fun'}; $self->$fn($n)};
		    }
		}
	}

return undef;
}

# register_post_action(&function, args)
sub register_post_action
{
	my $self = shift;

push(@main::post_actions, [ @_ ]);
}

# run_post_actions()
# Run all registered post-modification actions
sub run_post_actions
{
	my $self = shift;

local $a;
local %done;
foreach $a (@main::post_actions) {
	next if ($done{join(",", @$a)}++);
	local ($afunc, @aargs) = @$a;
	local $main::error_must_die = 1;
	eval { $self->$afunc(@aargs) };
	if ($@) {
		do { my $fn = $self->{'second_print'}; $self->$fn($self->text('setup_postfailure', $@))};
		}
	}
@{$self->{'post_actions_array'}} = ( );
}

# find_bandwidth_job()
# Returns the cron job used for bandwidth monitoring
sub find_bandwidth_job
{
	my $self = shift;

$self->foreign_require("cron", "cron-lib.pl");
local @jobs = $main::cron_object->list_cron_jobs();
local ($job) = grep { $_->{'user'} eq 'root' &&
		      $_->{'command'} eq $self->{'bw_cron_cmd'} } @jobs;
return $job;
}

# get_bandwidth(&domain)
# Returns the bandwidth usage object for some domain
sub get_bandwidth
{
	my $self = shift;

if (!defined($self->{'get_bandwidth_cache_hash'}->{$_[0]->{'id'}})) {
	local %bwinfo;
	$self->read_file("$self->{'bandwidth_dir'}/$_[0]->{'id'}", \%bwinfo);
	local $k;
	foreach $k (keys %bwinfo) {
		if ($k =~ /^\d+$/) {
			# Convert old web entries
			$bwinfo{"web_$k"} = $bwinfo{$k};
			delete($bwinfo{$k});
			}
		}
	$self->{'get_bandwidth_cache_hash'}->{$_[0]->{'id'}} = \%bwinfo;
	}
return $self->{'get_bandwidth_cache_hash'}->{$_[0]->{'id'}};
}

# save_bandwidth(&domain, &info)
sub save_bandwidth
{
	my $self = shift;

mkdir($self->{'bandwidth_dir'}, 0700);
$self->write_file("$self->{'bandwidth_dir'}/$_[0]->{'id'}", $_[1]);
$self->{'get_bandwidth_cache_hash'}->{$_[0]->{'id'}} ||= $_[1];
}

# bandwidth_input(name, value)
# Returns HTML for a bandwidth input field, with an 'unlimited' option
sub bandwidth_input
{
	my $self = shift;

local $rv;
local $value = '';
if ($_[1]) {
	$value = sprintf ('%0.1f',$_[1]/1024/1024);
}
$rv .= sprintf "<input type=radio name=$_[0]_def value=0 %s>\n",
$_[1] ? "checked" : "";
#local ($val, $u);
#if ($_[1] && $_[1]%(1024*1024*1024) == 0) {
#	$val = $_[1]/(1024*1024*1024);
#	$u = "GB";
#	}
#elsif ($_[1] && $_[1]%(1024*1024) == 0) {
#	$val = $_[1]/(1024*1024);
#	$u = "MB";
#	}
#elsif ($_[1] && $_[1]%(1024) == 0) {
#	$val = $_[1]/(1024);
#	$u = "kB";
#	}
#else {
#	$val = $_[1];
#	$u = "bytes";
#	}
#local $sel = "<select name=$_[0]_units>";
#local $t;
#foreach $t ("GB", "MB", "kB", "bytes") {
#	$sel .= sprintf "<option %s>%s\n",
#		$t eq $u ? "selected" : "", $t;
#	}
#$sel .= "</select>\n";
local $sel = "<input type=hidden name=$_[0]_units value='MB'>\n MB\n";
$rv .= "<input name=$_[0] size=10 value=$value>$sel\n";
#$rv .= &text('edit_bwpast_'.$config{'bw_past'},
#	     "<input name=$_[0] size=10 value='$val'> $sel",
#	     $config{'bw_period'});
$rv .= sprintf "<input type=radio name=$_[0]_def value=1 %s> %s\n",
$_[1] ? "" : "checked", $self->{'text_hash'}->{'edit_bwnone'};
return $rv;
}

# parse_bandwidth(name, error)
sub parse_bandwidth
{
	my $self = shift;

if ($self->{'in_hash'}->{"$_[0]_def"}) {
	return undef;
	}
else {
	$self->{'in_hash'}->{$_[0]} =~ /^\d+\.?\d*$/ && $self->{'in_hash'}->{$_[0]} > 0 || $self->error($_[1]);
	local $m = $self->{'in_hash'}->{"$_[0]_units"} eq "GB" ? 1024*1024*1024 :
		   $self->{'in_hash'}->{"$_[0]_units"} eq "MB" ? 1024*1024 :
		   $self->{'in_hash'}->{"$_[0]_units"} eq "kB" ? 1024 : 1;
	return $self->{'in_hash'}->{$_[0]} * $m;
	}
}

# email_template_input(template-file, subject, other-cc)
# Returns HTML for fields for editing an email template
sub email_template_input
{
	my $self = shift;

local $rv;
$rv .= "<table>\n";
$rv .= "<tr> <td><b>$self->{'text_hash'}->{'newdom_subject'}</b></td>\n";
$rv .= sprintf "<td><input name=subject value='%s' size=60></td> </tr>\n",
	$_[1];
$rv .= "<tr> <td><b>$self->{'text_hash'}->{'newdom_cc'}</b></td>\n";
$rv .= sprintf "<td><input name=cc value='%s' size=60></td> </tr>\n",
	$_[2];
$rv .= "</table>\n";
if ($_[0]) {
	$rv .= "<textarea name=template rows=20 cols=70>";
	open(FILE, $_[0]);
	while(<FILE>) {
		$rv .= $self->html_escape($_);
		}
	close(FILE);
	$rv .= "</textarea>\n";
	}
return $rv;
}

# parse_email_template(file, subject-config, cc-config)
sub parse_email_template
{
	my $self = shift;

$self->{'in_hash'}->{'template'} =~ s/\r//g;
$self->open_lock_tempfile(FILE, ">$_[0]", 1) ||
	$self->error($self->text('efilewrite', $self->{'file'}, $!));
$self->print_tempfile(FILE, $self->{'in_hash'}->{'template'});
$self->close_tempfile(FILE);

$self->lock_file($self->{'module_config_file'});
$self->{'config_hash'}->{$_[1]} = $self->{'in_hash'}->{'subject'};
$self->{'config_hash'}->{$_[2]} = $self->{'in_hash'}->{'cc'};
$self->{'config_hash'}->{'last_check'} = time()+1;	# no need for check.cgi to be run
$self->save_module_config();
$self->unlock_file($self->{'module_config_file'});
}

# escape_user(username)
# Returns a Unix username with characters unsuitable for use in a mail
# destination (like @) escaped
sub escape_user
{
	my $self = shift;

local $escuser = $_[0];
$escuser =~ s/\@/\\\@/g;
return $escuser;
}

# unescape_user(username)
# The reverse of escape_user
sub unescape_user
{
	my $self = shift;

local $escuser = $_[0];
$escuser =~ s/\\\@/\@/g;
return $escuser;
}

# escape_alias(username)
# Converts a username into a suitable alias name
sub escape_alias
{
	my $self = shift;

local $escuser = $_[0];
$escuser =~ s/\@/-/g;
return $escuser;
}

# dotqmail_file(&user)
sub dotqmail_file
{
	my $self = shift;

return "$_[0]->{'home'}/.qmail";
}

# get_dotqmail(file)
sub get_dotqmail
{
	my $self = shift;

$_[0] =~ /\.qmail(-(\S+))?$/;
local $alias = { 'file' => $_[0],
		 'name' => $2 };
open(AFILE, $_[0]) || return undef;
while(<AFILE>) {
	s/\r|\n//g;
	s/#.*$//g;
	if (/\S/) {
		push(@{$alias->{'values'}}, $_);
		}
	}
close(AFILE);
return $alias;
}

# save_dotqmail(&alias, file, username|aliasname)
sub save_dotqmail
{
	my $self = shift;

if (@{$_[0]->{'values'}}) {
	$self->open_lock_tempfile(AFILE, ">$_[1]");
	local $v;
	foreach $v (@{$_[0]->{'values'}}) {
		if ($v eq "\\$_[2]" || $v eq "\\NEWUSER") {
			# Delivery to this user means to his maildir
			$self->print_tempfile(AFILE, "./Maildir/\n");
			}
		else {
			$self->print_tempfile(AFILE, $v,"\n");
			}
		}
	$self->close_tempfile(AFILE);
	}
else {
	unlink($_[1]);
	}
}

# list_templates()
# Returns a list of all virtual server templates, including two defaults for
# top-level and sub-servers
sub list_templates
{
	my $self = shift;

local @rv;
push(@rv, { 'id' => 0,
	    # 20-02-07 (add 'namedconf' for fix feature-dns.pl ver 3.35)
	    'namedconf' => $self->{'config_hash'}->{'namedconf'} || "none",
	    'name' => 'Default Settings',
	    'standard' => 1,
	    'default' => 1,
	    'web' => $self->{'config_hash'}->{'apache_config'},
	    'web_suexec' => $self->{'config_hash'}->{'suexec'},
	    'web_writelogs' => $self->{'config_hash'}->{'web_writelogs'},
	    'web_user' => $self->{'config_hash'}->{'web_user'},
	    'web_html_dir' => $self->{'config_hash'}->{'html_dir'},
	    'web_html_perms' => $self->{'config_hash'}->{'html_perms'} || 755,
	    'web_stats_dir' => $self->{'config_hash'}->{'stats_dir'},
	    'web_stats_hdir' => $self->{'config_hash'}->{'stats_hdir'},
	    'web_stats_pass' => $self->{'config_hash'}->{'stats_pass'},
	    'web_port' => $self->{'default_web_port'},
	    'web_sslport' => $self->{'default_web_sslport'},
	    'web_alias' => $self->{'config_hash'}->{'alias_mode'},
	    'web_webmin_ssl' => $self->{'config_hash'}->{'webmin_ssl'},
	    'web_usermin_ssl' => $self->{'config_hash'}->{'usermin_ssl'},
	    'webalizer' => $self->{'config_hash'}->{'def_webalizer'} || "none",
	    'dns' => $self->{'config_hash'}->{'bind_config'},
	    'dns_replace' => $self->{'config_hash'}->{'bind_replace'},
	    'dns_view' => $self->{'config_hash'}->{'dns_view'},
	    'ftp' => $self->{'config_hash'}->{'proftpd_config'},
	    'ftp_vsftpd' => $self->{'config_hash'}->{'vsftpd_config'},
	    'ftp_dir' => $self->{'config_hash'}->{'ftp_dir'},
	    'mail_on' => $self->{'config_hash'}->{'domain_template'},
	    'mail' => $self->{'config_hash'}->{'domain_template'} eq "none" ||
		      $self->{'config_hash'}->{'domain_template'} eq "default" ?
				$self->cat_file("domain-template") :
				$self->cat_file($self->{'config_hash'}->{'domain_template'}),
	    'mail_subject' => $self->{'config_hash'}->{'newdom_subject'} ||
			      $self->entities_to_ascii($self->{'text_hash'}->{'mail_dsubject'}),
	    'mail_cc' => $self->{'config_hash'}->{'newdom_cc'},
	    'user_aliases' => $self->{'config_hash'}->{'newuser_aliases'} || "none",
	    'dom_aliases' => $self->{'config_hash'}->{'newdom_aliases'} || "none",
	    'mysql' => $self->{'config_hash'}->{'mysql_db'} || '${PREFIX}',
	    'mysql_wild' => $self->{'config_hash'}->{'mysql_wild'},
	    'mysql_hosts' => $self->{'config_hash'}->{'mysql_hosts'} || "none",
	    'mysql_mkdb' => $self->{'config_hash'}->{'mysql_mkdb'},
	    'mysql_nopass' => $self->{'config_hash'}->{'mysql_nopass'},
	    'skel' => $self->{'config_hash'}->{'virtual_skel'} || "none",
	    'frame' => $self->cat_file("framefwd-template"),
	    'gacl' => 1,
	    'gacl_umode' => $self->{'config_hash'}->{'gacl_umode'},
	    'gacl_uusers' => $self->{'config_hash'}->{'gacl_uusers'},
	    'gacl_ugroups' => $self->{'config_hash'}->{'gacl_ugroups'},
	    'gacl_groups' => $self->{'config_hash'}->{'gacl_groups'},
	    'gacl_root' => $self->{'config_hash'}->{'gacl_root'},
	    'ugroup' => $self->{'config_hash'}->{'defugroup'} || "none",
	    'quota' => $self->{'config_hash'}->{'defquota'} || "none",
	    'uquota' => $self->{'config_hash'}->{'defuquota'} || "none",
	    'mailboxlimit' => $self->{'config_hash'}->{'defmailboxlimit'} eq "" ? "none" :
			      $self->{'config_hash'}->{'defmailboxlimit'},
	    'dbslimit' => $self->{'config_hash'}->{'defdbslimit'} eq "" ? "none" :
			  $self->{'config_hash'}->{'defdbslimit'},
	    'domslimit' => $self->{'config_hash'}->{'defdomslimit'} eq "" ? 0 :
			   $self->{'config_hash'}->{'defdomslimit'} eq "*" ? "none" :
			   $self->{'config_hash'}->{'defdomslimit'},
	    'bwlimit' => $self->{'config_hash'}->{'defbwlimit'} eq "" ? "none" :
			 $self->{'config_hash'}->{'defbwlimit'},
	    'for_parent' => 1,
	    'for_sub' => 0,
	    'for_alias' => 1,
	    'for_users' => 1,
	  } );
push(@rv, { 'id' => 1,
	    'name' => 'Defaults Settings For Sub-Servers',
	    'standard' => 1,
	    'mail_on' => $self->{'config_hash'}->{'subdomain_template'} eq "none" ? 0 : 1,
	    'mail' => $self->{'config_hash'}->{'subdomain_template'} eq "none" ||
		      $self->{'config_hash'}->{'subdomain_template'} eq "default" ?
				$self->cat_file("subdomain-template") :
				$self->cat_file($self->{'config_hash'}->{'subdomain_template'}),
	    'mail_subject' => $self->{'config_hash'}->{'newsubdom_subject'} ||
			      $self->entities_to_ascii($self->{'text_hash'}->{'mail_dsubject'}),
	    'mail_cc' => $self->{'config_hash'}->{'newsubdom_cc'},
	    'skel' => $self->{'config_hash'}->{'sub_skel'} || "none",
	    'for_parent' => 0,
	    'for_sub' => 1,
	    'for_alias' => 0,
	    'for_users' => 1
	  } );
local $f;
opendir(DIR, $self->{'templates_dir'});
while($f = readdir(DIR)) {
	if ($f ne "." && $f ne "..") {
		local %tmpl;
		$self->read_file("$self->{'templates_dir'}/$f", \%tmpl);
		$tmpl{'mail'} =~ s/\t/\n/g;
		if ($tmpl{'id'} == 1) {
			foreach $k (keys %tmpl) {
				$rv[1]->{$k} = $tmpl{$k}
					if (!defined($rv[1]->{$k}));
				}
			}
		else {
			push(@rv, \%tmpl);
			}
		}
	}
closedir(DIR);
return @rv;
}

# save_template(&template)
# Create or update a template. If saving the standard template, updates the
# appropriate config options instead of the template file.
sub save_template
{
	my $self = shift;

local $save_config = 0;
if (!defined($_[0]->{'id'})) {
	$_[0]->{'id'} = $self->domain_id();
	}
if ($_[0]->{'id'} == 0) {
	# Update appropriate config entries
	$self->{'config_hash'}->{'apache_config'} = $_[0]->{'web'};
	$self->{'config_hash'}->{'suexec'} = $_[0]->{'web_suexec'};
	$self->{'config_hash'}->{'web_writelogs'} = $_[0]->{'web_writelogs'};
	$self->{'config_hash'}->{'web_user'} = $_[0]->{'web_user'};
	$self->{'config_hash'}->{'html_dir'} = $_[0]->{'web_html_dir'};
	$self->{'config_hash'}->{'html_perms'} = $_[0]->{'web_html_perms'};
	$self->{'config_hash'}->{'stats_dir'} = $_[0]->{'web_stats_dir'};
	$self->{'config_hash'}->{'stats_hdir'} = $_[0]->{'web_stats_hdir'};
	$self->{'config_hash'}->{'stats_pass'} = $_[0]->{'web_stats_pass'};
	$self->{'config_hash'}->{'web_port'} = $_[0]->{'web_port'};
	$self->{'config_hash'}->{'web_sslport'} = $_[0]->{'web_sslport'};
	$self->{'config_hash'}->{'webmin_ssl'} = $_[0]->{'web_webmin_ssl'};
	$self->{'config_hash'}->{'usermin_ssl'} = $_[0]->{'web_usermin_ssl'};
	$self->{'config_hash'}->{'def_webalizer'} = $_[0]->{'webalizer'} eq "none" ? "" :
					$_[0]->{'webalizer'};
	$self->{'config_hash'}->{'alias_mode'} = $_[0]->{'web_alias'};
	$self->{'config_hash'}->{'bind_config'} = $_[0]->{'dns'};
	$self->{'config_hash'}->{'bind_replace'} = $_[0]->{'dns_replace'};
	$self->{'config_hash'}->{'dns_view'} = $_[0]->{'dns_view'};
	# 20-02-07 (add $self->{'config_hash'}->{'namedconf'} for fix feature-dns.pl ver3.35) 
	$self->{'config_hash'}->{'namedconf'} = $_[0]->{'namedconf'} eq 'none' ? undef :
						    $_[0]->{'namedconf'};
	delete($self->{'config_hash'}->{'mx_server'});
	$self->{'config_hash'}->{'proftpd_config'} = $_[0]->{'ftp'};
	$self->{'config_hash'}->{'vsftpd_config'} = $_[0]->{'ftp_vsftpd'};
	$self->{'config_hash'}->{'ftp_dir'} = $_[0]->{'ftp_dir'};
	if ($_[0]->{'mail_on'} ne "none") {
		if ($self->{'config_hash'}->{'domain_template'} eq "none") {
			$self->{'config_hash'}->{'domain_template'} = "default";
			}
		}
	else {
		$self->{'config_hash'}->{'domain_template'} = "none";
		}
	$self->uncat_file($self->{'config_hash'}->{'domain_template'} eq "none" ||
		    $self->{'config_hash'}->{'domain_template'} eq "default" ?
			"domain-template" :
			$self->{'config_hash'}->{'domain_template'}, $_[0]->{'mail'});
	$self->{'config_hash'}->{'newdom_subject'} = $_[0]->{'mail_subject'};
	$self->{'config_hash'}->{'newdom_cc'} = $_[0]->{'mail_cc'};
	$self->{'config_hash'}->{'newuser_aliases'} = $_[0]->{'user_aliases'} eq "none" ?
					"" : $_[0]->{'user_aliases'};
	$self->{'config_hash'}->{'newdom_aliases'} = $_[0]->{'dom_aliases'} eq "none" ?
					"" : $_[0]->{'dom_aliases'};
	$self->{'config_hash'}->{'mysql_db'} = $_[0]->{'mysql'};
	$self->{'config_hash'}->{'mysql_wild'} = $_[0]->{'mysql_wild'};
	$self->{'config_hash'}->{'mysql_hosts'} = $_[0]->{'mysql_hosts'} eq "none" ?
					"" : $_[0]->{'mysql_hosts'};
	$self->{'config_hash'}->{'mysql_mkdb'} = $_[0]->{'mysql_mkdb'};
	$self->{'config_hash'}->{'mysql_nopass'} = $_[0]->{'mysql_nopass'};
	$self->{'config_hash'}->{'virtual_skel'} = $_[0]->{'skel'} eq "none" ? "" :
				  $_[0]->{'skel'};
	$self->{'config_hash'}->{'gacl_umode'} = $_[0]->{'gacl_umode'};
	$self->{'config_hash'}->{'gacl_ugroups'} = $_[0]->{'gacl_ugroups'};
	$self->{'config_hash'}->{'gacl_users'} = $_[0]->{'gacl_users'};
	$self->{'config_hash'}->{'gacl_groups'} = $_[0]->{'gacl_groups'};
	$self->{'config_hash'}->{'gacl_root'} = $_[0]->{'gacl_root'};
	$self->{'config_hash'}->{'defugroup'} = $_[0]->{'ugroup'};
	$self->{'config_hash'}->{'defquota'} = $_[0]->{'quota'};
	$self->{'config_hash'}->{'defuquota'} = $_[0]->{'uquota'};
	$self->{'config_hash'}->{'defmailboxlimit'} = $_[0]->{'mailboxlimit'} eq 'none' ? undef :
				     $_[0]->{'mailboxlimit'};
	$self->{'config_hash'}->{'defdbslimit'} = $_[0]->{'dbslimit'} eq 'none' ? undef :
				 $_[0]->{'dbslimit'};
	$self->{'config_hash'}->{'defdomslimit'} = $_[0]->{'domslimit'} eq 'none' ? "*" :
				  $_[0]->{'domslimit'} eq '0' ? "" :
				  $_[0]->{'domslimit'};
	$self->{'config_hash'}->{'defbwlimit'} = $_[0]->{'bwlimit'} eq 'none' ? undef :
				$_[0]->{'bwlimit'};
	$self->uncat_file("framefwd-template", $_[0]->{'frame'});
	$save_config = 1;
	}
elsif ($_[0]->{'id'} == 1) {
	# For the default for sub-servers, update mail and skel in config only
	if ($_[0]->{'mail_on'} ne "none") {
		if ($self->{'config_hash'}->{'subdomain_template'} eq "none") {
			$self->{'config_hash'}->{'subdomain_template'} = "default";
			}
		}
	else {
		$self->{'config_hash'}->{'subdomain_template'} = "none";
		}
	$self->uncat_file($self->{'config_hash'}->{'subdomain_template'} eq "none" ||
		    $self->{'config_hash'}->{'subdomain_template'} eq "default" ?
			"subdomain-template" :
			$self->{'config_hash'}->{'subdomain_template'}, $_[0]->{'mail'});
	$self->{'config_hash'}->{'newsubdom_subject'} = $_[0]->{'mail_subject'};
	$self->{'config_hash'}->{'newsubdom_cc'} = $_[0]->{'mail_cc'};
	$self->{'config_hash'}->{'sub_skel'} = $_[0]->{'skel'} eq "none" ? "" :
			      $_[0]->{'skel'};
	$save_config = 1;
	}
if ($_[0]->{'id'} != 0) {
	# Just save the file
	mkdir($self->{'templates_dir'}, 0700);
	$_[0]->{'created'} ||= time();
	$_[0]->{'mail'} =~ s/\n/\t/g;
	$self->lock_file("$self->{'templates_dir'}/$_[0]->{'id'}");
	$self->write_file("$self->{'templates_dir'}/$_[0]->{'id'}", $_[0]);
	$self->unlock_file("$self->{'templates_dir'}/$_[0]->{'id'}");
	}
if ($save_config) {
	$self->lock_file($self->{'module_config_file'});
	$self->{'config_hash'}->{'last_check'} = time()+1;
	$self->write_file($self->{'module_config_file'}, \%{$self->{'config_hash'}});
	$self->unlock_file($self->{'module_config_file'});
	}
}

# get_template(id)
# Returns a template, with any default settings filled in from real default
sub get_template
{
	my $self = shift;

local @tmpls = $self->list_templates();
local ($tmpl) = grep { $_->{'id'} == $_[0] } @tmpls;
if (!$tmpl->{'default'}) {
	local $def = $tmpls[0];
	$tmpl->{'skel'} = $def->{'skel'} if (!$tmpl->{'skel'});
	local $p;
	# 20-02-07 (add "namedconf" for fix feature-dns.pl v 3.35)
	foreach $p ("web", "dns", "ftp", "mail", "frame", "user_aliases",
		    "ugroup", "quota", "uquota", "mailboxlimit", "domslimit",
		    "dbslimit", "bwlimit", "mysql_hosts", "mysql_mkdb",
		    "mysql_nopasswd", "webalizer", "dom_aliases", "namedconf" ) {
		if ($tmpl->{$p} eq "") {
			local $k;
			foreach $k (keys %$def) {
				$tmpl->{$k} = $def->{$k}
					if ($k =~ /^\Q$p\E_/ || $k eq $p);
				}
			}
		}
	}
return $tmpl;
}

# delete_template(&template)
# Just mark this template as deleted, as it may be needed later
sub delete_template
{
	my $self = shift;

local %tmpl;
$self->lock_file("$self->{'templates_dir'}/$_[0]->{'id'}");
$self->read_file("$self->{'templates_dir'}/$_[0]->{'id'}", \%tmpl);
$tmpl{'deleted'} = 1;
$self->write_file("$self->{'templates_dir'}/$_[0]->{'id'}", \%tmpl);
$self->unlock_file("$self->{'templates_dir'}/$_[0]->{'id'}");
}

# cat_file(file)
# Returns the contents of some file
sub cat_file
{
	my $self = shift;

local $path = $_[0] =~ /^\// ? $_[0] : "$self->{'module_config_directory'}/$_[0]";
local $rv;
open(FILE, $path);
while(<FILE>) {
	$rv .= $_;
	}
close(FILE);
return $rv;
}

# uncat_file(file, data)
# Writes to some file
sub uncat_file
{
	my $self = shift;

local $path = $_[0] =~ /^\// ? $_[0] : "$self->{'module_config_directory'}/$_[0]";
$self->open_lock_tempfile(FILE, ">$path");
$self->print_tempfile(FILE, $_[1]);
$self->close_tempfile(FILE);
}

# plugin_call(module, function, [arg, ...])
# If some plugin function is defined, call it and return the result,
# otherwise return undef
sub plugin_call
{
	my $self = shift;

local ($mod, $func, @args) = @_;
if ($self->plugin_defined($mod, $func)) {
	if ($self->{'module_name'} ne "virtual_server") {
		# Set up virtual_server package
		$self->foreign_require("virtual_server", "virtual-server-lib.pl");
		$main::virtual_server_object->{'first_print'} = $self->{'first_print'};
		$main::virtual_server_object->{'second_print'} = $self->{'second_print'};
		$main::virtual_server_object->{'indent_print'} = $self->{'indent_print'};
		$main::virtual_server_object->{'outdent_print'} = $self->{'outdent_print'};
		}
	return $self->foreign_call($mod, $func, @args);
	}
else {
	return wantarray ? ( ) : undef;
	}
}

# plugin_defined(module, function)
# Returns 1 if some function is defined in a plugin
sub plugin_defined
{
	my $self = shift;

local $pkg = $_[0];
$pkg =~ s/[^A-Za-z0-9]/_/g;
local $func = "${pkg}::$_[1]";
return defined(&$func);
}

# database_feature([&domain])
# Returns 1 if any feature that uses a database is enabled (perhaps in a domain)
sub database_feature
{
	my $self = shift;

local $ok = 0;
foreach my $f ('mysql', 'postgres', @database_plugins) {
        $ok = 1 if ($self->{'config_hash'}->{$f} &&
                    (!$_[0] || $_[0]->{$f}));
        }
return $ok;
}

# list_custom_fields()
# Returns a list of structures containing custom field details
sub list_custom_fields
{
	my $self = shift;

local @rv;
open(FIELDS, $self->{'custom_fields_file'});
while(<FIELDS>) {
	s/\r|\n//g;
	local @a = split(/:/, $_, 4);
	push(@rv, { 'name' => $a[0],
		    'type' => $a[1],
		    'opts' => $a[2],
		    'desc' => $a[3] });

	}
close(FIELDS);
return @rv;
}

# save_custom_fields(&fields)
sub save_custom_fields
{
	my $self = shift;

$self->open_tempfile(FIELDS, ">$self->{'custom_fields_file'}");
foreach $a (@{$_[0]}) {
	$self->print_tempfile(FIELDS, $a->{'name'},":",$a->{'type'},":",
		     $a->{'opts'},":",$a->{'desc'},"\n");
	}
$self->close_tempfile(FIELDS);
}

# show_custom_fields([&domain])
# Returns HTML for custom field inputs, for inclusion in a table
sub show_custom_fields
{
	my $self = shift;

local $rv;
local $f;
local $col = 0;
foreach $f ($self->list_custom_fields()) {
	$rv .= "<tr>\n" if ($col%2 == 0);
	$rv .= "<td valign=top><b>$f->{'desc'}</b></td>\n";
	$rv .= "<td>";
	local $n = "field_".$f->{'name'};
	local $v = $_[0] ? $_[0]->{"field_".$f->{'name'}} : undef;
	if ($f->{'type'} == 0) {
		local $sz = $f->{'opts'} || 30;
		$rv .= $self->ui_textbox($n, $v, $sz);
		}
	elsif ($f->{'type'} == 1 || $f->{'type'} == 2) {
		$rv .= $self->ui_user_textbox($n, $v);
		}
	elsif ($f->{'type'} == 3 || $f->{'type'} == 4) {
		$rv .= $self->ui_group_textbox($n, $v);
		}
	elsif ($f->{'type'} == 5 || $f->{'type'} == 6) {
		$rv .= $self->ui_textbox($n, $v, 30)." ".
			$self->file_chooser_button($n, $f->{'type'}-5);
		}
	elsif ($f->{'type'} == 7) {
		$rv .= $self->ui_radio($n, $v ? 1 : 0, [ [ 1, $self->{'text_hash'}->{'yes'} ],
						   [ 0, $self->{'text_hash'}->{'no'} ] ]);
		}
	elsif ($f->{'type'} == 8) {
		local $sz = $f->{'opts'} || 30;
		$rv .= $self->ui_password($n, $v, $sz);
		}
	elsif ($f->{'type'} == 9) {
		local @opts = $self->read_opts_file($f->{'opts'});
		local ($found) = grep { $_->[0] eq $v } @opts;
		push(@opts, [ $v, $v ]) if (!$found);
		$rv .= $self->ui_select($n, $v, \@opts);
		}
	elsif ($f->{'type'} == 10) {
		local ($w, $h) = split(/\s+/, $f->{'opts'});
		$h ||= 4;
		$w ||= 30;
		$v =~ s/\t/\n/g;
		$rv .= $self->ui_textarea($n, $v, $h, $w);
		}
	$rv .= "</td>\n";
	$rv .= "</tr>\n" if ($col++%2 == 1);
	}
$rv .= "</tr>\n" if ($col++%2 == 1);
return $rv;
}

# parse_custom_fields(&domain, &in)
# Updates a domain with custom fields
sub parse_custom_fields
{
	my $self = shift;

local $f;
local %in = %{$_[1]};
foreach $f ($self->list_custom_fields()) {
	local $n = "field_".$f->{'name'};
	local $rv;
	if ($f->{'type'} == 0 || $f->{'type'} == 5 ||
	    $f->{'type'} == 6 || $f->{'type'} == 8) {
		$rv = $in{$n};
		}
	elsif ($f->{'type'} == 10) {
		$rv = $in{$n};
		$rv =~ s/\r//g;
		$rv =~ s/\n/\t/g;
		}
	elsif ($f->{'type'} == 1 || $f->{'type'} == 2) {
		local @u = getpwnam($in{$n});
		$rv = $f->{'type'} == 1 ? $in{$n} : $u[2];
		}
	elsif ($f->{'type'} == 3 || $f->{'type'} == 4) {
		local @g = getgrnam($in{$n});
		$rv = $f->{'type'} == 3 ? $in{$n} : $g[2];
		}
	elsif ($f->{'type'} == 7) {
		$rv = $in{$n} ? $f->{'opts'} : "";
		}
	elsif ($f->{'type'} == 9) {
		$rv = $in{$n};
		}
	$_[0]->{"field_".$f->{'name'}} = $rv;
	}
}

# read_opts_file(file)
sub read_opts_file
{
	my $self = shift;

local @rv;
local $file = $_[0];
if ($file !~ /^\//) {
	local @uinfo = getpwnam($self->{'remote_user'});
	if (@uinfo) {
		$file = "$uinfo[7]/$file";
		}
	}
open(FILE, $file);
while(<FILE>) {
	s/\r|\n//g;
	if (/^"([^"]*)"\s+"([^"]*)"$/) {
		push(@rv, [ $1, $2 ]);
		}
	elsif (/^"([^"]*)"$/) {
		push(@rv, [ $1, $1 ]);
		}
	elsif (/^(\S+)\s+(\S.*)/) {
		push(@rv, [ $1, $2 ]);
		}
	else {
		push(@rv, [ $_, $_ ]);
		}
	}
close(FILE);
return @rv;
}

# connect_qmail_ldap([return-error])
# Connect to the LDAP server used for Qmail. Returns an LDAP handle on success,
# or an error message on failure.
sub connect_qmail_ldap
{
	my $self = shift;

eval "use Net::LDAP";
if ($@) {
	local $err = $self->text('ldap_emod', "<tt>Net::LDAP</tt>");
	if ($_[0]) { return $err; }
	else { $self->error($err); }
	}

# Connect to server
local $port = $self->{'config_hash'}->{'ldap_port'} || 389;
local $ldap = Net::LDAP->new($self->{'config_hash'}->{'ldap_host'}, port => $port);
if (!$ldap) {
	local $err = $self->text('ldap_econn',
			   "<tt>$self->{'config_hash'}->{'ldap_host'}</tt>","<tt>$port</tt>");
	if ($_[0]) { return $err; }
	else { $self->error($err); }
	}

# Start TLS if configured
if ($self->{'config_hash'}->{'ldap_tls'}) {
	$ldap->start_tls();
	}

# Login
local $mesg;
if ($self->{'config_hash'}->{'ldap_login'}) {
	$mesg = $ldap->bind(dn => $self->{'config_hash'}->{'ldap_login'},
			    password => $self->{'config_hash'}->{'ldap_pass'});
	}
else {
	$mesg = $ldap->bind(anonymous => 1);
	}
if (!$mesg || $mesg->code) {
	local $err = $self->text('ldap_elogin', "<tt>$self->{'config_hash'}->{'ldap_host'}</tt>",
		     $self->{'dn'}, $mesg ? $mesg->error : "Unknown error");
	if ($_[0]) { return $err; }
	else { $self->error($err); }
	}
return $ldap;
}

# qmail_dn_to_hash(&ldap-object)
# Given a LDAP object containing user details, convert it to a hash
sub qmail_dn_to_hash
{
	my $self = shift;

local $x;
local %oc = map { $_, 1 } $_[0]->get_value("objectClass");
local %user = ( 'dn' => $_[0]->dn(),
		'qmail' => 1,
		'user' => scalar($_[0]->get_value("uid")),
		'plainpass' => scalar($_[0]->get_value("cuserPassword")),
		'uid' => $oc{'posixAccount'} ?
			scalar($_[0]->get_value("uidNumber")) :
			scalar($_[0]->get_value("qmailUID")),
		'gid' => $oc{'posixAccount'} ?
			scalar($_[0]->get_value("gidNumber")) :
			scalar($_[0]->get_value("qmailGID")),
		'real' => scalar($_[0]->get_value("cn")),
		'shell' => scalar($_[0]->get_value("loginShell")),
		'home' => scalar($_[0]->get_value("homeDirectory")),
		'pass' => scalar($_[0]->get_value("userPassword")),
		'mailstore' => scalar($_[0]->get_value("mailMessageStore")),
		'qquota' => scalar($_[0]->get_value("mailQuotaSize")),
		'qfiles' => scalar($_[0]->get_value("mailQuotaCount")),
		'email' => scalar($_[0]->get_value("mail")),
		'extraemail' => [ $_[0]->get_value("mailAlternateAddress") ],
	      );
local @fwd = $_[0]->get_value("mailForwardingAddress");
if (@fwd) {
	$user{'to'} = \@fwd;
	}
$user{'pass'} =~ s/^{[a-z0-9]+}//i;
$user{'qmail'} = 1;
$user{'unix'} = 1 if ($oc{'posixAccount'});
$user{'person'} = 1 if ($oc{'person'} || $oc{'inetOrgPerson'} ||
			$oc{'posixAccount'});
$user{'mailquota'} = 1;
$user{'mailquotafiles'} = 1;
return %user;
}

# qmail_user_to_dn(&user, &classes, &domain)
# Given a useradmin-style user hash, returns a list of properties to 
# add/update and to delete
sub qmail_user_to_dn
{
	my $self = shift;

local $pfx = $_[0]->{'pass'} =~ /^\{[a-z0-9]+\}/i ? undef : "{crypt}";
local @ee = @{$_[0]->{'extraemail'}};
local @to = @{$_[0]->{'to'}};
local @delrv;
local @rv = (
	 "uid" => $_[0]->{'user'},
	 "qmailUID" => $_[0]->{'uid'},
	 "qmailGID" => $_[0]->{'gid'},
	 "homeDirectory" => $_[0]->{'home'},
	 "userPassword" => $pfx.$_[0]->{'pass'},
	 "mailMessageStore" => $_[0]->{'mailstore'},
	 "mailQuotaSize" => $_[0]->{'qquota'},
	 "mailQuotaCount" => $_[0]->{'qfiles'},
	 "mail" => $_[0]->{'email'},
	 "mailHost" => $self->get_system_hostname(),
	 "accountStatus" => "active",
	);
if (@ee) {
	push(@rv, "mailAlternateAddress" => \@ee );
	}
else {	
	push(@delrv, "mailAlternateAddress");
	}
if (@to) {
	push(@rv, "mailForwardingAddress" => \@to );
	push(@rv, "deliveryMode", "nolocal");
	}
else {	
	push(@delrv, "mailForwardingAddress");
	push(@rv, "deliveryMode", "noforward");
	}
if ($_[0]->{'unix'}) {
	push(@rv, "uidNumber" => $_[0]->{'uid'},
		  "gidNumber" => $_[0]->{'gid'},
		  "loginShell" => $_[0]->{'shell'});
	}
if ($_[0]->{'person'}) {
	push(@rv, "cn" => $_[0]->{'real'});
	}
if ($self->indexof("person", @{$_[1]}) >= 0 ||
    $self->indexof("inetOrgPerson", @{$_[1]}) >= 0) {
	# Have to set sn
	push(@rv, "sn" => $_[0]->{'user'});
	}
# Add extra attribs, which can override those set above
local %subs = %{$_[0]};
$self->userdom_substitutions(\%subs, $_[2]);
local @props = $self->split_props($self->{'config_hash'}->{'ldap_props'}, \%subs);
local @addprops;
local $i;
local %over;
for($i=0; $i<@props; $i+=2) {
	if ($props[$i+1] ne "") {
		push(@addprops, $props[$i], $props[$i+1]);
		}
	else {
		push(@delrv, $props[$i]);
		}
	$over{$props[$i]} = $props[$i+1];
	}
for($i=0; $i<@rv; $i+=2) {
	if (exists($over{$rv[$i]})) {
		splice(@rv, $i, 2);
		$i -= 2;
		}
	}
push(@rv, @addprops);
return wantarray ? ( \@rv, \@delrv ) : \@rv;
}

# split_props(text, &user)
# Splits up LDAP properties
sub split_props
{
	my $self = shift;

local %pmap;
foreach $p (split(/\t+/, $self->substitute_template($_[0], $_[1]))) {
	if ($p =~ /^(\S+):\s*(.*)/) {
		push(@{$pmap{$1}}, $2);
		}
	}
local @rv;
local $k;
foreach $k (keys %pmap) {
	local $v = $pmap{$k};
	if (@$v == 1) {
		push(@rv, $k, $v->[0]);
		}
	else {
		push(@rv, $k, $v);
		}
	}
return @rv;
}

# create_initial_user(&dom)
# Returns a structure for a new mailbox user
sub create_initial_user
{
	my $self = shift;

local $user;
if ($self->{'config_hash'}->{'mail_system'} == 4) {
	# User is for Qmail+LDAP
	$user = { 'qmail' => 1,
		  'mailquota' => 1,
		  'mailquotafiles' => 1,
		  'person' => $self->{'config_hash'}->{'ldap_classes'} =~ /person|inetOrgPerson/ || $self->{'config_hash'}->{'ldap_unix'} ? 1 : 0,
		  'unix' => $self->{'config_hash'}->{'ldap_unix'} };
	}
elsif ($self->{'config_hash'}->{'mail_system'} == 5) {
	# VPOPMail user
	$user = { 'vpopmail' => 1,
		  'mailquota' => 1,
		  'person' => 1,
		  'fixedhome' => 1,
		  'noappend' => 1,
		  'noprimary' => 1 };
	}
else {
	# Normal unix user
	$user = { 'unix' => 1,
		  'person' => 1 };
	}
if ($_[0]) {
	local $tmpl = $self->get_template($_[0]->{'template'});
	if ($tmpl->{'user_aliases'} ne 'none') {
		$user->{'to'} = [ split(/\t+/, $tmpl->{'user_aliases'}) ];
		}
	}
$user->{'secs'} = [ ];
return $user;
}

# valid_domain_name(&parent, newdomain)
# Returns an error message if some domain name is invalid, or undef if OK
sub valid_domain_name
{
	my $self = shift;

if ($self->{'access_hash'}->{'forceunder'}) {
	local $pd = $_[0]->{'dom'};
	if ($_[1] !~ /\.\Q$pd\E$/i) {
		return $self->text('setup_eforceunder', $self->{'parentdom'}->{'dom'});
		}
	}
return undef;
}

# domain_databases(&domain)
# Returns a list of structures for databases in a domain
sub domain_databases
{
	my $self = shift;

local @dbs;
foreach my $db (split(/\s+/, $_[0]->{'db_mysql'})) {
        push(@dbs, { 'name' => $db,
                     'type' => 'mysql',
                     'desc' => $self->{'text_hash'}->{'databases_mysql'} });
        }
foreach my $db (split(/\s+/, $_[0]->{'db_postgres'})) {
        push(@dbs, { 'name' => $db,
                     'type' => 'postgres',
                     'desc' => $self->{'text_hash'}->{'databases_postgres'} });
        }
foreach my $f (@database_plugins) {
        push(@dbs, $self->plugin_call($f, "database_list", $_[0]));
        }
return @dbs;
}

# all_databases()
# Returns a list of all known databases on the system
sub all_databases
{
	my $self = shift;

local @rv;
if ($self->{'config_hash'}->{'mysql'}) {
        $self->require_mysql();
        push(@rv, map { { 'name' => $_,
                          'type' => 'mysql',
                          'desc' => $self->{'text_hash'}->{'databases_mysql'},
                          'special' => $_ eq "mysql" } }
                      $main::mysql_object->list_databases());
        }
if ($self->{'config_hash'}->{'postgres'}) {
        $self->require_postgres();
        push(@rv, map { { 'name' => $_,
                          'type' => 'postgres',
                          'desc' => $self->{'text_hash'}->{'databases_postgres'},
                          'special' => ($_ =~ /^template/i) } }
                      $main::postgresql_object->list_databases());
        }
foreach my $f (@database_plugins) {
        push(@rv, $self->plugin_call($f, "databases_all", $_[0]));
        }
return @rv;
}

# count_ftp_bandwidth(logfile, start, &bw-hash, &users, prefix, include-rotated)
# Scans an FTP server log file for downloads by some user, and returns the
# total bytes and time of last log entry.
sub count_ftp_bandwidth
{
	my $self = shift;

require 'timelocal.pl';
local $max_ltime = $_[1];
local $f;
foreach $f ($_[5] ? $self->all_log_files($_[0], $max_ltime) : ( $_[0] )) {
	if ($f =~ /\.gz$/i) {
		open(LOG, "gunzip -c ".quotemeta($f)." |");
		}
	elsif ($f =~ /\.Z$/i) {
		open(LOG, "uncompress -c ".quotemeta($f)." |");
		}
	else {
		open(LOG, $f);
		}
	while(<LOG>) {
		if (/^(\S+)\s+(\S+)\s+(\S+)\s+\[(\d+)\/(\S+)\/(\d+):(\d+):(\d+):(\d+)\s+(\S+)\]\s+"([^"]*)"\s+(\S+)\s+(\S+)/) {
			# ProFTPD extended log format line
			local $ltime = timelocal($9, $8, $7, $4, $self->{'apache_mmap_hash'}->{lc($5)}, $6-1900);
			$max_ltime = $ltime if ($ltime > $max_ltime);
			next if ($_[3] && $self->indexof($3, @{$_[3]}) < 0);	# user
			next if (substr($11, 0, 4) ne "RETR" &&
				 substr($11, 0, 4) ne "STOR");
			if ($ltime > $_[1]) {
				local $day = int($ltime / (24*60*60));
				$_[2]->{$_[4]."_".$day} += $13;
				}
			}
		elsif (/^\S+\s+(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(\d+)\s+\d+\s+\S+\s+(\d+)\s+\S+\s+\S+\s+\S+\s+(\S+)\s+\S+\s+(\S+)/) {
			# xferlog format line
			local $ltime = timelocal($5, $4, $3, $2, $self->{'apache_mmap_hash'}->{lc($1)}, $6-1900);
			$max_ltime = $ltime if ($ltime > $max_ltime);
			next if ($_[3] && $self->indexof($9, @{$_[3]}) < 0);	# user
			next if ($8 ne "o" && $8 ne "i");
			if ($ltime > $_[1]) {
				local $day = int($ltime / (24*60*60));
				$_[2]->{$_[4]."_".$day} += $7;
				print STDERR "adding $7 bytes for $9\n";
				}
			}
		}
	close(LOG);
	}
return $max_ltime;
}

# random_password()
# Returns an 8-character random password
sub random_password
{
	my $self = shift;

$self->seed_random();
$self->require_useradmin();
local $random_password;
foreach (1 .. 15) {
	$random_password .= $main::useradmin_object->{'random_password_chars'}[
			rand(scalar(@{$main::useradmin_object->{'random_password_chars_array'}}))];
	}
return $random_password;
}

# try_function(feature, function, arg, ...)
# Executes some function, and if it fails prints an error message
sub try_function
{
	my $self = shift;

local ($f, $func, @args) = @_;
local $main::error_must_die = 1;
eval { $self->$func(@args) };
if ($@) {
	do { my $fn = $self->{'second_print'}; $self->$fn($self->text('setup_failure',
		$self->{'text_hash'}->{'feature_'.$f}, $@))};
	return 0;
	}
return 1;
}

# bandwidth_period_start()
# Returns the day number on which the current bandwidth period started
sub bandwidth_period_start
{
	my $self = shift;

local $now = time();
local $day = int($now / (24*60*60));
local @tm = localtime(time());
if ($self->{'config_hash'}->{'bw_past'} eq 'week') {
	# Start on last sunday
	return $day - $tm[6];
	}
elsif ($self->{'config_hash'}->{'bw_past'} eq 'month') {
	# Start at 1st of month
	return int(timelocal(0, 0, 0, 1, $tm[4], $tm[5]) / (24*60*60));
	}
elsif ($self->{'config_hash'}->{'bw_past'} eq 'year') {
	# Start at start of year
	return int(timelocal(0, 0, 0, 1, 0, $tm[5]) / (24*60*60));
	}
else {
	# Start N days ago
	return $day - $self->{'config_hash'}->{'bw_period'};
	}
}

# servers_input(name, &ids, &domains)
# Returns HTML for a multi-server selection field
sub servers_input
{
	my $self = shift;

local ($name, $ids, $doms) = @_;
local $sz = scalar(@$doms) > 10 ? 10 : scalar(@$doms);
return $self->ui_select($name, $ids,
		  [ map { [ $_->{'id'}, $_->{'dom'} ] }
			sort { $a->{'dom'} cmp $b->{'dom'} } @$doms ],
		  $sz, 1);
}

# can_monitor_bandwidth(&domain)
# Returns 1 if bandwidth monitoring is enabled for some server
sub can_monitor_bandwidth
{
	my $self = shift;

if ($self->{'config_hash'}->{'bw_servers'} eq "") {
	return 1;	# always
	}
elsif ($self->{'config_hash'}->{'bw_servers'} =~ /^\!(.*)$/) {
	# List of servers not to check
	local @ids = split(/\s+/, $1);
	return $self->indexof($_[0]->{'id'}, @ids) == -1;
	}
else {
	# List of servers to check
	local @ids = split(/\s+/, $self->{'config_hash'}->{'bw_servers'});
	return $self->indexof($_[0]->{'id'}, @ids) != -1;
	}
}

# allowed_secondary_groups([&domain])
# Returns a list of secondary groups that users in some domain can belong to
sub allowed_secondary_groups
{
	my $self = shift;

return split(/\s+/, $self->{'config_hash'}->{'othergroups'});
}

# lock_user_db()
# Take out a lock on all mailbox users. Should be called before performing
# any user-related options
sub lock_user_db
{
	my $self = shift;

return $self->lock_file("$self->{'module_user_directory'}/userdb");
}

# unlock_user_db()
# Releases the lock take out by lock_user_db()
sub unlock_user_db
{
	my $self = shift;

return $self->unlock_file("$self->{'module_user_directory'}/userdb");
}

# get_user_domain(user)
# Given a username, returns it's virtual server details
sub get_user_domain
{
	my $self = shift;

local @uinfo = getpwnam($_[0]);
local @doms;
if (@uinfo) {
	# Is a Unix user .. find the domains for his GID
	@doms = $self->get_domain_by("gid", $uinfo[3]);
	}
else {
	# Need to check all domains :(
	@doms = $self->list_domains();
	}
local $d;
foreach $d (@doms) {
	local @users = $self->list_domain_users($d, 0, 1, 1, 1);
	local $u;
	foreach $u (@users) {
		if ($u->{'user'} eq $_[0]) {
			return $d;
			}
		}
	}
return undef;
}

# sub ztl_check_quotas
sub ztl_check_quotas {
	my $self = shift;

local $qerr;
local %config = ();
$self->read_file("$self->{'module_config_directory'}/config", \%config);

$config{'home_quotas'} = '';
$config{'mail_quotas'} = '';
$config{'group_quotas'} = '';
if ($config{'quotas'}) {
	# Make sure quotas are enabled, and work out where they are needed
	$self->require_useradmin();
	if (!$self->{'home_base'}) {
		$qerr = "<p><b>",$self->text('index_ehomebase'),"</b><p>\n";
		}
	else {
		$self->foreign_require("mount", "mount-lib.pl");
		$self->{'mail_base'} = $self->simplify_path($self->resolve_links(
				$self->mail_system_base()));
		($self->{'home_mtab'}, $self->{'home_fstab'}) = $self->mount_point($self->{'home_base'});
		($self->{'mail_mtab'}, $self->{'mail_fstab'}) = $self->mount_point($self->{'mail_base'});
		if (!$self->{'home_mtab'}) {
			$qerr = "<p><b>" . $self->text('index_ehomemtab' . "<tt>$self->{'home_base'}</tt>") . "</b><p>\n";
			}
		elsif (!$self->{'mail_mtab'}) {
			$qerr = "<p><b>" . $self->text('index_emailmtab' . "<tt>$self->{'mail_base'}</tt>") . "</b><p>\n";
			}
		else {
			# Check if quotas are enabled for home filesystem
			local $nohome;
			if (!($self->{'home_mtab'}->[4] = $main::quota_object->quota_can(
				$self->{'home_mtab'}, $self->{'home_fstab'}))%2 ||
			    !$main::quota_object->quota_now($self->{'home_mtab'}, $self->{'home_fstab'})) {
				$nohome++;
				}
			else {
				if ($self->{'home_mtab'}->[4] >= 2) {
					# Group quotas are active too
					$config{'group_quotas'} = 1;
					}
				}

			if ($self->{'home_mtab'}->[0] eq $self->{'mail_mtab'}->[0]) {
				# Home and mail are the same filesystem
				if ($nohome) {
					# Neither are enabled
					$qerr = $self->text('index_equota2',
					    "<tt>$self->{'home_mtab'}->[0]</tt>",
					    "<tt>$self->{'home_base'}</tt>",
					    "<tt>$self->{'mail_base'}</tt>");
					}
				else {
					# Both are enabled
					$config{'home_quotas'} =
						$self->{'home_mtab'}->[0];
					$config{'mail_quotas'} =
						$self->{'home_mtab'}->[0];
					}
				}
			else {
				# Different .. so check mail too
				local $nomail;
				if (!($self->{'mail_mtab'}->[4] = $main::quota_object->quota_can(
					$self->{'mail_mtab'}, $self->{'mail_fstab'})) ||
				    !$main::quota_object->quota_now($self->{'mail_mtab'},
						       $self->{'mail_fstab'})) {
					$nomail++;
					}
				if ($nohome) {
					$qerr = $self->text('index_equota3',
					    "<tt>$self->{'home_mtab'}->[0]</tt>",
					    "<tt>$self->{'home_base'}</tt>");
					}
				else {
					$config{'home_quotas'} =
						$self->{'home_mtab'}->[0];
					}
				if ($nomail) {
					$qerr = $self->text('index_equota4',
					    "<tt>$self->{'mail_mtab'}->[0]</tt>",
					    "<tt>$self->{'mail_base'}</tt>");
					}
				else {
					$config{'mail_quotas'} =
						$self->{'mail_mtab'}->[0];
					}
				}
			}
		}
	}

return $qerr if $qerr;

$self->lock_file($self->{'module_config_file'});
$self->write_file($self->{'module_config_file'}, \%config);
$self->unlock_file($self->{'module_config_file'});
$self->write_file("$self->{'module_config_directory'}/last-config", \%config);
}

# mount_point(dir)
# Returns both the mtab and fstab details for the parent mount for a directory
sub mount_point
{
	my $self = shift;

local $dir = $self->resolve_links($_[0]);
local @mounts = $main::mount_object->list_mounts();
local @mounted = $main::mount_object->list_mounted();
@mounts = @mounted if (!@mounts);
foreach $m (sort { length($b->[0]) <=> length($a->[0]) } @mounted) {
	if ($dir eq $m->[0] || $m->[0] eq "/" ||
	    substr($dir, 0, length($m->[0])+1) eq "$m->[0]/") {
		local ($m2) = grep { $_->[0] eq $m->[0] } @mounts;
		if ($m2) {
			return ($m, $m2);
			}
		}
	}
$self->error("Failed to find mount point for $dir");
}

######################################################
# ADD needed function from new virtual-server-lib.pl:
######################################################
# list_mx_servers()
# Returns the objects for servers used as secondary MXs
sub list_mx_servers
{
	my $self = shift;

if ($self->foreign_check("servers")) {
	$self->foreign_require("servers", "servers-lib.pl");
	local %servers = map { $_->{'id'}, $_ } $main::servers_object->list_servers();
	local @rv;
	foreach my $idname (split(/\s+/, $self->{'config_hash'}->{'mx_servers'})) {
		my ($id, $name) = split(/=/, $idname);
		local $s = $servers{$id};
		if ($s) {
			$s->{'mxname'} = $name;
			push(@rv, $s);
			}
		}
	return @rv;
	}
return ();
}

# substitute_domain_template(string, &domain)
# Does $VAR substitution in a string for a given domain, pulling in
# PARENT_DOMAIN variables too
sub substitute_domain_template
{
	my $self = shift;

local ($str, $d) = @_;
local %hash = %$d;
if ($d->{'parent'}) {
    local $parent = $self->get_domain($d->{'parent'});
    foreach my $k (keys %$parent) {
	$hash{'parent_domain_'.$k} = $parent->{$k};
        }
    }
return $self->substitute_template($str, \%hash);
}


1;

